/*!--------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
define("vs/base/browser/iframe", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var hasDifferentOriginAncestorFlag = false;
    var sameOriginWindowChainCache = null;
    function getParentWindowIfSameOrigin(w) {
        if (!w.parent || w.parent === w) {
            return null;
        }
        // Cannot really tell if we have access to the parent window unless we try to access something in it
        try {
            var location_1 = w.location;
            var parentLocation = w.parent.location;
            if (location_1.protocol !== parentLocation.protocol || location_1.hostname !== parentLocation.hostname || location_1.port !== parentLocation.port) {
                hasDifferentOriginAncestorFlag = true;
                return null;
            }
        }
        catch (e) {
            hasDifferentOriginAncestorFlag = true;
            return null;
        }
        return w.parent;
    }
    function findIframeElementInParentWindow(parentWindow, childWindow) {
        var parentWindowIframes = parentWindow.document.getElementsByTagName('iframe');
        var iframe;
        for (var i = 0, len = parentWindowIframes.length; i < len; i++) {
            iframe = parentWindowIframes[i];
            if (iframe.contentWindow === childWindow) {
                return iframe;
            }
        }
        return null;
    }
    var IframeUtils = (function () {
        function IframeUtils() {
        }
        /**
         * Returns a chain of embedded windows with the same origin (which can be accessed programmatically).
         * Having a chain of length 1 might mean that the current execution environment is running outside of an iframe or inside an iframe embedded in a window with a different origin.
         * To distinguish if at one point the current execution environment is running inside a window with a different origin, see hasDifferentOriginAncestor()
         */
        IframeUtils.getSameOriginWindowChain = function () {
            if (!sameOriginWindowChainCache) {
                sameOriginWindowChainCache = [];
                var w = window, parent_1;
                do {
                    parent_1 = getParentWindowIfSameOrigin(w);
                    if (parent_1) {
                        sameOriginWindowChainCache.push({
                            window: w,
                            iframeElement: findIframeElementInParentWindow(parent_1, w)
                        });
                    }
                    else {
                        sameOriginWindowChainCache.push({
                            window: w,
                            iframeElement: null
                        });
                    }
                    w = parent_1;
                } while (w);
            }
            return sameOriginWindowChainCache.slice(0);
        };
        /**
         * Returns true if the current execution environment is chained in a list of iframes which at one point ends in a window with a different origin.
         * Returns false if the current execution environment is not running inside an iframe or if the entire chain of iframes have the same origin.
         */
        IframeUtils.hasDifferentOriginAncestor = function () {
            if (!sameOriginWindowChainCache) {
                this.getSameOriginWindowChain();
            }
            return hasDifferentOriginAncestorFlag;
        };
        /**
         * Returns the position of `childWindow` relative to `ancestorWindow`
         */
        IframeUtils.getPositionOfChildWindowRelativeToAncestorWindow = function (childWindow, ancestorWindow) {
            if (!ancestorWindow || childWindow === ancestorWindow) {
                return {
                    top: 0,
                    left: 0
                };
            }
            var top = 0, left = 0;
            var windowChain = this.getSameOriginWindowChain();
            for (var i = 0; i < windowChain.length; i++) {
                var windowChainEl = windowChain[i];
                if (windowChainEl.window === ancestorWindow) {
                    break;
                }
                if (!windowChainEl.iframeElement) {
                    break;
                }
                var boundingRect = windowChainEl.iframeElement.getBoundingClientRect();
                top += boundingRect.top;
                left += boundingRect.left;
            }
            return {
                top: top,
                left: left
            };
        };
        return IframeUtils;
    }());
    exports.IframeUtils = IframeUtils;
});

define("vs/base/browser/styleMutator", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.StyleMutator = {
        setMaxWidth: function (domNode, maxWidth) {
            var desiredValue = maxWidth + 'px';
            if (domNode.style.maxWidth !== desiredValue) {
                domNode.style.maxWidth = desiredValue;
            }
        },
        setWidth: function (domNode, width) {
            var desiredValue = width + 'px';
            if (domNode.style.width !== desiredValue) {
                domNode.style.width = desiredValue;
            }
        },
        setHeight: function (domNode, height) {
            var desiredValue = height + 'px';
            if (domNode.style.height !== desiredValue) {
                domNode.style.height = desiredValue;
            }
        },
        setTop: function (domNode, top) {
            var desiredValue = top + 'px';
            if (domNode.style.top !== desiredValue) {
                domNode.style.top = desiredValue;
            }
        },
        setLeft: function (domNode, left) {
            var desiredValue = left + 'px';
            if (domNode.style.left !== desiredValue) {
                domNode.style.left = desiredValue;
            }
        },
        setBottom: function (domNode, bottom) {
            var desiredValue = bottom + 'px';
            if (domNode.style.bottom !== desiredValue) {
                domNode.style.bottom = desiredValue;
            }
        },
        setRight: function (domNode, right) {
            var desiredValue = right + 'px';
            if (domNode.style.right !== desiredValue) {
                domNode.style.right = desiredValue;
            }
        },
        setFontSize: function (domNode, fontSize) {
            var desiredValue = fontSize + 'px';
            if (domNode.style.fontSize !== desiredValue) {
                domNode.style.fontSize = desiredValue;
            }
        },
        setLineHeight: function (domNode, lineHeight) {
            var desiredValue = lineHeight + 'px';
            if (domNode.style.lineHeight !== desiredValue) {
                domNode.style.lineHeight = desiredValue;
            }
        },
        setTransform: null,
        setDisplay: function (domNode, desiredValue) {
            if (domNode.style.display !== desiredValue) {
                domNode.style.display = desiredValue;
            }
        },
        setVisibility: function (domNode, desiredValue) {
            if (domNode.style.visibility !== desiredValue) {
                domNode.style.visibility = desiredValue;
            }
        },
    };
    // Define setTransform
    function setWebkitTransform(domNode, desiredValue) {
        if (domNode.getAttribute('data-transform') !== desiredValue) {
            domNode.setAttribute('data-transform', desiredValue);
            domNode.style.webkitTransform = desiredValue;
        }
    }
    function setTransform(domNode, desiredValue) {
        if (domNode.getAttribute('data-transform') !== desiredValue) {
            domNode.setAttribute('data-transform', desiredValue);
            domNode.style.transform = desiredValue;
        }
    }
    (function () {
        var testDomNode = document.createElement('div');
        if (typeof testDomNode.style.webkitTransform !== 'undefined') {
            exports.StyleMutator.setTransform = setWebkitTransform;
        }
        else {
            exports.StyleMutator.setTransform = setTransform;
        }
    })();
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/ui/list/rangeMap", ["require", "exports"], function (require, exports) {
    "use strict";
    /**
     * Returns the intersection between two ranges as a range itself.
     * Returns `null` if the intersection is empty.
     */
    function intersect(one, other) {
        if (one.start >= other.end || other.start >= one.end) {
            return null;
        }
        var start = Math.max(one.start, other.start);
        var end = Math.min(one.end, other.end);
        if (end - start <= 0) {
            return null;
        }
        return { start: start, end: end };
    }
    exports.intersect = intersect;
    /**
     * Returns the intersection between a ranged group and a range.
     * Returns `[]` if the intersection is empty.
     */
    function groupIntersect(range, groups) {
        var result = [];
        for (var _i = 0, groups_1 = groups; _i < groups_1.length; _i++) {
            var r = groups_1[_i];
            if (range.start >= r.range.end) {
                continue;
            }
            if (range.end < r.range.start) {
                break;
            }
            var intersection = intersect(range, r.range);
            if (!intersection) {
                continue;
            }
            result.push({
                range: intersection,
                size: r.size
            });
        }
        return result;
    }
    exports.groupIntersect = groupIntersect;
    /**
     * Shifts a range by that `much`.
     */
    function shift(_a, much) {
        var start = _a.start, end = _a.end;
        return { start: start + much, end: end + much };
    }
    /**
     * Consolidates a collection of ranged groups.
     *
     * Consolidation is the process of merging consecutive ranged groups
     * that share the same `size`.
     */
    function consolidate(groups) {
        var result = [];
        var previousGroup = null;
        for (var _i = 0, groups_2 = groups; _i < groups_2.length; _i++) {
            var group = groups_2[_i];
            var start = group.range.start;
            var end = group.range.end;
            var size = group.size;
            if (previousGroup && size === previousGroup.size) {
                previousGroup.range.end = end;
                continue;
            }
            previousGroup = { range: { start: start, end: end }, size: size };
            result.push(previousGroup);
        }
        return result;
    }
    exports.consolidate = consolidate;
    /**
     * Concatenates several collections of ranged groups into a single
     * collection.
     */
    function concat() {
        var groups = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            groups[_i - 0] = arguments[_i];
        }
        return consolidate(groups.reduce(function (r, g) { return r.concat(g); }, []));
    }
    var RangeMap = (function () {
        function RangeMap() {
            this.groups = [];
            this._size = 0;
        }
        RangeMap.prototype.splice = function (index, deleteCount) {
            var items = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                items[_i - 2] = arguments[_i];
            }
            var diff = items.length - deleteCount;
            var before = groupIntersect({ start: 0, end: index }, this.groups);
            var after = groupIntersect({ start: index + deleteCount, end: Number.POSITIVE_INFINITY }, this.groups)
                .map(function (g) { return ({ range: shift(g.range, diff), size: g.size }); });
            var middle = items.map(function (item, i) { return ({
                range: { start: index + i, end: index + i + 1 },
                size: item.size
            }); });
            this.groups = concat(before, middle, after);
            this._size = this.groups.reduce(function (t, g) { return t + (g.size * (g.range.end - g.range.start)); }, 0);
        };
        Object.defineProperty(RangeMap.prototype, "count", {
            /**
             * Returns the number of items in the range map.
             */
            get: function () {
                var len = this.groups.length;
                if (!len) {
                    return 0;
                }
                return this.groups[len - 1].range.end;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RangeMap.prototype, "size", {
            /**
             * Returns the sum of the sizes of all items in the range map.
             */
            get: function () {
                return this._size;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Returns the index of the item at the given position.
         */
        RangeMap.prototype.indexAt = function (position) {
            if (position < 0) {
                return -1;
            }
            var index = 0;
            var size = 0;
            for (var _i = 0, _a = this.groups; _i < _a.length; _i++) {
                var group = _a[_i];
                var count = group.range.end - group.range.start;
                var newSize = size + (count * group.size);
                if (position < newSize) {
                    return index + Math.floor((position - size) / group.size);
                }
                index += count;
                size = newSize;
            }
            return index;
        };
        /**
         * Returns the index of the item right after the item at the
         * index of the given position.
         */
        RangeMap.prototype.indexAfter = function (position) {
            return Math.min(this.indexAt(position) + 1, this.count);
        };
        /**
         * Returns the start position of the item at the given index.
         */
        RangeMap.prototype.positionAt = function (index) {
            if (index < 0) {
                return -1;
            }
            var position = 0;
            var count = 0;
            for (var _i = 0, _a = this.groups; _i < _a.length; _i++) {
                var group = _a[_i];
                var groupCount = group.range.end - group.range.start;
                var newCount = count + groupCount;
                if (index < newCount) {
                    return position + ((index - count) * group.size);
                }
                position += groupCount * group.size;
                count = newCount;
            }
            return -1;
        };
        RangeMap.prototype.dispose = function () {
            this.groups = null;
        };
        return RangeMap;
    }());
    exports.RangeMap = RangeMap;
});

define("vs/base/browser/ui/scrollbar/scrollableElement", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (Visibility) {
        Visibility[Visibility["Auto"] = 0] = "Auto";
        Visibility[Visibility["Hidden"] = 1] = "Hidden";
        Visibility[Visibility["Visible"] = 2] = "Visible";
    })(exports.Visibility || (exports.Visibility = {}));
    var Visibility = exports.Visibility;
    function visibilityFromString(visibility) {
        switch (visibility) {
            case 'hidden':
                return Visibility.Hidden;
            case 'visible':
                return Visibility.Visible;
            default:
                return Visibility.Auto;
        }
    }
    exports.visibilityFromString = visibilityFromString;
});

define("vs/base/common/arrays", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Returns the last element of an array.
     * @param array The array.
     * @param n Which element from the end (default ist zero).
     */
    function tail(array, n) {
        if (n === void 0) { n = 0; }
        return array[array.length - (1 + n)];
    }
    exports.tail = tail;
    /**
     * Iterates the provided array and allows to remove
     * elements while iterating.
     */
    function forEach(array, callback) {
        for (var i = 0, len = array.length; i < len; i++) {
            callback(array[i], function () {
                array.splice(i, 1);
                i--;
                len--;
            });
        }
    }
    exports.forEach = forEach;
    function equals(one, other, itemEquals) {
        if (one.length !== other.length) {
            return false;
        }
        for (var i = 0, len = one.length; i < len; i++) {
            if (!itemEquals(one[i], other[i])) {
                return false;
            }
        }
        return true;
    }
    exports.equals = equals;
    function binarySearch(array, key, comparator) {
        var low = 0, high = array.length - 1;
        while (low <= high) {
            var mid = ((low + high) / 2) | 0;
            var comp = comparator(array[mid], key);
            if (comp < 0) {
                low = mid + 1;
            }
            else if (comp > 0) {
                high = mid - 1;
            }
            else {
                return mid;
            }
        }
        return -(low + 1);
    }
    exports.binarySearch = binarySearch;
    /**
     * Takes a sorted array and a function p. The array is sorted in such a way that all elements where p(x) is false
     * are located before all elements where p(x) is true.
     * @returns the least x for which p(x) is true or array.length if no element fullfills the given function.
     */
    function findFirst(array, p) {
        var low = 0, high = array.length;
        if (high === 0) {
            return 0; // no children
        }
        while (low < high) {
            var mid = Math.floor((low + high) / 2);
            if (p(array[mid])) {
                high = mid;
            }
            else {
                low = mid + 1;
            }
        }
        return low;
    }
    exports.findFirst = findFirst;
    function merge(arrays, hashFn) {
        var result = new Array();
        if (!hashFn) {
            for (var i = 0, len = arrays.length; i < len; i++) {
                result.push.apply(result, arrays[i]);
            }
        }
        else {
            var map = {};
            for (var i = 0; i < arrays.length; i++) {
                for (var j = 0; j < arrays[i].length; j++) {
                    var element = arrays[i][j], hash = hashFn(element);
                    if (!map.hasOwnProperty(hash)) {
                        map[hash] = true;
                        result.push(element);
                    }
                }
            }
        }
        return result;
    }
    exports.merge = merge;
    /**
     * @returns a new array with all undefined or null values removed. The original array is not modified at all.
     */
    function coalesce(array) {
        if (!array) {
            return array;
        }
        return array.filter(function (e) { return !!e; });
    }
    exports.coalesce = coalesce;
    /**
     * @returns true if the given item is contained in the array.
     */
    function contains(array, item) {
        return array.indexOf(item) >= 0;
    }
    exports.contains = contains;
    /**
     * Swaps the elements in the array for the provided positions.
     */
    function swap(array, pos1, pos2) {
        var element1 = array[pos1];
        var element2 = array[pos2];
        array[pos1] = element2;
        array[pos2] = element1;
    }
    exports.swap = swap;
    /**
     * Moves the element in the array for the provided positions.
     */
    function move(array, from, to) {
        array.splice(to, 0, array.splice(from, 1)[0]);
    }
    exports.move = move;
    /**
     * @returns {{false}} if the provided object is an array
     * 	and not empty.
     */
    function isFalsyOrEmpty(obj) {
        return !Array.isArray(obj) || obj.length === 0;
    }
    exports.isFalsyOrEmpty = isFalsyOrEmpty;
    /**
     * Removes duplicates from the given array. The optional keyFn allows to specify
     * how elements are checked for equalness by returning a unique string for each.
     */
    function distinct(array, keyFn) {
        if (!keyFn) {
            return array.filter(function (element, position) {
                return array.indexOf(element) === position;
            });
        }
        var seen = {};
        return array.filter(function (elem) {
            var key = keyFn(elem);
            if (seen[key]) {
                return false;
            }
            seen[key] = true;
            return true;
        });
    }
    exports.distinct = distinct;
    function first(array, fn, notFoundValue) {
        if (notFoundValue === void 0) { notFoundValue = null; }
        for (var i = 0; i < array.length; i++) {
            var element = array[i];
            if (fn(element)) {
                return element;
            }
        }
        return notFoundValue;
    }
    exports.first = first;
    function commonPrefixLength(one, other, equals) {
        if (equals === void 0) { equals = function (a, b) { return a === b; }; }
        var result = 0;
        for (var i = 0, len = Math.min(one.length, other.length); i < len && equals(one[i], other[i]); i++) {
            result++;
        }
        return result;
    }
    exports.commonPrefixLength = commonPrefixLength;
    function flatten(arr) {
        return arr.reduce(function (r, v) { return r.concat(v); }, []);
    }
    exports.flatten = flatten;
});

define("vs/base/common/assert", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Throws an error with the provided message if the provided value does not evaluate to a true Javascript value.
     */
    function ok(value, message) {
        if (!value || value === null) {
            throw new Error(message ? 'Assertion failed (' + message + ')' : 'Assertion Failed');
        }
    }
    exports.ok = ok;
});

define("vs/base/common/collections", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function createStringDictionary() {
        return Object.create(null);
    }
    exports.createStringDictionary = createStringDictionary;
    function createNumberDictionary() {
        return Object.create(null);
    }
    exports.createNumberDictionary = createNumberDictionary;
    function lookup(from, what, alternate) {
        if (alternate === void 0) { alternate = null; }
        var key = String(what);
        if (contains(from, key)) {
            return from[key];
        }
        return alternate;
    }
    exports.lookup = lookup;
    function lookupOrInsert(from, stringOrNumber, alternate) {
        var key = String(stringOrNumber);
        if (contains(from, key)) {
            return from[key];
        }
        else {
            if (typeof alternate === 'function') {
                alternate = alternate();
            }
            from[key] = alternate;
            return alternate;
        }
    }
    exports.lookupOrInsert = lookupOrInsert;
    function insert(into, data, hashFn) {
        into[hashFn(data)] = data;
    }
    exports.insert = insert;
    var hasOwnProperty = Object.prototype.hasOwnProperty;
    function contains(from, what) {
        return hasOwnProperty.call(from, what);
    }
    exports.contains = contains;
    function values(from) {
        var result = [];
        for (var key in from) {
            if (hasOwnProperty.call(from, key)) {
                result.push(from[key]);
            }
        }
        return result;
    }
    exports.values = values;
    function forEach(from, callback) {
        for (var key in from) {
            if (hasOwnProperty.call(from, key)) {
                var result = callback({ key: key, value: from[key] }, function () {
                    delete from[key];
                });
                if (result === false) {
                    return;
                }
            }
        }
    }
    exports.forEach = forEach;
    function remove(from, key) {
        if (!hasOwnProperty.call(from, key)) {
            return false;
        }
        delete from[key];
        return true;
    }
    exports.remove = remove;
    /**
     * Groups the collection into a dictionary based on the provided
     * group function.
     */
    function groupBy(data, groupFn) {
        var result = createStringDictionary();
        data.forEach(function (element) { return lookupOrInsert(result, groupFn(element), []).push(element); });
        return result;
    }
    exports.groupBy = groupBy;
});

define("vs/base/common/diff/diffChange", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.DifferenceType = {
        Add: 0,
        Remove: 1,
        Change: 2
    };
    /**
     * Represents information about a specific difference between two sequences.
     */
    var DiffChange = (function () {
        /**
         * Constructs a new DiffChange with the given sequence information
         * and content.
         */
        function DiffChange(originalStart, originalLength, modifiedStart, modifiedLength) {
            //Debug.Assert(originalLength > 0 || modifiedLength > 0, "originalLength and modifiedLength cannot both be <= 0");
            this.originalStart = originalStart;
            this.originalLength = originalLength;
            this.modifiedStart = modifiedStart;
            this.modifiedLength = modifiedLength;
        }
        /**
         * The type of difference.
         */
        DiffChange.prototype.getChangeType = function () {
            if (this.originalLength === 0) {
                return exports.DifferenceType.Add;
            }
            else if (this.modifiedLength === 0) {
                return exports.DifferenceType.Remove;
            }
            else {
                return exports.DifferenceType.Change;
            }
        };
        /**
         * The end point (exclusive) of the change in the original sequence.
         */
        DiffChange.prototype.getOriginalEnd = function () {
            return this.originalStart + this.originalLength;
        };
        /**
         * The end point (exclusive) of the change in the modified sequence.
         */
        DiffChange.prototype.getModifiedEnd = function () {
            return this.modifiedStart + this.modifiedLength;
        };
        return DiffChange;
    }());
    exports.DiffChange = DiffChange;
});

define("vs/base/common/diff/diff", ["require", "exports", 'vs/base/common/diff/diffChange'], function (require, exports, diffChange_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    //
    // The code below has been ported from a C# implementation in VS
    //
    var Debug = (function () {
        function Debug() {
        }
        Debug.Assert = function (condition, message) {
            if (!condition) {
                throw new Error(message);
            }
        };
        return Debug;
    }());
    exports.Debug = Debug;
    var MyArray = (function () {
        function MyArray() {
        }
        /**
         * Copies a range of elements from an Array starting at the specified source index and pastes
         * them to another Array starting at the specified destination index. The length and the indexes
         * are specified as 64-bit integers.
         * sourceArray:
         *		The Array that contains the data to copy.
         * sourceIndex:
         *		A 64-bit integer that represents the index in the sourceArray at which copying begins.
         * destinationArray:
         *		The Array that receives the data.
         * destinationIndex:
         *		A 64-bit integer that represents the index in the destinationArray at which storing begins.
         * length:
         *		A 64-bit integer that represents the number of elements to copy.
         */
        MyArray.Copy = function (sourceArray, sourceIndex, destinationArray, destinationIndex, length) {
            for (var i = 0; i < length; i++) {
                destinationArray[destinationIndex + i] = sourceArray[sourceIndex + i];
            }
        };
        return MyArray;
    }());
    exports.MyArray = MyArray;
    //*****************************************************************************
    // LcsDiff.cs
    //
    // An implementation of the difference algorithm described in
    // "An O(ND) Difference Algorithm and its letiations" by Eugene W. Myers
    //
    // Copyright (C) 2008 Microsoft Corporation @minifier_do_not_preserve
    //*****************************************************************************
    // Our total memory usage for storing history is (worst-case):
    // 2 * [(MaxDifferencesHistory + 1) * (MaxDifferencesHistory + 1) - 1] * sizeof(int)
    // 2 * [1448*1448 - 1] * 4 = 16773624 = 16MB
    var MaxDifferencesHistory = 1447;
    //let MaxDifferencesHistory = 100;
    /**
     * A utility class which helps to create the set of DiffChanges from
     * a difference operation. This class accepts original DiffElements and
     * modified DiffElements that are involved in a particular change. The
     * MarktNextChange() method can be called to mark the separation between
     * distinct changes. At the end, the Changes property can be called to retrieve
     * the constructed changes.
     */
    var DiffChangeHelper = (function () {
        /**
         * Constructs a new DiffChangeHelper for the given DiffSequences.
         */
        function DiffChangeHelper() {
            this.m_changes = [];
            this.m_originalStart = Number.MAX_VALUE;
            this.m_modifiedStart = Number.MAX_VALUE;
            this.m_originalCount = 0;
            this.m_modifiedCount = 0;
        }
        /**
         * Marks the beginning of the next change in the set of differences.
         */
        DiffChangeHelper.prototype.MarkNextChange = function () {
            // Only add to the list if there is something to add
            if (this.m_originalCount > 0 || this.m_modifiedCount > 0) {
                // Add the new change to our list
                this.m_changes.push(new diffChange_1.DiffChange(this.m_originalStart, this.m_originalCount, this.m_modifiedStart, this.m_modifiedCount));
            }
            // Reset for the next change
            this.m_originalCount = 0;
            this.m_modifiedCount = 0;
            this.m_originalStart = Number.MAX_VALUE;
            this.m_modifiedStart = Number.MAX_VALUE;
        };
        /**
         * Adds the original element at the given position to the elements
         * affected by the current change. The modified index gives context
         * to the change position with respect to the original sequence.
         * @param originalIndex The index of the original element to add.
         * @param modifiedIndex The index of the modified element that provides corresponding position in the modified sequence.
         */
        DiffChangeHelper.prototype.AddOriginalElement = function (originalIndex, modifiedIndex) {
            // The 'true' start index is the smallest of the ones we've seen
            this.m_originalStart = Math.min(this.m_originalStart, originalIndex);
            this.m_modifiedStart = Math.min(this.m_modifiedStart, modifiedIndex);
            this.m_originalCount++;
        };
        /**
         * Adds the modified element at the given position to the elements
         * affected by the current change. The original index gives context
         * to the change position with respect to the modified sequence.
         * @param originalIndex The index of the original element that provides corresponding position in the original sequence.
         * @param modifiedIndex The index of the modified element to add.
         */
        DiffChangeHelper.prototype.AddModifiedElement = function (originalIndex, modifiedIndex) {
            // The 'true' start index is the smallest of the ones we've seen
            this.m_originalStart = Math.min(this.m_originalStart, originalIndex);
            this.m_modifiedStart = Math.min(this.m_modifiedStart, modifiedIndex);
            this.m_modifiedCount++;
        };
        /**
         * Retrieves all of the changes marked by the class.
         */
        DiffChangeHelper.prototype.getChanges = function () {
            if (this.m_originalCount > 0 || this.m_modifiedCount > 0) {
                // Finish up on whatever is left
                this.MarkNextChange();
            }
            return this.m_changes;
        };
        DiffChangeHelper.prototype.getReverseChanges = function () {
            /// <summary>
            /// Retrieves all of the changes marked by the class in the reverse order
            /// </summary>
            if (this.m_originalCount > 0 || this.m_modifiedCount > 0) {
                // Finish up on whatever is left
                this.MarkNextChange();
            }
            this.m_changes.reverse();
            return this.m_changes;
        };
        return DiffChangeHelper;
    }());
    /**
     * An implementation of the difference algorithm described in
     * "An O(ND) Difference Algorithm and its letiations" by Eugene W. Myers
     */
    var LcsDiff = (function () {
        /**
         * Constructs the DiffFinder
         */
        function LcsDiff(originalSequence, newSequence, continueProcessingPredicate) {
            if (continueProcessingPredicate === void 0) { continueProcessingPredicate = null; }
            this.OriginalSequence = originalSequence;
            this.ModifiedSequence = newSequence;
            this.ContinueProcessingPredicate = continueProcessingPredicate;
            this.m_originalIds = [];
            this.m_modifiedIds = [];
            this.m_forwardHistory = [];
            this.m_reverseHistory = [];
            this.ComputeUniqueIdentifiers();
        }
        LcsDiff.prototype.ComputeUniqueIdentifiers = function () {
            var originalSequenceLength = this.OriginalSequence.getLength();
            var modifiedSequenceLength = this.ModifiedSequence.getLength();
            this.m_originalIds = new Array(originalSequenceLength);
            this.m_modifiedIds = new Array(modifiedSequenceLength);
            // Create a new hash table for unique elements from the original
            // sequence.
            var hashTable = {};
            var currentUniqueId = 1;
            var i;
            // Fill up the hash table for unique elements
            for (i = 0; i < originalSequenceLength; i++) {
                var originalElementHash = this.OriginalSequence.getElementHash(i);
                if (!hashTable.hasOwnProperty(originalElementHash)) {
                    // No entry in the hashtable so this is a new unique element.
                    // Assign the element a new unique identifier and add it to the
                    // hash table
                    this.m_originalIds[i] = currentUniqueId++;
                    hashTable[originalElementHash] = this.m_originalIds[i];
                }
                else {
                    this.m_originalIds[i] = hashTable[originalElementHash];
                }
            }
            // Now match up modified elements
            for (i = 0; i < modifiedSequenceLength; i++) {
                var modifiedElementHash = this.ModifiedSequence.getElementHash(i);
                if (!hashTable.hasOwnProperty(modifiedElementHash)) {
                    this.m_modifiedIds[i] = currentUniqueId++;
                    hashTable[modifiedElementHash] = this.m_modifiedIds[i];
                }
                else {
                    this.m_modifiedIds[i] = hashTable[modifiedElementHash];
                }
            }
        };
        LcsDiff.prototype.ElementsAreEqual = function (originalIndex, newIndex) {
            return this.m_originalIds[originalIndex] === this.m_modifiedIds[newIndex];
        };
        LcsDiff.prototype.ComputeDiff = function () {
            return this._ComputeDiff(0, this.OriginalSequence.getLength() - 1, 0, this.ModifiedSequence.getLength() - 1);
        };
        /**
         * Computes the differences between the original and modified input
         * sequences on the bounded range.
         * @returns An array of the differences between the two input sequences.
         */
        LcsDiff.prototype._ComputeDiff = function (originalStart, originalEnd, modifiedStart, modifiedEnd) {
            var quitEarlyArr = [false];
            return this.ComputeDiffRecursive(originalStart, originalEnd, modifiedStart, modifiedEnd, quitEarlyArr);
        };
        /**
         * Private helper method which computes the differences on the bounded range
         * recursively.
         * @returns An array of the differences between the two input sequences.
         */
        LcsDiff.prototype.ComputeDiffRecursive = function (originalStart, originalEnd, modifiedStart, modifiedEnd, quitEarlyArr) {
            quitEarlyArr[0] = false;
            // Find the start of the differences
            while (originalStart <= originalEnd && modifiedStart <= modifiedEnd && this.ElementsAreEqual(originalStart, modifiedStart)) {
                originalStart++;
                modifiedStart++;
            }
            // Find the end of the differences
            while (originalEnd >= originalStart && modifiedEnd >= modifiedStart && this.ElementsAreEqual(originalEnd, modifiedEnd)) {
                originalEnd--;
                modifiedEnd--;
            }
            // In the special case where we either have all insertions or all deletions or the sequences are identical
            if (originalStart > originalEnd || modifiedStart > modifiedEnd) {
                var changes = void 0;
                if (modifiedStart <= modifiedEnd) {
                    Debug.Assert(originalStart === originalEnd + 1, 'originalStart should only be one more than originalEnd');
                    // All insertions
                    changes = [
                        new diffChange_1.DiffChange(originalStart, 0, modifiedStart, modifiedEnd - modifiedStart + 1)
                    ];
                }
                else if (originalStart <= originalEnd) {
                    Debug.Assert(modifiedStart === modifiedEnd + 1, 'modifiedStart should only be one more than modifiedEnd');
                    // All deletions
                    changes = [
                        new diffChange_1.DiffChange(originalStart, originalEnd - originalStart + 1, modifiedStart, 0)
                    ];
                }
                else {
                    Debug.Assert(originalStart === originalEnd + 1, 'originalStart should only be one more than originalEnd');
                    Debug.Assert(modifiedStart === modifiedEnd + 1, 'modifiedStart should only be one more than modifiedEnd');
                    // Identical sequences - No differences
                    changes = [];
                }
                return changes;
            }
            // This problem can be solved using the Divide-And-Conquer technique.
            var midOriginalArr = [0], midModifiedArr = [0];
            var result = this.ComputeRecursionPoint(originalStart, originalEnd, modifiedStart, modifiedEnd, midOriginalArr, midModifiedArr, quitEarlyArr);
            var midOriginal = midOriginalArr[0];
            var midModified = midModifiedArr[0];
            if (result !== null) {
                // Result is not-null when there was enough memory to compute the changes while
                // searching for the recursion point
                return result;
            }
            else if (!quitEarlyArr[0]) {
                // We can break the problem down recursively by finding the changes in the
                // First Half:   (originalStart, modifiedStart) to (midOriginal, midModified)
                // Second Half:  (midOriginal + 1, minModified + 1) to (originalEnd, modifiedEnd)
                // NOTE: ComputeDiff() is inclusive, therefore the second range starts on the next point
                var leftChanges = this.ComputeDiffRecursive(originalStart, midOriginal, modifiedStart, midModified, quitEarlyArr);
                var rightChanges = [];
                if (!quitEarlyArr[0]) {
                    rightChanges = this.ComputeDiffRecursive(midOriginal + 1, originalEnd, midModified + 1, modifiedEnd, quitEarlyArr);
                }
                else {
                    // We did't have time to finish the first half, so we don't have time to compute this half.
                    // Consider the entire rest of the sequence different.
                    rightChanges = [
                        new diffChange_1.DiffChange(midOriginal + 1, originalEnd - (midOriginal + 1) + 1, midModified + 1, modifiedEnd - (midModified + 1) + 1)
                    ];
                }
                return this.ConcatenateChanges(leftChanges, rightChanges);
            }
            // If we hit here, we quit early, and so can't return anything meaningful
            return [
                new diffChange_1.DiffChange(originalStart, originalEnd - originalStart + 1, modifiedStart, modifiedEnd - modifiedStart + 1)
            ];
        };
        LcsDiff.prototype.WALKTRACE = function (diagonalForwardBase, diagonalForwardStart, diagonalForwardEnd, diagonalForwardOffset, diagonalReverseBase, diagonalReverseStart, diagonalReverseEnd, diagonalReverseOffset, forwardPoints, reversePoints, originalIndex, originalEnd, midOriginalArr, modifiedIndex, modifiedEnd, midModifiedArr, deltaIsEven, quitEarlyArr) {
            var forwardChanges = null, reverseChanges = null;
            // First, walk backward through the forward diagonals history
            var changeHelper = new DiffChangeHelper();
            var diagonalMin = diagonalForwardStart;
            var diagonalMax = diagonalForwardEnd;
            var diagonalRelative = (midOriginalArr[0] - midModifiedArr[0]) - diagonalForwardOffset;
            var lastOriginalIndex = Number.MIN_VALUE;
            var historyIndex = this.m_forwardHistory.length - 1;
            var diagonal;
            do {
                // Get the diagonal index from the relative diagonal number
                diagonal = diagonalRelative + diagonalForwardBase;
                // Figure out where we came from
                if (diagonal === diagonalMin || (diagonal < diagonalMax && forwardPoints[diagonal - 1] < forwardPoints[diagonal + 1])) {
                    // Vertical line (the element is an insert)
                    originalIndex = forwardPoints[diagonal + 1];
                    modifiedIndex = originalIndex - diagonalRelative - diagonalForwardOffset;
                    if (originalIndex < lastOriginalIndex) {
                        changeHelper.MarkNextChange();
                    }
                    lastOriginalIndex = originalIndex;
                    changeHelper.AddModifiedElement(originalIndex + 1, modifiedIndex);
                    diagonalRelative = (diagonal + 1) - diagonalForwardBase; //Setup for the next iteration
                }
                else {
                    // Horizontal line (the element is a deletion)
                    originalIndex = forwardPoints[diagonal - 1] + 1;
                    modifiedIndex = originalIndex - diagonalRelative - diagonalForwardOffset;
                    if (originalIndex < lastOriginalIndex) {
                        changeHelper.MarkNextChange();
                    }
                    lastOriginalIndex = originalIndex - 1;
                    changeHelper.AddOriginalElement(originalIndex, modifiedIndex + 1);
                    diagonalRelative = (diagonal - 1) - diagonalForwardBase; //Setup for the next iteration
                }
                if (historyIndex >= 0) {
                    forwardPoints = this.m_forwardHistory[historyIndex];
                    diagonalForwardBase = forwardPoints[0]; //We stored this in the first spot
                    diagonalMin = 1;
                    diagonalMax = forwardPoints.length - 1;
                }
            } while (--historyIndex >= -1);
            // Ironically, we get the forward changes as the reverse of the
            // order we added them since we technically added them backwards
            forwardChanges = changeHelper.getReverseChanges();
            if (quitEarlyArr[0]) {
                // TODO: Calculate a partial from the reverse diagonals.
                //       For now, just assume everything after the midOriginal/midModified point is a diff
                var originalStartPoint = midOriginalArr[0] + 1;
                var modifiedStartPoint = midModifiedArr[0] + 1;
                if (forwardChanges !== null && forwardChanges.length > 0) {
                    var lastForwardChange = forwardChanges[forwardChanges.length - 1];
                    originalStartPoint = Math.max(originalStartPoint, lastForwardChange.getOriginalEnd());
                    modifiedStartPoint = Math.max(modifiedStartPoint, lastForwardChange.getModifiedEnd());
                }
                reverseChanges = [
                    new diffChange_1.DiffChange(originalStartPoint, originalEnd - originalStartPoint + 1, modifiedStartPoint, modifiedEnd - modifiedStartPoint + 1)
                ];
            }
            else {
                // Now walk backward through the reverse diagonals history
                changeHelper = new DiffChangeHelper();
                diagonalMin = diagonalReverseStart;
                diagonalMax = diagonalReverseEnd;
                diagonalRelative = (midOriginalArr[0] - midModifiedArr[0]) - diagonalReverseOffset;
                lastOriginalIndex = Number.MAX_VALUE;
                historyIndex = (deltaIsEven) ? this.m_reverseHistory.length - 1 : this.m_reverseHistory.length - 2;
                do {
                    // Get the diagonal index from the relative diagonal number
                    diagonal = diagonalRelative + diagonalReverseBase;
                    // Figure out where we came from
                    if (diagonal === diagonalMin || (diagonal < diagonalMax && reversePoints[diagonal - 1] >= reversePoints[diagonal + 1])) {
                        // Horizontal line (the element is a deletion))
                        originalIndex = reversePoints[diagonal + 1] - 1;
                        modifiedIndex = originalIndex - diagonalRelative - diagonalReverseOffset;
                        if (originalIndex > lastOriginalIndex) {
                            changeHelper.MarkNextChange();
                        }
                        lastOriginalIndex = originalIndex + 1;
                        changeHelper.AddOriginalElement(originalIndex + 1, modifiedIndex + 1);
                        diagonalRelative = (diagonal + 1) - diagonalReverseBase; //Setup for the next iteration
                    }
                    else {
                        // Vertical line (the element is an insertion)
                        originalIndex = reversePoints[diagonal - 1];
                        modifiedIndex = originalIndex - diagonalRelative - diagonalReverseOffset;
                        if (originalIndex > lastOriginalIndex) {
                            changeHelper.MarkNextChange();
                        }
                        lastOriginalIndex = originalIndex;
                        changeHelper.AddModifiedElement(originalIndex + 1, modifiedIndex + 1);
                        diagonalRelative = (diagonal - 1) - diagonalReverseBase; //Setup for the next iteration
                    }
                    if (historyIndex >= 0) {
                        reversePoints = this.m_reverseHistory[historyIndex];
                        diagonalReverseBase = reversePoints[0]; //We stored this in the first spot
                        diagonalMin = 1;
                        diagonalMax = reversePoints.length - 1;
                    }
                } while (--historyIndex >= -1);
                // There are cases where the reverse history will find diffs that
                // are correct, but not intuitive, so we need shift them.
                reverseChanges = changeHelper.getChanges();
            }
            return this.ConcatenateChanges(forwardChanges, reverseChanges);
        };
        /**
         * Given the range to compute the diff on, this method finds the point:
         * (midOriginal, midModified)
         * that exists in the middle of the LCS of the two sequences and
         * is the point at which the LCS problem may be broken down recursively.
         * This method will try to keep the LCS trace in memory. If the LCS recursion
         * point is calculated and the full trace is available in memory, then this method
         * will return the change list.
         * @param originalStart The start bound of the original sequence range
         * @param originalEnd The end bound of the original sequence range
         * @param modifiedStart The start bound of the modified sequence range
         * @param modifiedEnd The end bound of the modified sequence range
         * @param midOriginal The middle point of the original sequence range
         * @param midModified The middle point of the modified sequence range
         * @returns The diff changes, if available, otherwise null
         */
        LcsDiff.prototype.ComputeRecursionPoint = function (originalStart, originalEnd, modifiedStart, modifiedEnd, midOriginalArr, midModifiedArr, quitEarlyArr) {
            var originalIndex, modifiedIndex;
            var diagonalForwardStart = 0, diagonalForwardEnd = 0;
            var diagonalReverseStart = 0, diagonalReverseEnd = 0;
            var numDifferences;
            // To traverse the edit graph and produce the proper LCS, our actual
            // start position is just outside the given boundary
            originalStart--;
            modifiedStart--;
            // We set these up to make the compiler happy, but they will
            // be replaced before we return with the actual recursion point
            midOriginalArr[0] = 0;
            midModifiedArr[0] = 0;
            // Clear out the history
            this.m_forwardHistory = [];
            this.m_reverseHistory = [];
            // Each cell in the two arrays corresponds to a diagonal in the edit graph.
            // The integer value in the cell represents the originalIndex of the furthest
            // reaching point found so far that ends in that diagonal.
            // The modifiedIndex can be computed mathematically from the originalIndex and the diagonal number.
            var maxDifferences = (originalEnd - originalStart) + (modifiedEnd - modifiedStart);
            var numDiagonals = maxDifferences + 1;
            var forwardPoints = new Array(numDiagonals);
            var reversePoints = new Array(numDiagonals);
            // diagonalForwardBase: Index into forwardPoints of the diagonal which passes through (originalStart, modifiedStart)
            // diagonalReverseBase: Index into reversePoints of the diagonal which passes through (originalEnd, modifiedEnd)
            var diagonalForwardBase = (modifiedEnd - modifiedStart);
            var diagonalReverseBase = (originalEnd - originalStart);
            // diagonalForwardOffset: Geometric offset which allows modifiedIndex to be computed from originalIndex and the
            //    diagonal number (relative to diagonalForwardBase)
            // diagonalReverseOffset: Geometric offset which allows modifiedIndex to be computed from originalIndex and the
            //    diagonal number (relative to diagonalReverseBase)
            var diagonalForwardOffset = (originalStart - modifiedStart);
            var diagonalReverseOffset = (originalEnd - modifiedEnd);
            // delta: The difference between the end diagonal and the start diagonal. This is used to relate diagonal numbers
            //   relative to the start diagonal with diagonal numbers relative to the end diagonal.
            // The Even/Oddn-ness of this delta is important for determining when we should check for overlap
            var delta = diagonalReverseBase - diagonalForwardBase;
            var deltaIsEven = (delta % 2 === 0);
            // Here we set up the start and end points as the furthest points found so far
            // in both the forward and reverse directions, respectively
            forwardPoints[diagonalForwardBase] = originalStart;
            reversePoints[diagonalReverseBase] = originalEnd;
            // Remember if we quit early, and thus need to do a best-effort result instead of a real result.
            quitEarlyArr[0] = false;
            // A couple of points:
            // --With this method, we iterate on the number of differences between the two sequences.
            //   The more differences there actually are, the longer this will take.
            // --Also, as the number of differences increases, we have to search on diagonals further
            //   away from the reference diagonal (which is diagonalForwardBase for forward, diagonalReverseBase for reverse).
            // --We extend on even diagonals (relative to the reference diagonal) only when numDifferences
            //   is even and odd diagonals only when numDifferences is odd.
            var diagonal, tempOriginalIndex;
            for (numDifferences = 1; numDifferences <= (maxDifferences / 2) + 1; numDifferences++) {
                var furthestOriginalIndex = 0;
                var furthestModifiedIndex = 0;
                // Run the algorithm in the forward direction
                diagonalForwardStart = this.ClipDiagonalBound(diagonalForwardBase - numDifferences, numDifferences, diagonalForwardBase, numDiagonals);
                diagonalForwardEnd = this.ClipDiagonalBound(diagonalForwardBase + numDifferences, numDifferences, diagonalForwardBase, numDiagonals);
                for (diagonal = diagonalForwardStart; diagonal <= diagonalForwardEnd; diagonal += 2) {
                    // STEP 1: We extend the furthest reaching point in the present diagonal
                    // by looking at the diagonals above and below and picking the one whose point
                    // is further away from the start point (originalStart, modifiedStart)
                    if (diagonal === diagonalForwardStart || (diagonal < diagonalForwardEnd && forwardPoints[diagonal - 1] < forwardPoints[diagonal + 1])) {
                        originalIndex = forwardPoints[diagonal + 1];
                    }
                    else {
                        originalIndex = forwardPoints[diagonal - 1] + 1;
                    }
                    modifiedIndex = originalIndex - (diagonal - diagonalForwardBase) - diagonalForwardOffset;
                    // Save the current originalIndex so we can test for false overlap in step 3
                    tempOriginalIndex = originalIndex;
                    // STEP 2: We can continue to extend the furthest reaching point in the present diagonal
                    // so long as the elements are equal.
                    while (originalIndex < originalEnd && modifiedIndex < modifiedEnd && this.ElementsAreEqual(originalIndex + 1, modifiedIndex + 1)) {
                        originalIndex++;
                        modifiedIndex++;
                    }
                    forwardPoints[diagonal] = originalIndex;
                    if (originalIndex + modifiedIndex > furthestOriginalIndex + furthestModifiedIndex) {
                        furthestOriginalIndex = originalIndex;
                        furthestModifiedIndex = modifiedIndex;
                    }
                    // STEP 3: If delta is odd (overlap first happens on forward when delta is odd)
                    // and diagonal is in the range of reverse diagonals computed for numDifferences-1
                    // (the previous iteration; we haven't computed reverse diagonals for numDifferences yet)
                    // then check for overlap.
                    if (!deltaIsEven && Math.abs(diagonal - diagonalReverseBase) <= (numDifferences - 1)) {
                        if (originalIndex >= reversePoints[diagonal]) {
                            midOriginalArr[0] = originalIndex;
                            midModifiedArr[0] = modifiedIndex;
                            if (tempOriginalIndex <= reversePoints[diagonal] && MaxDifferencesHistory > 0 && numDifferences <= (MaxDifferencesHistory + 1)) {
                                // BINGO! We overlapped, and we have the full trace in memory!
                                return this.WALKTRACE(diagonalForwardBase, diagonalForwardStart, diagonalForwardEnd, diagonalForwardOffset, diagonalReverseBase, diagonalReverseStart, diagonalReverseEnd, diagonalReverseOffset, forwardPoints, reversePoints, originalIndex, originalEnd, midOriginalArr, modifiedIndex, modifiedEnd, midModifiedArr, deltaIsEven, quitEarlyArr);
                            }
                            else {
                                // Either false overlap, or we didn't have enough memory for the full trace
                                // Just return the recursion point
                                return null;
                            }
                        }
                    }
                }
                // Check to see if we should be quitting early, before moving on to the next iteration.
                var matchLengthOfLongest = ((furthestOriginalIndex - originalStart) + (furthestModifiedIndex - modifiedStart) - numDifferences) / 2;
                if (this.ContinueProcessingPredicate !== null && !this.ContinueProcessingPredicate(furthestOriginalIndex, this.OriginalSequence, matchLengthOfLongest)) {
                    // We can't finish, so skip ahead to generating a result from what we have.
                    quitEarlyArr[0] = true;
                    // Use the furthest distance we got in the forward direction.
                    midOriginalArr[0] = furthestOriginalIndex;
                    midModifiedArr[0] = furthestModifiedIndex;
                    if (matchLengthOfLongest > 0 && MaxDifferencesHistory > 0 && numDifferences <= (MaxDifferencesHistory + 1)) {
                        // Enough of the history is in memory to walk it backwards
                        return this.WALKTRACE(diagonalForwardBase, diagonalForwardStart, diagonalForwardEnd, diagonalForwardOffset, diagonalReverseBase, diagonalReverseStart, diagonalReverseEnd, diagonalReverseOffset, forwardPoints, reversePoints, originalIndex, originalEnd, midOriginalArr, modifiedIndex, modifiedEnd, midModifiedArr, deltaIsEven, quitEarlyArr);
                    }
                    else {
                        // We didn't actually remember enough of the history.
                        //Since we are quiting the diff early, we need to shift back the originalStart and modified start
                        //back into the boundary limits since we decremented their value above beyond the boundary limit.
                        originalStart++;
                        modifiedStart++;
                        return [
                            new diffChange_1.DiffChange(originalStart, originalEnd - originalStart + 1, modifiedStart, modifiedEnd - modifiedStart + 1)
                        ];
                    }
                }
                // Run the algorithm in the reverse direction
                diagonalReverseStart = this.ClipDiagonalBound(diagonalReverseBase - numDifferences, numDifferences, diagonalReverseBase, numDiagonals);
                diagonalReverseEnd = this.ClipDiagonalBound(diagonalReverseBase + numDifferences, numDifferences, diagonalReverseBase, numDiagonals);
                for (diagonal = diagonalReverseStart; diagonal <= diagonalReverseEnd; diagonal += 2) {
                    // STEP 1: We extend the furthest reaching point in the present diagonal
                    // by looking at the diagonals above and below and picking the one whose point
                    // is further away from the start point (originalEnd, modifiedEnd)
                    if (diagonal === diagonalReverseStart || (diagonal < diagonalReverseEnd && reversePoints[diagonal - 1] >= reversePoints[diagonal + 1])) {
                        originalIndex = reversePoints[diagonal + 1] - 1;
                    }
                    else {
                        originalIndex = reversePoints[diagonal - 1];
                    }
                    modifiedIndex = originalIndex - (diagonal - diagonalReverseBase) - diagonalReverseOffset;
                    // Save the current originalIndex so we can test for false overlap
                    tempOriginalIndex = originalIndex;
                    // STEP 2: We can continue to extend the furthest reaching point in the present diagonal
                    // as long as the elements are equal.
                    while (originalIndex > originalStart && modifiedIndex > modifiedStart && this.ElementsAreEqual(originalIndex, modifiedIndex)) {
                        originalIndex--;
                        modifiedIndex--;
                    }
                    reversePoints[diagonal] = originalIndex;
                    // STEP 4: If delta is even (overlap first happens on reverse when delta is even)
                    // and diagonal is in the range of forward diagonals computed for numDifferences
                    // then check for overlap.
                    if (deltaIsEven && Math.abs(diagonal - diagonalForwardBase) <= numDifferences) {
                        if (originalIndex <= forwardPoints[diagonal]) {
                            midOriginalArr[0] = originalIndex;
                            midModifiedArr[0] = modifiedIndex;
                            if (tempOriginalIndex >= forwardPoints[diagonal] && MaxDifferencesHistory > 0 && numDifferences <= (MaxDifferencesHistory + 1)) {
                                // BINGO! We overlapped, and we have the full trace in memory!
                                return this.WALKTRACE(diagonalForwardBase, diagonalForwardStart, diagonalForwardEnd, diagonalForwardOffset, diagonalReverseBase, diagonalReverseStart, diagonalReverseEnd, diagonalReverseOffset, forwardPoints, reversePoints, originalIndex, originalEnd, midOriginalArr, modifiedIndex, modifiedEnd, midModifiedArr, deltaIsEven, quitEarlyArr);
                            }
                            else {
                                // Either false overlap, or we didn't have enough memory for the full trace
                                // Just return the recursion point
                                return null;
                            }
                        }
                    }
                }
                // Save current vectors to history before the next iteration
                if (numDifferences <= MaxDifferencesHistory) {
                    // We are allocating space for one extra int, which we fill with
                    // the index of the diagonal base index
                    var temp = new Array(diagonalForwardEnd - diagonalForwardStart + 2);
                    temp[0] = diagonalForwardBase - diagonalForwardStart + 1;
                    MyArray.Copy(forwardPoints, diagonalForwardStart, temp, 1, diagonalForwardEnd - diagonalForwardStart + 1);
                    this.m_forwardHistory.push(temp);
                    temp = new Array(diagonalReverseEnd - diagonalReverseStart + 2);
                    temp[0] = diagonalReverseBase - diagonalReverseStart + 1;
                    MyArray.Copy(reversePoints, diagonalReverseStart, temp, 1, diagonalReverseEnd - diagonalReverseStart + 1);
                    this.m_reverseHistory.push(temp);
                }
            }
            // If we got here, then we have the full trace in history. We just have to convert it to a change list
            // NOTE: This part is a bit messy
            return this.WALKTRACE(diagonalForwardBase, diagonalForwardStart, diagonalForwardEnd, diagonalForwardOffset, diagonalReverseBase, diagonalReverseStart, diagonalReverseEnd, diagonalReverseOffset, forwardPoints, reversePoints, originalIndex, originalEnd, midOriginalArr, modifiedIndex, modifiedEnd, midModifiedArr, deltaIsEven, quitEarlyArr);
        };
        /**
         * Concatenates the two input DiffChange lists and returns the resulting
         * list.
         * @param The left changes
         * @param The right changes
         * @returns The concatenated list
         */
        LcsDiff.prototype.ConcatenateChanges = function (left, right) {
            var mergedChangeArr = [];
            var result = null;
            if (left.length === 0 || right.length === 0) {
                return (right.length > 0) ? right : left;
            }
            else if (this.ChangesOverlap(left[left.length - 1], right[0], mergedChangeArr)) {
                // Since we break the problem down recursively, it is possible that we
                // might recurse in the middle of a change thereby splitting it into
                // two changes. Here in the combining stage, we detect and fuse those
                // changes back together
                result = new Array(left.length + right.length - 1);
                MyArray.Copy(left, 0, result, 0, left.length - 1);
                result[left.length - 1] = mergedChangeArr[0];
                MyArray.Copy(right, 1, result, left.length, right.length - 1);
                return result;
            }
            else {
                result = new Array(left.length + right.length);
                MyArray.Copy(left, 0, result, 0, left.length);
                MyArray.Copy(right, 0, result, left.length, right.length);
                return result;
            }
        };
        /**
         * Returns true if the two changes overlap and can be merged into a single
         * change
         * @param left The left change
         * @param right The right change
         * @param mergedChange The merged change if the two overlap, null otherwise
         * @returns True if the two changes overlap
         */
        LcsDiff.prototype.ChangesOverlap = function (left, right, mergedChangeArr) {
            Debug.Assert(left.originalStart <= right.originalStart, 'Left change is not less than or equal to right change');
            Debug.Assert(left.modifiedStart <= right.modifiedStart, 'Left change is not less than or equal to right change');
            if (left.originalStart + left.originalLength >= right.originalStart || left.modifiedStart + left.modifiedLength >= right.modifiedStart) {
                var originalStart = left.originalStart;
                var originalLength = left.originalLength;
                var modifiedStart = left.modifiedStart;
                var modifiedLength = left.modifiedLength;
                if (left.originalStart + left.originalLength >= right.originalStart) {
                    originalLength = right.originalStart + right.originalLength - left.originalStart;
                }
                if (left.modifiedStart + left.modifiedLength >= right.modifiedStart) {
                    modifiedLength = right.modifiedStart + right.modifiedLength - left.modifiedStart;
                }
                mergedChangeArr[0] = new diffChange_1.DiffChange(originalStart, originalLength, modifiedStart, modifiedLength);
                return true;
            }
            else {
                mergedChangeArr[0] = null;
                return false;
            }
        };
        /**
         * Helper method used to clip a diagonal index to the range of valid
         * diagonals. This also decides whether or not the diagonal index,
         * if it exceeds the boundary, should be clipped to the boundary or clipped
         * one inside the boundary depending on the Even/Odd status of the boundary
         * and numDifferences.
         * @param diagonal The index of the diagonal to clip.
         * @param numDifferences The current number of differences being iterated upon.
         * @param diagonalBaseIndex The base reference diagonal.
         * @param numDiagonals The total number of diagonals.
         * @returns The clipped diagonal index.
         */
        LcsDiff.prototype.ClipDiagonalBound = function (diagonal, numDifferences, diagonalBaseIndex, numDiagonals) {
            if (diagonal >= 0 && diagonal < numDiagonals) {
                // Nothing to clip, its in range
                return diagonal;
            }
            // diagonalsBelow: The number of diagonals below the reference diagonal
            // diagonalsAbove: The number of diagonals above the reference diagonal
            var diagonalsBelow = diagonalBaseIndex;
            var diagonalsAbove = numDiagonals - diagonalBaseIndex - 1;
            var diffEven = (numDifferences % 2 === 0);
            if (diagonal < 0) {
                var lowerBoundEven = (diagonalsBelow % 2 === 0);
                return (diffEven === lowerBoundEven) ? 0 : 1;
            }
            else {
                var upperBoundEven = (diagonalsAbove % 2 === 0);
                return (diffEven === upperBoundEven) ? numDiagonals - 1 : numDiagonals - 2;
            }
        };
        return LcsDiff;
    }());
    exports.LcsDiff = LcsDiff;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/common/events", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Event = (function () {
        function Event(originalEvent) {
            this.time = (new Date()).getTime();
            this.originalEvent = originalEvent;
            this.source = null;
        }
        return Event;
    }());
    exports.Event = Event;
    var PropertyChangeEvent = (function (_super) {
        __extends(PropertyChangeEvent, _super);
        function PropertyChangeEvent(key, oldValue, newValue, originalEvent) {
            _super.call(this, originalEvent);
            this.key = key;
            this.oldValue = oldValue;
            this.newValue = newValue;
        }
        return PropertyChangeEvent;
    }(Event));
    exports.PropertyChangeEvent = PropertyChangeEvent;
    var ViewerEvent = (function (_super) {
        __extends(ViewerEvent, _super);
        function ViewerEvent(element, originalEvent) {
            _super.call(this, originalEvent);
            this.element = element;
        }
        return ViewerEvent;
    }(Event));
    exports.ViewerEvent = ViewerEvent;
    exports.EventType = {
        PROPERTY_CHANGED: 'propertyChanged',
        SELECTION: 'selection',
        FOCUS: 'focus',
        BLUR: 'blur',
        HIGHLIGHT: 'highlight',
        EXPAND: 'expand',
        COLLAPSE: 'collapse',
        TOGGLE: 'toggle',
        CONTENTS_CHANGED: 'contentsChanged',
        BEFORE_RUN: 'beforeRun',
        RUN: 'run',
        EDIT: 'edit',
        SAVE: 'save',
        CANCEL: 'cancel',
        CHANGE: 'change',
        DISPOSE: 'dispose',
    };
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
// copied from TypeScript/src/compiler/core/hash.ts and adjusted to external module syntax
define("vs/base/common/hash", ["require", "exports"], function (require, exports) {
    'use strict';
    function computeMurmur2StringHashCode(key) {
        // 'm' and 'r' are mixing constants generated offline.
        // They're not really 'magic', they just happen to work well.
        var m = 0x5bd1e995;
        var r = 24;
        var start = 0;
        var len = key.length;
        var numberOfCharsLeft = len;
        // Initialize the hash to a 'random' value.
        var h = (0 ^ numberOfCharsLeft);
        // Mix 4 bytes at a time into the hash.  NOTE: 4 bytes is two chars, so we iterate
        // through the string two chars at a time.
        var index = start;
        while (numberOfCharsLeft >= 2) {
            var c1 = key.charCodeAt(index);
            var c2 = key.charCodeAt(index + 1);
            var k = c1 | (c2 << 16);
            k *= m;
            k ^= k >> r;
            k *= m;
            h *= m;
            h ^= k;
            index += 2;
            numberOfCharsLeft -= 2;
        }
        // Handle the last char (or 2 bytes) if they exist.  This happens if the original string had
        // odd length.
        if (numberOfCharsLeft === 1) {
            h ^= key.charCodeAt(index);
            h *= m;
        }
        // Do a few final mixes of the hash to ensure the last few bytes are well-incorporated.
        h ^= h >> 13;
        h *= m;
        h ^= h >> 15;
        return h;
    }
    exports.computeMurmur2StringHashCode = computeMurmur2StringHashCode;
    function combine(value, currentHash) {
        // Ensure we stay within 31 bits.
        return (((currentHash << 5) + currentHash) + value) & 0x7FFFFFFF;
    }
    exports.combine = combine;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/common/htmlContent", ["require", "exports"], function (require, exports) {
    'use strict';
    function htmlContentElementCodeEqual(a, b) {
        if (!a && !b) {
            return true;
        }
        if (!a || !b) {
            return false;
        }
        return (a.language === b.language
            && a.value === b.value);
    }
    function htmlContentElementEqual(a, b) {
        return (a.formattedText === b.formattedText
            && a.text === b.text
            && a.className === b.className
            && a.style === b.style
            && a.customStyle === b.customStyle
            && a.tagName === b.tagName
            && a.isText === b.isText
            && a.role === b.role
            && a.markdown === b.markdown
            && htmlContentElementCodeEqual(a.code, b.code)
            && htmlContentElementArrEquals(a.children, b.children));
    }
    function htmlContentElementArrEquals(a, b) {
        if (!a && !b) {
            return true;
        }
        if (!a || !b) {
            return false;
        }
        var aLen = a.length, bLen = b.length;
        if (aLen !== bLen) {
            return false;
        }
        for (var i = 0; i < aLen; i++) {
            if (!htmlContentElementEqual(a[i], b[i])) {
                return false;
            }
        }
        return true;
    }
    exports.htmlContentElementArrEquals = htmlContentElementArrEquals;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/common/iterator", ["require", "exports"], function (require, exports) {
    'use strict';
    var ArrayIterator = (function () {
        function ArrayIterator(items, start, end) {
            if (start === void 0) { start = 0; }
            if (end === void 0) { end = items.length; }
            this.items = items;
            this.start = start;
            this.end = end;
            this.index = start - 1;
        }
        ArrayIterator.prototype.next = function () {
            this.index = Math.min(this.index + 1, this.end);
            if (this.index === this.end) {
                return null;
            }
            return this.items[this.index];
        };
        return ArrayIterator;
    }());
    exports.ArrayIterator = ArrayIterator;
    var MappedIterator = (function () {
        function MappedIterator(iterator, fn) {
            this.iterator = iterator;
            this.fn = fn;
            // noop
        }
        MappedIterator.prototype.next = function () { return this.fn(this.iterator.next()); };
        return MappedIterator;
    }());
    exports.MappedIterator = MappedIterator;
    var MappedNavigator = (function (_super) {
        __extends(MappedNavigator, _super);
        function MappedNavigator(navigator, fn) {
            _super.call(this, navigator, fn);
            this.navigator = navigator;
        }
        MappedNavigator.prototype.current = function () { return this.fn(this.navigator.current()); };
        MappedNavigator.prototype.previous = function () { return this.fn(this.navigator.previous()); };
        MappedNavigator.prototype.parent = function () { return this.fn(this.navigator.parent()); };
        MappedNavigator.prototype.first = function () { return this.fn(this.navigator.first()); };
        MappedNavigator.prototype.last = function () { return this.fn(this.navigator.last()); };
        return MappedNavigator;
    }(MappedIterator));
    exports.MappedNavigator = MappedNavigator;
});

define("vs/base/common/lifecycle", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.empty = Object.freeze({
        dispose: function () { }
    });
    function dispose(disposable) {
        if (disposable) {
            disposable.dispose();
        }
        return null;
    }
    exports.dispose = dispose;
    function disposeAll(arr) {
        if (arr) {
            for (var i = 0, len = arr.length; i < len; i++) {
                if (arr[i]) {
                    arr[i].dispose();
                }
            }
        }
        return [];
    }
    exports.disposeAll = disposeAll;
    function combinedDispose() {
        var disposables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            disposables[_i - 0] = arguments[_i];
        }
        return {
            dispose: function () { return disposeAll(disposables); }
        };
    }
    exports.combinedDispose = combinedDispose;
    function combinedDispose2(disposables) {
        return {
            dispose: function () { return disposeAll(disposables); }
        };
    }
    exports.combinedDispose2 = combinedDispose2;
    function fnToDisposable(fn) {
        return {
            dispose: function () { return fn(); }
        };
    }
    exports.fnToDisposable = fnToDisposable;
    function toDisposable() {
        var fns = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            fns[_i - 0] = arguments[_i];
        }
        return combinedDispose2(fns.map(fnToDisposable));
    }
    exports.toDisposable = toDisposable;
    function callAll(arg) {
        if (!arg) {
            return null;
        }
        else if (typeof arg === 'function') {
            arg();
            return null;
        }
        else if (Array.isArray(arg)) {
            while (arg.length > 0) {
                arg.pop()();
            }
            return arg;
        }
        else {
            return null;
        }
    }
    /**
     * Calls all functions that are being passed to it.
     */
    exports.cAll = callAll;
    var Disposable = (function () {
        function Disposable() {
            this._toDispose = [];
        }
        Disposable.prototype.dispose = function () {
            this._toDispose = disposeAll(this._toDispose);
        };
        Disposable.prototype._register = function (t) {
            this._toDispose.push(t);
            return t;
        };
        return Disposable;
    }());
    exports.Disposable = Disposable;
});


/**
 * marked - a markdown parser
 * Copyright (c) 2011-2014, Christopher Jeffrey. (MIT Licensed)
 * https://github.com/chjj/marked
 */

// TODO MonacoChange: we have our own way of defining modules
// ;(function() {
// END MonacoChange

/**
 * Block-Level Grammar
 */

var block = {
  newline: /^\n+/,
  code: /^( {4}[^\n]+\n*)+/,
  fences: noop,
  hr: /^( *[-*_]){3,} *(?:\n+|$)/,
  heading: /^ *(#{1,6}) *([^\n]+?) *#* *(?:\n+|$)/,
  nptable: noop,
  lheading: /^([^\n]+)\n *(=|-){2,} *(?:\n+|$)/,
  blockquote: /^( *>[^\n]+(\n(?!def)[^\n]+)*\n*)+/,
  list: /^( *)(bull) [\s\S]+?(?:hr|def|\n{2,}(?! )(?!\1bull )\n*|\s*$)/,
  html: /^ *(?:comment *(?:\n|\s*$)|closed *(?:\n{2,}|\s*$)|closing *(?:\n{2,}|\s*$))/,
  def: /^ *\[([^\]]+)\]: *<?([^\s>]+)>?(?: +["(]([^\n]+)[")])? *(?:\n+|$)/,
  table: noop,
  paragraph: /^((?:[^\n]+\n?(?!hr|heading|lheading|blockquote|tag|def))+)\n*/,
  text: /^[^\n]+/
};

block.bullet = /(?:[*+-]|\d+\.)/;
block.item = /^( *)(bull) [^\n]*(?:\n(?!\1bull )[^\n]*)*/;
block.item = replace(block.item, 'gm')
  (/bull/g, block.bullet)
  ();

block.list = replace(block.list)
  (/bull/g, block.bullet)
  ('hr', '\\n+(?=\\1?(?:[-*_] *){3,}(?:\\n+|$))')
  ('def', '\\n+(?=' + block.def.source + ')')
  ();

block.blockquote = replace(block.blockquote)
  ('def', block.def)
  ();

block._tag = '(?!(?:'
  + 'a|em|strong|small|s|cite|q|dfn|abbr|data|time|code'
  + '|var|samp|kbd|sub|sup|i|b|u|mark|ruby|rt|rp|bdi|bdo'
  + '|span|br|wbr|ins|del|img)\\b)\\w+(?!:/|[^\\w\\s@]*@)\\b';

block.html = replace(block.html)
  ('comment', /<!--[\s\S]*?-->/)
  ('closed', /<(tag)[\s\S]+?<\/\1>/)
  ('closing', /<tag(?:"[^"]*"|'[^']*'|[^'">])*?>/)
  (/tag/g, block._tag)
  ();

block.paragraph = replace(block.paragraph)
  ('hr', block.hr)
  ('heading', block.heading)
  ('lheading', block.lheading)
  ('blockquote', block.blockquote)
  ('tag', '<' + block._tag)
  ('def', block.def)
  ();

/**
 * Normal Block Grammar
 */

block.normal = merge({}, block);

/**
 * GFM Block Grammar
 */

block.gfm = merge({}, block.normal, {
  fences: /^ *(`{3,}|~{3,}) *(\S+)? *\n([\s\S]+?)\s*\1 *(?:\n+|$)/,
  paragraph: /^/
});

block.gfm.paragraph = replace(block.paragraph)
  ('(?!', '(?!'
    + block.gfm.fences.source.replace('\\1', '\\2') + '|'
    + block.list.source.replace('\\1', '\\3') + '|')
  ();

/**
 * GFM + Tables Block Grammar
 */

block.tables = merge({}, block.gfm, {
  nptable: /^ *(\S.*\|.*)\n *([-:]+ *\|[-| :]*)\n((?:.*\|.*(?:\n|$))*)\n*/,
  table: /^ *\|(.+)\n *\|( *[-:]+[-| :]*)\n((?: *\|.*(?:\n|$))*)\n*/
});

/**
 * Block Lexer
 */

function Lexer(options) {
  this.tokens = [];
  this.tokens.links = {};
  this.options = options || marked.defaults;
  this.rules = block.normal;

  if (this.options.gfm) {
    if (this.options.tables) {
      this.rules = block.tables;
    } else {
      this.rules = block.gfm;
    }
  }
}

/**
 * Expose Block Rules
 */

Lexer.rules = block;

/**
 * Static Lex Method
 */

Lexer.lex = function(src, options) {
  var lexer = new Lexer(options);
  return lexer.lex(src);
};

/**
 * Preprocessing
 */

Lexer.prototype.lex = function(src) {
  src = src
    .replace(/\r\n|\r/g, '\n')
    .replace(/\t/g, '    ')
    .replace(/\u00a0/g, ' ')
    .replace(/\u2424/g, '\n');

  return this.token(src, true);
};

/**
 * Lexing
 */

Lexer.prototype.token = function(src, top, bq) {
  var src = src.replace(/^ +$/gm, '')
    , next
    , loose
    , cap
    , bull
    , b
    , item
    , space
    , i
    , l;

  while (src) {
    // newline
    if (cap = this.rules.newline.exec(src)) {
      src = src.substring(cap[0].length);
      if (cap[0].length > 1) {
        this.tokens.push({
          type: 'space'
        });
      }
    }

    // code
    if (cap = this.rules.code.exec(src)) {
      src = src.substring(cap[0].length);
      cap = cap[0].replace(/^ {4}/gm, '');
      this.tokens.push({
        type: 'code',
        text: !this.options.pedantic
          ? cap.replace(/\n+$/, '')
          : cap
      });
      continue;
    }

    // fences (gfm)
    if (cap = this.rules.fences.exec(src)) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'code',
        lang: cap[2],
        text: cap[3]
      });
      continue;
    }

    // heading
    if (cap = this.rules.heading.exec(src)) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'heading',
        depth: cap[1].length,
        text: cap[2]
      });
      continue;
    }

    // table no leading pipe (gfm)
    if (top && (cap = this.rules.nptable.exec(src))) {
      src = src.substring(cap[0].length);

      item = {
        type: 'table',
        header: cap[1].replace(/^ *| *\| *$/g, '').split(/ *\| */),
        align: cap[2].replace(/^ *|\| *$/g, '').split(/ *\| */),
        cells: cap[3].replace(/\n$/, '').split('\n')
      };

      for (i = 0; i < item.align.length; i++) {
        if (/^ *-+: *$/.test(item.align[i])) {
          item.align[i] = 'right';
        } else if (/^ *:-+: *$/.test(item.align[i])) {
          item.align[i] = 'center';
        } else if (/^ *:-+ *$/.test(item.align[i])) {
          item.align[i] = 'left';
        } else {
          item.align[i] = null;
        }
      }

      for (i = 0; i < item.cells.length; i++) {
        item.cells[i] = item.cells[i].split(/ *\| */);
      }

      this.tokens.push(item);

      continue;
    }

    // lheading
    if (cap = this.rules.lheading.exec(src)) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'heading',
        depth: cap[2] === '=' ? 1 : 2,
        text: cap[1]
      });
      continue;
    }

    // hr
    if (cap = this.rules.hr.exec(src)) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'hr'
      });
      continue;
    }

    // blockquote
    if (cap = this.rules.blockquote.exec(src)) {
      src = src.substring(cap[0].length);

      this.tokens.push({
        type: 'blockquote_start'
      });

      cap = cap[0].replace(/^ *> ?/gm, '');

      // Pass `top` to keep the current
      // "toplevel" state. This is exactly
      // how markdown.pl works.
      this.token(cap, top, true);

      this.tokens.push({
        type: 'blockquote_end'
      });

      continue;
    }

    // list
    if (cap = this.rules.list.exec(src)) {
      src = src.substring(cap[0].length);
      bull = cap[2];

      this.tokens.push({
        type: 'list_start',
        ordered: bull.length > 1
      });

      // Get each top-level item.
      cap = cap[0].match(this.rules.item);

      next = false;
      l = cap.length;
      i = 0;

      for (; i < l; i++) {
        item = cap[i];

        // Remove the list item's bullet
        // so it is seen as the next token.
        space = item.length;
        item = item.replace(/^ *([*+-]|\d+\.) +/, '');

        // Outdent whatever the
        // list item contains. Hacky.
        if (~item.indexOf('\n ')) {
          space -= item.length;
          item = !this.options.pedantic
            ? item.replace(new RegExp('^ {1,' + space + '}', 'gm'), '')
            : item.replace(/^ {1,4}/gm, '');
        }

        // Determine whether the next list item belongs here.
        // Backpedal if it does not belong in this list.
        if (this.options.smartLists && i !== l - 1) {
          b = block.bullet.exec(cap[i + 1])[0];
          if (bull !== b && !(bull.length > 1 && b.length > 1)) {
            src = cap.slice(i + 1).join('\n') + src;
            i = l - 1;
          }
        }

        // Determine whether item is loose or not.
        // Use: /(^|\n)(?! )[^\n]+\n\n(?!\s*$)/
        // for discount behavior.
        loose = next || /\n\n(?!\s*$)/.test(item);
        if (i !== l - 1) {
          next = item.charAt(item.length - 1) === '\n';
          if (!loose) loose = next;
        }

        this.tokens.push({
          type: loose
            ? 'loose_item_start'
            : 'list_item_start'
        });

        // Recurse.
        this.token(item, false, bq);

        this.tokens.push({
          type: 'list_item_end'
        });
      }

      this.tokens.push({
        type: 'list_end'
      });

      continue;
    }

    // html
    if (cap = this.rules.html.exec(src)) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: this.options.sanitize
          ? 'paragraph'
          : 'html',
        pre: cap[1] === 'pre' || cap[1] === 'script' || cap[1] === 'style',
        text: cap[0]
      });
      continue;
    }

    // def
    if ((!bq && top) && (cap = this.rules.def.exec(src))) {
      src = src.substring(cap[0].length);
      this.tokens.links[cap[1].toLowerCase()] = {
        href: cap[2],
        title: cap[3]
      };
      continue;
    }

    // table (gfm)
    if (top && (cap = this.rules.table.exec(src))) {
      src = src.substring(cap[0].length);

      item = {
        type: 'table',
        header: cap[1].replace(/^ *| *\| *$/g, '').split(/ *\| */),
        align: cap[2].replace(/^ *|\| *$/g, '').split(/ *\| */),
        cells: cap[3].replace(/(?: *\| *)?\n$/, '').split('\n')
      };

      for (i = 0; i < item.align.length; i++) {
        if (/^ *-+: *$/.test(item.align[i])) {
          item.align[i] = 'right';
        } else if (/^ *:-+: *$/.test(item.align[i])) {
          item.align[i] = 'center';
        } else if (/^ *:-+ *$/.test(item.align[i])) {
          item.align[i] = 'left';
        } else {
          item.align[i] = null;
        }
      }

      for (i = 0; i < item.cells.length; i++) {
        item.cells[i] = item.cells[i]
          .replace(/^ *\| *| *\| *$/g, '')
          .split(/ *\| */);
      }

      this.tokens.push(item);

      continue;
    }

    // top-level paragraph
    if (top && (cap = this.rules.paragraph.exec(src))) {
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'paragraph',
        text: cap[1].charAt(cap[1].length - 1) === '\n'
          ? cap[1].slice(0, -1)
          : cap[1]
      });
      continue;
    }

    // text
    if (cap = this.rules.text.exec(src)) {
      // Top-level should never reach here.
      src = src.substring(cap[0].length);
      this.tokens.push({
        type: 'text',
        text: cap[0]
      });
      continue;
    }

    if (src) {
      throw new
        Error('Infinite loop on byte: ' + src.charCodeAt(0));
    }
  }

  return this.tokens;
};

/**
 * Inline-Level Grammar
 */

var inline = {
  escape: /^\\([\\`*{}\[\]()#+\-.!_>])/,
  autolink: /^<([^ >]+(@|:\/)[^ >]+)>/,
  url: noop,
  tag: /^<!--[\s\S]*?-->|^<\/?\w+(?:"[^"]*"|'[^']*'|[^'">])*?>/,
  link: /^!?\[(inside)\]\(href\)/,
  reflink: /^!?\[(inside)\]\s*\[([^\]]*)\]/,
  nolink: /^!?\[((?:\[[^\]]*\]|[^\[\]])*)\]/,
  strong: /^__([\s\S]+?)__(?!_)|^\*\*([\s\S]+?)\*\*(?!\*)/,
  em: /^\b_((?:__|[\s\S])+?)_\b|^\*((?:\*\*|[\s\S])+?)\*(?!\*)/,
  code: /^(`+)\s*([\s\S]*?[^`])\s*\1(?!`)/,
  br: /^ {2,}\n(?!\s*$)/,
  del: noop,
  text: /^[\s\S]+?(?=[\\<!\[_*`]| {2,}\n|$)/
};

inline._inside = /(?:\[[^\]]*\]|[^\[\]]|\](?=[^\[]*\]))*/;
inline._href = /\s*<?([\s\S]*?)>?(?:\s+['"]([\s\S]*?)['"])?\s*/;

inline.link = replace(inline.link)
  ('inside', inline._inside)
  ('href', inline._href)
  ();

inline.reflink = replace(inline.reflink)
  ('inside', inline._inside)
  ();

/**
 * Normal Inline Grammar
 */

inline.normal = merge({}, inline);

/**
 * Pedantic Inline Grammar
 */

inline.pedantic = merge({}, inline.normal, {
  strong: /^__(?=\S)([\s\S]*?\S)__(?!_)|^\*\*(?=\S)([\s\S]*?\S)\*\*(?!\*)/,
  em: /^_(?=\S)([\s\S]*?\S)_(?!_)|^\*(?=\S)([\s\S]*?\S)\*(?!\*)/
});

/**
 * GFM Inline Grammar
 */

inline.gfm = merge({}, inline.normal, {
  escape: replace(inline.escape)('])', '~|])')(),
  url: /^(https?:\/\/[^\s<]+[^<.,:;"')\]\s])/,
  del: /^~~(?=\S)([\s\S]*?\S)~~/,
  text: replace(inline.text)
    (']|', '~]|')
    ('|', '|https?://|')
    ()
});

/**
 * GFM + Line Breaks Inline Grammar
 */

inline.breaks = merge({}, inline.gfm, {
  br: replace(inline.br)('{2,}', '*')(),
  text: replace(inline.gfm.text)('{2,}', '*')()
});

/**
 * Inline Lexer & Compiler
 */

function InlineLexer(links, options) {
  this.options = options || marked.defaults;
  this.links = links;
  this.rules = inline.normal;
  this.renderer = this.options.renderer || new Renderer;
  this.renderer.options = this.options;

  if (!this.links) {
    throw new
      Error('Tokens array requires a `links` property.');
  }

  if (this.options.gfm) {
    if (this.options.breaks) {
      this.rules = inline.breaks;
    } else {
      this.rules = inline.gfm;
    }
  } else if (this.options.pedantic) {
    this.rules = inline.pedantic;
  }
}

/**
 * Expose Inline Rules
 */

InlineLexer.rules = inline;

/**
 * Static Lexing/Compiling Method
 */

InlineLexer.output = function(src, links, options) {
  var inline = new InlineLexer(links, options);
  return inline.output(src);
};

/**
 * Lexing/Compiling
 */

InlineLexer.prototype.output = function(src) {
  var out = ''
    , link
    , text
    , href
    , cap;

  while (src) {
    // escape
    if (cap = this.rules.escape.exec(src)) {
      src = src.substring(cap[0].length);
      out += cap[1];
      continue;
    }

    // autolink
    if (cap = this.rules.autolink.exec(src)) {
      src = src.substring(cap[0].length);
      if (cap[2] === '@') {
        text = cap[1].charAt(6) === ':'
          ? this.mangle(cap[1].substring(7))
          : this.mangle(cap[1]);
        href = this.mangle('mailto:') + text;
      } else {
        text = escape(cap[1]);
        href = text;
      }
      out += this.renderer.link(href, null, text);
      continue;
    }

    // url (gfm)
    if (!this.inLink && (cap = this.rules.url.exec(src))) {
      src = src.substring(cap[0].length);
      text = escape(cap[1]);
      href = text;
      out += this.renderer.link(href, null, text);
      continue;
    }

    // tag
    if (cap = this.rules.tag.exec(src)) {
      if (!this.inLink && /^<a /i.test(cap[0])) {
        this.inLink = true;
      } else if (this.inLink && /^<\/a>/i.test(cap[0])) {
        this.inLink = false;
      }
      src = src.substring(cap[0].length);
      out += this.options.sanitize
        ? escape(cap[0])
        : cap[0];
      continue;
    }

    // link
    if (cap = this.rules.link.exec(src)) {
      src = src.substring(cap[0].length);
      this.inLink = true;
      out += this.outputLink(cap, {
        href: cap[2],
        title: cap[3]
      });
      this.inLink = false;
      continue;
    }

    // reflink, nolink
    if ((cap = this.rules.reflink.exec(src))
        || (cap = this.rules.nolink.exec(src))) {
      src = src.substring(cap[0].length);
      link = (cap[2] || cap[1]).replace(/\s+/g, ' ');
      link = this.links[link.toLowerCase()];
      if (!link || !link.href) {
        out += cap[0].charAt(0);
        src = cap[0].substring(1) + src;
        continue;
      }
      this.inLink = true;
      out += this.outputLink(cap, link);
      this.inLink = false;
      continue;
    }

    // strong
    if (cap = this.rules.strong.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.strong(this.output(cap[2] || cap[1]));
      continue;
    }

    // em
    if (cap = this.rules.em.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.em(this.output(cap[2] || cap[1]));
      continue;
    }

    // code
    if (cap = this.rules.code.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.codespan(escape(cap[2], true));
      continue;
    }

    // br
    if (cap = this.rules.br.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.br();
      continue;
    }

    // del (gfm)
    if (cap = this.rules.del.exec(src)) {
      src = src.substring(cap[0].length);
      out += this.renderer.del(this.output(cap[1]));
      continue;
    }

    // text
    if (cap = this.rules.text.exec(src)) {
      src = src.substring(cap[0].length);
      out += escape(this.smartypants(cap[0]));
      continue;
    }

    if (src) {
      throw new
        Error('Infinite loop on byte: ' + src.charCodeAt(0));
    }
  }

  return out;
};

/**
 * Compile Link
 */

InlineLexer.prototype.outputLink = function(cap, link) {
  var href = escape(link.href)
    , title = link.title ? escape(link.title) : null;

  return cap[0].charAt(0) !== '!'
    ? this.renderer.link(href, title, this.output(cap[1]))
    : this.renderer.image(href, title, escape(cap[1]));
};

/**
 * Smartypants Transformations
 */
// TODO MonacoChange: Our build fails over the following lines if they are not commented out
InlineLexer.prototype.smartypants = function(text) {
	return text;
//  if (!this.options.smartypants) return text;
//  return text
//    // em-dashes
//    .replace(/--/g, '\u2014')
//    // opening singles
//    .replace(/(^|[-\u2014/(\[{"\s])'/g, '$1\u2018')
//    // closing singles & apostrophes
//    .replace(/'/g, '\u2019')
//    // opening doubles
//    .replace(/(^|[-\u2014/(\[{\u2018\s])"/g, '$1\u201c')
//    // closing doubles
//    .replace(/"/g, '\u201d')
//    // ellipses
//    .replace(/\.{3}/g, '\u2026');
// END MonacoChange
};

/**
 * Mangle Links
 */

InlineLexer.prototype.mangle = function(text) {
  var out = ''
    , l = text.length
    , i = 0
    , ch;

  for (; i < l; i++) {
    ch = text.charCodeAt(i);
    if (Math.random() > 0.5) {
      ch = 'x' + ch.toString(16);
    }
    out += '&#' + ch + ';';
  }

  return out;
};

/**
 * Renderer
 */

function Renderer(options) {
  this.options = options || {};
}

Renderer.prototype.code = function(code, lang, escaped) {
  if (this.options.highlight) {
    var out = this.options.highlight(code, lang);
    if (out != null && out !== code) {
      escaped = true;
      code = out;
    }
  }

  if (!lang) {
    return '<pre><code>'
      + (escaped ? code : escape(code, true))
      + '\n</code></pre>';
  }

  return '<pre><code class="'
    + this.options.langPrefix
    + escape(lang, true)
    + '">'
    + (escaped ? code : escape(code, true))
    + '\n</code></pre>\n';
};

Renderer.prototype.blockquote = function(quote) {
  return '<blockquote>\n' + quote + '</blockquote>\n';
};

Renderer.prototype.html = function(html) {
  return html;
};

Renderer.prototype.heading = function(text, level, raw) {
  return '<h'
    + level
    + ' id="'
    + this.options.headerPrefix
    + raw.toLowerCase().replace(/[^\w]+/g, '-')
    + '">'
    + text
    + '</h'
    + level
    + '>\n';
};

Renderer.prototype.hr = function() {
  return this.options.xhtml ? '<hr/>\n' : '<hr>\n';
};

Renderer.prototype.list = function(body, ordered) {
  var type = ordered ? 'ol' : 'ul';
  return '<' + type + '>\n' + body + '</' + type + '>\n';
};

Renderer.prototype.listitem = function(text) {
  return '<li>' + text + '</li>\n';
};

Renderer.prototype.paragraph = function(text) {
  return '<p>' + text + '</p>\n';
};

Renderer.prototype.table = function(header, body) {
  return '<table>\n'
    + '<thead>\n'
    + header
    + '</thead>\n'
    + '<tbody>\n'
    + body
    + '</tbody>\n'
    + '</table>\n';
};

Renderer.prototype.tablerow = function(content) {
  return '<tr>\n' + content + '</tr>\n';
};

Renderer.prototype.tablecell = function(content, flags) {
  var type = flags.header ? 'th' : 'td';
  var tag = flags.align
    ? '<' + type + ' style="text-align:' + flags.align + '">'
    : '<' + type + '>';
  return tag + content + '</' + type + '>\n';
};

// span level renderer
Renderer.prototype.strong = function(text) {
  return '<strong>' + text + '</strong>';
};

Renderer.prototype.em = function(text) {
  return '<em>' + text + '</em>';
};

Renderer.prototype.codespan = function(text) {
  return '<code>' + text + '</code>';
};

Renderer.prototype.br = function() {
  return this.options.xhtml ? '<br/>' : '<br>';
};

Renderer.prototype.del = function(text) {
  return '<del>' + text + '</del>';
};

Renderer.prototype.link = function(href, title, text) {
  if (this.options.sanitize) {
    try {
      var prot = decodeURIComponent(unescape(href))
        .replace(/[^\w:]/g, '')
        .toLowerCase();
    } catch (e) {
      return '';
    }
    if (prot.indexOf('javascript:') === 0) {
      return '';
    }
  }
  var out = '<a href="' + href + '"';
  if (title) {
    out += ' title="' + title + '"';
  }
  out += '>' + text + '</a>';
  return out;
};

Renderer.prototype.image = function(href, title, text) {
  var out = '<img src="' + href + '" alt="' + text + '"';
  if (title) {
    out += ' title="' + title + '"';
  }
  out += this.options.xhtml ? '/>' : '>';
  return out;
};

/**
 * Parsing & Compiling
 */

function Parser(options) {
  this.tokens = [];
  this.token = null;
  this.options = options || marked.defaults;
  this.options.renderer = this.options.renderer || new Renderer;
  this.renderer = this.options.renderer;
  this.renderer.options = this.options;
}

/**
 * Static Parse Method
 */

Parser.parse = function(src, options, renderer) {
  var parser = new Parser(options, renderer);
  return parser.parse(src);
};

/**
 * Parse Loop
 */

Parser.prototype.parse = function(src) {
  this.inline = new InlineLexer(src.links, this.options, this.renderer);
  this.tokens = src.reverse();

  var out = '';
  while (this.next()) {
    out += this.tok();
  }

  return out;
};

/**
 * Next Token
 */

Parser.prototype.next = function() {
  return this.token = this.tokens.pop();
};

/**
 * Preview Next Token
 */

Parser.prototype.peek = function() {
  return this.tokens[this.tokens.length - 1] || 0;
};

/**
 * Parse Text Tokens
 */

Parser.prototype.parseText = function() {
  var body = this.token.text;

  while (this.peek().type === 'text') {
    body += '\n' + this.next().text;
  }

  return this.inline.output(body);
};

/**
 * Parse Current Token
 */

Parser.prototype.tok = function() {
  switch (this.token.type) {
    case 'space': {
      return '';
    }
    case 'hr': {
      return this.renderer.hr();
    }
    case 'heading': {
      return this.renderer.heading(
        this.inline.output(this.token.text),
        this.token.depth,
        this.token.text);
    }
    case 'code': {
      return this.renderer.code(this.token.text,
        this.token.lang,
        this.token.escaped);
    }
    case 'table': {
      var header = ''
        , body = ''
        , i
        , row
        , cell
        , flags
        , j;

      // header
      cell = '';
      for (i = 0; i < this.token.header.length; i++) {
        flags = { header: true, align: this.token.align[i] };
        cell += this.renderer.tablecell(
          this.inline.output(this.token.header[i]),
          { header: true, align: this.token.align[i] }
        );
      }
      header += this.renderer.tablerow(cell);

      for (i = 0; i < this.token.cells.length; i++) {
        row = this.token.cells[i];

        cell = '';
        for (j = 0; j < row.length; j++) {
          cell += this.renderer.tablecell(
            this.inline.output(row[j]),
            { header: false, align: this.token.align[j] }
          );
        }

        body += this.renderer.tablerow(cell);
      }
      return this.renderer.table(header, body);
    }
    case 'blockquote_start': {
      var body = '';

      while (this.next().type !== 'blockquote_end') {
        body += this.tok();
      }

      return this.renderer.blockquote(body);
    }
    case 'list_start': {
      var body = ''
        , ordered = this.token.ordered;

      while (this.next().type !== 'list_end') {
        body += this.tok();
      }

      return this.renderer.list(body, ordered);
    }
    case 'list_item_start': {
      var body = '';

      while (this.next().type !== 'list_item_end') {
        body += this.token.type === 'text'
          ? this.parseText()
          : this.tok();
      }

      return this.renderer.listitem(body);
    }
    case 'loose_item_start': {
      var body = '';

      while (this.next().type !== 'list_item_end') {
        body += this.tok();
      }

      return this.renderer.listitem(body);
    }
    case 'html': {
      var html = !this.token.pre && !this.options.pedantic
        ? this.inline.output(this.token.text)
        : this.token.text;
      return this.renderer.html(html);
    }
    case 'paragraph': {
      return this.renderer.paragraph(this.inline.output(this.token.text));
    }
    case 'text': {
      return this.renderer.paragraph(this.parseText());
    }
  }
};

/**
 * Helpers
 */

function escape(html, encode) {
  return html
    .replace(!encode ? /&(?!#?\w+;)/g : /&/g, '&amp;')
    .replace(/</g, '&lt;')
    .replace(/>/g, '&gt;')
    .replace(/"/g, '&quot;')
    .replace(/'/g, '&#39;');
}

function unescape(html) {
  return html.replace(/&([#\w]+);/g, function(_, n) {
    n = n.toLowerCase();
    if (n === 'colon') return ':';
    if (n.charAt(0) === '#') {
      return n.charAt(1) === 'x'
        ? String.fromCharCode(parseInt(n.substring(2), 16))
        : String.fromCharCode(+n.substring(1));
    }
    return '';
  });
}

function replace(regex, opt) {
  regex = regex.source;
  opt = opt || '';
  return function self(name, val) {
    if (!name) return new RegExp(regex, opt);
    val = val.source || val;
    val = val.replace(/(^|[^\[])\^/g, '$1');
    regex = regex.replace(name, val);
    return self;
  };
}

function noop() {}
noop.exec = noop;

function merge(obj) {
  var i = 1
    , target
    , key;

  for (; i < arguments.length; i++) {
    target = arguments[i];
    for (key in target) {
      if (Object.prototype.hasOwnProperty.call(target, key)) {
        obj[key] = target[key];
      }
    }
  }

  return obj;
}


/**
 * Marked
 */

function marked(src, opt, callback) {
  if (callback || typeof opt === 'function') {
    if (!callback) {
      callback = opt;
      opt = null;
    }

    opt = merge({}, marked.defaults, opt || {});

    var highlight = opt.highlight
      , tokens
      , pending
      , i = 0;

    try {
      tokens = Lexer.lex(src, opt)
    } catch (e) {
      return callback(e);
    }

    pending = tokens.length;

    var done = function(err) {
      if (err) {
        opt.highlight = highlight;
        return callback(err);
      }

      var out;

      try {
        out = Parser.parse(tokens, opt);
      } catch (e) {
        err = e;
      }

      opt.highlight = highlight;

      return err
        ? callback(err)
        : callback(null, out);
    };

    if (!highlight || highlight.length < 3) {
      return done();
    }

    delete opt.highlight;

    if (!pending) return done();

    for (; i < tokens.length; i++) {
      (function(token) {
        if (token.type !== 'code') {
          return --pending || done();
        }
        return highlight(token.text, token.lang, function(err, code) {
          if (err) return done(err);
          if (code == null || code === token.text) {
            return --pending || done();
          }
          token.text = code;
          token.escaped = true;
          --pending || done();
        });
      })(tokens[i]);
    }

    return;
  }
  try {
    if (opt) opt = merge({}, marked.defaults, opt);
    return Parser.parse(Lexer.lex(src, opt), opt);
  } catch (e) {
    e.message += '\nPlease report this to https://github.com/chjj/marked.';
    if ((opt || marked.defaults).silent) {
      return '<p>An error occured:</p><pre>'
        + escape(e.message + '', true)
        + '</pre>';
    }
    throw e;
  }
}

/**
 * Options
 */

marked.options =
marked.setOptions = function(opt) {
  merge(marked.defaults, opt);
  return marked;
};

marked.defaults = {
  gfm: true,
  tables: true,
  breaks: false,
  pedantic: false,
  sanitize: false,
  smartLists: false,
  silent: false,
  highlight: null,
  langPrefix: 'lang-',
  smartypants: false,
  headerPrefix: '',
  renderer: new Renderer,
  xhtml: false
};

/**
 * Expose
 */

marked.Parser = Parser;
marked.parser = Parser.parse;

marked.Renderer = Renderer;

marked.Lexer = Lexer;
marked.lexer = Lexer.lex;

marked.InlineLexer = InlineLexer;
marked.inlineLexer = InlineLexer.output;

marked.parse = marked;

// TODO MonacoChange: we have our own way of defining modules
this.marked = marked;
//if (typeof module !== 'undefined' && typeof exports === 'object') {
//  module.exports = marked;
//} else if (typeof define === 'function' && define.amd) {
//  define(function() { return marked; });
//} else {
//  this.marked = marked;
//}
//
//}).call(function() {
//  return this || (typeof window !== 'undefined' ? window : global);
//}());
// END MonacoChange
;
define("vs/base/common/marked/raw.marked", [], function () {
				return this.marked;
			});
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/

require.config({
	shim: {
		'vs/base/common/marked/raw.marked': {
			exports: function () {
				return this.marked;
			}
		}
	}
});

define("vs/base/common/marked/marked", ['./raw.marked'], function (marked) {
	return {
		marked: marked
	};
});
define("vs/base/common/network", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Schemas;
    (function (Schemas) {
        /**
         * A schema that is used for models that exist in memory
         * only and that have no correspondence on a server or such.
         */
        Schemas.inMemory = 'inmemory';
        Schemas.http = 'http';
        Schemas.https = 'https';
        Schemas.file = 'file';
    })(Schemas = exports.Schemas || (exports.Schemas = {}));
});

define("vs/base/common/platform", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // --- THIS FILE IS TEMPORARY UNTIL ENV.TS IS CLEANED UP. IT CAN SAFELY BE USED IN ALL TARGET EXECUTION ENVIRONMENTS (node & dom) ---
    var _isWindows = false;
    var _isMacintosh = false;
    var _isLinux = false;
    var _isNative = false;
    var _isWeb = false;
    var _isQunit = false;
    var _locale = undefined;
    var _language = undefined;
    // OS detection
    if (typeof process === 'object') {
        _isWindows = (process.platform === 'win32');
        _isMacintosh = (process.platform === 'darwin');
        _isLinux = (process.platform === 'linux');
        var vscode_nls_config = process.env['VSCODE_NLS_CONFIG'];
        if (vscode_nls_config) {
            try {
                var nlsConfig = JSON.parse(vscode_nls_config);
                var resolved = nlsConfig.availableLanguages['*'];
                _locale = nlsConfig.locale;
                // VSCode's default language is 'en'
                _language = resolved ? resolved : 'en';
            }
            catch (e) {
            }
        }
        _isNative = true;
    }
    else if (typeof navigator === 'object') {
        var userAgent = navigator.userAgent;
        _isWindows = userAgent.indexOf('Windows') >= 0;
        _isMacintosh = userAgent.indexOf('Macintosh') >= 0;
        _isLinux = userAgent.indexOf('Linux') >= 0;
        _isWeb = true;
        _locale = navigator.language;
        _language = _locale;
        _isQunit = !!self.QUnit;
    }
    (function (Platform) {
        Platform[Platform["Web"] = 0] = "Web";
        Platform[Platform["Mac"] = 1] = "Mac";
        Platform[Platform["Linux"] = 2] = "Linux";
        Platform[Platform["Windows"] = 3] = "Windows";
    })(exports.Platform || (exports.Platform = {}));
    var Platform = exports.Platform;
    exports._platform = Platform.Web;
    if (_isNative) {
        if (_isMacintosh) {
            exports._platform = Platform.Mac;
        }
        else if (_isWindows) {
            exports._platform = Platform.Windows;
        }
        else if (_isLinux) {
            exports._platform = Platform.Linux;
        }
    }
    exports.isWindows = _isWindows;
    exports.isMacintosh = _isMacintosh;
    exports.isLinux = _isLinux;
    exports.isNative = _isNative;
    exports.isWeb = _isWeb;
    exports.isQunit = _isQunit;
    exports.platform = exports._platform;
    /**
     * The language used for the user interface. The format of
     * the string is all lower case (e.g. zh-tw for Traditional
     * Chinese)
     */
    exports.language = _language;
    /**
     * The OS locale or the locale specified by --locale. The format of
     * the string is all lower case (e.g. zh-tw for Traditional
     * Chinese)
     */
    exports.locale = _locale;
    var _globals = (typeof self === 'object' ? self : global);
    exports.globals = _globals;
    function hasWebWorkerSupport() {
        return typeof _globals.Worker !== 'undefined';
    }
    exports.hasWebWorkerSupport = hasWebWorkerSupport;
    exports.setTimeout = _globals.setTimeout.bind(_globals);
    exports.clearTimeout = _globals.clearTimeout.bind(_globals);
    exports.setInterval = _globals.setInterval.bind(_globals);
    exports.clearInterval = _globals.clearInterval.bind(_globals);
});

define("vs/base/common/diagnostics", ["require", "exports", 'vs/base/common/platform'], function (require, exports, Platform) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * To enable diagnostics, open a browser console and type: window.Monaco.Diagnostics.<diagnostics name> = true.
     * Then trigger an action that will write to diagnostics to see all cached output from the past.
     */
    var globals = Platform.globals;
    if (!globals.Monaco) {
        globals.Monaco = {};
    }
    globals.Monaco.Diagnostics = {};
    var switches = globals.Monaco.Diagnostics;
    var map = {};
    var data = [];
    function fifo(array, size) {
        while (array.length > size) {
            array.shift();
        }
    }
    function register(what, fn) {
        var disable = true; // Otherwise we have unreachable code.
        if (disable) {
            return function () {
                // Intentional empty, disable for now because it is leaking memory
            };
        }
        // register switch
        var flag = switches[what] || false;
        switches[what] = flag;
        // register function
        var tracers = map[what] || [];
        tracers.push(fn);
        map[what] = tracers;
        var result = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            var idx;
            if (switches[what] === true) {
                // replay back-in-time functions
                var allArgs = [arguments];
                idx = data.indexOf(fn);
                if (idx !== -1) {
                    allArgs.unshift.apply(allArgs, data[idx + 1] || []);
                    data[idx + 1] = [];
                }
                var doIt = function () {
                    var thisArguments = allArgs.shift();
                    fn.apply(fn, thisArguments);
                    if (allArgs.length > 0) {
                        Platform.setTimeout(doIt, 500);
                    }
                };
                doIt();
            }
            else {
                // know where to store
                idx = data.indexOf(fn);
                idx = idx !== -1 ? idx : data.length;
                var dataIdx = idx + 1;
                // store arguments
                var allargs = data[dataIdx] || [];
                allargs.push(arguments);
                fifo(allargs, 50);
                // store data
                data[idx] = fn;
                data[dataIdx] = allargs;
            }
        };
        return result;
    }
    exports.register = register;
});

define("vs/base/common/flags", ["require", "exports", 'vs/base/common/platform'], function (require, exports, platform_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.enableTasks = environment('enableTasks');
    exports.enableSendASmile = environment('enableSendASmile');
    exports.enableJavaScriptRewriting = environment('enableJavaScriptRewriting');
    exports.enableTypeScriptServiceMode = environment('enableTypeScriptServiceMode');
    exports.enableTypeScriptServiceModeForJS = environment('enableTypeScriptServiceModeForJS');
    // Telemetry endpoint (used in the standalone editor) for hosts that want to collect editor telemetry
    exports.standaloneEditorTelemetryEndpoint = environment('telemetryEndpoint', null);
    // Option for hosts to overwrite the worker script url (used in the standalone editor)
    exports.getCrossOriginWorkerScriptUrl = environment('getWorkerUrl', null);
    function environment(name, fallback) {
        if (fallback === void 0) { fallback = false; }
        if (platform_1.globals.MonacoEnvironment && platform_1.globals.MonacoEnvironment.hasOwnProperty(name)) {
            return platform_1.globals.MonacoEnvironment[name];
        }
        return fallback;
    }
    function workersCount(defaultCount) {
        return environment('workersCount', defaultCount);
    }
    exports.workersCount = workersCount;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/common/scorer", ["require", "exports"], function (require, exports) {
    'use strict';
    // Based on material from:
    /*!
    * string_score.js: String Scoring Algorithm 0.1.22
    *
    * http://joshaven.com/string_score
    * https://github.com/joshaven/string_score
    *
    * Copyright (C) 2009-2014 Joshaven Potter <yourtech@gmail.com>
    * Special thanks to all of the contributors listed here https://github.com/joshaven/string_score
    * MIT License: http://opensource.org/licenses/MIT
    *
    * Date: Tue Mar 1 2011
    * Updated: Tue Mar 10 2015
    */
    /**
     * Compute a score for the given string and the given query.
     *
     * Rules:
     * Character score: 1
     * Same case bonus: 1
     * Upper case bonus: 1
     * Start of word/path bonus: 7
     * Start of string bonus: 8
     */
    var wordPathBoundary = ['-', '_', ' ', '/', '\\', '.'];
    function score(target, query, cache) {
        if (!target || !query) {
            return 0; // return early if target or query are undefined
        }
        var hash = target + query;
        var cached = cache && cache[hash];
        if (typeof cached === 'number') {
            return cached;
        }
        var queryLen = query.length;
        var targetLower = target.toLowerCase();
        var queryLower = query.toLowerCase();
        var index = 0;
        var lastIndexOf = -1;
        var score = 0;
        while (index < queryLen) {
            var indexOf = targetLower.indexOf(queryLower[index], lastIndexOf + 1);
            if (indexOf < 0) {
                score = 0; // This makes sure that the query is contained in the target
                break;
            }
            lastIndexOf = indexOf;
            // Character Match Bonus
            score += 1;
            // Same Case Bonous
            if (target[indexOf] === query[indexOf]) {
                score += 1;
            }
            // Prefix Bonus
            if (indexOf === 0) {
                score += 8;
            }
            else if (wordPathBoundary.some(function (w) { return w === target[indexOf - 1]; })) {
                score += 7;
            }
            else if (isUpper(target.charCodeAt(indexOf))) {
                score += 1;
            }
            index++;
        }
        if (cache) {
            cache[hash] = score;
        }
        return score;
    }
    exports.score = score;
    function isUpper(code) {
        return 65 <= code && code <= 90;
    }
    /**
     * A fast method to check if a given string would produce a score > 0 for the given query.
     */
    function matches(target, queryLower) {
        if (!target || !queryLower) {
            return false; // return early if target or query are undefined
        }
        var queryLen = queryLower.length;
        var targetLower = target.toLowerCase();
        var index = 0;
        var lastIndexOf = -1;
        while (index < queryLen) {
            var indexOf = targetLower.indexOf(queryLower[index], lastIndexOf + 1);
            if (indexOf < 0) {
                return false;
            }
            lastIndexOf = indexOf;
            index++;
        }
        return true;
    }
    exports.matches = matches;
});

define("vs/base/common/sortedList", ["require", "exports", 'vs/base/common/arrays'], function (require, exports, Arrays) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var SortedList = (function () {
        function SortedList(comparator) {
            this.keys = [];
            this.values = [];
            this.comparator = comparator || SortedList.DEFAULT_COMPARATOR;
        }
        Object.defineProperty(SortedList.prototype, "count", {
            get: function () {
                return this.keys.length;
            },
            enumerable: true,
            configurable: true
        });
        SortedList.prototype.getValueByIndex = function (index) {
            if (0 <= index && index < this.values.length) {
                return this.values[index];
            }
            return null;
        };
        SortedList.prototype.getKey = function (index) {
            if (0 <= index && index < this.keys.length) {
                return this.keys[index];
            }
            return null;
        };
        SortedList.prototype.getKeys = function () {
            return new ListIterator(this.keys);
        };
        SortedList.prototype.getValue = function (key) {
            if (!key) {
                throw new Error('Key must be defined.');
            }
            var indexOfKey = this.indexOfKey(key);
            if (indexOfKey >= 0) {
                return this.values[indexOfKey];
            }
            return null;
        };
        SortedList.prototype.getValues = function () {
            return new ListIterator(this.values);
        };
        SortedList.prototype.indexOfKey = function (key) {
            if (!key) {
                throw new Error('Key must be defined.');
            }
            return Math.max(-1, Arrays.binarySearch(this.keys, key, this.comparator));
        };
        SortedList.prototype.add = function (key, value) {
            if (!key || !value) {
                throw new Error('Key and value must be defined.');
            }
            var position = 0;
            while (position < this.keys.length && this.comparator(key, this.keys[position]) > 0) {
                position++;
            }
            this.keys.splice(position, 0, key);
            this.values.splice(position, 0, value);
        };
        SortedList.prototype.remove = function (key) {
            if (!key) {
                throw new Error('Key must be defined.');
            }
            var indexOfKey = this.indexOfKey(key);
            if (indexOfKey >= 0) {
                this.values.splice(indexOfKey, 1);
                this.keys.splice(indexOfKey, 1);
            }
            return indexOfKey >= 0;
        };
        SortedList.prototype.getIterator = function () {
            return new SortedListIterator(this.keys, this.values);
        };
        SortedList.DEFAULT_COMPARATOR = function (first, second) {
            return first < second ? -1 : first > second ? 1 : 0;
        };
        return SortedList;
    }());
    exports.SortedList = SortedList;
    var SortedListIterator = (function () {
        function SortedListIterator(keys, values) {
            this.keys = keys;
            this.values = values;
            this.index = -1;
        }
        Object.defineProperty(SortedListIterator.prototype, "current", {
            get: function () {
                if (this.index < 0 || this.keys.length < this.index) {
                    return null;
                }
                return {
                    key: this.keys[this.index],
                    value: this.values[this.index]
                };
            },
            enumerable: true,
            configurable: true
        });
        SortedListIterator.prototype.moveNext = function () {
            this.index++;
            return this.index < this.keys.length;
        };
        SortedListIterator.prototype.hasNext = function () {
            return this.index + 1 < this.keys.length;
        };
        SortedListIterator.prototype.reset = function () {
            this.index = -1;
        };
        SortedListIterator.prototype.dispose = function () {
            this.keys = null;
            this.values = null;
        };
        return SortedListIterator;
    }());
    var ListIterator = (function () {
        function ListIterator(values) {
            this.values = values;
            this.index = -1;
        }
        Object.defineProperty(ListIterator.prototype, "current", {
            get: function () {
                if (this.index < 0 || this.values.length < this.index) {
                    return null;
                }
                return this.values[this.index];
            },
            enumerable: true,
            configurable: true
        });
        ListIterator.prototype.moveNext = function () {
            this.index++;
            return this.index < this.values.length;
        };
        ListIterator.prototype.hasNext = function () {
            return this.index + 1 < this.values.length;
        };
        ListIterator.prototype.reset = function () {
            this.index = -1;
        };
        ListIterator.prototype.dispose = function () {
            this.values = null;
        };
        return ListIterator;
    }());
});

define("vs/base/common/stopwatch", ["require", "exports", 'vs/base/common/platform'], function (require, exports, platform_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var hasPerformanceNow = (platform_1.globals.performance && typeof platform_1.globals.performance.now === 'function');
    var StopWatch = (function () {
        function StopWatch(highResolution) {
            this._highResolution = hasPerformanceNow && highResolution;
            this._startTime = this._now();
            this._stopTime = -1;
        }
        StopWatch.create = function (highResolution) {
            if (highResolution === void 0) { highResolution = true; }
            return new StopWatch(highResolution);
        };
        StopWatch.prototype.stop = function () {
            this._stopTime = this._now();
        };
        StopWatch.prototype.elapsed = function () {
            if (this._stopTime !== -1) {
                return this._stopTime - this._startTime;
            }
            return this._now() - this._startTime;
        };
        StopWatch.prototype._now = function () {
            return this._highResolution ? platform_1.globals.performance.now() : new Date().getTime();
        };
        return StopWatch;
    }());
    exports.StopWatch = StopWatch;
});

define("vs/base/common/strings", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The empty string.
     */
    exports.empty = '';
    /**
     * @returns the provided number with the given number of preceding zeros.
     */
    function pad(n, l, char) {
        if (char === void 0) { char = '0'; }
        var str = '' + n;
        var r = [str];
        for (var i = str.length; i < l; i++) {
            r.push(char);
        }
        return r.reverse().join('');
    }
    exports.pad = pad;
    var _formatRegexp = /{(\d+)}/g;
    /**
     * Helper to produce a string with a variable number of arguments. Insert variable segments
     * into the string using the {n} notation where N is the index of the argument following the string.
     * @param value string to which formatting is applied
     * @param args replacements for {n}-entries
     */
    function format(value) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (args.length === 0) {
            return value;
        }
        return value.replace(_formatRegexp, function (match, group) {
            var idx = parseInt(group, 10);
            return isNaN(idx) || idx < 0 || idx >= args.length ?
                match :
                args[idx];
        });
    }
    exports.format = format;
    /**
     * Converts HTML characters inside the string to use entities instead. Makes the string safe from
     * being used e.g. in HTMLElement.innerHTML.
     */
    function escape(html) {
        return html.replace(/[<|>|&]/g, function (match) {
            switch (match) {
                case '<': return '&lt;';
                case '>': return '&gt;';
                case '&': return '&amp;';
                default: return match;
            }
        });
    }
    exports.escape = escape;
    /**
     * Escapes regular expression characters in a given string
     */
    function escapeRegExpCharacters(value) {
        return value.replace(/[\-\\\{\}\*\+\?\|\^\$\.\,\[\]\(\)\#\s]/g, '\\$&');
    }
    exports.escapeRegExpCharacters = escapeRegExpCharacters;
    /**
     * Removes all occurrences of needle from the beginning and end of haystack.
     * @param haystack string to trim
     * @param needle the thing to trim (default is a blank)
     */
    function trim(haystack, needle) {
        if (needle === void 0) { needle = ' '; }
        var trimmed = ltrim(haystack, needle);
        return rtrim(trimmed, needle);
    }
    exports.trim = trim;
    /**
     * Removes all occurrences of needle from the beginning of haystack.
     * @param haystack string to trim
     * @param needle the thing to trim
     */
    function ltrim(haystack, needle) {
        if (!haystack || !needle) {
            return haystack;
        }
        var needleLen = needle.length;
        if (needleLen === 0 || haystack.length === 0) {
            return haystack;
        }
        var offset = 0, idx = -1;
        while ((idx = haystack.indexOf(needle, offset)) === offset) {
            offset = offset + needleLen;
        }
        return haystack.substring(offset);
    }
    exports.ltrim = ltrim;
    /**
     * Removes all occurrences of needle from the end of haystack.
     * @param haystack string to trim
     * @param needle the thing to trim
     */
    function rtrim(haystack, needle) {
        if (!haystack || !needle) {
            return haystack;
        }
        var needleLen = needle.length, haystackLen = haystack.length;
        if (needleLen === 0 || haystackLen === 0) {
            return haystack;
        }
        var offset = haystackLen, idx = -1;
        while (true) {
            idx = haystack.lastIndexOf(needle, offset - 1);
            if (idx === -1 || idx + needleLen !== offset) {
                break;
            }
            if (idx === 0) {
                return '';
            }
            offset = idx;
        }
        return haystack.substring(0, offset);
    }
    exports.rtrim = rtrim;
    function convertSimple2RegExpPattern(pattern) {
        return pattern.replace(/[\-\\\{\}\+\?\|\^\$\.\,\[\]\(\)\#\s]/g, '\\$&').replace(/[\*]/g, '.*');
    }
    exports.convertSimple2RegExpPattern = convertSimple2RegExpPattern;
    function stripWildcards(pattern) {
        return pattern.replace(/\*/g, '');
    }
    exports.stripWildcards = stripWildcards;
    /**
     * Determines if haystack starts with needle.
     */
    function startsWith(haystack, needle) {
        if (haystack.length < needle.length) {
            return false;
        }
        for (var i = 0; i < needle.length; i++) {
            if (haystack[i] !== needle[i]) {
                return false;
            }
        }
        return true;
    }
    exports.startsWith = startsWith;
    /**
     * Determines if haystack ends with needle.
     */
    function endsWith(haystack, needle) {
        var diff = haystack.length - needle.length;
        if (diff > 0) {
            return haystack.lastIndexOf(needle) === haystack.length - needle.length;
        }
        else if (diff === 0) {
            return haystack === needle;
        }
        else {
            return false;
        }
    }
    exports.endsWith = endsWith;
    function createRegExp(searchString, isRegex, matchCase, wholeWord, global) {
        if (searchString === '') {
            throw new Error('Cannot create regex from empty string');
        }
        if (!isRegex) {
            searchString = searchString.replace(/[\-\\\{\}\*\+\?\|\^\$\.\,\[\]\(\)\#\s]/g, '\\$&');
        }
        if (wholeWord) {
            if (!/\B/.test(searchString.charAt(0))) {
                searchString = '\\b' + searchString;
            }
            if (!/\B/.test(searchString.charAt(searchString.length - 1))) {
                searchString = searchString + '\\b';
            }
        }
        var modifiers = '';
        if (global) {
            modifiers += 'g';
        }
        if (!matchCase) {
            modifiers += 'i';
        }
        return new RegExp(searchString, modifiers);
    }
    exports.createRegExp = createRegExp;
    /**
     * Create a regular expression only if it is valid and it doesn't lead to endless loop.
     */
    function createSafeRegExp(searchString, isRegex, matchCase, wholeWord) {
        if (searchString === '') {
            return null;
        }
        // Try to create a RegExp out of the params
        var regex = null;
        try {
            regex = createRegExp(searchString, isRegex, matchCase, wholeWord, true);
        }
        catch (err) {
            return null;
        }
        // Guard against endless loop RegExps & wrap around try-catch as very long regexes produce an exception when executed the first time
        try {
            if (regExpLeadsToEndlessLoop(regex)) {
                return null;
            }
        }
        catch (err) {
            return null;
        }
        return regex;
    }
    exports.createSafeRegExp = createSafeRegExp;
    function regExpLeadsToEndlessLoop(regexp) {
        // Exit early if it's one of these special cases which are meant to match
        // against an empty string
        if (regexp.source === '^' || regexp.source === '^$' || regexp.source === '$') {
            return false;
        }
        // We check against an empty string. If the regular expression doesn't advance
        // (e.g. ends in an endless loop) it will match an empty string.
        var match = regexp.exec('');
        return (match && regexp.lastIndex === 0);
    }
    exports.regExpLeadsToEndlessLoop = regExpLeadsToEndlessLoop;
    /**
     * The normalize() method returns the Unicode Normalization Form of a given string. The form will be
     * the Normalization Form Canonical Composition.
     *
     * @see {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/normalize}
     */
    exports.canNormalize = typeof (''.normalize) === 'function';
    var nonAsciiCharactersPattern = /[^\u0000-\u0080]/;
    var normalizedCache = Object.create(null);
    var cacheCounter = 0;
    function normalizeNFC(str) {
        if (!exports.canNormalize || !str) {
            return str;
        }
        var cached = normalizedCache[str];
        if (cached) {
            return cached;
        }
        var res;
        if (nonAsciiCharactersPattern.test(str)) {
            res = str.normalize('NFC');
        }
        else {
            res = str;
        }
        // Use the cache for fast lookup but do not let it grow unbounded
        if (cacheCounter < 10000) {
            normalizedCache[str] = res;
            cacheCounter++;
        }
        return res;
    }
    exports.normalizeNFC = normalizeNFC;
    /**
     * Returns first index of the string that is not whitespace.
     * If string is empty or contains only whitespaces, returns -1
     */
    function firstNonWhitespaceIndex(str) {
        for (var i = 0, len = str.length; i < len; i++) {
            if (str.charAt(i) !== ' ' && str.charAt(i) !== '\t') {
                return i;
            }
        }
        return -1;
    }
    exports.firstNonWhitespaceIndex = firstNonWhitespaceIndex;
    /**
     * Returns the leading whitespace of the string.
     * If the string contains only whitespaces, returns entire string
     */
    function getLeadingWhitespace(str) {
        for (var i = 0, len = str.length; i < len; i++) {
            if (str.charAt(i) !== ' ' && str.charAt(i) !== '\t') {
                return str.substring(0, i);
            }
        }
        return str;
    }
    exports.getLeadingWhitespace = getLeadingWhitespace;
    /**
     * Returns last index of the string that is not whitespace.
     * If string is empty or contains only whitespaces, returns -1
     */
    function lastNonWhitespaceIndex(str) {
        for (var i = str.length - 1; i >= 0; i--) {
            if (str.charAt(i) !== ' ' && str.charAt(i) !== '\t') {
                return i;
            }
        }
        return -1;
    }
    exports.lastNonWhitespaceIndex = lastNonWhitespaceIndex;
    function localeCompare(strA, strB) {
        return strA.localeCompare(strB);
    }
    exports.localeCompare = localeCompare;
    function isAsciiChar(code) {
        return (code >= 97 && code <= 122) || (code >= 65 && code <= 90);
    }
    function equalsIgnoreCase(a, b) {
        var len1 = a.length, len2 = b.length;
        if (len1 !== len2) {
            return false;
        }
        for (var i = 0; i < len1; i++) {
            var codeA = a.charCodeAt(i), codeB = b.charCodeAt(i);
            if (codeA === codeB) {
                continue;
            }
            else if (isAsciiChar(codeA) && isAsciiChar(codeB)) {
                var diff = Math.abs(codeA - codeB);
                if (diff !== 0 && diff !== 32) {
                    return false;
                }
            }
            else {
                if (String.fromCharCode(codeA).toLocaleLowerCase() !== String.fromCharCode(codeB).toLocaleLowerCase()) {
                    return false;
                }
            }
        }
        return true;
    }
    exports.equalsIgnoreCase = equalsIgnoreCase;
    /**
     * @returns the length of the common prefix of the two strings.
     */
    function commonPrefixLength(a, b) {
        var i, len = Math.min(a.length, b.length);
        for (i = 0; i < len; i++) {
            if (a.charCodeAt(i) !== b.charCodeAt(i)) {
                return i;
            }
        }
        return len;
    }
    exports.commonPrefixLength = commonPrefixLength;
    /**
     * @returns the length of the common suffix of the two strings.
     */
    function commonSuffixLength(a, b) {
        var i, len = Math.min(a.length, b.length);
        var aLastIndex = a.length - 1;
        var bLastIndex = b.length - 1;
        for (i = 0; i < len; i++) {
            if (a.charCodeAt(aLastIndex - i) !== b.charCodeAt(bLastIndex - i)) {
                return i;
            }
        }
        return len;
    }
    exports.commonSuffixLength = commonSuffixLength;
    // --- unicode
    // http://en.wikipedia.org/wiki/Surrogate_pair
    // Returns the code point starting at a specified index in a string
    // Code points U+0000 to U+D7FF and U+E000 to U+FFFF are represented on a single character
    // Code points U+10000 to U+10FFFF are represented on two consecutive characters
    //export function getUnicodePoint(str:string, index:number, len:number):number {
    //	let chrCode = str.charCodeAt(index);
    //	if (0xD800 <= chrCode && chrCode <= 0xDBFF && index + 1 < len) {
    //		let nextChrCode = str.charCodeAt(index + 1);
    //		if (0xDC00 <= nextChrCode && nextChrCode <= 0xDFFF) {
    //			return (chrCode - 0xD800) << 10 + (nextChrCode - 0xDC00) + 0x10000;
    //		}
    //	}
    //	return chrCode;
    //}
    //export function isLeadSurrogate(chr:string) {
    //	let chrCode = chr.charCodeAt(0);
    //	return ;
    //}
    //
    //export function isTrailSurrogate(chr:string) {
    //	let chrCode = chr.charCodeAt(0);
    //	return 0xDC00 <= chrCode && chrCode <= 0xDFFF;
    //}
    function isFullWidthCharacter(charCode) {
        // Do a cheap trick to better support wrapping of wide characters, treat them as 2 columns
        // http://jrgraphix.net/research/unicode_blocks.php
        //          2E80 — 2EFF   CJK Radicals Supplement
        //          2F00 — 2FDF   Kangxi Radicals
        //          2FF0 — 2FFF   Ideographic Description Characters
        //          3000 — 303F   CJK Symbols and Punctuation
        //          3040 — 309F   Hiragana
        //          30A0 — 30FF   Katakana
        //          3100 — 312F   Bopomofo
        //          3130 — 318F   Hangul Compatibility Jamo
        //          3190 — 319F   Kanbun
        //          31A0 — 31BF   Bopomofo Extended
        //          31F0 — 31FF   Katakana Phonetic Extensions
        //          3200 — 32FF   Enclosed CJK Letters and Months
        //          3300 — 33FF   CJK Compatibility
        //          3400 — 4DBF   CJK Unified Ideographs Extension A
        //          4DC0 — 4DFF   Yijing Hexagram Symbols
        //          4E00 — 9FFF   CJK Unified Ideographs
        //          A000 — A48F   Yi Syllables
        //          A490 — A4CF   Yi Radicals
        //          AC00 — D7AF   Hangul Syllables
        // [IGNORE] D800 — DB7F   High Surrogates
        // [IGNORE] DB80 — DBFF   High Private Use Surrogates
        // [IGNORE] DC00 — DFFF   Low Surrogates
        // [IGNORE] E000 — F8FF   Private Use Area
        //          F900 — FAFF   CJK Compatibility Ideographs
        // [IGNORE] FB00 — FB4F   Alphabetic Presentation Forms
        // [IGNORE] FB50 — FDFF   Arabic Presentation Forms-A
        // [IGNORE] FE00 — FE0F   Variation Selectors
        // [IGNORE] FE20 — FE2F   Combining Half Marks
        // [IGNORE] FE30 — FE4F   CJK Compatibility Forms
        // [IGNORE] FE50 — FE6F   Small Form Variants
        // [IGNORE] FE70 — FEFF   Arabic Presentation Forms-B
        //          FF00 — FFEF   Halfwidth and Fullwidth Forms
        //               [https://en.wikipedia.org/wiki/Halfwidth_and_fullwidth_forms]
        //               of which FF01 - FF5E fullwidth ASCII of 21 to 7E
        // [IGNORE]    and FF65 - FFDC halfwidth of Katakana and Hangul
        // [IGNORE] FFF0 — FFFF   Specials
        return ((charCode >= 0x2E80 && charCode <= 0xD7AF)
            || (charCode >= 0xF900 && charCode <= 0xFAFF)
            || (charCode >= 0xFF01 && charCode <= 0xFF5E));
    }
    exports.isFullWidthCharacter = isFullWidthCharacter;
    /**
     * Computes the difference score for two strings. More similar strings have a higher score.
     * We use largest common subsequence dynamic programming approach but penalize in the end for length differences.
     * Strings that have a large length difference will get a bad default score 0.
     * Complexity - both time and space O(first.length * second.length)
     * Dynamic programming LCS computation http://en.wikipedia.org/wiki/Longest_common_subsequence_problem
     *
     * @param first a string
     * @param second a string
     */
    function difference(first, second, maxLenDelta) {
        if (maxLenDelta === void 0) { maxLenDelta = 4; }
        var lengthDifference = Math.abs(first.length - second.length);
        // We only compute score if length of the currentWord and length of entry.name are similar.
        if (lengthDifference > maxLenDelta) {
            return 0;
        }
        // Initialize LCS (largest common subsequence) matrix.
        var LCS = [];
        var zeroArray = [];
        var i, j;
        for (i = 0; i < second.length + 1; ++i) {
            zeroArray.push(0);
        }
        for (i = 0; i < first.length + 1; ++i) {
            LCS.push(zeroArray);
        }
        for (i = 1; i < first.length + 1; ++i) {
            for (j = 1; j < second.length + 1; ++j) {
                if (first[i - 1] === second[j - 1]) {
                    LCS[i][j] = LCS[i - 1][j - 1] + 1;
                }
                else {
                    LCS[i][j] = Math.max(LCS[i - 1][j], LCS[i][j - 1]);
                }
            }
        }
        return LCS[first.length][second.length] - Math.sqrt(lengthDifference);
    }
    exports.difference = difference;
    /**
     * Returns an array in which every entry is the offset of a
     * line. There is always one entry which is zero.
     */
    function computeLineStarts(text) {
        var regexp = /\r\n|\r|\n/g, ret = [0], match;
        while ((match = regexp.exec(text))) {
            ret.push(regexp.lastIndex);
        }
        return ret;
    }
    exports.computeLineStarts = computeLineStarts;
    /**
     * Given a string and a max length returns a shorted version. Shorting
     * happens at favorable positions - such as whitespace or punctuation characters.
     */
    function lcut(text, n) {
        if (text.length < n) {
            return text;
        }
        var segments = text.split(/\b/), count = 0;
        for (var i = segments.length - 1; i >= 0; i--) {
            count += segments[i].length;
            if (count > n) {
                segments.splice(0, i);
                break;
            }
        }
        return segments.join(exports.empty).replace(/^\s/, exports.empty);
    }
    exports.lcut = lcut;
    // Escape codes
    // http://en.wikipedia.org/wiki/ANSI_escape_code
    var EL = /\x1B\x5B[12]?K/g; // Erase in line
    var LF = /\xA/g; // line feed
    var COLOR_START = /\x1b\[\d+m/g; // Color
    var COLOR_END = /\x1b\[0?m/g; // Color
    function removeAnsiEscapeCodes(str) {
        if (str) {
            str = str.replace(EL, '');
            str = str.replace(LF, '\n');
            str = str.replace(COLOR_START, '');
            str = str.replace(COLOR_END, '');
        }
        return str;
    }
    exports.removeAnsiEscapeCodes = removeAnsiEscapeCodes;
    // -- UTF-8 BOM
    var __utf8_bom = 65279;
    exports.UTF8_BOM_CHARACTER = String.fromCharCode(__utf8_bom);
    function startsWithUTF8BOM(str) {
        return (str && str.length > 0 && str.charCodeAt(0) === __utf8_bom);
    }
    exports.startsWithUTF8BOM = startsWithUTF8BOM;
});

define("vs/base/common/comparers", ["require", "exports", 'vs/base/common/strings'], function (require, exports, strings) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FileNameMatch = /^(.*)\.([^.]*)|([^.]+)$/;
    function compareFileNames(one, other) {
        var oneMatch = FileNameMatch.exec(one.toLowerCase());
        var otherMatch = FileNameMatch.exec(other.toLowerCase());
        var oneName = oneMatch[1] || oneMatch[3] || '';
        var oneExtension = oneMatch[2] || '';
        var otherName = otherMatch[1] || otherMatch[3] || '';
        var otherExtension = otherMatch[2] || '';
        if (oneName !== otherName) {
            return oneName < otherName ? -1 : 1;
        }
        return oneExtension < otherExtension ? -1 : 1;
    }
    exports.compareFileNames = compareFileNames;
    function compareAnything(one, other, lookFor) {
        var elementAName = one.toLowerCase();
        var elementBName = other.toLowerCase();
        // Sort prefix matches over non prefix matches
        var prefixCompare = compareByPrefix(one, other, lookFor);
        if (prefixCompare) {
            return prefixCompare;
        }
        // Sort suffix matches over non suffix matches
        var elementASuffixMatch = strings.endsWith(elementAName, lookFor);
        var elementBSuffixMatch = strings.endsWith(elementBName, lookFor);
        if (elementASuffixMatch !== elementBSuffixMatch) {
            return elementASuffixMatch ? -1 : 1;
        }
        // Understand file names
        var r = compareFileNames(elementAName, elementBName);
        if (r !== 0) {
            return r;
        }
        // Compare by name
        return strings.localeCompare(elementAName, elementBName);
    }
    exports.compareAnything = compareAnything;
    function compareByPrefix(one, other, lookFor) {
        var elementAName = one.toLowerCase();
        var elementBName = other.toLowerCase();
        // Sort prefix matches over non prefix matches
        var elementAPrefixMatch = elementAName.indexOf(lookFor) === 0;
        var elementBPrefixMatch = elementBName.indexOf(lookFor) === 0;
        if (elementAPrefixMatch !== elementBPrefixMatch) {
            return elementAPrefixMatch ? -1 : 1;
        }
        else if (elementAPrefixMatch && elementBPrefixMatch) {
            if (elementAName.length < elementBName.length) {
                return -1;
            }
            if (elementAName.length > elementBName.length) {
                return 1;
            }
        }
        return 0;
    }
    exports.compareByPrefix = compareByPrefix;
});

define("vs/base/common/filters", ["require", "exports", 'vs/base/common/strings'], function (require, exports, strings) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // Combined filters
    /**
     * @returns A filter which combines the provided set
     * of filters with an or. The *first* filters that
     * matches defined the return value of the returned
     * filter.
     */
    function or() {
        var filter = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            filter[_i - 0] = arguments[_i];
        }
        return function (word, wordToMatchAgainst) {
            for (var i = 0, len = filter.length; i < len; i++) {
                var match = filter[i](word, wordToMatchAgainst);
                if (match) {
                    return match;
                }
            }
            return null;
        };
    }
    exports.or = or;
    /**
     * @returns A filter which combines the provided set
     * of filters with an and. The combines matches are
     * returned if *all* filters match.
     */
    function and() {
        var filter = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            filter[_i - 0] = arguments[_i];
        }
        return function (word, wordToMatchAgainst) {
            var result = [];
            for (var i = 0, len = filter.length; i < len; i++) {
                var match = filter[i](word, wordToMatchAgainst);
                if (!match) {
                    return null;
                }
                result = result.concat(match);
            }
            return result;
        };
    }
    exports.and = and;
    // Prefix
    exports.matchesStrictPrefix = function (word, wordToMatchAgainst) { return _matchesPrefix(false, word, wordToMatchAgainst); };
    exports.matchesPrefix = function (word, wordToMatchAgainst) { return _matchesPrefix(true, word, wordToMatchAgainst); };
    function _matchesPrefix(ignoreCase, word, wordToMatchAgainst) {
        if (wordToMatchAgainst.length === 0 || wordToMatchAgainst.length < word.length) {
            return null;
        }
        if (ignoreCase) {
            word = word.toLowerCase();
            wordToMatchAgainst = wordToMatchAgainst.toLowerCase();
        }
        for (var i = 0; i < word.length; i++) {
            if (word[i] !== wordToMatchAgainst[i]) {
                return null;
            }
        }
        return word.length > 0 ? [{ start: 0, end: word.length }] : [];
    }
    // Contiguous Substring
    function matchesContiguousSubString(word, wordToMatchAgainst) {
        var index = wordToMatchAgainst.toLowerCase().indexOf(word.toLowerCase());
        if (index === -1) {
            return null;
        }
        return [{ start: index, end: index + word.length }];
    }
    exports.matchesContiguousSubString = matchesContiguousSubString;
    // Substring
    function matchesSubString(word, wordToMatchAgainst) {
        return _matchesSubString(word.toLowerCase(), wordToMatchAgainst.toLowerCase(), 0, 0);
    }
    exports.matchesSubString = matchesSubString;
    function _matchesSubString(word, wordToMatchAgainst, i, j) {
        if (i === word.length) {
            return [];
        }
        else if (j === wordToMatchAgainst.length) {
            return null;
        }
        else {
            if (word[i] === wordToMatchAgainst[j]) {
                var result = null;
                if (result = _matchesSubString(word, wordToMatchAgainst, i + 1, j + 1)) {
                    return join({ start: j, end: j + 1 }, result);
                }
            }
            return _matchesSubString(word, wordToMatchAgainst, i, j + 1);
        }
    }
    // CamelCase
    function isLower(code) {
        return 97 <= code && code <= 122;
    }
    function isUpper(code) {
        return 65 <= code && code <= 90;
    }
    function isNumber(code) {
        return 48 <= code && code <= 57;
    }
    function isWhitespace(code) {
        return [32, 9, 10, 13].indexOf(code) > -1;
    }
    function isAlphanumeric(code) {
        return isLower(code) || isUpper(code) || isNumber(code);
    }
    function join(head, tail) {
        if (tail.length === 0) {
            tail = [head];
        }
        else if (head.end === tail[0].start) {
            tail[0].start = head.start;
        }
        else {
            tail.unshift(head);
        }
        return tail;
    }
    function nextAnchor(camelCaseWord, start) {
        for (var i = start; i < camelCaseWord.length; i++) {
            var c = camelCaseWord.charCodeAt(i);
            if (isUpper(c) || isNumber(c) || (i > 0 && !isAlphanumeric(camelCaseWord.charCodeAt(i - 1)))) {
                return i;
            }
        }
        return camelCaseWord.length;
    }
    function _matchesCamelCase(word, camelCaseWord, i, j) {
        if (i === word.length) {
            return [];
        }
        else if (j === camelCaseWord.length) {
            return null;
        }
        else if (word[i] !== camelCaseWord[j].toLowerCase()) {
            return null;
        }
        else {
            var result = null;
            var nextUpperIndex = j + 1;
            result = _matchesCamelCase(word, camelCaseWord, i + 1, j + 1);
            while (!result && (nextUpperIndex = nextAnchor(camelCaseWord, nextUpperIndex)) < camelCaseWord.length) {
                result = _matchesCamelCase(word, camelCaseWord, i + 1, nextUpperIndex);
                nextUpperIndex++;
            }
            return result === null ? null : join({ start: j, end: j + 1 }, result);
        }
    }
    // Heuristic to avoid computing camel case matcher for words that don't
    // look like camelCaseWords.
    function isCamelCaseWord(word) {
        if (word.length > 60) {
            return false;
        }
        var upper = 0, lower = 0, alpha = 0, numeric = 0, code = 0;
        for (var i = 0; i < word.length; i++) {
            code = word.charCodeAt(i);
            if (isUpper(code)) {
                upper++;
            }
            if (isLower(code)) {
                lower++;
            }
            if (isAlphanumeric(code)) {
                alpha++;
            }
            if (isNumber(code)) {
                numeric++;
            }
        }
        var upperPercent = upper / word.length;
        var lowerPercent = lower / word.length;
        var alphaPercent = alpha / word.length;
        var numericPercent = numeric / word.length;
        return lowerPercent > 0.2 && upperPercent < 0.8 && alphaPercent > 0.6 && numericPercent < 0.2;
    }
    // Heuristic to avoid computing camel case matcher for words that don't
    // look like camel case patterns.
    function isCamelCasePattern(word) {
        var upper = 0, lower = 0, code = 0, whitespace = 0;
        for (var i = 0; i < word.length; i++) {
            code = word.charCodeAt(i);
            if (isUpper(code)) {
                upper++;
            }
            if (isLower(code)) {
                lower++;
            }
            if (isWhitespace(code)) {
                whitespace++;
            }
        }
        if ((upper === 0 || lower === 0) && whitespace === 0) {
            return word.length <= 30;
        }
        else {
            return upper <= 5;
        }
    }
    function matchesCamelCase(word, camelCaseWord) {
        if (camelCaseWord.length === 0) {
            return null;
        }
        if (!isCamelCasePattern(word)) {
            return null;
        }
        if (!isCamelCaseWord(camelCaseWord)) {
            return null;
        }
        var result = null;
        var i = 0;
        while (i < camelCaseWord.length && (result = _matchesCamelCase(word.toLowerCase(), camelCaseWord, 0, i)) === null) {
            i = nextAnchor(camelCaseWord, i + 1);
        }
        return result;
    }
    exports.matchesCamelCase = matchesCamelCase;
    // Fuzzy
    (function (SubstringMatching) {
        SubstringMatching[SubstringMatching["Contiguous"] = 0] = "Contiguous";
        SubstringMatching[SubstringMatching["Separate"] = 1] = "Separate";
    })(exports.SubstringMatching || (exports.SubstringMatching = {}));
    var SubstringMatching = exports.SubstringMatching;
    var fuzzyContiguousFilter = or(exports.matchesPrefix, matchesCamelCase, matchesContiguousSubString);
    var fuzzySeparateFilter = or(exports.matchesPrefix, matchesCamelCase, matchesSubString);
    var fuzzyRegExpCache = {};
    function matchesFuzzy(word, wordToMatchAgainst, enableSeparateSubstringMatching) {
        if (enableSeparateSubstringMatching === void 0) { enableSeparateSubstringMatching = false; }
        // Form RegExp for wildcard matches
        var regexp = fuzzyRegExpCache[word];
        if (!regexp) {
            regexp = new RegExp(strings.convertSimple2RegExpPattern(word), 'i');
            fuzzyRegExpCache[word] = regexp;
        }
        // RegExp Filter
        var match = regexp.exec(wordToMatchAgainst);
        if (match) {
            return [{ start: match.index, end: match.index + match[0].length }];
        }
        // Default Filter
        return enableSeparateSubstringMatching ? fuzzySeparateFilter(word, wordToMatchAgainst) : fuzzyContiguousFilter(word, wordToMatchAgainst);
    }
    exports.matchesFuzzy = matchesFuzzy;
});

define("vs/base/common/paths", ["require", "exports", 'vs/base/common/platform', 'vs/base/common/strings'], function (require, exports, platform_1, strings_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The forward slash path separator.
     */
    exports.sep = '/';
    /**
     * The native path separator depending on the OS.
     */
    exports.nativeSep = platform_1.isWindows ? '\\' : '/';
    function relative(from, to) {
        from = normalize(from);
        to = normalize(to);
        var fromParts = from.split(exports.sep), toParts = to.split(exports.sep);
        while (fromParts.length > 0 && toParts.length > 0) {
            if (fromParts[0] === toParts[0]) {
                fromParts.shift();
                toParts.shift();
            }
            else {
                break;
            }
        }
        for (var i = 0, len = fromParts.length; i < len; i++) {
            toParts.unshift('..');
        }
        return toParts.join(exports.sep);
    }
    exports.relative = relative;
    var _dotSegment = /[\\\/]\.\.?[\\\/]?|[\\\/]?\.\.?[\\\/]/;
    function normalize(path, toOSPath) {
        if (!path) {
            return path;
        }
        // a path is already normal if it contains no .. or . parts
        // and already uses the proper path separator
        if (!_dotSegment.test(path)) {
            // badSep is the path separator we don't want. Usually
            // the backslash, unless isWindows && toOSPath
            var badSep = toOSPath && platform_1.isWindows ? '/' : '\\';
            if (path.indexOf(badSep) === -1) {
                return path;
            }
        }
        var parts = path.split(/[\\\/]/);
        for (var i = 0, len = parts.length; i < len; i++) {
            if (parts[i] === '.' && !!parts[i + 1]) {
                parts.splice(i, 1);
                i -= 1;
            }
            else if (parts[i] === '..' && !!parts[i - 1]) {
                parts.splice(i - 1, 2);
                i -= 2;
            }
        }
        return parts.join(toOSPath ? exports.nativeSep : exports.sep);
    }
    exports.normalize = normalize;
    function dirnames(path) {
        var value = path, done = false;
        function next() {
            if (value === '.' || value === '/' || value === '\\') {
                value = undefined;
                done = true;
            }
            else {
                value = dirname(value);
            }
            return {
                value: value,
                done: done
            };
        }
        return {
            next: next
        };
    }
    exports.dirnames = dirnames;
    /**
     * @returns the directory name of a path.
     */
    function dirname(path) {
        var idx = ~path.lastIndexOf('/') || ~path.lastIndexOf('\\');
        if (idx === 0) {
            return '.';
        }
        else if (~idx === 0) {
            return path[0];
        }
        else {
            return path.substring(0, ~idx);
        }
    }
    exports.dirname = dirname;
    /**
     * @returns the base name of a path.
     */
    function basename(path) {
        var idx = ~path.lastIndexOf('/') || ~path.lastIndexOf('\\');
        if (idx === 0) {
            return path;
        }
        else if (~idx === path.length - 1) {
            return basename(path.substring(0, path.length - 1));
        }
        else {
            return path.substr(~idx + 1);
        }
    }
    exports.basename = basename;
    /**
     * @returns {{.far}} from boo.far or the empty string.
     */
    function extname(path) {
        path = basename(path);
        var idx = ~path.lastIndexOf('.');
        return idx ? path.substring(~idx) : '';
    }
    exports.extname = extname;
    function getRootLength(path) {
        if (!path) {
            return 0;
        }
        path = path.replace(/\/|\\/g, '/');
        if (path[0] === '/') {
            if (path[1] !== '/') {
                // /far/boo
                return 1;
            }
            else {
                // //server/far/boo
                return 2;
            }
        }
        if (path[1] === ':') {
            if (path[2] === '/') {
                // c:/boo/far.txt
                return 3;
            }
            else {
                // c:
                return 2;
            }
        }
        if (path.indexOf('file:///') === 0) {
            return 8; // 8 -> 'file:///'.length
        }
        var idx = path.indexOf('://');
        if (idx !== -1) {
            return idx + 3; // 3 -> "://".length
        }
        return 0;
    }
    function join() {
        var parts = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parts[_i - 0] = arguments[_i];
        }
        var rootLen = getRootLength(parts[0]), root;
        // simply preserve things like c:/, //localhost/, file:///, http://, etc
        root = parts[0].substr(0, rootLen);
        parts[0] = parts[0].substr(rootLen);
        var allParts = [], endsWithSep = /[\\\/]$/.test(parts[parts.length - 1]);
        for (var i = 0; i < parts.length; i++) {
            allParts.push.apply(allParts, parts[i].split(/\/|\\/));
        }
        for (var i = 0; i < allParts.length; i++) {
            var part = allParts[i];
            if (part === '.' || part.length === 0) {
                allParts.splice(i, 1);
                i -= 1;
            }
            else if (part === '..' && !!allParts[i - 1] && allParts[i - 1] !== '..') {
                allParts.splice(i - 1, 2);
                i -= 2;
            }
        }
        if (endsWithSep) {
            allParts.push('');
        }
        var ret = allParts.join('/');
        if (root) {
            ret = root.replace(/\/|\\/g, '/') + ret;
        }
        return ret;
    }
    exports.join = join;
    function isUNC(path) {
        if (!platform_1.isWindows || !path) {
            return false; // UNC is a windows concept
        }
        path = this.normalize(path, true);
        return path[0] === exports.nativeSep && path[1] === exports.nativeSep;
    }
    exports.isUNC = isUNC;
    function isPosixAbsolute(path) {
        return path && path[0] === '/';
    }
    function makeAbsolute(path, isPathNormalized) {
        return isPosixAbsolute(!isPathNormalized ? normalize(path) : path) ? path : exports.sep + path;
    }
    exports.makeAbsolute = makeAbsolute;
    function isRelative(path) {
        return path && path.length > 1 && path[0] === '.';
    }
    exports.isRelative = isRelative;
    var _slash = '/'.charCodeAt(0);
    function isEqualOrParent(path, candidate) {
        if (path === candidate) {
            return true;
        }
        path = normalize(path);
        candidate = normalize(candidate);
        var candidateLen = candidate.length;
        var lastCandidateChar = candidate.charCodeAt(candidateLen - 1);
        if (lastCandidateChar === _slash) {
            candidate = candidate.substring(0, candidateLen - 1);
            candidateLen -= 1;
        }
        if (path === candidate) {
            return true;
        }
        if (!platform_1.isLinux) {
            // case insensitive
            path = path.toLowerCase();
            candidate = candidate.toLowerCase();
        }
        if (path === candidate) {
            return true;
        }
        if (path.indexOf(candidate) !== 0) {
            return false;
        }
        var char = path.charCodeAt(candidateLen);
        return char === _slash;
    }
    exports.isEqualOrParent = isEqualOrParent;
    // Reference: https://en.wikipedia.org/wiki/Filename
    var INVALID_FILE_CHARS = platform_1.isWindows ? /[\\/:\*\?"<>\|]/g : /[\\/]/g;
    var WINDOWS_FORBIDDEN_NAMES = /^(con|prn|aux|clock\$|nul|lpt[0-9]|com[0-9])$/i;
    function isValidBasename(name) {
        if (!name || name.length === 0 || /^\s+$/.test(name)) {
            return false; // require a name that is not just whitespace
        }
        INVALID_FILE_CHARS.lastIndex = 0; // the holy grail of software development
        if (INVALID_FILE_CHARS.test(name)) {
            return false; // check for certain invalid file characters
        }
        if (platform_1.isWindows && WINDOWS_FORBIDDEN_NAMES.test(name)) {
            return false; // check for certain invalid file names
        }
        if (name === '.' || name === '..') {
            return false; // check for reserved values
        }
        if (platform_1.isWindows && strings_1.endsWith(name, '.')) {
            return false; // Windows: file cannot end with a "."
        }
        if (platform_1.isWindows && name.length !== name.trim().length) {
            return false; // Windows: file cannot end with a whitespace
        }
        return true;
    }
    exports.isValidBasename = isValidBasename;
    exports.isAbsoluteRegex = /^((\/|[a-zA-Z]:\\)[^\(\)<>\\'\"\[\]]+)/;
    /**
     * If you have access to node, it is recommended to use node's path.isAbsolute().
     * This is a simple regex based approach.
     */
    function isAbsolute(path) {
        return exports.isAbsoluteRegex.test(path);
    }
    exports.isAbsolute = isAbsolute;
});

define("vs/base/common/glob", ["require", "exports", 'vs/base/common/strings', 'vs/base/common/paths'], function (require, exports, strings, paths) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var CACHE = Object.create(null);
    var PATH_REGEX = '[/\\\\]'; // any slash or backslash
    var NO_PATH_REGEX = '[^/\\\\]'; // any non-slash and non-backslash
    function starsToRegExp(starCount) {
        switch (starCount) {
            case 0:
                return '';
            case 1:
                return NO_PATH_REGEX + '*?'; // 1 star matches any number of characters except path separator (/ and \) - non greedy (?)
            default:
                // Matches:  (Path Sep    OR     Path Val followed by Path Sep     OR    Path Sep followed by Path Val) 0-many times
                // Group is non capturing because we don't need to capture at all (?:...)
                // Overall we use non-greedy matching because it could be that we match too much
                return '(?:' + PATH_REGEX + '|' + NO_PATH_REGEX + '+' + PATH_REGEX + '|' + PATH_REGEX + NO_PATH_REGEX + '+)*?';
        }
    }
    function splitGlobAware(pattern, splitChar) {
        if (!pattern) {
            return [];
        }
        var segments = [];
        var inBraces = false;
        var inBrackets = false;
        var char;
        var curVal = '';
        for (var i = 0; i < pattern.length; i++) {
            char = pattern[i];
            switch (char) {
                case splitChar:
                    if (!inBraces && !inBrackets) {
                        segments.push(curVal);
                        curVal = '';
                        continue;
                    }
                    break;
                case '{':
                    inBraces = true;
                    break;
                case '}':
                    inBraces = false;
                    break;
                case '[':
                    inBrackets = true;
                    break;
                case ']':
                    inBrackets = false;
                    break;
            }
            curVal += char;
        }
        // Tail
        if (curVal) {
            segments.push(curVal);
        }
        return segments;
    }
    exports.splitGlobAware = splitGlobAware;
    function parseRegExp(pattern) {
        if (!pattern) {
            return '';
        }
        var regEx = '';
        // Split up into segments for each slash found
        var segments = splitGlobAware(pattern, '/');
        // Special case where we only have globstars
        if (segments.every(function (s) { return s === '**'; })) {
            regEx = '.*';
        }
        else {
            var previousSegmentWasGlobStar_1 = false;
            segments.forEach(function (segment, index) {
                // Globstar is special
                if (segment === '**') {
                    // if we have more than one globstar after another, just ignore it
                    if (!previousSegmentWasGlobStar_1) {
                        regEx += starsToRegExp(2);
                        previousSegmentWasGlobStar_1 = true;
                    }
                    return;
                }
                // States
                var inBraces = false;
                var braceVal = '';
                var inBrackets = false;
                var bracketVal = '';
                var char;
                for (var i = 0; i < segment.length; i++) {
                    char = segment[i];
                    // Support brace expansion
                    if (char !== '}' && inBraces) {
                        braceVal += char;
                        continue;
                    }
                    // Support brackets
                    if (char !== ']' && inBrackets) {
                        var res = void 0;
                        switch (char) {
                            case '-':
                                res = char;
                                break;
                            case '^':
                                res = char;
                                break;
                            default:
                                res = strings.escapeRegExpCharacters(char);
                        }
                        bracketVal += res;
                        continue;
                    }
                    switch (char) {
                        case '{':
                            inBraces = true;
                            continue;
                        case '[':
                            inBrackets = true;
                            continue;
                        case '}':
                            var choices = splitGlobAware(braceVal, ',');
                            // Converts {foo,bar} => [foo|bar]
                            var braceRegExp = '(?:' + choices.reduce(function (prevValue, curValue, i, array) {
                                return prevValue + '|' + parseRegExp(curValue);
                            }, parseRegExp(choices[0]) /* parse the first segment as regex and give as initial value */) + ')';
                            regEx += braceRegExp;
                            inBraces = false;
                            braceVal = '';
                            break;
                        case ']':
                            regEx += ('[' + bracketVal + ']');
                            inBrackets = false;
                            bracketVal = '';
                            break;
                        case '?':
                            regEx += NO_PATH_REGEX; // 1 ? matches any single character except path separator (/ and \)
                            continue;
                        case '*':
                            regEx += starsToRegExp(1);
                            continue;
                        default:
                            regEx += strings.escapeRegExpCharacters(char);
                    }
                }
                // Tail: Add the slash we had split on if there is more to come and the next one is not a globstar
                if (index < segments.length - 1 && segments[index + 1] !== '**') {
                    regEx += PATH_REGEX;
                }
                // reset state
                previousSegmentWasGlobStar_1 = false;
            });
        }
        return regEx;
    }
    function globToRegExp(pattern) {
        if (!pattern) {
            return null;
        }
        // Whitespace trimming
        pattern = pattern.trim();
        // Check cache
        if (CACHE[pattern]) {
            var cached = CACHE[pattern];
            cached.lastIndex = 0; // reset RegExp to its initial state to reuse it!
            return cached;
        }
        var regEx = parseRegExp(pattern);
        // Wrap it
        regEx = '^' + regEx + '$';
        // Convert to regexp and be ready for errors
        var result = toRegExp(regEx);
        // Make sure to cache
        CACHE[pattern] = result;
        return result;
    }
    function toRegExp(regEx) {
        try {
            return new RegExp(regEx);
        }
        catch (error) {
            return /.^/; // create a regex that matches nothing if we cannot parse the pattern
        }
    }
    function match(arg1, path, siblings) {
        if (!arg1 || !path) {
            return false;
        }
        // Glob with String
        if (typeof arg1 === 'string') {
            var regExp = globToRegExp(arg1);
            return regExp && regExp.test(path);
        }
        // Glob with Expression
        return matchExpression(arg1, path, siblings);
    }
    exports.match = match;
    function matchExpression(expression, path, siblings) {
        var patterns = Object.getOwnPropertyNames(expression);
        var basename;
        var _loop_1 = function(i) {
            var pattern = patterns[i];
            var value = expression[pattern];
            if (value === false) {
                return "continue"; // pattern is disabled
            }
            // Pattern matches path
            if (match(pattern, path)) {
                // Expression Pattern is <boolean>
                if (typeof value === 'boolean') {
                    return { value: pattern };
                }
                // Expression Pattern is <SiblingClause>
                if (value && typeof value.when === 'string') {
                    if (!siblings || !siblings.length) {
                        return "continue"; // pattern is malformed or we don't have siblings
                    }
                    if (!basename) {
                        basename = strings.rtrim(paths.basename(path), paths.extname(path));
                    }
                    var clause = value;
                    var clausePattern_1 = clause.when.replace('$(basename)', basename);
                    if (siblings.some(function (sibling) { return sibling === clausePattern_1; })) {
                        return { value: pattern };
                    }
                    else {
                        return "continue"; // pattern does not match in the end because the when clause is not satisfied
                    }
                }
                // Expression is Anything
                return { value: pattern };
            }
        };
        for (var i = 0; i < patterns.length; i++) {
            var state_1 = _loop_1(i);
            if (typeof state_1 === "object") return state_1.value;
            if (state_1 === "continue") continue;
        }
        return null;
    }
});

define("vs/base/common/types", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * @returns whether the provided parameter is a JavaScript Array or not.
     */
    function isArray(array) {
        if (Array.isArray) {
            return Array.isArray(array);
        }
        if (array && typeof (array.length) === 'number' && array.constructor === Array) {
            return true;
        }
        return false;
    }
    exports.isArray = isArray;
    /**
     * @returns whether the provided parameter is a JavaScript String or not.
     */
    function isString(str) {
        if (typeof (str) === 'string' || str instanceof String) {
            return true;
        }
        return false;
    }
    exports.isString = isString;
    /**
     * @returns whether the provided parameter is a JavaScript Array and each element in the array is a string.
     */
    function isStringArray(value) {
        return isArray(value) && value.every(function (elem) { return isString(elem); });
    }
    exports.isStringArray = isStringArray;
    /**
     * @returns whether the provided parameter is a JavaScript Object or not.
     */
    function isObject(obj) {
        // Needed for IE8
        if (typeof obj === 'undefined' || obj === null) {
            return false;
        }
        return Object.prototype.toString.call(obj) === '[object Object]';
    }
    exports.isObject = isObject;
    /**
     * @returns whether the provided parameter is a JavaScript Number or not.
     */
    function isNumber(obj) {
        if ((typeof (obj) === 'number' || obj instanceof Number) && !isNaN(obj)) {
            return true;
        }
        return false;
    }
    exports.isNumber = isNumber;
    /**
     * @returns whether the provided parameter is a JavaScript Boolean or not.
     */
    function isBoolean(obj) {
        return obj === true || obj === false;
    }
    exports.isBoolean = isBoolean;
    /**
     * @returns whether the provided parameter is undefined.
     */
    function isUndefined(obj) {
        return typeof (obj) === 'undefined';
    }
    exports.isUndefined = isUndefined;
    /**
     * @returns whether the provided parameter is undefined or null.
     */
    function isUndefinedOrNull(obj) {
        return isUndefined(obj) || obj === null;
    }
    exports.isUndefinedOrNull = isUndefinedOrNull;
    var hasOwnProperty = Object.prototype.hasOwnProperty;
    /**
     * @returns whether the provided parameter is an empty JavaScript Object or not.
     */
    function isEmptyObject(obj) {
        if (!isObject(obj)) {
            return false;
        }
        for (var key in obj) {
            if (hasOwnProperty.call(obj, key)) {
                return false;
            }
        }
        return true;
    }
    exports.isEmptyObject = isEmptyObject;
    /**
     * @returns whether the provided parameter is a JavaScript Function or not.
     */
    function isFunction(obj) {
        return Object.prototype.toString.call(obj) === '[object Function]';
    }
    exports.isFunction = isFunction;
    /**
     * @returns whether the provided parameters is are JavaScript Function or not.
     */
    function areFunctions() {
        var objects = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            objects[_i - 0] = arguments[_i];
        }
        return objects && objects.length > 0 && objects.every(function (object) { return isFunction(object); });
    }
    exports.areFunctions = areFunctions;
    function validateConstraints(args, constraints) {
        var len = Math.min(args.length, constraints.length);
        for (var i = 0; i < len; i++) {
            validateConstraint(args[i], constraints[i]);
        }
    }
    exports.validateConstraints = validateConstraints;
    function validateConstraint(arg, constraint) {
        if (typeof constraint === 'string') {
            if (typeof arg !== constraint) {
                throw new Error("argument does not match constraint: typeof " + constraint);
            }
        }
        else if (typeof constraint === 'function') {
            if (arg instanceof constraint) {
                return;
            }
            if (arg && arg.constructor === constraint) {
                return;
            }
            if (constraint.length === 1 && constraint.call(undefined, arg) === true) {
                return;
            }
            throw new Error("argument does not match one of these constraints: arg instanceof constraint, arg.constructor === constraint, nor constraint(arg) === true");
        }
    }
    exports.validateConstraint = validateConstraint;
    /**
     * Creates a new object of the provided class and will call the constructor with
     * any additional argument supplied.
     */
    function create(ctor) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var obj = Object.create(ctor.prototype);
        ctor.apply(obj, args);
        return obj;
    }
    exports.create = create;
});

define("vs/base/browser/browser", ["require", "exports", 'vs/base/common/types', 'vs/base/common/platform'], function (require, exports, types, Platform) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var globals = (typeof self === 'object' ? self : global);
    // MAC:
    // chrome: "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.100 Safari/535.2"
    // safari: "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22"
    //
    // WINDOWS:
    // chrome: "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.102 Safari/535.2"
    // IE: "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; MS-RTC LM 8; InfoPath.3; Zune 4.7)"
    // Opera:	"Opera/9.80 (Windows NT 6.1; U; en) Presto/2.9.168 Version/11.52"
    // FF: "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:8.0) Gecko/20100101 Firefox/8.0"
    // LINUX:
    // chrome: "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.71 Safari/537.36"
    // firefox: "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:34.0) Gecko/20100101 Firefox/34.0"
    var userAgent = globals.navigator ? globals.navigator.userAgent : '';
    // DOCUMENTED FOR FUTURE REFERENCE:
    // When running IE11 in IE10 document mode, the code below will identify the browser as being IE10,
    // which is correct because IE11 in IE10 document mode will reimplement all the bugs of IE10
    exports.isIE11 = (userAgent.indexOf('Trident') >= 0 && userAgent.indexOf('MSIE') < 0);
    exports.isIE10 = (userAgent.indexOf('MSIE 10') >= 0);
    exports.isIE9 = (userAgent.indexOf('MSIE 9') >= 0);
    exports.isIE11orEarlier = exports.isIE11 || exports.isIE10 || exports.isIE9;
    exports.isIE10orEarlier = exports.isIE10 || exports.isIE9;
    exports.isIE10orLater = exports.isIE11 || exports.isIE10;
    exports.isOpera = (userAgent.indexOf('Opera') >= 0);
    exports.isFirefox = (userAgent.indexOf('Firefox') >= 0);
    exports.isWebKit = (userAgent.indexOf('AppleWebKit') >= 0);
    exports.isChrome = (userAgent.indexOf('Chrome') >= 0);
    exports.isSafari = (userAgent.indexOf('Chrome') === -1) && (userAgent.indexOf('Safari') >= 0);
    exports.isIPad = (userAgent.indexOf('iPad') >= 0);
    exports.canUseTranslate3d = !exports.isIE9 && !exports.isFirefox;
    exports.enableEmptySelectionClipboard = exports.isWebKit;
    var _disablePushState = false;
    /**
     * Returns if the browser supports the history.pushState function or not.
     */
    function canPushState() {
        return (!_disablePushState && globals.history && globals.history.pushState);
    }
    exports.canPushState = canPushState;
    /**
     * Helpful when we detect that pushing state does not work for some reason (e.g. FF prevents pushState for security reasons in some cases)
     */
    function disablePushState() {
        _disablePushState = true;
    }
    exports.disablePushState = disablePushState;
    /**
     * Returns if the browser supports CSS 3 animations.
     */
    function hasCSSAnimationSupport() {
        if (this._hasCSSAnimationSupport === true || this._hasCSSAnimationSupport === false) {
            return this._hasCSSAnimationSupport;
        }
        if (!globals.document) {
            return false;
        }
        var supported = false;
        var element = globals.document.createElement('div');
        var properties = ['animationName', 'webkitAnimationName', 'msAnimationName', 'MozAnimationName', 'OAnimationName'];
        for (var i = 0; i < properties.length; i++) {
            var property = properties[i];
            if (!types.isUndefinedOrNull(element.style[property]) || element.style.hasOwnProperty(property)) {
                supported = true;
                break;
            }
        }
        if (supported) {
            this._hasCSSAnimationSupport = true;
        }
        else {
            this._hasCSSAnimationSupport = false;
        }
        return this._hasCSSAnimationSupport;
    }
    exports.hasCSSAnimationSupport = hasCSSAnimationSupport;
    /**
     * Returns if the browser supports the provided video mime type or not.
     */
    function canPlayVideo(type) {
        if (!globals.document) {
            return false;
        }
        var video = globals.document.createElement('video');
        if (video.canPlayType) {
            var canPlay = video.canPlayType(type);
            return canPlay === 'maybe' || canPlay === 'probably';
        }
        return false;
    }
    exports.canPlayVideo = canPlayVideo;
    /**
     * Returns if the browser supports the provided audio mime type or not.
     */
    function canPlayAudio(type) {
        if (!globals.document) {
            return false;
        }
        var audio = globals.document.createElement('audio');
        if (audio.canPlayType) {
            var canPlay = audio.canPlayType(type);
            return canPlay === 'maybe' || canPlay === 'probably';
        }
        return false;
    }
    exports.canPlayAudio = canPlayAudio;
    function isInWebWorker() {
        return !globals.document && typeof (globals.importScripts) !== 'undefined';
    }
    exports.isInWebWorker = isInWebWorker;
    function supportsExecCommand(command) {
        return ((exports.isIE11orEarlier || Platform.isNative)
            && document.queryCommandSupported(command));
    }
    exports.supportsExecCommand = supportsExecCommand;
});

define("vs/base/browser/browserService", ["require", "exports", 'vs/base/browser/browser'], function (require, exports, browser_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function regularIsHTMLElement(o) {
        if (typeof HTMLElement === 'object') {
            return o instanceof HTMLElement;
        }
        return o && typeof o === 'object' && o.nodeType === 1 && typeof o.nodeName === 'string';
    }
    exports.regularIsHTMLElement = regularIsHTMLElement;
    var BrowserService = (function () {
        function BrowserService() {
            this.restore();
        }
        BrowserService.prototype.mock = function (source) {
            this.document = source.document;
            this.window = source.window;
            this.isHTMLElement = source.isHTMLElement;
        };
        BrowserService.prototype.restore = function () {
            this.isHTMLElement = regularIsHTMLElement;
            if (browser_1.isInWebWorker()) {
                this.document = null;
                this.window = null;
            }
            else {
                this.document = window.document;
                this.window = window;
            }
        };
        return BrowserService;
    }());
    var browserService = new BrowserService();
    function getService() {
        return browserService;
    }
    exports.getService = getService;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/mouseEvent", ["require", "exports", 'vs/base/common/platform', 'vs/base/browser/browser', 'vs/base/browser/iframe'], function (require, exports, platform, browser, iframe_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var StandardMouseEvent = (function () {
        function StandardMouseEvent(e) {
            var _this = this;
            this.timestamp = Date.now();
            this.browserEvent = e;
            this.leftButton = e.button === 0;
            this.middleButton = e.button === 1;
            this.rightButton = e.button === 2;
            this.target = e.target || e.targetNode || e.srcElement;
            this.detail = e.detail || 1;
            if (e.type === 'dblclick') {
                this.detail = 2;
            }
            this.posx = 0;
            this.posy = 0;
            this.ctrlKey = e.ctrlKey;
            this.shiftKey = e.shiftKey;
            this.altKey = e.altKey;
            this.metaKey = e.metaKey;
            var readClientCoords = function () {
                if (e.clientX || e.clientY) {
                    _this.posx = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
                    _this.posy = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
                    return true;
                }
                return false;
            };
            var readPageCoords = function () {
                if (e.pageX || e.pageY) {
                    _this.posx = e.pageX;
                    _this.posy = e.pageY;
                    return true;
                }
                return false;
            };
            var test1 = readPageCoords, test2 = readClientCoords;
            if (browser.isIE10) {
                // The if A elseif B logic here is inversed in IE10 due to an IE10 issue
                test1 = readClientCoords;
                test2 = readPageCoords;
            }
            if (!test1()) {
                test2();
            }
            // Find the position of the iframe this code is executing in relative to the iframe where the event was captured.
            var iframeOffsets = iframe_1.IframeUtils.getPositionOfChildWindowRelativeToAncestorWindow(self, e.view);
            this.posx -= iframeOffsets.left;
            this.posy -= iframeOffsets.top;
        }
        StandardMouseEvent.prototype.preventDefault = function () {
            if (this.browserEvent.preventDefault) {
                this.browserEvent.preventDefault();
            }
        };
        StandardMouseEvent.prototype.stopPropagation = function () {
            if (this.browserEvent.stopPropagation) {
                this.browserEvent.stopPropagation();
            }
        };
        return StandardMouseEvent;
    }());
    exports.StandardMouseEvent = StandardMouseEvent;
    var DragMouseEvent = (function (_super) {
        __extends(DragMouseEvent, _super);
        function DragMouseEvent(e) {
            _super.call(this, e);
            this.dataTransfer = e.dataTransfer;
        }
        return DragMouseEvent;
    }(StandardMouseEvent));
    exports.DragMouseEvent = DragMouseEvent;
    var DropMouseEvent = (function (_super) {
        __extends(DropMouseEvent, _super);
        function DropMouseEvent(e) {
            _super.call(this, e);
        }
        return DropMouseEvent;
    }(DragMouseEvent));
    exports.DropMouseEvent = DropMouseEvent;
    var StandardMouseWheelEvent = (function () {
        function StandardMouseWheelEvent(e, deltaX, deltaY) {
            if (deltaX === void 0) { deltaX = 0; }
            if (deltaY === void 0) { deltaY = 0; }
            this.browserEvent = e || null;
            this.target = e ? (e.target || e.targetNode || e.srcElement) : null;
            this.deltaY = deltaY;
            this.deltaX = deltaX;
            if (e) {
                var e1 = e;
                var e2 = e;
                // vertical delta scroll
                if (typeof e1.wheelDeltaY !== 'undefined') {
                    this.deltaY = e1.wheelDeltaY / 120;
                }
                else if (typeof e2.VERTICAL_AXIS !== 'undefined' && e2.axis === e2.VERTICAL_AXIS) {
                    this.deltaY = -e2.detail / 3;
                }
                // horizontal delta scroll
                if (typeof e1.wheelDeltaX !== 'undefined') {
                    if (browser.isSafari && platform.isWindows) {
                        this.deltaX = -(e1.wheelDeltaX / 120);
                    }
                    else {
                        this.deltaX = e1.wheelDeltaX / 120;
                    }
                }
                else if (typeof e2.HORIZONTAL_AXIS !== 'undefined' && e2.axis === e2.HORIZONTAL_AXIS) {
                    this.deltaX = -e.detail / 3;
                }
                // Assume a vertical scroll if nothing else worked
                if (this.deltaY === 0 && this.deltaX === 0 && e.wheelDelta) {
                    this.deltaY = e.wheelDelta / 120;
                }
            }
        }
        StandardMouseWheelEvent.prototype.preventDefault = function () {
            if (this.browserEvent) {
                if (this.browserEvent.preventDefault) {
                    this.browserEvent.preventDefault();
                }
            }
        };
        StandardMouseWheelEvent.prototype.stopPropagation = function () {
            if (this.browserEvent) {
                if (this.browserEvent.stopPropagation) {
                    this.browserEvent.stopPropagation();
                }
            }
        };
        return StandardMouseWheelEvent;
    }());
    exports.StandardMouseWheelEvent = StandardMouseWheelEvent;
});

define("vs/base/common/graph", ["require", "exports", 'vs/base/common/types', 'vs/base/common/collections'], function (require, exports, objects, collections) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function newNode(data) {
        return {
            data: data,
            incoming: {},
            outgoing: {}
        };
    }
    exports.newNode = newNode;
    var Graph = (function () {
        function Graph(_hashFn) {
            this._hashFn = _hashFn;
            this._nodes = Object.create(null);
            // empty
        }
        Graph.prototype.roots = function () {
            var ret = [];
            collections.forEach(this._nodes, function (entry) {
                if (objects.isEmptyObject(entry.value.outgoing)) {
                    ret.push(entry.value);
                }
            });
            return ret;
        };
        Graph.prototype.traverse = function (start, inwards, callback) {
            var startNode = this.lookup(start);
            if (!startNode) {
                return;
            }
            this._traverse(startNode, inwards, {}, callback);
        };
        Graph.prototype._traverse = function (node, inwards, seen, callback) {
            var _this = this;
            var key = this._hashFn(node.data);
            if (collections.contains(seen, key)) {
                return;
            }
            seen[key] = true;
            callback(node.data);
            var nodes = inwards ? node.outgoing : node.incoming;
            collections.forEach(nodes, function (entry) { return _this._traverse(entry.value, inwards, seen, callback); });
        };
        Graph.prototype.insertEdge = function (from, to) {
            var fromNode = this.lookupOrInsertNode(from), toNode = this.lookupOrInsertNode(to);
            fromNode.outgoing[this._hashFn(to)] = toNode;
            toNode.incoming[this._hashFn(from)] = fromNode;
        };
        Graph.prototype.removeNode = function (data) {
            var key = this._hashFn(data);
            delete this._nodes[key];
            collections.forEach(this._nodes, function (entry) {
                delete entry.value.outgoing[key];
                delete entry.value.incoming[key];
            });
        };
        Graph.prototype.lookupOrInsertNode = function (data) {
            var key = this._hashFn(data), node = collections.lookup(this._nodes, key);
            if (!node) {
                node = newNode(data);
                this._nodes[key] = node;
            }
            return node;
        };
        Graph.prototype.lookup = function (data) {
            return collections.lookup(this._nodes, this._hashFn(data));
        };
        Object.defineProperty(Graph.prototype, "length", {
            get: function () {
                return Object.keys(this._nodes).length;
            },
            enumerable: true,
            configurable: true
        });
        return Graph;
    }());
    exports.Graph = Graph;
});

define("vs/base/common/labels", ["require", "exports", 'vs/base/common/platform', 'vs/base/common/types', 'vs/base/common/strings', 'vs/base/common/paths'], function (require, exports, platform, types, strings, paths) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var PathLabelProvider = (function () {
        function PathLabelProvider(arg1) {
            this.root = arg1 && getPath(arg1);
        }
        PathLabelProvider.prototype.getLabel = function (arg1) {
            return getPathLabel(getPath(arg1), this.root);
        };
        return PathLabelProvider;
    }());
    exports.PathLabelProvider = PathLabelProvider;
    function getPathLabel(arg1, arg2) {
        var basepath = arg2 && getPath(arg2);
        var absolutePath = getPath(arg1);
        if (basepath && paths.isEqualOrParent(absolutePath, basepath)) {
            return paths.normalize(strings.ltrim(absolutePath.substr(basepath.length), paths.nativeSep), true);
        }
        if (platform.isWindows && absolutePath[1] === ':') {
            return paths.normalize(absolutePath.charAt(0).toUpperCase() + absolutePath.slice(1), true);
        }
        return paths.normalize(absolutePath, true);
    }
    exports.getPathLabel = getPathLabel;
    function getPath(arg1) {
        if (!arg1) {
            return null;
        }
        if (typeof arg1 === 'string') {
            return arg1;
        }
        if (types.isFunction(arg1.getWorkspace)) {
            var ws = arg1.getWorkspace();
            return ws ? ws.resource.fsPath : void 0;
        }
        return arg1.fsPath;
    }
});

define("vs/base/common/mime", ["require", "exports", 'vs/base/common/paths', 'vs/base/common/types', 'vs/base/common/strings', 'vs/base/common/glob'], function (require, exports, paths, types, strings, glob_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.MIME_TEXT = 'text/plain';
    exports.MIME_BINARY = 'application/octet-stream';
    exports.MIME_UNKNOWN = 'application/unknown';
    var registeredAssociations = [];
    /**
     * Associate a text mime to the registry.
     */
    function registerTextMime(association) {
        // Register
        registeredAssociations.push(association);
        // Check for conflicts unless this is a user configured association
        if (!association.userConfigured) {
            registeredAssociations.forEach(function (a) {
                if (a.mime === association.mime || a.userConfigured) {
                    return; // same mime or userConfigured is ok
                }
                if (association.extension && a.extension === association.extension) {
                    console.warn("Overwriting extension <<" + association.extension + ">> to now point to mime <<" + association.mime + ">>");
                }
                if (association.filename && a.filename === association.filename) {
                    console.warn("Overwriting filename <<" + association.filename + ">> to now point to mime <<" + association.mime + ">>");
                }
                if (association.filepattern && a.filepattern === association.filepattern) {
                    console.warn("Overwriting filepattern <<" + association.filepattern + ">> to now point to mime <<" + association.mime + ">>");
                }
                if (association.firstline && a.firstline === association.firstline) {
                    console.warn("Overwriting firstline <<" + association.firstline + ">> to now point to mime <<" + association.mime + ">>");
                }
            });
        }
    }
    exports.registerTextMime = registerTextMime;
    /**
     * Clear text mimes from the registry.
     */
    function clearTextMimes(onlyUserConfigured) {
        if (!onlyUserConfigured) {
            registeredAssociations = [];
        }
        else {
            registeredAssociations = registeredAssociations.filter(function (a) { return !a.userConfigured; });
        }
    }
    exports.clearTextMimes = clearTextMimes;
    /**
     * Given a file, return the best matching mime type for it
     */
    function guessMimeTypes(path, firstLine) {
        if (!path) {
            return [exports.MIME_UNKNOWN];
        }
        path = path.toLowerCase();
        // 1.) User configured mappings have highest priority
        var configuredMime = guessMimeTypeByPath(path, registeredAssociations.filter(function (a) { return a.userConfigured; }));
        if (configuredMime) {
            return [configuredMime, exports.MIME_TEXT];
        }
        // 2.) Registered mappings have middle priority
        var registeredMime = guessMimeTypeByPath(path, registeredAssociations.filter(function (a) { return !a.userConfigured; }));
        if (registeredMime) {
            return [registeredMime, exports.MIME_TEXT];
        }
        // 3.) Firstline has lowest priority
        if (firstLine) {
            var firstlineMime = guessMimeTypeByFirstline(firstLine);
            if (firstlineMime) {
                return [firstlineMime, exports.MIME_TEXT];
            }
        }
        return [exports.MIME_UNKNOWN];
    }
    exports.guessMimeTypes = guessMimeTypes;
    function guessMimeTypeByPath(path, associations) {
        var filename = paths.basename(path);
        var filenameMatch;
        var patternMatch;
        var extensionMatch;
        for (var i = 0; i < associations.length; i++) {
            var association = associations[i];
            // First exact name match
            if (association.filename && filename === association.filename.toLowerCase()) {
                filenameMatch = association;
                break; // take it!
            }
            // Longest pattern match
            if (association.filepattern) {
                var target = association.filepattern.indexOf(paths.sep) >= 0 ? path : filename; // match on full path if pattern contains path separator
                if (glob_1.match(association.filepattern.toLowerCase(), target)) {
                    if (!patternMatch || association.filepattern.length > patternMatch.filepattern.length) {
                        patternMatch = association;
                    }
                }
            }
            // Longest extension match
            if (association.extension) {
                if (strings.endsWith(filename, association.extension.toLowerCase())) {
                    if (!extensionMatch || association.extension.length > extensionMatch.extension.length) {
                        extensionMatch = association;
                    }
                }
            }
        }
        // 1.) Exact name match has second highest prio
        if (filenameMatch) {
            return filenameMatch.mime;
        }
        // 2.) Match on pattern
        if (patternMatch) {
            return patternMatch.mime;
        }
        // 3.) Match on extension comes next
        if (extensionMatch) {
            return extensionMatch.mime;
        }
        return null;
    }
    function guessMimeTypeByFirstline(firstLine) {
        if (strings.startsWithUTF8BOM(firstLine)) {
            firstLine = firstLine.substr(1);
        }
        if (firstLine.length > 0) {
            for (var i = 0; i < registeredAssociations.length; ++i) {
                var association = registeredAssociations[i];
                if (!association.firstline) {
                    continue;
                }
                // Make sure the entire line matches, not just a subpart.
                var matches = firstLine.match(association.firstline);
                if (matches && matches.length > 0 && matches[0].length === firstLine.length) {
                    return association.mime;
                }
            }
        }
        return null;
    }
    function isBinaryMime(mimes) {
        if (!mimes) {
            return false;
        }
        var mimeVals;
        if (types.isArray(mimes)) {
            mimeVals = mimes;
        }
        else {
            mimeVals = mimes.split(',').map(function (mime) { return mime.trim(); });
        }
        return mimeVals.indexOf(exports.MIME_BINARY) >= 0;
    }
    exports.isBinaryMime = isBinaryMime;
    function isUnspecific(mime) {
        if (!mime) {
            return true;
        }
        if (typeof mime === 'string') {
            return mime === exports.MIME_BINARY || mime === exports.MIME_TEXT || mime === exports.MIME_UNKNOWN;
        }
        return mime.length === 1 && isUnspecific(mime[0]);
    }
    exports.isUnspecific = isUnspecific;
    function suggestFilename(theMime, prefix) {
        for (var i = 0; i < registeredAssociations.length; i++) {
            var association = registeredAssociations[i];
            if (association.userConfigured) {
                continue; // only support registered ones
            }
            if (association.mime === theMime && association.extension) {
                return prefix + association.extension;
            }
        }
        return null;
    }
    exports.suggestFilename = suggestFilename;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/common/numbers", ["require", "exports", 'vs/base/common/types'], function (require, exports, types) {
    'use strict';
    function count(fromOrTo, toOrCallback, callback) {
        var from, to;
        if (types.isNumber(toOrCallback)) {
            from = fromOrTo;
            to = toOrCallback;
        }
        else {
            from = 0;
            to = fromOrTo;
            callback = toOrCallback;
        }
        var op = from <= to ? function (i) { return i + 1; } : function (i) { return i - 1; };
        var cmp = from <= to ? function (a, b) { return a < b; } : function (a, b) { return a > b; };
        for (var i = from; cmp(i, to); i = op(i)) {
            callback(i);
        }
    }
    exports.count = count;
    function countToArray(fromOrTo, to) {
        var result = [];
        var fn = function (i) { return result.push(i); };
        if (types.isUndefined(to)) {
            count(fromOrTo, fn);
        }
        else {
            count(fromOrTo, to, fn);
        }
        return result;
    }
    exports.countToArray = countToArray;
});

define("vs/base/common/objects", ["require", "exports", 'vs/base/common/types'], function (require, exports, Types) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function clone(obj) {
        if (!obj || typeof obj !== 'object') {
            return obj;
        }
        if (obj instanceof RegExp) {
            return obj;
        }
        var result = (Array.isArray(obj)) ? [] : {};
        Object.keys(obj).forEach(function (key) {
            if (obj[key] && typeof obj[key] === 'object') {
                result[key] = clone(obj[key]);
            }
            else {
                result[key] = obj[key];
            }
        });
        return result;
    }
    exports.clone = clone;
    function deepClone(obj) {
        if (!obj || typeof obj !== 'object') {
            return obj;
        }
        var result = (Array.isArray(obj)) ? [] : {};
        Object.getOwnPropertyNames(obj).forEach(function (key) {
            if (obj[key] && typeof obj[key] === 'object') {
                result[key] = deepClone(obj[key]);
            }
            else {
                result[key] = obj[key];
            }
        });
        return result;
    }
    exports.deepClone = deepClone;
    var hasOwnProperty = Object.prototype.hasOwnProperty;
    function cloneAndChange(obj, changer) {
        return _cloneAndChange(obj, changer, []);
    }
    exports.cloneAndChange = cloneAndChange;
    function _cloneAndChange(obj, changer, encounteredObjects) {
        if (Types.isUndefinedOrNull(obj)) {
            return obj;
        }
        var changed = changer(obj);
        if (typeof changed !== 'undefined') {
            return changed;
        }
        if (Types.isArray(obj)) {
            var r1 = [];
            for (var i1 = 0; i1 < obj.length; i1++) {
                r1.push(_cloneAndChange(obj[i1], changer, encounteredObjects));
            }
            return r1;
        }
        if (Types.isObject(obj)) {
            if (encounteredObjects.indexOf(obj) >= 0) {
                throw new Error('Cannot clone recursive data-structure');
            }
            encounteredObjects.push(obj);
            var r2 = {};
            for (var i2 in obj) {
                if (hasOwnProperty.call(obj, i2)) {
                    r2[i2] = _cloneAndChange(obj[i2], changer, encounteredObjects);
                }
            }
            encounteredObjects.pop();
            return r2;
        }
        return obj;
    }
    // DON'T USE THESE FUNCTION UNLESS YOU KNOW HOW CHROME
    // WORKS... WE HAVE SEEN VERY WEIRD BEHAVIOUR WITH CHROME >= 37
    ///**
    // * Recursively call Object.freeze on object and any properties that are objects.
    // */
    //export function deepFreeze(obj:any):void {
    //	Object.freeze(obj);
    //	Object.keys(obj).forEach((key) => {
    //		if(!(typeof obj[key] === 'object') || Object.isFrozen(obj[key])) {
    //			return;
    //		}
    //
    //		deepFreeze(obj[key]);
    //	});
    //	if(!Object.isFrozen(obj)) {
    //		console.log('too warm');
    //	}
    //}
    //
    //export function deepSeal(obj:any):void {
    //	Object.seal(obj);
    //	Object.keys(obj).forEach((key) => {
    //		if(!(typeof obj[key] === 'object') || Object.isSealed(obj[key])) {
    //			return;
    //		}
    //
    //		deepSeal(obj[key]);
    //	});
    //	if(!Object.isSealed(obj)) {
    //		console.log('NOT sealed');
    //	}
    //}
    /**
     * Copies all properties of source into destination. The optional parameter "overwrite" allows to control
     * if existing properties on the destination should be overwritten or not. Defaults to true (overwrite).
     */
    function mixin(destination, source, overwrite) {
        if (overwrite === void 0) { overwrite = true; }
        if (!Types.isObject(destination)) {
            return source;
        }
        if (Types.isObject(source)) {
            Object.keys(source).forEach(function (key) {
                if (key in destination) {
                    if (overwrite) {
                        if (Types.isObject(destination[key]) && Types.isObject(source[key])) {
                            mixin(destination[key], source[key], overwrite);
                        }
                        else {
                            destination[key] = source[key];
                        }
                    }
                }
                else {
                    destination[key] = source[key];
                }
            });
        }
        return destination;
    }
    exports.mixin = mixin;
    function assign(destination) {
        var sources = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            sources[_i - 1] = arguments[_i];
        }
        sources.forEach(function (source) { return Object.keys(source).forEach(function (key) { return destination[key] = source[key]; }); });
        return destination;
    }
    exports.assign = assign;
    function toObject(arr, keyMap, valueMap) {
        if (valueMap === void 0) { valueMap = function (x) { return x; }; }
        return arr.reduce(function (o, d) { return assign(o, (_a = {}, _a[keyMap(d)] = valueMap(d), _a)); var _a; }, Object.create(null));
    }
    exports.toObject = toObject;
    /**
     * Returns a new object that has all values of {{obj}}
     * plus those from {{defaults}}.
     */
    function withDefaults(obj, defaults) {
        return mixin(clone(defaults), obj || {});
    }
    exports.withDefaults = withDefaults;
    function equals(one, other) {
        if (one === other) {
            return true;
        }
        if (one === null || one === undefined || other === null || other === undefined) {
            return false;
        }
        if (typeof one !== typeof other) {
            return false;
        }
        if (typeof one !== 'object') {
            return false;
        }
        if ((Array.isArray(one)) !== (Array.isArray(other))) {
            return false;
        }
        var i, key;
        if (Array.isArray(one)) {
            if (one.length !== other.length) {
                return false;
            }
            for (i = 0; i < one.length; i++) {
                if (!equals(one[i], other[i])) {
                    return false;
                }
            }
        }
        else {
            var oneKeys = [];
            for (key in one) {
                oneKeys.push(key);
            }
            oneKeys.sort();
            var otherKeys = [];
            for (key in other) {
                otherKeys.push(key);
            }
            otherKeys.sort();
            if (!equals(oneKeys, otherKeys)) {
                return false;
            }
            for (i = 0; i < oneKeys.length; i++) {
                if (!equals(one[oneKeys[i]], other[oneKeys[i]])) {
                    return false;
                }
            }
        }
        return true;
    }
    exports.equals = equals;
    function ensureProperty(obj, property, defaultValue) {
        if (typeof obj[property] === 'undefined') {
            obj[property] = defaultValue;
        }
    }
    exports.ensureProperty = ensureProperty;
    function arrayToHash(array) {
        var result = {};
        for (var i = 0; i < array.length; ++i) {
            result[array[i]] = true;
        }
        return result;
    }
    exports.arrayToHash = arrayToHash;
    /**
     * Given an array of strings, returns a function which, given a string
     * returns true or false whether the string is in that array.
     */
    function createKeywordMatcher(arr, caseInsensitive) {
        if (caseInsensitive === void 0) { caseInsensitive = false; }
        if (caseInsensitive) {
            arr = arr.map(function (x) { return x.toLowerCase(); });
        }
        var hash = arrayToHash(arr);
        if (caseInsensitive) {
            return function (word) {
                return hash[word.toLowerCase()] !== undefined && hash.hasOwnProperty(word.toLowerCase());
            };
        }
        else {
            return function (word) {
                return hash[word] !== undefined && hash.hasOwnProperty(word);
            };
        }
    }
    exports.createKeywordMatcher = createKeywordMatcher;
    /**
     * Started from TypeScript's __extends function to make a type a subclass of a specific class.
     * Modified to work with properties already defined on the derivedClass, since we can't get TS
     * to call this method before the constructor definition.
     */
    function derive(baseClass, derivedClass) {
        for (var prop in baseClass) {
            if (baseClass.hasOwnProperty(prop)) {
                derivedClass[prop] = baseClass[prop];
            }
        }
        derivedClass = derivedClass || function () { };
        var basePrototype = baseClass.prototype;
        var derivedPrototype = derivedClass.prototype;
        derivedClass.prototype = Object.create(basePrototype);
        for (var prop in derivedPrototype) {
            if (derivedPrototype.hasOwnProperty(prop)) {
                // handle getters and setters properly
                Object.defineProperty(derivedClass.prototype, prop, Object.getOwnPropertyDescriptor(derivedPrototype, prop));
            }
        }
        // Cast to any due to Bug 16188:PropertyDescriptor set and get function should be optional.
        Object.defineProperty(derivedClass.prototype, 'constructor', { value: derivedClass, writable: true, configurable: true, enumerable: true });
    }
    exports.derive = derive;
    /**
     * Calls JSON.Stringify with a replacer to break apart any circular references.
     * This prevents JSON.stringify from throwing the exception
     *  "Uncaught TypeError: Converting circular structure to JSON"
     */
    function safeStringify(obj) {
        var seen = [];
        return JSON.stringify(obj, function (key, value) {
            if (Types.isObject(value) || Array.isArray(value)) {
                if (seen.indexOf(value) !== -1) {
                    return '[Circular]';
                }
                else {
                    seen.push(value);
                }
            }
            return value;
        });
    }
    exports.safeStringify = safeStringify;
});

define("vs/base/common/parsers", ["require", "exports", 'vs/base/common/types'], function (require, exports, Types) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (ValidationState) {
        ValidationState[ValidationState["OK"] = 0] = "OK";
        ValidationState[ValidationState["Info"] = 1] = "Info";
        ValidationState[ValidationState["Warning"] = 2] = "Warning";
        ValidationState[ValidationState["Error"] = 3] = "Error";
        ValidationState[ValidationState["Fatal"] = 4] = "Fatal";
    })(exports.ValidationState || (exports.ValidationState = {}));
    var ValidationState = exports.ValidationState;
    var ValidationStatus = (function () {
        function ValidationStatus() {
            this._state = ValidationState.OK;
        }
        Object.defineProperty(ValidationStatus.prototype, "state", {
            get: function () {
                return this._state;
            },
            set: function (value) {
                if (value > this._state) {
                    this._state = value;
                }
            },
            enumerable: true,
            configurable: true
        });
        ValidationStatus.prototype.isOK = function () {
            return this._state === ValidationState.OK;
        };
        ValidationStatus.prototype.isFatal = function () {
            return this._state === ValidationState.Fatal;
        };
        return ValidationStatus;
    }());
    exports.ValidationStatus = ValidationStatus;
    var Parser = (function () {
        function Parser(logger, validationStatus) {
            if (validationStatus === void 0) { validationStatus = new ValidationStatus(); }
            this._logger = logger;
            this.validationStatus = validationStatus;
        }
        Object.defineProperty(Parser.prototype, "logger", {
            get: function () {
                return this._logger;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Parser.prototype, "status", {
            get: function () {
                return this.validationStatus;
            },
            enumerable: true,
            configurable: true
        });
        Parser.prototype.log = function (message) {
            this._logger.log(message);
        };
        Parser.prototype.is = function (value, func, wrongTypeState, wrongTypeMessage, undefinedState, undefinedMessage) {
            if (Types.isUndefined(value)) {
                if (undefinedState) {
                    this.validationStatus.state = undefinedState;
                }
                if (undefinedMessage) {
                    this.log(undefinedMessage);
                }
                return false;
            }
            if (!func(value)) {
                if (wrongTypeState) {
                    this.validationStatus.state = wrongTypeState;
                }
                if (wrongTypeMessage) {
                    this.log(wrongTypeMessage);
                }
                return false;
            }
            return true;
        };
        Parser.merge = function (destination, source, overwrite) {
            var _this = this;
            Object.keys(source).forEach(function (key) {
                var destValue = destination[key];
                var sourceValue = source[key];
                if (Types.isUndefined(sourceValue)) {
                    return;
                }
                if (Types.isUndefined(destValue)) {
                    destination[key] = sourceValue;
                }
                else {
                    if (overwrite) {
                        if (Types.isObject(destValue) && Types.isObject(sourceValue)) {
                            _this.merge(destValue, sourceValue, overwrite);
                        }
                        else {
                            destination[key] = sourceValue;
                        }
                    }
                }
            });
        };
        return Parser;
    }());
    exports.Parser = Parser;
    var AbstractSystemVariables = (function () {
        function AbstractSystemVariables() {
        }
        AbstractSystemVariables.prototype.resolve = function (value) {
            if (Types.isString(value)) {
                return this.__resolveString(value);
            }
            else if (Types.isArray(value)) {
                return this.__resolveArray(value);
            }
            else if (Types.isObject(value)) {
                return this.__resolveLiteral(value);
            }
            return value;
        };
        AbstractSystemVariables.prototype.resolveAny = function (value) {
            if (Types.isString(value)) {
                return this.__resolveString(value);
            }
            else if (Types.isArray(value)) {
                return this.__resolveAnyArray(value);
            }
            else if (Types.isObject(value)) {
                return this.__resolveAnyLiteral(value);
            }
            return value;
        };
        AbstractSystemVariables.prototype.__resolveString = function (value) {
            var _this = this;
            var regexp = /\$\{(.*?)\}/g;
            return value.replace(regexp, function (match, name) {
                var newValue = _this[name];
                if (Types.isString(newValue)) {
                    return newValue;
                }
                else {
                    return match && match.indexOf('env.') > 0 ? '' : match;
                }
            });
        };
        AbstractSystemVariables.prototype.__resolveLiteral = function (values) {
            var _this = this;
            var result = Object.create(null);
            Object.keys(values).forEach(function (key) {
                var value = values[key];
                result[key] = _this.resolve(value);
            });
            return result;
        };
        AbstractSystemVariables.prototype.__resolveAnyLiteral = function (values) {
            var _this = this;
            var result = Object.create(null);
            Object.keys(values).forEach(function (key) {
                var value = values[key];
                result[key] = _this.resolveAny(value);
            });
            return result;
        };
        AbstractSystemVariables.prototype.__resolveArray = function (value) {
            var _this = this;
            return value.map(function (s) { return _this.__resolveString(s); });
        };
        AbstractSystemVariables.prototype.__resolveAnyArray = function (value) {
            var _this = this;
            return value.map(function (s) { return _this.resolveAny(s); });
        };
        return AbstractSystemVariables;
    }());
    exports.AbstractSystemVariables = AbstractSystemVariables;
});

define("vs/base/common/uri", ["require", "exports", 'vs/base/common/platform'], function (require, exports, platform) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/encodeURIComponent
    function fixedEncodeURIComponent(str) {
        return encodeURIComponent(str).replace(/[!'()*]/g, function (c) { return '%' + c.charCodeAt(0).toString(16).toUpperCase(); });
    }
    /**
     * Uniform Resource Identifier (URI) http://tools.ietf.org/html/rfc3986.
     * This class is a simple parser which creates the basic component paths
     * (http://tools.ietf.org/html/rfc3986#section-3) with minimal validation
     * and encoding.
     *
     *       foo://example.com:8042/over/there?name=ferret#nose
     *       \_/   \______________/\_________/ \_________/ \__/
     *        |           |            |            |        |
     *     scheme     authority       path        query   fragment
     *        |   _____________________|__
     *       / \ /                        \
     *       urn:example:animal:ferret:nose
     *
     *
     */
    var URI = (function () {
        function URI() {
            this._scheme = URI._empty;
            this._authority = URI._empty;
            this._path = URI._empty;
            this._query = URI._empty;
            this._fragment = URI._empty;
        }
        Object.defineProperty(URI.prototype, "scheme", {
            /**
             * scheme is the 'http' part of 'http://www.msft.com/some/path?query#fragment'.
             * The part before the first colon.
             */
            get: function () {
                return this._scheme;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(URI.prototype, "authority", {
            /**
             * authority is the 'www.msft.com' part of 'http://www.msft.com/some/path?query#fragment'.
             * The part between the first double slashes and the next slash.
             */
            get: function () {
                return this._authority;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(URI.prototype, "path", {
            /**
             * path is the '/some/path' part of 'http://www.msft.com/some/path?query#fragment'.
             */
            get: function () {
                return this._path;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(URI.prototype, "query", {
            /**
             * query is the 'query' part of 'http://www.msft.com/some/path?query#fragment'.
             */
            get: function () {
                return this._query;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(URI.prototype, "fragment", {
            /**
             * fragment is the 'fragment' part of 'http://www.msft.com/some/path?query#fragment'.
             */
            get: function () {
                return this._fragment;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(URI.prototype, "fsPath", {
            /**
             * Returns a string representing the corresponding file system path of this URI.
             * Will handle UNC paths and normalize windows drive letters to lower-case. Also
             * uses the platform specific path separator. Will *not* validate the path for
             * invalid characters and semantics. Will *not* look at the scheme of this URI.
             */
            get: function () {
                if (!this._fsPath) {
                    var value;
                    if (this._authority && this.scheme === 'file') {
                        // unc path: file://shares/c$/far/boo
                        value = "//" + this._authority + this._path;
                    }
                    else if (URI._driveLetterPath.test(this._path)) {
                        // windows drive letter: file:///c:/far/boo
                        value = this._path[1].toLowerCase() + this._path.substr(2);
                    }
                    else {
                        // other path
                        value = this._path;
                    }
                    if (platform.isWindows) {
                        value = value.replace(/\//g, '\\');
                    }
                    this._fsPath = value;
                }
                return this._fsPath;
            },
            enumerable: true,
            configurable: true
        });
        // ---- modify to new -------------------------
        URI.prototype.with = function (scheme, authority, path, query, fragment) {
            var ret = new URI();
            ret._scheme = scheme || this.scheme;
            ret._authority = authority || this.authority;
            ret._path = path || this.path;
            ret._query = query || this.query;
            ret._fragment = fragment || this.fragment;
            URI._validate(ret);
            return ret;
        };
        URI.prototype.withScheme = function (value) {
            return this.with(value, undefined, undefined, undefined, undefined);
        };
        URI.prototype.withAuthority = function (value) {
            return this.with(undefined, value, undefined, undefined, undefined);
        };
        URI.prototype.withPath = function (value) {
            return this.with(undefined, undefined, value, undefined, undefined);
        };
        URI.prototype.withQuery = function (value) {
            return this.with(undefined, undefined, undefined, value, undefined);
        };
        URI.prototype.withFragment = function (value) {
            return this.with(undefined, undefined, undefined, undefined, value);
        };
        // ---- parse & validate ------------------------
        URI.parse = function (value) {
            var ret = new URI();
            var data = URI._parseComponents(value);
            ret._scheme = data.scheme;
            ret._authority = decodeURIComponent(data.authority);
            ret._path = decodeURIComponent(data.path);
            ret._query = decodeURIComponent(data.query);
            ret._fragment = decodeURIComponent(data.fragment);
            URI._validate(ret);
            return ret;
        };
        URI.file = function (path) {
            path = path.replace(/\\/g, '/');
            path = path.replace(/%/g, '%25');
            path = path.replace(/#/g, '%23');
            path = path.replace(/\?/g, '%3F');
            // makes sure something like 'C:/Users' isn't
            // parsed as scheme='C', path='Users'
            path = URI._driveLetter.test(path)
                ? '/' + path
                : path;
            var data = URI._parseComponents(path);
            if (data.scheme || data.fragment || data.query) {
                throw new Error('Path contains a scheme, fragment or a query. Can not convert it to a file uri.');
            }
            var ret = new URI();
            ret._scheme = 'file';
            ret._authority = data.authority;
            ret._path = decodeURIComponent(data.path[0] === '/' ? data.path : '/' + data.path); // path starts with slash
            ret._query = data.query;
            ret._fragment = data.fragment;
            URI._validate(ret);
            return ret;
        };
        URI._parseComponents = function (value) {
            var ret = {
                scheme: URI._empty,
                authority: URI._empty,
                path: URI._empty,
                query: URI._empty,
                fragment: URI._empty,
            };
            var match = URI._regexp.exec(value);
            if (match) {
                ret.scheme = match[2] || ret.scheme;
                ret.authority = match[4] || ret.authority;
                ret.path = match[5] || ret.path;
                ret.query = match[7] || ret.query;
                ret.fragment = match[9] || ret.fragment;
            }
            return ret;
        };
        URI.create = function (scheme, authority, path, query, fragment) {
            return new URI().with(scheme, authority, path, query, fragment);
        };
        URI._validate = function (ret) {
            // validation
            // path, http://tools.ietf.org/html/rfc3986#section-3.3
            // If a URI contains an authority component, then the path component
            // must either be empty or begin with a slash ("/") character.  If a URI
            // does not contain an authority component, then the path cannot begin
            // with two slash characters ("//").
            if (ret.authority && ret.path && ret.path[0] !== '/') {
                throw new Error('[UriError]: If a URI contains an authority component, then the path component must either be empty or begin with a slash ("/") character');
            }
            if (!ret.authority && ret.path.indexOf('//') === 0) {
                throw new Error('[UriError]: If a URI does not contain an authority component, then the path cannot begin with two slash characters ("//")');
            }
        };
        URI.prototype.toString = function () {
            if (!this._formatted) {
                var parts = [];
                if (this._scheme) {
                    parts.push(this._scheme);
                    parts.push(':');
                }
                if (this._authority || this._scheme === 'file') {
                    parts.push('//');
                }
                if (this._authority) {
                    var authority = this._authority, idx;
                    authority = authority.toLowerCase();
                    idx = authority.indexOf(':');
                    if (idx === -1) {
                        parts.push(fixedEncodeURIComponent(authority));
                    }
                    else {
                        parts.push(fixedEncodeURIComponent(authority.substr(0, idx)));
                        parts.push(authority.substr(idx));
                    }
                }
                if (this._path) {
                    // encode every segment of the path
                    var path = this._path, segments;
                    // lower-case win drive letters in /C:/fff
                    if (URI._driveLetterPath.test(path)) {
                        path = '/' + path[1].toLowerCase() + path.substr(2);
                    }
                    else if (URI._driveLetter.test(path)) {
                        path = path[0].toLowerCase() + path.substr(1);
                    }
                    segments = path.split('/');
                    for (var i = 0, len = segments.length; i < len; i++) {
                        segments[i] = fixedEncodeURIComponent(segments[i]);
                    }
                    parts.push(segments.join('/'));
                }
                if (this._query) {
                    // in http(s) querys often use 'key=value'-pairs and
                    // ampersand characters for multiple pairs
                    var encoder = /https?/i.test(this.scheme)
                        ? encodeURI
                        : fixedEncodeURIComponent;
                    parts.push('?');
                    parts.push(encoder(this._query));
                }
                if (this._fragment) {
                    parts.push('#');
                    parts.push(fixedEncodeURIComponent(this._fragment));
                }
                this._formatted = parts.join('');
            }
            return this._formatted;
        };
        URI.prototype.toJSON = function () {
            return {
                scheme: this.scheme,
                authority: this.authority,
                path: this.path,
                fsPath: this.fsPath,
                query: this.query,
                fragment: this.fragment.replace(/URL_MARSHAL_REMOVE.*$/, ''),
                external: this.toString().replace(/#?URL_MARSHAL_REMOVE.*$/, ''),
                $mid: 1
            };
        };
        URI.revive = function (data) {
            var result = new URI();
            result._scheme = data.scheme;
            result._authority = data.authority;
            result._path = data.path;
            result._query = data.query;
            result._fragment = data.fragment;
            result._fsPath = data.fsPath;
            result._formatted = data.external;
            URI._validate(result);
            return result;
        };
        URI._empty = '';
        URI._regexp = /^(([^:/?#]+?):)?(\/\/([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?/;
        URI._driveLetterPath = /^\/[a-zA-z]:/;
        URI._driveLetter = /^[a-zA-z]:/;
        return URI;
    }());
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = URI;
});

define("vs/base/common/marshalling", ["require", "exports", 'vs/base/common/uri'], function (require, exports, uri_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function stringify(obj) {
        return JSON.stringify(obj, replacer);
    }
    exports.stringify = stringify;
    function parse(text) {
        return JSON.parse(text, reviver);
    }
    exports.parse = parse;
    function replacer(key, value) {
        // URI is done via toJSON-member
        if (value instanceof RegExp) {
            return {
                $mid: 2,
                source: value.source,
                flags: (value.global ? 'g' : '') + (value.ignoreCase ? 'i' : '') + (value.multiline ? 'm' : ''),
            };
        }
        return value;
    }
    function reviver(key, value) {
        var marshallingConst;
        if (value !== void 0 && value !== null) {
            marshallingConst = value.$mid;
        }
        if (marshallingConst === 1) {
            return uri_1.default.revive(value);
        }
        else if (marshallingConst === 2) {
            return new RegExp(value.source, value.flags);
        }
        else {
            return value;
        }
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/common/uuid", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ValueUUID = (function () {
        function ValueUUID(_value) {
            this._value = _value;
            // empty
        }
        ValueUUID.prototype.asHex = function () {
            return this._value;
        };
        ValueUUID.prototype.equals = function (other) {
            return this.asHex() === other.asHex();
        };
        return ValueUUID;
    }());
    var V4UUID = (function (_super) {
        __extends(V4UUID, _super);
        function V4UUID() {
            _super.call(this, [
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                '-',
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                '-',
                '4',
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                '-',
                V4UUID._oneOf(V4UUID._timeHighBits),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                '-',
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
                V4UUID._randomHex(),
            ].join(''));
        }
        V4UUID._oneOf = function (array) {
            var idx = Math.floor(array.length * Math.random());
            return array[idx];
        };
        V4UUID._randomHex = function () {
            return V4UUID._oneOf(V4UUID._chars);
        };
        V4UUID._chars = ['0', '1', '2', '3', '4', '5', '6', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'];
        V4UUID._timeHighBits = ['8', '9', 'a', 'b'];
        return V4UUID;
    }(ValueUUID));
    /**
     * An empty UUID that contains only zeros.
     */
    exports.empty = new ValueUUID('00000000-0000-0000-0000-000000000000');
    function v4() {
        return new V4UUID();
    }
    exports.v4 = v4;
    var _UUIDPattern = /^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12}$/;
    /**
     * Parses a UUID that is of the format xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx.
     * @param value A uuid string.
     */
    function parse(value) {
        if (!_UUIDPattern.test(value)) {
            throw new Error('invalid uuid');
        }
        return new ValueUUID(value);
    }
    exports.parse = parse;
    function generateUuid() {
        return v4().asHex();
    }
    exports.generateUuid = generateUuid;
});

/// <loc filename="Metadata\base_loc_oam.xml" format="messagebundle" />
/*! @minifier_do_not_preserve
  © Microsoft. All rights reserved.

  This library is supported for use in Windows Store apps only.

  Build: 1.0.9200.20602.win8_ldr.130108-1504

  Version: Microsoft.WinJS.1.0
*/

/*
	Note: Copied out of base.js.
	Changes:
		- we have only kept the first 2554 lines.
		- we have patched WinJS.xhr to add the hedader X-Requested-With:XMLHttpRequest
		- we have wrapped the entire code in an if statement to make WinJS re-entrant (if already defined)
		- we have to define setImmediate if not running in IE 10 since its a IE 10 only function
		- we have removed some getter syntax
*/

// MONACO CHANGE: Make WinJS re-entrant (if already defined)
if (typeof WinJS === 'undefined') {

// MONACO CHANGE: define setImmediate
(function (global) {
    if (!global.setImmediate) {
        if (typeof process !== 'undefined' && typeof process.nextTick === 'function') {
            // running in node
            global.setImmediate = function(callback) {
                return process.nextTick(callback);
            };
        } else {
            // running in browser
            global.setImmediate = function(callback) {
                return setTimeout(callback, 0);
            };
        }
	}

})(this);

/// <reference path="ms-appx://Microsoft.WinJS.1.0/js/base.js" />
(function baseInit(global, undefined) {
    "use strict";

    function initializeProperties(target, members) {
        var keys = Object.keys(members);
        var properties;
        var i, len;
        for (i = 0, len = keys.length; i < len; i++) {
            var key = keys[i];
            var enumerable = key.charCodeAt(0) !== /*_*/95;
            var member = members[key];
            if (member && typeof member === 'object') {
                if (member.value !== undefined || typeof member.get === 'function' || typeof member.set === 'function') {
                    if (member.enumerable === undefined) {
                        member.enumerable = enumerable;
                    }
                    properties = properties || {};
                    properties[key] = member;
                    continue;
                }
            }
            if (!enumerable) {
                properties = properties || {};
                properties[key] = { value: member, enumerable: enumerable, configurable: true, writable: true }
                continue;
            }
            target[key] = member;
        }
        if (properties) {
            Object.defineProperties(target, properties);
        }
    }

    (function (rootNamespace) {

        // Create the rootNamespace in the global namespace
        if (!global[rootNamespace]) {
            global[rootNamespace] = Object.create(Object.prototype);
        }

        // Cache the rootNamespace we just created in a local variable
        var _rootNamespace = global[rootNamespace];
        if (!_rootNamespace.Namespace) {
            _rootNamespace.Namespace = Object.create(Object.prototype);
        }

        function defineWithParent(parentNamespace, name, members) {
            /// <signature helpKeyword="WinJS.Namespace.defineWithParent">
            /// <summary locid="WinJS.Namespace.defineWithParent">
            /// Defines a new namespace with the specified name under the specified parent namespace.
            /// </summary>
            /// <param name="parentNamespace" type="Object" locid="WinJS.Namespace.defineWithParent_p:parentNamespace">
            /// The parent namespace.
            /// </param>
            /// <param name="name" type="String" locid="WinJS.Namespace.defineWithParent_p:name">
            /// The name of the new namespace.
            /// </param>
            /// <param name="members" type="Object" locid="WinJS.Namespace.defineWithParent_p:members">
            /// The members of the new namespace.
            /// </param>
            /// <returns type="Object" locid="WinJS.Namespace.defineWithParent_returnValue">
            /// The newly-defined namespace.
            /// </returns>
            /// </signature>
            var currentNamespace = parentNamespace,
                namespaceFragments = name.split(".");

            for (var i = 0, len = namespaceFragments.length; i < len; i++) {
                var namespaceName = namespaceFragments[i];
                if (!currentNamespace[namespaceName]) {
                    Object.defineProperty(currentNamespace, namespaceName,
                        { value: {}, writable: false, enumerable: true, configurable: true }
                    );
                }
                currentNamespace = currentNamespace[namespaceName];
            }

            if (members) {
                initializeProperties(currentNamespace, members);
            }

            return currentNamespace;
        }

        function define(name, members) {
            /// <signature helpKeyword="WinJS.Namespace.define">
            /// <summary locid="WinJS.Namespace.define">
            /// Defines a new namespace with the specified name.
            /// </summary>
            /// <param name="name" type="String" locid="WinJS.Namespace.define_p:name">
            /// The name of the namespace. This could be a dot-separated name for nested namespaces.
            /// </param>
            /// <param name="members" type="Object" locid="WinJS.Namespace.define_p:members">
            /// The members of the new namespace.
            /// </param>
            /// <returns type="Object" locid="WinJS.Namespace.define_returnValue">
            /// The newly-defined namespace.
            /// </returns>
            /// </signature>
            return defineWithParent(global, name, members);
        }

        // Establish members of the "WinJS.Namespace" namespace
        Object.defineProperties(_rootNamespace.Namespace, {

            defineWithParent: { value: defineWithParent, writable: true, enumerable: true, configurable: true },

            define: { value: define, writable: true, enumerable: true, configurable: true }

        });

    })("WinJS");

    (function (WinJS) {

        function define(constructor, instanceMembers, staticMembers) {
            /// <signature helpKeyword="WinJS.Class.define">
            /// <summary locid="WinJS.Class.define">
            /// Defines a class using the given constructor and the specified instance members.
            /// </summary>
            /// <param name="constructor" type="Function" locid="WinJS.Class.define_p:constructor">
            /// A constructor function that is used to instantiate this class.
            /// </param>
            /// <param name="instanceMembers" type="Object" locid="WinJS.Class.define_p:instanceMembers">
            /// The set of instance fields, properties, and methods made available on the class.
            /// </param>
            /// <param name="staticMembers" type="Object" locid="WinJS.Class.define_p:staticMembers">
            /// The set of static fields, properties, and methods made available on the class.
            /// </param>
            /// <returns type="Function" locid="WinJS.Class.define_returnValue">
            /// The newly-defined class.
            /// </returns>
            /// </signature>
            constructor = constructor || function () { };
            WinJS.Utilities.markSupportedForProcessing(constructor);
            if (instanceMembers) {
                initializeProperties(constructor.prototype, instanceMembers);
            }
            if (staticMembers) {
                initializeProperties(constructor, staticMembers);
            }
            return constructor;
        }

        function derive(baseClass, constructor, instanceMembers, staticMembers) {
            /// <signature helpKeyword="WinJS.Class.derive">
            /// <summary locid="WinJS.Class.derive">
            /// Creates a sub-class based on the supplied baseClass parameter, using prototypal inheritance.
            /// </summary>
            /// <param name="baseClass" type="Function" locid="WinJS.Class.derive_p:baseClass">
            /// The class to inherit from.
            /// </param>
            /// <param name="constructor" type="Function" locid="WinJS.Class.derive_p:constructor">
            /// A constructor function that is used to instantiate this class.
            /// </param>
            /// <param name="instanceMembers" type="Object" locid="WinJS.Class.derive_p:instanceMembers">
            /// The set of instance fields, properties, and methods to be made available on the class.
            /// </param>
            /// <param name="staticMembers" type="Object" locid="WinJS.Class.derive_p:staticMembers">
            /// The set of static fields, properties, and methods to be made available on the class.
            /// </param>
            /// <returns type="Function" locid="WinJS.Class.derive_returnValue">
            /// The newly-defined class.
            /// </returns>
            /// </signature>
            if (baseClass) {
                constructor = constructor || function () { };
                var basePrototype = baseClass.prototype;
                constructor.prototype = Object.create(basePrototype);
                WinJS.Utilities.markSupportedForProcessing(constructor);
                Object.defineProperty(constructor.prototype, "constructor", { value: constructor, writable: true, configurable: true, enumerable: true });
                if (instanceMembers) {
                    initializeProperties(constructor.prototype, instanceMembers);
                }
                if (staticMembers) {
                    initializeProperties(constructor, staticMembers);
                }
                return constructor;
            } else {
                return define(constructor, instanceMembers, staticMembers);
            }
        }

        function mix(constructor) {
            /// <signature helpKeyword="WinJS.Class.mix">
            /// <summary locid="WinJS.Class.mix">
            /// Defines a class using the given constructor and the union of the set of instance members
            /// specified by all the mixin objects. The mixin parameter list is of variable length.
            /// </summary>
            /// <param name="constructor" locid="WinJS.Class.mix_p:constructor">
            /// A constructor function that is used to instantiate this class.
            /// </param>
            /// <returns type="Function" locid="WinJS.Class.mix_returnValue">
            /// The newly-defined class.
            /// </returns>
            /// </signature>
            constructor = constructor || function () { };
            var i, len;
            for (i = 1, len = arguments.length; i < len; i++) {
                initializeProperties(constructor.prototype, arguments[i]);
            }
            return constructor;
        }

        // Establish members of "WinJS.Class" namespace
        WinJS.Namespace.define("WinJS.Class", {
            define: define,
            derive: derive,
            mix: mix
        });

    })(global.WinJS);

})(this);


(function baseUtilsInit(global, WinJS) {
    "use strict";

    var hasWinRT = !!global.Windows;

    var strings = {
		// MONACOCHANGE
        //get notSupportedForProcessing() { return WinJS.Resources._getWinJSString("base/notSupportedForProcessing").value; }
		notSupportedForProcessing: "Value is not supported within a declarative processing context, if you want it to be supported mark it using WinJS.Utilities.markSupportedForProcessing. The value was: '{0}'"
    };

    function nop(v) {
        return v;
    }

    function getMemberFiltered(name, root, filter) {
        return name.split(".").reduce(function (currentNamespace, name) {
            if (currentNamespace) {
                return filter(currentNamespace[name]);
            }
            return null;
        }, root);
    }

    // Establish members of "WinJS.Utilities" namespace
    WinJS.Namespace.define("WinJS.Utilities", {
        // Used for mocking in tests
        _setHasWinRT: {
            value: function (value) {
                hasWinRT = value;
            },
            configurable: false,
            writable: false,
            enumerable: false
        },

        /// <field type="Boolean" locid="WinJS.Utilities.hasWinRT" helpKeyword="WinJS.Utilities.hasWinRT">Determine if WinRT is accessible in this script context.</field>
        hasWinRT: {
            get: function () { return hasWinRT; },
            configurable: false,
            enumerable: true
        },

        _getMemberFiltered: getMemberFiltered,

        getMember: function (name, root) {
            /// <signature helpKeyword="WinJS.Utilities.getMember">
            /// <summary locid="WinJS.Utilities.getMember">
            /// Gets the leaf-level type or namespace specified by the name parameter.
            /// </summary>
            /// <param name="name" locid="WinJS.Utilities.getMember_p:name">
            /// The name of the member.
            /// </param>
            /// <param name="root" locid="WinJS.Utilities.getMember_p:root">
            /// The root to start in. Defaults to the global object.
            /// </param>
            /// <returns type="Object" locid="WinJS.Utilities.getMember_returnValue">
            /// The leaf-level type or namespace in the specified parent namespace.
            /// </returns>
            /// </signature>
            if (!name) {
                return null;
            }
            return getMemberFiltered(name, root || global, nop);
        },

        ready: function (callback, async) {
            /// <signature helpKeyword="WinJS.Utilities.ready">
            /// <summary locid="WinJS.Utilities.ready">
            /// Ensures that the specified function executes only after the DOMContentLoaded event has fired
            /// for the current page.
            /// </summary>
            /// <returns type="WinJS.Promise" locid="WinJS.Utilities.ready_returnValue">A promise that completes after DOMContentLoaded has occurred.</returns>
            /// <param name="callback" optional="true" locid="WinJS.Utilities.ready_p:callback">
            /// A function that executes after DOMContentLoaded has occurred.
            /// </param>
            /// <param name="async" optional="true" locid="WinJS.Utilities.ready_p:async">
            /// If true, the callback should be executed asynchronously.
            /// </param>
            /// </signature>
            return new WinJS.Promise(function (c, e) {
                function complete() {
                    if (callback) {
                        try {
                            callback();
                            c();
                        }
                        catch (err) {
                            e(err);
                        }
                    }
                    else {
                        c();
                    }
                }

                var readyState = WinJS.Utilities.testReadyState;
                if (!readyState) {
                    if (global.document) {
                        readyState = document.readyState;
                    }
                    else {
                        readyState = "complete";
                    }
                }
                if (readyState === "complete" || (global.document && document.body !== null)) {
                    if (async) {
                        global.setImmediate(complete);
                    }
                    else {
                        complete();
                    }
                }
                else {
                    global.addEventListener("DOMContentLoaded", complete, false);
                }
            });
        },

        /// <field type="Boolean" locid="WinJS.Utilities.strictProcessing" helpKeyword="WinJS.Utilities.strictProcessing">Determines if strict declarative processing is enabled in this script context.</field>
        strictProcessing: {
            get: function () { return true; },
            configurable: false,
            enumerable: true,
        },

        markSupportedForProcessing: {
            value: function (func) {
                /// <signature helpKeyword="WinJS.Utilities.markSupportedForProcessing">
                /// <summary locid="WinJS.Utilities.markSupportedForProcessing">
                /// Marks a function as being compatible with declarative processing, such as WinJS.UI.processAll
                /// or WinJS.Binding.processAll.
                /// </summary>
                /// <param name="func" type="Function" locid="WinJS.Utilities.markSupportedForProcessing_p:func">
                /// The function to be marked as compatible with declarative processing.
                /// </param>
                /// <returns type="Function" locid="WinJS.Utilities.markSupportedForProcessing_returnValue">
                /// The input function.
                /// </returns>
                /// </signature>
                func.supportedForProcessing = true;
                return func;
            },
            configurable: false,
            writable: false,
            enumerable: true
        },

        requireSupportedForProcessing: {
            value: function (value) {
                /// <signature helpKeyword="WinJS.Utilities.requireSupportedForProcessing">
                /// <summary locid="WinJS.Utilities.requireSupportedForProcessing">
                /// Asserts that the value is compatible with declarative processing, such as WinJS.UI.processAll
                /// or WinJS.Binding.processAll. If it is not compatible an exception will be thrown.
                /// </summary>
                /// <param name="value" type="Object" locid="WinJS.Utilities.requireSupportedForProcessing_p:value">
                /// The value to be tested for compatibility with declarative processing. If the
                /// value is a function it must be marked with a property 'supportedForProcessing'
                /// with a value of true.
                /// </param>
                /// <returns type="Object" locid="WinJS.Utilities.requireSupportedForProcessing_returnValue">
                /// The input value.
                /// </returns>
                /// </signature>
                var supportedForProcessing = true;

                supportedForProcessing = supportedForProcessing && !(value === global);
                supportedForProcessing = supportedForProcessing && !(value === global.location);
                supportedForProcessing = supportedForProcessing && !(value instanceof HTMLIFrameElement);
                supportedForProcessing = supportedForProcessing && !(typeof value === "function" && !value.supportedForProcessing);

                switch (global.frames.length) {
                    case 0:
                        break;

                    case 1:
                        supportedForProcessing = supportedForProcessing && !(value === global.frames[0]);
                        break;

                    default:
                        for (var i = 0, len = global.frames.length; supportedForProcessing && i < len; i++) {
                            supportedForProcessing = supportedForProcessing && !(value === global.frames[i]);
                        }
                        break;
                }

                if (supportedForProcessing) {
                    return value;
                }

                throw new WinJS.ErrorFromName("WinJS.Utilities.requireSupportedForProcessing", WinJS.Resources._formatString(strings.notSupportedForProcessing, value));
            },
            configurable: false,
            writable: false,
            enumerable: true
        },

    });

    WinJS.Namespace.define("WinJS", {
        validation: false,

        strictProcessing: {
            value: function () {
                /// <signature helpKeyword="WinJS.strictProcessing">
                /// <summary locid="WinJS.strictProcessing">
                /// Strict processing is always enforced, this method has no effect.
                /// </summary>
                /// </signature>
            },
            configurable: false,
            writable: false,
            enumerable: false
        },
    });
})(this, this.WinJS);


(function logInit(WinJS) {
    "use strict";

    var spaceR = /\s+/g;
    var typeR = /^(error|warn|info|log)$/;

    function format(message, tag, type) {
        /// <signature helpKeyword="WinJS.Utilities.formatLog">
        /// <summary locid="WinJS.Utilities.formatLog">
        /// Adds tags and type to a logging message.
        /// </summary>
        /// <param name="message" type="String" locid="WinJS.Utilities.startLog_p:message">The message to be formatted.</param>
        /// <param name="tag" type="String" locid="WinJS.Utilities.startLog_p:tag">The tag(s) to be applied to the message. Multiple tags should be separated by spaces.</param>
        /// <param name="type" type="String" locid="WinJS.Utilities.startLog_p:type">The type of the message.</param>
        /// <returns type="String" locid="WinJS.Utilities.startLog_returnValue">The formatted message.</returns>
        /// </signature>
        var m = message;
        if (typeof (m) === "function") { m = m(); }

        return ((type && typeR.test(type)) ? ("") : (type ? (type + ": ") : "")) +
            (tag ? tag.replace(spaceR, ":") + ": " : "") +
            m;
    }
    function defAction(message, tag, type) {
        var m = WinJS.Utilities.formatLog(message, tag, type);
        console[(type && typeR.test(type)) ? type : "log"](m);
    }
    function escape(s) {
        // \s (whitespace) is used as separator, so don't escape it
        return s.replace(/[-[\]{}()*+?.,\\^$|#]/g, "\\$&");
    }
    WinJS.Namespace.define("WinJS.Utilities", {
        startLog: function (options) {
            /// <signature helpKeyword="WinJS.Utilities.startLog">
            /// <summary locid="WinJS.Utilities.startLog">
            /// Configures a logger that writes messages containing the specified tags from WinJS.log to console.log.
            /// </summary>
            /// <param name="options" type="String" locid="WinJS.Utilities.startLog_p:options">The tags for messages to log. Multiple tags should be separated by spaces.</param>
            /// </signature>
            /// <signature>
            /// <summary locid="WinJS.Utilities.startLog2">
            /// Configure a logger to write WinJS.log output.
            /// </summary>
            /// <param name="options" type="Object" locid="WinJS.Utilities.startLog_p:options2">
            /// May contain .type, .tags, .excludeTags and .action properties.
            /// - .type is a required tag.
            /// - .excludeTags is a space-separated list of tags, any of which will result in a message not being logged.
            /// - .tags is a space-separated list of tags, any of which will result in a message being logged.
            /// - .action is a function that, if present, will be called with the log message, tags and type. The default is to log to the console.
            /// </param>
            /// </signature>
            options = options || {};
            if (typeof options === "string") {
                options = { tags: options };
            }
            var el = options.type && new RegExp("^(" + escape(options.type).replace(spaceR, " ").split(" ").join("|") + ")$");
            var not = options.excludeTags && new RegExp("(^|\\s)(" + escape(options.excludeTags).replace(spaceR, " ").split(" ").join("|") + ")(\\s|$)", "i");
            var has = options.tags && new RegExp("(^|\\s)(" + escape(options.tags).replace(spaceR, " ").split(" ").join("|") + ")(\\s|$)", "i");
            var action = options.action || defAction;

            if (!el && !not && !has && !WinJS.log) {
                WinJS.log = action;
                return;
            }

            var result = function (message, tag, type) {
                if (!((el && !el.test(type))          // if the expected log level is not satisfied
                    || (not && not.test(tag))         // if any of the excluded categories exist
                    || (has && !has.test(tag)))) {    // if at least one of the included categories doesn't exist
                        action(message, tag, type);
                    }

                result.next && result.next(message, tag, type);
            };
            result.next = WinJS.log;
            WinJS.log = result;
        },
        stopLog: function () {
            /// <signature helpKeyword="WinJS.Utilities.stopLog">
            /// <summary locid="WinJS.Utilities.stopLog">
            /// Removes the previously set up logger.
            /// </summary>
            /// </signature>
            delete WinJS.log;
        },
        formatLog: format
    });
})(this.WinJS);

(function eventsInit(WinJS, undefined) {
    "use strict";


    function createEventProperty(name) {
        var eventPropStateName = "_on" + name + "state";

        return {
            get: function () {
                var state = this[eventPropStateName];
                return state && state.userHandler;
            },
            set: function (handler) {
                var state = this[eventPropStateName];
                if (handler) {
                    if (!state) {
                        state = { wrapper: function (evt) { return state.userHandler(evt); }, userHandler: handler };
                        Object.defineProperty(this, eventPropStateName, { value: state, enumerable: false, writable:true, configurable: true });
                        this.addEventListener(name, state.wrapper, false);
                    }
                    state.userHandler = handler;
                } else if (state) {
                    this.removeEventListener(name, state.wrapper, false);
                    this[eventPropStateName] = null;
                }
            },
            enumerable: true
        }
    }

    function createEventProperties(events) {
        /// <signature helpKeyword="WinJS.Utilities.createEventProperties">
        /// <summary locid="WinJS.Utilities.createEventProperties">
        /// Creates an object that has one property for each name passed to the function.
        /// </summary>
        /// <param name="events" locid="WinJS.Utilities.createEventProperties_p:events">
        /// A variable list of property names.
        /// </param>
        /// <returns type="Object" locid="WinJS.Utilities.createEventProperties_returnValue">
        /// The object with the specified properties. The names of the properties are prefixed with 'on'.
        /// </returns>
        /// </signature>
        var props = {};
        for (var i = 0, len = arguments.length; i < len; i++) {
            var name = arguments[i];
            props["on" + name] = createEventProperty(name);
        }
        return props;
    }

    var EventMixinEvent = WinJS.Class.define(
        function EventMixinEvent_ctor(type, detail, target) {
            this.detail = detail;
            this.target = target;
            this.timeStamp = Date.now();
            this.type = type;
        },
        {
            bubbles: { value: false, writable: false },
            cancelable: { value: false, writable: false },
            currentTarget: {
                get: function () { return this.target; }
            },
            defaultPrevented: {
                get: function () { return this._preventDefaultCalled; }
            },
            trusted: { value: false, writable: false },
            eventPhase: { value: 0, writable: false },
            target: null,
            timeStamp: null,
            type: null,

            preventDefault: function () {
                this._preventDefaultCalled = true;
            },
            stopImmediatePropagation: function () {
                this._stopImmediatePropagationCalled = true;
            },
            stopPropagation: function () {
            }
        }, {
            supportedForProcessing: false,
        }
    );

    var eventMixin = {
        _listeners: null,

        addEventListener: function (type, listener, useCapture) {
            /// <signature helpKeyword="WinJS.Utilities.eventMixin.addEventListener">
            /// <summary locid="WinJS.Utilities.eventMixin.addEventListener">
            /// Adds an event listener to the control.
            /// </summary>
            /// <param name="type" locid="WinJS.Utilities.eventMixin.addEventListener_p:type">
            /// The type (name) of the event.
            /// </param>
            /// <param name="listener" locid="WinJS.Utilities.eventMixin.addEventListener_p:listener">
            /// The listener to invoke when the event gets raised.
            /// </param>
            /// <param name="useCapture" locid="WinJS.Utilities.eventMixin.addEventListener_p:useCapture">
            /// if true initiates capture, otherwise false.
            /// </param>
            /// </signature>
            useCapture = useCapture || false;
            this._listeners = this._listeners || {};
            var eventListeners = (this._listeners[type] = this._listeners[type] || []);
            for (var i = 0, len = eventListeners.length; i < len; i++) {
                var l = eventListeners[i];
                if (l.useCapture === useCapture && l.listener === listener) {
                    return;
                }
            }
            eventListeners.push({ listener: listener, useCapture: useCapture });
        },
        dispatchEvent: function (type, details) {
            /// <signature helpKeyword="WinJS.Utilities.eventMixin.dispatchEvent">
            /// <summary locid="WinJS.Utilities.eventMixin.dispatchEvent">
            /// Raises an event of the specified type and with the specified additional properties.
            /// </summary>
            /// <param name="type" locid="WinJS.Utilities.eventMixin.dispatchEvent_p:type">
            /// The type (name) of the event.
            /// </param>
            /// <param name="details" locid="WinJS.Utilities.eventMixin.dispatchEvent_p:details">
            /// The set of additional properties to be attached to the event object when the event is raised.
            /// </param>
            /// <returns type="Boolean" locid="WinJS.Utilities.eventMixin.dispatchEvent_returnValue">
            /// true if preventDefault was called on the event.
            /// </returns>
            /// </signature>
            var listeners = this._listeners && this._listeners[type];
            if (listeners) {
                var eventValue = new EventMixinEvent(type, details, this);
                // Need to copy the array to protect against people unregistering while we are dispatching
                listeners = listeners.slice(0, listeners.length);
                for (var i = 0, len = listeners.length; i < len && !eventValue._stopImmediatePropagationCalled; i++) {
                    listeners[i].listener(eventValue);
                }
                return eventValue.defaultPrevented || false;
            }
            return false;
        },
        removeEventListener: function (type, listener, useCapture) {
            /// <signature helpKeyword="WinJS.Utilities.eventMixin.removeEventListener">
            /// <summary locid="WinJS.Utilities.eventMixin.removeEventListener">
            /// Removes an event listener from the control.
            /// </summary>
            /// <param name="type" locid="WinJS.Utilities.eventMixin.removeEventListener_p:type">
            /// The type (name) of the event.
            /// </param>
            /// <param name="listener" locid="WinJS.Utilities.eventMixin.removeEventListener_p:listener">
            /// The listener to remove.
            /// </param>
            /// <param name="useCapture" locid="WinJS.Utilities.eventMixin.removeEventListener_p:useCapture">
            /// Specifies whether to initiate capture.
            /// </param>
            /// </signature>
            useCapture = useCapture || false;
            var listeners = this._listeners && this._listeners[type];
            if (listeners) {
                for (var i = 0, len = listeners.length; i < len; i++) {
                    var l = listeners[i];
                    if (l.listener === listener && l.useCapture === useCapture) {
                        listeners.splice(i, 1);
                        if (listeners.length === 0) {
                            delete this._listeners[type];
                        }
                        // Only want to remove one element for each call to removeEventListener
                        break;
                    }
                }
            }
        }
    };

    WinJS.Namespace.define("WinJS.Utilities", {
        _createEventProperty: createEventProperty,
        createEventProperties: createEventProperties,
        eventMixin: eventMixin
    });

})(this.WinJS);


(function resourcesInit(global, WinJS, undefined) {
    "use strict";

    var resourceMap;
    var mrtEventHook = false;
    var contextChangedET = "contextchanged";

    var ListenerType = WinJS.Class.mix(WinJS.Class.define(null, { /* empty */ }, { supportedForProcessing: false }), WinJS.Utilities.eventMixin);
    var listeners = new ListenerType();

    var strings = {
		// MONACO CHANGE
        //get malformedFormatStringInput() { return WinJS.Resources._getWinJSString("base/malformedFormatStringInput").value; },
		malformedFormatStringInput: "Malformed, did you mean to escape your '{0}'?"
    };

    WinJS.Namespace.define("WinJS.Resources", {
        addEventListener: function (type, listener, useCapture) {
            /// <signature helpKeyword="WinJS.Resources.addEventListener">
            /// <summary locid="WinJS.Resources.addEventListener">
            /// Registers an event handler for the specified event.
            /// </summary>
            /// <param name="type" type="String" locid="WinJS.Resources.addEventListener_p:type">
            /// The name of the event to handle.
            /// </param>
            /// <param name="listener" type="Function" locid="WinJS.Resources.addEventListener_p:listener">
            /// The listener to invoke when the event gets raised.
            /// </param>
            /// <param name="useCapture" type="Boolean" locid="WinJS.Resources.addEventListener_p:useCapture">
            /// Set to true to register the event handler for the capturing phase; set to false to register for the bubbling phase.
            /// </param>
            /// </signature>
            if (WinJS.Utilities.hasWinRT && !mrtEventHook) {
                if (type === contextChangedET) {
                    try {
                        Windows.ApplicationModel.Resources.Core.ResourceManager.current.defaultContext.qualifierValues.addEventListener("mapchanged", function (e) {
                            WinJS.Resources.dispatchEvent(contextChangedET, { qualifier: e.key, changed: e.target[e.key] });
                        }, false);

                        mrtEventHook = true;
                    } catch (e) {
                    }
                }
            }
            listeners.addEventListener(type, listener, useCapture);
        },
        removeEventListener: listeners.removeEventListener.bind(listeners),
        dispatchEvent: listeners.dispatchEvent.bind(listeners),

        _formatString: function (string) {
            var args = arguments;
            if (args.length > 1) {
                string = string.replace(/({{)|(}})|{(\d+)}|({)|(})/g, function (unused, left, right, index, illegalLeft, illegalRight) {
                    if (illegalLeft || illegalRight) { throw WinJS.Resources._formatString(strings.malformedFormatStringInput, illegalLeft || illegalRight); }
                    return (left && "{") || (right && "}") || args[(index|0) + 1];
                });
            }
            return string;
        },

        _getStringWinRT: function (resourceId) {
            if (!resourceMap) {
                var mainResourceMap = Windows.ApplicationModel.Resources.Core.ResourceManager.current.mainResourceMap;
                try {
                    resourceMap = mainResourceMap.getSubtree('Resources');
                }
                catch (e) {
                }
                if (!resourceMap) {
                    resourceMap = mainResourceMap;
                }
            }

            var stringValue;
            var langValue;
            var resCandidate;
            try {
                resCandidate = resourceMap.getValue(resourceId);
                if (resCandidate) {
                    stringValue = resCandidate.valueAsString;
                    if (stringValue === undefined) {
                        stringValue = resCandidate.toString();
                    }
                }
            }
            catch (e) {}

            if (!stringValue) {
                return { value: resourceId, empty: true };
            }

            try {
                langValue = resCandidate.getQualifierValue("Language");
            }
            catch (e) {
                return { value: stringValue };
            }

            return { value: stringValue, lang: langValue };
        },

        _getStringJS: function (resourceId) {
            var str = global.strings && global.strings[resourceId];
            if (typeof str === "string") {
                str = { value: str };
            }
            return str || { value: resourceId, empty: true };
        }
    });

    Object.defineProperties(WinJS.Resources, WinJS.Utilities.createEventProperties(contextChangedET));

    var getStringImpl;

    WinJS.Resources.getString = function (resourceId) {
        /// <signature helpKeyword="WinJS.Resources.getString">
        /// <summary locid="WinJS.Resources.getString">
        /// Retrieves the resource string that has the specified resource id.
        /// </summary>
        /// <param name="resourceId" type="Number" locid="WinJS.Resources.getString._p:resourceId">
        /// The resource id of the string to retrieve.
        /// </param>
        /// <returns type="Object" locid="WinJS.Resources.getString_returnValue">
        /// An object that can contain these properties:
        ///
        /// value:
        /// The value of the requested string. This property is always present.
        ///
        /// empty:
        /// A value that specifies whether the requested string wasn't found.
        /// If its true, the string wasn't found. If its false or undefined,
        /// the requested string was found.
        ///
        /// lang:
        /// The language of the string, if specified. This property is only present
        /// for multi-language resources.
        ///
        /// </returns>
        /// </signature>
        getStringImpl =
            getStringImpl ||
                (WinJS.Utilities.hasWinRT
                    ? WinJS.Resources._getStringWinRT
                    : WinJS.Resources._getStringJS);

        return getStringImpl(resourceId);
    };


})(this, this.WinJS);


(function promiseInit(global, WinJS, undefined) {
    "use strict";

    global.Debug && (global.Debug.setNonUserCodeExceptions = true);

    var ListenerType = WinJS.Class.mix(WinJS.Class.define(null, { /*empty*/ }, { supportedForProcessing: false }), WinJS.Utilities.eventMixin);
    var promiseEventListeners = new ListenerType();
    // make sure there is a listeners collection so that we can do a more trivial check below
    promiseEventListeners._listeners = {};
    var errorET = "error";
    var canceledName = "Canceled";
    var tagWithStack = false;
    var tag = {
        promise:            0x01,
        thenPromise:        0x02,
        errorPromise:       0x04,
        exceptionPromise:   0x08,
        completePromise:    0x10,
    };
    tag.all = tag.promise | tag.thenPromise | tag.errorPromise | tag.exceptionPromise | tag.completePromise;

    //
    // Global error counter, for each error which enters the system we increment this once and then
    // the error number travels with the error as it traverses the tree of potential handlers.
    //
    // When someone has registered to be told about errors (WinJS.Promise.callonerror) promises
    // which are in error will get tagged with a ._errorId field. This tagged field is the
    // contract by which nested promises with errors will be identified as chaining for the
    // purposes of the callonerror semantics. If a nested promise in error is encountered without
    // a ._errorId it will be assumed to be foreign and treated as an interop boundary and
    // a new error id will be minted.
    //
    var error_number = 1;

    //
    // The state machine has a interesting hiccup in it with regards to notification, in order
    // to flatten out notification and avoid recursion for synchronous completion we have an
    // explicit set of *_notify states which are responsible for notifying their entire tree
    // of children. They can do this because they know that immediate children are always
    // ThenPromise instances and we can therefore reach into their state to access the
    // _listeners collection.
    //
    // So, what happens is that a Promise will be fulfilled through the _completed or _error
    // messages at which point it will enter a *_notify state and be responsible for to move
    // its children into an (as appropriate) success or error state and also notify that child's
    // listeners of the state transition, until leaf notes are reached.
    //

    var state_created,              // -> working
        state_working,              // -> error | error_notify | success | success_notify | canceled | waiting
        state_waiting,              // -> error | error_notify | success | success_notify | waiting_canceled
        state_waiting_canceled,     // -> error | error_notify | success | success_notify | canceling
        state_canceled,             // -> error | error_notify | success | success_notify | canceling
        state_canceling,            // -> error_notify
        state_success_notify,       // -> success
        state_success,              // -> .
        state_error_notify,         // -> error
        state_error;                // -> .

    // Noop function, used in the various states to indicate that they don't support a given
    // message. Named with the somewhat cute name '_' because it reads really well in the states.

    function _() { }

    // Initial state
    //
    state_created = {
        name: "created",
        enter: function (promise) {
            promise._setState(state_working);
        },
        cancel: _,
        done: _,
        then: _,
        _completed: _,
        _error: _,
        _notify: _,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    // Ready state, waiting for a message (completed/error/progress), able to be canceled
    //
    state_working = {
        name: "working",
        enter: _,
        cancel: function (promise) {
            promise._setState(state_canceled);
        },
        done: done,
        then: then,
        _completed: completed,
        _error: error,
        _notify: _,
        _progress: progress,
        _setCompleteValue: setCompleteValue,
        _setErrorValue: setErrorValue
    };

    // Waiting state, if a promise is completed with a value which is itself a promise
    // (has a then() method) it signs up to be informed when that child promise is
    // fulfilled at which point it will be fulfilled with that value.
    //
    state_waiting = {
        name: "waiting",
        enter: function (promise) {
            var waitedUpon = promise._value;
            var error = function (value) {
                if (waitedUpon._errorId) {
                    promise._chainedError(value, waitedUpon);
                } else {
                    // Because this is an interop boundary we want to indicate that this
                    //  error has been handled by the promise infrastructure before we
                    //  begin a new handling chain.
                    //
                    callonerror(promise, value, detailsForHandledError, waitedUpon, error);
                    promise._error(value);
                }
            };
            error.handlesOnError = true;
            waitedUpon.then(
                promise._completed.bind(promise),
                error,
                promise._progress.bind(promise)
            );
        },
        cancel: function (promise) {
            promise._setState(state_waiting_canceled);
        },
        done: done,
        then: then,
        _completed: completed,
        _error: error,
        _notify: _,
        _progress: progress,
        _setCompleteValue: setCompleteValue,
        _setErrorValue: setErrorValue
    };

    // Waiting canceled state, when a promise has been in a waiting state and receives a
    // request to cancel its pending work it will forward that request to the child promise
    // and then waits to be informed of the result. This promise moves itself into the
    // canceling state but understands that the child promise may instead push it to a
    // different state.
    //
    state_waiting_canceled = {
        name: "waiting_canceled",
        enter: function (promise) {
            // Initiate a transition to canceling. Triggering a cancel on the promise
            // that we are waiting upon may result in a different state transition
            // before the state machine pump runs again.
            promise._setState(state_canceling);
            var waitedUpon = promise._value;
            if (waitedUpon.cancel) {
                waitedUpon.cancel();
            }
        },
        cancel: _,
        done: done,
        then: then,
        _completed: completed,
        _error: error,
        _notify: _,
        _progress: progress,
        _setCompleteValue: setCompleteValue,
        _setErrorValue: setErrorValue
    };

    // Canceled state, moves to the canceling state and then tells the promise to do
    // whatever it might need to do on cancelation.
    //
    state_canceled = {
        name: "canceled",
        enter: function (promise) {
            // Initiate a transition to canceling. The _cancelAction may change the state
            // before the state machine pump runs again.
            promise._setState(state_canceling);
            promise._cancelAction();
        },
        cancel: _,
        done: done,
        then: then,
        _completed: completed,
        _error: error,
        _notify: _,
        _progress: progress,
        _setCompleteValue: setCompleteValue,
        _setErrorValue: setErrorValue
    };

    // Canceling state, commits to the promise moving to an error state with an error
    // object whose 'name' and 'message' properties contain the string "Canceled"
    //
    state_canceling = {
        name: "canceling",
        enter: function (promise) {
            var error = new Error(canceledName);
            error.name = error.message;
            promise._value = error;
            promise._setState(state_error_notify);
        },
        cancel: _,
        done: _,
        then: _,
        _completed: _,
        _error: _,
        _notify: _,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    // Success notify state, moves a promise to the success state and notifies all children
    //
    state_success_notify = {
        name: "complete_notify",
        enter: function (promise) {
            promise.done = CompletePromise.prototype.done;
            promise.then = CompletePromise.prototype.then;
            if (promise._listeners) {
                var queue = [promise];
                var p;
                while (queue.length) {
                    p = queue.pop();
                    p._state._notify(p, queue);
                }
            }
            promise._setState(state_success);
        },
        cancel: _,
        done: null, /*error to get here */
        then: null, /*error to get here */
        _completed: _,
        _error: _,
        _notify: notifySuccess,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    // Success state, moves a promise to the success state and does NOT notify any children.
    // Some upstream promise is owning the notification pass.
    //
    state_success = {
        name: "success",
        enter: function (promise) {
            promise.done = CompletePromise.prototype.done;
            promise.then = CompletePromise.prototype.then;
            promise._cleanupAction();
        },
        cancel: _,
        done: null, /*error to get here */
        then: null, /*error to get here */
        _completed: _,
        _error: _,
        _notify: notifySuccess,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    // Error notify state, moves a promise to the error state and notifies all children
    //
    state_error_notify = {
        name: "error_notify",
        enter: function (promise) {
            promise.done = ErrorPromise.prototype.done;
            promise.then = ErrorPromise.prototype.then;
            if (promise._listeners) {
                var queue = [promise];
                var p;
                while (queue.length) {
                    p = queue.pop();
                    p._state._notify(p, queue);
                }
            }
            promise._setState(state_error);
        },
        cancel: _,
        done: null, /*error to get here*/
        then: null, /*error to get here*/
        _completed: _,
        _error: _,
        _notify: notifyError,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    // Error state, moves a promise to the error state and does NOT notify any children.
    // Some upstream promise is owning the notification pass.
    //
    state_error = {
        name: "error",
        enter: function (promise) {
            promise.done = ErrorPromise.prototype.done;
            promise.then = ErrorPromise.prototype.then;
            promise._cleanupAction();
        },
        cancel: _,
        done: null, /*error to get here*/
        then: null, /*error to get here*/
        _completed: _,
        _error: _,
        _notify: notifyError,
        _progress: _,
        _setCompleteValue: _,
        _setErrorValue: _
    };

    //
    // The statemachine implementation follows a very particular pattern, the states are specified
    // as static stateless bags of functions which are then indirected through the state machine
    // instance (a Promise). As such all of the functions on each state have the promise instance
    // passed to them explicitly as a parameter and the Promise instance members do a little
    // dance where they indirect through the state and insert themselves in the argument list.
    //
    // We could instead call directly through the promise states however then every caller
    // would have to remember to do things like pumping the state machine to catch state transitions.
    //

    var PromiseStateMachine = WinJS.Class.define(null, {
        _listeners: null,
        _nextState: null,
        _state: null,
        _value: null,

        cancel: function () {
            /// <signature helpKeyword="WinJS.PromiseStateMachine.cancel">
            /// <summary locid="WinJS.PromiseStateMachine.cancel">
            /// Attempts to cancel the fulfillment of a promised value. If the promise hasn't
            /// already been fulfilled and cancellation is supported, the promise enters
            /// the error state with a value of Error("Canceled").
            /// </summary>
            /// </signature>
            this._state.cancel(this);
            this._run();
        },
        done: function Promise_done(onComplete, onError, onProgress) {
            /// <signature helpKeyword="WinJS.PromiseStateMachine.done">
            /// <summary locid="WinJS.PromiseStateMachine.done">
            /// Allows you to specify the work to be done on the fulfillment of the promised value,
            /// the error handling to be performed if the promise fails to fulfill
            /// a value, and the handling of progress notifications along the way.
            ///
            /// After the handlers have finished executing, this function throws any error that would have been returned
            /// from then() as a promise in the error state.
            /// </summary>
            /// <param name="onComplete" type="Function" locid="WinJS.PromiseStateMachine.done_p:onComplete">
            /// The function to be called if the promise is fulfilled successfully with a value.
            /// The fulfilled value is passed as the single argument. If the value is null,
            /// the fulfilled value is returned. The value returned
            /// from the function becomes the fulfilled value of the promise returned by
            /// then(). If an exception is thrown while executing the function, the promise returned
            /// by then() moves into the error state.
            /// </param>
            /// <param name="onError" type="Function" optional="true" locid="WinJS.PromiseStateMachine.done_p:onError">
            /// The function to be called if the promise is fulfilled with an error. The error
            /// is passed as the single argument. If it is null, the error is forwarded.
            /// The value returned from the function is the fulfilled value of the promise returned by then().
            /// </param>
            /// <param name="onProgress" type="Function" optional="true" locid="WinJS.PromiseStateMachine.done_p:onProgress">
            /// the function to be called if the promise reports progress. Data about the progress
            /// is passed as the single argument. Promises are not required to support
            /// progress.
            /// </param>
            /// </signature>
            this._state.done(this, onComplete, onError, onProgress);
        },
        then: function Promise_then(onComplete, onError, onProgress) {
            /// <signature helpKeyword="WinJS.PromiseStateMachine.then">
            /// <summary locid="WinJS.PromiseStateMachine.then">
            /// Allows you to specify the work to be done on the fulfillment of the promised value,
            /// the error handling to be performed if the promise fails to fulfill
            /// a value, and the handling of progress notifications along the way.
            /// </summary>
            /// <param name="onComplete" type="Function" locid="WinJS.PromiseStateMachine.then_p:onComplete">
            /// The function to be called if the promise is fulfilled successfully with a value.
            /// The value is passed as the single argument. If the value is null, the value is returned.
            /// The value returned from the function becomes the fulfilled value of the promise returned by
            /// then(). If an exception is thrown while this function is being executed, the promise returned
            /// by then() moves into the error state.
            /// </param>
            /// <param name="onError" type="Function" optional="true" locid="WinJS.PromiseStateMachine.then_p:onError">
            /// The function to be called if the promise is fulfilled with an error. The error
            /// is passed as the single argument. If it is null, the error is forwarded.
            /// The value returned from the function becomes the fulfilled value of the promise returned by then().
            /// </param>
            /// <param name="onProgress" type="Function" optional="true" locid="WinJS.PromiseStateMachine.then_p:onProgress">
            /// The function to be called if the promise reports progress. Data about the progress
            /// is passed as the single argument. Promises are not required to support
            /// progress.
            /// </param>
            /// <returns type="WinJS.Promise" locid="WinJS.PromiseStateMachine.then_returnValue">
            /// The promise whose value is the result of executing the complete or
            /// error function.
            /// </returns>
            /// </signature>
            return this._state.then(this, onComplete, onError, onProgress);
        },

        _chainedError: function (value, context) {
            var result = this._state._error(this, value, detailsForChainedError, context);
            this._run();
            return result;
        },
        _completed: function (value) {
            var result = this._state._completed(this, value);
            this._run();
            return result;
        },
        _error: function (value) {
            var result = this._state._error(this, value, detailsForError);
            this._run();
            return result;
        },
        _progress: function (value) {
            this._state._progress(this, value);
        },
        _setState: function (state) {
            this._nextState = state;
        },
        _setCompleteValue: function (value) {
            this._state._setCompleteValue(this, value);
            this._run();
        },
        _setChainedErrorValue: function (value, context) {
            var result = this._state._setErrorValue(this, value, detailsForChainedError, context);
            this._run();
            return result;
        },
        _setExceptionValue: function (value) {
            var result = this._state._setErrorValue(this, value, detailsForException);
            this._run();
            return result;
        },
        _run: function () {
            while (this._nextState) {
                this._state = this._nextState;
                this._nextState = null;
                this._state.enter(this);
            }
        }
    }, {
        supportedForProcessing: false
    });

    //
    // Implementations of shared state machine code.
    //

    function completed(promise, value) {
        var targetState;
        if (value && typeof value === "object" && typeof value.then === "function") {
            targetState = state_waiting;
        } else {
            targetState = state_success_notify;
        }
        promise._value = value;
        promise._setState(targetState);
    }
    function createErrorDetails(exception, error, promise, id, parent, handler) {
        return {
            exception: exception,
            error: error,
            promise: promise,
            handler: handler,
            id: id,
            parent: parent
        };
    }
    function detailsForHandledError(promise, errorValue, context, handler) {
        var exception = context._isException;
        var errorId = context._errorId;
        return createErrorDetails(
            exception ? errorValue : null,
            exception ? null : errorValue,
            promise,
            errorId,
            context,
            handler
        );
    }
    function detailsForChainedError(promise, errorValue, context) {
        var exception = context._isException;
        var errorId = context._errorId;
        setErrorInfo(promise, errorId, exception);
        return createErrorDetails(
            exception ? errorValue : null,
            exception ? null : errorValue,
            promise,
            errorId,
            context
        );
    }
    function detailsForError(promise, errorValue) {
        var errorId = ++error_number;
        setErrorInfo(promise, errorId);
        return createErrorDetails(
            null,
            errorValue,
            promise,
            errorId
        );
    }
    function detailsForException(promise, exceptionValue) {
        var errorId = ++error_number;
        setErrorInfo(promise, errorId, true);
        return createErrorDetails(
            exceptionValue,
            null,
            promise,
            errorId
        );
    }
    function done(promise, onComplete, onError, onProgress) {
        pushListener(promise, { c: onComplete, e: onError, p: onProgress });
    }
    function error(promise, value, onerrorDetails, context) {
        promise._value = value;
        callonerror(promise, value, onerrorDetails, context);
        promise._setState(state_error_notify);
    }
    function notifySuccess(promise, queue) {
        var value = promise._value;
        var listeners = promise._listeners;
        if (!listeners) {
            return;
        }
        promise._listeners = null;
        var i, len;
        for (i = 0, len = Array.isArray(listeners) ? listeners.length : 1; i < len; i++) {
            var listener = len === 1 ? listeners : listeners[i];
            var onComplete = listener.c;
            var target = listener.promise;
            if (target) {
                try {
                    target._setCompleteValue(onComplete ? onComplete(value) : value);
                } catch (ex) {
                    target._setExceptionValue(ex);
                }
                if (target._state !== state_waiting && target._listeners) {
                    queue.push(target);
                }
            } else {
                CompletePromise.prototype.done.call(promise, onComplete);
            }
        }
    }
    function notifyError(promise, queue) {
        var value = promise._value;
        var listeners = promise._listeners;
        if (!listeners) {
            return;
        }
        promise._listeners = null;
        var i, len;
        for (i = 0, len = Array.isArray(listeners) ? listeners.length : 1; i < len; i++) {
            var listener = len === 1 ? listeners : listeners[i];
            var onError = listener.e;
            var target = listener.promise;
            if (target) {
                try {
                    if (onError) {
                        if (!onError.handlesOnError) {
                            callonerror(target, value, detailsForHandledError, promise, onError);
                        }
                        target._setCompleteValue(onError(value))
                    } else {
                        target._setChainedErrorValue(value, promise);
                    }
                } catch (ex) {
                    target._setExceptionValue(ex);
                }
                if (target._state !== state_waiting && target._listeners) {
                    queue.push(target);
                }
            } else {
                ErrorPromise.prototype.done.call(promise, null, onError);
            }
        }
    }
    function callonerror(promise, value, onerrorDetailsGenerator, context, handler) {
        if (promiseEventListeners._listeners[errorET]) {
            if (value instanceof Error && value.message === canceledName) {
                return;
            }
            promiseEventListeners.dispatchEvent(errorET, onerrorDetailsGenerator(promise, value, context, handler));
        }
    }
    function progress(promise, value) {
        var listeners = promise._listeners;
        if (listeners) {
            var i, len;
            for (i = 0, len = Array.isArray(listeners) ? listeners.length : 1; i < len; i++) {
                var listener = len === 1 ? listeners : listeners[i];
                var onProgress = listener.p;
                if (onProgress) {
                    try { onProgress(value); } catch (ex) { }
                }
                if (!(listener.c || listener.e) && listener.promise) {
                    listener.promise._progress(value);
                }
            }
        }
    }
    function pushListener(promise, listener) {
        var listeners = promise._listeners;
        if (listeners) {
            // We may have either a single listener (which will never be wrapped in an array)
            // or 2+ listeners (which will be wrapped). Since we are now adding one more listener
            // we may have to wrap the single listener before adding the second.
            listeners = Array.isArray(listeners) ? listeners : [listeners];
            listeners.push(listener);
        } else {
            listeners = listener;
        }
        promise._listeners = listeners;
    }
    // The difference beween setCompleteValue()/setErrorValue() and complete()/error() is that setXXXValue() moves
    // a promise directly to the success/error state without starting another notification pass (because one
    // is already ongoing).
    function setErrorInfo(promise, errorId, isException) {
        promise._isException = isException || false;
        promise._errorId = errorId;
    }
    function setErrorValue(promise, value, onerrorDetails, context) {
        promise._value = value;
        callonerror(promise, value, onerrorDetails, context);
        promise._setState(state_error);
    }
    function setCompleteValue(promise, value) {
        var targetState;
        if (value && typeof value === "object" && typeof value.then === "function") {
            targetState = state_waiting;
        } else {
            targetState = state_success;
        }
        promise._value = value;
        promise._setState(targetState);
    }
    function then(promise, onComplete, onError, onProgress) {
        var result = new ThenPromise(promise);
        pushListener(promise, { promise: result, c: onComplete, e: onError, p: onProgress });
        return result;
    }

    //
    // Internal implementation detail promise, ThenPromise is created when a promise needs
    // to be returned from a then() method.
    //
    var ThenPromise = WinJS.Class.derive(PromiseStateMachine,
        function (creator) {

            if (tagWithStack && (tagWithStack === true || (tagWithStack & tag.thenPromise))) {
                this._stack = WinJS.Promise._getStack();
            }

            this._creator = creator;
            this._setState(state_created);
            this._run();
        }, {
            _creator: null,

            _cancelAction: function () { if (this._creator) { this._creator.cancel(); } },
            _cleanupAction: function () { this._creator = null; }
        }, {
            supportedForProcessing: false
        }
    );

    //
    // Slim promise implementations for already completed promises, these are created
    // under the hood on synchronous completion paths as well as by WinJS.Promise.wrap
    // and WinJS.Promise.wrapError.
    //

    var ErrorPromise = WinJS.Class.define(
        function ErrorPromise_ctor(value) {

            if (tagWithStack && (tagWithStack === true || (tagWithStack & tag.errorPromise))) {
                this._stack = WinJS.Promise._getStack();
            }

            this._value = value;
            callonerror(this, value, detailsForError);
        }, {
            cancel: function () {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.cancel">
                /// <summary locid="WinJS.PromiseStateMachine.cancel">
                /// Attempts to cancel the fulfillment of a promised value. If the promise hasn't
                /// already been fulfilled and cancellation is supported, the promise enters
                /// the error state with a value of Error("Canceled").
                /// </summary>
                /// </signature>
            },
            done: function ErrorPromise_done(unused, onError) {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.done">
                /// <summary locid="WinJS.PromiseStateMachine.done">
                /// Allows you to specify the work to be done on the fulfillment of the promised value,
                /// the error handling to be performed if the promise fails to fulfill
                /// a value, and the handling of progress notifications along the way.
                ///
                /// After the handlers have finished executing, this function throws any error that would have been returned
                /// from then() as a promise in the error state.
                /// </summary>
                /// <param name='onComplete' type='Function' locid="WinJS.PromiseStateMachine.done_p:onComplete">
                /// The function to be called if the promise is fulfilled successfully with a value.
                /// The fulfilled value is passed as the single argument. If the value is null,
                /// the fulfilled value is returned. The value returned
                /// from the function becomes the fulfilled value of the promise returned by
                /// then(). If an exception is thrown while executing the function, the promise returned
                /// by then() moves into the error state.
                /// </param>
                /// <param name='onError' type='Function' optional='true' locid="WinJS.PromiseStateMachine.done_p:onError">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument. If it is null, the error is forwarded.
                /// The value returned from the function is the fulfilled value of the promise returned by then().
                /// </param>
                /// <param name='onProgress' type='Function' optional='true' locid="WinJS.PromiseStateMachine.done_p:onProgress">
                /// the function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// </signature>
                var value = this._value;
                if (onError) {
                    try {
                        if (!onError.handlesOnError) {
                            callonerror(null, value, detailsForHandledError, this, onError);
                        }
                        var result = onError(value);
                        if (result && typeof result === "object" && typeof result.done === "function") {
                            // If a promise is returned we need to wait on it.
                            result.done();
                        }
                        return;
                    } catch (ex) {
                        value = ex;
                    }
                }
                if (value instanceof Error && value.message === canceledName) {
                    // suppress cancel
                    return;
                }
                // force the exception to be thrown asyncronously to avoid any try/catch blocks
                //
                setImmediate(function () {
                    throw value;
                });
            },
            then: function ErrorPromise_then(unused, onError) {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.then">
                /// <summary locid="WinJS.PromiseStateMachine.then">
                /// Allows you to specify the work to be done on the fulfillment of the promised value,
                /// the error handling to be performed if the promise fails to fulfill
                /// a value, and the handling of progress notifications along the way.
                /// </summary>
                /// <param name='onComplete' type='Function' locid="WinJS.PromiseStateMachine.then_p:onComplete">
                /// The function to be called if the promise is fulfilled successfully with a value.
                /// The value is passed as the single argument. If the value is null, the value is returned.
                /// The value returned from the function becomes the fulfilled value of the promise returned by
                /// then(). If an exception is thrown while this function is being executed, the promise returned
                /// by then() moves into the error state.
                /// </param>
                /// <param name='onError' type='Function' optional='true' locid="WinJS.PromiseStateMachine.then_p:onError">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument. If it is null, the error is forwarded.
                /// The value returned from the function becomes the fulfilled value of the promise returned by then().
                /// </param>
                /// <param name='onProgress' type='Function' optional='true' locid="WinJS.PromiseStateMachine.then_p:onProgress">
                /// The function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.PromiseStateMachine.then_returnValue">
                /// The promise whose value is the result of executing the complete or
                /// error function.
                /// </returns>
                /// </signature>

                // If the promise is already in a error state and no error handler is provided
                // we optimize by simply returning the promise instead of creating a new one.
                //
                if (!onError) { return this; }
                var result;
                var value = this._value;
                try {
                    if (!onError.handlesOnError) {
                        callonerror(null, value, detailsForHandledError, this, onError);
                    }
                    result = new CompletePromise(onError(value));
                } catch (ex) {
                    // If the value throw from the error handler is the same as the value
                    // provided to the error handler then there is no need for a new promise.
                    //
                    if (ex === value) {
                        result = this;
                    } else {
                        result = new ExceptionPromise(ex);
                    }
                }
                return result;
            }
        }, {
            supportedForProcessing: false
        }
    );

    var ExceptionPromise = WinJS.Class.derive(ErrorPromise,
        function ExceptionPromise_ctor(value) {

            if (tagWithStack && (tagWithStack === true || (tagWithStack & tag.exceptionPromise))) {
                this._stack = WinJS.Promise._getStack();
            }

            this._value = value;
            callonerror(this, value, detailsForException);
        }, {
            /* empty */
        }, {
            supportedForProcessing: false
        }
    );

    var CompletePromise = WinJS.Class.define(
        function CompletePromise_ctor(value) {

            if (tagWithStack && (tagWithStack === true || (tagWithStack & tag.completePromise))) {
                this._stack = WinJS.Promise._getStack();
            }

            if (value && typeof value === "object" && typeof value.then === "function") {
                var result = new ThenPromise(null);
                result._setCompleteValue(value);
                return result;
            }
            this._value = value;
        }, {
            cancel: function () {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.cancel">
                /// <summary locid="WinJS.PromiseStateMachine.cancel">
                /// Attempts to cancel the fulfillment of a promised value. If the promise hasn't
                /// already been fulfilled and cancellation is supported, the promise enters
                /// the error state with a value of Error("Canceled").
                /// </summary>
                /// </signature>
            },
            done: function CompletePromise_done(onComplete) {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.done">
                /// <summary locid="WinJS.PromiseStateMachine.done">
                /// Allows you to specify the work to be done on the fulfillment of the promised value,
                /// the error handling to be performed if the promise fails to fulfill
                /// a value, and the handling of progress notifications along the way.
                ///
                /// After the handlers have finished executing, this function throws any error that would have been returned
                /// from then() as a promise in the error state.
                /// </summary>
                /// <param name='onComplete' type='Function' locid="WinJS.PromiseStateMachine.done_p:onComplete">
                /// The function to be called if the promise is fulfilled successfully with a value.
                /// The fulfilled value is passed as the single argument. If the value is null,
                /// the fulfilled value is returned. The value returned
                /// from the function becomes the fulfilled value of the promise returned by
                /// then(). If an exception is thrown while executing the function, the promise returned
                /// by then() moves into the error state.
                /// </param>
                /// <param name='onError' type='Function' optional='true' locid="WinJS.PromiseStateMachine.done_p:onError">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument. If it is null, the error is forwarded.
                /// The value returned from the function is the fulfilled value of the promise returned by then().
                /// </param>
                /// <param name='onProgress' type='Function' optional='true' locid="WinJS.PromiseStateMachine.done_p:onProgress">
                /// the function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// </signature>
                if (!onComplete) { return; }
                try {
                    var result = onComplete(this._value);
                    if (result && typeof result === "object" && typeof result.done === "function") {
                        result.done();
                    }
                } catch (ex) {
                    // force the exception to be thrown asynchronously to avoid any try/catch blocks
                    setImmediate(function () {
                        throw ex;
                    });
                }
            },
            then: function CompletePromise_then(onComplete) {
                /// <signature helpKeyword="WinJS.PromiseStateMachine.then">
                /// <summary locid="WinJS.PromiseStateMachine.then">
                /// Allows you to specify the work to be done on the fulfillment of the promised value,
                /// the error handling to be performed if the promise fails to fulfill
                /// a value, and the handling of progress notifications along the way.
                /// </summary>
                /// <param name='onComplete' type='Function' locid="WinJS.PromiseStateMachine.then_p:onComplete">
                /// The function to be called if the promise is fulfilled successfully with a value.
                /// The value is passed as the single argument. If the value is null, the value is returned.
                /// The value returned from the function becomes the fulfilled value of the promise returned by
                /// then(). If an exception is thrown while this function is being executed, the promise returned
                /// by then() moves into the error state.
                /// </param>
                /// <param name='onError' type='Function' optional='true' locid="WinJS.PromiseStateMachine.then_p:onError">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument. If it is null, the error is forwarded.
                /// The value returned from the function becomes the fulfilled value of the promise returned by then().
                /// </param>
                /// <param name='onProgress' type='Function' optional='true' locid="WinJS.PromiseStateMachine.then_p:onProgress">
                /// The function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.PromiseStateMachine.then_returnValue">
                /// The promise whose value is the result of executing the complete or
                /// error function.
                /// </returns>
                /// </signature>
                try {
                    // If the value returned from the completion handler is the same as the value
                    // provided to the completion handler then there is no need for a new promise.
                    //
                    var newValue = onComplete ? onComplete(this._value) : this._value;
                    return newValue === this._value ? this : new CompletePromise(newValue);
                } catch (ex) {
                    return new ExceptionPromise(ex);
                }
            }
        }, {
            supportedForProcessing: false
        }
    );

    //
    // Promise is the user-creatable WinJS.Promise object.
    //

    function timeout(timeoutMS) {
        var id;
        return new WinJS.Promise(
            function (c) {
                if (timeoutMS) {
                    id = setTimeout(c, timeoutMS);
                } else {
                    setImmediate(c);
                }
            },
            function () {
                if (id) {
                    clearTimeout(id);
                }
            }
        );
    }

    function timeoutWithPromise(timeout, promise) {
        var cancelPromise = function () { promise.cancel(); }
        var cancelTimeout = function () { timeout.cancel(); }
        timeout.then(cancelPromise);
        promise.then(cancelTimeout, cancelTimeout);
        return promise;
    }

    var staticCanceledPromise;

    var Promise = WinJS.Class.derive(PromiseStateMachine,
        function Promise_ctor(init, oncancel) {
            /// <signature helpKeyword="WinJS.Promise">
            /// <summary locid="WinJS.Promise">
            /// A promise provides a mechanism to schedule work to be done on a value that
            /// has not yet been computed. It is a convenient abstraction for managing
            /// interactions with asynchronous APIs.
            /// </summary>
            /// <param name="init" type="Function" locid="WinJS.Promise_p:init">
            /// The function that is called during construction of the  promise. The function
            /// is given three arguments (complete, error, progress). Inside this function
            /// you should add event listeners for the notifications supported by this value.
            /// </param>
            /// <param name="oncancel" optional="true" locid="WinJS.Promise_p:oncancel">
            /// The function to call if a consumer of this promise wants
            /// to cancel its undone work. Promises are not required to
            /// support cancellation.
            /// </param>
            /// </signature>

            if (tagWithStack && (tagWithStack === true || (tagWithStack & tag.promise))) {
                this._stack = WinJS.Promise._getStack();
            }

            this._oncancel = oncancel;
            this._setState(state_created);
            this._run();

            try {
                var complete = this._completed.bind(this);
                var error = this._error.bind(this);
                var progress = this._progress.bind(this);
                init(complete, error, progress);
            } catch (ex) {
                this._setExceptionValue(ex);
            }
        }, {
            _oncancel: null,

            _cancelAction: function () {
				try {
            		if (this._oncancel) {
						this._oncancel();
					} else {
						throw new Error('Promise did not implement oncancel');
					}
				} catch (ex) {
					// Access fields to get them created
					var msg = ex.message;
					var stack = ex.stack;
					promiseEventListeners.dispatchEvent('error', ex);
				}
            },
            _cleanupAction: function () { this._oncancel = null; }
        }, {

            addEventListener: function Promise_addEventListener(eventType, listener, capture) {
                /// <signature helpKeyword="WinJS.Promise.addEventListener">
                /// <summary locid="WinJS.Promise.addEventListener">
                /// Adds an event listener to the control.
                /// </summary>
                /// <param name="eventType" locid="WinJS.Promise.addEventListener_p:eventType">
                /// The type (name) of the event.
                /// </param>
                /// <param name="listener" locid="WinJS.Promise.addEventListener_p:listener">
                /// The listener to invoke when the event is raised.
                /// </param>
                /// <param name="capture" locid="WinJS.Promise.addEventListener_p:capture">
                /// Specifies whether or not to initiate capture.
                /// </param>
                /// </signature>
                promiseEventListeners.addEventListener(eventType, listener, capture);
            },
            any: function Promise_any(values) {
                /// <signature helpKeyword="WinJS.Promise.any">
                /// <summary locid="WinJS.Promise.any">
                /// Returns a promise that is fulfilled when one of the input promises
                /// has been fulfilled.
                /// </summary>
                /// <param name="values" type="Array" locid="WinJS.Promise.any_p:values">
                /// An array that contains promise objects or objects whose property
                /// values include promise objects.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.any_returnValue">
                /// A promise that on fulfillment yields the value of the input (complete or error).
                /// </returns>
                /// </signature>
                return new Promise(
                    function (complete, error, progress) {
                        var keys = Object.keys(values);
                        var errors = Array.isArray(values) ? [] : {};
                        if (keys.length === 0) {
                            complete();
                        }
                        var canceled = 0;
                        keys.forEach(function (key) {
                            Promise.as(values[key]).then(
                                function () { complete({ key: key, value: values[key] }); },
                                function (e) {
                                    if (e instanceof Error && e.name === canceledName) {
                                        if ((++canceled) === keys.length) {
                                            complete(WinJS.Promise.cancel);
                                        }
                                        return;
                                    }
                                    error({ key: key, value: values[key] });
                                }
                            );
                        });
                    },
                    function () {
                        var keys = Object.keys(values);
                        keys.forEach(function (key) {
                            var promise = Promise.as(values[key]);
                            if (typeof promise.cancel === "function") {
                                promise.cancel();
                            }
                        });
                    }
                );
            },
            as: function Promise_as(value) {
                /// <signature helpKeyword="WinJS.Promise.as">
                /// <summary locid="WinJS.Promise.as">
                /// Returns a promise. If the object is already a promise it is returned;
                /// otherwise the object is wrapped in a promise.
                /// </summary>
                /// <param name="value" locid="WinJS.Promise.as_p:value">
                /// The value to be treated as a promise.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.as_returnValue">
                /// A promise.
                /// </returns>
                /// </signature>
                if (value && typeof value === "object" && typeof value.then === "function") {
                    return value;
                }
                return new CompletePromise(value);
            },
            /// <field type="WinJS.Promise" helpKeyword="WinJS.Promise.cancel" locid="WinJS.Promise.cancel">
            /// Canceled promise value, can be returned from a promise completion handler
            /// to indicate cancelation of the promise chain.
            /// </field>
            cancel: {
                get: function () {
                    return (staticCanceledPromise = staticCanceledPromise || new ErrorPromise(new WinJS.ErrorFromName(canceledName)));
                }
            },
            dispatchEvent: function Promise_dispatchEvent(eventType, details) {
                /// <signature helpKeyword="WinJS.Promise.dispatchEvent">
                /// <summary locid="WinJS.Promise.dispatchEvent">
                /// Raises an event of the specified type and properties.
                /// </summary>
                /// <param name="eventType" locid="WinJS.Promise.dispatchEvent_p:eventType">
                /// The type (name) of the event.
                /// </param>
                /// <param name="details" locid="WinJS.Promise.dispatchEvent_p:details">
                /// The set of additional properties to be attached to the event object.
                /// </param>
                /// <returns type="Boolean" locid="WinJS.Promise.dispatchEvent_returnValue">
                /// Specifies whether preventDefault was called on the event.
                /// </returns>
                /// </signature>
                return promiseEventListeners.dispatchEvent(eventType, details);
            },
            is: function Promise_is(value) {
                /// <signature helpKeyword="WinJS.Promise.is">
                /// <summary locid="WinJS.Promise.is">
                /// Determines whether a value fulfills the promise contract.
                /// </summary>
                /// <param name="value" locid="WinJS.Promise.is_p:value">
                /// A value that may be a promise.
                /// </param>
                /// <returns type="Boolean" locid="WinJS.Promise.is_returnValue">
                /// true if the specified value is a promise, otherwise false.
                /// </returns>
                /// </signature>
                return value && typeof value === "object" && typeof value.then === "function";
            },
            join: function Promise_join(values) {
                /// <signature helpKeyword="WinJS.Promise.join">
                /// <summary locid="WinJS.Promise.join">
                /// Creates a promise that is fulfilled when all the values are fulfilled.
                /// </summary>
                /// <param name="values" type="Object" locid="WinJS.Promise.join_p:values">
                /// An object whose fields contain values, some of which may be promises.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.join_returnValue">
                /// A promise whose value is an object with the same field names as those of the object in the values parameter, where
                /// each field value is the fulfilled value of a promise.
                /// </returns>
                /// </signature>
                return new Promise(
                    function (complete, error, progress) {
                        var keys = Object.keys(values);
                        var errors = Array.isArray(values) ? [] : {};
                        var results = Array.isArray(values) ? [] : {};
                        var undefineds = 0;
                        var pending = keys.length;
                        var argDone = function (key) {
                            if ((--pending) === 0) {
                                var errorCount = Object.keys(errors).length;
                                if (errorCount === 0) {
                                    complete(results);
                                } else {
                                    var canceledCount = 0;
                                    keys.forEach(function (key) {
                                        var e = errors[key];
                                        if (e instanceof Error && e.name === canceledName) {
                                            canceledCount++;
                                        }
                                    });
                                    if (canceledCount === errorCount) {
                                        complete(WinJS.Promise.cancel);
                                    } else {
                                        error(errors);
                                    }
                                }
                            } else {
                                progress({ Key: key, Done: true });
                            }
                        };
                        keys.forEach(function (key) {
                            var value = values[key];
                            if (value === undefined) {
                                undefineds++;
                            } else {
                                Promise.then(value,
                                    function (value) { results[key] = value; argDone(key); },
                                    function (value) { errors[key] = value; argDone(key); }
                                );
                            }
                        });
                        pending -= undefineds;
                        if (pending === 0) {
                            complete(results);
                            return;
                        }
                    },
                    function () {
                        Object.keys(values).forEach(function (key) {
                            var promise = Promise.as(values[key]);
                            if (typeof promise.cancel === "function") {
                                promise.cancel();
                            }
                        });
                    }
                );
            },
            removeEventListener: function Promise_removeEventListener(eventType, listener, capture) {
                /// <signature helpKeyword="WinJS.Promise.removeEventListener">
                /// <summary locid="WinJS.Promise.removeEventListener">
                /// Removes an event listener from the control.
                /// </summary>
                /// <param name='eventType' locid="WinJS.Promise.removeEventListener_eventType">
                /// The type (name) of the event.
                /// </param>
                /// <param name='listener' locid="WinJS.Promise.removeEventListener_listener">
                /// The listener to remove.
                /// </param>
                /// <param name='capture' locid="WinJS.Promise.removeEventListener_capture">
                /// Specifies whether or not to initiate capture.
                /// </param>
                /// </signature>
                promiseEventListeners.removeEventListener(eventType, listener, capture);
            },
            supportedForProcessing: false,
            then: function Promise_then(value, onComplete, onError, onProgress) {
                /// <signature helpKeyword="WinJS.Promise.then">
                /// <summary locid="WinJS.Promise.then">
                /// A static version of the promise instance method then().
                /// </summary>
                /// <param name="value" locid="WinJS.Promise.then_p:value">
                /// the value to be treated as a promise.
                /// </param>
                /// <param name="onComplete" type="Function" locid="WinJS.Promise.then_p:complete">
                /// The function to be called if the promise is fulfilled with a value.
                /// If it is null, the promise simply
                /// returns the value. The value is passed as the single argument.
                /// </param>
                /// <param name="onError" type="Function" optional="true" locid="WinJS.Promise.then_p:error">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument.
                /// </param>
                /// <param name="onProgress" type="Function" optional="true" locid="WinJS.Promise.then_p:progress">
                /// The function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.then_returnValue">
                /// A promise whose value is the result of executing the provided complete function.
                /// </returns>
                /// </signature>
                return Promise.as(value).then(onComplete, onError, onProgress);
            },
            thenEach: function Promise_thenEach(values, onComplete, onError, onProgress) {
                /// <signature helpKeyword="WinJS.Promise.thenEach">
                /// <summary locid="WinJS.Promise.thenEach">
                /// Performs an operation on all the input promises and returns a promise
                /// that has the shape of the input and contains the result of the operation
                /// that has been performed on each input.
                /// </summary>
                /// <param name="values" locid="WinJS.Promise.thenEach_p:values">
                /// A set of values (which could be either an array or an object) of which some or all are promises.
                /// </param>
                /// <param name="onComplete" type="Function" locid="WinJS.Promise.thenEach_p:complete">
                /// The function to be called if the promise is fulfilled with a value.
                /// If the value is null, the promise returns the value.
                /// The value is passed as the single argument.
                /// </param>
                /// <param name="onError" type="Function" optional="true" locid="WinJS.Promise.thenEach_p:error">
                /// The function to be called if the promise is fulfilled with an error. The error
                /// is passed as the single argument.
                /// </param>
                /// <param name="onProgress" type="Function" optional="true" locid="WinJS.Promise.thenEach_p:progress">
                /// The function to be called if the promise reports progress. Data about the progress
                /// is passed as the single argument. Promises are not required to support
                /// progress.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.thenEach_returnValue">
                /// A promise that is the result of calling Promise.join on the values parameter.
                /// </returns>
                /// </signature>
                var result = Array.isArray(values) ? [] : {};
                Object.keys(values).forEach(function (key) {
                    result[key] = Promise.as(values[key]).then(onComplete, onError, onProgress);
                });
                return Promise.join(result);
            },
            timeout: function Promise_timeout(time, promise) {
                /// <signature helpKeyword="WinJS.Promise.timeout">
                /// <summary locid="WinJS.Promise.timeout">
                /// Creates a promise that is fulfilled after a timeout.
                /// </summary>
                /// <param name="timeout" type="Number" optional="true" locid="WinJS.Promise.timeout_p:timeout">
                /// The timeout period in milliseconds. If this value is zero or not specified
                /// setImmediate is called, otherwise setTimeout is called.
                /// </param>
                /// <param name="promise" type="Promise" optional="true" locid="WinJS.Promise.timeout_p:promise">
                /// A promise that will be canceled if it doesn't complete before the
                /// timeout has expired.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.timeout_returnValue">
                /// A promise that is completed asynchronously after the specified timeout.
                /// </returns>
                /// </signature>
                var to = timeout(time);
                return promise ? timeoutWithPromise(to, promise) : to;
            },
            wrap: function Promise_wrap(value) {
                /// <signature helpKeyword="WinJS.Promise.wrap">
                /// <summary locid="WinJS.Promise.wrap">
                /// Wraps a non-promise value in a promise. You can use this function if you need
                /// to pass a value to a function that requires a promise.
                /// </summary>
                /// <param name="value" locid="WinJS.Promise.wrap_p:value">
                /// Some non-promise value to be wrapped in a promise.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.wrap_returnValue">
                /// A promise that is successfully fulfilled with the specified value
                /// </returns>
                /// </signature>
                return new CompletePromise(value);
            },
            wrapError: function Promise_wrapError(error) {
                /// <signature helpKeyword="WinJS.Promise.wrapError">
                /// <summary locid="WinJS.Promise.wrapError">
                /// Wraps a non-promise error value in a promise. You can use this function if you need
                /// to pass an error to a function that requires a promise.
                /// </summary>
                /// <param name="error" locid="WinJS.Promise.wrapError_p:error">
                /// A non-promise error value to be wrapped in a promise.
                /// </param>
                /// <returns type="WinJS.Promise" locid="WinJS.Promise.wrapError_returnValue">
                /// A promise that is in an error state with the specified value.
                /// </returns>
                /// </signature>
                return new ErrorPromise(error);
            },

            _veryExpensiveTagWithStack: {
                get: function () { return tagWithStack; },
                set: function (value) { tagWithStack = value; }
            },
            _veryExpensiveTagWithStack_tag: tag,
            _getStack: function () {
                if (Debug.debuggerEnabled) {
                    try { throw new Error(); } catch (e) { return e.stack; }
                }
            },

        }
    );
    Object.defineProperties(Promise, WinJS.Utilities.createEventProperties(errorET));

    var SignalPromise = WinJS.Class.derive(PromiseStateMachine,
        function (cancel) {
            this._oncancel = cancel;
            this._setState(state_created);
            this._run();
        }, {
            _cancelAction: function () { this._oncancel && this._oncancel(); },
            _cleanupAction: function () { this._oncancel = null; }
        }, {
            supportedForProcessing: false
        }
    );

    var Signal = WinJS.Class.define(
        function Signal_ctor(oncancel) {
            this._promise = new SignalPromise(oncancel);
        }, {
            promise: {
                get: function () { return this._promise; }
            },

            cancel: function Signal_cancel() {
                this._promise.cancel();
            },
            complete: function Signal_complete(value) {
                this._promise._completed(value);
            },
            error: function Signal_error(value) {
                this._promise._error(value);
            },
            progress: function Signal_progress(value) {
                this._promise._progress(value);
            }
        }, {
            supportedForProcessing: false,
        }
    );

    // Publish WinJS.Promise
    //
    WinJS.Namespace.define("WinJS", {
        Promise: Promise,
        _Signal: Signal
    });

}(this, this.WinJS));

(function errorsInit(global, WinJS) {
    "use strict";


    WinJS.Namespace.define("WinJS", {
        // ErrorFromName establishes a simple pattern for returning error codes.
        //
        ErrorFromName: WinJS.Class.derive(Error, function (name, message) {
            /// <signature helpKeyword="WinJS.ErrorFromName">
            /// <summary locid="WinJS.ErrorFromName">
            /// Creates an Error object with the specified name and message properties.
            /// </summary>
            /// <param name="name" type="String" locid="WinJS.ErrorFromName_p:name">The name of this error. The name is meant to be consumed programmatically and should not be localized.</param>
            /// <param name="message" type="String" optional="true" locid="WinJS.ErrorFromName_p:message">The message for this error. The message is meant to be consumed by humans and should be localized.</param>
            /// <returns type="Error" locid="WinJS.ErrorFromName_returnValue">Error instance with .name and .message properties populated</returns>
            /// </signature>
            this.name = name;
            this.message = message || name;
        }, {
            /* empty */
        }, {
            supportedForProcessing: false,
        })
    });

})(this, this.WinJS);


(function xhrInit(WinJS) {
    "use strict";


    WinJS.Namespace.define("WinJS", {
        xhr: function (options) {
            /// <signature helpKeyword="WinJS.xhr">
            /// <summary locid="WinJS.xhr">
            /// Wraps calls to XMLHttpRequest in a promise.
            /// </summary>
            /// <param name="options" type="Object" locid="WinJS.xhr_p:options">
            /// The options that are applied to the XMLHttpRequest object. They are: type,
            /// url, user, password, headers, responseType, data, and customRequestInitializer.
            /// </param>
            /// <returns type="WinJS.Promise" locid="WinJS.xhr_returnValue">
            /// A promise that returns the XMLHttpRequest object when it completes.
            /// </returns>
            /// </signature>
            var req;
            return new WinJS.Promise(
                function (c, e, p) {
                    /// <returns value="c(new XMLHttpRequest())" locid="WinJS.xhr.constructor._returnValue" />
                    req = new XMLHttpRequest();
                    req.onreadystatechange = function () {
                        if (req._canceled) { return; }

                        if (req.readyState === 4) {
							// MONACO CHANGE: Handle 1223: http://bugs.jquery.com/ticket/1450
                            if ((req.status >= 200 && req.status < 300) || req.status === 1223) {
                                c(req);
                            } else {
                                e(req);
                            }
                            req.onreadystatechange = function () { };
                        } else {
                            p(req);
                        }
                    };

                    req.open(
                        options.type || "GET",
                        options.url,
                        // Promise based XHR does not support sync.
                        //
                        true,
                        options.user,
                        options.password
                    );
                    req.responseType = options.responseType || "";

                    Object.keys(options.headers || {}).forEach(function (k) {
                        req.setRequestHeader(k, options.headers[k]);
                    });

                    if (options.customRequestInitializer) {
                        options.customRequestInitializer(req);
                    }

                    req.send(options.data);
                },
                function () {
                    req._canceled = true;
                    req.abort();
                }
            );
        }
    });

})(this.WinJS);


(function safeHTMLInit(global, WinJS, undefined) {
    "use strict";


    var setInnerHTML,
        setInnerHTMLUnsafe,
        setOuterHTML,
        setOuterHTMLUnsafe,
        insertAdjacentHTML,
        insertAdjacentHTMLUnsafe;

    var strings = {
		// MONACO CHANGE
        //get nonStaticHTML() { return WinJS.Resources._getWinJSString("base/nonStaticHTML").value; },
		nonStaticHTML: "Unable to add dynamic content. A script attempted to inject dynamic content, or elements previously modified dynamically, that might be unsafe. For example, using the innerHTML property or the document.write method to add a script element will generate this exception. If the content is safe and from a trusted source, use a method to explicitly manipulate elements and attributes, such as createElement, or use setInnerHTMLUnsafe (or other unsafe method)."
    };

    setInnerHTML = setInnerHTMLUnsafe = function (element, text) {
        /// <signature helpKeyword="WinJS.Utilities.setInnerHTML">
        /// <summary locid="WinJS.Utilities.setInnerHTML">
        /// Sets the innerHTML property of the specified element to the specified text.
        /// </summary>
        /// <param name="element" type="HTMLElement" locid="WinJS.Utilities.setInnerHTML_p:element">
        /// The element on which the innerHTML property is to be set.
        /// </param>
        /// <param name="text" type="String" locid="WinJS.Utilities.setInnerHTML_p:text">
        /// The value to be set to the innerHTML property.
        /// </param>
        /// </signature>
        element.innerHTML = text;
    };
    setOuterHTML = setOuterHTMLUnsafe = function (element, text) {
        /// <signature helpKeyword="WinJS.Utilities.setOuterHTML">
        /// <summary locid="WinJS.Utilities.setOuterHTML">
        /// Sets the outerHTML property of the specified element to the specified text.
        /// </summary>
        /// <param name="element" type="HTMLElement" locid="WinJS.Utilities.setOuterHTML_p:element">
        /// The element on which the outerHTML property is to be set.
        /// </param>
        /// <param name="text" type="String" locid="WinJS.Utilities.setOuterHTML_p:text">
        /// The value to be set to the outerHTML property.
        /// </param>
        /// </signature>
        element.outerHTML = text;
    };
    insertAdjacentHTML = insertAdjacentHTMLUnsafe = function (element, position, text) {
        /// <signature helpKeyword="WinJS.Utilities.insertAdjacentHTML">
        /// <summary locid="WinJS.Utilities.insertAdjacentHTML">
        /// Calls insertAdjacentHTML on the specified element.
        /// </summary>
        /// <param name="element" type="HTMLElement" locid="WinJS.Utilities.insertAdjacentHTML_p:element">
        /// The element on which insertAdjacentHTML is to be called.
        /// </param>
        /// <param name="position" type="String" locid="WinJS.Utilities.insertAdjacentHTML_p:position">
        /// The position relative to the element at which to insert the HTML.
        /// </param>
        /// <param name="text" type="String" locid="WinJS.Utilities.insertAdjacentHTML_p:text">
        /// The value to be provided to insertAdjacentHTML.
        /// </param>
        /// </signature>
        element.insertAdjacentHTML(position, text);
    };

    var msApp = global.MSApp;
    if (msApp) {
        setInnerHTMLUnsafe = function (element, text) {
            /// <signature helpKeyword="WinJS.Utilities.setInnerHTMLUnsafe">
            /// <summary locid="WinJS.Utilities.setInnerHTMLUnsafe">
            /// Sets the innerHTML property of the specified element to the specified text.
            /// </summary>
            /// <param name='element' type='HTMLElement' locid="WinJS.Utilities.setInnerHTMLUnsafe_p:element">
            /// The element on which the innerHTML property is to be set.
            /// </param>
            /// <param name='text' type="String" locid="WinJS.Utilities.setInnerHTMLUnsafe_p:text">
            /// The value to be set to the innerHTML property.
            /// </param>
            /// </signature>
            msApp.execUnsafeLocalFunction(function () {
                element.innerHTML = text;
            });
        };
        setOuterHTMLUnsafe = function (element, text) {
            /// <signature helpKeyword="WinJS.Utilities.setOuterHTMLUnsafe">
            /// <summary locid="WinJS.Utilities.setOuterHTMLUnsafe">
            /// Sets the outerHTML property of the specified element to the specified text
            /// in the context of msWWA.execUnsafeLocalFunction.
            /// </summary>
            /// <param name="element" type="HTMLElement" locid="WinJS.Utilities.setOuterHTMLUnsafe_p:element">
            /// The element on which the outerHTML property is to be set.
            /// </param>
            /// <param name="text" type="String" locid="WinJS.Utilities.setOuterHTMLUnsafe_p:text">
            /// The value to be set to the outerHTML property.
            /// </param>
            /// </signature>
            msApp.execUnsafeLocalFunction(function () {
                element.outerHTML = text;
            });
        };
        insertAdjacentHTMLUnsafe = function (element, position, text) {
            /// <signature helpKeyword="WinJS.Utilities.insertAdjacentHTMLUnsafe">
            /// <summary locid="WinJS.Utilities.insertAdjacentHTMLUnsafe">
            /// Calls insertAdjacentHTML on the specified element in the context
            /// of msWWA.execUnsafeLocalFunction.
            /// </summary>
            /// <param name="element" type="HTMLElement" locid="WinJS.Utilities.insertAdjacentHTMLUnsafe_p:element">
            /// The element on which insertAdjacentHTML is to be called.
            /// </param>
            /// <param name="position" type="String" locid="WinJS.Utilities.insertAdjacentHTMLUnsafe_p:position">
            /// The position relative to the element at which to insert the HTML.
            /// </param>
            /// <param name="text" type="String" locid="WinJS.Utilities.insertAdjacentHTMLUnsafe_p:text">
            /// Value to be provided to insertAdjacentHTML.
            /// </param>
            /// </signature>
            msApp.execUnsafeLocalFunction(function () {
                element.insertAdjacentHTML(position, text);
            });
        };
    }
    else if (global.msIsStaticHTML) {
        var check = function (str) {
            if (!global.msIsStaticHTML(str)) {
                throw new WinJS.ErrorFromName("WinJS.Utitilies.NonStaticHTML", strings.nonStaticHTML);
            }
        }
        // If we ever get isStaticHTML we can attempt to recreate the behavior we have in the local
        // compartment, in the mean-time all we can do is sanitize the input.
        //
        setInnerHTML = function (element, text) {
            /// <signature helpKeyword="WinJS.Utilities.setInnerHTML">
            /// <summary locid="WinJS.Utilities.msIsStaticHTML.setInnerHTML">
            /// Sets the innerHTML property of a element to the specified text
            /// if it passes a msIsStaticHTML check.
            /// </summary>
            /// <param name="element" type="HTMLElement" locid="WinJS.Utilities.msIsStaticHTML.setInnerHTML_p:element">
            /// The element on which the innerHTML property is to be set.
            /// </param>
            /// <param name="text" type="String" locid="WinJS.Utilities.msIsStaticHTML.setInnerHTML_p:text">
            /// The value to be set to the innerHTML property.
            /// </param>
            /// </signature>
            check(text);
            element.innerHTML = text;
        };
        setOuterHTML = function (element, text) {
            /// <signature helpKeyword="WinJS.Utilities.setOuterHTML">
            /// <summary locid="WinJS.Utilities.msIsStaticHTML.setOuterHTML">
            /// Sets the outerHTML property of a element to the specified text
            /// if it passes a msIsStaticHTML check.
            /// </summary>
            /// <param name="element" type="HTMLElement" locid="WinJS.Utilities.msIsStaticHTML.setOuterHTML_p:element">
            /// The element on which the outerHTML property is to be set.
            /// </param>
            /// <param name="text" type="String" locid="WinJS.Utilities.msIsStaticHTML.setOuterHTML_p:text">
            /// The value to be set to the outerHTML property.
            /// </param>
            /// </signature>
            check(text);
            element.outerHTML = text;
        };
        insertAdjacentHTML = function (element, position, text) {
            /// <signature helpKeyword="WinJS.Utilities.insertAdjacentHTML">
            /// <summary locid="WinJS.Utilities.msIsStaticHTML.insertAdjacentHTML">
            /// Calls insertAdjacentHTML on the element if it passes
            /// a msIsStaticHTML check.
            /// </summary>
            /// <param name="element" type="HTMLElement" locid="WinJS.Utilities.msIsStaticHTML.insertAdjacentHTML_p:element">
            /// The element on which insertAdjacentHTML is to be called.
            /// </param>
            /// <param name="position" type="String" locid="WinJS.Utilities.msIsStaticHTML.insertAdjacentHTML_p:position">
            /// The position relative to the element at which to insert the HTML.
            /// </param>
            /// <param name="text" type="String" locid="WinJS.Utilities.msIsStaticHTML.insertAdjacentHTML_p:text">
            /// The value to be provided to insertAdjacentHTML.
            /// </param>
            /// </signature>
            check(text);
            element.insertAdjacentHTML(position, text);
        };
    }

    WinJS.Namespace.define("WinJS.Utilities", {
        setInnerHTML: setInnerHTML,
        setInnerHTMLUnsafe: setInnerHTMLUnsafe,
        setOuterHTML: setOuterHTML,
        setOuterHTMLUnsafe: setOuterHTMLUnsafe,
        insertAdjacentHTML: insertAdjacentHTML,
        insertAdjacentHTMLUnsafe: insertAdjacentHTMLUnsafe
    });

}(this, this.WinJS));



// MONACO CHANGE
} // if (typeof WinJS === 'undefined')

(function(global) {

    if (typeof exports === 'undefined' && typeof define === 'function' && define.amd) {
        define("vs/base/common/winjs.base.raw", global.WinJS);
    } else {
        module.exports = global.WinJS;
    }

})(this);
define("vs/base/node/aiAdapter", ["require", "exports", 'vs/base/common/types', 'vs/base/common/objects', 'applicationinsights'], function (require, exports, types, objects_1, appInsights) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var AIAdapter = (function () {
        function AIAdapter(aiKey, eventPrefix, 
            /* for test only */
            client, additionalDataToLog) {
            this.aiKey = aiKey;
            this.eventPrefix = eventPrefix;
            this.additionalDataToLog = additionalDataToLog;
            // for test
            if (client) {
                this.appInsights = client;
                return;
            }
            if (aiKey) {
                // if another client is already initialized
                if (appInsights.client) {
                    this.appInsights = appInsights.getClient(aiKey);
                    // no other way to enable offline mode
                    this.appInsights.channel.setOfflineMode(true);
                }
                else {
                    this.appInsights = appInsights.setup(aiKey)
                        .setAutoCollectRequests(false)
                        .setAutoCollectPerformance(false)
                        .setAutoCollectExceptions(false)
                        .setOfflineMode(true)
                        .start()
                        .client;
                }
                if (aiKey.indexOf('AIF-') === 0) {
                    this.appInsights.config.endpointUrl = 'https://vortex.data.microsoft.com/collect/v1';
                }
                this.setupAIClient(this.appInsights);
            }
        }
        AIAdapter.prototype.setupAIClient = function (client) {
            //prevent App Insights from reporting machine name
            if (client && client.context &&
                client.context.keys && client.context.tags) {
                var machineNameKey = client.context.keys.deviceMachineName;
                client.context.tags[machineNameKey] = '';
            }
        };
        AIAdapter.prototype.getData = function (data) {
            var properties = {};
            var measurements = {};
            var event_data = this.flaten(data);
            for (var prop in event_data) {
                // enforce property names less than 150 char, take the last 150 char
                var propName = prop && prop.length > 150 ? prop.substr(prop.length - 149) : prop;
                var property = event_data[prop];
                if (types.isNumber(property)) {
                    measurements[propName] = property;
                }
                else if (types.isBoolean(property)) {
                    measurements[propName] = property ? 1 : 0;
                }
                else if (types.isString(property)) {
                    //enforce proeprty value to be less than 1024 char, take the first 1024 char
                    var propValue = property && property.length > 1024 ? property.substring(0, 1023) : property;
                    properties[propName] = propValue;
                }
                else if (!types.isUndefined(property) && property !== null) {
                    properties[propName] = property;
                }
            }
            return {
                properties: properties,
                measurements: measurements
            };
        };
        AIAdapter.prototype.flaten = function (obj, order, prefix) {
            if (order === void 0) { order = 0; }
            var result = {};
            var properties = obj ? Object.getOwnPropertyNames(obj) : [];
            for (var i = 0; i < properties.length; i++) {
                var item = properties[i];
                var index = prefix ? prefix + item : item;
                if (types.isArray(obj[item])) {
                    try {
                        result[index] = objects_1.safeStringify(obj[item]);
                    }
                    catch (e) {
                        // workaround for catching the edge case for #18383
                        // safe stringfy should never throw circular object exception
                        result[index] = '[Circular-Array]';
                    }
                }
                else if (obj[item] instanceof Date) {
                    result[index] = obj[item].toISOString();
                }
                else if (types.isObject(obj[item])) {
                    if (order < 2) {
                        var item_result = this.flaten(obj[item], order + 1, index + '.');
                        for (var prop in item_result) {
                            result[prop] = item_result[prop];
                        }
                    }
                    else {
                        try {
                            result[index] = objects_1.safeStringify(obj[item]);
                        }
                        catch (e) {
                            // workaround for catching the edge case for #18383
                            // safe stringfy should never throw circular object exception
                            result[index] = '[Circular]';
                        }
                    }
                }
                else {
                    result[index] = obj[item];
                }
            }
            return result;
        };
        AIAdapter.prototype.log = function (eventName, data) {
            if (this.additionalDataToLog) {
                data = objects_1.mixin(data, this.additionalDataToLog);
            }
            var result = this.getData(data);
            if (this.appInsights) {
                this.appInsights.trackEvent(this.eventPrefix + '/' + eventName, result.properties, result.measurements);
            }
        };
        AIAdapter.prototype.logException = function (exception) {
            if (this.appInsights) {
                this.appInsights.trackException(exception);
            }
        };
        AIAdapter.prototype.dispose = function () {
            this.appInsights = null;
        };
        return AIAdapter;
    }());
    exports.AIAdapter = AIAdapter;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/decoder", ["require", "exports", 'string_decoder'], function (require, exports, sd) {
    'use strict';
    /**
     * Convenient way to iterate over output line by line. This helper accommodates for the fact that
     * a buffer might not end with new lines all the way.
     *
     * To use:
     * - call the write method
     * - forEach() over the result to get the lines
     */
    var LineDecoder = (function () {
        function LineDecoder(encoding) {
            if (encoding === void 0) { encoding = 'utf8'; }
            this.stringDecoder = new sd.StringDecoder(encoding);
            this.remaining = null;
        }
        LineDecoder.prototype.write = function (buffer) {
            var result = [];
            var value = this.remaining
                ? this.remaining + this.stringDecoder.write(buffer)
                : this.stringDecoder.write(buffer);
            if (value.length < 1) {
                return result;
            }
            var start = 0;
            var ch;
            while (start < value.length && ((ch = value.charCodeAt(start)) === 13 || ch === 10)) {
                start++;
            }
            var idx = start;
            while (idx < value.length) {
                ch = value.charCodeAt(idx);
                if (ch === 13 || ch === 10) {
                    result.push(value.substring(start, idx));
                    idx++;
                    while (idx < value.length && ((ch = value.charCodeAt(idx)) === 13 || ch === 10)) {
                        idx++;
                    }
                    start = idx;
                }
                else {
                    idx++;
                }
            }
            this.remaining = start < value.length ? value.substr(start) : null;
            return result;
        };
        LineDecoder.prototype.end = function () {
            return this.remaining;
        };
        return LineDecoder;
    }());
    exports.LineDecoder = LineDecoder;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/flow", ["require", "exports", 'assert'], function (require, exports, assert) {
    'use strict';
    /**
     * Executes the given function (fn) over the given array of items (list) in parallel and returns the resulting errors and results as
     * array to the callback (callback). The resulting errors and results are evaluated by calling the provided callback function.
     */
    function parallel(list, fn, callback) {
        var results = new Array(list.length);
        var errors = new Array(list.length);
        var didErrorOccur = false;
        var doneCount = 0;
        if (list.length === 0) {
            return callback(null, []);
        }
        list.forEach(function (item, index) {
            fn(item, function (error, result) {
                if (error) {
                    didErrorOccur = true;
                    results[index] = null;
                    errors[index] = error;
                }
                else {
                    results[index] = result;
                    errors[index] = null;
                }
                if (++doneCount === list.length) {
                    return callback(didErrorOccur ? errors : null, results);
                }
            });
        });
    }
    exports.parallel = parallel;
    function loop(param, fn, callback) {
        // Assert
        assert.ok(param, 'Missing first parameter');
        assert.ok(typeof (fn) === 'function', 'Second parameter must be a function that is called for each element');
        assert.ok(typeof (callback) === 'function', 'Third parameter must be a function that is called on error and success');
        // Param is function, execute to retrieve array
        if (typeof (param) === 'function') {
            try {
                param(function (error, result) {
                    if (error) {
                        callback(error, null);
                    }
                    else {
                        loop(result, fn, callback);
                    }
                });
            }
            catch (error) {
                callback(error, null);
            }
        }
        else {
            var results_1 = [];
            var looper_1 = function (i) {
                // Still work to do
                if (i < param.length) {
                    // Execute function on array element
                    try {
                        fn(param[i], function (error, result) {
                            // A method might only send a boolean value as return value (e.g. fs.exists), support this case gracefully
                            if (error === true || error === false) {
                                result = error;
                                error = null;
                            }
                            // Quit looping on error
                            if (error) {
                                callback(error, null);
                            }
                            else {
                                if (result) {
                                    results_1.push(result);
                                }
                                process.nextTick(function () {
                                    looper_1(i + 1);
                                });
                            }
                        }, i, param.length);
                    }
                    catch (error) {
                        callback(error, null);
                    }
                }
                else {
                    callback(null, results_1);
                }
            };
            // Start looping with first element in array
            looper_1(0);
        }
    }
    exports.loop = loop;
    function Sequence(sequences) {
        // Assert
        assert.ok(sequences.length > 1, 'Need at least one error handler and one function to process sequence');
        sequences.forEach(function (sequence) {
            assert.ok(typeof (sequence) === 'function');
        });
        // Execute in Loop
        var errorHandler = sequences.splice(0, 1)[0]; //Remove error handler
        var sequenceResult = null;
        loop(sequences, function (sequence, clb) {
            var sequenceFunction = function (error, result) {
                // A method might only send a boolean value as return value (e.g. fs.exists), support this case gracefully
                if (error === true || error === false) {
                    result = error;
                    error = null;
                }
                // Handle Error and Result
                if (error) {
                    clb(error, null);
                }
                else {
                    sequenceResult = result; //Remember result of sequence
                    clb(null, null); //Don't pass on result to Looper as we are not aggregating it
                }
            };
            // We call the sequence function setting "this" to be the callback we define here
            // and we pass in the "sequenceResult" as first argument. Doing all this avoids having
            // to pass in a callback to the sequence because the callback is already "this".
            try {
                sequence.call(sequenceFunction, sequenceResult);
            }
            catch (error) {
                clb(error, null);
            }
        }, function (error, result) {
            if (error) {
                errorHandler(error);
            }
        });
    }
    function sequence(sequences) {
        Sequence((Array.isArray(sequences)) ? sequences : Array.prototype.slice.call(arguments));
    }
    exports.sequence = sequence;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/extfs", ["require", "exports", 'vs/base/common/uuid', 'vs/base/common/strings', 'vs/base/common/platform', 'vs/base/node/flow', 'fs', 'path'], function (require, exports, uuid, strings, platform, flow, fs, paths) {
    'use strict';
    var loop = flow.loop;
    function readdir(path, callback) {
        // Mac: uses NFD unicode form on disk, but we want NFC
        // See also https://github.com/nodejs/node/issues/2165
        if (platform.isMacintosh) {
            return readdirNormalize(path, function (error, children) {
                if (error) {
                    return callback(error, null);
                }
                return callback(null, children.map(function (c) { return strings.normalizeNFC(c); }));
            });
        }
        return readdirNormalize(path, callback);
    }
    exports.readdir = readdir;
    function readdirNormalize(path, callback) {
        fs.readdir(path, function (error, children) {
            if (error) {
                return callback(error, null);
            }
            // Bug in node: In some environments we get "." and ".." as entries from the call to readdir().
            // For example Sharepoint via WebDav on Windows includes them. We never want those
            // entries in the result set though because they are not valid children of the folder
            // for our concerns.
            // See https://github.com/nodejs/node/issues/4002
            return callback(null, children.filter(function (c) { return c !== '.' && c !== '..'; }));
        });
    }
    function mkdirp(path, mode, callback) {
        fs.exists(path, function (exists) {
            if (exists) {
                return isDirectory(path, function (err, itIs) {
                    if (err) {
                        return callback(err);
                    }
                    if (!itIs) {
                        return callback(new Error('"' + path + '" is not a directory.'));
                    }
                    callback(null);
                });
            }
            mkdirp(paths.dirname(path), mode, function (err) {
                if (err) {
                    callback(err);
                    return;
                }
                if (mode) {
                    fs.mkdir(path, mode, function (error) {
                        if (error) {
                            return callback(error);
                        }
                        fs.chmod(path, mode, callback); // we need to explicitly chmod because of https://github.com/nodejs/node/issues/1104
                    });
                }
                else {
                    fs.mkdir(path, null, callback);
                }
            });
        });
    }
    exports.mkdirp = mkdirp;
    function isDirectory(path, callback) {
        fs.stat(path, function (error, stat) {
            if (error) {
                return callback(error);
            }
            callback(null, stat.isDirectory());
        });
    }
    function copy(source, target, callback, copiedSources) {
        if (!copiedSources) {
            copiedSources = Object.create(null);
        }
        fs.stat(source, function (error, stat) {
            if (error) {
                return callback(error);
            }
            if (!stat.isDirectory()) {
                return pipeFs(source, target, stat.mode & 511, callback);
            }
            if (copiedSources[source]) {
                return callback(null); // escape when there are cycles (can happen with symlinks)
            }
            else {
                copiedSources[source] = true; // remember as copied
            }
            mkdirp(target, stat.mode & 511, function (err) {
                readdir(source, function (err, files) {
                    loop(files, function (file, clb) {
                        copy(paths.join(source, file), paths.join(target, file), clb, copiedSources);
                    }, callback);
                });
            });
        });
    }
    exports.copy = copy;
    function pipeFs(source, target, mode, callback) {
        var callbackHandled = false;
        var readStream = fs.createReadStream(source);
        var writeStream = fs.createWriteStream(target, { mode: mode });
        var onError = function (error) {
            if (!callbackHandled) {
                callbackHandled = true;
                callback(error);
            }
        };
        readStream.on('error', onError);
        writeStream.on('error', onError);
        readStream.on('end', function () {
            writeStream.end(function () {
                if (!callbackHandled) {
                    callbackHandled = true;
                    fs.chmod(target, mode, callback); // we need to explicitly chmod because of https://github.com/nodejs/node/issues/1104
                }
            });
        });
        // In node 0.8 there is no easy way to find out when the pipe operation has finished. As such, we use the end property = false
        // so that we are in charge of calling end() on the write stream and we will be notified when the write stream is really done.
        // We can do this because file streams have an end() method that allows to pass in a callback.
        // In node 0.10 there is an event 'finish' emitted from the write stream that can be used. See
        // https://groups.google.com/forum/?fromgroups=#!topic/nodejs/YWQ1sRoXOdI
        readStream.pipe(writeStream, { end: false });
    }
    // Deletes the given path by first moving it out of the workspace. This has two benefits. For one, the operation can return fast because
    // after the rename, the contents are out of the workspace although not yet deleted. The greater benefit however is that this operation
    // will fail in case any file is used by another process. fs.unlink() in node will not bail if a file unlinked is used by another process.
    // However, the consequences are bad as outlined in all the related bugs from https://github.com/joyent/node/issues/7164
    function del(path, tmpFolder, callback, done) {
        fs.exists(path, function (exists) {
            if (!exists) {
                return callback(null);
            }
            fs.stat(path, function (err, stat) {
                if (err || !stat) {
                    return callback(err);
                }
                // Special windows workaround: A file or folder that ends with a "." cannot be moved to another place
                // because it is not a valid file name. In this case, we really have to do the deletion without prior move.
                if (path[path.length - 1] === '.' || strings.endsWith(path, './') || strings.endsWith(path, '.\\')) {
                    return rmRecursive(path, callback);
                }
                var pathInTemp = paths.join(tmpFolder, uuid.generateUuid());
                fs.rename(path, pathInTemp, function (error) {
                    if (error) {
                        return rmRecursive(path, callback); // if rename fails, delete without tmp dir
                    }
                    // Return early since the move succeeded
                    callback(null);
                    // do the heavy deletion outside the callers callback
                    rmRecursive(pathInTemp, function (error) {
                        if (error) {
                            console.error(error);
                        }
                        if (done) {
                            done(error);
                        }
                    });
                });
            });
        });
    }
    exports.del = del;
    function rmRecursive(path, callback) {
        if (path === '\\' || path === '/') {
            return callback(new Error('Will not delete root!'));
        }
        fs.exists(path, function (exists) {
            if (!exists) {
                callback(null);
            }
            else {
                fs.lstat(path, function (err, stat) {
                    if (err || !stat) {
                        callback(err);
                    }
                    else if (!stat.isDirectory() || stat.isSymbolicLink() /* !!! never recurse into links when deleting !!! */) {
                        var mode = stat.mode;
                        if (!(mode & 128)) {
                            fs.chmod(path, mode | 128, function (err) {
                                if (err) {
                                    callback(err);
                                }
                                else {
                                    fs.unlink(path, callback);
                                }
                            });
                        }
                        else {
                            fs.unlink(path, callback);
                        }
                    }
                    else {
                        readdir(path, function (err, children) {
                            if (err || !children) {
                                callback(err);
                            }
                            else if (children.length === 0) {
                                fs.rmdir(path, callback);
                            }
                            else {
                                var firstError_1 = null;
                                var childrenLeft_1 = children.length;
                                children.forEach(function (child) {
                                    rmRecursive(paths.join(path, child), function (err) {
                                        childrenLeft_1--;
                                        if (err) {
                                            firstError_1 = firstError_1 || err;
                                        }
                                        if (childrenLeft_1 === 0) {
                                            if (firstError_1) {
                                                callback(firstError_1);
                                            }
                                            else {
                                                fs.rmdir(path, callback);
                                            }
                                        }
                                    });
                                });
                            }
                        });
                    }
                });
            }
        });
    }
    function mv(source, target, callback) {
        if (source === target) {
            return callback(null);
        }
        function updateMtime(err) {
            if (err) {
                return callback(err);
            }
            fs.stat(target, function (error, stat) {
                if (error) {
                    return callback(error);
                }
                if (stat.isDirectory()) {
                    return callback(null);
                }
                fs.open(target, 'a', null, function (err, fd) {
                    if (err) {
                        return callback(err);
                    }
                    fs.futimes(fd, stat.atime, new Date(), function (err) {
                        if (err) {
                            return callback(err);
                        }
                        fs.close(fd, callback);
                    });
                });
            });
        }
        // Try native rename()
        fs.rename(source, target, function (err) {
            if (!err) {
                return updateMtime(null);
            }
            // In two cases we fallback to classic copy and delete:
            //
            // 1.) The EXDEV error indicates that source and target are on different devices
            // In this case, fallback to using a copy() operation as there is no way to
            // rename() between different devices.
            //
            // 2.) The user tries to rename a file/folder that ends with a dot. This is not
            // really possible to move then, at least on UNC devices.
            if (err && source.toLowerCase() !== target.toLowerCase() && (err.code === 'EXDEV') || strings.endsWith(source, '.')) {
                return copy(source, target, function (err) {
                    if (err) {
                        return callback(err);
                    }
                    rmRecursive(source, updateMtime);
                });
            }
            return callback(err);
        });
    }
    exports.mv = mv;
});

define("vs/base/node/plist", ["require", "exports", 'sax'], function (require, exports, sax) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function parse(content) {
        var errors = [];
        var currObject = null;
        var result = null;
        var text = null;
        var parser = sax.parser(false, { lowercase: true });
        parser.onerror = function (e) {
            errors.push(e.message);
        };
        parser.ontext = function (s) {
            text = s;
        };
        parser.onopentag = function (tag) {
            switch (tag.name) {
                case 'dict':
                    currObject = { parent: currObject, value: {} };
                    break;
                case 'array':
                    currObject = { parent: currObject, value: [] };
                    break;
                case 'key':
                    if (currObject) {
                        currObject.lastKey = null;
                    }
                    break;
            }
            text = '';
        };
        parser.onclosetag = function (tagName) {
            var value;
            switch (tagName) {
                case 'key':
                    if (!currObject || Array.isArray(currObject.value)) {
                        errors.push('key can only be used inside an open dict element');
                        return;
                    }
                    currObject.lastKey = text;
                    return;
                case 'dict':
                case 'array':
                    if (!currObject) {
                        errors.push(tagName + ' closing tag found, without opening tag');
                        return;
                    }
                    value = currObject.value;
                    currObject = currObject.parent;
                    break;
                case 'string':
                case 'data':
                    value = text;
                    break;
                case 'date':
                    value = new Date(text);
                    break;
                case 'integer':
                    value = parseInt(text);
                    if (isNaN(value)) {
                        errors.push(text + ' is not a integer');
                        return;
                    }
                    break;
                case 'real':
                    value = parseFloat(text);
                    if (isNaN(value)) {
                        errors.push(text + ' is not a float');
                        return;
                    }
                    break;
                case 'true':
                    value = true;
                    break;
                case 'false':
                    value = false;
                    break;
                case 'plist':
                    return;
                default:
                    errors.push('Invalid tag name: ' + tagName);
                    return;
            }
            if (!currObject) {
                result = value;
            }
            else if (Array.isArray(currObject.value)) {
                currObject.value.push(value);
            }
            else {
                if (currObject.lastKey) {
                    currObject.value[currObject.lastKey] = value;
                }
                else {
                    errors.push('Dictionary key missing for value ' + value);
                }
            }
        };
        parser.write(content);
        return { errors: errors, value: result };
    }
    exports.parse = parse;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/ports", ["require", "exports", 'net'], function (require, exports, net) {
    'use strict';
    /**
     * Given a start point and a max number of retries, will find a port that
     * is openable. Will return 0 in case no free port can be found.
     */
    function findFreePort(startPort, giveUpAfter, clb) {
        if (giveUpAfter === 0) {
            return clb(0);
        }
        var tryPort = startPort;
        var server = net.createServer();
        server.listen(tryPort, function (err) {
            server.once('close', function () {
                return clb(tryPort);
            });
            server.close();
        });
        server.on('error', function (err) {
            findFreePort(startPort + 1, giveUpAfter - 1, clb);
        });
    }
    exports.findFreePort = findFreePort;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/proxy", ["require", "exports", 'url', 'vs/base/common/types', 'http-proxy-agent', 'https-proxy-agent'], function (require, exports, url_1, types_1, HttpProxyAgent, HttpsProxyAgent) {
    'use strict';
    function getSystemProxyURI(requestURL) {
        if (requestURL.protocol === 'http:') {
            return process.env.HTTP_PROXY || process.env.http_proxy || null;
        }
        else if (requestURL.protocol === 'https:') {
            return process.env.HTTPS_PROXY || process.env.https_proxy || process.env.HTTP_PROXY || process.env.http_proxy || null;
        }
        return null;
    }
    function getProxyAgent(rawRequestURL, options) {
        if (options === void 0) { options = {}; }
        var requestURL = url_1.parse(rawRequestURL);
        var proxyURL = options.proxyUrl || getSystemProxyURI(requestURL);
        if (!proxyURL) {
            return null;
        }
        var proxyEndpoint = url_1.parse(proxyURL);
        if (!/^https?:$/.test(proxyEndpoint.protocol)) {
            return null;
        }
        var opts = {
            host: proxyEndpoint.hostname,
            port: Number(proxyEndpoint.port),
            auth: proxyEndpoint.auth,
            rejectUnauthorized: types_1.isBoolean(options.strictSSL) ? options.strictSSL : true
        };
        return requestURL.protocol === 'http:' ? new HttpProxyAgent(opts) : new HttpsProxyAgent(opts);
    }
    exports.getProxyAgent = getProxyAgent;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/stdFork", ["require", "exports", 'path', 'os', 'net', 'child_process', 'vs/base/common/uri'], function (require, exports, path, os, net, cp, uri_1) {
    'use strict';
    function makeRandomHexString(length) {
        var chars = ['0', '1', '2', '3', '4', '5', '6', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'];
        var result = '';
        for (var i = 0; i < length; i++) {
            var idx = Math.floor(chars.length * Math.random());
            result += chars[idx];
        }
        return result;
    }
    function generatePipeName() {
        var randomName = 'vscode-' + makeRandomHexString(40);
        if (process.platform === 'win32') {
            return '\\\\.\\pipe\\' + randomName + '-sock';
        }
        // Mac/Unix: use socket file
        return path.join(os.tmpdir(), randomName + '.sock');
    }
    function generatePatchedEnv(env, stdInPipeName, stdOutPipeName, stdErrPipeName) {
        // Set the two unique pipe names and the electron flag as process env
        var newEnv = {};
        for (var key in env) {
            newEnv[key] = env[key];
        }
        newEnv['STDIN_PIPE_NAME'] = stdInPipeName;
        newEnv['STDOUT_PIPE_NAME'] = stdOutPipeName;
        newEnv['STDERR_PIPE_NAME'] = stdErrPipeName;
        newEnv['ATOM_SHELL_INTERNAL_RUN_AS_NODE'] = '1';
        return newEnv;
    }
    function fork(modulePath, args, options, callback) {
        var callbackCalled = false;
        var resolve = function (result) {
            if (callbackCalled) {
                return;
            }
            callbackCalled = true;
            callback(null, result);
        };
        var reject = function (err) {
            if (callbackCalled) {
                return;
            }
            callbackCalled = true;
            callback(err, null);
        };
        // Generate three unique pipe names
        var stdInPipeName = generatePipeName();
        var stdOutPipeName = generatePipeName();
        var stdErrPipeName = generatePipeName();
        var newEnv = generatePatchedEnv(options.env || process.env, stdInPipeName, stdOutPipeName, stdErrPipeName);
        var childProcess;
        // Begin listening to stderr pipe
        var stdErrServer = net.createServer(function (stdErrStream) {
            // From now on the childProcess.stderr is available for reading
            childProcess.stderr = stdErrStream;
        });
        stdErrServer.listen(stdErrPipeName);
        // Begin listening to stdout pipe
        var stdOutServer = net.createServer(function (stdOutStream) {
            // The child process will write exactly one chunk with content `ready` when it has installed a listener to the stdin pipe
            stdOutStream.once('data', function (chunk) {
                // The child process is sending me the `ready` chunk, time to connect to the stdin pipe
                childProcess.stdin = net.connect(stdInPipeName);
                // From now on the childProcess.stdout is available for reading
                childProcess.stdout = stdOutStream;
                resolve(childProcess);
            });
        });
        stdOutServer.listen(stdOutPipeName);
        var serverClosed = false;
        var closeServer = function () {
            if (serverClosed) {
                return;
            }
            serverClosed = true;
            stdOutServer.close();
            stdErrServer.close();
        };
        // Create the process
        var bootstrapperPath = (uri_1.default.parse(require.toUrl('./stdForkStart.js')).fsPath);
        childProcess = cp.fork(bootstrapperPath, [modulePath].concat(args), {
            silent: true,
            cwd: options.cwd,
            env: newEnv,
            execArgv: options.execArgv
        });
        childProcess.once('error', function (err) {
            closeServer();
            reject(err);
        });
        childProcess.once('exit', function (err) {
            closeServer();
            reject(err);
        });
    }
    exports.fork = fork;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/stream", ["require", "exports", 'fs'], function (require, exports, fs) {
    'use strict';
    /**
     * Reads up to total bytes from the provided stream.
     */
    function readExactlyByStream(stream, totalBytes, callback) {
        var done = false;
        var buffer = new Buffer(totalBytes);
        var bytesRead = 0;
        stream.on('data', function (data) {
            var bytesToRead = Math.min(totalBytes - bytesRead, data.length);
            data.copy(buffer, bytesRead, 0, bytesToRead);
            bytesRead += bytesToRead;
            if (bytesRead === totalBytes) {
                stream.destroy(); // Will trigger the close event eventually
            }
        });
        stream.on('error', function (e) {
            if (!done) {
                done = true;
                callback(e, null, null);
            }
        });
        var onSuccess = function () {
            if (!done) {
                done = true;
                callback(null, buffer, bytesRead);
            }
        };
        stream.on('close', onSuccess);
    }
    exports.readExactlyByStream = readExactlyByStream;
    /**
     * Reads totalBytes from the provided file.
     */
    function readExactlyByFile(file, totalBytes, callback) {
        fs.open(file, 'r', null, function (err, fd) {
            if (err) {
                return callback(err, null, 0);
            }
            function end(err, resultBuffer, bytesRead) {
                fs.close(fd, function (closeError) {
                    if (closeError) {
                        return callback(closeError, null, bytesRead);
                    }
                    if (err && err.code === 'EISDIR') {
                        return callback(err, null, bytesRead); // we want to bubble this error up (file is actually a folder)
                    }
                    return callback(null, resultBuffer, bytesRead);
                });
            }
            var buffer = new Buffer(totalBytes);
            var bytesRead = 0;
            var zeroAttempts = 0;
            function loop() {
                fs.read(fd, buffer, bytesRead, totalBytes - bytesRead, null, function (err, moreBytesRead) {
                    if (err) {
                        return end(err, null, 0);
                    }
                    // Retry up to N times in case 0 bytes where read
                    if (moreBytesRead === 0) {
                        if (++zeroAttempts === 10) {
                            return end(null, buffer, bytesRead);
                        }
                        return loop();
                    }
                    bytesRead += moreBytesRead;
                    if (bytesRead === totalBytes) {
                        return end(null, buffer, bytesRead);
                    }
                    return loop();
                });
            }
            loop();
        });
    }
    exports.readExactlyByFile = readExactlyByFile;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/encoding", ["require", "exports", 'vs/base/node/stream', 'iconv-lite'], function (require, exports, stream, iconv) {
    'use strict';
    exports.UTF8 = 'utf8';
    exports.UTF8_with_bom = 'utf8bom';
    exports.UTF16be = 'utf16be';
    exports.UTF16le = 'utf16le';
    function decode(buffer, encoding, options) {
        return iconv.decode(buffer, toNodeEncoding(encoding), options);
    }
    exports.decode = decode;
    function encode(content, encoding, options) {
        return iconv.encode(content, toNodeEncoding(encoding), options);
    }
    exports.encode = encode;
    function encodingExists(encoding) {
        return iconv.encodingExists(toNodeEncoding(encoding));
    }
    exports.encodingExists = encodingExists;
    function decodeStream(encoding) {
        return iconv.decodeStream(toNodeEncoding(encoding));
    }
    exports.decodeStream = decodeStream;
    function encodeStream(encoding) {
        return iconv.encodeStream(toNodeEncoding(encoding));
    }
    exports.encodeStream = encodeStream;
    function toNodeEncoding(enc) {
        if (enc === exports.UTF8_with_bom) {
            return exports.UTF8; // iconv does not distinguish UTF 8 with or without BOM, so we need to help it
        }
        return enc;
    }
    function detectEncodingByBOMFromBuffer(buffer, bytesRead) {
        if (!buffer || bytesRead < 2) {
            return null;
        }
        var b0 = buffer.readUInt8(0);
        var b1 = buffer.readUInt8(1);
        // UTF-16 BE
        if (b0 === 0xFE && b1 === 0xFF) {
            return exports.UTF16be;
        }
        // UTF-16 LE
        if (b0 === 0xFF && b1 === 0xFE) {
            return exports.UTF16le;
        }
        if (bytesRead < 3) {
            return null;
        }
        var b2 = buffer.readUInt8(2);
        // UTF-8
        if (b0 === 0xEF && b1 === 0xBB && b2 === 0xBF) {
            return exports.UTF8;
        }
        return null;
    }
    exports.detectEncodingByBOMFromBuffer = detectEncodingByBOMFromBuffer;
    /**
     * Detects the Byte Order Mark in a given file.
     * If no BOM is detected, `encoding` will be null.
     */
    function detectEncodingByBOM(file, callback) {
        stream.readExactlyByFile(file, 3, function (err, buffer, bytesRead) {
            if (err) {
                return callback(err, null);
            }
            return callback(null, detectEncodingByBOMFromBuffer(buffer, bytesRead));
        });
    }
    exports.detectEncodingByBOM = detectEncodingByBOM;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/mime", ["require", "exports", 'vs/base/common/mime', 'vs/base/node/stream', 'vs/base/node/encoding'], function (require, exports, mime, stream, encoding) {
    'use strict';
    /**
     * Lots of binary file types exists where the type can be determined by matching the first few bytes against some "magic patterns".
     * E.g. PDF files always start with %PDF- and the rest of the file contains mostly text, but sometimes binary data (for fonts and images).
     * In order to detect these types correctly (and independently from the file's extension), the content base mime type detection must be performed
     * on any file, not only on text files.
     *
     * Here is the original mime type detection in pseudocode:
     *
     * let mimes = [];
     *
     * read file extension
     *
     * if (file extension matches) {
     * 	if (file extension is bogus) {
     * 		// ignore.
     * 		// this covers *.manifest files which can contain arbitrary content, so the extension is of no value.
     * 		// a consequence of this is that the content based mime type becomes the most specific type in the array
     * 	} else {
     * 		mimes.push(associated mime type)	  // first element: most specific
     * 	}
     * }
     *
     * read file contents
     *
     * if (content based match found) {	// this is independent from text or binary
     * 	mimes.push(associated mime type)
     * 	if (a second mime exists for the match) {   // should be rare; text/plain should never be included here
     * 		// e.g. for svg: ['image/svg+xml', 'application/xml']
     * 		mimes.push(second mime)
     * 	}
     * }
     *
     * if (content == text)
     * 	mimes.push('text/plain')   // last element: least specific
     * else
     * 	mimes.push('application/octet-stream')    // last element: least specific
     */
    var BUFFER_READ_MAX_LEN = 512; // max buffer len to use when detecting encoding/mime
    function doDetectMimesFromStream(instream, callback) {
        stream.readExactlyByStream(instream, BUFFER_READ_MAX_LEN, function (err, buffer, bytesRead) {
            handleReadResult(err, buffer, bytesRead, callback);
        });
    }
    function doDetectMimesFromFile(absolutePath, callback) {
        stream.readExactlyByFile(absolutePath, BUFFER_READ_MAX_LEN, function (err, buffer, bytesRead) {
            handleReadResult(err, buffer, bytesRead, callback);
        });
    }
    function handleReadResult(err, buffer, bytesRead, callback) {
        if (err) {
            return callback(err, null);
        }
        return callback(null, detectMimeAndEncodingFromBuffer(buffer, bytesRead));
    }
    function detectMimeAndEncodingFromBuffer(buffer, bytesRead) {
        var enc = encoding.detectEncodingByBOMFromBuffer(buffer, bytesRead);
        // Detect 0 bytes to see if file is binary (ignore for UTF 16 though)
        var isText = true;
        if (enc !== encoding.UTF16be && enc !== encoding.UTF16le) {
            for (var i = 0; i < bytesRead && i < BUFFER_READ_MAX_LEN; i++) {
                if (buffer.readInt8(i) === 0) {
                    isText = false;
                    break;
                }
            }
        }
        return {
            mimes: isText ? [mime.MIME_TEXT] : [mime.MIME_BINARY],
            encoding: enc
        };
    }
    exports.detectMimeAndEncodingFromBuffer = detectMimeAndEncodingFromBuffer;
    function filterAndSortMimes(detectedMimes, guessedMimes) {
        var mimes = detectedMimes;
        // Add extension based mime as first element as this is the desire of whoever created the file.
        // Never care about application/octet-stream or application/unknown as guessed mime, as this is the fallback of the guess which is never accurate
        var guessedMime = guessedMimes[0];
        if (guessedMime !== mime.MIME_BINARY && guessedMime !== mime.MIME_UNKNOWN) {
            mimes.unshift(guessedMime);
        }
        // Remove duplicate elements from array and sort unspecific mime to the end
        var uniqueSortedMimes = mimes.filter(function (element, position) {
            return element && mimes.indexOf(element) === position;
        }).sort(function (mimeA, mimeB) {
            if (mimeA === mime.MIME_BINARY) {
                return 1;
            }
            if (mimeB === mime.MIME_BINARY) {
                return -1;
            }
            if (mimeA === mime.MIME_TEXT) {
                return 1;
            }
            if (mimeB === mime.MIME_TEXT) {
                return -1;
            }
            return 0;
        });
        return uniqueSortedMimes;
    }
    /**
     * Opens the given stream to detect its mime type. Returns an array of mime types sorted from most specific to unspecific.
     * @param instream the readable stream to detect the mime types from.
     * @param nameHint an additional hint that can be used to detect a mime from a file extension.
     */
    function detectMimesFromStream(instream, nameHint, callback) {
        doDetectMimesFromStream(instream, function (error, result) {
            handleMimeResult(nameHint, error, result, callback);
        });
    }
    exports.detectMimesFromStream = detectMimesFromStream;
    /**
     * Opens the given file to detect its mime type. Returns an array of mime types sorted from most specific to unspecific.
     * @param absolutePath the absolute path of the file.
     */
    function detectMimesFromFile(absolutePath, callback) {
        doDetectMimesFromFile(absolutePath, function (error, result) {
            handleMimeResult(absolutePath, error, result, callback);
        });
    }
    exports.detectMimesFromFile = detectMimesFromFile;
    function handleMimeResult(nameHint, error, result, callback) {
        if (error) {
            return callback(error, null);
        }
        var filterAndSortedMimes = filterAndSortMimes(result.mimes, mime.guessMimeTypes(nameHint));
        result.mimes = filterAndSortedMimes;
        callback(null, result);
    }
});

define("vs/base/parts/quickopen/common/quickOpen", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (Mode) {
        Mode[Mode["PREVIEW"] = 0] = "PREVIEW";
        Mode[Mode["OPEN"] = 1] = "OPEN";
    })(exports.Mode || (exports.Mode = {}));
    var Mode = exports.Mode;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/parts/tree/browser/tree", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ContextMenuEvent = (function () {
        function ContextMenuEvent(posx, posy, target) {
            this._posx = posx;
            this._posy = posy;
            this._target = target;
        }
        ContextMenuEvent.prototype.preventDefault = function () {
            // no-op
        };
        ContextMenuEvent.prototype.stopPropagation = function () {
            // no-op
        };
        Object.defineProperty(ContextMenuEvent.prototype, "posx", {
            get: function () {
                return this._posx;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContextMenuEvent.prototype, "posy", {
            get: function () {
                return this._posy;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContextMenuEvent.prototype, "target", {
            get: function () {
                return this._target;
            },
            enumerable: true,
            configurable: true
        });
        return ContextMenuEvent;
    }());
    exports.ContextMenuEvent = ContextMenuEvent;
    var MouseContextMenuEvent = (function (_super) {
        __extends(MouseContextMenuEvent, _super);
        function MouseContextMenuEvent(originalEvent) {
            _super.call(this, originalEvent.posx, originalEvent.posy, originalEvent.target);
            this.originalEvent = originalEvent;
        }
        MouseContextMenuEvent.prototype.preventDefault = function () {
            this.originalEvent.preventDefault();
        };
        MouseContextMenuEvent.prototype.stopPropagation = function () {
            this.originalEvent.stopPropagation();
        };
        return MouseContextMenuEvent;
    }(ContextMenuEvent));
    exports.MouseContextMenuEvent = MouseContextMenuEvent;
    var KeyboardContextMenuEvent = (function (_super) {
        __extends(KeyboardContextMenuEvent, _super);
        function KeyboardContextMenuEvent(posx, posy, originalEvent) {
            _super.call(this, posx, posy, originalEvent.target);
            this.originalEvent = originalEvent;
        }
        KeyboardContextMenuEvent.prototype.preventDefault = function () {
            this.originalEvent.preventDefault();
        };
        KeyboardContextMenuEvent.prototype.stopPropagation = function () {
            this.originalEvent.stopPropagation();
        };
        return KeyboardContextMenuEvent;
    }(ContextMenuEvent));
    exports.KeyboardContextMenuEvent = KeyboardContextMenuEvent;
    (function (DragOverEffect) {
        DragOverEffect[DragOverEffect["COPY"] = 0] = "COPY";
        DragOverEffect[DragOverEffect["MOVE"] = 1] = "MOVE";
    })(exports.DragOverEffect || (exports.DragOverEffect = {}));
    var DragOverEffect = exports.DragOverEffect;
    (function (DragOverBubble) {
        DragOverBubble[DragOverBubble["BUBBLE_DOWN"] = 0] = "BUBBLE_DOWN";
        DragOverBubble[DragOverBubble["BUBBLE_UP"] = 1] = "BUBBLE_UP";
    })(exports.DragOverBubble || (exports.DragOverBubble = {}));
    var DragOverBubble = exports.DragOverBubble;
    exports.DRAG_OVER_REJECT = { accept: false };
    exports.DRAG_OVER_ACCEPT = { accept: true };
    exports.DRAG_OVER_ACCEPT_BUBBLE_UP = { accept: true, bubble: DragOverBubble.BUBBLE_UP };
    exports.DRAG_OVER_ACCEPT_BUBBLE_DOWN = { accept: true, bubble: DragOverBubble.BUBBLE_DOWN };
    exports.DRAG_OVER_ACCEPT_BUBBLE_UP_COPY = { accept: true, bubble: DragOverBubble.BUBBLE_UP, effect: DragOverEffect.COPY };
    exports.DRAG_OVER_ACCEPT_BUBBLE_DOWN_COPY = { accept: true, bubble: DragOverBubble.BUBBLE_DOWN, effect: DragOverEffect.COPY };
});

define("vs/base/parts/tree/browser/treeDnd", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ElementsDragAndDropData = (function () {
        function ElementsDragAndDropData(elements) {
            this.elements = elements;
        }
        ElementsDragAndDropData.prototype.update = function (event) {
            // no-op
        };
        ElementsDragAndDropData.prototype.getData = function () {
            return this.elements;
        };
        return ElementsDragAndDropData;
    }());
    exports.ElementsDragAndDropData = ElementsDragAndDropData;
    var ExternalElementsDragAndDropData = (function () {
        function ExternalElementsDragAndDropData(elements) {
            this.elements = elements;
        }
        ExternalElementsDragAndDropData.prototype.update = function (event) {
            // no-op
        };
        ExternalElementsDragAndDropData.prototype.getData = function () {
            return this.elements;
        };
        return ExternalElementsDragAndDropData;
    }());
    exports.ExternalElementsDragAndDropData = ExternalElementsDragAndDropData;
    var DesktopDragAndDropData = (function () {
        function DesktopDragAndDropData() {
            this.types = [];
            this.files = [];
        }
        DesktopDragAndDropData.prototype.update = function (event) {
            if (event.dataTransfer.types) {
                this.types = [];
                Array.prototype.push.apply(this.types, event.dataTransfer.types);
            }
            if (event.dataTransfer.files) {
                this.files = [];
                Array.prototype.push.apply(this.files, event.dataTransfer.files);
                this.files = this.files.filter(function (f) { return f.size || f.type; });
            }
        };
        DesktopDragAndDropData.prototype.getData = function () {
            return {
                types: this.types,
                files: this.files
            };
        };
        return DesktopDragAndDropData;
    }());
    exports.DesktopDragAndDropData = DesktopDragAndDropData;
});

define("vs/css!vs/base/browser/builder",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/actionbar/actionbar",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/aria/aria",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/button/button",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/checkbox/checkbox",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/contextview/contextview",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/countBadge/countBadge",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/dropdown/dropdown",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/filelabel/filelabel",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/findinput/findInput",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/inputbox/inputBox",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/leftRightWidget/leftRightWidget",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/list/list",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/messagelist/messageList",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/octiconLabel/octicons/octicons",['vs/css!vs/workbench/workbench.main'], {});
define("vs/base/browser/ui/octiconLabel/octiconLabel", ["require", "exports", 'vs/base/common/strings', 'vs/css!./octicons/octicons'], function (require, exports, strings_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function expand(text) {
        return text.replace(/\$\(([^)]+)\)/g, function (match, g1) {
            return "<span class=\"octicon octicon-" + g1 + "\"></span>";
        });
    }
    exports.expand = expand;
    var OcticonLabel = (function () {
        function OcticonLabel(container) {
            this._container = container;
        }
        Object.defineProperty(OcticonLabel.prototype, "text", {
            set: function (text) {
                var innerHTML = text || '';
                innerHTML = strings_1.escape(innerHTML);
                innerHTML = expand(innerHTML);
                this._container.innerHTML = innerHTML;
            },
            enumerable: true,
            configurable: true
        });
        return OcticonLabel;
    }());
    exports.OcticonLabel = OcticonLabel;
});

define("vs/css!vs/base/browser/ui/progressbar/progressbar",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/resourceviewer/resourceviewer",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/sash/sash",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/scrollbar/media/scrollbars",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/splitview/splitview",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/timer/timer",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/browser/ui/toolbar/toolbar",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/parts/quickopen/browser/quickopen",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/parts/tree/browser/actionsRenderer",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/base/parts/tree/browser/tree",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/contentWidgets/contentWidgets",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/currentLineHighlight/currentLineHighlight",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/decorations/decorations",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/glyphMargin/glyphMargin",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/lineNumbers/lineNumbers",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/linesDecorations/linesDecorations",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/overlayWidgets/overlayWidgets",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/rulers/rulers",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/scrollDecoration/scrollDecoration",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/selections/selections",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/viewParts/viewCursors/viewCursors",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/widget/media/default-theme",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/widget/media/diffEditor",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/widget/media/editor",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/browser/widget/media/tokens",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/accessibility/browser/accessibility",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/clipboard/browser/clipboard",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/codelens/browser/codelens",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/color/browser/color",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/defineKeybinding/browser/defineKeybinding",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/find/browser/findWidget",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/folding/browser/folding",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/goToDeclaration/browser/goToDeclaration",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/gotoError/browser/gotoError",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/hover/browser/hover",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/iPadShowKeyboard/browser/iPadShowKeyboard",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/links/browser/links",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/outlineMarker/browser/outlineMarker",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/parameterHints/browser/parameterHints",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/quickFix/browser/quickFix",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/referenceSearch/browser/referenceSearchWidget",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/rename/browser/rename",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/snippet/browser/snippet",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/suggest/browser/suggest",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/workerStatusReporter/browser/workerStatusReporter",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/zoneWidget/browser/peekViewWidget",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/editor/contrib/zoneWidget/browser/zoneWidget",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/languages/css/common/css-hover",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/platform/keybinding/browser/keybindings",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/media/hc-black-theme",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/media/part",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/media/vs-dark-theme",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/media/vs-theme",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/media/workbench",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/activitybar/media/activityBarPart",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/activitybar/media/activityaction",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/editor/media/binarydiffeditor",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/editor/media/binaryeditor",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/editor/media/editorpart",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/editor/media/editorstatus",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/editor/media/iframeeditor",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/editor/media/sidebyside",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/editor/media/textdiffeditor",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/editor/media/texteditor",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/media/compositePart",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/panel/media/panelPart",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/quickopen/media/quickopen",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/sidebar/media/sidebarpart",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/browser/parts/statusbar/media/statusbarPart",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/electron-browser/media/shell",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/debug/browser/media/breakpointWidget",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/debug/browser/media/debug.contribution",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/debug/browser/media/debugHover",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/extensions/electron-browser/media/extensions",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/feedback/browser/media/feedback",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/files/browser/media/explorerviewlet",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/files/browser/media/fileactions",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/files/browser/media/files.contribution",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/git/browser/media/git.contribution",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/markdown/browser/media/markdownactions",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/output/browser/media/output.contribution",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/quickopen/browser/media/commandsHandler",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/quickopen/browser/media/gotoSymbolHandler",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/quickopen/browser/media/markerHandler",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/search/browser/media/search.contribution",['vs/css!vs/workbench/workbench.main'], {});
define("vs/css!vs/workbench/parts/tasks/electron-browser/media/task.contribution",['vs/css!vs/workbench/workbench.main'], {});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/config/elementSizeObserver", ["require", "exports", 'vs/base/common/lifecycle'], function (require, exports, lifecycle_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ElementSizeObserver = (function (_super) {
        __extends(ElementSizeObserver, _super);
        function ElementSizeObserver(referenceDomElement, changeCallback) {
            _super.call(this);
            this.referenceDomElement = referenceDomElement;
            this.changeCallback = changeCallback;
            this.measureReferenceDomElementToken = -1;
            this.width = -1;
            this.height = -1;
            this.measureReferenceDomElement(false);
        }
        ElementSizeObserver.prototype.dispose = function () {
            this.stopObserving();
            _super.prototype.dispose.call(this);
        };
        ElementSizeObserver.prototype.getWidth = function () {
            return this.width;
        };
        ElementSizeObserver.prototype.getHeight = function () {
            return this.height;
        };
        ElementSizeObserver.prototype.startObserving = function () {
            var _this = this;
            if (this.measureReferenceDomElementToken === -1) {
                this.measureReferenceDomElementToken = setInterval(function () { return _this.measureReferenceDomElement(true); }, 100);
            }
        };
        ElementSizeObserver.prototype.stopObserving = function () {
            if (this.measureReferenceDomElementToken !== -1) {
                clearInterval(this.measureReferenceDomElementToken);
                this.measureReferenceDomElementToken = -1;
            }
        };
        ElementSizeObserver.prototype.observe = function (dimension) {
            this.measureReferenceDomElement(true, dimension);
        };
        ElementSizeObserver.prototype.measureReferenceDomElement = function (callChangeCallback, dimension) {
            var observedWidth = 0;
            var observedHeight = 0;
            if (dimension) {
                observedWidth = dimension.width;
                observedHeight = dimension.height;
            }
            else if (this.referenceDomElement) {
                observedWidth = this.referenceDomElement.clientWidth;
                observedHeight = this.referenceDomElement.clientHeight;
            }
            observedWidth = Math.max(5, observedWidth);
            observedHeight = Math.max(5, observedHeight);
            if (this.width !== observedWidth || this.height !== observedHeight) {
                this.width = observedWidth;
                this.height = observedHeight;
                if (callChangeCallback) {
                    this.changeCallback();
                }
            }
        };
        return ElementSizeObserver;
    }(lifecycle_1.Disposable));
    exports.ElementSizeObserver = ElementSizeObserver;
});

define("vs/editor/browser/editorBrowser", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.ClassNames = {
        TEXTAREA_COVER: 'textAreaCover',
        TEXTAREA: 'inputarea',
        LINES_CONTENT: 'lines-content',
        OVERFLOW_GUARD: 'overflow-guard',
        VIEW_LINES: 'view-lines',
        VIEW_LINE: 'view-line',
        SCROLLABLE_ELEMENT: 'editor-scrollable',
        CONTENT_WIDGETS: 'contentWidgets',
        OVERFLOWING_CONTENT_WIDGETS: 'overflowingContentWidgets',
        OVERLAY_WIDGETS: 'overlayWidgets',
        MARGIN_VIEW_OVERLAYS: 'margin-view-overlays',
        LINE_NUMBERS: 'line-numbers',
        GLYPH_MARGIN: 'glyph-margin',
        SCROLL_DECORATION: 'scroll-decoration',
        VIEW_CURSORS_LAYER: 'cursors-layer',
        VIEW_ZONES: 'view-zones'
    };
    /**
     * A positioning preference for rendering content widgets.
     */
    (function (ContentWidgetPositionPreference) {
        /**
         * Place the content widget exactly at a position
         */
        ContentWidgetPositionPreference[ContentWidgetPositionPreference["EXACT"] = 0] = "EXACT";
        /**
         * Place the content widget above a position
         */
        ContentWidgetPositionPreference[ContentWidgetPositionPreference["ABOVE"] = 1] = "ABOVE";
        /**
         * Place the content widget below a position
         */
        ContentWidgetPositionPreference[ContentWidgetPositionPreference["BELOW"] = 2] = "BELOW";
    })(exports.ContentWidgetPositionPreference || (exports.ContentWidgetPositionPreference = {}));
    var ContentWidgetPositionPreference = exports.ContentWidgetPositionPreference;
    /**
     * A positioning preference for rendering overlay widgets.
     */
    (function (OverlayWidgetPositionPreference) {
        /**
         * Position the overlay widget in the top right corner
         */
        OverlayWidgetPositionPreference[OverlayWidgetPositionPreference["TOP_RIGHT_CORNER"] = 0] = "TOP_RIGHT_CORNER";
        /**
         * Position the overlay widget in the bottom right corner
         */
        OverlayWidgetPositionPreference[OverlayWidgetPositionPreference["BOTTOM_RIGHT_CORNER"] = 1] = "BOTTOM_RIGHT_CORNER";
        /**
         * Position the overlay widget in the top center
         */
        OverlayWidgetPositionPreference[OverlayWidgetPositionPreference["TOP_CENTER"] = 2] = "TOP_CENTER";
    })(exports.OverlayWidgetPositionPreference || (exports.OverlayWidgetPositionPreference = {}));
    var OverlayWidgetPositionPreference = exports.OverlayWidgetPositionPreference;
});

define("vs/editor/browser/viewParts/viewCursors/viewCursor", ["require", "exports", 'vs/base/browser/styleMutator'], function (require, exports, styleMutator_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewCursor = (function () {
        function ViewCursor(context, isSecondary) {
            this._context = context;
            this._isInEditableRange = true;
            this._domNode = this._createCursorDomNode(isSecondary);
            this._isVisible = true;
            styleMutator_1.StyleMutator.setDisplay(this._domNode, 'none');
            this.updatePosition({
                lineNumber: 1,
                column: 1
            });
        }
        ViewCursor.prototype._createCursorDomNode = function (isSecondary) {
            var domNode = document.createElement('div');
            domNode.className = 'cursor';
            if (isSecondary) {
                domNode.className += ' secondary';
            }
            styleMutator_1.StyleMutator.setHeight(domNode, this._context.configuration.editor.lineHeight);
            styleMutator_1.StyleMutator.setTop(domNode, 0);
            styleMutator_1.StyleMutator.setLeft(domNode, 0);
            domNode.setAttribute('role', 'presentation');
            domNode.setAttribute('aria-hidden', 'true');
            return domNode;
        };
        ViewCursor.prototype.getDomNode = function () {
            return this._domNode;
        };
        ViewCursor.prototype.getIsInEditableRange = function () {
            return this._isInEditableRange;
        };
        ViewCursor.prototype.getPositionTop = function () {
            return this._positionTop;
        };
        ViewCursor.prototype.getPosition = function () {
            return this._position;
        };
        ViewCursor.prototype.show = function () {
            if (!this._isVisible) {
                styleMutator_1.StyleMutator.setVisibility(this._domNode, 'inherit');
                this._isVisible = true;
            }
        };
        ViewCursor.prototype.hide = function () {
            if (this._isVisible) {
                styleMutator_1.StyleMutator.setVisibility(this._domNode, 'hidden');
                this._isVisible = false;
            }
        };
        ViewCursor.prototype.onModelFlushed = function () {
            this.updatePosition({
                lineNumber: 1,
                column: 1
            });
            this._isInEditableRange = true;
            return true;
        };
        ViewCursor.prototype.onCursorPositionChanged = function (position, isInEditableRange) {
            this.updatePosition(position);
            this._isInEditableRange = isInEditableRange;
            return true;
        };
        ViewCursor.prototype.onConfigurationChanged = function (e) {
            if (e.lineHeight) {
                styleMutator_1.StyleMutator.setHeight(this._domNode, this._context.configuration.editor.lineHeight);
            }
            return true;
        };
        ViewCursor.prototype.prepareRender = function (ctx) {
            var visibleRange = ctx.visibleRangeForPosition(this._position);
            if (visibleRange) {
                this._positionTop = visibleRange.top;
                this._positionLeft = visibleRange.left;
                this._isInViewport = true;
            }
            else {
                this._isInViewport = false;
            }
        };
        ViewCursor.prototype.render = function (ctx) {
            if (this._isInViewport) {
                styleMutator_1.StyleMutator.setDisplay(this._domNode, 'block');
                styleMutator_1.StyleMutator.setLeft(this._domNode, this._positionLeft);
                styleMutator_1.StyleMutator.setTop(this._domNode, this._positionTop + ctx.viewportTop - ctx.bigNumbersDelta);
            }
            else {
                styleMutator_1.StyleMutator.setDisplay(this._domNode, 'none');
            }
        };
        ViewCursor.prototype.updatePosition = function (newPosition) {
            this._position = newPosition;
            this._domNode.setAttribute('lineNumber', this._position.lineNumber.toString());
            this._domNode.setAttribute('column', this._position.column.toString());
            this._isInViewport = false;
        };
        return ViewCursor;
    }());
    exports.ViewCursor = ViewCursor;
});

define("vs/editor/common/controller/handlerDispatcher", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DispatcherEvent = (function () {
        function DispatcherEvent(source, data) {
            this.source = source;
            this.data = data;
        }
        DispatcherEvent.prototype.getSource = function () {
            return this.source;
        };
        DispatcherEvent.prototype.getData = function () {
            return this.data;
        };
        return DispatcherEvent;
    }());
    exports.DispatcherEvent = DispatcherEvent;
    var HandlerDispatcher = (function () {
        function HandlerDispatcher() {
            this.registry = {};
        }
        HandlerDispatcher.prototype.setHandler = function (handlerId, handlerCallback) {
            this.registry[handlerId] = handlerCallback;
        };
        HandlerDispatcher.prototype.clearHandlers = function () {
            this.registry = {};
        };
        HandlerDispatcher.prototype.getHandler = function (handlerId) {
            return this.registry.hasOwnProperty(handlerId) ? this.registry[handlerId] : null;
        };
        HandlerDispatcher.prototype.trigger = function (source, handlerId, payload) {
            var handler = this.getHandler(handlerId);
            var handled = false;
            if (handler) {
                var e = new DispatcherEvent(source, payload);
                handled = handler(e);
            }
            return handled;
        };
        return HandlerDispatcher;
    }());
    exports.HandlerDispatcher = HandlerDispatcher;
});

define("vs/editor/common/core/arrays", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Arrays;
    (function (Arrays) {
        /**
         * Given a sorted array of natural number segments, find the segment containing a natural number.
         *    For example, the segments [0, 5), [5, 9), [9, infinity) will be represented in the following manner:
         *       [{ startIndex: 0 }, { startIndex: 5 }, { startIndex: 9 }]
         *    Searching for 0, 1, 2, 3 or 4 will return 0.
         *    Searching for 5, 6, 7 or 8 will return 1.
         *    Searching for 9, 10, 11, ... will return 2.
         * @param arr A sorted array representing natural number segments
         * @param desiredIndex The search
         * @return The index of the containing segment in the array.
         */
        function findIndexInSegmentsArray(arr, desiredIndex) {
            var low = 0, high = arr.length - 1, mid;
            while (low < high) {
                mid = low + Math.ceil((high - low) / 2);
                if (arr[mid].startIndex > desiredIndex) {
                    high = mid - 1;
                }
                else {
                    low = mid;
                }
            }
            return low;
        }
        Arrays.findIndexInSegmentsArray = findIndexInSegmentsArray;
    })(Arrays = exports.Arrays || (exports.Arrays = {}));
});

define("vs/editor/common/core/idGenerator", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var IdGenerator = (function () {
        function IdGenerator(prefix) {
            this._prefix = prefix;
            this._lastId = 0;
        }
        IdGenerator.prototype.generate = function () {
            return this._prefix + (++this._lastId);
        };
        return IdGenerator;
    }());
    exports.IdGenerator = IdGenerator;
});

define("vs/editor/common/core/position", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Position = (function () {
        function Position(lineNumber, column) {
            this.lineNumber = lineNumber;
            this.column = column;
        }
        Position.prototype.equals = function (other) {
            return Position.equals(this, other);
        };
        Position.equals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            return (!!a &&
                !!b &&
                a.lineNumber === b.lineNumber &&
                a.column === b.column);
        };
        Position.prototype.isBefore = function (other) {
            return Position.isBefore(this, other);
        };
        Position.isBefore = function (a, b) {
            if (a.lineNumber < b.lineNumber) {
                return true;
            }
            if (b.lineNumber < a.lineNumber) {
                return false;
            }
            return a.column < b.column;
        };
        Position.prototype.isBeforeOrEqual = function (other) {
            return Position.isBeforeOrEqual(this, other);
        };
        Position.isBeforeOrEqual = function (a, b) {
            if (a.lineNumber < b.lineNumber) {
                return true;
            }
            if (b.lineNumber < a.lineNumber) {
                return false;
            }
            return a.column <= b.column;
        };
        Position.prototype.clone = function () {
            return new Position(this.lineNumber, this.column);
        };
        Position.prototype.toString = function () {
            return '(' + this.lineNumber + ',' + this.column + ')';
        };
        // ---
        Position.lift = function (pos) {
            return new Position(pos.lineNumber, pos.column);
        };
        Position.isIPosition = function (obj) {
            return (obj
                && (typeof obj.lineNumber === 'number')
                && (typeof obj.column === 'number'));
        };
        Position.asEmptyRange = function (position) {
            return {
                startLineNumber: position.lineNumber,
                startColumn: position.column,
                endLineNumber: position.lineNumber,
                endColumn: position.column
            };
        };
        Position.startPosition = function (range) {
            return {
                lineNumber: range.startLineNumber,
                column: range.startColumn
            };
        };
        Position.endPosition = function (range) {
            return {
                lineNumber: range.endLineNumber,
                column: range.endColumn
            };
        };
        return Position;
    }());
    exports.Position = Position;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/common/core/range", ["require", "exports", 'vs/editor/common/core/position'], function (require, exports, position_1) {
    'use strict';
    var Range = (function () {
        function Range(startLineNumber, startColumn, endLineNumber, endColumn) {
            if ((startLineNumber > endLineNumber) || (startLineNumber === endLineNumber && startColumn > endColumn)) {
                this.startLineNumber = endLineNumber;
                this.startColumn = endColumn;
                this.endLineNumber = startLineNumber;
                this.endColumn = startColumn;
            }
            else {
                this.startLineNumber = startLineNumber;
                this.startColumn = startColumn;
                this.endLineNumber = endLineNumber;
                this.endColumn = endColumn;
            }
        }
        Range.prototype.isEmpty = function () {
            return Range.isEmpty(this);
        };
        Range.prototype.containsPosition = function (position) {
            return Range.containsPosition(this, position);
        };
        Range.prototype.containsRange = function (range) {
            return Range.containsRange(this, range);
        };
        Range.prototype.plusRange = function (range) {
            return Range.plusRange(this, range);
        };
        Range.prototype.intersectRanges = function (range) {
            return Range.intersectRanges(this, range);
        };
        Range.prototype.equalsRange = function (other) {
            return Range.equalsRange(this, other);
        };
        Range.prototype.getEndPosition = function () {
            return new position_1.Position(this.endLineNumber, this.endColumn);
        };
        Range.prototype.getStartPosition = function () {
            return new position_1.Position(this.startLineNumber, this.startColumn);
        };
        Range.prototype.cloneRange = function () {
            return new Range(this.startLineNumber, this.startColumn, this.endLineNumber, this.endColumn);
        };
        Range.prototype.toString = function () {
            return '[' + this.startLineNumber + ',' + this.startColumn + ' -> ' + this.endLineNumber + ',' + this.endColumn + ']';
        };
        Range.prototype.setEndPosition = function (endLineNumber, endColumn) {
            return new Range(this.startLineNumber, this.startColumn, endLineNumber, endColumn);
        };
        Range.prototype.setStartPosition = function (startLineNumber, startColumn) {
            return new Range(startLineNumber, startColumn, this.endLineNumber, this.endColumn);
        };
        Range.prototype.collapseToStart = function () {
            return new Range(this.startLineNumber, this.startColumn, this.startLineNumber, this.startColumn);
        };
        // ---
        Range.lift = function (range) {
            if (!range) {
                return null;
            }
            return new Range(range.startLineNumber, range.startColumn, range.endLineNumber, range.endColumn);
        };
        Range.isIRange = function (obj) {
            return (obj
                && (typeof obj.startLineNumber === 'number')
                && (typeof obj.startColumn === 'number')
                && (typeof obj.endLineNumber === 'number')
                && (typeof obj.endColumn === 'number'));
        };
        Range.isEmpty = function (range) {
            return (range.startLineNumber === range.endLineNumber && range.startColumn === range.endColumn);
        };
        Range.containsPosition = function (range, position) {
            if (position.lineNumber < range.startLineNumber || position.lineNumber > range.endLineNumber) {
                return false;
            }
            if (position.lineNumber === range.startLineNumber && position.column < range.startColumn) {
                return false;
            }
            if (position.lineNumber === range.endLineNumber && position.column > range.endColumn) {
                return false;
            }
            return true;
        };
        Range.containsRange = function (range, otherRange) {
            if (otherRange.startLineNumber < range.startLineNumber || otherRange.endLineNumber < range.startLineNumber) {
                return false;
            }
            if (otherRange.startLineNumber > range.endLineNumber || otherRange.endLineNumber > range.endLineNumber) {
                return false;
            }
            if (otherRange.startLineNumber === range.startLineNumber && otherRange.startColumn < range.startColumn) {
                return false;
            }
            if (otherRange.endLineNumber === range.endLineNumber && otherRange.endColumn > range.endColumn) {
                return false;
            }
            return true;
        };
        Range.areIntersectingOrTouching = function (a, b) {
            // Check if `a` is before `b`
            if (a.endLineNumber < b.startLineNumber || (a.endLineNumber === b.startLineNumber && a.endColumn < b.startColumn)) {
                return false;
            }
            // Check if `b` is before `a`
            if (b.endLineNumber < a.startLineNumber || (b.endLineNumber === a.startLineNumber && b.endColumn < a.startColumn)) {
                return false;
            }
            // These ranges must intersect
            return true;
        };
        Range.intersectRanges = function (a, b) {
            var resultStartLineNumber = a.startLineNumber, resultStartColumn = a.startColumn, resultEndLineNumber = a.endLineNumber, resultEndColumn = a.endColumn, otherStartLineNumber = b.startLineNumber, otherStartColumn = b.startColumn, otherEndLineNumber = b.endLineNumber, otherEndColumn = b.endColumn;
            if (resultStartLineNumber < otherStartLineNumber) {
                resultStartLineNumber = otherStartLineNumber;
                resultStartColumn = otherStartColumn;
            }
            else if (resultStartLineNumber === otherStartLineNumber) {
                resultStartColumn = Math.max(resultStartColumn, otherStartColumn);
            }
            if (resultEndLineNumber > otherEndLineNumber) {
                resultEndLineNumber = otherEndLineNumber;
                resultEndColumn = otherEndColumn;
            }
            else if (resultEndLineNumber === otherEndLineNumber) {
                resultEndColumn = Math.min(resultEndColumn, otherEndColumn);
            }
            // Check if selection is now empty
            if (resultStartLineNumber > resultEndLineNumber) {
                return null;
            }
            if (resultStartLineNumber === resultEndLineNumber && resultStartColumn > resultEndColumn) {
                return null;
            }
            return new Range(resultStartLineNumber, resultStartColumn, resultEndLineNumber, resultEndColumn);
        };
        Range.plusRange = function (a, b) {
            var startLineNumber, startColumn, endLineNumber, endColumn;
            if (b.startLineNumber < a.startLineNumber) {
                startLineNumber = b.startLineNumber;
                startColumn = b.startColumn;
            }
            else if (b.startLineNumber === a.startLineNumber) {
                startLineNumber = b.startLineNumber;
                startColumn = Math.min(b.startColumn, a.startColumn);
            }
            else {
                startLineNumber = a.startLineNumber;
                startColumn = a.startColumn;
            }
            if (b.endLineNumber > a.endLineNumber) {
                endLineNumber = b.endLineNumber;
                endColumn = b.endColumn;
            }
            else if (b.endLineNumber === a.endLineNumber) {
                endLineNumber = b.endLineNumber;
                endColumn = Math.max(b.endColumn, a.endColumn);
            }
            else {
                endLineNumber = a.endLineNumber;
                endColumn = a.endColumn;
            }
            return new Range(startLineNumber, startColumn, endLineNumber, endColumn);
        };
        Range.equalsRange = function (a, b) {
            return (!!a &&
                !!b &&
                a.startLineNumber === b.startLineNumber &&
                a.startColumn === b.startColumn &&
                a.endLineNumber === b.endLineNumber &&
                a.endColumn === b.endColumn);
        };
        /**
         * A function that compares ranges, useful for sorting ranges
         * It will first compare ranges on the startPosition and then on the endPosition
         */
        Range.compareRangesUsingStarts = function (a, b) {
            if (a.startLineNumber === b.startLineNumber) {
                if (a.startColumn === b.startColumn) {
                    if (a.endLineNumber === b.endLineNumber) {
                        return a.endColumn - b.endColumn;
                    }
                    return a.endLineNumber - b.endLineNumber;
                }
                return a.startColumn - b.startColumn;
            }
            return a.startLineNumber - b.startLineNumber;
        };
        /**
         * A function that compares ranges, useful for sorting ranges
         * It will first compare ranges on the endPosition and then on the startPosition
         */
        Range.compareRangesUsingEnds = function (a, b) {
            if (a.endLineNumber === b.endLineNumber) {
                if (a.endColumn === b.endColumn) {
                    if (a.startLineNumber === b.startLineNumber) {
                        return a.startColumn - b.startColumn;
                    }
                    return a.startLineNumber - b.startLineNumber;
                }
                return a.endColumn - b.endColumn;
            }
            return a.endLineNumber - b.endLineNumber;
        };
        Range.spansMultipleLines = function (range) {
            return range.endLineNumber > range.startLineNumber;
        };
        Range.collapseToStart = function (range) {
            return {
                startLineNumber: range.startLineNumber,
                startColumn: range.startColumn,
                endLineNumber: range.startLineNumber,
                endColumn: range.startColumn
            };
        };
        return Range;
    }());
    exports.Range = Range;
});

define("vs/editor/common/core/editOperation", ["require", "exports", 'vs/editor/common/core/range'], function (require, exports, range_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EditOperation = (function () {
        function EditOperation() {
        }
        EditOperation.insert = function (position, text) {
            return {
                identifier: null,
                range: new range_1.Range(position.lineNumber, position.column, position.lineNumber, position.column),
                text: text,
                forceMoveMarkers: true
            };
        };
        EditOperation.delete = function (range) {
            return {
                identifier: null,
                range: range,
                text: null,
                forceMoveMarkers: true
            };
        };
        EditOperation.replace = function (range, text) {
            return {
                identifier: null,
                range: range,
                text: text,
                forceMoveMarkers: false
            };
        };
        EditOperation.replaceMove = function (range, text) {
            return {
                identifier: null,
                range: range,
                text: text,
                forceMoveMarkers: true
            };
        };
        return EditOperation;
    }());
    exports.EditOperation = EditOperation;
});

define("vs/editor/common/commands/trimTrailingWhitespaceCommand", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/core/editOperation', 'vs/editor/common/core/range'], function (require, exports, strings, editOperation_1, range_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var TrimTrailingWhitespaceCommand = (function () {
        function TrimTrailingWhitespaceCommand(selection) {
            this.selection = selection;
        }
        TrimTrailingWhitespaceCommand.prototype.getEditOperations = function (model, builder) {
            var ops = trimTrailingWhitespace(model, []);
            for (var i = 0, len = ops.length; i < len; i++) {
                var op = ops[i];
                builder.addEditOperation(op.range, op.text);
            }
            this.selectionId = builder.trackSelection(this.selection);
        };
        TrimTrailingWhitespaceCommand.prototype.computeCursorState = function (model, helper) {
            return helper.getTrackedSelection(this.selectionId);
        };
        return TrimTrailingWhitespaceCommand;
    }());
    exports.TrimTrailingWhitespaceCommand = TrimTrailingWhitespaceCommand;
    /**
     * Generate commands for trimming trailing whitespace on a model and ignore lines on which cursors are sitting.
     */
    function trimTrailingWhitespace(model, cursors) {
        // Sort cursors ascending
        cursors.sort(function (a, b) {
            if (a.lineNumber === b.lineNumber) {
                return a.column - b.column;
            }
            return a.lineNumber - b.lineNumber;
        });
        // Reduce multiple cursors on the same line and only keep the last one on the line
        for (var i = cursors.length - 2; i >= 0; i--) {
            if (cursors[i].lineNumber === cursors[i + 1].lineNumber) {
                // Remove cursor at `i`
                cursors.splice(i, 1);
            }
        }
        var r = [], cursorIndex = 0, cursorLen = cursors.length, lineNumber, lineCount, lineContent, minEditColumn, maxLineColumn, fromColumn, 
        // toColumn:number,
        lastNonWhitespaceIndex;
        for (lineNumber = 1, lineCount = model.getLineCount(); lineNumber <= lineCount; lineNumber++) {
            lineContent = model.getLineContent(lineNumber);
            maxLineColumn = lineContent.length + 1;
            minEditColumn = 0;
            if (cursorIndex < cursorLen && cursors[cursorIndex].lineNumber === lineNumber) {
                minEditColumn = cursors[cursorIndex].column;
                cursorIndex++;
                if (minEditColumn === maxLineColumn) {
                    // The cursor is at the end of the line => no edits for sure on this line
                    continue;
                }
            }
            if (lineContent.length === 0) {
                continue;
            }
            lastNonWhitespaceIndex = strings.lastNonWhitespaceIndex(lineContent);
            fromColumn = 0;
            if (lastNonWhitespaceIndex === -1) {
                // Entire line is whitespace
                fromColumn = 1;
            }
            else if (lastNonWhitespaceIndex !== lineContent.length - 1) {
                // There is trailing whitespace
                fromColumn = lastNonWhitespaceIndex + 2;
            }
            else {
                // There is no trailing whitespace
                continue;
            }
            fromColumn = Math.max(minEditColumn, fromColumn);
            r.push(editOperation_1.EditOperation.delete(new range_1.Range(lineNumber, fromColumn, lineNumber, maxLineColumn)));
        }
        return r;
    }
    exports.trimTrailingWhitespace = trimTrailingWhitespace;
});

define("vs/editor/common/model/indentationGuesser", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var __space = ' '.charCodeAt(0);
    var __tab = '\t'.charCodeAt(0);
    /**
     * Compute the diff in spaces between two line's indentation.
     */
    function spacesDiff(a, aLength, b, bLength) {
        // This can go both ways (e.g.):
        //  - a: "\t"
        //  - b: "\t    "
        //  => This should count 1 tab and 4 spaces
        var i;
        for (i = 0; i < aLength && i < bLength; i++) {
            var aCharCode = a.charCodeAt(i);
            var bCharCode = b.charCodeAt(i);
            if (aCharCode !== bCharCode) {
                break;
            }
        }
        var aSpacesCnt = 0, aTabsCount = 0;
        for (var j = i; j < aLength; j++) {
            var aCharCode = a.charCodeAt(j);
            if (aCharCode === __space) {
                aSpacesCnt++;
            }
            else {
                aTabsCount++;
            }
        }
        var bSpacesCnt = 0, bTabsCount = 0;
        for (var j = i; j < bLength; j++) {
            var bCharCode = b.charCodeAt(j);
            if (bCharCode === __space) {
                bSpacesCnt++;
            }
            else {
                bTabsCount++;
            }
        }
        if (aSpacesCnt > 0 && aTabsCount > 0) {
            return 0;
        }
        if (bSpacesCnt > 0 && bTabsCount > 0) {
            return 0;
        }
        var tabsDiff = Math.abs(aTabsCount - bTabsCount);
        var spacesDiff = Math.abs(aSpacesCnt - bSpacesCnt);
        if (tabsDiff === 0) {
            return spacesDiff;
        }
        if (spacesDiff % tabsDiff === 0) {
            return spacesDiff / tabsDiff;
        }
        return 0;
    }
    function guessIndentation(lines, defaultTabSize, defaultInsertSpaces) {
        var linesIndentedWithTabsCount = 0; // number of lines that contain at least one tab in indentation
        var linesIndentedWithSpacesCount = 0; // number of lines that contain only spaces in indentation
        var previousLineText = ''; // content of latest line that contained non-whitespace chars
        var previousLineIndentation = 0; // index at which latest line contained the first non-whitespace char
        var ALLOWED_TAB_SIZE_GUESSES = [2, 4, 6, 8]; // limit guesses for `tabSize` to 2, 4, 6 or 8.
        var MAX_ALLOWED_TAB_SIZE_GUESS = 8; // max(2,4,6,8) = 8
        var spacesDiffCount = [0, 0, 0, 0, 0, 0, 0, 0, 0]; // `tabSize` scores
        for (var i = 0, len = lines.length; i < len; i++) {
            var currentLineText = lines[i];
            var currentLineHasContent = false; // does `currentLineText` contain non-whitespace chars
            var currentLineIndentation = 0; // index at which `currentLineText` contains the first non-whitespace char
            var currentLineSpacesCount = 0; // count of spaces found in `currentLineText` indentation
            var currentLineTabsCount = 0; // count of tabs found in `currentLineText` indentation
            for (var j = 0, lenJ = currentLineText.length; j < lenJ; j++) {
                var charCode = currentLineText.charCodeAt(j);
                if (charCode === __tab) {
                    currentLineTabsCount++;
                }
                else if (charCode === __space) {
                    currentLineSpacesCount++;
                }
                else {
                    // Hit non whitespace character on this line
                    currentLineHasContent = true;
                    currentLineIndentation = j;
                    break;
                }
            }
            // Ignore empty or only whitespace lines
            if (!currentLineHasContent) {
                continue;
            }
            if (currentLineTabsCount > 0) {
                linesIndentedWithTabsCount++;
            }
            else if (currentLineSpacesCount > 1) {
                linesIndentedWithSpacesCount++;
            }
            var currentSpacesDiff = spacesDiff(previousLineText, previousLineIndentation, currentLineText, currentLineIndentation);
            if (currentSpacesDiff <= MAX_ALLOWED_TAB_SIZE_GUESS) {
                spacesDiffCount[currentSpacesDiff]++;
            }
            previousLineText = currentLineText;
            previousLineIndentation = currentLineIndentation;
        }
        // Take into account the last line as well
        var deltaSpacesCount = spacesDiff(previousLineText, previousLineIndentation, '', 0);
        if (deltaSpacesCount <= MAX_ALLOWED_TAB_SIZE_GUESS) {
            spacesDiffCount[deltaSpacesCount]++;
        }
        var insertSpaces = defaultInsertSpaces;
        if (linesIndentedWithTabsCount !== linesIndentedWithSpacesCount) {
            insertSpaces = (linesIndentedWithTabsCount < linesIndentedWithSpacesCount);
        }
        var tabSize = defaultTabSize;
        var tabSizeScore = (insertSpaces ? 0 : 0.1 * lines.length);
        // console.log("score threshold: " + tabSizeScore);
        ALLOWED_TAB_SIZE_GUESSES.forEach(function (possibleTabSize) {
            var possibleTabSizeScore = spacesDiffCount[possibleTabSize];
            if (possibleTabSizeScore > tabSizeScore) {
                tabSizeScore = possibleTabSizeScore;
                tabSize = possibleTabSize;
            }
        });
        // console.log('--------------------------');
        // console.log('linesIndentedWithTabsCount: ' + linesIndentedWithTabsCount + ', linesIndentedWithSpacesCount: ' + linesIndentedWithSpacesCount);
        // console.log('spacesDiffCount: ' + spacesDiffCount);
        // console.log('tabSize: ' + tabSize + ', tabSizeScore: ' + tabSizeScore);
        return {
            insertSpaces: insertSpaces,
            tabSize: tabSize
        };
    }
    exports.guessIndentation = guessIndentation;
});

define("vs/editor/common/modes", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (IndentAction) {
        IndentAction[IndentAction["None"] = 0] = "None";
        IndentAction[IndentAction["Indent"] = 1] = "Indent";
        IndentAction[IndentAction["IndentOutdent"] = 2] = "IndentOutdent";
        IndentAction[IndentAction["Outdent"] = 3] = "Outdent";
    })(exports.IndentAction || (exports.IndentAction = {}));
    var IndentAction = exports.IndentAction;
});

define("vs/editor/common/modes/abstractState", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var AbstractState = (function () {
        function AbstractState(mode, stateData) {
            if (stateData === void 0) { stateData = null; }
            this.mode = mode;
            this.stateData = stateData;
        }
        AbstractState.prototype.getMode = function () {
            return this.mode;
        };
        AbstractState.prototype.clone = function () {
            var result = this.makeClone();
            result.initializeFrom(this);
            return result;
        };
        AbstractState.prototype.makeClone = function () {
            throw new Error('Abstract Method');
        };
        AbstractState.prototype.initializeFrom = function (other) {
            this.stateData = other.stateData !== null ? other.stateData.clone() : null;
        };
        AbstractState.prototype.getStateData = function () {
            return this.stateData;
        };
        AbstractState.prototype.setStateData = function (state) {
            this.stateData = state;
        };
        AbstractState.prototype.equals = function (other) {
            if (other === null || this.mode !== other.getMode()) {
                return false;
            }
            if (other instanceof AbstractState) {
                return AbstractState.safeEquals(this.stateData, other.stateData);
            }
            return false;
        };
        AbstractState.prototype.tokenize = function (stream) {
            throw new Error('Abstract Method');
        };
        AbstractState.safeEquals = function (a, b) {
            if (a === null && b === null) {
                return true;
            }
            if (a === null || b === null) {
                return false;
            }
            return a.equals(b);
        };
        AbstractState.safeClone = function (state) {
            if (state) {
                return state.clone();
            }
            return null;
        };
        return AbstractState;
    }());
    exports.AbstractState = AbstractState;
});

define("vs/editor/common/modes/TMState", ["require", "exports", 'vs/editor/common/modes/abstractState'], function (require, exports, abstractState_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function stackElementEquals(a, b) {
        if (!a && !b) {
            return true;
        }
        if (!a || !b) {
            return false;
        }
        // Not comparing enterPos since it does not represent state across lines
        return (a.ruleId === b.ruleId
            && a.endRule === b.endRule
            && a.scopeName === b.scopeName
            && a.contentName === b.contentName);
    }
    var TMState = (function () {
        function TMState(mode, parentEmbedderState, ruleStack) {
            this._mode = mode;
            this._parentEmbedderState = parentEmbedderState;
            this._ruleStack = ruleStack;
        }
        TMState.prototype.clone = function () {
            var parentEmbedderStateClone = abstractState_1.AbstractState.safeClone(this._parentEmbedderState);
            var ruleStackClone = this._ruleStack ? this._ruleStack.map(function (el) { return el.clone(); }) : null;
            return new TMState(this._mode, parentEmbedderStateClone, ruleStackClone);
        };
        TMState.prototype.equals = function (other) {
            if (!other || !(other instanceof TMState)) {
                return false;
            }
            var otherState = other;
            // Equals on `_parentEmbedderState`
            if (!abstractState_1.AbstractState.safeEquals(this._parentEmbedderState, otherState._parentEmbedderState)) {
                return false;
            }
            // Equals on `_ruleStack`
            if (this._ruleStack === null && otherState._ruleStack === null) {
                return true;
            }
            if (this._ruleStack === null || otherState._ruleStack === null) {
                return false;
            }
            if (this._ruleStack.length !== otherState._ruleStack.length) {
                return false;
            }
            for (var i = 0, len = this._ruleStack.length; i < len; i++) {
                if (!stackElementEquals(this._ruleStack[i], otherState._ruleStack[i])) {
                    return false;
                }
            }
            return true;
        };
        TMState.prototype.getMode = function () {
            return this._mode;
        };
        TMState.prototype.tokenize = function (stream) {
            throw new Error();
        };
        TMState.prototype.getStateData = function () {
            return this._parentEmbedderState;
        };
        TMState.prototype.setStateData = function (state) {
            this._parentEmbedderState = state;
        };
        TMState.prototype.getRuleStack = function () {
            return this._ruleStack;
        };
        TMState.prototype.setRuleStack = function (ruleStack) {
            this._ruleStack = ruleStack;
        };
        return TMState;
    }());
    exports.TMState = TMState;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/common/modes/languageSelector", ["require", "exports", 'vs/base/common/glob'], function (require, exports, glob_1) {
    'use strict';
    function matches(selection, uri, language) {
        return score(selection, uri, language) > 0;
    }
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = matches;
    function score(selector, uri, language) {
        if (Array.isArray(selector)) {
            // for each
            var values = selector.map(function (item) { return score(item, uri, language); });
            return Math.max.apply(Math, values);
        }
        else if (typeof selector === 'string') {
            // compare language id
            if (selector === language) {
                return 10;
            }
            else if (selector === '*') {
                return 5;
            }
            else {
                return 0;
            }
        }
        else if (selector) {
            var filter = selector;
            var value = 0;
            // language id
            if (filter.language) {
                if (filter.language === language) {
                    value += 10;
                }
                else if (filter.language === '*') {
                    value += 5;
                }
                else {
                    return 0;
                }
            }
            // scheme
            if (filter.scheme) {
                if (filter.scheme === uri.scheme) {
                    value += 10;
                }
                else {
                    return 0;
                }
            }
            // match fsPath with pattern
            if (filter.pattern) {
                if (filter.pattern === uri.fsPath) {
                    value += 10;
                }
                else if (glob_1.match(filter.pattern, uri.fsPath)) {
                    value += 5;
                }
                else {
                    return 0;
                }
            }
            return value;
        }
    }
    exports.score = score;
});

define("vs/editor/common/modes/lineStream", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LineStream = (function () {
        function LineStream(source) {
            this._source = source;
            this.sourceLength = source.length;
            this._pos = 0;
            this.whitespace = '\t \u00a0';
            this.whitespaceArr = this.stringToArray(this.whitespace);
            this.separators = '';
            this.separatorsArr = this.stringToArray(this.separators);
            this.tokenStart = -1;
            this.tokenEnd = -1;
        }
        LineStream.prototype.stringToArray = function (str) {
            if (!LineStream.STRING_TO_ARRAY_CACHE.hasOwnProperty(str)) {
                LineStream.STRING_TO_ARRAY_CACHE[str] = this.actualStringToArray(str);
            }
            return LineStream.STRING_TO_ARRAY_CACHE[str];
        };
        LineStream.prototype.actualStringToArray = function (str) {
            var maxCharCode = 0;
            for (var i = 0; i < str.length; i++) {
                maxCharCode = Math.max(maxCharCode, str.charCodeAt(i));
            }
            var r = [];
            for (var i = 0; i <= maxCharCode; i++) {
                r[i] = false;
            }
            for (var i = 0; i < str.length; i++) {
                r[str.charCodeAt(i)] = true;
            }
            return r;
        };
        LineStream.prototype.pos = function () {
            return this._pos;
        };
        LineStream.prototype.eos = function () {
            return this._pos >= this.sourceLength;
        };
        LineStream.prototype.peek = function () {
            // Check EOS
            if (this._pos >= this.sourceLength) {
                throw new Error('Stream is at the end');
            }
            return this._source[this._pos];
        };
        LineStream.prototype.next = function () {
            // Check EOS
            if (this._pos >= this.sourceLength) {
                throw new Error('Stream is at the end');
            }
            // Reset peeked token
            this.tokenStart = -1;
            this.tokenEnd = -1;
            return this._source[this._pos++];
        };
        LineStream.prototype.next2 = function () {
            // Check EOS
            if (this._pos >= this.sourceLength) {
                throw new Error('Stream is at the end');
            }
            // Reset peeked token
            this.tokenStart = -1;
            this.tokenEnd = -1;
            this._pos++;
        };
        LineStream.prototype.advance = function (n) {
            if (n === 0) {
                return '';
            }
            var oldPos = this._pos;
            this._pos += n;
            // Reset peeked token
            this.tokenStart = -1;
            this.tokenEnd = -1;
            return this._source.substring(oldPos, this._pos);
        };
        LineStream.prototype._advance2 = function (n) {
            if (n === 0) {
                return n;
            }
            this._pos += n;
            // Reset peeked token
            this.tokenStart = -1;
            this.tokenEnd = -1;
            return n;
        };
        LineStream.prototype.advanceToEOS = function () {
            var oldPos = this._pos;
            this._pos = this.sourceLength;
            this.resetPeekedToken();
            return this._source.substring(oldPos, this._pos);
        };
        LineStream.prototype.goBack = function (n) {
            this._pos -= n;
            this.resetPeekedToken();
        };
        LineStream.prototype.createPeeker = function (condition) {
            var _this = this;
            if (condition instanceof RegExp) {
                return function () {
                    var result = condition.exec(_this._source.substr(_this._pos));
                    if (result === null) {
                        return 0;
                    }
                    else if (result.index !== 0) {
                        throw new Error('Regular expression must begin with the character "^"');
                    }
                    return result[0].length;
                };
            }
            else if ((condition instanceof String || (typeof condition) === 'string') && condition) {
                return function () {
                    var len = condition.length, match = _this._pos + len <= _this.sourceLength;
                    for (var i = 0; match && i < len; i++) {
                        match = _this._source.charCodeAt(_this._pos + i) === condition.charCodeAt(i);
                    }
                    return match ? len : 0;
                };
            }
            throw new Error('Condition must be either a regular expression, function or a non-empty string');
        };
        // --- BEGIN `_advanceIfStringCaseInsensitive`
        LineStream.prototype._advanceIfStringCaseInsensitive = function (condition) {
            var oldPos = this._pos, source = this._source, len = condition.length, i;
            if (len < 1 || oldPos + len > this.sourceLength) {
                return 0;
            }
            for (i = 0; i < len; i++) {
                if (source.charAt(oldPos + i).toLowerCase() !== condition.charAt(i).toLowerCase()) {
                    return 0;
                }
            }
            return len;
        };
        LineStream.prototype.advanceIfStringCaseInsensitive = function (condition) {
            return this.advance(this._advanceIfStringCaseInsensitive(condition));
        };
        LineStream.prototype.advanceIfStringCaseInsensitive2 = function (condition) {
            return this._advance2(this._advanceIfStringCaseInsensitive(condition));
        };
        // --- END
        // --- BEGIN `advanceIfString`
        LineStream.prototype._advanceIfString = function (condition) {
            var oldPos = this._pos, source = this._source, len = condition.length, i;
            if (len < 1 || oldPos + len > this.sourceLength) {
                return 0;
            }
            for (i = 0; i < len; i++) {
                if (source.charCodeAt(oldPos + i) !== condition.charCodeAt(i)) {
                    return 0;
                }
            }
            return len;
        };
        LineStream.prototype.advanceIfString = function (condition) {
            return this.advance(this._advanceIfString(condition));
        };
        LineStream.prototype.advanceIfString2 = function (condition) {
            return this._advance2(this._advanceIfString(condition));
        };
        // --- END
        // --- BEGIN `advanceIfString`
        LineStream.prototype._advanceIfCharCode = function (charCode) {
            if (this._pos < this.sourceLength && this._source.charCodeAt(this._pos) === charCode) {
                return 1;
            }
            return 0;
        };
        LineStream.prototype.advanceIfCharCode = function (charCode) {
            return this.advance(this._advanceIfCharCode(charCode));
        };
        LineStream.prototype.advanceIfCharCode2 = function (charCode) {
            return this._advance2(this._advanceIfCharCode(charCode));
        };
        // --- END
        // --- BEGIN `advanceIfRegExp`
        LineStream.prototype._advanceIfRegExp = function (condition) {
            if (this._pos >= this.sourceLength) {
                return 0;
            }
            if (!condition.test(this._source.substr(this._pos))) {
                return 0;
            }
            return RegExp.lastMatch.length;
        };
        LineStream.prototype.advanceIfRegExp = function (condition) {
            return this.advance(this._advanceIfRegExp(condition));
        };
        LineStream.prototype.advanceIfRegExp2 = function (condition) {
            return this._advance2(this._advanceIfRegExp(condition));
        };
        // --- END
        LineStream.prototype.advanceLoop = function (condition, isWhile, including) {
            if (this.eos()) {
                return '';
            }
            var peeker = this.createPeeker(condition);
            var oldPos = this._pos;
            var n = 0;
            var f = null;
            if (isWhile) {
                f = function (n) {
                    return n > 0;
                };
            }
            else {
                f = function (n) {
                    return n === 0;
                };
            }
            while (!this.eos() && f(n = peeker())) {
                if (n > 0) {
                    this.advance(n);
                }
                else {
                    this.next();
                }
            }
            if (including && !this.eos()) {
                this.advance(n);
            }
            return this._source.substring(oldPos, this._pos);
        };
        LineStream.prototype.advanceWhile = function (condition) {
            return this.advanceLoop(condition, true, false);
        };
        LineStream.prototype.advanceUntil = function (condition, including) {
            return this.advanceLoop(condition, false, including);
        };
        // --- BEGIN `advanceUntilString`
        LineStream.prototype._advanceUntilString = function (condition, including) {
            if (this.eos() || condition.length === 0) {
                return 0;
            }
            var oldPos = this._pos;
            var index = this._source.indexOf(condition, oldPos);
            if (index === -1) {
                // String was not found => advanced to `eos`
                return (this.sourceLength - oldPos);
            }
            if (including) {
                // String was found => advance to include `condition`
                return (index + condition.length - oldPos);
            }
            // String was found => advance right before `condition`
            return (index - oldPos);
        };
        LineStream.prototype.advanceUntilString = function (condition, including) {
            return this.advance(this._advanceUntilString(condition, including));
        };
        LineStream.prototype.advanceUntilString2 = function (condition, including) {
            return this._advance2(this._advanceUntilString(condition, including));
        };
        // --- END
        LineStream.prototype.resetPeekedToken = function () {
            this.tokenStart = -1;
            this.tokenEnd = -1;
        };
        LineStream.prototype.setTokenRules = function (separators, whitespace) {
            if (this.separators !== separators || this.whitespace !== whitespace) {
                this.separators = separators;
                this.separatorsArr = this.stringToArray(this.separators);
                this.whitespace = whitespace;
                this.whitespaceArr = this.stringToArray(this.whitespace);
                this.resetPeekedToken();
            }
        };
        // --- tokens
        LineStream.prototype.peekToken = function () {
            if (this.tokenStart !== -1) {
                return this._source.substring(this.tokenStart, this.tokenEnd);
            }
            var source = this._source, sourceLength = this.sourceLength, whitespaceArr = this.whitespaceArr, separatorsArr = this.separatorsArr, tokenStart = this._pos;
            // Check EOS
            if (tokenStart >= sourceLength) {
                throw new Error('Stream is at the end');
            }
            // Skip whitespace
            while (whitespaceArr[source.charCodeAt(tokenStart)] && tokenStart < sourceLength) {
                tokenStart++;
            }
            var tokenEnd = tokenStart;
            // If a separator is hit, it is a token
            if (separatorsArr[source.charCodeAt(tokenEnd)] && tokenEnd < sourceLength) {
                tokenEnd++;
            }
            else {
                // Advance until a separator or a whitespace is hit
                while (!separatorsArr[source.charCodeAt(tokenEnd)] && !whitespaceArr[source.charCodeAt(tokenEnd)] && tokenEnd < sourceLength) {
                    tokenEnd++;
                }
            }
            // Cache peeked token
            this.tokenStart = tokenStart;
            this.tokenEnd = tokenEnd;
            return source.substring(tokenStart, tokenEnd);
        };
        LineStream.prototype.nextToken = function () {
            // Check EOS
            if (this._pos >= this.sourceLength) {
                throw new Error('Stream is at the end');
            }
            // Peek token if necessary
            var result;
            if (this.tokenStart === -1) {
                result = this.peekToken();
            }
            else {
                result = this._source.substring(this.tokenStart, this.tokenEnd);
            }
            // Advance to tokenEnd
            this._pos = this.tokenEnd;
            // Reset peeked token
            this.tokenStart = -1;
            this.tokenEnd = -1;
            return result;
        };
        // -- whitespace
        LineStream.prototype.peekWhitespace = function () {
            var source = this._source, sourceLength = this.sourceLength, whitespaceArr = this.whitespaceArr, peek = this._pos;
            while (whitespaceArr[source.charCodeAt(peek)] && peek < sourceLength) {
                peek++;
            }
            return source.substring(this._pos, peek);
        };
        // --- BEGIN `advanceIfRegExp`
        LineStream.prototype._skipWhitespace = function () {
            var source = this._source, sourceLength = this.sourceLength, whitespaceArr = this.whitespaceArr, oldPos = this._pos, peek = this._pos;
            while (whitespaceArr[source.charCodeAt(peek)] && peek < sourceLength) {
                peek++;
            }
            return (peek - oldPos);
        };
        LineStream.prototype.skipWhitespace = function () {
            return this.advance(this._skipWhitespace());
        };
        LineStream.prototype.skipWhitespace2 = function () {
            return this._advance2(this._skipWhitespace());
        };
        LineStream.STRING_TO_ARRAY_CACHE = {};
        return LineStream;
    }());
    exports.LineStream = LineStream;
});

define("vs/editor/common/modes/modesFilters", ["require", "exports", 'vs/base/common/arrays', 'vs/base/common/filters'], function (require, exports, arrays_1, filters) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function wrapBaseFilter(filter) {
        return function (word, suggestion) {
            var result = filter(word, suggestion.filterText || suggestion.label);
            return arrays_1.isFalsyOrEmpty(result) ? undefined : result;
        };
    }
    exports.StrictPrefix = wrapBaseFilter(filters.matchesStrictPrefix);
    exports.Prefix = wrapBaseFilter(filters.matchesPrefix);
    exports.CamelCase = wrapBaseFilter(filters.matchesCamelCase);
    exports.ContiguousSubString = wrapBaseFilter(filters.matchesContiguousSubString);
    // Combined Filters
    function or(first, second) {
        return function (word, suggestion) { return first(word, suggestion) || second(word, suggestion); };
    }
    exports.or = or;
    function and(first, second) {
        return function (word, suggestion) { return first(word, suggestion) && second(word, suggestion); };
    }
    exports.and = and;
    exports.DefaultFilter = or(or(exports.Prefix, exports.CamelCase), exports.ContiguousSubString);
});

define("vs/editor/common/modes/monarch/monarchCommon", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /*
     * This module exports common types and functionality shared between
     * the Monarch compiler that compiles JSON to ILexer, and the Monarch
     * Tokenizer (that highlights at runtime)
     */
    /*
     * Type definitions to be used internally to Monarch.
     * Inside monarch we use fully typed definitions and compiled versions of the more abstract JSON descriptions.
     */
    (function (MonarchBracket) {
        MonarchBracket[MonarchBracket["None"] = 0] = "None";
        MonarchBracket[MonarchBracket["Open"] = 1] = "Open";
        MonarchBracket[MonarchBracket["Close"] = -1] = "Close";
    })(exports.MonarchBracket || (exports.MonarchBracket = {}));
    var MonarchBracket = exports.MonarchBracket;
    // Small helper functions
    /**
     * Is a string null, undefined, or empty?
     */
    function empty(s) {
        return (s ? false : true);
    }
    exports.empty = empty;
    /**
     * Puts a string to lower case if 'ignoreCase' is set.
     */
    function fixCase(lexer, str) {
        return (lexer.ignoreCase && str ? str.toLowerCase() : str);
    }
    exports.fixCase = fixCase;
    /**
     * Ensures there are no bad characters in a CSS token class.
     */
    function sanitize(s) {
        return s.replace(/[&<>'"_]/g, '-'); // used on all output token CSS classes
    }
    exports.sanitize = sanitize;
    // Logging
    /**
     * Logs a message.
     */
    function log(lexer, msg) {
        console.log(lexer.name + ": " + msg);
    }
    exports.log = log;
    // Throwing errors
    /**
     * Throws error. May actually just log the error and continue.
     */
    function throwError(lexer, msg) {
        throw new Error(lexer.name + ": " + msg);
    }
    exports.throwError = throwError;
    // Helper functions for rule finding and substitution
    /**
     * substituteMatches is used on lexer strings and can substitutes predefined patterns:
     * 		$$  => $
     * 		$#  => id
     * 		$n  => matched entry n
     * 		@attr => contents of lexer[attr]
     *
     * See documentation for more info
     */
    function substituteMatches(lexer, str, id, matches, state) {
        var re = /\$((\$)|(#)|(\d\d?)|[sS](\d\d?)|@(\w+))/g;
        var stateMatches = null;
        return str.replace(re, function (full, sub, dollar, hash, n, s, attr, ofs, total) {
            if (!empty(dollar)) {
                return '$'; // $$
            }
            if (!empty(hash)) {
                return fixCase(lexer, id); // default $#
            }
            if (!empty(n) && n < matches.length) {
                return fixCase(lexer, matches[n]); // $n
            }
            if (!empty(attr) && lexer && typeof (lexer[attr]) === 'string') {
                return lexer[attr]; //@attribute
            }
            if (stateMatches === null) {
                stateMatches = state.split('.');
                stateMatches.unshift(state);
            }
            if (!empty(s) && s < stateMatches.length) {
                return fixCase(lexer, stateMatches[s]); //$Sn
            }
            return '';
        });
    }
    exports.substituteMatches = substituteMatches;
    /**
     * Find the tokenizer rules for a specific state (i.e. next action)
     */
    function findRules(lexer, state) {
        while (state && state.length > 0) {
            var rules = lexer.tokenizer[state];
            if (rules) {
                return rules;
            }
            var idx = state.lastIndexOf('.');
            if (idx < 0) {
                state = null; // no further parent
            }
            else {
                state = state.substr(0, idx);
            }
        }
        return null;
    }
    exports.findRules = findRules;
    /**
     * Is a certain state defined? In contrast to 'findRules' this works on a ILexerMin.
     * This is used during compilation where we may know the defined states
     * but not yet whether the corresponding rules are correct.
     */
    function stateExists(lexer, state) {
        while (state && state.length > 0) {
            var exist = lexer.stateNames[state];
            if (exist) {
                return true;
            }
            var idx = state.lastIndexOf('.');
            if (idx < 0) {
                state = null; // no further parent
            }
            else {
                state = state.substr(0, idx);
            }
        }
        return false;
    }
    exports.stateExists = stateExists;
});

define("vs/editor/common/modes/monarch/monarchCompile", ["require", "exports", 'vs/base/common/objects', 'vs/editor/common/modes/monarch/monarchCommon'], function (require, exports, objects, monarchCommon) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /*
     * Type helpers
     *
     * Note: this is just for sanity checks on the JSON description which is
     * helpful for the programmer. No checks are done anymore once the lexer is
     * already 'compiled and checked'.
     *
     */
    function isArrayOf(elemType, obj) {
        if (!obj) {
            return false;
        }
        if (!(Array.isArray(obj))) {
            return false;
        }
        var idx;
        for (idx in obj) {
            if (obj.hasOwnProperty(idx)) {
                if (!(elemType(obj[idx]))) {
                    return false;
                }
            }
        }
        return true;
    }
    function bool(prop, def, onerr) {
        if (typeof (prop) === 'boolean') {
            return prop;
        }
        if (onerr && (prop || def === undefined)) {
            onerr(); // type is wrong, or there is no default
        }
        return (def === undefined ? null : def);
    }
    function string(prop, def, onerr) {
        if (typeof (prop) === 'string') {
            return prop;
        }
        if (onerr && (prop || def === undefined)) {
            onerr(); // type is wrong, or there is no default
        }
        return (def === undefined ? null : def);
    }
    // Lexer helpers
    /**
     * Compiles a regular expression string, adding the 'i' flag if 'ignoreCase' is set.
     * Also replaces @\w+ or sequences with the content of the specified attribute
     */
    function compileRegExp(lexer, str) {
        if (typeof (str) !== 'string') {
            return null;
        }
        var n = 0;
        while (str.indexOf('@') >= 0 && n < 5) {
            n++;
            str = str.replace(/@(\w+)/g, function (s, attr) {
                var sub = '';
                if (typeof (lexer[attr]) === 'string') {
                    sub = lexer[attr];
                }
                else if (lexer[attr] && lexer[attr] instanceof RegExp) {
                    sub = lexer[attr].source;
                }
                else {
                    if (lexer[attr] === undefined) {
                        monarchCommon.throwError(lexer, 'language definition does not contain attribute \'' + attr + '\', used at: ' + str);
                    }
                    else {
                        monarchCommon.throwError(lexer, 'attribute reference \'' + attr + '\' must be a string, used at: ' + str);
                    }
                }
                return (monarchCommon.empty(sub) ? '' : '(?:' + sub + ')');
            });
        }
        return new RegExp(str, (lexer.ignoreCase ? 'i' : ''));
    }
    /**
     * Compiles guard functions for case matches.
     * This compiles 'cases' attributes into efficient match functions.
     *
     */
    function selectScrutinee(id, matches, state, num) {
        if (num < 0) {
            return id;
        }
        if (num < matches.length) {
            return matches[num];
        }
        if (num >= 100) {
            num = num - 100;
            var parts = state.split('.');
            parts.unshift(state);
            if (num < parts.length) {
                return parts[num];
            }
        }
        return null;
    }
    function createGuard(lexer, ruleName, tkey, val) {
        // get the scrutinee and pattern
        var scrut = -1; // -1: $!, 0-99: $n, 100+n: $Sn
        var oppat = tkey;
        var matches = tkey.match(/^\$(([sS]?)(\d\d?)|#)(.*)$/);
        if (matches) {
            if (matches[3]) {
                scrut = parseInt(matches[3]);
                if (matches[2]) {
                    scrut = scrut + 100; // if [sS] present
                }
            }
            oppat = matches[4];
        }
        // get operator
        var op = '~';
        var pat = oppat;
        if (!oppat || oppat.length === 0) {
            op = '!=';
            pat = '';
        }
        else if (/^\w*$/.test(pat)) {
            op = '==';
        }
        else {
            matches = oppat.match(/^(@|!@|~|!~|==|!=)(.*)$/);
            if (matches) {
                op = matches[1];
                pat = matches[2];
            }
        }
        // set the tester function
        var tester;
        // special case a regexp that matches just words
        if ((op === '~' || op === '!~') && /^(\w|\|)*$/.test(pat)) {
            var inWords = objects.createKeywordMatcher(pat.split('|'), lexer.ignoreCase);
            tester = function (s) { return (op === '~' ? inWords(s) : !inWords(s)); };
        }
        else if (op === '@' || op === '!@') {
            var words = lexer[pat];
            if (!words) {
                monarchCommon.throwError(lexer, 'the @ match target \'' + pat + '\' is not defined, in rule: ' + ruleName);
            }
            if (!(isArrayOf(function (elem) { return (typeof (elem) === 'string'); }, words))) {
                monarchCommon.throwError(lexer, 'the @ match target \'' + pat + '\' must be an array of strings, in rule: ' + ruleName);
            }
            var inWords = objects.createKeywordMatcher(words, lexer.ignoreCase);
            tester = function (s) { return (op === '@' ? inWords(s) : !inWords(s)); };
        }
        else if (op === '~' || op === '!~') {
            if (pat.indexOf('$') < 0) {
                // precompile regular expression
                var re = compileRegExp(lexer, '^' + pat + '$');
                tester = function (s) { return (op === '~' ? re.test(s) : !re.test(s)); };
            }
            else {
                tester = function (s, id, matches, state) {
                    var re = compileRegExp(lexer, '^' + monarchCommon.substituteMatches(lexer, pat, id, matches, state) + '$');
                    return re.test(s);
                };
            }
        }
        else {
            if (pat.indexOf('$') < 0) {
                var patx = monarchCommon.fixCase(lexer, pat);
                tester = function (s) { return (op === '==' ? s === patx : s !== patx); };
            }
            else {
                var patx = monarchCommon.fixCase(lexer, pat);
                tester = function (s, id, matches, state, eos) {
                    var patexp = monarchCommon.substituteMatches(lexer, patx, id, matches, state);
                    return (op === '==' ? s === patexp : s !== patexp);
                };
            }
        }
        // return the branch object
        if (scrut === -1) {
            return {
                name: tkey, value: val, test: function (id, matches, state, eos) {
                    return tester(id, id, matches, state, eos);
                }
            };
        }
        else {
            return {
                name: tkey, value: val, test: function (id, matches, state, eos) {
                    var scrutinee = selectScrutinee(id, matches, state, scrut);
                    return tester(!scrutinee ? '' : scrutinee, id, matches, state, eos);
                }
            };
        }
    }
    /**
     * Compiles an action: i.e. optimize regular expressions and case matches
     * and do many sanity checks.
     *
     * This is called only during compilation but if the lexer definition
     * contains user functions as actions (which is usually not allowed), then this
     * may be called during lexing. It is important therefore to compile common cases efficiently
     */
    function compileAction(lexer, ruleName, action) {
        if (!action) {
            return { token: '' };
        }
        else if (typeof (action) === 'string') {
            return action; // { token: action };
        }
        else if (action.token || action.token === '') {
            if (typeof (action.token) !== 'string') {
                monarchCommon.throwError(lexer, 'a \'token\' attribute must be of type string, in rule: ' + ruleName);
                return { token: '' };
            }
            else {
                // only copy specific typed fields (only happens once during compile Lexer)
                var newAction = { token: action.token };
                if (action.token.indexOf('$') >= 0) {
                    newAction.tokenSubst = true;
                }
                if (typeof (action.bracket) === 'string') {
                    if (action.bracket === '@open') {
                        newAction.bracket = monarchCommon.MonarchBracket.Open;
                    }
                    else if (action.bracket === '@close') {
                        newAction.bracket = monarchCommon.MonarchBracket.Close;
                    }
                    else {
                        monarchCommon.throwError(lexer, 'a \'bracket\' attribute must be either \'@open\' or \'@close\', in rule: ' + ruleName);
                    }
                }
                if (action.next) {
                    if (typeof (action.next) !== 'string') {
                        monarchCommon.throwError(lexer, 'the next state must be a string value in rule: ' + ruleName);
                    }
                    else {
                        var next = action.next;
                        if (!/^(@pop|@push|@popall)$/.test(next)) {
                            if (next[0] === '@') {
                                next = next.substr(1); // peel off starting @ sign
                            }
                            if (next.indexOf('$') < 0) {
                                if (!monarchCommon.stateExists(lexer, monarchCommon.substituteMatches(lexer, next, '', [], ''))) {
                                    monarchCommon.throwError(lexer, 'the next state \'' + action.next + '\' is not defined in rule: ' + ruleName);
                                }
                            }
                        }
                        newAction.next = next;
                    }
                }
                if (typeof (action.goBack) === 'number') {
                    newAction.goBack = action.goBack;
                }
                if (typeof (action.switchTo) === 'string') {
                    newAction.switchTo = action.switchTo;
                }
                if (typeof (action.log) === 'string') {
                    newAction.log = action.log;
                }
                if (typeof (action.nextEmbedded) === 'string') {
                    newAction.nextEmbedded = action.nextEmbedded;
                    lexer.usesEmbedded = true;
                }
                return newAction;
            }
        }
        else if (Array.isArray(action)) {
            var results = [];
            var idx;
            for (idx in action) {
                if (action.hasOwnProperty(idx)) {
                    results[idx] = compileAction(lexer, ruleName, action[idx]);
                }
            }
            return { group: results };
        }
        else if (action.cases) {
            // build an array of test cases
            var cases = [];
            // for each case, push a test function and result value
            var tkey;
            for (tkey in action.cases) {
                if (action.cases.hasOwnProperty(tkey)) {
                    var val = compileAction(lexer, ruleName, action.cases[tkey]);
                    // what kind of case
                    if (tkey === '@default' || tkey === '@' || tkey === '') {
                        cases.push({ test: null, value: val, name: tkey });
                    }
                    else if (tkey === '@eos') {
                        cases.push({ test: function (id, matches, state, eos) { return eos; }, value: val, name: tkey });
                    }
                    else {
                        cases.push(createGuard(lexer, ruleName, tkey, val)); // call separate function to avoid local variable capture
                    }
                }
            }
            // create a matching function
            var def = lexer.defaultToken;
            return {
                test: function (id, matches, state, eos) {
                    var idx;
                    for (idx in cases) {
                        if (cases.hasOwnProperty(idx)) {
                            var didmatch = (!cases[idx].test || cases[idx].test(id, matches, state, eos));
                            if (didmatch) {
                                return cases[idx].value;
                            }
                        }
                    }
                    return def;
                }
            };
        }
        else {
            monarchCommon.throwError(lexer, 'an action must be a string, an object with a \'token\' or \'cases\' attribute, or an array of actions; in rule: ' + ruleName);
            return '';
        }
    }
    /**
     * Helper class for creating matching rules
     */
    var Rule = (function () {
        function Rule(name) {
            this.regex = new RegExp('');
            this.action = { token: '' };
            this.matchOnlyAtLineStart = false;
            this.name = '';
            this.name = name;
        }
        Rule.prototype.setRegex = function (lexer, re) {
            var sregex;
            if (typeof (re) === 'string') {
                sregex = re;
            }
            else if (re instanceof RegExp) {
                sregex = re.source;
            }
            else {
                monarchCommon.throwError(lexer, 'rules must start with a match string or regular expression: ' + this.name);
            }
            this.matchOnlyAtLineStart = (sregex.length > 0 && sregex[0] === '^');
            this.name = this.name + ': ' + sregex;
            this.regex = compileRegExp(lexer, '^(?:' + (this.matchOnlyAtLineStart ? sregex.substr(1) : sregex) + ')');
        };
        Rule.prototype.setAction = function (lexer, act) {
            this.action = compileAction(lexer, this.name, act);
        };
        return Rule;
    }());
    /**
     * Compiles a json description function into json where all regular expressions,
     * case matches etc, are compiled and all include rules are expanded.
     * We also compile the bracket definitions, supply defaults, and do many sanity checks.
     * If the 'jsonStrict' parameter is 'false', we allow at certain locations
     * regular expression objects and functions that get called during lexing.
     * (Currently we have no samples that need this so perhaps we should always have
     * jsonStrict to true).
     */
    function compile(json) {
        if (!json || typeof (json) !== 'object') {
            throw new Error('Monarch: expecting a language definition object');
        }
        // Get names
        if (typeof (json.name) !== 'string') {
            throw new Error('Monarch: a language definition must include a string \'name\' attribute');
        }
        // Create our lexer
        var lexer = {};
        lexer.name = json.name;
        lexer.displayName = string(json.displayName, lexer.name);
        lexer.noThrow = false; // raise exceptions during compilation
        lexer.maxStack = 100;
        // Set standard fields: be defensive about types
        lexer.start = string(json.start);
        lexer.ignoreCase = bool(json.ignoreCase, false);
        lexer.lineComment = string(json.lineComment, '//');
        lexer.blockCommentStart = string(json.blockCommentStart, '/*');
        lexer.blockCommentEnd = string(json.blockCommentEnd, '*/');
        lexer.tokenPostfix = string(json.tokenPostfix, '.' + lexer.name);
        lexer.defaultToken = string(json.defaultToken, 'source', function () { monarchCommon.throwError(lexer, 'the \'defaultToken\' must be a string'); });
        lexer.usesEmbedded = false; // becomes true if we find a nextEmbedded action
        lexer.wordDefinition = json.wordDefinition || undefined;
        // COMPAT: with earlier monarch versions
        if (!lexer.lineComment && json.lineComments) {
            if (typeof (json.lineComments) === 'string') {
                lexer.lineComment = json.lineComments;
            }
            else if (typeof (json.lineComments[0]) === 'string') {
                lexer.lineComment = json.lineComments[0];
            }
        }
        lexer.suggestSupport = {
            textualCompletions: true,
            disableAutoTrigger: false,
            triggerCharacters: [],
            snippets: []
        };
        if (typeof json.suggestSupport !== 'undefined') {
            var suggestSupport = json.suggestSupport;
            if (Array.isArray(suggestSupport.snippets)) {
                var _snippets = suggestSupport.snippets;
                for (var i = 0, len = _snippets.length; i < len; i++) {
                    if (typeof _snippets[i] === 'string') {
                        lexer.suggestSupport.snippets.push({
                            type: 'snippet',
                            label: _snippets[i],
                            codeSnippet: _snippets[i]
                        });
                    }
                    else {
                        lexer.suggestSupport.snippets.push(_snippets[i]);
                    }
                }
            }
            if (Array.isArray(suggestSupport.triggerCharacters)) {
                lexer.suggestSupport.triggerCharacters = suggestSupport.triggerCharacters;
            }
            if (typeof suggestSupport.textualCompletions !== 'undefined') {
                lexer.suggestSupport.textualCompletions = suggestSupport.textualCompletions;
            }
            if (typeof suggestSupport.disableAutoTrigger !== 'undefined') {
                lexer.suggestSupport.disableAutoTrigger = suggestSupport.disableAutoTrigger;
            }
        }
        // For calling compileAction later on
        var lexerMin = json;
        lexerMin.name = lexer.name;
        lexerMin.displayName = lexer.displayName;
        lexerMin.ignoreCase = lexer.ignoreCase;
        lexerMin.noThrow = lexer.noThrow;
        lexerMin.usesEmbedded = lexer.usesEmbedded;
        lexerMin.stateNames = json.tokenizer;
        lexerMin.defaultToken = lexer.defaultToken;
        // Compile an array of rules into newrules where RegExp objects are created.
        function addRules(state, newrules, rules) {
            var idx;
            for (idx in rules) {
                if (rules.hasOwnProperty(idx)) {
                    var rule = rules[idx];
                    var include = rule.include;
                    if (include) {
                        if (typeof (include) !== 'string') {
                            monarchCommon.throwError(lexer, 'an \'include\' attribute must be a string at: ' + state);
                        }
                        if (include[0] === '@') {
                            include = include.substr(1); // peel off starting @
                        }
                        if (!json.tokenizer[include]) {
                            monarchCommon.throwError(lexer, 'include target \'' + include + '\' is not defined at: ' + state);
                        }
                        addRules(state + '.' + include, newrules, json.tokenizer[include]);
                    }
                    else {
                        var newrule = new Rule(state);
                        // Set up new rule attributes
                        if (Array.isArray(rule) && rule.length >= 1 && rule.length <= 3) {
                            newrule.setRegex(lexerMin, rule[0]);
                            if (rule.length >= 3) {
                                if (typeof (rule[1]) === 'string') {
                                    newrule.setAction(lexerMin, { token: rule[1], next: rule[2] });
                                }
                                else if (typeof (rule[1]) === 'object') {
                                    var rule1 = rule[1];
                                    rule1.next = rule[2];
                                    newrule.setAction(lexerMin, rule1);
                                }
                                else {
                                    monarchCommon.throwError(lexer, 'a next state as the last element of a rule can only be given if the action is either an object or a string, at: ' + state);
                                }
                            }
                            else {
                                newrule.setAction(lexerMin, rule[1]);
                            }
                        }
                        else {
                            if (!rule.regex) {
                                monarchCommon.throwError(lexer, 'a rule must either be an array, or an object with a \'regex\' or \'include\' field at: ' + state);
                            }
                            if (rule.name) {
                                newrule.name = string(rule.name);
                            }
                            if (rule.matchOnlyAtStart) {
                                newrule.matchOnlyAtLineStart = bool(rule.matchOnlyAtLineStart);
                            }
                            newrule.setRegex(lexerMin, rule.regex);
                            newrule.setAction(lexerMin, rule.action);
                        }
                        newrules.push(newrule);
                    }
                }
            }
        }
        // compile the tokenizer rules
        if (!json.tokenizer || typeof (json.tokenizer) !== 'object') {
            monarchCommon.throwError(lexer, 'a language definition must define the \'tokenizer\' attribute as an object');
        }
        lexer.tokenizer = [];
        var key;
        for (key in json.tokenizer) {
            if (json.tokenizer.hasOwnProperty(key)) {
                if (!lexer.start) {
                    lexer.start = key;
                }
                var rules = json.tokenizer[key];
                lexer.tokenizer[key] = new Array();
                addRules('tokenizer.' + key, lexer.tokenizer[key], rules);
            }
        }
        lexer.usesEmbedded = lexerMin.usesEmbedded; // can be set during compileAction
        // Set simple brackets
        if (json.brackets) {
            if (!(Array.isArray(json.brackets))) {
                monarchCommon.throwError(lexer, 'the \'brackets\' attribute must be defined as an array');
            }
        }
        else {
            json.brackets = [
                { open: '{', close: '}', token: 'delimiter.curly' },
                { open: '[', close: ']', token: 'delimiter.square' },
                { open: '(', close: ')', token: 'delimiter.parenthesis' },
                { open: '<', close: '>', token: 'delimiter.angle' }];
        }
        var brackets = [];
        for (var bracketIdx in json.brackets) {
            if (json.brackets.hasOwnProperty(bracketIdx)) {
                var desc = json.brackets[bracketIdx];
                if (desc && Array.isArray(desc) && desc.length === 3) {
                    desc = { token: desc[2], open: desc[0], close: desc[1] };
                }
                if (desc.open === desc.close) {
                    monarchCommon.throwError(lexer, 'open and close brackets in a \'brackets\' attribute must be different: ' + desc.open +
                        '\n hint: use the \'bracket\' attribute if matching on equal brackets is required.');
                }
                if (typeof (desc.open) === 'string' && typeof (desc.token) === 'string') {
                    brackets.push({
                        token: string(desc.token) + lexer.tokenPostfix,
                        open: monarchCommon.fixCase(lexer, string(desc.open)),
                        close: monarchCommon.fixCase(lexer, string(desc.close))
                    });
                }
                else {
                    monarchCommon.throwError(lexer, 'every element in the \'brackets\' array must be a \'{open,close,token}\' object or array');
                }
            }
        }
        lexer.brackets = brackets;
        // Set default auto closing pairs
        var autoClosingPairs;
        if (json.autoClosingPairs) {
            if (!(Array.isArray(json.autoClosingPairs))) {
                monarchCommon.throwError(lexer, 'the \'autoClosingPairs\' attribute must be an array of string pairs (as arrays)');
            }
            autoClosingPairs = json.autoClosingPairs.slice(0);
        }
        else {
            autoClosingPairs = [['"', '"'], ['\'', '\''], ['@brackets']];
        }
        // set auto closing pairs
        lexer.autoClosingPairs = [];
        if (autoClosingPairs) {
            for (var autoClosingPairIdx in autoClosingPairs) {
                if (autoClosingPairs.hasOwnProperty(autoClosingPairIdx)) {
                    var pair = autoClosingPairs[autoClosingPairIdx];
                    var openClose;
                    if (pair === '@brackets' || pair[0] === '@brackets') {
                        var bidx;
                        for (bidx in brackets) {
                            if (brackets.hasOwnProperty(bidx)) {
                                if (brackets[bidx].open && brackets[bidx].open.length === 1 &&
                                    brackets[bidx].close && brackets[bidx].close.length === 1) {
                                    openClose = { open: brackets[bidx].open, close: brackets[bidx].close, notIn: ['string', 'comment'] };
                                    lexer.autoClosingPairs.push(openClose);
                                }
                            }
                        }
                    }
                    else if (Array.isArray(pair) && pair.length === 2 &&
                        typeof (pair[0]) === 'string' && pair[0].length === 1 &&
                        typeof (pair[1]) === 'string' && pair[1].length === 1) {
                        openClose = { open: monarchCommon.fixCase(lexer, pair[0]), close: monarchCommon.fixCase(lexer, pair[1]), notIn: ['string', 'comment'] };
                        lexer.autoClosingPairs.push(openClose);
                    }
                    else if (typeof (pair.open) === 'string' && pair.open.length === 1 &&
                        typeof (pair.close) === 'string' && pair.close.length === 1) {
                        openClose = { open: monarchCommon.fixCase(lexer, pair.open[0]), close: monarchCommon.fixCase(lexer, pair.close[0]), notIn: ['string', 'comment'] };
                        lexer.autoClosingPairs.push(openClose);
                    }
                    else {
                        monarchCommon.throwError(lexer, 'every element in an \'autoClosingPairs\' array must be a pair of 1 character strings, or a \'@brackets\' directive');
                    }
                }
            }
        }
        // Set enhanced brackets
        // var enhancedBrackets : IRegexBracketPair[] = [];
        // if (json.enhancedBrackets) {
        // 	if (!(Array.isArray(<any>json.enhancedBrackets))) {
        // 		monarchCommon.throwError(lexer, 'the \'enhancedBrackets\' attribute must be defined as an array');
        // 	}
        // 	for (var bracketIdx in json.enhancedBrackets) {
        // 		if (json.enhancedBrackets.hasOwnProperty(bracketIdx)) {
        // 			var desc = <any> json.enhancedBrackets[bracketIdx];
        // 			if (desc.hasOwnProperty('openTrigger') && typeof (desc.openTrigger) !== 'string') {
        // 				monarchCommon.throwError(lexer, 'openTrigger in the \'enhancedBrackets\' array must be a string');
        // 			}
        // 			if (desc.hasOwnProperty('open') && !(desc.open instanceof RegExp)) {
        // 				monarchCommon.throwError(lexer, 'open in the \'enhancedBrackets\' array must be a regex');
        // 			}
        // 			if (desc.hasOwnProperty('closeComplete') && typeof (desc.closeComplete) !== 'string') {
        // 				monarchCommon.throwError(lexer, 'closeComplete in the \'enhancedBrackets\' array must be a string');
        // 			}
        // 			if (desc.hasOwnProperty('matchCase') && typeof (desc.matchCase) !== 'boolean') {
        // 				monarchCommon.throwError(lexer, 'matchCase in the \'enhancedBrackets\' array must be a boolean');
        // 			}
        // 			if (desc.hasOwnProperty('closeTrigger') && typeof (desc.closeTrigger) !== 'string') {
        // 				monarchCommon.throwError(lexer, 'closeTrigger in the \'enhancedBrackets\' array must be a string');
        // 			}
        // 			if (desc.hasOwnProperty('close') && !(desc.close instanceof RegExp)) {
        // 				monarchCommon.throwError(lexer, 'close in the \'enhancedBrackets\' array must be a regex');
        // 			}
        // 			if (desc.hasOwnProperty('tokenType')) {
        // 				if (typeof (desc.tokenType) !== 'string') {
        // 					monarchCommon.throwError(lexer, 'tokenType in the \'enhancedBrackets\' array must be a string');
        // 				}
        // 				else {
        // 					desc.tokenType += lexer.tokenPostfix;
        // 				}
        // 			}
        // 			enhancedBrackets.push(desc);
        // 		}
        // 	}
        // }
        // lexer.enhancedBrackets = enhancedBrackets;
        var standardBrackets = [];
        for (var i = 0; i < brackets.length; ++i) {
            standardBrackets.push([brackets[i].open, brackets[i].close]);
        }
        lexer.standardBrackets = standardBrackets;
        lexer.outdentTriggers = string(json.outdentTriggers, '');
        // Disable throw so the syntax highlighter goes, no matter what
        lexer.noThrow = true;
        return lexer;
    }
    exports.compile = compile;
});

define("vs/editor/common/modes/supports", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/core/arrays'], function (require, exports, strings, arrays_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Token = (function () {
        function Token(startIndex, type) {
            this.startIndex = startIndex;
            this.type = type;
        }
        Token.prototype.toString = function () {
            return '(' + this.startIndex + ', ' + this.type + ')';
        };
        return Token;
    }());
    exports.Token = Token;
    function handleEvent(context, offset, runner) {
        var modeTransitions = context.modeTransitions;
        if (modeTransitions.length === 1) {
            return runner(modeTransitions[0].mode, context, offset);
        }
        var modeIndex = arrays_1.Arrays.findIndexInSegmentsArray(modeTransitions, offset);
        var nestedMode = modeTransitions[modeIndex].mode;
        var modeStartIndex = modeTransitions[modeIndex].startIndex;
        var firstTokenInModeIndex = context.findIndexOfOffset(modeStartIndex);
        var nextCharacterAfterModeIndex = -1;
        var nextTokenAfterMode = -1;
        if (modeIndex + 1 < modeTransitions.length) {
            nextTokenAfterMode = context.findIndexOfOffset(modeTransitions[modeIndex + 1].startIndex);
            nextCharacterAfterModeIndex = context.getTokenStartIndex(nextTokenAfterMode);
        }
        else {
            nextTokenAfterMode = context.getTokenCount();
            nextCharacterAfterModeIndex = context.getLineContent().length;
        }
        var firstTokenCharacterOffset = context.getTokenStartIndex(firstTokenInModeIndex);
        var newCtx = new FilteredLineContext(context, nestedMode, firstTokenInModeIndex, nextTokenAfterMode, firstTokenCharacterOffset, nextCharacterAfterModeIndex);
        return runner(nestedMode, newCtx, offset - firstTokenCharacterOffset);
    }
    exports.handleEvent = handleEvent;
    /**
     * Returns {{true}} if the line token at the specified
     * offset matches one of the provided types. Matching
     * happens on a substring start from the end, unless
     * anywhereInToken is set to true in which case matches
     * happen on a substring at any position.
     */
    function isLineToken(context, offset, types, anywhereInToken) {
        if (anywhereInToken === void 0) { anywhereInToken = false; }
        if (!Array.isArray(types) || types.length === 0) {
            return false;
        }
        if (context.getLineContent().length <= offset) {
            return false;
        }
        var tokenIdx = context.findIndexOfOffset(offset);
        var type = context.getTokenType(tokenIdx);
        for (var i = 0, len = types.length; i < len; i++) {
            if (anywhereInToken) {
                if (type.indexOf(types[i]) >= 0) {
                    return true;
                }
            }
            else {
                if (strings.endsWith(type, types[i])) {
                    return true;
                }
            }
        }
        return false;
    }
    exports.isLineToken = isLineToken;
    var FilteredLineContext = (function () {
        function FilteredLineContext(actual, mode, firstTokenInModeIndex, nextTokenAfterMode, firstTokenCharacterOffset, nextCharacterAfterModeIndex) {
            this.modeTransitions = [{
                    startIndex: 0,
                    mode: mode
                }];
            this._actual = actual;
            this._firstTokenInModeIndex = firstTokenInModeIndex;
            this._nextTokenAfterMode = nextTokenAfterMode;
            this._firstTokenCharacterOffset = firstTokenCharacterOffset;
            this._nextCharacterAfterModeIndex = nextCharacterAfterModeIndex;
        }
        FilteredLineContext.prototype.getLineContent = function () {
            var actualLineContent = this._actual.getLineContent();
            return actualLineContent.substring(this._firstTokenCharacterOffset, this._nextCharacterAfterModeIndex);
        };
        FilteredLineContext.prototype.getTokenCount = function () {
            return this._nextTokenAfterMode - this._firstTokenInModeIndex;
        };
        FilteredLineContext.prototype.findIndexOfOffset = function (offset) {
            return this._actual.findIndexOfOffset(offset + this._firstTokenCharacterOffset) - this._firstTokenInModeIndex;
        };
        FilteredLineContext.prototype.getTokenStartIndex = function (tokenIndex) {
            return this._actual.getTokenStartIndex(tokenIndex + this._firstTokenInModeIndex) - this._firstTokenCharacterOffset;
        };
        FilteredLineContext.prototype.getTokenEndIndex = function (tokenIndex) {
            return this._actual.getTokenEndIndex(tokenIndex + this._firstTokenInModeIndex) - this._firstTokenCharacterOffset;
        };
        FilteredLineContext.prototype.getTokenType = function (tokenIndex) {
            return this._actual.getTokenType(tokenIndex + this._firstTokenInModeIndex);
        };
        FilteredLineContext.prototype.getTokenText = function (tokenIndex) {
            return this._actual.getTokenText(tokenIndex + this._firstTokenInModeIndex);
        };
        return FilteredLineContext;
    }());
    exports.FilteredLineContext = FilteredLineContext;
    function ignoreBracketsInToken(tokenType) {
        return /\b(comment|string|regex)\b/.test(tokenType);
    }
    exports.ignoreBracketsInToken = ignoreBracketsInToken;
    // TODO@Martin: find a better home for this code:
    // TODO@Martin: modify suggestSupport to return a boolean if snippets should be presented or not
    //       and turn this into a real registry
    var SnippetsRegistry = (function () {
        function SnippetsRegistry() {
        }
        SnippetsRegistry.registerDefaultSnippets = function (modeId, snippets) {
            this._defaultSnippets[modeId] = (this._defaultSnippets[modeId] || []).concat(snippets);
        };
        SnippetsRegistry.registerSnippets = function (modeId, path, snippets) {
            var snippetsByMode = this._snippets[modeId];
            if (!snippetsByMode) {
                this._snippets[modeId] = snippetsByMode = {};
            }
            snippetsByMode[path] = snippets;
        };
        SnippetsRegistry.getSnippets = function (model, position) {
            var word = model.getWordAtPosition(position);
            var currentPrefix = word ? word.word.substring(0, position.column - word.startColumn) : '';
            var result = {
                currentWord: currentPrefix,
                suggestions: []
            };
            // to avoid that snippets are too prominent in the intellisense proposals:
            // - force that the current prefix matches with the snippet prefix
            // if there's no prfix, only show snippets at the beginning of the line, or after a whitespace
            var filter = null;
            if (currentPrefix.length === 0) {
                if (position.column > 1) {
                    var previousCharacter = model.getValueInRange({ startLineNumber: position.lineNumber, startColumn: position.column - 1, endLineNumber: position.lineNumber, endColumn: position.column });
                    if (previousCharacter.trim().length !== 0) {
                        return result;
                    }
                }
            }
            else {
                var lowerCasePrefix_1 = currentPrefix.toLowerCase();
                filter = function (p) {
                    return strings.startsWith(p.label.toLowerCase(), lowerCasePrefix_1);
                };
            }
            var modeId = model.getMode().getId();
            var snippets = [];
            var snipppetsByMode = this._snippets[modeId];
            if (snipppetsByMode) {
                for (var s in snipppetsByMode) {
                    snippets = snippets.concat(snipppetsByMode[s]);
                }
            }
            var defaultSnippets = this._defaultSnippets[modeId];
            if (defaultSnippets) {
                snippets = snippets.concat(defaultSnippets);
            }
            result.suggestions = filter ? snippets.filter(filter) : snippets;
            // if (result.suggestions.length > 0) {
            // 	if (word) {
            // 		// Push also the current word as first suggestion, to avoid unexpected snippet acceptance on Enter.
            // 		result.suggestions = result.suggestions.slice(0);
            // 		result.suggestions.unshift({
            // 			codeSnippet: word.word,
            // 			label: word.word,
            // 			type: 'text'
            // 		});
            // 	}
            // 	result.incomplete = true;
            // }
            return result;
        };
        SnippetsRegistry._defaultSnippets = Object.create(null);
        SnippetsRegistry._snippets = Object.create(null);
        return SnippetsRegistry;
    }());
    exports.SnippetsRegistry = SnippetsRegistry;
});

define("vs/editor/common/modes/supports/characterPair", ["require", "exports", 'vs/editor/common/modes/supports'], function (require, exports, supports_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var CharacterPairSupport = (function () {
        function CharacterPairSupport(modeId, contribution) {
            this._modeId = modeId;
            this._autoClosingPairs = contribution.autoClosingPairs;
            this._surroundingPairs = Array.isArray(contribution.surroundingPairs) ? contribution.surroundingPairs : contribution.autoClosingPairs;
        }
        CharacterPairSupport.prototype.getAutoClosingPairs = function () {
            return this._autoClosingPairs;
        };
        CharacterPairSupport.prototype.shouldAutoClosePair = function (character, context, offset) {
            var _this = this;
            return supports_1.handleEvent(context, offset, function (nestedMode, context, offset) {
                if (_this._modeId === nestedMode.getId()) {
                    // Always complete on empty line
                    if (context.getTokenCount() === 0) {
                        return true;
                    }
                    var tokenIndex = context.findIndexOfOffset(offset - 1);
                    var tokenType = context.getTokenType(tokenIndex);
                    for (var i = 0; i < _this._autoClosingPairs.length; ++i) {
                        if (_this._autoClosingPairs[i].open === character) {
                            if (_this._autoClosingPairs[i].notIn) {
                                for (var notInIndex = 0; notInIndex < _this._autoClosingPairs[i].notIn.length; ++notInIndex) {
                                    if (tokenType.indexOf(_this._autoClosingPairs[i].notIn[notInIndex]) > -1) {
                                        return false;
                                    }
                                }
                            }
                            break;
                        }
                    }
                    return true;
                }
                else if (nestedMode.richEditSupport && nestedMode.richEditSupport.characterPair) {
                    return nestedMode.richEditSupport.characterPair.shouldAutoClosePair(character, context, offset);
                }
                else {
                    return null;
                }
            });
        };
        CharacterPairSupport.prototype.getSurroundingPairs = function () {
            return this._surroundingPairs;
        };
        return CharacterPairSupport;
    }());
    exports.CharacterPairSupport = CharacterPairSupport;
});

define("vs/editor/common/modes/supports/declarationSupport", ["require", "exports", 'vs/editor/common/modes/supports'], function (require, exports, supports_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DeclarationSupport = (function () {
        /**
         * Provide the token type postfixes for the tokens where a declaration can be found in the 'tokens' argument.
         */
        function DeclarationSupport(modeId, contribution) {
            this._modeId = modeId;
            this.contribution = contribution;
        }
        DeclarationSupport.prototype.canFindDeclaration = function (context, offset) {
            var _this = this;
            return supports_1.handleEvent(context, offset, function (nestedMode, context, offset) {
                if (_this._modeId === nestedMode.getId()) {
                    return (!Array.isArray(_this.contribution.tokens) ||
                        _this.contribution.tokens.length < 1 ||
                        supports_1.isLineToken(context, offset, _this.contribution.tokens));
                }
                else if (nestedMode.declarationSupport) {
                    return nestedMode.declarationSupport.canFindDeclaration(context, offset);
                }
                else {
                    return false;
                }
            });
        };
        DeclarationSupport.prototype.findDeclaration = function (resource, position) {
            return this.contribution.findDeclaration(resource, position);
        };
        return DeclarationSupport;
    }());
    exports.DeclarationSupport = DeclarationSupport;
});

define("vs/editor/common/modes/supports/parameterHintsSupport", ["require", "exports", 'vs/editor/common/modes/supports'], function (require, exports, supports_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ParameterHintsSupport = (function () {
        function ParameterHintsSupport(modeId, contribution) {
            this._modeId = modeId;
            this.contribution = contribution;
        }
        ParameterHintsSupport.prototype.getParameterHintsTriggerCharacters = function () {
            return this.contribution.triggerCharacters;
        };
        ParameterHintsSupport.prototype.shouldTriggerParameterHints = function (context, offset) {
            var _this = this;
            return supports_1.handleEvent(context, offset, function (nestedMode, context, offset) {
                if (_this._modeId === nestedMode.getId()) {
                    if (!Array.isArray(_this.contribution.excludeTokens)) {
                        return true;
                    }
                    if (_this.contribution.excludeTokens.length === 1 && _this.contribution.excludeTokens[0] === '*') {
                        return false;
                    }
                    return !supports_1.isLineToken(context, offset - 1, _this.contribution.excludeTokens);
                }
                else if (nestedMode.parameterHintsSupport) {
                    return nestedMode.parameterHintsSupport.shouldTriggerParameterHints(context, offset);
                }
                else {
                    return false;
                }
            });
        };
        ParameterHintsSupport.prototype.getParameterHints = function (resource, position) {
            return this.contribution.getParameterHints(resource, position);
        };
        return ParameterHintsSupport;
    }());
    exports.ParameterHintsSupport = ParameterHintsSupport;
});

define("vs/editor/common/modes/supports/referenceSupport", ["require", "exports", 'vs/editor/common/modes/supports'], function (require, exports, supports_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ReferenceSupport = (function () {
        /**
         * Provide the token type postfixes for the tokens where a reference can be found in the 'tokens' argument.
         */
        function ReferenceSupport(modeId, contribution) {
            this._modeId = modeId;
            this.contribution = contribution;
        }
        ReferenceSupport.prototype.canFindReferences = function (context, offset) {
            var _this = this;
            return supports_1.handleEvent(context, offset, function (nestedMode, context, offset) {
                if (_this._modeId === nestedMode.getId()) {
                    return (!Array.isArray(_this.contribution.tokens) ||
                        _this.contribution.tokens.length < 1 ||
                        supports_1.isLineToken(context, offset, _this.contribution.tokens));
                }
                else if (nestedMode.referenceSupport) {
                    return nestedMode.referenceSupport.canFindReferences(context, offset);
                }
                else {
                    return false;
                }
            });
        };
        ReferenceSupport.prototype.findReferences = function (resource, position, includeDeclaration) {
            return this.contribution.findReferences(resource, position, includeDeclaration);
        };
        return ReferenceSupport;
    }());
    exports.ReferenceSupport = ReferenceSupport;
});

define("vs/editor/common/modes/supports/richEditBrackets", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/core/range'], function (require, exports, strings, range_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var RichEditBrackets = (function () {
        function RichEditBrackets(modeId, brackets) {
            var _this = this;
            this.brackets = brackets.map(function (b) {
                return {
                    modeId: modeId,
                    open: b[0],
                    close: b[1],
                    forwardRegex: getRegexForBracketPair({ open: b[0], close: b[1] }),
                    reversedRegex: getReversedRegexForBracketPair({ open: b[0], close: b[1] })
                };
            });
            this.forwardRegex = getRegexForBrackets(this.brackets);
            this.reversedRegex = getReversedRegexForBrackets(this.brackets);
            this.textIsBracket = {};
            this.textIsOpenBracket = {};
            this.maxBracketLength = 0;
            this.brackets.forEach(function (b) {
                _this.textIsBracket[b.open] = b;
                _this.textIsBracket[b.close] = b;
                _this.textIsOpenBracket[b.open] = true;
                _this.textIsOpenBracket[b.close] = false;
                _this.maxBracketLength = Math.max(_this.maxBracketLength, b.open.length);
                _this.maxBracketLength = Math.max(_this.maxBracketLength, b.close.length);
            });
        }
        return RichEditBrackets;
    }());
    exports.RichEditBrackets = RichEditBrackets;
    function once(keyFn, computeFn) {
        var cache = {};
        return function (input) {
            var key = keyFn(input);
            if (!cache.hasOwnProperty(key)) {
                cache[key] = computeFn(input);
            }
            return cache[key];
        };
    }
    var getRegexForBracketPair = once(function (input) { return (input.open + ";" + input.close); }, function (input) {
        return createOrRegex([input.open, input.close]);
    });
    var getReversedRegexForBracketPair = once(function (input) { return (input.open + ";" + input.close); }, function (input) {
        return createOrRegex([toReversedString(input.open), toReversedString(input.close)]);
    });
    var getRegexForBrackets = once(function (input) { return input.map(function (b) { return (b.open + ";" + b.close); }).join(';'); }, function (input) {
        var pieces = [];
        input.forEach(function (b) {
            pieces.push(b.open);
            pieces.push(b.close);
        });
        return createOrRegex(pieces);
    });
    var getReversedRegexForBrackets = once(function (input) { return input.map(function (b) { return (b.open + ";" + b.close); }).join(';'); }, function (input) {
        var pieces = [];
        input.forEach(function (b) {
            pieces.push(toReversedString(b.open));
            pieces.push(toReversedString(b.close));
        });
        return createOrRegex(pieces);
    });
    function createOrRegex(pieces) {
        var regexStr = "(" + pieces.map(strings.escapeRegExpCharacters).join(')|(') + ")";
        return strings.createRegExp(regexStr, true, false, false, false);
    }
    function toReversedString(str) {
        var reversedStr = '';
        for (var i = str.length - 1; i >= 0; i--) {
            reversedStr += str.charAt(i);
        }
        return reversedStr;
    }
    var BracketsUtils = (function () {
        function BracketsUtils() {
        }
        BracketsUtils._findPrevBracketInText = function (reversedBracketRegex, lineNumber, reversedText, offset) {
            var m = reversedText.match(reversedBracketRegex);
            if (!m) {
                return null;
            }
            var matchOffset = reversedText.length - m.index;
            var matchLength = m[0].length;
            var absoluteMatchOffset = offset + matchOffset;
            return new range_1.Range(lineNumber, absoluteMatchOffset - matchLength + 1, lineNumber, absoluteMatchOffset + 1);
        };
        BracketsUtils.findPrevBracketInToken = function (reversedBracketRegex, lineNumber, lineText, currentTokenStart, currentTokenEnd) {
            // Because JS does not support backwards regex search, we search forwards in a reversed string with a reversed regex ;)
            var currentTokenReversedText = '';
            for (var index = currentTokenEnd - 1; index >= currentTokenStart; index--) {
                currentTokenReversedText += lineText.charAt(index);
            }
            return this._findPrevBracketInText(reversedBracketRegex, lineNumber, currentTokenReversedText, currentTokenStart);
        };
        BracketsUtils.findNextBracketInText = function (bracketRegex, lineNumber, text, offset) {
            var m = text.match(bracketRegex);
            if (!m) {
                return null;
            }
            var matchOffset = m.index;
            var matchLength = m[0].length;
            var absoluteMatchOffset = offset + matchOffset;
            return new range_1.Range(lineNumber, absoluteMatchOffset + 1, lineNumber, absoluteMatchOffset + 1 + matchLength);
        };
        BracketsUtils.findNextBracketInToken = function (bracketRegex, lineNumber, lineText, currentTokenStart, currentTokenEnd) {
            var currentTokenText = lineText.substring(currentTokenStart, currentTokenEnd);
            return this.findNextBracketInText(bracketRegex, lineNumber, currentTokenText, currentTokenStart);
        };
        return BracketsUtils;
    }());
    exports.BracketsUtils = BracketsUtils;
});

define("vs/editor/common/modes/supports/electricCharacter", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/modes/supports', 'vs/editor/common/modes/supports/richEditBrackets'], function (require, exports, strings, supports_1, richEditBrackets_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var BracketElectricCharacterSupport = (function () {
        function BracketElectricCharacterSupport(modeId, brackets, contribution) {
            this._modeId = modeId;
            this.contribution = contribution || {};
            this.brackets = new Brackets(modeId, brackets, this.contribution.docComment, this.contribution.caseInsensitive);
        }
        BracketElectricCharacterSupport.prototype.getElectricCharacters = function () {
            if (Array.isArray(this.contribution.embeddedElectricCharacters)) {
                return this.contribution.embeddedElectricCharacters.concat(this.brackets.getElectricCharacters());
            }
            return this.brackets.getElectricCharacters();
        };
        BracketElectricCharacterSupport.prototype.onElectricCharacter = function (context, offset) {
            var _this = this;
            return supports_1.handleEvent(context, offset, function (nestedMode, context, offset) {
                if (_this._modeId === nestedMode.getId()) {
                    return _this.brackets.onElectricCharacter(context, offset);
                }
                else if (nestedMode.richEditSupport && nestedMode.richEditSupport.electricCharacter) {
                    return nestedMode.richEditSupport.electricCharacter.onElectricCharacter(context, offset);
                }
                else {
                    return null;
                }
            });
        };
        return BracketElectricCharacterSupport;
    }());
    exports.BracketElectricCharacterSupport = BracketElectricCharacterSupport;
    var Brackets = (function () {
        function Brackets(modeId, richEditBrackets, docComment, caseInsensitive) {
            if (docComment === void 0) { docComment = null; }
            if (caseInsensitive === void 0) { caseInsensitive = false; }
            this._modeId = modeId;
            this._richEditBrackets = richEditBrackets;
            this._docComment = docComment ? docComment : null;
        }
        Brackets.prototype.getElectricCharacters = function () {
            var result = [];
            if (this._richEditBrackets) {
                for (var i = 0, len = this._richEditBrackets.brackets.length; i < len; i++) {
                    var bracketPair = this._richEditBrackets.brackets[i];
                    var lastChar = bracketPair.close.charAt(bracketPair.close.length - 1);
                    result.push(lastChar);
                }
            }
            // Doc comments
            if (this._docComment) {
                result.push(this._docComment.open.charAt(this._docComment.open.length - 1));
            }
            // Filter duplicate entries
            result = result.filter(function (item, pos, array) {
                return array.indexOf(item) === pos;
            });
            return result;
        };
        Brackets.prototype.onElectricCharacter = function (context, offset) {
            if (context.getTokenCount() === 0) {
                return null;
            }
            return (this._onElectricCharacterDocComment(context, offset) ||
                this._onElectricCharacterStandardBrackets(context, offset));
        };
        Brackets.prototype.containsTokenTypes = function (fullTokenSpec, tokensToLookFor) {
            var array = tokensToLookFor.split('.');
            for (var i = 0; i < array.length; ++i) {
                if (fullTokenSpec.indexOf(array[i]) < 0) {
                    return false;
                }
            }
            return true;
        };
        Brackets.prototype._onElectricCharacterStandardBrackets = function (context, offset) {
            if (!this._richEditBrackets || this._richEditBrackets.brackets.length === 0) {
                return null;
            }
            var reversedBracketRegex = this._richEditBrackets.reversedRegex;
            var lineText = context.getLineContent();
            var tokenIndex = context.findIndexOfOffset(offset);
            var tokenStart = context.getTokenStartIndex(tokenIndex);
            var tokenEnd = offset + 1;
            var firstNonWhitespaceIndex = strings.firstNonWhitespaceIndex(context.getLineContent());
            if (firstNonWhitespaceIndex !== -1 && firstNonWhitespaceIndex < tokenStart) {
                return null;
            }
            if (!supports_1.ignoreBracketsInToken(context.getTokenType(tokenIndex))) {
                var r = richEditBrackets_1.BracketsUtils.findPrevBracketInToken(reversedBracketRegex, 1, lineText, tokenStart, tokenEnd);
                if (r) {
                    var text = lineText.substring(r.startColumn - 1, r.endColumn - 1);
                    var isOpen = this._richEditBrackets.textIsOpenBracket[text];
                    if (!isOpen) {
                        return {
                            matchOpenBracket: text
                        };
                    }
                }
            }
            return null;
        };
        Brackets.prototype._onElectricCharacterDocComment = function (context, offset) {
            // We only auto-close, so do nothing if there is no closing part.
            if (!this._docComment || !this._docComment.close) {
                return null;
            }
            var line = context.getLineContent();
            var char = line[offset];
            // See if the right electric character was pressed
            if (char !== this._docComment.open.charAt(this._docComment.open.length - 1)) {
                return null;
            }
            // If this line already contains the closing tag, do nothing.
            if (line.indexOf(this._docComment.close, offset) >= 0) {
                return null;
            }
            // If we're not in a documentation comment, do nothing.
            var lastTokenIndex = context.findIndexOfOffset(offset);
            if (!this.containsTokenTypes(context.getTokenType(lastTokenIndex), this._docComment.scope)) {
                return null;
            }
            if (line.substring(context.getTokenStartIndex(lastTokenIndex), offset + 1 /* include electric char*/) !== this._docComment.open) {
                return null;
            }
            return { appendText: this._docComment.close };
        };
        return Brackets;
    }());
    exports.Brackets = Brackets;
});

define("vs/editor/common/services/editorSimpleWorkerCommon", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EditorSimpleWorker = (function () {
        function EditorSimpleWorker() {
        }
        EditorSimpleWorker.prototype.acceptNewModel = function (data) {
            throw new Error('Not implemented!');
        };
        EditorSimpleWorker.prototype.acceptModelChanged = function (modelUrl, events) {
            throw new Error('Not implemented!');
        };
        EditorSimpleWorker.prototype.acceptRemovedModel = function (modelUrl) {
            throw new Error('Not implemented!');
        };
        EditorSimpleWorker.prototype.computeDiff = function (originalUrl, modifiedUrl, ignoreTrimWhitespace) {
            throw new Error('Not implemented!');
        };
        EditorSimpleWorker.prototype.computeDirtyDiff = function (originalUrl, modifiedUrl, ignoreTrimWhitespace) {
            throw new Error('Not implemented!');
        };
        EditorSimpleWorker.prototype.computeLinks = function (modelUrl) {
            throw new Error('Not implemented!');
        };
        EditorSimpleWorker.prototype.textualSuggest = function (modelUrl, position, wordDef, wordDefFlags) {
            throw new Error('Not implemented!');
        };
        EditorSimpleWorker.prototype.navigateValueSet = function (modelUrl, range, up, wordDef, wordDefFlags) {
            throw new Error('Not implemented!');
        };
        return EditorSimpleWorker;
    }());
    exports.EditorSimpleWorker = EditorSimpleWorker;
});

define("vs/editor/common/viewLayout/editorLayoutProvider", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EditorLayoutProvider = (function () {
        function EditorLayoutProvider() {
        }
        EditorLayoutProvider.compute = function (opts) {
            var lineNumbersWidth = this.computeLineNumbersWidth(opts);
            var glyphMarginWidth = this.computeGlyphMarginWidth(opts);
            var contentWidth = opts.outerWidth - glyphMarginWidth - lineNumbersWidth - opts.lineDecorationsWidth;
            var glyphMarginLeft = 0;
            var lineNumbersLeft = glyphMarginLeft + glyphMarginWidth;
            var decorationsLeft = lineNumbersLeft + lineNumbersWidth;
            var contentLeft = decorationsLeft + opts.lineDecorationsWidth;
            var verticalArrowSize = (opts.verticalScrollbarHasArrows ? opts.scrollbarArrowSize : 0);
            return {
                width: opts.outerWidth,
                height: opts.outerHeight,
                glyphMarginLeft: glyphMarginLeft,
                glyphMarginWidth: glyphMarginWidth,
                glyphMarginHeight: opts.outerHeight,
                lineNumbersLeft: lineNumbersLeft,
                lineNumbersWidth: lineNumbersWidth,
                lineNumbersHeight: opts.outerHeight,
                decorationsLeft: decorationsLeft,
                decorationsWidth: opts.lineDecorationsWidth,
                decorationsHeight: opts.outerHeight,
                contentLeft: contentLeft,
                contentWidth: contentWidth,
                contentHeight: opts.outerHeight,
                verticalScrollbarWidth: opts.verticalScrollbarWidth,
                horizontalScrollbarHeight: opts.horizontalScrollbarHeight,
                overviewRuler: {
                    top: verticalArrowSize,
                    width: opts.verticalScrollbarWidth,
                    height: (opts.outerHeight - 2 * verticalArrowSize),
                    right: 0
                }
            };
        };
        EditorLayoutProvider.layoutEqual = function (a, b) {
            return (a.width === b.width
                && a.height === b.height
                && a.glyphMarginLeft === b.glyphMarginLeft
                && a.glyphMarginWidth === b.glyphMarginWidth
                && a.glyphMarginHeight === b.glyphMarginHeight
                && a.lineNumbersLeft === b.lineNumbersLeft
                && a.lineNumbersWidth === b.lineNumbersWidth
                && a.lineNumbersHeight === b.lineNumbersHeight
                && a.decorationsLeft === b.decorationsLeft
                && a.decorationsWidth === b.decorationsWidth
                && a.decorationsHeight === b.decorationsHeight
                && a.contentLeft === b.contentLeft
                && a.contentWidth === b.contentWidth
                && a.contentHeight === b.contentHeight
                && a.verticalScrollbarWidth === b.verticalScrollbarWidth
                && a.horizontalScrollbarHeight === b.horizontalScrollbarHeight
                && a.overviewRuler.top === b.overviewRuler.top
                && a.overviewRuler.width === b.overviewRuler.width
                && a.overviewRuler.height === b.overviewRuler.height
                && a.overviewRuler.right === b.overviewRuler.right);
        };
        EditorLayoutProvider.computeGlyphMarginWidth = function (opts) {
            if (opts.showGlyphMargin) {
                return opts.lineHeight;
            }
            return 0;
        };
        EditorLayoutProvider.digitCount = function (n) {
            var r = 0;
            while (n) {
                n = Math.floor(n / 10);
                r++;
            }
            return r ? r : 1;
        };
        EditorLayoutProvider.computeLineNumbersWidth = function (opts) {
            if (opts.showLineNumbers) {
                var digitCount = Math.max(this.digitCount(opts.lineCount), opts.lineNumbersMinChars);
                return Math.round(digitCount * opts.maxDigitWidth);
            }
            return 0;
        };
        return EditorLayoutProvider;
    }());
    exports.EditorLayoutProvider = EditorLayoutProvider;
});

define("vs/editor/common/viewLayout/viewLineParts", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/core/arrays', 'vs/editor/common/core/range'], function (require, exports, strings, arrays_1, range_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function cmpLineDecorations(a, b) {
        return range_1.Range.compareRangesUsingStarts(a.range, b.range);
    }
    function createLineParts(lineNumber, minLineColumn, lineContent, lineTokens, rawLineDecorations, renderWhitespace) {
        if (renderWhitespace) {
            var oldLength = rawLineDecorations.length;
            rawLineDecorations = insertWhitespace(lineNumber, lineContent, lineTokens.getFauxIndentLength(), rawLineDecorations);
            if (rawLineDecorations.length !== oldLength) {
                rawLineDecorations.sort(cmpLineDecorations);
            }
        }
        if (rawLineDecorations.length > 0) {
            return new ViewLineParts(lineNumber, minLineColumn, lineTokens, lineContent, rawLineDecorations);
        }
        else {
            return new FastViewLineParts(lineTokens, lineContent);
        }
    }
    exports.createLineParts = createLineParts;
    function trimEmptyTrailingPart(parts, lineContent) {
        if (parts.length <= 1) {
            return parts;
        }
        var lastPartStartIndex = parts[parts.length - 1].startIndex;
        if (lastPartStartIndex < lineContent.length) {
            // All is good
            return parts;
        }
        // Remove last line part
        return parts.slice(0, parts.length - 1);
    }
    function insertWhitespace(lineNumber, lineContent, fauxIndentLength, rawLineDecorations) {
        if (lineContent.length === 0) {
            return rawLineDecorations;
        }
        var prepend = null, append = null, computeTrailing = true;
        var firstNonWhitespaceIndex = strings.firstNonWhitespaceIndex(lineContent);
        if (firstNonWhitespaceIndex !== 0) {
            // There is leading whitespace
            if (firstNonWhitespaceIndex === -1) {
                // The entire string is whitespace
                computeTrailing = false;
                prepend = {
                    startLineNumber: lineNumber,
                    endLineNumber: lineNumber,
                    startColumn: 1,
                    endColumn: lineContent.length + 1
                };
            }
            else {
                prepend = {
                    startLineNumber: lineNumber,
                    endLineNumber: lineNumber,
                    startColumn: 1,
                    endColumn: firstNonWhitespaceIndex + 1
                };
            }
        }
        // Adjust prepend to `fauxIndentLength`
        if (prepend) {
            if (fauxIndentLength + 1 >= prepend.endColumn) {
                prepend = null;
            }
            else {
                prepend.startColumn = fauxIndentLength + 1;
            }
        }
        if (computeTrailing) {
            var lastNonWhitespaceIndex = strings.lastNonWhitespaceIndex(lineContent);
            if (lastNonWhitespaceIndex !== lineContent.length - 1) {
                // There is trailing whitespace
                // No need to handle the case that the entire string is empty, since it is handled above
                append = {
                    startLineNumber: lineNumber,
                    endLineNumber: lineNumber,
                    startColumn: lastNonWhitespaceIndex + 2,
                    endColumn: lineContent.length + 1
                };
            }
        }
        if (prepend || append) {
            var result = rawLineDecorations.slice(0);
            if (prepend) {
                result.unshift({
                    options: {
                        inlineClassName: 'leading whitespace'
                    },
                    range: prepend
                });
            }
            if (append) {
                result.push({
                    options: {
                        inlineClassName: 'trailing whitespace'
                    },
                    range: append
                });
            }
            return result;
        }
        return rawLineDecorations;
    }
    var FastViewLineParts = (function () {
        function FastViewLineParts(lineTokens, lineContent) {
            this.lineTokens = lineTokens;
            this.parts = lineTokens.getTokens();
            this.parts = trimEmptyTrailingPart(this.parts, lineContent);
        }
        FastViewLineParts.prototype.getParts = function () {
            return this.parts;
        };
        FastViewLineParts.prototype.equals = function (other) {
            if (other instanceof FastViewLineParts) {
                var otherFastLineParts = other;
                return this.lineTokens.equals(otherFastLineParts.lineTokens);
            }
            return false;
        };
        FastViewLineParts.prototype.findIndexOfOffset = function (offset) {
            return arrays_1.Arrays.findIndexInSegmentsArray(this.parts, offset);
        };
        return FastViewLineParts;
    }());
    exports.FastViewLineParts = FastViewLineParts;
    var ViewLineParts = (function () {
        function ViewLineParts(lineNumber, minLineColumn, lineTokens, lineContent, rawLineDecorations) {
            // lineDecorations might overlap on top of each other, so they need to be normalized
            var lineDecorations = LineDecorationsNormalizer.normalize(lineNumber, minLineColumn, rawLineDecorations), lineDecorationsIndex = 0, lineDecorationsLength = lineDecorations.length;
            var actualLineTokens = lineTokens.getTokens(), nextStartOffset, currentTokenEndOffset, currentTokenClassName;
            var parts = [];
            for (var i = 0, len = actualLineTokens.length; i < len; i++) {
                nextStartOffset = actualLineTokens[i].startIndex;
                currentTokenEndOffset = (i + 1 < len ? actualLineTokens[i + 1].startIndex : lineTokens.getTextLength());
                currentTokenClassName = actualLineTokens[i].type;
                while (lineDecorationsIndex < lineDecorationsLength && lineDecorations[lineDecorationsIndex].startOffset < currentTokenEndOffset) {
                    if (lineDecorations[lineDecorationsIndex].startOffset > nextStartOffset) {
                        // the first decorations starts after the token
                        parts.push(new LinePart(nextStartOffset, currentTokenClassName));
                        nextStartOffset = lineDecorations[lineDecorationsIndex].startOffset;
                    }
                    parts.push(new LinePart(nextStartOffset, currentTokenClassName + ' ' + lineDecorations[lineDecorationsIndex].className));
                    if (lineDecorations[lineDecorationsIndex].endOffset >= currentTokenEndOffset) {
                        // this decoration goes on to the next token
                        nextStartOffset = currentTokenEndOffset;
                        break;
                    }
                    else {
                        // this decorations stops inside this token
                        nextStartOffset = lineDecorations[lineDecorationsIndex].endOffset + 1;
                        lineDecorationsIndex++;
                    }
                }
                if (nextStartOffset < currentTokenEndOffset) {
                    parts.push(new LinePart(nextStartOffset, currentTokenClassName));
                }
            }
            this.parts = parts;
            this.lastPartIndex = parts.length - 1;
            this.lastEndOffset = currentTokenEndOffset;
        }
        ViewLineParts.prototype.getParts = function () {
            return this.parts;
        };
        ViewLineParts.prototype.equals = function (other) {
            if (other instanceof ViewLineParts) {
                var otherSimpleLineParts = other;
                if (this.lastPartIndex !== otherSimpleLineParts.lastPartIndex) {
                    return false;
                }
                if (this.lastEndOffset !== otherSimpleLineParts.lastEndOffset) {
                    return false;
                }
                for (var i = 0, len = this.parts.length; i < len; i++) {
                    if (this.parts[i].startIndex !== otherSimpleLineParts.parts[i].startIndex) {
                        return false;
                    }
                    if (this.parts[i].type !== otherSimpleLineParts.parts[i].type) {
                        return false;
                    }
                }
                return true;
            }
            return false;
        };
        ViewLineParts.prototype.findIndexOfOffset = function (offset) {
            return arrays_1.Arrays.findIndexInSegmentsArray(this.parts, offset);
        };
        return ViewLineParts;
    }());
    exports.ViewLineParts = ViewLineParts;
    var LinePart = (function () {
        function LinePart(startIndex, type) {
            this.startIndex = startIndex;
            this.type = type;
        }
        return LinePart;
    }());
    var DecorationSegment = (function () {
        function DecorationSegment(startOffset, endOffset, className) {
            this.startOffset = startOffset;
            this.endOffset = endOffset;
            this.className = className;
        }
        return DecorationSegment;
    }());
    exports.DecorationSegment = DecorationSegment;
    var Stack = (function () {
        function Stack() {
            this.stopOffsets = [];
            this.classNames = [];
            this.count = 0;
        }
        Stack.prototype.consumeLowerThan = function (maxStopOffset, nextStartOffset, result) {
            while (this.count > 0 && this.stopOffsets[0] < maxStopOffset) {
                var i = 0;
                // Take all equal stopping offsets
                while (i + 1 < this.count && this.stopOffsets[i] === this.stopOffsets[i + 1]) {
                    i++;
                }
                // Basically we are consuming the first i + 1 elements of the stack
                result.push(new DecorationSegment(nextStartOffset, this.stopOffsets[i], this.classNames.join(' ')));
                nextStartOffset = this.stopOffsets[i] + 1;
                // Consume them
                this.stopOffsets.splice(0, i + 1);
                this.classNames.splice(0, i + 1);
                this.count -= (i + 1);
            }
            if (this.count > 0 && nextStartOffset < maxStopOffset) {
                result.push(new DecorationSegment(nextStartOffset, maxStopOffset - 1, this.classNames.join(' ')));
                nextStartOffset = maxStopOffset;
            }
            return nextStartOffset;
        };
        Stack.prototype.insert = function (stopOffset, className) {
            if (this.count === 0 || this.stopOffsets[this.count - 1] <= stopOffset) {
                // Insert at the end
                this.stopOffsets.push(stopOffset);
                this.classNames.push(className);
            }
            else {
                // Find the insertion position for `stopOffset`
                for (var i = 0; i < this.count; i++) {
                    if (this.stopOffsets[i] >= stopOffset) {
                        this.stopOffsets.splice(i, 0, stopOffset);
                        this.classNames.splice(i, 0, className);
                        break;
                    }
                }
            }
            this.count++;
            return;
        };
        return Stack;
    }());
    var LineDecorationsNormalizer = (function () {
        function LineDecorationsNormalizer() {
        }
        /**
         * Normalize line decorations. Overlapping decorations will generate multiple segments
         */
        LineDecorationsNormalizer.normalize = function (lineNumber, minLineColumn, lineDecorations) {
            var result = [];
            if (lineDecorations.length === 0) {
                return result;
            }
            var stack = new Stack(), nextStartOffset = 0, d, currentStartOffset, currentEndOffset, i, len;
            for (i = 0, len = lineDecorations.length; i < len; i++) {
                d = lineDecorations[i];
                if (d.range.endLineNumber < lineNumber || d.range.startLineNumber > lineNumber) {
                    // Ignore decorations that sit outside this line
                    continue;
                }
                if (d.range.startLineNumber === d.range.endLineNumber && d.range.startColumn === d.range.endColumn) {
                    // Ignore empty range decorations
                    continue;
                }
                currentStartOffset = (d.range.startLineNumber === lineNumber ? d.range.startColumn - 1 : minLineColumn - 1);
                currentEndOffset = (d.range.endLineNumber === lineNumber ? d.range.endColumn - 2 : LineDecorationsNormalizer.MAX_LINE_LENGTH - 1);
                if (currentEndOffset < 0) {
                    // An empty decoration (endColumn === 1)
                    continue;
                }
                nextStartOffset = stack.consumeLowerThan(currentStartOffset, nextStartOffset, result);
                if (stack.count === 0) {
                    nextStartOffset = currentStartOffset;
                }
                stack.insert(currentEndOffset, d.options.inlineClassName);
            }
            stack.consumeLowerThan(LineDecorationsNormalizer.MAX_LINE_LENGTH, nextStartOffset, result);
            return result;
        };
        /**
         * A number that is guaranteed to be larger than the maximum line column
         */
        LineDecorationsNormalizer.MAX_LINE_LENGTH = 10000000;
        return LineDecorationsNormalizer;
    }());
    exports.LineDecorationsNormalizer = LineDecorationsNormalizer;
});

define("vs/editor/common/viewLayout/viewLineRenderer", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var _space = ' '.charCodeAt(0);
    var _tab = '\t'.charCodeAt(0);
    var _lowerThan = '<'.charCodeAt(0);
    var _greaterThan = '>'.charCodeAt(0);
    var _ampersand = '&'.charCodeAt(0);
    var _carriageReturn = '\r'.charCodeAt(0);
    var _lineSeparator = '\u2028'.charCodeAt(0); //http://www.fileformat.info/info/unicode/char/2028/index.htm
    var _bom = 65279;
    function renderLine(input) {
        var lineText = input.lineContent;
        var lineTextLength = lineText.length;
        var tabSize = input.tabSize;
        var actualLineParts = input.parts;
        var renderWhitespace = input.renderWhitespace;
        var charBreakIndex = (input.stopRenderingLineAfter === -1 ? lineTextLength : input.stopRenderingLineAfter - 1);
        if (lineTextLength === 0) {
            return {
                charOffsetInPart: [],
                lastRenderedPartIndex: 0,
                // This is basically for IE's hit test to work
                output: ['<span><span>&nbsp;</span></span>']
            };
        }
        if (actualLineParts.length === 0) {
            throw new Error('Cannot render non empty line without line parts!');
        }
        var charIndex = 0;
        var out = [];
        var charOffsetInPartArr = [];
        var charOffsetInPart = 0;
        var tabsCharDelta = 0;
        out.push('<span>');
        for (var partIndex = 0, partIndexLen = actualLineParts.length; partIndex < partIndexLen; partIndex++) {
            var part = actualLineParts[partIndex];
            out.push('<span class="');
            out.push('token ');
            out.push(part.type.replace(/[^a-z0-9\-]/gi, ' '));
            out.push('">');
            var partRendersWhitespace = false;
            if (renderWhitespace) {
                partRendersWhitespace = (/\bwhitespace\b/.test(part.type));
            }
            var toCharIndex = lineTextLength;
            if (partIndex + 1 < partIndexLen) {
                toCharIndex = Math.min(lineTextLength, actualLineParts[partIndex + 1].startIndex);
            }
            charOffsetInPart = 0;
            for (; charIndex < toCharIndex; charIndex++) {
                charOffsetInPartArr[charIndex] = charOffsetInPart;
                var charCode = lineText.charCodeAt(charIndex);
                switch (charCode) {
                    case _tab:
                        var insertSpacesCount = tabSize - (charIndex + tabsCharDelta) % tabSize;
                        tabsCharDelta += insertSpacesCount - 1;
                        charOffsetInPart += insertSpacesCount - 1;
                        if (insertSpacesCount > 0) {
                            out.push(partRendersWhitespace ? '&rarr;' : '&nbsp;');
                            insertSpacesCount--;
                        }
                        while (insertSpacesCount > 0) {
                            out.push('&nbsp;');
                            insertSpacesCount--;
                        }
                        break;
                    case _space:
                        out.push(partRendersWhitespace ? '&middot;' : '&nbsp;');
                        break;
                    case _lowerThan:
                        out.push('&lt;');
                        break;
                    case _greaterThan:
                        out.push('&gt;');
                        break;
                    case _ampersand:
                        out.push('&amp;');
                        break;
                    case 0:
                        out.push('&#00;');
                        break;
                    case _bom:
                    case _lineSeparator:
                        out.push('\ufffd');
                        break;
                    case _carriageReturn:
                        // zero width space, because carriage return would introduce a line break
                        out.push('&#8203');
                        break;
                    default:
                        out.push(lineText.charAt(charIndex));
                }
                charOffsetInPart++;
                if (charIndex >= charBreakIndex) {
                    out.push('&hellip;</span></span>');
                    charOffsetInPartArr[charOffsetInPartArr.length - 1]++;
                    return {
                        charOffsetInPart: charOffsetInPartArr,
                        lastRenderedPartIndex: partIndex,
                        output: out
                    };
                }
            }
            out.push('</span>');
        }
        out.push('</span>');
        // When getting client rects for the last character, we will position the
        // text range at the end of the span, insteaf of at the beginning of next span
        charOffsetInPartArr.push(charOffsetInPart);
        return {
            charOffsetInPart: charOffsetInPartArr,
            lastRenderedPartIndex: actualLineParts.length - 1,
            output: out
        };
    }
    exports.renderLine = renderLine;
});

define("vs/editor/common/viewLayout/whitespaceComputer", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Represent whitespaces in between lines and provide fast CRUD management methods.
     * The whitespaces are sorted ascending by `afterLineNumber`.
     */
    var WhitespaceComputer = (function () {
        function WhitespaceComputer() {
            this.heights = [];
            this.ids = [];
            this.afterLineNumbers = [];
            this.ordinals = [];
            this.prefixSum = [];
            this.prefixSumValidIndex = -1;
            this.whitespaceId2Index = {};
            this.lastWhitespaceId = 0;
        }
        /**
         * Find the insertion index for a new value inside a sorted array of values.
         * If the value is already present in the sorted array, the insertion index will be after the already existing value.
         */
        WhitespaceComputer.findInsertionIndex = function (sortedArray, value, ordinals, valueOrdinal) {
            var low = 0, high = sortedArray.length, mid;
            while (low < high) {
                mid = Math.floor((low + high) / 2);
                if (value === sortedArray[mid]) {
                    if (valueOrdinal < ordinals[mid]) {
                        high = mid;
                    }
                    else {
                        low = mid + 1;
                    }
                }
                else if (value < sortedArray[mid]) {
                    high = mid;
                }
                else {
                    low = mid + 1;
                }
            }
            return low;
        };
        /**
         * Insert a new whitespace of a certain height after a line number.
         * The whitespace has a "sticky" characteristic.
         * Irrespective of edits above or below `afterLineNumber`, the whitespace will follow the initial line.
         *
         * @param afterLineNumber The conceptual position of this whitespace. The whitespace will follow this line as best as possible even when deleting/inserting lines above/below.
         * @param heightInPx The height of the whitespace, in pixels.
         * @return An id that can be used later to mutate or delete the whitespace
         */
        WhitespaceComputer.prototype.insertWhitespace = function (afterLineNumber, ordinal, heightInPx) {
            var id = (++this.lastWhitespaceId);
            var insertionIndex = WhitespaceComputer.findInsertionIndex(this.afterLineNumbers, afterLineNumber, this.ordinals, ordinal);
            this.insertWhitespaceAtIndex(id, insertionIndex, afterLineNumber, ordinal, heightInPx);
            return id;
        };
        WhitespaceComputer.prototype.insertWhitespaceAtIndex = function (id, insertIndex, afterLineNumber, ordinal, heightInPx) {
            this.heights.splice(insertIndex, 0, heightInPx);
            this.ids.splice(insertIndex, 0, id);
            this.afterLineNumbers.splice(insertIndex, 0, afterLineNumber);
            this.ordinals.splice(insertIndex, 0, ordinal);
            this.prefixSum.splice(insertIndex, 0, 0);
            var sid, oldIndex;
            for (sid in this.whitespaceId2Index) {
                if (this.whitespaceId2Index.hasOwnProperty(sid)) {
                    oldIndex = this.whitespaceId2Index[sid];
                    if (oldIndex >= insertIndex) {
                        this.whitespaceId2Index[sid] = oldIndex + 1;
                    }
                }
            }
            this.whitespaceId2Index[id.toString()] = insertIndex;
            this.prefixSumValidIndex = Math.min(this.prefixSumValidIndex, insertIndex - 1);
        };
        WhitespaceComputer.prototype.changeWhitespace = function (id, newAfterLineNumber, newHeight) {
            var hasChanges = false;
            hasChanges = this.changeWhitespaceHeight(id, newHeight) || hasChanges;
            hasChanges = this.changeWhitespaceAfterLineNumber(id, newAfterLineNumber) || hasChanges;
            return hasChanges;
        };
        /**
         * Change the height of an existing whitespace
         *
         * @param id The whitespace to change
         * @param newHeightInPx The new height of the whitespace, in pixels
         * @return Returns true if the whitespace is found and if the new height is different than the old height
         */
        WhitespaceComputer.prototype.changeWhitespaceHeight = function (id, newHeightInPx) {
            var sid = id.toString();
            if (this.whitespaceId2Index.hasOwnProperty(sid)) {
                var index = this.whitespaceId2Index[sid];
                if (this.heights[index] !== newHeightInPx) {
                    this.heights[index] = newHeightInPx;
                    this.prefixSumValidIndex = Math.min(this.prefixSumValidIndex, index - 1);
                    return true;
                }
            }
            return false;
        };
        /**
         * Change the line number after which an existing whitespace flows.
         *
         * @param id The whitespace to change
         * @param newAfterLineNumber The new line number the whitespace will follow
         * @return Returns true if the whitespace is found and if the new line number is different than the old line number
         */
        WhitespaceComputer.prototype.changeWhitespaceAfterLineNumber = function (id, newAfterLineNumber) {
            var sid = id.toString();
            if (this.whitespaceId2Index.hasOwnProperty(sid)) {
                var index = this.whitespaceId2Index[sid];
                if (this.afterLineNumbers[index] !== newAfterLineNumber) {
                    // `afterLineNumber` changed for this whitespace
                    // Record old ordinal
                    var ordinal = this.ordinals[index];
                    // Record old height
                    var heightInPx = this.heights[index];
                    // Since changing `afterLineNumber` can trigger a reordering, we're gonna remove this whitespace
                    this.removeWhitespace(id);
                    // And add it again
                    var insertionIndex = WhitespaceComputer.findInsertionIndex(this.afterLineNumbers, newAfterLineNumber, this.ordinals, ordinal);
                    this.insertWhitespaceAtIndex(id, insertionIndex, newAfterLineNumber, ordinal, heightInPx);
                    return true;
                }
            }
            return false;
        };
        /**
         * Remove an existing whitespace.
         *
         * @param id The whitespace to remove
         * @return Returns true if the whitespace is found and it is removed.
         */
        WhitespaceComputer.prototype.removeWhitespace = function (id) {
            var sid = id.toString();
            if (this.whitespaceId2Index.hasOwnProperty(sid)) {
                var index = this.whitespaceId2Index[sid];
                delete this.whitespaceId2Index[sid];
                this.removeWhitespaceAtIndex(index);
                return true;
            }
            return false;
        };
        WhitespaceComputer.prototype.removeWhitespaceAtIndex = function (removeIndex) {
            this.heights.splice(removeIndex, 1);
            this.ids.splice(removeIndex, 1);
            this.afterLineNumbers.splice(removeIndex, 1);
            this.ordinals.splice(removeIndex, 1);
            this.prefixSum.splice(removeIndex, 1);
            this.prefixSumValidIndex = Math.min(this.prefixSumValidIndex, removeIndex - 1);
            var sid, oldIndex;
            for (sid in this.whitespaceId2Index) {
                if (this.whitespaceId2Index.hasOwnProperty(sid)) {
                    oldIndex = this.whitespaceId2Index[sid];
                    if (oldIndex >= removeIndex) {
                        this.whitespaceId2Index[sid] = oldIndex - 1;
                    }
                }
            }
        };
        /**
         * Notify the computer that lines have been deleted (a continuous zone of lines).
         * This gives it a chance to update `afterLineNumber` for whitespaces, giving the "sticky" characteristic.
         *
         * @param fromLineNumber The line number at which the deletion started, inclusive
         * @param toLineNumber The line number at which the deletion ended, inclusive
         */
        WhitespaceComputer.prototype.onModelLinesDeleted = function (fromLineNumber, toLineNumber) {
            var afterLineNumber, i, len;
            for (i = 0, len = this.afterLineNumbers.length; i < len; i++) {
                afterLineNumber = this.afterLineNumbers[i];
                if (fromLineNumber <= afterLineNumber && afterLineNumber <= toLineNumber) {
                    // The line this whitespace was after has been deleted
                    //  => move whitespace to before first deleted line
                    this.afterLineNumbers[i] = fromLineNumber - 1;
                }
                else if (afterLineNumber > toLineNumber) {
                    // The line this whitespace was after has been moved up
                    //  => move whitespace up
                    this.afterLineNumbers[i] -= (toLineNumber - fromLineNumber + 1);
                }
            }
        };
        /**
         * Notify the computer that lines have been inserted (a continuous zone of lines).
         * This gives it a chance to update `afterLineNumber` for whitespaces, giving the "sticky" characteristic.
         *
         * @param fromLineNumber The line number at which the insertion started, inclusive
         * @param toLineNumber The line number at which the insertion ended, inclusive.
         */
        WhitespaceComputer.prototype.onModelLinesInserted = function (fromLineNumber, toLineNumber) {
            var afterLineNumber, i, len;
            for (i = 0, len = this.afterLineNumbers.length; i < len; i++) {
                afterLineNumber = this.afterLineNumbers[i];
                if (fromLineNumber <= afterLineNumber) {
                    this.afterLineNumbers[i] += (toLineNumber - fromLineNumber + 1);
                }
            }
        };
        /**
         * Get the sum of all the whitespaces.
         */
        WhitespaceComputer.prototype.getTotalHeight = function () {
            if (this.heights.length === 0) {
                return 0;
            }
            return this.getAccumulatedHeight(this.heights.length - 1);
        };
        /**
         * Return the sum of the heights of the whitespaces at [0..index].
         * This includes the whitespace at `index`.
         *
         * @param index The index of the whitespace.
         * @return The sum of the heights of all whitespaces before the one at `index`, including the one at `index`.
         */
        WhitespaceComputer.prototype.getAccumulatedHeight = function (index) {
            var startIndex = Math.max(0, this.prefixSumValidIndex + 1);
            if (startIndex === 0) {
                this.prefixSum[0] = this.heights[0];
                startIndex++;
            }
            for (var i = startIndex; i <= index; i++) {
                this.prefixSum[i] = this.prefixSum[i - 1] + this.heights[i];
            }
            this.prefixSumValidIndex = Math.max(this.prefixSumValidIndex, index);
            return this.prefixSum[index];
        };
        /**
         * Find all whitespaces with `afterLineNumber` < `lineNumber` and return the sum of their heights.
         *
         * @param lineNumber The line number whitespaces should be before.
         * @return The sum of the heights of the whitespaces before `lineNumber`.
         */
        WhitespaceComputer.prototype.getAccumulatedHeightBeforeLineNumber = function (lineNumber) {
            var lastWhitespaceBeforeLineNumber = this.findLastWhitespaceBeforeLineNumber(lineNumber);
            if (lastWhitespaceBeforeLineNumber === -1) {
                return 0;
            }
            return this.getAccumulatedHeight(lastWhitespaceBeforeLineNumber);
        };
        WhitespaceComputer.prototype.findLastWhitespaceBeforeLineNumber = function (lineNumber) {
            // Find the whitespace before line number
            var afterLineNumbers = this.afterLineNumbers, low = 0, high = afterLineNumbers.length - 1, mid;
            while (low <= high) {
                mid = Math.floor((low + high) / 2);
                if (afterLineNumbers[mid] < lineNumber) {
                    if (mid + 1 >= afterLineNumbers.length || afterLineNumbers[mid + 1] >= lineNumber) {
                        return mid;
                    }
                    else {
                        low = mid + 1;
                    }
                }
                else {
                    high = mid - 1;
                }
            }
            return -1;
        };
        WhitespaceComputer.prototype.findFirstWhitespaceAfterLineNumber = function (lineNumber) {
            var lastWhitespaceBeforeLineNumber = this.findLastWhitespaceBeforeLineNumber(lineNumber);
            var firstWhitespaceAfterLineNumber = lastWhitespaceBeforeLineNumber + 1;
            if (firstWhitespaceAfterLineNumber < this.heights.length) {
                return firstWhitespaceAfterLineNumber;
            }
            return -1;
        };
        /**
         * Find the index of the first whitespace which has `afterLineNumber` >= `lineNumber`.
         * @return The index of the first whitespace with `afterLineNumber` >= `lineNumber` or -1 if no whitespace is found.
         */
        WhitespaceComputer.prototype.getFirstWhitespaceIndexAfterLineNumber = function (lineNumber) {
            return this.findFirstWhitespaceAfterLineNumber(lineNumber);
        };
        /**
         * The number of whitespaces.
         */
        WhitespaceComputer.prototype.getCount = function () {
            return this.heights.length;
        };
        /**
         * Get the `afterLineNumber` for whitespace at index `index`.
         *
         * @param index The index of the whitespace.
         * @return `afterLineNumber` of whitespace at `index`.
         */
        WhitespaceComputer.prototype.getAfterLineNumberForWhitespaceIndex = function (index) {
            return this.afterLineNumbers[index];
        };
        /**
         * Get the `id` for whitespace at index `index`.
         *
         * @param index The index of the whitespace.
         * @return `id` of whitespace at `index`.
         */
        WhitespaceComputer.prototype.getIdForWhitespaceIndex = function (index) {
            return this.ids[index];
        };
        /**
         * Get the `height` for whitespace at index `index`.
         *
         * @param index The index of the whitespace.
         * @return `height` of whitespace at `index`.
         */
        WhitespaceComputer.prototype.getHeightForWhitespaceIndex = function (index) {
            return this.heights[index];
        };
        WhitespaceComputer.prototype.getWhitespaces = function (deviceLineHeight) {
            var result = [];
            for (var i = 0; i < this.heights.length; i++) {
                result.push({
                    id: this.ids[i],
                    afterLineNumber: this.afterLineNumbers[i],
                    heightInLines: this.heights[i] / deviceLineHeight
                });
            }
            return result;
        };
        return WhitespaceComputer;
    }());
    exports.WhitespaceComputer = WhitespaceComputer;
});

define("vs/editor/common/viewLayout/verticalObjects", ["require", "exports", 'vs/editor/common/viewLayout/whitespaceComputer'], function (require, exports, whitespaceComputer_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Layouting of objects that take vertical space (by having a height) and push down other objects.
     *
     * These objects are basically either text (lines) or spaces between those lines (whitespaces).
     * This provides commodity operations for working with lines that contain whitespace that pushes lines lower (vertically).
     * This is written with no knowledge of an editor in mind.
     */
    var VerticalObjects = (function () {
        function VerticalObjects() {
            this.whitespaces = new whitespaceComputer_1.WhitespaceComputer();
        }
        /**
         * Set the number of lines.
         *
         * @param newLineCount New number of lines.
         */
        VerticalObjects.prototype.replaceLines = function (newLineCount) {
            this.linesCount = newLineCount;
        };
        /**
         * Insert a new whitespace of a certain height after a line number.
         * The whitespace has a "sticky" characteristic.
         * Irrespective of edits above or below `afterLineNumber`, the whitespace will follow the initial line.
         *
         * @param afterLineNumber The conceptual position of this whitespace. The whitespace will follow this line as best as possible even when deleting/inserting lines above/below.
         * @param heightInPx The height of the whitespace, in pixels.
         * @return An id that can be used later to mutate or delete the whitespace
         */
        VerticalObjects.prototype.insertWhitespace = function (afterLineNumber, ordinal, heightInPx) {
            return this.whitespaces.insertWhitespace(afterLineNumber, ordinal, heightInPx);
        };
        VerticalObjects.prototype.changeWhitespace = function (id, newAfterLineNumber, newHeight) {
            return this.whitespaces.changeWhitespace(id, newAfterLineNumber, newHeight);
        };
        /**
         * Remove an existing whitespace.
         *
         * @param id The whitespace to remove
         * @return Returns true if the whitespace is found and it is removed.
         */
        VerticalObjects.prototype.removeWhitespace = function (id) {
            return this.whitespaces.removeWhitespace(id);
        };
        /**
         * Notify the layouter that lines have been deleted (a continuous zone of lines).
         *
         * @param fromLineNumber The line number at which the deletion started, inclusive
         * @param toLineNumber The line number at which the deletion ended, inclusive
         */
        VerticalObjects.prototype.onModelLinesDeleted = function (fromLineNumber, toLineNumber) {
            this.linesCount -= (toLineNumber - fromLineNumber + 1);
            this.whitespaces.onModelLinesDeleted(fromLineNumber, toLineNumber);
        };
        /**
         * Notify the layouter that lines have been inserted (a continuous zone of lines).
         *
         * @param fromLineNumber The line number at which the insertion started, inclusive
         * @param toLineNumber The line number at which the insertion ended, inclusive.
         */
        VerticalObjects.prototype.onModelLinesInserted = function (fromLineNumber, toLineNumber) {
            this.linesCount += (toLineNumber - fromLineNumber + 1);
            this.whitespaces.onModelLinesInserted(fromLineNumber, toLineNumber);
        };
        /**
         * Get the sum of heights for all objects.
         *
         * @param deviceLineHeight The height, in pixels, for one rendered line.
         * @return The sum of heights for all objects.
         */
        VerticalObjects.prototype.getTotalHeight = function (deviceLineHeight) {
            var linesHeight = deviceLineHeight * this.linesCount;
            var whitespacesHeight = this.whitespaces.getTotalHeight();
            return linesHeight + whitespacesHeight;
        };
        /**
         * Get the vertical offset (the sum of heights for all objects above) a certain line number.
         *
         * @param lineNumber The line number
         * @param deviceLineHeight The height, in pixels, for one rendered line.
         * @return The sum of heights for all objects above `lineNumber`.
         */
        VerticalObjects.prototype.getVerticalOffsetForLineNumber = function (lineNumber, deviceLineHeight) {
            var previousLinesHeight;
            if (lineNumber > 1) {
                previousLinesHeight = deviceLineHeight * (lineNumber - 1);
            }
            else {
                previousLinesHeight = 0;
            }
            var previousWhitespacesHeight = this.whitespaces.getAccumulatedHeightBeforeLineNumber(lineNumber);
            return previousLinesHeight + previousWhitespacesHeight;
        };
        /**
         * Returns the accumulated height of whitespaces before the given line number.
         *
         * @param lineNumber The line number
         */
        VerticalObjects.prototype.getWhitespaceAccumulatedHeightBeforeLineNumber = function (lineNumber) {
            return this.whitespaces.getAccumulatedHeightBeforeLineNumber(lineNumber);
        };
        /**
         * Returns if there is any whitespace in the document.
         */
        VerticalObjects.prototype.hasWhitespace = function () {
            return this.whitespaces.getCount() > 0;
        };
        VerticalObjects.prototype.isAfterLines = function (verticalOffset, deviceLineHeight) {
            var totalHeight = this.getTotalHeight(deviceLineHeight);
            return verticalOffset > totalHeight;
        };
        /**
         * Find the first line number that is at or after vertical offset `verticalOffset`.
         * i.e. if getVerticalOffsetForLine(line) is x and getVerticalOffsetForLine(line + 1) is y, then
         * getLineNumberAtOrAfterVerticalOffset(i) = line, x <= i < y.
         *
         * @param verticalOffset The vertical offset to search at.
         * @param deviceLineHeight The height, in piexels, for one rendered line.
         * @return The line number at or after vertical offset `verticalOffset`.
         */
        VerticalObjects.prototype.getLineNumberAtOrAfterVerticalOffset = function (verticalOffset, deviceLineHeight) {
            if (verticalOffset < 0) {
                return 1;
            }
            var minLineNumber = 1, maxLineNumber = this.linesCount, midLineNumber, midLineNumberVerticalOffset, midLineNumberHeight;
            while (minLineNumber < maxLineNumber) {
                midLineNumber = Math.floor((minLineNumber + maxLineNumber) / 2);
                midLineNumberVerticalOffset = this.getVerticalOffsetForLineNumber(midLineNumber, deviceLineHeight);
                midLineNumberHeight = deviceLineHeight;
                if (verticalOffset >= midLineNumberVerticalOffset + midLineNumberHeight) {
                    // vertical offset is after mid line number
                    minLineNumber = midLineNumber + 1;
                }
                else if (verticalOffset >= midLineNumberVerticalOffset) {
                    // Hit
                    return midLineNumber;
                }
                else {
                    // vertical offset is before mid line number, but mid line number could still be what we're searching for
                    maxLineNumber = midLineNumber;
                }
            }
            if (minLineNumber > this.linesCount) {
                return this.linesCount;
            }
            return minLineNumber;
        };
        /**
         * Get the line that appears visually in the center between `verticalOffset1` and `verticalOffset2`.
         *
         * @param verticalOffset1 The beginning of the viewport
         * @param verticalOffset2 The end of the viewport.
         * @param deviceLineHeight The height, in pixels, for one rendered line.
         * @return The line number that is closest to the center between `verticalOffset1` and `verticalOffset2`.
         */
        VerticalObjects.prototype.getCenteredLineInViewport = function (verticalOffset1, verticalOffset2, deviceLineHeight) {
            var viewportData = this.getLinesViewportData(verticalOffset1, verticalOffset2, deviceLineHeight);
            var verticalCenter = (verticalOffset2 - verticalOffset1) / 2;
            var currentLineActualTop, currentLineActualBottom;
            for (var lineNumber = viewportData.startLineNumber; lineNumber <= viewportData.endLineNumber; lineNumber++) {
                currentLineActualTop = viewportData.visibleRangesDeltaTop + viewportData.relativeVerticalOffset[lineNumber - viewportData.startLineNumber];
                currentLineActualBottom = currentLineActualTop + deviceLineHeight;
                if ((currentLineActualTop <= verticalCenter && verticalCenter < currentLineActualBottom) || currentLineActualTop > verticalCenter) {
                    return lineNumber;
                }
            }
            return viewportData.endLineNumber;
        };
        /**
         * Get all the lines and their relative vertical offsets that are positioned between `verticalOffset1` and `verticalOffset2`.
         *
         * @param verticalOffset1 The beginning of the viewport.
         * @param verticalOffset2 The end of the viewport.
         * @param deviceLineHeight The height, in pixels, for one rendered line.
         * @return A structure describing the lines positioned between `verticalOffset1` and `verticalOffset2`.
         */
        VerticalObjects.prototype.getLinesViewportData = function (verticalOffset1, verticalOffset2, deviceLineHeight) {
            // Find first line number
            // We don't live in a perfect world, so the line number might start before or after verticalOffset1
            var startLineNumber = this.getLineNumberAtOrAfterVerticalOffset(verticalOffset1, deviceLineHeight);
            var endLineNumber = this.linesCount, startLineNumberVerticalOffset = this.getVerticalOffsetForLineNumber(startLineNumber, deviceLineHeight);
            // Also keep track of what whitespace we've got
            var whitespaceIndex = this.whitespaces.getFirstWhitespaceIndexAfterLineNumber(startLineNumber), whitespaceCount = this.whitespaces.getCount(), currentWhitespaceHeight, currentWhitespaceAfterLineNumber;
            if (whitespaceIndex === -1) {
                whitespaceIndex = whitespaceCount;
                currentWhitespaceAfterLineNumber = endLineNumber + 1;
            }
            else {
                currentWhitespaceAfterLineNumber = this.whitespaces.getAfterLineNumberForWhitespaceIndex(whitespaceIndex);
                currentWhitespaceHeight = this.whitespaces.getHeightForWhitespaceIndex(whitespaceIndex);
            }
            var currentVerticalOffset = startLineNumberVerticalOffset;
            var currentLineRelativeOffset = currentVerticalOffset;
            // IE (all versions) cannot handle units above about 1,533,908 px, so every 500k pixels bring numbers down
            var STEP_SIZE = 500000;
            var bigNumbersDelta = 0;
            if (startLineNumberVerticalOffset >= STEP_SIZE) {
                // Compute a delta that guarantees that lines are positioned at `lineHeight` increments
                bigNumbersDelta = Math.floor(startLineNumberVerticalOffset / STEP_SIZE) * STEP_SIZE;
                bigNumbersDelta = Math.floor(bigNumbersDelta / deviceLineHeight) * deviceLineHeight;
                currentLineRelativeOffset -= bigNumbersDelta;
            }
            var linesOffsets = [];
            // Figure out how far the lines go
            for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
                // Count current line height in the vertical offsets
                currentVerticalOffset += deviceLineHeight;
                linesOffsets.push(currentLineRelativeOffset);
                // Next line starts immediately after this one
                currentLineRelativeOffset += deviceLineHeight;
                while (currentWhitespaceAfterLineNumber === lineNumber) {
                    // Push down next line with the height of the current whitespace
                    currentLineRelativeOffset += currentWhitespaceHeight;
                    // Count current whitespace in the vertical offsets
                    currentVerticalOffset += currentWhitespaceHeight;
                    whitespaceIndex++;
                    if (whitespaceIndex >= whitespaceCount) {
                        currentWhitespaceAfterLineNumber = endLineNumber + 1;
                    }
                    else {
                        currentWhitespaceAfterLineNumber = this.whitespaces.getAfterLineNumberForWhitespaceIndex(whitespaceIndex);
                        currentWhitespaceHeight = this.whitespaces.getHeightForWhitespaceIndex(whitespaceIndex);
                    }
                }
                if (currentVerticalOffset > verticalOffset2) {
                    // We have covered the entire viewport area, time to stop
                    endLineNumber = lineNumber;
                    break;
                }
            }
            return {
                viewportTop: verticalOffset1 - bigNumbersDelta,
                viewportHeight: verticalOffset2 - verticalOffset1,
                bigNumbersDelta: bigNumbersDelta,
                startLineNumber: startLineNumber,
                endLineNumber: endLineNumber,
                visibleRangesDeltaTop: -(verticalOffset1 - bigNumbersDelta),
                relativeVerticalOffset: linesOffsets,
                visibleRange: null,
                getInlineDecorationsForLineInViewport: null,
                getDecorationsInViewport: null // This will be filled in by linesLayout
            };
        };
        VerticalObjects.prototype.getVerticalOffsetForWhitespaceIndex = function (whitespaceIndex, deviceLineHeight) {
            var afterLineNumber = this.whitespaces.getAfterLineNumberForWhitespaceIndex(whitespaceIndex);
            var previousLinesHeight;
            if (afterLineNumber >= 1) {
                previousLinesHeight = deviceLineHeight * afterLineNumber;
            }
            else {
                previousLinesHeight = 0;
            }
            var previousWhitespacesHeight;
            if (whitespaceIndex > 0) {
                previousWhitespacesHeight = this.whitespaces.getAccumulatedHeight(whitespaceIndex - 1);
            }
            else {
                previousWhitespacesHeight = 0;
            }
            return previousLinesHeight + previousWhitespacesHeight;
        };
        VerticalObjects.prototype.getWhitespaceIndexAtOrAfterVerticallOffset = function (verticalOffset, deviceLineHeight) {
            var midWhitespaceIndex, minWhitespaceIndex = 0, maxWhitespaceIndex = this.whitespaces.getCount() - 1, midWhitespaceVerticalOffset, midWhitespaceHeight;
            if (maxWhitespaceIndex < 0) {
                return -1;
            }
            // Special case: nothing to be found
            var maxWhitespaceVerticalOffset = this.getVerticalOffsetForWhitespaceIndex(maxWhitespaceIndex, deviceLineHeight);
            var maxWhitespaceHeight = this.whitespaces.getHeightForWhitespaceIndex(maxWhitespaceIndex);
            if (verticalOffset >= maxWhitespaceVerticalOffset + maxWhitespaceHeight) {
                return -1;
            }
            while (minWhitespaceIndex < maxWhitespaceIndex) {
                midWhitespaceIndex = Math.floor((minWhitespaceIndex + maxWhitespaceIndex) / 2);
                midWhitespaceVerticalOffset = this.getVerticalOffsetForWhitespaceIndex(midWhitespaceIndex, deviceLineHeight);
                midWhitespaceHeight = this.whitespaces.getHeightForWhitespaceIndex(midWhitespaceIndex);
                if (verticalOffset >= midWhitespaceVerticalOffset + midWhitespaceHeight) {
                    // vertical offset is after whitespace
                    minWhitespaceIndex = midWhitespaceIndex + 1;
                }
                else if (verticalOffset >= midWhitespaceVerticalOffset) {
                    // Hit
                    return midWhitespaceIndex;
                }
                else {
                    // vertical offset is before whitespace, but midWhitespaceIndex might still be what we're searching for
                    maxWhitespaceIndex = midWhitespaceIndex;
                }
            }
            return minWhitespaceIndex;
        };
        /**
         * Get exactly the whitespace that is layouted at `verticalOffset`.
         *
         * @param verticalOffset The vertical offset.
         * @param deviceLineHeight The height, in pixels, for one rendered line.
         * @return Precisely the whitespace that is layouted at `verticaloffset` or null.
         */
        VerticalObjects.prototype.getWhitespaceAtVerticalOffset = function (verticalOffset, deviceLineHeight) {
            var candidateIndex = this.getWhitespaceIndexAtOrAfterVerticallOffset(verticalOffset, deviceLineHeight);
            if (candidateIndex < 0) {
                return null;
            }
            if (candidateIndex >= this.whitespaces.getCount()) {
                return null;
            }
            var candidateTop = this.getVerticalOffsetForWhitespaceIndex(candidateIndex, deviceLineHeight);
            if (candidateTop > verticalOffset) {
                return null;
            }
            var candidateHeight = this.whitespaces.getHeightForWhitespaceIndex(candidateIndex);
            var candidateId = this.whitespaces.getIdForWhitespaceIndex(candidateIndex);
            var candidateAfterLineNumber = this.whitespaces.getAfterLineNumberForWhitespaceIndex(candidateIndex);
            return {
                id: candidateId,
                afterLineNumber: candidateAfterLineNumber,
                verticalOffset: candidateTop,
                height: candidateHeight
            };
        };
        /**
         * Get a list of whitespaces that are positioned between `verticalOffset1` and `verticalOffset2`.
         *
         * @param verticalOffset1 The beginning of the viewport.
         * @param verticalOffset2 The end of the viewport.
         * @param deviceLineHeight The height, in pixels, for one rendered line.
         * @return An array with all the whitespaces in the viewport. If no whitespace is in viewport, the array is empty.
         */
        VerticalObjects.prototype.getWhitespaceViewportData = function (verticalOffset1, verticalOffset2, deviceLineHeight) {
            var startIndex = this.getWhitespaceIndexAtOrAfterVerticallOffset(verticalOffset1, deviceLineHeight);
            var endIndex = this.whitespaces.getCount() - 1;
            if (startIndex < 0) {
                return [];
            }
            var result = [], i, top, height;
            for (i = startIndex; i <= endIndex; i++) {
                top = this.getVerticalOffsetForWhitespaceIndex(i, deviceLineHeight);
                height = this.whitespaces.getHeightForWhitespaceIndex(i);
                if (top >= verticalOffset2) {
                    break;
                }
                result.push({
                    id: this.whitespaces.getIdForWhitespaceIndex(i),
                    afterLineNumber: this.whitespaces.getAfterLineNumberForWhitespaceIndex(i),
                    verticalOffset: top,
                    height: height
                });
            }
            return result;
        };
        VerticalObjects.prototype.getWhitespaces = function (deviceLineHeight) {
            return this.whitespaces.getWhitespaces(deviceLineHeight);
        };
        return VerticalObjects;
    }());
    exports.VerticalObjects = VerticalObjects;
});

define("vs/editor/common/viewLayout/linesLayout", ["require", "exports", 'vs/editor/common/viewLayout/verticalObjects'], function (require, exports, verticalObjects_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Layouting of objects that take vertical space (by having a height) and push down other objects.
     *
     * These objects are basically either text (lines) or spaces between those lines (whitespaces).
     * This provides commodity operations for working with lines that contain whitespace that pushes lines lower (vertically).
     * This is a thin wrapper around VerticalObjects.VerticalObjects, with knowledge of the editor.
     */
    var LinesLayout = (function () {
        function LinesLayout(configuration, model) {
            this.configuration = configuration;
            this.model = model;
            this.verticalObjects = new verticalObjects_1.VerticalObjects();
            this.verticalObjects.replaceLines(model.getLineCount());
        }
        /**
         * Insert a new whitespace of a certain height after a line number.
         * The whitespace has a "sticky" characteristic.
         * Irrespective of edits above or below `afterLineNumber`, the whitespace will follow the initial line.
         *
         * @param afterLineNumber The conceptual position of this whitespace. The whitespace will follow this line as best as possible even when deleting/inserting lines above/below.
         * @param heightInPx The height of the whitespace, in pixels.
         * @return An id that can be used later to mutate or delete the whitespace
         */
        LinesLayout.prototype.insertWhitespace = function (afterLineNumber, ordinal, height) {
            return this.verticalObjects.insertWhitespace(afterLineNumber, ordinal, height);
        };
        LinesLayout.prototype.changeWhitespace = function (id, newAfterLineNumber, newHeight) {
            return this.verticalObjects.changeWhitespace(id, newAfterLineNumber, newHeight);
        };
        /**
         * Remove an existing whitespace.
         *
         * @param id The whitespace to remove
         * @return Returns true if the whitespace is found and it is removed.
         */
        LinesLayout.prototype.removeWhitespace = function (id) {
            return this.verticalObjects.removeWhitespace(id);
        };
        /**
         * Event handler, call when the model associated to this view has been flushed.
         */
        LinesLayout.prototype.onModelFlushed = function () {
            this.verticalObjects.replaceLines(this.model.getLineCount());
        };
        /**
         * Event handler, call when the model has had lines deleted.
         */
        LinesLayout.prototype.onModelLinesDeleted = function (e) {
            this.verticalObjects.onModelLinesDeleted(e.fromLineNumber, e.toLineNumber);
        };
        /**
         * Event handler, call when the model has had lines inserted.
         */
        LinesLayout.prototype.onModelLinesInserted = function (e) {
            this.verticalObjects.onModelLinesInserted(e.fromLineNumber, e.toLineNumber);
        };
        /**
         * Get the vertical offset (the sum of heights for all objects above) a certain line number.
         *
         * @param lineNumber The line number
         * @return The sum of heights for all objects above `lineNumber`.
         */
        LinesLayout.prototype.getVerticalOffsetForLineNumber = function (lineNumber) {
            return this.verticalObjects.getVerticalOffsetForLineNumber(lineNumber, this.configuration.editor.lineHeight);
        };
        LinesLayout.prototype.getLinesTotalHeight = function () {
            return this.verticalObjects.getTotalHeight(this.configuration.editor.lineHeight);
        };
        /**
         * Get the sum of heights for all objects and compute basically the `scrollHeight` for the editor content.
         *
         * Take into account the `scrollBeyondLastLine` and `reserveHorizontalScrollbarHeight` and produce a scrollHeight that is at least as large as `viewport`.height.
         *
         * @param viewport The viewport.
         * @param reserveHorizontalScrollbarHeight The height of the horizontal scrollbar.
         * @return Basically, the `scrollHeight` for the editor content.
         */
        LinesLayout.prototype.getTotalHeight = function (viewport, reserveHorizontalScrollbarHeight) {
            var totalLinesHeight = this.getLinesTotalHeight();
            //		if (this.context.configuration.editor.autoSize) {
            //			return linesHeight;
            //		}
            if (this.configuration.editor.scrollBeyondLastLine) {
                totalLinesHeight += viewport.height - this.configuration.editor.lineHeight;
            }
            else {
                totalLinesHeight += reserveHorizontalScrollbarHeight;
            }
            return Math.max(viewport.height, totalLinesHeight);
        };
        LinesLayout.prototype.isAfterLines = function (verticalOffset) {
            return this.verticalObjects.isAfterLines(verticalOffset, this.configuration.editor.lineHeight);
        };
        /**
         * Find the first line number that is at or after vertical offset `verticalOffset`.
         * i.e. if getVerticalOffsetForLine(line) is x and getVerticalOffsetForLine(line + 1) is y, then
         * getLineNumberAtOrAfterVerticalOffset(i) = line, x <= i < y.
         *
         * @param verticalOffset The vertical offset to search at.
         * @return The line number at or after vertical offset `verticalOffset`.
         */
        LinesLayout.prototype.getLineNumberAtOrAfterVerticalOffset = function (verticalOffset) {
            return this.verticalObjects.getLineNumberAtOrAfterVerticalOffset(verticalOffset, this.configuration.editor.lineHeight);
        };
        /**
         * Get the height, in pixels, for line `lineNumber`.
         *
         * @param lineNumber The line number
         * @return The height, in pixels, for line `lineNumber`.
         */
        LinesLayout.prototype.getHeightForLineNumber = function (lineNumber) {
            return this.configuration.editor.lineHeight;
        };
        /**
         * Get a list of whitespaces that are positioned inside `viewport`.
         *
         * @param viewport The viewport.
         * @return An array with all the whitespaces in the viewport. If no whitespace is in viewport, the array is empty.
         */
        LinesLayout.prototype.getWhitespaceViewportData = function (visibleBox) {
            return this.verticalObjects.getWhitespaceViewportData(visibleBox.top, visibleBox.top + visibleBox.height, this.configuration.editor.lineHeight);
        };
        LinesLayout.prototype.getWhitespaces = function () {
            return this.verticalObjects.getWhitespaces(this.configuration.editor.lineHeight);
        };
        /**
         * Get exactly the whitespace that is layouted at `verticalOffset`.
         *
         * @param verticalOffset The vertical offset.
         * @return Precisely the whitespace that is layouted at `verticaloffset` or null.
         */
        LinesLayout.prototype.getWhitespaceAtVerticalOffset = function (verticalOffset) {
            return this.verticalObjects.getWhitespaceAtVerticalOffset(verticalOffset, this.configuration.editor.lineHeight);
        };
        /**
         * Get all the lines and their relative vertical offsets that are positioned inside `viewport`.
         *
         * @param viewport The viewport.
         * @return A structure describing the lines positioned between `verticalOffset1` and `verticalOffset2`.
         */
        LinesLayout.prototype.getLinesViewportData = function (visibleBox) {
            var viewportData = this.verticalObjects.getLinesViewportData(visibleBox.top, visibleBox.top + visibleBox.height, this.configuration.editor.lineHeight);
            var decorationsResolver = this.model.getDecorationsResolver(viewportData.startLineNumber, viewportData.endLineNumber);
            viewportData.getDecorationsInViewport = function () { return decorationsResolver.getDecorations(); };
            viewportData.getInlineDecorationsForLineInViewport = function (lineNumber) { return decorationsResolver.getInlineDecorations(lineNumber); };
            return viewportData;
        };
        /**
         * Get the line that appears visually in the center of `viewport`.
         *
         * @param viewport The viewport.
         * @return The line number that is closest to the center of `viewport`.
         */
        LinesLayout.prototype.getCenteredLineInViewport = function (visibleBox) {
            return this.verticalObjects.getCenteredLineInViewport(visibleBox.top, visibleBox.top + visibleBox.height, this.configuration.editor.lineHeight);
        };
        /**
         * Returns the accumulated height of whitespaces before the given line number.
         *
         * @param lineNumber The line number
         */
        LinesLayout.prototype.getWhitespaceAccumulatedHeightBeforeLineNumber = function (lineNumber) {
            return this.verticalObjects.getWhitespaceAccumulatedHeightBeforeLineNumber(lineNumber);
        };
        /**
         * Returns if there is any whitespace in the document.
         */
        LinesLayout.prototype.hasWhitespace = function () {
            return this.verticalObjects.hasWhitespace();
        };
        return LinesLayout;
    }());
    exports.LinesLayout = LinesLayout;
});

define("vs/editor/common/viewModel/prefixSumComputer", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var PrefixSumComputer = (function () {
        function PrefixSumComputer(values) {
            this.values = values;
            this.prefixSum = [];
            for (var i = 0, len = this.values.length; i < len; i++) {
                this.prefixSum[i] = 0;
            }
            this.prefixSumValidIndex = -1;
        }
        PrefixSumComputer.prototype.getCount = function () {
            return this.values.length;
        };
        PrefixSumComputer.prototype.insertValue = function (insertIndex, value) {
            this.values.splice(insertIndex, 0, value);
            this.prefixSum.splice(insertIndex, 0, 0);
            if (insertIndex - 1 < this.prefixSumValidIndex) {
                this.prefixSumValidIndex = insertIndex - 1;
            }
        };
        PrefixSumComputer.prototype.insertValues = function (insertIndex, values) {
            if (values.length === 0) {
                return;
            }
            this.values = this.values.slice(0, insertIndex).concat(values).concat(this.values.slice(insertIndex));
            this.prefixSum = this.prefixSum.slice(0, insertIndex).concat(PrefixSumComputer._zeroArray(values.length)).concat(this.prefixSum.slice(insertIndex));
            if (insertIndex - 1 < this.prefixSumValidIndex) {
                this.prefixSumValidIndex = insertIndex - 1;
            }
        };
        PrefixSumComputer._zeroArray = function (count) {
            var r = new Array(count);
            for (var i = 0; i < count; i++) {
                r[i] = 0;
            }
            return r;
        };
        PrefixSumComputer.prototype.changeValue = function (index, value) {
            if (this.values[index] === value) {
                return;
            }
            this.values[index] = value;
            if (index - 1 < this.prefixSumValidIndex) {
                this.prefixSumValidIndex = index - 1;
            }
        };
        PrefixSumComputer.prototype.removeValues = function (startIndex, cnt) {
            this.values.splice(startIndex, cnt);
            this.prefixSum.splice(startIndex, cnt);
            if (startIndex - 1 < this.prefixSumValidIndex) {
                this.prefixSumValidIndex = startIndex - 1;
            }
        };
        PrefixSumComputer.prototype.getTotalValue = function () {
            if (this.values.length === 0) {
                return 0;
            }
            return this.getAccumulatedValue(this.values.length - 1);
        };
        PrefixSumComputer.prototype.getAccumulatedValue = function (index) {
            if (index < 0) {
                return 0;
            }
            if (index <= this.prefixSumValidIndex) {
                return this.prefixSum[index];
            }
            var startIndex = this.prefixSumValidIndex + 1;
            if (startIndex === 0) {
                this.prefixSum[0] = this.values[0];
                startIndex++;
            }
            if (index >= this.values.length) {
                index = this.values.length - 1;
            }
            for (var i = startIndex; i <= index; i++) {
                this.prefixSum[i] = this.prefixSum[i - 1] + this.values[i];
            }
            this.prefixSumValidIndex = Math.max(this.prefixSumValidIndex, index);
            return this.prefixSum[index];
        };
        PrefixSumComputer.prototype.getIndexOf = function (accumulatedValue, result) {
            var low = 0, high = this.values.length - 1, mid, midStart, midStop;
            while (low <= high) {
                mid = low + ((high - low) / 2) | 0;
                midStop = this.getAccumulatedValue(mid);
                midStart = midStop - this.values[mid];
                if (accumulatedValue < midStart) {
                    high = mid - 1;
                }
                else if (accumulatedValue >= midStop) {
                    low = mid + 1;
                }
                else {
                    break;
                }
            }
            result.index = mid;
            result.remainder = accumulatedValue - midStart;
        };
        return PrefixSumComputer;
    }());
    exports.PrefixSumComputer = PrefixSumComputer;
});

define("vs/editor/common/model/mirrorModel2", ["require", "exports", 'vs/editor/common/viewModel/prefixSumComputer'], function (require, exports, prefixSumComputer_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var MirrorModel2 = (function () {
        function MirrorModel2(uri, lines, eol, versionId) {
            this._uri = uri;
            this._lines = lines;
            this._eol = eol;
            this._versionId = versionId;
        }
        MirrorModel2.prototype.dispose = function () {
            this._lines.length = 0;
        };
        Object.defineProperty(MirrorModel2.prototype, "version", {
            get: function () {
                return this._versionId;
            },
            enumerable: true,
            configurable: true
        });
        MirrorModel2.prototype.getText = function () {
            return this._lines.join(this._eol);
        };
        MirrorModel2.prototype.onEvents = function (events) {
            var newEOL = null;
            for (var i = 0, len = events.length; i < len; i++) {
                var e = events[i];
                if (e.eol) {
                    newEOL = e.eol;
                }
            }
            if (newEOL && newEOL !== this._eol) {
                this._eol = newEOL;
                this._lineStarts = null;
            }
            // Update my lines
            var lastVersionId = -1;
            for (var i = 0, len = events.length; i < len; i++) {
                var e = events[i];
                this._acceptDeleteRange(e.range);
                this._acceptInsertText({
                    lineNumber: e.range.startLineNumber,
                    column: e.range.startColumn
                }, e.text);
                lastVersionId = Math.max(lastVersionId, e.versionId);
            }
            if (lastVersionId !== -1) {
                this._versionId = lastVersionId;
            }
        };
        MirrorModel2.prototype._ensureLineStarts = function () {
            if (!this._lineStarts) {
                var lineStartValues = [];
                var eolLength = this._eol.length;
                for (var i = 0, len = this._lines.length; i < len; i++) {
                    lineStartValues.push(this._lines[i].length + eolLength);
                }
                this._lineStarts = new prefixSumComputer_1.PrefixSumComputer(lineStartValues);
            }
        };
        /**
         * All changes to a line's text go through this method
         */
        MirrorModel2.prototype._setLineText = function (lineIndex, newValue) {
            this._lines[lineIndex] = newValue;
            if (this._lineStarts) {
                // update prefix sum
                this._lineStarts.changeValue(lineIndex, this._lines[lineIndex].length + this._eol.length);
            }
        };
        MirrorModel2.prototype._acceptDeleteRange = function (range) {
            if (range.startLineNumber === range.endLineNumber) {
                if (range.startColumn === range.endColumn) {
                    // Nothing to delete
                    return;
                }
                // Delete text on the affected line
                this._setLineText(range.startLineNumber - 1, this._lines[range.startLineNumber - 1].substring(0, range.startColumn - 1)
                    + this._lines[range.startLineNumber - 1].substring(range.endColumn - 1));
                return;
            }
            // Take remaining text on last line and append it to remaining text on first line
            this._setLineText(range.startLineNumber - 1, this._lines[range.startLineNumber - 1].substring(0, range.startColumn - 1)
                + this._lines[range.endLineNumber - 1].substring(range.endColumn - 1));
            // Delete middle lines
            this._lines.splice(range.startLineNumber, range.endLineNumber - range.startLineNumber);
            if (this._lineStarts) {
                // update prefix sum
                this._lineStarts.removeValues(range.startLineNumber, range.endLineNumber - range.startLineNumber);
            }
        };
        MirrorModel2.prototype._acceptInsertText = function (position, insertText) {
            if (insertText.length === 0) {
                // Nothing to insert
                return;
            }
            var insertLines = insertText.split(/\r\n|\r|\n/);
            if (insertLines.length === 1) {
                // Inserting text on one line
                this._setLineText(position.lineNumber - 1, this._lines[position.lineNumber - 1].substring(0, position.column - 1)
                    + insertLines[0]
                    + this._lines[position.lineNumber - 1].substring(position.column - 1));
                return;
            }
            // Append overflowing text from first line to the end of text to insert
            insertLines[insertLines.length - 1] += this._lines[position.lineNumber - 1].substring(position.column - 1);
            // Delete overflowing text from first line and insert text on first line
            this._setLineText(position.lineNumber - 1, this._lines[position.lineNumber - 1].substring(0, position.column - 1)
                + insertLines[0]);
            // Insert new lines & store lengths
            var newLengths = new Array(insertLines.length - 1);
            for (var i = 1; i < insertLines.length; i++) {
                this._lines.splice(position.lineNumber + i - 1, 0, insertLines[i]);
                newLengths[i - 1] = insertLines[i].length + this._eol.length;
            }
            if (this._lineStarts) {
                // update prefix sum
                this._lineStarts.insertValues(position.lineNumber, newLengths);
            }
        };
        return MirrorModel2;
    }());
    exports.MirrorModel2 = MirrorModel2;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/folding/common/indentFoldStrategy", ["require", "exports"], function (require, exports) {
    'use strict';
    function computeRanges(model, tabSize, minimumRangeSize) {
        if (minimumRangeSize === void 0) { minimumRangeSize = 1; }
        var result = [];
        var previousRegions = [];
        previousRegions.push({ indent: -1, line: model.getLineCount() + 1 }); // sentinel, to make sure there's at least one entry
        for (var line = model.getLineCount(); line > 0; line--) {
            var indent = computeIndentLevel(model.getLineContent(line), tabSize);
            if (indent === -1) {
                continue; // only whitespace
            }
            var previous = previousRegions[previousRegions.length - 1];
            if (previous.indent > indent) {
                // discard all regions with larger indent
                do {
                    previousRegions.pop();
                    previous = previousRegions[previousRegions.length - 1];
                } while (previous.indent > indent);
                // new folding range
                var endLineNumber = previous.line - 1;
                if (endLineNumber - line >= minimumRangeSize) {
                    result.push({ startLineNumber: line, endLineNumber: endLineNumber, indent: indent });
                }
            }
            if (previous.indent === indent) {
                previous.line = line;
            }
            else {
                // new region with a bigger indent
                previousRegions.push({ indent: indent, line: line });
            }
        }
        return result.reverse();
    }
    exports.computeRanges = computeRanges;
    function computeIndentLevel(line, tabSize) {
        var i = 0;
        var indent = 0;
        while (i < line.length) {
            var ch = line.charAt(i);
            if (ch === ' ') {
                indent++;
            }
            else if (ch === '\t') {
                indent = indent - indent % tabSize + tabSize;
            }
            else {
                break;
            }
            i++;
        }
        if (i === line.length) {
            return -1; // line only consists of whitespace
        }
        return indent;
    }
    exports.computeIndentLevel = computeIndentLevel;
    /**
     * Limits the number of folding ranges by removing ranges with larger indent levels
     */
    function limitByIndent(ranges, maxEntries) {
        if (ranges.length <= maxEntries) {
            return ranges;
        }
        var indentOccurrences = [];
        ranges.forEach(function (r) {
            if (r.indent < 1000) {
                indentOccurrences[r.indent] = (indentOccurrences[r.indent] || 0) + 1;
            }
        });
        var maxIndent = indentOccurrences.length;
        for (var i = 0; i < indentOccurrences.length; i++) {
            if (indentOccurrences[i]) {
                maxEntries -= indentOccurrences[i];
                if (maxEntries < 0) {
                    maxIndent = i;
                    break;
                }
            }
        }
        return ranges.filter(function (r) { return r.indent < maxIndent; });
    }
    exports.limitByIndent = limitByIndent;
});

define("vs/editor/contrib/format/common/formatCommand", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/core/range'], function (require, exports, strings, range_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EditOperationsCommand = (function () {
        function EditOperationsCommand(edits, initialSelection) {
            this._edits = edits;
            this._initialSelection = initialSelection;
        }
        EditOperationsCommand.prototype.getEditOperations = function (model, builder) {
            this._edits
                .map(function (edit) { return EditOperationsCommand.trimEdit(edit, model); })
                .filter(function (edit) { return edit !== null; }) // produced above in case the edit.text is identical to the existing text
                .forEach(function (edit) { return builder.addEditOperation(range_1.Range.lift(edit.range), edit.text); });
            var selectionIsSet = false;
            if (Array.isArray(this._edits) && this._edits.length === 1 && this._initialSelection.isEmpty()) {
                if (this._edits[0].range.startColumn === this._initialSelection.endColumn &&
                    this._edits[0].range.startLineNumber === this._initialSelection.endLineNumber) {
                    selectionIsSet = true;
                    this._selectionId = builder.trackSelection(this._initialSelection, true);
                }
                else if (this._edits[0].range.endColumn === this._initialSelection.startColumn &&
                    this._edits[0].range.endLineNumber === this._initialSelection.startLineNumber) {
                    selectionIsSet = true;
                    this._selectionId = builder.trackSelection(this._initialSelection, false);
                }
            }
            if (!selectionIsSet) {
                this._selectionId = builder.trackSelection(this._initialSelection);
            }
        };
        EditOperationsCommand.prototype.computeCursorState = function (model, helper) {
            return helper.getTrackedSelection(this._selectionId);
        };
        EditOperationsCommand.fixLineTerminators = function (edit, model) {
            edit.text = edit.text.replace(/\r\n|\r|\n/g, model.getEOL());
        };
        /**
         * This is used to minimize the edits by removing changes that appear on the edges of the range which are identical
         * to the current text.
         *
         * The reason this was introduced is to allow better selection tracking of the current cursor and solve
         * bug #15108. There the cursor was jumping since the tracked selection was in the middle of the range edit
         * and was lost.
         */
        EditOperationsCommand.trimEdit = function (edit, model) {
            this.fixLineTerminators(edit, model);
            return this._trimEdit(model.validateRange(edit.range), edit.text, edit.forceMoveMarkers, model);
        };
        EditOperationsCommand._trimEdit = function (editRange, editText, editForceMoveMarkers, model) {
            var currentText = model.getValueInRange(editRange);
            // Find the equal characters in the front
            var commonPrefixLength = strings.commonPrefixLength(editText, currentText);
            // If the two strings are identical, return no edit (no-op)
            if (commonPrefixLength === currentText.length && commonPrefixLength === editText.length) {
                return null;
            }
            if (commonPrefixLength > 0) {
                // Apply front trimming
                var newStartPosition = model.modifyPosition(editRange.getStartPosition(), commonPrefixLength);
                editRange = new range_1.Range(newStartPosition.lineNumber, newStartPosition.column, editRange.endLineNumber, editRange.endColumn);
                editText = editText.substring(commonPrefixLength);
                currentText = currentText.substr(commonPrefixLength);
            }
            // Find the equal characters in the rear
            var commonSuffixLength = strings.commonSuffixLength(editText, currentText);
            if (commonSuffixLength > 0) {
                // Apply rear trimming
                var newEndPosition = model.modifyPosition(editRange.getEndPosition(), -commonSuffixLength);
                editRange = new range_1.Range(editRange.startLineNumber, editRange.startColumn, newEndPosition.lineNumber, newEndPosition.column);
                editText = editText.substring(0, editText.length - commonSuffixLength);
                currentText = currentText.substring(0, currentText.length - commonSuffixLength);
            }
            return {
                text: editText,
                range: editRange,
                forceMoveMarkers: editForceMoveMarkers
            };
        };
        return EditOperationsCommand;
    }());
    exports.EditOperationsCommand = EditOperationsCommand;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/indentation/common/indentationCommands", ["require", "exports", 'vs/editor/common/core/range'], function (require, exports, range_1) {
    "use strict";
    function getIndentationEditOperations(model, builder, tabSize, tabsToSpaces) {
        if (model.getLineCount() === 1 && model.getLineMaxColumn(1) === 1) {
            // Model is empty
            return;
        }
        var spaces = '';
        for (var i = 0; i < tabSize; i++) {
            spaces += ' ';
        }
        var content = model.getLinesContent();
        for (var i = 0; i < content.length; i++) {
            var lastIndentationColumn = model.getLineFirstNonWhitespaceColumn(i + 1);
            if (lastIndentationColumn === 0) {
                lastIndentationColumn = model.getLineMaxColumn(i + 1);
            }
            var text = (tabsToSpaces ? content[i].substr(0, lastIndentationColumn).replace(/\t/ig, spaces) :
                content[i].substr(0, lastIndentationColumn).replace(new RegExp(spaces, 'gi'), '\t')) +
                content[i].substr(lastIndentationColumn);
            builder.addEditOperation(new range_1.Range(i + 1, 1, i + 1, model.getLineMaxColumn(i + 1)), text);
        }
    }
    var IndentationToSpacesCommand = (function () {
        function IndentationToSpacesCommand(selection, tabSize) {
            this.selection = selection;
            this.tabSize = tabSize;
        }
        IndentationToSpacesCommand.prototype.getEditOperations = function (model, builder) {
            this.selectionId = builder.trackSelection(this.selection);
            getIndentationEditOperations(model, builder, this.tabSize, true);
        };
        IndentationToSpacesCommand.prototype.computeCursorState = function (model, helper) {
            return helper.getTrackedSelection(this.selectionId);
        };
        return IndentationToSpacesCommand;
    }());
    exports.IndentationToSpacesCommand = IndentationToSpacesCommand;
    var IndentationToTabsCommand = (function () {
        function IndentationToTabsCommand(selection, tabSize) {
            this.selection = selection;
            this.tabSize = tabSize;
        }
        IndentationToTabsCommand.prototype.getEditOperations = function (model, builder) {
            this.selectionId = builder.trackSelection(this.selection);
            getIndentationEditOperations(model, builder, this.tabSize, false);
        };
        IndentationToTabsCommand.prototype.computeCursorState = function (model, helper) {
            return helper.getTrackedSelection(this.selectionId);
        };
        return IndentationToTabsCommand;
    }());
    exports.IndentationToTabsCommand = IndentationToTabsCommand;
});

define("vs/editor/contrib/linesOperations/common/sortLinesCommand", ["require", "exports", 'vs/editor/common/core/editOperation', 'vs/editor/common/core/range'], function (require, exports, editOperation_1, range_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var SortLinesCommand = (function () {
        function SortLinesCommand(selection, descending) {
            this.selection = selection;
            this.descending = descending;
        }
        SortLinesCommand.prototype.getEditOperations = function (model, builder) {
            var op = sortLines(model, this.selection, this.descending);
            if (op) {
                builder.addEditOperation(op.range, op.text);
            }
            this.selectionId = builder.trackSelection(this.selection);
        };
        SortLinesCommand.prototype.computeCursorState = function (model, helper) {
            return helper.getTrackedSelection(this.selectionId);
        };
        return SortLinesCommand;
    }());
    exports.SortLinesCommand = SortLinesCommand;
    /**
     * Generate commands for sorting lines on a model.
     */
    function sortLines(model, selection, descending) {
        var startLineNumber = selection.startLineNumber;
        var endLineNumber = selection.endLineNumber;
        if (selection.endColumn === 1) {
            endLineNumber--;
        }
        // Nothing to sort if user didn't select anything.
        if (startLineNumber >= endLineNumber) {
            return null;
        }
        var linesToSort = [];
        // Get the contents of the selection to be sorted.
        for (var lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
            linesToSort.push(model.getLineContent(lineNumber));
        }
        var sorted = linesToSort.sort(function (a, b) {
            return a.toLowerCase().localeCompare(b.toLowerCase());
        });
        // If descending, reverse the order.
        if (descending === true) {
            sorted = sorted.reverse();
        }
        return editOperation_1.EditOperation.replace(new range_1.Range(startLineNumber, 1, endLineNumber, model.getLineMaxColumn(endLineNumber)), sorted.join('\n'));
    }
    exports.sortLines = sortLines;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/smartSelect/common/tokenTree", ["require", "exports", 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/modes/supports', 'vs/editor/common/modes/supports/richEditBrackets'], function (require, exports, position_1, range_1, supports_1, richEditBrackets_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (TokenTreeBracket) {
        TokenTreeBracket[TokenTreeBracket["None"] = 0] = "None";
        TokenTreeBracket[TokenTreeBracket["Open"] = 1] = "Open";
        TokenTreeBracket[TokenTreeBracket["Close"] = -1] = "Close";
    })(exports.TokenTreeBracket || (exports.TokenTreeBracket = {}));
    var TokenTreeBracket = exports.TokenTreeBracket;
    var Node = (function () {
        function Node() {
        }
        Object.defineProperty(Node.prototype, "range", {
            get: function () {
                return {
                    startLineNumber: this.start.lineNumber,
                    startColumn: this.start.column,
                    endLineNumber: this.end.lineNumber,
                    endColumn: this.end.column
                };
            },
            enumerable: true,
            configurable: true
        });
        return Node;
    }());
    exports.Node = Node;
    var NodeList = (function (_super) {
        __extends(NodeList, _super);
        function NodeList() {
            _super.apply(this, arguments);
        }
        Object.defineProperty(NodeList.prototype, "start", {
            get: function () {
                return this.hasChildren
                    ? this.children[0].start
                    : this.parent.start;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NodeList.prototype, "end", {
            get: function () {
                return this.hasChildren
                    ? this.children[this.children.length - 1].end
                    : this.parent.end;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NodeList.prototype, "hasChildren", {
            get: function () {
                return this.children && this.children.length > 0;
            },
            enumerable: true,
            configurable: true
        });
        NodeList.prototype.append = function (node) {
            if (!node) {
                return false;
            }
            node.parent = this;
            if (!this.children) {
                this.children = [];
            }
            if (node instanceof NodeList) {
                if (node.children) {
                    this.children.push.apply(this.children, node.children);
                }
            }
            else {
                this.children.push(node);
            }
            return true;
        };
        return NodeList;
    }(Node));
    exports.NodeList = NodeList;
    var Block = (function (_super) {
        __extends(Block, _super);
        function Block() {
            _super.call(this);
            this.elements = new NodeList();
            this.elements.parent = this;
        }
        Object.defineProperty(Block.prototype, "start", {
            get: function () {
                return this.open.start;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Block.prototype, "end", {
            get: function () {
                return this.close.end;
            },
            enumerable: true,
            configurable: true
        });
        return Block;
    }(Node));
    exports.Block = Block;
    function newNode(token) {
        var node = new Node();
        node.start = position_1.Position.startPosition(token.range);
        node.end = position_1.Position.endPosition(token.range);
        return node;
    }
    var TokenScanner = (function () {
        function TokenScanner(model) {
            this._model = model;
            this._versionId = model.getVersionId();
            this._currentLineNumber = 1;
        }
        TokenScanner.prototype.next = function () {
            if (this._versionId !== this._model.getVersionId()) {
                // model has been modified
                return null;
            }
            if (this._currentLineNumber >= this._model.getLineCount() + 1) {
                // all line visisted
                return null;
            }
            if (!this._currentLineTokens) {
                // no tokens for this line
                this._currentLineTokens = this._model.getLineTokens(this._currentLineNumber);
                this._currentLineText = this._model.getLineContent(this._currentLineNumber);
                this._currentLineModeTransitions = this._model._getLineModeTransitions(this._currentLineNumber);
                this._currentTokenIndex = 0;
                this._currentTokenStart = 0;
                this._currentModeIndex = -1;
                this._nextModeStart = 0;
            }
            if (this._currentTokenIndex >= this._currentLineTokens.getTokenCount()) {
                // last token of line visited
                this._currentLineNumber += 1;
                this._currentLineTokens = null;
                return this.next();
            }
            if (this._currentTokenStart >= this._nextModeStart) {
                this._currentModeIndex++;
                this._nextModeStart = (this._currentModeIndex + 1 < this._currentLineModeTransitions.length ? this._currentLineModeTransitions[this._currentModeIndex + 1].startIndex : this._currentLineText.length + 1);
                var mode = (this._currentModeIndex < this._currentLineModeTransitions.length ? this._currentLineModeTransitions[this._currentModeIndex] : null);
                this._currentModeBrackets = (mode && mode.mode.richEditSupport ? mode.mode.richEditSupport.brackets : null);
            }
            var tokenType = this._currentLineTokens.getTokenType(this._currentTokenIndex);
            var tokenEndIndex = this._currentLineTokens.getTokenEndIndex(this._currentTokenIndex, this._currentLineText.length);
            var nextBracket = null;
            if (this._currentModeBrackets && !supports_1.ignoreBracketsInToken(tokenType)) {
                nextBracket = richEditBrackets_1.BracketsUtils.findNextBracketInToken(this._currentModeBrackets.forwardRegex, this._currentLineNumber, this._currentLineText, this._currentTokenStart, tokenEndIndex);
            }
            if (nextBracket && this._currentTokenStart < nextBracket.startColumn - 1) {
                // found a bracket, but it is not at the beginning of the token
                tokenEndIndex = nextBracket.startColumn - 1;
                nextBracket = null;
            }
            var bracketData = null;
            var bracketIsOpen = false;
            if (nextBracket) {
                var bracketText = this._currentLineText.substring(nextBracket.startColumn - 1, nextBracket.endColumn - 1);
                bracketData = this._currentModeBrackets.textIsBracket[bracketText];
                bracketIsOpen = this._currentModeBrackets.textIsOpenBracket[bracketText];
            }
            if (!bracketData) {
                var token_1 = {
                    type: tokenType,
                    bracket: TokenTreeBracket.None,
                    range: {
                        startLineNumber: this._currentLineNumber,
                        startColumn: 1 + this._currentTokenStart,
                        endLineNumber: this._currentLineNumber,
                        endColumn: 1 + tokenEndIndex
                    }
                };
                // console.log('TOKEN: <<' + this._currentLineText.substring(this._currentTokenStart, tokenEndIndex) + '>>');
                this._currentTokenIndex += 1;
                this._currentTokenStart = (this._currentTokenIndex < this._currentLineTokens.getTokenCount() ? this._currentLineTokens.getTokenStartIndex(this._currentTokenIndex) : 0);
                return token_1;
            }
            var type = bracketData.modeId + ";" + bracketData.open + ";" + bracketData.close;
            var token = {
                type: type,
                bracket: bracketIsOpen ? TokenTreeBracket.Open : TokenTreeBracket.Close,
                range: {
                    startLineNumber: this._currentLineNumber,
                    startColumn: 1 + this._currentTokenStart,
                    endLineNumber: this._currentLineNumber,
                    endColumn: nextBracket.endColumn
                }
            };
            // console.log('BRACKET: <<' + this._currentLineText.substring(this._currentTokenStart, nextBracket.endColumn - 1) + '>>');
            if (nextBracket.endColumn - 1 < tokenEndIndex) {
                // found a bracket, but it is not at the end of the token
                this._currentTokenStart = nextBracket.endColumn - 1;
            }
            else {
                this._currentTokenIndex += 1;
                this._currentTokenStart = (this._currentTokenIndex < this._currentLineTokens.getTokenCount() ? this._currentLineTokens.getTokenStartIndex(this._currentTokenIndex) : 0);
            }
            return token;
        };
        return TokenScanner;
    }());
    var TokenTreeBuilder = (function () {
        function TokenTreeBuilder(model) {
            this._stack = [];
            this._scanner = new TokenScanner(model);
        }
        TokenTreeBuilder.prototype.build = function () {
            var node = new NodeList();
            while (node.append(this._line() || this._any())) {
            }
            return node;
        };
        TokenTreeBuilder.prototype._accept = function (condt) {
            var token = this._stack.pop() || this._scanner.next();
            if (!token) {
                return false;
            }
            var accepted = condt(token);
            if (!accepted) {
                this._stack.push(token);
                this._currentToken = null;
            }
            else {
                this._currentToken = token;
            }
            return accepted;
        };
        TokenTreeBuilder.prototype._peek = function (condt) {
            var ret = false;
            this._accept(function (info) {
                ret = condt(info);
                return false;
            });
            return ret;
        };
        TokenTreeBuilder.prototype._line = function () {
            var node = new NodeList(), lineNumber;
            // capture current linenumber
            this._peek(function (info) {
                lineNumber = info.range.startLineNumber;
                return false;
            });
            while (this._peek(function (info) { return info.range.startLineNumber === lineNumber; })
                && node.append(this._token() || this._block())) {
            }
            if (!node.children || node.children.length === 0) {
                return null;
            }
            else if (node.children.length === 1) {
                return node.children[0];
            }
            else {
                return node;
            }
        };
        TokenTreeBuilder.prototype._token = function () {
            if (!this._accept(function (token) { return token.bracket === TokenTreeBracket.None; })) {
                return null;
            }
            return newNode(this._currentToken);
        };
        TokenTreeBuilder.prototype._block = function () {
            var bracketType, accepted;
            accepted = this._accept(function (token) {
                bracketType = token.type;
                return token.bracket === TokenTreeBracket.Open;
            });
            if (!accepted) {
                return null;
            }
            var bracket = new Block();
            bracket.open = newNode(this._currentToken);
            while (bracket.elements.append(this._line())) {
            }
            if (!this._accept(function (token) { return token.bracket === TokenTreeBracket.Close && token.type === bracketType; })) {
                // missing closing bracket -> return just a node list
                var nodelist = new NodeList();
                nodelist.append(bracket.open);
                nodelist.append(bracket.elements);
                return nodelist;
            }
            bracket.close = newNode(this._currentToken);
            return bracket;
        };
        TokenTreeBuilder.prototype._any = function () {
            if (!this._accept(function (_) { return true; })) {
                return null;
            }
            return newNode(this._currentToken);
        };
        return TokenTreeBuilder;
    }());
    /**
     * Parses this grammar:
     *	grammer = { line }
     *	line = { block | "token" }
     *	block = "open_bracket" { line } "close_bracket"
     */
    function build(model) {
        var node = new TokenTreeBuilder(model).build();
        return node;
    }
    exports.build = build;
    function find(node, position) {
        if (!range_1.Range.containsPosition(node.range, position)) {
            return null;
        }
        var result;
        if (node instanceof NodeList) {
            for (var i = 0, len = node.children.length; i < len && !result; i++) {
                result = find(node.children[i], position);
            }
        }
        else if (node instanceof Block) {
            result = find(node.open, position) || find(node.elements, position) || find(node.close, position);
        }
        return result || node;
    }
    exports.find = find;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/snippet/browser/snippet", ["require", "exports", 'vs/css!./snippet'], function (require, exports) {
    'use strict';
});

define("vs/languages/css/common/level", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (Level) {
        Level[Level["Ignore"] = 1] = "Ignore";
        Level[Level["Warning"] = 2] = "Warning";
        Level[Level["Error"] = 4] = "Error";
    })(exports.Level || (exports.Level = {}));
    var Level = exports.Level;
    function toLevel(level) {
        switch (level) {
            case 'ignore': return Level.Ignore;
            case 'warning': return Level.Warning;
            case 'error': return Level.Error;
        }
        return null;
    }
    exports.toLevel = toLevel;
});

define("vs/nls!vs/base/browser/ui/actionbar/actionbar",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/browser/ui/actionbar/actionbar", data); });
define("vs/nls!vs/base/browser/ui/aria/aria",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/browser/ui/aria/aria", data); });
define("vs/nls!vs/base/browser/ui/findinput/findInput",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/browser/ui/findinput/findInput", data); });
define("vs/nls!vs/base/browser/ui/inputbox/inputBox",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/browser/ui/inputbox/inputBox", data); });
define("vs/nls!vs/base/browser/ui/messagelist/messageList",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/browser/ui/messagelist/messageList", data); });
define("vs/nls!vs/base/browser/ui/resourceviewer/resourceViewer",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/browser/ui/resourceviewer/resourceViewer", data); });
define("vs/nls!vs/base/browser/ui/toolbar/toolbar",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/browser/ui/toolbar/toolbar", data); });
define("vs/nls!vs/base/common/errors",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/common/errors", data); });
define("vs/base/common/errors", ["require", "exports", 'vs/nls!vs/base/common/errors', 'vs/base/common/objects', 'vs/base/common/platform', 'vs/base/common/types', 'vs/base/common/arrays', 'vs/base/common/strings'], function (require, exports, nls, objects, platform, types, arrays, strings) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // Avoid circular dependency on EventEmitter by implementing a subset of the interface.
    var ErrorHandler = (function () {
        function ErrorHandler() {
            this.listeners = [];
            this.unexpectedErrorHandler = function (e) {
                platform.setTimeout(function () {
                    if (e.stack) {
                        throw new Error(e.message + '\n\n' + e.stack);
                    }
                    throw e;
                }, 0);
            };
        }
        ErrorHandler.prototype.addListener = function (listener) {
            var _this = this;
            this.listeners.push(listener);
            return function () {
                _this._removeListener(listener);
            };
        };
        ErrorHandler.prototype.emit = function (e) {
            this.listeners.forEach(function (listener) {
                listener(e);
            });
        };
        ErrorHandler.prototype._removeListener = function (listener) {
            this.listeners.splice(this.listeners.indexOf(listener), 1);
        };
        ErrorHandler.prototype.setUnexpectedErrorHandler = function (newUnexpectedErrorHandler) {
            this.unexpectedErrorHandler = newUnexpectedErrorHandler;
        };
        ErrorHandler.prototype.getUnexpectedErrorHandler = function () {
            return this.unexpectedErrorHandler;
        };
        ErrorHandler.prototype.onUnexpectedError = function (e) {
            this.unexpectedErrorHandler(e);
            this.emit(e);
        };
        return ErrorHandler;
    }());
    exports.ErrorHandler = ErrorHandler;
    exports.errorHandler = new ErrorHandler();
    function setUnexpectedErrorHandler(newUnexpectedErrorHandler) {
        exports.errorHandler.setUnexpectedErrorHandler(newUnexpectedErrorHandler);
    }
    exports.setUnexpectedErrorHandler = setUnexpectedErrorHandler;
    function onUnexpectedError(e) {
        // ignore errors from cancelled promises
        if (!isPromiseCanceledError(e)) {
            exports.errorHandler.onUnexpectedError(e);
        }
    }
    exports.onUnexpectedError = onUnexpectedError;
    function onUnexpectedPromiseError(promise) {
        return promise.then(null, onUnexpectedError);
    }
    exports.onUnexpectedPromiseError = onUnexpectedPromiseError;
    function transformErrorForSerialization(error) {
        if (error instanceof Error) {
            var name_1 = error.name, message = error.message;
            var stack = error.stacktrace || error.stack;
            return {
                $isError: true,
                name: name_1,
                message: message,
                stack: stack
            };
        }
        // return as is
        return error;
    }
    exports.transformErrorForSerialization = transformErrorForSerialization;
    /**
     * The base class for all connection errors originating from XHR requests.
     */
    var ConnectionError = (function () {
        function ConnectionError(arg) {
            this.status = arg.status;
            this.statusText = arg.statusText;
            this.name = 'ConnectionError';
            try {
                this.responseText = arg.responseText;
            }
            catch (e) {
                this.responseText = '';
            }
            this.errorMessage = null;
            this.errorCode = null;
            this.errorObject = null;
            if (this.responseText) {
                try {
                    var errorObj = JSON.parse(this.responseText);
                    this.errorMessage = errorObj.message;
                    this.errorCode = errorObj.code;
                    this.errorObject = errorObj;
                }
                catch (error) {
                }
            }
        }
        Object.defineProperty(ConnectionError.prototype, "message", {
            get: function () {
                return this.connectionErrorToMessage(this, false);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ConnectionError.prototype, "verboseMessage", {
            get: function () {
                return this.connectionErrorToMessage(this, true);
            },
            enumerable: true,
            configurable: true
        });
        ConnectionError.prototype.connectionErrorDetailsToMessage = function (error, verbose) {
            var errorCode = error.errorCode;
            var errorMessage = error.errorMessage;
            if (errorCode !== null && errorMessage !== null) {
                return nls.localize(0, null, strings.rtrim(errorMessage, '.'), errorCode);






            }
            if (errorMessage !== null) {
                return errorMessage;
            }
            if (verbose && error.responseText !== null) {
                return error.responseText;
            }
            return null;
        };
        ConnectionError.prototype.connectionErrorToMessage = function (error, verbose) {
            var details = this.connectionErrorDetailsToMessage(error, verbose);
            // Status Code based Error
            if (error.status === 401) {
                if (details !== null) {
                    return nls.localize(1, null, details);





                }
                return nls.localize(2, null);
            }
            // Return error details if present
            if (details) {
                return details;
            }
            // Fallback to HTTP Status and Code
            if (error.status > 0 && error.statusText !== null) {
                if (verbose && error.responseText !== null && error.responseText.length > 0) {
                    return nls.localize(3, null, error.statusText, error.status, error.responseText);
                }
                return nls.localize(4, null, error.statusText, error.status);
            }
            // Finally its an Unknown Connection Error
            if (verbose && error.responseText !== null && error.responseText.length > 0) {
                return nls.localize(5, null, error.responseText);
            }
            return nls.localize(6, null);
        };
        return ConnectionError;
    }());
    exports.ConnectionError = ConnectionError;
    // Bug: Can not subclass a JS Type. Do it manually (as done in WinJS.Class.derive)
    objects.derive(Error, ConnectionError);
    function xhrToErrorMessage(xhr, verbose) {
        var ce = new ConnectionError(xhr);
        if (verbose) {
            return ce.verboseMessage;
        }
        else {
            return ce.message;
        }
    }
    function exceptionToErrorMessage(exception, verbose) {
        if (exception.message) {
            if (verbose && (exception.stack || exception.stacktrace)) {
                return nls.localize(7, null, detectSystemErrorMessage(exception), exception.stack || exception.stacktrace);
            }
            return detectSystemErrorMessage(exception);
        }
        return nls.localize(8, null);
    }
    function detectSystemErrorMessage(exception) {
        // See https://nodejs.org/api/errors.html#errors_class_system_error
        if (typeof exception.code === 'string' && typeof exception.errno === 'number' && typeof exception.syscall === 'string') {
            return nls.localize(9, null, exception.message);
        }
        return exception.message;
    }
    /**
     * Tries to generate a human readable error message out of the error. If the verbose parameter
     * is set to true, the error message will include stacktrace details if provided.
     * @returns A string containing the error message.
     */
    function toErrorMessage(error, verbose) {
        if (error === void 0) { error = null; }
        if (verbose === void 0) { verbose = false; }
        if (!error) {
            return nls.localize(10, null);
        }
        if (Array.isArray(error)) {
            var errors = arrays.coalesce(error);
            var msg = toErrorMessage(errors[0], verbose);
            if (errors.length > 1) {
                return nls.localize(11, null, msg, errors.length);
            }
            return msg;
        }
        if (types.isString(error)) {
            return error;
        }
        if (!types.isUndefinedOrNull(error.status)) {
            return xhrToErrorMessage(error, verbose);
        }
        if (error.detail) {
            var detail = error.detail;
            if (detail.error) {
                if (detail.error && !types.isUndefinedOrNull(detail.error.status)) {
                    return xhrToErrorMessage(detail.error, verbose);
                }
                if (types.isArray(detail.error)) {
                    for (var i = 0; i < detail.error.length; i++) {
                        if (detail.error[i] && !types.isUndefinedOrNull(detail.error[i].status)) {
                            return xhrToErrorMessage(detail.error[i], verbose);
                        }
                    }
                }
                else {
                    return exceptionToErrorMessage(detail.error, verbose);
                }
            }
            if (detail.exception) {
                if (!types.isUndefinedOrNull(detail.exception.status)) {
                    return xhrToErrorMessage(detail.exception, verbose);
                }
                return exceptionToErrorMessage(detail.exception, verbose);
            }
        }
        if (error.stack) {
            return exceptionToErrorMessage(error, verbose);
        }
        if (error.message) {
            return error.message;
        }
        return nls.localize(12, null);
    }
    exports.toErrorMessage = toErrorMessage;
    var canceledName = 'Canceled';
    /**
     * Checks if the given error is a promise in canceled state
     */
    function isPromiseCanceledError(error) {
        return error instanceof Error && error.name === canceledName && error.message === canceledName;
    }
    exports.isPromiseCanceledError = isPromiseCanceledError;
    /**
     * Returns an error that signals cancellation.
     */
    function canceled() {
        var error = new Error(canceledName);
        error.name = error.message;
        return error;
    }
    exports.canceled = canceled;
    /**
     * Returns an error that signals something is not implemented.
     */
    function notImplemented() {
        return new Error(nls.localize(13, null));
    }
    exports.notImplemented = notImplemented;
    function illegalArgument(name) {
        if (name) {
            return new Error(nls.localize(14, null, name));
        }
        else {
            return new Error(nls.localize(15, null));
        }
    }
    exports.illegalArgument = illegalArgument;
    function illegalState(name) {
        if (name) {
            return new Error(nls.localize(16, null, name));
        }
        else {
            return new Error(nls.localize(17, null));
        }
    }
    exports.illegalState = illegalState;
    function readonly() {
        return new Error('readonly property cannot be changed');
    }
    exports.readonly = readonly;
    function loaderError(err) {
        if (platform.isWeb) {
            return new Error(nls.localize(18, null));
        }
        return new Error(nls.localize(19, null, JSON.stringify(err)));
    }
    exports.loaderError = loaderError;
    function create(message, options) {
        if (options === void 0) { options = {}; }
        var result = new Error(message);
        if (types.isNumber(options.severity)) {
            result.severity = options.severity;
        }
        if (options.actions) {
            result.actions = options.actions;
        }
        return result;
    }
    exports.create = create;
});

define("vs/base/common/callbackList", ["require", "exports", 'vs/base/common/errors'], function (require, exports, errors_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var CallbackList = (function () {
        function CallbackList() {
        }
        CallbackList.prototype.add = function (callback, context, bucket) {
            var _this = this;
            if (context === void 0) { context = null; }
            if (!this._callbacks) {
                this._callbacks = [];
                this._contexts = [];
            }
            this._callbacks.push(callback);
            this._contexts.push(context);
            if (Array.isArray(bucket)) {
                bucket.push({ dispose: function () { return _this.remove(callback, context); } });
            }
        };
        CallbackList.prototype.remove = function (callback, context) {
            if (context === void 0) { context = null; }
            if (!this._callbacks) {
                return;
            }
            var foundCallbackWithDifferentContext = false;
            for (var i = 0, len = this._callbacks.length; i < len; i++) {
                if (this._callbacks[i] === callback) {
                    if (this._contexts[i] === context) {
                        // callback & context match => remove it
                        this._callbacks.splice(i, 1);
                        this._contexts.splice(i, 1);
                        return;
                    }
                    else {
                        foundCallbackWithDifferentContext = true;
                    }
                }
            }
            if (foundCallbackWithDifferentContext) {
                throw new Error('When adding a listener with a context, you should remove it with the same context');
            }
        };
        CallbackList.prototype.invoke = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            if (!this._callbacks) {
                return;
            }
            var ret = [], callbacks = this._callbacks.slice(0), contexts = this._contexts.slice(0);
            for (var i = 0, len = callbacks.length; i < len; i++) {
                try {
                    ret.push(callbacks[i].apply(contexts[i], args));
                }
                catch (e) {
                    errors_1.onUnexpectedError(e);
                }
            }
            return ret;
        };
        CallbackList.prototype.isEmpty = function () {
            return !this._callbacks || this._callbacks.length === 0;
        };
        CallbackList.prototype.dispose = function () {
            this._callbacks = undefined;
            this._contexts = undefined;
        };
        return CallbackList;
    }());
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = CallbackList;
});

define("vs/base/common/event", ["require", "exports", 'vs/base/common/callbackList'], function (require, exports, callbackList_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Event;
    (function (Event) {
        var _disposable = { dispose: function () { } };
        Event.None = function () { return _disposable; };
    })(Event || (Event = {}));
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = Event;
    /**
     * The Emitter can be used to expose an Event to the public
     * to fire it from the insides.
     * Sample:
        class Document {
    
            private _onDidChange = new Emitter<(value:string)=>any>();
    
            public onDidChange = this._onDidChange.event;
    
            // getter-style
            // get onDidChange(): Event<(value:string)=>any> {
            // 	return this._onDidChange.event;
            // }
    
            private _doIt() {
                //...
                this._onDidChange.fire(value);
            }
        }
     */
    var Emitter = (function () {
        function Emitter(_options) {
            this._options = _options;
        }
        Object.defineProperty(Emitter.prototype, "event", {
            /**
             * For the public to allow to subscribe
             * to events from this Emitter
             */
            get: function () {
                var _this = this;
                if (!this._event) {
                    this._event = function (listener, thisArgs, disposables) {
                        if (!_this._callbacks) {
                            _this._callbacks = new callbackList_1.default();
                        }
                        if (_this._options && _this._options.onFirstListenerAdd && _this._callbacks.isEmpty()) {
                            _this._options.onFirstListenerAdd(_this);
                        }
                        _this._callbacks.add(listener, thisArgs);
                        var result;
                        result = {
                            dispose: function () {
                                result.dispose = Emitter._noop;
                                if (!_this._disposed) {
                                    _this._callbacks.remove(listener, thisArgs);
                                    if (_this._options && _this._options.onLastListenerRemove && _this._callbacks.isEmpty()) {
                                        _this._options.onLastListenerRemove(_this);
                                    }
                                }
                            }
                        };
                        if (Array.isArray(disposables)) {
                            disposables.push(result);
                        }
                        return result;
                    };
                }
                return this._event;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * To be kept private to fire an event to
         * subscribers
         */
        Emitter.prototype.fire = function (event) {
            if (this._callbacks) {
                this._callbacks.invoke.call(this._callbacks, event);
            }
        };
        Emitter.prototype.dispose = function () {
            if (this._callbacks) {
                this._callbacks.dispose();
                this._callbacks = undefined;
                this._disposed = true;
            }
        };
        Emitter._noop = function () { };
        return Emitter;
    }());
    exports.Emitter = Emitter;
    /**
     * Creates an Event which is backed-up by the event emitter. This allows
     * to use the existing eventing pattern and is likely using less memory.
     * Sample:
     *
     * 	class Document {
     *
     *		private _eventbus = new EventEmitter();
     *
     *		public onDidChange = fromEventEmitter(this._eventbus, 'changed');
     *
     *		// getter-style
     *		// get onDidChange(): Event<(value:string)=>any> {
     *		// 	cache fromEventEmitter result and return
     *		// }
     *
     *		private _doIt() {
     *			// ...
     *			this._eventbus.emit('changed', value)
     *		}
     *	}
     */
    function fromEventEmitter(emitter, eventType) {
        return function (listener, thisArgs, disposables) {
            var result = emitter.addListener2(eventType, function () {
                listener.apply(thisArgs, arguments);
            });
            if (Array.isArray(disposables)) {
                disposables.push(result);
            }
            return result;
        };
    }
    exports.fromEventEmitter = fromEventEmitter;
    function mapEvent(event, map) {
        return function (listener, thisArgs, disposables) {
            return event(function (i) { return listener(map(i)); }, thisArgs, disposables);
        };
    }
    exports.mapEvent = mapEvent;
    var EventDelayerState;
    (function (EventDelayerState) {
        EventDelayerState[EventDelayerState["Idle"] = 0] = "Idle";
        EventDelayerState[EventDelayerState["Running"] = 1] = "Running";
    })(EventDelayerState || (EventDelayerState = {}));
    /**
     * The EventDelayer is useful in situations in which you want
     * to delay firing your events during some code.
     * You can wrap that code and be sure that the event will not
     * be fired during that wrap.
     *
     * ```
     * const emitter: Emitter;
     * const delayer = new EventDelayer();
     * const delayedEvent = delayer.delay(emitter.event);
     *
     * delayedEvent(console.log);
     *
     * delayer.wrap(() => {
     *   emitter.fire(); // event will not be fired yet
     * });
     *
     * // event will only be fired at this point
     * ```
     */
    var EventBufferer = (function () {
        function EventBufferer() {
            this.buffers = [];
        }
        EventBufferer.prototype.wrapEvent = function (event) {
            var _this = this;
            return function (listener, thisArgs, disposables) {
                return event(function (i) {
                    var buffer = _this.buffers[_this.buffers.length - 1];
                    if (buffer) {
                        buffer.push(function () { return listener.call(thisArgs, i); });
                    }
                    else {
                        listener.call(thisArgs, i);
                    }
                }, void 0, disposables);
            };
        };
        EventBufferer.prototype.bufferEvents = function (fn) {
            var buffer = [];
            this.buffers.push(buffer);
            fn();
            this.buffers.pop();
            buffer.forEach(function (flush) { return flush(); });
        };
        return EventBufferer;
    }());
    exports.EventBufferer = EventBufferer;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/common/cancellation", ["require", "exports", 'vs/base/common/event'], function (require, exports, event_1) {
    'use strict';
    var CancellationToken;
    (function (CancellationToken) {
        CancellationToken.None = Object.freeze({
            isCancellationRequested: false,
            onCancellationRequested: event_1.default.None
        });
        CancellationToken.Cancelled = Object.freeze({
            isCancellationRequested: true,
            onCancellationRequested: event_1.default.None
        });
    })(CancellationToken = exports.CancellationToken || (exports.CancellationToken = {}));
    var shortcutEvent = Object.freeze(function (callback, context) {
        var handle = setTimeout(callback.bind(context), 0);
        return { dispose: function () { clearTimeout(handle); } };
    });
    var MutableToken = (function () {
        function MutableToken() {
            this._isCancelled = false;
        }
        MutableToken.prototype.cancel = function () {
            if (!this._isCancelled) {
                this._isCancelled = true;
                if (this._emitter) {
                    this._emitter.fire(undefined);
                    this._emitter = undefined;
                }
            }
        };
        Object.defineProperty(MutableToken.prototype, "isCancellationRequested", {
            get: function () {
                return this._isCancelled;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MutableToken.prototype, "onCancellationRequested", {
            get: function () {
                if (this._isCancelled) {
                    return shortcutEvent;
                }
                if (!this._emitter) {
                    this._emitter = new event_1.Emitter();
                }
                return this._emitter.event;
            },
            enumerable: true,
            configurable: true
        });
        return MutableToken;
    }());
    var CancellationTokenSource = (function () {
        function CancellationTokenSource() {
        }
        Object.defineProperty(CancellationTokenSource.prototype, "token", {
            get: function () {
                if (!this._token) {
                    // be lazy and create the token only when
                    // actually needed
                    this._token = new MutableToken();
                }
                return this._token;
            },
            enumerable: true,
            configurable: true
        });
        CancellationTokenSource.prototype.cancel = function () {
            if (!this._token) {
                // save an object by returning the default
                // cancelled token when cancellation happens
                // before someone asks for the token
                this._token = CancellationToken.Cancelled;
            }
            else {
                this._token.cancel();
            }
        };
        CancellationTokenSource.prototype.dispose = function () {
            this.cancel();
        };
        return CancellationTokenSource;
    }());
    exports.CancellationTokenSource = CancellationTokenSource;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/common/eventEmitter", ["require", "exports", 'vs/base/common/errors'], function (require, exports, Errors) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EmitterEvent = (function () {
        function EmitterEvent(eventType, data, emitterType) {
            if (emitterType === void 0) { emitterType = null; }
            this._type = eventType;
            this._data = data;
            this._emitterType = emitterType;
        }
        EmitterEvent.prototype.getType = function () {
            return this._type;
        };
        EmitterEvent.prototype.getData = function () {
            return this._data;
        };
        EmitterEvent.prototype.getEmitterType = function () {
            return this._emitterType;
        };
        return EmitterEvent;
    }());
    exports.EmitterEvent = EmitterEvent;
    var EventEmitter = (function () {
        function EventEmitter(allowedEventTypes) {
            if (allowedEventTypes === void 0) { allowedEventTypes = null; }
            this._listeners = {};
            this._bulkListeners = [];
            this._collectedEvents = [];
            this._deferredCnt = 0;
            if (allowedEventTypes) {
                this._allowedEventTypes = {};
                for (var i = 0; i < allowedEventTypes.length; i++) {
                    this._allowedEventTypes[allowedEventTypes[i]] = true;
                }
            }
            else {
                this._allowedEventTypes = null;
            }
        }
        EventEmitter.prototype.dispose = function () {
            this._listeners = {};
            this._bulkListeners = [];
            this._collectedEvents = [];
            this._deferredCnt = 0;
            this._allowedEventTypes = null;
        };
        EventEmitter.prototype.addListener = function (eventType, listener) {
            if (eventType === '*') {
                throw new Error('Use addBulkListener(listener) to register your listener!');
            }
            if (this._allowedEventTypes && !this._allowedEventTypes.hasOwnProperty(eventType)) {
                throw new Error('This object will never emit this event type!');
            }
            if (this._listeners.hasOwnProperty(eventType)) {
                this._listeners[eventType].push(listener);
            }
            else {
                this._listeners[eventType] = [listener];
            }
            var bound = this;
            return function () {
                if (!bound) {
                    // Already called
                    return;
                }
                bound._removeListener(eventType, listener);
                // Prevent leakers from holding on to the event emitter
                bound = null;
                listener = null;
            };
        };
        EventEmitter.prototype.addListener2 = function (eventType, listener) {
            var dispose = this.addListener(eventType, listener);
            return {
                dispose: dispose
            };
        };
        EventEmitter.prototype.on = function (eventType, listener) {
            return this.addListener(eventType, listener);
        };
        EventEmitter.prototype.addOneTimeListener = function (eventType, listener) {
            var unbind = this.addListener(eventType, function (value) {
                unbind();
                listener(value);
            });
            return unbind;
        };
        EventEmitter.prototype.addOneTimeDisposableListener = function (eventType, listener) {
            var dispose = this.addOneTimeListener(eventType, listener);
            return {
                dispose: dispose
            };
        };
        EventEmitter.prototype.addBulkListener = function (listener) {
            var _this = this;
            this._bulkListeners.push(listener);
            return function () {
                _this._removeBulkListener(listener);
            };
        };
        EventEmitter.prototype.addBulkListener2 = function (listener) {
            var dispose = this.addBulkListener(listener);
            return {
                dispose: dispose
            };
        };
        EventEmitter.prototype.addEmitter = function (eventEmitter, emitterType) {
            var _this = this;
            if (emitterType === void 0) { emitterType = null; }
            return eventEmitter.addBulkListener(function (events) {
                var newEvents = events;
                if (emitterType) {
                    // If the emitter has an emitterType, recreate events
                    newEvents = [];
                    for (var i = 0, len = events.length; i < len; i++) {
                        newEvents.push(new EmitterEvent(events[i].getType(), events[i].getData(), emitterType));
                    }
                }
                if (_this._deferredCnt === 0) {
                    _this._emitEvents(newEvents);
                }
                else {
                    // Collect for later
                    _this._collectedEvents.push.apply(_this._collectedEvents, newEvents);
                }
            });
        };
        EventEmitter.prototype.addEmitter2 = function (eventEmitter, emitterType) {
            var dispose = this.addEmitter(eventEmitter, emitterType);
            return {
                dispose: dispose
            };
        };
        EventEmitter.prototype.addEmitterTypeListener = function (eventType, emitterType, listener) {
            if (emitterType) {
                if (eventType === '*') {
                    throw new Error('Bulk listeners cannot specify an emitter type');
                }
                return this.addListener(eventType + '/' + emitterType, listener);
            }
            else {
                return this.addListener(eventType, listener);
            }
        };
        EventEmitter.prototype._removeListener = function (eventType, listener) {
            if (this._listeners.hasOwnProperty(eventType)) {
                var listeners = this._listeners[eventType];
                for (var i = 0, len = listeners.length; i < len; i++) {
                    if (listeners[i] === listener) {
                        listeners.splice(i, 1);
                        break;
                    }
                }
            }
        };
        EventEmitter.prototype._removeBulkListener = function (listener) {
            for (var i = 0, len = this._bulkListeners.length; i < len; i++) {
                if (this._bulkListeners[i] === listener) {
                    this._bulkListeners.splice(i, 1);
                    break;
                }
            }
        };
        EventEmitter.prototype._emitToSpecificTypeListeners = function (eventType, data) {
            if (this._listeners.hasOwnProperty(eventType)) {
                var listeners = this._listeners[eventType].slice(0);
                for (var i = 0, len = listeners.length; i < len; i++) {
                    try {
                        listeners[i](data);
                    }
                    catch (e) {
                        Errors.onUnexpectedError(e);
                    }
                }
            }
        };
        EventEmitter.prototype._emitToBulkListeners = function (events) {
            var bulkListeners = this._bulkListeners.slice(0);
            for (var i = 0, len = bulkListeners.length; i < len; i++) {
                try {
                    bulkListeners[i](events);
                }
                catch (e) {
                    Errors.onUnexpectedError(e);
                }
            }
        };
        EventEmitter.prototype._emitEvents = function (events) {
            if (this._bulkListeners.length > 0) {
                this._emitToBulkListeners(events);
            }
            for (var i = 0, len = events.length; i < len; i++) {
                var e = events[i];
                this._emitToSpecificTypeListeners(e.getType(), e.getData());
                if (e.getEmitterType()) {
                    this._emitToSpecificTypeListeners(e.getType() + '/' + e.getEmitterType(), e.getData());
                }
            }
        };
        EventEmitter.prototype.emit = function (eventType, data) {
            if (data === void 0) { data = {}; }
            if (this._allowedEventTypes && !this._allowedEventTypes.hasOwnProperty(eventType)) {
                throw new Error('Cannot emit this event type because it wasn\'t white-listed!');
            }
            // Early return if no listeners would get this
            if (!this._listeners.hasOwnProperty(eventType) && this._bulkListeners.length === 0) {
                return;
            }
            var emitterEvent = new EmitterEvent(eventType, data);
            if (this._deferredCnt === 0) {
                this._emitEvents([emitterEvent]);
            }
            else {
                // Collect for later
                this._collectedEvents.push(emitterEvent);
            }
        };
        EventEmitter.prototype.deferredEmit = function (callback) {
            this._deferredCnt = this._deferredCnt + 1;
            var result = null;
            try {
                result = callback();
            }
            catch (e) {
                Errors.onUnexpectedError(e);
            }
            this._deferredCnt = this._deferredCnt - 1;
            if (this._deferredCnt === 0) {
                this._emitCollected();
            }
            return result;
        };
        EventEmitter.prototype._emitCollected = function () {
            // Flush collected events
            var events = this._collectedEvents;
            this._collectedEvents = [];
            if (events.length > 0) {
                this._emitEvents(events);
            }
        };
        return EventEmitter;
    }());
    exports.EventEmitter = EventEmitter;
    var EmitQueueElement = (function () {
        function EmitQueueElement(target, arg) {
            this.target = target;
            this.arg = arg;
        }
        return EmitQueueElement;
    }());
    /**
     * Same as EventEmitter, but guarantees events are delivered in order to each listener
     */
    var OrderGuaranteeEventEmitter = (function (_super) {
        __extends(OrderGuaranteeEventEmitter, _super);
        function OrderGuaranteeEventEmitter(allowedEventTypes) {
            if (allowedEventTypes === void 0) { allowedEventTypes = null; }
            _super.call(this, allowedEventTypes);
            this._emitQueue = [];
        }
        OrderGuaranteeEventEmitter.prototype._emitToSpecificTypeListeners = function (eventType, data) {
            if (this._listeners.hasOwnProperty(eventType)) {
                var listeners = this._listeners[eventType];
                for (var i = 0, len = listeners.length; i < len; i++) {
                    this._emitQueue.push(new EmitQueueElement(listeners[i], data));
                }
            }
        };
        OrderGuaranteeEventEmitter.prototype._emitToBulkListeners = function (events) {
            var bulkListeners = this._bulkListeners;
            for (var i = 0, len = bulkListeners.length; i < len; i++) {
                this._emitQueue.push(new EmitQueueElement(bulkListeners[i], events));
            }
        };
        OrderGuaranteeEventEmitter.prototype._emitEvents = function (events) {
            _super.prototype._emitEvents.call(this, events);
            while (this._emitQueue.length > 0) {
                var queueElement = this._emitQueue.shift();
                try {
                    queueElement.target(queueElement.arg);
                }
                catch (e) {
                    Errors.onUnexpectedError(e);
                }
            }
        };
        return OrderGuaranteeEventEmitter;
    }(EventEmitter));
    exports.OrderGuaranteeEventEmitter = OrderGuaranteeEventEmitter;
});

define("vs/base/common/timer", ["require", "exports", 'vs/base/common/platform', 'vs/base/common/errors', 'vs/base/common/stopwatch'], function (require, exports, Platform, errors, precision) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.ENABLE_TIMER = false;
    var msWriteProfilerMark = Platform.globals['msWriteProfilerMark'];
    (function (Topic) {
        Topic[Topic["EDITOR"] = 0] = "EDITOR";
        Topic[Topic["LANGUAGES"] = 1] = "LANGUAGES";
        Topic[Topic["WORKER"] = 2] = "WORKER";
        Topic[Topic["WORKBENCH"] = 3] = "WORKBENCH";
        Topic[Topic["STARTUP"] = 4] = "STARTUP";
    })(exports.Topic || (exports.Topic = {}));
    var Topic = exports.Topic;
    var NullTimerEvent = (function () {
        function NullTimerEvent() {
        }
        NullTimerEvent.prototype.stop = function () {
            return;
        };
        NullTimerEvent.prototype.timeTaken = function () {
            return -1;
        };
        return NullTimerEvent;
    }());
    var TimerEvent = (function () {
        function TimerEvent(timeKeeper, name, topic, startTime, description) {
            this.timeKeeper = timeKeeper;
            this.name = name;
            this.description = description;
            this.topic = topic;
            this.stopTime = null;
            if (startTime) {
                this.startTime = startTime;
                return;
            }
            this.startTime = new Date();
            this.sw = precision.StopWatch.create();
            if (msWriteProfilerMark) {
                var profilerName = ['Monaco', this.topic, this.name, 'start'];
                msWriteProfilerMark(profilerName.join('|'));
            }
        }
        TimerEvent.prototype.stop = function (stopTime) {
            // already stopped
            if (this.stopTime !== null) {
                return;
            }
            if (stopTime) {
                this.stopTime = stopTime;
                this.sw = null;
                this.timeKeeper._onEventStopped(this);
                return;
            }
            this.stopTime = new Date();
            if (this.sw) {
                this.sw.stop();
            }
            this.timeKeeper._onEventStopped(this);
            if (msWriteProfilerMark) {
                var profilerName = ['Monaco', this.topic, this.name, 'stop'];
                msWriteProfilerMark(profilerName.join('|'));
            }
        };
        TimerEvent.prototype.timeTaken = function () {
            if (this.sw) {
                return this.sw.elapsed();
            }
            if (this.stopTime) {
                return this.stopTime.getTime() - this.startTime.getTime();
            }
            return -1;
        };
        return TimerEvent;
    }());
    var TimeKeeper /*extends EventEmitter.EventEmitter*/ = (function () {
        function TimeKeeper /*extends EventEmitter.EventEmitter*/() {
            this.cleaningIntervalId = -1;
            this.collectedEvents = [];
            this.listeners = [];
        }
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype.isEnabled = function () {
            return exports.ENABLE_TIMER;
        };
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype.start = function (topic, name, start, description) {
            if (!this.isEnabled()) {
                return exports.nullEvent;
            }
            var strTopic;
            if (typeof topic === 'string') {
                strTopic = topic;
            }
            else if (topic === Topic.EDITOR) {
                strTopic = 'Editor';
            }
            else if (topic === Topic.LANGUAGES) {
                strTopic = 'Languages';
            }
            else if (topic === Topic.WORKER) {
                strTopic = 'Worker';
            }
            else if (topic === Topic.WORKBENCH) {
                strTopic = 'Workbench';
            }
            else if (topic === Topic.STARTUP) {
                strTopic = 'Startup';
            }
            this.initAutoCleaning();
            var event = new TimerEvent(this, name, strTopic, start, description);
            this.addEvent(event);
            return event;
        };
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype.dispose = function () {
            if (this.cleaningIntervalId !== -1) {
                Platform.clearInterval(this.cleaningIntervalId);
                this.cleaningIntervalId = -1;
            }
        };
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype.addListener = function (listener) {
            this.listeners.push(listener);
        };
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype.removeListener = function (listener) {
            for (var i = 0; i < this.listeners.length; i++) {
                if (this.listeners[i] === listener) {
                    this.listeners.splice(i, 1);
                    return;
                }
            }
        };
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype.addEvent = function (event) {
            event.id = TimeKeeper.EVENT_ID;
            TimeKeeper.EVENT_ID++;
            this.collectedEvents.push(event);
            // expire items from the front of the cache
            if (this.collectedEvents.length > TimeKeeper._EVENT_CACHE_LIMIT) {
                this.collectedEvents.shift();
            }
        };
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype.initAutoCleaning = function () {
            var _this = this;
            if (this.cleaningIntervalId === -1) {
                this.cleaningIntervalId = Platform.setInterval(function () {
                    var now = Date.now();
                    _this.collectedEvents.forEach(function (event) {
                        if (!event.stopTime && (now - event.startTime.getTime()) >= TimeKeeper._MAX_TIMER_LENGTH) {
                            event.stop();
                        }
                    });
                }, TimeKeeper._CLEAN_UP_INTERVAL);
            }
        };
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype.getCollectedEvents = function () {
            return this.collectedEvents.slice(0);
        };
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype.clearCollectedEvents = function () {
            this.collectedEvents = [];
        };
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype._onEventStopped = function (event) {
            var emitEvents = [event];
            var listeners = this.listeners.slice(0);
            for (var i = 0; i < listeners.length; i++) {
                try {
                    listeners[i](emitEvents);
                }
                catch (e) {
                    errors.onUnexpectedError(e);
                }
            }
        };
        TimeKeeper /*extends EventEmitter.EventEmitter*/.prototype.setInitialCollectedEvents = function (events, startTime) {
            var _this = this;
            if (!this.isEnabled()) {
                return;
            }
            if (startTime) {
                TimeKeeper.PARSE_TIME = startTime;
            }
            events.forEach(function (event) {
                var e = new TimerEvent(_this, event.name, event.topic, event.startTime, event.description);
                e.stop(event.stopTime);
                _this.addEvent(e);
            });
        };
        /**
         * After being started for 1 minute, all timers are automatically stopped.
         */
        TimeKeeper /*extends EventEmitter.EventEmitter*/._MAX_TIMER_LENGTH = 60000; // 1 minute
        /**
         * Every 2 minutes, a sweep of current started timers is done.
         */
        TimeKeeper /*extends EventEmitter.EventEmitter*/._CLEAN_UP_INTERVAL = 120000; // 2 minutes
        /**
         * Collect at most 1000 events.
         */
        TimeKeeper /*extends EventEmitter.EventEmitter*/._EVENT_CACHE_LIMIT = 1000;
        TimeKeeper /*extends EventEmitter.EventEmitter*/.EVENT_ID = 1;
        TimeKeeper /*extends EventEmitter.EventEmitter*/.PARSE_TIME = new Date();
        return TimeKeeper /*extends EventEmitter.EventEmitter*/;
    }());
    exports.TimeKeeper /*extends EventEmitter.EventEmitter*/ = TimeKeeper /*extends EventEmitter.EventEmitter*/;
    var timeKeeper = new TimeKeeper();
    exports.nullEvent = new NullTimerEvent();
    function start(topic, name, start, description) {
        return timeKeeper.start(topic, name, start, description);
    }
    exports.start = start;
    function getTimeKeeper() {
        return timeKeeper;
    }
    exports.getTimeKeeper = getTimeKeeper;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/

define("vs/base/common/winjs.base", ['./winjs.base.raw', 'vs/base/common/errors'], function (raw, __Errors__) {
	'use strict';

	var winjs = raw;

	var outstandingPromiseErrors = {};
	function promiseErrorHandler(e) {

		//
		// e.detail looks like: { exception, error, promise, handler, id, parent }
		//
		var details = e.detail;
		var id = details.id;

		// If the error has a parent promise then this is not the origination of the
		//  error so we check if it has a handler, and if so we mark that the error
		//  was handled by removing it from outstandingPromiseErrors
		//
		if (details.parent) {
			if (details.handler && outstandingPromiseErrors) {
				delete outstandingPromiseErrors[id];
			}
			return;
		}

		// Indicate that this error was originated and needs to be handled
		outstandingPromiseErrors[id] = details;

		// The first time the queue fills up this iteration, schedule a timeout to
		// check if any errors are still unhandled.
		if (Object.keys(outstandingPromiseErrors).length === 1) {
			setTimeout(function () {
				var errors = outstandingPromiseErrors;
				outstandingPromiseErrors = {};
				Object.keys(errors).forEach(function (errorId) {
					var error = errors[errorId];
					if(error.exception) {
						__Errors__.onUnexpectedError(error.exception);
					} else if(error.error) {
						__Errors__.onUnexpectedError(error.error);
					}
					console.log("WARNING: Promise with no error callback:" + error.id);
					console.log(error);
					if(error.exception) {
						console.log(error.exception.stack);
					}
				});
			}, 0);
		}
	}

	winjs.Promise.addEventListener("error", promiseErrorHandler);


	function decoratePromise(promise, completeCallback, errorCallback) {
		var pc, pe, pp;

		var resultPromise = new winjs.Promise(
			function (c, e, p) {
				pc = c;
				pe = e;
				pp = p;
			}, function () {
				promise.cancel();
			}
		);

		promise.then(function (r) {
			if (completeCallback) {
				completeCallback(r);
			}
			pc(r);
		}, function (e) {
			if (errorCallback) {
				errorCallback(e);
			}
			pe(e);
		}, pp);

		return resultPromise;
	}

	return {
		decoratePromise: decoratePromise,
		Class: winjs.Class,
		xhr: winjs.xhr,
		Promise: winjs.Promise,
		TPromise: winjs.Promise,
		PPromise: winjs.Promise,
		Utilities: winjs.Utilities
	};
});
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/common/actions", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/eventEmitter', 'vs/base/common/events'], function (require, exports, winjs_base_1, eventEmitter_1, Events) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Checks if the provided object is compatible
     * with the IAction interface.
     * @param thing an object
     */
    function isAction(thing) {
        if (!thing) {
            return false;
        }
        else if (thing instanceof Action) {
            return true;
        }
        else if (typeof thing.id !== 'string') {
            return false;
        }
        else if (typeof thing.label !== 'string') {
            return false;
        }
        else if (typeof thing.class !== 'string') {
            return false;
        }
        else if (typeof thing.enabled !== 'boolean') {
            return false;
        }
        else if (typeof thing.checked !== 'boolean') {
            return false;
        }
        else if (typeof thing.run !== 'function') {
            return false;
        }
        else {
            return true;
        }
    }
    exports.isAction = isAction;
    var Action = (function (_super) {
        __extends(Action, _super);
        function Action(id, label, cssClass, enabled, actionCallback) {
            if (label === void 0) { label = ''; }
            if (cssClass === void 0) { cssClass = ''; }
            if (enabled === void 0) { enabled = true; }
            if (actionCallback === void 0) { actionCallback = null; }
            _super.call(this);
            this._id = id;
            this._label = label;
            this._cssClass = cssClass;
            this._enabled = enabled;
            this._actionCallback = actionCallback;
        }
        Object.defineProperty(Action.prototype, "id", {
            get: function () {
                return this._id;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Action.prototype, "label", {
            get: function () {
                return this._label;
            },
            set: function (value) {
                this._setLabel(value);
            },
            enumerable: true,
            configurable: true
        });
        Action.prototype._setLabel = function (value) {
            if (this._label !== value) {
                this._label = value;
                this.emit(Action.LABEL, { source: this });
            }
        };
        Object.defineProperty(Action.prototype, "tooltip", {
            get: function () {
                return this._tooltip;
            },
            set: function (value) {
                this._setTooltip(value);
            },
            enumerable: true,
            configurable: true
        });
        Action.prototype._setTooltip = function (value) {
            if (this._tooltip !== value) {
                this._tooltip = value;
                this.emit(Action.TOOLTIP, { source: this });
            }
        };
        Object.defineProperty(Action.prototype, "class", {
            get: function () {
                return this._cssClass;
            },
            set: function (value) {
                this._setClass(value);
            },
            enumerable: true,
            configurable: true
        });
        Action.prototype._setClass = function (value) {
            if (this._cssClass !== value) {
                this._cssClass = value;
                this.emit(Action.CLASS, { source: this });
            }
        };
        Object.defineProperty(Action.prototype, "enabled", {
            get: function () {
                return this._enabled;
            },
            set: function (value) {
                this._setEnabled(value);
            },
            enumerable: true,
            configurable: true
        });
        Action.prototype._setEnabled = function (value) {
            if (this._enabled !== value) {
                this._enabled = value;
                this.emit(Action.ENABLED, { source: this });
            }
        };
        Object.defineProperty(Action.prototype, "checked", {
            get: function () {
                return this._checked;
            },
            set: function (value) {
                this._setChecked(value);
            },
            enumerable: true,
            configurable: true
        });
        Action.prototype._setChecked = function (value) {
            if (this._checked !== value) {
                this._checked = value;
                this.emit(Action.CHECKED, { source: this });
            }
        };
        Object.defineProperty(Action.prototype, "order", {
            get: function () {
                return this._order;
            },
            set: function (value) {
                this._order = value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Action.prototype, "actionCallback", {
            get: function () {
                return this._actionCallback;
            },
            set: function (value) {
                this._actionCallback = value;
            },
            enumerable: true,
            configurable: true
        });
        Action.prototype.run = function (event) {
            if (this._actionCallback !== null) {
                return this._actionCallback(event);
            }
            else {
                return winjs_base_1.TPromise.as(true);
            }
        };
        Action.LABEL = 'label';
        Action.TOOLTIP = 'tooltip';
        Action.CLASS = 'class';
        Action.ENABLED = 'enabled';
        Action.CHECKED = 'checked';
        return Action;
    }(eventEmitter_1.EventEmitter));
    exports.Action = Action;
    var ProxyAction = (function (_super) {
        __extends(ProxyAction, _super);
        function ProxyAction(delegate, runHandler) {
            _super.call(this, delegate.id, delegate.label, delegate.class, delegate.enabled, null);
            this.delegate = delegate;
            this.runHandler = runHandler;
        }
        Object.defineProperty(ProxyAction.prototype, "id", {
            get: function () {
                return this.delegate.id;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ProxyAction.prototype, "label", {
            get: function () {
                return this.delegate.label;
            },
            set: function (value) {
                this.delegate.label = value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ProxyAction.prototype, "class", {
            get: function () {
                return this.delegate.class;
            },
            set: function (value) {
                this.delegate.class = value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ProxyAction.prototype, "enabled", {
            get: function () {
                return this.delegate.enabled;
            },
            set: function (value) {
                this.delegate.enabled = value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ProxyAction.prototype, "checked", {
            get: function () {
                return this.delegate.checked;
            },
            set: function (value) {
                this.delegate.checked = value;
            },
            enumerable: true,
            configurable: true
        });
        ProxyAction.prototype.run = function (event) {
            this.runHandler(event);
            return this.delegate.run(event);
        };
        ProxyAction.prototype.addListener = function (eventType, listener) {
            return this.delegate.addListener(eventType, listener);
        };
        ProxyAction.prototype.addBulkListener = function (listener) {
            return this.delegate.addBulkListener(listener);
        };
        ProxyAction.prototype.addEmitter = function (eventEmitter, emitterType) {
            return this.delegate.addEmitter(eventEmitter, emitterType);
        };
        ProxyAction.prototype.addEmitterTypeListener = function (eventType, emitterType, listener) {
            return this.delegate.addEmitterTypeListener(eventType, emitterType, listener);
        };
        ProxyAction.prototype.emit = function (eventType, data) {
            this.delegate.emit(eventType, data);
        };
        return ProxyAction;
    }(Action));
    var ActionRunner = (function (_super) {
        __extends(ActionRunner, _super);
        function ActionRunner() {
            _super.apply(this, arguments);
        }
        ActionRunner.prototype.run = function (action, context) {
            var _this = this;
            if (!action.enabled) {
                return winjs_base_1.TPromise.as(null);
            }
            this.emit(Events.EventType.BEFORE_RUN, { action: action });
            return winjs_base_1.TPromise.as(action.run(context)).then(function (result) {
                _this.emit(Events.EventType.RUN, { action: action, result: result });
            }, function (error) {
                _this.emit(Events.EventType.RUN, { action: action, error: error });
            });
        };
        return ActionRunner;
    }(eventEmitter_1.EventEmitter));
    exports.ActionRunner = ActionRunner;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/common/async", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/winjs.base', 'vs/base/common/platform', 'vs/base/common/cancellation', 'vs/base/common/lifecycle'], function (require, exports, errors, winjs_base_1, platform, cancellation_1, lifecycle_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function isThenable(obj) {
        return obj && typeof obj.then === 'function';
    }
    function asWinJsPromise(callback) {
        var source = new cancellation_1.CancellationTokenSource();
        return new winjs_base_1.TPromise(function (resolve, reject) {
            var item = callback(source.token);
            if (isThenable(item)) {
                item.then(resolve, reject);
            }
            else {
                resolve(item);
            }
        }, function () {
            source.cancel();
        });
    }
    exports.asWinJsPromise = asWinJsPromise;
    /**
     * A helper to prevent accumulation of sequential async tasks.
     *
     * Imagine a mail man with the sole task of delivering letters. As soon as
     * a letter submitted for delivery, he drives to the destination, delivers it
     * and returns to his base. Imagine that during the trip, N more letters were submitted.
     * When the mail man returns, he picks those N letters and delivers them all in a
     * single trip. Even though N+1 submissions occurred, only 2 deliveries were made.
     *
     * The throttler implements this via the queue() method, by providing it a task
     * factory. Following the example:
     *
     * 		var throttler = new Throttler();
     * 		var letters = [];
     *
     * 		function letterReceived(l) {
     * 			letters.push(l);
     * 			throttler.queue(() => { return makeTheTrip(); });
     * 		}
     */
    var Throttler = (function () {
        function Throttler() {
            this.activePromise = null;
            this.queuedPromise = null;
            this.queuedPromiseFactory = null;
        }
        Throttler.prototype.queue = function (promiseFactory) {
            var _this = this;
            if (this.activePromise) {
                this.queuedPromiseFactory = promiseFactory;
                if (!this.queuedPromise) {
                    var onComplete_1 = function () {
                        _this.queuedPromise = null;
                        var result = _this.queue(_this.queuedPromiseFactory);
                        _this.queuedPromiseFactory = null;
                        return result;
                    };
                    this.queuedPromise = new winjs_base_1.Promise(function (c, e, p) {
                        _this.activePromise.then(onComplete_1, onComplete_1, p).done(c);
                    }, function () {
                        _this.activePromise.cancel();
                    });
                }
                return new winjs_base_1.Promise(function (c, e, p) {
                    _this.queuedPromise.then(c, e, p);
                }, function () {
                    // no-op
                });
            }
            this.activePromise = promiseFactory();
            return new winjs_base_1.Promise(function (c, e, p) {
                _this.activePromise.done(function (result) {
                    _this.activePromise = null;
                    c(result);
                }, function (err) {
                    _this.activePromise = null;
                    e(err);
                }, p);
            }, function () {
                _this.activePromise.cancel();
            });
        };
        return Throttler;
    }());
    exports.Throttler = Throttler;
    /**
     * A helper to delay execution of a task that is being requested often.
     *
     * Following the throttler, now imagine the mail man wants to optimize the number of
     * trips proactively. The trip itself can be long, so the he decides not to make the trip
     * as soon as a letter is submitted. Instead he waits a while, in case more
     * letters are submitted. After said waiting period, if no letters were submitted, he
     * decides to make the trip. Imagine that N more letters were submitted after the first
     * one, all within a short period of time between each other. Even though N+1
     * submissions occurred, only 1 delivery was made.
     *
     * The delayer offers this behavior via the trigger() method, into which both the task
     * to be executed and the waiting period (delay) must be passed in as arguments. Following
     * the example:
     *
     * 		var delayer = new Delayer(WAITING_PERIOD);
     * 		var letters = [];
     *
     * 		function letterReceived(l) {
     * 			letters.push(l);
     * 			delayer.trigger(() => { return makeTheTrip(); });
     * 		}
     */
    var Delayer = (function () {
        function Delayer(defaultDelay) {
            this.defaultDelay = defaultDelay;
            this.timeout = null;
            this.completionPromise = null;
            this.onSuccess = null;
            this.task = null;
        }
        Delayer.prototype.trigger = function (task, delay) {
            var _this = this;
            if (delay === void 0) { delay = this.defaultDelay; }
            this.task = task;
            this.cancelTimeout();
            if (!this.completionPromise) {
                this.completionPromise = new winjs_base_1.Promise(function (c) {
                    _this.onSuccess = c;
                }, function () {
                    // no-op
                }).then(function () {
                    _this.completionPromise = null;
                    _this.onSuccess = null;
                    var task = _this.task;
                    _this.task = null;
                    return task();
                });
            }
            this.timeout = setTimeout(function () {
                _this.timeout = null;
                _this.onSuccess(null);
            }, delay);
            return this.completionPromise;
        };
        Delayer.prototype.isTriggered = function () {
            return this.timeout !== null;
        };
        Delayer.prototype.cancel = function () {
            this.cancelTimeout();
            if (this.completionPromise) {
                this.completionPromise.cancel();
                this.completionPromise = null;
            }
        };
        Delayer.prototype.cancelTimeout = function () {
            if (this.timeout !== null) {
                clearTimeout(this.timeout);
                this.timeout = null;
            }
        };
        return Delayer;
    }());
    exports.Delayer = Delayer;
    /**
     * A helper to delay execution of a task that is being requested often, while
     * preventing accumulation of consecutive executions, while the task runs.
     *
     * Simply combine the two mail man strategies from the Throttler and Delayer
     * helpers, for an analogy.
     */
    var ThrottledDelayer = (function (_super) {
        __extends(ThrottledDelayer, _super);
        function ThrottledDelayer(defaultDelay) {
            _super.call(this, defaultDelay);
            this.throttler = new Throttler();
        }
        ThrottledDelayer.prototype.trigger = function (promiseFactory, delay) {
            var _this = this;
            return _super.prototype.trigger.call(this, function () { return _this.throttler.queue(promiseFactory); }, delay);
        };
        return ThrottledDelayer;
    }(Delayer));
    exports.ThrottledDelayer = ThrottledDelayer;
    /**
     * Similar to the ThrottledDelayer, except it also guarantees that the promise
     * factory doesn't get called more often than every `minimumPeriod` milliseconds.
     */
    var PeriodThrottledDelayer = (function (_super) {
        __extends(PeriodThrottledDelayer, _super);
        function PeriodThrottledDelayer(defaultDelay, minimumPeriod) {
            if (minimumPeriod === void 0) { minimumPeriod = 0; }
            _super.call(this, defaultDelay);
            this.minimumPeriod = minimumPeriod;
            this.periodThrottler = new Throttler();
        }
        PeriodThrottledDelayer.prototype.trigger = function (promiseFactory, delay) {
            var _this = this;
            return _super.prototype.trigger.call(this, function () {
                return _this.periodThrottler.queue(function () {
                    return winjs_base_1.Promise.join([
                        winjs_base_1.TPromise.timeout(_this.minimumPeriod),
                        promiseFactory()
                    ]).then(function (r) { return r[1]; });
                });
            }, delay);
        };
        return PeriodThrottledDelayer;
    }(ThrottledDelayer));
    exports.PeriodThrottledDelayer = PeriodThrottledDelayer;
    var PromiseSource = (function () {
        function PromiseSource() {
            var _this = this;
            this._value = new winjs_base_1.TPromise(function (c, e) {
                _this._completeCallback = c;
                _this._errorCallback = e;
            });
        }
        Object.defineProperty(PromiseSource.prototype, "value", {
            get: function () {
                return this._value;
            },
            enumerable: true,
            configurable: true
        });
        PromiseSource.prototype.complete = function (value) {
            this._completeCallback(value);
        };
        PromiseSource.prototype.error = function (err) {
            this._errorCallback(err);
        };
        return PromiseSource;
    }());
    exports.PromiseSource = PromiseSource;
    var ShallowCancelThenPromise = (function (_super) {
        __extends(ShallowCancelThenPromise, _super);
        function ShallowCancelThenPromise(outer) {
            var completeCallback, errorCallback, progressCallback;
            _super.call(this, function (c, e, p) {
                completeCallback = c;
                errorCallback = e;
                progressCallback = p;
            }, function () {
                // cancel this promise but not the
                // outer promise
                errorCallback(errors.canceled());
            });
            outer.then(completeCallback, errorCallback, progressCallback);
        }
        return ShallowCancelThenPromise;
    }(winjs_base_1.TPromise));
    exports.ShallowCancelThenPromise = ShallowCancelThenPromise;
    /**
     * Returns a new promise that joins the provided promise. Upon completion of
     * the provided promise the provided function will always be called. This
     * method is comparable to a try-finally code block.
     * @param promise a promise
     * @param f a function that will be call in the success and error case.
     */
    function always(promise, f) {
        return new winjs_base_1.TPromise(function (c, e, p) {
            promise.done(function (result) {
                try {
                    f(result);
                }
                catch (e1) {
                    errors.onUnexpectedError(e1);
                }
                c(result);
            }, function (err) {
                try {
                    f(err);
                }
                catch (e1) {
                    errors.onUnexpectedError(e1);
                }
                e(err);
            }, function (progress) {
                p(progress);
            });
        }, function () {
            promise.cancel();
        });
    }
    exports.always = always;
    /**
     * Runs the provided list of promise factories in sequential order. The returned
     * promise will complete to an array of results from each promise.
     */
    function sequence(promiseFactory) {
        var results = [];
        // reverse since we start with last element using pop()
        promiseFactory = promiseFactory.reverse();
        function next() {
            if (promiseFactory.length) {
                return promiseFactory.pop()();
            }
            return null;
        }
        function thenHandler(result) {
            if (result) {
                results.push(result);
            }
            var n = next();
            if (n) {
                return n.then(thenHandler);
            }
            return winjs_base_1.TPromise.as(results);
        }
        return winjs_base_1.TPromise.as(null).then(thenHandler);
    }
    exports.sequence = sequence;
    function once(fn) {
        var _this = this;
        var didCall = false;
        var result;
        return function () {
            if (didCall) {
                return result;
            }
            didCall = true;
            result = fn.apply(_this, arguments);
            return result;
        };
    }
    exports.once = once;
    /**
     * A helper to queue N promises and run them all with a max degree of parallelism. The helper
     * ensures that at any time no more than M promises are running at the same time.
     */
    var Limiter = (function () {
        function Limiter(maxDegreeOfParalellism) {
            this.maxDegreeOfParalellism = maxDegreeOfParalellism;
            this.outstandingPromises = [];
            this.runningPromises = 0;
        }
        Limiter.prototype.queue = function (promiseFactory) {
            var _this = this;
            return new winjs_base_1.TPromise(function (c, e, p) {
                _this.outstandingPromises.push({
                    factory: promiseFactory,
                    c: c,
                    e: e,
                    p: p
                });
                _this.consume();
            });
        };
        Limiter.prototype.consume = function () {
            var _this = this;
            while (this.outstandingPromises.length && this.runningPromises < this.maxDegreeOfParalellism) {
                var iLimitedTask = this.outstandingPromises.shift();
                this.runningPromises++;
                var promise = iLimitedTask.factory();
                promise.done(iLimitedTask.c, iLimitedTask.e, iLimitedTask.p);
                promise.done(function () { return _this.consumed(); }, function () { return _this.consumed(); });
            }
        };
        Limiter.prototype.consumed = function () {
            this.runningPromises--;
            this.consume();
        };
        return Limiter;
    }());
    exports.Limiter = Limiter;
    var TimeoutTimer = (function (_super) {
        __extends(TimeoutTimer, _super);
        function TimeoutTimer() {
            _super.call(this);
            this._token = -1;
        }
        TimeoutTimer.prototype.dispose = function () {
            this.cancel();
            _super.prototype.dispose.call(this);
        };
        TimeoutTimer.prototype.cancel = function () {
            if (this._token !== -1) {
                platform.clearTimeout(this._token);
                this._token = -1;
            }
        };
        TimeoutTimer.prototype.cancelAndSet = function (runner, timeout) {
            var _this = this;
            this.cancel();
            this._token = platform.setTimeout(function () {
                _this._token = -1;
                runner();
            }, timeout);
        };
        TimeoutTimer.prototype.setIfNotSet = function (runner, timeout) {
            var _this = this;
            if (this._token !== -1) {
                // timer is already set
                return;
            }
            this._token = platform.setTimeout(function () {
                _this._token = -1;
                runner();
            }, timeout);
        };
        return TimeoutTimer;
    }(lifecycle_1.Disposable));
    exports.TimeoutTimer = TimeoutTimer;
    var IntervalTimer = (function (_super) {
        __extends(IntervalTimer, _super);
        function IntervalTimer() {
            _super.call(this);
            this._token = -1;
        }
        IntervalTimer.prototype.dispose = function () {
            this.cancel();
            _super.prototype.dispose.call(this);
        };
        IntervalTimer.prototype.cancel = function () {
            if (this._token !== -1) {
                platform.clearInterval(this._token);
                this._token = -1;
            }
        };
        IntervalTimer.prototype.cancelAndSet = function (runner, interval) {
            this.cancel();
            this._token = platform.setInterval(function () {
                runner();
            }, interval);
        };
        return IntervalTimer;
    }(lifecycle_1.Disposable));
    exports.IntervalTimer = IntervalTimer;
    var RunOnceScheduler = (function () {
        function RunOnceScheduler(runner, timeout) {
            this.timeoutToken = -1;
            this.runner = runner;
            this.timeout = timeout;
            this.timeoutHandler = this.onTimeout.bind(this);
        }
        /**
         * Dispose RunOnceScheduler
         */
        RunOnceScheduler.prototype.dispose = function () {
            this.cancel();
            this.runner = null;
        };
        /**
         * Cancel current scheduled runner (if any).
         */
        RunOnceScheduler.prototype.cancel = function () {
            if (this.isScheduled()) {
                platform.clearTimeout(this.timeoutToken);
                this.timeoutToken = -1;
            }
        };
        /**
         * Replace runner. If there is a runner already scheduled, the new runner will be called.
         */
        RunOnceScheduler.prototype.setRunner = function (runner) {
            this.runner = runner;
        };
        /**
         * Set timeout. This change will only impact new schedule calls.
         */
        RunOnceScheduler.prototype.setTimeout = function (timeout) {
            this.timeout = timeout;
        };
        /**
         * Cancel previous runner (if any) & schedule a new runner.
         */
        RunOnceScheduler.prototype.schedule = function () {
            this.cancel();
            this.timeoutToken = platform.setTimeout(this.timeoutHandler, this.timeout);
        };
        /**
         * Returns true if scheduled.
         */
        RunOnceScheduler.prototype.isScheduled = function () {
            return this.timeoutToken !== -1;
        };
        RunOnceScheduler.prototype.onTimeout = function () {
            this.timeoutToken = -1;
            if (this.runner) {
                this.runner();
            }
        };
        return RunOnceScheduler;
    }());
    exports.RunOnceScheduler = RunOnceScheduler;
    function nfcall(fn) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return new winjs_base_1.Promise(function (c, e) { return fn.apply(void 0, args.concat([function (err, result) { return err ? e(err) : c(result); }])); });
    }
    exports.nfcall = nfcall;
    function ninvoke(thisArg, fn) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        return new winjs_base_1.Promise(function (c, e) { return fn.call.apply(fn, [thisArg].concat(args, [function (err, result) { return err ? e(err) : c(result); }])); });
    }
    exports.ninvoke = ninvoke;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/common/service", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/objects', 'vs/base/common/lifecycle', 'vs/base/common/event'], function (require, exports, winjs_base_1, objects_1, lifecycle_1, event_1) {
    'use strict';
    var RequestType;
    (function (RequestType) {
        RequestType[RequestType["Common"] = 0] = "Common";
        RequestType[RequestType["Cancel"] = 1] = "Cancel";
    })(RequestType || (RequestType = {}));
    var ResponseType;
    (function (ResponseType) {
        ResponseType[ResponseType["Initialize"] = 0] = "Initialize";
        ResponseType[ResponseType["Success"] = 1] = "Success";
        ResponseType[ResponseType["Progress"] = 2] = "Progress";
        ResponseType[ResponseType["Error"] = 3] = "Error";
        ResponseType[ResponseType["ErrorObj"] = 4] = "ErrorObj";
    })(ResponseType || (ResponseType = {}));
    var ServiceState;
    (function (ServiceState) {
        ServiceState[ServiceState["Uninitialized"] = 0] = "Uninitialized";
        ServiceState[ServiceState["Idle"] = 1] = "Idle";
    })(ServiceState || (ServiceState = {}));
    var ServiceEventProperty = '$__SERVICE_EVENT';
    /**
     * Use this as a property decorator.
     */
    function ServiceEvent(target, key) {
        target[key] = (_a = {}, _a[ServiceEventProperty] = true, _a);
        var _a;
    }
    exports.ServiceEvent = ServiceEvent;
    function isServiceEvent(target) {
        return target[ServiceEventProperty];
    }
    exports.isServiceEvent = isServiceEvent;
    var Server = (function () {
        function Server(protocol) {
            var _this = this;
            this.protocol = protocol;
            this.services = Object.create(null);
            this.activeRequests = Object.create(null);
            this.protocol.onMessage(function (r) { return _this.onMessage(r); });
            this.protocol.send({ type: ResponseType.Initialize });
        }
        Server.prototype.registerService = function (serviceName, service) {
            this.services[serviceName] = service;
        };
        Server.prototype.onMessage = function (request) {
            switch (request.type) {
                case RequestType.Common:
                    this.onCommonRequest(request);
                    break;
                case RequestType.Cancel:
                    this.onCancelRequest(request);
                    break;
            }
        };
        Server.prototype.onCommonRequest = function (request) {
            var service = this.services[request.serviceName];
            var servicePrototype = service.constructor.prototype;
            var prototypeMethod = servicePrototype && servicePrototype[request.name];
            var isEvent = prototypeMethod && prototypeMethod[ServiceEventProperty];
            var method = service[request.name];
            var promise;
            if (isEvent) {
                var disposable_1;
                promise = new winjs_base_1.Promise(function (c, e, p) { return disposable_1 = method.call(service, p); }, function () { return disposable_1.dispose(); });
            }
            else {
                if (!method) {
                    promise = winjs_base_1.Promise.wrapError(new Error(request.name + " is not a valid method on " + request.serviceName));
                }
                else {
                    try {
                        promise = method.call.apply(method, [service].concat(request.args));
                    }
                    catch (err) {
                        promise = winjs_base_1.Promise.wrapError(err);
                    }
                }
                if (!winjs_base_1.Promise.is(promise)) {
                    var message = "'" + request.name + "' did not return a promise";
                    console.warn(message);
                    promise = winjs_base_1.Promise.wrapError(new Error(message));
                }
            }
            this.onPromiseRequest(promise, request);
        };
        Server.prototype.onPromiseRequest = function (promise, request) {
            var _this = this;
            var id = request.id;
            var requestPromise = promise.then(function (data) {
                _this.protocol.send({ id: id, data: data, type: ResponseType.Success });
                delete _this.activeRequests[request.id];
            }, function (data) {
                if (data instanceof Error) {
                    _this.protocol.send({ id: id, data: {
                            message: data.message,
                            name: data.name,
                            stack: data.stack ? data.stack.split('\n') : void 0
                        }, type: ResponseType.Error });
                }
                else {
                    _this.protocol.send({ id: id, data: data, type: ResponseType.ErrorObj });
                }
                delete _this.activeRequests[request.id];
            }, function (data) {
                _this.protocol.send({ id: id, data: data, type: ResponseType.Progress });
            });
            this.activeRequests[request.id] = lifecycle_1.fnToDisposable(function () { return requestPromise.cancel(); });
        };
        Server.prototype.onCancelRequest = function (request) {
            var disposable = this.activeRequests[request.id];
            if (disposable) {
                disposable.dispose();
                delete this.activeRequests[request.id];
            }
        };
        Server.prototype.dispose = function () {
            var _this = this;
            Object.keys(this.activeRequests).forEach(function (id) {
                _this.activeRequests[id].dispose();
            });
            this.activeRequests = null;
        };
        return Server;
    }());
    exports.Server = Server;
    var Client = (function () {
        function Client(protocol) {
            var _this = this;
            this.protocol = protocol;
            this.state = ServiceState.Uninitialized;
            this.bufferedRequests = [];
            this.handlers = Object.create(null);
            this.lastRequestId = 0;
            this.protocol.onMessage(function (r) { return _this.onMessage(r); });
        }
        Client.prototype.getService = function (serviceName, serviceCtor) {
            var _this = this;
            var props = Object.keys(serviceCtor.prototype)
                .filter(function (key) { return key !== 'constructor'; });
            return props.reduce(function (service, key) {
                if (serviceCtor.prototype[key][ServiceEventProperty]) {
                    var promise_1;
                    var emitter_1 = new event_1.Emitter({
                        onFirstListenerAdd: function () {
                            promise_1 = _this.request(serviceName, key)
                                .then(null, null, function (event) { return emitter_1.fire(event); });
                        },
                        onLastListenerRemove: function () {
                            promise_1.cancel();
                            promise_1 = null;
                        }
                    });
                    return objects_1.assign(service, (_a = {}, _a[key] = emitter_1.event, _a));
                }
                return objects_1.assign(service, (_b = {}, _b[key] = function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i - 0] = arguments[_i];
                    }
                    return _this.request.apply(_this, [serviceName, key].concat(args));
                }, _b));
                var _a, _b;
            }, {});
        };
        Client.prototype.request = function (serviceName, name) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var request = {
                raw: {
                    id: this.lastRequestId++,
                    type: RequestType.Common,
                    serviceName: serviceName,
                    name: name,
                    args: args
                }
            };
            if (this.state === ServiceState.Uninitialized) {
                return this.bufferRequest(request);
            }
            return this.doRequest(request);
        };
        Client.prototype.doRequest = function (request) {
            var _this = this;
            var id = request.raw.id;
            return new winjs_base_1.Promise(function (c, e, p) {
                _this.handlers[id] = function (response) {
                    switch (response.type) {
                        case ResponseType.Success:
                            delete _this.handlers[id];
                            c(response.data);
                            break;
                        case ResponseType.Error:
                            delete _this.handlers[id];
                            var error = new Error(response.data.message);
                            error.stack = response.data.stack;
                            error.name = response.data.name;
                            e(error);
                            break;
                        case ResponseType.ErrorObj:
                            delete _this.handlers[id];
                            e(response.data);
                            break;
                        case ResponseType.Progress:
                            p(response.data);
                            break;
                    }
                };
                _this.send(request.raw);
            }, function () { return _this.send({ id: id, type: RequestType.Cancel }); });
        };
        Client.prototype.bufferRequest = function (request) {
            var _this = this;
            var flushedRequest = null;
            return new winjs_base_1.Promise(function (c, e, p) {
                _this.bufferedRequests.push(request);
                request.flush = function () {
                    request.flush = null;
                    flushedRequest = _this.doRequest(request).then(c, e, p);
                };
            }, function () {
                request.flush = null;
                if (_this.state !== ServiceState.Uninitialized) {
                    if (flushedRequest) {
                        flushedRequest.cancel();
                        flushedRequest = null;
                    }
                    return;
                }
                var idx = _this.bufferedRequests.indexOf(request);
                if (idx === -1) {
                    return;
                }
                _this.bufferedRequests.splice(idx, 1);
            });
        };
        Client.prototype.onMessage = function (response) {
            if (this.state === ServiceState.Uninitialized && response.type === ResponseType.Initialize) {
                this.state = ServiceState.Idle;
                this.bufferedRequests.forEach(function (r) { return r.flush && r.flush(); });
                this.bufferedRequests = null;
                return;
            }
            var handler = this.handlers[response.id];
            if (handler) {
                handler(response);
            }
        };
        Client.prototype.send = function (raw) {
            try {
                this.protocol.send(raw);
            }
            catch (err) {
            }
        };
        return Client;
    }());
    exports.Client = Client;
    /**
     * Useful when the service itself is needed right away but the client
     * is wrapped within a promise.
     */
    function getService(clientPromise, serviceName, serviceCtor) {
        var _servicePromise;
        var servicePromise = function () {
            if (!_servicePromise) {
                _servicePromise = clientPromise.then(function (client) { return client.getService(serviceName, serviceCtor); });
            }
            return _servicePromise;
        };
        return Object.keys(serviceCtor.prototype)
            .filter(function (key) { return key !== 'constructor'; })
            .reduce(function (result, key) {
            if (isServiceEvent(serviceCtor.prototype[key])) {
                var promise_2;
                var disposable_2;
                var emitter_2 = new event_1.Emitter({
                    onFirstListenerAdd: function () {
                        promise_2 = servicePromise().then(function (service) {
                            disposable_2 = service[key](function (e) { return emitter_2.fire(e); });
                        });
                    },
                    onLastListenerRemove: function () {
                        if (disposable_2) {
                            disposable_2.dispose();
                            disposable_2 = null;
                        }
                        promise_2.cancel();
                        promise_2 = null;
                    }
                });
                return objects_1.assign(result, (_a = {}, _a[key] = emitter_2.event, _a));
            }
            return objects_1.assign(result, (_b = {},
                _b[key] = function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i - 0] = arguments[_i];
                    }
                    return servicePromise().then(function (service) { return service[key].apply(service, args); });
                },
                _b
            ));
            var _a, _b;
        }, {});
    }
    exports.getService = getService;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/common/worker/simpleWorker", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base'], function (require, exports, errors_1, lifecycle_1, winjs_base_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var INITIALIZE = '$initialize';
    var SimpleWorkerProtocol = (function () {
        function SimpleWorkerProtocol(handler) {
            this._workerId = -1;
            this._handler = handler;
            this._lastSentReq = 0;
            this._pendingReplies = Object.create(null);
        }
        SimpleWorkerProtocol.prototype.setWorkerId = function (workerId) {
            this._workerId = workerId;
        };
        SimpleWorkerProtocol.prototype.sendMessage = function (method, args) {
            var req = String(++this._lastSentReq);
            var reply = {
                c: null,
                e: null
            };
            var result = new winjs_base_1.TPromise(function (c, e, p) {
                reply.c = c;
                reply.e = e;
            }, function () {
                // Cancel not supported
            });
            this._pendingReplies[req] = reply;
            this._send({
                vsWorker: this._workerId,
                req: req,
                method: method,
                args: args
            });
            return result;
        };
        SimpleWorkerProtocol.prototype.handleMessage = function (serializedMessage) {
            var message;
            try {
                message = JSON.parse(serializedMessage);
            }
            catch (e) {
            }
            if (!message.vsWorker) {
                return;
            }
            if (this._workerId !== -1 && message.vsWorker !== this._workerId) {
                return;
            }
            this._handleMessage(message);
        };
        SimpleWorkerProtocol.prototype._handleMessage = function (msg) {
            var _this = this;
            if (msg.seq) {
                var replyMessage = msg;
                if (!this._pendingReplies[replyMessage.seq]) {
                    console.warn('Got reply to unknown seq');
                    return;
                }
                var reply = this._pendingReplies[replyMessage.seq];
                delete this._pendingReplies[replyMessage.seq];
                if (replyMessage.err) {
                    var err = replyMessage.err;
                    if (replyMessage.err.$isError) {
                        err = new Error();
                        err.name = replyMessage.err.name;
                        err.message = replyMessage.err.message;
                        err.stack = replyMessage.err.stack;
                    }
                    reply.e(err);
                    return;
                }
                reply.c(replyMessage.res);
                return;
            }
            var requestMessage = msg;
            var req = requestMessage.req;
            var result = this._handler.handleMessage(requestMessage.method, requestMessage.args);
            result.then(function (r) {
                _this._send({
                    vsWorker: _this._workerId,
                    seq: req,
                    res: r,
                    err: undefined
                });
            }, function (e) {
                _this._send({
                    vsWorker: _this._workerId,
                    seq: req,
                    res: undefined,
                    err: errors_1.transformErrorForSerialization(e)
                });
            });
        };
        SimpleWorkerProtocol.prototype._send = function (msg) {
            var strMsg = JSON.stringify(msg);
            // console.log('SENDING: ' + strMsg);
            this._handler.sendMessage(strMsg);
        };
        return SimpleWorkerProtocol;
    }());
    /**
     * Main thread side
     */
    var SimpleWorkerClient = (function (_super) {
        __extends(SimpleWorkerClient, _super);
        function SimpleWorkerClient(workerFactory, moduleId, ctor) {
            var _this = this;
            _super.call(this);
            this._worker = this._register(workerFactory.create('vs/base/common/worker/simpleWorker', function (msg) {
                _this._protocol.handleMessage(msg);
            }));
            this._protocol = new SimpleWorkerProtocol({
                sendMessage: function (msg) {
                    _this._worker.postMessage(msg);
                },
                handleMessage: function (method, args) {
                    // Intentionally not supporting worker -> main requests
                    return winjs_base_1.TPromise.as(null);
                }
            });
            this._protocol.setWorkerId(this._worker.getId());
            // Gather loader configuration
            var loaderConfiguration = null;
            var globalRequire = window.require;
            if (typeof globalRequire.getConfig === 'function') {
                // Get the configuration from the Monaco AMD Loader
                loaderConfiguration = globalRequire.getConfig();
            }
            else if (typeof window.requirejs !== 'undefined') {
                // Get the configuration from requirejs
                loaderConfiguration = window.requirejs.s.contexts._.config;
            }
            // Send initialize message
            this._onModuleLoaded = this._protocol.sendMessage(INITIALIZE, [
                this._worker.getId(),
                moduleId,
                loaderConfiguration
            ]);
            this._onModuleLoaded.then(null, function (e) { return _this._onError('Worker failed to load ' + moduleId, e); });
            // Create proxy to loaded code
            var proxyMethodRequest = function (method, args) {
                return _this._request(method, args);
            };
            var createProxyMethod = function (method, proxyMethodRequest) {
                return function () {
                    var args = Array.prototype.slice.call(arguments, 0);
                    return proxyMethodRequest(method, args);
                };
            };
            this._proxy = {};
            for (var prop in ctor.prototype) {
                if (ctor.prototype.hasOwnProperty(prop)) {
                    if (typeof ctor.prototype[prop] === 'function') {
                        this._proxy[prop] = createProxyMethod(prop, proxyMethodRequest);
                    }
                }
            }
        }
        SimpleWorkerClient.prototype.get = function () {
            return this._proxy;
        };
        SimpleWorkerClient.prototype._request = function (method, args) {
            var _this = this;
            return new winjs_base_1.TPromise(function (c, e, p) {
                _this._onModuleLoaded.then(function () {
                    _this._protocol.sendMessage(method, args).then(c, e);
                }, e);
            }, function () {
                // Cancel intentionally not supported
            });
        };
        SimpleWorkerClient.prototype._onError = function (message, error) {
            console.error(message);
            console.info(error);
        };
        return SimpleWorkerClient;
    }(lifecycle_1.Disposable));
    exports.SimpleWorkerClient = SimpleWorkerClient;
    /**
     * Worker side
     */
    var SimpleWorkerServer = (function () {
        function SimpleWorkerServer(postSerializedMessage) {
            var _this = this;
            this._protocol = new SimpleWorkerProtocol({
                sendMessage: function (msg) {
                    postSerializedMessage(msg);
                },
                handleMessage: function (method, args) { return _this._handleMessage(method, args); }
            });
        }
        SimpleWorkerServer.prototype.onmessage = function (msg) {
            this._protocol.handleMessage(msg);
        };
        SimpleWorkerServer.prototype._handleMessage = function (method, args) {
            if (method === INITIALIZE) {
                return this.initialize(args[0], args[1], args[2]);
            }
            if (!this._requestHandler || typeof this._requestHandler[method] !== 'function') {
                return winjs_base_1.TPromise.wrapError(new Error('Missing requestHandler or method: ' + method));
            }
            try {
                return winjs_base_1.TPromise.as(this._requestHandler[method].apply(this._requestHandler, args));
            }
            catch (e) {
                return winjs_base_1.TPromise.wrapError(e);
            }
        };
        SimpleWorkerServer.prototype.initialize = function (workerId, moduleId, loaderConfig) {
            var _this = this;
            this._protocol.setWorkerId(workerId);
            // TODO@Alex: share this code with workerServer
            if (loaderConfig) {
                // Remove 'baseUrl', handling it is beyond scope for now
                if (typeof loaderConfig.baseUrl !== 'undefined') {
                    delete loaderConfig['baseUrl'];
                }
                if (typeof loaderConfig.paths !== 'undefined') {
                    if (typeof loaderConfig.paths.vs !== 'undefined') {
                        delete loaderConfig.paths['vs'];
                    }
                }
                var nlsConfig_1 = loaderConfig['vs/nls'];
                // We need to have pseudo translation
                if (nlsConfig_1 && nlsConfig_1.pseudo) {
                    require(['vs/nls'], function (nlsPlugin) {
                        nlsPlugin.setPseudoTranslation(nlsConfig_1.pseudo);
                    });
                }
                // Since this is in a web worker, enable catching errors
                loaderConfig.catchError = true;
                self.require.config(loaderConfig);
            }
            var cc;
            var ee;
            var r = new winjs_base_1.TPromise(function (c, e, p) {
                cc = c;
                ee = e;
            });
            require([moduleId], function () {
                var result = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    result[_i - 0] = arguments[_i];
                }
                var handlerModule = result[0];
                _this._requestHandler = handlerModule.create();
                cc(null);
            }, ee);
            return r;
        };
        return SimpleWorkerServer;
    }());
    exports.SimpleWorkerServer = SimpleWorkerServer;
    /**
     * Called on the worker side
     */
    function create(postMessage) {
        return new SimpleWorkerServer(postMessage);
    }
    exports.create = create;
});

define("vs/base/common/worker/workerProtocol", ["require", "exports", 'vs/base/common/winjs.base'], function (require, exports, winjs_base_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.MessageType = {
        INITIALIZE: '$initialize',
        REPLY: '$reply',
        PRINT: '$print'
    };
    exports.ReplyType = {
        COMPLETE: 'complete',
        ERROR: 'error',
        PROGRESS: 'progress'
    };
    exports.PrintType = {
        LOG: 'log',
        DEBUG: 'debug',
        INFO: 'info',
        WARN: 'warn',
        ERROR: 'error'
    };
    var RemoteCom = (function () {
        function RemoteCom(requester) {
            this._requester = requester;
            this._bigHandler = null;
        }
        RemoteCom.prototype.callOnRemote = function (proxyId, path, args) {
            return this._requester.request('_proxyObj', {
                proxyId: proxyId,
                path: path,
                args: args
            });
        };
        RemoteCom.prototype.setManyHandler = function (handler) {
            this._bigHandler = handler;
        };
        RemoteCom.prototype.handleMessage = function (msg) {
            if (!this._bigHandler) {
                throw new Error('got message before big handler attached!');
            }
            return this._invokeHandler(msg.proxyId, msg.path, msg.args);
        };
        RemoteCom.prototype._invokeHandler = function (rpcId, method, args) {
            try {
                return winjs_base_1.TPromise.as(this._bigHandler.handle(rpcId, method, args));
            }
            catch (err) {
                return winjs_base_1.TPromise.wrapError(err);
            }
        };
        return RemoteCom;
    }());
    exports.RemoteCom = RemoteCom;
});

define("vs/base/common/worker/workerClient", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/marshalling', 'vs/base/common/timer', 'vs/base/common/winjs.base', 'vs/base/common/worker/workerProtocol'], function (require, exports, errors_1, marshalling_1, timer, winjs_base_1, workerProtocol) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var WorkerClient = (function () {
        function WorkerClient(workerFactory, moduleId, decodeMessageName) {
            var _this = this;
            this._decodeMessageName = decodeMessageName;
            this._lastMessageId = 0;
            this._promises = {};
            this._messagesQueue = [];
            this._processQueueTimeout = -1;
            this._waitingForWorkerReply = false;
            this._lastTimerEvent = null;
            this._worker = workerFactory.create('vs/base/common/worker/workerServer', function (msg) { return _this._onSerializedMessage(msg); });
            var loaderConfiguration = null;
            var globalRequire = window.require;
            if (typeof globalRequire.getConfig === 'function') {
                // Get the configuration from the Monaco AMD Loader
                loaderConfiguration = globalRequire.getConfig();
            }
            else if (typeof window.requirejs !== 'undefined') {
                // Get the configuration from requirejs
                loaderConfiguration = window.requirejs.s.contexts._.config;
            }
            var MonacoEnvironment = window.MonacoEnvironment || null;
            this.onModuleLoaded = this._sendMessage(workerProtocol.MessageType.INITIALIZE, {
                id: this._worker.getId(),
                moduleId: moduleId,
                loaderConfiguration: loaderConfiguration,
                MonacoEnvironment: MonacoEnvironment
            });
            this.onModuleLoaded.then(null, function (e) { return _this._onError('Worker failed to load ' + moduleId, e); });
            this._remoteCom = new workerProtocol.RemoteCom(this);
        }
        WorkerClient.prototype.getRemoteCom = function () {
            return this._remoteCom;
        };
        WorkerClient.prototype.getQueueSize = function () {
            return this._messagesQueue.length + (this._waitingForWorkerReply ? 1 : 0);
        };
        WorkerClient.prototype.request = function (requestName, payload, forceTimestamp) {
            var _this = this;
            if (requestName.charAt(0) === '$') {
                throw new Error('Illegal requestName: ' + requestName);
            }
            var shouldCancelPromise = false, messagePromise;
            return new winjs_base_1.TPromise(function (c, e, p) {
                // hide the initialize promise inside this
                // promise so that it won't be canceled by accident
                _this.onModuleLoaded.then(function () {
                    if (!shouldCancelPromise) {
                        messagePromise = _this._sendMessage(requestName, payload, forceTimestamp).then(c, e, p);
                    }
                }, e, p);
            }, function () {
                // forward cancel to the proper promise
                if (messagePromise) {
                    messagePromise.cancel();
                }
                else {
                    shouldCancelPromise = true;
                }
            });
        };
        WorkerClient.prototype.destroy = function () {
            this.dispose();
        };
        WorkerClient.prototype.dispose = function () {
            var promises = Object.keys(this._promises);
            if (promises.length > 0) {
                console.warn('Terminating a worker with ' + promises.length + ' pending promises:');
                console.warn(this._promises);
                for (var id in this._promises) {
                    if (promises.hasOwnProperty(id)) {
                        this._promises[id].error('Worker forcefully terminated');
                    }
                }
            }
            this._worker.dispose();
        };
        WorkerClient.prototype._sendMessage = function (type, payload, forceTimestamp) {
            var _this = this;
            if (forceTimestamp === void 0) { forceTimestamp = (new Date()).getTime(); }
            var msg = {
                id: ++this._lastMessageId,
                type: type,
                timestamp: forceTimestamp,
                payload: payload
            };
            var pc, pe, pp;
            var promise = new winjs_base_1.TPromise(function (c, e, p) {
                pc = c;
                pe = e;
                pp = p;
            }, function () {
                _this._removeMessage(msg.id);
            });
            this._promises[msg.id] = {
                complete: pc,
                error: pe,
                progress: pp,
                type: type,
                payload: payload
            };
            this._enqueueMessage(msg);
            return promise;
        };
        WorkerClient.prototype._enqueueMessage = function (msg) {
            var lastIndexSmallerOrEqual = -1, i;
            // Find the right index to insert at - keep the queue ordered by timestamp
            for (i = this._messagesQueue.length - 1; i >= 0; i--) {
                if (this._messagesQueue[i].timestamp <= msg.timestamp) {
                    lastIndexSmallerOrEqual = i;
                    break;
                }
            }
            this._messagesQueue.splice(lastIndexSmallerOrEqual + 1, 0, msg);
            this._processMessagesQueue();
        };
        WorkerClient.prototype._removeMessage = function (msgId) {
            for (var i = 0, len = this._messagesQueue.length; i < len; i++) {
                if (this._messagesQueue[i].id === msgId) {
                    if (this._promises.hasOwnProperty(String(msgId))) {
                        delete this._promises[String(msgId)];
                    }
                    this._messagesQueue.splice(i, 1);
                    this._processMessagesQueue();
                    return;
                }
            }
        };
        WorkerClient.prototype._processMessagesQueue = function () {
            var _this = this;
            if (this._processQueueTimeout !== -1) {
                clearTimeout(this._processQueueTimeout);
                this._processQueueTimeout = -1;
            }
            if (this._messagesQueue.length === 0) {
                return;
            }
            if (this._waitingForWorkerReply) {
                return;
            }
            var delayUntilNextMessage = this._messagesQueue[0].timestamp - (new Date()).getTime();
            delayUntilNextMessage = Math.max(0, delayUntilNextMessage);
            this._processQueueTimeout = setTimeout(function () {
                _this._processQueueTimeout = -1;
                if (_this._messagesQueue.length === 0) {
                    return;
                }
                _this._waitingForWorkerReply = true;
                var msg = _this._messagesQueue.shift();
                _this._lastTimerEvent = timer.start(timer.Topic.WORKER, _this._decodeMessageName(msg));
                _this._postMessage(msg);
            }, delayUntilNextMessage);
        };
        WorkerClient.prototype._postMessage = function (msg) {
            this._worker.postMessage(marshalling_1.stringify(msg));
        };
        WorkerClient.prototype._onSerializedMessage = function (msg) {
            var message = null;
            try {
                message = marshalling_1.parse(msg);
            }
            catch (e) {
            }
            if (message) {
                this._onmessage(message);
            }
        };
        WorkerClient.prototype._onmessage = function (msg) {
            if (!msg.monacoWorker) {
                return;
            }
            if (msg.from && msg.from !== this._worker.getId()) {
                return;
            }
            switch (msg.type) {
                case workerProtocol.MessageType.REPLY:
                    var serverReplyMessage = msg;
                    this._waitingForWorkerReply = false;
                    if (this._lastTimerEvent) {
                        this._lastTimerEvent.stop();
                    }
                    if (!this._promises.hasOwnProperty(String(serverReplyMessage.id))) {
                        this._onError('Received unexpected message from Worker:', msg);
                        return;
                    }
                    switch (serverReplyMessage.action) {
                        case workerProtocol.ReplyType.COMPLETE:
                            this._promises[serverReplyMessage.id].complete(serverReplyMessage.payload);
                            delete this._promises[serverReplyMessage.id];
                            break;
                        case workerProtocol.ReplyType.ERROR:
                            this._onError('Main Thread sent to worker the following message:', {
                                type: this._promises[serverReplyMessage.id].type,
                                payload: this._promises[serverReplyMessage.id].payload
                            });
                            this._onError('And the worker replied with an error:', serverReplyMessage.payload);
                            errors_1.onUnexpectedError(serverReplyMessage.payload);
                            this._promises[serverReplyMessage.id].error(serverReplyMessage.payload);
                            delete this._promises[serverReplyMessage.id];
                            break;
                        case workerProtocol.ReplyType.PROGRESS:
                            this._promises[serverReplyMessage.id].progress(serverReplyMessage.payload);
                            break;
                    }
                    break;
                case workerProtocol.MessageType.PRINT:
                    var serverPrintMessage = msg;
                    this._consoleLog(serverPrintMessage.level, serverPrintMessage.payload);
                    break;
                default:
                    this._dispatchRequestFromWorker(msg);
            }
            this._processMessagesQueue();
        };
        WorkerClient.prototype._dispatchRequestFromWorker = function (msg) {
            var _this = this;
            this._handleWorkerRequest(msg).then(function (result) {
                var reply = {
                    id: 0,
                    type: workerProtocol.MessageType.REPLY,
                    timestamp: (new Date()).getTime(),
                    seq: msg.req,
                    payload: (result instanceof Error ? errors_1.transformErrorForSerialization(result) : result),
                    err: null
                };
                _this._postMessage(reply);
            }, function (err) {
                var reply = {
                    id: 0,
                    type: workerProtocol.MessageType.REPLY,
                    timestamp: (new Date()).getTime(),
                    seq: msg.req,
                    payload: null,
                    err: (err instanceof Error ? errors_1.transformErrorForSerialization(err) : err)
                };
                _this._postMessage(reply);
            });
        };
        WorkerClient.prototype._handleWorkerRequest = function (msg) {
            if (msg.type === '_proxyObj') {
                return this._remoteCom.handleMessage(msg.payload);
            }
            if (typeof this[msg.type] === 'function') {
                return this._invokeHandler(this[msg.type], this, msg.payload);
            }
            this._onError('Received unexpected message from Worker:', msg);
            return winjs_base_1.TPromise.wrapError(new Error('No handler found'));
        };
        WorkerClient.prototype._invokeHandler = function (handler, handlerCtx, payload) {
            try {
                return winjs_base_1.TPromise.as(handler.call(handlerCtx, payload));
            }
            catch (err) {
                return winjs_base_1.TPromise.wrapError(err);
            }
        };
        WorkerClient.prototype._consoleLog = function (level, payload) {
            switch (level) {
                case workerProtocol.PrintType.LOG:
                    console.log(payload);
                    break;
                case workerProtocol.PrintType.DEBUG:
                    console.info(payload);
                    break;
                case workerProtocol.PrintType.INFO:
                    console.info(payload);
                    break;
                case workerProtocol.PrintType.WARN:
                    console.warn(payload);
                    break;
                case workerProtocol.PrintType.ERROR:
                    console.error(payload);
                    break;
                default:
                    this._onError('Received unexpected message from Worker:', payload);
            }
        };
        WorkerClient.prototype._onError = function (message, error) {
            console.error(message);
            console.info(error);
        };
        return WorkerClient;
    }());
    exports.WorkerClient = WorkerClient;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/pfs", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/node/extfs', 'vs/base/common/paths', 'path', 'vs/base/common/async', 'fs'], function (require, exports, winjs_base_1, extfs, paths, path_1, async_1, fs) {
    'use strict';
    function isRoot(path) {
        return path === path_1.dirname(path);
    }
    exports.isRoot = isRoot;
    function readdir(path) {
        return async_1.nfcall(extfs.readdir, path);
    }
    exports.readdir = readdir;
    function exists(path) {
        return new winjs_base_1.Promise(function (c) { return fs.exists(path, c); });
    }
    exports.exists = exists;
    function chmod(path, mode) {
        return async_1.nfcall(fs.chmod, path, mode);
    }
    exports.chmod = chmod;
    function mkdirp(path, mode) {
        var mkdir = function () { return async_1.nfcall(fs.mkdir, path, mode)
            .then(null, function (err) {
            if (err.code === 'EEXIST') {
                return async_1.nfcall(fs.stat, path)
                    .then(function (stat) { return stat.isDirectory
                    ? null
                    : winjs_base_1.Promise.wrapError(new Error("'" + path + "' exists and is not a directory.")); });
            }
            return winjs_base_1.TPromise.wrapError(err);
        }); };
        if (isRoot(path)) {
            return winjs_base_1.TPromise.as(true);
        }
        return mkdir().then(null, function (err) {
            if (err.code === 'ENOENT') {
                return mkdirp(path_1.dirname(path), mode).then(mkdir);
            }
            return winjs_base_1.TPromise.wrapError(err);
        });
    }
    exports.mkdirp = mkdirp;
    function rimraf(path) {
        return stat(path).then(function (stat) {
            if (stat.isDirectory()) {
                return readdir(path)
                    .then(function (children) { return winjs_base_1.TPromise.join(children.map(function (child) { return rimraf(path_1.join(path, child)); })); })
                    .then(function () { return rmdir(path); });
            }
            else {
                return unlink(path);
            }
        }, function (err) {
            if (err.code === 'ENOENT') {
                return;
            }
            return winjs_base_1.TPromise.wrapError(err);
        });
    }
    exports.rimraf = rimraf;
    function realpath(path) {
        return async_1.nfcall(fs.realpath, path, null);
    }
    exports.realpath = realpath;
    function stat(path) {
        return async_1.nfcall(fs.stat, path);
    }
    exports.stat = stat;
    function lstat(path) {
        return async_1.nfcall(fs.lstat, path);
    }
    exports.lstat = lstat;
    function mstat(paths) {
        return doStatMultiple(paths.slice(0));
    }
    exports.mstat = mstat;
    function rename(oldPath, newPath) {
        return async_1.nfcall(fs.rename, oldPath, newPath);
    }
    exports.rename = rename;
    function rmdir(path) {
        return async_1.nfcall(fs.rmdir, path);
    }
    exports.rmdir = rmdir;
    function unlink(path) {
        return async_1.nfcall(fs.unlink, path);
    }
    exports.unlink = unlink;
    function symlink(target, path, type) {
        return async_1.nfcall(fs.symlink, target, path, type);
    }
    exports.symlink = symlink;
    function readlink(path) {
        return async_1.nfcall(fs.readlink, path);
    }
    exports.readlink = readlink;
    function doStatMultiple(paths) {
        var path = paths.shift();
        return stat(path).then(function (value) {
            return {
                path: path,
                stats: value
            };
        }, function (err) {
            if (paths.length === 0) {
                return err;
            }
            return mstat(paths);
        });
    }
    function readFile(path, encoding) {
        return async_1.nfcall(fs.readFile, path, encoding);
    }
    exports.readFile = readFile;
    function writeFile(path, data, encoding) {
        if (encoding === void 0) { encoding = 'utf8'; }
        return async_1.nfcall(fs.writeFile, path, data, encoding);
    }
    exports.writeFile = writeFile;
    /**
    * Read a dir and return only subfolders
    */
    function readDirsInDir(dirPath) {
        return readdir(dirPath).then(function (children) {
            return winjs_base_1.TPromise.join(children.map(function (child) { return dirExistsWithResult(paths.join(dirPath, child), child); })).then(function (subdirs) {
                return removeNull(subdirs);
            });
        });
    }
    exports.readDirsInDir = readDirsInDir;
    function dirExistsWithResult(path, successResult) {
        return dirExists(path).then(function (exists) {
            return exists ? successResult : null;
        });
    }
    /**
    * `path` exists and is a directory
    */
    function dirExists(path) {
        return stat(path).then(function (stat) { return stat.isDirectory(); }, function () { return false; });
    }
    exports.dirExists = dirExists;
    /**
    * `path` exists and is a file.
    */
    function fileExists(path) {
        return stat(path).then(function (stat) { return stat.isFile(); }, function () { return false; });
    }
    exports.fileExists = fileExists;
    /**
    * Read dir at `path` and return only files matching `pattern`
    */
    function readFiles(path, pattern) {
        return readdir(path).then(function (children) {
            children = children.filter(function (child) {
                return pattern.test(child);
            });
            var fileChildren = children.map(function (child) {
                return fileExistsWithResult(paths.join(path, child), child);
            });
            return winjs_base_1.TPromise.join(fileChildren).then(function (subdirs) {
                return removeNull(subdirs);
            });
        });
    }
    exports.readFiles = readFiles;
    function fileExistsWithResult(path, successResult) {
        return fileExists(path).then(function (exists) {
            return exists ? successResult : null;
        }, function (err) {
            return winjs_base_1.TPromise.wrapError(err);
        });
    }
    exports.fileExistsWithResult = fileExistsWithResult;
    function removeNull(arr) {
        return arr.filter(function (item) { return (item !== null); });
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/request", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/types', 'https', 'http', 'url', 'fs', 'vs/base/common/objects'], function (require, exports, winjs_base_1, types_1, https, http, url_1, fs_1, objects_1) {
    'use strict';
    function request(options) {
        var req;
        return new winjs_base_1.TPromise(function (c, e) {
            var endpoint = url_1.parse(options.url);
            var opts = {
                hostname: endpoint.hostname,
                port: endpoint.port ? parseInt(endpoint.port) : (endpoint.protocol === 'https:' ? 443 : 80),
                path: endpoint.path,
                method: options.type || 'GET',
                headers: options.headers,
                agent: options.agent,
                rejectUnauthorized: types_1.isBoolean(options.strictSSL) ? options.strictSSL : true
            };
            if (options.user && options.password) {
                opts.auth = options.user + ':' + options.password;
            }
            var protocol = endpoint.protocol === 'https:' ? https : http;
            req = protocol.request(opts, function (res) {
                if (res.statusCode >= 300 && res.statusCode < 400 && options.followRedirects && options.followRedirects > 0 && res.headers['location']) {
                    c(request(objects_1.assign({}, options, {
                        url: res.headers['location'],
                        followRedirects: options.followRedirects - 1
                    })));
                }
                else {
                    c({ req: req, res: res });
                }
            });
            req.on('error', e);
            if (options.timeout) {
                req.setTimeout(options.timeout);
            }
            if (options.data) {
                req.write(options.data);
            }
            req.end();
        }, function () { return req && req.abort(); });
    }
    exports.request = request;
    function download(filePath, opts) {
        return request(objects_1.assign(opts, { followRedirects: 3 })).then(function (pair) { return new winjs_base_1.TPromise(function (c, e) {
            var out = fs_1.createWriteStream(filePath);
            out.once('finish', function () { return c(null); });
            pair.res.once('error', e);
            pair.res.pipe(out);
        }); });
    }
    exports.download = download;
    function json(opts) {
        return request(opts).then(function (pair) { return new winjs_base_1.Promise(function (c, e) {
            if (!((pair.res.statusCode >= 200 && pair.res.statusCode < 300) || pair.res.statusCode === 1223)) {
                return e('Server returned ' + pair.res.statusCode);
            }
            if (pair.res.statusCode === 204) {
                return c(null);
            }
            if (!/application\/json/.test(pair.res.headers['content-type'])) {
                return e('Response doesn\'t appear to be JSON');
            }
            var buffer = [];
            pair.res.on('data', function (d) { return buffer.push(d); });
            pair.res.on('end', function () { return c(JSON.parse(buffer.join(''))); });
            pair.res.on('error', e);
        }); });
    }
    exports.json = json;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/node/service.cp", ["require", "exports", 'child_process', 'vs/base/common/winjs.base', 'vs/base/common/async', 'vs/base/common/objects', 'vs/base/common/service'], function (require, exports, cp, winjs_base_1, async_1, objects_1, service_1) {
    "use strict";
    var Server = (function (_super) {
        __extends(Server, _super);
        function Server() {
            var _this = this;
            _super.call(this, {
                send: function (r) { try {
                    process.send(r);
                }
                catch (e) { } },
                onMessage: function (cb) { return process.on('message', cb); }
            });
            process.once('disconnect', function () { return _this.dispose(); });
        }
        return Server;
    }(service_1.Server));
    exports.Server = Server;
    var Client = (function () {
        function Client(modulePath, options) {
            this.modulePath = modulePath;
            this.options = options;
            var timeout = options && options.timeout ? options.timeout : Number.MAX_VALUE;
            this.disposeDelayer = new async_1.Delayer(timeout);
            this.activeRequests = [];
            this.child = null;
            this._client = null;
            this.services = Object.create(null);
        }
        Client.prototype.getService = function (serviceName, serviceCtor) {
            var _this = this;
            return Object.keys(serviceCtor.prototype)
                .filter(function (key) { return key !== 'constructor'; })
                .reduce(function (service, key) { return objects_1.assign(service, (_a = {}, _a[key] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i - 0] = arguments[_i];
                }
                return _this.request.apply(_this, [serviceName, serviceCtor, key].concat(args));
            }, _a)); var _a; }, {});
        };
        Client.prototype.request = function (serviceName, serviceCtor, name) {
            var _this = this;
            var args = [];
            for (var _i = 3; _i < arguments.length; _i++) {
                args[_i - 3] = arguments[_i];
            }
            this.disposeDelayer.cancel();
            var service = this.services[serviceName];
            if (!service) {
                service = this.services[serviceName] = this.client.getService(serviceName, serviceCtor);
            }
            var request = service[name].apply(service, args);
            // Progress doesn't propagate across 'then', we need to create a promise wrapper
            var result = new winjs_base_1.Promise(function (c, e, p) {
                request.then(c, e, p).done(function () {
                    _this.activeRequests.splice(_this.activeRequests.indexOf(result), 1);
                    _this.disposeDelayer.trigger(function () { return _this.disposeClient(); });
                });
            }, function () { return request.cancel(); });
            this.activeRequests.push(result);
            return result;
        };
        Object.defineProperty(Client.prototype, "client", {
            get: function () {
                var _this = this;
                if (!this._client) {
                    var args = this.options && this.options.args ? this.options.args : [];
                    var forkOpts = undefined;
                    if (this.options) {
                        forkOpts = Object.create(null);
                        if (this.options.env) {
                            forkOpts.env = objects_1.assign(objects_1.clone(process.env), this.options.env);
                        }
                        if (typeof this.options.debug === 'number') {
                            forkOpts.execArgv = ['--nolazy', '--debug=' + this.options.debug];
                        }
                        if (typeof this.options.debugBrk === 'number') {
                            forkOpts.execArgv = ['--nolazy', '--debug-brk=' + this.options.debugBrk];
                        }
                    }
                    this.child = cp.fork(this.modulePath, args, forkOpts);
                    this._client = new service_1.Client({
                        send: function (r) { return _this.child && _this.child.connected && _this.child.send(r); },
                        onMessage: function (cb) {
                            _this.child.on('message', function (msg) {
                                // Handle console logs specially
                                if (msg && msg.type === '__$console') {
                                    var args_1 = ['%c[Service Library: ' + _this.options.serverName + ']', 'color: darkgreen'];
                                    try {
                                        var parsed_1 = JSON.parse(msg.arguments);
                                        args_1 = args_1.concat(Object.getOwnPropertyNames(parsed_1).map(function (o) { return parsed_1[o]; }));
                                    }
                                    catch (error) {
                                        args_1.push(msg.arguments);
                                    }
                                    console[msg.severity].apply(console, args_1);
                                }
                                else {
                                    cb(msg);
                                }
                            });
                        }
                    });
                    var onExit_1 = function () { return _this.disposeClient(); };
                    process.once('exit', onExit_1);
                    this.child.on('error', function (err) { return console.warn('Service "' + _this.options.serverName + '" errored with ' + err); });
                    this.child.on('exit', function (code, signal) {
                        process.removeListener('exit', onExit_1);
                        if (_this.activeRequests) {
                            _this.activeRequests.forEach(function (req) { return req.cancel(); });
                            _this.activeRequests = [];
                        }
                        if (code && signal !== 'SIGTERM') {
                            console.warn('Service "' + _this.options.serverName + '" crashed with exit code ' + code);
                            _this.disposeDelayer.cancel();
                            _this.disposeClient();
                        }
                    });
                }
                return this._client;
            },
            enumerable: true,
            configurable: true
        });
        Client.prototype.disposeClient = function () {
            if (this._client) {
                this.child.kill();
                this.child = null;
                this._client = null;
                this.services = Object.create(null);
            }
        };
        Client.prototype.dispose = function () {
            this.disposeDelayer.cancel();
            this.disposeDelayer = null;
            this.disposeClient();
            this.activeRequests = null;
        };
        return Client;
    }());
    exports.Client = Client;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/service.net", ["require", "exports", 'net', 'vs/base/common/event', 'vs/base/common/service', 'vs/base/common/winjs.base'], function (require, exports, net, event_1, service_1, winjs_base_1) {
    'use strict';
    function bufferIndexOf(buffer, value, start) {
        if (start === void 0) { start = 0; }
        while (start < buffer.length && buffer[start] !== value) {
            start++;
        }
        return start;
    }
    var Protocol = (function () {
        function Protocol(socket) {
            this.socket = socket;
            this.buffer = null;
        }
        Protocol.prototype.send = function (message) {
            this.socket.write(JSON.stringify(message));
            this.socket.write(Protocol.Boundary);
        };
        Protocol.prototype.onMessage = function (callback) {
            var _this = this;
            this.socket.on('data', function (data) {
                var lastIndex = 0;
                var index = 0;
                while ((index = bufferIndexOf(data, 0, lastIndex)) < data.length) {
                    var dataToParse = data.slice(lastIndex, index);
                    if (_this.buffer) {
                        callback(JSON.parse(Buffer.concat([_this.buffer, dataToParse]).toString('utf8')));
                        _this.buffer = null;
                    }
                    else {
                        callback(JSON.parse(dataToParse.toString('utf8')));
                    }
                    lastIndex = index + 1;
                }
                if (index - lastIndex > 0) {
                    var dataToBuffer = data.slice(lastIndex, index);
                    if (_this.buffer) {
                        _this.buffer = Buffer.concat([_this.buffer, dataToBuffer]);
                    }
                    else {
                        _this.buffer = dataToBuffer;
                    }
                }
            });
        };
        Protocol.Boundary = new Buffer([0]);
        return Protocol;
    }());
    var Server = (function () {
        function Server(server) {
            var _this = this;
            this.server = server;
            this.services = Object.create(null);
            this.server.on('connection', function (socket) {
                var ipcServer = new service_1.Server(new Protocol(socket));
                Object.keys(_this.services)
                    .forEach(function (name) { return ipcServer.registerService(name, _this.services[name]); });
                socket.once('close', function () { return ipcServer.dispose(); });
            });
        }
        Server.prototype.registerService = function (serviceName, service) {
            this.services[serviceName] = service;
        };
        Server.prototype.dispose = function () {
            this.services = null;
            this.server.close();
            this.server = null;
        };
        return Server;
    }());
    exports.Server = Server;
    var Client = (function () {
        function Client(socket) {
            var _this = this;
            this.socket = socket;
            this._onClose = new event_1.Emitter();
            this.ipcClient = new service_1.Client(new Protocol(socket));
            socket.once('close', function () { return _this._onClose.fire(); });
        }
        Object.defineProperty(Client.prototype, "onClose", {
            get: function () { return this._onClose.event; },
            enumerable: true,
            configurable: true
        });
        Client.prototype.getService = function (serviceName, serviceCtor) {
            return this.ipcClient.getService(serviceName, serviceCtor);
        };
        Client.prototype.dispose = function () {
            this.socket.end();
            this.socket = null;
            this.ipcClient = null;
        };
        return Client;
    }());
    exports.Client = Client;
    function serve(hook) {
        return new winjs_base_1.TPromise(function (c, e) {
            var server = net.createServer();
            server.on('error', e);
            server.listen(hook, function () {
                server.removeListener('error', e);
                c(new Server(server));
            });
        });
    }
    exports.serve = serve;
    function connect(hook) {
        return new winjs_base_1.TPromise(function (c, e) {
            var socket = net.createConnection(hook, function () {
                socket.removeListener('error', e);
                c(new Client(socket));
            });
            socket.once('error', e);
        });
    }
    exports.connect = connect;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/parts/quickopen/browser/quickOpenViewer", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/types'], function (require, exports, winjs_base_1, types_1) {
    "use strict";
    var DataSource = (function () {
        function DataSource(arg) {
            this.modelProvider = types_1.isFunction(arg.getModel) ? arg : { getModel: function () { return arg; } };
        }
        DataSource.prototype.getId = function (tree, element) {
            if (!element) {
                return null;
            }
            var model = this.modelProvider.getModel();
            return model === element ? '__root__' : model.dataSource.getId(element);
        };
        DataSource.prototype.hasChildren = function (tree, element) {
            var model = this.modelProvider.getModel();
            return model && model === element && model.entries.length > 0;
        };
        DataSource.prototype.getChildren = function (tree, element) {
            var model = this.modelProvider.getModel();
            return winjs_base_1.TPromise.as(model === element ? model.entries : []);
        };
        DataSource.prototype.getParent = function (tree, element) {
            return winjs_base_1.TPromise.as(null);
        };
        return DataSource;
    }());
    exports.DataSource = DataSource;
    var AccessibilityProvider = (function () {
        function AccessibilityProvider(modelProvider) {
            this.modelProvider = modelProvider;
        }
        AccessibilityProvider.prototype.getAriaLabel = function (tree, element) {
            var model = this.modelProvider.getModel();
            return model.accessibilityProvider && model.accessibilityProvider.getAriaLabel(element);
        };
        return AccessibilityProvider;
    }());
    exports.AccessibilityProvider = AccessibilityProvider;
    var Filter = (function () {
        function Filter(modelProvider) {
            this.modelProvider = modelProvider;
        }
        Filter.prototype.isVisible = function (tree, element) {
            var model = this.modelProvider.getModel();
            if (!model.filter) {
                return true;
            }
            return model.filter.isVisible(element);
        };
        return Filter;
    }());
    exports.Filter = Filter;
    var Renderer = (function () {
        function Renderer(modelProvider) {
            this.modelProvider = modelProvider;
        }
        Renderer.prototype.getHeight = function (tree, element) {
            var model = this.modelProvider.getModel();
            return model.renderer.getHeight(element);
        };
        Renderer.prototype.getTemplateId = function (tree, element) {
            var model = this.modelProvider.getModel();
            return model.renderer.getTemplateId(element);
        };
        Renderer.prototype.renderTemplate = function (tree, templateId, container) {
            var model = this.modelProvider.getModel();
            return model.renderer.renderTemplate(templateId, container);
        };
        Renderer.prototype.renderElement = function (tree, element, templateId, templateData) {
            var model = this.modelProvider.getModel();
            model.renderer.renderElement(element, templateId, templateData);
        };
        Renderer.prototype.disposeTemplate = function (tree, templateId, templateData) {
            var model = this.modelProvider.getModel();
            model.renderer.disposeTemplate(templateId, templateData);
        };
        return Renderer;
    }());
    exports.Renderer = Renderer;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/parts/tree/browser/treeModel", ["require", "exports", 'vs/base/common/assert', 'vs/base/common/errors', 'vs/base/common/lifecycle', 'vs/base/common/arrays', 'vs/base/common/eventEmitter', 'vs/base/common/winjs.base'], function (require, exports, Assert, errors_1, lifecycle_1, arrays, Events, WinJS) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LockData = (function (_super) {
        __extends(LockData, _super);
        function LockData(item) {
            _super.call(this);
            this._item = item;
        }
        Object.defineProperty(LockData.prototype, "item", {
            get: function () {
                return this._item;
            },
            enumerable: true,
            configurable: true
        });
        LockData.prototype.dispose = function () {
            this.emit('unlock');
            _super.prototype.dispose.call(this);
        };
        return LockData;
    }(Events.EventEmitter));
    exports.LockData = LockData;
    var Lock = (function () {
        function Lock() {
            this.locks = Object.create({});
        }
        Lock.prototype.isLocked = function (item) {
            return !!this.locks[item.id];
        };
        Lock.prototype.run = function (item, fn) {
            var _this = this;
            var lock = this.getLock(item);
            if (lock) {
                var unbindListener;
                return new WinJS.Promise(function (c, e) {
                    unbindListener = lock.addOneTimeListener('unlock', function () {
                        return _this.run(item, fn).then(c, e);
                    });
                }, function () { return unbindListener(); });
            }
            var result;
            return new WinJS.Promise(function (c, e) {
                if (item.isDisposed()) {
                    return e(new Error('Item is disposed.'));
                }
                var lock = _this.locks[item.id] = new LockData(item);
                result = fn().then(function (r) {
                    delete _this.locks[item.id];
                    lock.dispose();
                    return r;
                }).then(c, e);
                return result;
            }, function () { return result.cancel(); });
        };
        Lock.prototype.getLock = function (item) {
            var key;
            for (key in this.locks) {
                var lock = this.locks[key];
                if (item.intersects(lock.item)) {
                    return lock;
                }
            }
            return null;
        };
        return Lock;
    }());
    exports.Lock = Lock;
    var ItemRegistry = (function (_super) {
        __extends(ItemRegistry, _super);
        function ItemRegistry() {
            _super.call(this);
            this.items = {};
        }
        ItemRegistry.prototype.register = function (item) {
            Assert.ok(!this.isRegistered(item.id), 'item already registered: ' + item.id);
            this.items[item.id] = { item: item, disposable: this.addEmitter2(item) };
        };
        ItemRegistry.prototype.deregister = function (item) {
            Assert.ok(this.isRegistered(item.id), 'item not registered: ' + item.id);
            this.items[item.id].disposable.dispose();
            delete this.items[item.id];
        };
        ItemRegistry.prototype.isRegistered = function (id) {
            return this.items.hasOwnProperty(id);
        };
        ItemRegistry.prototype.getItem = function (id) {
            var result = this.items[id];
            return result ? result.item : null;
        };
        ItemRegistry.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this.items = null;
        };
        return ItemRegistry;
    }(Events.EventEmitter));
    exports.ItemRegistry = ItemRegistry;
    var Item = (function (_super) {
        __extends(Item, _super);
        function Item(id, registry, context, lock, element) {
            _super.call(this);
            this.registry = registry;
            this.context = context;
            this.lock = lock;
            this.element = element;
            this.id = id;
            this.registry.register(this);
            this.doesHaveChildren = this.context.dataSource.hasChildren(this.context.tree, this.element);
            this.needsChildrenRefresh = true;
            this.parent = null;
            this.previous = null;
            this.next = null;
            this.firstChild = null;
            this.lastChild = null;
            this.userContent = null;
            this.traits = {};
            this.depth = 0;
            this.expanded = false;
            this.emit('item:create', { item: this });
            this.visible = this._isVisible();
            this.height = this._getHeight();
            this._isDisposed = false;
        }
        Item.prototype.getElement = function () {
            return this.element;
        };
        Item.prototype.hasChildren = function () {
            return this.doesHaveChildren;
        };
        Item.prototype.getDepth = function () {
            return this.depth;
        };
        Item.prototype.isVisible = function () {
            return this.visible;
        };
        Item.prototype.setVisible = function (value) {
            this.visible = value;
        };
        Item.prototype.isExpanded = function () {
            return this.expanded;
        };
        /* protected */ Item.prototype._setExpanded = function (value) {
            this.expanded = value;
        };
        Item.prototype.reveal = function (relativeTop) {
            if (relativeTop === void 0) { relativeTop = null; }
            var eventData = { item: this, relativeTop: relativeTop };
            this.emit('item:reveal', eventData);
        };
        Item.prototype.expand = function () {
            var _this = this;
            if (this.isExpanded() || !this.doesHaveChildren || this.lock.isLocked(this)) {
                return WinJS.TPromise.as(false);
            }
            var result = this.lock.run(this, function () {
                var eventData = { item: _this };
                var result;
                _this.emit('item:expanding', eventData);
                if (_this.needsChildrenRefresh) {
                    result = _this.refreshChildren(false, true, true);
                }
                else {
                    result = WinJS.TPromise.as(null);
                }
                return result.then(function () {
                    _this._setExpanded(true);
                    _this.emit('item:expanded', eventData);
                    return true;
                });
            });
            return result.then(function (r) {
                if (_this.isDisposed()) {
                    return false;
                }
                // Auto expand single child folders
                if (_this.context.options.autoExpandSingleChildren && r && _this.firstChild !== null && _this.firstChild === _this.lastChild && _this.firstChild.isVisible()) {
                    return _this.firstChild.expand().then(function () { return true; });
                }
                return r;
            });
        };
        Item.prototype.collapse = function (recursive) {
            var _this = this;
            if (recursive === void 0) { recursive = false; }
            if (recursive) {
                var collapseChildrenPromise = WinJS.TPromise.as(null);
                this.forEachChild(function (child) {
                    collapseChildrenPromise = collapseChildrenPromise.then(function () { return child.collapse(true); });
                });
                return collapseChildrenPromise.then(function () {
                    return _this.collapse(false);
                });
            }
            else {
                if (!this.isExpanded() || this.lock.isLocked(this)) {
                    return WinJS.TPromise.as(false);
                }
                return this.lock.run(this, function () {
                    var eventData = { item: _this };
                    _this.emit('item:collapsing', eventData);
                    _this._setExpanded(false);
                    _this.emit('item:collapsed', eventData);
                    return WinJS.TPromise.as(true);
                });
            }
        };
        Item.prototype.addTrait = function (trait) {
            var eventData = { item: this, trait: trait };
            this.traits[trait] = true;
            this.emit('item:addTrait', eventData);
        };
        Item.prototype.removeTrait = function (trait) {
            var eventData = { item: this, trait: trait };
            delete this.traits[trait];
            this.emit('item:removeTrait', eventData);
        };
        Item.prototype.hasTrait = function (trait) {
            return this.traits[trait] || false;
        };
        Item.prototype.getAllTraits = function () {
            var result = [];
            var trait;
            for (trait in this.traits) {
                if (this.traits.hasOwnProperty(trait) && this.traits[trait]) {
                    result.push(trait);
                }
            }
            return result;
        };
        Item.prototype.getHeight = function () {
            return this.height;
        };
        Item.prototype.refreshChildren = function (recursive, safe, force) {
            var _this = this;
            if (safe === void 0) { safe = false; }
            if (force === void 0) { force = false; }
            if (!force && !this.isExpanded()) {
                this.needsChildrenRefresh = true;
                return WinJS.TPromise.as(this);
            }
            this.needsChildrenRefresh = false;
            var doRefresh = function () {
                var eventData = { item: _this, isNested: safe };
                _this.emit('item:childrenRefreshing', eventData);
                var childrenPromise;
                if (_this.doesHaveChildren) {
                    childrenPromise = _this.context.dataSource.getChildren(_this.context.tree, _this.element);
                }
                else {
                    childrenPromise = WinJS.TPromise.as([]);
                }
                var result = childrenPromise.then(function (elements) {
                    elements = !elements ? [] : elements.slice(0);
                    elements = _this.sort(elements);
                    var staleItems = {};
                    while (_this.firstChild !== null) {
                        staleItems[_this.firstChild.id] = _this.firstChild;
                        _this.removeChild(_this.firstChild);
                    }
                    for (var i = 0, len = elements.length; i < len; i++) {
                        var element = elements[i];
                        var id = _this.context.dataSource.getId(_this.context.tree, element);
                        var item = staleItems[id] || new Item(id, _this.registry, _this.context, _this.lock, element);
                        item.element = element;
                        if (recursive) {
                            item.needsChildrenRefresh = recursive;
                        }
                        delete staleItems[id];
                        _this.addChild(item);
                    }
                    for (var staleItemId in staleItems) {
                        if (staleItems.hasOwnProperty(staleItemId)) {
                            staleItems[staleItemId].dispose();
                        }
                    }
                    if (recursive) {
                        return WinJS.Promise.join(_this.mapEachChild(function (child) {
                            return child.doRefresh(recursive, true);
                        }));
                    }
                    else {
                        return WinJS.TPromise.as(null);
                    }
                });
                return result
                    .then(null, errors_1.onUnexpectedError)
                    .then(function () { return _this.emit('item:childrenRefreshed', eventData); });
            };
            return safe ? doRefresh() : this.lock.run(this, doRefresh);
        };
        Item.prototype.doRefresh = function (recursive, safe) {
            if (safe === void 0) { safe = false; }
            var eventData = { item: this };
            this.doesHaveChildren = this.context.dataSource.hasChildren(this.context.tree, this.element);
            this.height = this._getHeight();
            this.setVisible(this._isVisible());
            this.emit('item:refresh', eventData);
            return this.refreshChildren(recursive, safe);
        };
        Item.prototype.refresh = function (recursive) {
            return this.doRefresh(recursive);
        };
        Item.prototype.getNavigator = function () {
            return new TreeNavigator(this);
        };
        Item.prototype.intersects = function (other) {
            return this.isAncestorOf(other) || other.isAncestorOf(this);
        };
        Item.prototype.getHierarchy = function () {
            var result = [];
            var node = this;
            do {
                result.push(node);
                node = node.parent;
            } while (node);
            result.reverse();
            return result;
        };
        Item.prototype.isAncestorOf = function (item) {
            while (item) {
                if (item.id === this.id) {
                    return true;
                }
                item = item.parent;
            }
            return false;
        };
        Item.prototype.addChild = function (item, afterItem) {
            if (afterItem === void 0) { afterItem = this.lastChild; }
            var isEmpty = this.firstChild === null;
            var atHead = afterItem === null;
            var atTail = afterItem === this.lastChild;
            if (isEmpty) {
                this.firstChild = this.lastChild = item;
                item.next = item.previous = null;
            }
            else if (atHead) {
                this.firstChild.previous = item;
                item.next = this.firstChild;
                item.previous = null;
                this.firstChild = item;
            }
            else if (atTail) {
                this.lastChild.next = item;
                item.next = null;
                item.previous = this.lastChild;
                this.lastChild = item;
            }
            else {
                item.previous = afterItem;
                item.next = afterItem.next;
                afterItem.next.previous = item;
                afterItem.next = item;
            }
            item.parent = this;
            item.depth = this.depth + 1;
        };
        Item.prototype.removeChild = function (item) {
            var isFirstChild = this.firstChild === item;
            var isLastChild = this.lastChild === item;
            if (isFirstChild && isLastChild) {
                this.firstChild = this.lastChild = null;
            }
            else if (isFirstChild) {
                item.next.previous = null;
                this.firstChild = item.next;
            }
            else if (isLastChild) {
                item.previous.next = null;
                this.lastChild = item.previous;
            }
            else {
                item.next.previous = item.previous;
                item.previous.next = item.next;
            }
            item.parent = null;
            item.depth = null;
        };
        Item.prototype.forEachChild = function (fn) {
            var child = this.firstChild, next;
            while (child) {
                next = child.next;
                fn(child);
                child = next;
            }
        };
        Item.prototype.mapEachChild = function (fn) {
            var result = [];
            this.forEachChild(function (child) {
                result.push(fn(child));
            });
            return result;
        };
        Item.prototype.sort = function (elements) {
            var _this = this;
            if (this.context.sorter) {
                return elements.sort(function (element, otherElement) {
                    return _this.context.sorter.compare(_this.context.tree, element, otherElement);
                });
            }
            return elements;
        };
        /* protected */ Item.prototype._getHeight = function () {
            return this.context.renderer.getHeight(this.context.tree, this.element);
        };
        /* protected */ Item.prototype._isVisible = function () {
            return this.context.filter.isVisible(this.context.tree, this.element);
        };
        Item.prototype.isDisposed = function () {
            return this._isDisposed;
        };
        Item.prototype.dispose = function () {
            this.forEachChild(function (child) { return child.dispose(); });
            this.parent = null;
            this.previous = null;
            this.next = null;
            this.firstChild = null;
            this.lastChild = null;
            var eventData = { item: this };
            this.emit('item:dispose', eventData);
            this.registry.deregister(this);
            _super.prototype.dispose.call(this);
            this._isDisposed = true;
        };
        return Item;
    }(Events.EventEmitter));
    exports.Item = Item;
    var RootItem = (function (_super) {
        __extends(RootItem, _super);
        function RootItem(id, registry, context, lock, element) {
            _super.call(this, id, registry, context, lock, element);
        }
        RootItem.prototype.isVisible = function () {
            return false;
        };
        RootItem.prototype.setVisible = function (value) {
            // no-op
        };
        RootItem.prototype.isExpanded = function () {
            return true;
        };
        /* protected */ RootItem.prototype._setExpanded = function (value) {
            // no-op
        };
        RootItem.prototype.render = function () {
            // no-op
        };
        /* protected */ RootItem.prototype._getHeight = function () {
            return 0;
        };
        /* protected */ RootItem.prototype._isVisible = function () {
            return false;
        };
        return RootItem;
    }(Item));
    var TreeNavigator = (function () {
        function TreeNavigator(item, subTreeOnly) {
            if (subTreeOnly === void 0) { subTreeOnly = true; }
            this.item = item;
            this.start = subTreeOnly ? item : null;
        }
        TreeNavigator.lastDescendantOf = function (item) {
            if (!item) {
                return null;
            }
            else {
                if (!item.isVisible() || !item.isExpanded() || item.lastChild === null) {
                    return item;
                }
                else {
                    return TreeNavigator.lastDescendantOf(item.lastChild);
                }
            }
        };
        TreeNavigator.prototype.current = function () {
            return this.item || null;
        };
        TreeNavigator.prototype.next = function () {
            if (this.item) {
                do {
                    if ((this.item instanceof RootItem || (this.item.isVisible() && this.item.isExpanded())) && this.item.firstChild) {
                        this.item = this.item.firstChild;
                    }
                    else if (this.item === this.start) {
                        this.item = null;
                    }
                    else {
                        // select next brother, next uncle, next great-uncle, etc...
                        while (this.item && this.item !== this.start && !this.item.next) {
                            this.item = this.item.parent;
                        }
                        if (this.item === this.start) {
                            this.item = null;
                        }
                        this.item = !this.item ? null : this.item.next;
                    }
                } while (this.item && !this.item.isVisible());
            }
            return this.item || null;
        };
        TreeNavigator.prototype.previous = function () {
            if (this.item) {
                do {
                    var previous = TreeNavigator.lastDescendantOf(this.item.previous);
                    if (previous) {
                        this.item = previous;
                    }
                    else if (this.item.parent && this.item.parent !== this.start && this.item.parent.isVisible()) {
                        this.item = this.item.parent;
                    }
                    else {
                        this.item = null;
                    }
                } while (this.item && !this.item.isVisible());
            }
            return this.item || null;
        };
        TreeNavigator.prototype.parent = function () {
            if (this.item) {
                var parent = this.item.parent;
                if (parent && parent !== this.start && parent.isVisible()) {
                    this.item = parent;
                }
                else {
                    this.item = null;
                }
            }
            return this.item || null;
        };
        TreeNavigator.prototype.first = function () {
            this.item = this.start;
            this.next();
            return this.item || null;
        };
        TreeNavigator.prototype.last = function () {
            if (this.start && this.start.isExpanded()) {
                this.item = this.start.lastChild;
                if (this.item && !this.item.isVisible()) {
                    this.previous();
                }
            }
            return this.item || null;
        };
        return TreeNavigator;
    }());
    exports.TreeNavigator = TreeNavigator;
    function getRange(one, other) {
        var oneHierarchy = one.getHierarchy();
        var otherHierarchy = other.getHierarchy();
        var length = arrays.commonPrefixLength(oneHierarchy, otherHierarchy);
        var item = oneHierarchy[length - 1];
        var nav = item.getNavigator();
        var oneIndex = null;
        var otherIndex = null;
        var index = 0;
        var result = [];
        while (item && (oneIndex === null || otherIndex === null)) {
            result.push(item);
            if (item === one) {
                oneIndex = index;
            }
            if (item === other) {
                otherIndex = index;
            }
            index++;
            item = nav.next();
        }
        if (oneIndex === null || otherIndex === null) {
            return [];
        }
        var min = Math.min(oneIndex, otherIndex);
        var max = Math.max(oneIndex, otherIndex);
        return result.slice(min, max + 1);
    }
    var TreeModel = (function (_super) {
        __extends(TreeModel, _super);
        function TreeModel(context) {
            _super.call(this);
            this.context = context;
            this.input = null;
            this.traitsToItems = {};
        }
        TreeModel.prototype.setInput = function (element) {
            var _this = this;
            var eventData = { item: this.input };
            this.emit('clearingInput', eventData);
            this.setSelection([]);
            this.setFocus();
            this.setHighlight();
            this.lock = new Lock();
            if (this.input) {
                this.input.dispose();
            }
            if (this.registry) {
                this.registry.dispose();
                this.registryDisposable.dispose();
            }
            this.registry = new ItemRegistry();
            this.registryDisposable = lifecycle_1.combinedDispose(this.addEmitter2(this.registry), this.registry.addListener2('item:dispose', function (event) {
                event.item.getAllTraits()
                    .forEach(function (trait) { return delete _this.traitsToItems[trait][event.item.id]; });
            }));
            var id = this.context.dataSource.getId(this.context.tree, element);
            this.input = new RootItem(id, this.registry, this.context, this.lock, element);
            eventData = { item: this.input };
            this.emit('setInput', eventData);
            return this.refresh(this.input);
        };
        TreeModel.prototype.getInput = function () {
            return this.input ? this.input.getElement() : null;
        };
        TreeModel.prototype.refresh = function (element, recursive) {
            var _this = this;
            if (element === void 0) { element = null; }
            if (recursive === void 0) { recursive = true; }
            var item = this.getItem(element);
            if (!item) {
                return WinJS.TPromise.as(null);
            }
            var eventData = { item: item, recursive: recursive };
            this.emit('refreshing', eventData);
            return item.refresh(recursive).then(function () {
                _this.emit('refreshed', eventData);
            });
        };
        TreeModel.prototype.refreshAll = function (elements, recursive) {
            var _this = this;
            if (recursive === void 0) { recursive = true; }
            var promises = [];
            this.deferredEmit(function () {
                for (var i = 0, len = elements.length; i < len; i++) {
                    promises.push(_this.refresh(elements[i], recursive));
                }
            });
            return WinJS.Promise.join(promises);
        };
        TreeModel.prototype.expand = function (element) {
            var item = this.getItem(element);
            if (!item) {
                return WinJS.TPromise.as(false);
            }
            return item.expand();
        };
        TreeModel.prototype.expandAll = function (elements) {
            if (!elements) {
                elements = [];
                var item;
                var nav = this.getNavigator();
                while (item = nav.next()) {
                    elements.push(item);
                }
            }
            var promises = [];
            for (var i = 0, len = elements.length; i < len; i++) {
                promises.push(this.expand(elements[i]));
            }
            return WinJS.Promise.join(promises);
        };
        TreeModel.prototype.collapse = function (element, recursive) {
            if (recursive === void 0) { recursive = false; }
            var item = this.getItem(element);
            if (!item) {
                return WinJS.TPromise.as(false);
            }
            return item.collapse(recursive);
        };
        TreeModel.prototype.collapseAll = function (elements, recursive) {
            if (elements === void 0) { elements = null; }
            if (recursive === void 0) { recursive = false; }
            if (!elements) {
                elements = [this.input];
                recursive = true;
            }
            var promises = [];
            for (var i = 0, len = elements.length; i < len; i++) {
                promises.push(this.collapse(elements[i], recursive));
            }
            return WinJS.Promise.join(promises);
        };
        TreeModel.prototype.toggleExpansion = function (element) {
            return this.isExpanded(element) ? this.collapse(element) : this.expand(element);
        };
        TreeModel.prototype.toggleExpansionAll = function (elements) {
            var promises = [];
            for (var i = 0, len = elements.length; i < len; i++) {
                promises.push(this.toggleExpansion(elements[i]));
            }
            return WinJS.Promise.join(promises);
        };
        TreeModel.prototype.isExpanded = function (element) {
            var item = this.getItem(element);
            if (!item) {
                return false;
            }
            return item.isExpanded();
        };
        TreeModel.prototype.getExpandedElements = function () {
            var result = [];
            var item;
            var nav = this.getNavigator();
            while (item = nav.next()) {
                if (item.isExpanded()) {
                    result.push(item.getElement());
                }
            }
            return result;
        };
        TreeModel.prototype.reveal = function (element, relativeTop) {
            var _this = this;
            if (relativeTop === void 0) { relativeTop = null; }
            return this.resolveUnknownParentChain(element).then(function (chain) {
                var result = WinJS.TPromise.as(null);
                chain.forEach(function (e) {
                    result = result.then(function () { return _this.expand(e); });
                });
                return result;
            }).then(function () {
                var item = _this.getItem(element);
                if (item) {
                    return item.reveal(relativeTop);
                }
            });
        };
        TreeModel.prototype.resolveUnknownParentChain = function (element) {
            var _this = this;
            return this.context.dataSource.getParent(this.context.tree, element).then(function (parent) {
                if (!parent) {
                    return WinJS.TPromise.as([]);
                }
                return _this.resolveUnknownParentChain(parent).then(function (result) {
                    result.push(parent);
                    return result;
                });
            });
        };
        TreeModel.prototype.setHighlight = function (element, eventPayload) {
            this.setTraits('highlighted', element ? [element] : []);
            var eventData = { highlight: this.getHighlight(), payload: eventPayload };
            this.emit('highlight', eventData);
        };
        TreeModel.prototype.getHighlight = function (includeHidden) {
            var result = this.getElementsWithTrait('highlighted', includeHidden);
            return result.length === 0 ? null : result[0];
        };
        TreeModel.prototype.isHighlighted = function (element) {
            var item = this.getItem(element);
            if (!item) {
                return false;
            }
            return item.hasTrait('highlighted');
        };
        TreeModel.prototype.select = function (element, eventPayload) {
            this.selectAll([element], eventPayload);
        };
        TreeModel.prototype.selectRange = function (fromElement, toElement, eventPayload) {
            var fromItem = this.getItem(fromElement);
            var toItem = this.getItem(toElement);
            if (!fromItem || !toItem) {
                return;
            }
            this.selectAll(getRange(fromItem, toItem), eventPayload);
        };
        TreeModel.prototype.deselectRange = function (fromElement, toElement, eventPayload) {
            var fromItem = this.getItem(fromElement);
            var toItem = this.getItem(toElement);
            if (!fromItem || !toItem) {
                return;
            }
            this.deselectAll(getRange(fromItem, toItem), eventPayload);
        };
        TreeModel.prototype.selectAll = function (elements, eventPayload) {
            this.addTraits('selected', elements);
            var eventData = { selection: this.getSelection(), payload: eventPayload };
            this.emit('selection', eventData);
        };
        TreeModel.prototype.deselect = function (element, eventPayload) {
            this.deselectAll([element], eventPayload);
        };
        TreeModel.prototype.deselectAll = function (elements, eventPayload) {
            this.removeTraits('selected', elements);
            var eventData = { selection: this.getSelection(), payload: eventPayload };
            this.emit('selection', eventData);
        };
        TreeModel.prototype.setSelection = function (elements, eventPayload) {
            this.setTraits('selected', elements);
            var eventData = { selection: this.getSelection(), payload: eventPayload };
            this.emit('selection', eventData);
        };
        TreeModel.prototype.toggleSelection = function (element, eventPayload) {
            this.toggleTrait('selected', element);
            var eventData = { selection: this.getSelection(), payload: eventPayload };
            this.emit('selection', eventData);
        };
        TreeModel.prototype.isSelected = function (element) {
            var item = this.getItem(element);
            if (!item) {
                return false;
            }
            return item.hasTrait('selected');
        };
        TreeModel.prototype.getSelection = function (includeHidden) {
            return this.getElementsWithTrait('selected', includeHidden);
        };
        TreeModel.prototype.selectNext = function (count, clearSelection, eventPayload) {
            if (count === void 0) { count = 1; }
            if (clearSelection === void 0) { clearSelection = true; }
            var selection = this.getSelection();
            var item = selection.length > 0 ? selection[0] : this.input;
            var nextItem;
            var nav = this.getNavigator(item, false);
            for (var i = 0; i < count; i++) {
                nextItem = nav.next();
                if (!nextItem) {
                    break;
                }
                item = nextItem;
            }
            if (clearSelection) {
                this.setSelection([item], eventPayload);
            }
            else {
                this.select(item, eventPayload);
            }
        };
        TreeModel.prototype.selectPrevious = function (count, clearSelection, eventPayload) {
            if (count === void 0) { count = 1; }
            if (clearSelection === void 0) { clearSelection = true; }
            var selection = this.getSelection(), item = null, previousItem = null;
            if (selection.length === 0) {
                var nav = this.getNavigator(this.input);
                while (item = nav.next()) {
                    previousItem = item;
                }
                item = previousItem;
            }
            else {
                item = selection[0];
                var nav = this.getNavigator(item, false);
                for (var i = 0; i < count; i++) {
                    previousItem = nav.previous();
                    if (!previousItem) {
                        break;
                    }
                    item = previousItem;
                }
            }
            if (clearSelection) {
                this.setSelection([item], eventPayload);
            }
            else {
                this.select(item, eventPayload);
            }
        };
        TreeModel.prototype.selectParent = function (eventPayload, clearSelection) {
            if (clearSelection === void 0) { clearSelection = true; }
            var selection = this.getSelection();
            var item = selection.length > 0 ? selection[0] : this.input;
            var nav = this.getNavigator(item, false);
            var parent = nav.parent();
            if (parent) {
                if (clearSelection) {
                    this.setSelection([parent], eventPayload);
                }
                else {
                    this.select(parent, eventPayload);
                }
            }
        };
        TreeModel.prototype.setFocus = function (element, eventPayload) {
            this.setTraits('focused', element ? [element] : []);
            var eventData = { focus: this.getFocus(), payload: eventPayload };
            this.emit('focus', eventData);
        };
        TreeModel.prototype.isFocused = function (element) {
            var item = this.getItem(element);
            if (!item) {
                return false;
            }
            return item.hasTrait('focused');
        };
        TreeModel.prototype.getFocus = function (includeHidden) {
            var result = this.getElementsWithTrait('focused', includeHidden);
            return result.length === 0 ? null : result[0];
        };
        TreeModel.prototype.focusNext = function (count, eventPayload) {
            if (count === void 0) { count = 1; }
            var item = this.getFocus() || this.input;
            var nextItem;
            var nav = this.getNavigator(item, false);
            for (var i = 0; i < count; i++) {
                nextItem = nav.next();
                if (!nextItem) {
                    break;
                }
                item = nextItem;
            }
            this.setFocus(item, eventPayload);
        };
        TreeModel.prototype.focusPrevious = function (count, eventPayload) {
            if (count === void 0) { count = 1; }
            var item = this.getFocus() || this.input;
            var previousItem;
            var nav = this.getNavigator(item, false);
            for (var i = 0; i < count; i++) {
                previousItem = nav.previous();
                if (!previousItem) {
                    break;
                }
                item = previousItem;
            }
            this.setFocus(item, eventPayload);
        };
        TreeModel.prototype.focusParent = function (eventPayload) {
            var item = this.getFocus() || this.input;
            var nav = this.getNavigator(item, false);
            var parent = nav.parent();
            if (parent) {
                this.setFocus(parent, eventPayload);
            }
        };
        TreeModel.prototype.focusFirst = function (eventPayload) {
            this.focusNth(0, eventPayload);
        };
        TreeModel.prototype.focusNth = function (index, eventPayload) {
            var nav = this.getNavigator(this.input);
            var item = nav.first();
            for (var i = 0; i < index; i++) {
                item = nav.next();
            }
            if (item) {
                this.setFocus(item, eventPayload);
            }
        };
        TreeModel.prototype.focusLast = function (eventPayload) {
            var nav = this.getNavigator(this.input);
            var item = nav.last();
            if (item) {
                this.setFocus(item, eventPayload);
            }
        };
        TreeModel.prototype.getNavigator = function (element, subTreeOnly) {
            if (element === void 0) { element = null; }
            if (subTreeOnly === void 0) { subTreeOnly = true; }
            return new TreeNavigator(this.getItem(element), subTreeOnly);
        };
        TreeModel.prototype.getItem = function (element) {
            if (element === void 0) { element = null; }
            if (element === null) {
                return this.input;
            }
            else if (element instanceof Item) {
                return element;
            }
            else if (typeof element === 'string') {
                return this.registry.getItem(element);
            }
            else {
                return this.registry.getItem(this.context.dataSource.getId(this.context.tree, element));
            }
        };
        TreeModel.prototype.addTraits = function (trait, elements) {
            var items = this.traitsToItems[trait] || {};
            var item;
            for (var i = 0, len = elements.length; i < len; i++) {
                item = this.getItem(elements[i]);
                if (item) {
                    item.addTrait(trait);
                    items[item.id] = item;
                }
            }
            this.traitsToItems[trait] = items;
        };
        TreeModel.prototype.removeTraits = function (trait, elements) {
            var items = this.traitsToItems[trait] || {};
            var item;
            var id;
            if (elements.length === 0) {
                for (id in items) {
                    if (items.hasOwnProperty(id)) {
                        item = items[id];
                        item.removeTrait(trait);
                    }
                }
                delete this.traitsToItems[trait];
            }
            else {
                for (var i = 0, len = elements.length; i < len; i++) {
                    item = this.getItem(elements[i]);
                    if (item) {
                        item.removeTrait(trait);
                        delete items[item.id];
                    }
                }
            }
        };
        TreeModel.prototype.hasTrait = function (trait, element) {
            var item = this.getItem(element);
            return item && item.hasTrait(trait);
        };
        TreeModel.prototype.toggleTrait = function (trait, element) {
            var item = this.getItem(element);
            if (!item) {
                return;
            }
            if (item.hasTrait(trait)) {
                this.removeTraits(trait, [element]);
            }
            else {
                this.addTraits(trait, [element]);
            }
        };
        TreeModel.prototype.setTraits = function (trait, elements) {
            if (elements.length === 0) {
                this.removeTraits(trait, elements);
            }
            else {
                var items = {};
                var item;
                for (var i = 0, len = elements.length; i < len; i++) {
                    item = this.getItem(elements[i]);
                    if (item) {
                        items[item.id] = item;
                    }
                }
                var traitItems = this.traitsToItems[trait] || {};
                var itemsToRemoveTrait = [];
                var id;
                for (id in traitItems) {
                    if (traitItems.hasOwnProperty(id)) {
                        if (items.hasOwnProperty(id)) {
                            delete items[id];
                        }
                        else {
                            itemsToRemoveTrait.push(traitItems[id]);
                        }
                    }
                }
                for (var i = 0, len = itemsToRemoveTrait.length; i < len; i++) {
                    item = itemsToRemoveTrait[i];
                    item.removeTrait(trait);
                    delete traitItems[item.id];
                }
                for (id in items) {
                    if (items.hasOwnProperty(id)) {
                        item = items[id];
                        item.addTrait(trait);
                        traitItems[id] = item;
                    }
                }
                this.traitsToItems[trait] = traitItems;
            }
        };
        TreeModel.prototype.getElementsWithTrait = function (trait, includeHidden) {
            var elements = [];
            var items = this.traitsToItems[trait] || {};
            var id;
            for (id in items) {
                if (items.hasOwnProperty(id) && (items[id].isVisible() || includeHidden)) {
                    elements.push(items[id].getElement());
                }
            }
            return elements;
        };
        TreeModel.prototype.dispose = function () {
            if (this.registry) {
                this.registry.dispose();
                this.registry = null;
            }
            _super.prototype.dispose.call(this);
        };
        return TreeModel;
    }(Events.EventEmitter));
    exports.TreeModel = TreeModel;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/parts/tree/browser/treeViewModel", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/base/common/iterator'], function (require, exports, eventEmitter_1, iterator_1) {
    "use strict";
    var HeightMap = (function (_super) {
        __extends(HeightMap, _super);
        function HeightMap() {
            _super.call(this);
            this.heightMap = [];
            this.indexes = {};
        }
        HeightMap.prototype.getTotalHeight = function () {
            var last = this.heightMap[this.heightMap.length - 1];
            return !last ? 0 : last.top + last.height;
        };
        HeightMap.prototype.onInsertItems = function (iterator, afterItemId) {
            if (afterItemId === void 0) { afterItemId = null; }
            var item;
            var viewItem;
            var i, j;
            var totalSize;
            var sizeDiff = 0;
            if (afterItemId === null) {
                i = 0;
                totalSize = 0;
            }
            else {
                i = this.indexes[afterItemId] + 1;
                viewItem = this.heightMap[i - 1];
                if (!viewItem) {
                    throw new Error('Tree error, onInsertItems: viewItem doesn\'t exist.');
                }
                totalSize = viewItem.top + viewItem.height;
            }
            var boundSplice = this.heightMap.splice.bind(this.heightMap, i, 0);
            var itemsToInsert = [];
            while (item = iterator.next()) {
                viewItem = this.createViewItem(item);
                viewItem.top = totalSize + sizeDiff;
                this.emit('viewItem:create', { item: viewItem.model });
                this.indexes[item.id] = i++;
                itemsToInsert.push(viewItem);
                sizeDiff += viewItem.height;
            }
            boundSplice.apply(this.heightMap, itemsToInsert);
            for (j = i; j < this.heightMap.length; j++) {
                viewItem = this.heightMap[j];
                viewItem.top += sizeDiff;
                this.indexes[viewItem.model.id] = j;
            }
            for (j = itemsToInsert.length - 1; j >= 0; j--) {
                this.onInsertItem(itemsToInsert[j]);
            }
            for (j = this.heightMap.length - 1; j >= i; j--) {
                this.onRefreshItem(this.heightMap[j]);
            }
            return sizeDiff;
        };
        HeightMap.prototype.onInsertItem = function (item) {
            // noop
        };
        // Contiguous items
        HeightMap.prototype.onRemoveItems = function (iterator) {
            var itemId;
            var viewItem;
            var startIndex = null;
            var i;
            var sizeDiff = 0;
            while (itemId = iterator.next()) {
                i = this.indexes[itemId];
                viewItem = this.heightMap[i];
                if (!viewItem) {
                    throw new Error('Tree error, onRemoveItems: viewItem doesn\'t exist.');
                }
                sizeDiff -= viewItem.height;
                delete this.indexes[itemId];
                this.onRemoveItem(viewItem);
                if (startIndex === null) {
                    startIndex = i;
                }
            }
            if (sizeDiff === 0) {
                return;
            }
            this.heightMap.splice(startIndex, i - startIndex + 1);
            for (i = startIndex; i < this.heightMap.length; i++) {
                viewItem = this.heightMap[i];
                viewItem.top += sizeDiff;
                this.indexes[viewItem.model.id] = i;
                this.onRefreshItem(viewItem);
            }
        };
        HeightMap.prototype.onRemoveItem = function (item) {
            // noop
        };
        HeightMap.prototype.onRefreshItemSet = function (items) {
            var _this = this;
            var sortedItems = items.sort(function (a, b) { return _this.indexes[a.id] - _this.indexes[b.id]; });
            this.onRefreshItems(new iterator_1.ArrayIterator(sortedItems));
        };
        // Ordered, but not necessarily contiguous items
        HeightMap.prototype.onRefreshItems = function (iterator) {
            var item;
            var viewItem;
            var newHeight;
            var i, j = null;
            var cummDiff = 0;
            while (item = iterator.next()) {
                i = this.indexes[item.id];
                for (; cummDiff !== 0 && j !== null && j < i; j++) {
                    viewItem = this.heightMap[j];
                    viewItem.top += cummDiff;
                    this.onRefreshItem(viewItem);
                }
                viewItem = this.heightMap[i];
                newHeight = item.getHeight();
                viewItem.top += cummDiff;
                cummDiff += newHeight - viewItem.height;
                viewItem.height = newHeight;
                this.onRefreshItem(viewItem, true);
                j = i + 1;
            }
            if (cummDiff !== 0 && j !== null) {
                for (; j < this.heightMap.length; j++) {
                    viewItem = this.heightMap[j];
                    viewItem.top += cummDiff;
                    this.onRefreshItem(viewItem);
                }
            }
        };
        HeightMap.prototype.onRefreshItem = function (item, needsRender) {
            if (needsRender === void 0) { needsRender = false; }
            // noop
        };
        HeightMap.prototype.itemsCount = function () {
            return this.heightMap.length;
        };
        HeightMap.prototype.itemAt = function (position) {
            return this.heightMap[this.indexAt(position)].model.id;
        };
        HeightMap.prototype.withItemsInRange = function (start, end, fn) {
            start = this.indexAt(start);
            end = this.indexAt(end);
            for (var i = start; i <= end; i++) {
                fn(this.heightMap[i].model.id);
            }
        };
        HeightMap.prototype.indexAt = function (position) {
            var left = 0;
            var right = this.heightMap.length;
            var center;
            var item;
            // Binary search
            while (left < right) {
                center = Math.floor((left + right) / 2);
                item = this.heightMap[center];
                if (position < item.top) {
                    right = center;
                }
                else if (position >= item.top + item.height) {
                    if (left === center) {
                        break;
                    }
                    left = center;
                }
                else {
                    return center;
                }
            }
            return this.heightMap.length;
        };
        HeightMap.prototype.indexAfter = function (position) {
            return Math.min(this.indexAt(position) + 1, this.heightMap.length);
        };
        HeightMap.prototype.itemAtIndex = function (index) {
            return this.heightMap[index];
        };
        HeightMap.prototype.itemAfter = function (item) {
            return this.heightMap[this.indexes[item.model.id] + 1] || null;
        };
        HeightMap.prototype.createViewItem = function (item) {
            throw new Error('not implemented');
        };
        HeightMap.prototype.dispose = function () {
            this.heightMap = null;
            this.indexes = null;
        };
        return HeightMap;
    }(eventEmitter_1.EventEmitter));
    exports.HeightMap = HeightMap;
});

define("vs/editor/browser/view/viewEventDispatcher", ["require", "exports", 'vs/base/common/eventEmitter'], function (require, exports, eventEmitter_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewEventDispatcher = (function () {
        function ViewEventDispatcher(eventHandlerGateKeeper) {
            this.eventHandlerGateKeeper = eventHandlerGateKeeper;
            this.eventHandlers = [];
            this.eventQueue = [];
            this.isConsumingQueue = false;
        }
        ViewEventDispatcher.prototype.addEventHandler = function (eventHandler) {
            for (var i = 0, len = this.eventHandlers.length; i < len; i++) {
                if (this.eventHandlers[i] === eventHandler) {
                    console.warn('Detected duplicate listener in ViewEventDispatcher', eventHandler);
                }
            }
            this.eventHandlers.push(eventHandler);
        };
        ViewEventDispatcher.prototype.removeEventHandler = function (eventHandler) {
            for (var i = 0; i < this.eventHandlers.length; i++) {
                if (this.eventHandlers[i] === eventHandler) {
                    this.eventHandlers.splice(i, 1);
                    break;
                }
            }
        };
        ViewEventDispatcher.prototype.emit = function (eventType, data) {
            this.eventQueue.push(new eventEmitter_1.EmitterEvent(eventType, data));
            if (!this.isConsumingQueue) {
                this.consumeQueue();
            }
        };
        ViewEventDispatcher.prototype.emitMany = function (events) {
            this.eventQueue = this.eventQueue.concat(events);
            if (!this.isConsumingQueue) {
                this.consumeQueue();
            }
        };
        ViewEventDispatcher.prototype.consumeQueue = function () {
            var _this = this;
            this.eventHandlerGateKeeper(function () {
                try {
                    _this.isConsumingQueue = true;
                    var i, len, eventHandlers, events;
                    while (_this.eventQueue.length > 0) {
                        // Empty event queue, as events might come in while sending these off
                        events = _this.eventQueue;
                        _this.eventQueue = [];
                        // Use a clone of the event handlers list, as they might remove themselves
                        eventHandlers = _this.eventHandlers.slice(0);
                        for (i = 0, len = eventHandlers.length; i < len; i++) {
                            eventHandlers[i].handleEvents(events);
                        }
                    }
                }
                finally {
                    _this.isConsumingQueue = false;
                }
            });
        };
        return ViewEventDispatcher;
    }());
    exports.ViewEventDispatcher = ViewEventDispatcher;
});

define("vs/editor/common/model/editStack", ["require", "exports", 'vs/base/common/errors'], function (require, exports, errors_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EditStack = (function () {
        function EditStack(model) {
            this.model = model;
            this.currentOpenStackElement = null;
            this.past = [];
            this.future = [];
        }
        EditStack.prototype.pushStackElement = function () {
            if (this.currentOpenStackElement !== null) {
                this.past.push(this.currentOpenStackElement);
                this.currentOpenStackElement = null;
            }
        };
        EditStack.prototype.clear = function () {
            this.currentOpenStackElement = null;
            this.past = [];
            this.future = [];
        };
        EditStack.prototype.pushEditOperation = function (beforeCursorState, editOperations, cursorStateComputer) {
            // No support for parallel universes :(
            this.future = [];
            if (!this.currentOpenStackElement) {
                this.currentOpenStackElement = {
                    beforeVersionId: this.model.getAlternativeVersionId(),
                    beforeCursorState: beforeCursorState,
                    editOperations: [],
                    afterCursorState: null,
                    afterVersionId: -1
                };
            }
            var inverseEditOperation = {
                operations: this.model.applyEdits(editOperations)
            };
            this.currentOpenStackElement.editOperations.push(inverseEditOperation);
            try {
                this.currentOpenStackElement.afterCursorState = cursorStateComputer ? cursorStateComputer(inverseEditOperation.operations) : null;
            }
            catch (e) {
                errors_1.onUnexpectedError(e);
                this.currentOpenStackElement.afterCursorState = null;
            }
            this.currentOpenStackElement.afterVersionId = this.model.getVersionId();
            return this.currentOpenStackElement.afterCursorState;
        };
        EditStack.prototype.undo = function () {
            this.pushStackElement();
            if (this.past.length > 0) {
                var pastStackElement = this.past.pop();
                try {
                    // Apply all operations in reverse order
                    for (var i = pastStackElement.editOperations.length - 1; i >= 0; i--) {
                        pastStackElement.editOperations[i] = {
                            operations: this.model.applyEdits(pastStackElement.editOperations[i].operations)
                        };
                    }
                }
                catch (e) {
                    this.clear();
                    return null;
                }
                this.future.push(pastStackElement);
                return {
                    selections: pastStackElement.beforeCursorState,
                    recordedVersionId: pastStackElement.beforeVersionId
                };
            }
            return null;
        };
        EditStack.prototype.redo = function () {
            if (this.future.length > 0) {
                if (this.currentOpenStackElement) {
                    throw new Error('How is this possible?');
                }
                var futureStackElement = this.future.pop();
                try {
                    // Apply all operations
                    for (var i = 0; i < futureStackElement.editOperations.length; i++) {
                        futureStackElement.editOperations[i] = {
                            operations: this.model.applyEdits(futureStackElement.editOperations[i].operations)
                        };
                    }
                }
                catch (e) {
                    this.clear();
                    return null;
                }
                this.past.push(futureStackElement);
                return {
                    selections: futureStackElement.afterCursorState,
                    recordedVersionId: futureStackElement.afterVersionId
                };
            }
            return null;
        };
        return EditStack;
    }());
    exports.EditStack = EditStack;
});

define("vs/editor/common/model/tokensBinaryEncoding", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/strings'], function (require, exports, errors_1, strings) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var InflatedToken = (function () {
        function InflatedToken(startIndex, type) {
            this.startIndex = startIndex;
            this.type = type;
        }
        InflatedToken.prototype.toString = function () {
            return '{ ' + this.startIndex + ', \'' + this.type + '\'}';
        };
        return InflatedToken;
    }());
    exports.START_INDEX_MASK = 0xffffffff;
    exports.TYPE_MASK = 0xffff;
    exports.START_INDEX_OFFSET = 1;
    exports.TYPE_OFFSET = Math.pow(2, 32);
    var DEFAULT_TOKEN = {
        startIndex: 0,
        type: ''
    };
    var INFLATED_TOKENS_EMPTY_TEXT = [];
    var DEFLATED_TOKENS_EMPTY_TEXT = [];
    var INFLATED_TOKENS_NON_EMPTY_TEXT = [DEFAULT_TOKEN];
    var DEFLATED_TOKENS_NON_EMPTY_TEXT = [0];
    function deflateArr(map, tokens) {
        if (tokens.length === 0) {
            return DEFLATED_TOKENS_EMPTY_TEXT;
        }
        if (tokens.length === 1 && tokens[0].startIndex === 0 && !tokens[0].type) {
            return DEFLATED_TOKENS_NON_EMPTY_TEXT;
        }
        var i, len, deflatedToken, deflated, token, inflateMap = map._inflate, deflateMap = map._deflate, prevStartIndex = -1, result = new Array(tokens.length);
        for (i = 0, len = tokens.length; i < len; i++) {
            token = tokens[i];
            if (token.startIndex <= prevStartIndex) {
                token.startIndex = prevStartIndex + 1;
                errors_1.onUnexpectedError({
                    message: 'Invalid tokens detected',
                    tokens: tokens
                });
            }
            if (deflateMap.hasOwnProperty(token.type)) {
                deflatedToken = deflateMap[token.type];
            }
            else {
                deflatedToken = inflateMap.length;
                deflateMap[token.type] = deflatedToken;
                inflateMap.push(token.type);
            }
            // http://stackoverflow.com/a/2803010
            // All numbers in JavaScript are actually IEEE-754 compliant floating-point doubles.
            // These have a 53-bit mantissa which should mean that any integer value with a magnitude
            // of approximately 9 quadrillion or less -- more specifically, 9,007,199,254,740,991 --
            // will be represented accurately.
            // http://stackoverflow.com/a/6729252
            // Bitwise operations cast numbers to 32bit representation in JS
            // 32 bits for startIndex => up to 2^32 = 4,294,967,296
            // 16 bits for token => up to 2^16 = 65,536
            // [token][startIndex]
            deflated = deflatedToken * exports.TYPE_OFFSET + token.startIndex * exports.START_INDEX_OFFSET;
            result[i] = deflated;
            prevStartIndex = token.startIndex;
        }
        return result;
    }
    exports.deflateArr = deflateArr;
    function inflate(map, binaryEncodedToken) {
        if (binaryEncodedToken === 0) {
            return DEFAULT_TOKEN;
        }
        var startIndex = (binaryEncodedToken / exports.START_INDEX_OFFSET) & exports.START_INDEX_MASK;
        var deflatedType = (binaryEncodedToken / exports.TYPE_OFFSET) & exports.TYPE_MASK;
        return new InflatedToken(startIndex, map._inflate[deflatedType]);
    }
    exports.inflate = inflate;
    function getStartIndex(binaryEncodedToken) {
        return (binaryEncodedToken / exports.START_INDEX_OFFSET) & exports.START_INDEX_MASK;
    }
    exports.getStartIndex = getStartIndex;
    function getType(map, binaryEncodedToken) {
        var deflatedType = (binaryEncodedToken / exports.TYPE_OFFSET) & exports.TYPE_MASK;
        if (deflatedType === 0) {
            return strings.empty;
        }
        return map._inflate[deflatedType];
    }
    exports.getType = getType;
    function inflateArr(map, binaryEncodedTokens) {
        if (binaryEncodedTokens.length === 0) {
            return INFLATED_TOKENS_EMPTY_TEXT;
        }
        if (binaryEncodedTokens.length === 1 && binaryEncodedTokens[0] === 0) {
            return INFLATED_TOKENS_NON_EMPTY_TEXT;
        }
        var result = new Array(binaryEncodedTokens.length), i, len, deflated, startIndex, deflatedType, inflateMap = map._inflate;
        for (i = 0, len = binaryEncodedTokens.length; i < len; i++) {
            deflated = binaryEncodedTokens[i];
            startIndex = (deflated / exports.START_INDEX_OFFSET) & exports.START_INDEX_MASK;
            deflatedType = (deflated / exports.TYPE_OFFSET) & exports.TYPE_MASK;
            result[i] = new InflatedToken(startIndex, inflateMap[deflatedType]);
        }
        return result;
    }
    exports.inflateArr = inflateArr;
    function findIndexOfOffset(binaryEncodedTokens, offset) {
        return findIndexInSegmentsArray(binaryEncodedTokens, offset);
    }
    exports.findIndexOfOffset = findIndexOfOffset;
    function sliceAndInflate(map, binaryEncodedTokens, startOffset, endOffset, deltaStartIndex) {
        if (binaryEncodedTokens.length === 0) {
            return INFLATED_TOKENS_EMPTY_TEXT;
        }
        if (binaryEncodedTokens.length === 1 && binaryEncodedTokens[0] === 0) {
            return INFLATED_TOKENS_NON_EMPTY_TEXT;
        }
        var startIndex = findIndexInSegmentsArray(binaryEncodedTokens, startOffset), i, len, originalToken, originalStartIndex, newStartIndex, deflatedType, result = [], inflateMap = map._inflate;
        originalToken = binaryEncodedTokens[startIndex];
        deflatedType = (originalToken / exports.TYPE_OFFSET) & exports.TYPE_MASK;
        newStartIndex = 0;
        result.push(new InflatedToken(newStartIndex, inflateMap[deflatedType]));
        for (i = startIndex + 1, len = binaryEncodedTokens.length; i < len; i++) {
            originalToken = binaryEncodedTokens[i];
            originalStartIndex = (originalToken / exports.START_INDEX_OFFSET) & exports.START_INDEX_MASK;
            if (originalStartIndex >= endOffset) {
                break;
            }
            deflatedType = (originalToken / exports.TYPE_OFFSET) & exports.TYPE_MASK;
            newStartIndex = originalStartIndex - startOffset + deltaStartIndex;
            result.push(new InflatedToken(newStartIndex, inflateMap[deflatedType]));
        }
        return result;
    }
    exports.sliceAndInflate = sliceAndInflate;
    function findIndexInSegmentsArray(arr, desiredIndex) {
        var low = 0, high = arr.length - 1, mid, value;
        while (low < high) {
            mid = low + Math.ceil((high - low) / 2);
            value = arr[mid] & 0xffffffff;
            if (value > desiredIndex) {
                high = mid - 1;
            }
            else {
                low = mid;
            }
        }
        return low;
    }
    exports.findIndexInSegmentsArray = findIndexInSegmentsArray;
});

define("vs/editor/common/editorCommon", ["require", "exports", 'vs/editor/common/model/tokensBinaryEncoding'], function (require, exports, TokensBinaryEncoding) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The direction of a selection.
     */
    (function (SelectionDirection) {
        /**
         * The selection starts above where it ends.
         */
        SelectionDirection[SelectionDirection["LTR"] = 0] = "LTR";
        /**
         * The selection starts below where it ends.
         */
        SelectionDirection[SelectionDirection["RTL"] = 1] = "RTL";
    })(exports.SelectionDirection || (exports.SelectionDirection = {}));
    var SelectionDirection = exports.SelectionDirection;
    (function (WrappingIndent) {
        WrappingIndent[WrappingIndent["None"] = 0] = "None";
        WrappingIndent[WrappingIndent["Same"] = 1] = "Same";
        WrappingIndent[WrappingIndent["Indent"] = 2] = "Indent";
    })(exports.WrappingIndent || (exports.WrappingIndent = {}));
    var WrappingIndent = exports.WrappingIndent;
    function wrappingIndentFromString(wrappingIndent) {
        if (wrappingIndent === 'indent') {
            return WrappingIndent.Indent;
        }
        else if (wrappingIndent === 'same') {
            return WrappingIndent.Same;
        }
        else {
            return WrappingIndent.None;
        }
    }
    exports.wrappingIndentFromString = wrappingIndentFromString;
    /**
     * Vertical Lane in the overview ruler of the editor.
     */
    (function (OverviewRulerLane) {
        OverviewRulerLane[OverviewRulerLane["Left"] = 1] = "Left";
        OverviewRulerLane[OverviewRulerLane["Center"] = 2] = "Center";
        OverviewRulerLane[OverviewRulerLane["Right"] = 4] = "Right";
        OverviewRulerLane[OverviewRulerLane["Full"] = 7] = "Full";
    })(exports.OverviewRulerLane || (exports.OverviewRulerLane = {}));
    var OverviewRulerLane = exports.OverviewRulerLane;
    /**
     * End of line character preference.
     */
    (function (EndOfLinePreference) {
        /**
         * Use the end of line character identified in the text buffer.
         */
        EndOfLinePreference[EndOfLinePreference["TextDefined"] = 0] = "TextDefined";
        /**
         * Use line feed (\n) as the end of line character.
         */
        EndOfLinePreference[EndOfLinePreference["LF"] = 1] = "LF";
        /**
         * Use carriage return and line feed (\r\n) as the end of line character.
         */
        EndOfLinePreference[EndOfLinePreference["CRLF"] = 2] = "CRLF";
    })(exports.EndOfLinePreference || (exports.EndOfLinePreference = {}));
    var EndOfLinePreference = exports.EndOfLinePreference;
    /**
     * The default end of line to use when instantiating models.
     */
    (function (DefaultEndOfLine) {
        /**
         * Use line feed (\n) as the end of line character.
         */
        DefaultEndOfLine[DefaultEndOfLine["LF"] = 1] = "LF";
        /**
         * Use carriage return and line feed (\r\n) as the end of line character.
         */
        DefaultEndOfLine[DefaultEndOfLine["CRLF"] = 2] = "CRLF";
    })(exports.DefaultEndOfLine || (exports.DefaultEndOfLine = {}));
    var DefaultEndOfLine = exports.DefaultEndOfLine;
    /**
     * End of line character preference.
     */
    (function (EndOfLineSequence) {
        /**
         * Use line feed (\n) as the end of line character.
         */
        EndOfLineSequence[EndOfLineSequence["LF"] = 0] = "LF";
        /**
         * Use carriage return and line feed (\r\n) as the end of line character.
         */
        EndOfLineSequence[EndOfLineSequence["CRLF"] = 1] = "CRLF";
    })(exports.EndOfLineSequence || (exports.EndOfLineSequence = {}));
    var EndOfLineSequence = exports.EndOfLineSequence;
    exports.LineTokensBinaryEncoding = TokensBinaryEncoding;
    (function (TrackedRangeStickiness) {
        TrackedRangeStickiness[TrackedRangeStickiness["AlwaysGrowsWhenTypingAtEdges"] = 0] = "AlwaysGrowsWhenTypingAtEdges";
        TrackedRangeStickiness[TrackedRangeStickiness["NeverGrowsWhenTypingAtEdges"] = 1] = "NeverGrowsWhenTypingAtEdges";
        TrackedRangeStickiness[TrackedRangeStickiness["GrowsOnlyWhenTypingBefore"] = 2] = "GrowsOnlyWhenTypingBefore";
        TrackedRangeStickiness[TrackedRangeStickiness["GrowsOnlyWhenTypingAfter"] = 3] = "GrowsOnlyWhenTypingAfter";
    })(exports.TrackedRangeStickiness || (exports.TrackedRangeStickiness = {}));
    var TrackedRangeStickiness = exports.TrackedRangeStickiness;
    (function (VerticalRevealType) {
        VerticalRevealType[VerticalRevealType["Simple"] = 0] = "Simple";
        VerticalRevealType[VerticalRevealType["Center"] = 1] = "Center";
        VerticalRevealType[VerticalRevealType["CenterIfOutsideViewport"] = 2] = "CenterIfOutsideViewport";
    })(exports.VerticalRevealType || (exports.VerticalRevealType = {}));
    var VerticalRevealType = exports.VerticalRevealType;
    /**
     * Type of hit element with the mouse in the editor.
     */
    (function (MouseTargetType) {
        /**
         * Mouse is on top of an unknown element.
         */
        MouseTargetType[MouseTargetType["UNKNOWN"] = 0] = "UNKNOWN";
        /**
         * Mouse is on top of the textarea used for input.
         */
        MouseTargetType[MouseTargetType["TEXTAREA"] = 1] = "TEXTAREA";
        /**
         * Mouse is on top of the glyph margin
         */
        MouseTargetType[MouseTargetType["GUTTER_GLYPH_MARGIN"] = 2] = "GUTTER_GLYPH_MARGIN";
        /**
         * Mouse is on top of the line numbers
         */
        MouseTargetType[MouseTargetType["GUTTER_LINE_NUMBERS"] = 3] = "GUTTER_LINE_NUMBERS";
        /**
         * Mouse is on top of the line decorations
         */
        MouseTargetType[MouseTargetType["GUTTER_LINE_DECORATIONS"] = 4] = "GUTTER_LINE_DECORATIONS";
        /**
         * Mouse is on top of the whitespace left in the gutter by a view zone.
         */
        MouseTargetType[MouseTargetType["GUTTER_VIEW_ZONE"] = 5] = "GUTTER_VIEW_ZONE";
        /**
         * Mouse is on top of text in the content.
         */
        MouseTargetType[MouseTargetType["CONTENT_TEXT"] = 6] = "CONTENT_TEXT";
        /**
         * Mouse is on top of empty space in the content (e.g. after line text or below last line)
         */
        MouseTargetType[MouseTargetType["CONTENT_EMPTY"] = 7] = "CONTENT_EMPTY";
        /**
         * Mouse is on top of a view zone in the content.
         */
        MouseTargetType[MouseTargetType["CONTENT_VIEW_ZONE"] = 8] = "CONTENT_VIEW_ZONE";
        /**
         * Mouse is on top of a content widget.
         */
        MouseTargetType[MouseTargetType["CONTENT_WIDGET"] = 9] = "CONTENT_WIDGET";
        /**
         * Mouse is on top of the decorations overview ruler.
         */
        MouseTargetType[MouseTargetType["OVERVIEW_RULER"] = 10] = "OVERVIEW_RULER";
        /**
         * Mouse is on top of a scrollbar.
         */
        MouseTargetType[MouseTargetType["SCROLLBAR"] = 11] = "SCROLLBAR";
        /**
         * Mouse is on top of an overlay widget.
         */
        MouseTargetType[MouseTargetType["OVERLAY_WIDGET"] = 12] = "OVERLAY_WIDGET";
    })(exports.MouseTargetType || (exports.MouseTargetType = {}));
    var MouseTargetType = exports.MouseTargetType;
    exports.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS = 'editorTextFocus';
    exports.KEYBINDING_CONTEXT_EDITOR_FOCUS = 'editorFocus';
    exports.KEYBINDING_CONTEXT_EDITOR_TAB_MOVES_FOCUS = 'editorTabMovesFocus';
    exports.KEYBINDING_CONTEXT_EDITOR_HAS_MULTIPLE_SELECTIONS = 'editorHasMultipleSelections';
    exports.KEYBINDING_CONTEXT_EDITOR_HAS_NON_EMPTY_SELECTION = 'editorHasSelection';
    exports.KEYBINDING_CONTEXT_EDITOR_LANGUAGE_ID = 'editorLangId';
    exports.SHOW_ACCESSIBILITY_HELP_ACTION_ID = 'editor.action.showAccessibilityHelp';
    exports.ViewEventNames = {
        ModelFlushedEvent: 'modelFlushedEvent',
        LinesDeletedEvent: 'linesDeletedEvent',
        LinesInsertedEvent: 'linesInsertedEvent',
        LineChangedEvent: 'lineChangedEvent',
        TokensChangedEvent: 'tokensChangedEvent',
        DecorationsChangedEvent: 'decorationsChangedEvent',
        CursorPositionChangedEvent: 'cursorPositionChangedEvent',
        CursorSelectionChangedEvent: 'cursorSelectionChangedEvent',
        RevealRangeEvent: 'revealRangeEvent',
        LineMappingChangedEvent: 'lineMappingChangedEvent',
        ScrollRequestEvent: 'scrollRequestEvent'
    };
    (function (CodeEditorStateFlag) {
        CodeEditorStateFlag[CodeEditorStateFlag["Value"] = 0] = "Value";
        CodeEditorStateFlag[CodeEditorStateFlag["Selection"] = 1] = "Selection";
        CodeEditorStateFlag[CodeEditorStateFlag["Position"] = 2] = "Position";
        CodeEditorStateFlag[CodeEditorStateFlag["Scroll"] = 3] = "Scroll";
    })(exports.CodeEditorStateFlag || (exports.CodeEditorStateFlag = {}));
    var CodeEditorStateFlag = exports.CodeEditorStateFlag;
    exports.EditorType = {
        ICodeEditor: 'vs.editor.ICodeEditor',
        IDiffEditor: 'vs.editor.IDiffEditor'
    };
    exports.ClassName = {
        EditorWarningDecoration: 'greensquiggly',
        EditorErrorDecoration: 'redsquiggly'
    };
    exports.EventType = {
        Disposed: 'disposed',
        ConfigurationChanged: 'configurationChanged',
        ModelDispose: 'modelDispose',
        ModelChanged: 'modelChanged',
        ModelTokensChanged: 'modelTokensChanged',
        ModelModeChanged: 'modelsModeChanged',
        ModelModeSupportChanged: 'modelsModeSupportChanged',
        ModelOptionsChanged: 'modelOptionsChanged',
        ModelContentChanged: 'contentChanged',
        ModelContentChanged2: 'contentChanged2',
        ModelContentChangedFlush: 'flush',
        ModelContentChangedLinesDeleted: 'linesDeleted',
        ModelContentChangedLinesInserted: 'linesInserted',
        ModelContentChangedLineChanged: 'lineChanged',
        EditorTextBlur: 'blur',
        EditorTextFocus: 'focus',
        EditorFocus: 'widgetFocus',
        EditorBlur: 'widgetBlur',
        ModelDecorationsChanged: 'decorationsChanged',
        CursorPositionChanged: 'positionChanged',
        CursorSelectionChanged: 'selectionChanged',
        CursorRevealRange: 'revealRange',
        CursorScrollRequest: 'scrollRequest',
        ViewFocusGained: 'focusGained',
        ViewFocusLost: 'focusLost',
        ViewFocusChanged: 'focusChanged',
        ViewScrollWidthChanged: 'scrollWidthChanged',
        ViewScrollHeightChanged: 'scrollHeightChanged',
        ViewScrollChanged: 'scrollChanged',
        ViewZonesChanged: 'zonesChanged',
        ViewLayoutChanged: 'viewLayoutChanged',
        ContextMenu: 'contextMenu',
        MouseDown: 'mousedown',
        MouseUp: 'mouseup',
        MouseMove: 'mousemove',
        MouseLeave: 'mouseleave',
        KeyDown: 'keydown',
        KeyUp: 'keyup',
        EditorLayout: 'editorLayout',
        DiffUpdated: 'diffUpdated'
    };
    exports.Handler = {
        ExecuteCommand: 'executeCommand',
        ExecuteCommands: 'executeCommands',
        CursorLeft: 'cursorLeft',
        CursorLeftSelect: 'cursorLeftSelect',
        CursorWordLeft: 'cursorWordLeft',
        CursorWordStartLeft: 'cursorWordStartLeft',
        CursorWordEndLeft: 'cursorWordEndLeft',
        CursorWordLeftSelect: 'cursorWordLeftSelect',
        CursorWordStartLeftSelect: 'cursorWordStartLeftSelect',
        CursorWordEndLeftSelect: 'cursorWordEndLeftSelect',
        CursorRight: 'cursorRight',
        CursorRightSelect: 'cursorRightSelect',
        CursorWordRight: 'cursorWordRight',
        CursorWordStartRight: 'cursorWordStartRight',
        CursorWordEndRight: 'cursorWordEndRight',
        CursorWordRightSelect: 'cursorWordRightSelect',
        CursorWordStartRightSelect: 'cursorWordStartRightSelect',
        CursorWordEndRightSelect: 'cursorWordEndRightSelect',
        CursorUp: 'cursorUp',
        CursorUpSelect: 'cursorUpSelect',
        CursorDown: 'cursorDown',
        CursorDownSelect: 'cursorDownSelect',
        CursorPageUp: 'cursorPageUp',
        CursorPageUpSelect: 'cursorPageUpSelect',
        CursorPageDown: 'cursorPageDown',
        CursorPageDownSelect: 'cursorPageDownSelect',
        CursorHome: 'cursorHome',
        CursorHomeSelect: 'cursorHomeSelect',
        CursorEnd: 'cursorEnd',
        CursorEndSelect: 'cursorEndSelect',
        ExpandLineSelection: 'expandLineSelection',
        CursorTop: 'cursorTop',
        CursorTopSelect: 'cursorTopSelect',
        CursorBottom: 'cursorBottom',
        CursorBottomSelect: 'cursorBottomSelect',
        CursorColumnSelectLeft: 'cursorColumnSelectLeft',
        CursorColumnSelectRight: 'cursorColumnSelectRight',
        CursorColumnSelectUp: 'cursorColumnSelectUp',
        CursorColumnSelectPageUp: 'cursorColumnSelectPageUp',
        CursorColumnSelectDown: 'cursorColumnSelectDown',
        CursorColumnSelectPageDown: 'cursorColumnSelectPageDown',
        AddCursorDown: 'addCursorDown',
        AddCursorUp: 'addCursorUp',
        CursorUndo: 'cursorUndo',
        MoveTo: 'moveTo',
        MoveToSelect: 'moveToSelect',
        ColumnSelect: 'columnSelect',
        CreateCursor: 'createCursor',
        LastCursorMoveToSelect: 'lastCursorMoveToSelect',
        JumpToBracket: 'jumpToBracket',
        Type: 'type',
        ReplacePreviousChar: 'replacePreviousChar',
        Paste: 'paste',
        Tab: 'tab',
        Indent: 'indent',
        Outdent: 'outdent',
        DeleteLeft: 'deleteLeft',
        DeleteRight: 'deleteRight',
        DeleteWordLeft: 'deleteWordLeft',
        DeleteWordStartLeft: 'deleteWordStartLeft',
        DeleteWordEndLeft: 'deleteWordEndLeft',
        DeleteWordRight: 'deleteWordRight',
        DeleteWordStartRight: 'deleteWordStartRight',
        DeleteWordEndRight: 'deleteWordEndRight',
        DeleteAllLeft: 'deleteAllLeft',
        DeleteAllRight: 'deleteAllRight',
        Enter: 'enter',
        RemoveSecondaryCursors: 'removeSecondaryCursors',
        CancelSelection: 'cancelSelection',
        Cut: 'cut',
        Undo: 'undo',
        Redo: 'redo',
        WordSelect: 'wordSelect',
        WordSelectDrag: 'wordSelectDrag',
        LastCursorWordSelect: 'lastCursorWordSelect',
        LineSelect: 'lineSelect',
        LineSelectDrag: 'lineSelectDrag',
        LastCursorLineSelect: 'lastCursorLineSelect',
        LastCursorLineSelectDrag: 'lastCursorLineSelectDrag',
        LineInsertBefore: 'lineInsertBefore',
        LineInsertAfter: 'lineInsertAfter',
        LineBreakInsert: 'lineBreakInsert',
        SelectAll: 'selectAll',
        ScrollLineUp: 'scrollLineUp',
        ScrollLineDown: 'scrollLineDown',
        ScrollPageUp: 'scrollPageUp',
        ScrollPageDown: 'scrollPageDown'
    };
    var VisibleRange = (function () {
        function VisibleRange(top, left, width) {
            this.top = top;
            this.left = left;
            this.width = width;
        }
        return VisibleRange;
    }());
    exports.VisibleRange = VisibleRange;
    (function (TextEditorCursorStyle) {
        TextEditorCursorStyle[TextEditorCursorStyle["Line"] = 1] = "Line";
        TextEditorCursorStyle[TextEditorCursorStyle["Block"] = 2] = "Block";
        TextEditorCursorStyle[TextEditorCursorStyle["Underline"] = 3] = "Underline";
    })(exports.TextEditorCursorStyle || (exports.TextEditorCursorStyle = {}));
    var TextEditorCursorStyle = exports.TextEditorCursorStyle;
    function cursorStyleFromString(cursorStyle) {
        if (cursorStyle === 'line') {
            return TextEditorCursorStyle.Line;
        }
        else if (cursorStyle === 'block') {
            return TextEditorCursorStyle.Block;
        }
        else if (cursorStyle === 'underline') {
            return TextEditorCursorStyle.Underline;
        }
        return TextEditorCursorStyle.Line;
    }
    exports.cursorStyleFromString = cursorStyleFromString;
    function cursorStyleToString(cursorStyle) {
        if (cursorStyle === TextEditorCursorStyle.Line) {
            return 'line';
        }
        else if (cursorStyle === TextEditorCursorStyle.Block) {
            return 'block';
        }
        else if (cursorStyle === TextEditorCursorStyle.Underline) {
            return 'underline';
        }
        else {
            throw new Error('cursorStyleToString: Unknown cursorStyle');
        }
    }
    exports.cursorStyleToString = cursorStyleToString;
    var HorizontalRange = (function () {
        function HorizontalRange(left, width) {
            this.left = left;
            this.width = width;
        }
        return HorizontalRange;
    }());
    exports.HorizontalRange = HorizontalRange;
    var LineVisibleRanges = (function () {
        function LineVisibleRanges(lineNumber, ranges) {
            this.lineNumber = lineNumber;
            this.ranges = ranges;
        }
        return LineVisibleRanges;
    }());
    exports.LineVisibleRanges = LineVisibleRanges;
});

define("vs/editor/browser/controller/mouseTarget", ["require", "exports", 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowser'], function (require, exports, position_1, range_1, editorCommon_1, editorBrowser_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var MouseTarget = (function () {
        function MouseTarget(element, type, mouseColumn, position, range, detail) {
            if (mouseColumn === void 0) { mouseColumn = 0; }
            if (position === void 0) { position = null; }
            if (range === void 0) { range = null; }
            if (detail === void 0) { detail = null; }
            this.element = element;
            this.type = type;
            this.mouseColumn = mouseColumn;
            this.position = position;
            if (!range && position) {
                range = new range_1.Range(position.lineNumber, position.column, position.lineNumber, position.column);
            }
            this.range = range;
            this.detail = detail;
        }
        MouseTarget.prototype._typeToString = function () {
            if (this.type === editorCommon_1.MouseTargetType.TEXTAREA) {
                return 'TEXTAREA';
            }
            if (this.type === editorCommon_1.MouseTargetType.GUTTER_GLYPH_MARGIN) {
                return 'GUTTER_GLYPH_MARGIN';
            }
            if (this.type === editorCommon_1.MouseTargetType.GUTTER_LINE_NUMBERS) {
                return 'GUTTER_LINE_NUMBERS';
            }
            if (this.type === editorCommon_1.MouseTargetType.GUTTER_LINE_DECORATIONS) {
                return 'GUTTER_LINE_DECORATIONS';
            }
            if (this.type === editorCommon_1.MouseTargetType.GUTTER_VIEW_ZONE) {
                return 'GUTTER_VIEW_ZONE';
            }
            if (this.type === editorCommon_1.MouseTargetType.CONTENT_TEXT) {
                return 'CONTENT_TEXT';
            }
            if (this.type === editorCommon_1.MouseTargetType.CONTENT_EMPTY) {
                return 'CONTENT_EMPTY';
            }
            if (this.type === editorCommon_1.MouseTargetType.CONTENT_VIEW_ZONE) {
                return 'CONTENT_VIEW_ZONE';
            }
            if (this.type === editorCommon_1.MouseTargetType.CONTENT_WIDGET) {
                return 'CONTENT_WIDGET';
            }
            if (this.type === editorCommon_1.MouseTargetType.OVERVIEW_RULER) {
                return 'OVERVIEW_RULER';
            }
            if (this.type === editorCommon_1.MouseTargetType.SCROLLBAR) {
                return 'SCROLLBAR';
            }
            if (this.type === editorCommon_1.MouseTargetType.OVERLAY_WIDGET) {
                return 'OVERLAY_WIDGET';
            }
            return 'UNKNOWN';
        };
        MouseTarget.prototype.toString = function () {
            return this._typeToString() + ': ' + this.position + ' - ' + this.range + ' - ' + this.detail;
        };
        return MouseTarget;
    }());
    // e.g. of paths:
    // - overflow-guard/monaco-scrollable-element editor-scrollable vs/lines-content/view-lines/view-line
    // - overflow-guard/monaco-scrollable-element editor-scrollable vs/lines-content/view-lines/view-line/token comment js
    // etc.
    var REGEX = (function () {
        function nodeWithClass(className) {
            return '[^/]*' + className + '[^/]*';
        }
        function anyNode() {
            return '[^/]+';
        }
        var ANCHOR = '^' + editorBrowser_1.ClassNames.OVERFLOW_GUARD + '\\/';
        function createRegExp() {
            var pieces = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                pieces[_i - 0] = arguments[_i];
            }
            var forceEndMatch = false;
            if (pieces[pieces.length - 1] === '$') {
                forceEndMatch = true;
                pieces.pop();
            }
            return new RegExp(ANCHOR + pieces.join('\\/') + (forceEndMatch ? '$' : ''));
        }
        return {
            IS_TEXTAREA_COVER: createRegExp(nodeWithClass(editorBrowser_1.ClassNames.TEXTAREA_COVER), '$'),
            IS_TEXTAREA: createRegExp(editorBrowser_1.ClassNames.TEXTAREA, '$'),
            IS_VIEW_LINES: createRegExp(anyNode(), anyNode(), editorBrowser_1.ClassNames.VIEW_LINES, '$'),
            IS_CURSORS_LAYER: createRegExp(anyNode(), anyNode(), nodeWithClass(editorBrowser_1.ClassNames.VIEW_CURSORS_LAYER), '$'),
            IS_CHILD_OF_VIEW_LINES: createRegExp(anyNode(), anyNode(), editorBrowser_1.ClassNames.VIEW_LINES),
            IS_CHILD_OF_SCROLLABLE_ELEMENT: createRegExp(nodeWithClass(editorBrowser_1.ClassNames.SCROLLABLE_ELEMENT)),
            IS_CHILD_OF_CONTENT_WIDGETS: createRegExp(anyNode(), anyNode(), editorBrowser_1.ClassNames.CONTENT_WIDGETS),
            IS_CHILD_OF_OVERFLOWING_CONTENT_WIDGETS: new RegExp('^' + editorBrowser_1.ClassNames.OVERFLOWING_CONTENT_WIDGETS + '\\/'),
            IS_CHILD_OF_OVERLAY_WIDGETS: createRegExp(editorBrowser_1.ClassNames.OVERLAY_WIDGETS),
            IS_CHILD_OF_VIEW_OVERLAYS: createRegExp(editorBrowser_1.ClassNames.MARGIN_VIEW_OVERLAYS),
            IS_CHILD_OF_VIEW_ZONES: createRegExp(anyNode(), anyNode(), editorBrowser_1.ClassNames.VIEW_ZONES),
        };
    })();
    var MouseTargetFactory = (function () {
        function MouseTargetFactory(context, viewHelper) {
            this.context = context;
            this.viewHelper = viewHelper;
        }
        MouseTargetFactory.prototype.getClassNamePathTo = function (child, stopAt) {
            var path = [], className;
            while (child && child !== document.body) {
                if (child === stopAt) {
                    break;
                }
                if (child.nodeType === child.ELEMENT_NODE) {
                    className = child.className;
                    if (className) {
                        path.unshift(className);
                    }
                }
                child = child.parentNode;
            }
            return path.join('/');
        };
        MouseTargetFactory.prototype.mouseTargetIsWidget = function (e) {
            var t = e.target;
            var path = this.getClassNamePathTo(t, this.viewHelper.viewDomNode);
            // Is it a content widget?
            if (REGEX.IS_CHILD_OF_CONTENT_WIDGETS.test(path) || REGEX.IS_CHILD_OF_OVERFLOWING_CONTENT_WIDGETS.test(path)) {
                return true;
            }
            // Is it an overlay widget?
            if (REGEX.IS_CHILD_OF_OVERLAY_WIDGETS.test(path)) {
                return true;
            }
            return false;
        };
        MouseTargetFactory.prototype.createMouseTarget = function (layoutInfo, editorContent, e, testEventTarget) {
            try {
                var r = this._unsafeCreateMouseTarget(layoutInfo, editorContent, e, testEventTarget);
                return r;
            }
            catch (e) {
                return this.createMouseTargetFromUnknownTarget(e.target);
            }
        };
        MouseTargetFactory.prototype._unsafeCreateMouseTarget = function (layoutInfo, editorContent, e, testEventTarget) {
            var mouseVerticalOffset = Math.max(0, this.viewHelper.getScrollTop() + (e.posy - editorContent.top));
            var mouseContentHorizontalOffset = this.viewHelper.getScrollLeft() + (e.posx - editorContent.left) - layoutInfo.contentLeft;
            var mouseColumn = this._getMouseColumn(mouseContentHorizontalOffset);
            var t = e.target;
            var path = this.getClassNamePathTo(t, this.viewHelper.viewDomNode);
            // Is it a cursor ?
            var lineNumberAttribute = t.hasAttribute && t.hasAttribute('lineNumber') ? t.getAttribute('lineNumber') : null;
            var columnAttribute = t.hasAttribute && t.hasAttribute('column') ? t.getAttribute('column') : null;
            if (lineNumberAttribute && columnAttribute) {
                return this.createMouseTargetFromViewCursor(t, parseInt(lineNumberAttribute, 10), parseInt(columnAttribute, 10), mouseColumn);
            }
            // Is it a content widget?
            if (REGEX.IS_CHILD_OF_CONTENT_WIDGETS.test(path) || REGEX.IS_CHILD_OF_OVERFLOWING_CONTENT_WIDGETS.test(path)) {
                return this.createMouseTargetFromContentWidgetsChild(t, mouseColumn);
            }
            // Is it an overlay widget?
            if (REGEX.IS_CHILD_OF_OVERLAY_WIDGETS.test(path)) {
                return this.createMouseTargetFromOverlayWidgetsChild(t, mouseColumn);
            }
            // Is it the textarea cover?
            if (REGEX.IS_TEXTAREA_COVER.test(path)) {
                if (this.context.configuration.editor.glyphMargin) {
                    return this.createMouseTargetFromGlyphMargin(t, mouseVerticalOffset, mouseColumn);
                }
                else if (this.context.configuration.editor.lineNumbers) {
                    return this.createMouseTargetFromLineNumbers(t, mouseVerticalOffset, mouseColumn);
                }
                else {
                    return this.createMouseTargetFromLinesDecorationsChild(t, mouseVerticalOffset, mouseColumn);
                }
            }
            // Is it the textarea?
            if (REGEX.IS_TEXTAREA.test(path)) {
                return new MouseTarget(t, editorCommon_1.MouseTargetType.TEXTAREA);
            }
            // Is it a view zone?
            if (REGEX.IS_CHILD_OF_VIEW_ZONES.test(path)) {
                // Check if it is at a view zone
                var viewZoneData = this._getZoneAtCoord(mouseVerticalOffset);
                if (viewZoneData) {
                    return new MouseTarget(t, editorCommon_1.MouseTargetType.CONTENT_VIEW_ZONE, mouseColumn, viewZoneData.position, null, viewZoneData);
                }
                return this.createMouseTargetFromUnknownTarget(t);
            }
            // Is it the view lines container?
            if (REGEX.IS_VIEW_LINES.test(path)) {
                // Sometimes, IE returns this target when right clicking on top of text
                // -> See Bug #12990: [F12] Context menu shows incorrect position while doing a resize
                // Check if it is below any lines and any view zones
                if (this.viewHelper.isAfterLines(mouseVerticalOffset)) {
                    return this.createMouseTargetFromViewLines(t, mouseVerticalOffset, mouseColumn);
                }
                // Check if it is at a view zone
                var viewZoneData = this._getZoneAtCoord(mouseVerticalOffset);
                if (viewZoneData) {
                    return new MouseTarget(t, editorCommon_1.MouseTargetType.CONTENT_VIEW_ZONE, mouseColumn, viewZoneData.position, null, viewZoneData);
                }
                // Check if it hits a position
                var hitTestResult = this._doHitTest(editorContent, e, mouseVerticalOffset);
                if (hitTestResult.position) {
                    return this.createMouseTargetFromHitTestPosition(t, hitTestResult.position.lineNumber, hitTestResult.position.column, mouseContentHorizontalOffset, mouseColumn);
                }
                // Fall back to view lines
                return this.createMouseTargetFromViewLines(t, mouseVerticalOffset, mouseColumn);
            }
            // Is it a child of the view lines container?
            if (!testEventTarget || REGEX.IS_CHILD_OF_VIEW_LINES.test(path)) {
                var hitTestResult = this._doHitTest(editorContent, e, mouseVerticalOffset);
                if (hitTestResult.position) {
                    return this.createMouseTargetFromHitTestPosition(t, hitTestResult.position.lineNumber, hitTestResult.position.column, mouseContentHorizontalOffset, mouseColumn);
                }
                else if (hitTestResult.hitTarget) {
                    t = hitTestResult.hitTarget;
                    path = this.getClassNamePathTo(t, this.viewHelper.viewDomNode);
                    // TODO@Alex: try again with this different target, but guard against recursion.
                    // Is it a cursor ?
                    var lineNumberAttribute = t.hasAttribute && t.hasAttribute('lineNumber') ? t.getAttribute('lineNumber') : null;
                    var columnAttribute = t.hasAttribute && t.hasAttribute('column') ? t.getAttribute('column') : null;
                    if (lineNumberAttribute && columnAttribute) {
                        return this.createMouseTargetFromViewCursor(t, parseInt(lineNumberAttribute, 10), parseInt(columnAttribute, 10), mouseColumn);
                    }
                }
            }
            // Is it the cursors layer?
            if (REGEX.IS_CURSORS_LAYER.test(path)) {
                return new MouseTarget(t, editorCommon_1.MouseTargetType.UNKNOWN);
            }
            // Is it a child of the scrollable element?
            if (REGEX.IS_CHILD_OF_SCROLLABLE_ELEMENT.test(path)) {
                return this.createMouseTargetFromScrollbar(t, mouseVerticalOffset, mouseColumn);
            }
            if (REGEX.IS_CHILD_OF_VIEW_OVERLAYS.test(path)) {
                var offset = Math.abs(e.posx - editorContent.left);
                if (offset <= layoutInfo.glyphMarginWidth) {
                    // On the glyph margin
                    return this.createMouseTargetFromGlyphMargin(t, mouseVerticalOffset, mouseColumn);
                }
                offset -= layoutInfo.glyphMarginWidth;
                if (offset <= layoutInfo.lineNumbersWidth) {
                    // On the line numbers
                    return this.createMouseTargetFromLineNumbers(t, mouseVerticalOffset, mouseColumn);
                }
                offset -= layoutInfo.lineNumbersWidth;
                // On the line decorations
                return this.createMouseTargetFromLinesDecorationsChild(t, mouseVerticalOffset, mouseColumn);
            }
            if (/OverviewRuler/i.test(path)) {
                return this.createMouseTargetFromScrollbar(t, mouseVerticalOffset, mouseColumn);
            }
            return this.createMouseTargetFromUnknownTarget(t);
        };
        MouseTargetFactory.prototype._isChild = function (testChild, testAncestor, stopAt) {
            while (testChild && testChild !== document.body) {
                if (testChild === testAncestor) {
                    return true;
                }
                if (testChild === stopAt) {
                    return false;
                }
                testChild = testChild.parentNode;
            }
            return false;
        };
        MouseTargetFactory.prototype._findAttribute = function (element, attr, stopAt) {
            while (element && element !== document.body) {
                if (element.hasAttribute && element.hasAttribute(attr)) {
                    return element.getAttribute(attr);
                }
                if (element === stopAt) {
                    return null;
                }
                element = element.parentNode;
            }
            return null;
        };
        /**
         * Most probably WebKit browsers
         */
        MouseTargetFactory.prototype._doHitTestWithCaretRangeFromPoint = function (editorContent, e, mouseVerticalOffset) {
            // In Chrome, especially on Linux it is possible to click between lines,
            // so try to adjust the `hity` below so that it lands in the center of a line
            var lineNumber = this.viewHelper.getLineNumberAtVerticalOffset(mouseVerticalOffset);
            var lineVerticalOffset = this.viewHelper.getVerticalOffsetForLineNumber(lineNumber);
            var centeredVerticalOffset = lineVerticalOffset + Math.floor(this.context.configuration.editor.lineHeight / 2);
            var adjustedPosy = e.posy + (centeredVerticalOffset - mouseVerticalOffset);
            if (adjustedPosy <= editorContent.top) {
                adjustedPosy = editorContent.top + 1;
            }
            if (adjustedPosy >= editorContent.top + this.context.configuration.editor.observedOuterHeight) {
                adjustedPosy = editorContent.top + this.context.configuration.editor.observedOuterHeight - 1;
            }
            var hitx = e.posx - document.body.scrollLeft;
            var r = this._actualDoHitTestWithCaretRangeFromPoint(hitx, adjustedPosy - document.body.scrollTop);
            if (r.position) {
                return r;
            }
            // Also try to hit test without the adjustment (for the edge cases that we are near the top or bottom)
            return this._actualDoHitTestWithCaretRangeFromPoint(hitx, e.posy - document.body.scrollTop);
        };
        MouseTargetFactory.prototype._actualDoHitTestWithCaretRangeFromPoint = function (hitx, hity) {
            var range = document.caretRangeFromPoint(hitx, hity);
            if (!range || !range.startContainer) {
                return {
                    position: null,
                    hitTarget: null
                };
            }
            // Chrome always hits a TEXT_NODE, while Edge sometimes hits a token span
            var startContainer = range.startContainer;
            var hitTarget;
            if (startContainer.nodeType === startContainer.TEXT_NODE) {
                // startContainer is expected to be the token text
                var parent1 = startContainer.parentNode; // expected to be the token span
                var parent2 = parent1 ? parent1.parentNode : null; // expected to be the view line container span
                var parent3 = parent2 ? parent2.parentNode : null; // expected to be the view line div
                var parent3ClassName = parent3 && parent3.nodeType === parent3.ELEMENT_NODE ? parent3.className : null;
                if (parent3ClassName === editorBrowser_1.ClassNames.VIEW_LINE) {
                    return {
                        position: this.viewHelper.getPositionFromDOMInfo(parent1, range.startOffset),
                        hitTarget: null
                    };
                }
                else {
                    hitTarget = startContainer.parentNode;
                }
            }
            else if (startContainer.nodeType === startContainer.ELEMENT_NODE) {
                // startContainer is expected to be the token span
                var parent1 = startContainer.parentNode; // expected to be the view line container span
                var parent2 = parent1 ? parent1.parentNode : null; // expected to be the view line div
                var parent2ClassName = parent2 && parent2.nodeType === parent2.ELEMENT_NODE ? parent2.className : null;
                if (parent2ClassName === editorBrowser_1.ClassNames.VIEW_LINE) {
                    return {
                        position: this.viewHelper.getPositionFromDOMInfo(startContainer, startContainer.textContent.length),
                        hitTarget: null
                    };
                }
                else {
                    hitTarget = startContainer;
                }
            }
            return {
                position: null,
                hitTarget: hitTarget
            };
        };
        /**
         * Most probably Gecko
         */
        MouseTargetFactory.prototype._doHitTestWithCaretPositionFromPoint = function (e) {
            var resultPosition = null;
            var resultHitTarget = null;
            var hitx = e.posx - document.body.scrollLeft - document.documentElement.scrollLeft;
            var hity = e.posy - document.body.scrollTop - document.documentElement.scrollTop;
            var hitResult = document.caretPositionFromPoint(hitx, hity);
            var range = document.createRange();
            range.setStart(hitResult.offsetNode, hitResult.offset);
            range.collapse(true);
            resultPosition = this.viewHelper.getPositionFromDOMInfo(range.startContainer.parentNode, range.startOffset);
            range.detach();
            return {
                position: resultPosition,
                hitTarget: resultHitTarget
            };
        };
        /**
         * Most probably IE
         */
        MouseTargetFactory.prototype._doHitTestWithMoveToPoint = function (e) {
            var resultPosition = null;
            var resultHitTarget = null;
            var textRange = document.body.createTextRange();
            try {
                var hitx = e.posx - document.body.scrollLeft - document.documentElement.scrollLeft;
                var hity = e.posy - document.body.scrollTop - document.documentElement.scrollTop;
                textRange.moveToPoint(hitx, hity);
            }
            catch (err) {
                return {
                    position: null,
                    hitTarget: null
                };
            }
            textRange.collapse(true);
            // Now, let's do our best to figure out what we hit :)
            var parentElement = textRange ? textRange.parentElement() : null;
            var parent1 = parentElement ? parentElement.parentNode : null;
            var parent2 = parent1 ? parent1.parentNode : null;
            var parent2ClassName = parent2 && parent2.nodeType === parent2.ELEMENT_NODE ? parent2.className : '';
            if (parent2ClassName === editorBrowser_1.ClassNames.VIEW_LINE) {
                var rangeToContainEntireSpan = textRange.duplicate();
                rangeToContainEntireSpan.moveToElementText(parentElement);
                rangeToContainEntireSpan.setEndPoint('EndToStart', textRange);
                resultPosition = this.viewHelper.getPositionFromDOMInfo(parentElement, rangeToContainEntireSpan.text.length);
                // Move range out of the span node, IE doesn't like having many ranges in
                // the same spot and will act badly for lines containing dashes ('-')
                rangeToContainEntireSpan.moveToElementText(this.viewHelper.viewDomNode);
            }
            else {
                // Looks like we've hit the hover or something foreign
                resultHitTarget = parentElement;
            }
            // Move range out of the span node, IE doesn't like having many ranges in
            // the same spot and will act badly for lines containing dashes ('-')
            textRange.moveToElementText(this.viewHelper.viewDomNode);
            return {
                position: resultPosition,
                hitTarget: resultHitTarget
            };
        };
        MouseTargetFactory.prototype._doHitTest = function (editorContent, e, mouseVerticalOffset) {
            // State of the art (18.10.2012):
            // The spec says browsers should support document.caretPositionFromPoint, but nobody implemented it (http://dev.w3.org/csswg/cssom-view/)
            // Gecko:
            //    - they tried to implement it once, but failed: https://bugzilla.mozilla.org/show_bug.cgi?id=654352
            //    - however, they do give out rangeParent/rangeOffset properties on mouse events
            // Webkit:
            //    - they have implemented a previous version of the spec which was using document.caretRangeFromPoint
            // IE:
            //    - they have a proprietary method on ranges, moveToPoint: http://msdn.microsoft.com/en-us/library/ie/ms536632(v=vs.85).aspx
            // Thank you browsers for making this so 'easy' :)
            if (document.caretRangeFromPoint) {
                return this._doHitTestWithCaretRangeFromPoint(editorContent, e, mouseVerticalOffset);
            }
            else if (document.caretPositionFromPoint) {
                return this._doHitTestWithCaretPositionFromPoint(e);
            }
            else if (document.body.createTextRange) {
                return this._doHitTestWithMoveToPoint(e);
            }
            return {
                position: null,
                hitTarget: null
            };
        };
        MouseTargetFactory.prototype._getZoneAtCoord = function (mouseVerticalOffset) {
            // The target is either a view zone or the empty space after the last view-line
            var viewZoneWhitespace = this.viewHelper.getWhitespaceAtVerticalOffset(mouseVerticalOffset);
            if (viewZoneWhitespace) {
                var viewZoneMiddle = viewZoneWhitespace.verticalOffset + viewZoneWhitespace.height / 2, lineCount = this.context.model.getLineCount(), positionBefore = null, position, positionAfter = null;
                if (viewZoneWhitespace.afterLineNumber !== lineCount) {
                    // There are more lines after this view zone
                    positionAfter = new position_1.Position(viewZoneWhitespace.afterLineNumber + 1, 1);
                }
                if (viewZoneWhitespace.afterLineNumber > 0) {
                    // There are more lines above this view zone
                    positionBefore = new position_1.Position(viewZoneWhitespace.afterLineNumber, this.context.model.getLineMaxColumn(viewZoneWhitespace.afterLineNumber));
                }
                if (positionAfter === null) {
                    position = positionBefore;
                }
                else if (positionBefore === null) {
                    position = positionAfter;
                }
                else if (mouseVerticalOffset < viewZoneMiddle) {
                    position = positionBefore;
                }
                else {
                    position = positionAfter;
                }
                return {
                    viewZoneId: viewZoneWhitespace.id,
                    afterLineNumber: viewZoneWhitespace.afterLineNumber,
                    positionBefore: positionBefore,
                    positionAfter: positionAfter,
                    position: position
                };
            }
            return null;
        };
        MouseTargetFactory.prototype._getFullLineRangeAtCoord = function (mouseVerticalOffset) {
            if (this.viewHelper.isAfterLines(mouseVerticalOffset)) {
                // Below the last line
                var lineNumber = this.context.model.getLineCount();
                var maxLineColumn = this.context.model.getLineMaxColumn(lineNumber);
                return {
                    range: new range_1.Range(lineNumber, maxLineColumn, lineNumber, maxLineColumn),
                    isAfterLines: true
                };
            }
            var lineNumber = this.viewHelper.getLineNumberAtVerticalOffset(mouseVerticalOffset);
            var maxLineColumn = this.context.model.getLineMaxColumn(lineNumber);
            return {
                range: new range_1.Range(lineNumber, 1, lineNumber, maxLineColumn),
                isAfterLines: false
            };
        };
        MouseTargetFactory.prototype.getMouseColumn = function (layoutInfo, editorContent, e) {
            var mouseContentHorizontalOffset = this.viewHelper.getScrollLeft() + (e.posx - editorContent.left) - layoutInfo.contentLeft;
            return this._getMouseColumn(mouseContentHorizontalOffset);
        };
        MouseTargetFactory.prototype._getMouseColumn = function (mouseContentHorizontalOffset) {
            if (mouseContentHorizontalOffset < 0) {
                return 1;
            }
            var charWidth = this.context.configuration.editor.typicalHalfwidthCharacterWidth;
            var chars = Math.round(mouseContentHorizontalOffset / charWidth);
            return (chars + 1);
        };
        MouseTargetFactory.prototype.createMouseTargetFromViewCursor = function (target, lineNumber, column, mouseColumn) {
            return new MouseTarget(target, editorCommon_1.MouseTargetType.CONTENT_TEXT, mouseColumn, new position_1.Position(lineNumber, column));
        };
        MouseTargetFactory.prototype.createMouseTargetFromViewLines = function (target, mouseVerticalOffset, mouseColumn) {
            // This most likely indicates it happened after the last view-line
            var lineCount = this.context.model.getLineCount();
            var maxLineColumn = this.context.model.getLineMaxColumn(lineCount);
            return new MouseTarget(target, editorCommon_1.MouseTargetType.CONTENT_EMPTY, mouseColumn, new position_1.Position(lineCount, maxLineColumn));
        };
        MouseTargetFactory.prototype.createMouseTargetFromHitTestPosition = function (target, lineNumber, column, mouseHorizontalOffset, mouseColumn) {
            var pos = new position_1.Position(lineNumber, column);
            var lineWidth = this.viewHelper.getLineWidth(lineNumber);
            if (mouseHorizontalOffset > lineWidth) {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.CONTENT_EMPTY, mouseColumn, pos);
            }
            var visibleRange = this.viewHelper.visibleRangeForPosition2(lineNumber, column);
            if (!visibleRange) {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.UNKNOWN, mouseColumn, pos);
            }
            var columnHorizontalOffset = visibleRange.left;
            if (mouseHorizontalOffset === columnHorizontalOffset) {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.CONTENT_TEXT, mouseColumn, pos);
            }
            var mouseIsBetween;
            if (column > 1) {
                var prevColumnHorizontalOffset = visibleRange.left;
                mouseIsBetween = false;
                mouseIsBetween = mouseIsBetween || (prevColumnHorizontalOffset < mouseHorizontalOffset && mouseHorizontalOffset < columnHorizontalOffset); // LTR case
                mouseIsBetween = mouseIsBetween || (columnHorizontalOffset < mouseHorizontalOffset && mouseHorizontalOffset < prevColumnHorizontalOffset); // RTL case
                if (mouseIsBetween) {
                    var rng = new range_1.Range(lineNumber, column, lineNumber, column - 1);
                    return new MouseTarget(target, editorCommon_1.MouseTargetType.CONTENT_TEXT, mouseColumn, pos, rng);
                }
            }
            var lineMaxColumn = this.context.model.getLineMaxColumn(lineNumber);
            if (column < lineMaxColumn) {
                var nextColumnVisibleRange = this.viewHelper.visibleRangeForPosition2(lineNumber, column + 1);
                if (nextColumnVisibleRange) {
                    var nextColumnHorizontalOffset = nextColumnVisibleRange.left;
                    mouseIsBetween = false;
                    mouseIsBetween = mouseIsBetween || (columnHorizontalOffset < mouseHorizontalOffset && mouseHorizontalOffset < nextColumnHorizontalOffset); // LTR case
                    mouseIsBetween = mouseIsBetween || (nextColumnHorizontalOffset < mouseHorizontalOffset && mouseHorizontalOffset < columnHorizontalOffset); // RTL case
                    if (mouseIsBetween) {
                        var rng = new range_1.Range(lineNumber, column, lineNumber, column + 1);
                        return new MouseTarget(target, editorCommon_1.MouseTargetType.CONTENT_TEXT, mouseColumn, pos, rng);
                    }
                }
            }
            return new MouseTarget(target, editorCommon_1.MouseTargetType.CONTENT_TEXT, mouseColumn, pos);
        };
        MouseTargetFactory.prototype.createMouseTargetFromContentWidgetsChild = function (target, mouseColumn) {
            var widgetId = this._findAttribute(target, 'widgetId', this.viewHelper.viewDomNode);
            if (widgetId) {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.CONTENT_WIDGET, mouseColumn, null, null, widgetId);
            }
            else {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.UNKNOWN);
            }
        };
        MouseTargetFactory.prototype.createMouseTargetFromOverlayWidgetsChild = function (target, mouseColumn) {
            var widgetId = this._findAttribute(target, 'widgetId', this.viewHelper.viewDomNode);
            if (widgetId) {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.OVERLAY_WIDGET, mouseColumn, null, null, widgetId);
            }
            else {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.UNKNOWN);
            }
        };
        MouseTargetFactory.prototype.createMouseTargetFromLinesDecorationsChild = function (target, mouseVerticalOffset, mouseColumn) {
            var viewZoneData = this._getZoneAtCoord(mouseVerticalOffset);
            if (viewZoneData) {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.GUTTER_VIEW_ZONE, mouseColumn, viewZoneData.position, null, viewZoneData);
            }
            var res = this._getFullLineRangeAtCoord(mouseVerticalOffset);
            return new MouseTarget(target, editorCommon_1.MouseTargetType.GUTTER_LINE_DECORATIONS, mouseColumn, new position_1.Position(res.range.startLineNumber, res.range.startColumn), res.range, res.isAfterLines);
        };
        MouseTargetFactory.prototype.createMouseTargetFromLineNumbers = function (target, mouseVerticalOffset, mouseColumn) {
            var viewZoneData = this._getZoneAtCoord(mouseVerticalOffset);
            if (viewZoneData) {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.GUTTER_VIEW_ZONE, mouseColumn, viewZoneData.position, null, viewZoneData);
            }
            var res = this._getFullLineRangeAtCoord(mouseVerticalOffset);
            return new MouseTarget(target, editorCommon_1.MouseTargetType.GUTTER_LINE_NUMBERS, mouseColumn, new position_1.Position(res.range.startLineNumber, res.range.startColumn), res.range, res.isAfterLines);
        };
        MouseTargetFactory.prototype.createMouseTargetFromGlyphMargin = function (target, mouseVerticalOffset, mouseColumn) {
            var viewZoneData = this._getZoneAtCoord(mouseVerticalOffset);
            if (viewZoneData) {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.GUTTER_VIEW_ZONE, mouseColumn, viewZoneData.position, null, viewZoneData);
            }
            var res = this._getFullLineRangeAtCoord(mouseVerticalOffset);
            return new MouseTarget(target, editorCommon_1.MouseTargetType.GUTTER_GLYPH_MARGIN, mouseColumn, new position_1.Position(res.range.startLineNumber, res.range.startColumn), res.range, res.isAfterLines);
        };
        MouseTargetFactory.prototype.createMouseTargetFromScrollbar = function (target, mouseVerticalOffset, mouseColumn) {
            var possibleLineNumber = this.viewHelper.getLineNumberAtVerticalOffset(mouseVerticalOffset);
            var maxColumn = this.context.model.getLineMaxColumn(possibleLineNumber);
            return new MouseTarget(target, editorCommon_1.MouseTargetType.SCROLLBAR, mouseColumn, new position_1.Position(possibleLineNumber, maxColumn));
        };
        MouseTargetFactory.prototype.createMouseTargetFromUnknownTarget = function (target) {
            var isInView = this._isChild(target, this.viewHelper.viewDomNode, this.viewHelper.viewDomNode);
            var widgetId = null;
            if (isInView) {
                widgetId = this._findAttribute(target, 'widgetId', this.viewHelper.viewDomNode);
            }
            if (widgetId) {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.OVERLAY_WIDGET, null, null, widgetId);
            }
            else {
                return new MouseTarget(target, editorCommon_1.MouseTargetType.UNKNOWN);
            }
        };
        return MouseTargetFactory;
    }());
    exports.MouseTargetFactory = MouseTargetFactory;
});

define("vs/editor/browser/standalone/colorizer", ["require", "exports", 'vs/base/common/async', 'vs/base/common/winjs.base', 'vs/editor/common/editorCommon', 'vs/editor/common/viewLayout/viewLineRenderer'], function (require, exports, async_1, winjs_base_1, editorCommon_1, viewLineRenderer_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Colorizer = (function () {
        function Colorizer() {
        }
        Colorizer.colorizeElement = function (modeService, domNode, options) {
            options = options || {};
            var theme = options.theme || 'vs';
            var mimeType = options.mimeType || domNode.getAttribute('lang') || domNode.getAttribute('data-lang');
            if (!mimeType) {
                console.error('Mode not detected');
                return;
            }
            var text = domNode.firstChild.nodeValue;
            domNode.className += 'monaco-editor ' + theme;
            var render = function (str) {
                domNode.innerHTML = str;
            };
            return this.colorize(modeService, text, mimeType, options).then(render, function (err) { return console.error(err); }, render);
        };
        Colorizer.colorize = function (modeService, text, mimeType, options) {
            options = options || {};
            if (typeof options.tabSize === 'undefined') {
                options.tabSize = 4;
            }
            var lines = text.split('\n'), c, e, p, isCancelled = false, mode;
            var result = new winjs_base_1.TPromise(function (_c, _e, _p) {
                c = _c;
                e = _e;
                p = _p;
            }, function () {
                isCancelled = true;
            });
            var colorize = new async_1.RunOnceScheduler(function () {
                if (isCancelled) {
                    return;
                }
                var r = actualColorize(lines, mode, options.tabSize);
                if (r.retokenize.length > 0) {
                    // There are retokenization requests
                    r.retokenize.forEach(function (p) { return p.then(scheduleColorize); });
                    p(r.result);
                }
                else {
                    // There are no (more) retokenization requests
                    c(r.result);
                }
            }, 0);
            var scheduleColorize = function () { return colorize.schedule(); };
            modeService.getOrCreateMode(mimeType).then(function (_mode) {
                if (!_mode) {
                    e('Mode not found: "' + mimeType + '".');
                    return;
                }
                if (!_mode.tokenizationSupport) {
                    e('Mode found ("' + _mode.getId() + '"), but does not support tokenization.');
                    return;
                }
                mode = _mode;
                scheduleColorize();
            });
            return result;
        };
        Colorizer.colorizeLine = function (line, tokens, tabSize) {
            if (tabSize === void 0) { tabSize = 4; }
            var renderResult = viewLineRenderer_1.renderLine({
                lineContent: line,
                parts: tokens,
                stopRenderingLineAfter: -1,
                renderWhitespace: false,
                tabSize: tabSize
            });
            return renderResult.output.join('');
        };
        Colorizer.colorizeModelLine = function (model, lineNumber, tabSize) {
            if (tabSize === void 0) { tabSize = 4; }
            var content = model.getLineContent(lineNumber);
            var tokens = model.getLineTokens(lineNumber, false);
            var inflatedTokens = editorCommon_1.LineTokensBinaryEncoding.inflateArr(tokens.getBinaryEncodedTokensMap(), tokens.getBinaryEncodedTokens());
            return this.colorizeLine(content, inflatedTokens, tabSize);
        };
        return Colorizer;
    }());
    exports.Colorizer = Colorizer;
    function actualColorize(lines, mode, tabSize) {
        var tokenization = mode.tokenizationSupport, html = [], state = tokenization.getInitialState(), i, length, line, tokenizeResult, renderResult, retokenize = [];
        for (i = 0, length = lines.length; i < length; i++) {
            line = lines[i];
            tokenizeResult = tokenization.tokenize(line, state);
            if (tokenizeResult.retokenize) {
                retokenize.push(tokenizeResult.retokenize);
            }
            renderResult = viewLineRenderer_1.renderLine({
                lineContent: line,
                parts: tokenizeResult.tokens,
                stopRenderingLineAfter: -1,
                renderWhitespace: false,
                tabSize: tabSize
            });
            html = html.concat(renderResult.output);
            html.push('<br/>');
            state = tokenizeResult.endState;
        }
        return {
            result: html.join(''),
            retokenize: retokenize
        };
    }
});

define("vs/editor/browser/view/viewController", ["require", "exports", 'vs/editor/common/core/position', 'vs/editor/common/editorCommon'], function (require, exports, position_1, editorCommon) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewController = (function () {
        function ViewController(viewModel, configuration, outgoingEventBus, keybindingService) {
            this.viewModel = viewModel;
            this.configuration = configuration;
            this.outgoingEventBus = outgoingEventBus;
            this.keybindingService = keybindingService;
        }
        ViewController.prototype.paste = function (source, text, pasteOnNewLine) {
            this.keybindingService.executeCommand(editorCommon.Handler.Paste, {
                text: text,
                pasteOnNewLine: pasteOnNewLine,
            });
        };
        ViewController.prototype.type = function (source, text) {
            this.keybindingService.executeCommand(editorCommon.Handler.Type, {
                text: text
            });
        };
        ViewController.prototype.replacePreviousChar = function (source, text, replaceCharCnt) {
            this.keybindingService.executeCommand(editorCommon.Handler.ReplacePreviousChar, {
                text: text,
                replaceCharCnt: replaceCharCnt
            });
        };
        ViewController.prototype.cut = function (source) {
            this.keybindingService.executeCommand(editorCommon.Handler.Cut, {});
        };
        ViewController.prototype._validateViewColumn = function (viewPosition) {
            var minColumn = this.viewModel.getLineMinColumn(viewPosition.lineNumber);
            if (viewPosition.column < minColumn) {
                return new position_1.Position(viewPosition.lineNumber, minColumn);
            }
            return viewPosition;
        };
        ViewController.prototype.dispatchMouse = function (data) {
            if (data.startedOnLineNumbers) {
                // If the dragging started on the gutter, then have operations work on the entire line
                if (data.altKey) {
                    if (data.inSelectionMode) {
                        this.lastCursorLineSelect('mouse', data.position);
                    }
                    else {
                        this.createCursor('mouse', data.position, true);
                    }
                }
                else {
                    if (data.inSelectionMode) {
                        this.lineSelectDrag('mouse', data.position);
                    }
                    else {
                        this.lineSelect('mouse', data.position);
                    }
                }
            }
            else if (data.mouseDownCount >= 4) {
                this.selectAll('mouse');
            }
            else if (data.mouseDownCount === 3) {
                if (data.altKey) {
                    if (data.inSelectionMode) {
                        this.lastCursorLineSelectDrag('mouse', data.position);
                    }
                    else {
                        this.lastCursorLineSelect('mouse', data.position);
                    }
                }
                else {
                    if (data.inSelectionMode) {
                        this.lineSelectDrag('mouse', data.position);
                    }
                    else {
                        this.lineSelect('mouse', data.position);
                    }
                }
            }
            else if (data.mouseDownCount === 2) {
                if (data.altKey) {
                    this.lastCursorWordSelect('mouse', data.position);
                }
                else {
                    if (data.inSelectionMode) {
                        this.wordSelectDrag('mouse', data.position);
                    }
                    else {
                        this.wordSelect('mouse', data.position);
                    }
                }
            }
            else {
                if (data.altKey) {
                    if (!data.ctrlKey && !data.metaKey) {
                        if (data.shiftKey) {
                            this.columnSelect('mouse', data.position, data.mouseColumn);
                        }
                        else {
                            // Do multi-cursor operations only when purely alt is pressed
                            if (data.inSelectionMode) {
                                this.lastCursorMoveToSelect('mouse', data.position);
                            }
                            else {
                                this.createCursor('mouse', data.position, false);
                            }
                        }
                    }
                }
                else {
                    if (data.inSelectionMode) {
                        this.moveToSelect('mouse', data.position);
                    }
                    else {
                        this.moveTo('mouse', data.position);
                    }
                }
            }
        };
        ViewController.prototype.moveTo = function (source, viewPosition) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.MoveTo, {
                position: this.convertViewToModelPosition(viewPosition),
                viewPosition: viewPosition
            });
        };
        ViewController.prototype.moveToSelect = function (source, viewPosition) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.MoveToSelect, {
                position: this.convertViewToModelPosition(viewPosition),
                viewPosition: viewPosition
            });
        };
        ViewController.prototype.columnSelect = function (source, viewPosition, mouseColumn) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.ColumnSelect, {
                position: this.convertViewToModelPosition(viewPosition),
                viewPosition: viewPosition,
                mouseColumn: mouseColumn
            });
        };
        ViewController.prototype.createCursor = function (source, viewPosition, wholeLine) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.CreateCursor, {
                position: this.convertViewToModelPosition(viewPosition),
                viewPosition: viewPosition,
                wholeLine: wholeLine
            });
        };
        ViewController.prototype.lastCursorMoveToSelect = function (source, viewPosition) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.LastCursorMoveToSelect, {
                position: this.convertViewToModelPosition(viewPosition),
                viewPosition: viewPosition
            });
        };
        ViewController.prototype.wordSelect = function (source, viewPosition) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.WordSelect, {
                position: this.convertViewToModelPosition(viewPosition)
            });
        };
        ViewController.prototype.wordSelectDrag = function (source, viewPosition) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.WordSelectDrag, {
                position: this.convertViewToModelPosition(viewPosition)
            });
        };
        ViewController.prototype.lastCursorWordSelect = function (source, viewPosition) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.LastCursorWordSelect, {
                position: this.convertViewToModelPosition(viewPosition)
            });
        };
        ViewController.prototype.lineSelect = function (source, viewPosition) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.LineSelect, {
                position: this.convertViewToModelPosition(viewPosition),
                viewPosition: viewPosition
            });
        };
        ViewController.prototype.lineSelectDrag = function (source, viewPosition) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.LineSelectDrag, {
                position: this.convertViewToModelPosition(viewPosition),
                viewPosition: viewPosition
            });
        };
        ViewController.prototype.lastCursorLineSelect = function (source, viewPosition) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.LastCursorLineSelect, {
                position: this.convertViewToModelPosition(viewPosition),
                viewPosition: viewPosition
            });
        };
        ViewController.prototype.lastCursorLineSelectDrag = function (source, viewPosition) {
            viewPosition = this._validateViewColumn(viewPosition);
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.LastCursorLineSelectDrag, {
                position: this.convertViewToModelPosition(viewPosition),
                viewPosition: viewPosition
            });
        };
        ViewController.prototype.selectAll = function (source) {
            this.configuration.handlerDispatcher.trigger(source, editorCommon.Handler.SelectAll, null);
        };
        // ----------------------
        ViewController.prototype.convertViewToModelPosition = function (viewPosition) {
            return this.viewModel.convertViewPositionToModelPosition(viewPosition.lineNumber, viewPosition.column);
        };
        ViewController.prototype.convertViewToModelRange = function (viewRange) {
            return this.viewModel.convertViewRangeToModelRange(viewRange);
        };
        ViewController.prototype.convertViewToModelMouseEvent = function (e) {
            if (e.target) {
                if (e.target.position) {
                    e.target.position = this.convertViewToModelPosition(e.target.position);
                }
                if (e.target.range) {
                    e.target.range = this.convertViewToModelRange(e.target.range);
                }
            }
        };
        ViewController.prototype.emitKeyDown = function (e) {
            this.outgoingEventBus.emit(editorCommon.EventType.KeyDown, e);
        };
        ViewController.prototype.emitKeyUp = function (e) {
            this.outgoingEventBus.emit(editorCommon.EventType.KeyUp, e);
        };
        ViewController.prototype.emitContextMenu = function (e) {
            this.convertViewToModelMouseEvent(e);
            this.outgoingEventBus.emit(editorCommon.EventType.ContextMenu, e);
        };
        ViewController.prototype.emitMouseMove = function (e) {
            this.convertViewToModelMouseEvent(e);
            this.outgoingEventBus.emit(editorCommon.EventType.MouseMove, e);
        };
        ViewController.prototype.emitMouseLeave = function (e) {
            this.convertViewToModelMouseEvent(e);
            this.outgoingEventBus.emit(editorCommon.EventType.MouseLeave, e);
        };
        ViewController.prototype.emitMouseUp = function (e) {
            this.convertViewToModelMouseEvent(e);
            this.outgoingEventBus.emit(editorCommon.EventType.MouseUp, e);
        };
        ViewController.prototype.emitMouseDown = function (e) {
            this.convertViewToModelMouseEvent(e);
            this.outgoingEventBus.emit(editorCommon.EventType.MouseDown, e);
        };
        return ViewController;
    }());
    exports.ViewController = ViewController;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/lines/viewLine", ["require", "exports", 'vs/base/browser/browser', 'vs/base/browser/styleMutator', 'vs/editor/common/editorCommon', 'vs/editor/common/viewLayout/viewLineParts', 'vs/editor/common/viewLayout/viewLineRenderer', 'vs/editor/browser/editorBrowser'], function (require, exports, browser, styleMutator_1, editorCommon_1, viewLineParts_1, viewLineRenderer_1, editorBrowser_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewLine = (function () {
        function ViewLine(context) {
            this._context = context;
            this._domNode = null;
            this._isInvalid = true;
            this._isMaybeInvalid = false;
            this._lineParts = null;
            this._charOffsetInPart = [];
            this._lastRenderedPartIndex = 0;
        }
        // --- begin IVisibleLineData
        ViewLine.prototype.getDomNode = function () {
            return this._domNode;
        };
        ViewLine.prototype.setDomNode = function (domNode) {
            this._domNode = domNode;
        };
        ViewLine.prototype.onContentChanged = function () {
            this._isInvalid = true;
        };
        ViewLine.prototype.onLinesInsertedAbove = function () {
            this._isMaybeInvalid = true;
        };
        ViewLine.prototype.onLinesDeletedAbove = function () {
            this._isMaybeInvalid = true;
        };
        ViewLine.prototype.onLineChangedAbove = function () {
            this._isMaybeInvalid = true;
        };
        ViewLine.prototype.onTokensChanged = function () {
            this._isMaybeInvalid = true;
        };
        ViewLine.prototype.onModelDecorationsChanged = function () {
            this._isMaybeInvalid = true;
        };
        ViewLine.prototype.onConfigurationChanged = function (e) {
            this._isInvalid = true;
        };
        ViewLine.prototype.shouldUpdateHTML = function (lineNumber, inlineDecorations) {
            var newLineParts = null;
            if (this._isMaybeInvalid || this._isInvalid) {
                // Compute new line parts only if there is some evidence that something might have changed
                newLineParts = viewLineParts_1.createLineParts(lineNumber, this._context.model.getLineMinColumn(lineNumber), this._context.model.getLineContent(lineNumber), this._context.model.getLineTokens(lineNumber), inlineDecorations, this._context.configuration.editor.renderWhitespace);
            }
            // Decide if isMaybeInvalid flips isInvalid to true
            if (this._isMaybeInvalid) {
                if (!this._isInvalid) {
                    if (!this._lineParts || !this._lineParts.equals(newLineParts)) {
                        this._isInvalid = true;
                    }
                }
                this._isMaybeInvalid = false;
            }
            if (this._isInvalid) {
                this._lineParts = newLineParts;
            }
            return this._isInvalid;
        };
        ViewLine.prototype.getLineOuterHTML = function (out, lineNumber, deltaTop) {
            out.push('<div lineNumber="');
            out.push(lineNumber.toString());
            out.push('" style="top:');
            out.push(deltaTop.toString());
            out.push('px;height:');
            out.push(this._context.configuration.editor.lineHeight.toString());
            out.push('px;" class="');
            out.push(editorBrowser_1.ClassNames.VIEW_LINE);
            out.push('">');
            out.push(this.getLineInnerHTML(lineNumber));
            out.push('</div>');
        };
        ViewLine.prototype.getLineInnerHTML = function (lineNumber) {
            this._isInvalid = false;
            return this._render(lineNumber, this._lineParts).join('');
        };
        ViewLine.prototype.layoutLine = function (lineNumber, deltaTop) {
            var desiredLineNumber = String(lineNumber);
            var currentLineNumber = this._domNode.getAttribute('lineNumber');
            if (currentLineNumber !== desiredLineNumber) {
                this._domNode.setAttribute('lineNumber', desiredLineNumber);
            }
            styleMutator_1.StyleMutator.setTop(this._domNode, deltaTop);
            styleMutator_1.StyleMutator.setHeight(this._domNode, this._context.configuration.editor.lineHeight);
        };
        // --- end IVisibleLineData
        ViewLine.prototype._render = function (lineNumber, lineParts) {
            this._cachedWidth = -1;
            var r = viewLineRenderer_1.renderLine({
                lineContent: this._context.model.getLineContent(lineNumber),
                tabSize: this._context.model.getTabSize(),
                stopRenderingLineAfter: this._context.configuration.editor.stopRenderingLineAfter,
                renderWhitespace: this._context.configuration.editor.renderWhitespace,
                parts: lineParts.getParts()
            });
            this._charOffsetInPart = r.charOffsetInPart;
            this._lastRenderedPartIndex = r.lastRenderedPartIndex;
            return r.output;
        };
        // --- Reading from the DOM methods
        ViewLine.prototype._getReadingTarget = function () {
            return this._domNode.firstChild;
        };
        /**
         * Width of the line in pixels
         */
        ViewLine.prototype.getWidth = function () {
            if (this._cachedWidth === -1) {
                this._cachedWidth = this._getReadingTarget().offsetWidth;
            }
            return this._cachedWidth;
        };
        /**
         * Visible ranges for a model range
         */
        ViewLine.prototype.getVisibleRangesForRange = function (startColumn, endColumn, clientRectDeltaLeft, endNode) {
            var stopRenderingLineAfter = this._context.configuration.editor.stopRenderingLineAfter;
            if (stopRenderingLineAfter !== -1 && startColumn > stopRenderingLineAfter && endColumn > stopRenderingLineAfter) {
                // This range is obviously not visible
                return null;
            }
            if (stopRenderingLineAfter !== -1 && startColumn > stopRenderingLineAfter) {
                startColumn = stopRenderingLineAfter;
            }
            if (stopRenderingLineAfter !== -1 && endColumn > stopRenderingLineAfter) {
                endColumn = stopRenderingLineAfter;
            }
            return this._readVisibleRangesForRange(startColumn, endColumn, clientRectDeltaLeft, endNode);
        };
        ViewLine.prototype._readVisibleRangesForRange = function (startColumn, endColumn, clientRectDeltaLeft, endNode) {
            var result;
            if (startColumn === endColumn) {
                result = this._readRawVisibleRangesForPosition(startColumn, clientRectDeltaLeft, endNode);
            }
            else {
                result = this._readRawVisibleRangesForRange(startColumn, endColumn, clientRectDeltaLeft, endNode);
            }
            if (!result || result.length <= 1) {
                return result;
            }
            result.sort(compareVisibleRanges);
            var output = [];
            var prevRange = result[0];
            for (var i = 1, len = result.length; i < len; i++) {
                var currRange = result[i];
                if (prevRange.left + prevRange.width + 0.9 /* account for browser's rounding errors*/ >= currRange.left) {
                    prevRange.width = Math.max(prevRange.width, currRange.left + currRange.width - prevRange.left);
                }
                else {
                    output.push(prevRange);
                    prevRange = currRange;
                }
            }
            output.push(prevRange);
            return output;
        };
        ViewLine.prototype._readRawVisibleRangesForPosition = function (column, clientRectDeltaLeft, endNode) {
            if (this._charOffsetInPart.length === 0) {
                // This line is empty
                return [new editorCommon_1.HorizontalRange(0, 0)];
            }
            var partIndex = findIndexInArrayWithMax(this._lineParts, column - 1, this._lastRenderedPartIndex);
            var charOffsetInPart = this._charOffsetInPart[column - 1];
            return this._readRawVisibleRangesFrom(this._getReadingTarget(), partIndex, charOffsetInPart, partIndex, charOffsetInPart, clientRectDeltaLeft, endNode);
        };
        ViewLine.prototype._readRawVisibleRangesForRange = function (startColumn, endColumn, clientRectDeltaLeft, endNode) {
            if (startColumn === 1 && endColumn === this._charOffsetInPart.length) {
                // This branch helps IE with bidi text & gives a performance boost to other browsers when reading visible ranges for an entire line
                return [this._readRawVisibleRangeForEntireLine()];
            }
            var startPartIndex = findIndexInArrayWithMax(this._lineParts, startColumn - 1, this._lastRenderedPartIndex);
            var startCharOffsetInPart = this._charOffsetInPart[startColumn - 1];
            var endPartIndex = findIndexInArrayWithMax(this._lineParts, endColumn - 1, this._lastRenderedPartIndex);
            var endCharOffsetInPart = this._charOffsetInPart[endColumn - 1];
            return this._readRawVisibleRangesFrom(this._getReadingTarget(), startPartIndex, startCharOffsetInPart, endPartIndex, endCharOffsetInPart, clientRectDeltaLeft, endNode);
        };
        ViewLine.prototype._readRawVisibleRangeForEntireLine = function () {
            return new editorCommon_1.HorizontalRange(0, this._getReadingTarget().offsetWidth);
        };
        ViewLine.prototype._readRawVisibleRangesFrom = function (domNode, startChildIndex, startOffset, endChildIndex, endOffset, clientRectDeltaLeft, endNode) {
            var range = RangeUtil.createRange();
            try {
                // Panic check
                var min = 0;
                var max = domNode.children.length - 1;
                if (min > max) {
                    return null;
                }
                startChildIndex = Math.min(max, Math.max(min, startChildIndex));
                endChildIndex = Math.min(max, Math.max(min, endChildIndex));
                // If crossing over to a span only to select offset 0, then use the previous span's maximum offset
                // Chrome is buggy and doesn't handle 0 offsets well sometimes.
                if (startChildIndex !== endChildIndex) {
                    if (endChildIndex > 0 && endOffset === 0) {
                        endChildIndex--;
                        endOffset = Number.MAX_VALUE;
                    }
                }
                var startElement = domNode.children[startChildIndex].firstChild;
                var endElement = domNode.children[endChildIndex].firstChild;
                if (!startElement || !endElement) {
                    return null;
                }
                startOffset = Math.min(startElement.textContent.length, Math.max(0, startOffset));
                endOffset = Math.min(endElement.textContent.length, Math.max(0, endOffset));
                range.setStart(startElement, startOffset);
                range.setEnd(endElement, endOffset);
                var clientRects = range.getClientRects();
                if (clientRects.length === 0) {
                    return null;
                }
                return this._createRawVisibleRangesFromClientRects(clientRects, clientRectDeltaLeft);
            }
            catch (e) {
                // This is life ...
                return null;
            }
            finally {
                RangeUtil.detachRange(range, endNode);
            }
        };
        ViewLine.prototype._createRawVisibleRangesFromClientRects = function (clientRects, clientRectDeltaLeft) {
            var result = [];
            for (var i = 0, len = clientRects.length; i < len; i++) {
                var cR = clientRects[i];
                result.push(new editorCommon_1.HorizontalRange(Math.max(0, cR.left - clientRectDeltaLeft), cR.width));
            }
            return result;
        };
        /**
         * Returns the column for the text found at a specific offset inside a rendered dom node
         */
        ViewLine.prototype.getColumnOfNodeOffset = function (lineNumber, spanNode, offset) {
            var spanIndex = -1;
            while (spanNode) {
                spanNode = spanNode.previousSibling;
                spanIndex++;
            }
            var lineParts = this._lineParts.getParts();
            if (spanIndex >= lineParts.length) {
                return this._context.configuration.editor.stopRenderingLineAfter;
            }
            if (offset === 0) {
                return lineParts[spanIndex].startIndex + 1;
            }
            var originalMin = lineParts[spanIndex].startIndex;
            var originalMax;
            var originalMaxStartOffset;
            if (spanIndex + 1 < lineParts.length) {
                // Stop searching characters at the beginning of the next part
                originalMax = lineParts[spanIndex + 1].startIndex;
                originalMaxStartOffset = this._charOffsetInPart[originalMax - 1] + this._charOffsetInPart[originalMax];
            }
            else {
                originalMax = this._context.model.getLineMaxColumn(lineNumber) - 1;
                originalMaxStartOffset = this._charOffsetInPart[originalMax];
            }
            var min = originalMin;
            var max = originalMax;
            if (this._context.configuration.editor.stopRenderingLineAfter !== -1) {
                max = Math.min(this._context.configuration.editor.stopRenderingLineAfter - 1, originalMax);
            }
            var nextStartOffset;
            var prevStartOffset;
            // Here are the variables and their relation plotted on an axis
            // prevStartOffset    a    midStartOffset    b    nextStartOffset
            // ------|------------|----------|-----------|-----------|--------->
            // Everything in (a;b] will match mid
            while (min < max) {
                var mid = Math.floor((min + max) / 2);
                var midStartOffset = this._charOffsetInPart[mid];
                if (mid === originalMax) {
                    // Using Number.MAX_VALUE to ensure that any offset after midStartOffset will match mid
                    nextStartOffset = Number.MAX_VALUE;
                }
                else if (mid + 1 === originalMax) {
                    // mid + 1 is already in next part and might have the _charOffsetInPart = 0
                    nextStartOffset = originalMaxStartOffset;
                }
                else {
                    nextStartOffset = this._charOffsetInPart[mid + 1];
                }
                if (mid === originalMin) {
                    // Using Number.MIN_VALUE to ensure that any offset before midStartOffset will match mid
                    prevStartOffset = Number.MIN_VALUE;
                }
                else {
                    prevStartOffset = this._charOffsetInPart[mid - 1];
                }
                var a = (prevStartOffset + midStartOffset) / 2;
                var b = (midStartOffset + nextStartOffset) / 2;
                if (a < offset && offset <= b) {
                    // Hit!
                    return mid + 1;
                }
                if (offset <= a) {
                    max = mid - 1;
                }
                else {
                    min = mid + 1;
                }
            }
            return min + 1;
        };
        return ViewLine;
    }());
    exports.ViewLine = ViewLine;
    var IEViewLine = (function (_super) {
        __extends(IEViewLine, _super);
        function IEViewLine(context) {
            _super.call(this, context);
        }
        IEViewLine.prototype._createRawVisibleRangesFromClientRects = function (clientRects, clientRectDeltaLeft) {
            var ratioX = screen.logicalXDPI / screen.deviceXDPI;
            var result = [];
            for (var i = 0, len = clientRects.length; i < len; i++) {
                var cR = clientRects[i];
                result[i] = new editorCommon_1.HorizontalRange(Math.max(0, cR.left * ratioX - clientRectDeltaLeft), cR.width * ratioX);
            }
            return result;
        };
        return IEViewLine;
    }(ViewLine));
    var WebKitViewLine = (function (_super) {
        __extends(WebKitViewLine, _super);
        function WebKitViewLine(context) {
            _super.call(this, context);
        }
        WebKitViewLine.prototype._readVisibleRangesForRange = function (startColumn, endColumn, clientRectDeltaLeft, endNode) {
            var output = _super.prototype._readVisibleRangesForRange.call(this, startColumn, endColumn, clientRectDeltaLeft, endNode);
            if (this._context.configuration.editor.fontLigatures && output.length === 1 && endColumn > 1 && endColumn === this._charOffsetInPart.length) {
                var lastSpanBoundingClientRect = this._getReadingTarget().lastChild.getBoundingClientRect();
                var lastSpanBoundingClientRectRight = lastSpanBoundingClientRect.right - clientRectDeltaLeft;
                if (startColumn === endColumn) {
                    output[0].left = lastSpanBoundingClientRectRight;
                    output[0].width = 0;
                }
                else {
                    output[0].width = lastSpanBoundingClientRectRight - output[0].left;
                }
                return output;
            }
            if (!output || output.length === 0 || startColumn === endColumn || (startColumn === 1 && endColumn === this._charOffsetInPart.length)) {
                return output;
            }
            // WebKit is buggy and returns an expanded range (to contain words in some cases)
            // The last client rect is enlarged (I think)
            // This is an attempt to patch things up
            // Find position of previous column
            var beforeEndVisibleRanges = this._readRawVisibleRangesForPosition(endColumn - 1, clientRectDeltaLeft, endNode);
            // Find position of last column
            var endVisibleRanges = this._readRawVisibleRangesForPosition(endColumn, clientRectDeltaLeft, endNode);
            if (beforeEndVisibleRanges && beforeEndVisibleRanges.length > 0 && endVisibleRanges && endVisibleRanges.length > 0) {
                var beforeEndVisibleRange = beforeEndVisibleRanges[0];
                var endVisibleRange = endVisibleRanges[0];
                var isLTR = (beforeEndVisibleRange.left <= endVisibleRange.left);
                var lastRange = output[output.length - 1];
                if (isLTR && lastRange.left < endVisibleRange.left) {
                    // Trim down the width of the last visible range to not go after the last column's position
                    lastRange.width = endVisibleRange.left - lastRange.left;
                }
            }
            return output;
        };
        return WebKitViewLine;
    }(ViewLine));
    var RangeUtil = (function () {
        function RangeUtil() {
        }
        RangeUtil.createRange = function () {
            if (!RangeUtil._handyReadyRange) {
                RangeUtil._handyReadyRange = document.createRange();
            }
            return RangeUtil._handyReadyRange;
        };
        RangeUtil.detachRange = function (range, endNode) {
            // Move range out of the span node, IE doesn't like having many ranges in
            // the same spot and will act badly for lines containing dashes ('-')
            range.selectNodeContents(endNode);
        };
        return RangeUtil;
    }());
    function compareVisibleRanges(a, b) {
        return a.left - b.left;
    }
    function findIndexInArrayWithMax(lineParts, desiredIndex, maxResult) {
        var r = lineParts.findIndexOfOffset(desiredIndex);
        return r <= maxResult ? r : maxResult;
    }
    exports.createLine = (function () {
        if (window.screen && window.screen.deviceXDPI && (navigator.userAgent.indexOf('Trident/6.0') >= 0 || navigator.userAgent.indexOf('Trident/5.0') >= 0)) {
            // IE11 doesn't need the screen.logicalXDPI / screen.deviceXDPI ratio multiplication
            // for TextRange.getClientRects() anymore
            return createIELine;
        }
        else if (browser.isWebKit) {
            return createWebKitLine;
        }
        return createNormalLine;
    })();
    function createIELine(context) {
        return new IEViewLine(context);
    }
    function createWebKitLine(context) {
        return new WebKitViewLine(context);
    }
    function createNormalLine(context) {
        return new ViewLine(context);
    }
});

define("vs/editor/browser/viewParts/overviewRuler/overviewRulerImpl", ["require", "exports", 'vs/base/browser/browser', 'vs/base/browser/styleMutator', 'vs/editor/common/editorCommon'], function (require, exports, browser, styleMutator_1, editorCommon_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function zoneEquals(a, b) {
        return (a.startLineNumber === b.startLineNumber
            && a.endLineNumber === b.endLineNumber
            && a.forceHeight === b.forceHeight
            && a.color === b.color
            && a.position === b.position);
    }
    function zonesEqual(a, b) {
        if (a === b) {
            return true;
        }
        if (a.length !== b.length) {
            return false;
        }
        for (var i = 0, len = a.length; i < len; i++) {
            if (!zoneEquals(a[i], b[i])) {
                return false;
            }
        }
        return true;
    }
    var OverviewRulerImpl = (function () {
        function OverviewRulerImpl(canvasLeftOffset, cssClassName, scrollHeight, lineHeight, minimumHeight, maximumHeight, getVerticalOffsetForLine) {
            this._canvasLeftOffset = canvasLeftOffset;
            this._minimumHeight = minimumHeight;
            this._maximumHeight = maximumHeight;
            this._getVerticalOffsetForLine = getVerticalOffsetForLine;
            this._zones = [];
            this._renderedZones = [];
            this._useDarkColor = false;
            this._domNode = document.createElement('canvas');
            this._domNode.className = cssClassName;
            this._domNode.style.position = 'absolute';
            if (browser.canUseTranslate3d) {
                this._domNode.style.transform = 'translate3d(0px, 0px, 0px)';
            }
            this._width = 0;
            this._height = 0;
            this._outerHeight = scrollHeight;
            this._lineHeight = lineHeight;
            this._lanesCount = 3;
        }
        OverviewRulerImpl.prototype.dispose = function () {
            this._zones = [];
        };
        OverviewRulerImpl.prototype.setLayout = function (position, render) {
            styleMutator_1.StyleMutator.setTop(this._domNode, position.top);
            styleMutator_1.StyleMutator.setRight(this._domNode, position.right);
            if (this._width !== position.width || this._height !== position.height) {
                this._width = position.width;
                this._height = position.height;
                this._domNode.width = this._width;
                this._domNode.height = this._height;
                if (render) {
                    this.render(true);
                }
            }
        };
        OverviewRulerImpl.prototype.getLanesCount = function () {
            return this._lanesCount;
        };
        OverviewRulerImpl.prototype.setLanesCount = function (newLanesCount, render) {
            this._lanesCount = newLanesCount;
            if (render) {
                this.render(true);
            }
        };
        OverviewRulerImpl.prototype.setUseDarkColor = function (useDarkColor, render) {
            this._useDarkColor = useDarkColor;
            if (render) {
                this.render(true);
            }
        };
        OverviewRulerImpl.prototype.getDomNode = function () {
            return this._domNode;
        };
        OverviewRulerImpl.prototype.getWidth = function () {
            return this._width;
        };
        OverviewRulerImpl.prototype.getHeight = function () {
            return this._height;
        };
        OverviewRulerImpl.prototype.setScrollHeight = function (scrollHeight, render) {
            this._outerHeight = scrollHeight;
            if (render) {
                this.render(true);
            }
        };
        OverviewRulerImpl.prototype.setLineHeight = function (lineHeight, render) {
            this._lineHeight = lineHeight;
            if (render) {
                this.render(true);
            }
        };
        OverviewRulerImpl.prototype.setZones = function (zones, render) {
            this._zones = zones;
            if (render) {
                this.render(false);
            }
        };
        OverviewRulerImpl.prototype._insertZone = function (colorsZones, y1, y2, minimumHeight, maximumHeight, color) {
            var ycenter = Math.floor((y1 + y2) / 2);
            var halfHeight = (y2 - ycenter);
            if (halfHeight > maximumHeight / 2) {
                halfHeight = maximumHeight / 2;
            }
            if (halfHeight < minimumHeight / 2) {
                halfHeight = minimumHeight / 2;
            }
            if (ycenter - halfHeight < 0) {
                ycenter = halfHeight;
            }
            if (ycenter + halfHeight > this._height) {
                ycenter = this._height - halfHeight;
            }
            colorsZones[color] = colorsZones[color] || [];
            colorsZones[color].push({
                from: ycenter - halfHeight,
                to: ycenter + halfHeight
            });
        };
        OverviewRulerImpl.prototype._getColorForZone = function (zone) {
            if (this._useDarkColor) {
                return zone.darkColor;
            }
            return zone.color;
        };
        OverviewRulerImpl.prototype._renderVerticalPatch = function (ctx, heightRatio, laneMask, xpos, width) {
            var colorsZones = {};
            var i, len, zone, y1, y2, zoneLineNumbers, zoneMaximumHeight;
            for (i = 0, len = this._zones.length; i < len; i++) {
                zone = this._zones[i];
                if (!(zone.position & laneMask)) {
                    continue;
                }
                y1 = this._getVerticalOffsetForLine(zone.startLineNumber);
                y2 = this._getVerticalOffsetForLine(zone.endLineNumber) + this._lineHeight;
                y1 *= heightRatio;
                y2 *= heightRatio;
                if (zone.forceHeight) {
                    y2 = y1 + zone.forceHeight;
                    this._insertZone(colorsZones, y1, y2, zone.forceHeight, zone.forceHeight, this._getColorForZone(zone));
                }
                else {
                    // Figure out if we can render this in one continuous zone
                    zoneLineNumbers = zone.endLineNumber - zone.startLineNumber + 1;
                    zoneMaximumHeight = zoneLineNumbers * this._maximumHeight;
                    if (y2 - y1 > zoneMaximumHeight) {
                        // We need to draw one zone per line
                        for (var lineNumber = zone.startLineNumber; lineNumber <= zone.endLineNumber; lineNumber++) {
                            y1 = this._getVerticalOffsetForLine(lineNumber);
                            y2 = y1 + this._lineHeight;
                            y1 *= heightRatio;
                            y2 *= heightRatio;
                            this._insertZone(colorsZones, y1, y2, this._minimumHeight, this._maximumHeight, this._getColorForZone(zone));
                        }
                    }
                    else {
                        this._insertZone(colorsZones, y1, y2, this._minimumHeight, zoneMaximumHeight, this._getColorForZone(zone));
                    }
                }
            }
            var sorter = function (a, b) {
                return a.from - b.from;
            };
            // Merge color zones
            var colorName, colorZones, currentFrom, currentTo;
            for (colorName in colorsZones) {
                if (colorsZones.hasOwnProperty(colorName)) {
                    colorZones = colorsZones[colorName];
                    // Merge & Render zones
                    colorZones.sort(sorter);
                    currentFrom = colorZones[0].from;
                    currentTo = colorZones[0].to;
                    ctx.fillStyle = colorName;
                    for (i = 1, len = colorZones.length; i < len; i++) {
                        if (currentTo >= colorZones[i].from) {
                            currentTo = Math.max(currentTo, colorZones[i].to);
                        }
                        else {
                            ctx.fillRect(xpos, currentFrom, width, currentTo - currentFrom);
                            currentFrom = colorZones[i].from;
                            currentTo = colorZones[i].to;
                        }
                    }
                    ctx.fillRect(xpos, currentFrom, width, currentTo - currentFrom);
                }
            }
        };
        OverviewRulerImpl.prototype.render = function (forceRender) {
            if (this._outerHeight === 0) {
                return false;
            }
            if (!OverviewRulerImpl.hasCanvas) {
                return false;
            }
            var shouldRender = forceRender || !zonesEqual(this._renderedZones, this._zones);
            if (shouldRender) {
                var heightRatio = this._height / this._outerHeight;
                var ctx = this._domNode.getContext('2d');
                ctx.clearRect(0, 0, this._width, this._height);
                var remainingWidth = this._width - this._canvasLeftOffset;
                if (this._lanesCount >= 3) {
                    this._renderThreeLanes(ctx, heightRatio, remainingWidth);
                }
                else if (this._lanesCount === 2) {
                    this._renderTwoLanes(ctx, heightRatio, remainingWidth);
                }
                else if (this._lanesCount === 1) {
                    this._renderOneLane(ctx, heightRatio, remainingWidth);
                }
            }
            this._renderedZones = this._zones;
            return shouldRender;
        };
        OverviewRulerImpl.prototype._renderOneLane = function (ctx, heightRatio, w) {
            this._renderVerticalPatch(ctx, heightRatio, editorCommon_1.OverviewRulerLane.Left | editorCommon_1.OverviewRulerLane.Center | editorCommon_1.OverviewRulerLane.Right, this._canvasLeftOffset, w);
        };
        OverviewRulerImpl.prototype._renderTwoLanes = function (ctx, heightRatio, w) {
            var leftWidth = Math.floor(w / 2), rightWidth = w - leftWidth, leftOffset = this._canvasLeftOffset, rightOffset = this._canvasLeftOffset + leftWidth;
            this._renderVerticalPatch(ctx, heightRatio, editorCommon_1.OverviewRulerLane.Left | editorCommon_1.OverviewRulerLane.Center, leftOffset, leftWidth);
            this._renderVerticalPatch(ctx, heightRatio, editorCommon_1.OverviewRulerLane.Right, rightOffset, rightWidth);
        };
        OverviewRulerImpl.prototype._renderThreeLanes = function (ctx, heightRatio, w) {
            var leftWidth = Math.floor(w / 3), rightWidth = Math.floor(w / 3), centerWidth = w - leftWidth - rightWidth, leftOffset = this._canvasLeftOffset, centerOffset = this._canvasLeftOffset + leftWidth, rightOffset = this._canvasLeftOffset + leftWidth + centerWidth;
            this._renderVerticalPatch(ctx, heightRatio, editorCommon_1.OverviewRulerLane.Left, leftOffset, leftWidth);
            this._renderVerticalPatch(ctx, heightRatio, editorCommon_1.OverviewRulerLane.Center, centerOffset, centerWidth);
            this._renderVerticalPatch(ctx, heightRatio, editorCommon_1.OverviewRulerLane.Right, rightOffset, rightWidth);
        };
        OverviewRulerImpl.hasCanvas = (window.navigator.userAgent.indexOf('MSIE 8') === -1);
        return OverviewRulerImpl;
    }());
    exports.OverviewRulerImpl = OverviewRulerImpl;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/controller/textAreaState", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon'], function (require, exports, strings_1, range_1, editorCommon_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (TextAreaStrategy) {
        TextAreaStrategy[TextAreaStrategy["IENarrator"] = 0] = "IENarrator";
        TextAreaStrategy[TextAreaStrategy["NVDA"] = 1] = "NVDA";
    })(exports.TextAreaStrategy || (exports.TextAreaStrategy = {}));
    var TextAreaStrategy = exports.TextAreaStrategy;
    var USE_NVDA_FULL_TEXT = false;
    function createTextAreaState(strategy) {
        if (strategy === TextAreaStrategy.IENarrator) {
            return IENarratorTextAreaState.EMPTY;
        }
        if (USE_NVDA_FULL_TEXT) {
            return NVDAFullTextAreaState.EMPTY;
        }
        return NVDAPagedTextAreaState.EMPTY;
    }
    exports.createTextAreaState = createTextAreaState;
    var TextAreaState = (function () {
        function TextAreaState(previousState, value, selectionStart, selectionEnd, isInOverwriteMode) {
            this.previousState = previousState ? previousState.shallowClone() : null;
            this.value = value;
            this.selectionStart = selectionStart;
            this.selectionEnd = selectionEnd;
            this.isInOverwriteMode = isInOverwriteMode;
        }
        TextAreaState.prototype.getSelectionStart = function () {
            return this.selectionStart;
        };
        TextAreaState.prototype.getValue = function () {
            return this.value;
        };
        TextAreaState.prototype.applyToTextArea = function (reason, textArea, select) {
            // console.log(Date.now() + ': applyToTextArea ' + reason + ': ' + this.toString());
            if (textArea.getValue() !== this.value) {
                textArea.setValue(reason, this.value);
            }
            if (select) {
                textArea.setSelectionRange(this.selectionStart, this.selectionEnd);
            }
        };
        TextAreaState.prototype.deduceInput = function () {
            if (!this.previousState) {
                // This is the EMPTY state
                return {
                    text: '',
                    replaceCharCnt: 0
                };
            }
            // console.log('------------------------deduceInput');
            // console.log('CURRENT STATE: ' + this.toString());
            // console.log('PREVIOUS STATE: ' + this.previousState.toString());
            var previousValue = this.previousState.value;
            var previousSelectionStart = this.previousState.selectionStart;
            var previousSelectionEnd = this.previousState.selectionEnd;
            var currentValue = this.value;
            var currentSelectionStart = this.selectionStart;
            var currentSelectionEnd = this.selectionEnd;
            // Strip the previous suffix from the value (without interfering with the current selection)
            var previousSuffix = previousValue.substring(previousSelectionEnd);
            var currentSuffix = currentValue.substring(currentSelectionEnd);
            var suffixLength = strings_1.commonSuffixLength(previousSuffix, currentSuffix);
            currentValue = currentValue.substring(0, currentValue.length - suffixLength);
            previousValue = previousValue.substring(0, previousValue.length - suffixLength);
            var previousPrefix = previousValue.substring(0, previousSelectionStart);
            var currentPrefix = currentValue.substring(0, currentSelectionStart);
            var prefixLength = strings_1.commonPrefixLength(previousPrefix, currentPrefix);
            currentValue = currentValue.substring(prefixLength);
            previousValue = previousValue.substring(prefixLength);
            currentSelectionStart -= prefixLength;
            previousSelectionStart -= prefixLength;
            currentSelectionEnd -= prefixLength;
            previousSelectionEnd -= prefixLength;
            // console.log('AFTER DIFFING CURRENT STATE: <' + currentValue + '>, selectionStart: ' + currentSelectionStart + ', selectionEnd: ' + currentSelectionEnd);
            // console.log('AFTER DIFFING PREVIOUS STATE: <' + previousValue + '>, selectionStart: ' + previousSelectionStart + ', selectionEnd: ' + previousSelectionEnd);
            if (currentSelectionStart === currentSelectionEnd) {
                // composition accept case
                // [blahblah] => blahblah|
                if (previousValue === currentValue
                    && previousSelectionStart === 0
                    && previousSelectionEnd === previousValue.length
                    && currentSelectionStart === currentValue.length
                    && currentValue.indexOf('\n') === -1) {
                    return {
                        text: '',
                        replaceCharCnt: 0
                    };
                }
                // no current selection
                var replacePreviousCharacters_1 = (previousPrefix.length - prefixLength);
                // console.log('REMOVE PREVIOUS: ' + (previousPrefix.length - prefixLength) + ' chars');
                return {
                    text: currentValue,
                    replaceCharCnt: replacePreviousCharacters_1
                };
            }
            // there is a current selection => composition case
            var replacePreviousCharacters = previousSelectionEnd - previousSelectionStart;
            return {
                text: currentValue,
                replaceCharCnt: replacePreviousCharacters
            };
        };
        return TextAreaState;
    }());
    exports.TextAreaState = TextAreaState;
    var IENarratorTextAreaState = (function (_super) {
        __extends(IENarratorTextAreaState, _super);
        function IENarratorTextAreaState(previousState, value, selectionStart, selectionEnd, isInOverwriteMode, selectionToken) {
            _super.call(this, previousState, value, selectionStart, selectionEnd, isInOverwriteMode);
            this.selectionToken = selectionToken;
        }
        IENarratorTextAreaState.prototype.shallowClone = function () {
            return new IENarratorTextAreaState(null, this.value, this.selectionStart, this.selectionEnd, this.isInOverwriteMode, this.selectionToken);
        };
        IENarratorTextAreaState.prototype.toEmpty = function () {
            return IENarratorTextAreaState.EMPTY;
        };
        IENarratorTextAreaState.prototype.toString = function () {
            return '[ <' + this.value + '>, selectionStart: ' + this.selectionStart + ', selectionEnd: ' + this.selectionEnd + ', isInOverwriteMode: ' + this.isInOverwriteMode + ', selectionToken: ' + this.selectionToken + ']';
        };
        IENarratorTextAreaState.prototype.toStrategy = function (strategy) {
            if (strategy === TextAreaStrategy.IENarrator) {
                return this;
            }
            if (USE_NVDA_FULL_TEXT) {
                return new NVDAFullTextAreaState(this.previousState, this.value, this.selectionStart, this.selectionEnd, this.isInOverwriteMode);
            }
            return new NVDAPagedTextAreaState(this.previousState, this.value, this.selectionStart, this.selectionEnd, this.isInOverwriteMode);
        };
        IENarratorTextAreaState.prototype.equals = function (other) {
            if (other instanceof IENarratorTextAreaState) {
                return (this.value === other.value
                    && this.selectionStart === other.selectionStart
                    && this.selectionEnd === other.selectionEnd
                    && this.isInOverwriteMode === other.isInOverwriteMode
                    && this.selectionToken === other.selectionToken);
            }
            return false;
        };
        IENarratorTextAreaState.prototype.fromTextArea = function (textArea) {
            return new IENarratorTextAreaState(this, textArea.getValue(), textArea.getSelectionStart(), textArea.getSelectionEnd(), textArea.isInOverwriteMode(), this.selectionToken);
        };
        IENarratorTextAreaState.prototype.fromEditorSelection = function (model, selection) {
            var LIMIT_CHARS = 100;
            var PADDING_LINES_COUNT = 0;
            var selectionStartLineNumber = selection.startLineNumber, selectionStartColumn = selection.startColumn, selectionEndLineNumber = selection.endLineNumber, selectionEndColumn = selection.endColumn, selectionEndLineNumberMaxColumn = model.getLineMaxColumn(selectionEndLineNumber);
            // If the selection is empty and we have switched line numbers, expand selection to full line (helps Narrator trigger a full line read)
            if (selection.isEmpty() && this.selectionToken !== selectionStartLineNumber) {
                selectionStartColumn = 1;
                selectionEndColumn = selectionEndLineNumberMaxColumn;
            }
            // `pretext` contains the text before the selection
            var pretext = '';
            var startLineNumber = Math.max(1, selectionStartLineNumber - PADDING_LINES_COUNT);
            if (startLineNumber < selectionStartLineNumber) {
                pretext = model.getValueInRange(new range_1.Range(startLineNumber, 1, selectionStartLineNumber, 1), editorCommon_1.EndOfLinePreference.LF);
            }
            pretext += model.getValueInRange(new range_1.Range(selectionStartLineNumber, 1, selectionStartLineNumber, selectionStartColumn), editorCommon_1.EndOfLinePreference.LF);
            if (pretext.length > LIMIT_CHARS) {
                pretext = pretext.substring(pretext.length - LIMIT_CHARS, pretext.length);
            }
            // `posttext` contains the text after the selection
            var posttext = '';
            var endLineNumber = Math.min(selectionEndLineNumber + PADDING_LINES_COUNT, model.getLineCount());
            posttext += model.getValueInRange(new range_1.Range(selectionEndLineNumber, selectionEndColumn, selectionEndLineNumber, selectionEndLineNumberMaxColumn), editorCommon_1.EndOfLinePreference.LF);
            if (endLineNumber > selectionEndLineNumber) {
                posttext = '\n' + model.getValueInRange(new range_1.Range(selectionEndLineNumber + 1, 1, endLineNumber, model.getLineMaxColumn(endLineNumber)), editorCommon_1.EndOfLinePreference.LF);
            }
            if (posttext.length > LIMIT_CHARS) {
                posttext = posttext.substring(0, LIMIT_CHARS);
            }
            // `text` contains the text of the selection
            var text = model.getValueInRange(new range_1.Range(selectionStartLineNumber, selectionStartColumn, selectionEndLineNumber, selectionEndColumn), editorCommon_1.EndOfLinePreference.LF);
            if (text.length > 2 * LIMIT_CHARS) {
                text = text.substring(0, LIMIT_CHARS) + String.fromCharCode(8230) + text.substring(text.length - LIMIT_CHARS, text.length);
            }
            return new IENarratorTextAreaState(this, pretext + text + posttext, pretext.length, pretext.length + text.length, false, selectionStartLineNumber);
        };
        IENarratorTextAreaState.prototype.fromText = function (text) {
            return new IENarratorTextAreaState(this, text, 0, text.length, false, 0);
        };
        IENarratorTextAreaState.prototype.resetSelection = function () {
            return new IENarratorTextAreaState(this.previousState, this.value, this.value.length, this.value.length, this.isInOverwriteMode, this.selectionToken);
        };
        IENarratorTextAreaState.EMPTY = new IENarratorTextAreaState(null, '', 0, 0, false, 0);
        return IENarratorTextAreaState;
    }(TextAreaState));
    exports.IENarratorTextAreaState = IENarratorTextAreaState;
    var NVDAPagedTextAreaState = (function (_super) {
        __extends(NVDAPagedTextAreaState, _super);
        function NVDAPagedTextAreaState(previousState, value, selectionStart, selectionEnd, isInOverwriteMode) {
            _super.call(this, previousState, value, selectionStart, selectionEnd, isInOverwriteMode);
        }
        NVDAPagedTextAreaState.prototype.shallowClone = function () {
            return new NVDAPagedTextAreaState(null, this.value, this.selectionStart, this.selectionEnd, this.isInOverwriteMode);
        };
        NVDAPagedTextAreaState.prototype.toEmpty = function () {
            return NVDAPagedTextAreaState.EMPTY;
        };
        NVDAPagedTextAreaState.prototype.toString = function () {
            return '[ <' + this.value + '>, selectionStart: ' + this.selectionStart + ', selectionEnd: ' + this.selectionEnd + ', isInOverwriteMode: ' + this.isInOverwriteMode + ']';
        };
        NVDAPagedTextAreaState.prototype.toStrategy = function (strategy) {
            if (strategy === TextAreaStrategy.NVDA) {
                return this;
            }
            return new IENarratorTextAreaState(this.previousState, this.value, this.selectionStart, this.selectionEnd, this.isInOverwriteMode, 0);
        };
        NVDAPagedTextAreaState.prototype.equals = function (other) {
            if (other instanceof NVDAPagedTextAreaState) {
                return (this.value === other.value
                    && this.selectionStart === other.selectionStart
                    && this.selectionEnd === other.selectionEnd
                    && this.isInOverwriteMode === other.isInOverwriteMode);
            }
            return false;
        };
        NVDAPagedTextAreaState.prototype.fromTextArea = function (textArea) {
            return new NVDAPagedTextAreaState(this, textArea.getValue(), textArea.getSelectionStart(), textArea.getSelectionEnd(), textArea.isInOverwriteMode());
        };
        NVDAPagedTextAreaState._getPageOfLine = function (lineNumber) {
            return Math.floor((lineNumber - 1) / NVDAPagedTextAreaState._LINES_PER_PAGE);
        };
        NVDAPagedTextAreaState._getRangeForPage = function (page) {
            var offset = page * NVDAPagedTextAreaState._LINES_PER_PAGE;
            var startLineNumber = offset + 1;
            var endLineNumber = offset + NVDAPagedTextAreaState._LINES_PER_PAGE;
            return new range_1.Range(startLineNumber, 1, endLineNumber, Number.MAX_VALUE);
        };
        NVDAPagedTextAreaState.prototype.fromEditorSelection = function (model, selection) {
            var selectionStartPage = NVDAPagedTextAreaState._getPageOfLine(selection.startLineNumber);
            var selectionStartPageRange = NVDAPagedTextAreaState._getRangeForPage(selectionStartPage);
            var selectionEndPage = NVDAPagedTextAreaState._getPageOfLine(selection.endLineNumber);
            var selectionEndPageRange = NVDAPagedTextAreaState._getRangeForPage(selectionEndPage);
            var pretextRange = selectionStartPageRange.intersectRanges(new range_1.Range(1, 1, selection.startLineNumber, selection.startColumn));
            var pretext = model.getValueInRange(pretextRange, editorCommon_1.EndOfLinePreference.LF);
            var lastLine = model.getLineCount();
            var lastLineMaxColumn = model.getLineMaxColumn(lastLine);
            var posttextRange = selectionEndPageRange.intersectRanges(new range_1.Range(selection.endLineNumber, selection.endColumn, lastLine, lastLineMaxColumn));
            var posttext = model.getValueInRange(posttextRange, editorCommon_1.EndOfLinePreference.LF);
            var text = null;
            if (selectionStartPage <= selectionEndPage) {
                // take full selection
                text = model.getValueInRange(selection, editorCommon_1.EndOfLinePreference.LF);
            }
            else {
                var selectionRange1 = selectionStartPageRange.intersectRanges(selection);
                var selectionRange2 = selectionEndPageRange.intersectRanges(selection);
                text = (model.getValueInRange(selectionRange1, editorCommon_1.EndOfLinePreference.LF)
                    + String.fromCharCode(8230)
                    + model.getValueInRange(selectionRange2, editorCommon_1.EndOfLinePreference.LF));
            }
            return new NVDAPagedTextAreaState(this, pretext + text + posttext, pretext.length, pretext.length + text.length, false);
        };
        NVDAPagedTextAreaState.prototype.fromText = function (text) {
            return new NVDAPagedTextAreaState(this, text, 0, text.length, false);
        };
        NVDAPagedTextAreaState.prototype.resetSelection = function () {
            return new NVDAPagedTextAreaState(this.previousState, this.value, this.value.length, this.value.length, this.isInOverwriteMode);
        };
        NVDAPagedTextAreaState.EMPTY = new NVDAPagedTextAreaState(null, '', 0, 0, false);
        NVDAPagedTextAreaState._LINES_PER_PAGE = 10;
        return NVDAPagedTextAreaState;
    }(TextAreaState));
    exports.NVDAPagedTextAreaState = NVDAPagedTextAreaState;
    var NVDAFullTextAreaState = (function (_super) {
        __extends(NVDAFullTextAreaState, _super);
        function NVDAFullTextAreaState(previousState, value, selectionStart, selectionEnd, isInOverwriteMode) {
            _super.call(this, previousState, value, selectionStart, selectionEnd, isInOverwriteMode);
        }
        NVDAFullTextAreaState.prototype.shallowClone = function () {
            return new NVDAFullTextAreaState(null, this.value, this.selectionStart, this.selectionEnd, this.isInOverwriteMode);
        };
        NVDAFullTextAreaState.prototype.toEmpty = function () {
            return NVDAFullTextAreaState.EMPTY;
        };
        NVDAFullTextAreaState.prototype.toString = function () {
            return '[ <ENTIRE TEXT' + '>, selectionStart: ' + this.selectionStart + ', selectionEnd: ' + this.selectionEnd + ', isInOverwriteMode: ' + this.isInOverwriteMode + ']';
        };
        NVDAFullTextAreaState.prototype.toStrategy = function (strategy) {
            if (strategy === TextAreaStrategy.NVDA) {
                return this;
            }
            return new IENarratorTextAreaState(this.previousState, this.value, this.selectionStart, this.selectionEnd, this.isInOverwriteMode, 0);
        };
        NVDAFullTextAreaState.prototype.equals = function (other) {
            if (other instanceof NVDAFullTextAreaState) {
                return (this.value === other.value
                    && this.selectionStart === other.selectionStart
                    && this.selectionEnd === other.selectionEnd
                    && this.isInOverwriteMode === other.isInOverwriteMode);
            }
            return false;
        };
        NVDAFullTextAreaState.prototype.fromTextArea = function (textArea) {
            return new NVDAFullTextAreaState(this, textArea.getValue(), textArea.getSelectionStart(), textArea.getSelectionEnd(), textArea.isInOverwriteMode());
        };
        NVDAFullTextAreaState.prototype.fromEditorSelection = function (model, selection) {
            var pretext = model.getValueInRange(new range_1.Range(1, 1, selection.startLineNumber, selection.startColumn), editorCommon_1.EndOfLinePreference.LF);
            var text = model.getValueInRange(selection, editorCommon_1.EndOfLinePreference.LF);
            var lastLine = model.getLineCount();
            var lastLineMaxColumn = model.getLineMaxColumn(lastLine);
            var posttext = model.getValueInRange(new range_1.Range(selection.endLineNumber, selection.endColumn, lastLine, lastLineMaxColumn), editorCommon_1.EndOfLinePreference.LF);
            return new NVDAFullTextAreaState(this, pretext + text + posttext, pretext.length, pretext.length + text.length, false);
        };
        NVDAFullTextAreaState.prototype.fromText = function (text) {
            return new NVDAFullTextAreaState(this, text, 0, text.length, false);
        };
        NVDAFullTextAreaState.prototype.resetSelection = function () {
            return new NVDAFullTextAreaState(this.previousState, this.value, this.value.length, this.value.length, this.isInOverwriteMode);
        };
        NVDAFullTextAreaState.EMPTY = new NVDAFullTextAreaState(null, '', 0, 0, false);
        return NVDAFullTextAreaState;
    }(TextAreaState));
    exports.NVDAFullTextAreaState = NVDAFullTextAreaState;
});

define("vs/editor/common/core/editorState", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/editorCommon'], function (require, exports, strings, editorCommon_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EditorState = (function () {
        function EditorState(editor, flags) {
            var _this = this;
            this.flags = flags;
            flags.forEach(function (flag) {
                switch (flag) {
                    case editorCommon_1.CodeEditorStateFlag.Value:
                        var model = editor.getModel();
                        _this.modelVersionId = model ? strings.format('{0}#{1}', model.getAssociatedResource().toString(), model.getVersionId()) : null;
                        break;
                    case editorCommon_1.CodeEditorStateFlag.Position:
                        _this.position = editor.getPosition();
                        break;
                    case editorCommon_1.CodeEditorStateFlag.Selection:
                        _this.selection = editor.getSelection();
                        break;
                    case editorCommon_1.CodeEditorStateFlag.Scroll:
                        _this.scrollLeft = editor.getScrollLeft();
                        _this.scrollTop = editor.getScrollTop();
                        break;
                }
            });
        }
        EditorState.prototype._equals = function (other) {
            if (!(other instanceof EditorState)) {
                return false;
            }
            var state = other;
            if (this.modelVersionId !== state.modelVersionId) {
                return false;
            }
            if (this.scrollLeft !== state.scrollLeft || this.scrollTop !== state.scrollTop) {
                return false;
            }
            if (!this.position && state.position || this.position && !state.position || this.position && state.position && !this.position.equals(state.position)) {
                return false;
            }
            if (!this.selection && state.selection || this.selection && !state.selection || this.selection && state.selection && !this.selection.equalsRange(state.selection)) {
                return false;
            }
            return true;
        };
        EditorState.prototype.validate = function (editor) {
            return this._equals(new EditorState(editor, this.flags));
        };
        return EditorState;
    }());
    exports.EditorState = EditorState;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/core/selection", ["require", "exports", 'vs/editor/common/core/range', 'vs/editor/common/editorCommon'], function (require, exports, range_1, editorCommon_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Selection = (function (_super) {
        __extends(Selection, _super);
        function Selection(selectionStartLineNumber, selectionStartColumn, positionLineNumber, positionColumn) {
            _super.call(this, selectionStartLineNumber, selectionStartColumn, positionLineNumber, positionColumn);
            this.selectionStartLineNumber = selectionStartLineNumber;
            this.selectionStartColumn = selectionStartColumn;
            this.positionLineNumber = positionLineNumber;
            this.positionColumn = positionColumn;
        }
        Selection.prototype.clone = function () {
            return new Selection(this.selectionStartLineNumber, this.selectionStartColumn, this.positionLineNumber, this.positionColumn);
        };
        Selection.prototype.toString = function () {
            return '[' + this.selectionStartLineNumber + ',' + this.selectionStartColumn + ' -> ' + this.positionLineNumber + ',' + this.positionColumn + ']';
        };
        Selection.prototype.equalsSelection = function (other) {
            return (Selection.selectionsEqual(this, other));
        };
        Selection.prototype.getDirection = function () {
            if (this.selectionStartLineNumber === this.startLineNumber && this.selectionStartColumn === this.startColumn) {
                return editorCommon_1.SelectionDirection.LTR;
            }
            return editorCommon_1.SelectionDirection.RTL;
        };
        Selection.prototype.setEndPosition = function (endLineNumber, endColumn) {
            if (this.getDirection() === editorCommon_1.SelectionDirection.LTR) {
                return new Selection(this.startLineNumber, this.startColumn, endLineNumber, endColumn);
            }
            return new Selection(endLineNumber, endColumn, this.startLineNumber, this.startColumn);
        };
        Selection.prototype.setStartPosition = function (startLineNumber, startColumn) {
            if (this.getDirection() === editorCommon_1.SelectionDirection.LTR) {
                return new Selection(startLineNumber, startColumn, this.endLineNumber, this.endColumn);
            }
            return new Selection(this.endLineNumber, this.endColumn, startLineNumber, startColumn);
        };
        // ----
        Selection.createSelection = function (selectionStartLineNumber, selectionStartColumn, positionLineNumber, positionColumn) {
            return new Selection(selectionStartLineNumber, selectionStartColumn, positionLineNumber, positionColumn);
        };
        Selection.liftSelection = function (sel) {
            return new Selection(sel.selectionStartLineNumber, sel.selectionStartColumn, sel.positionLineNumber, sel.positionColumn);
        };
        Selection.selectionsEqual = function (a, b) {
            return (a.selectionStartLineNumber === b.selectionStartLineNumber &&
                a.selectionStartColumn === b.selectionStartColumn &&
                a.positionLineNumber === b.positionLineNumber &&
                a.positionColumn === b.positionColumn);
        };
        Selection.selectionsArrEqual = function (a, b) {
            if (a && !b || !a && b) {
                return false;
            }
            if (!a && !b) {
                return true;
            }
            if (a.length !== b.length) {
                return false;
            }
            for (var i = 0, len = a.length; i < len; i++) {
                if (!this.selectionsEqual(a[i], b[i])) {
                    return false;
                }
            }
            return true;
        };
        Selection.isISelection = function (obj) {
            return (obj
                && (typeof obj.selectionStartLineNumber === 'number')
                && (typeof obj.selectionStartColumn === 'number')
                && (typeof obj.positionLineNumber === 'number')
                && (typeof obj.positionColumn === 'number'));
        };
        Selection.createWithDirection = function (startLineNumber, startColumn, endLineNumber, endColumn, direction) {
            if (direction === editorCommon_1.SelectionDirection.LTR) {
                return new Selection(startLineNumber, startColumn, endLineNumber, endColumn);
            }
            return new Selection(endLineNumber, endColumn, startLineNumber, startColumn);
        };
        return Selection;
    }(range_1.Range));
    exports.Selection = Selection;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/commands/replaceCommand", ["require", "exports", 'vs/editor/common/core/selection'], function (require, exports, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ReplaceCommand = (function () {
        function ReplaceCommand(range, text) {
            this._range = range;
            this._text = text;
        }
        ReplaceCommand.prototype.getText = function () {
            return this._text;
        };
        ReplaceCommand.prototype.getRange = function () {
            return this._range;
        };
        ReplaceCommand.prototype.setRange = function (newRange) {
            this._range = newRange;
        };
        ReplaceCommand.prototype.getEditOperations = function (model, builder) {
            builder.addEditOperation(this._range, this._text);
        };
        ReplaceCommand.prototype.computeCursorState = function (model, helper) {
            var inverseEditOperations = helper.getInverseEditOperations();
            var srcRange = inverseEditOperations[0].range;
            return new selection_1.Selection(srcRange.endLineNumber, srcRange.endColumn, srcRange.endLineNumber, srcRange.endColumn);
        };
        return ReplaceCommand;
    }());
    exports.ReplaceCommand = ReplaceCommand;
    var ReplaceCommandWithoutChangingPosition = (function (_super) {
        __extends(ReplaceCommandWithoutChangingPosition, _super);
        function ReplaceCommandWithoutChangingPosition(range, text) {
            _super.call(this, range, text);
        }
        ReplaceCommandWithoutChangingPosition.prototype.computeCursorState = function (model, helper) {
            var inverseEditOperations = helper.getInverseEditOperations();
            var srcRange = inverseEditOperations[0].range;
            return new selection_1.Selection(srcRange.startLineNumber, srcRange.startColumn, srcRange.startLineNumber, srcRange.startColumn);
        };
        return ReplaceCommandWithoutChangingPosition;
    }(ReplaceCommand));
    exports.ReplaceCommandWithoutChangingPosition = ReplaceCommandWithoutChangingPosition;
    var ReplaceCommandWithOffsetCursorState = (function (_super) {
        __extends(ReplaceCommandWithOffsetCursorState, _super);
        function ReplaceCommandWithOffsetCursorState(range, text, lineNumberDeltaOffset, columnDeltaOffset) {
            _super.call(this, range, text);
            this._columnDeltaOffset = columnDeltaOffset;
            this._lineNumberDeltaOffset = lineNumberDeltaOffset;
        }
        ReplaceCommandWithOffsetCursorState.prototype.computeCursorState = function (model, helper) {
            var inverseEditOperations = helper.getInverseEditOperations();
            var srcRange = inverseEditOperations[0].range;
            return new selection_1.Selection(srcRange.endLineNumber + this._lineNumberDeltaOffset, srcRange.endColumn + this._columnDeltaOffset, srcRange.endLineNumber + this._lineNumberDeltaOffset, srcRange.endColumn + this._columnDeltaOffset);
        };
        return ReplaceCommandWithOffsetCursorState;
    }(ReplaceCommand));
    exports.ReplaceCommandWithOffsetCursorState = ReplaceCommandWithOffsetCursorState;
    var ReplaceCommandThatPreservesSelection = (function (_super) {
        __extends(ReplaceCommandThatPreservesSelection, _super);
        function ReplaceCommandThatPreservesSelection(editRange, text, initialSelection) {
            _super.call(this, editRange, text);
            this._initialSelection = initialSelection;
        }
        ReplaceCommandThatPreservesSelection.prototype.getEditOperations = function (model, builder) {
            _super.prototype.getEditOperations.call(this, model, builder);
            this._selectionId = builder.trackSelection(this._initialSelection);
        };
        ReplaceCommandThatPreservesSelection.prototype.computeCursorState = function (model, helper) {
            return helper.getTrackedSelection(this._selectionId);
        };
        return ReplaceCommandThatPreservesSelection;
    }(ReplaceCommand));
    exports.ReplaceCommandThatPreservesSelection = ReplaceCommandThatPreservesSelection;
});

define("vs/editor/common/commands/surroundSelectionCommand", ["require", "exports", 'vs/editor/common/core/range', 'vs/editor/common/core/selection'], function (require, exports, range_1, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var SurroundSelectionCommand = (function () {
        function SurroundSelectionCommand(range, charBeforeSelection, charAfterSelection) {
            this._range = range;
            this._charBeforeSelection = charBeforeSelection;
            this._charAfterSelection = charAfterSelection;
        }
        SurroundSelectionCommand.prototype.getEditOperations = function (model, builder) {
            builder.addEditOperation(new range_1.Range(this._range.startLineNumber, this._range.startColumn, this._range.startLineNumber, this._range.startColumn), this._charBeforeSelection);
            builder.addEditOperation(new range_1.Range(this._range.endLineNumber, this._range.endColumn, this._range.endLineNumber, this._range.endColumn), this._charAfterSelection);
        };
        SurroundSelectionCommand.prototype.computeCursorState = function (model, helper) {
            var inverseEditOperations = helper.getInverseEditOperations();
            var firstOperationRange = inverseEditOperations[0].range;
            var secondOperationRange = inverseEditOperations[1].range;
            return new selection_1.Selection(firstOperationRange.endLineNumber, firstOperationRange.endColumn, secondOperationRange.endLineNumber, secondOperationRange.endColumn - this._charAfterSelection.length);
        };
        return SurroundSelectionCommand;
    }());
    exports.SurroundSelectionCommand = SurroundSelectionCommand;
});

define("vs/editor/common/controller/cursorMoveHelper", ["require", "exports", 'vs/editor/common/core/selection'], function (require, exports, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function isHighSurrogate(model, lineNumber, column) {
        var code = model.getLineContent(lineNumber).charCodeAt(column - 1);
        return 0xD800 <= code && code <= 0xDBFF;
    }
    function isLowSurrogate(model, lineNumber, column) {
        var code = model.getLineContent(lineNumber).charCodeAt(column - 1);
        return 0xDC00 <= code && code <= 0xDFFF;
    }
    var CursorMoveHelper = (function () {
        function CursorMoveHelper(configuration) {
            this.configuration = configuration;
        }
        CursorMoveHelper.prototype.getLeftOfPosition = function (model, lineNumber, column) {
            if (column > model.getLineMinColumn(lineNumber)) {
                column = column - (isLowSurrogate(model, lineNumber, column - 1) ? 2 : 1);
            }
            else if (lineNumber > 1) {
                lineNumber = lineNumber - 1;
                column = model.getLineMaxColumn(lineNumber);
            }
            return {
                lineNumber: lineNumber,
                column: column
            };
        };
        CursorMoveHelper.prototype.getRightOfPosition = function (model, lineNumber, column) {
            if (column < model.getLineMaxColumn(lineNumber)) {
                column = column + (isHighSurrogate(model, lineNumber, column) ? 2 : 1);
            }
            else if (lineNumber < model.getLineCount()) {
                lineNumber = lineNumber + 1;
                column = model.getLineMinColumn(lineNumber);
            }
            return {
                lineNumber: lineNumber,
                column: column
            };
        };
        CursorMoveHelper.prototype.getPositionUp = function (model, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnFirstLine) {
            var currentVisibleColumn = this.visibleColumnFromColumn(model, lineNumber, column) + leftoverVisibleColumns;
            lineNumber = lineNumber - count;
            if (lineNumber < 1) {
                lineNumber = 1;
                if (allowMoveOnFirstLine) {
                    column = model.getLineMinColumn(lineNumber);
                }
                else {
                    column = Math.min(model.getLineMaxColumn(lineNumber), column);
                }
            }
            else {
                column = this.columnFromVisibleColumn(model, lineNumber, currentVisibleColumn);
            }
            leftoverVisibleColumns = currentVisibleColumn - this.visibleColumnFromColumn(model, lineNumber, column);
            return {
                lineNumber: lineNumber,
                column: column,
                leftoverVisibleColumns: leftoverVisibleColumns
            };
        };
        CursorMoveHelper.prototype.getPositionDown = function (model, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnLastLine) {
            var currentVisibleColumn = this.visibleColumnFromColumn(model, lineNumber, column) + leftoverVisibleColumns;
            lineNumber = lineNumber + count;
            var lineCount = model.getLineCount();
            if (lineNumber > lineCount) {
                lineNumber = lineCount;
                if (allowMoveOnLastLine) {
                    column = model.getLineMaxColumn(lineNumber);
                }
                else {
                    column = Math.min(model.getLineMaxColumn(lineNumber), column);
                }
            }
            else {
                column = this.columnFromVisibleColumn(model, lineNumber, currentVisibleColumn);
            }
            leftoverVisibleColumns = currentVisibleColumn - this.visibleColumnFromColumn(model, lineNumber, column);
            return {
                lineNumber: lineNumber,
                column: column,
                leftoverVisibleColumns: leftoverVisibleColumns
            };
        };
        CursorMoveHelper.prototype.columnSelect = function (model, fromLineNumber, fromVisibleColumn, toLineNumber, toVisibleColumn) {
            var lineCount = Math.abs(toLineNumber - fromLineNumber) + 1;
            var reversed = (fromLineNumber > toLineNumber);
            var isRTL = (fromVisibleColumn > toVisibleColumn);
            var isLTR = (fromVisibleColumn < toVisibleColumn);
            var result = [];
            // console.log(`fromVisibleColumn: ${fromVisibleColumn}, toVisibleColumn: ${toVisibleColumn}`);
            for (var i = 0; i < lineCount; i++) {
                var lineNumber = fromLineNumber + (reversed ? -i : i);
                var startColumn = this.columnFromVisibleColumn(model, lineNumber, fromVisibleColumn);
                var endColumn = this.columnFromVisibleColumn(model, lineNumber, toVisibleColumn);
                var visibleStartColumn = this.visibleColumnFromColumn(model, lineNumber, startColumn);
                var visibleEndColumn = this.visibleColumnFromColumn(model, lineNumber, endColumn);
                // console.log(`lineNumber: ${lineNumber}: visibleStartColumn: ${visibleStartColumn}, visibleEndColumn: ${visibleEndColumn}`);
                if (isLTR) {
                    if (visibleStartColumn > toVisibleColumn) {
                        continue;
                    }
                    if (visibleEndColumn < fromVisibleColumn) {
                        continue;
                    }
                }
                if (isRTL) {
                    if (visibleEndColumn > fromVisibleColumn) {
                        continue;
                    }
                    if (visibleStartColumn < toVisibleColumn) {
                        continue;
                    }
                }
                result.push(new selection_1.Selection(lineNumber, startColumn, lineNumber, endColumn));
            }
            return {
                viewSelections: result,
                reversed: reversed
            };
        };
        CursorMoveHelper.prototype.getColumnAtBeginningOfLine = function (model, lineNumber, column) {
            var firstNonBlankColumn = model.getLineFirstNonWhitespaceColumn(lineNumber) || 1;
            var minColumn = model.getLineMinColumn(lineNumber);
            if (column !== minColumn && column <= firstNonBlankColumn) {
                column = minColumn;
            }
            else {
                column = firstNonBlankColumn;
            }
            return column;
        };
        CursorMoveHelper.prototype.getColumnAtEndOfLine = function (model, lineNumber, column) {
            var maxColumn = model.getLineMaxColumn(lineNumber);
            var lastNonBlankColumn = model.getLineLastNonWhitespaceColumn(lineNumber) || maxColumn;
            if (column !== maxColumn && column >= lastNonBlankColumn) {
                column = maxColumn;
            }
            else {
                column = lastNonBlankColumn;
            }
            return column;
        };
        CursorMoveHelper.prototype.visibleColumnFromColumn = function (model, lineNumber, column) {
            return CursorMoveHelper.visibleColumnFromColumn(model, lineNumber, column, this.configuration.getIndentationOptions().tabSize);
        };
        CursorMoveHelper.visibleColumnFromColumn = function (model, lineNumber, column, tabSize) {
            return CursorMoveHelper.visibleColumnFromColumn2(model.getLineContent(lineNumber), column, tabSize);
        };
        CursorMoveHelper.visibleColumnFromColumn2 = function (line, column, tabSize) {
            var result = 0;
            for (var i = 0; i < column - 1; i++) {
                result = (line.charAt(i) === '\t') ? CursorMoveHelper.nextTabColumn(result, tabSize) : result + 1;
            }
            return result;
        };
        CursorMoveHelper.prototype.columnFromVisibleColumn = function (model, lineNumber, visibleColumn) {
            var line = model.getLineContent(lineNumber);
            var lastVisibleColumn = -1;
            var thisVisibleColumn = 0;
            for (var i = 0; i < line.length && thisVisibleColumn <= visibleColumn; i++) {
                lastVisibleColumn = thisVisibleColumn;
                thisVisibleColumn = (line.charAt(i) === '\t') ? CursorMoveHelper.nextTabColumn(thisVisibleColumn, this.configuration.getIndentationOptions().tabSize) : thisVisibleColumn + 1;
            }
            // Choose the closest
            thisVisibleColumn = Math.abs(visibleColumn - thisVisibleColumn);
            lastVisibleColumn = Math.abs(visibleColumn - lastVisibleColumn);
            var result;
            if (thisVisibleColumn < lastVisibleColumn) {
                result = i + 1;
            }
            else {
                result = i;
            }
            var minColumn = model.getLineMinColumn(lineNumber);
            if (result < minColumn) {
                result = minColumn;
            }
            return result;
        };
        /**
         * ATTENTION: This works with 0-based columns (as oposed to the regular 1-based columns)
         */
        CursorMoveHelper.nextTabColumn = function (column, tabSize) {
            return column + tabSize - column % tabSize;
        };
        /**
         * ATTENTION: This works with 0-based columns (as oposed to the regular 1-based columns)
         */
        CursorMoveHelper.prevTabColumn = function (column, tabSize) {
            return column - 1 - (column - 1) % tabSize;
        };
        return CursorMoveHelper;
    }());
    exports.CursorMoveHelper = CursorMoveHelper;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/editorActionEnablement", ["require", "exports", 'vs/base/common/lifecycle', 'vs/editor/common/editorCommon'], function (require, exports, lifecycle_1, editorCommon_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (Behaviour) {
        Behaviour[Behaviour["TextFocus"] = 1] = "TextFocus";
        Behaviour[Behaviour["WidgetFocus"] = 2] = "WidgetFocus";
        Behaviour[Behaviour["Writeable"] = 4] = "Writeable";
        Behaviour[Behaviour["UpdateOnModelChange"] = 8] = "UpdateOnModelChange";
        Behaviour[Behaviour["UpdateOnConfigurationChange"] = 16] = "UpdateOnConfigurationChange";
        Behaviour[Behaviour["ShowInContextMenu"] = 32] = "ShowInContextMenu";
        Behaviour[Behaviour["UpdateOnCursorPositionChange"] = 64] = "UpdateOnCursorPositionChange";
    })(exports.Behaviour || (exports.Behaviour = {}));
    var Behaviour = exports.Behaviour;
    function createActionEnablement(editor, condition, action) {
        return new CompositeEnablementState([new InternalEnablementState(condition, editor), new DescentEnablementState(condition, editor, action)]);
    }
    exports.createActionEnablement = createActionEnablement;
    /**
     * A composite that acts like a logical AND on
     * enablement states
     */
    var CompositeEnablementState = (function () {
        function CompositeEnablementState(_delegates) {
            this._delegates = _delegates;
            // empty
        }
        CompositeEnablementState.prototype.value = function () {
            return this._delegates.every(function (d) { return d.value(); });
        };
        CompositeEnablementState.prototype.reset = function () {
            this._delegates.forEach(function (d) {
                if (d instanceof CachingEnablementState) {
                    d.reset();
                }
            });
        };
        CompositeEnablementState.prototype.dispose = function () {
            this._delegates.forEach(function (d) { return d.dispose(); });
        };
        return CompositeEnablementState;
    }());
    /**
     * A enablement state that caches its result until
     * reset is called.
     */
    var CachingEnablementState = (function () {
        function CachingEnablementState() {
            this._value = null;
        }
        CachingEnablementState.prototype.reset = function () {
            this._value = null;
        };
        CachingEnablementState.prototype.dispose = function () {
            //
        };
        CachingEnablementState.prototype.value = function () {
            if (this._value === null) {
                this._value = this._computeValue();
            }
            return this._value;
        };
        CachingEnablementState.prototype._computeValue = function () {
            return false;
        };
        return CachingEnablementState;
    }());
    /**
     * An enablement state that checks behaviours of the
     * editor action that can be check inside the action,
     * for instance: widget focus, text focus, readonly-ness
     */
    var InternalEnablementState = (function (_super) {
        __extends(InternalEnablementState, _super);
        function InternalEnablementState(_behaviour, editor) {
            var _this = this;
            _super.call(this);
            this._behaviour = _behaviour;
            this.editor = editor;
            this.hasTextFocus = false;
            this.hasWidgetFocus = false;
            this.isReadOnly = false;
            this._callOnDispose = [];
            if (this._behaviour & Behaviour.TextFocus) {
                this._callOnDispose.push(this.editor.addListener2(editorCommon_1.EventType.EditorTextFocus, function () { return _this._updateTextFocus(true); }));
                this._callOnDispose.push(this.editor.addListener2(editorCommon_1.EventType.EditorTextBlur, function () { return _this._updateTextFocus(false); }));
            }
            if (this._behaviour & Behaviour.WidgetFocus) {
                this._callOnDispose.push(this.editor.addListener2(editorCommon_1.EventType.EditorFocus, function () { return _this._updateWidgetFocus(true); }));
                this._callOnDispose.push(this.editor.addListener2(editorCommon_1.EventType.EditorBlur, function () { return _this._updateWidgetFocus(false); }));
            }
            if (this._behaviour & Behaviour.Writeable) {
                this._callOnDispose.push(this.editor.addListener2(editorCommon_1.EventType.ConfigurationChanged, function (e) { return _this._update(); }));
            }
        }
        InternalEnablementState.prototype._updateTextFocus = function (hasTextFocus) {
            this.hasTextFocus = hasTextFocus;
            this.reset();
        };
        InternalEnablementState.prototype._updateWidgetFocus = function (hasWidgetFocus) {
            this.hasWidgetFocus = hasWidgetFocus;
            this.reset();
        };
        InternalEnablementState.prototype._update = function () {
            this.isReadOnly = this.editor.getConfiguration().readOnly;
            this.reset();
        };
        InternalEnablementState.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            lifecycle_1.disposeAll(this._callOnDispose);
        };
        InternalEnablementState.prototype._computeValue = function () {
            if (this._behaviour & Behaviour.TextFocus && !this.hasTextFocus) {
                return false;
            }
            if (this._behaviour & Behaviour.WidgetFocus && !this.hasWidgetFocus) {
                return false;
            }
            if (this._behaviour & Behaviour.Writeable && this.isReadOnly) {
                return false;
            }
            return true;
        };
        return InternalEnablementState;
    }(CachingEnablementState));
    /**
     * An enablement state that makes uses of the
     * {{isSupported}} and {{getEnablementState}}
     * functions that are supposed to be overwritten.
     */
    var DescentEnablementState = (function (_super) {
        __extends(DescentEnablementState, _super);
        function DescentEnablementState(behaviour, editor, _action) {
            var _this = this;
            _super.call(this);
            this.editor = editor;
            this._action = _action;
            this._callOnDispose = [];
            if (behaviour & Behaviour.UpdateOnModelChange) {
                this._callOnDispose.push(this.editor.addListener(editorCommon_1.EventType.ModelChanged, function () { return _this.reset(); }));
                this._callOnDispose.push(this.editor.addListener(editorCommon_1.EventType.ModelModeChanged, function () { return _this.reset(); }));
                this._callOnDispose.push(this.editor.addListener(editorCommon_1.EventType.ModelModeSupportChanged, function () { return _this.reset(); }));
            }
            if (behaviour & Behaviour.UpdateOnCursorPositionChange) {
                this._callOnDispose.push(this.editor.addListener(editorCommon_1.EventType.CursorPositionChanged, function () { return _this.reset(); }));
            }
        }
        DescentEnablementState.prototype._computeValue = function () {
            if (!this.editor.getModel()) {
                return false;
            }
            if (!this._action.isSupported()) {
                return false;
            }
            if (!this._action.getEnablementState()) {
                return false;
            }
            return true;
        };
        return DescentEnablementState;
    }(CachingEnablementState));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/editorAction", ["require", "exports", 'vs/base/common/actions', 'vs/base/common/strings', 'vs/base/common/winjs.base', 'vs/editor/common/editorActionEnablement'], function (require, exports, actions_1, strings, winjs_base_1, editorActionEnablement_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var defaultBehaviour = editorActionEnablement_1.Behaviour.TextFocus | editorActionEnablement_1.Behaviour.Writeable | editorActionEnablement_1.Behaviour.UpdateOnModelChange;
    var EditorAction = (function (_super) {
        __extends(EditorAction, _super);
        function EditorAction(descriptor, editor, condition) {
            if (condition === void 0) { condition = defaultBehaviour; }
            _super.call(this, descriptor.id);
            this.editor = editor;
            this._descriptor = descriptor;
            this.label = descriptor.label || '';
            this._enablementState = editorActionEnablement_1.createActionEnablement(editor, condition, this);
            this._shouldShowInContextMenu = !!(condition & editorActionEnablement_1.Behaviour.ShowInContextMenu);
            this._supportsReadonly = !(condition & editorActionEnablement_1.Behaviour.Writeable);
        }
        EditorAction.prototype.getId = function () {
            return this.id;
        };
        EditorAction.prototype.dispose = function () {
            this._enablementState.dispose();
            _super.prototype.dispose.call(this);
        };
        /**
         * A helper to be able to group and sort actions when they are presented visually.
         */
        EditorAction.prototype.getGroupId = function () {
            return this.id;
        };
        EditorAction.prototype.shouldShowInContextMenu = function () {
            return this._shouldShowInContextMenu;
        };
        EditorAction.prototype.getDescriptor = function () {
            return this._descriptor;
        };
        Object.defineProperty(EditorAction.prototype, "enabled", {
            // ---- enablement state mangament --------------------------------------------------------
            get: function () {
                return this._enablementState.value();
            },
            set: function (value) {
                // call reset?
                var e = new Error();
                console.log('setting EditorAction.enabled is UNCOOL. Use resetEnablementState and getEnablementState');
                console.log(e.stack);
            },
            enumerable: true,
            configurable: true
        });
        EditorAction.prototype.resetEnablementState = function () {
            this._enablementState.reset();
        };
        /**
         * Returns {{true}} in case this action works
         * with the current mode. To be overwritten
         * in subclasses.
         */
        EditorAction.prototype.isSupported = function () {
            if (!this._supportsReadonly) {
                if (this.editor.getConfiguration().readOnly) {
                    return false; // action requires a writeable model
                }
                var model = this.editor.getModel();
                if (model && model.hasEditableRange()) {
                    return false; // editable ranges are an indicator for mostly readonly models
                }
            }
            return true;
        };
        /**
         * Returns the enablement state of this action. This
         * method is being called in the process of {{updateEnablementState}}
         * and overwriters should call super (this method).
         */
        EditorAction.prototype.getEnablementState = function () {
            return true;
        };
        return EditorAction;
    }(actions_1.Action));
    exports.EditorAction = EditorAction;
    var HandlerEditorAction = (function (_super) {
        __extends(HandlerEditorAction, _super);
        function HandlerEditorAction(descriptor, editor, handlerId) {
            _super.call(this, descriptor, editor);
            this._handlerId = handlerId;
        }
        HandlerEditorAction.prototype.run = function () {
            this.editor.trigger(this.getId(), this._handlerId, null);
            return winjs_base_1.TPromise.as(true);
        };
        return HandlerEditorAction;
    }(EditorAction));
    exports.HandlerEditorAction = HandlerEditorAction;
    var DynamicEditorAction = (function (_super) {
        __extends(DynamicEditorAction, _super);
        function DynamicEditorAction(descriptor, editor) {
            var enablement = descriptor.enablement || {};
            _super.call(this, {
                id: descriptor.id,
                label: descriptor.label
            }, editor, DynamicEditorAction._transformBehaviour(enablement, descriptor.contextMenuGroupId));
            this._contextMenuGroupId = descriptor.contextMenuGroupId;
            this._run = descriptor.run;
            this._tokensAtPosition = enablement.tokensAtPosition;
            this._wordAtPosition = enablement.wordAtPosition;
        }
        DynamicEditorAction._transformBehaviour = function (behaviour, contextMenuGroupId) {
            var r = 0;
            if (contextMenuGroupId) {
                r |= editorActionEnablement_1.Behaviour.ShowInContextMenu;
            }
            else if (behaviour.textFocus) {
                // Allowed to set text focus only if not appearing in the context menu
                r |= editorActionEnablement_1.Behaviour.TextFocus;
            }
            if (behaviour.widgetFocus) {
                r |= editorActionEnablement_1.Behaviour.WidgetFocus;
            }
            if (behaviour.writeableEditor) {
                r |= editorActionEnablement_1.Behaviour.Writeable;
            }
            if (typeof behaviour.tokensAtPosition !== 'undefined') {
                r |= editorActionEnablement_1.Behaviour.UpdateOnCursorPositionChange;
            }
            if (typeof behaviour.wordAtPosition !== 'undefined') {
                r |= editorActionEnablement_1.Behaviour.UpdateOnCursorPositionChange;
            }
            return r;
        };
        DynamicEditorAction.prototype.getGroupId = function () {
            return this._contextMenuGroupId;
        };
        DynamicEditorAction.prototype.run = function () {
            return winjs_base_1.TPromise.as(this._run(this.editor));
        };
        DynamicEditorAction.prototype.getEnablementState = function () {
            return this._getEnablementOnTokens() && this._getEnablementOnWord();
        };
        DynamicEditorAction.prototype._getEnablementOnTokens = function () {
            if (!this._tokensAtPosition) {
                return true;
            }
            var model = this.editor.getModel(), position = this.editor.getSelection().getStartPosition(), lineContext = model.getLineContext(position.lineNumber), offset = position.column - 1;
            return isToken(lineContext, offset, this._tokensAtPosition);
        };
        DynamicEditorAction.prototype._getEnablementOnWord = function () {
            if (!this._wordAtPosition) {
                return true;
            }
            var model = this.editor.getModel(), position = this.editor.getSelection().getStartPosition(), wordAtPosition = model.getWordAtPosition(position);
            return (!!wordAtPosition);
        };
        return DynamicEditorAction;
    }(EditorAction));
    exports.DynamicEditorAction = DynamicEditorAction;
    function isToken(context, offset, types) {
        if (context.getLineContent().length <= offset) {
            return false;
        }
        var tokenIdx = context.findIndexOfOffset(offset);
        var type = context.getTokenType(tokenIdx);
        for (var i = 0, len = types.length; i < len; i++) {
            if (types[i] === '') {
                if (type === '') {
                    return true;
                }
            }
            else {
                if (strings.startsWith(type, types[i])) {
                    return true;
                }
            }
        }
        return false;
    }
});

define("vs/editor/common/model/modelLine", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/editorCommon'], function (require, exports, strings, editorCommon_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var NO_OP_TOKENS_ADJUSTER = {
        adjust: function () { },
        finish: function () { }
    };
    var NO_OP_MARKERS_ADJUSTER = {
        adjustDelta: function () { },
        adjustSet: function () { },
        finish: function () { }
    };
    var MarkerMoveSemantics;
    (function (MarkerMoveSemantics) {
        MarkerMoveSemantics[MarkerMoveSemantics["MarkerDefined"] = 0] = "MarkerDefined";
        MarkerMoveSemantics[MarkerMoveSemantics["ForceMove"] = 1] = "ForceMove";
        MarkerMoveSemantics[MarkerMoveSemantics["ForceStay"] = 2] = "ForceStay";
    })(MarkerMoveSemantics || (MarkerMoveSemantics = {}));
    var ModelLine = (function () {
        function ModelLine(lineNumber, text) {
            this.lineNumber = lineNumber;
            this.text = text;
            this.isInvalid = false;
        }
        // --- BEGIN STATE
        ModelLine.prototype.setState = function (state) {
            this._state = state;
        };
        ModelLine.prototype.getState = function () {
            return this._state || null;
        };
        // --- END STATE
        // --- BEGIN MODE TRANSITIONS
        ModelLine.prototype._setModeTransitions = function (topLevelMode, modeTransitions) {
            var desired = toModeTransitions(topLevelMode, modeTransitions);
            if (desired === null) {
                // saving memory
                if (typeof this._modeTransitions === 'undefined') {
                    return;
                }
                this._modeTransitions = null;
                return;
            }
            this._modeTransitions = desired;
        };
        ModelLine.prototype.getModeTransitions = function () {
            if (this._modeTransitions) {
                return this._modeTransitions;
            }
            return DefaultModeTransitions.INSTANCE;
        };
        // --- END MODE TRANSITIONS
        // --- BEGIN TOKENS
        ModelLine.prototype.setTokens = function (map, tokens, topLevelMode, modeTransitions) {
            this._setLineTokens(map, tokens);
            this._setModeTransitions(topLevelMode, modeTransitions);
        };
        ModelLine.prototype._setLineTokens = function (map, tokens) {
            var desired = toLineTokens(map, tokens, this.text.length);
            if (desired === null) {
                // saving memory
                if (typeof this._lineTokens === 'undefined') {
                    return;
                }
                this._lineTokens = null;
                return;
            }
            this._lineTokens = desired;
        };
        ModelLine.prototype.getTokens = function () {
            if (this._lineTokens) {
                return this._lineTokens;
            }
            if (this.text.length === 0) {
                return EmptyLineTokens.INSTANCE;
            }
            return DefaultLineTokens.INSTANCE;
        };
        // --- END TOKENS
        ModelLine.prototype._createTokensAdjuster = function () {
            if (!this._lineTokens) {
                // This line does not have real tokens, so there is nothing to adjust
                return NO_OP_TOKENS_ADJUSTER;
            }
            var lineTokens = this._lineTokens;
            var BIN = editorCommon_1.LineTokensBinaryEncoding;
            var tokens = lineTokens.getBinaryEncodedTokens();
            var tokensLength = tokens.length;
            var tokensIndex = 0;
            var currentTokenStartIndex = 0;
            var adjust = function (toColumn, delta, minimumAllowedColumn) {
                // console.log('before call: tokensIndex: ' + tokensIndex + ': ' + String(this.getTokens()));
                // console.log('adjustTokens: ' + toColumn + ' with delta: ' + delta + ' and [' + minimumAllowedColumn + ']');
                // console.log('currentTokenStartIndex: ' + currentTokenStartIndex);
                var minimumAllowedIndex = minimumAllowedColumn - 1;
                while (currentTokenStartIndex < toColumn && tokensIndex < tokensLength) {
                    if (currentTokenStartIndex > 0 && delta !== 0) {
                        // adjust token's `startIndex` by `delta`
                        var deflatedType = (tokens[tokensIndex] / BIN.TYPE_OFFSET) & BIN.TYPE_MASK;
                        var newStartIndex = Math.max(minimumAllowedIndex, currentTokenStartIndex + delta);
                        var newToken = deflatedType * BIN.TYPE_OFFSET + newStartIndex * BIN.START_INDEX_OFFSET;
                        if (delta < 0) {
                            // pop all previous tokens that have become `collapsed`
                            while (tokensIndex > 0) {
                                var prevTokenStartIndex = (tokens[tokensIndex - 1] / BIN.START_INDEX_OFFSET) & BIN.START_INDEX_MASK;
                                if (prevTokenStartIndex >= newStartIndex) {
                                    // Token at `tokensIndex` - 1 is now `collapsed` => pop it
                                    tokens.splice(tokensIndex - 1, 1);
                                    tokensLength--;
                                    tokensIndex--;
                                }
                                else {
                                    break;
                                }
                            }
                        }
                        tokens[tokensIndex] = newToken;
                    }
                    tokensIndex++;
                    if (tokensIndex < tokensLength) {
                        currentTokenStartIndex = (tokens[tokensIndex] / BIN.START_INDEX_OFFSET) & BIN.START_INDEX_MASK;
                    }
                }
                // console.log('after call: tokensIndex: ' + tokensIndex + ': ' + String(this.getTokens()));
            };
            var finish = function (delta, lineTextLength) {
                adjust(Number.MAX_VALUE, delta, 1);
            };
            return {
                adjust: adjust,
                finish: finish
            };
        };
        ModelLine.prototype._setText = function (text) {
            this.text = text;
            if (this._lineTokens) {
                var BIN = editorCommon_1.LineTokensBinaryEncoding, map = this._lineTokens.getBinaryEncodedTokensMap(), tokens = this._lineTokens.getBinaryEncodedTokens(), lineTextLength = this.text.length;
                // Remove overflowing tokens
                while (tokens.length > 0) {
                    var lastTokenStartIndex = (tokens[tokens.length - 1] / BIN.START_INDEX_OFFSET) & BIN.START_INDEX_MASK;
                    if (lastTokenStartIndex < lineTextLength) {
                        // Valid token
                        break;
                    }
                    // This token now overflows the text => remove it
                    tokens.pop();
                }
                this._setLineTokens(map, tokens);
            }
        };
        // private _printMarkers(): string {
        // 	if (!this._markers) {
        // 		return '[]';
        // 	}
        // 	if (this._markers.length === 0) {
        // 		return '[]';
        // 	}
        // 	var markers = this._markers;
        // 	var printMarker = (m:ILineMarker) => {
        // 		if (m.stickToPreviousCharacter) {
        // 			return '|' + m.column;
        // 		}
        // 		return m.column + '|';
        // 	};
        // 	return '[' + markers.map(printMarker).join(', ') + ']';
        // }
        ModelLine.prototype._createMarkersAdjuster = function (changedMarkers) {
            var _this = this;
            if (!this._markers) {
                return NO_OP_MARKERS_ADJUSTER;
            }
            if (this._markers.length === 0) {
                return NO_OP_MARKERS_ADJUSTER;
            }
            this._markers.sort(ModelLine._compareMarkers);
            var markers = this._markers;
            var markersLength = markers.length;
            var markersIndex = 0;
            var marker = markers[markersIndex];
            // console.log('------------- INITIAL MARKERS: ' + this._printMarkers());
            var adjustMarkerBeforeColumn = function (toColumn, moveSemantics) {
                if (marker.column < toColumn) {
                    return true;
                }
                if (marker.column > toColumn) {
                    return false;
                }
                if (moveSemantics === MarkerMoveSemantics.ForceMove) {
                    return false;
                }
                if (moveSemantics === MarkerMoveSemantics.ForceStay) {
                    return true;
                }
                return marker.stickToPreviousCharacter;
            };
            var adjustDelta = function (toColumn, delta, minimumAllowedColumn, moveSemantics) {
                // console.log('------------------------------');
                // console.log('adjustDelta called: toColumn: ' + toColumn + ', delta: ' + delta + ', minimumAllowedColumn: ' + minimumAllowedColumn + ', moveSemantics: ' + MarkerMoveSemantics[moveSemantics]);
                // console.log('BEFORE::: markersIndex: ' + markersIndex + ' : ' + this._printMarkers());
                while (markersIndex < markersLength && adjustMarkerBeforeColumn(toColumn, moveSemantics)) {
                    if (delta !== 0) {
                        var newColumn = Math.max(minimumAllowedColumn, marker.column + delta);
                        if (marker.column !== newColumn) {
                            changedMarkers[marker.id] = true;
                            marker.oldLineNumber = marker.oldLineNumber || _this.lineNumber;
                            marker.oldColumn = marker.oldColumn || marker.column;
                            marker.column = newColumn;
                        }
                    }
                    markersIndex++;
                    if (markersIndex < markersLength) {
                        marker = markers[markersIndex];
                    }
                }
                // console.log('AFTER::: markersIndex: ' + markersIndex + ' : ' + this._printMarkers());
            };
            var adjustSet = function (toColumn, newColumn, moveSemantics) {
                // console.log('------------------------------');
                // console.log('adjustSet called: toColumn: ' + toColumn + ', newColumn: ' + newColumn + ', moveSemantics: ' + MarkerMoveSemantics[moveSemantics]);
                // console.log('BEFORE::: markersIndex: ' + markersIndex + ' : ' + this._printMarkers());
                while (markersIndex < markersLength && adjustMarkerBeforeColumn(toColumn, moveSemantics)) {
                    if (marker.column !== newColumn) {
                        changedMarkers[marker.id] = true;
                        marker.oldLineNumber = marker.oldLineNumber || _this.lineNumber;
                        marker.oldColumn = marker.oldColumn || marker.column;
                        marker.column = newColumn;
                    }
                    markersIndex++;
                    if (markersIndex < markersLength) {
                        marker = markers[markersIndex];
                    }
                }
                // console.log('AFTER::: markersIndex: ' + markersIndex + ' : ' + this._printMarkers());
            };
            var finish = function (delta, lineTextLength) {
                adjustDelta(Number.MAX_VALUE, delta, 1, MarkerMoveSemantics.MarkerDefined);
                // console.log('------------- FINAL MARKERS: ' + this._printMarkers());
            };
            return {
                adjustDelta: adjustDelta,
                adjustSet: adjustSet,
                finish: finish
            };
        };
        ModelLine.prototype.applyEdits = function (changedMarkers, edits) {
            var deltaColumn = 0;
            var resultText = this.text;
            var tokensAdjuster = this._createTokensAdjuster();
            var markersAdjuster = this._createMarkersAdjuster(changedMarkers);
            for (var i = 0, len = edits.length; i < len; i++) {
                var edit = edits[i];
                // console.log();
                // console.log('=============================');
                // console.log('EDIT #' + i + ' [ ' + edit.startColumn + ' -> ' + edit.endColumn + ' ] : <<<' + edit.text + '>>>, forceMoveMarkers: ' + edit.forceMoveMarkers);
                // console.log('deltaColumn: ' + deltaColumn);
                var startColumn = deltaColumn + edit.startColumn;
                var endColumn = deltaColumn + edit.endColumn;
                var deletingCnt = endColumn - startColumn;
                var insertingCnt = edit.text.length;
                // Adjust tokens & markers before this edit
                // console.log('Adjust tokens & markers before this edit');
                tokensAdjuster.adjust(edit.startColumn - 1, deltaColumn, 1);
                markersAdjuster.adjustDelta(edit.startColumn, deltaColumn, 1, edit.forceMoveMarkers ? MarkerMoveSemantics.ForceMove : (deletingCnt > 0 ? MarkerMoveSemantics.ForceStay : MarkerMoveSemantics.MarkerDefined));
                // Adjust tokens & markers for the common part of this edit
                var commonLength = Math.min(deletingCnt, insertingCnt);
                if (commonLength > 0) {
                    // console.log('Adjust tokens & markers for the common part of this edit');
                    tokensAdjuster.adjust(edit.startColumn - 1 + commonLength, deltaColumn, startColumn);
                    if (!edit.forceMoveMarkers) {
                        markersAdjuster.adjustDelta(edit.startColumn + commonLength, deltaColumn, startColumn, edit.forceMoveMarkers ? MarkerMoveSemantics.ForceMove : (deletingCnt > insertingCnt ? MarkerMoveSemantics.ForceStay : MarkerMoveSemantics.MarkerDefined));
                    }
                }
                // Perform the edit & update `deltaColumn`
                resultText = resultText.substring(0, startColumn - 1) + edit.text + resultText.substring(endColumn - 1);
                deltaColumn += insertingCnt - deletingCnt;
                // Adjust tokens & markers inside this edit
                // console.log('Adjust tokens & markers inside this edit');
                tokensAdjuster.adjust(edit.endColumn, deltaColumn, startColumn);
                markersAdjuster.adjustSet(edit.endColumn, startColumn + insertingCnt, edit.forceMoveMarkers ? MarkerMoveSemantics.ForceMove : MarkerMoveSemantics.MarkerDefined);
            }
            // Wrap up tokens & markers; adjust remaining if needed
            tokensAdjuster.finish(deltaColumn, resultText.length);
            markersAdjuster.finish(deltaColumn, resultText.length);
            // Save the resulting text
            this._setText(resultText);
            return deltaColumn;
        };
        ModelLine.prototype.split = function (changedMarkers, splitColumn, forceMoveMarkers) {
            // console.log('--> split @ ' + splitColumn + '::: ' + this._printMarkers());
            var myText = this.text.substring(0, splitColumn - 1);
            var otherText = this.text.substring(splitColumn - 1);
            var otherMarkers = null;
            if (this._markers) {
                this._markers.sort(ModelLine._compareMarkers);
                for (var i = 0, len = this._markers.length; i < len; i++) {
                    var marker = this._markers[i];
                    if (marker.column > splitColumn
                        || (marker.column === splitColumn
                            && (forceMoveMarkers
                                || !marker.stickToPreviousCharacter))) {
                        var myMarkers = this._markers.slice(0, i);
                        otherMarkers = this._markers.slice(i);
                        this._markers = myMarkers;
                        break;
                    }
                }
                if (otherMarkers) {
                    for (var i = 0, len = otherMarkers.length; i < len; i++) {
                        var marker = otherMarkers[i];
                        changedMarkers[marker.id] = true;
                        marker.oldLineNumber = marker.oldLineNumber || this.lineNumber;
                        marker.oldColumn = marker.oldColumn || marker.column;
                        marker.column -= splitColumn - 1;
                    }
                }
            }
            this._setText(myText);
            var otherLine = new ModelLine(this.lineNumber + 1, otherText);
            if (otherMarkers) {
                otherLine.addMarkers(otherMarkers);
            }
            return otherLine;
        };
        ModelLine.prototype.append = function (changedMarkers, other) {
            // console.log('--> append: THIS :: ' + this._printMarkers());
            // console.log('--> append: OTHER :: ' + this._printMarkers());
            var thisTextLength = this.text.length;
            this._setText(this.text + other.text);
            var otherLineTokens = other._lineTokens;
            if (otherLineTokens) {
                // Other has real tokens
                var otherTokens = otherLineTokens.getBinaryEncodedTokens();
                // Adjust other tokens
                if (thisTextLength > 0) {
                    var BIN = editorCommon_1.LineTokensBinaryEncoding;
                    for (var i = 0, len = otherTokens.length; i < len; i++) {
                        var token = otherTokens[i];
                        var deflatedStartIndex = (token / BIN.START_INDEX_OFFSET) & BIN.START_INDEX_MASK;
                        var deflatedType = (token / BIN.TYPE_OFFSET) & BIN.TYPE_MASK;
                        var newStartIndex = deflatedStartIndex + thisTextLength;
                        var newToken = deflatedType * BIN.TYPE_OFFSET + newStartIndex * BIN.START_INDEX_OFFSET;
                        otherTokens[i] = newToken;
                    }
                }
                // Append other tokens
                var myLineTokens = this._lineTokens;
                if (myLineTokens) {
                    // I have real tokens
                    this._setLineTokens(myLineTokens.getBinaryEncodedTokensMap(), myLineTokens.getBinaryEncodedTokens().concat(otherTokens));
                }
                else {
                    // I don't have real tokens
                    this._setLineTokens(otherLineTokens.getBinaryEncodedTokensMap(), otherTokens);
                }
            }
            if (other._markers) {
                // Other has markers
                var otherMarkers = other._markers;
                // Adjust other markers
                for (var i = 0, len = otherMarkers.length; i < len; i++) {
                    var marker = otherMarkers[i];
                    changedMarkers[marker.id] = true;
                    marker.oldLineNumber = marker.oldLineNumber || other.lineNumber;
                    marker.oldColumn = marker.oldColumn || marker.column;
                    marker.column += thisTextLength;
                }
                this.addMarkers(otherMarkers);
            }
        };
        ModelLine.prototype.addMarker = function (marker) {
            marker.line = this;
            if (!this._markers) {
                this._markers = [marker];
            }
            else {
                this._markers.push(marker);
            }
        };
        ModelLine.prototype.addMarkers = function (markers) {
            if (markers.length === 0) {
                return;
            }
            var i, len;
            for (i = 0, len = markers.length; i < len; i++) {
                markers[i].line = this;
            }
            if (!this._markers) {
                this._markers = markers.slice(0);
            }
            else {
                this._markers = this._markers.concat(markers);
            }
        };
        ModelLine._compareMarkers = function (a, b) {
            if (a.column === b.column) {
                return (a.stickToPreviousCharacter ? 0 : 1) - (b.stickToPreviousCharacter ? 0 : 1);
            }
            return a.column - b.column;
        };
        ModelLine.prototype.removeMarker = function (marker) {
            var index = this._indexOfMarkerId(marker.id);
            if (index >= 0) {
                this._markers.splice(index, 1);
            }
            marker.line = null;
        };
        ModelLine.prototype.removeMarkers = function (deleteMarkers) {
            if (!this._markers) {
                return;
            }
            for (var i = 0, len = this._markers.length; i < len; i++) {
                var marker = this._markers[i];
                if (deleteMarkers[marker.id]) {
                    marker.line = null;
                    this._markers.splice(i, 1);
                    len--;
                    i--;
                }
            }
        };
        ModelLine.prototype.getMarkers = function () {
            if (!this._markers) {
                return [];
            }
            return this._markers.slice(0);
        };
        ModelLine.prototype.updateLineNumber = function (changedMarkers, newLineNumber) {
            if (this._markers) {
                var markers = this._markers, i, len, marker;
                for (i = 0, len = markers.length; i < len; i++) {
                    marker = markers[i];
                    changedMarkers[marker.id] = true;
                    marker.oldLineNumber = marker.oldLineNumber || this.lineNumber;
                }
            }
            this.lineNumber = newLineNumber;
        };
        ModelLine.prototype.deleteLine = function (changedMarkers, setMarkersColumn, setMarkersOldLineNumber) {
            // console.log('--> deleteLine: ');
            if (this._markers) {
                var markers = this._markers, i, len, marker;
                // Mark all these markers as changed
                for (i = 0, len = markers.length; i < len; i++) {
                    marker = markers[i];
                    changedMarkers[marker.id] = true;
                    marker.oldColumn = marker.oldColumn || marker.column;
                    marker.oldLineNumber = marker.oldLineNumber || setMarkersOldLineNumber;
                    marker.column = setMarkersColumn;
                }
                return markers;
            }
            return [];
        };
        ModelLine.prototype._indexOfMarkerId = function (markerId) {
            if (this._markers) {
                var markers = this._markers, i, len;
                for (i = 0, len = markers.length; i < len; i++) {
                    if (markers[i].id === markerId) {
                        return i;
                    }
                }
            }
            return -1;
        };
        return ModelLine;
    }());
    exports.ModelLine = ModelLine;
    function areDeflatedTokens(tokens) {
        return (typeof tokens[0] === 'number');
    }
    function toLineTokens(map, tokens, textLength) {
        if (textLength === 0) {
            return null;
        }
        if (!tokens || tokens.length === 0) {
            return null;
        }
        if (tokens.length === 1) {
            if (areDeflatedTokens(tokens)) {
                if (tokens[0] === 0) {
                    return null;
                }
            }
            else {
                if (tokens[0].startIndex === 0 && tokens[0].type === '') {
                    return null;
                }
            }
        }
        return new LineTokens(map, tokens);
    }
    var getStartIndex = editorCommon_1.LineTokensBinaryEncoding.getStartIndex;
    var getType = editorCommon_1.LineTokensBinaryEncoding.getType;
    var findIndexOfOffset = editorCommon_1.LineTokensBinaryEncoding.findIndexOfOffset;
    var LineTokens = (function () {
        function LineTokens(map, tokens) {
            this.map = map;
            if (areDeflatedTokens(tokens)) {
                this._tokens = tokens;
            }
            else {
                this._tokens = editorCommon_1.LineTokensBinaryEncoding.deflateArr(map, tokens);
            }
        }
        LineTokens.prototype.toString = function () {
            return editorCommon_1.LineTokensBinaryEncoding.inflateArr(this.map, this._tokens).toString();
        };
        LineTokens.prototype.getBinaryEncodedTokensMap = function () {
            return this.map;
        };
        LineTokens.prototype.getBinaryEncodedTokens = function () {
            return this._tokens;
        };
        LineTokens.prototype.getTokenCount = function () {
            return this._tokens.length;
        };
        LineTokens.prototype.getTokenStartIndex = function (tokenIndex) {
            return getStartIndex(this._tokens[tokenIndex]);
        };
        LineTokens.prototype.getTokenType = function (tokenIndex) {
            return getType(this.map, this._tokens[tokenIndex]);
        };
        LineTokens.prototype.getTokenEndIndex = function (tokenIndex, textLength) {
            if (tokenIndex + 1 < this._tokens.length) {
                return getStartIndex(this._tokens[tokenIndex + 1]);
            }
            return textLength;
        };
        LineTokens.prototype.equals = function (other) {
            return this === other;
        };
        LineTokens.prototype.findIndexOfOffset = function (offset) {
            return findIndexOfOffset(this._tokens, offset);
        };
        return LineTokens;
    }());
    exports.LineTokens = LineTokens;
    var EmptyLineTokens = (function () {
        function EmptyLineTokens() {
        }
        EmptyLineTokens.prototype.getBinaryEncodedTokens = function () {
            return EmptyLineTokens.TOKENS;
        };
        EmptyLineTokens.prototype.getBinaryEncodedTokensMap = function () {
            return null;
        };
        EmptyLineTokens.prototype.getTokenCount = function () {
            return 0;
        };
        EmptyLineTokens.prototype.getTokenStartIndex = function (tokenIndex) {
            return 0;
        };
        EmptyLineTokens.prototype.getTokenType = function (tokenIndex) {
            return strings.empty;
        };
        EmptyLineTokens.prototype.getTokenEndIndex = function (tokenIndex, textLength) {
            return 0;
        };
        EmptyLineTokens.prototype.equals = function (other) {
            return other === this;
        };
        EmptyLineTokens.prototype.findIndexOfOffset = function (offset) {
            return 0;
        };
        EmptyLineTokens.INSTANCE = new EmptyLineTokens();
        EmptyLineTokens.TOKENS = [];
        return EmptyLineTokens;
    }());
    var DefaultLineTokens = (function () {
        function DefaultLineTokens() {
        }
        DefaultLineTokens.prototype.getBinaryEncodedTokensMap = function () {
            return null;
        };
        DefaultLineTokens.prototype.getBinaryEncodedTokens = function () {
            return DefaultLineTokens.TOKENS;
        };
        DefaultLineTokens.prototype.getTokenCount = function () {
            return 1;
        };
        DefaultLineTokens.prototype.getTokenStartIndex = function (tokenIndex) {
            return 0;
        };
        DefaultLineTokens.prototype.getTokenType = function (tokenIndex) {
            return strings.empty;
        };
        DefaultLineTokens.prototype.getTokenEndIndex = function (tokenIndex, textLength) {
            return textLength;
        };
        DefaultLineTokens.prototype.equals = function (other) {
            return this === other;
        };
        DefaultLineTokens.prototype.findIndexOfOffset = function (offset) {
            return 0;
        };
        DefaultLineTokens.INSTANCE = new DefaultLineTokens();
        DefaultLineTokens.TOKENS = [0];
        return DefaultLineTokens;
    }());
    exports.DefaultLineTokens = DefaultLineTokens;
    function toModeTransitions(topLevelMode, modeTransitions) {
        if (!modeTransitions || modeTransitions.length === 0) {
            return null;
        }
        else if (modeTransitions.length === 1 && modeTransitions[0].startIndex === 0) {
            if (modeTransitions[0].mode === topLevelMode) {
                return null;
            }
            else {
                return new SingleModeTransition(modeTransitions[0].mode);
            }
        }
        return new ModeTransitions(modeTransitions);
    }
    var DefaultModeTransitions = (function () {
        function DefaultModeTransitions() {
        }
        DefaultModeTransitions.prototype.toArray = function (topLevelMode) {
            return [{
                    startIndex: 0,
                    mode: topLevelMode
                }];
        };
        DefaultModeTransitions.INSTANCE = new DefaultModeTransitions();
        return DefaultModeTransitions;
    }());
    var SingleModeTransition = (function () {
        function SingleModeTransition(mode) {
            this._mode = mode;
        }
        SingleModeTransition.prototype.toArray = function (topLevelMode) {
            return [{
                    startIndex: 0,
                    mode: this._mode
                }];
        };
        return SingleModeTransition;
    }());
    var ModeTransitions = (function () {
        function ModeTransitions(modeTransitions) {
            this._modeTransitions = modeTransitions;
        }
        ModeTransitions.prototype.toArray = function (topLevelMode) {
            return this._modeTransitions.slice(0);
        };
        return ModeTransitions;
    }());
});

define("vs/editor/common/model/tokenIterator", ["require", "exports", 'vs/editor/common/editorCommon'], function (require, exports, editorCommon) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var getStartIndex = editorCommon.LineTokensBinaryEncoding.getStartIndex;
    var inflate = editorCommon.LineTokensBinaryEncoding.inflate;
    var TokenIterator = (function () {
        function TokenIterator(model, position) {
            this._model = model;
            this._currentLineNumber = position.lineNumber;
            this._currentTokenIndex = 0;
            this._readLineTokens(this._currentLineNumber);
            this._next = null;
            this._prev = null;
            // start with a position to next/prev run
            var columnIndex = position.column - 1, tokenEndIndex = Number.MAX_VALUE;
            for (var i = this._currentTokens.length - 1; i >= 0; i--) {
                if (getStartIndex(this._currentTokens[i]) <= columnIndex && columnIndex <= tokenEndIndex) {
                    this._currentTokenIndex = i;
                    this._next = this._current();
                    this._prev = this._current();
                    break;
                }
                tokenEndIndex = getStartIndex(this._currentTokens[i]);
            }
        }
        TokenIterator.prototype._readLineTokens = function (lineNumber) {
            this._currentLineTokens = this._model.getLineTokens(lineNumber, false);
            this._currentTokens = this._currentLineTokens.getBinaryEncodedTokens();
            this._map = this._currentLineTokens.getBinaryEncodedTokensMap();
        };
        TokenIterator.prototype._advanceNext = function () {
            this._prev = this._next;
            this._next = null;
            if (this._currentTokenIndex + 1 < this._currentTokens.length) {
                // There are still tokens on current line
                this._currentTokenIndex++;
                this._next = this._current();
            }
            else {
                // find the next line with tokens
                while (this._currentLineNumber + 1 <= this._model.getLineCount()) {
                    this._currentLineNumber++;
                    this._readLineTokens(this._currentLineNumber);
                    if (this._currentTokens.length > 0) {
                        this._currentTokenIndex = 0;
                        this._next = this._current();
                        break;
                    }
                }
                if (this._next === null) {
                    // prepare of a previous run
                    this._readLineTokens(this._currentLineNumber);
                    this._currentTokenIndex = this._currentTokens.length;
                    this._advancePrev();
                    this._next = null;
                }
            }
        };
        TokenIterator.prototype._advancePrev = function () {
            this._next = this._prev;
            this._prev = null;
            if (this._currentTokenIndex > 0) {
                // There are still tokens on current line
                this._currentTokenIndex--;
                this._prev = this._current();
            }
            else {
                // find previous line with tokens
                while (this._currentLineNumber > 1) {
                    this._currentLineNumber--;
                    this._readLineTokens(this._currentLineNumber);
                    if (this._currentTokens.length > 0) {
                        this._currentTokenIndex = this._currentTokens.length - 1;
                        this._prev = this._current();
                        break;
                    }
                }
            }
        };
        TokenIterator.prototype._current = function () {
            return {
                token: inflate(this._map, this._currentTokens[this._currentTokenIndex]),
                lineNumber: this._currentLineNumber,
                startColumn: getStartIndex(this._currentTokens[this._currentTokenIndex]) + 1,
                endColumn: this._currentTokenIndex + 1 < this._currentTokens.length ? getStartIndex(this._currentTokens[this._currentTokenIndex + 1]) + 1 : this._model.getLineContent(this._currentLineNumber).length + 1
            };
        };
        TokenIterator.prototype.hasNext = function () {
            return this._next !== null;
        };
        TokenIterator.prototype.next = function () {
            var result = this._next;
            this._advanceNext();
            return result;
        };
        TokenIterator.prototype.hasPrev = function () {
            return this._prev !== null;
        };
        TokenIterator.prototype.prev = function () {
            var result = this._prev;
            this._advancePrev();
            return result;
        };
        TokenIterator.prototype._invalidate = function () {
            // replace all public functions with errors
            var errorFn = function () {
                throw new Error('iteration isn\'t valid anymore');
            };
            this.hasNext = errorFn;
            this.next = errorFn;
            this.hasPrev = errorFn;
            this.prev = errorFn;
        };
        return TokenIterator;
    }());
    exports.TokenIterator = TokenIterator;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/common/modes/languageFeatureRegistry", ["require", "exports", 'vs/base/common/arrays', 'vs/base/common/event', 'vs/editor/common/modes/languageSelector'], function (require, exports, arrays_1, event_1, languageSelector_1) {
    'use strict';
    var LanguageFeatureRegistry = (function () {
        function LanguageFeatureRegistry(supportName) {
            this._clock = 0;
            this._entries = [];
            this._onDidChange = new event_1.Emitter();
            this._supportName = supportName;
        }
        Object.defineProperty(LanguageFeatureRegistry.prototype, "onDidChange", {
            get: function () {
                return this._onDidChange.event;
            },
            enumerable: true,
            configurable: true
        });
        LanguageFeatureRegistry.prototype.register = function (selector, provider) {
            var _this = this;
            var entry = {
                selector: selector,
                provider: provider,
                _score: -1,
                _time: this._clock++
            };
            this._entries.push(entry);
            this._lastCandidate = undefined;
            this._onDidChange.fire(this._entries.length);
            return {
                dispose: function () {
                    if (entry) {
                        var idx = _this._entries.indexOf(entry);
                        if (idx >= 0) {
                            _this._entries.splice(idx, 1);
                            _this._lastCandidate = undefined;
                            _this._onDidChange.fire(_this._entries.length);
                            entry = undefined;
                        }
                    }
                }
            };
        };
        LanguageFeatureRegistry.prototype.has = function (model) {
            return this.all(model).length > 0;
        };
        LanguageFeatureRegistry.prototype.all = function (model) {
            if (!model || model.isTooLargeForHavingAMode()) {
                return [];
            }
            this._updateScores(model);
            var result = [];
            // (1) from registry
            for (var _i = 0, _a = this._entries; _i < _a.length; _i++) {
                var entry = _a[_i];
                if (entry._score > 0) {
                    result.push(entry.provider);
                }
            }
            // (2) from mode
            if (model.getMode() && model.getMode()[this._supportName]) {
                result.push(model.getMode()[this._supportName]);
            }
            return result;
        };
        LanguageFeatureRegistry.prototype.ordered = function (model) {
            var result = [];
            this._orderedForEach(model, function (entry) { return result.push(entry.provider); });
            return result;
        };
        LanguageFeatureRegistry.prototype.orderedGroups = function (model) {
            var result = [];
            var lastBucket;
            var lastBucketScore;
            this._orderedForEach(model, function (entry) {
                if (lastBucket && lastBucketScore === entry._score) {
                    lastBucket.push(entry.provider);
                }
                else {
                    lastBucketScore = entry._score;
                    lastBucket = [entry.provider];
                    result.push(lastBucket);
                }
            });
            return result;
        };
        LanguageFeatureRegistry.prototype._orderedForEach = function (model, callback) {
            if (!model || model.isTooLargeForHavingAMode()) {
                return;
            }
            this._updateScores(model);
            var supportIndex = -1;
            var supportEntry;
            if (model.getMode() && model.getMode()[this._supportName]) {
                supportEntry = {
                    selector: undefined,
                    provider: model.getMode()[this._supportName],
                    _score: .5,
                    _time: -1
                };
                supportIndex = ~arrays_1.binarySearch(this._entries, supportEntry, LanguageFeatureRegistry._compareByScoreAndTime);
            }
            var to = Math.max(supportIndex + 1, this._entries.length);
            for (var from = 0; from < to; from++) {
                if (from === supportIndex) {
                    callback(supportEntry);
                }
                else {
                    var entry = this._entries[from];
                    if (entry._score > 0) {
                        callback(entry);
                    }
                }
            }
        };
        LanguageFeatureRegistry.prototype._updateScores = function (model) {
            var candidate = {
                uri: model.getAssociatedResource().toString(),
                language: model.getModeId()
            };
            if (this._lastCandidate
                && this._lastCandidate.language === candidate.language
                && this._lastCandidate.uri === candidate.uri) {
                // nothing has changed
                return;
            }
            this._lastCandidate = candidate;
            for (var _i = 0, _a = this._entries; _i < _a.length; _i++) {
                var entry = _a[_i];
                entry._score = languageSelector_1.score(entry.selector, model.getAssociatedResource(), model.getModeId());
            }
            // needs sorting
            this._entries.sort(LanguageFeatureRegistry._compareByScoreAndTime);
        };
        LanguageFeatureRegistry._compareByScoreAndTime = function (a, b) {
            if (a._score < b._score) {
                return 1;
            }
            else if (a._score > b._score) {
                return -1;
            }
            else if (a._time < b._time) {
                return 1;
            }
            else if (a._time > b._time) {
                return -1;
            }
            else {
                return 0;
            }
        };
        return LanguageFeatureRegistry;
    }());
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = LanguageFeatureRegistry;
});

define("vs/editor/common/modes/supports/onEnter", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/strings', 'vs/editor/common/core/position', 'vs/editor/common/modes', 'vs/editor/common/modes/supports'], function (require, exports, errors_1, strings, position_1, modes_1, supports_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var OnEnterSupport = (function () {
        function OnEnterSupport(modeId, opts) {
            opts = opts || {};
            opts.brackets = opts.brackets || [
                ['(', ')'],
                ['{', '}'],
                ['[', ']']
            ];
            this._modeId = modeId;
            this._brackets = opts.brackets.map(function (bracket) {
                return {
                    open: bracket[0],
                    openRegExp: OnEnterSupport._createOpenBracketRegExp(bracket[0]),
                    close: bracket[1],
                    closeRegExp: OnEnterSupport._createCloseBracketRegExp(bracket[1]),
                };
            });
            this._regExpRules = opts.regExpRules || [];
            this._indentationRules = opts.indentationRules;
        }
        OnEnterSupport.prototype.onEnter = function (model, position) {
            var _this = this;
            var context = model.getLineContext(position.lineNumber);
            return supports_1.handleEvent(context, position.column - 1, function (nestedMode, context, offset) {
                if (_this._modeId === nestedMode.getId()) {
                    return _this._onEnter(model, position);
                }
                else if (nestedMode.richEditSupport && nestedMode.richEditSupport.onEnter) {
                    return nestedMode.richEditSupport.onEnter.onEnter(model, position);
                }
                else {
                    return null;
                }
            });
        };
        OnEnterSupport.prototype._onEnter = function (model, position) {
            var lineText = model.getLineContent(position.lineNumber);
            var beforeEnterText = lineText.substr(0, position.column - 1);
            var afterEnterText = lineText.substr(position.column - 1);
            var oneLineAboveText = position.lineNumber === 1 ? '' : model.getLineContent(position.lineNumber - 1);
            return this._actualOnEnter(oneLineAboveText, beforeEnterText, afterEnterText);
        };
        OnEnterSupport.prototype._actualOnEnter = function (oneLineAboveText, beforeEnterText, afterEnterText) {
            // (1): `regExpRules`
            for (var i = 0, len = this._regExpRules.length; i < len; i++) {
                var rule = this._regExpRules[i];
                if (rule.beforeText.test(beforeEnterText)) {
                    if (rule.afterText) {
                        if (rule.afterText.test(afterEnterText)) {
                            return rule.action;
                        }
                    }
                    else {
                        return rule.action;
                    }
                }
            }
            // (2): Special indent-outdent
            if (beforeEnterText.length > 0 && afterEnterText.length > 0) {
                for (var i = 0, len = this._brackets.length; i < len; i++) {
                    var bracket = this._brackets[i];
                    if (bracket.openRegExp.test(beforeEnterText) && bracket.closeRegExp.test(afterEnterText)) {
                        return OnEnterSupport._INDENT_OUTDENT;
                    }
                }
            }
            // (3): Indentation Support
            if (this._indentationRules) {
                if (this._indentationRules.increaseIndentPattern && this._indentationRules.increaseIndentPattern.test(beforeEnterText)) {
                    return OnEnterSupport._INDENT;
                }
                if (this._indentationRules.indentNextLinePattern && this._indentationRules.indentNextLinePattern.test(beforeEnterText)) {
                    return OnEnterSupport._INDENT;
                }
                if (/^\s/.test(beforeEnterText)) {
                    // No reason to run regular expressions if there is nothing to outdent from
                    if (this._indentationRules.decreaseIndentPattern && this._indentationRules.decreaseIndentPattern.test(afterEnterText)) {
                        return OnEnterSupport._OUTDENT;
                    }
                    if (this._indentationRules.indentNextLinePattern && this._indentationRules.indentNextLinePattern.test(oneLineAboveText)) {
                        return OnEnterSupport._OUTDENT;
                    }
                }
            }
            // (4): Open bracket based logic
            if (beforeEnterText.length > 0) {
                for (var i = 0, len = this._brackets.length; i < len; i++) {
                    var bracket = this._brackets[i];
                    if (bracket.openRegExp.test(beforeEnterText)) {
                        return OnEnterSupport._INDENT;
                    }
                }
            }
            return null;
        };
        OnEnterSupport._createOpenBracketRegExp = function (bracket) {
            var str = strings.escapeRegExpCharacters(bracket);
            if (!/\B/.test(str.charAt(0))) {
                str = '\\b' + str;
            }
            str += '\\s*$';
            return OnEnterSupport._safeRegExp(str);
        };
        OnEnterSupport._createCloseBracketRegExp = function (bracket) {
            var str = strings.escapeRegExpCharacters(bracket);
            if (!/\B/.test(str.charAt(str.length - 1))) {
                str = str + '\\b';
            }
            str = '^\\s*' + str;
            return OnEnterSupport._safeRegExp(str);
        };
        OnEnterSupport._safeRegExp = function (def) {
            try {
                return new RegExp(def);
            }
            catch (err) {
                errors_1.onUnexpectedError(err);
                return null;
            }
        };
        OnEnterSupport._INDENT = { indentAction: modes_1.IndentAction.Indent };
        OnEnterSupport._INDENT_OUTDENT = { indentAction: modes_1.IndentAction.IndentOutdent };
        OnEnterSupport._OUTDENT = { indentAction: modes_1.IndentAction.Outdent };
        return OnEnterSupport;
    }());
    exports.OnEnterSupport = OnEnterSupport;
    function getRawEnterActionAtPosition(model, lineNumber, column) {
        var result;
        var richEditSupport = model.getMode().richEditSupport;
        if (richEditSupport && richEditSupport.onEnter) {
            try {
                result = richEditSupport.onEnter.onEnter(model, new position_1.Position(lineNumber, column));
            }
            catch (e) {
                errors_1.onUnexpectedError(e);
            }
        }
        return result;
    }
    exports.getRawEnterActionAtPosition = getRawEnterActionAtPosition;
    function getEnterActionAtPosition(model, lineNumber, column) {
        var lineText = model.getLineContent(lineNumber);
        var indentation = strings.getLeadingWhitespace(lineText);
        if (indentation.length > column - 1) {
            indentation = indentation.substring(0, column - 1);
        }
        var enterAction = getRawEnterActionAtPosition(model, lineNumber, column);
        if (!enterAction) {
            enterAction = {
                indentAction: modes_1.IndentAction.None,
                appendText: '',
            };
        }
        else {
            if (!enterAction.appendText) {
                if ((enterAction.indentAction === modes_1.IndentAction.Indent) ||
                    (enterAction.indentAction === modes_1.IndentAction.IndentOutdent)) {
                    enterAction.appendText = '\t';
                }
                else {
                    enterAction.appendText = '';
                }
            }
        }
        if (enterAction.removeText) {
            indentation = indentation.substring(0, indentation.length - 1);
        }
        return {
            enterAction: enterAction,
            indentation: indentation
        };
    }
    exports.getEnterActionAtPosition = getEnterActionAtPosition;
});

define("vs/editor/common/commands/shiftCommand", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/controller/cursorMoveHelper', 'vs/editor/common/core/range', 'vs/editor/common/core/selection', 'vs/editor/common/modes/supports/onEnter'], function (require, exports, strings, cursorMoveHelper_1, range_1, selection_1, onEnter_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ShiftCommand = (function () {
        function ShiftCommand(range, opts) {
            this._opts = opts;
            this._selection = range;
            this._useLastEditRangeForCursorEndPosition = false;
        }
        ShiftCommand.unshiftIndentCount = function (line, column, tabSize) {
            // Determine the visible column where the content starts
            var contentStartVisibleColumn = cursorMoveHelper_1.CursorMoveHelper.visibleColumnFromColumn2(line, column, tabSize);
            var desiredTabStop = cursorMoveHelper_1.CursorMoveHelper.prevTabColumn(contentStartVisibleColumn, tabSize);
            // The `desiredTabStop` is a multiple of `tabSize` => determine the number of indents
            return desiredTabStop / tabSize;
        };
        ShiftCommand.shiftIndentCount = function (line, column, tabSize) {
            // Determine the visible column where the content starts
            var contentStartVisibleColumn = cursorMoveHelper_1.CursorMoveHelper.visibleColumnFromColumn2(line, column, tabSize);
            var desiredTabStop = cursorMoveHelper_1.CursorMoveHelper.nextTabColumn(contentStartVisibleColumn, tabSize);
            // The `desiredTabStop` is a multiple of `tabSize` => determine the number of indents
            return desiredTabStop / tabSize;
        };
        ShiftCommand.prototype.getEditOperations = function (model, builder) {
            var startLine = this._selection.startLineNumber, endLine = this._selection.endLineNumber, _SPACE = ' '.charCodeAt(0);
            if (this._selection.endColumn === 1 && startLine !== endLine) {
                endLine = endLine - 1;
            }
            var lineNumber, tabSize = this._opts.tabSize, oneIndent = this._opts.oneIndent, shouldIndentEmptyLines = (startLine === endLine);
            // indents[i] represents i * oneIndent
            var indents = ['', oneIndent];
            // if indenting or outdenting on a whitespace only line
            if (this._selection.isEmpty()) {
                if (/^\s*$/.test(model.getLineContent(startLine))) {
                    this._useLastEditRangeForCursorEndPosition = true;
                }
            }
            // keep track of previous line's "miss-alignment"
            var previousLineExtraSpaces = 0, extraSpaces = 0;
            for (lineNumber = startLine; lineNumber <= endLine; lineNumber++, previousLineExtraSpaces = extraSpaces) {
                extraSpaces = 0;
                var lineText = model.getLineContent(lineNumber);
                var indentationEndIndex = strings.firstNonWhitespaceIndex(lineText);
                if (this._opts.isUnshift && (lineText.length === 0 || indentationEndIndex === 0)) {
                    // empty line or line with no leading whitespace => nothing to do
                    continue;
                }
                if (!shouldIndentEmptyLines && !this._opts.isUnshift && lineText.length === 0) {
                    // do not indent empty lines => nothing to do
                    continue;
                }
                if (indentationEndIndex === -1) {
                    // the entire line is whitespace
                    indentationEndIndex = lineText.length;
                }
                if (lineNumber > 1) {
                    var contentStartVisibleColumn = cursorMoveHelper_1.CursorMoveHelper.visibleColumnFromColumn2(lineText, indentationEndIndex + 1, tabSize);
                    if (contentStartVisibleColumn % tabSize !== 0) {
                        // The current line is "miss-aligned", so let's see if this is expected...
                        // This can only happen when it has trailing commas in the indent
                        var enterAction = onEnter_1.getRawEnterActionAtPosition(model, lineNumber - 1, model.getLineMaxColumn(lineNumber - 1));
                        if (enterAction) {
                            extraSpaces = previousLineExtraSpaces;
                            if (enterAction.appendText) {
                                for (var j = 0, lenJ = enterAction.appendText.length; j < lenJ && extraSpaces < tabSize; j++) {
                                    if (enterAction.appendText.charCodeAt(j) === _SPACE) {
                                        extraSpaces++;
                                    }
                                    else {
                                        break;
                                    }
                                }
                            }
                            if (enterAction.removeText) {
                                extraSpaces = Math.max(0, extraSpaces - enterAction.removeText);
                            }
                            // Act as if `prefixSpaces` is not part of the indentation
                            for (var j = 0; j < extraSpaces; j++) {
                                if (indentationEndIndex === 0 || lineText.charCodeAt(indentationEndIndex - 1) !== _SPACE) {
                                    break;
                                }
                                indentationEndIndex--;
                            }
                        }
                    }
                }
                if (this._opts.isUnshift && indentationEndIndex === 0) {
                    // line with no leading whitespace => nothing to do
                    continue;
                }
                var desiredIndentCount = void 0;
                if (this._opts.isUnshift) {
                    desiredIndentCount = ShiftCommand.unshiftIndentCount(lineText, indentationEndIndex + 1, tabSize);
                }
                else {
                    desiredIndentCount = ShiftCommand.shiftIndentCount(lineText, indentationEndIndex + 1, tabSize);
                }
                // Fill `indents`, as needed
                for (var j = indents.length; j <= desiredIndentCount; j++) {
                    indents[j] = indents[j - 1] + oneIndent;
                }
                builder.addEditOperation(new range_1.Range(lineNumber, 1, lineNumber, indentationEndIndex + 1), indents[desiredIndentCount]);
            }
            this._selectionId = builder.trackSelection(this._selection);
        };
        ShiftCommand.prototype.computeCursorState = function (model, helper) {
            if (this._useLastEditRangeForCursorEndPosition) {
                var lastOp = helper.getInverseEditOperations()[0];
                return new selection_1.Selection(lastOp.range.endLineNumber, lastOp.range.endColumn, lastOp.range.endLineNumber, lastOp.range.endColumn);
            }
            return helper.getTrackedSelection(this._selectionId);
        };
        return ShiftCommand;
    }());
    exports.ShiftCommand = ShiftCommand;
});

define("vs/editor/common/controller/oneCursor", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/strings', 'vs/editor/common/commands/replaceCommand', 'vs/editor/common/commands/shiftCommand', 'vs/editor/common/commands/surroundSelectionCommand', 'vs/editor/common/controller/cursorMoveHelper', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/core/selection', 'vs/editor/common/editorCommon', 'vs/editor/common/modes', 'vs/editor/common/modes/supports/onEnter'], function (require, exports, errors_1, strings, replaceCommand_1, shiftCommand_1, surroundSelectionCommand_1, cursorMoveHelper_1, position_1, range_1, selection_1, editorCommon, modes_1, onEnter_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (WordType) {
        WordType[WordType["None"] = 0] = "None";
        WordType[WordType["Regular"] = 1] = "Regular";
        WordType[WordType["Separator"] = 2] = "Separator";
    })(exports.WordType || (exports.WordType = {}));
    var WordType = exports.WordType;
    ;
    var CharacterClass;
    (function (CharacterClass) {
        CharacterClass[CharacterClass["Regular"] = 0] = "Regular";
        CharacterClass[CharacterClass["Whitespace"] = 1] = "Whitespace";
        CharacterClass[CharacterClass["WordSeparator"] = 2] = "WordSeparator";
    })(CharacterClass || (CharacterClass = {}));
    ;
    (function (WordNavigationType) {
        WordNavigationType[WordNavigationType["WordStart"] = 0] = "WordStart";
        WordNavigationType[WordNavigationType["WordEnd"] = 1] = "WordEnd";
    })(exports.WordNavigationType || (exports.WordNavigationType = {}));
    var WordNavigationType = exports.WordNavigationType;
    var CH_REGULAR = CharacterClass.Regular;
    var CH_WHITESPACE = CharacterClass.Whitespace;
    var CH_WORD_SEPARATOR = CharacterClass.WordSeparator;
    var W_NONE = WordType.None;
    var W_REGULAR = WordType.Regular;
    var W_SEPARATOR = WordType.Separator;
    var OneCursor = (function () {
        function OneCursor(editorId, model, configuration, modeConfiguration, viewModelHelper) {
            this.editorId = editorId;
            this.model = model;
            this.configuration = configuration;
            this.modeConfiguration = modeConfiguration;
            this.viewModelHelper = viewModelHelper;
            this.helper = new CursorHelper(this.model, this.configuration);
            this.bracketDecorations = [];
            this._set(new range_1.Range(1, 1, 1, 1), 0, new position_1.Position(1, 1), 0, new range_1.Range(1, 1, 1, 1), new position_1.Position(1, 1));
        }
        OneCursor.prototype._set = function (selectionStart, selectionStartLeftoverVisibleColumns, position, leftoverVisibleColumns, viewSelectionStart, viewPosition) {
            this.selectionStart = selectionStart;
            this.selectionStartLeftoverVisibleColumns = selectionStartLeftoverVisibleColumns;
            this.position = position;
            this.leftoverVisibleColumns = leftoverVisibleColumns;
            this.viewSelectionStart = viewSelectionStart;
            this.viewPosition = viewPosition;
            this._cachedSelection = OneCursor.computeSelection(this.selectionStart, this.position);
            this._cachedViewSelection = OneCursor.computeSelection(this.viewSelectionStart, this.viewPosition);
            this._selStartMarker = this._ensureMarker(this._selStartMarker, this._cachedSelection.startLineNumber, this._cachedSelection.startColumn, true);
            this._selEndMarker = this._ensureMarker(this._selEndMarker, this._cachedSelection.endLineNumber, this._cachedSelection.endColumn, false);
            this._selDirection = this._cachedSelection.getDirection();
        };
        OneCursor.prototype._ensureMarker = function (markerId, lineNumber, column, stickToPreviousCharacter) {
            if (!markerId) {
                return this.model._addMarker(lineNumber, column, stickToPreviousCharacter);
            }
            else {
                this.model._changeMarker(markerId, lineNumber, column);
                this.model._changeMarkerStickiness(markerId, stickToPreviousCharacter);
                return markerId;
            }
        };
        OneCursor.prototype.saveState = function () {
            return {
                selectionStart: this.selectionStart,
                viewSelectionStart: this.viewSelectionStart,
                position: this.position,
                viewPosition: this.viewPosition,
                leftoverVisibleColumns: this.leftoverVisibleColumns,
                selectionStartLeftoverVisibleColumns: this.selectionStartLeftoverVisibleColumns
            };
        };
        OneCursor.prototype.restoreState = function (state) {
            var position = this.model.validatePosition(state.position);
            var selectionStart;
            if (state.selectionStart) {
                selectionStart = this.model.validateRange(state.selectionStart);
            }
            else {
                selectionStart = new range_1.Range(position.lineNumber, position.column, position.lineNumber, position.column);
            }
            var viewPosition = this.viewModelHelper.validateViewPosition(state.viewPosition.lineNumber, state.viewPosition.column, position);
            var viewSelectionStart;
            if (state.viewSelectionStart) {
                viewSelectionStart = this.viewModelHelper.validateViewRange(state.viewSelectionStart.startLineNumber, state.viewSelectionStart.startColumn, state.viewSelectionStart.endLineNumber, state.viewSelectionStart.endColumn, selectionStart);
            }
            else {
                viewSelectionStart = this.viewModelHelper.convertModelRangeToViewRange(selectionStart);
            }
            this._set(selectionStart, state.selectionStartLeftoverVisibleColumns, position, state.leftoverVisibleColumns, viewSelectionStart, viewPosition);
        };
        OneCursor.prototype.updateModeConfiguration = function (modeConfiguration) {
            this.modeConfiguration = modeConfiguration;
        };
        OneCursor.prototype.duplicate = function () {
            var result = new OneCursor(this.editorId, this.model, this.configuration, this.modeConfiguration, this.viewModelHelper);
            result._set(this.selectionStart, this.selectionStartLeftoverVisibleColumns, this.position, this.leftoverVisibleColumns, this.viewSelectionStart, this.viewPosition);
            return result;
        };
        OneCursor.prototype.dispose = function () {
            this.model._removeMarker(this._selStartMarker);
            this.model._removeMarker(this._selEndMarker);
            this.bracketDecorations = this.model.deltaDecorations(this.bracketDecorations, [], this.editorId);
        };
        OneCursor.prototype.adjustBracketDecorations = function () {
            var bracketMatch = null;
            var selection = this.getSelection();
            if (selection.isEmpty()) {
                bracketMatch = this.model.matchBracket(this.position, /*inaccurateResultAcceptable*/ true);
            }
            var newDecorations = [];
            if (bracketMatch && bracketMatch.brackets) {
                var options = {
                    stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
                    className: 'bracket-match'
                };
                newDecorations.push({ range: bracketMatch.brackets[0], options: options });
                newDecorations.push({ range: bracketMatch.brackets[1], options: options });
            }
            this.bracketDecorations = this.model.deltaDecorations(this.bracketDecorations, newDecorations, this.editorId);
        };
        OneCursor.computeSelection = function (selectionStart, position) {
            var startLineNumber, startColumn, endLineNumber, endColumn;
            if (selectionStart.isEmpty()) {
                startLineNumber = selectionStart.startLineNumber;
                startColumn = selectionStart.startColumn;
                endLineNumber = position.lineNumber;
                endColumn = position.column;
            }
            else {
                if (position.isBeforeOrEqual(selectionStart.getStartPosition())) {
                    startLineNumber = selectionStart.endLineNumber;
                    startColumn = selectionStart.endColumn;
                    endLineNumber = position.lineNumber;
                    endColumn = position.column;
                }
                else {
                    startLineNumber = selectionStart.startLineNumber;
                    startColumn = selectionStart.startColumn;
                    endLineNumber = position.lineNumber;
                    endColumn = position.column;
                }
            }
            return new selection_1.Selection(startLineNumber, startColumn, endLineNumber, endColumn);
        };
        OneCursor.prototype.setSelection = function (desiredSelection) {
            var position = this.model.validatePosition({
                lineNumber: desiredSelection.positionLineNumber,
                column: desiredSelection.positionColumn
            });
            var selectionStartPosition = this.model.validatePosition({
                lineNumber: desiredSelection.selectionStartLineNumber,
                column: desiredSelection.selectionStartColumn
            });
            var selectionStart = new range_1.Range(selectionStartPosition.lineNumber, selectionStartPosition.column, selectionStartPosition.lineNumber, selectionStartPosition.column);
            var viewPosition = this.viewModelHelper.convertModelPositionToViewPosition(position.lineNumber, position.column);
            var viewSelectionStart = this.viewModelHelper.convertModelRangeToViewRange(selectionStart);
            this._set(selectionStart, 0, position, 0, viewSelectionStart, viewPosition);
        };
        OneCursor.prototype.setViewSelection = function (desiredViewSel) {
            var viewSelectionStart = this.viewModelHelper.validateViewRange(desiredViewSel.selectionStartLineNumber, desiredViewSel.selectionStartColumn, desiredViewSel.selectionStartLineNumber, desiredViewSel.selectionStartColumn, this.selectionStart);
            var viewPosition = this.viewModelHelper.validateViewPosition(desiredViewSel.positionLineNumber, desiredViewSel.positionColumn, this.position);
            this._set(this.selectionStart, 0, this.position, 0, viewSelectionStart, viewPosition);
        };
        // -------------------- START modifications
        OneCursor.prototype.setSelectionStart = function (rng, viewRng) {
            this._set(rng, this.selectionStartLeftoverVisibleColumns, this.position, this.leftoverVisibleColumns, viewRng, this.viewPosition);
        };
        OneCursor.prototype.collapseSelection = function () {
            var selectionStart = new range_1.Range(this.position.lineNumber, this.position.column, this.position.lineNumber, this.position.column);
            var viewSelectionStart = new range_1.Range(this.viewPosition.lineNumber, this.viewPosition.column, this.viewPosition.lineNumber, this.viewPosition.column);
            this._set(selectionStart, 0, this.position, this.leftoverVisibleColumns, viewSelectionStart, this.viewPosition);
        };
        OneCursor.prototype.moveModelPosition = function (inSelectionMode, lineNumber, column, leftoverVisibleColumns, ensureInEditableRange) {
            var viewPosition = this.viewModelHelper.convertModelPositionToViewPosition(lineNumber, column);
            this._move(inSelectionMode, lineNumber, column, viewPosition.lineNumber, viewPosition.column, leftoverVisibleColumns, ensureInEditableRange);
        };
        OneCursor.prototype.moveViewPosition = function (inSelectionMode, viewLineNumber, viewColumn, leftoverVisibleColumns, ensureInEditableRange) {
            var modelPosition = this.viewModelHelper.convertViewToModelPosition(viewLineNumber, viewColumn);
            this._move(inSelectionMode, modelPosition.lineNumber, modelPosition.column, viewLineNumber, viewColumn, leftoverVisibleColumns, ensureInEditableRange);
        };
        OneCursor.prototype._move = function (inSelectionMode, lineNumber, column, viewLineNumber, viewColumn, leftoverVisibleColumns, ensureInEditableRange) {
            if (ensureInEditableRange) {
                var editableRange = this.model.getEditableRange();
                if (lineNumber < editableRange.startLineNumber || (lineNumber === editableRange.startLineNumber && column < editableRange.startColumn)) {
                    lineNumber = editableRange.startLineNumber;
                    column = editableRange.startColumn;
                    var viewPosition = this.viewModelHelper.convertModelPositionToViewPosition(lineNumber, column);
                    viewLineNumber = viewPosition.lineNumber;
                    viewColumn = viewPosition.column;
                }
                else if (lineNumber > editableRange.endLineNumber || (lineNumber === editableRange.endLineNumber && column > editableRange.endColumn)) {
                    lineNumber = editableRange.endLineNumber;
                    column = editableRange.endColumn;
                    var viewPosition = this.viewModelHelper.convertModelPositionToViewPosition(lineNumber, column);
                    viewLineNumber = viewPosition.lineNumber;
                    viewColumn = viewPosition.column;
                }
            }
            this._actualMove(inSelectionMode, new position_1.Position(lineNumber, column), new position_1.Position(viewLineNumber, viewColumn), leftoverVisibleColumns);
        };
        OneCursor.prototype._actualMove = function (inSelectionMode, position, viewPosition, leftoverVisibleColumns) {
            if (inSelectionMode) {
                // move just position
                this._set(this.selectionStart, this.selectionStartLeftoverVisibleColumns, position, leftoverVisibleColumns, this.viewSelectionStart, viewPosition);
            }
            else {
                // move everything
                var selectionStart = new range_1.Range(position.lineNumber, position.column, position.lineNumber, position.column);
                var viewSelectionStart = new range_1.Range(viewPosition.lineNumber, viewPosition.column, viewPosition.lineNumber, viewPosition.column);
                this._set(selectionStart, leftoverVisibleColumns, position, leftoverVisibleColumns, viewSelectionStart, viewPosition);
            }
        };
        OneCursor.prototype._recoverSelectionFromMarkers = function () {
            var start = this.model._getMarker(this._selStartMarker);
            var end = this.model._getMarker(this._selEndMarker);
            if (this._selDirection === editorCommon.SelectionDirection.LTR) {
                return new selection_1.Selection(start.lineNumber, start.column, end.lineNumber, end.column);
            }
            return new selection_1.Selection(end.lineNumber, end.column, start.lineNumber, start.column);
        };
        OneCursor.prototype.recoverSelectionFromMarkers = function (ctx) {
            ctx.cursorPositionChangeReason = 'recoverFromMarkers';
            ctx.shouldPushStackElementBefore = true;
            ctx.shouldPushStackElementAfter = true;
            ctx.shouldReveal = false;
            ctx.shouldRevealHorizontal = false;
            var recoveredSelection = this._recoverSelectionFromMarkers();
            var selectionStart = new range_1.Range(recoveredSelection.selectionStartLineNumber, recoveredSelection.selectionStartColumn, recoveredSelection.selectionStartLineNumber, recoveredSelection.selectionStartColumn);
            var position = new position_1.Position(recoveredSelection.positionLineNumber, recoveredSelection.positionColumn);
            var viewSelectionStart = this.viewModelHelper.convertModelRangeToViewRange(selectionStart);
            var viewPosition = this.viewModelHelper.convertViewToModelPosition(position.lineNumber, position.column);
            this._set(selectionStart, 0, position, 0, viewSelectionStart, viewPosition);
            return true;
        };
        // -------------------- END modifications
        // -------------------- START reading API
        OneCursor.prototype.getSelectionStart = function () {
            return this.selectionStart;
        };
        OneCursor.prototype.getPosition = function () {
            return this.position;
        };
        OneCursor.prototype.getSelection = function () {
            return this._cachedSelection;
        };
        OneCursor.prototype.getViewPosition = function () {
            return this.viewPosition;
        };
        OneCursor.prototype.getViewSelection = function () {
            return this._cachedViewSelection;
        };
        OneCursor.prototype.getValidViewPosition = function () {
            return this.viewModelHelper.validateViewPosition(this.viewPosition.lineNumber, this.viewPosition.column, this.position);
        };
        OneCursor.prototype.hasSelection = function () {
            return (!this.getSelection().isEmpty() || !this.selectionStart.isEmpty());
        };
        OneCursor.prototype.getBracketsDecorations = function () {
            return this.bracketDecorations;
        };
        OneCursor.prototype.getLeftoverVisibleColumns = function () {
            return this.leftoverVisibleColumns;
        };
        OneCursor.prototype.getSelectionStartLeftoverVisibleColumns = function () {
            return this.selectionStartLeftoverVisibleColumns;
        };
        OneCursor.prototype.setSelectionStartLeftoverVisibleColumns = function (value) {
            this.selectionStartLeftoverVisibleColumns = value;
        };
        // -- utils
        OneCursor.prototype.validatePosition = function (position) {
            return this.model.validatePosition(position);
        };
        OneCursor.prototype.validateViewPosition = function (viewLineNumber, viewColumn, modelPosition) {
            return this.viewModelHelper.validateViewPosition(viewLineNumber, viewColumn, modelPosition);
        };
        OneCursor.prototype.convertViewToModelPosition = function (lineNumber, column) {
            return this.viewModelHelper.convertViewToModelPosition(lineNumber, column);
        };
        OneCursor.prototype.convertViewSelectionToModelSelection = function (viewSelection) {
            return this.viewModelHelper.convertViewSelectionToModelSelection(viewSelection);
        };
        OneCursor.prototype.convertModelPositionToViewPosition = function (lineNumber, column) {
            return this.viewModelHelper.convertModelPositionToViewPosition(lineNumber, column);
        };
        // -- model
        OneCursor.prototype.getLineContent = function (lineNumber) {
            return this.model.getLineContent(lineNumber);
        };
        // public findWord(position:editorCommon.IEditorPosition, preference:string, skipSyntaxTokens?:boolean): editorCommon.IWordRange {
        // 	return this.helper.findWord(position, preference, skipSyntaxTokens);
        // }
        OneCursor.prototype.findPreviousWordOnLine = function (position) {
            return this.helper.findPreviousWordOnLine(position);
        };
        OneCursor.prototype.findNextWordOnLine = function (position) {
            return this.helper.findNextWordOnLine(position);
        };
        OneCursor.prototype.getLeftOfPosition = function (lineNumber, column) {
            return this.helper.getLeftOfPosition(this.model, lineNumber, column);
        };
        OneCursor.prototype.getRightOfPosition = function (lineNumber, column) {
            return this.helper.getRightOfPosition(this.model, lineNumber, column);
        };
        OneCursor.prototype.getPositionUp = function (lineNumber, column, leftoverVisibleColumns, count, allowMoveOnFirstLine) {
            return this.helper.getPositionUp(this.model, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnFirstLine);
        };
        OneCursor.prototype.getPositionDown = function (lineNumber, column, leftoverVisibleColumns, count, allowMoveOnLastLine) {
            return this.helper.getPositionDown(this.model, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnLastLine);
        };
        OneCursor.prototype.getColumnAtBeginningOfLine = function (lineNumber, column) {
            return this.helper.getColumnAtBeginningOfLine(this.model, lineNumber, column);
        };
        OneCursor.prototype.getColumnAtEndOfLine = function (lineNumber, column) {
            return this.helper.getColumnAtEndOfLine(this.model, lineNumber, column);
        };
        OneCursor.prototype.getVisibleColumnFromColumn = function (lineNumber, column) {
            return this.helper.visibleColumnFromColumn(this.model, lineNumber, column);
        };
        OneCursor.prototype.getViewVisibleColumnFromColumn = function (viewLineNumber, viewColumn) {
            return this.helper.visibleColumnFromColumn(this.viewModelHelper.viewModel, viewLineNumber, viewColumn);
        };
        // -- view
        OneCursor.prototype.getViewLineCount = function () {
            return this.viewModelHelper.viewModel.getLineCount();
        };
        OneCursor.prototype.getViewLineMaxColumn = function (lineNumber) {
            return this.viewModelHelper.viewModel.getLineMaxColumn(lineNumber);
        };
        OneCursor.prototype.getLeftOfViewPosition = function (lineNumber, column) {
            return this.helper.getLeftOfPosition(this.viewModelHelper.viewModel, lineNumber, column);
        };
        OneCursor.prototype.getRightOfViewPosition = function (lineNumber, column) {
            return this.helper.getRightOfPosition(this.viewModelHelper.viewModel, lineNumber, column);
        };
        OneCursor.prototype.getViewPositionUp = function (lineNumber, column, leftoverVisibleColumns, count, allowMoveOnFirstLine) {
            return this.helper.getPositionUp(this.viewModelHelper.viewModel, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnFirstLine);
        };
        OneCursor.prototype.getViewPositionDown = function (lineNumber, column, leftoverVisibleColumns, count, allowMoveOnLastLine) {
            return this.helper.getPositionDown(this.viewModelHelper.viewModel, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnLastLine);
        };
        OneCursor.prototype.getColumnAtBeginningOfViewLine = function (lineNumber, column) {
            return this.helper.getColumnAtBeginningOfLine(this.viewModelHelper.viewModel, lineNumber, column);
        };
        OneCursor.prototype.getColumnAtEndOfViewLine = function (lineNumber, column) {
            return this.helper.getColumnAtEndOfLine(this.viewModelHelper.viewModel, lineNumber, column);
        };
        OneCursor.prototype.columnSelect = function (fromViewLineNumber, fromViewVisibleColumn, toViewLineNumber, toViewVisibleColumn) {
            var _this = this;
            var r = this.helper.columnSelect(this.viewModelHelper.viewModel, fromViewLineNumber, fromViewVisibleColumn, toViewLineNumber, toViewVisibleColumn);
            return {
                reversed: r.reversed,
                viewSelections: r.viewSelections,
                selections: r.viewSelections.map(function (sel) { return _this.convertViewSelectionToModelSelection(sel); }),
                toLineNumber: toViewLineNumber,
                toVisualColumn: toViewVisibleColumn
            };
        };
        return OneCursor;
    }());
    exports.OneCursor = OneCursor;
    var OneCursorOp = (function () {
        function OneCursorOp() {
        }
        // -------------------- START handlers that simply change cursor state
        OneCursorOp.jumpToBracket = function (cursor, ctx) {
            var bracketDecorations = cursor.getBracketsDecorations();
            if (bracketDecorations.length !== 2) {
                return false;
            }
            var firstBracket = cursor.model.getDecorationRange(bracketDecorations[0]);
            var secondBracket = cursor.model.getDecorationRange(bracketDecorations[1]);
            var position = cursor.getPosition();
            if (Utils.isPositionAtRangeEdges(position, firstBracket) || Utils.isPositionInsideRange(position, firstBracket)) {
                cursor.moveModelPosition(false, secondBracket.endLineNumber, secondBracket.endColumn, 0, false);
                return true;
            }
            if (Utils.isPositionAtRangeEdges(position, secondBracket) || Utils.isPositionInsideRange(position, secondBracket)) {
                cursor.moveModelPosition(false, firstBracket.endLineNumber, firstBracket.endColumn, 0, false);
                return true;
            }
            return false;
        };
        OneCursorOp.moveTo = function (cursor, inSelectionMode, position, viewPosition, eventSource, ctx) {
            var validatedPosition = cursor.model.validatePosition(position);
            var validatedViewPosition;
            if (viewPosition) {
                validatedViewPosition = cursor.validateViewPosition(viewPosition.lineNumber, viewPosition.column, validatedPosition);
            }
            else {
                validatedViewPosition = cursor.convertModelPositionToViewPosition(validatedPosition.lineNumber, validatedPosition.column);
            }
            var reason = (eventSource === 'mouse' ? 'explicit' : null);
            if (eventSource === 'api') {
                ctx.shouldRevealVerticalInCenter = true;
            }
            if (reason) {
                ctx.cursorPositionChangeReason = reason;
            }
            cursor.moveViewPosition(inSelectionMode, validatedViewPosition.lineNumber, validatedViewPosition.column, 0, false);
            return true;
        };
        OneCursorOp._columnSelectOp = function (cursor, toViewLineNumber, toViewVisualColumn) {
            var viewStartSelection = cursor.getViewSelection();
            var fromVisibleColumn = cursor.getVisibleColumnFromColumn(viewStartSelection.selectionStartLineNumber, viewStartSelection.selectionStartColumn);
            return cursor.columnSelect(viewStartSelection.selectionStartLineNumber, fromVisibleColumn, toViewLineNumber, toViewVisualColumn);
        };
        OneCursorOp.columnSelectMouse = function (cursor, position, viewPosition, toViewVisualColumn) {
            var validatedPosition = cursor.model.validatePosition(position);
            var validatedViewPosition;
            if (viewPosition) {
                validatedViewPosition = cursor.validateViewPosition(viewPosition.lineNumber, viewPosition.column, validatedPosition);
            }
            else {
                validatedViewPosition = cursor.convertModelPositionToViewPosition(validatedPosition.lineNumber, validatedPosition.column);
            }
            return this._columnSelectOp(cursor, validatedViewPosition.lineNumber, toViewVisualColumn);
        };
        OneCursorOp.columnSelectLeft = function (cursor, toViewLineNumber, toViewVisualColumn) {
            if (toViewVisualColumn > 1) {
                toViewVisualColumn--;
            }
            return this._columnSelectOp(cursor, toViewLineNumber, toViewVisualColumn);
        };
        OneCursorOp.columnSelectRight = function (cursor, toViewLineNumber, toViewVisualColumn) {
            var maxVisualViewColumn = 0;
            var minViewLineNumber = Math.min(cursor.getViewPosition().lineNumber, toViewLineNumber);
            var maxViewLineNumber = Math.max(cursor.getViewPosition().lineNumber, toViewLineNumber);
            for (var lineNumber = minViewLineNumber; lineNumber <= maxViewLineNumber; lineNumber++) {
                var lineMaxViewColumn = cursor.getViewLineMaxColumn(lineNumber);
                var lineMaxVisualViewColumn = cursor.getViewVisibleColumnFromColumn(lineNumber, lineMaxViewColumn);
                maxVisualViewColumn = Math.max(maxVisualViewColumn, lineMaxVisualViewColumn);
            }
            if (toViewVisualColumn < maxVisualViewColumn) {
                toViewVisualColumn++;
            }
            return this._columnSelectOp(cursor, toViewLineNumber, toViewVisualColumn);
        };
        OneCursorOp.columnSelectUp = function (isPaged, cursor, toViewLineNumber, toViewVisualColumn) {
            var linesCount = isPaged ? cursor.configuration.editor.pageSize : 1;
            toViewLineNumber -= linesCount;
            if (toViewLineNumber < 1) {
                toViewLineNumber = 1;
            }
            return this._columnSelectOp(cursor, toViewLineNumber, toViewVisualColumn);
        };
        OneCursorOp.columnSelectDown = function (isPaged, cursor, toViewLineNumber, toViewVisualColumn) {
            var linesCount = isPaged ? cursor.configuration.editor.pageSize : 1;
            toViewLineNumber += linesCount;
            if (toViewLineNumber > cursor.getViewLineCount()) {
                toViewLineNumber = cursor.getViewLineCount();
            }
            return this._columnSelectOp(cursor, toViewLineNumber, toViewVisualColumn);
        };
        OneCursorOp.moveLeft = function (cursor, inSelectionMode, ctx) {
            var viewLineNumber, viewColumn;
            if (cursor.hasSelection() && !inSelectionMode) {
                // If we are in selection mode, move left without selection cancels selection and puts cursor at the beginning of the selection
                var viewSelection = cursor.getViewSelection();
                var viewSelectionStart = cursor.validateViewPosition(viewSelection.startLineNumber, viewSelection.startColumn, cursor.getSelection().getStartPosition());
                viewLineNumber = viewSelectionStart.lineNumber;
                viewColumn = viewSelectionStart.column;
            }
            else {
                var validatedViewPosition = cursor.getValidViewPosition();
                var r = cursor.getLeftOfViewPosition(validatedViewPosition.lineNumber, validatedViewPosition.column);
                viewLineNumber = r.lineNumber;
                viewColumn = r.column;
            }
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveViewPosition(inSelectionMode, viewLineNumber, viewColumn, 0, true);
            return true;
        };
        OneCursorOp.moveWordLeft = function (cursor, inSelectionMode, wordNavigationType, ctx) {
            var position = cursor.getPosition();
            var lineNumber = position.lineNumber;
            var column = position.column;
            if (column === 1) {
                if (lineNumber > 1) {
                    lineNumber = lineNumber - 1;
                    column = cursor.model.getLineMaxColumn(lineNumber);
                }
            }
            var prevWordOnLine = cursor.findPreviousWordOnLine(new position_1.Position(lineNumber, column));
            if (wordNavigationType === WordNavigationType.WordStart) {
                if (prevWordOnLine) {
                    column = prevWordOnLine.start + 1;
                }
                else {
                    column = 1;
                }
            }
            else {
                if (prevWordOnLine && column <= prevWordOnLine.end + 1) {
                    prevWordOnLine = cursor.findPreviousWordOnLine(new position_1.Position(lineNumber, prevWordOnLine.start + 1));
                }
                if (prevWordOnLine) {
                    column = prevWordOnLine.end + 1;
                }
                else {
                    column = 1;
                }
            }
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveModelPosition(inSelectionMode, lineNumber, column, 0, true);
            return true;
        };
        OneCursorOp.moveRight = function (cursor, inSelectionMode, ctx) {
            var viewLineNumber, viewColumn;
            if (cursor.hasSelection() && !inSelectionMode) {
                // If we are in selection mode, move right without selection cancels selection and puts cursor at the end of the selection
                var viewSelection = cursor.getViewSelection();
                var viewSelectionEnd = cursor.validateViewPosition(viewSelection.endLineNumber, viewSelection.endColumn, cursor.getSelection().getEndPosition());
                viewLineNumber = viewSelectionEnd.lineNumber;
                viewColumn = viewSelectionEnd.column;
            }
            else {
                var validatedViewPosition = cursor.getValidViewPosition();
                var r = cursor.getRightOfViewPosition(validatedViewPosition.lineNumber, validatedViewPosition.column);
                viewLineNumber = r.lineNumber;
                viewColumn = r.column;
            }
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveViewPosition(inSelectionMode, viewLineNumber, viewColumn, 0, true);
            return true;
        };
        OneCursorOp.moveWordRight = function (cursor, inSelectionMode, wordNavigationType, ctx) {
            var position = cursor.getPosition();
            var lineNumber = position.lineNumber;
            var column = position.column;
            if (column === cursor.model.getLineMaxColumn(lineNumber)) {
                if (lineNumber < cursor.model.getLineCount()) {
                    lineNumber = lineNumber + 1;
                    column = 1;
                }
            }
            var nextWordOnLine = cursor.findNextWordOnLine(new position_1.Position(lineNumber, column));
            if (wordNavigationType === WordNavigationType.WordEnd) {
                if (nextWordOnLine) {
                    column = nextWordOnLine.end + 1;
                }
                else {
                    column = cursor.model.getLineMaxColumn(lineNumber);
                }
            }
            else {
                if (nextWordOnLine && column >= nextWordOnLine.start + 1) {
                    nextWordOnLine = cursor.findNextWordOnLine(new position_1.Position(lineNumber, nextWordOnLine.end + 1));
                }
                if (nextWordOnLine) {
                    column = nextWordOnLine.start + 1;
                }
                else {
                    column = cursor.model.getLineMaxColumn(lineNumber);
                }
            }
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveModelPosition(inSelectionMode, lineNumber, column, 0, true);
            return true;
        };
        OneCursorOp.moveDown = function (cursor, inSelectionMode, isPaged, ctx) {
            var linesCount = isPaged ? cursor.configuration.editor.pageSize : 1;
            var viewLineNumber, viewColumn;
            if (cursor.hasSelection() && !inSelectionMode) {
                // If we are in selection mode, move down acts relative to the end of selection
                var viewSelection = cursor.getViewSelection();
                var viewSelectionEnd = cursor.validateViewPosition(viewSelection.endLineNumber, viewSelection.endColumn, cursor.getSelection().getEndPosition());
                viewLineNumber = viewSelectionEnd.lineNumber;
                viewColumn = viewSelectionEnd.column;
            }
            else {
                var validatedViewPosition = cursor.getValidViewPosition();
                viewLineNumber = validatedViewPosition.lineNumber;
                viewColumn = validatedViewPosition.column;
            }
            var r = cursor.getViewPositionDown(viewLineNumber, viewColumn, cursor.getLeftoverVisibleColumns(), linesCount, true);
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveViewPosition(inSelectionMode, r.lineNumber, r.column, r.leftoverVisibleColumns, true);
            return true;
        };
        OneCursorOp.translateDown = function (cursor, ctx) {
            var selection = cursor.getViewSelection();
            var selectionStart = cursor.getViewPositionDown(selection.selectionStartLineNumber, selection.selectionStartColumn, cursor.getSelectionStartLeftoverVisibleColumns(), 1, false);
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveViewPosition(false, selectionStart.lineNumber, selectionStart.column, cursor.getLeftoverVisibleColumns(), true);
            var position = cursor.getViewPositionDown(selection.positionLineNumber, selection.positionColumn, cursor.getLeftoverVisibleColumns(), 1, false);
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveViewPosition(true, position.lineNumber, position.column, position.leftoverVisibleColumns, true);
            cursor.setSelectionStartLeftoverVisibleColumns(selectionStart.leftoverVisibleColumns);
            return true;
        };
        OneCursorOp.moveUp = function (cursor, inSelectionMode, isPaged, ctx) {
            var linesCount = isPaged ? cursor.configuration.editor.pageSize : 1;
            var viewLineNumber, viewColumn;
            if (cursor.hasSelection() && !inSelectionMode) {
                // If we are in selection mode, move up acts relative to the beginning of selection
                var viewSelection = cursor.getViewSelection();
                var viewSelectionStart = cursor.validateViewPosition(viewSelection.startLineNumber, viewSelection.startColumn, cursor.getSelection().getStartPosition());
                viewLineNumber = viewSelectionStart.lineNumber;
                viewColumn = viewSelectionStart.column;
            }
            else {
                var validatedViewPosition = cursor.getValidViewPosition();
                viewLineNumber = validatedViewPosition.lineNumber;
                viewColumn = validatedViewPosition.column;
            }
            var r = cursor.getViewPositionUp(viewLineNumber, viewColumn, cursor.getLeftoverVisibleColumns(), linesCount, true);
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveViewPosition(inSelectionMode, r.lineNumber, r.column, r.leftoverVisibleColumns, true);
            return true;
        };
        OneCursorOp.translateUp = function (cursor, ctx) {
            var selection = cursor.getViewSelection();
            var selectionStart = cursor.getViewPositionUp(selection.selectionStartLineNumber, selection.selectionStartColumn, cursor.getSelectionStartLeftoverVisibleColumns(), 1, false);
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveViewPosition(false, selectionStart.lineNumber, selectionStart.column, cursor.getLeftoverVisibleColumns(), true);
            var position = cursor.getViewPositionUp(selection.positionLineNumber, selection.positionColumn, cursor.getLeftoverVisibleColumns(), 1, false);
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveViewPosition(true, position.lineNumber, position.column, position.leftoverVisibleColumns, true);
            cursor.setSelectionStartLeftoverVisibleColumns(selectionStart.leftoverVisibleColumns);
            return true;
        };
        OneCursorOp.moveToBeginningOfLine = function (cursor, inSelectionMode, ctx) {
            var validatedViewPosition = cursor.getValidViewPosition();
            var viewLineNumber = validatedViewPosition.lineNumber;
            var viewColumn = validatedViewPosition.column;
            viewColumn = cursor.getColumnAtBeginningOfViewLine(viewLineNumber, viewColumn);
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveViewPosition(inSelectionMode, viewLineNumber, viewColumn, 0, true);
            return true;
        };
        OneCursorOp.moveToEndOfLine = function (cursor, inSelectionMode, ctx) {
            var validatedViewPosition = cursor.getValidViewPosition();
            var viewLineNumber = validatedViewPosition.lineNumber;
            var viewColumn = validatedViewPosition.column;
            viewColumn = cursor.getColumnAtEndOfViewLine(viewLineNumber, viewColumn);
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveViewPosition(inSelectionMode, viewLineNumber, viewColumn, 0, true);
            return true;
        };
        OneCursorOp.expandLineSelection = function (cursor, ctx) {
            ctx.cursorPositionChangeReason = 'explicit';
            var viewSel = cursor.getViewSelection();
            var viewStartLineNumber = viewSel.startLineNumber;
            var viewStartColumn = viewSel.startColumn;
            var viewEndLineNumber = viewSel.endLineNumber;
            var viewEndColumn = viewSel.endColumn;
            var viewEndMaxColumn = cursor.getViewLineMaxColumn(viewEndLineNumber);
            if (viewStartColumn !== 1 || viewEndColumn !== viewEndMaxColumn) {
                viewStartColumn = 1;
                viewEndColumn = viewEndMaxColumn;
            }
            else {
                // Expand selection with one more line down
                var moveResult = cursor.getViewPositionDown(viewEndLineNumber, viewEndColumn, 0, 1, true);
                viewEndLineNumber = moveResult.lineNumber;
                viewEndColumn = cursor.getViewLineMaxColumn(viewEndLineNumber);
            }
            cursor.moveViewPosition(false, viewStartLineNumber, viewStartColumn, 0, true);
            cursor.moveViewPosition(true, viewEndLineNumber, viewEndColumn, 0, true);
            return true;
        };
        OneCursorOp.moveToBeginningOfBuffer = function (cursor, inSelectionMode, ctx) {
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveModelPosition(inSelectionMode, 1, 1, 0, true);
            return true;
        };
        OneCursorOp.moveToEndOfBuffer = function (cursor, inSelectionMode, ctx) {
            var lastLineNumber = cursor.model.getLineCount();
            var lastColumn = cursor.model.getLineMaxColumn(lastLineNumber);
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveModelPosition(inSelectionMode, lastLineNumber, lastColumn, 0, true);
            return true;
        };
        OneCursorOp.selectAll = function (cursor, ctx) {
            var selectEntireBuffer = true;
            var newSelectionStartLineNumber, newSelectionStartColumn, newPositionLineNumber, newPositionColumn;
            if (cursor.model.hasEditableRange()) {
                // Toggle between selecting editable range and selecting the entire buffer
                var editableRange = cursor.model.getEditableRange();
                var selection = cursor.getSelection();
                if (!selection.equalsRange(editableRange)) {
                    // Selection is not editable range => select editable range
                    selectEntireBuffer = false;
                    newSelectionStartLineNumber = editableRange.startLineNumber;
                    newSelectionStartColumn = editableRange.startColumn;
                    newPositionLineNumber = editableRange.endLineNumber;
                    newPositionColumn = editableRange.endColumn;
                }
            }
            if (selectEntireBuffer) {
                newSelectionStartLineNumber = 1;
                newSelectionStartColumn = 1;
                newPositionLineNumber = cursor.model.getLineCount();
                newPositionColumn = cursor.model.getLineMaxColumn(newPositionLineNumber);
            }
            cursor.moveModelPosition(false, newSelectionStartLineNumber, newSelectionStartColumn, 0, false);
            cursor.moveModelPosition(true, newPositionLineNumber, newPositionColumn, 0, false);
            ctx.shouldReveal = false;
            ctx.shouldRevealHorizontal = false;
            return true;
        };
        OneCursorOp.line = function (cursor, inSelectionMode, _position, _viewPosition, ctx) {
            // TODO@Alex -> select in editable range
            var position = cursor.validatePosition(_position);
            var viewPosition = (_viewPosition ?
                cursor.validateViewPosition(_viewPosition.lineNumber, _viewPosition.column, position)
                : cursor.convertModelPositionToViewPosition(position.lineNumber, position.column));
            ctx.cursorPositionChangeReason = 'explicit';
            ctx.shouldRevealHorizontal = false;
            if (!inSelectionMode || !cursor.hasSelection()) {
                // Entering line selection for the first time
                var selectToLineNumber = position.lineNumber + 1;
                var selectToColumn = 1;
                if (selectToLineNumber > cursor.model.getLineCount()) {
                    selectToLineNumber = cursor.model.getLineCount();
                    selectToColumn = cursor.model.getLineMaxColumn(selectToLineNumber);
                }
                var selectionStartRange = new range_1.Range(position.lineNumber, 1, selectToLineNumber, selectToColumn);
                var r1 = cursor.convertModelPositionToViewPosition(position.lineNumber, 1);
                var r2 = cursor.convertModelPositionToViewPosition(selectToLineNumber, selectToColumn);
                cursor.setSelectionStart(selectionStartRange, new range_1.Range(r1.lineNumber, r1.column, r2.lineNumber, r2.column));
                cursor.moveModelPosition(cursor.hasSelection(), selectionStartRange.endLineNumber, selectionStartRange.endColumn, 0, false);
                return true;
            }
            else {
                // Continuing line selection
                var enteringLineNumber = cursor.getSelectionStart().getStartPosition().lineNumber;
                if (position.lineNumber < enteringLineNumber) {
                    cursor.moveViewPosition(cursor.hasSelection(), viewPosition.lineNumber, 1, 0, false);
                }
                else if (position.lineNumber > enteringLineNumber) {
                    var selectToViewLineNumber = viewPosition.lineNumber + 1;
                    var selectToViewColumn = 1;
                    if (selectToViewLineNumber > cursor.getViewLineCount()) {
                        selectToViewLineNumber = cursor.getViewLineCount();
                        selectToViewColumn = cursor.getViewLineMaxColumn(selectToViewLineNumber);
                    }
                    cursor.moveViewPosition(cursor.hasSelection(), selectToViewLineNumber, selectToViewColumn, 0, false);
                }
                else {
                    var endPositionOfSelectionStart = cursor.getSelectionStart().getEndPosition();
                    cursor.moveModelPosition(cursor.hasSelection(), endPositionOfSelectionStart.lineNumber, endPositionOfSelectionStart.column, 0, false);
                }
                return true;
            }
        };
        OneCursorOp.word = function (cursor, inSelectionMode, position, ctx) {
            // TODO@Alex -> select in editable range
            var validatedPosition = cursor.validatePosition(position);
            var prevWord = cursor.findPreviousWordOnLine(validatedPosition);
            var isInPrevWord = (prevWord && prevWord.wordType === WordType.Regular && prevWord.start < validatedPosition.column - 1 && validatedPosition.column - 1 <= prevWord.end);
            var nextWord = cursor.findNextWordOnLine(validatedPosition);
            var isInNextWord = (nextWord && nextWord.wordType === WordType.Regular && nextWord.start < validatedPosition.column - 1 && validatedPosition.column - 1 <= nextWord.end);
            var lineNumber;
            var column;
            if (!inSelectionMode || !cursor.hasSelection()) {
                var startColumn = void 0;
                var endColumn = void 0;
                if (isInPrevWord) {
                    startColumn = prevWord.start + 1;
                    endColumn = prevWord.end + 1;
                }
                else if (isInNextWord) {
                    startColumn = nextWord.start + 1;
                    endColumn = nextWord.end + 1;
                }
                else {
                    if (prevWord) {
                        startColumn = prevWord.end + 1;
                    }
                    else {
                        startColumn = 1;
                    }
                    if (nextWord) {
                        endColumn = nextWord.start + 1;
                    }
                    else {
                        endColumn = cursor.model.getLineMaxColumn(validatedPosition.lineNumber);
                    }
                }
                var selectionStartRange = new range_1.Range(validatedPosition.lineNumber, startColumn, validatedPosition.lineNumber, endColumn);
                var r1 = cursor.convertModelPositionToViewPosition(validatedPosition.lineNumber, startColumn);
                var r2 = cursor.convertModelPositionToViewPosition(validatedPosition.lineNumber, endColumn);
                cursor.setSelectionStart(selectionStartRange, new range_1.Range(r1.lineNumber, r1.column, r2.lineNumber, r2.column));
                lineNumber = selectionStartRange.endLineNumber;
                column = selectionStartRange.endColumn;
            }
            else {
                var startColumn = void 0;
                var endColumn = void 0;
                if (isInPrevWord) {
                    startColumn = prevWord.start + 1;
                    endColumn = prevWord.end + 1;
                }
                else if (isInNextWord) {
                    startColumn = nextWord.start + 1;
                    endColumn = nextWord.end + 1;
                }
                else {
                    startColumn = validatedPosition.column;
                    endColumn = validatedPosition.column;
                }
                lineNumber = validatedPosition.lineNumber;
                if (validatedPosition.isBeforeOrEqual(cursor.getSelectionStart().getStartPosition())) {
                    column = startColumn;
                    var possiblePosition = new position_1.Position(lineNumber, column);
                    if (cursor.getSelectionStart().containsPosition(possiblePosition)) {
                        column = cursor.getSelectionStart().endColumn;
                    }
                }
                else {
                    column = endColumn;
                    var possiblePosition = new position_1.Position(lineNumber, column);
                    if (cursor.getSelectionStart().containsPosition(possiblePosition)) {
                        column = cursor.getSelectionStart().startColumn;
                    }
                }
            }
            ctx.cursorPositionChangeReason = 'explicit';
            cursor.moveModelPosition(cursor.hasSelection(), lineNumber, column, 0, false);
            return true;
        };
        OneCursorOp.cancelSelection = function (cursor, ctx) {
            if (!cursor.hasSelection()) {
                return false;
            }
            cursor.collapseSelection();
            return true;
        };
        // -------------------- STOP handlers that simply change cursor state
        // -------------------- START type interceptors & co.
        OneCursorOp._typeInterceptorEnter = function (cursor, ch, ctx) {
            if (ch !== '\n') {
                return false;
            }
            return this._enter(cursor, false, ctx);
        };
        OneCursorOp.lineInsertBefore = function (cursor, ctx) {
            var lineNumber = cursor.getPosition().lineNumber;
            if (lineNumber === 1) {
                ctx.executeCommand = new replaceCommand_1.ReplaceCommandWithoutChangingPosition(new range_1.Range(1, 1, 1, 1), '\n');
                return true;
            }
            lineNumber--;
            var column = cursor.model.getLineMaxColumn(lineNumber);
            return this._enter(cursor, false, ctx, new position_1.Position(lineNumber, column), new range_1.Range(lineNumber, column, lineNumber, column));
        };
        OneCursorOp.lineInsertAfter = function (cursor, ctx) {
            var position = cursor.getPosition();
            var column = cursor.model.getLineMaxColumn(position.lineNumber);
            return this._enter(cursor, false, ctx, new position_1.Position(position.lineNumber, column), new range_1.Range(position.lineNumber, column, position.lineNumber, column));
        };
        OneCursorOp.lineBreakInsert = function (cursor, ctx) {
            return this._enter(cursor, true, ctx);
        };
        OneCursorOp._enter = function (cursor, keepPosition, ctx, position, range) {
            if (typeof position === 'undefined') {
                position = cursor.getPosition();
            }
            if (typeof range === 'undefined') {
                range = cursor.getSelection();
            }
            ctx.shouldPushStackElementBefore = true;
            var r = onEnter_1.getEnterActionAtPosition(cursor.model, position.lineNumber, position.column);
            var enterAction = r.enterAction;
            var indentation = r.indentation;
            if (enterAction.indentAction === modes_1.IndentAction.None) {
                // Nothing special
                this.actualType(cursor, '\n' + cursor.model.normalizeIndentation(indentation + enterAction.appendText), keepPosition, ctx, range);
            }
            else if (enterAction.indentAction === modes_1.IndentAction.Indent) {
                // Indent once
                this.actualType(cursor, '\n' + cursor.model.normalizeIndentation(indentation + enterAction.appendText), keepPosition, ctx, range);
            }
            else if (enterAction.indentAction === modes_1.IndentAction.IndentOutdent) {
                // Ultra special
                var normalIndent = cursor.model.normalizeIndentation(indentation);
                var increasedIndent = cursor.model.normalizeIndentation(indentation + enterAction.appendText);
                var typeText = '\n' + increasedIndent + '\n' + normalIndent;
                if (keepPosition) {
                    ctx.executeCommand = new replaceCommand_1.ReplaceCommandWithoutChangingPosition(range, typeText);
                }
                else {
                    ctx.executeCommand = new replaceCommand_1.ReplaceCommandWithOffsetCursorState(range, typeText, -1, increasedIndent.length - normalIndent.length);
                }
            }
            else if (enterAction.indentAction === modes_1.IndentAction.Outdent) {
                var desiredIndentCount = shiftCommand_1.ShiftCommand.unshiftIndentCount(indentation, indentation.length + 1, cursor.model.getOptions().tabSize);
                var actualIndentation = '';
                for (var i = 0; i < desiredIndentCount; i++) {
                    actualIndentation += '\t';
                }
                this.actualType(cursor, '\n' + cursor.model.normalizeIndentation(actualIndentation + enterAction.appendText), keepPosition, ctx, range);
            }
            return true;
        };
        OneCursorOp._typeInterceptorAutoClosingCloseChar = function (cursor, ch, ctx) {
            if (!cursor.configuration.editor.autoClosingBrackets) {
                return false;
            }
            var selection = cursor.getSelection();
            if (!selection.isEmpty() || !cursor.modeConfiguration.autoClosingPairsClose.hasOwnProperty(ch)) {
                return false;
            }
            var position = cursor.getPosition();
            var lineText = cursor.model.getLineContent(position.lineNumber);
            var beforeCharacter = lineText[position.column - 1];
            if (beforeCharacter !== ch) {
                return false;
            }
            var typeSelection = new range_1.Range(position.lineNumber, position.column, position.lineNumber, position.column + 1);
            ctx.executeCommand = new replaceCommand_1.ReplaceCommand(typeSelection, ch);
            return true;
        };
        OneCursorOp._typeInterceptorAutoClosingOpenChar = function (cursor, ch, ctx) {
            if (!cursor.configuration.editor.autoClosingBrackets) {
                return false;
            }
            var selection = cursor.getSelection();
            if (!selection.isEmpty() || !cursor.modeConfiguration.autoClosingPairsOpen.hasOwnProperty(ch)) {
                return false;
            }
            var richEditSupport = cursor.model.getMode().richEditSupport;
            if (!richEditSupport || !richEditSupport.characterPair) {
                return false;
            }
            var position = cursor.getPosition();
            var lineText = cursor.model.getLineContent(position.lineNumber);
            var beforeCharacter = lineText[position.column - 1];
            // Only consider auto closing the pair if a space follows or if another autoclosed pair follows
            if (beforeCharacter) {
                var isBeforeCloseBrace = false;
                for (var closeBrace in cursor.modeConfiguration.autoClosingPairsClose) {
                    if (beforeCharacter === closeBrace) {
                        isBeforeCloseBrace = true;
                        break;
                    }
                }
                if (!isBeforeCloseBrace && !/\s/.test(beforeCharacter)) {
                    return false;
                }
            }
            var lineContext = cursor.model.getLineContext(position.lineNumber);
            var shouldAutoClosePair = false;
            try {
                shouldAutoClosePair = richEditSupport.characterPair.shouldAutoClosePair(ch, lineContext, position.column - 1);
            }
            catch (e) {
                errors_1.onUnexpectedError(e);
            }
            if (!shouldAutoClosePair) {
                return false;
            }
            ctx.shouldPushStackElementBefore = true;
            var closeCharacter = cursor.modeConfiguration.autoClosingPairsOpen[ch];
            ctx.executeCommand = new replaceCommand_1.ReplaceCommandWithOffsetCursorState(selection, ch + closeCharacter, 0, -closeCharacter.length);
            return true;
        };
        OneCursorOp._typeInterceptorSurroundSelection = function (cursor, ch, ctx) {
            if (!cursor.configuration.editor.autoClosingBrackets) {
                return false;
            }
            var selection = cursor.getSelection();
            if (selection.isEmpty() || !cursor.modeConfiguration.surroundingPairs.hasOwnProperty(ch)) {
                return false;
            }
            var selectionContainsOnlyWhitespace = true, lineNumber, startIndex, endIndex, charIndex, charCode, lineText, _tab = '\t'.charCodeAt(0), _space = ' '.charCodeAt(0);
            for (lineNumber = selection.startLineNumber; lineNumber <= selection.endLineNumber; lineNumber++) {
                lineText = cursor.model.getLineContent(lineNumber);
                startIndex = (lineNumber === selection.startLineNumber ? selection.startColumn - 1 : 0);
                endIndex = (lineNumber === selection.endLineNumber ? selection.endColumn - 1 : lineText.length);
                for (charIndex = startIndex; charIndex < endIndex; charIndex++) {
                    charCode = lineText.charCodeAt(charIndex);
                    if (charCode !== _tab && charCode !== _space) {
                        selectionContainsOnlyWhitespace = false;
                        // Break outer loop
                        lineNumber = selection.endLineNumber + 1;
                        // Break inner loop
                        charIndex = endIndex;
                    }
                }
            }
            if (selectionContainsOnlyWhitespace) {
                return false;
            }
            var closeCharacter = cursor.modeConfiguration.surroundingPairs[ch];
            ctx.shouldPushStackElementBefore = true;
            ctx.shouldPushStackElementAfter = true;
            ctx.executeCommand = new surroundSelectionCommand_1.SurroundSelectionCommand(selection, ch, closeCharacter);
            return true;
        };
        OneCursorOp._typeInterceptorElectricChar = function (cursor, ch, ctx) {
            var _this = this;
            if (!cursor.modeConfiguration.electricChars.hasOwnProperty(ch)) {
                return false;
            }
            ctx.postOperationRunnable = function (postOperationCtx) { return _this._typeInterceptorElectricCharRunnable(cursor, postOperationCtx); };
            return this.actualType(cursor, ch, false, ctx);
        };
        OneCursorOp._typeInterceptorElectricCharRunnable = function (cursor, ctx) {
            var position = cursor.getPosition();
            var lineText = cursor.model.getLineContent(position.lineNumber);
            var lineContext = cursor.model.getLineContext(position.lineNumber);
            var electricAction;
            var richEditSupport = cursor.model.getMode().richEditSupport;
            if (richEditSupport && richEditSupport.electricCharacter) {
                try {
                    electricAction = richEditSupport.electricCharacter.onElectricCharacter(lineContext, position.column - 2);
                }
                catch (e) {
                    errors_1.onUnexpectedError(e);
                }
            }
            if (electricAction) {
                var matchOpenBracket = electricAction.matchOpenBracket;
                var appendText = electricAction.appendText;
                if (matchOpenBracket) {
                    var match = cursor.model.findMatchingBracketUp(matchOpenBracket, {
                        lineNumber: position.lineNumber,
                        column: position.column - matchOpenBracket.length
                    });
                    if (match) {
                        var matchLineNumber = match.startLineNumber;
                        var matchLine = cursor.model.getLineContent(matchLineNumber);
                        var matchLineIndentation = strings.getLeadingWhitespace(matchLine);
                        var newIndentation = cursor.model.normalizeIndentation(matchLineIndentation);
                        var lineFirstNonBlankColumn = cursor.model.getLineFirstNonWhitespaceColumn(position.lineNumber) || position.column;
                        var oldIndentation = lineText.substring(0, lineFirstNonBlankColumn - 1);
                        if (oldIndentation !== newIndentation) {
                            var prefix = lineText.substring(lineFirstNonBlankColumn - 1, position.column - 1);
                            var typeText = newIndentation + prefix;
                            var typeSelection = new range_1.Range(position.lineNumber, 1, position.lineNumber, position.column);
                            ctx.shouldPushStackElementAfter = true;
                            ctx.executeCommand = new replaceCommand_1.ReplaceCommand(typeSelection, typeText);
                        }
                    }
                }
                else if (appendText) {
                    var columnDeltaOffset = -appendText.length;
                    if (electricAction.advanceCount) {
                        columnDeltaOffset += electricAction.advanceCount;
                    }
                    ctx.shouldPushStackElementAfter = true;
                    ctx.executeCommand = new replaceCommand_1.ReplaceCommandWithOffsetCursorState(cursor.getSelection(), appendText, 0, columnDeltaOffset);
                }
            }
        };
        OneCursorOp.actualType = function (cursor, text, keepPosition, ctx, range) {
            if (typeof range === 'undefined') {
                range = cursor.getSelection();
            }
            if (keepPosition) {
                ctx.executeCommand = new replaceCommand_1.ReplaceCommandWithoutChangingPosition(range, text);
            }
            else {
                ctx.executeCommand = new replaceCommand_1.ReplaceCommand(range, text);
            }
            return true;
        };
        OneCursorOp.type = function (cursor, ch, ctx) {
            if (this._typeInterceptorEnter(cursor, ch, ctx)) {
                return true;
            }
            if (this._typeInterceptorAutoClosingCloseChar(cursor, ch, ctx)) {
                return true;
            }
            if (this._typeInterceptorAutoClosingOpenChar(cursor, ch, ctx)) {
                return true;
            }
            if (this._typeInterceptorSurroundSelection(cursor, ch, ctx)) {
                return true;
            }
            if (this._typeInterceptorElectricChar(cursor, ch, ctx)) {
                return true;
            }
            return this.actualType(cursor, ch, false, ctx);
        };
        OneCursorOp.replacePreviousChar = function (cursor, txt, replaceCharCnt, ctx) {
            var pos = cursor.getPosition();
            var range;
            var startColumn = Math.max(1, pos.column - replaceCharCnt);
            range = new range_1.Range(pos.lineNumber, startColumn, pos.lineNumber, pos.column);
            ctx.executeCommand = new replaceCommand_1.ReplaceCommand(range, txt);
            return true;
        };
        OneCursorOp._goodIndentForLine = function (cursor, lineNumber) {
            var lastLineNumber = lineNumber - 1;
            for (lastLineNumber = lineNumber - 1; lastLineNumber >= 1; lastLineNumber--) {
                var lineText = cursor.model.getLineContent(lastLineNumber);
                var nonWhitespaceIdx = strings.lastNonWhitespaceIndex(lineText);
                if (nonWhitespaceIdx >= 0) {
                    break;
                }
            }
            if (lastLineNumber < 1) {
                // No previous line with content found
                return '\t';
            }
            var r = onEnter_1.getEnterActionAtPosition(cursor.model, lastLineNumber, cursor.model.getLineMaxColumn(lastLineNumber));
            var indentation;
            if (r.enterAction.indentAction === modes_1.IndentAction.Outdent) {
                var modelOpts = cursor.model.getOptions();
                var desiredIndentCount = shiftCommand_1.ShiftCommand.unshiftIndentCount(r.indentation, r.indentation.length, modelOpts.tabSize);
                indentation = '';
                for (var i = 0; i < desiredIndentCount; i++) {
                    indentation += '\t';
                }
                indentation = cursor.model.normalizeIndentation(indentation);
            }
            else {
                indentation = r.indentation;
            }
            var result = indentation + r.enterAction.appendText;
            if (result.length === 0) {
                // good position is at column 1, but we gotta do something...
                return '\t';
            }
            return result;
        };
        OneCursorOp.tab = function (cursor, ctx) {
            var selection = cursor.getSelection();
            if (selection.isEmpty()) {
                var typeText = '';
                if (cursor.model.getLineMaxColumn(selection.startLineNumber) === 1) {
                    // Line is empty => indent straight to the right place
                    typeText = cursor.model.normalizeIndentation(this._goodIndentForLine(cursor, selection.startLineNumber));
                }
                else {
                    var position = cursor.getPosition();
                    var modelOpts = cursor.model.getOptions();
                    if (modelOpts.insertSpaces) {
                        var visibleColumnFromColumn = cursor.getVisibleColumnFromColumn(position.lineNumber, position.column);
                        var tabSize = modelOpts.tabSize;
                        var spacesCnt = tabSize - (visibleColumnFromColumn % tabSize);
                        for (var i = 0; i < spacesCnt; i++) {
                            typeText += ' ';
                        }
                    }
                    else {
                        typeText = '\t';
                    }
                }
                ctx.executeCommand = new replaceCommand_1.ReplaceCommand(selection, typeText);
                return true;
            }
            else {
                return this.indent(cursor, ctx);
            }
        };
        OneCursorOp.indent = function (cursor, ctx) {
            var selection = cursor.getSelection();
            ctx.shouldPushStackElementBefore = true;
            ctx.shouldPushStackElementAfter = true;
            ctx.executeCommand = new shiftCommand_1.ShiftCommand(selection, {
                isUnshift: false,
                tabSize: cursor.model.getOptions().tabSize,
                oneIndent: cursor.model.getOneIndent()
            });
            ctx.shouldRevealHorizontal = false;
            return true;
        };
        OneCursorOp.outdent = function (cursor, ctx) {
            var selection = cursor.getSelection();
            ctx.shouldPushStackElementBefore = true;
            ctx.shouldPushStackElementAfter = true;
            ctx.executeCommand = new shiftCommand_1.ShiftCommand(selection, {
                isUnshift: true,
                tabSize: cursor.model.getOptions().tabSize,
                oneIndent: cursor.model.getOneIndent()
            });
            ctx.shouldRevealHorizontal = false;
            return true;
        };
        OneCursorOp.paste = function (cursor, text, pasteOnNewLine, ctx) {
            var position = cursor.getPosition();
            ctx.cursorPositionChangeReason = 'paste';
            if (pasteOnNewLine && text.charAt(text.length - 1) === '\n') {
                if (text.indexOf('\n') === text.length - 1) {
                    // Paste entire line at the beginning of line
                    var typeSelection = new range_1.Range(position.lineNumber, 1, position.lineNumber, 1);
                    ctx.executeCommand = new replaceCommand_1.ReplaceCommand(typeSelection, text);
                    return true;
                }
            }
            ctx.executeCommand = new replaceCommand_1.ReplaceCommand(cursor.getSelection(), text);
            return true;
        };
        // -------------------- END type interceptors & co.
        // -------------------- START delete handlers & co.
        OneCursorOp._autoClosingPairDelete = function (cursor, ctx) {
            // Returns true if delete was handled.
            if (!cursor.configuration.editor.autoClosingBrackets) {
                return false;
            }
            if (!cursor.getSelection().isEmpty()) {
                return false;
            }
            var position = cursor.getPosition();
            var lineText = cursor.model.getLineContent(position.lineNumber);
            var character = lineText[position.column - 2];
            if (!cursor.modeConfiguration.autoClosingPairsOpen.hasOwnProperty(character)) {
                return false;
            }
            var afterCharacter = lineText[position.column - 1];
            var closeCharacter = cursor.modeConfiguration.autoClosingPairsOpen[character];
            if (afterCharacter !== closeCharacter) {
                return false;
            }
            var deleteSelection = new range_1.Range(position.lineNumber, position.column - 1, position.lineNumber, position.column + 1);
            ctx.executeCommand = new replaceCommand_1.ReplaceCommand(deleteSelection, '');
            return true;
        };
        OneCursorOp.deleteLeft = function (cursor, ctx) {
            if (this._autoClosingPairDelete(cursor, ctx)) {
                // This was a case for an auto-closing pair delete
                return true;
            }
            var deleteSelection = cursor.getSelection();
            if (deleteSelection.isEmpty()) {
                var position = cursor.getPosition();
                var leftOfPosition = cursor.getLeftOfPosition(position.lineNumber, position.column);
                deleteSelection = new range_1.Range(leftOfPosition.lineNumber, leftOfPosition.column, position.lineNumber, position.column);
            }
            if (deleteSelection.isEmpty()) {
                // Probably at beginning of file => ignore
                return true;
            }
            if (deleteSelection.startLineNumber !== deleteSelection.endLineNumber) {
                ctx.shouldPushStackElementBefore = true;
            }
            ctx.executeCommand = new replaceCommand_1.ReplaceCommand(deleteSelection, '');
            return true;
        };
        OneCursorOp._findLastNonWhitespaceChar = function (str, startIndex) {
            for (var chIndex = startIndex; chIndex >= 0; chIndex--) {
                var ch = str.charAt(chIndex);
                if (ch !== ' ' && ch !== '\t') {
                    return chIndex;
                }
            }
            return -1;
        };
        OneCursorOp.deleteWordLeftWhitespace = function (cursor, ctx) {
            var position = cursor.getPosition();
            var lineContent = cursor.getLineContent(position.lineNumber);
            var startIndex = position.column - 2;
            var lastNonWhitespace = this._findLastNonWhitespaceChar(lineContent, startIndex);
            if (lastNonWhitespace + 1 < startIndex) {
                // bingo
                ctx.executeCommand = new replaceCommand_1.ReplaceCommand(new range_1.Range(position.lineNumber, lastNonWhitespace + 2, position.lineNumber, position.column), '');
                return true;
            }
            return false;
        };
        OneCursorOp.deleteWordLeft = function (cursor, whitespaceHeuristics, wordNavigationType, ctx) {
            if (this._autoClosingPairDelete(cursor, ctx)) {
                // This was a case for an auto-closing pair delete
                return true;
            }
            var selection = cursor.getSelection();
            if (selection.isEmpty()) {
                var position = cursor.getPosition();
                var lineNumber = position.lineNumber;
                var column = position.column;
                if (lineNumber === 1 && column === 1) {
                    // Ignore deleting at beginning of file
                    return true;
                }
                if (whitespaceHeuristics && this.deleteWordLeftWhitespace(cursor, ctx)) {
                    return true;
                }
                var prevWordOnLine = cursor.findPreviousWordOnLine(position);
                if (wordNavigationType === WordNavigationType.WordStart) {
                    if (prevWordOnLine) {
                        column = prevWordOnLine.start + 1;
                    }
                    else {
                        column = 1;
                    }
                }
                else {
                    if (prevWordOnLine && column <= prevWordOnLine.end + 1) {
                        prevWordOnLine = cursor.findPreviousWordOnLine(new position_1.Position(lineNumber, prevWordOnLine.start + 1));
                    }
                    if (prevWordOnLine) {
                        column = prevWordOnLine.end + 1;
                    }
                    else {
                        column = 1;
                    }
                }
                var deleteSelection = new range_1.Range(lineNumber, column, lineNumber, position.column);
                if (!deleteSelection.isEmpty()) {
                    ctx.executeCommand = new replaceCommand_1.ReplaceCommand(deleteSelection, '');
                    return true;
                }
            }
            return this.deleteLeft(cursor, ctx);
        };
        OneCursorOp.deleteRight = function (cursor, ctx) {
            var deleteSelection = cursor.getSelection();
            if (deleteSelection.isEmpty()) {
                var position = cursor.getPosition();
                var rightOfPosition = cursor.getRightOfPosition(position.lineNumber, position.column);
                deleteSelection = new range_1.Range(rightOfPosition.lineNumber, rightOfPosition.column, position.lineNumber, position.column);
            }
            if (deleteSelection.isEmpty()) {
                // Probably at end of file => ignore
                return true;
            }
            if (deleteSelection.startLineNumber !== deleteSelection.endLineNumber) {
                ctx.shouldPushStackElementBefore = true;
            }
            ctx.executeCommand = new replaceCommand_1.ReplaceCommand(deleteSelection, '');
            return true;
        };
        OneCursorOp._findFirstNonWhitespaceChar = function (str, startIndex) {
            var len = str.length;
            for (var chIndex = startIndex; chIndex < len; chIndex++) {
                var ch = str.charAt(chIndex);
                if (ch !== ' ' && ch !== '\t') {
                    return chIndex;
                }
            }
            return len;
        };
        OneCursorOp.deleteWordRightWhitespace = function (cursor, ctx) {
            var position = cursor.getPosition();
            var lineContent = cursor.getLineContent(position.lineNumber);
            var startIndex = position.column - 1;
            var firstNonWhitespace = this._findFirstNonWhitespaceChar(lineContent, startIndex);
            if (startIndex + 1 < firstNonWhitespace) {
                // bingo
                ctx.executeCommand = new replaceCommand_1.ReplaceCommand(new range_1.Range(position.lineNumber, position.column, position.lineNumber, firstNonWhitespace + 1), '');
                return true;
            }
            return false;
        };
        OneCursorOp.deleteWordRight = function (cursor, whitespaceHeuristics, wordNavigationType, ctx) {
            var selection = cursor.getSelection();
            if (selection.isEmpty()) {
                var position = cursor.getPosition();
                var lineNumber = position.lineNumber;
                var column = position.column;
                var lineCount = cursor.model.getLineCount();
                var maxColumn = cursor.model.getLineMaxColumn(lineNumber);
                if (lineNumber === lineCount && column === maxColumn) {
                    // Ignore deleting at end of file
                    return true;
                }
                if (whitespaceHeuristics && this.deleteWordRightWhitespace(cursor, ctx)) {
                    return true;
                }
                var nextWordOnLine = cursor.findNextWordOnLine(position);
                if (wordNavigationType === WordNavigationType.WordEnd) {
                    if (nextWordOnLine) {
                        column = nextWordOnLine.end + 1;
                    }
                    else {
                        column = maxColumn;
                    }
                }
                else {
                    if (nextWordOnLine && column >= nextWordOnLine.start + 1) {
                        nextWordOnLine = cursor.findNextWordOnLine(new position_1.Position(lineNumber, nextWordOnLine.end + 1));
                    }
                    if (nextWordOnLine) {
                        column = nextWordOnLine.start + 1;
                    }
                    else {
                        column = maxColumn;
                    }
                }
                var deleteSelection = new range_1.Range(lineNumber, column, lineNumber, position.column);
                if (!deleteSelection.isEmpty()) {
                    ctx.executeCommand = new replaceCommand_1.ReplaceCommand(deleteSelection, '');
                    return true;
                }
            }
            // fall back to normal deleteRight behavior
            return this.deleteRight(cursor, ctx);
        };
        OneCursorOp.deleteAllLeft = function (cursor, ctx) {
            if (this._autoClosingPairDelete(cursor, ctx)) {
                // This was a case for an auto-closing pair delete
                return true;
            }
            var selection = cursor.getSelection();
            if (selection.isEmpty()) {
                var position = cursor.getPosition();
                var lineNumber = position.lineNumber;
                var column = position.column;
                if (column === 1) {
                    // Ignore deleting at beginning of line
                    return true;
                }
                var deleteSelection = new range_1.Range(lineNumber, 1, lineNumber, column);
                if (!deleteSelection.isEmpty()) {
                    ctx.executeCommand = new replaceCommand_1.ReplaceCommand(deleteSelection, '');
                    return true;
                }
            }
            return this.deleteLeft(cursor, ctx);
        };
        OneCursorOp.deleteAllRight = function (cursor, ctx) {
            var selection = cursor.getSelection();
            if (selection.isEmpty()) {
                var position = cursor.getPosition();
                var lineNumber = position.lineNumber;
                var column = position.column;
                var maxColumn = cursor.model.getLineMaxColumn(lineNumber);
                if (column === maxColumn) {
                    // Ignore deleting at end of file
                    return true;
                }
                var deleteSelection = new range_1.Range(lineNumber, column, lineNumber, maxColumn);
                if (!deleteSelection.isEmpty()) {
                    ctx.executeCommand = new replaceCommand_1.ReplaceCommand(deleteSelection, '');
                    return true;
                }
            }
            return this.deleteRight(cursor, ctx);
        };
        OneCursorOp.cut = function (cursor, enableEmptySelectionClipboard, ctx) {
            var selection = cursor.getSelection();
            if (selection.isEmpty()) {
                if (enableEmptySelectionClipboard) {
                    // This is a full line cut
                    var position = cursor.getPosition();
                    var startLineNumber, startColumn, endLineNumber, endColumn;
                    if (position.lineNumber < cursor.model.getLineCount()) {
                        // Cutting a line in the middle of the model
                        startLineNumber = position.lineNumber;
                        startColumn = 1;
                        endLineNumber = position.lineNumber + 1;
                        endColumn = 1;
                    }
                    else if (position.lineNumber > 1) {
                        // Cutting the last line & there are more than 1 lines in the model
                        startLineNumber = position.lineNumber - 1;
                        startColumn = cursor.model.getLineMaxColumn(position.lineNumber - 1);
                        endLineNumber = position.lineNumber;
                        endColumn = cursor.model.getLineMaxColumn(position.lineNumber);
                    }
                    else {
                        // Cutting the single line that the model contains
                        startLineNumber = position.lineNumber;
                        startColumn = 1;
                        endLineNumber = position.lineNumber;
                        endColumn = cursor.model.getLineMaxColumn(position.lineNumber);
                    }
                    var deleteSelection = new range_1.Range(startLineNumber, startColumn, endLineNumber, endColumn);
                    if (!deleteSelection.isEmpty()) {
                        ctx.executeCommand = new replaceCommand_1.ReplaceCommand(deleteSelection, '');
                    }
                }
                else {
                    // Cannot cut empty selection
                    return false;
                }
            }
            else {
                // Delete left or right, they will both result in the selection being deleted
                this.deleteRight(cursor, ctx);
            }
            return true;
        };
        return OneCursorOp;
    }());
    exports.OneCursorOp = OneCursorOp;
    var CursorHelper = (function () {
        function CursorHelper(model, configuration) {
            var _this = this;
            this.model = model;
            this.configuration = configuration;
            this.moveHelper = new cursorMoveHelper_1.CursorMoveHelper({
                getIndentationOptions: function () {
                    return _this.model.getOptions();
                }
            });
        }
        CursorHelper.prototype.getLeftOfPosition = function (model, lineNumber, column) {
            return this.moveHelper.getLeftOfPosition(model, lineNumber, column);
        };
        CursorHelper.prototype.getRightOfPosition = function (model, lineNumber, column) {
            return this.moveHelper.getRightOfPosition(model, lineNumber, column);
        };
        CursorHelper.prototype.getPositionUp = function (model, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnFirstLine) {
            return this.moveHelper.getPositionUp(model, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnFirstLine);
        };
        CursorHelper.prototype.getPositionDown = function (model, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnLastLine) {
            return this.moveHelper.getPositionDown(model, lineNumber, column, leftoverVisibleColumns, count, allowMoveOnLastLine);
        };
        CursorHelper.prototype.getColumnAtBeginningOfLine = function (model, lineNumber, column) {
            return this.moveHelper.getColumnAtBeginningOfLine(model, lineNumber, column);
        };
        CursorHelper.prototype.getColumnAtEndOfLine = function (model, lineNumber, column) {
            return this.moveHelper.getColumnAtEndOfLine(model, lineNumber, column);
        };
        CursorHelper.prototype.columnSelect = function (model, fromLineNumber, fromVisibleColumn, toLineNumber, toVisibleColumn) {
            return this.moveHelper.columnSelect(model, fromLineNumber, fromVisibleColumn, toLineNumber, toVisibleColumn);
        };
        CursorHelper.prototype.visibleColumnFromColumn = function (model, lineNumber, column) {
            return this.moveHelper.visibleColumnFromColumn(model, lineNumber, column);
        };
        // /**
        //  * ATTENTION: This works with 0-based columns (as oposed to the regular 1-based columns)
        //  */
        // public nextTabColumn(column:number): number {
        // 	return CursorMoveHelper.nextTabColumn(column, this.configuration.getIndentationOptions().tabSize);
        // }
        // /**
        //  * ATTENTION: This works with 0-based columns (as oposed to the regular 1-based columns)
        //  */
        // public prevTabColumn(column:number): number {
        // 	return CursorMoveHelper.prevTabColumn(column, this.configuration.getIndentationOptions().tabSize);
        // }
        // public findWord(position:editorCommon.IEditorPosition, preference:string, skipSyntaxTokens:boolean=false): editorCommon.IWordRange {
        // 	var words = this.model.getWords(position.lineNumber);
        // 	var searchIndex:number, i:number, len:number;
        // 	if (skipSyntaxTokens) {
        // 		searchIndex = position.column - 1;
        // 		if (preference === 'left') {
        // 			for (i = words.length - 1; i >= 0; i--) {
        // 				if (words[i].start >= searchIndex) {
        // 					continue;
        // 				}
        // 				return words[i];
        // 			}
        // 		} else {
        // 			for (i = 0, len = words.length; i < len; i++) {
        // 				if (words[i].end <= searchIndex) {
        // 					continue;
        // 				}
        // 				return words[i];
        // 			}
        // 		}
        // 	} else {
        // 		searchIndex = position.column;
        // 		if (preference === 'left') {
        // 			if (searchIndex !== 1) {
        // 				searchIndex = searchIndex - 0.1;
        // 			}
        // 		} else {
        // 			if (searchIndex !== this.model.getLineMaxColumn(position.lineNumber)) {
        // 				searchIndex = searchIndex + 0.1;
        // 			}
        // 		}
        // 		searchIndex = searchIndex - 1;
        // 		for (i = 0, len = words.length; i < len; i++) {
        // 			if (words[i].start <= searchIndex && searchIndex <= words[i].end) {
        // 				return words[i];
        // 			}
        // 		}
        // 	}
        // 	return null;
        // }
        CursorHelper.prototype._createWord = function (lineContent, wordType, start, end) {
            // console.log('WORD ==> ' + start + ' => ' + end + ':::: <<<' + lineContent.substring(start, end) + '>>>');
            return { start: start, end: end, wordType: wordType };
        };
        CursorHelper.prototype.findPreviousWordOnLine = function (_position) {
            var position = this.model.validatePosition(_position);
            var wordSeparators = getMapForWordSeparators(this.configuration.editor.wordSeparators);
            var lineContent = this.model.getLineContent(position.lineNumber);
            var wordType = W_NONE;
            for (var chIndex = position.column - 2; chIndex >= 0; chIndex--) {
                var chCode = lineContent.charCodeAt(chIndex);
                var chClass = (wordSeparators[chCode] || CharacterClass.Regular);
                if (chClass === CH_REGULAR) {
                    if (wordType === W_SEPARATOR) {
                        return this._createWord(lineContent, wordType, chIndex + 1, this._findEndOfWord(lineContent, wordSeparators, wordType, chIndex + 1));
                    }
                    wordType = W_REGULAR;
                }
                else if (chClass === CH_WORD_SEPARATOR) {
                    if (wordType === W_REGULAR) {
                        return this._createWord(lineContent, wordType, chIndex + 1, this._findEndOfWord(lineContent, wordSeparators, wordType, chIndex + 1));
                    }
                    wordType = W_SEPARATOR;
                }
                else if (chClass === CH_WHITESPACE) {
                    if (wordType !== W_NONE) {
                        return this._createWord(lineContent, wordType, chIndex + 1, this._findEndOfWord(lineContent, wordSeparators, wordType, chIndex + 1));
                    }
                }
            }
            if (wordType !== W_NONE) {
                return this._createWord(lineContent, wordType, 0, this._findEndOfWord(lineContent, wordSeparators, wordType, 0));
            }
            return null;
        };
        CursorHelper.prototype._findEndOfWord = function (lineContent, wordSeparators, wordType, startIndex) {
            var len = lineContent.length;
            for (var chIndex = startIndex; chIndex < len; chIndex++) {
                var chCode = lineContent.charCodeAt(chIndex);
                var chClass = (wordSeparators[chCode] || CharacterClass.Regular);
                if (chClass === CH_WHITESPACE) {
                    return chIndex;
                }
                if (wordType === W_REGULAR && chClass === CH_WORD_SEPARATOR) {
                    return chIndex;
                }
                if (wordType === W_SEPARATOR && chClass === CH_REGULAR) {
                    return chIndex;
                }
            }
            return len;
        };
        CursorHelper.prototype.findNextWordOnLine = function (_position) {
            var position = this.model.validatePosition(_position);
            var wordSeparators = getMapForWordSeparators(this.configuration.editor.wordSeparators);
            var lineContent = this.model.getLineContent(position.lineNumber);
            var wordType = W_NONE;
            var len = lineContent.length;
            for (var chIndex = position.column - 1; chIndex < len; chIndex++) {
                var chCode = lineContent.charCodeAt(chIndex);
                var chClass = (wordSeparators[chCode] || CharacterClass.Regular);
                if (chClass === CH_REGULAR) {
                    if (wordType === W_SEPARATOR) {
                        return this._createWord(lineContent, wordType, this._findStartOfWord(lineContent, wordSeparators, wordType, chIndex - 1), chIndex);
                    }
                    wordType = W_REGULAR;
                }
                else if (chClass === CH_WORD_SEPARATOR) {
                    if (wordType === W_REGULAR) {
                        return this._createWord(lineContent, wordType, this._findStartOfWord(lineContent, wordSeparators, wordType, chIndex - 1), chIndex);
                    }
                    wordType = W_SEPARATOR;
                }
                else if (chClass === CH_WHITESPACE) {
                    if (wordType !== W_NONE) {
                        return this._createWord(lineContent, wordType, this._findStartOfWord(lineContent, wordSeparators, wordType, chIndex - 1), chIndex);
                    }
                }
            }
            if (wordType !== W_NONE) {
                return this._createWord(lineContent, wordType, this._findStartOfWord(lineContent, wordSeparators, wordType, len - 1), len);
            }
            return null;
        };
        CursorHelper.prototype._findStartOfWord = function (lineContent, wordSeparators, wordType, startIndex) {
            for (var chIndex = startIndex; chIndex >= 0; chIndex--) {
                var chCode = lineContent.charCodeAt(chIndex);
                var chClass = (wordSeparators[chCode] || CharacterClass.Regular);
                if (chClass === CH_WHITESPACE) {
                    return chIndex + 1;
                }
                if (wordType === W_REGULAR && chClass === CH_WORD_SEPARATOR) {
                    return chIndex + 1;
                }
                if (wordType === W_SEPARATOR && chClass === CH_REGULAR) {
                    return chIndex + 1;
                }
            }
            return 0;
        };
        return CursorHelper;
    }());
    function once(keyFn, computeFn) {
        var cache = {};
        return function (input) {
            var key = keyFn(input);
            if (!cache.hasOwnProperty(key)) {
                cache[key] = computeFn(input);
            }
            return cache[key];
        };
    }
    var getMapForWordSeparators = once(function (input) { return input; }, function (input) {
        var r = [];
        // Make array fast for ASCII text
        for (var chCode = 0; chCode < 256; chCode++) {
            r[chCode] = CharacterClass.Regular;
        }
        for (var i = 0, len = input.length; i < len; i++) {
            r[input.charCodeAt(i)] = CharacterClass.WordSeparator;
        }
        r[' '.charCodeAt(0)] = CharacterClass.Whitespace;
        r['\t'.charCodeAt(0)] = CharacterClass.Whitespace;
        return r;
    });
    var Utils = (function () {
        function Utils() {
        }
        /**
         * Range contains position (including edges)?
         */
        Utils.rangeContainsPosition = function (range, position) {
            if (position.lineNumber < range.startLineNumber || position.lineNumber > range.endLineNumber) {
                return false;
            }
            if (position.lineNumber === range.startLineNumber && position.column < range.startColumn) {
                return false;
            }
            if (position.lineNumber === range.endLineNumber && position.column > range.endColumn) {
                return false;
            }
            return true;
        };
        /**
         * Tests if position is contained inside range.
         * If position is either the starting or ending of a range, false is returned.
         */
        Utils.isPositionInsideRange = function (position, range) {
            if (position.lineNumber < range.startLineNumber) {
                return false;
            }
            if (position.lineNumber > range.endLineNumber) {
                return false;
            }
            if (position.lineNumber === range.startLineNumber && position.column < range.startColumn) {
                return false;
            }
            if (position.lineNumber === range.endLineNumber && position.column > range.endColumn) {
                return false;
            }
            return true;
        };
        Utils.isPositionAtRangeEdges = function (position, range) {
            if (position.lineNumber === range.startLineNumber && position.column === range.startColumn) {
                return true;
            }
            if (position.lineNumber === range.endLineNumber && position.column === range.endColumn) {
                return true;
            }
            return false;
        };
        return Utils;
    }());
});

define("vs/editor/common/controller/cursorCollection", ["require", "exports", 'vs/base/common/errors', 'vs/editor/common/controller/oneCursor', 'vs/editor/common/core/selection'], function (require, exports, errors_1, oneCursor_1, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var CursorCollection = (function () {
        function CursorCollection(editorId, model, configuration, viewModelHelper) {
            this.editorId = editorId;
            this.model = model;
            this.configuration = configuration;
            this.viewModelHelper = viewModelHelper;
            this.modeConfiguration = this.getModeConfiguration();
            this.primaryCursor = new oneCursor_1.OneCursor(this.editorId, this.model, this.configuration, this.modeConfiguration, this.viewModelHelper);
            this.secondaryCursors = [];
            this.lastAddedCursorIndex = 0;
        }
        CursorCollection.prototype.dispose = function () {
            this.primaryCursor.dispose();
            this.killSecondaryCursors();
        };
        CursorCollection.prototype.saveState = function () {
            return {
                primary: this.primaryCursor.saveState(),
                secondary: this.secondaryCursors.map(function (c) { return c.saveState(); })
            };
        };
        CursorCollection.prototype.restoreState = function (state) {
            this.primaryCursor.restoreState(state.primary);
            this.killSecondaryCursors();
            for (var i = 0; i < state.secondary.length; i++) {
                this.addSecondaryCursor(null);
                this.secondaryCursors[i].restoreState(state.secondary[i]);
            }
        };
        CursorCollection.prototype.updateMode = function () {
            var _this = this;
            this.modeConfiguration = this.getModeConfiguration();
            this.getAll().forEach(function (cursor) {
                cursor.updateModeConfiguration(_this.modeConfiguration);
            });
        };
        CursorCollection.prototype.getAll = function () {
            var result = [];
            result.push(this.primaryCursor);
            result = result.concat(this.secondaryCursors);
            return result;
        };
        CursorCollection.prototype.getPosition = function (index) {
            if (index === 0) {
                return this.primaryCursor.getPosition();
            }
            else {
                return this.secondaryCursors[index - 1].getPosition();
            }
        };
        CursorCollection.prototype.getViewPosition = function (index) {
            if (index === 0) {
                return this.primaryCursor.getViewPosition();
            }
            else {
                return this.secondaryCursors[index - 1].getViewPosition();
            }
        };
        CursorCollection.prototype.getPositions = function () {
            var result = [];
            result.push(this.primaryCursor.getPosition());
            for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
                result.push(this.secondaryCursors[i].getPosition());
            }
            return result;
        };
        CursorCollection.prototype.getViewPositions = function () {
            var result = [];
            result.push(this.primaryCursor.getViewPosition());
            for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
                result.push(this.secondaryCursors[i].getViewPosition());
            }
            return result;
        };
        CursorCollection.prototype.getSelection = function (index) {
            if (index === 0) {
                return this.primaryCursor.getSelection();
            }
            else {
                return this.secondaryCursors[index - 1].getSelection();
            }
        };
        CursorCollection.prototype.getSelections = function () {
            var result = [];
            result.push(this.primaryCursor.getSelection());
            for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
                result.push(this.secondaryCursors[i].getSelection());
            }
            return result;
        };
        CursorCollection.prototype.getViewSelections = function () {
            var result = [];
            result.push(this.primaryCursor.getViewSelection());
            for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
                result.push(this.secondaryCursors[i].getViewSelection());
            }
            return result;
        };
        CursorCollection.prototype.setSelections = function (selections, viewSelections) {
            this.primaryCursor.setSelection(selections[0]);
            this._setSecondarySelections(selections.slice(1));
            if (viewSelections) {
                this.primaryCursor.setViewSelection(viewSelections[0]);
                for (var i = 0; i < this.secondaryCursors.length; i++) {
                    this.secondaryCursors[i].setViewSelection(viewSelections[i + 1]);
                }
            }
        };
        CursorCollection.prototype.killSecondaryCursors = function () {
            return (this._setSecondarySelections([]) > 0);
        };
        CursorCollection.prototype.normalize = function () {
            this._mergeCursorsIfNecessary();
            this.primaryCursor.adjustBracketDecorations();
            for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
                this.secondaryCursors[i].adjustBracketDecorations();
            }
        };
        CursorCollection.prototype.addSecondaryCursor = function (selection) {
            var newCursor = new oneCursor_1.OneCursor(this.editorId, this.model, this.configuration, this.modeConfiguration, this.viewModelHelper);
            if (selection) {
                newCursor.setSelection(selection);
            }
            this.secondaryCursors.push(newCursor);
            this.lastAddedCursorIndex = this.secondaryCursors.length;
        };
        CursorCollection.prototype.duplicateCursors = function () {
            var newCursors = [];
            newCursors.push(this.primaryCursor.duplicate());
            for (var i = 0, len = this.secondaryCursors.length; i < len; i++) {
                newCursors.push(this.secondaryCursors[i].duplicate());
            }
            this.secondaryCursors = this.secondaryCursors.concat(newCursors);
            this.lastAddedCursorIndex = this.secondaryCursors.length;
        };
        CursorCollection.prototype.getLastAddedCursor = function () {
            if (this.secondaryCursors.length === 0 || this.lastAddedCursorIndex === 0) {
                return this.primaryCursor;
            }
            return this.secondaryCursors[this.lastAddedCursorIndex - 1];
        };
        /**
         * Creates or disposes secondary cursors as necessary to match the number of `secondarySelections`.
         * Return value:
         * 		- a positive number indicates the number of secondary cursors added
         * 		- a negative number indicates the number of secondary cursors removed
         * 		- 0 indicates that no changes have been done to the secondary cursors list
         */
        CursorCollection.prototype._setSecondarySelections = function (secondarySelections) {
            var secondaryCursorsLength = this.secondaryCursors.length;
            var secondarySelectionsLength = secondarySelections.length;
            var returnValue = secondarySelectionsLength - secondaryCursorsLength;
            if (secondaryCursorsLength < secondarySelectionsLength) {
                var createCnt = secondarySelectionsLength - secondaryCursorsLength;
                for (var i = 0; i < createCnt; i++) {
                    this.addSecondaryCursor(null);
                }
            }
            else if (secondaryCursorsLength > secondarySelectionsLength) {
                var removeCnt = secondaryCursorsLength - secondarySelectionsLength;
                for (var i = 0; i < removeCnt; i++) {
                    this._removeSecondaryCursor(this.secondaryCursors.length - 1);
                }
            }
            for (var i = 0; i < secondarySelectionsLength; i++) {
                if (secondarySelections[i]) {
                    this.secondaryCursors[i].setSelection(secondarySelections[i]);
                }
            }
            return returnValue;
        };
        CursorCollection.prototype._removeSecondaryCursor = function (removeIndex) {
            if (this.lastAddedCursorIndex >= removeIndex + 1) {
                this.lastAddedCursorIndex--;
            }
            this.secondaryCursors[removeIndex].dispose();
            this.secondaryCursors.splice(removeIndex, 1);
        };
        CursorCollection.prototype._mergeCursorsIfNecessary = function () {
            if (this.secondaryCursors.length === 0) {
                return;
            }
            var cursors = this.getAll();
            var sortedCursors = [];
            for (var i = 0; i < cursors.length; i++) {
                sortedCursors.push({
                    index: i,
                    selection: cursors[i].getSelection(),
                    viewSelection: cursors[i].getViewSelection()
                });
            }
            sortedCursors.sort(function (a, b) {
                if (a.viewSelection.startLineNumber === b.viewSelection.startLineNumber) {
                    return a.viewSelection.startColumn - b.viewSelection.startColumn;
                }
                return a.viewSelection.startLineNumber - b.viewSelection.startLineNumber;
            });
            for (var sortedCursorIndex = 0; sortedCursorIndex < sortedCursors.length - 1; sortedCursorIndex++) {
                var current = sortedCursors[sortedCursorIndex];
                var next = sortedCursors[sortedCursorIndex + 1];
                var currentViewSelection = current.viewSelection;
                var nextViewSelection = next.viewSelection;
                if (nextViewSelection.getStartPosition().isBeforeOrEqual(currentViewSelection.getEndPosition())) {
                    var winnerSortedCursorIndex = current.index < next.index ? sortedCursorIndex : sortedCursorIndex + 1;
                    var looserSortedCursorIndex = current.index < next.index ? sortedCursorIndex + 1 : sortedCursorIndex;
                    var looserIndex = sortedCursors[looserSortedCursorIndex].index;
                    var winnerIndex = sortedCursors[winnerSortedCursorIndex].index;
                    var looserSelection = sortedCursors[looserSortedCursorIndex].selection;
                    var winnerSelection = sortedCursors[winnerSortedCursorIndex].selection;
                    if (!looserSelection.equalsSelection(winnerSelection)) {
                        var resultingRange = looserSelection.plusRange(winnerSelection);
                        var looserSelectionIsLTR = (looserSelection.selectionStartLineNumber === looserSelection.startLineNumber && looserSelection.selectionStartColumn === looserSelection.startColumn);
                        var winnerSelectionIsLTR = (winnerSelection.selectionStartLineNumber === winnerSelection.startLineNumber && winnerSelection.selectionStartColumn === winnerSelection.startColumn);
                        // Give more importance to the last added cursor (think Ctrl-dragging + hitting another cursor)
                        var resultingSelectionIsLTR;
                        if (looserIndex === this.lastAddedCursorIndex) {
                            resultingSelectionIsLTR = looserSelectionIsLTR;
                            this.lastAddedCursorIndex = winnerIndex;
                        }
                        else {
                            // Winner takes it all
                            resultingSelectionIsLTR = winnerSelectionIsLTR;
                        }
                        var resultingSelection;
                        if (resultingSelectionIsLTR) {
                            resultingSelection = new selection_1.Selection(resultingRange.startLineNumber, resultingRange.startColumn, resultingRange.endLineNumber, resultingRange.endColumn);
                        }
                        else {
                            resultingSelection = new selection_1.Selection(resultingRange.endLineNumber, resultingRange.endColumn, resultingRange.startLineNumber, resultingRange.startColumn);
                        }
                        sortedCursors[winnerSortedCursorIndex].selection = resultingSelection;
                        cursors[winnerIndex].setSelection(resultingSelection);
                    }
                    for (var j = 0; j < sortedCursors.length; j++) {
                        if (sortedCursors[j].index > looserIndex) {
                            sortedCursors[j].index--;
                        }
                    }
                    cursors.splice(looserIndex, 1);
                    sortedCursors.splice(looserSortedCursorIndex, 1);
                    this._removeSecondaryCursor(looserIndex - 1);
                    sortedCursorIndex--;
                }
            }
        };
        CursorCollection.prototype.getModeConfiguration = function () {
            var i;
            var result = {
                electricChars: {},
                autoClosingPairsOpen: {},
                autoClosingPairsClose: {},
                surroundingPairs: {}
            };
            var richEditSupport = this.model.getMode().richEditSupport;
            var electricChars;
            if (richEditSupport && richEditSupport.electricCharacter) {
                try {
                    electricChars = richEditSupport.electricCharacter.getElectricCharacters();
                }
                catch (e) {
                    errors_1.onUnexpectedError(e);
                    electricChars = null;
                }
            }
            if (electricChars) {
                for (i = 0; i < electricChars.length; i++) {
                    result.electricChars[electricChars[i]] = true;
                }
            }
            var autoClosingPairs;
            if (richEditSupport && richEditSupport.characterPair) {
                try {
                    autoClosingPairs = richEditSupport.characterPair.getAutoClosingPairs();
                }
                catch (e) {
                    errors_1.onUnexpectedError(e);
                    autoClosingPairs = null;
                }
            }
            if (autoClosingPairs) {
                for (i = 0; i < autoClosingPairs.length; i++) {
                    result.autoClosingPairsOpen[autoClosingPairs[i].open] = autoClosingPairs[i].close;
                    result.autoClosingPairsClose[autoClosingPairs[i].close] = autoClosingPairs[i].open;
                }
            }
            var surroundingPairs;
            if (richEditSupport && richEditSupport.characterPair) {
                try {
                    surroundingPairs = richEditSupport.characterPair.getSurroundingPairs();
                }
                catch (e) {
                    errors_1.onUnexpectedError(e);
                    surroundingPairs = null;
                }
            }
            if (surroundingPairs) {
                for (i = 0; i < surroundingPairs.length; i++) {
                    result.surroundingPairs[surroundingPairs[i].open] = surroundingPairs[i].close;
                }
            }
            return result;
        };
        return CursorCollection;
    }());
    exports.CursorCollection = CursorCollection;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/modes/supports/suggestSupport", ["require", "exports", 'vs/base/common/winjs.base', 'vs/editor/common/modes/modesFilters', 'vs/editor/common/modes/supports'], function (require, exports, winjs_base_1, modesFilters_1, supports_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var SuggestSupport = (function () {
        function SuggestSupport(modeId, contribution) {
            this._modeId = modeId;
            this.contribution = contribution;
            this.suggest = function (resource, position) { return contribution.suggest(resource, position); };
            if (typeof contribution.getSuggestionDetails === 'function') {
                this.getSuggestionDetails = function (resource, position, suggestion) { return contribution.getSuggestionDetails(resource, position, suggestion); };
            }
        }
        SuggestSupport.prototype.shouldAutotriggerSuggest = function (context, offset, triggeredByCharacter) {
            var _this = this;
            return supports_1.handleEvent(context, offset, function (nestedMode, context, offset) {
                if (_this._modeId === nestedMode.getId()) {
                    if (_this.contribution.disableAutoTrigger) {
                        return false;
                    }
                    if (!Array.isArray(_this.contribution.excludeTokens)) {
                        return true;
                    }
                    if (_this.contribution.excludeTokens.length === 1 && _this.contribution.excludeTokens[0] === '*') {
                        return false;
                    }
                    return !supports_1.isLineToken(context, offset - 1, _this.contribution.excludeTokens, true);
                }
                else if (nestedMode.suggestSupport) {
                    return nestedMode.suggestSupport.shouldAutotriggerSuggest(context, offset, triggeredByCharacter);
                }
                else {
                    return false;
                }
            });
        };
        SuggestSupport.prototype.getFilter = function () {
            return modesFilters_1.DefaultFilter;
        };
        SuggestSupport.prototype.getTriggerCharacters = function () {
            return this.contribution.triggerCharacters;
        };
        SuggestSupport.prototype.shouldShowEmptySuggestionList = function () {
            return true;
        };
        return SuggestSupport;
    }());
    exports.SuggestSupport = SuggestSupport;
    var TextualSuggestSupport = (function () {
        function TextualSuggestSupport(modeId, editorWorkerService) {
            this._modeId = modeId;
            this._editorWorkerService = editorWorkerService;
        }
        TextualSuggestSupport.prototype.suggest = function (resource, position, triggerCharacter) {
            return this._editorWorkerService.textualSuggest(resource, position);
        };
        TextualSuggestSupport.prototype.getFilter = function () {
            return modesFilters_1.StrictPrefix;
        };
        TextualSuggestSupport.prototype.getTriggerCharacters = function () {
            return [];
        };
        TextualSuggestSupport.prototype.shouldShowEmptySuggestionList = function () {
            return true;
        };
        TextualSuggestSupport.prototype.shouldAutotriggerSuggest = function (context, offset, triggeredByCharacter) {
            var _this = this;
            return supports_1.handleEvent(context, offset, function (nestedMode, context, offset) {
                if (_this._modeId === nestedMode.getId()) {
                    return true;
                }
                else if (nestedMode.suggestSupport) {
                    return nestedMode.suggestSupport.shouldAutotriggerSuggest(context, offset, triggeredByCharacter);
                }
                else {
                    return false;
                }
            });
        };
        return TextualSuggestSupport;
    }());
    exports.TextualSuggestSupport = TextualSuggestSupport;
    var PredefinedResultSuggestSupport = (function (_super) {
        __extends(PredefinedResultSuggestSupport, _super);
        function PredefinedResultSuggestSupport(modeId, modelService, predefined, triggerCharacters, disableAutoTrigger) {
            _super.call(this, modeId, {
                triggerCharacters: triggerCharacters,
                disableAutoTrigger: disableAutoTrigger,
                excludeTokens: [],
                suggest: function (resource, position) {
                    var model = modelService.getModel(resource);
                    var result = _addSuggestionsAtPosition(model, position, predefined, null);
                    return winjs_base_1.TPromise.as(result);
                }
            });
        }
        return PredefinedResultSuggestSupport;
    }(SuggestSupport));
    exports.PredefinedResultSuggestSupport = PredefinedResultSuggestSupport;
    var TextualAndPredefinedResultSuggestSupport = (function (_super) {
        __extends(TextualAndPredefinedResultSuggestSupport, _super);
        function TextualAndPredefinedResultSuggestSupport(modeId, modelService, editorWorkerService, predefined, triggerCharacters, disableAutoTrigger) {
            _super.call(this, modeId, {
                triggerCharacters: triggerCharacters,
                disableAutoTrigger: disableAutoTrigger,
                excludeTokens: [],
                suggest: function (resource, position) {
                    return editorWorkerService.textualSuggest(resource, position).then(function (textualSuggestions) {
                        var model = modelService.getModel(resource);
                        var result = _addSuggestionsAtPosition(model, position, predefined, textualSuggestions);
                        return result;
                    });
                }
            });
        }
        return TextualAndPredefinedResultSuggestSupport;
    }(SuggestSupport));
    exports.TextualAndPredefinedResultSuggestSupport = TextualAndPredefinedResultSuggestSupport;
    function _addSuggestionsAtPosition(model, position, predefined, superSuggestions) {
        if (!predefined || predefined.length === 0) {
            return superSuggestions;
        }
        if (!superSuggestions) {
            superSuggestions = [];
        }
        superSuggestions.push({
            currentWord: model.getWordUntilPosition(position).word,
            suggestions: predefined.slice(0)
        });
        return superSuggestions;
    }
    function filterSuggestions(value) {
        if (!value) {
            return;
        }
        // filter suggestions
        var accept = modesFilters_1.DefaultFilter, result = [];
        result.push({
            currentWord: value.currentWord,
            suggestions: value.suggestions.filter(function (element) { return !!accept(value.currentWord, element); }),
            incomplete: value.incomplete
        });
        return result;
    }
    exports.filterSuggestions = filterSuggestions;
});

define("vs/editor/common/modes/monarch/monarchDefinition", ["require", "exports", 'vs/editor/common/modes/supports/suggestSupport'], function (require, exports, suggestSupport_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function createRichEditSupport(lexer) {
        return {
            wordPattern: lexer.wordDefinition,
            comments: {
                lineComment: lexer.lineComment,
                blockComment: [lexer.blockCommentStart, lexer.blockCommentEnd]
            },
            brackets: lexer.standardBrackets,
            __electricCharacterSupport: {
                // regexBrackets: lexer.enhancedBrackets,
                caseInsensitive: lexer.ignoreCase,
                embeddedElectricCharacters: lexer.outdentTriggers.split('')
            },
            __characterPairSupport: {
                autoClosingPairs: lexer.autoClosingPairs
            }
        };
    }
    exports.createRichEditSupport = createRichEditSupport;
    function createSuggestSupport(modelService, editorWorkerService, modeId, lexer) {
        if (lexer.suggestSupport.textualCompletions) {
            return new suggestSupport_1.TextualAndPredefinedResultSuggestSupport(modeId, modelService, editorWorkerService, lexer.suggestSupport.snippets, lexer.suggestSupport.triggerCharacters, lexer.suggestSupport.disableAutoTrigger);
        }
        else {
            return new suggestSupport_1.PredefinedResultSuggestSupport(modeId, modelService, lexer.suggestSupport.snippets, lexer.suggestSupport.triggerCharacters, lexer.suggestSupport.disableAutoTrigger);
        }
    }
    exports.createSuggestSupport = createSuggestSupport;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/viewLayout/editorScrollable", ["require", "exports", 'vs/base/common/eventEmitter'], function (require, exports, eventEmitter_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EditorScrollable = (function (_super) {
        __extends(EditorScrollable, _super);
        function EditorScrollable() {
            _super.call(this, [
                EditorScrollable._SCROLL_EVENT,
                EditorScrollable._INTERNAL_SIZE_CHANGED_EVENT
            ]);
            this.scrollTop = 0;
            this.scrollLeft = 0;
            this.scrollWidth = 0;
            this.scrollHeight = 0;
            this.width = 0;
            this.height = 0;
        }
        EditorScrollable.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
        };
        // ------------ (visible) width
        EditorScrollable.prototype.getWidth = function () {
            return this.width;
        };
        EditorScrollable.prototype.setWidth = function (width) {
            width = Math.floor(width);
            if (width < 0) {
                width = 0;
            }
            if (this.width !== width) {
                this.width = width;
                // Revalidate
                this.setScrollWidth(this.scrollWidth);
                this.setScrollLeft(this.scrollLeft);
            }
        };
        // ------------ scroll width
        EditorScrollable.prototype.getScrollWidth = function () {
            return this.scrollWidth;
        };
        EditorScrollable.prototype.setScrollWidth = function (scrollWidth) {
            scrollWidth = Math.floor(scrollWidth);
            if (scrollWidth < this.width) {
                scrollWidth = this.width;
            }
            if (this.scrollWidth !== scrollWidth) {
                this.scrollWidth = scrollWidth;
                // Revalidate
                this.setScrollLeft(this.scrollLeft);
                this._emitInternalSizeEvent();
            }
        };
        // ------------ scroll left
        EditorScrollable.prototype.getScrollLeft = function () {
            return this.scrollLeft;
        };
        EditorScrollable.prototype.setScrollLeft = function (scrollLeft) {
            scrollLeft = Math.floor(scrollLeft);
            if (scrollLeft < 0) {
                scrollLeft = 0;
            }
            if (scrollLeft + this.width > this.scrollWidth) {
                scrollLeft = this.scrollWidth - this.width;
            }
            if (this.scrollLeft !== scrollLeft) {
                this.scrollLeft = scrollLeft;
                this._emitScrollEvent(false, true);
            }
        };
        // ------------ (visible) height
        EditorScrollable.prototype.getHeight = function () {
            return this.height;
        };
        EditorScrollable.prototype.setHeight = function (height) {
            height = Math.floor(height);
            if (height < 0) {
                height = 0;
            }
            if (this.height !== height) {
                this.height = height;
                // Revalidate
                this.setScrollHeight(this.scrollHeight);
                this.setScrollTop(this.scrollTop);
            }
        };
        // ------------ scroll height
        EditorScrollable.prototype.getScrollHeight = function () {
            return this.scrollHeight;
        };
        EditorScrollable.prototype.setScrollHeight = function (scrollHeight) {
            scrollHeight = Math.floor(scrollHeight);
            if (scrollHeight < this.height) {
                scrollHeight = this.height;
            }
            if (this.scrollHeight !== scrollHeight) {
                this.scrollHeight = scrollHeight;
                // Revalidate
                this.setScrollTop(this.scrollTop);
                this._emitInternalSizeEvent();
            }
        };
        // ------------ scroll top
        EditorScrollable.prototype.getScrollTop = function () {
            return this.scrollTop;
        };
        EditorScrollable.prototype.setScrollTop = function (scrollTop) {
            scrollTop = Math.floor(scrollTop);
            if (scrollTop < 0) {
                scrollTop = 0;
            }
            if (scrollTop + this.height > this.scrollHeight) {
                scrollTop = this.scrollHeight - this.height;
            }
            if (this.scrollTop !== scrollTop) {
                this.scrollTop = scrollTop;
                this._emitScrollEvent(true, false);
            }
        };
        EditorScrollable.prototype._emitScrollEvent = function (vertical, horizontal) {
            var e = {
                vertical: vertical,
                horizontal: horizontal,
                scrollTop: this.scrollTop,
                scrollLeft: this.scrollLeft
            };
            this.emit(EditorScrollable._SCROLL_EVENT, e);
        };
        EditorScrollable.prototype.addScrollListener = function (listener) {
            return this.addListener2(EditorScrollable._SCROLL_EVENT, listener);
        };
        EditorScrollable.prototype._emitInternalSizeEvent = function () {
            this.emit(EditorScrollable._INTERNAL_SIZE_CHANGED_EVENT);
        };
        EditorScrollable.prototype.addInternalSizeChangeListener = function (listener) {
            return this.addListener2(EditorScrollable._INTERNAL_SIZE_CHANGED_EVENT, listener);
        };
        // ------------ events
        EditorScrollable._SCROLL_EVENT = 'scroll';
        EditorScrollable._INTERNAL_SIZE_CHANGED_EVENT = 'internalSizeChanged';
        return EditorScrollable;
    }(eventEmitter_1.EventEmitter));
    exports.EditorScrollable = EditorScrollable;
});

define("vs/editor/common/viewModel/characterHardWrappingLineMapper", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/editorCommon', 'vs/editor/common/viewModel/prefixSumComputer'], function (require, exports, strings, editorCommon_1, prefixSumComputer_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var throwawayIndexOfResult = {
        index: -1,
        remainder: -1
    };
    var BREAK_BEFORE_CLASS = 1;
    var BREAK_AFTER_CLASS = 2;
    var BREAK_OBTRUSIVE_CLASS = 3;
    function buildCharacterClassesMap(BREAK_BEFORE, BREAK_AFTER, BREAK_OBTRUSIVE) {
        var result = [], maxCharCode = 0, allBreakingChars = BREAK_BEFORE + BREAK_AFTER + BREAK_OBTRUSIVE, i;
        for (i = 0; i < allBreakingChars.length; i++) {
            maxCharCode = Math.max(maxCharCode, allBreakingChars.charCodeAt(i));
        }
        for (i = 0; i <= maxCharCode; i++) {
            result[i] = 0;
        }
        for (i = 0; i < BREAK_BEFORE.length; i++) {
            result[BREAK_BEFORE.charCodeAt(i)] = BREAK_BEFORE_CLASS;
        }
        for (i = 0; i < BREAK_AFTER.length; i++) {
            result[BREAK_AFTER.charCodeAt(i)] = BREAK_AFTER_CLASS;
        }
        for (i = 0; i < BREAK_OBTRUSIVE.length; i++) {
            result[BREAK_OBTRUSIVE.charCodeAt(i)] = BREAK_OBTRUSIVE_CLASS;
        }
        return result;
    }
    var CharacterHardWrappingLineMapperFactory = (function () {
        function CharacterHardWrappingLineMapperFactory(breakBeforeChars, breakAfterChars, breakObtrusiveChars) {
            this.characterClasses = buildCharacterClassesMap(breakBeforeChars, breakAfterChars, breakObtrusiveChars);
        }
        // TODO@Alex -> duplicated in lineCommentCommand
        CharacterHardWrappingLineMapperFactory.nextVisibleColumn = function (currentVisibleColumn, tabSize, isTab, columnSize) {
            if (isTab) {
                return currentVisibleColumn + (tabSize - (currentVisibleColumn % tabSize));
            }
            return currentVisibleColumn + columnSize;
        };
        CharacterHardWrappingLineMapperFactory.prototype.createLineMapping = function (lineText, tabSize, breakingColumn, columnsForFullWidthChar, hardWrappingIndent) {
            if (breakingColumn === -1) {
                return null;
            }
            var wrappedTextIndentVisibleColumn = 0, wrappedTextIndent = '', TAB_CHAR_CODE = '\t'.charCodeAt(0);
            if (hardWrappingIndent !== editorCommon_1.WrappingIndent.None) {
                var firstNonWhitespaceIndex = strings.firstNonWhitespaceIndex(lineText);
                if (firstNonWhitespaceIndex !== -1) {
                    wrappedTextIndent = lineText.substring(0, firstNonWhitespaceIndex);
                    for (var i = 0; i < firstNonWhitespaceIndex; i++) {
                        wrappedTextIndentVisibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(wrappedTextIndentVisibleColumn, tabSize, lineText.charCodeAt(i) === TAB_CHAR_CODE, 1);
                    }
                    if (hardWrappingIndent === editorCommon_1.WrappingIndent.Indent) {
                        wrappedTextIndent += '\t';
                        wrappedTextIndentVisibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(wrappedTextIndentVisibleColumn, tabSize, true, 1);
                    }
                    // Force sticking to beginning of line if indentColumn > 66% breakingColumn
                    if (wrappedTextIndentVisibleColumn > 1 / 2 * breakingColumn) {
                        wrappedTextIndent = '';
                        wrappedTextIndentVisibleColumn = 0;
                    }
                }
            }
            var characterClasses = this.characterClasses, lastBreakingOffset = 0, // Last 0-based offset in the lineText at which a break happened
            breakingLengths = [], // The length of each broken-up line text
            breakingLengthsIndex = 0, // The count of breaks already done
            i, len, visibleColumn, // Visible column since the beginning of the current line
            charCode, charCodeIsTab, charCodeClass, breakBeforeOffset, // 0-based offset in the lineText before which breaking
            restoreVisibleColumnFrom; // visible column used to re-establish a correct `visibleColumn`
            var niceBreakOffset = -1, // Last index of a character that indicates a break should happen before it (more desirable)
            niceBreakVisibleColumn = 0, // visible column if a break were to be later introduced before `niceBreakOffset`
            obtrusiveBreakOffset = -1, // Last index of a character that indicates a break should happen before it (less desirable)
            obtrusiveBreakVisibleColumn = 0; // visible column if a break were to be later introduced before `obtrusiveBreakOffset`
            visibleColumn = 0;
            for (i = 0, len = lineText.length; i < len; i++) {
                // At this point, there is a certainty that the character before `i` fits on the current line,
                // but the character at `i` might not fit
                charCode = lineText.charCodeAt(i);
                charCodeIsTab = (charCode === TAB_CHAR_CODE);
                charCodeClass = charCode < characterClasses.length ? characterClasses[charCode] : 0;
                if (charCodeClass === BREAK_BEFORE_CLASS) {
                    // This is a character that indicates that a break should happen before it
                    // Since we are certain the character before `i` fits, there's no extra checking needed,
                    // just mark it as a nice breaking opportunity
                    niceBreakOffset = i;
                    niceBreakVisibleColumn = 0;
                }
                var charColumnSize = 1;
                if (strings.isFullWidthCharacter(charCode)) {
                    charColumnSize = columnsForFullWidthChar;
                }
                // Advance visibleColumn with character at `i`
                visibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(visibleColumn, tabSize, charCodeIsTab, charColumnSize);
                if (visibleColumn > breakingColumn && i !== 0) {
                    // We need to break at least before character at `i`:
                    //  - break before niceBreakLastOffset if it exists (and re-establish a correct visibleColumn by using niceBreakVisibleColumn + charAt(i))
                    //  - otherwise, break before obtrusiveBreakLastOffset if it exists (and re-establish a correct visibleColumn by using obtrusiveBreakVisibleColumn + charAt(i))
                    //  - otherwise, break before i (and re-establish a correct visibleColumn by charAt(i))
                    if (niceBreakOffset !== -1) {
                        // We will break before `niceBreakLastOffset`
                        breakBeforeOffset = niceBreakOffset;
                        restoreVisibleColumnFrom = niceBreakVisibleColumn + wrappedTextIndentVisibleColumn;
                    }
                    else if (obtrusiveBreakOffset !== -1) {
                        // We will break before `obtrusiveBreakLastOffset`
                        breakBeforeOffset = obtrusiveBreakOffset;
                        restoreVisibleColumnFrom = obtrusiveBreakVisibleColumn + wrappedTextIndentVisibleColumn;
                    }
                    else {
                        // We will break before `i`
                        breakBeforeOffset = i;
                        restoreVisibleColumnFrom = 0 + wrappedTextIndentVisibleColumn;
                    }
                    // Break before character at `breakBeforeOffset`
                    breakingLengths[breakingLengthsIndex++] = breakBeforeOffset - lastBreakingOffset;
                    lastBreakingOffset = breakBeforeOffset;
                    // Re-establish visibleColumn by taking character at `i` into account
                    visibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(restoreVisibleColumnFrom, tabSize, charCodeIsTab, charColumnSize);
                    // Reset markers
                    niceBreakOffset = -1;
                    niceBreakVisibleColumn = 0;
                    obtrusiveBreakOffset = -1;
                    obtrusiveBreakVisibleColumn = 0;
                }
                // At this point, there is a certainty that the character at `i` fits on the current line
                if (niceBreakOffset !== -1) {
                    // Advance niceBreakVisibleColumn
                    niceBreakVisibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(niceBreakVisibleColumn, tabSize, charCodeIsTab, charColumnSize);
                }
                if (obtrusiveBreakOffset !== -1) {
                    // Advance obtrusiveBreakVisibleColumn
                    obtrusiveBreakVisibleColumn = CharacterHardWrappingLineMapperFactory.nextVisibleColumn(obtrusiveBreakVisibleColumn, tabSize, charCodeIsTab, charColumnSize);
                }
                if (charCodeClass === BREAK_AFTER_CLASS) {
                    // This is a character that indicates that a break should happen after it
                    niceBreakOffset = i + 1;
                    niceBreakVisibleColumn = 0;
                }
                if (charCodeClass === BREAK_OBTRUSIVE_CLASS) {
                    // This is an obtrusive character that indicates that a break should happen after it
                    obtrusiveBreakOffset = i + 1;
                    obtrusiveBreakVisibleColumn = 0;
                }
            }
            if (breakingLengthsIndex === 0) {
                return null;
            }
            // Add last segment
            breakingLengths[breakingLengthsIndex++] = len - lastBreakingOffset;
            return new CharacterHardWrappingLineMapping(new prefixSumComputer_1.PrefixSumComputer(breakingLengths), wrappedTextIndent);
        };
        return CharacterHardWrappingLineMapperFactory;
    }());
    exports.CharacterHardWrappingLineMapperFactory = CharacterHardWrappingLineMapperFactory;
    var CharacterHardWrappingLineMapping = (function () {
        function CharacterHardWrappingLineMapping(prefixSums, wrappedLinesIndent) {
            this._prefixSums = prefixSums;
            this._wrappedLinesIndent = wrappedLinesIndent;
        }
        CharacterHardWrappingLineMapping.prototype.getOutputLineCount = function () {
            return this._prefixSums.getCount();
        };
        CharacterHardWrappingLineMapping.prototype.getWrappedLinesIndent = function () {
            return this._wrappedLinesIndent;
        };
        CharacterHardWrappingLineMapping.prototype.getInputOffsetOfOutputPosition = function (outputLineIndex, outputOffset) {
            if (outputLineIndex === 0) {
                return outputOffset;
            }
            else {
                return this._prefixSums.getAccumulatedValue(outputLineIndex - 1) + outputOffset;
            }
        };
        CharacterHardWrappingLineMapping.prototype.getOutputPositionOfInputOffset = function (inputOffset, result) {
            this._prefixSums.getIndexOf(inputOffset, throwawayIndexOfResult);
            result.outputLineIndex = throwawayIndexOfResult.index;
            result.outputOffset = throwawayIndexOfResult.remainder;
        };
        return CharacterHardWrappingLineMapping;
    }());
    exports.CharacterHardWrappingLineMapping = CharacterHardWrappingLineMapping;
});

define("vs/editor/common/viewModel/filteredLineTokens", ["require", "exports", 'vs/editor/common/core/arrays', 'vs/editor/common/editorCommon'], function (require, exports, arrays_1, editorCommon_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FilteredLineTokens = (function () {
        /**
         * [startOffset; endOffset) (i.e. do not include endOffset)
         */
        function FilteredLineTokens(original, startOffset, endOffset, deltaStartIndex) {
            this._original = original;
            this._startOffset = startOffset;
            this._endOffset = endOffset;
            this._deltaStartIndex = deltaStartIndex;
            this.inflatedTokens = editorCommon_1.LineTokensBinaryEncoding.sliceAndInflate(original.getBinaryEncodedTokensMap(), original.getBinaryEncodedTokens(), startOffset, endOffset, deltaStartIndex);
        }
        FilteredLineTokens.prototype.getTokens = function () {
            return this.inflatedTokens;
        };
        FilteredLineTokens.prototype.getFauxIndentLength = function () {
            return this._deltaStartIndex;
        };
        FilteredLineTokens.prototype.getTextLength = function () {
            return this._endOffset - this._startOffset + this._deltaStartIndex;
        };
        FilteredLineTokens.prototype.equals = function (other) {
            if (other instanceof FilteredLineTokens) {
                var otherFilteredLineTokens = other;
                if (this._startOffset !== otherFilteredLineTokens._startOffset) {
                    return false;
                }
                if (this._endOffset !== otherFilteredLineTokens._endOffset) {
                    return false;
                }
                if (this._deltaStartIndex !== otherFilteredLineTokens._deltaStartIndex) {
                    return false;
                }
                return this._original.equals(otherFilteredLineTokens._original);
            }
            return false;
        };
        FilteredLineTokens.prototype.findIndexOfOffset = function (offset) {
            return arrays_1.Arrays.findIndexInSegmentsArray(this.inflatedTokens, offset);
        };
        return FilteredLineTokens;
    }());
    exports.FilteredLineTokens = FilteredLineTokens;
    var IdentityFilteredLineTokens = (function () {
        function IdentityFilteredLineTokens(original, textLength) {
            this._original = original;
            this._textLength = textLength;
        }
        IdentityFilteredLineTokens.prototype.getTokens = function () {
            return editorCommon_1.LineTokensBinaryEncoding.inflateArr(this._original.getBinaryEncodedTokensMap(), this._original.getBinaryEncodedTokens());
        };
        IdentityFilteredLineTokens.prototype.getFauxIndentLength = function () {
            return 0;
        };
        IdentityFilteredLineTokens.prototype.getTextLength = function () {
            return this._textLength;
        };
        IdentityFilteredLineTokens.prototype.equals = function (other) {
            if (other instanceof IdentityFilteredLineTokens) {
                var otherFilteredLineTokens = other;
                return this._original.equals(otherFilteredLineTokens._original);
            }
            return false;
        };
        IdentityFilteredLineTokens.prototype.findIndexOfOffset = function (offset) {
            return this._original.findIndexOfOffset(offset);
        };
        return IdentityFilteredLineTokens;
    }());
    exports.IdentityFilteredLineTokens = IdentityFilteredLineTokens;
});

define("vs/editor/common/viewModel/splitLinesCollection", ["require", "exports", 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/viewModel/filteredLineTokens', 'vs/editor/common/viewModel/prefixSumComputer'], function (require, exports, position_1, range_1, editorCommon, filteredLineTokens_1, prefixSumComputer_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var tmpOutputPosition = {
        outputLineIndex: 0,
        outputOffset: 0
    };
    var IdentitySplitLine = (function () {
        function IdentitySplitLine(isVisible) {
            this._isVisible = isVisible;
        }
        IdentitySplitLine.prototype.isVisible = function () {
            return this._isVisible;
        };
        IdentitySplitLine.prototype.setVisible = function (isVisible) {
            this._isVisible = isVisible;
        };
        IdentitySplitLine.prototype.getOutputLineCount = function () {
            if (!this._isVisible) {
                return 0;
            }
            return 1;
        };
        IdentitySplitLine.prototype.getOutputLineContent = function (model, myLineNumber, outputLineIndex) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            return model.getLineContent(myLineNumber);
        };
        IdentitySplitLine.prototype.getOutputLineMinColumn = function (model, myLineNumber, outputLineIndex) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            return model.getLineMinColumn(myLineNumber);
        };
        IdentitySplitLine.prototype.getOutputLineMaxColumn = function (model, myLineNumber, outputLineIndex) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            return model.getLineMaxColumn(myLineNumber);
        };
        IdentitySplitLine.prototype.getOutputLineTokens = function (model, myLineNumber, outputLineIndex, inaccurateTokensAcceptable) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            return new filteredLineTokens_1.IdentityFilteredLineTokens(model.getLineTokens(myLineNumber, inaccurateTokensAcceptable), model.getLineMaxColumn(myLineNumber) - 1);
        };
        IdentitySplitLine.prototype.getInputColumnOfOutputPosition = function (outputLineIndex, outputColumn) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            return outputColumn;
        };
        IdentitySplitLine.prototype.getOutputPositionOfInputPosition = function (deltaLineNumber, inputColumn) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            return new position_1.Position(deltaLineNumber, inputColumn);
        };
        return IdentitySplitLine;
    }());
    var SplitLine = (function () {
        function SplitLine(positionMapper, isVisible) {
            this.positionMapper = positionMapper;
            this.wrappedIndent = this.positionMapper.getWrappedLinesIndent();
            this.wrappedIndentLength = this.wrappedIndent.length;
            this.outputLineCount = this.positionMapper.getOutputLineCount();
            this._isVisible = isVisible;
        }
        SplitLine.prototype.isVisible = function () {
            return this._isVisible;
        };
        SplitLine.prototype.setVisible = function (isVisible) {
            this._isVisible = isVisible;
        };
        SplitLine.prototype.getOutputLineCount = function () {
            if (!this._isVisible) {
                return 0;
            }
            return this.outputLineCount;
        };
        SplitLine.prototype.getInputStartOffsetOfOutputLineIndex = function (outputLineIndex) {
            return this.positionMapper.getInputOffsetOfOutputPosition(outputLineIndex, 0);
        };
        SplitLine.prototype.getInputEndOffsetOfOutputLineIndex = function (model, myLineNumber, outputLineIndex) {
            if (outputLineIndex + 1 === this.outputLineCount) {
                return model.getLineMaxColumn(myLineNumber) - 1;
            }
            return this.positionMapper.getInputOffsetOfOutputPosition(outputLineIndex + 1, 0);
        };
        SplitLine.prototype.getOutputLineContent = function (model, myLineNumber, outputLineIndex) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            var startOffset = this.getInputStartOffsetOfOutputLineIndex(outputLineIndex);
            var endOffset = this.getInputEndOffsetOfOutputLineIndex(model, myLineNumber, outputLineIndex);
            var r = model.getLineContent(myLineNumber).substring(startOffset, endOffset);
            if (outputLineIndex > 0) {
                r = this.wrappedIndent + r;
            }
            return r;
        };
        SplitLine.prototype.getOutputLineMinColumn = function (model, myLineNumber, outputLineIndex) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            if (outputLineIndex > 0) {
                return this.wrappedIndentLength + 1;
            }
            return 1;
        };
        SplitLine.prototype.getOutputLineMaxColumn = function (model, myLineNumber, outputLineIndex) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            return this.getOutputLineContent(model, myLineNumber, outputLineIndex).length + 1;
        };
        SplitLine.prototype.getOutputLineTokens = function (model, myLineNumber, outputLineIndex, inaccurateTokensAcceptable) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            var startOffset = this.getInputStartOffsetOfOutputLineIndex(outputLineIndex);
            var endOffset = this.getInputEndOffsetOfOutputLineIndex(model, myLineNumber, outputLineIndex);
            var deltaStartIndex = 0;
            if (outputLineIndex > 0) {
                deltaStartIndex = this.wrappedIndentLength;
            }
            return new filteredLineTokens_1.FilteredLineTokens(model.getLineTokens(myLineNumber, inaccurateTokensAcceptable), startOffset, endOffset, deltaStartIndex);
        };
        SplitLine.prototype.getInputColumnOfOutputPosition = function (outputLineIndex, outputColumn) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            var adjustedColumn = outputColumn - 1;
            if (outputLineIndex > 0) {
                if (adjustedColumn < this.wrappedIndentLength) {
                    adjustedColumn = 0;
                }
                else {
                    adjustedColumn -= this.wrappedIndentLength;
                }
            }
            return this.positionMapper.getInputOffsetOfOutputPosition(outputLineIndex, adjustedColumn) + 1;
        };
        SplitLine.prototype.getOutputPositionOfInputPosition = function (deltaLineNumber, inputColumn) {
            if (!this._isVisible) {
                throw new Error('Not supported');
            }
            this.positionMapper.getOutputPositionOfInputOffset(inputColumn - 1, tmpOutputPosition);
            var outputLineIndex = tmpOutputPosition.outputLineIndex;
            var outputColumn = tmpOutputPosition.outputOffset + 1;
            if (outputLineIndex > 0) {
                outputColumn += this.wrappedIndentLength;
            }
            //		console.log('in -> out ' + deltaLineNumber + ',' + inputColumn + ' ===> ' + (deltaLineNumber+outputLineIndex) + ',' + outputColumn);
            return new position_1.Position(deltaLineNumber + outputLineIndex, outputColumn);
        };
        return SplitLine;
    }());
    exports.SplitLine = SplitLine;
    function createSplitLine(linePositionMapperFactory, text, tabSize, wrappingColumn, columnsForFullWidthChar, wrappingIndent, isVisible) {
        var positionMapper = linePositionMapperFactory.createLineMapping(text, tabSize, wrappingColumn, columnsForFullWidthChar, wrappingIndent);
        if (positionMapper === null) {
            // No mapping needed
            return new IdentitySplitLine(isVisible);
        }
        else {
            return new SplitLine(positionMapper, isVisible);
        }
    }
    var SplitLinesCollection = (function () {
        function SplitLinesCollection(model, linePositionMapperFactory, tabSize, wrappingColumn, columnsForFullWidthChar, wrappingIndent) {
            this.model = model;
            this._validModelVersionId = -1;
            this.tabSize = tabSize;
            this.wrappingColumn = wrappingColumn;
            this.columnsForFullWidthChar = columnsForFullWidthChar;
            this.wrappingIndent = wrappingIndent;
            this.linePositionMapperFactory = linePositionMapperFactory;
            this._constructLines(true);
            this.tmpIndexOfResult = {
                index: 0,
                remainder: 0
            };
        }
        SplitLinesCollection.prototype.dispose = function () {
            this.hiddenAreasIds = this.model.deltaDecorations(this.hiddenAreasIds, []);
        };
        SplitLinesCollection.prototype._ensureValidState = function () {
            var modelVersion = this.model.getVersionId();
            if (modelVersion !== this._validModelVersionId) {
                throw new Error('SplitLinesCollection: attempt to access a \'newer\' model');
            }
        };
        SplitLinesCollection.prototype._constructLines = function (resetHiddenAreas) {
            var _this = this;
            this.lines = [];
            if (resetHiddenAreas) {
                this.hiddenAreasIds = [];
            }
            var values = [];
            var linesContent = this.model.getLinesContent();
            var lineCount = linesContent.length;
            var hiddenAreas = this.hiddenAreasIds.map(function (areaId) { return _this.model.getDecorationRange(areaId); }).sort(range_1.Range.compareRangesUsingStarts);
            var hiddenAreaStart = 1, hiddenAreaEnd = 0;
            var hiddenAreaIdx = -1;
            var nextLineNumberToUpdateHiddenArea = (hiddenAreaIdx + 1 < hiddenAreas.length) ? hiddenAreaEnd + 1 : lineCount + 2;
            for (var i = 0; i < lineCount; i++) {
                var lineNumber = i + 1;
                if (lineNumber === nextLineNumberToUpdateHiddenArea) {
                    hiddenAreaIdx++;
                    hiddenAreaStart = hiddenAreas[hiddenAreaIdx].startLineNumber;
                    hiddenAreaEnd = hiddenAreas[hiddenAreaIdx].endLineNumber;
                    nextLineNumberToUpdateHiddenArea = (hiddenAreaIdx + 1 < hiddenAreas.length) ? hiddenAreaEnd + 1 : lineCount + 2;
                }
                var isInHiddenArea = (lineNumber >= hiddenAreaStart && lineNumber <= hiddenAreaEnd);
                var line = createSplitLine(this.linePositionMapperFactory, linesContent[i], this.tabSize, this.wrappingColumn, this.columnsForFullWidthChar, this.wrappingIndent, !isInHiddenArea);
                values[i] = line.getOutputLineCount();
                this.lines[i] = line;
            }
            this._validModelVersionId = this.model.getVersionId();
            this.prefixSumComputer = new prefixSumComputer_1.PrefixSumComputer(values);
        };
        SplitLinesCollection.prototype.getHiddenAreas = function () {
            var _this = this;
            return this.hiddenAreasIds.map(function (decId) {
                return _this.model.getDecorationRange(decId);
            }).sort(range_1.Range.compareRangesUsingStarts);
        };
        SplitLinesCollection.prototype._reduceRanges = function (_ranges) {
            var _this = this;
            if (_ranges.length === 0) {
                return [];
            }
            var ranges = _ranges.map(function (r) { return _this.model.validateRange(r); }).sort(range_1.Range.compareRangesUsingStarts);
            var result = [];
            var currentRangeStart = ranges[0].startLineNumber;
            var currentRangeEnd = ranges[0].endLineNumber;
            for (var i = 1, len = ranges.length; i < len; i++) {
                var range = ranges[i];
                if (range.startLineNumber > currentRangeEnd + 1) {
                    result.push(new range_1.Range(currentRangeStart, 1, currentRangeEnd, 1));
                    currentRangeStart = range.startLineNumber;
                    currentRangeEnd = range.endLineNumber;
                }
                else if (range.endLineNumber > currentRangeEnd) {
                    currentRangeEnd = range.endLineNumber;
                }
            }
            result.push(new range_1.Range(currentRangeStart, 1, currentRangeEnd, 1));
            return result;
        };
        SplitLinesCollection.prototype.setHiddenAreas = function (_ranges, emit) {
            var _this = this;
            var newRanges = this._reduceRanges(_ranges);
            // BEGIN TODO@Martin: Please stop calling this method on each model change!
            var oldRanges = this.hiddenAreasIds.map(function (areaId) { return _this.model.getDecorationRange(areaId); }).sort(range_1.Range.compareRangesUsingStarts);
            if (newRanges.length === oldRanges.length) {
                var hasDifference = false;
                for (var i = 0; i < newRanges.length; i++) {
                    if (!newRanges[i].equalsRange(oldRanges[i])) {
                        hasDifference = true;
                        break;
                    }
                }
                if (!hasDifference) {
                    return false;
                }
            }
            // END TODO@Martin: Please stop calling this method on each model change!
            var newDecorations = [];
            for (var i = 0; i < newRanges.length; i++) {
                newDecorations.push({
                    range: newRanges[i],
                    options: {}
                });
            }
            this.hiddenAreasIds = this.model.deltaDecorations(this.hiddenAreasIds, newDecorations);
            var hiddenAreas = newRanges;
            var hiddenAreaStart = 1, hiddenAreaEnd = 0;
            var hiddenAreaIdx = -1;
            var nextLineNumberToUpdateHiddenArea = (hiddenAreaIdx + 1 < hiddenAreas.length) ? hiddenAreaEnd + 1 : this.lines.length + 2;
            for (var i = 0; i < this.lines.length; i++) {
                var lineNumber = i + 1;
                if (lineNumber === nextLineNumberToUpdateHiddenArea) {
                    hiddenAreaIdx++;
                    hiddenAreaStart = hiddenAreas[hiddenAreaIdx].startLineNumber;
                    hiddenAreaEnd = hiddenAreas[hiddenAreaIdx].endLineNumber;
                    nextLineNumberToUpdateHiddenArea = (hiddenAreaIdx + 1 < hiddenAreas.length) ? hiddenAreaEnd + 1 : this.lines.length + 2;
                }
                var lineChanged = false;
                if (lineNumber >= hiddenAreaStart && lineNumber <= hiddenAreaEnd) {
                    // Line should be hidden
                    if (this.lines[i].isVisible()) {
                        this.lines[i].setVisible(false);
                        lineChanged = true;
                    }
                }
                else {
                    // Line should be visible
                    if (!this.lines[i].isVisible()) {
                        this.lines[i].setVisible(true);
                        lineChanged = true;
                    }
                }
                if (lineChanged) {
                    var newOutputLineCount = this.lines[i].getOutputLineCount();
                    this.prefixSumComputer.changeValue(i, newOutputLineCount);
                }
            }
            emit(editorCommon.ViewEventNames.ModelFlushedEvent, null);
            return true;
        };
        SplitLinesCollection.prototype.inputPositionIsVisible = function (inputLineNumber, inputColumn) {
            if (inputLineNumber < 1 || inputLineNumber > this.lines.length) {
                // invalid arguments
                return false;
            }
            return this.lines[inputLineNumber - 1].isVisible();
        };
        SplitLinesCollection.prototype.setTabSize = function (newTabSize, emit) {
            if (this.tabSize === newTabSize) {
                return false;
            }
            this.tabSize = newTabSize;
            this._constructLines(false);
            emit(editorCommon.ViewEventNames.ModelFlushedEvent, null);
            return true;
        };
        SplitLinesCollection.prototype.setWrappingIndent = function (newWrappingIndent, emit) {
            if (this.wrappingIndent === newWrappingIndent) {
                return false;
            }
            this.wrappingIndent = newWrappingIndent;
            this._constructLines(false);
            emit(editorCommon.ViewEventNames.ModelFlushedEvent, null);
            return true;
        };
        SplitLinesCollection.prototype.setWrappingColumn = function (newWrappingColumn, columnsForFullWidthChar, emit) {
            if (this.wrappingColumn === newWrappingColumn && this.columnsForFullWidthChar === columnsForFullWidthChar) {
                return false;
            }
            this.wrappingColumn = newWrappingColumn;
            this.columnsForFullWidthChar = columnsForFullWidthChar;
            this._constructLines(false);
            emit(editorCommon.ViewEventNames.ModelFlushedEvent, null);
            return true;
        };
        SplitLinesCollection.prototype.onModelFlushed = function (versionId, emit) {
            this._constructLines(true);
            emit(editorCommon.ViewEventNames.ModelFlushedEvent, null);
        };
        SplitLinesCollection.prototype.onModelLinesDeleted = function (versionId, fromLineNumber, toLineNumber, emit) {
            if (versionId <= this._validModelVersionId) {
                return;
            }
            this._validModelVersionId = versionId;
            var outputFromLineNumber = (fromLineNumber === 1 ? 1 : this.prefixSumComputer.getAccumulatedValue(fromLineNumber - 2) + 1);
            var outputToLineNumber = this.prefixSumComputer.getAccumulatedValue(toLineNumber - 1);
            this.lines.splice(fromLineNumber - 1, toLineNumber - fromLineNumber + 1);
            this.prefixSumComputer.removeValues(fromLineNumber - 1, toLineNumber - fromLineNumber + 1);
            var e = {
                fromLineNumber: outputFromLineNumber,
                toLineNumber: outputToLineNumber
            };
            emit(editorCommon.ViewEventNames.LinesDeletedEvent, e);
        };
        SplitLinesCollection.prototype.onModelLinesInserted = function (versionId, fromLineNumber, toLineNumber, text, emit) {
            if (versionId <= this._validModelVersionId) {
                return;
            }
            this._validModelVersionId = versionId;
            var hiddenAreas = this.getHiddenAreas();
            var isInHiddenArea = false;
            var testPosition = new position_1.Position(fromLineNumber, 1);
            for (var i = 0; i < hiddenAreas.length; i++) {
                if (hiddenAreas[i].containsPosition(testPosition)) {
                    isInHiddenArea = true;
                    break;
                }
            }
            var outputFromLineNumber = (fromLineNumber === 1 ? 1 : this.prefixSumComputer.getAccumulatedValue(fromLineNumber - 2) + 1);
            var line, outputLineCount, totalOutputLineCount = 0;
            var insertLines = [], insertPrefixSumValues = [];
            for (var i = 0, len = text.length; i < len; i++) {
                var line = createSplitLine(this.linePositionMapperFactory, text[i], this.tabSize, this.wrappingColumn, this.columnsForFullWidthChar, this.wrappingIndent, !isInHiddenArea);
                insertLines.push(line);
                outputLineCount = line.getOutputLineCount();
                totalOutputLineCount += outputLineCount;
                insertPrefixSumValues.push(outputLineCount);
            }
            this.lines = this.lines.slice(0, fromLineNumber - 1).concat(insertLines).concat(this.lines.slice(fromLineNumber - 1));
            this.prefixSumComputer.insertValues(fromLineNumber - 1, insertPrefixSumValues);
            var e = {
                fromLineNumber: outputFromLineNumber,
                toLineNumber: outputFromLineNumber + totalOutputLineCount - 1
            };
            emit(editorCommon.ViewEventNames.LinesInsertedEvent, e);
        };
        SplitLinesCollection.prototype.onModelLineChanged = function (versionId, lineNumber, newText, emit) {
            if (versionId <= this._validModelVersionId) {
                return;
            }
            this._validModelVersionId = versionId;
            var lineIndex = lineNumber - 1;
            var oldOutputLineCount = this.lines[lineIndex].getOutputLineCount();
            var isVisible = this.lines[lineIndex].isVisible();
            var line = createSplitLine(this.linePositionMapperFactory, newText, this.tabSize, this.wrappingColumn, this.columnsForFullWidthChar, this.wrappingIndent, isVisible);
            this.lines[lineIndex] = line;
            var newOutputLineCount = this.lines[lineIndex].getOutputLineCount();
            var lineMappingChanged = false, changeFrom = 0, changeTo = -1, insertFrom = 0, insertTo = -1, deleteFrom = 0, deleteTo = -1;
            if (oldOutputLineCount > newOutputLineCount) {
                changeFrom = (lineNumber === 1 ? 1 : this.prefixSumComputer.getAccumulatedValue(lineNumber - 2) + 1);
                changeTo = changeFrom + newOutputLineCount - 1;
                deleteFrom = changeTo + 1;
                deleteTo = deleteFrom + (oldOutputLineCount - newOutputLineCount) - 1;
                lineMappingChanged = true;
            }
            else if (oldOutputLineCount < newOutputLineCount) {
                changeFrom = (lineNumber === 1 ? 1 : this.prefixSumComputer.getAccumulatedValue(lineNumber - 2) + 1);
                changeTo = changeFrom + oldOutputLineCount - 1;
                insertFrom = changeTo + 1;
                insertTo = insertFrom + (newOutputLineCount - oldOutputLineCount) - 1;
                lineMappingChanged = true;
            }
            else {
                changeFrom = (lineNumber === 1 ? 1 : this.prefixSumComputer.getAccumulatedValue(lineNumber - 2) + 1);
                changeTo = changeFrom + newOutputLineCount - 1;
            }
            this.prefixSumComputer.changeValue(lineIndex, newOutputLineCount);
            var i, e1, e2, e3;
            if (changeFrom <= changeTo) {
                for (var i = changeFrom; i <= changeTo; i++) {
                    e1 = {
                        lineNumber: i
                    };
                    emit(editorCommon.ViewEventNames.LineChangedEvent, e1);
                }
            }
            if (insertFrom <= insertTo) {
                e2 = {
                    fromLineNumber: insertFrom,
                    toLineNumber: insertTo
                };
                emit(editorCommon.ViewEventNames.LinesInsertedEvent, e2);
            }
            if (deleteFrom <= deleteTo) {
                e3 = {
                    fromLineNumber: deleteFrom,
                    toLineNumber: deleteTo
                };
                emit(editorCommon.ViewEventNames.LinesDeletedEvent, e3);
            }
            return lineMappingChanged;
        };
        SplitLinesCollection.prototype.getOutputLineCount = function () {
            this._ensureValidState();
            return this.prefixSumComputer.getTotalValue();
        };
        SplitLinesCollection.prototype._toValidOutputLineNumber = function (outputLineNumber) {
            if (outputLineNumber < 1) {
                return 1;
            }
            var outputLineCount = this.getOutputLineCount();
            if (outputLineNumber > outputLineCount) {
                return outputLineCount;
            }
            return outputLineNumber;
        };
        SplitLinesCollection.prototype.getOutputLineContent = function (outputLineNumber) {
            this._ensureValidState();
            outputLineNumber = this._toValidOutputLineNumber(outputLineNumber);
            this.prefixSumComputer.getIndexOf(outputLineNumber - 1, this.tmpIndexOfResult);
            var lineIndex = this.tmpIndexOfResult.index;
            var remainder = this.tmpIndexOfResult.remainder;
            return this.lines[lineIndex].getOutputLineContent(this.model, lineIndex + 1, remainder);
        };
        SplitLinesCollection.prototype.getOutputLineMinColumn = function (outputLineNumber) {
            this._ensureValidState();
            outputLineNumber = this._toValidOutputLineNumber(outputLineNumber);
            this.prefixSumComputer.getIndexOf(outputLineNumber - 1, this.tmpIndexOfResult);
            var lineIndex = this.tmpIndexOfResult.index;
            var remainder = this.tmpIndexOfResult.remainder;
            return this.lines[lineIndex].getOutputLineMinColumn(this.model, lineIndex + 1, remainder);
        };
        SplitLinesCollection.prototype.getOutputLineMaxColumn = function (outputLineNumber) {
            this._ensureValidState();
            outputLineNumber = this._toValidOutputLineNumber(outputLineNumber);
            this.prefixSumComputer.getIndexOf(outputLineNumber - 1, this.tmpIndexOfResult);
            var lineIndex = this.tmpIndexOfResult.index;
            var remainder = this.tmpIndexOfResult.remainder;
            return this.lines[lineIndex].getOutputLineMaxColumn(this.model, lineIndex + 1, remainder);
        };
        SplitLinesCollection.prototype.getOutputLineTokens = function (outputLineNumber, inaccurateTokensAcceptable) {
            this._ensureValidState();
            outputLineNumber = this._toValidOutputLineNumber(outputLineNumber);
            this.prefixSumComputer.getIndexOf(outputLineNumber - 1, this.tmpIndexOfResult);
            var lineIndex = this.tmpIndexOfResult.index;
            var remainder = this.tmpIndexOfResult.remainder;
            return this.lines[lineIndex].getOutputLineTokens(this.model, lineIndex + 1, remainder, inaccurateTokensAcceptable);
        };
        SplitLinesCollection.prototype.convertOutputPositionToInputPosition = function (viewLineNumber, viewColumn) {
            this._ensureValidState();
            viewLineNumber = this._toValidOutputLineNumber(viewLineNumber);
            this.prefixSumComputer.getIndexOf(viewLineNumber - 1, this.tmpIndexOfResult);
            var lineIndex = this.tmpIndexOfResult.index;
            var remainder = this.tmpIndexOfResult.remainder;
            var inputColumn = this.lines[lineIndex].getInputColumnOfOutputPosition(remainder, viewColumn);
            // console.log('out -> in ' + viewLineNumber + ',' + viewColumn + ' ===> ' + (lineIndex+1) + ',' + inputColumn);
            return this.model.validatePosition(new position_1.Position(lineIndex + 1, inputColumn));
        };
        SplitLinesCollection.prototype.convertInputPositionToOutputPosition = function (_inputLineNumber, _inputColumn) {
            this._ensureValidState();
            var validPosition = this.model.validatePosition(new position_1.Position(_inputLineNumber, _inputColumn));
            var inputLineNumber = validPosition.lineNumber;
            var inputColumn = validPosition.column;
            var lineIndex = inputLineNumber - 1, lineIndexChanged = false;
            while (lineIndex > 0 && !this.lines[lineIndex].isVisible()) {
                lineIndex--;
                lineIndexChanged = true;
            }
            if (lineIndex === 0 && !this.lines[lineIndex].isVisible()) {
                // Could not reach a real line
                // console.log('in -> out ' + inputLineNumber + ',' + inputColumn + ' ===> ' + 1 + ',' + 1);
                return new position_1.Position(1, 1);
            }
            var deltaLineNumber = 1 + (lineIndex === 0 ? 0 : this.prefixSumComputer.getAccumulatedValue(lineIndex - 1));
            var r;
            if (lineIndexChanged) {
                r = this.lines[lineIndex].getOutputPositionOfInputPosition(deltaLineNumber, this.model.getLineMaxColumn(lineIndex + 1));
            }
            else {
                r = this.lines[inputLineNumber - 1].getOutputPositionOfInputPosition(deltaLineNumber, inputColumn);
            }
            // console.log('in -> out ' + inputLineNumber + ',' + inputColumn + ' ===> ' + r.lineNumber + ',' + r);
            return r;
        };
        return SplitLinesCollection;
    }());
    exports.SplitLinesCollection = SplitLinesCollection;
});

define("vs/editor/common/viewModel/viewEventHandler", ["require", "exports", 'vs/editor/common/editorCommon'], function (require, exports, editorCommon) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewEventHandler = (function () {
        function ViewEventHandler() {
            this.shouldRender = true;
        }
        // --- begin event handlers
        ViewEventHandler.prototype.onLineMappingChanged = function () {
            return false;
        };
        ViewEventHandler.prototype.onModelFlushed = function () {
            return false;
        };
        ViewEventHandler.prototype.onModelDecorationsChanged = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onModelLinesDeleted = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onModelLineChanged = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onModelLinesInserted = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onModelTokensChanged = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onCursorPositionChanged = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onCursorSelectionChanged = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onCursorRevealRange = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onCursorScrollRequest = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onConfigurationChanged = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onLayoutChanged = function (layoutInfo) {
            return false;
        };
        ViewEventHandler.prototype.onScrollChanged = function (e) {
            return false;
        };
        ViewEventHandler.prototype.onZonesChanged = function () {
            return false;
        };
        ViewEventHandler.prototype.onScrollWidthChanged = function (scrollWidth) {
            return false;
        };
        ViewEventHandler.prototype.onScrollHeightChanged = function (scrollHeight) {
            return false;
        };
        ViewEventHandler.prototype.onViewFocusChanged = function (isFocused) {
            return false;
        };
        // --- end event handlers
        ViewEventHandler.prototype.handleEvents = function (events) {
            var i, len, e, data;
            for (i = 0, len = events.length; i < len; i++) {
                e = events[i];
                data = e.getData();
                switch (e.getType()) {
                    case editorCommon.ViewEventNames.LineMappingChangedEvent:
                        this.shouldRender = this.onLineMappingChanged() || this.shouldRender;
                        break;
                    case editorCommon.ViewEventNames.ModelFlushedEvent:
                        this.shouldRender = this.onModelFlushed() || this.shouldRender;
                        break;
                    case editorCommon.ViewEventNames.LinesDeletedEvent:
                        this.shouldRender = this.onModelLinesDeleted(data) || this.shouldRender;
                        break;
                    case editorCommon.ViewEventNames.LinesInsertedEvent:
                        this.shouldRender = this.onModelLinesInserted(data) || this.shouldRender;
                        break;
                    case editorCommon.ViewEventNames.LineChangedEvent:
                        this.shouldRender = this.onModelLineChanged(data) || this.shouldRender;
                        break;
                    case editorCommon.ViewEventNames.TokensChangedEvent:
                        this.shouldRender = this.onModelTokensChanged(data) || this.shouldRender;
                        break;
                    case editorCommon.ViewEventNames.DecorationsChangedEvent:
                        this.shouldRender = this.onModelDecorationsChanged(data) || this.shouldRender;
                        break;
                    case editorCommon.ViewEventNames.CursorPositionChangedEvent:
                        this.shouldRender = this.onCursorPositionChanged(data) || this.shouldRender;
                        break;
                    case editorCommon.ViewEventNames.CursorSelectionChangedEvent:
                        this.shouldRender = this.onCursorSelectionChanged(data) || this.shouldRender;
                        break;
                    case editorCommon.ViewEventNames.RevealRangeEvent:
                        this.shouldRender = this.onCursorRevealRange(data) || this.shouldRender;
                        break;
                    case editorCommon.ViewEventNames.ScrollRequestEvent:
                        this.shouldRender = this.onCursorScrollRequest(data) || this.shouldRender;
                        break;
                    case editorCommon.EventType.ConfigurationChanged:
                        this.shouldRender = this.onConfigurationChanged(data) || this.shouldRender;
                        break;
                    case editorCommon.EventType.ViewLayoutChanged:
                        this.shouldRender = this.onLayoutChanged(data) || this.shouldRender;
                        break;
                    case editorCommon.EventType.ViewScrollChanged:
                        this.shouldRender = this.onScrollChanged(data) || this.shouldRender;
                        break;
                    case editorCommon.EventType.ViewZonesChanged:
                        this.shouldRender = this.onZonesChanged() || this.shouldRender;
                        break;
                    case editorCommon.EventType.ViewScrollWidthChanged:
                        this.shouldRender = this.onScrollWidthChanged(data) || this.shouldRender;
                        break;
                    case editorCommon.EventType.ViewScrollHeightChanged:
                        this.shouldRender = this.onScrollHeightChanged(data) || this.shouldRender;
                        break;
                    case editorCommon.EventType.ViewFocusChanged:
                        this.shouldRender = this.onViewFocusChanged(data) || this.shouldRender;
                        break;
                    default:
                        console.info('View received unknown event: ');
                        console.info(e);
                }
            }
        };
        return ViewEventHandler;
    }());
    exports.ViewEventHandler = ViewEventHandler;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/view/viewPart", ["require", "exports", 'vs/editor/common/viewModel/viewEventHandler'], function (require, exports, viewEventHandler_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewPart = (function (_super) {
        __extends(ViewPart, _super);
        function ViewPart(context) {
            _super.call(this);
            this._context = context;
            this._context.addEventHandler(this);
            this._modificationBeforeRenderingRunners = [];
            this._modificationRunners = [];
        }
        ViewPart.prototype.dispose = function () {
            this._context.removeEventHandler(this);
            this._context = null;
            this._modificationBeforeRenderingRunners = [];
            this._modificationRunners = [];
        };
        /**
         * Modify the DOM right before when the orchestrated rendering occurs.
         */
        ViewPart.prototype._requestModificationFrameBeforeRendering = function (runner) {
            this._modificationBeforeRenderingRunners.push(runner);
        };
        /**
         * Modify the DOM when the orchestrated rendering occurs.
         */
        ViewPart.prototype._requestModificationFrame = function (runner) {
            this._modificationRunners.push(runner);
        };
        ViewPart.prototype.onBeforeForcedLayout = function () {
            if (this._modificationBeforeRenderingRunners.length > 0) {
                for (var i = 0; i < this._modificationBeforeRenderingRunners.length; i++) {
                    this._modificationBeforeRenderingRunners[i]();
                }
                this._modificationBeforeRenderingRunners = [];
            }
        };
        ViewPart.prototype.onReadAfterForcedLayout = function (ctx) {
            if (!this.shouldRender) {
                return;
            }
            this._render(ctx);
        };
        ViewPart.prototype.onWriteAfterForcedLayout = function () {
            if (!this.shouldRender) {
                return;
            }
            this.shouldRender = false;
            this._executeModificationRunners();
        };
        ViewPart.prototype._executeModificationRunners = function () {
            if (this._modificationRunners.length > 0) {
                for (var i = 0; i < this._modificationRunners.length; i++) {
                    this._modificationRunners[i]();
                }
                this._modificationRunners = [];
            }
        };
        ViewPart.prototype._render = function (ctx) {
            throw new Error('Implement me!');
        };
        return ViewPart;
    }(viewEventHandler_1.ViewEventHandler));
    exports.ViewPart = ViewPart;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/currentLineHighlight/currentLineHighlight", ["require", "exports", 'vs/editor/common/viewModel/viewEventHandler', 'vs/css!./currentLineHighlight'], function (require, exports, viewEventHandler_1) {
    'use strict';
    var CurrentLineHighlightOverlay = (function (_super) {
        __extends(CurrentLineHighlightOverlay, _super);
        function CurrentLineHighlightOverlay(context, layoutProvider) {
            _super.call(this);
            this._context = context;
            this._layoutProvider = layoutProvider;
            this._selectionIsEmpty = true;
            this._primaryCursorIsInEditableRange = true;
            this._primaryCursorLineNumber = 1;
            this._scrollWidth = this._layoutProvider.getScrollWidth();
            this._context.addEventHandler(this);
        }
        CurrentLineHighlightOverlay.prototype.dispose = function () {
            this._context.removeEventHandler(this);
            this._context = null;
        };
        // --- begin event handlers
        CurrentLineHighlightOverlay.prototype.onModelFlushed = function () {
            this._primaryCursorIsInEditableRange = true;
            this._selectionIsEmpty = true;
            this._primaryCursorLineNumber = 1;
            this._scrollWidth = this._layoutProvider.getScrollWidth();
            return true;
        };
        CurrentLineHighlightOverlay.prototype.onModelLinesDeleted = function (e) {
            return true;
        };
        CurrentLineHighlightOverlay.prototype.onModelLinesInserted = function (e) {
            return true;
        };
        CurrentLineHighlightOverlay.prototype.onCursorPositionChanged = function (e) {
            var hasChanged = false;
            if (this._primaryCursorIsInEditableRange !== e.isInEditableRange) {
                this._primaryCursorIsInEditableRange = e.isInEditableRange;
                hasChanged = true;
            }
            if (this._primaryCursorLineNumber !== e.position.lineNumber) {
                this._primaryCursorLineNumber = e.position.lineNumber;
                hasChanged = true;
            }
            return hasChanged;
        };
        CurrentLineHighlightOverlay.prototype.onCursorSelectionChanged = function (e) {
            var isEmpty = e.selection.isEmpty();
            if (this._selectionIsEmpty !== isEmpty) {
                this._selectionIsEmpty = isEmpty;
                return true;
            }
            return false;
        };
        CurrentLineHighlightOverlay.prototype.onConfigurationChanged = function (e) {
            return true;
        };
        CurrentLineHighlightOverlay.prototype.onLayoutChanged = function (layoutInfo) {
            return true;
        };
        CurrentLineHighlightOverlay.prototype.onScrollChanged = function (e) {
            return true;
        };
        CurrentLineHighlightOverlay.prototype.onZonesChanged = function () {
            return true;
        };
        CurrentLineHighlightOverlay.prototype.onScrollWidthChanged = function (scrollWidth) {
            if (this._scrollWidth !== scrollWidth) {
                this._scrollWidth = scrollWidth;
                return true;
            }
            return false;
        };
        // --- end event handlers
        CurrentLineHighlightOverlay.prototype.shouldCallRender2 = function (ctx) {
            if (!this.shouldRender) {
                return false;
            }
            this.shouldRender = false;
            this._scrollWidth = ctx.scrollWidth;
            return true;
        };
        CurrentLineHighlightOverlay.prototype.render2 = function (lineNumber) {
            if (lineNumber === this._primaryCursorLineNumber) {
                if (this._shouldShowCurrentLine()) {
                    return [
                        '<div class="current-line" style="width:',
                        String(this._scrollWidth),
                        'px; height:',
                        String(this._context.configuration.editor.lineHeight),
                        'px;"></div>'
                    ];
                }
                else {
                    return null;
                }
            }
            return null;
        };
        CurrentLineHighlightOverlay.prototype._shouldShowCurrentLine = function () {
            return this._selectionIsEmpty && this._primaryCursorIsInEditableRange && !this._context.configuration.editor.readOnly;
        };
        return CurrentLineHighlightOverlay;
    }(viewEventHandler_1.ViewEventHandler));
    exports.CurrentLineHighlightOverlay = CurrentLineHighlightOverlay;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/decorations/decorations", ["require", "exports", 'vs/editor/common/viewModel/viewEventHandler', 'vs/css!./decorations'], function (require, exports, viewEventHandler_1) {
    'use strict';
    var DecorationsOverlay = (function (_super) {
        __extends(DecorationsOverlay, _super);
        function DecorationsOverlay(context) {
            _super.call(this);
            this._context = context;
            this._renderResult = null;
            this._context.addEventHandler(this);
        }
        DecorationsOverlay.prototype.dispose = function () {
            this._context.removeEventHandler(this);
            this._context = null;
            this._renderResult = null;
        };
        // --- begin event handlers
        DecorationsOverlay.prototype.onModelFlushed = function () {
            return true;
        };
        DecorationsOverlay.prototype.onModelDecorationsChanged = function (e) {
            return true;
        };
        DecorationsOverlay.prototype.onModelLinesDeleted = function (e) {
            return true;
        };
        DecorationsOverlay.prototype.onModelLineChanged = function (e) {
            return true;
        };
        DecorationsOverlay.prototype.onModelLinesInserted = function (e) {
            return true;
        };
        DecorationsOverlay.prototype.onCursorPositionChanged = function (e) {
            return false;
        };
        DecorationsOverlay.prototype.onCursorSelectionChanged = function (e) {
            return false;
        };
        DecorationsOverlay.prototype.onCursorRevealRange = function (e) {
            return false;
        };
        DecorationsOverlay.prototype.onConfigurationChanged = function (e) {
            return true;
        };
        DecorationsOverlay.prototype.onLayoutChanged = function (layoutInfo) {
            return true;
        };
        DecorationsOverlay.prototype.onScrollChanged = function (e) {
            return e.vertical;
        };
        DecorationsOverlay.prototype.onZonesChanged = function () {
            return true;
        };
        DecorationsOverlay.prototype.onScrollWidthChanged = function (scrollWidth) {
            return true;
        };
        DecorationsOverlay.prototype.onScrollHeightChanged = function (scrollHeight) {
            return false;
        };
        // --- end event handlers
        DecorationsOverlay.prototype.shouldCallRender2 = function (ctx) {
            if (!this.shouldRender) {
                return false;
            }
            this.shouldRender = false;
            var decorations = ctx.getDecorationsInViewport();
            // Keep only decorations with `className`
            decorations = decorations.filter(function (d) { return !!d.options.className; });
            // Sort decorations for consistent render output
            decorations = decorations.sort(function (a, b) {
                if (a.options.className < b.options.className) {
                    return -1;
                }
                if (a.options.className > b.options.className) {
                    return 1;
                }
                if (a.range.startLineNumber === b.range.startLineNumber) {
                    if (a.range.startColumn === b.range.startColumn) {
                        if (a.range.endLineNumber === b.range.endLineNumber) {
                            return a.range.endColumn - b.range.endColumn;
                        }
                        return a.range.endLineNumber - b.range.endLineNumber;
                    }
                    return a.range.startColumn - b.range.startColumn;
                }
                return a.range.startLineNumber - b.range.startLineNumber;
            });
            // Render first whole line decorations and then regular decorations
            var output = {};
            this._renderWholeLineDecorations(ctx, decorations, output);
            this._renderNormalDecorations(ctx, decorations, output);
            this._renderResult = output;
            return true;
        };
        DecorationsOverlay.prototype._renderWholeLineDecorations = function (ctx, decorations, output) {
            var lineHeight = String(this._context.configuration.editor.lineHeight);
            for (var i = 0, lenI = decorations.length; i < lenI; i++) {
                var d = decorations[i];
                if (!d.options.isWholeLine) {
                    continue;
                }
                var decorationOutput = [
                    '<div class="cdr ',
                    d.options.className,
                    '" style="left:0;width:100%;height:',
                    lineHeight,
                    'px;"></div>'
                ].join('');
                var startLineNumber = d.range.startLineNumber;
                var endLineNumber = d.range.endLineNumber;
                for (var j = startLineNumber; j <= endLineNumber; j++) {
                    if (!ctx.lineIsVisible(j)) {
                        continue;
                    }
                    var strLineNumber = String(j);
                    if (output.hasOwnProperty(strLineNumber)) {
                        output[strLineNumber].push(decorationOutput);
                    }
                    else {
                        output[strLineNumber] = [decorationOutput];
                    }
                }
            }
        };
        DecorationsOverlay.prototype._renderNormalDecorations = function (ctx, decorations, output) {
            var lineHeight = String(this._context.configuration.editor.lineHeight);
            for (var i = 0, lenI = decorations.length; i < lenI; i++) {
                var d = decorations[i];
                if (d.options.isWholeLine) {
                    continue;
                }
                var linesVisibleRanges = ctx.linesVisibleRangesForRange(d.range, false);
                if (!linesVisibleRanges) {
                    continue;
                }
                var className = d.options.className;
                for (var j = 0, lenJ = linesVisibleRanges.length; j < lenJ; j++) {
                    var lineVisibleRanges = linesVisibleRanges[j];
                    var strLineNumber = String(lineVisibleRanges.lineNumber);
                    var lineOutput = void 0;
                    if (output.hasOwnProperty(strLineNumber)) {
                        lineOutput = output[strLineNumber];
                    }
                    else {
                        lineOutput = [];
                        output[strLineNumber] = lineOutput;
                    }
                    for (var k = 0, lenK = lineVisibleRanges.ranges.length; k < lenK; k++) {
                        var visibleRange = lineVisibleRanges.ranges[k];
                        lineOutput.push('<div class="cdr ');
                        lineOutput.push(className);
                        lineOutput.push('" style="left:');
                        lineOutput.push(String(visibleRange.left));
                        lineOutput.push('px;width:');
                        lineOutput.push(String(visibleRange.width));
                        lineOutput.push('px;height:');
                        lineOutput.push(lineHeight);
                        lineOutput.push('px;"></div>');
                    }
                }
            }
        };
        DecorationsOverlay.prototype.render2 = function (lineNumber) {
            if (this._renderResult && this._renderResult.hasOwnProperty(lineNumber.toString())) {
                return this._renderResult[lineNumber.toString()];
            }
            return null;
        };
        return DecorationsOverlay;
    }(viewEventHandler_1.ViewEventHandler));
    exports.DecorationsOverlay = DecorationsOverlay;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/glyphMargin/glyphMargin", ["require", "exports", 'vs/editor/common/viewModel/viewEventHandler', 'vs/css!./glyphMargin'], function (require, exports, viewEventHandler_1) {
    'use strict';
    var GlyphMarginOverlay = (function (_super) {
        __extends(GlyphMarginOverlay, _super);
        function GlyphMarginOverlay(context) {
            _super.call(this);
            this._context = context;
            this._glyphMarginLeft = 0;
            this._glyphMarginWidth = 0;
            this._renderResult = null;
            this._context.addEventHandler(this);
        }
        GlyphMarginOverlay.prototype.dispose = function () {
            this._context.removeEventHandler(this);
            this._context = null;
            this._renderResult = null;
        };
        // --- begin event handlers
        GlyphMarginOverlay.prototype.onModelFlushed = function () {
            return true;
        };
        GlyphMarginOverlay.prototype.onModelDecorationsChanged = function (e) {
            return true;
        };
        GlyphMarginOverlay.prototype.onModelLinesDeleted = function (e) {
            return true;
        };
        GlyphMarginOverlay.prototype.onModelLineChanged = function (e) {
            return true;
        };
        GlyphMarginOverlay.prototype.onModelLinesInserted = function (e) {
            return true;
        };
        GlyphMarginOverlay.prototype.onCursorPositionChanged = function (e) {
            return false;
        };
        GlyphMarginOverlay.prototype.onCursorSelectionChanged = function (e) {
            return false;
        };
        GlyphMarginOverlay.prototype.onCursorRevealRange = function (e) {
            return false;
        };
        GlyphMarginOverlay.prototype.onConfigurationChanged = function (e) {
            return true;
        };
        GlyphMarginOverlay.prototype.onLayoutChanged = function (layoutInfo) {
            this._glyphMarginLeft = layoutInfo.glyphMarginLeft;
            this._glyphMarginWidth = layoutInfo.glyphMarginWidth;
            return true;
        };
        GlyphMarginOverlay.prototype.onScrollChanged = function (e) {
            return e.vertical;
        };
        GlyphMarginOverlay.prototype.onZonesChanged = function () {
            return true;
        };
        GlyphMarginOverlay.prototype.onScrollWidthChanged = function (scrollWidth) {
            return false;
        };
        GlyphMarginOverlay.prototype.onScrollHeightChanged = function (scrollHeight) {
            return false;
        };
        // --- end event handlers
        GlyphMarginOverlay.prototype.shouldCallRender2 = function (ctx) {
            if (!this.shouldRender) {
                return false;
            }
            this.shouldRender = false;
            if (!this._context.configuration.editor.glyphMargin) {
                this._renderResult = null;
                return false;
            }
            var output = {};
            var count = 0;
            var decorations = ctx.getDecorationsInViewport(), lineHeight = this._context.configuration.editor.lineHeight.toString(), d, rng, i, lenI, classNames = {}, lineClassNames, className, lineOutput, lineNumber, lineNumberStr;
            for (i = 0, lenI = decorations.length; i < lenI; i++) {
                d = decorations[i];
                if (!d.options.glyphMarginClassName) {
                    continue;
                }
                rng = d.range;
                for (lineNumber = rng.startLineNumber; lineNumber <= rng.endLineNumber; lineNumber++) {
                    if (!ctx.lineIsVisible(lineNumber)) {
                        continue;
                    }
                    lineNumberStr = lineNumber.toString();
                    if (!classNames.hasOwnProperty(lineNumberStr)) {
                        classNames[lineNumberStr] = {};
                    }
                    classNames[lineNumberStr][d.options.glyphMarginClassName] = true;
                }
            }
            var left = this._glyphMarginLeft.toString(), width = this._glyphMarginWidth.toString();
            var common = '" style="left:' + left + 'px;width:' + width + 'px' + ';height:' + lineHeight + 'px;"></div>';
            for (lineNumberStr in classNames) {
                lineClassNames = classNames[lineNumberStr];
                lineOutput = [];
                lineOutput.push('<div class="cgmr');
                for (className in lineClassNames) {
                    // Count one more glyph
                    count++;
                    lineOutput.push(' ');
                    lineOutput.push(className);
                }
                lineOutput.push(common);
                output[lineNumberStr] = lineOutput;
            }
            this._renderResult = output;
            return true;
        };
        GlyphMarginOverlay.prototype.render2 = function (lineNumber) {
            if (this._renderResult && this._renderResult.hasOwnProperty(lineNumber.toString())) {
                return this._renderResult[lineNumber.toString()];
            }
            return null;
        };
        return GlyphMarginOverlay;
    }(viewEventHandler_1.ViewEventHandler));
    exports.GlyphMarginOverlay = GlyphMarginOverlay;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/lineNumbers/lineNumbers", ["require", "exports", 'vs/base/common/platform', 'vs/editor/common/viewModel/viewEventHandler', 'vs/editor/browser/editorBrowser', 'vs/css!./lineNumbers'], function (require, exports, platform, viewEventHandler_1, editorBrowser_1) {
    'use strict';
    var LineNumbersOverlay = (function (_super) {
        __extends(LineNumbersOverlay, _super);
        function LineNumbersOverlay(context) {
            _super.call(this);
            this._context = context;
            this._lineNumbersLeft = 0;
            this._lineNumbersWidth = 0;
            this._renderResult = null;
            this._context.addEventHandler(this);
        }
        LineNumbersOverlay.prototype.dispose = function () {
            this._context.removeEventHandler(this);
            this._context = null;
            this._renderResult = null;
        };
        // --- begin event handlers
        LineNumbersOverlay.prototype.onModelFlushed = function () {
            return true;
        };
        LineNumbersOverlay.prototype.onModelDecorationsChanged = function (e) {
            return false;
        };
        LineNumbersOverlay.prototype.onModelLinesDeleted = function (e) {
            return true;
        };
        LineNumbersOverlay.prototype.onModelLineChanged = function (e) {
            return true;
        };
        LineNumbersOverlay.prototype.onModelLinesInserted = function (e) {
            return true;
        };
        LineNumbersOverlay.prototype.onCursorPositionChanged = function (e) {
            return false;
        };
        LineNumbersOverlay.prototype.onCursorSelectionChanged = function (e) {
            return false;
        };
        LineNumbersOverlay.prototype.onCursorRevealRange = function (e) {
            return false;
        };
        LineNumbersOverlay.prototype.onConfigurationChanged = function (e) {
            return true;
        };
        LineNumbersOverlay.prototype.onLayoutChanged = function (layoutInfo) {
            this._lineNumbersLeft = layoutInfo.lineNumbersLeft;
            this._lineNumbersWidth = layoutInfo.lineNumbersWidth;
            return true;
        };
        LineNumbersOverlay.prototype.onScrollChanged = function (e) {
            return e.vertical;
        };
        LineNumbersOverlay.prototype.onZonesChanged = function () {
            return true;
        };
        LineNumbersOverlay.prototype.onScrollWidthChanged = function (scrollWidth) {
            return false;
        };
        LineNumbersOverlay.prototype.onScrollHeightChanged = function (scrollHeight) {
            return false;
        };
        // --- end event handlers
        LineNumbersOverlay.prototype.shouldCallRender2 = function (ctx) {
            if (!this.shouldRender) {
                return false;
            }
            this.shouldRender = false;
            if (!this._context.configuration.editor.lineNumbers) {
                this._renderResult = null;
                return false;
            }
            var output = {};
            var lineHeightClassName = (platform.isLinux ? (this._context.configuration.editor.lineHeight % 2 === 0 ? ' lh-even' : ' lh-odd') : '');
            var lineHeight = this._context.configuration.editor.lineHeight.toString(), lineNumber, renderLineNumber;
            var common = '<div class="' + editorBrowser_1.ClassNames.LINE_NUMBERS + lineHeightClassName + '" style="left:' + this._lineNumbersLeft.toString() + 'px;width:' + this._lineNumbersWidth.toString() + 'px;height:' + lineHeight + 'px;">';
            for (lineNumber = ctx.visibleRange.startLineNumber; lineNumber <= ctx.visibleRange.endLineNumber; lineNumber++) {
                renderLineNumber = this._context.model.getLineRenderLineNumber(lineNumber);
                if (renderLineNumber) {
                    var lineOutput = [
                        common,
                        this._context.model.getLineRenderLineNumber(lineNumber),
                        '</div>'
                    ];
                    output[lineNumber.toString()] = lineOutput;
                }
            }
            this._renderResult = output;
            return true;
        };
        LineNumbersOverlay.prototype.render2 = function (lineNumber) {
            if (this._renderResult && this._renderResult.hasOwnProperty(lineNumber.toString())) {
                return this._renderResult[lineNumber.toString()];
            }
            return null;
        };
        return LineNumbersOverlay;
    }(viewEventHandler_1.ViewEventHandler));
    exports.LineNumbersOverlay = LineNumbersOverlay;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/linesDecorations/linesDecorations", ["require", "exports", 'vs/editor/common/viewModel/viewEventHandler', 'vs/css!./linesDecorations'], function (require, exports, viewEventHandler_1) {
    'use strict';
    var LinesDecorationsOverlay = (function (_super) {
        __extends(LinesDecorationsOverlay, _super);
        function LinesDecorationsOverlay(context) {
            _super.call(this);
            this._context = context;
            this._decorationsLeft = 0;
            this._decorationsWidth = 0;
            this._renderResult = null;
            this._context.addEventHandler(this);
        }
        LinesDecorationsOverlay.prototype.dispose = function () {
            this._context.removeEventHandler(this);
            this._context = null;
            this._renderResult = null;
        };
        // --- begin event handlers
        LinesDecorationsOverlay.prototype.onModelFlushed = function () {
            return true;
        };
        LinesDecorationsOverlay.prototype.onModelDecorationsChanged = function (e) {
            return true;
        };
        LinesDecorationsOverlay.prototype.onModelLinesDeleted = function (e) {
            return true;
        };
        LinesDecorationsOverlay.prototype.onModelLineChanged = function (e) {
            return true;
        };
        LinesDecorationsOverlay.prototype.onModelLinesInserted = function (e) {
            return true;
        };
        LinesDecorationsOverlay.prototype.onCursorPositionChanged = function (e) {
            return false;
        };
        LinesDecorationsOverlay.prototype.onCursorSelectionChanged = function (e) {
            return false;
        };
        LinesDecorationsOverlay.prototype.onCursorRevealRange = function (e) {
            return false;
        };
        LinesDecorationsOverlay.prototype.onConfigurationChanged = function (e) {
            return true;
        };
        LinesDecorationsOverlay.prototype.onLayoutChanged = function (layoutInfo) {
            this._decorationsLeft = layoutInfo.decorationsLeft;
            this._decorationsWidth = layoutInfo.decorationsWidth;
            return true;
        };
        LinesDecorationsOverlay.prototype.onScrollChanged = function (e) {
            return e.vertical;
        };
        LinesDecorationsOverlay.prototype.onZonesChanged = function () {
            return true;
        };
        LinesDecorationsOverlay.prototype.onScrollWidthChanged = function (scrollWidth) {
            return false;
        };
        LinesDecorationsOverlay.prototype.onScrollHeightChanged = function (scrollHeight) {
            return false;
        };
        // --- end event handlers
        LinesDecorationsOverlay.prototype.shouldCallRender2 = function (ctx) {
            if (!this.shouldRender) {
                return false;
            }
            this.shouldRender = false;
            var output = {};
            var renderedCount = 0;
            var decorations = ctx.getDecorationsInViewport(), lineHeight = this._context.configuration.editor.lineHeight.toString(), d, rng, i, lenI, classNames = {}, lineClassNames, className, lineOutput, lineNumber, lineNumberStr;
            for (i = 0, lenI = decorations.length; i < lenI; i++) {
                d = decorations[i];
                if (!d.options.linesDecorationsClassName) {
                    continue;
                }
                rng = d.range;
                for (lineNumber = rng.startLineNumber; lineNumber <= rng.endLineNumber; lineNumber++) {
                    if (!ctx.lineIsVisible(lineNumber)) {
                        continue;
                    }
                    lineNumberStr = lineNumber.toString();
                    //					oldTop = ctx.getViewportVerticalOffsetForLineNumber(j);
                    if (!classNames.hasOwnProperty(lineNumberStr)) {
                        classNames[lineNumberStr] = {};
                    }
                    classNames[lineNumberStr][d.options.linesDecorationsClassName] = true;
                }
            }
            var left = this._decorationsLeft.toString(), width = this._decorationsWidth.toString();
            var common = '" style="left:' + left + 'px;width:' + width + 'px' + ';height:' + lineHeight + 'px;"></div>';
            for (lineNumberStr in classNames) {
                lineClassNames = classNames[lineNumberStr];
                lineOutput = [];
                lineOutput.push('<div class="cldr');
                for (className in lineClassNames) {
                    // Count one more glyph
                    renderedCount++;
                    lineOutput.push(' ');
                    lineOutput.push(className);
                }
                lineOutput.push(common);
                output[lineNumberStr] = lineOutput;
            }
            this._renderResult = output;
            return true;
        };
        LinesDecorationsOverlay.prototype.render2 = function (lineNumber) {
            if (this._renderResult && this._renderResult.hasOwnProperty(lineNumber.toString())) {
                return this._renderResult[lineNumber.toString()];
            }
            return null;
        };
        return LinesDecorationsOverlay;
    }(viewEventHandler_1.ViewEventHandler));
    exports.LinesDecorationsOverlay = LinesDecorationsOverlay;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/overlayWidgets/overlayWidgets", ["require", "exports", 'vs/base/browser/styleMutator', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/view/viewPart', 'vs/css!./overlayWidgets'], function (require, exports, styleMutator_1, editorBrowser_1, viewPart_1) {
    'use strict';
    var ViewOverlayWidgets = (function (_super) {
        __extends(ViewOverlayWidgets, _super);
        function ViewOverlayWidgets(context) {
            _super.call(this, context);
            this._widgets = {};
            this._verticalScrollbarWidth = 0;
            this._horizontalScrollbarHeight = 0;
            this._editorHeight = 0;
            this.domNode = document.createElement('div');
            this.domNode.className = editorBrowser_1.ClassNames.OVERLAY_WIDGETS;
        }
        ViewOverlayWidgets.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this._widgets = null;
        };
        // ---- begin view event handlers
        ViewOverlayWidgets.prototype.onLayoutChanged = function (layoutInfo) {
            var _this = this;
            this._verticalScrollbarWidth = layoutInfo.verticalScrollbarWidth;
            this._horizontalScrollbarHeight = layoutInfo.horizontalScrollbarHeight;
            this._editorHeight = layoutInfo.height;
            this._requestModificationFrame(function () {
                styleMutator_1.StyleMutator.setWidth(_this.domNode, layoutInfo.width);
            });
            return true;
        };
        // ---- end view event handlers
        ViewOverlayWidgets.prototype.addWidget = function (widget) {
            this._widgets[widget.getId()] = {
                widget: widget,
                preference: null
            };
            // This is sync because a widget wants to be in the dom
            var domNode = widget.getDomNode();
            domNode.style.position = 'absolute';
            domNode.setAttribute('widgetId', widget.getId());
            this.domNode.appendChild(domNode);
        };
        ViewOverlayWidgets.prototype.setWidgetPosition = function (widget, preference) {
            var _this = this;
            var widgetData = this._widgets[widget.getId()];
            if (widgetData.preference === preference) {
                return false;
            }
            widgetData.preference = preference;
            this._requestModificationFrame(function () {
                if (_this._widgets.hasOwnProperty(widget.getId())) {
                    _this._renderWidget(widgetData);
                }
            });
            return true;
        };
        ViewOverlayWidgets.prototype.removeWidget = function (widget) {
            var widgetId = widget.getId();
            if (this._widgets.hasOwnProperty(widgetId)) {
                var widgetData = this._widgets[widgetId];
                var domNode = widgetData.widget.getDomNode();
                delete this._widgets[widgetId];
                domNode.parentNode.removeChild(domNode);
            }
        };
        ViewOverlayWidgets.prototype._renderWidget = function (widgetData) {
            var _RESTORE_STYLE_TOP = 'data-editor-restoreStyleTop', domNode = widgetData.widget.getDomNode();
            if (widgetData.preference === null) {
                if (domNode.hasAttribute(_RESTORE_STYLE_TOP)) {
                    var previousTop = domNode.getAttribute(_RESTORE_STYLE_TOP);
                    domNode.removeAttribute(_RESTORE_STYLE_TOP);
                    domNode.style.top = previousTop;
                }
                return;
            }
            if (widgetData.preference === editorBrowser_1.OverlayWidgetPositionPreference.TOP_RIGHT_CORNER) {
                if (!domNode.hasAttribute(_RESTORE_STYLE_TOP)) {
                    domNode.setAttribute(_RESTORE_STYLE_TOP, domNode.style.top);
                }
                styleMutator_1.StyleMutator.setTop(domNode, 0);
                styleMutator_1.StyleMutator.setRight(domNode, (2 * this._verticalScrollbarWidth));
            }
            else if (widgetData.preference === editorBrowser_1.OverlayWidgetPositionPreference.BOTTOM_RIGHT_CORNER) {
                if (!domNode.hasAttribute(_RESTORE_STYLE_TOP)) {
                    domNode.setAttribute(_RESTORE_STYLE_TOP, domNode.style.top);
                }
                var widgetHeight = domNode.clientHeight;
                styleMutator_1.StyleMutator.setTop(domNode, (this._editorHeight - widgetHeight - 2 * this._horizontalScrollbarHeight));
                styleMutator_1.StyleMutator.setRight(domNode, (2 * this._verticalScrollbarWidth));
            }
            else if (widgetData.preference === editorBrowser_1.OverlayWidgetPositionPreference.TOP_CENTER) {
                if (!domNode.hasAttribute(_RESTORE_STYLE_TOP)) {
                    domNode.setAttribute(_RESTORE_STYLE_TOP, domNode.style.top);
                }
                styleMutator_1.StyleMutator.setTop(domNode, 0);
                domNode.style.right = '50%';
            }
        };
        ViewOverlayWidgets.prototype._render = function (ctx) {
            var _this = this;
            var widgetId;
            this._requestModificationFrame(function () {
                for (widgetId in _this._widgets) {
                    if (_this._widgets.hasOwnProperty(widgetId)) {
                        _this._renderWidget(_this._widgets[widgetId]);
                    }
                }
            });
        };
        ViewOverlayWidgets.prototype.onReadAfterForcedLayout = function (ctx) {
            // Overwriting to bypass `shouldRender` flag
            this._render(ctx);
            return null;
        };
        ViewOverlayWidgets.prototype.onWriteAfterForcedLayout = function () {
            // Overwriting to bypass `shouldRender` flag
            this._executeModificationRunners();
        };
        return ViewOverlayWidgets;
    }(viewPart_1.ViewPart));
    exports.ViewOverlayWidgets = ViewOverlayWidgets;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/overviewRuler/overviewRuler", ["require", "exports", 'vs/editor/common/viewModel/viewEventHandler', 'vs/editor/browser/viewParts/overviewRuler/overviewRulerImpl'], function (require, exports, viewEventHandler_1, overviewRulerImpl_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var OverviewRuler = (function (_super) {
        __extends(OverviewRuler, _super);
        function OverviewRuler(context, cssClassName, scrollHeight, minimumHeight, maximumHeight, getVerticalOffsetForLine) {
            _super.call(this);
            this._context = context;
            this._overviewRuler = new overviewRulerImpl_1.OverviewRulerImpl(0, cssClassName, scrollHeight, this._context.configuration.editor.lineHeight, minimumHeight, maximumHeight, getVerticalOffsetForLine);
            this._context.addEventHandler(this);
        }
        OverviewRuler.prototype.destroy = function () {
            this.dispose();
        };
        OverviewRuler.prototype.dispose = function () {
            this._context.removeEventHandler(this);
            this._overviewRuler.dispose();
        };
        OverviewRuler.prototype.onConfigurationChanged = function (e) {
            if (e.lineHeight) {
                this._overviewRuler.setLineHeight(this._context.configuration.editor.lineHeight, true);
                return true;
            }
            return false;
        };
        OverviewRuler.prototype.onZonesChanged = function () {
            return true;
        };
        OverviewRuler.prototype.onModelFlushed = function () {
            return true;
        };
        OverviewRuler.prototype.onScrollHeightChanged = function (scrollHeight) {
            this._overviewRuler.setScrollHeight(scrollHeight, true);
            return true;
        };
        OverviewRuler.prototype.getDomNode = function () {
            return this._overviewRuler.getDomNode();
        };
        OverviewRuler.prototype.setLayout = function (position) {
            this._overviewRuler.setLayout(position, true);
        };
        OverviewRuler.prototype.setZones = function (zones) {
            this._overviewRuler.setZones(zones, true);
        };
        return OverviewRuler;
    }(viewEventHandler_1.ViewEventHandler));
    exports.OverviewRuler = OverviewRuler;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/rulers/rulers", ["require", "exports", 'vs/base/browser/styleMutator', 'vs/editor/browser/view/viewPart', 'vs/css!./rulers'], function (require, exports, styleMutator_1, viewPart_1) {
    'use strict';
    var Rulers = (function (_super) {
        __extends(Rulers, _super);
        function Rulers(context, layoutProvider) {
            _super.call(this, context);
            this._layoutProvider = layoutProvider;
            this.domNode = document.createElement('div');
            this.domNode.className = 'view-rulers';
            this._rulers = this._context.configuration.editor.rulers;
            this._height = this._context.configuration.editor.layoutInfo.contentHeight;
            this._typicalHalfwidthCharacterWidth = this._context.configuration.editor.typicalHalfwidthCharacterWidth;
        }
        Rulers.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
        };
        // --- begin event handlers
        Rulers.prototype.onConfigurationChanged = function (e) {
            if (e.rulers || e.layoutInfo || e.typicalHalfwidthCharacterWidth) {
                this._rulers = this._context.configuration.editor.rulers;
                this._height = this._context.configuration.editor.layoutInfo.contentHeight;
                this._typicalHalfwidthCharacterWidth = this._context.configuration.editor.typicalHalfwidthCharacterWidth;
                return true;
            }
            return false;
        };
        Rulers.prototype.onScrollHeightChanged = function (scrollHeight) {
            return true;
        };
        // --- end event handlers
        Rulers.prototype._render = function (ctx) {
            var _this = this;
            this._requestModificationFrame(function () {
                var existingRulersLength = _this.domNode.children.length;
                var max = Math.max(existingRulersLength, _this._rulers.length);
                for (var i = 0; i < max; i++) {
                    if (i >= _this._rulers.length) {
                        _this.domNode.removeChild(_this.domNode.lastChild);
                        continue;
                    }
                    var node = void 0;
                    if (i < existingRulersLength) {
                        node = _this.domNode.children[i];
                    }
                    else {
                        node = document.createElement('div');
                        node.className = 'view-ruler';
                        _this.domNode.appendChild(node);
                    }
                    styleMutator_1.StyleMutator.setHeight(node, Math.min(_this._layoutProvider.getTotalHeight(), 1000000));
                    styleMutator_1.StyleMutator.setLeft(node, _this._rulers[i] * _this._typicalHalfwidthCharacterWidth);
                }
            });
        };
        return Rulers;
    }(viewPart_1.ViewPart));
    exports.Rulers = Rulers;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/selections/selections", ["require", "exports", 'vs/editor/common/viewModel/viewEventHandler', 'vs/css!./selections'], function (require, exports, viewEventHandler_1) {
    'use strict';
    var CornerStyle;
    (function (CornerStyle) {
        CornerStyle[CornerStyle["EXTERN"] = 0] = "EXTERN";
        CornerStyle[CornerStyle["INTERN"] = 1] = "INTERN";
        CornerStyle[CornerStyle["FLAT"] = 2] = "FLAT";
    })(CornerStyle || (CornerStyle = {}));
    var HorizontalRangeWithStyle = (function () {
        function HorizontalRangeWithStyle(other) {
            this.left = other.left;
            this.width = other.width;
            this.startStyle = null;
            this.endStyle = null;
        }
        return HorizontalRangeWithStyle;
    }());
    var LineVisibleRangesWithStyle = (function () {
        function LineVisibleRangesWithStyle(lineNumber, ranges) {
            this.lineNumber = lineNumber;
            this.ranges = ranges;
        }
        return LineVisibleRangesWithStyle;
    }());
    function toStyledRange(item) {
        return new HorizontalRangeWithStyle(item);
    }
    function toStyled(item) {
        return new LineVisibleRangesWithStyle(item.lineNumber, item.ranges.map(toStyledRange));
    }
    // TODO@Alex: Remove this once IE11 fixes Bug #524217
    // The problem in IE11 is that it does some sort of auto-zooming to accomodate for displays with different pixel density.
    // Unfortunately, this auto-zooming is buggy around dealing with rounded borders
    var isIEWithZoomingIssuesNearRoundedBorders = ((navigator.userAgent.indexOf('Trident/7.0') >= 0)
        || (navigator.userAgent.indexOf('Edge/12') >= 0));
    var SelectionsOverlay = (function (_super) {
        __extends(SelectionsOverlay, _super);
        function SelectionsOverlay(context) {
            _super.call(this);
            this._previousFrameVisibleRangesWithStyle = [];
            this._context = context;
            this._selections = [];
            this._renderResult = null;
            this._context.addEventHandler(this);
        }
        SelectionsOverlay.prototype.dispose = function () {
            this._context.removeEventHandler(this);
            this._context = null;
            this._selections = null;
            this._renderResult = null;
        };
        // --- begin event handlers
        SelectionsOverlay.prototype.onModelFlushed = function () {
            return true;
        };
        SelectionsOverlay.prototype.onModelDecorationsChanged = function (e) {
            // true for inline decorations that can end up relayouting text
            return e.inlineDecorationsChanged;
        };
        SelectionsOverlay.prototype.onModelLinesDeleted = function (e) {
            return true;
        };
        SelectionsOverlay.prototype.onModelLineChanged = function (e) {
            return true;
        };
        SelectionsOverlay.prototype.onModelLinesInserted = function (e) {
            return true;
        };
        SelectionsOverlay.prototype.onCursorPositionChanged = function (e) {
            return false;
        };
        SelectionsOverlay.prototype.onCursorSelectionChanged = function (e) {
            this._selections = [e.selection];
            this._selections = this._selections.concat(e.secondarySelections);
            return true;
        };
        SelectionsOverlay.prototype.onCursorRevealRange = function (e) {
            return false;
        };
        SelectionsOverlay.prototype.onConfigurationChanged = function (e) {
            return true;
        };
        SelectionsOverlay.prototype.onLayoutChanged = function (layoutInfo) {
            return true;
        };
        SelectionsOverlay.prototype.onScrollChanged = function (e) {
            return e.vertical;
        };
        SelectionsOverlay.prototype.onZonesChanged = function () {
            return true;
        };
        SelectionsOverlay.prototype.onScrollWidthChanged = function (scrollWidth) {
            return false;
        };
        SelectionsOverlay.prototype.onScrollHeightChanged = function (scrollHeight) {
            return false;
        };
        // --- end event handlers
        SelectionsOverlay.prototype._visibleRangesHaveGaps = function (linesVisibleRanges) {
            var i, len, lineVisibleRanges;
            for (i = 0, len = linesVisibleRanges.length; i < len; i++) {
                lineVisibleRanges = linesVisibleRanges[i];
                if (lineVisibleRanges.ranges.length > 1) {
                    // There are two ranges on the same line
                    return true;
                }
            }
            return false;
        };
        SelectionsOverlay.prototype._enrichVisibleRangesWithStyle = function (linesVisibleRanges, previousFrame) {
            var curLineRange, curLeft, curRight, prevLeft, prevRight, nextLeft, nextRight, startStyle, endStyle, i, len;
            var previousFrameTop = null, previousFrameBottom = null;
            if (previousFrame && previousFrame.length > 0 && linesVisibleRanges.length > 0) {
                var topLineNumber = linesVisibleRanges[0].lineNumber;
                for (var i = 0; !previousFrameTop && i < previousFrame.length; i++) {
                    if (previousFrame[i].lineNumber === topLineNumber) {
                        previousFrameTop = previousFrame[i].ranges[0];
                    }
                }
                var bottomLineNumber = linesVisibleRanges[linesVisibleRanges.length - 1].lineNumber;
                for (var i = previousFrame.length - 1; !previousFrameBottom && i >= 0; i--) {
                    if (previousFrame[i].lineNumber === bottomLineNumber) {
                        previousFrameBottom = previousFrame[i].ranges[0];
                    }
                }
                if (previousFrameTop && !previousFrameTop.startStyle) {
                    previousFrameTop = null;
                }
                if (previousFrameBottom && !previousFrameBottom.startStyle) {
                    previousFrameBottom = null;
                }
            }
            for (i = 0, len = linesVisibleRanges.length; i < len; i++) {
                // We know for a fact that there is precisely one range on each line
                curLineRange = linesVisibleRanges[i].ranges[0];
                curLeft = curLineRange.left;
                curRight = curLineRange.left + curLineRange.width;
                startStyle = {
                    top: CornerStyle.EXTERN,
                    bottom: CornerStyle.EXTERN
                };
                endStyle = {
                    top: CornerStyle.EXTERN,
                    bottom: CornerStyle.EXTERN
                };
                if (i > 0) {
                    // Look above
                    prevLeft = linesVisibleRanges[i - 1].ranges[0].left;
                    prevRight = linesVisibleRanges[i - 1].ranges[0].left + linesVisibleRanges[i - 1].ranges[0].width;
                    if (curLeft === prevLeft) {
                        startStyle.top = CornerStyle.FLAT;
                    }
                    else if (curLeft > prevLeft) {
                        startStyle.top = CornerStyle.INTERN;
                    }
                    if (curRight === prevRight) {
                        endStyle.top = CornerStyle.FLAT;
                    }
                    else if (prevLeft < curRight && curRight < prevRight) {
                        endStyle.top = CornerStyle.INTERN;
                    }
                }
                else if (previousFrameTop) {
                    // Accept some hick-ups near the viewport edges to save on repaints
                    startStyle.top = previousFrameTop.startStyle.top;
                    endStyle.top = previousFrameTop.endStyle.top;
                }
                if (i + 1 < len) {
                    // Look below
                    nextLeft = linesVisibleRanges[i + 1].ranges[0].left;
                    nextRight = linesVisibleRanges[i + 1].ranges[0].left + linesVisibleRanges[i + 1].ranges[0].width;
                    if (curLeft === nextLeft) {
                        startStyle.bottom = CornerStyle.FLAT;
                    }
                    else if (nextLeft < curLeft && curLeft < nextRight) {
                        startStyle.bottom = CornerStyle.INTERN;
                    }
                    if (curRight === nextRight) {
                        endStyle.bottom = CornerStyle.FLAT;
                    }
                    else if (curRight < nextRight) {
                        endStyle.bottom = CornerStyle.INTERN;
                    }
                }
                else if (previousFrameBottom) {
                    // Accept some hick-ups near the viewport edges to save on repaints
                    startStyle.bottom = previousFrameBottom.startStyle.bottom;
                    endStyle.bottom = previousFrameBottom.endStyle.bottom;
                }
                curLineRange.startStyle = startStyle;
                curLineRange.endStyle = endStyle;
            }
        };
        SelectionsOverlay.prototype._getVisibleRangesWithStyle = function (selection, ctx, previousFrame) {
            var _linesVisibleRanges = ctx.linesVisibleRangesForRange(selection, true) || [];
            var linesVisibleRanges = _linesVisibleRanges.map(toStyled);
            var visibleRangesHaveGaps = this._visibleRangesHaveGaps(linesVisibleRanges);
            if (!isIEWithZoomingIssuesNearRoundedBorders && !visibleRangesHaveGaps && this._context.configuration.editor.roundedSelection) {
                this._enrichVisibleRangesWithStyle(linesVisibleRanges, previousFrame);
            }
            // The visible ranges are sorted TOP-BOTTOM and LEFT-RIGHT
            return linesVisibleRanges;
        };
        SelectionsOverlay.prototype._createSelectionPiece = function (lineOutput, top, height, className, left, width) {
            lineOutput.push('<div class="cslr ');
            lineOutput.push(className);
            lineOutput.push('" style="top:');
            lineOutput.push(top.toString());
            lineOutput.push('px;left:');
            lineOutput.push(left.toString());
            lineOutput.push('px;width:');
            lineOutput.push(width.toString());
            lineOutput.push('px;height:');
            lineOutput.push(height);
            lineOutput.push('px;"></div>');
        };
        SelectionsOverlay.prototype._actualRenderOneSelection = function (output, hasMultipleSelections, visibleRanges) {
            var visibleRangesHaveStyle = (visibleRanges.length > 0 && visibleRanges[0].ranges[0].startStyle), lineVisibleRanges, lineOutput, className, fullLineHeight = (this._context.configuration.editor.lineHeight).toString(), reducedLineHeight = (this._context.configuration.editor.lineHeight - 1).toString(), i, len, j, lenJ, piecesCount = 0, visibleRange;
            var firstLineNumber = (visibleRanges.length > 0 ? visibleRanges[0].lineNumber : 0);
            var lastLineNumber = (visibleRanges.length > 0 ? visibleRanges[visibleRanges.length - 1].lineNumber : 0);
            for (i = 0, len = visibleRanges.length; i < len; i++) {
                lineVisibleRanges = visibleRanges[i];
                var lineNumber = lineVisibleRanges.lineNumber;
                var lineHeight = hasMultipleSelections ? (lineNumber === lastLineNumber || lineNumber === firstLineNumber ? reducedLineHeight : fullLineHeight) : fullLineHeight;
                var top_1 = hasMultipleSelections ? (lineNumber === firstLineNumber ? 1 : 0) : 0;
                if (output.hasOwnProperty(lineNumber.toString())) {
                    lineOutput = output[lineNumber.toString()];
                }
                else {
                    lineOutput = [];
                    output[lineNumber.toString()] = lineOutput;
                }
                for (j = 0, lenJ = lineVisibleRanges.ranges.length; j < lenJ; j++) {
                    visibleRange = lineVisibleRanges.ranges[j];
                    if (visibleRangesHaveStyle) {
                        if (visibleRange.startStyle.top === CornerStyle.INTERN || visibleRange.startStyle.bottom === CornerStyle.INTERN) {
                            // Reverse rounded corner to the left
                            // First comes the selection (blue layer)
                            piecesCount++;
                            this._createSelectionPiece(lineOutput, top_1, lineHeight, SelectionsOverlay.SELECTION_CLASS_NAME, visibleRange.left - SelectionsOverlay.ROUNDED_PIECE_WIDTH, SelectionsOverlay.ROUNDED_PIECE_WIDTH);
                            // Second comes the background (white layer) with inverse border radius
                            className = SelectionsOverlay.EDITOR_BACKGROUND_CLASS_NAME;
                            if (visibleRange.startStyle.top === CornerStyle.INTERN) {
                                className += ' ' + SelectionsOverlay.SELECTION_TOP_RIGHT;
                            }
                            if (visibleRange.startStyle.bottom === CornerStyle.INTERN) {
                                className += ' ' + SelectionsOverlay.SELECTION_BOTTOM_RIGHT;
                            }
                            piecesCount++;
                            this._createSelectionPiece(lineOutput, top_1, lineHeight, className, visibleRange.left - SelectionsOverlay.ROUNDED_PIECE_WIDTH, SelectionsOverlay.ROUNDED_PIECE_WIDTH);
                        }
                        if (visibleRange.endStyle.top === CornerStyle.INTERN || visibleRange.endStyle.bottom === CornerStyle.INTERN) {
                            // Reverse rounded corner to the right
                            // First comes the selection (blue layer)
                            piecesCount++;
                            this._createSelectionPiece(lineOutput, top_1, lineHeight, SelectionsOverlay.SELECTION_CLASS_NAME, visibleRange.left + visibleRange.width, SelectionsOverlay.ROUNDED_PIECE_WIDTH);
                            // Second comes the background (white layer) with inverse border radius
                            className = SelectionsOverlay.EDITOR_BACKGROUND_CLASS_NAME;
                            if (visibleRange.endStyle.top === CornerStyle.INTERN) {
                                className += ' ' + SelectionsOverlay.SELECTION_TOP_LEFT;
                            }
                            if (visibleRange.endStyle.bottom === CornerStyle.INTERN) {
                                className += ' ' + SelectionsOverlay.SELECTION_BOTTOM_LEFT;
                            }
                            piecesCount++;
                            this._createSelectionPiece(lineOutput, top_1, lineHeight, className, visibleRange.left + visibleRange.width, SelectionsOverlay.ROUNDED_PIECE_WIDTH);
                        }
                    }
                    className = SelectionsOverlay.SELECTION_CLASS_NAME;
                    if (visibleRangesHaveStyle) {
                        if (visibleRange.startStyle.top === CornerStyle.EXTERN) {
                            className += ' ' + SelectionsOverlay.SELECTION_TOP_LEFT;
                        }
                        if (visibleRange.startStyle.bottom === CornerStyle.EXTERN) {
                            className += ' ' + SelectionsOverlay.SELECTION_BOTTOM_LEFT;
                        }
                        if (visibleRange.endStyle.top === CornerStyle.EXTERN) {
                            className += ' ' + SelectionsOverlay.SELECTION_TOP_RIGHT;
                        }
                        if (visibleRange.endStyle.bottom === CornerStyle.EXTERN) {
                            className += ' ' + SelectionsOverlay.SELECTION_BOTTOM_RIGHT;
                        }
                    }
                    piecesCount++;
                    this._createSelectionPiece(lineOutput, top_1, lineHeight, className, visibleRange.left, visibleRange.width);
                }
            }
            return piecesCount;
        };
        SelectionsOverlay.prototype.shouldCallRender2 = function (ctx) {
            if (!this.shouldRender) {
                return false;
            }
            this.shouldRender = false;
            var output = {}, selection, visibleRangesWithStyle, piecesCount = 0, i, thisFrameVisibleRangesWithStyle = [];
            for (i = 0; i < this._selections.length; i++) {
                selection = this._selections[i];
                if (selection.isEmpty()) {
                    thisFrameVisibleRangesWithStyle.push(null);
                    continue;
                }
                visibleRangesWithStyle = this._getVisibleRangesWithStyle(selection, ctx, this._previousFrameVisibleRangesWithStyle[i]);
                thisFrameVisibleRangesWithStyle.push(visibleRangesWithStyle);
                piecesCount += this._actualRenderOneSelection(output, this._selections.length > 1, visibleRangesWithStyle);
            }
            this._previousFrameVisibleRangesWithStyle = thisFrameVisibleRangesWithStyle;
            this._renderResult = output;
            return true;
        };
        SelectionsOverlay.prototype.render2 = function (lineNumber) {
            if (this._renderResult && this._renderResult.hasOwnProperty(lineNumber.toString())) {
                return this._renderResult[lineNumber.toString()];
            }
            return null;
        };
        SelectionsOverlay.SELECTION_CLASS_NAME = 'selected-text';
        SelectionsOverlay.SELECTION_TOP_LEFT = 'top-left-radius';
        SelectionsOverlay.SELECTION_BOTTOM_LEFT = 'bottom-left-radius';
        SelectionsOverlay.SELECTION_TOP_RIGHT = 'top-right-radius';
        SelectionsOverlay.SELECTION_BOTTOM_RIGHT = 'bottom-right-radius';
        SelectionsOverlay.EDITOR_BACKGROUND_CLASS_NAME = 'monaco-editor-background';
        SelectionsOverlay.ROUNDED_PIECE_WIDTH = 10;
        return SelectionsOverlay;
    }(viewEventHandler_1.ViewEventHandler));
    exports.SelectionsOverlay = SelectionsOverlay;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/viewCursors/viewCursors", ["require", "exports", 'vs/base/browser/browser', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/view/viewPart', 'vs/editor/browser/viewParts/viewCursors/viewCursor', 'vs/css!./viewCursors'], function (require, exports, browser, editorCommon, editorBrowser_1, viewPart_1, viewCursor_1) {
    'use strict';
    var RenderType;
    (function (RenderType) {
        RenderType[RenderType["Hidden"] = 0] = "Hidden";
        RenderType[RenderType["Visible"] = 1] = "Visible";
        RenderType[RenderType["Blink"] = 2] = "Blink";
    })(RenderType || (RenderType = {}));
    var ViewCursors = (function (_super) {
        __extends(ViewCursors, _super);
        function ViewCursors(context) {
            _super.call(this, context);
            this._primaryCursor = new viewCursor_1.ViewCursor(this._context, false);
            this._secondaryCursors = [];
            this._domNode = document.createElement('div');
            this._updateDomClassName();
            if (browser.canUseTranslate3d) {
                this._domNode.style.transform = 'translate3d(0px, 0px, 0px)';
            }
            this._domNode.appendChild(this._primaryCursor.getDomNode());
            this._blinkTimer = -1;
            this._editorHasFocus = false;
            this._updateBlinking();
        }
        ViewCursors.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            if (this._blinkTimer !== -1) {
                window.clearInterval(this._blinkTimer);
                this._blinkTimer = -1;
            }
        };
        ViewCursors.prototype.getDomNode = function () {
            return this._domNode;
        };
        // --- begin event handlers
        ViewCursors.prototype.onModelFlushed = function () {
            this._primaryCursor.onModelFlushed();
            for (var i = 0, len = this._secondaryCursors.length; i < len; i++) {
                var domNode = this._secondaryCursors[i].getDomNode();
                domNode.parentNode.removeChild(domNode);
            }
            this._secondaryCursors = [];
            return true;
        };
        ViewCursors.prototype.onModelDecorationsChanged = function (e) {
            // true for inline decorations that can end up relayouting text
            return e.inlineDecorationsChanged;
        };
        ViewCursors.prototype.onModelLinesDeleted = function (e) {
            return true;
        };
        ViewCursors.prototype.onModelLineChanged = function (e) {
            return true;
        };
        ViewCursors.prototype.onModelLinesInserted = function (e) {
            return true;
        };
        ViewCursors.prototype.onModelTokensChanged = function (e) {
            var shouldRender = function (position) {
                return e.fromLineNumber <= position.lineNumber && position.lineNumber <= e.toLineNumber;
            };
            if (shouldRender(this._primaryCursor.getPosition())) {
                return true;
            }
            for (var i = 0; i < this._secondaryCursors.length; i++) {
                if (shouldRender(this._secondaryCursors[i].getPosition())) {
                    return true;
                }
            }
            return false;
        };
        ViewCursors.prototype.onCursorPositionChanged = function (e) {
            this._primaryCursor.onCursorPositionChanged(e.position, e.isInEditableRange);
            this._updateBlinking();
            if (this._secondaryCursors.length < e.secondaryPositions.length) {
                // Create new cursors
                var addCnt = e.secondaryPositions.length - this._secondaryCursors.length;
                for (var i = 0; i < addCnt; i++) {
                    var newCursor = new viewCursor_1.ViewCursor(this._context, true);
                    this._primaryCursor.getDomNode().parentNode.insertBefore(newCursor.getDomNode(), this._primaryCursor.getDomNode().nextSibling);
                    this._secondaryCursors.push(newCursor);
                }
            }
            else if (this._secondaryCursors.length > e.secondaryPositions.length) {
                // Remove some cursors
                var removeCnt = this._secondaryCursors.length - e.secondaryPositions.length;
                for (var i = 0; i < removeCnt; i++) {
                    this._secondaryCursors[0].getDomNode().parentNode.removeChild(this._secondaryCursors[0].getDomNode());
                    this._secondaryCursors.splice(0, 1);
                }
            }
            for (var i = 0; i < e.secondaryPositions.length; i++) {
                this._secondaryCursors[i].onCursorPositionChanged(e.secondaryPositions[i], e.isInEditableRange);
            }
            return true;
        };
        ViewCursors.prototype.onCursorSelectionChanged = function (e) {
            return false;
        };
        ViewCursors.prototype.onConfigurationChanged = function (e) {
            this._primaryCursor.onConfigurationChanged(e);
            this._updateBlinking();
            if (e.cursorStyle) {
                this._updateDomClassName();
            }
            for (var i = 0, len = this._secondaryCursors.length; i < len; i++) {
                this._secondaryCursors[i].onConfigurationChanged(e);
            }
            return true;
        };
        ViewCursors.prototype.onLayoutChanged = function (layoutInfo) {
            return true;
        };
        ViewCursors.prototype.onScrollChanged = function (e) {
            return true;
        };
        ViewCursors.prototype.onZonesChanged = function () {
            return true;
        };
        ViewCursors.prototype.onScrollWidthChanged = function (scrollWidth) {
            return true;
        };
        ViewCursors.prototype.onScrollHeightChanged = function (scrollHeight) {
            return false;
        };
        ViewCursors.prototype.onViewFocusChanged = function (isFocused) {
            this._editorHasFocus = isFocused;
            this._updateBlinking();
            return false;
        };
        // --- end event handlers
        ViewCursors.prototype.getPosition = function () {
            return this._primaryCursor.getPosition();
        };
        // ---- blinking logic
        ViewCursors.prototype._getRenderType = function () {
            if (this._editorHasFocus) {
                if (this._primaryCursor.getIsInEditableRange() && !this._context.configuration.editor.readOnly) {
                    switch (this._context.configuration.editor.cursorBlinking) {
                        case 'blink':
                            return RenderType.Blink;
                        case 'visible':
                            return RenderType.Visible;
                        case 'hidden':
                            return RenderType.Hidden;
                        default:
                            return RenderType.Blink;
                    }
                }
                return RenderType.Visible;
            }
            return RenderType.Hidden;
        };
        ViewCursors.prototype._updateBlinking = function () {
            var _this = this;
            if (this._blinkTimer !== -1) {
                window.clearInterval(this._blinkTimer);
                this._blinkTimer = -1;
            }
            var renderType = this._getRenderType();
            if (renderType === RenderType.Visible || renderType === RenderType.Blink) {
                this._show();
            }
            else {
                this._hide();
            }
            if (renderType === RenderType.Blink) {
                this._blinkTimer = window.setInterval(function () { return _this._blink(); }, ViewCursors.BLINK_INTERVAL);
            }
        };
        // --- end blinking logic
        ViewCursors.prototype._updateDomClassName = function () {
            this._domNode.className = this._getClassName();
        };
        ViewCursors.prototype._getClassName = function () {
            var result = editorBrowser_1.ClassNames.VIEW_CURSORS_LAYER;
            var extraClassName;
            switch (this._context.configuration.editor.cursorStyle) {
                case editorCommon.TextEditorCursorStyle.Line:
                    extraClassName = 'cursor-line-style';
                    break;
                case editorCommon.TextEditorCursorStyle.Block:
                    extraClassName = 'cursor-block-style';
                    break;
                case editorCommon.TextEditorCursorStyle.Underline:
                    extraClassName = 'cursor-underline-style';
                    break;
                default:
                    extraClassName = 'cursor-line-style';
            }
            return result + ' ' + extraClassName;
        };
        ViewCursors.prototype._blink = function () {
            if (this._isVisible) {
                this._hide();
            }
            else {
                this._show();
            }
        };
        ViewCursors.prototype._show = function () {
            this._primaryCursor.show();
            for (var i = 0, len = this._secondaryCursors.length; i < len; i++) {
                this._secondaryCursors[i].show();
            }
            this._isVisible = true;
        };
        ViewCursors.prototype._hide = function () {
            this._primaryCursor.hide();
            for (var i = 0, len = this._secondaryCursors.length; i < len; i++) {
                this._secondaryCursors[i].hide();
            }
            this._isVisible = false;
        };
        // ---- IViewPart implementation
        ViewCursors.prototype._render = function (ctx) {
            var _this = this;
            this._primaryCursor.prepareRender(ctx);
            for (var i = 0, len = this._secondaryCursors.length; i < len; i++) {
                this._secondaryCursors[i].prepareRender(ctx);
            }
            this._requestModificationFrame(function () {
                _this._primaryCursor.render(ctx);
                for (var i = 0, len = _this._secondaryCursors.length; i < len; i++) {
                    _this._secondaryCursors[i].render(ctx);
                }
            });
        };
        ViewCursors.BLINK_INTERVAL = 500;
        return ViewCursors;
    }(viewPart_1.ViewPart));
    exports.ViewCursors = ViewCursors;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/viewZones/viewZones", ["require", "exports", 'vs/base/common/errors', 'vs/base/browser/styleMutator', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/view/viewPart'], function (require, exports, errors_1, styleMutator_1, editorBrowser_1, viewPart_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewZones = (function (_super) {
        __extends(ViewZones, _super);
        function ViewZones(context, whitespaceManager) {
            _super.call(this, context);
            this._whitespaceManager = whitespaceManager;
            this.domNode = document.createElement('div');
            this.domNode.className = editorBrowser_1.ClassNames.VIEW_ZONES;
            this.domNode.style.position = 'absolute';
            this.domNode.setAttribute('role', 'presentation');
            this.domNode.setAttribute('aria-hidden', 'true');
            this._zones = {};
        }
        ViewZones.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this._whitespaceManager = null;
            this._zones = {};
        };
        // ---- begin view event handlers
        ViewZones.prototype._recomputeWhitespacesProps = function () {
            var _this = this;
            var id;
            var zone2Height = {};
            var hadAChange = false;
            for (id in this._zones) {
                if (this._zones.hasOwnProperty(id)) {
                    var zone = this._zones[id];
                    var props = this._computeWhitespaceProps(zone.delegate);
                    if (this._whitespaceManager.changeWhitespace(parseInt(id, 10), props.afterViewLineNumber, props.heightInPx)) {
                        this._safeCallOnComputedHeight(zone.delegate, props.heightInPx);
                        zone2Height[id] = props.heightInPx;
                        hadAChange = true;
                    }
                }
            }
            if (hadAChange) {
                this._requestModificationFrame(function () {
                    for (id in _this._zones) {
                        if (_this._zones.hasOwnProperty(id)) {
                            if (zone2Height.hasOwnProperty(id)) {
                                // TODO@Alex - edit dom node properties only in render()
                                styleMutator_1.StyleMutator.setHeight(_this._zones[id].delegate.domNode, zone2Height[id]);
                            }
                        }
                    }
                });
            }
            return hadAChange;
        };
        ViewZones.prototype.onConfigurationChanged = function (e) {
            if (e.lineHeight) {
                return this._recomputeWhitespacesProps();
            }
            return false;
        };
        ViewZones.prototype.onLineMappingChanged = function () {
            return this._recomputeWhitespacesProps();
        };
        ViewZones.prototype.onLayoutChanged = function (layoutInfo) {
            return true;
        };
        ViewZones.prototype.onScrollChanged = function (e) {
            return e.vertical;
        };
        ViewZones.prototype.onScrollWidthChanged = function (newScrollWidth) {
            return true;
        };
        ViewZones.prototype.onZonesChanged = function () {
            return true;
        };
        ViewZones.prototype.onModelLinesDeleted = function (e) {
            return true;
        };
        ViewZones.prototype.onModelLinesInserted = function (e) {
            return true;
        };
        // ---- end view event handlers
        ViewZones.prototype._getZoneOrdinal = function (zone) {
            if (typeof zone.afterColumn !== 'undefined') {
                return zone.afterColumn;
            }
            return 10000;
        };
        ViewZones.prototype._computeWhitespaceProps = function (zone) {
            if (zone.afterLineNumber === 0) {
                return {
                    afterViewLineNumber: 0,
                    heightInPx: this._heightInPixels(zone)
                };
            }
            var zoneAfterModelPosition;
            if (typeof zone.afterColumn !== 'undefined') {
                zoneAfterModelPosition = this._context.model.validateModelPosition({
                    lineNumber: zone.afterLineNumber,
                    column: zone.afterColumn
                });
            }
            else {
                var validAfterLineNumber = this._context.model.validateModelPosition({
                    lineNumber: zone.afterLineNumber,
                    column: 1
                }).lineNumber;
                zoneAfterModelPosition = {
                    lineNumber: validAfterLineNumber,
                    column: this._context.model.getModelLineMaxColumn(validAfterLineNumber)
                };
            }
            var zoneBeforeModelPosition;
            if (zoneAfterModelPosition.column === this._context.model.getModelLineMaxColumn(zoneAfterModelPosition.lineNumber)) {
                zoneBeforeModelPosition = this._context.model.validateModelPosition({
                    lineNumber: zoneAfterModelPosition.lineNumber + 1,
                    column: 1
                });
            }
            else {
                zoneBeforeModelPosition = this._context.model.validateModelPosition({
                    lineNumber: zoneAfterModelPosition.lineNumber,
                    column: zoneAfterModelPosition.column + 1
                });
            }
            var viewPosition = this._context.model.convertModelPositionToViewPosition(zoneAfterModelPosition.lineNumber, zoneAfterModelPosition.column);
            var isVisible = this._context.model.modelPositionIsVisible(zoneBeforeModelPosition);
            return {
                afterViewLineNumber: viewPosition.lineNumber,
                heightInPx: (isVisible ? this._heightInPixels(zone) : 0)
            };
        };
        ViewZones.prototype.addZone = function (zone) {
            var props = this._computeWhitespaceProps(zone);
            var whitespaceId = this._whitespaceManager.addWhitespace(props.afterViewLineNumber, this._getZoneOrdinal(zone), props.heightInPx);
            var myZone = {
                whitespaceId: whitespaceId,
                delegate: zone,
                isVisible: false
            };
            this._safeCallOnComputedHeight(myZone.delegate, props.heightInPx);
            this._requestModificationFrame(function () {
                if (!myZone.delegate.domNode.hasAttribute('monaco-view-zone')) {
                    // Do not position zone if it was removed in the meantime
                    return;
                }
                myZone.delegate.domNode.style.position = 'absolute';
                styleMutator_1.StyleMutator.setHeight(myZone.delegate.domNode, props.heightInPx);
                myZone.delegate.domNode.style.width = '100%';
                styleMutator_1.StyleMutator.setDisplay(myZone.delegate.domNode, 'none');
            });
            this._zones[myZone.whitespaceId.toString()] = myZone;
            myZone.delegate.domNode.setAttribute('monaco-view-zone', myZone.whitespaceId.toString());
            this.domNode.appendChild(myZone.delegate.domNode);
            return myZone.whitespaceId;
        };
        ViewZones.prototype.removeZone = function (id) {
            if (this._zones.hasOwnProperty(id.toString())) {
                var zone = this._zones[id.toString()];
                delete this._zones[id.toString()];
                this._whitespaceManager.removeWhitespace(zone.whitespaceId);
                zone.delegate.domNode.removeAttribute('monaco-visible-view-zone');
                zone.delegate.domNode.removeAttribute('monaco-view-zone');
                this._requestModificationFrame(function () {
                    if (zone.delegate.domNode.hasAttribute('monaco-view-zone')) {
                        // This dom node was added again as a view zone, so no need to mutate the DOM here
                        return;
                    }
                    if (zone.delegate.domNode.parentNode) {
                        zone.delegate.domNode.parentNode.removeChild(zone.delegate.domNode);
                    }
                });
                return true;
            }
            return false;
        };
        ViewZones.prototype.layoutZone = function (id) {
            var changed = false;
            if (this._zones.hasOwnProperty(id.toString())) {
                var zone = this._zones[id.toString()];
                var props = this._computeWhitespaceProps(zone.delegate);
                // let newOrdinal = this._getZoneOrdinal(zone.delegate);
                changed = this._whitespaceManager.changeWhitespace(zone.whitespaceId, props.afterViewLineNumber, props.heightInPx) || changed;
            }
            return changed;
        };
        ViewZones.prototype.shouldSuppressMouseDownOnViewZone = function (id) {
            if (this._zones.hasOwnProperty(id.toString())) {
                var zone = this._zones[id.toString()];
                return zone.delegate.suppressMouseDown;
            }
            return false;
        };
        ViewZones.prototype._heightInPixels = function (zone) {
            if (typeof zone.heightInPx === 'number') {
                return zone.heightInPx;
            }
            if (typeof zone.heightInLines === 'number') {
                return this._context.configuration.editor.lineHeight * zone.heightInLines;
            }
            return this._context.configuration.editor.lineHeight;
        };
        ViewZones.prototype._safeCallOnComputedHeight = function (zone, height) {
            if (typeof zone.onComputedHeight === 'function') {
                try {
                    zone.onComputedHeight(height);
                }
                catch (e) {
                    errors_1.onUnexpectedError(e);
                }
            }
        };
        ViewZones.prototype._safeCallOnDomNodeTop = function (zone, top) {
            if (typeof zone.onDomNodeTop === 'function') {
                try {
                    zone.onDomNodeTop(top);
                }
                catch (e) {
                    errors_1.onUnexpectedError(e);
                }
            }
        };
        ViewZones.prototype._render = function (ctx) {
            var _this = this;
            var visibleWhitespaces = this._whitespaceManager.getWhitespaceViewportData();
            this._requestModificationFrame(function () {
                var visibleZones = {}, i, len, hasVisibleZone = false;
                for (i = 0, len = visibleWhitespaces.length; i < len; i++) {
                    visibleZones[visibleWhitespaces[i].id.toString()] = visibleWhitespaces[i];
                    hasVisibleZone = true;
                }
                var id, zone;
                for (id in _this._zones) {
                    if (_this._zones.hasOwnProperty(id)) {
                        zone = _this._zones[id];
                        if (visibleZones.hasOwnProperty(id)) {
                            // zone is visible
                            styleMutator_1.StyleMutator.setTop(zone.delegate.domNode, (visibleZones[id].verticalOffset - ctx.bigNumbersDelta));
                            styleMutator_1.StyleMutator.setHeight(zone.delegate.domNode, visibleZones[id].height);
                            if (!zone.isVisible) {
                                styleMutator_1.StyleMutator.setDisplay(zone.delegate.domNode, 'block');
                                zone.delegate.domNode.setAttribute('monaco-visible-view-zone', 'true');
                                zone.isVisible = true;
                            }
                            _this._safeCallOnDomNodeTop(zone.delegate, ctx.getScrolledTopFromAbsoluteTop(visibleZones[id].verticalOffset));
                        }
                        else {
                            if (zone.isVisible) {
                                styleMutator_1.StyleMutator.setDisplay(zone.delegate.domNode, 'none');
                                zone.delegate.domNode.removeAttribute('monaco-visible-view-zone');
                                zone.isVisible = false;
                            }
                            _this._safeCallOnDomNodeTop(zone.delegate, ctx.getScrolledTopFromAbsoluteTop(-1000000));
                        }
                    }
                }
                if (hasVisibleZone) {
                    styleMutator_1.StyleMutator.setWidth(_this.domNode, ctx.scrollWidth);
                }
            });
        };
        return ViewZones;
    }(viewPart_1.ViewPart));
    exports.ViewZones = ViewZones;
});

define("vs/editor/common/viewModel/viewModelCursors", ["require", "exports", 'vs/editor/common/core/range', 'vs/editor/common/core/selection', 'vs/editor/common/editorCommon'], function (require, exports, range_1, selection_1, editorCommon) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewModelCursors = (function () {
        function ViewModelCursors(configuration, converter) {
            this.configuration = configuration;
            this.converter = converter;
            this.lastCursorPositionChangedEvent = null;
            this.lastCursorSelectionChangedEvent = null;
        }
        ViewModelCursors.prototype.getSelections = function () {
            if (this.lastCursorSelectionChangedEvent) {
                var selections = [];
                selections.push(this.converter.convertModelSelectionToViewSelection(this.lastCursorSelectionChangedEvent.selection));
                for (var i = 0, len = this.lastCursorSelectionChangedEvent.secondarySelections.length; i < len; i++) {
                    selections.push(this.converter.convertModelSelectionToViewSelection(this.lastCursorSelectionChangedEvent.secondarySelections[i]));
                }
                return selections;
            }
            else {
                return [new selection_1.Selection(1, 1, 1, 1)];
            }
        };
        ViewModelCursors.prototype.onCursorPositionChanged = function (e, emit) {
            this.lastCursorPositionChangedEvent = e;
            var position = this.converter.validateViewPosition(e.viewPosition.lineNumber, e.viewPosition.column, e.position), stopRenderingLineAfter = this.configuration.editor.stopRenderingLineAfter;
            // Limit position to be somewhere where it can actually be rendered
            if (stopRenderingLineAfter !== -1 && position.column > stopRenderingLineAfter) {
                position = position.clone();
                position.column = stopRenderingLineAfter;
            }
            var secondaryPositions = [];
            for (var i = 0, len = e.secondaryPositions.length; i < len; i++) {
                secondaryPositions[i] = this.converter.validateViewPosition(e.secondaryViewPositions[i].lineNumber, e.secondaryViewPositions[i].column, e.secondaryPositions[i]);
                // Limit position to be somewhere where it can actually be rendered
                if (stopRenderingLineAfter !== -1 && secondaryPositions[i].column > stopRenderingLineAfter) {
                    secondaryPositions[i] = secondaryPositions[i].clone();
                    secondaryPositions[i].column = stopRenderingLineAfter;
                }
            }
            var newEvent = {
                position: position,
                secondaryPositions: secondaryPositions,
                isInEditableRange: e.isInEditableRange
            };
            emit(editorCommon.ViewEventNames.CursorPositionChangedEvent, newEvent);
        };
        ViewModelCursors.prototype.onCursorSelectionChanged = function (e, emit) {
            this.lastCursorSelectionChangedEvent = e;
            var selection = this.converter.validateViewSelection(e.viewSelection, e.selection);
            var secondarySelections = [];
            for (var i = 0, len = e.secondarySelections.length; i < len; i++) {
                secondarySelections[i] = this.converter.validateViewSelection(e.secondaryViewSelections[i], e.secondarySelections[i]);
            }
            var newEvent = {
                selection: selection,
                secondarySelections: secondarySelections
            };
            emit(editorCommon.ViewEventNames.CursorSelectionChangedEvent, newEvent);
        };
        ViewModelCursors.prototype.onCursorRevealRange = function (e, emit) {
            var viewRange = null;
            if (e.viewRange) {
                var viewStartRange = this.converter.validateViewPosition(e.viewRange.startLineNumber, e.viewRange.startColumn, e.range.getStartPosition());
                var viewEndRange = this.converter.validateViewPosition(e.viewRange.endLineNumber, e.viewRange.endColumn, e.range.getEndPosition());
                viewRange = new range_1.Range(viewStartRange.lineNumber, viewStartRange.column, viewEndRange.lineNumber, viewEndRange.column);
            }
            else {
                viewRange = this.converter.convertModelRangeToViewRange(e.range);
            }
            var newEvent = {
                range: viewRange,
                verticalType: e.verticalType,
                revealHorizontal: e.revealHorizontal
            };
            emit(editorCommon.ViewEventNames.RevealRangeEvent, newEvent);
        };
        ViewModelCursors.prototype.onCursorScrollRequest = function (e, emit) {
            var newEvent = {
                deltaLines: e.deltaLines
            };
            emit(editorCommon.ViewEventNames.ScrollRequestEvent, newEvent);
        };
        ViewModelCursors.prototype.onLineMappingChanged = function (emit) {
            if (this.lastCursorPositionChangedEvent) {
                this.onCursorPositionChanged(this.lastCursorPositionChangedEvent, emit);
            }
            if (this.lastCursorSelectionChangedEvent) {
                this.onCursorSelectionChanged(this.lastCursorSelectionChangedEvent, emit);
            }
        };
        return ViewModelCursors;
    }());
    exports.ViewModelCursors = ViewModelCursors;
});

define("vs/editor/common/viewModel/viewModelDecorations", ["require", "exports", 'vs/editor/common/core/range', 'vs/editor/common/editorCommon'], function (require, exports, range_1, editorCommon) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewModelDecoration = (function () {
        function ViewModelDecoration(source, range) {
            this.id = source.id;
            this.options = source.options;
            this.ownerId = source.ownerId;
            this.modelRange = source.range;
            this.range = range;
        }
        return ViewModelDecoration;
    }());
    var ViewModelDecorations = (function () {
        function ViewModelDecorations(editorId, configuration, converter) {
            this.editorId = editorId;
            this.configuration = configuration;
            this.converter = converter;
            this.decorations = [];
            this._clearCachedModelDecorationsResolver();
        }
        ViewModelDecorations.prototype._clearCachedModelDecorationsResolver = function () {
            this._cachedModelDecorationsResolver = null;
            this._cachedModelDecorationsResolverStartLineNumber = 0;
            this._cachedModelDecorationsResolverEndLineNumber = 0;
        };
        ViewModelDecorations.prototype.dispose = function () {
            this.converter = null;
            this.decorations = null;
            this._clearCachedModelDecorationsResolver();
        };
        ViewModelDecorations.compareDecorations = function (a, b) {
            return range_1.Range.compareRangesUsingStarts(a.range, b.range);
        };
        ViewModelDecorations.prototype.reset = function (model) {
            var decorations = model.getAllDecorations(this.editorId, this.configuration.editor.readOnly), i, len, theirDecoration, myDecoration;
            this.decorations = [];
            for (i = 0, len = decorations.length; i < len; i++) {
                theirDecoration = decorations[i];
                myDecoration = new ViewModelDecoration(theirDecoration, this.converter.convertModelRangeToViewRange(theirDecoration.range, theirDecoration.options.isWholeLine));
                this.decorations[i] = myDecoration;
            }
            this._clearCachedModelDecorationsResolver();
            this.decorations.sort(ViewModelDecorations.compareDecorations);
        };
        ViewModelDecorations.prototype.onModelDecorationsChanged = function (e, emit) {
            var somethingChanged = false, inlineDecorationsChanged = false;
            // -----------------------------------
            // Interpret addedOrChangedDecorations
            var removedMap = {}, addedOrChangedMap = {}, theirDecoration, i, skipValidation = this.configuration.editor.readOnly, len;
            for (i = 0, len = e.addedOrChangedDecorations.length; i < len; i++) {
                theirDecoration = e.addedOrChangedDecorations[i];
                if (skipValidation && theirDecoration.isForValidation) {
                    continue;
                }
                if (theirDecoration.ownerId && theirDecoration.ownerId !== this.editorId) {
                    continue;
                }
                addedOrChangedMap[theirDecoration.id] = theirDecoration;
            }
            for (i = 0, len = e.removedDecorations.length; i < len; i++) {
                removedMap[e.removedDecorations[i]] = true;
            }
            // Interpret changed decorations
            var usedMap = {}, myDecoration;
            for (i = 0, len = this.decorations.length; i < len; i++) {
                myDecoration = this.decorations[i];
                if (addedOrChangedMap.hasOwnProperty(myDecoration.id)) {
                    usedMap[myDecoration.id] = true;
                    theirDecoration = addedOrChangedMap[myDecoration.id];
                    myDecoration.options = theirDecoration.options;
                    myDecoration.modelRange = theirDecoration.range;
                    myDecoration.range = this.converter.convertModelRangeToViewRange(theirDecoration.range, theirDecoration.options.isWholeLine);
                    //				console.log(theirDecoration.range.toString() + '--->' + myDecoration.range.toString());
                    if (myDecoration.options.inlineClassName) {
                        inlineDecorationsChanged = true;
                    }
                    somethingChanged = true;
                }
                if (removedMap.hasOwnProperty(myDecoration.id)) {
                    if (this.decorations[i].options.inlineClassName) {
                        inlineDecorationsChanged = true;
                    }
                    this.decorations.splice(i, 1);
                    len--;
                    i--;
                    somethingChanged = true;
                }
            }
            // Interpret new decorations
            var id;
            for (id in addedOrChangedMap) {
                if (!usedMap.hasOwnProperty(id) && addedOrChangedMap.hasOwnProperty(id)) {
                    theirDecoration = addedOrChangedMap[id];
                    myDecoration = new ViewModelDecoration(theirDecoration, this.converter.convertModelRangeToViewRange(theirDecoration.range, theirDecoration.options.isWholeLine));
                    //				console.log(theirDecoration.range.toString() + '--->' + myDecoration.range.toString());
                    this.decorations.push(myDecoration);
                    if (myDecoration.options.inlineClassName) {
                        inlineDecorationsChanged = true;
                    }
                    somethingChanged = true;
                }
            }
            if (somethingChanged) {
                this._clearCachedModelDecorationsResolver();
                this.decorations.sort(ViewModelDecorations.compareDecorations);
                var newEvent = {
                    inlineDecorationsChanged: inlineDecorationsChanged
                };
                emit(editorCommon.ViewEventNames.DecorationsChangedEvent, newEvent);
            }
        };
        ViewModelDecorations.prototype.onLineMappingChanged = function (emit) {
            var decorations = this.decorations, d, i, newRange, somethingChanged = false, inlineDecorationsChanged = false, len;
            for (i = 0, len = decorations.length; i < len; i++) {
                d = decorations[i];
                newRange = this.converter.convertModelRangeToViewRange(d.modelRange, d.options.isWholeLine);
                if (!inlineDecorationsChanged && d.options.inlineClassName && !range_1.Range.equalsRange(newRange, d.range)) {
                    inlineDecorationsChanged = true;
                }
                if (!somethingChanged && !range_1.Range.equalsRange(newRange, d.range)) {
                    somethingChanged = true;
                }
                d.range = newRange;
            }
            if (somethingChanged) {
                this._clearCachedModelDecorationsResolver();
                this.decorations.sort(ViewModelDecorations.compareDecorations);
                var newEvent = {
                    inlineDecorationsChanged: inlineDecorationsChanged
                };
                emit(editorCommon.ViewEventNames.DecorationsChangedEvent, newEvent);
            }
        };
        ViewModelDecorations.prototype.getAllDecorations = function () {
            return this.decorations;
        };
        ViewModelDecorations.prototype.getDecorationsResolver = function (startLineNumber, endLineNumber) {
            var cacheIsValid = true;
            cacheIsValid = cacheIsValid && (this._cachedModelDecorationsResolver !== null);
            cacheIsValid = cacheIsValid && (this._cachedModelDecorationsResolverStartLineNumber === startLineNumber);
            cacheIsValid = cacheIsValid && (this._cachedModelDecorationsResolverEndLineNumber === endLineNumber);
            if (!cacheIsValid) {
                this._cachedModelDecorationsResolver = this._createDecorationsResolver(startLineNumber, endLineNumber);
                this._cachedModelDecorationsResolverStartLineNumber = startLineNumber;
                this._cachedModelDecorationsResolverEndLineNumber = endLineNumber;
            }
            return this._cachedModelDecorationsResolver;
        };
        ViewModelDecorations.prototype._createDecorationsResolver = function (startLineNumber, endLineNumber) {
            var decorationsInViewport = [], inlineDecorations = [], j, intersectedStartLineNumber, intersectedEndLineNumber, decorations = this.decorations, d, r, i, len;
            for (j = startLineNumber; j <= endLineNumber; j++) {
                inlineDecorations[j - startLineNumber] = [];
            }
            for (i = 0, len = decorations.length; i < len; i++) {
                d = decorations[i];
                r = d.range;
                if (r.startLineNumber > endLineNumber) {
                    // Decorations are sorted ascending by line number, it is safe to stop now
                    break;
                }
                if (r.endLineNumber < startLineNumber) {
                    continue;
                }
                decorationsInViewport.push(d);
                if (d.options.inlineClassName) {
                    intersectedStartLineNumber = Math.max(startLineNumber, r.startLineNumber);
                    intersectedEndLineNumber = Math.min(endLineNumber, r.endLineNumber);
                    for (j = intersectedStartLineNumber; j <= intersectedEndLineNumber; j++) {
                        inlineDecorations[j - startLineNumber].push(d);
                    }
                }
            }
            return {
                getDecorations: function () {
                    return decorationsInViewport;
                },
                getInlineDecorations: function (lineNumber) {
                    if (lineNumber < startLineNumber || lineNumber > endLineNumber) {
                        throw new Error('Unexpected line outside the ViewModelDecorationsResolver preconfigured range');
                    }
                    return inlineDecorations[lineNumber - startLineNumber];
                }
            };
        };
        return ViewModelDecorations;
    }());
    exports.ViewModelDecorations = ViewModelDecorations;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/viewModel/viewModel", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/base/common/lifecycle', 'vs/base/common/strings', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/core/selection', 'vs/editor/common/editorCommon', 'vs/editor/common/viewModel/viewModelCursors', 'vs/editor/common/viewModel/viewModelDecorations'], function (require, exports, eventEmitter_1, lifecycle_1, strings, position_1, range_1, selection_1, editorCommon, viewModelCursors_1, viewModelDecorations_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewModel = (function (_super) {
        __extends(ViewModel, _super);
        function ViewModel(lines, editorId, configuration, model, getCurrentCenteredModelRange) {
            var _this = this;
            _super.call(this);
            this.lines = lines;
            this.editorId = editorId;
            this.configuration = configuration;
            this.model = model;
            this.getCurrentCenteredModelRange = getCurrentCenteredModelRange;
            this.decorations = new viewModelDecorations_1.ViewModelDecorations(this.editorId, this.configuration, {
                convertModelRangeToViewRange: function (modelRange, isWholeLine) {
                    if (isWholeLine) {
                        return _this.convertWholeLineModelRangeToViewRange(modelRange);
                    }
                    return _this.convertModelRangeToViewRange(modelRange);
                }
            });
            this.decorations.reset(this.model);
            this.cursors = new viewModelCursors_1.ViewModelCursors(this.configuration, this);
            this._updateShouldForceTokenization();
            this.listenersToRemove = [];
            this._toDispose = [];
            this.listenersToRemove.push(this.model.addBulkListener(function (events) { return _this.onEvents(events); }));
            this._toDispose.push(this.configuration.onDidChange(function (e) {
                _this.onEvents([new eventEmitter_1.EmitterEvent(editorCommon.EventType.ConfigurationChanged, e)]);
            }));
        }
        ViewModel.prototype.setHiddenAreas = function (ranges) {
            var _this = this;
            this.deferredEmit(function () {
                var lineMappingChanged = _this.lines.setHiddenAreas(ranges, function (eventType, payload) { return _this.emit(eventType, payload); });
                if (lineMappingChanged) {
                    _this.emit(editorCommon.ViewEventNames.LineMappingChangedEvent);
                    _this.decorations.onLineMappingChanged(function (eventType, payload) { return _this.emit(eventType, payload); });
                    _this.cursors.onLineMappingChanged(function (eventType, payload) { return _this.emit(eventType, payload); });
                    _this._updateShouldForceTokenization();
                }
            });
        };
        ViewModel.prototype.modelPositionIsVisible = function (position) {
            return this.lines.inputPositionIsVisible(position.lineNumber, position.column);
        };
        ViewModel.prototype.dispose = function () {
            this.listenersToRemove.forEach(function (element) {
                element();
            });
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
            this.listenersToRemove = [];
            this.decorations.dispose();
            this.decorations = null;
            this.lines.dispose();
            this.lines = null;
            this.configuration = null;
            this.model = null;
        };
        ViewModel.prototype._updateShouldForceTokenization = function () {
            this.shouldForceTokenization = (this.lines.getOutputLineCount() <= this.configuration.editor.forcedTokenizationBoundary);
        };
        ViewModel.prototype._onTabSizeChange = function (newTabSize) {
            var _this = this;
            var lineMappingChanged = this.lines.setTabSize(newTabSize, function (eventType, payload) { return _this.emit(eventType, payload); });
            if (lineMappingChanged) {
                this.emit(editorCommon.ViewEventNames.LineMappingChangedEvent);
                this.decorations.onLineMappingChanged(function (eventType, payload) { return _this.emit(eventType, payload); });
                this.cursors.onLineMappingChanged(function (eventType, payload) { return _this.emit(eventType, payload); });
                this._updateShouldForceTokenization();
            }
            return lineMappingChanged;
        };
        ViewModel.prototype._onWrappingIndentChange = function (newWrappingIndent) {
            var _this = this;
            var lineMappingChanged = this.lines.setWrappingIndent(editorCommon.wrappingIndentFromString(newWrappingIndent), function (eventType, payload) { return _this.emit(eventType, payload); });
            if (lineMappingChanged) {
                this.emit(editorCommon.ViewEventNames.LineMappingChangedEvent);
                this.decorations.onLineMappingChanged(function (eventType, payload) { return _this.emit(eventType, payload); });
                this.cursors.onLineMappingChanged(function (eventType, payload) { return _this.emit(eventType, payload); });
                this._updateShouldForceTokenization();
            }
            return lineMappingChanged;
        };
        ViewModel.prototype._restoreCenteredModelRange = function (range) {
            // modelLine -> viewLine
            var newCenteredViewRange = this.convertModelRangeToViewRange(range);
            // Send a reveal event to restore the centered content
            var restoreRevealEvent = {
                range: newCenteredViewRange,
                verticalType: editorCommon.VerticalRevealType.Center,
                revealHorizontal: false
            };
            this.emit(editorCommon.ViewEventNames.RevealRangeEvent, restoreRevealEvent);
        };
        ViewModel.prototype._onWrappingColumnChange = function (newWrappingColumn, columnsForFullWidthChar) {
            var _this = this;
            var lineMappingChanged = this.lines.setWrappingColumn(newWrappingColumn, columnsForFullWidthChar, function (eventType, payload) { return _this.emit(eventType, payload); });
            if (lineMappingChanged) {
                this.emit(editorCommon.ViewEventNames.LineMappingChangedEvent);
                this.decorations.onLineMappingChanged(function (eventType, payload) { return _this.emit(eventType, payload); });
                this.cursors.onLineMappingChanged(function (eventType, payload) { return _this.emit(eventType, payload); });
                this._updateShouldForceTokenization();
            }
            return lineMappingChanged;
        };
        ViewModel.prototype.addEventSource = function (eventSource) {
            var _this = this;
            this.listenersToRemove.push(eventSource.addBulkListener(function (events) { return _this.onEvents(events); }));
        };
        ViewModel.prototype.onEvents = function (events) {
            var _this = this;
            this.deferredEmit(function () {
                var hasContentChange = events.some(function (e) { return e.getType() === editorCommon.EventType.ModelContentChanged; }), previousCenteredModelRange;
                if (!hasContentChange) {
                    // We can only convert the current centered view range to the current centered model range if the model has no changes.
                    previousCenteredModelRange = _this.getCurrentCenteredModelRange();
                }
                var i, len, e, data, shouldUpdateForceTokenization = false, modelContentChangedEvent, hadOtherModelChange = false, hadModelLineChangeThatChangedLineMapping = false, revealPreviousCenteredModelRange = false;
                for (i = 0, len = events.length; i < len; i++) {
                    e = events[i];
                    data = e.getData();
                    switch (e.getType()) {
                        case editorCommon.EventType.ModelContentChanged:
                            modelContentChangedEvent = data;
                            switch (modelContentChangedEvent.changeType) {
                                case editorCommon.EventType.ModelContentChangedFlush:
                                    _this.onModelFlushed(modelContentChangedEvent);
                                    hadOtherModelChange = true;
                                    break;
                                case editorCommon.EventType.ModelContentChangedLinesDeleted:
                                    _this.onModelLinesDeleted(modelContentChangedEvent);
                                    hadOtherModelChange = true;
                                    break;
                                case editorCommon.EventType.ModelContentChangedLinesInserted:
                                    _this.onModelLinesInserted(modelContentChangedEvent);
                                    hadOtherModelChange = true;
                                    break;
                                case editorCommon.EventType.ModelContentChangedLineChanged:
                                    hadModelLineChangeThatChangedLineMapping = _this.onModelLineChanged(modelContentChangedEvent);
                                    break;
                                default:
                                    console.info('ViewModel received unknown event: ');
                                    console.info(e);
                            }
                            shouldUpdateForceTokenization = true;
                            break;
                        case editorCommon.EventType.ModelTokensChanged:
                            _this.onModelTokensChanged(data);
                            break;
                        case editorCommon.EventType.ModelModeChanged:
                            // That's ok, a model tokens changed event will follow shortly
                            break;
                        case editorCommon.EventType.ModelModeSupportChanged:
                            // That's ok, no work to do
                            break;
                        case editorCommon.EventType.ModelContentChanged2:
                            // Ignore
                            break;
                        case editorCommon.EventType.ModelOptionsChanged:
                            // A tab size change causes a line mapping changed event => all view parts will repaint OK, no further event needed here
                            var prevLineCount = _this.lines.getOutputLineCount();
                            var tabSizeChanged = _this._onTabSizeChange(_this.model.getOptions().tabSize);
                            var newLineCount = _this.lines.getOutputLineCount();
                            if (tabSizeChanged && prevLineCount !== newLineCount) {
                                revealPreviousCenteredModelRange = true;
                            }
                            break;
                        case editorCommon.EventType.ModelDecorationsChanged:
                            _this.onModelDecorationsChanged(data);
                            break;
                        case editorCommon.EventType.ModelDispose:
                            // Ignore, since the editor will take care of this and destroy the view shortly
                            break;
                        case editorCommon.EventType.CursorPositionChanged:
                            _this.onCursorPositionChanged(data);
                            break;
                        case editorCommon.EventType.CursorSelectionChanged:
                            _this.onCursorSelectionChanged(data);
                            break;
                        case editorCommon.EventType.CursorRevealRange:
                            _this.onCursorRevealRange(data);
                            break;
                        case editorCommon.EventType.CursorScrollRequest:
                            _this.onCursorScrollRequest(data);
                            break;
                        case editorCommon.EventType.ConfigurationChanged:
                            revealPreviousCenteredModelRange = _this._onWrappingIndentChange(_this.configuration.editor.wrappingIndent) || revealPreviousCenteredModelRange;
                            revealPreviousCenteredModelRange = _this._onWrappingColumnChange(_this.configuration.editor.wrappingInfo.wrappingColumn, _this.configuration.editor.typicalFullwidthCharacterWidth / _this.configuration.editor.typicalHalfwidthCharacterWidth) || revealPreviousCenteredModelRange;
                            if (data.readOnly) {
                                // Must read again all decorations due to readOnly filtering
                                _this.decorations.reset(_this.model);
                                var decorationsChangedEvent = {
                                    inlineDecorationsChanged: false
                                };
                                _this.emit(editorCommon.ViewEventNames.DecorationsChangedEvent, decorationsChangedEvent);
                            }
                            _this.emit(e.getType(), data);
                            break;
                        default:
                            console.info('View received unknown event: ');
                            console.info(e);
                    }
                }
                if (shouldUpdateForceTokenization) {
                    _this._updateShouldForceTokenization();
                }
                if (!hadOtherModelChange && hadModelLineChangeThatChangedLineMapping) {
                    _this.emit(editorCommon.ViewEventNames.LineMappingChangedEvent);
                    _this.decorations.onLineMappingChanged(function (eventType, payload) { return _this.emit(eventType, payload); });
                    _this.cursors.onLineMappingChanged(function (eventType, payload) { return _this.emit(eventType, payload); });
                    _this._updateShouldForceTokenization();
                }
                if (revealPreviousCenteredModelRange && previousCenteredModelRange) {
                    _this._restoreCenteredModelRange(previousCenteredModelRange);
                }
            });
        };
        // --- begin inbound event conversion
        ViewModel.prototype.onModelFlushed = function (e) {
            var _this = this;
            this.lines.onModelFlushed(e.versionId, function (eventType, payload) { return _this.emit(eventType, payload); });
            this.decorations.reset(this.model);
        };
        ViewModel.prototype.onModelDecorationsChanged = function (e) {
            var _this = this;
            this.decorations.onModelDecorationsChanged(e, function (eventType, payload) { return _this.emit(eventType, payload); });
        };
        ViewModel.prototype.onModelLinesDeleted = function (e) {
            var _this = this;
            this.lines.onModelLinesDeleted(e.versionId, e.fromLineNumber, e.toLineNumber, function (eventType, payload) { return _this.emit(eventType, payload); });
        };
        ViewModel.prototype.onModelTokensChanged = function (e) {
            var viewStartLineNumber = this.convertModelPositionToViewPosition(e.fromLineNumber, 1).lineNumber;
            var viewEndLineNumber = this.convertModelPositionToViewPosition(e.toLineNumber, this.model.getLineMaxColumn(e.toLineNumber)).lineNumber;
            var e = {
                fromLineNumber: viewStartLineNumber,
                toLineNumber: viewEndLineNumber
            };
            this.emit(editorCommon.ViewEventNames.TokensChangedEvent, e);
        };
        ViewModel.prototype.onModelLineChanged = function (e) {
            var _this = this;
            var lineMappingChanged = this.lines.onModelLineChanged(e.versionId, e.lineNumber, e.detail, function (eventType, payload) { return _this.emit(eventType, payload); });
            return lineMappingChanged;
        };
        ViewModel.prototype.onModelLinesInserted = function (e) {
            var _this = this;
            this.lines.onModelLinesInserted(e.versionId, e.fromLineNumber, e.toLineNumber, e.detail.split('\n'), function (eventType, payload) { return _this.emit(eventType, payload); });
        };
        ViewModel.prototype.validateViewRange = function (viewStartLineNumber, viewStartColumn, viewEndLineNumber, viewEndColumn, modelRange) {
            var validViewStart = this.validateViewPosition(viewStartColumn, viewStartColumn, modelRange.getStartPosition());
            var validViewEnd = this.validateViewPosition(viewEndLineNumber, viewEndColumn, modelRange.getEndPosition());
            return new range_1.Range(validViewStart.lineNumber, validViewStart.column, validViewEnd.lineNumber, validViewEnd.column);
        };
        ViewModel.prototype.validateViewPosition = function (viewLineNumber, viewColumn, modelPosition) {
            if (viewLineNumber < 1) {
                viewLineNumber = 1;
            }
            var lineCount = this.getLineCount();
            if (viewLineNumber > lineCount) {
                viewLineNumber = lineCount;
            }
            var viewMinColumn = this.getLineMinColumn(viewLineNumber);
            var viewMaxColumn = this.getLineMaxColumn(viewLineNumber);
            if (viewColumn < viewMinColumn) {
                viewColumn = viewMinColumn;
            }
            if (viewColumn > viewMaxColumn) {
                viewColumn = viewMaxColumn;
            }
            var computedModelPosition = this.convertViewPositionToModelPosition(viewLineNumber, viewColumn);
            if (computedModelPosition.equals(modelPosition)) {
                return new position_1.Position(viewLineNumber, viewColumn);
            }
            return this.convertModelPositionToViewPosition(modelPosition.lineNumber, modelPosition.column);
        };
        ViewModel.prototype.validateViewSelection = function (viewSelection, modelSelection) {
            var modelSelectionStart = new position_1.Position(modelSelection.selectionStartLineNumber, modelSelection.selectionStartColumn);
            var modelPosition = new position_1.Position(modelSelection.positionLineNumber, modelSelection.positionColumn);
            var viewSelectionStart = this.validateViewPosition(viewSelection.selectionStartLineNumber, viewSelection.selectionStartColumn, modelSelectionStart);
            var viewPosition = this.validateViewPosition(viewSelection.positionLineNumber, viewSelection.positionColumn, modelPosition);
            return new selection_1.Selection(viewSelectionStart.lineNumber, viewSelectionStart.column, viewPosition.lineNumber, viewPosition.column);
        };
        ViewModel.prototype.onCursorPositionChanged = function (e) {
            var _this = this;
            this.cursors.onCursorPositionChanged(e, function (eventType, payload) { return _this.emit(eventType, payload); });
        };
        ViewModel.prototype.onCursorSelectionChanged = function (e) {
            var _this = this;
            this.cursors.onCursorSelectionChanged(e, function (eventType, payload) { return _this.emit(eventType, payload); });
        };
        ViewModel.prototype.onCursorRevealRange = function (e) {
            var _this = this;
            this.cursors.onCursorRevealRange(e, function (eventType, payload) { return _this.emit(eventType, payload); });
        };
        ViewModel.prototype.onCursorScrollRequest = function (e) {
            var _this = this;
            this.cursors.onCursorScrollRequest(e, function (eventType, payload) { return _this.emit(eventType, payload); });
        };
        // --- end inbound event conversion
        ViewModel.prototype.getTabSize = function () {
            return this.model.getOptions().tabSize;
        };
        ViewModel.prototype.getLineCount = function () {
            return this.lines.getOutputLineCount();
        };
        ViewModel.prototype.getLineContent = function (lineNumber) {
            return this.lines.getOutputLineContent(lineNumber);
        };
        ViewModel.prototype.getLineMinColumn = function (lineNumber) {
            return this.lines.getOutputLineMinColumn(lineNumber);
        };
        ViewModel.prototype.getLineMaxColumn = function (lineNumber) {
            return this.lines.getOutputLineMaxColumn(lineNumber);
        };
        ViewModel.prototype.getLineFirstNonWhitespaceColumn = function (lineNumber) {
            var result = strings.firstNonWhitespaceIndex(this.getLineContent(lineNumber));
            if (result === -1) {
                return 0;
            }
            return result + 1;
        };
        ViewModel.prototype.getLineLastNonWhitespaceColumn = function (lineNumber) {
            var result = strings.lastNonWhitespaceIndex(this.getLineContent(lineNumber));
            if (result === -1) {
                return 0;
            }
            return result + 2;
        };
        ViewModel.prototype.getLineTokens = function (lineNumber) {
            return this.lines.getOutputLineTokens(lineNumber, !this.shouldForceTokenization);
        };
        ViewModel.prototype.getLineRenderLineNumber = function (viewLineNumber) {
            var modelPosition = this.convertViewPositionToModelPosition(viewLineNumber, 1);
            if (modelPosition.column !== 1) {
                return '';
            }
            var modelLineNumber = modelPosition.lineNumber;
            if (typeof this.configuration.editor.lineNumbers === 'function') {
                return this.configuration.editor.lineNumbers(modelLineNumber);
            }
            return modelLineNumber.toString();
        };
        ViewModel.prototype.getDecorationsResolver = function (startLineNumber, endLineNumber) {
            return this.decorations.getDecorationsResolver(startLineNumber, endLineNumber);
        };
        ViewModel.prototype.getAllDecorations = function () {
            return this.decorations.getAllDecorations();
        };
        ViewModel.prototype.getEOL = function () {
            return this.model.getEOL();
        };
        ViewModel.prototype.getValueInRange = function (range, eol) {
            var modelRange = this.convertViewRangeToModelRange(range);
            return this.model.getValueInRange(modelRange, eol);
        };
        ViewModel.prototype.getModelLineContent = function (modelLineNumber) {
            return this.model.getLineContent(modelLineNumber);
        };
        ViewModel.prototype.getSelections = function () {
            return this.cursors.getSelections();
        };
        ViewModel.prototype.getModelLineMaxColumn = function (modelLineNumber) {
            return this.model.getLineMaxColumn(modelLineNumber);
        };
        ViewModel.prototype.validateModelPosition = function (position) {
            return this.model.validatePosition(position);
        };
        ViewModel.prototype.convertViewPositionToModelPosition = function (viewLineNumber, viewColumn) {
            return this.lines.convertOutputPositionToInputPosition(viewLineNumber, viewColumn);
        };
        ViewModel.prototype.convertViewRangeToModelRange = function (viewRange) {
            var start = this.convertViewPositionToModelPosition(viewRange.startLineNumber, viewRange.startColumn);
            var end = this.convertViewPositionToModelPosition(viewRange.endLineNumber, viewRange.endColumn);
            return new range_1.Range(start.lineNumber, start.column, end.lineNumber, end.column);
        };
        ViewModel.prototype.convertViewSelectionToModelSelection = function (viewSelection) {
            var selectionStart = this.convertViewPositionToModelPosition(viewSelection.selectionStartLineNumber, viewSelection.selectionStartColumn);
            var position = this.convertViewPositionToModelPosition(viewSelection.positionLineNumber, viewSelection.positionColumn);
            return new selection_1.Selection(selectionStart.lineNumber, selectionStart.column, position.lineNumber, position.column);
        };
        ViewModel.prototype.convertModelPositionToViewPosition = function (modelLineNumber, modelColumn) {
            return this.lines.convertInputPositionToOutputPosition(modelLineNumber, modelColumn);
        };
        ViewModel.prototype.convertModelSelectionToViewSelection = function (modelSelection) {
            var selectionStart = this.convertModelPositionToViewPosition(modelSelection.selectionStartLineNumber, modelSelection.selectionStartColumn);
            var position = this.convertModelPositionToViewPosition(modelSelection.positionLineNumber, modelSelection.positionColumn);
            return new selection_1.Selection(selectionStart.lineNumber, selectionStart.column, position.lineNumber, position.column);
        };
        ViewModel.prototype.convertModelRangeToViewRange = function (modelRange) {
            var start = this.convertModelPositionToViewPosition(modelRange.startLineNumber, modelRange.startColumn);
            var end = this.convertModelPositionToViewPosition(modelRange.endLineNumber, modelRange.endColumn);
            return new range_1.Range(start.lineNumber, start.column, end.lineNumber, end.column);
        };
        ViewModel.prototype.convertWholeLineModelRangeToViewRange = function (modelRange) {
            var start = this.convertModelPositionToViewPosition(modelRange.startLineNumber, 1);
            var end = this.convertModelPositionToViewPosition(modelRange.endLineNumber, this.model.getLineMaxColumn(modelRange.endLineNumber));
            return new range_1.Range(start.lineNumber, start.column, end.lineNumber, end.column);
        };
        return ViewModel;
    }(eventEmitter_1.EventEmitter));
    exports.ViewModel = ViewModel;
});

define("vs/editor/contrib/comment/common/blockCommentCommand", ["require", "exports", 'vs/editor/common/core/editOperation', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/core/selection'], function (require, exports, editOperation_1, position_1, range_1, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var BlockCommentCommand = (function () {
        function BlockCommentCommand(selection) {
            this._selection = selection;
            this._usedEndToken = null;
        }
        BlockCommentCommand._haystackHasNeedleAtOffset = function (haystack, needle, offset) {
            if (offset < 0) {
                return false;
            }
            var needleLength = needle.length;
            var haystackLength = haystack.length;
            if (offset + needleLength > haystackLength) {
                return false;
            }
            for (var i = 0; i < needleLength; i++) {
                if (haystack.charCodeAt(offset + i) !== needle.charCodeAt(i)) {
                    return false;
                }
            }
            return true;
        };
        BlockCommentCommand.prototype._createOperationsForBlockComment = function (selection, config, model, builder) {
            var startLineNumber = selection.startLineNumber;
            var startColumn = selection.startColumn;
            var endLineNumber = selection.endLineNumber;
            var endColumn = selection.endColumn;
            var startToken = config.blockCommentStartToken;
            var endToken = config.blockCommentEndToken;
            var startTokenIndex = model.getLineContent(startLineNumber).lastIndexOf(startToken, startColumn - 1 + startToken.length);
            var endTokenIndex = model.getLineContent(endLineNumber).indexOf(endToken, endColumn - 1 - endToken.length);
            var ops;
            if (startTokenIndex !== -1 && endTokenIndex !== -1) {
                ops = BlockCommentCommand._createRemoveBlockCommentOperations({
                    startLineNumber: startLineNumber,
                    startColumn: startTokenIndex + 1 + startToken.length,
                    endLineNumber: endLineNumber,
                    endColumn: endTokenIndex + 1
                }, startToken, endToken);
            }
            else {
                ops = BlockCommentCommand._createAddBlockCommentOperations(selection, startToken, endToken);
                this._usedEndToken = ops.length === 1 ? endToken : null;
            }
            for (var i = 0; i < ops.length; i++) {
                builder.addEditOperation(ops[i].range, ops[i].text);
            }
        };
        BlockCommentCommand._createRemoveBlockCommentOperations = function (r, startToken, endToken) {
            var res = [];
            if (!range_1.Range.isEmpty(r)) {
                // Remove block comment start
                res.push(editOperation_1.EditOperation.delete(new range_1.Range(r.startLineNumber, r.startColumn - startToken.length, r.startLineNumber, r.startColumn)));
                // Remove block comment end
                res.push(editOperation_1.EditOperation.delete(new range_1.Range(r.endLineNumber, r.endColumn, r.endLineNumber, r.endColumn + endToken.length)));
            }
            else {
                // Remove both continuously
                res.push(editOperation_1.EditOperation.delete(new range_1.Range(r.startLineNumber, r.startColumn - startToken.length, r.endLineNumber, r.endColumn + endToken.length)));
            }
            return res;
        };
        BlockCommentCommand._createAddBlockCommentOperations = function (r, startToken, endToken) {
            var res = [];
            if (!range_1.Range.isEmpty(r)) {
                // Insert block comment start
                res.push(editOperation_1.EditOperation.insert(new position_1.Position(r.startLineNumber, r.startColumn), startToken));
                // Insert block comment end
                res.push(editOperation_1.EditOperation.insert(new position_1.Position(r.endLineNumber, r.endColumn), endToken));
            }
            else {
                // Insert both continuously
                res.push(editOperation_1.EditOperation.replace(new range_1.Range(r.startLineNumber, r.startColumn, r.endLineNumber, r.endColumn), startToken + endToken));
            }
            return res;
        };
        BlockCommentCommand.prototype.getEditOperations = function (model, builder) {
            var startLineNumber = this._selection.startLineNumber;
            var startColumn = this._selection.startColumn;
            var endLineNumber = this._selection.endLineNumber;
            var endColumn = this._selection.endColumn;
            var richEditSupport = model.getModeAtPosition(startLineNumber, startColumn).richEditSupport;
            var config = richEditSupport ? richEditSupport.comments : null;
            if (!config || !config.blockCommentStartToken || !config.blockCommentEndToken) {
                // Mode does not support block comments
                return;
            }
            this._createOperationsForBlockComment({
                startLineNumber: startLineNumber,
                startColumn: startColumn,
                endLineNumber: endLineNumber,
                endColumn: endColumn
            }, config, model, builder);
        };
        BlockCommentCommand.prototype.computeCursorState = function (model, helper) {
            var inverseEditOperations = helper.getInverseEditOperations();
            if (inverseEditOperations.length === 2) {
                var startTokenEditOperation = inverseEditOperations[0];
                var endTokenEditOperation = inverseEditOperations[1];
                return selection_1.Selection.createSelection(startTokenEditOperation.range.endLineNumber, startTokenEditOperation.range.endColumn, endTokenEditOperation.range.startLineNumber, endTokenEditOperation.range.startColumn);
            }
            else {
                var srcRange = inverseEditOperations[0].range;
                var deltaColumn = this._usedEndToken ? -this._usedEndToken.length : 0;
                return selection_1.Selection.createSelection(srcRange.endLineNumber, srcRange.endColumn + deltaColumn, srcRange.endLineNumber, srcRange.endColumn + deltaColumn);
            }
        };
        return BlockCommentCommand;
    }());
    exports.BlockCommentCommand = BlockCommentCommand;
});

define("vs/editor/contrib/comment/common/lineCommentCommand", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/core/editOperation', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/core/selection', './blockCommentCommand'], function (require, exports, strings, editOperation_1, position_1, range_1, selection_1, blockCommentCommand_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (Type) {
        Type[Type["Toggle"] = 0] = "Toggle";
        Type[Type["ForceAdd"] = 1] = "ForceAdd";
        Type[Type["ForceRemove"] = 2] = "ForceRemove";
    })(exports.Type || (exports.Type = {}));
    var Type = exports.Type;
    var LineCommentCommand = (function () {
        function LineCommentCommand(selection, tabSize, type) {
            this._selection = selection;
            this._tabSize = tabSize;
            this._type = type;
            this._deltaColumn = 0;
        }
        /**
         * Do an initial pass over the lines and gather info about the line comment string.
         * Returns null if any of the lines doesn't support a line comment string.
         */
        LineCommentCommand._gatherPreflightCommentStrings = function (model, startLineNumber, endLineNumber) {
            var lines = [], config, commentStr, seenModes = Object.create(null), i, lineCount, lineNumber, mode, modeId;
            for (i = 0, lineCount = endLineNumber - startLineNumber + 1; i < lineCount; i++) {
                lineNumber = startLineNumber + i;
                mode = model.getModeAtPosition(lineNumber, 1);
                modeId = mode.getId();
                // Find the commentStr for this line, if none is found then bail out: we cannot do line comments
                if (seenModes[modeId]) {
                    commentStr = seenModes[modeId];
                }
                else {
                    config = (mode.richEditSupport ? mode.richEditSupport.comments : null);
                    commentStr = (config ? config.lineCommentToken : null);
                    if (!commentStr) {
                        // Mode does not support line comments
                        return null;
                    }
                    seenModes[modeId] = commentStr;
                }
                lines.push({
                    ignore: false,
                    commentStr: commentStr,
                    commentStrOffset: 0,
                    commentStrLength: commentStr.length
                });
            }
            return lines;
        };
        /**
         * Analyze lines and decide which lines are relevant and what the toggle should do.
         * Also, build up several offsets and lengths useful in the generation of editor operations.
         */
        LineCommentCommand._analyzeLines = function (type, model, lines, startLineNumber) {
            var lineData, lineContentStartOffset, commentStrEndOffset, i, lineCount, lineNumber, shouldRemoveComments, lineContent, _space = ' '.charCodeAt(0), onlyWhitespaceLines = true;
            if (type === Type.Toggle) {
                shouldRemoveComments = true;
            }
            else if (type === Type.ForceAdd) {
                shouldRemoveComments = false;
            }
            else {
                shouldRemoveComments = true;
            }
            for (i = 0, lineCount = lines.length; i < lineCount; i++) {
                lineData = lines[i];
                lineNumber = startLineNumber + i;
                lineContent = model.getLineContent(lineNumber);
                lineContentStartOffset = strings.firstNonWhitespaceIndex(lineContent);
                if (lineContentStartOffset === -1) {
                    // Empty or whitespace only line
                    if (type === Type.Toggle) {
                        lineData.ignore = true;
                    }
                    else if (type === Type.ForceAdd) {
                        lineData.ignore = true;
                    }
                    else {
                        lineData.ignore = true;
                    }
                    lineData.commentStrOffset = lineContent.length;
                    continue;
                }
                onlyWhitespaceLines = false;
                lineData.ignore = false;
                lineData.commentStrOffset = lineContentStartOffset;
                if (shouldRemoveComments && !blockCommentCommand_1.BlockCommentCommand._haystackHasNeedleAtOffset(lineContent, lineData.commentStr, lineContentStartOffset)) {
                    if (type === Type.Toggle) {
                        // Every line so far has been a line comment, but this one is not
                        shouldRemoveComments = false;
                    }
                    else if (type === Type.ForceAdd) {
                    }
                    else {
                        lineData.ignore = true;
                    }
                }
                if (shouldRemoveComments) {
                    commentStrEndOffset = lineContentStartOffset + lineData.commentStrLength;
                    if (commentStrEndOffset < lineContent.length && lineContent.charCodeAt(commentStrEndOffset) === _space) {
                        lineData.commentStrLength += 1;
                    }
                }
            }
            if (type === Type.Toggle && onlyWhitespaceLines) {
                // For only whitespace lines, we insert comments
                shouldRemoveComments = false;
                // Also, no longer ignore them
                for (i = 0, lineCount = lines.length; i < lineCount; i++) {
                    lines[i].ignore = false;
                }
            }
            return {
                supported: true,
                shouldRemoveComments: shouldRemoveComments,
                lines: lines
            };
        };
        /**
         * Analyze all lines and decide exactly what to do => not supported | insert line comments | remove line comments
         */
        LineCommentCommand._gatherPreflightData = function (type, model, startLineNumber, endLineNumber) {
            var lines = LineCommentCommand._gatherPreflightCommentStrings(model, startLineNumber, endLineNumber);
            if (lines === null) {
                return {
                    supported: false,
                    shouldRemoveComments: false,
                    lines: null
                };
            }
            return LineCommentCommand._analyzeLines(type, model, lines, startLineNumber);
        };
        /**
         * Given a successful analysis, execute either insert line comments, either remove line comments
         */
        LineCommentCommand.prototype._executeLineComments = function (model, builder, data, s) {
            var ops;
            if (data.shouldRemoveComments) {
                ops = LineCommentCommand._createRemoveLineCommentsOperations(data.lines, s.startLineNumber);
            }
            else {
                LineCommentCommand._normalizeInsertionPoint(model, data.lines, s.startLineNumber, this._tabSize);
                ops = LineCommentCommand._createAddLineCommentsOperations(data.lines, s.startLineNumber);
            }
            var cursorPosition = new position_1.Position(s.positionLineNumber, s.positionColumn);
            for (var i = 0, len = ops.length; i < len; i++) {
                builder.addEditOperation(ops[i].range, ops[i].text);
                if (ops[i].range.isEmpty() && ops[i].range.getStartPosition().equals(cursorPosition)) {
                    this._deltaColumn = ops[i].text.length;
                }
            }
            this._selectionId = builder.trackSelection(s);
        };
        LineCommentCommand.prototype._attemptRemoveBlockComment = function (model, s, startToken, endToken) {
            var startLineNumber = s.startLineNumber;
            var endLineNumber = s.endLineNumber;
            var startTokenAllowedBeforeColumn = endToken.length + Math.max(model.getLineFirstNonWhitespaceColumn(s.startLineNumber), s.startColumn);
            var startTokenIndex = model.getLineContent(startLineNumber).lastIndexOf(startToken, startTokenAllowedBeforeColumn - 1);
            var endTokenIndex = model.getLineContent(endLineNumber).indexOf(endToken, s.endColumn - 1 - startToken.length);
            if (startTokenIndex !== -1 && endTokenIndex === -1) {
                endTokenIndex = model.getLineContent(startLineNumber).indexOf(endToken, startTokenIndex + startToken.length);
                endLineNumber = startLineNumber;
            }
            if (startTokenIndex === -1 && endTokenIndex !== -1) {
                startTokenIndex = model.getLineContent(endLineNumber).lastIndexOf(startToken, endTokenIndex);
                startLineNumber = endLineNumber;
            }
            if (s.isEmpty() && (startTokenIndex === -1 || endTokenIndex === -1)) {
                startTokenIndex = model.getLineContent(startLineNumber).indexOf(startToken);
                if (startTokenIndex !== -1) {
                    endTokenIndex = model.getLineContent(startLineNumber).indexOf(endToken, startTokenIndex + startToken.length);
                }
            }
            if (startTokenIndex !== -1 && endTokenIndex !== -1) {
                return blockCommentCommand_1.BlockCommentCommand._createRemoveBlockCommentOperations({
                    startLineNumber: startLineNumber,
                    startColumn: startTokenIndex + startToken.length + 1,
                    endLineNumber: endLineNumber,
                    endColumn: endTokenIndex + 1
                }, startToken, endToken);
            }
            return null;
        };
        /**
         * Given an unsuccessful analysis, delegate to the block comment command
         */
        LineCommentCommand.prototype._executeBlockComment = function (model, builder, s) {
            var richEditSupport = model.getModeAtPosition(s.startLineNumber, s.startColumn).richEditSupport;
            var config = richEditSupport ? richEditSupport.comments : null;
            if (!config || !config.blockCommentStartToken || !config.blockCommentEndToken) {
                // Mode does not support block comments
                return;
            }
            var startToken = config.blockCommentStartToken;
            var endToken = config.blockCommentEndToken;
            var ops = this._attemptRemoveBlockComment(model, s, startToken, endToken);
            if (!ops) {
                if (s.isEmpty()) {
                    var lineContent = model.getLineContent(s.startLineNumber);
                    var firstNonWhitespaceIndex = strings.firstNonWhitespaceIndex(lineContent);
                    if (firstNonWhitespaceIndex === -1) {
                        // Line is empty or contains only whitespace
                        firstNonWhitespaceIndex = lineContent.length;
                    }
                    ops = blockCommentCommand_1.BlockCommentCommand._createAddBlockCommentOperations({
                        startLineNumber: s.startLineNumber,
                        startColumn: firstNonWhitespaceIndex + 1,
                        endLineNumber: s.startLineNumber,
                        endColumn: lineContent.length + 1
                    }, startToken, endToken);
                }
                else {
                    ops = blockCommentCommand_1.BlockCommentCommand._createAddBlockCommentOperations({
                        startLineNumber: s.startLineNumber,
                        startColumn: model.getLineFirstNonWhitespaceColumn(s.startLineNumber),
                        endLineNumber: s.endLineNumber,
                        endColumn: model.getLineMaxColumn(s.endLineNumber)
                    }, startToken, endToken);
                }
                if (ops.length === 1) {
                    this._deltaColumn = startToken.length;
                }
            }
            this._selectionId = builder.trackSelection(s);
            for (var i = 0; i < ops.length; i++) {
                builder.addEditOperation(ops[i].range, ops[i].text);
            }
        };
        LineCommentCommand.prototype.getEditOperations = function (model, builder) {
            var s = this._selection;
            this._moveEndPositionDown = false;
            if (s.startLineNumber < s.endLineNumber && s.endColumn === 1) {
                this._moveEndPositionDown = true;
                s = s.setEndPosition(s.endLineNumber - 1, model.getLineMaxColumn(s.endLineNumber - 1));
            }
            var data = LineCommentCommand._gatherPreflightData(this._type, model, s.startLineNumber, s.endLineNumber);
            if (data.supported) {
                return this._executeLineComments(model, builder, data, s);
            }
            return this._executeBlockComment(model, builder, s);
        };
        LineCommentCommand.prototype.computeCursorState = function (model, helper) {
            var result = helper.getTrackedSelection(this._selectionId);
            if (this._moveEndPositionDown) {
                result = result.setEndPosition(result.endLineNumber + 1, 1);
            }
            return selection_1.Selection.createSelection(result.startLineNumber, result.startColumn + this._deltaColumn, result.endLineNumber, result.endColumn + this._deltaColumn);
        };
        /**
         * Generate edit operations in the remove line comment case
         */
        LineCommentCommand._createRemoveLineCommentsOperations = function (lines, startLineNumber) {
            var i, len, lineData, res = [];
            for (i = 0, len = lines.length; i < len; i++) {
                lineData = lines[i];
                if (lineData.ignore) {
                    continue;
                }
                res.push(editOperation_1.EditOperation.delete(new range_1.Range(startLineNumber + i, lineData.commentStrOffset + 1, startLineNumber + i, lineData.commentStrOffset + lineData.commentStrLength + 1)));
            }
            return res;
        };
        /**
         * Generate edit operations in the add line comment case
         */
        LineCommentCommand._createAddLineCommentsOperations = function (lines, startLineNumber) {
            var i, len, lineData, res = [];
            for (i = 0, len = lines.length; i < len; i++) {
                lineData = lines[i];
                if (lineData.ignore) {
                    continue;
                }
                res.push(editOperation_1.EditOperation.insert(new position_1.Position(startLineNumber + i, lineData.commentStrOffset + 1), lineData.commentStr + ' '));
            }
            return res;
        };
        // TODO@Alex -> duplicated in characterHardWrappingLineMapper
        LineCommentCommand.nextVisibleColumn = function (currentVisibleColumn, tabSize, isTab, columnSize) {
            if (isTab) {
                return currentVisibleColumn + (tabSize - (currentVisibleColumn % tabSize));
            }
            return currentVisibleColumn + columnSize;
        };
        /**
         * Adjust insertion points to have them vertically aligned in the add line comment case
         */
        LineCommentCommand._normalizeInsertionPoint = function (model, lines, startLineNumber, tabSize) {
            var minVisibleColumn = Number.MAX_VALUE, i, len, lineContent, j, lenJ, currentVisibleColumn, _tab = '\t'.charCodeAt(0);
            for (i = 0, len = lines.length; i < len; i++) {
                if (lines[i].ignore) {
                    continue;
                }
                lineContent = model.getLineContent(startLineNumber + i);
                currentVisibleColumn = 0;
                for (j = 0, lenJ = lines[i].commentStrOffset; currentVisibleColumn < minVisibleColumn && j < lenJ; j++) {
                    currentVisibleColumn = LineCommentCommand.nextVisibleColumn(currentVisibleColumn, tabSize, lineContent.charCodeAt(j) === _tab, 1);
                }
                if (currentVisibleColumn < minVisibleColumn) {
                    minVisibleColumn = currentVisibleColumn;
                }
            }
            minVisibleColumn = Math.floor(minVisibleColumn / tabSize) * tabSize;
            for (i = 0, len = lines.length; i < len; i++) {
                if (lines[i].ignore) {
                    continue;
                }
                lineContent = model.getLineContent(startLineNumber + i);
                currentVisibleColumn = 0;
                for (j = 0, lenJ = lines[i].commentStrOffset; currentVisibleColumn < minVisibleColumn && j < lenJ; j++) {
                    currentVisibleColumn = LineCommentCommand.nextVisibleColumn(currentVisibleColumn, tabSize, lineContent.charCodeAt(j) === _tab, 1);
                }
                if (currentVisibleColumn > minVisibleColumn) {
                    lines[i].commentStrOffset = j - 1;
                }
                else {
                    lines[i].commentStrOffset = j;
                }
            }
        };
        return LineCommentCommand;
    }());
    exports.LineCommentCommand = LineCommentCommand;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/diffNavigator/common/diffNavigator", ["require", "exports", 'vs/base/common/assert', 'vs/base/common/eventEmitter', 'vs/base/common/objects', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon'], function (require, exports, assert, eventEmitter_1, objects, range_1, editorCommon_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var defaultOptions = {
        followsCaret: true,
        ignoreCharChanges: true,
        alwaysRevealFirst: true
    };
    /**
     * Create a new diff navigator for the provided diff editor.
     */
    var DiffNavigator = (function (_super) {
        __extends(DiffNavigator, _super);
        function DiffNavigator(editor, options) {
            var _this = this;
            if (options === void 0) { options = {}; }
            _super.call(this, [
                DiffNavigator.Events.UPDATED
            ]);
            this.editor = editor;
            this.options = objects.mixin(options, defaultOptions, false);
            this.disposed = false;
            this.toUnbind = [];
            this.nextIdx = -1;
            this.ranges = [];
            this.ignoreSelectionChange = false;
            this.revealFirst = this.options.alwaysRevealFirst;
            // hook up to diff editor for diff, disposal, and caret move
            this.toUnbind.push(this.editor.addListener(editorCommon_1.EventType.Disposed, function () { return _this.dispose(); }));
            this.toUnbind.push(this.editor.addListener(editorCommon_1.EventType.DiffUpdated, function () { return _this.onDiffUpdated(); }));
            if (this.options.followsCaret) {
                this.toUnbind.push(this.editor.getModifiedEditor().addListener(editorCommon_1.EventType.CursorPositionChanged, function (e) {
                    if (_this.ignoreSelectionChange) {
                        return;
                    }
                    _this.nextIdx = -1;
                }));
            }
            if (this.options.alwaysRevealFirst) {
                this.toUnbind.push(this.editor.getModifiedEditor().addListener(editorCommon_1.EventType.ModelChanged, function (e) {
                    _this.revealFirst = true;
                }));
            }
            // init things
            this.init();
        }
        DiffNavigator.prototype.init = function () {
            var changes = this.editor.getLineChanges();
            if (!changes) {
                return;
            }
        };
        DiffNavigator.prototype.onDiffUpdated = function () {
            this.init();
            this.compute(this.editor.getLineChanges());
            if (this.revealFirst) {
                // Only reveal first on first non-null changes
                if (this.editor.getLineChanges() !== null) {
                    this.revealFirst = false;
                    this.nextIdx = -1;
                    this.next();
                }
            }
        };
        DiffNavigator.prototype.compute = function (lineChanges) {
            var _this = this;
            // new ranges
            this.ranges = [];
            if (lineChanges) {
                // create ranges from changes
                lineChanges.forEach(function (lineChange) {
                    if (!_this.options.ignoreCharChanges && lineChange.charChanges) {
                        lineChange.charChanges.forEach(function (charChange) {
                            _this.ranges.push({
                                rhs: true,
                                range: new range_1.Range(charChange.modifiedStartLineNumber, charChange.modifiedStartColumn, charChange.modifiedEndLineNumber, charChange.modifiedEndColumn)
                            });
                        });
                    }
                    else {
                        _this.ranges.push({
                            rhs: true,
                            range: new range_1.Range(lineChange.modifiedStartLineNumber, 1, lineChange.modifiedStartLineNumber, 1)
                        });
                    }
                });
            }
            // sort
            this.ranges.sort(function (left, right) {
                if (left.range.getStartPosition().isBeforeOrEqual(right.range.getStartPosition())) {
                    return -1;
                }
                else if (right.range.getStartPosition().isBeforeOrEqual(left.range.getStartPosition())) {
                    return 1;
                }
                else {
                    return 0;
                }
            });
            this.emit(DiffNavigator.Events.UPDATED, {});
        };
        DiffNavigator.prototype.initIdx = function (fwd) {
            var found = false;
            var position = this.editor.getPosition();
            for (var i = 0, len = this.ranges.length; i < len && !found; i++) {
                var range = this.ranges[i].range;
                if (position.isBeforeOrEqual(range.getStartPosition())) {
                    this.nextIdx = i + (fwd ? 0 : -1);
                    found = true;
                }
            }
            if (!found) {
                // after the last change
                this.nextIdx = fwd ? 0 : this.ranges.length - 1;
            }
            if (this.nextIdx < 0) {
                this.nextIdx = this.ranges.length - 1;
            }
        };
        DiffNavigator.prototype.move = function (fwd) {
            assert.ok(!this.disposed, 'Illegal State - diff navigator has been disposed');
            if (!this.canNavigate()) {
                return;
            }
            if (this.nextIdx === -1) {
                this.initIdx(fwd);
            }
            else if (fwd) {
                this.nextIdx += 1;
                if (this.nextIdx >= this.ranges.length) {
                    this.nextIdx = 0;
                }
            }
            else {
                this.nextIdx -= 1;
                if (this.nextIdx < 0) {
                    this.nextIdx = this.ranges.length - 1;
                }
            }
            var info = this.ranges[this.nextIdx];
            this.ignoreSelectionChange = true;
            try {
                var pos = info.range.getStartPosition();
                this.editor.setPosition(pos);
                this.editor.revealPositionInCenter(pos);
            }
            finally {
                this.ignoreSelectionChange = false;
            }
        };
        DiffNavigator.prototype.canNavigate = function () {
            return this.ranges && this.ranges.length > 0;
        };
        DiffNavigator.prototype.next = function () {
            this.move(true);
        };
        DiffNavigator.prototype.previous = function () {
            this.move(false);
        };
        DiffNavigator.prototype.dispose = function () {
            while (this.toUnbind.length > 0) {
                this.toUnbind.pop()();
            }
            this.ranges = null;
            this.disposed = true;
            _super.prototype.dispose.call(this);
        };
        DiffNavigator.Events = {
            UPDATED: 'navigation.updated'
        };
        return DiffNavigator;
    }(eventEmitter_1.EventEmitter));
    exports.DiffNavigator = DiffNavigator;
});

define("vs/editor/contrib/find/common/findDecorations", ["require", "exports", 'vs/editor/common/editorCommon'], function (require, exports, editorCommon) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FindDecorations = (function () {
        function FindDecorations(editor) {
            this._editor = editor;
            this._decorations = [];
            this._findScopeDecorationId = null;
            this._highlightedDecorationId = null;
            this._startPosition = this._editor.getPosition();
        }
        FindDecorations.prototype.dispose = function () {
            this._editor.deltaDecorations(this._allDecorations(), []);
            this._editor = null;
            this._decorations = [];
            this._findScopeDecorationId = null;
            this._highlightedDecorationId = null;
            this._startPosition = null;
        };
        FindDecorations.prototype.reset = function () {
            this._decorations = [];
            this._findScopeDecorationId = null;
            this._highlightedDecorationId = null;
        };
        FindDecorations.prototype.getCount = function () {
            return this._decorations.length;
        };
        FindDecorations.prototype.getFindScope = function () {
            if (this._findScopeDecorationId) {
                return this._editor.getModel().getDecorationRange(this._findScopeDecorationId);
            }
            return null;
        };
        FindDecorations.prototype.getStartPosition = function () {
            return this._startPosition;
        };
        FindDecorations.prototype.setStartPosition = function (newStartPosition) {
            this._startPosition = newStartPosition;
            this.setCurrentFindMatch(null);
        };
        FindDecorations.prototype.getCurrentMatchesPosition = function (desiredRange) {
            for (var i = 0, len = this._decorations.length; i < len; i++) {
                var range = this._editor.getModel().getDecorationRange(this._decorations[i]);
                if (desiredRange.equalsRange(range)) {
                    return (i + 1);
                }
            }
            return 1;
        };
        FindDecorations.prototype.setCurrentFindMatch = function (nextMatch) {
            var _this = this;
            var newCurrentDecorationId = null;
            var matchPosition = 0;
            if (nextMatch) {
                for (var i = 0, len = this._decorations.length; i < len; i++) {
                    var range = this._editor.getModel().getDecorationRange(this._decorations[i]);
                    if (nextMatch.equalsRange(range)) {
                        newCurrentDecorationId = this._decorations[i];
                        matchPosition = (i + 1);
                        break;
                    }
                }
            }
            if (this._highlightedDecorationId !== null || newCurrentDecorationId !== null) {
                this._editor.changeDecorations(function (changeAccessor) {
                    if (_this._highlightedDecorationId !== null) {
                        changeAccessor.changeDecorationOptions(_this._highlightedDecorationId, FindDecorations.createFindMatchDecorationOptions(false));
                        _this._highlightedDecorationId = null;
                    }
                    if (newCurrentDecorationId !== null) {
                        _this._highlightedDecorationId = newCurrentDecorationId;
                        changeAccessor.changeDecorationOptions(_this._highlightedDecorationId, FindDecorations.createFindMatchDecorationOptions(true));
                    }
                });
            }
            return matchPosition;
        };
        FindDecorations.prototype.set = function (matches, findScope) {
            var newDecorations = matches.map(function (match) {
                return {
                    range: match,
                    options: FindDecorations.createFindMatchDecorationOptions(false)
                };
            });
            if (findScope) {
                newDecorations.unshift({
                    range: findScope,
                    options: FindDecorations.createFindScopeDecorationOptions()
                });
            }
            var tmpDecorations = this._editor.deltaDecorations(this._allDecorations(), newDecorations);
            if (findScope) {
                this._findScopeDecorationId = tmpDecorations.shift();
            }
            else {
                this._findScopeDecorationId = null;
            }
            this._decorations = tmpDecorations;
            this._highlightedDecorationId = null;
        };
        FindDecorations.prototype._allDecorations = function () {
            var result = [];
            result = result.concat(this._decorations);
            if (this._findScopeDecorationId) {
                result.push(this._findScopeDecorationId);
            }
            return result;
        };
        FindDecorations.createFindMatchDecorationOptions = function (isCurrent) {
            return {
                stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
                className: isCurrent ? 'currentFindMatch' : 'findMatch',
                overviewRuler: {
                    color: 'rgba(246, 185, 77, 0.7)',
                    darkColor: 'rgba(246, 185, 77, 0.7)',
                    position: editorCommon.OverviewRulerLane.Center
                }
            };
        };
        FindDecorations.createFindScopeDecorationOptions = function () {
            return {
                className: 'findScope',
                isWholeLine: true
            };
        };
        return FindDecorations;
    }());
    exports.FindDecorations = FindDecorations;
});

define("vs/editor/contrib/find/common/findState", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/editor/common/core/range'], function (require, exports, eventEmitter_1, range_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FindReplaceState = (function () {
        function FindReplaceState() {
            this._searchString = '';
            this._replaceString = '';
            this._isRevealed = false;
            this._isReplaceRevealed = false;
            this._isRegex = false;
            this._wholeWord = false;
            this._matchCase = false;
            this._searchScope = null;
            this._matchesPosition = 0;
            this._matchesCount = 0;
            this._eventEmitter = new eventEmitter_1.EventEmitter();
        }
        Object.defineProperty(FindReplaceState.prototype, "searchString", {
            get: function () { return this._searchString; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FindReplaceState.prototype, "replaceString", {
            get: function () { return this._replaceString; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FindReplaceState.prototype, "isRevealed", {
            get: function () { return this._isRevealed; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FindReplaceState.prototype, "isReplaceRevealed", {
            get: function () { return this._isReplaceRevealed; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FindReplaceState.prototype, "isRegex", {
            get: function () { return this._isRegex; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FindReplaceState.prototype, "wholeWord", {
            get: function () { return this._wholeWord; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FindReplaceState.prototype, "matchCase", {
            get: function () { return this._matchCase; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FindReplaceState.prototype, "searchScope", {
            get: function () { return this._searchScope; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FindReplaceState.prototype, "matchesPosition", {
            get: function () { return this._matchesPosition; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FindReplaceState.prototype, "matchesCount", {
            get: function () { return this._matchesCount; },
            enumerable: true,
            configurable: true
        });
        FindReplaceState.prototype.dispose = function () {
            this._eventEmitter.dispose();
        };
        FindReplaceState.prototype.addChangeListener = function (listener) {
            return this._eventEmitter.addListener2(FindReplaceState._CHANGED_EVENT, listener);
        };
        FindReplaceState.prototype.changeMatchInfo = function (matchesPosition, matchesCount) {
            var changeEvent = {
                moveCursor: false,
                searchString: false,
                replaceString: false,
                isRevealed: false,
                isReplaceRevealed: false,
                isRegex: false,
                wholeWord: false,
                matchCase: false,
                searchScope: false,
                matchesPosition: false,
                matchesCount: false
            };
            var somethingChanged = false;
            if (matchesCount === 0) {
                matchesPosition = 0;
            }
            if (matchesPosition > matchesCount) {
                matchesPosition = matchesCount;
            }
            if (this._matchesPosition !== matchesPosition) {
                this._matchesPosition = matchesPosition;
                changeEvent.matchesPosition = true;
                somethingChanged = true;
            }
            if (this._matchesCount !== matchesCount) {
                this._matchesCount = matchesCount;
                changeEvent.matchesCount = true;
                somethingChanged = true;
            }
            if (somethingChanged) {
                this._eventEmitter.emit(FindReplaceState._CHANGED_EVENT, changeEvent);
            }
        };
        FindReplaceState.prototype.change = function (newState, moveCursor) {
            var changeEvent = {
                moveCursor: moveCursor,
                searchString: false,
                replaceString: false,
                isRevealed: false,
                isReplaceRevealed: false,
                isRegex: false,
                wholeWord: false,
                matchCase: false,
                searchScope: false,
                matchesPosition: false,
                matchesCount: false
            };
            var somethingChanged = false;
            if (typeof newState.searchString !== 'undefined') {
                if (this._searchString !== newState.searchString) {
                    this._searchString = newState.searchString;
                    changeEvent.searchString = true;
                    somethingChanged = true;
                }
            }
            if (typeof newState.replaceString !== 'undefined') {
                if (this._replaceString !== newState.replaceString) {
                    this._replaceString = newState.replaceString;
                    changeEvent.replaceString = true;
                    somethingChanged = true;
                }
            }
            if (typeof newState.isRevealed !== 'undefined') {
                if (this._isRevealed !== newState.isRevealed) {
                    this._isRevealed = newState.isRevealed;
                    changeEvent.isRevealed = true;
                    somethingChanged = true;
                }
            }
            if (typeof newState.isReplaceRevealed !== 'undefined') {
                if (this._isReplaceRevealed !== newState.isReplaceRevealed) {
                    this._isReplaceRevealed = newState.isReplaceRevealed;
                    changeEvent.isReplaceRevealed = true;
                    somethingChanged = true;
                }
            }
            if (typeof newState.isRegex !== 'undefined') {
                if (this._isRegex !== newState.isRegex) {
                    this._isRegex = newState.isRegex;
                    changeEvent.isRegex = true;
                    somethingChanged = true;
                }
            }
            if (typeof newState.wholeWord !== 'undefined') {
                if (this._wholeWord !== newState.wholeWord) {
                    this._wholeWord = newState.wholeWord;
                    changeEvent.wholeWord = true;
                    somethingChanged = true;
                }
            }
            if (typeof newState.matchCase !== 'undefined') {
                if (this._matchCase !== newState.matchCase) {
                    this._matchCase = newState.matchCase;
                    changeEvent.matchCase = true;
                    somethingChanged = true;
                }
            }
            if (typeof newState.searchScope !== 'undefined') {
                if (!range_1.Range.equalsRange(this._searchScope, newState.searchScope)) {
                    this._searchScope = newState.searchScope;
                    changeEvent.searchScope = true;
                    somethingChanged = true;
                }
            }
            if (somethingChanged) {
                this._eventEmitter.emit(FindReplaceState._CHANGED_EVENT, changeEvent);
            }
        };
        FindReplaceState._CHANGED_EVENT = 'changed';
        return FindReplaceState;
    }());
    exports.FindReplaceState = FindReplaceState;
});

define("vs/editor/contrib/find/common/replaceAllCommand", ["require", "exports", 'vs/editor/common/core/range', 'vs/editor/common/core/selection'], function (require, exports, range_1, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ReplaceAllCommand = (function () {
        function ReplaceAllCommand(ranges, replaceStrings) {
            this._ranges = ranges;
            this._replaceStrings = replaceStrings;
        }
        ReplaceAllCommand.prototype.getEditOperations = function (model, builder) {
            if (this._ranges.length > 0) {
                // Collect all edit operations
                var ops = [];
                for (var i = 0; i < this._ranges.length; i++) {
                    ops.push({
                        range: this._ranges[i],
                        text: this._replaceStrings[i]
                    });
                }
                // Sort them in ascending order by range starts
                ops.sort(function (o1, o2) {
                    return range_1.Range.compareRangesUsingStarts(o1.range, o2.range);
                });
                // Merge operations that touch each other
                var resultOps = [];
                var previousOp = ops[0];
                for (var i = 1; i < ops.length; i++) {
                    if (previousOp.range.endLineNumber === ops[i].range.startLineNumber && previousOp.range.endColumn === ops[i].range.startColumn) {
                        // These operations are one after another and can be merged
                        previousOp.range = previousOp.range.plusRange(ops[i].range);
                        previousOp.text = previousOp.text + ops[i].text;
                    }
                    else {
                        resultOps.push(previousOp);
                        previousOp = ops[i];
                    }
                }
                resultOps.push(previousOp);
                for (var i = 0; i < resultOps.length; i++) {
                    builder.addEditOperation(resultOps[i].range, resultOps[i].text);
                }
            }
        };
        ReplaceAllCommand.prototype.computeCursorState = function (model, helper) {
            var inverseEditOperations = helper.getInverseEditOperations();
            var srcRange = inverseEditOperations[inverseEditOperations.length - 1].range;
            return selection_1.Selection.createSelection(srcRange.endLineNumber, srcRange.endColumn, srcRange.endLineNumber, srcRange.endColumn);
        };
        return ReplaceAllCommand;
    }());
    exports.ReplaceAllCommand = ReplaceAllCommand;
});

define("vs/editor/contrib/find/common/findModel", ["require", "exports", 'vs/base/common/async', 'vs/base/common/lifecycle', 'vs/base/common/strings', 'vs/editor/common/commands/replaceCommand', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', './findDecorations', './replaceAllCommand'], function (require, exports, async_1, lifecycle_1, strings, replaceCommand_1, position_1, range_1, editorCommon, findDecorations_1, replaceAllCommand_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.FIND_IDS = {
        StartFindAction: 'actions.find',
        NextMatchFindAction: 'editor.action.nextMatchFindAction',
        PreviousMatchFindAction: 'editor.action.previousMatchFindAction',
        NextSelectionMatchFindAction: 'editor.action.nextSelectionMatchFindAction',
        PreviousSelectionMatchFindAction: 'editor.action.previousSelectionMatchFindAction',
        AddSelectionToNextFindMatchAction: 'editor.action.addSelectionToNextFindMatch',
        MoveSelectionToNextFindMatchAction: 'editor.action.moveSelectionToNextFindMatch',
        StartFindReplaceAction: 'editor.action.startFindReplaceAction',
        CloseFindWidgetCommand: 'closeFindWidget',
        ToggleCaseSensitiveCommand: 'toggleFindCaseSensitive',
        ToggleWholeWordCommand: 'toggleFindWholeWord',
        ToggleRegexCommand: 'toggleFindRegex',
        ReplaceOneAction: 'editor.action.replaceOne',
        ReplaceAllAction: 'editor.action.replaceAll'
    };
    exports.MATCHES_LIMIT = 999;
    var FindModelBoundToEditorModel = (function () {
        function FindModelBoundToEditorModel(editor, state) {
            var _this = this;
            this._editor = editor;
            this._state = state;
            this._toDispose = [];
            this._decorations = new findDecorations_1.FindDecorations(editor);
            this._toDispose.push(this._decorations);
            this._updateDecorationsScheduler = new async_1.RunOnceScheduler(function () { return _this.research(false); }, 100);
            this._toDispose.push(this._updateDecorationsScheduler);
            this._toDispose.push(this._editor.addListener2(editorCommon.EventType.CursorPositionChanged, function (e) {
                if (e.reason === 'explicit' || e.reason === 'undo' || e.reason === 'redo') {
                    _this._decorations.setStartPosition(_this._editor.getPosition());
                }
            }));
            this._ignoreModelContentChanged = false;
            this._toDispose.push(this._editor.addListener2(editorCommon.EventType.ModelContentChanged, function (e) {
                if (_this._ignoreModelContentChanged) {
                    return;
                }
                if (e.changeType === editorCommon.EventType.ModelContentChangedFlush) {
                    // a model.setValue() was called
                    _this._decorations.reset();
                }
                _this._decorations.setStartPosition(_this._editor.getPosition());
                _this._updateDecorationsScheduler.schedule();
            }));
            this._toDispose.push(this._state.addChangeListener(function (e) { return _this._onStateChanged(e); }));
            this.research(false, this._state.searchScope);
        }
        FindModelBoundToEditorModel.prototype.dispose = function () {
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        FindModelBoundToEditorModel.prototype._onStateChanged = function (e) {
            if (e.searchString || e.isReplaceRevealed || e.isRegex || e.wholeWord || e.matchCase || e.searchScope) {
                if (e.searchScope) {
                    this.research(e.moveCursor, this._state.searchScope);
                }
                else {
                    this.research(e.moveCursor);
                }
            }
        };
        FindModelBoundToEditorModel._getSearchRange = function (model, searchOnlyEditableRange, findScope) {
            var searchRange;
            if (searchOnlyEditableRange) {
                searchRange = model.getEditableRange();
            }
            else {
                searchRange = model.getFullModelRange();
            }
            // If we have set now or before a find scope, use it for computing the search range
            if (findScope) {
                searchRange = searchRange.intersectRanges(findScope);
            }
            return searchRange;
        };
        FindModelBoundToEditorModel.prototype.research = function (moveCursor, newFindScope) {
            var findScope = null;
            if (typeof newFindScope !== 'undefined') {
                findScope = newFindScope;
            }
            else {
                findScope = this._decorations.getFindScope();
            }
            if (findScope !== null) {
                findScope = new range_1.Range(findScope.startLineNumber, 1, findScope.endLineNumber, this._editor.getModel().getLineMaxColumn(findScope.endLineNumber));
            }
            var findMatches = this._findMatches(findScope, exports.MATCHES_LIMIT);
            this._decorations.set(findMatches, findScope);
            this._state.changeMatchInfo(this._decorations.getCurrentMatchesPosition(this._editor.getSelection()), this._decorations.getCount());
            if (moveCursor) {
                this._moveToNextMatch(this._decorations.getStartPosition());
            }
        };
        FindModelBoundToEditorModel.prototype._hasMatches = function () {
            return (this._state.matchesCount > 0);
        };
        FindModelBoundToEditorModel.prototype._cannotFind = function () {
            if (!this._hasMatches()) {
                var findScope = this._decorations.getFindScope();
                if (findScope) {
                    // Reveal the selection so user is reminded that 'selection find' is on.
                    this._editor.revealRangeInCenterIfOutsideViewport(findScope);
                }
                return true;
            }
            return false;
        };
        FindModelBoundToEditorModel.prototype._moveToPrevMatch = function (before, isRecursed) {
            if (isRecursed === void 0) { isRecursed = false; }
            if (this._cannotFind()) {
                return;
            }
            var findScope = this._decorations.getFindScope();
            var searchRange = FindModelBoundToEditorModel._getSearchRange(this._editor.getModel(), this._state.isReplaceRevealed, findScope);
            // ...(----)...|...
            if (searchRange.getEndPosition().isBefore(before)) {
                before = searchRange.getEndPosition();
            }
            // ...|...(----)...
            if (before.isBefore(searchRange.getStartPosition())) {
                before = searchRange.getEndPosition();
            }
            var lineNumber = before.lineNumber, column = before.column;
            var model = this._editor.getModel();
            if (this._state.isRegex) {
                // Force advancing to the previous line if searching for $
                if (this._state.searchString === '$') {
                    if (lineNumber === 1) {
                        lineNumber = model.getLineCount();
                    }
                    else {
                        lineNumber--;
                    }
                    column = model.getLineMaxColumn(lineNumber);
                }
                // Force advancing to the previous line if searching for ^ or ^$ and cursor is at the beginning
                if (this._state.searchString === '^' || this._state.searchString === '^$') {
                    if (column === 1) {
                        if (lineNumber === 1) {
                            lineNumber = model.getLineCount();
                        }
                        else {
                            lineNumber--;
                        }
                        column = model.getLineMaxColumn(lineNumber);
                    }
                }
            }
            var position = new position_1.Position(lineNumber, column);
            var prevMatch = model.findPreviousMatch(this._state.searchString, position, this._state.isRegex, this._state.matchCase, this._state.wholeWord);
            if (!prevMatch) {
                // there is precisely one match and selection is on top of it
                return;
            }
            if (!isRecursed && !searchRange.containsRange(prevMatch)) {
                return this._moveToPrevMatch(prevMatch.getStartPosition(), true);
            }
            var matchesPosition = this._decorations.setCurrentFindMatch(prevMatch);
            this._state.changeMatchInfo(matchesPosition, this._decorations.getCount());
            this._editor.setSelection(prevMatch);
            this._editor.revealRangeInCenterIfOutsideViewport(prevMatch);
        };
        FindModelBoundToEditorModel.prototype.moveToPrevMatch = function () {
            this._moveToPrevMatch(this._editor.getSelection().getStartPosition());
        };
        FindModelBoundToEditorModel.prototype._moveToNextMatch = function (after, isRecursed) {
            if (isRecursed === void 0) { isRecursed = false; }
            if (this._cannotFind()) {
                return;
            }
            var findScope = this._decorations.getFindScope();
            var searchRange = FindModelBoundToEditorModel._getSearchRange(this._editor.getModel(), this._state.isReplaceRevealed, findScope);
            // ...(----)...|...
            if (searchRange.getEndPosition().isBefore(after)) {
                after = searchRange.getStartPosition();
            }
            // ...|...(----)...
            if (after.isBefore(searchRange.getStartPosition())) {
                after = searchRange.getStartPosition();
            }
            var lineNumber = after.lineNumber, column = after.column;
            var model = this._editor.getModel();
            if (this._state.isRegex) {
                // Force advancing to the next line if searching for ^ or ^$
                if (this._state.searchString === '^' || this._state.searchString === '^$') {
                    if (lineNumber === model.getLineCount()) {
                        lineNumber = 1;
                    }
                    else {
                        lineNumber++;
                    }
                    column = 1;
                }
                // Force advancing to the next line if searching for $ and at the end of the line
                if (this._state.searchString === '$') {
                    if (column === model.getLineMaxColumn(lineNumber)) {
                        if (lineNumber === model.getLineCount()) {
                            lineNumber = 1;
                        }
                        else {
                            lineNumber++;
                        }
                        column = 1;
                    }
                }
            }
            var position = new position_1.Position(lineNumber, column);
            var nextMatch = model.findNextMatch(this._state.searchString, position, this._state.isRegex, this._state.matchCase, this._state.wholeWord);
            if (!nextMatch) {
                // there is precisely one match and selection is on top of it
                return;
            }
            if (!isRecursed && !searchRange.containsRange(nextMatch)) {
                return this._moveToNextMatch(nextMatch.getEndPosition(), true);
            }
            var matchesPosition = this._decorations.setCurrentFindMatch(nextMatch);
            this._state.changeMatchInfo(matchesPosition, this._decorations.getCount());
            this._editor.setSelection(nextMatch);
            this._editor.revealRangeInCenterIfOutsideViewport(nextMatch);
        };
        FindModelBoundToEditorModel.prototype.moveToNextMatch = function () {
            this._moveToNextMatch(this._editor.getSelection().getEndPosition());
        };
        FindModelBoundToEditorModel.prototype.getReplaceString = function (matchedString) {
            if (!this._state.isRegex) {
                return this._state.replaceString;
            }
            var regexp = strings.createRegExp(this._state.searchString, this._state.isRegex, this._state.matchCase, this._state.wholeWord, true);
            // Parse the replace string to support that \t or \n mean the right thing
            var parsedReplaceString = parseReplaceString(this._state.replaceString);
            return matchedString.replace(regexp, parsedReplaceString);
        };
        FindModelBoundToEditorModel.prototype._rangeIsMatch = function (range) {
            var selection = this._editor.getSelection();
            var selectionText = this._editor.getModel().getValueInRange(selection);
            var regexp = strings.createSafeRegExp(this._state.searchString, this._state.isRegex, this._state.matchCase, this._state.wholeWord);
            var m = selectionText.match(regexp);
            return (m && m[0].length === selectionText.length);
        };
        FindModelBoundToEditorModel.prototype.replace = function () {
            if (!this._hasMatches()) {
                return;
            }
            var selection = this._editor.getSelection();
            var selectionText = this._editor.getModel().getValueInRange(selection);
            if (this._rangeIsMatch(selection)) {
                // selection sits on a find match => replace it!
                var replaceString = this.getReplaceString(selectionText);
                var command = new replaceCommand_1.ReplaceCommand(selection, replaceString);
                this._executeEditorCommand('replace', command);
                this._decorations.setStartPosition(new position_1.Position(selection.startLineNumber, selection.startColumn + replaceString.length));
                this.research(true);
            }
            else {
                this._decorations.setStartPosition(this._editor.getPosition());
                this.moveToNextMatch();
            }
        };
        FindModelBoundToEditorModel.prototype._findMatches = function (findScope, limitResultCount) {
            var searchRange = FindModelBoundToEditorModel._getSearchRange(this._editor.getModel(), this._state.isReplaceRevealed, findScope);
            return this._editor.getModel().findMatches(this._state.searchString, searchRange, this._state.isRegex, this._state.matchCase, this._state.wholeWord, limitResultCount);
        };
        FindModelBoundToEditorModel.prototype.replaceAll = function () {
            if (!this._hasMatches()) {
                return;
            }
            var model = this._editor.getModel();
            var findScope = this._decorations.getFindScope();
            // Get all the ranges (even more than the highlighted ones)
            var ranges = this._findMatches(findScope, Number.MAX_VALUE);
            var replaceStrings = [];
            for (var i = 0, len = ranges.length; i < len; i++) {
                replaceStrings.push(this.getReplaceString(model.getValueInRange(ranges[i])));
            }
            var command = new replaceAllCommand_1.ReplaceAllCommand(ranges, replaceStrings);
            this._executeEditorCommand('replaceAll', command);
            this.research(false);
        };
        FindModelBoundToEditorModel.prototype._executeEditorCommand = function (source, command) {
            try {
                this._ignoreModelContentChanged = true;
                this._editor.executeCommand(source, command);
            }
            finally {
                this._ignoreModelContentChanged = false;
            }
        };
        return FindModelBoundToEditorModel;
    }());
    exports.FindModelBoundToEditorModel = FindModelBoundToEditorModel;
    var BACKSLASH_CHAR_CODE = '\\'.charCodeAt(0);
    var DOLLAR_CHAR_CODE = '$'.charCodeAt(0);
    var ZERO_CHAR_CODE = '0'.charCodeAt(0);
    var n_CHAR_CODE = 'n'.charCodeAt(0);
    var t_CHAR_CODE = 't'.charCodeAt(0);
    /**
     * \n => LF
     * \t => TAB
     * \\ => \
     * $0 => $& (see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/replace#Specifying_a_string_as_a_parameter)
     * everything else stays untouched
     */
    function parseReplaceString(input) {
        if (!input || input.length === 0) {
            return input;
        }
        var substrFrom = 0, result = '';
        for (var i = 0, len = input.length; i < len; i++) {
            var chCode = input.charCodeAt(i);
            if (chCode === BACKSLASH_CHAR_CODE) {
                // move to next char
                i++;
                if (i >= len) {
                    // string ends with a \
                    break;
                }
                var nextChCode = input.charCodeAt(i);
                var replaceWithCharacter = null;
                switch (nextChCode) {
                    case BACKSLASH_CHAR_CODE:
                        // \\ => \
                        replaceWithCharacter = '\\';
                        break;
                    case n_CHAR_CODE:
                        // \n => LF
                        replaceWithCharacter = '\n';
                        break;
                    case t_CHAR_CODE:
                        // \t => TAB
                        replaceWithCharacter = '\t';
                        break;
                }
                if (replaceWithCharacter) {
                    result += input.substring(substrFrom, i - 1) + replaceWithCharacter;
                    substrFrom = i + 1;
                }
            }
            if (chCode === DOLLAR_CHAR_CODE) {
                // move to next char
                i++;
                if (i >= len) {
                    // string ends with a $
                    break;
                }
                var nextChCode = input.charCodeAt(i);
                var replaceWithCharacter = null;
                switch (nextChCode) {
                    case ZERO_CHAR_CODE:
                        // $0 => $&
                        replaceWithCharacter = '$&';
                        break;
                }
                if (replaceWithCharacter) {
                    result += input.substring(substrFrom, i - 1) + replaceWithCharacter;
                    substrFrom = i + 1;
                }
            }
        }
        if (substrFrom === 0) {
            // no replacement occured
            return input;
        }
        return result + input.substring(substrFrom);
    }
    exports.parseReplaceString = parseReplaceString;
});

define("vs/editor/contrib/hover/browser/hoverOperation", ["require", "exports", 'vs/base/common/async', 'vs/base/common/errors'], function (require, exports, async_1, errors_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ComputeHoverOperationState;
    (function (ComputeHoverOperationState) {
        ComputeHoverOperationState[ComputeHoverOperationState["IDLE"] = 0] = "IDLE";
        ComputeHoverOperationState[ComputeHoverOperationState["FIRST_WAIT"] = 1] = "FIRST_WAIT";
        ComputeHoverOperationState[ComputeHoverOperationState["SECOND_WAIT"] = 2] = "SECOND_WAIT";
        ComputeHoverOperationState[ComputeHoverOperationState["WAITING_FOR_ASYNC_COMPUTATION"] = 3] = "WAITING_FOR_ASYNC_COMPUTATION";
    })(ComputeHoverOperationState || (ComputeHoverOperationState = {}));
    var HoverOperation = (function () {
        function HoverOperation(computer, success, error, progress) {
            var _this = this;
            this._computer = computer;
            this._state = ComputeHoverOperationState.IDLE;
            this._firstWaitScheduler = new async_1.RunOnceScheduler(function () { return _this._triggerAsyncComputation(); }, this._getHoverTimeMillis() / 2);
            this._secondWaitScheduler = new async_1.RunOnceScheduler(function () { return _this._triggerSyncComputation(); }, this._getHoverTimeMillis() / 2);
            this._loadingMessageScheduler = new async_1.RunOnceScheduler(function () { return _this._showLoadingMessage(); }, 3 * this._getHoverTimeMillis());
            this._asyncComputationPromise = null;
            this._asyncComputationPromiseDone = false;
            this._completeCallback = success;
            this._errorCallback = error;
            this._progressCallback = progress;
        }
        HoverOperation.prototype.getComputer = function () {
            return this._computer;
        };
        HoverOperation.prototype._getHoverTimeMillis = function () {
            if (this._computer.getHoverTimeMillis) {
                return this._computer.getHoverTimeMillis();
            }
            return HoverOperation.HOVER_TIME;
        };
        HoverOperation.prototype._triggerAsyncComputation = function () {
            var _this = this;
            this._state = ComputeHoverOperationState.SECOND_WAIT;
            this._secondWaitScheduler.schedule();
            if (this._computer.computeAsync) {
                this._asyncComputationPromiseDone = false;
                this._asyncComputationPromise = this._computer.computeAsync();
                this._asyncComputationPromise.then(function (asyncResult) {
                    _this._asyncComputationPromiseDone = true;
                    _this._withAsyncResult(asyncResult);
                }).done(null, function () { return _this._onError; });
            }
            else {
                this._asyncComputationPromiseDone = true;
            }
        };
        HoverOperation.prototype._triggerSyncComputation = function () {
            if (this._computer.computeSync) {
                this._computer.onResult(this._computer.computeSync(), true);
            }
            if (this._asyncComputationPromiseDone) {
                this._state = ComputeHoverOperationState.IDLE;
                this._onComplete(this._computer.getResult());
            }
            else {
                this._state = ComputeHoverOperationState.WAITING_FOR_ASYNC_COMPUTATION;
                this._onProgress(this._computer.getResult());
            }
        };
        HoverOperation.prototype._showLoadingMessage = function () {
            if (this._state === ComputeHoverOperationState.WAITING_FOR_ASYNC_COMPUTATION) {
                this._onProgress(this._computer.getResultWithLoadingMessage());
            }
        };
        HoverOperation.prototype._withAsyncResult = function (asyncResult) {
            if (asyncResult) {
                this._computer.onResult(asyncResult, false);
            }
            if (this._state === ComputeHoverOperationState.WAITING_FOR_ASYNC_COMPUTATION) {
                this._state = ComputeHoverOperationState.IDLE;
                this._onComplete(this._computer.getResult());
            }
        };
        HoverOperation.prototype._onComplete = function (value) {
            if (this._completeCallback) {
                this._completeCallback(value);
            }
        };
        HoverOperation.prototype._onError = function (error) {
            if (this._errorCallback) {
                this._errorCallback(error);
            }
            else {
                errors_1.onUnexpectedError(error);
            }
        };
        HoverOperation.prototype._onProgress = function (value) {
            if (this._progressCallback) {
                this._progressCallback(value);
            }
        };
        HoverOperation.prototype.start = function () {
            if (this._state === ComputeHoverOperationState.IDLE) {
                this._state = ComputeHoverOperationState.FIRST_WAIT;
                this._firstWaitScheduler.schedule();
                this._loadingMessageScheduler.schedule();
            }
        };
        HoverOperation.prototype.cancel = function () {
            this._loadingMessageScheduler.cancel();
            if (this._state === ComputeHoverOperationState.FIRST_WAIT) {
                this._firstWaitScheduler.cancel();
            }
            if (this._state === ComputeHoverOperationState.SECOND_WAIT) {
                this._secondWaitScheduler.cancel();
                if (this._asyncComputationPromise) {
                    this._asyncComputationPromise.cancel();
                }
            }
            if (this._state === ComputeHoverOperationState.WAITING_FOR_ASYNC_COMPUTATION) {
                if (this._asyncComputationPromise) {
                    this._asyncComputationPromise.cancel();
                }
            }
            this._state = ComputeHoverOperationState.IDLE;
        };
        HoverOperation.HOVER_TIME = 300;
        return HoverOperation;
    }());
    exports.HoverOperation = HoverOperation;
});

define("vs/editor/contrib/inPlaceReplace/common/inPlaceReplaceCommand", ["require", "exports", 'vs/editor/common/core/selection'], function (require, exports, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var InPlaceReplaceCommand = (function () {
        function InPlaceReplaceCommand(editRange, originalSelection, text) {
            this._editRange = editRange;
            this._originalSelection = originalSelection;
            this._text = text;
        }
        InPlaceReplaceCommand.prototype.getEditOperations = function (model, builder) {
            builder.addEditOperation(this._editRange, this._text);
        };
        InPlaceReplaceCommand.prototype.computeCursorState = function (model, helper) {
            var inverseEditOperations = helper.getInverseEditOperations();
            var srcRange = inverseEditOperations[0].range;
            if (!this._originalSelection.isEmpty()) {
                // Preserve selection and extends to typed text
                return selection_1.Selection.createSelection(srcRange.endLineNumber, srcRange.endColumn - this._text.length, srcRange.endLineNumber, srcRange.endColumn);
            }
            return selection_1.Selection.createSelection(srcRange.endLineNumber, Math.min(this._originalSelection.positionColumn, srcRange.endColumn), srcRange.endLineNumber, Math.min(this._originalSelection.positionColumn, srcRange.endColumn));
        };
        return InPlaceReplaceCommand;
    }());
    exports.InPlaceReplaceCommand = InPlaceReplaceCommand;
});

define("vs/editor/contrib/linesOperations/common/copyLinesCommand", ["require", "exports", 'vs/editor/common/core/range', 'vs/editor/common/core/selection'], function (require, exports, range_1, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var CopyLinesCommand = (function () {
        function CopyLinesCommand(selection, isCopyingDown) {
            this._selection = selection;
            this._isCopyingDown = isCopyingDown;
        }
        CopyLinesCommand.prototype.getEditOperations = function (model, builder) {
            var s = this._selection;
            this._startLineNumberDelta = 0;
            this._endLineNumberDelta = 0;
            if (s.startLineNumber < s.endLineNumber && s.endColumn === 1) {
                this._endLineNumberDelta = 1;
                s = s.setEndPosition(s.endLineNumber - 1, model.getLineMaxColumn(s.endLineNumber - 1));
            }
            var sourceLines = [];
            for (var i = s.startLineNumber; i <= s.endLineNumber; i++) {
                sourceLines.push(model.getLineContent(i));
            }
            var sourceText = sourceLines.join('\n');
            if (sourceText === '') {
                // Duplicating empty line
                if (this._isCopyingDown) {
                    this._startLineNumberDelta++;
                    this._endLineNumberDelta++;
                }
            }
            if (!this._isCopyingDown) {
                builder.addEditOperation(new range_1.Range(s.endLineNumber, model.getLineMaxColumn(s.endLineNumber), s.endLineNumber, model.getLineMaxColumn(s.endLineNumber)), '\n' + sourceText);
            }
            else {
                builder.addEditOperation(new range_1.Range(s.startLineNumber, 1, s.startLineNumber, 1), sourceText + '\n');
            }
            this._selectionId = builder.trackSelection(s);
            this._selectionDirection = this._selection.getDirection();
        };
        CopyLinesCommand.prototype.computeCursorState = function (model, helper) {
            var result = helper.getTrackedSelection(this._selectionId);
            if (this._startLineNumberDelta !== 0 || this._endLineNumberDelta !== 0) {
                var startLineNumber = result.startLineNumber, startColumn = result.startColumn, endLineNumber = result.endLineNumber, endColumn = result.endColumn;
                if (this._startLineNumberDelta !== 0) {
                    startLineNumber = startLineNumber + this._startLineNumberDelta;
                    startColumn = 1;
                }
                if (this._endLineNumberDelta !== 0) {
                    endLineNumber = endLineNumber + this._endLineNumberDelta;
                    endColumn = 1;
                }
                result = selection_1.Selection.createWithDirection(startLineNumber, startColumn, endLineNumber, endColumn, this._selectionDirection);
            }
            return result;
        };
        return CopyLinesCommand;
    }());
    exports.CopyLinesCommand = CopyLinesCommand;
});

define("vs/editor/contrib/linesOperations/common/deleteLinesCommand", ["require", "exports", 'vs/editor/common/core/range', 'vs/editor/common/core/selection'], function (require, exports, range_1, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DeleteLinesCommand = (function () {
        function DeleteLinesCommand(startLineNumber, endLineNumber, restoreCursorToColumn) {
            this.startLineNumber = startLineNumber;
            this.endLineNumber = endLineNumber;
            this.restoreCursorToColumn = restoreCursorToColumn;
        }
        DeleteLinesCommand.createFromSelection = function (selection) {
            var endLineNumber = selection.endLineNumber;
            if (selection.startLineNumber < selection.endLineNumber && selection.endColumn === 1) {
                endLineNumber -= 1;
            }
            return new DeleteLinesCommand(selection.startLineNumber, endLineNumber, selection.positionColumn);
        };
        DeleteLinesCommand.prototype.getEditOperations = function (model, builder) {
            if (model.getLineCount() === 1 && model.getLineMaxColumn(1) === 1) {
                // Model is empty
                return;
            }
            var startLineNumber = this.startLineNumber;
            var endLineNumber = this.endLineNumber;
            var startColumn = 1;
            var endColumn = model.getLineMaxColumn(endLineNumber);
            if (endLineNumber < model.getLineCount()) {
                endLineNumber += 1;
                endColumn = 1;
            }
            else if (startLineNumber > 1) {
                startLineNumber -= 1;
                startColumn = model.getLineMaxColumn(startLineNumber);
            }
            builder.addEditOperation(new range_1.Range(startLineNumber, startColumn, endLineNumber, endColumn), null);
        };
        DeleteLinesCommand.prototype.computeCursorState = function (model, helper) {
            var inverseEditOperations = helper.getInverseEditOperations();
            var srcRange = inverseEditOperations[0].range;
            return selection_1.Selection.createSelection(srcRange.endLineNumber, this.restoreCursorToColumn, srcRange.endLineNumber, this.restoreCursorToColumn);
        };
        return DeleteLinesCommand;
    }());
    exports.DeleteLinesCommand = DeleteLinesCommand;
});

define("vs/editor/contrib/linesOperations/common/moveLinesCommand", ["require", "exports", 'vs/editor/common/core/range', 'vs/editor/common/core/selection'], function (require, exports, range_1, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var MoveLinesCommand = (function () {
        function MoveLinesCommand(selection, isMovingDown) {
            this._selection = selection;
            this._isMovingDown = isMovingDown;
        }
        MoveLinesCommand.prototype.getEditOperations = function (model, builder) {
            var modelLineCount = model.getLineCount();
            if (this._isMovingDown && this._selection.endLineNumber === modelLineCount) {
                return;
            }
            if (!this._isMovingDown && this._selection.startLineNumber === 1) {
                return;
            }
            this._moveEndPositionDown = false;
            var s = this._selection;
            if (s.startLineNumber < s.endLineNumber && s.endColumn === 1) {
                this._moveEndPositionDown = true;
                s = s.setEndPosition(s.endLineNumber - 1, model.getLineMaxColumn(s.endLineNumber - 1));
            }
            if (s.startLineNumber === s.endLineNumber && model.getLineMaxColumn(s.startLineNumber) === 1) {
                // Current line is empty
                var lineNumber = s.startLineNumber;
                var otherLineNumber = (this._isMovingDown ? lineNumber + 1 : lineNumber - 1);
                if (model.getLineMaxColumn(otherLineNumber) === 1) {
                    // Other line number is empty too, so no editing is needed
                    // Add a no-op to force running by the model
                    builder.addEditOperation(new range_1.Range(1, 1, 1, 1), null);
                }
                else {
                    // Type content from other line number on line number
                    builder.addEditOperation(new range_1.Range(lineNumber, 1, lineNumber, 1), model.getLineContent(otherLineNumber));
                    // Remove content from other line number
                    builder.addEditOperation(new range_1.Range(otherLineNumber, 1, otherLineNumber, model.getLineMaxColumn(otherLineNumber)), null);
                }
                // Track selection at the other line number
                s = selection_1.Selection.createSelection(otherLineNumber, 1, otherLineNumber, 1);
            }
            else {
                var movingLineNumber, movingLineText;
                if (this._isMovingDown) {
                    movingLineNumber = s.endLineNumber + 1;
                    movingLineText = model.getLineContent(movingLineNumber);
                    // Delete line that needs to be moved
                    builder.addEditOperation(new range_1.Range(movingLineNumber - 1, model.getLineMaxColumn(movingLineNumber - 1), movingLineNumber, model.getLineMaxColumn(movingLineNumber)), null);
                    // Insert line that needs to be moved before
                    builder.addEditOperation(new range_1.Range(s.startLineNumber, 1, s.startLineNumber, 1), movingLineText + '\n');
                }
                else {
                    movingLineNumber = s.startLineNumber - 1;
                    movingLineText = model.getLineContent(movingLineNumber);
                    // Delete line that needs to be moved
                    builder.addEditOperation(new range_1.Range(movingLineNumber, 1, movingLineNumber + 1, 1), null);
                    // Insert line that needs to be moved after
                    builder.addEditOperation(new range_1.Range(s.endLineNumber, model.getLineMaxColumn(s.endLineNumber), s.endLineNumber, model.getLineMaxColumn(s.endLineNumber)), '\n' + movingLineText);
                }
            }
            this._selectionId = builder.trackSelection(s);
        };
        MoveLinesCommand.prototype.computeCursorState = function (model, helper) {
            var result = helper.getTrackedSelection(this._selectionId);
            if (this._moveEndPositionDown) {
                result = result.setEndPosition(result.endLineNumber + 1, 1);
            }
            return result;
        };
        return MoveLinesCommand;
    }());
    exports.MoveLinesCommand = MoveLinesCommand;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/referenceSearch/browser/referenceSearchModel", ["require", "exports", 'vs/base/common/collections', 'vs/base/common/eventEmitter', 'vs/base/common/paths', 'vs/base/common/strings', 'vs/base/common/uuid', 'vs/base/common/winjs.base', 'vs/editor/common/core/range'], function (require, exports, collections, eventEmitter_1, paths_1, strings, uuid_1, winjs_base_1, range_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EventType;
    (function (EventType) {
        EventType.OnReferenceRangeChanged = 'refrence.rangeChanged';
        EventType.CurrentReferenceChanged = 'reference.currentChanged';
    })(EventType = exports.EventType || (exports.EventType = {}));
    var OneReference = (function () {
        function OneReference(_parent, reference) {
            this._parent = _parent;
            this._id = uuid_1.generateUuid();
            this._range = reference.range;
        }
        Object.defineProperty(OneReference.prototype, "id", {
            get: function () {
                return this._id;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(OneReference.prototype, "model", {
            get: function () {
                return this._parent;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(OneReference.prototype, "parent", {
            get: function () {
                return this._parent;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(OneReference.prototype, "resource", {
            get: function () {
                return this._parent.resource;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(OneReference.prototype, "name", {
            get: function () {
                return this._parent.name;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(OneReference.prototype, "directory", {
            get: function () {
                return this._parent.directory;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(OneReference.prototype, "range", {
            get: function () {
                return this._range;
            },
            set: function (value) {
                this._range = value;
                this.parent.parent.emit(EventType.OnReferenceRangeChanged, this);
            },
            enumerable: true,
            configurable: true
        });
        return OneReference;
    }());
    exports.OneReference = OneReference;
    var FilePreview = (function () {
        function FilePreview(value) {
            this._value = value;
            this._lineStarts = strings.computeLineStarts(value);
        }
        FilePreview.prototype.preview = function (range, n) {
            if (n === void 0) { n = 8; }
            var lineStart = this._lineStarts[range.startLineNumber - 1], rangeStart = lineStart + range.startColumn - 1, rangeEnd = this._lineStarts[range.endLineNumber - 1] + range.endColumn - 1, lineEnd = range.endLineNumber >= this._lineStarts.length ? this._value.length : this._lineStarts[range.endLineNumber];
            var ret = {
                before: this._value.substring(lineStart, rangeStart).replace(/^\s+/, strings.empty),
                inside: this._value.substring(rangeStart, rangeEnd),
                after: this._value.substring(rangeEnd, lineEnd).replace(/\s+$/, strings.empty)
            };
            // long before parts will be cut at the best position
            ret.before = strings.lcut(ret.before, n);
            return ret;
        };
        return FilePreview;
    }());
    exports.FilePreview = FilePreview;
    var FileReferences = (function () {
        function FileReferences(_parent, _resource, _editorService) {
            this._parent = _parent;
            this._resource = _resource;
            this._editorService = _editorService;
            this._children = [];
        }
        Object.defineProperty(FileReferences.prototype, "id", {
            get: function () {
                return this._resource.toString();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileReferences.prototype, "parent", {
            get: function () {
                return this._parent;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileReferences.prototype, "children", {
            get: function () {
                return this._children;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileReferences.prototype, "resource", {
            get: function () {
                return this._resource;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileReferences.prototype, "name", {
            get: function () {
                return paths_1.basename(this.resource.fsPath);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileReferences.prototype, "directory", {
            get: function () {
                return paths_1.dirname(this.resource.fsPath);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileReferences.prototype, "preview", {
            get: function () {
                return this._preview;
            },
            enumerable: true,
            configurable: true
        });
        FileReferences.prototype.resolve = function () {
            var _this = this;
            if (this._resolved) {
                return winjs_base_1.TPromise.as(this);
            }
            return this._editorService.resolveEditorModel({ resource: this._resource }).then(function (model) {
                _this._preview = new FilePreview(model.textEditorModel.getValue());
                _this._resolved = true;
                return _this;
            });
        };
        return FileReferences;
    }());
    exports.FileReferences = FileReferences;
    var ReferencesModel = (function (_super) {
        __extends(ReferencesModel, _super);
        function ReferencesModel(references, editorService) {
            var _this = this;
            _super.call(this, [
                EventType.CurrentReferenceChanged,
                EventType.OnReferenceRangeChanged
            ]);
            var referencesByFile = Object.create(null);
            var seen = Object.create(null);
            references.forEach(function (reference) {
                var hash = ReferencesModel._hash(reference);
                if (!seen[hash]) {
                    seen[hash] = true;
                    var resource = reference.resource;
                    var fileReferences = new FileReferences(_this, resource, editorService);
                    fileReferences = collections.lookupOrInsert(referencesByFile, fileReferences.id, fileReferences);
                    fileReferences.children.push(new OneReference(fileReferences, reference));
                }
            });
            this._references = collections.values(referencesByFile);
            this._references.sort(ReferencesModel._compare);
        }
        Object.defineProperty(ReferencesModel.prototype, "children", {
            get: function () {
                return this._references;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ReferencesModel.prototype, "currentReference", {
            get: function () {
                return this._currentReference;
            },
            set: function (reference) {
                this._currentReference = reference;
                this.emit(EventType.CurrentReferenceChanged, this);
            },
            enumerable: true,
            configurable: true
        });
        ReferencesModel.prototype.nextReference = function (reference) {
            var idx = reference.parent.children.indexOf(reference), len = reference.parent.children.length, totalLength = reference.parent.parent.children.length;
            if (idx + 1 < len || totalLength === 1) {
                return reference.parent.children[(idx + 1) % len];
            }
            idx = reference.parent.parent.children.indexOf(reference.parent);
            idx = (idx + 1) % totalLength;
            return reference.parent.parent.children[idx].children[0];
        };
        ReferencesModel.prototype.findReference = function (resource, position) {
            for (var i = 0, len = this._references.length; i < len; i++) {
                var reference = this._references[i];
                if (reference.resource.toString() !== resource.toString()) {
                    continue;
                }
                var result;
                reference.children.some(function (element) {
                    if (range_1.Range.containsPosition(element.range, position)) {
                        result = element;
                        return true;
                    }
                    return false;
                });
                if (result) {
                    return result;
                }
            }
            if (this._references.length > 0) {
                return this._references[0].children[0];
            }
            return null;
        };
        ReferencesModel._hash = function (reference) {
            var _a = reference.range, startLineNumber = _a.startLineNumber, startColumn = _a.startColumn, endLineNumber = _a.endLineNumber, endColumn = _a.endColumn;
            return [reference.resource.toString(),
                startLineNumber, startColumn, endLineNumber, endColumn].join(',');
        };
        ReferencesModel._compare = function (a, b) {
            return strings.localeCompare(a.directory, b.directory) || strings.localeCompare(a.name, b.name);
        };
        return ReferencesModel;
    }(eventEmitter_1.EventEmitter));
    exports.ReferencesModel = ReferencesModel;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/suggest/browser/completionModel", ["require", "exports", 'vs/base/common/arrays', 'vs/base/common/objects', 'vs/base/common/winjs.base', 'vs/editor/common/modes/modesFilters'], function (require, exports, arrays_1, objects_1, winjs_base_1, modesFilters_1) {
    'use strict';
    function completionItemCompare(item, otherItem) {
        var suggestion = item.suggestion;
        var otherSuggestion = otherItem.suggestion;
        if (typeof suggestion.sortText === 'string' && typeof otherSuggestion.sortText === 'string') {
            var one = suggestion.sortText.toLowerCase();
            var other = otherSuggestion.sortText.toLowerCase();
            if (one < other) {
                return -1;
            }
            else if (one > other) {
                return 1;
            }
        }
        return suggestion.label.toLowerCase() < otherSuggestion.label.toLowerCase() ? -1 : 1;
    }
    var CompletionItem = (function () {
        function CompletionItem(group, suggestion, container) {
            this.group = group;
            this.support = container.support;
            this.suggestion = suggestion;
            this.container = container;
        }
        CompletionItem.prototype.resolveDetails = function (resource, position) {
            if (!this.support || typeof this.support.getSuggestionDetails !== 'function') {
                return winjs_base_1.TPromise.as(this.suggestion);
            }
            return this.support.getSuggestionDetails(resource, position, this.suggestion);
        };
        CompletionItem.prototype.updateDetails = function (value) {
            this.suggestion = objects_1.assign(this.suggestion, value);
        };
        return CompletionItem;
    }());
    exports.CompletionItem = CompletionItem;
    var CompletionGroup = (function () {
        function CompletionGroup(model, raw) {
            var _this = this;
            this.model = model;
            this._items = raw.reduce(function (items, result) {
                return items.concat(result.suggestions
                    .map(function (suggestion) { return new CompletionItem(_this, suggestion, result); }));
            }, []).sort(completionItemCompare);
            this.filter = modesFilters_1.DefaultFilter;
            if (this._items.length > 0) {
                var first = this._items[0];
                if (first.support) {
                    this.filter = first.support.getFilter && first.support.getFilter() || this.filter;
                }
            }
        }
        CompletionGroup.prototype.getItems = function (currentWord) {
            var _this = this;
            if (currentWord === this.cacheCurrentWord) {
                return this.cache;
            }
            var set;
            // try to narrow down when possible, instead of always filtering everything
            if (this.cacheCurrentWord && currentWord.substr(0, this.cacheCurrentWord.length) === this.cacheCurrentWord) {
                set = this.cache;
            }
            else {
                set = this._items;
            }
            var highlights = set.map(function (item) { return _this.filter(currentWord, item.suggestion); });
            var count = highlights.filter(function (h) { return !arrays_1.isFalsyOrEmpty(h); }).length;
            if (count === 0) {
                return [];
            }
            this.cacheCurrentWord = currentWord;
            this.cache = set
                .map(function (item, index) { return objects_1.assign(item, { highlights: highlights[index] }); })
                .filter(function (item) { return !arrays_1.isFalsyOrEmpty(item.highlights); });
            return this.cache;
        };
        CompletionGroup.prototype.invalidateCache = function () {
            this.cacheCurrentWord = null;
        };
        return CompletionGroup;
    }());
    exports.CompletionGroup = CompletionGroup;
    var CompletionModel = (function () {
        function CompletionModel(raw, currentWord) {
            var _this = this;
            this.raw = raw;
            this.currentWord = currentWord;
            this.groups = raw
                .filter(function (s) { return !!s; })
                .map(function (suggestResults) { return new CompletionGroup(_this, suggestResults); });
        }
        Object.defineProperty(CompletionModel.prototype, "items", {
            get: function () {
                var _this = this;
                if (this.cacheCurrentWord === this.currentWord) {
                    return this.cache;
                }
                var result = this.groups.reduce(function (r, groups) { return r.concat(groups.getItems(_this.currentWord)); }, []);
                // let's only cache stuff that actually has results
                if (result.length > 0) {
                    this.cache = result;
                    this.cacheCurrentWord = this.currentWord;
                }
                return result;
            },
            enumerable: true,
            configurable: true
        });
        return CompletionModel;
    }());
    exports.CompletionModel = CompletionModel;
});

define("vs/nls!vs/base/common/json",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/common/json", data); });
define("vs/base/common/json", ["require", "exports", 'vs/nls!vs/base/common/json'], function (require, exports, nls) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (ScanError) {
        ScanError[ScanError["None"] = 0] = "None";
        ScanError[ScanError["UnexpectedEndOfComment"] = 1] = "UnexpectedEndOfComment";
        ScanError[ScanError["UnexpectedEndOfString"] = 2] = "UnexpectedEndOfString";
        ScanError[ScanError["UnexpectedEndOfNumber"] = 3] = "UnexpectedEndOfNumber";
        ScanError[ScanError["InvalidUnicode"] = 4] = "InvalidUnicode";
        ScanError[ScanError["InvalidEscapeCharacter"] = 5] = "InvalidEscapeCharacter";
    })(exports.ScanError || (exports.ScanError = {}));
    var ScanError = exports.ScanError;
    (function (SyntaxKind) {
        SyntaxKind[SyntaxKind["Unknown"] = 0] = "Unknown";
        SyntaxKind[SyntaxKind["OpenBraceToken"] = 1] = "OpenBraceToken";
        SyntaxKind[SyntaxKind["CloseBraceToken"] = 2] = "CloseBraceToken";
        SyntaxKind[SyntaxKind["OpenBracketToken"] = 3] = "OpenBracketToken";
        SyntaxKind[SyntaxKind["CloseBracketToken"] = 4] = "CloseBracketToken";
        SyntaxKind[SyntaxKind["CommaToken"] = 5] = "CommaToken";
        SyntaxKind[SyntaxKind["ColonToken"] = 6] = "ColonToken";
        SyntaxKind[SyntaxKind["NullKeyword"] = 7] = "NullKeyword";
        SyntaxKind[SyntaxKind["TrueKeyword"] = 8] = "TrueKeyword";
        SyntaxKind[SyntaxKind["FalseKeyword"] = 9] = "FalseKeyword";
        SyntaxKind[SyntaxKind["StringLiteral"] = 10] = "StringLiteral";
        SyntaxKind[SyntaxKind["NumericLiteral"] = 11] = "NumericLiteral";
        SyntaxKind[SyntaxKind["LineCommentTrivia"] = 12] = "LineCommentTrivia";
        SyntaxKind[SyntaxKind["BlockCommentTrivia"] = 13] = "BlockCommentTrivia";
        SyntaxKind[SyntaxKind["LineBreakTrivia"] = 14] = "LineBreakTrivia";
        SyntaxKind[SyntaxKind["Trivia"] = 15] = "Trivia";
        SyntaxKind[SyntaxKind["EOF"] = 16] = "EOF";
    })(exports.SyntaxKind || (exports.SyntaxKind = {}));
    var SyntaxKind = exports.SyntaxKind;
    function createScanner(text, ignoreTrivia) {
        if (ignoreTrivia === void 0) { ignoreTrivia = false; }
        var pos = 0, len = text.length, value = '', tokenOffset = 0, token = SyntaxKind.Unknown, scanError = ScanError.None;
        function scanHexDigits(count, exact) {
            var digits = 0;
            var value = 0;
            while (digits < count || !exact) {
                var ch = text.charCodeAt(pos);
                if (ch >= CharacterCodes._0 && ch <= CharacterCodes._9) {
                    value = value * 16 + ch - CharacterCodes._0;
                }
                else if (ch >= CharacterCodes.A && ch <= CharacterCodes.F) {
                    value = value * 16 + ch - CharacterCodes.A + 10;
                }
                else if (ch >= CharacterCodes.a && ch <= CharacterCodes.f) {
                    value = value * 16 + ch - CharacterCodes.a + 10;
                }
                else {
                    break;
                }
                pos++;
                digits++;
            }
            if (digits < count) {
                value = -1;
            }
            return value;
        }
        function scanNumber() {
            var start = pos;
            if (text.charCodeAt(pos) === CharacterCodes._0) {
                pos++;
            }
            else {
                pos++;
                while (pos < text.length && isDigit(text.charCodeAt(pos))) {
                    pos++;
                }
            }
            if (pos < text.length && text.charCodeAt(pos) === CharacterCodes.dot) {
                pos++;
                if (pos < text.length && isDigit(text.charCodeAt(pos))) {
                    pos++;
                    while (pos < text.length && isDigit(text.charCodeAt(pos))) {
                        pos++;
                    }
                }
                else {
                    scanError = ScanError.UnexpectedEndOfNumber;
                    return text.substring(start, end);
                }
            }
            var end = pos;
            if (pos < text.length && (text.charCodeAt(pos) === CharacterCodes.E || text.charCodeAt(pos) === CharacterCodes.e)) {
                pos++;
                if (pos < text.length && text.charCodeAt(pos) === CharacterCodes.plus || text.charCodeAt(pos) === CharacterCodes.minus) {
                    pos++;
                }
                if (pos < text.length && isDigit(text.charCodeAt(pos))) {
                    pos++;
                    while (pos < text.length && isDigit(text.charCodeAt(pos))) {
                        pos++;
                    }
                    end = pos;
                }
                else {
                    scanError = ScanError.UnexpectedEndOfNumber;
                }
            }
            return text.substring(start, end);
        }
        function scanString() {
            var result = '', start = pos;
            while (true) {
                if (pos >= len) {
                    result += text.substring(start, pos);
                    scanError = ScanError.UnexpectedEndOfString;
                    break;
                }
                var ch = text.charCodeAt(pos);
                if (ch === CharacterCodes.doubleQuote) {
                    result += text.substring(start, pos);
                    pos++;
                    break;
                }
                if (ch === CharacterCodes.backslash) {
                    result += text.substring(start, pos);
                    pos++;
                    if (pos >= len) {
                        scanError = ScanError.UnexpectedEndOfString;
                        break;
                    }
                    ch = text.charCodeAt(pos++);
                    switch (ch) {
                        case CharacterCodes.doubleQuote:
                            result += '\"';
                            break;
                        case CharacterCodes.backslash:
                            result += '\\';
                            break;
                        case CharacterCodes.slash:
                            result += '/';
                            break;
                        case CharacterCodes.b:
                            result += '\b';
                            break;
                        case CharacterCodes.f:
                            result += '\f';
                            break;
                        case CharacterCodes.n:
                            result += '\n';
                            break;
                        case CharacterCodes.r:
                            result += '\r';
                            break;
                        case CharacterCodes.t:
                            result += '\t';
                            break;
                        case CharacterCodes.u:
                            var ch = scanHexDigits(4, true);
                            if (ch >= 0) {
                                result += String.fromCharCode(ch);
                            }
                            else {
                                scanError = ScanError.InvalidUnicode;
                            }
                            break;
                        default:
                            scanError = ScanError.InvalidEscapeCharacter;
                    }
                    start = pos;
                    continue;
                }
                if (isLineBreak(ch)) {
                    result += text.substring(start, pos);
                    scanError = ScanError.UnexpectedEndOfString;
                    break;
                }
                pos++;
            }
            return result;
        }
        function scanNext() {
            value = '';
            scanError = ScanError.None;
            tokenOffset = pos;
            if (pos >= len) {
                // at the end
                tokenOffset = len;
                return token = SyntaxKind.EOF;
            }
            var code = text.charCodeAt(pos);
            // trivia: whitespace
            if (isWhiteSpace(code)) {
                do {
                    pos++;
                    value += String.fromCharCode(code);
                    code = text.charCodeAt(pos);
                } while (isWhiteSpace(code));
                return token = SyntaxKind.Trivia;
            }
            // trivia: newlines
            if (isLineBreak(code)) {
                pos++;
                value += String.fromCharCode(code);
                if (code === CharacterCodes.carriageReturn && text.charCodeAt(pos) === CharacterCodes.lineFeed) {
                    pos++;
                    value += '\n';
                }
                return token = SyntaxKind.LineBreakTrivia;
            }
            switch (code) {
                // tokens: []{}:,
                case CharacterCodes.openBrace:
                    pos++;
                    return token = SyntaxKind.OpenBraceToken;
                case CharacterCodes.closeBrace:
                    pos++;
                    return token = SyntaxKind.CloseBraceToken;
                case CharacterCodes.openBracket:
                    pos++;
                    return token = SyntaxKind.OpenBracketToken;
                case CharacterCodes.closeBracket:
                    pos++;
                    return token = SyntaxKind.CloseBracketToken;
                case CharacterCodes.colon:
                    pos++;
                    return token = SyntaxKind.ColonToken;
                case CharacterCodes.comma:
                    pos++;
                    return token = SyntaxKind.CommaToken;
                // strings
                case CharacterCodes.doubleQuote:
                    pos++;
                    value = scanString();
                    return token = SyntaxKind.StringLiteral;
                // comments
                case CharacterCodes.slash:
                    var start = pos - 1;
                    // Single-line comment
                    if (text.charCodeAt(pos + 1) === CharacterCodes.slash) {
                        pos += 2;
                        while (pos < len) {
                            if (isLineBreak(text.charCodeAt(pos))) {
                                break;
                            }
                            pos++;
                        }
                        value = text.substring(start, pos);
                        return token = SyntaxKind.LineCommentTrivia;
                    }
                    // Multi-line comment
                    if (text.charCodeAt(pos + 1) === CharacterCodes.asterisk) {
                        pos += 2;
                        var safeLength = len - 1; // For lookahead.
                        var commentClosed = false;
                        while (pos < safeLength) {
                            var ch = text.charCodeAt(pos);
                            if (ch === CharacterCodes.asterisk && text.charCodeAt(pos + 1) === CharacterCodes.slash) {
                                pos += 2;
                                commentClosed = true;
                                break;
                            }
                            pos++;
                        }
                        if (!commentClosed) {
                            pos++;
                            scanError = ScanError.UnexpectedEndOfComment;
                        }
                        value = text.substring(start, pos);
                        return token = SyntaxKind.BlockCommentTrivia;
                    }
                    // just a single slash
                    value += String.fromCharCode(code);
                    pos++;
                    return token = SyntaxKind.Unknown;
                // numbers
                case CharacterCodes.minus:
                    value += String.fromCharCode(code);
                    pos++;
                    if (pos === len || !isDigit(text.charCodeAt(pos))) {
                        return token = SyntaxKind.Unknown;
                    }
                // found a minus, followed by a number so
                // we fall through to proceed with scanning
                // numbers
                case CharacterCodes._0:
                case CharacterCodes._1:
                case CharacterCodes._2:
                case CharacterCodes._3:
                case CharacterCodes._4:
                case CharacterCodes._5:
                case CharacterCodes._6:
                case CharacterCodes._7:
                case CharacterCodes._8:
                case CharacterCodes._9:
                    value += scanNumber();
                    return token = SyntaxKind.NumericLiteral;
                // literals and unknown symbols
                default:
                    // is a literal? Read the full word.
                    while (pos < len && isUnknownContentCharacter(code)) {
                        pos++;
                        code = text.charCodeAt(pos);
                    }
                    if (tokenOffset !== pos) {
                        value = text.substring(tokenOffset, pos);
                        // keywords: true, false, null
                        switch (value) {
                            case 'true': return token = SyntaxKind.TrueKeyword;
                            case 'false': return token = SyntaxKind.FalseKeyword;
                            case 'null': return token = SyntaxKind.NullKeyword;
                        }
                        return token = SyntaxKind.Unknown;
                    }
                    // some
                    value += String.fromCharCode(code);
                    pos++;
                    return token = SyntaxKind.Unknown;
            }
        }
        function isUnknownContentCharacter(code) {
            if (isWhiteSpace(code) || isLineBreak(code)) {
                return false;
            }
            switch (code) {
                case CharacterCodes.closeBrace:
                case CharacterCodes.closeBracket:
                case CharacterCodes.openBrace:
                case CharacterCodes.openBracket:
                case CharacterCodes.doubleQuote:
                case CharacterCodes.colon:
                case CharacterCodes.comma:
                    return false;
            }
            return true;
        }
        function scanNextNonTrivia() {
            var result;
            do {
                result = scanNext();
            } while (result >= SyntaxKind.LineCommentTrivia && result <= SyntaxKind.Trivia);
            return result;
        }
        return {
            getPosition: function () { return pos; },
            scan: ignoreTrivia ? scanNextNonTrivia : scanNext,
            getToken: function () { return token; },
            getTokenValue: function () { return value; },
            getTokenOffset: function () { return tokenOffset; },
            getTokenLength: function () { return pos - tokenOffset; },
            getTokenError: function () { return scanError; }
        };
    }
    exports.createScanner = createScanner;
    function isWhiteSpace(ch) {
        return ch === CharacterCodes.space || ch === CharacterCodes.tab || ch === CharacterCodes.verticalTab || ch === CharacterCodes.formFeed ||
            ch === CharacterCodes.nonBreakingSpace || ch === CharacterCodes.ogham || ch >= CharacterCodes.enQuad && ch <= CharacterCodes.zeroWidthSpace ||
            ch === CharacterCodes.narrowNoBreakSpace || ch === CharacterCodes.mathematicalSpace || ch === CharacterCodes.ideographicSpace || ch === CharacterCodes.byteOrderMark;
    }
    function isLineBreak(ch) {
        return ch === CharacterCodes.lineFeed || ch === CharacterCodes.carriageReturn || ch === CharacterCodes.lineSeparator || ch === CharacterCodes.paragraphSeparator;
    }
    function isDigit(ch) {
        return ch >= CharacterCodes._0 && ch <= CharacterCodes._9;
    }
    function isLetter(ch) {
        return ch >= CharacterCodes.a && ch <= CharacterCodes.z || ch >= CharacterCodes.A && ch <= CharacterCodes.Z;
    }
    exports.isLetter = isLetter;
    var CharacterCodes;
    (function (CharacterCodes) {
        CharacterCodes[CharacterCodes["nullCharacter"] = 0] = "nullCharacter";
        CharacterCodes[CharacterCodes["maxAsciiCharacter"] = 127] = "maxAsciiCharacter";
        CharacterCodes[CharacterCodes["lineFeed"] = 10] = "lineFeed";
        CharacterCodes[CharacterCodes["carriageReturn"] = 13] = "carriageReturn";
        CharacterCodes[CharacterCodes["lineSeparator"] = 8232] = "lineSeparator";
        CharacterCodes[CharacterCodes["paragraphSeparator"] = 8233] = "paragraphSeparator";
        // REVIEW: do we need to support this?  The scanner doesn't, but our IText does.  This seems
        // like an odd disparity?  (Or maybe it's completely fine for them to be different).
        CharacterCodes[CharacterCodes["nextLine"] = 133] = "nextLine";
        // Unicode 3.0 space characters
        CharacterCodes[CharacterCodes["space"] = 32] = "space";
        CharacterCodes[CharacterCodes["nonBreakingSpace"] = 160] = "nonBreakingSpace";
        CharacterCodes[CharacterCodes["enQuad"] = 8192] = "enQuad";
        CharacterCodes[CharacterCodes["emQuad"] = 8193] = "emQuad";
        CharacterCodes[CharacterCodes["enSpace"] = 8194] = "enSpace";
        CharacterCodes[CharacterCodes["emSpace"] = 8195] = "emSpace";
        CharacterCodes[CharacterCodes["threePerEmSpace"] = 8196] = "threePerEmSpace";
        CharacterCodes[CharacterCodes["fourPerEmSpace"] = 8197] = "fourPerEmSpace";
        CharacterCodes[CharacterCodes["sixPerEmSpace"] = 8198] = "sixPerEmSpace";
        CharacterCodes[CharacterCodes["figureSpace"] = 8199] = "figureSpace";
        CharacterCodes[CharacterCodes["punctuationSpace"] = 8200] = "punctuationSpace";
        CharacterCodes[CharacterCodes["thinSpace"] = 8201] = "thinSpace";
        CharacterCodes[CharacterCodes["hairSpace"] = 8202] = "hairSpace";
        CharacterCodes[CharacterCodes["zeroWidthSpace"] = 8203] = "zeroWidthSpace";
        CharacterCodes[CharacterCodes["narrowNoBreakSpace"] = 8239] = "narrowNoBreakSpace";
        CharacterCodes[CharacterCodes["ideographicSpace"] = 12288] = "ideographicSpace";
        CharacterCodes[CharacterCodes["mathematicalSpace"] = 8287] = "mathematicalSpace";
        CharacterCodes[CharacterCodes["ogham"] = 5760] = "ogham";
        CharacterCodes[CharacterCodes["_"] = 95] = "_";
        CharacterCodes[CharacterCodes["$"] = 36] = "$";
        CharacterCodes[CharacterCodes["_0"] = 48] = "_0";
        CharacterCodes[CharacterCodes["_1"] = 49] = "_1";
        CharacterCodes[CharacterCodes["_2"] = 50] = "_2";
        CharacterCodes[CharacterCodes["_3"] = 51] = "_3";
        CharacterCodes[CharacterCodes["_4"] = 52] = "_4";
        CharacterCodes[CharacterCodes["_5"] = 53] = "_5";
        CharacterCodes[CharacterCodes["_6"] = 54] = "_6";
        CharacterCodes[CharacterCodes["_7"] = 55] = "_7";
        CharacterCodes[CharacterCodes["_8"] = 56] = "_8";
        CharacterCodes[CharacterCodes["_9"] = 57] = "_9";
        CharacterCodes[CharacterCodes["a"] = 97] = "a";
        CharacterCodes[CharacterCodes["b"] = 98] = "b";
        CharacterCodes[CharacterCodes["c"] = 99] = "c";
        CharacterCodes[CharacterCodes["d"] = 100] = "d";
        CharacterCodes[CharacterCodes["e"] = 101] = "e";
        CharacterCodes[CharacterCodes["f"] = 102] = "f";
        CharacterCodes[CharacterCodes["g"] = 103] = "g";
        CharacterCodes[CharacterCodes["h"] = 104] = "h";
        CharacterCodes[CharacterCodes["i"] = 105] = "i";
        CharacterCodes[CharacterCodes["j"] = 106] = "j";
        CharacterCodes[CharacterCodes["k"] = 107] = "k";
        CharacterCodes[CharacterCodes["l"] = 108] = "l";
        CharacterCodes[CharacterCodes["m"] = 109] = "m";
        CharacterCodes[CharacterCodes["n"] = 110] = "n";
        CharacterCodes[CharacterCodes["o"] = 111] = "o";
        CharacterCodes[CharacterCodes["p"] = 112] = "p";
        CharacterCodes[CharacterCodes["q"] = 113] = "q";
        CharacterCodes[CharacterCodes["r"] = 114] = "r";
        CharacterCodes[CharacterCodes["s"] = 115] = "s";
        CharacterCodes[CharacterCodes["t"] = 116] = "t";
        CharacterCodes[CharacterCodes["u"] = 117] = "u";
        CharacterCodes[CharacterCodes["v"] = 118] = "v";
        CharacterCodes[CharacterCodes["w"] = 119] = "w";
        CharacterCodes[CharacterCodes["x"] = 120] = "x";
        CharacterCodes[CharacterCodes["y"] = 121] = "y";
        CharacterCodes[CharacterCodes["z"] = 122] = "z";
        CharacterCodes[CharacterCodes["A"] = 65] = "A";
        CharacterCodes[CharacterCodes["B"] = 66] = "B";
        CharacterCodes[CharacterCodes["C"] = 67] = "C";
        CharacterCodes[CharacterCodes["D"] = 68] = "D";
        CharacterCodes[CharacterCodes["E"] = 69] = "E";
        CharacterCodes[CharacterCodes["F"] = 70] = "F";
        CharacterCodes[CharacterCodes["G"] = 71] = "G";
        CharacterCodes[CharacterCodes["H"] = 72] = "H";
        CharacterCodes[CharacterCodes["I"] = 73] = "I";
        CharacterCodes[CharacterCodes["J"] = 74] = "J";
        CharacterCodes[CharacterCodes["K"] = 75] = "K";
        CharacterCodes[CharacterCodes["L"] = 76] = "L";
        CharacterCodes[CharacterCodes["M"] = 77] = "M";
        CharacterCodes[CharacterCodes["N"] = 78] = "N";
        CharacterCodes[CharacterCodes["O"] = 79] = "O";
        CharacterCodes[CharacterCodes["P"] = 80] = "P";
        CharacterCodes[CharacterCodes["Q"] = 81] = "Q";
        CharacterCodes[CharacterCodes["R"] = 82] = "R";
        CharacterCodes[CharacterCodes["S"] = 83] = "S";
        CharacterCodes[CharacterCodes["T"] = 84] = "T";
        CharacterCodes[CharacterCodes["U"] = 85] = "U";
        CharacterCodes[CharacterCodes["V"] = 86] = "V";
        CharacterCodes[CharacterCodes["W"] = 87] = "W";
        CharacterCodes[CharacterCodes["X"] = 88] = "X";
        CharacterCodes[CharacterCodes["Y"] = 89] = "Y";
        CharacterCodes[CharacterCodes["Z"] = 90] = "Z";
        CharacterCodes[CharacterCodes["ampersand"] = 38] = "ampersand";
        CharacterCodes[CharacterCodes["asterisk"] = 42] = "asterisk";
        CharacterCodes[CharacterCodes["at"] = 64] = "at";
        CharacterCodes[CharacterCodes["backslash"] = 92] = "backslash";
        CharacterCodes[CharacterCodes["bar"] = 124] = "bar";
        CharacterCodes[CharacterCodes["caret"] = 94] = "caret";
        CharacterCodes[CharacterCodes["closeBrace"] = 125] = "closeBrace";
        CharacterCodes[CharacterCodes["closeBracket"] = 93] = "closeBracket";
        CharacterCodes[CharacterCodes["closeParen"] = 41] = "closeParen";
        CharacterCodes[CharacterCodes["colon"] = 58] = "colon";
        CharacterCodes[CharacterCodes["comma"] = 44] = "comma";
        CharacterCodes[CharacterCodes["dot"] = 46] = "dot";
        CharacterCodes[CharacterCodes["doubleQuote"] = 34] = "doubleQuote";
        CharacterCodes[CharacterCodes["equals"] = 61] = "equals";
        CharacterCodes[CharacterCodes["exclamation"] = 33] = "exclamation";
        CharacterCodes[CharacterCodes["greaterThan"] = 62] = "greaterThan";
        CharacterCodes[CharacterCodes["lessThan"] = 60] = "lessThan";
        CharacterCodes[CharacterCodes["minus"] = 45] = "minus";
        CharacterCodes[CharacterCodes["openBrace"] = 123] = "openBrace";
        CharacterCodes[CharacterCodes["openBracket"] = 91] = "openBracket";
        CharacterCodes[CharacterCodes["openParen"] = 40] = "openParen";
        CharacterCodes[CharacterCodes["percent"] = 37] = "percent";
        CharacterCodes[CharacterCodes["plus"] = 43] = "plus";
        CharacterCodes[CharacterCodes["question"] = 63] = "question";
        CharacterCodes[CharacterCodes["semicolon"] = 59] = "semicolon";
        CharacterCodes[CharacterCodes["singleQuote"] = 39] = "singleQuote";
        CharacterCodes[CharacterCodes["slash"] = 47] = "slash";
        CharacterCodes[CharacterCodes["tilde"] = 126] = "tilde";
        CharacterCodes[CharacterCodes["backspace"] = 8] = "backspace";
        CharacterCodes[CharacterCodes["formFeed"] = 12] = "formFeed";
        CharacterCodes[CharacterCodes["byteOrderMark"] = 65279] = "byteOrderMark";
        CharacterCodes[CharacterCodes["tab"] = 9] = "tab";
        CharacterCodes[CharacterCodes["verticalTab"] = 11] = "verticalTab";
    })(CharacterCodes || (CharacterCodes = {}));
    /**
     * Takes JSON with JavaScript-style comments and remove
     * them. Optionally replaces every none-newline character
     * of comments with a replaceCharacter
     */
    function stripComments(text, replaceCh) {
        var _scanner = createScanner(text), parts = [], kind, offset = 0, pos;
        do {
            pos = _scanner.getPosition();
            kind = _scanner.scan();
            switch (kind) {
                case SyntaxKind.LineCommentTrivia:
                case SyntaxKind.BlockCommentTrivia:
                case SyntaxKind.EOF:
                    if (offset !== pos) {
                        parts.push(text.substring(offset, pos));
                    }
                    if (replaceCh !== void 0) {
                        parts.push(_scanner.getTokenValue().replace(/[^\r\n]/g, replaceCh));
                    }
                    offset = _scanner.getPosition();
                    break;
            }
        } while (kind !== SyntaxKind.EOF);
        return parts.join('');
    }
    exports.stripComments = stripComments;
    function parse(text, errors) {
        if (errors === void 0) { errors = []; }
        var noMatch = Object();
        var _scanner = createScanner(text, true);
        function scanNext() {
            var token = _scanner.scan();
            while (token === SyntaxKind.Unknown) {
                handleError(nls.localize(0, null));
                token = _scanner.scan();
            }
            return token;
        }
        function handleError(message, skipUntilAfter, skipUntil) {
            if (skipUntilAfter === void 0) { skipUntilAfter = []; }
            if (skipUntil === void 0) { skipUntil = []; }
            errors.push(message);
            if (skipUntilAfter.length + skipUntil.length > 0) {
                var token = _scanner.getToken();
                while (token !== SyntaxKind.EOF) {
                    if (skipUntilAfter.indexOf(token) !== -1) {
                        scanNext();
                        break;
                    }
                    else if (skipUntil.indexOf(token) !== -1) {
                        break;
                    }
                    token = scanNext();
                }
            }
        }
        function parseString() {
            if (_scanner.getToken() !== SyntaxKind.StringLiteral) {
                return noMatch;
            }
            var value = _scanner.getTokenValue();
            scanNext();
            return value;
        }
        function parseLiteral() {
            var value;
            switch (_scanner.getToken()) {
                case SyntaxKind.NumericLiteral:
                    try {
                        value = JSON.parse(_scanner.getTokenValue());
                        if (typeof value !== 'number') {
                            handleError(nls.localize(1, null));
                            value = 0;
                        }
                    }
                    catch (e) {
                        value = 0;
                    }
                    break;
                case SyntaxKind.NullKeyword:
                    value = null;
                    break;
                case SyntaxKind.TrueKeyword:
                    value = true;
                    break;
                case SyntaxKind.FalseKeyword:
                    value = false;
                    break;
                default:
                    return noMatch;
            }
            scanNext();
            return value;
        }
        function parseProperty(result) {
            var key = parseString();
            if (key === noMatch) {
                handleError(nls.localize(2, null), [], [SyntaxKind.CloseBraceToken, SyntaxKind.CommaToken]);
                return false;
            }
            if (_scanner.getToken() === SyntaxKind.ColonToken) {
                scanNext(); // consume colon
                var value = parseValue();
                if (value !== noMatch) {
                    result[key] = value;
                }
                else {
                    handleError(nls.localize(3, null), [], [SyntaxKind.CloseBraceToken, SyntaxKind.CommaToken]);
                }
            }
            else {
                handleError(nls.localize(4, null), [], [SyntaxKind.CloseBraceToken, SyntaxKind.CommaToken]);
            }
            return true;
        }
        function parseObject() {
            if (_scanner.getToken() !== SyntaxKind.OpenBraceToken) {
                return noMatch;
            }
            var obj = {};
            scanNext(); // consume open brace
            var needsComma = false;
            while (_scanner.getToken() !== SyntaxKind.CloseBraceToken && _scanner.getToken() !== SyntaxKind.EOF) {
                if (_scanner.getToken() === SyntaxKind.CommaToken) {
                    if (!needsComma) {
                        handleError(nls.localize(5, null), [], []);
                    }
                    scanNext(); // consume comma
                }
                else if (needsComma) {
                    handleError(nls.localize(6, null), [], []);
                }
                var propertyParsed = parseProperty(obj);
                if (!propertyParsed) {
                    handleError(nls.localize(7, null), [], [SyntaxKind.CloseBraceToken, SyntaxKind.CommaToken]);
                }
                needsComma = true;
            }
            if (_scanner.getToken() !== SyntaxKind.CloseBraceToken) {
                handleError(nls.localize(8, null), [SyntaxKind.CloseBraceToken], []);
            }
            else {
                scanNext(); // consume close brace
            }
            return obj;
        }
        function parseArray() {
            if (_scanner.getToken() !== SyntaxKind.OpenBracketToken) {
                return noMatch;
            }
            var arr = [];
            scanNext(); // consume open bracket
            var needsComma = false;
            while (_scanner.getToken() !== SyntaxKind.CloseBracketToken && _scanner.getToken() !== SyntaxKind.EOF) {
                if (_scanner.getToken() === SyntaxKind.CommaToken) {
                    if (!needsComma) {
                        handleError(nls.localize(9, null), [], []);
                    }
                    scanNext(); // consume comma
                }
                else if (needsComma) {
                    handleError(nls.localize(10, null), [], []);
                }
                var value = parseValue();
                if (value === noMatch) {
                    handleError(nls.localize(11, null), [], [SyntaxKind.CloseBracketToken, SyntaxKind.CommaToken]);
                }
                else {
                    arr.push(value);
                }
                needsComma = true;
            }
            if (_scanner.getToken() !== SyntaxKind.CloseBracketToken) {
                handleError(nls.localize(12, null), [SyntaxKind.CloseBracketToken], []);
            }
            else {
                scanNext(); // consume close bracket
            }
            return arr;
        }
        function parseValue() {
            var result = parseArray();
            if (result !== noMatch) {
                return result;
            }
            result = parseObject();
            if (result !== noMatch) {
                return result;
            }
            result = parseString();
            if (result !== noMatch) {
                return result;
            }
            return parseLiteral();
        }
        scanNext();
        var value = parseValue();
        if (value === noMatch) {
            handleError(nls.localize(13, null), [], []);
            return void 0;
        }
        if (_scanner.getToken() !== SyntaxKind.EOF) {
            handleError(nls.localize(14, null), [], []);
        }
        return value;
    }
    exports.parse = parse;
});

define("vs/nls!vs/base/common/keyCodes",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/common/keyCodes", data); });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/common/keyCodes", ["require", "exports", 'vs/nls!vs/base/common/keyCodes', 'vs/base/common/platform'], function (require, exports, nls, defaultPlatform) {
    'use strict';
    /**
     * Virtual Key Codes, the value does not hold any inherent meaning.
     * Inspired somewhat from https://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx
     * But these are "more general", as they should work across browsers & OS`s.
     */
    (function (KeyCode) {
        /**
         * Placed first to cover the 0 value of the enum.
         */
        KeyCode[KeyCode["Unknown"] = 0] = "Unknown";
        KeyCode[KeyCode["Backspace"] = 1] = "Backspace";
        KeyCode[KeyCode["Tab"] = 2] = "Tab";
        KeyCode[KeyCode["Enter"] = 3] = "Enter";
        KeyCode[KeyCode["Shift"] = 4] = "Shift";
        KeyCode[KeyCode["Ctrl"] = 5] = "Ctrl";
        KeyCode[KeyCode["Alt"] = 6] = "Alt";
        KeyCode[KeyCode["PauseBreak"] = 7] = "PauseBreak";
        KeyCode[KeyCode["CapsLock"] = 8] = "CapsLock";
        KeyCode[KeyCode["Escape"] = 9] = "Escape";
        KeyCode[KeyCode["Space"] = 10] = "Space";
        KeyCode[KeyCode["PageUp"] = 11] = "PageUp";
        KeyCode[KeyCode["PageDown"] = 12] = "PageDown";
        KeyCode[KeyCode["End"] = 13] = "End";
        KeyCode[KeyCode["Home"] = 14] = "Home";
        KeyCode[KeyCode["LeftArrow"] = 15] = "LeftArrow";
        KeyCode[KeyCode["UpArrow"] = 16] = "UpArrow";
        KeyCode[KeyCode["RightArrow"] = 17] = "RightArrow";
        KeyCode[KeyCode["DownArrow"] = 18] = "DownArrow";
        KeyCode[KeyCode["Insert"] = 19] = "Insert";
        KeyCode[KeyCode["Delete"] = 20] = "Delete";
        KeyCode[KeyCode["KEY_0"] = 21] = "KEY_0";
        KeyCode[KeyCode["KEY_1"] = 22] = "KEY_1";
        KeyCode[KeyCode["KEY_2"] = 23] = "KEY_2";
        KeyCode[KeyCode["KEY_3"] = 24] = "KEY_3";
        KeyCode[KeyCode["KEY_4"] = 25] = "KEY_4";
        KeyCode[KeyCode["KEY_5"] = 26] = "KEY_5";
        KeyCode[KeyCode["KEY_6"] = 27] = "KEY_6";
        KeyCode[KeyCode["KEY_7"] = 28] = "KEY_7";
        KeyCode[KeyCode["KEY_8"] = 29] = "KEY_8";
        KeyCode[KeyCode["KEY_9"] = 30] = "KEY_9";
        KeyCode[KeyCode["KEY_A"] = 31] = "KEY_A";
        KeyCode[KeyCode["KEY_B"] = 32] = "KEY_B";
        KeyCode[KeyCode["KEY_C"] = 33] = "KEY_C";
        KeyCode[KeyCode["KEY_D"] = 34] = "KEY_D";
        KeyCode[KeyCode["KEY_E"] = 35] = "KEY_E";
        KeyCode[KeyCode["KEY_F"] = 36] = "KEY_F";
        KeyCode[KeyCode["KEY_G"] = 37] = "KEY_G";
        KeyCode[KeyCode["KEY_H"] = 38] = "KEY_H";
        KeyCode[KeyCode["KEY_I"] = 39] = "KEY_I";
        KeyCode[KeyCode["KEY_J"] = 40] = "KEY_J";
        KeyCode[KeyCode["KEY_K"] = 41] = "KEY_K";
        KeyCode[KeyCode["KEY_L"] = 42] = "KEY_L";
        KeyCode[KeyCode["KEY_M"] = 43] = "KEY_M";
        KeyCode[KeyCode["KEY_N"] = 44] = "KEY_N";
        KeyCode[KeyCode["KEY_O"] = 45] = "KEY_O";
        KeyCode[KeyCode["KEY_P"] = 46] = "KEY_P";
        KeyCode[KeyCode["KEY_Q"] = 47] = "KEY_Q";
        KeyCode[KeyCode["KEY_R"] = 48] = "KEY_R";
        KeyCode[KeyCode["KEY_S"] = 49] = "KEY_S";
        KeyCode[KeyCode["KEY_T"] = 50] = "KEY_T";
        KeyCode[KeyCode["KEY_U"] = 51] = "KEY_U";
        KeyCode[KeyCode["KEY_V"] = 52] = "KEY_V";
        KeyCode[KeyCode["KEY_W"] = 53] = "KEY_W";
        KeyCode[KeyCode["KEY_X"] = 54] = "KEY_X";
        KeyCode[KeyCode["KEY_Y"] = 55] = "KEY_Y";
        KeyCode[KeyCode["KEY_Z"] = 56] = "KEY_Z";
        KeyCode[KeyCode["Meta"] = 57] = "Meta";
        KeyCode[KeyCode["ContextMenu"] = 58] = "ContextMenu";
        KeyCode[KeyCode["F1"] = 59] = "F1";
        KeyCode[KeyCode["F2"] = 60] = "F2";
        KeyCode[KeyCode["F3"] = 61] = "F3";
        KeyCode[KeyCode["F4"] = 62] = "F4";
        KeyCode[KeyCode["F5"] = 63] = "F5";
        KeyCode[KeyCode["F6"] = 64] = "F6";
        KeyCode[KeyCode["F7"] = 65] = "F7";
        KeyCode[KeyCode["F8"] = 66] = "F8";
        KeyCode[KeyCode["F9"] = 67] = "F9";
        KeyCode[KeyCode["F10"] = 68] = "F10";
        KeyCode[KeyCode["F11"] = 69] = "F11";
        KeyCode[KeyCode["F12"] = 70] = "F12";
        KeyCode[KeyCode["F13"] = 71] = "F13";
        KeyCode[KeyCode["F14"] = 72] = "F14";
        KeyCode[KeyCode["F15"] = 73] = "F15";
        KeyCode[KeyCode["F16"] = 74] = "F16";
        KeyCode[KeyCode["F17"] = 75] = "F17";
        KeyCode[KeyCode["F18"] = 76] = "F18";
        KeyCode[KeyCode["F19"] = 77] = "F19";
        KeyCode[KeyCode["NumLock"] = 78] = "NumLock";
        KeyCode[KeyCode["ScrollLock"] = 79] = "ScrollLock";
        /**
         * Used for miscellaneous characters; it can vary by keyboard.
         * For the US standard keyboard, the ';:' key
         */
        KeyCode[KeyCode["US_SEMICOLON"] = 80] = "US_SEMICOLON";
        /**
         * For any country/region, the '+' key
         * For the US standard keyboard, the '=+' key
         */
        KeyCode[KeyCode["US_EQUAL"] = 81] = "US_EQUAL";
        /**
         * For any country/region, the ',' key
         * For the US standard keyboard, the ',<' key
         */
        KeyCode[KeyCode["US_COMMA"] = 82] = "US_COMMA";
        /**
         * For any country/region, the '-' key
         * For the US standard keyboard, the '-_' key
         */
        KeyCode[KeyCode["US_MINUS"] = 83] = "US_MINUS";
        /**
         * For any country/region, the '.' key
         * For the US standard keyboard, the '.>' key
         */
        KeyCode[KeyCode["US_DOT"] = 84] = "US_DOT";
        /**
         * Used for miscellaneous characters; it can vary by keyboard.
         * For the US standard keyboard, the '/?' key
         */
        KeyCode[KeyCode["US_SLASH"] = 85] = "US_SLASH";
        /**
         * Used for miscellaneous characters; it can vary by keyboard.
         * For the US standard keyboard, the '`~' key
         */
        KeyCode[KeyCode["US_BACKTICK"] = 86] = "US_BACKTICK";
        /**
         * Used for miscellaneous characters; it can vary by keyboard.
         * For the US standard keyboard, the '[{' key
         */
        KeyCode[KeyCode["US_OPEN_SQUARE_BRACKET"] = 87] = "US_OPEN_SQUARE_BRACKET";
        /**
         * Used for miscellaneous characters; it can vary by keyboard.
         * For the US standard keyboard, the '\|' key
         */
        KeyCode[KeyCode["US_BACKSLASH"] = 88] = "US_BACKSLASH";
        /**
         * Used for miscellaneous characters; it can vary by keyboard.
         * For the US standard keyboard, the ']}' key
         */
        KeyCode[KeyCode["US_CLOSE_SQUARE_BRACKET"] = 89] = "US_CLOSE_SQUARE_BRACKET";
        /**
         * Used for miscellaneous characters; it can vary by keyboard.
         * For the US standard keyboard, the ''"' key
         */
        KeyCode[KeyCode["US_QUOTE"] = 90] = "US_QUOTE";
        /**
         * Used for miscellaneous characters; it can vary by keyboard.
         */
        KeyCode[KeyCode["OEM_8"] = 91] = "OEM_8";
        /**
         * Either the angle bracket key or the backslash key on the RT 102-key keyboard.
         */
        KeyCode[KeyCode["OEM_102"] = 92] = "OEM_102";
        KeyCode[KeyCode["NUMPAD_0"] = 93] = "NUMPAD_0";
        KeyCode[KeyCode["NUMPAD_1"] = 94] = "NUMPAD_1";
        KeyCode[KeyCode["NUMPAD_2"] = 95] = "NUMPAD_2";
        KeyCode[KeyCode["NUMPAD_3"] = 96] = "NUMPAD_3";
        KeyCode[KeyCode["NUMPAD_4"] = 97] = "NUMPAD_4";
        KeyCode[KeyCode["NUMPAD_5"] = 98] = "NUMPAD_5";
        KeyCode[KeyCode["NUMPAD_6"] = 99] = "NUMPAD_6";
        KeyCode[KeyCode["NUMPAD_7"] = 100] = "NUMPAD_7";
        KeyCode[KeyCode["NUMPAD_8"] = 101] = "NUMPAD_8";
        KeyCode[KeyCode["NUMPAD_9"] = 102] = "NUMPAD_9";
        KeyCode[KeyCode["NUMPAD_MULTIPLY"] = 103] = "NUMPAD_MULTIPLY";
        KeyCode[KeyCode["NUMPAD_ADD"] = 104] = "NUMPAD_ADD";
        KeyCode[KeyCode["NUMPAD_SEPARATOR"] = 105] = "NUMPAD_SEPARATOR";
        KeyCode[KeyCode["NUMPAD_SUBTRACT"] = 106] = "NUMPAD_SUBTRACT";
        KeyCode[KeyCode["NUMPAD_DECIMAL"] = 107] = "NUMPAD_DECIMAL";
        KeyCode[KeyCode["NUMPAD_DIVIDE"] = 108] = "NUMPAD_DIVIDE";
        /**
         * Placed last to cover the length of the enum.
         */
        KeyCode[KeyCode["MAX_VALUE"] = 109] = "MAX_VALUE";
    })(exports.KeyCode || (exports.KeyCode = {}));
    var KeyCode = exports.KeyCode;
    var Mapping = (function () {
        function Mapping(fromKeyCode, toKeyCode) {
            this._fromKeyCode = fromKeyCode;
            this._toKeyCode = toKeyCode;
        }
        Mapping.prototype.fromKeyCode = function (keyCode) {
            return this._fromKeyCode[keyCode];
        };
        Mapping.prototype.toKeyCode = function (str) {
            if (this._toKeyCode.hasOwnProperty(str)) {
                return this._toKeyCode[str];
            }
            return KeyCode.Unknown;
        };
        return Mapping;
    }());
    function createMapping(fill1, fill2) {
        var MAP = [];
        fill1(MAP);
        var REVERSE_MAP = {};
        for (var i = 0, len = MAP.length; i < len; i++) {
            if (!MAP[i]) {
                continue;
            }
            REVERSE_MAP[MAP[i]] = i;
        }
        fill2(REVERSE_MAP);
        var FINAL_REVERSE_MAP = {};
        for (var entry in REVERSE_MAP) {
            if (REVERSE_MAP.hasOwnProperty(entry)) {
                FINAL_REVERSE_MAP[entry] = REVERSE_MAP[entry];
                FINAL_REVERSE_MAP[entry.toLowerCase()] = REVERSE_MAP[entry];
            }
        }
        return new Mapping(MAP, FINAL_REVERSE_MAP);
    }
    var STRING = createMapping(function (TO_STRING_MAP) {
        TO_STRING_MAP[KeyCode.Unknown] = 'unknown';
        TO_STRING_MAP[KeyCode.Backspace] = 'Backspace';
        TO_STRING_MAP[KeyCode.Tab] = 'Tab';
        TO_STRING_MAP[KeyCode.Enter] = 'Enter';
        TO_STRING_MAP[KeyCode.Shift] = 'Shift';
        TO_STRING_MAP[KeyCode.Ctrl] = 'Ctrl';
        TO_STRING_MAP[KeyCode.Alt] = 'Alt';
        TO_STRING_MAP[KeyCode.PauseBreak] = 'PauseBreak';
        TO_STRING_MAP[KeyCode.CapsLock] = 'CapsLock';
        TO_STRING_MAP[KeyCode.Escape] = 'Escape';
        TO_STRING_MAP[KeyCode.Space] = 'Space';
        TO_STRING_MAP[KeyCode.PageUp] = 'PageUp';
        TO_STRING_MAP[KeyCode.PageDown] = 'PageDown';
        TO_STRING_MAP[KeyCode.End] = 'End';
        TO_STRING_MAP[KeyCode.Home] = 'Home';
        TO_STRING_MAP[KeyCode.LeftArrow] = 'LeftArrow';
        TO_STRING_MAP[KeyCode.UpArrow] = 'UpArrow';
        TO_STRING_MAP[KeyCode.RightArrow] = 'RightArrow';
        TO_STRING_MAP[KeyCode.DownArrow] = 'DownArrow';
        TO_STRING_MAP[KeyCode.Insert] = 'Insert';
        TO_STRING_MAP[KeyCode.Delete] = 'Delete';
        TO_STRING_MAP[KeyCode.KEY_0] = '0';
        TO_STRING_MAP[KeyCode.KEY_1] = '1';
        TO_STRING_MAP[KeyCode.KEY_2] = '2';
        TO_STRING_MAP[KeyCode.KEY_3] = '3';
        TO_STRING_MAP[KeyCode.KEY_4] = '4';
        TO_STRING_MAP[KeyCode.KEY_5] = '5';
        TO_STRING_MAP[KeyCode.KEY_6] = '6';
        TO_STRING_MAP[KeyCode.KEY_7] = '7';
        TO_STRING_MAP[KeyCode.KEY_8] = '8';
        TO_STRING_MAP[KeyCode.KEY_9] = '9';
        TO_STRING_MAP[KeyCode.KEY_A] = 'A';
        TO_STRING_MAP[KeyCode.KEY_B] = 'B';
        TO_STRING_MAP[KeyCode.KEY_C] = 'C';
        TO_STRING_MAP[KeyCode.KEY_D] = 'D';
        TO_STRING_MAP[KeyCode.KEY_E] = 'E';
        TO_STRING_MAP[KeyCode.KEY_F] = 'F';
        TO_STRING_MAP[KeyCode.KEY_G] = 'G';
        TO_STRING_MAP[KeyCode.KEY_H] = 'H';
        TO_STRING_MAP[KeyCode.KEY_I] = 'I';
        TO_STRING_MAP[KeyCode.KEY_J] = 'J';
        TO_STRING_MAP[KeyCode.KEY_K] = 'K';
        TO_STRING_MAP[KeyCode.KEY_L] = 'L';
        TO_STRING_MAP[KeyCode.KEY_M] = 'M';
        TO_STRING_MAP[KeyCode.KEY_N] = 'N';
        TO_STRING_MAP[KeyCode.KEY_O] = 'O';
        TO_STRING_MAP[KeyCode.KEY_P] = 'P';
        TO_STRING_MAP[KeyCode.KEY_Q] = 'Q';
        TO_STRING_MAP[KeyCode.KEY_R] = 'R';
        TO_STRING_MAP[KeyCode.KEY_S] = 'S';
        TO_STRING_MAP[KeyCode.KEY_T] = 'T';
        TO_STRING_MAP[KeyCode.KEY_U] = 'U';
        TO_STRING_MAP[KeyCode.KEY_V] = 'V';
        TO_STRING_MAP[KeyCode.KEY_W] = 'W';
        TO_STRING_MAP[KeyCode.KEY_X] = 'X';
        TO_STRING_MAP[KeyCode.KEY_Y] = 'Y';
        TO_STRING_MAP[KeyCode.KEY_Z] = 'Z';
        TO_STRING_MAP[KeyCode.ContextMenu] = 'ContextMenu';
        TO_STRING_MAP[KeyCode.F1] = 'F1';
        TO_STRING_MAP[KeyCode.F2] = 'F2';
        TO_STRING_MAP[KeyCode.F3] = 'F3';
        TO_STRING_MAP[KeyCode.F4] = 'F4';
        TO_STRING_MAP[KeyCode.F5] = 'F5';
        TO_STRING_MAP[KeyCode.F6] = 'F6';
        TO_STRING_MAP[KeyCode.F7] = 'F7';
        TO_STRING_MAP[KeyCode.F8] = 'F8';
        TO_STRING_MAP[KeyCode.F9] = 'F9';
        TO_STRING_MAP[KeyCode.F10] = 'F10';
        TO_STRING_MAP[KeyCode.F11] = 'F11';
        TO_STRING_MAP[KeyCode.F12] = 'F12';
        TO_STRING_MAP[KeyCode.F13] = 'F13';
        TO_STRING_MAP[KeyCode.F14] = 'F14';
        TO_STRING_MAP[KeyCode.F15] = 'F15';
        TO_STRING_MAP[KeyCode.F16] = 'F16';
        TO_STRING_MAP[KeyCode.F17] = 'F17';
        TO_STRING_MAP[KeyCode.F18] = 'F18';
        TO_STRING_MAP[KeyCode.F19] = 'F19';
        TO_STRING_MAP[KeyCode.NumLock] = 'NumLock';
        TO_STRING_MAP[KeyCode.ScrollLock] = 'ScrollLock';
        TO_STRING_MAP[KeyCode.US_SEMICOLON] = ';';
        TO_STRING_MAP[KeyCode.US_EQUAL] = '=';
        TO_STRING_MAP[KeyCode.US_COMMA] = ',';
        TO_STRING_MAP[KeyCode.US_MINUS] = '-';
        TO_STRING_MAP[KeyCode.US_DOT] = '.';
        TO_STRING_MAP[KeyCode.US_SLASH] = '/';
        TO_STRING_MAP[KeyCode.US_BACKTICK] = '`';
        TO_STRING_MAP[KeyCode.US_OPEN_SQUARE_BRACKET] = '[';
        TO_STRING_MAP[KeyCode.US_BACKSLASH] = '\\';
        TO_STRING_MAP[KeyCode.US_CLOSE_SQUARE_BRACKET] = ']';
        TO_STRING_MAP[KeyCode.US_QUOTE] = '\'';
        TO_STRING_MAP[KeyCode.OEM_8] = 'OEM_8';
        TO_STRING_MAP[KeyCode.OEM_102] = 'OEM_102';
        TO_STRING_MAP[KeyCode.NUMPAD_0] = 'NumPad0';
        TO_STRING_MAP[KeyCode.NUMPAD_1] = 'NumPad1';
        TO_STRING_MAP[KeyCode.NUMPAD_2] = 'NumPad2';
        TO_STRING_MAP[KeyCode.NUMPAD_3] = 'NumPad3';
        TO_STRING_MAP[KeyCode.NUMPAD_4] = 'NumPad4';
        TO_STRING_MAP[KeyCode.NUMPAD_5] = 'NumPad5';
        TO_STRING_MAP[KeyCode.NUMPAD_6] = 'NumPad6';
        TO_STRING_MAP[KeyCode.NUMPAD_7] = 'NumPad7';
        TO_STRING_MAP[KeyCode.NUMPAD_8] = 'NumPad8';
        TO_STRING_MAP[KeyCode.NUMPAD_9] = 'NumPad9';
        TO_STRING_MAP[KeyCode.NUMPAD_MULTIPLY] = 'NumPad_Multiply';
        TO_STRING_MAP[KeyCode.NUMPAD_ADD] = 'NumPad_Add';
        TO_STRING_MAP[KeyCode.NUMPAD_SEPARATOR] = 'NumPad_Separator';
        TO_STRING_MAP[KeyCode.NUMPAD_SUBTRACT] = 'NumPad_Subtract';
        TO_STRING_MAP[KeyCode.NUMPAD_DECIMAL] = 'NumPad_Decimal';
        TO_STRING_MAP[KeyCode.NUMPAD_DIVIDE] = 'NumPad_Divide';
        // for (let i = 0; i < KeyCode.MAX_VALUE; i++) {
        // 	if (!TO_STRING_MAP[i]) {
        // 		console.warn('Missing string representation for ' + KeyCode[i]);
        // 	}
        // }
    }, function (FROM_STRING_MAP) {
        FROM_STRING_MAP['\r'] = KeyCode.Enter;
    });
    var USER_SETTINGS = createMapping(function (TO_USER_SETTINGS_MAP) {
        for (var i = 0, len = STRING._fromKeyCode.length; i < len; i++) {
            TO_USER_SETTINGS_MAP[i] = STRING._fromKeyCode[i];
        }
        TO_USER_SETTINGS_MAP[KeyCode.LeftArrow] = 'Left';
        TO_USER_SETTINGS_MAP[KeyCode.UpArrow] = 'Up';
        TO_USER_SETTINGS_MAP[KeyCode.RightArrow] = 'Right';
        TO_USER_SETTINGS_MAP[KeyCode.DownArrow] = 'Down';
    }, function (FROM_USER_SETTINGS_MAP) {
        FROM_USER_SETTINGS_MAP['OEM_1'] = KeyCode.US_SEMICOLON;
        FROM_USER_SETTINGS_MAP['OEM_PLUS'] = KeyCode.US_EQUAL;
        FROM_USER_SETTINGS_MAP['OEM_COMMA'] = KeyCode.US_COMMA;
        FROM_USER_SETTINGS_MAP['OEM_MINUS'] = KeyCode.US_MINUS;
        FROM_USER_SETTINGS_MAP['OEM_PERIOD'] = KeyCode.US_DOT;
        FROM_USER_SETTINGS_MAP['OEM_2'] = KeyCode.US_SLASH;
        FROM_USER_SETTINGS_MAP['OEM_3'] = KeyCode.US_BACKTICK;
        FROM_USER_SETTINGS_MAP['OEM_4'] = KeyCode.US_OPEN_SQUARE_BRACKET;
        FROM_USER_SETTINGS_MAP['OEM_5'] = KeyCode.US_BACKSLASH;
        FROM_USER_SETTINGS_MAP['OEM_6'] = KeyCode.US_CLOSE_SQUARE_BRACKET;
        FROM_USER_SETTINGS_MAP['OEM_7'] = KeyCode.US_QUOTE;
        FROM_USER_SETTINGS_MAP['OEM_8'] = KeyCode.OEM_8;
        FROM_USER_SETTINGS_MAP['OEM_102'] = KeyCode.OEM_102;
    });
    var KeyCode;
    (function (KeyCode) {
        function toString(key) {
            return STRING.fromKeyCode(key);
        }
        KeyCode.toString = toString;
        function fromString(key) {
            return STRING.toKeyCode(key);
        }
        KeyCode.fromString = fromString;
    })(KeyCode = exports.KeyCode || (exports.KeyCode = {}));
    // Binary encoding strategy:
    // 15:  1 bit for ctrlCmd
    // 14:  1 bit for shift
    // 13:  1 bit for alt
    // 12:  1 bit for winCtrl
    //  0: 12 bits for keyCode (up to a maximum keyCode of 4096. Given we have 83 at this point thats good enough)
    var BIN_CTRLCMD_MASK = 1 << 15;
    var BIN_SHIFT_MASK = 1 << 14;
    var BIN_ALT_MASK = 1 << 13;
    var BIN_WINCTRL_MASK = 1 << 12;
    var BIN_KEYCODE_MASK = 0x00000fff;
    var BinaryKeybindings = (function () {
        function BinaryKeybindings() {
        }
        BinaryKeybindings.extractFirstPart = function (keybinding) {
            return keybinding & 0x0000ffff;
        };
        BinaryKeybindings.extractChordPart = function (keybinding) {
            return (keybinding >> 16) & 0x0000ffff;
        };
        BinaryKeybindings.hasChord = function (keybinding) {
            return (this.extractChordPart(keybinding) !== 0);
        };
        BinaryKeybindings.hasCtrlCmd = function (keybinding) {
            return (keybinding & BIN_CTRLCMD_MASK ? true : false);
        };
        BinaryKeybindings.hasShift = function (keybinding) {
            return (keybinding & BIN_SHIFT_MASK ? true : false);
        };
        BinaryKeybindings.hasAlt = function (keybinding) {
            return (keybinding & BIN_ALT_MASK ? true : false);
        };
        BinaryKeybindings.hasWinCtrl = function (keybinding) {
            return (keybinding & BIN_WINCTRL_MASK ? true : false);
        };
        BinaryKeybindings.extractKeyCode = function (keybinding) {
            return (keybinding & BIN_KEYCODE_MASK);
        };
        return BinaryKeybindings;
    }());
    exports.BinaryKeybindings = BinaryKeybindings;
    var KeyMod = (function () {
        function KeyMod() {
        }
        KeyMod.chord = function (firstPart, secondPart) {
            return firstPart | ((secondPart & 0x0000ffff) << 16);
        };
        KeyMod.CtrlCmd = BIN_CTRLCMD_MASK;
        KeyMod.Shift = BIN_SHIFT_MASK;
        KeyMod.Alt = BIN_ALT_MASK;
        KeyMod.WinCtrl = BIN_WINCTRL_MASK;
        return KeyMod;
    }());
    exports.KeyMod = KeyMod;
    /**
     * A set of usual keybindings that can be reused in code
     */
    var CommonKeybindings = (function () {
        function CommonKeybindings() {
        }
        CommonKeybindings.ENTER = KeyCode.Enter;
        CommonKeybindings.SHIFT_ENTER = KeyMod.Shift | KeyCode.Enter;
        CommonKeybindings.CTRLCMD_ENTER = KeyMod.CtrlCmd | KeyCode.Enter;
        CommonKeybindings.WINCTRL_ENTER = KeyMod.WinCtrl | KeyCode.Enter;
        CommonKeybindings.TAB = KeyCode.Tab;
        CommonKeybindings.SHIFT_TAB = KeyMod.Shift | KeyCode.Tab;
        CommonKeybindings.ESCAPE = KeyCode.Escape;
        CommonKeybindings.SPACE = KeyCode.Space;
        CommonKeybindings.DELETE = KeyCode.Delete;
        CommonKeybindings.SHIFT_DELETE = KeyMod.Shift | KeyCode.Delete;
        CommonKeybindings.CTRLCMD_BACKSPACE = KeyMod.CtrlCmd | KeyCode.Backspace;
        CommonKeybindings.UP_ARROW = KeyCode.UpArrow;
        CommonKeybindings.SHIFT_UP_ARROW = KeyMod.Shift | KeyCode.UpArrow;
        CommonKeybindings.CTRLCMD_UP_ARROW = KeyMod.CtrlCmd | KeyCode.UpArrow;
        CommonKeybindings.DOWN_ARROW = KeyCode.DownArrow;
        CommonKeybindings.SHIFT_DOWN_ARROW = KeyMod.Shift | KeyCode.DownArrow;
        CommonKeybindings.CTRLCMD_DOWN_ARROW = KeyMod.CtrlCmd | KeyCode.DownArrow;
        CommonKeybindings.LEFT_ARROW = KeyCode.LeftArrow;
        CommonKeybindings.RIGHT_ARROW = KeyCode.RightArrow;
        CommonKeybindings.PAGE_UP = KeyCode.PageUp;
        CommonKeybindings.SHIFT_PAGE_UP = KeyMod.Shift | KeyCode.PageUp;
        CommonKeybindings.PAGE_DOWN = KeyCode.PageDown;
        CommonKeybindings.SHIFT_PAGE_DOWN = KeyMod.Shift | KeyCode.PageDown;
        CommonKeybindings.F2 = KeyCode.F2;
        CommonKeybindings.CTRLCMD_S = KeyMod.CtrlCmd | KeyCode.KEY_S;
        CommonKeybindings.CTRLCMD_C = KeyMod.CtrlCmd | KeyCode.KEY_C;
        CommonKeybindings.CTRLCMD_V = KeyMod.CtrlCmd | KeyCode.KEY_V;
        return CommonKeybindings;
    }());
    exports.CommonKeybindings = CommonKeybindings;
    var Keybinding = (function () {
        function Keybinding(keybinding) {
            this.value = keybinding;
        }
        /**
         * Format the binding to a format appropiate for rendering in the UI
         */
        Keybinding._toUSLabel = function (value, Platform) {
            return _asString(value, (Platform.isMacintosh ? MacUIKeyLabelProvider.INSTANCE : ClassicUIKeyLabelProvider.INSTANCE), Platform);
        };
        /**
         * Format the binding to a format appropiate for placing in an aria-label.
         */
        Keybinding._toUSAriaLabel = function (value, Platform) {
            return _asString(value, AriaKeyLabelProvider.INSTANCE, Platform);
        };
        /**
         * Format the binding to a format appropiate for rendering in the UI
         */
        Keybinding._toUSHTMLLabel = function (value, Platform) {
            return _asHTML(value, (Platform.isMacintosh ? MacUIKeyLabelProvider.INSTANCE : ClassicUIKeyLabelProvider.INSTANCE), Platform);
        };
        /**
         * Format the binding to a format appropiate for rendering in the UI
         */
        Keybinding._toCustomLabel = function (value, labelProvider, Platform) {
            return _asString(value, labelProvider, Platform);
        };
        /**
         * Format the binding to a format appropiate for rendering in the UI
         */
        Keybinding._toCustomHTMLLabel = function (value, labelProvider, Platform) {
            return _asHTML(value, labelProvider, Platform);
        };
        /**
         * This prints the binding in a format suitable for electron's accelerators.
         * See https://github.com/atom/electron/blob/master/docs/api/accelerator.md
         */
        Keybinding._toElectronAccelerator = function (value, Platform) {
            if (BinaryKeybindings.hasChord(value)) {
                // Electron cannot handle chords
                return null;
            }
            return _asString(value, ElectronAcceleratorLabelProvider.INSTANCE, Platform);
        };
        Keybinding.getUserSettingsKeybindingRegex = function () {
            if (!this._cachedKeybindingRegex) {
                var numpadKey = 'numpad(0|1|2|3|4|5|6|7|8|9|_multiply|_add|_subtract|_decimal|_divide|_separator)';
                var oemKey = '`|\\-|=|\\[|\\]|\\\\\\\\|;|\'|,|\\.|\\/|oem_8|oem_102';
                var specialKey = 'left|up|right|down|pageup|pagedown|end|home|tab|enter|escape|space|backspace|delete|pausebreak|capslock|insert|contextmenu|numlock|scrolllock';
                var casualKey = '[a-z]|[0-9]|f(1|2|3|4|5|6|7|8|9|10|11|12|13|14|15|16|17|18|19)';
                var key = '((' + [numpadKey, oemKey, specialKey, casualKey].join(')|(') + '))';
                var mod = '((ctrl|shift|alt|cmd|win|meta)\\+)*';
                var keybinding = '(' + mod + key + ')';
                this._cachedKeybindingRegex = '"\\s*(' + keybinding + '(\\s+' + keybinding + ')?' + ')\\s*"';
            }
            return this._cachedKeybindingRegex;
        };
        /**
         * Format the binding to a format appropiate for the user settings file.
         */
        Keybinding.toUserSettingsLabel = function (value, Platform) {
            if (Platform === void 0) { Platform = defaultPlatform; }
            var result = _asString(value, UserSettingsKeyLabelProvider.INSTANCE, Platform);
            result = result.toLowerCase();
            if (Platform.isMacintosh) {
                result = result.replace(/meta/g, 'cmd');
            }
            else if (Platform.isWindows) {
                result = result.replace(/meta/g, 'win');
            }
            return result;
        };
        Keybinding.fromUserSettingsLabel = function (input, Platform) {
            if (Platform === void 0) { Platform = defaultPlatform; }
            if (!input) {
                return null;
            }
            input = input.toLowerCase().trim();
            var ctrlCmd = false, shift = false, alt = false, winCtrl = false, key = '';
            while (/^(ctrl|shift|alt|meta|win|cmd)(\+|\-)/.test(input)) {
                if (/^ctrl(\+|\-)/.test(input)) {
                    if (Platform.isMacintosh) {
                        winCtrl = true;
                    }
                    else {
                        ctrlCmd = true;
                    }
                    input = input.substr('ctrl-'.length);
                }
                if (/^shift(\+|\-)/.test(input)) {
                    shift = true;
                    input = input.substr('shift-'.length);
                }
                if (/^alt(\+|\-)/.test(input)) {
                    alt = true;
                    input = input.substr('alt-'.length);
                }
                if (/^meta(\+|\-)/.test(input)) {
                    if (Platform.isMacintosh) {
                        ctrlCmd = true;
                    }
                    else {
                        winCtrl = true;
                    }
                    input = input.substr('meta-'.length);
                }
                if (/^win(\+|\-)/.test(input)) {
                    if (Platform.isMacintosh) {
                        ctrlCmd = true;
                    }
                    else {
                        winCtrl = true;
                    }
                    input = input.substr('win-'.length);
                }
                if (/^cmd(\+|\-)/.test(input)) {
                    if (Platform.isMacintosh) {
                        ctrlCmd = true;
                    }
                    else {
                        winCtrl = true;
                    }
                    input = input.substr('cmd-'.length);
                }
            }
            var chord = 0;
            var firstSpaceIdx = input.indexOf(' ');
            if (firstSpaceIdx > 0) {
                key = input.substring(0, firstSpaceIdx);
                chord = Keybinding.fromUserSettingsLabel(input.substring(firstSpaceIdx), Platform);
            }
            else {
                key = input;
            }
            var keyCode = USER_SETTINGS.toKeyCode(key);
            var result = 0;
            if (ctrlCmd) {
                result |= KeyMod.CtrlCmd;
            }
            if (shift) {
                result |= KeyMod.Shift;
            }
            if (alt) {
                result |= KeyMod.Alt;
            }
            if (winCtrl) {
                result |= KeyMod.WinCtrl;
            }
            result |= keyCode;
            return KeyMod.chord(result, chord);
        };
        Keybinding.prototype.hasCtrlCmd = function () {
            return BinaryKeybindings.hasCtrlCmd(this.value);
        };
        Keybinding.prototype.hasShift = function () {
            return BinaryKeybindings.hasShift(this.value);
        };
        Keybinding.prototype.hasAlt = function () {
            return BinaryKeybindings.hasAlt(this.value);
        };
        Keybinding.prototype.hasWinCtrl = function () {
            return BinaryKeybindings.hasWinCtrl(this.value);
        };
        Keybinding.prototype.extractKeyCode = function () {
            return BinaryKeybindings.extractKeyCode(this.value);
        };
        /**
         * Format the binding to a format appropiate for rendering in the UI
         */
        Keybinding.prototype._toUSLabel = function (Platform) {
            if (Platform === void 0) { Platform = defaultPlatform; }
            return Keybinding._toUSLabel(this.value, Platform);
        };
        /**
         * Format the binding to a format appropiate for placing in an aria-label.
         */
        Keybinding.prototype._toUSAriaLabel = function (Platform) {
            if (Platform === void 0) { Platform = defaultPlatform; }
            return Keybinding._toUSAriaLabel(this.value, Platform);
        };
        /**
         * Format the binding to a format appropiate for rendering in the UI
         */
        Keybinding.prototype._toUSHTMLLabel = function (Platform) {
            if (Platform === void 0) { Platform = defaultPlatform; }
            return Keybinding._toUSHTMLLabel(this.value, Platform);
        };
        /**
         * Format the binding to a format appropiate for rendering in the UI
         */
        Keybinding.prototype.toCustomLabel = function (labelProvider, Platform) {
            if (Platform === void 0) { Platform = defaultPlatform; }
            return Keybinding._toCustomLabel(this.value, labelProvider, Platform);
        };
        /**
         * Format the binding to a format appropiate for rendering in the UI
         */
        Keybinding.prototype.toCustomHTMLLabel = function (labelProvider, Platform) {
            if (Platform === void 0) { Platform = defaultPlatform; }
            return Keybinding._toCustomHTMLLabel(this.value, labelProvider, Platform);
        };
        /**
         * This prints the binding in a format suitable for electron's accelerators.
         * See https://github.com/atom/electron/blob/master/docs/api/accelerator.md
         */
        Keybinding.prototype._toElectronAccelerator = function (Platform) {
            if (Platform === void 0) { Platform = defaultPlatform; }
            return Keybinding._toElectronAccelerator(this.value, Platform);
        };
        /**
         * Format the binding to a format appropiate for the user settings file.
         */
        Keybinding.prototype.toUserSettingsLabel = function (Platform) {
            if (Platform === void 0) { Platform = defaultPlatform; }
            return Keybinding.toUserSettingsLabel(this.value, Platform);
        };
        Keybinding._cachedKeybindingRegex = null;
        return Keybinding;
    }());
    exports.Keybinding = Keybinding;
    /**
     * Print for Electron
     */
    var ElectronAcceleratorLabelProvider = (function () {
        function ElectronAcceleratorLabelProvider() {
            this.ctrlKeyLabel = 'Ctrl';
            this.shiftKeyLabel = 'Shift';
            this.altKeyLabel = 'Alt';
            this.cmdKeyLabel = 'Cmd';
            this.windowsKeyLabel = 'Super';
            this.modifierSeparator = '+';
        }
        ElectronAcceleratorLabelProvider.prototype.getLabelForKey = function (keyCode) {
            switch (keyCode) {
                case KeyCode.UpArrow:
                    return 'Up';
                case KeyCode.DownArrow:
                    return 'Down';
                case KeyCode.LeftArrow:
                    return 'Left';
                case KeyCode.RightArrow:
                    return 'Right';
            }
            return KeyCode.toString(keyCode);
        };
        ElectronAcceleratorLabelProvider.INSTANCE = new ElectronAcceleratorLabelProvider();
        return ElectronAcceleratorLabelProvider;
    }());
    exports.ElectronAcceleratorLabelProvider = ElectronAcceleratorLabelProvider;
    /**
     * Print for Mac UI
     */
    var MacUIKeyLabelProvider = (function () {
        function MacUIKeyLabelProvider() {
            this.ctrlKeyLabel = '\u2303';
            this.shiftKeyLabel = '\u21E7';
            this.altKeyLabel = '\u2325';
            this.cmdKeyLabel = '\u2318';
            this.windowsKeyLabel = nls.localize(0, null);
            this.modifierSeparator = '';
        }
        MacUIKeyLabelProvider.prototype.getLabelForKey = function (keyCode) {
            switch (keyCode) {
                case KeyCode.LeftArrow:
                    return MacUIKeyLabelProvider.leftArrowUnicodeLabel;
                case KeyCode.UpArrow:
                    return MacUIKeyLabelProvider.upArrowUnicodeLabel;
                case KeyCode.RightArrow:
                    return MacUIKeyLabelProvider.rightArrowUnicodeLabel;
                case KeyCode.DownArrow:
                    return MacUIKeyLabelProvider.downArrowUnicodeLabel;
            }
            return KeyCode.toString(keyCode);
        };
        MacUIKeyLabelProvider.INSTANCE = new MacUIKeyLabelProvider();
        MacUIKeyLabelProvider.leftArrowUnicodeLabel = String.fromCharCode(8592);
        MacUIKeyLabelProvider.upArrowUnicodeLabel = String.fromCharCode(8593);
        MacUIKeyLabelProvider.rightArrowUnicodeLabel = String.fromCharCode(8594);
        MacUIKeyLabelProvider.downArrowUnicodeLabel = String.fromCharCode(8595);
        return MacUIKeyLabelProvider;
    }());
    exports.MacUIKeyLabelProvider = MacUIKeyLabelProvider;
    /**
     * Aria label provider for Mac.
     */
    var AriaKeyLabelProvider = (function () {
        function AriaKeyLabelProvider() {
            this.ctrlKeyLabel = nls.localize(1, null);
            this.shiftKeyLabel = nls.localize(2, null);
            this.altKeyLabel = nls.localize(3, null);
            this.cmdKeyLabel = nls.localize(4, null);
            this.windowsKeyLabel = nls.localize(5, null);
            this.modifierSeparator = '+';
        }
        AriaKeyLabelProvider.prototype.getLabelForKey = function (keyCode) {
            return KeyCode.toString(keyCode);
        };
        AriaKeyLabelProvider.INSTANCE = new MacUIKeyLabelProvider();
        return AriaKeyLabelProvider;
    }());
    exports.AriaKeyLabelProvider = AriaKeyLabelProvider;
    /**
     * Print for Windows, Linux UI
     */
    var ClassicUIKeyLabelProvider = (function () {
        function ClassicUIKeyLabelProvider() {
            this.ctrlKeyLabel = nls.localize(6, null);
            this.shiftKeyLabel = nls.localize(7, null);
            this.altKeyLabel = nls.localize(8, null);
            this.cmdKeyLabel = nls.localize(9, null);
            this.windowsKeyLabel = nls.localize(10, null);
            this.modifierSeparator = '+';
        }
        ClassicUIKeyLabelProvider.prototype.getLabelForKey = function (keyCode) {
            return KeyCode.toString(keyCode);
        };
        ClassicUIKeyLabelProvider.INSTANCE = new ClassicUIKeyLabelProvider();
        return ClassicUIKeyLabelProvider;
    }());
    exports.ClassicUIKeyLabelProvider = ClassicUIKeyLabelProvider;
    /**
     * Print for the user settings file.
     */
    var UserSettingsKeyLabelProvider = (function () {
        function UserSettingsKeyLabelProvider() {
            this.ctrlKeyLabel = 'Ctrl';
            this.shiftKeyLabel = 'Shift';
            this.altKeyLabel = 'Alt';
            this.cmdKeyLabel = 'Meta';
            this.windowsKeyLabel = 'Meta';
            this.modifierSeparator = '+';
        }
        UserSettingsKeyLabelProvider.prototype.getLabelForKey = function (keyCode) {
            return USER_SETTINGS.fromKeyCode(keyCode);
        };
        UserSettingsKeyLabelProvider.INSTANCE = new UserSettingsKeyLabelProvider();
        return UserSettingsKeyLabelProvider;
    }());
    function _asString(keybinding, labelProvider, Platform) {
        var result = [], ctrlCmd = BinaryKeybindings.hasCtrlCmd(keybinding), shift = BinaryKeybindings.hasShift(keybinding), alt = BinaryKeybindings.hasAlt(keybinding), winCtrl = BinaryKeybindings.hasWinCtrl(keybinding), keyCode = BinaryKeybindings.extractKeyCode(keybinding);
        var keyLabel = labelProvider.getLabelForKey(keyCode);
        if (!keyLabel) {
            // cannot trigger this key code under this kb layout
            return '';
        }
        // translate modifier keys: Ctrl-Shift-Alt-Meta
        if ((ctrlCmd && !Platform.isMacintosh) || (winCtrl && Platform.isMacintosh)) {
            result.push(labelProvider.ctrlKeyLabel);
        }
        if (shift) {
            result.push(labelProvider.shiftKeyLabel);
        }
        if (alt) {
            result.push(labelProvider.altKeyLabel);
        }
        if (ctrlCmd && Platform.isMacintosh) {
            result.push(labelProvider.cmdKeyLabel);
        }
        if (winCtrl && !Platform.isMacintosh) {
            result.push(labelProvider.windowsKeyLabel);
        }
        // the actual key
        result.push(keyLabel);
        var actualResult = result.join(labelProvider.modifierSeparator);
        if (BinaryKeybindings.hasChord(keybinding)) {
            return actualResult + ' ' + _asString(BinaryKeybindings.extractChordPart(keybinding), labelProvider, Platform);
        }
        return actualResult;
    }
    function _pushKey(result, str) {
        if (result.length > 0) {
            result.push({
                tagName: 'span',
                text: '+'
            });
        }
        result.push({
            tagName: 'span',
            className: 'monaco-kbkey',
            text: str
        });
    }
    function _asHTML(keybinding, labelProvider, Platform, isChord) {
        if (isChord === void 0) { isChord = false; }
        var result = [], ctrlCmd = BinaryKeybindings.hasCtrlCmd(keybinding), shift = BinaryKeybindings.hasShift(keybinding), alt = BinaryKeybindings.hasAlt(keybinding), winCtrl = BinaryKeybindings.hasWinCtrl(keybinding), keyCode = BinaryKeybindings.extractKeyCode(keybinding);
        var keyLabel = labelProvider.getLabelForKey(keyCode);
        if (!keyLabel) {
            // cannot trigger this key code under this kb layout
            return [];
        }
        // translate modifier keys: Ctrl-Shift-Alt-Meta
        if ((ctrlCmd && !Platform.isMacintosh) || (winCtrl && Platform.isMacintosh)) {
            _pushKey(result, labelProvider.ctrlKeyLabel);
        }
        if (shift) {
            _pushKey(result, labelProvider.shiftKeyLabel);
        }
        if (alt) {
            _pushKey(result, labelProvider.altKeyLabel);
        }
        if (ctrlCmd && Platform.isMacintosh) {
            _pushKey(result, labelProvider.cmdKeyLabel);
        }
        if (winCtrl && !Platform.isMacintosh) {
            _pushKey(result, labelProvider.windowsKeyLabel);
        }
        // the actual key
        _pushKey(result, keyLabel);
        var chordTo = null;
        if (BinaryKeybindings.hasChord(keybinding)) {
            chordTo = _asHTML(BinaryKeybindings.extractChordPart(keybinding), labelProvider, Platform, true);
            result.push({
                tagName: 'span',
                text: ' '
            });
            result = result.concat(chordTo);
        }
        if (isChord) {
            return result;
        }
        return [{
                tagName: 'span',
                className: 'monaco-kb',
                children: result
            }];
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/keyboardEvent", ["require", "exports", 'vs/base/common/keyCodes', 'vs/base/common/platform', 'vs/base/browser/browser'], function (require, exports, keyCodes_1, platform, browser) {
    'use strict';
    var KEY_CODE_MAP = {};
    (function () {
        KEY_CODE_MAP[8] = keyCodes_1.KeyCode.Backspace;
        KEY_CODE_MAP[9] = keyCodes_1.KeyCode.Tab;
        KEY_CODE_MAP[13] = keyCodes_1.KeyCode.Enter;
        KEY_CODE_MAP[16] = keyCodes_1.KeyCode.Shift;
        KEY_CODE_MAP[17] = keyCodes_1.KeyCode.Ctrl;
        KEY_CODE_MAP[18] = keyCodes_1.KeyCode.Alt;
        KEY_CODE_MAP[19] = keyCodes_1.KeyCode.PauseBreak;
        KEY_CODE_MAP[20] = keyCodes_1.KeyCode.CapsLock;
        KEY_CODE_MAP[27] = keyCodes_1.KeyCode.Escape;
        KEY_CODE_MAP[32] = keyCodes_1.KeyCode.Space;
        KEY_CODE_MAP[33] = keyCodes_1.KeyCode.PageUp;
        KEY_CODE_MAP[34] = keyCodes_1.KeyCode.PageDown;
        KEY_CODE_MAP[35] = keyCodes_1.KeyCode.End;
        KEY_CODE_MAP[36] = keyCodes_1.KeyCode.Home;
        KEY_CODE_MAP[37] = keyCodes_1.KeyCode.LeftArrow;
        KEY_CODE_MAP[38] = keyCodes_1.KeyCode.UpArrow;
        KEY_CODE_MAP[39] = keyCodes_1.KeyCode.RightArrow;
        KEY_CODE_MAP[40] = keyCodes_1.KeyCode.DownArrow;
        KEY_CODE_MAP[45] = keyCodes_1.KeyCode.Insert;
        KEY_CODE_MAP[46] = keyCodes_1.KeyCode.Delete;
        KEY_CODE_MAP[48] = keyCodes_1.KeyCode.KEY_0;
        KEY_CODE_MAP[49] = keyCodes_1.KeyCode.KEY_1;
        KEY_CODE_MAP[50] = keyCodes_1.KeyCode.KEY_2;
        KEY_CODE_MAP[51] = keyCodes_1.KeyCode.KEY_3;
        KEY_CODE_MAP[52] = keyCodes_1.KeyCode.KEY_4;
        KEY_CODE_MAP[53] = keyCodes_1.KeyCode.KEY_5;
        KEY_CODE_MAP[54] = keyCodes_1.KeyCode.KEY_6;
        KEY_CODE_MAP[55] = keyCodes_1.KeyCode.KEY_7;
        KEY_CODE_MAP[56] = keyCodes_1.KeyCode.KEY_8;
        KEY_CODE_MAP[57] = keyCodes_1.KeyCode.KEY_9;
        KEY_CODE_MAP[65] = keyCodes_1.KeyCode.KEY_A;
        KEY_CODE_MAP[66] = keyCodes_1.KeyCode.KEY_B;
        KEY_CODE_MAP[67] = keyCodes_1.KeyCode.KEY_C;
        KEY_CODE_MAP[68] = keyCodes_1.KeyCode.KEY_D;
        KEY_CODE_MAP[69] = keyCodes_1.KeyCode.KEY_E;
        KEY_CODE_MAP[70] = keyCodes_1.KeyCode.KEY_F;
        KEY_CODE_MAP[71] = keyCodes_1.KeyCode.KEY_G;
        KEY_CODE_MAP[72] = keyCodes_1.KeyCode.KEY_H;
        KEY_CODE_MAP[73] = keyCodes_1.KeyCode.KEY_I;
        KEY_CODE_MAP[74] = keyCodes_1.KeyCode.KEY_J;
        KEY_CODE_MAP[75] = keyCodes_1.KeyCode.KEY_K;
        KEY_CODE_MAP[76] = keyCodes_1.KeyCode.KEY_L;
        KEY_CODE_MAP[77] = keyCodes_1.KeyCode.KEY_M;
        KEY_CODE_MAP[78] = keyCodes_1.KeyCode.KEY_N;
        KEY_CODE_MAP[79] = keyCodes_1.KeyCode.KEY_O;
        KEY_CODE_MAP[80] = keyCodes_1.KeyCode.KEY_P;
        KEY_CODE_MAP[81] = keyCodes_1.KeyCode.KEY_Q;
        KEY_CODE_MAP[82] = keyCodes_1.KeyCode.KEY_R;
        KEY_CODE_MAP[83] = keyCodes_1.KeyCode.KEY_S;
        KEY_CODE_MAP[84] = keyCodes_1.KeyCode.KEY_T;
        KEY_CODE_MAP[85] = keyCodes_1.KeyCode.KEY_U;
        KEY_CODE_MAP[86] = keyCodes_1.KeyCode.KEY_V;
        KEY_CODE_MAP[87] = keyCodes_1.KeyCode.KEY_W;
        KEY_CODE_MAP[88] = keyCodes_1.KeyCode.KEY_X;
        KEY_CODE_MAP[89] = keyCodes_1.KeyCode.KEY_Y;
        KEY_CODE_MAP[90] = keyCodes_1.KeyCode.KEY_Z;
        KEY_CODE_MAP[93] = keyCodes_1.KeyCode.ContextMenu;
        KEY_CODE_MAP[96] = keyCodes_1.KeyCode.NUMPAD_0;
        KEY_CODE_MAP[97] = keyCodes_1.KeyCode.NUMPAD_1;
        KEY_CODE_MAP[98] = keyCodes_1.KeyCode.NUMPAD_2;
        KEY_CODE_MAP[99] = keyCodes_1.KeyCode.NUMPAD_3;
        KEY_CODE_MAP[100] = keyCodes_1.KeyCode.NUMPAD_4;
        KEY_CODE_MAP[101] = keyCodes_1.KeyCode.NUMPAD_5;
        KEY_CODE_MAP[102] = keyCodes_1.KeyCode.NUMPAD_6;
        KEY_CODE_MAP[103] = keyCodes_1.KeyCode.NUMPAD_7;
        KEY_CODE_MAP[104] = keyCodes_1.KeyCode.NUMPAD_8;
        KEY_CODE_MAP[105] = keyCodes_1.KeyCode.NUMPAD_9;
        KEY_CODE_MAP[106] = keyCodes_1.KeyCode.NUMPAD_MULTIPLY;
        KEY_CODE_MAP[107] = keyCodes_1.KeyCode.NUMPAD_ADD;
        KEY_CODE_MAP[108] = keyCodes_1.KeyCode.NUMPAD_SEPARATOR;
        KEY_CODE_MAP[109] = keyCodes_1.KeyCode.NUMPAD_SUBTRACT;
        KEY_CODE_MAP[110] = keyCodes_1.KeyCode.NUMPAD_DECIMAL;
        KEY_CODE_MAP[111] = keyCodes_1.KeyCode.NUMPAD_DIVIDE;
        KEY_CODE_MAP[112] = keyCodes_1.KeyCode.F1;
        KEY_CODE_MAP[113] = keyCodes_1.KeyCode.F2;
        KEY_CODE_MAP[114] = keyCodes_1.KeyCode.F3;
        KEY_CODE_MAP[115] = keyCodes_1.KeyCode.F4;
        KEY_CODE_MAP[116] = keyCodes_1.KeyCode.F5;
        KEY_CODE_MAP[117] = keyCodes_1.KeyCode.F6;
        KEY_CODE_MAP[118] = keyCodes_1.KeyCode.F7;
        KEY_CODE_MAP[119] = keyCodes_1.KeyCode.F8;
        KEY_CODE_MAP[120] = keyCodes_1.KeyCode.F9;
        KEY_CODE_MAP[121] = keyCodes_1.KeyCode.F10;
        KEY_CODE_MAP[122] = keyCodes_1.KeyCode.F11;
        KEY_CODE_MAP[123] = keyCodes_1.KeyCode.F12;
        KEY_CODE_MAP[124] = keyCodes_1.KeyCode.F13;
        KEY_CODE_MAP[125] = keyCodes_1.KeyCode.F14;
        KEY_CODE_MAP[126] = keyCodes_1.KeyCode.F15;
        KEY_CODE_MAP[127] = keyCodes_1.KeyCode.F16;
        KEY_CODE_MAP[128] = keyCodes_1.KeyCode.F17;
        KEY_CODE_MAP[129] = keyCodes_1.KeyCode.F18;
        KEY_CODE_MAP[130] = keyCodes_1.KeyCode.F19;
        KEY_CODE_MAP[144] = keyCodes_1.KeyCode.NumLock;
        KEY_CODE_MAP[145] = keyCodes_1.KeyCode.ScrollLock;
        KEY_CODE_MAP[186] = keyCodes_1.KeyCode.US_SEMICOLON;
        KEY_CODE_MAP[187] = keyCodes_1.KeyCode.US_EQUAL;
        KEY_CODE_MAP[188] = keyCodes_1.KeyCode.US_COMMA;
        KEY_CODE_MAP[189] = keyCodes_1.KeyCode.US_MINUS;
        KEY_CODE_MAP[190] = keyCodes_1.KeyCode.US_DOT;
        KEY_CODE_MAP[191] = keyCodes_1.KeyCode.US_SLASH;
        KEY_CODE_MAP[192] = keyCodes_1.KeyCode.US_BACKTICK;
        KEY_CODE_MAP[219] = keyCodes_1.KeyCode.US_OPEN_SQUARE_BRACKET;
        KEY_CODE_MAP[220] = keyCodes_1.KeyCode.US_BACKSLASH;
        KEY_CODE_MAP[221] = keyCodes_1.KeyCode.US_CLOSE_SQUARE_BRACKET;
        KEY_CODE_MAP[222] = keyCodes_1.KeyCode.US_QUOTE;
        KEY_CODE_MAP[223] = keyCodes_1.KeyCode.OEM_8;
        KEY_CODE_MAP[226] = keyCodes_1.KeyCode.OEM_102;
        if (browser.isIE11orEarlier) {
            KEY_CODE_MAP[91] = keyCodes_1.KeyCode.Meta;
        }
        else if (browser.isFirefox) {
            KEY_CODE_MAP[59] = keyCodes_1.KeyCode.US_SEMICOLON;
            KEY_CODE_MAP[107] = keyCodes_1.KeyCode.US_EQUAL;
            KEY_CODE_MAP[109] = keyCodes_1.KeyCode.US_MINUS;
            if (platform.isMacintosh) {
                KEY_CODE_MAP[224] = keyCodes_1.KeyCode.Meta;
            }
        }
        else if (browser.isWebKit) {
            KEY_CODE_MAP[91] = keyCodes_1.KeyCode.Meta;
            if (platform.isMacintosh) {
                // the two meta keys in the Mac have different key codes (91 and 93)
                KEY_CODE_MAP[93] = keyCodes_1.KeyCode.Meta;
            }
            else {
                KEY_CODE_MAP[92] = keyCodes_1.KeyCode.Meta;
            }
        }
    })();
    function lookupKeyCode(e) {
        return KEY_CODE_MAP[e.keyCode] || keyCodes_1.KeyCode.Unknown;
    }
    exports.lookupKeyCode = lookupKeyCode;
    var extractKeyCode = function extractKeyCode(e) {
        if (e.charCode) {
            // "keypress" events mostly
            var char = String.fromCharCode(e.charCode).toUpperCase();
            return keyCodes_1.KeyCode.fromString(char);
        }
        return lookupKeyCode(e);
    };
    function setExtractKeyCode(newExtractKeyCode) {
        extractKeyCode = newExtractKeyCode;
    }
    exports.setExtractKeyCode = setExtractKeyCode;
    var ctrlKeyMod = (platform.isMacintosh ? keyCodes_1.KeyMod.WinCtrl : keyCodes_1.KeyMod.CtrlCmd);
    var altKeyMod = keyCodes_1.KeyMod.Alt;
    var shiftKeyMod = keyCodes_1.KeyMod.Shift;
    var metaKeyMod = (platform.isMacintosh ? keyCodes_1.KeyMod.CtrlCmd : keyCodes_1.KeyMod.WinCtrl);
    var StandardKeyboardEvent = (function () {
        function StandardKeyboardEvent(source) {
            if (source instanceof StandardKeyboardEvent) {
                this.browserEvent = null;
                this.target = source.target;
                this.ctrlKey = source.ctrlKey;
                this.shiftKey = source.shiftKey;
                this.altKey = source.altKey;
                this.metaKey = source.metaKey;
                this.keyCode = source.keyCode;
                this._asKeybinding = source._asKeybinding;
            }
            else {
                var e = source;
                this.browserEvent = e;
                this.target = e.target || e.targetNode;
                this.ctrlKey = e.ctrlKey;
                this.shiftKey = e.shiftKey;
                this.altKey = e.altKey;
                this.metaKey = e.metaKey;
                this.keyCode = extractKeyCode(e);
                // console.info(e.type + ": keyCode: " + e.keyCode + ", which: " + e.which + ", charCode: " + e.charCode + ", detail: " + e.detail + " ====> " + this.keyCode + ' -- ' + KeyCode[this.keyCode]);
                this.ctrlKey = this.ctrlKey || this.keyCode === keyCodes_1.KeyCode.Ctrl;
                this.altKey = this.altKey || this.keyCode === keyCodes_1.KeyCode.Alt;
                this.shiftKey = this.shiftKey || this.keyCode === keyCodes_1.KeyCode.Shift;
                this.metaKey = this.metaKey || this.keyCode === keyCodes_1.KeyCode.Meta;
                this._asKeybinding = this._computeKeybinding();
            }
        }
        StandardKeyboardEvent.prototype.preventDefault = function () {
            if (this.browserEvent && this.browserEvent.preventDefault) {
                this.browserEvent.preventDefault();
            }
        };
        StandardKeyboardEvent.prototype.stopPropagation = function () {
            if (this.browserEvent && this.browserEvent.stopPropagation) {
                this.browserEvent.stopPropagation();
            }
        };
        StandardKeyboardEvent.prototype.clone = function () {
            return new StandardKeyboardEvent(this);
        };
        StandardKeyboardEvent.prototype.asKeybinding = function () {
            return this._asKeybinding;
        };
        StandardKeyboardEvent.prototype.equals = function (other) {
            return (this._asKeybinding === other);
        };
        StandardKeyboardEvent.prototype._computeKeybinding = function () {
            var key = keyCodes_1.KeyCode.Unknown;
            if (this.keyCode !== keyCodes_1.KeyCode.Ctrl && this.keyCode !== keyCodes_1.KeyCode.Shift && this.keyCode !== keyCodes_1.KeyCode.Alt && this.keyCode !== keyCodes_1.KeyCode.Meta) {
                key = this.keyCode;
            }
            var result = 0;
            if (this.ctrlKey) {
                result |= ctrlKeyMod;
            }
            if (this.altKey) {
                result |= altKeyMod;
            }
            if (this.shiftKey) {
                result |= shiftKeyMod;
            }
            if (this.metaKey) {
                result |= metaKeyMod;
            }
            result |= key;
            return result;
        };
        return StandardKeyboardEvent;
    }());
    exports.StandardKeyboardEvent = StandardKeyboardEvent;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/dom", ["require", "exports", 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/eventEmitter', 'vs/base/common/lifecycle', 'vs/base/common/types', 'vs/base/browser/browser', 'vs/base/browser/browserService', 'vs/base/browser/keyboardEvent', 'vs/base/browser/mouseEvent'], function (require, exports, async_1, errors_1, eventEmitter_1, lifecycle_1, types_1, browser_1, browserService_1, keyboardEvent_1, mouseEvent_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function clearNode(node) {
        while (node.firstChild) {
            node.removeChild(node.firstChild);
        }
    }
    exports.clearNode = clearNode;
    /**
     * Calls JSON.Stringify with a replacer to break apart any circular references.
     * This prevents JSON.stringify from throwing the exception
     *  "Uncaught TypeError: Converting circular structure to JSON"
     */
    function safeStringifyDOMAware(obj) {
        var seen = [];
        return JSON.stringify(obj, function (key, value) {
            // HTML elements are never going to serialize nicely
            if (value instanceof Element) {
                return '[Element]';
            }
            if (types_1.isObject(value) || Array.isArray(value)) {
                if (seen.indexOf(value) !== -1) {
                    return '[Circular]';
                }
                else {
                    seen.push(value);
                }
            }
            return value;
        });
    }
    exports.safeStringifyDOMAware = safeStringifyDOMAware;
    function isInDOM(node) {
        while (node) {
            if (node === document.body) {
                return true;
            }
            node = node.parentNode;
        }
        return false;
    }
    exports.isInDOM = isInDOM;
    var _blank = ' '.charCodeAt(0);
    var lastStart, lastEnd;
    function _findClassName(node, className) {
        var classes = node.className;
        if (!classes) {
            lastStart = -1;
            return;
        }
        className = className.trim();
        var classesLen = classes.length, classLen = className.length;
        if (classLen === 0) {
            lastStart = -1;
            return;
        }
        if (classesLen < classLen) {
            lastStart = -1;
            return;
        }
        if (classes === className) {
            lastStart = 0;
            lastEnd = classesLen;
            return;
        }
        var idx = -1, idxEnd;
        while ((idx = classes.indexOf(className, idx + 1)) >= 0) {
            idxEnd = idx + classLen;
            // a class that is followed by another class
            if ((idx === 0 || classes.charCodeAt(idx - 1) === _blank) && classes.charCodeAt(idxEnd) === _blank) {
                lastStart = idx;
                lastEnd = idxEnd + 1;
                return;
            }
            // last class
            if (idx > 0 && classes.charCodeAt(idx - 1) === _blank && idxEnd === classesLen) {
                lastStart = idx - 1;
                lastEnd = idxEnd;
                return;
            }
            // equal - duplicate of cmp above
            if (idx === 0 && idxEnd === classesLen) {
                lastStart = 0;
                lastEnd = idxEnd;
                return;
            }
        }
        lastStart = -1;
    }
    /**
     * @param node a dom node
     * @param className a class name
     * @return true if the className attribute of the provided node contains the provided className
     */
    function hasClass(node, className) {
        _findClassName(node, className);
        return lastStart !== -1;
    }
    exports.hasClass = hasClass;
    /**
     * Adds the provided className to the provided node. This is a no-op
     * if the class is already set.
     * @param node a dom node
     * @param className a class name
     */
    function addClass(node, className) {
        if (!node.className) {
            node.className = className;
        }
        else {
            _findClassName(node, className); // see if it's already there
            if (lastStart === -1) {
                node.className = node.className + ' ' + className;
            }
        }
    }
    exports.addClass = addClass;
    /**
     * Removes the className for the provided node. This is a no-op
     * if the class isn't present.
     * @param node a dom node
     * @param className a class name
     */
    function removeClass(node, className) {
        _findClassName(node, className);
        if (lastStart === -1) {
            return; // Prevent styles invalidation if not necessary
        }
        else {
            node.className = node.className.substring(0, lastStart) + node.className.substring(lastEnd);
        }
    }
    exports.removeClass = removeClass;
    /**
     * @param node a dom node
     * @param className a class name
     * @param shouldHaveIt
     */
    function toggleClass(node, className, shouldHaveIt) {
        _findClassName(node, className);
        if (lastStart !== -1 && !shouldHaveIt) {
            removeClass(node, className);
        }
        if (lastStart === -1 && shouldHaveIt) {
            addClass(node, className);
        }
    }
    exports.toggleClass = toggleClass;
    var DomListener = (function (_super) {
        __extends(DomListener, _super);
        function DomListener(node, type, handler, useCapture) {
            _super.call(this);
            this._node = node;
            this._type = type;
            this._useCapture = (useCapture || false);
            this._wrapHandler = function (e) {
                e = e || window.event;
                handler(e);
            };
            if (typeof this._node.addEventListener === 'function') {
                this._usedAddEventListener = true;
                this._node.addEventListener(this._type, this._wrapHandler, this._useCapture);
            }
            else {
                this._usedAddEventListener = false;
                this._node.attachEvent('on' + this._type, this._wrapHandler);
            }
        }
        DomListener.prototype.dispose = function () {
            if (!this._wrapHandler) {
                // Already disposed
                return;
            }
            if (this._usedAddEventListener) {
                this._node.removeEventListener(this._type, this._wrapHandler, this._useCapture);
            }
            else {
                this._node.detachEvent('on' + this._type, this._wrapHandler);
            }
            // Prevent leakers from holding on to the dom or handler func
            this._node = null;
            this._wrapHandler = null;
        };
        return DomListener;
    }(lifecycle_1.Disposable));
    function addDisposableListener(node, type, handler, useCapture) {
        return new DomListener(node, type, handler, useCapture);
    }
    exports.addDisposableListener = addDisposableListener;
    function _wrapAsStandardMouseEvent(handler) {
        return function (e) {
            return handler(new mouseEvent_1.StandardMouseEvent(e));
        };
    }
    function _wrapAsStandardKeyboardEvent(handler) {
        return function (e) {
            return handler(new keyboardEvent_1.StandardKeyboardEvent(e));
        };
    }
    exports.addStandardDisposableListener = function addStandardDisposableListener(node, type, handler, useCapture) {
        var wrapHandler = handler;
        if (type === 'click') {
            wrapHandler = _wrapAsStandardMouseEvent(handler);
        }
        else if (type === 'keydown' || type === 'keypress' || type === 'keyup') {
            wrapHandler = _wrapAsStandardKeyboardEvent(handler);
        }
        node.addEventListener(type, wrapHandler, useCapture || false);
        return {
            dispose: function () {
                if (!wrapHandler) {
                    // Already removed
                    return;
                }
                node.removeEventListener(type, wrapHandler, useCapture || false);
                // Prevent leakers from holding on to the dom node or handler func
                wrapHandler = null;
                node = null;
                handler = null;
            }
        };
    };
    function addDisposableNonBubblingMouseOutListener(node, handler) {
        return addDisposableListener(node, 'mouseout', function (e) {
            // Mouse out bubbles, so this is an attempt to ignore faux mouse outs coming from children elements
            var toElement = (e.relatedTarget || e.toElement);
            while (toElement && toElement !== node) {
                toElement = toElement.parentNode;
            }
            if (toElement === node) {
                return;
            }
            handler(e);
        });
    }
    exports.addDisposableNonBubblingMouseOutListener = addDisposableNonBubblingMouseOutListener;
    var _animationFrame = (function () {
        var emulatedRequestAnimationFrame = function (callback) {
            return setTimeout(function () { return callback(new Date().getTime()); }, 0);
        };
        var nativeRequestAnimationFrame = self.requestAnimationFrame
            || self.msRequestAnimationFrame
            || self.webkitRequestAnimationFrame
            || self.mozRequestAnimationFrame
            || self.oRequestAnimationFrame;
        var emulatedCancelAnimationFrame = function (id) { };
        var nativeCancelAnimationFrame = self.cancelAnimationFrame || self.cancelRequestAnimationFrame
            || self.msCancelAnimationFrame || self.msCancelRequestAnimationFrame
            || self.webkitCancelAnimationFrame || self.webkitCancelRequestAnimationFrame
            || self.mozCancelAnimationFrame || self.mozCancelRequestAnimationFrame
            || self.oCancelAnimationFrame || self.oCancelRequestAnimationFrame;
        var isNative = !!nativeRequestAnimationFrame;
        var request = nativeRequestAnimationFrame || emulatedRequestAnimationFrame;
        var cancel = nativeCancelAnimationFrame || emulatedCancelAnimationFrame;
        return {
            isNative: isNative,
            request: function (callback) {
                return request(callback);
            },
            cancel: function (id) {
                return cancel(id);
            }
        };
    })();
    var AnimationFrameQueueItem = (function () {
        function AnimationFrameQueueItem(runner, priority) {
            this._runner = runner;
            this.priority = priority;
            this._canceled = false;
        }
        AnimationFrameQueueItem.prototype.dispose = function () {
            this._canceled = true;
        };
        AnimationFrameQueueItem.prototype.execute = function () {
            if (this._canceled) {
                return;
            }
            try {
                this._runner();
            }
            catch (e) {
                errors_1.onUnexpectedError(e);
            }
        };
        // Sort by priority (largest to lowest)
        AnimationFrameQueueItem.sort = function (a, b) {
            return b.priority - a.priority;
        };
        return AnimationFrameQueueItem;
    }());
    (function () {
        /**
         * The runners scheduled at the next animation frame
         */
        var NEXT_QUEUE = [];
        /**
         * The runners scheduled at the current animation frame
         */
        var CURRENT_QUEUE = null;
        /**
         * A flag to keep track if the native requestAnimationFrame was already called
         */
        var animFrameRequested = false;
        /**
         * A flag to indicate if currently handling a native requestAnimationFrame callback
         */
        var inAnimationFrameRunner = false;
        var animationFrameRunner = function () {
            animFrameRequested = false;
            CURRENT_QUEUE = NEXT_QUEUE;
            NEXT_QUEUE = [];
            inAnimationFrameRunner = true;
            while (CURRENT_QUEUE.length > 0) {
                CURRENT_QUEUE.sort(AnimationFrameQueueItem.sort);
                var top_1 = CURRENT_QUEUE.shift();
                top_1.execute();
            }
            inAnimationFrameRunner = false;
        };
        exports.scheduleAtNextAnimationFrame = function (runner, priority) {
            if (priority === void 0) { priority = 0; }
            var item = new AnimationFrameQueueItem(runner, priority);
            NEXT_QUEUE.push(item);
            if (!animFrameRequested) {
                animFrameRequested = true;
                // TODO@Alex: also check if it is electron
                if (browser_1.isChrome) {
                    var handle_1;
                    _animationFrame.request(function () {
                        clearTimeout(handle_1);
                        animationFrameRunner();
                    });
                    // This is a fallback in-case chrome dropped
                    // the request for an animation frame. This
                    // is sick but was spotted in the wild
                    handle_1 = setTimeout(animationFrameRunner, 1000);
                }
                else {
                    _animationFrame.request(animationFrameRunner);
                }
            }
            return item;
        };
        exports.runAtThisOrScheduleAtNextAnimationFrame = function (runner, priority) {
            if (inAnimationFrameRunner) {
                var item = new AnimationFrameQueueItem(runner, priority);
                CURRENT_QUEUE.push(item);
                return item;
            }
            else {
                return exports.scheduleAtNextAnimationFrame(runner, priority);
            }
        };
    })();
    var MINIMUM_TIME_MS = 16;
    var DEFAULT_EVENT_MERGER = function (lastEvent, currentEvent) {
        return currentEvent;
    };
    var TimeoutThrottledDomListener = (function (_super) {
        __extends(TimeoutThrottledDomListener, _super);
        function TimeoutThrottledDomListener(node, type, handler, eventMerger, minimumTimeMs) {
            if (eventMerger === void 0) { eventMerger = DEFAULT_EVENT_MERGER; }
            if (minimumTimeMs === void 0) { minimumTimeMs = MINIMUM_TIME_MS; }
            _super.call(this);
            var lastEvent = null;
            var lastHandlerTime = 0;
            var timeout = this._register(new async_1.TimeoutTimer());
            var invokeHandler = function () {
                lastHandlerTime = (new Date()).getTime();
                handler(lastEvent);
                lastEvent = null;
            };
            this._register(addDisposableListener(node, type, function (e) {
                lastEvent = eventMerger(lastEvent, e);
                var elapsedTime = (new Date()).getTime() - lastHandlerTime;
                if (elapsedTime >= minimumTimeMs) {
                    timeout.cancel();
                    invokeHandler();
                }
                else {
                    timeout.setIfNotSet(invokeHandler, minimumTimeMs - elapsedTime);
                }
            }));
        }
        return TimeoutThrottledDomListener;
    }(lifecycle_1.Disposable));
    function addDisposableThrottledListener(node, type, handler, eventMerger, minimumTimeMs) {
        return new TimeoutThrottledDomListener(node, type, handler, eventMerger, minimumTimeMs);
    }
    exports.addDisposableThrottledListener = addDisposableThrottledListener;
    function getComputedStyle(el) {
        return document.defaultView.getComputedStyle(el, null);
    }
    exports.getComputedStyle = getComputedStyle;
    // Adapted from WinJS
    // Converts a CSS positioning string for the specified element to pixels.
    var convertToPixels = (function () {
        return function (element, value) {
            return parseFloat(value) || 0;
        };
    })();
    function getDimension(element, cssPropertyName, jsPropertyName) {
        var computedStyle = getComputedStyle(element);
        var value = '0';
        if (computedStyle) {
            if (computedStyle.getPropertyValue) {
                value = computedStyle.getPropertyValue(cssPropertyName);
            }
            else {
                // IE8
                value = computedStyle.getAttribute(jsPropertyName);
            }
        }
        return convertToPixels(element, value);
    }
    var sizeUtils = {
        getBorderLeftWidth: function (element) {
            return getDimension(element, 'border-left-width', 'borderLeftWidth');
        },
        getBorderTopWidth: function (element) {
            return getDimension(element, 'border-top-width', 'borderTopWidth');
        },
        getBorderRightWidth: function (element) {
            return getDimension(element, 'border-right-width', 'borderRightWidth');
        },
        getBorderBottomWidth: function (element) {
            return getDimension(element, 'border-bottom-width', 'borderBottomWidth');
        },
        getPaddingLeft: function (element) {
            return getDimension(element, 'padding-left', 'paddingLeft');
        },
        getPaddingTop: function (element) {
            return getDimension(element, 'padding-top', 'paddingTop');
        },
        getPaddingRight: function (element) {
            return getDimension(element, 'padding-right', 'paddingRight');
        },
        getPaddingBottom: function (element) {
            return getDimension(element, 'padding-bottom', 'paddingBottom');
        },
        getMarginLeft: function (element) {
            return getDimension(element, 'margin-left', 'marginLeft');
        },
        getMarginTop: function (element) {
            return getDimension(element, 'margin-top', 'marginTop');
        },
        getMarginRight: function (element) {
            return getDimension(element, 'margin-right', 'marginRight');
        },
        getMarginBottom: function (element) {
            return getDimension(element, 'margin-bottom', 'marginBottom');
        },
        __commaSentinel: false
    };
    // ----------------------------------------------------------------------------------------
    // Position & Dimension
    function getTopLeftOffset(element) {
        // Adapted from WinJS.Utilities.getPosition
        // and added borders to the mix
        var offsetParent = element.offsetParent, top = element.offsetTop, left = element.offsetLeft;
        while ((element = element.parentNode) !== null && element !== document.body && element !== document.documentElement) {
            top -= element.scrollTop;
            var c = getComputedStyle(element);
            if (c) {
                left -= c.direction !== 'rtl' ? element.scrollLeft : -element.scrollLeft;
            }
            if (element === offsetParent) {
                left += sizeUtils.getBorderLeftWidth(element);
                top += sizeUtils.getBorderTopWidth(element);
                top += element.offsetTop;
                left += element.offsetLeft;
                offsetParent = element.offsetParent;
            }
        }
        return {
            left: left,
            top: top
        };
    }
    exports.getTopLeftOffset = getTopLeftOffset;
    function getDomNodePosition(domNode) {
        var r = getTopLeftOffset(domNode);
        return {
            left: r.left,
            top: r.top,
            width: domNode.clientWidth,
            height: domNode.clientHeight
        };
    }
    exports.getDomNodePosition = getDomNodePosition;
    // Adapted from WinJS
    // Gets the width of the content of the specified element. The content width does not include borders or padding.
    function getContentWidth(element) {
        var border = sizeUtils.getBorderLeftWidth(element) + sizeUtils.getBorderRightWidth(element);
        var padding = sizeUtils.getPaddingLeft(element) + sizeUtils.getPaddingRight(element);
        return element.offsetWidth - border - padding;
    }
    exports.getContentWidth = getContentWidth;
    // Adapted from WinJS
    // Gets the width of the element, including margins.
    function getTotalWidth(element) {
        var margin = sizeUtils.getMarginLeft(element) + sizeUtils.getMarginRight(element);
        return element.offsetWidth + margin;
    }
    exports.getTotalWidth = getTotalWidth;
    // Adapted from WinJS
    // Gets the height of the content of the specified element. The content height does not include borders or padding.
    function getContentHeight(element) {
        var border = sizeUtils.getBorderTopWidth(element) + sizeUtils.getBorderBottomWidth(element);
        var padding = sizeUtils.getPaddingTop(element) + sizeUtils.getPaddingBottom(element);
        return element.offsetHeight - border - padding;
    }
    exports.getContentHeight = getContentHeight;
    // Adapted from WinJS
    // Gets the height of the element, including its margins.
    function getTotalHeight(element) {
        var margin = sizeUtils.getMarginTop(element) + sizeUtils.getMarginBottom(element);
        return element.offsetHeight + margin;
    }
    exports.getTotalHeight = getTotalHeight;
    // Adapted from WinJS
    // Gets the left coordinate of the specified element relative to the specified parent.
    function getRelativeLeft(element, parent) {
        if (element === null) {
            return 0;
        }
        var left = element.offsetLeft;
        var e = element.parentNode;
        while (e !== null) {
            left -= e.offsetLeft;
            if (e === parent) {
                break;
            }
            e = e.parentNode;
        }
        return left;
    }
    exports.getRelativeLeft = getRelativeLeft;
    // Adapted from WinJS
    // Gets the top coordinate of the element relative to the specified parent.
    function getRelativeTop(element, parent) {
        if (element === null) {
            return 0;
        }
        var top = element.offsetTop;
        var e = element.parentNode;
        while (e !== null) {
            top -= e.offsetTop;
            if (e === parent) {
                break;
            }
            e = e.parentNode;
        }
        return top;
    }
    exports.getRelativeTop = getRelativeTop;
    // ----------------------------------------------------------------------------------------
    function isAncestor(testChild, testAncestor) {
        while (testChild) {
            if (testChild === testAncestor) {
                return true;
            }
            testChild = testChild.parentNode;
        }
        return false;
    }
    exports.isAncestor = isAncestor;
    function findParentWithClass(node, clazz, stopAtClazz) {
        while (node) {
            if (hasClass(node, clazz)) {
                return node;
            }
            if (stopAtClazz && hasClass(node, stopAtClazz)) {
                return null;
            }
            node = node.parentNode;
        }
        return null;
    }
    exports.findParentWithClass = findParentWithClass;
    function createStyleSheet() {
        var style = document.createElement('style');
        style.type = 'text/css';
        style.media = 'screen';
        document.getElementsByTagName('head')[0].appendChild(style);
        return style;
    }
    exports.createStyleSheet = createStyleSheet;
    var sharedStyle = createStyleSheet();
    function getDynamicStyleSheetRules(style) {
        if (style && style.sheet && style.sheet.rules) {
            // Chrome, IE
            return style.sheet.rules;
        }
        if (style && style.sheet && style.sheet.cssRules) {
            // FF
            return style.sheet.cssRules;
        }
        return [];
    }
    function createCSSRule(selector, cssText, style) {
        if (style === void 0) { style = sharedStyle; }
        if (!style || !cssText) {
            return;
        }
        style.sheet.insertRule(selector + '{' + cssText + '}', 0);
    }
    exports.createCSSRule = createCSSRule;
    function getCSSRule(selector, style) {
        if (style === void 0) { style = sharedStyle; }
        if (!style) {
            return null;
        }
        var rules = getDynamicStyleSheetRules(style);
        for (var i = 0; i < rules.length; i++) {
            var rule = rules[i];
            var normalizedSelectorText = rule.selectorText.replace(/::/gi, ':');
            if (normalizedSelectorText === selector) {
                return rule;
            }
        }
        return null;
    }
    exports.getCSSRule = getCSSRule;
    function removeCSSRulesWithPrefix(ruleName, style) {
        if (style === void 0) { style = sharedStyle; }
        if (!style) {
            return;
        }
        var rules = getDynamicStyleSheetRules(style);
        var toDelete = [];
        for (var i = 0; i < rules.length; i++) {
            var rule = rules[i];
            var normalizedSelectorText = rule.selectorText.replace(/::/gi, ':');
            if (normalizedSelectorText.indexOf(ruleName) === 0) {
                toDelete.push(i);
            }
        }
        for (var i = toDelete.length - 1; i >= 0; i--) {
            style.sheet.deleteRule(toDelete[i]);
        }
    }
    exports.removeCSSRulesWithPrefix = removeCSSRulesWithPrefix;
    function isHTMLElement(o) {
        return browserService_1.getService().isHTMLElement(o);
    }
    exports.isHTMLElement = isHTMLElement;
    exports.EventType = {
        // Mouse
        CLICK: 'click',
        DBLCLICK: 'dblclick',
        MOUSE_UP: 'mouseup',
        MOUSE_DOWN: 'mousedown',
        MOUSE_OVER: 'mouseover',
        MOUSE_MOVE: 'mousemove',
        MOUSE_OUT: 'mouseout',
        CONTEXT_MENU: 'contextmenu',
        // Keyboard
        KEY_DOWN: 'keydown',
        KEY_PRESS: 'keypress',
        KEY_UP: 'keyup',
        // HTML Document
        LOAD: 'load',
        UNLOAD: 'unload',
        ABORT: 'abort',
        ERROR: 'error',
        RESIZE: 'resize',
        SCROLL: 'scroll',
        // Form
        SELECT: 'select',
        CHANGE: 'change',
        SUBMIT: 'submit',
        RESET: 'reset',
        FOCUS: 'focus',
        BLUR: 'blur',
        INPUT: 'input',
        // Local Storage
        STORAGE: 'storage',
        // Drag
        DRAG_START: 'dragstart',
        DRAG: 'drag',
        DRAG_ENTER: 'dragenter',
        DRAG_LEAVE: 'dragleave',
        DRAG_OVER: 'dragover',
        DROP: 'drop',
        DRAG_END: 'dragend',
        // Animation
        ANIMATION_START: browser_1.isWebKit ? 'webkitAnimationStart' : 'animationstart',
        ANIMATION_END: browser_1.isWebKit ? 'webkitAnimationEnd' : 'animationend',
        ANIMATION_ITERATION: browser_1.isWebKit ? 'webkitAnimationIteration' : 'animationiteration'
    };
    exports.EventHelper = {
        stop: function (e, cancelBubble) {
            if (e.preventDefault) {
                e.preventDefault();
            }
            else {
                // IE8
                e.returnValue = false;
            }
            if (cancelBubble) {
                if (e.stopPropagation) {
                    e.stopPropagation();
                }
                else {
                    // IE8
                    e.cancelBubble = true;
                }
            }
        }
    };
    function saveParentsScrollTop(node) {
        var r = [];
        for (var i = 0; node && node.nodeType === node.ELEMENT_NODE; i++) {
            r[i] = node.scrollTop;
            node = node.parentNode;
        }
        return r;
    }
    exports.saveParentsScrollTop = saveParentsScrollTop;
    function restoreParentsScrollTop(node, state) {
        for (var i = 0; node && node.nodeType === node.ELEMENT_NODE; i++) {
            if (node.scrollTop !== state[i]) {
                node.scrollTop = state[i];
            }
            node = node.parentNode;
        }
    }
    exports.restoreParentsScrollTop = restoreParentsScrollTop;
    var FocusTracker = (function (_super) {
        __extends(FocusTracker, _super);
        function FocusTracker(element) {
            var _this = this;
            _super.call(this);
            var hasFocus = false;
            var loosingFocus = false;
            this._eventEmitter = this._register(new eventEmitter_1.EventEmitter());
            var onFocus = function (event) {
                loosingFocus = false;
                if (!hasFocus) {
                    hasFocus = true;
                    _this._eventEmitter.emit('focus', {});
                }
            };
            var onBlur = function (event) {
                if (hasFocus) {
                    loosingFocus = true;
                    window.setTimeout(function () {
                        if (loosingFocus) {
                            loosingFocus = false;
                            hasFocus = false;
                            _this._eventEmitter.emit('blur', {});
                        }
                    }, 0);
                }
            };
            this._register(addDisposableListener(element, exports.EventType.FOCUS, onFocus, true));
            this._register(addDisposableListener(element, exports.EventType.BLUR, onBlur, true));
        }
        FocusTracker.prototype.addFocusListener = function (fn) {
            return this._eventEmitter.addListener2('focus', fn);
        };
        FocusTracker.prototype.addBlurListener = function (fn) {
            return this._eventEmitter.addListener2('blur', fn);
        };
        return FocusTracker;
    }(lifecycle_1.Disposable));
    function trackFocus(element) {
        return new FocusTracker(element);
    }
    exports.trackFocus = trackFocus;
    function append(parent, child) {
        parent.appendChild(child);
        return child;
    }
    exports.append = append;
    var SELECTOR_REGEX = /([\w\-]+)?(#([\w\-]+))?((.([\w\-]+))*)/;
    // Similar to builder, but much more lightweight
    function emmet(description) {
        var match = SELECTOR_REGEX.exec(description);
        if (!match) {
            throw new Error('Bad use of emmet');
        }
        var result = document.createElement(match[1] || 'div');
        if (match[3]) {
            result.id = match[3];
        }
        if (match[4]) {
            result.className = match[4].replace(/\./g, ' ').trim();
        }
        return result;
    }
    exports.emmet = emmet;
    function show() {
        var elements = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            elements[_i - 0] = arguments[_i];
        }
        for (var _a = 0, elements_1 = elements; _a < elements_1.length; _a++) {
            var element = elements_1[_a];
            element.style.display = null;
        }
    }
    exports.show = show;
    function hide() {
        var elements = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            elements[_i - 0] = arguments[_i];
        }
        for (var _a = 0, elements_2 = elements; _a < elements_2.length; _a++) {
            var element = elements_2[_a];
            element.style.display = 'none';
        }
    }
    exports.hide = hide;
    function findParentWithAttribute(node, attribute) {
        while (node) {
            if (node instanceof HTMLElement && node.hasAttribute(attribute)) {
                return node;
            }
            node = node.parentNode;
        }
        return null;
    }
    function removeTabIndexAndUpdateFocus(node) {
        if (!node || !node.hasAttribute('tabIndex')) {
            return;
        }
        // If we are the currently focused element and tabIndex is removed,
        // standard DOM behavior is to move focus to the <body> element. We
        // typically never want that, rather put focus to the closest element
        // in the hierarchy of the parent DOM nodes.
        if (document.activeElement === node) {
            var parentFocusable = findParentWithAttribute(node.parentElement, 'tabIndex');
            if (parentFocusable) {
                parentFocusable.focus();
            }
        }
        node.removeAttribute('tabindex');
    }
    exports.removeTabIndexAndUpdateFocus = removeTabIndexAndUpdateFocus;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/builder", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/types', 'vs/base/common/lifecycle', 'vs/base/common/strings', 'vs/base/common/assert', 'vs/base/browser/dom', 'vs/base/browser/browserService', 'vs/css!./builder'], function (require, exports, winjs_base_1, types, lifecycle_1, strings, assert, DOM, BrowserService) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Create a new builder from the element that is uniquely identified by the given identifier. If the
     *  second parameter "offdom" is set to true, the created elements will only be added to the provided
     *  element when the build() method is called.
     */
    function withElementById(id, offdom) {
        assert.ok(types.isString(id), 'Expected String as parameter');
        var element = BrowserService.getService().document.getElementById(id);
        if (element) {
            return new Builder(element, offdom);
        }
        return null;
    }
    exports.withElementById = withElementById;
    exports.Build = {
        withElementById: withElementById
    };
    // --- Implementation starts here
    var MS_DATA_KEY = '_msDataKey';
    var DATA_BINDING_ID = '__$binding';
    var LISTENER_BINDING_ID = '__$listeners';
    var VISIBILITY_BINDING_ID = '__$visibility';
    var Position = (function () {
        function Position(x, y) {
            this.x = x;
            this.y = y;
        }
        return Position;
    }());
    exports.Position = Position;
    var Box = (function () {
        function Box(top, right, bottom, left) {
            this.top = top;
            this.right = right;
            this.bottom = bottom;
            this.left = left;
        }
        return Box;
    }());
    exports.Box = Box;
    var Dimension = (function () {
        function Dimension(width, height) {
            this.width = width;
            this.height = height;
        }
        Dimension.prototype.substract = function (box) {
            return new Dimension(this.width - box.left - box.right, this.height - box.top - box.bottom);
        };
        return Dimension;
    }());
    exports.Dimension = Dimension;
    function data(element) {
        if (!element[MS_DATA_KEY]) {
            element[MS_DATA_KEY] = {};
        }
        return element[MS_DATA_KEY];
    }
    function hasData(element) {
        return !!element[MS_DATA_KEY];
    }
    /**
     *  Wraps around the provided element to manipulate it and add more child elements.
     */
    var Builder = (function () {
        function Builder(element, offdom) {
            this.offdom = offdom;
            this.container = element;
            this.currentElement = element;
            this.createdElements = [];
            this.toUnbind = {};
            this.captureToUnbind = {};
            this.browserService = BrowserService.getService();
        }
        /**
         *  Returns a new builder that lets the current HTML Element of this builder be the container
         *  for future additions on the builder.
         */
        Builder.prototype.asContainer = function () {
            return withBuilder(this, this.offdom);
        };
        /**
         *  Clones the builder providing the same properties as this one.
         */
        Builder.prototype.clone = function () {
            var builder = new Builder(this.container, this.offdom);
            builder.currentElement = this.currentElement;
            builder.createdElements = this.createdElements;
            builder.captureToUnbind = this.captureToUnbind;
            builder.toUnbind = this.toUnbind;
            return builder;
        };
        Builder.prototype.and = function (obj) {
            // Convert HTMLElement to Builder as necessary
            if (!(obj instanceof Builder) && !(obj instanceof MultiBuilder)) {
                obj = new Builder(obj, this.offdom);
            }
            // Wrap Builders into MultiBuilder
            var builders = [this];
            if (obj instanceof MultiBuilder) {
                for (var i = 0; i < obj.length; i++) {
                    builders.push(obj.item(i));
                }
            }
            else {
                builders.push(obj);
            }
            return new MultiBuilder(builders);
        };
        Builder.prototype.build = function (container, index) {
            assert.ok(this.offdom, 'This builder was not created off-dom, so build() can not be called.');
            // Use builders own container if present
            if (!container) {
                container = this.container;
            }
            else if (container instanceof Builder) {
                container = container.getHTMLElement();
            }
            assert.ok(container, 'Builder can only be build() with a container provided.');
            assert.ok(DOM.isHTMLElement(container), 'The container must either be a HTMLElement or a Builder.');
            var htmlContainer = container;
            // Append
            var i, len;
            var childNodes = htmlContainer.childNodes;
            if (types.isNumber(index) && index < childNodes.length) {
                for (i = 0, len = this.createdElements.length; i < len; i++) {
                    htmlContainer.insertBefore(this.createdElements[i], childNodes[index++]);
                }
            }
            else {
                for (i = 0, len = this.createdElements.length; i < len; i++) {
                    htmlContainer.appendChild(this.createdElements[i]);
                }
            }
            return this;
        };
        Builder.prototype.appendTo = function (container, index) {
            // Use builders own container if present
            if (!container) {
                container = this.container;
            }
            else if (container instanceof Builder) {
                container = container.getHTMLElement();
            }
            assert.ok(container, 'Builder can only be build() with a container provided.');
            assert.ok(DOM.isHTMLElement(container), 'The container must either be a HTMLElement or a Builder.');
            var htmlContainer = container;
            // Remove node from parent, if needed
            if (this.currentElement.parentNode) {
                this.currentElement.parentNode.removeChild(this.currentElement);
            }
            var childNodes = htmlContainer.childNodes;
            if (types.isNumber(index) && index < childNodes.length) {
                htmlContainer.insertBefore(this.currentElement, childNodes[index]);
            }
            else {
                htmlContainer.appendChild(this.currentElement);
            }
            return this;
        };
        Builder.prototype.append = function (child, index) {
            assert.ok(child, 'Need a child to append');
            if (DOM.isHTMLElement(child)) {
                child = withElement(child);
            }
            assert.ok(child instanceof Builder || child instanceof MultiBuilder, 'Need a child to append');
            child.appendTo(this, index);
            return this;
        };
        /**
         *  Removes the current element of this builder from its parent node.
         */
        Builder.prototype.offDOM = function () {
            if (this.currentElement.parentNode) {
                this.currentElement.parentNode.removeChild(this.currentElement);
            }
            return this;
        };
        /**
         *  Returns the HTML Element the builder is currently active on.
         */
        Builder.prototype.getHTMLElement = function () {
            return this.currentElement;
        };
        /**
         *  Returns the HTML Element the builder is building in.
         */
        Builder.prototype.getContainer = function () {
            return this.container;
        };
        // HTML Elements
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.div = function (attributes, fn) {
            return this.doElement('div', attributes, fn);
        };
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.p = function (attributes, fn) {
            return this.doElement('p', attributes, fn);
        };
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.ul = function (attributes, fn) {
            return this.doElement('ul', attributes, fn);
        };
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.ol = function (attributes, fn) {
            return this.doElement('ol', attributes, fn);
        };
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.li = function (attributes, fn) {
            return this.doElement('li', attributes, fn);
        };
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.span = function (attributes, fn) {
            return this.doElement('span', attributes, fn);
        };
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.img = function (attributes, fn) {
            return this.doElement('img', attributes, fn);
        };
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.a = function (attributes, fn) {
            return this.doElement('a', attributes, fn);
        };
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.header = function (attributes, fn) {
            return this.doElement('header', attributes, fn);
        };
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.section = function (attributes, fn) {
            return this.doElement('section', attributes, fn);
        };
        /**
         *  Creates a new element of this kind as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.footer = function (attributes, fn) {
            return this.doElement('footer', attributes, fn);
        };
        /**
         *  Creates a new element of given tag name as child of the current element or parent.
         *  Accepts an object literal as first parameter that can be used to describe the
         *  attributes of the element.
         *  Accepts a function as second parameter that can be used to create child elements
         *  of the element. The function will be called with a new builder created with the
         *  provided element.
         */
        Builder.prototype.element = function (name, attributes, fn) {
            return this.doElement(name, attributes, fn);
        };
        Builder.prototype.doElement = function (name, attributesOrFn, fn) {
            // Create Element
            var element = this.browserService.document.createElement(name);
            this.currentElement = element;
            // Off-DOM: Remember in array of created elements
            if (this.offdom) {
                this.createdElements.push(element);
            }
            // Object (apply properties as attributes to HTML element)
            if (types.isObject(attributesOrFn)) {
                this.attr(attributesOrFn);
            }
            // Support second argument being function
            if (types.isFunction(attributesOrFn)) {
                fn = attributesOrFn;
            }
            // Apply Functions (Elements created in Functions will be added as child to current element)
            if (types.isFunction(fn)) {
                var builder = new Builder(element);
                fn.call(builder, builder); // Set both 'this' and the first parameter to the new builder
            }
            // Add to parent
            if (!this.offdom) {
                this.container.appendChild(element);
            }
            return this;
        };
        /**
         *  Calls focus() on the current HTML element;
         */
        Builder.prototype.domFocus = function () {
            this.currentElement.focus();
            return this;
        };
        /**
         *  Returns true if the current element of this builder is the active element.
         */
        Builder.prototype.hasFocus = function () {
            var activeElement = this.browserService.document.activeElement;
            return (activeElement === this.currentElement);
        };
        /**
         *  Calls select() on the current HTML element;
         */
        Builder.prototype.domSelect = function (range) {
            if (range === void 0) { range = null; }
            var input = this.currentElement;
            input.select();
            if (range) {
                input.setSelectionRange(range.start, range.end);
            }
            return this;
        };
        /**
         *  Calls blur() on the current HTML element;
         */
        Builder.prototype.domBlur = function () {
            this.currentElement.blur();
            return this;
        };
        /**
         *  Calls click() on the current HTML element;
         */
        Builder.prototype.domClick = function () {
            this.currentElement.click();
            return this;
        };
        Builder.prototype.on = function (arg1, fn, listenerToUnbindContainer, useCapture) {
            var _this = this;
            // Event Type Array
            if (types.isArray(arg1)) {
                arg1.forEach(function (type) {
                    _this.on(type, fn, listenerToUnbindContainer, useCapture);
                });
            }
            else {
                var type = arg1;
                // Add Listener
                var unbind_1 = DOM.addDisposableListener(this.currentElement, type, function (e) {
                    fn(e, _this, unbind_1); // Pass in Builder as Second Argument
                }, useCapture || false);
                // Remember for off() use
                if (useCapture) {
                    if (!this.captureToUnbind[type]) {
                        this.captureToUnbind[type] = [];
                    }
                    this.captureToUnbind[type].push(unbind_1);
                }
                else {
                    if (!this.toUnbind[type]) {
                        this.toUnbind[type] = [];
                    }
                    this.toUnbind[type].push(unbind_1);
                }
                // Bind to Element
                var listenerBinding = this.getProperty(LISTENER_BINDING_ID, []);
                listenerBinding.push(unbind_1);
                this.setProperty(LISTENER_BINDING_ID, listenerBinding);
                // Add to Array if passed in
                if (listenerToUnbindContainer && types.isArray(listenerToUnbindContainer)) {
                    listenerToUnbindContainer.push(unbind_1);
                }
            }
            return this;
        };
        Builder.prototype.off = function (arg1, useCapture) {
            var _this = this;
            // Event Type Array
            if (types.isArray(arg1)) {
                arg1.forEach(function (type) {
                    _this.off(type);
                });
            }
            else {
                var type = arg1;
                if (useCapture) {
                    if (this.captureToUnbind[type]) {
                        this.captureToUnbind[type] = lifecycle_1.disposeAll(this.captureToUnbind[type]);
                    }
                }
                else {
                    if (this.toUnbind[type]) {
                        this.toUnbind[type] = lifecycle_1.disposeAll(this.toUnbind[type]);
                    }
                }
            }
            return this;
        };
        Builder.prototype.once = function (arg1, fn, listenerToUnbindContainer, useCapture) {
            var _this = this;
            // Event Type Array
            if (types.isArray(arg1)) {
                arg1.forEach(function (type) {
                    _this.once(type, fn);
                });
            }
            else {
                var type = arg1;
                // Add Listener
                var unbind_2 = DOM.addDisposableListener(this.currentElement, type, function (e) {
                    fn(e, _this, unbind_2); // Pass in Builder as Second Argument
                    unbind_2.dispose();
                }, useCapture || false);
                // Add to Array if passed in
                if (listenerToUnbindContainer && types.isArray(listenerToUnbindContainer)) {
                    listenerToUnbindContainer.push(unbind_2);
                }
            }
            return this;
        };
        Builder.prototype.preventDefault = function (arg1, cancelBubble, listenerToUnbindContainer, useCapture) {
            var fn = function (e) {
                e.preventDefault();
                if (cancelBubble) {
                    if (e.stopPropagation) {
                        e.stopPropagation();
                    }
                    else {
                        e.cancelBubble = true;
                    }
                }
            };
            return this.on(arg1, fn, listenerToUnbindContainer);
        };
        Builder.prototype.attr = function (firstP, secondP) {
            // Apply Object Literal to Attributes of Element
            if (types.isObject(firstP)) {
                for (var prop in firstP) {
                    if (firstP.hasOwnProperty(prop)) {
                        var value = firstP[prop];
                        this.doSetAttr(prop, value);
                    }
                }
                return this;
            }
            // Get Attribute Value
            if (types.isString(firstP) && !types.isString(secondP)) {
                return this.currentElement.getAttribute(firstP);
            }
            // Set Attribute Value
            if (types.isString(firstP)) {
                if (!types.isString(secondP)) {
                    secondP = String(secondP);
                }
                this.doSetAttr(firstP, secondP);
            }
            return this;
        };
        Builder.prototype.doSetAttr = function (prop, value) {
            if (prop === 'class') {
                prop = 'addClass'; // Workaround for the issue that a function name can not be 'class' in ES
            }
            if (this[prop]) {
                if (types.isArray(value)) {
                    this[prop].apply(this, value);
                }
                else {
                    this[prop].call(this, value);
                }
            }
            else {
                this.currentElement.setAttribute(prop, value);
            }
        };
        /**
         * Removes an attribute by the given name.
         */
        Builder.prototype.removeAttribute = function (prop) {
            this.currentElement.removeAttribute(prop);
        };
        /**
         *  Sets the id attribute to the value provided for the current HTML element of the builder.
         */
        Builder.prototype.id = function (id) {
            this.currentElement.setAttribute('id', id);
            return this;
        };
        /**
         *  Sets the src attribute to the value provided for the current HTML element of the builder.
         */
        Builder.prototype.src = function (src) {
            this.currentElement.setAttribute('src', src);
            return this;
        };
        /**
         *  Sets the href attribute to the value provided for the current HTML element of the builder.
         */
        Builder.prototype.href = function (href) {
            this.currentElement.setAttribute('href', href);
            return this;
        };
        /**
         *  Sets the title attribute to the value provided for the current HTML element of the builder.
         */
        Builder.prototype.title = function (title) {
            this.currentElement.setAttribute('title', title);
            return this;
        };
        /**
         *  Sets the name attribute to the value provided for the current HTML element of the builder.
         */
        Builder.prototype.name = function (name) {
            this.currentElement.setAttribute('name', name);
            return this;
        };
        /**
         *  Sets the type attribute to the value provided for the current HTML element of the builder.
         */
        Builder.prototype.type = function (type) {
            this.currentElement.setAttribute('type', type);
            return this;
        };
        /**
         *  Sets the value attribute to the value provided for the current HTML element of the builder.
         */
        Builder.prototype.value = function (value) {
            this.currentElement.setAttribute('value', value);
            return this;
        };
        /**
         *  Sets the alt attribute to the value provided for the current HTML element of the builder.
         */
        Builder.prototype.alt = function (alt) {
            this.currentElement.setAttribute('alt', alt);
            return this;
        };
        /**
         *  Sets the name draggable to the value provided for the current HTML element of the builder.
         */
        Builder.prototype.draggable = function (isDraggable) {
            this.currentElement.setAttribute('draggable', isDraggable ? 'true' : 'false');
            return this;
        };
        /**
         *  Sets the tabindex attribute to the value provided for the current HTML element of the builder.
         */
        Builder.prototype.tabindex = function (index) {
            this.currentElement.setAttribute('tabindex', index.toString());
            return this;
        };
        Builder.prototype.style = function (firstP, secondP) {
            // Apply Object Literal to Styles of Element
            if (types.isObject(firstP)) {
                for (var prop in firstP) {
                    if (firstP.hasOwnProperty(prop)) {
                        var value = firstP[prop];
                        this.doSetStyle(prop, value);
                    }
                }
            }
            else if (types.isString(firstP) && !types.isString(secondP)) {
                return this.currentElement.style[this.cssKeyToJavaScriptProperty(firstP)];
            }
            else if (types.isString(firstP) && types.isString(secondP)) {
                this.doSetStyle(firstP, secondP);
            }
            return this;
        };
        Builder.prototype.doSetStyle = function (key, value) {
            if (key.indexOf('-') >= 0) {
                var segments = key.split('-');
                key = segments[0];
                for (var i = 1; i < segments.length; i++) {
                    var segment = segments[i];
                    key = key + segment.charAt(0).toUpperCase() + segment.substr(1);
                }
            }
            this.currentElement.style[this.cssKeyToJavaScriptProperty(key)] = value;
        };
        Builder.prototype.cssKeyToJavaScriptProperty = function (key) {
            // Automagically convert dashes as they are not allowed when programmatically
            // setting a CSS style property
            if (key.indexOf('-') >= 0) {
                var segments = key.split('-');
                key = segments[0];
                for (var i = 1; i < segments.length; i++) {
                    var segment = segments[i];
                    key = key + segment.charAt(0).toUpperCase() + segment.substr(1);
                }
            }
            else if (key === 'float') {
                key = 'cssFloat';
            }
            return key;
        };
        /**
         *  Returns the computed CSS style for the current HTML element of the builder.
         */
        Builder.prototype.getComputedStyle = function () {
            return DOM.getComputedStyle(this.currentElement);
        };
        /**
         *  Adds the variable list of arguments as class names to the current HTML element of the builder.
         */
        Builder.prototype.addClass = function () {
            var _this = this;
            var classes = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                classes[_i - 0] = arguments[_i];
            }
            classes.forEach(function (nameValue) {
                var names = nameValue.split(' ');
                names.forEach(function (name) {
                    DOM.addClass(_this.currentElement, name);
                });
            });
            return this;
        };
        /**
         *  Sets the class name of the current HTML element of the builder to the provided className.
         *  If shouldAddClass is provided - for true class is added, for false class is removed.
         */
        Builder.prototype.setClass = function (className, shouldAddClass) {
            if (shouldAddClass === void 0) { shouldAddClass = null; }
            if (shouldAddClass === null) {
                this.currentElement.className = className;
            }
            else if (shouldAddClass) {
                this.addClass(className);
            }
            else {
                this.removeClass(className);
            }
            return this;
        };
        /**
         *  Returns whether the current HTML element of the builder has the provided class assigned.
         */
        Builder.prototype.hasClass = function (className) {
            return DOM.hasClass(this.currentElement, className);
        };
        /**
         *  Removes the variable list of arguments as class names from the current HTML element of the builder.
         */
        Builder.prototype.removeClass = function () {
            var _this = this;
            var classes = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                classes[_i - 0] = arguments[_i];
            }
            classes.forEach(function (nameValue) {
                var names = nameValue.split(' ');
                names.forEach(function (name) {
                    DOM.removeClass(_this.currentElement, name);
                });
            });
            return this;
        };
        /**
         *  Sets the first class to the current HTML element of the builder if the second class is currently set
         *  and vice versa otherwise.
         */
        Builder.prototype.swapClass = function (classA, classB) {
            if (this.hasClass(classA)) {
                this.removeClass(classA);
                this.addClass(classB);
            }
            else {
                this.removeClass(classB);
                this.addClass(classA);
            }
            return this;
        };
        /**
         *  Adds or removes the provided className for the current HTML element of the builder.
         */
        Builder.prototype.toggleClass = function (className) {
            if (this.hasClass(className)) {
                this.removeClass(className);
            }
            else {
                this.addClass(className);
            }
            return this;
        };
        /**
         *  Sets the CSS property color.
         */
        Builder.prototype.color = function (color) {
            this.currentElement.style.color = color;
            return this;
        };
        /**
         *  Sets the CSS property background.
         */
        Builder.prototype.background = function (color) {
            this.currentElement.style.backgroundColor = color;
            return this;
        };
        Builder.prototype.padding = function (top, right, bottom, left) {
            if (types.isString(top) && top.indexOf(' ') >= 0) {
                return this.padding.apply(this, top.split(' '));
            }
            if (!types.isUndefinedOrNull(top)) {
                this.currentElement.style.paddingTop = this.toPixel(top);
            }
            if (!types.isUndefinedOrNull(right)) {
                this.currentElement.style.paddingRight = this.toPixel(right);
            }
            if (!types.isUndefinedOrNull(bottom)) {
                this.currentElement.style.paddingBottom = this.toPixel(bottom);
            }
            if (!types.isUndefinedOrNull(left)) {
                this.currentElement.style.paddingLeft = this.toPixel(left);
            }
            return this;
        };
        Builder.prototype.margin = function (top, right, bottom, left) {
            if (types.isString(top) && top.indexOf(' ') >= 0) {
                return this.margin.apply(this, top.split(' '));
            }
            if (!types.isUndefinedOrNull(top)) {
                this.currentElement.style.marginTop = this.toPixel(top);
            }
            if (!types.isUndefinedOrNull(right)) {
                this.currentElement.style.marginRight = this.toPixel(right);
            }
            if (!types.isUndefinedOrNull(bottom)) {
                this.currentElement.style.marginBottom = this.toPixel(bottom);
            }
            if (!types.isUndefinedOrNull(left)) {
                this.currentElement.style.marginLeft = this.toPixel(left);
            }
            return this;
        };
        Builder.prototype.position = function (top, right, bottom, left, position) {
            if (types.isString(top) && top.indexOf(' ') >= 0) {
                return this.position.apply(this, top.split(' '));
            }
            if (!types.isUndefinedOrNull(top)) {
                this.currentElement.style.top = this.toPixel(top);
            }
            if (!types.isUndefinedOrNull(right)) {
                this.currentElement.style.right = this.toPixel(right);
            }
            if (!types.isUndefinedOrNull(bottom)) {
                this.currentElement.style.bottom = this.toPixel(bottom);
            }
            if (!types.isUndefinedOrNull(left)) {
                this.currentElement.style.left = this.toPixel(left);
            }
            if (!position) {
                position = 'absolute';
            }
            this.currentElement.style.position = position;
            return this;
        };
        Builder.prototype.size = function (width, height) {
            if (types.isString(width) && width.indexOf(' ') >= 0) {
                return this.size.apply(this, width.split(' '));
            }
            if (!types.isUndefinedOrNull(width)) {
                this.currentElement.style.width = this.toPixel(width);
            }
            if (!types.isUndefinedOrNull(height)) {
                this.currentElement.style.height = this.toPixel(height);
            }
            return this;
        };
        Builder.prototype.minSize = function (width, height) {
            if (types.isString(width) && width.indexOf(' ') >= 0) {
                return this.minSize.apply(this, width.split(' '));
            }
            if (!types.isUndefinedOrNull(width)) {
                this.currentElement.style.minWidth = this.toPixel(width);
            }
            if (!types.isUndefinedOrNull(height)) {
                this.currentElement.style.minHeight = this.toPixel(height);
            }
            return this;
        };
        Builder.prototype.maxSize = function (width, height) {
            if (types.isString(width) && width.indexOf(' ') >= 0) {
                return this.maxSize.apply(this, width.split(' '));
            }
            if (!types.isUndefinedOrNull(width)) {
                this.currentElement.style.maxWidth = this.toPixel(width);
            }
            if (!types.isUndefinedOrNull(height)) {
                this.currentElement.style.maxHeight = this.toPixel(height);
            }
            return this;
        };
        /**
         *  Sets the CSS property float.
         */
        Builder.prototype.float = function (float) {
            this.currentElement.style.cssFloat = float;
            return this;
        };
        /**
         *  Sets the CSS property clear.
         */
        Builder.prototype.clear = function (clear) {
            this.currentElement.style.clear = clear;
            return this;
        };
        /**
         *  Sets the CSS property for fonts back to default.
         */
        Builder.prototype.normal = function () {
            this.currentElement.style.fontStyle = 'normal';
            this.currentElement.style.fontWeight = 'normal';
            this.currentElement.style.textDecoration = 'none';
            return this;
        };
        /**
         *  Sets the CSS property font-style to italic.
         */
        Builder.prototype.italic = function () {
            this.currentElement.style.fontStyle = 'italic';
            return this;
        };
        /**
         *  Sets the CSS property font-weight to bold.
         */
        Builder.prototype.bold = function () {
            this.currentElement.style.fontWeight = 'bold';
            return this;
        };
        /**
         *  Sets the CSS property text-decoration to underline.
         */
        Builder.prototype.underline = function () {
            this.currentElement.style.textDecoration = 'underline';
            return this;
        };
        /**
         *  Sets the CSS property overflow.
         */
        Builder.prototype.overflow = function (overflow) {
            this.currentElement.style.overflow = overflow;
            return this;
        };
        /**
         *  Sets the CSS property display.
         */
        Builder.prototype.display = function (display) {
            this.currentElement.style.display = display;
            return this;
        };
        Builder.prototype.disable = function () {
            this.currentElement.setAttribute('disabled', 'disabled');
            return this;
        };
        Builder.prototype.enable = function () {
            this.currentElement.removeAttribute('disabled');
            return this;
        };
        /**
         *  Shows the current element of the builder.
         */
        Builder.prototype.show = function () {
            if (this.hasClass('hidden')) {
                this.removeClass('hidden');
            }
            this.attr('aria-hidden', 'false');
            // Cancel any pending showDelayed() invocation
            this.cancelVisibilityPromise();
            return this;
        };
        /**
         *  Shows the current builder element after the provided delay. If the builder
         *  was set to hidden using the hide() method before this method executed, the
         *  function will return without showing the current element. This is useful to
         *  only show the element when a specific delay is reached (e.g. for a long running
         *  operation.
         */
        Builder.prototype.showDelayed = function (delay) {
            var _this = this;
            // Cancel any pending showDelayed() invocation
            this.cancelVisibilityPromise();
            var promise = winjs_base_1.TPromise.timeout(delay);
            this.setProperty(VISIBILITY_BINDING_ID, promise);
            promise.done(function () {
                _this.removeProperty(VISIBILITY_BINDING_ID);
                _this.show();
            });
            return this;
        };
        /**
         *  Hides the current element of the builder.
         */
        Builder.prototype.hide = function () {
            if (!this.hasClass('hidden')) {
                this.addClass('hidden');
            }
            this.attr('aria-hidden', 'true');
            // Cancel any pending showDelayed() invocation
            this.cancelVisibilityPromise();
            return this;
        };
        /**
         *  Returns true if the current element of the builder is hidden.
         */
        Builder.prototype.isHidden = function () {
            return this.hasClass('hidden') || this.currentElement.style.display === 'none';
        };
        /**
         *  Toggles visibility of the current element of the builder.
         */
        Builder.prototype.toggleVisibility = function () {
            // Cancel any pending showDelayed() invocation
            this.cancelVisibilityPromise();
            this.swapClass('builder-visible', 'hidden');
            if (this.isHidden()) {
                this.attr('aria-hidden', 'true');
            }
            else {
                this.attr('aria-hidden', 'false');
            }
            return this;
        };
        Builder.prototype.cancelVisibilityPromise = function () {
            var promise = this.getProperty(VISIBILITY_BINDING_ID);
            if (promise) {
                promise.cancel();
                this.removeProperty(VISIBILITY_BINDING_ID);
            }
        };
        Builder.prototype.border = function (width, style, color) {
            if (types.isString(width) && width.indexOf(' ') >= 0) {
                return this.border.apply(this, width.split(' '));
            }
            this.currentElement.style.borderWidth = this.toPixel(width);
            if (color) {
                this.currentElement.style.borderColor = color;
            }
            if (style) {
                this.currentElement.style.borderStyle = style;
            }
            return this;
        };
        Builder.prototype.borderTop = function (width, style, color) {
            if (types.isString(width) && width.indexOf(' ') >= 0) {
                return this.borderTop.apply(this, width.split(' '));
            }
            this.currentElement.style.borderTopWidth = this.toPixel(width);
            if (color) {
                this.currentElement.style.borderTopColor = color;
            }
            if (style) {
                this.currentElement.style.borderTopStyle = style;
            }
            return this;
        };
        Builder.prototype.borderBottom = function (width, style, color) {
            if (types.isString(width) && width.indexOf(' ') >= 0) {
                return this.borderBottom.apply(this, width.split(' '));
            }
            this.currentElement.style.borderBottomWidth = this.toPixel(width);
            if (color) {
                this.currentElement.style.borderBottomColor = color;
            }
            if (style) {
                this.currentElement.style.borderBottomStyle = style;
            }
            return this;
        };
        Builder.prototype.borderLeft = function (width, style, color) {
            if (types.isString(width) && width.indexOf(' ') >= 0) {
                return this.borderLeft.apply(this, width.split(' '));
            }
            this.currentElement.style.borderLeftWidth = this.toPixel(width);
            if (color) {
                this.currentElement.style.borderLeftColor = color;
            }
            if (style) {
                this.currentElement.style.borderLeftStyle = style;
            }
            return this;
        };
        Builder.prototype.borderRight = function (width, style, color) {
            if (types.isString(width) && width.indexOf(' ') >= 0) {
                return this.borderRight.apply(this, width.split(' '));
            }
            this.currentElement.style.borderRightWidth = this.toPixel(width);
            if (color) {
                this.currentElement.style.borderRightColor = color;
            }
            if (style) {
                this.currentElement.style.borderRightStyle = style;
            }
            return this;
        };
        /**
         *  Sets the CSS property text-align.
         */
        Builder.prototype.textAlign = function (textAlign) {
            this.currentElement.style.textAlign = textAlign;
            return this;
        };
        /**
         *  Sets the CSS property vertical-align.
         */
        Builder.prototype.verticalAlign = function (valign) {
            this.currentElement.style.verticalAlign = valign;
            return this;
        };
        Builder.prototype.toPixel = function (obj) {
            if (obj.toString().indexOf('px') === -1) {
                return obj.toString() + 'px';
            }
            return obj;
        };
        /**
         *  Sets the innerHTML attribute.
         */
        Builder.prototype.innerHtml = function (html, append) {
            if (append) {
                this.currentElement.innerHTML += html;
            }
            else {
                this.currentElement.innerHTML = html;
            }
            return this;
        };
        /**
         *  Sets the textContent property of the element.
         *  All HTML special characters will be escaped.
         */
        Builder.prototype.text = function (text, append) {
            if (append) {
                // children is child Elements versus childNodes includes textNodes
                if (this.currentElement.children.length === 0) {
                    this.currentElement.textContent += text;
                }
                else {
                    // if there are elements inside this node, append the string as a new text node
                    // to avoid wiping out the innerHTML and replacing it with only text content
                    this.currentElement.appendChild(this.browserService.document.createTextNode(text));
                }
            }
            else {
                this.currentElement.textContent = text;
            }
            return this;
        };
        /**
         *  Sets the innerHTML attribute in escaped form.
         */
        Builder.prototype.safeInnerHtml = function (html, append) {
            return this.innerHtml(strings.escape(html), append);
        };
        /**
         *  Adds the provided object as property to the current element. Call getBinding()
         *  to retrieve it again.
         */
        Builder.prototype.bind = function (object) {
            bindElement(this.currentElement, object);
            return this;
        };
        /**
         *  Removes the binding of the current element.
         */
        Builder.prototype.unbind = function () {
            unbindElement(this.currentElement);
            return this;
        };
        /**
         *  Returns the object that was passed into the bind() call.
         */
        Builder.prototype.getBinding = function () {
            return getBindingFromElement(this.currentElement);
        };
        /**
         *  Allows to store arbritary data into the current element.
         */
        Builder.prototype.setProperty = function (key, value) {
            setPropertyOnElement(this.currentElement, key, value);
            return this;
        };
        /**
         *  Allows to get arbritary data from the current element.
         */
        Builder.prototype.getProperty = function (key, fallback) {
            return getPropertyFromElement(this.currentElement, key, fallback);
        };
        /**
         *  Removes a property from the current element that is stored under the given key.
         */
        Builder.prototype.removeProperty = function (key) {
            if (hasData(this.currentElement)) {
                delete data(this.currentElement)[key];
            }
            return this;
        };
        /**
         *  Returns a new builder with the parent element of the current element of the builder.
         */
        Builder.prototype.parent = function (offdom) {
            assert.ok(!this.offdom, 'Builder was created with offdom = true and thus has no parent set');
            return withElement(this.currentElement.parentNode, offdom);
        };
        /**
         *  Returns a new builder with all child elements of the current element of the builder.
         */
        Builder.prototype.children = function (offdom) {
            var children = this.currentElement.children;
            var builders = [];
            for (var i = 0; i < children.length; i++) {
                builders.push(withElement(children.item(i), offdom));
            }
            return new MultiBuilder(builders);
        };
        /**
         *  Removes the current HTMLElement from the given builder from this builder if this builders
         *  current HTMLElement is the direct parent.
         */
        Builder.prototype.removeChild = function (builder) {
            if (this.currentElement === builder.parent().getHTMLElement()) {
                this.currentElement.removeChild(builder.getHTMLElement());
            }
            return this;
        };
        /**
         *  Returns a new builder with all elements matching the provided selector scoped to the
         *  current element of the builder. Use Build.withElementsBySelector() to run the selector
         *  over the entire DOM.
         *  The returned builder is an instance of array that can have 0 elements if the selector does not match any
         *  elements.
         */
        Builder.prototype.select = function (selector, offdom) {
            assert.ok(types.isString(selector), 'Expected String as parameter');
            var elements = this.currentElement.querySelectorAll(selector);
            var builders = [];
            for (var i = 0; i < elements.length; i++) {
                builders.push(withElement(elements.item(i), offdom));
            }
            return new MultiBuilder(builders);
        };
        /**
         *  Returns true if the current element of the builder matches the given selector and false otherwise.
         */
        Builder.prototype.matches = function (selector) {
            var element = this.currentElement;
            var matches = element.webkitMatchesSelector || element.mozMatchesSelector || element.msMatchesSelector || element.oMatchesSelector;
            return matches && matches.call(element, selector);
        };
        /**
         *  Returns true if the current element of the builder has no children.
         */
        Builder.prototype.isEmpty = function () {
            return !this.currentElement.childNodes || this.currentElement.childNodes.length === 0;
        };
        /**
         * Recurse through all descendant nodes and remove their data binding.
         */
        Builder.prototype.unbindDescendants = function (current) {
            if (current && current.children) {
                for (var i = 0, length_1 = current.children.length; i < length_1; i++) {
                    var element = current.children.item(i);
                    // Unbind
                    if (hasData(element)) {
                        // Listeners
                        var listeners = data(element)[LISTENER_BINDING_ID];
                        if (types.isArray(listeners)) {
                            while (listeners.length) {
                                listeners.pop().dispose();
                            }
                        }
                        // Delete Data Slot
                        delete element[MS_DATA_KEY];
                    }
                    // Recurse
                    this.unbindDescendants(element);
                }
            }
        };
        /**
         *  Removes all HTML elements from the current element of the builder. Will also clean up any
         *  event listners registered and also clear any data binding and properties stored
         *  to any child element.
         */
        Builder.prototype.empty = function () {
            this.unbindDescendants(this.currentElement);
            this.clearChildren();
            if (this.offdom) {
                this.createdElements = [];
            }
            return this;
        };
        /**
         *  Removes all HTML elements from the current element of the builder.
         */
        Builder.prototype.clearChildren = function () {
            // Remove Elements
            if (this.currentElement) {
                DOM.clearNode(this.currentElement);
            }
            return this;
        };
        /**
         *  Removes the current HTML element and all its children from its parent and unbinds
         *  all listeners and properties set to the data slots.
         */
        Builder.prototype.destroy = function () {
            if (this.currentElement) {
                // Remove from parent
                if (this.currentElement.parentNode) {
                    this.currentElement.parentNode.removeChild(this.currentElement);
                }
                // Empty to clear listeners and bindings from children
                this.empty();
                // Unbind
                if (hasData(this.currentElement)) {
                    // Listeners
                    var listeners = data(this.currentElement)[LISTENER_BINDING_ID];
                    if (types.isArray(listeners)) {
                        while (listeners.length) {
                            listeners.pop().dispose();
                        }
                    }
                    // Delete Data Slot
                    delete this.currentElement[MS_DATA_KEY];
                }
            }
            var type;
            for (type in this.toUnbind) {
                if (this.toUnbind.hasOwnProperty(type) && types.isArray(this.toUnbind[type])) {
                    this.toUnbind[type] = lifecycle_1.disposeAll(this.toUnbind[type]);
                }
            }
            for (type in this.captureToUnbind) {
                if (this.captureToUnbind.hasOwnProperty(type) && types.isArray(this.captureToUnbind[type])) {
                    this.captureToUnbind[type] = lifecycle_1.disposeAll(this.captureToUnbind[type]);
                }
            }
            // Nullify fields
            this.currentElement = null;
            this.container = null;
            this.offdom = null;
            this.createdElements = null;
            this.captureToUnbind = null;
            this.toUnbind = null;
        };
        /**
         *  Removes the current HTML element and all its children from its parent and unbinds
         *  all listeners and properties set to the data slots.
         */
        Builder.prototype.dispose = function () {
            this.destroy();
        };
        Builder.prototype.getPositionRelativeTo = function (element) {
            if (element instanceof Builder) {
                element = element.getHTMLElement();
            }
            var left = DOM.getRelativeLeft(this.currentElement, element);
            var top = DOM.getRelativeTop(this.currentElement, element);
            return new Box(top, -1, -1, left);
        };
        /**
         *  Gets the absolute coordinates of the element.
         */
        Builder.prototype.getPosition = function () {
            var position = DOM.getTopLeftOffset(this.currentElement);
            return new Box(position.top, -1, -1, position.left);
        };
        /**
         *  Gets the size (in pixels) of an element, including the margin.
         */
        Builder.prototype.getTotalSize = function () {
            var totalWidth = DOM.getTotalWidth(this.currentElement);
            var totalHeight = DOM.getTotalHeight(this.currentElement);
            return new Dimension(totalWidth, totalHeight);
        };
        /**
         *  Gets the size (in pixels) of the inside of the element, excluding the border and padding.
         */
        Builder.prototype.getContentSize = function () {
            var contentWidth = DOM.getContentWidth(this.currentElement);
            var contentHeight = DOM.getContentHeight(this.currentElement);
            return new Dimension(contentWidth, contentHeight);
        };
        /**
         *  Another variant of getting the inner dimensions of an element.
         */
        Builder.prototype.getClientArea = function () {
            // 0.) Try with DOM getDomNodePosition
            if (this.currentElement !== this.browserService.document.body) {
                var dimensions = DOM.getDomNodePosition(this.currentElement);
                return new Dimension(dimensions.width, dimensions.height);
            }
            // 1.) Try innerWidth / innerHeight
            if (this.browserService.window.innerWidth && this.browserService.window.innerHeight) {
                return new Dimension(this.browserService.window.innerWidth, this.browserService.window.innerHeight);
            }
            // 2.) Try with document.body.clientWidth / document.body.clientHeigh
            if (this.browserService.document.body && this.browserService.document.body.clientWidth && this.browserService.document.body.clientWidth) {
                return new Dimension(this.browserService.document.body.clientWidth, this.browserService.document.body.clientHeight);
            }
            // 3.) Try with document.documentElement.clientWidth / document.documentElement.clientHeight
            if (this.browserService.document.documentElement && this.browserService.document.documentElement.clientWidth && this.browserService.document.documentElement.clientHeight) {
                return new Dimension(this.browserService.document.documentElement.clientWidth, this.browserService.document.documentElement.clientHeight);
            }
            throw new Error('Unable to figure out browser width and height');
        };
        return Builder;
    }());
    exports.Builder = Builder;
    /**
     *  The multi builder provides the same methods as the builder, but allows to call
     *  them on an array of builders.
     */
    var MultiBuilder = (function (_super) {
        __extends(MultiBuilder, _super);
        function MultiBuilder(builders) {
            assert.ok(types.isArray(builders) || builders instanceof MultiBuilder, 'Expected Array or MultiBuilder as parameter');
            _super.call(this);
            this.length = 0;
            this.builders = [];
            // Add Builders to Array
            if (types.isArray(builders)) {
                for (var i = 0; i < builders.length; i++) {
                    if (builders[i] instanceof HTMLElement) {
                        this.push(withElement(builders[i]));
                    }
                    else {
                        this.push(builders[i]);
                    }
                }
            }
            else {
                for (var i = 0; i < builders.length; i++) {
                    this.push(builders.item(i));
                }
            }
            // Mixin Builder functions to operate on all builders
            var $outer = this;
            var propertyFn = function (prop) {
                $outer[prop] = function () {
                    var args = Array.prototype.slice.call(arguments);
                    var returnValues;
                    var mergeBuilders = false;
                    for (var i = 0; i < $outer.length; i++) {
                        var res = $outer.item(i)[prop].apply($outer.item(i), args);
                        // Merge MultiBuilders into one
                        if (res instanceof MultiBuilder) {
                            if (!returnValues) {
                                returnValues = [];
                            }
                            mergeBuilders = true;
                            for (var j = 0; j < res.length; j++) {
                                returnValues.push(res.item(j));
                            }
                        }
                        else if (!types.isUndefined(res) && !(res instanceof Builder)) {
                            if (!returnValues) {
                                returnValues = [];
                            }
                            returnValues.push(res);
                        }
                    }
                    if (returnValues && mergeBuilders) {
                        return new MultiBuilder(returnValues);
                    }
                    return returnValues || $outer;
                };
            };
            for (var prop in Builder.prototype) {
                if (prop !== 'clone' && prop !== 'and') {
                    if (Builder.prototype.hasOwnProperty(prop) && types.isFunction(Builder.prototype[prop])) {
                        propertyFn(prop);
                    }
                }
            }
        }
        MultiBuilder.prototype.item = function (i) {
            return this.builders[i];
        };
        MultiBuilder.prototype.push = function () {
            var items = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                items[_i - 0] = arguments[_i];
            }
            for (var i = 0; i < items.length; i++) {
                this.builders.push(items[i]);
            }
            this.length = this.builders.length;
        };
        MultiBuilder.prototype.pop = function () {
            var element = this.builders.pop();
            this.length = this.builders.length;
            return element;
        };
        MultiBuilder.prototype.concat = function (items) {
            var elements = this.builders.concat(items);
            this.length = this.builders.length;
            return elements;
        };
        MultiBuilder.prototype.shift = function () {
            var element = this.builders.shift();
            this.length = this.builders.length;
            return element;
        };
        MultiBuilder.prototype.unshift = function (item) {
            var res = this.builders.unshift(item);
            this.length = this.builders.length;
            return res;
        };
        MultiBuilder.prototype.slice = function (start, end) {
            var elements = this.builders.slice(start, end);
            this.length = this.builders.length;
            return elements;
        };
        MultiBuilder.prototype.splice = function (start, deleteCount) {
            var elements = this.builders.splice(start, deleteCount);
            this.length = this.builders.length;
            return elements;
        };
        MultiBuilder.prototype.clone = function () {
            return new MultiBuilder(this);
        };
        MultiBuilder.prototype.and = function (obj) {
            // Convert HTMLElement to Builder as necessary
            if (!(obj instanceof Builder) && !(obj instanceof MultiBuilder)) {
                obj = new Builder(obj);
            }
            var builders = [];
            if (obj instanceof MultiBuilder) {
                for (var i = 0; i < obj.length; i++) {
                    builders.push(obj.item(i));
                }
            }
            else {
                builders.push(obj);
            }
            this.push.apply(this, builders);
            return this;
        };
        return MultiBuilder;
    }(Builder));
    exports.MultiBuilder = MultiBuilder;
    function withBuilder(builder, offdom) {
        if (builder instanceof MultiBuilder) {
            return new MultiBuilder(builder);
        }
        return new Builder(builder.getHTMLElement(), offdom);
    }
    function withElement(element, offdom) {
        return new Builder(element, offdom);
    }
    function offDOM() {
        return new Builder(null, true);
    }
    // Binding functions
    /**
     *  Allows to store arbritary data into element.
     */
    function setPropertyOnElement(element, key, value) {
        data(element)[key] = value;
    }
    exports.setPropertyOnElement = setPropertyOnElement;
    /**
     *  Allows to get arbritary data from element.
     */
    function getPropertyFromElement(element, key, fallback) {
        if (hasData(element)) {
            var value = data(element)[key];
            if (!types.isUndefined(value)) {
                return value;
            }
        }
        return fallback;
    }
    exports.getPropertyFromElement = getPropertyFromElement;
    /**
     *  Removes a property from an element.
     */
    function removePropertyFromElement(element, key) {
        if (hasData(element)) {
            delete data(element)[key];
        }
    }
    exports.removePropertyFromElement = removePropertyFromElement;
    /**
     *  Adds the provided object as property to the given element. Call getBinding()
     *  to retrieve it again.
     */
    function bindElement(element, object) {
        setPropertyOnElement(element, DATA_BINDING_ID, object);
    }
    exports.bindElement = bindElement;
    /**
     *  Removes the binding of the given element.
     */
    function unbindElement(element) {
        removePropertyFromElement(element, DATA_BINDING_ID);
    }
    exports.unbindElement = unbindElement;
    /**
     *  Returns the object that was passed into the bind() call for the element.
     */
    function getBindingFromElement(element) {
        return getPropertyFromElement(element, DATA_BINDING_ID);
    }
    exports.getBindingFromElement = getBindingFromElement;
    exports.Binding = {
        setPropertyOnElement: setPropertyOnElement,
        getPropertyFromElement: getPropertyFromElement,
        removePropertyFromElement: removePropertyFromElement,
        bindElement: bindElement,
        unbindElement: unbindElement,
        getBindingFromElement: getBindingFromElement
    };
    var SELECTOR_REGEX = /([\w\-]+)?(#([\w\-]+))?((.([\w\-]+))*)/;
    exports.$ = function (arg) {
        // Off-DOM use
        if (types.isUndefined(arg)) {
            return offDOM();
        }
        // Falsified values cause error otherwise
        if (!arg) {
            throw new Error('Bad use of $');
        }
        // Wrap the given element
        if (DOM.isHTMLElement(arg) || arg === window) {
            return withElement(arg);
        }
        // Wrap the given builders
        if (types.isArray(arg)) {
            return new MultiBuilder(arg);
        }
        // Wrap the given builder
        if (arg instanceof Builder) {
            return withBuilder(arg);
        }
        if (types.isString(arg)) {
            // Use the argument as HTML code
            if (arg[0] === '<') {
                var element = void 0;
                var container = BrowserService.getService().document.createElement('div');
                container.innerHTML = strings.format.apply(strings, arguments);
                if (container.children.length === 0) {
                    throw new Error('Bad use of $');
                }
                if (container.children.length === 1) {
                    element = container.firstChild;
                    container.removeChild(element);
                    return withElement(element);
                }
                var builders = [];
                while (container.firstChild) {
                    element = container.firstChild;
                    container.removeChild(element);
                    builders.push(withElement(element));
                }
                return new MultiBuilder(builders);
            }
            else if (arguments.length === 1) {
                var match = SELECTOR_REGEX.exec(arg);
                if (!match) {
                    throw new Error('Bad use of $');
                }
                var tag = match[1] || 'div';
                var id = match[3] || undefined;
                var classes = (match[4] || '').replace(/\./g, ' ');
                var props = {};
                if (id) {
                    props['id'] = id;
                }
                if (classes) {
                    props['class'] = classes;
                }
                return offDOM().element(tag, props);
            }
            else {
                var result = offDOM();
                result.element.apply(result, arguments);
                return result;
            }
        }
        else {
            throw new Error('Bad use of $');
        }
    };
    exports.$.Box = Box;
    exports.$.Dimension = Dimension;
    exports.$.Position = Position;
    exports.$.Builder = Builder;
    exports.$.MultiBuilder = MultiBuilder;
    exports.$.Build = exports.Build;
    exports.$.Binding = exports.Binding;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/dnd", ["require", "exports", 'vs/base/browser/builder'], function (require, exports, builder_1) {
    'use strict';
    /**
     * A helper that will execute a provided function when the provided HTMLElement receives
     *  dragover event for 800ms. If the drag is aborted before, the callback will not be triggered.
     */
    var DelayedDragHandler = (function () {
        function DelayedDragHandler(container, callback) {
            var _this = this;
            builder_1.$(container).on('dragover', function () {
                if (!_this.timeout) {
                    _this.timeout = setTimeout(function () {
                        callback();
                        _this.timeout = null;
                    }, 800);
                }
            });
            builder_1.$(container).on(['dragleave', 'drop', 'dragend'], function () { return _this.clearDragTimeout(); });
        }
        DelayedDragHandler.prototype.clearDragTimeout = function () {
            if (this.timeout) {
                clearTimeout(this.timeout);
                this.timeout = null;
            }
        };
        DelayedDragHandler.prototype.dispose = function () {
            this.clearDragTimeout();
        };
        return DelayedDragHandler;
    }());
    exports.DelayedDragHandler = DelayedDragHandler;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/globalMouseMoveMonitor", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/browser/dom', 'vs/base/browser/iframe', 'vs/base/browser/mouseEvent'], function (require, exports, lifecycle_1, dom, iframe_1, mouseEvent_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function standardMouseMoveMerger(lastEvent, currentEvent) {
        var ev = new mouseEvent_1.StandardMouseEvent(currentEvent);
        ev.preventDefault();
        return {
            leftButton: ev.leftButton,
            posx: ev.posx,
            posy: ev.posy
        };
    }
    exports.standardMouseMoveMerger = standardMouseMoveMerger;
    var GlobalMouseMoveMonitor = (function (_super) {
        __extends(GlobalMouseMoveMonitor, _super);
        function GlobalMouseMoveMonitor() {
            _super.call(this);
            this.hooks = [];
            this.mouseMoveEventMerger = null;
            this.mouseMoveCallback = null;
            this.onStopCallback = null;
        }
        GlobalMouseMoveMonitor.prototype.dispose = function () {
            this.stopMonitoring(false);
            _super.prototype.dispose.call(this);
        };
        GlobalMouseMoveMonitor.prototype.stopMonitoring = function (invokeStopCallback) {
            if (!this.isMonitoring()) {
                // Not monitoring
                return;
            }
            // Unhook
            this.hooks = lifecycle_1.disposeAll(this.hooks);
            this.mouseMoveEventMerger = null;
            this.mouseMoveCallback = null;
            var onStopCallback = this.onStopCallback;
            this.onStopCallback = null;
            if (invokeStopCallback) {
                onStopCallback();
            }
        };
        GlobalMouseMoveMonitor.prototype.isMonitoring = function () {
            return this.hooks.length > 0;
        };
        GlobalMouseMoveMonitor.prototype.startMonitoring = function (mouseMoveEventMerger, mouseMoveCallback, onStopCallback) {
            var _this = this;
            if (this.isMonitoring()) {
                // I am already hooked
                return;
            }
            this.mouseMoveEventMerger = mouseMoveEventMerger;
            this.mouseMoveCallback = mouseMoveCallback;
            this.onStopCallback = onStopCallback;
            var windowChain = iframe_1.IframeUtils.getSameOriginWindowChain();
            for (var i = 0; i < windowChain.length; i++) {
                this.hooks.push(dom.addDisposableThrottledListener(windowChain[i].window.document, 'mousemove', function (data) { return _this.mouseMoveCallback(data); }, function (lastEvent, currentEvent) { return _this.mouseMoveEventMerger(lastEvent, currentEvent); }));
                this.hooks.push(dom.addDisposableListener(windowChain[i].window.document, 'mouseup', function (e) { return _this.stopMonitoring(true); }));
            }
            if (iframe_1.IframeUtils.hasDifferentOriginAncestor()) {
                var lastSameOriginAncestor = windowChain[windowChain.length - 1];
                // We might miss a mouse up if it happens outside the iframe
                // This one is for Chrome
                this.hooks.push(dom.addDisposableListener(lastSameOriginAncestor.window.document, 'mouseout', function (browserEvent) {
                    var e = new mouseEvent_1.StandardMouseEvent(browserEvent);
                    if (e.target.tagName.toLowerCase() === 'html') {
                        _this.stopMonitoring(true);
                    }
                }));
                // This one is for FF
                this.hooks.push(dom.addDisposableListener(lastSameOriginAncestor.window.document, 'mouseover', function (browserEvent) {
                    var e = new mouseEvent_1.StandardMouseEvent(browserEvent);
                    if (e.target.tagName.toLowerCase() === 'html') {
                        _this.stopMonitoring(true);
                    }
                }));
                // This one is for IE
                this.hooks.push(dom.addDisposableListener(lastSameOriginAncestor.window.document.body, 'mouseleave', function (browserEvent) {
                    _this.stopMonitoring(true);
                }));
            }
        };
        return GlobalMouseMoveMonitor;
    }(lifecycle_1.Disposable));
    exports.GlobalMouseMoveMonitor = GlobalMouseMoveMonitor;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/htmlContentRenderer", ["require", "exports", 'vs/base/browser/dom', 'vs/base/common/marked/marked'], function (require, exports, DOM, marked_1) {
    'use strict';
    /**
     * Create html nodes for the given content element.
     *
     * @param content a html element description
     * @param actionCallback a callback function for any action links in the string. Argument is the zero-based index of the clicked action.
     */
    function renderHtml(content, options) {
        if (options === void 0) { options = {}; }
        if (typeof content === 'string') {
            return _renderHtml({ isText: true, text: content }, options);
        }
        else if (Array.isArray(content)) {
            return _renderHtml({ children: content }, options);
        }
        else if (content) {
            return _renderHtml(content, options);
        }
    }
    exports.renderHtml = renderHtml;
    function _renderHtml(content, options) {
        if (options === void 0) { options = {}; }
        var codeBlockRenderer = options.codeBlockRenderer, actionCallback = options.actionCallback;
        if (content.isText) {
            return document.createTextNode(content.text);
        }
        var tagName = getSafeTagName(content.tagName) || 'div';
        var element = document.createElement(tagName);
        if (content.className) {
            element.className = content.className;
        }
        if (content.text) {
            element.textContent = content.text;
        }
        if (content.style) {
            element.setAttribute('style', content.style);
        }
        if (content.customStyle) {
            Object.keys(content.customStyle).forEach(function (key) {
                element.style[key] = content.customStyle[key];
            });
        }
        if (content.code && codeBlockRenderer) {
            var html = codeBlockRenderer(content.code.language, content.code.value);
            element.innerHTML = html;
        }
        if (content.children) {
            content.children.forEach(function (child) {
                element.appendChild(renderHtml(child, options));
            });
        }
        if (content.formattedText) {
            renderFormattedText(element, parseFormattedText(content.formattedText), actionCallback);
        }
        if (content.markdown) {
            var renderer = new marked_1.marked.Renderer();
            renderer.link = function (href, title, text) {
                return "<a href=\"#\" data-href=\"" + href + "\" title=\"" + (title || text) + "\">" + text + "</a>";
            };
            renderer.paragraph = function (text) {
                return "<div>" + text + "</div>";
            };
            if (options.codeBlockRenderer) {
                renderer.code = function (code, lang) {
                    return options.codeBlockRenderer(lang, code);
                };
            }
            if (options.actionCallback) {
                DOM.addStandardDisposableListener(element, 'click', function (event) {
                    if (event.target.tagName === 'A') {
                        var href = event.target.dataset['href'];
                        if (href) {
                            options.actionCallback(href, event);
                        }
                    }
                });
            }
            element.innerHTML = marked_1.marked(content.markdown, {
                sanitize: true,
                renderer: renderer
            });
        }
        return element;
    }
    var SAFE_TAG_NAMES = {
        a: true,
        b: true,
        blockquote: true,
        code: true,
        del: true,
        dd: true,
        div: true,
        dl: true,
        dt: true,
        em: true,
        h1h2h3i: true,
        img: true,
        kbd: true,
        li: true,
        ol: true,
        p: true,
        pre: true,
        s: true,
        span: true,
        sup: true,
        sub: true,
        strong: true,
        strike: true,
        ul: true,
        br: true,
        hr: true,
    };
    function getSafeTagName(tagName) {
        if (!tagName) {
            return null;
        }
        if (SAFE_TAG_NAMES.hasOwnProperty(tagName)) {
            return tagName;
        }
        return null;
    }
    // // --- markdown worker renderer
    // namespace marked {
    // 	const workerFactory = new DefaultWorkerFactory();
    // 	let worker: WorkerClient;
    // 	let workerDisposeHandle: number;
    // 	export function html(source: string): TPromise<string> {
    // 		const t1 = Date.now();
    // 		if (!worker) {
    // 			worker = new WorkerClient(workerFactory, 'vs/base/common/marked/simpleMarkedWorker', (msg) => msg.type, client => { shutdown(); });
    // 		}
    // 		function shutdown() {
    // 			if (worker) {
    // 				worker.dispose();
    // 				worker = undefined;
    // 			}
    // 		}
    // 		// re-schedule termination
    // 		clearTimeout(workerDisposeHandle);
    // 		workerDisposeHandle = setTimeout(shutdown, 1000 * 5);
    // 		return worker.request('markdownToHtml', { source, hightlight: false }).then(html => {
    // 			console.log(`t1: ${Date.now() - t1}ms`);
    // 			return html;
    // 		});
    // 	}
    // }
    // --- formatted string parsing
    var StringStream = (function () {
        function StringStream(source) {
            this.source = source;
            this.index = 0;
        }
        StringStream.prototype.eos = function () {
            return this.index >= this.source.length;
        };
        StringStream.prototype.next = function () {
            var next = this.peek();
            this.advance();
            return next;
        };
        StringStream.prototype.peek = function () {
            return this.source[this.index];
        };
        StringStream.prototype.advance = function () {
            this.index++;
        };
        return StringStream;
    }());
    var FormatType;
    (function (FormatType) {
        FormatType[FormatType["Invalid"] = 0] = "Invalid";
        FormatType[FormatType["Root"] = 1] = "Root";
        FormatType[FormatType["Text"] = 2] = "Text";
        FormatType[FormatType["Bold"] = 3] = "Bold";
        FormatType[FormatType["Italics"] = 4] = "Italics";
        FormatType[FormatType["Action"] = 5] = "Action";
        FormatType[FormatType["ActionClose"] = 6] = "ActionClose";
        FormatType[FormatType["NewLine"] = 7] = "NewLine";
    })(FormatType || (FormatType = {}));
    function renderFormattedText(element, treeNode, actionCallback) {
        var child;
        if (treeNode.type === FormatType.Text) {
            child = document.createTextNode(treeNode.content);
        }
        else if (treeNode.type === FormatType.Bold) {
            child = document.createElement('b');
        }
        else if (treeNode.type === FormatType.Italics) {
            child = document.createElement('i');
        }
        else if (treeNode.type === FormatType.Action) {
            var a = document.createElement('a');
            a.href = '#';
            DOM.addStandardDisposableListener(a, 'click', function (event) {
                actionCallback(String(treeNode.index), event);
            });
            child = a;
        }
        else if (treeNode.type === FormatType.NewLine) {
            child = document.createElement('br');
        }
        else if (treeNode.type === FormatType.Root) {
            child = element;
        }
        if (element !== child) {
            element.appendChild(child);
        }
        if (Array.isArray(treeNode.children)) {
            treeNode.children.forEach(function (nodeChild) {
                renderFormattedText(child, nodeChild, actionCallback);
            });
        }
    }
    function parseFormattedText(content) {
        var root = {
            type: FormatType.Root,
            children: []
        };
        var actionItemIndex = 0;
        var current = root;
        var stack = [];
        var stream = new StringStream(content);
        while (!stream.eos()) {
            var next = stream.next();
            var isEscapedFormatType = (next === '\\' && formatTagType(stream.peek()) !== FormatType.Invalid);
            if (isEscapedFormatType) {
                next = stream.next(); // unread the backslash if it escapes a format tag type
            }
            if (!isEscapedFormatType && isFormatTag(next) && next === stream.peek()) {
                stream.advance();
                if (current.type === FormatType.Text) {
                    current = stack.pop();
                }
                var type = formatTagType(next);
                if (current.type === type || (current.type === FormatType.Action && type === FormatType.ActionClose)) {
                    current = stack.pop();
                }
                else {
                    var newCurrent = {
                        type: type,
                        children: []
                    };
                    if (type === FormatType.Action) {
                        newCurrent.index = actionItemIndex;
                        actionItemIndex++;
                    }
                    current.children.push(newCurrent);
                    stack.push(current);
                    current = newCurrent;
                }
            }
            else if (next === '\n') {
                if (current.type === FormatType.Text) {
                    current = stack.pop();
                }
                current.children.push({
                    type: FormatType.NewLine
                });
            }
            else {
                if (current.type !== FormatType.Text) {
                    var textCurrent = {
                        type: FormatType.Text,
                        content: next
                    };
                    current.children.push(textCurrent);
                    stack.push(current);
                    current = textCurrent;
                }
                else {
                    current.content += next;
                }
            }
        }
        if (current.type === FormatType.Text) {
            current = stack.pop();
        }
        if (stack.length) {
        }
        return root;
    }
    function isFormatTag(char) {
        return formatTagType(char) !== FormatType.Invalid;
    }
    function formatTagType(char) {
        switch (char) {
            case '*':
                return FormatType.Bold;
            case '_':
                return FormatType.Italics;
            case '[':
                return FormatType.Action;
            case ']':
                return FormatType.ActionClose;
            default:
                return FormatType.Invalid;
        }
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/idleMonitor", ["require", "exports", 'vs/base/common/async', 'vs/base/common/eventEmitter', 'vs/base/common/lifecycle', 'vs/base/browser/browserService', 'vs/base/browser/dom'], function (require, exports, async_1, eventEmitter_1, lifecycle_1, browserService_1, dom) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (UserStatus) {
        UserStatus[UserStatus["Idle"] = 0] = "Idle";
        UserStatus[UserStatus["Active"] = 1] = "Active";
    })(exports.UserStatus || (exports.UserStatus = {}));
    var UserStatus = exports.UserStatus;
    exports.DEFAULT_IDLE_TIME = 60 * 60 * 1000; // 60 minutes
    var IdleMonitor = (function (_super) {
        __extends(IdleMonitor, _super);
        function IdleMonitor(idleTime) {
            var _this = this;
            if (idleTime === void 0) { idleTime = exports.DEFAULT_IDLE_TIME; }
            _super.call(this);
            this._status = null;
            this._idleCheckTimeout = this._register(new async_1.TimeoutTimer());
            this._lastActiveTime = -1;
            this._idleTime = idleTime;
            this._eventEmitter = this._register(new eventEmitter_1.EventEmitter());
            this._register(dom.addDisposableListener(browserService_1.getService().document, 'mousemove', function () { return _this._onUserActive(); }));
            this._register(dom.addDisposableListener(browserService_1.getService().document, 'keydown', function () { return _this._onUserActive(); }));
            this._onUserActive();
        }
        IdleMonitor.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
        };
        IdleMonitor.prototype.addOneTimeActiveListener = function (callback) {
            return this._eventEmitter.addOneTimeDisposableListener('onActive', callback);
        };
        IdleMonitor.prototype.addOneTimeIdleListener = function (callback) {
            return this._eventEmitter.addOneTimeDisposableListener('onIdle', callback);
        };
        IdleMonitor.prototype.getStatus = function () {
            return this._status;
        };
        IdleMonitor.prototype._onUserActive = function () {
            this._lastActiveTime = (new Date()).getTime();
            if (this._status !== UserStatus.Active) {
                this._status = UserStatus.Active;
                this._scheduleIdleCheck();
                this._eventEmitter.emit('onActive');
            }
        };
        IdleMonitor.prototype._onUserIdle = function () {
            if (this._status !== UserStatus.Idle) {
                this._status = UserStatus.Idle;
                this._eventEmitter.emit('onIdle');
            }
        };
        IdleMonitor.prototype._scheduleIdleCheck = function () {
            var _this = this;
            var minimumTimeWhenUserCanBecomeIdle = this._lastActiveTime + this._idleTime;
            this._idleCheckTimeout.setIfNotSet(function () {
                _this._checkIfUserIsIdle();
            }, minimumTimeWhenUserCanBecomeIdle - (new Date()).getTime());
        };
        IdleMonitor.prototype._checkIfUserIsIdle = function () {
            var actualIdleTime = (new Date()).getTime() - this._lastActiveTime;
            if (actualIdleTime >= this._idleTime) {
                this._onUserIdle();
            }
            else {
                this._scheduleIdleCheck();
            }
        };
        return IdleMonitor;
    }(lifecycle_1.Disposable));
    exports.IdleMonitor = IdleMonitor;
});

define("vs/base/browser/touch", ["require", "exports", 'vs/base/common/arrays', 'vs/base/common/lifecycle', 'vs/base/browser/dom'], function (require, exports, arrays, lifecycle_1, DomUtils) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EventType;
    (function (EventType) {
        EventType.Tap = '-monaco-gesturetap';
        EventType.Change = '-monaco-gesturechange';
        EventType.Start = '-monaco-gesturestart';
        EventType.End = '-monaco-gesturesend';
        EventType.Contextmenu = '-monaco-gesturecontextmenu';
    })(EventType = exports.EventType || (exports.EventType = {}));
    var Gesture = (function () {
        function Gesture(target) {
            this.callOnTarget = [];
            this.activeTouches = {};
            this.target = target;
            this.handle = null;
        }
        Gesture.prototype.dispose = function () {
            this.target = null;
            if (this.handle) {
                this.handle.dispose();
                this.handle = null;
            }
        };
        Object.defineProperty(Gesture.prototype, "target", {
            set: function (element) {
                var _this = this;
                this.callOnTarget = lifecycle_1.disposeAll(this.callOnTarget);
                this.activeTouches = {};
                this.targetElement = element;
                if (!this.targetElement) {
                    return;
                }
                this.callOnTarget.push(DomUtils.addDisposableListener(this.targetElement, 'touchstart', function (e) { return _this.onTouchStart(e); }));
                this.callOnTarget.push(DomUtils.addDisposableListener(this.targetElement, 'touchend', function (e) { return _this.onTouchEnd(e); }));
                this.callOnTarget.push(DomUtils.addDisposableListener(this.targetElement, 'touchmove', function (e) { return _this.onTouchMove(e); }));
            },
            enumerable: true,
            configurable: true
        });
        Gesture.newGestureEvent = function (type) {
            var event = document.createEvent('CustomEvent');
            event.initEvent(type, false, true);
            return event;
        };
        Gesture.prototype.onTouchStart = function (e) {
            var timestamp = Date.now(); // use Date.now() because on FF e.timeStamp is not epoch based.
            e.preventDefault();
            e.stopPropagation();
            if (this.handle) {
                this.handle.dispose();
                this.handle = null;
            }
            for (var i = 0, len = e.targetTouches.length; i < len; i++) {
                var touch = e.targetTouches.item(i);
                this.activeTouches[touch.identifier] = {
                    id: touch.identifier,
                    initialTarget: touch.target,
                    initialTimeStamp: timestamp,
                    initialPageX: touch.pageX,
                    initialPageY: touch.pageY,
                    rollingTimestamps: [timestamp],
                    rollingPageX: [touch.pageX],
                    rollingPageY: [touch.pageY]
                };
                var evt = Gesture.newGestureEvent(EventType.Start);
                evt.pageX = touch.pageX;
                evt.pageY = touch.pageY;
                this.targetElement.dispatchEvent(evt);
            }
        };
        Gesture.prototype.onTouchEnd = function (e) {
            var timestamp = Date.now(); // use Date.now() because on FF e.timeStamp is not epoch based.
            e.preventDefault();
            e.stopPropagation();
            var activeTouchCount = Object.keys(this.activeTouches).length;
            for (var i = 0, len = e.changedTouches.length; i < len; i++) {
                var touch = e.changedTouches.item(i);
                if (!this.activeTouches.hasOwnProperty(String(touch.identifier))) {
                    console.warn('move of an UNKNOWN touch', touch);
                    continue;
                }
                var data = this.activeTouches[touch.identifier], holdTime = Date.now() - data.initialTimeStamp;
                if (holdTime < Gesture.HOLD_DELAY
                    && Math.abs(data.initialPageX - arrays.tail(data.rollingPageX)) < 30
                    && Math.abs(data.initialPageY - arrays.tail(data.rollingPageY)) < 30) {
                    var evt = Gesture.newGestureEvent(EventType.Tap);
                    evt.initialTarget = data.initialTarget;
                    evt.pageX = arrays.tail(data.rollingPageX);
                    evt.pageY = arrays.tail(data.rollingPageY);
                    this.targetElement.dispatchEvent(evt);
                }
                else if (holdTime >= Gesture.HOLD_DELAY
                    && Math.abs(data.initialPageX - arrays.tail(data.rollingPageX)) < 30
                    && Math.abs(data.initialPageY - arrays.tail(data.rollingPageY)) < 30) {
                    var evt = Gesture.newGestureEvent(EventType.Contextmenu);
                    evt.initialTarget = data.initialTarget;
                    evt.pageX = arrays.tail(data.rollingPageX);
                    evt.pageY = arrays.tail(data.rollingPageY);
                    this.targetElement.dispatchEvent(evt);
                }
                else if (activeTouchCount === 1) {
                    var finalX = arrays.tail(data.rollingPageX);
                    var finalY = arrays.tail(data.rollingPageY);
                    var deltaT = arrays.tail(data.rollingTimestamps) - data.rollingTimestamps[0];
                    var deltaX = finalX - data.rollingPageX[0];
                    var deltaY = finalY - data.rollingPageY[0];
                    this.inertia(timestamp, // time now
                    Math.abs(deltaX) / deltaT, // speed
                    deltaX > 0 ? 1 : -1, // x direction
                    finalX, // x now
                    Math.abs(deltaY) / deltaT, // y speed
                    deltaY > 0 ? 1 : -1, // y direction
                    finalY // y now
                    );
                }
                // forget about this touch
                delete this.activeTouches[touch.identifier];
            }
        };
        Gesture.prototype.inertia = function (t1, vX, dirX, x, vY, dirY, y) {
            var _this = this;
            this.handle = DomUtils.scheduleAtNextAnimationFrame(function () {
                var now = Date.now();
                // velocity: old speed + accel_over_time
                var deltaT = now - t1, delta_pos_x = 0, delta_pos_y = 0, stopped = true;
                vX += Gesture.SCROLL_FRICTION * deltaT;
                vY += Gesture.SCROLL_FRICTION * deltaT;
                if (vX > 0) {
                    stopped = false;
                    delta_pos_x = dirX * vX * deltaT;
                }
                if (vY > 0) {
                    stopped = false;
                    delta_pos_y = dirY * vY * deltaT;
                }
                // dispatch translation event
                var evt = Gesture.newGestureEvent(EventType.Change);
                evt.translationX = delta_pos_x;
                evt.translationY = delta_pos_y;
                _this.targetElement.dispatchEvent(evt);
                if (!stopped) {
                    _this.inertia(now, vX, dirX, x + delta_pos_x, vY, dirY, y + delta_pos_y);
                }
            });
        };
        Gesture.prototype.onTouchMove = function (e) {
            var timestamp = Date.now(); // use Date.now() because on FF e.timeStamp is not epoch based.
            e.preventDefault();
            e.stopPropagation();
            for (var i = 0, len = e.changedTouches.length; i < len; i++) {
                var touch = e.changedTouches.item(i);
                if (!this.activeTouches.hasOwnProperty(String(touch.identifier))) {
                    console.warn('end of an UNKNOWN touch', touch);
                    continue;
                }
                var data = this.activeTouches[touch.identifier];
                var evt = Gesture.newGestureEvent(EventType.Change);
                evt.translationX = touch.pageX - arrays.tail(data.rollingPageX);
                evt.translationY = touch.pageY - arrays.tail(data.rollingPageY);
                evt.pageX = touch.pageX;
                evt.pageY = touch.pageY;
                this.targetElement.dispatchEvent(evt);
                // only keep a few data points, to average the final speed
                if (data.rollingPageX.length > 3) {
                    data.rollingPageX.shift();
                    data.rollingPageY.shift();
                    data.rollingTimestamps.shift();
                }
                data.rollingPageX.push(touch.pageX);
                data.rollingPageY.push(touch.pageY);
                data.rollingTimestamps.push(timestamp);
            }
        };
        Gesture.HOLD_DELAY = 700;
        Gesture.SCROLL_FRICTION = -0.005;
        return Gesture;
    }());
    exports.Gesture = Gesture;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/actionbar/actionbar", ["require", "exports", 'vs/nls!vs/base/browser/ui/actionbar/actionbar', 'vs/base/common/lifecycle', 'vs/base/browser/builder', 'vs/base/common/platform', 'vs/base/common/actions', 'vs/base/browser/dom', 'vs/base/common/events', 'vs/base/common/types', 'vs/base/common/eventEmitter', 'vs/base/browser/touch', 'vs/base/browser/keyboardEvent', 'vs/base/common/keyCodes', 'vs/css!./actionbar'], function (require, exports, nls, lifecycle, builder_1, platform, actions_1, DOM, events_1, types, eventEmitter_1, touch_1, keyboardEvent_1, keyCodes_1) {
    'use strict';
    var BaseActionItem = (function (_super) {
        __extends(BaseActionItem, _super);
        function BaseActionItem(context, action) {
            var _this = this;
            _super.call(this);
            this._callOnDispose = [];
            this._context = context || this;
            this._action = action;
            if (action instanceof actions_1.Action) {
                var l = action.addBulkListener(function (events) {
                    if (!_this.builder) {
                        // we have not been rendered yet, so there
                        // is no point in updating the UI
                        return;
                    }
                    events.forEach(function (event) {
                        switch (event.getType()) {
                            case actions_1.Action.ENABLED:
                                _this._updateEnabled();
                                break;
                            case actions_1.Action.LABEL:
                                _this._updateLabel();
                                _this._updateTooltip();
                                break;
                            case actions_1.Action.TOOLTIP:
                                _this._updateTooltip();
                                break;
                            case actions_1.Action.CLASS:
                                _this._updateClass();
                                break;
                            case actions_1.Action.CHECKED:
                                _this._updateChecked();
                                break;
                            default:
                                _this._updateUnknown(event);
                                break;
                        }
                    });
                });
                this._callOnDispose.push(l);
            }
        }
        Object.defineProperty(BaseActionItem.prototype, "callOnDispose", {
            get: function () {
                return this._callOnDispose;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseActionItem.prototype, "actionRunner", {
            get: function () {
                return this._actionRunner;
            },
            set: function (actionRunner) {
                this._actionRunner = actionRunner;
            },
            enumerable: true,
            configurable: true
        });
        BaseActionItem.prototype.getAction = function () {
            return this._action;
        };
        BaseActionItem.prototype.isEnabled = function () {
            return this._action.enabled;
        };
        BaseActionItem.prototype.setActionContext = function (newContext) {
            this._context = newContext;
        };
        BaseActionItem.prototype.render = function (container) {
            var _this = this;
            this.builder = builder_1.$(container);
            this.gesture = new touch_1.Gesture(container);
            this.builder.on(DOM.EventType.CLICK, function (event) { _this.onClick(event); });
            this.builder.on(touch_1.EventType.Tap, function (e) { _this.onClick(e); });
            if (platform.isMacintosh) {
                this.builder.on(DOM.EventType.CONTEXT_MENU, function (event) { _this.onClick(event); }); // https://github.com/Microsoft/vscode/issues/1011
            }
            this.builder.on('mousedown', function (e) {
                if (e.button === 0 && _this._action.enabled) {
                    _this.builder.addClass('active');
                }
            });
            this.builder.on(['mouseup', 'mouseout'], function (e) {
                if (e.button === 0 && _this._action.enabled) {
                    _this.builder.removeClass('active');
                }
            });
        };
        BaseActionItem.prototype.onClick = function (event) {
            DOM.EventHelper.stop(event, true);
            this._actionRunner.run(this._action, this._context || event);
        };
        BaseActionItem.prototype.focus = function () {
            if (this.builder) {
                this.builder.domFocus();
            }
        };
        BaseActionItem.prototype.blur = function () {
            if (this.builder) {
                this.builder.domBlur();
            }
        };
        BaseActionItem.prototype._updateEnabled = function () {
            // implement in subclass
        };
        BaseActionItem.prototype._updateLabel = function () {
            // implement in subclass
        };
        BaseActionItem.prototype._updateTooltip = function () {
            // implement in subclass
        };
        BaseActionItem.prototype._updateClass = function () {
            // implement in subclass
        };
        BaseActionItem.prototype._updateChecked = function () {
            // implement in subclass
        };
        BaseActionItem.prototype._updateUnknown = function (event) {
            // can implement in subclass
        };
        BaseActionItem.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            if (this.builder) {
                this.builder.destroy();
                this.builder = null;
            }
            if (this.gesture) {
                this.gesture.dispose();
                this.gesture = null;
            }
            lifecycle.cAll(this._callOnDispose);
        };
        return BaseActionItem;
    }(eventEmitter_1.EventEmitter));
    exports.BaseActionItem = BaseActionItem;
    var Separator = (function (_super) {
        __extends(Separator, _super);
        function Separator(label, order) {
            _super.call(this, Separator.ID, label, label ? 'separator text' : 'separator');
            this.checked = false;
            this.enabled = false;
            this.order = order;
        }
        Separator.ID = 'actions.monaco.separator';
        return Separator;
    }(actions_1.Action));
    exports.Separator = Separator;
    var ActionItem = (function (_super) {
        __extends(ActionItem, _super);
        function ActionItem(context, action, options) {
            if (options === void 0) { options = {}; }
            _super.call(this, context, action);
            this.options = options;
            this.options.icon = options.icon !== undefined ? options.icon : false;
            this.options.label = options.label !== undefined ? options.label : true;
            this.cssClass = '';
        }
        ActionItem.prototype.render = function (container) {
            _super.prototype.render.call(this, container);
            this.$e = builder_1.$('a.action-label').appendTo(this.builder);
            this.$e.attr({ role: 'button' });
            if (this.options.label && this.options.keybinding) {
                builder_1.$('span.keybinding').text(this.options.keybinding).appendTo(this.builder);
            }
            this._updateClass();
            this._updateLabel();
            this._updateTooltip();
            this._updateEnabled();
            this._updateChecked();
        };
        ActionItem.prototype.focus = function () {
            _super.prototype.focus.call(this);
            this.$e.domFocus();
        };
        ActionItem.prototype._updateLabel = function () {
            if (this.options.label) {
                this.$e.text(this.getAction().label);
            }
        };
        ActionItem.prototype._updateTooltip = function () {
            var title = null;
            if (this.getAction().tooltip) {
                title = this.getAction().tooltip;
            }
            else if (!this.options.label && this.getAction().label && this.options.icon) {
                title = this.getAction().label;
                if (this.options.keybinding) {
                    title = nls.localize(0, null, title, this.options.keybinding);
                }
            }
            if (title) {
                this.$e.attr({ title: title });
            }
        };
        ActionItem.prototype._updateClass = function () {
            if (this.cssClass) {
                this.$e.removeClass(this.cssClass);
            }
            if (this.options.icon) {
                this.cssClass = this.getAction().class;
                this.$e.addClass('icon');
                if (this.cssClass) {
                    this.$e.addClass(this.cssClass);
                }
                this._updateEnabled();
            }
            else {
                this.$e.removeClass('icon');
            }
        };
        ActionItem.prototype._updateEnabled = function () {
            if (this.getAction().enabled) {
                this.builder.removeClass('disabled');
                this.$e.removeClass('disabled');
                this.$e.attr({ tabindex: 0 });
            }
            else {
                this.builder.addClass('disabled');
                this.$e.addClass('disabled');
                DOM.removeTabIndexAndUpdateFocus(this.$e.getHTMLElement());
            }
        };
        ActionItem.prototype._updateChecked = function () {
            if (this.getAction().checked) {
                this.$e.addClass('checked');
            }
            else {
                this.$e.removeClass('checked');
            }
        };
        return ActionItem;
    }(BaseActionItem));
    exports.ActionItem = ActionItem;
    var ProgressItem = (function (_super) {
        __extends(ProgressItem, _super);
        function ProgressItem() {
            _super.apply(this, arguments);
        }
        ProgressItem.prototype.render = function (parent) {
            var container = document.createElement('div');
            builder_1.$(container).addClass('progress-item');
            var label = document.createElement('div');
            builder_1.$(label).addClass('label');
            label.textContent = this.getAction().label;
            label.title = this.getAction().label;
            _super.prototype.render.call(this, label);
            var progress = document.createElement('div');
            progress.textContent = '\u2026';
            builder_1.$(progress).addClass('tag', 'progress');
            var done = document.createElement('div');
            done.textContent = '\u2713';
            builder_1.$(done).addClass('tag', 'done');
            var error = document.createElement('div');
            error.textContent = '!';
            builder_1.$(error).addClass('tag', 'error');
            this.callOnDispose.push(this.addListener(events_1.EventType.BEFORE_RUN, function () {
                builder_1.$(progress).addClass('active');
                builder_1.$(done).removeClass('active');
                builder_1.$(error).removeClass('active');
            }));
            this.callOnDispose.push(this.addListener(events_1.EventType.RUN, function (result) {
                builder_1.$(progress).removeClass('active');
                if (result.error) {
                    builder_1.$(done).removeClass('active');
                    builder_1.$(error).addClass('active');
                }
                else {
                    builder_1.$(error).removeClass('active');
                    builder_1.$(done).addClass('active');
                }
            }));
            container.appendChild(label);
            container.appendChild(progress);
            container.appendChild(done);
            container.appendChild(error);
            parent.appendChild(container);
        };
        ProgressItem.prototype.dispose = function () {
            lifecycle.cAll(this.callOnDispose);
            _super.prototype.dispose.call(this);
        };
        return ProgressItem;
    }(BaseActionItem));
    exports.ProgressItem = ProgressItem;
    (function (ActionsOrientation) {
        ActionsOrientation[ActionsOrientation["HORIZONTAL"] = 1] = "HORIZONTAL";
        ActionsOrientation[ActionsOrientation["VERTICAL"] = 2] = "VERTICAL";
    })(exports.ActionsOrientation || (exports.ActionsOrientation = {}));
    var ActionsOrientation = exports.ActionsOrientation;
    var defaultOptions = {
        orientation: ActionsOrientation.HORIZONTAL,
        context: null
    };
    var ActionBar = (function (_super) {
        __extends(ActionBar, _super);
        function ActionBar(container, options) {
            var _this = this;
            if (options === void 0) { options = defaultOptions; }
            _super.call(this);
            this.options = options;
            this._context = options.context;
            this.toDispose = [];
            this._actionRunner = this.options.actionRunner;
            if (!this._actionRunner) {
                this._actionRunner = new actions_1.ActionRunner();
                this.toDispose.push(this._actionRunner);
            }
            this.toDispose.push(this.addEmitter2(this._actionRunner));
            this.items = [];
            this.focusedItem = undefined;
            this.domNode = document.createElement('div');
            this.domNode.className = 'monaco-action-bar';
            var isVertical = this.options.orientation === ActionsOrientation.VERTICAL;
            if (isVertical) {
                this.domNode.className += ' vertical';
            }
            builder_1.$(this.domNode).on(DOM.EventType.KEY_DOWN, function (e) {
                var event = new keyboardEvent_1.StandardKeyboardEvent(e);
                var eventHandled = true;
                if (event.equals(isVertical ? keyCodes_1.CommonKeybindings.UP_ARROW : keyCodes_1.CommonKeybindings.LEFT_ARROW)) {
                    _this.focusPrevious();
                }
                else if (event.equals(isVertical ? keyCodes_1.CommonKeybindings.DOWN_ARROW : keyCodes_1.CommonKeybindings.RIGHT_ARROW)) {
                    _this.focusNext();
                }
                else if (event.equals(keyCodes_1.CommonKeybindings.ESCAPE)) {
                    _this.cancel();
                }
                else if (event.equals(keyCodes_1.CommonKeybindings.ENTER) || event.equals(keyCodes_1.CommonKeybindings.SPACE)) {
                }
                else {
                    eventHandled = false;
                }
                if (eventHandled) {
                    event.preventDefault();
                    event.stopPropagation();
                }
            });
            // Prevent native context menu on actions
            builder_1.$(this.domNode).on(DOM.EventType.CONTEXT_MENU, function (e) {
                e.preventDefault();
                e.stopPropagation();
            });
            builder_1.$(this.domNode).on(DOM.EventType.KEY_UP, function (e) {
                var event = new keyboardEvent_1.StandardKeyboardEvent(e);
                // Run action on Enter/Space
                if (event.equals(keyCodes_1.CommonKeybindings.ENTER) || event.equals(keyCodes_1.CommonKeybindings.SPACE)) {
                    _this.doTrigger(event);
                    event.preventDefault();
                    event.stopPropagation();
                }
                else if (event.equals(keyCodes_1.CommonKeybindings.TAB) || event.equals(keyCodes_1.CommonKeybindings.SHIFT_TAB)) {
                    _this.updateFocusedItem();
                }
            });
            this.focusTracker = DOM.trackFocus(this.domNode);
            this.focusTracker.addBlurListener(function () {
                if (document.activeElement === _this.domNode || !DOM.isAncestor(document.activeElement, _this.domNode)) {
                    _this.emit(DOM.EventType.BLUR, {});
                    _this.focusedItem = undefined;
                }
            });
            this.focusTracker.addFocusListener(function () { return _this.updateFocusedItem(); });
            this.actionsList = document.createElement('ul');
            this.actionsList.className = 'actions-container';
            this.actionsList.setAttribute('role', 'toolbar');
            if (this.options.ariaLabel) {
                this.actionsList.setAttribute('aria-label', this.options.ariaLabel);
            }
            this.domNode.appendChild(this.actionsList);
            container = (container instanceof builder_1.Builder) ? container.getHTMLElement() : container;
            container.appendChild(this.domNode);
        }
        ActionBar.prototype.setAriaLabel = function (label) {
            if (label) {
                this.actionsList.setAttribute('aria-label', label);
            }
            else {
                this.actionsList.removeAttribute('aria-label');
            }
        };
        ActionBar.prototype.updateFocusedItem = function () {
            for (var i = 0; i < this.actionsList.children.length; i++) {
                var elem = this.actionsList.children[i];
                if (DOM.isAncestor(document.activeElement, elem)) {
                    this.focusedItem = i;
                    break;
                }
            }
        };
        Object.defineProperty(ActionBar.prototype, "context", {
            get: function () {
                return this._context;
            },
            set: function (context) {
                this._context = context;
                this.items.forEach(function (i) { return i.setActionContext(context); });
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ActionBar.prototype, "actionRunner", {
            get: function () {
                return this._actionRunner;
            },
            set: function (actionRunner) {
                if (actionRunner) {
                    this._actionRunner = actionRunner;
                    this.items.forEach(function (item) { return item.actionRunner = actionRunner; });
                }
            },
            enumerable: true,
            configurable: true
        });
        ActionBar.prototype.getContainer = function () {
            return builder_1.$(this.domNode);
        };
        ActionBar.prototype.push = function (actions, options) {
            var _this = this;
            if (options === void 0) { options = {}; }
            if (!Array.isArray(actions)) {
                actions = [actions];
            }
            var index = types.isNumber(options.index) ? options.index : null;
            actions.forEach(function (action) {
                var actionItemElement = document.createElement('li');
                actionItemElement.className = 'action-item';
                actionItemElement.setAttribute('role', 'presentation');
                var item = null;
                if (_this.options.actionItemProvider) {
                    item = _this.options.actionItemProvider(action);
                }
                if (!item) {
                    item = new ActionItem(_this.context, action, options);
                }
                item.actionRunner = _this._actionRunner;
                item.setActionContext(_this.context);
                _this.addEmitter(item);
                item.render(actionItemElement);
                if (index === null || index < 0 || index >= _this.actionsList.children.length) {
                    _this.actionsList.appendChild(actionItemElement);
                }
                else {
                    _this.actionsList.insertBefore(actionItemElement, _this.actionsList.children[index++]);
                }
                _this.items.push(item);
            });
        };
        ActionBar.prototype.clear = function () {
            var item;
            while (item = this.items.pop()) {
                item.dispose();
            }
            builder_1.$(this.actionsList).empty();
        };
        ActionBar.prototype.length = function () {
            return this.items.length;
        };
        ActionBar.prototype.isEmpty = function () {
            return this.items.length === 0;
        };
        ActionBar.prototype.onContentsChange = function () {
            this.emit(events_1.EventType.CONTENTS_CHANGED);
        };
        ActionBar.prototype.focus = function (selectFirst) {
            if (selectFirst && typeof this.focusedItem === 'undefined') {
                this.focusedItem = 0;
            }
            this.updateFocus();
        };
        ActionBar.prototype.focusNext = function () {
            if (typeof this.focusedItem === 'undefined') {
                this.focusedItem = this.items.length - 1;
            }
            var startIndex = this.focusedItem;
            var item;
            do {
                this.focusedItem = (this.focusedItem + 1) % this.items.length;
                item = this.items[this.focusedItem];
            } while (this.focusedItem !== startIndex && !item.isEnabled());
            if (this.focusedItem === startIndex && !item.isEnabled()) {
                this.focusedItem = undefined;
            }
            this.updateFocus();
        };
        ActionBar.prototype.focusPrevious = function () {
            if (typeof this.focusedItem === 'undefined') {
                this.focusedItem = 0;
            }
            var startIndex = this.focusedItem;
            var item;
            do {
                this.focusedItem = this.focusedItem - 1;
                if (this.focusedItem < 0) {
                    this.focusedItem = this.items.length - 1;
                }
                item = this.items[this.focusedItem];
            } while (this.focusedItem !== startIndex && !item.isEnabled());
            if (this.focusedItem === startIndex && !item.isEnabled()) {
                this.focusedItem = undefined;
            }
            this.updateFocus();
        };
        ActionBar.prototype.updateFocus = function () {
            if (typeof this.focusedItem === 'undefined') {
                this.domNode.focus();
                return;
            }
            for (var i = 0; i < this.items.length; i++) {
                var item = this.items[i];
                var actionItem = item;
                if (i === this.focusedItem) {
                    if (types.isFunction(actionItem.focus)) {
                        actionItem.focus();
                    }
                }
                else {
                    if (types.isFunction(actionItem.blur)) {
                        actionItem.blur();
                    }
                }
            }
        };
        ActionBar.prototype.doTrigger = function (event) {
            if (typeof this.focusedItem === 'undefined') {
                return; //nothing to focus
            }
            // trigger action
            var actionItem = this.items[this.focusedItem];
            this.run(actionItem._action, actionItem._context || event).done();
        };
        ActionBar.prototype.cancel = function () {
            if (document.activeElement instanceof HTMLElement) {
                document.activeElement.blur(); // remove focus from focussed action
            }
            this.emit(events_1.EventType.CANCEL);
        };
        ActionBar.prototype.run = function (action, context) {
            return this._actionRunner.run(action, context);
        };
        ActionBar.prototype.dispose = function () {
            if (this.items !== null) {
                this.clear();
            }
            this.items = null;
            if (this.focusTracker) {
                this.focusTracker.dispose();
                this.focusTracker = null;
            }
            this.toDispose = lifecycle.disposeAll(this.toDispose);
            this.getContainer().destroy();
            _super.prototype.dispose.call(this);
        };
        return ActionBar;
    }(eventEmitter_1.EventEmitter));
    exports.ActionBar = ActionBar;
    var SelectActionItem = (function (_super) {
        __extends(SelectActionItem, _super);
        function SelectActionItem(ctx, action, options, selected) {
            _super.call(this, ctx, action);
            this.select = document.createElement('select');
            this.select.className = 'action-bar-select';
            this.options = options;
            this.selected = selected;
            this.toDispose = [];
            this.registerListeners();
        }
        SelectActionItem.prototype.setOptions = function (options, selected) {
            this.options = options;
            this.selected = selected;
            this.doSetOptions();
        };
        SelectActionItem.prototype.registerListeners = function () {
            var _this = this;
            this.toDispose.push(DOM.addStandardDisposableListener(this.select, 'change', function (e) {
                _this.actionRunner.run(_this._action, e.target.value).done();
            }));
        };
        SelectActionItem.prototype.focus = function () {
            if (this.select) {
                this.select.focus();
            }
        };
        SelectActionItem.prototype.blur = function () {
            if (this.select) {
                this.select.blur();
            }
        };
        SelectActionItem.prototype.render = function (container) {
            DOM.addClass(container, 'select-container');
            container.appendChild(this.select);
            this.doSetOptions();
        };
        SelectActionItem.prototype.doSetOptions = function () {
            var _this = this;
            this.select.options.length = 0;
            this.options.forEach(function (option) {
                _this.select.add(_this.createOption(option));
            });
            if (this.selected >= 0) {
                this.select.selectedIndex = this.selected;
            }
        };
        SelectActionItem.prototype.createOption = function (value) {
            var option = document.createElement('option');
            option.value = value;
            option.text = value;
            return option;
        };
        SelectActionItem.prototype.dispose = function () {
            this.toDispose = lifecycle.disposeAll(this.toDispose);
            _super.prototype.dispose.call(this);
        };
        return SelectActionItem;
    }(BaseActionItem));
    exports.SelectActionItem = SelectActionItem;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/ui/aria/aria", ["require", "exports", 'vs/nls!vs/base/browser/ui/aria/aria', 'vs/base/common/platform', 'vs/base/browser/builder', 'vs/css!./aria'], function (require, exports, nls, platform_1, builder_1) {
    'use strict';
    var ariaContainer;
    var alertContainer;
    var statusContainer;
    function setARIAContainer(parent) {
        ariaContainer = builder_1.$('.aria-container').appendTo(parent);
        alertContainer = builder_1.$('.alert').appendTo(ariaContainer).attr({ 'role': 'alert', 'aria-atomic': 'true' });
        statusContainer = builder_1.$('.status').appendTo(ariaContainer).attr({ 'role': 'status', 'aria-atomic': 'true' });
    }
    exports.setARIAContainer = setARIAContainer;
    /**
     * Given the provided message, will make sure that it is read as alert to screen readers.
     */
    function alert(msg) {
        insertMessage(alertContainer, msg);
    }
    exports.alert = alert;
    /**
     * Given the provided message, will make sure that it is read as status to screen readers.
     */
    function status(msg) {
        if (platform_1.isMacintosh) {
            alert(msg); // VoiceOver does not seem to support status role
        }
        else {
            insertMessage(statusContainer, msg);
        }
    }
    exports.status = status;
    function insertMessage(target, msg) {
        if (!ariaContainer) {
            console.warn('ARIA support needs a container. Call setARIAContainer() first.');
            return;
        }
        if (target.getHTMLElement().textContent === msg) {
            msg = nls.localize(0, null, msg);
        }
        builder_1.$(target).empty();
        builder_1.$(target).text(msg);
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/button/button", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/base/browser/dom', 'vs/base/browser/builder', 'vs/base/browser/keyboardEvent', 'vs/base/common/keyCodes', 'vs/css!./button'], function (require, exports, eventEmitter_1, DOM, builder_1, keyboardEvent_1, keyCodes_1) {
    'use strict';
    var Button = (function (_super) {
        __extends(Button, _super);
        function Button(container) {
            var _this = this;
            _super.call(this);
            this.$el = builder_1.$('a.monaco-button').attr({
                'tabIndex': '0',
                'role': 'button'
            }).appendTo(container);
            this.$el.on(DOM.EventType.CLICK, function (e) {
                if (!_this.enabled) {
                    DOM.EventHelper.stop(e);
                    return;
                }
                _this.emit(DOM.EventType.CLICK, e);
            });
            this.$el.on(DOM.EventType.KEY_DOWN, function (e) {
                var event = new keyboardEvent_1.StandardKeyboardEvent(e);
                var eventHandled = false;
                if (_this.enabled && event.equals(keyCodes_1.CommonKeybindings.ENTER) || event.equals(keyCodes_1.CommonKeybindings.SPACE)) {
                    _this.emit(DOM.EventType.CLICK, e);
                    eventHandled = true;
                }
                else if (event.equals(keyCodes_1.CommonKeybindings.ESCAPE)) {
                    _this.$el.domBlur();
                    eventHandled = true;
                }
                if (eventHandled) {
                    DOM.EventHelper.stop(event, true);
                }
            });
        }
        Button.prototype.getElement = function () {
            return this.$el.getHTMLElement();
        };
        Object.defineProperty(Button.prototype, "label", {
            set: function (value) {
                this.$el.text(value);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Button.prototype, "enabled", {
            get: function () {
                return !this.$el.hasClass('disabled');
            },
            set: function (value) {
                if (value) {
                    this.$el.removeClass('disabled');
                    this.$el.attr({
                        'aria-disabled': 'false',
                        'tabIndex': '0'
                    });
                }
                else {
                    this.$el.addClass('disabled');
                    this.$el.attr('aria-disabled', String(true));
                    DOM.removeTabIndexAndUpdateFocus(this.$el.getHTMLElement());
                }
            },
            enumerable: true,
            configurable: true
        });
        Button.prototype.dispose = function () {
            if (this.$el) {
                this.$el.dispose();
                this.$el = null;
            }
            _super.prototype.dispose.call(this);
        };
        return Button;
    }(eventEmitter_1.EventEmitter));
    exports.Button = Button;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/contextview/contextview", ["require", "exports", 'vs/base/browser/builder', 'vs/base/browser/dom', 'vs/base/common/lifecycle', 'vs/base/common/eventEmitter', 'vs/css!./contextview'], function (require, exports, builder_1, DOM, lifecycle_1, eventEmitter_1) {
    'use strict';
    (function (AnchorAlignment) {
        AnchorAlignment[AnchorAlignment["LEFT"] = 0] = "LEFT";
        AnchorAlignment[AnchorAlignment["RIGHT"] = 1] = "RIGHT";
    })(exports.AnchorAlignment || (exports.AnchorAlignment = {}));
    var AnchorAlignment = exports.AnchorAlignment;
    (function (AnchorPosition) {
        AnchorPosition[AnchorPosition["BELOW"] = 0] = "BELOW";
        AnchorPosition[AnchorPosition["ABOVE"] = 1] = "ABOVE";
    })(exports.AnchorPosition || (exports.AnchorPosition = {}));
    var AnchorPosition = exports.AnchorPosition;
    function layout(view, around, inside, anchorPosition, anchorAlignment) {
        var top, left;
        if (anchorPosition === AnchorPosition.BELOW) {
            top = around.top + around.height - inside.top;
            if (inside.top + top + view.height > inside.height && around.top - inside.top > view.height) {
                top = around.top - view.height - inside.top;
            }
        }
        else {
            top = around.top - view.height - inside.top;
            if (top + inside.top < 0 && around.top + around.height + view.height - inside.top < inside.height) {
                top = around.top + around.height - inside.top;
            }
        }
        if (anchorAlignment === AnchorAlignment.LEFT) {
            left = around.left - inside.left;
            if (inside.left + left + view.width > inside.width) {
                left -= view.width - around.width;
            }
        }
        else {
            left = around.left + around.width - view.width - inside.left;
            if (left + inside.left < 0 && around.left + view.width < inside.width) {
                left = around.left - inside.left;
            }
        }
        return { top: top, left: left };
    }
    exports.layout = layout;
    var ContextView = (function (_super) {
        __extends(ContextView, _super);
        function ContextView(container) {
            var _this = this;
            _super.call(this);
            this.$view = builder_1.$('.context-view').hide();
            this.setContainer(container);
            this.toDispose = [{
                    dispose: function () {
                        _this.setContainer(null);
                    }
                }];
            this.toDisposeOnClean = null;
        }
        ContextView.prototype.setContainer = function (container) {
            var _this = this;
            if (this.$container) {
                this.$container.off(ContextView.BUBBLE_UP_EVENTS);
                this.$container.off(ContextView.BUBBLE_DOWN_EVENTS, true);
                this.$container = null;
            }
            if (container) {
                this.$container = builder_1.$(container);
                this.$view.appendTo(this.$container);
                this.$container.on(ContextView.BUBBLE_UP_EVENTS, function (e) {
                    _this.onDOMEvent(e, document.activeElement, false);
                });
                this.$container.on(ContextView.BUBBLE_DOWN_EVENTS, function (e) {
                    _this.onDOMEvent(e, document.activeElement, true);
                }, null, true);
            }
        };
        ContextView.prototype.show = function (delegate) {
            if (this.isVisible()) {
                this.hide();
            }
            // Show static box
            this.$view.setClass('context-view').empty().style({ top: '0px', left: '0px' }).show();
            // Render content
            this.toDisposeOnClean = delegate.render(this.$view.getHTMLElement());
            // Set active delegate
            this.delegate = delegate;
            // Layout
            this.doLayout();
        };
        ContextView.prototype.layout = function () {
            if (!this.isVisible()) {
                return;
            }
            if (this.delegate.canRelayout === false) {
                this.hide();
                return;
            }
            if (this.delegate.layout) {
                this.delegate.layout();
            }
            this.doLayout();
        };
        ContextView.prototype.doLayout = function () {
            // Get anchor
            var anchor = this.delegate.getAnchor();
            // Compute around
            var around;
            // Get the element's position and size (to anchor the view)
            if (DOM.isHTMLElement(anchor)) {
                var $anchor = builder_1.$(anchor);
                var elementPosition = $anchor.getPosition();
                var elementSize = $anchor.getTotalSize();
                around = {
                    top: elementPosition.top,
                    left: elementPosition.left,
                    width: elementSize.width,
                    height: elementSize.height
                };
            }
            else {
                var realAnchor = anchor;
                around = {
                    top: realAnchor.y,
                    left: realAnchor.x,
                    width: realAnchor.width || 0,
                    height: realAnchor.height || 0
                };
            }
            // Get the container's position
            var insidePosition = this.$container.getPosition();
            var inside = {
                top: insidePosition.top,
                left: insidePosition.left,
                height: window.innerHeight,
                width: window.innerWidth
            };
            // Get the view's size
            var viewSize = this.$view.getTotalSize();
            var view = { width: viewSize.width, height: viewSize.height };
            var anchorPosition = this.delegate.anchorPosition || AnchorPosition.BELOW;
            var anchorAlignment = this.delegate.anchorAlignment || AnchorAlignment.LEFT;
            var result = layout(view, around, inside, anchorPosition, anchorAlignment);
            this.$view.removeClass('top', 'bottom', 'left', 'right');
            this.$view.addClass(anchorPosition === AnchorPosition.BELOW ? 'bottom' : 'top');
            this.$view.addClass(anchorAlignment === AnchorAlignment.LEFT ? 'left' : 'right');
            this.$view.style({ top: result.top + 'px', left: result.left + 'px', width: 'initial' });
        };
        ContextView.prototype.hide = function (data) {
            if (this.delegate && this.delegate.onHide) {
                this.delegate.onHide(data);
            }
            this.delegate = null;
            if (this.toDisposeOnClean) {
                this.toDisposeOnClean.dispose();
                this.toDisposeOnClean = null;
            }
            this.$view.hide();
        };
        ContextView.prototype.isVisible = function () {
            return !!this.delegate;
        };
        ContextView.prototype.onDOMEvent = function (e, element, onCapture) {
            if (this.delegate) {
                if (this.delegate.onDOMEvent) {
                    this.delegate.onDOMEvent(e, document.activeElement);
                }
                else if (onCapture && !DOM.isAncestor(e.target, this.$container.getHTMLElement())) {
                    this.hide();
                }
            }
        };
        ContextView.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this.hide();
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
        };
        ContextView.BUBBLE_UP_EVENTS = ['click', 'keydown', 'focus', 'blur'];
        ContextView.BUBBLE_DOWN_EVENTS = ['click'];
        return ContextView;
    }(eventEmitter_1.EventEmitter));
    exports.ContextView = ContextView;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/ui/countBadge/countBadge", ["require", "exports", 'vs/base/browser/builder', 'vs/base/common/strings', 'vs/css!./countBadge'], function (require, exports, builder_1, strings) {
    'use strict';
    var CountBadge = (function () {
        function CountBadge(container, count, titleFormat) {
            this.$el = builder_1.$('.monaco-count-badge').appendTo(container);
            this.titleFormat = titleFormat || '';
            this.setCount(count || 0);
        }
        CountBadge.prototype.setCount = function (count) {
            this.count = count;
            this.render();
        };
        CountBadge.prototype.setTitleFormat = function (titleFormat) {
            this.titleFormat = titleFormat;
            this.render();
        };
        CountBadge.prototype.render = function () {
            this.$el.text('' + this.count);
            this.$el.title(strings.format(this.titleFormat, this.count));
        };
        CountBadge.prototype.dispose = function () {
            if (this.$el) {
                this.$el.destroy();
                this.$el = null;
            }
        };
        return CountBadge;
    }());
    exports.CountBadge = CountBadge;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/dropdown/dropdown", ["require", "exports", 'vs/base/browser/builder', 'vs/base/common/winjs.base', 'vs/base/browser/touch', 'vs/base/common/actions', 'vs/base/browser/ui/actionbar/actionbar', 'vs/base/common/eventEmitter', 'vs/base/common/lifecycle', 'vs/css!./dropdown'], function (require, exports, builder_1, winjs_base_1, touch_1, actions_1, actionbar_1, eventEmitter_1, lifecycle_1) {
    'use strict';
    var BaseDropdown = (function (_super) {
        __extends(BaseDropdown, _super);
        function BaseDropdown(container, options) {
            var _this = this;
            _super.call(this);
            this.toDispose = [];
            this.$el = builder_1.$('.dropdown').appendTo(container);
            this.$label = builder_1.$('.dropdown-label');
            if (options.tick || options.action) {
                this.$label.addClass('tick');
            }
            var labelRenderer = options.labelRenderer;
            if (!labelRenderer && options.action) {
                this.$action = builder_1.$('.dropdown-action').appendTo(this.$el);
                var item_1 = new actionbar_1.ActionItem(null, options.action, {
                    icon: true,
                    label: true
                });
                item_1.actionRunner = this;
                item_1.render(this.$action.getHTMLElement());
                labelRenderer = function (container) {
                    container.innerText = '';
                    return item_1;
                };
            }
            if (!labelRenderer) {
                labelRenderer = function (container) {
                    builder_1.$(container).text(options.label || '');
                    return null;
                };
            }
            this.$label.on(['mousedown', touch_1.EventType.Tap], function (e) {
                e.preventDefault();
                e.stopPropagation();
                _this.toggleDropdown();
            }).appendTo(this.$el);
            var cleanupFn = labelRenderer(this.$label.getHTMLElement());
            if (cleanupFn) {
                this.toDispose.push(cleanupFn);
            }
            this.toDispose.push(new touch_1.Gesture(this.$label.getHTMLElement()));
        }
        Object.defineProperty(BaseDropdown.prototype, "tooltip", {
            set: function (tooltip) {
                this.$label.title(tooltip);
            },
            enumerable: true,
            configurable: true
        });
        /*protected*/ BaseDropdown.prototype.toggleDropdown = function () {
            if (this.$el.hasClass('active')) {
                this.hide();
            }
            else {
                this.show();
            }
        };
        /*protected*/ BaseDropdown.prototype.show = function () {
            // noop
        };
        /*protected*/ BaseDropdown.prototype.hide = function () {
            // noop
        };
        /*protected*/ BaseDropdown.prototype.onEvent = function (e, activeElement) {
            this.hide();
        };
        BaseDropdown.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this.hide();
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            if (this.$boxContainer) {
                this.$boxContainer.destroy();
                this.$boxContainer = null;
            }
            if (this.$contents) {
                this.$contents.destroy();
                this.$contents = null;
            }
            if (this.$label) {
                this.$label.destroy();
                this.$label = null;
            }
        };
        return BaseDropdown;
    }(actions_1.ActionRunner));
    exports.BaseDropdown = BaseDropdown;
    var Dropdown = (function (_super) {
        __extends(Dropdown, _super);
        function Dropdown(container, options) {
            _super.call(this, container, options);
            this.contextViewProvider = options.contextViewProvider;
        }
        Object.defineProperty(Dropdown.prototype, "contextViewProvider", {
            /*protected*/ get: function () {
                return this._contextViewProvider;
            },
            /*protected*/ set: function (contextViewProvider) {
                this._contextViewProvider = contextViewProvider;
            },
            enumerable: true,
            configurable: true
        });
        /*protected*/ Dropdown.prototype.show = function () {
            var _this = this;
            this.$el.addClass('active');
            this._contextViewProvider.showContextView({
                getAnchor: function () { return _this.$el.getHTMLElement(); },
                render: function (container) {
                    return _this.renderContents(container);
                },
                onDOMEvent: function (e, activeElement) {
                    _this.onEvent(e, activeElement);
                },
                onHide: function () {
                    _this.$el.removeClass('active');
                }
            });
        };
        /*protected*/ Dropdown.prototype.hide = function () {
            if (this._contextViewProvider) {
                this._contextViewProvider.hideContextView();
            }
        };
        /*protected*/ Dropdown.prototype.renderContents = function (container) {
            return null;
        };
        return Dropdown;
    }(BaseDropdown));
    exports.Dropdown = Dropdown;
    var DropdownMenu = (function (_super) {
        __extends(DropdownMenu, _super);
        function DropdownMenu(container, options) {
            _super.call(this, container, options);
            this._contextMenuProvider = options.contextMenuProvider;
            this.currentContainer = null;
            this.actions = options.actions || [];
            this.actionProvider = options.actionProvider;
            this.menuClassName = options.menuClassName || '';
        }
        Object.defineProperty(DropdownMenu.prototype, "contextMenuProvider", {
            /*protected*/ get: function () {
                return this._contextMenuProvider;
            },
            /*protected*/ set: function (contextMenuProvider) {
                this._contextMenuProvider = contextMenuProvider;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DropdownMenu.prototype, "menuOptions", {
            get: function () {
                return this._menuOptions;
            },
            set: function (options) {
                this._menuOptions = options;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DropdownMenu.prototype, "actions", {
            /*protected*/ get: function () {
                if (this.actionProvider) {
                    return this.actionProvider.getActions();
                }
                return this._actions;
            },
            /*protected*/ set: function (actions) {
                this._actions = actions;
            },
            enumerable: true,
            configurable: true
        });
        /*protected*/ DropdownMenu.prototype.show = function () {
            var _this = this;
            this.$el.addClass('active');
            this._contextMenuProvider.showContextMenu({
                getAnchor: function () { return _this.$el.getHTMLElement(); },
                getActions: function () { return winjs_base_1.TPromise.as(_this.actions); },
                getActionsContext: function () { return _this.menuOptions ? _this.menuOptions.context : null; },
                getActionItem: function (action) { return _this.menuOptions && _this.menuOptions.actionItemProvider ? _this.menuOptions.actionItemProvider(action) : null; },
                getMenuClassName: function () { return _this.menuClassName; },
                onHide: function () {
                    _this.$el.removeClass('active');
                    _this.currentContainer = null;
                }
            });
        };
        /*protected*/ DropdownMenu.prototype.hide = function () {
            // noop
        };
        return DropdownMenu;
    }(BaseDropdown));
    exports.DropdownMenu = DropdownMenu;
    var DropdownGroup = (function (_super) {
        __extends(DropdownGroup, _super);
        function DropdownGroup(container) {
            _super.call(this);
            this.el = document.createElement('div');
            this.el.className = 'dropdown-group';
            container.appendChild(this.el);
        }
        Object.defineProperty(DropdownGroup.prototype, "element", {
            get: function () {
                return this.el;
            },
            enumerable: true,
            configurable: true
        });
        return DropdownGroup;
    }(eventEmitter_1.EventEmitter));
    exports.DropdownGroup = DropdownGroup;
});

define("vs/base/browser/ui/filelabel/fileLabel", ["require", "exports", 'vs/base/browser/dom', 'vs/base/common/paths', 'vs/base/common/types', 'vs/base/common/labels', 'vs/css!./filelabel'], function (require, exports, dom, paths, types, labels_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FileLabel = (function () {
        function FileLabel(container, arg2, arg3) {
            this.domNode = document.createElement('span');
            this.domNode.className = 'monaco-file-label';
            container.appendChild(this.domNode);
            if (arg3) {
                this.basepath = getPath(arg3);
            }
            if (arg2) {
                this.setValue(arg2);
            }
        }
        FileLabel.prototype.getHTMLElement = function () {
            return this.domNode;
        };
        FileLabel.prototype.setValue = function (arg1) {
            var newPath = getPath(arg1);
            if (this.renderedOnce && this.path === newPath) {
                // don't render again if nothing has changed
                return;
            }
            this.path = newPath;
            this.render();
            this.renderedOnce = true;
        };
        FileLabel.prototype.render = function () {
            dom.clearNode(this.domNode);
            var htmlContent = [];
            htmlContent.push('<span class="file-name">');
            htmlContent.push(paths.basename(this.path));
            htmlContent.push('</span>');
            var parent = paths.dirname(this.path);
            if (parent && parent !== '.') {
                var pathLabel = labels_1.getPathLabel(parent, this.basepath);
                htmlContent.push('<span class="file-path" title="' + pathLabel + '">');
                htmlContent.push(pathLabel);
                htmlContent.push('</span>');
            }
            this.domNode.innerHTML = htmlContent.join('');
        };
        return FileLabel;
    }());
    exports.FileLabel = FileLabel;
    function getPath(arg1) {
        if (!arg1) {
            return null;
        }
        if (typeof arg1 === 'string') {
            return arg1;
        }
        if (types.isFunction(arg1.getWorkspace)) {
            var ws = arg1.getWorkspace();
            return ws ? ws.resource.fsPath : void 0;
        }
        return arg1.fsPath;
    }
});

define("vs/base/browser/ui/highlightedlabel/highlightedLabel", ["require", "exports", 'vs/base/common/strings', 'vs/base/browser/dom', 'vs/base/common/objects', 'vs/base/browser/ui/octiconLabel/octiconLabel'], function (require, exports, strings_1, dom, objects, octiconLabel_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var HighlightedLabel = (function () {
        function HighlightedLabel(container) {
            this.domNode = document.createElement('span');
            this.domNode.className = 'monaco-highlighted-label';
            this.didEverRender = false;
            container.appendChild(this.domNode);
        }
        Object.defineProperty(HighlightedLabel.prototype, "element", {
            get: function () {
                return this.domNode;
            },
            enumerable: true,
            configurable: true
        });
        HighlightedLabel.prototype.set = function (text, highlights) {
            if (highlights === void 0) { highlights = []; }
            if (!text) {
                text = '';
            }
            if (this.didEverRender && this.text === text && objects.equals(this.highlights, highlights)) {
                return;
            }
            if (!Array.isArray(highlights)) {
                highlights = [];
            }
            this.text = text;
            this.highlights = highlights;
            this.render();
        };
        HighlightedLabel.prototype.render = function () {
            dom.clearNode(this.domNode);
            var htmlContent = [], highlight, pos = 0;
            for (var i = 0; i < this.highlights.length; i++) {
                highlight = this.highlights[i];
                if (highlight.end === highlight.start) {
                    continue;
                }
                if (pos < highlight.start) {
                    htmlContent.push('<span>');
                    htmlContent.push(octiconLabel_1.expand(strings_1.escape(this.text.substring(pos, highlight.start))));
                    htmlContent.push('</span>');
                    pos = highlight.end;
                }
                htmlContent.push('<span class="highlight">');
                htmlContent.push(octiconLabel_1.expand(strings_1.escape(this.text.substring(highlight.start, highlight.end))));
                htmlContent.push('</span>');
                pos = highlight.end;
            }
            if (pos < this.text.length) {
                htmlContent.push('<span>');
                htmlContent.push(octiconLabel_1.expand(strings_1.escape(this.text.substring(pos))));
                htmlContent.push('</span>');
            }
            this.domNode.innerHTML = htmlContent.join('');
            this.didEverRender = true;
        };
        HighlightedLabel.prototype.dispose = function () {
            this.text = null;
            this.highlights = null;
        };
        return HighlightedLabel;
    }());
    exports.HighlightedLabel = HighlightedLabel;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/ui/leftRightWidget/leftRightWidget", ["require", "exports", 'vs/base/browser/builder', 'vs/css!./leftRightWidget'], function (require, exports, builder_1) {
    'use strict';
    var LeftRightWidget = (function () {
        function LeftRightWidget(container, renderLeftFn, renderRightFn) {
            this.$el = builder_1.$('.monaco-left-right-widget').appendTo(container);
            this.toDispose = [
                renderRightFn(builder_1.$('.right').appendTo(this.$el).getHTMLElement()),
                renderLeftFn(builder_1.$('span.left').appendTo(this.$el).getHTMLElement())
            ].filter(function (x) { return !!x; });
        }
        LeftRightWidget.prototype.dispose = function () {
            if (this.$el) {
                this.$el.destroy();
                this.$el = null;
            }
        };
        return LeftRightWidget;
    }());
    exports.LeftRightWidget = LeftRightWidget;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/ui/list/rowCache", ["require", "exports", 'vs/base/browser/dom'], function (require, exports, dom_1) {
    "use strict";
    function getLastScrollTime(element) {
        var value = element.getAttribute('last-scroll-time');
        return value ? parseInt(value, 10) : 0;
    }
    function removeFromParent(element) {
        try {
            element.parentElement.removeChild(element);
        }
        catch (e) {
        }
    }
    var RowCache = (function () {
        function RowCache(renderers) {
            this.renderers = renderers;
            this.cache = Object.create(null);
            this.scrollingRow = null;
        }
        /**
         * Returns a row either by creating a new one or reusing
         * a previously released row which shares the same templateId.
         */
        RowCache.prototype.alloc = function (templateId) {
            var result = this.getTemplateCache(templateId).pop();
            if (!result) {
                var domNode = dom_1.emmet('.monaco-list-row');
                var renderer = this.renderers[templateId];
                var templateData = renderer.renderTemplate(domNode);
                result = { domNode: domNode, templateId: templateId, templateData: templateData };
            }
            return result;
        };
        /**
         * Releases the row for eventual reuse. The row's domNode
         * will eventually be removed from its parent, given that
         * it is not the currently scrolling row (for OS X ballistic
         * scrolling).
         */
        RowCache.prototype.release = function (row) {
            var lastScrollTime = getLastScrollTime(row.domNode);
            if (!lastScrollTime) {
                removeFromParent(row.domNode);
                this.getTemplateCache(row.templateId).push(row);
                return;
            }
            if (this.scrollingRow) {
                var lastKnownScrollTime = getLastScrollTime(this.scrollingRow.domNode);
                if (lastKnownScrollTime > lastScrollTime) {
                    removeFromParent(row.domNode);
                    this.getTemplateCache(row.templateId).push(row);
                    return;
                }
                if (this.scrollingRow.domNode.parentElement) {
                    removeFromParent(this.scrollingRow.domNode);
                    dom_1.removeClass(this.scrollingRow.domNode, 'scrolling');
                    this.getTemplateCache(this.scrollingRow.templateId).push(this.scrollingRow);
                }
            }
            this.scrollingRow = row;
            dom_1.addClass(this.scrollingRow.domNode, 'scrolling');
        };
        RowCache.prototype.getTemplateCache = function (templateId) {
            return this.cache[templateId] || (this.cache[templateId] = []);
        };
        RowCache.prototype.garbageCollect = function () {
            var _this = this;
            if (this.cache) {
                Object.keys(this.cache).forEach(function (templateId) {
                    _this.cache[templateId].forEach(function (cachedRow) {
                        var renderer = _this.renderers[templateId];
                        renderer.disposeTemplate(cachedRow.templateData);
                        cachedRow.domNode = null;
                        cachedRow.templateData = null;
                    });
                    delete _this.cache[templateId];
                });
            }
            if (this.scrollingRow) {
                var renderer = this.renderers[this.scrollingRow.templateId];
                renderer.disposeTemplate(this.scrollingRow.templateData);
                this.scrollingRow = null;
            }
        };
        RowCache.prototype.dispose = function () {
            this.garbageCollect();
            this.cache = null;
            this.renderers = null;
        };
        return RowCache;
    }());
    exports.RowCache = RowCache;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/ui/messagelist/messageList", ["require", "exports", 'vs/nls!vs/base/browser/ui/messagelist/messageList', 'vs/base/common/winjs.base', 'vs/base/browser/builder', 'vs/base/browser/dom', 'vs/base/common/errors', 'vs/base/browser/ui/aria/aria', 'vs/base/common/types', 'vs/base/common/event', 'vs/base/common/actions', 'vs/base/browser/htmlContentRenderer', 'vs/base/browser/keyboardEvent', 'vs/base/common/keyCodes', 'vs/css!./messageList'], function (require, exports, nls, winjs_base_1, builder_1, DOM, errors, aria, types, event_1, actions_1, htmlRenderer, keyboardEvent_1, keyCodes_1) {
    'use strict';
    (function (Severity) {
        Severity[Severity["Info"] = 0] = "Info";
        Severity[Severity["Warning"] = 1] = "Warning";
        Severity[Severity["Error"] = 2] = "Error";
    })(exports.Severity || (exports.Severity = {}));
    var Severity = exports.Severity;
    var IMessageListOptions = (function () {
        function IMessageListOptions() {
        }
        return IMessageListOptions;
    }());
    exports.IMessageListOptions = IMessageListOptions;
    var MessageList = (function () {
        function MessageList(containerElementId, usageLogger, options) {
            if (options === void 0) { options = { purgeInterval: MessageList.DEFAULT_MESSAGE_PURGER_INTERVAL, maxMessages: MessageList.DEFAULT_MAX_MESSAGES, maxMessageLength: MessageList.DEFAULT_MAX_MESSAGE_LENGTH }; }
            this.messages = [];
            this.messageListPurger = null;
            this.containerElementId = containerElementId;
            this.usageLogger = usageLogger;
            this.options = options;
            this._onMessagesShowing = new event_1.Emitter();
            this._onMessagesCleared = new event_1.Emitter();
        }
        Object.defineProperty(MessageList.prototype, "onMessagesShowing", {
            get: function () {
                return this._onMessagesShowing.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MessageList.prototype, "onMessagesCleared", {
            get: function () {
                return this._onMessagesCleared.event;
            },
            enumerable: true,
            configurable: true
        });
        MessageList.prototype.showMessage = function (severity, message) {
            var _this = this;
            if (Array.isArray(message)) {
                var closeFns_1 = [];
                message.forEach(function (msg) { return closeFns_1.push(_this.showMessage(severity, msg)); });
                return function () { return closeFns_1.forEach(function (fn) { return fn(); }); };
            }
            // Return only if we are unable to extract a message text
            var messageText = this.getMessageText(message);
            if (!messageText || typeof messageText !== 'string') {
                return function () { };
            }
            // Show message
            return this.doShowMessage(message, messageText, severity);
        };
        MessageList.prototype.getMessageText = function (message) {
            if (types.isString(message)) {
                return message;
            }
            if (message instanceof Error) {
                return errors.toErrorMessage(message, false);
            }
            if (message.message) {
                return message.message;
            }
            return null;
        };
        MessageList.prototype.doShowMessage = function (id, message, severity) {
            var _this = this;
            // Trigger Auto-Purge of messages to keep list small
            this.purgeMessages();
            // Store in Memory (new messages come first so that they show up on top)
            this.messages.unshift({
                id: id,
                text: message,
                severity: severity,
                time: new Date().getTime(),
                actions: id.actions
            });
            // Render
            this.renderMessages(true, 1);
            // Support in Screen Readers too
            var alertText;
            if (severity === Severity.Error) {
                alertText = nls.localize(0, null, message);
            }
            else if (severity === Severity.Warning) {
                alertText = nls.localize(1, null, message);
            }
            else {
                alertText = nls.localize(2, null, message);
            }
            aria.alert(alertText);
            return function () {
                _this.hideMessage(id);
            };
        };
        MessageList.prototype.renderMessages = function (animate, delta) {
            var _this = this;
            var container = builder_1.withElementById(this.containerElementId);
            if (!container) {
                return; // Cannot build container for messages yet, return
            }
            // Lazily create, otherwise clear old
            if (!this.messageListContainer) {
                this.messageListContainer = builder_1.$('.global-message-list').appendTo(container);
            }
            else {
                builder_1.$(this.messageListContainer).empty();
                builder_1.$(this.messageListContainer).removeClass('transition');
            }
            // Support animation for messages by moving the container out of view and then in
            if (animate) {
                builder_1.$(this.messageListContainer).style('top', '-35px');
            }
            // Render Messages as List Items
            builder_1.$(this.messageListContainer).ul({ 'class': 'message-list' }, function (ul) {
                var messages = _this.prepareMessages();
                if (messages.length > 0) {
                    _this._onMessagesShowing.fire();
                }
                else {
                    _this._onMessagesCleared.fire();
                }
                messages.forEach(function (message, index) {
                    _this.renderMessage(message, builder_1.$(ul), messages.length, delta);
                });
                // Support animation for messages by moving the container out of view and then in
                if (animate) {
                    setTimeout(function () {
                        builder_1.$(_this.messageListContainer).addClass('transition');
                        builder_1.$(_this.messageListContainer).style('top', '0');
                    }, 50 /* Need this delay to reliably get the animation on some browsers */);
                }
            });
        };
        MessageList.prototype.renderMessage = function (message, container, total, delta) {
            var _this = this;
            container.li({ class: 'message-list-entry message-list-entry-with-action' }, function (li) {
                // Actions (if none provided, add one default action to hide message)
                var messageActions = _this.getMessageActions(message);
                li.div({ class: (total > 1 || delta < 0) ? 'actions-container multiple' : 'actions-container' }, function (actionContainer) {
                    var _loop_1 = function(i) {
                        var action = messageActions[i];
                        actionContainer.div({ class: 'message-action' }, function (div) {
                            div.a({ class: 'action-button', tabindex: '0', role: 'button' }).text(action.label).on([DOM.EventType.CLICK, DOM.EventType.KEY_DOWN], function (e) {
                                if (e instanceof KeyboardEvent) {
                                    var event_2 = new keyboardEvent_1.StandardKeyboardEvent(e);
                                    if (!event_2.equals(keyCodes_1.CommonKeybindings.ENTER) && !event_2.equals(keyCodes_1.CommonKeybindings.SPACE)) {
                                        return; // Only handle Enter/Escape for keyboard access
                                    }
                                }
                                DOM.EventHelper.stop(e, true);
                                if (_this.usageLogger) {
                                    _this.usageLogger.publicLog('workbenchActionExecuted', { id: action.id, from: 'message' });
                                }
                                (action.run() || winjs_base_1.TPromise.as(null))
                                    .then(null, function (error) { return _this.showMessage(Severity.Error, error); })
                                    .done(function (r) {
                                    if (r === false) {
                                        return;
                                    }
                                    _this.hideMessage(message.text); // hide all matching the text since there may be duplicates
                                });
                            });
                        });
                    };
                    for (var i = messageActions.length - 1; i >= 0; i--) {
                        _loop_1(i);
                    }
                });
                // Text
                var text = message.text.substr(0, _this.options.maxMessageLength);
                li.div({ class: 'message-left-side' }, function (div) {
                    div.addClass('message-overflow-ellipsis');
                    // Severity indicator
                    var sev = message.severity;
                    var label = (sev === Severity.Error) ? nls.localize(3, null) : (sev === Severity.Warning) ? nls.localize(4, null) : nls.localize(5, null);
                    builder_1.$().span({ class: 'message-left-side severity ' + ((sev === Severity.Error) ? 'app-error' : (sev === Severity.Warning) ? 'app-warning' : 'app-info'), text: label }).appendTo(div);
                    // Error message
                    var messageContentElement = htmlRenderer.renderHtml({
                        tagName: 'span',
                        className: 'message-left-side',
                        formattedText: text
                    });
                    builder_1.$(messageContentElement).title(messageContentElement.textContent).appendTo(div);
                });
            });
        };
        MessageList.prototype.getMessageActions = function (message) {
            var _this = this;
            var messageActions;
            if (message.actions && message.actions.length > 0) {
                messageActions = message.actions;
            }
            else {
                messageActions = [
                    new actions_1.Action('close.message.action', nls.localize(6, null), null, true, function () {
                        _this.hideMessage(message.text); // hide all matching the text since there may be duplicates
                        return winjs_base_1.TPromise.as(true);
                    })
                ];
            }
            return messageActions;
        };
        MessageList.prototype.prepareMessages = function () {
            // Aggregate Messages by text to reduce their count
            var messages = [];
            var handledMessages = {};
            var offset = 0;
            for (var i = 0; i < this.messages.length; i++) {
                var message = this.messages[i];
                if (types.isUndefinedOrNull(handledMessages[message.text])) {
                    message.count = 1;
                    messages.push(message);
                    handledMessages[message.text] = offset++;
                }
                else {
                    messages[handledMessages[message.text]].count++;
                }
            }
            if (messages.length > this.options.maxMessages) {
                return messages.splice(messages.length - this.options.maxMessages, messages.length);
            }
            return messages;
        };
        MessageList.prototype.disposeMessages = function (messages) {
            messages.forEach(function (message) {
                if (message.actions) {
                    message.actions.forEach(function (action) {
                        action.dispose();
                    });
                }
            });
        };
        MessageList.prototype.hideMessages = function () {
            this.hideMessage();
        };
        MessageList.prototype.show = function () {
            if (this.messageListContainer && this.messageListContainer.isHidden()) {
                this.messageListContainer.show();
            }
        };
        MessageList.prototype.hide = function () {
            if (this.messageListContainer && !this.messageListContainer.isHidden()) {
                this.messageListContainer.hide();
            }
        };
        MessageList.prototype.hideMessage = function (messageObj) {
            var messageFound = false;
            for (var i = 0; i < this.messages.length; i++) {
                var message = this.messages[i];
                var hide = false;
                // Hide specific message
                if (messageObj) {
                    hide = ((types.isString(messageObj) && message.text === messageObj) || message.id === messageObj);
                }
                else {
                    hide = true;
                }
                if (hide) {
                    this.disposeMessages(this.messages.splice(i, 1));
                    i--;
                    messageFound = true;
                }
            }
            if (messageFound) {
                this.renderMessages(false, -1);
            }
        };
        MessageList.prototype.purgeMessages = function () {
            var _this = this;
            // Cancel previous
            if (this.messageListPurger) {
                this.messageListPurger.cancel();
            }
            // Configure
            this.messageListPurger = winjs_base_1.TPromise.timeout(this.options.purgeInterval).then(function () {
                var needsUpdate = false;
                var counter = 0;
                for (var i = 0; i < _this.messages.length; i++) {
                    var message = _this.messages[i];
                    // Only purge infos and warnings and only if they are not providing actions
                    if (message.severity !== Severity.Error && !message.actions) {
                        _this.disposeMessages(_this.messages.splice(i, 1));
                        counter--;
                        i--;
                        needsUpdate = true;
                    }
                }
                if (needsUpdate) {
                    _this.renderMessages(false, counter);
                }
            });
        };
        MessageList.DEFAULT_MESSAGE_PURGER_INTERVAL = 10000;
        MessageList.DEFAULT_MAX_MESSAGES = 5;
        MessageList.DEFAULT_MAX_MESSAGE_LENGTH = 500;
        return MessageList;
    }());
    exports.MessageList = MessageList;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/ui/progressbar/progressbar", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/assert', 'vs/base/browser/browser', 'vs/base/browser/builder', 'vs/base/browser/dom', 'vs/base/common/uuid', 'vs/base/common/lifecycle', 'vs/css!./progressbar'], function (require, exports, winjs_base_1, assert, browser, builder_1, DOM, uuid, lifecycle_1) {
    'use strict';
    var css_done = 'done';
    var css_active = 'active';
    var css_infinite = 'infinite';
    var css_discrete = 'discrete';
    var css_progress_container = 'progress-container';
    var css_progress_bit = 'progress-bit';
    /**
     * A progress bar with support for infinite or discrete progress.
     */
    var ProgressBar = (function () {
        function ProgressBar(builder) {
            this.toUnbind = [];
            this.workedVal = 0;
            this.create(builder);
        }
        ProgressBar.prototype.create = function (parent) {
            var _this = this;
            parent.div({ 'class': css_progress_container }, function (builder) {
                _this.element = builder.clone();
                builder.div({ 'class': css_progress_bit }).on([DOM.EventType.ANIMATION_START, DOM.EventType.ANIMATION_END, DOM.EventType.ANIMATION_ITERATION], function (e) {
                    switch (e.type) {
                        case DOM.EventType.ANIMATION_START:
                        case DOM.EventType.ANIMATION_END:
                            _this.animationRunning = e.type === DOM.EventType.ANIMATION_START;
                            break;
                        case DOM.EventType.ANIMATION_ITERATION:
                            if (_this.animationStopToken) {
                                _this.animationStopToken(null);
                            }
                            break;
                    }
                }, _this.toUnbind);
                _this.bit = builder.getHTMLElement();
            });
        };
        ProgressBar.prototype.off = function () {
            this.bit.style.width = 'inherit';
            this.bit.style.opacity = '1';
            this.element.removeClass(css_active);
            this.element.removeClass(css_infinite);
            this.element.removeClass(css_discrete);
            this.workedVal = 0;
            this.totalWork = undefined;
        };
        /**
         * Indicates to the progress bar that all work is done.
         */
        ProgressBar.prototype.done = function () {
            return this.doDone(true);
        };
        /**
         * Stops the progressbar from showing any progress instantly without fading out.
         */
        ProgressBar.prototype.stop = function () {
            return this.doDone(false);
        };
        ProgressBar.prototype.doDone = function (delayed) {
            var _this = this;
            this.element.addClass(css_done);
            // let it grow to 100% width and hide afterwards
            if (!this.element.hasClass(css_infinite)) {
                this.bit.style.width = 'inherit';
                if (delayed) {
                    winjs_base_1.TPromise.timeout(200).then(function () { return _this.off(); });
                }
                else {
                    this.off();
                }
            }
            else {
                this.bit.style.opacity = '0';
                if (delayed) {
                    winjs_base_1.TPromise.timeout(200).then(function () { return _this.off(); });
                }
                else {
                    this.off();
                }
            }
            return this;
        };
        /**
         * Use this mode to indicate progress that has no total number of work units.
         */
        ProgressBar.prototype.infinite = function () {
            this.bit.style.width = '2%';
            this.bit.style.opacity = '1';
            this.element.removeClass(css_discrete);
            this.element.removeClass(css_done);
            this.element.addClass(css_active);
            this.element.addClass(css_infinite);
            if (!browser.hasCSSAnimationSupport()) {
                // Use a generated token to avoid race conditions from reentrant calls to this function
                var currentProgressToken = uuid.v4().asHex();
                this.currentProgressToken = currentProgressToken;
                this.manualInfinite(currentProgressToken);
            }
            return this;
        };
        ProgressBar.prototype.manualInfinite = function (currentProgressToken) {
            var _this = this;
            this.bit.style.width = '5%';
            this.bit.style.display = 'inherit';
            var counter = 0;
            var animationFn = function () {
                winjs_base_1.TPromise.timeout(50).then(function () {
                    // Return if another manualInfinite() call was made
                    if (currentProgressToken !== _this.currentProgressToken) {
                        return;
                    }
                    else if (_this.element.hasClass(css_done)) {
                        _this.bit.style.display = 'none';
                        _this.bit.style.left = '0';
                    }
                    else if (_this.element.isHidden()) {
                        animationFn();
                    }
                    else {
                        counter = (counter + 1) % 95;
                        _this.bit.style.left = counter + '%';
                        animationFn();
                    }
                });
            };
            // Start Animation
            animationFn();
        };
        /**
         * Tells the progress bar the total number of work. Use in combination with workedVal() to let
         * the progress bar show the actual progress based on the work that is done.
         */
        ProgressBar.prototype.total = function (value) {
            this.workedVal = 0;
            this.totalWork = value;
            return this;
        };
        /**
         * Finds out if this progress bar is configured with total work
         */
        ProgressBar.prototype.hasTotal = function () {
            return !isNaN(this.totalWork);
        };
        /**
         * Tells the progress bar that an amount of work has been completed.
         */
        ProgressBar.prototype.worked = function (value) {
            assert.ok(!isNaN(this.totalWork), 'Total work not set');
            value = Number(value);
            assert.ok(!isNaN(value), 'Value is not a number');
            value = Math.max(1, value);
            this.workedVal += value;
            this.workedVal = Math.min(this.totalWork, this.workedVal);
            if (this.element.hasClass(css_infinite)) {
                this.element.removeClass(css_infinite);
            }
            if (this.element.hasClass(css_done)) {
                this.element.removeClass(css_done);
            }
            if (!this.element.hasClass(css_active)) {
                this.element.addClass(css_active);
            }
            if (!this.element.hasClass(css_discrete)) {
                this.element.addClass(css_discrete);
            }
            this.bit.style.width = 100 * (this.workedVal / this.totalWork) + '%';
            return this;
        };
        /**
         * Returns the builder this progress bar is building in.
         */
        ProgressBar.prototype.getContainer = function () {
            return builder_1.$(this.element);
        };
        ProgressBar.prototype.dispose = function () {
            this.toUnbind = lifecycle_1.disposeAll(this.toUnbind);
        };
        return ProgressBar;
    }());
    exports.ProgressBar = ProgressBar;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/ui/resourceviewer/resourceViewer", ["require", "exports", 'vs/nls!vs/base/browser/ui/resourceviewer/resourceViewer', 'vs/base/common/mime', 'vs/base/common/paths', 'vs/base/browser/builder', 'vs/base/browser/dom', 'vs/css!./resourceviewer'], function (require, exports, nls, mimes, paths, builder_1, DOM) {
    'use strict';
    // Known media mimes that we can handle
    var mapExtToMediaMimes = {
        '.bmp': 'image/bmp',
        '.gif': 'image/gif',
        '.jpg': 'image/jpg',
        '.jpeg': 'image/jpg',
        '.jpe': 'image/jpg',
        '.png': 'image/png',
        '.tiff': 'image/tiff',
        '.tif': 'image/tiff',
        '.ico': 'image/x-icon',
        '.tga': 'image/x-tga',
        '.psd': 'image/vnd.adobe.photoshop',
        '.mid': 'audio/midi',
        '.midi': 'audio/midi',
        '.mp4a': 'audio/mp4',
        '.mpga': 'audio/mpeg',
        '.mp2': 'audio/mpeg',
        '.mp2a': 'audio/mpeg',
        '.mp3': 'audio/mpeg',
        '.m2a': 'audio/mpeg',
        '.m3a': 'audio/mpeg',
        '.oga': 'audio/ogg',
        '.ogg': 'audio/ogg',
        '.spx': 'audio/ogg',
        '.aac': 'audio/x-aac',
        '.wav': 'audio/x-wav',
        '.wma': 'audio/x-ms-wma',
        '.mp4': 'video/mp4',
        '.mp4v': 'video/mp4',
        '.mpg4': 'video/mp4',
        '.mpeg': 'video/mpeg',
        '.mpg': 'video/mpeg',
        '.mpe': 'video/mpeg',
        '.m1v': 'video/mpeg',
        '.m2v': 'video/mpeg',
        '.ogv': 'video/ogg',
        '.qt': 'video/quicktime',
        '.mov': 'video/quicktime',
        '.webm': 'video/webm',
        '.mkv': 'video/x-matroska',
        '.mk3d': 'video/x-matroska',
        '.mks': 'video/x-matroska',
        '.wmv': 'video/x-ms-wmv',
        '.flv': 'video/x-flv',
        '.avi': 'video/x-msvideo',
        '.movie': 'video/x-sgi-movie'
    };
    /**
     * Helper to actually render the given resource into the provided container. Will adjust scrollbar (if provided) automatically based on loading
     * progress of the binary resource.
     */
    var ResourceViewer = (function () {
        function ResourceViewer() {
        }
        ResourceViewer.show = function (name, resource, container, scrollbar) {
            // Ensure CSS class
            builder_1.$(container).addClass('monaco-resource-viewer');
            // Lookup media mime if any
            var mime;
            var ext = paths.extname(resource.toString());
            if (ext) {
                mime = mapExtToMediaMimes[ext.toLowerCase()];
            }
            if (!mime) {
                mime = mimes.MIME_BINARY;
            }
            // Show Image inline
            if (mime.indexOf('image/') >= 0) {
                builder_1.$(container)
                    .empty()
                    .style({ paddingLeft: '20px' }) // restore CSS value in case the user saw a PDF before where we remove padding
                    .img({
                    src: resource.toString() + '?' + new Date().getTime() // We really want to avoid the browser from caching this resource, so we add a fake query param that is unique
                }).on(DOM.EventType.LOAD, function () {
                    if (scrollbar) {
                        scrollbar.onElementInternalDimensions();
                    }
                });
            }
            else if (false /* PDF is currently not supported in Electron it seems */ && mime.indexOf('pdf') >= 0) {
                builder_1.$(container)
                    .empty()
                    .style({ padding: 0, margin: 0 }) // We really do not want any paddings or margins when displaying PDFs
                    .element('object')
                    .attr({
                    data: resource.toString() + '?' + new Date().getTime(),
                    width: '100%',
                    height: '100%',
                    type: mime
                });
            }
            else if (mime.indexOf('audio/') >= 0) {
                builder_1.$(container)
                    .empty()
                    .style({ paddingLeft: '20px' }) // restore CSS value in case the user saw a PDF before where we remove padding
                    .element('audio')
                    .attr({
                    src: resource.toString() + '?' + new Date().getTime(),
                    text: nls.localize(0, null),
                    controls: 'controls'
                }).on(DOM.EventType.LOAD, function () {
                    if (scrollbar) {
                        scrollbar.onElementInternalDimensions();
                    }
                });
            }
            else if (mime.indexOf('video/') >= 0) {
                builder_1.$(container)
                    .empty()
                    .style({ paddingLeft: '20px' }) // restore CSS value in case the user saw a PDF before where we remove padding
                    .element('video')
                    .attr({
                    src: resource.toString() + '?' + new Date().getTime(),
                    text: nls.localize(1, null),
                    controls: 'controls'
                }).on(DOM.EventType.LOAD, function () {
                    if (scrollbar) {
                        scrollbar.onElementInternalDimensions();
                    }
                });
            }
            else {
                builder_1.$(container)
                    .empty()
                    .style({ paddingLeft: '20px' }) // restore CSS value in case the user saw a PDF before where we remove padding
                    .span({
                    text: nls.localize(2, null)
                });
                if (scrollbar) {
                    scrollbar.onElementInternalDimensions();
                }
            }
        };
        return ResourceViewer;
    }());
    exports.ResourceViewer = ResourceViewer;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/sash/sash", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/browser/builder', 'vs/base/browser/browser', 'vs/base/common/types', 'vs/base/browser/dom', 'vs/base/browser/touch', 'vs/base/common/eventEmitter', 'vs/base/browser/mouseEvent', 'vs/css!./sash'], function (require, exports, lifecycle_1, builder_1, browser_1, types, DOM, touch_1, eventEmitter_1, mouseEvent_1) {
    'use strict';
    (function (Orientation) {
        Orientation[Orientation["VERTICAL"] = 0] = "VERTICAL";
        Orientation[Orientation["HORIZONTAL"] = 1] = "HORIZONTAL";
    })(exports.Orientation || (exports.Orientation = {}));
    var Orientation = exports.Orientation;
    var Sash = (function (_super) {
        __extends(Sash, _super);
        function Sash(container, layoutProvider, options) {
            var _this = this;
            if (options === void 0) { options = {}; }
            _super.call(this);
            this.$e = builder_1.$('.monaco-sash').appendTo(container);
            this.gesture = new touch_1.Gesture(this.$e.getHTMLElement());
            this.$e.on('mousedown', function (e) { _this.onMouseDown(e); });
            this.$e.on(touch_1.EventType.Start, function (e) { _this.onTouchStart(e); });
            this.orientation = options.orientation || Orientation.VERTICAL;
            this.$e.addClass(this.orientation === Orientation.HORIZONTAL ? 'horizontal' : 'vertical');
            this.size = options.baseSize || 5;
            if (browser_1.isIPad) {
                this.size *= 4; // see also http://ux.stackexchange.com/questions/39023/what-is-the-optimum-button-size-of-touch-screen-applications
                this.$e.addClass('touch');
            }
            if (this.orientation === Orientation.HORIZONTAL) {
                this.$e.size(null, this.size);
            }
            else {
                this.$e.size(this.size);
            }
            this.isDisabled = false;
            this.hidden = false;
            this.layoutProvider = layoutProvider;
        }
        Sash.prototype.getHTMLElement = function () {
            return this.$e.getHTMLElement();
        };
        Sash.prototype.onMouseDown = function (e) {
            var _this = this;
            DOM.EventHelper.stop(e, false);
            if (this.isDisabled) {
                return;
            }
            var mouseDownEvent = new mouseEvent_1.StandardMouseEvent(e);
            var startX = mouseDownEvent.posx;
            var startY = mouseDownEvent.posy;
            var startEvent = {
                startX: startX,
                currentX: startX,
                startY: startY,
                currentY: startY
            };
            this.$e.addClass('active');
            this.emit('start', startEvent);
            var overlayDiv = builder_1.$('div').style({
                position: 'absolute',
                top: 0,
                left: 0,
                width: '100%',
                height: '100%',
                zIndex: 1000000,
                cursor: this.orientation === Orientation.VERTICAL ? 'ew-resize' : 'ns-resize'
            });
            var $window = builder_1.$(window);
            var lastCurrentX = startX;
            var lastCurrentY = startY;
            $window.on('mousemove', function (e) {
                DOM.EventHelper.stop(e, false);
                var mouseMoveEvent = new mouseEvent_1.StandardMouseEvent(e);
                var event = {
                    startX: startX,
                    currentX: mouseMoveEvent.posx,
                    startY: startY,
                    currentY: mouseMoveEvent.posy
                };
                lastCurrentX = mouseMoveEvent.posx;
                lastCurrentY = mouseMoveEvent.posy;
                _this.emit('change', event);
            }).once('mouseup', function (e) {
                DOM.EventHelper.stop(e, false);
                _this.$e.removeClass('active');
                _this.emit('end');
                $window.off('mousemove');
                overlayDiv.destroy();
            });
            overlayDiv.appendTo(document.body);
        };
        Sash.prototype.onTouchStart = function (event) {
            var _this = this;
            DOM.EventHelper.stop(event);
            var listeners = [];
            var startX = event.pageX;
            var startY = event.pageY;
            this.emit('start', {
                startX: startX,
                currentX: startX,
                startY: startY,
                currentY: startY
            });
            var lastCurrentX = startX;
            var lastCurrentY = startY;
            listeners.push(DOM.addDisposableListener(this.$e.getHTMLElement(), touch_1.EventType.Change, function (event) {
                if (types.isNumber(event.pageX) && types.isNumber(event.pageY)) {
                    _this.emit('change', {
                        startX: startX,
                        currentX: event.pageX,
                        startY: startY,
                        currentY: event.pageY
                    });
                    lastCurrentX = event.pageX;
                    lastCurrentY = event.pageY;
                }
            }));
            listeners.push(DOM.addDisposableListener(this.$e.getHTMLElement(), touch_1.EventType.End, function (event) {
                _this.emit('end');
                lifecycle_1.disposeAll(listeners);
            }));
        };
        Sash.prototype.layout = function () {
            var style;
            if (this.orientation === Orientation.VERTICAL) {
                var verticalProvider = this.layoutProvider;
                style = { left: verticalProvider.getVerticalSashLeft(this) - (this.size / 2) + 'px' };
                if (verticalProvider.getVerticalSashTop) {
                    style.top = verticalProvider.getVerticalSashTop(this) + 'px';
                }
                if (verticalProvider.getVerticalSashHeight) {
                    style.height = verticalProvider.getVerticalSashHeight(this) + 'px';
                }
            }
            else {
                var horizontalProvider = this.layoutProvider;
                style = { top: horizontalProvider.getHorizontalSashTop(this) - (this.size / 2) + 'px' };
                if (horizontalProvider.getHorizontalSashLeft) {
                    style.left = horizontalProvider.getHorizontalSashLeft(this) + 'px';
                }
                if (horizontalProvider.getHorizontalSashWidth) {
                    style.width = horizontalProvider.getHorizontalSashWidth(this) + 'px';
                }
            }
            this.$e.style(style);
        };
        Sash.prototype.show = function () {
            this.hidden = false;
            this.$e.show();
        };
        Sash.prototype.hide = function () {
            this.hidden = true;
            this.$e.hide();
        };
        Sash.prototype.isHidden = function () {
            return this.hidden;
        };
        Sash.prototype.enable = function () {
            this.$e.removeClass('disabled');
            this.isDisabled = false;
        };
        Sash.prototype.disable = function () {
            this.$e.addClass('disabled');
            this.isDisabled = true;
        };
        Sash.prototype.dispose = function () {
            if (this.$e) {
                this.$e.destroy();
                this.$e = null;
            }
            _super.prototype.dispose.call(this);
        };
        return Sash;
    }(eventEmitter_1.EventEmitter));
    exports.Sash = Sash;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/scrollbar/domNodeScrollable", ["require", "exports", 'vs/base/browser/dom', 'vs/base/browser/touch', 'vs/base/common/lifecycle', 'vs/base/common/event'], function (require, exports, DomUtils, touch_1, lifecycle_1, event_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DomNodeScrollable = (function (_super) {
        __extends(DomNodeScrollable, _super);
        function DomNodeScrollable(domNode) {
            var _this = this;
            _super.call(this);
            this._onScroll = this._register(new event_1.Emitter());
            this._domNode = domNode;
            this._gestureHandler = this._register(new touch_1.Gesture(this._domNode));
            this._register(DomUtils.addDisposableListener(this._domNode, 'scroll', function (e) { return _this._onScroll.fire(void 0); }));
        }
        DomNodeScrollable.prototype.dispose = function () {
            this._domNode = null;
            _super.prototype.dispose.call(this);
        };
        DomNodeScrollable.prototype.getScrollHeight = function () {
            return this._domNode.scrollHeight;
        };
        DomNodeScrollable.prototype.getScrollWidth = function () {
            return this._domNode.scrollWidth;
        };
        DomNodeScrollable.prototype.getScrollLeft = function () {
            return this._domNode.scrollLeft;
        };
        DomNodeScrollable.prototype.setScrollLeft = function (scrollLeft) {
            this._domNode.scrollLeft = scrollLeft;
        };
        DomNodeScrollable.prototype.getScrollTop = function () {
            return this._domNode.scrollTop;
        };
        DomNodeScrollable.prototype.setScrollTop = function (scrollTop) {
            this._domNode.scrollTop = scrollTop;
        };
        DomNodeScrollable.prototype.addScrollListener = function (callback) {
            return this._onScroll.event(callback);
        };
        return DomNodeScrollable;
    }(lifecycle_1.Disposable));
    exports.DomNodeScrollable = DomNodeScrollable;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/splitview/splitview", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/common/eventEmitter', 'vs/base/common/types', 'vs/base/common/objects', 'vs/base/browser/dom', 'vs/base/common/numbers', 'vs/base/browser/ui/sash/sash', 'vs/base/browser/keyboardEvent', 'vs/base/common/keyCodes', 'vs/base/common/event', 'vs/css!./splitview'], function (require, exports, lifecycle, ee, types, objects, dom, numbers, sash, keyboardEvent_1, keyCodes_1, event_1) {
    'use strict';
    (function (Orientation) {
        Orientation[Orientation["VERTICAL"] = 0] = "VERTICAL";
        Orientation[Orientation["HORIZONTAL"] = 1] = "HORIZONTAL";
    })(exports.Orientation || (exports.Orientation = {}));
    var Orientation = exports.Orientation;
    (function (ViewSizing) {
        ViewSizing[ViewSizing["Flexible"] = 0] = "Flexible";
        ViewSizing[ViewSizing["Fixed"] = 1] = "Fixed";
    })(exports.ViewSizing || (exports.ViewSizing = {}));
    var ViewSizing = exports.ViewSizing;
    var View = (function (_super) {
        __extends(View, _super);
        function View(opts) {
            _super.call(this);
            this.size = 0;
            this._sizing = types.isUndefined(opts.sizing) ? ViewSizing.Flexible : opts.sizing;
            this._fixedSize = types.isUndefined(opts.fixedSize) ? 22 : opts.fixedSize;
            this._minimumSize = types.isUndefined(opts.minimumSize) ? 22 : opts.minimumSize;
        }
        Object.defineProperty(View.prototype, "sizing", {
            get: function () { return this._sizing; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(View.prototype, "fixedSize", {
            get: function () { return this._fixedSize; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(View.prototype, "minimumSize", {
            get: function () { return this.sizing === ViewSizing.Fixed ? this.fixedSize : this._minimumSize; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(View.prototype, "maximumSize", {
            get: function () { return this.sizing === ViewSizing.Fixed ? this.fixedSize : Number.POSITIVE_INFINITY; },
            enumerable: true,
            configurable: true
        });
        // protected?
        View.prototype.setFlexible = function (size) {
            this._sizing = ViewSizing.Flexible;
            this.emit('change', types.isUndefined(size) ? this._minimumSize : size);
        };
        // protected?
        View.prototype.setFixed = function (size) {
            this._sizing = ViewSizing.Fixed;
            this._fixedSize = types.isUndefined(size) ? this._fixedSize : size;
            this.emit('change', this._fixedSize);
        };
        View.prototype.render = function (container, orientation) {
            // to implement
        };
        View.prototype.focus = function () {
            // to implement
        };
        View.prototype.layout = function (size, orientation) {
            // to optionally implement
        };
        return View;
    }(ee.EventEmitter));
    exports.View = View;
    var HeaderView = (function (_super) {
        __extends(HeaderView, _super);
        function HeaderView(opts) {
            _super.call(this, opts);
            this.headerSize = types.isUndefined(opts.headerSize) ? 22 : opts.headerSize;
        }
        HeaderView.prototype.render = function (container, orientation) {
            this.header = document.createElement('div');
            this.header.className = 'header';
            var headerSize = this.headerSize + 'px';
            if (orientation === Orientation.HORIZONTAL) {
                this.header.style.width = headerSize;
            }
            else {
                this.header.style.height = headerSize;
            }
            this.renderHeader(this.header);
            container.appendChild(this.header);
            this.body = document.createElement('div');
            this.body.className = 'body';
            this.layoutBodyContainer(orientation);
            this.renderBody(this.body);
            container.appendChild(this.body);
        };
        HeaderView.prototype.layout = function (size, orientation) {
            this.layoutBodyContainer(orientation);
            this.layoutBody(size - this.headerSize);
        };
        HeaderView.prototype.renderHeader = function (container) {
            throw new Error('not implemented');
        };
        HeaderView.prototype.renderBody = function (container) {
            throw new Error('not implemented');
        };
        HeaderView.prototype.layoutBodyContainer = function (orientation) {
            var size = "calc(100% - " + this.headerSize + "px)";
            if (orientation === Orientation.HORIZONTAL) {
                this.body.style.width = size;
            }
            else {
                this.body.style.height = size;
            }
        };
        HeaderView.prototype.layoutBody = function (size) {
            // to optionally implement
        };
        HeaderView.prototype.dispose = function () {
            this.header = null;
            this.body = null;
            _super.prototype.dispose.call(this);
        };
        return HeaderView;
    }(View));
    exports.HeaderView = HeaderView;
    (function (CollapsibleState) {
        CollapsibleState[CollapsibleState["EXPANDED"] = 0] = "EXPANDED";
        CollapsibleState[CollapsibleState["COLLAPSED"] = 1] = "COLLAPSED";
    })(exports.CollapsibleState || (exports.CollapsibleState = {}));
    var CollapsibleState = exports.CollapsibleState;
    var AbstractCollapsibleView = (function (_super) {
        __extends(AbstractCollapsibleView, _super);
        function AbstractCollapsibleView(opts) {
            _super.call(this, opts);
            this.ariaHeaderLabel = opts && opts.ariaHeaderLabel;
            this.changeState(types.isUndefined(opts.initialState) ? CollapsibleState.EXPANDED : opts.initialState);
        }
        AbstractCollapsibleView.prototype.render = function (container, orientation) {
            var _this = this;
            _super.prototype.render.call(this, container, orientation);
            dom.addClass(this.header, 'collapsible');
            dom.addClass(this.body, 'collapsible');
            // Keyboard access
            this.header.setAttribute('tabindex', '0');
            this.header.setAttribute('role', 'toolbar');
            if (this.ariaHeaderLabel) {
                this.header.setAttribute('aria-label', this.ariaHeaderLabel);
            }
            this.header.setAttribute('aria-expanded', String(this.state === CollapsibleState.EXPANDED));
            this.headerKeyListener = dom.addDisposableListener(this.header, dom.EventType.KEY_DOWN, function (e) {
                var event = new keyboardEvent_1.StandardKeyboardEvent(e);
                var eventHandled = false;
                if (event.equals(keyCodes_1.CommonKeybindings.ENTER) || event.equals(keyCodes_1.CommonKeybindings.SPACE) || (event.equals(keyCodes_1.CommonKeybindings.LEFT_ARROW) && _this.state === CollapsibleState.EXPANDED) || (event.equals(keyCodes_1.CommonKeybindings.RIGHT_ARROW) && _this.state === CollapsibleState.COLLAPSED)) {
                    _this.toggleExpansion();
                    eventHandled = true;
                }
                else if (event.equals(keyCodes_1.CommonKeybindings.ESCAPE)) {
                    _this.header.blur();
                    eventHandled = true;
                }
                else if (event.equals(keyCodes_1.CommonKeybindings.UP_ARROW)) {
                    _this.emit('focusPrevious');
                    eventHandled = true;
                }
                else if (event.equals(keyCodes_1.CommonKeybindings.DOWN_ARROW)) {
                    _this.emit('focusNext');
                    eventHandled = true;
                }
                if (eventHandled) {
                    dom.EventHelper.stop(event, true);
                }
            });
            // Mouse access
            this.headerClickListener = dom.addDisposableListener(this.header, dom.EventType.CLICK, function () { return _this.toggleExpansion(); });
            // Track state of focus in header so that other components can adjust styles based on that
            // (for example show or hide actions based on the state of being focused or not)
            this.focusTracker = dom.trackFocus(this.header);
            this.focusTracker.addFocusListener(function () {
                dom.addClass(_this.header, 'focused');
            });
            this.focusTracker.addBlurListener(function () {
                dom.removeClass(_this.header, 'focused');
            });
        };
        AbstractCollapsibleView.prototype.focus = function () {
            if (this.header) {
                this.header.focus();
            }
        };
        AbstractCollapsibleView.prototype.layout = function (size, orientation) {
            this.layoutHeader();
            _super.prototype.layout.call(this, size, orientation);
        };
        AbstractCollapsibleView.prototype.isExpanded = function () {
            return this.state === CollapsibleState.EXPANDED;
        };
        AbstractCollapsibleView.prototype.expand = function () {
            if (this.isExpanded()) {
                return;
            }
            this.changeState(CollapsibleState.EXPANDED);
        };
        AbstractCollapsibleView.prototype.collapse = function () {
            if (!this.isExpanded()) {
                return;
            }
            this.changeState(CollapsibleState.COLLAPSED);
        };
        AbstractCollapsibleView.prototype.toggleExpansion = function () {
            if (this.isExpanded()) {
                this.collapse();
            }
            else {
                this.expand();
            }
        };
        AbstractCollapsibleView.prototype.layoutHeader = function () {
            if (!this.header) {
                return;
            }
            if (this.state === CollapsibleState.COLLAPSED) {
                dom.addClass(this.header, 'collapsed');
            }
            else {
                dom.removeClass(this.header, 'collapsed');
            }
        };
        AbstractCollapsibleView.prototype.changeState = function (state) {
            this.state = state;
            if (this.header) {
                this.header.setAttribute('aria-expanded', String(this.state === CollapsibleState.EXPANDED));
            }
            this.layoutHeader();
        };
        AbstractCollapsibleView.prototype.dispose = function () {
            if (this.headerClickListener) {
                this.headerClickListener.dispose();
                this.headerClickListener = null;
            }
            if (this.headerKeyListener) {
                this.headerKeyListener.dispose();
                this.headerKeyListener = null;
            }
            if (this.focusTracker) {
                this.focusTracker.dispose();
                this.focusTracker = null;
            }
            _super.prototype.dispose.call(this);
        };
        return AbstractCollapsibleView;
    }(HeaderView));
    exports.AbstractCollapsibleView = AbstractCollapsibleView;
    var CollapsibleView = (function (_super) {
        __extends(CollapsibleView, _super);
        function CollapsibleView(opts) {
            _super.call(this, opts);
            this.previousSize = null;
        }
        CollapsibleView.prototype.changeState = function (state) {
            _super.prototype.changeState.call(this, state);
            if (state === CollapsibleState.EXPANDED) {
                this.setFlexible(this.previousSize || this._minimumSize);
            }
            else {
                this.previousSize = this.size;
                this.setFixed();
            }
        };
        return CollapsibleView;
    }(AbstractCollapsibleView));
    exports.CollapsibleView = CollapsibleView;
    var FixedCollapsibleView = (function (_super) {
        __extends(FixedCollapsibleView, _super);
        function FixedCollapsibleView(opts) {
            _super.call(this, objects.mixin({ sizing: ViewSizing.Fixed }, opts));
            this._expandedBodySize = types.isUndefined(opts.expandedBodySize) ? 22 : opts.expandedBodySize;
        }
        Object.defineProperty(FixedCollapsibleView.prototype, "fixedSize", {
            get: function () { return this.state === CollapsibleState.EXPANDED ? this.expandedSize : this.headerSize; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FixedCollapsibleView.prototype, "expandedSize", {
            get: function () { return this.expandedBodySize + this.headerSize; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FixedCollapsibleView.prototype, "expandedBodySize", {
            get: function () { return this._expandedBodySize; },
            set: function (size) {
                this._expandedBodySize = size;
                this.setFixed(this.fixedSize);
            },
            enumerable: true,
            configurable: true
        });
        FixedCollapsibleView.prototype.changeState = function (state) {
            _super.prototype.changeState.call(this, state);
            this.setFixed(this.fixedSize);
        };
        return FixedCollapsibleView;
    }(AbstractCollapsibleView));
    exports.FixedCollapsibleView = FixedCollapsibleView;
    var DeadView = (function (_super) {
        __extends(DeadView, _super);
        function DeadView(view) {
            _super.call(this, { sizing: ViewSizing.Fixed, fixedSize: 0 });
            this.size = view.size;
        }
        return DeadView;
    }(View));
    function sum(a, b) { return a + b; }
    var SplitView = (function () {
        function SplitView(container, options) {
            options = options || {};
            this.orientation = types.isUndefined(options.orientation) ? Orientation.VERTICAL : options.orientation;
            this.el = document.createElement('div');
            dom.addClass(this.el, 'monaco-split-view');
            dom.addClass(this.el, this.orientation === Orientation.VERTICAL ? 'vertical' : 'horizontal');
            container.appendChild(this.el);
            this.size = null;
            this.viewElements = [];
            this.views = [];
            this.viewChangeListeners = [];
            this.viewFocusPreviousListeners = [];
            this.viewFocusNextListeners = [];
            this.viewFocusListeners = [];
            this.initialWeights = [];
            this.sashes = [];
            this.sashesListeners = [];
            this.animationTimeout = null;
            this._onFocus = new event_1.Emitter();
            this.sashOrientation = this.orientation === Orientation.VERTICAL
                ? sash.Orientation.HORIZONTAL
                : sash.Orientation.VERTICAL;
            if (this.orientation === Orientation.VERTICAL) {
                this.measureContainerSize = function () { return dom.getContentHeight(container); };
                this.layoutViewElement = function (viewElement, size) { return viewElement.style.height = size + 'px'; };
                this.eventWrapper = function (e) { return { start: e.startY, current: e.currentY }; };
            }
            else {
                this.measureContainerSize = function () { return dom.getContentWidth(container); };
                this.layoutViewElement = function (viewElement, size) { return viewElement.style.width = size + 'px'; };
                this.eventWrapper = function (e) { return { start: e.startX, current: e.currentX }; };
            }
            // The void space exists to handle the case where all other views are fixed size
            this.addView(new View({
                sizing: ViewSizing.Fixed,
                minimumSize: 0,
                fixedSize: 0
            }), 1, 0);
        }
        Object.defineProperty(SplitView.prototype, "onFocus", {
            get: function () {
                return this._onFocus.event;
            },
            enumerable: true,
            configurable: true
        });
        SplitView.prototype.addView = function (view, initialWeight, index) {
            var _this = this;
            if (initialWeight === void 0) { initialWeight = 1; }
            if (index === void 0) { index = this.views.length - 1; }
            if (initialWeight <= 0) {
                throw new Error('Initial weight must be a positive number.');
            }
            var viewCount = this.views.length;
            // Create view container
            var viewElement = document.createElement('div');
            dom.addClass(viewElement, 'split-view-view');
            this.viewElements.splice(index, 0, viewElement);
            // Create view
            view.render(viewElement, this.orientation);
            this.views.splice(index, 0, view);
            // Initial weight
            this.initialWeights.splice(index, 0, initialWeight);
            // Render view
            if (index === viewCount) {
                this.el.appendChild(viewElement);
            }
            else {
                this.el.insertBefore(viewElement, this.el.children.item(index));
            }
            // Add sash
            if (this.views.length > 2) {
                var s_1 = new sash.Sash(this.el, this, { orientation: this.sashOrientation });
                this.sashes.splice(index - 1, 0, s_1);
                this.sashesListeners.push(s_1.addListener2('start', function (e) { return _this.onSashStart(s_1, _this.eventWrapper(e)); }));
                this.sashesListeners.push(s_1.addListener2('change', function (e) { return _this.onSashChange(s_1, _this.eventWrapper(e)); }));
            }
            this.viewChangeListeners.splice(index, 0, view.addListener2('change', function (size) { return _this.onViewChange(view, size); }));
            this.onViewChange(view, view.minimumSize);
            var viewFocusTracker = dom.trackFocus(viewElement);
            this.viewFocusListeners.splice(index, 0, viewFocusTracker);
            viewFocusTracker.addFocusListener(function () { return _this._onFocus.fire(view); });
            this.viewFocusPreviousListeners.splice(index, 0, view.addListener2('focusPrevious', function () { return index > 0 && _this.views[index - 1].focus(); }));
            this.viewFocusNextListeners.splice(index, 0, view.addListener2('focusNext', function () { return index < _this.views.length && _this.views[index + 1].focus(); }));
        };
        SplitView.prototype.removeView = function (view) {
            var index = this.views.indexOf(view);
            if (index < 0) {
                return;
            }
            var deadView = new DeadView(view);
            this.views[index] = deadView;
            this.onViewChange(deadView, 0);
            var sashIndex = Math.max(index - 1, 0);
            this.sashes[sashIndex].dispose();
            this.sashes.splice(sashIndex, 1);
            this.viewChangeListeners[index].dispose();
            this.viewChangeListeners.splice(index, 1);
            this.viewFocusPreviousListeners[index].dispose();
            this.viewFocusPreviousListeners.splice(index, 1);
            this.viewFocusListeners[index].dispose();
            this.viewFocusListeners.splice(index, 1);
            this.viewFocusNextListeners[index].dispose();
            this.viewFocusNextListeners.splice(index, 1);
            this.views.splice(index, 1);
            this.el.removeChild(this.viewElements[index]);
            this.viewElements.splice(index, 1);
            deadView.dispose();
            view.dispose();
        };
        SplitView.prototype.layout = function (size) {
            size = size || this.measureContainerSize();
            if (this.size === null) {
                this.size = size;
                this.initialLayout();
                return;
            }
            size = Math.max(size, this.views.reduce(function (t, v) { return t + v.minimumSize; }, 0));
            var diff = Math.abs(this.size - size);
            var up = numbers.countToArray(this.views.length - 1, -1);
            var collapses = this.views.map(function (v) { return v.size - v.minimumSize; });
            var expands = this.views.map(function (v) { return v.maximumSize - v.size; });
            if (size < this.size) {
                this.expandCollapse(Math.min(diff, collapses.reduce(sum, 0)), collapses, expands, up, []);
            }
            else if (size > this.size) {
                this.expandCollapse(Math.min(diff, expands.reduce(sum, 0)), collapses, expands, [], up);
            }
            this.size = size;
            this.layoutViews();
        };
        SplitView.prototype.onSashStart = function (sash, event) {
            var i = this.sashes.indexOf(sash);
            var collapses = this.views.map(function (v) { return v.size - v.minimumSize; });
            var expands = this.views.map(function (v) { return v.maximumSize - v.size; });
            var up = numbers.countToArray(i, -1);
            var down = numbers.countToArray(i + 1, this.views.length);
            var collapsesUp = up.map(function (i) { return collapses[i]; });
            var collapsesDown = down.map(function (i) { return collapses[i]; });
            var expandsUp = up.map(function (i) { return expands[i]; });
            var expandsDown = down.map(function (i) { return expands[i]; });
            this.state = {
                start: event.start,
                sizes: this.views.map(function (v) { return v.size; }),
                up: up,
                down: down,
                maxUp: Math.min(collapsesUp.reduce(sum, 0), expandsDown.reduce(sum, 0)),
                maxDown: Math.min(expandsUp.reduce(sum, 0), collapsesDown.reduce(sum, 0)),
                collapses: collapses,
                expands: expands
            };
        };
        SplitView.prototype.onSashChange = function (sash, event) {
            var diff = event.current - this.state.start;
            for (var i = 0; i < this.views.length; i++) {
                this.views[i].size = this.state.sizes[i];
            }
            if (diff < 0) {
                this.expandCollapse(Math.min(-diff, this.state.maxUp), this.state.collapses, this.state.expands, this.state.up, this.state.down);
            }
            else {
                this.expandCollapse(Math.min(diff, this.state.maxDown), this.state.collapses, this.state.expands, this.state.down, this.state.up);
            }
            this.layoutViews();
        };
        // Main algorithm
        SplitView.prototype.expandCollapse = function (collapse, collapses, expands, collapseIndexes, expandIndexes) {
            var _this = this;
            var totalCollapse = collapse;
            var totalExpand = totalCollapse;
            collapseIndexes.forEach(function (i) {
                var collapse = Math.min(collapses[i], totalCollapse);
                totalCollapse -= collapse;
                _this.views[i].size -= collapse;
            });
            expandIndexes.forEach(function (i) {
                var expand = Math.min(expands[i], totalExpand);
                totalExpand -= expand;
                _this.views[i].size += expand;
            });
        };
        SplitView.prototype.initialLayout = function () {
            var _this = this;
            var totalWeight = 0;
            var fixedSize = 0;
            this.views.forEach(function (v, i) {
                if (v.sizing === ViewSizing.Flexible) {
                    totalWeight += _this.initialWeights[i];
                }
                else {
                    fixedSize += v.fixedSize;
                }
            });
            var flexibleSize = this.size - fixedSize;
            this.views.forEach(function (v, i) {
                if (v.sizing === ViewSizing.Flexible) {
                    v.size = _this.initialWeights[i] * flexibleSize / totalWeight;
                }
                else {
                    v.size = v.fixedSize;
                }
            });
            // Leftover
            var index = this.getLastFlexibleViewIndex();
            if (index >= 0) {
                this.views[index].size += this.size - this.views.reduce(function (t, v) { return t + v.size; }, 0);
            }
            // Layout
            this.layoutViews();
        };
        SplitView.prototype.getLastFlexibleViewIndex = function (exceptIndex) {
            if (exceptIndex === void 0) { exceptIndex = null; }
            for (var i = this.views.length - 1; i >= 0; i--) {
                if (exceptIndex === i) {
                    continue;
                }
                if (this.views[i].sizing === ViewSizing.Flexible) {
                    return i;
                }
            }
            return -1;
        };
        SplitView.prototype.layoutViews = function () {
            for (var i = 0; i < this.views.length; i++) {
                // Layout the view elements
                this.layoutViewElement(this.viewElements[i], this.views[i].size);
                // Layout the views themselves
                this.views[i].layout(this.views[i].size, this.orientation);
            }
            // Layout the sashes
            this.sashes.forEach(function (s) { return s.layout(); });
            // Update sashes enablement
            var previous = false;
            var collapsesDown = this.views.map(function (v) { return previous = (v.size - v.minimumSize > 0) || previous; });
            previous = false;
            var expandsDown = this.views.map(function (v) { return previous = (v.maximumSize - v.size > 0) || previous; });
            var reverseViews = this.views.slice().reverse();
            previous = false;
            var collapsesUp = reverseViews.map(function (v) { return previous = (v.size - v.minimumSize > 0) || previous; }).reverse();
            previous = false;
            var expandsUp = reverseViews.map(function (v) { return previous = (v.maximumSize - v.size > 0) || previous; }).reverse();
            this.sashes.forEach(function (s, i) {
                if ((collapsesDown[i] && expandsUp[i + 1]) || (expandsDown[i] && collapsesUp[i + 1])) {
                    s.enable();
                }
                else {
                    s.disable();
                }
            });
        };
        SplitView.prototype.onViewChange = function (view, size) {
            if (view !== this.voidView) {
                if (this.areAllViewsFixed()) {
                    this.voidView.setFlexible();
                }
                else {
                    this.voidView.setFixed();
                }
            }
            if (this.size === null) {
                return;
            }
            if (size === view.size) {
                return;
            }
            this.setupAnimation();
            var index = this.views.indexOf(view);
            var diff = Math.abs(size - view.size);
            var up = numbers.countToArray(index - 1, -1);
            var down = numbers.countToArray(index + 1, this.views.length);
            var downUp = down.concat(up);
            var collapses = this.views.map(function (v) { return Math.max(v.size - v.minimumSize, 0); });
            var expands = this.views.map(function (v) { return Math.max(v.maximumSize - v.size, 0); });
            var collapse, collapseIndexes, expandIndexes;
            if (size < view.size) {
                collapse = Math.min(downUp.reduce(function (t, i) { return t + expands[i]; }, 0), diff);
                collapseIndexes = [index];
                expandIndexes = downUp;
            }
            else {
                collapse = Math.min(downUp.reduce(function (t, i) { return t + collapses[i]; }, 0), diff);
                collapseIndexes = downUp;
                expandIndexes = [index];
            }
            this.expandCollapse(collapse, collapses, expands, collapseIndexes, expandIndexes);
            this.layoutViews();
        };
        SplitView.prototype.setupAnimation = function () {
            var _this = this;
            if (types.isNumber(this.animationTimeout)) {
                window.clearTimeout(this.animationTimeout);
            }
            dom.addClass(this.el, 'animated');
            this.animationTimeout = window.setTimeout(function () { return _this.clearAnimation(); }, 200);
        };
        SplitView.prototype.clearAnimation = function () {
            this.animationTimeout = null;
            dom.removeClass(this.el, 'animated');
        };
        Object.defineProperty(SplitView.prototype, "voidView", {
            get: function () {
                return this.views[this.views.length - 1];
            },
            enumerable: true,
            configurable: true
        });
        SplitView.prototype.areAllViewsFixed = function () {
            var _this = this;
            return this.views.every(function (v, i) { return v.sizing === ViewSizing.Fixed || i === _this.views.length - 1; });
        };
        SplitView.prototype.getVerticalSashLeft = function (sash) {
            return this.getSashPosition(sash);
        };
        SplitView.prototype.getHorizontalSashTop = function (sash) {
            return this.getSashPosition(sash);
        };
        SplitView.prototype.getSashPosition = function (sash) {
            var index = this.sashes.indexOf(sash);
            var position = 0;
            for (var i = 0; i <= index; i++) {
                position += this.views[i].size;
            }
            return position;
        };
        SplitView.prototype.dispose = function () {
            var _this = this;
            if (types.isNumber(this.animationTimeout)) {
                window.clearTimeout(this.animationTimeout);
            }
            this.orientation = null;
            this.size = null;
            this.viewElements.forEach(function (e) { return _this.el.removeChild(e); });
            this.el = null;
            this.viewElements = [];
            this.views = lifecycle.disposeAll(this.views);
            this.sashes = lifecycle.disposeAll(this.sashes);
            this.sashesListeners = lifecycle.disposeAll(this.sashesListeners);
            this.measureContainerSize = null;
            this.layoutViewElement = null;
            this.eventWrapper = null;
            this.state = null;
        };
        return SplitView;
    }());
    exports.SplitView = SplitView;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/ui/timer/timer", ["require", "exports", 'vs/base/common/timer', 'vs/base/common/lifecycle', 'vs/base/browser/dom', 'vs/css!./timer'], function (require, exports, timer_1, lifecycle_1, DomUtils) {
    'use strict';
    var TimeKeeperRenderer = (function () {
        function TimeKeeperRenderer(onHide) {
            var _this = this;
            this.timeKeeper = timer_1.getTimeKeeper();
            this.onHide = onHide;
            this.lastEventIndex = 0;
            this.renderedEvents = {};
            this.renderCnt = 0;
            this.listenersToRemove = [];
            this.domNode = this._createDomNode();
            this.intervalTokenId = window.setInterval(function () { return _this._render(); }, 500);
        }
        TimeKeeperRenderer.prototype.destroy = function () {
            document.body.removeChild(this.outerDomNode);
            window.clearInterval(this.intervalTokenId);
            this.listenersToRemove = lifecycle_1.disposeAll(this.listenersToRemove);
        };
        TimeKeeperRenderer.prototype._createDomNode = function () {
            var _this = this;
            this.outerDomNode = document.createElement('div');
            this.outerDomNode.className = 'benchmarktimerbox';
            // Clear
            var cancel = document.createElement('input');
            cancel.type = 'button';
            cancel.value = 'Clear';
            this.listenersToRemove.push(DomUtils.addDisposableListener(cancel, 'click', function () { return _this._onClear(); }));
            this.outerDomNode.appendChild(cancel);
            // Text filter
            this.textFilterDomNode = document.createElement('input');
            this.textFilterDomNode.type = 'text';
            this.textFilterDomNode.className = 'textFilter';
            this.listenersToRemove.push(DomUtils.addDisposableListener(this.textFilterDomNode, 'keydown', function () { return _this.onTextFilterChange(); }));
            this.textFilter = '';
            this.outerDomNode.appendChild(document.createTextNode('Filter'));
            this.outerDomNode.appendChild(this.textFilterDomNode);
            // Time filter
            this.timeFilterDomNode = document.createElement('input');
            this.timeFilterDomNode.type = 'text';
            this.timeFilterDomNode.value = '0';
            this.timeFilterDomNode.className = 'timeFilter';
            this.listenersToRemove.push(DomUtils.addDisposableListener(this.timeFilterDomNode, 'keydown', function () { return _this.onTimeFilterChange(); }));
            this.timeFilter = 0;
            this.outerDomNode.appendChild(document.createTextNode('Hide time under'));
            this.outerDomNode.appendChild(this.timeFilterDomNode);
            var hide = document.createElement('input');
            hide.type = 'button';
            hide.value = 'Close';
            this.listenersToRemove.push(DomUtils.addDisposableListener(hide, 'click', function () {
                _this.onHide();
            }));
            this.outerDomNode.appendChild(hide);
            var heading = document.createElement('pre');
            heading.appendChild(document.createTextNode(this.renderRow('TOPIC', 'NAME', 'TOOK', 'START', 'END')));
            this.outerDomNode.appendChild(heading);
            this.outerDomNode.appendChild(document.createElement('hr'));
            var domNode = document.createElement('div');
            domNode.className = 'inner';
            this.outerDomNode.appendChild(domNode);
            document.body.appendChild(this.outerDomNode);
            return domNode;
        };
        TimeKeeperRenderer.prototype.onTextFilterChange = function () {
            var _this = this;
            setTimeout(function () {
                _this.refilter();
            });
        };
        TimeKeeperRenderer.prototype.onTimeFilterChange = function () {
            var _this = this;
            setTimeout(function () {
                _this.refilter();
            });
        };
        TimeKeeperRenderer.prototype.matchesTextFilter = function (event) {
            if (!this.textFilter) {
                return true;
            }
            if (event.topic.toLowerCase().indexOf(this.textFilter.toLowerCase()) >= 0) {
                return true;
            }
            if (event.name.toLowerCase().indexOf(this.textFilter.toLowerCase()) >= 0) {
                return true;
            }
            return false;
        };
        TimeKeeperRenderer.prototype.matchesTimeFilter = function (event) {
            if (!this.timeFilter) {
                return true;
            }
            if (event.timeTaken() >= this.timeFilter) {
                return true;
            }
            return false;
        };
        TimeKeeperRenderer.prototype.shouldShow = function (event) {
            return this.matchesTextFilter(event) && this.matchesTimeFilter(event);
        };
        TimeKeeperRenderer.prototype.refilter = function () {
            this.textFilter = this.textFilterDomNode.value;
            this.timeFilter = parseInt(this.timeFilterDomNode.value, 10);
            var domNodes = Array.prototype.slice.call(this.domNode.children, 0);
            for (var i = 0; i < domNodes.length; i++) {
                var eventId = domNodes[i].getAttribute('data-event-id');
                var event_1 = this.renderedEvents[eventId];
                if (this.shouldShow(event_1)) {
                    domNodes[i].style.display = 'inherit';
                }
                else {
                    domNodes[i].style.display = 'none';
                }
            }
        };
        TimeKeeperRenderer.prototype._onClear = function () {
            this.lastEventIndex = this.timeKeeper.getCollectedEvents().length;
            this.renderedEvents = {};
            this.renderCnt = 0;
            DomUtils.clearNode(this.domNode);
        };
        TimeKeeperRenderer.prototype.leftPaddedString = function (size, padChar, str) {
            var spaces = this._repeatStr(padChar, Math.max(0, size - str.length));
            return spaces + str;
        };
        TimeKeeperRenderer.prototype.rightPaddedString = function (size, padChar, str) {
            var spaces = this._repeatStr(padChar, Math.max(0, size - str.length));
            return str + spaces;
        };
        TimeKeeperRenderer.prototype.renderRow = function (topic, name, timeTook, timeStart, timerEnd) {
            var result = ' ';
            result += this.rightPaddedString(10, ' ', topic);
            result += this.rightPaddedString(30, ' ', name);
            result += ' ' + this.leftPaddedString(15, ' ', timeTook);
            result += ' ' + this.leftPaddedString(13, ' ', timeStart);
            return result;
        };
        TimeKeeperRenderer.prototype._suffix0 = function (s) {
            if (s.charAt(s.length - 3) === '.') {
                return s;
            }
            if (s.charAt(s.length - 2) === '.') {
                return s + '0';
            }
            return s + '.00';
        };
        TimeKeeperRenderer.prototype._twoPrecision = function (a) {
            return this._suffix0(Math.round(a * 100) / 100 + '');
        };
        TimeKeeperRenderer.prototype._absoluteTime = function (t) {
            if (t < 1000) {
                return this._twoPrecision(t) + ' ms';
            }
            t /= 1000;
            if (t < 60) {
                return this._twoPrecision(t) + ' s';
            }
            t /= 60;
            if (t < 60) {
                return this._twoPrecision(t) + ' m';
            }
            t /= 60;
            return this._twoPrecision(t) + ' h';
        };
        TimeKeeperRenderer.prototype._renderEvent = function (domNode, event) {
            var start = event.startTime.getTime() - timer_1.TimeKeeper.PARSE_TIME.getTime();
            var result = this.renderRow(event.topic, event.name, this._twoPrecision(event.timeTaken()), this._absoluteTime(start) + '', this._absoluteTime(start + event.timeTaken()));
            domNode.textContent = '';
            domNode.appendChild(document.createTextNode(result));
        };
        TimeKeeperRenderer.prototype._renderStartTimerEvent = function (event) {
            var domNode = document.createElement('pre');
            this._renderEvent(domNode, event);
            this.domNode.appendChild(domNode);
            var idString = event.id.toString();
            domNode.setAttribute('data-event-id', idString);
            domNode.className = 'timer-event-' + (event.id % 2);
            this.renderedEvents[idString] = event;
            if (this.shouldShow(this.renderedEvents[idString])) {
                domNode.style.display = 'inherit';
            }
            else {
                domNode.style.display = 'none';
            }
            this.renderCnt++;
        };
        TimeKeeperRenderer.prototype._render = function () {
            var allEvents = this.timeKeeper.getCollectedEvents(), didSomething = false;
            for (var i = this.lastEventIndex; i < allEvents.length; i++) {
                var ev = allEvents[i];
                if (!ev.stopTime) {
                    // This event is not yet finished => block
                    this.lastEventIndex = i;
                    if (didSomething) {
                        this.domNode.scrollTop = 100000;
                    }
                    return;
                }
                this._renderStartTimerEvent(ev);
                didSomething = true;
            }
            if (didSomething) {
                this.domNode.scrollTop = 100000;
            }
            this.lastEventIndex = allEvents.length;
        };
        TimeKeeperRenderer.prototype._repeatStr = function (str, cnt) {
            var r = '';
            for (var i = 0; i < cnt; i++) {
                r += str;
            }
            return r;
        };
        return TimeKeeperRenderer;
    }());
    exports.TimeKeeperRenderer = TimeKeeperRenderer;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/toolbar/toolbar", ["require", "exports", 'vs/nls!vs/base/browser/ui/toolbar/toolbar', 'vs/base/common/winjs.base', 'vs/base/browser/builder', 'vs/base/common/types', 'vs/base/common/actions', 'vs/base/browser/ui/actionbar/actionbar', 'vs/base/browser/ui/dropdown/dropdown', 'vs/css!./toolbar'], function (require, exports, nls, winjs_base_1, builder_1, types, actions_1, actionbar_1, dropdown_1) {
    'use strict';
    exports.CONTEXT = 'context.toolbar';
    /**
     * A widget that combines an action bar for primary actions and a dropdown for secondary actions.
     */
    var ToolBar = (function () {
        function ToolBar(container, contextMenuProvider, options) {
            var _this = this;
            if (options === void 0) { options = { orientation: actionbar_1.ActionsOrientation.HORIZONTAL }; }
            this.options = options;
            this.toggleMenuAction = new ToggleMenuAction(function () { return _this.toggleMenuActionItem && _this.toggleMenuActionItem.show(); });
            var element = document.createElement('div');
            element.className = 'monaco-toolbar';
            container.appendChild(element);
            this.actionBar = new actionbar_1.ActionBar(builder_1.$(element), {
                orientation: options.orientation,
                ariaLabel: options.ariaLabel,
                actionItemProvider: function (action) {
                    // Return special action item for the toggle menu action
                    if (action.id === ToggleMenuAction.ID) {
                        // Dispose old
                        if (_this.toggleMenuActionItem) {
                            _this.toggleMenuActionItem.dispose();
                        }
                        // Create new
                        _this.toggleMenuActionItem = new DropdownMenuActionItem(action, action.menuActions, contextMenuProvider, _this.options.actionItemProvider, _this.actionRunner, 'toolbar-toggle-more');
                        return _this.toggleMenuActionItem;
                    }
                    return options.actionItemProvider ? options.actionItemProvider(action) : null;
                }
            });
        }
        Object.defineProperty(ToolBar.prototype, "actionRunner", {
            get: function () {
                return this.actionBar.actionRunner;
            },
            set: function (actionRunner) {
                this.actionBar.actionRunner = actionRunner;
            },
            enumerable: true,
            configurable: true
        });
        ToolBar.prototype.getContainer = function () {
            return this.actionBar.getContainer();
        };
        ToolBar.prototype.setAriaLabel = function (label) {
            this.actionBar.setAriaLabel(label);
        };
        ToolBar.prototype.setActions = function (primaryActions, secondaryActions) {
            var _this = this;
            return function () {
                var primaryActionsToSet = primaryActions ? primaryActions.slice(0) : [];
                // Inject additional action to open secondary actions if present
                _this.hasSecondaryActions = secondaryActions && secondaryActions.length > 0;
                if (_this.hasSecondaryActions) {
                    _this.toggleMenuAction.menuActions = secondaryActions.slice(0);
                    primaryActionsToSet.push(_this.toggleMenuAction);
                }
                _this.actionBar.clear();
                _this.actionBar.push(primaryActionsToSet, { icon: true, label: false });
            };
        };
        ToolBar.prototype.addPrimaryAction = function (primaryActions) {
            var _this = this;
            return function () {
                // Add after the "..." action if we have secondary actions
                if (_this.hasSecondaryActions) {
                    var itemCount = _this.actionBar.length();
                    _this.actionBar.push(primaryActions, { icon: true, label: false, index: itemCount });
                }
                else {
                    _this.actionBar.push(primaryActions, { icon: true, label: false });
                }
            };
        };
        ToolBar.prototype.dispose = function () {
            this.actionBar.dispose();
            this.toggleMenuAction.dispose();
            if (this.toggleMenuActionItem) {
                this.toggleMenuActionItem.dispose();
            }
        };
        return ToolBar;
    }());
    exports.ToolBar = ToolBar;
    var ToggleMenuAction = (function (_super) {
        __extends(ToggleMenuAction, _super);
        function ToggleMenuAction(toggleDropdownMenu) {
            _super.call(this, ToggleMenuAction.ID, nls.localize(0, null), null, true);
            this.toggleDropdownMenu = toggleDropdownMenu;
        }
        ToggleMenuAction.prototype.run = function () {
            this.toggleDropdownMenu();
            return winjs_base_1.TPromise.as(true);
        };
        Object.defineProperty(ToggleMenuAction.prototype, "menuActions", {
            get: function () {
                return this._menuActions;
            },
            set: function (actions) {
                this._menuActions = actions;
            },
            enumerable: true,
            configurable: true
        });
        ToggleMenuAction.ID = 'toolbar.toggle.more';
        return ToggleMenuAction;
    }(actions_1.Action));
    var DropdownMenuActionItem = (function (_super) {
        __extends(DropdownMenuActionItem, _super);
        function DropdownMenuActionItem(action, menuActionsOrProvider, contextMenuProvider, actionItemProvider, actionRunner, clazz) {
            _super.call(this, null, action);
            this.menuActionsOrProvider = menuActionsOrProvider;
            this.contextMenuProvider = contextMenuProvider;
            this.actionItemProvider = actionItemProvider;
            this.actionRunner = actionRunner;
            this.clazz = clazz;
        }
        DropdownMenuActionItem.prototype.render = function (container) {
            var _this = this;
            var labelRenderer = function (el) {
                _this.builder = builder_1.$('a.action-label').attr({
                    tabIndex: '0',
                    role: 'button',
                    'aria-haspopup': 'true',
                    title: _this._action.label || '',
                    class: _this.clazz
                });
                _this.builder.appendTo(el);
                return null;
            };
            var options = {
                contextMenuProvider: this.contextMenuProvider,
                labelRenderer: labelRenderer
            };
            // Render the DropdownMenu around a simple action to toggle it
            if (types.isArray(this.menuActionsOrProvider)) {
                options.actions = this.menuActionsOrProvider;
            }
            else {
                options.actionProvider = this.menuActionsOrProvider;
            }
            this.dropdownMenu = new dropdown_1.DropdownMenu(container, options);
            this.dropdownMenu.menuOptions = {
                actionItemProvider: this.actionItemProvider,
                actionRunner: this.actionRunner
            };
            // Reemit events for running actions
            this.toUnbind = this.addEmitter(this.dropdownMenu);
        };
        DropdownMenuActionItem.prototype.show = function () {
            if (this.dropdownMenu) {
                this.dropdownMenu.show();
            }
        };
        DropdownMenuActionItem.prototype.dispose = function () {
            this.toUnbind();
            this.dropdownMenu.dispose();
            _super.prototype.dispose.call(this);
        };
        return DropdownMenuActionItem;
    }(actionbar_1.BaseActionItem));
    exports.DropdownMenuActionItem = DropdownMenuActionItem;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/widget", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/browser/mouseEvent', 'vs/base/browser/keyboardEvent', 'vs/base/browser/dom'], function (require, exports, lifecycle_1, mouseEvent_1, keyboardEvent_1, DomUtils) {
    'use strict';
    var Widget = (function (_super) {
        __extends(Widget, _super);
        function Widget() {
            _super.apply(this, arguments);
        }
        Widget.prototype.onclick = function (domNode, listener) {
            this._register(DomUtils.addDisposableListener(domNode, DomUtils.EventType.CLICK, function (e) { return listener(new mouseEvent_1.StandardMouseEvent(e)); }));
        };
        Widget.prototype.onmousedown = function (domNode, listener) {
            this._register(DomUtils.addDisposableListener(domNode, DomUtils.EventType.MOUSE_DOWN, function (e) { return listener(new mouseEvent_1.StandardMouseEvent(e)); }));
        };
        Widget.prototype.onmouseover = function (domNode, listener) {
            this._register(DomUtils.addDisposableListener(domNode, DomUtils.EventType.MOUSE_OVER, function (e) { return listener(new mouseEvent_1.StandardMouseEvent(e)); }));
        };
        Widget.prototype.onnonbubblingmouseout = function (domNode, listener) {
            this._register(DomUtils.addDisposableNonBubblingMouseOutListener(domNode, function (e) { return listener(new mouseEvent_1.StandardMouseEvent(e)); }));
        };
        Widget.prototype.onkeydown = function (domNode, listener) {
            this._register(DomUtils.addDisposableListener(domNode, DomUtils.EventType.KEY_DOWN, function (e) { return listener(new keyboardEvent_1.StandardKeyboardEvent(e)); }));
        };
        Widget.prototype.onkeyup = function (domNode, listener) {
            this._register(DomUtils.addDisposableListener(domNode, DomUtils.EventType.KEY_UP, function (e) { return listener(new keyboardEvent_1.StandardKeyboardEvent(e)); }));
        };
        Widget.prototype.oninput = function (domNode, listener) {
            this._register(DomUtils.addDisposableListener(domNode, DomUtils.EventType.INPUT, listener));
        };
        Widget.prototype.onblur = function (domNode, listener) {
            this._register(DomUtils.addDisposableListener(domNode, DomUtils.EventType.BLUR, listener));
        };
        Widget.prototype.onfocus = function (domNode, listener) {
            this._register(DomUtils.addDisposableListener(domNode, DomUtils.EventType.FOCUS, listener));
        };
        Widget.prototype.onchange = function (domNode, listener) {
            this._register(DomUtils.addDisposableListener(domNode, DomUtils.EventType.CHANGE, listener));
        };
        return Widget;
    }(lifecycle_1.Disposable));
    exports.Widget = Widget;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/checkbox/checkbox", ["require", "exports", 'vs/base/browser/dom', 'vs/base/common/keyCodes', 'vs/base/browser/ui/widget', 'vs/css!./checkbox'], function (require, exports, DOM, keyCodes_1, widget_1) {
    'use strict';
    var Checkbox = (function (_super) {
        __extends(Checkbox, _super);
        function Checkbox(opts) {
            var _this = this;
            _super.call(this);
            this._opts = opts;
            this._checked = this._opts.isChecked;
            this.domNode = document.createElement('div');
            this.domNode.title = this._opts.title;
            this.domNode.className = this._className();
            this.domNode.tabIndex = 0;
            this.domNode.setAttribute('role', 'checkbox');
            this.domNode.setAttribute('aria-checked', String(this._checked));
            this.domNode.setAttribute('aria-label', this._opts.title);
            this.onclick(this.domNode, function (ev) {
                _this.checked = !_this._checked;
                _this._opts.onChange(false);
                ev.preventDefault();
            });
            this.onkeydown(this.domNode, function (keyboardEvent) {
                if (keyboardEvent.keyCode === keyCodes_1.KeyCode.Space || keyboardEvent.keyCode === keyCodes_1.KeyCode.Enter) {
                    _this.checked = !_this._checked;
                    _this._opts.onChange(true);
                    keyboardEvent.preventDefault();
                    return;
                }
                if (_this._opts.onKeyDown) {
                    _this._opts.onKeyDown(keyboardEvent);
                }
            });
        }
        Checkbox.prototype.focus = function () {
            this.domNode.focus();
        };
        Object.defineProperty(Checkbox.prototype, "checked", {
            get: function () {
                return this._checked;
            },
            set: function (newIsChecked) {
                this._checked = newIsChecked;
                this.domNode.setAttribute('aria-checked', String(this._checked));
                this.domNode.className = this._className();
            },
            enumerable: true,
            configurable: true
        });
        Checkbox.prototype._className = function () {
            return 'custom-checkbox ' + this._opts.actionClassName + ' ' + (this._checked ? 'checked' : 'unchecked');
        };
        Checkbox.prototype.width = function () {
            return 2 /*marginleft*/ + 2 /*border*/ + 2 /*padding*/ + 16 /* icon width */;
        };
        Checkbox.prototype.enable = function () {
            this.domNode.tabIndex = 0;
            this.domNode.setAttribute('aria-disabled', String(false));
        };
        Checkbox.prototype.disable = function () {
            DOM.removeTabIndexAndUpdateFocus(this.domNode);
            this.domNode.setAttribute('aria-disabled', String(true));
        };
        return Checkbox;
    }(widget_1.Widget));
    exports.Checkbox = Checkbox;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/inputbox/inputBox", ["require", "exports", 'vs/nls!vs/base/browser/ui/inputbox/inputBox', 'vs/base/browser/browser', 'vs/base/browser/dom', 'vs/base/browser/browserService', 'vs/base/browser/htmlContentRenderer', 'vs/base/browser/ui/aria/aria', 'vs/base/browser/ui/actionbar/actionbar', 'vs/base/browser/ui/contextview/contextview', 'vs/base/common/event', 'vs/base/browser/ui/widget', 'vs/css!./inputBox'], function (require, exports, nls, Bal, dom, browser, htmlContentRenderer_1, aria, actionbar_1, contextview_1, event_1, widget_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var $ = dom.emmet;
    (function (MessageType) {
        MessageType[MessageType["INFO"] = 1] = "INFO";
        MessageType[MessageType["WARNING"] = 2] = "WARNING";
        MessageType[MessageType["ERROR"] = 3] = "ERROR";
    })(exports.MessageType || (exports.MessageType = {}));
    var MessageType = exports.MessageType;
    var InputBox = (function (_super) {
        __extends(InputBox, _super);
        function InputBox(container, contextViewProvider, options) {
            var _this = this;
            _super.call(this);
            this.state = 'idle';
            this._onDidChange = this._register(new event_1.Emitter());
            this.onDidChange = this._onDidChange.event;
            this._onDidHeightChange = this._register(new event_1.Emitter());
            this.onDidHeightChange = this._onDidHeightChange.event;
            this.contextViewProvider = contextViewProvider;
            this.options = options || Object.create(null);
            this.message = null;
            this.cachedHeight = null;
            this.placeholder = this.options.placeholder || '';
            this.ariaLabel = this.options.ariaLabel || '';
            if (this.options.validationOptions) {
                this.validation = this.options.validationOptions.validation;
                this.showValidationMessage = this.options.validationOptions.showMessage || false;
            }
            this.element = dom.append(container, $('.monaco-inputbox.idle'));
            var tagName = this.options.flexibleHeight ? 'textarea' : 'input';
            var wrapper = dom.append(this.element, $('.wrapper'));
            this.input = dom.append(wrapper, $(tagName + '.input'));
            this.input.setAttribute('autocorrect', 'off');
            this.input.setAttribute('autocapitalize', 'off');
            this.input.setAttribute('spellcheck', 'false');
            this.onfocus(this.input, function () { return dom.addClass(_this.element, 'synthetic-focus'); });
            this.onblur(this.input, function () { return dom.removeClass(_this.element, 'synthetic-focus'); });
            if (this.options.flexibleHeight) {
                this.mirror = dom.append(wrapper, $('div.mirror'));
            }
            else {
                this.input.type = this.options.type || 'text';
                this.input.setAttribute('wrap', 'off');
            }
            if (this.ariaLabel) {
                this.input.setAttribute('aria-label', this.ariaLabel);
            }
            if (this.placeholder) {
                this.input.setAttribute('placeholder', this.placeholder);
            }
            this.oninput(this.input, function () { return _this.onValueChange(); });
            this.onblur(this.input, function () { return _this.onBlur(); });
            this.onfocus(this.input, function () { return _this.onFocus(); });
            // Add placeholder shim for IE because IE decides to hide the placeholder on focus (we dont want that!)
            if (this.placeholder && Bal.isIE11orEarlier) {
                this.onclick(this.input, function (e) {
                    dom.EventHelper.stop(e, true);
                    _this.input.focus();
                });
                if (Bal.isIE9) {
                    this.onkeyup(this.input, function () { return _this.onValueChange(); });
                }
            }
            setTimeout(function () { return _this.layout(); }, 0);
            // Support actions
            if (this.options.actions) {
                this.actionbar = this._register(new actionbar_1.ActionBar(this.element));
                this.actionbar.push(this.options.actions, { icon: true, label: false });
            }
        }
        InputBox.prototype.onBlur = function () {
            this._hideMessage();
        };
        InputBox.prototype.onFocus = function () {
            this._showMessage();
        };
        InputBox.prototype.setPlaceHolder = function (placeHolder) {
            if (this.input) {
                this.input.setAttribute('placeholder', placeHolder);
            }
        };
        InputBox.prototype.setAriaLabel = function (label) {
            this.ariaLabel = label;
            if (this.input) {
                if (label) {
                    this.input.setAttribute('aria-label', this.ariaLabel);
                }
                else {
                    this.input.removeAttribute('aria-label');
                }
            }
        };
        InputBox.prototype.setContextViewProvider = function (contextViewProvider) {
            this.contextViewProvider = contextViewProvider;
        };
        Object.defineProperty(InputBox.prototype, "inputElement", {
            get: function () {
                return this.input;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(InputBox.prototype, "value", {
            get: function () {
                return this.input.value;
            },
            set: function (newValue) {
                if (this.input.value !== newValue) {
                    this.input.value = newValue;
                    this.onValueChange();
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(InputBox.prototype, "height", {
            get: function () {
                return this.cachedHeight === null ? dom.getTotalHeight(this.element) : this.cachedHeight;
            },
            enumerable: true,
            configurable: true
        });
        InputBox.prototype.focus = function () {
            this.input.focus();
        };
        InputBox.prototype.blur = function () {
            this.input.blur();
        };
        InputBox.prototype.hasFocus = function () {
            return browser.getService().document.activeElement === this.input;
        };
        InputBox.prototype.select = function (range) {
            if (range === void 0) { range = null; }
            this.input.select();
            if (range) {
                this.input.setSelectionRange(range.start, range.end);
            }
        };
        InputBox.prototype.enable = function () {
            this.input.removeAttribute('disabled');
        };
        InputBox.prototype.disable = function () {
            this.input.disabled = true;
            this._hideMessage();
        };
        InputBox.prototype.setEnabled = function (enabled) {
            if (enabled) {
                this.enable();
            }
            else {
                this.disable();
            }
        };
        Object.defineProperty(InputBox.prototype, "width", {
            get: function () {
                return dom.getTotalWidth(this.input);
            },
            set: function (width) {
                this.input.style.width = width + 'px';
            },
            enumerable: true,
            configurable: true
        });
        InputBox.prototype.showMessage = function (message, force) {
            this.message = message;
            dom.removeClass(this.element, 'idle');
            dom.removeClass(this.element, 'info');
            dom.removeClass(this.element, 'warning');
            dom.removeClass(this.element, 'error');
            dom.addClass(this.element, this.classForType(message.type));
            // ARIA Support
            var alertText;
            if (message.type === MessageType.ERROR) {
                alertText = nls.localize(0, null, message.content);
            }
            else if (message.type === MessageType.WARNING) {
                alertText = nls.localize(1, null, message.content);
            }
            else {
                alertText = nls.localize(2, null, message.content);
            }
            aria.alert(alertText);
            if (this.hasFocus() || force) {
                this._showMessage();
            }
        };
        InputBox.prototype.hideMessage = function () {
            this.message = null;
            dom.removeClass(this.element, 'info');
            dom.removeClass(this.element, 'warning');
            dom.removeClass(this.element, 'error');
            dom.addClass(this.element, 'idle');
            this._hideMessage();
        };
        InputBox.prototype.isInputValid = function () {
            return !!this.validation && !this.validation(this.value);
        };
        InputBox.prototype.validate = function () {
            var result = null;
            if (this.validation) {
                result = this.validation(this.value);
                if (!result) {
                    this.inputElement.removeAttribute('aria-invalid');
                    this.hideMessage();
                }
                else {
                    this.inputElement.setAttribute('aria-invalid', 'true');
                    this.showMessage(result);
                }
            }
            return !result;
        };
        InputBox.prototype.classForType = function (type) {
            switch (type) {
                case MessageType.INFO: return 'info';
                case MessageType.WARNING: return 'warning';
                default: return 'error';
            }
        };
        InputBox.prototype._showMessage = function () {
            var _this = this;
            if (!this.contextViewProvider || !this.message) {
                return;
            }
            var div;
            var layout = function () { return div.style.width = dom.getTotalWidth(_this.element) + 'px'; };
            this.state = 'open';
            this.contextViewProvider.showContextView({
                getAnchor: function () { return _this.element; },
                anchorAlignment: contextview_1.AnchorAlignment.RIGHT,
                render: function (container) {
                    div = dom.append(container, $('.monaco-inputbox-container'));
                    layout();
                    var renderOptions = {
                        tagName: 'span',
                        className: 'monaco-inputbox-message',
                    };
                    if (_this.message.formatContent) {
                        renderOptions.formattedText = _this.message.content;
                    }
                    else {
                        renderOptions.text = _this.message.content;
                    }
                    var spanElement = htmlContentRenderer_1.renderHtml(renderOptions);
                    dom.addClass(spanElement, _this.classForType(_this.message.type));
                    dom.append(div, spanElement);
                    return null;
                },
                layout: layout
            });
        };
        InputBox.prototype._hideMessage = function () {
            if (!this.contextViewProvider || this.state !== 'open') {
                return;
            }
            this.state = 'idle';
            this.contextViewProvider.hideContextView();
        };
        InputBox.prototype.onValueChange = function () {
            this._onDidChange.fire(this.value);
            this.validate();
            if (this.mirror) {
                var lastCharCode = this.value.charCodeAt(this.value.length - 1);
                var suffix = lastCharCode === 10 ? ' ' : '';
                this.mirror.textContent = this.value + suffix;
                this.layout();
            }
        };
        InputBox.prototype.layout = function () {
            if (!this.mirror) {
                return;
            }
            var previousHeight = this.cachedHeight;
            this.cachedHeight = dom.getTotalHeight(this.mirror);
            if (previousHeight !== this.cachedHeight) {
                this.input.style.height = this.cachedHeight + 'px';
                this._onDidHeightChange.fire(this.cachedHeight);
            }
        };
        InputBox.prototype.dispose = function () {
            this._hideMessage();
            this.element = null;
            this.input = null;
            this.contextViewProvider = null;
            this.message = null;
            this.placeholder = null;
            this.ariaLabel = null;
            this.validation = null;
            this.showValidationMessage = null;
            this.state = null;
            this.actionbar = null;
            _super.prototype.dispose.call(this);
        };
        return InputBox;
    }(widget_1.Widget));
    exports.InputBox = InputBox;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/findinput/findInput", ["require", "exports", 'vs/nls!vs/base/browser/ui/findinput/findInput', 'vs/base/browser/dom', 'vs/base/browser/ui/inputbox/inputBox', 'vs/base/browser/ui/checkbox/checkbox', 'vs/base/browser/ui/widget', 'vs/base/common/event', 'vs/base/common/keyCodes', 'vs/css!./findInput'], function (require, exports, nls, dom, inputBox_1, checkbox_1, widget_1, event_1, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var NLS_REGEX_CHECKBOX_LABEL = nls.localize(0, null);
    var NLS_WHOLE_WORD_CHECKBOX_LABEL = nls.localize(1, null);
    var NLS_CASE_SENSITIVE_CHECKBOX_LABEL = nls.localize(2, null);
    var NLS_DEFAULT_LABEL = nls.localize(3, null);
    var FindInput = (function (_super) {
        __extends(FindInput, _super);
        function FindInput(parent, contextViewProvider, options) {
            var _this = this;
            _super.call(this);
            this._onDidOptionChange = this._register(new event_1.Emitter());
            this.onDidOptionChange = this._onDidOptionChange.event;
            this._onKeyDown = this._register(new event_1.Emitter());
            this.onKeyDown = this._onKeyDown.event;
            this._onInput = this._register(new event_1.Emitter());
            this.onInput = this._onInput.event;
            this._onKeyUp = this._register(new event_1.Emitter());
            this.onKeyUp = this._onKeyUp.event;
            this._onCaseSensitiveKeyDown = this._register(new event_1.Emitter());
            this.onCaseSensitiveKeyDown = this._onCaseSensitiveKeyDown.event;
            this.contextViewProvider = contextViewProvider;
            this.width = options.width || 100;
            this.placeholder = options.placeholder || '';
            this.validation = options.validation;
            this.label = options.label || NLS_DEFAULT_LABEL;
            this.regex = null;
            this.wholeWords = null;
            this.caseSensitive = null;
            this.domNode = null;
            this.inputBox = null;
            this.buildDomNode(options.appendCaseSensitiveLabel || '', options.appendWholeWordsLabel || '', options.appendRegexLabel || '');
            if (Boolean(parent)) {
                parent.appendChild(this.domNode);
            }
            this.onkeydown(this.inputBox.inputElement, function (e) { return _this._onKeyDown.fire(e); });
            this.onkeyup(this.inputBox.inputElement, function (e) { return _this._onKeyUp.fire(e); });
            this.oninput(this.inputBox.inputElement, function (e) { return _this._onInput.fire(); });
        }
        FindInput.prototype.enable = function () {
            dom.removeClass(this.domNode, 'disabled');
            this.inputBox.enable();
            this.regex.enable();
            this.wholeWords.enable();
            this.caseSensitive.enable();
        };
        FindInput.prototype.disable = function () {
            dom.addClass(this.domNode, 'disabled');
            this.inputBox.disable();
            this.regex.disable();
            this.wholeWords.disable();
            this.caseSensitive.disable();
        };
        FindInput.prototype.setEnabled = function (enabled) {
            if (enabled) {
                this.enable();
            }
            else {
                this.disable();
            }
        };
        FindInput.prototype.clear = function () {
            this.clearValidation();
            this.setValue('');
            this.focus();
        };
        FindInput.prototype.setWidth = function (newWidth) {
            this.width = newWidth;
            this.domNode.style.width = this.width + 'px';
            this.contextViewProvider.layout();
            this.setInputWidth();
        };
        FindInput.prototype.getValue = function () {
            return this.inputBox.value;
        };
        FindInput.prototype.setValue = function (value) {
            if (this.inputBox.value !== value) {
                this.inputBox.value = value;
            }
        };
        FindInput.prototype.setMatchCountState = function (state) {
            this.matchCount.setState(state);
            this.setInputWidth();
        };
        FindInput.prototype.select = function () {
            this.inputBox.select();
        };
        FindInput.prototype.focus = function () {
            this.inputBox.focus();
        };
        FindInput.prototype.getCaseSensitive = function () {
            return this.caseSensitive.checked;
        };
        FindInput.prototype.setCaseSensitive = function (value) {
            this.caseSensitive.checked = value;
            this.setInputWidth();
        };
        FindInput.prototype.getWholeWords = function () {
            return this.wholeWords.checked;
        };
        FindInput.prototype.setWholeWords = function (value) {
            this.wholeWords.checked = value;
            this.setInputWidth();
        };
        FindInput.prototype.getRegex = function () {
            return this.regex.checked;
        };
        FindInput.prototype.setRegex = function (value) {
            this.regex.checked = value;
            this.setInputWidth();
        };
        FindInput.prototype.focusOnCaseSensitive = function () {
            this.caseSensitive.focus();
        };
        FindInput.prototype.setInputWidth = function () {
            var w = this.width - this.matchCount.width() - this.caseSensitive.width() - this.wholeWords.width() - this.regex.width();
            this.inputBox.width = w;
        };
        FindInput.prototype.buildDomNode = function (appendCaseSensitiveLabel, appendWholeWordsLabel, appendRegexLabel) {
            var _this = this;
            this.domNode = document.createElement('div');
            this.domNode.style.width = this.width + 'px';
            dom.addClass(this.domNode, 'monaco-findInput');
            this.inputBox = this._register(new inputBox_1.InputBox(this.domNode, this.contextViewProvider, {
                placeholder: this.placeholder || '',
                ariaLabel: this.label || '',
                validationOptions: {
                    validation: this.validation || null,
                    showMessage: true
                }
            }));
            this.regex = this._register(new checkbox_1.Checkbox({
                actionClassName: 'regex',
                title: NLS_REGEX_CHECKBOX_LABEL + appendRegexLabel,
                isChecked: false,
                onChange: function (viaKeyboard) {
                    _this._onDidOptionChange.fire(viaKeyboard);
                    if (!viaKeyboard) {
                        _this.inputBox.focus();
                    }
                    _this.setInputWidth();
                    _this.validate();
                }
            }));
            this.wholeWords = this._register(new checkbox_1.Checkbox({
                actionClassName: 'whole-word',
                title: NLS_WHOLE_WORD_CHECKBOX_LABEL + appendWholeWordsLabel,
                isChecked: false,
                onChange: function (viaKeyboard) {
                    _this._onDidOptionChange.fire(viaKeyboard);
                    if (!viaKeyboard) {
                        _this.inputBox.focus();
                    }
                    _this.setInputWidth();
                    _this.validate();
                }
            }));
            this.caseSensitive = this._register(new checkbox_1.Checkbox({
                actionClassName: 'case-sensitive',
                title: NLS_CASE_SENSITIVE_CHECKBOX_LABEL + appendCaseSensitiveLabel,
                isChecked: false,
                onChange: function (viaKeyboard) {
                    _this._onDidOptionChange.fire(viaKeyboard);
                    if (!viaKeyboard) {
                        _this.inputBox.focus();
                    }
                    _this.setInputWidth();
                    _this.validate();
                },
                onKeyDown: function (e) {
                    _this._onCaseSensitiveKeyDown.fire(e);
                }
            }));
            this.matchCount = this._register(new MatchCount({
                onClick: function (e) {
                    _this.inputBox.focus();
                    e.preventDefault();
                }
            }));
            // Arrow-Key support to navigate between options
            var indexes = [this.caseSensitive.domNode, this.wholeWords.domNode, this.regex.domNode];
            this.onkeydown(this.domNode, function (event) {
                if (event.equals(keyCodes_1.CommonKeybindings.LEFT_ARROW) || event.equals(keyCodes_1.CommonKeybindings.RIGHT_ARROW) || event.equals(keyCodes_1.CommonKeybindings.ESCAPE)) {
                    var index = indexes.indexOf(document.activeElement);
                    if (index >= 0) {
                        var newIndex = void 0;
                        if (event.equals(keyCodes_1.CommonKeybindings.RIGHT_ARROW)) {
                            newIndex = (index + 1) % indexes.length;
                        }
                        else if (event.equals(keyCodes_1.CommonKeybindings.LEFT_ARROW)) {
                            if (index === 0) {
                                newIndex = indexes.length - 1;
                            }
                            else {
                                newIndex = index - 1;
                            }
                        }
                        if (event.equals(keyCodes_1.CommonKeybindings.ESCAPE)) {
                            indexes[index].blur();
                        }
                        else if (newIndex >= 0) {
                            indexes[newIndex].focus();
                        }
                        dom.EventHelper.stop(event, true);
                    }
                }
            });
            this.setInputWidth();
            var controls = document.createElement('div');
            controls.className = 'controls';
            controls.appendChild(this.matchCount.domNode);
            controls.appendChild(this.caseSensitive.domNode);
            controls.appendChild(this.wholeWords.domNode);
            controls.appendChild(this.regex.domNode);
            this.domNode.appendChild(controls);
        };
        FindInput.prototype.validate = function () {
            this.inputBox.validate();
        };
        FindInput.prototype.showMessage = function (message) {
            this.inputBox.showMessage(message);
        };
        FindInput.prototype.clearMessage = function () {
            this.inputBox.hideMessage();
        };
        FindInput.prototype.clearValidation = function () {
            this.inputBox.hideMessage();
        };
        FindInput.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
        };
        FindInput.OPTION_CHANGE = 'optionChange';
        return FindInput;
    }(widget_1.Widget));
    exports.FindInput = FindInput;
    var MatchCount = (function (_super) {
        __extends(MatchCount, _super);
        function MatchCount(opts) {
            _super.call(this);
            this.domNode = document.createElement('div');
            this.domNode.className = 'matchCount';
            this.setState({
                isVisible: false,
                count: '0',
                title: ''
            });
            this.onclick(this.domNode, opts.onClick);
        }
        MatchCount.prototype.width = function () {
            return this.isVisible ? 30 : 0;
        };
        MatchCount.prototype.setState = function (state) {
            dom.clearNode(this.domNode);
            this.domNode.appendChild(document.createTextNode(state.count));
            this.domNode.title = state.title;
            this.isVisible = state.isVisible;
            if (this.isVisible) {
                this.domNode.style.display = 'block';
            }
            else {
                this.domNode.style.display = 'none';
            }
        };
        return MatchCount;
    }(widget_1.Widget));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/scrollbar/abstractScrollbar", ["require", "exports", 'vs/base/browser/browser', 'vs/base/common/platform', 'vs/base/browser/dom', 'vs/base/browser/mouseEvent', 'vs/base/browser/ui/scrollbar/scrollableElement', 'vs/base/common/lifecycle', 'vs/base/browser/globalMouseMoveMonitor', 'vs/base/browser/ui/widget', 'vs/base/common/async'], function (require, exports, Browser, Platform, DomUtils, mouseEvent_1, scrollableElement_1, lifecycle_1, globalMouseMoveMonitor_1, widget_1, async_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The arrow image size.
     */
    exports.ARROW_IMG_SIZE = 11;
    /**
     * The orthogonal distance to the slider at which dragging "resets". This implements "snapping"
     */
    var MOUSE_DRAG_RESET_DISTANCE = 140;
    /**
     * The minimal size of the slider (such that it can still be clickable) -- it is artificially enlarged.
     */
    var MINIMUM_SLIDER_SIZE = 20;
    var ScrollbarState = (function () {
        function ScrollbarState(arrowSize, scrollbarSize, oppositeScrollbarSize) {
            this._visibleSize = 0;
            this._scrollSize = 0;
            this._scrollPosition = 0;
            this._scrollbarSize = scrollbarSize;
            this._oppositeScrollbarSize = oppositeScrollbarSize;
            this._arrowSize = arrowSize;
            this._refreshComputedValues();
        }
        ScrollbarState.prototype.setVisibleSize = function (visibleSize) {
            if (this._visibleSize !== visibleSize) {
                this._visibleSize = visibleSize;
                this._refreshComputedValues();
                return true;
            }
            return false;
        };
        ScrollbarState.prototype.setScrollSize = function (scrollSize) {
            if (this._scrollSize !== scrollSize) {
                this._scrollSize = scrollSize;
                this._refreshComputedValues();
                return true;
            }
            return false;
        };
        ScrollbarState.prototype.setScrollPosition = function (scrollPosition) {
            if (this._scrollPosition !== scrollPosition) {
                this._scrollPosition = scrollPosition;
                this._refreshComputedValues();
                return true;
            }
            return false;
        };
        ScrollbarState.prototype._refreshComputedValues = function () {
            this._computedAvailableSize = Math.max(0, this._visibleSize - this._oppositeScrollbarSize);
            this._computedRepresentableSize = Math.max(0, this._computedAvailableSize - 2 * this._arrowSize);
            this._computedRatio = this._scrollSize > 0 ? (this._computedRepresentableSize / this._scrollSize) : 0;
            this._computedIsNeeded = (this._scrollSize > this._visibleSize);
            if (!this._computedIsNeeded) {
                this._computedSliderSize = this._computedRepresentableSize;
                this._computedSliderPosition = 0;
            }
            else {
                this._computedSliderSize = Math.floor(this._visibleSize * this._computedRatio);
                this._computedSliderPosition = Math.floor(this._scrollPosition * this._computedRatio);
                if (this._computedSliderSize < MINIMUM_SLIDER_SIZE) {
                    // We must artificially increase the size of the slider, since the slider would be too small otherwise
                    // The effort is to keep the slider centered around the original position, but we must take into
                    // account the cases when the slider is too close to the top or too close to the bottom
                    var sliderArtificialOffset = (MINIMUM_SLIDER_SIZE - this._computedSliderSize) / 2;
                    this._computedSliderSize = MINIMUM_SLIDER_SIZE;
                    this._computedSliderPosition -= sliderArtificialOffset;
                    if (this._computedSliderPosition + this._computedSliderSize > this._computedRepresentableSize) {
                        // Slider is too close to the bottom, so we glue it to the bottom
                        this._computedSliderPosition = this._computedRepresentableSize - this._computedSliderSize;
                    }
                    if (this._computedSliderPosition < 0) {
                        // Slider is too close to the top, so we glue it to the top
                        this._computedSliderPosition = 0;
                    }
                }
            }
        };
        ScrollbarState.prototype.getArrowSize = function () {
            return this._arrowSize;
        };
        ScrollbarState.prototype.getRectangleLargeSize = function () {
            return this._computedAvailableSize;
        };
        ScrollbarState.prototype.getRectangleSmallSize = function () {
            return this._scrollbarSize;
        };
        ScrollbarState.prototype.isNeeded = function () {
            return this._computedIsNeeded;
        };
        ScrollbarState.prototype.getSliderSize = function () {
            return this._computedSliderSize;
        };
        ScrollbarState.prototype.getSliderPosition = function () {
            return this._computedSliderPosition;
        };
        ScrollbarState.prototype.convertSliderPositionToScrollPosition = function (desiredSliderPosition) {
            return desiredSliderPosition / this._computedRatio;
        };
        ScrollbarState.prototype.validateScrollPosition = function (desiredScrollPosition) {
            desiredScrollPosition = Math.round(desiredScrollPosition);
            desiredScrollPosition = Math.max(desiredScrollPosition, 0);
            desiredScrollPosition = Math.min(desiredScrollPosition, this._scrollSize - this._visibleSize);
            return desiredScrollPosition;
        };
        return ScrollbarState;
    }());
    exports.ScrollbarState = ScrollbarState;
    var ScrollbarArrow = (function (_super) {
        __extends(ScrollbarArrow, _super);
        function ScrollbarArrow(className, top, left, bottom, right, bgWidth, bgHeight, mouseWheelEventFactory, parent) {
            var _this = this;
            _super.call(this);
            this._parent = parent;
            this._mouseWheelEventFactory = mouseWheelEventFactory;
            this.bgDomNode = document.createElement('div');
            this.bgDomNode.className = 'arrow-background';
            this.bgDomNode.style.position = 'absolute';
            setSize(this.bgDomNode, bgWidth, bgHeight);
            setPosition(this.bgDomNode, (top !== null ? 0 : null), (left !== null ? 0 : null), (bottom !== null ? 0 : null), (right !== null ? 0 : null));
            this.domNode = document.createElement('div');
            this.domNode.className = className;
            this.domNode.style.position = 'absolute';
            setSize(this.domNode, exports.ARROW_IMG_SIZE, exports.ARROW_IMG_SIZE);
            setPosition(this.domNode, top, left, bottom, right);
            this._mouseMoveMonitor = this._register(new globalMouseMoveMonitor_1.GlobalMouseMoveMonitor());
            this.onmousedown(this.bgDomNode, function (e) { return _this._arrowMouseDown(e); });
            this.onmousedown(this.domNode, function (e) { return _this._arrowMouseDown(e); });
            this._mousedownRepeatTimer = this._register(new async_1.IntervalTimer());
            this._mousedownScheduleRepeatTimer = this._register(new async_1.TimeoutTimer());
        }
        ScrollbarArrow.prototype._arrowMouseDown = function (e) {
            var _this = this;
            var repeater = function () {
                _this._parent.onMouseWheel(_this._mouseWheelEventFactory());
            };
            var scheduleRepeater = function () {
                _this._mousedownRepeatTimer.cancelAndSet(repeater, 1000 / 24);
            };
            repeater();
            this._mousedownRepeatTimer.cancel();
            this._mousedownScheduleRepeatTimer.cancelAndSet(scheduleRepeater, 200);
            this._mouseMoveMonitor.startMonitoring(globalMouseMoveMonitor_1.standardMouseMoveMerger, function (mouseMoveData) {
                /* Intentional empty */
            }, function () {
                _this._mousedownRepeatTimer.cancel();
                _this._mousedownScheduleRepeatTimer.cancel();
            });
            e.preventDefault();
        };
        return ScrollbarArrow;
    }(widget_1.Widget));
    var VisibilityController = (function (_super) {
        __extends(VisibilityController, _super);
        function VisibilityController(visibility, visibleClassName, invisibleClassName) {
            _super.call(this);
            this._visibility = visibility;
            this._visibleClassName = visibleClassName;
            this._invisibleClassName = invisibleClassName;
            this._domNode = null;
            this._isVisible = false;
            this._isNeeded = false;
            this._shouldBeVisible = false;
            this._revealTimer = this._register(new async_1.TimeoutTimer());
        }
        // ----------------- Hide / Reveal
        VisibilityController.prototype.applyVisibilitySetting = function (shouldBeVisible) {
            if (this._visibility === scrollableElement_1.Visibility.Hidden) {
                return false;
            }
            if (this._visibility === scrollableElement_1.Visibility.Visible) {
                return true;
            }
            return shouldBeVisible;
        };
        VisibilityController.prototype.setShouldBeVisible = function (rawShouldBeVisible) {
            var shouldBeVisible = this.applyVisibilitySetting(rawShouldBeVisible);
            if (this._shouldBeVisible !== shouldBeVisible) {
                this._shouldBeVisible = shouldBeVisible;
                this.ensureVisibility();
            }
        };
        VisibilityController.prototype.setIsNeeded = function (isNeeded) {
            if (this._isNeeded !== isNeeded) {
                this._isNeeded = isNeeded;
                this.ensureVisibility();
            }
        };
        VisibilityController.prototype.setDomNode = function (domNode) {
            this._domNode = domNode;
            this._domNode.className = this._invisibleClassName;
            // Now that the flags & the dom node are in a consistent state, ensure the Hidden/Visible configuration
            this.setShouldBeVisible(false);
        };
        VisibilityController.prototype.ensureVisibility = function () {
            if (!this._isNeeded) {
                // Nothing to be rendered
                this._hide(false);
                return;
            }
            if (this._shouldBeVisible) {
                this._reveal();
            }
            else {
                this._hide(true);
            }
        };
        VisibilityController.prototype._reveal = function () {
            var _this = this;
            if (this._isVisible) {
                return;
            }
            this._isVisible = true;
            // The CSS animation doesn't play otherwise
            this._revealTimer.setIfNotSet(function () {
                _this._domNode.className = _this._visibleClassName;
            }, 0);
        };
        VisibilityController.prototype._hide = function (withFadeAway) {
            this._revealTimer.cancel();
            if (!this._isVisible) {
                return;
            }
            this._isVisible = false;
            this._domNode.className = this._invisibleClassName + (withFadeAway ? ' fade' : '');
        };
        return VisibilityController;
    }(lifecycle_1.Disposable));
    var AbstractScrollbar = (function (_super) {
        __extends(AbstractScrollbar, _super);
        function AbstractScrollbar(forbidTranslate3dUse, parent, scrollbarState, visibility, extraScrollbarClassName) {
            _super.call(this);
            this._forbidTranslate3dUse = forbidTranslate3dUse;
            this._parent = parent;
            this._scrollbarState = scrollbarState;
            this._visibilityController = this._register(new VisibilityController(visibility, 'visible scrollbar ' + extraScrollbarClassName, 'invisible scrollbar ' + extraScrollbarClassName));
            this._mouseMoveMonitor = this._register(new globalMouseMoveMonitor_1.GlobalMouseMoveMonitor());
        }
        // ----------------- initialize & clean-up
        /**
         * Creates the container dom node for the scrollbar & hooks up the events
         */
        AbstractScrollbar.prototype._createDomNode = function () {
            var _this = this;
            this.domNode = document.createElement('div');
            if (!this._forbidTranslate3dUse && Browser.canUseTranslate3d) {
                // Put the worker reporter in its own layer
                this.domNode.style.transform = 'translate3d(0px, 0px, 0px)';
            }
            this._visibilityController.setDomNode(this.domNode);
            this.domNode.style.position = 'absolute';
            this.onmousedown(this.domNode, function (e) { return _this._domNodeMouseDown(e); });
        };
        /**
         * Creates the dom node for an arrow & adds it to the container
         */
        AbstractScrollbar.prototype._createArrow = function (className, top, left, bottom, right, bgWidth, bgHeight, mouseWheelEventFactory) {
            var arrow = this._register(new ScrollbarArrow(className, top, left, bottom, right, bgWidth, bgHeight, mouseWheelEventFactory, this._parent));
            this.domNode.appendChild(arrow.bgDomNode);
            this.domNode.appendChild(arrow.domNode);
        };
        /**
         * Creates the slider dom node, adds it to the container & hooks up the events
         */
        AbstractScrollbar.prototype._createSlider = function (top, left, width, height) {
            var _this = this;
            this.slider = document.createElement('div');
            this.slider.className = 'slider';
            this.slider.style.position = 'absolute';
            setPosition(this.slider, top, left, null, null);
            setSize(this.slider, width, height);
            this.domNode.appendChild(this.slider);
            this.onmousedown(this.slider, function (e) { return _this._sliderMouseDown(e); });
        };
        // ----------------- Update state
        AbstractScrollbar.prototype.onElementSize = function (visibleSize) {
            if (this._scrollbarState.setVisibleSize(visibleSize)) {
                this._renderDomNode(this._scrollbarState.getRectangleLargeSize(), this._scrollbarState.getRectangleSmallSize());
                this._renderSlider();
                this._visibilityController.setIsNeeded(this._scrollbarState.isNeeded());
            }
        };
        AbstractScrollbar.prototype.onElementScrollSize = function (elementScrollSize) {
            if (this._scrollbarState.setScrollSize(elementScrollSize)) {
                this._renderSlider();
                this._visibilityController.setIsNeeded(this._scrollbarState.isNeeded());
            }
        };
        AbstractScrollbar.prototype.onElementScrollPosition = function (elementScrollPosition) {
            if (this._scrollbarState.setScrollPosition(elementScrollPosition)) {
                this._renderSlider();
                this._visibilityController.setIsNeeded(this._scrollbarState.isNeeded());
            }
        };
        // ----------------- rendering
        AbstractScrollbar.prototype.beginReveal = function () {
            this._visibilityController.setShouldBeVisible(true);
        };
        AbstractScrollbar.prototype.beginHide = function () {
            this._visibilityController.setShouldBeVisible(false);
        };
        AbstractScrollbar.prototype._renderSlider = function () {
            this._updateSlider(this._scrollbarState.getSliderSize(), this._scrollbarState.getArrowSize() + this._scrollbarState.getSliderPosition());
        };
        // ----------------- DOM events
        AbstractScrollbar.prototype._domNodeMouseDown = function (e) {
            if (e.target !== this.domNode) {
                return;
            }
            this._onMouseDown(e);
        };
        AbstractScrollbar.prototype.delegateMouseDown = function (browserEvent) {
            var e = new mouseEvent_1.StandardMouseEvent(browserEvent);
            var domTop = this.domNode.getClientRects()[0].top;
            var sliderStart = domTop + this._scrollbarState.getSliderPosition();
            var sliderStop = domTop + this._scrollbarState.getSliderPosition() + this._scrollbarState.getSliderSize();
            var mousePos = this._sliderMousePosition(e);
            if (sliderStart <= mousePos && mousePos <= sliderStop) {
                // Act as if it was a mouse down on the slider
                this._sliderMouseDown(e);
            }
            else {
                // Act as if it was a mouse down on the scrollbar
                this._onMouseDown(e);
            }
        };
        AbstractScrollbar.prototype._onMouseDown = function (e) {
            var domNodePosition = DomUtils.getDomNodePosition(this.domNode);
            var desiredSliderPosition = this._mouseDownRelativePosition(e, domNodePosition) - this._scrollbarState.getArrowSize() - this._scrollbarState.getSliderSize() / 2;
            this.setDesiredScrollPosition(this._scrollbarState.convertSliderPositionToScrollPosition(desiredSliderPosition));
            this._sliderMouseDown(e);
        };
        AbstractScrollbar.prototype._sliderMouseDown = function (e) {
            var _this = this;
            if (e.leftButton) {
                var initialMouseOrthogonalPosition_1 = this._sliderOrthogonalMousePosition(e);
                var initialScrollPosition_1 = this._getScrollPosition();
                var draggingDelta_1 = this._sliderMousePosition(e) - this._scrollbarState.getSliderPosition();
                DomUtils.toggleClass(this.slider, 'active', true);
                this._mouseMoveMonitor.startMonitoring(globalMouseMoveMonitor_1.standardMouseMoveMerger, function (mouseMoveData) {
                    var mouseOrthogonalPosition = _this._sliderOrthogonalMousePosition(mouseMoveData);
                    var mouseOrthogonalDelta = Math.abs(mouseOrthogonalPosition - initialMouseOrthogonalPosition_1);
                    // console.log(initialMouseOrthogonalPosition + ' -> ' + mouseOrthogonalPosition + ': ' + mouseOrthogonalDelta);
                    if (Platform.isWindows && mouseOrthogonalDelta > MOUSE_DRAG_RESET_DISTANCE) {
                        // The mouse has wondered away from the scrollbar => reset dragging
                        _this.setDesiredScrollPosition(initialScrollPosition_1);
                    }
                    else {
                        var desiredSliderPosition = _this._sliderMousePosition(mouseMoveData) - draggingDelta_1;
                        _this.setDesiredScrollPosition(_this._scrollbarState.convertSliderPositionToScrollPosition(desiredSliderPosition));
                    }
                }, function () {
                    DomUtils.toggleClass(_this.slider, 'active', false);
                    _this._parent.onDragEnd();
                });
                e.preventDefault();
                this._parent.onDragStart();
            }
        };
        AbstractScrollbar.prototype.validateScrollPosition = function (desiredScrollPosition) {
            return this._scrollbarState.validateScrollPosition(desiredScrollPosition);
        };
        AbstractScrollbar.prototype.setDesiredScrollPosition = function (desiredScrollPosition) {
            desiredScrollPosition = this.validateScrollPosition(desiredScrollPosition);
            this._setScrollPosition(desiredScrollPosition);
            this.onElementScrollPosition(desiredScrollPosition);
            this._renderSlider();
        };
        return AbstractScrollbar;
    }(widget_1.Widget));
    exports.AbstractScrollbar = AbstractScrollbar;
    function setPosition(domNode, top, left, bottom, right) {
        if (top !== null) {
            domNode.style.top = top + 'px';
        }
        if (left !== null) {
            domNode.style.left = left + 'px';
        }
        if (bottom !== null) {
            domNode.style.bottom = bottom + 'px';
        }
        if (right !== null) {
            domNode.style.right = right + 'px';
        }
    }
    function setSize(domNode, width, height) {
        if (width !== null) {
            domNode.style.width = width + 'px';
        }
        if (height !== null) {
            domNode.style.height = height + 'px';
        }
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/scrollbar/horizontalScrollbar", ["require", "exports", 'vs/base/browser/browser', 'vs/base/browser/ui/scrollbar/abstractScrollbar', 'vs/base/browser/mouseEvent', 'vs/base/browser/ui/scrollbar/scrollableElement', 'vs/base/browser/styleMutator'], function (require, exports, Browser, abstractScrollbar_1, mouseEvent_1, scrollableElement_1, styleMutator_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var HorizontalScrollbar = (function (_super) {
        __extends(HorizontalScrollbar, _super);
        function HorizontalScrollbar(scrollable, parent, options) {
            var _this = this;
            var s = new abstractScrollbar_1.ScrollbarState((options.horizontalHasArrows ? options.arrowSize : 0), (options.horizontal === scrollableElement_1.Visibility.Hidden ? 0 : options.horizontalScrollbarSize), (options.vertical === scrollableElement_1.Visibility.Hidden ? 0 : options.verticalScrollbarSize));
            _super.call(this, options.forbidTranslate3dUse, parent, s, options.horizontal, 'horizontal');
            this._scrollable = scrollable;
            this._createDomNode();
            if (options.horizontalHasArrows) {
                var arrowDelta = (options.arrowSize - abstractScrollbar_1.ARROW_IMG_SIZE) / 2;
                var scrollbarDelta = (options.horizontalScrollbarSize - abstractScrollbar_1.ARROW_IMG_SIZE) / 2;
                this._createArrow('left-arrow', scrollbarDelta, arrowDelta, null, null, options.arrowSize, options.horizontalScrollbarSize, function () { return _this._createMouseWheelEvent(1); });
                this._createArrow('right-arrow', scrollbarDelta, null, null, arrowDelta, options.arrowSize, options.horizontalScrollbarSize, function () { return _this._createMouseWheelEvent(-1); });
            }
            this._createSlider(Math.floor((options.horizontalScrollbarSize - options.horizontalSliderSize) / 2), 0, null, options.horizontalSliderSize);
        }
        HorizontalScrollbar.prototype._createMouseWheelEvent = function (sign) {
            return new mouseEvent_1.StandardMouseWheelEvent(null, sign, 0);
        };
        HorizontalScrollbar.prototype._updateSlider = function (sliderSize, sliderPosition) {
            styleMutator_1.StyleMutator.setWidth(this.slider, sliderSize);
            if (!this._forbidTranslate3dUse && Browser.canUseTranslate3d) {
                styleMutator_1.StyleMutator.setTransform(this.slider, 'translate3d(' + sliderPosition + 'px, 0px, 0px)');
            }
            else {
                styleMutator_1.StyleMutator.setLeft(this.slider, sliderPosition);
            }
        };
        HorizontalScrollbar.prototype._renderDomNode = function (largeSize, smallSize) {
            styleMutator_1.StyleMutator.setWidth(this.domNode, largeSize);
            styleMutator_1.StyleMutator.setHeight(this.domNode, smallSize);
            styleMutator_1.StyleMutator.setLeft(this.domNode, 0);
            styleMutator_1.StyleMutator.setBottom(this.domNode, 0);
        };
        HorizontalScrollbar.prototype._mouseDownRelativePosition = function (e, domNodePosition) {
            return e.posx - domNodePosition.left;
        };
        HorizontalScrollbar.prototype._sliderMousePosition = function (e) {
            return e.posx;
        };
        HorizontalScrollbar.prototype._sliderOrthogonalMousePosition = function (e) {
            return e.posy;
        };
        HorizontalScrollbar.prototype._getScrollPosition = function () {
            return this._scrollable.getScrollLeft();
        };
        HorizontalScrollbar.prototype._setScrollPosition = function (scrollPosition) {
            this._scrollable.setScrollLeft(scrollPosition);
        };
        return HorizontalScrollbar;
    }(abstractScrollbar_1.AbstractScrollbar));
    exports.HorizontalScrollbar = HorizontalScrollbar;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/scrollbar/verticalScrollbar", ["require", "exports", 'vs/base/browser/browser', 'vs/base/browser/ui/scrollbar/abstractScrollbar', 'vs/base/browser/mouseEvent', 'vs/base/browser/ui/scrollbar/scrollableElement', 'vs/base/browser/styleMutator'], function (require, exports, Browser, abstractScrollbar_1, mouseEvent_1, scrollableElement_1, styleMutator_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var VerticalScrollbar = (function (_super) {
        __extends(VerticalScrollbar, _super);
        function VerticalScrollbar(scrollable, parent, options) {
            var _this = this;
            var s = new abstractScrollbar_1.ScrollbarState((options.verticalHasArrows ? options.arrowSize : 0), (options.vertical === scrollableElement_1.Visibility.Hidden ? 0 : options.verticalScrollbarSize), 
            // give priority to vertical scroll bar over horizontal and let it scroll all the way to the bottom
            0);
            _super.call(this, options.forbidTranslate3dUse, parent, s, options.vertical, 'vertical');
            this._scrollable = scrollable;
            this._createDomNode();
            if (options.verticalHasArrows) {
                var arrowDelta = (options.arrowSize - abstractScrollbar_1.ARROW_IMG_SIZE) / 2;
                var scrollbarDelta = (options.verticalScrollbarSize - abstractScrollbar_1.ARROW_IMG_SIZE) / 2;
                this._createArrow('up-arrow', arrowDelta, scrollbarDelta, null, null, options.verticalScrollbarSize, options.arrowSize, function () { return _this._createMouseWheelEvent(1); });
                this._createArrow('down-arrow', null, scrollbarDelta, arrowDelta, null, options.verticalScrollbarSize, options.arrowSize, function () { return _this._createMouseWheelEvent(-1); });
            }
            this._createSlider(0, Math.floor((options.verticalScrollbarSize - options.verticalSliderSize) / 2), options.verticalSliderSize, null);
        }
        VerticalScrollbar.prototype._createMouseWheelEvent = function (sign) {
            return new mouseEvent_1.StandardMouseWheelEvent(null, 0, sign);
        };
        VerticalScrollbar.prototype._updateSlider = function (sliderSize, sliderPosition) {
            styleMutator_1.StyleMutator.setHeight(this.slider, sliderSize);
            if (!this._forbidTranslate3dUse && Browser.canUseTranslate3d) {
                styleMutator_1.StyleMutator.setTransform(this.slider, 'translate3d(0px, ' + sliderPosition + 'px, 0px)');
            }
            else {
                styleMutator_1.StyleMutator.setTop(this.slider, sliderPosition);
            }
        };
        VerticalScrollbar.prototype._renderDomNode = function (largeSize, smallSize) {
            styleMutator_1.StyleMutator.setWidth(this.domNode, smallSize);
            styleMutator_1.StyleMutator.setHeight(this.domNode, largeSize);
            styleMutator_1.StyleMutator.setRight(this.domNode, 0);
            styleMutator_1.StyleMutator.setTop(this.domNode, 0);
        };
        VerticalScrollbar.prototype._mouseDownRelativePosition = function (e, domNodePosition) {
            return e.posy - domNodePosition.top;
        };
        VerticalScrollbar.prototype._sliderMousePosition = function (e) {
            return e.posy;
        };
        VerticalScrollbar.prototype._sliderOrthogonalMousePosition = function (e) {
            return e.posx;
        };
        VerticalScrollbar.prototype._getScrollPosition = function () {
            return this._scrollable.getScrollTop();
        };
        VerticalScrollbar.prototype._setScrollPosition = function (scrollPosition) {
            this._scrollable.setScrollTop(scrollPosition);
        };
        return VerticalScrollbar;
    }(abstractScrollbar_1.AbstractScrollbar));
    exports.VerticalScrollbar = VerticalScrollbar;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/scrollbar/scrollableElementImpl", ["require", "exports", 'vs/base/browser/dom', 'vs/base/common/platform', 'vs/base/browser/mouseEvent', 'vs/base/browser/ui/scrollbar/domNodeScrollable', 'vs/base/browser/ui/scrollbar/horizontalScrollbar', 'vs/base/browser/ui/scrollbar/verticalScrollbar', 'vs/base/browser/ui/scrollbar/scrollableElement', 'vs/base/common/lifecycle', 'vs/base/browser/ui/widget', 'vs/base/common/async', 'vs/css!./media/scrollbars'], function (require, exports, DomUtils, Platform, mouseEvent_1, domNodeScrollable_1, horizontalScrollbar_1, verticalScrollbar_1, scrollableElement_1, lifecycle_1, widget_1, async_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var HIDE_TIMEOUT = 500;
    var SCROLL_WHEEL_SENSITIVITY = 50;
    var ScrollableElement = (function (_super) {
        __extends(ScrollableElement, _super);
        function ScrollableElement(element, options, dimensions) {
            var _this = this;
            if (dimensions === void 0) { dimensions = null; }
            _super.call(this);
            this._originalElement = element;
            this._originalElement.style.overflow = 'hidden';
            this._options = this._createOptions(options);
            if (this._options.scrollable) {
                this._scrollable = this._options.scrollable;
            }
            else {
                this._scrollable = this._register(new domNodeScrollable_1.DomNodeScrollable(this._originalElement));
            }
            this.verticalScrollbarWidth = this._options.verticalScrollbarSize;
            this.horizontalScrollbarHeight = this._options.horizontalScrollbarSize;
            this._verticalScrollbar = this._register(new verticalScrollbar_1.VerticalScrollbar(this._scrollable, this, this._options));
            this._horizontalScrollbar = this._register(new horizontalScrollbar_1.HorizontalScrollbar(this._scrollable, this, this._options));
            this._domNode = document.createElement('div');
            this._domNode.className = 'monaco-scrollable-element ' + this._options.className;
            this._domNode.setAttribute('role', 'presentation');
            this._domNode.style.position = 'relative';
            this._domNode.style.overflow = 'hidden';
            this._domNode.appendChild(this._originalElement);
            this._domNode.appendChild(this._horizontalScrollbar.domNode);
            this._domNode.appendChild(this._verticalScrollbar.domNode);
            if (this._options.useShadows) {
                this._leftShadowDomNode = document.createElement('div');
                this._leftShadowDomNode.className = 'shadow';
                this._domNode.appendChild(this._leftShadowDomNode);
            }
            if (this._options.useShadows) {
                this._topShadowDomNode = document.createElement('div');
                this._topShadowDomNode.className = 'shadow';
                this._domNode.appendChild(this._topShadowDomNode);
                this._topLeftShadowDomNode = document.createElement('div');
                this._topLeftShadowDomNode.className = 'shadow top-left-corner';
                this._domNode.appendChild(this._topLeftShadowDomNode);
            }
            this._listenOnDomNode = this._options.listenOnDomNode || this._domNode;
            this._register(this._scrollable.addScrollListener(function () { return _this._onScroll(); }));
            this._mouseWheelToDispose = [];
            this._setListeningToMouseWheel(this._options.handleMouseWheel);
            this.onmouseover(this._listenOnDomNode, function (e) { return _this._onMouseOver(e); });
            this.onnonbubblingmouseout(this._listenOnDomNode, function (e) { return _this._onMouseOut(e); });
            this._onElementDimensionsTimeout = this._register(new async_1.TimeoutTimer());
            this._onElementInternalDimensionsTimeout = this._register(new async_1.TimeoutTimer());
            this._hideTimeout = this._register(new async_1.TimeoutTimer());
            this._isDragging = false;
            this._mouseIsOver = false;
            this.onElementDimensions(dimensions, true);
            this.onElementInternalDimensions(true);
        }
        ScrollableElement.prototype.dispose = function () {
            this._mouseWheelToDispose = lifecycle_1.disposeAll(this._mouseWheelToDispose);
            _super.prototype.dispose.call(this);
        };
        ScrollableElement.prototype.getDomNode = function () {
            return this._domNode;
        };
        ScrollableElement.prototype.getOverviewRulerLayoutInfo = function () {
            return {
                parent: this._domNode,
                insertBefore: this._verticalScrollbar.domNode,
            };
        };
        ScrollableElement.prototype.delegateVerticalScrollbarMouseDown = function (browserEvent) {
            this._verticalScrollbar.delegateMouseDown(browserEvent);
        };
        ScrollableElement.prototype.onElementDimensions = function (dimensions, synchronous) {
            var _this = this;
            if (dimensions === void 0) { dimensions = null; }
            if (synchronous === void 0) { synchronous = false; }
            if (synchronous) {
                this._actualElementDimensions(dimensions);
                this._onElementDimensionsTimeout.cancel();
            }
            else {
                this._onElementDimensionsTimeout.cancelAndSet(function () { return _this._actualElementDimensions(dimensions); }, 0);
            }
        };
        ScrollableElement.prototype._actualElementDimensions = function (dimensions) {
            if (dimensions === void 0) { dimensions = null; }
            if (!dimensions) {
                dimensions = {
                    width: this._domNode.clientWidth,
                    height: this._domNode.clientHeight
                };
            }
            this._dimensions = this._computeDimensions(dimensions.width, dimensions.height);
            this._verticalScrollbar.onElementSize(this._dimensions.height);
            this._horizontalScrollbar.onElementSize(this._dimensions.width);
        };
        ScrollableElement.prototype.onElementInternalDimensions = function (synchronous) {
            var _this = this;
            if (synchronous === void 0) { synchronous = false; }
            if (synchronous) {
                this._actualElementInternalDimensions();
                this._onElementInternalDimensionsTimeout.cancel();
            }
            else {
                this._onElementInternalDimensionsTimeout.setIfNotSet(function () { return _this._actualElementInternalDimensions(); }, 0);
            }
        };
        ScrollableElement.prototype._actualElementInternalDimensions = function () {
            this._horizontalScrollbar.onElementScrollSize(this._scrollable.getScrollWidth());
            this._verticalScrollbar.onElementScrollSize(this._scrollable.getScrollHeight());
        };
        ScrollableElement.prototype.updateClassName = function (newClassName) {
            this._options.className = newClassName;
            // Defaults are different on Macs
            if (Platform.isMacintosh) {
                this._options.className += ' mac';
            }
            this._domNode.className = 'monaco-scrollable-element ' + this._options.className;
        };
        ScrollableElement.prototype.updateOptions = function (newOptions) {
            // only support handleMouseWheel changes for now
            var massagedOptions = this._createOptions(newOptions);
            this._options.handleMouseWheel = massagedOptions.handleMouseWheel;
            this._options.mouseWheelScrollSensitivity = massagedOptions.mouseWheelScrollSensitivity;
            this._setListeningToMouseWheel(this._options.handleMouseWheel);
        };
        // -------------------- mouse wheel scrolling --------------------
        ScrollableElement.prototype._setListeningToMouseWheel = function (shouldListen) {
            var _this = this;
            var isListening = (this._mouseWheelToDispose.length > 0);
            if (isListening === shouldListen) {
                // No change
                return;
            }
            // Stop listening (if necessary)
            this._mouseWheelToDispose = lifecycle_1.disposeAll(this._mouseWheelToDispose);
            // Start listening (if necessary)
            if (shouldListen) {
                var onMouseWheel = function (browserEvent) {
                    var e = new mouseEvent_1.StandardMouseWheelEvent(browserEvent);
                    _this.onMouseWheel(e);
                };
                this._mouseWheelToDispose.push(DomUtils.addDisposableListener(this._listenOnDomNode, 'mousewheel', onMouseWheel));
                this._mouseWheelToDispose.push(DomUtils.addDisposableListener(this._listenOnDomNode, 'DOMMouseScroll', onMouseWheel));
            }
        };
        ScrollableElement.prototype.onMouseWheel = function (e) {
            if (Platform.isMacintosh && e.browserEvent && this._options.saveLastScrollTimeOnClassName) {
                // Mark dom node with timestamp of wheel event
                var target = e.browserEvent.target;
                if (target && target.nodeType === 1) {
                    var r = DomUtils.findParentWithClass(target, this._options.saveLastScrollTimeOnClassName);
                    if (r) {
                        r.setAttribute('last-scroll-time', String(new Date().getTime()));
                    }
                }
            }
            var desiredScrollTop = -1;
            var desiredScrollLeft = -1;
            if (e.deltaY || e.deltaX) {
                var deltaY = e.deltaY * this._options.mouseWheelScrollSensitivity;
                var deltaX = e.deltaX * this._options.mouseWheelScrollSensitivity;
                if (this._options.flipAxes) {
                    deltaY = e.deltaX;
                    deltaX = e.deltaY;
                }
                if (Platform.isMacintosh) {
                    // Give preference to vertical scrolling
                    if (deltaY && Math.abs(deltaX) < 0.2) {
                        deltaX = 0;
                    }
                    if (Math.abs(deltaY) > Math.abs(deltaX) * 0.5) {
                        deltaX = 0;
                    }
                }
                if (deltaY) {
                    var currentScrollTop = this._scrollable.getScrollTop();
                    desiredScrollTop = this._verticalScrollbar.validateScrollPosition((desiredScrollTop !== -1 ? desiredScrollTop : currentScrollTop) - SCROLL_WHEEL_SENSITIVITY * deltaY);
                    if (desiredScrollTop === currentScrollTop) {
                        desiredScrollTop = -1;
                    }
                }
                if (deltaX) {
                    var currentScrollLeft = this._scrollable.getScrollLeft();
                    desiredScrollLeft = this._horizontalScrollbar.validateScrollPosition((desiredScrollLeft !== -1 ? desiredScrollLeft : currentScrollLeft) - SCROLL_WHEEL_SENSITIVITY * deltaX);
                    if (desiredScrollLeft === currentScrollLeft) {
                        desiredScrollLeft = -1;
                    }
                }
                if (desiredScrollTop !== -1 || desiredScrollLeft !== -1) {
                    if (desiredScrollTop !== -1) {
                        this._verticalScrollbar.setDesiredScrollPosition(desiredScrollTop);
                        desiredScrollTop = -1;
                    }
                    if (desiredScrollLeft !== -1) {
                        this._horizontalScrollbar.setDesiredScrollPosition(desiredScrollLeft);
                        desiredScrollLeft = -1;
                    }
                }
            }
            e.preventDefault();
            e.stopPropagation();
        };
        ScrollableElement.prototype._onScroll = function () {
            var scrollHeight = this._scrollable.getScrollHeight();
            var scrollTop = this._scrollable.getScrollTop();
            var scrollWidth = this._scrollable.getScrollWidth();
            var scrollLeft = this._scrollable.getScrollLeft();
            this._verticalScrollbar.onElementScrollPosition(scrollTop);
            this._horizontalScrollbar.onElementScrollPosition(scrollLeft);
            if (this._options.useShadows) {
                var enableTop = scrollHeight > 0 && scrollTop > 0;
                var enableLeft = this._options.useShadows && scrollWidth > 0 && scrollLeft > 0;
                if (this._topShadowDomNode) {
                    DomUtils.toggleClass(this._topShadowDomNode, 'top', enableTop);
                }
                if (this._topLeftShadowDomNode) {
                    DomUtils.toggleClass(this._topLeftShadowDomNode, 'top', enableTop);
                }
                if (this._leftShadowDomNode) {
                    DomUtils.toggleClass(this._leftShadowDomNode, 'left', enableLeft);
                }
                if (this._topLeftShadowDomNode) {
                    DomUtils.toggleClass(this._topLeftShadowDomNode, 'left', enableLeft);
                }
            }
            this._reveal();
        };
        // -------------------- fade in / fade out --------------------
        ScrollableElement.prototype.onDragStart = function () {
            this._isDragging = true;
            this._reveal();
        };
        ScrollableElement.prototype.onDragEnd = function () {
            this._isDragging = false;
            this._hide();
        };
        ScrollableElement.prototype._onMouseOut = function (e) {
            this._mouseIsOver = false;
            this._hide();
        };
        ScrollableElement.prototype._onMouseOver = function (e) {
            this._mouseIsOver = true;
            this._reveal();
        };
        ScrollableElement.prototype._reveal = function () {
            this._verticalScrollbar.beginReveal();
            this._horizontalScrollbar.beginReveal();
            this._scheduleHide();
        };
        ScrollableElement.prototype._hide = function () {
            if (!this._mouseIsOver && !this._isDragging) {
                this._verticalScrollbar.beginHide();
                this._horizontalScrollbar.beginHide();
            }
        };
        ScrollableElement.prototype._scheduleHide = function () {
            var _this = this;
            this._hideTimeout.cancelAndSet(function () { return _this._hide(); }, HIDE_TIMEOUT);
        };
        // -------------------- size & layout --------------------
        ScrollableElement.prototype._computeDimensions = function (clientWidth, clientHeight) {
            var width = clientWidth;
            var height = clientHeight;
            return {
                width: width,
                height: height
            };
        };
        ScrollableElement.prototype._createOptions = function (options) {
            function ensureValue(source, prop, value) {
                if (source.hasOwnProperty(prop)) {
                    return source[prop];
                }
                return value;
            }
            var result = {
                forbidTranslate3dUse: ensureValue(options, 'forbidTranslate3dUse', false),
                className: ensureValue(options, 'className', ''),
                useShadows: ensureValue(options, 'useShadows', true),
                handleMouseWheel: ensureValue(options, 'handleMouseWheel', true),
                flipAxes: ensureValue(options, 'flipAxes', false),
                mouseWheelScrollSensitivity: ensureValue(options, 'mouseWheelScrollSensitivity', 1),
                arrowSize: ensureValue(options, 'arrowSize', 11),
                scrollable: ensureValue(options, 'scrollable', null),
                listenOnDomNode: ensureValue(options, 'listenOnDomNode', null),
                horizontal: scrollableElement_1.visibilityFromString(ensureValue(options, 'horizontal', 'auto')),
                horizontalScrollbarSize: ensureValue(options, 'horizontalScrollbarSize', 10),
                horizontalSliderSize: 0,
                horizontalHasArrows: ensureValue(options, 'horizontalHasArrows', false),
                vertical: scrollableElement_1.visibilityFromString(ensureValue(options, 'vertical', 'auto')),
                verticalScrollbarSize: ensureValue(options, 'verticalScrollbarSize', 10),
                verticalHasArrows: ensureValue(options, 'verticalHasArrows', false),
                verticalSliderSize: 0,
                saveLastScrollTimeOnClassName: ensureValue(options, 'saveLastScrollTimeOnClassName', null)
            };
            result.horizontalSliderSize = ensureValue(options, 'horizontalSliderSize', result.horizontalScrollbarSize);
            result.verticalSliderSize = ensureValue(options, 'verticalSliderSize', result.verticalScrollbarSize);
            // Defaults are different on Macs
            if (Platform.isMacintosh) {
                result.className += ' mac';
            }
            return result;
        };
        return ScrollableElement;
    }(widget_1.Widget));
    exports.ScrollableElement = ScrollableElement;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/browser/ui/list/listView", ["require", "exports", 'vs/base/common/event', 'vs/base/common/objects', 'vs/base/common/lifecycle', 'vs/base/browser/touch', 'vs/base/browser/dom', 'vs/base/browser/ui/scrollbar/scrollableElementImpl', './rangeMap', './rowCache', 'vs/base/common/diff/diff'], function (require, exports, event_1, objects_1, lifecycle_1, touch_1, DOM, scrollableElementImpl_1, rangeMap_1, rowCache_1, diff_1) {
    "use strict";
    function toSequence(itemRanges) {
        return {
            getLength: function () { return itemRanges.length; },
            getElementHash: function (i) { return (itemRanges[i].item.id + ":" + itemRanges[i].range.start + ":" + itemRanges[i].range.end); }
        };
    }
    var MouseEventTypes = ['click',
        'dblclick',
        'mouseup',
        'mousedown',
        'mouseover',
        'mousemove',
        'mouseout',
        'contextmenu'
    ];
    var ListView = (function () {
        function ListView(container, delegate, renderers) {
            this.delegate = delegate;
            this._onScroll = new event_1.Emitter();
            this.items = [];
            this.itemId = 0;
            this.rangeMap = new rangeMap_1.RangeMap();
            this.renderers = objects_1.toObject(renderers, function (r) { return r.templateId; });
            this.cache = new rowCache_1.RowCache(this.renderers);
            this.renderTop = 0;
            this._renderHeight = 0;
            this._domNode = document.createElement('div');
            this._domNode.className = 'monaco-list';
            this._domNode.tabIndex = 0;
            this.rowsContainer = document.createElement('div');
            this.rowsContainer.className = 'monaco-list-rows';
            this.gesture = new touch_1.Gesture(this.rowsContainer);
            this.scrollableElement = new scrollableElementImpl_1.ScrollableElement(this.rowsContainer, {
                forbidTranslate3dUse: true,
                scrollable: this,
                horizontal: 'hidden',
                vertical: 'auto',
                useShadows: false,
                saveLastScrollTimeOnClassName: 'monaco-list-row'
            });
            this._domNode.appendChild(this.scrollableElement.getDomNode());
            container.appendChild(this._domNode);
            this.toDispose = [this.rangeMap, this.gesture, this.scrollableElement, this._onScroll];
            this.layout();
        }
        Object.defineProperty(ListView.prototype, "domNode", {
            get: function () {
                return this._domNode;
            },
            enumerable: true,
            configurable: true
        });
        ListView.prototype.splice = function (start, deleteCount) {
            var _this = this;
            var elements = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                elements[_i - 2] = arguments[_i];
            }
            var before = this.getRenderedItemRanges();
            var inserted = elements.map(function (element) { return ({
                id: String(_this.itemId++),
                element: element,
                size: _this.delegate.getHeight(element),
                templateId: _this.delegate.getTemplateId(element),
                row: null
            }); });
            (_a = this.rangeMap).splice.apply(_a, [start, deleteCount].concat(inserted));
            var deleted = (_b = this.items).splice.apply(_b, [start, deleteCount].concat(inserted));
            var after = this.getRenderedItemRanges();
            var lcs = new diff_1.LcsDiff(toSequence(before), toSequence(after), null);
            var diffs = lcs.ComputeDiff();
            for (var _c = 0, diffs_1 = diffs; _c < diffs_1.length; _c++) {
                var diff = diffs_1[_c];
                for (var i = 0; i < diff.originalLength; i++) {
                    this.removeItemFromDOM(before[diff.originalStart + i].item);
                }
                for (var i = 0; i < diff.modifiedLength; i++) {
                    this.insertItemInDOM(after[diff.modifiedStart + i].item, after[0].index + diff.modifiedStart + i);
                }
            }
            this.rowsContainer.style.height = this.rangeMap.size + "px";
            this.setScrollTop(this.renderTop);
            this.scrollableElement.onElementInternalDimensions();
            return deleted.map(function (i) { return i.element; });
            var _a, _b;
        };
        Object.defineProperty(ListView.prototype, "length", {
            get: function () {
                return this.items.length;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ListView.prototype, "renderHeight", {
            get: function () {
                return this._renderHeight;
            },
            enumerable: true,
            configurable: true
        });
        ListView.prototype.element = function (index) {
            return this.items[index].element;
        };
        ListView.prototype.elementHeight = function (index) {
            return this.items[index].size;
        };
        ListView.prototype.elementTop = function (index) {
            return this.rangeMap.positionAt(index);
        };
        ListView.prototype.indexAt = function (position) {
            return this.rangeMap.indexAt(position);
        };
        ListView.prototype.indexAfter = function (position) {
            return this.rangeMap.indexAfter(position);
        };
        ListView.prototype.layout = function (height) {
            this.setRenderHeight(height || DOM.getContentHeight(this._domNode));
            this.setScrollTop(this.renderTop);
            this.scrollableElement.onElementDimensions();
            this.scrollableElement.onElementInternalDimensions();
        };
        // Render
        ListView.prototype.setRenderHeight = function (viewHeight) {
            this.render(this.renderTop, viewHeight);
            this._renderHeight = viewHeight;
        };
        ListView.prototype.render = function (renderTop, renderHeight) {
            var renderBottom = renderTop + renderHeight;
            var thisRenderBottom = this.renderTop + this._renderHeight;
            var i, stop;
            // when view scrolls down, start rendering from the renderBottom
            for (i = this.rangeMap.indexAfter(renderBottom) - 1, stop = this.rangeMap.indexAt(Math.max(thisRenderBottom, renderTop)); i >= stop; i--) {
                this.insertItemInDOM(this.items[i], i);
            }
            // when view scrolls up, start rendering from either this.renderTop or renderBottom
            for (i = Math.min(this.rangeMap.indexAt(this.renderTop), this.rangeMap.indexAfter(renderBottom)) - 1, stop = this.rangeMap.indexAt(renderTop); i >= stop; i--) {
                this.insertItemInDOM(this.items[i], i);
            }
            // when view scrolls down, start unrendering from renderTop
            for (i = this.rangeMap.indexAt(this.renderTop), stop = Math.min(this.rangeMap.indexAt(renderTop), this.rangeMap.indexAfter(thisRenderBottom)); i < stop; i++) {
                this.removeItemFromDOM(this.items[i]);
            }
            // when view scrolls up, start unrendering from either renderBottom this.renderTop
            for (i = Math.max(this.rangeMap.indexAfter(renderBottom), this.rangeMap.indexAt(this.renderTop)), stop = this.rangeMap.indexAfter(thisRenderBottom); i < stop; i++) {
                this.removeItemFromDOM(this.items[i]);
            }
            this.rowsContainer.style.transform = "translate3d(0px, -" + renderTop + "px, 0px)";
            this.renderTop = renderTop;
            this._renderHeight = renderBottom - renderTop;
        };
        ListView.prototype.getRenderedItemRanges = function () {
            var result = [];
            var renderBottom = this.renderTop + this._renderHeight;
            var start = this.renderTop;
            var index = this.rangeMap.indexAt(start);
            var item = this.items[index];
            var end = -1;
            while (item && start <= renderBottom) {
                end = start + item.size;
                result.push({ item: item, index: index, range: { start: start, end: end } });
                start = end;
                item = this.items[++index];
            }
            return result;
        };
        // DOM operations
        ListView.prototype.insertItemInDOM = function (item, index) {
            if (!item.row) {
                item.row = this.cache.alloc(item.templateId);
            }
            if (!item.row.domNode.parentElement) {
                this.rowsContainer.appendChild(item.row.domNode);
            }
            var renderer = this.renderers[item.templateId];
            item.row.domNode.style.top = this.elementTop(index) + "px";
            item.row.domNode.style.height = item.size + "px";
            item.row.domNode.setAttribute('data-index', "" + index);
            renderer.renderElement(item.element, index, item.row.templateData);
        };
        ListView.prototype.removeItemFromDOM = function (item) {
            this.cache.release(item.row);
            item.row = null;
        };
        // IScrollable
        ListView.prototype.getScrollHeight = function () {
            return this.rangeMap.size;
        };
        ListView.prototype.getScrollWidth = function () {
            return 0;
        };
        ListView.prototype.getScrollLeft = function () {
            return 0;
        };
        ListView.prototype.setScrollLeft = function (scrollLeft) {
            // noop
        };
        ListView.prototype.getScrollTop = function () {
            return this.renderTop;
        };
        ListView.prototype.setScrollTop = function (scrollTop) {
            scrollTop = Math.min(scrollTop, this.getScrollHeight() - this._renderHeight);
            scrollTop = Math.max(scrollTop, 0);
            this.render(scrollTop, this._renderHeight);
            this.renderTop = scrollTop;
            this._onScroll.fire({ vertical: true, horizontal: false });
        };
        ListView.prototype.addScrollListener = function (callback) {
            return this._onScroll.event(callback);
        };
        // Events
        ListView.prototype.addListener = function (type, handler, useCapture) {
            var _this = this;
            if (MouseEventTypes.indexOf(type) > -1) {
                var userHandler_1 = handler;
                handler = function (event) {
                    var index = _this.getItemIndex(event);
                    if (index < 0) {
                        return;
                    }
                    var element = _this.items[index].element;
                    userHandler_1(objects_1.assign(event, { element: element, index: index }));
                };
            }
            return DOM.addDisposableListener(this.domNode, type, handler, useCapture);
        };
        ListView.prototype.getItemIndex = function (event) {
            var target = event.target;
            while (target instanceof HTMLElement && target !== this.rowsContainer) {
                var element = target;
                var rawIndex = element.getAttribute('data-index');
                if (rawIndex) {
                    var index = Number(rawIndex);
                    if (!isNaN(index)) {
                        return index;
                    }
                }
                target = element.parentElement;
            }
            return -1;
        };
        // Dispose
        ListView.prototype.dispose = function () {
            this.items = null;
            if (this._domNode && this._domNode.parentElement) {
                this._domNode.parentNode.removeChild(this._domNode);
                this._domNode = null;
            }
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
        };
        return ListView;
    }());
    exports.ListView = ListView;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/browser/ui/list/listWidget", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/common/types', 'vs/base/browser/dom', 'vs/base/common/event', './listView', 'vs/css!./list'], function (require, exports, lifecycle_1, types_1, DOM, event_1, listView_1) {
    "use strict";
    var TraitRenderer = (function () {
        function TraitRenderer(controller, renderer) {
            this.controller = controller;
            this.renderer = renderer;
        }
        Object.defineProperty(TraitRenderer.prototype, "templateId", {
            get: function () {
                return this.renderer.templateId;
            },
            enumerable: true,
            configurable: true
        });
        TraitRenderer.prototype.renderTemplate = function (container) {
            var data = this.renderer.renderTemplate(container);
            return { container: container, data: data };
        };
        TraitRenderer.prototype.renderElement = function (element, index, templateData) {
            this.controller.renderElement(element, index, templateData.container);
            this.renderer.renderElement(element, index, templateData.data);
        };
        TraitRenderer.prototype.disposeTemplate = function (templateData) {
            return this.renderer.disposeTemplate(templateData.data);
        };
        return TraitRenderer;
    }());
    var Trait = (function () {
        function Trait(_trait) {
            this._trait = _trait;
            this._onChange = new event_1.Emitter();
            this.indexes = [];
        }
        Object.defineProperty(Trait.prototype, "onChange", {
            get: function () { return this._onChange.event; },
            enumerable: true,
            configurable: true
        });
        Trait.prototype.splice = function (start, deleteCount, insertCount) {
            var diff = insertCount - deleteCount;
            var end = start + deleteCount;
            var indexes = [];
            for (var _i = 0, indexes_1 = indexes; _i < indexes_1.length; _i++) {
                var index = indexes_1[_i];
                if (index >= start && index < end) {
                    continue;
                }
                indexes.push(index > start ? index + diff : index);
            }
            this.indexes = indexes;
            this._onChange.fire({ indexes: indexes });
        };
        Trait.prototype.renderElement = function (element, index, container) {
            DOM.toggleClass(container, this._trait, this.contains(index));
        };
        Trait.prototype.set = function () {
            var indexes = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                indexes[_i - 0] = arguments[_i];
            }
            var result = this.indexes;
            this.indexes = indexes;
            this._onChange.fire({ indexes: indexes });
            return result;
        };
        Trait.prototype.get = function () {
            return this.indexes;
        };
        Trait.prototype.contains = function (index) {
            return this.indexes.some(function (i) { return i === index; });
        };
        Trait.prototype.wrapRenderer = function (renderer) {
            return new TraitRenderer(this, renderer);
        };
        Trait.prototype.dispose = function () {
            this.indexes = null;
            this._onChange = lifecycle_1.dispose(this._onChange);
        };
        return Trait;
    }());
    var FocusTrait = (function (_super) {
        __extends(FocusTrait, _super);
        function FocusTrait(getElementId) {
            _super.call(this, 'focused');
            this.getElementId = getElementId;
        }
        FocusTrait.prototype.renderElement = function (element, index, container) {
            _super.prototype.renderElement.call(this, element, index, container);
            container.setAttribute('role', 'option');
            container.setAttribute('id', this.getElementId(index));
        };
        return FocusTrait;
    }(Trait));
    var Controller = (function () {
        function Controller(list, view) {
            var _this = this;
            this.list = list;
            this.view = view;
            this.toDispose = [];
            this.toDispose.push(view.addListener('click', function (e) { return _this.onClick(e); }));
        }
        Controller.prototype.onClick = function (e) {
            this.list.setSelection(e.index);
        };
        Controller.prototype.dispose = function () {
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
        };
        return Controller;
    }());
    var List = (function () {
        function List(container, delegate, renderers) {
            var _this = this;
            this.idPrefix = "list_id_" + ++List.InstanceCount;
            this.focus = new FocusTrait(function (i) { return _this.getElementId(i); });
            this.selection = new Trait('selected');
            this.eventBufferer = new event_1.EventBufferer();
            renderers = renderers.map(function (r) {
                r = _this.focus.wrapRenderer(r);
                r = _this.selection.wrapRenderer(r);
                return r;
            });
            this.view = new listView_1.ListView(container, delegate, renderers);
            this.view.domNode.setAttribute('role', 'listbox');
            this.controller = new Controller(this, this.view);
        }
        Object.defineProperty(List.prototype, "onFocusChange", {
            get: function () {
                var _this = this;
                return this.eventBufferer.wrapEvent(event_1.mapEvent(this.focus.onChange, function (e) { return _this.toListEvent(e); }));
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(List.prototype, "onSelectionChange", {
            get: function () {
                var _this = this;
                return this.eventBufferer.wrapEvent(event_1.mapEvent(this.selection.onChange, function (e) { return _this.toListEvent(e); }));
            },
            enumerable: true,
            configurable: true
        });
        List.prototype.splice = function (start, deleteCount) {
            var _this = this;
            var elements = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                elements[_i - 2] = arguments[_i];
            }
            this.eventBufferer.bufferEvents(function () {
                _this.focus.splice(start, deleteCount, elements.length);
                _this.selection.splice(start, deleteCount, elements.length);
                (_a = _this.view).splice.apply(_a, [start, deleteCount].concat(elements));
                var _a;
            });
        };
        Object.defineProperty(List.prototype, "length", {
            get: function () {
                return this.view.length;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(List.prototype, "contentHeight", {
            get: function () {
                return this.view.getScrollHeight();
            },
            enumerable: true,
            configurable: true
        });
        List.prototype.layout = function (height) {
            this.view.layout(height);
        };
        List.prototype.setSelection = function () {
            var _this = this;
            var indexes = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                indexes[_i - 0] = arguments[_i];
            }
            this.eventBufferer.bufferEvents(function () {
                indexes = indexes.concat((_a = _this.selection).set.apply(_a, indexes));
                indexes.forEach(function (i) { return _this.view.splice(i, 1, _this.view.element(i)); });
                var _a;
            });
        };
        List.prototype.selectNext = function (n, loop) {
            if (n === void 0) { n = 1; }
            if (loop === void 0) { loop = false; }
            if (this.length === 0) {
                return;
            }
            var selection = this.selection.get();
            var index = selection.length > 0 ? selection[0] + n : 0;
            this.setSelection(loop ? index % this.length : Math.min(index, this.length - 1));
        };
        List.prototype.selectPrevious = function (n, loop) {
            if (n === void 0) { n = 1; }
            if (loop === void 0) { loop = false; }
            if (this.length === 0) {
                return;
            }
            var selection = this.selection.get();
            var index = selection.length > 0 ? selection[0] - n : 0;
            if (loop && index < 0) {
                index = this.length + (index % this.length);
            }
            this.setSelection(Math.max(index, 0));
        };
        List.prototype.setFocus = function () {
            var _this = this;
            var indexes = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                indexes[_i - 0] = arguments[_i];
            }
            this.eventBufferer.bufferEvents(function () {
                indexes = indexes.concat((_a = _this.focus).set.apply(_a, indexes));
                indexes.forEach(function (i) { return _this.view.splice(i, 1, _this.view.element(i)); });
                var _a;
            });
        };
        List.prototype.focusNext = function (n, loop) {
            if (n === void 0) { n = 1; }
            if (loop === void 0) { loop = false; }
            if (this.length === 0) {
                return;
            }
            var focus = this.focus.get();
            var index = focus.length > 0 ? focus[0] + n : 0;
            this.setFocus(loop ? index % this.length : Math.min(index, this.length - 1));
        };
        List.prototype.focusPrevious = function (n, loop) {
            if (n === void 0) { n = 1; }
            if (loop === void 0) { loop = false; }
            if (this.length === 0) {
                return;
            }
            var focus = this.focus.get();
            var index = focus.length > 0 ? focus[0] - n : 0;
            if (loop && index < 0) {
                index = (this.length + (index % this.length)) % this.length;
            }
            this.setFocus(Math.max(index, 0));
        };
        List.prototype.focusNextPage = function () {
            var _this = this;
            var lastPageIndex = this.view.indexAt(this.view.getScrollTop() + this.view.renderHeight);
            lastPageIndex = lastPageIndex === 0 ? 0 : lastPageIndex - 1;
            var lastPageElement = this.view.element(lastPageIndex);
            var currentlyFocusedElement = this.getFocus()[0];
            if (currentlyFocusedElement !== lastPageElement) {
                this.setFocus(lastPageIndex);
            }
            else {
                var previousScrollTop = this.view.getScrollTop();
                this.view.setScrollTop(previousScrollTop + this.view.renderHeight - this.view.elementHeight(lastPageIndex));
                if (this.view.getScrollTop() !== previousScrollTop) {
                    // Let the scroll event listener run
                    setTimeout(function () { return _this.focusNextPage(); }, 0);
                }
            }
        };
        List.prototype.focusPreviousPage = function () {
            var _this = this;
            var firstPageIndex;
            var scrollTop = this.view.getScrollTop();
            if (scrollTop === 0) {
                firstPageIndex = this.view.indexAt(scrollTop);
            }
            else {
                firstPageIndex = this.view.indexAfter(scrollTop - 1);
            }
            var firstPageElement = this.view.element(firstPageIndex);
            var currentlyFocusedElement = this.getFocus()[0];
            if (currentlyFocusedElement !== firstPageElement) {
                this.setFocus(firstPageIndex);
            }
            else {
                var previousScrollTop = scrollTop;
                this.view.setScrollTop(scrollTop - this.view.renderHeight);
                if (this.view.getScrollTop() !== previousScrollTop) {
                    // Let the scroll event listener run
                    setTimeout(function () { return _this.focusPreviousPage(); }, 0);
                }
            }
        };
        List.prototype.getFocus = function () {
            var _this = this;
            return this.focus.get().map(function (i) { return _this.view.element(i); });
        };
        List.prototype.reveal = function (index, relativeTop) {
            var scrollTop = this.view.getScrollTop();
            var elementTop = this.view.elementTop(index);
            var elementHeight = this.view.elementHeight(index);
            if (types_1.isNumber(relativeTop)) {
                relativeTop = relativeTop < 0 ? 0 : relativeTop;
                relativeTop = relativeTop > 1 ? 1 : relativeTop;
                // y = mx + b
                var m = elementHeight - this.view.renderHeight;
                this.view.setScrollTop(m * relativeTop + elementTop);
            }
            else {
                var viewItemBottom = elementTop + elementHeight;
                var wrapperBottom = scrollTop + this.view.renderHeight;
                if (elementTop < scrollTop) {
                    this.view.setScrollTop(elementTop);
                }
                else if (viewItemBottom >= wrapperBottom) {
                    this.view.setScrollTop(viewItemBottom - this.view.renderHeight);
                }
            }
        };
        List.prototype.getElementId = function (index) {
            return this.idPrefix + "_" + index;
        };
        List.prototype.toListEvent = function (_a) {
            var _this = this;
            var indexes = _a.indexes;
            return { indexes: indexes, elements: indexes.map(function (i) { return _this.view.element(i); }) };
        };
        List.prototype.dispose = function () {
            this.view = lifecycle_1.dispose(this.view);
            this.focus = lifecycle_1.dispose(this.focus);
            this.selection = lifecycle_1.dispose(this.selection);
        };
        List.InstanceCount = 0;
        return List;
    }());
    exports.List = List;
});

define("vs/base/parts/tree/browser/treeDefaults", ["require", "exports", 'vs/base/common/platform', 'vs/base/common/errors', 'vs/base/browser/dom', 'vs/base/common/keyCodes'], function (require, exports, platform, errors, dom, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LegacyRenderer = (function () {
        function LegacyRenderer() {
        }
        LegacyRenderer.prototype.getHeight = function (tree, element) {
            return 20;
        };
        LegacyRenderer.prototype.getTemplateId = function (tree, element) {
            return 'legacy';
        };
        LegacyRenderer.prototype.renderTemplate = function (tree, templateId, container) {
            return {
                root: container,
                element: null,
                previousCleanupFn: null
            };
        };
        LegacyRenderer.prototype.renderElement = function (tree, element, templateId, templateData) {
            if (templateData.previousCleanupFn) {
                templateData.previousCleanupFn(tree, templateData.element);
            }
            while (templateData.root.firstChild) {
                templateData.root.removeChild(templateData.root.firstChild);
            }
            templateData.element = element;
            templateData.previousCleanupFn = this.render(tree, element, templateData.root);
        };
        LegacyRenderer.prototype.disposeTemplate = function (tree, templateId, templateData) {
            if (templateData.previousCleanupFn) {
                templateData.previousCleanupFn(tree, templateData.element);
            }
            templateData.root = null;
            templateData.element = null;
            templateData.previousCleanupFn = null;
        };
        LegacyRenderer.prototype.render = function (tree, element, container, previousCleanupFn) {
            container.textContent = '' + element;
            return null;
        };
        return LegacyRenderer;
    }());
    exports.LegacyRenderer = LegacyRenderer;
    (function (ClickBehavior) {
        /**
         * Handle the click when the mouse button is pressed but not released yet.
         */
        ClickBehavior[ClickBehavior["ON_MOUSE_DOWN"] = 0] = "ON_MOUSE_DOWN";
        /**
         * Handle the click when the mouse button is released.
         */
        ClickBehavior[ClickBehavior["ON_MOUSE_UP"] = 1] = "ON_MOUSE_UP";
    })(exports.ClickBehavior || (exports.ClickBehavior = {}));
    var ClickBehavior = exports.ClickBehavior;
    var KeybindingDispatcher = (function () {
        function KeybindingDispatcher() {
            this._arr = [];
        }
        KeybindingDispatcher.prototype.set = function (keybinding, callback) {
            this._arr.push({
                keybinding: keybinding,
                callback: callback
            });
        };
        KeybindingDispatcher.prototype.dispatch = function (keybinding) {
            // Loop from the last to the first to handle overwrites
            for (var i = this._arr.length - 1; i >= 0; i--) {
                var item = this._arr[i];
                if (keybinding === item.keybinding) {
                    return item.callback;
                }
            }
            return null;
        };
        return KeybindingDispatcher;
    }());
    exports.KeybindingDispatcher = KeybindingDispatcher;
    var DefaultController = (function () {
        function DefaultController(options) {
            var _this = this;
            if (options === void 0) { options = { clickBehavior: ClickBehavior.ON_MOUSE_UP }; }
            this.options = options;
            this.downKeyBindingDispatcher = new KeybindingDispatcher();
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.SPACE, function (t, e) { return _this.onSpace(t, e); });
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.UP_ARROW, function (t, e) { return _this.onUp(t, e); });
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.PAGE_UP, function (t, e) { return _this.onPageUp(t, e); });
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.DOWN_ARROW, function (t, e) { return _this.onDown(t, e); });
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.PAGE_DOWN, function (t, e) { return _this.onPageDown(t, e); });
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.LEFT_ARROW, function (t, e) { return _this.onLeft(t, e); });
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.RIGHT_ARROW, function (t, e) { return _this.onRight(t, e); });
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.ESCAPE, function (t, e) { return _this.onEscape(t, e); });
            this.upKeyBindingDispatcher = new KeybindingDispatcher();
            this.upKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.ENTER, this.onEnter.bind(this));
            this.upKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.CTRLCMD_ENTER, this.onEnter.bind(this));
        }
        DefaultController.prototype.onMouseDown = function (tree, element, event, origin) {
            if (origin === void 0) { origin = 'mouse'; }
            if (this.options.clickBehavior === ClickBehavior.ON_MOUSE_DOWN && event.leftButton) {
                if (event.target) {
                    if (event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
                        return false; // Ignore event if target is a form input field (avoids browser specific issues)
                    }
                    if (dom.findParentWithClass(event.target, 'monaco-action-bar', 'row')) {
                        return false; // Ignore event if target is over an action bar of the row
                    }
                }
                // Propagate to onLeftClick now
                return this.onLeftClick(tree, element, event, origin);
            }
            return false;
        };
        DefaultController.prototype.onClick = function (tree, element, event) {
            var isMac = platform.isMacintosh;
            // A Ctrl click on the Mac is a context menu event
            if (isMac && event.ctrlKey) {
                event.preventDefault();
                event.stopPropagation();
                return false;
            }
            if (event.middleButton) {
                return false; // Give contents of the item a chance to handle this (e.g. open link in new tab)
            }
            if (event.target && event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
                return false; // Ignore event if target is a form input field (avoids browser specific issues)
            }
            if (this.options.clickBehavior === ClickBehavior.ON_MOUSE_DOWN && event.leftButton) {
                return false; // Already handled by onMouseDown
            }
            return this.onLeftClick(tree, element, event);
        };
        DefaultController.prototype.onLeftClick = function (tree, element, eventish, origin) {
            if (origin === void 0) { origin = 'mouse'; }
            var payload = { origin: origin, originalEvent: eventish };
            if (tree.getInput() === element) {
                tree.clearFocus(payload);
                tree.clearSelection(payload);
            }
            else {
                var isMouseDown = eventish && eventish.browserEvent && eventish.browserEvent.type === 'mousedown';
                if (!isMouseDown) {
                    eventish.preventDefault(); // we cannot preventDefault onMouseDown because this would break DND otherwise
                }
                eventish.stopPropagation();
                tree.DOMFocus();
                tree.setSelection([element], payload);
                tree.setFocus(element, payload);
                if (tree.isExpanded(element)) {
                    tree.collapse(element).done(null, errors.onUnexpectedError);
                }
                else {
                    tree.expand(element).done(null, errors.onUnexpectedError);
                }
            }
            return true;
        };
        DefaultController.prototype.onContextMenu = function (tree, element, event) {
            if (event.target && event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
                return false; // allow context menu on input fields
            }
            // Prevent native context menu from showing up
            if (event) {
                event.preventDefault();
                event.stopPropagation();
            }
            return false;
        };
        DefaultController.prototype.onTap = function (tree, element, event) {
            var target = event.initialTarget;
            if (target && target.tagName && target.tagName.toLowerCase() === 'input') {
                return false; // Ignore event if target is a form input field (avoids browser specific issues)
            }
            return this.onLeftClick(tree, element, event, 'touch');
        };
        DefaultController.prototype.onKeyDown = function (tree, event) {
            return this.onKey(this.downKeyBindingDispatcher, tree, event);
        };
        DefaultController.prototype.onKeyUp = function (tree, event) {
            return this.onKey(this.upKeyBindingDispatcher, tree, event);
        };
        DefaultController.prototype.onKey = function (bindings, tree, event) {
            var handler = bindings.dispatch(event.asKeybinding());
            if (handler) {
                if (handler(tree, event)) {
                    event.preventDefault();
                    event.stopPropagation();
                    return true;
                }
            }
            return false;
        };
        DefaultController.prototype.onUp = function (tree, event) {
            var payload = { origin: 'keyboard', originalEvent: event };
            if (tree.getHighlight()) {
                tree.clearHighlight(payload);
            }
            else {
                tree.focusPrevious(1, payload);
                tree.reveal(tree.getFocus()).done(null, errors.onUnexpectedError);
            }
            return true;
        };
        DefaultController.prototype.onPageUp = function (tree, event) {
            var payload = { origin: 'keyboard', originalEvent: event };
            if (tree.getHighlight()) {
                tree.clearHighlight(payload);
            }
            else {
                tree.focusPreviousPage(payload);
                tree.reveal(tree.getFocus()).done(null, errors.onUnexpectedError);
            }
            return true;
        };
        DefaultController.prototype.onDown = function (tree, event) {
            var payload = { origin: 'keyboard', originalEvent: event };
            if (tree.getHighlight()) {
                tree.clearHighlight(payload);
            }
            else {
                tree.focusNext(1, payload);
                tree.reveal(tree.getFocus()).done(null, errors.onUnexpectedError);
            }
            return true;
        };
        DefaultController.prototype.onPageDown = function (tree, event) {
            var payload = { origin: 'keyboard', originalEvent: event };
            if (tree.getHighlight()) {
                tree.clearHighlight(payload);
            }
            else {
                tree.focusNextPage(payload);
                tree.reveal(tree.getFocus()).done(null, errors.onUnexpectedError);
            }
            return true;
        };
        DefaultController.prototype.onLeft = function (tree, event) {
            var payload = { origin: 'keyboard', originalEvent: event };
            if (tree.getHighlight()) {
                tree.clearHighlight(payload);
            }
            else {
                var focus = tree.getFocus();
                tree.collapse(focus).done(function (didCollapse) {
                    if (focus && !didCollapse) {
                        tree.focusParent(payload);
                    }
                });
            }
            return true;
        };
        DefaultController.prototype.onRight = function (tree, event) {
            var payload = { origin: 'keyboard', originalEvent: event };
            if (tree.getHighlight()) {
                tree.clearHighlight(payload);
            }
            else {
                var focus = tree.getFocus();
                tree.expand(focus).done(null, errors.onUnexpectedError);
            }
            return true;
        };
        DefaultController.prototype.onEnter = function (tree, event) {
            var payload = { origin: 'keyboard', originalEvent: event };
            if (tree.getHighlight()) {
                return false;
            }
            var focus = tree.getFocus();
            if (focus) {
                tree.setSelection([focus], payload);
            }
            return true;
        };
        DefaultController.prototype.onSpace = function (tree, event) {
            if (tree.getHighlight()) {
                return false;
            }
            var focus = tree.getFocus();
            if (focus) {
                tree.toggleExpansion(focus);
            }
            return true;
        };
        DefaultController.prototype.onEscape = function (tree, event) {
            var payload = { origin: 'keyboard', originalEvent: event };
            if (tree.getHighlight()) {
                tree.clearHighlight(payload);
                return true;
            }
            if (tree.getSelection().length) {
                tree.clearSelection(payload);
                return true;
            }
            if (tree.getFocus()) {
                tree.clearFocus(payload);
                return true;
            }
            return false;
        };
        return DefaultController;
    }());
    exports.DefaultController = DefaultController;
    var DefaultDragAndDrop = (function () {
        function DefaultDragAndDrop() {
        }
        DefaultDragAndDrop.prototype.getDragURI = function (tree, element) {
            return null;
        };
        DefaultDragAndDrop.prototype.onDragStart = function (tree, data, originalEvent) {
            return;
        };
        DefaultDragAndDrop.prototype.onDragOver = function (tree, data, targetElement, originalEvent) {
            return null;
        };
        DefaultDragAndDrop.prototype.drop = function (tree, data, targetElement, originalEvent) {
            return;
        };
        return DefaultDragAndDrop;
    }());
    exports.DefaultDragAndDrop = DefaultDragAndDrop;
    var DefaultFilter = (function () {
        function DefaultFilter() {
        }
        DefaultFilter.prototype.isVisible = function (tree, element) {
            return true;
        };
        return DefaultFilter;
    }());
    exports.DefaultFilter = DefaultFilter;
    var DefaultSorter = (function () {
        function DefaultSorter() {
        }
        DefaultSorter.prototype.compare = function (tree, element, otherElement) {
            return 0;
        };
        return DefaultSorter;
    }());
    exports.DefaultSorter = DefaultSorter;
    var DefaultAccessibilityProvider = (function () {
        function DefaultAccessibilityProvider() {
        }
        DefaultAccessibilityProvider.prototype.getAriaLabel = function (tree, element) {
            return null;
        };
        return DefaultAccessibilityProvider;
    }());
    exports.DefaultAccessibilityProvider = DefaultAccessibilityProvider;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/parts/tree/browser/actionsRenderer", ["require", "exports", 'vs/base/browser/builder', 'vs/base/browser/dom', 'vs/base/common/events', 'vs/base/browser/ui/actionbar/actionbar', './treeDefaults', 'vs/css!./actionsRenderer'], function (require, exports, Builder, Dom, Events, ActionBar, TreeDefaults) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var $ = Builder.$;
    var ActionsRenderer = (function (_super) {
        __extends(ActionsRenderer, _super);
        function ActionsRenderer(opts) {
            _super.call(this);
            this.actionProvider = opts.actionProvider;
            this.actionRunner = opts.actionRunner;
        }
        ActionsRenderer.prototype.getHeight = function (tree, element) {
            return this.getContentHeight(tree, element);
        };
        ActionsRenderer.prototype.render = function (tree, element, container, previousCleanupFn) {
            var _this = this;
            try {
                Dom.clearNode(container);
            }
            catch (e) {
                if (!/The node to be removed is no longer a child of this node/.test(e.message)) {
                    throw e;
                }
            }
            if (previousCleanupFn) {
                previousCleanupFn(tree, element);
            }
            var $container = $(container).addClass('actions');
            var $subContainer = $('.sub-content').appendTo($container);
            var actionBar;
            var actionBarListener;
            if (this.actionProvider.hasActions(tree, element)) {
                $container.addClass('has-actions');
                actionBar = new ActionBar.ActionBar($('.primary-action-bar').appendTo($container), {
                    context: this.getActionContext(tree, element),
                    actionItemProvider: function (a) { return _this.actionProvider.getActionItem(tree, element, a); },
                    actionRunner: this.actionRunner
                });
                this.actionProvider.getActions(tree, element).then(function (actions) {
                    actionBar.push(actions, { icon: true, label: false });
                });
                actionBarListener = actionBar.addListener2(Events.EventType.RUN, function (event) {
                    if (event.error) {
                        _this.onError(event.error);
                    }
                });
            }
            else {
                $container.removeClass('has-actions');
            }
            var previousContentsCleanupFn = (previousCleanupFn ? previousCleanupFn[ActionsRenderer.CONTENTS_CLEANUP_FN_KEY] : ActionsRenderer.NO_OP) || ActionsRenderer.NO_OP;
            previousContentsCleanupFn(tree, element);
            var contentsCleanupFn = this.renderContents(tree, element, $subContainer.getHTMLElement(), null);
            var cleanupFn = function () {
                if (actionBarListener) {
                    actionBarListener.dispose();
                }
                if (actionBar) {
                    actionBar.dispose();
                }
                if (contentsCleanupFn) {
                    contentsCleanupFn(tree, element);
                }
            };
            cleanupFn[ActionsRenderer.CONTENTS_CLEANUP_FN_KEY] = contentsCleanupFn;
            return cleanupFn;
        };
        /* protected */ ActionsRenderer.prototype.getContentHeight = function (tree, element) {
            return 20;
        };
        /* protected */ ActionsRenderer.prototype.renderContents = function (tree, element, container, previousCleanupFn) {
            return null;
        };
        /* protected */ ActionsRenderer.prototype.getActionContext = function (tree, element) {
            return null;
        };
        /* protected */ ActionsRenderer.prototype.onError = function (error) {
            return;
        };
        ActionsRenderer.prototype.dispose = function () {
            this.actionProvider = null;
        };
        ActionsRenderer.CONTENTS_CLEANUP_FN_KEY = '__$ActionsRenderer.contentCleanupFn';
        ActionsRenderer.NO_OP = function () { };
        return ActionsRenderer;
    }(TreeDefaults.LegacyRenderer));
    exports.ActionsRenderer = ActionsRenderer;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/parts/tree/browser/treeView", ["require", "exports", 'vs/base/common/platform', 'vs/base/browser/browser', 'vs/base/common/winjs.base', 'vs/base/common/lifecycle', 'vs/base/browser/dom', 'vs/base/common/diff/diff', 'vs/base/browser/touch', 'vs/base/browser/mouseEvent', 'vs/base/browser/keyboardEvent', './treeDnd', 'vs/base/common/iterator', 'vs/base/browser/ui/scrollbar/scrollableElementImpl', 'vs/base/parts/tree/browser/treeViewModel', 'vs/base/parts/tree/browser/tree', 'vs/base/common/keyCodes'], function (require, exports, Platform, Browser, WinJS, Lifecycle, DOM, Diff, Touch, Mouse, Keyboard, dnd, iterator_1, ScrollableElementImpl, treeViewModel_1, _, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function getLastScrollTime(element) {
        var value = element.getAttribute('last-scroll-time');
        return value ? parseInt(value, 10) : 0;
    }
    function removeFromParent(element) {
        try {
            element.parentElement.removeChild(element);
        }
        catch (e) {
        }
    }
    var RowCache = (function () {
        function RowCache(context) {
            this.context = context;
            this._cache = { '': [] };
            this.scrollingRow = null;
        }
        RowCache.prototype.alloc = function (templateId) {
            var result = this.cache(templateId).pop();
            if (!result) {
                var content = document.createElement('div');
                content.className = 'content';
                var row = document.createElement('div');
                row.appendChild(content);
                result = {
                    element: row,
                    templateId: templateId,
                    templateData: this.context.renderer.renderTemplate(this.context.tree, templateId, content)
                };
            }
            return result;
        };
        RowCache.prototype.release = function (templateId, row) {
            var lastScrollTime = getLastScrollTime(row.element);
            if (!lastScrollTime) {
                removeFromParent(row.element);
                this.cache(templateId).push(row);
                return;
            }
            if (this.scrollingRow) {
                var lastKnownScrollTime = getLastScrollTime(this.scrollingRow.element);
                if (lastKnownScrollTime > lastScrollTime) {
                    removeFromParent(row.element);
                    this.cache(templateId).push(row);
                    return;
                }
                if (this.scrollingRow.element.parentElement) {
                    removeFromParent(this.scrollingRow.element);
                    DOM.removeClass(this.scrollingRow.element, 'scrolling');
                    this.cache(this.scrollingRow.templateId).push(this.scrollingRow);
                }
            }
            this.scrollingRow = row;
            DOM.addClass(this.scrollingRow.element, 'scrolling');
        };
        RowCache.prototype.cache = function (templateId) {
            return this._cache[templateId] || (this._cache[templateId] = []);
        };
        RowCache.prototype.garbageCollect = function () {
            var _this = this;
            if (this._cache) {
                Object.keys(this._cache).forEach(function (templateId) {
                    _this._cache[templateId].forEach(function (cachedRow) {
                        _this.context.renderer.disposeTemplate(_this.context.tree, templateId, cachedRow.templateData);
                        cachedRow.element = null;
                        cachedRow.templateData = null;
                    });
                    delete _this._cache[templateId];
                });
            }
            if (this.scrollingRow) {
                this.context.renderer.disposeTemplate(this.context.tree, this.scrollingRow.templateId, this.scrollingRow.templateData);
                this.scrollingRow = null;
            }
        };
        RowCache.prototype.dispose = function () {
            this.garbageCollect();
            this._cache = null;
            this.context = null;
        };
        return RowCache;
    }());
    exports.RowCache = RowCache;
    var ViewItem = (function () {
        function ViewItem(context, model) {
            var _this = this;
            this.context = context;
            this.model = model;
            this.id = this.model.id;
            this.row = null;
            this.top = 0;
            this.height = model.getHeight();
            this._styles = {};
            model.getAllTraits().forEach(function (t) { return _this._styles[t] = true; });
            if (model.isExpanded()) {
                this.addClass('expanded');
            }
        }
        Object.defineProperty(ViewItem.prototype, "expanded", {
            set: function (value) {
                value ? this.addClass('expanded') : this.removeClass('expanded');
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewItem.prototype, "loading", {
            set: function (value) {
                value ? this.addClass('loading') : this.removeClass('loading');
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewItem.prototype, "draggable", {
            get: function () {
                return this._draggable;
            },
            set: function (value) {
                this._draggable = value;
                this.render(true);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewItem.prototype, "dropTarget", {
            set: function (value) {
                value ? this.addClass('drop-target') : this.removeClass('drop-target');
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewItem.prototype, "element", {
            get: function () {
                return this.row && this.row.element;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewItem.prototype, "templateId", {
            get: function () {
                return this._templateId || (this._templateId = (this.context.renderer.getTemplateId && this.context.renderer.getTemplateId(this.context.tree, this.model.getElement())));
            },
            enumerable: true,
            configurable: true
        });
        ViewItem.prototype.addClass = function (name) {
            this._styles[name] = true;
            this.render(true);
        };
        ViewItem.prototype.removeClass = function (name) {
            delete this._styles[name]; // is this slow?
            this.render(true);
        };
        ViewItem.prototype.render = function (skipUserRender) {
            var _this = this;
            if (skipUserRender === void 0) { skipUserRender = false; }
            if (!this.model || !this.element) {
                return;
            }
            var classes = ['monaco-tree-row'];
            classes.push.apply(classes, Object.keys(this._styles));
            if (this.model.hasChildren()) {
                classes.push('has-children');
            }
            this.element.className = classes.join(' ');
            this.element.draggable = this.draggable;
            this.element.style.height = this.height + 'px';
            // ARIA
            this.element.setAttribute('role', 'treeitem');
            if (this.model.hasTrait('focused')) {
                var base64Id = btoa(this.model.id);
                var ariaLabel = this.context.accessibilityProvider.getAriaLabel(this.context.tree, this.model.getElement());
                this.element.setAttribute('aria-selected', 'true');
                this.element.setAttribute('id', base64Id);
                if (ariaLabel) {
                    this.element.setAttribute('aria-label', ariaLabel);
                }
                else {
                    this.element.setAttribute('aria-labelledby', base64Id); // force screen reader to compute label from children (helps NVDA at least)
                }
            }
            else {
                this.element.setAttribute('aria-selected', 'false');
                this.element.removeAttribute('id');
                this.element.removeAttribute('aria-label');
                this.element.removeAttribute('aria-labelledby');
            }
            if (this.model.hasChildren()) {
                this.element.setAttribute('aria-expanded', String(this.model.isExpanded()));
            }
            else {
                this.element.removeAttribute('aria-expanded');
            }
            this.element.setAttribute('aria-level', String(this.model.getDepth()));
            if (this.context.options.paddingOnRow) {
                this.element.style.paddingLeft = this.context.options.twistiePixels + ((this.model.getDepth() - 1) * this.context.options.indentPixels) + 'px';
            }
            else {
                this.element.style.paddingLeft = ((this.model.getDepth() - 1) * this.context.options.indentPixels) + 'px';
                this.row.element.firstElementChild.style.paddingLeft = this.context.options.twistiePixels + 'px';
            }
            var uri = this.context.dnd.getDragURI(this.context.tree, this.model.getElement());
            if (uri !== this.uri) {
                if (this.unbindDragStart) {
                    this.unbindDragStart.dispose();
                    this.unbindDragStart = null;
                }
                if (uri) {
                    this.uri = uri;
                    this.draggable = true;
                    this.unbindDragStart = DOM.addDisposableListener(this.element, 'dragstart', function (e) {
                        _this.onDragStart(e);
                    });
                }
                else {
                    this.uri = null;
                }
            }
            if (!skipUserRender) {
                this.context.renderer.renderElement(this.context.tree, this.model.getElement(), this.templateId, this.row.templateData);
            }
        };
        ViewItem.prototype.insertInDOM = function (container, afterElement) {
            if (!this.row) {
                this.row = this.context.cache.alloc(this.templateId);
                // used in reverse lookup from HTMLElement to Item
                this.element[TreeView.BINDING] = this;
            }
            if (this.element.parentElement) {
                return;
            }
            if (afterElement === null) {
                container.appendChild(this.element);
            }
            else {
                try {
                    container.insertBefore(this.element, afterElement);
                }
                catch (e) {
                    console.warn('Failed to locate previous tree element');
                    container.appendChild(this.element);
                }
            }
            this.render();
        };
        ViewItem.prototype.removeFromDOM = function () {
            if (!this.row) {
                return;
            }
            if (this.unbindDragStart) {
                this.unbindDragStart.dispose();
                this.unbindDragStart = null;
            }
            this.uri = null;
            this.element[TreeView.BINDING] = null;
            this.context.cache.release(this.templateId, this.row);
            this.row = null;
        };
        ViewItem.prototype.dispose = function () {
            this.row = null;
            this.model = null;
        };
        return ViewItem;
    }());
    exports.ViewItem = ViewItem;
    var RootViewItem = (function (_super) {
        __extends(RootViewItem, _super);
        function RootViewItem(context, model, wrapper) {
            _super.call(this, context, model);
            this.row = {
                element: wrapper,
                templateData: null,
                templateId: null
            };
        }
        RootViewItem.prototype.render = function () {
            if (!this.model || !this.element) {
                return;
            }
            var classes = ['monaco-tree-wrapper'];
            classes.push.apply(classes, Object.keys(this._styles));
            if (this.model.hasChildren()) {
                classes.push('has-children');
            }
            this.element.className = classes.join(' ');
        };
        RootViewItem.prototype.insertInDOM = function (container, afterElement) {
            // noop
        };
        RootViewItem.prototype.removeFromDOM = function () {
            // noop
        };
        return RootViewItem;
    }(ViewItem));
    function reactionEquals(one, other) {
        if (!one && !other) {
            return true;
        }
        else if (!one || !other) {
            return false;
        }
        else if (one.accept !== other.accept) {
            return false;
        }
        else if (one.bubble !== other.bubble) {
            return false;
        }
        else if (one.effect !== other.effect) {
            return false;
        }
        else {
            return true;
        }
    }
    var TreeView = (function (_super) {
        __extends(TreeView, _super);
        function TreeView(context, container) {
            var _this = this;
            _super.call(this);
            this.lastClickTimeStamp = 0;
            this.isRefreshing = false;
            this.refreshingPreviousChildrenIds = {};
            this.context = {
                dataSource: context.dataSource,
                renderer: context.renderer,
                controller: context.controller,
                dnd: context.dnd,
                filter: context.filter,
                sorter: context.sorter,
                tree: context.tree,
                accessibilityProvider: context.accessibilityProvider,
                options: context.options,
                cache: new RowCache(context)
            };
            this.modelListeners = [];
            this.viewListeners = [];
            this.dragAndDropListeners = [];
            this.model = null;
            this.items = {};
            this.domNode = document.createElement('div');
            this.domNode.className = 'monaco-tree no-focused-item';
            this.domNode.tabIndex = 0;
            // ARIA
            this.domNode.setAttribute('role', 'tree');
            if (this.context.options.ariaLabel) {
                this.domNode.setAttribute('aria-label', this.context.options.ariaLabel);
            }
            if (this.context.options.alwaysFocused) {
                DOM.addClass(this.domNode, 'focused');
            }
            if (this.context.options.bare) {
                DOM.addClass(this.domNode, 'bare');
            }
            if (!this.context.options.paddingOnRow) {
                DOM.addClass(this.domNode, 'no-row-padding');
            }
            this.wrapper = document.createElement('div');
            this.wrapper.className = 'monaco-tree-wrapper';
            this.scrollableElement = new ScrollableElementImpl.ScrollableElement(this.wrapper, {
                forbidTranslate3dUse: true,
                scrollable: this,
                horizontal: 'hidden',
                vertical: context.options.verticalScrollMode || 'auto',
                useShadows: context.options.useShadows,
                saveLastScrollTimeOnClassName: 'monaco-tree-row'
            });
            if (Browser.isIE11orEarlier) {
                this.wrapper.style.msTouchAction = 'none';
                this.wrapper.style.msContentZooming = 'none';
            }
            else {
                this.wrapperGesture = new Touch.Gesture(this.wrapper);
            }
            this.rowsContainer = document.createElement('div');
            this.rowsContainer.className = 'monaco-tree-rows';
            var focusTracker = DOM.trackFocus(this.domNode);
            focusTracker.addFocusListener(function () { return _this.onFocus(); });
            focusTracker.addBlurListener(function () { return _this.onBlur(); });
            this.viewListeners.push(focusTracker);
            this.viewListeners.push(DOM.addDisposableListener(this.domNode, 'keydown', function (e) { return _this.onKeyDown(e); }));
            this.viewListeners.push(DOM.addDisposableListener(this.domNode, 'keyup', function (e) { return _this.onKeyUp(e); }));
            this.viewListeners.push(DOM.addDisposableListener(this.domNode, 'mousedown', function (e) { return _this.onMouseDown(e); }));
            this.viewListeners.push(DOM.addDisposableListener(this.domNode, 'mouseup', function (e) { return _this.onMouseUp(e); }));
            this.viewListeners.push(DOM.addDisposableListener(this.wrapper, 'click', function (e) { return _this.onClick(e); }));
            this.viewListeners.push(DOM.addDisposableListener(this.domNode, 'contextmenu', function (e) { return _this.onContextMenu(e); }));
            this.viewListeners.push(DOM.addDisposableListener(this.wrapper, Touch.EventType.Tap, function (e) { return _this.onTap(e); }));
            this.viewListeners.push(DOM.addDisposableListener(this.wrapper, Touch.EventType.Change, function (e) { return _this.onTouchChange(e); }));
            if (Browser.isIE11orEarlier) {
                this.viewListeners.push(DOM.addDisposableListener(this.wrapper, 'MSPointerDown', function (e) { return _this.onMsPointerDown(e); }));
                this.viewListeners.push(DOM.addDisposableListener(this.wrapper, 'MSGestureTap', function (e) { return _this.onMsGestureTap(e); }));
                // these events come too fast, we throttle them
                this.viewListeners.push(DOM.addDisposableThrottledListener(this.wrapper, 'MSGestureChange', function (e) { return _this.onThrottledMsGestureChange(e); }, function (lastEvent, event) {
                    event.stopPropagation();
                    event.preventDefault();
                    var result = { translationY: event.translationY, translationX: event.translationX };
                    if (lastEvent) {
                        result.translationY += lastEvent.translationY;
                        result.translationX += lastEvent.translationX;
                    }
                    return result;
                }));
            }
            this.viewListeners.push(DOM.addDisposableListener(window, 'dragover', function (e) { return _this.onDragOver(e); }));
            this.viewListeners.push(DOM.addDisposableListener(window, 'drop', function (e) { return _this.onDrop(e); }));
            this.viewListeners.push(DOM.addDisposableListener(window, 'dragend', function (e) { return _this.onDragEnd(e); }));
            this.viewListeners.push(DOM.addDisposableListener(window, 'dragleave', function (e) { return _this.onDragOver(e); }));
            this.wrapper.appendChild(this.rowsContainer);
            this.domNode.appendChild(this.scrollableElement.getDomNode());
            container.appendChild(this.domNode);
            this._scrollTop = 0;
            this._viewHeight = 0;
            this.renderTop = 0;
            this.renderHeight = 0;
            this.didJustPressContextMenuKey = false;
            this.currentDropTarget = null;
            this.currentDropTargets = [];
            this.shouldInvalidateDropReaction = false;
            this.dragAndDropScrollInterval = null;
            this.dragAndDropScrollTimeout = null;
            this.onHiddenScrollTop = null;
            this.onRowsChanged();
            this.layout();
            this.setupMSGesture();
        }
        TreeView.prototype.createViewItem = function (item) {
            return new ViewItem(this.context, item);
        };
        TreeView.prototype.getHTMLElement = function () {
            return this.domNode;
        };
        TreeView.prototype.focus = function () {
            this.domNode.focus();
        };
        TreeView.prototype.isFocused = function () {
            return document.activeElement === this.domNode;
        };
        TreeView.prototype.blur = function () {
            this.domNode.blur();
        };
        TreeView.prototype.onVisible = function () {
            this.scrollTop = this.onHiddenScrollTop;
            this.onHiddenScrollTop = null;
            this.scrollableElement.onElementDimensions();
            this.scrollableElement.onElementInternalDimensions();
            this.setupMSGesture();
        };
        TreeView.prototype.setupMSGesture = function () {
            var _this = this;
            if (window.MSGesture) {
                this.msGesture = new MSGesture();
                setTimeout(function () { return _this.msGesture.target = _this.wrapper; }, 100); // TODO@joh, TODO@IETeam
            }
        };
        TreeView.prototype.onHidden = function () {
            this.onHiddenScrollTop = this.scrollTop;
        };
        TreeView.prototype.isTreeVisible = function () {
            return this.onHiddenScrollTop === null;
        };
        TreeView.prototype.layout = function (height) {
            if (!this.isTreeVisible()) {
                return;
            }
            this.viewHeight = height || DOM.getContentHeight(this.wrapper); // render
            this.scrollTop = this.scrollTop; // render
            this.scrollableElement.onElementDimensions();
            this.scrollableElement.onElementInternalDimensions();
        };
        TreeView.prototype.render = function (scrollTop, viewHeight) {
            var scrollBottom = scrollTop + viewHeight;
            var thisScrollBottom = this.scrollTop + this.viewHeight;
            var i;
            var stop;
            var renderTop = scrollTop;
            renderTop = Math.max(renderTop, 0);
            var renderBottom = scrollBottom;
            var thisRenderBottom = thisScrollBottom === 0 ? 0 : thisScrollBottom;
            // when view scrolls down, start rendering from the renderBottom
            for (i = this.indexAfter(renderBottom) - 1, stop = this.indexAt(Math.max(thisRenderBottom, renderTop)); i >= stop; i--) {
                this.insertItemInDOM(this.itemAtIndex(i));
            }
            // when view scrolls up, start rendering from either this.renderTop or renderBottom
            for (i = Math.min(this.indexAt(this.renderTop), this.indexAfter(renderBottom)) - 1, stop = this.indexAt(renderTop); i >= stop; i--) {
                this.insertItemInDOM(this.itemAtIndex(i));
            }
            // when view scrolls down, start unrendering from renderTop
            for (i = this.indexAt(this.renderTop), stop = Math.min(this.indexAt(renderTop), this.indexAfter(thisRenderBottom)); i < stop; i++) {
                this.removeItemFromDOM(this.itemAtIndex(i));
            }
            // when view scrolls up, start unrendering from either renderBottom this.renderTop
            for (i = Math.max(this.indexAfter(renderBottom), this.indexAt(this.renderTop)), stop = this.indexAfter(thisRenderBottom); i < stop; i++) {
                this.removeItemFromDOM(this.itemAtIndex(i));
            }
            var topItem = this.itemAtIndex(this.indexAt(renderTop));
            if (topItem) {
                this.rowsContainer.style.top = (topItem.top - renderTop) + 'px';
            }
            this.renderTop = renderTop;
            this.renderHeight = renderBottom - renderTop;
        };
        TreeView.prototype.setModel = function (newModel) {
            var _this = this;
            this.releaseModel();
            this.model = newModel;
            this.modelListeners.push(this.model.addBulkListener(function (e) { return _this.onModelEvents(e); }));
        };
        TreeView.prototype.onModelEvents = function (events) {
            var elementsToRefresh = [];
            for (var i = 0, len = events.length; i < len; i++) {
                var event = events[i];
                var data = event.getData();
                switch (event.getType()) {
                    case 'refreshing':
                        this.onRefreshing();
                        break;
                    case 'refreshed':
                        this.onRefreshed();
                        break;
                    case 'clearingInput':
                        this.onClearingInput(data);
                        break;
                    case 'setInput':
                        this.onSetInput(data);
                        break;
                    case 'item:childrenRefreshing':
                        this.onItemChildrenRefreshing(data);
                        break;
                    case 'item:childrenRefreshed':
                        this.onItemChildrenRefreshed(data);
                        break;
                    case 'item:refresh':
                        elementsToRefresh.push(data.item);
                        break;
                    case 'item:expanding':
                        this.onItemExpanding(data);
                        break;
                    case 'item:expanded':
                        this.onItemExpanded(data);
                        break;
                    case 'item:collapsing':
                        this.onItemCollapsing(data);
                        break;
                    case 'item:reveal':
                        this.onItemReveal(data);
                        break;
                    case 'item:addTrait':
                        this.onItemAddTrait(data);
                        break;
                    case 'item:removeTrait':
                        this.onItemRemoveTrait(data);
                        break;
                    case 'focus':
                        this.onModelFocusChange();
                        break;
                }
            }
            if (elementsToRefresh.length > 0) {
                this.onItemsRefresh(elementsToRefresh);
            }
        };
        TreeView.prototype.onRefreshing = function () {
            this.isRefreshing = true;
        };
        TreeView.prototype.onRefreshed = function () {
            this.isRefreshing = false;
            this.onRowsChanged();
        };
        TreeView.prototype.onRowsChanged = function (scrollTop) {
            if (scrollTop === void 0) { scrollTop = this.scrollTop; }
            if (this.isRefreshing) {
                return;
            }
            this.scrollTop = scrollTop;
            this.scrollableElement.onElementInternalDimensions();
        };
        TreeView.prototype.focusNextPage = function (eventPayload) {
            var _this = this;
            var lastPageIndex = this.indexAt(this.scrollTop + this.viewHeight);
            lastPageIndex = lastPageIndex === 0 ? 0 : lastPageIndex - 1;
            var lastPageElement = this.itemAtIndex(lastPageIndex).model.getElement();
            var currentlyFocusedElement = this.model.getFocus();
            if (currentlyFocusedElement !== lastPageElement) {
                this.model.setFocus(lastPageElement, eventPayload);
            }
            else {
                var previousScrollTop = this.scrollTop;
                this.scrollTop += this.viewHeight;
                if (this.scrollTop !== previousScrollTop) {
                    // Let the scroll event listener run
                    setTimeout(function () {
                        _this.focusNextPage(eventPayload);
                    }, 0);
                }
            }
        };
        TreeView.prototype.focusPreviousPage = function (eventPayload) {
            var _this = this;
            var firstPageIndex;
            if (this.scrollTop === 0) {
                firstPageIndex = this.indexAt(this.scrollTop);
            }
            else {
                firstPageIndex = this.indexAfter(this.scrollTop - 1);
            }
            var firstPageElement = this.itemAtIndex(firstPageIndex).model.getElement();
            var currentlyFocusedElement = this.model.getFocus();
            if (currentlyFocusedElement !== firstPageElement) {
                this.model.setFocus(firstPageElement, eventPayload);
            }
            else {
                var previousScrollTop = this.scrollTop;
                this.scrollTop -= this.viewHeight;
                if (this.scrollTop !== previousScrollTop) {
                    // Let the scroll event listener run
                    setTimeout(function () {
                        _this.focusPreviousPage(eventPayload);
                    }, 0);
                }
            }
        };
        Object.defineProperty(TreeView.prototype, "viewHeight", {
            get: function () {
                return this._viewHeight;
            },
            set: function (viewHeight) {
                this.render(this.scrollTop, viewHeight);
                this._viewHeight = viewHeight;
            },
            enumerable: true,
            configurable: true
        });
        // IScrollable
        TreeView.prototype.getScrollHeight = function () {
            return this.getTotalHeight();
        };
        TreeView.prototype.getScrollWidth = function () {
            return 0;
        };
        TreeView.prototype.getScrollLeft = function () {
            return 0;
        };
        TreeView.prototype.setScrollLeft = function (scrollLeft) {
            // noop
        };
        Object.defineProperty(TreeView.prototype, "scrollTop", {
            get: function () {
                return this._scrollTop;
            },
            set: function (scrollTop) {
                this.setScrollTop(scrollTop);
            },
            enumerable: true,
            configurable: true
        });
        TreeView.prototype.getScrollTop = function () {
            return this._scrollTop;
        };
        TreeView.prototype.setScrollTop = function (scrollTop) {
            scrollTop = Math.min(scrollTop, this.getTotalHeight() - this.viewHeight);
            scrollTop = Math.max(scrollTop, 0);
            this.render(scrollTop, this.viewHeight);
            this._scrollTop = scrollTop;
            this.emit('scroll', { vertical: true, horizontal: false });
        };
        TreeView.prototype.addScrollListener = function (callback) {
            return this.addListener2('scroll', callback);
        };
        TreeView.prototype.getScrollPosition = function () {
            var height = this.getTotalHeight() - this.viewHeight;
            return height <= 0 ? 0 : this.scrollTop / height;
        };
        TreeView.prototype.setScrollPosition = function (pos) {
            var height = this.getTotalHeight() - this.viewHeight;
            this.scrollTop = height * pos;
        };
        // Events
        TreeView.prototype.onClearingInput = function (e) {
            var item = e.item;
            if (item) {
                this.onRemoveItems(new iterator_1.MappedIterator(item.getNavigator(), function (item) { return item && item.id; }));
                this.onRowsChanged();
            }
        };
        TreeView.prototype.onSetInput = function (e) {
            this.context.cache.garbageCollect();
            this.inputItem = new RootViewItem(this.context, e.item, this.wrapper);
            this.emit('viewItem:create', { item: this.inputItem.model });
        };
        TreeView.prototype.onItemChildrenRefreshing = function (e) {
            var item = e.item;
            var viewItem = this.items[item.id];
            if (viewItem) {
                viewItem.loadingPromise = WinJS.TPromise.timeout(TreeView.LOADING_DECORATION_DELAY).then(function () {
                    viewItem.loadingPromise = null;
                    viewItem.loading = true;
                });
            }
            if (!e.isNested) {
                var childrenIds = [];
                var navigator = item.getNavigator();
                var childItem;
                while (childItem = navigator.next()) {
                    childrenIds.push(childItem.id);
                }
                this.refreshingPreviousChildrenIds[item.id] = childrenIds;
            }
        };
        TreeView.prototype.onItemChildrenRefreshed = function (e) {
            var _this = this;
            var item = e.item;
            var viewItem = this.items[item.id];
            if (viewItem) {
                if (viewItem.loadingPromise) {
                    viewItem.loadingPromise.cancel();
                    viewItem.loadingPromise = null;
                }
                viewItem.loading = false;
            }
            if (!e.isNested) {
                var previousChildrenIds = this.refreshingPreviousChildrenIds[item.id];
                var afterModelItems = [];
                var navigator = item.getNavigator();
                var childItem;
                while (childItem = navigator.next()) {
                    afterModelItems.push(childItem);
                }
                var skipDiff = Math.abs(previousChildrenIds.length - afterModelItems.length) > 1000;
                var diff = void 0;
                var doToInsertItemsAlreadyExist = void 0;
                if (!skipDiff) {
                    var lcs = new Diff.LcsDiff({
                        getLength: function () { return previousChildrenIds.length; },
                        getElementHash: function (i) { return previousChildrenIds[i]; }
                    }, {
                        getLength: function () { return afterModelItems.length; },
                        getElementHash: function (i) { return afterModelItems[i].id; }
                    }, null);
                    diff = lcs.ComputeDiff();
                    // this means that the result of the diff algorithm would result
                    // in inserting items that were already registered. this can only
                    // happen if the data provider returns bad ids OR if the sorting
                    // of the elements has changed
                    doToInsertItemsAlreadyExist = diff.some(function (d) {
                        if (d.modifiedLength > 0) {
                            for (var i = d.modifiedStart, len = d.modifiedStart + d.modifiedLength; i < len; i++) {
                                if (_this.items.hasOwnProperty(afterModelItems[i].id)) {
                                    return true;
                                }
                            }
                        }
                        return false;
                    });
                }
                // 50 is an optimization number, at some point we're better off
                // just replacing everything
                if (!skipDiff && !doToInsertItemsAlreadyExist && diff.length < 50) {
                    for (var i = 0, len = diff.length; i < len; i++) {
                        var diffChange = diff[i];
                        if (diffChange.originalLength > 0) {
                            this.onRemoveItems(new iterator_1.ArrayIterator(previousChildrenIds, diffChange.originalStart, diffChange.originalStart + diffChange.originalLength));
                        }
                        if (diffChange.modifiedLength > 0) {
                            var beforeItem = afterModelItems[diffChange.modifiedStart - 1] || item;
                            beforeItem = beforeItem.getDepth() > 0 ? beforeItem : null;
                            this.onInsertItems(new iterator_1.ArrayIterator(afterModelItems, diffChange.modifiedStart, diffChange.modifiedStart + diffChange.modifiedLength), beforeItem ? beforeItem.id : null);
                        }
                    }
                }
                else if (skipDiff || diff.length) {
                    this.onRemoveItems(new iterator_1.ArrayIterator(previousChildrenIds));
                    this.onInsertItems(new iterator_1.ArrayIterator(afterModelItems));
                }
                if (skipDiff || diff.length) {
                    this.onRowsChanged();
                }
            }
        };
        TreeView.prototype.onItemsRefresh = function (items) {
            var _this = this;
            this.onRefreshItemSet(items.filter(function (item) { return _this.items.hasOwnProperty(item.id); }));
            this.onRowsChanged();
        };
        TreeView.prototype.onItemExpanding = function (e) {
            var viewItem = this.items[e.item.id];
            if (viewItem) {
                viewItem.expanded = true;
            }
        };
        TreeView.prototype.onItemExpanded = function (e) {
            var item = e.item;
            var viewItem = this.items[item.id];
            if (viewItem) {
                viewItem.expanded = true;
                var height = this.onInsertItems(item.getNavigator(), item.id);
                var scrollTop = this.scrollTop;
                if (viewItem.top + viewItem.height <= this.scrollTop) {
                    scrollTop += height;
                }
                this.onRowsChanged(scrollTop);
            }
        };
        TreeView.prototype.onItemCollapsing = function (e) {
            var item = e.item;
            var viewItem = this.items[item.id];
            if (viewItem) {
                viewItem.expanded = false;
                this.onRemoveItems(new iterator_1.MappedIterator(item.getNavigator(), function (item) { return item && item.id; }));
                this.onRowsChanged();
            }
        };
        TreeView.prototype.onItemReveal = function (e) {
            var item = e.item;
            var relativeTop = e.relativeTop;
            var viewItem = this.items[item.id];
            if (viewItem) {
                if (relativeTop !== null) {
                    relativeTop = relativeTop < 0 ? 0 : relativeTop;
                    relativeTop = relativeTop > 1 ? 1 : relativeTop;
                    // y = mx + b
                    var m = viewItem.height - this.viewHeight;
                    this.scrollTop = m * relativeTop + viewItem.top;
                }
                else {
                    var viewItemBottom = viewItem.top + viewItem.height;
                    var wrapperBottom = this.scrollTop + this.viewHeight;
                    if (viewItem.top < this.scrollTop) {
                        this.scrollTop = viewItem.top;
                    }
                    else if (viewItemBottom >= wrapperBottom) {
                        this.scrollTop = viewItemBottom - this.viewHeight;
                    }
                }
            }
        };
        TreeView.prototype.onItemAddTrait = function (e) {
            var item = e.item;
            var trait = e.trait;
            var viewItem = this.items[item.id];
            if (viewItem) {
                viewItem.addClass(trait);
            }
            if (trait === 'highlighted') {
                DOM.addClass(this.domNode, trait);
                // Ugly Firefox fix: input fields can't be selected if parent nodes are draggable
                if (viewItem) {
                    this.highlightedItemWasDraggable = !!viewItem.draggable;
                    if (viewItem.draggable) {
                        viewItem.draggable = false;
                    }
                }
            }
        };
        TreeView.prototype.onItemRemoveTrait = function (e) {
            var item = e.item;
            var trait = e.trait;
            var viewItem = this.items[item.id];
            if (viewItem) {
                viewItem.removeClass(trait);
            }
            if (trait === 'highlighted') {
                DOM.removeClass(this.domNode, trait);
                // Ugly Firefox fix: input fields can't be selected if parent nodes are draggable
                if (this.highlightedItemWasDraggable) {
                    viewItem.draggable = true;
                }
                this.highlightedItemWasDraggable = false;
            }
        };
        TreeView.prototype.onModelFocusChange = function () {
            var focus = this.model && this.model.getFocus();
            DOM.toggleClass(this.domNode, 'no-focused-item', !focus);
            // ARIA
            if (focus) {
                this.domNode.setAttribute('aria-activedescendant', btoa(this.context.dataSource.getId(this.context.tree, focus)));
            }
            else {
                this.domNode.removeAttribute('aria-activedescendant');
            }
        };
        // HeightMap "events"
        TreeView.prototype.onInsertItem = function (item) {
            var _this = this;
            item.onDragStart = function (e) { _this.onDragStart(item, e); };
            item.needsRender = true;
            this.refreshViewItem(item);
            this.items[item.id] = item;
        };
        TreeView.prototype.onRefreshItem = function (item, needsRender) {
            if (needsRender === void 0) { needsRender = false; }
            item.needsRender = item.needsRender || needsRender;
            this.refreshViewItem(item);
        };
        TreeView.prototype.onRemoveItem = function (item) {
            this.removeItemFromDOM(item);
            item.dispose();
            this.emit('viewItem:dispose', { item: this.inputItem.model });
            delete this.items[item.id];
        };
        // ViewItem refresh
        TreeView.prototype.refreshViewItem = function (item) {
            item.render();
            if (this.shouldBeRendered(item)) {
                this.insertItemInDOM(item);
            }
            else {
                this.removeItemFromDOM(item);
            }
        };
        // DOM Events
        TreeView.prototype.onClick = function (e) {
            if (this.lastPointerType && this.lastPointerType !== 'mouse') {
                return;
            }
            var event = new Mouse.StandardMouseEvent(e);
            var item = this.getItemAround(event.target);
            if (!item) {
                return;
            }
            if (Browser.isIE10orLater && Date.now() - this.lastClickTimeStamp < 300) {
                // IE10+ doesn't set the detail property correctly. While IE10 simply
                // counts the number of clicks, IE11 reports always 1. To align with
                // other browser, we set the value to 2 if clicks events come in a 300ms
                // sequence.
                event.detail = 2;
            }
            this.lastClickTimeStamp = Date.now();
            this.context.controller.onClick(this.context.tree, item.model.getElement(), event);
        };
        TreeView.prototype.onMouseDown = function (e) {
            this.didJustPressContextMenuKey = false;
            if (!this.context.controller.onMouseDown) {
                return;
            }
            if (this.lastPointerType && this.lastPointerType !== 'mouse') {
                return;
            }
            var event = new Mouse.StandardMouseEvent(e);
            if (event.ctrlKey && Platform.isNative && Platform.isMacintosh) {
                return;
            }
            var item = this.getItemAround(event.target);
            if (!item) {
                return;
            }
            this.context.controller.onMouseDown(this.context.tree, item.model.getElement(), event);
        };
        TreeView.prototype.onMouseUp = function (e) {
            if (!this.context.controller.onMouseUp) {
                return;
            }
            if (this.lastPointerType && this.lastPointerType !== 'mouse') {
                return;
            }
            var event = new Mouse.StandardMouseEvent(e);
            if (event.ctrlKey && Platform.isNative && Platform.isMacintosh) {
                return;
            }
            var item = this.getItemAround(event.target);
            if (!item) {
                return;
            }
            this.context.controller.onMouseUp(this.context.tree, item.model.getElement(), event);
        };
        TreeView.prototype.onTap = function (e) {
            var item = this.getItemAround(e.initialTarget);
            if (!item) {
                return;
            }
            this.context.controller.onTap(this.context.tree, item.model.getElement(), e);
        };
        TreeView.prototype.onTouchChange = function (event) {
            event.preventDefault();
            event.stopPropagation();
            this.scrollTop -= event.translationY;
        };
        TreeView.prototype.onContextMenu = function (event) {
            var resultEvent;
            var element;
            if (event instanceof KeyboardEvent || this.didJustPressContextMenuKey) {
                this.didJustPressContextMenuKey = false;
                var keyboardEvent = new Keyboard.StandardKeyboardEvent(event);
                element = this.model.getFocus();
                if (!element) {
                    return;
                }
                var id = this.context.dataSource.getId(this.context.tree, element);
                var viewItem = this.items[id];
                var position = DOM.getDomNodePosition(viewItem.element);
                resultEvent = new _.KeyboardContextMenuEvent(position.left + position.width, position.top, keyboardEvent);
            }
            else {
                var mouseEvent = new Mouse.StandardMouseEvent(event);
                var item = this.getItemAround(mouseEvent.target);
                if (!item) {
                    return;
                }
                element = item.model.getElement();
                resultEvent = new _.MouseContextMenuEvent(mouseEvent);
            }
            this.context.controller.onContextMenu(this.context.tree, element, resultEvent);
        };
        TreeView.prototype.onKeyDown = function (e) {
            var event = new Keyboard.StandardKeyboardEvent(e);
            this.didJustPressContextMenuKey = event.keyCode === keyCodes_1.KeyCode.ContextMenu || (event.shiftKey && event.keyCode === keyCodes_1.KeyCode.F10);
            if (this.didJustPressContextMenuKey) {
                event.preventDefault();
                event.stopPropagation();
            }
            if (event.target && event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
                return; // Ignore event if target is a form input field (avoids browser specific issues)
            }
            this.context.controller.onKeyDown(this.context.tree, event);
        };
        TreeView.prototype.onKeyUp = function (e) {
            if (this.didJustPressContextMenuKey) {
                this.onContextMenu(e);
            }
            this.didJustPressContextMenuKey = false;
            this.context.controller.onKeyUp(this.context.tree, new Keyboard.StandardKeyboardEvent(e));
        };
        TreeView.prototype.onDragStart = function (item, e) {
            if (this.model.getHighlight()) {
                return;
            }
            var element = item.model.getElement();
            var selection = this.model.getSelection();
            var elements;
            if (selection.indexOf(element) > -1) {
                elements = selection;
            }
            else {
                elements = [element];
            }
            e.dataTransfer.effectAllowed = 'copyMove';
            e.dataTransfer.setData('URL', item.uri);
            if (e.dataTransfer.setDragImage) {
                if (elements.length === 1 && item.element) {
                    e.dataTransfer.setDragImage(item.element, e.offsetX || 6, e.offsetY || 6);
                }
                else if (elements.length > 1) {
                    var dragImage = document.createElement('div');
                    dragImage.className = 'monaco-tree-drag-image';
                    dragImage.textContent = '' + elements.length;
                    document.body.appendChild(dragImage);
                    e.dataTransfer.setDragImage(dragImage, -10, -10);
                    setTimeout(function () { return document.body.removeChild(dragImage); }, 0);
                }
            }
            this.currentDragAndDropData = new dnd.ElementsDragAndDropData(elements);
            TreeView.currentExternalDragAndDropData = new dnd.ExternalElementsDragAndDropData(elements);
            this.context.dnd.onDragStart(this.context.tree, this.currentDragAndDropData, new Mouse.DragMouseEvent(e));
        };
        TreeView.prototype.setupDragAndDropScrollInterval = function () {
            var _this = this;
            var viewTop = DOM.getTopLeftOffset(this.wrapper).top;
            if (!this.dragAndDropScrollInterval) {
                this.dragAndDropScrollInterval = window.setInterval(function () {
                    if (_this.dragAndDropMouseY === undefined) {
                        return;
                    }
                    var diff = _this.dragAndDropMouseY - viewTop;
                    var scrollDiff = 0;
                    var upperLimit = _this.viewHeight - 35;
                    if (diff < 35) {
                        scrollDiff = Math.max(-14, 0.2 * (diff - 35));
                    }
                    else if (diff > upperLimit) {
                        scrollDiff = Math.min(14, 0.2 * (diff - upperLimit));
                    }
                    _this.scrollTop += scrollDiff;
                }, 10);
                this.cancelDragAndDropScrollTimeout();
                this.dragAndDropScrollTimeout = window.setTimeout(function () {
                    _this.cancelDragAndDropScrollInterval();
                    _this.dragAndDropScrollTimeout = null;
                }, 1000);
            }
        };
        TreeView.prototype.cancelDragAndDropScrollInterval = function () {
            if (this.dragAndDropScrollInterval) {
                window.clearInterval(this.dragAndDropScrollInterval);
                this.dragAndDropScrollInterval = null;
            }
            this.cancelDragAndDropScrollTimeout();
        };
        TreeView.prototype.cancelDragAndDropScrollTimeout = function () {
            if (this.dragAndDropScrollTimeout) {
                window.clearTimeout(this.dragAndDropScrollTimeout);
                this.dragAndDropScrollTimeout = null;
            }
        };
        TreeView.prototype.onDragOver = function (e) {
            var _this = this;
            var event = new Mouse.DragMouseEvent(e);
            var viewItem = this.getItemAround(event.target);
            if (!viewItem) {
                // dragging outside of tree
                if (this.currentDropTarget) {
                    // clear previously hovered element feedback
                    this.currentDropTargets.forEach(function (i) { return i.dropTarget = false; });
                    this.currentDropTargets = [];
                    if (this.currentDropPromise) {
                        this.currentDropPromise.cancel();
                        this.currentDropPromise = null;
                    }
                }
                this.cancelDragAndDropScrollInterval();
                this.currentDropTarget = null;
                this.currentDropElement = null;
                this.dragAndDropMouseY = null;
                return false;
            }
            // dragging inside the tree
            this.setupDragAndDropScrollInterval();
            this.dragAndDropMouseY = event.posy;
            if (!this.currentDragAndDropData) {
                // just started dragging
                if (TreeView.currentExternalDragAndDropData) {
                    this.currentDragAndDropData = TreeView.currentExternalDragAndDropData;
                }
                else {
                    if (!event.dataTransfer.types) {
                        return false;
                    }
                    this.currentDragAndDropData = new dnd.DesktopDragAndDropData();
                }
            }
            this.currentDragAndDropData.update(event);
            var element;
            var item = viewItem.model;
            var reaction;
            // check the bubble up behavior
            do {
                element = item ? item.getElement() : this.model.getInput();
                reaction = this.context.dnd.onDragOver(this.context.tree, this.currentDragAndDropData, element, event);
                if (!reaction || reaction.bubble !== _.DragOverBubble.BUBBLE_UP) {
                    break;
                }
                item = item && item.parent;
            } while (item);
            if (!item) {
                this.currentDropElement = null;
                return false;
            }
            var canDrop = reaction && reaction.accept;
            if (canDrop) {
                this.currentDropElement = item.getElement();
                event.preventDefault();
                event.dataTransfer.dropEffect = reaction.effect === _.DragOverEffect.COPY ? 'copy' : 'move';
            }
            else {
                this.currentDropElement = null;
            }
            // item is the model item where drop() should be called
            // can be null
            var currentDropTarget = item.id === this.inputItem.id ? this.inputItem : this.items[item.id];
            if (this.shouldInvalidateDropReaction || this.currentDropTarget !== currentDropTarget || !reactionEquals(this.currentDropElementReaction, reaction)) {
                this.shouldInvalidateDropReaction = false;
                if (this.currentDropTarget) {
                    this.currentDropTargets.forEach(function (i) { return i.dropTarget = false; });
                    this.currentDropTargets = [];
                    if (this.currentDropPromise) {
                        this.currentDropPromise.cancel();
                        this.currentDropPromise = null;
                    }
                }
                this.currentDropTarget = currentDropTarget;
                this.currentDropElementReaction = reaction;
                if (canDrop) {
                    // setup hover feedback for drop target
                    if (this.currentDropTarget) {
                        this.currentDropTarget.dropTarget = true;
                        this.currentDropTargets.push(this.currentDropTarget);
                    }
                    if (reaction.bubble === _.DragOverBubble.BUBBLE_DOWN) {
                        var nav = item.getNavigator();
                        var child;
                        while (child = nav.next()) {
                            viewItem = this.items[child.id];
                            if (viewItem) {
                                viewItem.dropTarget = true;
                                this.currentDropTargets.push(viewItem);
                            }
                        }
                    }
                    this.currentDropPromise = WinJS.TPromise.timeout(500).then(function () {
                        return _this.context.tree.expand(_this.currentDropElement).then(function () {
                            _this.shouldInvalidateDropReaction = true;
                        });
                    });
                }
            }
            return true;
        };
        TreeView.prototype.onDrop = function (e) {
            if (this.currentDropElement) {
                var event = new Mouse.DragMouseEvent(e);
                event.preventDefault();
                this.currentDragAndDropData.update(event);
                this.context.dnd.drop(this.context.tree, this.currentDragAndDropData, this.currentDropElement, event);
                this.onDragEnd(e);
            }
            this.cancelDragAndDropScrollInterval();
        };
        TreeView.prototype.onDragEnd = function (e) {
            if (this.currentDropTarget) {
                this.currentDropTargets.forEach(function (i) { return i.dropTarget = false; });
                this.currentDropTargets = [];
            }
            if (this.currentDropPromise) {
                this.currentDropPromise.cancel();
                this.currentDropPromise = null;
            }
            this.cancelDragAndDropScrollInterval();
            this.currentDragAndDropData = null;
            TreeView.currentExternalDragAndDropData = null;
            this.currentDropElement = null;
            this.currentDropTarget = null;
            this.dragAndDropMouseY = null;
        };
        TreeView.prototype.onFocus = function () {
            if (!this.context.options.alwaysFocused) {
                DOM.addClass(this.domNode, 'focused');
            }
        };
        TreeView.prototype.onBlur = function () {
            if (!this.context.options.alwaysFocused) {
                DOM.removeClass(this.domNode, 'focused');
            }
            this.domNode.removeAttribute('aria-activedescendant'); // ARIA
        };
        // MS specific DOM Events
        TreeView.prototype.onMsPointerDown = function (event) {
            if (!this.msGesture) {
                return;
            }
            // Circumvent IE11 breaking change in e.pointerType & TypeScript's stale definitions
            var pointerType = event.pointerType;
            if (pointerType === (event.MSPOINTER_TYPE_MOUSE || 'mouse')) {
                this.lastPointerType = 'mouse';
                return;
            }
            else if (pointerType === (event.MSPOINTER_TYPE_TOUCH || 'touch')) {
                this.lastPointerType = 'touch';
            }
            else {
                return;
            }
            event.stopPropagation();
            event.preventDefault();
            this.msGesture.addPointer(event.pointerId);
        };
        TreeView.prototype.onThrottledMsGestureChange = function (event) {
            this.scrollTop -= event.translationY;
        };
        TreeView.prototype.onMsGestureTap = function (event) {
            event.initialTarget = document.elementFromPoint(event.clientX, event.clientY);
            this.onTap(event);
        };
        // DOM changes
        TreeView.prototype.insertItemInDOM = function (item) {
            var elementAfter = null;
            var itemAfter = this.itemAfter(item);
            if (itemAfter && itemAfter.element) {
                elementAfter = itemAfter.element;
            }
            item.insertInDOM(this.rowsContainer, elementAfter);
        };
        TreeView.prototype.removeItemFromDOM = function (item) {
            item.removeFromDOM();
        };
        // Helpers
        TreeView.prototype.shouldBeRendered = function (item) {
            return item.top < this.renderTop + this.renderHeight && item.top + item.height > this.renderTop;
        };
        TreeView.prototype.getItemAround = function (element) {
            var candidate = this.inputItem;
            do {
                if (element[TreeView.BINDING]) {
                    candidate = element[TreeView.BINDING];
                }
                if (element === this.wrapper || element === this.domNode) {
                    return candidate;
                }
                if (element === document.body) {
                    return null;
                }
            } while (element = element.parentElement);
        };
        // Cleanup
        TreeView.prototype.releaseModel = function () {
            if (this.model) {
                while (this.modelListeners.length) {
                    this.modelListeners.pop()();
                }
                this.model = null;
            }
        };
        TreeView.prototype.dispose = function () {
            // TODO@joao: improve
            this.scrollableElement.dispose();
            this.releaseModel();
            this.modelListeners = null;
            this.viewListeners = Lifecycle.disposeAll(this.viewListeners);
            if (this.domNode.parentNode) {
                this.domNode.parentNode.removeChild(this.domNode);
            }
            this.domNode = null;
            if (this.wrapperGesture) {
                this.wrapperGesture.dispose();
                this.wrapperGesture = null;
            }
            if (this.context.cache) {
                this.context.cache.dispose();
                this.context.cache = null;
            }
            _super.prototype.dispose.call(this);
        };
        TreeView.BINDING = 'monaco-tree-row';
        TreeView.LOADING_DECORATION_DELAY = 800;
        TreeView.currentExternalDragAndDropData = null;
        return TreeView;
    }(treeViewModel_1.HeightMap));
    exports.TreeView = TreeView;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/parts/tree/browser/treeImpl", ["require", "exports", 'vs/base/parts/tree/browser/treeDefaults', 'vs/base/common/eventEmitter', 'vs/base/parts/tree/browser/treeModel', './treeView', 'vs/base/common/iterator', 'vs/css!./tree'], function (require, exports, TreeDefaults, Events, Model, View, iterator_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var TreeContext = (function () {
        function TreeContext(tree, configuration, options) {
            if (options === void 0) { options = {}; }
            this.tree = tree;
            this.configuration = configuration;
            this.options = options;
            if (!configuration.dataSource) {
                throw new Error('You must provide a Data Source to the tree.');
            }
            this.dataSource = configuration.dataSource;
            this.renderer = configuration.renderer || new TreeDefaults.LegacyRenderer();
            this.controller = configuration.controller || new TreeDefaults.DefaultController();
            this.dnd = configuration.dnd || new TreeDefaults.DefaultDragAndDrop();
            this.filter = configuration.filter || new TreeDefaults.DefaultFilter();
            this.sorter = configuration.sorter || null;
            this.accessibilityProvider = configuration.accessibilityProvider || new TreeDefaults.DefaultAccessibilityProvider();
        }
        return TreeContext;
    }());
    exports.TreeContext = TreeContext;
    var Tree = (function (_super) {
        __extends(Tree, _super);
        function Tree(container, configuration, options) {
            if (options === void 0) { options = {}; }
            _super.call(this);
            this.container = container;
            this.configuration = configuration;
            this.options = options;
            this.options.twistiePixels = typeof this.options.twistiePixels === 'number' ? this.options.twistiePixels : 32;
            this.options.indentPixels = typeof this.options.indentPixels === 'number' ? this.options.indentPixels : 12;
            this.options.alwaysFocused = this.options.alwaysFocused === true ? true : false;
            this.options.bare = this.options.bare === true ? true : false;
            this.options.useShadows = this.options.useShadows === false ? false : true;
            this.options.paddingOnRow = this.options.paddingOnRow === false ? false : true;
            this.context = new TreeContext(this, configuration, options);
            this.model = new Model.TreeModel(this.context);
            this.view = new View.TreeView(this.context, this.container);
            this.view.setModel(this.model);
            this.addEmitter(this.model);
            this.addEmitter(this.view);
        }
        Tree.prototype.getHTMLElement = function () {
            return this.view.getHTMLElement();
        };
        Tree.prototype.layout = function (height) {
            this.view.layout(height);
        };
        Tree.prototype.DOMFocus = function () {
            this.view.focus();
        };
        Tree.prototype.isDOMFocused = function () {
            return this.view.isFocused();
        };
        Tree.prototype.DOMBlur = function () {
            this.view.blur();
        };
        Tree.prototype.onVisible = function () {
            this.view.onVisible();
        };
        Tree.prototype.onHidden = function () {
            this.view.onHidden();
        };
        Tree.prototype.setInput = function (element) {
            return this.model.setInput(element);
        };
        Tree.prototype.getInput = function () {
            return this.model.getInput();
        };
        Tree.prototype.refresh = function (element, recursive) {
            if (element === void 0) { element = null; }
            if (recursive === void 0) { recursive = true; }
            return this.model.refresh(element, recursive);
        };
        Tree.prototype.refreshAll = function (elements, recursive) {
            if (recursive === void 0) { recursive = true; }
            return this.model.refreshAll(elements, recursive);
        };
        Tree.prototype.expand = function (element) {
            return this.model.expand(element);
        };
        Tree.prototype.expandAll = function (elements) {
            return this.model.expandAll(elements);
        };
        Tree.prototype.collapse = function (element, recursive) {
            if (recursive === void 0) { recursive = false; }
            return this.model.collapse(element);
        };
        Tree.prototype.collapseAll = function (elements, recursive) {
            if (elements === void 0) { elements = null; }
            if (recursive === void 0) { recursive = false; }
            return this.model.collapseAll(elements, recursive);
        };
        Tree.prototype.toggleExpansion = function (element) {
            return this.model.toggleExpansion(element);
        };
        Tree.prototype.toggleExpansionAll = function (elements) {
            return this.model.toggleExpansionAll(elements);
        };
        Tree.prototype.isExpanded = function (element) {
            return this.model.isExpanded(element);
        };
        Tree.prototype.getExpandedElements = function () {
            return this.model.getExpandedElements();
        };
        Tree.prototype.reveal = function (element, relativeTop) {
            if (relativeTop === void 0) { relativeTop = null; }
            return this.model.reveal(element, relativeTop);
        };
        Tree.prototype.getScrollPosition = function () {
            return this.view.getScrollPosition();
        };
        Tree.prototype.setScrollPosition = function (pos) {
            this.view.setScrollPosition(pos);
        };
        Tree.prototype.getContentHeight = function () {
            return this.view.getTotalHeight();
        };
        Tree.prototype.setHighlight = function (element, eventPayload) {
            this.model.setHighlight(element, eventPayload);
        };
        Tree.prototype.getHighlight = function () {
            return this.model.getHighlight();
        };
        Tree.prototype.isHighlighted = function (element) {
            return this.model.isFocused(element);
        };
        Tree.prototype.clearHighlight = function (eventPayload) {
            this.model.setHighlight(null, eventPayload);
        };
        Tree.prototype.select = function (element, eventPayload) {
            this.model.select(element, eventPayload);
        };
        Tree.prototype.selectRange = function (fromElement, toElement, eventPayload) {
            this.model.selectRange(fromElement, toElement, eventPayload);
        };
        Tree.prototype.deselectRange = function (fromElement, toElement, eventPayload) {
            this.model.deselectRange(fromElement, toElement, eventPayload);
        };
        Tree.prototype.selectAll = function (elements, eventPayload) {
            this.model.selectAll(elements, eventPayload);
        };
        Tree.prototype.deselect = function (element, eventPayload) {
            this.model.deselect(element, eventPayload);
        };
        Tree.prototype.deselectAll = function (elements, eventPayload) {
            this.model.deselectAll(elements, eventPayload);
        };
        Tree.prototype.setSelection = function (elements, eventPayload) {
            this.model.setSelection(elements, eventPayload);
        };
        Tree.prototype.toggleSelection = function (element, eventPayload) {
            this.model.toggleSelection(element, eventPayload);
        };
        Tree.prototype.isSelected = function (element) {
            return this.model.isSelected(element);
        };
        Tree.prototype.getSelection = function () {
            return this.model.getSelection();
        };
        Tree.prototype.clearSelection = function (eventPayload) {
            this.model.setSelection([], eventPayload);
        };
        Tree.prototype.selectNext = function (count, clearSelection, eventPayload) {
            this.model.selectNext(count, clearSelection, eventPayload);
        };
        Tree.prototype.selectPrevious = function (count, clearSelection, eventPayload) {
            this.model.selectPrevious(count, clearSelection, eventPayload);
        };
        Tree.prototype.selectParent = function (clearSelection, eventPayload) {
            this.model.selectParent(clearSelection, eventPayload);
        };
        Tree.prototype.setFocus = function (element, eventPayload) {
            this.model.setFocus(element, eventPayload);
        };
        Tree.prototype.isFocused = function (element) {
            return this.model.isFocused(element);
        };
        Tree.prototype.getFocus = function () {
            return this.model.getFocus();
        };
        Tree.prototype.focusNext = function (count, eventPayload) {
            this.model.focusNext(count, eventPayload);
        };
        Tree.prototype.focusPrevious = function (count, eventPayload) {
            this.model.focusPrevious(count, eventPayload);
        };
        Tree.prototype.focusParent = function (eventPayload) {
            this.model.focusParent(eventPayload);
        };
        Tree.prototype.focusFirst = function (eventPayload) {
            this.model.focusFirst(eventPayload);
        };
        Tree.prototype.focusNth = function (index, eventPayload) {
            this.model.focusNth(index, eventPayload);
        };
        Tree.prototype.focusLast = function (eventPayload) {
            this.model.focusLast(eventPayload);
        };
        Tree.prototype.focusNextPage = function (eventPayload) {
            this.view.focusNextPage(eventPayload);
        };
        Tree.prototype.focusPreviousPage = function (eventPayload) {
            this.view.focusPreviousPage(eventPayload);
        };
        Tree.prototype.clearFocus = function (eventPayload) {
            this.model.setFocus(null, eventPayload);
        };
        Tree.prototype.addTraits = function (trait, elements) {
            this.model.addTraits(trait, elements);
        };
        Tree.prototype.removeTraits = function (trait, elements) {
            this.model.removeTraits(trait, elements);
        };
        Tree.prototype.toggleTrait = function (trait, element) {
            this.model.hasTrait(trait, element) ? this.model.removeTraits(trait, [element])
                : this.model.addTraits(trait, [element]);
        };
        Tree.prototype.hasTrait = function (trait, element) {
            return this.model.hasTrait(trait, element);
        };
        Tree.prototype.getNavigator = function () {
            return new iterator_1.MappedNavigator(this.model.getNavigator(), function (i) { return i && i.getElement(); });
        };
        Tree.prototype.dispose = function () {
            if (this.model !== null) {
                this.model.dispose();
                this.model = null;
            }
            if (this.view !== null) {
                this.view.dispose();
                this.view = null;
            }
            _super.prototype.dispose.call(this);
        };
        return Tree;
    }(Events.EventEmitter));
    exports.Tree = Tree;
});

define("vs/base/worker/defaultWorkerFactory", ["require", "exports", 'vs/base/common/flags', 'vs/base/common/lifecycle', 'vs/base/browser/dom'], function (require, exports, flags, lifecycle_1, dom) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function defaultGetWorkerUrl(workerId, label) {
        return require.toUrl('./' + workerId + '?' + encodeURIComponent(label));
    }
    var getWorkerUrl = flags.getCrossOriginWorkerScriptUrl || defaultGetWorkerUrl;
    /**
     * A worker that uses HTML5 web workers so that is has
     * its own global scope and its own thread.
     */
    var WebWorker = (function () {
        function WebWorker(moduleId, id, label, onMessageCallback) {
            this.id = id;
            this.worker = new Worker(getWorkerUrl('workerMain.js', label));
            this.postMessage(moduleId);
            this.worker.onmessage = function (ev) {
                onMessageCallback(ev.data);
            };
        }
        WebWorker.prototype.getId = function () {
            return this.id;
        };
        WebWorker.prototype.postMessage = function (msg) {
            this.worker.postMessage(msg);
        };
        WebWorker.prototype.dispose = function () {
            this.worker.terminate();
            this.worker = null;
        };
        return WebWorker;
    }());
    /**
     * A worker that runs in an iframe and therefore does have its
     * own global scope, but no own thread.
     */
    var FrameWorker = (function () {
        function FrameWorker(moduleId, id, onMessageCallback) {
            var _this = this;
            this.id = id;
            this._listeners = [];
            // Collect all messages sent to the worker until the iframe is loaded
            this.loaded = false;
            this.beforeLoadMessages = [];
            this.postMessage(moduleId);
            this.iframe = document.createElement('iframe');
            this.iframe.id = this.iframeId();
            this.iframe.src = require.toUrl('./workerMainCompatibility.html');
            this.iframe.frameborder = this.iframe.height = this.iframe.width = '0';
            this.iframe.style.display = 'none';
            this._listeners.push(dom.addDisposableListener(this.iframe, 'load', function () { return _this.onLoaded(); }));
            this.onMessage = function (ev) {
                onMessageCallback(ev.data);
            };
            this._listeners.push(dom.addDisposableListener(window, 'message', this.onMessage));
            document.body.appendChild(this.iframe);
        }
        FrameWorker.prototype.dispose = function () {
            this._listeners = lifecycle_1.disposeAll(this._listeners);
            window.removeEventListener('message', this.onMessage);
            window.frames[this.iframeId()].close();
        };
        FrameWorker.prototype.iframeId = function () {
            return 'worker_iframe_' + this.id;
        };
        FrameWorker.prototype.onLoaded = function () {
            this.loaded = true;
            while (this.beforeLoadMessages.length > 0) {
                this.postMessage(this.beforeLoadMessages.shift());
            }
        };
        FrameWorker.prototype.getId = function () {
            return this.id;
        };
        FrameWorker.prototype.postMessage = function (msg) {
            if (this.loaded === true) {
                var iframe = window.frames[this.iframeId()];
                if (iframe.postMessage) {
                    iframe.postMessage(msg, '*');
                }
                else {
                    iframe.contentWindow.postMessage(msg, '*');
                }
            }
            else {
                this.beforeLoadMessages.push(msg);
            }
        };
        return FrameWorker;
    }());
    var DefaultWorkerFactory = (function () {
        function DefaultWorkerFactory() {
        }
        DefaultWorkerFactory.prototype.create = function (moduleId, onMessageCallback) {
            var workerId = (++DefaultWorkerFactory.LAST_WORKER_ID);
            if (typeof WebWorker !== 'undefined') {
                return new WebWorker(moduleId, workerId, 'service' + workerId, onMessageCallback);
            }
            return new FrameWorker(moduleId, workerId, onMessageCallback);
        };
        DefaultWorkerFactory.LAST_WORKER_ID = 0;
        return DefaultWorkerFactory;
    }());
    exports.DefaultWorkerFactory = DefaultWorkerFactory;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/controller/mouseHandler", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/common/platform', 'vs/base/browser/browser', 'vs/base/browser/dom', 'vs/base/browser/globalMouseMoveMonitor', 'vs/base/browser/mouseEvent', 'vs/editor/common/core/position', 'vs/editor/common/core/selection', 'vs/editor/common/editorCommon', 'vs/editor/common/viewModel/viewEventHandler', 'vs/editor/browser/controller/mouseTarget', 'vs/base/common/async'], function (require, exports, lifecycle_1, platform, browser, dom, globalMouseMoveMonitor_1, mouseEvent_1, position_1, selection_1, editorCommon, viewEventHandler_1, mouseTarget_1, async_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Merges mouse events when mouse move events are throttled
     */
    function createMouseMoveEventMerger(mouseTargetFactory) {
        return function (lastEvent, currentEvent) {
            var r = new mouseEvent_1.StandardMouseEvent(currentEvent);
            var targetIsWidget = false;
            if (mouseTargetFactory) {
                targetIsWidget = mouseTargetFactory.mouseTargetIsWidget(r);
            }
            if (!targetIsWidget) {
                r.preventDefault();
            }
            return r;
        };
    }
    var EventGateKeeper = (function (_super) {
        __extends(EventGateKeeper, _super);
        function EventGateKeeper(destination, condition) {
            var _this = this;
            _super.call(this);
            this._destination = destination;
            this._condition = condition;
            this._retryTimer = this._register(new async_1.TimeoutTimer());
            this.handler = function (value) { return _this._handle(value); };
        }
        EventGateKeeper.prototype.dispose = function () {
            this._retryValue = null;
            _super.prototype.dispose.call(this);
        };
        EventGateKeeper.prototype._handle = function (value) {
            var _this = this;
            if (this._condition()) {
                this._retryTimer.cancel();
                this._retryValue = null;
                this._destination(value);
            }
            else {
                this._retryValue = value;
                this._retryTimer.setIfNotSet(function () {
                    var tmp = _this._retryValue;
                    _this._retryValue = null;
                    _this._handle(tmp);
                }, 10);
            }
        };
        return EventGateKeeper;
    }(lifecycle_1.Disposable));
    var MousePosition = (function () {
        function MousePosition(position, mouseColumn) {
            this.position = position;
            this.mouseColumn = mouseColumn;
        }
        return MousePosition;
    }());
    var MouseHandler = (function (_super) {
        __extends(MouseHandler, _super);
        function MouseHandler(context, viewController, viewHelper) {
            var _this = this;
            _super.call(this);
            this.context = context;
            this.viewController = viewController;
            this.viewHelper = viewHelper;
            this.mouseTargetFactory = new mouseTarget_1.MouseTargetFactory(this.context, viewHelper);
            this.listenersToRemove = [];
            this._mouseDownOperation = new MouseDownOperation(this.context, this.viewController, this.viewHelper, function (e, testEventTarget) { return _this._createMouseTarget(e, testEventTarget); }, function (e) { return _this._getMouseColumn(e); });
            this.toDispose = [];
            this.lastMouseLeaveTime = -1;
            this.listenersToRemove.push(dom.addDisposableListener(this.viewHelper.viewDomNode, 'contextmenu', function (e) { return _this._onContextMenu(e, true); }));
            this._mouseMoveEventHandler = new EventGateKeeper(function (e) { return _this._onMouseMove(e); }, function () { return !_this.viewHelper.isDirty(); });
            this.toDispose.push(this._mouseMoveEventHandler);
            this.listenersToRemove.push(dom.addDisposableThrottledListener(this.viewHelper.viewDomNode, 'mousemove', this._mouseMoveEventHandler.handler, createMouseMoveEventMerger(this.mouseTargetFactory), MouseHandler.MOUSE_MOVE_MINIMUM_TIME));
            this.listenersToRemove.push(dom.addDisposableListener(this.viewHelper.viewDomNode, 'mouseup', function (e) { return _this._onMouseUp(e); }));
            this.listenersToRemove.push(dom.addDisposableNonBubblingMouseOutListener(this.viewHelper.viewDomNode, function (e) { return _this._onMouseLeave(e); }));
            this.listenersToRemove.push(dom.addDisposableListener(this.viewHelper.viewDomNode, 'mousedown', function (e) { return _this._onMouseDown(e); }));
            this.context.addEventHandler(this);
        }
        MouseHandler.prototype.dispose = function () {
            this.context.removeEventHandler(this);
            this.listenersToRemove = lifecycle_1.disposeAll(this.listenersToRemove);
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            this._mouseDownOperation.dispose();
        };
        MouseHandler.prototype.onLayoutChanged = function (layoutInfo) {
            this._layoutInfo = layoutInfo;
            return false;
        };
        MouseHandler.prototype.onScrollChanged = function (e) {
            this._mouseDownOperation.onScrollChanged();
            return false;
        };
        MouseHandler.prototype.onCursorSelectionChanged = function (e) {
            this._mouseDownOperation.onCursorSelectionChanged(e);
            return false;
        };
        // --- end event handlers
        MouseHandler.prototype._createMouseTarget = function (e, testEventTarget) {
            var editorContent = dom.getDomNodePosition(this.viewHelper.viewDomNode);
            return this.mouseTargetFactory.createMouseTarget(this._layoutInfo, editorContent, e, testEventTarget);
        };
        MouseHandler.prototype._getMouseColumn = function (e) {
            var editorContent = dom.getDomNodePosition(this.viewHelper.viewDomNode);
            return this.mouseTargetFactory.getMouseColumn(this._layoutInfo, editorContent, e);
        };
        MouseHandler.prototype._onContextMenu = function (rawEvent, testEventTarget) {
            var e = new mouseEvent_1.StandardMouseEvent(rawEvent);
            var t = this._createMouseTarget(e, testEventTarget);
            var mouseEvent = {
                event: e,
                target: t
            };
            this.viewController.emitContextMenu(mouseEvent);
        };
        MouseHandler.prototype._onMouseMove = function (e) {
            if (this._mouseDownOperation.isActive()) {
                // In selection/drag operation
                return;
            }
            var actualMouseMoveTime = e.timestamp;
            if (actualMouseMoveTime < this.lastMouseLeaveTime) {
                // Due to throttling, this event occured before the mouse left the editor, therefore ignore it.
                return;
            }
            var t = this._createMouseTarget(e, true);
            var mouseEvent = {
                event: e,
                target: t
            };
            this.viewController.emitMouseMove(mouseEvent);
        };
        MouseHandler.prototype._onMouseLeave = function (rawEvent) {
            this.lastMouseLeaveTime = (new Date()).getTime();
            var mouseEvent = {
                event: new mouseEvent_1.StandardMouseEvent(rawEvent),
                target: null
            };
            this.viewController.emitMouseLeave(mouseEvent);
        };
        MouseHandler.prototype._onMouseUp = function (rawEvent) {
            var e = new mouseEvent_1.StandardMouseEvent(rawEvent);
            var t = this._createMouseTarget(e, true);
            var mouseEvent = {
                event: e,
                target: t
            };
            this.viewController.emitMouseUp(mouseEvent);
        };
        MouseHandler.prototype._onMouseDown = function (rawEvent) {
            var _this = this;
            var e = new mouseEvent_1.StandardMouseEvent(rawEvent);
            var t = this._createMouseTarget(e, true);
            var targetIsContent = (t.type === editorCommon.MouseTargetType.CONTENT_TEXT || t.type === editorCommon.MouseTargetType.CONTENT_EMPTY);
            var targetIsGutter = (t.type === editorCommon.MouseTargetType.GUTTER_GLYPH_MARGIN || t.type === editorCommon.MouseTargetType.GUTTER_LINE_NUMBERS || t.type === editorCommon.MouseTargetType.GUTTER_LINE_DECORATIONS);
            var targetIsLineNumbers = (t.type === editorCommon.MouseTargetType.GUTTER_LINE_NUMBERS);
            var selectOnLineNumbers = this.context.configuration.editor.selectOnLineNumbers;
            var targetIsViewZone = (t.type === editorCommon.MouseTargetType.CONTENT_VIEW_ZONE || t.type === editorCommon.MouseTargetType.GUTTER_VIEW_ZONE);
            var shouldHandle = e.leftButton;
            if (platform.isMacintosh && e.ctrlKey) {
                shouldHandle = false;
            }
            if (shouldHandle && (targetIsContent || (targetIsLineNumbers && selectOnLineNumbers))) {
                if (browser.isIE11orEarlier) {
                    // IE does not want to focus when coming in from the browser's address bar
                    if (e.browserEvent.fromElement) {
                        e.preventDefault();
                        this.viewHelper.focusTextArea();
                    }
                    else {
                        // TODO@Alex -> cancel this if focus is lost
                        setTimeout(function () {
                            _this.viewHelper.focusTextArea();
                        });
                    }
                }
                else {
                    e.preventDefault();
                    this.viewHelper.focusTextArea();
                }
                this._mouseDownOperation.start(t.type, e);
            }
            else if (targetIsGutter) {
                // Do not steal focus
                e.preventDefault();
            }
            else if (targetIsViewZone) {
                var viewZoneData = t.detail;
                if (this.viewHelper.shouldSuppressMouseDownOnViewZone(viewZoneData.viewZoneId)) {
                    e.preventDefault();
                }
            }
            var mouseEvent = {
                event: e,
                target: t
            };
            this.viewController.emitMouseDown(mouseEvent);
        };
        MouseHandler.MOUSE_MOVE_MINIMUM_TIME = 100; // ms
        return MouseHandler;
    }(viewEventHandler_1.ViewEventHandler));
    exports.MouseHandler = MouseHandler;
    var MouseDownOperation = (function (_super) {
        __extends(MouseDownOperation, _super);
        function MouseDownOperation(context, viewController, viewHelper, createMouseTarget, getMouseColumn) {
            var _this = this;
            _super.call(this);
            this._context = context;
            this._viewController = viewController;
            this._viewHelper = viewHelper;
            this._createMouseTarget = createMouseTarget;
            this._getMouseColumn = getMouseColumn;
            this._currentSelection = selection_1.Selection.createSelection(1, 1, 1, 1);
            this._mouseState = new MouseDownState();
            this._onScrollTimeout = this._register(new async_1.TimeoutTimer());
            this._isActive = false;
            this._lastMouseEvent = null;
            this._mouseMoveMonitor = this._register(new globalMouseMoveMonitor_1.GlobalMouseMoveMonitor());
            this._mouseDownThenMoveEventHandler = this._register(new EventGateKeeper(function (e) { return _this._onMouseDownThenMove(e); }, function () { return !_this._viewHelper.isDirty(); }));
        }
        MouseDownOperation.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
        };
        MouseDownOperation.prototype.isActive = function () {
            return this._isActive;
        };
        MouseDownOperation.prototype._onMouseDownThenMove = function (e) {
            this._lastMouseEvent = e;
            this._mouseState.setModifiers(e);
            var position = this._findMousePosition(e, true);
            if (!position) {
                // Ignoring because position is unknown
                return;
            }
            this._dispatchMouse(position, true);
        };
        MouseDownOperation.prototype.start = function (targetType, e) {
            var _this = this;
            this._lastMouseEvent = e;
            this._mouseState.setStartedOnLineNumbers(targetType === editorCommon.MouseTargetType.GUTTER_LINE_NUMBERS);
            this._mouseState.setModifiers(e);
            var position = this._findMousePosition(e, true);
            if (!position) {
                // Ignoring because position is unknown
                return;
            }
            this._mouseState.trySetCount(e.detail, position.position);
            // Overwrite the detail of the MouseEvent, as it will be sent out in an event and contributions might rely on it.
            e.detail = this._mouseState.count;
            this._dispatchMouse(position, e.shiftKey);
            if (!this._isActive) {
                this._isActive = true;
                this._mouseMoveMonitor.startMonitoring(createMouseMoveEventMerger(null), this._mouseDownThenMoveEventHandler.handler, function () { return _this._stop(); });
            }
        };
        MouseDownOperation.prototype._stop = function () {
            this._isActive = false;
            this._onScrollTimeout.cancel();
        };
        MouseDownOperation.prototype.onScrollChanged = function () {
            var _this = this;
            if (!this._isActive) {
                return;
            }
            this._onScrollTimeout.setIfNotSet(function () {
                var position = _this._findMousePosition(_this._lastMouseEvent, false);
                if (!position) {
                    // Ignoring because position is unknown
                    return;
                }
                _this._dispatchMouse(position, true);
            }, 10);
        };
        MouseDownOperation.prototype.onCursorSelectionChanged = function (e) {
            this._currentSelection = e.selection;
        };
        // private _getMouseColumn(e: ISimplifiedMouseEvent)
        MouseDownOperation.prototype._getPositionOutsideEditor = function (e) {
            var editorContent = dom.getDomNodePosition(this._viewHelper.viewDomNode);
            var mouseColumn = this._getMouseColumn(e);
            if (e.posy < editorContent.top) {
                var aboveLineNumber = this._viewHelper.getLineNumberAtVerticalOffset(Math.max(this._viewHelper.getScrollTop() - (editorContent.top - e.posy), 0));
                return new MousePosition(new position_1.Position(aboveLineNumber, 1), mouseColumn);
            }
            if (e.posy > editorContent.top + editorContent.height) {
                var belowLineNumber = this._viewHelper.getLineNumberAtVerticalOffset(this._viewHelper.getScrollTop() + (e.posy - editorContent.top));
                return new MousePosition(new position_1.Position(belowLineNumber, this._context.model.getLineMaxColumn(belowLineNumber)), mouseColumn);
            }
            var possibleLineNumber = this._viewHelper.getLineNumberAtVerticalOffset(this._viewHelper.getScrollTop() + (e.posy - editorContent.top));
            if (e.posx < editorContent.left) {
                return new MousePosition(new position_1.Position(possibleLineNumber, 1), mouseColumn);
            }
            if (e.posx > editorContent.left + editorContent.width) {
                return new MousePosition(new position_1.Position(possibleLineNumber, this._context.model.getLineMaxColumn(possibleLineNumber)), mouseColumn);
            }
            return null;
        };
        MouseDownOperation.prototype._findMousePosition = function (e, testEventTarget) {
            var positionOutsideEditor = this._getPositionOutsideEditor(e);
            if (positionOutsideEditor) {
                return positionOutsideEditor;
            }
            var t = this._createMouseTarget(e, testEventTarget);
            var hintedPosition = t.position;
            if (!hintedPosition) {
                return null;
            }
            if (t.type === editorCommon.MouseTargetType.CONTENT_VIEW_ZONE || t.type === editorCommon.MouseTargetType.GUTTER_VIEW_ZONE) {
                // Force position on view zones to go above or below depending on where selection started from
                var selectionStart = new position_1.Position(this._currentSelection.selectionStartLineNumber, this._currentSelection.selectionStartColumn);
                var viewZoneData = t.detail;
                var positionBefore = viewZoneData.positionBefore;
                var positionAfter = viewZoneData.positionAfter;
                if (positionBefore && positionAfter) {
                    if (positionBefore.isBefore(selectionStart)) {
                        return new MousePosition(positionBefore, t.mouseColumn);
                    }
                    else {
                        return new MousePosition(positionAfter, t.mouseColumn);
                    }
                }
            }
            return new MousePosition(hintedPosition, t.mouseColumn);
        };
        MouseDownOperation.prototype._dispatchMouse = function (position, inSelectionMode) {
            this._viewController.dispatchMouse({
                position: position.position,
                mouseColumn: position.mouseColumn,
                startedOnLineNumbers: this._mouseState.startedOnLineNumbers,
                inSelectionMode: inSelectionMode,
                mouseDownCount: this._mouseState.count,
                altKey: this._mouseState.altKey,
                ctrlKey: this._mouseState.ctrlKey,
                metaKey: this._mouseState.metaKey,
                shiftKey: this._mouseState.shiftKey,
            });
        };
        return MouseDownOperation;
    }(lifecycle_1.Disposable));
    var MouseDownState = (function () {
        function MouseDownState() {
            this._altKey = false;
            this._ctrlKey = false;
            this._metaKey = false;
            this._shiftKey = false;
            this._startedOnLineNumbers = false;
            this._lastMouseDownPosition = null;
            this._lastMouseDownPositionEqualCount = 0;
            this._lastMouseDownCount = 0;
            this._lastSetMouseDownCountTime = 0;
        }
        Object.defineProperty(MouseDownState.prototype, "altKey", {
            get: function () { return this._altKey; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MouseDownState.prototype, "ctrlKey", {
            get: function () { return this._ctrlKey; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MouseDownState.prototype, "metaKey", {
            get: function () { return this._metaKey; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MouseDownState.prototype, "shiftKey", {
            get: function () { return this._shiftKey; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MouseDownState.prototype, "startedOnLineNumbers", {
            get: function () { return this._startedOnLineNumbers; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MouseDownState.prototype, "count", {
            get: function () {
                return this._lastMouseDownCount;
            },
            enumerable: true,
            configurable: true
        });
        MouseDownState.prototype.setModifiers = function (source) {
            this._altKey = source.altKey;
            this._ctrlKey = source.ctrlKey;
            this._metaKey = source.metaKey;
            this._shiftKey = source.shiftKey;
        };
        MouseDownState.prototype.setStartedOnLineNumbers = function (startedOnLineNumbers) {
            this._startedOnLineNumbers = startedOnLineNumbers;
        };
        MouseDownState.prototype.trySetCount = function (setMouseDownCount, newMouseDownPosition) {
            // a. Invalidate multiple clicking if too much time has passed (will be hit by IE because the detail field of mouse events contains garbage in IE10)
            var currentTime = (new Date()).getTime();
            if (currentTime - this._lastSetMouseDownCountTime > MouseDownState.CLEAR_MOUSE_DOWN_COUNT_TIME) {
                setMouseDownCount = 1;
            }
            this._lastSetMouseDownCountTime = currentTime;
            // b. Ensure that we don't jump from single click to triple click in one go (will be hit by IE because the detail field of mouse events contains garbage in IE10)
            if (setMouseDownCount > this._lastMouseDownCount + 1) {
                setMouseDownCount = this._lastMouseDownCount + 1;
            }
            // c. Invalidate multiple clicking if the logical position is different
            if (this._lastMouseDownPosition && this._lastMouseDownPosition.equals(newMouseDownPosition)) {
                this._lastMouseDownPositionEqualCount++;
            }
            else {
                this._lastMouseDownPositionEqualCount = 1;
            }
            this._lastMouseDownPosition = newMouseDownPosition;
            // Finally set the lastMouseDownCount
            this._lastMouseDownCount = Math.min(setMouseDownCount, this._lastMouseDownPositionEqualCount);
        };
        MouseDownState.CLEAR_MOUSE_DOWN_COUNT_TIME = 400; // ms
        return MouseDownState;
    }());
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/controller/pointerHandler", ["require", "exports", 'vs/base/browser/dom', 'vs/base/browser/mouseEvent', 'vs/base/browser/touch', 'vs/editor/browser/controller/mouseHandler'], function (require, exports, dom, mouseEvent_1, touch_1, mouseHandler_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var gestureChangeEventMerger = function (lastEvent, currentEvent) {
        var r = {
            translationY: currentEvent.translationY,
            translationX: currentEvent.translationX
        };
        if (lastEvent) {
            r.translationY += lastEvent.translationY;
            r.translationX += lastEvent.translationX;
        }
        return r;
    };
    /**
     * Basically IE10 and IE11
     */
    var MsPointerHandler = (function (_super) {
        __extends(MsPointerHandler, _super);
        function MsPointerHandler(context, viewController, viewHelper) {
            var _this = this;
            _super.call(this, context, viewController, viewHelper);
            this.viewHelper.linesContentDomNode.style.msTouchAction = 'none';
            this.viewHelper.linesContentDomNode.style.msContentZooming = 'none';
            // TODO@Alex -> this expects that the view is added in 100 ms, might not be the case
            // This handler should be added when the dom node is in the dom tree
            this._installGestureHandlerTimeout = window.setTimeout(function () {
                _this._installGestureHandlerTimeout = -1;
                if (window.MSGesture) {
                    var touchGesture = new MSGesture();
                    var penGesture = new MSGesture();
                    touchGesture.target = _this.viewHelper.linesContentDomNode;
                    penGesture.target = _this.viewHelper.linesContentDomNode;
                    _this.viewHelper.linesContentDomNode.addEventListener('MSPointerDown', function (e) {
                        // Circumvent IE11 breaking change in e.pointerType & TypeScript's stale definitions
                        var pointerType = e.pointerType;
                        if (pointerType === (e.MSPOINTER_TYPE_MOUSE || 'mouse')) {
                            _this._lastPointerType = 'mouse';
                            return;
                        }
                        else if (pointerType === (e.MSPOINTER_TYPE_TOUCH || 'touch')) {
                            _this._lastPointerType = 'touch';
                            touchGesture.addPointer(e.pointerId);
                        }
                        else {
                            _this._lastPointerType = 'pen';
                            penGesture.addPointer(e.pointerId);
                        }
                    });
                    _this.listenersToRemove.push(dom.addDisposableThrottledListener(_this.viewHelper.linesContentDomNode, 'MSGestureChange', function (e) { return _this._onGestureChange(e); }, gestureChangeEventMerger));
                    _this.listenersToRemove.push(dom.addDisposableListener(_this.viewHelper.linesContentDomNode, 'MSGestureTap', function (e) { return _this._onCaptureGestureTap(e); }, true));
                }
            }, 100);
            this._lastPointerType = 'mouse';
        }
        MsPointerHandler.prototype._onMouseDown = function (e) {
            if (this._lastPointerType === 'mouse') {
                _super.prototype._onMouseDown.call(this, e);
            }
        };
        MsPointerHandler.prototype._onCaptureGestureTap = function (rawEvent) {
            var _this = this;
            var e = new mouseEvent_1.StandardMouseEvent(rawEvent);
            var t = this._createMouseTarget(e, false);
            if (t.position) {
                this.viewController.moveTo('mouse', t.position);
            }
            // IE does not want to focus when coming in from the browser's address bar
            if (e.browserEvent.fromElement) {
                e.preventDefault();
                this.viewHelper.focusTextArea();
            }
            else {
                // TODO@Alex -> cancel this is focus is lost
                setTimeout(function () {
                    _this.viewHelper.focusTextArea();
                });
            }
        };
        MsPointerHandler.prototype._onGestureChange = function (e) {
            this.viewHelper.setScrollTop(this.viewHelper.getScrollTop() - e.translationY);
            this.viewHelper.setScrollLeft(this.viewHelper.getScrollLeft() - e.translationX);
        };
        MsPointerHandler.prototype.dispose = function () {
            window.clearTimeout(this._installGestureHandlerTimeout);
            _super.prototype.dispose.call(this);
        };
        return MsPointerHandler;
    }(mouseHandler_1.MouseHandler));
    /**
     * Basically Edge but should be modified to handle any pointerEnabled, even without support of MSGesture
     */
    var StandardPointerHandler = (function (_super) {
        __extends(StandardPointerHandler, _super);
        function StandardPointerHandler(context, viewController, viewHelper) {
            var _this = this;
            _super.call(this, context, viewController, viewHelper);
            this.viewHelper.linesContentDomNode.style.touchAction = 'none';
            // TODO@Alex -> this expects that the view is added in 100 ms, might not be the case
            // This handler should be added when the dom node is in the dom tree
            this._installGestureHandlerTimeout = window.setTimeout(function () {
                _this._installGestureHandlerTimeout = -1;
                // TODO@Alex: replace the usage of MSGesture here with something that works across all browsers
                if (window.MSGesture) {
                    var touchGesture = new MSGesture();
                    var penGesture = new MSGesture();
                    touchGesture.target = _this.viewHelper.linesContentDomNode;
                    penGesture.target = _this.viewHelper.linesContentDomNode;
                    _this.viewHelper.linesContentDomNode.addEventListener('pointerdown', function (e) {
                        var pointerType = e.pointerType;
                        if (pointerType === 'mouse') {
                            _this._lastPointerType = 'mouse';
                            return;
                        }
                        else if (pointerType === 'touch') {
                            _this._lastPointerType = 'touch';
                            touchGesture.addPointer(e.pointerId);
                        }
                        else {
                            _this._lastPointerType = 'pen';
                            penGesture.addPointer(e.pointerId);
                        }
                    });
                    _this.listenersToRemove.push(dom.addDisposableThrottledListener(_this.viewHelper.linesContentDomNode, 'MSGestureChange', function (e) { return _this._onGestureChange(e); }, gestureChangeEventMerger));
                    _this.listenersToRemove.push(dom.addDisposableListener(_this.viewHelper.linesContentDomNode, 'MSGestureTap', function (e) { return _this._onCaptureGestureTap(e); }, true));
                }
            }, 100);
            this._lastPointerType = 'mouse';
        }
        StandardPointerHandler.prototype._onMouseDown = function (e) {
            if (this._lastPointerType === 'mouse') {
                _super.prototype._onMouseDown.call(this, e);
            }
        };
        StandardPointerHandler.prototype._onCaptureGestureTap = function (rawEvent) {
            var _this = this;
            var e = new mouseEvent_1.StandardMouseEvent(rawEvent);
            var t = this._createMouseTarget(e, false);
            if (t.position) {
                this.viewController.moveTo('mouse', t.position);
            }
            // IE does not want to focus when coming in from the browser's address bar
            if (e.browserEvent.fromElement) {
                e.preventDefault();
                this.viewHelper.focusTextArea();
            }
            else {
                // TODO@Alex -> cancel this is focus is lost
                setTimeout(function () {
                    _this.viewHelper.focusTextArea();
                });
            }
        };
        StandardPointerHandler.prototype._onGestureChange = function (e) {
            this.viewHelper.setScrollTop(this.viewHelper.getScrollTop() - e.translationY);
            this.viewHelper.setScrollLeft(this.viewHelper.getScrollLeft() - e.translationX);
        };
        StandardPointerHandler.prototype.dispose = function () {
            window.clearTimeout(this._installGestureHandlerTimeout);
            _super.prototype.dispose.call(this);
        };
        return StandardPointerHandler;
    }(mouseHandler_1.MouseHandler));
    var TouchHandler = (function (_super) {
        __extends(TouchHandler, _super);
        function TouchHandler(context, viewController, viewHelper) {
            var _this = this;
            _super.call(this, context, viewController, viewHelper);
            this.gesture = new touch_1.Gesture(this.viewHelper.linesContentDomNode);
            this.listenersToRemove.push(dom.addDisposableListener(this.viewHelper.linesContentDomNode, touch_1.EventType.Tap, function (e) { return _this.onTap(e); }));
            this.listenersToRemove.push(dom.addDisposableListener(this.viewHelper.linesContentDomNode, touch_1.EventType.Change, function (e) { return _this.onChange(e); }));
            this.listenersToRemove.push(dom.addDisposableListener(this.viewHelper.linesContentDomNode, touch_1.EventType.Contextmenu, function (e) { return _this._onContextMenu(e, false); }));
        }
        TouchHandler.prototype.dispose = function () {
            this.gesture.dispose();
            _super.prototype.dispose.call(this);
        };
        TouchHandler.prototype.onTap = function (event) {
            event.preventDefault();
            this.viewHelper.focusTextArea();
            var mouseEvent = new mouseEvent_1.StandardMouseEvent(event);
            var target = this._createMouseTarget(mouseEvent, false);
            if (target.position) {
                this.viewController.moveTo('mouse', target.position);
            }
        };
        TouchHandler.prototype.onChange = function (event) {
            this.viewHelper.setScrollTop(this.viewHelper.getScrollTop() - event.translationY);
            this.viewHelper.setScrollLeft(this.viewHelper.getScrollLeft() - event.translationX);
        };
        return TouchHandler;
    }(mouseHandler_1.MouseHandler));
    var PointerHandler = (function () {
        function PointerHandler(context, viewController, viewHelper) {
            if (window.navigator.msPointerEnabled) {
                this.handler = new MsPointerHandler(context, viewController, viewHelper);
            }
            else if (window.TouchEvent) {
                this.handler = new TouchHandler(context, viewController, viewHelper);
            }
            else if (window.navigator.pointerEnabled) {
                this.handler = new StandardPointerHandler(context, viewController, viewHelper);
            }
            else {
                this.handler = new mouseHandler_1.MouseHandler(context, viewController, viewHelper);
            }
        }
        PointerHandler.prototype.onScrollChanged = function (e) {
            this.handler.onScrollChanged(e);
        };
        PointerHandler.prototype.dispose = function () {
            this.handler.dispose();
        };
        return PointerHandler;
    }());
    exports.PointerHandler = PointerHandler;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/view/viewLayer", ["require", "exports", 'vs/base/browser/dom', 'vs/editor/browser/view/viewPart'], function (require, exports, dom, viewPart_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewLayer = (function (_super) {
        __extends(ViewLayer, _super);
        function ViewLayer(context) {
            var _this = this;
            _super.call(this, context);
            // ---- end view event handlers
            this._scrollDomNode = null;
            this._scrollDomNodeIsAbove = false;
            this.domNode = this._createDomNode();
            this._lines = [];
            this._rendLineNumberStart = 1;
            this._renderer = new ViewLayerRenderer(function () { return _this._createLine(); }, function () { return _this._extraDomNodeHTML(); });
        }
        ViewLayer.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this._lines = null;
        };
        ViewLayer.prototype._extraDomNodeHTML = function () {
            return '';
        };
        // ---- begin view event handlers
        ViewLayer.prototype.onConfigurationChanged = function (e) {
            for (var i = 0; i < this._lines.length; i++) {
                this._lines[i].onConfigurationChanged(e);
            }
            return true;
        };
        ViewLayer.prototype.onLayoutChanged = function (layoutInfo) {
            return true;
        };
        ViewLayer.prototype.onScrollChanged = function (e) {
            return e.vertical;
        };
        ViewLayer.prototype.onZonesChanged = function () {
            return true;
        };
        ViewLayer.prototype.onModelFlushed = function () {
            this._lines = [];
            this._rendLineNumberStart = 1;
            dom.clearNode(this.domNode);
            return true;
        };
        ViewLayer.prototype.onModelLinesDeleted = function (e) {
            var from = Math.max(e.fromLineNumber - this._rendLineNumberStart, 0);
            var to = Math.min(e.toLineNumber - this._rendLineNumberStart, this._lines.length - 1);
            var i;
            // Adjust this._rendLineNumberStart
            if (e.fromLineNumber < this._rendLineNumberStart) {
                // Deleting lines starting above the viewport
                if (e.toLineNumber < this._rendLineNumberStart) {
                    // All deleted lines are above the viewport
                    this._rendLineNumberStart -= (e.toLineNumber - e.fromLineNumber + 1);
                }
                else {
                    // Some deleted lines are inside the viewport
                    this._rendLineNumberStart = e.fromLineNumber;
                }
            }
            // Remove lines if they fall in the viewport
            if (from <= to) {
                // Remove from DOM
                for (i = from; i <= to; i++) {
                    var lineDomNode = this._lines[i].getDomNode();
                    if (lineDomNode) {
                        this.domNode.removeChild(lineDomNode);
                    }
                }
                // Remove from array
                this._lines.splice(from, to - from + 1);
            }
            // Mark the rest of the visible lines as possibly invalid
            for (i = from; i < this._lines.length; i++) {
                this._lines[i].onLinesDeletedAbove();
            }
            return true;
        };
        ViewLayer.prototype.onModelLineChanged = function (e) {
            var lineIndex = e.lineNumber - this._rendLineNumberStart, shouldRender = false;
            if (lineIndex >= 0 && lineIndex < this._lines.length) {
                this._lines[lineIndex].onContentChanged();
                shouldRender = true;
            }
            // Mark the rest of the visible lines as possibly invalid
            for (var i = Math.max(lineIndex, 0); i < this._lines.length; i++) {
                this._lines[i].onLineChangedAbove();
                shouldRender = true;
            }
            return shouldRender;
        };
        ViewLayer.prototype.onModelLinesInserted = function (e) {
            var i;
            if (e.fromLineNumber <= this._rendLineNumberStart) {
                // a. We are inserting lines above the viewport
                this._rendLineNumberStart += (e.toLineNumber - e.fromLineNumber + 1);
                // Mark the visible lines as possibly invalid
                for (i = 0; i < this._lines.length; i++) {
                    this._lines[i].onLinesInsertedAbove();
                }
                return true;
            }
            if (e.fromLineNumber >= this._rendLineNumberStart + this._lines.length) {
                // b. We are inserting lines below the viewport
                return false;
            }
            // c. We are inserting lines in the viewport
            var insertFrom = Math.min(e.fromLineNumber - this._rendLineNumberStart, this._lines.length - 1);
            var insertTo = Math.min(e.toLineNumber - this._rendLineNumberStart, this._lines.length - 1);
            if (insertFrom <= insertTo) {
                // Insert lines that fall inside the viewport
                for (i = insertFrom; i <= insertTo; i++) {
                    var line = this._createLine();
                    this._lines.splice(i, 0, line);
                }
                // We need to remove lines that are pushed outside the viewport by this insertion,
                // due to the Math.min above on `insertTo`. Otherwise, it is possible for the next line
                // after the insertion to be marked `maybeInvalid` when it should be definitely `invalid`.
                var insertCount = insertTo - insertFrom + 1;
                for (i = 0; i < insertCount; i++) {
                    // Remove from array
                    var lastLine = this._lines.pop();
                    // Remove from DOM
                    var lineDomNode = lastLine.getDomNode();
                    if (lineDomNode) {
                        this.domNode.removeChild(lineDomNode);
                    }
                }
            }
            // Mark the rest of the lines as possibly invalid
            for (i = insertTo; i < this._lines.length; i++) {
                this._lines[i].onLinesInsertedAbove();
            }
            return true;
        };
        ViewLayer.prototype.onModelTokensChanged = function (e) {
            var changedFromIndex = e.fromLineNumber - this._rendLineNumberStart;
            var changedToIndex = e.toLineNumber - this._rendLineNumberStart;
            if (changedToIndex < 0 || changedFromIndex >= this._lines.length) {
                return false;
            }
            var fromIndex = Math.min(Math.max(changedFromIndex, 0), this._lines.length - 1);
            var toIndex = Math.min(Math.max(changedToIndex, 0), this._lines.length - 1);
            var somethingMayHaveChanged = false;
            for (var i = fromIndex; i <= toIndex; i++) {
                somethingMayHaveChanged = true;
                this._lines[i].onTokensChanged();
            }
            return somethingMayHaveChanged;
        };
        ViewLayer.prototype._renderLines = function (linesViewportData) {
            var ctx = {
                domNode: this.domNode,
                rendLineNumberStart: this._rendLineNumberStart,
                lines: this._lines,
                linesLength: this._lines.length,
                getInlineDecorationsForLineInViewport: function (lineNumber) { return linesViewportData.getInlineDecorationsForLineInViewport(lineNumber); },
                viewportTop: linesViewportData.viewportTop,
                viewportHeight: linesViewportData.viewportHeight,
                scrollDomNode: this._scrollDomNode,
                scrollDomNodeIsAbove: this._scrollDomNodeIsAbove
            };
            // Decide if this render will do a single update (single large .innerHTML) or many updates (inserting/removing dom nodes)
            var resCtx = this._renderer.renderWithManyUpdates(ctx, linesViewportData.startLineNumber, linesViewportData.endLineNumber, linesViewportData.relativeVerticalOffset);
            this._rendLineNumberStart = resCtx.rendLineNumberStart;
            this._lines = resCtx.lines;
            this._scrollDomNode = resCtx.scrollDomNode;
            this._scrollDomNodeIsAbove = resCtx.scrollDomNodeIsAbove;
        };
        ViewLayer.prototype._createDomNode = function () {
            var domNode = document.createElement('div');
            domNode.className = 'view-layer';
            domNode.style.position = 'absolute';
            domNode.setAttribute('role', 'presentation');
            domNode.setAttribute('aria-hidden', 'true');
            return domNode;
        };
        ViewLayer.prototype._createLine = function () {
            throw new Error('Implement me!');
        };
        return ViewLayer;
    }(viewPart_1.ViewPart));
    exports.ViewLayer = ViewLayer;
    var ViewLayerRenderer = (function () {
        function ViewLayerRenderer(createLine, extraDomNodeHTML) {
            this._createLine = createLine;
            this._extraDomNodeHTML = extraDomNodeHTML;
        }
        ViewLayerRenderer.prototype.renderWithManyUpdates = function (ctx, startLineNumber, stopLineNumber, deltaTop) {
            return this._render(ctx, startLineNumber, stopLineNumber, deltaTop);
        };
        ViewLayerRenderer.prototype._render = function (inContext, startLineNumber, stopLineNumber, deltaTop) {
            var ctx = {
                domNode: inContext.domNode,
                rendLineNumberStart: inContext.rendLineNumberStart,
                lines: inContext.lines.slice(0),
                linesLength: inContext.linesLength,
                getInlineDecorationsForLineInViewport: inContext.getInlineDecorationsForLineInViewport,
                viewportTop: inContext.viewportTop,
                viewportHeight: inContext.viewportHeight,
                scrollDomNode: inContext.scrollDomNode,
                scrollDomNodeIsAbove: inContext.scrollDomNodeIsAbove
            };
            var canRemoveScrollDomNode = true;
            if (ctx.scrollDomNode) {
                var time = this._getScrollDomNodeTime(ctx.scrollDomNode);
                if ((new Date()).getTime() - time < 1000) {
                    canRemoveScrollDomNode = false;
                }
            }
            if (canRemoveScrollDomNode && ((ctx.rendLineNumberStart + ctx.linesLength - 1 < startLineNumber) || (stopLineNumber < ctx.rendLineNumberStart))) {
                // There is no overlap whatsoever
                ctx.rendLineNumberStart = startLineNumber;
                ctx.linesLength = stopLineNumber - startLineNumber + 1;
                ctx.lines = [];
                for (var x = startLineNumber; x <= stopLineNumber; x++) {
                    ctx.lines[x - startLineNumber] = this._createLine();
                }
                this._finishRendering(ctx, true, deltaTop);
                ctx.scrollDomNode = null;
                return ctx;
            }
            // Update lines which will remain untouched
            this._renderUntouchedLines(ctx, Math.max(startLineNumber - ctx.rendLineNumberStart, 0), Math.min(stopLineNumber - ctx.rendLineNumberStart, ctx.linesLength - 1), deltaTop, startLineNumber);
            var fromLineNumber, toLineNumber, removeCnt;
            if (ctx.rendLineNumberStart > startLineNumber) {
                // Insert lines before
                fromLineNumber = startLineNumber;
                toLineNumber = Math.min(stopLineNumber, ctx.rendLineNumberStart - 1);
                if (fromLineNumber <= toLineNumber) {
                    this._insertLinesBefore(ctx, fromLineNumber, toLineNumber, deltaTop, startLineNumber);
                    ctx.linesLength += toLineNumber - fromLineNumber + 1;
                    // Clean garbage above
                    if (ctx.scrollDomNode && ctx.scrollDomNodeIsAbove) {
                        if (ctx.scrollDomNode.parentNode) {
                            ctx.scrollDomNode.parentNode.removeChild(ctx.scrollDomNode);
                        }
                        ctx.scrollDomNode = null;
                    }
                }
            }
            else if (ctx.rendLineNumberStart < startLineNumber) {
                // Remove lines before
                removeCnt = Math.min(ctx.linesLength, startLineNumber - ctx.rendLineNumberStart);
                if (removeCnt > 0) {
                    this._removeLinesBefore(ctx, removeCnt);
                    ctx.linesLength -= removeCnt;
                }
            }
            ctx.rendLineNumberStart = startLineNumber;
            if (ctx.rendLineNumberStart + ctx.linesLength - 1 < stopLineNumber) {
                // Insert lines after
                fromLineNumber = ctx.rendLineNumberStart + ctx.linesLength;
                toLineNumber = stopLineNumber;
                if (fromLineNumber <= toLineNumber) {
                    this._insertLinesAfter(ctx, fromLineNumber, toLineNumber, deltaTop, startLineNumber);
                    ctx.linesLength += toLineNumber - fromLineNumber + 1;
                    // Clean garbage below
                    if (ctx.scrollDomNode && !ctx.scrollDomNodeIsAbove) {
                        if (ctx.scrollDomNode.parentNode) {
                            ctx.scrollDomNode.parentNode.removeChild(ctx.scrollDomNode);
                        }
                        ctx.scrollDomNode = null;
                    }
                }
            }
            else if (ctx.rendLineNumberStart + ctx.linesLength - 1 > stopLineNumber) {
                // Remove lines after
                fromLineNumber = Math.max(0, stopLineNumber - ctx.rendLineNumberStart + 1);
                toLineNumber = ctx.linesLength - 1;
                removeCnt = toLineNumber - fromLineNumber + 1;
                if (removeCnt > 0) {
                    this._removeLinesAfter(ctx, removeCnt);
                    ctx.linesLength -= removeCnt;
                }
            }
            this._finishRendering(ctx, false, deltaTop);
            return ctx;
        };
        ViewLayerRenderer.prototype._renderUntouchedLines = function (ctx, startIndex, endIndex, deltaTop, deltaLN) {
            var i, lineNumber;
            for (i = startIndex; i <= endIndex; i++) {
                lineNumber = ctx.rendLineNumberStart + i;
                var lineDomNode = ctx.lines[i].getDomNode();
                if (lineDomNode) {
                    ctx.lines[i].layoutLine(lineNumber, deltaTop[lineNumber - deltaLN]);
                }
            }
        };
        ViewLayerRenderer.prototype._insertLinesBefore = function (ctx, fromLineNumber, toLineNumber, deltaTop, deltaLN) {
            var newLines = [], line, lineNumber;
            for (lineNumber = fromLineNumber; lineNumber <= toLineNumber; lineNumber++) {
                line = this._createLine();
                newLines.push(line);
            }
            ctx.lines = newLines.concat(ctx.lines);
        };
        ViewLayerRenderer.prototype._getScrollDomNodeTime = function (domNode) {
            var lastScrollTime = domNode.getAttribute('last-scroll-time');
            if (lastScrollTime) {
                return parseInt(lastScrollTime, 10);
            }
            return 0;
        };
        ViewLayerRenderer.prototype._removeIfNotScrollDomNode = function (ctx, domNode, isAbove) {
            var time = this._getScrollDomNodeTime(domNode);
            if (!time) {
                ctx.domNode.removeChild(domNode);
                return;
            }
            if (ctx.scrollDomNode) {
                var otherTime = this._getScrollDomNodeTime(ctx.scrollDomNode);
                if (otherTime > time) {
                    // The other is the real scroll dom node
                    ctx.domNode.removeChild(domNode);
                    return;
                }
                if (ctx.scrollDomNode.parentNode) {
                    ctx.scrollDomNode.parentNode.removeChild(ctx.scrollDomNode);
                }
                ctx.scrollDomNode = null;
            }
            ctx.scrollDomNode = domNode;
            ctx.scrollDomNodeIsAbove = isAbove;
        };
        ViewLayerRenderer.prototype._removeLinesBefore = function (ctx, removeCount) {
            var i;
            for (i = 0; i < removeCount; i++) {
                var lineDomNode = ctx.lines[i].getDomNode();
                if (lineDomNode) {
                    this._removeIfNotScrollDomNode(ctx, lineDomNode, true);
                }
            }
            ctx.lines.splice(0, removeCount);
        };
        ViewLayerRenderer.prototype._insertLinesAfter = function (ctx, fromLineNumber, toLineNumber, deltaTop, deltaLN) {
            var newLines = [], line, lineNumber;
            for (lineNumber = fromLineNumber; lineNumber <= toLineNumber; lineNumber++) {
                line = this._createLine();
                newLines.push(line);
            }
            ctx.lines = ctx.lines.concat(newLines);
        };
        ViewLayerRenderer.prototype._removeLinesAfter = function (ctx, removeCount) {
            var i, removeIndex = ctx.linesLength - removeCount;
            for (i = 0; i < removeCount; i++) {
                var lineDomNode = ctx.lines[removeIndex + i].getDomNode();
                if (lineDomNode) {
                    this._removeIfNotScrollDomNode(ctx, lineDomNode, false);
                }
            }
            ctx.lines.splice(removeIndex, removeCount);
        };
        ViewLayerRenderer.prototype._finishRendering = function (ctx, domNodeIsEmpty, deltaTop) {
            var i, len, line, lineNumber, hadNewLine = false, wasNew = [], newLinesHTML = [], hadInvalidLine = false, wasInvalid = [], invalidLinesHTML = [];
            for (i = 0, len = ctx.linesLength; i < len; i++) {
                line = ctx.lines[i];
                lineNumber = i + ctx.rendLineNumberStart;
                if (line.shouldUpdateHTML(lineNumber, ctx.getInlineDecorationsForLineInViewport(lineNumber))) {
                    var lineDomNode = line.getDomNode();
                    if (!lineDomNode) {
                        // Line is new
                        line.getLineOuterHTML(newLinesHTML, lineNumber, deltaTop[i]);
                        wasNew[i] = true;
                        hadNewLine = true;
                    }
                    else {
                        // Line is invalid
                        line.getLineOuterHTML(invalidLinesHTML, lineNumber, deltaTop[i]);
                        wasInvalid[i] = true;
                        hadInvalidLine = true;
                    }
                }
            }
            if (hadNewLine) {
                var lastChild = ctx.domNode.lastChild;
                if (domNodeIsEmpty || !lastChild) {
                    ctx.domNode.innerHTML = this._extraDomNodeHTML() + newLinesHTML.join('');
                }
                else {
                    lastChild.insertAdjacentHTML('afterend', newLinesHTML.join(''));
                }
                var currChild = ctx.domNode.lastChild;
                for (i = ctx.linesLength - 1; i >= 0; i--) {
                    line = ctx.lines[i];
                    if (wasNew[i]) {
                        line.setDomNode(currChild);
                        currChild = currChild.previousSibling;
                    }
                }
            }
            if (hadInvalidLine) {
                var hugeDomNode = document.createElement('div');
                hugeDomNode.innerHTML = invalidLinesHTML.join('');
                var lineDomNode, source;
                for (i = 0; i < ctx.linesLength; i++) {
                    line = ctx.lines[i];
                    if (wasInvalid[i]) {
                        source = hugeDomNode.firstChild;
                        lineDomNode = line.getDomNode();
                        lineDomNode.parentNode.replaceChild(source, lineDomNode);
                        line.setDomNode(source);
                    }
                }
            }
        };
        return ViewLayerRenderer;
    }());
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/view/viewOverlays", ["require", "exports", 'vs/base/browser/browser', 'vs/base/browser/dom', 'vs/base/browser/styleMutator', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/view/viewLayer'], function (require, exports, browser, dom, styleMutator_1, editorBrowser, viewLayer_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewOverlays = (function (_super) {
        __extends(ViewOverlays, _super);
        function ViewOverlays(context, layoutProvider) {
            _super.call(this, context);
            this._layoutProvider = layoutProvider;
            this._dynamicOverlays = [];
            this.domNode.className = 'view-overlays';
        }
        ViewOverlays.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this._layoutProvider = null;
            for (var i = 0; i < this._dynamicOverlays.length; i++) {
                this._dynamicOverlays[i].dispose();
            }
            this._dynamicOverlays = null;
        };
        ViewOverlays.prototype.getDomNode = function () {
            return this.domNode;
        };
        ViewOverlays.prototype.addDynamicOverlay = function (overlay) {
            this._dynamicOverlays.push(overlay);
        };
        // ----- event handlers
        ViewOverlays.prototype.onViewFocusChanged = function (isFocused) {
            var _this = this;
            this._requestModificationFrame(function () {
                dom.toggleClass(_this.domNode, 'focused', isFocused);
            });
            return false;
        };
        // ----- end event handlers
        ViewOverlays.prototype._createLine = function () {
            var r = new ViewOverlayLine(this._context, this._dynamicOverlays);
            return r;
        };
        ViewOverlays.prototype.onReadAfterForcedLayout = function (ctx) {
            var _this = this;
            // Overwriting to bypass `shouldRender` flag
            for (var i = 0; i < this._dynamicOverlays.length; i++) {
                this._dynamicOverlays[i].shouldCallRender2(ctx);
            }
            this._requestModificationFrame(function () {
                _this._viewOverlaysRender(ctx);
            });
            return null;
        };
        ViewOverlays.prototype._viewOverlaysRender = function (ctx) {
            _super.prototype._renderLines.call(this, ctx.linesViewportData);
        };
        ViewOverlays.prototype.onWriteAfterForcedLayout = function () {
            // Overwriting to bypass `shouldRender` flag
            this._executeModificationRunners();
        };
        return ViewOverlays;
    }(viewLayer_1.ViewLayer));
    exports.ViewOverlays = ViewOverlays;
    var ViewOverlayLine = (function () {
        function ViewOverlayLine(context, dynamicOverlays) {
            this._context = context;
            this._dynamicOverlays = dynamicOverlays;
            this._domNode = null;
            this._renderPieces = null;
        }
        ViewOverlayLine.prototype.getDomNode = function () {
            return this._domNode;
        };
        ViewOverlayLine.prototype.setDomNode = function (domNode) {
            this._domNode = domNode;
        };
        ViewOverlayLine.prototype.onContentChanged = function () {
            // Nothing
        };
        ViewOverlayLine.prototype.onLinesInsertedAbove = function () {
            // Nothing
        };
        ViewOverlayLine.prototype.onLinesDeletedAbove = function () {
            // Nothing
        };
        ViewOverlayLine.prototype.onLineChangedAbove = function () {
            // Nothing
        };
        ViewOverlayLine.prototype.onTokensChanged = function () {
            // Nothing
        };
        ViewOverlayLine.prototype.onConfigurationChanged = function (e) {
            // Nothing
        };
        ViewOverlayLine.prototype._piecesEqual = function (newPieces) {
            if (!this._renderPieces || this._renderPieces.length !== newPieces.length) {
                return false;
            }
            for (var i = 0, len = newPieces.length; i < len; i++) {
                if (this._renderPieces[i] !== newPieces[i]) {
                    return false;
                }
            }
            return true;
        };
        ViewOverlayLine.prototype.shouldUpdateHTML = function (lineNumber, inlineDecorations) {
            var newPieces = [];
            for (var i = 0; i < this._dynamicOverlays.length; i++) {
                var pieces = this._dynamicOverlays[i].render2(lineNumber);
                if (pieces && pieces.length > 0) {
                    newPieces = newPieces.concat(pieces);
                }
            }
            var piecesEqual = this._piecesEqual(newPieces);
            if (!piecesEqual) {
                this._renderPieces = newPieces;
            }
            return !piecesEqual;
        };
        ViewOverlayLine.prototype.getLineOuterHTML = function (out, lineNumber, deltaTop) {
            out.push('<div lineNumber="');
            out.push(lineNumber.toString());
            out.push('" style="top:');
            out.push(deltaTop.toString());
            out.push('px;height:');
            out.push(this._context.configuration.editor.lineHeight.toString());
            out.push('px;" class="');
            out.push(editorBrowser.ClassNames.VIEW_LINE);
            out.push('">');
            out.push(this.getLineInnerHTML(lineNumber));
            out.push('</div>');
        };
        ViewOverlayLine.prototype.getLineInnerHTML = function (lineNumber) {
            return this._renderPieces.join('');
        };
        ViewOverlayLine.prototype.layoutLine = function (lineNumber, deltaTop) {
            var currentLineNumber = this._domNode.getAttribute('lineNumber');
            if (currentLineNumber !== lineNumber.toString()) {
                this._domNode.setAttribute('lineNumber', lineNumber.toString());
            }
            styleMutator_1.StyleMutator.setTop(this._domNode, deltaTop);
            styleMutator_1.StyleMutator.setHeight(this._domNode, this._context.configuration.editor.lineHeight);
        };
        return ViewOverlayLine;
    }());
    var ContentViewOverlays = (function (_super) {
        __extends(ContentViewOverlays, _super);
        function ContentViewOverlays(context, layoutProvider) {
            _super.call(this, context, layoutProvider);
            styleMutator_1.StyleMutator.setWidth(this.domNode, 0);
            styleMutator_1.StyleMutator.setHeight(this.domNode, 0);
        }
        ContentViewOverlays.prototype.onScrollWidthChanged = function (scrollWidth) {
            return true;
        };
        ContentViewOverlays.prototype._viewOverlaysRender = function (ctx) {
            _super.prototype._viewOverlaysRender.call(this, ctx);
            styleMutator_1.StyleMutator.setWidth(this.domNode, this._layoutProvider.getScrollWidth());
        };
        return ContentViewOverlays;
    }(ViewOverlays));
    exports.ContentViewOverlays = ContentViewOverlays;
    var MarginViewOverlays = (function (_super) {
        __extends(MarginViewOverlays, _super);
        function MarginViewOverlays(context, layoutProvider) {
            _super.call(this, context, layoutProvider);
            this._hasVerticalScroll = false;
            this._glyphMarginLeft = 0;
            this._glyphMarginWidth = 0;
            this._scrollHeight = layoutProvider.getScrollHeight();
            this.domNode.className = editorBrowser.ClassNames.MARGIN_VIEW_OVERLAYS + ' monaco-editor-background';
            styleMutator_1.StyleMutator.setWidth(this.domNode, 1);
            this._hasVerticalScroll = true;
        }
        MarginViewOverlays.prototype._extraDomNodeHTML = function () {
            return [
                '<div class="',
                editorBrowser.ClassNames.GLYPH_MARGIN,
                '" style="left:',
                String(this._glyphMarginLeft),
                'px;width:',
                String(this._glyphMarginWidth),
                'px;height:',
                String(this._scrollHeight),
                'px;"></div>'
            ].join('');
        };
        MarginViewOverlays.prototype._getGlyphMarginDomNode = function () {
            return this.domNode.children[0];
        };
        MarginViewOverlays.prototype.onScrollHeightChanged = function (scrollHeight) {
            var _this = this;
            this._scrollHeight = scrollHeight;
            this._requestModificationFrame(function () {
                var glyphMargin = _this._getGlyphMarginDomNode();
                if (glyphMargin) {
                    styleMutator_1.StyleMutator.setHeight(glyphMargin, _this._scrollHeight);
                }
            });
            return _super.prototype.onScrollHeightChanged.call(this, scrollHeight) || true;
        };
        MarginViewOverlays.prototype.onLayoutChanged = function (layoutInfo) {
            var _this = this;
            this._glyphMarginLeft = layoutInfo.glyphMarginLeft;
            this._glyphMarginWidth = layoutInfo.glyphMarginWidth;
            this._scrollHeight = this._layoutProvider.getScrollHeight();
            this._requestModificationFrame(function () {
                styleMutator_1.StyleMutator.setWidth(_this.domNode, layoutInfo.contentLeft);
                var glyphMargin = _this._getGlyphMarginDomNode();
                if (glyphMargin) {
                    styleMutator_1.StyleMutator.setLeft(glyphMargin, layoutInfo.glyphMarginLeft);
                    styleMutator_1.StyleMutator.setWidth(glyphMargin, layoutInfo.glyphMarginWidth);
                }
            });
            return _super.prototype.onLayoutChanged.call(this, layoutInfo) || true;
        };
        MarginViewOverlays.prototype.onScrollChanged = function (e) {
            this._hasVerticalScroll = this._hasVerticalScroll || e.vertical;
            return _super.prototype.onScrollChanged.call(this, e);
        };
        MarginViewOverlays.prototype._viewOverlaysRender = function (ctx) {
            _super.prototype._viewOverlaysRender.call(this, ctx);
            if (this._hasVerticalScroll) {
                if (browser.canUseTranslate3d) {
                    var transform = 'translate3d(0px, ' + ctx.linesViewportData.visibleRangesDeltaTop + 'px, 0px)';
                    styleMutator_1.StyleMutator.setTransform(this.domNode, transform);
                }
                else {
                    if (this._hasVerticalScroll) {
                        styleMutator_1.StyleMutator.setTop(this.domNode, ctx.linesViewportData.visibleRangesDeltaTop);
                    }
                }
                this._hasVerticalScroll = false;
            }
            var height = Math.min(this._layoutProvider.getTotalHeight(), 1000000);
            styleMutator_1.StyleMutator.setHeight(this.domNode, height);
        };
        return MarginViewOverlays;
    }(ViewOverlays));
    exports.MarginViewOverlays = MarginViewOverlays;
});

define("vs/editor/browser/viewLayout/scrollManager", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/browser/dom', 'vs/base/browser/ui/scrollbar/scrollableElementImpl', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowser'], function (require, exports, lifecycle_1, dom, scrollableElementImpl_1, editorCommon_1, editorBrowser_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function addPropertyIfPresent(src, dst, prop) {
        if (src.hasOwnProperty(prop)) {
            dst[prop] = src[prop];
        }
    }
    var ScrollManager = (function () {
        function ScrollManager(scrollable, configuration, privateViewEventBus, linesContent, viewDomNode, overflowGuardDomNode) {
            var _this = this;
            this.toDispose = [];
            this.scrollable = scrollable;
            this.configuration = configuration;
            this.privateViewEventBus = privateViewEventBus;
            this.linesContent = linesContent;
            this.toDispose.push(this.scrollable.addScrollListener(function (e) {
                _this.privateViewEventBus.emit(editorCommon_1.EventType.ViewScrollChanged, e);
            }));
            var configScrollbarOpts = this.configuration.editor.scrollbar;
            var scrollbarOptions = {
                scrollable: this.scrollable,
                listenOnDomNode: viewDomNode,
                vertical: configScrollbarOpts.vertical,
                horizontal: configScrollbarOpts.horizontal,
                className: editorBrowser_1.ClassNames.SCROLLABLE_ELEMENT + ' ' + this.configuration.editor.theme,
                useShadows: false,
                saveLastScrollTimeOnClassName: editorBrowser_1.ClassNames.VIEW_LINE
            };
            addPropertyIfPresent(configScrollbarOpts, scrollbarOptions, 'verticalHasArrows');
            addPropertyIfPresent(configScrollbarOpts, scrollbarOptions, 'horizontalHasArrows');
            addPropertyIfPresent(configScrollbarOpts, scrollbarOptions, 'verticalScrollbarSize');
            addPropertyIfPresent(configScrollbarOpts, scrollbarOptions, 'verticalSliderSize');
            addPropertyIfPresent(configScrollbarOpts, scrollbarOptions, 'horizontalScrollbarSize');
            addPropertyIfPresent(configScrollbarOpts, scrollbarOptions, 'horizontalSliderSize');
            addPropertyIfPresent(configScrollbarOpts, scrollbarOptions, 'handleMouseWheel');
            addPropertyIfPresent(configScrollbarOpts, scrollbarOptions, 'arrowSize');
            addPropertyIfPresent(configScrollbarOpts, scrollbarOptions, 'mouseWheelScrollSensitivity');
            this.scrollbar = new scrollableElementImpl_1.ScrollableElement(linesContent, scrollbarOptions, {
                width: this.configuration.editor.layoutInfo.contentWidth,
                height: this.configuration.editor.layoutInfo.contentHeight,
            });
            this.toDispose.push(this.scrollbar);
            this.toDispose.push(this.scrollable.addInternalSizeChangeListener(function () {
                _this.scrollbar.onElementInternalDimensions();
            }));
            this.toDispose.push(this.configuration.onDidChange(function (e) {
                _this.scrollbar.updateClassName(_this.configuration.editor.theme);
                if (e.scrollbar) {
                    _this.scrollbar.updateOptions(_this.configuration.editor.scrollbar);
                }
            }));
            // When having a zone widget that calls .focus() on one of its dom elements,
            // the browser will try desperately to reveal that dom node, unexpectedly
            // changing the .scrollTop of this.linesContent
            var onBrowserDesperateReveal = function (domNode, lookAtScrollTop, lookAtScrollLeft) {
                if (lookAtScrollTop) {
                    var deltaTop = domNode.scrollTop;
                    if (deltaTop) {
                        _this.scrollable.setScrollTop(_this.scrollable.getScrollTop() + deltaTop);
                        domNode.scrollTop = 0;
                    }
                }
                if (lookAtScrollLeft) {
                    var deltaLeft = domNode.scrollLeft;
                    if (deltaLeft) {
                        _this.scrollable.setScrollLeft(_this.scrollable.getScrollLeft() + deltaLeft);
                        domNode.scrollLeft = 0;
                    }
                }
            };
            // I've seen this happen both on the view dom node & on the lines content dom node.
            this.toDispose.push(dom.addDisposableListener(viewDomNode, 'scroll', function (e) { return onBrowserDesperateReveal(viewDomNode, true, true); }));
            this.toDispose.push(dom.addDisposableListener(linesContent, 'scroll', function (e) { return onBrowserDesperateReveal(linesContent, true, false); }));
            this.toDispose.push(dom.addDisposableListener(overflowGuardDomNode, 'scroll', function (e) { return onBrowserDesperateReveal(overflowGuardDomNode, true, false); }));
        }
        ScrollManager.prototype.dispose = function () {
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
        };
        ScrollManager.prototype.onSizeProviderLayoutChanged = function () {
            if (this.scrollbar) {
                this.scrollbar.onElementDimensions({
                    width: this.configuration.editor.layoutInfo.contentWidth,
                    height: this.configuration.editor.layoutInfo.contentHeight,
                });
            }
        };
        ScrollManager.prototype.getOverviewRulerLayoutInfo = function () {
            if (this.scrollbar) {
                return this.scrollbar.getOverviewRulerLayoutInfo();
            }
            return null;
        };
        ScrollManager.prototype.getScrollbarContainerDomNode = function () {
            if (this.scrollbar) {
                return this.scrollbar.getDomNode();
            }
            return this.linesContent;
        };
        ScrollManager.prototype.delegateVerticalScrollbarMouseDown = function (browserEvent) {
            if (this.scrollbar) {
                this.scrollbar.delegateVerticalScrollbarMouseDown(browserEvent);
            }
        };
        return ScrollManager;
    }());
    exports.ScrollManager = ScrollManager;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewLayout/layoutProvider", ["require", "exports", 'vs/editor/common/editorCommon', 'vs/editor/common/viewLayout/editorScrollable', 'vs/editor/common/viewLayout/linesLayout', 'vs/editor/common/viewModel/viewEventHandler', 'vs/editor/browser/viewLayout/scrollManager'], function (require, exports, editorCommon, editorScrollable_1, linesLayout_1, viewEventHandler_1, scrollManager_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LayoutProvider = (function (_super) {
        __extends(LayoutProvider, _super);
        function LayoutProvider(configuration, model, privateViewEventBus, linesContent, viewDomNode, overflowGuardDomNode) {
            _super.call(this);
            this.configuration = configuration;
            this.privateViewEventBus = privateViewEventBus;
            this.model = model;
            this.scrollable = new editorScrollable_1.EditorScrollable();
            this.scrollable.setWidth(this.configuration.editor.layoutInfo.contentWidth);
            this.scrollable.setHeight(this.configuration.editor.layoutInfo.contentHeight);
            this.scrollManager = new scrollManager_1.ScrollManager(this.scrollable, configuration, privateViewEventBus, linesContent, viewDomNode, overflowGuardDomNode);
            this.configuration.setLineCount(this.model.getLineCount());
            this.linesLayout = new linesLayout_1.LinesLayout(configuration, model);
            this._updateHeight();
        }
        LayoutProvider.prototype.dispose = function () {
            this.scrollManager.dispose();
            this.scrollable.dispose();
        };
        LayoutProvider.prototype.updateLineCount = function () {
            this.configuration.setLineCount(this.model.getLineCount());
        };
        // ---- begin view event handlers
        LayoutProvider.prototype.onZonesChanged = function () {
            this._updateHeight();
            return false;
        };
        LayoutProvider.prototype.onModelFlushed = function () {
            this.linesLayout.onModelFlushed();
            this.updateLineCount();
            this._updateHeight();
            return false;
        };
        LayoutProvider.prototype.onModelLinesDeleted = function (e) {
            this.linesLayout.onModelLinesDeleted(e);
            this.updateLineCount();
            this._updateHeight();
            return false;
        };
        LayoutProvider.prototype.onModelLinesInserted = function (e) {
            this.linesLayout.onModelLinesInserted(e);
            this.updateLineCount();
            this._updateHeight();
            return false;
        };
        LayoutProvider.prototype.onConfigurationChanged = function (e) {
            if (e.layoutInfo) {
                this.scrollable.setWidth(this.configuration.editor.layoutInfo.contentWidth);
                this.scrollable.setHeight(this.configuration.editor.layoutInfo.contentHeight);
                this.scrollManager.onSizeProviderLayoutChanged();
                this._emitLayoutChangedEvent();
            }
            this._updateHeight();
            return false;
        };
        LayoutProvider.prototype._updateHeight = function () {
            var oldScrollHeight = this.scrollable.getScrollHeight();
            this.scrollable.setScrollHeight(this.getTotalHeight());
            var newScrollHeight = this.scrollable.getScrollHeight();
            if (oldScrollHeight !== newScrollHeight) {
                this.privateViewEventBus.emit(editorCommon.EventType.ViewScrollHeightChanged, newScrollHeight);
            }
        };
        // ---- end view event handlers
        // ---- Layouting logic
        LayoutProvider.prototype.getCurrentViewport = function () {
            return {
                top: this.scrollable.getScrollTop(),
                left: this.scrollable.getScrollLeft(),
                width: this.configuration.editor.layoutInfo.contentWidth,
                height: this.configuration.editor.layoutInfo.contentHeight
            };
        };
        LayoutProvider.prototype.getCenteredViewLineNumberInViewport = function () {
            return this.linesLayout.getCenteredLineInViewport(this.getCurrentViewport());
        };
        LayoutProvider.prototype._emitLayoutChangedEvent = function () {
            this.privateViewEventBus.emit(editorCommon.EventType.ViewLayoutChanged, this.configuration.editor.layoutInfo);
        };
        LayoutProvider.prototype.emitLayoutChangedEvent = function () {
            this._emitLayoutChangedEvent();
        };
        LayoutProvider.prototype._computeScrollWidth = function (maxLineWidth, viewportWidth) {
            var isViewportWrapping = this.configuration.editor.wrappingInfo.isViewportWrapping;
            if (!isViewportWrapping) {
                return Math.max(maxLineWidth + LayoutProvider.LINES_HORIZONTAL_EXTRA_PX, viewportWidth);
            }
            return Math.max(maxLineWidth, viewportWidth);
        };
        LayoutProvider.prototype.onMaxLineWidthChanged = function (maxLineWidth) {
            var newScrollWidth = this._computeScrollWidth(maxLineWidth, this.getCurrentViewport().width);
            var oldScrollWidth = this.scrollable.getScrollWidth();
            this.scrollable.setScrollWidth(newScrollWidth);
            newScrollWidth = this.scrollable.getScrollWidth();
            if (newScrollWidth !== oldScrollWidth) {
                this.privateViewEventBus.emit(editorCommon.EventType.ViewScrollWidthChanged, newScrollWidth);
                // The height might depend on the fact that there is a horizontal scrollbar or not
                this._updateHeight();
            }
        };
        // ---- view state
        LayoutProvider.prototype.saveState = function () {
            var scrollTop = this.scrollable.getScrollTop();
            var firstLineNumberInViewport = this.linesLayout.getLineNumberAtOrAfterVerticalOffset(scrollTop);
            var whitespaceAboveFirstLine = this.linesLayout.getWhitespaceAccumulatedHeightBeforeLineNumber(firstLineNumberInViewport);
            return {
                scrollTop: scrollTop,
                scrollTopWithoutViewZones: scrollTop - whitespaceAboveFirstLine,
                scrollLeft: this.scrollable.getScrollLeft()
            };
        };
        LayoutProvider.prototype.restoreState = function (state) {
            var restoreScrollTop = state.scrollTop;
            if (typeof state.scrollTopWithoutViewZones === 'number' && !this.linesLayout.hasWhitespace()) {
                restoreScrollTop = state.scrollTopWithoutViewZones;
            }
            this.scrollable.setScrollTop(restoreScrollTop);
            this.scrollable.setScrollLeft(state.scrollLeft);
        };
        // ---- IVerticalLayoutProvider
        LayoutProvider.prototype.addWhitespace = function (afterLineNumber, ordinal, height) {
            return this.linesLayout.insertWhitespace(afterLineNumber, ordinal, height);
        };
        LayoutProvider.prototype.changeWhitespace = function (id, newAfterLineNumber, newHeight) {
            return this.linesLayout.changeWhitespace(id, newAfterLineNumber, newHeight);
        };
        LayoutProvider.prototype.removeWhitespace = function (id) {
            return this.linesLayout.removeWhitespace(id);
        };
        LayoutProvider.prototype.getVerticalOffsetForLineNumber = function (lineNumber) {
            return this.linesLayout.getVerticalOffsetForLineNumber(lineNumber);
        };
        LayoutProvider.prototype.heightInPxForLine = function (lineNumber) {
            return this.linesLayout.getHeightForLineNumber(lineNumber);
        };
        LayoutProvider.prototype.isAfterLines = function (verticalOffset) {
            return this.linesLayout.isAfterLines(verticalOffset);
        };
        LayoutProvider.prototype.getLineNumberAtVerticalOffset = function (verticalOffset) {
            return this.linesLayout.getLineNumberAtOrAfterVerticalOffset(verticalOffset);
        };
        LayoutProvider.prototype.getTotalHeight = function () {
            var reserveHorizontalScrollbarHeight = 0;
            if (this.scrollable.getScrollWidth() > this.scrollable.getWidth()) {
                reserveHorizontalScrollbarHeight = this.configuration.editor.scrollbar.horizontalScrollbarSize;
            }
            return this.linesLayout.getTotalHeight(this.getCurrentViewport(), reserveHorizontalScrollbarHeight);
        };
        LayoutProvider.prototype.getWhitespaceAtVerticalOffset = function (verticalOffset) {
            return this.linesLayout.getWhitespaceAtVerticalOffset(verticalOffset);
        };
        LayoutProvider.prototype.getLinesViewportData = function () {
            return this.linesLayout.getLinesViewportData(this.getCurrentViewport());
        };
        LayoutProvider.prototype.getWhitespaceViewportData = function () {
            return this.linesLayout.getWhitespaceViewportData(this.getCurrentViewport());
        };
        LayoutProvider.prototype.getWhitespaces = function () {
            return this.linesLayout.getWhitespaces();
        };
        // ---- IScrollingProvider
        LayoutProvider.prototype.getOverviewRulerInsertData = function () {
            var layoutInfo = this.scrollManager.getOverviewRulerLayoutInfo();
            return {
                parent: layoutInfo.parent,
                insertBefore: layoutInfo.insertBefore
            };
        };
        LayoutProvider.prototype.getScrollbarContainerDomNode = function () {
            return this.scrollManager.getScrollbarContainerDomNode();
        };
        LayoutProvider.prototype.delegateVerticalScrollbarMouseDown = function (browserEvent) {
            this.scrollManager.delegateVerticalScrollbarMouseDown(browserEvent);
        };
        LayoutProvider.prototype.getScrollHeight = function () {
            return this.scrollable.getScrollHeight();
        };
        LayoutProvider.prototype.getScrollWidth = function () {
            return this.scrollable.getScrollWidth();
        };
        LayoutProvider.prototype.getScrollLeft = function () {
            return this.scrollable.getScrollLeft();
        };
        LayoutProvider.prototype.setScrollLeft = function (scrollLeft) {
            this.scrollable.setScrollLeft(scrollLeft);
        };
        LayoutProvider.prototype.getScrollTop = function () {
            return this.scrollable.getScrollTop();
        };
        LayoutProvider.prototype.setScrollTop = function (scrollTop) {
            this.scrollable.setScrollTop(scrollTop);
        };
        LayoutProvider.prototype.getScrolledTopFromAbsoluteTop = function (top) {
            return top - this.scrollable.getScrollTop();
        };
        LayoutProvider.LINES_HORIZONTAL_EXTRA_PX = 30;
        return LayoutProvider;
    }(viewEventHandler_1.ViewEventHandler));
    exports.LayoutProvider = LayoutProvider;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/contentWidgets/contentWidgets", ["require", "exports", 'vs/base/browser/dom', 'vs/base/browser/styleMutator', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/view/viewPart', 'vs/css!./contentWidgets'], function (require, exports, dom, styleMutator_1, editorBrowser_1, viewPart_1) {
    'use strict';
    var ViewContentWidgets = (function (_super) {
        __extends(ViewContentWidgets, _super);
        function ViewContentWidgets(context, viewDomNode) {
            _super.call(this, context);
            this._viewDomNode = viewDomNode;
            this._widgets = {};
            this._contentWidth = 0;
            this._contentLeft = 0;
            this.domNode = document.createElement('div');
            this.domNode.className = editorBrowser_1.ClassNames.CONTENT_WIDGETS;
            this.overflowingContentWidgetsDomNode = document.createElement('div');
            this.overflowingContentWidgetsDomNode.className = editorBrowser_1.ClassNames.OVERFLOWING_CONTENT_WIDGETS;
        }
        ViewContentWidgets.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this._widgets = null;
            this.domNode = null;
        };
        // --- begin event handlers
        ViewContentWidgets.prototype.onModelFlushed = function () {
            return true;
        };
        ViewContentWidgets.prototype.onModelDecorationsChanged = function (e) {
            // true for inline decorations that can end up relayouting text
            return e.inlineDecorationsChanged;
        };
        ViewContentWidgets.prototype.onModelLinesDeleted = function (e) {
            return true;
        };
        ViewContentWidgets.prototype.onModelLineChanged = function (e) {
            return true;
        };
        ViewContentWidgets.prototype.onModelLinesInserted = function (e) {
            return true;
        };
        ViewContentWidgets.prototype.onCursorPositionChanged = function (e) {
            return false;
        };
        ViewContentWidgets.prototype.onCursorSelectionChanged = function (e) {
            return false;
        };
        ViewContentWidgets.prototype.onCursorRevealRange = function (e) {
            return false;
        };
        ViewContentWidgets.prototype.onConfigurationChanged = function (e) {
            return true;
        };
        ViewContentWidgets.prototype.onLayoutChanged = function (layoutInfo) {
            var _this = this;
            this._contentWidth = layoutInfo.contentWidth;
            this._contentLeft = layoutInfo.contentLeft;
            this._requestModificationFrameBeforeRendering(function () {
                // update the maxWidth on widgets nodes, such that `onReadAfterForcedLayout`
                // below can read out the adjusted width/height of widgets
                var widgetId;
                for (widgetId in _this._widgets) {
                    if (_this._widgets.hasOwnProperty(widgetId)) {
                        styleMutator_1.StyleMutator.setMaxWidth(_this._widgets[widgetId].widget.getDomNode(), _this._contentWidth);
                    }
                }
            });
            return true;
        };
        ViewContentWidgets.prototype.onScrollChanged = function (e) {
            return true;
        };
        ViewContentWidgets.prototype.onZonesChanged = function () {
            return true;
        };
        ViewContentWidgets.prototype.onScrollWidthChanged = function (scrollWidth) {
            return false;
        };
        ViewContentWidgets.prototype.onScrollHeightChanged = function (scrollHeight) {
            return false;
        };
        // ---- end view event handlers
        ViewContentWidgets.prototype.addWidget = function (widget) {
            var widgetData = {
                allowEditorOverflow: widget.allowEditorOverflow || false,
                widget: widget,
                position: null,
                preference: null,
                isVisible: false
            };
            this._widgets[widget.getId()] = widgetData;
            var domNode = widget.getDomNode();
            domNode.style.position = 'absolute';
            styleMutator_1.StyleMutator.setMaxWidth(domNode, this._contentWidth);
            styleMutator_1.StyleMutator.setVisibility(domNode, 'hidden');
            domNode.setAttribute('widgetId', widget.getId());
            if (widgetData.allowEditorOverflow) {
                this.overflowingContentWidgetsDomNode.appendChild(domNode);
            }
            else {
                this.domNode.appendChild(domNode);
            }
            this.shouldRender = true;
        };
        ViewContentWidgets.prototype.setWidgetPosition = function (widget, position, preference) {
            var widgetData = this._widgets[widget.getId()];
            widgetData.position = position;
            widgetData.preference = preference;
            this.shouldRender = true;
        };
        ViewContentWidgets.prototype.removeWidget = function (widget) {
            var widgetId = widget.getId();
            if (this._widgets.hasOwnProperty(widgetId)) {
                var widgetData = this._widgets[widgetId];
                delete this._widgets[widgetId];
                var domNode = widgetData.widget.getDomNode();
                domNode.parentNode.removeChild(domNode);
                domNode.removeAttribute('monaco-visible-content-widget');
                this.shouldRender = true;
            }
        };
        ViewContentWidgets.prototype._layoutBoxInViewport = function (position, domNode, ctx) {
            var visibleRange = ctx.visibleRangeForPosition(position);
            if (!visibleRange) {
                return null;
            }
            var width = domNode.clientWidth;
            var height = domNode.clientHeight;
            // Our visible box is split horizontally by the current line => 2 boxes
            // a) the box above the line
            var aboveLineTop = visibleRange.top;
            var heightAboveLine = aboveLineTop;
            // b) the box under the line
            var underLineTop = visibleRange.top + this._context.configuration.editor.lineHeight;
            var heightUnderLine = ctx.viewportHeight - underLineTop;
            var aboveTop = aboveLineTop - height;
            var fitsAbove = (heightAboveLine >= height);
            var belowTop = underLineTop;
            var fitsBelow = (heightUnderLine >= height);
            // And its left
            var actualLeft = visibleRange.left;
            if (actualLeft + width > ctx.viewportLeft + ctx.viewportWidth) {
                actualLeft = ctx.viewportLeft + ctx.viewportWidth - width;
            }
            if (actualLeft < ctx.viewportLeft) {
                actualLeft = ctx.viewportLeft;
            }
            return {
                aboveTop: aboveTop,
                fitsAbove: fitsAbove,
                belowTop: belowTop,
                fitsBelow: fitsBelow,
                left: actualLeft
            };
        };
        ViewContentWidgets.prototype._layoutBoxInPage = function (position, domNode, ctx) {
            var visibleRange = ctx.visibleRangeForPosition(position);
            if (!visibleRange) {
                return null;
            }
            var left0 = visibleRange.left - ctx.viewportLeft;
            var width = domNode.clientWidth, height = domNode.clientHeight;
            if (left0 + width < 0 || left0 > this._contentWidth) {
                return null;
            }
            var aboveTop = visibleRange.top - height, belowTop = visibleRange.top + this._context.configuration.editor.lineHeight, left = left0 + this._contentLeft;
            var domNodePosition = dom.getDomNodePosition(this._viewDomNode);
            var absoluteAboveTop = domNodePosition.top + aboveTop - document.body.scrollTop - document.documentElement.scrollTop, absoluteBelowTop = domNodePosition.top + belowTop - document.body.scrollTop - document.documentElement.scrollTop, absoluteLeft = domNodePosition.left + left - document.body.scrollLeft - document.documentElement.scrollLeft;
            var INNER_WIDTH = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth, INNER_HEIGHT = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;
            // Leave some clearance to the bottom
            var BOTTOM_PADDING = 22;
            var fitsAbove = (absoluteAboveTop >= 0), fitsBelow = (absoluteBelowTop + height <= INNER_HEIGHT - BOTTOM_PADDING);
            if (absoluteLeft + width + 20 > INNER_WIDTH) {
                var delta = absoluteLeft - (INNER_WIDTH - width - 20);
                absoluteLeft -= delta;
                left -= delta;
            }
            if (absoluteLeft < 0) {
                var delta = absoluteLeft;
                absoluteLeft -= delta;
                left -= delta;
            }
            return {
                aboveTop: aboveTop,
                fitsAbove: fitsAbove,
                belowTop: belowTop,
                fitsBelow: fitsBelow,
                left: left
            };
        };
        ViewContentWidgets.prototype._prepareRenderWidgetAtExactPosition = function (position, ctx) {
            var visibleRange = ctx.visibleRangeForPosition(position);
            if (!visibleRange) {
                return null;
            }
            return {
                top: visibleRange.top,
                left: visibleRange.left
            };
        };
        ViewContentWidgets.prototype._prepareRenderWidget = function (widgetData, ctx) {
            var _this = this;
            if (!widgetData.position || !widgetData.preference) {
                return null;
            }
            // Do not trust that widgets have a valid position
            var validModelPosition = this._context.model.validateModelPosition(widgetData.position);
            if (!this._context.model.modelPositionIsVisible(validModelPosition)) {
                // this position is hidden by the view model
                return null;
            }
            var position = this._context.model.convertModelPositionToViewPosition(validModelPosition.lineNumber, validModelPosition.column);
            var placement = null;
            var fetchPlacement = function () {
                if (placement) {
                    return;
                }
                var domNode = widgetData.widget.getDomNode();
                if (widgetData.allowEditorOverflow) {
                    placement = _this._layoutBoxInPage(position, domNode, ctx);
                }
                else {
                    placement = _this._layoutBoxInViewport(position, domNode, ctx);
                }
            };
            // Do two passes, first for perfect fit, second picks first option
            for (var pass = 1; pass <= 2; pass++) {
                for (var i = 0; i < widgetData.preference.length; i++) {
                    var pref = widgetData.preference[i];
                    if (pref === editorBrowser_1.ContentWidgetPositionPreference.ABOVE) {
                        fetchPlacement();
                        if (!placement) {
                            // Widget outside of viewport
                            return null;
                        }
                        if (pass === 2 || placement.fitsAbove) {
                            return {
                                top: placement.aboveTop,
                                left: placement.left
                            };
                        }
                    }
                    else if (pref === editorBrowser_1.ContentWidgetPositionPreference.BELOW) {
                        fetchPlacement();
                        if (!placement) {
                            // Widget outside of viewport
                            return null;
                        }
                        if (pass === 2 || placement.fitsBelow) {
                            return {
                                top: placement.belowTop,
                                left: placement.left
                            };
                        }
                    }
                    else {
                        return this._prepareRenderWidgetAtExactPosition(position, ctx);
                    }
                }
            }
        };
        ViewContentWidgets.prototype._render = function (ctx) {
            var _this = this;
            var data = {}, renderData, widgetId;
            for (widgetId in this._widgets) {
                if (this._widgets.hasOwnProperty(widgetId)) {
                    renderData = this._prepareRenderWidget(this._widgets[widgetId], ctx);
                    if (renderData) {
                        data[widgetId] = renderData;
                    }
                }
            }
            this._requestModificationFrame(function () {
                var widgetId, widget, domNode;
                for (widgetId in _this._widgets) {
                    if (_this._widgets.hasOwnProperty(widgetId)) {
                        widget = _this._widgets[widgetId];
                        domNode = _this._widgets[widgetId].widget.getDomNode();
                        if (data.hasOwnProperty(widgetId)) {
                            if (widget.allowEditorOverflow) {
                                styleMutator_1.StyleMutator.setTop(domNode, data[widgetId].top);
                                styleMutator_1.StyleMutator.setLeft(domNode, data[widgetId].left);
                            }
                            else {
                                styleMutator_1.StyleMutator.setTop(domNode, data[widgetId].top + ctx.viewportTop - ctx.bigNumbersDelta);
                                styleMutator_1.StyleMutator.setLeft(domNode, data[widgetId].left);
                            }
                            if (!widget.isVisible) {
                                styleMutator_1.StyleMutator.setVisibility(domNode, 'inherit');
                                domNode.setAttribute('monaco-visible-content-widget', 'true');
                                widget.isVisible = true;
                            }
                        }
                        else {
                            if (widget.isVisible) {
                                domNode.removeAttribute('monaco-visible-content-widget');
                                widget.isVisible = false;
                                styleMutator_1.StyleMutator.setVisibility(domNode, 'hidden');
                            }
                        }
                    }
                }
            });
        };
        return ViewContentWidgets;
    }(viewPart_1.ViewPart));
    exports.ViewContentWidgets = ViewContentWidgets;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/lines/viewLines", ["require", "exports", 'vs/base/common/async', 'vs/base/browser/browser', 'vs/base/browser/styleMutator', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/view/viewLayer', 'vs/editor/browser/viewParts/lines/viewLine'], function (require, exports, async_1, browser, styleMutator_1, range_1, editorCommon, editorBrowser_1, viewLayer_1, viewLine_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ViewLines = (function (_super) {
        __extends(ViewLines, _super);
        function ViewLines(context, layoutProvider) {
            var _this = this;
            _super.call(this, context);
            this._hasVerticalScroll = false;
            this._hasHorizontalScroll = false;
            this._layoutProvider = layoutProvider;
            this.domNode.className = editorBrowser_1.ClassNames.VIEW_LINES;
            // --- width & height
            this._maxLineWidth = 0;
            this._asyncUpdateLineWidths = new async_1.RunOnceScheduler(function () {
                _this._updateLineWidths();
            }, 200);
            this._currentVisibleRange = new range_1.Range(1, 1, 1, 1);
            this._bigNumbersDelta = 0;
            this._lastCursorRevealRangeHorizontallyEvent = null;
            this.textRangeRestingSpot = document.createElement('div');
            this.textRangeRestingSpot.className = 'textRangeRestingSpot';
            this._hasVerticalScroll = true;
            this._hasHorizontalScroll = true;
        }
        ViewLines.prototype.dispose = function () {
            this._asyncUpdateLineWidths.dispose();
            this._layoutProvider = null;
            _super.prototype.dispose.call(this);
        };
        // ---- begin view event handlers
        ViewLines.prototype.onConfigurationChanged = function (e) {
            var shouldRender = _super.prototype.onConfigurationChanged.call(this, e);
            if (e.wrappingInfo) {
                this._maxLineWidth = 0;
            }
            return shouldRender;
        };
        ViewLines.prototype.onLayoutChanged = function (layoutInfo) {
            var shouldRender = _super.prototype.onLayoutChanged.call(this, layoutInfo);
            this._maxLineWidth = 0;
            return shouldRender;
        };
        ViewLines.prototype.onModelFlushed = function () {
            var shouldRender = _super.prototype.onModelFlushed.call(this);
            this._maxLineWidth = 0;
            return shouldRender;
        };
        ViewLines.prototype.onScrollWidthChanged = function (scrollWidth) {
            styleMutator_1.StyleMutator.setWidth(this.domNode, scrollWidth);
            return false;
        };
        ViewLines.prototype.onModelDecorationsChanged = function (e) {
            var shouldRender = _super.prototype.onModelDecorationsChanged.call(this, e);
            for (var i = 0; i < this._lines.length; i++) {
                this._lines[i].onModelDecorationsChanged();
            }
            return shouldRender || true;
        };
        ViewLines.prototype.onCursorRevealRange = function (e) {
            var newScrollTop = this._computeScrollTopToRevealRange(this._layoutProvider.getCurrentViewport(), e.range, e.verticalType);
            if (e.revealHorizontal) {
                this._lastCursorRevealRangeHorizontallyEvent = e;
            }
            this._layoutProvider.setScrollTop(newScrollTop);
            return true;
        };
        ViewLines.prototype.onCursorScrollRequest = function (e) {
            var currentScrollTop = this._layoutProvider.getScrollTop();
            var newScrollTop = currentScrollTop + e.deltaLines * this._context.configuration.editor.lineHeight;
            this._layoutProvider.setScrollTop(newScrollTop);
            return true;
        };
        ViewLines.prototype.onScrollChanged = function (e) {
            this._hasVerticalScroll = this._hasVerticalScroll || e.vertical;
            this._hasHorizontalScroll = this._hasHorizontalScroll || e.horizontal;
            return _super.prototype.onScrollChanged.call(this, e);
        };
        // ---- end view event handlers
        // ----------- HELPERS FOR OTHERS
        ViewLines.prototype.getPositionFromDOMInfo = function (spanNode, offset) {
            var lineNumber = this._getLineNumberFromDOMInfo(spanNode);
            if (lineNumber === -1) {
                // Couldn't find span node
                return null;
            }
            if (lineNumber < 1 || lineNumber > this._context.model.getLineCount()) {
                // lineNumber is outside range
                return null;
            }
            if (this._context.model.getLineMaxColumn(lineNumber) === 1) {
                // Line is empty
                return {
                    lineNumber: lineNumber,
                    column: 1
                };
            }
            var lineIndex = lineNumber - this._rendLineNumberStart;
            if (lineIndex < 0 || lineIndex >= this._lines.length) {
                // Couldn't find line
                return null;
            }
            var column = this._lines[lineIndex].getColumnOfNodeOffset(lineNumber, spanNode, offset);
            var minColumn = this._context.model.getLineMinColumn(lineNumber);
            if (column < minColumn) {
                column = minColumn;
            }
            return {
                lineNumber: lineNumber,
                column: column
            };
        };
        ViewLines.prototype._getLineNumberFromDOMInfo = function (spanNode) {
            while (spanNode && spanNode.nodeType === 1) {
                if (spanNode.className === editorBrowser_1.ClassNames.VIEW_LINE) {
                    return parseInt(spanNode.getAttribute('lineNumber'), 10);
                }
                spanNode = spanNode.parentElement;
            }
            return -1;
        };
        ViewLines.prototype.getLineWidth = function (lineNumber) {
            var lineIndex = lineNumber - this._rendLineNumberStart;
            if (lineIndex < 0 || lineIndex >= this._lines.length) {
                return -1;
            }
            return this._lines[lineIndex].getWidth();
        };
        ViewLines.prototype.linesVisibleRangesForRange = function (range, includeNewLines) {
            if (this.shouldRender) {
                // Cannot read from the DOM because it is dirty
                // i.e. the model & the dom are out of sync, so I'd be reading something stale
                return null;
            }
            var originalEndLineNumber = range.endLineNumber;
            range = range_1.Range.intersectRanges(range, this._currentVisibleRange);
            if (!range) {
                return null;
            }
            var visibleRangesForLine, visibleRanges = [], lineNumber, lineIndex, startColumn, endColumn;
            var boundingClientRect = this.domNode.getBoundingClientRect();
            var clientRectDeltaLeft = boundingClientRect.left;
            var currentLineModelLineNumber, nextLineModelLineNumber;
            if (includeNewLines) {
                nextLineModelLineNumber = this._context.model.convertViewPositionToModelPosition(range.startLineNumber, 1).lineNumber;
            }
            for (lineNumber = range.startLineNumber; lineNumber <= range.endLineNumber; lineNumber++) {
                lineIndex = lineNumber - this._rendLineNumberStart;
                if (lineIndex < 0 || lineIndex >= this._lines.length) {
                    continue;
                }
                startColumn = lineNumber === range.startLineNumber ? range.startColumn : 1;
                endColumn = lineNumber === range.endLineNumber ? range.endColumn : this._context.model.getLineMaxColumn(lineNumber);
                visibleRangesForLine = this._lines[lineIndex].getVisibleRangesForRange(startColumn, endColumn, clientRectDeltaLeft, this.textRangeRestingSpot);
                if (!visibleRangesForLine || visibleRangesForLine.length === 0) {
                    continue;
                }
                if (includeNewLines && lineNumber < originalEndLineNumber) {
                    currentLineModelLineNumber = nextLineModelLineNumber;
                    nextLineModelLineNumber = this._context.model.convertViewPositionToModelPosition(lineNumber + 1, 1).lineNumber;
                    if (currentLineModelLineNumber !== nextLineModelLineNumber) {
                        visibleRangesForLine[visibleRangesForLine.length - 1].width += ViewLines.LINE_FEED_WIDTH;
                    }
                }
                visibleRanges.push(new editorCommon.LineVisibleRanges(lineNumber, visibleRangesForLine));
            }
            if (visibleRanges.length === 0) {
                return null;
            }
            return visibleRanges;
        };
        ViewLines.prototype.visibleRangesForRange2 = function (range, deltaTop) {
            if (this.shouldRender) {
                // Cannot read from the DOM because it is dirty
                // i.e. the model & the dom are out of sync, so I'd be reading something stale
                return null;
            }
            range = range_1.Range.intersectRanges(range, this._currentVisibleRange);
            if (!range) {
                return null;
            }
            var result = [];
            var boundingClientRect = this.domNode.getBoundingClientRect();
            var clientRectDeltaLeft = boundingClientRect.left;
            for (var lineNumber = range.startLineNumber; lineNumber <= range.endLineNumber; lineNumber++) {
                var lineIndex = lineNumber - this._rendLineNumberStart;
                if (lineIndex < 0 || lineIndex >= this._lines.length) {
                    continue;
                }
                var startColumn = lineNumber === range.startLineNumber ? range.startColumn : 1;
                var endColumn = lineNumber === range.endLineNumber ? range.endColumn : this._context.model.getLineMaxColumn(lineNumber);
                var visibleRangesForLine = this._lines[lineIndex].getVisibleRangesForRange(startColumn, endColumn, clientRectDeltaLeft, this.textRangeRestingSpot);
                if (!visibleRangesForLine || visibleRangesForLine.length === 0) {
                    continue;
                }
                var adjustedLineNumberVerticalOffset = this._layoutProvider.getVerticalOffsetForLineNumber(lineNumber) - this._bigNumbersDelta + deltaTop;
                for (var i = 0, len = visibleRangesForLine.length; i < len; i++) {
                    result.push(new editorCommon.VisibleRange(adjustedLineNumberVerticalOffset, visibleRangesForLine[i].left, visibleRangesForLine[i].width));
                }
            }
            if (result.length === 0) {
                return null;
            }
            return result;
        };
        // --- implementation
        ViewLines.prototype._createLine = function () {
            return viewLine_1.createLine(this._context);
        };
        ViewLines.prototype._renderAndUpdateLineHeights = function (linesViewportData) {
            _super.prototype._renderLines.call(this, linesViewportData);
            // Update internal current visible range
            this._currentVisibleRange = new range_1.Range(0 + this._rendLineNumberStart, 1, this._lines.length - 1 + this._rendLineNumberStart, this._context.model.getLineMaxColumn(this._lines.length - 1 + this._rendLineNumberStart));
            if (this._lastCursorRevealRangeHorizontallyEvent) {
                var newScrollLeft = this._computeScrollLeftToRevealRange(this._lastCursorRevealRangeHorizontallyEvent.range);
                this._lastCursorRevealRangeHorizontallyEvent = null;
                var isViewportWrapping = this._context.configuration.editor.wrappingInfo.isViewportWrapping;
                if (!isViewportWrapping) {
                    this._ensureMaxLineWidth(newScrollLeft.maxHorizontalOffset);
                }
                this._layoutProvider.setScrollLeft(newScrollLeft.scrollLeft);
            }
        };
        ViewLines.prototype._updateLineWidths = function () {
            var i, localMaxLineWidth = 1, widthInPx;
            // Read line widths
            for (i = 0; i < this._lines.length; i++) {
                widthInPx = this._lines[i].getWidth();
                localMaxLineWidth = Math.max(localMaxLineWidth, widthInPx);
            }
            this._ensureMaxLineWidth(localMaxLineWidth);
        };
        ViewLines.prototype.render = function () {
            var linesViewportData = this._layoutProvider.getLinesViewportData();
            this._bigNumbersDelta = linesViewportData.bigNumbersDelta;
            if (this.shouldRender) {
                this.shouldRender = false;
                this._renderAndUpdateLineHeights(linesViewportData);
                // Update max line width (not so important, it is just so the horizontal scrollbar doesn't get too small)
                this._asyncUpdateLineWidths.schedule();
            }
            if (this._hasVerticalScroll || this._hasHorizontalScroll) {
                if (browser.canUseTranslate3d) {
                    var transform = 'translate3d(' + -this._layoutProvider.getScrollLeft() + 'px, ' + linesViewportData.visibleRangesDeltaTop + 'px, 0px)';
                    styleMutator_1.StyleMutator.setTransform(this.domNode.parentNode, transform);
                }
                else {
                    if (this._hasVerticalScroll) {
                        styleMutator_1.StyleMutator.setTop(this.domNode.parentNode, linesViewportData.visibleRangesDeltaTop);
                    }
                    if (this._hasHorizontalScroll) {
                        styleMutator_1.StyleMutator.setLeft(this.domNode.parentNode, -this._layoutProvider.getScrollLeft());
                    }
                }
                this._hasVerticalScroll = false;
                this._hasHorizontalScroll = false;
            }
            styleMutator_1.StyleMutator.setWidth(this.domNode, this._layoutProvider.getScrollWidth());
            styleMutator_1.StyleMutator.setHeight(this.domNode, Math.min(this._layoutProvider.getTotalHeight(), 1000000));
            linesViewportData.visibleRange = this._currentVisibleRange;
            return linesViewportData;
        };
        // --- width
        ViewLines.prototype._ensureMaxLineWidth = function (lineWidth) {
            if (this._maxLineWidth < lineWidth) {
                this._maxLineWidth = lineWidth;
                this._layoutProvider.onMaxLineWidthChanged(this._maxLineWidth);
            }
        };
        ViewLines.prototype._computeScrollTopToRevealRange = function (viewport, range, verticalType) {
            var viewportStartY = viewport.top, viewportHeight = viewport.height, viewportEndY = viewportStartY + viewportHeight, boxStartY, boxEndY;
            // Have a box that includes one extra line height (for the horizontal scrollbar)
            boxStartY = this._layoutProvider.getVerticalOffsetForLineNumber(range.startLineNumber);
            boxEndY = this._layoutProvider.getVerticalOffsetForLineNumber(range.endLineNumber) + this._layoutProvider.heightInPxForLine(range.endLineNumber);
            if (verticalType === editorCommon.VerticalRevealType.Simple) {
                // Reveal one line more for the arrow down case, when the last line would be covered by the scrollbar
                boxEndY += this._context.configuration.editor.lineHeight;
            }
            var newScrollTop;
            if (verticalType === editorCommon.VerticalRevealType.Center || verticalType === editorCommon.VerticalRevealType.CenterIfOutsideViewport) {
                if (verticalType === editorCommon.VerticalRevealType.CenterIfOutsideViewport && viewportStartY <= boxStartY && boxEndY <= viewportEndY) {
                    // Box is already in the viewport... do nothing
                    newScrollTop = viewportStartY;
                }
                else {
                    // Box is outside the viewport... center it
                    var boxMiddleY = (boxStartY + boxEndY) / 2;
                    newScrollTop = Math.max(0, boxMiddleY - viewportHeight / 2);
                }
            }
            else {
                newScrollTop = this._computeMinimumScrolling(viewportStartY, viewportEndY, boxStartY, boxEndY);
            }
            return newScrollTop;
        };
        ViewLines.prototype._computeScrollLeftToRevealRange = function (range) {
            var maxHorizontalOffset = 0;
            if (range.startLineNumber !== range.endLineNumber) {
                // Two or more lines? => scroll to base (That's how you see most of the two lines)
                return {
                    scrollLeft: 0,
                    maxHorizontalOffset: maxHorizontalOffset
                };
            }
            var viewport = this._layoutProvider.getCurrentViewport(), viewportStartX = viewport.left, viewportEndX = viewportStartX + viewport.width;
            var visibleRanges = this.visibleRangesForRange2(range, 0), boxStartX = Number.MAX_VALUE, boxEndX = 0;
            if (!visibleRanges) {
                // Unknown
                return {
                    scrollLeft: viewportStartX,
                    maxHorizontalOffset: maxHorizontalOffset
                };
            }
            var i, visibleRange;
            for (i = 0; i < visibleRanges.length; i++) {
                visibleRange = visibleRanges[i];
                if (visibleRange.left < boxStartX) {
                    boxStartX = visibleRange.left;
                }
                if (visibleRange.left + visibleRange.width > boxEndX) {
                    boxEndX = visibleRange.left + visibleRange.width;
                }
            }
            maxHorizontalOffset = boxEndX;
            boxStartX = Math.max(0, boxStartX - ViewLines.HORIZONTAL_EXTRA_PX);
            boxEndX += this._context.configuration.editor.revealHorizontalRightPadding;
            var newScrollLeft = this._computeMinimumScrolling(viewportStartX, viewportEndX, boxStartX, boxEndX);
            return {
                scrollLeft: newScrollLeft,
                maxHorizontalOffset: maxHorizontalOffset
            };
        };
        ViewLines.prototype._computeMinimumScrolling = function (viewportStart, viewportEnd, boxStart, boxEnd) {
            var viewportLength = viewportEnd - viewportStart, boxLength = boxEnd - boxStart;
            if (boxLength < viewportLength) {
                // The box would fit in the viewport
                if (boxStart < viewportStart) {
                    // The box is above the viewport
                    return boxStart;
                }
                else if (boxEnd > viewportEnd) {
                    // The box is below the viewport
                    return Math.max(0, boxEnd - viewportLength);
                }
            }
            else {
                // The box would not fit in the viewport
                // Reveal the beginning of the box
                return boxStart;
            }
            return viewportStart;
        };
        /**
         * Width to extends a line to render the line feed at the end of the line
         */
        ViewLines.LINE_FEED_WIDTH = 10;
        /**
         * Adds this ammount of pixels to the right of lines (no-one wants to type near the edge of the viewport)
         */
        ViewLines.HORIZONTAL_EXTRA_PX = 30;
        return ViewLines;
    }(viewLayer_1.ViewLayer));
    exports.ViewLines = ViewLines;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/scrollDecoration/scrollDecoration", ["require", "exports", 'vs/base/browser/dom', 'vs/base/browser/styleMutator', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/view/viewPart', 'vs/css!./scrollDecoration'], function (require, exports, dom, styleMutator_1, editorBrowser_1, viewPart_1) {
    'use strict';
    var ScrollDecorationViewPart = (function (_super) {
        __extends(ScrollDecorationViewPart, _super);
        function ScrollDecorationViewPart(context) {
            _super.call(this, context);
            this._scrollTop = 0;
            this._width = 0;
            this._shouldShow = false;
            this._domNode = document.createElement('div');
        }
        ScrollDecorationViewPart.prototype._updateShouldShow = function () {
            var newShouldShow = (this._context.configuration.editor.scrollbar.useShadows && this._scrollTop > 0);
            if (this._shouldShow !== newShouldShow) {
                this._shouldShow = newShouldShow;
                return true;
            }
            return false;
        };
        ScrollDecorationViewPart.prototype.getDomNode = function () {
            return this._domNode;
        };
        // --- begin event handlers
        ScrollDecorationViewPart.prototype.onConfigurationChanged = function (e) {
            return this._updateShouldShow();
        };
        ScrollDecorationViewPart.prototype.onLayoutChanged = function (layoutInfo) {
            if (this._width !== layoutInfo.width) {
                this._width = layoutInfo.width;
                return true;
            }
            return false;
        };
        ScrollDecorationViewPart.prototype.onScrollChanged = function (e) {
            this._scrollTop = e.scrollTop;
            return this._updateShouldShow();
        };
        // --- end event handlers
        ScrollDecorationViewPart.prototype._render = function (ctx) {
            var _this = this;
            this._requestModificationFrame(function () {
                styleMutator_1.StyleMutator.setWidth(_this._domNode, _this._width);
                dom.toggleClass(_this._domNode, editorBrowser_1.ClassNames.SCROLL_DECORATION, _this._shouldShow);
            });
        };
        return ScrollDecorationViewPart;
    }(viewPart_1.ViewPart));
    exports.ScrollDecorationViewPart = ScrollDecorationViewPart;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/controller/textAreaHandler", ["require", "exports", 'vs/base/common/async', 'vs/base/common/event', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/editor/common/controller/textAreaState', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon'], function (require, exports, async_1, event_1, keyCodes_1, lifecycle_1, textAreaState_1, position_1, range_1, editorCommon_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ReadFromTextArea;
    (function (ReadFromTextArea) {
        ReadFromTextArea[ReadFromTextArea["Type"] = 0] = "Type";
        ReadFromTextArea[ReadFromTextArea["Paste"] = 1] = "Paste";
    })(ReadFromTextArea || (ReadFromTextArea = {}));
    var TextAreaHandler = (function (_super) {
        __extends(TextAreaHandler, _super);
        function TextAreaHandler(Browser, strategy, textArea, model, flushAnyAccumulatedEvents) {
            var _this = this;
            _super.call(this);
            this._onKeyDown = this._register(new event_1.Emitter());
            this.onKeyDown = this._onKeyDown.event;
            this._onKeyUp = this._register(new event_1.Emitter());
            this.onKeyUp = this._onKeyUp.event;
            this._onCut = this._register(new event_1.Emitter());
            this.onCut = this._onCut.event;
            this._onPaste = this._register(new event_1.Emitter());
            this.onPaste = this._onPaste.event;
            this._onType = this._register(new event_1.Emitter());
            this.onType = this._onType.event;
            this._onCompositionStart = this._register(new event_1.Emitter());
            this.onCompositionStart = this._onCompositionStart.event;
            this._onCompositionEnd = this._register(new event_1.Emitter());
            this.onCompositionEnd = this._onCompositionEnd.event;
            this.Browser = Browser;
            this.textArea = textArea;
            this.model = model;
            this.flushAnyAccumulatedEvents = flushAnyAccumulatedEvents;
            this.selection = new range_1.Range(1, 1, 1, 1);
            this.selections = [new range_1.Range(1, 1, 1, 1)];
            this.cursorPosition = new position_1.Position(1, 1);
            this._nextCommand = ReadFromTextArea.Type;
            this.asyncTriggerCut = new async_1.RunOnceScheduler(function () { return _this._onCut.fire(); }, 0);
            this.lastCopiedValue = null;
            this.lastCopiedValueIsFromEmptySelection = false;
            this.textAreaState = textAreaState_1.createTextAreaState(strategy);
            this.hasFocus = false;
            this.lastCompositionEndTime = 0;
            this._register(this.textArea.onKeyDown(function (e) { return _this._onKeyDownHandler(e); }));
            this._register(this.textArea.onKeyUp(function (e) { return _this._onKeyUp.fire(e); }));
            this._register(this.textArea.onKeyPress(function (e) { return _this._onKeyPressHandler(e); }));
            this.textareaIsShownAtCursor = false;
            this._register(this.textArea.onCompositionStart(function () {
                var timeSinceLastCompositionEnd = (new Date().getTime()) - _this.lastCompositionEndTime;
                if (_this.textareaIsShownAtCursor) {
                    return;
                }
                _this.textareaIsShownAtCursor = true;
                // In IE we cannot set .value when handling 'compositionstart' because the entire composition will get canceled.
                var shouldEmptyTextArea = (timeSinceLastCompositionEnd >= 100);
                if (shouldEmptyTextArea) {
                    if (!_this.Browser.isIE11orEarlier) {
                        _this.setTextAreaState('compositionstart', _this.textAreaState.toEmpty());
                    }
                }
                var showAtLineNumber;
                var showAtColumn;
                // In IE we cannot set .value when handling 'compositionstart' because the entire composition will get canceled.
                if (_this.Browser.isIE11orEarlier) {
                    // Ensure selection start is in viewport
                    showAtLineNumber = _this.selection.startLineNumber;
                    showAtColumn = (_this.selection.startColumn - _this.textAreaState.getSelectionStart());
                }
                else {
                    showAtLineNumber = _this.cursorPosition.lineNumber;
                    showAtColumn = _this.cursorPosition.column;
                }
                _this._onCompositionStart.fire({
                    showAtLineNumber: showAtLineNumber,
                    showAtColumn: showAtColumn
                });
            }));
            var readFromTextArea = function () {
                _this.textAreaState = _this.textAreaState.fromTextArea(_this.textArea);
                var typeInput = _this.textAreaState.deduceInput();
                // console.log('==> DEDUCED INPUT: ' + JSON.stringify(typeInput));
                if (_this._nextCommand === ReadFromTextArea.Type) {
                    if (typeInput.text !== '') {
                        _this._onType.fire(typeInput);
                    }
                }
                else {
                    _this.executePaste(typeInput.text);
                    _this._nextCommand = ReadFromTextArea.Type;
                }
            };
            this._register(this.textArea.onCompositionEnd(function () {
                // console.log('onCompositionEnd: ' + this.textArea.getValue());
                // readFromTextArea();
                _this.lastCompositionEndTime = (new Date()).getTime();
                if (!_this.textareaIsShownAtCursor) {
                    return;
                }
                _this.textareaIsShownAtCursor = false;
                _this._onCompositionEnd.fire();
            }));
            this._register(this.textArea.onInput(function () {
                // console.log('onInput: ' + this.textArea.getValue());
                if (_this.textareaIsShownAtCursor) {
                    // console.log('::ignoring input event because the textarea is shown at cursor: ' + this.textArea.getValue());
                    return;
                }
                readFromTextArea();
            }));
            // --- Clipboard operations
            this._register(this.textArea.onCut(function (e) {
                // Ensure we have the latest selection => ask all pending events to be sent
                _this.flushAnyAccumulatedEvents();
                _this._ensureClipboardGetsEditorSelection(e);
                _this.asyncTriggerCut.schedule();
            }));
            this._register(this.textArea.onCopy(function (e) {
                // Ensure we have the latest selection => ask all pending events to be sent
                _this.flushAnyAccumulatedEvents();
                _this._ensureClipboardGetsEditorSelection(e);
            }));
            this._register(this.textArea.onPaste(function (e) {
                if (e.canUseTextData()) {
                    _this.executePaste(e.getTextData());
                }
                else {
                    if (_this.textArea.getSelectionStart() !== _this.textArea.getSelectionEnd()) {
                        // Clean up the textarea, to get a clean paste
                        _this.setTextAreaState('paste', _this.textAreaState.toEmpty());
                    }
                    _this._nextCommand = ReadFromTextArea.Paste;
                }
            }));
            this._writePlaceholderAndSelectTextArea('ctor');
        }
        TextAreaHandler.prototype.dispose = function () {
            this.asyncTriggerCut.dispose();
            _super.prototype.dispose.call(this);
        };
        // --- begin event handlers
        TextAreaHandler.prototype.setStrategy = function (strategy) {
            this.textAreaState = this.textAreaState.toStrategy(strategy);
        };
        TextAreaHandler.prototype.setHasFocus = function (isFocused) {
            if (this.hasFocus === isFocused) {
                // no change
                return;
            }
            this.hasFocus = isFocused;
            if (this.hasFocus) {
                this._writePlaceholderAndSelectTextArea('focusgain');
            }
        };
        TextAreaHandler.prototype.setCursorSelections = function (primary, secondary) {
            this.selection = primary;
            this.selections = [primary].concat(secondary);
            this._writePlaceholderAndSelectTextArea('selection changed');
        };
        TextAreaHandler.prototype.setCursorPosition = function (primary) {
            this.cursorPosition = primary;
        };
        // --- end event handlers
        TextAreaHandler.prototype.setTextAreaState = function (reason, textAreaState) {
            if (!this.hasFocus) {
                textAreaState = textAreaState.resetSelection();
            }
            textAreaState.applyToTextArea(reason, this.textArea, this.hasFocus);
            this.textAreaState = textAreaState;
        };
        TextAreaHandler.prototype._onKeyDownHandler = function (e) {
            if (e.equals(keyCodes_1.CommonKeybindings.ESCAPE)) {
                // Prevent default always for `Esc`, otherwise it will generate a keypress
                // See http://msdn.microsoft.com/en-us/library/ie/ms536939(v=vs.85).aspx
                e.preventDefault();
            }
            this._onKeyDown.fire(e);
        };
        TextAreaHandler.prototype._onKeyPressHandler = function (e) {
            if (!this.hasFocus) {
                // Sometimes, when doing Alt-Tab, in FF, a 'keypress' is sent before a 'focus'
                return;
            }
        };
        // ------------- Operations that are always executed asynchronously
        TextAreaHandler.prototype.executePaste = function (txt) {
            if (txt === '') {
                return;
            }
            var pasteOnNewLine = false;
            if (this.Browser.enableEmptySelectionClipboard) {
                pasteOnNewLine = (txt === this.lastCopiedValue && this.lastCopiedValueIsFromEmptySelection);
            }
            this._onPaste.fire({
                text: txt,
                pasteOnNewLine: pasteOnNewLine
            });
        };
        TextAreaHandler.prototype.writePlaceholderAndSelectTextAreaSync = function () {
            this._writePlaceholderAndSelectTextArea('focusTextArea');
        };
        TextAreaHandler.prototype._writePlaceholderAndSelectTextArea = function (reason) {
            if (!this.textareaIsShownAtCursor) {
                // Do not write to the textarea if it is visible.
                if (this.Browser.isIPad) {
                    // Do not place anything in the textarea for the iPad
                    this.setTextAreaState(reason, this.textAreaState.toEmpty());
                }
                else {
                    this.setTextAreaState(reason, this.textAreaState.fromEditorSelection(this.model, this.selection));
                }
            }
        };
        // ------------- Clipboard operations
        TextAreaHandler.prototype._ensureClipboardGetsEditorSelection = function (e) {
            var whatToCopy = this._getPlainTextToCopy();
            if (e.canUseTextData()) {
                e.setTextData(whatToCopy);
            }
            else {
                this.setTextAreaState('copy or cut', this.textAreaState.fromText(whatToCopy));
            }
            if (this.Browser.enableEmptySelectionClipboard) {
                if (this.Browser.isFirefox) {
                    // When writing "LINE\r\n" to the clipboard and then pasting,
                    // Firefox pastes "LINE\n", so let's work around this quirk
                    this.lastCopiedValue = whatToCopy.replace(/\r\n/g, '\n');
                }
                else {
                    this.lastCopiedValue = whatToCopy;
                }
                var selections = this.selections;
                this.lastCopiedValueIsFromEmptySelection = (selections.length === 1 && selections[0].isEmpty());
            }
        };
        TextAreaHandler.prototype._getPlainTextToCopy = function () {
            var newLineCharacter = this.model.getEOL();
            var selections = this.selections;
            if (selections.length === 1) {
                var range = selections[0];
                if (range.isEmpty()) {
                    if (this.Browser.enableEmptySelectionClipboard) {
                        var modelLineNumber = this.model.convertViewPositionToModelPosition(range.startLineNumber, 1).lineNumber;
                        return this.model.getModelLineContent(modelLineNumber) + newLineCharacter;
                    }
                    else {
                        return '';
                    }
                }
                return this.model.getValueInRange(range, editorCommon_1.EndOfLinePreference.TextDefined);
            }
            else {
                selections = selections.slice(0).sort(range_1.Range.compareRangesUsingStarts);
                var result = [];
                for (var i = 0; i < selections.length; i++) {
                    result.push(this.model.getValueInRange(selections[i], editorCommon_1.EndOfLinePreference.TextDefined));
                }
                return result.join(newLineCharacter);
            }
        };
        return TextAreaHandler;
    }(lifecycle_1.Disposable));
    exports.TextAreaHandler = TextAreaHandler;
});

define("vs/editor/contrib/hover/browser/hoverWidgets", ["require", "exports", 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/browser/dom', 'vs/base/browser/styleMutator', 'vs/editor/common/core/position', 'vs/editor/browser/editorBrowser'], function (require, exports, keyCodes_1, lifecycle_1, dom, styleMutator_1, position_1, editorBrowser) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ContentHoverWidget = (function () {
        function ContentHoverWidget(id, editor) {
            var _this = this;
            // Editor.IContentWidget.allowEditorOverflow
            this.allowEditorOverflow = true;
            this._id = id;
            this._editor = editor;
            this._isVisible = false;
            this._containerDomNode = document.createElement('div');
            this._containerDomNode.className = 'monaco-editor-hover monaco-editor-background';
            this._domNode = document.createElement('div');
            this._domNode.style.display = 'inline-block';
            this._containerDomNode.appendChild(this._domNode);
            this._containerDomNode.tabIndex = 0;
            this._toDispose = [];
            this._toDispose.push(dom.addStandardDisposableListener(this._containerDomNode, 'keydown', function (e) {
                if (e.equals(keyCodes_1.CommonKeybindings.ESCAPE)) {
                    _this.hide();
                }
            }));
            this._editor.addContentWidget(this);
            this._showAtPosition = null;
        }
        ContentHoverWidget.prototype.getId = function () {
            return this._id;
        };
        ContentHoverWidget.prototype.getDomNode = function () {
            return this._containerDomNode;
        };
        ContentHoverWidget.prototype.showAt = function (position, focus) {
            // Position has changed
            this._showAtPosition = new position_1.Position(position.lineNumber, position.column);
            this._isVisible = true;
            var editorMaxWidth = Math.min(800, parseInt(this._containerDomNode.style.maxWidth, 10));
            // When scrolled horizontally, the div does not want to occupy entire visible area.
            styleMutator_1.StyleMutator.setWidth(this._containerDomNode, editorMaxWidth);
            styleMutator_1.StyleMutator.setHeight(this._containerDomNode, 0);
            styleMutator_1.StyleMutator.setLeft(this._containerDomNode, 0);
            var renderedWidth = Math.min(editorMaxWidth, this._domNode.clientWidth + 5);
            var renderedHeight = this._domNode.clientHeight + 1;
            styleMutator_1.StyleMutator.setWidth(this._containerDomNode, renderedWidth);
            styleMutator_1.StyleMutator.setHeight(this._containerDomNode, renderedHeight);
            this._editor.layoutContentWidget(this);
            // Simply force a synchronous render on the editor
            // such that the widget does not really render with left = '0px'
            this._editor.render();
            this._stoleFocus = focus;
            if (focus) {
                this._containerDomNode.focus();
            }
        };
        ContentHoverWidget.prototype.hide = function () {
            if (!this._isVisible) {
                return;
            }
            this._isVisible = false;
            this._editor.layoutContentWidget(this);
            if (this._stoleFocus) {
                this._editor.focus();
            }
        };
        ContentHoverWidget.prototype.getPosition = function () {
            if (this._isVisible) {
                return {
                    position: this._showAtPosition,
                    preference: [
                        editorBrowser.ContentWidgetPositionPreference.ABOVE,
                        editorBrowser.ContentWidgetPositionPreference.BELOW
                    ]
                };
            }
            return null;
        };
        ContentHoverWidget.prototype.dispose = function () {
            this.hide();
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        return ContentHoverWidget;
    }());
    exports.ContentHoverWidget = ContentHoverWidget;
    var GlyphHoverWidget = (function () {
        function GlyphHoverWidget(id, editor) {
            this._id = id;
            this._editor = editor;
            this._isVisible = false;
            this._domNode = document.createElement('div');
            this._domNode.className = 'monaco-editor-hover monaco-editor-background';
            this._domNode.style.display = 'none';
            this._domNode.setAttribute('aria-hidden', 'true');
            this._domNode.setAttribute('role', 'presentation');
            this._showAtLineNumber = -1;
            this._editor.addOverlayWidget(this);
        }
        GlyphHoverWidget.prototype.getId = function () {
            return this._id;
        };
        GlyphHoverWidget.prototype.getDomNode = function () {
            return this._domNode;
        };
        GlyphHoverWidget.prototype.showAt = function (lineNumber) {
            this._showAtLineNumber = lineNumber;
            if (!this._isVisible) {
                this._isVisible = true;
                this._domNode.style.display = 'block';
            }
            var editorLayout = this._editor.getLayoutInfo();
            var topForLineNumber = this._editor.getTopForLineNumber(this._showAtLineNumber);
            var editorScrollTop = this._editor.getScrollTop();
            this._domNode.style.left = (editorLayout.glyphMarginLeft + editorLayout.glyphMarginWidth) + 'px';
            this._domNode.style.top = (topForLineNumber - editorScrollTop) + 'px';
        };
        GlyphHoverWidget.prototype.hide = function () {
            if (!this._isVisible) {
                return;
            }
            this._isVisible = false;
            this._domNode.style.display = 'none';
        };
        GlyphHoverWidget.prototype.getPosition = function () {
            return null;
        };
        GlyphHoverWidget.prototype.dispose = function () {
            this.hide();
        };
        return GlyphHoverWidget;
    }());
    exports.GlyphHoverWidget = GlyphHoverWidget;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/hover/browser/modesGlyphHover", ["require", "exports", './hoverOperation', './hoverWidgets'], function (require, exports, hoverOperation_1, hoverWidgets_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var MarginComputer = (function () {
        function MarginComputer(editor) {
            this._editor = editor;
            this._lineNumber = -1;
        }
        MarginComputer.prototype.setLineNumber = function (lineNumber) {
            this._lineNumber = lineNumber;
            this._result = [];
        };
        MarginComputer.prototype.clearResult = function () {
            this._result = [];
        };
        MarginComputer.prototype.computeSync = function () {
            var result = [], lineDecorations = this._editor.getLineDecorations(this._lineNumber), i, len, d;
            for (i = 0, len = lineDecorations.length; i < len; i++) {
                d = lineDecorations[i];
                if (d.options.glyphMarginClassName && d.options.hoverMessage) {
                    result.push({
                        value: d.options.hoverMessage
                    });
                }
            }
            return result;
        };
        MarginComputer.prototype.onResult = function (result, isFromSynchronousComputation) {
            this._result = this._result.concat(result);
        };
        MarginComputer.prototype.getResult = function () {
            return this._result;
        };
        MarginComputer.prototype.getResultWithLoadingMessage = function () {
            return this.getResult();
        };
        return MarginComputer;
    }());
    var ModesGlyphHoverWidget = (function (_super) {
        __extends(ModesGlyphHoverWidget, _super);
        function ModesGlyphHoverWidget(editor) {
            var _this = this;
            _super.call(this, ModesGlyphHoverWidget.ID, editor);
            this._lastLineNumber = -1;
            this._computer = new MarginComputer(this._editor);
            this._hoverOperation = new hoverOperation_1.HoverOperation(this._computer, function (result) { return _this._withResult(result); }, null, function (result) { return _this._withResult(result); });
        }
        ModesGlyphHoverWidget.prototype.onModelDecorationsChanged = function () {
            if (this._isVisible) {
                // The decorations have changed and the hover is visible,
                // we need to recompute the displayed text
                this._hoverOperation.cancel();
                this._computer.clearResult();
                this._hoverOperation.start();
            }
        };
        ModesGlyphHoverWidget.prototype.startShowingAt = function (lineNumber) {
            if (this._lastLineNumber === lineNumber) {
                // We have to show the widget at the exact same line number as before, so no work is needed
                return;
            }
            this._hoverOperation.cancel();
            this.hide();
            this._lastLineNumber = lineNumber;
            this._computer.setLineNumber(lineNumber);
            this._hoverOperation.start();
        };
        ModesGlyphHoverWidget.prototype.hide = function () {
            this._lastLineNumber = -1;
            this._hoverOperation.cancel();
            _super.prototype.hide.call(this);
        };
        ModesGlyphHoverWidget.prototype._withResult = function (result) {
            this._messages = result;
            if (this._messages.length > 0) {
                this._renderMessages(this._lastLineNumber, this._messages);
            }
            else {
                this.hide();
            }
        };
        ModesGlyphHoverWidget.prototype._renderMessages = function (lineNumber, messages) {
            var fragment = document.createDocumentFragment();
            messages.forEach(function (msg) {
                var row = document.createElement('div');
                var span = null;
                if (msg.className) {
                    span = document.createElement('span');
                    span.textContent = msg.value;
                    span.className = msg.className;
                    row.appendChild(span);
                }
                else {
                    row.textContent = msg.value;
                }
                fragment.appendChild(row);
            });
            this._domNode.textContent = '';
            this._domNode.appendChild(fragment);
            // show
            this.showAt(lineNumber);
        };
        ModesGlyphHoverWidget.ID = 'editor.contrib.modesGlyphHoverWidget';
        return ModesGlyphHoverWidget;
    }(hoverWidgets_1.GlyphHoverWidget));
    exports.ModesGlyphHoverWidget = ModesGlyphHoverWidget;
});

define("vs/editor/contrib/quickFix/browser/lightBulpWidget", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/browser/dom', 'vs/editor/common/core/position', 'vs/editor/browser/editorBrowser'], function (require, exports, lifecycle_1, dom, position_1, editorBrowser_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LightBulpWidget = (function () {
        function LightBulpWidget(editor, onclick) {
            // Editor.IContentWidget.allowEditorOverflow
            this.allowEditorOverflow = true;
            this.editor = editor;
            this.onclick = onclick;
            this.toDispose = [];
            this.editor.addContentWidget(this);
        }
        LightBulpWidget.prototype.dispose = function () {
            this.editor.removeContentWidget(this);
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
        };
        LightBulpWidget.prototype.getId = function () {
            return '__lightBulpWidget';
        };
        LightBulpWidget.prototype.getDomNode = function () {
            var _this = this;
            if (!this.domNode) {
                this.domNode = document.createElement('div');
                this.domNode.style.width = '20px';
                this.domNode.style.height = '20px';
                this.domNode.className = 'lightbulp-glyph';
                this.toDispose.push(dom.addDisposableListener(this.domNode, 'click', function (e) {
                    _this.editor.focus();
                    _this.onclick(_this.position);
                }));
            }
            return this.domNode;
        };
        LightBulpWidget.prototype.getPosition = function () {
            return this.visible
                ? { position: this.position, preference: [editorBrowser_1.ContentWidgetPositionPreference.BELOW, editorBrowser_1.ContentWidgetPositionPreference.ABOVE] }
                : null;
        };
        LightBulpWidget.prototype.show = function (where) {
            if (this.visible && position_1.Position.equals(this.position, where)) {
                return;
            }
            this.position = where;
            this.visible = true;
            this.editor.layoutContentWidget(this);
        };
        LightBulpWidget.prototype.hide = function () {
            if (!this.visible) {
                return;
            }
            this.visible = false;
            this.editor.layoutContentWidget(this);
        };
        return LightBulpWidget;
    }());
    exports.LightBulpWidget = LightBulpWidget;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/zoneWidget/browser/zoneWidget", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/base/common/objects', 'vs/base/browser/dom', 'vs/editor/common/editorCommon', 'vs/css!./zoneWidget'], function (require, exports, eventEmitter_1, objects, dom, editorCommon_1) {
    'use strict';
    var defaultOptions = {
        showArrow: true,
        showFrame: true,
        frameColor: '',
        className: ''
    };
    var WIDGET_ID = 'vs.editor.contrib.zoneWidget';
    var ViewZoneDelegate = (function () {
        function ViewZoneDelegate(domNode, afterLineNumber, afterColumn, heightInLines, onDomNodeTop, onComputedHeight) {
            this.domNode = domNode;
            this.afterLineNumber = afterLineNumber;
            this.afterColumn = afterColumn;
            this.heightInLines = heightInLines;
            this._onDomNodeTop = onDomNodeTop;
            this._onComputedHeight = onComputedHeight;
        }
        ViewZoneDelegate.prototype.onDomNodeTop = function (top) {
            this._onDomNodeTop(top);
        };
        ViewZoneDelegate.prototype.onComputedHeight = function (height) {
            this._onComputedHeight(height);
        };
        return ViewZoneDelegate;
    }());
    var OverlayWidgetDelegate = (function () {
        function OverlayWidgetDelegate(id, domNode) {
            this._id = id;
            this._domNode = domNode;
        }
        OverlayWidgetDelegate.prototype.getId = function () {
            return this._id;
        };
        OverlayWidgetDelegate.prototype.getDomNode = function () {
            return this._domNode;
        };
        OverlayWidgetDelegate.prototype.getPosition = function () {
            return null;
        };
        return OverlayWidgetDelegate;
    }());
    // TODO@Joh - this is an event emitter, why?
    var ZoneWidget = (function (_super) {
        __extends(ZoneWidget, _super);
        function ZoneWidget(editor, options) {
            var _this = this;
            if (options === void 0) { options = {}; }
            _super.call(this);
            this.editor = editor;
            this.options = objects.mixin(objects.clone(defaultOptions), options);
            this.zoneId = -1;
            this.overlayWidget = null;
            this.lastView = null;
            this.domNode = document.createElement('div');
            if (!this.options.isAccessible) {
                this.domNode.setAttribute('aria-hidden', 'true');
                this.domNode.setAttribute('role', 'presentation');
            }
            this.container = null;
            this.listenersToRemove = [];
            this.listenersToRemove.push(this.editor.addListener(editorCommon_1.EventType.EditorLayout, function (info) {
                var width = _this.getWidth(info);
                _this.domNode.style.width = width + 'px';
                _this.onWidth(width);
            }));
        }
        ZoneWidget.prototype.create = function () {
            dom.addClass(this.domNode, 'zone-widget');
            dom.addClass(this.domNode, this.options.className);
            this.container = document.createElement('div');
            dom.addClass(this.container, 'zone-widget-container');
            this.domNode.appendChild(this.container);
            this.fillContainer(this.container);
        };
        ZoneWidget.prototype.getWidth = function (info) {
            if (info === void 0) { info = this.editor.getLayoutInfo(); }
            return info.width - info.verticalScrollbarWidth;
        };
        ZoneWidget.prototype.onViewZoneTop = function (top) {
            this.domNode.style.top = top + 'px';
        };
        ZoneWidget.prototype.onViewZoneHeight = function (height) {
            this.domNode.style.height = height + 'px';
            this.doLayout(height - this._decoratingElementsHeight());
        };
        ZoneWidget.prototype.show = function (where, heightInLines) {
            if (typeof where.startLineNumber === 'number') {
                this.showImpl(where, heightInLines);
            }
            else {
                this.showImpl({
                    startLineNumber: where.lineNumber,
                    startColumn: where.column,
                    endLineNumber: where.lineNumber,
                    endColumn: where.column
                }, heightInLines);
            }
        };
        ZoneWidget.prototype._decoratingElementsHeight = function () {
            var lineHeight = this.editor.getConfiguration().lineHeight;
            var result = 0;
            if (this.options.showArrow) {
                var arrowHeight = Math.round(lineHeight / 3);
                result += 2 * arrowHeight;
            }
            if (this.options.showFrame) {
                var frameThickness = Math.round(lineHeight / 9);
                result += 2 * frameThickness;
            }
            return result;
        };
        ZoneWidget.prototype.showImpl = function (where, heightInLines) {
            var _this = this;
            var position = {
                lineNumber: where.startLineNumber,
                column: where.startColumn
            };
            this.domNode.style.width = this.getWidth() + 'px';
            // Reveal position, to get the line rendered, such that the arrow can be positioned properly
            this.editor.revealPosition(position);
            // Render the widget as zone (rendering) and widget (lifecycle)
            var viewZoneDomNode = document.createElement('div'), arrow = document.createElement('div'), lineHeight = this.editor.getConfiguration().lineHeight, arrowHeight = 0, frameThickness = 0;
            // Render the arrow one 1/3 of an editor line height
            if (this.options.showArrow) {
                arrowHeight = Math.round(lineHeight / 3);
                arrow = document.createElement('div');
                arrow.className = 'zone-widget-arrow below';
                arrow.style.top = -arrowHeight + 'px';
                arrow.style.borderWidth = arrowHeight + 'px';
                arrow.style.left = this.editor.getOffsetForColumn(position.lineNumber, position.column) + 'px';
                arrow.style.borderBottomColor = this.options.frameColor;
                viewZoneDomNode.appendChild(arrow);
            }
            // Render the frame as 1/9 of an editor line height
            if (this.options.showFrame) {
                frameThickness = Math.round(lineHeight / 9);
            }
            // insert zone widget
            this.editor.changeViewZones(function (accessor) {
                if (_this.zoneId !== -1) {
                    accessor.removeZone(_this.zoneId);
                }
                if (_this.overlayWidget) {
                    _this.editor.removeOverlayWidget(_this.overlayWidget);
                    _this.overlayWidget = null;
                }
                _this.domNode.style.top = '-1000px';
                var viewZone = new ViewZoneDelegate(viewZoneDomNode, position.lineNumber, position.column, heightInLines, function (top) { return _this.onViewZoneTop(top); }, function (height) { return _this.onViewZoneHeight(height); });
                _this.zoneId = accessor.addZone(viewZone);
                _this.overlayWidget = new OverlayWidgetDelegate(WIDGET_ID + _this.zoneId, _this.domNode);
                _this.editor.addOverlayWidget(_this.overlayWidget);
            });
            if (this.options.showFrame) {
                this.container.style.borderTopColor = this.options.frameColor;
                this.container.style.borderBottomColor = this.options.frameColor;
                this.container.style.borderTopWidth = frameThickness + 'px';
                this.container.style.borderBottomWidth = frameThickness + 'px';
            }
            var containerHeight = heightInLines * lineHeight - this._decoratingElementsHeight();
            this.container.style.top = arrowHeight + 'px';
            this.container.style.height = containerHeight + 'px';
            this.container.style.overflow = 'hidden';
            this.doLayout(containerHeight);
            this.editor.setSelection(where);
            // Reveal the line above or below the zone widget, to get the zone widget in the viewport
            var revealLineNumber = Math.min(this.editor.getModel().getLineCount(), Math.max(1, where.endLineNumber + 1));
            this.editor.revealLine(revealLineNumber);
            this.position = position;
        };
        ZoneWidget.prototype.dispose = function () {
            var _this = this;
            this.listenersToRemove.forEach(function (element) {
                element();
            });
            this.listenersToRemove = [];
            if (this.overlayWidget) {
                this.editor.removeOverlayWidget(this.overlayWidget);
                this.overlayWidget = null;
            }
            if (this.zoneId !== -1) {
                this.editor.changeViewZones(function (accessor) {
                    accessor.removeZone(_this.zoneId);
                    _this.zoneId = -1;
                });
            }
        };
        ZoneWidget.prototype.fillContainer = function (container) {
            // implement in subclass
        };
        ZoneWidget.prototype.onWidth = function (widthInPixel) {
            // implement in subclass
        };
        ZoneWidget.prototype.doLayout = function (heightInPixel) {
            // implement in subclass
        };
        return ZoneWidget;
    }(eventEmitter_1.EventEmitter));
    exports.ZoneWidget = ZoneWidget;
});

define("vs/nls!vs/base/common/processes",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/common/processes", data); });
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/common/processes", ["require", "exports", 'vs/nls!vs/base/common/processes', 'vs/base/common/objects', 'vs/base/common/platform', 'vs/base/common/types', 'vs/base/common/parsers'], function (require, exports, NLS, Objects, Platform, Types, parsers_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (Source) {
        Source[Source["stdout"] = 0] = "stdout";
        Source[Source["stderr"] = 1] = "stderr";
    })(exports.Source || (exports.Source = {}));
    var Source = exports.Source;
    var ExecutableParser = (function (_super) {
        __extends(ExecutableParser, _super);
        function ExecutableParser(logger, validationStatus) {
            if (validationStatus === void 0) { validationStatus = new parsers_1.ValidationStatus(); }
            _super.call(this, logger, validationStatus);
        }
        ExecutableParser.prototype.parse = function (json, parserOptions) {
            if (parserOptions === void 0) { parserOptions = { globals: null, emptyCommand: false, noDefaults: false }; }
            var result = this.parseExecutable(json, parserOptions.globals);
            if (this.status.isFatal()) {
                return result;
            }
            var osExecutable;
            if (json.windows && Platform.platform === Platform.Platform.Windows) {
                osExecutable = this.parseExecutable(json.windows);
            }
            else if (json.osx && Platform.platform === Platform.Platform.Mac) {
                osExecutable = this.parseExecutable(json.osx);
            }
            else if (json.linux && Platform.platform === Platform.Platform.Linux) {
                osExecutable = this.parseExecutable(json.linux);
            }
            if (osExecutable) {
                result = ExecutableParser.mergeExecutable(result, osExecutable);
            }
            if ((!result || !result.command) && !parserOptions.emptyCommand) {
                this.status.state = parsers_1.ValidationState.Fatal;
                this.log(NLS.localize(0, null));
                return null;
            }
            if (!parserOptions.noDefaults) {
                parsers_1.Parser.merge(result, {
                    command: undefined,
                    isShellCommand: false,
                    args: [],
                    options: {}
                }, false);
            }
            return result;
        };
        ExecutableParser.prototype.parseExecutable = function (json, globals) {
            var command = undefined;
            var isShellCommand = undefined;
            var args = undefined;
            var options = undefined;
            if (this.is(json.command, Types.isString)) {
                command = json.command;
            }
            if (this.is(json.isShellCommand, Types.isBoolean, parsers_1.ValidationState.Warning, NLS.localize(1, null, json.isShellCommand))) {
                isShellCommand = json.isShellCommand;
            }
            if (this.is(json.args, Types.isStringArray, parsers_1.ValidationState.Warning, NLS.localize(2, null, json.isShellCommand))) {
                args = json.args.slice(0);
            }
            if (this.is(json.options, Types.isObject)) {
                options = this.parseCommandOptions(json.options);
            }
            return { command: command, isShellCommand: isShellCommand, args: args, options: options };
        };
        ExecutableParser.prototype.parseCommandOptions = function (json) {
            var result = {};
            if (!json) {
                return result;
            }
            if (this.is(json.cwd, Types.isString, parsers_1.ValidationState.Warning, NLS.localize(3, null, json.cwd))) {
                result.cwd = json.cwd;
            }
            if (!Types.isUndefined(json.env)) {
                result.env = Objects.clone(json.env);
            }
            return result;
        };
        ExecutableParser.mergeExecutable = function (executable, other) {
            if (!executable) {
                return other;
            }
            parsers_1.Parser.merge(executable, other, true);
            return executable;
        };
        return ExecutableParser;
    }(parsers_1.Parser));
    exports.ExecutableParser = ExecutableParser;
    function resolveCommandOptions(options, variables) {
        var result = Objects.clone(options);
        if (result.cwd) {
            result.cwd = variables.resolve(result.cwd);
        }
        if (result.env) {
            result.env = variables.resolve(result.env);
        }
        return result;
    }
    exports.resolveCommandOptions = resolveCommandOptions;
    function resolveExecutable(executable, variables) {
        var result = Objects.clone(executable);
        result.command = variables.resolve(result.command);
        result.args = variables.resolve(result.args);
        if (result.options) {
            result.options = resolveCommandOptions(result.options, variables);
        }
        return result;
    }
    exports.resolveExecutable = resolveExecutable;
});

define("vs/nls!vs/base/common/severity",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/common/severity", data); });
define("vs/base/common/severity", ["require", "exports", 'vs/nls!vs/base/common/severity', 'vs/base/common/strings'], function (require, exports, nls, strings) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Severity;
    (function (Severity) {
        Severity[Severity["Ignore"] = 0] = "Ignore";
        Severity[Severity["Info"] = 1] = "Info";
        Severity[Severity["Warning"] = 2] = "Warning";
        Severity[Severity["Error"] = 3] = "Error";
    })(Severity || (Severity = {}));
    var Severity;
    (function (Severity) {
        var _error = 'error', _warning = 'warning', _warn = 'warn', _info = 'info';
        var _displayStrings = Object.create(null);
        _displayStrings[Severity.Error] = nls.localize(0, null);
        _displayStrings[Severity.Warning] = nls.localize(1, null);
        _displayStrings[Severity.Info] = nls.localize(2, null);
        /**
         * Parses 'error', 'warning', 'warn', 'info' in call casings
         * and falls back to ignore.
         */
        function fromValue(value) {
            if (!value) {
                return Severity.Ignore;
            }
            if (strings.equalsIgnoreCase(_error, value)) {
                return Severity.Error;
            }
            if (strings.equalsIgnoreCase(_warning, value) || strings.equalsIgnoreCase(_warn, value)) {
                return Severity.Warning;
            }
            if (strings.equalsIgnoreCase(_info, value)) {
                return Severity.Info;
            }
            return Severity.Ignore;
        }
        Severity.fromValue = fromValue;
        function toString(value) {
            return _displayStrings[value] || strings.empty;
        }
        Severity.toString = toString;
        function compare(a, b) {
            return b - a;
        }
        Severity.compare = compare;
    })(Severity || (Severity = {}));
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = Severity;
});

define("vs/nls!vs/base/node/processes",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/node/processes", data); });
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/node/processes", ["require", "exports", 'path', 'child_process', 'stream', './stdFork', 'vs/nls!vs/base/node/processes', 'vs/base/common/winjs.base', 'vs/base/common/types', 'vs/base/common/uri', 'vs/base/common/objects', 'vs/base/common/paths', 'vs/base/common/platform', 'vs/base/node/decoder', 'vs/base/common/processes'], function (require, exports, path, cp, stream_1, stdFork_1, nls, winjs_base_1, Types, uri_1, Objects, TPath, Platform, decoder_1, processes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var exec = cp.exec;
    var spawn = cp.spawn;
    exports.Source = processes_1.Source;
    function terminateProcess(process, cwd) {
        if (Platform.isWindows) {
            try {
                // This we run in Electron execFileSync is available.
                // Ignore stderr since this is otherwise piped to parent.stderr
                // which might be already closed.
                var options = {
                    stdio: ['pipe', 'pipe', 'ignore']
                };
                if (cwd) {
                    options.cwd = cwd;
                }
                cp.execFileSync('taskkill', ['/T', '/F', '/PID', process.pid.toString()], options);
            }
            catch (err) {
                return { success: false, error: err };
            }
        }
        else if (Platform.isLinux || Platform.isMacintosh) {
            try {
                var cmd = uri_1.default.parse(require.toUrl('vs/base/node/terminateProcess.sh')).fsPath;
                var result = cp.spawnSync(cmd, [process.pid.toString()]);
                if (result.error) {
                    return { success: false, error: result.error };
                }
            }
            catch (err) {
                return { success: false, error: err };
            }
        }
        else {
            process.kill('SIGKILL');
        }
        return { success: true };
    }
    exports.terminateProcess = terminateProcess;
    function getWindowsShell() {
        return process.env['comspec'] || 'cmd.exe';
    }
    exports.getWindowsShell = getWindowsShell;
    var AbstractProcess = (function () {
        function AbstractProcess(arg1, arg2, arg3, arg4) {
            var _this = this;
            if (arg4) {
                this.cmd = arg1;
                this.args = arg2;
                this.shell = arg3;
                this.options = arg4;
            }
            else if (arg3 && arg2) {
                this.module = arg1;
                this.args = arg2;
                this.shell = false;
                this.options = arg3;
            }
            else {
                var executable = arg1;
                this.cmd = executable.command;
                this.shell = executable.isShellCommand;
                this.args = executable.args.slice(0);
                this.options = executable.options || {};
            }
            this.childProcess = null;
            this.terminateRequested = false;
            if (this.options.env) {
                var newEnv_1 = Object.create(null);
                Object.keys(process.env).forEach(function (key) {
                    newEnv_1[key] = process.env[key];
                });
                Object.keys(this.options.env).forEach(function (key) {
                    newEnv_1[key] = _this.options.env[key];
                });
                this.options.env = newEnv_1;
            }
        }
        AbstractProcess.prototype.getSanitizedCommand = function () {
            var result = this.cmd.toLowerCase();
            var index = result.lastIndexOf(path.sep);
            if (index !== -1) {
                result = result.substring(index + 1);
            }
            if (AbstractProcess.WellKnowCommands[result]) {
                return result;
            }
            return 'other';
        };
        AbstractProcess.prototype.start = function () {
            var _this = this;
            if (Platform.isWindows && ((this.options && this.options.cwd && TPath.isUNC(this.options.cwd)) || !this.options && !this.options.cwd && TPath.isUNC(process.cwd()))) {
                return winjs_base_1.Promise.wrapError(nls.localize(0, null));
            }
            return this.useExec().then(function (useExec) {
                var cc;
                var ee;
                var pp;
                var result = new winjs_base_1.PPromise(function (c, e, p) {
                    cc = c;
                    ee = e;
                    pp = p;
                });
                if (useExec) {
                    var cmd = _this.cmd;
                    if (_this.args) {
                        cmd = cmd + ' ' + _this.args.join(' ');
                    }
                    _this.childProcess = exec(cmd, _this.options, function (error, stdout, stderr) {
                        _this.childProcess = null;
                        var err = error;
                        // This is tricky since executing a command shell reports error back in case the executed command return an
                        // error or the command didn't exist at all. So we can't blindly treat an error as a failed command. So we
                        // always parse the output and report success unless the job got killed.
                        if (err && err.killed) {
                            ee({ killed: _this.terminateRequested, stdout: stdout.toString(), stderr: stderr.toString() });
                        }
                        else {
                            _this.handleExec(cc, pp, error, stdout, stderr);
                        }
                    });
                }
                else {
                    var childProcess = null;
                    var closeHandler_1 = function (data) {
                        _this.childProcess = null;
                        _this.childProcessPromise = null;
                        _this.handleClose(data, cc, pp, ee);
                        var result = {
                            terminated: _this.terminateRequested
                        };
                        if (Types.isNumber(data)) {
                            result.cmdCode = data;
                        }
                        cc(result);
                    };
                    if (_this.shell && Platform.isWindows) {
                        var options = Objects.clone(_this.options);
                        options.windowsVerbatimArguments = true;
                        options.detached = false;
                        var quotedCommand = false;
                        var quotedArg_1 = false;
                        var commandLine_1 = [];
                        var quoted_1 = _this.ensureQuotes(_this.cmd);
                        commandLine_1.push(quoted_1.value);
                        quotedCommand = quoted_1.quoted;
                        if (_this.args) {
                            _this.args.forEach(function (elem) {
                                quoted_1 = _this.ensureQuotes(elem);
                                commandLine_1.push(quoted_1.value);
                                quotedArg_1 = quotedArg_1 && quoted_1.quoted;
                            });
                        }
                        var args = [
                            '/s',
                            '/c',
                        ];
                        if (quotedCommand) {
                            if (quotedArg_1) {
                                args.push('"' + commandLine_1.join(' ') + '"');
                            }
                            else if (commandLine_1.length > 1) {
                                args.push('"' + commandLine_1[0] + '"' + ' ' + commandLine_1.slice(1).join(' '));
                            }
                            else {
                                args.push('"' + commandLine_1[0] + '"');
                            }
                        }
                        else {
                            args.push(commandLine_1.join(' '));
                        }
                        childProcess = spawn(getWindowsShell(), args, options);
                    }
                    else {
                        if (_this.cmd) {
                            childProcess = spawn(_this.cmd, _this.args, _this.options);
                        }
                        else if (_this.module) {
                            _this.childProcessPromise = new winjs_base_1.TPromise(function (c, e, p) {
                                stdFork_1.fork(_this.module, _this.args, _this.options, function (error, childProcess) {
                                    if (error) {
                                        e(error);
                                        ee({ terminated: _this.terminateRequested, error: error });
                                        return;
                                    }
                                    _this.childProcess = childProcess;
                                    _this.childProcess.on('close', closeHandler_1);
                                    _this.handleSpawn(childProcess, cc, pp, ee, false);
                                    c(childProcess);
                                });
                            });
                        }
                    }
                    if (childProcess) {
                        _this.childProcess = childProcess;
                        _this.childProcessPromise = winjs_base_1.TPromise.as(childProcess);
                        childProcess.on('error', function (error) {
                            _this.childProcess = null;
                            ee({ terminated: _this.terminateRequested, error: error });
                        });
                        if (childProcess.pid) {
                            _this.childProcess.on('close', closeHandler_1);
                            _this.handleSpawn(childProcess, cc, pp, ee, true);
                        }
                    }
                }
                return result;
            });
        };
        AbstractProcess.prototype.handleClose = function (data, cc, pp, ee) {
            // Default is to do nothing.
        };
        AbstractProcess.prototype.ensureQuotes = function (value) {
            if (AbstractProcess.regexp.test(value)) {
                return {
                    value: '"' + value + '"',
                    quoted: true
                };
            }
            else {
                return {
                    value: value,
                    quoted: value.length > 0 && value[0] === '"' && value[value.length - 1] === '"'
                };
            }
        };
        AbstractProcess.prototype.isRunning = function () {
            return this.childProcessPromise !== null;
        };
        Object.defineProperty(AbstractProcess.prototype, "pid", {
            get: function () {
                return this.childProcessPromise.then(function (childProcess) { return childProcess.pid; }, function (err) { return -1; });
            },
            enumerable: true,
            configurable: true
        });
        AbstractProcess.prototype.terminate = function () {
            var _this = this;
            if (!this.childProcessPromise) {
                return winjs_base_1.TPromise.as({ success: true });
            }
            return this.childProcessPromise.then(function (childProcess) {
                _this.terminateRequested = true;
                var result = terminateProcess(childProcess, _this.options.cwd);
                if (result.success) {
                    _this.childProcess = null;
                }
                return result;
            }, function (err) {
                return { success: true };
            });
        };
        AbstractProcess.prototype.useExec = function () {
            var _this = this;
            return new winjs_base_1.TPromise(function (c, e, p) {
                if (!_this.shell || !Platform.isWindows) {
                    c(false);
                }
                var cmdShell = spawn(getWindowsShell(), ['/s', '/c']);
                cmdShell.on('error', function (error) {
                    c(true);
                });
                cmdShell.on('exit', function (data) {
                    c(false);
                });
            });
        };
        AbstractProcess.WellKnowCommands = {
            'ant': true,
            'cmake': true,
            'eslint': true,
            'gradle': true,
            'grunt': true,
            'gulp': true,
            'jake': true,
            'jenkins': true,
            'jshint': true,
            'make': true,
            'maven': true,
            'msbuild': true,
            'msc': true,
            'nmake': true,
            'npm': true,
            'rake': true,
            'tsc': true,
            'xbuild': true
        };
        AbstractProcess.regexp = /^[^"].* .*[^"]/;
        return AbstractProcess;
    }());
    exports.AbstractProcess = AbstractProcess;
    var LineProcess = (function (_super) {
        __extends(LineProcess, _super);
        function LineProcess(arg1, arg2, arg3, arg4) {
            _super.call(this, arg1, arg2, arg3, arg4);
        }
        LineProcess.prototype.handleExec = function (cc, pp, error, stdout, stderr) {
            [stdout, stderr].forEach(function (buffer, index) {
                var lineDecoder = new decoder_1.LineDecoder();
                var lines = lineDecoder.write(buffer);
                lines.forEach(function (line) {
                    pp({ line: line, source: index === 0 ? processes_1.Source.stdout : processes_1.Source.stderr });
                });
                var line = lineDecoder.end();
                if (line) {
                    pp({ line: line, source: index === 0 ? processes_1.Source.stdout : processes_1.Source.stderr });
                }
            });
            cc({ terminated: this.terminateRequested, error: error });
        };
        LineProcess.prototype.handleSpawn = function (childProcess, cc, pp, ee, sync) {
            var _this = this;
            this.stdoutLineDecoder = new decoder_1.LineDecoder();
            this.stderrLineDecoder = new decoder_1.LineDecoder();
            childProcess.stdout.on('data', function (data) {
                var lines = _this.stdoutLineDecoder.write(data);
                lines.forEach(function (line) { return pp({ line: line, source: processes_1.Source.stdout }); });
            });
            childProcess.stderr.on('data', function (data) {
                var lines = _this.stderrLineDecoder.write(data);
                lines.forEach(function (line) { return pp({ line: line, source: processes_1.Source.stderr }); });
            });
        };
        LineProcess.prototype.handleClose = function (data, cc, pp, ee) {
            [this.stdoutLineDecoder.end(), this.stderrLineDecoder.end()].forEach(function (line, index) {
                if (line) {
                    pp({ line: line, source: index === 0 ? processes_1.Source.stdout : processes_1.Source.stderr });
                }
            });
        };
        return LineProcess;
    }(AbstractProcess));
    exports.LineProcess = LineProcess;
    var BufferProcess = (function (_super) {
        __extends(BufferProcess, _super);
        function BufferProcess(arg1, arg2, arg3, arg4) {
            _super.call(this, arg1, arg2, arg3, arg4);
        }
        BufferProcess.prototype.handleExec = function (cc, pp, error, stdout, stderr) {
            pp({ data: stdout, source: processes_1.Source.stdout });
            pp({ data: stderr, source: processes_1.Source.stderr });
            cc({ terminated: this.terminateRequested, error: error });
        };
        BufferProcess.prototype.handleSpawn = function (childProcess, cc, pp, ee, sync) {
            childProcess.stdout.on('data', function (data) {
                pp({ data: data, source: processes_1.Source.stdout });
            });
            childProcess.stderr.on('data', function (data) {
                pp({ data: data, source: processes_1.Source.stderr });
            });
        };
        return BufferProcess;
    }(AbstractProcess));
    exports.BufferProcess = BufferProcess;
    var StreamProcess = (function (_super) {
        __extends(StreamProcess, _super);
        function StreamProcess(arg1, arg2, arg3, arg4) {
            _super.call(this, arg1, arg2, arg3, arg4);
        }
        StreamProcess.prototype.handleExec = function (cc, pp, error, stdout, stderr) {
            var stdoutStream = new stream_1.PassThrough();
            stdoutStream.end(stdout);
            var stderrStream = new stream_1.PassThrough();
            stderrStream.end(stderr);
            pp({ stdin: null, stdout: stdoutStream, stderr: stderrStream });
            cc({ terminated: this.terminateRequested, error: error });
        };
        StreamProcess.prototype.handleSpawn = function (childProcess, cc, pp, ee, sync) {
            if (sync) {
                process.nextTick(function () {
                    pp({ stdin: childProcess.stdin, stdout: childProcess.stdout, stderr: childProcess.stderr });
                });
            }
            else {
                pp({ stdin: childProcess.stdin, stdout: childProcess.stdout, stderr: childProcess.stderr });
            }
        };
        return StreamProcess;
    }(AbstractProcess));
    exports.StreamProcess = StreamProcess;
});

define("vs/nls!vs/base/node/zip",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/node/zip", data); });
define("vs/nls!vs/base/parts/quickopen/browser/quickOpenModel",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/parts/quickopen/browser/quickOpenModel", data); });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/parts/quickopen/browser/quickOpenModel", ["require", "exports", 'vs/nls!vs/base/parts/quickopen/browser/quickOpenModel', 'vs/base/common/winjs.base', 'vs/base/common/types', 'vs/base/common/filters', 'vs/base/common/strings', 'vs/base/common/paths', 'vs/base/common/comparers', 'vs/base/browser/ui/actionbar/actionbar', 'vs/base/parts/tree/browser/treeDefaults', 'vs/base/browser/ui/highlightedlabel/highlightedLabel', 'vs/base/browser/dom', 'vs/base/common/scorer'], function (require, exports, nls, winjs_base_1, types, filters, strings, paths, comparers_1, actionbar_1, treeDefaults_1, highlightedLabel_1, DOM, scorer) {
    'use strict';
    var IDS = 0;
    var QuickOpenEntry = (function () {
        function QuickOpenEntry(highlights) {
            if (highlights === void 0) { highlights = []; }
            this.id = (IDS++).toString();
            this.labelHighlights = highlights;
            this.descriptionHighlights = [];
        }
        /**
         * A unique identifier for the entry
         */
        QuickOpenEntry.prototype.getId = function () {
            return this.id;
        };
        /**
         * The prefix to show in front of the label if any
         */
        QuickOpenEntry.prototype.getPrefix = function () {
            return this.labelPrefix;
        };
        /**
         * The label of the entry to identify it from others in the list
         */
        QuickOpenEntry.prototype.getLabel = function () {
            return null;
        };
        /**
         * The label of the entry to use when a screen reader wants to read about the entry
         */
        QuickOpenEntry.prototype.getAriaLabel = function () {
            return this.getLabel();
        };
        /**
         * Detail information about the entry that is optional and can be shown below the label
         */
        QuickOpenEntry.prototype.getDetail = function () {
            return null;
        };
        /**
         * The icon of the entry to identify it from others in the list
         */
        QuickOpenEntry.prototype.getIcon = function () {
            return null;
        };
        /**
         * A secondary description that is optional and can be shown right to the label
         */
        QuickOpenEntry.prototype.getDescription = function () {
            return null;
        };
        /**
         * A resource for this entry. Resource URIs can be used to compare different kinds of entries and group
         * them together.
         */
        QuickOpenEntry.prototype.getResource = function () {
            return null;
        };
        /**
         * Allows to reuse the same model while filtering. Hidden entries will not show up in the viewer.
         */
        QuickOpenEntry.prototype.isHidden = function () {
            return this.hidden;
        };
        /**
         * Allows to reuse the same model while filtering. Hidden entries will not show up in the viewer.
         */
        QuickOpenEntry.prototype.setHidden = function (hidden) {
            this.hidden = hidden;
        };
        /**
         * Sets the prefix to show in front of the label
         */
        QuickOpenEntry.prototype.setPrefix = function (prefix) {
            this.labelPrefix = prefix;
        };
        /**
         * Allows to set highlight ranges that should show up for the entry label and optionally description if set.
         */
        QuickOpenEntry.prototype.setHighlights = function (labelHighlights, descriptionHighlights, detailHighlights) {
            this.labelHighlights = labelHighlights;
            this.descriptionHighlights = descriptionHighlights;
            this.detailHighlights = detailHighlights;
        };
        /**
         * Allows to return highlight ranges that should show up for the entry label and description.
         */
        QuickOpenEntry.prototype.getHighlights = function () {
            return [this.labelHighlights, this.descriptionHighlights, this.detailHighlights];
        };
        /**
         * Called when the entry is selected for opening. Returns a boolean value indicating if an action was performed or not.
         * The mode parameter gives an indication if the element is previewed (using arrow keys) or opened.
         *
         * The context parameter provides additional context information how the run was triggered.
         */
        QuickOpenEntry.prototype.run = function (mode, context) {
            return false;
        };
        /**
         * A good default sort implementation for quick open entries respecting highlight information
         * as well as associated resources.
         */
        QuickOpenEntry.compare = function (elementA, elementB, lookFor) {
            // Normalize
            if (lookFor) {
                lookFor = strings.stripWildcards(lookFor).toLowerCase();
            }
            // Give matches with label highlights higher priority over
            // those with only description highlights
            var labelHighlightsA = elementA.getHighlights()[0] || [];
            var labelHighlightsB = elementB.getHighlights()[0] || [];
            if (labelHighlightsA.length && !labelHighlightsB.length) {
                return -1;
            }
            else if (!labelHighlightsA.length && labelHighlightsB.length) {
                return 1;
            }
            // Fallback to the full path if labels are identical and we have associated resources
            var nameA = elementA.getLabel();
            var nameB = elementB.getLabel();
            if (nameA === nameB) {
                var resourceA = elementA.getResource();
                var resourceB = elementB.getResource();
                if (resourceA && resourceB) {
                    nameA = resourceA.fsPath;
                    nameB = resourceB.fsPath;
                }
            }
            return comparers_1.compareAnything(nameA, nameB, lookFor);
        };
        QuickOpenEntry.compareByScore = function (elementA, elementB, lookFor, lookForNormalizedLower, scorerCache) {
            var labelA = elementA.getLabel();
            var labelB = elementB.getLabel();
            // treat prefix matches highest in any case
            var prefixCompare = comparers_1.compareByPrefix(labelA, labelB, lookFor);
            if (prefixCompare) {
                return prefixCompare;
            }
            // Give higher importance to label score
            var labelAScore = scorer.score(labelA, lookFor, scorerCache);
            var labelBScore = scorer.score(labelB, lookFor, scorerCache);
            // Useful for understanding the scoring
            // elementA.setPrefix(labelAScore + ' ');
            // elementB.setPrefix(labelBScore + ' ');
            if (labelAScore !== labelBScore) {
                return labelAScore > labelBScore ? -1 : 1;
            }
            // Score on full resource path comes next (if available)
            var resourceA = elementA.getResource();
            var resourceB = elementB.getResource();
            if (resourceA && resourceB) {
                var resourceAScore = scorer.score(resourceA.fsPath, lookFor, scorerCache);
                var resourceBScore = scorer.score(resourceB.fsPath, lookFor, scorerCache);
                // Useful for understanding the scoring
                // elementA.setPrefix(elementA.getPrefix() + ' ' + resourceAScore + ': ');
                // elementB.setPrefix(elementB.getPrefix() + ' ' + resourceBScore + ': ');
                if (resourceAScore !== resourceBScore) {
                    return resourceAScore > resourceBScore ? -1 : 1;
                }
            }
            // At this place, the scores are identical so we check for string lengths and favor shorter ones
            if (labelA.length !== labelB.length) {
                return labelA.length < labelB.length ? -1 : 1;
            }
            if (resourceA && resourceB && resourceA.fsPath.length !== resourceB.fsPath.length) {
                return resourceA.fsPath.length < resourceB.fsPath.length ? -1 : 1;
            }
            // Finally compare by label or resource path
            if (labelA === labelB && resourceA && resourceB) {
                return comparers_1.compareAnything(resourceA.fsPath, resourceB.fsPath, lookForNormalizedLower);
            }
            return comparers_1.compareAnything(labelA, labelB, lookForNormalizedLower);
        };
        /**
         * A good default highlight implementation for an entry with label and description.
         */
        QuickOpenEntry.highlight = function (entry, lookFor, fuzzyHighlight) {
            if (fuzzyHighlight === void 0) { fuzzyHighlight = false; }
            var labelHighlights = [];
            var descriptionHighlights = [];
            var label = entry.getLabel();
            var description = entry.getDescription();
            // Highlight file aware
            if (entry.getResource()) {
                // Highlight entire label and description if searching for full absolute path
                if (lookFor.toLowerCase() === entry.getResource().fsPath.toLowerCase()) {
                    labelHighlights.push({ start: 0, end: label.length });
                    descriptionHighlights.push({ start: 0, end: description.length });
                }
                else if (fuzzyHighlight || lookFor.indexOf(paths.nativeSep) >= 0) {
                    var candidateLabelHighlights = filters.matchesFuzzy(lookFor, label, fuzzyHighlight);
                    if (!candidateLabelHighlights) {
                        var pathPrefix = description ? (description + paths.nativeSep) : '';
                        var pathPrefixLength_1 = pathPrefix.length;
                        // If there are no highlights in the label, build a path out of description and highlight and match on both,
                        // then extract the individual label and description highlights back to the original positions
                        var pathHighlights = filters.matchesFuzzy(lookFor, pathPrefix + label, fuzzyHighlight);
                        if (pathHighlights) {
                            pathHighlights.forEach(function (h) {
                                // Match overlaps label and description part, we need to split it up
                                if (h.start < pathPrefixLength_1 && h.end > pathPrefixLength_1) {
                                    labelHighlights.push({ start: 0, end: h.end - pathPrefixLength_1 });
                                    descriptionHighlights.push({ start: h.start, end: pathPrefixLength_1 });
                                }
                                else if (h.start >= pathPrefixLength_1) {
                                    labelHighlights.push({ start: h.start - pathPrefixLength_1, end: h.end - pathPrefixLength_1 });
                                }
                                else {
                                    descriptionHighlights.push(h);
                                }
                            });
                        }
                    }
                    else {
                        labelHighlights = candidateLabelHighlights;
                    }
                }
                else {
                    labelHighlights = filters.matchesFuzzy(lookFor, label);
                }
            }
            else {
                labelHighlights = filters.matchesFuzzy(lookFor, label);
            }
            return { labelHighlights: labelHighlights, descriptionHighlights: descriptionHighlights };
        };
        return QuickOpenEntry;
    }());
    exports.QuickOpenEntry = QuickOpenEntry;
    var QuickOpenEntryItem = (function (_super) {
        __extends(QuickOpenEntryItem, _super);
        function QuickOpenEntryItem() {
            _super.apply(this, arguments);
        }
        /**
         * Must return the height as being used by the render function.
         */
        QuickOpenEntryItem.prototype.getHeight = function () {
            return 0;
        };
        /**
         * Allows to present the quick open entry in a custom way inside the tree.
         */
        QuickOpenEntryItem.prototype.render = function (tree, container, previousCleanupFn) {
            return null;
        };
        return QuickOpenEntryItem;
    }(QuickOpenEntry));
    exports.QuickOpenEntryItem = QuickOpenEntryItem;
    var QuickOpenEntryGroup = (function (_super) {
        __extends(QuickOpenEntryGroup, _super);
        function QuickOpenEntryGroup(entry, groupLabel, withBorder) {
            _super.call(this);
            this.entry = entry;
            this.groupLabel = groupLabel;
            this.withBorder = withBorder;
        }
        /**
         * The label of the group or null if none.
         */
        QuickOpenEntryGroup.prototype.getGroupLabel = function () {
            return this.groupLabel;
        };
        QuickOpenEntryGroup.prototype.setGroupLabel = function (groupLabel) {
            this.groupLabel = groupLabel;
        };
        /**
         * Whether to show a border on top of the group entry or not.
         */
        QuickOpenEntryGroup.prototype.showBorder = function () {
            return this.withBorder;
        };
        QuickOpenEntryGroup.prototype.setShowBorder = function (showBorder) {
            this.withBorder = showBorder;
        };
        QuickOpenEntryGroup.prototype.getPrefix = function () {
            return this.entry ? this.entry.getPrefix() : _super.prototype.getPrefix.call(this);
        };
        QuickOpenEntryGroup.prototype.getLabel = function () {
            return this.entry ? this.entry.getLabel() : _super.prototype.getLabel.call(this);
        };
        QuickOpenEntryGroup.prototype.getAriaLabel = function () {
            return this.entry ? this.entry.getAriaLabel() : _super.prototype.getAriaLabel.call(this);
        };
        QuickOpenEntryGroup.prototype.getDetail = function () {
            return this.entry ? this.entry.getDetail() : _super.prototype.getDetail.call(this);
        };
        QuickOpenEntryGroup.prototype.getResource = function () {
            return this.entry ? this.entry.getResource() : _super.prototype.getResource.call(this);
        };
        QuickOpenEntryGroup.prototype.getIcon = function () {
            return this.entry ? this.entry.getIcon() : _super.prototype.getIcon.call(this);
        };
        QuickOpenEntryGroup.prototype.getDescription = function () {
            return this.entry ? this.entry.getDescription() : _super.prototype.getDescription.call(this);
        };
        QuickOpenEntryGroup.prototype.getEntry = function () {
            return this.entry;
        };
        QuickOpenEntryGroup.prototype.getHighlights = function () {
            return this.entry ? this.entry.getHighlights() : _super.prototype.getHighlights.call(this);
        };
        QuickOpenEntryGroup.prototype.isHidden = function () {
            return this.entry ? this.entry.isHidden() : _super.prototype.isHidden.call(this);
        };
        QuickOpenEntryGroup.prototype.setHighlights = function (labelHighlights, descriptionHighlights) {
            this.entry ? this.entry.setHighlights(labelHighlights, descriptionHighlights) : _super.prototype.setHighlights.call(this, labelHighlights, descriptionHighlights);
        };
        QuickOpenEntryGroup.prototype.setHidden = function (hidden) {
            this.entry ? this.entry.setHidden(hidden) : _super.prototype.setHidden.call(this, hidden);
        };
        QuickOpenEntryGroup.prototype.run = function (mode, context) {
            return this.entry ? this.entry.run(mode, context) : _super.prototype.run.call(this, mode, context);
        };
        return QuickOpenEntryGroup;
    }(QuickOpenEntry));
    exports.QuickOpenEntryGroup = QuickOpenEntryGroup;
    var templateEntry = 'quickOpenEntry';
    var templateEntryGroup = 'quickOpenEntryGroup';
    var templateEntryItem = 'quickOpenEntryItem';
    var EntryItemRenderer = (function (_super) {
        __extends(EntryItemRenderer, _super);
        function EntryItemRenderer() {
            _super.apply(this, arguments);
        }
        EntryItemRenderer.prototype.getTemplateId = function (tree, element) {
            return templateEntryItem;
        };
        EntryItemRenderer.prototype.render = function (tree, element, container, previousCleanupFn) {
            if (element instanceof QuickOpenEntryItem) {
                return element.render(tree, container, previousCleanupFn);
            }
            return _super.prototype.render.call(this, tree, element, container, previousCleanupFn);
        };
        return EntryItemRenderer;
    }(treeDefaults_1.LegacyRenderer));
    var NoActionProvider = (function () {
        function NoActionProvider() {
        }
        NoActionProvider.prototype.hasActions = function (tree, element) {
            return false;
        };
        NoActionProvider.prototype.getActions = function (tree, element) {
            return winjs_base_1.TPromise.as(null);
        };
        NoActionProvider.prototype.hasSecondaryActions = function (tree, element) {
            return false;
        };
        NoActionProvider.prototype.getSecondaryActions = function (tree, element) {
            return winjs_base_1.TPromise.as(null);
        };
        NoActionProvider.prototype.getActionItem = function (tree, element, action) {
            return null;
        };
        return NoActionProvider;
    }());
    var Renderer = (function () {
        function Renderer(actionProvider, actionRunner) {
            if (actionProvider === void 0) { actionProvider = new NoActionProvider(); }
            if (actionRunner === void 0) { actionRunner = null; }
            this.actionProvider = actionProvider;
            this.actionRunner = actionRunner;
            this.entryItemRenderer = new EntryItemRenderer();
        }
        Renderer.prototype.getHeight = function (entry) {
            if (entry instanceof QuickOpenEntryItem) {
                return entry.getHeight();
            }
            if (entry.getDetail()) {
                return 44;
            }
            return 22;
        };
        Renderer.prototype.getTemplateId = function (entry) {
            if (entry instanceof QuickOpenEntryItem) {
                return templateEntryItem;
            }
            if (entry instanceof QuickOpenEntryGroup) {
                return templateEntryGroup;
            }
            return templateEntry;
        };
        Renderer.prototype.renderTemplate = function (templateId, container) {
            // Entry Item
            if (templateId === templateEntryItem) {
                return this.entryItemRenderer.renderTemplate(null, templateId, container);
            }
            // Entry Group
            var group;
            if (templateId === templateEntryGroup) {
                group = document.createElement('div');
                DOM.addClass(group, 'results-group');
                container.appendChild(group);
            }
            // Action Bar
            DOM.addClass(container, 'actions');
            var entryContainer = document.createElement('div');
            DOM.addClass(entryContainer, 'sub-content');
            container.appendChild(entryContainer);
            var actionBarContainer = document.createElement('div');
            DOM.addClass(actionBarContainer, 'primary-action-bar');
            container.appendChild(actionBarContainer);
            var actionBar = new actionbar_1.ActionBar(actionBarContainer, {
                actionRunner: this.actionRunner
            });
            // Entry
            var entry = document.createElement('div');
            DOM.addClass(entry, 'quick-open-entry');
            entryContainer.appendChild(entry);
            // Icon
            var icon = document.createElement('span');
            entry.appendChild(icon);
            // Prefix
            var prefix = document.createElement('span');
            entry.appendChild(prefix);
            // Label
            var label = new highlightedLabel_1.HighlightedLabel(entry);
            // Description
            var descriptionContainer = document.createElement('span');
            entry.appendChild(descriptionContainer);
            DOM.addClass(descriptionContainer, 'quick-open-entry-description');
            var description = new highlightedLabel_1.HighlightedLabel(descriptionContainer);
            // Detail
            var detailContainer = document.createElement('div');
            entry.appendChild(detailContainer);
            DOM.addClass(detailContainer, 'quick-open-entry-meta');
            var detail = new highlightedLabel_1.HighlightedLabel(detailContainer);
            return {
                container: container,
                icon: icon,
                prefix: prefix,
                label: label,
                detail: detail,
                description: description,
                group: group,
                actionBar: actionBar
            };
        };
        Renderer.prototype.renderElement = function (entry, templateId, templateData) {
            // Entry Item
            if (templateId === templateEntryItem) {
                this.entryItemRenderer.renderElement(null, entry, templateId, templateData);
                return;
            }
            var data = templateData;
            // Action Bar
            if (this.actionProvider.hasActions(null, entry)) {
                DOM.addClass(data.container, 'has-actions');
            }
            else {
                DOM.removeClass(data.container, 'has-actions');
            }
            data.actionBar.context = entry; // make sure the context is the current element
            this.actionProvider.getActions(null, entry).then(function (actions) {
                // TODO@Ben this will not work anymore as soon as quick open has more actions
                // but as long as there is only one are ok
                if (data.actionBar.isEmpty() && actions && actions.length > 0) {
                    data.actionBar.push(actions, { icon: true, label: false });
                }
                else if (!data.actionBar.isEmpty() && (!actions || actions.length === 0)) {
                    data.actionBar.clear();
                }
            });
            // Entry group
            if (entry instanceof QuickOpenEntryGroup) {
                var group = entry;
                // Border
                if (group.showBorder()) {
                    DOM.addClass(data.container, 'results-group-separator');
                }
                else {
                    DOM.removeClass(data.container, 'results-group-separator');
                }
                // Group Label
                var groupLabel = group.getGroupLabel() || '';
                templateData.group.textContent = groupLabel;
            }
            // Normal Entry
            if (entry instanceof QuickOpenEntry) {
                var _a = entry.getHighlights(), labelHighlights = _a[0], descriptionHighlights = _a[1], detailHighlights = _a[2];
                // Icon
                var iconClass = entry.getIcon() ? ('quick-open-entry-icon ' + entry.getIcon()) : '';
                data.icon.className = iconClass;
                // Prefix
                var prefix = entry.getPrefix() || '';
                data.prefix.textContent = prefix;
                // Label
                data.label.set(entry.getLabel(), labelHighlights || []);
                // Meta
                data.detail.set(entry.getDetail(), detailHighlights);
                // Description
                data.description.set(entry.getDescription(), descriptionHighlights || []);
            }
        };
        Renderer.prototype.disposeTemplate = function (templateId, templateData) {
            if (templateId === templateEntryItem) {
                this.entryItemRenderer.disposeTemplate(null, templateId, templateData);
            }
            else {
                var data = templateData;
                data.actionBar.dispose();
                data.actionBar = null;
                data.container = null;
                data.description.dispose();
                data.description = null;
                data.detail.dispose();
                data.detail = null;
                data.group = null;
                data.icon = null;
                data.label.dispose();
                data.label = null;
                data.prefix = null;
            }
        };
        return Renderer;
    }());
    var QuickOpenModel = (function () {
        function QuickOpenModel(entries, actionProvider) {
            if (entries === void 0) { entries = []; }
            if (actionProvider === void 0) { actionProvider = new NoActionProvider(); }
            this._entries = entries;
            this._dataSource = this;
            this._renderer = new Renderer(actionProvider);
            this._filter = this;
            this._runner = this;
            this._accessibilityProvider = this;
        }
        Object.defineProperty(QuickOpenModel.prototype, "entries", {
            get: function () { return this._entries; },
            set: function (entries) {
                this._entries = entries;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QuickOpenModel.prototype, "dataSource", {
            get: function () { return this._dataSource; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QuickOpenModel.prototype, "renderer", {
            get: function () { return this._renderer; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QuickOpenModel.prototype, "filter", {
            get: function () { return this._filter; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QuickOpenModel.prototype, "runner", {
            get: function () { return this._runner; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QuickOpenModel.prototype, "accessibilityProvider", {
            get: function () { return this._accessibilityProvider; },
            enumerable: true,
            configurable: true
        });
        /**
         * Adds entries that should show up in the quick open viewer.
         */
        QuickOpenModel.prototype.addEntries = function (entries) {
            if (types.isArray(entries)) {
                this._entries = this._entries.concat(entries);
            }
        };
        /**
         * Set the entries that should show up in the quick open viewer.
         */
        QuickOpenModel.prototype.setEntries = function (entries) {
            if (types.isArray(entries)) {
                this._entries = entries;
            }
        };
        /**
         * Get the entries that should show up in the quick open viewer.
         *
         * @visibleOnly optional parameter to only return visible entries
         */
        QuickOpenModel.prototype.getEntries = function (visibleOnly) {
            if (visibleOnly) {
                return this._entries.filter(function (e) { return !e.isHidden(); });
            }
            return this._entries;
        };
        QuickOpenModel.prototype.getId = function (entry) {
            return entry.getId();
        };
        QuickOpenModel.prototype.getLabel = function (entry) {
            return entry.getLabel();
        };
        QuickOpenModel.prototype.getAriaLabel = function (entry) {
            var ariaLabel = entry.getAriaLabel();
            if (ariaLabel) {
                return nls.localize(0, null, entry.getAriaLabel());
            }
            return nls.localize(1, null);
        };
        QuickOpenModel.prototype.isVisible = function (entry) {
            return !entry.isHidden();
        };
        QuickOpenModel.prototype.run = function (entry, mode, context) {
            return entry.run(mode, context);
        };
        return QuickOpenModel;
    }());
    exports.QuickOpenModel = QuickOpenModel;
});

define("vs/nls!vs/base/parts/quickopen/browser/quickOpenWidget",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/base/parts/quickopen/browser/quickOpenWidget", data); });
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/base/parts/quickopen/browser/quickOpenWidget", ["require", "exports", 'vs/nls!vs/base/parts/quickopen/browser/quickOpenWidget', 'vs/base/common/winjs.base', 'vs/base/common/platform', 'vs/base/browser/browser', 'vs/base/common/events', 'vs/base/common/types', 'vs/base/common/errors', 'vs/base/common/uuid', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenViewer', 'vs/base/browser/builder', 'vs/base/browser/ui/inputbox/inputBox', 'vs/base/common/severity', 'vs/base/parts/tree/browser/treeImpl', 'vs/base/browser/ui/progressbar/progressbar', 'vs/base/browser/keyboardEvent', 'vs/base/parts/tree/browser/treeDefaults', 'vs/base/browser/dom', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/css!./quickopen'], function (require, exports, nls, winjs_base_1, platform, browser, events_1, types, errors, uuid, quickOpen_1, quickOpenViewer_1, builder_1, inputBox_1, severity_1, treeImpl_1, progressbar_1, keyboardEvent_1, treeDefaults_1, DOM, keyCodes_1, lifecycle_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var QuickOpenController = (function (_super) {
        __extends(QuickOpenController, _super);
        function QuickOpenController() {
            _super.apply(this, arguments);
        }
        QuickOpenController.prototype.onContextMenu = function (tree, element, event) {
            if (platform.isMacintosh) {
                return this.onLeftClick(tree, element, event); // https://github.com/Microsoft/vscode/issues/1011
            }
            return _super.prototype.onContextMenu.call(this, tree, element, event);
        };
        return QuickOpenController;
    }(treeDefaults_1.DefaultController));
    exports.QuickOpenController = QuickOpenController;
    var DEFAULT_INPUT_ARIA_LABEL = nls.localize(0, null);
    var QuickOpenWidget = (function () {
        function QuickOpenWidget(container, callbacks, options, usageLogger) {
            this.toUnbind = [];
            this.container = container;
            this.callbacks = callbacks;
            this.options = options;
            this.usageLogger = usageLogger;
            this.model = null;
        }
        QuickOpenWidget.prototype.getModel = function () {
            return this.model;
        };
        QuickOpenWidget.prototype.setCallbacks = function (callbacks) {
            this.callbacks = callbacks;
        };
        QuickOpenWidget.prototype.create = function () {
            var _this = this;
            this.builder = builder_1.$().div(function (div) {
                // Eventing
                div.on(DOM.EventType.KEY_DOWN, function (e) {
                    var keyboardEvent = new keyboardEvent_1.StandardKeyboardEvent(e);
                    if (keyboardEvent.keyCode === keyCodes_1.KeyCode.Escape) {
                        DOM.EventHelper.stop(e, true);
                        _this.hide(true);
                    }
                })
                    .on(DOM.EventType.CONTEXT_MENU, function (e) { return DOM.EventHelper.stop(e, true); }) // Do this to fix an issue on Mac where the menu goes into the way
                    .on(DOM.EventType.FOCUS, function (e) { return _this.gainingFocus(); }, null, true)
                    .on(DOM.EventType.BLUR, function (e) { return _this.loosingFocus(e); }, null, true);
                // Progress Bar
                _this.progressBar = new progressbar_1.ProgressBar(div.clone());
                _this.progressBar.getContainer().hide();
                // Input Field
                div.div({ 'class': 'quick-open-input' }, function (inputContainer) {
                    _this.inputContainer = inputContainer;
                    _this.inputBox = new inputBox_1.InputBox(inputContainer.getHTMLElement(), null, {
                        placeholder: _this.options.inputPlaceHolder || '',
                        ariaLabel: DEFAULT_INPUT_ARIA_LABEL
                    });
                    // ARIA
                    _this.inputElement = _this.inputBox.inputElement;
                    _this.inputElement.setAttribute('role', 'combobox');
                    _this.inputElement.setAttribute('aria-haspopup', 'false');
                    _this.inputElement.setAttribute('aria-autocomplete', 'list');
                    DOM.addDisposableListener(_this.inputBox.inputElement, DOM.EventType.KEY_DOWN, function (e) {
                        var keyboardEvent = new keyboardEvent_1.StandardKeyboardEvent(e);
                        // Do not handle Tab: It is used to navigate between elements without mouse
                        if (keyboardEvent.keyCode === keyCodes_1.KeyCode.Tab) {
                            return;
                        }
                        else if (keyboardEvent.keyCode === keyCodes_1.KeyCode.Tab || keyboardEvent.keyCode === keyCodes_1.KeyCode.DownArrow || keyboardEvent.keyCode === keyCodes_1.KeyCode.UpArrow || keyboardEvent.keyCode === keyCodes_1.KeyCode.PageDown || keyboardEvent.keyCode === keyCodes_1.KeyCode.PageUp) {
                            DOM.EventHelper.stop(e, true);
                            _this.navigateInTree(keyboardEvent.keyCode, keyboardEvent.shiftKey);
                        }
                        else if (keyboardEvent.keyCode === keyCodes_1.KeyCode.Enter) {
                            DOM.EventHelper.stop(e, true);
                            var focus_1 = _this.tree.getFocus();
                            if (focus_1) {
                                _this.elementSelected(focus_1, e);
                            }
                        }
                        else if (browser.isIE9 && (keyboardEvent.keyCode === keyCodes_1.KeyCode.Backspace || keyboardEvent.keyCode === keyCodes_1.KeyCode.Delete)) {
                            _this.onType();
                        }
                    });
                    DOM.addDisposableListener(_this.inputBox.inputElement, DOM.EventType.INPUT, function (e) {
                        _this.onType();
                    });
                });
                // Tree
                _this.treeContainer = div.div({
                    'class': 'quick-open-tree'
                }, function (div) {
                    _this.tree = new treeImpl_1.Tree(div.getHTMLElement(), {
                        dataSource: new quickOpenViewer_1.DataSource(_this),
                        controller: new QuickOpenController({ clickBehavior: treeDefaults_1.ClickBehavior.ON_MOUSE_UP }),
                        renderer: new quickOpenViewer_1.Renderer(_this),
                        filter: new quickOpenViewer_1.Filter(_this),
                        accessibilityProvider: new quickOpenViewer_1.AccessibilityProvider(_this)
                    }, {
                        twistiePixels: 11,
                        indentPixels: 0,
                        alwaysFocused: true,
                        verticalScrollMode: 'visible',
                        ariaLabel: nls.localize(1, null)
                    });
                    _this.treeElement = _this.tree.getHTMLElement();
                    // Handle Focus and Selection event
                    _this.toUnbind.push(_this.tree.addListener(events_1.EventType.FOCUS, function (event) {
                        _this.elementFocused(event.focus, event);
                    }));
                    _this.toUnbind.push(_this.tree.addListener(events_1.EventType.SELECTION, function (event) {
                        if (event.selection && event.selection.length > 0) {
                            _this.elementSelected(event.selection[0], event);
                        }
                    }));
                }).
                    on(DOM.EventType.KEY_DOWN, function (e) {
                    var keyboardEvent = new keyboardEvent_1.StandardKeyboardEvent(e);
                    // Only handle when in quick navigation mode
                    if (!_this.quickNavigateConfiguration) {
                        return;
                    }
                    // Support keyboard navigation in quick navigation mode
                    if (keyboardEvent.keyCode === keyCodes_1.KeyCode.DownArrow || keyboardEvent.keyCode === keyCodes_1.KeyCode.UpArrow || keyboardEvent.keyCode === keyCodes_1.KeyCode.PageDown || keyboardEvent.keyCode === keyCodes_1.KeyCode.PageUp) {
                        DOM.EventHelper.stop(e, true);
                        _this.navigateInTree(keyboardEvent.keyCode);
                    }
                }).
                    on(DOM.EventType.KEY_UP, function (e) {
                    var keyboardEvent = new keyboardEvent_1.StandardKeyboardEvent(e);
                    var keyCode = keyboardEvent.keyCode;
                    // Only handle when in quick navigation mode
                    if (!_this.quickNavigateConfiguration) {
                        return;
                    }
                    // Select element when keys are pressed that signal it
                    var quickNavKeys = _this.quickNavigateConfiguration.keybindings;
                    var wasTriggerKeyPressed = keyCode === keyCodes_1.KeyCode.Enter || quickNavKeys.some(function (k) {
                        if (k.hasShift() && keyCode === keyCodes_1.KeyCode.Shift) {
                            if (keyboardEvent.ctrlKey || keyboardEvent.altKey || keyboardEvent.metaKey) {
                                return false; // this is an optimistic check for the shift key being used to navigate back in quick open
                            }
                            return true;
                        }
                        if (k.hasAlt() && keyCode === keyCodes_1.KeyCode.Alt) {
                            return true;
                        }
                        // Mac is a bit special
                        if (platform.isMacintosh) {
                            if (k.hasCtrlCmd() && keyCode === keyCodes_1.KeyCode.Meta) {
                                return true;
                            }
                            if (k.hasWinCtrl() && keyCode === keyCodes_1.KeyCode.Ctrl) {
                                return true;
                            }
                        }
                        else {
                            if (k.hasCtrlCmd() && keyCode === keyCodes_1.KeyCode.Ctrl) {
                                return true;
                            }
                            if (k.hasWinCtrl() && keyCode === keyCodes_1.KeyCode.Meta) {
                                return true;
                            }
                        }
                        return false;
                    });
                    if (wasTriggerKeyPressed) {
                        var focus_2 = _this.tree.getFocus();
                        if (focus_2) {
                            _this.elementSelected(focus_2, e);
                        }
                    }
                }).
                    clone();
            })
                .addClass('quick-open-widget')
                .addClass((browser.isIE10orEarlier) ? ' no-shadow' : '')
                .build(this.container);
            // Support layout
            if (this.layoutDimensions) {
                this.layout(this.layoutDimensions);
            }
        };
        QuickOpenWidget.prototype.onType = function () {
            var value = this.inputBox.value;
            // Adjust help text as needed if present
            if (this.helpText) {
                if (value) {
                    this.helpText.hide();
                }
                else {
                    this.helpText.show();
                }
            }
            // Send to callbacks
            this.callbacks.onType(value);
        };
        QuickOpenWidget.prototype.quickNavigate = function (configuration, next) {
            if (this.isVisible) {
                // Transition into quick navigate mode if not yet done
                if (!this.quickNavigateConfiguration) {
                    this.quickNavigateConfiguration = configuration;
                    this.tree.DOMFocus();
                }
                // Navigate
                this.navigateInTree(next ? keyCodes_1.KeyCode.DownArrow : keyCodes_1.KeyCode.UpArrow);
            }
        };
        QuickOpenWidget.prototype.navigateInTree = function (keyCode, isShift) {
            var model = this.tree.getInput();
            var entries = model ? model.entries : [];
            var focus = this.tree.getFocus();
            var cycled = false;
            var revealToTop = false;
            // Support cycle-through navigation
            if (entries.length > 1) {
                // Up from no entry or first entry goes down to last
                if ((keyCode === keyCodes_1.KeyCode.UpArrow || (keyCode === keyCodes_1.KeyCode.Tab && isShift)) && (focus === entries[0] || !focus)) {
                    this.tree.focusLast();
                    cycled = true;
                }
                else if ((keyCode === keyCodes_1.KeyCode.DownArrow || keyCode === keyCodes_1.KeyCode.Tab && !isShift) && focus === entries[entries.length - 1]) {
                    this.tree.focusFirst();
                    cycled = true;
                }
            }
            // Normal Navigation
            if (!cycled) {
                switch (keyCode) {
                    case keyCodes_1.KeyCode.DownArrow:
                        this.tree.focusNext();
                        break;
                    case keyCodes_1.KeyCode.UpArrow:
                        this.tree.focusPrevious();
                        break;
                    case keyCodes_1.KeyCode.PageDown:
                        this.tree.focusNextPage();
                        break;
                    case keyCodes_1.KeyCode.PageUp:
                        this.tree.focusPreviousPage();
                        break;
                    case keyCodes_1.KeyCode.Tab:
                        if (isShift) {
                            this.tree.focusPrevious();
                        }
                        else {
                            this.tree.focusNext();
                        }
                        break;
                }
            }
            // Reveal
            focus = this.tree.getFocus();
            if (focus) {
                revealToTop ? this.tree.reveal(focus, 0).done(null, errors.onUnexpectedError) : this.tree.reveal(focus).done(null, errors.onUnexpectedError);
            }
        };
        QuickOpenWidget.prototype.elementFocused = function (value, event) {
            if (!value || !this.isVisible()) {
                return;
            }
            // ARIA
            this.inputElement.setAttribute('aria-activedescendant', this.treeElement.getAttribute('aria-activedescendant'));
            var context = { event: event, quickNavigateConfiguration: this.quickNavigateConfiguration };
            this.model.runner.run(value, quickOpen_1.Mode.PREVIEW, context);
        };
        QuickOpenWidget.prototype.elementSelected = function (value, event) {
            var hide = true;
            // Trigger open of element on selection
            if (this.isVisible()) {
                var context = { event: event, quickNavigateConfiguration: this.quickNavigateConfiguration };
                hide = this.model.runner.run(value, quickOpen_1.Mode.OPEN, context);
            }
            // add telemetry when an item is accepted, logging the index of the item in the list and the length of the list
            // to measure the rate of the success and the relevance of the order
            if (this.usageLogger) {
                var indexOfAcceptedElement = this.model.entries.indexOf(value);
                var entriesCount = this.model.entries.length;
                this.usageLogger.publicLog('quickOpenWidgetItemAccepted', { index: indexOfAcceptedElement, count: entriesCount, isQuickNavigate: this.quickNavigateConfiguration ? true : false });
            }
            // Hide if command was run successfully
            if (hide) {
                this.hide();
            }
        };
        QuickOpenWidget.prototype.show = function (param, autoFocus, quickNavigateConfiguration) {
            if (types.isUndefined(autoFocus)) {
                autoFocus = {};
            }
            this.visible = true;
            this.isLoosingFocus = false;
            this.quickNavigateConfiguration = quickNavigateConfiguration;
            // Adjust UI for quick navigate mode
            if (this.quickNavigateConfiguration) {
                this.inputContainer.hide();
                if (this.options.enableAnimations) {
                    this.treeContainer.removeClass('transition');
                }
                this.builder.show();
                this.tree.DOMFocus();
            }
            else {
                this.inputContainer.show();
                if (this.options.enableAnimations) {
                    this.treeContainer.addClass('transition');
                }
                this.builder.show();
                this.inputBox.focus();
            }
            // Adjust Help text for IE
            if (this.helpText) {
                if (this.quickNavigateConfiguration || types.isString(param)) {
                    this.helpText.hide();
                }
                else {
                    this.helpText.show();
                }
            }
            // Show based on param
            if (types.isString(param)) {
                this.doShowWithPrefix(param);
            }
            else {
                this.doShowWithInput(param, autoFocus);
            }
            if (this.callbacks.onShow) {
                this.callbacks.onShow();
            }
        };
        QuickOpenWidget.prototype.doShowWithPrefix = function (prefix) {
            this.inputBox.value = prefix;
            this.callbacks.onType(prefix);
        };
        QuickOpenWidget.prototype.doShowWithInput = function (input, autoFocus) {
            this.setInput(input, autoFocus);
        };
        QuickOpenWidget.prototype.setInputAndLayout = function (input, autoFocus) {
            var _this = this;
            // Use a generated token to avoid race conditions from setting input
            var currentInputToken = uuid.generateUuid();
            this.currentInputToken = currentInputToken;
            // setInput and Layout
            this.setTreeHeightForInput(input).then(function () {
                if (_this.currentInputToken === currentInputToken) {
                    _this.tree.setInput(null).then(function () {
                        _this.model = input;
                        // ARIA
                        _this.inputElement.setAttribute('aria-haspopup', String(input && input.entries && input.entries.length > 0));
                        return _this.tree.setInput(input);
                    }).done(function () {
                        // Indicate entries to tree
                        _this.tree.layout();
                        // Handle auto focus
                        if (input && input.entries.some(function (e) { return _this.isElementVisible(input, e); })) {
                            _this.autoFocus(input, autoFocus);
                        }
                    }, errors.onUnexpectedError);
                }
            });
        };
        QuickOpenWidget.prototype.isElementVisible = function (input, e) {
            if (!input.filter) {
                return true;
            }
            return input.filter.isVisible(e);
        };
        QuickOpenWidget.prototype.autoFocus = function (input, autoFocus) {
            var _this = this;
            if (autoFocus === void 0) { autoFocus = {}; }
            var entries = input.entries.filter(function (e) { return _this.isElementVisible(input, e); });
            // First check for auto focus of prefix matches
            if (autoFocus.autoFocusPrefixMatch) {
                var caseSensitiveMatch = void 0;
                var caseInsensitiveMatch = void 0;
                var prefix = autoFocus.autoFocusPrefixMatch;
                var lowerCasePrefix = prefix.toLowerCase();
                for (var i = 0; i < entries.length; i++) {
                    var entry = entries[i];
                    var label = input.dataSource.getLabel(entry);
                    if (!caseSensitiveMatch && label.indexOf(prefix) === 0) {
                        caseSensitiveMatch = entry;
                    }
                    else if (!caseInsensitiveMatch && label.toLowerCase().indexOf(lowerCasePrefix) === 0) {
                        caseInsensitiveMatch = entry;
                    }
                    if (caseSensitiveMatch && caseInsensitiveMatch) {
                        break;
                    }
                }
                var entryToFocus = caseSensitiveMatch || caseInsensitiveMatch;
                if (entryToFocus) {
                    this.tree.setFocus(entryToFocus);
                    this.tree.reveal(entryToFocus, 0).done(null, errors.onUnexpectedError);
                    return;
                }
            }
            // Second check for auto focus of first entry
            if (autoFocus.autoFocusFirstEntry) {
                this.tree.focusFirst();
                this.tree.reveal(this.tree.getFocus(), 0).done(null, errors.onUnexpectedError);
            }
            else if (typeof autoFocus.autoFocusIndex === 'number') {
                if (entries.length > autoFocus.autoFocusIndex) {
                    this.tree.focusNth(autoFocus.autoFocusIndex);
                    this.tree.reveal(this.tree.getFocus()).done(null, errors.onUnexpectedError);
                }
            }
            else if (autoFocus.autoFocusSecondEntry) {
                if (entries.length > 1) {
                    this.tree.focusNth(1);
                }
            }
        };
        QuickOpenWidget.prototype.refresh = function (input, autoFocus) {
            var _this = this;
            if (!this.isVisible()) {
                return;
            }
            // Apply height & Refresh
            this.setTreeHeightForInput(input).then(function () {
                _this.tree.refresh().done(function () {
                    // Indicate entries to tree
                    _this.tree.layout();
                    // Handle auto focus
                    if (!_this.tree.getFocus() && input && input.entries.some(function (e) { return _this.isElementVisible(input, e); })) {
                        _this.autoFocus(input, autoFocus);
                    }
                }, errors.onUnexpectedError);
            });
        };
        QuickOpenWidget.prototype.setTreeHeightForInput = function (input) {
            var _this = this;
            var newHeight = this.getHeight(input) + 'px';
            var oldHeight = this.treeContainer.style('height');
            // Apply
            this.treeContainer.style({ height: newHeight });
            // Return instantly if we don't CSS transition or the height is the same as old
            if (!this.treeContainer.hasClass('transition') || oldHeight === newHeight) {
                return winjs_base_1.TPromise.as(null);
            }
            // Otherwise return promise that only fulfills when the CSS transition has ended
            return new winjs_base_1.TPromise(function (c, e) {
                var unbind = [];
                var complete = false;
                var completeHandler = function () {
                    if (!complete) {
                        complete = true;
                        unbind = lifecycle_1.disposeAll(unbind);
                        c(null);
                    }
                };
                _this.treeContainer.once('webkitTransitionEnd', completeHandler, unbind);
                _this.treeContainer.once('transitionend', completeHandler, unbind);
            });
        };
        QuickOpenWidget.prototype.getHeight = function (input) {
            var _this = this;
            var renderer = input.renderer;
            if (!input) {
                var itemHeight = renderer.getHeight(null);
                return this.options.minItemsToShow ? this.options.minItemsToShow * itemHeight : 0;
            }
            var height = 0;
            var preferredItemsHeight;
            if (this.layoutDimensions && this.layoutDimensions.height) {
                preferredItemsHeight = (this.layoutDimensions.height - 50 /* subtract height of input field (30px) and some spacing (drop shadow) to fit */) * 0.40 /* max 40% of screen */;
            }
            if (!preferredItemsHeight || preferredItemsHeight > QuickOpenWidget.MAX_ITEMS_HEIGHT) {
                preferredItemsHeight = QuickOpenWidget.MAX_ITEMS_HEIGHT;
            }
            var entries = input.entries.filter(function (e) { return _this.isElementVisible(input, e); });
            var maxEntries = this.options.maxItemsToShow || entries.length;
            for (var i = 0; i < maxEntries && i < entries.length; i++) {
                var entryHeight = renderer.getHeight(entries[i]);
                if (height + entryHeight <= preferredItemsHeight) {
                    height += entryHeight;
                }
                else {
                    break;
                }
            }
            return height;
        };
        QuickOpenWidget.prototype.hide = function (isCancel) {
            var _this = this;
            if (isCancel === void 0) { isCancel = false; }
            if (!this.isVisible()) {
                return;
            }
            this.visible = false;
            this.builder.hide();
            this.builder.domBlur();
            // report failure cases
            if (isCancel) {
                if (this.model) {
                    var entriesCount = this.model.entries.filter(function (e) { return _this.isElementVisible(_this.model, e); }).length;
                    if (this.usageLogger) {
                        this.usageLogger.publicLog('quickOpenWidgetCancelled', { count: entriesCount, isQuickNavigate: this.quickNavigateConfiguration ? true : false });
                    }
                }
            }
            // Clear input field and clear tree
            this.inputBox.value = '';
            this.tree.setInput(null);
            // ARIA
            this.inputElement.setAttribute('aria-haspopup', 'false');
            // Reset Tree Height
            this.treeContainer.style({ height: (this.options.minItemsToShow ? this.options.minItemsToShow * 22 : 0) + 'px' });
            // Clear any running Progress
            this.progressBar.stop().getContainer().hide();
            // Clear Focus
            if (this.tree.isDOMFocused()) {
                this.tree.DOMBlur();
            }
            else if (this.inputBox.hasFocus()) {
                this.inputBox.blur();
            }
            // Callbacks
            if (isCancel) {
                this.callbacks.onCancel();
            }
            else {
                this.callbacks.onOk();
            }
            if (this.callbacks.onHide) {
                this.callbacks.onHide();
            }
        };
        QuickOpenWidget.prototype.setPlaceHolder = function (placeHolder) {
            if (this.inputBox) {
                this.inputBox.setPlaceHolder(placeHolder);
            }
        };
        QuickOpenWidget.prototype.setValue = function (value) {
            if (this.inputBox) {
                this.inputBox.value = value;
                this.inputBox.select();
            }
        };
        QuickOpenWidget.prototype.setPassword = function (isPassword) {
            if (this.inputBox) {
                this.inputBox.inputElement.type = isPassword ? 'password' : 'text';
            }
        };
        QuickOpenWidget.prototype.setInput = function (input, autoFocus, ariaLabel) {
            if (!this.isVisible()) {
                return;
            }
            // Adapt tree height to entries and apply input
            this.setInputAndLayout(input, autoFocus);
            // Apply ARIA
            if (this.inputBox) {
                this.inputBox.setAriaLabel(ariaLabel || DEFAULT_INPUT_ARIA_LABEL);
            }
        };
        QuickOpenWidget.prototype.getInput = function () {
            return this.tree.getInput();
        };
        QuickOpenWidget.prototype.showInputDecoration = function (decoration) {
            if (this.inputBox) {
                this.inputBox.showMessage({ type: decoration === severity_1.default.Info ? inputBox_1.MessageType.INFO : decoration === severity_1.default.Warning ? inputBox_1.MessageType.WARNING : inputBox_1.MessageType.ERROR, content: '' });
            }
        };
        QuickOpenWidget.prototype.clearInputDecoration = function () {
            if (this.inputBox) {
                this.inputBox.hideMessage();
            }
        };
        QuickOpenWidget.prototype.runFocus = function () {
            var focus = this.tree.getFocus();
            if (focus) {
                this.elementSelected(focus);
                return true;
            }
            return false;
        };
        QuickOpenWidget.prototype.getProgressBar = function () {
            return this.progressBar;
        };
        QuickOpenWidget.prototype.setExtraClass = function (clazz) {
            var previousClass = this.builder.getProperty('extra-class');
            if (previousClass) {
                this.builder.removeClass(previousClass);
            }
            if (clazz) {
                this.builder.addClass(clazz);
                this.builder.setProperty('extra-class', clazz);
            }
            else if (previousClass) {
                this.builder.removeProperty('extra-class');
            }
        };
        QuickOpenWidget.prototype.isVisible = function () {
            return this.visible;
        };
        QuickOpenWidget.prototype.layout = function (dimension) {
            this.layoutDimensions = dimension;
            // Apply to quick open width (height is dynamic by number of items to show)
            var quickOpenWidth = Math.min(this.layoutDimensions.width * 0.62 /* golden cut */, QuickOpenWidget.MAX_WIDTH);
            if (this.builder) {
                // quick open
                this.builder.style({
                    width: quickOpenWidth + 'px',
                    marginLeft: '-' + (quickOpenWidth / 2) + 'px'
                });
                // input field
                this.inputContainer.style({
                    width: (quickOpenWidth - 12) + 'px'
                });
            }
        };
        QuickOpenWidget.prototype.gainingFocus = function () {
            this.isLoosingFocus = false;
        };
        QuickOpenWidget.prototype.loosingFocus = function (e) {
            var _this = this;
            if (!this.isVisible()) {
                return;
            }
            var relatedTarget = e.relatedTarget;
            if (!this.quickNavigateConfiguration && DOM.isAncestor(relatedTarget, this.builder.getHTMLElement())) {
                return; // user clicked somewhere into quick open widget, do not close thereby
            }
            this.isLoosingFocus = true;
            winjs_base_1.TPromise.timeout(0).then(function () {
                if (!_this.isLoosingFocus) {
                    return;
                }
                var veto = _this.callbacks.onFocusLost && _this.callbacks.onFocusLost();
                if (!veto) {
                    _this.hide(false /* Do not treat loosing focus as cancel! */);
                }
            });
        };
        QuickOpenWidget.prototype.dispose = function () {
            while (this.toUnbind.length) {
                this.toUnbind.pop()();
            }
            this.progressBar.dispose();
            this.inputBox.dispose();
            this.tree.dispose();
        };
        QuickOpenWidget.MAX_WIDTH = 600; // Max total width of quick open widget
        QuickOpenWidget.MAX_ITEMS_HEIGHT = 20 * 22; // Max height of item list below input field
        return QuickOpenWidget;
    }());
    exports.QuickOpenWidget = QuickOpenWidget;
});

define("vs/nls!vs/editor/common/commonCodeEditor",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/common/commonCodeEditor", data); });
define("vs/nls!vs/editor/common/config/commonEditorConfig",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/common/config/commonEditorConfig", data); });
define("vs/nls!vs/editor/common/config/defaultConfig",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/common/config/defaultConfig", data); });
define("vs/editor/common/config/defaultConfig", ["require", "exports", 'vs/nls!vs/editor/common/config/defaultConfig'], function (require, exports, nls) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.USUAL_WORD_SEPARATORS = '`~!@#$%^&*()-=+[{]}\\|;:\'",.<>/?';
    exports.DEFAULT_INDENTATION = {
        tabSize: 4,
        insertSpaces: true,
        detectIndentation: true
    };
    var ConfigClass = (function () {
        function ConfigClass() {
            this.editor = {
                experimentalScreenReader: true,
                rulers: [],
                wordSeparators: exports.USUAL_WORD_SEPARATORS,
                selectionClipboard: false,
                ariaLabel: nls.localize(0, null),
                lineNumbers: true,
                selectOnLineNumbers: true,
                lineNumbersMinChars: 5,
                glyphMargin: false,
                lineDecorationsWidth: 10,
                revealHorizontalRightPadding: 30,
                roundedSelection: true,
                theme: 'vs',
                readOnly: false,
                scrollbar: {
                    verticalScrollbarSize: 14,
                    horizontal: 'auto',
                    useShadows: true,
                    verticalHasArrows: false,
                    horizontalHasArrows: false
                },
                overviewRulerLanes: 2,
                cursorBlinking: 'blink',
                cursorStyle: 'line',
                fontLigatures: false,
                hideCursorInOverviewRuler: false,
                scrollBeyondLastLine: true,
                automaticLayout: false,
                wrappingColumn: 300,
                wrappingIndent: 'same',
                wordWrapBreakBeforeCharacters: '{([+',
                wordWrapBreakAfterCharacters: ' \t})]?|&,;',
                wordWrapBreakObtrusiveCharacters: '.',
                tabFocusMode: false,
                // stopLineTokenizationAfter
                // stopRenderingLineAfter
                longLineBoundary: 300,
                forcedTokenizationBoundary: 1000,
                // Features
                hover: true,
                contextmenu: true,
                mouseWheelScrollSensitivity: 1,
                quickSuggestions: true,
                quickSuggestionsDelay: 10,
                iconsInSuggestions: true,
                autoClosingBrackets: true,
                formatOnType: false,
                suggestOnTriggerCharacters: true,
                acceptSuggestionOnEnter: true,
                selectionHighlight: true,
                outlineMarkers: false,
                referenceInfos: true,
                folding: true,
                renderWhitespace: false,
                fontFamily: '',
                fontSize: 0,
                lineHeight: 0
            };
        }
        return ConfigClass;
    }());
    exports.DefaultConfig = new ConfigClass();
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/model/textModel", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/base/common/strings', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/model/modelLine', 'vs/editor/common/model/indentationGuesser', 'vs/editor/common/config/defaultConfig'], function (require, exports, eventEmitter_1, strings, position_1, range_1, editorCommon, modelLine_1, indentationGuesser_1, defaultConfig_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LIMIT_FIND_COUNT = 999;
    var TextModel = (function (_super) {
        __extends(TextModel, _super);
        function TextModel(allowedEventTypes, rawText) {
            allowedEventTypes.push(editorCommon.EventType.ModelContentChanged, editorCommon.EventType.ModelOptionsChanged);
            _super.call(this, allowedEventTypes);
            this._options = rawText.options;
            this._constructLines(rawText);
            this._setVersionId(1);
            this._isDisposed = false;
            this._isDisposing = false;
        }
        TextModel.prototype.getOptions = function () {
            if (this._isDisposed) {
                throw new Error('TextModel.getOptions: Model is disposed');
            }
            return this._options;
        };
        TextModel.prototype.updateOptions = function (newOpts) {
            var somethingChanged = false;
            var changed = {
                tabSize: false,
                insertSpaces: false
            };
            if (typeof newOpts.insertSpaces !== 'undefined') {
                if (this._options.insertSpaces !== newOpts.insertSpaces) {
                    somethingChanged = true;
                    changed.insertSpaces = true;
                    this._options.insertSpaces = newOpts.insertSpaces;
                }
            }
            if (typeof newOpts.tabSize !== 'undefined') {
                if (this._options.tabSize !== newOpts.tabSize) {
                    somethingChanged = true;
                    changed.tabSize = true;
                    this._options.tabSize = newOpts.tabSize;
                }
            }
            if (somethingChanged) {
                this.emit(editorCommon.EventType.ModelOptionsChanged, changed);
            }
        };
        TextModel.prototype.detectIndentation = function (defaultInsertSpaces, defaultTabSize) {
            var lines = this._lines.map(function (line) { return line.text; });
            var guessedIndentation = indentationGuesser_1.guessIndentation(lines, defaultTabSize, defaultInsertSpaces);
            this.updateOptions({
                insertSpaces: guessedIndentation.insertSpaces,
                tabSize: guessedIndentation.tabSize
            });
        };
        TextModel.prototype._normalizeIndentationFromWhitespace = function (str) {
            var tabSize = this._options.tabSize;
            var insertSpaces = this._options.insertSpaces;
            var spacesCnt = 0;
            for (var i = 0; i < str.length; i++) {
                if (str.charAt(i) === '\t') {
                    spacesCnt += tabSize;
                }
                else {
                    spacesCnt++;
                }
            }
            var result = '';
            if (!insertSpaces) {
                var tabsCnt = Math.floor(spacesCnt / tabSize);
                spacesCnt = spacesCnt % tabSize;
                for (var i = 0; i < tabsCnt; i++) {
                    result += '\t';
                }
            }
            for (var i = 0; i < spacesCnt; i++) {
                result += ' ';
            }
            return result;
        };
        TextModel.prototype.normalizeIndentation = function (str) {
            var firstNonWhitespaceIndex = strings.firstNonWhitespaceIndex(str);
            if (firstNonWhitespaceIndex === -1) {
                firstNonWhitespaceIndex = str.length;
            }
            return this._normalizeIndentationFromWhitespace(str.substring(0, firstNonWhitespaceIndex)) + str.substring(firstNonWhitespaceIndex);
        };
        TextModel.prototype.getOneIndent = function () {
            var tabSize = this._options.tabSize;
            var insertSpaces = this._options.insertSpaces;
            if (insertSpaces) {
                var result = '';
                for (var i = 0; i < tabSize; i++) {
                    result += ' ';
                }
                return result;
            }
            else {
                return '\t';
            }
        };
        TextModel.prototype.getVersionId = function () {
            if (this._isDisposed) {
                throw new Error('TextModel.getVersionId: Model is disposed');
            }
            return this._versionId;
        };
        TextModel.prototype.getAlternativeVersionId = function () {
            if (this._isDisposed) {
                throw new Error('TextModel.getAlternativeVersionId: Model is disposed');
            }
            return this._alternativeVersionId;
        };
        TextModel.prototype._increaseVersionId = function () {
            this._setVersionId(this._versionId + 1);
        };
        TextModel.prototype._setVersionId = function (newVersionId) {
            this._versionId = newVersionId;
            this._alternativeVersionId = this._versionId;
        };
        TextModel.prototype._overwriteAlternativeVersionId = function (newAlternativeVersionId) {
            this._alternativeVersionId = newAlternativeVersionId;
        };
        TextModel.prototype.isDisposed = function () {
            return this._isDisposed;
        };
        TextModel.prototype.dispose = function () {
            if (this._isDisposed) {
                throw new Error('TextModel.dispose: Model is disposed');
            }
            this._isDisposed = true;
            // Null out members, such that any use of a disposed model will throw exceptions sooner rather than later
            this._lines = null;
            this._EOL = null;
            this._BOM = null;
            _super.prototype.dispose.call(this);
        };
        TextModel.prototype._createContentChangedFlushEvent = function () {
            return {
                changeType: editorCommon.EventType.ModelContentChangedFlush,
                detail: null,
                // TODO@Alex -> remove these fields from here
                versionId: -1,
                isUndoing: false,
                isRedoing: false
            };
        };
        TextModel.prototype._emitContentChanged2 = function (startLineNumber, startColumn, endLineNumber, endColumn, rangeLength, text, isUndoing, isRedoing) {
            var e = {
                range: new range_1.Range(startLineNumber, startColumn, endLineNumber, endColumn),
                rangeLength: rangeLength,
                text: text,
                eol: this._EOL,
                versionId: this.getVersionId(),
                isUndoing: isUndoing,
                isRedoing: isRedoing
            };
            if (!this._isDisposing) {
                this.emit(editorCommon.EventType.ModelContentChanged2, e);
            }
        };
        TextModel.prototype._resetValue = function (e, newValue) {
            this._constructLines(newValue);
            this._increaseVersionId();
            e.detail = this.toRawText();
            e.versionId = this._versionId;
        };
        TextModel.prototype.toRawText = function () {
            return {
                BOM: this._BOM,
                EOL: this._EOL,
                lines: this.getLinesContent(),
                length: this.getValueLength(),
                options: this._options
            };
        };
        TextModel.prototype.equals = function (other) {
            if (this._BOM !== other.BOM) {
                return false;
            }
            if (this._EOL !== other.EOL) {
                return false;
            }
            if (this._lines.length !== other.lines.length) {
                return false;
            }
            for (var i = 0, len = this._lines.length; i < len; i++) {
                if (this._lines[i].text !== other.lines[i]) {
                    return false;
                }
            }
            return true;
        };
        TextModel.prototype.setValue = function (value) {
            if (this._isDisposed) {
                throw new Error('TextModel.setValue: Model is disposed');
            }
            var rawText = null;
            if (value !== null) {
                rawText = TextModel.toRawText(value, {
                    tabSize: this._options.tabSize,
                    insertSpaces: this._options.insertSpaces,
                    detectIndentation: false,
                    defaultEOL: this._options.defaultEOL
                });
            }
            this.setValueFromRawText(rawText);
        };
        TextModel.prototype.setValueFromRawText = function (newValue) {
            if (this._isDisposed) {
                throw new Error('TextModel.setValueFromRawText: Model is disposed');
            }
            if (newValue === null) {
                // There's nothing to do
                return;
            }
            var oldFullModelRange = this.getFullModelRange();
            var oldModelValueLength = this.getValueLengthInRange(oldFullModelRange);
            var endLineNumber = this.getLineCount();
            var endColumn = this.getLineMaxColumn(endLineNumber);
            var e = this._createContentChangedFlushEvent();
            this._resetValue(e, newValue);
            this._emitModelContentChangedFlushEvent(e);
            this._emitContentChanged2(1, 1, endLineNumber, endColumn, oldModelValueLength, this.getValue(), false, false);
        };
        TextModel.prototype.getValue = function (eol, preserveBOM) {
            if (preserveBOM === void 0) { preserveBOM = false; }
            if (this._isDisposed) {
                throw new Error('TextModel.getValue: Model is disposed');
            }
            var fullModelRange = this.getFullModelRange();
            var fullModelValue = this.getValueInRange(fullModelRange, eol);
            if (preserveBOM) {
                return this._BOM + fullModelValue;
            }
            return fullModelValue;
        };
        TextModel.prototype.getValueLength = function (eol, preserveBOM) {
            if (preserveBOM === void 0) { preserveBOM = false; }
            if (this._isDisposed) {
                throw new Error('TextModel.getValueLength: Model is disposed');
            }
            var fullModelRange = this.getFullModelRange();
            var fullModelValue = this.getValueLengthInRange(fullModelRange, eol);
            if (preserveBOM) {
                return this._BOM.length + fullModelValue;
            }
            return fullModelValue;
        };
        TextModel.prototype.getEmptiedValueInRange = function (rawRange, fillCharacter, eol) {
            if (fillCharacter === void 0) { fillCharacter = ''; }
            if (eol === void 0) { eol = editorCommon.EndOfLinePreference.TextDefined; }
            if (this._isDisposed) {
                throw new Error('TextModel.getEmptiedValueInRange: Model is disposed');
            }
            var range = this.validateRange(rawRange);
            if (range.isEmpty()) {
                return '';
            }
            if (range.startLineNumber === range.endLineNumber) {
                return this._repeatCharacter(fillCharacter, range.endColumn - range.startColumn);
            }
            var lineEnding = this._getEndOfLine(eol), startLineIndex = range.startLineNumber - 1, endLineIndex = range.endLineNumber - 1, resultLines = [];
            resultLines.push(this._repeatCharacter(fillCharacter, this._lines[startLineIndex].text.length - range.startColumn + 1));
            for (var i = startLineIndex + 1; i < endLineIndex; i++) {
                resultLines.push(this._repeatCharacter(fillCharacter, this._lines[i].text.length));
            }
            resultLines.push(this._repeatCharacter(fillCharacter, range.endColumn - 1));
            return resultLines.join(lineEnding);
        };
        TextModel.prototype._repeatCharacter = function (fillCharacter, count) {
            var r = '';
            for (var i = 0; i < count; i++) {
                r += fillCharacter;
            }
            return r;
        };
        TextModel.prototype.getValueInRange = function (rawRange, eol) {
            if (eol === void 0) { eol = editorCommon.EndOfLinePreference.TextDefined; }
            if (this._isDisposed) {
                throw new Error('TextModel.getValueInRange: Model is disposed');
            }
            var range = this.validateRange(rawRange);
            if (range.isEmpty()) {
                return '';
            }
            if (range.startLineNumber === range.endLineNumber) {
                return this._lines[range.startLineNumber - 1].text.substring(range.startColumn - 1, range.endColumn - 1);
            }
            var lineEnding = this._getEndOfLine(eol), startLineIndex = range.startLineNumber - 1, endLineIndex = range.endLineNumber - 1, resultLines = [];
            resultLines.push(this._lines[startLineIndex].text.substring(range.startColumn - 1));
            for (var i = startLineIndex + 1; i < endLineIndex; i++) {
                resultLines.push(this._lines[i].text);
            }
            resultLines.push(this._lines[endLineIndex].text.substring(0, range.endColumn - 1));
            return resultLines.join(lineEnding);
        };
        TextModel.prototype.getValueLengthInRange = function (rawRange, eol) {
            if (eol === void 0) { eol = editorCommon.EndOfLinePreference.TextDefined; }
            if (this._isDisposed) {
                throw new Error('TextModel.getValueInRange: Model is disposed');
            }
            var range = this.validateRange(rawRange);
            if (range.isEmpty()) {
                return 0;
            }
            if (range.startLineNumber === range.endLineNumber) {
                return (range.endColumn - range.startColumn);
            }
            var lineEndingLength = this._getEndOfLine(eol).length, startLineIndex = range.startLineNumber - 1, endLineIndex = range.endLineNumber - 1, result = 0;
            result += (this._lines[startLineIndex].text.length - range.startColumn + 1);
            for (var i = startLineIndex + 1; i < endLineIndex; i++) {
                result += lineEndingLength + this._lines[i].text.length;
            }
            result += lineEndingLength + (range.endColumn - 1);
            return result;
        };
        TextModel.prototype.isDominatedByLongLines = function (longLineBoundary) {
            if (this._isDisposed) {
                throw new Error('TextModel.isDominatedByLongLines: Model is disposed');
            }
            var smallLineCharCount = 0, longLineCharCount = 0, i, len, lines = this._lines, lineLength;
            for (i = 0, len = this._lines.length; i < len; i++) {
                lineLength = lines[i].text.length;
                if (lineLength >= longLineBoundary) {
                    longLineCharCount += lineLength;
                }
                else {
                    smallLineCharCount += lineLength;
                }
            }
            return (longLineCharCount > smallLineCharCount);
        };
        TextModel.prototype.getLineCount = function () {
            if (this._isDisposed) {
                throw new Error('TextModel.getLineCount: Model is disposed');
            }
            return this._lines.length;
        };
        TextModel.prototype.getLineContent = function (lineNumber) {
            if (this._isDisposed) {
                throw new Error('TextModel.getLineContent: Model is disposed');
            }
            if (lineNumber < 1 || lineNumber > this.getLineCount()) {
                throw new Error('Illegal value ' + lineNumber + ' for `lineNumber`');
            }
            return this._lines[lineNumber - 1].text;
        };
        TextModel.prototype.getLinesContent = function () {
            if (this._isDisposed) {
                throw new Error('TextModel.getLineContent: Model is disposed');
            }
            var r = [];
            for (var i = 0, len = this._lines.length; i < len; i++) {
                r[i] = this._lines[i].text;
            }
            return r;
        };
        TextModel.prototype.getEOL = function () {
            if (this._isDisposed) {
                throw new Error('TextModel.getEOL: Model is disposed');
            }
            return this._EOL;
        };
        TextModel.prototype.setEOL = function (eol) {
            var newEOL = (eol === editorCommon.EndOfLineSequence.CRLF ? '\r\n' : '\n');
            if (this._EOL === newEOL) {
                // Nothing to do
                return;
            }
            var oldFullModelRange = this.getFullModelRange();
            var oldModelValueLength = this.getValueLengthInRange(oldFullModelRange);
            var endLineNumber = this.getLineCount();
            var endColumn = this.getLineMaxColumn(endLineNumber);
            this._EOL = newEOL;
            this._increaseVersionId();
            var e = this._createContentChangedFlushEvent();
            e.detail = this.toRawText();
            e.versionId = this._versionId;
            this._emitModelContentChangedFlushEvent(e);
            this._emitContentChanged2(1, 1, endLineNumber, endColumn, oldModelValueLength, this.getValue(), false, false);
        };
        TextModel.prototype.getLineMinColumn = function (lineNumber) {
            return 1;
        };
        TextModel.prototype.getLineMaxColumn = function (lineNumber) {
            if (this._isDisposed) {
                throw new Error('TextModel.getLineMaxColumn: Model is disposed');
            }
            if (lineNumber < 1 || lineNumber > this.getLineCount()) {
                throw new Error('Illegal value ' + lineNumber + ' for `lineNumber`');
            }
            return this._lines[lineNumber - 1].text.length + 1;
        };
        TextModel.prototype.getLineFirstNonWhitespaceColumn = function (lineNumber) {
            if (this._isDisposed) {
                throw new Error('TextModel.getLineFirstNonWhitespaceColumn: Model is disposed');
            }
            if (lineNumber < 1 || lineNumber > this.getLineCount()) {
                throw new Error('Illegal value ' + lineNumber + ' for `lineNumber`');
            }
            var result = strings.firstNonWhitespaceIndex(this._lines[lineNumber - 1].text);
            if (result === -1) {
                return 0;
            }
            return result + 1;
        };
        TextModel.prototype.getLineLastNonWhitespaceColumn = function (lineNumber) {
            if (this._isDisposed) {
                throw new Error('TextModel.getLineLastNonWhitespaceColumn: Model is disposed');
            }
            if (lineNumber < 1 || lineNumber > this.getLineCount()) {
                throw new Error('Illegal value ' + lineNumber + ' for `lineNumber`');
            }
            var result = strings.lastNonWhitespaceIndex(this._lines[lineNumber - 1].text);
            if (result === -1) {
                return 0;
            }
            return result + 2;
        };
        TextModel.prototype.validateLineNumber = function (lineNumber) {
            if (this._isDisposed) {
                throw new Error('TextModel.validateLineNumber: Model is disposed');
            }
            if (lineNumber < 1) {
                lineNumber = 1;
            }
            if (lineNumber > this._lines.length) {
                lineNumber = this._lines.length;
            }
            return lineNumber;
        };
        TextModel.prototype.validatePosition = function (position) {
            if (this._isDisposed) {
                throw new Error('TextModel.validatePosition: Model is disposed');
            }
            var lineNumber = position.lineNumber ? position.lineNumber : 1;
            var column = position.column ? position.column : 1;
            if (lineNumber < 1) {
                lineNumber = 1;
            }
            if (lineNumber > this._lines.length) {
                lineNumber = this._lines.length;
            }
            if (column < 1) {
                column = 1;
            }
            var maxColumn = this.getLineMaxColumn(lineNumber);
            if (column > maxColumn) {
                column = maxColumn;
            }
            return new position_1.Position(lineNumber, column);
        };
        TextModel.prototype.validateRange = function (range) {
            if (this._isDisposed) {
                throw new Error('TextModel.validateRange: Model is disposed');
            }
            var start = this.validatePosition(new position_1.Position(range.startLineNumber, range.startColumn));
            var end = this.validatePosition(new position_1.Position(range.endLineNumber, range.endColumn));
            return new range_1.Range(start.lineNumber, start.column, end.lineNumber, end.column);
        };
        TextModel.prototype.modifyPosition = function (rawPosition, offset) {
            if (this._isDisposed) {
                throw new Error('TextModel.modifyPosition: Model is disposed');
            }
            var position = this.validatePosition(rawPosition);
            // Handle positive offsets, one line at a time
            while (offset > 0) {
                var maxColumn = this.getLineMaxColumn(position.lineNumber);
                // Get to end of line
                if (position.column < maxColumn) {
                    var subtract = Math.min(offset, maxColumn - position.column);
                    offset -= subtract;
                    position.column += subtract;
                }
                if (offset === 0) {
                    break;
                }
                // Go to next line
                offset -= this._EOL.length;
                if (offset < 0) {
                    throw new Error('TextModel.modifyPosition: Breaking line terminators');
                }
                ++position.lineNumber;
                if (position.lineNumber > this._lines.length) {
                    throw new Error('TextModel.modifyPosition: Offset goes beyond the end of the model');
                }
                position.column = 1;
            }
            // Handle negative offsets, one line at a time
            while (offset < 0) {
                // Get to the start of the line
                if (position.column > 1) {
                    var add = Math.min(-offset, position.column - 1);
                    offset += add;
                    position.column -= add;
                }
                if (offset === 0) {
                    break;
                }
                // Go to the previous line
                offset += this._EOL.length;
                if (offset > 0) {
                    throw new Error('TextModel.modifyPosition: Breaking line terminators');
                }
                --position.lineNumber;
                if (position.lineNumber < 1) {
                    throw new Error('TextModel.modifyPosition: Offset goes beyond the beginning of the model');
                }
                position.column = this.getLineMaxColumn(position.lineNumber);
            }
            return position;
        };
        TextModel.prototype.getFullModelRange = function () {
            if (this._isDisposed) {
                throw new Error('TextModel.getFullModelRange: Model is disposed');
            }
            var lineCount = this.getLineCount();
            return new range_1.Range(1, 1, lineCount, this.getLineMaxColumn(lineCount));
        };
        TextModel.prototype._emitModelContentChangedFlushEvent = function (e) {
            if (!this._isDisposing) {
                this.emit(editorCommon.EventType.ModelContentChanged, e);
            }
        };
        TextModel.toRawText = function (rawText, opts) {
            // Count the number of lines that end with \r\n
            var carriageReturnCnt = 0, lastCarriageReturnIndex = -1;
            while ((lastCarriageReturnIndex = rawText.indexOf('\r', lastCarriageReturnIndex + 1)) !== -1) {
                carriageReturnCnt++;
            }
            // Split the text into liens
            var lines = rawText.split(/\r\n|\r|\n/);
            // Remove the BOM (if present)
            var BOM = '';
            if (strings.startsWithUTF8BOM(lines[0])) {
                BOM = strings.UTF8_BOM_CHARACTER;
                lines[0] = lines[0].substr(1);
            }
            var lineFeedCnt = lines.length - 1;
            var EOL = '';
            if (lineFeedCnt === 0) {
                // This is an empty file or a file with precisely one line
                EOL = (opts.defaultEOL === editorCommon.DefaultEndOfLine.LF ? '\n' : '\r\n');
            }
            else if (carriageReturnCnt > lineFeedCnt / 2) {
                // More than half of the file contains \r\n ending lines
                EOL = '\r\n';
            }
            else {
                // At least one line more ends in \n
                EOL = '\n';
            }
            var resolvedOpts;
            if (opts.detectIndentation) {
                var guessedIndentation = indentationGuesser_1.guessIndentation(lines, opts.tabSize, opts.insertSpaces);
                resolvedOpts = {
                    tabSize: guessedIndentation.tabSize,
                    insertSpaces: guessedIndentation.insertSpaces,
                    defaultEOL: opts.defaultEOL
                };
            }
            else {
                resolvedOpts = {
                    tabSize: opts.tabSize,
                    insertSpaces: opts.insertSpaces,
                    defaultEOL: opts.defaultEOL
                };
            }
            return {
                BOM: BOM,
                EOL: EOL,
                lines: lines,
                length: rawText.length,
                options: resolvedOpts
            };
        };
        TextModel.prototype._constructLines = function (rawText) {
            var rawLines = rawText.lines, modelLines = [], i, len;
            for (i = 0, len = rawLines.length; i < len; i++) {
                modelLines.push(new modelLine_1.ModelLine(i + 1, rawLines[i]));
            }
            this._BOM = rawText.BOM;
            this._EOL = rawText.EOL;
            this._lines = modelLines;
        };
        TextModel.prototype._getEndOfLine = function (eol) {
            switch (eol) {
                case editorCommon.EndOfLinePreference.LF:
                    return '\n';
                case editorCommon.EndOfLinePreference.CRLF:
                    return '\r\n';
                case editorCommon.EndOfLinePreference.TextDefined:
                    return this.getEOL();
            }
            throw new Error('Unknown EOL preference');
        };
        TextModel.prototype.findMatches = function (searchString, rawSearchScope, isRegex, matchCase, wholeWord, limitResultCount) {
            if (limitResultCount === void 0) { limitResultCount = LIMIT_FIND_COUNT; }
            if (this._isDisposed) {
                throw new Error('Model.findMatches: Model is disposed');
            }
            var regex = strings.createSafeRegExp(searchString, isRegex, matchCase, wholeWord);
            if (!regex) {
                return [];
            }
            var searchRange;
            if (range_1.Range.isIRange(rawSearchScope)) {
                searchRange = rawSearchScope;
            }
            else {
                searchRange = this.getFullModelRange();
            }
            return this._doFindMatches(searchRange, regex, limitResultCount);
        };
        TextModel.prototype.findNextMatch = function (searchString, rawSearchStart, isRegex, matchCase, wholeWord) {
            if (this._isDisposed) {
                throw new Error('Model.findNextMatch: Model is disposed');
            }
            var regex = strings.createSafeRegExp(searchString, isRegex, matchCase, wholeWord);
            if (!regex) {
                return null;
            }
            var searchStart = this.validatePosition(rawSearchStart), lineCount = this.getLineCount(), startLineNumber = searchStart.lineNumber, text, r;
            // Look in first line
            text = this._lines[startLineNumber - 1].text.substring(searchStart.column - 1);
            r = this._findMatchInLine(regex, text, startLineNumber, searchStart.column - 1);
            if (r) {
                return r;
            }
            for (var i = 1; i <= lineCount; i++) {
                var lineIndex = (startLineNumber + i - 1) % lineCount;
                text = this._lines[lineIndex].text;
                r = this._findMatchInLine(regex, text, lineIndex + 1, 0);
                if (r) {
                    return r;
                }
            }
            return null;
        };
        TextModel.prototype.findPreviousMatch = function (searchString, rawSearchStart, isRegex, matchCase, wholeWord) {
            if (this._isDisposed) {
                throw new Error('Model.findPreviousMatch: Model is disposed');
            }
            var regex = strings.createSafeRegExp(searchString, isRegex, matchCase, wholeWord);
            if (!regex) {
                return null;
            }
            var searchStart = this.validatePosition(rawSearchStart), lineCount = this.getLineCount(), startLineNumber = searchStart.lineNumber, text, r;
            // Look in first line
            text = this._lines[startLineNumber - 1].text.substring(0, searchStart.column - 1);
            r = this._findLastMatchInLine(regex, text, startLineNumber);
            if (r) {
                return r;
            }
            for (var i = 1; i <= lineCount; i++) {
                var lineIndex = (lineCount + startLineNumber - i - 1) % lineCount;
                text = this._lines[lineIndex].text;
                r = this._findLastMatchInLine(regex, text, lineIndex + 1);
                if (r) {
                    return r;
                }
            }
            return null;
        };
        TextModel.prototype._doFindMatches = function (searchRange, searchRegex, limitResultCount) {
            var result = [], text, counter = 0;
            // Early case for a search range that starts & stops on the same line number
            if (searchRange.startLineNumber === searchRange.endLineNumber) {
                text = this._lines[searchRange.startLineNumber - 1].text.substring(searchRange.startColumn - 1, searchRange.endColumn - 1);
                counter = this._findMatchesInLine(searchRegex, text, searchRange.startLineNumber, searchRange.startColumn - 1, counter, result, limitResultCount);
                return result;
            }
            // Collect results from first line
            text = this._lines[searchRange.startLineNumber - 1].text.substring(searchRange.startColumn - 1);
            counter = this._findMatchesInLine(searchRegex, text, searchRange.startLineNumber, searchRange.startColumn - 1, counter, result, limitResultCount);
            // Collect results from middle lines
            for (var lineNumber = searchRange.startLineNumber + 1; lineNumber < searchRange.endLineNumber && counter < limitResultCount; lineNumber++) {
                counter = this._findMatchesInLine(searchRegex, this._lines[lineNumber - 1].text, lineNumber, 0, counter, result, limitResultCount);
            }
            // Collect results from last line
            if (counter < limitResultCount) {
                text = this._lines[searchRange.endLineNumber - 1].text.substring(0, searchRange.endColumn - 1);
                counter = this._findMatchesInLine(searchRegex, text, searchRange.endLineNumber, 0, counter, result, limitResultCount);
            }
            return result;
        };
        TextModel.prototype._findMatchInLine = function (searchRegex, text, lineNumber, deltaOffset) {
            var m = searchRegex.exec(text);
            if (!m) {
                return null;
            }
            return new range_1.Range(lineNumber, m.index + 1 + deltaOffset, lineNumber, m.index + 1 + m[0].length + deltaOffset);
        };
        TextModel.prototype._findLastMatchInLine = function (searchRegex, text, lineNumber) {
            var bestResult = null;
            var m;
            while ((m = searchRegex.exec(text))) {
                var result = new range_1.Range(lineNumber, m.index + 1, lineNumber, m.index + 1 + m[0].length);
                if (result.equalsRange(bestResult)) {
                    break;
                }
                bestResult = result;
            }
            return bestResult;
        };
        TextModel.prototype._findMatchesInLine = function (searchRegex, text, lineNumber, deltaOffset, counter, result, limitResultCount) {
            var m;
            // Reset regex to search from the beginning
            searchRegex.lastIndex = 0;
            do {
                m = searchRegex.exec(text);
                if (m) {
                    var range = new range_1.Range(lineNumber, m.index + 1 + deltaOffset, lineNumber, m.index + 1 + m[0].length + deltaOffset);
                    // Exit early if the regex matches the same range
                    if (range.equalsRange(result[result.length - 1])) {
                        return counter;
                    }
                    result.push(range);
                    counter++;
                    if (counter >= limitResultCount) {
                        return counter;
                    }
                }
            } while (m);
            return counter;
        };
        TextModel.DEFAULT_CREATION_OPTIONS = {
            tabSize: defaultConfig_1.DEFAULT_INDENTATION.tabSize,
            insertSpaces: defaultConfig_1.DEFAULT_INDENTATION.insertSpaces,
            detectIndentation: false,
            defaultEOL: editorCommon.DefaultEndOfLine.LF
        };
        return TextModel;
    }(eventEmitter_1.OrderGuaranteeEventEmitter));
    exports.TextModel = TextModel;
    var RawText = (function () {
        function RawText() {
        }
        RawText.fromString = function (rawText, opts) {
            return TextModel.toRawText(rawText, opts);
        };
        RawText.fromStringWithModelOptions = function (rawText, model) {
            var opts = model.getOptions();
            return TextModel.toRawText(rawText, {
                tabSize: opts.tabSize,
                insertSpaces: opts.insertSpaces,
                detectIndentation: false,
                defaultEOL: opts.defaultEOL
            });
        };
        return RawText;
    }());
    exports.RawText = RawText;
});

define("vs/editor/common/modes/nullMode", ["require", "exports", 'vs/editor/common/config/defaultConfig'], function (require, exports, defaultConfig_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var NullState = (function () {
        function NullState(mode, stateData) {
            this.mode = mode;
            this.stateData = stateData;
        }
        NullState.prototype.clone = function () {
            var stateDataClone = (this.stateData ? this.stateData.clone() : null);
            return new NullState(this.mode, stateDataClone);
        };
        NullState.prototype.equals = function (other) {
            if (this.mode !== other.getMode()) {
                return false;
            }
            var otherStateData = other.getStateData();
            if (!this.stateData && !otherStateData) {
                return true;
            }
            if (this.stateData && otherStateData) {
                return this.stateData.equals(otherStateData);
            }
            return false;
        };
        NullState.prototype.getMode = function () {
            return this.mode;
        };
        NullState.prototype.tokenize = function (stream) {
            stream.advanceToEOS();
            return { type: '' };
        };
        NullState.prototype.getStateData = function () {
            return this.stateData;
        };
        NullState.prototype.setStateData = function (stateData) {
            this.stateData = stateData;
        };
        return NullState;
    }());
    exports.NullState = NullState;
    var NullMode = (function () {
        function NullMode() {
            this.richEditSupport = {
                wordDefinition: NullMode.DEFAULT_WORD_REGEXP
            };
        }
        /**
         * Create a word definition regular expression based on default word separators.
         * Optionally provide allowed separators that should be included in words.
         *
         * The default would look like this:
         * /(-?\d*\.\d\w*)|([^\`\~\!\@\#\$\%\^\&\*\(\)\-\=\+\[\{\]\}\\\|\;\:\'\"\,\.\<\>\/\?\s]+)/g
         */
        NullMode.createWordRegExp = function (allowInWords) {
            if (allowInWords === void 0) { allowInWords = ''; }
            var usualSeparators = defaultConfig_1.USUAL_WORD_SEPARATORS;
            var source = '(-?\\d*\\.\\d\\w*)|([^';
            for (var i = 0; i < usualSeparators.length; i++) {
                if (allowInWords.indexOf(usualSeparators[i]) >= 0) {
                    continue;
                }
                source += '\\' + usualSeparators[i];
            }
            source += '\\s]+)';
            return new RegExp(source, 'g');
        };
        NullMode.prototype.getId = function () {
            return NullMode.ID;
        };
        NullMode.prototype.toSimplifiedMode = function () {
            return this;
        };
        // catches numbers (including floating numbers) in the first group, and alphanum in the second
        NullMode.DEFAULT_WORD_REGEXP = NullMode.createWordRegExp();
        NullMode.ID = 'vs.editor.modes.nullMode';
        return NullMode;
    }());
    exports.NullMode = NullMode;
    function nullTokenize(mode, buffer, state, deltaOffset, stopAtOffset) {
        if (deltaOffset === void 0) { deltaOffset = 0; }
        var tokens = [
            {
                startIndex: deltaOffset,
                type: ''
            }
        ];
        var modeTransitions = [
            {
                startIndex: deltaOffset,
                mode: mode
            }
        ];
        return {
            tokens: tokens,
            actualStopOffset: deltaOffset + buffer.length,
            endState: state,
            modeTransitions: modeTransitions
        };
    }
    exports.nullTokenize = nullTokenize;
});

define("vs/editor/common/model/textModelWithTokensHelpers", ["require", "exports", 'vs/editor/common/core/arrays', 'vs/editor/common/modes/nullMode'], function (require, exports, arrays_1, nullMode_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var WordHelper = (function () {
        function WordHelper() {
        }
        WordHelper._safeGetWordDefinition = function (mode) {
            return (mode.richEditSupport ? mode.richEditSupport.wordDefinition : null);
        };
        WordHelper.ensureValidWordDefinition = function (wordDefinition) {
            var result = nullMode_1.NullMode.DEFAULT_WORD_REGEXP;
            if (wordDefinition && (wordDefinition instanceof RegExp)) {
                if (!wordDefinition.global) {
                    var flags = 'g';
                    if (wordDefinition.ignoreCase) {
                        flags += 'i';
                    }
                    if (wordDefinition.multiline) {
                        flags += 'm';
                    }
                    result = new RegExp(wordDefinition.source, flags);
                }
                else {
                    result = wordDefinition;
                }
            }
            result.lastIndex = 0;
            return result;
        };
        WordHelper.massageWordDefinitionOf = function (mode) {
            return WordHelper.ensureValidWordDefinition(WordHelper._safeGetWordDefinition(mode));
        };
        WordHelper.getWords = function (textSource, lineNumber) {
            if (!textSource._lineIsTokenized(lineNumber)) {
                return WordHelper._getWordsInText(textSource.getLineContent(lineNumber), WordHelper.massageWordDefinitionOf(textSource.getMode()));
            }
            var r = [], txt = textSource.getLineContent(lineNumber);
            if (txt.length > 0) {
                var modeTransitions = textSource._getLineModeTransitions(lineNumber), i, len, k, lenK, currentModeStartIndex, currentModeEndIndex, currentWordDefinition, currentModeText, words, startWord, endWord, word;
                // Go through all the modes
                for (i = 0, currentModeStartIndex = 0, len = modeTransitions.length; i < len; i++) {
                    currentWordDefinition = WordHelper.massageWordDefinitionOf(modeTransitions[i].mode);
                    currentModeStartIndex = modeTransitions[i].startIndex;
                    currentModeEndIndex = (i + 1 < len ? modeTransitions[i + 1].startIndex : txt.length);
                    currentModeText = txt.substring(currentModeStartIndex, currentModeEndIndex);
                    words = currentModeText.match(currentWordDefinition);
                    if (!words) {
                        continue;
                    }
                    endWord = 0;
                    for (k = 0, lenK = words.length; k < lenK; k++) {
                        word = words[k];
                        if (word.length > 0) {
                            startWord = currentModeText.indexOf(word, endWord);
                            endWord = startWord + word.length;
                            r.push({
                                start: currentModeStartIndex + startWord,
                                end: currentModeStartIndex + endWord
                            });
                        }
                    }
                }
            }
            return r;
        };
        WordHelper._getWordsInText = function (text, wordDefinition) {
            var words = text.match(wordDefinition) || [], k, startWord, endWord, startColumn, endColumn, word, r = [];
            for (k = 0; k < words.length; k++) {
                word = words[k].trim();
                if (word.length > 0) {
                    startWord = text.indexOf(word, endWord);
                    endWord = startWord + word.length;
                    startColumn = startWord;
                    endColumn = endWord;
                    r.push({
                        start: startColumn,
                        end: endColumn
                    });
                }
            }
            return r;
        };
        WordHelper._getWordAtColumn = function (txt, column, modeIndex, modeTransitions) {
            var modeStartIndex = modeTransitions[modeIndex].startIndex, modeEndIndex = (modeIndex + 1 < modeTransitions.length ? modeTransitions[modeIndex + 1].startIndex : txt.length), mode = modeTransitions[modeIndex].mode;
            return WordHelper._getWordAtText(column, WordHelper.massageWordDefinitionOf(mode), txt.substring(modeStartIndex, modeEndIndex), modeStartIndex);
        };
        WordHelper.getWordAtPosition = function (textSource, position) {
            if (!textSource._lineIsTokenized(position.lineNumber)) {
                return WordHelper._getWordAtText(position.column, WordHelper.massageWordDefinitionOf(textSource.getMode()), textSource.getLineContent(position.lineNumber), 0);
            }
            var result = null;
            var txt = textSource.getLineContent(position.lineNumber), modeTransitions = textSource._getLineModeTransitions(position.lineNumber), columnIndex = position.column - 1, modeIndex = arrays_1.Arrays.findIndexInSegmentsArray(modeTransitions, columnIndex);
            result = WordHelper._getWordAtColumn(txt, position.column, modeIndex, modeTransitions);
            if (!result && modeIndex > 0 && modeTransitions[modeIndex].startIndex === columnIndex) {
                // The position is right at the beginning of `modeIndex`, so try looking at `modeIndex` - 1 too
                result = WordHelper._getWordAtColumn(txt, position.column, modeIndex - 1, modeTransitions);
            }
            return result;
        };
        WordHelper._getWordAtText = function (column, wordDefinition, text, textOffset) {
            // console.log('_getWordAtText: ', column, text, textOffset);
            var words = text.match(wordDefinition), k, startWord, endWord, startColumn, endColumn, word;
            if (words) {
                for (k = 0; k < words.length; k++) {
                    word = words[k].trim();
                    if (word.length > 0) {
                        startWord = text.indexOf(word, endWord);
                        endWord = startWord + word.length;
                        startColumn = textOffset + startWord + 1;
                        endColumn = textOffset + endWord + 1;
                        if (startColumn <= column && column <= endColumn) {
                            return {
                                word: word,
                                startColumn: startColumn,
                                endColumn: endColumn
                            };
                        }
                    }
                }
            }
            return null;
        };
        return WordHelper;
    }());
    exports.WordHelper = WordHelper;
});

define("vs/editor/common/modes/supports/richEditSupport", ["require", "exports", 'vs/editor/common/modes/nullMode', 'vs/editor/common/modes/supports/characterPair', 'vs/editor/common/modes/supports/electricCharacter', 'vs/editor/common/modes/supports/onEnter', 'vs/editor/common/modes/supports/richEditBrackets'], function (require, exports, nullMode_1, characterPair_1, electricCharacter_1, onEnter_1, richEditBrackets_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var RichEditSupport = (function () {
        function RichEditSupport(modeId, previous, rawConf) {
            var prev = null;
            if (previous instanceof RichEditSupport) {
                prev = previous._conf;
            }
            this._conf = RichEditSupport._mergeConf(prev, rawConf);
            if (this._conf.brackets) {
                this.brackets = new richEditBrackets_1.RichEditBrackets(modeId, this._conf.brackets);
            }
            this._handleOnEnter(modeId, this._conf);
            this._handleComments(modeId, this._conf);
            if (this._conf.__characterPairSupport) {
                this.characterPair = new characterPair_1.CharacterPairSupport(modeId, this._conf.__characterPairSupport);
            }
            if (this._conf.__electricCharacterSupport || this._conf.brackets) {
                this.electricCharacter = new electricCharacter_1.BracketElectricCharacterSupport(modeId, this.brackets, this._conf.__electricCharacterSupport);
            }
            this.wordDefinition = this._conf.wordPattern || nullMode_1.NullMode.DEFAULT_WORD_REGEXP;
        }
        RichEditSupport._mergeConf = function (prev, current) {
            return {
                comments: (prev ? current.comments || prev.comments : current.comments),
                brackets: (prev ? current.brackets || prev.brackets : current.brackets),
                wordPattern: (prev ? current.wordPattern || prev.wordPattern : current.wordPattern),
                indentationRules: (prev ? current.indentationRules || prev.indentationRules : current.indentationRules),
                onEnterRules: (prev ? current.onEnterRules || prev.onEnterRules : current.onEnterRules),
                __electricCharacterSupport: (prev ? current.__electricCharacterSupport || prev.__electricCharacterSupport : current.__electricCharacterSupport),
                __characterPairSupport: (prev ? current.__characterPairSupport || prev.__characterPairSupport : current.__characterPairSupport),
            };
        };
        RichEditSupport.prototype._handleOnEnter = function (modeId, conf) {
            // on enter
            var onEnter = {};
            var empty = true;
            if (conf.brackets) {
                empty = false;
                onEnter.brackets = conf.brackets;
            }
            if (conf.indentationRules) {
                empty = false;
                onEnter.indentationRules = conf.indentationRules;
            }
            if (conf.onEnterRules) {
                empty = false;
                onEnter.regExpRules = conf.onEnterRules;
            }
            if (!empty) {
                this.onEnter = new onEnter_1.OnEnterSupport(modeId, onEnter);
            }
        };
        RichEditSupport.prototype._handleComments = function (modeId, conf) {
            var commentRule = conf.comments;
            // comment configuration
            if (commentRule) {
                this.comments = {};
                if (commentRule.lineComment) {
                    this.comments.lineCommentToken = commentRule.lineComment;
                }
                if (commentRule.blockComment) {
                    var _a = commentRule.blockComment, blockStart = _a[0], blockEnd = _a[1];
                    this.comments.blockCommentStartToken = blockStart;
                    this.comments.blockCommentEndToken = blockEnd;
                }
            }
        };
        return RichEditSupport;
    }());
    exports.RichEditSupport = RichEditSupport;
});

define("vs/editor/common/modes/supports/tokenizationSupport", ["require", "exports", 'vs/editor/common/modes/lineStream', 'vs/editor/common/modes/nullMode', 'vs/editor/common/modes/supports'], function (require, exports, lineStream_1, nullMode_1, supports_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function isFunction(something) {
        return typeof something === 'function';
    }
    var TokenizationSupport = (function () {
        function TokenizationSupport(mode, customization, supportsNestedModes, shouldGenerateEmbeddedModels) {
            this._mode = mode;
            this.customization = customization;
            this.supportsNestedModes = supportsNestedModes;
            this._embeddedModesListeners = {};
            if (this.supportsNestedModes) {
                if (!this._mode.registerSupport) {
                    throw new Error('Cannot be a mode with nested modes unless I can emit a tokenizationSupport changed event!');
                }
            }
            this.shouldGenerateEmbeddedModels = shouldGenerateEmbeddedModels;
            this.defaults = {
                enterNestedMode: !isFunction(customization.enterNestedMode),
                getNestedMode: !isFunction(customization.getNestedMode),
                getNestedModeInitialState: !isFunction(customization.getNestedModeInitialState),
                getLeavingNestedModeData: !isFunction(customization.getLeavingNestedModeData),
                onReturningFromNestedMode: !isFunction(customization.onReturningFromNestedMode)
            };
        }
        TokenizationSupport.prototype.dispose = function () {
            for (var listener in this._embeddedModesListeners) {
                this._embeddedModesListeners[listener].dispose();
                delete this._embeddedModesListeners[listener];
            }
        };
        TokenizationSupport.prototype.getInitialState = function () {
            return this.customization.getInitialState();
        };
        TokenizationSupport.prototype.tokenize = function (line, state, deltaOffset, stopAtOffset) {
            if (deltaOffset === void 0) { deltaOffset = 0; }
            if (stopAtOffset === void 0) { stopAtOffset = deltaOffset + line.length; }
            if (state.getMode() !== this._mode) {
                return this._nestedTokenize(line, state, deltaOffset, stopAtOffset, [], []);
            }
            else {
                return this._myTokenize(line, state, deltaOffset, stopAtOffset, [], []);
            }
        };
        /**
         * Precondition is: nestedModeState.getMode() !== this
         * This means we are in a nested mode when parsing starts on this line.
         */
        TokenizationSupport.prototype._nestedTokenize = function (buffer, nestedModeState, deltaOffset, stopAtOffset, prependTokens, prependModeTransitions) {
            var myStateBeforeNestedMode = nestedModeState.getStateData();
            var leavingNestedModeData = this.getLeavingNestedModeData(buffer, myStateBeforeNestedMode);
            // Be sure to give every embedded mode the
            // opportunity to leave nested mode.
            // i.e. Don't go straight to the most nested mode
            var stepOnceNestedState = nestedModeState;
            while (stepOnceNestedState.getStateData() && stepOnceNestedState.getStateData().getMode() !== this._mode) {
                stepOnceNestedState = stepOnceNestedState.getStateData();
            }
            var nestedMode = stepOnceNestedState.getMode();
            if (!leavingNestedModeData) {
                // tokenization will not leave nested mode
                var result;
                if (nestedMode.tokenizationSupport) {
                    result = nestedMode.tokenizationSupport.tokenize(buffer, nestedModeState, deltaOffset, stopAtOffset);
                }
                else {
                    // The nested mode doesn't have tokenization support,
                    // unfortunatelly this means we have to fake it
                    result = nullMode_1.nullTokenize(nestedMode, buffer, nestedModeState, deltaOffset);
                }
                result.tokens = prependTokens.concat(result.tokens);
                result.modeTransitions = prependModeTransitions.concat(result.modeTransitions);
                return result;
            }
            var nestedModeBuffer = leavingNestedModeData.nestedModeBuffer;
            if (nestedModeBuffer.length > 0) {
                // Tokenize with the nested mode
                var nestedModeLineTokens;
                if (nestedMode.tokenizationSupport) {
                    nestedModeLineTokens = nestedMode.tokenizationSupport.tokenize(nestedModeBuffer, nestedModeState, deltaOffset, stopAtOffset);
                }
                else {
                    // The nested mode doesn't have tokenization support,
                    // unfortunatelly this means we have to fake it
                    nestedModeLineTokens = nullMode_1.nullTokenize(nestedMode, nestedModeBuffer, nestedModeState, deltaOffset);
                }
                // Save last state of nested mode
                nestedModeState = nestedModeLineTokens.endState;
                // Prepend nested mode's result to our result
                prependTokens = prependTokens.concat(nestedModeLineTokens.tokens);
                prependModeTransitions = prependModeTransitions.concat(nestedModeLineTokens.modeTransitions);
            }
            var bufferAfterNestedMode = leavingNestedModeData.bufferAfterNestedMode;
            var myStateAfterNestedMode = leavingNestedModeData.stateAfterNestedMode;
            myStateAfterNestedMode.setStateData(myStateBeforeNestedMode.getStateData());
            this.onReturningFromNestedMode(myStateAfterNestedMode, nestedModeState);
            return this._myTokenize(bufferAfterNestedMode, myStateAfterNestedMode, deltaOffset + nestedModeBuffer.length, stopAtOffset, prependTokens, prependModeTransitions);
        };
        /**
         * Precondition is: state.getMode() === this
         * This means we are in the current mode when parsing starts on this line.
         */
        TokenizationSupport.prototype._myTokenize = function (buffer, myState, deltaOffset, stopAtOffset, prependTokens, prependModeTransitions) {
            var _this = this;
            var lineStream = new lineStream_1.LineStream(buffer);
            var tokenResult, beforeTokenizeStreamPos;
            var previousType = null;
            var retokenize = null;
            myState = myState.clone();
            if (prependModeTransitions.length <= 0 || prependModeTransitions[prependModeTransitions.length - 1].mode !== this._mode) {
                // Avoid transitioning to the same mode (this can happen in case of empty embedded modes)
                prependModeTransitions.push({
                    startIndex: deltaOffset,
                    mode: this._mode
                });
            }
            var maxPos = Math.min(stopAtOffset - deltaOffset, buffer.length);
            while (lineStream.pos() < maxPos) {
                beforeTokenizeStreamPos = lineStream.pos();
                do {
                    tokenResult = myState.tokenize(lineStream);
                    if (tokenResult === null || tokenResult === undefined ||
                        ((tokenResult.type === undefined || tokenResult.type === null) &&
                            (tokenResult.nextState === undefined || tokenResult.nextState === null))) {
                        throw new Error('Tokenizer must return a valid state');
                    }
                    if (tokenResult.nextState) {
                        tokenResult.nextState.setStateData(myState.getStateData());
                        myState = tokenResult.nextState;
                    }
                    if (lineStream.pos() <= beforeTokenizeStreamPos) {
                        throw new Error('Stream did not advance while tokenizing. Mode id is ' + this._mode.getId() + ' (stuck at token type: "' + tokenResult.type + '", prepend tokens: "' + (prependTokens.map(function (t) { return t.type; }).join(',')) + '").');
                    }
                } while (!tokenResult.type && tokenResult.type !== '');
                if (previousType !== tokenResult.type || tokenResult.dontMergeWithPrev || previousType === null) {
                    prependTokens.push(new supports_1.Token(beforeTokenizeStreamPos + deltaOffset, tokenResult.type));
                }
                previousType = tokenResult.type;
                if (this.supportsNestedModes && this.enterNestedMode(myState)) {
                    var currentEmbeddedLevels = this._getEmbeddedLevel(myState);
                    if (currentEmbeddedLevels < TokenizationSupport.MAX_EMBEDDED_LEVELS) {
                        var nestedModeState = this.getNestedModeInitialState(myState);
                        // Re-emit tokenizationSupport change events from all modes that I ever embedded
                        var embeddedMode = nestedModeState.state.getMode();
                        if (typeof embeddedMode.addSupportChangedListener === 'function' && !this._embeddedModesListeners.hasOwnProperty(embeddedMode.getId())) {
                            var emitting = false;
                            this._embeddedModesListeners[embeddedMode.getId()] = embeddedMode.addSupportChangedListener(function (e) {
                                if (emitting) {
                                    return;
                                }
                                if (e.tokenizationSupport) {
                                    emitting = true;
                                    _this._mode.registerSupport('tokenizationSupport', function (mode) {
                                        return mode.tokenizationSupport;
                                    });
                                    emitting = false;
                                }
                            });
                        }
                        if (!lineStream.eos()) {
                            // There is content from the embedded mode
                            var restOfBuffer = buffer.substr(lineStream.pos());
                            var result = this._nestedTokenize(restOfBuffer, nestedModeState.state, deltaOffset + lineStream.pos(), stopAtOffset, prependTokens, prependModeTransitions);
                            result.retokenize = result.retokenize || nestedModeState.missingModePromise;
                            return result;
                        }
                        else {
                            // Transition to the nested mode state
                            myState = nestedModeState.state;
                            retokenize = nestedModeState.missingModePromise;
                        }
                    }
                }
            }
            return {
                tokens: prependTokens,
                actualStopOffset: lineStream.pos() + deltaOffset,
                modeTransitions: prependModeTransitions,
                endState: myState,
                retokenize: retokenize
            };
        };
        TokenizationSupport.prototype._getEmbeddedLevel = function (state) {
            var result = -1;
            while (state) {
                result++;
                state = state.getStateData();
            }
            return result;
        };
        TokenizationSupport.prototype.enterNestedMode = function (state) {
            if (this.defaults.enterNestedMode) {
                return false;
            }
            return this.customization.enterNestedMode(state);
        };
        TokenizationSupport.prototype.getNestedMode = function (state) {
            if (this.defaults.getNestedMode) {
                return null;
            }
            return this.customization.getNestedMode(state);
        };
        TokenizationSupport._validatedNestedMode = function (input) {
            var mode = new nullMode_1.NullMode(), missingModePromise = null;
            if (input && input.mode) {
                mode = input.mode;
            }
            if (input && input.missingModePromise) {
                missingModePromise = input.missingModePromise;
            }
            return {
                mode: mode,
                missingModePromise: missingModePromise
            };
        };
        TokenizationSupport.prototype.getNestedModeInitialState = function (state) {
            if (this.defaults.getNestedModeInitialState) {
                var nestedMode = TokenizationSupport._validatedNestedMode(this.getNestedMode(state));
                var missingModePromise = nestedMode.missingModePromise;
                var nestedModeState;
                if (nestedMode.mode.tokenizationSupport) {
                    nestedModeState = nestedMode.mode.tokenizationSupport.getInitialState();
                }
                else {
                    nestedModeState = new nullMode_1.NullState(nestedMode.mode, null);
                }
                nestedModeState.setStateData(state);
                return {
                    state: nestedModeState,
                    missingModePromise: missingModePromise
                };
            }
            return this.customization.getNestedModeInitialState(state);
        };
        TokenizationSupport.prototype.getLeavingNestedModeData = function (line, state) {
            if (this.defaults.getLeavingNestedModeData) {
                return null;
            }
            return this.customization.getLeavingNestedModeData(line, state);
        };
        TokenizationSupport.prototype.onReturningFromNestedMode = function (myStateAfterNestedMode, lastNestedModeState) {
            if (this.defaults.onReturningFromNestedMode) {
                return null;
            }
            return this.customization.onReturningFromNestedMode(myStateAfterNestedMode, lastNestedModeState);
        };
        TokenizationSupport.MAX_EMBEDDED_LEVELS = 5;
        return TokenizationSupport;
    }());
    exports.TokenizationSupport = TokenizationSupport;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/modes/monarch/monarchLexer", ["require", "exports", 'vs/editor/common/modes/abstractState', 'vs/editor/common/modes/lineStream', 'vs/editor/common/modes/monarch/monarchCommon', 'vs/editor/common/modes/supports/tokenizationSupport'], function (require, exports, abstractState_1, lineStream_1, monarchCommon, tokenizationSupport_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The MonarchLexer class implements a monaco lexer that highlights source code.
     * It takes a compiled lexer to guide the tokenizer and maintains a stack of
     * lexer states.
     */
    var MonarchLexer = (function (_super) {
        __extends(MonarchLexer, _super);
        function MonarchLexer(mode, modeService, lexer, stack, embeddedMode) {
            _super.call(this, mode);
            this.id = MonarchLexer.ID++; // for debugging, assigns unique id to each instance
            this.modeService = modeService;
            this.lexer = lexer; // (compiled) lexer description
            this.stack = (stack ? stack : [lexer.start]); // stack of states
            this.embeddedMode = (embeddedMode ? embeddedMode : null); // are we scanning an embedded section?
            // did we encounter an embedded start on this line?
            // no need for cloning or equality since it is used only within a line
            this.embeddedEntered = false;
            // regular expression group matching
            // these never need cloning or equality since they are only used within a line match
            this.groupActions = null;
            this.groupMatches = null;
            this.groupMatched = null;
            this.groupRule = null;
        }
        MonarchLexer.prototype.makeClone = function () {
            return new MonarchLexer(this.getMode(), this.modeService, this.lexer, this.stack.slice(0), this.embeddedMode);
        };
        MonarchLexer.prototype.equals = function (other) {
            if (!_super.prototype.equals.call(this, other)) {
                return false;
            }
            if (!(other instanceof MonarchLexer)) {
                return false;
            }
            var otherm = other;
            if ((this.stack.length !== otherm.stack.length) || (this.lexer.name !== otherm.lexer.name) ||
                (this.embeddedMode !== otherm.embeddedMode)) {
                return false;
            }
            var idx;
            for (idx in this.stack) {
                if (this.stack.hasOwnProperty(idx)) {
                    if (this.stack[idx] !== otherm.stack[idx]) {
                        return false;
                    }
                }
            }
            return true;
        };
        /**
         * The main tokenizer: this function gets called by monaco to tokenize lines
         * Note: we don't want to raise exceptions here and always keep going..
         *
         * TODO: there are many optimizations possible here for the common cases
         * but for now I concentrated on functionality and correctness.
         */
        MonarchLexer.prototype.tokenize = function (stream, noConsumeIsOk) {
            var stackLen0 = this.stack.length; // these are saved to check progress
            var groupLen0 = 0;
            var state = this.stack[0]; // the current state
            this.embeddedEntered = false;
            var matches = null;
            var matched = null;
            var action = null;
            var next = null;
            var rule = null;
            // check if we need to process group matches first
            if (this.groupActions) {
                groupLen0 = this.groupActions.length;
                matches = this.groupMatches;
                matched = this.groupMatched.shift();
                action = this.groupActions.shift();
                rule = this.groupRule;
                // cleanup if necessary
                if (this.groupActions.length === 0) {
                    this.groupActions = null;
                    this.groupMatches = null;
                    this.groupMatched = null;
                    this.groupRule = null;
                }
            }
            else {
                // nothing to do
                if (stream.eos()) {
                    return { type: '' };
                }
                // get the entire line
                var line = stream.advanceToEOS();
                stream.goBack(line.length);
                // get the rules for this state
                var rules = this.lexer.tokenizer[state];
                if (!rules) {
                    rules = monarchCommon.findRules(this.lexer, state); // do parent matching
                }
                if (!rules) {
                    monarchCommon.throwError(this.lexer, 'tokenizer state is not defined: ' + state);
                }
                else {
                    // try each rule until we match
                    rule = null;
                    var pos = stream.pos();
                    var idx;
                    for (idx in rules) {
                        if (rules.hasOwnProperty(idx)) {
                            rule = rules[idx];
                            if (pos === 0 || !rule.matchOnlyAtLineStart) {
                                matches = line.match(rule.regex);
                                if (matches) {
                                    matched = matches[0];
                                    action = rule.action;
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            // We matched 'rule' with 'matches' and 'action'
            if (!matches) {
                matches = [''];
                matched = '';
            }
            if (!action) {
                // bad: we didn't match anything, and there is no action to take
                // we need to advance the stream or we get progress trouble
                if (!stream.eos()) {
                    matches = [stream.peek()];
                    matched = matches[0];
                }
                action = this.lexer.defaultToken;
            }
            // advance stream
            stream.advance(matched.length);
            // maybe call action function (used for 'cases')
            while (action.test) {
                var callres = action.test(matched, matches, state, stream.eos());
                action = callres;
            }
            // set the result: either a string or an array of actions
            var result = null;
            if (typeof (action) === 'string' || Array.isArray(action)) {
                result = action;
            }
            else if (action.group) {
                result = action.group;
            }
            else if (action.token !== null && action.token !== undefined) {
                result = action.token;
                // do $n replacements?
                if (action.tokenSubst) {
                    result = monarchCommon.substituteMatches(this.lexer, result, matched, matches, state);
                }
                // enter embedded mode?
                if (action.nextEmbedded) {
                    if (action.nextEmbedded === '@pop') {
                        if (!this.embeddedMode) {
                            monarchCommon.throwError(this.lexer, 'cannot pop embedded mode if not inside one');
                        }
                        this.embeddedMode = null;
                    }
                    else if (this.embeddedMode) {
                        monarchCommon.throwError(this.lexer, 'cannot enter embedded mode from within an embedded mode');
                    }
                    else {
                        this.embeddedMode = monarchCommon.substituteMatches(this.lexer, action.nextEmbedded, matched, matches, state);
                        // substitute language alias to known modes to support syntax highlighting
                        var embeddedMode = this.modeService.getModeIdForLanguageName(this.embeddedMode);
                        if (this.embeddedMode && embeddedMode) {
                            this.embeddedMode = embeddedMode;
                        }
                        this.embeddedEntered = true;
                    }
                }
                // state transformations
                if (action.goBack) {
                    stream.goBack(action.goBack);
                }
                if (action.switchTo && typeof action.switchTo === 'string') {
                    var nextState = monarchCommon.substituteMatches(this.lexer, action.switchTo, matched, matches, state); // switch state without a push...
                    if (nextState[0] === '@') {
                        nextState = nextState.substr(1); // peel off starting '@'
                    }
                    if (!monarchCommon.findRules(this.lexer, nextState)) {
                        monarchCommon.throwError(this.lexer, 'trying to switch to a state \'' + nextState + '\' that is undefined in rule: ' + rule.name);
                    }
                    else {
                        this.stack[0] = nextState;
                    }
                    next = null;
                }
                else if (action.transform && typeof action.transform === 'function') {
                    this.stack = action.transform(this.stack); // if you need to do really funky stuff...
                    next = null;
                }
                else if (action.next) {
                    if (action.next === '@push') {
                        if (this.stack.length >= this.lexer.maxStack) {
                            monarchCommon.throwError(this.lexer, 'maximum tokenizer stack size reached: [' +
                                this.stack[0] + ',' + this.stack[1] + ',...,' +
                                this.stack[this.stack.length - 2] + ',' + this.stack[this.stack.length - 1] + ']');
                        }
                        else {
                            this.stack.unshift(state);
                        }
                    }
                    else if (action.next === '@pop') {
                        if (this.stack.length <= 1) {
                            monarchCommon.throwError(this.lexer, 'trying to pop an empty stack in rule: ' + rule.name);
                        }
                        else {
                            this.stack.shift();
                        }
                    }
                    else if (action.next === '@popall') {
                        if (this.stack.length > 1) {
                            this.stack = [this.stack[this.stack.length - 1]];
                        }
                    }
                    else {
                        var nextState = monarchCommon.substituteMatches(this.lexer, action.next, matched, matches, state);
                        if (nextState[0] === '@') {
                            nextState = nextState.substr(1); // peel off starting '@'
                        }
                        if (!monarchCommon.findRules(this.lexer, nextState)) {
                            monarchCommon.throwError(this.lexer, 'trying to set a next state \'' + nextState + '\' that is undefined in rule: ' + rule.name);
                        }
                        else {
                            this.stack.unshift(nextState);
                        }
                    }
                }
                if (action.log && typeof (action.log) === 'string') {
                    monarchCommon.log(this.lexer, this.lexer.displayName + ': ' + monarchCommon.substituteMatches(this.lexer, action.log, matched, matches, state));
                }
            }
            // check result
            if (result === null) {
                monarchCommon.throwError(this.lexer, 'lexer rule has no well-defined action in rule: ' + rule.name);
                result = this.lexer.defaultToken;
            }
            // is the result a group match?
            if (Array.isArray(result)) {
                if (this.groupActions && this.groupActions.length > 0) {
                    monarchCommon.throwError(this.lexer, 'groups cannot be nested: ' + rule.name);
                }
                if (matches.length !== result.length + 1) {
                    monarchCommon.throwError(this.lexer, 'matched number of groups does not match the number of actions in rule: ' + rule.name);
                }
                var totalLen = 0;
                for (var i = 1; i < matches.length; i++) {
                    totalLen += matches[i].length;
                }
                if (totalLen !== matched.length) {
                    monarchCommon.throwError(this.lexer, 'with groups, all characters should be matched in consecutive groups in rule: ' + rule.name);
                }
                this.groupMatches = matches;
                this.groupMatched = matches.slice(1);
                this.groupActions = result.slice(0);
                this.groupRule = rule;
                stream.goBack(matched.length);
                return this.tokenize(stream); // call recursively to initiate first result match
            }
            else {
                // check for '@rematch'
                if (result === '@rematch') {
                    stream.goBack(matched.length);
                    matched = ''; // better set the next state too..
                    matches = null;
                    result = '';
                }
                // check progress
                if (matched.length === 0) {
                    if (stackLen0 !== this.stack.length || state !== this.stack[0]
                        || (!this.groupActions ? 0 : this.groupActions.length) !== groupLen0) {
                        if (!noConsumeIsOk) {
                            return this.tokenize(stream); // tokenize again in the new state
                        }
                    }
                    else {
                        monarchCommon.throwError(this.lexer, 'no progress in tokenizer in rule: ' + rule.name);
                        stream.advanceToEOS(); // must make progress or editor loops
                    }
                }
                // return the result (and check for brace matching)
                // todo: for efficiency we could pre-sanitize tokenPostfix and substitutions
                if (result.indexOf('@brackets') === 0) {
                    var rest = result.substr('@brackets'.length);
                    var bracket = findBracket(this.lexer, matched);
                    if (!bracket) {
                        monarchCommon.throwError(this.lexer, '@brackets token returned but no bracket defined as: ' + matched);
                        bracket = { token: '', bracketType: monarchCommon.MonarchBracket.None };
                    }
                    return { type: monarchCommon.sanitize(bracket.token + rest) };
                }
                else {
                    var token = (result === '' ? '' : result + this.lexer.tokenPostfix);
                    return { type: monarchCommon.sanitize(token) };
                }
            }
        };
        MonarchLexer.ID = 0;
        return MonarchLexer;
    }(abstractState_1.AbstractState));
    exports.MonarchLexer = MonarchLexer;
    /**
     * Searches for a bracket in the 'brackets' attribute that matches the input.
     */
    function findBracket(lexer, matched) {
        if (!matched) {
            return null;
        }
        matched = monarchCommon.fixCase(lexer, matched);
        var brackets = lexer.brackets;
        for (var i = 0; i < brackets.length; i++) {
            var bracket = brackets[i];
            if (bracket.open === matched) {
                return { token: bracket.token, bracketType: monarchCommon.MonarchBracket.Open };
            }
            else if (bracket.close === matched) {
                return { token: bracket.token, bracketType: monarchCommon.MonarchBracket.Close };
            }
        }
        return null;
    }
    function createTokenizationSupport(modeService, mode, lexer) {
        return new tokenizationSupport_1.TokenizationSupport(mode, {
            getInitialState: function () {
                return new MonarchLexer(mode, modeService, lexer);
            },
            enterNestedMode: function (state) {
                if (state instanceof MonarchLexer) {
                    return state.embeddedEntered;
                }
                return false;
            },
            getNestedMode: function (rawState) {
                var mime = rawState.embeddedMode;
                if (!modeService.isRegisteredMode(mime)) {
                    // unknown mode
                    return {
                        mode: modeService.getMode('text/plain'),
                        missingModePromise: null
                    };
                }
                var mode = modeService.getMode(mime);
                if (mode) {
                    // mode is available
                    return {
                        mode: mode,
                        missingModePromise: null
                    };
                }
                // mode is not yet loaded
                return {
                    mode: modeService.getMode('text/plain'),
                    missingModePromise: modeService.getOrCreateMode(mime).then(function () { return null; })
                };
            },
            getLeavingNestedModeData: function (line, state) {
                // state = state.clone();
                var mstate = state.clone();
                var stream = new lineStream_1.LineStream(line);
                while (!stream.eos() && mstate.embeddedMode) {
                    mstate.tokenize(stream, true); // allow no consumption for @rematch
                }
                if (mstate.embeddedMode) {
                    return null; // don't leave yet
                }
                var end = stream.pos();
                return {
                    nestedModeBuffer: line.substring(0, end),
                    bufferAfterNestedMode: line.substring(end),
                    stateAfterNestedMode: mstate
                };
            }
        }, lexer.usesEmbedded, false);
    }
    exports.createTokenizationSupport = createTokenizationSupport;
});

define("vs/editor/common/modes/textToHtmlTokenizer", ["require", "exports", 'vs/base/common/strings', 'vs/editor/common/modes/nullMode'], function (require, exports, strings, nullMode_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function tokenizeToHtmlContent(text, mode) {
        return _tokenizeToHtmlContent(text, _getSafeTokenizationSupport(mode));
    }
    exports.tokenizeToHtmlContent = tokenizeToHtmlContent;
    function tokenizeToString(text, mode, extraTokenClass) {
        return _tokenizeToString(text, _getSafeTokenizationSupport(mode), extraTokenClass);
    }
    exports.tokenizeToString = tokenizeToString;
    function _getSafeTokenizationSupport(mode) {
        if (mode && mode.tokenizationSupport) {
            return mode.tokenizationSupport;
        }
        return {
            shouldGenerateEmbeddedModels: false,
            getInitialState: function () { return new nullMode_1.NullState(null, null); },
            tokenize: function (buffer, state, deltaOffset, stopAtOffset) {
                if (deltaOffset === void 0) { deltaOffset = 0; }
                return nullMode_1.nullTokenize(null, buffer, state, deltaOffset, stopAtOffset);
            }
        };
    }
    function _tokenizeToHtmlContent(text, tokenizationSupport) {
        var result = {
            tagName: 'div',
            style: 'white-space: pre-wrap',
            children: []
        };
        var emitToken = function (className, tokenText) {
            result.children.push({
                tagName: 'span',
                className: className,
                text: tokenText
            });
        };
        var emitNewLine = function () {
            result.children.push({
                tagName: 'br'
            });
        };
        _tokenizeLines(text, tokenizationSupport, emitToken, emitNewLine);
        return result;
    }
    function _tokenizeToString(text, tokenizationSupport, extraTokenClass) {
        if (extraTokenClass === void 0) { extraTokenClass = ''; }
        if (extraTokenClass && extraTokenClass.length > 0) {
            extraTokenClass = ' ' + extraTokenClass;
        }
        var result = '';
        var emitToken = function (className, tokenText) {
            result += '<span class="' + className + extraTokenClass + '">' + strings.escape(tokenText) + '</span>';
        };
        var emitNewLine = function () {
            result += '<br/>';
        };
        result = '<div style="white-space: pre-wrap;">';
        _tokenizeLines(text, tokenizationSupport, emitToken, emitNewLine);
        result += '</div>';
        return result;
    }
    function _tokenizeLines(text, tokenizationSupport, emitToken, emitNewLine) {
        var lines = text.split(/\r\n|\r|\n/);
        var currentState = tokenizationSupport.getInitialState();
        for (var i = 0; i < lines.length; i++) {
            currentState = _tokenizeLine(lines[i], tokenizationSupport, emitToken, currentState);
            // Keep new lines
            if (i < lines.length - 1) {
                emitNewLine();
            }
        }
    }
    function _tokenizeLine(line, tokenizationSupport, emitToken, startState) {
        var tokenized = tokenizationSupport.tokenize(line, startState), endState = tokenized.endState, tokens = tokenized.tokens, offset = 0, tokenText;
        // For each token inject spans with proper class names based on token type
        for (var j = 0; j < tokens.length; j++) {
            var token = tokens[j];
            // Tokens only provide a startIndex from where they are valid from. As such, we need to
            // look ahead the value of the token by advancing until the next tokens start inex or the
            // end of the line.
            if (j < tokens.length - 1) {
                tokenText = line.substring(offset, tokens[j + 1].startIndex);
                offset = tokens[j + 1].startIndex;
            }
            else {
                tokenText = line.substr(offset);
            }
            var className = 'token';
            var safeType = token.type.replace(/[^a-z0-9\-]/gi, ' ');
            if (safeType.length > 0) {
                className += ' ' + safeType;
            }
            emitToken(className, tokenText);
        }
        return endState;
    }
});

define("vs/nls!vs/editor/common/controller/cursor",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/common/controller/cursor", data); });
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/controller/cursor", ["require", "exports", 'vs/nls!vs/editor/common/controller/cursor', 'vs/base/common/errors', 'vs/base/common/eventEmitter', 'vs/base/common/lifecycle', 'vs/editor/common/commands/replaceCommand', 'vs/editor/common/controller/cursorCollection', 'vs/editor/common/controller/handlerDispatcher', 'vs/editor/common/controller/oneCursor', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/core/selection', 'vs/editor/common/editorCommon'], function (require, exports, nls, errors_1, eventEmitter_1, lifecycle_1, replaceCommand_1, cursorCollection_1, handlerDispatcher_1, oneCursor_1, position_1, range_1, selection_1, editorCommon) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var RevealTarget;
    (function (RevealTarget) {
        RevealTarget[RevealTarget["Primary"] = 0] = "Primary";
        RevealTarget[RevealTarget["TopMost"] = 1] = "TopMost";
        RevealTarget[RevealTarget["BottomMost"] = 2] = "BottomMost";
    })(RevealTarget || (RevealTarget = {}));
    var Cursor = (function (_super) {
        __extends(Cursor, _super);
        function Cursor(editorId, configuration, model, viewModelHelper, enableEmptySelectionClipboard) {
            var _this = this;
            _super.call(this, [
                editorCommon.EventType.CursorPositionChanged,
                editorCommon.EventType.CursorSelectionChanged,
                editorCommon.EventType.CursorRevealRange,
                editorCommon.EventType.CursorScrollRequest
            ]);
            this._columnSelectToLineNumber = 0;
            this._columnSelectToVisualColumn = 0;
            this.editorId = editorId;
            this.configuration = configuration;
            this.model = model;
            this.viewModelHelper = viewModelHelper;
            this.enableEmptySelectionClipboard = enableEmptySelectionClipboard;
            if (!this.viewModelHelper) {
                this.viewModelHelper = {
                    viewModel: this.model,
                    convertModelPositionToViewPosition: function (lineNumber, column) {
                        return new position_1.Position(lineNumber, column);
                    },
                    convertModelRangeToViewRange: function (modelRange) {
                        return modelRange;
                    },
                    convertViewToModelPosition: function (lineNumber, column) {
                        return new position_1.Position(lineNumber, column);
                    },
                    convertViewSelectionToModelSelection: function (viewSelection) {
                        return viewSelection;
                    },
                    validateViewPosition: function (viewLineNumber, viewColumn, modelPosition) {
                        return modelPosition;
                    },
                    validateViewRange: function (viewStartLineNumber, viewStartColumn, viewEndLineNumber, viewEndColumn, modelRange) {
                        return modelRange;
                    }
                };
            }
            this.cursors = new cursorCollection_1.CursorCollection(this.editorId, this.model, this.configuration, this.viewModelHelper);
            this.cursorUndoStack = [];
            this.typingListeners = {};
            this._isHandling = false;
            this.modelUnbinds = [];
            this.modelUnbinds.push(this.model.addListener2(editorCommon.EventType.ModelContentChanged, function (e) {
                _this._onModelContentChanged(e);
            }));
            this.modelUnbinds.push(this.model.addListener2(editorCommon.EventType.ModelModeChanged, function (e) {
                _this._onModelModeChanged();
            }));
            this.modelUnbinds.push(this.model.addListener2(editorCommon.EventType.ModelModeSupportChanged, function (e) {
                // TODO@Alex: react only if certain supports changed?
                _this._onModelModeChanged();
            }));
            this._registerHandlers();
        }
        Cursor.prototype.dispose = function () {
            this.modelUnbinds = lifecycle_1.disposeAll(this.modelUnbinds);
            this.model = null;
            this.cursors.dispose();
            this.cursors = null;
            this.configuration.handlerDispatcher.clearHandlers();
            this.configuration = null;
            this.viewModelHelper = null;
            _super.prototype.dispose.call(this);
        };
        Cursor.prototype.saveState = function () {
            var selections = this.cursors.getSelections(), result = [], selection;
            for (var i = 0; i < selections.length; i++) {
                selection = selections[i];
                result.push({
                    inSelectionMode: !selection.isEmpty(),
                    selectionStart: {
                        lineNumber: selection.selectionStartLineNumber,
                        column: selection.selectionStartColumn,
                    },
                    position: {
                        lineNumber: selection.positionLineNumber,
                        column: selection.positionColumn,
                    }
                });
            }
            return result;
        };
        Cursor.prototype.restoreState = function (states) {
            var _this = this;
            var desiredSelections = [], state;
            for (var i = 0; i < states.length; i++) {
                state = states[i];
                var positionLineNumber = 1, positionColumn = 1;
                // Avoid missing properties on the literal
                if (state.position && state.position.lineNumber) {
                    positionLineNumber = state.position.lineNumber;
                }
                if (state.position && state.position.column) {
                    positionColumn = state.position.column;
                }
                var selectionStartLineNumber = positionLineNumber, selectionStartColumn = positionColumn;
                // Avoid missing properties on the literal
                if (state.selectionStart && state.selectionStart.lineNumber) {
                    selectionStartLineNumber = state.selectionStart.lineNumber;
                }
                if (state.selectionStart && state.selectionStart.column) {
                    selectionStartColumn = state.selectionStart.column;
                }
                desiredSelections.push({
                    selectionStartLineNumber: selectionStartLineNumber,
                    selectionStartColumn: selectionStartColumn,
                    positionLineNumber: positionLineNumber,
                    positionColumn: positionColumn
                });
            }
            this._onHandler('restoreState', function (ctx) {
                _this.cursors.setSelections(desiredSelections);
                return false;
            }, new handlerDispatcher_1.DispatcherEvent('restoreState', null));
        };
        Cursor.prototype.setEditableRange = function (range) {
            this.model.setEditableRange(range);
        };
        Cursor.prototype.getEditableRange = function () {
            return this.model.getEditableRange();
        };
        Cursor.prototype.addTypingListener = function (character, callback) {
            if (!this.typingListeners.hasOwnProperty(character)) {
                this.typingListeners[character] = [];
            }
            this.typingListeners[character].push(callback);
        };
        Cursor.prototype.removeTypingListener = function (character, callback) {
            if (this.typingListeners.hasOwnProperty(character)) {
                var listeners = this.typingListeners[character];
                for (var i = 0; i < listeners.length; i++) {
                    if (listeners[i] === callback) {
                        listeners.splice(i, 1);
                        return;
                    }
                }
            }
        };
        Cursor.prototype._onModelModeChanged = function () {
            // the mode of this model has changed
            this.cursors.updateMode();
        };
        Cursor.prototype._onModelContentChanged = function (e) {
            var _this = this;
            if (e.changeType === editorCommon.EventType.ModelContentChangedFlush) {
                // a model.setValue() was called
                this.cursors.dispose();
                this.cursors = new cursorCollection_1.CursorCollection(this.editorId, this.model, this.configuration, this.viewModelHelper);
                this.emitCursorPositionChanged('model', 'contentFlush');
                this.emitCursorSelectionChanged('model', 'contentFlush');
            }
            else {
                if (!this._isHandling) {
                    this._onHandler('recoverSelectionFromMarkers', function (ctx) {
                        var result = _this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor.recoverSelectionFromMarkers(oneCtx); });
                        ctx.shouldPushStackElementBefore = false;
                        ctx.shouldPushStackElementAfter = false;
                        return result;
                    }, new handlerDispatcher_1.DispatcherEvent('modelChange', null));
                }
            }
        };
        // ------ some getters/setters
        Cursor.prototype.getSelection = function () {
            return this.cursors.getSelection(0);
        };
        Cursor.prototype.getSelections = function () {
            return this.cursors.getSelections();
        };
        Cursor.prototype.getPosition = function () {
            return this.cursors.getPosition(0);
        };
        Cursor.prototype.setSelections = function (source, selections) {
            var _this = this;
            this._onHandler('setSelections', function (ctx) {
                ctx.shouldReveal = false;
                _this.cursors.setSelections(selections);
                return false;
            }, new handlerDispatcher_1.DispatcherEvent(source, null));
        };
        // ------ auxiliary handling logic
        Cursor.prototype._createAndInterpretHandlerCtx = function (eventSource, eventData, callback) {
            var currentHandlerCtx = {
                cursorPositionChangeReason: '',
                shouldReveal: true,
                shouldRevealVerticalInCenter: false,
                shouldRevealHorizontal: true,
                shouldRevealTarget: RevealTarget.Primary,
                eventSource: eventSource,
                eventData: eventData,
                executeCommands: [],
                hasExecutedCommands: false,
                isCursorUndo: false,
                postOperationRunnables: [],
                shouldPushStackElementBefore: false,
                shouldPushStackElementAfter: false,
                requestScrollDeltaLines: 0,
                setColumnSelectToLineNumber: 0,
                setColumnSelectToVisualColumn: 0
            };
            callback(currentHandlerCtx);
            this._interpretHandlerContext(currentHandlerCtx);
            this.cursors.normalize();
            return currentHandlerCtx.hasExecutedCommands;
        };
        Cursor.prototype._onHandler = function (command, handler, e) {
            this._isHandling = true;
            this.charactersTyped = '';
            var handled = false;
            try {
                var oldSelections = this.cursors.getSelections();
                var oldViewSelections = this.cursors.getViewSelections();
                var prevCursorsState = this.cursors.saveState();
                var eventSource = e.getSource();
                var cursorPositionChangeReason;
                var shouldReveal;
                var shouldRevealVerticalInCenter;
                var shouldRevealHorizontal;
                var shouldRevealTarget;
                var isCursorUndo;
                var requestScrollDeltaLines;
                var hasExecutedCommands = this._createAndInterpretHandlerCtx(eventSource, e.getData(), function (currentHandlerCtx) {
                    handled = handler(currentHandlerCtx);
                    cursorPositionChangeReason = currentHandlerCtx.cursorPositionChangeReason;
                    shouldReveal = currentHandlerCtx.shouldReveal;
                    shouldRevealTarget = currentHandlerCtx.shouldRevealTarget;
                    shouldRevealVerticalInCenter = currentHandlerCtx.shouldRevealVerticalInCenter;
                    shouldRevealHorizontal = currentHandlerCtx.shouldRevealHorizontal;
                    isCursorUndo = currentHandlerCtx.isCursorUndo;
                    requestScrollDeltaLines = currentHandlerCtx.requestScrollDeltaLines;
                });
                if (hasExecutedCommands) {
                    this.cursorUndoStack = [];
                }
                // Ping typing listeners after the model emits events & after I emit events
                for (var i = 0; i < this.charactersTyped.length; i++) {
                    var chr = this.charactersTyped.charAt(i);
                    if (this.typingListeners.hasOwnProperty(chr)) {
                        var listeners = this.typingListeners[chr].slice(0);
                        for (var j = 0, lenJ = listeners.length; j < lenJ; j++) {
                            // Hoping that listeners understand that the view might be in an awkward state
                            try {
                                listeners[j]();
                            }
                            catch (e) {
                                errors_1.onUnexpectedError(e);
                            }
                        }
                    }
                }
                var newSelections = this.cursors.getSelections();
                var newViewSelections = this.cursors.getViewSelections();
                var somethingChanged = false;
                if (oldSelections.length !== newSelections.length) {
                    somethingChanged = true;
                }
                else {
                    for (var i = 0, len = oldSelections.length; !somethingChanged && i < len; i++) {
                        if (!oldSelections[i].equalsSelection(newSelections[i])) {
                            somethingChanged = true;
                        }
                    }
                    for (var i = 0, len = oldViewSelections.length; !somethingChanged && i < len; i++) {
                        if (!oldViewSelections[i].equalsSelection(newViewSelections[i])) {
                            somethingChanged = true;
                        }
                    }
                }
                if (somethingChanged) {
                    if (!hasExecutedCommands && !isCursorUndo) {
                        this.cursorUndoStack.push(prevCursorsState);
                    }
                    if (this.cursorUndoStack.length > 50) {
                        this.cursorUndoStack = this.cursorUndoStack.splice(0, this.cursorUndoStack.length - 50);
                    }
                    this.emitCursorPositionChanged(eventSource, cursorPositionChangeReason);
                    if (shouldReveal) {
                        this.emitCursorRevealRange(shouldRevealTarget, shouldRevealVerticalInCenter ? editorCommon.VerticalRevealType.Center : editorCommon.VerticalRevealType.Simple, shouldRevealHorizontal);
                    }
                    this.emitCursorSelectionChanged(eventSource, cursorPositionChangeReason);
                }
                if (requestScrollDeltaLines) {
                    this.emitCursorScrollRequest(requestScrollDeltaLines);
                }
            }
            catch (err) {
                errors_1.onUnexpectedError(err);
            }
            this._isHandling = false;
            return handled;
        };
        Cursor.prototype._interpretHandlerContext = function (ctx) {
            if (ctx.shouldPushStackElementBefore) {
                this.model.pushStackElement();
                ctx.shouldPushStackElementBefore = false;
            }
            this._columnSelectToLineNumber = ctx.setColumnSelectToLineNumber;
            this._columnSelectToVisualColumn = ctx.setColumnSelectToVisualColumn;
            ctx.hasExecutedCommands = this._internalExecuteCommands(ctx.executeCommands, ctx.postOperationRunnables) || ctx.hasExecutedCommands;
            ctx.executeCommands = [];
            if (ctx.shouldPushStackElementAfter) {
                this.model.pushStackElement();
                ctx.shouldPushStackElementAfter = false;
            }
            var hasPostOperationRunnables = false;
            for (var i = 0, len = ctx.postOperationRunnables.length; i < len; i++) {
                if (ctx.postOperationRunnables[i]) {
                    hasPostOperationRunnables = true;
                    break;
                }
            }
            if (hasPostOperationRunnables) {
                var postOperationRunnables = ctx.postOperationRunnables.slice(0);
                ctx.postOperationRunnables = [];
                this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) {
                    if (postOperationRunnables[cursorIndex]) {
                        postOperationRunnables[cursorIndex](oneCtx);
                    }
                    return false;
                });
                this._interpretHandlerContext(ctx);
            }
        };
        Cursor.prototype._interpretCommandResult = function (cursorState) {
            if (!cursorState) {
                return false;
            }
            this.cursors.setSelections(cursorState);
            return true;
        };
        Cursor.prototype._getEditOperationsFromCommand = function (ctx, majorIdentifier, command) {
            var _this = this;
            // This method acts as a transaction, if the command fails
            // everything it has done is ignored
            var operations = [], operationMinor = 0;
            var addEditOperation = function (selection, text) {
                if (selection.isEmpty() && text === '') {
                    // This command wants to add a no-op => no thank you
                    return;
                }
                operations.push({
                    identifier: {
                        major: majorIdentifier,
                        minor: operationMinor++
                    },
                    range: selection,
                    text: text,
                    forceMoveMarkers: false
                });
            };
            var hadTrackedRange = false;
            var trackSelection = function (selection, trackPreviousOnEmpty) {
                var selectionMarkerStickToPreviousCharacter, positionMarkerStickToPreviousCharacter;
                if (selection.isEmpty()) {
                    // Try to lock it with surrounding text
                    if (typeof trackPreviousOnEmpty === 'boolean') {
                        selectionMarkerStickToPreviousCharacter = trackPreviousOnEmpty;
                        positionMarkerStickToPreviousCharacter = trackPreviousOnEmpty;
                    }
                    else {
                        var maxLineColumn = _this.model.getLineMaxColumn(selection.startLineNumber);
                        if (selection.startColumn === maxLineColumn) {
                            selectionMarkerStickToPreviousCharacter = true;
                            positionMarkerStickToPreviousCharacter = true;
                        }
                        else {
                            selectionMarkerStickToPreviousCharacter = false;
                            positionMarkerStickToPreviousCharacter = false;
                        }
                    }
                }
                else {
                    if (selection.getDirection() === editorCommon.SelectionDirection.LTR) {
                        selectionMarkerStickToPreviousCharacter = false;
                        positionMarkerStickToPreviousCharacter = true;
                    }
                    else {
                        selectionMarkerStickToPreviousCharacter = true;
                        positionMarkerStickToPreviousCharacter = false;
                    }
                }
                var l = ctx.selectionStartMarkers.length;
                ctx.selectionStartMarkers[l] = _this.model._addMarker(selection.selectionStartLineNumber, selection.selectionStartColumn, selectionMarkerStickToPreviousCharacter);
                ctx.positionMarkers[l] = _this.model._addMarker(selection.positionLineNumber, selection.positionColumn, positionMarkerStickToPreviousCharacter);
                return l.toString();
            };
            var editOperationBuilder = {
                addEditOperation: addEditOperation,
                trackSelection: trackSelection
            };
            try {
                command.getEditOperations(this.model, editOperationBuilder);
            }
            catch (e) {
                e.friendlyMessage = nls.localize(0, null);
                errors_1.onUnexpectedError(e);
                return {
                    operations: [],
                    hadTrackedRange: false
                };
            }
            return {
                operations: operations,
                hadTrackedRange: hadTrackedRange
            };
        };
        Cursor.prototype._getEditOperations = function (ctx, commands) {
            var oneResult;
            var operations = [];
            var hadTrackedRanges = [];
            var anyoneHadTrackedRange;
            for (var i = 0; i < commands.length; i++) {
                if (commands[i]) {
                    oneResult = this._getEditOperationsFromCommand(ctx, i, commands[i]);
                    operations = operations.concat(oneResult.operations);
                    hadTrackedRanges[i] = oneResult.hadTrackedRange;
                    anyoneHadTrackedRange = anyoneHadTrackedRange || hadTrackedRanges[i];
                }
                else {
                    hadTrackedRanges[i] = false;
                }
            }
            return {
                operations: operations,
                hadTrackedRanges: hadTrackedRanges,
                anyoneHadTrackedRange: anyoneHadTrackedRange
            };
        };
        Cursor.prototype._getLoserCursorMap = function (operations) {
            // This is destructive on the array
            operations = operations.slice(0);
            // Sort operations with last one first
            operations.sort(function (a, b) {
                // Note the minus!
                return -(range_1.Range.compareRangesUsingEnds(a.range, b.range));
            });
            // Operations can not overlap!
            var loserCursorsMap = {};
            var previousOp;
            var currentOp;
            var loserMajor;
            for (var i = 1; i < operations.length; i++) {
                previousOp = operations[i - 1];
                currentOp = operations[i];
                if (previousOp.range.getStartPosition().isBefore(currentOp.range.getEndPosition())) {
                    if (previousOp.identifier.major > currentOp.identifier.major) {
                        // previousOp loses the battle
                        loserMajor = previousOp.identifier.major;
                    }
                    else {
                        loserMajor = currentOp.identifier.major;
                    }
                    loserCursorsMap[loserMajor.toString()] = true;
                    for (var j = 0; j < operations.length; j++) {
                        if (operations[j].identifier.major === loserMajor) {
                            operations.splice(j, 1);
                            if (j < i) {
                                i--;
                            }
                            j--;
                        }
                    }
                    if (i > 0) {
                        i--;
                    }
                }
            }
            return loserCursorsMap;
        };
        Cursor.prototype._collapseDeleteCommands = function (rawCmds, postOperationRunnables) {
            if (rawCmds.length === 1) {
                return;
            }
            // Merge adjacent delete commands
            var allAreDeleteCommands = rawCmds.every(function (command) {
                if (!(command instanceof replaceCommand_1.ReplaceCommand)) {
                    return false;
                }
                var replCmd = command;
                if (replCmd.getText().length > 0) {
                    return false;
                }
                return true;
            });
            if (!allAreDeleteCommands) {
                return;
            }
            var commands = rawCmds;
            var cursors = commands.map(function (cmd, i) {
                return {
                    range: commands[i].getRange(),
                    postOperationRunnable: postOperationRunnables[i],
                    order: i
                };
            });
            cursors.sort(function (a, b) {
                return range_1.Range.compareRangesUsingStarts(a.range, b.range);
            });
            var previousCursor = cursors[0];
            for (var i = 1; i < cursors.length; i++) {
                if (previousCursor.range.endLineNumber === cursors[i].range.startLineNumber && previousCursor.range.endColumn === cursors[i].range.startColumn) {
                    // Merge ranges
                    var mergedRange = new range_1.Range(previousCursor.range.startLineNumber, previousCursor.range.startColumn, cursors[i].range.endLineNumber, cursors[i].range.endColumn);
                    previousCursor.range = mergedRange;
                    commands[cursors[i].order].setRange(mergedRange);
                    commands[previousCursor.order].setRange(mergedRange);
                }
                else {
                    // Push previous cursor
                    previousCursor = cursors[i];
                }
            }
        };
        Cursor.prototype._internalExecuteCommands = function (commands, postOperationRunnables) {
            var ctx = {
                selectionStartMarkers: [],
                positionMarkers: []
            };
            this._collapseDeleteCommands(commands, postOperationRunnables);
            var r = this._innerExecuteCommands(ctx, commands, postOperationRunnables);
            for (var i = 0; i < ctx.selectionStartMarkers.length; i++) {
                this.model._removeMarker(ctx.selectionStartMarkers[i]);
                this.model._removeMarker(ctx.positionMarkers[i]);
            }
            return r;
        };
        Cursor.prototype._arrayIsEmpty = function (commands) {
            var i, len;
            for (i = 0, len = commands.length; i < len; i++) {
                if (commands[i]) {
                    return false;
                }
            }
            return true;
        };
        Cursor.prototype._innerExecuteCommands = function (ctx, commands, postOperationRunnables) {
            var _this = this;
            if (this.configuration.editor.readOnly) {
                return false;
            }
            if (this._arrayIsEmpty(commands)) {
                return false;
            }
            var selectionsBefore = this.cursors.getSelections();
            var commandsData = this._getEditOperations(ctx, commands);
            if (commandsData.operations.length === 0 && !commandsData.anyoneHadTrackedRange) {
                return false;
            }
            var rawOperations = commandsData.operations;
            var editableRange = this.model.getEditableRange();
            var editableRangeStart = editableRange.getStartPosition();
            var editableRangeEnd = editableRange.getEndPosition();
            for (var i = 0; i < rawOperations.length; i++) {
                var operationRange = rawOperations[i].range;
                if (!editableRangeStart.isBeforeOrEqual(operationRange.getStartPosition()) || !operationRange.getEndPosition().isBeforeOrEqual(editableRangeEnd)) {
                    // These commands are outside of the editable range
                    return false;
                }
            }
            var loserCursorsMap = this._getLoserCursorMap(rawOperations);
            if (loserCursorsMap.hasOwnProperty('0')) {
                // These commands are very messed up
                console.warn('Ignoring commands');
                return false;
            }
            // Remove operations belonging to losing cursors
            var filteredOperations = [];
            for (var i = 0; i < rawOperations.length; i++) {
                if (!loserCursorsMap.hasOwnProperty(rawOperations[i].identifier.major.toString())) {
                    filteredOperations.push(rawOperations[i]);
                }
            }
            var selectionsAfter = this.model.pushEditOperations(selectionsBefore, filteredOperations, function (inverseEditOperations) {
                var groupedInverseEditOperations = [];
                for (var i = 0; i < selectionsBefore.length; i++) {
                    groupedInverseEditOperations[i] = [];
                }
                for (var i = 0; i < inverseEditOperations.length; i++) {
                    var op = inverseEditOperations[i];
                    groupedInverseEditOperations[op.identifier.major].push(op);
                }
                var minorBasedSorter = function (a, b) {
                    return a.identifier.minor - b.identifier.minor;
                };
                var cursorSelections = [];
                for (var i = 0; i < selectionsBefore.length; i++) {
                    if (groupedInverseEditOperations[i].length > 0 || commandsData.hadTrackedRanges[i]) {
                        groupedInverseEditOperations[i].sort(minorBasedSorter);
                        cursorSelections[i] = commands[i].computeCursorState(_this.model, {
                            getInverseEditOperations: function () {
                                return groupedInverseEditOperations[i];
                            },
                            getTrackedSelection: function (id) {
                                var idx = parseInt(id, 10);
                                var selectionStartMarker = _this.model._getMarker(ctx.selectionStartMarkers[idx]);
                                var positionMarker = _this.model._getMarker(ctx.positionMarkers[idx]);
                                return new selection_1.Selection(selectionStartMarker.lineNumber, selectionStartMarker.column, positionMarker.lineNumber, positionMarker.column);
                            }
                        });
                    }
                    else {
                        cursorSelections[i] = selectionsBefore[i];
                    }
                }
                return cursorSelections;
            });
            // Extract losing cursors
            var losingCursorIndex;
            var losingCursors = [];
            for (losingCursorIndex in loserCursorsMap) {
                if (loserCursorsMap.hasOwnProperty(losingCursorIndex)) {
                    losingCursors.push(parseInt(losingCursorIndex, 10));
                }
            }
            // Sort losing cursors descending
            losingCursors.sort(function (a, b) {
                return b - a;
            });
            // Remove losing cursors
            for (var i = 0; i < losingCursors.length; i++) {
                selectionsAfter.splice(losingCursors[i], 1);
                postOperationRunnables.splice(losingCursors[i], 1);
            }
            return this._interpretCommandResult(selectionsAfter);
        };
        // -----------------------------------------------------------------------------------------------------------
        // ----- emitting events
        Cursor.prototype.emitCursorPositionChanged = function (source, reason) {
            var positions = this.cursors.getPositions();
            var primaryPosition = positions[0];
            var secondaryPositions = positions.slice(1);
            var viewPositions = this.cursors.getViewPositions();
            var primaryViewPosition = viewPositions[0];
            var secondaryViewPositions = viewPositions.slice(1);
            var isInEditableRange = true;
            if (this.model.hasEditableRange()) {
                var editableRange = this.model.getEditableRange();
                if (!editableRange.containsPosition(primaryPosition)) {
                    isInEditableRange = false;
                }
            }
            var e = {
                position: primaryPosition,
                viewPosition: primaryViewPosition,
                secondaryPositions: secondaryPositions,
                secondaryViewPositions: secondaryViewPositions,
                reason: reason,
                source: source,
                isInEditableRange: isInEditableRange
            };
            this.emit(editorCommon.EventType.CursorPositionChanged, e);
        };
        Cursor.prototype.emitCursorSelectionChanged = function (source, reason) {
            var selections = this.cursors.getSelections();
            var primarySelection = selections[0];
            var secondarySelections = selections.slice(1);
            var viewSelections = this.cursors.getViewSelections();
            var primaryViewSelection = viewSelections[0];
            var secondaryViewSelections = viewSelections.slice(1);
            var e = {
                selection: primarySelection,
                viewSelection: primaryViewSelection,
                secondarySelections: secondarySelections,
                secondaryViewSelections: secondaryViewSelections,
                source: source,
                reason: reason
            };
            this.emit(editorCommon.EventType.CursorSelectionChanged, e);
        };
        Cursor.prototype.emitCursorScrollRequest = function (lineScrollOffset) {
            var e = {
                deltaLines: lineScrollOffset
            };
            this.emit(editorCommon.EventType.CursorScrollRequest, e);
        };
        Cursor.prototype.emitCursorRevealRange = function (revealTarget, verticalType, revealHorizontal) {
            var positions = this.cursors.getPositions();
            var viewPositions = this.cursors.getViewPositions();
            var position = positions[0];
            var viewPosition = viewPositions[0];
            if (revealTarget === RevealTarget.TopMost) {
                for (var i = 1; i < positions.length; i++) {
                    if (positions[i].isBefore(position)) {
                        position = positions[i];
                        viewPosition = viewPositions[i];
                    }
                }
            }
            else if (revealTarget === RevealTarget.BottomMost) {
                for (var i = 1; i < positions.length; i++) {
                    if (position.isBeforeOrEqual(positions[i])) {
                        position = positions[i];
                        viewPosition = viewPositions[i];
                    }
                }
            }
            else {
                if (positions.length > 1) {
                    // no revealing!
                    return;
                }
            }
            var range = new range_1.Range(position.lineNumber, position.column, position.lineNumber, position.column);
            var viewRange = new range_1.Range(viewPosition.lineNumber, viewPosition.column, viewPosition.lineNumber, viewPosition.column);
            var e = {
                range: range,
                viewRange: viewRange,
                verticalType: verticalType,
                revealHorizontal: revealHorizontal
            };
            this.emit(editorCommon.EventType.CursorRevealRange, e);
        };
        // -----------------------------------------------------------------------------------------------------------
        // ----- handlers beyond this point
        Cursor.prototype._registerHandlers = function () {
            var _this = this;
            var H = editorCommon.Handler;
            var handlersMap = {};
            handlersMap[H.JumpToBracket] = function (ctx) { return _this._jumpToBracket(ctx); };
            handlersMap[H.MoveTo] = function (ctx) { return _this._moveTo(false, ctx); };
            handlersMap[H.MoveToSelect] = function (ctx) { return _this._moveTo(true, ctx); };
            handlersMap[H.ColumnSelect] = function (ctx) { return _this._columnSelectMouse(ctx); };
            handlersMap[H.AddCursorUp] = function (ctx) { return _this._addCursorUp(ctx); };
            handlersMap[H.AddCursorDown] = function (ctx) { return _this._addCursorDown(ctx); };
            handlersMap[H.CreateCursor] = function (ctx) { return _this._createCursor(ctx); };
            handlersMap[H.LastCursorMoveToSelect] = function (ctx) { return _this._lastCursorMoveTo(ctx); };
            handlersMap[H.CursorLeft] = function (ctx) { return _this._moveLeft(false, ctx); };
            handlersMap[H.CursorLeftSelect] = function (ctx) { return _this._moveLeft(true, ctx); };
            handlersMap[H.CursorWordLeft] = function (ctx) { return _this._moveWordLeft(false, oneCursor_1.WordNavigationType.WordStart, ctx); };
            handlersMap[H.CursorWordStartLeft] = function (ctx) { return _this._moveWordLeft(false, oneCursor_1.WordNavigationType.WordStart, ctx); };
            handlersMap[H.CursorWordEndLeft] = function (ctx) { return _this._moveWordLeft(false, oneCursor_1.WordNavigationType.WordEnd, ctx); };
            handlersMap[H.CursorWordLeftSelect] = function (ctx) { return _this._moveWordLeft(true, oneCursor_1.WordNavigationType.WordStart, ctx); };
            handlersMap[H.CursorWordStartLeftSelect] = function (ctx) { return _this._moveWordLeft(true, oneCursor_1.WordNavigationType.WordStart, ctx); };
            handlersMap[H.CursorWordEndLeftSelect] = function (ctx) { return _this._moveWordLeft(true, oneCursor_1.WordNavigationType.WordEnd, ctx); };
            handlersMap[H.CursorRight] = function (ctx) { return _this._moveRight(false, ctx); };
            handlersMap[H.CursorRightSelect] = function (ctx) { return _this._moveRight(true, ctx); };
            handlersMap[H.CursorWordRight] = function (ctx) { return _this._moveWordRight(false, oneCursor_1.WordNavigationType.WordEnd, ctx); };
            handlersMap[H.CursorWordStartRight] = function (ctx) { return _this._moveWordRight(false, oneCursor_1.WordNavigationType.WordStart, ctx); };
            handlersMap[H.CursorWordEndRight] = function (ctx) { return _this._moveWordRight(false, oneCursor_1.WordNavigationType.WordEnd, ctx); };
            handlersMap[H.CursorWordRightSelect] = function (ctx) { return _this._moveWordRight(true, oneCursor_1.WordNavigationType.WordEnd, ctx); };
            handlersMap[H.CursorWordStartRightSelect] = function (ctx) { return _this._moveWordRight(true, oneCursor_1.WordNavigationType.WordStart, ctx); };
            handlersMap[H.CursorWordEndRightSelect] = function (ctx) { return _this._moveWordRight(true, oneCursor_1.WordNavigationType.WordEnd, ctx); };
            handlersMap[H.CursorUp] = function (ctx) { return _this._moveUp(false, false, ctx); };
            handlersMap[H.CursorUpSelect] = function (ctx) { return _this._moveUp(true, false, ctx); };
            handlersMap[H.CursorDown] = function (ctx) { return _this._moveDown(false, false, ctx); };
            handlersMap[H.CursorDownSelect] = function (ctx) { return _this._moveDown(true, false, ctx); };
            handlersMap[H.CursorPageUp] = function (ctx) { return _this._moveUp(false, true, ctx); };
            handlersMap[H.CursorPageUpSelect] = function (ctx) { return _this._moveUp(true, true, ctx); };
            handlersMap[H.CursorPageDown] = function (ctx) { return _this._moveDown(false, true, ctx); };
            handlersMap[H.CursorPageDownSelect] = function (ctx) { return _this._moveDown(true, true, ctx); };
            handlersMap[H.CursorHome] = function (ctx) { return _this._moveToBeginningOfLine(false, ctx); };
            handlersMap[H.CursorHomeSelect] = function (ctx) { return _this._moveToBeginningOfLine(true, ctx); };
            handlersMap[H.CursorEnd] = function (ctx) { return _this._moveToEndOfLine(false, ctx); };
            handlersMap[H.CursorEndSelect] = function (ctx) { return _this._moveToEndOfLine(true, ctx); };
            handlersMap[H.CursorTop] = function (ctx) { return _this._moveToBeginningOfBuffer(false, ctx); };
            handlersMap[H.CursorTopSelect] = function (ctx) { return _this._moveToBeginningOfBuffer(true, ctx); };
            handlersMap[H.CursorBottom] = function (ctx) { return _this._moveToEndOfBuffer(false, ctx); };
            handlersMap[H.CursorBottomSelect] = function (ctx) { return _this._moveToEndOfBuffer(true, ctx); };
            handlersMap[H.CursorColumnSelectLeft] = function (ctx) { return _this._columnSelectLeft(ctx); };
            handlersMap[H.CursorColumnSelectRight] = function (ctx) { return _this._columnSelectRight(ctx); };
            handlersMap[H.CursorColumnSelectUp] = function (ctx) { return _this._columnSelectUp(false, ctx); };
            handlersMap[H.CursorColumnSelectPageUp] = function (ctx) { return _this._columnSelectUp(true, ctx); };
            handlersMap[H.CursorColumnSelectDown] = function (ctx) { return _this._columnSelectDown(false, ctx); };
            handlersMap[H.CursorColumnSelectPageDown] = function (ctx) { return _this._columnSelectDown(true, ctx); };
            handlersMap[H.SelectAll] = function (ctx) { return _this._selectAll(ctx); };
            handlersMap[H.LineSelect] = function (ctx) { return _this._line(false, ctx); };
            handlersMap[H.LineSelectDrag] = function (ctx) { return _this._line(true, ctx); };
            handlersMap[H.LastCursorLineSelect] = function (ctx) { return _this._lastCursorLine(false, ctx); };
            handlersMap[H.LastCursorLineSelectDrag] = function (ctx) { return _this._lastCursorLine(true, ctx); };
            handlersMap[H.LineInsertBefore] = function (ctx) { return _this._lineInsertBefore(ctx); };
            handlersMap[H.LineInsertAfter] = function (ctx) { return _this._lineInsertAfter(ctx); };
            handlersMap[H.LineBreakInsert] = function (ctx) { return _this._lineBreakInsert(ctx); };
            handlersMap[H.WordSelect] = function (ctx) { return _this._word(false, ctx); };
            handlersMap[H.WordSelectDrag] = function (ctx) { return _this._word(true, ctx); };
            handlersMap[H.LastCursorWordSelect] = function (ctx) { return _this._lastCursorWord(ctx); };
            handlersMap[H.CancelSelection] = function (ctx) { return _this._cancelSelection(ctx); };
            handlersMap[H.RemoveSecondaryCursors] = function (ctx) { return _this._removeSecondaryCursors(ctx); };
            handlersMap[H.Type] = function (ctx) { return _this._type(ctx); };
            handlersMap[H.ReplacePreviousChar] = function (ctx) { return _this._replacePreviousChar(ctx); };
            handlersMap[H.Tab] = function (ctx) { return _this._tab(ctx); };
            handlersMap[H.Indent] = function (ctx) { return _this._indent(ctx); };
            handlersMap[H.Outdent] = function (ctx) { return _this._outdent(ctx); };
            handlersMap[H.Paste] = function (ctx) { return _this._paste(ctx); };
            handlersMap[H.ScrollLineUp] = function (ctx) { return _this._scrollUp(false, ctx); };
            handlersMap[H.ScrollLineDown] = function (ctx) { return _this._scrollDown(false, ctx); };
            handlersMap[H.ScrollPageUp] = function (ctx) { return _this._scrollUp(true, ctx); };
            handlersMap[H.ScrollPageDown] = function (ctx) { return _this._scrollDown(true, ctx); };
            handlersMap[H.DeleteLeft] = function (ctx) { return _this._deleteLeft(ctx); };
            handlersMap[H.DeleteWordLeft] = function (ctx) { return _this._deleteWordLeft(true, oneCursor_1.WordNavigationType.WordStart, ctx); };
            handlersMap[H.DeleteWordStartLeft] = function (ctx) { return _this._deleteWordLeft(false, oneCursor_1.WordNavigationType.WordStart, ctx); };
            handlersMap[H.DeleteWordEndLeft] = function (ctx) { return _this._deleteWordLeft(false, oneCursor_1.WordNavigationType.WordEnd, ctx); };
            handlersMap[H.DeleteRight] = function (ctx) { return _this._deleteRight(ctx); };
            handlersMap[H.DeleteWordRight] = function (ctx) { return _this._deleteWordRight(true, oneCursor_1.WordNavigationType.WordEnd, ctx); };
            handlersMap[H.DeleteWordStartRight] = function (ctx) { return _this._deleteWordRight(false, oneCursor_1.WordNavigationType.WordStart, ctx); };
            handlersMap[H.DeleteWordEndRight] = function (ctx) { return _this._deleteWordRight(false, oneCursor_1.WordNavigationType.WordEnd, ctx); };
            handlersMap[H.DeleteAllLeft] = function (ctx) { return _this._deleteAllLeft(ctx); };
            handlersMap[H.DeleteAllRight] = function (ctx) { return _this._deleteAllRight(ctx); };
            handlersMap[H.Cut] = function (ctx) { return _this._cut(ctx); };
            handlersMap[H.ExpandLineSelection] = function (ctx) { return _this._expandLineSelection(ctx); };
            handlersMap[H.Undo] = function (ctx) { return _this._undo(ctx); };
            handlersMap[H.CursorUndo] = function (ctx) { return _this._cursorUndo(ctx); };
            handlersMap[H.Redo] = function (ctx) { return _this._redo(ctx); };
            handlersMap[H.ExecuteCommand] = function (ctx) { return _this._externalExecuteCommand(ctx); };
            handlersMap[H.ExecuteCommands] = function (ctx) { return _this._externalExecuteCommands(ctx); };
            var createHandler = function (handlerId, handlerExec) {
                return function (e) { return _this._onHandler(handlerId, handlerExec, e); };
            };
            var handler;
            for (handler in handlersMap) {
                if (handlersMap.hasOwnProperty(handler)) {
                    this.configuration.handlerDispatcher.setHandler(handler, createHandler(handler, handlersMap[handler]));
                }
            }
        };
        Cursor.prototype._invokeForAllSorted = function (ctx, callable, pushStackElementBefore, pushStackElementAfter) {
            if (pushStackElementBefore === void 0) { pushStackElementBefore = true; }
            if (pushStackElementAfter === void 0) { pushStackElementAfter = true; }
            return this._doInvokeForAll(ctx, true, callable, pushStackElementBefore, pushStackElementAfter);
        };
        Cursor.prototype._invokeForAll = function (ctx, callable, pushStackElementBefore, pushStackElementAfter) {
            if (pushStackElementBefore === void 0) { pushStackElementBefore = true; }
            if (pushStackElementAfter === void 0) { pushStackElementAfter = true; }
            return this._doInvokeForAll(ctx, false, callable, pushStackElementBefore, pushStackElementAfter);
        };
        Cursor.prototype._doInvokeForAll = function (ctx, sorted, callable, pushStackElementBefore, pushStackElementAfter) {
            if (pushStackElementBefore === void 0) { pushStackElementBefore = true; }
            if (pushStackElementAfter === void 0) { pushStackElementAfter = true; }
            var result = false;
            var cursors = this.cursors.getAll();
            if (sorted) {
                cursors = cursors.sort(function (a, b) {
                    return range_1.Range.compareRangesUsingStarts(a.getSelection(), b.getSelection());
                });
            }
            var context;
            ctx.shouldPushStackElementBefore = pushStackElementBefore;
            ctx.shouldPushStackElementAfter = pushStackElementAfter;
            for (var i = 0; i < cursors.length; i++) {
                context = {
                    cursorPositionChangeReason: '',
                    shouldReveal: true,
                    shouldRevealVerticalInCenter: false,
                    shouldRevealHorizontal: true,
                    executeCommand: null,
                    postOperationRunnable: null,
                    shouldPushStackElementBefore: false,
                    shouldPushStackElementAfter: false,
                    requestScrollDeltaLines: 0
                };
                result = callable(i, cursors[i], context) || result;
                if (i === 0) {
                    ctx.cursorPositionChangeReason = context.cursorPositionChangeReason;
                    ctx.shouldRevealHorizontal = context.shouldRevealHorizontal;
                    ctx.shouldReveal = context.shouldReveal;
                    ctx.shouldRevealVerticalInCenter = context.shouldRevealVerticalInCenter;
                    ctx.requestScrollDeltaLines = context.requestScrollDeltaLines;
                }
                ctx.shouldPushStackElementBefore = ctx.shouldPushStackElementBefore || context.shouldPushStackElementBefore;
                ctx.shouldPushStackElementAfter = ctx.shouldPushStackElementAfter || context.shouldPushStackElementAfter;
                ctx.executeCommands[i] = context.executeCommand;
                ctx.postOperationRunnables[i] = context.postOperationRunnable;
            }
            return result;
        };
        Cursor.prototype._jumpToBracket = function (ctx) {
            this.cursors.killSecondaryCursors();
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.jumpToBracket(oneCursor, oneCtx); });
        };
        Cursor.prototype._moveTo = function (inSelectionMode, ctx) {
            this.cursors.killSecondaryCursors();
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveTo(oneCursor, inSelectionMode, ctx.eventData.position, ctx.eventData.viewPosition, ctx.eventSource, oneCtx); });
        };
        Cursor.prototype._getColumnSelectToLineNumber = function () {
            if (!this._columnSelectToLineNumber) {
                var primaryCursor = this.cursors.getAll()[0];
                var primaryPos = primaryCursor.getViewPosition();
                return primaryPos.lineNumber;
            }
            return this._columnSelectToLineNumber;
        };
        Cursor.prototype._getColumnSelectToVisualColumn = function () {
            if (!this._columnSelectToVisualColumn) {
                var primaryCursor = this.cursors.getAll()[0];
                var primaryPos = primaryCursor.getViewPosition();
                return primaryCursor.getViewVisibleColumnFromColumn(primaryPos.lineNumber, primaryPos.column);
            }
            return this._columnSelectToVisualColumn;
        };
        Cursor.prototype._columnSelectMouse = function (ctx) {
            var cursors = this.cursors.getAll();
            var result = oneCursor_1.OneCursorOp.columnSelectMouse(cursors[0], ctx.eventData.position, ctx.eventData.viewPosition, ctx.eventData.mouseColumn - 1);
            ctx.shouldRevealTarget = (result.reversed ? RevealTarget.TopMost : RevealTarget.BottomMost);
            ctx.shouldReveal = true;
            ctx.setColumnSelectToLineNumber = result.toLineNumber;
            ctx.setColumnSelectToVisualColumn = result.toVisualColumn;
            this.cursors.setSelections(result.selections, result.viewSelections);
            return true;
        };
        Cursor.prototype._columnSelectOp = function (ctx, op) {
            var primary = this.cursors.getAll()[0];
            var result = op(primary, this._getColumnSelectToLineNumber(), this._getColumnSelectToVisualColumn());
            ctx.shouldRevealTarget = (result.reversed ? RevealTarget.TopMost : RevealTarget.BottomMost);
            ctx.shouldReveal = true;
            ctx.setColumnSelectToLineNumber = result.toLineNumber;
            ctx.setColumnSelectToVisualColumn = result.toVisualColumn;
            this.cursors.setSelections(result.selections, result.viewSelections);
            return true;
        };
        Cursor.prototype._columnSelectLeft = function (ctx) {
            return this._columnSelectOp(ctx, function (cursor, toViewLineNumber, toViewVisualColumn) { return oneCursor_1.OneCursorOp.columnSelectLeft(cursor, toViewLineNumber, toViewVisualColumn); });
        };
        Cursor.prototype._columnSelectRight = function (ctx) {
            return this._columnSelectOp(ctx, function (cursor, toViewLineNumber, toViewVisualColumn) { return oneCursor_1.OneCursorOp.columnSelectRight(cursor, toViewLineNumber, toViewVisualColumn); });
        };
        Cursor.prototype._columnSelectUp = function (isPaged, ctx) {
            return this._columnSelectOp(ctx, function (cursor, toViewLineNumber, toViewVisualColumn) { return oneCursor_1.OneCursorOp.columnSelectUp(isPaged, cursor, toViewLineNumber, toViewVisualColumn); });
        };
        Cursor.prototype._columnSelectDown = function (isPaged, ctx) {
            return this._columnSelectOp(ctx, function (cursor, toViewLineNumber, toViewVisualColumn) { return oneCursor_1.OneCursorOp.columnSelectDown(isPaged, cursor, toViewLineNumber, toViewVisualColumn); });
        };
        Cursor.prototype._createCursor = function (ctx) {
            if (this.configuration.editor.readOnly || this.model.hasEditableRange()) {
                return false;
            }
            this.cursors.addSecondaryCursor({
                selectionStartLineNumber: 1,
                selectionStartColumn: 1,
                positionLineNumber: 1,
                positionColumn: 1
            });
            // Manually move to get events
            var lastAddedCursor = this.cursors.getLastAddedCursor();
            this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) {
                if (oneCursor === lastAddedCursor) {
                    if (ctx.eventData.wholeLine) {
                        return oneCursor_1.OneCursorOp.line(oneCursor, false, ctx.eventData.position, ctx.eventData.viewPosition, oneCtx);
                    }
                    else {
                        return oneCursor_1.OneCursorOp.moveTo(oneCursor, false, ctx.eventData.position, ctx.eventData.viewPosition, ctx.eventSource, oneCtx);
                    }
                }
                return false;
            });
            ctx.shouldReveal = false;
            ctx.shouldRevealHorizontal = false;
            return true;
        };
        Cursor.prototype._lastCursorMoveTo = function (ctx) {
            if (this.configuration.editor.readOnly || this.model.hasEditableRange()) {
                return false;
            }
            var lastAddedCursor = this.cursors.getLastAddedCursor();
            this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) {
                if (oneCursor === lastAddedCursor) {
                    return oneCursor_1.OneCursorOp.moveTo(oneCursor, true, ctx.eventData.position, ctx.eventData.viewPosition, ctx.eventSource, oneCtx);
                }
                return false;
            });
            ctx.shouldReveal = false;
            ctx.shouldRevealHorizontal = false;
            return true;
        };
        Cursor.prototype._addCursorUp = function (ctx) {
            if (this.configuration.editor.readOnly) {
                return false;
            }
            var originalCnt = this.cursors.getSelections().length;
            this.cursors.duplicateCursors();
            ctx.shouldRevealTarget = RevealTarget.TopMost;
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) {
                if (cursorIndex >= originalCnt) {
                    return oneCursor_1.OneCursorOp.translateUp(oneCursor, oneCtx);
                }
                return false;
            });
        };
        Cursor.prototype._addCursorDown = function (ctx) {
            if (this.configuration.editor.readOnly) {
                return false;
            }
            var originalCnt = this.cursors.getSelections().length;
            this.cursors.duplicateCursors();
            ctx.shouldRevealTarget = RevealTarget.BottomMost;
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) {
                if (cursorIndex >= originalCnt) {
                    return oneCursor_1.OneCursorOp.translateDown(oneCursor, oneCtx);
                }
                return false;
            });
        };
        Cursor.prototype._moveLeft = function (inSelectionMode, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveLeft(oneCursor, inSelectionMode, oneCtx); });
        };
        Cursor.prototype._moveWordLeft = function (inSelectionMode, wordNavigationType, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveWordLeft(oneCursor, inSelectionMode, wordNavigationType, oneCtx); });
        };
        Cursor.prototype._moveRight = function (inSelectionMode, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveRight(oneCursor, inSelectionMode, oneCtx); });
        };
        Cursor.prototype._moveWordRight = function (inSelectionMode, wordNavigationType, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveWordRight(oneCursor, inSelectionMode, wordNavigationType, oneCtx); });
        };
        Cursor.prototype._moveDown = function (inSelectionMode, isPaged, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveDown(oneCursor, inSelectionMode, isPaged, oneCtx); });
        };
        Cursor.prototype._moveUp = function (inSelectionMode, isPaged, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveUp(oneCursor, inSelectionMode, isPaged, oneCtx); });
        };
        Cursor.prototype._moveToBeginningOfLine = function (inSelectionMode, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveToBeginningOfLine(oneCursor, inSelectionMode, oneCtx); });
        };
        Cursor.prototype._moveToEndOfLine = function (inSelectionMode, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveToEndOfLine(oneCursor, inSelectionMode, oneCtx); });
        };
        Cursor.prototype._moveToBeginningOfBuffer = function (inSelectionMode, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveToBeginningOfBuffer(oneCursor, inSelectionMode, oneCtx); });
        };
        Cursor.prototype._moveToEndOfBuffer = function (inSelectionMode, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.moveToEndOfBuffer(oneCursor, inSelectionMode, oneCtx); });
        };
        Cursor.prototype._selectAll = function (ctx) {
            this.cursors.killSecondaryCursors();
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.selectAll(oneCursor, oneCtx); });
        };
        Cursor.prototype._line = function (inSelectionMode, ctx) {
            this.cursors.killSecondaryCursors();
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.line(oneCursor, inSelectionMode, ctx.eventData.position, ctx.eventData.viewPosition, oneCtx); });
        };
        Cursor.prototype._lastCursorLine = function (inSelectionMode, ctx) {
            if (this.configuration.editor.readOnly || this.model.hasEditableRange()) {
                return false;
            }
            var lastAddedCursor = this.cursors.getLastAddedCursor();
            this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) {
                if (oneCursor === lastAddedCursor) {
                    return oneCursor_1.OneCursorOp.line(oneCursor, inSelectionMode, ctx.eventData.position, ctx.eventData.viewPosition, oneCtx);
                }
                return false;
            });
            ctx.shouldReveal = false;
            ctx.shouldRevealHorizontal = false;
            return true;
        };
        Cursor.prototype._expandLineSelection = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.expandLineSelection(oneCursor, oneCtx); });
        };
        Cursor.prototype._lineInsertBefore = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.lineInsertBefore(oneCursor, oneCtx); });
        };
        Cursor.prototype._lineInsertAfter = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.lineInsertAfter(oneCursor, oneCtx); });
        };
        Cursor.prototype._lineBreakInsert = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.lineBreakInsert(oneCursor, oneCtx); });
        };
        Cursor.prototype._word = function (inSelectionMode, ctx) {
            this.cursors.killSecondaryCursors();
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.word(oneCursor, inSelectionMode, ctx.eventData.position, oneCtx); });
        };
        Cursor.prototype._lastCursorWord = function (ctx) {
            if (this.configuration.editor.readOnly || this.model.hasEditableRange()) {
                return false;
            }
            var lastAddedCursor = this.cursors.getLastAddedCursor();
            this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) {
                if (oneCursor === lastAddedCursor) {
                    return oneCursor_1.OneCursorOp.word(oneCursor, true, ctx.eventData.position, oneCtx);
                }
                return false;
            });
            ctx.shouldReveal = false;
            ctx.shouldRevealHorizontal = false;
            return true;
        };
        Cursor.prototype._removeSecondaryCursors = function (ctx) {
            this.cursors.killSecondaryCursors();
            return true;
        };
        Cursor.prototype._cancelSelection = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.cancelSelection(oneCursor, oneCtx); });
        };
        Cursor.prototype._type = function (ctx) {
            var _this = this;
            var text = ctx.eventData.text;
            if (ctx.eventSource === 'keyboard') {
                // If this event is coming straight from the keyboard, look for electric characters and enter
                var i, len, chr;
                for (i = 0, len = text.length; i < len; i++) {
                    chr = text.charAt(i);
                    this.charactersTyped += chr;
                    // Here we must interpret each typed character individually, that's why we create a new context
                    ctx.hasExecutedCommands = this._createAndInterpretHandlerCtx(ctx.eventSource, ctx.eventData, function (charHandlerCtx) {
                        _this._invokeForAll(charHandlerCtx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.type(oneCursor, chr, oneCtx); }, false, false);
                        // The last typed character gets to win
                        ctx.cursorPositionChangeReason = charHandlerCtx.cursorPositionChangeReason;
                        ctx.shouldReveal = charHandlerCtx.shouldReveal;
                        ctx.shouldRevealVerticalInCenter = charHandlerCtx.shouldRevealVerticalInCenter;
                        ctx.shouldRevealHorizontal = charHandlerCtx.shouldRevealHorizontal;
                    }) || ctx.hasExecutedCommands;
                }
            }
            else {
                this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.actualType(oneCursor, text, false, oneCtx); });
            }
            return true;
        };
        Cursor.prototype._replacePreviousChar = function (ctx) {
            var text = ctx.eventData.text;
            var replaceCharCnt = ctx.eventData.replaceCharCnt;
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.replacePreviousChar(oneCursor, text, replaceCharCnt, oneCtx); });
        };
        Cursor.prototype._tab = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.tab(oneCursor, oneCtx); }, false, false);
        };
        Cursor.prototype._indent = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.indent(oneCursor, oneCtx); });
        };
        Cursor.prototype._outdent = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.outdent(oneCursor, oneCtx); });
        };
        Cursor.prototype._paste = function (ctx) {
            var distributedPaste = this._distributePasteToCursors(ctx);
            if (distributedPaste) {
                return this._invokeForAllSorted(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.paste(oneCursor, distributedPaste[cursorIndex], false, oneCtx); });
            }
            else {
                return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.paste(oneCursor, ctx.eventData.text, ctx.eventData.pasteOnNewLine, oneCtx); });
            }
        };
        Cursor.prototype._scrollUp = function (isPaged, ctx) {
            ctx.requestScrollDeltaLines = isPaged ? -this.configuration.editor.pageSize : -1;
            return true;
        };
        Cursor.prototype._scrollDown = function (isPaged, ctx) {
            ctx.requestScrollDeltaLines = isPaged ? this.configuration.editor.pageSize : 1;
            return true;
        };
        Cursor.prototype._distributePasteToCursors = function (ctx) {
            if (ctx.eventData.pasteOnNewLine) {
                return null;
            }
            var selections = this.cursors.getSelections();
            if (selections.length === 1) {
                return null;
            }
            for (var i = 0; i < selections.length; i++) {
                if (selections[i].startLineNumber !== selections[i].endLineNumber) {
                    return null;
                }
            }
            var pastePieces = ctx.eventData.text.split(/\r\n|\r|\n/);
            if (pastePieces.length !== selections.length) {
                return null;
            }
            return pastePieces;
        };
        Cursor.prototype._deleteLeft = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.deleteLeft(oneCursor, oneCtx); }, false, false);
        };
        Cursor.prototype._deleteWordLeft = function (whitespaceHeuristics, wordNavigationType, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.deleteWordLeft(oneCursor, whitespaceHeuristics, wordNavigationType, oneCtx); }, false, false);
        };
        Cursor.prototype._deleteRight = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.deleteRight(oneCursor, oneCtx); }, false, false);
        };
        Cursor.prototype._deleteWordRight = function (whitespaceHeuristics, wordNavigationType, ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.deleteWordRight(oneCursor, whitespaceHeuristics, wordNavigationType, oneCtx); }, false, false);
        };
        Cursor.prototype._deleteAllLeft = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.deleteAllLeft(oneCursor, oneCtx); }, false, false);
        };
        Cursor.prototype._deleteAllRight = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.deleteAllRight(oneCursor, oneCtx); }, false, false);
        };
        Cursor.prototype._cut = function (ctx) {
            var _this = this;
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) { return oneCursor_1.OneCursorOp.cut(oneCursor, _this.enableEmptySelectionClipboard, oneCtx); });
        };
        Cursor.prototype._undo = function (ctx) {
            ctx.cursorPositionChangeReason = 'undo';
            ctx.hasExecutedCommands = true;
            this._interpretCommandResult(this.model.undo());
            return true;
        };
        Cursor.prototype._cursorUndo = function (ctx) {
            if (this.cursorUndoStack.length === 0) {
                return false;
            }
            ctx.cursorPositionChangeReason = 'undo';
            ctx.isCursorUndo = true;
            this.cursors.restoreState(this.cursorUndoStack.pop());
            return true;
        };
        Cursor.prototype._redo = function (ctx) {
            ctx.cursorPositionChangeReason = 'redo';
            ctx.hasExecutedCommands = true;
            this._interpretCommandResult(this.model.redo());
            return true;
        };
        Cursor.prototype._externalExecuteCommand = function (ctx) {
            this.cursors.killSecondaryCursors();
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) {
                oneCtx.shouldPushStackElementBefore = true;
                oneCtx.shouldPushStackElementAfter = true;
                oneCtx.executeCommand = ctx.eventData;
                return false;
            });
        };
        Cursor.prototype._externalExecuteCommands = function (ctx) {
            return this._invokeForAll(ctx, function (cursorIndex, oneCursor, oneCtx) {
                oneCtx.shouldPushStackElementBefore = true;
                oneCtx.shouldPushStackElementAfter = true;
                oneCtx.executeCommand = ctx.eventData[cursorIndex];
                return false;
            });
        };
        return Cursor;
    }(eventEmitter_1.EventEmitter));
    exports.Cursor = Cursor;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/commonCodeEditor", ["require", "exports", 'vs/nls!vs/editor/common/commonCodeEditor', 'vs/base/common/actions', 'vs/base/common/errors', 'vs/base/common/eventEmitter', 'vs/base/common/lifecycle', 'vs/base/common/objects', 'vs/base/common/timer', 'vs/base/common/winjs.base', 'vs/editor/common/config/defaultConfig', 'vs/editor/common/controller/cursor', 'vs/editor/common/controller/cursorMoveHelper', 'vs/editor/common/core/editorState', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/core/selection', 'vs/editor/common/editorAction', 'vs/editor/common/editorCommon', 'vs/editor/common/viewModel/characterHardWrappingLineMapper', 'vs/editor/common/viewModel/splitLinesCollection', 'vs/editor/common/viewModel/viewModel'], function (require, exports, nls, actions_1, errors_1, eventEmitter_1, lifecycle_1, objects, timer, winjs_base_1, defaultConfig_1, cursor_1, cursorMoveHelper_1, editorState_1, position_1, range_1, selection_1, editorAction_1, editorCommon, characterHardWrappingLineMapper_1, splitLinesCollection_1, viewModel_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EDITOR_ID = 0;
    var CommonCodeEditor = (function (_super) {
        __extends(CommonCodeEditor, _super);
        function CommonCodeEditor(domElement, options, instantiationService, codeEditorService, keybindingService, telemetryService) {
            var _this = this;
            _super.call(this);
            this.domElement = domElement;
            this.id = (++EDITOR_ID);
            this._codeEditorService = codeEditorService;
            var timerEvent = timer.start(timer.Topic.EDITOR, 'CodeEditor.ctor');
            this._lifetimeDispose = [];
            this._keybindingService = keybindingService.createScoped(domElement);
            this._editorIdContextKey = this._keybindingService.createKey('editorId', this.getId());
            this._editorFocusContextKey = this._keybindingService.createKey(editorCommon.KEYBINDING_CONTEXT_EDITOR_FOCUS, undefined);
            this._editorTabMovesFocusKey = this._keybindingService.createKey(editorCommon.KEYBINDING_CONTEXT_EDITOR_TAB_MOVES_FOCUS, false);
            this._hasMultipleSelectionsKey = this._keybindingService.createKey(editorCommon.KEYBINDING_CONTEXT_EDITOR_HAS_MULTIPLE_SELECTIONS, false);
            this._hasNonEmptySelectionKey = this._keybindingService.createKey(editorCommon.KEYBINDING_CONTEXT_EDITOR_HAS_NON_EMPTY_SELECTION, false);
            this._langIdKey = this._keybindingService.createKey(editorCommon.KEYBINDING_CONTEXT_EDITOR_LANGUAGE_ID, undefined);
            // listeners that are kept during the whole editor lifetime
            this._decorationTypeKeysToIds = {};
            options = options || {};
            if (typeof options.ariaLabel === 'undefined') {
                options.ariaLabel = defaultConfig_1.DefaultConfig.editor.ariaLabel;
            }
            options.ariaLabel += this._ariaLabelAppendMessage();
            this._configuration = this._createConfiguration(options);
            if (this._configuration.editor.tabFocusMode) {
                this._editorTabMovesFocusKey.set(true);
            }
            this._lifetimeDispose.push(this._configuration.onDidChange(function (e) { return _this.emit(editorCommon.EventType.ConfigurationChanged, e); }));
            this.forcedWidgetFocusCount = 0;
            this._telemetryService = telemetryService;
            this._instantiationService = instantiationService.createChild({
                keybindingService: this._keybindingService
            });
            this._attachModel(null);
            // Create editor contributions
            this.contributions = {};
            timerEvent.stop();
            this._codeEditorService.addCodeEditor(this);
        }
        CommonCodeEditor.prototype.getId = function () {
            return this.getEditorType() + ':' + this.id;
        };
        CommonCodeEditor.prototype.getEditorType = function () {
            return editorCommon.EditorType.ICodeEditor;
        };
        CommonCodeEditor.prototype.destroy = function () {
            this.dispose();
        };
        CommonCodeEditor.prototype.dispose = function () {
            this._codeEditorService.removeCodeEditor(this);
            this._lifetimeDispose = lifecycle_1.disposeAll(this._lifetimeDispose);
            var contributionId;
            for (contributionId in this.contributions) {
                if (this.contributions.hasOwnProperty(contributionId)) {
                    this.contributions[contributionId].dispose();
                }
            }
            this.contributions = {};
            this._postDetachModelCleanup(this._detachModel());
            this._configuration.dispose();
            this._keybindingService.dispose();
            this.emit(editorCommon.EventType.Disposed, {});
            _super.prototype.dispose.call(this);
        };
        CommonCodeEditor.prototype.captureState = function () {
            var flags = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                flags[_i - 0] = arguments[_i];
            }
            return new editorState_1.EditorState(this, flags);
        };
        CommonCodeEditor.prototype._ariaLabelAppendMessage = function () {
            var keybindings = this._keybindingService.lookupKeybindings(editorCommon.SHOW_ACCESSIBILITY_HELP_ACTION_ID);
            if (keybindings.length > 0) {
                return nls.localize(0, null, this._keybindingService.getAriaLabelFor(keybindings[0]));
            }
            return '';
        };
        CommonCodeEditor.prototype.updateOptions = function (newOptions) {
            if (typeof newOptions.ariaLabel !== 'undefined') {
                newOptions.ariaLabel += this._ariaLabelAppendMessage();
            }
            this._configuration.updateOptions(newOptions);
            if (this._configuration.editor.tabFocusMode) {
                this._editorTabMovesFocusKey.set(true);
            }
            else {
                this._editorTabMovesFocusKey.reset();
            }
        };
        CommonCodeEditor.prototype.getConfiguration = function () {
            return this._configuration.editorClone;
        };
        CommonCodeEditor.prototype.getRawConfiguration = function () {
            return this._configuration.getRawOptions();
        };
        CommonCodeEditor.prototype.getValue = function (options) {
            if (options === void 0) { options = null; }
            if (this.model) {
                var preserveBOM = (options && options.preserveBOM) ? true : false;
                var eolPreference = editorCommon.EndOfLinePreference.TextDefined;
                if (options && options.lineEnding && options.lineEnding === '\n') {
                    eolPreference = editorCommon.EndOfLinePreference.LF;
                }
                else if (options && options.lineEnding && options.lineEnding === '\r\n') {
                    eolPreference = editorCommon.EndOfLinePreference.CRLF;
                }
                return this.model.getValue(eolPreference, preserveBOM);
            }
            return '';
        };
        CommonCodeEditor.prototype.setValue = function (newValue) {
            if (this.model) {
                this.model.setValue(newValue);
            }
        };
        CommonCodeEditor.prototype.getModel = function () {
            return this.model;
        };
        CommonCodeEditor.prototype.setModel = function (model) {
            if (model === void 0) { model = null; }
            if (this.model === model) {
                // Current model is the new model
                return;
            }
            var timerEvent = timer.start(timer.Topic.EDITOR, 'CodeEditor.setModel');
            var detachedModel = this._detachModel();
            this._attachModel(model);
            var oldModelUrl = null;
            var newModelUrl = null;
            if (detachedModel) {
                oldModelUrl = detachedModel.getAssociatedResource().toString();
            }
            if (model) {
                newModelUrl = model.getAssociatedResource().toString();
            }
            var e = {
                oldModelUrl: oldModelUrl,
                newModelUrl: newModelUrl
            };
            timerEvent.stop();
            this.emit(editorCommon.EventType.ModelChanged, e);
            this._postDetachModelCleanup(detachedModel);
        };
        CommonCodeEditor.prototype.getVisibleColumnFromPosition = function (rawPosition) {
            if (!this.model) {
                return rawPosition.column;
            }
            var position = this.model.validatePosition(rawPosition);
            var tabSize = this.model.getOptions().tabSize;
            return cursorMoveHelper_1.CursorMoveHelper.visibleColumnFromColumn(this.model, position.lineNumber, position.column, tabSize) + 1;
        };
        CommonCodeEditor.prototype.getPosition = function () {
            if (!this.cursor) {
                return null;
            }
            return this.cursor.getPosition().clone();
        };
        CommonCodeEditor.prototype.setPosition = function (position, reveal, revealVerticalInCenter, revealHorizontal) {
            if (reveal === void 0) { reveal = false; }
            if (revealVerticalInCenter === void 0) { revealVerticalInCenter = false; }
            if (revealHorizontal === void 0) { revealHorizontal = false; }
            if (!this.cursor) {
                return;
            }
            if (!position_1.Position.isIPosition(position)) {
                throw new Error('Invalid arguments');
            }
            this.cursor.setSelections('api', [{
                    selectionStartLineNumber: position.lineNumber,
                    selectionStartColumn: position.column,
                    positionLineNumber: position.lineNumber,
                    positionColumn: position.column
                }]);
            if (reveal) {
                this.revealPosition(position, revealVerticalInCenter, revealHorizontal);
            }
        };
        CommonCodeEditor.prototype._sendRevealRange = function (range, verticalType, revealHorizontal) {
            if (!this.model || !this.cursor) {
                return;
            }
            if (!range_1.Range.isIRange(range)) {
                throw new Error('Invalid arguments');
            }
            var validatedRange = this.model.validateRange(range);
            var revealRangeEvent = {
                range: validatedRange,
                viewRange: null,
                verticalType: verticalType,
                revealHorizontal: revealHorizontal
            };
            this.cursor.emit(editorCommon.EventType.CursorRevealRange, revealRangeEvent);
        };
        CommonCodeEditor.prototype.revealLine = function (lineNumber) {
            this._sendRevealRange({
                startLineNumber: lineNumber,
                startColumn: 1,
                endLineNumber: lineNumber,
                endColumn: 1
            }, editorCommon.VerticalRevealType.Simple, false);
        };
        CommonCodeEditor.prototype.revealLineInCenter = function (lineNumber) {
            this._sendRevealRange({
                startLineNumber: lineNumber,
                startColumn: 1,
                endLineNumber: lineNumber,
                endColumn: 1
            }, editorCommon.VerticalRevealType.Center, false);
        };
        CommonCodeEditor.prototype.revealLineInCenterIfOutsideViewport = function (lineNumber) {
            this._sendRevealRange({
                startLineNumber: lineNumber,
                startColumn: 1,
                endLineNumber: lineNumber,
                endColumn: 1
            }, editorCommon.VerticalRevealType.CenterIfOutsideViewport, false);
        };
        CommonCodeEditor.prototype.revealPosition = function (position, revealVerticalInCenter, revealHorizontal) {
            if (revealVerticalInCenter === void 0) { revealVerticalInCenter = false; }
            if (revealHorizontal === void 0) { revealHorizontal = false; }
            if (!position_1.Position.isIPosition(position)) {
                throw new Error('Invalid arguments');
            }
            this._sendRevealRange({
                startLineNumber: position.lineNumber,
                startColumn: position.column,
                endLineNumber: position.lineNumber,
                endColumn: position.column
            }, revealVerticalInCenter ? editorCommon.VerticalRevealType.Center : editorCommon.VerticalRevealType.Simple, revealHorizontal);
        };
        CommonCodeEditor.prototype.revealPositionInCenter = function (position) {
            if (!position_1.Position.isIPosition(position)) {
                throw new Error('Invalid arguments');
            }
            this._sendRevealRange({
                startLineNumber: position.lineNumber,
                startColumn: position.column,
                endLineNumber: position.lineNumber,
                endColumn: position.column
            }, editorCommon.VerticalRevealType.Center, true);
        };
        CommonCodeEditor.prototype.revealPositionInCenterIfOutsideViewport = function (position) {
            if (!position_1.Position.isIPosition(position)) {
                throw new Error('Invalid arguments');
            }
            this._sendRevealRange({
                startLineNumber: position.lineNumber,
                startColumn: position.column,
                endLineNumber: position.lineNumber,
                endColumn: position.column
            }, editorCommon.VerticalRevealType.CenterIfOutsideViewport, true);
        };
        CommonCodeEditor.prototype.getSelection = function () {
            if (!this.cursor) {
                return null;
            }
            return this.cursor.getSelection().clone();
        };
        CommonCodeEditor.prototype.getSelections = function () {
            if (!this.cursor) {
                return null;
            }
            var selections = this.cursor.getSelections();
            var result = [];
            for (var i = 0, len = selections.length; i < len; i++) {
                result[i] = selections[i].clone();
            }
            return result;
        };
        CommonCodeEditor.prototype.setSelection = function (something, reveal, revealVerticalInCenter, revealHorizontal) {
            if (reveal === void 0) { reveal = false; }
            if (revealVerticalInCenter === void 0) { revealVerticalInCenter = false; }
            if (revealHorizontal === void 0) { revealHorizontal = false; }
            var isSelection = selection_1.Selection.isISelection(something);
            var isRange = range_1.Range.isIRange(something);
            if (!isSelection && !isRange) {
                throw new Error('Invalid arguments');
            }
            if (isSelection) {
                this._setSelectionImpl(something, reveal, revealVerticalInCenter, revealHorizontal);
            }
            else if (isRange) {
                // act as if it was an IRange
                var selection = {
                    selectionStartLineNumber: something.startLineNumber,
                    selectionStartColumn: something.startColumn,
                    positionLineNumber: something.endLineNumber,
                    positionColumn: something.endColumn
                };
                this._setSelectionImpl(selection, reveal, revealVerticalInCenter, revealHorizontal);
            }
        };
        CommonCodeEditor.prototype._setSelectionImpl = function (sel, reveal, revealVerticalInCenter, revealHorizontal) {
            if (!this.cursor) {
                return;
            }
            var selection = selection_1.Selection.createSelection(sel.selectionStartLineNumber, sel.selectionStartColumn, sel.positionLineNumber, sel.positionColumn);
            this.cursor.setSelections('api', [selection]);
            if (reveal) {
                this.revealRange(selection, revealVerticalInCenter, revealHorizontal);
            }
        };
        CommonCodeEditor.prototype.revealLines = function (startLineNumber, endLineNumber) {
            this._sendRevealRange({
                startLineNumber: startLineNumber,
                startColumn: 1,
                endLineNumber: endLineNumber,
                endColumn: 1
            }, editorCommon.VerticalRevealType.Simple, false);
        };
        CommonCodeEditor.prototype.revealLinesInCenter = function (startLineNumber, endLineNumber) {
            this._sendRevealRange({
                startLineNumber: startLineNumber,
                startColumn: 1,
                endLineNumber: endLineNumber,
                endColumn: 1
            }, editorCommon.VerticalRevealType.Center, false);
        };
        CommonCodeEditor.prototype.revealLinesInCenterIfOutsideViewport = function (startLineNumber, endLineNumber) {
            this._sendRevealRange({
                startLineNumber: startLineNumber,
                startColumn: 1,
                endLineNumber: endLineNumber,
                endColumn: 1
            }, editorCommon.VerticalRevealType.CenterIfOutsideViewport, false);
        };
        CommonCodeEditor.prototype.revealRange = function (range, revealVerticalInCenter, revealHorizontal) {
            if (revealVerticalInCenter === void 0) { revealVerticalInCenter = false; }
            if (revealHorizontal === void 0) { revealHorizontal = true; }
            this._sendRevealRange(range, revealVerticalInCenter ? editorCommon.VerticalRevealType.Center : editorCommon.VerticalRevealType.Simple, revealHorizontal);
        };
        CommonCodeEditor.prototype.revealRangeInCenter = function (range) {
            this._sendRevealRange(range, editorCommon.VerticalRevealType.Center, true);
        };
        CommonCodeEditor.prototype.revealRangeInCenterIfOutsideViewport = function (range) {
            this._sendRevealRange(range, editorCommon.VerticalRevealType.CenterIfOutsideViewport, true);
        };
        CommonCodeEditor.prototype.setSelections = function (ranges) {
            if (!this.cursor) {
                return;
            }
            if (!ranges || ranges.length === 0) {
                throw new Error('Invalid arguments');
            }
            for (var i = 0, len = ranges.length; i < len; i++) {
                if (!selection_1.Selection.isISelection(ranges[i])) {
                    throw new Error('Invalid arguments');
                }
            }
            this.cursor.setSelections('api', ranges);
        };
        CommonCodeEditor.prototype.onVisible = function () {
        };
        CommonCodeEditor.prototype.onHide = function () {
        };
        CommonCodeEditor.prototype.beginForcedWidgetFocus = function () {
            this.forcedWidgetFocusCount++;
        };
        CommonCodeEditor.prototype.endForcedWidgetFocus = function () {
            this.forcedWidgetFocusCount--;
        };
        CommonCodeEditor.prototype.getContribution = function (id) {
            return this.contributions[id] || null;
        };
        CommonCodeEditor.prototype.addAction = function (descriptor) {
            var action = this._instantiationService.createInstance(editorAction_1.DynamicEditorAction, descriptor, this);
            this.contributions[action.getId()] = action;
        };
        CommonCodeEditor.prototype.getActions = function () {
            var result = [];
            var id;
            for (id in this.contributions) {
                if (this.contributions.hasOwnProperty(id)) {
                    var contribution = this.contributions[id];
                    // contribution instanceof IAction
                    if (actions_1.isAction(contribution)) {
                        result.push(contribution);
                    }
                }
            }
            return result;
        };
        CommonCodeEditor.prototype.getAction = function (id) {
            var contribution = this.contributions[id];
            if (contribution) {
                // contribution instanceof IAction
                if (actions_1.isAction(contribution)) {
                    return contribution;
                }
            }
            return null;
        };
        CommonCodeEditor.prototype.trigger = function (source, handlerId, payload) {
            var candidate = this.getAction(handlerId);
            if (candidate !== null) {
                if (candidate.enabled) {
                    this._telemetryService.publicLog('editorActionInvoked', { name: candidate.label, id: candidate.id });
                    winjs_base_1.TPromise.as(candidate.run()).done(null, errors_1.onUnexpectedError);
                }
            }
            else {
                // forward to handler dispatcher
                var r = this._configuration.handlerDispatcher.trigger(source, handlerId, payload);
                if (!r) {
                }
            }
        };
        CommonCodeEditor.prototype.executeCommand = function (source, command) {
            // forward to handler dispatcher
            return this._configuration.handlerDispatcher.trigger(source, editorCommon.Handler.ExecuteCommand, command);
        };
        CommonCodeEditor.prototype.executeEdits = function (source, edits) {
            var _this = this;
            if (!this.cursor) {
                // no view, no cursor
                return false;
            }
            if (this._configuration.editor.readOnly) {
                // read only editor => sorry!
                return false;
            }
            this.model.pushEditOperations(this.cursor.getSelections(), edits, function () {
                return _this.cursor.getSelections();
            });
            return true;
        };
        CommonCodeEditor.prototype.executeCommands = function (source, commands) {
            // forward to handler dispatcher
            return this._configuration.handlerDispatcher.trigger(source, editorCommon.Handler.ExecuteCommands, commands);
        };
        CommonCodeEditor.prototype.changeDecorations = function (callback) {
            if (!this.model) {
                //			console.warn('Cannot change decorations on editor that is not attached to a model');
                // callback will not be called
                return null;
            }
            return this.model.changeDecorations(callback, this.id);
        };
        CommonCodeEditor.prototype.getLineDecorations = function (lineNumber) {
            if (!this.model) {
                return null;
            }
            return this.model.getLineDecorations(lineNumber, this.id, this._configuration.editor.readOnly);
        };
        CommonCodeEditor.prototype.deltaDecorations = function (oldDecorations, newDecorations) {
            if (!this.model) {
                return [];
            }
            if (oldDecorations.length === 0 && newDecorations.length === 0) {
                return oldDecorations;
            }
            return this.model.deltaDecorations(oldDecorations, newDecorations, this.id);
        };
        CommonCodeEditor.prototype.setDecorations = function (decorationTypeKey, ranges) {
            var opts = this._codeEditorService.resolveDecorationType(decorationTypeKey);
            var oldDecorationIds = this._decorationTypeKeysToIds[decorationTypeKey] || [];
            this._decorationTypeKeysToIds[decorationTypeKey] = this.deltaDecorations(oldDecorationIds, ranges.map(function (r) {
                var decOpts;
                if (r.hoverMessage) {
                    // TODO@Alex: avoid objects.clone
                    decOpts = objects.clone(opts);
                    decOpts.htmlMessage = r.hoverMessage;
                }
                else {
                    decOpts = opts;
                }
                return {
                    range: r.range,
                    options: decOpts
                };
            }));
        };
        CommonCodeEditor.prototype.removeDecorations = function (decorationTypeKey) {
            if (this._decorationTypeKeysToIds.hasOwnProperty(decorationTypeKey)) {
                this.deltaDecorations(this._decorationTypeKeysToIds[decorationTypeKey], []);
                delete this._decorationTypeKeysToIds[decorationTypeKey];
            }
        };
        CommonCodeEditor.prototype.addTypingListener = function (character, callback) {
            var _this = this;
            if (!this.cursor) {
                return function () {
                    // no-op
                };
            }
            this.cursor.addTypingListener(character, callback);
            return function () {
                if (_this.cursor) {
                    _this.cursor.removeTypingListener(character, callback);
                }
            };
        };
        CommonCodeEditor.prototype.getLayoutInfo = function () {
            return this._configuration.editor.layoutInfo;
        };
        CommonCodeEditor.prototype._attachModel = function (model) {
            var _this = this;
            this.model = model ? model : null;
            this.listenersToRemove = [];
            this.viewModel = null;
            this.cursor = null;
            if (this.model) {
                this.domElement.setAttribute('data-mode-id', this.model.getMode().getId());
                this._langIdKey.set(this.model.getMode().getId());
                this.model.setStopLineTokenizationAfter(this._configuration.editor.stopLineTokenizationAfter);
                this._configuration.setIsDominatedByLongLines(this.model.isDominatedByLongLines(this._configuration.editor.longLineBoundary));
                this.model.onBeforeAttached();
                var hardWrappingLineMapperFactory = new characterHardWrappingLineMapper_1.CharacterHardWrappingLineMapperFactory(this._configuration.editor.wordWrapBreakBeforeCharacters, this._configuration.editor.wordWrapBreakAfterCharacters, this._configuration.editor.wordWrapBreakObtrusiveCharacters);
                var linesCollection = new splitLinesCollection_1.SplitLinesCollection(this.model, hardWrappingLineMapperFactory, this.model.getOptions().tabSize, this._configuration.editor.wrappingInfo.wrappingColumn, this._configuration.editor.typicalFullwidthCharacterWidth / this._configuration.editor.typicalHalfwidthCharacterWidth, editorCommon.wrappingIndentFromString(this._configuration.editor.wrappingIndent));
                this.viewModel = new viewModel_1.ViewModel(linesCollection, this.id, this._configuration, this.model, function () { return _this.getCenteredRangeInViewport(); });
                var viewModelHelper = {
                    viewModel: this.viewModel,
                    convertModelPositionToViewPosition: function (lineNumber, column) {
                        return _this.viewModel.convertModelPositionToViewPosition(lineNumber, column);
                    },
                    convertModelRangeToViewRange: function (modelRange) {
                        return _this.viewModel.convertModelRangeToViewRange(modelRange);
                    },
                    convertViewToModelPosition: function (lineNumber, column) {
                        return _this.viewModel.convertViewPositionToModelPosition(lineNumber, column);
                    },
                    convertViewSelectionToModelSelection: function (viewSelection) {
                        return _this.viewModel.convertViewSelectionToModelSelection(viewSelection);
                    },
                    validateViewPosition: function (viewLineNumber, viewColumn, modelPosition) {
                        return _this.viewModel.validateViewPosition(viewLineNumber, viewColumn, modelPosition);
                    },
                    validateViewRange: function (viewStartLineNumber, viewStartColumn, viewEndLineNumber, viewEndColumn, modelRange) {
                        return _this.viewModel.validateViewRange(viewStartLineNumber, viewStartColumn, viewEndLineNumber, viewEndColumn, modelRange);
                    }
                };
                this.cursor = new cursor_1.Cursor(this.id, this._configuration, this.model, viewModelHelper, this._enableEmptySelectionClipboard());
                this.viewModel.addEventSource(this.cursor);
                this._createView();
                this.listenersToRemove.push(this._getViewInternalEventBus().addBulkListener(function (events) {
                    for (var i = 0, len = events.length; i < len; i++) {
                        var eventType = events[i].getType();
                        var e = events[i].getData();
                        switch (eventType) {
                            case editorCommon.EventType.ViewFocusGained:
                                _this.emit(editorCommon.EventType.EditorTextFocus);
                                // In IE, the focus is not synchronous, so we give it a little help
                                _this.emit(editorCommon.EventType.EditorFocus, {});
                                break;
                            case 'scroll':
                                _this.emit('scroll', e);
                                break;
                            case 'scrollSize':
                                _this.emit('scrollSize', e);
                                break;
                            case editorCommon.EventType.ViewFocusLost:
                                _this.emit(editorCommon.EventType.EditorTextBlur);
                                break;
                            case editorCommon.EventType.ContextMenu:
                                _this.emit(editorCommon.EventType.ContextMenu, e);
                                break;
                            case editorCommon.EventType.MouseDown:
                                _this.emit(editorCommon.EventType.MouseDown, e);
                                break;
                            case editorCommon.EventType.MouseUp:
                                _this.emit(editorCommon.EventType.MouseUp, e);
                                break;
                            case editorCommon.EventType.KeyUp:
                                _this.emit(editorCommon.EventType.KeyUp, e);
                                break;
                            case editorCommon.EventType.MouseMove:
                                _this.emit(editorCommon.EventType.MouseMove, e);
                                break;
                            case editorCommon.EventType.MouseLeave:
                                _this.emit(editorCommon.EventType.MouseLeave, e);
                                break;
                            case editorCommon.EventType.KeyDown:
                                _this.emit(editorCommon.EventType.KeyDown, e);
                                break;
                            case editorCommon.EventType.ViewLayoutChanged:
                                _this.emit(editorCommon.EventType.EditorLayout, e);
                                break;
                            default:
                        }
                    }
                }));
                this.listenersToRemove.push(this.model.addBulkListener(function (events) {
                    for (var i = 0, len = events.length; i < len; i++) {
                        var eventType = events[i].getType();
                        var e = events[i].getData();
                        switch (eventType) {
                            case editorCommon.EventType.ModelDecorationsChanged:
                                _this.emit(editorCommon.EventType.ModelDecorationsChanged, e);
                                break;
                            case editorCommon.EventType.ModelModeChanged:
                                _this.domElement.setAttribute('data-mode-id', _this.model.getMode().getId());
                                _this._langIdKey.set(_this.model.getMode().getId());
                                _this.emit(editorCommon.EventType.ModelModeChanged, e);
                                break;
                            case editorCommon.EventType.ModelModeSupportChanged:
                                _this.emit(editorCommon.EventType.ModelModeSupportChanged, e);
                                break;
                            case editorCommon.EventType.ModelContentChanged:
                                // TODO@Alex
                                _this.emit(editorCommon.EventType.ModelContentChanged, e);
                                _this.emit('change', {});
                                break;
                            case editorCommon.EventType.ModelOptionsChanged:
                                _this.emit(editorCommon.EventType.ModelOptionsChanged, e);
                                break;
                            case editorCommon.EventType.ModelDispose:
                                // Someone might destroy the model from under the editor, so prevent any exceptions by setting a null model
                                _this.setModel(null);
                                break;
                            default:
                        }
                    }
                }));
                var _hasNonEmptySelection = function (e) {
                    var allSelections = [e.selection].concat(e.secondarySelections);
                    return allSelections.some(function (s) { return !s.isEmpty(); });
                };
                this.listenersToRemove.push(this.cursor.addBulkListener(function (events) {
                    var updateHasMultipleCursors = false, hasMultipleCursors = false, updateHasNonEmptySelection = false, hasNonEmptySelection = false;
                    for (var i = 0, len = events.length; i < len; i++) {
                        var eventType = events[i].getType();
                        var e = events[i].getData();
                        switch (eventType) {
                            case editorCommon.EventType.CursorPositionChanged:
                                var cursorPositionChangedEvent = e;
                                updateHasMultipleCursors = true;
                                hasMultipleCursors = (cursorPositionChangedEvent.secondaryPositions.length > 0);
                                _this.emit(editorCommon.EventType.CursorPositionChanged, e);
                                break;
                            case editorCommon.EventType.CursorSelectionChanged:
                                var cursorSelectionChangedEvent = e;
                                updateHasMultipleCursors = true;
                                hasMultipleCursors = (cursorSelectionChangedEvent.secondarySelections.length > 0);
                                updateHasNonEmptySelection = true;
                                hasNonEmptySelection = _hasNonEmptySelection(cursorSelectionChangedEvent);
                                _this.emit(editorCommon.EventType.CursorSelectionChanged, e);
                                break;
                            default:
                        }
                    }
                    if (updateHasMultipleCursors) {
                        if (hasMultipleCursors) {
                            _this._hasMultipleSelectionsKey.set(true);
                        }
                        else {
                            _this._hasMultipleSelectionsKey.reset();
                        }
                    }
                    if (updateHasNonEmptySelection) {
                        if (hasNonEmptySelection) {
                            _this._hasNonEmptySelectionKey.set(true);
                        }
                        else {
                            _this._hasNonEmptySelectionKey.reset();
                        }
                    }
                }));
            }
            else {
                this.hasView = false;
            }
        };
        CommonCodeEditor.prototype._postDetachModelCleanup = function (detachedModel) {
            if (detachedModel) {
                this._decorationTypeKeysToIds = {};
                detachedModel.removeAllDecorationsWithOwnerId(this.id);
            }
        };
        CommonCodeEditor.prototype._detachModel = function () {
            if (this.model) {
                this.model.onBeforeDetached();
            }
            this.hasView = false;
            this.listenersToRemove.forEach(function (element) {
                element();
            });
            this.listenersToRemove = [];
            if (this.cursor) {
                this.cursor.dispose();
                this.cursor = null;
            }
            if (this.viewModel) {
                this.viewModel.dispose();
                this.viewModel = null;
            }
            var result = this.model;
            this.model = null;
            this.domElement.removeAttribute('data-mode-id');
            return result;
        };
        return CommonCodeEditor;
    }(eventEmitter_1.EventEmitter));
    exports.CommonCodeEditor = CommonCodeEditor;
});

define("vs/nls!vs/editor/common/model/textModelWithTokens",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/common/model/textModelWithTokens", data); });
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/model/textModelWithTokens", ["require", "exports", 'vs/nls!vs/editor/common/model/textModelWithTokens', 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/lifecycle', 'vs/base/common/stopwatch', 'vs/base/common/timer', 'vs/base/common/winjs.base', 'vs/editor/common/config/defaultConfig', 'vs/editor/common/core/arrays', 'vs/editor/common/editorCommon', 'vs/editor/common/model/textModel', 'vs/editor/common/model/textModelWithTokensHelpers', 'vs/editor/common/model/tokenIterator', 'vs/editor/common/modes/nullMode', 'vs/editor/common/modes/supports', 'vs/editor/common/modes/supports/richEditBrackets'], function (require, exports, nls, async_1, errors_1, lifecycle_1, stopwatch_1, timer, winjs_base_1, defaultConfig_1, arrays_1, editorCommon, textModel_1, textModelWithTokensHelpers_1, tokenIterator_1, nullMode_1, supports_1, richEditBrackets_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var TokensInflatorMap = (function () {
        function TokensInflatorMap() {
            this._inflate = [''];
            this._deflate = { '': 0 };
        }
        return TokensInflatorMap;
    }());
    exports.TokensInflatorMap = TokensInflatorMap;
    var ModeToModelBinder = (function () {
        function ModeToModelBinder(modePromise, model) {
            var _this = this;
            this._modePromise = modePromise;
            // Create an external mode promise that fires after the mode is set to the model
            this._externalModePromise = new winjs_base_1.TPromise(function (c, e, p) {
                _this._externalModePromise_c = c;
                _this._externalModePromise_e = e;
            }, function () {
                // this promise cannot be canceled
            });
            this._model = model;
            this._isDisposed = false;
            // Ensure asynchronicity
            winjs_base_1.TPromise.timeout(0).then(function () {
                return _this._modePromise;
            }).then(function (mode) {
                if (_this._isDisposed) {
                    _this._externalModePromise_c(false);
                    return;
                }
                var model = _this._model;
                _this.dispose();
                model.setMode(mode);
                model._warmUpTokens();
                _this._externalModePromise_c(true);
            }).done(null, function (err) {
                _this._externalModePromise_e(err);
                errors_1.onUnexpectedError(err);
            });
        }
        ModeToModelBinder.prototype.getModePromise = function () {
            return this._externalModePromise;
        };
        ModeToModelBinder.prototype.dispose = function () {
            this._modePromise = null;
            this._model = null;
            this._isDisposed = true;
        };
        return ModeToModelBinder;
    }());
    var FullModelRetokenizer = (function () {
        function FullModelRetokenizer(retokenizePromise, model) {
            var _this = this;
            this._retokenizePromise = retokenizePromise;
            this._model = model;
            this._isDisposed = false;
            this.isFulfilled = false;
            // Ensure asynchronicity
            winjs_base_1.TPromise.timeout(0).then(function () {
                return _this._retokenizePromise;
            }).then(function () {
                if (_this._isDisposed) {
                    return;
                }
                _this.isFulfilled = true;
                _this._model.onRetokenizerFulfilled();
            }).done(null, errors_1.onUnexpectedError);
        }
        FullModelRetokenizer.prototype.getRange = function () {
            return null;
        };
        FullModelRetokenizer.prototype.dispose = function () {
            this._retokenizePromise = null;
            this._model = null;
            this._isDisposed = true;
        };
        return FullModelRetokenizer;
    }());
    exports.FullModelRetokenizer = FullModelRetokenizer;
    var LineContext = (function () {
        function LineContext(topLevelMode, line) {
            this.modeTransitions = line.getModeTransitions().toArray(topLevelMode);
            this._text = line.text;
            this._lineTokens = line.getTokens();
        }
        LineContext.prototype.getLineContent = function () {
            return this._text;
        };
        LineContext.prototype.getTokenCount = function () {
            return this._lineTokens.getTokenCount();
        };
        LineContext.prototype.getTokenStartIndex = function (tokenIndex) {
            return this._lineTokens.getTokenStartIndex(tokenIndex);
        };
        LineContext.prototype.getTokenEndIndex = function (tokenIndex) {
            return this._lineTokens.getTokenEndIndex(tokenIndex, this._text.length);
        };
        LineContext.prototype.getTokenType = function (tokenIndex) {
            return this._lineTokens.getTokenType(tokenIndex);
        };
        LineContext.prototype.getTokenText = function (tokenIndex) {
            var startIndex = this._lineTokens.getTokenStartIndex(tokenIndex);
            var endIndex = this._lineTokens.getTokenEndIndex(tokenIndex, this._text.length);
            return this._text.substring(startIndex, endIndex);
        };
        LineContext.prototype.findIndexOfOffset = function (offset) {
            return this._lineTokens.findIndexOfOffset(offset);
        };
        return LineContext;
    }());
    var TextModelWithTokens = (function (_super) {
        __extends(TextModelWithTokens, _super);
        function TextModelWithTokens(allowedEventTypes, rawText, shouldAutoTokenize, modeOrPromise) {
            var _this = this;
            allowedEventTypes.push(editorCommon.EventType.ModelTokensChanged);
            allowedEventTypes.push(editorCommon.EventType.ModelModeChanged);
            allowedEventTypes.push(editorCommon.EventType.ModelModeSupportChanged);
            _super.call(this, allowedEventTypes, rawText);
            this._shouldAutoTokenize = shouldAutoTokenize;
            this._shouldSimplifyMode = (rawText.length > TextModelWithTokens.MODEL_SYNC_LIMIT);
            this._shouldDenyMode = (rawText.length > TextModelWithTokens.MODEL_TOKENIZATION_LIMIT);
            this._stopLineTokenizationAfter = defaultConfig_1.DefaultConfig.editor.stopLineTokenizationAfter;
            if (!modeOrPromise) {
                this._mode = new nullMode_1.NullMode();
            }
            else if (winjs_base_1.TPromise.is(modeOrPromise)) {
                // TODO@Alex: To avoid mode id changes, we check if this promise is resolved
                var promiseValue = modeOrPromise._value;
                if (promiseValue && typeof promiseValue.getId === 'function') {
                    // The promise is already resolved
                    this._mode = this._massageMode(promiseValue);
                    this._resetModeListener(this._mode);
                }
                else {
                    var modePromise = modeOrPromise;
                    this._modeToModelBinder = new ModeToModelBinder(modePromise, this);
                    this._mode = new nullMode_1.NullMode();
                }
            }
            else {
                this._mode = this._massageMode(modeOrPromise);
                this._resetModeListener(this._mode);
            }
            this._revalidateTokensTimeout = -1;
            this._scheduleRetokenizeNow = new async_1.RunOnceScheduler(function () { return _this._retokenizeNow(); }, 200);
            this._retokenizers = [];
            this._resetTokenizationState();
        }
        TextModelWithTokens.prototype.dispose = function () {
            if (this._modeToModelBinder) {
                this._modeToModelBinder.dispose();
                this._modeToModelBinder = null;
            }
            this._resetModeListener(null);
            this._clearTimers();
            this._mode = null;
            this._lastState = null;
            this._tokensInflatorMap = null;
            this._retokenizers = lifecycle_1.disposeAll(this._retokenizers);
            this._scheduleRetokenizeNow.dispose();
            _super.prototype.dispose.call(this);
        };
        TextModelWithTokens.prototype.isTooLargeForHavingAMode = function () {
            return this._shouldDenyMode;
        };
        TextModelWithTokens.prototype.isTooLargeForHavingARichMode = function () {
            return this._shouldSimplifyMode;
        };
        TextModelWithTokens.prototype._massageMode = function (mode) {
            if (this.isTooLargeForHavingAMode()) {
                return new nullMode_1.NullMode();
            }
            if (this.isTooLargeForHavingARichMode()) {
                return mode.toSimplifiedMode();
            }
            return mode;
        };
        TextModelWithTokens.prototype.whenModeIsReady = function () {
            var _this = this;
            if (this._modeToModelBinder) {
                // Still waiting for some mode to load
                return this._modeToModelBinder.getModePromise().then(function () { return _this._mode; });
            }
            return winjs_base_1.TPromise.as(this._mode);
        };
        TextModelWithTokens.prototype.onRetokenizerFulfilled = function () {
            this._scheduleRetokenizeNow.schedule();
        };
        TextModelWithTokens.prototype._retokenizeNow = function () {
            var fulfilled = this._retokenizers.filter(function (r) { return r.isFulfilled; });
            this._retokenizers = this._retokenizers.filter(function (r) { return !r.isFulfilled; });
            var hasFullModel = false;
            for (var i = 0; i < fulfilled.length; i++) {
                if (!fulfilled[i].getRange()) {
                    hasFullModel = true;
                }
            }
            if (hasFullModel) {
                // Just invalidate all the lines
                for (var i = 0, len = this._lines.length; i < len; i++) {
                    this._lines[i].isInvalid = true;
                }
                this._invalidLineStartIndex = 0;
            }
            else {
                var minLineNumber = Number.MAX_VALUE;
                for (var i = 0; i < fulfilled.length; i++) {
                    var range = fulfilled[i].getRange();
                    minLineNumber = Math.min(minLineNumber, range.startLineNumber);
                    for (var lineNumber = range.startLineNumber; lineNumber <= range.endLineNumber; lineNumber++) {
                        this._lines[lineNumber - 1].isInvalid = true;
                    }
                }
                if (minLineNumber - 1 < this._invalidLineStartIndex) {
                    if (this._invalidLineStartIndex < this._lines.length) {
                        this._lines[this._invalidLineStartIndex].isInvalid = true;
                    }
                    this._invalidLineStartIndex = minLineNumber - 1;
                }
            }
            this._beginBackgroundTokenization();
            for (var i = 0; i < fulfilled.length; i++) {
                fulfilled[i].dispose();
            }
        };
        TextModelWithTokens.prototype._createRetokenizer = function (retokenizePromise, lineNumber) {
            return new FullModelRetokenizer(retokenizePromise, this);
        };
        TextModelWithTokens.prototype._resetValue = function (e, newValue) {
            _super.prototype._resetValue.call(this, e, newValue);
            // Cancel tokenization, clear all tokens and begin tokenizing
            this._resetTokenizationState();
        };
        TextModelWithTokens.prototype._resetMode = function (e, newMode) {
            // Cancel tokenization, clear all tokens and begin tokenizing
            this._mode = newMode;
            this._resetModeListener(newMode);
            this._resetTokenizationState();
            this.emitModelTokensChangedEvent(1, this.getLineCount());
        };
        TextModelWithTokens.prototype._resetModeListener = function (newMode) {
            var _this = this;
            if (this._modeListener) {
                this._modeListener.dispose();
                this._modeListener = null;
            }
            if (newMode && typeof newMode.addSupportChangedListener === 'function') {
                this._modeListener = newMode.addSupportChangedListener(function (e) { return _this._onModeSupportChanged(e); });
            }
        };
        TextModelWithTokens.prototype._onModeSupportChanged = function (e) {
            this._emitModelModeSupportChangedEvent(e);
            if (e.tokenizationSupport) {
                this._resetTokenizationState();
                this.emitModelTokensChangedEvent(1, this.getLineCount());
            }
        };
        TextModelWithTokens.prototype._resetTokenizationState = function () {
            this._retokenizers = lifecycle_1.disposeAll(this._retokenizers);
            this._scheduleRetokenizeNow.cancel();
            this._clearTimers();
            for (var i = 0; i < this._lines.length; i++) {
                this._lines[i].setState(null);
            }
            this._initializeTokenizationState();
            this._tokenizationElapsedTime = 0;
            this._tokenizationTotalCharacters = 1;
        };
        TextModelWithTokens.prototype._clearTimers = function () {
            if (this._revalidateTokensTimeout !== -1) {
                clearTimeout(this._revalidateTokensTimeout);
                this._revalidateTokensTimeout = -1;
            }
        };
        TextModelWithTokens.prototype._initializeTokenizationState = function () {
            // Initialize tokenization states
            var initialState = null;
            if (this._mode.tokenizationSupport) {
                try {
                    initialState = this._mode.tokenizationSupport.getInitialState();
                }
                catch (e) {
                    e.friendlyMessage = TextModelWithTokens.MODE_TOKENIZATION_FAILED_MSG;
                    errors_1.onUnexpectedError(e);
                    this._mode = new nullMode_1.NullMode();
                }
            }
            if (!initialState) {
                initialState = new nullMode_1.NullState(this._mode, null);
            }
            this._lines[0].setState(initialState);
            this._lastState = null;
            this._tokensInflatorMap = new TokensInflatorMap();
            this._invalidLineStartIndex = 0;
            this._beginBackgroundTokenization();
        };
        TextModelWithTokens.prototype.setStopLineTokenizationAfter = function (stopLineTokenizationAfter) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.setStopLineTokenizationAfter: Model is disposed');
            }
            this._stopLineTokenizationAfter = stopLineTokenizationAfter;
        };
        TextModelWithTokens.prototype.getLineTokens = function (lineNumber, inaccurateTokensAcceptable) {
            if (inaccurateTokensAcceptable === void 0) { inaccurateTokensAcceptable = false; }
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.getLineTokens: Model is disposed');
            }
            if (lineNumber < 1 || lineNumber > this.getLineCount()) {
                throw new Error('Illegal value ' + lineNumber + ' for `lineNumber`');
            }
            if (!inaccurateTokensAcceptable) {
                this._updateTokensUntilLine(lineNumber, true);
            }
            return this._lines[lineNumber - 1].getTokens();
        };
        TextModelWithTokens.prototype.getLineContext = function (lineNumber) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.getLineContext: Model is disposed');
            }
            if (lineNumber < 1 || lineNumber > this.getLineCount()) {
                throw new Error('Illegal value ' + lineNumber + ' for `lineNumber`');
            }
            this._updateTokensUntilLine(lineNumber, true);
            return new LineContext(this._mode, this._lines[lineNumber - 1]);
        };
        TextModelWithTokens.prototype._getInternalTokens = function (lineNumber) {
            this._updateTokensUntilLine(lineNumber, true);
            return this._lines[lineNumber - 1].getTokens();
        };
        TextModelWithTokens.prototype.setValue = function (value, newModeOrPromise) {
            if (newModeOrPromise === void 0) { newModeOrPromise = null; }
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.setValue: Model is disposed');
            }
            var rawText = null;
            if (value !== null) {
                rawText = textModel_1.TextModel.toRawText(value, {
                    tabSize: this._options.tabSize,
                    insertSpaces: this._options.insertSpaces,
                    detectIndentation: false,
                    defaultEOL: this._options.defaultEOL
                });
            }
            this.setValueFromRawText(rawText, newModeOrPromise);
        };
        TextModelWithTokens.prototype.setValueFromRawText = function (value, newModeOrPromise) {
            if (newModeOrPromise === void 0) { newModeOrPromise = null; }
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.setValueFromRawText: Model is disposed');
            }
            if (value !== null) {
                _super.prototype.setValueFromRawText.call(this, value);
            }
            if (newModeOrPromise) {
                if (this._modeToModelBinder) {
                    this._modeToModelBinder.dispose();
                    this._modeToModelBinder = null;
                }
                if (winjs_base_1.TPromise.is(newModeOrPromise)) {
                    this._modeToModelBinder = new ModeToModelBinder(newModeOrPromise, this);
                }
                else {
                    var actualNewMode = this._massageMode(newModeOrPromise);
                    if (this._mode !== actualNewMode) {
                        var e2 = {
                            oldMode: this._mode,
                            newMode: actualNewMode
                        };
                        this._resetMode(e2, actualNewMode);
                        this._emitModelModeChangedEvent(e2);
                    }
                }
            }
        };
        TextModelWithTokens.prototype.getMode = function () {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.getMode: Model is disposed');
            }
            return this._mode;
        };
        TextModelWithTokens.prototype.setMode = function (newModeOrPromise) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.setMode: Model is disposed');
            }
            if (!newModeOrPromise) {
                // There's nothing to do
                return;
            }
            this.setValueFromRawText(null, newModeOrPromise);
        };
        TextModelWithTokens.prototype.getModeAtPosition = function (_lineNumber, _column) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.getModeAtPosition: Model is disposed');
            }
            var validPosition = this.validatePosition({
                lineNumber: _lineNumber,
                column: _column
            });
            var lineNumber = validPosition.lineNumber;
            var column = validPosition.column;
            if (column === 1) {
                return this.getStateBeforeLine(lineNumber).getMode();
            }
            else if (column === this.getLineMaxColumn(lineNumber)) {
                return this.getStateAfterLine(lineNumber).getMode();
            }
            else {
                var modeTransitions = this._getLineModeTransitions(lineNumber);
                var modeTransitionIndex = arrays_1.Arrays.findIndexInSegmentsArray(modeTransitions, column - 1);
                return modeTransitions[modeTransitionIndex].mode;
            }
        };
        TextModelWithTokens.prototype._invalidateLine = function (lineIndex) {
            this._lines[lineIndex].isInvalid = true;
            if (lineIndex < this._invalidLineStartIndex) {
                if (this._invalidLineStartIndex < this._lines.length) {
                    this._lines[this._invalidLineStartIndex].isInvalid = true;
                }
                this._invalidLineStartIndex = lineIndex;
                this._beginBackgroundTokenization();
            }
        };
        TextModelWithTokens.prototype._updateLineTokens = function (lineIndex, map, topLevelMode, r) {
            this._lines[lineIndex].setTokens(map, r.tokens, topLevelMode, r.modeTransitions);
        };
        TextModelWithTokens.prototype._beginBackgroundTokenization = function () {
            var _this = this;
            if (this._shouldAutoTokenize && this._revalidateTokensTimeout === -1) {
                this._revalidateTokensTimeout = setTimeout(function () {
                    _this._revalidateTokensTimeout = -1;
                    _this._revalidateTokensNow();
                }, 0);
            }
        };
        TextModelWithTokens.prototype._warmUpTokens = function () {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens._warmUpTokens: Model is disposed');
            }
            // Warm up first 100 lines (if it takes less than 50ms)
            var maxLineNumber = Math.min(100, this.getLineCount());
            var toLineNumber = maxLineNumber;
            for (var lineNumber = 1; lineNumber <= maxLineNumber; lineNumber++) {
                var text = this._lines[lineNumber - 1].text;
                if (text.length >= 200) {
                    // This line is over 200 chars long, so warm up without it
                    toLineNumber = lineNumber - 1;
                    break;
                }
            }
            this._revalidateTokensNow(toLineNumber);
        };
        TextModelWithTokens.prototype._revalidateTokensNow = function (toLineNumber) {
            if (toLineNumber === void 0) { toLineNumber = this._invalidLineStartIndex + 1000000; }
            var t1 = timer.start(timer.Topic.EDITOR, 'backgroundTokenization');
            toLineNumber = Math.min(this._lines.length, toLineNumber);
            var MAX_ALLOWED_TIME = 20, fromLineNumber = this._invalidLineStartIndex + 1, tokenizedChars = 0, currentCharsToTokenize = 0, currentEstimatedTimeToTokenize = 0, stopLineTokenizationAfter = this._stopLineTokenizationAfter, sw = stopwatch_1.StopWatch.create(false), elapsedTime;
            // Tokenize at most 1000 lines. Estimate the tokenization speed per character and stop when:
            // - MAX_ALLOWED_TIME is reached
            // - tokenizing the next line would go above MAX_ALLOWED_TIME
            for (var lineNumber = fromLineNumber; lineNumber <= toLineNumber; lineNumber++) {
                elapsedTime = sw.elapsed();
                if (elapsedTime > MAX_ALLOWED_TIME) {
                    // Stop if MAX_ALLOWED_TIME is reached
                    toLineNumber = lineNumber - 1;
                    break;
                }
                // Compute how many characters will be tokenized for this line
                currentCharsToTokenize = this._lines[lineNumber - 1].text.length;
                if (stopLineTokenizationAfter !== -1 && currentCharsToTokenize > stopLineTokenizationAfter) {
                    currentCharsToTokenize = stopLineTokenizationAfter;
                }
                if (tokenizedChars > 0) {
                    // If we have enough history, estimate how long tokenizing this line would take
                    currentEstimatedTimeToTokenize = (elapsedTime / tokenizedChars) * currentCharsToTokenize;
                    if (elapsedTime + currentEstimatedTimeToTokenize > MAX_ALLOWED_TIME) {
                        // Tokenizing this line will go above MAX_ALLOWED_TIME
                        toLineNumber = lineNumber - 1;
                        break;
                    }
                }
                this._updateTokensUntilLine(lineNumber, false);
                tokenizedChars += currentCharsToTokenize;
            }
            elapsedTime = sw.elapsed();
            //		console.log('TOKENIZED LOCAL (' + this._mode.getId() + ') ' + tokenizedChars + '\t in \t' + elapsedTime + '\t speed \t' + tokenizedChars/elapsedTime);
            //		console.log('TOKENIZED GLOBAL(' + this._mode.getId() + ') ' + this._tokenizationTotalCharacters + '\t in*\t' + this._tokenizationElapsedTime + '\t speed*\t' + this._tokenizationTotalCharacters/this._tokenizationElapsedTime);
            var t2 = timer.start(timer.Topic.EDITOR, '**speed: ' + this._tokenizationTotalCharacters / this._tokenizationElapsedTime);
            t2.stop();
            if (fromLineNumber <= toLineNumber) {
                this.emitModelTokensChangedEvent(fromLineNumber, toLineNumber);
            }
            if (this._invalidLineStartIndex < this._lines.length) {
                this._beginBackgroundTokenization();
            }
            t1.stop();
        };
        TextModelWithTokens.prototype.getStateBeforeLine = function (lineNumber) {
            this._updateTokensUntilLine(lineNumber - 1, true);
            return this._lines[lineNumber - 1].getState();
        };
        TextModelWithTokens.prototype.getStateAfterLine = function (lineNumber) {
            this._updateTokensUntilLine(lineNumber, true);
            return lineNumber < this._lines.length ? this._lines[lineNumber].getState() : this._lastState;
        };
        TextModelWithTokens.prototype._getLineModeTransitions = function (lineNumber) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens._getLineModeTransitions: Model is disposed');
            }
            if (lineNumber < 1 || lineNumber > this.getLineCount()) {
                throw new Error('Illegal value ' + lineNumber + ' for `lineNumber`');
            }
            this._updateTokensUntilLine(lineNumber, true);
            return this._lines[lineNumber - 1].getModeTransitions().toArray(this._mode);
        };
        TextModelWithTokens.prototype._updateTokensUntilLine = function (lineNumber, emitEvents) {
            var linesLength = this._lines.length;
            var endLineIndex = lineNumber - 1;
            var stopLineTokenizationAfter = this._stopLineTokenizationAfter;
            if (stopLineTokenizationAfter === -1) {
                stopLineTokenizationAfter = 1000000000; // 1 billion, if a line is so long, you have other trouble :).
            }
            var sw = stopwatch_1.StopWatch.create(false);
            var tokenizedCharacters = 0;
            var fromLineNumber = this._invalidLineStartIndex + 1, toLineNumber = lineNumber;
            // Validate all states up to and including endLineIndex
            for (var lineIndex = this._invalidLineStartIndex; lineIndex <= endLineIndex; lineIndex++) {
                var endStateIndex = lineIndex + 1;
                var r = null;
                var text = this._lines[lineIndex].text;
                if (this._mode.tokenizationSupport) {
                    try {
                        // Tokenize only the first X characters
                        r = this._mode.tokenizationSupport.tokenize(this._lines[lineIndex].text, this._lines[lineIndex].getState(), 0, stopLineTokenizationAfter);
                        tokenizedCharacters = r ? r.actualStopOffset : this._lines[lineIndex].text.length;
                    }
                    catch (e) {
                        e.friendlyMessage = TextModelWithTokens.MODE_TOKENIZATION_FAILED_MSG;
                        errors_1.onUnexpectedError(e);
                    }
                    if (r && r.retokenize) {
                        this._retokenizers.push(this._createRetokenizer(r.retokenize, lineIndex + 1));
                    }
                    if (r && r.tokens && r.tokens.length > 0) {
                        // Cannot have a stop offset before the last token
                        r.actualStopOffset = Math.max(r.actualStopOffset, r.tokens[r.tokens.length - 1].startIndex + 1);
                    }
                    if (r && r.actualStopOffset < text.length) {
                        // Treat the rest of the line (if above limit) as one default token
                        r.tokens.push({
                            startIndex: r.actualStopOffset,
                            type: ''
                        });
                        // Use as end state the starting state
                        r.endState = this._lines[lineIndex].getState();
                    }
                }
                if (!r) {
                    r = nullMode_1.nullTokenize(this._mode, text, this._lines[lineIndex].getState());
                }
                if (!r.modeTransitions) {
                    r.modeTransitions = [];
                }
                if (r.modeTransitions.length === 0) {
                    // Make sure there is at least the transition to the top-most mode
                    r.modeTransitions.push({
                        startIndex: 0,
                        mode: this._mode
                    });
                }
                this._updateLineTokens(lineIndex, this._tokensInflatorMap, this._mode, r);
                if (this._lines[lineIndex].isInvalid) {
                    this._lines[lineIndex].isInvalid = false;
                }
                if (endStateIndex < linesLength) {
                    if (this._lines[endStateIndex].getState() !== null && r.endState.equals(this._lines[endStateIndex].getState())) {
                        // The end state of this line remains the same
                        var nextInvalidLineIndex = lineIndex + 1;
                        while (nextInvalidLineIndex < linesLength) {
                            if (this._lines[nextInvalidLineIndex].isInvalid) {
                                break;
                            }
                            if (nextInvalidLineIndex + 1 < linesLength) {
                                if (this._lines[nextInvalidLineIndex + 1].getState() === null) {
                                    break;
                                }
                            }
                            else {
                                if (this._lastState === null) {
                                    break;
                                }
                            }
                            nextInvalidLineIndex++;
                        }
                        this._invalidLineStartIndex = Math.max(this._invalidLineStartIndex, nextInvalidLineIndex);
                        lineIndex = nextInvalidLineIndex - 1; // -1 because the outer loop increments it
                    }
                    else {
                        this._lines[endStateIndex].setState(r.endState);
                    }
                }
                else {
                    this._lastState = r.endState;
                }
            }
            this._invalidLineStartIndex = Math.max(this._invalidLineStartIndex, endLineIndex + 1);
            this._tokenizationElapsedTime += sw.elapsed();
            this._tokenizationTotalCharacters += tokenizedCharacters;
            if (emitEvents && fromLineNumber <= toLineNumber) {
                this.emitModelTokensChangedEvent(fromLineNumber, toLineNumber);
            }
        };
        TextModelWithTokens.prototype.emitModelTokensChangedEvent = function (fromLineNumber, toLineNumber) {
            var e = {
                fromLineNumber: fromLineNumber,
                toLineNumber: toLineNumber
            };
            if (!this._isDisposing) {
                this.emit(editorCommon.EventType.ModelTokensChanged, e);
            }
        };
        TextModelWithTokens.prototype._emitModelModeChangedEvent = function (e) {
            if (!this._isDisposing) {
                this.emit(editorCommon.EventType.ModelModeChanged, e);
            }
        };
        TextModelWithTokens.prototype._emitModelModeSupportChangedEvent = function (e) {
            if (!this._isDisposing) {
                this.emit(editorCommon.EventType.ModelModeSupportChanged, e);
            }
        };
        // Having tokens allows implementing additional helper methods
        TextModelWithTokens.prototype._lineIsTokenized = function (lineNumber) {
            return this._invalidLineStartIndex > lineNumber - 1;
        };
        TextModelWithTokens.prototype._getWordDefinition = function () {
            return textModelWithTokensHelpers_1.WordHelper.massageWordDefinitionOf(this._mode);
        };
        TextModelWithTokens.prototype.getWordAtPosition = function (position) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.getWordAtPosition: Model is disposed');
            }
            return textModelWithTokensHelpers_1.WordHelper.getWordAtPosition(this, this.validatePosition(position));
        };
        TextModelWithTokens.prototype.getWordUntilPosition = function (position) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.getWordUntilPosition: Model is disposed');
            }
            var wordAtPosition = this.getWordAtPosition(position);
            if (!wordAtPosition) {
                return {
                    word: '',
                    startColumn: position.column,
                    endColumn: position.column
                };
            }
            return {
                word: wordAtPosition.word.substr(0, position.column - wordAtPosition.startColumn),
                startColumn: wordAtPosition.startColumn,
                endColumn: position.column
            };
        };
        TextModelWithTokens.prototype.getWords = function (lineNumber) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.getWords: Model is disposed');
            }
            if (lineNumber < 1 || lineNumber > this.getLineCount()) {
                throw new Error('Illegal value ' + lineNumber + ' for `lineNumber`');
            }
            return textModelWithTokensHelpers_1.WordHelper.getWords(this, this.validateLineNumber(lineNumber));
        };
        TextModelWithTokens.prototype.tokenIterator = function (position, callback) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.tokenIterator: Model is disposed');
            }
            var iter = new tokenIterator_1.TokenIterator(this, this.validatePosition(position));
            var result = callback(iter);
            iter._invalidate();
            return result;
        };
        TextModelWithTokens.prototype.findMatchingBracketUp = function (bracket, _position) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.findMatchingBracketUp: Model is disposed');
            }
            var position = this.validatePosition(_position);
            var modeTransitions = this._lines[position.lineNumber - 1].getModeTransitions().toArray(this._mode);
            var currentModeIndex = arrays_1.Arrays.findIndexInSegmentsArray(modeTransitions, position.column - 1);
            var currentMode = modeTransitions[currentModeIndex];
            var currentModeBrackets = currentMode.mode.richEditSupport ? currentMode.mode.richEditSupport.brackets : null;
            if (!currentModeBrackets) {
                return null;
            }
            var data = currentModeBrackets.textIsBracket[bracket];
            if (!data) {
                return null;
            }
            return this._findMatchingBracketUp(data, position);
        };
        TextModelWithTokens.prototype.matchBracket = function (position, inaccurateResultAcceptable) {
            if (inaccurateResultAcceptable === void 0) { inaccurateResultAcceptable = false; }
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.matchBracket: Model is disposed');
            }
            return this._matchBracket(this.validatePosition(position));
        };
        TextModelWithTokens.prototype._matchBracket = function (position) {
            var tokensMap = this._tokensInflatorMap;
            var lineNumber = position.lineNumber;
            var lineText = this._lines[lineNumber - 1].text;
            var lineTokens = this._lines[lineNumber - 1].getTokens();
            var tokens = lineTokens.getBinaryEncodedTokens();
            var currentTokenIndex = lineTokens.findIndexOfOffset(position.column - 1);
            var currentTokenStart = getStartIndex(tokens[currentTokenIndex]);
            var modeTransitions = this._lines[lineNumber - 1].getModeTransitions().toArray(this._mode);
            var currentModeIndex = arrays_1.Arrays.findIndexInSegmentsArray(modeTransitions, position.column - 1);
            var currentMode = modeTransitions[currentModeIndex];
            var currentModeBrackets = currentMode.mode.richEditSupport ? currentMode.mode.richEditSupport.brackets : null;
            // If position is in between two tokens, try first looking in the previous token
            if (currentTokenIndex > 0 && currentTokenStart === position.column - 1) {
                var prevTokenIndex = currentTokenIndex - 1;
                var prevTokenType = getType(tokensMap, tokens[prevTokenIndex]);
                // check that previous token is not to be ignored
                if (!supports_1.ignoreBracketsInToken(prevTokenType)) {
                    var prevTokenStart = getStartIndex(tokens[prevTokenIndex]);
                    var prevMode = currentMode;
                    var prevModeBrackets = currentModeBrackets;
                    // check if previous token is in a different mode
                    if (currentModeIndex > 0 && currentMode.startIndex === position.column - 1) {
                        prevMode = modeTransitions[currentModeIndex - 1];
                        prevModeBrackets = prevMode.mode.richEditSupport ? prevMode.mode.richEditSupport.brackets : null;
                    }
                    if (prevModeBrackets) {
                        // limit search in case previous token is very large, there's no need to go beyond `maxBracketLength`
                        prevTokenStart = Math.max(prevTokenStart, position.column - 1 - prevModeBrackets.maxBracketLength);
                        var foundBracket = richEditBrackets_1.BracketsUtils.findPrevBracketInToken(prevModeBrackets.reversedRegex, lineNumber, lineText, prevTokenStart, currentTokenStart);
                        // check that we didn't hit a bracket too far away from position
                        if (foundBracket && foundBracket.startColumn <= position.column && position.column <= foundBracket.endColumn) {
                            var foundBracketText = lineText.substring(foundBracket.startColumn - 1, foundBracket.endColumn - 1);
                            var r = this._matchFoundBracket(foundBracket, prevModeBrackets.textIsBracket[foundBracketText], prevModeBrackets.textIsOpenBracket[foundBracketText]);
                            // check that we can actually match this bracket
                            if (r) {
                                return r;
                            }
                        }
                    }
                }
            }
            // check that the token is not to be ignored
            if (!supports_1.ignoreBracketsInToken(getType(tokensMap, tokens[currentTokenIndex]))) {
                if (currentModeBrackets) {
                    // limit search to not go before `maxBracketLength`
                    currentTokenStart = Math.max(currentTokenStart, position.column - 1 - currentModeBrackets.maxBracketLength);
                    // limit search to not go after `maxBracketLength`
                    var currentTokenEnd = (currentTokenIndex + 1 < tokens.length ? getStartIndex(tokens[currentTokenIndex + 1]) : lineText.length);
                    currentTokenEnd = Math.min(currentTokenEnd, position.column - 1 + currentModeBrackets.maxBracketLength);
                    // it might still be the case that [currentTokenStart -> currentTokenEnd] contains multiple brackets
                    while (true) {
                        var foundBracket = richEditBrackets_1.BracketsUtils.findNextBracketInText(currentModeBrackets.forwardRegex, lineNumber, lineText.substring(currentTokenStart, currentTokenEnd), currentTokenStart);
                        if (!foundBracket) {
                            // there are no brackets in this text
                            break;
                        }
                        // check that we didn't hit a bracket too far away from position
                        if (foundBracket.startColumn <= position.column && position.column <= foundBracket.endColumn) {
                            var foundBracketText = lineText.substring(foundBracket.startColumn - 1, foundBracket.endColumn - 1);
                            var r = this._matchFoundBracket(foundBracket, currentModeBrackets.textIsBracket[foundBracketText], currentModeBrackets.textIsOpenBracket[foundBracketText]);
                            // check that we can actually match this bracket
                            if (r) {
                                return r;
                            }
                        }
                        currentTokenStart = foundBracket.endColumn - 1;
                    }
                }
            }
            return {
                brackets: null,
                isAccurate: true
            };
        };
        TextModelWithTokens.prototype._matchFoundBracket = function (foundBracket, data, isOpen) {
            if (isOpen) {
                var matched = this._findMatchingBracketDown(data, foundBracket.getEndPosition());
                if (matched) {
                    return {
                        brackets: [foundBracket, matched],
                        isAccurate: true
                    };
                }
            }
            else {
                var matched = this._findMatchingBracketUp(data, foundBracket.getStartPosition());
                if (matched) {
                    return {
                        brackets: [foundBracket, matched],
                        isAccurate: true
                    };
                }
            }
            return null;
        };
        TextModelWithTokens.prototype._findMatchingBracketUp = function (bracket, position) {
            // console.log('_findMatchingBracketUp: ', 'bracket: ', JSON.stringify(bracket), 'startPosition: ', String(position));
            var modeId = bracket.modeId;
            var tokensMap = this._tokensInflatorMap;
            var reversedBracketRegex = bracket.reversedRegex;
            var count = -1;
            for (var lineNumber = position.lineNumber; lineNumber >= 1; lineNumber--) {
                var lineTokens = this._lines[lineNumber - 1].getTokens();
                var lineText = this._lines[lineNumber - 1].text;
                var tokens = lineTokens.getBinaryEncodedTokens();
                var modeTransitions = this._lines[lineNumber - 1].getModeTransitions().toArray(this._mode);
                var currentModeIndex = modeTransitions.length - 1;
                var currentModeStart = modeTransitions[currentModeIndex].startIndex;
                var currentModeId = modeTransitions[currentModeIndex].mode.getId();
                var tokensLength = tokens.length - 1;
                var currentTokenEnd = lineText.length;
                if (lineNumber === position.lineNumber) {
                    tokensLength = lineTokens.findIndexOfOffset(position.column - 1);
                    currentTokenEnd = position.column - 1;
                    currentModeIndex = arrays_1.Arrays.findIndexInSegmentsArray(modeTransitions, position.column - 1);
                    currentModeStart = modeTransitions[currentModeIndex].startIndex;
                    currentModeId = modeTransitions[currentModeIndex].mode.getId();
                }
                for (var tokenIndex = tokensLength; tokenIndex >= 0; tokenIndex--) {
                    var currentToken = tokens[tokenIndex];
                    var currentTokenType = getType(tokensMap, currentToken);
                    var currentTokenStart = getStartIndex(currentToken);
                    if (currentTokenStart < currentModeStart) {
                        currentModeIndex--;
                        currentModeStart = modeTransitions[currentModeIndex].startIndex;
                        currentModeId = modeTransitions[currentModeIndex].mode.getId();
                    }
                    if (currentModeId === modeId && !supports_1.ignoreBracketsInToken(currentTokenType)) {
                        while (true) {
                            var r = richEditBrackets_1.BracketsUtils.findPrevBracketInToken(reversedBracketRegex, lineNumber, lineText, currentTokenStart, currentTokenEnd);
                            if (!r) {
                                break;
                            }
                            var hitText = lineText.substring(r.startColumn - 1, r.endColumn - 1);
                            if (hitText === bracket.open) {
                                count++;
                            }
                            else if (hitText === bracket.close) {
                                count--;
                            }
                            if (count === 0) {
                                return r;
                            }
                            currentTokenEnd = r.startColumn - 1;
                        }
                    }
                    currentTokenEnd = currentTokenStart;
                }
            }
            return null;
        };
        TextModelWithTokens.prototype._findMatchingBracketDown = function (bracket, position) {
            // console.log('_findMatchingBracketDown: ', 'bracket: ', JSON.stringify(bracket), 'startPosition: ', String(position));
            var modeId = bracket.modeId;
            var tokensMap = this._tokensInflatorMap;
            var bracketRegex = bracket.forwardRegex;
            var count = 1;
            for (var lineNumber = position.lineNumber, lineCount = this.getLineCount(); lineNumber <= lineCount; lineNumber++) {
                var lineTokens = this._lines[lineNumber - 1].getTokens();
                var lineText = this._lines[lineNumber - 1].text;
                var tokens = lineTokens.getBinaryEncodedTokens();
                var modeTransitions = this._lines[lineNumber - 1].getModeTransitions().toArray(this._mode);
                var currentModeIndex = 0;
                var nextModeStart = (currentModeIndex + 1 < modeTransitions.length ? modeTransitions[currentModeIndex + 1].startIndex : lineText.length + 1);
                var currentModeId = modeTransitions[currentModeIndex].mode.getId();
                var startTokenIndex = 0;
                var currentTokenStart = getStartIndex(startTokenIndex);
                if (lineNumber === position.lineNumber) {
                    startTokenIndex = lineTokens.findIndexOfOffset(position.column - 1);
                    currentTokenStart = Math.max(currentTokenStart, position.column - 1);
                    currentModeIndex = arrays_1.Arrays.findIndexInSegmentsArray(modeTransitions, position.column - 1);
                    nextModeStart = (currentModeIndex + 1 < modeTransitions.length ? modeTransitions[currentModeIndex + 1].startIndex : lineText.length + 1);
                    currentModeId = modeTransitions[currentModeIndex].mode.getId();
                }
                for (var tokenIndex = startTokenIndex, tokensLength = tokens.length; tokenIndex < tokensLength; tokenIndex++) {
                    var currentToken = tokens[tokenIndex];
                    var currentTokenType = getType(tokensMap, currentToken);
                    var currentTokenEnd = tokenIndex + 1 < tokensLength ? getStartIndex(tokens[tokenIndex + 1]) : lineText.length;
                    if (currentTokenStart >= nextModeStart) {
                        currentModeIndex++;
                        nextModeStart = (currentModeIndex + 1 < modeTransitions.length ? modeTransitions[currentModeIndex + 1].startIndex : lineText.length + 1);
                        currentModeId = modeTransitions[currentModeIndex].mode.getId();
                    }
                    if (currentModeId === modeId && !supports_1.ignoreBracketsInToken(currentTokenType)) {
                        while (true) {
                            var r = richEditBrackets_1.BracketsUtils.findNextBracketInToken(bracketRegex, lineNumber, lineText, currentTokenStart, currentTokenEnd);
                            if (!r) {
                                break;
                            }
                            var hitText = lineText.substring(r.startColumn - 1, r.endColumn - 1);
                            if (hitText === bracket.open) {
                                count++;
                            }
                            else if (hitText === bracket.close) {
                                count--;
                            }
                            if (count === 0) {
                                return r;
                            }
                            currentTokenStart = r.endColumn - 1;
                        }
                    }
                    currentTokenStart = currentTokenEnd;
                }
            }
            return null;
        };
        TextModelWithTokens.prototype.findPrevBracket = function (_position) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.findPrevBracket: Model is disposed');
            }
            var position = this.validatePosition(_position);
            var tokensMap = this._tokensInflatorMap;
            var reversedBracketRegex = /[\(\)\[\]\{\}]/; // TODO@Alex: use mode's brackets
            for (var lineNumber = position.lineNumber; lineNumber >= 1; lineNumber--) {
                var lineTokens = this._lines[lineNumber - 1].getTokens();
                var lineText = this._lines[lineNumber - 1].text;
                var tokens = lineTokens.getBinaryEncodedTokens();
                var tokensLength = tokens.length - 1;
                var currentTokenEnd = lineText.length;
                if (lineNumber === position.lineNumber) {
                    tokensLength = lineTokens.findIndexOfOffset(position.column - 1);
                    currentTokenEnd = position.column - 1;
                }
                for (var tokenIndex = tokensLength; tokenIndex >= 0; tokenIndex--) {
                    var currentToken = tokens[tokenIndex];
                    var currentTokenType = getType(tokensMap, currentToken);
                    var currentTokenStart = getStartIndex(currentToken);
                    if (!supports_1.ignoreBracketsInToken(currentTokenType)) {
                        var r = richEditBrackets_1.BracketsUtils.findPrevBracketInToken(reversedBracketRegex, lineNumber, lineText, currentTokenStart, currentTokenEnd);
                        if (r) {
                            return this._toFoundBracket(r);
                        }
                    }
                    currentTokenEnd = currentTokenStart;
                }
            }
            return null;
        };
        TextModelWithTokens.prototype.findNextBracket = function (_position) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTokens.findNextBracket: Model is disposed');
            }
            var position = this.validatePosition(_position);
            var tokensMap = this._tokensInflatorMap;
            var bracketRegex = /[\(\)\[\]\{\}]/; // TODO@Alex: use mode's brackets
            for (var lineNumber = position.lineNumber, lineCount = this.getLineCount(); lineNumber <= lineCount; lineNumber++) {
                var lineTokens = this._lines[lineNumber - 1].getTokens();
                var lineText = this._lines[lineNumber - 1].text;
                var tokens = lineTokens.getBinaryEncodedTokens();
                var startTokenIndex = 0;
                var currentTokenStart = getStartIndex(startTokenIndex);
                if (lineNumber === position.lineNumber) {
                    startTokenIndex = lineTokens.findIndexOfOffset(position.column - 1);
                    currentTokenStart = Math.max(currentTokenStart, position.column - 1);
                }
                for (var tokenIndex = startTokenIndex, tokensLength = tokens.length; tokenIndex < tokensLength; tokenIndex++) {
                    var currentToken = tokens[tokenIndex];
                    var currentTokenType = getType(tokensMap, currentToken);
                    var currentTokenEnd = tokenIndex + 1 < tokensLength ? getStartIndex(tokens[tokenIndex + 1]) : lineText.length;
                    if (!supports_1.ignoreBracketsInToken(currentTokenType)) {
                        var r = richEditBrackets_1.BracketsUtils.findNextBracketInToken(bracketRegex, lineNumber, lineText, currentTokenStart, currentTokenEnd);
                        if (r) {
                            return this._toFoundBracket(r);
                        }
                    }
                    currentTokenStart = currentTokenEnd;
                }
            }
            return null;
        };
        TextModelWithTokens.prototype._toFoundBracket = function (r) {
            if (!r) {
                return null;
            }
            var text = this.getValueInRange(r);
            // TODO@Alex: use mode's brackets
            switch (text) {
                case '(': return { range: r, open: '(', close: ')', isOpen: true };
                case ')': return { range: r, open: '(', close: ')', isOpen: false };
                case '[': return { range: r, open: '[', close: ']', isOpen: true };
                case ']': return { range: r, open: '[', close: ']', isOpen: false };
                case '{': return { range: r, open: '{', close: '}', isOpen: true };
                case '}': return { range: r, open: '{', close: '}', isOpen: false };
            }
            return null;
        };
        TextModelWithTokens.MODE_TOKENIZATION_FAILED_MSG = nls.localize(0, null);
        TextModelWithTokens.MODEL_SYNC_LIMIT = 5 * 1024 * 1024; // 5 MB
        TextModelWithTokens.MODEL_TOKENIZATION_LIMIT = 20 * 1024 * 1024; // 20 MB
        return TextModelWithTokens;
    }(textModel_1.TextModel));
    exports.TextModelWithTokens = TextModelWithTokens;
    var getType = editorCommon.LineTokensBinaryEncoding.getType;
    var getStartIndex = editorCommon.LineTokensBinaryEncoding.getStartIndex;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/model/mirrorModel", ["require", "exports", 'vs/base/common/lifecycle', 'vs/editor/common/editorCommon', 'vs/editor/common/model/modelLine', 'vs/editor/common/model/textModel', 'vs/editor/common/model/textModelWithTokens', 'vs/editor/common/viewModel/prefixSumComputer'], function (require, exports, lifecycle_1, editorCommon, modelLine_1, textModel_1, textModelWithTokens_1, prefixSumComputer_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var AbstractMirrorModel = (function (_super) {
        __extends(AbstractMirrorModel, _super);
        function AbstractMirrorModel(allowedEventTypes, versionId, value, mode, associatedResource) {
            _super.call(this, allowedEventTypes.concat([editorCommon.EventType.ModelDispose]), value, false, mode);
            this._setVersionId(versionId);
            this._associatedResource = associatedResource;
        }
        AbstractMirrorModel.prototype.getModeId = function () {
            if (this._isDisposed) {
                throw new Error('AbstractMirrorModel.getModeId: Model is disposed');
            }
            return this.getMode().getId();
        };
        AbstractMirrorModel.prototype.getEmbeddedAtPosition = function (position) {
            return null;
        };
        AbstractMirrorModel.prototype.getAllEmbedded = function () {
            return [];
        };
        AbstractMirrorModel.prototype._constructLines = function (rawText) {
            _super.prototype._constructLines.call(this, rawText);
            // Force EOL to be \n
            this._EOL = '\n';
        };
        AbstractMirrorModel.prototype.destroy = function () {
            this.dispose();
        };
        AbstractMirrorModel.prototype.dispose = function () {
            this.emit(editorCommon.EventType.ModelDispose);
            _super.prototype.dispose.call(this);
        };
        AbstractMirrorModel.prototype.getAssociatedResource = function () {
            if (this._isDisposed) {
                throw new Error('AbstractMirrorModel.getAssociatedResource: Model is disposed');
            }
            return this._associatedResource;
        };
        AbstractMirrorModel.prototype._ensurePrefixSum = function () {
            if (!this._lineStarts) {
                var lineStartValues = [], eolLength = this.getEOL().length;
                for (var i = 0, len = this._lines.length; i < len; i++) {
                    lineStartValues.push(this._lines[i].text.length + eolLength);
                }
                this._lineStarts = new prefixSumComputer_1.PrefixSumComputer(lineStartValues);
            }
        };
        AbstractMirrorModel.prototype.getRangeFromOffsetAndLength = function (offset, length) {
            if (this._isDisposed) {
                throw new Error('AbstractMirrorModel.getRangeFromOffsetAndLength: Model is disposed');
            }
            var startPosition = this.getPositionFromOffset(offset), endPosition = this.getPositionFromOffset(offset + length);
            return {
                startLineNumber: startPosition.lineNumber,
                startColumn: startPosition.column,
                endLineNumber: endPosition.lineNumber,
                endColumn: endPosition.column
            };
        };
        AbstractMirrorModel.prototype.getOffsetAndLengthFromRange = function (range) {
            if (this._isDisposed) {
                throw new Error('AbstractMirrorModel.getOffsetAndLengthFromRange: Model is disposed');
            }
            var startOffset = this.getOffsetFromPosition({ lineNumber: range.startLineNumber, column: range.startColumn }), endOffset = this.getOffsetFromPosition({ lineNumber: range.endLineNumber, column: range.endColumn });
            return {
                offset: startOffset,
                length: endOffset - startOffset
            };
        };
        AbstractMirrorModel.prototype.getPositionFromOffset = function (offset) {
            if (this._isDisposed) {
                throw new Error('AbstractMirrorModel.getPositionFromOffset: Model is disposed');
            }
            this._ensurePrefixSum();
            var r = {
                index: 0,
                remainder: 0
            };
            this._lineStarts.getIndexOf(offset, r);
            return {
                lineNumber: r.index + 1,
                column: this.getEOL().length + r.remainder
            };
        };
        AbstractMirrorModel.prototype.getOffsetFromPosition = function (position) {
            if (this._isDisposed) {
                throw new Error('AbstractMirrorModel.getOffsetFromPosition: Model is disposed');
            }
            return this.getLineStart(position.lineNumber) + position.column - 1 /* column isn't zero-index based */;
        };
        AbstractMirrorModel.prototype.getLineStart = function (lineNumber) {
            if (this._isDisposed) {
                throw new Error('AbstractMirrorModel.getLineStart: Model is disposed');
            }
            this._ensurePrefixSum();
            var lineIndex = Math.min(lineNumber, this._lines.length) - 1;
            return this._lineStarts.getAccumulatedValue(lineIndex - 1);
        };
        AbstractMirrorModel.prototype.getAllWordsWithRange = function () {
            if (this._isDisposed) {
                throw new Error('AbstractMirrorModel.getAllWordsWithRange: Model is disposed');
            }
            if (this._lines.length > 10000) {
                // This is a very heavy method, unavailable for very heavy models
                return [];
            }
            var result = [], i;
            var toTextRange = function (info) {
                var s = line.text.substring(info.start, info.end);
                var r = { startLineNumber: i + 1, startColumn: info.start + 1, endLineNumber: i + 1, endColumn: info.end + 1 };
                result.push({ text: s, range: r });
            };
            for (i = 0; i < this._lines.length; i++) {
                var line = this._lines[i];
                this.wordenize(line.text).forEach(toTextRange);
            }
            return result;
        };
        AbstractMirrorModel.prototype.getAllWords = function () {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('AbstractMirrorModel.getAllWords: Model is disposed');
            }
            var result = [];
            this._lines.forEach(function (line) {
                _this.wordenize(line.text).forEach(function (info) {
                    result.push(line.text.substring(info.start, info.end));
                });
            });
            return result;
        };
        AbstractMirrorModel.prototype.getAllUniqueWords = function (skipWordOnce) {
            if (this._isDisposed) {
                throw new Error('AbstractMirrorModel.getAllUniqueWords: Model is disposed');
            }
            var foundSkipWord = false;
            var uniqueWords = {};
            return this.getAllWords().filter(function (word) {
                if (skipWordOnce && !foundSkipWord && skipWordOnce === word) {
                    foundSkipWord = true;
                    return false;
                }
                else if (uniqueWords[word]) {
                    return false;
                }
                else {
                    uniqueWords[word] = true;
                    return true;
                }
            });
        };
        //	// TODO@Joh, TODO@Alex - remove these and make sure the super-things work
        AbstractMirrorModel.prototype.wordenize = function (content) {
            var result = [];
            var match;
            var wordsRegexp = this._getWordDefinition();
            while (match = wordsRegexp.exec(content)) {
                result.push({ start: match.index, end: match.index + match[0].length });
            }
            return result;
        };
        return AbstractMirrorModel;
    }(textModelWithTokens_1.TextModelWithTokens));
    exports.AbstractMirrorModel = AbstractMirrorModel;
    var MirrorModelEmbedded = (function (_super) {
        __extends(MirrorModelEmbedded, _super);
        function MirrorModelEmbedded(actualModel, includeRanges, mode, url) {
            _super.call(this, ['changed'], actualModel.getVersionId(), MirrorModelEmbedded._getMirrorValueWithinRanges(actualModel, includeRanges), mode, url);
            this._actualModel = actualModel;
        }
        MirrorModelEmbedded._getMirrorValueWithinRanges = function (actualModel, includeRanges) {
            var resultingText = '', prevLineAdded = 1, prevColumnAdded = 1, i;
            for (i = 0; i < includeRanges.length; i++) {
                var includeRange = includeRanges[i];
                resultingText += actualModel.getEmptiedValueInRange({
                    startLineNumber: prevLineAdded,
                    startColumn: prevColumnAdded,
                    endLineNumber: includeRange.startLineNumber,
                    endColumn: includeRange.startColumn
                }, ' ');
                resultingText += actualModel.getValueInRange(includeRange);
                prevLineAdded = includeRange.endLineNumber;
                prevColumnAdded = includeRange.endColumn;
            }
            var lastLineNumber = actualModel.getLineCount(), lastColumn = actualModel.getLineMaxColumn(lastLineNumber);
            resultingText += actualModel.getEmptiedValueInRange({
                startLineNumber: prevLineAdded,
                startColumn: prevColumnAdded,
                endLineNumber: lastLineNumber,
                endColumn: lastColumn
            }, ' ');
            var actualModelOptions = actualModel.getOptions();
            return textModel_1.TextModel.toRawText(resultingText, {
                tabSize: actualModelOptions.tabSize,
                insertSpaces: actualModelOptions.insertSpaces,
                detectIndentation: false,
                defaultEOL: actualModelOptions.defaultEOL
            });
        };
        MirrorModelEmbedded.prototype.setIncludedRanges = function (newIncludedRanges) {
            var prevVersionId = this.getVersionId();
            // Force recreating of line starts (when used)
            this._lineStarts = null;
            this._constructLines(MirrorModelEmbedded._getMirrorValueWithinRanges(this._actualModel, newIncludedRanges));
            this._resetTokenizationState();
            this._setVersionId(prevVersionId + 1);
            this.emit('changed', {});
        };
        return MirrorModelEmbedded;
    }(AbstractMirrorModel));
    exports.MirrorModelEmbedded = MirrorModelEmbedded;
    var EmbeddedModeRange = (function () {
        function EmbeddedModeRange(mode) {
            this.mode = mode;
            this.ranges = [];
        }
        return EmbeddedModeRange;
    }());
    function createTestMirrorModelFromString(value, mode, associatedResource) {
        if (mode === void 0) { mode = null; }
        return new MirrorModel(null, 0, textModel_1.TextModel.toRawText(value, textModel_1.TextModel.DEFAULT_CREATION_OPTIONS), mode, associatedResource);
    }
    exports.createTestMirrorModelFromString = createTestMirrorModelFromString;
    var MirrorModel = (function (_super) {
        __extends(MirrorModel, _super);
        function MirrorModel(resourceService, versionId, value, mode, associatedResource) {
            _super.call(this, ['changed'], versionId, value, mode, associatedResource);
            this._resourceService = resourceService;
            this._embeddedModels = {};
            this._updateEmbeddedModels();
        }
        MirrorModel.prototype.getEmbeddedAtPosition = function (position) {
            if (this._isDisposed) {
                throw new Error('MirrorModel.getEmbeddedAtPosition: Model is disposed');
            }
            var modeAtPosition = this.getModeAtPosition(position.lineNumber, position.column);
            if (this._embeddedModels.hasOwnProperty(modeAtPosition.getId())) {
                return this._embeddedModels[modeAtPosition.getId()];
            }
            return null;
        };
        MirrorModel.prototype.getAllEmbedded = function () {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('MirrorModel.getAllEmbedded: Model is disposed');
            }
            return Object.keys(this._embeddedModels).map(function (embeddedModeId) { return _this._embeddedModels[embeddedModeId]; });
        };
        MirrorModel.prototype.dispose = function () {
            var _this = this;
            _super.prototype.dispose.call(this);
            var embeddedModels = Object.keys(this._embeddedModels).map(function (modeId) { return _this._embeddedModels[modeId]; });
            embeddedModels.forEach(function (embeddedModel) { return _this._resourceService.remove(embeddedModel.getAssociatedResource()); });
            lifecycle_1.disposeAll(embeddedModels);
            this._embeddedModels = {};
        };
        MirrorModel.prototype.setMode = function (newModeOrPromise) {
            _super.prototype.setMode.call(this, newModeOrPromise);
            this._updateEmbeddedModels();
        };
        MirrorModel._getModesRanges = function (model) {
            var encounteredModesRanges = {};
            var getOrCreateEmbeddedModeRange = function (modeId, mode) {
                if (!encounteredModesRanges.hasOwnProperty(modeId)) {
                    encounteredModesRanges[modeId] = new EmbeddedModeRange(mode);
                }
                return encounteredModesRanges[modeId];
            };
            var lineCount = model.getLineCount();
            var currentModeId = model.getMode().getId();
            var currentMode = model.getMode();
            var currentStartLineNumber = 1, currentStartColumn = 1;
            for (var lineNumber = 1; lineNumber <= lineCount; lineNumber++) {
                var modeTransitions = model._getLineModeTransitions(lineNumber);
                for (var i = 0; i < modeTransitions.length; i++) {
                    var modeTransition = modeTransitions[i];
                    if (modeTransition.mode.getId() !== currentModeId) {
                        var modeRange = getOrCreateEmbeddedModeRange(currentModeId, currentMode);
                        modeRange.ranges.push({
                            startLineNumber: currentStartLineNumber,
                            startColumn: currentStartColumn,
                            endLineNumber: lineNumber,
                            endColumn: modeTransition.startIndex + 1
                        });
                        currentModeId = modeTransition.mode.getId();
                        currentMode = modeTransition.mode;
                        currentStartLineNumber = lineNumber;
                        currentStartColumn = modeTransition.startIndex + 1;
                    }
                }
            }
            var lastLineNumber = lineCount;
            var lastColumn = model.getLineMaxColumn(lastLineNumber);
            if (currentStartLineNumber !== lastLineNumber || currentStartColumn !== lastColumn) {
                var modeRange = getOrCreateEmbeddedModeRange(currentModeId, currentMode);
                modeRange.ranges.push({
                    startLineNumber: currentStartLineNumber,
                    startColumn: currentStartColumn,
                    endLineNumber: lastLineNumber,
                    endColumn: lastColumn
                });
            }
            return encounteredModesRanges;
        };
        MirrorModel.prototype._updateEmbeddedModels = function () {
            if (!this._resourceService || !this.getMode().tokenizationSupport || !this.getMode().tokenizationSupport.shouldGenerateEmbeddedModels) {
                return false;
            }
            var newModesRanges = MirrorModel._getModesRanges(this);
            // Empty out embedded models that have disappeared
            var oldNestedModesIds = Object.keys(this._embeddedModels);
            for (var i = 0; i < oldNestedModesIds.length; i++) {
                var oldNestedModeId = oldNestedModesIds[i];
                if (!newModesRanges.hasOwnProperty(oldNestedModeId)) {
                    this._embeddedModels[oldNestedModeId].setIncludedRanges([{
                            startLineNumber: 1,
                            startColumn: 1,
                            endLineNumber: 1,
                            endColumn: 1
                        }]);
                }
            }
            var newNestedModesIds = Object.keys(newModesRanges);
            for (var i = 0; i < newNestedModesIds.length; i++) {
                var newNestedModeId = newNestedModesIds[i];
                if (this._embeddedModels.hasOwnProperty(newNestedModeId)) {
                    this._embeddedModels[newNestedModeId].setIncludedRanges(newModesRanges[newNestedModeId].ranges);
                }
                else {
                    // TODO@Alex: implement derived resources (embedded mirror models) better
                    var embeddedModelUrl = this.getAssociatedResource().withFragment(this.getAssociatedResource().fragment + 'URL_MARSHAL_REMOVE' + newNestedModeId);
                    this._embeddedModels[newNestedModeId] = new MirrorModelEmbedded(this, newModesRanges[newNestedModeId].ranges, newModesRanges[newNestedModeId].mode, embeddedModelUrl);
                    this._resourceService.insert(this._embeddedModels[newNestedModeId].getAssociatedResource(), this._embeddedModels[newNestedModeId]);
                }
            }
            return false;
        };
        MirrorModel.prototype.onEvents = function (events) {
            if (this._isDisposed) {
                throw new Error('MirrorModel.onEvents: Model is disposed');
            }
            var changed = false;
            for (var i = 0, len = events.contentChanged.length; i < len; i++) {
                var contentChangedEvent = events.contentChanged[i];
                // Force recreating of line starts
                this._lineStarts = null;
                this._setVersionId(contentChangedEvent.versionId);
                switch (contentChangedEvent.changeType) {
                    case editorCommon.EventType.ModelContentChangedFlush:
                        this._onLinesFlushed(contentChangedEvent);
                        changed = true;
                        break;
                    case editorCommon.EventType.ModelContentChangedLinesDeleted:
                        this._onLinesDeleted(contentChangedEvent);
                        changed = true;
                        break;
                    case editorCommon.EventType.ModelContentChangedLinesInserted:
                        this._onLinesInserted(contentChangedEvent);
                        changed = true;
                        break;
                    case editorCommon.EventType.ModelContentChangedLineChanged:
                        this._onLineChanged(contentChangedEvent);
                        changed = true;
                        break;
                }
            }
            var shouldFlushMarkers = false;
            if (changed) {
                this.emit('changed', {});
                shouldFlushMarkers = this._updateEmbeddedModels();
            }
            return shouldFlushMarkers;
        };
        MirrorModel.prototype._onLinesFlushed = function (e) {
            // Flush my lines
            this._constructLines(e.detail);
            this._resetTokenizationState();
        };
        MirrorModel.prototype._onLineChanged = function (e) {
            this._lines[e.lineNumber - 1].applyEdits({}, [{
                    startColumn: 1,
                    endColumn: Number.MAX_VALUE,
                    text: e.detail,
                    forceMoveMarkers: false
                }]);
            this._invalidateLine(e.lineNumber - 1);
        };
        MirrorModel.prototype._onLinesDeleted = function (e) {
            var fromLineIndex = e.fromLineNumber - 1, toLineIndex = e.toLineNumber - 1;
            // Save first line's state
            var firstLineState = this._lines[fromLineIndex].getState();
            this._lines.splice(fromLineIndex, toLineIndex - fromLineIndex + 1);
            if (fromLineIndex < this._lines.length) {
                // This check is always true in real world, but the tests forced this
                // Restore first line's state
                this._lines[fromLineIndex].setState(firstLineState);
                // Invalidate line
                this._invalidateLine(fromLineIndex);
            }
        };
        MirrorModel.prototype._onLinesInserted = function (e) {
            var lineIndex, i, splitLines = e.detail.split('\n');
            for (lineIndex = e.fromLineNumber - 1, i = 0; lineIndex < e.toLineNumber; lineIndex++, i++) {
                this._lines.splice(lineIndex, 0, new modelLine_1.ModelLine(0, splitLines[i]));
            }
            if (e.fromLineNumber >= 2) {
                // This check is always true in real world, but the tests forced this
                this._invalidateLine(e.fromLineNumber - 2);
            }
        };
        return MirrorModel;
    }(AbstractMirrorModel));
    exports.MirrorModel = MirrorModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/model/textModelWithMarkers", ["require", "exports", 'vs/editor/common/core/idGenerator', 'vs/editor/common/core/position', 'vs/editor/common/model/textModelWithTokens'], function (require, exports, idGenerator_1, position_1, textModelWithTokens_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LineMarker = (function () {
        function LineMarker(id, column, stickToPreviousCharacter) {
            this.id = id;
            this.column = column;
            this.stickToPreviousCharacter = stickToPreviousCharacter;
            this.oldLineNumber = 0;
            this.oldColumn = 0;
            this.line = null;
        }
        LineMarker.prototype.toString = function () {
            return '{\'' + this.id + '\';' + this.column + ',' + this.stickToPreviousCharacter + ',[' + this.oldLineNumber + ',' + this.oldColumn + ']}';
        };
        return LineMarker;
    }());
    exports.LineMarker = LineMarker;
    var _INSTANCE_COUNT = 0;
    var TextModelWithMarkers = (function (_super) {
        __extends(TextModelWithMarkers, _super);
        function TextModelWithMarkers(allowedEventTypes, rawText, modeOrPromise) {
            _super.call(this, allowedEventTypes, rawText, true, modeOrPromise);
            this._markerIdGenerator = new idGenerator_1.IdGenerator((++_INSTANCE_COUNT) + ';');
            this._markerIdToMarker = {};
        }
        TextModelWithMarkers.prototype.dispose = function () {
            this._markerIdToMarker = null;
            _super.prototype.dispose.call(this);
        };
        TextModelWithMarkers.prototype._resetValue = function (e, newValue) {
            _super.prototype._resetValue.call(this, e, newValue);
            // Destroy all my markers
            this._markerIdToMarker = {};
        };
        TextModelWithMarkers.prototype._addMarker = function (lineNumber, column, stickToPreviousCharacter) {
            if (this._isDisposed) {
                throw new Error('TextModelWithMarkers._addMarker: Model is disposed');
            }
            var pos = this.validatePosition(new position_1.Position(lineNumber, column));
            var marker = new LineMarker(this._markerIdGenerator.generate(), pos.column, stickToPreviousCharacter);
            this._markerIdToMarker[marker.id] = marker;
            this._lines[pos.lineNumber - 1].addMarker(marker);
            return marker.id;
        };
        TextModelWithMarkers.prototype._addMarkers = function (newMarkers) {
            var addMarkersPerLine = Object.create(null);
            var result = [];
            for (var i = 0, len = newMarkers.length; i < len; i++) {
                var newMarker = newMarkers[i];
                var marker = new LineMarker(this._markerIdGenerator.generate(), newMarker.column, newMarker.stickToPreviousCharacter);
                this._markerIdToMarker[marker.id] = marker;
                if (!addMarkersPerLine[newMarker.lineNumber]) {
                    addMarkersPerLine[newMarker.lineNumber] = [];
                }
                addMarkersPerLine[newMarker.lineNumber].push(marker);
                result.push(marker.id);
            }
            var lineNumbers = Object.keys(addMarkersPerLine);
            for (var i = 0, len = lineNumbers.length; i < len; i++) {
                var lineNumber = parseInt(lineNumbers[i], 10);
                this._lines[lineNumber - 1].addMarkers(addMarkersPerLine[lineNumbers[i]]);
            }
            return result;
        };
        TextModelWithMarkers.prototype._changeMarker = function (id, lineNumber, column) {
            if (this._isDisposed) {
                throw new Error('TextModelWithMarkers._changeMarker: Model is disposed');
            }
            if (this._markerIdToMarker.hasOwnProperty(id)) {
                var marker = this._markerIdToMarker[id];
                var newPos = this.validatePosition(new position_1.Position(lineNumber, column));
                if (newPos.lineNumber !== marker.line.lineNumber) {
                    // Move marker between lines
                    marker.line.removeMarker(marker);
                    this._lines[newPos.lineNumber - 1].addMarker(marker);
                }
                // Update marker column
                marker.column = newPos.column;
            }
        };
        TextModelWithMarkers.prototype._changeMarkerStickiness = function (id, newStickToPreviousCharacter) {
            if (this._isDisposed) {
                throw new Error('TextModelWithMarkers._changeMarkerStickiness: Model is disposed');
            }
            if (this._markerIdToMarker.hasOwnProperty(id)) {
                var marker = this._markerIdToMarker[id];
                if (marker.stickToPreviousCharacter !== newStickToPreviousCharacter) {
                    marker.stickToPreviousCharacter = newStickToPreviousCharacter;
                }
            }
        };
        TextModelWithMarkers.prototype._getMarker = function (id) {
            if (this._isDisposed) {
                throw new Error('TextModelWithMarkers._getMarker: Model is disposed');
            }
            if (this._markerIdToMarker.hasOwnProperty(id)) {
                var marker = this._markerIdToMarker[id];
                return new position_1.Position(marker.line.lineNumber, marker.column);
            }
            return null;
        };
        TextModelWithMarkers.prototype._getMarkersCount = function () {
            return Object.keys(this._markerIdToMarker).length;
        };
        TextModelWithMarkers.prototype._getLineMarkers = function (lineNumber) {
            if (this._isDisposed) {
                throw new Error('TextModelWithMarkers._getLineMarkers: Model is disposed');
            }
            if (lineNumber < 1 || lineNumber > this.getLineCount()) {
                throw new Error('Illegal value ' + lineNumber + ' for `lineNumber`');
            }
            return this._lines[lineNumber - 1].getMarkers();
        };
        TextModelWithMarkers.prototype._removeMarker = function (id) {
            if (this._isDisposed) {
                throw new Error('TextModelWithMarkers._removeMarker: Model is disposed');
            }
            if (this._markerIdToMarker.hasOwnProperty(id)) {
                var marker = this._markerIdToMarker[id];
                marker.line.removeMarker(marker);
                delete this._markerIdToMarker[id];
            }
        };
        TextModelWithMarkers.prototype._removeMarkers = function (ids) {
            var removeMarkersPerLine = Object.create(null);
            for (var i = 0, len = ids.length; i < len; i++) {
                var id = ids[i];
                if (!this._markerIdToMarker.hasOwnProperty(id)) {
                    continue;
                }
                var marker = this._markerIdToMarker[id];
                var lineNumber = marker.line.lineNumber;
                if (!removeMarkersPerLine[lineNumber]) {
                    removeMarkersPerLine[lineNumber] = Object.create(null);
                }
                removeMarkersPerLine[lineNumber][id] = true;
                delete this._markerIdToMarker[id];
            }
            var lineNumbers = Object.keys(removeMarkersPerLine);
            for (var i = 0, len = lineNumbers.length; i < len; i++) {
                var lineNumber = parseInt(lineNumbers[i], 10);
                this._lines[lineNumber - 1].removeMarkers(removeMarkersPerLine[lineNumbers[i]]);
            }
        };
        TextModelWithMarkers.prototype._getMarkersInMap = function (markersMap) {
            if (this._isDisposed) {
                throw new Error('TextModelWithMarkers._getMarkersInMap: Model is disposed');
            }
            var result = [], markerId;
            for (markerId in markersMap) {
                if (markersMap.hasOwnProperty(markerId) && this._markerIdToMarker.hasOwnProperty(markerId)) {
                    result.push(this._markerIdToMarker[markerId]);
                }
            }
            return result;
        };
        return TextModelWithMarkers;
    }(textModelWithTokens_1.TextModelWithTokens));
    exports.TextModelWithMarkers = TextModelWithMarkers;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/model/textModelWithTrackedRanges", ["require", "exports", 'vs/editor/common/core/idGenerator', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/model/textModelWithMarkers', 'vs/editor/common/model/textModelWithTokens'], function (require, exports, idGenerator_1, range_1, editorCommon, textModelWithMarkers_1, textModelWithTokens_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var TrackedRangeModelRetokenizer = (function (_super) {
        __extends(TrackedRangeModelRetokenizer, _super);
        function TrackedRangeModelRetokenizer(retokenizePromise, lineNumber, model) {
            _super.call(this, retokenizePromise, model);
            this.trackedRangeId = model.addTrackedRange({
                startLineNumber: lineNumber,
                startColumn: 1,
                endLineNumber: lineNumber,
                endColumn: model.getLineMaxColumn(lineNumber)
            }, editorCommon.TrackedRangeStickiness.AlwaysGrowsWhenTypingAtEdges);
        }
        TrackedRangeModelRetokenizer.prototype.getRange = function () {
            return this._model.getTrackedRange(this.trackedRangeId);
        };
        TrackedRangeModelRetokenizer.prototype.dispose = function () {
            var model = this._model;
            // if this .dispose() is being called as part of the model.dispose(), then the tracked ranges might no longer be available (e.g. throw exceptions)
            if (model.isValidTrackedRange(this.trackedRangeId)) {
                model.removeTrackedRange(this.trackedRangeId);
            }
            _super.prototype.dispose.call(this);
        };
        return TrackedRangeModelRetokenizer;
    }(textModelWithTokens_1.FullModelRetokenizer));
    var TrackedRange = (function () {
        function TrackedRange(id, startMarkedId, endMarkerId) {
            this.id = id;
            this.startMarkerId = startMarkedId;
            this.endMarkerId = endMarkerId;
        }
        return TrackedRange;
    }());
    var _INSTANCE_COUNT = 0;
    var TextModelWithTrackedRanges = (function (_super) {
        __extends(TextModelWithTrackedRanges, _super);
        function TextModelWithTrackedRanges(allowedEventTypes, rawText, modeOrPromise) {
            _super.call(this, allowedEventTypes, rawText, modeOrPromise);
            this._rangeIdGenerator = new idGenerator_1.IdGenerator((++_INSTANCE_COUNT) + ';');
            this._ranges = {};
            this._markerIdToRangeId = {};
            this._multiLineTrackedRanges = {};
        }
        TextModelWithTrackedRanges.prototype._createRetokenizer = function (retokenizePromise, lineNumber) {
            return new TrackedRangeModelRetokenizer(retokenizePromise, lineNumber, this);
        };
        TextModelWithTrackedRanges.prototype.dispose = function () {
            this._ranges = null;
            this._markerIdToRangeId = null;
            this._multiLineTrackedRanges = null;
            _super.prototype.dispose.call(this);
        };
        TextModelWithTrackedRanges.prototype._resetValue = function (e, newValue) {
            _super.prototype._resetValue.call(this, e, newValue);
            // Destroy all my tracked ranges
            this._ranges = {};
            this._markerIdToRangeId = {};
            this._multiLineTrackedRanges = {};
        };
        TextModelWithTrackedRanges.prototype._setRangeIsMultiLine = function (rangeId, rangeIsMultiLine) {
            var rangeWasMultiLine = this._multiLineTrackedRanges.hasOwnProperty(rangeId);
            if (!rangeWasMultiLine && rangeIsMultiLine) {
                this._multiLineTrackedRanges[rangeId] = true;
            }
            else if (rangeWasMultiLine && !rangeIsMultiLine) {
                delete this._multiLineTrackedRanges[rangeId];
            }
        };
        TextModelWithTrackedRanges.prototype._shouldStartMarkerSticksToPreviousCharacter = function (stickiness) {
            if (stickiness === editorCommon.TrackedRangeStickiness.AlwaysGrowsWhenTypingAtEdges || stickiness === editorCommon.TrackedRangeStickiness.GrowsOnlyWhenTypingBefore) {
                return true;
            }
            return false;
        };
        TextModelWithTrackedRanges.prototype._shouldEndMarkerSticksToPreviousCharacter = function (stickiness) {
            if (stickiness === editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges || stickiness === editorCommon.TrackedRangeStickiness.GrowsOnlyWhenTypingBefore) {
                return true;
            }
            return false;
        };
        TextModelWithTrackedRanges.prototype._getTrackedRangesCount = function () {
            return Object.keys(this._ranges).length;
        };
        TextModelWithTrackedRanges.prototype.addTrackedRange = function (textRange, stickiness) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTrackedRanges.addTrackedRange: Model is disposed');
            }
            textRange = this.validateRange(textRange);
            var startMarkerSticksToPreviousCharacter = this._shouldStartMarkerSticksToPreviousCharacter(stickiness);
            var endMarkerSticksToPreviousCharacter = this._shouldEndMarkerSticksToPreviousCharacter(stickiness);
            var startMarkerId = this._addMarker(textRange.startLineNumber, textRange.startColumn, startMarkerSticksToPreviousCharacter);
            var endMarkerId = this._addMarker(textRange.endLineNumber, textRange.endColumn, endMarkerSticksToPreviousCharacter);
            var range = new TrackedRange(this._rangeIdGenerator.generate(), startMarkerId, endMarkerId);
            this._ranges[range.id] = range;
            this._markerIdToRangeId[startMarkerId] = range.id;
            this._markerIdToRangeId[endMarkerId] = range.id;
            this._setRangeIsMultiLine(range.id, (textRange.startLineNumber !== textRange.endLineNumber));
            return range.id;
        };
        TextModelWithTrackedRanges.prototype._addTrackedRanges = function (textRanges, stickinessArr) {
            var addMarkers = [];
            for (var i = 0, len = textRanges.length; i < len; i++) {
                var textRange = textRanges[i];
                var stickiness = stickinessArr[i];
                addMarkers.push({
                    lineNumber: textRange.startLineNumber,
                    column: textRange.startColumn,
                    stickToPreviousCharacter: this._shouldStartMarkerSticksToPreviousCharacter(stickiness)
                });
                addMarkers.push({
                    lineNumber: textRange.endLineNumber,
                    column: textRange.endColumn,
                    stickToPreviousCharacter: this._shouldEndMarkerSticksToPreviousCharacter(stickiness)
                });
            }
            var markerIds = this._addMarkers(addMarkers);
            var result = [];
            for (var i = 0, len = textRanges.length; i < len; i++) {
                var textRange = textRanges[i];
                var startMarkerId = markerIds[2 * i];
                var endMarkerId = markerIds[2 * i + 1];
                var range = new TrackedRange(this._rangeIdGenerator.generate(), startMarkerId, endMarkerId);
                this._ranges[range.id] = range;
                this._markerIdToRangeId[startMarkerId] = range.id;
                this._markerIdToRangeId[endMarkerId] = range.id;
                this._setRangeIsMultiLine(range.id, (textRange.startLineNumber !== textRange.endLineNumber));
                result.push(range.id);
            }
            return result;
        };
        TextModelWithTrackedRanges.prototype.changeTrackedRange = function (rangeId, newTextRange) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTrackedRanges.changeTrackedRange: Model is disposed');
            }
            if (this._ranges.hasOwnProperty(rangeId)) {
                newTextRange = this.validateRange(newTextRange);
                var range = this._ranges[rangeId];
                this._changeMarker(range.startMarkerId, newTextRange.startLineNumber, newTextRange.startColumn);
                this._changeMarker(range.endMarkerId, newTextRange.endLineNumber, newTextRange.endColumn);
                this._setRangeIsMultiLine(range.id, (newTextRange.startLineNumber !== newTextRange.endLineNumber));
            }
        };
        TextModelWithTrackedRanges.prototype.changeTrackedRangeStickiness = function (rangeId, newStickiness) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTrackedRanges.changeTrackedRangeStickiness: Model is disposed');
            }
            if (this._ranges.hasOwnProperty(rangeId)) {
                var range = this._ranges[rangeId];
                this._changeMarkerStickiness(range.startMarkerId, this._shouldStartMarkerSticksToPreviousCharacter(newStickiness));
                this._changeMarkerStickiness(range.endMarkerId, this._shouldEndMarkerSticksToPreviousCharacter(newStickiness));
            }
        };
        TextModelWithTrackedRanges.prototype.isValidTrackedRange = function (rangeId) {
            if (this._isDisposed || !this._ranges) {
                return false;
            }
            return this._ranges.hasOwnProperty(rangeId);
        };
        TextModelWithTrackedRanges.prototype.removeTrackedRange = function (rangeId) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTrackedRanges.removeTrackedRange: Model is disposed');
            }
            if (this._ranges.hasOwnProperty(rangeId)) {
                var range = this._ranges[rangeId];
                this._removeMarker(range.startMarkerId);
                this._removeMarker(range.endMarkerId);
                this._setRangeIsMultiLine(range.id, false);
                delete this._ranges[range.id];
                delete this._markerIdToRangeId[range.startMarkerId];
                delete this._markerIdToRangeId[range.endMarkerId];
            }
        };
        TextModelWithTrackedRanges.prototype.removeTrackedRanges = function (ids) {
            var removeMarkers = [];
            for (var i = 0, len = ids.length; i < len; i++) {
                var rangeId = ids[i];
                if (!this._ranges.hasOwnProperty(rangeId)) {
                    continue;
                }
                var range = this._ranges[rangeId];
                removeMarkers.push(range.startMarkerId);
                removeMarkers.push(range.endMarkerId);
                this._setRangeIsMultiLine(range.id, false);
                delete this._ranges[range.id];
                delete this._markerIdToRangeId[range.startMarkerId];
                delete this._markerIdToRangeId[range.endMarkerId];
            }
            if (removeMarkers.length > 0) {
                this._removeMarkers(removeMarkers);
            }
        };
        TextModelWithTrackedRanges.prototype._newEditorRange = function (startPosition, endPosition) {
            if (endPosition.isBefore(startPosition)) {
                // This tracked range has turned in on itself (end marker before start marker)
                // This can happen in extreme editing conditions where lots of text is removed and lots is added
                // Treat it as a collapsed range
                return new range_1.Range(startPosition.lineNumber, startPosition.column, startPosition.lineNumber, startPosition.column);
            }
            return new range_1.Range(startPosition.lineNumber, startPosition.column, endPosition.lineNumber, endPosition.column);
        };
        TextModelWithTrackedRanges.prototype.getTrackedRange = function (rangeId) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTrackedRanges.getTrackedRange: Model is disposed');
            }
            var range = this._ranges[rangeId];
            var startMarker = this._getMarker(range.startMarkerId);
            var endMarker = this._getMarker(range.endMarkerId);
            return this._newEditorRange(startMarker, endMarker);
        };
        /**
         * Fetch only multi-line ranges that intersect with the given line number range
         */
        TextModelWithTrackedRanges.prototype._getMultiLineTrackedRanges = function (filterStartLineNumber, filterEndLineNumber) {
            var result = [], rangeId, range, startMarker, endMarker;
            for (rangeId in this._multiLineTrackedRanges) {
                if (this._multiLineTrackedRanges.hasOwnProperty(rangeId)) {
                    range = this._ranges[rangeId];
                    startMarker = this._getMarker(range.startMarkerId);
                    if (startMarker.lineNumber > filterEndLineNumber) {
                        continue;
                    }
                    endMarker = this._getMarker(range.endMarkerId);
                    if (endMarker.lineNumber < filterStartLineNumber) {
                        continue;
                    }
                    result.push({
                        id: range.id,
                        range: this._newEditorRange(startMarker, endMarker)
                    });
                }
            }
            return result;
        };
        TextModelWithTrackedRanges.prototype.getLinesTrackedRanges = function (startLineNumber, endLineNumber) {
            if (this._isDisposed) {
                throw new Error('TextModelWithTrackedRanges.getLinesTrackedRanges: Model is disposed');
            }
            var result = this._getMultiLineTrackedRanges(startLineNumber, endLineNumber), resultMap = {}, lineMarkers, lineMarker, rangeId, i, len, lineNumber, startMarker, endMarker;
            for (i = 0, len = result.length; i < len; i++) {
                resultMap[result[i].id] = true;
            }
            for (lineNumber = startLineNumber; lineNumber <= endLineNumber; lineNumber++) {
                lineMarkers = this._getLineMarkers(lineNumber);
                for (i = 0, len = lineMarkers.length; i < len; i++) {
                    lineMarker = lineMarkers[i];
                    if (this._markerIdToRangeId.hasOwnProperty(lineMarker.id)) {
                        rangeId = this._markerIdToRangeId[lineMarker.id];
                        if (!resultMap.hasOwnProperty(rangeId)) {
                            startMarker = this._getMarker(this._ranges[rangeId].startMarkerId);
                            endMarker = this._getMarker(this._ranges[rangeId].endMarkerId);
                            result.push({
                                id: rangeId,
                                range: this._newEditorRange(startMarker, endMarker)
                            });
                            resultMap[rangeId] = true;
                        }
                    }
                }
            }
            return result;
        };
        TextModelWithTrackedRanges.prototype._onChangedMarkers = function (changedMarkers) {
            var changedRanges = {}, changedRange, range, rangeId, marker, i, len;
            for (i = 0, len = changedMarkers.length; i < len; i++) {
                marker = changedMarkers[i];
                if (this._markerIdToRangeId.hasOwnProperty(marker.id)) {
                    rangeId = this._markerIdToRangeId[marker.id];
                    range = this._ranges[rangeId];
                    if (changedRanges.hasOwnProperty(range.id)) {
                        changedRange = changedRanges[range.id];
                    }
                    else {
                        changedRange = {
                            startLineNumber: 0,
                            startColumn: 0,
                            endLineNumber: 0,
                            endColumn: 0
                        };
                        changedRanges[range.id] = changedRange;
                    }
                    if (marker.id === range.startMarkerId) {
                        changedRange.startLineNumber = marker.oldLineNumber;
                        changedRange.startColumn = marker.oldColumn;
                    }
                    else {
                        changedRange.endLineNumber = marker.oldLineNumber;
                        changedRange.endColumn = marker.oldColumn;
                    }
                    this._setRangeIsMultiLine(range.id, (this._getMarker(range.startMarkerId).lineNumber !== this._getMarker(range.endMarkerId).lineNumber));
                }
            }
            return changedRanges;
        };
        return TextModelWithTrackedRanges;
    }(textModelWithMarkers_1.TextModelWithMarkers));
    exports.TextModelWithTrackedRanges = TextModelWithTrackedRanges;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/model/textModelWithDecorations", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/htmlContent', 'vs/base/common/strings', 'vs/editor/common/core/idGenerator', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/model/textModelWithTrackedRanges'], function (require, exports, errors_1, htmlContent_1, strings, idGenerator_1, range_1, editorCommon, textModelWithTrackedRanges_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DeferredEventsBuilder = (function () {
        function DeferredEventsBuilder() {
            this.changedMarkers = {};
            this.oldDecorationRange = {};
            this.oldDecorationOptions = {};
            this.newOrChangedDecorations = {};
            this.removedDecorations = {};
        }
        // --- Build decoration events
        DeferredEventsBuilder.prototype.addNewDecoration = function (id) {
            this.newOrChangedDecorations[id] = true;
        };
        DeferredEventsBuilder.prototype.addRemovedDecoration = function (id, ownerId, range, options) {
            if (this.newOrChangedDecorations.hasOwnProperty(id)) {
                delete this.newOrChangedDecorations[id];
            }
            if (!this.oldDecorationRange.hasOwnProperty(id)) {
                this.oldDecorationRange[id] = range;
            }
            if (!this.oldDecorationOptions.hasOwnProperty(id)) {
                this.oldDecorationOptions[id] = options;
            }
            this.removedDecorations[id] = true;
        };
        DeferredEventsBuilder.prototype.addMovedDecoration = function (id, oldRange) {
            if (!this.oldDecorationRange.hasOwnProperty(id)) {
                this.oldDecorationRange[id] = oldRange;
            }
            this.newOrChangedDecorations[id] = true;
        };
        DeferredEventsBuilder.prototype.addUpdatedDecoration = function (id, oldOptions) {
            if (!this.oldDecorationOptions.hasOwnProperty(id)) {
                this.oldDecorationOptions[id] = oldOptions;
            }
            this.newOrChangedDecorations[id] = true;
        };
        return DeferredEventsBuilder;
    }());
    exports.DeferredEventsBuilder = DeferredEventsBuilder;
    var _INSTANCE_COUNT = 0;
    var TextModelWithDecorations = (function (_super) {
        __extends(TextModelWithDecorations, _super);
        function TextModelWithDecorations(allowedEventTypes, rawText, modeOrPromise) {
            allowedEventTypes.push(editorCommon.EventType.ModelDecorationsChanged);
            _super.call(this, allowedEventTypes, rawText, modeOrPromise);
            // Initialize decorations
            this._decorationIdGenerator = new idGenerator_1.IdGenerator((++_INSTANCE_COUNT) + ';');
            this.decorations = {};
            this.rangeIdToDecorationId = {};
            this._currentDeferredEvents = null;
        }
        TextModelWithDecorations.prototype.dispose = function () {
            this.decorations = null;
            this.rangeIdToDecorationId = null;
            _super.prototype.dispose.call(this);
        };
        TextModelWithDecorations.prototype._resetValue = function (e, newValue) {
            _super.prototype._resetValue.call(this, e, newValue);
            // Destroy all my decorations
            this.decorations = {};
            this.rangeIdToDecorationId = {};
        };
        TextModelWithDecorations.prototype.changeDecorations = function (callback, ownerId) {
            var _this = this;
            if (ownerId === void 0) { ownerId = 0; }
            if (this._isDisposed) {
                throw new Error('TextModelWithDecorations.changeDecorations: Model is disposed');
            }
            return this._withDeferredEvents(function (deferredEventsBuilder) {
                var changeAccessor = {
                    addDecoration: function (range, options) {
                        return _this._addDecorationImpl(deferredEventsBuilder, ownerId, _this.validateRange(range), _normalizeOptions(options));
                    },
                    changeDecoration: function (id, newRange) {
                        _this._changeDecorationImpl(deferredEventsBuilder, id, _this.validateRange(newRange));
                    },
                    changeDecorationOptions: function (id, options) {
                        _this._changeDecorationOptionsImpl(deferredEventsBuilder, id, _normalizeOptions(options));
                    },
                    removeDecoration: function (id) {
                        _this._removeDecorationImpl(deferredEventsBuilder, id);
                    },
                    deltaDecorations: function (oldDecorations, newDecorations) {
                        return _this._deltaDecorationsImpl(deferredEventsBuilder, ownerId, oldDecorations, _this._normalizeDeltaDecorations(newDecorations));
                    }
                };
                var result = null;
                try {
                    result = callback(changeAccessor);
                }
                catch (e) {
                    errors_1.onUnexpectedError(e);
                }
                // Invalidate change accessor
                changeAccessor.addDecoration = null;
                changeAccessor.changeDecoration = null;
                changeAccessor.removeDecoration = null;
                changeAccessor.deltaDecorations = null;
                return result;
            });
        };
        TextModelWithDecorations.prototype.deltaDecorations = function (oldDecorations, newDecorations, ownerId) {
            if (ownerId === void 0) { ownerId = 0; }
            if (this._isDisposed) {
                throw new Error('TextModelWithDecorations.deltaDecorations: Model is disposed');
            }
            if (!oldDecorations) {
                oldDecorations = [];
            }
            return this.changeDecorations(function (changeAccessor) {
                return changeAccessor.deltaDecorations(oldDecorations, newDecorations);
            }, ownerId);
        };
        TextModelWithDecorations.prototype.removeAllDecorationsWithOwnerId = function (ownerId) {
            if (this._isDisposed) {
                throw new Error('TextModelWithDecorations.removeAllDecorationsWithOwnerId: Model is disposed');
            }
            var decorationId;
            var decoration;
            var toRemove = [];
            for (decorationId in this.decorations) {
                if (this.decorations.hasOwnProperty(decorationId)) {
                    decoration = this.decorations[decorationId];
                    if (decoration.ownerId === ownerId) {
                        toRemove.push(decoration.id);
                    }
                }
            }
            this._removeDecorationsImpl(null, toRemove);
        };
        TextModelWithDecorations.prototype.getDecorationOptions = function (decorationId) {
            if (this._isDisposed) {
                throw new Error('TextModelWithDecorations.getDecorationOptions: Model is disposed');
            }
            if (this.decorations.hasOwnProperty(decorationId)) {
                return this.decorations[decorationId].options;
            }
            return null;
        };
        TextModelWithDecorations.prototype.getDecorationRange = function (decorationId) {
            if (this._isDisposed) {
                throw new Error('TextModelWithDecorations.getDecorationRange: Model is disposed');
            }
            if (this.decorations.hasOwnProperty(decorationId)) {
                var decoration = this.decorations[decorationId];
                return this.getTrackedRange(decoration.rangeId);
            }
            return null;
        };
        TextModelWithDecorations.prototype.getLineDecorations = function (lineNumber, ownerId, filterOutValidation) {
            if (ownerId === void 0) { ownerId = 0; }
            if (filterOutValidation === void 0) { filterOutValidation = false; }
            if (this._isDisposed) {
                throw new Error('TextModelWithDecorations.getLineDecorations: Model is disposed');
            }
            if (lineNumber < 1 || lineNumber > this.getLineCount()) {
                return [];
            }
            return this.getLinesDecorations(lineNumber, lineNumber, ownerId, filterOutValidation);
        };
        TextModelWithDecorations.prototype._getDecorationsInRange = function (startLineNumber, startColumn, endLineNumber, endColumn, ownerId, filterOutValidation) {
            var result = [], decoration, lineRanges = this.getLinesTrackedRanges(startLineNumber, endLineNumber), i, lineRange, len;
            for (i = 0, len = lineRanges.length; i < len; i++) {
                lineRange = lineRanges[i];
                // Look at line range only if there is a corresponding decoration for it
                if (this.rangeIdToDecorationId.hasOwnProperty(lineRange.id)) {
                    decoration = this.decorations[this.rangeIdToDecorationId[lineRange.id]];
                    if (ownerId && decoration.ownerId && decoration.ownerId !== ownerId) {
                        continue;
                    }
                    if (filterOutValidation) {
                        if (decoration.options.className === editorCommon.ClassName.EditorErrorDecoration || decoration.options.className === editorCommon.ClassName.EditorWarningDecoration) {
                            continue;
                        }
                    }
                    if (lineRange.range.startLineNumber === startLineNumber && lineRange.range.endColumn < startColumn) {
                        continue;
                    }
                    if (lineRange.range.endLineNumber === endLineNumber && lineRange.range.startColumn > endColumn) {
                        continue;
                    }
                    result.push({
                        id: decoration.id,
                        ownerId: decoration.ownerId,
                        range: lineRange.range,
                        options: decoration.options
                    });
                }
            }
            return result;
        };
        TextModelWithDecorations.prototype.getLinesDecorations = function (startLineNumber, endLineNumber, ownerId, filterOutValidation) {
            if (ownerId === void 0) { ownerId = 0; }
            if (filterOutValidation === void 0) { filterOutValidation = false; }
            if (this._isDisposed) {
                throw new Error('TextModelWithDecorations.getLinesDecorations: Model is disposed');
            }
            var lineCount = this.getLineCount();
            startLineNumber = Math.min(lineCount, Math.max(1, startLineNumber));
            endLineNumber = Math.min(lineCount, Math.max(1, endLineNumber));
            return this._getDecorationsInRange(startLineNumber, 1, endLineNumber, Number.MAX_VALUE, ownerId, filterOutValidation);
        };
        TextModelWithDecorations.prototype.getDecorationsInRange = function (range, ownerId, filterOutValidation) {
            if (this._isDisposed) {
                throw new Error('TextModelWithDecorations.getDecorationsInRange: Model is disposed');
            }
            var validatedRange = this.validateRange(range);
            return this._getDecorationsInRange(validatedRange.startLineNumber, validatedRange.startColumn, validatedRange.endLineNumber, validatedRange.endColumn, ownerId, filterOutValidation);
        };
        TextModelWithDecorations.prototype.getAllDecorations = function (ownerId, filterOutValidation) {
            if (ownerId === void 0) { ownerId = 0; }
            if (filterOutValidation === void 0) { filterOutValidation = false; }
            if (this._isDisposed) {
                throw new Error('TextModelWithDecorations.getAllDecorations: Model is disposed');
            }
            var result = [];
            var decorationId;
            var decoration;
            for (decorationId in this.decorations) {
                if (this.decorations.hasOwnProperty(decorationId)) {
                    decoration = this.decorations[decorationId];
                    if (ownerId && decoration.ownerId && decoration.ownerId !== ownerId) {
                        continue;
                    }
                    if (filterOutValidation) {
                        if (decoration.options.className === editorCommon.ClassName.EditorErrorDecoration || decoration.options.className === editorCommon.ClassName.EditorWarningDecoration) {
                            continue;
                        }
                    }
                    result.push({
                        id: decoration.id,
                        ownerId: decoration.ownerId,
                        range: this.getTrackedRange(decoration.rangeId),
                        options: decoration.options
                    });
                }
            }
            return result;
        };
        TextModelWithDecorations.prototype._withDeferredEvents = function (callback) {
            var _this = this;
            return this.deferredEmit(function () {
                var createDeferredEvents = _this._currentDeferredEvents ? false : true;
                if (createDeferredEvents) {
                    _this._currentDeferredEvents = new DeferredEventsBuilder();
                }
                try {
                    var result = callback(_this._currentDeferredEvents);
                    if (createDeferredEvents) {
                        _this._handleCollectedEvents(_this._currentDeferredEvents);
                    }
                }
                finally {
                    if (createDeferredEvents) {
                        _this._currentDeferredEvents = null;
                    }
                }
                return result;
            });
        };
        TextModelWithDecorations.prototype._handleCollectedEvents = function (b) {
            // Normalize changed markers into an array
            var changedMarkers = this._getMarkersInMap(b.changedMarkers);
            // Collect changed tracked ranges
            var changedRanges = this._onChangedMarkers(changedMarkers);
            // Collect decoration change events with the deferred event builder
            this._onChangedRanges(b, changedRanges);
            // Emit a single decorations changed event
            this._handleCollectedDecorationsEvents(b);
            // Reset markers for next round of events
            for (var i = 0, len = changedMarkers.length; i < len; i++) {
                changedMarkers[i].oldLineNumber = 0;
                changedMarkers[i].oldColumn = 0;
            }
        };
        TextModelWithDecorations.prototype._onChangedRanges = function (eventBuilder, changedRanges) {
            var rangeId;
            var decorationId;
            for (rangeId in changedRanges) {
                if (changedRanges.hasOwnProperty(rangeId) && this.rangeIdToDecorationId.hasOwnProperty(rangeId)) {
                    decorationId = this.rangeIdToDecorationId[rangeId];
                    eventBuilder.addMovedDecoration(decorationId, changedRanges[rangeId]);
                }
            }
        };
        TextModelWithDecorations.prototype._handleCollectedDecorationsEvents = function (b) {
            var decorationId, addedOrChangedDecorations = [], removedDecorations = [], decorationIds = [], decorationData, oldRange;
            for (decorationId in b.newOrChangedDecorations) {
                if (b.newOrChangedDecorations.hasOwnProperty(decorationId)) {
                    decorationIds.push(decorationId);
                    decorationData = this._getDecorationData(decorationId);
                    decorationData.isForValidation = (decorationData.options.className === editorCommon.ClassName.EditorErrorDecoration || decorationData.options.className === editorCommon.ClassName.EditorWarningDecoration);
                    addedOrChangedDecorations.push(decorationData);
                    if (b.oldDecorationRange.hasOwnProperty(decorationId)) {
                        oldRange = b.oldDecorationRange[decorationId];
                        oldRange.startLineNumber = oldRange.startLineNumber || decorationData.range.startLineNumber;
                        oldRange.startColumn = oldRange.startColumn || decorationData.range.startColumn;
                        oldRange.endLineNumber = oldRange.endLineNumber || decorationData.range.endLineNumber;
                        oldRange.endColumn = oldRange.endColumn || decorationData.range.endColumn;
                    }
                }
            }
            for (decorationId in b.removedDecorations) {
                if (b.removedDecorations.hasOwnProperty(decorationId)) {
                    decorationIds.push(decorationId);
                    removedDecorations.push(decorationId);
                }
            }
            if (decorationIds.length > 0) {
                var e = {
                    ids: decorationIds,
                    addedOrChangedDecorations: addedOrChangedDecorations,
                    removedDecorations: removedDecorations,
                    oldOptions: b.oldDecorationOptions,
                    oldRanges: b.oldDecorationRange
                };
                this.emitModelDecorationsChangedEvent(e);
            }
        };
        TextModelWithDecorations.prototype._getDecorationData = function (decorationId) {
            var decoration = this.decorations[decorationId];
            return {
                id: decoration.id,
                ownerId: decoration.ownerId,
                range: this.getTrackedRange(decoration.rangeId),
                isForValidation: false,
                options: decoration.options
            };
        };
        TextModelWithDecorations.prototype.emitModelDecorationsChangedEvent = function (e) {
            if (!this._isDisposing) {
                this.emit(editorCommon.EventType.ModelDecorationsChanged, e);
            }
        };
        TextModelWithDecorations.prototype._normalizeDeltaDecorations = function (deltaDecorations) {
            var result = [];
            for (var i = 0, len = deltaDecorations.length; i < len; i++) {
                var deltaDecoration = deltaDecorations[i];
                result.push(new ModelDeltaDecoration(i, this.validateRange(deltaDecoration.range), _normalizeOptions(deltaDecoration.options)));
            }
            return result;
        };
        TextModelWithDecorations.prototype._addDecorationImpl = function (eventBuilder, ownerId, range, options) {
            var rangeId = this.addTrackedRange(range, options.stickiness);
            var decoration = new ModelInternalDecoration(this._decorationIdGenerator.generate(), ownerId, rangeId, options);
            this.decorations[decoration.id] = decoration;
            this.rangeIdToDecorationId[rangeId] = decoration.id;
            eventBuilder.addNewDecoration(decoration.id);
            return decoration.id;
        };
        TextModelWithDecorations.prototype._addDecorationsImpl = function (eventBuilder, ownerId, newDecorations) {
            var rangeIds = this._addTrackedRanges(newDecorations.map(function (d) { return d.range; }), newDecorations.map(function (d) { return d.options.stickiness; }));
            var result = [];
            for (var i = 0, len = newDecorations.length; i < len; i++) {
                var rangeId = rangeIds[i];
                var decoration = new ModelInternalDecoration(this._decorationIdGenerator.generate(), ownerId, rangeId, newDecorations[i].options);
                this.decorations[decoration.id] = decoration;
                this.rangeIdToDecorationId[rangeId] = decoration.id;
                eventBuilder.addNewDecoration(decoration.id);
                result.push(decoration.id);
            }
            return result;
        };
        TextModelWithDecorations.prototype._changeDecorationImpl = function (eventBuilder, id, newRange) {
            if (this.decorations.hasOwnProperty(id)) {
                var decoration = this.decorations[id];
                var oldRange = this.getTrackedRange(decoration.rangeId);
                this.changeTrackedRange(decoration.rangeId, newRange);
                eventBuilder.addMovedDecoration(id, oldRange);
            }
        };
        TextModelWithDecorations.prototype._changeDecorationOptionsImpl = function (eventBuilder, id, options) {
            if (this.decorations.hasOwnProperty(id)) {
                var decoration = this.decorations[id];
                var oldOptions = decoration.options;
                if (oldOptions.stickiness !== options.stickiness) {
                    this.changeTrackedRangeStickiness(decoration.rangeId, options.stickiness);
                }
                decoration.options = options;
                eventBuilder.addUpdatedDecoration(id, oldOptions);
            }
        };
        TextModelWithDecorations.prototype._removeDecorationImpl = function (eventBuilder, id) {
            if (this.decorations.hasOwnProperty(id)) {
                var decoration = this.decorations[id];
                var oldRange = null;
                if (eventBuilder) {
                    oldRange = this.getTrackedRange(decoration.rangeId);
                }
                this.removeTrackedRange(decoration.rangeId);
                delete this.rangeIdToDecorationId[decoration.rangeId];
                delete this.decorations[id];
                if (eventBuilder) {
                    eventBuilder.addRemovedDecoration(id, decoration.ownerId, oldRange, decoration.options);
                }
            }
        };
        TextModelWithDecorations.prototype._removeDecorationsImpl = function (eventBuilder, ids) {
            var removeTrackedRanges = [];
            for (var i = 0, len = ids.length; i < len; i++) {
                var id = ids[i];
                if (!this.decorations.hasOwnProperty(id)) {
                    continue;
                }
                var decoration = this.decorations[id];
                if (eventBuilder) {
                    var oldRange = this.getTrackedRange(decoration.rangeId);
                    eventBuilder.addRemovedDecoration(id, decoration.ownerId, oldRange, decoration.options);
                }
                removeTrackedRanges.push(decoration.rangeId);
                delete this.rangeIdToDecorationId[decoration.rangeId];
                delete this.decorations[id];
            }
            if (removeTrackedRanges.length > 0) {
                this.removeTrackedRanges(removeTrackedRanges);
            }
        };
        TextModelWithDecorations.prototype._resolveOldDecorations = function (oldDecorations) {
            var result = [];
            for (var i = 0, len = oldDecorations.length; i < len; i++) {
                var id = oldDecorations[i];
                if (!this.decorations.hasOwnProperty(id)) {
                    continue;
                }
                var decoration = this.decorations[id];
                result.push({
                    id: id,
                    range: this.getTrackedRange(decoration.rangeId),
                    options: decoration.options
                });
            }
            return result;
        };
        TextModelWithDecorations.prototype._deltaDecorationsImpl = function (eventBuilder, ownerId, oldDecorationsIds, newDecorations) {
            if (oldDecorationsIds.length === 0) {
                // Nothing to remove
                return this._addDecorationsImpl(eventBuilder, ownerId, newDecorations);
            }
            if (newDecorations.length === 0) {
                // Nothing to add
                this._removeDecorationsImpl(eventBuilder, oldDecorationsIds);
                return [];
            }
            var oldDecorations = this._resolveOldDecorations(oldDecorationsIds);
            oldDecorations.sort(function (a, b) { return range_1.Range.compareRangesUsingStarts(a.range, b.range); });
            newDecorations.sort(function (a, b) { return range_1.Range.compareRangesUsingStarts(a.range, b.range); });
            var result = [], oldDecorationsIndex = 0, oldDecorationsLength = oldDecorations.length, newDecorationsIndex = 0, newDecorationsLength = newDecorations.length, decorationsToAdd = [], decorationsToRemove = [];
            while (oldDecorationsIndex < oldDecorationsLength && newDecorationsIndex < newDecorationsLength) {
                var oldDecoration = oldDecorations[oldDecorationsIndex];
                var newDecoration = newDecorations[newDecorationsIndex];
                var comparison = range_1.Range.compareRangesUsingStarts(oldDecoration.range, newDecoration.range);
                if (comparison < 0) {
                    // `oldDecoration` is before `newDecoration` => remove `oldDecoration`
                    decorationsToRemove.push(oldDecoration.id);
                    oldDecorationsIndex++;
                    continue;
                }
                if (comparison > 0) {
                    // `newDecoration` is before `oldDecoration` => add `newDecoration`
                    decorationsToAdd.push(newDecoration);
                    newDecorationsIndex++;
                    continue;
                }
                // The ranges of `oldDecoration` and `newDecoration` are equal
                if (!oldDecoration.options.equals(newDecoration.options)) {
                    // The options do not match => remove `oldDecoration`
                    decorationsToRemove.push(oldDecoration.id);
                    oldDecorationsIndex++;
                    continue;
                }
                // Bingo! We can reuse `oldDecoration` for `newDecoration`
                result[newDecoration.index] = oldDecoration.id;
                oldDecorationsIndex++;
                newDecorationsIndex++;
            }
            while (oldDecorationsIndex < oldDecorationsLength) {
                // No more new decorations => remove decoration at `oldDecorationsIndex`
                decorationsToRemove.push(oldDecorations[oldDecorationsIndex].id);
                oldDecorationsIndex++;
            }
            while (newDecorationsIndex < newDecorationsLength) {
                // No more old decorations => add decoration at `newDecorationsIndex`
                decorationsToAdd.push(newDecorations[newDecorationsIndex]);
                newDecorationsIndex++;
            }
            // Remove `decorationsToRemove`
            if (decorationsToRemove.length > 0) {
                this._removeDecorationsImpl(eventBuilder, decorationsToRemove);
            }
            // Add `decorationsToAdd`
            if (decorationsToAdd.length > 0) {
                var newIds = this._addDecorationsImpl(eventBuilder, ownerId, decorationsToAdd);
                for (var i = 0, len = decorationsToAdd.length; i < len; i++) {
                    result[decorationsToAdd[i].index] = newIds[i];
                }
            }
            return result;
        };
        return TextModelWithDecorations;
    }(textModelWithTrackedRanges_1.TextModelWithTrackedRanges));
    exports.TextModelWithDecorations = TextModelWithDecorations;
    function cleanClassName(className) {
        return className.replace(/[^a-z0-9\-]/gi, ' ');
    }
    var ModelInternalDecoration = (function () {
        function ModelInternalDecoration(id, ownerId, rangeId, options) {
            this.id = id;
            this.ownerId = ownerId;
            this.rangeId = rangeId;
            this.options = options;
        }
        return ModelInternalDecoration;
    }());
    var ModelDecorationOptions = (function () {
        function ModelDecorationOptions(options) {
            this.stickiness = options.stickiness || editorCommon.TrackedRangeStickiness.AlwaysGrowsWhenTypingAtEdges;
            this.className = cleanClassName(options.className || strings.empty);
            this.hoverMessage = options.hoverMessage || strings.empty;
            this.htmlMessage = options.htmlMessage || [];
            this.isWholeLine = options.isWholeLine || false;
            this.overviewRuler = _normalizeOverviewRulerOptions(options.overviewRuler, options.showInOverviewRuler);
            this.glyphMarginClassName = cleanClassName(options.glyphMarginClassName || strings.empty);
            this.linesDecorationsClassName = cleanClassName(options.linesDecorationsClassName || strings.empty);
            this.inlineClassName = cleanClassName(options.inlineClassName || strings.empty);
        }
        ModelDecorationOptions._overviewRulerEquals = function (a, b) {
            return (a.color === b.color
                && a.position === b.position
                && a.darkColor === b.darkColor);
        };
        ModelDecorationOptions.prototype.equals = function (other) {
            return (this.stickiness === other.stickiness
                && this.className === other.className
                && this.hoverMessage === other.hoverMessage
                && this.isWholeLine === other.isWholeLine
                && this.showInOverviewRuler === other.showInOverviewRuler
                && this.glyphMarginClassName === other.glyphMarginClassName
                && this.linesDecorationsClassName === other.linesDecorationsClassName
                && this.inlineClassName === other.inlineClassName
                && htmlContent_1.htmlContentElementArrEquals(this.htmlMessage, other.htmlMessage)
                && ModelDecorationOptions._overviewRulerEquals(this.overviewRuler, other.overviewRuler));
        };
        return ModelDecorationOptions;
    }());
    var ModelDeltaDecoration = (function () {
        function ModelDeltaDecoration(index, range, options) {
            this.index = index;
            this.range = range;
            this.options = options;
        }
        return ModelDeltaDecoration;
    }());
    function _normalizeOptions(options) {
        return new ModelDecorationOptions(options);
    }
    var ModelDecorationOverviewRulerOptions = (function () {
        function ModelDecorationOverviewRulerOptions(options, legacyShowInOverviewRuler) {
            this.color = strings.empty;
            this.darkColor = strings.empty;
            this.position = editorCommon.OverviewRulerLane.Center;
            if (legacyShowInOverviewRuler) {
                this.color = legacyShowInOverviewRuler;
            }
            if (options && options.color) {
                this.color = options.color;
            }
            if (options && options.darkColor) {
                this.darkColor = options.darkColor;
            }
            if (options && options.hasOwnProperty('position')) {
                this.position = options.position;
            }
        }
        return ModelDecorationOverviewRulerOptions;
    }());
    function _normalizeOverviewRulerOptions(options, legacyShowInOverviewRuler) {
        if (legacyShowInOverviewRuler === void 0) { legacyShowInOverviewRuler = null; }
        return new ModelDecorationOverviewRulerOptions(options, legacyShowInOverviewRuler);
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/model/editableTextModel", ["require", "exports", 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/model/editStack', 'vs/editor/common/model/modelLine', 'vs/editor/common/model/textModelWithDecorations'], function (require, exports, range_1, editorCommon, editStack_1, modelLine_1, textModelWithDecorations_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EditableTextModel = (function (_super) {
        __extends(EditableTextModel, _super);
        function EditableTextModel(allowedEventTypes, rawText, modeOrPromise) {
            allowedEventTypes.push(editorCommon.EventType.ModelContentChanged);
            allowedEventTypes.push(editorCommon.EventType.ModelContentChanged2);
            _super.call(this, allowedEventTypes, rawText, modeOrPromise);
            this._commandManager = new editStack_1.EditStack(this);
            this._isUndoing = false;
            this._isRedoing = false;
            this._hasEditableRange = false;
            this._editableRangeId = null;
        }
        EditableTextModel.prototype.dispose = function () {
            this._commandManager = null;
            _super.prototype.dispose.call(this);
        };
        EditableTextModel.prototype._resetValue = function (e, newValue) {
            _super.prototype._resetValue.call(this, e, newValue);
            // Destroy my edit history and settings
            this._commandManager = new editStack_1.EditStack(this);
            this._hasEditableRange = false;
            this._editableRangeId = null;
        };
        EditableTextModel.prototype.pushStackElement = function () {
            if (this._isDisposed) {
                throw new Error('EditableTextModel.pushStackElement: Model is disposed');
            }
            this._commandManager.pushStackElement();
        };
        EditableTextModel.prototype.pushEditOperations = function (beforeCursorState, editOperations, cursorStateComputer) {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('EditableTextModel.pushEditOperations: Model is disposed');
            }
            return this.deferredEmit(function () {
                return _this._commandManager.pushEditOperation(beforeCursorState, editOperations, cursorStateComputer);
            });
        };
        /**
         * Transform operations such that they represent the same logic edit,
         * but that they also do not cause OOM crashes.
         */
        EditableTextModel.prototype._reduceOperations = function (operations) {
            if (operations.length < 1000) {
                // We know from empirical testing that a thousand edits work fine regardless of their shape.
                return operations;
            }
            // At one point, due to how events are emitted and how each operation is handled,
            // some operations can trigger a high ammount of temporary string allocations,
            // that will immediately get edited again.
            // e.g. a formatter inserting ridiculous ammounts of \n on a model with a single line
            // Therefore, the strategy is to collapse all the operations into a huge single edit operation
            return [this._toSingleEditOperation(operations)];
        };
        EditableTextModel.prototype._toSingleEditOperation = function (operations) {
            var forceMoveMarkers = false, firstEditRange = operations[0].range, lastEditRange = operations[operations.length - 1].range, entireEditRange = new range_1.Range(firstEditRange.startLineNumber, firstEditRange.startColumn, lastEditRange.endLineNumber, lastEditRange.endColumn), lastEndLineNumber = firstEditRange.startLineNumber, lastEndColumn = firstEditRange.startColumn, result = [];
            for (var i = 0, len = operations.length; i < len; i++) {
                var operation = operations[i], range = operation.range;
                forceMoveMarkers = forceMoveMarkers || operation.forceMoveMarkers;
                // (1) -- Push old text
                for (var lineNumber = lastEndLineNumber; lineNumber < range.startLineNumber; lineNumber++) {
                    if (lineNumber === lastEndLineNumber) {
                        result.push(this._lines[lineNumber - 1].text.substring(lastEndColumn - 1));
                    }
                    else {
                        result.push('\n');
                        result.push(this._lines[lineNumber - 1].text);
                    }
                }
                if (range.startLineNumber === lastEndLineNumber) {
                    result.push(this._lines[range.startLineNumber - 1].text.substring(lastEndColumn - 1, range.startColumn - 1));
                }
                else {
                    result.push('\n');
                    result.push(this._lines[range.startLineNumber - 1].text.substring(0, range.startColumn - 1));
                }
                // (2) -- Push new text
                if (operation.lines) {
                    for (var j = 0, lenJ = operation.lines.length; j < lenJ; j++) {
                        if (j !== 0) {
                            result.push('\n');
                        }
                        result.push(operation.lines[j]);
                    }
                }
                lastEndLineNumber = operation.range.endLineNumber;
                lastEndColumn = operation.range.endColumn;
            }
            return {
                sortIndex: 0,
                identifier: operations[0].identifier,
                range: entireEditRange,
                rangeLength: this.getValueLengthInRange(entireEditRange),
                lines: result.join('').split('\n'),
                forceMoveMarkers: forceMoveMarkers
            };
        };
        EditableTextModel._sortOpsAscending = function (a, b) {
            var r = range_1.Range.compareRangesUsingEnds(a.range, b.range);
            if (r === 0) {
                return a.sortIndex - b.sortIndex;
            }
            return r;
        };
        EditableTextModel._sortOpsDescending = function (a, b) {
            var r = range_1.Range.compareRangesUsingEnds(a.range, b.range);
            if (r === 0) {
                return b.sortIndex - a.sortIndex;
            }
            return -r;
        };
        EditableTextModel.prototype.applyEdits = function (rawOperations) {
            if (rawOperations.length === 0) {
                return [];
            }
            var operations = [];
            for (var i = 0; i < rawOperations.length; i++) {
                var op = rawOperations[i];
                var validatedRange = this.validateRange(op.range);
                operations[i] = {
                    sortIndex: i,
                    identifier: op.identifier,
                    range: validatedRange,
                    rangeLength: this.getValueLengthInRange(validatedRange),
                    lines: op.text ? op.text.split(/\r\n|\r|\n/) : null,
                    forceMoveMarkers: op.forceMoveMarkers
                };
            }
            // Sort operations ascending
            operations.sort(EditableTextModel._sortOpsAscending);
            for (var i = 0, count = operations.length - 1; i < count; i++) {
                var rangeEnd = operations[i].range.getEndPosition();
                var nextRangeStart = operations[i + 1].range.getStartPosition();
                if (nextRangeStart.isBefore(rangeEnd)) {
                    // overlapping ranges
                    throw new Error('Overlapping ranges are not allowed!');
                }
            }
            operations = this._reduceOperations(operations);
            var editableRange = this.getEditableRange();
            var editableRangeStart = editableRange.getStartPosition();
            var editableRangeEnd = editableRange.getEndPosition();
            for (var i = 0; i < operations.length; i++) {
                var operationRange = operations[i].range;
                if (!editableRangeStart.isBeforeOrEqual(operationRange.getStartPosition()) || !operationRange.getEndPosition().isBeforeOrEqual(editableRangeEnd)) {
                    throw new Error('Editing outside of editable range not allowed!');
                }
            }
            // Delta encode operations
            var reverseRanges = EditableTextModel._getInverseEditRanges(operations);
            var reverseOperations = [];
            for (var i = 0; i < operations.length; i++) {
                reverseOperations[i] = {
                    identifier: operations[i].identifier,
                    range: reverseRanges[i],
                    text: this.getValueInRange(operations[i].range),
                    forceMoveMarkers: operations[i].forceMoveMarkers
                };
            }
            this._applyEdits(operations);
            return reverseOperations;
        };
        /**
         * Assumes `operations` are validated and sorted ascending
         */
        EditableTextModel._getInverseEditRanges = function (operations) {
            var result = [];
            var prevOpEndLineNumber;
            var prevOpEndColumn;
            var prevOp = null;
            for (var i = 0, len = operations.length; i < len; i++) {
                var op = operations[i];
                var startLineNumber = void 0;
                var startColumn = void 0;
                if (prevOp) {
                    if (prevOp.range.endLineNumber === op.range.startLineNumber) {
                        startLineNumber = prevOpEndLineNumber;
                        startColumn = prevOpEndColumn + (op.range.startColumn - prevOp.range.endColumn);
                    }
                    else {
                        startLineNumber = prevOpEndLineNumber + (op.range.startLineNumber - prevOp.range.endLineNumber);
                        startColumn = op.range.startColumn;
                    }
                }
                else {
                    startLineNumber = op.range.startLineNumber;
                    startColumn = op.range.startColumn;
                }
                var resultRange = void 0;
                if (op.lines && op.lines.length > 0) {
                    // the operation inserts something
                    var lineCount = op.lines.length;
                    var firstLine = op.lines[0];
                    var lastLine = op.lines[lineCount - 1];
                    if (lineCount === 1) {
                        // single line insert
                        resultRange = new range_1.Range(startLineNumber, startColumn, startLineNumber, startColumn + firstLine.length);
                    }
                    else {
                        // multi line insert
                        resultRange = new range_1.Range(startLineNumber, startColumn, startLineNumber + lineCount - 1, lastLine.length + 1);
                    }
                }
                else {
                    // There is nothing to insert
                    resultRange = new range_1.Range(startLineNumber, startColumn, startLineNumber, startColumn);
                }
                prevOpEndLineNumber = resultRange.endLineNumber;
                prevOpEndColumn = resultRange.endColumn;
                result.push(resultRange);
                prevOp = op;
            }
            return result;
        };
        EditableTextModel.prototype._applyEdits = function (operations) {
            var _this = this;
            // Sort operations descending
            operations.sort(EditableTextModel._sortOpsDescending);
            this._withDeferredEvents(function (deferredEventsBuilder) {
                var contentChangedEvents = [];
                var contentChanged2Events = [];
                var lineEditsQueue = [];
                var queueLineEdit = function (lineEdit) {
                    if (lineEdit.startColumn === lineEdit.endColumn && lineEdit.text.length === 0) {
                        // empty edit => ignore it
                        return;
                    }
                    lineEditsQueue.push(lineEdit);
                };
                var flushLineEdits = function () {
                    if (lineEditsQueue.length === 0) {
                        return;
                    }
                    lineEditsQueue.reverse();
                    // `lineEditsQueue` now contains edits from smaller (line number,column) to larger (line number,column)
                    var currentLineNumber = lineEditsQueue[0].lineNumber, currentLineNumberStart = 0;
                    for (var i = 1, len = lineEditsQueue.length; i < len; i++) {
                        var lineNumber = lineEditsQueue[i].lineNumber;
                        if (lineNumber === currentLineNumber) {
                            continue;
                        }
                        _this._invalidateLine(currentLineNumber - 1);
                        _this._lines[currentLineNumber - 1].applyEdits(deferredEventsBuilder.changedMarkers, lineEditsQueue.slice(currentLineNumberStart, i));
                        contentChangedEvents.push(_this._createLineChangedEvent(currentLineNumber));
                        currentLineNumber = lineNumber;
                        currentLineNumberStart = i;
                    }
                    _this._invalidateLine(currentLineNumber - 1);
                    _this._lines[currentLineNumber - 1].applyEdits(deferredEventsBuilder.changedMarkers, lineEditsQueue.slice(currentLineNumberStart, lineEditsQueue.length));
                    contentChangedEvents.push(_this._createLineChangedEvent(currentLineNumber));
                    lineEditsQueue = [];
                };
                var minTouchedLineNumber = operations[operations.length - 1].range.startLineNumber;
                var maxTouchedLineNumber = operations[0].range.endLineNumber + 1;
                var totalLinesCountDelta = 0;
                for (var i = 0, len = operations.length; i < len; i++) {
                    var op = operations[i];
                    // console.log();
                    // console.log('-------------------');
                    // console.log('OPERATION #' + (i));
                    // console.log('op: ', op);
                    // console.log('<<<\n' + this._lines.map(l => l.text).join('\n') + '\n>>>');
                    var startLineNumber = op.range.startLineNumber;
                    var startColumn = op.range.startColumn;
                    var endLineNumber = op.range.endLineNumber;
                    var endColumn = op.range.endColumn;
                    if (startLineNumber === endLineNumber && startColumn === endColumn && (!op.lines || op.lines.length === 0)) {
                        // no-op
                        continue;
                    }
                    var deletingLinesCnt = endLineNumber - startLineNumber;
                    var insertingLinesCnt = (op.lines ? op.lines.length - 1 : 0);
                    var editingLinesCnt = Math.min(deletingLinesCnt, insertingLinesCnt);
                    totalLinesCountDelta += (insertingLinesCnt - deletingLinesCnt);
                    // Iterating descending to overlap with previous op
                    // in case there are common lines being edited in both
                    for (var j = editingLinesCnt; j >= 0; j--) {
                        var editLineNumber = startLineNumber + j;
                        queueLineEdit({
                            lineNumber: editLineNumber,
                            startColumn: (editLineNumber === startLineNumber ? startColumn : 1),
                            endColumn: (editLineNumber === endLineNumber ? endColumn : _this.getLineMaxColumn(editLineNumber)),
                            text: (op.lines ? op.lines[j] : ''),
                            forceMoveMarkers: op.forceMoveMarkers
                        });
                    }
                    if (editingLinesCnt < deletingLinesCnt) {
                        // Must delete some lines
                        // Flush any pending line edits
                        flushLineEdits();
                        var spliceStartLineNumber = startLineNumber + editingLinesCnt;
                        var spliceStartColumn = _this.getLineMaxColumn(spliceStartLineNumber);
                        var endLineRemains = _this._lines[endLineNumber - 1].split(deferredEventsBuilder.changedMarkers, endColumn, false);
                        _this._invalidateLine(spliceStartLineNumber - 1);
                        var spliceCnt = endLineNumber - spliceStartLineNumber;
                        // Collect all these markers
                        var markersOnDeletedLines = [];
                        for (var j = 0; j < spliceCnt; j++) {
                            var deleteLineIndex = spliceStartLineNumber + j;
                            markersOnDeletedLines = markersOnDeletedLines.concat(_this._lines[deleteLineIndex].deleteLine(deferredEventsBuilder.changedMarkers, spliceStartColumn, deleteLineIndex + 1));
                        }
                        _this._lines.splice(spliceStartLineNumber, spliceCnt);
                        // Reconstruct first line
                        _this._lines[spliceStartLineNumber - 1].append(deferredEventsBuilder.changedMarkers, endLineRemains);
                        _this._lines[spliceStartLineNumber - 1].addMarkers(markersOnDeletedLines);
                        contentChangedEvents.push(_this._createLineChangedEvent(spliceStartLineNumber));
                        contentChangedEvents.push(_this._createLinesDeletedEvent(spliceStartLineNumber + 1, spliceStartLineNumber + spliceCnt));
                    }
                    if (editingLinesCnt < insertingLinesCnt) {
                        // Must insert some lines
                        // Flush any pending line edits
                        flushLineEdits();
                        var spliceLineNumber = startLineNumber + editingLinesCnt;
                        var spliceColumn = (spliceLineNumber === startLineNumber ? startColumn : 1);
                        if (op.lines) {
                            spliceColumn += op.lines[editingLinesCnt].length;
                        }
                        // Split last line
                        var leftoverLine = _this._lines[spliceLineNumber - 1].split(deferredEventsBuilder.changedMarkers, spliceColumn, op.forceMoveMarkers);
                        contentChangedEvents.push(_this._createLineChangedEvent(spliceLineNumber));
                        _this._invalidateLine(spliceLineNumber - 1);
                        // Lines in the middle
                        var newLinesContent = [];
                        for (var j = editingLinesCnt + 1; j <= insertingLinesCnt; j++) {
                            var newLineNumber = startLineNumber + j;
                            _this._lines.splice(newLineNumber - 1, 0, new modelLine_1.ModelLine(newLineNumber, op.lines[j]));
                            newLinesContent.push(op.lines[j]);
                        }
                        newLinesContent[newLinesContent.length - 1] += leftoverLine.text;
                        // Last line
                        _this._lines[startLineNumber + insertingLinesCnt - 1].append(deferredEventsBuilder.changedMarkers, leftoverLine);
                        contentChangedEvents.push(_this._createLinesInsertedEvent(spliceLineNumber + 1, startLineNumber + insertingLinesCnt, newLinesContent.join('\n')));
                    }
                    contentChanged2Events.push({
                        range: new range_1.Range(startLineNumber, startColumn, endLineNumber, endColumn),
                        rangeLength: op.rangeLength,
                        text: op.lines ? op.lines.join(_this.getEOL()) : '',
                        eol: _this._EOL,
                        versionId: -1,
                        isUndoing: _this._isUndoing,
                        isRedoing: _this._isRedoing
                    });
                }
                flushLineEdits();
                maxTouchedLineNumber = Math.max(1, Math.min(_this.getLineCount(), maxTouchedLineNumber + totalLinesCountDelta));
                if (totalLinesCountDelta !== 0) {
                    // must update line numbers all the way to the bottom
                    maxTouchedLineNumber = _this.getLineCount();
                }
                for (var lineNumber = minTouchedLineNumber; lineNumber <= maxTouchedLineNumber; lineNumber++) {
                    _this._lines[lineNumber - 1].updateLineNumber(deferredEventsBuilder.changedMarkers, lineNumber);
                }
                if (contentChangedEvents.length !== 0 || contentChanged2Events.length !== 0) {
                    if (contentChangedEvents.length === 0) {
                        // Fabricate a fake line changed event to get an event out
                        // This most likely occurs when there edit operations are no-ops
                        contentChangedEvents.push(_this._createLineChangedEvent(minTouchedLineNumber));
                    }
                    var versionBumps = Math.max(contentChangedEvents.length, contentChanged2Events.length);
                    var finalVersionId = _this.getVersionId() + versionBumps;
                    _this._setVersionId(finalVersionId);
                    for (var i = contentChangedEvents.length - 1, versionId = finalVersionId; i >= 0; i--, versionId--) {
                        contentChangedEvents[i].versionId = versionId;
                    }
                    for (var i = contentChanged2Events.length - 1, versionId = finalVersionId; i >= 0; i--, versionId--) {
                        contentChanged2Events[i].versionId = versionId;
                    }
                    for (var i = 0, len = contentChangedEvents.length; i < len; i++) {
                        _this.emit(editorCommon.EventType.ModelContentChanged, contentChangedEvents[i]);
                    }
                    for (var i = 0, len = contentChanged2Events.length; i < len; i++) {
                        _this.emit(editorCommon.EventType.ModelContentChanged2, contentChanged2Events[i]);
                    }
                }
                // this._assertLineNumbersOK();
            });
        };
        EditableTextModel.prototype._assertLineNumbersOK = function () {
            var foundMarkersCnt = 0;
            for (var i = 0, len = this._lines.length; i < len; i++) {
                var line = this._lines[i];
                var lineNumber = i + 1;
                if (line.lineNumber !== lineNumber) {
                    throw new Error('Invalid lineNumber at line: ' + lineNumber + '; text is: ' + this.getValue());
                }
                var markers = line.getMarkers();
                for (var j = 0, lenJ = markers.length; j < lenJ; j++) {
                    foundMarkersCnt++;
                    var markerId = markers[j].id;
                    var marker = this._markerIdToMarker[markerId];
                    if (marker.line !== line) {
                        throw new Error('Misplaced marker with id ' + markerId);
                    }
                }
            }
            var totalMarkersCnt = Object.keys(this._markerIdToMarker).length;
            if (totalMarkersCnt !== foundMarkersCnt) {
                throw new Error('There are misplaced markers!');
            }
        };
        EditableTextModel.prototype.undo = function () {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('EditableTextModel.undo: Model is disposed');
            }
            return this._withDeferredEvents(function () {
                _this._isUndoing = true;
                var r = _this._commandManager.undo();
                _this._isUndoing = false;
                if (!r) {
                    return null;
                }
                _this._overwriteAlternativeVersionId(r.recordedVersionId);
                return r.selections;
            });
        };
        EditableTextModel.prototype.redo = function () {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('EditableTextModel.redo: Model is disposed');
            }
            return this._withDeferredEvents(function () {
                _this._isRedoing = true;
                var r = _this._commandManager.redo();
                _this._isRedoing = false;
                if (!r) {
                    return null;
                }
                _this._overwriteAlternativeVersionId(r.recordedVersionId);
                return r.selections;
            });
        };
        EditableTextModel.prototype.setEditableRange = function (range) {
            if (this._isDisposed) {
                throw new Error('EditableTextModel.setEditableRange: Model is disposed');
            }
            this._commandManager.clear();
            if (this._hasEditableRange) {
                this.removeTrackedRange(this._editableRangeId);
                this._editableRangeId = null;
                this._hasEditableRange = false;
            }
            if (range) {
                this._hasEditableRange = true;
                this._editableRangeId = this.addTrackedRange(range, editorCommon.TrackedRangeStickiness.AlwaysGrowsWhenTypingAtEdges);
            }
        };
        EditableTextModel.prototype.hasEditableRange = function () {
            if (this._isDisposed) {
                throw new Error('EditableTextModel.hasEditableRange: Model is disposed');
            }
            return this._hasEditableRange;
        };
        EditableTextModel.prototype.getEditableRange = function () {
            if (this._isDisposed) {
                throw new Error('EditableTextModel.getEditableRange: Model is disposed');
            }
            if (this._hasEditableRange) {
                return this.getTrackedRange(this._editableRangeId);
            }
            else {
                return this.getFullModelRange();
            }
        };
        EditableTextModel.prototype._createLineChangedEvent = function (lineNumber) {
            return {
                changeType: editorCommon.EventType.ModelContentChangedLineChanged,
                lineNumber: lineNumber,
                detail: this._lines[lineNumber - 1].text,
                versionId: -1,
                isUndoing: this._isUndoing,
                isRedoing: this._isRedoing
            };
        };
        EditableTextModel.prototype._createLinesDeletedEvent = function (fromLineNumber, toLineNumber) {
            return {
                changeType: editorCommon.EventType.ModelContentChangedLinesDeleted,
                fromLineNumber: fromLineNumber,
                toLineNumber: toLineNumber,
                versionId: -1,
                isUndoing: this._isUndoing,
                isRedoing: this._isRedoing
            };
        };
        EditableTextModel.prototype._createLinesInsertedEvent = function (fromLineNumber, toLineNumber, newLinesContent) {
            return {
                changeType: editorCommon.EventType.ModelContentChangedLinesInserted,
                fromLineNumber: fromLineNumber,
                toLineNumber: toLineNumber,
                detail: newLinesContent,
                versionId: -1,
                isUndoing: this._isUndoing,
                isRedoing: this._isRedoing
            };
        };
        return EditableTextModel;
    }(textModelWithDecorations_1.TextModelWithDecorations));
    exports.EditableTextModel = EditableTextModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/model/model", ["require", "exports", 'vs/base/common/uri', 'vs/editor/common/editorCommon', 'vs/editor/common/model/editableTextModel', 'vs/editor/common/model/textModel'], function (require, exports, uri_1, editorCommon_1, editableTextModel_1, textModel_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // The hierarchy is:
    // Model -> EditableTextModel -> TextModelWithDecorations -> TextModelWithTrackedRanges -> TextModelWithMarkers -> TextModelWithTokens -> TextModel
    var MODEL_ID = 0;
    var aliveModels = {};
    // var LAST_CNT = 0;
    // setInterval(() => {
    // 	var cnt = Object.keys(aliveModels).length;
    // 	if (cnt === LAST_CNT) {
    // 		return;
    // 	}
    // 	console.warn('ALIVE MODELS:');
    // 	console.log(Object.keys(aliveModels).join('\n'));
    // 	LAST_CNT = cnt;
    // }, 100);
    var Model = (function (_super) {
        __extends(Model, _super);
        /**
         * Instantiates a new model
         * @param rawText
         *   The raw text buffer. It may start with a UTF-16 BOM, which can be
         *   optionally preserved when doing a getValue call. The lines may be
         *   separated by different EOL combinations, such as \n or \r\n. These
         *   can also be preserved when doing a getValue call.
         * @param mode
         *   The language service name this model is bound to.
         * @param associatedResource
         *   The resource associated with this model. If the value is not provided an
         *   unique in memory URL is constructed as the associated resource.
         */
        function Model(rawText, options, modeOrPromise, associatedResource) {
            if (associatedResource === void 0) { associatedResource = null; }
            _super.call(this, [
                editorCommon_1.EventType.ModelDispose
            ], textModel_1.TextModel.toRawText(rawText, options), modeOrPromise);
            // Generate a new unique model id
            MODEL_ID++;
            this.id = '$model' + MODEL_ID;
            if (typeof associatedResource === 'undefined' || associatedResource === null) {
                this._associatedResource = uri_1.default.parse('inmemory://model/' + MODEL_ID);
            }
            else {
                this._associatedResource = associatedResource;
            }
            if (aliveModels[String(this._associatedResource)]) {
                throw new Error('Cannot instantiate a second Model with the same URI!');
            }
            this._attachedEditorCount = 0;
            aliveModels[String(this._associatedResource)] = true;
            // console.log('ALIVE MODELS: ' + Object.keys(aliveModels).join('\n'));
        }
        Model.prototype.getModeId = function () {
            return this.getMode().getId();
        };
        Model.prototype.destroy = function () {
            this.dispose();
        };
        Model.prototype.dispose = function () {
            this._isDisposing = true;
            delete aliveModels[String(this._associatedResource)];
            this.emit(editorCommon_1.EventType.ModelDispose);
            _super.prototype.dispose.call(this);
            this._isDisposing = false;
            // console.log('ALIVE MODELS: ' + Object.keys(aliveModels).join('\n'));
        };
        Model.prototype.onBeforeAttached = function () {
            if (this._isDisposed) {
                throw new Error('Model.onBeforeAttached: Model is disposed');
            }
            this._attachedEditorCount++;
            // Warm up tokens for the editor
            this._warmUpTokens();
        };
        Model.prototype.onBeforeDetached = function () {
            if (this._isDisposed) {
                throw new Error('Model.onBeforeDetached: Model is disposed');
            }
            this._attachedEditorCount--;
            // Intentional empty (for now)
        };
        Model.prototype.isAttachedToEditor = function () {
            return this._attachedEditorCount > 0;
        };
        Model.prototype.getAssociatedResource = function () {
            if (this._isDisposed) {
                throw new Error('Model.getAssociatedResource: Model is disposed');
            }
            return this._associatedResource;
        };
        Model.DEFAULT_CREATION_OPTIONS = textModel_1.TextModel.DEFAULT_CREATION_OPTIONS;
        return Model;
    }(editableTextModel_1.EditableTextModel));
    exports.Model = Model;
});

define("vs/nls!vs/editor/common/modes/modesRegistry",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/common/modes/modesRegistry", data); });
define("vs/nls!vs/editor/common/services/bulkEdit",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/common/services/bulkEdit", data); });
define("vs/nls!vs/editor/common/services/modeServiceImpl",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/common/services/modeServiceImpl", data); });
define("vs/nls!vs/editor/common/services/modelServiceImpl",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/common/services/modelServiceImpl", data); });
define("vs/nls!vs/editor/contrib/accessibility/browser/accessibility",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/accessibility/browser/accessibility", data); });
define("vs/nls!vs/editor/contrib/clipboard/browser/clipboard",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/clipboard/browser/clipboard", data); });
define("vs/nls!vs/editor/contrib/comment/common/comment",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/comment/common/comment", data); });
define("vs/nls!vs/editor/contrib/contextmenu/browser/contextmenu",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/contextmenu/browser/contextmenu", data); });
define("vs/nls!vs/editor/contrib/defineKeybinding/browser/defineKeybinding",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/defineKeybinding/browser/defineKeybinding", data); });
define("vs/nls!vs/editor/contrib/find/browser/findWidget",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/find/browser/findWidget", data); });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/find/browser/findWidget", ["require", "exports", 'vs/nls!vs/editor/contrib/find/browser/findWidget', 'vs/base/common/errors', 'vs/base/common/keyCodes', 'vs/base/common/strings', 'vs/base/browser/dom', 'vs/base/browser/ui/findinput/findInput', 'vs/base/browser/ui/inputbox/inputBox', 'vs/base/browser/ui/widget', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowser', 'vs/editor/contrib/find/common/findModel', 'vs/css!./findWidget'], function (require, exports, nls, errors_1, keyCodes_1, strings, dom, findInput_1, inputBox_1, widget_1, editorCommon_1, editorBrowser_1, findModel_1) {
    'use strict';
    var NLS_FIND_INPUT_LABEL = nls.localize(0, null);
    var NLS_FIND_INPUT_PLACEHOLDER = nls.localize(1, null);
    var NLS_PREVIOUS_MATCH_BTN_LABEL = nls.localize(2, null);
    var NLS_NEXT_MATCH_BTN_LABEL = nls.localize(3, null);
    var NLS_TOGGLE_SELECTION_FIND_TITLE = nls.localize(4, null);
    var NLS_CLOSE_BTN_LABEL = nls.localize(5, null);
    var NLS_REPLACE_INPUT_LABEL = nls.localize(6, null);
    var NLS_REPLACE_INPUT_PLACEHOLDER = nls.localize(7, null);
    var NLS_REPLACE_BTN_LABEL = nls.localize(8, null);
    var NLS_REPLACE_ALL_BTN_LABEL = nls.localize(9, null);
    var NLS_TOGGLE_REPLACE_MODE_BTN_LABEL = nls.localize(10, null);
    var NLS_MATCHES_COUNT_LIMIT_TITLE = nls.localize(11, null);
    var NLS_MATCHES_LOCATION = nls.localize(12, null);
    var NLS_NO_RESULTS = nls.localize(13, null);
    var FindWidget = (function (_super) {
        __extends(FindWidget, _super);
        function FindWidget(codeEditor, controller, state, contextViewProvider, keybindingService) {
            var _this = this;
            _super.call(this);
            this._codeEditor = codeEditor;
            this._controller = controller;
            this._state = state;
            this._contextViewProvider = contextViewProvider;
            this._keybindingService = keybindingService;
            this._isVisible = false;
            this._isReplaceVisible = false;
            this._register(this._state.addChangeListener(function (e) { return _this._onStateChanged(e); }));
            this._buildDomNode();
            this._updateButtons();
            this.focusTracker = this._register(dom.trackFocus(this._findInput.inputBox.inputElement));
            this.focusTracker.addFocusListener(function () { return _this._reseedFindScope(); });
            this._register(this._codeEditor.addListener2(editorCommon_1.EventType.ConfigurationChanged, function (e) {
                if (e.readOnly) {
                    if (_this._codeEditor.getConfiguration().readOnly) {
                        // Hide replace part if editor becomes read only
                        _this._state.change({ isReplaceRevealed: false }, false);
                    }
                    _this._updateButtons();
                }
            }));
            this._register(this._codeEditor.addListener2(editorCommon_1.EventType.CursorSelectionChanged, function () {
                if (_this._isVisible) {
                    _this._updateToggleSelectionFindButton();
                }
            }));
            this._codeEditor.addOverlayWidget(this);
        }
        FindWidget.prototype._reseedFindScope = function () {
            var selection = this._codeEditor.getSelection();
            if (selection.startLineNumber !== selection.endLineNumber) {
                // Reseed find scope
                this._state.change({ searchScope: selection }, true);
            }
        };
        // ----- IOverlayWidget API
        FindWidget.prototype.getId = function () {
            return FindWidget.ID;
        };
        FindWidget.prototype.getDomNode = function () {
            return this._domNode;
        };
        FindWidget.prototype.getPosition = function () {
            if (this._isVisible) {
                return {
                    preference: editorBrowser_1.OverlayWidgetPositionPreference.TOP_RIGHT_CORNER
                };
            }
            return null;
        };
        // ----- React to state changes
        FindWidget.prototype._onStateChanged = function (e) {
            if (e.searchString) {
                this._findInput.setValue(this._state.searchString);
                this._updateButtons();
            }
            if (e.replaceString) {
                this._replaceInputBox.value = this._state.replaceString;
            }
            if (e.isRevealed) {
                if (this._state.isRevealed) {
                    this._reveal(true);
                }
                else {
                    this._hide(true);
                }
            }
            if (e.isReplaceRevealed) {
                if (this._state.isReplaceRevealed) {
                    if (!this._codeEditor.getConfiguration().readOnly && !this._isReplaceVisible) {
                        this._isReplaceVisible = true;
                        this._updateButtons();
                    }
                }
                else {
                    if (this._isReplaceVisible) {
                        this._isReplaceVisible = false;
                        this._updateButtons();
                    }
                }
            }
            if (e.isRegex) {
                this._findInput.setRegex(this._state.isRegex);
            }
            if (e.wholeWord) {
                this._findInput.setWholeWords(this._state.wholeWord);
            }
            if (e.matchCase) {
                this._findInput.setCaseSensitive(this._state.matchCase);
            }
            if (e.searchScope) {
                if (this._state.searchScope) {
                    this._toggleSelectionFind.checked = true;
                }
                else {
                    this._toggleSelectionFind.checked = false;
                }
                this._updateToggleSelectionFindButton();
            }
            if (e.searchString || e.matchesCount || e.matchesPosition) {
                var showRedOutline = (this._state.searchString.length > 0 && this._state.matchesCount === 0);
                dom.toggleClass(this._domNode, 'no-results', showRedOutline);
                this._updateMatchesCount();
            }
        };
        FindWidget.prototype._updateMatchesCount = function () {
            if (this._state.matchesCount >= findModel_1.MATCHES_LIMIT) {
                this._matchesCount.title = NLS_MATCHES_COUNT_LIMIT_TITLE;
            }
            else {
                this._matchesCount.title = '';
            }
            // remove previous content
            if (this._matchesCount.firstChild) {
                this._matchesCount.removeChild(this._matchesCount.firstChild);
            }
            var label;
            if (this._state.matchesCount > 0) {
                var matchesCount = String(this._state.matchesCount);
                if (this._state.matchesCount >= findModel_1.MATCHES_LIMIT) {
                    matchesCount += '+';
                }
                var matchesPosition = String(this._state.matchesPosition);
                if (matchesPosition === '0') {
                    matchesPosition = '?';
                }
                label = strings.format(NLS_MATCHES_LOCATION, matchesPosition, matchesCount);
            }
            else {
                label = NLS_NO_RESULTS;
            }
            this._matchesCount.appendChild(document.createTextNode(label));
        };
        // ----- actions
        /**
         * If 'selection find' is ON we should not disable the button (its function is to cancel 'selection find').
         * If 'selection find' is OFF we enable the button only if there is a multi line selection.
         */
        FindWidget.prototype._updateToggleSelectionFindButton = function () {
            var selection = this._codeEditor.getSelection();
            var isMultiLineSelection = selection ? (selection.startLineNumber !== selection.endLineNumber) : false;
            var isChecked = this._toggleSelectionFind.checked;
            this._toggleSelectionFind.setEnabled(this._isVisible && (isChecked || isMultiLineSelection));
        };
        FindWidget.prototype._updateButtons = function () {
            this._findInput.setEnabled(this._isVisible);
            this._replaceInputBox.setEnabled(this._isVisible && this._isReplaceVisible);
            this._updateToggleSelectionFindButton();
            this._closeBtn.setEnabled(this._isVisible);
            var findInputIsNonEmpty = (this._state.searchString.length > 0);
            this._prevBtn.setEnabled(this._isVisible && findInputIsNonEmpty);
            this._nextBtn.setEnabled(this._isVisible && findInputIsNonEmpty);
            this._replaceBtn.setEnabled(this._isVisible && this._isReplaceVisible && findInputIsNonEmpty);
            this._replaceAllBtn.setEnabled(this._isVisible && this._isReplaceVisible && findInputIsNonEmpty);
            dom.toggleClass(this._domNode, 'replaceToggled', this._isReplaceVisible);
            this._toggleReplaceBtn.toggleClass('collapse', !this._isReplaceVisible);
            this._toggleReplaceBtn.toggleClass('expand', this._isReplaceVisible);
            this._toggleReplaceBtn.setExpanded(this._isReplaceVisible);
            var canReplace = !this._codeEditor.getConfiguration().readOnly;
            this._toggleReplaceBtn.setEnabled(this._isVisible && canReplace);
        };
        FindWidget.prototype._reveal = function (animate) {
            var _this = this;
            if (!this._isVisible) {
                this._isVisible = true;
                this._updateButtons();
                setTimeout(function () {
                    dom.addClass(_this._domNode, 'visible');
                    if (!animate) {
                        dom.addClass(_this._domNode, 'noanimation');
                        setTimeout(function () {
                            dom.removeClass(_this._domNode, 'noanimation');
                        }, 200);
                    }
                }, 0);
                this._codeEditor.layoutOverlayWidget(this);
            }
        };
        FindWidget.prototype._hide = function (focusTheEditor) {
            if (this._isVisible) {
                this._isVisible = false;
                this._updateButtons();
                dom.removeClass(this._domNode, 'visible');
                if (focusTheEditor) {
                    this._codeEditor.focus();
                }
                this._codeEditor.layoutOverlayWidget(this);
            }
        };
        // ----- Public
        FindWidget.prototype.focusFindInput = function () {
            this._findInput.select();
            // Edge browser requires focus() in addition to select()
            this._findInput.focus();
        };
        FindWidget.prototype.focusReplaceInput = function () {
            this._replaceInputBox.select();
            // Edge browser requires focus() in addition to select()
            this._replaceInputBox.focus();
        };
        FindWidget.prototype._onFindInputKeyDown = function (e) {
            switch (e.asKeybinding()) {
                case keyCodes_1.CommonKeybindings.ENTER:
                    this._codeEditor.getAction(findModel_1.FIND_IDS.NextMatchFindAction).run().done(null, errors_1.onUnexpectedError);
                    e.preventDefault();
                    return;
                case keyCodes_1.CommonKeybindings.SHIFT_ENTER:
                    this._codeEditor.getAction(findModel_1.FIND_IDS.PreviousMatchFindAction).run().done(null, errors_1.onUnexpectedError);
                    e.preventDefault();
                    return;
                case keyCodes_1.CommonKeybindings.TAB:
                    if (this._isReplaceVisible) {
                        this._replaceInputBox.focus();
                    }
                    else {
                        this._findInput.focusOnCaseSensitive();
                    }
                    e.preventDefault();
                    return;
                case keyCodes_1.CommonKeybindings.CTRLCMD_DOWN_ARROW:
                    this._codeEditor.focus();
                    e.preventDefault();
                    return;
            }
        };
        FindWidget.prototype._onReplaceInputKeyDown = function (e) {
            switch (e.asKeybinding()) {
                case keyCodes_1.CommonKeybindings.ENTER:
                    this._controller.replace();
                    e.preventDefault();
                    return;
                case keyCodes_1.CommonKeybindings.CTRLCMD_ENTER:
                    this._controller.replaceAll();
                    e.preventDefault();
                    return;
                case keyCodes_1.CommonKeybindings.TAB:
                    this._findInput.focusOnCaseSensitive();
                    e.preventDefault();
                    return;
                case keyCodes_1.CommonKeybindings.SHIFT_TAB:
                    this._findInput.focus();
                    e.preventDefault();
                    return;
                case keyCodes_1.CommonKeybindings.CTRLCMD_DOWN_ARROW:
                    this._codeEditor.focus();
                    e.preventDefault();
                    return;
            }
        };
        // ----- initialization
        FindWidget.prototype._keybindingLabelFor = function (actionId) {
            var keybindings = this._keybindingService.lookupKeybindings(actionId);
            if (keybindings.length === 0) {
                return '';
            }
            return ' (' + this._keybindingService.getLabelFor(keybindings[0]) + ')';
        };
        FindWidget.prototype._buildFindPart = function () {
            var _this = this;
            // Find input
            this._findInput = this._register(new findInput_1.FindInput(null, this._contextViewProvider, {
                width: FindWidget.FIND_INPUT_AREA_WIDTH,
                label: NLS_FIND_INPUT_LABEL,
                placeholder: NLS_FIND_INPUT_PLACEHOLDER,
                appendCaseSensitiveLabel: this._keybindingLabelFor(findModel_1.FIND_IDS.ToggleCaseSensitiveCommand),
                appendWholeWordsLabel: this._keybindingLabelFor(findModel_1.FIND_IDS.ToggleWholeWordCommand),
                appendRegexLabel: this._keybindingLabelFor(findModel_1.FIND_IDS.ToggleRegexCommand),
                validation: function (value) {
                    if (value.length === 0) {
                        return null;
                    }
                    if (!_this._findInput.getRegex()) {
                        return null;
                    }
                    try {
                        /* tslint:disable:no-unused-expression */
                        new RegExp(value);
                        /* tslint:enable:no-unused-expression */
                        return null;
                    }
                    catch (e) {
                        return { content: e.message };
                    }
                }
            }));
            this._register(this._findInput.onKeyDown(function (e) { return _this._onFindInputKeyDown(e); }));
            this._register(this._findInput.onInput(function () {
                _this._state.change({ searchString: _this._findInput.getValue() }, true);
            }));
            this._register(this._findInput.onDidOptionChange(function () {
                _this._state.change({
                    isRegex: _this._findInput.getRegex(),
                    wholeWord: _this._findInput.getWholeWords(),
                    matchCase: _this._findInput.getCaseSensitive()
                }, true);
            }));
            this._register(this._findInput.onCaseSensitiveKeyDown(function (e) {
                if (e.equals(keyCodes_1.CommonKeybindings.SHIFT_TAB)) {
                    if (_this._isReplaceVisible) {
                        _this._replaceInputBox.focus();
                        e.preventDefault();
                    }
                }
            }));
            this._matchesCount = document.createElement('div');
            this._matchesCount.className = 'matchesCount';
            this._updateMatchesCount();
            // Previous button
            this._prevBtn = this._register(new SimpleButton({
                label: NLS_PREVIOUS_MATCH_BTN_LABEL + this._keybindingLabelFor(findModel_1.FIND_IDS.PreviousMatchFindAction),
                className: 'previous',
                onTrigger: function () {
                    _this._codeEditor.getAction(findModel_1.FIND_IDS.PreviousMatchFindAction).run().done(null, errors_1.onUnexpectedError);
                },
                onKeyDown: function (e) { }
            }));
            // Next button
            this._nextBtn = this._register(new SimpleButton({
                label: NLS_NEXT_MATCH_BTN_LABEL + this._keybindingLabelFor(findModel_1.FIND_IDS.NextMatchFindAction),
                className: 'next',
                onTrigger: function () {
                    _this._codeEditor.getAction(findModel_1.FIND_IDS.NextMatchFindAction).run().done(null, errors_1.onUnexpectedError);
                },
                onKeyDown: function (e) { }
            }));
            var findPart = document.createElement('div');
            findPart.className = 'find-part';
            findPart.appendChild(this._findInput.domNode);
            findPart.appendChild(this._matchesCount);
            findPart.appendChild(this._prevBtn.domNode);
            findPart.appendChild(this._nextBtn.domNode);
            // Toggle selection button
            this._toggleSelectionFind = this._register(new SimpleCheckbox({
                parent: findPart,
                title: NLS_TOGGLE_SELECTION_FIND_TITLE,
                onChange: function () {
                    if (_this._toggleSelectionFind.checked) {
                        _this._reseedFindScope();
                    }
                    else {
                        _this._state.change({ searchScope: null }, true);
                    }
                }
            }));
            // Close button
            this._closeBtn = this._register(new SimpleButton({
                label: NLS_CLOSE_BTN_LABEL + this._keybindingLabelFor(findModel_1.FIND_IDS.CloseFindWidgetCommand),
                className: 'close-fw',
                onTrigger: function () {
                    _this._state.change({ isRevealed: false }, false);
                },
                onKeyDown: function (e) {
                    if (e.equals(keyCodes_1.CommonKeybindings.TAB)) {
                        if (_this._isReplaceVisible) {
                            if (_this._replaceBtn.isEnabled()) {
                                _this._replaceBtn.focus();
                            }
                            else {
                                _this._codeEditor.focus();
                            }
                            e.preventDefault();
                        }
                    }
                }
            }));
            findPart.appendChild(this._closeBtn.domNode);
            return findPart;
        };
        FindWidget.prototype._buildReplacePart = function () {
            var _this = this;
            // Replace input
            var replaceInput = document.createElement('div');
            replaceInput.className = 'replace-input';
            replaceInput.style.width = FindWidget.REPLACE_INPUT_AREA_WIDTH + 'px';
            this._replaceInputBox = this._register(new inputBox_1.InputBox(replaceInput, null, {
                ariaLabel: NLS_REPLACE_INPUT_LABEL,
                placeholder: NLS_REPLACE_INPUT_PLACEHOLDER
            }));
            this._register(dom.addStandardDisposableListener(this._replaceInputBox.inputElement, 'keydown', function (e) { return _this._onReplaceInputKeyDown(e); }));
            this._register(dom.addStandardDisposableListener(this._replaceInputBox.inputElement, 'input', function (e) {
                _this._state.change({ replaceString: _this._replaceInputBox.value }, false);
            }));
            // Replace one button
            this._replaceBtn = this._register(new SimpleButton({
                label: NLS_REPLACE_BTN_LABEL + this._keybindingLabelFor(findModel_1.FIND_IDS.ReplaceOneAction),
                className: 'replace',
                onTrigger: function () {
                    _this._controller.replace();
                },
                onKeyDown: function (e) {
                    if (e.equals(keyCodes_1.CommonKeybindings.SHIFT_TAB)) {
                        _this._closeBtn.focus();
                        e.preventDefault();
                    }
                }
            }));
            // Replace all button
            this._replaceAllBtn = this._register(new SimpleButton({
                label: NLS_REPLACE_ALL_BTN_LABEL + this._keybindingLabelFor(findModel_1.FIND_IDS.ReplaceAllAction),
                className: 'replace-all',
                onTrigger: function () {
                    _this._controller.replaceAll();
                },
                onKeyDown: function (e) { }
            }));
            var replacePart = document.createElement('div');
            replacePart.className = 'replace-part';
            replacePart.appendChild(replaceInput);
            replacePart.appendChild(this._replaceBtn.domNode);
            replacePart.appendChild(this._replaceAllBtn.domNode);
            return replacePart;
        };
        FindWidget.prototype._buildDomNode = function () {
            var _this = this;
            // Find part
            var findPart = this._buildFindPart();
            // Replace part
            var replacePart = this._buildReplacePart();
            // Toggle replace button
            this._toggleReplaceBtn = this._register(new SimpleButton({
                label: NLS_TOGGLE_REPLACE_MODE_BTN_LABEL,
                className: 'toggle left',
                onTrigger: function () {
                    _this._state.change({ isReplaceRevealed: !_this._isReplaceVisible }, true);
                },
                onKeyDown: function (e) { }
            }));
            this._toggleReplaceBtn.toggleClass('expand', this._isReplaceVisible);
            this._toggleReplaceBtn.toggleClass('collapse', !this._isReplaceVisible);
            this._toggleReplaceBtn.setExpanded(this._isReplaceVisible);
            // Widget
            this._domNode = document.createElement('div');
            this._domNode.className = 'editor-widget find-widget';
            this._domNode.setAttribute('aria-hidden', 'false');
            this._domNode.appendChild(this._toggleReplaceBtn.domNode);
            this._domNode.appendChild(findPart);
            this._domNode.appendChild(replacePart);
        };
        FindWidget.ID = 'editor.contrib.findWidget';
        FindWidget.PART_WIDTH = 275;
        FindWidget.FIND_INPUT_AREA_WIDTH = FindWidget.PART_WIDTH - 54;
        FindWidget.REPLACE_INPUT_AREA_WIDTH = FindWidget.FIND_INPUT_AREA_WIDTH;
        return FindWidget;
    }(widget_1.Widget));
    exports.FindWidget = FindWidget;
    var SimpleCheckbox = (function (_super) {
        __extends(SimpleCheckbox, _super);
        function SimpleCheckbox(opts) {
            var _this = this;
            _super.call(this);
            this._opts = opts;
            this._domNode = document.createElement('div');
            this._domNode.className = 'monaco-checkbox';
            this._domNode.title = this._opts.title;
            this._checkbox = document.createElement('input');
            this._checkbox.type = 'checkbox';
            this._checkbox.className = 'checkbox';
            this._checkbox.id = 'checkbox-' + SimpleCheckbox._COUNTER++;
            this._label = document.createElement('label');
            this._label.className = 'label';
            // Connect the label and the checkbox. Checkbox will get checked when the label recieves a click.
            this._label.htmlFor = this._checkbox.id;
            this._domNode.appendChild(this._checkbox);
            this._domNode.appendChild(this._label);
            this._opts.parent.appendChild(this._domNode);
            this.onchange(this._checkbox, function (e) {
                _this._opts.onChange();
            });
        }
        Object.defineProperty(SimpleCheckbox.prototype, "domNode", {
            get: function () {
                return this._domNode;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SimpleCheckbox.prototype, "checked", {
            get: function () {
                return this._checkbox.checked;
            },
            set: function (newValue) {
                this._checkbox.checked = newValue;
            },
            enumerable: true,
            configurable: true
        });
        SimpleCheckbox.prototype.focus = function () {
            this._checkbox.focus();
        };
        SimpleCheckbox.prototype.enable = function () {
            this._checkbox.removeAttribute('disabled');
        };
        SimpleCheckbox.prototype.disable = function () {
            this._checkbox.disabled = true;
        };
        SimpleCheckbox.prototype.setEnabled = function (enabled) {
            if (enabled) {
                this.enable();
            }
            else {
                this.disable();
            }
        };
        SimpleCheckbox._COUNTER = 0;
        return SimpleCheckbox;
    }(widget_1.Widget));
    var SimpleButton = (function (_super) {
        __extends(SimpleButton, _super);
        function SimpleButton(opts) {
            var _this = this;
            _super.call(this);
            this._opts = opts;
            this._domNode = document.createElement('div');
            this._domNode.title = this._opts.label;
            this._domNode.tabIndex = 0;
            this._domNode.className = 'button ' + this._opts.className;
            this._domNode.setAttribute('role', 'button');
            this._domNode.setAttribute('aria-label', this._opts.label);
            this.onclick(this._domNode, function (e) {
                _this._opts.onTrigger();
                e.preventDefault();
            });
            this.onkeydown(this._domNode, function (e) {
                if (e.equals(keyCodes_1.CommonKeybindings.SPACE) || e.equals(keyCodes_1.CommonKeybindings.ENTER)) {
                    _this._opts.onTrigger();
                    e.preventDefault();
                    return;
                }
                _this._opts.onKeyDown(e);
            });
        }
        Object.defineProperty(SimpleButton.prototype, "domNode", {
            get: function () {
                return this._domNode;
            },
            enumerable: true,
            configurable: true
        });
        SimpleButton.prototype.isEnabled = function () {
            return (this._domNode.tabIndex >= 0);
        };
        SimpleButton.prototype.focus = function () {
            this._domNode.focus();
        };
        SimpleButton.prototype.setEnabled = function (enabled) {
            dom.toggleClass(this._domNode, 'disabled', !enabled);
            this._domNode.setAttribute('aria-disabled', String(!enabled));
            this._domNode.tabIndex = enabled ? 0 : -1;
        };
        SimpleButton.prototype.setExpanded = function (expanded) {
            this._domNode.setAttribute('aria-expanded', String(expanded));
        };
        SimpleButton.prototype.toggleClass = function (className, shouldHaveIt) {
            dom.toggleClass(this._domNode, className, shouldHaveIt);
        };
        return SimpleButton;
    }(widget_1.Widget));
});

define("vs/nls!vs/editor/contrib/find/common/findController",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/find/common/findController", data); });
define("vs/nls!vs/editor/contrib/folding/browser/folding",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/folding/browser/folding", data); });
define("vs/nls!vs/editor/contrib/format/common/formatActions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/format/common/formatActions", data); });
define("vs/nls!vs/editor/contrib/goToDeclaration/browser/goToDeclaration",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/goToDeclaration/browser/goToDeclaration", data); });
define("vs/nls!vs/editor/contrib/gotoError/browser/gotoError",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/gotoError/browser/gotoError", data); });
define("vs/nls!vs/editor/contrib/hover/browser/hover",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/hover/browser/hover", data); });
define("vs/nls!vs/editor/contrib/hover/browser/modesContentHover",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/hover/browser/modesContentHover", data); });
define("vs/nls!vs/editor/contrib/inPlaceReplace/common/inPlaceReplace",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/inPlaceReplace/common/inPlaceReplace", data); });
define("vs/nls!vs/editor/contrib/indentation/common/indentation",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/indentation/common/indentation", data); });
define("vs/nls!vs/editor/contrib/linesOperations/common/linesOperations",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/linesOperations/common/linesOperations", data); });
define("vs/nls!vs/editor/contrib/links/browser/links",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/links/browser/links", data); });
define("vs/nls!vs/editor/contrib/multicursor/common/multicursor",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/multicursor/common/multicursor", data); });
define("vs/nls!vs/editor/contrib/parameterHints/browser/parameterHints",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/parameterHints/browser/parameterHints", data); });
define("vs/nls!vs/editor/contrib/parameterHints/browser/parameterHintsWidget",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/parameterHints/browser/parameterHintsWidget", data); });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/parameterHints/browser/parameterHintsWidget", ["require", "exports", 'vs/nls!vs/editor/contrib/parameterHints/browser/parameterHintsWidget', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/base/browser/builder', 'vs/base/browser/ui/aria/aria', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowser', 'vs/css!./parameterHints'], function (require, exports, nls, lifecycle_1, winjs_base_1, builder_1, aria, editorCommon_1, editorBrowser_1) {
    'use strict';
    var ParameterHintsWidget = (function () {
        function ParameterHintsWidget(model, editor, onShown, onHidden) {
            var _this = this;
            // Editor.IContentWidget.allowEditorOverflow
            this.allowEditorOverflow = true;
            this._onShown = onShown;
            this._onHidden = onHidden;
            this.editor = editor;
            this.modelListenersToRemove = [];
            this.model = null;
            this.isVisible = false;
            this.isDisposed = false;
            this.setModel(model);
            this.$el = builder_1.$('.editor-widget.parameter-hints-widget').on('click', function () {
                _this.selectNext();
                _this.editor.focus();
            });
            this.$wrapper = builder_1.$('.wrapper.monaco-editor-background').appendTo(this.$el);
            var $buttons = builder_1.$('.buttons').appendTo(this.$wrapper);
            builder_1.$('.button.previous').on('click', function (e) {
                e.preventDefault();
                e.stopPropagation();
                _this.selectPrevious();
            }).appendTo($buttons);
            builder_1.$('.button.next').on('click', function (e) {
                e.preventDefault();
                e.stopPropagation();
                _this.selectNext();
            }).appendTo($buttons);
            this.$overloads = builder_1.$('.overloads').appendTo(this.$wrapper);
            this.$signatures = builder_1.$('.signatures').appendTo(this.$wrapper);
            this.signatureViews = [];
            this.currentSignature = 0;
            this.editor.addContentWidget(this);
            this.hide();
            this.toDispose = [];
            this.toDispose.push(this.editor.addListener2(editorCommon_1.EventType.CursorSelectionChanged, function (e) {
                if (_this.isVisible) {
                    _this.editor.layoutContentWidget(_this);
                }
            }));
        }
        ParameterHintsWidget.prototype.setModel = function (newModel) {
            var _this = this;
            this.releaseModel();
            this.model = newModel;
            this.modelListenersToRemove.push(this.model.addListener('hint', function (e) {
                _this.show();
                _this.parameterHints = e.hints;
                _this.render(e.hints);
                _this.currentSignature = e.hints.currentSignature;
                _this.select(_this.currentSignature);
            }));
            this.modelListenersToRemove.push(this.model.addListener('cancel', function (e) {
                _this.hide();
            }));
        };
        ParameterHintsWidget.prototype.show = function () {
            var _this = this;
            if (this.isDisposed) {
                return;
            }
            if (this.isVisible) {
                return;
            }
            this._onShown();
            this.isVisible = true;
            winjs_base_1.TPromise.timeout(100).done(function () {
                _this.$el.addClass('visible');
            });
            this.editor.layoutContentWidget(this);
        };
        ParameterHintsWidget.prototype.hide = function () {
            if (this.isDisposed) {
                return;
            }
            if (!this.isVisible) {
                return;
            }
            this._onHidden();
            this.isVisible = false;
            this.parameterHints = null;
            this.announcedLabel = null;
            this.$el.removeClass('visible');
            this.editor.layoutContentWidget(this);
        };
        ParameterHintsWidget.prototype.getPosition = function () {
            if (this.isVisible) {
                return {
                    position: this.editor.getPosition(),
                    preference: [editorBrowser_1.ContentWidgetPositionPreference.ABOVE, editorBrowser_1.ContentWidgetPositionPreference.BELOW]
                };
            }
            return null;
        };
        ParameterHintsWidget.prototype.render = function (hints) {
            if (hints.signatures.length > 1) {
                this.$el.addClass('multiple');
            }
            else {
                this.$el.removeClass('multiple');
            }
            this.$signatures.empty();
            this.signatureViews = [];
            var height = 0;
            for (var i = 0, len = hints.signatures.length; i < len; i++) {
                var signature = hints.signatures[i];
                var $signature = this.renderSignature(this.$signatures, signature, hints.currentParameter);
                this.renderDocumentation($signature, signature, hints.currentParameter);
                var signatureHeight = $signature.getClientArea().height;
                this.signatureViews.push({
                    top: height,
                    height: signatureHeight
                });
                height += signatureHeight;
            }
        };
        ParameterHintsWidget.prototype.renderSignature = function ($el, signature, currentParameter) {
            var $signature = builder_1.$('.signature').appendTo($el), hasParameters = signature.parameters.length > 0;
            if (!hasParameters) {
                $signature.append(builder_1.$('span').text(signature.label));
            }
            else {
                var $parameters = builder_1.$('span.parameters'), offset = 0;
                for (var i = 0, len = signature.parameters.length; i < len; i++) {
                    var parameter = signature.parameters[i];
                    (i === 0 ? $signature : $parameters).append(builder_1.$('span').text(signature.label.substring(offset, parameter.signatureLabelOffset)));
                    $parameters.append(builder_1.$('span.parameter').addClass(i === currentParameter ? 'active' : '').text(signature.label.substring(parameter.signatureLabelOffset, parameter.signatureLabelEnd)));
                    offset = parameter.signatureLabelEnd;
                }
                $signature.append($parameters);
                $signature.append(builder_1.$('span').text(signature.label.substring(offset)));
            }
            return $signature;
        };
        ParameterHintsWidget.prototype.renderDocumentation = function ($el, signature, activeParameterIdx) {
            if (signature.documentation) {
                $el.append(builder_1.$('.documentation').text(signature.documentation));
            }
            var activeParameter = signature.parameters[activeParameterIdx];
            if (activeParameter && activeParameter.documentation) {
                var $parameter = builder_1.$('.documentation');
                $parameter.append(builder_1.$('span.parameter').text(activeParameter.label));
                $parameter.append(builder_1.$('span').text(activeParameter.documentation));
                $el.append($parameter);
            }
        };
        ParameterHintsWidget.prototype.select = function (position) {
            var signature = this.signatureViews[position];
            if (!signature) {
                return;
            }
            this.$signatures.style({ height: signature.height + 'px' });
            this.$signatures.getHTMLElement().scrollTop = signature.top;
            var overloads = '' + (position + 1);
            if (this.signatureViews.length < 10) {
                overloads += '/' + this.signatureViews.length;
            }
            this.$overloads.text(overloads);
            if (this.parameterHints && this.parameterHints.signatures[position].parameters[this.parameterHints.currentParameter]) {
                var labelToAnnounce = this.parameterHints.signatures[position].parameters[this.parameterHints.currentParameter].label;
                // Select method gets called on every user type while parameter hints are visible.
                // We do not want to spam the user with same announcements, so we only announce if the current parameter changed.
                if (this.announcedLabel !== labelToAnnounce) {
                    aria.alert(nls.localize(0, null, labelToAnnounce));
                    this.announcedLabel = labelToAnnounce;
                }
            }
            this.editor.layoutContentWidget(this);
        };
        ParameterHintsWidget.prototype.selectNext = function () {
            if (this.signatureViews.length < 2) {
                this.cancel();
                return false;
            }
            this.currentSignature = (this.currentSignature + 1) % this.signatureViews.length;
            this.select(this.currentSignature);
            return true;
        };
        ParameterHintsWidget.prototype.selectPrevious = function () {
            if (this.signatureViews.length < 2) {
                this.cancel();
                return false;
            }
            this.currentSignature--;
            if (this.currentSignature < 0) {
                this.currentSignature = this.signatureViews.length - 1;
            }
            this.select(this.currentSignature);
            return true;
        };
        ParameterHintsWidget.prototype.cancel = function () {
            this.model.cancel();
        };
        ParameterHintsWidget.prototype.getDomNode = function () {
            return this.$el.getHTMLElement();
        };
        ParameterHintsWidget.prototype.getId = function () {
            return ParameterHintsWidget.ID;
        };
        ParameterHintsWidget.prototype.releaseModel = function () {
            var listener;
            while (listener = this.modelListenersToRemove.pop()) {
                listener();
            }
            if (this.model) {
                this.model.dispose();
                this.model = null;
            }
        };
        ParameterHintsWidget.prototype.destroy = function () {
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            this.releaseModel();
            if (this.$overloads) {
                this.$overloads.destroy();
                delete this.$overloads;
            }
            if (this.$signatures) {
                this.$signatures.destroy();
                delete this.$signatures;
            }
            if (this.$wrapper) {
                this.$wrapper.destroy();
                delete this.$wrapper;
            }
            if (this.$el) {
                this.$el.destroy();
                delete this.$el;
            }
            this.isDisposed = true;
        };
        ParameterHintsWidget.ID = 'editor.widget.parameterHintsWidget';
        return ParameterHintsWidget;
    }());
    exports.ParameterHintsWidget = ParameterHintsWidget;
});

define("vs/nls!vs/editor/contrib/quickFix/browser/quickFix",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/quickFix/browser/quickFix", data); });
define("vs/nls!vs/editor/contrib/quickFix/browser/quickFixSelectionWidget",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/quickFix/browser/quickFixSelectionWidget", data); });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/quickFix/browser/quickFixSelectionWidget", ["require", "exports", 'vs/nls!vs/editor/contrib/quickFix/browser/quickFixSelectionWidget', 'vs/base/common/errors', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/base/parts/tree/browser/treeDefaults', 'vs/base/parts/tree/browser/treeImpl', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowser', 'vs/base/browser/ui/aria/aria', 'vs/css!./quickFix'], function (require, exports, nls, errors_1, winjs_base_1, dom, treeDefaults_1, treeImpl_1, editorCommon_1, editorBrowser_1, aria_1) {
    'use strict';
    var $ = dom.emmet;
    function isQuickFix(quickfix) {
        return quickfix
            && typeof quickfix.command === 'object'
            && typeof quickfix.command.title === 'string';
    }
    function getAriaAlertLabel(item) {
        return nls.localize(0, null, item.command.title);
    }
    // To be used as a tree element when we want to show a message
    var Message = (function () {
        function Message(parent, message) {
            this.parent = parent;
            this.message = message;
            // nothing to do
        }
        return Message;
    }());
    exports.Message = Message;
    var MessageRoot = (function () {
        function MessageRoot(message) {
            this.child = new Message(this, message);
        }
        return MessageRoot;
    }());
    exports.MessageRoot = MessageRoot;
    var DataSource = (function () {
        function DataSource() {
            this.root = null;
        }
        DataSource.prototype.isRoot = function (element) {
            if (element instanceof MessageRoot) {
                return true;
            }
            else if (element instanceof Message) {
                return false;
            }
            else if (Array.isArray(element)) {
                this.root = element;
                return true;
            }
            else {
                return false;
            }
        };
        DataSource.prototype.getId = function (tree, element) {
            if (element instanceof MessageRoot) {
                return 'messageroot';
            }
            else if (element instanceof Message) {
                return 'message' + element.message;
            }
            else if (Array.isArray(element)) {
                return 'root';
            }
            else if (isQuickFix(element)) {
                return element.id;
            }
            else {
                throw errors_1.illegalArgument('element');
            }
        };
        DataSource.prototype.getParent = function (tree, element) {
            if (element instanceof MessageRoot) {
                return winjs_base_1.TPromise.as(null);
            }
            else if (element instanceof Message) {
                return winjs_base_1.TPromise.as(element.parent);
            }
            return winjs_base_1.TPromise.as(this.isRoot(element) ? null : this.root);
        };
        DataSource.prototype.getChildren = function (tree, element) {
            if (element instanceof MessageRoot) {
                return winjs_base_1.TPromise.as([element.child]);
            }
            else if (element instanceof Message) {
                return winjs_base_1.TPromise.as([]);
            }
            return winjs_base_1.TPromise.as(this.isRoot(element) ? element : []);
        };
        DataSource.prototype.hasChildren = function (tree, element) {
            return this.isRoot(element);
        };
        return DataSource;
    }());
    var Controller = (function (_super) {
        __extends(Controller, _super);
        function Controller() {
            _super.apply(this, arguments);
        }
        /* protected */ Controller.prototype.onLeftClick = function (tree, element, event) {
            event.preventDefault();
            event.stopPropagation();
            if (!(element instanceof Message)) {
                tree.setSelection([element], { origin: 'mouse' });
            }
            return true;
        };
        return Controller;
    }(treeDefaults_1.DefaultController));
    var AccessibilityProvider = (function () {
        function AccessibilityProvider() {
        }
        AccessibilityProvider.prototype.getAriaLabel = function (tree, element) {
            if (isQuickFix(element)) {
                return getAriaAlertLabel(element);
            }
        };
        return AccessibilityProvider;
    }());
    exports.AccessibilityProvider = AccessibilityProvider;
    function getHeight(tree, element) {
        // var fix = <IQuickFix2>element;
        // if (!(element instanceof Message) && !!fix.documentation && tree.isFocused(fix)) {
        // 	return 35;
        // }
        return 19;
    }
    var Renderer = (function () {
        function Renderer() {
        }
        Renderer.prototype.getHeight = function (tree, element) {
            return getHeight(tree, element);
        };
        Renderer.prototype.getTemplateId = function (tree, element) {
            return element instanceof Message ? 'message' : 'default';
        };
        Renderer.prototype.renderTemplate = function (tree, templateId, container) {
            if (templateId === 'message') {
                var messageElement = dom.append(container, $('span'));
                messageElement.style.opacity = '0.7';
                messageElement.style.paddingLeft = '12px';
                return { element: messageElement };
            }
            var result = {};
            var text = dom.append(container, $('.text'));
            result['main'] = dom.append(text, $('.main'));
            var docs = dom.append(text, $('.docs'));
            result['documentationLabel'] = dom.append(docs, $('span.docs-label'));
            return result;
        };
        Renderer.prototype.renderElement = function (tree, element, templateId, templateData) {
            if (templateId === 'message') {
                templateData.element.textContent = element.message;
                return;
            }
            var quickFix = element;
            templateData.main.textContent = quickFix.command.title;
            templateData.documentationLabel.textContent = '';
        };
        Renderer.prototype.disposeTemplate = function (tree, templateId, templateData) {
            // noop
        };
        return Renderer;
    }());
    var QuickFixSelectionWidget = (function () {
        function QuickFixSelectionWidget(editor, telemetryService, onShown, onHidden) {
            var _this = this;
            // Editor.IContentWidget.allowEditorOverflow
            this.allowEditorOverflow = true;
            this.editor = editor;
            this._onShown = onShown;
            this._onHidden = onHidden;
            this.shouldShowEmptyList = true;
            this.isActive = false;
            this.isLoading = false;
            this.isAuto = false;
            this.modelListenersToRemove = [];
            this.model = null;
            this.telemetryData = Object.create(null);
            this.telemetryService = telemetryService;
            this.listenersToRemove = [];
            this.domnode = $('.editor-widget.quickfix-widget.monaco-editor-background.no-icons');
            this.domnode.style.width = QuickFixSelectionWidget.WIDTH + 'px';
            this.tree = new treeImpl_1.Tree(this.domnode, {
                dataSource: new DataSource(),
                renderer: new Renderer(),
                controller: new Controller(),
                accessibilityProvider: new AccessibilityProvider()
            }, {
                twistiePixels: 0,
                alwaysFocused: true,
                verticalScrollMode: 'visible',
                useShadows: false,
                ariaLabel: nls.localize(3, null)
            });
            this.listenersToRemove.push(this.tree.addListener('selection', function (e) {
                if (e.selection && e.selection.length > 0) {
                    var element = e.selection[0];
                    if (isQuickFix(element) && !(element instanceof MessageRoot) && !(element instanceof Message)) {
                        _this.telemetryData.selectedIndex = _this.tree.getInput().indexOf(element);
                        _this.telemetryData.wasCancelled = false;
                        _this.submitTelemetryData();
                        aria_1.alert(nls.localize(4, null, element.command.title));
                        _this.model.accept(element, _this.range);
                        _this.editor.focus();
                    }
                }
            }));
            var oldFocus = null;
            this.listenersToRemove.push(this.tree.addListener('focus', function (e) {
                var focus = e.focus;
                var payload = e.payload;
                if (focus === oldFocus) {
                    return;
                }
                var elementsToRefresh = [];
                if (oldFocus) {
                    elementsToRefresh.push(oldFocus);
                }
                if (focus) {
                    elementsToRefresh.push(focus);
                    _this._ariaAlert(getAriaAlertLabel(focus));
                }
                oldFocus = focus;
                _this.tree.refreshAll(elementsToRefresh).done(function () {
                    _this.updateWidgetHeight();
                    if (focus) {
                        return _this.tree.reveal(focus, (payload && payload.firstSuggestion) ? 0 : null);
                    }
                }, errors_1.onUnexpectedError);
            }));
            this.editor.addContentWidget(this);
            this.listenersToRemove.push(this.editor.addListener(editorCommon_1.EventType.CursorSelectionChanged, function (e) {
                if (_this.isActive) {
                    _this.editor.layoutContentWidget(_this);
                }
            }));
            this.hide();
        }
        QuickFixSelectionWidget.prototype._ariaAlert = function (newAriaAlertLabel) {
            if (this._lastAriaAlertLabel === newAriaAlertLabel) {
                return;
            }
            this._lastAriaAlertLabel = newAriaAlertLabel;
            if (this._lastAriaAlertLabel) {
                aria_1.alert(this._lastAriaAlertLabel);
            }
        };
        QuickFixSelectionWidget.prototype.setModel = function (newModel) {
            var _this = this;
            this.releaseModel();
            this.model = newModel;
            var timer = null, loadingHandle;
            this.modelListenersToRemove.push(this.model.addListener('loading', function (e) {
                if (!_this.isActive) {
                    timer = _this.telemetryService.start('QuickFixSelectionWidgetLoadingTime');
                    _this.isLoading = true;
                    _this.isAuto = !!e.auto;
                    if (!_this.isAuto) {
                        loadingHandle = setTimeout(function () {
                            dom.removeClass(_this.domnode, 'empty');
                            _this.tree.setInput(QuickFixSelectionWidget.LOADING_MESSAGE).done(null, errors_1.onUnexpectedError);
                            _this.updateWidgetHeight();
                            _this.show();
                        }, 50);
                    }
                    if (!e.retrigger) {
                        _this.telemetryData = {
                            wasAutomaticallyTriggered: e.characterTriggered
                        };
                    }
                }
            }));
            this.modelListenersToRemove.push(this.model.addListener('suggest', function (e) {
                _this.isLoading = false;
                if (typeof loadingHandle !== 'undefined') {
                    clearTimeout(loadingHandle);
                    loadingHandle = void 0;
                }
                var fixes = e.fixes;
                var bestFixIndex = -1;
                var bestFix = fixes[0];
                var bestScore = -1;
                for (var i = 0, len = fixes.length; i < len; i++) {
                    var fix = fixes[i];
                    var score = fix.score;
                    if (score > bestScore) {
                        bestScore = score;
                        bestFix = fix;
                        bestFixIndex = i;
                    }
                }
                dom.removeClass(_this.domnode, 'empty');
                _this.tree.setInput(fixes).done(null, errors_1.onUnexpectedError);
                _this.tree.setFocus(bestFix, { firstSuggestion: true });
                _this.updateWidgetHeight();
                _this.range = e.range;
                _this.show();
                _this.telemetryData = _this.telemetryData || {};
                _this.telemetryData.suggestionCount = fixes.length;
                _this.telemetryData.suggestedIndex = bestFixIndex;
                if (timer) {
                    timer.data = { reason: 'results' };
                    timer.stop();
                    timer = null;
                }
            }));
            this.modelListenersToRemove.push(this.model.addListener('empty', function (e) {
                var wasLoading = _this.isLoading;
                _this.isLoading = false;
                if (typeof loadingHandle !== 'undefined') {
                    clearTimeout(loadingHandle);
                    loadingHandle = void 0;
                }
                if (e.auto) {
                    _this.hide();
                }
                else if (wasLoading) {
                    if (_this.shouldShowEmptyList) {
                        dom.removeClass(_this.domnode, 'empty');
                        _this.tree.setInput(QuickFixSelectionWidget.NO_SUGGESTIONS_MESSAGE).done(null, errors_1.onUnexpectedError);
                        _this.updateWidgetHeight();
                        _this.show();
                    }
                    else {
                        _this.hide();
                    }
                }
                else {
                    dom.addClass(_this.domnode, 'empty');
                }
                if (timer) {
                    timer.data = { reason: 'empty' };
                    timer.stop();
                    timer = null;
                }
            }));
            this.modelListenersToRemove.push(this.model.addListener('cancel', function (e) {
                _this.isLoading = false;
                if (typeof loadingHandle !== 'undefined') {
                    clearTimeout(loadingHandle);
                    loadingHandle = void 0;
                }
                if (!e.retrigger) {
                    _this.hide();
                    if (_this.telemetryData) {
                        _this.telemetryData.selectedIndex = -1;
                        _this.telemetryData.wasCancelled = true;
                        _this.submitTelemetryData();
                    }
                }
                if (timer) {
                    timer.data = { reason: 'cancel' };
                    timer.stop();
                    timer = null;
                }
            }));
        };
        QuickFixSelectionWidget.prototype.selectNextPage = function () {
            if (this.isLoading) {
                return !this.isAuto;
            }
            if (this.isActive) {
                this.tree.focusNextPage();
                return true;
            }
            return false;
        };
        QuickFixSelectionWidget.prototype.selectNext = function () {
            if (this.isLoading) {
                return !this.isAuto;
            }
            if (this.isActive) {
                var focus = this.tree.getFocus();
                this.tree.focusNext(1);
                if (focus === this.tree.getFocus()) {
                    this.tree.focusFirst();
                }
                return true;
            }
            return false;
        };
        QuickFixSelectionWidget.prototype.selectPreviousPage = function () {
            if (this.isLoading) {
                return !this.isAuto;
            }
            if (this.isActive) {
                this.tree.focusPreviousPage();
                return true;
            }
            return false;
        };
        QuickFixSelectionWidget.prototype.selectPrevious = function () {
            if (this.isLoading) {
                return !this.isAuto;
            }
            if (this.isActive) {
                var focus = this.tree.getFocus();
                this.tree.focusPrevious(1);
                if (focus === this.tree.getFocus()) {
                    this.tree.focusLast();
                }
                return true;
            }
            return false;
        };
        QuickFixSelectionWidget.prototype.acceptSelectedSuggestion = function () {
            if (this.isLoading) {
                return !this.isAuto;
            }
            if (this.isActive) {
                var focus = this.tree.getFocus();
                if (focus) {
                    this.tree.setSelection([focus]);
                }
                else {
                    this.model.cancelDialog();
                }
                return true;
            }
            return false;
        };
        QuickFixSelectionWidget.prototype.releaseModel = function () {
            var listener;
            while (listener = this.modelListenersToRemove.pop()) {
                listener();
            }
            this.model = null;
        };
        QuickFixSelectionWidget.prototype.show = function () {
            this.isActive = true;
            this.tree.layout();
            this.editor.layoutContentWidget(this);
            this._onShown();
        };
        QuickFixSelectionWidget.prototype.hide = function () {
            this._onHidden();
            this.isActive = false;
            this.editor.layoutContentWidget(this);
        };
        QuickFixSelectionWidget.prototype.getPosition = function () {
            if (this.isActive) {
                return {
                    position: this.editor.getPosition(),
                    preference: [editorBrowser_1.ContentWidgetPositionPreference.BELOW, editorBrowser_1.ContentWidgetPositionPreference.ABOVE]
                };
            }
            return null;
        };
        QuickFixSelectionWidget.prototype.getDomNode = function () {
            return this.domnode;
        };
        QuickFixSelectionWidget.prototype.getId = function () {
            return QuickFixSelectionWidget.ID;
        };
        QuickFixSelectionWidget.prototype.submitTelemetryData = function () {
            this.telemetryService.publicLog('QuickFixSelectionWidget', this.telemetryData);
            this.telemetryData = Object.create(null);
        };
        QuickFixSelectionWidget.prototype.updateWidgetHeight = function () {
            var input = this.tree.getInput();
            var height;
            if (input === QuickFixSelectionWidget.LOADING_MESSAGE || input === QuickFixSelectionWidget.NO_SUGGESTIONS_MESSAGE) {
                height = 19;
            }
            else {
                var fixes = input;
                height = Math.min(fixes.length - 1, 11) * 19;
                var focus = this.tree.getFocus();
                height += focus ? getHeight(this.tree, focus) : 19;
            }
            this.domnode.style.height = height + 'px';
            this.tree.layout(height);
            this.editor.layoutContentWidget(this);
        };
        QuickFixSelectionWidget.prototype.destroy = function () {
            this.releaseModel();
            this.domnode = null;
            this.tree.dispose();
            this.tree = null;
            this.listenersToRemove.forEach(function (element) {
                element();
            });
            this.listenersToRemove = null;
        };
        QuickFixSelectionWidget.ID = 'editor.widget.QuickFixSelectionWidget';
        QuickFixSelectionWidget.WIDTH = 360;
        QuickFixSelectionWidget.LOADING_MESSAGE = new MessageRoot(nls.localize(1, null));
        QuickFixSelectionWidget.NO_SUGGESTIONS_MESSAGE = new MessageRoot(nls.localize(2, null));
        return QuickFixSelectionWidget;
    }());
    exports.QuickFixSelectionWidget = QuickFixSelectionWidget;
});

define("vs/nls!vs/editor/contrib/referenceSearch/browser/referenceSearch",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/referenceSearch/browser/referenceSearch", data); });
define("vs/nls!vs/editor/contrib/referenceSearch/browser/referenceSearchWidget",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/referenceSearch/browser/referenceSearchWidget", data); });
define("vs/nls!vs/editor/contrib/rename/browser/rename2",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/rename/browser/rename2", data); });
define("vs/nls!vs/editor/contrib/rename/browser/renameInputField",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/rename/browser/renameInputField", data); });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/rename/browser/renameInputField", ["require", "exports", 'vs/nls!vs/editor/contrib/rename/browser/renameInputField', 'vs/base/common/errors', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowser', 'vs/css!./rename'], function (require, exports, nls_1, errors_1, lifecycle_1, winjs_base_1, range_1, editorCommon_1, editorBrowser_1) {
    'use strict';
    var RenameInputField = (function () {
        function RenameInputField(editor) {
            // Editor.IContentWidget.allowEditorOverflow
            this.allowEditorOverflow = true;
            this._currentAcceptInput = null;
            this._currentCancelInput = null;
            this._editor = editor;
            this._editor.addContentWidget(this);
        }
        RenameInputField.prototype.dispose = function () {
            this._editor.removeContentWidget(this);
        };
        RenameInputField.prototype.getId = function () {
            return '__renameInputWidget';
        };
        RenameInputField.prototype.getDomNode = function () {
            if (!this._domNode) {
                this._inputField = document.createElement('input');
                this._inputField.className = 'rename-input';
                this._inputField.type = 'text';
                this._inputField.setAttribute('aria-label', nls_1.localize(0, null));
                this._domNode = document.createElement('div');
                this._domNode.style.height = this._editor.getConfiguration().lineHeight + "px";
                this._domNode.className = 'monaco-editor rename-box';
                this._domNode.appendChild(this._inputField);
            }
            return this._domNode;
        };
        RenameInputField.prototype.getPosition = function () {
            return this._visible
                ? { position: this._position, preference: [editorBrowser_1.ContentWidgetPositionPreference.BELOW, editorBrowser_1.ContentWidgetPositionPreference.ABOVE] }
                : null;
        };
        RenameInputField.prototype.acceptInput = function () {
            if (this._currentAcceptInput) {
                this._currentAcceptInput();
            }
        };
        RenameInputField.prototype.cancelInput = function () {
            if (this._currentCancelInput) {
                this._currentCancelInput();
            }
        };
        RenameInputField.prototype.getInput = function (where, value, selectionStart, selectionEnd) {
            var _this = this;
            this._position = { lineNumber: where.startLineNumber, column: where.startColumn };
            this._inputField.value = value;
            this._inputField.setAttribute('selectionStart', selectionStart.toString());
            this._inputField.setAttribute('selectionEnd', selectionEnd.toString());
            this._inputField.size = Math.max((where.endColumn - where.startColumn) * 1.1, 20);
            var disposeOnDone = [], always;
            always = function () {
                lifecycle_1.disposeAll(disposeOnDone);
                _this._hide();
            };
            return new winjs_base_1.TPromise(function (c, e) {
                _this._currentCancelInput = function () {
                    _this._currentAcceptInput = null;
                    _this._currentCancelInput = null;
                    e(errors_1.canceled());
                    return true;
                };
                _this._currentAcceptInput = function () {
                    if (_this._inputField.value.trim().length === 0 || _this._inputField.value === value) {
                        // empty or whitespace only or not changed
                        _this._currentCancelInput();
                        return;
                    }
                    _this._currentAcceptInput = null;
                    _this._currentCancelInput = null;
                    c(_this._inputField.value);
                };
                var onCursorChanged = function () {
                    if (!range_1.Range.containsPosition(where, _this._editor.getPosition())) {
                        _this._currentCancelInput();
                    }
                };
                disposeOnDone.push(_this._editor.addListener2(editorCommon_1.EventType.CursorSelectionChanged, onCursorChanged));
                disposeOnDone.push(_this._editor.addListener2(editorCommon_1.EventType.EditorBlur, _this._currentCancelInput));
                _this._show();
            }, this._currentCancelInput).then(function (newValue) {
                always();
                return newValue;
            }, function (err) {
                always();
                return winjs_base_1.TPromise.wrapError(err);
            });
        };
        RenameInputField.prototype._show = function () {
            var _this = this;
            this._editor.revealLineInCenterIfOutsideViewport(this._position.lineNumber);
            this._visible = true;
            this._editor.layoutContentWidget(this);
            setTimeout(function () {
                _this._inputField.focus();
                _this._inputField.setSelectionRange(parseInt(_this._inputField.getAttribute('selectionStart')), parseInt(_this._inputField.getAttribute('selectionEnd')));
            }, 25);
        };
        RenameInputField.prototype._hide = function () {
            this._visible = false;
            this._editor.layoutContentWidget(this);
        };
        return RenameInputField;
    }());
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = RenameInputField;
});

define("vs/nls!vs/editor/contrib/rename/common/rename",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/rename/common/rename", data); });
define("vs/nls!vs/editor/contrib/smartSelect/common/jumpToBracket",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/smartSelect/common/jumpToBracket", data); });
define("vs/nls!vs/editor/contrib/smartSelect/common/smartSelect",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/smartSelect/common/smartSelect", data); });
define("vs/nls!vs/editor/contrib/suggest/browser/suggest",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/suggest/browser/suggest", data); });
define("vs/nls!vs/editor/contrib/suggest/browser/suggestWidget",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/suggest/browser/suggestWidget", data); });
define("vs/nls!vs/editor/contrib/toggleTabFocusMode/common/toggleTabFocusMode",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/toggleTabFocusMode/common/toggleTabFocusMode", data); });
define("vs/nls!vs/editor/contrib/toggleWordWrap/common/toggleWordWrap",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/toggleWordWrap/common/toggleWordWrap", data); });
define("vs/nls!vs/editor/contrib/zoneWidget/browser/peekViewWidget",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/contrib/zoneWidget/browser/peekViewWidget", data); });
define("vs/nls!vs/editor/node/languageConfiguration",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/node/languageConfiguration", data); });
define("vs/nls!vs/editor/node/textMate/TMSnippets",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/node/textMate/TMSnippets", data); });
define("vs/nls!vs/editor/node/textMate/TMSyntax",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/editor/node/textMate/TMSyntax", data); });
define("vs/nls!vs/languages/css/common/css.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/languages/css/common/css.contribution", data); });
define("vs/nls!vs/languages/css/common/services/lintRules",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/languages/css/common/services/lintRules", data); });
define("vs/languages/css/common/services/lintRules", ["require", "exports", 'vs/nls!vs/languages/css/common/services/lintRules', 'vs/languages/css/common/level'], function (require, exports, nls, _level) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Warning = 'warning';
    var Error = 'error';
    var Ignore = 'ignore';
    var Rule = (function () {
        function Rule(id, message, defaultValue) {
            this.id = id;
            this.message = message;
            this.defaultValue = defaultValue;
            // nothing to do
        }
        Rule.prototype.getConfiguration = function () {
            return {
                type: 'string',
                enum: [Ignore, Warning, Error],
                default: this.defaultValue,
                description: this.message
            };
        };
        return Rule;
    }());
    exports.Rule = Rule;
    exports.Rules = {
        AllVendorPrefixes: new Rule('compatibleVendorPrefixes', nls.localize(0, null), Ignore),
        IncludeStandardPropertyWhenUsingVendorPrefix: new Rule('vendorPrefix', nls.localize(1, null), Warning),
        DuplicateDeclarations: new Rule('duplicateProperties', nls.localize(2, null), Ignore),
        EmptyRuleSet: new Rule('emptyRules', nls.localize(3, null), Warning),
        ImportStatemement: new Rule('importStatement', nls.localize(4, null), Ignore),
        NoWidthOrHeightWhenPaddingOrBorder: new Rule('boxModel', nls.localize(5, null), Ignore),
        UniversalSelector: new Rule('universalSelector', nls.localize(6, null), Ignore),
        ZeroWithUnit: new Rule('zeroUnits', nls.localize(7, null), Ignore),
        RequiredPropertiesForFontFace: new Rule('fontFaceProperties', nls.localize(8, null), Warning),
        HexColorLength: new Rule('hexColorLength', nls.localize(9, null), Error),
        ArgsInColorFunction: new Rule('argumentsInColorFunction', nls.localize(10, null), Error),
        UnknownProperty: new Rule('unknownProperties', nls.localize(11, null), Warning),
        IEStarHack: new Rule('ieHack', nls.localize(12, null), Ignore),
        UnknownVendorSpecificProperty: new Rule('unknownVendorSpecificProperties', nls.localize(13, null), Ignore),
        PropertyIgnoredDueToDisplay: new Rule('propertyIgnoredDueToDisplay', nls.localize(14, null), Warning),
        AvoidImportant: new Rule('important', nls.localize(15, null), Ignore),
        AvoidFloat: new Rule('float', nls.localize(16, null), Ignore),
        AvoidIdSelector: new Rule('idSelector', nls.localize(17, null), Ignore),
    };
    function getConfigurationProperties(keyPrefix) {
        var properties = {};
        properties[keyPrefix + '.validate'] = {
            type: 'boolean',
            default: true,
            description: nls.localize(18, null)
        };
        for (var ruleName in exports.Rules) {
            var rule = exports.Rules[ruleName];
            properties[keyPrefix + '.lint.' + rule.id] = rule.getConfiguration();
        }
        return properties;
    }
    exports.getConfigurationProperties = getConfigurationProperties;
    function sanitize(conf) {
        var settings = {};
        for (var ruleName in exports.Rules) {
            var rule = exports.Rules[ruleName];
            var level = _level.toLevel(conf[rule.id]);
            if (level) {
                settings[rule.id] = level;
            }
        }
        return settings;
    }
    exports.sanitize = sanitize;
});
/* old rules
        'duplicate-background-images' : {
            'type': 'string',
            'enum': ['ignore', 'warning', 'error'],
            'default': 'ignore',
            'description': nls.localize('duplicateBackgroundImages', "Every background-image should be unique. Use a common class for e.g. sprites.")
        },
        'gradients' : {
            'type': 'string',
            'enum': ['ignore', 'warning', 'error'],
            'default': 'warning',
            'description': nls.localize('gradients', "When using a vendor-prefixed gradient, make sure to use them all.")
        },
        'outline-none' : {
            'type': 'string',
            'enum': ['ignore', 'warning', 'error'],
            'default': 'warning',
            'description': nls.localize('outlineNone', "Use of outline: none or outline: 0 should be limited to :focus rules.")
        },
        'overqualified-elements' : {
            'type': 'string',
            'enum': ['ignore', 'warning', 'error'],
            'default': 'ignore',
            'description': nls.localize('overqualifiedElements', "Don't use classes or IDs with elements (a.foo or a#foo).")
        },
        'qualified-headings' : {
            'type': 'string',
            'enum': ['ignore', 'warning', 'error'],
            'default': 'ignore',
            'description': nls.localize('qualifiedHeadings', "Headings should not be qualified (namespaced).")
        },
        'regex-selectors' : {
            'type': 'string',
            'enum': ['ignore', 'warning', 'error'],
            'default': 'ignore',
            'description': nls.localize('regexSelectors', "Selectors that look like regular expressions are slow and should be avoided.")
        },
        'shorthand' : {
            'type': 'string',
            'enum': ['ignore', 'warning', 'error'],
            'default': 'ignore',
            'description': nls.localize('shorthand', "Use shorthand properties where possible.")
        },
        'text-indent' : {
            'type': 'string',
            'enum': ['ignore', 'warning', 'error'],
            'default': 'ignore',
            'description': nls.localize('textIndent', "Checks for text indent less than -99px.")
        },
        'unique-headings' : {
            'type': 'string',
            'enum': ['ignore', 'warning', 'error'],
            'default': 'ignore',
            'description': nls.localize('uniqueHeadings', "Headings should be defined only once.")
        },

        'unqualified-attributes' : {
            'type': 'string',
            'enum': ['ignore', 'warning', 'error'],
            'default': 'ignore',
            'description': nls.localize('unqualifiedAttributes', "Unqualified attribute selectors are known to be slow.")
        },
        */

define("vs/nls!vs/languages/html/common/html.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/languages/html/common/html.contribution", data); });
define("vs/nls!vs/languages/less/common/less.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/languages/less/common/less.contribution", data); });
define("vs/nls!vs/languages/markdown/common/markdown.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/languages/markdown/common/markdown.contribution", data); });
define("vs/nls!vs/languages/sass/common/sass.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/languages/sass/common/sass.contribution", data); });
define("vs/nls!vs/platform/actions/common/actionsService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/actions/common/actionsService", data); });
define("vs/nls!vs/platform/configuration/common/configurationRegistry",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/configuration/common/configurationRegistry", data); });
define("vs/nls!vs/platform/extensions/common/abstractExtensionService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/extensions/common/abstractExtensionService", data); });
define("vs/nls!vs/platform/extensions/common/extensionsRegistry",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/extensions/common/extensionsRegistry", data); });
define("vs/nls!vs/platform/extensions/node/extensionValidator",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/extensions/node/extensionValidator", data); });
define("vs/nls!vs/platform/jsonschemas/common/jsonContributionRegistry",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/jsonschemas/common/jsonContributionRegistry", data); });
define("vs/nls!vs/platform/jsonschemas/common/jsonValidationExtensionPoint",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/jsonschemas/common/jsonValidationExtensionPoint", data); });
define("vs/nls!vs/platform/keybinding/browser/keybindingServiceImpl",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/keybinding/browser/keybindingServiceImpl", data); });
define("vs/nls!vs/platform/markers/common/problemMatcher",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/markers/common/problemMatcher", data); });
define("vs/nls!vs/platform/message/common/message",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/message/common/message", data); });
define("vs/nls!vs/platform/telemetry/electron-browser/electronTelemetryService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/telemetry/electron-browser/electronTelemetryService", data); });
define("vs/nls!vs/platform/theme/common/themes",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/platform/theme/common/themes", data); });
define("vs/nls!vs/workbench/api/node/extHostMessageService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/api/node/extHostMessageService", data); });
define("vs/nls!vs/workbench/browser/actions/configureLocale",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/actions/configureLocale", data); });
define("vs/nls!vs/workbench/browser/actions/openSettings",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/actions/openSettings", data); });
define("vs/nls!vs/workbench/browser/actions/toggleSidebarPosition",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/actions/toggleSidebarPosition", data); });
define("vs/nls!vs/workbench/browser/actions/toggleSidebarVisibility",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/actions/toggleSidebarVisibility", data); });
define("vs/nls!vs/workbench/browser/actions/triggerEditorActions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/actions/triggerEditorActions", data); });
define("vs/nls!vs/workbench/browser/actions/triggerNavigation",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/actions/triggerNavigation", data); });
define("vs/nls!vs/workbench/browser/actions/triggerQuickOpen",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/actions/triggerQuickOpen", data); });
define("vs/nls!vs/workbench/browser/parts/activitybar/activityAction",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/activitybar/activityAction", data); });
define("vs/nls!vs/workbench/browser/parts/activitybar/activitybarPart",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/activitybar/activitybarPart", data); });
define("vs/nls!vs/workbench/browser/parts/compositePart",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/compositePart", data); });
define("vs/nls!vs/workbench/browser/parts/editor/binaryDiffEditor",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/editor/binaryDiffEditor", data); });
define("vs/nls!vs/workbench/browser/parts/editor/binaryEditor",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/editor/binaryEditor", data); });
define("vs/nls!vs/workbench/browser/parts/editor/editor.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/editor/editor.contribution", data); });
define("vs/nls!vs/workbench/browser/parts/editor/editorPart",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/editor/editorPart", data); });
define("vs/nls!vs/workbench/browser/parts/editor/editorStatus",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/editor/editorStatus", data); });
define("vs/nls!vs/workbench/browser/parts/editor/iframeEditor",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/editor/iframeEditor", data); });
define("vs/nls!vs/workbench/browser/parts/editor/sideBySideEditorControl",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/editor/sideBySideEditorControl", data); });
define("vs/nls!vs/workbench/browser/parts/editor/stringEditor",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/editor/stringEditor", data); });
define("vs/nls!vs/workbench/browser/parts/editor/textDiffEditor",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/editor/textDiffEditor", data); });
define("vs/nls!vs/workbench/browser/parts/panel/panelPart",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/panel/panelPart", data); });
define("vs/nls!vs/workbench/browser/parts/quickopen/editorHistoryModel",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/quickopen/editorHistoryModel", data); });
define("vs/nls!vs/workbench/browser/parts/quickopen/quickOpenController",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/quickopen/quickOpenController", data); });
define("vs/nls!vs/workbench/browser/parts/sidebar/sidebarPart",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/sidebar/sidebarPart", data); });
define("vs/nls!vs/workbench/browser/parts/statusbar/statusbarPart",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/parts/statusbar/statusbarPart", data); });
define("vs/nls!vs/workbench/browser/quickopen",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/quickopen", data); });
define("vs/nls!vs/workbench/browser/viewlet",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/browser/viewlet", data); });
define("vs/nls!vs/workbench/common/editor/diffEditorInput",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/common/editor/diffEditorInput", data); });
define("vs/nls!vs/workbench/electron-browser/actions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/electron-browser/actions", data); });
define("vs/nls!vs/workbench/electron-browser/crashReporter",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/electron-browser/crashReporter", data); });
define("vs/nls!vs/workbench/electron-browser/darwin/cli.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/electron-browser/darwin/cli.contribution", data); });
define("vs/nls!vs/workbench/electron-browser/integration",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/electron-browser/integration", data); });
define("vs/nls!vs/workbench/electron-browser/main.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/electron-browser/main.contribution", data); });
define("vs/nls!vs/workbench/electron-browser/shell",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/electron-browser/shell", data); });
define("vs/nls!vs/workbench/electron-browser/update",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/electron-browser/update", data); });
define("vs/nls!vs/workbench/parts/debug/browser/breakpointWidget",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/browser/breakpointWidget", data); });
define("vs/nls!vs/workbench/parts/debug/browser/debugEditorContribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/browser/debugEditorContribution", data); });
define("vs/nls!vs/workbench/parts/debug/browser/debugEditorModelManager",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/browser/debugEditorModelManager", data); });
define("vs/nls!vs/workbench/parts/debug/browser/debugHover",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/browser/debugHover", data); });
define("vs/nls!vs/workbench/parts/debug/browser/debugViewer",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/browser/debugViewer", data); });
define("vs/nls!vs/workbench/parts/debug/common/debugModel",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/common/debugModel", data); });
define("vs/nls!vs/workbench/parts/debug/electron-browser/debug.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/electron-browser/debug.contribution", data); });
define("vs/nls!vs/workbench/parts/debug/electron-browser/debugActions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/electron-browser/debugActions", data); });
define("vs/nls!vs/workbench/parts/debug/electron-browser/debugService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/electron-browser/debugService", data); });
define("vs/nls!vs/workbench/parts/debug/node/debugAdapter",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/node/debugAdapter", data); });
define("vs/nls!vs/workbench/parts/debug/node/debugConfigurationManager",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/node/debugConfigurationManager", data); });
define("vs/nls!vs/workbench/parts/debug/node/rawDebugSession",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/debug/node/rawDebugSession", data); });
define("vs/nls!vs/workbench/parts/emmet/node/emmet.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/emmet/node/emmet.contribution", data); });
define("vs/nls!vs/workbench/parts/errorList/browser/errorList.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/errorList/browser/errorList.contribution", data); });
define("vs/nls!vs/workbench/parts/execution/electron-browser/executionService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/execution/electron-browser/executionService", data); });
define("vs/nls!vs/workbench/parts/execution/electron-browser/terminal.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/execution/electron-browser/terminal.contribution", data); });
define("vs/nls!vs/workbench/parts/extensions/common/extensions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/extensions/common/extensions", data); });
define("vs/nls!vs/workbench/parts/extensions/electron-browser/extensionsActions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/extensions/electron-browser/extensionsActions", data); });
define("vs/nls!vs/workbench/parts/extensions/electron-browser/extensionsQuickOpen",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/extensions/electron-browser/extensionsQuickOpen", data); });
define("vs/nls!vs/workbench/parts/extensions/electron-browser/extensionsWidgets",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/extensions/electron-browser/extensionsWidgets", data); });
define("vs/nls!vs/workbench/parts/extensions/electron-browser/extensionsWorkbenchExtension",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/extensions/electron-browser/extensionsWorkbenchExtension", data); });
define("vs/nls!vs/workbench/parts/extensions/node/extensionsService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/extensions/node/extensionsService", data); });
define("vs/nls!vs/workbench/parts/feedback/browser/feedback",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/feedback/browser/feedback", data); });
define("vs/nls!vs/workbench/parts/files/browser/editors/binaryFileEditor",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/editors/binaryFileEditor", data); });
define("vs/nls!vs/workbench/parts/files/browser/editors/fileEditorInput",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/editors/fileEditorInput", data); });
define("vs/nls!vs/workbench/parts/files/browser/editors/textFileEditor",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/editors/textFileEditor", data); });
define("vs/nls!vs/workbench/parts/files/browser/fileActions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/fileActions", data); });
define("vs/nls!vs/workbench/parts/files/browser/fileActions.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/fileActions.contribution", data); });
define("vs/nls!vs/workbench/parts/files/browser/fileTracker",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/fileTracker", data); });
define("vs/nls!vs/workbench/parts/files/browser/files.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/files.contribution", data); });
define("vs/nls!vs/workbench/parts/files/browser/saveErrorHandler",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/saveErrorHandler", data); });
define("vs/nls!vs/workbench/parts/files/browser/views/emptyView",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/views/emptyView", data); });
define("vs/nls!vs/workbench/parts/files/browser/views/explorerView",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/views/explorerView", data); });
define("vs/nls!vs/workbench/parts/files/browser/views/explorerViewer",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/views/explorerViewer", data); });
define("vs/nls!vs/workbench/parts/files/browser/views/workingFilesView",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/views/workingFilesView", data); });
define("vs/nls!vs/workbench/parts/files/browser/views/workingFilesViewer",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/views/workingFilesViewer", data); });
define("vs/nls!vs/workbench/parts/files/browser/workingFilesPicker",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/browser/workingFilesPicker", data); });
define("vs/nls!vs/workbench/parts/files/common/editors/textFileEditorModel",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/common/editors/textFileEditorModel", data); });
define("vs/nls!vs/workbench/parts/files/electron-browser/electronFileActions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/electron-browser/electronFileActions", data); });
define("vs/nls!vs/workbench/parts/files/electron-browser/files.electron.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/electron-browser/files.electron.contribution", data); });
define("vs/nls!vs/workbench/parts/files/electron-browser/textFileServices",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/files/electron-browser/textFileServices", data); });
define("vs/nls!vs/workbench/parts/git/browser/gitActions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/git/browser/gitActions", data); });
define("vs/nls!vs/workbench/parts/git/browser/gitActions.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/git/browser/gitActions.contribution", data); });
define("vs/nls!vs/workbench/parts/git/browser/gitQuickOpen",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/git/browser/gitQuickOpen", data); });
define("vs/nls!vs/workbench/parts/git/browser/gitServices",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/git/browser/gitServices", data); });
define("vs/nls!vs/workbench/parts/git/browser/gitWidgets",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/git/browser/gitWidgets", data); });
define("vs/nls!vs/workbench/parts/git/browser/gitWorkbenchContributions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/git/browser/gitWorkbenchContributions", data); });
define("vs/nls!vs/workbench/parts/git/node/git.lib",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/git/node/git.lib", data); });
define("vs/nls!vs/workbench/parts/html/browser/html.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/html/browser/html.contribution", data); });
define("vs/nls!vs/workbench/parts/html/browser/htmlPreviewPart",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/html/browser/htmlPreviewPart", data); });
define("vs/nls!vs/workbench/parts/markdown/browser/markdownActions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/markdown/browser/markdownActions", data); });
define("vs/nls!vs/workbench/parts/markdown/browser/markdownActions.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/markdown/browser/markdownActions.contribution", data); });
define("vs/nls!vs/workbench/parts/markdown/common/markdownEditorInput",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/markdown/common/markdownEditorInput", data); });
define("vs/nls!vs/workbench/parts/markdown/common/markdownEditorModel",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/markdown/common/markdownEditorModel", data); });
define("vs/nls!vs/workbench/parts/output/browser/output.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/output/browser/output.contribution", data); });
define("vs/nls!vs/workbench/parts/output/browser/outputActions",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/output/browser/outputActions", data); });
define("vs/nls!vs/workbench/parts/output/common/outputEditorInput",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/output/common/outputEditorInput", data); });
define("vs/nls!vs/workbench/parts/quickopen/browser/commandsHandler",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/quickopen/browser/commandsHandler", data); });
define("vs/nls!vs/workbench/parts/quickopen/browser/gotoLineHandler",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/quickopen/browser/gotoLineHandler", data); });
define("vs/nls!vs/workbench/parts/quickopen/browser/gotoSymbolHandler",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/quickopen/browser/gotoSymbolHandler", data); });
define("vs/nls!vs/workbench/parts/quickopen/browser/helpHandler",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/quickopen/browser/helpHandler", data); });
define("vs/nls!vs/workbench/parts/quickopen/browser/markersHandler",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/quickopen/browser/markersHandler", data); });
define("vs/nls!vs/workbench/parts/quickopen/browser/quickopen.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/quickopen/browser/quickopen.contribution", data); });
define("vs/nls!vs/workbench/parts/search/browser/search.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/search/browser/search.contribution", data); });
define("vs/nls!vs/workbench/parts/snippets/electron-browser/snippets.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/snippets/electron-browser/snippets.contribution", data); });
define("vs/nls!vs/workbench/parts/tasks/browser/taskQuickOpen",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/tasks/browser/taskQuickOpen", data); });
define("vs/nls!vs/workbench/parts/tasks/common/taskTemplates",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/tasks/common/taskTemplates", data); });
define("vs/nls!vs/workbench/parts/tasks/electron-browser/task.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/tasks/electron-browser/task.contribution", data); });
define("vs/nls!vs/workbench/parts/tasks/node/processRunnerConfiguration",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/tasks/node/processRunnerConfiguration", data); });
define("vs/nls!vs/workbench/parts/tasks/node/processRunnerDetector",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/tasks/node/processRunnerDetector", data); });
define("vs/nls!vs/workbench/parts/tasks/node/processRunnerSystem",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/tasks/node/processRunnerSystem", data); });
define("vs/nls!vs/workbench/parts/themes/electron-browser/themes.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/themes/electron-browser/themes.contribution", data); });
define("vs/nls!vs/workbench/parts/update/electron-browser/update.contribution",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/parts/update/electron-browser/update.contribution", data); });
define("vs/nls!vs/workbench/services/files/electron-browser/fileService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/services/files/electron-browser/fileService", data); });
define("vs/nls!vs/workbench/services/files/node/fileService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/services/files/node/fileService", data); });
define("vs/nls!vs/workbench/services/history/browser/history",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/services/history/browser/history", data); });
define("vs/nls!vs/workbench/services/keybinding/electron-browser/keybindingService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/services/keybinding/electron-browser/keybindingService", data); });
define("vs/nls!vs/workbench/services/message/electron-browser/messageService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/services/message/electron-browser/messageService", data); });
define("vs/nls!vs/workbench/services/request/node/requestService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/services/request/node/requestService", data); });
define("vs/nls!vs/workbench/services/themes/node/themeService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/services/themes/node/themeService", data); });
define("vs/nls!vs/workbench/services/thread/electron-browser/threadService",['vs/nls', 'vs/nls!vs/workbench/workbench.main'], function(nls, data) { return nls.create("vs/workbench/services/thread/electron-browser/threadService", data); });
define("vs/platform/extensions/common/ipcRemoteCom", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/marshalling', 'vs/base/common/errors'], function (require, exports, winjs, marshalling, errors) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var pendingRPCReplies = {};
    function createRPC(serializeAndSend) {
        var lastMessageId = 0;
        return function rpc(rpcId, method, args) {
            var req = String(++lastMessageId);
            var reply = {
                c: null,
                e: null,
                p: null
            };
            var r = new winjs.TPromise(function (c, e, p) {
                reply.c = c;
                reply.e = e;
                reply.p = p;
            }, function () {
                serializeAndSend({
                    cancel: req
                });
            });
            pendingRPCReplies[req] = reply;
            serializeAndSend({
                req: req,
                rpcId: rpcId,
                method: method,
                args: args
            });
            return r;
        };
    }
    function create(send) {
        var rpc = createRPC(marshallAndSend);
        var bigHandler = null;
        var invokedHandlers = Object.create(null);
        var messagesToSend = [];
        var messagesToReceive = [];
        var receiveOneMessage = function () {
            var rawmsg = messagesToReceive.shift();
            if (messagesToReceive.length > 0) {
                process.nextTick(receiveOneMessage);
            }
            var msg = marshalling.parse(rawmsg);
            if (msg.seq) {
                if (!pendingRPCReplies.hasOwnProperty(msg.seq)) {
                    console.warn('Got reply to unknown seq');
                    return;
                }
                var reply = pendingRPCReplies[msg.seq];
                delete pendingRPCReplies[msg.seq];
                if (msg.err) {
                    var err = msg.err;
                    if (msg.err.$isError) {
                        err = new Error();
                        err.name = msg.err.name;
                        err.message = msg.err.message;
                        err.stack = msg.err.stack;
                    }
                    reply.e(err);
                    return;
                }
                reply.c(msg.res);
                return;
            }
            if (msg.cancel) {
                if (invokedHandlers[msg.cancel]) {
                    invokedHandlers[msg.cancel].cancel();
                }
                return;
            }
            if (msg.err) {
                console.error(msg.err);
                return;
            }
            var rpcId = msg.rpcId;
            if (!bigHandler) {
                throw new Error('got message before big handler attached!');
            }
            var req = msg.req;
            invokedHandlers[req] = invokeHandler(rpcId, msg.method, msg.args);
            invokedHandlers[req].then(function (r) {
                delete invokedHandlers[req];
                marshallAndSend({
                    seq: req,
                    res: r
                });
            }, function (err) {
                delete invokedHandlers[req];
                marshallAndSend({
                    seq: req,
                    err: errors.transformErrorForSerialization(err)
                });
            });
        };
        var r = {
            callOnRemote: rpc,
            setManyHandler: function (_bigHandler) {
                bigHandler = _bigHandler;
            },
            handle: function (rawmsg) {
                // console.log('RECEIVED ' + rawmsg.length + ' MESSAGES.');
                if (messagesToReceive.length === 0) {
                    process.nextTick(receiveOneMessage);
                }
                messagesToReceive = messagesToReceive.concat(rawmsg);
            }
        };
        function sendAccumulated() {
            var tmp = messagesToSend;
            messagesToSend = [];
            // console.log('SENDING ' + tmp.length + ' MESSAGES.');
            send(tmp);
        }
        function marshallAndSend(msg) {
            var value = marshalling.stringify(msg);
            if (messagesToSend.length === 0) {
                process.nextTick(sendAccumulated);
            }
            messagesToSend.push(value);
        }
        function invokeHandler(rpcId, method, args) {
            try {
                return winjs.TPromise.as(bigHandler.handle(rpcId, method, args));
            }
            catch (err) {
                return winjs.TPromise.wrapError(err);
            }
        }
        return r;
    }
    exports.create = create;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/instantiation/common/descriptors", ["require", "exports", 'vs/base/common/errors'], function (require, exports, errors) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var AbstractDescriptor = (function () {
        function AbstractDescriptor(_staticArguments) {
            this._staticArguments = _staticArguments;
            // empty
        }
        AbstractDescriptor.prototype.appendStaticArguments = function (more) {
            this._staticArguments.push.apply(this._staticArguments, more);
        };
        AbstractDescriptor.prototype.staticArguments = function (nth) {
            if (isNaN(nth)) {
                return this._staticArguments.slice(0);
            }
            else {
                return this._staticArguments[nth];
            }
        };
        AbstractDescriptor.prototype._validate = function (type) {
            if (!type) {
                throw errors.illegalArgument('can not be falsy');
            }
        };
        return AbstractDescriptor;
    }());
    exports.AbstractDescriptor = AbstractDescriptor;
    var SyncDescriptor = (function (_super) {
        __extends(SyncDescriptor, _super);
        function SyncDescriptor(_ctor) {
            var staticArguments = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                staticArguments[_i - 1] = arguments[_i];
            }
            _super.call(this, staticArguments);
            this._ctor = _ctor;
        }
        Object.defineProperty(SyncDescriptor.prototype, "ctor", {
            get: function () {
                return this._ctor;
            },
            enumerable: true,
            configurable: true
        });
        SyncDescriptor.prototype.equals = function (other) {
            if (other === this) {
                return true;
            }
            if (!(other instanceof SyncDescriptor)) {
                return false;
            }
            return other.ctor === this.ctor;
        };
        SyncDescriptor.prototype.bind = function () {
            var moreStaticArguments = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                moreStaticArguments[_i - 0] = arguments[_i];
            }
            var allArgs = [];
            allArgs = allArgs.concat(this.staticArguments());
            allArgs = allArgs.concat(moreStaticArguments);
            return new (SyncDescriptor.bind.apply(SyncDescriptor, [void 0].concat([this._ctor], allArgs)))();
        };
        return SyncDescriptor;
    }(AbstractDescriptor));
    exports.SyncDescriptor = SyncDescriptor;
    exports.createSyncDescriptor = function (ctor) {
        var staticArguments = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            staticArguments[_i - 1] = arguments[_i];
        }
        return new (SyncDescriptor.bind.apply(SyncDescriptor, [void 0].concat([ctor], staticArguments)))();
    };
    var AsyncDescriptor = (function (_super) {
        __extends(AsyncDescriptor, _super);
        function AsyncDescriptor(_moduleName, _ctorName) {
            var staticArguments = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                staticArguments[_i - 2] = arguments[_i];
            }
            _super.call(this, staticArguments);
            this._moduleName = _moduleName;
            this._ctorName = _ctorName;
            if (typeof _moduleName !== 'string') {
                throw new Error('Invalid AsyncDescriptor arguments, expected `moduleName` to be a string!');
            }
        }
        AsyncDescriptor.create = function (moduleName, ctorName) {
            return new AsyncDescriptor(moduleName, ctorName);
        };
        Object.defineProperty(AsyncDescriptor.prototype, "moduleName", {
            get: function () {
                return this._moduleName;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AsyncDescriptor.prototype, "ctorName", {
            get: function () {
                return this._ctorName;
            },
            enumerable: true,
            configurable: true
        });
        AsyncDescriptor.prototype.equals = function (other) {
            if (other === this) {
                return true;
            }
            if (!(other instanceof AsyncDescriptor)) {
                return false;
            }
            return other.moduleName === this.moduleName &&
                other.ctorName === this.ctorName;
        };
        AsyncDescriptor.prototype.bind = function () {
            var moreStaticArguments = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                moreStaticArguments[_i - 0] = arguments[_i];
            }
            var allArgs = [];
            allArgs = allArgs.concat(this.staticArguments());
            allArgs = allArgs.concat(moreStaticArguments);
            return new (AsyncDescriptor.bind.apply(AsyncDescriptor, [void 0].concat([this.moduleName, this.ctorName], allArgs)))();
        };
        return AsyncDescriptor;
    }(AbstractDescriptor));
    exports.AsyncDescriptor = AsyncDescriptor;
    var _createAsyncDescriptor = function (moduleName, ctorName) {
        var staticArguments = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            staticArguments[_i - 2] = arguments[_i];
        }
        return new (AsyncDescriptor.bind.apply(AsyncDescriptor, [void 0].concat([moduleName, ctorName], staticArguments)))();
    };
    exports.createAsyncDescriptor0 = _createAsyncDescriptor;
    exports.createAsyncDescriptor1 = _createAsyncDescriptor;
    exports.createAsyncDescriptor2 = _createAsyncDescriptor;
    exports.createAsyncDescriptor3 = _createAsyncDescriptor;
    exports.createAsyncDescriptor4 = _createAsyncDescriptor;
    exports.createAsyncDescriptor5 = _createAsyncDescriptor;
    exports.createAsyncDescriptor6 = _createAsyncDescriptor;
    exports.createAsyncDescriptor7 = _createAsyncDescriptor;
});

define("vs/platform/instantiation/common/extensions", ["require", "exports", './descriptors'], function (require, exports, descriptors_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.Services = 'di.services';
    var _registry = [];
    function registerSingleton(id, ctor) {
        _registry.push({ id: id, descriptor: new descriptors_1.SyncDescriptor(ctor) });
    }
    exports.registerSingleton = registerSingleton;
    function getServices() {
        return _registry;
    }
    exports.getServices = getServices;
});

define("vs/platform/instantiation/common/instantiation", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // ----------------------- internal util -----------------------
    var _util;
    (function (_util) {
        _util.DI_TARGET = '$di$target';
        _util.DI_DEPENDENCIES = '$di$dependencies';
        _util.DI_PROVIDES = '$di$provides_service';
        function getServiceId(id) {
            return id[_util.DI_PROVIDES];
        }
        _util.getServiceId = getServiceId;
        function getServiceDependencies(ctor) {
            return ctor[_util.DI_DEPENDENCIES];
        }
        _util.getServiceDependencies = getServiceDependencies;
    })(_util = exports._util || (exports._util = {}));
    exports.IInstantiationService = createDecorator('instantiationService');
    /**
     * A *only* valid way to create a {{ServiceIdentifier}}.
     */
    function createDecorator(serviceId) {
        var ret = function (target, key, index) {
            if (arguments.length !== 3) {
                throw new Error('@IServiceName-decorator can only be used to decorate a parameter');
            }
            if (target[_util.DI_TARGET] === target) {
                target[_util.DI_DEPENDENCIES].push({ serviceId: serviceId, index: index });
            }
            else {
                target[_util.DI_DEPENDENCIES] = [{ serviceId: serviceId, index: index }];
                target[_util.DI_TARGET] = target;
            }
        };
        ret[_util.DI_PROVIDES] = serviceId;
        // ret['type'] = undefined;
        return ret;
    }
    exports.createDecorator = createDecorator;
});

define("vs/editor/common/services/codeEditorService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.ID_CODE_EDITOR_SERVICE = 'codeEditorService';
    exports.ICodeEditorService = instantiation_1.createDecorator(exports.ID_CODE_EDITOR_SERVICE);
});

define("vs/editor/common/services/abstractCodeEditorService", ["require", "exports", 'vs/base/common/event', 'vs/editor/common/services/codeEditorService'], function (require, exports, event_1, codeEditorService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var AbstractCodeEditorService = (function () {
        function AbstractCodeEditorService() {
            this.serviceId = codeEditorService_1.ICodeEditorService;
            this._codeEditors = Object.create(null);
            this._onCodeEditorAdd = new event_1.Emitter();
            this._onCodeEditorRemove = new event_1.Emitter();
        }
        AbstractCodeEditorService.prototype.addCodeEditor = function (editor) {
            this._codeEditors[editor.getId()] = editor;
            this._onCodeEditorAdd.fire(editor);
        };
        Object.defineProperty(AbstractCodeEditorService.prototype, "onCodeEditorAdd", {
            get: function () {
                return this._onCodeEditorAdd.event;
            },
            enumerable: true,
            configurable: true
        });
        AbstractCodeEditorService.prototype.removeCodeEditor = function (editor) {
            if (delete this._codeEditors[editor.getId()]) {
                this._onCodeEditorRemove.fire(editor);
            }
        };
        Object.defineProperty(AbstractCodeEditorService.prototype, "onCodeEditorRemove", {
            get: function () {
                return this._onCodeEditorRemove.event;
            },
            enumerable: true,
            configurable: true
        });
        AbstractCodeEditorService.prototype.getCodeEditor = function (editorId) {
            return this._codeEditors[editorId] || null;
        };
        AbstractCodeEditorService.prototype.listCodeEditors = function () {
            var _this = this;
            return Object.keys(this._codeEditors).map(function (id) { return _this._codeEditors[id]; });
        };
        return AbstractCodeEditorService;
    }());
    exports.AbstractCodeEditorService = AbstractCodeEditorService;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/services/codeEditorServiceImpl", ["require", "exports", 'vs/base/common/objects', 'vs/base/common/strings', 'vs/base/common/uri', 'vs/base/browser/dom', 'vs/editor/common/editorCommon', 'vs/editor/common/services/abstractCodeEditorService'], function (require, exports, objects, strings, uri_1, dom, editorCommon_1, abstractCodeEditorService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var CodeEditorServiceImpl = (function (_super) {
        __extends(CodeEditorServiceImpl, _super);
        function CodeEditorServiceImpl() {
            _super.call(this);
            this._styleSheet = dom.createStyleSheet();
            this._decorationRenderOptions = Object.create(null);
        }
        CodeEditorServiceImpl.prototype.registerDecorationType = function (key, options) {
            if (this._decorationRenderOptions[key]) {
                this._decorationRenderOptions[key].dispose();
                delete this._decorationRenderOptions[key];
            }
            var decorationRenderOptions = new DecorationRenderOptions(this._styleSheet, key, options);
            this._decorationRenderOptions[key] = decorationRenderOptions;
        };
        CodeEditorServiceImpl.prototype.removeDecorationType = function (key) {
            if (this._decorationRenderOptions[key]) {
                this._decorationRenderOptions[key].dispose();
                delete this._decorationRenderOptions[key];
                this.listCodeEditors().forEach(function (ed) { return ed.removeDecorations(key); });
            }
        };
        CodeEditorServiceImpl.prototype.resolveDecorationType = function (key) {
            if (this._decorationRenderOptions[key]) {
                return this._decorationRenderOptions[key];
            }
            throw new Error('Unknown decoration type key: ' + key);
        };
        return CodeEditorServiceImpl;
    }(abstractCodeEditorService_1.AbstractCodeEditorService));
    exports.CodeEditorServiceImpl = CodeEditorServiceImpl;
    var DecorationRenderOptions = (function () {
        function DecorationRenderOptions(styleSheet, key, options) {
            var themedOpts = resolveDecorationRenderOptions(options);
            this._styleSheet = styleSheet;
            this._key = key;
            this.className = DecorationRenderOptions._handle(this._styleSheet, this._key, ModelDecorationCSSRuleType.ClassName, DecorationRenderOptions._getCSSTextForModelDecorationClassName(themedOpts.light), DecorationRenderOptions._getCSSTextForModelDecorationClassName(themedOpts.dark));
            this.inlineClassName = DecorationRenderOptions._handle(this._styleSheet, this._key, ModelDecorationCSSRuleType.InlineClassName, DecorationRenderOptions._getCSSTextForModelDecorationInlineClassName(themedOpts.light), DecorationRenderOptions._getCSSTextForModelDecorationInlineClassName(themedOpts.dark));
            this.glyphMarginClassName = DecorationRenderOptions._handle(this._styleSheet, this._key, ModelDecorationCSSRuleType.GlyphMarginClassName, DecorationRenderOptions._getCSSTextForModelDecorationGlyphMarginClassName(themedOpts.light), DecorationRenderOptions._getCSSTextForModelDecorationGlyphMarginClassName(themedOpts.dark));
            this.isWholeLine = Boolean(options.isWholeLine);
            if (typeof themedOpts.light.overviewRulerColor !== 'undefined'
                || typeof themedOpts.dark.overviewRulerColor !== 'undefined') {
                this.overviewRuler = {
                    color: themedOpts.light.overviewRulerColor || themedOpts.dark.overviewRulerColor,
                    darkColor: themedOpts.dark.overviewRulerColor || themedOpts.light.overviewRulerColor,
                    position: options.overviewRulerLane || editorCommon_1.OverviewRulerLane.Center
                };
            }
        }
        DecorationRenderOptions.prototype.dispose = function () {
            dom.removeCSSRulesWithPrefix(CSSNameHelper.getDeletionPrefixFor(ThemeType.Light, this._key), this._styleSheet);
            dom.removeCSSRulesWithPrefix(CSSNameHelper.getDeletionPrefixFor(ThemeType.Dark, this._key), this._styleSheet);
        };
        /**
         * Build the CSS for decorations styled via `className`.
         */
        DecorationRenderOptions._getCSSTextForModelDecorationClassName = function (opts) {
            var cssTextArr = [];
            if (typeof opts.backgroundColor !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.backgroundColor, opts.backgroundColor));
            }
            if (typeof opts.outlineColor !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.outlineColor, opts.outlineColor));
            }
            if (typeof opts.outlineStyle !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.outlineStyle, opts.outlineStyle));
            }
            if (typeof opts.outlineWidth !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.outlineWidth, opts.outlineWidth));
            }
            if (typeof opts.borderColor !== 'undefined'
                || typeof opts.borderRadius !== 'undefined'
                || typeof opts.borderSpacing !== 'undefined'
                || typeof opts.borderStyle !== 'undefined'
                || typeof opts.borderWidth !== 'undefined') {
                cssTextArr.push(strings.format('box-sizing: border-box;'));
            }
            if (typeof opts.borderColor !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.borderColor, opts.borderColor));
            }
            if (typeof opts.borderRadius !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.borderRadius, opts.borderRadius));
            }
            if (typeof opts.borderSpacing !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.borderSpacing, opts.borderSpacing));
            }
            if (typeof opts.borderStyle !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.borderStyle, opts.borderStyle));
            }
            if (typeof opts.borderWidth !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.borderWidth, opts.borderWidth));
            }
            return cssTextArr.join('');
        };
        /**
         * Build the CSS for decorations styled via `inlineClassName`.
         */
        DecorationRenderOptions._getCSSTextForModelDecorationInlineClassName = function (opts) {
            var cssTextArr = [];
            if (typeof opts.textDecoration !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.textDecoration, opts.textDecoration));
            }
            if (typeof opts.cursor !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.cursor, opts.cursor));
            }
            if (typeof opts.color !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.color, opts.color));
            }
            if (typeof opts.letterSpacing !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.letterSpacing, opts.letterSpacing));
            }
            return cssTextArr.join('');
        };
        /**
         * Build the CSS for decorations styled via `glpyhMarginClassName`.
         */
        DecorationRenderOptions._getCSSTextForModelDecorationGlyphMarginClassName = function (opts) {
            var cssTextArr = [];
            if (typeof opts.gutterIconPath !== 'undefined') {
                cssTextArr.push(strings.format(this._CSS_MAP.gutterIconPath, uri_1.default.file(opts.gutterIconPath).toString()));
            }
            return cssTextArr.join('');
        };
        DecorationRenderOptions._handle = function (styleSheet, key, ruleType, lightCSS, darkCSS) {
            if (lightCSS.length > 0 || darkCSS.length > 0) {
                if (lightCSS.length > 0) {
                    this._createCSSSelector(styleSheet, ThemeType.Light, key, ruleType, lightCSS);
                }
                if (darkCSS.length > 0) {
                    this._createCSSSelector(styleSheet, ThemeType.Dark, key, ruleType, darkCSS);
                }
                return CSSNameHelper.getClassName(key, ruleType);
            }
            return undefined;
        };
        DecorationRenderOptions._createCSSSelector = function (styleSheet, themeType, key, ruleType, cssText) {
            dom.createCSSRule(CSSNameHelper.getSelector(themeType, key, ruleType), cssText, styleSheet);
        };
        DecorationRenderOptions._CSS_MAP = {
            color: 'color:{0} !important;',
            backgroundColor: 'background-color:{0};',
            outlineColor: 'outline-color:{0};',
            outlineStyle: 'outline-style:{0};',
            outlineWidth: 'outline-width:{0};',
            borderColor: 'border-color:{0};',
            borderRadius: 'border-radius:{0};',
            borderSpacing: 'border-spacing:{0};',
            borderStyle: 'border-style:{0};',
            borderWidth: 'border-width:{0};',
            textDecoration: 'text-decoration:{0};',
            cursor: 'cursor:{0};',
            letterSpacing: 'letter-spacing:{0};',
            gutterIconPath: 'background:url(\'{0}\') center center no-repeat;',
        };
        return DecorationRenderOptions;
    }());
    var ThemeType;
    (function (ThemeType) {
        ThemeType[ThemeType["Light"] = 0] = "Light";
        ThemeType[ThemeType["Dark"] = 1] = "Dark";
    })(ThemeType || (ThemeType = {}));
    var ModelDecorationCSSRuleType;
    (function (ModelDecorationCSSRuleType) {
        ModelDecorationCSSRuleType[ModelDecorationCSSRuleType["ClassName"] = 0] = "ClassName";
        ModelDecorationCSSRuleType[ModelDecorationCSSRuleType["InlineClassName"] = 1] = "InlineClassName";
        ModelDecorationCSSRuleType[ModelDecorationCSSRuleType["GlyphMarginClassName"] = 2] = "GlyphMarginClassName";
    })(ModelDecorationCSSRuleType || (ModelDecorationCSSRuleType = {}));
    var CSSNameHelper = (function () {
        function CSSNameHelper() {
        }
        CSSNameHelper._getSelectorPrefixOf = function (theme) {
            if (theme === ThemeType.Light) {
                return '.monaco-editor.vs';
            }
            return '.monaco-editor.vs-dark';
        };
        CSSNameHelper.getClassName = function (key, type) {
            return 'ced-' + key + '-' + type;
        };
        CSSNameHelper.getSelector = function (themeType, key, ruleType) {
            return this._getSelectorPrefixOf(themeType) + ' .' + this.getClassName(key, ruleType);
        };
        CSSNameHelper.getDeletionPrefixFor = function (themeType, key) {
            return this._getSelectorPrefixOf(themeType) + ' .ced-' + key;
        };
        return CSSNameHelper;
    }());
    function resolveDecorationRenderOptions(opts) {
        var light = objects.deepClone(opts);
        objects.mixin(light, opts.light);
        var dark = objects.deepClone(opts);
        objects.mixin(dark, opts.dark);
        return {
            light: light,
            dark: dark
        };
    }
});

define("vs/editor/common/services/editorWorkerService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.ID_EDITOR_WORKER_SERVICE = 'editorWorkerService';
    exports.IEditorWorkerService = instantiation_1.createDecorator(exports.ID_EDITOR_WORKER_SERVICE);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/common/modes/abstractMode", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/base/common/winjs.base', 'vs/platform/instantiation/common/descriptors', 'vs/editor/common/modes/nullMode', 'vs/editor/common/modes/supports/suggestSupport', 'vs/editor/common/services/editorWorkerService'], function (require, exports, eventEmitter_1, winjs_base_1, descriptors_1, nullMode_1, suggestSupport_1, editorWorkerService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function createWordRegExp(allowInWords) {
        if (allowInWords === void 0) { allowInWords = ''; }
        return nullMode_1.NullMode.createWordRegExp(allowInWords);
    }
    exports.createWordRegExp = createWordRegExp;
    var ModeWorkerManager = (function () {
        function ModeWorkerManager(descriptor, workerModuleId, workerClassName, superWorkerModuleId, instantiationService) {
            this._descriptor = descriptor;
            this._workerDescriptor = descriptors_1.createAsyncDescriptor2(workerModuleId, workerClassName);
            this._superWorkerModuleId = superWorkerModuleId;
            this._instantiationService = instantiationService;
            this._workerPiecePromise = null;
        }
        ModeWorkerManager.prototype.worker = function (runner) {
            return this._getOrCreateWorker().then(runner);
        };
        ModeWorkerManager.prototype._getOrCreateWorker = function () {
            var _this = this;
            if (!this._workerPiecePromise) {
                // TODO@Alex: workaround for missing `bundles` config
                // First, load the code of the worker super class
                var superWorkerCodePromise = (this._superWorkerModuleId ? ModeWorkerManager._loadModule(this._superWorkerModuleId) : winjs_base_1.TPromise.as(null));
                this._workerPiecePromise = superWorkerCodePromise.then(function () {
                    // Second, load the code of the worker (without instantiating it)
                    return ModeWorkerManager._loadModule(_this._workerDescriptor.moduleName);
                }).then(function () {
                    // Then, load & instantiate all the participants
                    var participants = _this._descriptor.workerParticipants;
                    return winjs_base_1.TPromise.join(participants.map(function (participant) {
                        return _this._instantiationService.createInstance(participant);
                    }));
                }).then(function (participants) {
                    // Finally, create the mode worker instance
                    return _this._instantiationService.createInstance(_this._workerDescriptor, _this._descriptor.id, participants);
                });
            }
            return this._workerPiecePromise;
        };
        ModeWorkerManager._loadModule = function (moduleName) {
            return new winjs_base_1.TPromise(function (c, e, p) {
                require([moduleName], c, e);
            }, function () {
                // Cannot cancel loading code
            });
        };
        return ModeWorkerManager;
    }());
    exports.ModeWorkerManager = ModeWorkerManager;
    var AbstractMode = (function () {
        function AbstractMode(modeId) {
            this._modeId = modeId;
            this._eventEmitter = new eventEmitter_1.EventEmitter();
            this._simplifiedMode = null;
        }
        AbstractMode.prototype.getId = function () {
            return this._modeId;
        };
        AbstractMode.prototype.toSimplifiedMode = function () {
            if (!this._simplifiedMode) {
                this._simplifiedMode = new SimplifiedMode(this);
            }
            return this._simplifiedMode;
        };
        AbstractMode.prototype.addSupportChangedListener = function (callback) {
            return this._eventEmitter.addListener2('modeSupportChanged', callback);
        };
        AbstractMode.prototype.registerSupport = function (support, callback) {
            var _this = this;
            var supportImpl = callback(this);
            this[support] = supportImpl;
            this._eventEmitter.emit('modeSupportChanged', _createModeSupportChangedEvent(support));
            return {
                dispose: function () {
                    if (_this[support] === supportImpl) {
                        delete _this[support];
                        _this._eventEmitter.emit('modeSupportChanged', _createModeSupportChangedEvent(support));
                    }
                }
            };
        };
        return AbstractMode;
    }());
    exports.AbstractMode = AbstractMode;
    var SimplifiedMode = (function () {
        function SimplifiedMode(sourceMode) {
            var _this = this;
            this._sourceMode = sourceMode;
            this._eventEmitter = new eventEmitter_1.EventEmitter();
            this._id = 'vs.editor.modes.simplifiedMode:' + sourceMode.getId();
            this._assignSupports();
            if (this._sourceMode.addSupportChangedListener) {
                this._sourceMode.addSupportChangedListener(function (e) {
                    if (e.tokenizationSupport || e.richEditSupport) {
                        _this._assignSupports();
                        var newEvent = SimplifiedMode._createModeSupportChangedEvent(e);
                        _this._eventEmitter.emit('modeSupportChanged', newEvent);
                    }
                });
            }
        }
        SimplifiedMode.prototype.getId = function () {
            return this._id;
        };
        SimplifiedMode.prototype.toSimplifiedMode = function () {
            return this;
        };
        SimplifiedMode.prototype._assignSupports = function () {
            this.tokenizationSupport = this._sourceMode.tokenizationSupport;
            this.richEditSupport = this._sourceMode.richEditSupport;
        };
        SimplifiedMode._createModeSupportChangedEvent = function (originalModeEvent) {
            var event = {
                codeLensSupport: false,
                tokenizationSupport: originalModeEvent.tokenizationSupport,
                occurrencesSupport: false,
                declarationSupport: false,
                typeDeclarationSupport: false,
                navigateTypesSupport: false,
                referenceSupport: false,
                suggestSupport: false,
                parameterHintsSupport: false,
                extraInfoSupport: false,
                outlineSupport: false,
                logicalSelectionSupport: false,
                formattingSupport: false,
                inplaceReplaceSupport: false,
                emitOutputSupport: false,
                linkSupport: false,
                configSupport: false,
                quickFixSupport: false,
                richEditSupport: originalModeEvent.richEditSupport,
            };
            return event;
        };
        return SimplifiedMode;
    }());
    exports.isDigit = (function () {
        var _0 = '0'.charCodeAt(0), _1 = '1'.charCodeAt(0), _2 = '2'.charCodeAt(0), _3 = '3'.charCodeAt(0), _4 = '4'.charCodeAt(0), _5 = '5'.charCodeAt(0), _6 = '6'.charCodeAt(0), _7 = '7'.charCodeAt(0), _8 = '8'.charCodeAt(0), _9 = '9'.charCodeAt(0), _a = 'a'.charCodeAt(0), _b = 'b'.charCodeAt(0), _c = 'c'.charCodeAt(0), _d = 'd'.charCodeAt(0), _e = 'e'.charCodeAt(0), _f = 'f'.charCodeAt(0), _A = 'A'.charCodeAt(0), _B = 'B'.charCodeAt(0), _C = 'C'.charCodeAt(0), _D = 'D'.charCodeAt(0), _E = 'E'.charCodeAt(0), _F = 'F'.charCodeAt(0);
        return function isDigit(character, base) {
            var c = character.charCodeAt(0);
            switch (base) {
                case 1:
                    return c === _0;
                case 2:
                    return c >= _0 && c <= _1;
                case 3:
                    return c >= _0 && c <= _2;
                case 4:
                    return c >= _0 && c <= _3;
                case 5:
                    return c >= _0 && c <= _4;
                case 6:
                    return c >= _0 && c <= _5;
                case 7:
                    return c >= _0 && c <= _6;
                case 8:
                    return c >= _0 && c <= _7;
                case 9:
                    return c >= _0 && c <= _8;
                case 10:
                    return c >= _0 && c <= _9;
                case 11:
                    return (c >= _0 && c <= _9) || (c === _a) || (c === _A);
                case 12:
                    return (c >= _0 && c <= _9) || (c >= _a && c <= _b) || (c >= _A && c <= _B);
                case 13:
                    return (c >= _0 && c <= _9) || (c >= _a && c <= _c) || (c >= _A && c <= _C);
                case 14:
                    return (c >= _0 && c <= _9) || (c >= _a && c <= _d) || (c >= _A && c <= _D);
                case 15:
                    return (c >= _0 && c <= _9) || (c >= _a && c <= _e) || (c >= _A && c <= _E);
                default:
                    return (c >= _0 && c <= _9) || (c >= _a && c <= _f) || (c >= _A && c <= _F);
            }
        };
    })();
    var FrankensteinMode = (function (_super) {
        __extends(FrankensteinMode, _super);
        function FrankensteinMode(descriptor, editorWorkerService) {
            _super.call(this, descriptor.id);
            if (editorWorkerService) {
                this.suggestSupport = new suggestSupport_1.TextualSuggestSupport(this.getId(), editorWorkerService);
            }
        }
        FrankensteinMode = __decorate([
            __param(1, editorWorkerService_1.IEditorWorkerService)
        ], FrankensteinMode);
        return FrankensteinMode;
    }(AbstractMode));
    exports.FrankensteinMode = FrankensteinMode;
    function _createModeSupportChangedEvent() {
        var changedSupports = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            changedSupports[_i - 0] = arguments[_i];
        }
        var event = {
            codeLensSupport: false,
            tokenizationSupport: false,
            occurrencesSupport: false,
            declarationSupport: false,
            typeDeclarationSupport: false,
            navigateTypesSupport: false,
            referenceSupport: false,
            suggestSupport: false,
            parameterHintsSupport: false,
            extraInfoSupport: false,
            outlineSupport: false,
            logicalSelectionSupport: false,
            formattingSupport: false,
            inplaceReplaceSupport: false,
            emitOutputSupport: false,
            linkSupport: false,
            configSupport: false,
            quickFixSupport: false,
            richEditSupport: false
        };
        changedSupports.forEach(function (support) { return event[support] = true; });
        return event;
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/modes/monarch/monarch", ["require", "exports", 'vs/editor/common/modes/abstractMode', 'vs/editor/common/modes/monarch/monarchDefinition', 'vs/editor/common/modes/monarch/monarchLexer', 'vs/editor/common/modes/supports/richEditSupport'], function (require, exports, abstractMode_1, monarchDefinition_1, monarchLexer_1, richEditSupport_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The MonarchMode creates a Monaco language mode given a certain language description
     */
    var MonarchMode = (function (_super) {
        __extends(MonarchMode, _super);
        function MonarchMode(modeId, lexer, modeService, modelService, editorWorkerService) {
            _super.call(this, modeId);
            this.tokenizationSupport = monarchLexer_1.createTokenizationSupport(modeService, this, lexer);
            this.richEditSupport = new richEditSupport_1.RichEditSupport(this.getId(), null, monarchDefinition_1.createRichEditSupport(lexer));
            this.suggestSupport = monarchDefinition_1.createSuggestSupport(modelService, editorWorkerService, this.getId(), lexer);
        }
        return MonarchMode;
    }(abstractMode_1.AbstractMode));
    exports.MonarchMode = MonarchMode;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/services/editorWorkerServiceImpl", ["require", "exports", 'vs/base/common/async', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/base/common/worker/simpleWorker', 'vs/base/worker/defaultWorkerFactory', 'vs/editor/common/editorCommon', 'vs/editor/common/model/textModelWithTokensHelpers', 'vs/editor/common/services/editorSimpleWorkerCommon', 'vs/editor/common/services/editorWorkerService'], function (require, exports, async_1, lifecycle_1, winjs_base_1, simpleWorker_1, defaultWorkerFactory_1, editorCommon, textModelWithTokensHelpers_1, editorSimpleWorkerCommon_1, editorWorkerService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Stop syncing a model to the worker if it was not needed for 1 min.
     */
    var STOP_SYNC_MODEL_DELTA_TIME_MS = 60 * 1000;
    /**
     * Stop the worker if it was not needed for 5 min.
     */
    var STOP_WORKER_DELTA_TIME_MS = 5 * 60 * 1000;
    var EditorWorkerServiceImpl = (function () {
        function EditorWorkerServiceImpl(modelService) {
            this.serviceId = editorWorkerService_1.IEditorWorkerService;
            this._workerManager = new WorkerManager(modelService);
        }
        EditorWorkerServiceImpl.prototype.computeDiff = function (original, modified, ignoreTrimWhitespace) {
            return this._workerManager.withWorker().then(function (client) { return client.computeDiff(original, modified, ignoreTrimWhitespace); });
        };
        EditorWorkerServiceImpl.prototype.computeDirtyDiff = function (original, modified, ignoreTrimWhitespace) {
            return this._workerManager.withWorker().then(function (client) { return client.computeDirtyDiff(original, modified, ignoreTrimWhitespace); });
        };
        EditorWorkerServiceImpl.prototype.computeLinks = function (resource) {
            return this._workerManager.withWorker().then(function (client) { return client.computeLinks(resource); });
        };
        EditorWorkerServiceImpl.prototype.textualSuggest = function (resource, position) {
            return this._workerManager.withWorker().then(function (client) { return client.textualSuggest(resource, position); });
        };
        EditorWorkerServiceImpl.prototype.navigateValueSet = function (resource, range, up) {
            return this._workerManager.withWorker().then(function (client) { return client.navigateValueSet(resource, range, up); });
        };
        return EditorWorkerServiceImpl;
    }());
    exports.EditorWorkerServiceImpl = EditorWorkerServiceImpl;
    var WorkerManager = (function (_super) {
        __extends(WorkerManager, _super);
        function WorkerManager(modelService) {
            var _this = this;
            _super.call(this);
            this._modelService = modelService;
            this._editorWorkerClient = null;
            var stopWorkerInterval = this._register(new async_1.IntervalTimer());
            stopWorkerInterval.cancelAndSet(function () { return _this._checkStopWorker(); }, Math.round(STOP_WORKER_DELTA_TIME_MS / 2));
        }
        WorkerManager.prototype.dispose = function () {
            if (this._editorWorkerClient) {
                this._editorWorkerClient.dispose();
                this._editorWorkerClient = null;
            }
            _super.prototype.dispose.call(this);
        };
        WorkerManager.prototype._checkStopWorker = function () {
            if (!this._editorWorkerClient) {
                return;
            }
            var timeSinceLastWorkerUsedTime = (new Date()).getTime() - this._lastWorkerUsedTime;
            if (timeSinceLastWorkerUsedTime > STOP_WORKER_DELTA_TIME_MS) {
                this._editorWorkerClient.dispose();
                this._editorWorkerClient = null;
            }
        };
        WorkerManager.prototype.withWorker = function () {
            this._lastWorkerUsedTime = (new Date()).getTime();
            if (!this._editorWorkerClient) {
                this._editorWorkerClient = new EditorWorkerClient(this._modelService);
            }
            return winjs_base_1.TPromise.as(this._editorWorkerClient);
        };
        return WorkerManager;
    }(lifecycle_1.Disposable));
    var EditorWorkerClient = (function (_super) {
        __extends(EditorWorkerClient, _super);
        function EditorWorkerClient(modelService) {
            var _this = this;
            _super.call(this);
            this._modelService = modelService;
            this._syncedModels = Object.create(null);
            this._syncedModelsLastUsedTime = Object.create(null);
            this._worker = this._register(new simpleWorker_1.SimpleWorkerClient(new defaultWorkerFactory_1.DefaultWorkerFactory(), 'vs/editor/common/services/editorSimpleWorker', editorSimpleWorkerCommon_1.EditorSimpleWorker));
            this._proxy = this._worker.get();
            var stopModelSyncInterval = this._register(new async_1.IntervalTimer());
            stopModelSyncInterval.cancelAndSet(function () { return _this._checkStopModelSync(); }, Math.round(STOP_SYNC_MODEL_DELTA_TIME_MS / 2));
        }
        EditorWorkerClient.prototype.dispose = function () {
            for (var modelUrl in this._syncedModels) {
                lifecycle_1.disposeAll(this._syncedModels[modelUrl]);
            }
            this._syncedModels = Object.create(null);
            this._syncedModelsLastUsedTime = Object.create(null);
            _super.prototype.dispose.call(this);
        };
        EditorWorkerClient.prototype._checkStopModelSync = function () {
            var currentTime = (new Date()).getTime();
            var toRemove = [];
            for (var modelUrl in this._syncedModelsLastUsedTime) {
                var elapsedTime = currentTime - this._syncedModelsLastUsedTime[modelUrl];
                if (elapsedTime > STOP_SYNC_MODEL_DELTA_TIME_MS) {
                    toRemove.push(modelUrl);
                }
            }
            for (var i = 0; i < toRemove.length; i++) {
                this._stopModelSync(toRemove[i]);
            }
        };
        EditorWorkerClient.prototype.computeDiff = function (original, modified, ignoreTrimWhitespace) {
            var _this = this;
            return this._withSyncedResources([original, modified]).then(function (_) {
                return _this._proxy.computeDiff(original.toString(), modified.toString(), ignoreTrimWhitespace);
            });
        };
        EditorWorkerClient.prototype.computeDirtyDiff = function (original, modified, ignoreTrimWhitespace) {
            var _this = this;
            return this._withSyncedResources([original, modified]).then(function (_) {
                return _this._proxy.computeDirtyDiff(original.toString(), modified.toString(), ignoreTrimWhitespace);
            });
        };
        EditorWorkerClient.prototype.computeLinks = function (resource) {
            var _this = this;
            return this._withSyncedResources([resource]).then(function (_) {
                return _this._proxy.computeLinks(resource.toString());
            });
        };
        EditorWorkerClient.prototype.textualSuggest = function (resource, position) {
            var _this = this;
            return this._withSyncedResources([resource]).then(function (_) {
                var model = _this._modelService.getModel(resource);
                if (!model) {
                    return null;
                }
                var wordDefRegExp = textModelWithTokensHelpers_1.WordHelper.massageWordDefinitionOf(model.getMode());
                var wordDef = wordDefRegExp.source;
                var wordDefFlags = (wordDefRegExp.global ? 'g' : '') + (wordDefRegExp.ignoreCase ? 'i' : '') + (wordDefRegExp.multiline ? 'm' : '');
                return _this._proxy.textualSuggest(resource.toString(), position, wordDef, wordDefFlags);
            });
        };
        EditorWorkerClient.prototype.navigateValueSet = function (resource, range, up) {
            var _this = this;
            return this._withSyncedResources([resource]).then(function (_) {
                var model = _this._modelService.getModel(resource);
                if (!model) {
                    return null;
                }
                var wordDefRegExp = textModelWithTokensHelpers_1.WordHelper.massageWordDefinitionOf(model.getMode());
                var wordDef = wordDefRegExp.source;
                var wordDefFlags = (wordDefRegExp.global ? 'g' : '') + (wordDefRegExp.ignoreCase ? 'i' : '') + (wordDefRegExp.multiline ? 'm' : '');
                return _this._proxy.navigateValueSet(resource.toString(), range, up, wordDef, wordDefFlags);
            });
        };
        EditorWorkerClient.prototype._withSyncedResources = function (resources) {
            for (var i = 0; i < resources.length; i++) {
                var resource = resources[i];
                var resourceStr = resource.toString();
                if (!this._syncedModels[resourceStr]) {
                    this._beginModelSync(resource);
                }
                if (this._syncedModels[resourceStr]) {
                    this._syncedModelsLastUsedTime[resourceStr] = (new Date()).getTime();
                }
            }
            return winjs_base_1.TPromise.as(null);
        };
        EditorWorkerClient.prototype._beginModelSync = function (resource) {
            var _this = this;
            var modelUrl = resource.toString();
            var model = this._modelService.getModel(resource);
            if (!model) {
                return;
            }
            if (model.isTooLargeForHavingARichMode()) {
                return;
            }
            this._proxy.acceptNewModel({
                url: model.getAssociatedResource().toString(),
                value: model.toRawText(),
                versionId: model.getVersionId()
            });
            var toDispose = [];
            toDispose.push(model.addBulkListener2(function (events) {
                var changedEvents = [];
                for (var i = 0, len = events.length; i < len; i++) {
                    var e = events[i];
                    switch (e.getType()) {
                        case editorCommon.EventType.ModelContentChanged2:
                            changedEvents.push(e.getData());
                            break;
                        case editorCommon.EventType.ModelDispose:
                            _this._stopModelSync(modelUrl);
                            return;
                    }
                }
                if (changedEvents.length > 0) {
                    _this._proxy.acceptModelChanged(modelUrl.toString(), changedEvents);
                }
            }));
            toDispose.push({
                dispose: function () {
                    _this._proxy.acceptRemovedModel(modelUrl);
                }
            });
            this._syncedModels[modelUrl] = toDispose;
        };
        EditorWorkerClient.prototype._stopModelSync = function (modelUrl) {
            var toDispose = this._syncedModels[modelUrl];
            delete this._syncedModels[modelUrl];
            delete this._syncedModelsLastUsedTime[modelUrl];
            lifecycle_1.disposeAll(toDispose);
        };
        return EditorWorkerClient;
    }(lifecycle_1.Disposable));
});

define("vs/editor/common/services/modeService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IModeService = instantiation_1.createDecorator('modeService');
});

define("vs/editor/common/services/modelService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IModelService = instantiation_1.createDecorator('modelService');
});

define("vs/editor/common/services/resourceService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // Resource Service
    exports.ResourceEvents = {
        ADDED: 'resource.added',
        REMOVED: 'resource.removed',
        CHANGED: 'resource.changed'
    };
    exports.IResourceService = instantiation_1.createDecorator('resourceService');
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/smartSelect/common/tokenSelectionSupport", ["require", "exports", 'vs/base/common/winjs.base', 'vs/editor/common/core/range', 'vs/editor/common/services/modelService', './tokenTree'], function (require, exports, winjs_base_1, range_1, modelService_1, tokenTree_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var TokenSelectionSupport = (function () {
        function TokenSelectionSupport(modelService) {
            this._modelService = modelService;
        }
        TokenSelectionSupport.prototype.getRangesToPosition = function (resource, position) {
            return winjs_base_1.TPromise.as(this.getRangesToPositionSync(resource, position));
        };
        TokenSelectionSupport.prototype.getRangesToPositionSync = function (resource, position) {
            var model = this._modelService.getModel(resource), entries = [];
            if (model) {
                this._doGetRangesToPosition(model, position).forEach(function (range) {
                    entries.push({
                        type: void 0,
                        range: range
                    });
                });
            }
            return entries;
        };
        TokenSelectionSupport.prototype._doGetRangesToPosition = function (model, position) {
            var tree = tokenTree_1.build(model), node, lastRange;
            node = tokenTree_1.find(tree, position);
            var ranges = [];
            while (node) {
                if (!lastRange || !range_1.Range.equalsRange(lastRange, node.range)) {
                    ranges.push(node.range);
                }
                lastRange = node.range;
                node = node.parent;
            }
            ranges = ranges.reverse();
            return ranges;
        };
        TokenSelectionSupport = __decorate([
            __param(0, modelService_1.IModelService)
        ], TokenSelectionSupport);
        return TokenSelectionSupport;
    }());
    exports.TokenSelectionSupport = TokenSelectionSupport;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/zoneWidget/browser/peekViewWidget", ["require", "exports", 'vs/nls!vs/editor/contrib/zoneWidget/browser/peekViewWidget', 'vs/base/common/actions', 'vs/base/common/strings', 'vs/base/browser/builder', 'vs/base/browser/dom', 'vs/base/browser/ui/actionbar/actionbar', 'vs/platform/instantiation/common/instantiation', 'vs/editor/common/services/codeEditorService', './zoneWidget', 'vs/css!./peekViewWidget'], function (require, exports, nls, actions_1, strings, builder_1, dom, actionbar_1, instantiation_1, codeEditorService_1, zoneWidget_1) {
    'use strict';
    exports.IPeekViewService = instantiation_1.createDecorator('peekViewService');
    var Events;
    (function (Events) {
        Events.Closed = 'closed';
    })(Events = exports.Events || (exports.Events = {}));
    var CONTEXT_OUTER_EDITOR = 'outerEditorId';
    function getOuterEditor(accessor, args) {
        var outerEditorId = args.context[CONTEXT_OUTER_EDITOR];
        if (!outerEditorId) {
            return null;
        }
        return accessor.get(codeEditorService_1.ICodeEditorService).getCodeEditor(outerEditorId);
    }
    exports.getOuterEditor = getOuterEditor;
    var PeekViewWidget = (function (_super) {
        __extends(PeekViewWidget, _super);
        function PeekViewWidget(editor, keybindingService, contextKey, options) {
            if (options === void 0) { options = {}; }
            _super.call(this, editor, options);
            this.serviceId = exports.IPeekViewService;
            this.contextKey = contextKey;
            keybindingService.createKey(CONTEXT_OUTER_EDITOR, editor.getId());
        }
        PeekViewWidget.prototype.dispose = function () {
            this._isActive = false;
            _super.prototype.dispose.call(this);
        };
        Object.defineProperty(PeekViewWidget.prototype, "isActive", {
            get: function () {
                return this._isActive;
            },
            enumerable: true,
            configurable: true
        });
        PeekViewWidget.prototype.getActiveWidget = function () {
            return this;
        };
        PeekViewWidget.prototype.show = function (where, heightInLines) {
            this._isActive = true;
            _super.prototype.show.call(this, where, heightInLines);
        };
        PeekViewWidget.prototype.fillContainer = function (container) {
            builder_1.$(container).addClass('peekview-widget');
            this._headElement = builder_1.$('.head').getHTMLElement();
            this._bodyElement = builder_1.$('.body').getHTMLElement();
            this._fillHead(this._headElement);
            this._fillBody(this._bodyElement);
            container.appendChild(this._headElement);
            container.appendChild(this._bodyElement);
        };
        PeekViewWidget.prototype._fillHead = function (container) {
            var _this = this;
            var titleElement = builder_1.$('.peekview-title').
                on(dom.EventType.CLICK, function (e) { return _this._onTitleClick(e); }).
                appendTo(this._headElement).
                getHTMLElement();
            this._primaryHeading = builder_1.$('span.filename').appendTo(titleElement).getHTMLElement();
            this._secondaryHeading = builder_1.$('span.dirname').appendTo(titleElement).getHTMLElement();
            this._metaHeading = builder_1.$('span.meta').appendTo(titleElement).getHTMLElement();
            this._actionbarWidget = new actionbar_1.ActionBar(builder_1.$('.peekview-actions').
                appendTo(this._headElement));
            this._actionbarWidget.push(new actions_1.Action('peekview.close', nls.localize(0, null), 'close-peekview-action', true, function () {
                _this.dispose();
                _this.emit(Events.Closed, _this);
                return null;
            }), { label: false, icon: true });
        };
        PeekViewWidget.prototype._onTitleClick = function (event) {
            // implement me
        };
        PeekViewWidget.prototype.setTitle = function (primaryHeading, secondaryHeading) {
            builder_1.$(this._primaryHeading).safeInnerHtml(primaryHeading);
            if (secondaryHeading) {
                builder_1.$(this._secondaryHeading).safeInnerHtml(secondaryHeading);
            }
            else {
                dom.clearNode(this._secondaryHeading);
            }
        };
        PeekViewWidget.prototype.setMetaTitle = function (value) {
            if (value) {
                builder_1.$(this._metaHeading).safeInnerHtml(value);
            }
            else {
                dom.clearNode(this._metaHeading);
            }
        };
        PeekViewWidget.prototype._fillBody = function (container) {
            // implement me
        };
        PeekViewWidget.prototype.doLayout = function (heightInPixel) {
            if (heightInPixel < 0) {
                // Looks like the view zone got folded away!
                this.dispose();
                this.emit(Events.Closed, this);
                return;
            }
            var headHeight = Math.ceil(this.editor.getConfiguration().lineHeight * 1.2), bodyHeight = heightInPixel - (headHeight + 2 /* the border-top/bottom width*/);
            this._doLayoutHead(headHeight);
            this._doLayoutBody(bodyHeight);
        };
        PeekViewWidget.prototype._doLayoutHead = function (heightInPixel) {
            this._headElement.style.height = strings.format('{0}px', heightInPixel);
            this._headElement.style.lineHeight = this._headElement.style.height;
        };
        PeekViewWidget.prototype._doLayoutBody = function (heightInPixel) {
            this._bodyElement.style.height = strings.format('{0}px', heightInPixel);
        };
        return PeekViewWidget;
    }(zoneWidget_1.ZoneWidget));
    exports.PeekViewWidget = PeekViewWidget;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/node/languageConfiguration", ["require", "exports", 'vs/nls!vs/editor/node/languageConfiguration', 'vs/base/common/json', 'vs/base/node/pfs', 'vs/editor/common/services/modeService'], function (require, exports, nls, json_1, pfs_1, modeService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LanguageConfigurationFileHandler = (function () {
        function LanguageConfigurationFileHandler(modeService) {
            var _this = this;
            this._modeService = modeService;
            this._handleModes(this._modeService.getRegisteredModes());
            this._modeService.onDidAddModes(function (modes) { return _this._handleModes(modes); });
        }
        LanguageConfigurationFileHandler.prototype._handleModes = function (modes) {
            var _this = this;
            modes.forEach(function (modeId) { return _this._handleMode(modeId); });
        };
        LanguageConfigurationFileHandler.prototype._handleMode = function (modeId) {
            var _this = this;
            var disposable = this._modeService.onDidCreateMode(function (mode) {
                if (mode.getId() !== modeId) {
                    return;
                }
                var configurationFiles = _this._modeService.getConfigurationFiles(modeId);
                configurationFiles.forEach(function (configFilePath) { return _this._handleConfigFile(modeId, configFilePath); });
                disposable.dispose();
            });
        };
        LanguageConfigurationFileHandler.prototype._handleConfigFile = function (modeId, configFilePath) {
            var _this = this;
            pfs_1.readFile(configFilePath).then(function (fileContents) {
                var errors = [];
                var configuration = json_1.parse(fileContents.toString(), errors);
                if (errors.length) {
                    console.error(nls.localize(0, null, configFilePath, errors.join('\n')));
                }
                _this._handleConfig(modeId, configuration);
            }, function (err) {
                console.error(err);
            });
        };
        LanguageConfigurationFileHandler.prototype._handleConfig = function (modeId, configuration) {
            var richEditConfig = {};
            if (configuration.comments) {
                richEditConfig.comments = configuration.comments;
            }
            if (configuration.brackets) {
                richEditConfig.brackets = configuration.brackets;
                richEditConfig.__characterPairSupport = {
                    autoClosingPairs: configuration.brackets.map(function (pair) {
                        var open = pair[0], close = pair[1];
                        return { open: open, close: close };
                    })
                };
            }
            this._modeService.registerRichEditSupport(modeId, richEditConfig);
        };
        LanguageConfigurationFileHandler = __decorate([
            __param(0, modeService_1.IModeService)
        ], LanguageConfigurationFileHandler);
        return LanguageConfigurationFileHandler;
    }());
    exports.LanguageConfigurationFileHandler = LanguageConfigurationFileHandler;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/actions/common/actions", ["require", "exports", 'vs/base/common/actions', 'vs/base/common/winjs.base', 'vs/base/common/assert', 'vs/platform/instantiation/common/descriptors', 'vs/platform/instantiation/common/instantiation'], function (require, exports, Actions, WinJS, Assert, Descriptors, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IActionsService = instantiation_1.createDecorator('actionsService');
    var SyncActionDescriptor = (function () {
        function SyncActionDescriptor(ctor, id, label, keybindings, keybindingContext, keybindingWeight) {
            this._id = id;
            this._label = label;
            this._keybindings = keybindings;
            this._keybindingContext = keybindingContext;
            this._keybindingWeight = keybindingWeight;
            this._descriptor = Descriptors.createSyncDescriptor(ctor, this._id, this._label);
        }
        Object.defineProperty(SyncActionDescriptor.prototype, "syncDescriptor", {
            get: function () {
                return this._descriptor;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SyncActionDescriptor.prototype, "id", {
            get: function () {
                return this._id;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SyncActionDescriptor.prototype, "label", {
            get: function () {
                return this._label;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SyncActionDescriptor.prototype, "keybindings", {
            get: function () {
                return this._keybindings;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SyncActionDescriptor.prototype, "keybindingContext", {
            get: function () {
                return this._keybindingContext;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SyncActionDescriptor.prototype, "keybindingWeight", {
            get: function () {
                return this._keybindingWeight;
            },
            enumerable: true,
            configurable: true
        });
        return SyncActionDescriptor;
    }());
    exports.SyncActionDescriptor = SyncActionDescriptor;
    /**
     * A proxy for an action that needs to load code in order to confunction. Can be used from contributions to defer
     * module loading up to the point until the run method is being executed.
     */
    var DeferredAction = (function (_super) {
        __extends(DeferredAction, _super);
        function DeferredAction(_instantiationService, _descriptor, id, label, cssClass, enabled) {
            if (label === void 0) { label = ''; }
            if (cssClass === void 0) { cssClass = ''; }
            if (enabled === void 0) { enabled = true; }
            _super.call(this, id, label, cssClass, enabled);
            this._instantiationService = _instantiationService;
            this._descriptor = _descriptor;
        }
        Object.defineProperty(DeferredAction.prototype, "cachedAction", {
            get: function () {
                return this._cachedAction;
            },
            set: function (action) {
                this._cachedAction = action;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DeferredAction.prototype, "id", {
            get: function () {
                if (this._cachedAction instanceof Actions.Action) {
                    return this._cachedAction.id;
                }
                return this._id;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DeferredAction.prototype, "label", {
            get: function () {
                if (this._cachedAction instanceof Actions.Action) {
                    return this._cachedAction.label;
                }
                return this._label;
            },
            set: function (value) {
                if (this._cachedAction instanceof Actions.Action) {
                    this._cachedAction.label = value;
                }
                else {
                    this._setLabel(value);
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DeferredAction.prototype, "class", {
            get: function () {
                if (this._cachedAction instanceof Actions.Action) {
                    return this._cachedAction.class;
                }
                return this._cssClass;
            },
            set: function (value) {
                if (this._cachedAction instanceof Actions.Action) {
                    this._cachedAction.class = value;
                }
                else {
                    this._setClass(value);
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DeferredAction.prototype, "enabled", {
            get: function () {
                if (this._cachedAction instanceof Actions.Action) {
                    return this._cachedAction.enabled;
                }
                return this._enabled;
            },
            set: function (value) {
                if (this._cachedAction instanceof Actions.Action) {
                    this._cachedAction.enabled = value;
                }
                else {
                    this._setEnabled(value);
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DeferredAction.prototype, "order", {
            get: function () {
                if (this._cachedAction instanceof Actions.Action) {
                    return this._cachedAction.order;
                }
                return this._order;
            },
            set: function (order) {
                if (this._cachedAction instanceof Actions.Action) {
                    this._cachedAction.order = order;
                }
                else {
                    this._order = order;
                }
            },
            enumerable: true,
            configurable: true
        });
        DeferredAction.prototype.run = function (event) {
            if (this._cachedAction) {
                return this._cachedAction.run(event);
            }
            return this._createAction().then(function (action) {
                return action.run(event);
            });
        };
        DeferredAction.prototype._createAction = function () {
            var _this = this;
            var promise = WinJS.TPromise.as(undefined);
            return promise.then(function () {
                return _this._instantiationService.createInstance(_this._descriptor);
            }).then(function (action) {
                Assert.ok(action instanceof Actions.Action, 'Action must be an instanceof Base Action');
                _this._cachedAction = action;
                // Pipe events from the instantated action through this deferred action
                _this._emitterUnbind = _this.addEmitter(_this._cachedAction);
                return action;
            });
        };
        DeferredAction.prototype.dispose = function () {
            if (this._emitterUnbind) {
                this._emitterUnbind();
            }
            if (this._cachedAction) {
                this._cachedAction.dispose();
            }
            _super.prototype.dispose.call(this);
        };
        return DeferredAction;
    }(Actions.Action));
    exports.DeferredAction = DeferredAction;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/platform/configuration/common/configuration", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    "use strict";
    exports.IConfigurationService = instantiation_1.createDecorator('configurationService');
    var ConfigurationServiceEventTypes = (function () {
        function ConfigurationServiceEventTypes() {
        }
        /**
         * This event happens after configuration is updated either programmatically
         * or through a file change. It will include a IConfigurationServiceEvent
         * object that includes the new config and which section was updated
         * or null if entire config was updated.
         *
         * Subscribers can use the provided updated configuration
         * rather than re-pulling for updates
         */
        ConfigurationServiceEventTypes.UPDATED = 'update';
        return ConfigurationServiceEventTypes;
    }());
    exports.ConfigurationServiceEventTypes = ConfigurationServiceEventTypes;
    function extractSetting(config, settingPath) {
        function accessSetting(config, path) {
            var current = config;
            for (var i = 0; i < path.length; i++) {
                current = current[path[i]];
                if (!current) {
                    return undefined;
                }
            }
            return current;
        }
        var path = settingPath.split('.');
        return accessSetting(config, path);
    }
    exports.extractSetting = extractSetting;
});

define("vs/platform/contextview/browser/contextView", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IContextViewService = instantiation_1.createDecorator('contextViewService');
    exports.IContextMenuService = instantiation_1.createDecorator('contextMenuService');
});

define("vs/platform/contextview/browser/contextViewService", ["require", "exports", './contextView', 'vs/base/browser/ui/contextview/contextview'], function (require, exports, contextView_1, contextview_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ContextViewService = (function () {
        function ContextViewService(container, telemetryService, messageService) {
            this.serviceId = contextView_1.IContextViewService;
            this.contextView = new contextview_1.ContextView(container);
        }
        ContextViewService.prototype.dispose = function () {
            this.contextView.dispose();
        };
        // ContextView
        ContextViewService.prototype.setContainer = function (container) {
            this.contextView.setContainer(container);
        };
        ContextViewService.prototype.showContextView = function (delegate) {
            this.contextView.show(delegate);
        };
        ContextViewService.prototype.layout = function () {
            this.contextView.layout();
        };
        ContextViewService.prototype.hideContextView = function (data) {
            this.contextView.hide(data);
        };
        return ContextViewService;
    }());
    exports.ContextViewService = ContextViewService;
});

define("vs/platform/editor/common/editor", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IEditorService = instantiation_1.createDecorator('editorService');
    /**
     * Possible locations for opening an editor.
     */
    (function (Position) {
        /** Opens the editor in the LEFT most position replacing the input currently showing */
        Position[Position["LEFT"] = 0] = "LEFT";
        /** Opens the editor in the CENTER position replacing the input currently showing */
        Position[Position["CENTER"] = 1] = "CENTER";
        /** Opens the editor in the RIGHT most position replacing the input currently showing */
        Position[Position["RIGHT"] = 2] = "RIGHT";
    })(exports.Position || (exports.Position = {}));
    var Position = exports.Position;
    exports.POSITIONS = [Position.LEFT, Position.CENTER, Position.RIGHT];
});

define("vs/platform/event/common/event", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IEventService = instantiation_1.createDecorator('eventService');
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/event/common/eventService", ["require", "exports", 'vs/base/common/eventEmitter', './event'], function (require, exports, eventEmitter_1, event_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // --- implementation ------------------------------------------
    var EventService = (function (_super) {
        __extends(EventService, _super);
        function EventService() {
            _super.call(this);
            this.serviceId = event_1.IEventService;
        }
        return EventService;
    }(eventEmitter_1.EventEmitter));
    exports.EventService = EventService;
});

define("vs/platform/extensions/common/extensions", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IExtensionService = instantiation_1.createDecorator('extensionService');
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/files/common/files", ["require", "exports", 'vs/base/common/paths', 'vs/base/common/events', 'vs/platform/instantiation/common/instantiation'], function (require, exports, paths, events, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IFileService = instantiation_1.createDecorator('fileService');
    /**
     * Possible changes that can occur to a file.
     */
    (function (FileChangeType) {
        FileChangeType[FileChangeType["UPDATED"] = 0] = "UPDATED";
        FileChangeType[FileChangeType["ADDED"] = 1] = "ADDED";
        FileChangeType[FileChangeType["DELETED"] = 2] = "DELETED";
    })(exports.FileChangeType || (exports.FileChangeType = {}));
    var FileChangeType = exports.FileChangeType;
    /**
     * Possible events to subscribe to
     */
    exports.EventType = {
        /**
        * Send on file changes.
        */
        FILE_CHANGES: 'files:fileChanges'
    };
    var FileChangesEvent = (function (_super) {
        __extends(FileChangesEvent, _super);
        function FileChangesEvent(changes) {
            _super.call(this);
            this._changes = changes;
        }
        Object.defineProperty(FileChangesEvent.prototype, "changes", {
            get: function () {
                return this._changes;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Returns true if this change event contains the provided file with the given change type. In case of
         * type DELETED, this method will also return true if a folder got deleted that is the parent of the
         * provided file path.
         */
        FileChangesEvent.prototype.contains = function (resource, type) {
            if (!resource) {
                return false;
            }
            return this.containsAny([resource], type);
        };
        /**
         * Returns true if this change event contains any of the provided files with the given change type. In case of
         * type DELETED, this method will also return true if a folder got deleted that is the parent of any of the
         * provided file paths.
         */
        FileChangesEvent.prototype.containsAny = function (resources, type) {
            if (!resources || !resources.length) {
                return false;
            }
            return this._changes.some(function (change) {
                if (change.type !== type) {
                    return false;
                }
                // For deleted also return true when deleted folder is parent of target path
                if (type === FileChangeType.DELETED) {
                    return resources.some(function (a) {
                        if (!a) {
                            return false;
                        }
                        return paths.isEqualOrParent(a.fsPath, change.resource.fsPath);
                    });
                }
                return resources.some(function (a) {
                    if (!a) {
                        return false;
                    }
                    return a.fsPath === change.resource.fsPath;
                });
            });
        };
        /**
         * Returns the changes that describe added files.
         */
        FileChangesEvent.prototype.getAdded = function () {
            return this.getOfType(FileChangeType.ADDED);
        };
        /**
         * Returns if this event contains added files.
         */
        FileChangesEvent.prototype.gotAdded = function () {
            return this.hasType(FileChangeType.ADDED);
        };
        /**
         * Returns the changes that describe deleted files.
         */
        FileChangesEvent.prototype.getDeleted = function () {
            return this.getOfType(FileChangeType.DELETED);
        };
        /**
         * Returns if this event contains deleted files.
         */
        FileChangesEvent.prototype.gotDeleted = function () {
            return this.hasType(FileChangeType.DELETED);
        };
        /**
         * Returns the changes that describe updated files.
         */
        FileChangesEvent.prototype.getUpdated = function () {
            return this.getOfType(FileChangeType.UPDATED);
        };
        /**
         * Returns if this event contains updated files.
         */
        FileChangesEvent.prototype.gotUpdated = function () {
            return this.hasType(FileChangeType.UPDATED);
        };
        FileChangesEvent.prototype.getOfType = function (type) {
            return this._changes.filter(function (change) { return change.type === type; });
        };
        FileChangesEvent.prototype.hasType = function (type) {
            return this._changes.some(function (change) {
                return change.type === type;
            });
        };
        return FileChangesEvent;
    }(events.Event));
    exports.FileChangesEvent = FileChangesEvent;
    (function (FileOperationResult) {
        FileOperationResult[FileOperationResult["FILE_IS_BINARY"] = 0] = "FILE_IS_BINARY";
        FileOperationResult[FileOperationResult["FILE_IS_DIRECTORY"] = 1] = "FILE_IS_DIRECTORY";
        FileOperationResult[FileOperationResult["FILE_NOT_FOUND"] = 2] = "FILE_NOT_FOUND";
        FileOperationResult[FileOperationResult["FILE_NOT_MODIFIED_SINCE"] = 3] = "FILE_NOT_MODIFIED_SINCE";
        FileOperationResult[FileOperationResult["FILE_MODIFIED_SINCE"] = 4] = "FILE_MODIFIED_SINCE";
        FileOperationResult[FileOperationResult["FILE_MOVE_CONFLICT"] = 5] = "FILE_MOVE_CONFLICT";
        FileOperationResult[FileOperationResult["FILE_READ_ONLY"] = 6] = "FILE_READ_ONLY";
        FileOperationResult[FileOperationResult["FILE_TOO_LARGE"] = 7] = "FILE_TOO_LARGE";
    })(exports.FileOperationResult || (exports.FileOperationResult = {}));
    var FileOperationResult = exports.FileOperationResult;
    exports.MAX_FILE_SIZE = 50 * 1024 * 1024;
    exports.AutoSaveConfiguration = {
        OFF: 'off',
        AFTER_DELAY: 'afterDelay',
        ON_FOCUS_CHANGE: 'onFocusChange'
    };
    exports.SUPPORTED_ENCODINGS = {
        utf8: {
            labelLong: 'UTF-8',
            labelShort: 'UTF-8',
            order: 1,
            alias: 'utf8bom'
        },
        utf8bom: {
            labelLong: 'UTF-8 with BOM',
            labelShort: 'UTF-8 with BOM',
            encodeOnly: true,
            order: 2,
            alias: 'utf8'
        },
        utf16le: {
            labelLong: 'UTF-16 LE',
            labelShort: 'UTF-16 LE',
            order: 3
        },
        utf16be: {
            labelLong: 'UTF-16 BE',
            labelShort: 'UTF-16 BE',
            order: 4
        },
        windows1252: {
            labelLong: 'Western (Windows 1252)',
            labelShort: 'Windows 1252',
            order: 5
        },
        iso88591: {
            labelLong: 'Western (ISO 8859-1)',
            labelShort: 'ISO 8859-1',
            order: 6
        },
        iso88593: {
            labelLong: 'Western (ISO 8859-3)',
            labelShort: 'ISO 8859-3',
            order: 7
        },
        iso885915: {
            labelLong: 'Western (ISO 8859-15)',
            labelShort: 'ISO 8859-15',
            order: 8
        },
        macroman: {
            labelLong: 'Western (Mac Roman)',
            labelShort: 'Mac Roman',
            order: 9
        },
        cp437: {
            labelLong: 'DOS (CP 437)',
            labelShort: 'CP437',
            order: 10
        },
        windows1256: {
            labelLong: 'Arabic (Windows 1256)',
            labelShort: 'Windows 1256',
            order: 11
        },
        iso88596: {
            labelLong: 'Arabic (ISO 8859-6)',
            labelShort: 'ISO 8859-6',
            order: 12
        },
        windows1257: {
            labelLong: 'Baltic (Windows 1257)',
            labelShort: 'Windows 1257',
            order: 13
        },
        iso88594: {
            labelLong: 'Baltic (ISO 8859-4)',
            labelShort: 'ISO 8859-4',
            order: 14
        },
        iso885914: {
            labelLong: 'Celtic (ISO 8859-14)',
            labelShort: 'ISO 8859-14',
            order: 15
        },
        windows1250: {
            labelLong: 'Central European (Windows 1250)',
            labelShort: 'Windows 1250',
            order: 16
        },
        iso88592: {
            labelLong: 'Central European (ISO 8859-2)',
            labelShort: 'ISO 8859-2',
            order: 17
        },
        windows1251: {
            labelLong: 'Cyrillic (Windows 1251)',
            labelShort: 'Windows 1251',
            order: 18
        },
        cp866: {
            labelLong: 'Cyrillic (CP 866)',
            labelShort: 'CP 866',
            order: 19
        },
        iso88595: {
            labelLong: 'Cyrillic (ISO 8859-5)',
            labelShort: 'ISO 8859-5',
            order: 20
        },
        koi8r: {
            labelLong: 'Cyrillic (KOI8-R)',
            labelShort: 'KOI8-R',
            order: 21
        },
        koi8u: {
            labelLong: 'Cyrillic (KOI8-U)',
            labelShort: 'KOI8-U',
            order: 22
        },
        iso885913: {
            labelLong: 'Estonian (ISO 8859-13)',
            labelShort: 'ISO 8859-13',
            order: 23
        },
        windows1253: {
            labelLong: 'Greek (Windows 1253)',
            labelShort: 'Windows 1253',
            order: 24
        },
        iso88597: {
            labelLong: 'Greek (ISO 8859-7)',
            labelShort: 'ISO 8859-7',
            order: 25
        },
        windows1255: {
            labelLong: 'Hebrew (Windows 1255)',
            labelShort: 'Windows 1255',
            order: 26
        },
        iso88598: {
            labelLong: 'Hebrew (ISO 8859-8)',
            labelShort: 'ISO 8859-8',
            order: 27
        },
        iso885910: {
            labelLong: 'Nordic (ISO 8859-10)',
            labelShort: 'ISO 8859-10',
            order: 28
        },
        iso885916: {
            labelLong: 'Romanian (ISO 8859-16)',
            labelShort: 'ISO 8859-16',
            order: 29
        },
        windows1254: {
            labelLong: 'Turkish (Windows 1254)',
            labelShort: 'Windows 1254',
            order: 30
        },
        iso88599: {
            labelLong: 'Turkish (ISO 8859-9)',
            labelShort: 'ISO 8859-9',
            order: 31
        },
        windows1258: {
            labelLong: 'Vietnamese (Windows 1258)',
            labelShort: 'Windows 1258',
            order: 32
        },
        gbk: {
            labelLong: 'Chinese (GBK)',
            labelShort: 'GBK',
            order: 33
        },
        gb18030: {
            labelLong: 'Chinese (GB18030)',
            labelShort: 'GB18030',
            order: 34
        },
        cp950: {
            labelLong: 'Traditional Chinese (Big5)',
            labelShort: 'Big5',
            order: 35
        },
        big5hkscs: {
            labelLong: 'Traditional Chinese (Big5-HKSCS)',
            labelShort: 'Big5-HKSCS',
            order: 36
        },
        shiftjis: {
            labelLong: 'Japanese (Shift JIS)',
            labelShort: 'Shift JIS',
            order: 37
        },
        eucjp: {
            labelLong: 'Japanese (EUC-JP)',
            labelShort: 'EUC-JP',
            order: 38
        },
        euckr: {
            labelLong: 'Korean (EUC-KR)',
            labelShort: 'EUC-KR',
            order: 39
        },
        windows874: {
            labelLong: 'Thai (Windows 874)',
            labelShort: 'Windows 874',
            order: 40
        },
        iso885911: {
            labelLong: 'Latin/Thai (ISO 8859-11)',
            labelShort: 'ISO 8859-11',
            order: 41
        },
        'koi8-ru': {
            labelLong: 'Cyrillic (KOI8-RU)',
            labelShort: 'KOI8-RU',
            order: 42
        },
        'koi8-t': {
            labelLong: 'Tajik (KOI8-T)',
            labelShort: 'KOI8-T',
            order: 43
        },
        GB2312: {
            labelLong: 'Simplified Chinese (GB 2312)',
            labelShort: 'GB 2312',
            order: 44
        }
    };
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/services/bulkEdit", ["require", "exports", 'vs/nls!vs/editor/common/services/bulkEdit', 'vs/base/common/arrays', 'vs/base/common/collections', 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/platform/files/common/files', 'vs/editor/common/core/editOperation', 'vs/editor/common/core/range', 'vs/editor/common/core/selection'], function (require, exports, nls, arrays_1, collections_1, uri_1, winjs_base_1, files_1, editOperation_1, range_1, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ChangeRecorder = (function () {
        function ChangeRecorder(eventService) {
            this._eventService = eventService;
        }
        ChangeRecorder.prototype.start = function () {
            var changes = Object.create(null);
            var stop = this._eventService.addListener(files_1.EventType.FILE_CHANGES, function (event) {
                event.changes.forEach(function (change) {
                    var key = String(change.resource), array = changes[key];
                    if (!array) {
                        changes[key] = array = [];
                    }
                    array.push(change);
                });
            });
            return {
                stop: stop,
                hasChanged: function (resource) { return !!changes[resource.toString()]; },
                allChanges: function () { return arrays_1.merge(collections_1.values(changes)); }
            };
        };
        return ChangeRecorder;
    }());
    var EditTask = (function () {
        function EditTask(model) {
            this._endCursorSelection = null;
            this._model = model;
            this._edits = [];
        }
        EditTask.prototype.addEdit = function (edit) {
            var range;
            if (!edit.range) {
                range = this._model.getFullModelRange();
            }
            else {
                range = edit.range;
            }
            this._edits.push(editOperation_1.EditOperation.replace(range_1.Range.lift(range), edit.newText));
        };
        EditTask.prototype.apply = function () {
            var _this = this;
            if (this._edits.length === 0) {
                return;
            }
            this._edits.sort(EditTask._editCompare);
            this._initialSelections = this._getInitialSelections();
            this._model.pushEditOperations(this._initialSelections, this._edits, function (edits) { return _this._getEndCursorSelections(edits); });
        };
        EditTask.prototype._getInitialSelections = function () {
            var firstRange = this._edits[0].range;
            var initialSelection = selection_1.Selection.createSelection(firstRange.startLineNumber, firstRange.startColumn, firstRange.endLineNumber, firstRange.endColumn);
            return [initialSelection];
        };
        EditTask.prototype._getEndCursorSelections = function (inverseEditOperations) {
            var relevantEditIndex = 0;
            for (var i = 0; i < inverseEditOperations.length; i++) {
                var editRange = inverseEditOperations[i].range;
                for (var j = 0; j < this._initialSelections.length; j++) {
                    var selectionRange = this._initialSelections[j];
                    if (range_1.Range.areIntersectingOrTouching(editRange, selectionRange)) {
                        relevantEditIndex = i;
                        break;
                    }
                }
            }
            var srcRange = inverseEditOperations[relevantEditIndex].range;
            this._endCursorSelection = selection_1.Selection.createSelection(srcRange.endLineNumber, srcRange.endColumn, srcRange.endLineNumber, srcRange.endColumn);
            return [this._endCursorSelection];
        };
        EditTask.prototype.getEndCursorSelection = function () {
            return this._endCursorSelection;
        };
        EditTask._editCompare = function (a, b) {
            return range_1.Range.compareRangesUsingStarts(a.range, b.range);
        };
        return EditTask;
    }());
    var SourceModelEditTask = (function (_super) {
        __extends(SourceModelEditTask, _super);
        function SourceModelEditTask(model, initialSelections) {
            _super.call(this, model);
            this._knownInitialSelections = initialSelections;
        }
        SourceModelEditTask.prototype._getInitialSelections = function () {
            return this._knownInitialSelections;
        };
        return SourceModelEditTask;
    }(EditTask));
    var BulkEditModel = (function () {
        function BulkEditModel(editorService, sourceModel, sourceSelections, edits) {
            this._numberOfResourcesToModify = 0;
            this._numberOfChanges = 0;
            this._edits = Object.create(null);
            this._editorService = editorService;
            this._sourceModel = sourceModel;
            this._sourceSelections = sourceSelections;
            this._sourceModelTask = null;
            for (var _i = 0, edits_1 = edits; _i < edits_1.length; _i++) {
                var edit = edits_1[_i];
                this._addEdit(edit);
            }
        }
        BulkEditModel.prototype.resourcesCount = function () {
            return this._numberOfResourcesToModify;
        };
        BulkEditModel.prototype.changeCount = function () {
            return this._numberOfChanges;
        };
        BulkEditModel.prototype._addEdit = function (edit) {
            var array = this._edits[edit.resource.toString()];
            if (!array) {
                this._edits[edit.resource.toString()] = array = [];
                this._numberOfResourcesToModify += 1;
            }
            this._numberOfChanges += 1;
            array.push(edit);
        };
        BulkEditModel.prototype.prepare = function () {
            var _this = this;
            if (this._tasks) {
                throw new Error('illegal state - already prepared');
            }
            this._tasks = [];
            var promises = [];
            collections_1.forEach(this._edits, function (entry) {
                var promise = _this._editorService.resolveEditorModel({ resource: uri_1.default.parse(entry.key) }).then(function (model) {
                    if (!model || !model.textEditorModel) {
                        throw new Error("Cannot load file " + entry.key);
                    }
                    var textEditorModel = model.textEditorModel, task;
                    if (_this._sourceModel && textEditorModel.getAssociatedResource().toString() === _this._sourceModel.toString()) {
                        _this._sourceModelTask = new SourceModelEditTask(textEditorModel, _this._sourceSelections);
                        task = _this._sourceModelTask;
                    }
                    else {
                        task = new EditTask(textEditorModel);
                    }
                    entry.value.forEach(function (edit) { return task.addEdit(edit); });
                    _this._tasks.push(task);
                });
                promises.push(promise);
            });
            return winjs_base_1.TPromise.join(promises).then(function (_) { return _this; });
        };
        BulkEditModel.prototype.apply = function () {
            this._tasks.forEach(function (task) { return task.apply(); });
            var r = null;
            if (this._sourceModelTask) {
                r = this._sourceModelTask.getEndCursorSelection();
            }
            return r;
        };
        return BulkEditModel;
    }());
    function bulkEdit(eventService, editorService, editor, edits) {
        var bulk = createBulkEdit(eventService, editorService, editor);
        bulk.add(edits);
        return bulk.finish();
    }
    exports.bulkEdit = bulkEdit;
    function createBulkEdit(eventService, editorService, editor) {
        var all = [];
        var recording = new ChangeRecorder(eventService).start();
        function add(edits) {
            all.push.apply(all, edits);
        }
        function getConcurrentEdits() {
            var names;
            for (var _i = 0, all_1 = all; _i < all_1.length; _i++) {
                var edit = all_1[_i];
                if (recording.hasChanged(edit.resource)) {
                    if (!names) {
                        names = [];
                    }
                    names.push(edit.resource.fsPath);
                }
            }
            if (names) {
                return nls.localize(0, null, names.join(', '));
            }
        }
        function finish() {
            if (all.length === 0) {
                return;
            }
            var concurrentEdits = getConcurrentEdits();
            if (concurrentEdits) {
                return winjs_base_1.TPromise.wrapError(concurrentEdits);
            }
            var uri;
            var selections;
            if (editor && editor.getModel()) {
                uri = editor.getModel().getAssociatedResource();
                selections = editor.getSelections();
            }
            var model = new BulkEditModel(editorService, uri, selections, all);
            return model.prepare().then(function (_) {
                var concurrentEdits = getConcurrentEdits();
                if (concurrentEdits) {
                    throw new Error(concurrentEdits);
                }
                recording.stop();
                return model.apply();
            });
        }
        return {
            add: add,
            finish: finish
        };
    }
    exports.createBulkEdit = createBulkEdit;
});

define("vs/platform/instantiation/common/instantiationService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/errors', 'vs/base/common/strings', 'vs/base/common/types', 'vs/base/common/collections', './descriptors', 'vs/base/common/graph', './instantiation'], function (require, exports, winjs, errors, strings, types, collections, descriptors, graph_1, instantiation) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var IInstantiationService = instantiation.IInstantiationService;
    /**
     * Creates a new instance of an instantiation service.
     */
    function createInstantiationService(services) {
        if (services === void 0) { services = Object.create(null); }
        var result = new InstantiationService(services, new AccessLock());
        return result;
    }
    exports.createInstantiationService = createInstantiationService;
    var AccessLock = (function () {
        function AccessLock() {
            this._value = 0;
        }
        Object.defineProperty(AccessLock.prototype, "locked", {
            get: function () {
                return this._value === 0;
            },
            enumerable: true,
            configurable: true
        });
        AccessLock.prototype.runUnlocked = function (r) {
            this._value++;
            try {
                return r();
            }
            finally {
                this._value--;
            }
        };
        return AccessLock;
    }());
    var ServicesMap = (function () {
        function ServicesMap(_services, _lock) {
            var _this = this;
            this._services = _services;
            this._lock = _lock;
            collections.forEach(this._services, function (entry) {
                // add a accessor to myselves
                _this.registerService(entry.key, entry.value);
            });
        }
        ServicesMap.prototype.registerService = function (name, service) {
            var _this = this;
            // add a accessor to myselves
            Object.defineProperty(this, name, {
                get: function () {
                    if (_this._lock.locked) {
                        throw errors.illegalState('the services map can only be used during construction');
                    }
                    if (!service) {
                        throw errors.illegalArgument(strings.format('service with \'{0}\' not found', name));
                    }
                    if (service instanceof descriptors.SyncDescriptor) {
                        var cached = _this._services[name];
                        if (cached instanceof descriptors.SyncDescriptor) {
                            _this._ensureInstances(name, service);
                            service = _this._services[name];
                        }
                        else {
                            service = cached;
                        }
                    }
                    return service;
                },
                set: function (value) {
                    throw errors.illegalState('services cannot be changed');
                },
                configurable: false,
                enumerable: false
            });
            // add to services map
            this._services[name] = service;
        };
        Object.defineProperty(ServicesMap.prototype, "lock", {
            get: function () {
                return this._lock;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ServicesMap.prototype, "services", {
            get: function () {
                return this._services;
            },
            enumerable: true,
            configurable: true
        });
        ServicesMap.prototype._ensureInstances = function (serviceId, desc) {
            var seen = Object.create(null);
            var graph = new graph_1.Graph(function (i) { return i.serviceId; });
            var stack = [{ serviceId: serviceId, desc: desc }];
            while (stack.length) {
                var item = stack.pop();
                graph.lookupOrInsertNode(item);
                // check for cycles between the descriptors
                if (seen[item.serviceId]) {
                    throw new Error("[createInstance] cyclic dependency: " + Object.keys(seen).join('>>'));
                }
                seen[item.serviceId] = true;
                // check all dependencies for existence and if the need to be created first
                var dependencies = instantiation._util.getServiceDependencies(item.desc.ctor);
                if (Array.isArray(dependencies)) {
                    for (var _i = 0, dependencies_1 = dependencies; _i < dependencies_1.length; _i++) {
                        var dependency = dependencies_1[_i];
                        var instanceOrDesc = this.services[dependency.serviceId];
                        if (!instanceOrDesc) {
                            throw new Error("[createInstance] " + serviceId + " depends on " + dependency.serviceId + " which is NOT registered.");
                        }
                        if (instanceOrDesc instanceof descriptors.SyncDescriptor) {
                            var d = { serviceId: dependency.serviceId, desc: instanceOrDesc };
                            stack.push(d);
                            graph.insertEdge(item, d);
                        }
                    }
                }
            }
            while (true) {
                var roots = graph.roots();
                // if there is no more roots but still
                // nodes in the graph we have a cycle
                if (roots.length === 0) {
                    if (graph.length !== 0) {
                        throw new Error('[createInstance] cyclinc dependency!');
                    }
                    break;
                }
                for (var _a = 0, roots_1 = roots; _a < roots_1.length; _a++) {
                    var root = roots_1[_a];
                    var instance = this.createInstance(root.data.desc, []);
                    this._services[root.data.serviceId] = instance;
                    graph.removeNode(root.data);
                }
            }
        };
        ServicesMap.prototype.invokeFunction = function (fn, args) {
            var _this = this;
            return this._lock.runUnlocked(function () {
                var accessor = {
                    get: function (id) {
                        var value = instantiation._util.getServiceId(id);
                        return _this[value];
                    }
                };
                return fn.apply(undefined, [accessor].concat(args));
            });
        };
        ServicesMap.prototype.createInstance = function (descriptor, args) {
            var _this = this;
            var allArguments = [];
            var serviceInjections = instantiation._util.getServiceDependencies(descriptor.ctor) || [];
            var fixedArguments = descriptor.staticArguments().concat(args);
            var expectedFirstServiceIndex = fixedArguments.length;
            var actualFirstServiceIndex = Number.MAX_VALUE;
            serviceInjections.forEach(function (si) {
                // @IServiceName
                var serviceId = si.serviceId, index = si.index;
                var service = _this._lock.runUnlocked(function () { return _this[serviceId]; });
                allArguments[index] = service;
                actualFirstServiceIndex = Math.min(actualFirstServiceIndex, si.index);
            });
            // insert the fixed arguments into the array of all ctor
            // arguments. don't overwrite existing values tho it indicates
            // something is off
            var i = 0;
            for (var _i = 0, fixedArguments_1 = fixedArguments; _i < fixedArguments_1.length; _i++) {
                var arg = fixedArguments_1[_i];
                var hasValue = allArguments[i] !== void 0;
                if (!hasValue) {
                    allArguments[i] = arg;
                }
                i += 1;
            }
            allArguments.unshift(descriptor.ctor); // ctor is first arg
            // services are the last arguments of ctor-calls. We check if static ctor arguments
            // (like those from a [sync|async] desriptor) or args that are passed by createInstance
            // don't override positions of those arguments
            if (actualFirstServiceIndex !== Number.MAX_VALUE
                && actualFirstServiceIndex !== expectedFirstServiceIndex) {
                var msg = ("[createInstance] constructor '" + descriptor.ctor.name + "' has first") +
                    (" service dependency at position " + (actualFirstServiceIndex + 1) + " but is called with") +
                    (" " + (expectedFirstServiceIndex - 1) + " static arguments that are expected to come first");
                // throw new Error(msg);
                console.warn(msg);
            }
            return this._lock.runUnlocked(function () {
                var instance = types.create.apply(null, allArguments);
                descriptor._validate(instance);
                return instance;
            });
        };
        return ServicesMap;
    }());
    var InstantiationService = (function () {
        function InstantiationService(services, lock) {
            this.serviceId = IInstantiationService;
            services['instantiationService'] = this;
            this._servicesMap = new ServicesMap(services, lock);
        }
        InstantiationService.prototype.createChild = function (services) {
            var childServices = {};
            // copy existing services
            collections.forEach(this._servicesMap.services, function (entry) {
                childServices[entry.key] = entry.value;
            });
            // insert new services (might overwrite)
            collections.forEach(services, function (entry) {
                childServices[entry.key] = entry.value;
            });
            return new InstantiationService(childServices, this._servicesMap.lock);
        };
        InstantiationService.prototype.registerService = function (name, service) {
            this._servicesMap.registerService(name, service);
        };
        InstantiationService.prototype.addSingleton = function (id, instanceOrDescriptor) {
            var name = instantiation._util.getServiceId(id);
            this._servicesMap.registerService(name, instanceOrDescriptor);
        };
        InstantiationService.prototype.getInstance = function (id) {
            var _this = this;
            var name = instantiation._util.getServiceId(id);
            var result = this._servicesMap.lock.runUnlocked(function () { return _this._servicesMap[name]; });
            return result;
        };
        InstantiationService.prototype.createInstance = function (param) {
            var rest = new Array(arguments.length - 1);
            for (var i = 1, len = arguments.length; i < len; i++) {
                rest[i - 1] = arguments[i];
            }
            if (param instanceof descriptors.SyncDescriptor) {
                return this._servicesMap.createInstance(param, rest);
            }
            else if (param instanceof descriptors.AsyncDescriptor) {
                return this._createInstanceAsync(param, rest);
            }
            else {
                return this._servicesMap.createInstance(new descriptors.SyncDescriptor(param), rest);
            }
        };
        InstantiationService.prototype._createInstanceAsync = function (descriptor, args) {
            var _this = this;
            var canceled;
            return new winjs.TPromise(function (c, e, p) {
                require([descriptor.moduleName], function (_module) {
                    if (canceled) {
                        e(canceled);
                    }
                    if (!_module) {
                        return e(errors.illegalArgument('module not found: ' + descriptor.moduleName));
                    }
                    var ctor;
                    if (!descriptor.ctorName) {
                        ctor = _module;
                    }
                    else {
                        ctor = _module[descriptor.ctorName];
                    }
                    if (typeof ctor !== 'function') {
                        return e(errors.illegalArgument('not a function: ' + descriptor.ctorName || descriptor.moduleName));
                    }
                    try {
                        args.unshift.apply(args, descriptor.staticArguments()); // instead of spread in ctor call
                        c(_this._servicesMap.createInstance(new descriptors.SyncDescriptor(ctor), args));
                    }
                    catch (error) {
                        return e(error);
                    }
                }, e);
            }, function () {
                canceled = errors.canceled();
            });
        };
        InstantiationService.prototype.invokeFunction = function (signature) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            return this._servicesMap.invokeFunction(signature, args);
        };
        return InstantiationService;
    }());
});

define("vs/platform/keybinding/common/keybindingService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var KbDefinedExpression = (function () {
        function KbDefinedExpression(key) {
            this.key = key;
        }
        KbDefinedExpression.prototype.equals = function (other) {
            if (other instanceof KbDefinedExpression) {
                return (this.key === other.key);
            }
            return false;
        };
        KbDefinedExpression.prototype.evaluate = function (context) {
            return (!!context[this.key]);
        };
        KbDefinedExpression.prototype.normalize = function () {
            return this;
        };
        KbDefinedExpression.prototype.serialize = function () {
            return this.key;
        };
        return KbDefinedExpression;
    }());
    exports.KbDefinedExpression = KbDefinedExpression;
    var KbEqualsExpression = (function () {
        function KbEqualsExpression(key, value) {
            this.key = key;
            this.value = value;
        }
        KbEqualsExpression.prototype.equals = function (other) {
            if (other instanceof KbEqualsExpression) {
                return (this.key === other.key && this.value === other.value);
            }
            return false;
        };
        KbEqualsExpression.prototype.evaluate = function (context) {
            /* tslint:disable:triple-equals */
            // Intentional ==
            return (context[this.key] == this.value);
            /* tslint:enable:triple-equals */
        };
        KbEqualsExpression.prototype.normalize = function () {
            if (typeof this.value === 'boolean') {
                if (this.value) {
                    return new KbDefinedExpression(this.key);
                }
                return new KbNotExpression(this.key);
            }
            return this;
        };
        KbEqualsExpression.prototype.serialize = function () {
            if (typeof this.value === 'boolean') {
                return this.normalize().serialize();
            }
            return this.key + ' == \'' + this.value + '\'';
        };
        return KbEqualsExpression;
    }());
    exports.KbEqualsExpression = KbEqualsExpression;
    var KbNotEqualsExpression = (function () {
        function KbNotEqualsExpression(key, value) {
            this.key = key;
            this.value = value;
        }
        KbNotEqualsExpression.prototype.equals = function (other) {
            if (other instanceof KbNotEqualsExpression) {
                return (this.key === other.key && this.value === other.value);
            }
            return false;
        };
        KbNotEqualsExpression.prototype.evaluate = function (context) {
            /* tslint:disable:triple-equals */
            // Intentional !=
            return (context[this.key] != this.value);
            /* tslint:enable:triple-equals */
        };
        KbNotEqualsExpression.prototype.normalize = function () {
            if (typeof this.value === 'boolean') {
                if (this.value) {
                    return new KbNotExpression(this.key);
                }
                return new KbDefinedExpression(this.key);
            }
            return this;
        };
        KbNotEqualsExpression.prototype.serialize = function () {
            if (typeof this.value === 'boolean') {
                return this.normalize().serialize();
            }
            return this.key + ' != \'' + this.value + '\'';
        };
        return KbNotEqualsExpression;
    }());
    exports.KbNotEqualsExpression = KbNotEqualsExpression;
    var KbNotExpression = (function () {
        function KbNotExpression(key) {
            this.key = key;
        }
        KbNotExpression.prototype.equals = function (other) {
            if (other instanceof KbNotExpression) {
                return (this.key === other.key);
            }
            return false;
        };
        KbNotExpression.prototype.evaluate = function (context) {
            return (!context[this.key]);
        };
        KbNotExpression.prototype.normalize = function () {
            return this;
        };
        KbNotExpression.prototype.serialize = function () {
            return '!' + this.key;
        };
        return KbNotExpression;
    }());
    exports.KbNotExpression = KbNotExpression;
    var KbAndExpression = (function () {
        function KbAndExpression(expr) {
            this.expr = expr || [];
        }
        KbAndExpression.prototype.equals = function (other) {
            return this === other;
        };
        KbAndExpression.prototype.evaluate = function (context) {
            for (var i = 0, len = this.expr.length; i < len; i++) {
                if (!this.expr[i].evaluate(context)) {
                    return false;
                }
            }
            return true;
        };
        KbAndExpression.prototype.normalize = function () {
            var expr = [];
            for (var i = 0, len = this.expr.length; i < len; i++) {
                var e = this.expr[i];
                if (!e) {
                    continue;
                }
                e = e.normalize();
                if (!e) {
                    continue;
                }
                if (e instanceof KbAndExpression) {
                    expr = expr.concat(e.expr);
                    continue;
                }
                expr.push(e);
            }
            if (expr.length === 0) {
                return null;
            }
            if (expr.length === 1) {
                return expr[0];
            }
            return new KbAndExpression(expr);
        };
        KbAndExpression.prototype.serialize = function () {
            if (this.expr.length === 0) {
                return '';
            }
            if (this.expr.length === 1) {
                return this.normalize().serialize();
            }
            return this.expr.map(function (e) { return e.serialize(); }).join(' && ');
        };
        return KbAndExpression;
    }());
    exports.KbAndExpression = KbAndExpression;
    exports.KbExpr = {
        has: function (key) { return new KbDefinedExpression(key); },
        equals: function (key, value) { return new KbEqualsExpression(key, value); },
        notEquals: function (key, value) { return new KbNotEqualsExpression(key, value); },
        not: function (key) { return new KbNotExpression(key); },
        and: function () {
            var expr = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                expr[_i - 0] = arguments[_i];
            }
            return new KbAndExpression(expr);
        },
        deserialize: function (serialized) {
            if (!serialized) {
                return null;
            }
            var pieces = serialized.split('&&');
            var result = new KbAndExpression(pieces.map(function (p) { return exports.KbExpr._deserializeOne(p); }));
            return result.normalize();
        },
        _deserializeOne: function (serializedOne) {
            serializedOne = serializedOne.trim();
            if (serializedOne.indexOf('!=') >= 0) {
                var pieces = serializedOne.split('!=');
                return new KbNotEqualsExpression(pieces[0].trim(), exports.KbExpr._deserializeValue(pieces[1]));
            }
            if (serializedOne.indexOf('==') >= 0) {
                var pieces = serializedOne.split('==');
                return new KbEqualsExpression(pieces[0].trim(), exports.KbExpr._deserializeValue(pieces[1]));
            }
            if (/^\!\s*/.test(serializedOne)) {
                return new KbNotExpression(serializedOne.substr(1).trim());
            }
            return new KbDefinedExpression(serializedOne);
        },
        _deserializeValue: function (serializedValue) {
            serializedValue = serializedValue.trim();
            if (serializedValue === 'true') {
                return true;
            }
            if (serializedValue === 'false') {
                return false;
            }
            var m = /^'([^']*)'$/.exec(serializedValue);
            if (m) {
                return m[1].trim();
            }
            return serializedValue;
        }
    };
    exports.IKeybindingService = instantiation_1.createDecorator('keybindingService');
    exports.SET_CONTEXT_COMMAND_ID = 'setContext';
});

define("vs/platform/keybinding/common/keybindingResolver", ["require", "exports", 'vs/base/common/keyCodes', 'vs/base/common/platform', 'vs/platform/keybinding/common/keybindingService'], function (require, exports, keyCodes_1, platform, keybindingService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var KeybindingResolver = (function () {
        function KeybindingResolver(defaultKeybindings, overrides, shouldWarnOnConflict) {
            if (shouldWarnOnConflict === void 0) { shouldWarnOnConflict = true; }
            defaultKeybindings = defaultKeybindings.slice(0).sort(sorter);
            this._defaultKeybindings = defaultKeybindings;
            this._shouldWarnOnConflict = shouldWarnOnConflict;
            this._defaultBoundCommands = Object.create(null);
            for (var i = 0, len = defaultKeybindings.length; i < len; i++) {
                this._defaultBoundCommands[defaultKeybindings[i].command] = true;
            }
            this._map = Object.create(null);
            this._lookupMap = Object.create(null);
            this._lookupMapUnreachable = Object.create(null);
            this._chords = Object.create(null);
            var defaultKeybindingsCount = defaultKeybindings.length;
            var allKeybindings = defaultKeybindings.concat(overrides);
            for (var i = 0, len = allKeybindings.length; i < len; i++) {
                var k = allKeybindings[i];
                if (k.keybinding === 0) {
                    continue;
                }
                if (k.context) {
                    k.context = k.context.normalize();
                }
                var entry = {
                    context: k.context,
                    keybinding: k.keybinding,
                    commandId: k.command
                };
                if (keyCodes_1.BinaryKeybindings.hasChord(k.keybinding)) {
                    // This is a chord
                    var keybindingFirstPart = keyCodes_1.BinaryKeybindings.extractFirstPart(k.keybinding);
                    var keybindingChordPart = keyCodes_1.BinaryKeybindings.extractChordPart(k.keybinding);
                    this._chords[keybindingFirstPart] = this._chords[keybindingFirstPart] || Object.create(null);
                    this._chords[keybindingFirstPart][keybindingChordPart] = this._chords[keybindingFirstPart][keybindingChordPart] || [];
                    this._chords[keybindingFirstPart][keybindingChordPart].push(entry);
                    this._addKeyPress(keybindingFirstPart, entry, k, i < defaultKeybindingsCount);
                }
                else {
                    this._addKeyPress(k.keybinding, entry, k, i < defaultKeybindingsCount);
                }
            }
        }
        KeybindingResolver.prototype._addKeyPress = function (keypress, entry, item, isDefault) {
            if (!this._map[keypress]) {
                // There is no conflict so far
                this._map[keypress] = [entry];
                this._addToLookupMap(item);
                return;
            }
            var conflicts = this._map[keypress];
            for (var i = conflicts.length - 1; i >= 0; i--) {
                var conflict = conflicts[i];
                if (conflict.commandId === item.command) {
                    continue;
                }
                if (keyCodes_1.BinaryKeybindings.hasChord(conflict.keybinding) && keyCodes_1.BinaryKeybindings.hasChord(entry.keybinding) && conflict.keybinding !== entry.keybinding) {
                    // The conflict only shares the chord start with this command
                    continue;
                }
                if (KeybindingResolver.contextIsEntirelyIncluded(true, conflict.context, item.context)) {
                    // `item` completely overwrites `conflict`
                    if (this._shouldWarnOnConflict && isDefault) {
                        console.warn('Conflict detected, command `' + conflict.commandId + '` cannot be triggered by ' + keyCodes_1.Keybinding.toUserSettingsLabel(keypress) + ' due to ' + item.command);
                    }
                    this._lookupMapUnreachable[conflict.commandId] = this._lookupMapUnreachable[conflict.commandId] || [];
                    this._lookupMapUnreachable[conflict.commandId].push(conflict.keybinding);
                }
            }
            conflicts.push(entry);
            this._addToLookupMap(item);
        };
        /**
         * Returns true if `a` is completely covered by `b`.
         * Returns true if `b` is a more relaxed `a`.
         * Return true if (`a` === true implies `b` === true).
         */
        KeybindingResolver.contextIsEntirelyIncluded = function (inNormalizedForm, a, b) {
            if (!inNormalizedForm) {
                a = a ? a.normalize() : null;
                b = b ? b.normalize() : null;
            }
            if (!b) {
                return true;
            }
            if (!a) {
                return false;
            }
            var aRulesArr = a.serialize().split(' && ');
            var bRulesArr = b.serialize().split(' && ');
            var aRules = Object.create(null);
            for (var i = 0, len = aRulesArr.length; i < len; i++) {
                aRules[aRulesArr[i]] = true;
            }
            for (var i = 0, len = bRulesArr.length; i < len; i++) {
                if (!aRules[bRulesArr[i]]) {
                    return false;
                }
            }
            return true;
        };
        KeybindingResolver.prototype._addToLookupMap = function (item) {
            if (!item.command) {
                return;
            }
            this._lookupMap[item.command] = this._lookupMap[item.command] || [];
            this._lookupMap[item.command].push(item);
        };
        KeybindingResolver.prototype.getDefaultBoundCommands = function () {
            return this._defaultBoundCommands;
        };
        KeybindingResolver.prototype.getDefaultKeybindings = function () {
            var out = new OutputBuilder();
            out.writeLine('[');
            var lastIndex = this._defaultKeybindings.length - 1;
            this._defaultKeybindings.forEach(function (k, index) {
                IOSupport.writeKeybindingItem(out, k);
                if (index !== lastIndex) {
                    out.writeLine(',');
                }
                else {
                    out.writeLine();
                }
            });
            out.writeLine(']');
            return out.toString();
        };
        KeybindingResolver.prototype.lookupKeybinding = function (commandId) {
            var rawPossibleTriggers = this._lookupMap[commandId];
            if (!rawPossibleTriggers) {
                return [];
            }
            var possibleTriggers = rawPossibleTriggers.map(function (possibleTrigger) { return possibleTrigger.keybinding; });
            var remove = this._lookupMapUnreachable[commandId];
            if (remove) {
                possibleTriggers = possibleTriggers.filter(function (possibleTrigger) {
                    return remove.indexOf(possibleTrigger) === -1;
                });
            }
            var seenKeys = [];
            var result = possibleTriggers.filter(function (possibleTrigger) {
                if (seenKeys.indexOf(possibleTrigger) >= 0) {
                    return false;
                }
                seenKeys.push(possibleTrigger);
                return true;
            });
            return result.map(function (trigger) {
                return new keyCodes_1.Keybinding(trigger);
            }).reverse(); // sort most specific to the top
        };
        KeybindingResolver.prototype.resolve = function (context, currentChord, keypress) {
            // console.log('resolve: ' + Keybinding.toLabel(keypress));
            var lookupMap = null;
            if (currentChord !== 0) {
                var chords = this._chords[currentChord];
                if (!chords) {
                    return null;
                }
                lookupMap = chords[keypress];
            }
            else {
                lookupMap = this._map[keypress];
            }
            var result = this._findCommand(context, lookupMap);
            if (!result) {
                return null;
            }
            if (currentChord === 0 && keyCodes_1.BinaryKeybindings.hasChord(result.keybinding)) {
                return {
                    enterChord: keypress,
                    commandId: null
                };
            }
            return {
                enterChord: 0,
                commandId: result.commandId
            };
        };
        KeybindingResolver.prototype._findCommand = function (context, matches) {
            if (!matches) {
                return null;
            }
            for (var i = matches.length - 1; i >= 0; i--) {
                var k = matches[i];
                if (!KeybindingResolver.contextMatchesRules(context, k.context)) {
                    continue;
                }
                return k;
            }
            return null;
        };
        KeybindingResolver.contextMatchesRules = function (context, rules) {
            if (!rules) {
                return true;
            }
            return rules.evaluate(context);
        };
        return KeybindingResolver;
    }());
    exports.KeybindingResolver = KeybindingResolver;
    function rightPaddedString(str, minChars) {
        if (str.length < minChars) {
            return str + (new Array(minChars - str.length).join(' '));
        }
        return str;
    }
    function sorter(a, b) {
        if (a.weight1 !== b.weight1) {
            return a.weight1 - b.weight1;
        }
        if (a.command < b.command) {
            return -1;
        }
        if (a.command > b.command) {
            return 1;
        }
        return a.weight2 - b.weight2;
    }
    var OutputBuilder = (function () {
        function OutputBuilder() {
            this._lines = [];
            this._currentLine = '';
        }
        OutputBuilder.prototype.write = function (str) {
            this._currentLine += str;
        };
        OutputBuilder.prototype.writeLine = function (str) {
            if (str === void 0) { str = ''; }
            this._lines.push(this._currentLine + str);
            this._currentLine = '';
        };
        OutputBuilder.prototype.toString = function () {
            this.writeLine();
            return this._lines.join('\n');
        };
        return OutputBuilder;
    }());
    exports.OutputBuilder = OutputBuilder;
    var IOSupport = (function () {
        function IOSupport() {
        }
        IOSupport.writeKeybindingItem = function (out, item) {
            var quotedSerializedKeybinding = JSON.stringify(IOSupport.writeKeybinding(item.keybinding));
            out.write("{ \"key\": " + rightPaddedString(quotedSerializedKeybinding + ',', 25) + " \"command\": ");
            var serializedContext = item.context ? item.context.serialize() : '';
            var quotedSerializeCommand = JSON.stringify(item.command);
            if (serializedContext.length > 0) {
                out.write(quotedSerializeCommand + ",");
                out.writeLine();
                out.write("                                     \"when\": \"" + serializedContext + "\" ");
            }
            else {
                out.write(quotedSerializeCommand + " ");
            }
            //		out.write(String(item.weight));
            out.write('}');
        };
        IOSupport.readKeybindingItem = function (input, index) {
            var key = IOSupport.readKeybinding(input.key);
            var context = IOSupport.readKeybindingContexts(input.when);
            return {
                keybinding: key,
                command: input.command,
                context: context,
                weight1: 1000,
                weight2: index
            };
        };
        IOSupport.writeKeybinding = function (input, Platform) {
            if (Platform === void 0) { Platform = platform; }
            return keyCodes_1.Keybinding.toUserSettingsLabel(input, Platform);
        };
        IOSupport.readKeybinding = function (input, Platform) {
            if (Platform === void 0) { Platform = platform; }
            return keyCodes_1.Keybinding.fromUserSettingsLabel(input, Platform);
        };
        IOSupport.readKeybindingContexts = function (input) {
            return keybindingService_1.KbExpr.deserialize(input);
        };
        return IOSupport;
    }());
    exports.IOSupport = IOSupport;
});

define("vs/platform/lifecycle/common/lifecycle", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.ILifecycleService = instantiation_1.createDecorator('lifecycleService');
});

define("vs/platform/lifecycle/common/baseLifecycleService", ["require", "exports", 'vs/base/common/event', './lifecycle'], function (require, exports, event_1, lifecycle_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var BaseLifecycleService = (function () {
        function BaseLifecycleService() {
            this.serviceId = lifecycle_1.ILifecycleService;
            this._beforeShutdownParticipants = [];
            this._onShutdown = new event_1.Emitter();
        }
        BaseLifecycleService.prototype.fireShutdown = function () {
            this._onShutdown.fire();
        };
        BaseLifecycleService.prototype.addBeforeShutdownParticipant = function (p) {
            this._beforeShutdownParticipants.push(p);
        };
        Object.defineProperty(BaseLifecycleService.prototype, "beforeShutdownParticipants", {
            get: function () {
                return this._beforeShutdownParticipants;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseLifecycleService.prototype, "onShutdown", {
            get: function () {
                return this._onShutdown.event;
            },
            enumerable: true,
            configurable: true
        });
        return BaseLifecycleService;
    }());
    exports.BaseLifecycleService = BaseLifecycleService;
});

define("vs/platform/markers/common/markers", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IMarkerService = instantiation_1.createDecorator('markerService');
    (function (MarkerType) {
        MarkerType[MarkerType["transient"] = 1] = "transient";
        MarkerType[MarkerType["permanent"] = 2] = "permanent";
    })(exports.MarkerType || (exports.MarkerType = {}));
    var MarkerType = exports.MarkerType;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/markers/common/problemMatcher", ["require", "exports", 'vs/nls!vs/platform/markers/common/problemMatcher', 'vs/base/common/objects', 'vs/base/common/strings', 'vs/base/common/assert', 'vs/base/common/paths', 'vs/base/common/types', 'vs/base/common/severity', 'vs/base/common/uri', 'vs/base/common/parsers'], function (require, exports, NLS, Objects, Strings, Assert, Paths, Types, severity_1, uri_1, parsers_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (FileLocationKind) {
        FileLocationKind[FileLocationKind["Auto"] = 0] = "Auto";
        FileLocationKind[FileLocationKind["Relative"] = 1] = "Relative";
        FileLocationKind[FileLocationKind["Absolute"] = 2] = "Absolute";
    })(exports.FileLocationKind || (exports.FileLocationKind = {}));
    var FileLocationKind = exports.FileLocationKind;
    var FileLocationKind;
    (function (FileLocationKind) {
        function fromString(value) {
            value = value.toLowerCase();
            if (value === 'absolute') {
                return FileLocationKind.Absolute;
            }
            else if (value === 'relative') {
                return FileLocationKind.Relative;
            }
            else {
                return undefined;
            }
        }
        FileLocationKind.fromString = fromString;
    })(FileLocationKind = exports.FileLocationKind || (exports.FileLocationKind = {}));
    exports.problemPatternProperties = ['file', 'message', 'location', 'line', 'column', 'endLine', 'endColumn', 'code', 'severity', 'loop', 'mostSignifikant'];
    (function (ApplyToKind) {
        ApplyToKind[ApplyToKind["allDocuments"] = 0] = "allDocuments";
        ApplyToKind[ApplyToKind["openDocuments"] = 1] = "openDocuments";
        ApplyToKind[ApplyToKind["closedDocuments"] = 2] = "closedDocuments";
    })(exports.ApplyToKind || (exports.ApplyToKind = {}));
    var ApplyToKind = exports.ApplyToKind;
    var ApplyToKind;
    (function (ApplyToKind) {
        function fromString(value) {
            value = value.toLowerCase();
            if (value === 'alldocuments') {
                return ApplyToKind.allDocuments;
            }
            else if (value === 'opendocuments') {
                return ApplyToKind.openDocuments;
            }
            else if (value === 'closeddocuments') {
                return ApplyToKind.closedDocuments;
            }
            else {
                return undefined;
            }
        }
        ApplyToKind.fromString = fromString;
    })(ApplyToKind = exports.ApplyToKind || (exports.ApplyToKind = {}));
    function isNamedProblemMatcher(value) {
        return Types.isString(value.name) ? true : false;
    }
    exports.isNamedProblemMatcher = isNamedProblemMatcher;
    var valueMap = {
        E: 'error',
        W: 'warning',
        I: 'info',
    };
    function getResource(filename, matcher) {
        var kind = matcher.fileLocation;
        var fullPath;
        if (kind === FileLocationKind.Absolute) {
            fullPath = filename;
        }
        else if (kind === FileLocationKind.Relative) {
            fullPath = Paths.join(matcher.filePrefix, filename);
        }
        fullPath = fullPath.replace(/\\/g, '/');
        if (fullPath[0] !== '/') {
            fullPath = '/' + fullPath;
        }
        return uri_1.default.parse('file://' + fullPath);
    }
    exports.getResource = getResource;
    function createLineMatcher(matcher) {
        var pattern = matcher.pattern;
        if (Types.isArray(pattern)) {
            return new MultiLineMatcher(matcher);
        }
        else {
            return new SingleLineMatcher(matcher);
        }
    }
    exports.createLineMatcher = createLineMatcher;
    var AbstractLineMatcher = (function () {
        function AbstractLineMatcher(matcher) {
            this.matcher = matcher;
        }
        AbstractLineMatcher.prototype.handle = function (lines, start) {
            if (start === void 0) { start = 0; }
            return { match: null, continue: false };
        };
        AbstractLineMatcher.prototype.next = function (line) {
            return null;
        };
        Object.defineProperty(AbstractLineMatcher.prototype, "matchLength", {
            get: function () {
                throw new Error('Subclass reponsibility');
            },
            enumerable: true,
            configurable: true
        });
        AbstractLineMatcher.prototype.fillProblemData = function (data, pattern, matches) {
            this.fillProperty(data, 'file', pattern, matches, true);
            this.fillProperty(data, 'message', pattern, matches, true);
            this.fillProperty(data, 'code', pattern, matches, true);
            this.fillProperty(data, 'severity', pattern, matches, true);
            this.fillProperty(data, 'location', pattern, matches, true);
            this.fillProperty(data, 'line', pattern, matches);
            this.fillProperty(data, 'column', pattern, matches);
            this.fillProperty(data, 'endLine', pattern, matches);
            this.fillProperty(data, 'endColumn', pattern, matches);
        };
        AbstractLineMatcher.prototype.fillProperty = function (data, property, pattern, matches, trim) {
            if (trim === void 0) { trim = false; }
            if (Types.isUndefined(data[property]) && !Types.isUndefined(pattern[property]) && pattern[property] < matches.length) {
                var value = matches[pattern[property]];
                if (trim) {
                    value = Strings.trim(value);
                }
                data[property] = value;
            }
        };
        AbstractLineMatcher.prototype.getMarkerMatch = function (data) {
            var location = this.getLocation(data);
            if (data.file && location && data.message) {
                var marker = {
                    severity: this.getSeverity(data),
                    startLineNumber: location.startLineNumber,
                    startColumn: location.startColumn,
                    endLineNumber: location.startLineNumber,
                    endColumn: location.endColumn,
                    message: data.message
                };
                if (!Types.isUndefined(data.code)) {
                    marker.code = data.code;
                }
                return {
                    description: this.matcher,
                    resource: this.getResource(data.file),
                    marker: marker
                };
            }
        };
        AbstractLineMatcher.prototype.getResource = function (filename) {
            return getResource(filename, this.matcher);
        };
        AbstractLineMatcher.prototype.getLocation = function (data) {
            if (data.location) {
                return this.parseLocationInfo(data.location);
            }
            if (!data.line) {
                return null;
            }
            var startLine = parseInt(data.line);
            var startColumn = data.column ? parseInt(data.column) : undefined;
            var endLine = data.endLine ? parseInt(data.endLine) : undefined;
            var endColumn = data.endColumn ? parseInt(data.endColumn) : undefined;
            return this.createLocation(startLine, startColumn, endLine, endColumn);
        };
        AbstractLineMatcher.prototype.parseLocationInfo = function (value) {
            if (!value || !value.match(/(\d+|\d+,\d+|\d+,\d+,\d+,\d+)/)) {
                return null;
            }
            var parts = value.split(',');
            var startLine = parseInt(parts[0]);
            var startColumn = parts.length > 1 ? parseInt(parts[1]) : undefined;
            if (parts.length > 3) {
                return this.createLocation(startLine, startColumn, parseInt(parts[2]), parseInt(parts[3]));
            }
            else {
                return this.createLocation(startLine, startColumn, undefined, undefined);
            }
        };
        AbstractLineMatcher.prototype.createLocation = function (startLine, startColumn, endLine, endColumn) {
            if (startLine && startColumn && endLine && endColumn) {
                return { startLineNumber: startLine, startColumn: startColumn, endLineNumber: endLine, endColumn: endColumn };
            }
            if (startLine && startColumn) {
                return { startLineNumber: startLine, startColumn: startColumn, endLineNumber: startLine, endColumn: startColumn };
            }
            return { startLineNumber: startLine, startColumn: 1, endLineNumber: startLine, endColumn: Number.MAX_VALUE };
        };
        AbstractLineMatcher.prototype.getSeverity = function (data) {
            var result = null;
            if (data.severity) {
                var value = data.severity;
                if (value && value.length > 0) {
                    if (value.length === 1 && valueMap[value[0]]) {
                        value = valueMap[value[0]];
                    }
                    result = severity_1.default.fromValue(value);
                }
            }
            if (result === null || result === severity_1.default.Ignore) {
                result = this.matcher.severity || severity_1.default.Error;
            }
            return result;
        };
        return AbstractLineMatcher;
    }());
    var SingleLineMatcher = (function (_super) {
        __extends(SingleLineMatcher, _super);
        function SingleLineMatcher(matcher) {
            _super.call(this, matcher);
            this.pattern = matcher.pattern;
        }
        Object.defineProperty(SingleLineMatcher.prototype, "matchLength", {
            get: function () {
                return 1;
            },
            enumerable: true,
            configurable: true
        });
        SingleLineMatcher.prototype.handle = function (lines, start) {
            if (start === void 0) { start = 0; }
            Assert.ok(lines.length - start === 1);
            var data = Object.create(null);
            var matches = this.pattern.regexp.exec(lines[start]);
            if (matches) {
                this.fillProblemData(data, this.pattern, matches);
                var match = this.getMarkerMatch(data);
                if (match) {
                    return { match: match, continue: false };
                }
            }
            return { match: null, continue: false };
        };
        SingleLineMatcher.prototype.next = function (line) {
            return null;
        };
        return SingleLineMatcher;
    }(AbstractLineMatcher));
    var MultiLineMatcher = (function (_super) {
        __extends(MultiLineMatcher, _super);
        function MultiLineMatcher(matcher) {
            _super.call(this, matcher);
            this.patterns = matcher.pattern;
        }
        Object.defineProperty(MultiLineMatcher.prototype, "matchLength", {
            get: function () {
                return this.patterns.length;
            },
            enumerable: true,
            configurable: true
        });
        MultiLineMatcher.prototype.handle = function (lines, start) {
            if (start === void 0) { start = 0; }
            Assert.ok(lines.length - start === this.patterns.length);
            this.data = Object.create(null);
            var data = this.data;
            for (var i = 0; i < this.patterns.length; i++) {
                var pattern = this.patterns[i];
                var matches = pattern.regexp.exec(lines[i + start]);
                if (!matches) {
                    return { match: null, continue: false };
                }
                else {
                    // Only the last pattern can loop
                    if (pattern.loop && i === this.patterns.length - 1) {
                        data = Objects.clone(data);
                    }
                    this.fillProblemData(data, pattern, matches);
                }
            }
            var loop = this.patterns[this.patterns.length - 1].loop;
            if (!loop) {
                this.data = null;
            }
            return { match: this.getMarkerMatch(data), continue: loop };
        };
        MultiLineMatcher.prototype.next = function (line) {
            var pattern = this.patterns[this.patterns.length - 1];
            Assert.ok(pattern.loop === true && this.data !== null);
            var matches = pattern.regexp.exec(line);
            if (!matches) {
                this.data = null;
                return null;
            }
            var data = Objects.clone(this.data);
            this.fillProblemData(data, pattern, matches);
            return this.getMarkerMatch(data);
        };
        return MultiLineMatcher;
    }(AbstractLineMatcher));
    var _defaultPatterns = Object.create(null);
    _defaultPatterns['msCompile'] = {
        regexp: /^([^\s].*)\((\d+|\d+,\d+|\d+,\d+,\d+,\d+)\):\s+(error|warning|info)\s+(\w{1,2}\d+)\s*:\s*(.*)$/,
        file: 1,
        location: 2,
        severity: 3,
        code: 4,
        message: 5
    };
    _defaultPatterns['gulp-tsc'] = {
        regexp: /^([^\s].*)\((\d+|\d+,\d+|\d+,\d+,\d+,\d+)\):\s+(\d+)\s+(.*)$/,
        file: 1,
        location: 2,
        code: 3,
        message: 4
    };
    _defaultPatterns['tsc'] = {
        regexp: /^([^\s].*)\((\d+|\d+,\d+|\d+,\d+,\d+,\d+)\):\s+(error|warning|info)\s+(TS\d+)\s*:\s*(.*)$/,
        file: 1,
        location: 2,
        severity: 3,
        code: 4,
        message: 5
    };
    _defaultPatterns['cpp'] = {
        regexp: /^([^\s].*)\((\d+|\d+,\d+|\d+,\d+,\d+,\d+)\):\s+(error|warning|info)\s+(C\d+)\s*:\s*(.*)$/,
        file: 1,
        location: 2,
        severity: 3,
        code: 4,
        message: 5
    };
    _defaultPatterns['csc'] = {
        regexp: /^([^\s].*)\((\d+|\d+,\d+|\d+,\d+,\d+,\d+)\):\s+(error|warning|info)\s+(CS\d+)\s*:\s*(.*)$/,
        file: 1,
        location: 2,
        severity: 3,
        code: 4,
        message: 5
    };
    _defaultPatterns['vb'] = {
        regexp: /^([^\s].*)\((\d+|\d+,\d+|\d+,\d+,\d+,\d+)\):\s+(error|warning|info)\s+(BC\d+)\s*:\s*(.*)$/,
        file: 1,
        location: 2,
        severity: 3,
        code: 4,
        message: 5
    };
    _defaultPatterns['lessCompile'] = {
        regexp: /^\s*(.*) in file (.*) line no. (\d+)$/,
        message: 1,
        file: 2,
        line: 3
    };
    _defaultPatterns['jshint'] = {
        regexp: /^(.*):\s+line\s+(\d+),\s+col\s+(\d+),\s(.+?)(?:\s+\((\w)(\d+)\))?$/,
        file: 1,
        line: 2,
        column: 3,
        message: 4,
        severity: 5,
        code: 6
    };
    _defaultPatterns['jshint-stylish'] = [
        {
            regexp: /^(.+)$/,
            file: 1
        },
        {
            regexp: /^\s+line\s+(\d+)\s+col\s+(\d+)\s+(.+?)(?:\s+\((\w)(\d+)\))?$/,
            line: 1,
            column: 2,
            message: 3,
            severity: 4,
            code: 5,
            loop: true
        }
    ];
    _defaultPatterns['eslint-compact'] = {
        regexp: /^(.+):\sline\s(\d+),\scol\s(\d+),\s(Error|Warning|Info)\s-\s(.+)\s\((.+)\)$/,
        file: 1,
        line: 2,
        column: 3,
        severity: 4,
        message: 5,
        code: 6
    };
    _defaultPatterns['eslint-stylish'] = [
        {
            regexp: /^([^\s].*)$/,
            file: 1
        },
        {
            regexp: /^\s+(\d+):(\d+)\s+(error|warning|info)\s+(.+?)\s\s+(.*)$/,
            line: 1,
            column: 2,
            severity: 3,
            message: 4,
            code: 5,
            loop: true
        }
    ];
    function defaultPattern(name) {
        return _defaultPatterns[name];
    }
    exports.defaultPattern = defaultPattern;
    var Config;
    (function (Config) {
        /**
        * Defines possible problem severity values
        */
        var ProblemSeverity;
        (function (ProblemSeverity) {
            ProblemSeverity.Error = 'error';
            ProblemSeverity.Warning = 'warning';
            ProblemSeverity.Info = 'info';
        })(ProblemSeverity = Config.ProblemSeverity || (Config.ProblemSeverity = {}));
        function isNamedProblemMatcher(value) {
            return Types.isString(value.name);
        }
        Config.isNamedProblemMatcher = isNamedProblemMatcher;
    })(Config = exports.Config || (exports.Config = {}));
    var ProblemMatcherParser = (function (_super) {
        __extends(ProblemMatcherParser, _super);
        function ProblemMatcherParser(resolver, logger, validationStatus) {
            if (validationStatus === void 0) { validationStatus = new parsers_1.ValidationStatus(); }
            _super.call(this, logger, validationStatus);
            this.resolver = resolver;
        }
        ProblemMatcherParser.prototype.parse = function (json) {
            var result = this.createProblemMatcher(json);
            if (!this.checkProblemMatcherValid(json, result)) {
                return null;
            }
            this.addWatchingMatcher(json, result);
            return result;
        };
        ProblemMatcherParser.prototype.checkProblemMatcherValid = function (externalProblemMatcher, problemMatcher) {
            if (!problemMatcher || !problemMatcher.pattern || !problemMatcher.owner || Types.isUndefined(problemMatcher.fileLocation)) {
                this.status.state = parsers_1.ValidationState.Fatal;
                this.log(NLS.localize(0, null, JSON.stringify(externalProblemMatcher, null, 4)));
                return false;
            }
            return true;
        };
        ProblemMatcherParser.prototype.createProblemMatcher = function (description) {
            var result = null;
            var owner = description.owner ? description.owner : 'external';
            var applyTo = Types.isString(description.applyTo) ? ApplyToKind.fromString(description.applyTo) : ApplyToKind.allDocuments;
            if (!applyTo) {
                applyTo = ApplyToKind.allDocuments;
            }
            var fileLocation = undefined;
            var filePrefix = undefined;
            var kind;
            if (Types.isUndefined(description.fileLocation)) {
                fileLocation = FileLocationKind.Relative;
                filePrefix = '${cwd}';
            }
            else if (Types.isString(description.fileLocation)) {
                kind = FileLocationKind.fromString(description.fileLocation);
                if (kind) {
                    fileLocation = kind;
                    if (kind === FileLocationKind.Relative) {
                        filePrefix = '${cwd}';
                    }
                }
            }
            else if (Types.isStringArray(description.fileLocation)) {
                var values = description.fileLocation;
                if (values.length > 0) {
                    kind = FileLocationKind.fromString(values[0]);
                    if (values.length === 1 && kind === FileLocationKind.Absolute) {
                        fileLocation = kind;
                    }
                    else if (values.length === 2 && kind === FileLocationKind.Relative && values[1]) {
                        fileLocation = kind;
                        filePrefix = values[1];
                    }
                }
            }
            var pattern = description.pattern ? this.createProblemPattern(description.pattern) : undefined;
            var severity = description.severity ? severity_1.default.fromValue(description.severity) : undefined;
            if (severity === severity_1.default.Ignore) {
                this.status.state = parsers_1.ValidationState.Info;
                this.log(NLS.localize(1, null, description.severity));
                severity = severity_1.default.Error;
            }
            if (Types.isString(description.base)) {
                var variableName = description.base;
                if (variableName.length > 1 && variableName[0] === '$') {
                    var base = this.resolver.get(variableName.substring(1));
                    if (base) {
                        result = Objects.clone(base);
                        if (description.owner) {
                            result.owner = owner;
                        }
                        if (fileLocation) {
                            result.fileLocation = fileLocation;
                        }
                        if (filePrefix) {
                            result.filePrefix = filePrefix;
                        }
                        if (description.pattern) {
                            result.pattern = pattern;
                        }
                        if (description.severity) {
                            result.severity = severity;
                        }
                    }
                }
            }
            else if (fileLocation) {
                result = {
                    owner: owner,
                    applyTo: applyTo,
                    fileLocation: fileLocation,
                    pattern: pattern,
                };
                if (filePrefix) {
                    result.filePrefix = filePrefix;
                }
                if (severity) {
                    result.severity = severity;
                }
            }
            if (Config.isNamedProblemMatcher(description)) {
                result.name = description.name;
            }
            return result;
        };
        ProblemMatcherParser.prototype.createProblemPattern = function (value) {
            var pattern;
            if (Types.isString(value)) {
                var variableName = value;
                if (variableName.length > 1 && variableName[0] === '$') {
                    return defaultPattern(variableName.substring(1));
                }
            }
            else if (Types.isArray(value)) {
                var values = value;
                var result = [];
                for (var i = 0; i < values.length; i++) {
                    pattern = this.createSingleProblemPattern(values[i], false);
                    if (i < values.length - 1) {
                        if (!Types.isUndefined(pattern.loop) && pattern.loop) {
                            pattern.loop = false;
                            this.status.state = parsers_1.ValidationState.Error;
                            this.log(NLS.localize(2, null));
                        }
                    }
                    result.push(pattern);
                }
                this.validateProblemPattern(result);
                return result;
            }
            else {
                pattern = this.createSingleProblemPattern(value, true);
                if (!Types.isUndefined(pattern.loop) && pattern.loop) {
                    pattern.loop = false;
                    this.status.state = parsers_1.ValidationState.Error;
                    this.log(NLS.localize(3, null));
                }
                this.validateProblemPattern([pattern]);
                return pattern;
            }
            return null;
        };
        ProblemMatcherParser.prototype.createSingleProblemPattern = function (value, setDefaults) {
            var result = {
                regexp: this.createRegularExpression(value.regexp)
            };
            exports.problemPatternProperties.forEach(function (property) {
                if (!Types.isUndefined(value[property])) {
                    result[property] = value[property];
                }
            });
            if (setDefaults) {
                if (result.location) {
                    result = Objects.mixin(result, {
                        file: 1,
                        message: 4
                    }, false);
                }
                else {
                    result = Objects.mixin(result, {
                        file: 1,
                        line: 2,
                        column: 3,
                        message: 4
                    }, false);
                }
            }
            return result;
        };
        ProblemMatcherParser.prototype.validateProblemPattern = function (values) {
            var file, message, location, line;
            var regexp = 0;
            values.forEach(function (pattern) {
                file = file || !!pattern.file;
                message = message || !!pattern.message;
                location = location || !!pattern.location;
                line = line || !!pattern.line;
                if (pattern.regexp) {
                    regexp++;
                }
            });
            if (regexp !== values.length) {
                this.status.state = parsers_1.ValidationState.Error;
                this.log(NLS.localize(4, null));
            }
            if (!(file && message && (location || line))) {
                this.status.state = parsers_1.ValidationState.Error;
                this.log(NLS.localize(5, null));
            }
        };
        ProblemMatcherParser.prototype.addWatchingMatcher = function (external, internal) {
            var oldBegins = this.createRegularExpression(external.watchedTaskBeginsRegExp);
            var oldEnds = this.createRegularExpression(external.watchedTaskEndsRegExp);
            if (oldBegins && oldEnds) {
                internal.watching = {
                    activeOnStart: false,
                    beginsPattern: { regexp: oldBegins },
                    endsPattern: { regexp: oldEnds }
                };
                return;
            }
            if (Types.isUndefinedOrNull(external.watching)) {
                return;
            }
            var watching = external.watching;
            var begins = this.createWatchingPattern(watching.beginsPattern);
            var ends = this.createWatchingPattern(watching.endsPattern);
            if (begins && ends) {
                internal.watching = {
                    activeOnStart: Types.isBoolean(watching.activeOnStart) ? watching.activeOnStart : false,
                    beginsPattern: begins,
                    endsPattern: ends
                };
                return;
            }
            if (begins || ends) {
                this.status.state = parsers_1.ValidationState.Error;
                this.log(NLS.localize(6, null));
            }
        };
        ProblemMatcherParser.prototype.createWatchingPattern = function (external) {
            if (Types.isUndefinedOrNull(external)) {
                return null;
            }
            var regexp;
            var file;
            if (Types.isString(external)) {
                regexp = this.createRegularExpression(external);
            }
            else {
                regexp = this.createRegularExpression(external.regexp);
                if (Types.isNumber(external.file)) {
                    file = external.file;
                }
            }
            if (!regexp) {
                return null;
            }
            return file ? { regexp: regexp, file: file } : { regexp: regexp };
        };
        ProblemMatcherParser.prototype.createRegularExpression = function (value) {
            var result = null;
            if (!value) {
                return result;
            }
            try {
                result = new RegExp(value);
            }
            catch (err) {
                this.status.state = parsers_1.ValidationState.Fatal;
                this.log(NLS.localize(7, null, value));
            }
            return result;
        };
        return ProblemMatcherParser;
    }(parsers_1.Parser));
    exports.ProblemMatcherParser = ProblemMatcherParser;
    // let problemMatchersExtPoint = ExtensionsRegistry.registerExtensionPoint<Config.NamedProblemMatcher | Config.NamedProblemMatcher[]>('problemMatchers', {
    // TODO@Dirk: provide here JSON schema for extension point
    // });
    var ProblemMatcherRegistry = (function () {
        function ProblemMatcherRegistry() {
            this.matchers = Object.create(null);
            /*
            problemMatchersExtPoint.setHandler((extensions, collector) => {
                // TODO@Dirk: validate extensions here and collect errors/warnings in `collector`
                extensions.forEach(extension => {
                    let extensions = extension.value;
                    if (Types.isArray(extensions)) {
                        (<Config.NamedProblemMatcher[]>extensions).forEach(this.onProblemMatcher, this);
                    } else {
                        this.onProblemMatcher(extensions)
                    }
                });
            });
            */
        }
        // private onProblemMatcher(json: Config.NamedProblemMatcher): void {
        // 	let logger: ILogger = {
        // 		log: (message) => { console.warn(message); }
        // 	}
        // 	let parser = new ProblemMatcherParser(this, logger);
        // 	let result = parser.parse(json);
        // 	if (isNamedProblemMatcher(result) && parser.status.isOK()) {
        // 		this.add(result.name, result);
        // 	}
        // }
        ProblemMatcherRegistry.prototype.add = function (name, matcher) {
            this.matchers[name] = matcher;
        };
        ProblemMatcherRegistry.prototype.get = function (name) {
            return this.matchers[name];
        };
        ProblemMatcherRegistry.prototype.exists = function (name) {
            return !!this.matchers[name];
        };
        ProblemMatcherRegistry.prototype.remove = function (name) {
            delete this.matchers[name];
        };
        return ProblemMatcherRegistry;
    }());
    exports.ProblemMatcherRegistry = ProblemMatcherRegistry;
    exports.registry = new ProblemMatcherRegistry();
    exports.registry.add('msCompile', {
        owner: 'msCompile',
        applyTo: ApplyToKind.allDocuments,
        fileLocation: FileLocationKind.Absolute,
        pattern: defaultPattern('msCompile')
    });
    exports.registry.add('lessCompile', {
        owner: 'lessCompile',
        applyTo: ApplyToKind.allDocuments,
        fileLocation: FileLocationKind.Absolute,
        pattern: defaultPattern('lessCompile'),
        severity: severity_1.default.Error
    });
    exports.registry.add('tsc', {
        owner: 'typescript',
        applyTo: ApplyToKind.closedDocuments,
        fileLocation: FileLocationKind.Relative,
        filePrefix: '${cwd}',
        pattern: defaultPattern('tsc')
    });
    var matcher = {
        owner: 'typescript',
        applyTo: ApplyToKind.closedDocuments,
        fileLocation: FileLocationKind.Relative,
        filePrefix: '${cwd}',
        pattern: defaultPattern('tsc'),
        watching: {
            activeOnStart: true,
            beginsPattern: { regexp: /^\s*(?:message TS6032:|\d{1,2}:\d{1,2}:\d{1,2}(?: AM| PM)? -) File change detected\. Starting incremental compilation\.\.\./ },
            endsPattern: { regexp: /^\s*(?:message TS6042:|\d{1,2}:\d{1,2}:\d{1,2}(?: AM| PM)? -) Compilation complete\. Watching for file changes\./ }
        }
    };
    matcher.tscWatch = true;
    exports.registry.add('tsc-watch', matcher);
    exports.registry.add('gulp-tsc', {
        owner: 'typescript',
        applyTo: ApplyToKind.closedDocuments,
        fileLocation: FileLocationKind.Relative,
        filePrefix: '${cwd}',
        pattern: defaultPattern('gulp-tsc')
    });
    exports.registry.add('jshint', {
        owner: 'javascript',
        applyTo: ApplyToKind.allDocuments,
        fileLocation: FileLocationKind.Absolute,
        pattern: defaultPattern('jshint')
    });
    exports.registry.add('jshint-stylish', {
        owner: 'javascript',
        applyTo: ApplyToKind.allDocuments,
        fileLocation: FileLocationKind.Absolute,
        pattern: defaultPattern('jshint-stylish')
    });
    exports.registry.add('eslint-compact', {
        owner: 'javascript',
        applyTo: ApplyToKind.allDocuments,
        fileLocation: FileLocationKind.Relative,
        filePrefix: '${cwd}',
        pattern: defaultPattern('eslint-compact')
    });
    exports.registry.add('eslint-stylish', {
        owner: 'javascript',
        applyTo: ApplyToKind.allDocuments,
        fileLocation: FileLocationKind.Absolute,
        pattern: defaultPattern('eslint-stylish')
    });
});

define("vs/platform/message/common/message", ["require", "exports", 'vs/nls!vs/platform/message/common/message', 'vs/base/common/winjs.base', 'vs/base/common/severity', 'vs/platform/instantiation/common/instantiation', 'vs/base/common/actions'], function (require, exports, nls, winjs_base_1, severity_1, instantiation_1, actions_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.CloseAction = new actions_1.Action('close.message', nls.localize(0, null), null, true, function () { return winjs_base_1.TPromise.as(true); });
    exports.CancelAction = new actions_1.Action('close.message', nls.localize(1, null), null, true, function () { return winjs_base_1.TPromise.as(true); });
    exports.IMessageService = instantiation_1.createDecorator('messageService');
    exports.Severity = severity_1.default;
});

define("vs/platform/platform", ["require", "exports", 'vs/base/common/types', 'vs/base/common/assert'], function (require, exports, Types, Assert) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var RegistryImpl = (function () {
        function RegistryImpl() {
            this.data = {};
        }
        RegistryImpl.prototype.add = function (id, data) {
            Assert.ok(Types.isString(id));
            Assert.ok(Types.isObject(data));
            Assert.ok(!this.data.hasOwnProperty(id), 'There is already an extension with this id');
            this.data[id] = data;
        };
        RegistryImpl.prototype.knows = function (id) {
            return this.data.hasOwnProperty(id);
        };
        RegistryImpl.prototype.as = function (id) {
            return this.data[id] || null;
        };
        return RegistryImpl;
    }());
    exports.Registry = new RegistryImpl();
    /**
     * A base class for registries that leverage the instantiation service to create instances.
     */
    var BaseRegistry = (function () {
        function BaseRegistry() {
            this.toBeInstantiated = [];
            this.instances = [];
        }
        BaseRegistry.prototype.setInstantiationService = function (service) {
            this.instantiationService = service;
            while (this.toBeInstantiated.length > 0) {
                var entry = this.toBeInstantiated.shift();
                this.instantiate(entry);
            }
        };
        BaseRegistry.prototype.instantiate = function (ctor) {
            var instance = this.instantiationService.createInstance(ctor);
            this.instances.push(instance);
        };
        BaseRegistry.prototype._register = function (ctor) {
            if (this.instantiationService) {
                this.instantiate(ctor);
            }
            else {
                this.toBeInstantiated.push(ctor);
            }
        };
        BaseRegistry.prototype._getInstances = function () {
            return this.instances.slice(0);
        };
        BaseRegistry.prototype._setInstances = function (instances) {
            this.instances = instances;
        };
        return BaseRegistry;
    }());
    exports.BaseRegistry = BaseRegistry;
});

define("vs/editor/browser/editorBrowserExtensions", ["require", "exports", 'vs/platform/platform'], function (require, exports, platform_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EditorBrowserRegistry;
    (function (EditorBrowserRegistry) {
        // --- Editor Contributions
        function registerEditorContribution(ctor) {
            platform_1.Registry.as(Extensions.EditorContributions).registerEditorBrowserContribution(ctor);
        }
        EditorBrowserRegistry.registerEditorContribution = registerEditorContribution;
        function getEditorContributions() {
            return platform_1.Registry.as(Extensions.EditorContributions).getEditorBrowserContributions();
        }
        EditorBrowserRegistry.getEditorContributions = getEditorContributions;
    })(EditorBrowserRegistry = exports.EditorBrowserRegistry || (exports.EditorBrowserRegistry = {}));
    var SimpleEditorContributionDescriptor = (function () {
        function SimpleEditorContributionDescriptor(ctor) {
            this._ctor = ctor;
        }
        SimpleEditorContributionDescriptor.prototype.createInstance = function (instantiationService, editor) {
            // cast added to help the compiler, can remove once IConstructorSignature1 has been removed
            return instantiationService.createInstance(this._ctor, editor);
        };
        return SimpleEditorContributionDescriptor;
    }());
    // Editor extension points
    var Extensions = {
        EditorContributions: 'editor.contributions'
    };
    var EditorContributionRegistry = (function () {
        function EditorContributionRegistry() {
            this.editorContributions = [];
        }
        EditorContributionRegistry.prototype.registerEditorBrowserContribution = function (ctor) {
            this.editorContributions.push(new SimpleEditorContributionDescriptor(ctor));
        };
        EditorContributionRegistry.prototype.getEditorBrowserContributions = function () {
            return this.editorContributions.slice(0);
        };
        return EditorContributionRegistry;
    }());
    platform_1.Registry.add(Extensions.EditorContributions, new EditorContributionRegistry());
});

define("vs/editor/common/modes/modesRegistry", ["require", "exports", 'vs/nls!vs/editor/common/modes/modesRegistry', 'vs/base/common/event', 'vs/platform/platform'], function (require, exports, nls, event_1, platform_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // Define extension point ids
    exports.Extensions = {
        ModesRegistry: 'editor.modesRegistry'
    };
    var EditorModesRegistry = (function () {
        function EditorModesRegistry() {
            this._onDidAddCompatModes = new event_1.Emitter();
            this.onDidAddCompatModes = this._onDidAddCompatModes.event;
            this._onDidAddLanguages = new event_1.Emitter();
            this.onDidAddLanguages = this._onDidAddLanguages.event;
            this._workerParticipants = [];
            this._compatModes = [];
            this._languages = [];
        }
        // --- worker participants
        EditorModesRegistry.prototype.registerWorkerParticipants = function (participants) {
            this._workerParticipants = participants;
        };
        EditorModesRegistry.prototype.registerWorkerParticipant = function (modeId, moduleId, ctorName) {
            if (typeof modeId !== 'string') {
                throw new Error('InvalidArgument: expected `modeId` to be a string');
            }
            if (typeof moduleId !== 'string') {
                throw new Error('InvalidArgument: expected `moduleId` to be a string');
            }
            this._workerParticipants.push({
                modeId: modeId,
                moduleId: moduleId,
                ctorName: ctorName
            });
        };
        EditorModesRegistry.prototype.getWorkerParticipantsForMode = function (modeId) {
            return this._workerParticipants.filter(function (p) { return p.modeId === modeId; });
        };
        EditorModesRegistry.prototype.getWorkerParticipants = function () {
            return this._workerParticipants;
        };
        // --- compat modes
        EditorModesRegistry.prototype.registerCompatModes = function (def) {
            this._compatModes = this._compatModes.concat(def);
            this._onDidAddCompatModes.fire(def);
        };
        EditorModesRegistry.prototype.registerCompatMode = function (def) {
            this._compatModes.push(def);
            this._onDidAddCompatModes.fire([def]);
        };
        EditorModesRegistry.prototype.getCompatModes = function () {
            return this._compatModes.slice(0);
        };
        // --- languages
        EditorModesRegistry.prototype.registerLanguage = function (def) {
            this._languages.push(def);
            this._onDidAddLanguages.fire([def]);
        };
        EditorModesRegistry.prototype.registerLanguages = function (def) {
            this._languages = this._languages.concat(def);
            this._onDidAddLanguages.fire(def);
        };
        EditorModesRegistry.prototype.getLanguages = function () {
            return this._languages.slice(0);
        };
        return EditorModesRegistry;
    }());
    exports.EditorModesRegistry = EditorModesRegistry;
    exports.ModesRegistry = new EditorModesRegistry();
    platform_1.Registry.add(exports.Extensions.ModesRegistry, exports.ModesRegistry);
    exports.ModesRegistry.registerLanguage({
        id: 'plaintext',
        extensions: ['.txt', '.gitignore'],
        aliases: [nls.localize(0, null), 'text'],
        mimetypes: ['text/plain']
    });
});

define("vs/editor/common/services/languagesRegistry", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/event', 'vs/base/common/mime', 'vs/base/common/strings', 'vs/editor/common/modes/modesRegistry'], function (require, exports, errors_1, event_1, mime, strings, modesRegistry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var hasOwnProperty = Object.prototype.hasOwnProperty;
    var LanguagesRegistry = (function () {
        function LanguagesRegistry(useModesRegistry) {
            var _this = this;
            if (useModesRegistry === void 0) { useModesRegistry = true; }
            this._onDidAddModes = new event_1.Emitter();
            this.onDidAddModes = this._onDidAddModes.event;
            this.knownModeIds = {};
            this.mime2LanguageId = {};
            this.name2LanguageId = {};
            this.id2Name = {};
            this.name2Extensions = {};
            this.compatModes = {};
            this.lowerName2Id = {};
            this.id2ConfigurationFiles = {};
            if (useModesRegistry) {
                this._registerCompatModes(modesRegistry_1.ModesRegistry.getCompatModes());
                modesRegistry_1.ModesRegistry.onDidAddCompatModes(function (m) { return _this._registerCompatModes(m); });
                this._registerLanguages(modesRegistry_1.ModesRegistry.getLanguages());
                modesRegistry_1.ModesRegistry.onDidAddLanguages(function (m) { return _this._registerLanguages(m); });
            }
        }
        LanguagesRegistry.prototype._registerCompatModes = function (defs) {
            var addedModes = [];
            for (var i = 0; i < defs.length; i++) {
                var def = defs[i];
                this._registerLanguage({
                    id: def.id,
                    extensions: def.extensions,
                    filenames: def.filenames,
                    firstLine: def.firstLine,
                    aliases: def.aliases,
                    mimetypes: def.mimetypes
                });
                this.compatModes[def.id] = {
                    moduleId: def.moduleId,
                    ctorName: def.ctorName
                };
                addedModes.push(def.id);
            }
            this._onDidAddModes.fire(addedModes);
        };
        LanguagesRegistry.prototype._registerLanguages = function (desc) {
            var addedModes = [];
            for (var i = 0; i < desc.length; i++) {
                this._registerLanguage(desc[i]);
                addedModes.push(desc[i].id);
            }
            this._onDidAddModes.fire(addedModes);
        };
        LanguagesRegistry.prototype._registerLanguage = function (lang) {
            this.knownModeIds[lang.id] = true;
            var primaryMime = null;
            if (typeof lang.mimetypes !== 'undefined' && Array.isArray(lang.mimetypes)) {
                for (var i = 0; i < lang.mimetypes.length; i++) {
                    if (!primaryMime) {
                        primaryMime = lang.mimetypes[i];
                    }
                    this.mime2LanguageId[lang.mimetypes[i]] = lang.id;
                }
            }
            if (!primaryMime) {
                primaryMime = 'text/x-' + lang.id;
                this.mime2LanguageId[primaryMime] = lang.id;
            }
            if (Array.isArray(lang.extensions)) {
                for (var _i = 0, _a = lang.extensions; _i < _a.length; _i++) {
                    var extension = _a[_i];
                    mime.registerTextMime({ mime: primaryMime, extension: extension });
                }
            }
            if (Array.isArray(lang.filenames)) {
                for (var _b = 0, _c = lang.filenames; _b < _c.length; _b++) {
                    var filename = _c[_b];
                    mime.registerTextMime({ mime: primaryMime, filename: filename });
                }
            }
            if (Array.isArray(lang.filenamePatterns)) {
                for (var _d = 0, _e = lang.filenamePatterns; _d < _e.length; _d++) {
                    var filenamePattern = _e[_d];
                    mime.registerTextMime({ mime: primaryMime, filepattern: filenamePattern });
                }
            }
            if (typeof lang.firstLine === 'string' && lang.firstLine.length > 0) {
                var firstLineRegexStr = lang.firstLine;
                if (firstLineRegexStr.charAt(0) !== '^') {
                    firstLineRegexStr = '^' + firstLineRegexStr;
                }
                try {
                    var firstLineRegex = new RegExp(firstLineRegexStr);
                    if (!strings.regExpLeadsToEndlessLoop(firstLineRegex)) {
                        mime.registerTextMime({ mime: primaryMime, firstline: firstLineRegex });
                    }
                }
                catch (err) {
                    // Most likely, the regex was bad
                    errors_1.onUnexpectedError(err);
                }
            }
            this.lowerName2Id[lang.id.toLowerCase()] = lang.id;
            if (typeof lang.aliases !== 'undefined' && Array.isArray(lang.aliases)) {
                for (var i = 0; i < lang.aliases.length; i++) {
                    if (!lang.aliases[i] || lang.aliases[i].length === 0) {
                        continue;
                    }
                    this.lowerName2Id[lang.aliases[i].toLowerCase()] = lang.id;
                }
            }
            if (!this.id2Name[lang.id]) {
                var bestName = null;
                if (typeof lang.aliases !== 'undefined' && Array.isArray(lang.aliases) && lang.aliases.length > 0) {
                    bestName = lang.aliases[0];
                }
                else {
                    bestName = lang.id;
                }
                if (bestName) {
                    this.name2LanguageId[bestName] = lang.id;
                    this.name2Extensions[bestName] = lang.extensions;
                    this.id2Name[lang.id] = bestName || '';
                }
            }
            if (typeof lang.configuration === 'string') {
                this.id2ConfigurationFiles[lang.id] = this.id2ConfigurationFiles[lang.id] || [];
                this.id2ConfigurationFiles[lang.id].push(lang.configuration);
            }
        };
        LanguagesRegistry.prototype.isRegisteredMode = function (mimetypeOrModeId) {
            // Is this a known mime type ?
            if (hasOwnProperty.call(this.mime2LanguageId, mimetypeOrModeId)) {
                return true;
            }
            // Is this a known mode id ?
            return hasOwnProperty.call(this.knownModeIds, mimetypeOrModeId);
        };
        LanguagesRegistry.prototype.getRegisteredModes = function () {
            return Object.keys(this.knownModeIds);
        };
        LanguagesRegistry.prototype.getRegisteredLanguageNames = function () {
            return Object.keys(this.name2LanguageId);
        };
        LanguagesRegistry.prototype.getLanguageName = function (modeId) {
            return this.id2Name[modeId] || null;
        };
        LanguagesRegistry.prototype.getModeIdForLanguageNameLowercase = function (languageNameLower) {
            return this.lowerName2Id[languageNameLower] || null;
        };
        LanguagesRegistry.prototype.getConfigurationFiles = function (modeId) {
            return this.id2ConfigurationFiles[modeId] || [];
        };
        LanguagesRegistry.prototype.getMimeForMode = function (theModeId) {
            for (var _mime in this.mime2LanguageId) {
                if (this.mime2LanguageId.hasOwnProperty(_mime)) {
                    var modeId = this.mime2LanguageId[_mime];
                    if (modeId === theModeId) {
                        return _mime;
                    }
                }
            }
            return null;
        };
        LanguagesRegistry.prototype.extractModeIds = function (commaSeparatedMimetypesOrCommaSeparatedIdsOrName) {
            var _this = this;
            if (!commaSeparatedMimetypesOrCommaSeparatedIdsOrName) {
                return [];
            }
            return (commaSeparatedMimetypesOrCommaSeparatedIdsOrName.
                split(',').
                map(function (mimeTypeOrIdOrName) { return mimeTypeOrIdOrName.trim(); }).
                map(function (mimeTypeOrIdOrName) {
                return _this.mime2LanguageId[mimeTypeOrIdOrName] || mimeTypeOrIdOrName;
            }).
                filter(function (modeId) {
                return _this.knownModeIds[modeId];
            }));
        };
        LanguagesRegistry.prototype.getModeIdsFromLanguageName = function (languageName) {
            if (!languageName) {
                return [];
            }
            if (hasOwnProperty.call(this.name2LanguageId, languageName)) {
                return [this.name2LanguageId[languageName]];
            }
            return [];
        };
        LanguagesRegistry.prototype.getModeIdsFromFilenameOrFirstLine = function (filename, firstLine) {
            if (!filename && !firstLine) {
                return [];
            }
            var mimeTypes = mime.guessMimeTypes(filename, firstLine);
            return this.extractModeIds(mimeTypes.join(','));
        };
        LanguagesRegistry.prototype.getCompatMode = function (modeId) {
            return this.compatModes[modeId] || null;
        };
        LanguagesRegistry.prototype.getExtensions = function (languageName) {
            return this.name2Extensions[languageName];
        };
        return LanguagesRegistry;
    }());
    exports.LanguagesRegistry = LanguagesRegistry;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/iPadShowKeyboard/browser/iPadShowKeyboard", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/browser/browser', 'vs/base/browser/dom', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/editorBrowserExtensions', 'vs/css!./iPadShowKeyboard'], function (require, exports, lifecycle_1, browser, dom, editorCommon_1, editorBrowser_1, editorBrowserExtensions_1) {
    'use strict';
    var IPadShowKeyboard = (function () {
        function IPadShowKeyboard(editor) {
            var _this = this;
            this.editor = editor;
            this.toDispose = [];
            if (browser.isIPad) {
                this.toDispose.push(editor.addListener2(editorCommon_1.EventType.ConfigurationChanged, function () { return _this.update(); }));
                this.update();
            }
        }
        IPadShowKeyboard.prototype.update = function () {
            var hasWidget = (!!this.widget);
            var shouldHaveWidget = (!this.editor.getConfiguration().readOnly);
            if (!hasWidget && shouldHaveWidget) {
                this.widget = new ShowKeyboardWidget(this.editor);
            }
            else if (hasWidget && !shouldHaveWidget) {
                this.widget.dispose();
                this.widget = null;
            }
        };
        IPadShowKeyboard.prototype.getId = function () {
            return IPadShowKeyboard.ID;
        };
        IPadShowKeyboard.prototype.dispose = function () {
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            if (this.widget) {
                this.widget.dispose();
                this.widget = null;
            }
        };
        IPadShowKeyboard.ID = 'editor.contrib.iPadShowKeyboard';
        return IPadShowKeyboard;
    }());
    exports.IPadShowKeyboard = IPadShowKeyboard;
    var ShowKeyboardWidget = (function () {
        function ShowKeyboardWidget(editor) {
            var _this = this;
            this.editor = editor;
            this._domNode = document.createElement('textarea');
            this._domNode.className = 'iPadShowKeyboard';
            this._toDispose = [];
            this._toDispose.push(dom.addDisposableListener(this._domNode, 'touchstart', function (e) {
                _this.editor.focus();
            }));
            this._toDispose.push(dom.addDisposableListener(this._domNode, 'focus', function (e) {
                _this.editor.focus();
            }));
            this.editor.addOverlayWidget(this);
        }
        ShowKeyboardWidget.prototype.dispose = function () {
            this.editor.removeOverlayWidget(this);
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        // ----- IOverlayWidget API
        ShowKeyboardWidget.prototype.getId = function () {
            return ShowKeyboardWidget.ID;
        };
        ShowKeyboardWidget.prototype.getDomNode = function () {
            return this._domNode;
        };
        ShowKeyboardWidget.prototype.getPosition = function () {
            return {
                preference: editorBrowser_1.OverlayWidgetPositionPreference.BOTTOM_RIGHT_CORNER
            };
        };
        ShowKeyboardWidget.ID = 'editor.contrib.ShowKeyboardWidget';
        return ShowKeyboardWidget;
    }());
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(IPadShowKeyboard);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/outlineMarker/browser/outlineMarker", ["require", "exports", 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/lifecycle', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowserExtensions', 'vs/css!./outlineMarker'], function (require, exports, async_1, errors_1, lifecycle_1, range_1, editorCommon, editorBrowserExtensions_1) {
    'use strict';
    var OutlineViewZone = (function () {
        function OutlineViewZone(range, outlineType) {
            this.afterLineNumber = range.startLineNumber - 1;
            this.heightInPx = 4;
            this.suppressMouseDown = true;
            this.domNode = document.createElement('div');
            var hr = document.createElement('hr');
            hr.className = 'outlineRule ' + outlineType;
            this.domNode.appendChild(hr);
        }
        return OutlineViewZone;
    }());
    var OutlineMarkerHelper = (function () {
        function OutlineMarkerHelper() {
            this._removeDecorations = [];
            this._addDecorations = [];
            this._addDecorationsCallbacks = [];
        }
        OutlineMarkerHelper.prototype.addDecoration = function (decoration, callback) {
            this._addDecorations.push(decoration);
            this._addDecorationsCallbacks.push(callback);
        };
        OutlineMarkerHelper.prototype.removeDecoration = function (decorationId) {
            this._removeDecorations.push(decorationId);
        };
        OutlineMarkerHelper.prototype.commit = function (changeAccessor) {
            var resultingDecorations = changeAccessor.deltaDecorations(this._removeDecorations, this._addDecorations);
            for (var i = 0, len = resultingDecorations.length; i < len; i++) {
                this._addDecorationsCallbacks[i](resultingDecorations[i]);
            }
        };
        return OutlineMarkerHelper;
    }());
    var OutlineMarker = (function () {
        function OutlineMarker(range, outlineType, _editor, helper, viewZoneChangeAccessor) {
            var _this = this;
            this._editor = _editor;
            this._viewZone = new OutlineViewZone(range, outlineType);
            this._viewZoneId = viewZoneChangeAccessor.addZone(this._viewZone);
            helper.addDecoration({
                range: range,
                options: {}
            }, function (decorationId) {
                _this._decorationId = decorationId;
            });
        }
        OutlineMarker.prototype.dispose = function (helper, viewZoneChangeAccessor) {
            helper.removeDecoration(this._decorationId);
            viewZoneChangeAccessor.removeZone(this._viewZoneId);
        };
        OutlineMarker.prototype.getLine = function () {
            return this._viewZone.afterLineNumber;
        };
        OutlineMarker.prototype.update = function (viewZoneChangeAccessor) {
            var range = this._editor.getModel().getDecorationRange(this._decorationId);
            this._viewZone.afterLineNumber = range.startLineNumber - 1;
            viewZoneChangeAccessor.layoutZone(this._viewZoneId);
        };
        return OutlineMarker;
    }());
    var OutlineMarkerContribution = (function () {
        function OutlineMarkerContribution(editor) {
            var _this = this;
            this._editor = editor;
            this._globalToDispose = [];
            this._localToDispose = [];
            this._markers = [];
            this._currentOutlinePromise = null;
            this._globalToDispose.push(this._editor.addListener2(editorCommon.EventType.ModelChanged, function () { return _this.onChange(true); }));
            this._globalToDispose.push(this._editor.addListener2(editorCommon.EventType.ModelModeChanged, function () { return _this.onChange(false); }));
            this._globalToDispose.push(this._editor.addListener2(editorCommon.EventType.ModelModeSupportChanged, function (e) {
                if (e.outlineSupport) {
                    _this.onChange(false);
                }
            }));
            this._globalToDispose.push(this._editor.addListener2(editorCommon.EventType.ConfigurationChanged, function (e) {
                if (e.outlineMarkers) {
                    _this.onChange(false);
                }
            }));
            this.onChange(false);
        }
        OutlineMarkerContribution.prototype.dispose = function () {
            this.localDispose();
            this._globalToDispose = lifecycle_1.disposeAll(this._globalToDispose);
        };
        OutlineMarkerContribution.prototype.localDispose = function () {
            if (this._currentOutlinePromise) {
                this._currentOutlinePromise.cancel();
            }
            this._localToDispose = lifecycle_1.disposeAll(this._localToDispose);
        };
        OutlineMarkerContribution.prototype.getId = function () {
            return OutlineMarkerContribution.ID;
        };
        OutlineMarkerContribution.prototype.onChange = function (markersAlreadyDisposed) {
            var _this = this;
            if (markersAlreadyDisposed) {
                this._markers = [];
            }
            this.localDispose();
            if (!this._editor.getConfiguration().outlineMarkers) {
                return;
            }
            var model = this._editor.getModel();
            if (!model) {
                return;
            }
            var mode = model.getMode();
            if (!mode.outlineSupport) {
                return;
            }
            var scheduler = new async_1.RunOnceScheduler(function () {
                if (_this._currentOutlinePromise) {
                    _this._currentOutlinePromise.cancel();
                }
                _this._currentOutlinePromise = mode.outlineSupport.getOutline(model.getAssociatedResource());
                _this._currentOutlinePromise.then(function (result) {
                    _this.renderOutlines(result);
                }, function (error) {
                    errors_1.onUnexpectedError(error);
                });
            }, 250);
            this._localToDispose.push(scheduler);
            this._localToDispose.push(this._editor.addListener2('change', function () {
                // Synchronously move markers
                _this._editor.changeViewZones(function (viewAccessor) {
                    _this._markers.forEach(function (marker) {
                        marker.update(viewAccessor);
                    });
                });
                scheduler.schedule();
            }));
            this._localToDispose.push({
                dispose: function () {
                    if (_this._markers.length > 0) {
                        var helper = new OutlineMarkerHelper();
                        _this._editor.changeViewZones(function (accessor) {
                            _this._markers.forEach(function (marker) { return marker.dispose(helper, accessor); });
                            _this._markers = [];
                        });
                        _this._editor.changeDecorations(function (accessor) {
                            helper.commit(accessor);
                        });
                    }
                }
            });
            scheduler.schedule();
        };
        OutlineMarkerContribution.prototype.renderOutlines = function (entries) {
            var _this = this;
            var centeredRange = this._editor.getCenteredRangeInViewport();
            var oldMarkersCount = this._markers.length;
            this._editor.changeDecorations(function (decorationsAccessor) {
                var helper = new OutlineMarkerHelper();
                _this._editor.changeViewZones(function (viewzonesAccessor) {
                    _this._markers.forEach(function (marker) { return marker.dispose(helper, viewzonesAccessor); });
                    _this._markers = [];
                    _this.renderOutlinesRecursive(entries, helper, viewzonesAccessor);
                });
                helper.commit(decorationsAccessor);
            });
            var newMarkersCount = this._markers.length;
            if (Math.abs(oldMarkersCount - newMarkersCount) > 1) {
                // Reveal only if the delta is more than 1 marker
                this._editor.revealRangeInCenter(centeredRange);
            }
        };
        OutlineMarkerContribution.prototype.renderOutlinesRecursive = function (entries, helper, viewZoneChangeAccessor) {
            var _this = this;
            if (entries) {
                entries.forEach(function (outline) {
                    if (outline.type === 'class' || outline.type === 'method' || outline.type === 'function') {
                        var range = range_1.Range.lift(outline.range);
                        if (!_this.alreadyHasMarkerAtRange(range)) {
                            var marker = new OutlineMarker(range, outline.type, _this._editor, helper, viewZoneChangeAccessor);
                            _this._markers.push(marker);
                        }
                    }
                    _this.renderOutlinesRecursive(outline.children, helper, viewZoneChangeAccessor);
                });
            }
        };
        OutlineMarkerContribution.prototype.alreadyHasMarkerAtRange = function (range) {
            for (var i = 0; i < this._markers.length; ++i) {
                if (this._markers[i].getLine() === range.startLineNumber - 1) {
                    return true;
                }
            }
            return false;
        };
        OutlineMarkerContribution.ID = 'editor.outlineMarker';
        return OutlineMarkerContribution;
    }());
    exports.OutlineMarkerContribution = OutlineMarkerContribution;
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(OutlineMarkerContribution);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/selectionClipboard/electron-browser/selectionClipboard", ["require", "exports", 'electron', 'vs/base/common/platform', 'vs/base/common/lifecycle', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowserExtensions', 'vs/platform/keybinding/common/keybindingService', 'vs/base/common/async', 'vs/editor/common/core/range'], function (require, exports, electron_1, platform, lifecycle_1, editorCommon_1, editorBrowserExtensions_1, keybindingService_1, async_1, range_1) {
    'use strict';
    var SelectionClipboard = (function (_super) {
        __extends(SelectionClipboard, _super);
        function SelectionClipboard(editor, keybindingService) {
            _super.call(this);
            if (platform.isLinux) {
                var isEnabled = editor.getConfiguration().selectionClipboard;
                this._register(editor.addListener2(editorCommon_1.EventType.ConfigurationChanged, function (e) {
                    if (e.selectionClipboard) {
                        isEnabled = editor.getConfiguration().selectionClipboard;
                    }
                }));
                this._register(editor.addListener2(editorCommon_1.EventType.MouseDown, function (e) {
                    if (!isEnabled) {
                        return;
                    }
                    if (!editor.getModel()) {
                        return;
                    }
                    if (e.event.middleButton) {
                        e.event.preventDefault();
                        editor.focus();
                        if (e.target.position) {
                            editor.setPosition(e.target.position);
                        }
                        process.nextTick(function () {
                            // TODO@Alex: electron weirdness: calling clipboard.readText('selection') generates a paste event, so no need to execute paste ourselves
                            electron_1.clipboard.readText('selection');
                            // keybindingService.executeCommand(Handler.Paste, {
                            // 	text: clipboard.readText('selection'),
                            // 	pasteOnNewLine: false
                            // });
                        });
                    }
                }));
                var setSelectionToClipboard_1 = this._register(new async_1.RunOnceScheduler(function () {
                    var model = editor.getModel();
                    if (!model) {
                        return;
                    }
                    var selections = editor.getSelections();
                    selections = selections.slice(0);
                    selections.sort(range_1.Range.compareRangesUsingStarts);
                    var result = [];
                    for (var i = 0; i < selections.length; i++) {
                        var sel = selections[i];
                        if (sel.isEmpty()) {
                            // Only write if all cursors have selection
                            return;
                        }
                        result.push(model.getValueInRange(sel, editorCommon_1.EndOfLinePreference.TextDefined));
                    }
                    var textToCopy = result.join(model.getEOL());
                    electron_1.clipboard.writeText(textToCopy, 'selection');
                }, 100));
                this._register(editor.addListener2(editorCommon_1.EventType.CursorSelectionChanged, function (e) {
                    if (!isEnabled) {
                        return;
                    }
                    setSelectionToClipboard_1.schedule();
                }));
            }
        }
        SelectionClipboard.prototype.getId = function () {
            return SelectionClipboard.ID;
        };
        SelectionClipboard.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
        };
        SelectionClipboard.ID = 'editor.contrib.selectionClipboard';
        SelectionClipboard = __decorate([
            __param(1, keybindingService_1.IKeybindingService)
        ], SelectionClipboard);
        return SelectionClipboard;
    }(lifecycle_1.Disposable));
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(SelectionClipboard);
});

define("vs/languages/handlebars/common/handlebars.contribution", ["require", "exports", 'vs/editor/common/modes/modesRegistry'], function (require, exports, modesRegistry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    modesRegistry_1.ModesRegistry.registerCompatMode({
        id: 'handlebars',
        extensions: ['.handlebars', '.hbs'],
        aliases: ['Handlebars', 'handlebars'],
        mimetypes: ['text/x-handlebars-template'],
        moduleId: 'vs/languages/handlebars/common/handlebars',
        ctorName: 'HandlebarsMode'
    });
});

define("vs/languages/php/common/php.contribution", ["require", "exports", 'vs/editor/common/modes/modesRegistry'], function (require, exports, modesRegistry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    modesRegistry_1.ModesRegistry.registerCompatMode({
        id: 'php',
        extensions: ['.php', '.php4', '.php5', '.phtml', '.ctp'],
        aliases: ['PHP', 'php'],
        mimetypes: ['application/x-php'],
        moduleId: 'vs/languages/php/common/php',
        ctorName: 'PHPMode'
    });
});

define("vs/languages/razor/common/razor.contribution", ["require", "exports", 'vs/editor/common/modes/modesRegistry'], function (require, exports, modesRegistry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    modesRegistry_1.ModesRegistry.registerCompatMode({
        id: 'razor',
        extensions: ['.cshtml'],
        aliases: ['Razor', 'razor'],
        mimetypes: ['text/x-cshtml'],
        moduleId: 'vs/languages/razor/common/razor',
        ctorName: 'RAZORMode'
    });
});

define("vs/platform/jsonschemas/common/jsonContributionRegistry", ["require", "exports", 'vs/nls!vs/platform/jsonschemas/common/jsonContributionRegistry', 'vs/platform/platform', 'vs/base/common/eventEmitter'], function (require, exports, nls, platform, eventEmitter_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.Extensions = {
        JSONContribution: 'base.contributions.json'
    };
    function normalizeId(id) {
        if (id.length > 0 && id.charAt(id.length - 1) === '#') {
            return id.substring(0, id.length - 1);
        }
        return id;
    }
    var JSONContributionRegistry = (function () {
        function JSONContributionRegistry() {
            this.schemasById = {};
            this.schemaAssociations = {};
            this.eventEmitter = new eventEmitter_1.EventEmitter();
        }
        JSONContributionRegistry.prototype.addRegistryChangedListener = function (callback) {
            return this.eventEmitter.addListener2('registryChanged', callback);
        };
        JSONContributionRegistry.prototype.registerSchema = function (uri, unresolvedSchemaContent) {
            this.schemasById[normalizeId(uri)] = unresolvedSchemaContent;
            this.eventEmitter.emit('registryChanged', {});
        };
        JSONContributionRegistry.prototype.addSchemaFileAssociation = function (pattern, uri) {
            var uris = this.schemaAssociations[pattern];
            if (!uris) {
                uris = [];
                this.schemaAssociations[pattern] = uris;
            }
            uris.push(uri);
            this.eventEmitter.emit('registryChanged', {});
        };
        JSONContributionRegistry.prototype.getSchemaContributions = function () {
            return {
                schemas: this.schemasById,
                schemaAssociations: this.schemaAssociations
            };
        };
        return JSONContributionRegistry;
    }());
    var jsonContributionRegistry = new JSONContributionRegistry();
    platform.Registry.add(exports.Extensions.JSONContribution, jsonContributionRegistry);
    // preload the schema-schema with a version that contains descriptions.
    jsonContributionRegistry.registerSchema('http://json-schema.org/draft-04/schema#', {
        'id': 'http://json-schema.org/draft-04/schema#',
        'title': nls.localize(0, null),
        '$schema': 'http://json-schema.org/draft-04/schema#',
        'definitions': {
            'schemaArray': {
                'type': 'array',
                'minItems': 1,
                'items': { '$ref': '#' }
            },
            'positiveInteger': {
                'type': 'integer',
                'minimum': 0
            },
            'positiveIntegerDefault0': {
                'allOf': [{ '$ref': '#/definitions/positiveInteger' }, { 'default': 0 }]
            },
            'simpleTypes': {
                'type': 'string',
                'enum': ['array', 'boolean', 'integer', 'null', 'number', 'object', 'string']
            },
            'stringArray': {
                'type': 'array',
                'items': { 'type': 'string' },
                'minItems': 1,
                'uniqueItems': true
            }
        },
        'type': 'object',
        'properties': {
            'id': {
                'type': 'string',
                'format': 'uri',
                'description': nls.localize(1, null)
            },
            '$schema': {
                'type': 'string',
                'format': 'uri',
                'description': nls.localize(2, null)
            },
            'title': {
                'type': 'string',
                'description': nls.localize(3, null)
            },
            'description': {
                'type': 'string',
                'description': nls.localize(4, null)
            },
            'default': {
                'description': nls.localize(5, null)
            },
            'multipleOf': {
                'type': 'number',
                'minimum': 0,
                'exclusiveMinimum': true,
                'description': nls.localize(6, null)
            },
            'maximum': {
                'type': 'number',
                'description': nls.localize(7, null)
            },
            'exclusiveMaximum': {
                'type': 'boolean',
                'default': false,
                'description': nls.localize(8, null)
            },
            'minimum': {
                'type': 'number',
                'description': nls.localize(9, null)
            },
            'exclusiveMinimum': {
                'type': 'boolean',
                'default': false,
                'description': nls.localize(10, null)
            },
            'maxLength': {
                'allOf': [
                    { '$ref': '#/definitions/positiveInteger' }
                ],
                'description': nls.localize(11, null)
            },
            'minLength': {
                'allOf': [
                    { '$ref': '#/definitions/positiveIntegerDefault0' }
                ],
                'description': nls.localize(12, null)
            },
            'pattern': {
                'type': 'string',
                'format': 'regex',
                'description': nls.localize(13, null)
            },
            'additionalItems': {
                'anyOf': [
                    { 'type': 'boolean' },
                    { '$ref': '#' }
                ],
                'default': {},
                'description': nls.localize(14, null)
            },
            'items': {
                'anyOf': [
                    { '$ref': '#' },
                    { '$ref': '#/definitions/schemaArray' }
                ],
                'default': {},
                'description': nls.localize(15, null)
            },
            'maxItems': {
                'allOf': [
                    { '$ref': '#/definitions/positiveInteger' }
                ],
                'description': nls.localize(16, null)
            },
            'minItems': {
                'allOf': [
                    { '$ref': '#/definitions/positiveIntegerDefault0' }
                ],
                'description': nls.localize(17, null)
            },
            'uniqueItems': {
                'type': 'boolean',
                'default': false,
                'description': nls.localize(18, null)
            },
            'maxProperties': {
                'allOf': [
                    { '$ref': '#/definitions/positiveInteger' }
                ],
                'description': nls.localize(19, null)
            },
            'minProperties': {
                'allOf': [
                    { '$ref': '#/definitions/positiveIntegerDefault0' },
                ],
                'description': nls.localize(20, null)
            },
            'required': {
                'allOf': [
                    { '$ref': '#/definitions/stringArray' }
                ],
                'description': nls.localize(21, null)
            },
            'additionalProperties': {
                'anyOf': [
                    { 'type': 'boolean' },
                    { '$ref': '#' }
                ],
                'default': {},
                'description': nls.localize(22, null)
            },
            'definitions': {
                'type': 'object',
                'additionalProperties': { '$ref': '#' },
                'default': {},
                'description': nls.localize(23, null)
            },
            'properties': {
                'type': 'object',
                'additionalProperties': { '$ref': '#' },
                'default': {},
                'description': nls.localize(24, null)
            },
            'patternProperties': {
                'type': 'object',
                'additionalProperties': { '$ref': '#' },
                'default': {},
                'description': nls.localize(25, null)
            },
            'dependencies': {
                'type': 'object',
                'additionalProperties': {
                    'anyOf': [
                        { '$ref': '#' },
                        { '$ref': '#/definitions/stringArray' }
                    ]
                },
                'description': nls.localize(26, null)
            },
            'enum': {
                'type': 'array',
                'minItems': 1,
                'uniqueItems': true,
                'description': nls.localize(27, null)
            },
            'type': {
                'anyOf': [
                    { '$ref': '#/definitions/simpleTypes' },
                    {
                        'type': 'array',
                        'items': { '$ref': '#/definitions/simpleTypes' },
                        'minItems': 1,
                        'uniqueItems': true
                    }
                ],
                'description': nls.localize(28, null)
            },
            'format': {
                'anyOf': [
                    {
                        'type': 'string',
                        'description': nls.localize(29, null),
                        'enum': ['date-time', 'uri', 'email', 'hostname', 'ipv4', 'ipv6', 'regex']
                    }, {
                        'type': 'string'
                    }
                ]
            },
            'allOf': {
                'allOf': [
                    { '$ref': '#/definitions/schemaArray' }
                ],
                'description': nls.localize(30, null)
            },
            'anyOf': {
                'allOf': [
                    { '$ref': '#/definitions/schemaArray' }
                ],
                'description': nls.localize(31, null)
            },
            'oneOf': {
                'allOf': [
                    { '$ref': '#/definitions/schemaArray' }
                ],
                'description': nls.localize(32, null)
            },
            'not': {
                'allOf': [
                    { '$ref': '#' }
                ],
                'description': nls.localize(33, null)
            }
        },
        'dependencies': {
            'exclusiveMaximum': ['maximum'],
            'exclusiveMinimum': ['minimum']
        },
        'default': {}
    });
});

define("vs/platform/extensions/common/extensionsRegistry", ["require", "exports", 'vs/nls!vs/platform/extensions/common/extensionsRegistry', 'vs/base/common/errors', 'vs/base/common/paths', 'vs/base/common/severity', 'vs/platform/jsonschemas/common/jsonContributionRegistry', 'vs/platform/platform'], function (require, exports, nls, errors_1, paths, severity_1, jsonContributionRegistry_1, platform_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExtensionMessageCollector = (function () {
        function ExtensionMessageCollector(messageHandler, source) {
            this._messageHandler = messageHandler;
            this._source = source;
        }
        ExtensionMessageCollector.prototype._msg = function (type, message) {
            this._messageHandler({
                type: type,
                message: message,
                source: this._source
            });
        };
        ExtensionMessageCollector.prototype.error = function (message) {
            this._msg(severity_1.default.Error, message);
        };
        ExtensionMessageCollector.prototype.warn = function (message) {
            this._msg(severity_1.default.Warning, message);
        };
        ExtensionMessageCollector.prototype.info = function (message) {
            this._msg(severity_1.default.Info, message);
        };
        return ExtensionMessageCollector;
    }());
    function isValidExtensionDescription(extensionFolderPath, extensionDescription, notices) {
        if (!extensionDescription) {
            notices.push(nls.localize(0, null));
            return false;
        }
        if (typeof extensionDescription.publisher !== 'string') {
            notices.push(nls.localize(1, null, 'publisher'));
            return false;
        }
        if (typeof extensionDescription.name !== 'string') {
            notices.push(nls.localize(2, null, 'name'));
            return false;
        }
        if (typeof extensionDescription.version !== 'string') {
            notices.push(nls.localize(3, null, 'version'));
            return false;
        }
        if (!extensionDescription.engines) {
            notices.push(nls.localize(4, null, 'engines'));
            return false;
        }
        if (typeof extensionDescription.engines.vscode !== 'string') {
            notices.push(nls.localize(5, null, 'engines.vscode'));
            return false;
        }
        if (typeof extensionDescription.extensionDependencies !== 'undefined') {
            if (!_isStringArray(extensionDescription.extensionDependencies)) {
                notices.push(nls.localize(6, null, 'extensionDependencies'));
                return false;
            }
        }
        if (typeof extensionDescription.activationEvents !== 'undefined') {
            if (!_isStringArray(extensionDescription.activationEvents)) {
                notices.push(nls.localize(7, null, 'activationEvents'));
                return false;
            }
            if (typeof extensionDescription.main === 'undefined') {
                notices.push(nls.localize(8, null, 'activationEvents', 'main'));
                return false;
            }
        }
        if (typeof extensionDescription.main !== 'undefined') {
            if (typeof extensionDescription.main !== 'string') {
                notices.push(nls.localize(9, null, 'main'));
                return false;
            }
            else {
                var normalizedAbsolutePath = paths.normalize(paths.join(extensionFolderPath, extensionDescription.main));
                if (normalizedAbsolutePath.indexOf(extensionFolderPath)) {
                    notices.push(nls.localize(10, null, normalizedAbsolutePath, extensionFolderPath));
                }
            }
            if (typeof extensionDescription.activationEvents === 'undefined') {
                notices.push(nls.localize(11, null, 'activationEvents', 'main'));
                return false;
            }
        }
        return true;
    }
    exports.isValidExtensionDescription = isValidExtensionDescription;
    var hasOwnProperty = Object.hasOwnProperty;
    var schemaRegistry = platform_1.Registry.as(jsonContributionRegistry_1.Extensions.JSONContribution);
    var ExtensionPoint = (function () {
        function ExtensionPoint(name, registry) {
            this.name = name;
            this._registry = registry;
            this._handler = null;
            this._messageHandler = null;
        }
        ExtensionPoint.prototype.setHandler = function (handler) {
            if (this._handler) {
                throw new Error('Handler already set!');
            }
            this._handler = handler;
            this._handle();
        };
        ExtensionPoint.prototype.handle = function (messageHandler) {
            this._messageHandler = messageHandler;
            this._handle();
        };
        ExtensionPoint.prototype._handle = function () {
            var _this = this;
            if (!this._handler || !this._messageHandler) {
                return;
            }
            this._registry.registerPointListener(this.name, function (descriptions) {
                var users = descriptions.map(function (desc) {
                    return {
                        description: desc,
                        value: desc.contributes[_this.name],
                        collector: new ExtensionMessageCollector(_this._messageHandler, desc.extensionFolderPath)
                    };
                });
                _this._handler(users);
            });
        };
        return ExtensionPoint;
    }());
    var schemaId = 'vscode://schemas/vscode-extensions';
    var schema = {
        default: {
            'name': '{{name}}',
            'description': '{{description}}',
            'author': '{{author}}',
            'version': '{{1.0.0}}',
            'main': '{{pathToMain}}',
            'dependencies': {}
        },
        properties: {
            // engines: {
            // 	required: [ 'vscode' ],
            // 	properties: {
            // 		'vscode': {
            // 			type: 'string',
            // 			description: nls.localize('vscode.extension.engines.vscode', 'Specifies that this package only runs inside VSCode of the given version.'),
            // 		}
            // 	}
            // },
            displayName: {
                description: nls.localize(12, null),
                type: 'string'
            },
            categories: {
                description: nls.localize(13, null),
                type: 'array',
                items: {
                    type: 'string',
                    enum: ['Languages', 'Snippets', 'Linters', 'Themes', 'Debuggers', 'Other']
                }
            },
            galleryBanner: {
                type: 'object',
                description: nls.localize(14, null),
                properties: {
                    color: {
                        description: nls.localize(15, null),
                        type: 'string'
                    },
                    theme: {
                        description: nls.localize(16, null),
                        type: 'string',
                        enum: ['dark', 'light']
                    }
                }
            },
            publisher: {
                description: nls.localize(17, null),
                type: 'string'
            },
            activationEvents: {
                description: nls.localize(18, null),
                type: 'array',
                items: {
                    type: 'string'
                }
            },
            extensionDependencies: {
                description: nls.localize(19, null),
                type: 'array',
                items: {
                    type: 'string'
                }
            },
            scripts: {
                type: 'object',
                properties: {
                    'vscode:prepublish': {
                        description: nls.localize(20, null),
                        type: 'string'
                    }
                }
            },
            contributes: {
                description: nls.localize(21, null),
                type: 'object',
                properties: {},
                default: {}
            }
        }
    };
    var ExtensionsRegistryImpl = (function () {
        function ExtensionsRegistryImpl() {
            this._extensionsMap = {};
            this._extensionsArr = [];
            this._activationMap = {};
            this._pointListeners = [];
            this._extensionPoints = {};
            this._oneTimeActivationEventListeners = {};
        }
        ExtensionsRegistryImpl.prototype.registerPointListener = function (point, handler) {
            var entry = {
                extensionPoint: point,
                listener: handler
            };
            this._pointListeners.push(entry);
            this._triggerPointListener(entry, ExtensionsRegistryImpl._filterWithExtPoint(this.getAllExtensionDescriptions(), point));
        };
        ExtensionsRegistryImpl.prototype.registerExtensionPoint = function (extensionPoint, jsonSchema) {
            if (hasOwnProperty.call(this._extensionPoints, extensionPoint)) {
                throw new Error('Duplicate extension point: ' + extensionPoint);
            }
            var result = new ExtensionPoint(extensionPoint, this);
            this._extensionPoints[extensionPoint] = result;
            schema.properties['contributes'].properties[extensionPoint] = jsonSchema;
            schemaRegistry.registerSchema(schemaId, schema);
            return result;
        };
        ExtensionsRegistryImpl.prototype.handleExtensionPoints = function (messageHandler) {
            var _this = this;
            Object.keys(this._extensionPoints).forEach(function (extensionPointName) {
                _this._extensionPoints[extensionPointName].handle(messageHandler);
            });
        };
        ExtensionsRegistryImpl.prototype._triggerPointListener = function (handler, desc) {
            // console.log('_triggerPointListeners: ' + desc.length + ' OF ' + handler.extensionPoint);
            if (!desc || desc.length === 0) {
                return;
            }
            try {
                handler.listener(desc);
            }
            catch (e) {
                errors_1.onUnexpectedError(e);
            }
        };
        ExtensionsRegistryImpl.prototype.registerExtensions = function (extensionDescriptions) {
            for (var i = 0, len = extensionDescriptions.length; i < len; i++) {
                var extensionDescription = extensionDescriptions[i];
                if (hasOwnProperty.call(this._extensionsMap, extensionDescription.id)) {
                    // No overwriting allowed!
                    console.error('Extension `' + extensionDescription.id + '` is already registered');
                    continue;
                }
                this._extensionsMap[extensionDescription.id] = extensionDescription;
                this._extensionsArr.push(extensionDescription);
                if (Array.isArray(extensionDescription.activationEvents)) {
                    for (var j = 0, lenJ = extensionDescription.activationEvents.length; j < lenJ; j++) {
                        var activationEvent = extensionDescription.activationEvents[j];
                        this._activationMap[activationEvent] = this._activationMap[activationEvent] || [];
                        this._activationMap[activationEvent].push(extensionDescription);
                    }
                }
            }
            for (var i = 0, len = this._pointListeners.length; i < len; i++) {
                var listenerEntry = this._pointListeners[i];
                var descriptions = ExtensionsRegistryImpl._filterWithExtPoint(extensionDescriptions, listenerEntry.extensionPoint);
                this._triggerPointListener(listenerEntry, descriptions);
            }
        };
        ExtensionsRegistryImpl._filterWithExtPoint = function (input, point) {
            return input.filter(function (desc) {
                return (desc.contributes && hasOwnProperty.call(desc.contributes, point));
            });
        };
        ExtensionsRegistryImpl.prototype.getExtensionDescriptionsForActivationEvent = function (activationEvent) {
            if (!hasOwnProperty.call(this._activationMap, activationEvent)) {
                return [];
            }
            return this._activationMap[activationEvent].slice(0);
        };
        ExtensionsRegistryImpl.prototype.getAllExtensionDescriptions = function () {
            return this._extensionsArr.slice(0);
        };
        ExtensionsRegistryImpl.prototype.getExtensionDescription = function (extensionId) {
            if (!hasOwnProperty.call(this._extensionsMap, extensionId)) {
                return null;
            }
            return this._extensionsMap[extensionId];
        };
        ExtensionsRegistryImpl.prototype.registerOneTimeActivationEventListener = function (activationEvent, listener) {
            if (!hasOwnProperty.call(this._oneTimeActivationEventListeners, activationEvent)) {
                this._oneTimeActivationEventListeners[activationEvent] = [];
            }
            this._oneTimeActivationEventListeners[activationEvent].push(listener);
        };
        ExtensionsRegistryImpl.prototype.triggerActivationEventListeners = function (activationEvent) {
            if (hasOwnProperty.call(this._oneTimeActivationEventListeners, activationEvent)) {
                var listeners = this._oneTimeActivationEventListeners[activationEvent];
                delete this._oneTimeActivationEventListeners[activationEvent];
                for (var i = 0, len = listeners.length; i < len; i++) {
                    var listener = listeners[i];
                    try {
                        listener();
                    }
                    catch (e) {
                        errors_1.onUnexpectedError(e);
                    }
                }
            }
        };
        return ExtensionsRegistryImpl;
    }());
    function _isStringArray(arr) {
        if (!Array.isArray(arr)) {
            return false;
        }
        for (var i = 0, len = arr.length; i < len; i++) {
            if (typeof arr[i] !== 'string') {
                return false;
            }
        }
        return true;
    }
    var PRExtensions = {
        ExtensionsRegistry: 'ExtensionsRegistry'
    };
    platform_1.Registry.add(PRExtensions.ExtensionsRegistry, new ExtensionsRegistryImpl());
    exports.ExtensionsRegistry = platform_1.Registry.as(PRExtensions.ExtensionsRegistry);
    schemaRegistry.registerSchema(schemaId, schema);
    schemaRegistry.addSchemaFileAssociation('/package.json', schemaId);
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/platform/actions/common/actionsService", ["require", "exports", 'vs/nls!vs/platform/actions/common/actionsService', 'vs/base/common/actions', 'vs/platform/extensions/common/extensions', 'vs/platform/extensions/common/extensionsRegistry', 'vs/platform/keybinding/common/keybindingService'], function (require, exports, nls_1, actions_1, extensions_1, extensionsRegistry_1, keybindingService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function isCommands(thing) {
        return Array.isArray(thing);
    }
    function isValidCommand(candidate, rejects) {
        if (!candidate) {
            rejects.push(nls_1.localize(0, null));
            return false;
        }
        if (typeof candidate.command !== 'string') {
            rejects.push(nls_1.localize(1, null, 'command'));
            return false;
        }
        if (typeof candidate.title !== 'string') {
            rejects.push(nls_1.localize(2, null, 'title'));
            return false;
        }
        if (candidate.category && typeof candidate.category !== 'string') {
            rejects.push(nls_1.localize(3, null, 'category'));
            return false;
        }
        return true;
    }
    var commandType = {
        type: 'object',
        properties: {
            command: {
                description: nls_1.localize(4, null),
                type: 'string'
            },
            title: {
                description: nls_1.localize(5, null),
                type: 'string'
            },
            category: {
                description: nls_1.localize(6, null),
                type: 'string'
            }
        }
    };
    var commandsExtPoint = extensionsRegistry_1.ExtensionsRegistry.registerExtensionPoint('commands', {
        description: nls_1.localize(7, null),
        oneOf: [
            commandType,
            {
                type: 'array',
                items: commandType
            }
        ]
    });
    var ActionsService = (function () {
        function ActionsService(extensionService, keybindingsService) {
            var _this = this;
            this._extensionsActions = [];
            this._extensionService = extensionService;
            this._keybindingsService = keybindingsService;
            commandsExtPoint.setHandler(function (extensions) {
                for (var _i = 0, extensions_2 = extensions; _i < extensions_2.length; _i++) {
                    var d = extensions_2[_i];
                    _this._onDescription(d.value, d.collector);
                }
            });
        }
        ActionsService.prototype._onDescription = function (commands, collector) {
            if (isCommands(commands)) {
                for (var _i = 0, commands_1 = commands; _i < commands_1.length; _i++) {
                    var command = commands_1[_i];
                    this._handleCommand(command, collector);
                }
            }
            else {
                this._handleCommand(commands, collector);
            }
        };
        ActionsService.prototype._handleCommand = function (command, collector) {
            var _this = this;
            var rejects = [];
            if (isValidCommand(command, rejects)) {
                // make sure this extension is activated by this command
                var activationEvent_1 = "onCommand:" + command.command;
                // action that (1) activates the extension and dispatches the command
                var label = command.category ? nls_1.localize(8, null, command.category, command.title) : command.title;
                var action = new actions_1.Action(command.command, label, undefined, true, function () {
                    return _this._extensionService.activateByEvent(activationEvent_1).then(function () {
                        return _this._keybindingsService.executeCommand(command.command);
                    });
                });
                this._extensionsActions.push(action);
            }
            if (rejects.length > 0) {
                collector.error(nls_1.localize(9, null, commandsExtPoint.name, rejects.join('\n')));
            }
        };
        ActionsService.prototype.getActions = function () {
            return this._extensionsActions.slice(0);
        };
        ActionsService = __decorate([
            __param(0, extensions_1.IExtensionService),
            __param(1, keybindingService_1.IKeybindingService)
        ], ActionsService);
        return ActionsService;
    }());
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = ActionsService;
});

define("vs/platform/configuration/common/configurationRegistry", ["require", "exports", 'vs/nls!vs/platform/configuration/common/configurationRegistry', 'vs/base/common/event', 'vs/platform/platform', 'vs/base/common/objects', 'vs/platform/extensions/common/extensionsRegistry', 'vs/platform/jsonschemas/common/jsonContributionRegistry'], function (require, exports, nls, event_1, platform, objects, extensionsRegistry_1, JSONContributionRegistry) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.Extensions = {
        Configuration: 'base.contributions.configuration'
    };
    var schemaId = 'vscode://schemas/settings';
    var contributionRegistry = platform.Registry.as(JSONContributionRegistry.Extensions.JSONContribution);
    var ConfigurationRegistry = (function () {
        function ConfigurationRegistry() {
            this.configurationContributors = [];
            this.configurationSchema = { allOf: [] };
            this._onDidRegisterConfiguration = new event_1.Emitter();
            contributionRegistry.registerSchema(schemaId, this.configurationSchema);
            contributionRegistry.addSchemaFileAssociation('vscode://defaultsettings/settings.json', schemaId);
            contributionRegistry.addSchemaFileAssociation('%APP_SETTINGS_HOME%/settings.json', schemaId);
            contributionRegistry.addSchemaFileAssociation('/.vscode/settings.json', schemaId);
        }
        Object.defineProperty(ConfigurationRegistry.prototype, "onDidRegisterConfiguration", {
            get: function () {
                return this._onDidRegisterConfiguration.event;
            },
            enumerable: true,
            configurable: true
        });
        ConfigurationRegistry.prototype.registerConfiguration = function (configuration) {
            this.configurationContributors.push(configuration);
            this.registerJSONConfiguration(configuration);
            this._onDidRegisterConfiguration.fire(this);
        };
        ConfigurationRegistry.prototype.getConfigurations = function () {
            return this.configurationContributors.slice(0);
        };
        ConfigurationRegistry.prototype.registerJSONConfiguration = function (configuration) {
            var schema = objects.clone(configuration);
            this.configurationSchema.allOf.push(schema);
            contributionRegistry.registerSchema(schemaId, this.configurationSchema);
        };
        return ConfigurationRegistry;
    }());
    var configurationRegistry = new ConfigurationRegistry();
    platform.Registry.add(exports.Extensions.Configuration, configurationRegistry);
    var configurationExtPoint = extensionsRegistry_1.ExtensionsRegistry.registerExtensionPoint('configuration', {
        description: nls.localize(0, null),
        type: 'object',
        defaultSnippets: [{ body: { title: '', properties: {} } }],
        properties: {
            title: {
                description: nls.localize(1, null),
                type: 'string'
            },
            properties: {
                description: nls.localize(2, null),
                type: 'object',
                additionalProperties: {
                    $ref: 'http://json-schema.org/draft-04/schema#'
                }
            }
        }
    });
    configurationExtPoint.setHandler(function (extensions) {
        for (var i = 0; i < extensions.length; i++) {
            var configuration = extensions[i].value;
            var collector = extensions[i].collector;
            if (configuration.type && configuration.type !== 'object') {
                collector.warn(nls.localize(3, null));
            }
            else {
                configuration.type = 'object';
            }
            if (configuration.title && (typeof configuration.title !== 'string')) {
                collector.error(nls.localize(4, null));
            }
            if (configuration.properties && (typeof configuration.properties !== 'object')) {
                collector.error(nls.localize(5, null));
                return;
            }
            var clonedConfiguration = objects.clone(configuration);
            clonedConfiguration.id = extensions[i].description.id;
            configurationRegistry.registerConfiguration(clonedConfiguration);
        }
    });
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/common/config/commonEditorConfig", ["require", "exports", 'vs/nls!vs/editor/common/config/commonEditorConfig', 'vs/base/common/event', 'vs/base/common/lifecycle', 'vs/base/common/objects', 'vs/base/common/platform', 'vs/platform/configuration/common/configurationRegistry', 'vs/platform/platform', 'vs/editor/common/config/defaultConfig', 'vs/editor/common/controller/handlerDispatcher', 'vs/editor/common/editorCommon', 'vs/editor/common/viewLayout/editorLayoutProvider'], function (require, exports, nls, event_1, lifecycle_1, objects, platform, configurationRegistry_1, platform_1, defaultConfig_1, handlerDispatcher_1, editorCommon, editorLayoutProvider_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Experimental screen reader support toggle
     */
    var GlobalScreenReaderNVDA = (function () {
        function GlobalScreenReaderNVDA() {
        }
        GlobalScreenReaderNVDA.getValue = function () {
            return this._value;
        };
        GlobalScreenReaderNVDA.setValue = function (value) {
            if (this._value === value) {
                return;
            }
            this._value = value;
            this._onChange.fire(this._value);
        };
        GlobalScreenReaderNVDA._value = false;
        GlobalScreenReaderNVDA._onChange = new event_1.Emitter();
        GlobalScreenReaderNVDA.onChange = GlobalScreenReaderNVDA._onChange.event;
        return GlobalScreenReaderNVDA;
    }());
    exports.GlobalScreenReaderNVDA = GlobalScreenReaderNVDA;
    var ConfigurationWithDefaults = (function () {
        function ConfigurationWithDefaults(options) {
            this._editor = objects.clone(defaultConfig_1.DefaultConfig.editor);
            this._mergeOptionsIn(options);
        }
        ConfigurationWithDefaults.prototype.getEditorOptions = function () {
            return this._editor;
        };
        ConfigurationWithDefaults.prototype._mergeOptionsIn = function (newOptions) {
            this._editor = objects.mixin(this._editor, newOptions || {});
        };
        ConfigurationWithDefaults.prototype.updateOptions = function (newOptions) {
            // Apply new options
            this._mergeOptionsIn(newOptions);
        };
        return ConfigurationWithDefaults;
    }());
    exports.ConfigurationWithDefaults = ConfigurationWithDefaults;
    function cloneInternalEditorOptions(opts) {
        return {
            experimentalScreenReader: opts.experimentalScreenReader,
            rulers: opts.rulers.slice(0),
            wordSeparators: opts.wordSeparators,
            selectionClipboard: opts.selectionClipboard,
            ariaLabel: opts.ariaLabel,
            lineNumbers: opts.lineNumbers,
            selectOnLineNumbers: opts.selectOnLineNumbers,
            glyphMargin: opts.glyphMargin,
            revealHorizontalRightPadding: opts.revealHorizontalRightPadding,
            roundedSelection: opts.roundedSelection,
            theme: opts.theme,
            readOnly: opts.readOnly,
            scrollbar: {
                arrowSize: opts.scrollbar.arrowSize,
                vertical: opts.scrollbar.vertical,
                horizontal: opts.scrollbar.horizontal,
                useShadows: opts.scrollbar.useShadows,
                verticalHasArrows: opts.scrollbar.verticalHasArrows,
                horizontalHasArrows: opts.scrollbar.horizontalHasArrows,
                handleMouseWheel: opts.scrollbar.handleMouseWheel,
                horizontalScrollbarSize: opts.scrollbar.horizontalScrollbarSize,
                horizontalSliderSize: opts.scrollbar.horizontalSliderSize,
                verticalScrollbarSize: opts.scrollbar.verticalScrollbarSize,
                verticalSliderSize: opts.scrollbar.verticalSliderSize,
                mouseWheelScrollSensitivity: opts.scrollbar.mouseWheelScrollSensitivity,
            },
            overviewRulerLanes: opts.overviewRulerLanes,
            cursorBlinking: opts.cursorBlinking,
            cursorStyle: opts.cursorStyle,
            fontLigatures: opts.fontLigatures,
            hideCursorInOverviewRuler: opts.hideCursorInOverviewRuler,
            scrollBeyondLastLine: opts.scrollBeyondLastLine,
            wrappingIndent: opts.wrappingIndent,
            wordWrapBreakBeforeCharacters: opts.wordWrapBreakBeforeCharacters,
            wordWrapBreakAfterCharacters: opts.wordWrapBreakAfterCharacters,
            wordWrapBreakObtrusiveCharacters: opts.wordWrapBreakObtrusiveCharacters,
            tabFocusMode: opts.tabFocusMode,
            stopLineTokenizationAfter: opts.stopLineTokenizationAfter,
            stopRenderingLineAfter: opts.stopRenderingLineAfter,
            longLineBoundary: opts.longLineBoundary,
            forcedTokenizationBoundary: opts.forcedTokenizationBoundary,
            hover: opts.hover,
            contextmenu: opts.contextmenu,
            quickSuggestions: opts.quickSuggestions,
            quickSuggestionsDelay: opts.quickSuggestionsDelay,
            iconsInSuggestions: opts.iconsInSuggestions,
            autoClosingBrackets: opts.autoClosingBrackets,
            formatOnType: opts.formatOnType,
            suggestOnTriggerCharacters: opts.suggestOnTriggerCharacters,
            acceptSuggestionOnEnter: opts.acceptSuggestionOnEnter,
            selectionHighlight: opts.selectionHighlight,
            outlineMarkers: opts.outlineMarkers,
            referenceInfos: opts.referenceInfos,
            folding: opts.folding,
            renderWhitespace: opts.renderWhitespace,
            layoutInfo: {
                width: opts.layoutInfo.width,
                height: opts.layoutInfo.height,
                glyphMarginLeft: opts.layoutInfo.glyphMarginLeft,
                glyphMarginWidth: opts.layoutInfo.glyphMarginWidth,
                glyphMarginHeight: opts.layoutInfo.glyphMarginHeight,
                lineNumbersLeft: opts.layoutInfo.lineNumbersLeft,
                lineNumbersWidth: opts.layoutInfo.lineNumbersWidth,
                lineNumbersHeight: opts.layoutInfo.lineNumbersHeight,
                decorationsLeft: opts.layoutInfo.decorationsLeft,
                decorationsWidth: opts.layoutInfo.decorationsWidth,
                decorationsHeight: opts.layoutInfo.decorationsHeight,
                contentLeft: opts.layoutInfo.contentLeft,
                contentWidth: opts.layoutInfo.contentWidth,
                contentHeight: opts.layoutInfo.contentHeight,
                verticalScrollbarWidth: opts.layoutInfo.verticalScrollbarWidth,
                horizontalScrollbarHeight: opts.layoutInfo.horizontalScrollbarHeight,
                overviewRuler: {
                    width: opts.layoutInfo.overviewRuler.width,
                    height: opts.layoutInfo.overviewRuler.height,
                    top: opts.layoutInfo.overviewRuler.top,
                    right: opts.layoutInfo.overviewRuler.right,
                }
            },
            stylingInfo: {
                editorClassName: opts.stylingInfo.editorClassName,
                fontFamily: opts.stylingInfo.fontFamily,
                fontSize: opts.stylingInfo.fontSize,
                lineHeight: opts.stylingInfo.lineHeight,
            },
            wrappingInfo: {
                isViewportWrapping: opts.wrappingInfo.isViewportWrapping,
                wrappingColumn: opts.wrappingInfo.wrappingColumn,
            },
            observedOuterWidth: opts.observedOuterWidth,
            observedOuterHeight: opts.observedOuterHeight,
            lineHeight: opts.lineHeight,
            pageSize: opts.pageSize,
            typicalHalfwidthCharacterWidth: opts.typicalHalfwidthCharacterWidth,
            typicalFullwidthCharacterWidth: opts.typicalFullwidthCharacterWidth,
            fontSize: opts.fontSize,
        };
    }
    var InternalEditorOptionsHelper = (function () {
        function InternalEditorOptionsHelper() {
        }
        InternalEditorOptionsHelper.createInternalEditorOptions = function (outerWidth, outerHeight, opts, editorClassName, requestedFontFamily, requestedFontSize, requestedLineHeight, adjustedLineHeight, themeOpts, isDominatedByLongLines, lineCount) {
            var wrappingColumn = toInteger(opts.wrappingColumn, -1);
            var stopLineTokenizationAfter;
            if (typeof opts.stopLineTokenizationAfter !== 'undefined') {
                stopLineTokenizationAfter = toInteger(opts.stopLineTokenizationAfter, -1);
            }
            else if (wrappingColumn >= 0) {
                stopLineTokenizationAfter = -1;
            }
            else {
                stopLineTokenizationAfter = 10000;
            }
            var stopRenderingLineAfter;
            if (typeof opts.stopRenderingLineAfter !== 'undefined') {
                stopRenderingLineAfter = toInteger(opts.stopRenderingLineAfter, -1);
            }
            else if (wrappingColumn >= 0) {
                stopRenderingLineAfter = -1;
            }
            else {
                stopRenderingLineAfter = 10000;
            }
            var mouseWheelScrollSensitivity = toFloat(opts.mouseWheelScrollSensitivity, 1);
            var scrollbar = this._sanitizeScrollbarOpts(opts.scrollbar, mouseWheelScrollSensitivity);
            var glyphMargin = toBoolean(opts.glyphMargin);
            var lineNumbers = opts.lineNumbers;
            var lineNumbersMinChars = toInteger(opts.lineNumbersMinChars, 1);
            var lineDecorationsWidth = toInteger(opts.lineDecorationsWidth, 0);
            if (opts.folding) {
                lineDecorationsWidth += 16;
            }
            var layoutInfo = editorLayoutProvider_1.EditorLayoutProvider.compute({
                outerWidth: outerWidth,
                outerHeight: outerHeight,
                showGlyphMargin: glyphMargin,
                lineHeight: themeOpts.lineHeight,
                showLineNumbers: !!lineNumbers,
                lineNumbersMinChars: lineNumbersMinChars,
                lineDecorationsWidth: lineDecorationsWidth,
                maxDigitWidth: themeOpts.maxDigitWidth,
                lineCount: lineCount,
                verticalScrollbarWidth: scrollbar.verticalScrollbarSize,
                horizontalScrollbarHeight: scrollbar.horizontalScrollbarSize,
                scrollbarArrowSize: scrollbar.arrowSize,
                verticalScrollbarHasArrows: scrollbar.verticalHasArrows
            });
            var pageSize = Math.floor(layoutInfo.height / themeOpts.lineHeight) - 2;
            if (isDominatedByLongLines && wrappingColumn > 0) {
                // Force viewport width wrapping if model is dominated by long lines
                wrappingColumn = 0;
            }
            var wrappingInfo;
            if (wrappingColumn === 0) {
                // If viewport width wrapping is enabled
                wrappingInfo = {
                    isViewportWrapping: true,
                    wrappingColumn: Math.max(1, Math.floor((layoutInfo.contentWidth - layoutInfo.verticalScrollbarWidth) / themeOpts.typicalHalfwidthCharacterWidth))
                };
            }
            else if (wrappingColumn > 0) {
                // Wrapping is enabled
                wrappingInfo = {
                    isViewportWrapping: false,
                    wrappingColumn: wrappingColumn
                };
            }
            else {
                wrappingInfo = {
                    isViewportWrapping: false,
                    wrappingColumn: -1
                };
            }
            var readOnly = toBoolean(opts.readOnly);
            var tabFocusMode = toBoolean(opts.tabFocusMode);
            if (readOnly) {
                tabFocusMode = true;
            }
            return {
                // ---- Options that are transparent - get no massaging
                lineNumbers: lineNumbers,
                selectOnLineNumbers: toBoolean(opts.selectOnLineNumbers),
                glyphMargin: glyphMargin,
                revealHorizontalRightPadding: toInteger(opts.revealHorizontalRightPadding, 0),
                roundedSelection: toBoolean(opts.roundedSelection),
                theme: opts.theme,
                readOnly: readOnly,
                scrollbar: scrollbar,
                overviewRulerLanes: toInteger(opts.overviewRulerLanes, 0, 3),
                cursorBlinking: opts.cursorBlinking,
                experimentalScreenReader: toBoolean(opts.experimentalScreenReader),
                rulers: toSortedIntegerArray(opts.rulers),
                wordSeparators: String(opts.wordSeparators),
                selectionClipboard: toBoolean(opts.selectionClipboard),
                ariaLabel: String(opts.ariaLabel),
                cursorStyle: editorCommon.cursorStyleFromString(opts.cursorStyle),
                fontLigatures: toBoolean(opts.fontLigatures),
                hideCursorInOverviewRuler: toBoolean(opts.hideCursorInOverviewRuler),
                scrollBeyondLastLine: toBoolean(opts.scrollBeyondLastLine),
                wrappingIndent: opts.wrappingIndent,
                wordWrapBreakBeforeCharacters: opts.wordWrapBreakBeforeCharacters,
                wordWrapBreakAfterCharacters: opts.wordWrapBreakAfterCharacters,
                wordWrapBreakObtrusiveCharacters: opts.wordWrapBreakObtrusiveCharacters,
                tabFocusMode: tabFocusMode,
                stopLineTokenizationAfter: stopLineTokenizationAfter,
                stopRenderingLineAfter: stopRenderingLineAfter,
                longLineBoundary: toInteger(opts.longLineBoundary),
                forcedTokenizationBoundary: toInteger(opts.forcedTokenizationBoundary),
                hover: toBoolean(opts.hover),
                contextmenu: toBoolean(opts.contextmenu),
                quickSuggestions: toBoolean(opts.quickSuggestions),
                quickSuggestionsDelay: toInteger(opts.quickSuggestionsDelay),
                iconsInSuggestions: toBoolean(opts.iconsInSuggestions),
                autoClosingBrackets: toBoolean(opts.autoClosingBrackets),
                formatOnType: toBoolean(opts.formatOnType),
                suggestOnTriggerCharacters: toBoolean(opts.suggestOnTriggerCharacters),
                acceptSuggestionOnEnter: toBoolean(opts.acceptSuggestionOnEnter),
                selectionHighlight: toBoolean(opts.selectionHighlight),
                outlineMarkers: toBoolean(opts.outlineMarkers),
                referenceInfos: toBoolean(opts.referenceInfos),
                folding: toBoolean(opts.folding),
                renderWhitespace: toBoolean(opts.renderWhitespace),
                layoutInfo: layoutInfo,
                stylingInfo: {
                    editorClassName: editorClassName,
                    fontFamily: requestedFontFamily,
                    fontSize: requestedFontSize,
                    lineHeight: adjustedLineHeight
                },
                wrappingInfo: wrappingInfo,
                observedOuterWidth: outerWidth,
                observedOuterHeight: outerHeight,
                lineHeight: themeOpts.lineHeight,
                pageSize: pageSize,
                typicalHalfwidthCharacterWidth: themeOpts.typicalHalfwidthCharacterWidth,
                typicalFullwidthCharacterWidth: themeOpts.typicalFullwidthCharacterWidth,
                fontSize: themeOpts.fontSize,
            };
        };
        InternalEditorOptionsHelper._sanitizeScrollbarOpts = function (raw, mouseWheelScrollSensitivity) {
            var horizontalScrollbarSize = toIntegerWithDefault(raw.horizontalScrollbarSize, 10);
            var verticalScrollbarSize = toIntegerWithDefault(raw.verticalScrollbarSize, 14);
            return {
                vertical: toStringSet(raw.vertical, ['auto', 'visible', 'hidden'], 'auto'),
                horizontal: toStringSet(raw.horizontal, ['auto', 'visible', 'hidden'], 'auto'),
                arrowSize: toIntegerWithDefault(raw.arrowSize, 11),
                useShadows: toBooleanWithDefault(raw.useShadows, true),
                verticalHasArrows: toBooleanWithDefault(raw.verticalHasArrows, false),
                horizontalHasArrows: toBooleanWithDefault(raw.horizontalHasArrows, false),
                horizontalScrollbarSize: horizontalScrollbarSize,
                horizontalSliderSize: toIntegerWithDefault(raw.horizontalSliderSize, horizontalScrollbarSize),
                verticalScrollbarSize: verticalScrollbarSize,
                verticalSliderSize: toIntegerWithDefault(raw.verticalSliderSize, verticalScrollbarSize),
                handleMouseWheel: toBooleanWithDefault(raw.handleMouseWheel, true),
                mouseWheelScrollSensitivity: mouseWheelScrollSensitivity
            };
        };
        InternalEditorOptionsHelper.createConfigurationChangedEvent = function (prevOpts, newOpts) {
            return {
                experimentalScreenReader: (prevOpts.experimentalScreenReader !== newOpts.experimentalScreenReader),
                rulers: (!this._numberArraysEqual(prevOpts.rulers, newOpts.rulers)),
                wordSeparators: (prevOpts.wordSeparators !== newOpts.wordSeparators),
                selectionClipboard: (prevOpts.selectionClipboard !== newOpts.selectionClipboard),
                ariaLabel: (prevOpts.ariaLabel !== newOpts.ariaLabel),
                lineNumbers: (prevOpts.lineNumbers !== newOpts.lineNumbers),
                selectOnLineNumbers: (prevOpts.selectOnLineNumbers !== newOpts.selectOnLineNumbers),
                glyphMargin: (prevOpts.glyphMargin !== newOpts.glyphMargin),
                revealHorizontalRightPadding: (prevOpts.revealHorizontalRightPadding !== newOpts.revealHorizontalRightPadding),
                roundedSelection: (prevOpts.roundedSelection !== newOpts.roundedSelection),
                theme: (prevOpts.theme !== newOpts.theme),
                readOnly: (prevOpts.readOnly !== newOpts.readOnly),
                scrollbar: (!this._scrollbarOptsEqual(prevOpts.scrollbar, newOpts.scrollbar)),
                overviewRulerLanes: (prevOpts.overviewRulerLanes !== newOpts.overviewRulerLanes),
                cursorBlinking: (prevOpts.cursorBlinking !== newOpts.cursorBlinking),
                cursorStyle: (prevOpts.cursorStyle !== newOpts.cursorStyle),
                fontLigatures: (prevOpts.fontLigatures !== newOpts.fontLigatures),
                hideCursorInOverviewRuler: (prevOpts.hideCursorInOverviewRuler !== newOpts.hideCursorInOverviewRuler),
                scrollBeyondLastLine: (prevOpts.scrollBeyondLastLine !== newOpts.scrollBeyondLastLine),
                wrappingIndent: (prevOpts.wrappingIndent !== newOpts.wrappingIndent),
                wordWrapBreakBeforeCharacters: (prevOpts.wordWrapBreakBeforeCharacters !== newOpts.wordWrapBreakBeforeCharacters),
                wordWrapBreakAfterCharacters: (prevOpts.wordWrapBreakAfterCharacters !== newOpts.wordWrapBreakAfterCharacters),
                wordWrapBreakObtrusiveCharacters: (prevOpts.wordWrapBreakObtrusiveCharacters !== newOpts.wordWrapBreakObtrusiveCharacters),
                tabFocusMode: (prevOpts.tabFocusMode !== newOpts.tabFocusMode),
                stopLineTokenizationAfter: (prevOpts.stopLineTokenizationAfter !== newOpts.stopLineTokenizationAfter),
                stopRenderingLineAfter: (prevOpts.stopRenderingLineAfter !== newOpts.stopRenderingLineAfter),
                longLineBoundary: (prevOpts.longLineBoundary !== newOpts.longLineBoundary),
                forcedTokenizationBoundary: (prevOpts.forcedTokenizationBoundary !== newOpts.forcedTokenizationBoundary),
                hover: (prevOpts.hover !== newOpts.hover),
                contextmenu: (prevOpts.contextmenu !== newOpts.contextmenu),
                quickSuggestions: (prevOpts.quickSuggestions !== newOpts.quickSuggestions),
                quickSuggestionsDelay: (prevOpts.quickSuggestionsDelay !== newOpts.quickSuggestionsDelay),
                iconsInSuggestions: (prevOpts.iconsInSuggestions !== newOpts.iconsInSuggestions),
                autoClosingBrackets: (prevOpts.autoClosingBrackets !== newOpts.autoClosingBrackets),
                formatOnType: (prevOpts.formatOnType !== newOpts.formatOnType),
                suggestOnTriggerCharacters: (prevOpts.suggestOnTriggerCharacters !== newOpts.suggestOnTriggerCharacters),
                selectionHighlight: (prevOpts.selectionHighlight !== newOpts.selectionHighlight),
                outlineMarkers: (prevOpts.outlineMarkers !== newOpts.outlineMarkers),
                referenceInfos: (prevOpts.referenceInfos !== newOpts.referenceInfos),
                folding: (prevOpts.folding !== newOpts.folding),
                renderWhitespace: (prevOpts.renderWhitespace !== newOpts.renderWhitespace),
                layoutInfo: (!editorLayoutProvider_1.EditorLayoutProvider.layoutEqual(prevOpts.layoutInfo, newOpts.layoutInfo)),
                stylingInfo: (!this._stylingInfoEqual(prevOpts.stylingInfo, newOpts.stylingInfo)),
                wrappingInfo: (!this._wrappingInfoEqual(prevOpts.wrappingInfo, newOpts.wrappingInfo)),
                observedOuterWidth: (prevOpts.observedOuterWidth !== newOpts.observedOuterWidth),
                observedOuterHeight: (prevOpts.observedOuterHeight !== newOpts.observedOuterHeight),
                lineHeight: (prevOpts.lineHeight !== newOpts.lineHeight),
                pageSize: (prevOpts.pageSize !== newOpts.pageSize),
                typicalHalfwidthCharacterWidth: (prevOpts.typicalHalfwidthCharacterWidth !== newOpts.typicalHalfwidthCharacterWidth),
                typicalFullwidthCharacterWidth: (prevOpts.typicalFullwidthCharacterWidth !== newOpts.typicalFullwidthCharacterWidth),
                fontSize: (prevOpts.fontSize !== newOpts.fontSize)
            };
        };
        InternalEditorOptionsHelper._scrollbarOptsEqual = function (a, b) {
            return (a.arrowSize === b.arrowSize
                && a.vertical === b.vertical
                && a.horizontal === b.horizontal
                && a.useShadows === b.useShadows
                && a.verticalHasArrows === b.verticalHasArrows
                && a.horizontalHasArrows === b.horizontalHasArrows
                && a.handleMouseWheel === b.handleMouseWheel
                && a.horizontalScrollbarSize === b.horizontalScrollbarSize
                && a.horizontalSliderSize === b.horizontalSliderSize
                && a.verticalScrollbarSize === b.verticalScrollbarSize
                && a.verticalSliderSize === b.verticalSliderSize
                && a.mouseWheelScrollSensitivity === b.mouseWheelScrollSensitivity);
        };
        InternalEditorOptionsHelper._stylingInfoEqual = function (a, b) {
            return (a.editorClassName === b.editorClassName
                && a.fontFamily === b.fontFamily
                && a.fontSize === b.fontSize
                && a.lineHeight === b.lineHeight);
        };
        InternalEditorOptionsHelper._wrappingInfoEqual = function (a, b) {
            return (a.isViewportWrapping === b.isViewportWrapping
                && a.wrappingColumn === b.wrappingColumn);
        };
        InternalEditorOptionsHelper._numberArraysEqual = function (a, b) {
            if (a.length !== b.length) {
                return false;
            }
            for (var i = 0; i < a.length; i++) {
                if (a[i] !== b[i]) {
                    return false;
                }
            }
            return true;
        };
        return InternalEditorOptionsHelper;
    }());
    function toBoolean(value) {
        return value === 'false' ? false : Boolean(value);
    }
    function toBooleanWithDefault(value, defaultValue) {
        if (typeof value === 'undefined') {
            return defaultValue;
        }
        return toBoolean(value);
    }
    function toFloat(source, defaultValue) {
        var r = parseFloat(source);
        if (isNaN(r)) {
            r = defaultValue;
        }
        return r;
    }
    function toInteger(source, minimum, maximum) {
        var r = parseInt(source, 10);
        if (isNaN(r)) {
            r = 0;
        }
        if (typeof minimum === 'number') {
            r = Math.max(minimum, r);
        }
        if (typeof maximum === 'number') {
            r = Math.min(maximum, r);
        }
        return r;
    }
    function toSortedIntegerArray(source) {
        if (!Array.isArray(source)) {
            return [];
        }
        var arrSource = source;
        var r = arrSource.map(function (el) { return toInteger(el); });
        r.sort();
        return r;
    }
    function toIntegerWithDefault(source, defaultValue) {
        if (typeof source === 'undefined') {
            return defaultValue;
        }
        return toInteger(source);
    }
    function toStringSet(source, allowedValues, defaultValue) {
        if (typeof source !== 'string') {
            return defaultValue;
        }
        if (allowedValues.indexOf(source) === -1) {
            return defaultValue;
        }
        return source;
    }
    var CommonEditorConfiguration = (function (_super) {
        __extends(CommonEditorConfiguration, _super);
        function CommonEditorConfiguration(options, elementSizeObserver) {
            if (elementSizeObserver === void 0) { elementSizeObserver = null; }
            _super.call(this);
            this._onDidChange = this._register(new event_1.Emitter());
            this.onDidChange = this._onDidChange.event;
            this._configWithDefaults = new ConfigurationWithDefaults(options);
            this._elementSizeObserver = elementSizeObserver;
            this._isDominatedByLongLines = false;
            this._lineCount = 1;
            this.handlerDispatcher = new handlerDispatcher_1.HandlerDispatcher();
            this.editor = this._computeInternalOptions();
            this.editorClone = cloneInternalEditorOptions(this.editor);
        }
        CommonEditorConfiguration.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
        };
        CommonEditorConfiguration.prototype._recomputeOptions = function () {
            var oldOpts = this.editor;
            this.editor = this._computeInternalOptions();
            this.editorClone = cloneInternalEditorOptions(this.editor);
            var changeEvent = InternalEditorOptionsHelper.createConfigurationChangedEvent(oldOpts, this.editor);
            var hasChanged = false;
            for (var key in changeEvent) {
                if (changeEvent.hasOwnProperty(key)) {
                    if (changeEvent[key]) {
                        hasChanged = true;
                        break;
                    }
                }
            }
            if (hasChanged) {
                this._onDidChange.fire(changeEvent);
            }
        };
        CommonEditorConfiguration.prototype.getRawOptions = function () {
            return this._configWithDefaults.getEditorOptions();
        };
        CommonEditorConfiguration.prototype._computeInternalOptions = function () {
            var opts = this._configWithDefaults.getEditorOptions();
            var editorClassName = this._getEditorClassName(opts.theme, toBoolean(opts.fontLigatures));
            var requestedFontFamily = opts.fontFamily || '';
            var requestedFontSize = toInteger(opts.fontSize, 0, 100);
            var requestedLineHeight = toInteger(opts.lineHeight, 0, 150);
            var adjustedLineHeight = requestedLineHeight;
            if (requestedFontSize > 0 && requestedLineHeight === 0) {
                adjustedLineHeight = Math.round(1.3 * requestedFontSize);
            }
            return InternalEditorOptionsHelper.createInternalEditorOptions(this.getOuterWidth(), this.getOuterHeight(), opts, editorClassName, requestedFontFamily, requestedFontSize, requestedLineHeight, adjustedLineHeight, this.readConfiguration(editorClassName, requestedFontFamily, requestedFontSize, adjustedLineHeight), this._isDominatedByLongLines, this._lineCount);
        };
        CommonEditorConfiguration.prototype.updateOptions = function (newOptions) {
            this._configWithDefaults.updateOptions(newOptions);
            this._recomputeOptions();
        };
        CommonEditorConfiguration.prototype.setIsDominatedByLongLines = function (isDominatedByLongLines) {
            this._isDominatedByLongLines = isDominatedByLongLines;
            this._recomputeOptions();
        };
        CommonEditorConfiguration.prototype.setLineCount = function (lineCount) {
            this._lineCount = lineCount;
            this._recomputeOptions();
        };
        return CommonEditorConfiguration;
    }(lifecycle_1.Disposable));
    exports.CommonEditorConfiguration = CommonEditorConfiguration;
    /**
     * Helper to update Monaco Editor Settings from configurations service.
     */
    var EditorConfiguration = (function () {
        function EditorConfiguration() {
        }
        EditorConfiguration.apply = function (config, editorOrArray) {
            if (!config) {
                return;
            }
            var editors = editorOrArray;
            if (!Array.isArray(editorOrArray)) {
                editors = [editorOrArray];
            }
            for (var i = 0; i < editors.length; i++) {
                var editor = editors[i];
                // Editor Settings (Code Editor, Diff, Terminal)
                if (editor && typeof editor.updateOptions === 'function') {
                    var type = editor.getEditorType();
                    if (type !== editorCommon.EditorType.ICodeEditor && type !== editorCommon.EditorType.IDiffEditor) {
                        continue;
                    }
                    var editorConfig = config[EditorConfiguration.EDITOR_SECTION];
                    if (type === editorCommon.EditorType.IDiffEditor) {
                        var diffEditorConfig = config[EditorConfiguration.DIFF_EDITOR_SECTION];
                        if (diffEditorConfig) {
                            if (!editorConfig) {
                                editorConfig = diffEditorConfig;
                            }
                            else {
                                editorConfig = objects.mixin(editorConfig, diffEditorConfig);
                            }
                        }
                    }
                    if (editorConfig) {
                        delete editorConfig.readOnly; // Prevent someone from making editor readonly
                        editor.updateOptions(editorConfig);
                    }
                }
            }
        };
        EditorConfiguration.EDITOR_SECTION = 'editor';
        EditorConfiguration.DIFF_EDITOR_SECTION = 'diffEditor';
        return EditorConfiguration;
    }());
    exports.EditorConfiguration = EditorConfiguration;
    var configurationRegistry = platform_1.Registry.as(configurationRegistry_1.Extensions.Configuration);
    var editorConfiguration = {
        'id': 'editor',
        'order': 5,
        'type': 'object',
        'title': nls.localize(0, null),
        'properties': {
            'editor.fontFamily': {
                'type': 'string',
                'default': defaultConfig_1.DefaultConfig.editor.fontFamily,
                'description': nls.localize(1, null)
            },
            'editor.fontSize': {
                'type': 'number',
                'default': defaultConfig_1.DefaultConfig.editor.fontSize,
                'description': nls.localize(2, null)
            },
            'editor.lineHeight': {
                'type': 'number',
                'default': defaultConfig_1.DefaultConfig.editor.lineHeight,
                'description': nls.localize(3, null)
            },
            'editor.lineNumbers': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.lineNumbers,
                'description': nls.localize(4, null)
            },
            'editor.glyphMargin': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.glyphMargin,
                'description': nls.localize(5, null)
            },
            'editor.rulers': {
                'type': 'array',
                'items': {
                    'type': 'number'
                },
                'default': defaultConfig_1.DefaultConfig.editor.rulers,
                'description': nls.localize(6, null)
            },
            'editor.wordSeparators': {
                'type': 'string',
                'default': defaultConfig_1.DefaultConfig.editor.wordSeparators,
                'description': nls.localize(7, null)
            },
            'editor.tabSize': {
                'type': 'number',
                'default': defaultConfig_1.DEFAULT_INDENTATION.tabSize,
                'minimum': 1,
                'description': nls.localize(8, null),
                'errorMessage': nls.localize(9, null)
            },
            'editor.insertSpaces': {
                'type': 'boolean',
                'default': defaultConfig_1.DEFAULT_INDENTATION.insertSpaces,
                'description': nls.localize(10, null),
                'errorMessage': nls.localize(11, null)
            },
            'editor.detectIndentation': {
                'type': 'boolean',
                'default': defaultConfig_1.DEFAULT_INDENTATION.detectIndentation,
                'description': nls.localize(12, null)
            },
            'editor.roundedSelection': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.roundedSelection,
                'description': nls.localize(13, null)
            },
            'editor.scrollBeyondLastLine': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.scrollBeyondLastLine,
                'description': nls.localize(14, null)
            },
            'editor.wrappingColumn': {
                'type': 'integer',
                'default': defaultConfig_1.DefaultConfig.editor.wrappingColumn,
                'minimum': -1,
                'description': nls.localize(15, null)
            },
            'editor.wrappingIndent': {
                'type': 'string',
                'enum': ['none', 'same', 'indent'],
                'default': defaultConfig_1.DefaultConfig.editor.wrappingIndent,
                'description': nls.localize(16, null)
            },
            'editor.mouseWheelScrollSensitivity': {
                'type': 'number',
                'default': defaultConfig_1.DefaultConfig.editor.mouseWheelScrollSensitivity,
                'description': nls.localize(17, null)
            },
            'editor.quickSuggestions': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.quickSuggestions,
                'description': nls.localize(18, null)
            },
            'editor.quickSuggestionsDelay': {
                'type': 'integer',
                'default': defaultConfig_1.DefaultConfig.editor.quickSuggestionsDelay,
                'minimum': 0,
                'description': nls.localize(19, null)
            },
            'editor.autoClosingBrackets': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.autoClosingBrackets,
                'description': nls.localize(20, null)
            },
            'editor.formatOnType': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.formatOnType,
                'description': nls.localize(21, null)
            },
            'editor.suggestOnTriggerCharacters': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.suggestOnTriggerCharacters,
                'description': nls.localize(22, null)
            },
            'editor.acceptSuggestionOnEnter': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.acceptSuggestionOnEnter,
                'description': nls.localize(23, null)
            },
            'editor.selectionHighlight': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.selectionHighlight,
                'description': nls.localize(24, null)
            },
            //		'editor.outlineMarkers' : {
            //			'type': 'boolean',
            //			'default': DefaultConfig.editor.outlineMarkers,
            //			'description': nls.localize('outlineMarkers', "Controls whether the editor should draw horizontal lines before classes and methods")
            //		},
            'editor.overviewRulerLanes': {
                'type': 'integer',
                'default': 3,
                'description': nls.localize(25, null)
            },
            'editor.cursorBlinking': {
                'type': 'string',
                'enum': ['blink', 'visible', 'hidden'],
                'default': defaultConfig_1.DefaultConfig.editor.cursorBlinking,
                'description': nls.localize(26, null)
            },
            'editor.cursorStyle': {
                'type': 'string',
                'enum': ['block', 'line'],
                'default': defaultConfig_1.DefaultConfig.editor.cursorStyle,
                'description': nls.localize(27, null)
            },
            'editor.fontLigatures': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.fontLigatures,
                'description': nls.localize(28, null)
            },
            'editor.hideCursorInOverviewRuler': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.hideCursorInOverviewRuler,
                'description': nls.localize(29, null)
            },
            'editor.renderWhitespace': {
                'type': 'boolean',
                default: defaultConfig_1.DefaultConfig.editor.renderWhitespace,
                description: nls.localize(30, null)
            },
            'editor.referenceInfos': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.referenceInfos,
                'description': nls.localize(31, null)
            },
            'editor.folding': {
                'type': 'boolean',
                'default': defaultConfig_1.DefaultConfig.editor.folding,
                'description': nls.localize(32, null)
            },
            'diffEditor.renderSideBySide': {
                'type': 'boolean',
                'default': true,
                'description': nls.localize(33, null)
            },
            'diffEditor.ignoreTrimWhitespace': {
                'type': 'boolean',
                'default': true,
                'description': nls.localize(34, null)
            }
        }
    };
    if (platform.isLinux) {
        editorConfiguration['properties']['editor.selectionClipboard'] = {
            'type': 'boolean',
            'default': defaultConfig_1.DefaultConfig.editor.selectionClipboard,
            'description': nls.localize(35, null)
        };
    }
    configurationRegistry.registerConfiguration(editorConfiguration);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/config/configuration", ["require", "exports", 'vs/base/common/event', 'vs/base/common/lifecycle', 'vs/base/common/platform', 'vs/base/browser/browser', 'vs/base/browser/dom', 'vs/editor/common/config/commonEditorConfig', 'vs/editor/browser/config/elementSizeObserver'], function (require, exports, event_1, lifecycle_1, platform, browser, dom, commonEditorConfig_1, elementSizeObserver_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var CSSBasedConfigurationCache = (function () {
        function CSSBasedConfigurationCache() {
            this._keys = {};
            this._values = {};
        }
        CSSBasedConfigurationCache.prototype.has = function (item) {
            return this._values.hasOwnProperty(CSSBasedConfigurationCache._key(item));
        };
        CSSBasedConfigurationCache.prototype.get = function (item) {
            return this._values[CSSBasedConfigurationCache._key(item)];
        };
        CSSBasedConfigurationCache.prototype.put = function (item, value) {
            this._values[CSSBasedConfigurationCache._key(item)] = value;
        };
        CSSBasedConfigurationCache.prototype.getKeys = function () {
            var r = [];
            for (var key in this._keys) {
                r.push(this._keys[key]);
            }
            return r;
        };
        CSSBasedConfigurationCache._key = function (item) {
            return item.editorClassName + '-' + item.fontFamily + '-' + item.fontSize + '-' + item.lineHeight;
        };
        return CSSBasedConfigurationCache;
    }());
    var CSSBasedConfiguration = (function (_super) {
        __extends(CSSBasedConfiguration, _super);
        function CSSBasedConfiguration() {
            _super.call(this);
            this._changeMonitorTimeout = -1;
            this._onDidChange = this._register(new event_1.Emitter());
            this.onDidChange = this._onDidChange.event;
            this._cache = new CSSBasedConfigurationCache();
        }
        CSSBasedConfiguration.prototype.dispose = function () {
            if (this._changeMonitorTimeout !== -1) {
                clearTimeout(this._changeMonitorTimeout);
                this._changeMonitorTimeout = -1;
            }
            _super.prototype.dispose.call(this);
        };
        CSSBasedConfiguration.prototype.readConfiguration = function (editorClassName, fontFamily, fontSize, lineHeight) {
            var styling = {
                editorClassName: editorClassName,
                fontFamily: fontFamily,
                fontSize: fontSize,
                lineHeight: lineHeight
            };
            if (!this._cache.has(styling)) {
                var readConfig = CSSBasedConfiguration._actualReadConfiguration(styling);
                if (readConfig.lineHeight <= 2 || readConfig.typicalHalfwidthCharacterWidth <= 2 || readConfig.typicalFullwidthCharacterWidth <= 2 || readConfig.maxDigitWidth <= 2) {
                    // Hey, it's Bug 14341 ... we couldn't read
                    readConfig.lineHeight = Math.max(readConfig.lineHeight, readConfig.fontSize, 5);
                    readConfig.typicalHalfwidthCharacterWidth = Math.max(readConfig.typicalHalfwidthCharacterWidth, readConfig.fontSize, 5);
                    readConfig.typicalFullwidthCharacterWidth = Math.max(readConfig.typicalFullwidthCharacterWidth, readConfig.fontSize, 5);
                    readConfig.maxDigitWidth = Math.max(readConfig.maxDigitWidth, readConfig.fontSize, 5);
                    this._installChangeMonitor();
                }
                this._cache.put(styling, readConfig);
            }
            return this._cache.get(styling);
        };
        CSSBasedConfiguration.prototype._installChangeMonitor = function () {
            var _this = this;
            if (this._changeMonitorTimeout === -1) {
                this._changeMonitorTimeout = setTimeout(function () {
                    _this._changeMonitorTimeout = -1;
                    _this._monitorForChanges();
                }, 500);
            }
        };
        CSSBasedConfiguration.prototype._monitorForChanges = function () {
            var shouldInstallChangeMonitor = false;
            var keys = this._cache.getKeys();
            for (var i = 0; i < keys.length; i++) {
                var styling = keys[i];
                var newValue = CSSBasedConfiguration._actualReadConfiguration(styling);
                if (newValue.lineHeight <= 2 || newValue.typicalHalfwidthCharacterWidth <= 2 || newValue.typicalFullwidthCharacterWidth <= 2 || newValue.maxDigitWidth <= 2) {
                    // We still couldn't read the CSS config
                    shouldInstallChangeMonitor = true;
                }
                else {
                    this._cache.put(styling, newValue);
                    this._onDidChange.fire();
                }
            }
            if (shouldInstallChangeMonitor) {
                this._installChangeMonitor();
            }
        };
        CSSBasedConfiguration._testElementId = function (index) {
            return 'editorSizeProvider' + index;
        };
        CSSBasedConfiguration._createTestElement = function (index, character) {
            var r = document.createElement('span');
            r.id = this._testElementId(index);
            var testString = (character === ' ' ? '&nbsp;' : character);
            // Repeat character 256 (2^8) times
            for (var i = 0; i < 8; i++) {
                testString += testString;
            }
            r.textContent = testString;
            return r;
        };
        CSSBasedConfiguration._createTestElements = function (styling) {
            var container = document.createElement('div');
            Configuration.applyEditorStyling(container, styling);
            container.style.position = 'absolute';
            container.style.top = '-50000px';
            container.style.width = '50000px';
            for (var i = 0, len = CSSBasedConfiguration._USUAL_CHARS.length; i < len; i++) {
                container.appendChild(document.createElement('br'));
                container.appendChild(this._createTestElement(i, CSSBasedConfiguration._USUAL_CHARS[i]));
            }
            var heightTestElementId = this._testElementId(CSSBasedConfiguration._USUAL_CHARS.length);
            var heightTestElement = document.createElement('div');
            heightTestElement.id = heightTestElementId;
            heightTestElement.appendChild(document.createTextNode('heightTestContent'));
            container.appendChild(document.createElement('br'));
            container.appendChild(heightTestElement);
            return container;
        };
        CSSBasedConfiguration._readFromTestElements = function () {
            var r = [];
            for (var i = 0, len = CSSBasedConfiguration._USUAL_CHARS.length; i < len; i++) {
                r.push(document.getElementById(this._testElementId(i)).offsetWidth / 256);
            }
            return r;
        };
        CSSBasedConfiguration._actualReadConfiguration = function (styling) {
            // Create a test container with all these test elements
            var testContainer = this._createTestElements(styling);
            // Add the container to the DOM
            document.body.appendChild(testContainer);
            // Read various properties
            var usualCharsWidths = this._readFromTestElements();
            var firstTestElement = document.getElementById(this._testElementId(0));
            var computedStyle = dom.getComputedStyle(firstTestElement);
            var result_font = this._getFontFromComputedStyle(computedStyle);
            var result_fontSize = computedStyle ? parseInt(computedStyle.fontSize, 10) : 0;
            var heightTestElement = document.getElementById(this._testElementId(CSSBasedConfiguration._USUAL_CHARS.length));
            var result_lineHeight = heightTestElement.clientHeight;
            // Remove the container from the DOM
            document.body.removeChild(testContainer);
            // Find maximum digit width and thinnest character width
            var maxDigitWidth = 0, typicalHalfwidthCharacterWidth = 0, typicalFullwidthCharacterWidth = 0;
            for (var i = 0, len = CSSBasedConfiguration._USUAL_CHARS.length; i < len; i++) {
                var character = CSSBasedConfiguration._USUAL_CHARS.charAt(i);
                if (character >= '0' && character <= '9') {
                    maxDigitWidth = Math.max(maxDigitWidth, usualCharsWidths[i]);
                }
                else if (character === CSSBasedConfiguration._HALF_WIDTH_TYPICAL) {
                    typicalHalfwidthCharacterWidth = usualCharsWidths[i];
                }
                else if (character === CSSBasedConfiguration._FULL_WIDTH_TYPICAL) {
                    typicalFullwidthCharacterWidth = usualCharsWidths[i];
                }
            }
            return {
                typicalHalfwidthCharacterWidth: typicalHalfwidthCharacterWidth,
                typicalFullwidthCharacterWidth: typicalFullwidthCharacterWidth,
                maxDigitWidth: maxDigitWidth,
                lineHeight: result_lineHeight,
                font: result_font,
                fontSize: result_fontSize
            };
        };
        CSSBasedConfiguration._getFontFromComputedStyle = function (computedStyle) {
            if (!computedStyle) {
                return 'unknown';
            }
            if (computedStyle.font) {
                return computedStyle.font;
            }
            return (computedStyle.fontFamily + ' ' +
                computedStyle.fontSize + ' ' +
                computedStyle.fontSizeAdjust + ' ' +
                computedStyle.fontStretch + ' ' +
                computedStyle.fontStyle + ' ' +
                computedStyle.fontVariant + ' ' +
                computedStyle.fontWeight + ' ');
        };
        CSSBasedConfiguration.INSTANCE = new CSSBasedConfiguration();
        CSSBasedConfiguration._HALF_WIDTH_TYPICAL = 'n';
        CSSBasedConfiguration._FULL_WIDTH_TYPICAL = '\uff4d';
        CSSBasedConfiguration._USUAL_CHARS = '0123456789' + CSSBasedConfiguration._HALF_WIDTH_TYPICAL + CSSBasedConfiguration._FULL_WIDTH_TYPICAL;
        return CSSBasedConfiguration;
    }(lifecycle_1.Disposable));
    var Configuration = (function (_super) {
        __extends(Configuration, _super);
        function Configuration(options, referenceDomElement) {
            var _this = this;
            if (referenceDomElement === void 0) { referenceDomElement = null; }
            _super.call(this, options, new elementSizeObserver_1.ElementSizeObserver(referenceDomElement, function () { return _this._onReferenceDomElementSizeChanged(); }));
            this._register(CSSBasedConfiguration.INSTANCE.onDidChange(function () { return function () { return _this._onCSSBasedConfigurationChanged(); }; }));
            if (this._configWithDefaults.getEditorOptions().automaticLayout) {
                this._elementSizeObserver.startObserving();
            }
        }
        Configuration.applyEditorStyling = function (domNode, styling) {
            domNode.className = styling.editorClassName;
            if (styling.fontFamily && styling.fontFamily.length > 0) {
                domNode.style.fontFamily = styling.fontFamily;
            }
            else {
                domNode.style.fontFamily = '';
            }
            if (styling.fontSize > 0) {
                domNode.style.fontSize = styling.fontSize + 'px';
            }
            else {
                domNode.style.fontSize = '';
            }
            if (styling.lineHeight > 0) {
                domNode.style.lineHeight = styling.lineHeight + 'px';
            }
            else {
                domNode.style.lineHeight = '';
            }
        };
        Configuration.prototype._onReferenceDomElementSizeChanged = function () {
            this._recomputeOptions();
        };
        Configuration.prototype._onCSSBasedConfigurationChanged = function () {
            this._recomputeOptions();
        };
        Configuration.prototype.observeReferenceElement = function (dimension) {
            this._elementSizeObserver.observe(dimension);
        };
        Configuration.prototype.dispose = function () {
            this._elementSizeObserver.dispose();
            _super.prototype.dispose.call(this);
        };
        Configuration.prototype._getEditorClassName = function (theme, fontLigatures) {
            var extra = '';
            if (browser.isIE11orEarlier) {
                extra += 'ie ';
            }
            else if (browser.isFirefox) {
                extra += 'ff ';
            }
            if (browser.isIE9) {
                extra += 'ie9 ';
            }
            if (platform.isMacintosh) {
                extra += 'mac ';
            }
            if (fontLigatures) {
                extra += 'enable-ligatures ';
            }
            return 'monaco-editor ' + extra + theme;
        };
        Configuration.prototype.getOuterWidth = function () {
            return this._elementSizeObserver.getWidth();
        };
        Configuration.prototype.getOuterHeight = function () {
            return this._elementSizeObserver.getHeight();
        };
        Configuration.prototype.readConfiguration = function (editorClassName, fontFamily, fontSize, lineHeight) {
            return CSSBasedConfiguration.INSTANCE.readConfiguration(editorClassName, fontFamily, fontSize, lineHeight);
        };
        return Configuration;
    }(commonEditorConfig_1.CommonEditorConfiguration));
    exports.Configuration = Configuration;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/controller/keyboardHandler", ["require", "exports", 'vs/base/common/event', 'vs/base/common/lifecycle', 'vs/base/browser/browser', 'vs/base/browser/dom', 'vs/base/browser/styleMutator', 'vs/editor/common/config/commonEditorConfig', 'vs/editor/common/controller/textAreaHandler', 'vs/editor/common/controller/textAreaState', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/viewModel/viewEventHandler'], function (require, exports, event_1, lifecycle_1, browser, dom, styleMutator_1, commonEditorConfig_1, textAreaHandler_1, textAreaState_1, range_1, editorCommon, viewEventHandler_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ClipboardEventWrapper = (function () {
        function ClipboardEventWrapper(event) {
            this._event = event;
        }
        ClipboardEventWrapper.prototype.canUseTextData = function () {
            if (this._event.clipboardData) {
                return true;
            }
            if (window.clipboardData) {
                return true;
            }
            return false;
        };
        ClipboardEventWrapper.prototype.setTextData = function (text) {
            if (this._event.clipboardData) {
                this._event.clipboardData.setData('text/plain', text);
                this._event.preventDefault();
                return;
            }
            if (window.clipboardData) {
                window.clipboardData.setData('Text', text);
                this._event.preventDefault();
                return;
            }
            throw new Error('ClipboardEventWrapper.setTextData: Cannot use text data!');
        };
        ClipboardEventWrapper.prototype.getTextData = function () {
            if (this._event.clipboardData) {
                this._event.preventDefault();
                return this._event.clipboardData.getData('text/plain');
            }
            if (window.clipboardData) {
                this._event.preventDefault();
                return window.clipboardData.getData('Text');
            }
            throw new Error('ClipboardEventWrapper.getTextData: Cannot use text data!');
        };
        return ClipboardEventWrapper;
    }());
    var KeyboardEventWrapper = (function () {
        function KeyboardEventWrapper(actual) {
            this._actual = actual;
        }
        KeyboardEventWrapper.prototype.equals = function (keybinding) {
            return this._actual.equals(keybinding);
        };
        KeyboardEventWrapper.prototype.preventDefault = function () {
            this._actual.preventDefault();
        };
        KeyboardEventWrapper.prototype.isDefaultPrevented = function () {
            if (this._actual.browserEvent) {
                return this._actual.browserEvent.defaultPrevented;
            }
            return false;
        };
        return KeyboardEventWrapper;
    }());
    var TextAreaWrapper = (function (_super) {
        __extends(TextAreaWrapper, _super);
        function TextAreaWrapper(textArea) {
            var _this = this;
            _super.call(this);
            this._onKeyDown = this._register(new event_1.Emitter());
            this.onKeyDown = this._onKeyDown.event;
            this._onKeyUp = this._register(new event_1.Emitter());
            this.onKeyUp = this._onKeyUp.event;
            this._onKeyPress = this._register(new event_1.Emitter());
            this.onKeyPress = this._onKeyPress.event;
            this._onCompositionStart = this._register(new event_1.Emitter());
            this.onCompositionStart = this._onCompositionStart.event;
            this._onCompositionEnd = this._register(new event_1.Emitter());
            this.onCompositionEnd = this._onCompositionEnd.event;
            this._onInput = this._register(new event_1.Emitter());
            this.onInput = this._onInput.event;
            this._onCut = this._register(new event_1.Emitter());
            this.onCut = this._onCut.event;
            this._onCopy = this._register(new event_1.Emitter());
            this.onCopy = this._onCopy.event;
            this._onPaste = this._register(new event_1.Emitter());
            this.onPaste = this._onPaste.event;
            this._textArea = textArea;
            this._register(dom.addStandardDisposableListener(this._textArea, 'keydown', function (e) { return _this._onKeyDown.fire(new KeyboardEventWrapper(e)); }));
            this._register(dom.addStandardDisposableListener(this._textArea, 'keyup', function (e) { return _this._onKeyUp.fire(new KeyboardEventWrapper(e)); }));
            this._register(dom.addStandardDisposableListener(this._textArea, 'keypress', function (e) { return _this._onKeyPress.fire(new KeyboardEventWrapper(e)); }));
            this._register(dom.addDisposableListener(this._textArea, 'compositionstart', function (e) { return _this._onCompositionStart.fire(); }));
            this._register(dom.addDisposableListener(this._textArea, 'compositionend', function (e) { return _this._onCompositionEnd.fire(); }));
            this._register(dom.addDisposableListener(this._textArea, 'input', function (e) { return _this._onInput.fire(); }));
            this._register(dom.addDisposableListener(this._textArea, 'cut', function (e) { return _this._onCut.fire(new ClipboardEventWrapper(e)); }));
            this._register(dom.addDisposableListener(this._textArea, 'copy', function (e) { return _this._onCopy.fire(new ClipboardEventWrapper(e)); }));
            this._register(dom.addDisposableListener(this._textArea, 'paste', function (e) { return _this._onPaste.fire(new ClipboardEventWrapper(e)); }));
        }
        Object.defineProperty(TextAreaWrapper.prototype, "actual", {
            get: function () {
                return this._textArea;
            },
            enumerable: true,
            configurable: true
        });
        TextAreaWrapper.prototype.getValue = function () {
            // console.log('current value: ' + this._textArea.value);
            return this._textArea.value;
        };
        TextAreaWrapper.prototype.setValue = function (reason, value) {
            // console.log('reason: ' + reason + ', current value: ' + this._textArea.value + ' => new value: ' + value);
            this._textArea.value = value;
        };
        TextAreaWrapper.prototype.getSelectionStart = function () {
            return this._textArea.selectionStart;
        };
        TextAreaWrapper.prototype.getSelectionEnd = function () {
            return this._textArea.selectionEnd;
        };
        TextAreaWrapper.prototype.setSelectionRange = function (selectionStart, selectionEnd) {
            // console.log('setSelectionRange: ' + selectionStart + ', ' + selectionEnd);
            try {
                var scrollState = dom.saveParentsScrollTop(this._textArea);
                this._textArea.focus();
                this._textArea.setSelectionRange(selectionStart, selectionEnd);
                dom.restoreParentsScrollTop(this._textArea, scrollState);
            }
            catch (e) {
                // Sometimes IE throws when setting selection (e.g. textarea is off-DOM)
                console.log('an error has been thrown!');
            }
        };
        TextAreaWrapper.prototype.isInOverwriteMode = function () {
            // In IE, pressing Insert will bring the typing into overwrite mode
            if (browser.isIE11orEarlier && document.queryCommandValue('OverWrite')) {
                return true;
            }
            return false;
        };
        return TextAreaWrapper;
    }(lifecycle_1.Disposable));
    var KeyboardHandler = (function (_super) {
        __extends(KeyboardHandler, _super);
        function KeyboardHandler(context, viewController, viewHelper) {
            var _this = this;
            _super.call(this);
            this.context = context;
            this.viewController = viewController;
            this.textArea = new TextAreaWrapper(viewHelper.textArea);
            this.viewHelper = viewHelper;
            this.contentLeft = 0;
            this.contentWidth = 0;
            this.scrollLeft = 0;
            this.textAreaHandler = new textAreaHandler_1.TextAreaHandler(browser, this._getStrategy(), this.textArea, this.context.model, function () { return _this.viewHelper.flushAnyAccumulatedEvents(); });
            this._toDispose = [];
            this._toDispose.push(this.textAreaHandler.onKeyDown(function (e) { return _this.viewController.emitKeyDown(e._actual); }));
            this._toDispose.push(this.textAreaHandler.onKeyUp(function (e) { return _this.viewController.emitKeyUp(e._actual); }));
            this._toDispose.push(this.textAreaHandler.onPaste(function (e) { return _this.viewController.paste('keyboard', e.text, e.pasteOnNewLine); }));
            this._toDispose.push(this.textAreaHandler.onCut(function (e) { return _this.viewController.cut('keyboard'); }));
            this._toDispose.push(this.textAreaHandler.onType(function (e) {
                if (e.replaceCharCnt) {
                    _this.viewController.replacePreviousChar('keyboard', e.text, e.replaceCharCnt);
                }
                else {
                    _this.viewController.type('keyboard', e.text);
                }
            }));
            this._toDispose.push(this.textAreaHandler.onCompositionStart(function (e) {
                var lineNumber = e.showAtLineNumber;
                var column = e.showAtColumn;
                var revealPositionEvent = {
                    range: new range_1.Range(lineNumber, column, lineNumber, column),
                    verticalType: editorCommon.VerticalRevealType.Simple,
                    revealHorizontal: true
                };
                _this.context.privateViewEventBus.emit(editorCommon.ViewEventNames.RevealRangeEvent, revealPositionEvent);
                // Find range pixel position
                var visibleRange = _this.viewHelper.visibleRangeForPositionRelativeToEditor(lineNumber, column);
                if (visibleRange) {
                    styleMutator_1.StyleMutator.setTop(_this.textArea.actual, visibleRange.top);
                    styleMutator_1.StyleMutator.setLeft(_this.textArea.actual, _this.contentLeft + visibleRange.left - _this.scrollLeft);
                }
                if (browser.isIE11orEarlier) {
                    styleMutator_1.StyleMutator.setWidth(_this.textArea.actual, _this.contentWidth);
                }
                // Show the textarea
                styleMutator_1.StyleMutator.setHeight(_this.textArea.actual, _this.context.configuration.editor.lineHeight);
                dom.addClass(_this.viewHelper.viewDomNode, 'ime-input');
            }));
            this._toDispose.push(this.textAreaHandler.onCompositionEnd(function (e) {
                _this.textArea.actual.style.height = '';
                _this.textArea.actual.style.width = '';
                styleMutator_1.StyleMutator.setLeft(_this.textArea.actual, 0);
                styleMutator_1.StyleMutator.setTop(_this.textArea.actual, 0);
                dom.removeClass(_this.viewHelper.viewDomNode, 'ime-input');
            }));
            this._toDispose.push(commonEditorConfig_1.GlobalScreenReaderNVDA.onChange(function (value) {
                _this.textAreaHandler.setStrategy(_this._getStrategy());
            }));
            this.context.addEventHandler(this);
        }
        KeyboardHandler.prototype.dispose = function () {
            this.context.removeEventHandler(this);
            this.textAreaHandler.dispose();
            this.textArea.dispose();
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        KeyboardHandler.prototype._getStrategy = function () {
            if (commonEditorConfig_1.GlobalScreenReaderNVDA.getValue()) {
                return textAreaState_1.TextAreaStrategy.NVDA;
            }
            if (this.context.configuration.editor.experimentalScreenReader) {
                return textAreaState_1.TextAreaStrategy.NVDA;
            }
            return textAreaState_1.TextAreaStrategy.IENarrator;
        };
        KeyboardHandler.prototype.focusTextArea = function () {
            this.textAreaHandler.writePlaceholderAndSelectTextAreaSync();
        };
        KeyboardHandler.prototype.onConfigurationChanged = function (e) {
            // Give textarea same font size & line height as editor, for the IME case (when the textarea is visible)
            styleMutator_1.StyleMutator.setFontSize(this.textArea.actual, this.context.configuration.editor.fontSize);
            styleMutator_1.StyleMutator.setLineHeight(this.textArea.actual, this.context.configuration.editor.lineHeight);
            if (e.experimentalScreenReader) {
                this.textAreaHandler.setStrategy(this._getStrategy());
            }
            return false;
        };
        KeyboardHandler.prototype.onScrollChanged = function (e) {
            this.scrollLeft = e.scrollLeft;
            return false;
        };
        KeyboardHandler.prototype.onViewFocusChanged = function (isFocused) {
            this.textAreaHandler.setHasFocus(isFocused);
            return false;
        };
        KeyboardHandler.prototype.onCursorSelectionChanged = function (e) {
            this.textAreaHandler.setCursorSelections(e.selection, e.secondarySelections);
            return false;
        };
        KeyboardHandler.prototype.onCursorPositionChanged = function (e) {
            this.textAreaHandler.setCursorPosition(e.position);
            return false;
        };
        KeyboardHandler.prototype.onLayoutChanged = function (layoutInfo) {
            this.contentLeft = layoutInfo.contentLeft;
            this.contentWidth = layoutInfo.contentWidth;
            return false;
        };
        return KeyboardHandler;
    }(viewEventHandler_1.ViewEventHandler));
    exports.KeyboardHandler = KeyboardHandler;
});

define("vs/languages/css/common/css.contribution", ["require", "exports", 'vs/nls!vs/languages/css/common/css.contribution', 'vs/platform/platform', 'vs/editor/common/modes/modesRegistry', 'vs/platform/configuration/common/configurationRegistry', 'vs/languages/css/common/services/lintRules', 'vs/css!vs/languages/css/common/css-hover'], function (require, exports, nls, Platform, modesRegistry_1, ConfigurationRegistry, lintRules) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    modesRegistry_1.ModesRegistry.registerCompatMode({
        id: 'css',
        extensions: ['.css'],
        aliases: ['CSS', 'css'],
        mimetypes: ['text/css'],
        moduleId: 'vs/languages/css/common/css',
        ctorName: 'CSSMode'
    });
    var configurationRegistry = Platform.Registry.as(ConfigurationRegistry.Extensions.Configuration);
    configurationRegistry.registerConfiguration({
        'id': 'css',
        'order': 20,
        'title': nls.localize(0, null),
        'allOf': [{
                'title': nls.localize(1, null),
                'properties': lintRules.getConfigurationProperties('css')
            }]
    });
});

define("vs/languages/html/common/html.contribution", ["require", "exports", 'vs/editor/common/modes/modesRegistry', 'vs/nls!vs/languages/html/common/html.contribution', 'vs/platform/platform', 'vs/platform/configuration/common/configurationRegistry'], function (require, exports, modesRegistry_1, nls, platform, ConfigurationRegistry) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    modesRegistry_1.ModesRegistry.registerCompatMode({
        id: 'html',
        extensions: ['.html', '.htm', '.shtml', '.mdoc', '.jsp', '.asp', '.aspx', '.jshtm'],
        aliases: ['HTML', 'htm', 'html', 'xhtml'],
        mimetypes: ['text/html', 'text/x-jshtm', 'text/template', 'text/ng-template'],
        moduleId: 'vs/languages/html/common/html',
        ctorName: 'HTMLMode'
    });
    var configurationRegistry = platform.Registry.as(ConfigurationRegistry.Extensions.Configuration);
    configurationRegistry.registerConfiguration({
        'id': 'html',
        'order': 20,
        'type': 'object',
        'title': nls.localize(0, null),
        'properties': {
            'html.format.wrapLineLength': {
                'type': 'integer',
                'default': 120,
                'description': nls.localize(1, null),
            },
            'html.format.unformatted': {
                'type': ['string', 'null'],
                'default': null,
                'description': nls.localize(2, null),
            },
            'html.format.indentInnerHtml': {
                'type': 'boolean',
                'default': false,
                'description': nls.localize(3, null),
            },
            'html.format.preserveNewLines': {
                'type': 'boolean',
                'default': true,
                'description': nls.localize(4, null),
            },
            'html.format.maxPreserveNewLines': {
                'type': ['number', 'null'],
                'default': null,
                'description': nls.localize(5, null),
            },
            'html.format.indentHandlebars': {
                'type': 'boolean',
                'default': false,
                'description': nls.localize(6, null),
            },
            'html.format.endWithNewline': {
                'type': 'boolean',
                'default': false,
                'description': nls.localize(7, null),
            },
            'html.format.extraLiners': {
                'type': ['string', 'null'],
                'default': null,
                'description': nls.localize(8, null),
            },
        }
    });
});

define("vs/languages/less/common/less.contribution", ["require", "exports", 'vs/platform/platform', 'vs/nls!vs/languages/less/common/less.contribution', 'vs/languages/css/common/services/lintRules', 'vs/editor/common/modes/modesRegistry', 'vs/platform/configuration/common/configurationRegistry', 'vs/css!vs/languages/css/common/css-hover'], function (require, exports, platform, nls, lintRules, modesRegistry_1, ConfigurationRegistry) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    modesRegistry_1.ModesRegistry.registerCompatMode({
        id: 'less',
        extensions: ['.less'],
        aliases: ['Less', 'less'],
        mimetypes: ['text/x-less', 'text/less'],
        moduleId: 'vs/languages/less/common/less',
        ctorName: 'LESSMode'
    });
    var configurationRegistry = platform.Registry.as(ConfigurationRegistry.Extensions.Configuration);
    configurationRegistry.registerConfiguration({
        'id': 'less',
        'order': 22,
        'type': 'object',
        'title': nls.localize(0, null),
        'allOf': [{
                'title': nls.localize(1, null),
                'properties': lintRules.getConfigurationProperties('less')
            }]
    });
});

define("vs/languages/markdown/common/markdown.contribution", ["require", "exports", 'vs/nls!vs/languages/markdown/common/markdown.contribution', 'vs/editor/common/modes/modesRegistry', 'vs/platform/configuration/common/configurationRegistry', 'vs/platform/platform'], function (require, exports, nls, modesRegistry_1, ConfigurationRegistry, Platform) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    modesRegistry_1.ModesRegistry.registerCompatMode({
        id: 'markdown',
        extensions: ['.md', '.markdown', '.mdown', '.mkdn', '.mkd', '.mdwn', '.mdtxt', '.mdtext'],
        aliases: ['Markdown', 'markdown'],
        mimetypes: ['text/x-web-markdown'],
        moduleId: 'vs/languages/markdown/common/markdown',
        ctorName: 'MarkdownMode'
    });
    // Configuration
    var configurationRegistry = Platform.Registry.as(ConfigurationRegistry.Extensions.Configuration);
    configurationRegistry.registerConfiguration({
        'id': 'markdown',
        'order': 20,
        'type': 'object',
        'title': nls.localize(0, null),
        'properties': {
            'markdown.styles': {
                'type': 'array',
                'description': nls.localize(1, null),
                'items': {
                    'type': 'string'
                }
            }
        }
    });
});

define("vs/languages/sass/common/sass.contribution", ["require", "exports", 'vs/editor/common/modes/modesRegistry', 'vs/platform/platform', 'vs/nls!vs/languages/sass/common/sass.contribution', 'vs/languages/css/common/services/lintRules', 'vs/platform/configuration/common/configurationRegistry', 'vs/css!vs/languages/css/common/css-hover'], function (require, exports, modesRegistry_1, Platform, nls, LintRules, ConfigurationRegistry) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    modesRegistry_1.ModesRegistry.registerCompatMode({
        id: 'sass',
        extensions: ['.scss'],
        aliases: ['Sass', 'sass', 'scss'],
        mimetypes: ['text/x-scss', 'text/scss'],
        moduleId: 'vs/languages/sass/common/sass',
        ctorName: 'SASSMode'
    });
    var configurationRegistry = Platform.Registry.as(ConfigurationRegistry.Extensions.Configuration);
    configurationRegistry.registerConfiguration({
        'id': 'sass',
        'order': 24,
        'title': nls.localize(0, null),
        'allOf': [{
                'title': nls.localize(1, null),
                'properties': LintRules.getConfigurationProperties('sass')
            }]
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/languages/languages.main", ["require", "exports", 'vs/languages/css/common/css.contribution', 'vs/languages/handlebars/common/handlebars.contribution', 'vs/languages/html/common/html.contribution', 'vs/languages/markdown/common/markdown.contribution', 'vs/languages/php/common/php.contribution', 'vs/languages/razor/common/razor.contribution', 'vs/languages/less/common/less.contribution', 'vs/languages/sass/common/sass.contribution'], function (require, exports) {
    'use strict';
});

define("vs/platform/configuration/common/model", ["require", "exports", 'vs/base/common/objects', 'vs/platform/platform', 'vs/base/common/types', 'vs/base/common/json', './configurationRegistry'], function (require, exports, objects, platform, types, json, configurationRegistry) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.CONFIG_DEFAULT_NAME = 'settings';
    function setNode(root, key, value) {
        var segments = key.split('.');
        var last = segments.pop();
        var curr = root;
        segments.forEach(function (s) {
            var obj = curr[s];
            switch (typeof obj) {
                case 'undefined':
                    obj = curr[s] = Object.create(null);
                    break;
                case 'object':
                    break;
                default:
                    console.log('Conflicting configuration setting: ' + key + ' at ' + s + ' with ' + JSON.stringify(obj));
            }
            curr = obj;
        });
        curr[last] = value;
    }
    function newConfigFile(value) {
        try {
            var root = Object.create(null);
            var contents = json.parse(value) || {};
            for (var key in contents) {
                setNode(root, key, contents[key]);
            }
            return {
                contents: root
            };
        }
        catch (e) {
            return {
                contents: {},
                parseError: e
            };
        }
    }
    exports.newConfigFile = newConfigFile;
    function merge(base, add, overwrite) {
        Object.keys(add).forEach(function (key) {
            if (key in base) {
                if (types.isObject(base[key]) && types.isObject(add[key])) {
                    merge(base[key], add[key], overwrite);
                }
                else if (overwrite) {
                    base[key] = add[key];
                }
            }
            else {
                base[key] = add[key];
            }
        });
    }
    exports.merge = merge;
    function consolidate(configMap) {
        var finalConfig = Object.create(null);
        var parseErrors = [];
        var regexp = /\/(team\.)?([^\.]*)*\.json/;
        // For each config file in .vscode folder
        Object.keys(configMap).forEach(function (configFileName) {
            var config = objects.clone(configMap[configFileName]);
            var matches = regexp.exec(configFileName);
            if (!matches) {
                return;
            }
            // If a file is team.foo.json, it indicates team settings, strip this away
            var isTeamSetting = !!matches[1];
            // Extract the config key from the file name (except for settings.json which is the default)
            var configElement = finalConfig;
            if (matches && matches[2] && matches[2] !== exports.CONFIG_DEFAULT_NAME) {
                // Use the name of the file as top level config section for all settings inside
                var configSection = matches[2];
                var element = configElement[configSection];
                if (!element) {
                    element = Object.create(null);
                    configElement[configSection] = element;
                }
                configElement = element;
            }
            merge(configElement, config.contents, !isTeamSetting /* user settings overrule team settings */);
            if (config.parseError) {
                parseErrors.push(configFileName);
            }
        });
        return {
            contents: finalConfig,
            parseErrors: parseErrors
        };
    }
    exports.consolidate = consolidate;
    // defaults...
    function processDefaultValues(withConfig) {
        var configurations = platform.Registry.as(configurationRegistry.Extensions.Configuration).getConfigurations();
        var visit = function (config, isFirst) {
            withConfig(config, isFirst);
            if (Array.isArray(config.allOf)) {
                config.allOf.forEach(function (c) {
                    visit(c, false);
                });
            }
        };
        configurations.sort(function (c1, c2) {
            if (typeof c1.order !== 'number') {
                return 1;
            }
            if (typeof c2.order !== 'number') {
                return -1;
            }
            return c1.order - c2.order;
        }).forEach(function (config) {
            visit(config, true);
        });
    }
    function getDefaultValues() {
        var ret = Object.create(null);
        var handleConfig = function (config, isTop) {
            if (config.properties) {
                Object.keys(config.properties).forEach(function (key) {
                    var prop = config.properties[key];
                    var value = prop.default;
                    if (types.isUndefined(prop.default)) {
                        value = getDefaultValue(prop.type);
                    }
                    setNode(ret, key, value);
                });
            }
        };
        processDefaultValues(handleConfig);
        return ret;
    }
    exports.getDefaultValues = getDefaultValues;
    function getDefaultValuesContent() {
        var lastEntry = -1;
        var result = [];
        result.push('{');
        var handleConfig = function (config, isTop) {
            if (config.title) {
                if (isTop) {
                    result.push('');
                    result.push('\t//-------- ' + config.title + ' --------');
                }
                else {
                    result.push('\t// ' + config.title);
                }
                result.push('');
            }
            if (config.properties) {
                Object.keys(config.properties).forEach(function (key) {
                    var prop = config.properties[key];
                    var defaultValue = prop.default;
                    if (types.isUndefined(defaultValue)) {
                        defaultValue = getDefaultValue(prop.type);
                    }
                    if (prop.description) {
                        result.push('\t// ' + prop.description);
                    }
                    var valueString = JSON.stringify(defaultValue, null, '\t');
                    if (valueString && (typeof defaultValue === 'object')) {
                        valueString = addIndent(valueString);
                    }
                    if (lastEntry !== -1) {
                        result[lastEntry] += ',';
                    }
                    lastEntry = result.length;
                    result.push('\t' + JSON.stringify(key) + ': ' + valueString);
                    result.push('');
                });
            }
        };
        processDefaultValues(handleConfig);
        result.push('}');
        return result.join('\n');
    }
    exports.getDefaultValuesContent = getDefaultValuesContent;
    function addIndent(str) {
        return str.split('\n').join('\n\t');
    }
    function getDefaultValue(type) {
        var t = Array.isArray(type) ? type[0] : type;
        switch (t) {
            case 'boolean':
                return false;
            case 'integer':
                return 0;
            case 'string':
                return '';
            case 'array':
                return [];
            case 'object':
                return {};
            default:
                return null;
        }
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/configuration/common/configurationService", ["require", "exports", 'vs/base/common/paths', 'vs/base/common/winjs.base', 'vs/base/common/eventEmitter', 'vs/base/common/objects', 'vs/base/common/errors', './model', 'vs/base/common/lifecycle', 'vs/base/common/collections', './configuration', 'vs/platform/files/common/files', './configurationRegistry', 'vs/platform/platform', 'vs/base/common/event'], function (require, exports, paths, winjs, eventEmitter, objects, errors, model, lifecycle, collections, configuration_1, Files, configurationRegistry_1, platform_1, event_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ConfigurationService = (function (_super) {
        __extends(ConfigurationService, _super);
        function ConfigurationService(contextService, eventService, workspaceSettingsRootFolder) {
            var _this = this;
            if (workspaceSettingsRootFolder === void 0) { workspaceSettingsRootFolder = '.vscode'; }
            _super.call(this);
            this.serviceId = configuration_1.IConfigurationService;
            this.contextService = contextService;
            this.eventService = eventService;
            this.workspaceSettingsRootFolder = workspaceSettingsRootFolder;
            this.workspaceFilePathToConfiguration = Object.create(null);
            var unbind = this.eventService.addListener(Files.EventType.FILE_CHANGES, function (events) { return _this.handleFileEvents(events); });
            var subscription = platform_1.Registry.as(configurationRegistry_1.Extensions.Configuration).onDidRegisterConfiguration(function () { return _this.reloadAndEmit(); });
            this.callOnDispose = function () {
                unbind();
                subscription.dispose();
            };
            this.onDidUpdateConfiguration = event_1.fromEventEmitter(this, configuration_1.ConfigurationServiceEventTypes.UPDATED);
        }
        ConfigurationService.prototype.dispose = function () {
            this.callOnDispose = lifecycle.cAll(this.callOnDispose);
            _super.prototype.dispose.call(this);
        };
        ConfigurationService.prototype.loadConfiguration = function (section) {
            if (!this.loadConfigurationPromise) {
                this.loadConfigurationPromise = this.doLoadConfiguration();
            }
            return this.loadConfigurationPromise.then(function (res) {
                var result = section ? res.merged[section] : res.merged;
                var parseErrors = res.consolidated.parseErrors;
                if (res.globals.parseErrors) {
                    parseErrors.push.apply(parseErrors, res.globals.parseErrors);
                }
                if (parseErrors.length > 0) {
                    if (!result) {
                        result = {};
                    }
                    result.$parseErrors = parseErrors;
                }
                return result;
            });
        };
        ConfigurationService.prototype.doLoadConfiguration = function () {
            var _this = this;
            // Load globals
            return this.loadGlobalConfiguration().then(function (globals) {
                // Load workspace locals
                return _this.loadWorkspaceConfiguration().then(function (values) {
                    // Consolidate
                    var consolidated = model.consolidate(values);
                    // Override with workspace locals
                    var merged = objects.mixin(objects.clone(globals.contents), // target: global/default values (but dont modify!)
                    consolidated.contents, // source: workspace configured values
                    true // overwrite
                    );
                    return {
                        merged: merged,
                        consolidated: consolidated,
                        globals: globals
                    };
                });
            });
        };
        ConfigurationService.prototype.loadGlobalConfiguration = function () {
            return winjs.TPromise.as({
                contents: model.getDefaultValues()
            });
        };
        ConfigurationService.prototype.hasWorkspaceConfiguration = function () {
            return !!this.workspaceFilePathToConfiguration['.vscode/' + model.CONFIG_DEFAULT_NAME + '.json'];
        };
        ConfigurationService.prototype.loadWorkspaceConfiguration = function (section) {
            var _this = this;
            // once: when invoked for the first time we fetch *all* json
            // files using the bulk stats and content routes
            if (!this.bulkFetchFromWorkspacePromise) {
                this.bulkFetchFromWorkspacePromise = this.resolveStat(this.contextService.toResource(this.workspaceSettingsRootFolder)).then(function (stat) {
                    if (!stat.isDirectory) {
                        return winjs.TPromise.as([]);
                    }
                    return _this.resolveContents(stat.children.filter(function (stat) { return paths.extname(stat.resource.fsPath) === '.json'; }).map(function (stat) { return stat.resource; }));
                }, function (err) {
                    if (err) {
                        return []; // never fail this call
                    }
                }).then(function (contents) {
                    contents.forEach(function (content) { return _this.workspaceFilePathToConfiguration[_this.contextService.toWorkspaceRelativePath(content.resource)] = winjs.TPromise.as(model.newConfigFile(content.value)); });
                }, errors.onUnexpectedError);
            }
            // on change: join on *all* configuration file promises so that
            // we can merge them into a single configuration object. this
            // happens whenever a config file changes, is deleted, or added
            return this.bulkFetchFromWorkspacePromise.then(function () {
                return winjs.TPromise.join(_this.workspaceFilePathToConfiguration);
            });
        };
        ConfigurationService.prototype.reloadAndEmit = function () {
            var _this = this;
            return this.reloadConfiguration().then(function (config) { return _this.emit(configuration_1.ConfigurationServiceEventTypes.UPDATED, { config: config }); });
        };
        ConfigurationService.prototype.reloadConfiguration = function (section) {
            this.loadConfigurationPromise = null;
            return this.loadConfiguration(section);
        };
        ConfigurationService.prototype.handleFileEvents = function (event) {
            var events = event.changes;
            var affectedByChanges = false;
            for (var i = 0, len = events.length; i < len; i++) {
                var workspacePath = this.contextService.toWorkspaceRelativePath(events[i].resource);
                if (!workspacePath) {
                    continue; // event is not inside workspace
                }
                // Handle case where ".vscode" got deleted
                if (workspacePath === this.workspaceSettingsRootFolder && events[i].type === Files.FileChangeType.DELETED) {
                    this.workspaceFilePathToConfiguration = Object.create(null);
                    affectedByChanges = true;
                }
                // outside my folder or not a *.json file
                if (paths.extname(workspacePath) !== '.json' || !paths.isEqualOrParent(workspacePath, this.workspaceSettingsRootFolder)) {
                    continue;
                }
                // insert 'fetch-promises' for add and update events and
                // remove promises for delete events
                switch (events[i].type) {
                    case Files.FileChangeType.DELETED:
                        affectedByChanges = collections.remove(this.workspaceFilePathToConfiguration, workspacePath);
                        break;
                    case Files.FileChangeType.UPDATED:
                    case Files.FileChangeType.ADDED:
                        this.workspaceFilePathToConfiguration[workspacePath] = this.resolveContent(events[i].resource).then(function (content) { return model.newConfigFile(content.value); }, errors.onUnexpectedError);
                        affectedByChanges = true;
                }
            }
            if (affectedByChanges) {
                this.reloadAndEmit();
            }
        };
        return ConfigurationService;
    }(eventEmitter.EventEmitter));
    exports.ConfigurationService = ConfigurationService;
});

define("vs/platform/extensions/common/abstractExtensionService", ["require", "exports", 'vs/nls!vs/platform/extensions/common/abstractExtensionService', 'vs/base/common/severity', 'vs/base/common/winjs.base', 'vs/platform/extensions/common/extensions', 'vs/platform/extensions/common/extensionsRegistry'], function (require, exports, nls, severity_1, winjs_base_1, extensions_1, extensionsRegistry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var hasOwnProperty = Object.hasOwnProperty;
    var ActivatedExtension = (function () {
        function ActivatedExtension(activationFailed) {
            this.activationFailed = activationFailed;
        }
        return ActivatedExtension;
    }());
    exports.ActivatedExtension = ActivatedExtension;
    var AbstractExtensionService = (function () {
        function AbstractExtensionService(isReadyByDefault) {
            var _this = this;
            this.serviceId = extensions_1.IExtensionService;
            if (isReadyByDefault) {
                this._onReady = winjs_base_1.TPromise.as(true);
                this._onReadyC = function (v) { };
            }
            else {
                this._onReady = new winjs_base_1.TPromise(function (c, e, p) {
                    _this._onReadyC = c;
                }, function () {
                    console.warn('You should really not try to cancel this ready promise!');
                });
            }
            this._activatingExtensions = {};
            this._activatedExtensions = {};
        }
        AbstractExtensionService.prototype._triggerOnReady = function () {
            this._onReadyC(true);
        };
        AbstractExtensionService.prototype.onReady = function () {
            return this._onReady;
        };
        AbstractExtensionService.prototype.getExtensionsStatus = function () {
            return null;
        };
        AbstractExtensionService.prototype.isActivated = function (extensionId) {
            return hasOwnProperty.call(this._activatedExtensions, extensionId);
        };
        AbstractExtensionService.prototype.activateByEvent = function (activationEvent) {
            var _this = this;
            return this._onReady.then(function () {
                extensionsRegistry_1.ExtensionsRegistry.triggerActivationEventListeners(activationEvent);
                var activateExtensions = extensionsRegistry_1.ExtensionsRegistry.getExtensionDescriptionsForActivationEvent(activationEvent);
                return _this._activateExtensions(activateExtensions, 0);
            });
        };
        AbstractExtensionService.prototype.activateById = function (extensionId) {
            var _this = this;
            return this._onReady.then(function () {
                var desc = extensionsRegistry_1.ExtensionsRegistry.getExtensionDescription(extensionId);
                if (!desc) {
                    throw new Error('Extension `' + extensionId + '` is not known');
                }
                return _this._activateExtensions([desc], 0);
            });
        };
        /**
         * Handle semantics related to dependencies for `currentExtension`.
         * semantics: `redExtensions` must wait for `greenExtensions`.
         */
        AbstractExtensionService.prototype._handleActivateRequest = function (currentExtension, greenExtensions, redExtensions) {
            var depIds = (typeof currentExtension.extensionDependencies === 'undefined' ? [] : currentExtension.extensionDependencies);
            var currentExtensionGetsGreenLight = true;
            for (var j = 0, lenJ = depIds.length; j < lenJ; j++) {
                var depId = depIds[j];
                var depDesc = extensionsRegistry_1.ExtensionsRegistry.getExtensionDescription(depId);
                if (!depDesc) {
                    // Error condition 1: unknown dependency
                    this._showMessage(severity_1.default.Error, nls.localize(0, null, depId, currentExtension.id));
                    this._activatedExtensions[currentExtension.id] = this._createFailedExtension();
                    return;
                }
                if (hasOwnProperty.call(this._activatedExtensions, depId)) {
                    var dep = this._activatedExtensions[depId];
                    if (dep.activationFailed) {
                        // Error condition 2: a dependency has already failed activation
                        this._showMessage(severity_1.default.Error, nls.localize(1, null, depId, currentExtension.id));
                        this._activatedExtensions[currentExtension.id] = this._createFailedExtension();
                        return;
                    }
                }
                else {
                    // must first wait for the dependency to activate
                    currentExtensionGetsGreenLight = false;
                    greenExtensions[depId] = depDesc;
                }
            }
            if (currentExtensionGetsGreenLight) {
                greenExtensions[currentExtension.id] = currentExtension;
            }
            else {
                redExtensions.push(currentExtension);
            }
        };
        AbstractExtensionService.prototype._activateExtensions = function (extensionDescriptions, recursionLevel) {
            var _this = this;
            // console.log(recursionLevel, '_activateExtensions: ', extensionDescriptions.map(p => p.id));
            if (extensionDescriptions.length === 0) {
                return winjs_base_1.TPromise.as(void 0);
            }
            extensionDescriptions = extensionDescriptions.filter(function (p) { return !hasOwnProperty.call(_this._activatedExtensions, p.id); });
            if (extensionDescriptions.length === 0) {
                return winjs_base_1.TPromise.as(void 0);
            }
            if (recursionLevel > 10) {
                // More than 10 dependencies deep => most likely a dependency loop
                for (var i = 0, len = extensionDescriptions.length; i < len; i++) {
                    // Error condition 3: dependency loop
                    this._showMessage(severity_1.default.Error, nls.localize(2, null, extensionDescriptions[i].id));
                    this._activatedExtensions[extensionDescriptions[i].id] = this._createFailedExtension();
                }
                return winjs_base_1.TPromise.as(void 0);
            }
            var greenMap = Object.create(null), red = [];
            for (var i = 0, len = extensionDescriptions.length; i < len; i++) {
                this._handleActivateRequest(extensionDescriptions[i], greenMap, red);
            }
            // Make sure no red is also green
            for (var i = 0, len = red.length; i < len; i++) {
                if (greenMap[red[i].id]) {
                    delete greenMap[red[i].id];
                }
            }
            var green = Object.keys(greenMap).map(function (id) { return greenMap[id]; });
            // console.log('greenExtensions: ', green.map(p => p.id));
            // console.log('redExtensions: ', red.map(p => p.id));
            if (red.length === 0) {
                // Finally reached only leafs!
                return winjs_base_1.TPromise.join(green.map(function (p) { return _this._activateExtension(p); })).then(function (_) { return void 0; });
            }
            return this._activateExtensions(green, recursionLevel + 1).then(function (_) {
                return _this._activateExtensions(red, recursionLevel + 1);
            });
        };
        AbstractExtensionService.prototype._activateExtension = function (extensionDescription) {
            var _this = this;
            if (hasOwnProperty.call(this._activatedExtensions, extensionDescription.id)) {
                return winjs_base_1.TPromise.as(void 0);
            }
            if (hasOwnProperty.call(this._activatingExtensions, extensionDescription.id)) {
                return this._activatingExtensions[extensionDescription.id];
            }
            this._activatingExtensions[extensionDescription.id] = this._actualActivateExtension(extensionDescription).then(null, function (err) {
                _this._showMessage(severity_1.default.Error, nls.localize(3, null, extensionDescription.id, err.message));
                console.error('Activating extension `' + extensionDescription.id + '` failed: ', err.message);
                console.log('Here is the error stack: ', err.stack);
                // Treat the extension as being empty
                return _this._createFailedExtension();
            }).then(function (x) {
                _this._activatedExtensions[extensionDescription.id] = x;
                delete _this._activatingExtensions[extensionDescription.id];
            });
            return this._activatingExtensions[extensionDescription.id];
        };
        return AbstractExtensionService;
    }());
    exports.AbstractExtensionService = AbstractExtensionService;
});

define("vs/platform/extensions/node/extensionValidator", ["require", "exports", 'vs/nls!vs/platform/extensions/node/extensionValidator', 'vs/platform/extensions/common/extensionsRegistry', 'semver'], function (require, exports, nls, extensionsRegistry_1, semver_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var VERSION_REGEXP = /^(\^)?((\d+)|x)\.((\d+)|x)\.((\d+)|x)(\-.*)?$/;
    function isValidVersionStr(version) {
        version = version.trim();
        return (version === '*' || VERSION_REGEXP.test(version));
    }
    exports.isValidVersionStr = isValidVersionStr;
    function parseVersion(version) {
        if (!isValidVersionStr(version)) {
            return null;
        }
        version = version.trim();
        if (version === '*') {
            return {
                hasCaret: false,
                majorBase: 0,
                majorMustEqual: false,
                minorBase: 0,
                minorMustEqual: false,
                patchBase: 0,
                patchMustEqual: false,
                preRelease: null
            };
        }
        var m = version.match(VERSION_REGEXP);
        return {
            hasCaret: !!m[1],
            majorBase: m[2] === 'x' ? 0 : parseInt(m[2], 10),
            majorMustEqual: (m[2] === 'x' ? false : true),
            minorBase: m[4] === 'x' ? 0 : parseInt(m[4], 10),
            minorMustEqual: (m[4] === 'x' ? false : true),
            patchBase: m[6] === 'x' ? 0 : parseInt(m[6], 10),
            patchMustEqual: (m[6] === 'x' ? false : true),
            preRelease: m[8] || null
        };
    }
    exports.parseVersion = parseVersion;
    function normalizeVersion(version) {
        if (!version) {
            return null;
        }
        var majorBase = version.majorBase, majorMustEqual = version.majorMustEqual, minorBase = version.minorBase, minorMustEqual = version.minorMustEqual, patchBase = version.patchBase, patchMustEqual = version.patchMustEqual;
        if (version.hasCaret) {
            if (majorBase === 0) {
                patchMustEqual = false;
            }
            else {
                minorMustEqual = false;
                patchMustEqual = false;
            }
        }
        return {
            majorBase: majorBase,
            majorMustEqual: majorMustEqual,
            minorBase: minorBase,
            minorMustEqual: minorMustEqual,
            patchBase: patchBase,
            patchMustEqual: patchMustEqual
        };
    }
    exports.normalizeVersion = normalizeVersion;
    function isValidVersion(_version, _desiredVersion) {
        var version;
        if (typeof _version === 'string') {
            version = normalizeVersion(parseVersion(_version));
        }
        else {
            version = _version;
        }
        var desiredVersion;
        if (typeof _desiredVersion === 'string') {
            desiredVersion = normalizeVersion(parseVersion(_desiredVersion));
        }
        else {
            desiredVersion = _desiredVersion;
        }
        if (!version || !desiredVersion) {
            return false;
        }
        var majorBase = version.majorBase;
        var minorBase = version.minorBase;
        var patchBase = version.patchBase;
        var desiredMajorBase = desiredVersion.majorBase;
        var desiredMinorBase = desiredVersion.minorBase;
        var desiredPatchBase = desiredVersion.patchBase;
        var majorMustEqual = desiredVersion.majorMustEqual;
        var minorMustEqual = desiredVersion.minorMustEqual;
        var patchMustEqual = desiredVersion.patchMustEqual;
        // Anything < 1.0.0 is compatible with >= 1.0.0, except exact matches
        if (majorBase === 1 && desiredMajorBase === 0 && (!majorMustEqual || !minorMustEqual || !patchMustEqual)) {
            desiredMajorBase = 1;
            desiredMinorBase = 0;
            desiredPatchBase = 0;
            majorMustEqual = true;
            minorMustEqual = false;
            patchMustEqual = false;
        }
        if (majorBase < desiredMajorBase) {
            // smaller major version
            return false;
        }
        if (majorBase > desiredMajorBase) {
            // higher major version
            return (!majorMustEqual);
        }
        // at this point, majorBase are equal
        if (minorBase < desiredMinorBase) {
            // smaller minor version
            return false;
        }
        if (minorBase > desiredMinorBase) {
            // higher minor version
            return (!minorMustEqual);
        }
        // at this point, minorBase are equal
        if (patchBase < desiredPatchBase) {
            // smaller patch version
            return false;
        }
        if (patchBase > desiredPatchBase) {
            // higher patch version
            return (!patchMustEqual);
        }
        // at this point, patchBase are equal
        return true;
    }
    exports.isValidVersion = isValidVersion;
    function isValidExtensionVersion(version, extensionDesc, notices) {
        if (extensionDesc.isBuiltin || typeof extensionDesc.main === 'undefined') {
            // No version check for builtin or declarative extensions
            return true;
        }
        var desiredVersion = normalizeVersion(parseVersion(extensionDesc.engines.vscode));
        if (!desiredVersion) {
            notices.push(nls.localize(0, null, extensionDesc.engines.vscode));
            return false;
        }
        // enforce that a breaking API version is specified.
        // for 0.X.Y, that means up to 0.X must be specified
        // otherwise for Z.X.Y, that means Z must be specified
        if (desiredVersion.majorBase === 0) {
            // force that major and minor must be specific
            if (!desiredVersion.majorMustEqual || !desiredVersion.minorMustEqual) {
                notices.push(nls.localize(1, null, extensionDesc.engines.vscode));
                return false;
            }
        }
        else {
            // force that major must be specific
            if (!desiredVersion.majorMustEqual) {
                notices.push(nls.localize(2, null, extensionDesc.engines.vscode));
                return false;
            }
        }
        if (!isValidVersion(version, desiredVersion)) {
            notices.push(nls.localize(3, null, version, extensionDesc.engines.vscode));
            return false;
        }
        return true;
    }
    exports.isValidExtensionVersion = isValidExtensionVersion;
    function isValidExtensionDescription(version, extensionFolderPath, extensionDescription, notices) {
        if (!extensionsRegistry_1.isValidExtensionDescription(extensionFolderPath, extensionDescription, notices)) {
            return false;
        }
        if (!semver_1.valid(extensionDescription.version)) {
            notices.push(nls.localize(4, null));
            return false;
        }
        return isValidExtensionVersion(version, extensionDescription, notices);
    }
    exports.isValidExtensionDescription = isValidExtensionDescription;
});

define("vs/platform/jsonschemas/common/jsonValidationExtensionPoint", ["require", "exports", 'vs/nls!vs/platform/jsonschemas/common/jsonValidationExtensionPoint', 'vs/platform/extensions/common/extensionsRegistry', 'vs/platform/platform', 'vs/base/common/uri', 'vs/platform/jsonschemas/common/jsonContributionRegistry', 'vs/base/common/strings', 'vs/base/common/paths'], function (require, exports, nls, extensionsRegistry_1, platform_1, uri_1, JSONContributionRegistry, strings, paths) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var schemaRegistry = platform_1.Registry.as(JSONContributionRegistry.Extensions.JSONContribution);
    var configurationExtPoint = extensionsRegistry_1.ExtensionsRegistry.registerExtensionPoint('jsonValidation', {
        description: nls.localize(0, null),
        type: 'array',
        defaultSnippets: [{ body: [{ fileMatch: '{{file.json}}', url: '{{url}}' }] }],
        items: {
            type: 'object',
            defaultSnippets: [{ body: { fileMatch: '{{file.json}}', url: '{{url}}' } }],
            properties: {
                fileMatch: {
                    type: 'string',
                    description: nls.localize(1, null),
                },
                url: {
                    description: nls.localize(2, null),
                    type: 'string'
                }
            }
        }
    });
    var JSONValidationExtensionPoint = (function () {
        function JSONValidationExtensionPoint() {
            configurationExtPoint.setHandler(function (extensions) {
                for (var i = 0; i < extensions.length; i++) {
                    var extensionValue = extensions[i].value;
                    var collector = extensions[i].collector;
                    var extensionPath = extensions[i].description.extensionFolderPath;
                    if (!extensionValue || !Array.isArray(extensionValue)) {
                        collector.error(nls.localize(3, null));
                        return;
                    }
                    extensionValue.forEach(function (extension) {
                        if (typeof extension.fileMatch !== 'string') {
                            collector.error(nls.localize(4, null));
                            return;
                        }
                        var uri = extension.url;
                        if (typeof extension.url !== 'string') {
                            collector.error(nls.localize(5, null));
                            return;
                        }
                        if (strings.startsWith(uri, './')) {
                            try {
                                uri = uri_1.default.file(paths.normalize(paths.join(extensionPath, uri))).toString();
                            }
                            catch (e) {
                                collector.error(nls.localize(6, null, e.message));
                            }
                        }
                        else if (!strings.startsWith(uri, 'https:/') && strings.startsWith(uri, 'https:/')) {
                            collector.error(nls.localize(7, null));
                            return;
                        }
                        var fileMatch = extension.fileMatch;
                        if (!strings.startsWith(extension.fileMatch, '/')) {
                            fileMatch = '/' + fileMatch;
                        }
                        schemaRegistry.addSchemaFileAssociation(fileMatch, uri);
                    });
                }
            });
        }
        return JSONValidationExtensionPoint;
    }());
    exports.JSONValidationExtensionPoint = JSONValidationExtensionPoint;
});

define("vs/platform/keybinding/common/keybindingsRegistry", ["require", "exports", 'vs/base/common/keyCodes', 'vs/base/common/platform', 'vs/base/common/types', 'vs/platform/platform'], function (require, exports, keyCodes_1, platform, types_1, platform_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var KeybindingsRegistryImpl = (function () {
        function KeybindingsRegistryImpl() {
            this.WEIGHT = {
                editorCore: function (importance) {
                    if (importance === void 0) { importance = 0; }
                    return 0 + importance;
                },
                editorContrib: function (importance) {
                    if (importance === void 0) { importance = 0; }
                    return 100 + importance;
                },
                workbenchContrib: function (importance) {
                    if (importance === void 0) { importance = 0; }
                    return 200 + importance;
                },
                builtinExtension: function (importance) {
                    if (importance === void 0) { importance = 0; }
                    return 300 + importance;
                },
                externalExtension: function (importance) {
                    if (importance === void 0) { importance = 0; }
                    return 400 + importance;
                }
            };
            this._keybindings = [];
            this._commands = Object.create(null);
        }
        /**
         * Take current platform into account and reduce to primary & secondary.
         */
        KeybindingsRegistryImpl.bindToCurrentPlatform = function (kb) {
            if (platform.isWindows) {
                if (kb && kb.win) {
                    return kb.win;
                }
            }
            else if (platform.isMacintosh) {
                if (kb && kb.mac) {
                    return kb.mac;
                }
            }
            else {
                if (kb && kb.linux) {
                    return kb.linux;
                }
            }
            return kb;
        };
        KeybindingsRegistryImpl.prototype.registerCommandRule = function (rule) {
            var _this = this;
            var actualKb = KeybindingsRegistryImpl.bindToCurrentPlatform(rule);
            if (actualKb && actualKb.primary) {
                this.registerDefaultKeybinding(actualKb.primary, rule.id, rule.weight, 0, rule.context);
            }
            if (actualKb && Array.isArray(actualKb.secondary)) {
                actualKb.secondary.forEach(function (k, i) { return _this.registerDefaultKeybinding(k, rule.id, rule.weight, -i - 1, rule.context); });
            }
        };
        KeybindingsRegistryImpl.prototype.registerCommandDesc = function (desc) {
            this.registerCommandRule(desc);
            // if (_commands[desc.id]) {
            // 	console.warn('Duplicate handler for command: ' + desc.id);
            // }
            // this._commands[desc.id] = desc.handler;
            var handler = desc.handler;
            var description = desc.description || handler.description;
            // add argument validation if rich command metadata is provided
            if (typeof description === 'object') {
                var constraints_1 = [];
                for (var _i = 0, _a = description.args; _i < _a.length; _i++) {
                    var arg = _a[_i];
                    constraints_1.push(arg.constraint);
                }
                handler = function (accesor, args) {
                    types_1.validateConstraints(args, constraints_1);
                    return desc.handler(accesor, args);
                };
            }
            // make sure description is there
            handler.description = description;
            // register handler
            this._commands[desc.id] = handler;
        };
        KeybindingsRegistryImpl.prototype.getCommands = function () {
            return this._commands;
        };
        KeybindingsRegistryImpl.prototype.registerDefaultKeybinding = function (keybinding, commandId, weight1, weight2, context) {
            if (platform.isWindows) {
                if (keyCodes_1.BinaryKeybindings.hasCtrlCmd(keybinding) && !keyCodes_1.BinaryKeybindings.hasShift(keybinding) && keyCodes_1.BinaryKeybindings.hasAlt(keybinding) && !keyCodes_1.BinaryKeybindings.hasWinCtrl(keybinding)) {
                    if (/^[A-Z0-9\[\]\|\;\'\,\.\/\`]$/.test(keyCodes_1.KeyCode.toString(keyCodes_1.BinaryKeybindings.extractKeyCode(keybinding)))) {
                        console.warn('Ctrl+Alt+ keybindings should not be used by default under Windows. Offender: ', keybinding, ' for ', commandId);
                    }
                }
            }
            this._keybindings.push({
                keybinding: keybinding,
                command: commandId,
                context: context,
                weight1: weight1,
                weight2: weight2
            });
        };
        KeybindingsRegistryImpl.prototype.getDefaultKeybindings = function () {
            return this._keybindings;
        };
        return KeybindingsRegistryImpl;
    }());
    exports.KeybindingsRegistry = new KeybindingsRegistryImpl();
    // Define extension point ids
    exports.Extensions = {
        EditorModes: 'platform.keybindingsRegistry'
    };
    platform_1.Registry.add(exports.Extensions.EditorModes, exports.KeybindingsRegistry);
});

define("vs/editor/common/config/config", ["require", "exports", 'vs/base/common/keyCodes', 'vs/platform/editor/common/editor', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/editor/common/editorCommon', 'vs/editor/common/services/codeEditorService'], function (require, exports, keyCodes_1, editor_1, keybindingService_1, keybindingsRegistry_1, editorCommon, codeEditorService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var H = editorCommon.Handler;
    function findFocusedEditor(commandId, accessor, args, complain) {
        var codeEditorService = accessor.get(codeEditorService_1.ICodeEditorService);
        var editorId = args.context.editorId;
        if (!editorId) {
            if (complain) {
                console.warn('Cannot execute ' + commandId + ' because no editor is focused.');
            }
            return null;
        }
        var editor = codeEditorService.getCodeEditor(editorId);
        if (!editor) {
            if (complain) {
                console.warn('Cannot execute ' + commandId + ' because editor `' + editorId + '` could not be found.');
            }
            return null;
        }
        return editor;
    }
    exports.findFocusedEditor = findFocusedEditor;
    function withCodeEditorFromCommandHandler(commandId, accessor, args, callback) {
        var editor = findFocusedEditor(commandId, accessor, args, true);
        if (editor) {
            callback(editor);
        }
    }
    exports.withCodeEditorFromCommandHandler = withCodeEditorFromCommandHandler;
    function getActiveEditor(accessor) {
        var editorService = accessor.get(editor_1.IEditorService);
        var activeEditor = editorService.getActiveEditor && editorService.getActiveEditor();
        if (activeEditor) {
            var editor = activeEditor.getControl();
            // Substitute for (editor instanceof ICodeEditor)
            if (editor && typeof editor.getEditorType === 'function') {
                var codeEditor = editor;
                return codeEditor;
            }
        }
        return null;
    }
    exports.getActiveEditor = getActiveEditor;
    function triggerEditorHandler(handlerId, accessor, args) {
        withCodeEditorFromCommandHandler(handlerId, accessor, args, function (editor) {
            editor.trigger('keyboard', handlerId, args);
        });
    }
    function registerCoreCommand(handlerId, kb, weight, context) {
        if (weight === void 0) { weight = keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorCore(); }
        var desc = {
            id: handlerId,
            handler: triggerEditorHandler.bind(null, handlerId),
            weight: weight,
            context: (context ? context : keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS)),
            primary: kb.primary,
            secondary: kb.secondary,
            win: kb.win,
            mac: kb.mac,
            linux: kb.linux
        };
        keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc(desc);
    }
    function registerCoreDispatchCommand2(handlerId) {
        var desc = {
            id: handlerId,
            handler: triggerEditorHandler.bind(null, handlerId),
            weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorCore(),
            context: null,
            primary: 0
        };
        keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc(desc);
        var desc2 = {
            id: 'default:' + handlerId,
            handler: function (accessor, args) {
                withCodeEditorFromCommandHandler(handlerId, accessor, args, function (editor) {
                    editor.trigger('keyboard', handlerId, args[0]);
                });
            },
            weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorCore(),
            context: null,
            primary: 0
        };
        keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc(desc2);
    }
    registerCoreDispatchCommand2(H.Type);
    registerCoreDispatchCommand2(H.ReplacePreviousChar);
    registerCoreDispatchCommand2(H.Paste);
    registerCoreDispatchCommand2(H.Cut);
    function getMacWordNavigationKB(shift, key) {
        // For macs, word navigation is based on the alt modifier
        if (shift) {
            return keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | key;
        }
        else {
            return keyCodes_1.KeyMod.Alt | key;
        }
    }
    function getWordNavigationKB(shift, key) {
        // Normally word navigation is based on the ctrl modifier
        if (shift) {
            return keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | key;
        }
        else {
            return keyCodes_1.KeyMod.CtrlCmd | key;
        }
    }
    // https://support.apple.com/en-gb/HT201236
    // [ADDED] Control-H					Delete the character to the left of the insertion point. Or use Delete.
    // [ADDED] Control-D					Delete the character to the right of the insertion point. Or use Fn-Delete.
    // [ADDED] Control-K					Delete the text between the insertion point and the end of the line or paragraph.
    // [ADDED] Command–Up Arrow				Move the insertion point to the beginning of the document.
    // [ADDED] Command–Down Arrow			Move the insertion point to the end of the document.
    // [ADDED] Command–Left Arrow			Move the insertion point to the beginning of the current line.
    // [ADDED] Command–Right Arrow			Move the insertion point to the end of the current line.
    // [ADDED] Option–Left Arrow			Move the insertion point to the beginning of the previous word.
    // [ADDED] Option–Right Arrow			Move the insertion point to the end of the next word.
    // [ADDED] Command–Shift–Up Arrow		Select the text between the insertion point and the beginning of the document.
    // [ADDED] Command–Shift–Down Arrow		Select the text between the insertion point and the end of the document.
    // [ADDED] Command–Shift–Left Arrow		Select the text between the insertion point and the beginning of the current line.
    // [ADDED] Command–Shift–Right Arrow	Select the text between the insertion point and the end of the current line.
    // [USED BY DUPLICATE LINES] Shift–Option–Up Arrow		Extend text selection to the beginning of the current paragraph, then to the beginning of the following paragraph if pressed again.
    // [USED BY DUPLICATE LINES] Shift–Option–Down Arrow	Extend text selection to the end of the current paragraph, then to the end of the following paragraph if pressed again.
    // [ADDED] Shift–Option–Left Arrow		Extend text selection to the beginning of the current word, then to the beginning of the following word if pressed again.
    // [ADDED] Shift–Option–Right Arrow		Extend text selection to the end of the current word, then to the end of the following word if pressed again.
    // [ADDED] Control-A					Move to the beginning of the line or paragraph.
    // [ADDED] Control-E					Move to the end of a line or paragraph.
    // [ADDED] Control-F					Move one character forward.
    // [ADDED] Control-B					Move one character backward.
    //Control-L								Center the cursor or selection in the visible area.
    // [ADDED] Control-P					Move up one line.
    // [ADDED] Control-N					Move down one line.
    // [ADDED] Control-O					Insert a new line after the insertion point.
    //Control-T								Swap the character behind the insertion point with the character in front of the insertion point.
    // Unconfirmed????
    //	Config.addKeyBinding(editorCommon.Handler.CursorPageDown,		KeyMod.WinCtrl | KeyCode.KEY_V);
    // OS X built in commands
    // Control+y => yank
    // [ADDED] Command+backspace => Delete to Hard BOL
    // [ADDED] Command+delete => Delete to Hard EOL
    // [ADDED] Control+k => Delete to Hard EOL
    // Control+l => show_at_center
    // Control+Command+d => noop
    // Control+Command+shift+d => noop
    registerCoreCommand(H.CursorLeft, {
        primary: keyCodes_1.KeyCode.LeftArrow,
        mac: { primary: keyCodes_1.KeyCode.LeftArrow, secondary: [keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_B] }
    });
    registerCoreCommand(H.CursorLeftSelect, {
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.LeftArrow
    });
    registerCoreCommand(H.CursorRight, {
        primary: keyCodes_1.KeyCode.RightArrow,
        mac: { primary: keyCodes_1.KeyCode.RightArrow, secondary: [keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_F] }
    });
    registerCoreCommand(H.CursorRightSelect, {
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.RightArrow
    });
    registerCoreCommand(H.CursorUp, {
        primary: keyCodes_1.KeyCode.UpArrow,
        mac: { primary: keyCodes_1.KeyCode.UpArrow, secondary: [keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_P] }
    });
    registerCoreCommand(H.CursorUpSelect, {
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.UpArrow,
        secondary: [getWordNavigationKB(true, keyCodes_1.KeyCode.UpArrow)],
        mac: { primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.UpArrow },
        linux: { primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.UpArrow }
    });
    registerCoreCommand(H.CursorDown, {
        primary: keyCodes_1.KeyCode.DownArrow,
        mac: { primary: keyCodes_1.KeyCode.DownArrow, secondary: [keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_N] }
    });
    registerCoreCommand(H.CursorDownSelect, {
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.DownArrow,
        secondary: [getWordNavigationKB(true, keyCodes_1.KeyCode.DownArrow)],
        mac: { primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.DownArrow },
        linux: { primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.DownArrow }
    });
    registerCoreCommand(H.CursorPageUp, {
        primary: keyCodes_1.KeyCode.PageUp
    });
    registerCoreCommand(H.CursorPageUpSelect, {
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.PageUp
    });
    registerCoreCommand(H.CursorPageDown, {
        primary: keyCodes_1.KeyCode.PageDown
    });
    registerCoreCommand(H.CursorPageDownSelect, {
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.PageDown
    });
    registerCoreCommand(H.CursorHome, {
        primary: keyCodes_1.KeyCode.Home,
        mac: { primary: keyCodes_1.KeyCode.Home, secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.LeftArrow, keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_A] }
    });
    registerCoreCommand(H.CursorHomeSelect, {
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Home,
        mac: { primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Home, secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.LeftArrow] }
    });
    registerCoreCommand(H.CursorEnd, {
        primary: keyCodes_1.KeyCode.End,
        mac: { primary: keyCodes_1.KeyCode.End, secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.RightArrow, keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_E] }
    });
    registerCoreCommand(H.CursorEndSelect, {
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.End,
        mac: { primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.End, secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.RightArrow] }
    });
    registerCoreCommand(H.ExpandLineSelection, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_I
    });
    registerCoreCommand(H.ScrollLineUp, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.UpArrow,
        mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.PageUp }
    });
    registerCoreCommand(H.ScrollLineDown, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.DownArrow,
        mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.PageDown }
    });
    registerCoreCommand(H.ScrollPageUp, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.PageUp
    });
    registerCoreCommand(H.ScrollPageDown, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.PageDown
    });
    registerCoreCommand(H.CursorColumnSelectLeft, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.LeftArrow,
        linux: { primary: 0 }
    });
    registerCoreCommand(H.CursorColumnSelectRight, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.RightArrow,
        linux: { primary: 0 }
    });
    registerCoreCommand(H.CursorColumnSelectUp, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.UpArrow,
        linux: { primary: 0 }
    });
    registerCoreCommand(H.CursorColumnSelectPageUp, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.PageUp,
        linux: { primary: 0 }
    });
    registerCoreCommand(H.CursorColumnSelectDown, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.DownArrow,
        linux: { primary: 0 }
    });
    registerCoreCommand(H.CursorColumnSelectPageDown, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.PageDown,
        linux: { primary: 0 }
    });
    registerCoreCommand(H.Tab, {
        primary: keyCodes_1.KeyCode.Tab
    }, keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorCore(), keybindingService_1.KbExpr.and(keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS), keybindingService_1.KbExpr.not(editorCommon.KEYBINDING_CONTEXT_EDITOR_TAB_MOVES_FOCUS)));
    registerCoreCommand(H.Outdent, {
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Tab
    }, keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorCore(), keybindingService_1.KbExpr.and(keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS), keybindingService_1.KbExpr.not(editorCommon.KEYBINDING_CONTEXT_EDITOR_TAB_MOVES_FOCUS)));
    registerCoreCommand(H.DeleteLeft, {
        primary: keyCodes_1.KeyCode.Backspace,
        secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Backspace],
        mac: { primary: keyCodes_1.KeyCode.Backspace, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Backspace, keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_H, keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.Backspace] }
    });
    registerCoreCommand(H.DeleteRight, {
        primary: keyCodes_1.KeyCode.Delete,
        mac: { primary: keyCodes_1.KeyCode.Delete, secondary: [keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_D, keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.Delete] }
    });
    registerCoreCommand(H.DeleteAllLeft, {
        primary: null,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.Backspace }
    });
    registerCoreCommand(H.DeleteAllRight, {
        primary: null,
        mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_K, secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.Delete] }
    });
    function registerWordCommand(handlerId, shift, key) {
        registerCoreCommand(handlerId, {
            primary: getWordNavigationKB(shift, key),
            mac: { primary: getMacWordNavigationKB(shift, key) }
        });
    }
    registerWordCommand(H.CursorWordStartLeft, false, keyCodes_1.KeyCode.LeftArrow);
    registerCoreCommand(H.CursorWordEndLeft, { primary: 0 });
    registerCoreCommand(H.CursorWordLeft, { primary: 0 });
    registerWordCommand(H.CursorWordStartLeftSelect, true, keyCodes_1.KeyCode.LeftArrow);
    registerCoreCommand(H.CursorWordEndLeftSelect, { primary: 0 });
    registerCoreCommand(H.CursorWordLeftSelect, { primary: 0 });
    registerWordCommand(H.CursorWordEndRight, false, keyCodes_1.KeyCode.RightArrow);
    registerCoreCommand(H.CursorWordStartRight, { primary: 0 });
    registerCoreCommand(H.CursorWordRight, { primary: 0 });
    registerWordCommand(H.CursorWordEndRightSelect, true, keyCodes_1.KeyCode.RightArrow);
    registerCoreCommand(H.CursorWordStartRightSelect, { primary: 0 });
    registerCoreCommand(H.CursorWordRightSelect, { primary: 0 });
    registerWordCommand(H.DeleteWordLeft, false, keyCodes_1.KeyCode.Backspace);
    registerCoreCommand(H.DeleteWordStartLeft, { primary: 0 });
    registerCoreCommand(H.DeleteWordEndLeft, { primary: 0 });
    registerWordCommand(H.DeleteWordRight, false, keyCodes_1.KeyCode.Delete);
    registerCoreCommand(H.DeleteWordStartRight, { primary: 0 });
    registerCoreCommand(H.DeleteWordEndRight, { primary: 0 });
    registerCoreCommand(H.CancelSelection, {
        primary: keyCodes_1.KeyCode.Escape,
        secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape]
    }, keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorCore(), keybindingService_1.KbExpr.and(keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS), keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_HAS_NON_EMPTY_SELECTION)));
    registerCoreCommand(H.RemoveSecondaryCursors, {
        primary: keyCodes_1.KeyCode.Escape,
        secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape]
    }, keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorCore(1), keybindingService_1.KbExpr.and(keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS), keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_HAS_MULTIPLE_SELECTIONS)));
    registerCoreCommand(H.CursorTop, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.Home,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.UpArrow }
    });
    registerCoreCommand(H.CursorTopSelect, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Home,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.UpArrow }
    });
    registerCoreCommand(H.CursorBottom, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.End,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.DownArrow }
    });
    registerCoreCommand(H.CursorBottomSelect, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.End,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.DownArrow }
    });
    registerCoreCommand(H.LineBreakInsert, {
        primary: null,
        mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_O }
    });
    registerCoreCommand(H.Undo, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_Z
    });
    registerCoreCommand(H.CursorUndo, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_U
    });
    registerCoreCommand(H.Redo, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_Y,
        secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_Z],
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_Z }
    });
    function selectAll(accessor, args) {
        var HANDLER = editorCommon.Handler.SelectAll;
        // If editor text focus
        if (args.context[editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS]) {
            var focusedEditor = findFocusedEditor(HANDLER, accessor, args, false);
            if (focusedEditor) {
                focusedEditor.trigger('keyboard', HANDLER, args);
                return;
            }
        }
        // Ignore this action when user is focussed on an element that allows for entering text
        var activeElement = document.activeElement;
        if (activeElement && ['input', 'textarea'].indexOf(activeElement.tagName.toLowerCase()) >= 0) {
            activeElement.select();
            return;
        }
        // Redirecting to last active editor
        var activeEditor = getActiveEditor(accessor);
        if (activeEditor) {
            activeEditor.trigger('keyboard', HANDLER, args);
            return;
        }
    }
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
        id: 'editor.action.selectAll',
        handler: selectAll,
        weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorCore(),
        context: null,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_A
    });
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/keybinding/browser/keybindingServiceImpl", ["require", "exports", 'vs/nls!vs/platform/keybinding/browser/keybindingServiceImpl', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/severity', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/base/browser/keyboardEvent', 'vs/platform/keybinding/common/keybindingResolver', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/css!./keybindings'], function (require, exports, nls, keyCodes_1, lifecycle_1, severity_1, winjs_base_1, dom, keyboardEvent_1, keybindingResolver_1, keybindingService_1, keybindingsRegistry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var KEYBINDING_CONTEXT_ATTR = 'data-keybinding-context';
    var KeybindingContext = (function () {
        function KeybindingContext(id, parent) {
            this._id = id;
            this._parent = parent;
            this._value = Object.create(null);
            this._value['_contextId'] = id;
        }
        KeybindingContext.prototype.setValue = function (key, value) {
            //		console.log('SET ' + key + ' = ' + value + ' ON ' + this._id);
            this._value[key] = value;
        };
        KeybindingContext.prototype.removeValue = function (key) {
            //		console.log('REMOVE ' + key + ' FROM ' + this._id);
            delete this._value[key];
        };
        KeybindingContext.prototype.fillInContext = function (bucket) {
            if (this._parent) {
                this._parent.fillInContext(bucket);
            }
            for (var key in this._value) {
                bucket[key] = this._value[key];
            }
        };
        return KeybindingContext;
    }());
    exports.KeybindingContext = KeybindingContext;
    var ConfigurationContext = (function () {
        function ConfigurationContext(configurationService) {
            var _this = this;
            this._subscription = configurationService.onDidUpdateConfiguration(function (e) { return _this._updateConfigurationContext(e.config); });
            configurationService.loadConfiguration().then(function (config) { return _this._updateConfigurationContext(config); });
        }
        ConfigurationContext.prototype.dispose = function () {
            this._subscription.dispose();
        };
        ConfigurationContext.prototype._updateConfigurationContext = function (config) {
            var _this = this;
            this._values = Object.create(null);
            var walk = function (obj, keys) {
                for (var key in obj) {
                    if (Object.prototype.hasOwnProperty.call(obj, key)) {
                        keys.push(key);
                        var value = obj[key];
                        if (typeof value === 'boolean') {
                            _this._values[keys.join('.')] = value;
                        }
                        else if (typeof value === 'object') {
                            walk(value, keys);
                        }
                        keys.pop();
                    }
                }
            };
            walk(config, ['config']);
        };
        ConfigurationContext.prototype.fillInContext = function (bucket) {
            if (this._values) {
                for (var key in this._values) {
                    bucket[key] = this._values[key];
                }
            }
        };
        return ConfigurationContext;
    }());
    exports.ConfigurationContext = ConfigurationContext;
    var KeybindingContextKey = (function () {
        function KeybindingContextKey(parent, key, defaultValue) {
            this._parent = parent;
            this._key = key;
            this._defaultValue = defaultValue;
            if (typeof this._defaultValue !== 'undefined') {
                this._parent.setContext(this._key, this._defaultValue);
            }
        }
        KeybindingContextKey.prototype.set = function (value) {
            this._parent.setContext(this._key, value);
        };
        KeybindingContextKey.prototype.reset = function () {
            if (typeof this._defaultValue === 'undefined') {
                this._parent.removeContext(this._key);
            }
            else {
                this._parent.setContext(this._key, this._defaultValue);
            }
        };
        return KeybindingContextKey;
    }());
    var AbstractKeybindingService = (function () {
        function AbstractKeybindingService(myContextId) {
            this.serviceId = keybindingService_1.IKeybindingService;
            this._myContextId = myContextId;
            this._instantiationService = null;
            this._messageService = null;
        }
        AbstractKeybindingService.prototype.setMessageService = function (messageService) {
            this._messageService = messageService;
        };
        AbstractKeybindingService.prototype.createKey = function (key, defaultValue) {
            return new KeybindingContextKey(this, key, defaultValue);
        };
        AbstractKeybindingService.prototype.setInstantiationService = function (instantiationService) {
            this._instantiationService = instantiationService;
        };
        AbstractKeybindingService.prototype.createScoped = function (domNode) {
            return new ScopedKeybindingService(this, domNode);
        };
        AbstractKeybindingService.prototype.setContext = function (key, value) {
            this.getContext(this._myContextId).setValue(key, value);
        };
        AbstractKeybindingService.prototype.removeContext = function (key) {
            this.getContext(this._myContextId).removeValue(key);
        };
        return AbstractKeybindingService;
    }());
    exports.AbstractKeybindingService = AbstractKeybindingService;
    var KeybindingService = (function (_super) {
        __extends(KeybindingService, _super);
        function KeybindingService(configurationService) {
            _super.call(this, 0);
            this._toDispose = [];
            this._lastContextId = 0;
            this._contexts = Object.create(null);
            this._contexts[String(this._myContextId)] = new KeybindingContext(this._myContextId, null);
            this._cachedResolver = null;
            this._firstTimeComputingResolver = true;
            this._currentChord = 0;
            this._currentChordStatusMessage = null;
            this._configurationContext = new ConfigurationContext(configurationService);
            this._toDispose.push(this._configurationContext);
        }
        KeybindingService.prototype._beginListening = function (domNode) {
            var _this = this;
            this._toDispose.push(dom.addDisposableListener(domNode, dom.EventType.KEY_DOWN, function (e) {
                var keyEvent = new keyboardEvent_1.StandardKeyboardEvent(e);
                _this._dispatch(keyEvent);
            }));
        };
        KeybindingService.prototype._getResolver = function () {
            if (!this._cachedResolver) {
                this._cachedResolver = new keybindingResolver_1.KeybindingResolver(keybindingsRegistry_1.KeybindingsRegistry.getDefaultKeybindings(), this._getExtraKeybindings(this._firstTimeComputingResolver));
                this._firstTimeComputingResolver = false;
            }
            return this._cachedResolver;
        };
        KeybindingService.prototype.dispose = function () {
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        KeybindingService.prototype.getLabelFor = function (keybinding) {
            return keybinding._toUSLabel();
        };
        KeybindingService.prototype.getHTMLLabelFor = function (keybinding) {
            return keybinding._toUSHTMLLabel();
        };
        KeybindingService.prototype.getAriaLabelFor = function (keybinding) {
            return keybinding._toUSAriaLabel();
        };
        KeybindingService.prototype.getElectronAcceleratorFor = function (keybinding) {
            return keybinding._toElectronAccelerator();
        };
        KeybindingService.prototype.updateResolver = function () {
            this._cachedResolver = null;
        };
        KeybindingService.prototype._getExtraKeybindings = function (isFirstTime) {
            return [];
        };
        KeybindingService.prototype.getDefaultKeybindings = function () {
            return this._getResolver().getDefaultKeybindings() + '\n\n' + this._getAllCommandsAsComment();
        };
        KeybindingService.prototype.customKeybindingsCount = function () {
            return 0;
        };
        KeybindingService.prototype.lookupKeybindings = function (commandId) {
            return this._getResolver().lookupKeybinding(commandId);
        };
        KeybindingService.prototype._getAllCommandsAsComment = function () {
            var boundCommands = this._getResolver().getDefaultBoundCommands();
            var unboundCommands = Object.keys(keybindingsRegistry_1.KeybindingsRegistry.getCommands()).filter(function (commandId) { return commandId[0] !== '_' && !boundCommands[commandId]; });
            unboundCommands.sort();
            var pretty = unboundCommands.join('\n// - ');
            return '// ' + nls.localize(0, null) + '\n// - ' + pretty;
        };
        KeybindingService.prototype._getCommandHandler = function (commandId) {
            return keybindingsRegistry_1.KeybindingsRegistry.getCommands()[commandId];
        };
        KeybindingService.prototype._dispatch = function (e) {
            var _this = this;
            var isModifierKey = (e.keyCode === keyCodes_1.KeyCode.Ctrl || e.keyCode === keyCodes_1.KeyCode.Shift || e.keyCode === keyCodes_1.KeyCode.Alt || e.keyCode === keyCodes_1.KeyCode.Meta);
            if (isModifierKey) {
                return;
            }
            var contextValue = Object.create(null);
            this.getContext(this._findContextAttr(e.target)).fillInContext(contextValue);
            this._configurationContext.fillInContext(contextValue);
            // console.log(JSON.stringify(contextValue, null, '\t'));
            var resolveResult = this._getResolver().resolve(contextValue, this._currentChord, e.asKeybinding());
            if (resolveResult && resolveResult.enterChord) {
                e.preventDefault();
                this._currentChord = resolveResult.enterChord;
                if (this._messageService) {
                    var firstPartLabel = this.getLabelFor(new keyCodes_1.Keybinding(this._currentChord));
                    this._currentChordStatusMessage = this._messageService.setStatusMessage(nls.localize(1, null, firstPartLabel));
                }
                return;
            }
            if (this._messageService && this._currentChord) {
                if (!resolveResult || !resolveResult.commandId) {
                    var firstPartLabel = this.getLabelFor(new keyCodes_1.Keybinding(this._currentChord));
                    var chordPartLabel = this.getLabelFor(new keyCodes_1.Keybinding(e.asKeybinding()));
                    this._messageService.setStatusMessage(nls.localize(2, null, firstPartLabel, chordPartLabel), 10 * 1000 /* 10s */);
                    e.preventDefault();
                }
            }
            if (this._currentChordStatusMessage) {
                this._currentChordStatusMessage.dispose();
                this._currentChordStatusMessage = null;
            }
            this._currentChord = 0;
            if (resolveResult && resolveResult.commandId) {
                if (!/^\^/.test(resolveResult.commandId)) {
                    e.preventDefault();
                }
                var commandId = resolveResult.commandId.replace(/^\^/, '');
                this._invokeHandler(commandId, { context: contextValue }).done(undefined, function (err) {
                    _this._messageService.show(severity_1.default.Warning, err);
                });
            }
        };
        KeybindingService.prototype._invokeHandler = function (commandId, args) {
            var handler = this._getCommandHandler(commandId);
            if (!handler) {
                return winjs_base_1.TPromise.wrapError(new Error("No handler found for the command: '" + commandId + "'. An extension might be missing an activation event."));
            }
            try {
                var result = this._instantiationService.invokeFunction(handler, args);
                return winjs_base_1.TPromise.as(result);
            }
            catch (err) {
                return winjs_base_1.TPromise.wrapError(err);
            }
        };
        KeybindingService.prototype._findContextAttr = function (domNode) {
            while (domNode) {
                if (domNode.hasAttribute(KEYBINDING_CONTEXT_ATTR)) {
                    return parseInt(domNode.getAttribute(KEYBINDING_CONTEXT_ATTR), 10);
                }
                domNode = domNode.parentElement;
            }
            return this._myContextId;
        };
        KeybindingService.prototype.getContext = function (contextId) {
            return this._contexts[String(contextId)];
        };
        KeybindingService.prototype.createChildContext = function (parentContextId) {
            if (parentContextId === void 0) { parentContextId = this._myContextId; }
            var id = (++this._lastContextId);
            this._contexts[String(id)] = new KeybindingContext(id, this.getContext(parentContextId));
            return id;
        };
        KeybindingService.prototype.disposeContext = function (contextId) {
            delete this._contexts[String(contextId)];
        };
        KeybindingService.prototype.executeCommand = function (commandId, args) {
            if (args === void 0) { args = {}; }
            if (!args.context) {
                args.context = Object.create(null);
                this.getContext(this._findContextAttr(document.activeElement)).fillInContext(args.context);
                this._configurationContext.fillInContext(args.context);
            }
            if (commandId === keybindingService_1.SET_CONTEXT_COMMAND_ID) {
                var contextKey = String(args[0]);
                var contextValue = args[1];
                this.setContext(contextKey, contextValue);
                return winjs_base_1.TPromise.as(null);
            }
            return this._invokeHandler(commandId, args);
        };
        return KeybindingService;
    }(AbstractKeybindingService));
    exports.KeybindingService = KeybindingService;
    var ScopedKeybindingService = (function (_super) {
        __extends(ScopedKeybindingService, _super);
        function ScopedKeybindingService(parent, domNode) {
            _super.call(this, parent.createChildContext());
            this._parent = parent;
            this._domNode = domNode;
            this._domNode.setAttribute(KEYBINDING_CONTEXT_ATTR, String(this._myContextId));
        }
        ScopedKeybindingService.prototype.dispose = function () {
            this._parent.disposeContext(this._myContextId);
            this._domNode.removeAttribute(KEYBINDING_CONTEXT_ATTR);
        };
        ScopedKeybindingService.prototype.getLabelFor = function (keybinding) {
            return this._parent.getLabelFor(keybinding);
        };
        ScopedKeybindingService.prototype.getHTMLLabelFor = function (keybinding) {
            return this._parent.getHTMLLabelFor(keybinding);
        };
        ScopedKeybindingService.prototype.getAriaLabelFor = function (keybinding) {
            return this._parent.getAriaLabelFor(keybinding);
        };
        ScopedKeybindingService.prototype.getElectronAcceleratorFor = function (keybinding) {
            return this._parent.getElectronAcceleratorFor(keybinding);
        };
        ScopedKeybindingService.prototype.getDefaultKeybindings = function () {
            return this._parent.getDefaultKeybindings();
        };
        ScopedKeybindingService.prototype.customKeybindingsCount = function () {
            return this._parent.customKeybindingsCount();
        };
        ScopedKeybindingService.prototype.lookupKeybindings = function (commandId) {
            return this._parent.lookupKeybindings(commandId);
        };
        ScopedKeybindingService.prototype.getContext = function (contextId) {
            return this._parent.getContext(contextId);
        };
        ScopedKeybindingService.prototype.createChildContext = function (parentContextId) {
            if (parentContextId === void 0) { parentContextId = this._myContextId; }
            return this._parent.createChildContext(parentContextId);
        };
        ScopedKeybindingService.prototype.disposeContext = function (contextId) {
            this._parent.disposeContext(contextId);
        };
        ScopedKeybindingService.prototype.executeCommand = function (commandId, args) {
            return this._parent.executeCommand(commandId, args);
        };
        return ScopedKeybindingService;
    }(AbstractKeybindingService));
});

define("vs/platform/progress/common/progress", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IProgressService = instantiation_1.createDecorator('progressService');
});

define("vs/platform/request/common/request", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IRequestService = instantiation_1.createDecorator('requestService');
});

define("vs/platform/request/common/baseRequestService", ["require", "exports", 'vs/base/common/uri', 'vs/base/common/strings', 'vs/base/common/timer', 'vs/base/common/async', 'vs/base/common/winjs.base', 'vs/base/common/objects', 'vs/platform/request/common/request'], function (require, exports, uri_1, strings, Timer, Async, winjs, objects, request_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Simple IRequestService implementation to allow sharing of this service implementation
     * between different layers of the platform.
     */
    var BaseRequestService = (function () {
        function BaseRequestService(contextService, telemetryService) {
            this.serviceId = request_1.IRequestService;
            var workspaceUri = null;
            var workspace = contextService.getWorkspace();
            this._serviceMap = workspace || Object.create(null);
            this._telemetryService = telemetryService;
            if (workspace) {
                workspaceUri = strings.rtrim(workspace.resource.toString(), '/') + '/';
            }
            this.computeOrigin(workspaceUri);
        }
        BaseRequestService.prototype.computeOrigin = function (workspaceUri) {
            if (workspaceUri) {
                // Find root server URL from configuration
                this._origin = workspaceUri;
                var urlPath = uri_1.default.parse(this._origin).path;
                if (urlPath && urlPath.length > 0) {
                    this._origin = this._origin.substring(0, this._origin.length - urlPath.length + 1);
                }
                if (!strings.endsWith(this._origin, '/')) {
                    this._origin += '/';
                }
            }
            else {
                this._origin = '/'; // Configuration not provided, fallback to default
            }
        };
        BaseRequestService.prototype.makeCrossOriginRequest = function (options) {
            return null;
        };
        BaseRequestService.prototype.makeRequest = function (options) {
            var timer = Timer.nullEvent;
            var isXhrRequestCORS = false;
            var url = options.url;
            if (!url) {
                throw new Error('IRequestService.makeRequest: Url is required');
            }
            if ((strings.startsWith(url, 'http://') || strings.startsWith(url, 'https://')) && this._origin && !strings.startsWith(url, this._origin)) {
                var coPromise = this.makeCrossOriginRequest(options);
                if (coPromise) {
                    return coPromise;
                }
                isXhrRequestCORS = true;
            }
            var xhrOptions = options;
            if (!isXhrRequestCORS) {
                var additionalHeaders = {};
                if (this._telemetryService) {
                    additionalHeaders['X-TelemetrySession'] = this._telemetryService.getSessionId();
                }
                additionalHeaders['X-Requested-With'] = 'XMLHttpRequest';
                xhrOptions.headers = objects.mixin(xhrOptions.headers, additionalHeaders);
            }
            if (options.timeout) {
                xhrOptions.customRequestInitializer = function (xhrRequest) {
                    xhrRequest.timeout = options.timeout;
                };
            }
            return Async.always(winjs.xhr(xhrOptions), (function (xhr) {
                if (timer.data) {
                    timer.data.status = xhr.status;
                }
                timer.stop();
            }));
        };
        return BaseRequestService;
    }());
    exports.BaseRequestService = BaseRequestService;
});

define("vs/platform/search/common/search", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.ID = 'searchService';
    exports.ISearchService = instantiation_1.createDecorator(exports.ID);
    (function (QueryType) {
        QueryType[QueryType["File"] = 1] = "File";
        QueryType[QueryType["Text"] = 2] = "Text";
    })(exports.QueryType || (exports.QueryType = {}));
    var QueryType = exports.QueryType;
    // ---- very simple implementation of the search model --------------------
    var FileMatch = (function () {
        function FileMatch(resource) {
            this.resource = resource;
            this.lineMatches = [];
            // empty
        }
        return FileMatch;
    }());
    exports.FileMatch = FileMatch;
    var LineMatch = (function () {
        function LineMatch(preview, lineNumber, offsetAndLengths) {
            this.preview = preview;
            this.lineNumber = lineNumber;
            this.offsetAndLengths = offsetAndLengths;
            // empty
        }
        return LineMatch;
    }());
    exports.LineMatch = LineMatch;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/selection/common/selection", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Selection = (function () {
        function Selection(selection) {
            this._selection = selection || [];
        }
        Object.defineProperty(Selection.prototype, "selection", {
            get: function () {
                return this._selection;
            },
            enumerable: true,
            configurable: true
        });
        Selection.prototype.isEmpty = function () {
            return this._selection.length === 0;
        };
        Selection.EMPTY = new Selection([]);
        return Selection;
    }());
    exports.Selection = Selection;
    var StructuredSelection = (function (_super) {
        __extends(StructuredSelection, _super);
        function StructuredSelection() {
            _super.apply(this, arguments);
        }
        StructuredSelection.prototype.toArray = function () {
            return this.selection;
        };
        return StructuredSelection;
    }(Selection));
    exports.StructuredSelection = StructuredSelection;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/storage/common/storage", ["require", "exports", 'vs/base/common/events', 'vs/platform/instantiation/common/instantiation'], function (require, exports, events_1, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.ID = 'storageService';
    exports.IStorageService = instantiation_1.createDecorator(exports.ID);
    var StorageEventType;
    (function (StorageEventType) {
        /**
         * Event type for when a storage value is changed.
         */
        StorageEventType.STORAGE = 'storage';
    })(StorageEventType = exports.StorageEventType || (exports.StorageEventType = {}));
    /**
     * Storage events are being emitted when user settings change which are persisted to local storage.
     */
    var StorageEvent = (function (_super) {
        __extends(StorageEvent, _super);
        function StorageEvent(key, before, after, originalEvent) {
            _super.call(this, key, before, after, originalEvent);
        }
        /**
         * Returns true if the storage change has occurred from this browser window and false if its coming from a different window.
         */
        StorageEvent.prototype.isLocal = function () {
            // By the spec a storage event is only ever emitted if it occurs from a different browser tab or window
            // so we can use the check for originalEvent being set or not as a way to find out if the event is local or not.
            return !this.originalEvent;
        };
        return StorageEvent;
    }(events_1.PropertyChangeEvent));
    exports.StorageEvent = StorageEvent;
    (function (StorageScope) {
        /**
         * The stored data will be scoped to all workspaces of this domain.
         */
        StorageScope[StorageScope["GLOBAL"] = 0] = "GLOBAL";
        /**
         * The stored data will be scoped to the current workspace.
         */
        StorageScope[StorageScope["WORKSPACE"] = 1] = "WORKSPACE";
    })(exports.StorageScope || (exports.StorageScope = {}));
    var StorageScope = exports.StorageScope;
});

define("vs/platform/telemetry/common/telemetry", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.ID = 'telemetryService';
    exports.ITelemetryService = instantiation_1.createDecorator(exports.ID);
    function anonymize(input) {
        if (!input) {
            return input;
        }
        var r = '';
        for (var i = 0; i < input.length; i++) {
            var ch = input[i];
            if (ch >= '0' && ch <= '9') {
                r += '0';
                continue;
            }
            if (ch >= 'a' && ch <= 'z') {
                r += 'a';
                continue;
            }
            if (ch >= 'A' && ch <= 'Z') {
                r += 'A';
                continue;
            }
            r += ch;
        }
        return r;
    }
    exports.anonymize = anonymize;
});

define("vs/editor/common/editorCommonExtensions", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/uri', 'vs/platform/instantiation/common/descriptors', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/platform/platform', 'vs/platform/telemetry/common/telemetry', 'vs/editor/common/config/config', 'vs/editor/common/core/position', 'vs/editor/common/editorCommon', 'vs/editor/common/services/modelService'], function (require, exports, errors_1, uri_1, descriptors_1, keybindingService_1, keybindingsRegistry_1, platform_1, telemetry_1, config_1, position_1, editorCommon, modelService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // --- Keybinding extensions to make it more concise to express keybindings conditions
    (function (ContextKey) {
        ContextKey[ContextKey["None"] = 0] = "None";
        ContextKey[ContextKey["EditorTextFocus"] = 1] = "EditorTextFocus";
        ContextKey[ContextKey["EditorFocus"] = 2] = "EditorFocus";
    })(exports.ContextKey || (exports.ContextKey = {}));
    var ContextKey = exports.ContextKey;
    // --- Editor Actions
    var EditorActionDescriptor = (function () {
        function EditorActionDescriptor(ctor, id, label, kbOpts) {
            if (kbOpts === void 0) { kbOpts = defaultEditorActionKeybindingOptions; }
            this.ctor = ctor;
            this.id = id;
            this.label = label;
            this.kbOpts = kbOpts;
        }
        return EditorActionDescriptor;
    }());
    exports.EditorActionDescriptor = EditorActionDescriptor;
    var CommonEditorRegistry;
    (function (CommonEditorRegistry) {
        function registerEditorAction(desc) {
            platform_1.Registry.as(Extensions.EditorCommonContributions).registerEditorAction(desc);
        }
        CommonEditorRegistry.registerEditorAction = registerEditorAction;
        // --- Editor Contributions
        function registerEditorContribution(ctor) {
            platform_1.Registry.as(Extensions.EditorCommonContributions).registerEditorContribution2(ctor);
        }
        CommonEditorRegistry.registerEditorContribution = registerEditorContribution;
        function getEditorContributions() {
            return platform_1.Registry.as(Extensions.EditorCommonContributions).getEditorContributions2();
        }
        CommonEditorRegistry.getEditorContributions = getEditorContributions;
        // --- Editor Commands
        function commandWeight(importance) {
            if (importance === void 0) { importance = 0; }
            return keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorContrib(importance);
        }
        CommonEditorRegistry.commandWeight = commandWeight;
        function registerEditorCommand(commandId, weight, keybinding, needsTextFocus, needsKey, handler) {
            var commandDesc = {
                id: commandId,
                handler: createCommandHandler(commandId, handler),
                weight: weight,
                context: contextRule(needsTextFocus, needsKey),
                primary: keybinding.primary,
                secondary: keybinding.secondary,
                win: keybinding.win,
                linux: keybinding.linux,
                mac: keybinding.mac,
            };
            keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc(commandDesc);
        }
        CommonEditorRegistry.registerEditorCommand = registerEditorCommand;
        function registerLanguageCommand(id, handler) {
            keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
                id: id,
                handler: function (accessor, args) {
                    if (args && args.length > 1 || typeof args[0] !== 'object') {
                        throw errors_1.illegalArgument();
                    }
                    return handler(accessor, args && args[0]);
                },
                weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorContrib(),
                primary: undefined,
                context: undefined,
            });
        }
        CommonEditorRegistry.registerLanguageCommand = registerLanguageCommand;
        function registerDefaultLanguageCommand(id, handler) {
            registerLanguageCommand(id, function (accessor, args) {
                var resource = args.resource, position = args.position;
                if (!(resource instanceof uri_1.default) || !position_1.Position.isIPosition(position)) {
                    throw errors_1.illegalArgument();
                }
                var model = accessor.get(modelService_1.IModelService).getModel(resource);
                if (!model) {
                    throw errors_1.illegalArgument();
                }
                return handler(model, position, args);
            });
        }
        CommonEditorRegistry.registerDefaultLanguageCommand = registerDefaultLanguageCommand;
    })(CommonEditorRegistry = exports.CommonEditorRegistry || (exports.CommonEditorRegistry = {}));
    var SimpleEditorContributionDescriptor = (function () {
        function SimpleEditorContributionDescriptor(ctor) {
            this._ctor = ctor;
        }
        SimpleEditorContributionDescriptor.prototype.createInstance = function (instantiationService, editor) {
            return instantiationService.createInstance(this._ctor, editor);
        };
        return SimpleEditorContributionDescriptor;
    }());
    var InternalEditorActionDescriptor = (function () {
        function InternalEditorActionDescriptor(ctor, id, label) {
            this._descriptor = descriptors_1.createSyncDescriptor(ctor, {
                id: id,
                label: label
            });
        }
        InternalEditorActionDescriptor.prototype.createInstance = function (instService, editor) {
            return instService.createInstance(this._descriptor, editor);
        };
        return InternalEditorActionDescriptor;
    }());
    // Editor extension points
    var Extensions = {
        EditorCommonContributions: 'editor.commonContributions'
    };
    var EditorContributionRegistry = (function () {
        function EditorContributionRegistry() {
            this.editorContributions = [];
        }
        EditorContributionRegistry.prototype.registerEditorContribution2 = function (ctor) {
            this.editorContributions.push(new SimpleEditorContributionDescriptor(ctor));
        };
        EditorContributionRegistry.prototype.registerEditorAction = function (desc) {
            var handler = desc.kbOpts.handler;
            if (!handler) {
                if (desc.kbOpts.context === ContextKey.EditorTextFocus || desc.kbOpts.context === ContextKey.EditorFocus) {
                    handler = triggerEditorAction.bind(null, desc.id);
                }
                else {
                    handler = triggerEditorActionGlobal.bind(null, desc.id);
                }
            }
            var context = null;
            if (typeof desc.kbOpts.kbExpr === 'undefined') {
                if (desc.kbOpts.context === ContextKey.EditorTextFocus) {
                    context = keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS);
                }
                else if (desc.kbOpts.context === ContextKey.EditorFocus) {
                    context = keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_FOCUS);
                }
            }
            else {
                context = desc.kbOpts.kbExpr;
            }
            var commandDesc = {
                id: desc.id,
                handler: handler,
                weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorContrib(),
                context: context,
                primary: desc.kbOpts.primary,
                secondary: desc.kbOpts.secondary,
                win: desc.kbOpts.win,
                linux: desc.kbOpts.linux,
                mac: desc.kbOpts.mac,
            };
            keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc(commandDesc);
            this.editorContributions.push(new InternalEditorActionDescriptor(desc.ctor, desc.id, desc.label));
        };
        EditorContributionRegistry.prototype.getEditorContributions2 = function () {
            return this.editorContributions.slice(0);
        };
        return EditorContributionRegistry;
    }());
    platform_1.Registry.add(Extensions.EditorCommonContributions, new EditorContributionRegistry());
    function triggerEditorAction(actionId, accessor, args) {
        config_1.withCodeEditorFromCommandHandler(actionId, accessor, args, function (editor) {
            editor.trigger('keyboard', actionId, args);
        });
    }
    function triggerEditorActionGlobal(actionId, accessor, args) {
        // TODO: this is not necessarily keyboard
        var focusedEditor = config_1.findFocusedEditor(actionId, accessor, args, false);
        if (focusedEditor) {
            focusedEditor.trigger('keyboard', actionId, args);
            return;
        }
        var activeEditor = config_1.getActiveEditor(accessor);
        if (activeEditor) {
            var action = activeEditor.getAction(actionId);
            if (action) {
                accessor.get(telemetry_1.ITelemetryService).publicLog('editorActionInvoked', { name: action.label });
                action.run().done(null, errors_1.onUnexpectedError);
            }
            return;
        }
    }
    var defaultEditorActionKeybindingOptions = { primary: null, context: ContextKey.EditorTextFocus };
    function contextRule(needsTextFocus, needsKey) {
        var base = keybindingService_1.KbExpr.has(needsTextFocus ? editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS : editorCommon.KEYBINDING_CONTEXT_EDITOR_FOCUS);
        if (needsKey) {
            return keybindingService_1.KbExpr.and(base, keybindingService_1.KbExpr.has(needsKey));
        }
        return base;
    }
    function createCommandHandler(commandId, handler) {
        return function (accessor, args) {
            config_1.withCodeEditorFromCommandHandler(commandId, accessor, args, function (editor) {
                handler(accessor, editor, args);
            });
        };
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/clipboard/browser/clipboard", ["require", "exports", 'vs/nls!vs/editor/contrib/clipboard/browser/clipboard', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/base/browser/browser', 'vs/editor/common/config/config', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/css!./clipboard'], function (require, exports, nls, keyCodes_1, lifecycle_1, winjs_base_1, browser, config_1, editorAction_1, editorActionEnablement_1, editorCommon, editorCommonExtensions_1) {
    'use strict';
    var ClipboardWritingAction = (function (_super) {
        __extends(ClipboardWritingAction, _super);
        function ClipboardWritingAction(descriptor, editor, condition) {
            var _this = this;
            _super.call(this, descriptor, editor, condition);
            this.toUnhook = [];
            this.toUnhook.push(this.editor.addListener(editorCommon.EventType.CursorSelectionChanged, function (e) {
                _this.resetEnablementState();
            }));
        }
        ClipboardWritingAction.prototype.dispose = function () {
            this.toUnhook = lifecycle_1.cAll(this.toUnhook);
            _super.prototype.dispose.call(this);
        };
        ClipboardWritingAction.prototype.getEnablementState = function () {
            if (browser.enableEmptySelectionClipboard) {
                return true;
            }
            else {
                return !this.editor.getSelection().isEmpty();
            }
        };
        return ClipboardWritingAction;
    }(editorAction_1.EditorAction));
    function editorCursorIsInEditableRange(editor) {
        var model = editor.getModel();
        if (!model) {
            return false;
        }
        var hasEditableRange = model.hasEditableRange();
        if (!hasEditableRange) {
            return true;
        }
        var editableRange = model.getEditableRange();
        var editorPosition = editor.getPosition();
        return editableRange.containsPosition(editorPosition);
    }
    var ExecCommandCutAction = (function (_super) {
        __extends(ExecCommandCutAction, _super);
        function ExecCommandCutAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.Writeable | editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.ShowInContextMenu | editorActionEnablement_1.Behaviour.UpdateOnCursorPositionChange);
        }
        ExecCommandCutAction.prototype.getGroupId = function () {
            return '3_edit/1_cut';
        };
        ExecCommandCutAction.prototype.getEnablementState = function () {
            return _super.prototype.getEnablementState.call(this) && editorCursorIsInEditableRange(this.editor);
        };
        ExecCommandCutAction.prototype.run = function () {
            this.editor.focus();
            document.execCommand('cut');
            return winjs_base_1.TPromise.as(true);
        };
        return ExecCommandCutAction;
    }(ClipboardWritingAction));
    var ExecCommandCopyAction = (function (_super) {
        __extends(ExecCommandCopyAction, _super);
        function ExecCommandCopyAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.ShowInContextMenu);
        }
        ExecCommandCopyAction.prototype.getGroupId = function () {
            return '3_edit/2_copy';
        };
        ExecCommandCopyAction.prototype.run = function () {
            this.editor.focus();
            document.execCommand('copy');
            return winjs_base_1.TPromise.as(true);
        };
        return ExecCommandCopyAction;
    }(ClipboardWritingAction));
    var ExecCommandPasteAction = (function (_super) {
        __extends(ExecCommandPasteAction, _super);
        function ExecCommandPasteAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.Writeable | editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.ShowInContextMenu | editorActionEnablement_1.Behaviour.UpdateOnCursorPositionChange);
        }
        ExecCommandPasteAction.prototype.getGroupId = function () {
            return '3_edit/3_paste';
        };
        ExecCommandPasteAction.prototype.getEnablementState = function () {
            return editorCursorIsInEditableRange(this.editor);
        };
        ExecCommandPasteAction.prototype.run = function () {
            this.editor.focus();
            document.execCommand('paste');
            return null;
        };
        return ExecCommandPasteAction;
    }(editorAction_1.EditorAction));
    function registerClipboardAction(desc) {
        if (!browser.supportsExecCommand(desc.execCommand)) {
            return;
        }
        editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(desc.ctor, desc.id, desc.label, {
            handler: execCommandToHandler.bind(null, desc.id, desc.execCommand),
            context: editorCommonExtensions_1.ContextKey.None,
            primary: desc.primary,
            secondary: desc.secondary,
            win: desc.win,
            linux: desc.linux,
            mac: desc.mac
        }));
    }
    registerClipboardAction({
        ctor: ExecCommandCutAction,
        id: 'editor.action.clipboardCutAction',
        label: nls.localize(0, null),
        execCommand: 'cut',
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_X,
        win: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_X, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Delete] }
    });
    registerClipboardAction({
        ctor: ExecCommandCopyAction,
        id: 'editor.action.clipboardCopyAction',
        label: nls.localize(1, null),
        execCommand: 'copy',
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_C,
        win: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_C, secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.Insert] }
    });
    registerClipboardAction({
        ctor: ExecCommandPasteAction,
        id: 'editor.action.clipboardPasteAction',
        label: nls.localize(2, null),
        execCommand: 'paste',
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_V,
        win: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_V, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Insert] }
    });
    function execCommandToHandler(actionId, browserCommand, accessor, args) {
        // If editor text focus
        if (args.context[editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS]) {
            var focusedEditor = config_1.findFocusedEditor(actionId, accessor, args, false);
            if (focusedEditor) {
                focusedEditor.trigger('keyboard', actionId, args);
                return;
            }
        }
        document.execCommand(browserCommand);
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/codelens/common/codelens", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry', 'vs/editor/common/services/modelService'], function (require, exports, errors_1, uri_1, winjs_base_1, editorCommonExtensions_1, languageFeatureRegistry_1, modelService_1) {
    'use strict';
    exports.CodeLensRegistry = new languageFeatureRegistry_1.default('codeLensSupport');
    function getCodeLensData(model) {
        var symbols = [];
        var promises = exports.CodeLensRegistry.all(model).map(function (support) {
            return support.findCodeLensSymbols(model.getAssociatedResource()).then(function (result) {
                if (!Array.isArray(result)) {
                    return;
                }
                for (var _i = 0, result_1 = result; _i < result_1.length; _i++) {
                    var symbol = result_1[_i];
                    symbols.push({ symbol: symbol, support: support });
                }
            }, function (err) {
                if (!errors_1.isPromiseCanceledError(err)) {
                    errors_1.onUnexpectedError(err);
                }
            });
        });
        return winjs_base_1.TPromise.join(promises).then(function () { return symbols; });
    }
    exports.getCodeLensData = getCodeLensData;
    editorCommonExtensions_1.CommonEditorRegistry.registerLanguageCommand('_executeCodeLensProvider', function (accessor, args) {
        var resource = args.resource;
        if (!(resource instanceof uri_1.default)) {
            throw errors_1.illegalArgument();
        }
        var model = accessor.get(modelService_1.IModelService).getModel(resource);
        if (!model) {
            throw errors_1.illegalArgument();
        }
        return getCodeLensData(model);
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/codelens/browser/codelens", ["require", "exports", 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/lifecycle', 'vs/base/common/severity', 'vs/base/common/strings', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/platform/configuration/common/configuration', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/message/common/message', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/services/modelService', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/editorBrowserExtensions', '../common/codelens', 'vs/css!./codelens'], function (require, exports, async_1, errors_1, lifecycle_1, severity_1, strings_1, winjs_base_1, dom, configuration_1, keybindingService_1, message_1, range_1, editorCommon, modelService_1, editorBrowser, editorBrowserExtensions_1, codelens_1) {
    'use strict';
    var CodeLensViewZone = (function () {
        function CodeLensViewZone(afterLineNumber) {
            this.afterLineNumber = afterLineNumber;
            this.heightInLines = 1;
            this.suppressMouseDown = true;
            this.domNode = document.createElement('div');
        }
        CodeLensViewZone.prototype.setAfterLineNumber = function (afterLineNumber) {
            this.afterLineNumber = afterLineNumber;
        };
        return CodeLensViewZone;
    }());
    var CodeLensContentWidget = (function () {
        function CodeLensContentWidget(editor, symbolRange, keybindingService, messageService) {
            var _this = this;
            this._commands = Object.create(null);
            this._id = 'codeLensWidget' + (++CodeLensContentWidget.ID);
            this._editor = editor;
            this.setSymbolRange(symbolRange);
            this._domNode = document.createElement('span');
            this._domNode.style.height = editor.getConfiguration().lineHeight + "px";
            this._domNode.innerHTML = '&nbsp;';
            dom.addClass(this._domNode, 'codelens-decoration');
            dom.addClass(this._domNode, 'invisible-cl');
            this._subscription = dom.addDisposableListener(this._domNode, 'click', function (e) {
                var element = e.target;
                if (element.tagName === 'A' && element.id) {
                    var command = _this._commands[element.id];
                    if (command) {
                        editor.focus();
                        keybindingService.executeCommand(command.id, command.arguments).done(undefined, function (err) {
                            messageService.show(severity_1.default.Error, err);
                        });
                    }
                }
            });
            this.updateVisibility();
        }
        CodeLensContentWidget.prototype.dispose = function () {
            this._subscription.dispose();
            this._symbolRange = null;
        };
        CodeLensContentWidget.prototype.updateVisibility = function () {
            if (this.isVisible()) {
                dom.removeClass(this._domNode, 'invisible-cl');
                dom.addClass(this._domNode, 'fadein');
            }
        };
        CodeLensContentWidget.prototype.withCommands = function (symbols) {
            this._commands = Object.create(null);
            if (!symbols || !symbols.length) {
                this._domNode.innerHTML = 'no commands';
                return;
            }
            var html = [];
            for (var i = 0; i < symbols.length; i++) {
                var command = symbols[i].command;
                var part = void 0;
                if (command.id) {
                    part = strings_1.format('<a id={0}>{1}</a>', i, command.title);
                    this._commands[i] = command;
                }
                else {
                    part = strings_1.format('<span>{0}</span>', command.title);
                }
                html.push(part);
            }
            this._domNode.innerHTML = html.join('<span>&nbsp;|&nbsp;</span>');
            this._editor.layoutContentWidget(this);
        };
        CodeLensContentWidget.prototype.getId = function () {
            return this._id;
        };
        CodeLensContentWidget.prototype.getDomNode = function () {
            return this._domNode;
        };
        CodeLensContentWidget.prototype.setSymbolRange = function (range) {
            this._symbolRange = range;
            var lineNumber = range.startLineNumber;
            var column = this._editor.getModel().getLineFirstNonWhitespaceColumn(lineNumber);
            this._widgetPosition = {
                position: { lineNumber: lineNumber, column: column },
                preference: [editorBrowser.ContentWidgetPositionPreference.ABOVE]
            };
        };
        CodeLensContentWidget.prototype.getPosition = function () {
            return this._widgetPosition;
        };
        CodeLensContentWidget.prototype.isVisible = function () {
            return this._domNode.hasAttribute('monaco-visible-content-widget');
        };
        CodeLensContentWidget.ID = 0;
        return CodeLensContentWidget;
    }());
    function modelsVersionId(modelService, modeId) {
        var result = 1;
        var models = modelService.getModels()
            .filter(function (model) { return model.getMode().getId() === modeId; })
            .map(function (model) {
            return {
                url: model.getAssociatedResource().toString(),
                versionId: model.getVersionId()
            };
        })
            .sort(function (a, b) {
            if (a.url < b.url) {
                return -1;
            }
            if (a.url > b.url) {
                return 1;
            }
            return 0;
        });
        for (var i = 0; i < models.length; i++) {
            result = (((31 * result) | 0) + models[i].versionId) | 0;
        }
        return result;
    }
    var CodeLensHelper = (function () {
        function CodeLensHelper() {
            this._removeDecorations = [];
            this._addDecorations = [];
            this._addDecorationsCallbacks = [];
        }
        CodeLensHelper.prototype.addDecoration = function (decoration, callback) {
            this._addDecorations.push(decoration);
            this._addDecorationsCallbacks.push(callback);
        };
        CodeLensHelper.prototype.removeDecoration = function (decorationId) {
            this._removeDecorations.push(decorationId);
        };
        CodeLensHelper.prototype.commit = function (changeAccessor) {
            var resultingDecorations = changeAccessor.deltaDecorations(this._removeDecorations, this._addDecorations);
            for (var i = 0, len = resultingDecorations.length; i < len; i++) {
                this._addDecorationsCallbacks[i](resultingDecorations[i]);
            }
        };
        return CodeLensHelper;
    }());
    var CodeLens = (function () {
        function CodeLens(data, editor, helper, viewZoneChangeAccessor, keybindingService, messageService) {
            var _this = this;
            this._editor = editor;
            this._data = data;
            this._decorationIds = new Array(this._data.length);
            var range;
            this._data.forEach(function (codeLensData, i) {
                helper.addDecoration({
                    range: codeLensData.symbol.range,
                    options: {}
                }, function (id) { return _this._decorationIds[i] = id; });
                // the range contain all lenses on this line
                for (var _i = 0, _a = _this._data; _i < _a.length; _i++) {
                    var lensData = _a[_i];
                    if (!range) {
                        range = lensData.symbol.range;
                    }
                    else {
                        range = range_1.Range.plusRange(range, lensData.symbol.range);
                    }
                }
            });
            this._viewZone = new CodeLensViewZone(range.startLineNumber - 1);
            this._contentWidget = new CodeLensContentWidget(editor, range_1.Range.lift(range), keybindingService, messageService);
            this._viewZoneId = viewZoneChangeAccessor.addZone(this._viewZone);
            this._editor.addContentWidget(this._contentWidget);
            this._lastUpdateModelsVersionId = -1;
        }
        CodeLens.prototype.dispose = function (helper, viewZoneChangeAccessor) {
            while (this._decorationIds.length) {
                helper.removeDecoration(this._decorationIds.pop());
            }
            if (viewZoneChangeAccessor) {
                viewZoneChangeAccessor.removeZone(this._viewZoneId);
            }
            this._editor.removeContentWidget(this._contentWidget);
            this._contentWidget.dispose();
        };
        CodeLens.prototype.isValid = function () {
            var _this = this;
            return this._decorationIds.some(function (id) {
                var range = _this._editor.getModel().getDecorationRange(id);
                return range && !range.isEmpty();
            });
        };
        CodeLens.prototype.updateCodeLensSymbols = function (data) {
            this._data = data;
        };
        CodeLens.prototype.computeIfNecessary = function (currentModelsVersionId, model) {
            this._contentWidget.updateVisibility(); // trigger the fade in
            if (!this._contentWidget.isVisible()) {
                return null;
            }
            if (this._lastUpdateModelsVersionId === currentModelsVersionId) {
                return null;
            }
            // Read editor current state
            for (var i = 0; i < this._decorationIds.length; i++) {
                this._data[i].symbol.range = model.getDecorationRange(this._decorationIds[i]);
            }
            return this._data;
        };
        CodeLens.prototype.updateCommands = function (symbols, currentModelsVersionId) {
            this._contentWidget.withCommands(symbols);
            this._lastUpdateModelsVersionId = currentModelsVersionId;
        };
        CodeLens.prototype.getLineNumber = function () {
            var range = this._editor.getModel().getDecorationRange(this._decorationIds[0]);
            if (range) {
                return range.startLineNumber;
            }
            return -1;
        };
        CodeLens.prototype.update = function (viewZoneChangeAccessor) {
            if (this.isValid()) {
                var range = this._editor.getModel().getDecorationRange(this._decorationIds[0]);
                this._viewZone.setAfterLineNumber(range.startLineNumber - 1);
                viewZoneChangeAccessor.layoutZone(this._viewZoneId);
                this._contentWidget.setSymbolRange(range);
                this._editor.layoutContentWidget(this._contentWidget);
            }
        };
        return CodeLens;
    }());
    var CodeLensContribution = (function () {
        function CodeLensContribution(editor, modelService, configurationService, keybindingService, messageService) {
            var _this = this;
            this._instanceCount = (++CodeLensContribution.INSTANCE_COUNT);
            this._editor = editor;
            this._modelService = modelService;
            this._configurationService = configurationService;
            this._keybindingService = keybindingService;
            this._messageService = messageService;
            this._globalToDispose = [];
            this._localToDispose = [];
            this._lenses = [];
            this._currentFindCodeLensSymbolsPromise = null;
            this._modelChangeCounter = 0;
            this._codeLenseDisabledByMode = true;
            this._globalToDispose.push(this._editor.addListener2(editorCommon.EventType.ModelChanged, function () { return _this.onModelChange(); }));
            this._globalToDispose.push(this._editor.addListener2(editorCommon.EventType.ModelModeChanged, function () { return _this.onModelChange(); }));
            this._globalToDispose.push(this._editor.addListener2(editorCommon.EventType.ModelModeSupportChanged, function (e) {
                if (e.codeLensSupport) {
                    _this.onModelChange();
                }
            }));
            this._globalToDispose.push(this._editor.addListener2(editorCommon.EventType.ConfigurationChanged, function (e) {
                if (e.referenceInfos) {
                    _this.onModelChange();
                }
            }));
            this._globalToDispose.push(codelens_1.CodeLensRegistry.onDidChange(this.onModelChange, this));
            this.onModelChange();
        }
        CodeLensContribution.prototype.dispose = function () {
            this.localDispose();
            this._globalToDispose = lifecycle_1.disposeAll(this._globalToDispose);
        };
        CodeLensContribution.prototype.localDispose = function () {
            if (this._currentFindCodeLensSymbolsPromise) {
                this._currentFindCodeLensSymbolsPromise.cancel();
                this._currentFindCodeLensSymbolsPromise = null;
                this._modelChangeCounter++;
            }
            if (this._currentFindOccPromise) {
                this._currentFindOccPromise.cancel();
                this._currentFindOccPromise = null;
            }
            this._localToDispose = lifecycle_1.disposeAll(this._localToDispose);
        };
        CodeLensContribution.prototype.getId = function () {
            return CodeLensContribution.ID;
        };
        CodeLensContribution.prototype.onModelChange = function () {
            var _this = this;
            this.localDispose();
            var model = this._editor.getModel();
            if (!model) {
                return;
            }
            if (!this._editor.getConfiguration().referenceInfos) {
                return;
            }
            if (!codelens_1.CodeLensRegistry.has(model)) {
                return;
            }
            var detectVisible = new async_1.RunOnceScheduler(function () {
                _this._onViewportChanged(model.getMode().getId());
            }, 500);
            var scheduler = new async_1.RunOnceScheduler(function () {
                if (_this._currentFindCodeLensSymbolsPromise) {
                    _this._currentFindCodeLensSymbolsPromise.cancel();
                }
                _this._currentFindCodeLensSymbolsPromise = codelens_1.getCodeLensData(model);
                var counterValue = ++_this._modelChangeCounter;
                _this._currentFindCodeLensSymbolsPromise.then(function (result) {
                    if (counterValue === _this._modelChangeCounter) {
                        _this.renderCodeLensSymbols(result);
                        detectVisible.schedule();
                    }
                }, function (error) {
                    errors_1.onUnexpectedError(error);
                });
            }, 250);
            this._localToDispose.push(scheduler);
            this._localToDispose.push(detectVisible);
            this._localToDispose.push(model.addBulkListener2(function (events) {
                var hadChange = false;
                for (var i = 0; i < events.length; i++) {
                    var eventType = events[i].getType();
                    if (eventType === editorCommon.EventType.ModelContentChanged) {
                        hadChange = true;
                        break;
                    }
                }
                if (hadChange) {
                    _this._editor.changeDecorations(function (changeAccessor) {
                        _this._editor.changeViewZones(function (viewAccessor) {
                            var toDispose = [];
                            _this._lenses.forEach(function (lens) {
                                if (lens.isValid()) {
                                    lens.update(viewAccessor);
                                }
                                else {
                                    toDispose.push(lens);
                                }
                            });
                            var helper = new CodeLensHelper();
                            toDispose.forEach(function (l) {
                                l.dispose(helper, viewAccessor);
                                _this._lenses.splice(_this._lenses.indexOf(l), 1);
                            });
                            helper.commit(changeAccessor);
                        });
                    });
                    // Compute new `visible` code lenses
                    detectVisible.schedule();
                    // Ask for all references again
                    scheduler.schedule();
                }
            }));
            this._localToDispose.push(this._editor.addListener2('scroll', function (e) {
                detectVisible.schedule();
            }));
            this._localToDispose.push({
                dispose: function () {
                    if (_this._editor.getModel()) {
                        _this._editor.changeDecorations(function (changeAccessor) {
                            _this._editor.changeViewZones(function (accessor) {
                                _this._disposeAllLenses(changeAccessor, accessor);
                            });
                        });
                    }
                    else {
                        // No accessors available
                        _this._disposeAllLenses(null, null);
                    }
                }
            });
            scheduler.schedule();
        };
        CodeLensContribution.prototype._disposeAllLenses = function (decChangeAccessor, viewZoneChangeAccessor) {
            var helper = new CodeLensHelper();
            this._lenses.forEach(function (lens) { return lens.dispose(helper, viewZoneChangeAccessor); });
            if (decChangeAccessor) {
                helper.commit(decChangeAccessor);
            }
            this._lenses = [];
        };
        CodeLensContribution.prototype.renderCodeLensSymbols = function (symbols) {
            var _this = this;
            if (!this._editor.getModel()) {
                return;
            }
            if (!symbols) {
                symbols = [];
            }
            else {
                symbols = symbols.sort(function (a, b) { return range_1.Range.compareRangesUsingStarts(a.symbol.range, b.symbol.range); });
            }
            var maxLineNumber = this._editor.getModel().getLineCount();
            var groups = [];
            var lastGroup;
            for (var _i = 0, symbols_1 = symbols; _i < symbols_1.length; _i++) {
                var symbol = symbols_1[_i];
                var line = symbol.symbol.range.startLineNumber;
                if (line < 1 || line >= maxLineNumber) {
                    // invalid code lens
                    continue;
                }
                else if (lastGroup && lastGroup[lastGroup.length - 1].symbol.range.startLineNumber === line) {
                    // on same line as previous
                    lastGroup.push(symbol);
                }
                else {
                    // on later line as previous
                    lastGroup = [symbol];
                    groups.push(lastGroup);
                }
            }
            var centeredRange = this._editor.getCenteredRangeInViewport();
            var shouldRestoreCenteredRange = (groups.length !== this._lenses.length && this._editor.getScrollTop() !== 0);
            this._editor.changeDecorations(function (changeAccessor) {
                _this._editor.changeViewZones(function (accessor) {
                    var codeLensIndex = 0, groupsIndex = 0, helper = new CodeLensHelper();
                    while (groupsIndex < groups.length && codeLensIndex < _this._lenses.length) {
                        var symbolsLineNumber = groups[groupsIndex][0].symbol.range.startLineNumber;
                        var codeLensLineNumber = _this._lenses[codeLensIndex].getLineNumber();
                        if (codeLensLineNumber < symbolsLineNumber) {
                            _this._lenses[codeLensIndex].dispose(helper, accessor);
                            _this._lenses.splice(codeLensIndex, 1);
                        }
                        else if (codeLensLineNumber === symbolsLineNumber) {
                            _this._lenses[codeLensIndex].updateCodeLensSymbols(groups[groupsIndex]);
                            groupsIndex++;
                            codeLensIndex++;
                        }
                        else {
                            _this._lenses.splice(codeLensIndex, 0, new CodeLens(groups[groupsIndex], _this._editor, helper, accessor, _this._keybindingService, _this._messageService));
                            codeLensIndex++;
                            groupsIndex++;
                        }
                    }
                    // Delete extra code lenses
                    while (codeLensIndex < _this._lenses.length) {
                        _this._lenses[codeLensIndex].dispose(helper, accessor);
                        _this._lenses.splice(codeLensIndex, 1);
                    }
                    // Create extra symbols
                    while (groupsIndex < groups.length) {
                        _this._lenses.push(new CodeLens(groups[groupsIndex], _this._editor, helper, accessor, _this._keybindingService, _this._messageService));
                        groupsIndex++;
                    }
                    helper.commit(changeAccessor);
                });
            });
            if (shouldRestoreCenteredRange) {
                this._editor.revealRangeInCenter(centeredRange);
            }
        };
        CodeLensContribution.prototype._onViewportChanged = function (modeId) {
            var _this = this;
            if (this._currentFindOccPromise) {
                this._currentFindOccPromise.cancel();
                this._currentFindOccPromise = null;
            }
            var model = this._editor.getModel();
            if (!model) {
                return;
            }
            var currentModelsVersionId = modelsVersionId(this._modelService, modeId);
            var toResolve = [];
            var lenses = [];
            this._lenses.forEach(function (lens) {
                var request = lens.computeIfNecessary(currentModelsVersionId, model);
                if (request) {
                    toResolve.push(request);
                    lenses.push(lens);
                }
            });
            if (toResolve.length === 0) {
                return;
            }
            var resource = model.getAssociatedResource();
            var promises = toResolve.map(function (request, i) {
                var resolvedSymbols = new Array(request.length);
                var promises = request.map(function (request, i) {
                    return request.support.resolveCodeLensSymbol(resource, request.symbol).then(function (symbol) {
                        resolvedSymbols[i] = symbol;
                    });
                });
                return winjs_base_1.TPromise.join(promises).then(function () {
                    lenses[i].updateCommands(resolvedSymbols, currentModelsVersionId);
                });
            });
            this._currentFindOccPromise = winjs_base_1.TPromise.join(promises).then(function () {
                _this._currentFindOccPromise = null;
            });
        };
        CodeLensContribution.ID = 'css.editor.codeLens';
        CodeLensContribution.INSTANCE_COUNT = 0;
        CodeLensContribution = __decorate([
            __param(1, modelService_1.IModelService),
            __param(2, configuration_1.IConfigurationService),
            __param(3, keybindingService_1.IKeybindingService),
            __param(4, message_1.IMessageService)
        ], CodeLensContribution);
        return CodeLensContribution;
    }());
    exports.CodeLensContribution = CodeLensContribution;
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(CodeLensContribution);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/color/browser/color", ["require", "exports", 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/lifecycle', 'vs/base/common/strings', 'vs/base/browser/dom', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/css!./color'], function (require, exports, async_1, errors_1, lifecycle_1, strings, dom, editorCommon, editorCommonExtensions_1) {
    'use strict';
    var ColorDecoration = (function () {
        function ColorDecoration(renderingDecorationId, trackingDecorationId) {
            this.renderingDecorationId = renderingDecorationId;
            this.trackingDecorationId = trackingDecorationId;
        }
        ColorDecoration.createRenderingDecoration = function (range, inlineClassName) {
            return {
                range: {
                    startLineNumber: range.startLineNumber,
                    startColumn: range.startColumn,
                    endLineNumber: range.startLineNumber,
                    endColumn: range.startColumn + 1
                },
                options: {
                    inlineClassName: 'inline-color-decoration ' + inlineClassName,
                    stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
                }
            };
        };
        ColorDecoration.createTrackingDecoration = function (range) {
            return {
                range: range,
                options: {
                    stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
                }
            };
        };
        ColorDecoration.prototype.getColorValue = function (model) {
            var range = model.getDecorationRange(this.trackingDecorationId);
            if (range) {
                return model.getValueInRange(range);
            }
            return '';
        };
        return ColorDecoration;
    }());
    var ColorContribution = (function () {
        function ColorContribution(editor) {
            var _this = this;
            this._instanceCount = (++ColorContribution.INSTANCE_COUNT);
            this._editor = editor;
            this._callOnDispose = [];
            this._callOnModelChange = [];
            this._currentDecorations = [];
            this._currentDynamicColors = [];
            this._contentChangedScheduler = new async_1.RunOnceScheduler(null, 250);
            this._decorationsChangedScheduler = new async_1.RunOnceScheduler(function () { return _this.onDecorationsChanged(); }, 250);
            this._currentFindColorDeclarationsPromise = null;
            this._callOnDispose.push(this._contentChangedScheduler);
            this._callOnDispose.push(this._decorationsChangedScheduler);
            this._callOnDispose.push(this._editor.addListener2(editorCommon.EventType.ModelChanged, function () { return _this.onModelChange(); }));
            this._callOnDispose.push(this._editor.addListener2(editorCommon.EventType.ModelModeChanged, function () { return _this.onModelChange(); }));
            this._callOnDispose.push(this._editor.addListener2(editorCommon.EventType.ModelModeSupportChanged, function (e) {
                _this.onModelChange();
            }));
            this._style = dom.createStyleSheet();
            this.onModelChange();
        }
        ColorContribution.prototype.dispose = function () {
            var _this = this;
            if (this._currentDecorations.length > 0) {
                this._editor.changeDecorations(function (changeAccessor) {
                    var oldDecorations = [];
                    for (var i = 0, len = _this._currentDecorations.length; i < len; i++) {
                        oldDecorations.push(_this._currentDecorations[i].renderingDecorationId);
                        oldDecorations.push(_this._currentDecorations[i].trackingDecorationId);
                    }
                    changeAccessor.deltaDecorations(oldDecorations, []);
                    _this._currentDecorations = null;
                });
            }
            this._style.parentNode.removeChild(this._style);
            this._style = null;
            this._callOnDispose = lifecycle_1.disposeAll(this._callOnDispose);
        };
        ColorContribution.prototype.getId = function () {
            return ColorContribution.ID;
        };
        ColorContribution.prototype.onModelChange = function () {
            var _this = this;
            lifecycle_1.cAll(this._callOnModelChange);
            var model = this._editor.getModel();
            if (!model) {
                return;
            }
            var rawMode = model.getMode();
            if (typeof rawMode['findColorDeclarations'] !== 'function') {
                return;
            }
            this._contentChangedScheduler.setRunner(function () {
                if (_this._currentFindColorDeclarationsPromise) {
                    _this._currentFindColorDeclarationsPromise.cancel();
                }
                _this._currentFindColorDeclarationsPromise = rawMode['findColorDeclarations'](model.getAssociatedResource());
                var myModelVersion = _this._editor.getModel().getVersionId();
                _this._currentFindColorDeclarationsPromise.then(function (result) {
                    if (myModelVersion !== _this._editor.getModel().getVersionId()) {
                        return;
                    }
                    _this.renderAndTrackColors(result);
                }, function (error) {
                    errors_1.onUnexpectedError(error);
                });
            });
            this._contentChangedScheduler.schedule();
            this._callOnModelChange.push(function () {
                _this._contentChangedScheduler.cancel();
                _this._decorationsChangedScheduler.cancel();
            });
            this._callOnModelChange.push(function () {
                if (_this._currentFindColorDeclarationsPromise) {
                    _this._currentFindColorDeclarationsPromise.cancel();
                }
                _this._currentFindColorDeclarationsPromise = null;
            });
            this._callOnModelChange.push(this._editor.addListener(editorCommon.EventType.ModelContentChanged, function (event) { return _this._contentChangedScheduler.schedule(); }));
            this._callOnModelChange.push(model.addListener(editorCommon.EventType.ModelDecorationsChanged, function (event) { return _this._decorationsChangedScheduler.schedule(); }));
        };
        ColorContribution.prototype.renderAndTrackColors = function (colors) {
            var _this = this;
            // Reduce to a maximum of 500 colors
            colors = colors.slice(0, 500);
            this._editor.changeDecorations(function (changeAccessor) {
                var oldDecorations = [];
                for (var i = 0, len = _this._currentDecorations.length; i < len; i++) {
                    oldDecorations.push(_this._currentDecorations[i].renderingDecorationId);
                    oldDecorations.push(_this._currentDecorations[i].trackingDecorationId);
                }
                var newDecorations = [];
                for (var i = 0, len = colors.length; i < len; i++) {
                    newDecorations.push(ColorDecoration.createRenderingDecoration(colors[i].range, _this.getCSSRuleName(i)));
                    newDecorations.push(ColorDecoration.createTrackingDecoration(colors[i].range));
                }
                var decorations = changeAccessor.deltaDecorations(oldDecorations, newDecorations);
                _this._currentDecorations = [];
                for (var i = 0, len = colors.length; i < len; i++) {
                    _this._currentDecorations.push(new ColorDecoration(decorations[2 * i], decorations[2 * i + 1]));
                }
            });
            this.onDecorationsChanged();
        };
        ColorContribution.prototype.onDecorationsChanged = function () {
            var _this = this;
            var model = this._editor.getModel(), i, len, range, renderingRange, desiredRenderingRange, decoration, desiredColors = [];
            this._editor.changeDecorations(function (changeAccessor) {
                for (i = 0, len = _this._currentDecorations.length; i < len; i++) {
                    decoration = _this._currentDecorations[i];
                    range = model.getDecorationRange(decoration.trackingDecorationId);
                    if (range && !range.isEmpty()) {
                        // Collect color for this decoration
                        desiredColors[i] = model.getValueInRange(range).replace(/[^%#a-z0-9.,()]/gi, '');
                        // Prevent rendering decorations from growing too much
                        renderingRange = model.getDecorationRange(decoration.renderingDecorationId);
                        desiredRenderingRange = model.validateRange({
                            startLineNumber: range.startLineNumber,
                            startColumn: range.startColumn,
                            endLineNumber: range.startLineNumber,
                            endColumn: range.startColumn + 1
                        });
                        if (!renderingRange || !renderingRange.equalsRange(desiredRenderingRange)) {
                            changeAccessor.changeDecoration(decoration.renderingDecorationId, desiredRenderingRange);
                        }
                    }
                    else {
                        desiredColors[i] = '';
                    }
                }
                _this.ensureColors(desiredColors);
            });
        };
        ColorContribution.prototype.getCSSRuleName = function (index) {
            if (index < 0) {
                return '.monaco-css-dynamic-' + this._instanceCount + '-';
            }
            return '.monaco-css-dynamic-' + this._instanceCount + '-' + index + ':before';
        };
        ColorContribution.prototype._changeCost = function (desiredColors) {
            if (this._currentDynamicColors.length !== desiredColors.length) {
                return Number.MAX_VALUE;
            }
            var modifiedCnt = 0;
            for (var i = 0; i < desiredColors.length; i++) {
                if (desiredColors[i] !== this._currentDynamicColors[i]) {
                    modifiedCnt++;
                }
            }
            return modifiedCnt;
        };
        ColorContribution.prototype.ensureColors = function (desiredColors) {
            var i, changeCost = this._changeCost(desiredColors);
            if (changeCost === 0) {
                // Nothing to change
                return;
            }
            if (changeCost < 10) {
                // Simply modify up to 10 rules (lengths will match for sure)
                for (i = 0; i < desiredColors.length; i++) {
                    if (desiredColors[i] !== this._currentDynamicColors[i]) {
                        var rule = dom.getCSSRule(this.getCSSRuleName(i), this._style);
                        if (rule) {
                            rule.style.backgroundColor = desiredColors[i];
                        }
                    }
                }
            }
            else {
                // .innerHTML is the friend here
                var result = [];
                for (i = 0; i < desiredColors.length; i++) {
                    result.push(this.getCSSRuleName(i));
                    result.push('{');
                    result.push(this.getCSSText(desiredColors[i]));
                    result.push('}');
                }
                this._style.innerHTML = result.join('');
            }
            this._currentDynamicColors = desiredColors;
        };
        ColorContribution.prototype.getCSSText = function (color) {
            // Variants:
            return strings.format('background-color:{0};', color);
        };
        ColorContribution.ID = 'css.editor.colorContribution';
        ColorContribution.INSTANCE_COUNT = 0;
        return ColorContribution;
    }());
    exports.ColorContribution = ColorContribution;
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorContribution(ColorContribution);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/comment/common/comment", ["require", "exports", 'vs/nls!vs/editor/contrib/comment/common/comment', 'vs/base/common/keyCodes', 'vs/base/common/winjs.base', 'vs/editor/common/editorAction', 'vs/editor/common/editorCommonExtensions', './blockCommentCommand', './lineCommentCommand'], function (require, exports, nls, keyCodes_1, winjs_base_1, editorAction_1, editorCommonExtensions_1, blockCommentCommand_1, lineCommentCommand_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var CommentLineAction = (function (_super) {
        __extends(CommentLineAction, _super);
        function CommentLineAction(descriptor, editor, type) {
            _super.call(this, descriptor, editor);
            this._type = type;
        }
        CommentLineAction.prototype.run = function () {
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var commands = [];
            var selections = this.editor.getSelections();
            var opts = model.getOptions();
            for (var i = 0; i < selections.length; i++) {
                commands.push(new lineCommentCommand_1.LineCommentCommand(selections[i], opts.tabSize, this._type));
            }
            this.editor.executeCommands(this.id, commands);
            return winjs_base_1.TPromise.as(null);
        };
        CommentLineAction.ID = 'editor.action.commentLine';
        return CommentLineAction;
    }(editorAction_1.EditorAction));
    var ToggleCommentLineAction = (function (_super) {
        __extends(ToggleCommentLineAction, _super);
        function ToggleCommentLineAction(descriptor, editor) {
            _super.call(this, descriptor, editor, lineCommentCommand_1.Type.Toggle);
        }
        ToggleCommentLineAction.ID = 'editor.action.commentLine';
        return ToggleCommentLineAction;
    }(CommentLineAction));
    var AddLineCommentAction = (function (_super) {
        __extends(AddLineCommentAction, _super);
        function AddLineCommentAction(descriptor, editor) {
            _super.call(this, descriptor, editor, lineCommentCommand_1.Type.ForceAdd);
        }
        AddLineCommentAction.ID = 'editor.action.addCommentLine';
        return AddLineCommentAction;
    }(CommentLineAction));
    var RemoveLineCommentAction = (function (_super) {
        __extends(RemoveLineCommentAction, _super);
        function RemoveLineCommentAction(descriptor, editor) {
            _super.call(this, descriptor, editor, lineCommentCommand_1.Type.ForceRemove);
        }
        RemoveLineCommentAction.ID = 'editor.action.removeCommentLine';
        return RemoveLineCommentAction;
    }(CommentLineAction));
    var BlockCommentAction = (function (_super) {
        __extends(BlockCommentAction, _super);
        function BlockCommentAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        BlockCommentAction.prototype.run = function () {
            var commands = [];
            var selections = this.editor.getSelections();
            for (var i = 0; i < selections.length; i++) {
                commands.push(new blockCommentCommand_1.BlockCommentCommand(selections[i]));
            }
            this.editor.executeCommands(this.id, commands);
            return winjs_base_1.TPromise.as(null);
        };
        BlockCommentAction.ID = 'editor.action.blockComment';
        return BlockCommentAction;
    }(editorAction_1.EditorAction));
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(ToggleCommentLineAction, ToggleCommentLineAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.US_SLASH
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(AddLineCommentAction, AddLineCommentAction.ID, nls.localize(1, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_C)
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(RemoveLineCommentAction, RemoveLineCommentAction.ID, nls.localize(2, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_U)
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(BlockCommentAction, BlockCommentAction.ID, nls.localize(3, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_A,
        linux: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_A }
    }));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/contextmenu/browser/contextmenu", ["require", "exports", 'vs/nls!vs/editor/contrib/contextmenu/browser/contextmenu', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/sortedList', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/base/browser/ui/actionbar/actionbar', 'vs/platform/contextview/browser/contextView', 'vs/platform/keybinding/common/keybindingService', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/browser/editorBrowserExtensions'], function (require, exports, nls, keyCodes_1, lifecycle_1, sortedList_1, winjs_base_1, dom, actionbar_1, contextView_1, keybindingService_1, editorAction_1, editorActionEnablement_1, editorCommon_1, editorCommonExtensions_1, editorBrowserExtensions_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ContextMenuController = (function () {
        function ContextMenuController(editor, contextMenuService, contextViewService, keybindingService) {
            var _this = this;
            this.contextMenuService = contextMenuService;
            this.contextViewService = contextViewService;
            this.keybindingService = keybindingService;
            this._editor = editor;
            this._toDispose = [];
            this._contextMenuIsBeingShownCount = 0;
            this._toDispose.push(this._editor.addListener2(editorCommon_1.EventType.ContextMenu, function (e) { return _this._onContextMenu(e); }));
            this._toDispose.push(this._editor.addListener2(editorCommon_1.EventType.KeyDown, function (e) {
                if (e.keyCode === keyCodes_1.KeyCode.ContextMenu) {
                    // Chrome is funny like that
                    e.preventDefault();
                    e.stopPropagation();
                    _this.showContextMenu();
                }
            }));
        }
        ContextMenuController.prototype._onContextMenu = function (e) {
            if (!this._editor.getConfiguration().contextmenu) {
                this._editor.focus();
                // Ensure the cursor is at the position of the mouse click
                if (e.target.position && !this._editor.getSelection().containsPosition(e.target.position)) {
                    this._editor.setPosition(e.target.position);
                }
                return; // Context menu is turned off through configuration
            }
            if (e.target.type === editorCommon_1.MouseTargetType.OVERLAY_WIDGET) {
                return; // allow native menu on widgets to support right click on input field for example in find
            }
            e.event.preventDefault();
            if (e.target.type !== editorCommon_1.MouseTargetType.CONTENT_TEXT && e.target.type !== editorCommon_1.MouseTargetType.CONTENT_EMPTY && e.target.type !== editorCommon_1.MouseTargetType.TEXTAREA) {
                return; // only support mouse click into text or native context menu key for now
            }
            // Ensure the editor gets focus if it hasn't, so the right events are being sent to other contributions
            this._editor.focus();
            // Ensure the cursor is at the position of the mouse click
            if (e.target.position && !this._editor.getSelection().containsPosition(e.target.position)) {
                this._editor.setPosition(e.target.position);
            }
            // Unless the user triggerd the context menu through Shift+F10, use the mouse position as menu position
            var forcedPosition;
            if (e.target.type !== editorCommon_1.MouseTargetType.TEXTAREA) {
                forcedPosition = { x: e.event.posx, y: e.event.posy + 1 };
            }
            // Show the context menu
            this.showContextMenu(forcedPosition);
        };
        ContextMenuController.prototype.showContextMenu = function (forcedPosition) {
            if (!this._editor.getConfiguration().contextmenu) {
                return; // Context menu is turned off through configuration
            }
            if (!this.contextMenuService) {
                this._editor.focus();
                return; // We need the context menu service to function
            }
            var position = this._editor.getPosition();
            var editorModel = this._editor.getModel();
            if (!position || !editorModel) {
                return;
            }
            // Find actions available for menu
            var menuActions = this._getMenuActions();
            // Show menu if we have actions to show
            if (menuActions.length > 0) {
                this._doShowContextMenu(menuActions, forcedPosition);
            }
        };
        ContextMenuController.prototype._getMenuActions = function () {
            var editorModel = this._editor.getModel();
            if (!editorModel) {
                return [];
            }
            var allActions = this._editor.getActions();
            var contributedActions = allActions.filter(function (action) { return (typeof action.shouldShowInContextMenu === 'function') && action.shouldShowInContextMenu() && action.isSupported(); });
            return this._prepareActions(contributedActions);
        };
        ContextMenuController.prototype._prepareActions = function (actions) {
            var list = new sortedList_1.SortedList();
            actions.forEach(function (action) {
                var groups = action.getGroupId().split('/');
                var actionsForGroup = list.getValue(groups[0]);
                if (!actionsForGroup) {
                    actionsForGroup = new sortedList_1.SortedList();
                    list.add(groups[0], actionsForGroup);
                }
                actionsForGroup.add(groups[1] || groups[0], action);
            });
            var sortedAndGroupedActions = [];
            var groupIterator = list.getIterator();
            while (groupIterator.moveNext()) {
                var group = groupIterator.current.value;
                var actionsIterator = group.getIterator();
                while (actionsIterator.moveNext()) {
                    var action = actionsIterator.current.value;
                    sortedAndGroupedActions.push(action);
                }
                if (groupIterator.hasNext()) {
                    sortedAndGroupedActions.push(new actionbar_1.Separator());
                }
            }
            return sortedAndGroupedActions;
        };
        ContextMenuController.prototype._doShowContextMenu = function (actions, forcedPosition) {
            var _this = this;
            if (forcedPosition === void 0) { forcedPosition = null; }
            // Make the editor believe one of its widgets is focused
            this._editor.beginForcedWidgetFocus();
            // Disable hover
            var oldHoverSetting = this._editor.getConfiguration().hover;
            this._editor.updateOptions({
                hover: false
            });
            var menuPosition = forcedPosition;
            if (!menuPosition) {
                // Ensure selection is visible
                this._editor.revealPosition(this._editor.getPosition());
                this._editor.render();
                var cursorCoords = this._editor.getScrolledVisiblePosition(this._editor.getPosition());
                // Translate to absolute editor position
                var editorCoords = dom.getDomNodePosition(this._editor.getDomNode());
                var posx = editorCoords.left + cursorCoords.left;
                var posy = editorCoords.top + cursorCoords.top + cursorCoords.height;
                menuPosition = { x: posx, y: posy };
            }
            // Show menu
            this.contextMenuService.showContextMenu({
                getAnchor: function () { return menuPosition; },
                getActions: function () {
                    return winjs_base_1.TPromise.as(actions);
                },
                getActionItem: function (action) {
                    var keybinding = _this._keybindingFor(action);
                    if (keybinding) {
                        return new actionbar_1.ActionItem(action, action, { label: true, keybinding: _this.keybindingService.getLabelFor(keybinding) });
                    }
                    var customActionItem = action;
                    if (typeof customActionItem.getActionItem === 'function') {
                        return customActionItem.getActionItem();
                    }
                    return null;
                },
                getKeyBinding: function (action) {
                    return _this._keybindingFor(action);
                },
                onHide: function (wasCancelled) {
                    _this._contextMenuIsBeingShownCount--;
                    _this._editor.focus();
                    _this._editor.endForcedWidgetFocus();
                    _this._editor.updateOptions({
                        hover: oldHoverSetting
                    });
                }
            });
        };
        ContextMenuController.prototype._keybindingFor = function (action) {
            var opts = this.keybindingService.lookupKeybindings(action.id);
            if (opts.length > 0) {
                return opts[0]; // only take the first one
            }
            return null;
        };
        ContextMenuController.prototype.getId = function () {
            return ContextMenuController.ID;
        };
        ContextMenuController.prototype.dispose = function () {
            if (this._contextMenuIsBeingShownCount > 0) {
                this.contextViewService.hideContextView();
            }
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        ContextMenuController.ID = 'editor.contrib.contextmenu';
        ContextMenuController = __decorate([
            __param(1, contextView_1.IContextMenuService),
            __param(2, contextView_1.IContextViewService),
            __param(3, keybindingService_1.IKeybindingService)
        ], ContextMenuController);
        return ContextMenuController;
    }());
    var ShowContextMenu = (function (_super) {
        __extends(ShowContextMenu, _super);
        function ShowContextMenu(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
        }
        ShowContextMenu.prototype.run = function () {
            var contribution = this.editor.getContribution(ContextMenuController.ID);
            if (!contribution) {
                return winjs_base_1.TPromise.as(null);
            }
            contribution.showContextMenu();
            return winjs_base_1.TPromise.as(null);
        };
        ShowContextMenu.ID = 'editor.action.showContextMenu';
        return ShowContextMenu;
    }(editorAction_1.EditorAction));
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(ContextMenuController);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(ShowContextMenu, ShowContextMenu.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.F10
    }));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/// <amd-dependency path="vs/css!./folding" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/folding/browser/folding", ["require", "exports", 'vs/nls!vs/editor/contrib/folding/browser/folding', 'vs/base/common/async', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/core/range', 'vs/editor/common/editorCommonExtensions', 'vs/editor/browser/editorBrowserExtensions', 'vs/editor/contrib/folding/common/indentFoldStrategy', "vs/css!./folding"], function (require, exports, nls, async_1, keyCodes_1, lifecycle_1, winjs_base_1, editorAction_1, editorActionEnablement_1, editorCommon, range_1, editorCommonExtensions_1, editorBrowserExtensions_1, indentFoldStrategy_1) {
    'use strict';
    var CollapsibleRegion = (function () {
        function CollapsibleRegion(range, model, changeAccessor) {
            this.decorationIds = [];
            this.update(range, model, changeAccessor);
        }
        Object.defineProperty(CollapsibleRegion.prototype, "isCollapsed", {
            get: function () {
                return this._isCollapsed;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CollapsibleRegion.prototype, "indent", {
            get: function () {
                return this._indent;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CollapsibleRegion.prototype, "startLineNumber", {
            get: function () {
                return this._lastRange ? this._lastRange.startLineNumber : void 0;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CollapsibleRegion.prototype, "endLineNumber", {
            get: function () {
                return this._lastRange ? this._lastRange.endLineNumber : void 0;
            },
            enumerable: true,
            configurable: true
        });
        CollapsibleRegion.prototype.setCollapsed = function (isCollaped, changeAccessor) {
            this._isCollapsed = isCollaped;
            if (this.decorationIds.length > 0) {
                changeAccessor.changeDecorationOptions(this.decorationIds[0], this.getVisualDecorationOptions());
            }
        };
        CollapsibleRegion.prototype.getDecorationRange = function (model) {
            if (this.decorationIds.length > 0) {
                return model.getDecorationRange(this.decorationIds[1]);
            }
            return null;
        };
        CollapsibleRegion.prototype.getVisualDecorationOptions = function () {
            if (this._isCollapsed) {
                return {
                    stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
                    inlineClassName: 'inline-folded',
                    linesDecorationsClassName: 'folding collapsed'
                };
            }
            else {
                return {
                    stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
                    linesDecorationsClassName: 'folding'
                };
            }
        };
        CollapsibleRegion.prototype.getRangeDecorationOptions = function () {
            return {
                stickiness: editorCommon.TrackedRangeStickiness.GrowsOnlyWhenTypingBefore
            };
        };
        CollapsibleRegion.prototype.update = function (newRange, model, changeAccessor) {
            this._lastRange = newRange;
            this._isCollapsed = !!newRange.isCollapsed;
            this._indent = newRange.indent;
            var newDecorations = [];
            var maxColumn = model.getLineMaxColumn(newRange.startLineNumber);
            var visualRng = {
                startLineNumber: newRange.startLineNumber,
                startColumn: maxColumn - 1,
                endLineNumber: newRange.startLineNumber,
                endColumn: maxColumn
            };
            newDecorations.push({ range: visualRng, options: this.getVisualDecorationOptions() });
            var colRng = {
                startLineNumber: newRange.startLineNumber,
                startColumn: 1,
                endLineNumber: newRange.endLineNumber,
                endColumn: model.getLineMaxColumn(newRange.endLineNumber)
            };
            newDecorations.push({ range: colRng, options: this.getRangeDecorationOptions() });
            this.decorationIds = changeAccessor.deltaDecorations(this.decorationIds, newDecorations);
        };
        CollapsibleRegion.prototype.dispose = function (changeAccessor) {
            this._lastRange = null;
            this.decorationIds = changeAccessor.deltaDecorations(this.decorationIds, []);
        };
        CollapsibleRegion.prototype.toString = function () {
            var str = this.isCollapsed ? 'collapsed ' : 'expanded ';
            if (this._lastRange) {
                str += (this._lastRange.startLineNumber + '/' + this._lastRange.endLineNumber);
            }
            else {
                str += 'no range';
            }
            return str;
        };
        return CollapsibleRegion;
    }());
    var FoldingController = (function () {
        function FoldingController(editor) {
            var _this = this;
            this.editor = editor;
            this.globalToDispose = [];
            this.localToDispose = [];
            this.decorations = [];
            this.computeToken = 0;
            this.globalToDispose.push(this.editor.addListener2(editorCommon.EventType.ModelChanged, function () { return _this.onModelChanged(); }));
            this.globalToDispose.push(this.editor.addListener2(editorCommon.EventType.ConfigurationChanged, function (e) {
                if (e.folding) {
                    _this.onModelChanged();
                }
            }));
            this.onModelChanged();
        }
        FoldingController.getFoldingController = function (editor) {
            return editor.getContribution(FoldingController.ID);
        };
        FoldingController.prototype.getId = function () {
            return FoldingController.ID;
        };
        FoldingController.prototype.dispose = function () {
            this.cleanState();
            this.globalToDispose = lifecycle_1.disposeAll(this.globalToDispose);
        };
        /**
         * Store view state.
         */
        FoldingController.prototype.saveViewState = function () {
            var model = this.editor.getModel();
            if (!model) {
                return {};
            }
            var collapsedRegions = [];
            this.decorations.forEach(function (d) {
                if (d.isCollapsed) {
                    var range = d.getDecorationRange(model);
                    if (range) {
                        collapsedRegions.push({ startLineNumber: range.startLineNumber, endLineNumber: range.endLineNumber, indent: d.indent, isCollapsed: true });
                    }
                }
            });
            return { collapsedRegions: collapsedRegions, lineCount: model.getLineCount() };
        };
        /**
         * Restore view state.
         */
        FoldingController.prototype.restoreViewState = function (state) {
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            if (!this.editor.getConfiguration().folding) {
                return;
            }
            if (!state || !Array.isArray(state.collapsedRegions) || state.collapsedRegions.length === 0 || state.lineCount !== model.getLineCount()) {
                return;
            }
            this.applyRegions(state.collapsedRegions);
        };
        FoldingController.prototype.cleanState = function () {
            this.localToDispose = lifecycle_1.disposeAll(this.localToDispose);
        };
        FoldingController.prototype.applyRegions = function (regions) {
            var _this = this;
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var updateHiddenRegions = false;
            regions = indentFoldStrategy_1.limitByIndent(regions, FoldingController.MAX_FOLDING_REGIONS).sort(function (r1, r2) { return r1.startLineNumber - r2.startLineNumber; });
            this.editor.changeDecorations(function (changeAccessor) {
                var newDecorations = [];
                var k = 0, i = 0;
                while (i < _this.decorations.length && k < regions.length) {
                    var dec = _this.decorations[i];
                    var decRange = dec.getDecorationRange(model);
                    if (!decRange) {
                        updateHiddenRegions = updateHiddenRegions || dec.isCollapsed;
                        dec.dispose(changeAccessor);
                        i++;
                    }
                    else {
                        while (k < regions.length && decRange.startLineNumber > regions[k].startLineNumber) {
                            var region = regions[k];
                            updateHiddenRegions = updateHiddenRegions || region.isCollapsed;
                            newDecorations.push(new CollapsibleRegion(region, model, changeAccessor));
                            k++;
                        }
                        if (k < regions.length) {
                            var currRange = regions[k];
                            if (decRange.startLineNumber < currRange.startLineNumber) {
                                updateHiddenRegions = updateHiddenRegions || dec.isCollapsed;
                                dec.dispose(changeAccessor);
                                i++;
                            }
                            else if (decRange.startLineNumber === currRange.startLineNumber) {
                                if (dec.isCollapsed && (dec.startLineNumber !== currRange.startLineNumber || dec.endLineNumber !== currRange.endLineNumber)) {
                                    updateHiddenRegions = true;
                                }
                                currRange.isCollapsed = dec.isCollapsed; // preserve collapse state
                                dec.update(currRange, model, changeAccessor);
                                newDecorations.push(dec);
                                i++;
                                k++;
                            }
                        }
                    }
                }
                while (i < _this.decorations.length) {
                    var dec = _this.decorations[i];
                    updateHiddenRegions = updateHiddenRegions || dec.isCollapsed;
                    dec.dispose(changeAccessor);
                    i++;
                }
                while (k < regions.length) {
                    var region = regions[k];
                    updateHiddenRegions = updateHiddenRegions || region.isCollapsed;
                    newDecorations.push(new CollapsibleRegion(region, model, changeAccessor));
                    k++;
                }
                _this.decorations = newDecorations;
            });
            if (updateHiddenRegions) {
                this.updateHiddenAreas(void 0);
            }
        };
        FoldingController.prototype.onModelChanged = function () {
            var _this = this;
            this.cleanState();
            var model = this.editor.getModel();
            if (!this.editor.getConfiguration().folding || !model) {
                return;
            }
            this.contentChangedScheduler = new async_1.RunOnceScheduler(function () {
                var myToken = (++_this.computeToken);
                _this.computeCollapsibleRegions().then(function (regions) {
                    if (myToken !== _this.computeToken) {
                        return; // A new request was made in the meantime or the model was changed
                    }
                    _this.applyRegions(regions);
                });
            }, 200);
            this.cursorChangedScheduler = new async_1.RunOnceScheduler(function () {
                _this.revealCursor();
            }, 200);
            this.localToDispose.push(this.contentChangedScheduler);
            this.localToDispose.push(this.cursorChangedScheduler);
            this.localToDispose.push(this.editor.addListener2('change', function () {
                _this.contentChangedScheduler.schedule();
            }));
            this.localToDispose.push({ dispose: function () {
                    ++_this.computeToken;
                    _this.editor.changeDecorations(function (changeAccessor) {
                        _this.decorations.forEach(function (dec) { return dec.dispose(changeAccessor); });
                    });
                    _this.decorations = [];
                    _this.editor.setHiddenAreas([]);
                } });
            this.localToDispose.push(this.editor.addListener2(editorCommon.EventType.MouseDown, function (e) { return _this.onEditorMouseDown(e); }));
            this.localToDispose.push(this.editor.addListener2(editorCommon.EventType.MouseUp, function (e) { return _this.onEditorMouseUp(e); }));
            this.localToDispose.push(this.editor.addListener2(editorCommon.EventType.CursorPositionChanged, function (e) {
                _this.cursorChangedScheduler.schedule();
            }));
            this.contentChangedScheduler.schedule();
        };
        FoldingController.prototype.computeCollapsibleRegions = function () {
            var model = this.editor.getModel();
            if (!model) {
                return winjs_base_1.TPromise.as([]);
            }
            var tabSize = model.getOptions().tabSize;
            var ranges = indentFoldStrategy_1.computeRanges(model, tabSize);
            return winjs_base_1.TPromise.as(ranges);
        };
        FoldingController.prototype.revealCursor = function () {
            var _this = this;
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var hasChanges = false;
            var position = this.editor.getPosition();
            var lineNumber = position.lineNumber;
            this.editor.changeDecorations(function (changeAccessor) {
                return _this.decorations.forEach(function (dec) {
                    if (dec.isCollapsed) {
                        var decRange = dec.getDecorationRange(model);
                        // reveal if cursor in in one of the collapsed line (not the first)
                        if (decRange && decRange.startLineNumber < lineNumber && lineNumber <= decRange.endLineNumber) {
                            dec.setCollapsed(false, changeAccessor);
                            hasChanges = true;
                        }
                    }
                });
            });
            if (hasChanges) {
                this.updateHiddenAreas(lineNumber);
            }
        };
        FoldingController.prototype.onEditorMouseDown = function (e) {
            this.mouseDownInfo = null;
            if (this.decorations.length === 0) {
                return;
            }
            var range = e.target.range;
            if (!range || !range.isEmpty) {
                return;
            }
            if (!e.event.leftButton) {
                return;
            }
            var model = this.editor.getModel();
            var iconClicked = false;
            switch (e.target.type) {
                case editorCommon.MouseTargetType.GUTTER_LINE_DECORATIONS:
                    iconClicked = true;
                    break;
                case editorCommon.MouseTargetType.CONTENT_TEXT:
                    if (range.isEmpty && range.startColumn === model.getLineMaxColumn(range.startLineNumber)) {
                        break;
                    }
                    return;
                default:
                    return;
            }
            this.mouseDownInfo = { lineNumber: range.startLineNumber, iconClicked: iconClicked };
        };
        FoldingController.prototype.onEditorMouseUp = function (e) {
            var _this = this;
            if (!this.mouseDownInfo) {
                return;
            }
            var lineNumber = this.mouseDownInfo.lineNumber;
            var iconClicked = this.mouseDownInfo.iconClicked;
            var range = e.target.range;
            if (!range || !range.isEmpty || range.startLineNumber !== lineNumber) {
                return;
            }
            var model = this.editor.getModel();
            if (iconClicked) {
                if (e.target.type !== editorCommon.MouseTargetType.GUTTER_LINE_DECORATIONS) {
                    return;
                }
            }
            else {
                if (range.startColumn !== model.getLineMaxColumn(lineNumber)) {
                    return;
                }
            }
            this.editor.changeDecorations(function (changeAccessor) {
                for (var i = 0; i < _this.decorations.length; i++) {
                    var dec = _this.decorations[i];
                    var decRange = dec.getDecorationRange(model);
                    if (decRange.startLineNumber === lineNumber) {
                        if (iconClicked || dec.isCollapsed) {
                            dec.setCollapsed(!dec.isCollapsed, changeAccessor);
                            _this.updateHiddenAreas(lineNumber);
                        }
                        return;
                    }
                }
            });
        };
        FoldingController.prototype.updateHiddenAreas = function (focusLine) {
            var model = this.editor.getModel();
            var selections = this.editor.getSelections();
            var updateSelections = false;
            var hiddenAreas = [];
            this.decorations.filter(function (dec) { return dec.isCollapsed; }).forEach(function (dec) {
                var decRange = dec.getDecorationRange(model);
                hiddenAreas.push({
                    startLineNumber: decRange.startLineNumber + 1,
                    startColumn: 1,
                    endLineNumber: decRange.endLineNumber,
                    endColumn: 1
                });
                selections.forEach(function (selection, i) {
                    if (range_1.Range.containsPosition(decRange, selection.getStartPosition())) {
                        selections[i] = selection = selection.setStartPosition(decRange.startLineNumber, model.getLineMaxColumn(decRange.startLineNumber));
                        updateSelections = true;
                    }
                    if (range_1.Range.containsPosition(decRange, selection.getEndPosition())) {
                        selections[i] = selection.setEndPosition(decRange.startLineNumber, model.getLineMaxColumn(decRange.startLineNumber));
                        updateSelections = true;
                    }
                });
            });
            var revealPosition;
            if (focusLine) {
                revealPosition = { lineNumber: focusLine, column: 1 };
            }
            else {
                revealPosition = selections[0].getStartPosition();
            }
            if (updateSelections) {
                this.editor.setSelections(selections);
            }
            this.editor.setHiddenAreas(hiddenAreas);
            this.editor.revealPositionInCenterIfOutsideViewport(revealPosition);
        };
        FoldingController.prototype.unfold = function () {
            var _this = this;
            var model = this.editor.getModel();
            var hasChanges = false;
            var selections = this.editor.getSelections();
            var selectionsHasChanged = false;
            selections.forEach(function (selection, index) {
                var lineNumber = selection.startLineNumber;
                var surroundingUnfolded;
                var _loop_1 = function(i, len) {
                    var dec = _this.decorations[i];
                    var decRange = dec.getDecorationRange(model);
                    if (!decRange) {
                        return "continue";
                    }
                    if (decRange.startLineNumber <= lineNumber) {
                        if (lineNumber <= decRange.endLineNumber) {
                            if (dec.isCollapsed) {
                                _this.editor.changeDecorations(function (changeAccessor) {
                                    dec.setCollapsed(false, changeAccessor);
                                    hasChanges = true;
                                });
                                return { value: void 0 };
                            }
                            surroundingUnfolded = decRange;
                        }
                    }
                    else {
                        if (surroundingUnfolded && range_1.Range.containsRange(surroundingUnfolded, decRange)) {
                            if (dec.isCollapsed) {
                                _this.editor.changeDecorations(function (changeAccessor) {
                                    dec.setCollapsed(false, changeAccessor);
                                    hasChanges = true;
                                    var lineNumber = decRange.startLineNumber, column = model.getLineMaxColumn(decRange.startLineNumber);
                                    selections[index] = selection.setEndPosition(lineNumber, column).setStartPosition(lineNumber, column);
                                    selectionsHasChanged = true;
                                });
                                return { value: void 0 };
                            }
                        }
                        else {
                            return { value: void 0 };
                        }
                    }
                };
                for (var i = 0, len = _this.decorations.length; i < len; i++) {
                    var state_1 = _loop_1(i, len);
                    if (typeof state_1 === "object") return state_1.value;
                    if (state_1 === "continue") continue;
                }
            });
            if (selectionsHasChanged) {
                this.editor.setSelections(selections);
            }
            if (hasChanges) {
                this.updateHiddenAreas(selections[0].startLineNumber);
            }
        };
        FoldingController.prototype.fold = function () {
            var _this = this;
            var hasChanges = false;
            var model = this.editor.getModel();
            var selections = this.editor.getSelections();
            selections.forEach(function (selection) {
                var lineNumber = selection.startLineNumber;
                var toFold = null;
                for (var i = 0, len = _this.decorations.length; i < len; i++) {
                    var dec = _this.decorations[i];
                    var decRange = dec.getDecorationRange(model);
                    if (!decRange) {
                        continue;
                    }
                    if (decRange.startLineNumber <= lineNumber) {
                        if (lineNumber <= decRange.endLineNumber && !dec.isCollapsed) {
                            toFold = dec;
                        }
                    }
                    else {
                        break;
                    }
                }
                ;
                if (toFold) {
                    _this.editor.changeDecorations(function (changeAccessor) {
                        toFold.setCollapsed(true, changeAccessor);
                        hasChanges = true;
                    });
                }
            });
            if (hasChanges) {
                this.updateHiddenAreas(selections[0].startLineNumber);
            }
        };
        FoldingController.prototype.changeAll = function (collapse) {
            var _this = this;
            if (this.decorations.length > 0) {
                var hasChanges_1 = true;
                this.editor.changeDecorations(function (changeAccessor) {
                    _this.decorations.forEach(function (d) {
                        if (collapse !== d.isCollapsed) {
                            d.setCollapsed(collapse, changeAccessor);
                            hasChanges_1 = true;
                        }
                    });
                });
                if (hasChanges_1) {
                    this.updateHiddenAreas(void 0);
                }
            }
        };
        FoldingController.prototype.foldLevel = function (foldLevel, selectedLineNumbers) {
            var _this = this;
            var model = this.editor.getModel();
            var foldingRegionStack = [model.getFullModelRange()]; // sentinel
            var hasChanges = false;
            this.editor.changeDecorations(function (changeAccessor) {
                _this.decorations.forEach(function (dec) {
                    var decRange = dec.getDecorationRange(model);
                    if (decRange) {
                        while (!range_1.Range.containsRange(foldingRegionStack[foldingRegionStack.length - 1], decRange)) {
                            foldingRegionStack.pop();
                        }
                        foldingRegionStack.push(decRange);
                        if (foldingRegionStack.length === foldLevel + 1 && !dec.isCollapsed && !selectedLineNumbers.some(function (lineNumber) { return decRange.startLineNumber < lineNumber && lineNumber <= decRange.endLineNumber; })) {
                            dec.setCollapsed(true, changeAccessor);
                            hasChanges = true;
                        }
                    }
                });
            });
            if (hasChanges) {
                this.updateHiddenAreas(selectedLineNumbers[0]);
            }
        };
        FoldingController.ID = 'editor.contrib.folding';
        FoldingController.MAX_FOLDING_REGIONS = 5000;
        return FoldingController;
    }());
    exports.FoldingController = FoldingController;
    var FoldingAction = (function (_super) {
        __extends(FoldingAction, _super);
        function FoldingAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
        }
        FoldingAction.prototype.run = function () {
            var foldingController = FoldingController.getFoldingController(this.editor);
            this.invoke(foldingController);
            return winjs_base_1.TPromise.as(true);
        };
        return FoldingAction;
    }(editorAction_1.EditorAction));
    var UnfoldAction = (function (_super) {
        __extends(UnfoldAction, _super);
        function UnfoldAction() {
            _super.apply(this, arguments);
        }
        UnfoldAction.prototype.invoke = function (foldingController) {
            foldingController.unfold();
        };
        UnfoldAction.ID = 'editor.unfold';
        return UnfoldAction;
    }(FoldingAction));
    var FoldAction = (function (_super) {
        __extends(FoldAction, _super);
        function FoldAction() {
            _super.apply(this, arguments);
        }
        FoldAction.prototype.invoke = function (foldingController) {
            foldingController.fold();
        };
        FoldAction.ID = 'editor.fold';
        return FoldAction;
    }(FoldingAction));
    var FoldAllAction = (function (_super) {
        __extends(FoldAllAction, _super);
        function FoldAllAction() {
            _super.apply(this, arguments);
        }
        FoldAllAction.prototype.invoke = function (foldingController) {
            foldingController.changeAll(true);
        };
        FoldAllAction.ID = 'editor.foldAll';
        return FoldAllAction;
    }(FoldingAction));
    var UnfoldAllAction = (function (_super) {
        __extends(UnfoldAllAction, _super);
        function UnfoldAllAction() {
            _super.apply(this, arguments);
        }
        UnfoldAllAction.prototype.invoke = function (foldingController) {
            foldingController.changeAll(false);
        };
        UnfoldAllAction.ID = 'editor.unfoldAll';
        return UnfoldAllAction;
    }(FoldingAction));
    var FoldLevelAction = (function (_super) {
        __extends(FoldLevelAction, _super);
        function FoldLevelAction() {
            _super.apply(this, arguments);
        }
        FoldLevelAction.prototype.getFoldingLevel = function () {
            return parseInt(this.id.substr(FoldLevelAction.ID_PREFIX.length));
        };
        FoldLevelAction.prototype.getSelectedLines = function () {
            return this.editor.getSelections().map(function (s) { return s.startLineNumber; });
        };
        FoldLevelAction.prototype.invoke = function (foldingController) {
            foldingController.foldLevel(this.getFoldingLevel(), this.getSelectedLines());
        };
        FoldLevelAction.ID_PREFIX = 'editor.foldLevel';
        FoldLevelAction.ID = function (level) { return FoldLevelAction.ID_PREFIX + level; };
        return FoldLevelAction;
    }(FoldingAction));
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(FoldingController);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(UnfoldAction, UnfoldAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.US_CLOSE_SQUARE_BRACKET
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(FoldAction, FoldAction.ID, nls.localize(1, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.US_OPEN_SQUARE_BRACKET
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(FoldAllAction, FoldAllAction.ID, nls.localize(2, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.US_OPEN_SQUARE_BRACKET
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(UnfoldAllAction, UnfoldAllAction.ID, nls.localize(3, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.US_CLOSE_SQUARE_BRACKET,
        secondary: [keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_J)]
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(FoldLevelAction, FoldLevelAction.ID(1), nls.localize(4, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_1)
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(FoldLevelAction, FoldLevelAction.ID(2), nls.localize(5, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_2)
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(FoldLevelAction, FoldLevelAction.ID(3), nls.localize(6, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_3)
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(FoldLevelAction, FoldLevelAction.ID(4), nls.localize(7, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_4)
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(FoldLevelAction, FoldLevelAction.ID(5), nls.localize(8, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_5)
    }));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/format/common/format", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/editor/common/core/range', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry', 'vs/editor/common/services/modelService'], function (require, exports, errors_1, uri_1, winjs_base_1, range_1, editorCommonExtensions_1, languageFeatureRegistry_1, modelService_1) {
    'use strict';
    exports.FormatRegistry = new languageFeatureRegistry_1.default('formattingSupport');
    exports.FormatOnTypeRegistry = new languageFeatureRegistry_1.default('formattingSupport');
    function formatRange(model, range, options) {
        var support = exports.FormatRegistry.ordered(model)
            .filter(function (s) { return typeof s.formatRange === 'function'; })[0];
        if (!support) {
            return winjs_base_1.TPromise.as(undefined);
        }
        return support.formatRange(model.getAssociatedResource(), range, options);
    }
    exports.formatRange = formatRange;
    function formatDocument(model, options) {
        var support = exports.FormatRegistry.ordered(model)[0];
        if (!support) {
            return winjs_base_1.TPromise.as(undefined);
        }
        if (typeof support.formatDocument !== 'function') {
            if (typeof support.formatRange === 'function') {
                return formatRange(model, model.getFullModelRange(), options);
            }
            else {
                return winjs_base_1.TPromise.as(undefined);
            }
        }
        return support.formatDocument(model.getAssociatedResource(), options);
    }
    exports.formatDocument = formatDocument;
    function formatAfterKeystroke(model, position, ch, options) {
        var support = exports.FormatOnTypeRegistry.ordered(model)[0];
        if (!support) {
            return winjs_base_1.TPromise.as(undefined);
        }
        if (support.autoFormatTriggerCharacters.indexOf(ch) < 0) {
            return winjs_base_1.TPromise.as(undefined);
        }
        return support.formatAfterKeystroke(model.getAssociatedResource(), position, ch, options);
    }
    exports.formatAfterKeystroke = formatAfterKeystroke;
    editorCommonExtensions_1.CommonEditorRegistry.registerLanguageCommand('_executeFormatRangeProvider', function (accessor, args) {
        var resource = args.resource, range = args.range, options = args.options;
        if (!(resource instanceof uri_1.default) || !range_1.Range.isIRange(range)) {
            throw errors_1.illegalArgument();
        }
        var model = accessor.get(modelService_1.IModelService).getModel(resource);
        if (!model) {
            throw errors_1.illegalArgument('resource');
        }
        return formatRange(model, range, options);
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerLanguageCommand('_executeFormatDocumentProvider', function (accessor, args) {
        var resource = args.resource, options = args.options;
        if (!(resource instanceof uri_1.default)) {
            throw errors_1.illegalArgument('resource');
        }
        var model = accessor.get(modelService_1.IModelService).getModel(resource);
        if (!model) {
            throw errors_1.illegalArgument('resource');
        }
        return formatDocument(model, options);
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerDefaultLanguageCommand('_executeFormatOnTypeProvider', function (model, position, args) {
        var ch = args.ch, options = args.options;
        if (typeof ch !== 'string') {
            throw errors_1.illegalArgument('ch');
        }
        return formatAfterKeystroke(model, position, ch, options);
    });
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/format/common/formatActions", ["require", "exports", 'vs/nls!vs/editor/contrib/format/common/formatActions', 'vs/base/common/arrays', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', '../common/format', './formatCommand'], function (require, exports, nls, arrays, keyCodes_1, lifecycle_1, winjs_base_1, editorAction_1, editorActionEnablement_1, editorCommon, editorCommonExtensions_1, format_1, formatCommand_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FormatOnType = (function () {
        function FormatOnType(editor) {
            var _this = this;
            this.editor = editor;
            this.callOnDispose = [];
            this.callOnModel = [];
            this.callOnDispose.push(editor.addListener2(editorCommon.EventType.ConfigurationChanged, function () { return _this.update(); }));
            this.callOnDispose.push(editor.addListener2(editorCommon.EventType.ModelChanged, function () { return _this.update(); }));
            this.callOnDispose.push(editor.addListener2(editorCommon.EventType.ModelModeChanged, function () { return _this.update(); }));
            this.callOnDispose.push(editor.addListener2(editorCommon.EventType.ModelModeSupportChanged, function (e) {
                if (e.formattingSupport) {
                    _this.update();
                }
            }));
            this.callOnDispose.push(format_1.FormatOnTypeRegistry.onDidChange(this.update, this));
        }
        FormatOnType.prototype.update = function () {
            var _this = this;
            // clean up
            this.callOnModel = lifecycle_1.cAll(this.callOnModel);
            // we are disabled
            if (!this.editor.getConfiguration().formatOnType) {
                return;
            }
            // no model
            if (!this.editor.getModel()) {
                return;
            }
            var model = this.editor.getModel();
            // no support
            var support = format_1.FormatOnTypeRegistry.ordered(model)[0];
            if (!support || !support.autoFormatTriggerCharacters) {
                return;
            }
            // register typing listeners that will trigger the format
            support.autoFormatTriggerCharacters.forEach(function (ch) {
                _this.callOnModel.push(_this.editor.addTypingListener(ch, _this.trigger.bind(_this, ch)));
            });
        };
        FormatOnType.prototype.trigger = function (ch) {
            var _this = this;
            if (this.editor.getSelections().length > 1) {
                return;
            }
            var model = this.editor.getModel(), position = this.editor.getPosition(), canceled = false;
            // install a listener that checks if edits happens before the
            // position on which we format right now. Iff so, we won't
            // apply the format edits
            var unbind = this.editor.addListener(editorCommon.EventType.ModelContentChanged, function (e) {
                if (e.changeType === editorCommon.EventType.ModelContentChangedFlush) {
                    // a model.setValue() was called
                    canceled = true;
                }
                else if (e.changeType === editorCommon.EventType.ModelContentChangedLineChanged) {
                    var changedLine = e.lineNumber;
                    canceled = changedLine <= position.lineNumber;
                }
                else if (e.changeType === editorCommon.EventType.ModelContentChangedLinesInserted) {
                    var insertLine = e.fromLineNumber;
                    canceled = insertLine <= position.lineNumber;
                }
                else if (e.changeType === editorCommon.EventType.ModelContentChangedLinesDeleted) {
                    var deleteLine2 = e.toLineNumber;
                    canceled = deleteLine2 <= position.lineNumber;
                }
                if (canceled) {
                    // cancel only once
                    unbind();
                }
            });
            var modelOpts = model.getOptions();
            format_1.formatAfterKeystroke(model, position, ch, {
                tabSize: modelOpts.tabSize,
                insertSpaces: modelOpts.insertSpaces
            }).then(function (edits) {
                unbind();
                if (canceled || arrays.isFalsyOrEmpty(edits)) {
                    return;
                }
                _this.editor.executeCommand(_this.getId(), new formatCommand_1.EditOperationsCommand(edits, _this.editor.getSelection()));
            }, function (err) {
                unbind();
                throw err;
            });
        };
        FormatOnType.prototype.getId = function () {
            return FormatOnType.ID;
        };
        FormatOnType.prototype.dispose = function () {
            this.callOnDispose = lifecycle_1.disposeAll(this.callOnDispose);
            while (this.callOnModel.length > 0) {
                this.callOnModel.pop()();
            }
        };
        FormatOnType.ID = 'editor.contrib.autoFormat';
        return FormatOnType;
    }());
    var FormatAction = (function (_super) {
        __extends(FormatAction, _super);
        function FormatAction(descriptor, editor) {
            var _this = this;
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.Writeable | editorActionEnablement_1.Behaviour.UpdateOnModelChange | editorActionEnablement_1.Behaviour.ShowInContextMenu);
            this._disposable = format_1.FormatRegistry.onDidChange(function () { return _this.resetEnablementState(); });
        }
        FormatAction.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this._disposable.dispose();
        };
        FormatAction.prototype.getGroupId = function () {
            return '2_change/2_format';
        };
        FormatAction.prototype.isSupported = function () {
            return format_1.FormatRegistry.has(this.editor.getModel()) && _super.prototype.isSupported.call(this);
        };
        FormatAction.prototype.run = function () {
            var _this = this;
            var model = this.editor.getModel(), editorSelection = this.editor.getSelection(), modelOpts = model.getOptions(), options = {
                tabSize: modelOpts.tabSize,
                insertSpaces: modelOpts.insertSpaces,
            };
            var formattingPromise;
            if (editorSelection.isEmpty()) {
                formattingPromise = format_1.formatDocument(model, options);
            }
            else {
                formattingPromise = format_1.formatRange(model, editorSelection, options);
            }
            if (!formattingPromise) {
                return winjs_base_1.TPromise.as(false);
            }
            // Capture the state of the editor
            var state = this.editor.captureState(editorCommon.CodeEditorStateFlag.Value, editorCommon.CodeEditorStateFlag.Position);
            // Receive formatted value from worker
            return formattingPromise.then(function (result) {
                if (!state.validate(_this.editor)) {
                    return false;
                }
                if (!result || result.length === 0) {
                    return false;
                }
                _this.apply(_this.editor, editorSelection, result);
                _this.editor.focus();
                return true;
            });
        };
        FormatAction.prototype.apply = function (editor, editorSelection, value) {
            var state = null;
            if (editorSelection.isEmpty()) {
                state = editor.saveViewState();
            }
            var command = new formatCommand_1.EditOperationsCommand(value, editorSelection);
            editor.executeCommand(this.id, command);
            if (state) {
                editor.restoreViewState(state);
            }
        };
        FormatAction.ID = 'editor.action.format';
        return FormatAction;
    }(editorAction_1.EditorAction));
    exports.FormatAction = FormatAction;
    // register action
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(FormatAction, FormatAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_F,
        linux: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_I }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorContribution(FormatOnType);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/goToDeclaration/common/goToDeclaration", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/winjs.base', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry'], function (require, exports, errors_1, winjs_base_1, editorCommonExtensions_1, languageFeatureRegistry_1) {
    'use strict';
    exports.DeclarationRegistry = new languageFeatureRegistry_1.default('declarationSupport');
    function getDeclarationsAtPosition(model, position) {
        var resource = model.getAssociatedResource();
        var provider = exports.DeclarationRegistry.ordered(model);
        // get results
        var promises = provider.map(function (provider, idx) {
            return provider.findDeclaration(resource, position).then(function (result) {
                return result;
            }, function (err) {
                errors_1.onUnexpectedError(err);
            });
        });
        return winjs_base_1.TPromise.join(promises).then(function (allReferences) {
            var result = [];
            for (var _i = 0, allReferences_1 = allReferences; _i < allReferences_1.length; _i++) {
                var references = allReferences_1[_i];
                if (Array.isArray(references)) {
                    result.push.apply(result, references);
                }
                else if (references) {
                    result.push(references);
                }
            }
            return result;
        });
    }
    exports.getDeclarationsAtPosition = getDeclarationsAtPosition;
    editorCommonExtensions_1.CommonEditorRegistry.registerDefaultLanguageCommand('_executeDefinitionProvider', getDeclarationsAtPosition);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/gotoError/browser/gotoError", ["require", "exports", 'vs/nls!vs/editor/contrib/gotoError/browser/gotoError', 'vs/base/common/errors', 'vs/base/common/event', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/severity', 'vs/base/common/strings', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/base/browser/htmlContentRenderer', 'vs/platform/editor/common/editor', 'vs/platform/event/common/event', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/markers/common/markers', 'vs/platform/telemetry/common/telemetry', 'vs/editor/common/core/position', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/services/bulkEdit', 'vs/editor/browser/editorBrowserExtensions', 'vs/editor/contrib/zoneWidget/browser/zoneWidget', 'vs/css!./gotoError'], function (require, exports, nls, errors_1, event_1, keyCodes_1, lifecycle_1, severity_1, strings, winjs_base_1, dom, htmlContentRenderer_1, editor_1, event_2, keybindingService_1, markers_1, telemetry_1, position_1, editorAction_1, editorActionEnablement_1, editorCommon, editorCommonExtensions_1, bulkEdit_1, editorBrowserExtensions_1, zoneWidget_1) {
    'use strict';
    var MarkerModel = (function () {
        function MarkerModel(editor, markers) {
            var _this = this;
            this._editor = editor;
            this._markers = null;
            this._nextIdx = -1;
            this._toUnbind = [];
            this._ignoreSelectionChange = false;
            this._onCurrentMarkerChanged = new event_1.Emitter();
            this._onMarkerSetChanged = new event_1.Emitter();
            this.setMarkers(markers);
            // listen on editor
            this._toUnbind.push(this._editor.addListener(editorCommon.EventType.Disposed, function () { return _this.dispose(); }));
            this._toUnbind.push(this._editor.addListener(editorCommon.EventType.CursorPositionChanged, function () {
                if (!_this._ignoreSelectionChange) {
                    _this._nextIdx = -1;
                }
            }));
        }
        Object.defineProperty(MarkerModel.prototype, "onCurrentMarkerChanged", {
            get: function () {
                return this._onCurrentMarkerChanged.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MarkerModel.prototype, "onMarkerSetChanged", {
            get: function () {
                return this._onMarkerSetChanged.event;
            },
            enumerable: true,
            configurable: true
        });
        MarkerModel.prototype.setMarkers = function (markers) {
            // assign
            this._markers = markers || [];
            // sort markers
            this._markers.sort(function (left, right) {
                if (left.startLineNumber === right.startLineNumber) {
                    return left.startColumn - right.startColumn;
                }
                else {
                    return left.startLineNumber - right.startLineNumber;
                }
            });
            this._nextIdx = -1;
            this._onMarkerSetChanged.fire(this);
        };
        MarkerModel.prototype.withoutWatchingEditorPosition = function (callback) {
            this._ignoreSelectionChange = true;
            try {
                callback();
            }
            finally {
                this._ignoreSelectionChange = false;
            }
        };
        MarkerModel.prototype.initIdx = function (fwd) {
            var found = false;
            var position = this._editor.getPosition();
            for (var i = 0, len = this._markers.length; i < len && !found; i++) {
                var pos = { lineNumber: this._markers[i].startLineNumber, column: this._markers[i].startColumn };
                if (position.isBeforeOrEqual(pos)) {
                    this._nextIdx = i + (fwd ? 0 : -1);
                    found = true;
                }
            }
            if (!found) {
                // after the last change
                this._nextIdx = fwd ? 0 : this._markers.length - 1;
            }
            if (this._nextIdx < 0) {
                this._nextIdx = this._markers.length - 1;
            }
        };
        MarkerModel.prototype.move = function (fwd) {
            if (!this.canNavigate()) {
                this._onCurrentMarkerChanged.fire(undefined);
                return;
            }
            if (this._nextIdx === -1) {
                this.initIdx(fwd);
            }
            else if (fwd) {
                this._nextIdx += 1;
                if (this._nextIdx >= this._markers.length) {
                    this._nextIdx = 0;
                }
            }
            else {
                this._nextIdx -= 1;
                if (this._nextIdx < 0) {
                    this._nextIdx = this._markers.length - 1;
                }
            }
            var marker = this._markers[this._nextIdx];
            this._onCurrentMarkerChanged.fire(marker);
        };
        MarkerModel.prototype.canNavigate = function () {
            return this._markers.length > 0;
        };
        MarkerModel.prototype.next = function () {
            this.move(true);
        };
        MarkerModel.prototype.previous = function () {
            this.move(false);
        };
        MarkerModel.prototype.goTo = function (pos) {
            for (var i = 0; i < this._markers.length; i++) {
                var marker = this._markers[i];
                if (marker.startLineNumber <= pos.lineNumber && marker.endLineNumber >= pos.lineNumber
                    && marker.startColumn <= pos.column && marker.endColumn >= pos.column) {
                    this._onCurrentMarkerChanged.fire(marker);
                    return;
                }
            }
            return null;
        };
        MarkerModel.prototype.indexOf = function (marker) {
            return this._markers.indexOf(marker);
        };
        MarkerModel.prototype.length = function () {
            return this._markers.length;
        };
        MarkerModel.prototype.reveal = function () {
            var _this = this;
            if (this._nextIdx === -1) {
                return;
            }
            this.withoutWatchingEditorPosition(function () {
                var pos = new position_1.Position(_this._markers[_this._nextIdx].startLineNumber, _this._markers[_this._nextIdx].startColumn);
                _this._editor.setPosition(pos);
                _this._editor.revealPositionInCenter(pos);
            });
        };
        MarkerModel.prototype.dispose = function () {
            this._toUnbind = lifecycle_1.cAll(this._toUnbind);
        };
        return MarkerModel;
    }());
    var zoneOptions = {
        showFrame: true,
        showArrow: true,
        isAccessible: true
    };
    var MarkerNavigationWidget = (function (_super) {
        __extends(MarkerNavigationWidget, _super);
        function MarkerNavigationWidget(eventService, editorService, editor, _model) {
            _super.call(this, editor, zoneOptions);
            this._model = _model;
            this._callOnDispose = [];
            this._localCleanup = [];
            this._eventService = eventService;
            this._editorService = editorService;
            this.create();
            this._wireModelAndView();
        }
        MarkerNavigationWidget.prototype.fillContainer = function (container) {
            var _this = this;
            this._container = container;
            dom.addClass(this._container, 'marker-widget');
            this._container.tabIndex = 0;
            this._container.setAttribute('role', 'tooltip');
            this._element = document.createElement('div');
            this._element.className = 'descriptioncontainer';
            this._element.setAttribute('aria-live', 'assertive');
            this._element.setAttribute('role', 'alert');
            this._container.appendChild(this._element);
            this._quickFixSection = document.createElement('div');
            this._container.appendChild(this._quickFixSection);
            this._callOnDispose.push(dom.addStandardDisposableListener(this._container, 'keydown', function (e) {
                switch (e.asKeybinding()) {
                    case keyCodes_1.CommonKeybindings.LEFT_ARROW:
                        _this._goLeft();
                        e.preventDefault();
                        e.stopPropagation();
                        break;
                    case keyCodes_1.CommonKeybindings.RIGHT_ARROW:
                        _this._goRight();
                        e.preventDefault();
                        e.stopPropagation();
                        break;
                }
            }));
        };
        MarkerNavigationWidget.prototype._goLeft = function () {
            if (!this._quickFixEntries) {
                return;
            }
            var idx = this._quickFixEntries.indexOf(document.activeElement);
            if (idx === -1) {
                idx = 1;
            }
            idx = (idx + this._quickFixEntries.length - 1) % this._quickFixEntries.length;
            this._quickFixEntries[idx].focus();
        };
        MarkerNavigationWidget.prototype._goRight = function () {
            if (!this._quickFixEntries) {
                return;
            }
            var idx = this._quickFixEntries.indexOf(document.activeElement);
            idx = (idx + 1) % this._quickFixEntries.length;
            this._quickFixEntries[idx].focus();
        };
        MarkerNavigationWidget.prototype.show = function (where, heightInLines) {
            _super.prototype.show.call(this, where, heightInLines);
            this._container.focus();
        };
        MarkerNavigationWidget.prototype._wireModelAndView = function () {
            this._model.onCurrentMarkerChanged(this.showAtMarker, this, this._callOnDispose);
        };
        MarkerNavigationWidget.prototype.showAtMarker = function (marker) {
            var _this = this;
            if (!marker) {
                return;
            }
            // set color
            switch (marker.severity) {
                case severity_1.default.Error:
                    this.options.frameColor = '#ff5a5a';
                    break;
                case severity_1.default.Warning:
                case severity_1.default.Info:
                    this.options.frameColor = '#5aac5a';
                    break;
            }
            this._localCleanup = lifecycle_1.disposeAll(this._localCleanup);
            // update label and show
            var text = strings.format('({0}/{1}) ', this._model.indexOf(marker) + 1, this._model.length());
            if (marker.source) {
                text = text + "[" + marker.source + "] ";
            }
            dom.clearNode(this._element);
            this._element.appendChild(document.createTextNode(text));
            this._element.appendChild(htmlContentRenderer_1.renderHtml(marker.message));
            var mode = this.editor.getModel().getMode();
            this._quickFixSection.style.display = 'none';
            if (mode.quickFixSupport) {
                var promise = mode.quickFixSupport.getQuickFixes(this.editor.getModel().getAssociatedResource(), marker);
                promise.then(function (result) {
                    dom.clearNode(_this._quickFixSection);
                    if (result.length > 0) {
                        _this._localCleanup.push({
                            dispose: function () {
                                _this._quickFixEntries = [];
                            }
                        });
                        var quickfixhead = document.createElement('span');
                        quickfixhead.className = 'quickfixhead';
                        quickfixhead.appendChild(document.createTextNode(result.length > 1 ? nls.localize(0, null) : nls.localize(1, null)));
                        _this._quickFixSection.appendChild(quickfixhead);
                        _this._quickFixEntries = [];
                        var quickfixcontainer_1 = document.createElement('span');
                        quickfixcontainer_1.className = 'quickfixcontainer';
                        result.forEach(function (fix, idx, arr) {
                            var container = quickfixcontainer_1;
                            if (idx > 0) {
                                var separator = document.createElement('span');
                                separator.appendChild(document.createTextNode(', '));
                                container.appendChild(separator);
                            }
                            var entry = document.createElement('a');
                            entry.tabIndex = 0;
                            entry.className = 'quickfixentry';
                            entry.appendChild(document.createTextNode(fix.command.title));
                            _this._localCleanup.push(dom.addDisposableListener(entry, dom.EventType.CLICK, function () {
                                mode.quickFixSupport.runQuickFixAction(_this.editor.getModel().getAssociatedResource(), marker, fix).then(function (result) {
                                    return bulkEdit_1.bulkEdit(_this._eventService, _this._editorService, _this.editor, result.edits);
                                });
                                return true;
                            }));
                            _this._localCleanup.push(dom.addStandardDisposableListener(entry, 'keydown', function (e) {
                                switch (e.asKeybinding()) {
                                    case keyCodes_1.CommonKeybindings.ENTER:
                                    case keyCodes_1.CommonKeybindings.SPACE:
                                        mode.quickFixSupport.runQuickFixAction(_this.editor.getModel().getAssociatedResource(), marker, fix).then(function (result) {
                                            return bulkEdit_1.bulkEdit(_this._eventService, _this._editorService, _this.editor, result.edits);
                                        });
                                        e.preventDefault();
                                        e.stopPropagation();
                                }
                            }));
                            container.appendChild(entry);
                            _this._quickFixEntries.push(entry);
                        });
                        _this._quickFixSection.appendChild(quickfixcontainer_1);
                        _this._quickFixSection.style.display = '';
                        _this.show(new position_1.Position(marker.startLineNumber, marker.startColumn), 4);
                    }
                }, function (error) {
                    errors_1.onUnexpectedError(error);
                });
            }
            this._model.withoutWatchingEditorPosition(function () {
                _this.show(new position_1.Position(marker.startLineNumber, marker.startColumn), 3);
            });
        };
        MarkerNavigationWidget.prototype.dispose = function () {
            this._callOnDispose = lifecycle_1.disposeAll(this._callOnDispose);
            _super.prototype.dispose.call(this);
        };
        return MarkerNavigationWidget;
    }(zoneWidget_1.ZoneWidget));
    var MarkerNavigationAction = (function (_super) {
        __extends(MarkerNavigationAction, _super);
        function MarkerNavigationAction(descriptor, editor, next, telemetryService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.Writeable | editorActionEnablement_1.Behaviour.UpdateOnModelChange);
            this.telemetryService = telemetryService;
            this._isNext = next;
        }
        MarkerNavigationAction.prototype.run = function () {
            var model = MarkerController.getMarkerController(this.editor).getOrCreateModel();
            this.telemetryService.publicLog('zoneWidgetShown', { mode: 'go to error' });
            if (model) {
                if (this._isNext) {
                    model.next();
                }
                else {
                    model.previous();
                }
                model.reveal();
            }
            return winjs_base_1.TPromise.as(true);
        };
        MarkerNavigationAction = __decorate([
            __param(3, telemetry_1.ITelemetryService)
        ], MarkerNavigationAction);
        return MarkerNavigationAction;
    }(editorAction_1.EditorAction));
    var MarkerController = (function () {
        function MarkerController(editor, markerService, keybindingService, eventService, editorService) {
            this._callOnClose = [];
            this.markerService = markerService;
            this.eventService = eventService;
            this.editorService = editorService;
            this.editor = editor;
            this._markersNavigationVisible = keybindingService.createKey(CONTEXT_MARKERS_NAVIGATION_VISIBLE, false);
        }
        MarkerController.getMarkerController = function (editor) {
            return editor.getContribution(MarkerController.ID);
        };
        MarkerController.prototype.getId = function () {
            return MarkerController.ID;
        };
        MarkerController.prototype.dispose = function () {
            this._cleanUp();
        };
        MarkerController.prototype._cleanUp = function () {
            this._markersNavigationVisible.reset();
            this._callOnClose = lifecycle_1.disposeAll(this._callOnClose);
            this._zone = null;
            this._model = null;
        };
        MarkerController.prototype.getOrCreateModel = function () {
            var _this = this;
            if (this._model) {
                return this._model;
            }
            var markers = this._getMarkers();
            this._model = new MarkerModel(this.editor, markers);
            this._zone = new MarkerNavigationWidget(this.eventService, this.editorService, this.editor, this._model);
            this._markersNavigationVisible.set(true);
            this._callOnClose.push(this._model);
            this._callOnClose.push(this._zone);
            this._callOnClose.push(this.editor.addListener2(editorCommon.EventType.ModelChanged, function () {
                _this._cleanUp();
            }));
            this._model.onCurrentMarkerChanged(function (marker) { return !marker && _this._cleanUp(); }, undefined, this._callOnClose);
            this.markerService.onMarkerChanged(this._onMarkerChanged, this, this._callOnClose);
            return this._model;
        };
        MarkerController.prototype.closeMarkersNavigation = function () {
            this._cleanUp();
            this.editor.focus();
        };
        MarkerController.prototype._onMarkerChanged = function (changedResources) {
            var _this = this;
            if (!changedResources.some(function (r) { return _this.editor.getModel().getAssociatedResource().toString() === r.toString(); })) {
                return;
            }
            this._model.setMarkers(this._getMarkers());
        };
        MarkerController.prototype._getMarkers = function () {
            var resource = this.editor.getModel().getAssociatedResource(), markers = this.markerService.read({ resource: resource });
            return markers;
        };
        MarkerController.ID = 'editor.contrib.markerController';
        MarkerController = __decorate([
            __param(1, markers_1.IMarkerService),
            __param(2, keybindingService_1.IKeybindingService),
            __param(3, event_2.IEventService),
            __param(4, editor_1.IEditorService)
        ], MarkerController);
        return MarkerController;
    }());
    var NextMarkerAction = (function (_super) {
        __extends(NextMarkerAction, _super);
        function NextMarkerAction(descriptor, editor, telemetryService) {
            _super.call(this, descriptor, editor, true, telemetryService);
        }
        NextMarkerAction.ID = 'editor.action.marker.next';
        NextMarkerAction = __decorate([
            __param(2, telemetry_1.ITelemetryService)
        ], NextMarkerAction);
        return NextMarkerAction;
    }(MarkerNavigationAction));
    var PrevMarkerAction = (function (_super) {
        __extends(PrevMarkerAction, _super);
        function PrevMarkerAction(descriptor, editor, telemetryService) {
            _super.call(this, descriptor, editor, false, telemetryService);
        }
        PrevMarkerAction.ID = 'editor.action.marker.prev';
        PrevMarkerAction = __decorate([
            __param(2, telemetry_1.ITelemetryService)
        ], PrevMarkerAction);
        return PrevMarkerAction;
    }(MarkerNavigationAction));
    var CONTEXT_MARKERS_NAVIGATION_VISIBLE = 'markersNavigationVisible';
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(NextMarkerAction, NextMarkerAction.ID, nls.localize(2, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyCode.F8
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(PrevMarkerAction, PrevMarkerAction.ID, nls.localize(3, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.F8
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('closeMarkersNavigation', editorCommonExtensions_1.CommonEditorRegistry.commandWeight(50), { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, false, CONTEXT_MARKERS_NAVIGATION_VISIBLE, function (ctx, editor, args) {
        var controller = MarkerController.getMarkerController(editor);
        controller.closeMarkersNavigation();
    });
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(MarkerController);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/hover/common/hover", ["require", "exports", 'vs/base/common/arrays', 'vs/base/common/errors', 'vs/base/common/winjs.base', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry'], function (require, exports, arrays_1, errors_1, winjs_base_1, editorCommonExtensions_1, languageFeatureRegistry_1) {
    'use strict';
    exports.ExtraInfoRegistry = new languageFeatureRegistry_1.default('extraInfoSupport');
    function getExtraInfoAtPosition(model, position) {
        var resource = model.getAssociatedResource();
        var supports = exports.ExtraInfoRegistry.ordered(model);
        var values = [];
        var promises = supports.map(function (support, idx) {
            return support.computeInfo(resource, position).then(function (result) {
                if (result) {
                    var hasRange = (typeof result.range !== 'undefined');
                    var hasValue = (typeof result.value !== 'undefined');
                    var hasHtmlContent = (typeof result.htmlContent !== 'undefined' && result.htmlContent && result.htmlContent.length > 0);
                    if (hasRange && (hasValue || hasHtmlContent)) {
                        values[idx] = result;
                    }
                }
            }, function (err) {
                errors_1.onUnexpectedError(err);
            });
        });
        return winjs_base_1.TPromise.join(promises).then(function () { return arrays_1.coalesce(values); });
    }
    exports.getExtraInfoAtPosition = getExtraInfoAtPosition;
    editorCommonExtensions_1.CommonEditorRegistry.registerDefaultLanguageCommand('_executeHoverProvider', getExtraInfoAtPosition);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/hover/browser/modesContentHover", ["require", "exports", 'vs/nls!vs/editor/contrib/hover/browser/modesContentHover', 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/base/browser/htmlContentRenderer', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/editor/common/core/range', 'vs/editor/common/modes/textToHtmlTokenizer', '../common/hover', './hoverOperation', './hoverWidgets', 'vs/css!vs/base/browser/ui/progressbar/progressbar'], function (require, exports, nls, uri_1, winjs_base_1, htmlContentRenderer_1, keybindingsRegistry_1, range_1, textToHtmlTokenizer_1, hover_1, hoverOperation_1, hoverWidgets_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ModesContentComputer = (function () {
        function ModesContentComputer(editor) {
            this._editor = editor;
            this._range = null;
        }
        ModesContentComputer.prototype.setRange = function (range) {
            this._range = range;
            this._result = [];
        };
        ModesContentComputer.prototype.clearResult = function () {
            this._result = [];
        };
        ModesContentComputer.prototype.computeAsync = function () {
            var model = this._editor.getModel();
            if (!hover_1.ExtraInfoRegistry.has(model)) {
                return winjs_base_1.TPromise.as(null);
            }
            return hover_1.getExtraInfoAtPosition(model, {
                lineNumber: this._range.startLineNumber,
                column: this._range.startColumn
            });
        };
        ModesContentComputer.prototype.computeSync = function () {
            var _this = this;
            var result = [];
            var lineNumber = this._range.startLineNumber;
            if (lineNumber > this._editor.getModel().getLineCount()) {
                // Illegal line number => no results
                return result;
            }
            var lineDecorations = this._editor.getLineDecorations(lineNumber);
            var maxColumn = this._editor.getModel().getLineMaxColumn(lineNumber);
            lineDecorations.forEach(function (d) {
                var startColumn = (d.range.startLineNumber === lineNumber) ? d.range.startColumn : 1;
                var endColumn = (d.range.endLineNumber === lineNumber) ? d.range.endColumn : maxColumn;
                if (startColumn <= _this._range.startColumn && _this._range.endColumn <= endColumn && (d.options.hoverMessage || (d.options.htmlMessage && d.options.htmlMessage.length > 0))) {
                    var obj = {
                        value: d.options.hoverMessage,
                        range: new range_1.Range(_this._range.startLineNumber, startColumn, _this._range.startLineNumber, endColumn)
                    };
                    if (d.options.htmlMessage) {
                        obj.htmlContent = d.options.htmlMessage;
                    }
                    result.push(obj);
                }
            });
            return result;
        };
        ModesContentComputer.prototype.onResult = function (result, isFromSynchronousComputation) {
            // Always put synchronous messages before asynchronous ones
            if (isFromSynchronousComputation) {
                this._result = result.concat(this._result);
            }
            else {
                this._result = this._result.concat(result);
            }
        };
        ModesContentComputer.prototype.getResult = function () {
            return this._result.slice(0);
        };
        ModesContentComputer.prototype.getResultWithLoadingMessage = function () {
            return this._result.slice(0).concat([this._getLoadingMessage()]);
        };
        ModesContentComputer.prototype._getLoadingMessage = function () {
            return {
                range: this._range,
                htmlContent: [{
                        tagName: 'div',
                        className: '',
                        children: [{
                                text: nls.localize(0, null)
                            }]
                    }]
            };
        };
        return ModesContentComputer;
    }());
    var ModesContentHoverWidget = (function (_super) {
        __extends(ModesContentHoverWidget, _super);
        function ModesContentHoverWidget(editor, editorService, keybindingService) {
            var _this = this;
            _super.call(this, ModesContentHoverWidget.ID, editor);
            this._computer = new ModesContentComputer(this._editor);
            this._highlightDecorations = [];
            this._isChangingDecorations = false;
            this._editorService = editorService;
            this._keybindingService = keybindingService;
            this._hoverOperation = new hoverOperation_1.HoverOperation(this._computer, function (result) { return _this._withResult(result, true); }, null, function (result) { return _this._withResult(result, false); });
        }
        ModesContentHoverWidget.prototype.onModelDecorationsChanged = function () {
            if (this._isChangingDecorations) {
                return;
            }
            if (this._isVisible) {
                // The decorations have changed and the hover is visible,
                // we need to recompute the displayed text
                this._hoverOperation.cancel();
                this._computer.clearResult();
                this._hoverOperation.start();
            }
        };
        ModesContentHoverWidget.prototype.startShowingAt = function (range, focus) {
            if (this._lastRange) {
                if (this._lastRange.equalsRange(range)) {
                    // We have to show the widget at the exact same range as before, so no work is needed
                    return;
                }
            }
            this._hoverOperation.cancel();
            if (this._isVisible) {
                // The range might have changed, but the hover is visible
                // Instead of hiding it completely, filter out messages that are still in the new range and
                // kick off a new computation
                if (this._showAtPosition.lineNumber !== range.startLineNumber) {
                    this.hide();
                }
                else {
                    var filteredMessages = [];
                    for (var i = 0, len = this._messages.length; i < len; i++) {
                        var msg = this._messages[i];
                        var rng = msg.range;
                        if (rng.startColumn <= range.startColumn && rng.endColumn >= range.endColumn) {
                            filteredMessages.push(msg);
                        }
                    }
                    if (filteredMessages.length > 0) {
                        this._renderMessages(range, filteredMessages);
                    }
                    else {
                        this.hide();
                    }
                }
            }
            this._lastRange = range;
            this._computer.setRange(range);
            this._shouldFocus = focus;
            this._hoverOperation.start();
        };
        ModesContentHoverWidget.prototype.hide = function () {
            this._lastRange = null;
            this._hoverOperation.cancel();
            _super.prototype.hide.call(this);
            this._isChangingDecorations = true;
            this._highlightDecorations = this._editor.deltaDecorations(this._highlightDecorations, []);
            this._isChangingDecorations = false;
        };
        ModesContentHoverWidget.prototype._withResult = function (result, complete) {
            this._messages = result;
            if (this._lastRange && this._messages.length > 0) {
                this._renderMessages(this._lastRange, this._messages);
            }
            else if (complete) {
                this.hide();
            }
        };
        // TODO@Alex: pull this out into a common utility class
        ModesContentHoverWidget.prototype._renderMessages = function (renderRange, messages) {
            var _this = this;
            // update column from which to show
            var renderColumn = Number.MAX_VALUE, highlightRange = messages[0].range, fragment = document.createDocumentFragment();
            messages.forEach(function (msg) {
                if (!msg.range) {
                    return;
                }
                renderColumn = Math.min(renderColumn, msg.range.startColumn);
                highlightRange = range_1.Range.plusRange(highlightRange, msg.range);
                var row = document.createElement('div');
                var span = null;
                var container = row;
                if (msg.className) {
                    span = document.createElement('span');
                    span.className = msg.className;
                    container = span;
                    row.appendChild(span);
                }
                if (msg.htmlContent && msg.htmlContent.length > 0) {
                    msg.htmlContent.forEach(function (content) {
                        container.appendChild(htmlContentRenderer_1.renderHtml(content, {
                            actionCallback: function (content) {
                                var promise;
                                if (keybindingsRegistry_1.KeybindingsRegistry.getCommands()[content]) {
                                    promise = _this._keybindingService.executeCommand(content);
                                }
                                else {
                                    try {
                                        var resource = uri_1.default.parse(content);
                                        promise = _this._editorService.openEditor({ resource: resource });
                                    }
                                    catch (e) {
                                    }
                                }
                                if (promise) {
                                    promise.then(undefined, function (err) { return console.log(err); });
                                }
                            },
                            codeBlockRenderer: function (modeId, value) {
                                var mode;
                                var model = _this._editor.getModel();
                                if (!model.isDisposed()) {
                                    mode = model.getMode();
                                }
                                return textToHtmlTokenizer_1.tokenizeToString(value, model.getMode());
                            }
                        }));
                    });
                }
                else {
                    container.textContent = msg.value;
                }
                fragment.appendChild(row);
            });
            this._domNode.textContent = '';
            this._domNode.appendChild(fragment);
            // show
            this.showAt({
                lineNumber: renderRange.startLineNumber,
                column: renderColumn
            }, this._shouldFocus);
            this._isChangingDecorations = true;
            this._highlightDecorations = this._editor.deltaDecorations(this._highlightDecorations, [{
                    range: highlightRange,
                    options: {
                        className: 'hoverHighlight'
                    }
                }]);
            this._isChangingDecorations = false;
        };
        ModesContentHoverWidget.ID = 'editor.contrib.modesContentHoverWidget';
        return ModesContentHoverWidget;
    }(hoverWidgets_1.ContentHoverWidget));
    exports.ModesContentHoverWidget = ModesContentHoverWidget;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/hover/browser/hover", ["require", "exports", 'vs/nls!vs/editor/contrib/hover/browser/hover', 'vs/base/common/keyCodes', 'vs/base/common/platform', 'vs/base/common/winjs.base', 'vs/platform/editor/common/editor', 'vs/platform/keybinding/common/keybindingService', 'vs/editor/common/core/range', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/browser/editorBrowserExtensions', './modesContentHover', './modesGlyphHover', 'vs/css!./hover'], function (require, exports, nls, keyCodes_1, platform, winjs_base_1, editor_1, keybindingService_1, range_1, editorAction_1, editorActionEnablement_1, editorCommon, editorCommonExtensions_1, editorBrowserExtensions_1, modesContentHover_1, modesGlyphHover_1) {
    'use strict';
    var ModesHoverController = (function () {
        function ModesHoverController(editor, editorService, keybindingService) {
            var _this = this;
            this._editor = editor;
            this._toUnhook = [];
            if (editor.getConfiguration().hover) {
                this._toUnhook.push(this._editor.addListener(editorCommon.EventType.MouseDown, function (e) { return _this._onEditorMouseDown(e); }));
                this._toUnhook.push(this._editor.addListener(editorCommon.EventType.MouseMove, function (e) { return _this._onEditorMouseMove(e); }));
                this._toUnhook.push(this._editor.addListener(editorCommon.EventType.MouseLeave, function (e) { return _this._hideWidgets(); }));
                this._toUnhook.push(this._editor.addListener(editorCommon.EventType.KeyDown, function (e) { return _this._onKeyDown(e); }));
                this._toUnhook.push(this._editor.addListener(editorCommon.EventType.ModelChanged, function () { return _this._hideWidgets(); }));
                this._toUnhook.push(this._editor.addListener(editorCommon.EventType.ModelDecorationsChanged, function () { return _this._onModelDecorationsChanged(); }));
                this._toUnhook.push(this._editor.addListener('scroll', function () { return _this._hideWidgets(); }));
                this._contentWidget = new modesContentHover_1.ModesContentHoverWidget(editor, editorService, keybindingService);
                this._glyphWidget = new modesGlyphHover_1.ModesGlyphHoverWidget(editor);
            }
        }
        ModesHoverController.getModesHoverController = function (editor) {
            return editor.getContribution(ModesHoverController.ID);
        };
        ModesHoverController.prototype._onModelDecorationsChanged = function () {
            this._contentWidget.onModelDecorationsChanged();
            this._glyphWidget.onModelDecorationsChanged();
        };
        ModesHoverController.prototype._onEditorMouseDown = function (mouseEvent) {
            var targetType = mouseEvent.target.type;
            if (targetType === editorCommon.MouseTargetType.CONTENT_WIDGET && mouseEvent.target.detail === modesContentHover_1.ModesContentHoverWidget.ID) {
                // mouse down on top of content hover widget
                return;
            }
            if (targetType === editorCommon.MouseTargetType.OVERLAY_WIDGET && mouseEvent.target.detail === modesGlyphHover_1.ModesGlyphHoverWidget.ID) {
                // mouse down on top of overlay hover widget
                return;
            }
            this._hideWidgets();
        };
        ModesHoverController.prototype._onEditorMouseMove = function (mouseEvent) {
            var targetType = mouseEvent.target.type;
            var stopKey = platform.isMacintosh ? 'metaKey' : 'ctrlKey';
            if (targetType === editorCommon.MouseTargetType.CONTENT_WIDGET && mouseEvent.target.detail === modesContentHover_1.ModesContentHoverWidget.ID && !mouseEvent.event[stopKey]) {
                // mouse moved on top of content hover widget
                return;
            }
            if (targetType === editorCommon.MouseTargetType.OVERLAY_WIDGET && mouseEvent.target.detail === modesGlyphHover_1.ModesGlyphHoverWidget.ID && !mouseEvent.event[stopKey]) {
                // mouse moved on top of overlay hover widget
                return;
            }
            if (this._editor.getConfiguration().hover && targetType === editorCommon.MouseTargetType.CONTENT_TEXT) {
                this._glyphWidget.hide();
                this._contentWidget.startShowingAt(mouseEvent.target.range, false);
            }
            else if (targetType === editorCommon.MouseTargetType.GUTTER_GLYPH_MARGIN) {
                this._contentWidget.hide();
                this._glyphWidget.startShowingAt(mouseEvent.target.position.lineNumber);
            }
            else {
                this._hideWidgets();
            }
        };
        ModesHoverController.prototype._onKeyDown = function (e) {
            var stopKey = platform.isMacintosh ? keyCodes_1.KeyCode.Meta : keyCodes_1.KeyCode.Ctrl;
            if (e.keyCode !== stopKey) {
                // Do not hide hover when Ctrl/Meta is pressed
                this._hideWidgets();
            }
        };
        ModesHoverController.prototype._hideWidgets = function () {
            this._glyphWidget.hide();
            this._contentWidget.hide();
        };
        ModesHoverController.prototype.showContentHover = function (range, focus) {
            this._contentWidget.startShowingAt(range, focus);
        };
        ModesHoverController.prototype.getId = function () {
            return ModesHoverController.ID;
        };
        ModesHoverController.prototype.dispose = function () {
            while (this._toUnhook.length > 0) {
                this._toUnhook.pop()();
            }
            if (this._glyphWidget) {
                this._glyphWidget.dispose();
                this._glyphWidget = null;
            }
            if (this._contentWidget) {
                this._contentWidget.dispose();
                this._contentWidget = null;
            }
        };
        ModesHoverController.ID = 'editor.contrib.hover';
        ModesHoverController = __decorate([
            __param(1, editor_1.IEditorService),
            __param(2, keybindingService_1.IKeybindingService)
        ], ModesHoverController);
        return ModesHoverController;
    }());
    var ShowHoverAction = (function (_super) {
        __extends(ShowHoverAction, _super);
        function ShowHoverAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
        }
        ShowHoverAction.prototype.run = function () {
            var position = this.editor.getPosition();
            var range = new range_1.Range(position.lineNumber, position.column, position.lineNumber, position.column);
            this.editor.getContribution(ModesHoverController.ID).showContentHover(range, true);
            return winjs_base_1.TPromise.as(null);
        };
        ShowHoverAction.ID = 'editor.action.showHover';
        return ShowHoverAction;
    }(editorAction_1.EditorAction));
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(ModesHoverController);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(ShowHoverAction, ShowHoverAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        kbExpr: keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS),
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_I)
    }));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/inPlaceReplace/common/inPlaceReplace", ["require", "exports", 'vs/nls!vs/editor/contrib/inPlaceReplace/common/inPlaceReplace', 'vs/base/common/keyCodes', 'vs/base/common/winjs.base', 'vs/editor/common/core/range', 'vs/editor/common/editorAction', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/services/editorWorkerService', './inPlaceReplaceCommand'], function (require, exports, nls, keyCodes_1, winjs_base_1, range_1, editorAction_1, editorCommon_1, editorCommonExtensions_1, editorWorkerService_1, inPlaceReplaceCommand_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var InPlaceReplace = (function (_super) {
        __extends(InPlaceReplace, _super);
        function InPlaceReplace(descriptor, editor, up, editorWorkerService) {
            _super.call(this, descriptor, editor);
            this.editorWorkerService = editorWorkerService;
            this.up = up;
            this.requestIdPool = 0;
            this.currentRequest = winjs_base_1.TPromise.as(null);
            this.decorationRemover = winjs_base_1.TPromise.as(null);
            this.decorationIds = [];
        }
        InPlaceReplace.prototype.run = function () {
            var _this = this;
            // cancel any pending request
            this.currentRequest.cancel();
            var selection = this.editor.getSelection(), model = this.editor.getModel(), support = model.getMode().inplaceReplaceSupport, modelURI = model.getAssociatedResource();
            if (selection.startLineNumber !== selection.endLineNumber) {
                // Can't accept multiline selection
                return null;
            }
            var state = this.editor.captureState(editorCommon_1.CodeEditorStateFlag.Value, editorCommon_1.CodeEditorStateFlag.Position);
            this.currentRequest = this.editorWorkerService.navigateValueSet(modelURI, selection, this.up);
            this.currentRequest = this.currentRequest.then(function (basicResult) {
                if (basicResult && basicResult.range && basicResult.value) {
                    return basicResult;
                }
                if (support) {
                    return support.navigateValueSet(modelURI, selection, _this.up);
                }
                return null;
            });
            return this.currentRequest.then(function (result) {
                if (!result || !result.range || !result.value) {
                    // No proper result
                    return;
                }
                if (!state.validate(_this.editor)) {
                    // state has changed
                    return;
                }
                // Selection
                var editRange = range_1.Range.lift(result.range), highlightRange = result.range, diff = result.value.length - (selection.endColumn - selection.startColumn);
                // highlight
                highlightRange.endColumn = highlightRange.startColumn + result.value.length;
                selection.endColumn += diff > 1 ? (diff - 1) : 0;
                // Insert new text
                var command = new inPlaceReplaceCommand_1.InPlaceReplaceCommand(editRange, selection, result.value);
                _this.editor.executeCommand(_this.id, command);
                // add decoration
                _this.decorationIds = _this.editor.deltaDecorations(_this.decorationIds, [{
                        range: highlightRange,
                        options: InPlaceReplace.DECORATION
                    }]);
                // remove decoration after delay
                _this.decorationRemover.cancel();
                _this.decorationRemover = winjs_base_1.TPromise.timeout(350);
                _this.decorationRemover.then(function () {
                    _this.editor.changeDecorations(function (accessor) {
                        _this.decorationIds = accessor.deltaDecorations(_this.decorationIds, []);
                    });
                });
                return true;
            });
        };
        InPlaceReplace.DECORATION = {
            className: 'valueSetReplacement'
        };
        InPlaceReplace = __decorate([
            __param(3, editorWorkerService_1.IEditorWorkerService)
        ], InPlaceReplace);
        return InPlaceReplace;
    }(editorAction_1.EditorAction));
    var InPlaceReplaceUp = (function (_super) {
        __extends(InPlaceReplaceUp, _super);
        function InPlaceReplaceUp(descriptor, editor, editorWorkerService) {
            _super.call(this, descriptor, editor, true, editorWorkerService);
        }
        InPlaceReplaceUp.ID = 'editor.action.inPlaceReplace.up';
        InPlaceReplaceUp = __decorate([
            __param(2, editorWorkerService_1.IEditorWorkerService)
        ], InPlaceReplaceUp);
        return InPlaceReplaceUp;
    }(InPlaceReplace));
    var InPlaceReplaceDown = (function (_super) {
        __extends(InPlaceReplaceDown, _super);
        function InPlaceReplaceDown(descriptor, editor, editorWorkerService) {
            _super.call(this, descriptor, editor, false, editorWorkerService);
        }
        InPlaceReplaceDown.ID = 'editor.action.inPlaceReplace.down';
        InPlaceReplaceDown = __decorate([
            __param(2, editorWorkerService_1.IEditorWorkerService)
        ], InPlaceReplaceDown);
        return InPlaceReplaceDown;
    }(InPlaceReplace));
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(InPlaceReplaceUp, InPlaceReplaceUp.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.US_COMMA
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(InPlaceReplaceDown, InPlaceReplaceDown.ID, nls.localize(1, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.US_DOT
    }));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/linesOperations/common/linesOperations", ["require", "exports", 'vs/nls!vs/editor/contrib/linesOperations/common/linesOperations', 'vs/base/common/keyCodes', 'vs/base/common/winjs.base', 'vs/editor/contrib/linesOperations/common/sortLinesCommand', 'vs/editor/common/commands/trimTrailingWhitespaceCommand', 'vs/editor/common/editorAction', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', './copyLinesCommand', './deleteLinesCommand', './moveLinesCommand'], function (require, exports, nls, keyCodes_1, winjs_base_1, sortLinesCommand_1, trimTrailingWhitespaceCommand_1, editorAction_1, editorCommon_1, editorCommonExtensions_1, copyLinesCommand_1, deleteLinesCommand_1, moveLinesCommand_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // copy lines
    var CopyLinesAction = (function (_super) {
        __extends(CopyLinesAction, _super);
        function CopyLinesAction(descriptor, editor, down) {
            _super.call(this, descriptor, editor);
            this.down = down;
        }
        CopyLinesAction.prototype.run = function () {
            var commands = [];
            var selections = this.editor.getSelections();
            for (var i = 0; i < selections.length; i++) {
                commands.push(new copyLinesCommand_1.CopyLinesCommand(selections[i], this.down));
            }
            this.editor.executeCommands(this.id, commands);
            return winjs_base_1.TPromise.as(true);
        };
        return CopyLinesAction;
    }(editorAction_1.EditorAction));
    var CopyLinesUpAction = (function (_super) {
        __extends(CopyLinesUpAction, _super);
        function CopyLinesUpAction(descriptor, editor) {
            _super.call(this, descriptor, editor, false);
        }
        CopyLinesUpAction.ID = 'editor.action.copyLinesUpAction';
        return CopyLinesUpAction;
    }(CopyLinesAction));
    var CopyLinesDownAction = (function (_super) {
        __extends(CopyLinesDownAction, _super);
        function CopyLinesDownAction(descriptor, editor) {
            _super.call(this, descriptor, editor, true);
        }
        CopyLinesDownAction.ID = 'editor.action.copyLinesDownAction';
        return CopyLinesDownAction;
    }(CopyLinesAction));
    // move lines
    var MoveLinesAction = (function (_super) {
        __extends(MoveLinesAction, _super);
        function MoveLinesAction(descriptor, editor, down) {
            _super.call(this, descriptor, editor);
            this.down = down;
        }
        MoveLinesAction.prototype.run = function () {
            var commands = [];
            var selections = this.editor.getSelections();
            for (var i = 0; i < selections.length; i++) {
                commands.push(new moveLinesCommand_1.MoveLinesCommand(selections[i], this.down));
            }
            this.editor.executeCommands(this.id, commands);
            return winjs_base_1.TPromise.as(true);
        };
        return MoveLinesAction;
    }(editorAction_1.EditorAction));
    var MoveLinesUpAction = (function (_super) {
        __extends(MoveLinesUpAction, _super);
        function MoveLinesUpAction(descriptor, editor) {
            _super.call(this, descriptor, editor, false);
        }
        MoveLinesUpAction.ID = 'editor.action.moveLinesUpAction';
        return MoveLinesUpAction;
    }(MoveLinesAction));
    var MoveLinesDownAction = (function (_super) {
        __extends(MoveLinesDownAction, _super);
        function MoveLinesDownAction(descriptor, editor) {
            _super.call(this, descriptor, editor, true);
        }
        MoveLinesDownAction.ID = 'editor.action.moveLinesDownAction';
        return MoveLinesDownAction;
    }(MoveLinesAction));
    var SortLinesAction = (function (_super) {
        __extends(SortLinesAction, _super);
        function SortLinesAction(descriptor, editor, descending) {
            _super.call(this, descriptor, editor);
            this.descending = descending;
        }
        SortLinesAction.prototype.run = function () {
            var command = new sortLinesCommand_1.SortLinesCommand(this.editor.getSelection(), this.descending);
            this.editor.executeCommands(this.id, [command]);
            return winjs_base_1.TPromise.as(true);
        };
        return SortLinesAction;
    }(editorAction_1.EditorAction));
    var SortLinesAscendingAction = (function (_super) {
        __extends(SortLinesAscendingAction, _super);
        function SortLinesAscendingAction(descriptor, editor) {
            _super.call(this, descriptor, editor, false);
        }
        SortLinesAscendingAction.ID = 'editor.action.sortLinesAscending';
        return SortLinesAscendingAction;
    }(SortLinesAction));
    var SortLinesDescendingAction = (function (_super) {
        __extends(SortLinesDescendingAction, _super);
        function SortLinesDescendingAction(descriptor, editor) {
            _super.call(this, descriptor, editor, true);
        }
        SortLinesDescendingAction.ID = 'editor.action.sortLinesDescending';
        return SortLinesDescendingAction;
    }(SortLinesAction));
    var TrimTrailingWhitespaceAction = (function (_super) {
        __extends(TrimTrailingWhitespaceAction, _super);
        function TrimTrailingWhitespaceAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        TrimTrailingWhitespaceAction.prototype.run = function () {
            var command = new trimTrailingWhitespaceCommand_1.TrimTrailingWhitespaceCommand(this.editor.getSelection());
            this.editor.executeCommands(this.id, [command]);
            return winjs_base_1.TPromise.as(true);
        };
        TrimTrailingWhitespaceAction.ID = 'editor.action.trimTrailingWhitespace';
        return TrimTrailingWhitespaceAction;
    }(editorAction_1.EditorAction));
    var AbstractRemoveLinesAction = (function (_super) {
        __extends(AbstractRemoveLinesAction, _super);
        function AbstractRemoveLinesAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        AbstractRemoveLinesAction.prototype._getLinesToRemove = function () {
            // Construct delete operations
            var operations = this.editor.getSelections().map(function (s) {
                var endLineNumber = s.endLineNumber;
                if (s.startLineNumber < s.endLineNumber && s.endColumn === 1) {
                    endLineNumber -= 1;
                }
                return {
                    startLineNumber: s.startLineNumber,
                    endLineNumber: endLineNumber,
                    positionColumn: s.positionColumn
                };
            });
            // Sort delete operations
            operations.sort(function (a, b) {
                return a.startLineNumber - b.startLineNumber;
            });
            // Merge delete operations on consecutive lines
            var mergedOperations = [];
            var previousOperation = operations[0];
            for (var i = 1; i < operations.length; i++) {
                if (previousOperation.endLineNumber + 1 === operations[i].startLineNumber) {
                    // Merge current operations into the previous one
                    previousOperation.endLineNumber = operations[i].endLineNumber;
                }
                else {
                    // Push previous operation
                    mergedOperations.push(previousOperation);
                    previousOperation = operations[i];
                }
            }
            // Push the last operation
            mergedOperations.push(previousOperation);
            return mergedOperations;
        };
        return AbstractRemoveLinesAction;
    }(editorAction_1.EditorAction));
    var DeleteLinesAction = (function (_super) {
        __extends(DeleteLinesAction, _super);
        function DeleteLinesAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        DeleteLinesAction.prototype.run = function () {
            var ops = this._getLinesToRemove();
            // Finally, construct the delete lines commands
            var commands = ops.map(function (op) {
                return new deleteLinesCommand_1.DeleteLinesCommand(op.startLineNumber, op.endLineNumber, op.positionColumn);
            });
            this.editor.executeCommands(this.id, commands);
            return winjs_base_1.TPromise.as(true);
        };
        DeleteLinesAction.ID = 'editor.action.deleteLines';
        return DeleteLinesAction;
    }(AbstractRemoveLinesAction));
    var IndentLinesAction = (function (_super) {
        __extends(IndentLinesAction, _super);
        function IndentLinesAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorCommon_1.Handler.Indent);
        }
        IndentLinesAction.ID = 'editor.action.indentLines';
        return IndentLinesAction;
    }(editorAction_1.HandlerEditorAction));
    var OutdentLinesAction = (function (_super) {
        __extends(OutdentLinesAction, _super);
        function OutdentLinesAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorCommon_1.Handler.Outdent);
        }
        OutdentLinesAction.ID = 'editor.action.outdentLines';
        return OutdentLinesAction;
    }(editorAction_1.HandlerEditorAction));
    var InsertLineBeforeAction = (function (_super) {
        __extends(InsertLineBeforeAction, _super);
        function InsertLineBeforeAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorCommon_1.Handler.LineInsertBefore);
        }
        InsertLineBeforeAction.ID = 'editor.action.insertLineBefore';
        return InsertLineBeforeAction;
    }(editorAction_1.HandlerEditorAction));
    var InsertLineAfterAction = (function (_super) {
        __extends(InsertLineAfterAction, _super);
        function InsertLineAfterAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorCommon_1.Handler.LineInsertAfter);
        }
        InsertLineAfterAction.ID = 'editor.action.insertLineAfter';
        return InsertLineAfterAction;
    }(editorAction_1.HandlerEditorAction));
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(DeleteLinesAction, DeleteLinesAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_K
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(SortLinesAscendingAction, SortLinesAscendingAction.ID, nls.localize(1, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_2
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(SortLinesDescendingAction, SortLinesDescendingAction.ID, nls.localize(2, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_3
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(TrimTrailingWhitespaceAction, TrimTrailingWhitespaceAction.ID, nls.localize(3, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_X
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(MoveLinesDownAction, MoveLinesDownAction.ID, nls.localize(4, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.DownArrow,
        linux: { primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.DownArrow }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(MoveLinesUpAction, MoveLinesUpAction.ID, nls.localize(5, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.UpArrow,
        linux: { primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.UpArrow }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(CopyLinesDownAction, CopyLinesDownAction.ID, nls.localize(6, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.DownArrow,
        linux: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.DownArrow }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(CopyLinesUpAction, CopyLinesUpAction.ID, nls.localize(7, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.UpArrow,
        linux: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.UpArrow }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(IndentLinesAction, IndentLinesAction.ID, nls.localize(8, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.US_CLOSE_SQUARE_BRACKET
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(OutdentLinesAction, OutdentLinesAction.ID, nls.localize(9, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.US_OPEN_SQUARE_BRACKET
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(InsertLineBeforeAction, InsertLineBeforeAction.ID, nls.localize(10, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Enter
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(InsertLineAfterAction, InsertLineAfterAction.ID, nls.localize(11, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.Enter
    }));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/links/browser/links", ["require", "exports", 'vs/nls!vs/editor/contrib/links/browser/links', 'vs/base/common/errors', 'vs/base/common/keyCodes', 'vs/base/common/platform', 'vs/base/common/severity', 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/platform/editor/common/editor', 'vs/platform/message/common/message', 'vs/editor/common/core/range', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/services/editorWorkerService', 'vs/css!./links'], function (require, exports, nls, errors_1, keyCodes_1, platform, severity_1, uri_1, winjs_base_1, editor_1, message_1, range_1, editorAction_1, editorActionEnablement_1, editorCommon, editorCommonExtensions_1, editorWorkerService_1) {
    'use strict';
    var LinkOccurence = (function () {
        function LinkOccurence(link, decorationId /*, changeAccessor:editorCommon.IModelDecorationsChangeAccessor*/) {
            this.link = link;
            this.decorationId = decorationId;
        }
        LinkOccurence.decoration = function (link) {
            return {
                range: {
                    startLineNumber: link.range.startLineNumber,
                    startColumn: link.range.startColumn,
                    endLineNumber: link.range.startLineNumber,
                    endColumn: link.range.endColumn
                },
                options: LinkOccurence._getOptions(link, false)
            };
        };
        LinkOccurence._getOptions = function (link, isActive) {
            var result = '';
            if (link.extraInlineClassName) {
                result = link.extraInlineClassName + ' ';
            }
            if (isActive) {
                result += LinkDetector.CLASS_NAME_ACTIVE;
            }
            else {
                result += LinkDetector.CLASS_NAME;
            }
            return {
                stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
                inlineClassName: result,
                hoverMessage: LinkDetector.HOVER_MESSAGE_GENERAL
            };
        };
        LinkOccurence.prototype.activate = function (changeAccessor) {
            changeAccessor.changeDecorationOptions(this.decorationId, LinkOccurence._getOptions(this.link, true));
        };
        LinkOccurence.prototype.deactivate = function (changeAccessor) {
            changeAccessor.changeDecorationOptions(this.decorationId, LinkOccurence._getOptions(this.link, false));
        };
        return LinkOccurence;
    }());
    var LinkDetector = (function () {
        function LinkDetector(editor, editorService, messageService, editorWorkerService) {
            var _this = this;
            this.editor = editor;
            this.editorService = editorService;
            this.messageService = messageService;
            this.editorWorkerService = editorWorkerService;
            this.listenersToRemove = [];
            this.listenersToRemove.push(editor.addListener('change', function (e) { return _this.onChange(); }));
            this.listenersToRemove.push(editor.addListener(editorCommon.EventType.ModelChanged, function (e) { return _this.onModelChanged(); }));
            this.listenersToRemove.push(editor.addListener(editorCommon.EventType.ModelModeChanged, function (e) { return _this.onModelModeChanged(); }));
            this.listenersToRemove.push(editor.addListener(editorCommon.EventType.ModelModeSupportChanged, function (e) {
                if (e.linkSupport) {
                    _this.onModelModeChanged();
                }
            }));
            this.listenersToRemove.push(this.editor.addListener(editorCommon.EventType.MouseUp, function (e) { return _this.onEditorMouseUp(e); }));
            this.listenersToRemove.push(this.editor.addListener(editorCommon.EventType.MouseMove, function (e) { return _this.onEditorMouseMove(e); }));
            this.listenersToRemove.push(this.editor.addListener(editorCommon.EventType.KeyDown, function (e) { return _this.onEditorKeyDown(e); }));
            this.listenersToRemove.push(this.editor.addListener(editorCommon.EventType.KeyUp, function (e) { return _this.onEditorKeyUp(e); }));
            this.timeoutPromise = null;
            this.computePromise = null;
            this.currentOccurences = {};
            this.activeLinkDecorationId = null;
            this.beginCompute();
        }
        LinkDetector.prototype.isComputing = function () {
            return winjs_base_1.TPromise.is(this.computePromise);
        };
        LinkDetector.prototype.onModelChanged = function () {
            this.lastMouseEvent = null;
            this.currentOccurences = {};
            this.activeLinkDecorationId = null;
            this.stop();
            this.beginCompute();
        };
        LinkDetector.prototype.onModelModeChanged = function () {
            this.stop();
            this.beginCompute();
        };
        LinkDetector.prototype.onChange = function () {
            var _this = this;
            if (!this.timeoutPromise) {
                this.timeoutPromise = winjs_base_1.TPromise.timeout(LinkDetector.RECOMPUTE_TIME);
                this.timeoutPromise.then(function () {
                    _this.timeoutPromise = null;
                    _this.beginCompute();
                });
            }
        };
        LinkDetector.prototype.beginCompute = function () {
            var _this = this;
            if (!this.editor.getModel()) {
                return;
            }
            var modePromise = winjs_base_1.TPromise.as(null);
            var mode = this.editor.getModel().getMode();
            if (mode.linkSupport) {
                modePromise = mode.linkSupport.computeLinks(this.editor.getModel().getAssociatedResource());
            }
            var standardPromise = this.editorWorkerService.computeLinks(this.editor.getModel().getAssociatedResource());
            this.computePromise = winjs_base_1.TPromise.join([modePromise, standardPromise]).then(function (r) {
                var a = r[0];
                var b = r[1];
                if (!a || a.length === 0) {
                    return b || [];
                }
                if (!b || b.length === 0) {
                    return a || [];
                }
                return LinkDetector._linksUnion(a, b);
            });
            this.computePromise.then(function (links) {
                _this.updateDecorations(links);
                _this.computePromise = null;
            });
        };
        LinkDetector._linksUnion = function (oldLinks, newLinks) {
            // reunite oldLinks with newLinks and remove duplicates
            var result = [], oldIndex, oldLen, newIndex, newLen, oldLink, newLink, comparisonResult;
            for (oldIndex = 0, newIndex = 0, oldLen = oldLinks.length, newLen = newLinks.length; oldIndex < oldLen && newIndex < newLen;) {
                oldLink = oldLinks[oldIndex];
                newLink = newLinks[newIndex];
                if (range_1.Range.areIntersectingOrTouching(oldLink.range, newLink.range)) {
                    // Remove the oldLink
                    oldIndex++;
                    continue;
                }
                comparisonResult = range_1.Range.compareRangesUsingStarts(oldLink.range, newLink.range);
                if (comparisonResult < 0) {
                    // oldLink is before
                    result.push(oldLink);
                    oldIndex++;
                }
                else {
                    // newLink is before
                    result.push(newLink);
                    newIndex++;
                }
            }
            for (; oldIndex < oldLen; oldIndex++) {
                result.push(oldLinks[oldIndex]);
            }
            for (; newIndex < newLen; newIndex++) {
                result.push(newLinks[newIndex]);
            }
            return result;
        };
        LinkDetector.prototype.updateDecorations = function (links) {
            var _this = this;
            this.editor.changeDecorations(function (changeAccessor) {
                var oldDecorations = [];
                for (var decorationId in _this.currentOccurences) {
                    if (_this.currentOccurences.hasOwnProperty(decorationId)) {
                        var occurance = _this.currentOccurences[decorationId];
                        oldDecorations.push(occurance.decorationId);
                    }
                }
                var newDecorations = [];
                if (links) {
                    // Not sure why this is sometimes null
                    for (var i = 0; i < links.length; i++) {
                        newDecorations.push(LinkOccurence.decoration(links[i]));
                    }
                }
                var decorations = changeAccessor.deltaDecorations(oldDecorations, newDecorations);
                _this.currentOccurences = {};
                _this.activeLinkDecorationId = null;
                for (var i_1 = 0, len = decorations.length; i_1 < len; i_1++) {
                    var occurance = new LinkOccurence(links[i_1], decorations[i_1]);
                    _this.currentOccurences[occurance.decorationId] = occurance;
                }
            });
        };
        LinkDetector.prototype.onEditorKeyDown = function (e) {
            if (e.keyCode === LinkDetector.TRIGGER_KEY_VALUE && this.lastMouseEvent) {
                this.onEditorMouseMove(this.lastMouseEvent, e);
            }
        };
        LinkDetector.prototype.onEditorKeyUp = function (e) {
            if (e.keyCode === LinkDetector.TRIGGER_KEY_VALUE) {
                this.cleanUpActiveLinkDecoration();
            }
        };
        LinkDetector.prototype.onEditorMouseMove = function (mouseEvent, withKey) {
            var _this = this;
            this.lastMouseEvent = mouseEvent;
            if (this.isEnabled(mouseEvent, withKey)) {
                this.cleanUpActiveLinkDecoration(); // always remove previous link decoration as their can only be one
                var occurence = this.getLinkOccurence(mouseEvent.target.position);
                if (occurence) {
                    this.editor.changeDecorations(function (changeAccessor) {
                        occurence.activate(changeAccessor);
                        _this.activeLinkDecorationId = occurence.decorationId;
                    });
                }
            }
            else {
                this.cleanUpActiveLinkDecoration();
            }
        };
        LinkDetector.prototype.cleanUpActiveLinkDecoration = function () {
            if (this.activeLinkDecorationId) {
                var occurence = this.currentOccurences[this.activeLinkDecorationId];
                if (occurence) {
                    this.editor.changeDecorations(function (changeAccessor) {
                        occurence.deactivate(changeAccessor);
                    });
                }
                this.activeLinkDecorationId = null;
            }
        };
        LinkDetector.prototype.onEditorMouseUp = function (mouseEvent) {
            if (!this.isEnabled(mouseEvent)) {
                return;
            }
            var occurence = this.getLinkOccurence(mouseEvent.target.position);
            if (!occurence) {
                return;
            }
            this.openLinkOccurence(occurence, mouseEvent.event.altKey);
        };
        LinkDetector.prototype.openLinkOccurence = function (occurence, openToSide) {
            if (!this.editorService) {
                return;
            }
            var link = occurence.link;
            var absoluteUrl = link.url;
            var hashIndex = absoluteUrl.indexOf('#');
            var lineNumber = -1;
            var column = -1;
            if (hashIndex >= 0) {
                var hash = absoluteUrl.substr(hashIndex + 1);
                var selection = hash.split(',');
                if (selection.length > 0) {
                    lineNumber = Number(selection[0]);
                }
                if (selection.length > 1) {
                    column = Number(selection[1]);
                }
                if (lineNumber >= 0 || column >= 0) {
                    absoluteUrl = absoluteUrl.substr(0, hashIndex);
                }
            }
            var url;
            try {
                url = uri_1.default.parse(absoluteUrl);
            }
            catch (err) {
                // invalid url
                this.messageService.show(severity_1.default.Warning, nls.localize(2, null, absoluteUrl));
                return;
            }
            var input = {
                resource: url
            };
            if (lineNumber >= 0) {
                input.options = {
                    selection: { startLineNumber: lineNumber, startColumn: column }
                };
            }
            this.editorService.openEditor(input, openToSide).done(null, errors_1.onUnexpectedError);
        };
        LinkDetector.prototype.getLinkOccurence = function (position) {
            var decorations = this.editor.getModel().getDecorationsInRange({
                startLineNumber: position.lineNumber,
                startColumn: position.column,
                endLineNumber: position.lineNumber,
                endColumn: position.column
            }, null, true);
            for (var i = 0; i < decorations.length; i++) {
                var decoration = decorations[i];
                var currentOccurence = this.currentOccurences[decoration.id];
                if (currentOccurence) {
                    return currentOccurence;
                }
            }
            return null;
        };
        LinkDetector.prototype.isEnabled = function (mouseEvent, withKey) {
            return mouseEvent.target.type === editorCommon.MouseTargetType.CONTENT_TEXT &&
                (mouseEvent.event[LinkDetector.TRIGGER_MODIFIER] || (withKey && withKey.keyCode === LinkDetector.TRIGGER_KEY_VALUE));
        };
        LinkDetector.prototype.stop = function () {
            if (this.timeoutPromise) {
                this.timeoutPromise.cancel();
                this.timeoutPromise = null;
            }
            if (this.computePromise) {
                this.computePromise.cancel();
                this.computePromise = null;
            }
        };
        LinkDetector.prototype.dispose = function () {
            this.listenersToRemove.forEach(function (element) {
                element();
            });
            this.listenersToRemove = [];
            this.stop();
        };
        LinkDetector.RECOMPUTE_TIME = 1000; // ms
        LinkDetector.TRIGGER_KEY_VALUE = platform.isMacintosh ? keyCodes_1.KeyCode.Meta : keyCodes_1.KeyCode.Ctrl;
        LinkDetector.TRIGGER_MODIFIER = platform.isMacintosh ? 'metaKey' : 'ctrlKey';
        LinkDetector.HOVER_MESSAGE_GENERAL = platform.isMacintosh ? nls.localize(0, null) : nls.localize(1, null);
        LinkDetector.CLASS_NAME = 'detected-link';
        LinkDetector.CLASS_NAME_ACTIVE = 'detected-link-active';
        return LinkDetector;
    }());
    var OpenLinkAction = (function (_super) {
        __extends(OpenLinkAction, _super);
        function OpenLinkAction(descriptor, editor, editorService, messageService, editorWorkerService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.UpdateOnCursorPositionChange);
            this._linkDetector = new LinkDetector(editor, editorService, messageService, editorWorkerService);
        }
        OpenLinkAction.prototype.dispose = function () {
            this._linkDetector.dispose();
            _super.prototype.dispose.call(this);
        };
        OpenLinkAction.prototype.getEnablementState = function () {
            if (this._linkDetector.isComputing()) {
                // optimistic enablement while state is being computed
                return true;
            }
            return !!this._linkDetector.getLinkOccurence(this.editor.getPosition());
        };
        OpenLinkAction.prototype.run = function () {
            var link = this._linkDetector.getLinkOccurence(this.editor.getPosition());
            if (link) {
                this._linkDetector.openLinkOccurence(link, false);
            }
            return winjs_base_1.TPromise.as(null);
        };
        OpenLinkAction.ID = 'editor.action.openLink';
        OpenLinkAction = __decorate([
            __param(2, editor_1.IEditorService),
            __param(3, message_1.IMessageService),
            __param(4, editorWorkerService_1.IEditorWorkerService)
        ], OpenLinkAction);
        return OpenLinkAction;
    }(editorAction_1.EditorAction));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(OpenLinkAction, OpenLinkAction.ID, nls.localize(3, null)));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/multicursor/common/multicursor", ["require", "exports", 'vs/nls!vs/editor/contrib/multicursor/common/multicursor', 'vs/base/common/keyCodes', 'vs/base/common/winjs.base', 'vs/editor/common/editorAction', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions'], function (require, exports, nls, keyCodes_1, winjs_base_1, editorAction_1, editorCommon_1, editorCommonExtensions_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var InsertCursorAbove = (function (_super) {
        __extends(InsertCursorAbove, _super);
        function InsertCursorAbove(descriptor, editor) {
            _super.call(this, descriptor, editor, editorCommon_1.Handler.AddCursorUp);
        }
        InsertCursorAbove.ID = 'editor.action.insertCursorAbove';
        return InsertCursorAbove;
    }(editorAction_1.HandlerEditorAction));
    var InsertCursorBelow = (function (_super) {
        __extends(InsertCursorBelow, _super);
        function InsertCursorBelow(descriptor, editor) {
            _super.call(this, descriptor, editor, editorCommon_1.Handler.AddCursorDown);
        }
        InsertCursorBelow.ID = 'editor.action.insertCursorBelow';
        return InsertCursorBelow;
    }(editorAction_1.HandlerEditorAction));
    var InsertCursorAtEndOfEachLineSelected = (function (_super) {
        __extends(InsertCursorAtEndOfEachLineSelected, _super);
        function InsertCursorAtEndOfEachLineSelected(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        InsertCursorAtEndOfEachLineSelected.prototype.run = function () {
            var selection = this.editor.getSelection();
            if (!selection.isEmpty()) {
                var model = this.editor.getModel();
                var newSelections = new Array();
                var selectionStart = selection.getStartPosition();
                var selectionEnd = selection.getEndPosition();
                for (var i = selectionStart.lineNumber; i <= selectionEnd.lineNumber; i++) {
                    if (i !== selectionEnd.lineNumber) {
                        var currentLineMaxColumn = model.getLineMaxColumn(i);
                        newSelections.push({
                            selectionStartLineNumber: i,
                            selectionStartColumn: currentLineMaxColumn,
                            positionLineNumber: i,
                            positionColumn: currentLineMaxColumn
                        });
                    }
                    else if (selectionEnd.column > 0) {
                        newSelections.push({
                            selectionStartLineNumber: selectionEnd.lineNumber,
                            selectionStartColumn: selectionEnd.column,
                            positionLineNumber: selectionEnd.lineNumber,
                            positionColumn: selectionEnd.column
                        });
                    }
                }
                this.editor.setSelections(newSelections);
            }
            return winjs_base_1.TPromise.as(true);
        };
        InsertCursorAtEndOfEachLineSelected.ID = 'editor.action.insertCursorAtEndOfEachLineSelected';
        return InsertCursorAtEndOfEachLineSelected;
    }(editorAction_1.EditorAction));
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(InsertCursorAbove, InsertCursorAbove.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.UpArrow,
        linux: {
            primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.UpArrow,
            secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.UpArrow]
        }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(InsertCursorBelow, InsertCursorBelow.ID, nls.localize(1, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.DownArrow,
        linux: {
            primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.DownArrow,
            secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.DownArrow]
        }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(InsertCursorAtEndOfEachLineSelected, InsertCursorAtEndOfEachLineSelected.ID, nls.localize(2, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_I
    }));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/parameterHints/common/parameterHints", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/winjs.base', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry'], function (require, exports, errors_1, winjs_base_1, editorCommonExtensions_1, languageFeatureRegistry_1) {
    'use strict';
    exports.ParameterHintsRegistry = new languageFeatureRegistry_1.default('parameterHintsSupport');
    function getParameterHints(model, position, triggerCharacter) {
        var support = exports.ParameterHintsRegistry.ordered(model)[0];
        if (!support) {
            return winjs_base_1.TPromise.as(undefined);
        }
        return support.getParameterHints(model.getAssociatedResource(), position, triggerCharacter);
    }
    exports.getParameterHints = getParameterHints;
    editorCommonExtensions_1.CommonEditorRegistry.registerDefaultLanguageCommand('_executeSignatureHelpProvider', function (model, position, args) {
        var triggerCharacter = args.triggerCharacter;
        if (triggerCharacter && typeof triggerCharacter !== 'string') {
            throw errors_1.illegalArgument('triggerCharacter');
        }
        return getParameterHints(model, position, triggerCharacter);
    });
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/parameterHints/browser/parameterHintsModel", ["require", "exports", 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/eventEmitter', 'vs/base/common/lifecycle', 'vs/editor/common/editorCommon', '../common/parameterHints'], function (require, exports, async_1, errors_1, eventEmitter_1, lifecycle_1, editorCommon_1, parameterHints_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ParameterHintsModel = (function (_super) {
        __extends(ParameterHintsModel, _super);
        function ParameterHintsModel(editor) {
            var _this = this;
            _super.call(this, ['cancel', 'hint', 'destroy']);
            this.editor = editor;
            this.toDispose = [];
            this.triggerCharactersListeners = [];
            this.throttledDelayer = new async_1.ThrottledDelayer(ParameterHintsModel.DELAY);
            this.active = false;
            this.prevResult = null;
            this.event(this.editor, editorCommon_1.EventType.ModelChanged, function (e) { return _this.onModelChanged(); });
            this.event(this.editor, editorCommon_1.EventType.ModelModeChanged, function (encodeURI) { return _this.onModelChanged(); });
            this.event(this.editor, editorCommon_1.EventType.ModelModeSupportChanged, function (e) { return _this.onModeChanged(e); });
            this.event(this.editor, editorCommon_1.EventType.CursorSelectionChanged, function (e) { return _this.onCursorChange(e); });
            this.toDispose.push(parameterHints_1.ParameterHintsRegistry.onDidChange(this.onModelChanged, this));
            this.onModelChanged();
        }
        ParameterHintsModel.prototype.cancel = function (silent, refresh) {
            if (silent === void 0) { silent = false; }
            if (refresh === void 0) { refresh = false; }
            this.active = false;
            if (!refresh) {
                this.prevResult = null;
            }
            this.throttledDelayer.cancel();
            if (!silent) {
                this.emit('cancel');
            }
        };
        ParameterHintsModel.prototype.trigger = function (triggerCharacter, delay) {
            var _this = this;
            if (delay === void 0) { delay = ParameterHintsModel.DELAY; }
            if (!parameterHints_1.ParameterHintsRegistry.has(this.editor.getModel())) {
                return;
            }
            this.cancel(true, true);
            return this.throttledDelayer.trigger(function () { return _this.doTrigger(triggerCharacter); }, delay);
        };
        ParameterHintsModel.prototype.doTrigger = function (triggerCharacter) {
            var _this = this;
            return parameterHints_1.getParameterHints(this.editor.getModel(), this.editor.getPosition(), triggerCharacter)
                .then(null, errors_1.onUnexpectedError)
                .then(function (result) {
                if (!result || result.signatures.length === 0) {
                    _this.cancel();
                    _this.emit('cancel');
                    return false;
                }
                _this.active = true;
                _this.prevResult = result;
                var event = { hints: result };
                _this.emit('hint', event);
                return true;
            });
        };
        ParameterHintsModel.prototype.isTriggered = function () {
            return this.active || this.throttledDelayer.isTriggered();
        };
        ParameterHintsModel.prototype.onModelChanged = function () {
            var _this = this;
            this.triggerCharactersListeners = lifecycle_1.disposeAll(this.triggerCharactersListeners);
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var support = parameterHints_1.ParameterHintsRegistry.ordered(model)[0];
            if (!support) {
                return;
            }
            this.triggerCharactersListeners = support.getParameterHintsTriggerCharacters().map(function (ch) {
                var listener = _this.editor.addTypingListener(ch, function () {
                    var position = _this.editor.getPosition();
                    var lineContext = model.getLineContext(position.lineNumber);
                    if (!support.shouldTriggerParameterHints(lineContext, position.column - 1)) {
                        return;
                    }
                    _this.trigger(ch);
                });
                return { dispose: listener };
            });
        };
        ParameterHintsModel.prototype.onModeChanged = function (e) {
            if (e.parameterHintsSupport) {
                this.onModelChanged();
            }
        };
        ParameterHintsModel.prototype.onCursorChange = function (e) {
            if (e.source === 'mouse') {
                this.cancel();
            }
            else if (this.isTriggered()) {
                this.trigger();
            }
        };
        ParameterHintsModel.prototype.event = function (emitter, eventType, cb) {
            this.toDispose.push(emitter.addListener2(eventType, cb));
        };
        ParameterHintsModel.prototype.dispose = function () {
            this.cancel(true);
            this.triggerCharactersListeners = lifecycle_1.disposeAll(this.triggerCharactersListeners);
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            this.emit('destroy', null);
            _super.prototype.dispose.call(this);
        };
        ParameterHintsModel.DELAY = 120; // ms
        return ParameterHintsModel;
    }(eventEmitter_1.EventEmitter));
    exports.ParameterHintsModel = ParameterHintsModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/parameterHints/browser/parameterHints", ["require", "exports", 'vs/nls!vs/editor/contrib/parameterHints/browser/parameterHints', 'vs/base/common/keyCodes', 'vs/base/common/winjs.base', 'vs/platform/keybinding/common/keybindingService', 'vs/editor/common/editorAction', 'vs/editor/common/editorCommonExtensions', 'vs/editor/browser/editorBrowserExtensions', '../common/parameterHints', './parameterHintsModel', './parameterHintsWidget'], function (require, exports, nls, keyCodes_1, winjs_base_1, keybindingService_1, editorAction_1, editorCommonExtensions_1, editorBrowserExtensions_1, parameterHints_1, parameterHintsModel_1, parameterHintsWidget_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ParameterHintsController = (function () {
        function ParameterHintsController(editor, keybindingService) {
            var _this = this;
            this.editor = editor;
            this.model = new parameterHintsModel_1.ParameterHintsModel(this.editor);
            this.parameterHintsVisible = keybindingService.createKey(CONTEXT_PARAMETER_HINTS_VISIBLE, false);
            this.widget = new parameterHintsWidget_1.ParameterHintsWidget(this.model, this.editor, function () {
                _this.parameterHintsVisible.set(true);
            }, function () {
                _this.parameterHintsVisible.reset();
            });
        }
        ParameterHintsController.get = function (editor) {
            return editor.getContribution(ParameterHintsController.ID);
        };
        ParameterHintsController.prototype.dispose = function () {
            this.model.dispose();
            this.model = null;
            this.widget.destroy();
            this.widget = null;
        };
        ParameterHintsController.prototype.getId = function () {
            return ParameterHintsController.ID;
        };
        ParameterHintsController.prototype.closeWidget = function () {
            this.widget.cancel();
        };
        ParameterHintsController.prototype.showPrevHint = function () {
            this.widget.selectPrevious();
        };
        ParameterHintsController.prototype.showNextHint = function () {
            this.widget.selectNext();
        };
        ParameterHintsController.prototype.trigger = function () {
            this.model.trigger(undefined, 0);
        };
        ParameterHintsController.ID = 'editor.controller.parameterHints';
        ParameterHintsController = __decorate([
            __param(1, keybindingService_1.IKeybindingService)
        ], ParameterHintsController);
        return ParameterHintsController;
    }());
    var TriggerParameterHintsAction = (function (_super) {
        __extends(TriggerParameterHintsAction, _super);
        function TriggerParameterHintsAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        TriggerParameterHintsAction.prototype.isSupported = function () {
            return parameterHints_1.ParameterHintsRegistry.has(this.editor.getModel()) && _super.prototype.isSupported.call(this);
        };
        TriggerParameterHintsAction.prototype.run = function () {
            ParameterHintsController.get(this.editor).trigger();
            return winjs_base_1.TPromise.as(true);
        };
        TriggerParameterHintsAction.ID = 'editor.action.triggerParameterHints';
        return TriggerParameterHintsAction;
    }(editorAction_1.EditorAction));
    exports.TriggerParameterHintsAction = TriggerParameterHintsAction;
    var CONTEXT_PARAMETER_HINTS_VISIBLE = 'parameterHintsVisible';
    var weight = editorCommonExtensions_1.CommonEditorRegistry.commandWeight(75);
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(ParameterHintsController);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(TriggerParameterHintsAction, TriggerParameterHintsAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Space
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('closeParameterHints', weight, { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, true, CONTEXT_PARAMETER_HINTS_VISIBLE, function (ctx, editor, args) {
        ParameterHintsController.get(editor).closeWidget();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('showPrevParameterHint', weight, { primary: keyCodes_1.KeyCode.UpArrow, secondary: [keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.UpArrow] }, true, CONTEXT_PARAMETER_HINTS_VISIBLE, function (ctx, editor, args) {
        ParameterHintsController.get(editor).showPrevHint();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('showNextParameterHint', weight, { primary: keyCodes_1.KeyCode.DownArrow, secondary: [keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.DownArrow] }, true, CONTEXT_PARAMETER_HINTS_VISIBLE, function (ctx, editor, args) {
        ParameterHintsController.get(editor).showNextHint();
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/quickFix/common/quickFix", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/editor/common/core/range', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry', 'vs/editor/common/services/modelService'], function (require, exports, errors_1, uri_1, winjs_base_1, range_1, editorCommonExtensions_1, languageFeatureRegistry_1, modelService_1) {
    'use strict';
    exports.QuickFixRegistry = new languageFeatureRegistry_1.default('quickFixSupport');
    function getQuickFixes(model, range) {
        var quickFixes = [];
        var idPool = 0;
        var promises = exports.QuickFixRegistry.all(model).map(function (support) {
            return support.getQuickFixes(model.getAssociatedResource(), range).then(function (result) {
                if (!Array.isArray(result)) {
                    return;
                }
                for (var _i = 0, result_1 = result; _i < result_1.length; _i++) {
                    var fix = result_1[_i];
                    quickFixes.push({
                        command: fix.command,
                        score: fix.score,
                        id: "quickfix_#" + idPool++,
                        support: support
                    });
                }
            }, function (err) {
                errors_1.onUnexpectedError(err);
            });
        });
        return winjs_base_1.TPromise.join(promises).then(function () { return quickFixes; });
    }
    exports.getQuickFixes = getQuickFixes;
    editorCommonExtensions_1.CommonEditorRegistry.registerLanguageCommand('_executeCodeActionProvider', function (accessor, args) {
        var resource = args.resource, range = args.range;
        if (!(resource instanceof uri_1.default) || !range_1.Range.isIRange(range)) {
            throw errors_1.illegalArgument();
        }
        var model = accessor.get(modelService_1.IModelService).getModel(resource);
        if (!model) {
            throw errors_1.illegalArgument();
        }
        return getQuickFixes(model, range);
    });
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/quickFix/browser/quickFixModel", ["require", "exports", 'vs/base/common/arrays', 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/eventEmitter', 'vs/base/common/lifecycle', 'vs/base/common/timer', 'vs/base/common/winjs.base', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', '../common/quickFix', './lightBulpWidget'], function (require, exports, arrays, async_1, errors_1, eventEmitter_1, lifecycle_1, timer, winjs_base_1, range_1, editorCommon_1, quickFix_1, lightBulpWidget_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var QuickFixSuggestState;
    (function (QuickFixSuggestState) {
        QuickFixSuggestState[QuickFixSuggestState["NOT_ACTIVE"] = 0] = "NOT_ACTIVE";
        QuickFixSuggestState[QuickFixSuggestState["MANUAL_TRIGGER"] = 1] = "MANUAL_TRIGGER";
        QuickFixSuggestState[QuickFixSuggestState["AUTO_TRIGGER"] = 2] = "AUTO_TRIGGER";
    })(QuickFixSuggestState || (QuickFixSuggestState = {}));
    var QuickFixModel = (function (_super) {
        __extends(QuickFixModel, _super);
        function QuickFixModel(editor, markerService, onAccept) {
            var _this = this;
            _super.call(this);
            this.editor = editor;
            this.markerService = markerService;
            this.onAccept = onAccept;
            this.quickFixRequestPromise = null;
            this.lightBulpDecoration = [];
            this.toDispose = [];
            this.toLocalDispose = [];
            this.lightBulp = new lightBulpWidget_1.LightBulpWidget(editor, function (pos) { _this.onLightBulpClicked(pos); });
            this.enableAutoQuckFix = false; // turn off for now
            this.autoSuggestDelay = this.editor.getConfiguration().quickSuggestionsDelay;
            if (isNaN(this.autoSuggestDelay) || (!this.autoSuggestDelay && this.autoSuggestDelay !== 0) || this.autoSuggestDelay > 2000 || this.autoSuggestDelay < 0) {
                this.autoSuggestDelay = 300;
            }
            this.toDispose.push(this.editor.addListener2(editorCommon_1.EventType.ModelChanged, function () { return _this.onModelChanged(); }));
            this.toDispose.push(this.editor.addListener2(editorCommon_1.EventType.ModelModeChanged, function () { return _this.onModelChanged(); }));
            this.toDispose.push(this.editor.addListener2(editorCommon_1.EventType.ModelModeSupportChanged, function (e) {
                if (e.quickFixSupport) {
                    _this.onModelChanged();
                }
            }));
            this.toDispose.push(quickFix_1.QuickFixRegistry.onDidChange(this.onModelChanged, this));
        }
        QuickFixModel.prototype.onModelChanged = function () {
            var _this = this;
            this.cancelDialog();
            this.localDispose();
            this.lastMarker = null;
            this.lightBulpPosition = null;
            this.markers = null;
            this.updateScheduler = null;
            if (!quickFix_1.QuickFixRegistry.has(this.editor.getModel()) || this.editor.getConfiguration().readOnly) {
                this.setDecoration(null);
                return;
            }
            this.markerService.onMarkerChanged(this.onMarkerChanged, this, this.toLocalDispose);
            this.toLocalDispose.push(this.editor.addListener2(editorCommon_1.EventType.CursorPositionChanged, function (e) {
                _this.onCursorPositionChanged();
            }));
        };
        QuickFixModel.prototype.onLightBulpClicked = function (pos) {
            this.triggerDialog(true, pos);
        };
        QuickFixModel.prototype.isSimilarMarker = function (marker1, marker2) {
            if (marker1) {
                return marker2 && marker1.owner === marker2.owner && marker1.code === marker2.code;
            }
            return !marker2;
        };
        QuickFixModel.prototype.onMarkerChanged = function (changedResources) {
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var associatedResource = model.getAssociatedResource();
            if (!changedResources.some(function (r) { return associatedResource.toString() === r.toString(); })) {
                return;
            }
            var lastMarker = this.lastMarker;
            this.markers = null;
            this.lastMarker = null;
            var currentMarker = this.findMarker(this.editor.getPosition(), false);
            if (this.isSimilarMarker(currentMarker, lastMarker)) {
                this.lastMarker = currentMarker;
            }
            else {
                this.onCursorPositionChanged();
            }
        };
        QuickFixModel.prototype.setDecoration = function (pos) {
            this.lightBulpPosition = pos;
            this.updateDecoration();
        };
        QuickFixModel.prototype.updateDecoration = function () {
            if (this.lightBulpPosition && this.state === QuickFixSuggestState.NOT_ACTIVE) {
                this.lightBulp.show(this.lightBulpPosition);
            }
            else {
                this.lightBulp.hide();
            }
        };
        QuickFixModel.prototype.onCursorPositionChanged = function () {
            var _this = this;
            if (this.triggerAutoSuggestPromise) {
                this.triggerAutoSuggestPromise.cancel();
                this.triggerAutoSuggestPromise = null;
            }
            this.cancelDialog();
            if (!this.updateScheduler) {
                this.updateScheduler = new async_1.RunOnceScheduler(function () {
                    var marker = _this.lastMarker;
                    var pos = _this.editor.getPosition();
                    if (marker && range_1.Range.containsPosition(marker, pos)) {
                        // still on the same marker
                        if (_this.lightBulpPosition) {
                            _this.setDecoration(pos);
                        }
                        return;
                    }
                    _this.lastMarker = marker = _this.findMarker(pos, false);
                    if (!marker) {
                        // remove lightbulp
                        _this.setDecoration(null);
                        return;
                    }
                    var $tRequest = timer.start(timer.Topic.EDITOR, 'quickfix/lighbulp');
                    var computeFixesPromise = _this.computeFixes(marker);
                    computeFixesPromise.done(function (fixes) {
                        _this.setDecoration(!arrays.isFalsyOrEmpty(fixes) ? pos : null);
                        _this.triggerAutoSuggest(marker);
                        $tRequest.stop();
                    }, function (error) {
                        errors_1.onUnexpectedError(error);
                        _this.setDecoration(null);
                        $tRequest.stop();
                    });
                }, 250);
                this.toLocalDispose.push(this.updateScheduler);
            }
            this.updateScheduler.schedule();
        };
        QuickFixModel.prototype.computeFixes = function (range) {
            var model = this.editor.getModel();
            if (!quickFix_1.QuickFixRegistry.has(model)) {
                return winjs_base_1.TPromise.as(null);
            }
            if (this.quickFixRequestPromise && range === this.quickFixRequestPromiseRange) {
                return this.quickFixRequestPromise;
            }
            if (this.quickFixRequestPromise) {
                this.quickFixRequestPromise.cancel();
                this.quickFixRequestPromise = null;
            }
            this.quickFixRequestPromiseRange = range;
            this.quickFixRequestPromise = quickFix_1.getQuickFixes(model, range);
            return this.quickFixRequestPromise;
        };
        /**
         * Returns all marker sorted by startLineNumber
         */
        QuickFixModel.prototype.getMarkers = function () {
            if (this.markers !== null) {
                return this.markers;
            }
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            this.markers = this.markerService.read({ resource: model.getAssociatedResource() })
                .sort(function (e1, e2) { return e1.startLineNumber - e2.startLineNumber; });
            return this.markers;
        };
        QuickFixModel.prototype.findMarker = function (pos, findOnSameLine) {
            if (this.lastMarker && range_1.Range.containsPosition(this.lastMarker, pos)) {
                return this.lastMarker;
            }
            var markers = this.getMarkers(); // makers sorted by line start number
            var result = null;
            var bestDistance = 0;
            var lineNumber = pos.lineNumber;
            // find first marker with a line number greater equal the current position
            var idx = arrays.findFirst(markers, function (m) { return m.startLineNumber >= lineNumber; });
            while (idx < markers.length && markers[idx].startLineNumber === lineNumber) {
                var marker = markers[idx];
                if (marker.startColumn <= pos.column && marker.endColumn >= pos.column) {
                    return marker;
                }
                if (findOnSameLine) {
                    var dist = pos.column < marker.startColumn ? marker.startColumn - pos.column : pos.column - marker.endColumn;
                    if (!result || dist < bestDistance) {
                        result = marker;
                        bestDistance = dist;
                    }
                }
                idx++;
            }
            return result;
        };
        QuickFixModel.prototype.cancelDialog = function (silent) {
            if (silent === void 0) { silent = false; }
            if (this.state !== QuickFixSuggestState.NOT_ACTIVE) {
                this.state = QuickFixSuggestState.NOT_ACTIVE;
                if (!silent) {
                    this.emit('cancel');
                }
                this.updateDecoration();
                return true;
            }
            return false;
        };
        QuickFixModel.prototype.isAutoSuggest = function () {
            return this.state === QuickFixSuggestState.AUTO_TRIGGER;
        };
        QuickFixModel.prototype.triggerAutoSuggest = function (marker) {
            var _this = this;
            if (this.enableAutoQuckFix && this.state === QuickFixSuggestState.NOT_ACTIVE) {
                this.triggerAutoSuggestPromise = winjs_base_1.TPromise.timeout(this.autoSuggestDelay);
                this.triggerAutoSuggestPromise.then(function () {
                    _this.triggerAutoSuggestPromise = null;
                    if (marker === _this.lastMarker) {
                        _this.triggerDialog(true, _this.editor.getPosition());
                    }
                });
            }
        };
        QuickFixModel.prototype.triggerDialog = function (auto, pos) {
            var _this = this;
            // Cancel previous requests, change state & update UI
            this.cancelDialog(false);
            var range;
            if (auto) {
                range = this.findMarker(pos, true);
                if (!range) {
                    return;
                }
            }
            else {
                range = this.findMarker(pos, true);
                if (!range) {
                    // no error on the same line: get code action for the current selection
                    range = this.editor.getSelection();
                }
                if (!range_1.Range.containsPosition(range, pos)) {
                    // move cursor
                    this.editor.setPosition({ lineNumber: range.startLineNumber, column: range.startColumn });
                }
            }
            var $tTrigger = timer.start(timer.Topic.EDITOR, 'quickfix/triggerdialog');
            this.state = auto ? QuickFixSuggestState.AUTO_TRIGGER : QuickFixSuggestState.MANUAL_TRIGGER;
            this.updateDecoration();
            this.emit('loading', { auto: this.isAutoSuggest() });
            this.computeFixes(range).done(function (fixes) {
                if (fixes && fixes.length > 0) {
                    fixes.sort(function (f1, f2) { return f2.score - f1.score; });
                    _this.emit('suggest', {
                        fixes: fixes,
                        range: range,
                        auto: _this.isAutoSuggest()
                    });
                }
                else {
                    _this.emit('empty', { auto: _this.isAutoSuggest() });
                }
                $tTrigger.stop();
            }, function (error) {
                errors_1.onUnexpectedError(error);
                _this.emit('empty', { auto: _this.isAutoSuggest() });
                $tTrigger.stop();
            });
        };
        QuickFixModel.prototype.accept = function (quickFix, range) {
            this.cancelDialog();
            if (!quickFix) {
                return false;
            }
            this.onAccept(quickFix, range);
            return true;
        };
        QuickFixModel.prototype.localDispose = function () {
            this.toLocalDispose = lifecycle_1.disposeAll(this.toLocalDispose);
            if (this.quickFixRequestPromise) {
                this.quickFixRequestPromise.cancel();
                this.quickFixRequestPromise = null;
            }
        };
        QuickFixModel.prototype.dispose = function () {
            this.localDispose();
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            this.emit('destroy', null);
        };
        return QuickFixModel;
    }(eventEmitter_1.EventEmitter));
    exports.QuickFixModel = QuickFixModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/quickFix/browser/quickFix", ["require", "exports", 'vs/nls!vs/editor/contrib/quickFix/browser/quickFix', 'vs/base/common/errors', 'vs/base/common/keyCodes', 'vs/base/common/severity', 'vs/base/common/winjs.base', 'vs/platform/editor/common/editor', 'vs/platform/event/common/event', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/markers/common/markers', 'vs/platform/message/common/message', 'vs/platform/telemetry/common/telemetry', 'vs/editor/common/editorAction', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/services/bulkEdit', 'vs/editor/browser/editorBrowserExtensions', '../common/quickFix', './quickFixModel', './quickFixSelectionWidget'], function (require, exports, nls, errors_1, keyCodes_1, severity_1, winjs_base_1, editor_1, event_1, keybindingService_1, markers_1, message_1, telemetry_1, editorAction_1, editorCommonExtensions_1, bulkEdit_1, editorBrowserExtensions_1, quickFix_1, quickFixModel_1, quickFixSelectionWidget_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var QuickFixController = (function () {
        function QuickFixController(editor, markerService, keybindingService, telemetryService, eventService, editorService, messageService) {
            var _this = this;
            this.editor = editor;
            this.model = new quickFixModel_1.QuickFixModel(this.editor, markerService, this.onAccept.bind(this));
            this.eventService = eventService;
            this.editorService = editorService;
            this.messageService = messageService;
            this.quickFixWidgetVisible = keybindingService.createKey(CONTEXT_QUICK_FIX_WIDGET_VISIBLE, false);
            this.suggestWidget = new quickFixSelectionWidget_1.QuickFixSelectionWidget(this.editor, telemetryService, function () {
                _this.quickFixWidgetVisible.set(true);
            }, function () {
                _this.quickFixWidgetVisible.reset();
            });
            this.suggestWidget.setModel(this.model);
        }
        QuickFixController.getQuickFixController = function (editor) {
            return editor.getContribution(QuickFixController.ID);
        };
        QuickFixController.prototype.getId = function () {
            return QuickFixController.ID;
        };
        QuickFixController.prototype.onAccept = function (fix, range) {
            var _this = this;
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            fix.support.runQuickFixAction(this.editor.getModel().getAssociatedResource(), range, { command: fix.command, score: fix.score }).then(function (result) {
                if (result) {
                    if (result.message) {
                        _this.messageService.show(severity_1.default.Info, result.message);
                    }
                    if (result.edits) {
                        return bulkEdit_1.bulkEdit(_this.eventService, _this.editorService, _this.editor, result.edits);
                    }
                }
                return winjs_base_1.TPromise.as(0);
            }).done(undefined, function (err) {
                errors_1.onUnexpectedError(err);
            });
        };
        QuickFixController.prototype.run = function () {
            this.model.triggerDialog(false, this.editor.getPosition());
            this.editor.focus();
            return winjs_base_1.TPromise.as(false);
        };
        QuickFixController.prototype.dispose = function () {
            if (this.suggestWidget) {
                this.suggestWidget.destroy();
                this.suggestWidget = null;
            }
            if (this.model) {
                this.model.dispose();
                this.model = null;
            }
        };
        QuickFixController.prototype.acceptSelectedSuggestion = function () {
            if (this.suggestWidget) {
                this.suggestWidget.acceptSelectedSuggestion();
            }
        };
        QuickFixController.prototype.closeWidget = function () {
            if (this.model) {
                this.model.cancelDialog();
            }
        };
        QuickFixController.prototype.selectNextSuggestion = function () {
            if (this.suggestWidget) {
                this.suggestWidget.selectNext();
            }
        };
        QuickFixController.prototype.selectNextPageSuggestion = function () {
            if (this.suggestWidget) {
                this.suggestWidget.selectNextPage();
            }
        };
        QuickFixController.prototype.selectPrevSuggestion = function () {
            if (this.suggestWidget) {
                this.suggestWidget.selectPrevious();
            }
        };
        QuickFixController.prototype.selectPrevPageSuggestion = function () {
            if (this.suggestWidget) {
                this.suggestWidget.selectPreviousPage();
            }
        };
        QuickFixController.ID = 'editor.contrib.quickFixController';
        QuickFixController = __decorate([
            __param(1, markers_1.IMarkerService),
            __param(2, keybindingService_1.IKeybindingService),
            __param(3, telemetry_1.ITelemetryService),
            __param(4, event_1.IEventService),
            __param(5, editor_1.IEditorService),
            __param(6, message_1.IMessageService)
        ], QuickFixController);
        return QuickFixController;
    }());
    exports.QuickFixController = QuickFixController;
    var QuickFixAction = (function (_super) {
        __extends(QuickFixAction, _super);
        function QuickFixAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        QuickFixAction.prototype.isSupported = function () {
            var model = this.editor.getModel();
            return quickFix_1.QuickFixRegistry.has(model) && !this.editor.getConfiguration().readOnly;
        };
        QuickFixAction.prototype.run = function () {
            return QuickFixController.getQuickFixController(this.editor).run();
        };
        QuickFixAction.ID = 'editor.action.quickFix';
        return QuickFixAction;
    }(editorAction_1.EditorAction));
    exports.QuickFixAction = QuickFixAction;
    var CONTEXT_QUICK_FIX_WIDGET_VISIBLE = 'quickFixWidgetVisible';
    var weight = editorCommonExtensions_1.CommonEditorRegistry.commandWeight(80);
    // register action
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(QuickFixAction, QuickFixAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.US_DOT
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('acceptQuickFixSuggestion', weight, { primary: keyCodes_1.KeyCode.Enter, secondary: [keyCodes_1.KeyCode.Tab] }, false, CONTEXT_QUICK_FIX_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = QuickFixController.getQuickFixController(editor);
        controller.acceptSelectedSuggestion();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('closeQuickFixWidget', weight, { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, false, CONTEXT_QUICK_FIX_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = QuickFixController.getQuickFixController(editor);
        controller.closeWidget();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('selectNextQuickFix', weight, { primary: keyCodes_1.KeyCode.DownArrow }, false, CONTEXT_QUICK_FIX_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = QuickFixController.getQuickFixController(editor);
        controller.selectNextSuggestion();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('selectNextPageQuickFix', weight, { primary: keyCodes_1.KeyCode.PageDown }, false, CONTEXT_QUICK_FIX_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = QuickFixController.getQuickFixController(editor);
        controller.selectNextPageSuggestion();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('selectPrevQuickFix', weight, { primary: keyCodes_1.KeyCode.UpArrow }, false, CONTEXT_QUICK_FIX_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = QuickFixController.getQuickFixController(editor);
        controller.selectPrevSuggestion();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('selectPrevPageQuickFix', weight, { primary: keyCodes_1.KeyCode.PageUp }, false, CONTEXT_QUICK_FIX_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = QuickFixController.getQuickFixController(editor);
        controller.selectPrevPageSuggestion();
    });
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(QuickFixController);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/quickOpen/common/quickOpen", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/editor/common/core/range', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry', 'vs/editor/common/services/modelService'], function (require, exports, errors_1, uri_1, winjs_base_1, range_1, editorCommonExtensions_1, languageFeatureRegistry_1, modelService_1) {
    'use strict';
    var OutlineRegistry = new languageFeatureRegistry_1.default('outlineSupport');
    exports.OutlineRegistry = OutlineRegistry;
    function getOutlineEntries(model) {
        var groupLabels = Object.create(null);
        var entries = [];
        var promises = OutlineRegistry.all(model).map(function (support) {
            if (support.outlineGroupLabel) {
                for (var key in support.outlineGroupLabel) {
                    if (Object.prototype.hasOwnProperty.call(support.outlineGroupLabel, key)) {
                        groupLabels[key] = support.outlineGroupLabel[key];
                    }
                }
            }
            return support.getOutline(model.getAssociatedResource()).then(function (result) {
                if (Array.isArray(result)) {
                    entries.push.apply(entries, result);
                }
            }, function (err) {
                errors_1.onUnexpectedError(err);
            });
        });
        return winjs_base_1.TPromise.join(promises).then(function () {
            var flatEntries = [];
            flatten(flatEntries, entries, '');
            flatEntries.sort(compareEntriesUsingStart);
            return {
                entries: flatEntries,
                outlineGroupLabel: groupLabels
            };
        });
    }
    exports.getOutlineEntries = getOutlineEntries;
    function compareEntriesUsingStart(a, b) {
        return range_1.Range.compareRangesUsingStarts(a.range, b.range);
    }
    function flatten(bucket, entries, overrideContainerLabel) {
        for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
            var entry = entries_1[_i];
            bucket.push({
                type: entry.type,
                range: entry.range,
                label: entry.label,
                icon: entry.icon,
                containerLabel: entry.containerLabel || overrideContainerLabel
            });
            if (entry.children) {
                flatten(bucket, entry.children, entry.label);
            }
        }
    }
    editorCommonExtensions_1.CommonEditorRegistry.registerLanguageCommand('_executeDocumentSymbolProvider', function (accessor, args) {
        var resource = args.resource;
        if (!(resource instanceof uri_1.default)) {
            throw errors_1.illegalArgument('resource');
        }
        var model = accessor.get(modelService_1.IModelService).getModel(resource);
        if (!model) {
            throw errors_1.illegalArgument('resource');
        }
        return getOutlineEntries(model);
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/referenceSearch/common/referenceSearch", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/winjs.base', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry'], function (require, exports, errors_1, winjs_base_1, editorCommonExtensions_1, languageFeatureRegistry_1) {
    'use strict';
    exports.ReferenceRegistry = new languageFeatureRegistry_1.default('referenceSupport');
    function findReferences(model, position) {
        // collect references from all providers
        var promises = exports.ReferenceRegistry.ordered(model).map(function (provider) {
            return provider.findReferences(model.getAssociatedResource(), position, true).then(function (result) {
                if (Array.isArray(result)) {
                    return result;
                }
            }, function (err) {
                errors_1.onUnexpectedError(err);
            });
        });
        return winjs_base_1.TPromise.join(promises).then(function (references) {
            var result = [];
            for (var _i = 0, references_1 = references; _i < references_1.length; _i++) {
                var ref = references_1[_i];
                if (ref) {
                    result.push.apply(result, ref);
                }
            }
            return result;
        });
    }
    exports.findReferences = findReferences;
    editorCommonExtensions_1.CommonEditorRegistry.registerDefaultLanguageCommand('_executeReferenceProvider', findReferences);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/contrib/rename/common/rename", ["require", "exports", 'vs/nls!vs/editor/contrib/rename/common/rename', 'vs/base/common/async', 'vs/base/common/errors', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry'], function (require, exports, nls_1, async_1, errors_1, editorCommonExtensions_1, languageFeatureRegistry_1) {
    'use strict';
    exports.RenameRegistry = new languageFeatureRegistry_1.default('renameSupport');
    function rename(model, position, newName) {
        var supports = exports.RenameRegistry.ordered(model);
        var resource = model.getAssociatedResource();
        var rejects = [];
        var hasResult = false;
        var factory = supports.map(function (support) {
            return function () {
                if (!hasResult) {
                    return support.rename(resource, position, newName).then(function (result) {
                        if (!result) {
                        }
                        else if (!result.rejectReason) {
                            hasResult = true;
                            return result;
                        }
                        else {
                            rejects.push(result.rejectReason);
                        }
                    });
                }
            };
        });
        return async_1.sequence(factory).then(function (values) {
            var result = values[0];
            if (rejects.length > 0) {
                return {
                    currentName: undefined,
                    edits: undefined,
                    rejectReason: rejects.join('\n')
                };
            }
            else if (!result) {
                return {
                    currentName: undefined,
                    edits: undefined,
                    rejectReason: nls_1.localize(0, null)
                };
            }
            else {
                return result;
            }
        });
    }
    exports.rename = rename;
    editorCommonExtensions_1.CommonEditorRegistry.registerDefaultLanguageCommand('_executeDocumentRenameProvider', function (model, position, args) {
        var newName = args.newName;
        if (typeof newName !== 'string') {
            throw errors_1.illegalArgument('newName');
        }
        return rename(model, position, newName);
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/rename/browser/rename2", ["require", "exports", 'vs/nls!vs/editor/contrib/rename/browser/rename2', 'vs/base/common/errors', 'vs/base/common/keyCodes', 'vs/base/common/severity', 'vs/base/common/winjs.base', 'vs/platform/editor/common/editor', 'vs/platform/event/common/event', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/message/common/message', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/supports', 'vs/editor/common/services/bulkEdit', '../common/rename', './renameInputField', 'vs/css!./rename'], function (require, exports, nls, errors_1, keyCodes_1, severity_1, winjs_base_1, editor_1, event_1, keybindingService_1, message_1, editorAction_1, editorActionEnablement_1, editorCommonExtensions_1, supports_1, bulkEdit_1, rename_1, renameInputField_1) {
    'use strict';
    // ---  register actions and commands
    var CONTEXT_RENAME_INPUT_VISIBLE = 'renameInputVisible';
    // ---- action implementation
    var RenameAction = (function (_super) {
        __extends(RenameAction, _super);
        function RenameAction(descriptor, editor, messageService, keybindingService, eventService, editorService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.Writeable | editorActionEnablement_1.Behaviour.ShowInContextMenu | editorActionEnablement_1.Behaviour.UpdateOnCursorPositionChange);
            this._messageService = messageService;
            this._eventService = eventService;
            this._editorService = editorService;
            this._renameInputField = new renameInputField_1.default(editor);
            this._renameInputVisible = keybindingService.createKey(CONTEXT_RENAME_INPUT_VISIBLE, false);
        }
        RenameAction.prototype.getGroupId = function () {
            return '2_change/1_rename2';
        };
        RenameAction.prototype.isSupported = function () {
            return rename_1.RenameRegistry.has(this.editor.getModel()) && !this.editor.getModel().hasEditableRange() && _super.prototype.isSupported.call(this);
        };
        RenameAction.prototype.getEnablementState = function () {
            var model = this.editor.getModel();
            var position = this.editor.getSelection().getStartPosition();
            var lineContext = model.getLineContext(position.lineNumber);
            return rename_1.RenameRegistry.ordered(model).some(function (support) {
                if (!support.filter) {
                    return true;
                }
                if (supports_1.isLineToken(lineContext, position.column - 1, support.filter)) {
                    return true;
                }
                if (position.column > 1 && supports_1.isLineToken(lineContext, position.column - 2, support.filter)) {
                    // in case we are in between two tokens
                    return true;
                }
            });
        };
        RenameAction.prototype.run = function (event) {
            var _this = this;
            var selection = this.editor.getSelection(), word = this.editor.getModel().getWordAtPosition(selection.getStartPosition());
            if (!word) {
                return;
            }
            var lineNumber = selection.startLineNumber, selectionStart = 0, selectionEnd = word.word.length, wordRange;
            wordRange = {
                startLineNumber: lineNumber,
                startColumn: word.startColumn,
                endLineNumber: lineNumber,
                endColumn: word.endColumn
            };
            if (!selection.isEmpty() && selection.startLineNumber === selection.endLineNumber) {
                selectionStart = Math.max(0, selection.startColumn - word.startColumn);
                selectionEnd = Math.min(word.endColumn, selection.endColumn) - word.startColumn;
            }
            this._renameInputVisible.set(true);
            return this._renameInputField.getInput(wordRange, word.word, selectionStart, selectionEnd).then(function (newName) {
                _this._renameInputVisible.reset();
                return _this._prepareRename(newName).then(function (edit) {
                    return edit.finish().then(function (selection) {
                        _this.editor.focus();
                        if (selection) {
                            _this.editor.setSelection(selection);
                        }
                    });
                }, function (err) {
                    if (typeof err === 'string') {
                        _this._messageService.show(severity_1.default.Info, err);
                    }
                    else {
                        return winjs_base_1.TPromise.wrapError(err);
                    }
                });
            }, function (err) {
                _this._renameInputVisible.reset();
                _this.editor.focus();
                if (!errors_1.isPromiseCanceledError(err)) {
                    return winjs_base_1.TPromise.wrapError(err);
                }
            });
        };
        RenameAction.prototype.acceptRenameInput = function () {
            this._renameInputField.acceptInput();
        };
        RenameAction.prototype.cancelRenameInput = function () {
            this._renameInputField.cancelInput();
        };
        RenameAction.prototype._prepareRename = function (newName) {
            // start recording of file changes so that we can figure out if a file that
            // is to be renamed conflicts with another (concurrent) modification
            var edit = bulkEdit_1.createBulkEdit(this._eventService, this._editorService, this.editor);
            return rename_1.rename(this.editor.getModel(), this.editor.getPosition(), newName).then(function (result) {
                if (result.rejectReason) {
                    return winjs_base_1.TPromise.wrapError(result.rejectReason);
                }
                edit.add(result.edits);
                return edit;
            });
        };
        RenameAction.ID = 'editor.action.rename';
        RenameAction = __decorate([
            __param(2, message_1.IMessageService),
            __param(3, keybindingService_1.IKeybindingService),
            __param(4, event_1.IEventService),
            __param(5, editor_1.IEditorService)
        ], RenameAction);
        return RenameAction;
    }(editorAction_1.EditorAction));
    exports.RenameAction = RenameAction;
    var weight = editorCommonExtensions_1.CommonEditorRegistry.commandWeight(99);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(RenameAction, RenameAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyCode.F2
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('acceptRenameInput', weight, { primary: keyCodes_1.KeyCode.Enter }, false, CONTEXT_RENAME_INPUT_VISIBLE, function (ctx, editor, args) {
        var action = editor.getAction(RenameAction.ID);
        action.acceptRenameInput();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('cancelRenameInput', weight, { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, false, CONTEXT_RENAME_INPUT_VISIBLE, function (ctx, editor, args) {
        var action = editor.getAction(RenameAction.ID);
        action.cancelRenameInput();
    });
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/smartSelect/common/jumpToBracket", ["require", "exports", 'vs/nls!vs/editor/contrib/smartSelect/common/jumpToBracket', 'vs/base/common/keyCodes', 'vs/base/common/winjs.base', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions'], function (require, exports, nls, keyCodes_1, winjs_base_1, editorAction_1, editorActionEnablement_1, editorCommon_1, editorCommonExtensions_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var SelectBracketAction = (function (_super) {
        __extends(SelectBracketAction, _super);
        function SelectBracketAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
        }
        SelectBracketAction.prototype.run = function () {
            this.editor.trigger(this.id, editorCommon_1.Handler.JumpToBracket, {});
            return winjs_base_1.TPromise.as(true);
        };
        SelectBracketAction.ID = 'editor.action.jumpToBracket';
        return SelectBracketAction;
    }(editorAction_1.EditorAction));
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(SelectBracketAction, SelectBracketAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.US_BACKSLASH
    }));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/smartSelect/common/smartSelect", ["require", "exports", 'vs/nls!vs/editor/contrib/smartSelect/common/smartSelect', 'vs/base/common/arrays', 'vs/base/common/keyCodes', 'vs/base/common/winjs.base', 'vs/platform/instantiation/common/instantiation', 'vs/editor/common/core/range', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', './tokenSelectionSupport'], function (require, exports, nls, arrays, keyCodes_1, winjs_base_1, instantiation_1, range_1, editorAction_1, editorActionEnablement_1, editorCommon_1, editorCommonExtensions_1, tokenSelectionSupport_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // --- selection state machine
    var State = (function () {
        function State(editor) {
            this.editor = editor;
            this.next = null;
            this.previous = null;
            this.selection = editor.getSelection();
        }
        return State;
    }());
    // --- shared state between grow and shrink actions
    var state = null;
    var ignoreSelection = false;
    // -- action implementation
    var SmartSelect = (function (_super) {
        __extends(SmartSelect, _super);
        function SmartSelect(descriptor, editor, forward, instantiationService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus | editorActionEnablement_1.Behaviour.UpdateOnModelChange);
            this._tokenSelectionSupport = instantiationService.createInstance(tokenSelectionSupport_1.TokenSelectionSupport);
            this._forward = forward;
        }
        SmartSelect.prototype.run = function () {
            var _this = this;
            var selection = this.editor.getSelection();
            var model = this.editor.getModel();
            var selectionSupport = model.getMode().logicalSelectionSupport || this._tokenSelectionSupport;
            // forget about current state
            if (state) {
                if (state.editor !== this.editor) {
                    state = null;
                }
            }
            var promise = winjs_base_1.TPromise.as(null);
            if (!state) {
                promise = selectionSupport.getRangesToPosition(model.getAssociatedResource(), selection.getStartPosition()).then(function (elements) {
                    if (arrays.isFalsyOrEmpty(elements)) {
                        return;
                    }
                    var lastState;
                    elements.filter(function (element) {
                        // filter ranges inside the selection
                        var selection = _this.editor.getSelection();
                        var range = new range_1.Range(element.range.startLineNumber, element.range.startColumn, element.range.endLineNumber, element.range.endColumn);
                        return range.containsPosition(selection.getStartPosition()) && range.containsPosition(selection.getEndPosition());
                    }).forEach(function (element) {
                        // create ranges
                        var range = element.range;
                        var state = new State(_this.editor);
                        state.selection = new range_1.Range(range.startLineNumber, range.startColumn, range.endLineNumber, range.endColumn);
                        if (lastState) {
                            state.next = lastState;
                            lastState.previous = state;
                        }
                        lastState = state;
                    });
                    // insert current selection
                    var editorState = new State(_this.editor);
                    editorState.next = lastState;
                    if (lastState) {
                        lastState.previous = editorState;
                    }
                    state = editorState;
                    // listen to caret move and forget about state
                    var unhook = _this.editor.addListener(editorCommon_1.EventType.CursorPositionChanged, function (e) {
                        if (ignoreSelection) {
                            return;
                        }
                        state = null;
                        unhook();
                    });
                });
            }
            return promise.then(function () {
                if (!state) {
                    return;
                }
                state = _this._forward ? state.next : state.previous;
                if (!state) {
                    return;
                }
                ignoreSelection = true;
                try {
                    _this.editor.setSelection(state.selection);
                }
                finally {
                    ignoreSelection = false;
                }
                return true;
            });
        };
        return SmartSelect;
    }(editorAction_1.EditorAction));
    var GrowSelectionAction = (function (_super) {
        __extends(GrowSelectionAction, _super);
        function GrowSelectionAction(descriptor, editor, instantiationService) {
            _super.call(this, descriptor, editor, true, instantiationService);
        }
        GrowSelectionAction.ID = 'editor.action.smartSelect.grow';
        GrowSelectionAction = __decorate([
            __param(2, instantiation_1.IInstantiationService)
        ], GrowSelectionAction);
        return GrowSelectionAction;
    }(SmartSelect));
    var ShrinkSelectionAction = (function (_super) {
        __extends(ShrinkSelectionAction, _super);
        function ShrinkSelectionAction(descriptor, editor, instantiationService) {
            _super.call(this, descriptor, editor, false, instantiationService);
        }
        ShrinkSelectionAction.ID = 'editor.action.smartSelect.shrink';
        ShrinkSelectionAction = __decorate([
            __param(2, instantiation_1.IInstantiationService)
        ], ShrinkSelectionAction);
        return ShrinkSelectionAction;
    }(SmartSelect));
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(GrowSelectionAction, GrowSelectionAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.RightArrow,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.RightArrow }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(ShrinkSelectionAction, ShrinkSelectionAction.ID, nls.localize(1, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.LeftArrow,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.LeftArrow }
    }));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/snippet/common/snippet", ["require", "exports", 'vs/base/common/collections', 'vs/base/common/keyCodes', 'vs/base/common/strings', 'vs/platform/keybinding/common/keybindingService', 'vs/editor/common/core/editOperation', 'vs/editor/common/core/range', 'vs/editor/common/core/selection', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions'], function (require, exports, collections, keyCodes_1, strings, keybindingService_1, editOperation_1, range_1, selection_1, editorCommon, editorCommonExtensions_1) {
    'use strict';
    (function (ExternalSnippetType) {
        ExternalSnippetType[ExternalSnippetType["TextMateSnippet"] = 0] = "TextMateSnippet";
        ExternalSnippetType[ExternalSnippetType["EmmetSnippet"] = 1] = "EmmetSnippet";
    })(exports.ExternalSnippetType || (exports.ExternalSnippetType = {}));
    var ExternalSnippetType = exports.ExternalSnippetType;
    var CodeSnippet = (function () {
        function CodeSnippet(snippetTemplate) {
            this.lines = [];
            this.placeHolders = [];
            this._lastGeneratedId = 0;
            this.startPlaceHolderIndex = 0;
            this.finishPlaceHolderIndex = -1;
            this.parseTemplate(snippetTemplate);
        }
        CodeSnippet.prototype.parseTemplate = function (template) {
            var placeHoldersMap = {};
            var i, len, j, lenJ, templateLines = template.split('\n');
            for (i = 0, len = templateLines.length; i < len; i++) {
                var parsedLine = this.parseLine(templateLines[i], function (id) {
                    if (collections.contains(placeHoldersMap, id)) {
                        return placeHoldersMap[id].value;
                    }
                    return '';
                });
                for (j = 0, lenJ = parsedLine.placeHolders.length; j < lenJ; j++) {
                    var linePlaceHolder = parsedLine.placeHolders[j];
                    var occurence = new range_1.Range(i + 1, linePlaceHolder.startColumn, i + 1, linePlaceHolder.endColumn);
                    var placeHolder;
                    if (collections.contains(placeHoldersMap, linePlaceHolder.id)) {
                        placeHolder = placeHoldersMap[linePlaceHolder.id];
                    }
                    else {
                        placeHolder = {
                            id: linePlaceHolder.id,
                            value: linePlaceHolder.value,
                            occurences: []
                        };
                        this.placeHolders.push(placeHolder);
                        if (linePlaceHolder.value === '') {
                            this.finishPlaceHolderIndex = this.placeHolders.length - 1;
                        }
                        placeHoldersMap[linePlaceHolder.id] = placeHolder;
                    }
                    placeHolder.occurences.push(occurence);
                }
                this.lines.push(parsedLine.line);
            }
            if (this.placeHolders.length > this.startPlaceHolderIndex) {
                var startPlaceHolder = this.placeHolders[this.startPlaceHolderIndex];
                if (startPlaceHolder.value === '' && startPlaceHolder.id === '') {
                    // Do not start at an empty placeholder if possible
                    if (this.placeHolders.length > 1) {
                        this.startPlaceHolderIndex++;
                    }
                }
            }
        };
        CodeSnippet.prototype.parseLine = function (line, findDefaultValueForId) {
            // Placeholder 0 is the entire line
            var placeHolderStack = [{ placeHolderId: '', placeHolderText: '' }];
            var placeHolders = [];
            var i = 0;
            var len = line.length;
            var resultIndex = 0;
            while (i < len) {
                var restOfLine = line.substr(i);
                // Look for the start of a placeholder {{
                if (/^{{/.test(restOfLine)) {
                    i += 2;
                    placeHolderStack.push({ placeHolderId: '', placeHolderText: '' });
                    // Look for id
                    var matches = restOfLine.match(/^{{(\w+):/);
                    if (Array.isArray(matches) && matches.length === 2) {
                        placeHolderStack[placeHolderStack.length - 1].placeHolderId = matches[1];
                        i += matches[1].length + 1; // +1 to account for the : at the end of the id
                    }
                    continue;
                }
                // Look for the end of a placeholder. placeHolderStack[0] is the top-level line.
                if (placeHolderStack.length > 1 && /^}}/.test(restOfLine)) {
                    i += 2;
                    if (placeHolderStack[placeHolderStack.length - 1].placeHolderId.length === 0) {
                        // This placeholder did not have an explicit id
                        placeHolderStack[placeHolderStack.length - 1].placeHolderId = placeHolderStack[placeHolderStack.length - 1].placeHolderText;
                        if (placeHolderStack[placeHolderStack.length - 1].placeHolderId === '_') {
                            // This is just an empty tab stop
                            placeHolderStack[placeHolderStack.length - 1].placeHolderId = 'TAB_STOP_' + String(++this._lastGeneratedId);
                            placeHolderStack[placeHolderStack.length - 1].placeHolderText = '';
                            --resultIndex; // Roll back one iteration of the result index as we made the text empty
                        }
                    }
                    if (placeHolderStack[placeHolderStack.length - 1].placeHolderText.length === 0) {
                        // This placeholder is empty or was a mirror
                        var defaultValue = findDefaultValueForId(placeHolderStack[placeHolderStack.length - 1].placeHolderId);
                        placeHolderStack[placeHolderStack.length - 1].placeHolderText = defaultValue;
                        resultIndex += defaultValue.length;
                    }
                    placeHolders.push({
                        id: placeHolderStack[placeHolderStack.length - 1].placeHolderId,
                        value: placeHolderStack[placeHolderStack.length - 1].placeHolderText,
                        startColumn: resultIndex + 1 - placeHolderStack[placeHolderStack.length - 1].placeHolderText.length,
                        endColumn: resultIndex + 1
                    });
                    // Insert our text into the previous placeholder
                    placeHolderStack[placeHolderStack.length - 2].placeHolderText += placeHolderStack[placeHolderStack.length - 1].placeHolderText;
                    placeHolderStack.pop();
                    continue;
                }
                // Look for escapes
                if (/^\\./.test(restOfLine)) {
                    if (restOfLine.charAt(1) === '{' || restOfLine.charAt(1) === '}' || restOfLine.charAt(1) === '\\') {
                        ++i; // Skip the escape slash and take the character literally
                    }
                    else {
                        // invalid escapes
                        placeHolderStack[placeHolderStack.length - 1].placeHolderText += line.charAt(i);
                        ++resultIndex;
                        ++i;
                    }
                }
                //This is an escape sequence or not a special character, just insert it
                placeHolderStack[placeHolderStack.length - 1].placeHolderText += line.charAt(i);
                ++resultIndex;
                ++i;
            }
            // Sort the placeholder in order of apperance:
            placeHolders.sort(function (a, b) {
                if (a.startColumn < b.startColumn) {
                    return -1;
                }
                if (a.startColumn > b.startColumn) {
                    return 1;
                }
                if (a.endColumn < b.endColumn) {
                    return -1;
                }
                if (a.endColumn > b.endColumn) {
                    return 1;
                }
                return 0;
            });
            return {
                line: placeHolderStack[0].placeHolderText,
                placeHolders: placeHolders
            };
        };
        // This is used for both TextMate and Emmet
        CodeSnippet.convertExternalSnippet = function (snippet, snippetType) {
            var openBraces = 0;
            var convertedSnippet = '';
            var i = 0;
            var len = snippet.length;
            while (i < len) {
                var restOfLine = snippet.substr(i);
                // Cursor tab stop
                if (/^\$0/.test(restOfLine)) {
                    i += 2;
                    convertedSnippet += snippetType === ExternalSnippetType.EmmetSnippet ? '{{_}}' : '{{}}';
                    continue;
                }
                if (/^\$\{0\}/.test(restOfLine)) {
                    i += 4;
                    convertedSnippet += snippetType === ExternalSnippetType.EmmetSnippet ? '{{_}}' : '{{}}';
                    continue;
                }
                if (snippetType === ExternalSnippetType.EmmetSnippet && /^\|/.test(restOfLine)) {
                    ++i;
                    convertedSnippet += '{{}}';
                    continue;
                }
                // Tab stops
                var matches = restOfLine.match(/^\$(\d+)/);
                if (Array.isArray(matches) && matches.length === 2) {
                    i += 1 + matches[1].length;
                    convertedSnippet += '{{' + matches[1] + ':}}';
                    continue;
                }
                matches = restOfLine.match(/^\$\{(\d+)\}/);
                if (Array.isArray(matches) && matches.length === 2) {
                    i += 3 + matches[1].length;
                    convertedSnippet += '{{' + matches[1] + ':}}';
                    continue;
                }
                // Open brace patterns placeholder
                if (/^\${/.test(restOfLine)) {
                    i += 2;
                    ++openBraces;
                    convertedSnippet += '{{';
                    continue;
                }
                // Close brace patterns placeholder
                if (openBraces > 0 && /^}/.test(restOfLine)) {
                    i += 1;
                    --openBraces;
                    convertedSnippet += '}}';
                    continue;
                }
                // Escapes
                if (/^\\./.test(restOfLine)) {
                    i += 2;
                    if (/^\\\$/.test(restOfLine)) {
                        convertedSnippet += '$';
                    }
                    else {
                        convertedSnippet += restOfLine.substr(0, 2);
                    }
                    continue;
                }
                // Escape braces that don't belong to a placeholder
                matches = restOfLine.match(/^({|})/);
                if (Array.isArray(matches) && matches.length === 2) {
                    i += 1;
                    convertedSnippet += '\\' + matches[1];
                    continue;
                }
                i += 1;
                convertedSnippet += restOfLine.charAt(0);
            }
            return convertedSnippet;
        };
        CodeSnippet.prototype.extractLineIndentation = function (str, maxColumn) {
            if (maxColumn === void 0) { maxColumn = Number.MAX_VALUE; }
            var fullIndentation = strings.getLeadingWhitespace(str);
            if (fullIndentation.length > maxColumn - 1) {
                return fullIndentation.substring(0, maxColumn - 1);
            }
            return fullIndentation;
        };
        CodeSnippet.prototype.bind = function (referenceLine, deltaLine, firstLineDeltaColumn, config) {
            var resultLines = [];
            var resultPlaceHolders = [];
            var referenceIndentation = this.extractLineIndentation(referenceLine, firstLineDeltaColumn + 1);
            var originalLine, originalLineIndentation, remainingLine, indentation;
            var i, len, j, lenJ;
            // Compute resultLines & keep deltaColumns as a reference for adjusting placeholders
            var deltaColumns = [];
            for (i = 0, len = this.lines.length; i < len; i++) {
                originalLine = this.lines[i];
                if (i === 0) {
                    deltaColumns[i + 1] = firstLineDeltaColumn;
                    resultLines[i] = originalLine;
                }
                else {
                    originalLineIndentation = this.extractLineIndentation(originalLine);
                    remainingLine = originalLine.substr(originalLineIndentation.length);
                    indentation = config.normalizeIndentation(referenceIndentation + originalLineIndentation);
                    deltaColumns[i + 1] = indentation.length - originalLineIndentation.length;
                    resultLines[i] = indentation + remainingLine;
                }
            }
            // Compute resultPlaceHolders
            var originalPlaceHolder, originalOccurence, resultOccurences;
            for (i = 0, len = this.placeHolders.length; i < len; i++) {
                originalPlaceHolder = this.placeHolders[i];
                resultOccurences = [];
                for (j = 0, lenJ = originalPlaceHolder.occurences.length; j < lenJ; j++) {
                    originalOccurence = originalPlaceHolder.occurences[j];
                    resultOccurences.push({
                        startLineNumber: originalOccurence.startLineNumber + deltaLine,
                        startColumn: originalOccurence.startColumn + deltaColumns[originalOccurence.startLineNumber],
                        endLineNumber: originalOccurence.endLineNumber + deltaLine,
                        endColumn: originalOccurence.endColumn + deltaColumns[originalOccurence.endLineNumber]
                    });
                }
                resultPlaceHolders.push({
                    id: originalPlaceHolder.id,
                    value: originalPlaceHolder.value,
                    occurences: resultOccurences
                });
            }
            return {
                lines: resultLines,
                placeHolders: resultPlaceHolders,
                startPlaceHolderIndex: this.startPlaceHolderIndex,
                finishPlaceHolderIndex: this.finishPlaceHolderIndex
            };
        };
        return CodeSnippet;
    }());
    exports.CodeSnippet = CodeSnippet;
    var InsertSnippetController = (function () {
        function InsertSnippetController(editor, adaptedSnippet, startLineNumber, initialAlternativeVersionId, onStop) {
            this.editor = editor;
            this._onStop = onStop;
            this.model = editor.getModel();
            this.finishPlaceHolderIndex = adaptedSnippet.finishPlaceHolderIndex;
            this.trackedPlaceHolders = [];
            this.placeHolderDecorations = [];
            this.currentPlaceHolderIndex = adaptedSnippet.startPlaceHolderIndex;
            this.highlightDecorationId = null;
            this.isFinished = false;
            this._initialAlternativeVersionId = initialAlternativeVersionId;
            this.initialize(adaptedSnippet, startLineNumber);
        }
        InsertSnippetController.prototype.dispose = function () {
            this.stopAll();
        };
        InsertSnippetController.prototype.initialize = function (adaptedSnippet, startLineNumber) {
            var _this = this;
            var i, len;
            for (i = 0, len = adaptedSnippet.placeHolders.length; i < len; i++) {
                var placeHolder = adaptedSnippet.placeHolders[i];
                var trackedRanges = [];
                for (var j = 0, lenJ = placeHolder.occurences.length; j < lenJ; j++) {
                    trackedRanges.push(this.model.addTrackedRange(placeHolder.occurences[j], editorCommon.TrackedRangeStickiness.AlwaysGrowsWhenTypingAtEdges));
                }
                this.trackedPlaceHolders.push({
                    ranges: trackedRanges
                });
            }
            this.editor.changeDecorations(function (changeAccessor) {
                var newDecorations = [];
                var endLineNumber = startLineNumber + adaptedSnippet.lines.length - 1;
                var endLineNumberMaxColumn = _this.model.getLineMaxColumn(endLineNumber);
                newDecorations.push({
                    range: new range_1.Range(startLineNumber, 1, endLineNumber, endLineNumberMaxColumn),
                    options: {
                        className: 'new-snippet',
                        isWholeLine: true
                    }
                });
                for (var i_1 = 0, len_1 = _this.trackedPlaceHolders.length; i_1 < len_1; i_1++) {
                    var className = (i_1 === _this.finishPlaceHolderIndex) ? 'finish-snippet-placeholder' : 'snippet-placeholder';
                    newDecorations.push({
                        range: _this.model.getTrackedRange(_this.trackedPlaceHolders[i_1].ranges[0]),
                        options: {
                            className: className
                        }
                    });
                }
                var decorations = changeAccessor.deltaDecorations([], newDecorations);
                _this.highlightDecorationId = decorations[0];
                _this.placeHolderDecorations = decorations.slice(1);
            });
            this.listenersToRemove = [];
            this.listenersToRemove.push(this.editor.addListener(editorCommon.EventType.ModelContentChanged, function (e) {
                if (_this.isFinished) {
                    return;
                }
                if (e.changeType === editorCommon.EventType.ModelContentChangedFlush) {
                    // a model.setValue() was called
                    _this.stopAll();
                }
                else if (e.changeType === editorCommon.EventType.ModelContentChangedLineChanged) {
                    var changedLine = e.lineNumber;
                    var highlightRange = _this.model.getDecorationRange(_this.highlightDecorationId);
                    if (changedLine < highlightRange.startLineNumber || changedLine > highlightRange.endLineNumber) {
                        _this.stopAll();
                    }
                }
                else if (e.changeType === editorCommon.EventType.ModelContentChangedLinesInserted) {
                    var insertLine = e.fromLineNumber;
                    var highlightRange = _this.model.getDecorationRange(_this.highlightDecorationId);
                    if (insertLine < highlightRange.startLineNumber || insertLine > highlightRange.endLineNumber) {
                        _this.stopAll();
                    }
                }
                else if (e.changeType === editorCommon.EventType.ModelContentChangedLinesDeleted) {
                    var deleteLine1 = e.fromLineNumber;
                    var deleteLine2 = e.toLineNumber;
                    var highlightRange = _this.model.getDecorationRange(_this.highlightDecorationId);
                    var deletedLinesAbove = (deleteLine2 < highlightRange.startLineNumber);
                    var deletedLinesBelow = (deleteLine1 > highlightRange.endLineNumber);
                    if (deletedLinesAbove || deletedLinesBelow) {
                        _this.stopAll();
                    }
                }
                var newAlternateVersionId = _this.editor.getModel().getAlternativeVersionId();
                if (_this._initialAlternativeVersionId === newAlternateVersionId) {
                    // We executed undo until we reached the same version we started with
                    _this.stopAll();
                }
            }));
            this.listenersToRemove.push(this.editor.addListener(editorCommon.EventType.CursorPositionChanged, function (e) {
                if (_this.isFinished) {
                    return;
                }
                var highlightRange = _this.model.getDecorationRange(_this.highlightDecorationId);
                var lineNumber = e.position.lineNumber;
                if (lineNumber < highlightRange.startLineNumber || lineNumber > highlightRange.endLineNumber) {
                    _this.stopAll();
                }
            }));
            this.listenersToRemove.push(this.editor.addListener(editorCommon.EventType.ModelChanged, function () {
                _this.stopAll();
            }));
            var blurTimeout = -1;
            this.listenersToRemove.push(this.editor.addListener(editorCommon.EventType.EditorBlur, function () {
                // Blur if within 100ms we do not focus back
                blurTimeout = setTimeout(function () {
                    _this.stopAll();
                }, 100);
            }));
            this.listenersToRemove.push(this.editor.addListener(editorCommon.EventType.EditorFocus, function () {
                // Cancel the blur timeout (if any)
                if (blurTimeout !== -1) {
                    clearTimeout(blurTimeout);
                    blurTimeout = -1;
                }
            }));
            this.listenersToRemove.push(this.model.addListener(editorCommon.EventType.ModelDecorationsChanged, function (e) {
                if (_this.isFinished) {
                    return;
                }
                var modelEditableRange = _this.model.getEditableRange(), previousRange = null, allCollapsed = true, allEqualToEditableRange = true;
                for (var i = 0; (allCollapsed || allEqualToEditableRange) && i < _this.trackedPlaceHolders.length; i++) {
                    var ranges = _this.trackedPlaceHolders[i].ranges;
                    for (var j = 0; (allCollapsed || allEqualToEditableRange) && j < ranges.length; j++) {
                        var range = _this.model.getTrackedRange(ranges[j]);
                        if (allCollapsed) {
                            if (!range.isEmpty()) {
                                allCollapsed = false;
                            }
                            else if (previousRange === null) {
                                previousRange = range;
                            }
                            else if (!previousRange.equalsRange(range)) {
                                allCollapsed = false;
                            }
                        }
                        if (allEqualToEditableRange && !modelEditableRange.equalsRange(range)) {
                            allEqualToEditableRange = false;
                        }
                    }
                }
                if (allCollapsed || allEqualToEditableRange) {
                    _this.stopAll();
                }
                else {
                    if (_this.finishPlaceHolderIndex !== -1) {
                        var finishPlaceHolderDecorationId = _this.placeHolderDecorations[_this.finishPlaceHolderIndex];
                        var finishPlaceHolderRange = _this.model.getDecorationRange(finishPlaceHolderDecorationId);
                        var finishPlaceHolderOptions = _this.model.getDecorationOptions(finishPlaceHolderDecorationId);
                        var finishPlaceHolderRangeIsEmpty = finishPlaceHolderRange.isEmpty();
                        var finishPlaceHolderClassNameIsForEmpty = (finishPlaceHolderOptions.className === 'finish-snippet-placeholder');
                        // Remember xor? :)
                        var needsChanging = Number(finishPlaceHolderRangeIsEmpty) ^ Number(finishPlaceHolderClassNameIsForEmpty);
                        if (needsChanging) {
                            _this.editor.changeDecorations(function (changeAccessor) {
                                var className = finishPlaceHolderRangeIsEmpty ? 'finish-snippet-placeholder' : 'snippet-placeholder';
                                changeAccessor.changeDecorationOptions(finishPlaceHolderDecorationId, {
                                    className: className
                                });
                            });
                        }
                    }
                }
            }));
            this.doLinkEditing();
        };
        InsertSnippetController.prototype.onNextPlaceHolder = function () {
            return this.changePlaceHolder(true);
        };
        InsertSnippetController.prototype.onPrevPlaceHolder = function () {
            return this.changePlaceHolder(false);
        };
        InsertSnippetController.prototype.changePlaceHolder = function (goToNext) {
            if (this.isFinished) {
                return false;
            }
            var oldPlaceHolderIndex = this.currentPlaceHolderIndex;
            var oldRange = this.model.getTrackedRange(this.trackedPlaceHolders[oldPlaceHolderIndex].ranges[0]);
            var sameRange = true;
            do {
                if (goToNext) {
                    this.currentPlaceHolderIndex = (this.currentPlaceHolderIndex + 1) % this.trackedPlaceHolders.length;
                }
                else {
                    this.currentPlaceHolderIndex = (this.trackedPlaceHolders.length + this.currentPlaceHolderIndex - 1) % this.trackedPlaceHolders.length;
                }
                var newRange = this.model.getTrackedRange(this.trackedPlaceHolders[this.currentPlaceHolderIndex].ranges[0]);
                sameRange = oldRange.equalsRange(newRange);
            } while (this.currentPlaceHolderIndex !== oldPlaceHolderIndex && sameRange);
            this.doLinkEditing();
            return true;
        };
        InsertSnippetController.prototype.onAccept = function () {
            if (this.isFinished) {
                return false;
            }
            if (this.finishPlaceHolderIndex !== -1) {
                var finishRange = this.model.getTrackedRange(this.trackedPlaceHolders[this.finishPlaceHolderIndex].ranges[0]);
                // Let's just position cursor at the end of the finish range
                this.editor.setPosition({
                    lineNumber: finishRange.endLineNumber,
                    column: finishRange.endColumn
                });
            }
            this.stopAll();
            return true;
        };
        InsertSnippetController.prototype.onEscape = function () {
            if (this.isFinished) {
                return false;
            }
            this.stopAll();
            // Cancel multi-cursor
            this.editor.setSelections([this.editor.getSelections()[0]]);
            return true;
        };
        InsertSnippetController.prototype.doLinkEditing = function () {
            var selections = [];
            for (var i = 0, len = this.trackedPlaceHolders[this.currentPlaceHolderIndex].ranges.length; i < len; i++) {
                var range = this.model.getTrackedRange(this.trackedPlaceHolders[this.currentPlaceHolderIndex].ranges[i]);
                selections.push({
                    selectionStartLineNumber: range.startLineNumber,
                    selectionStartColumn: range.startColumn,
                    positionLineNumber: range.endLineNumber,
                    positionColumn: range.endColumn
                });
            }
            this.editor.setSelections(selections);
        };
        InsertSnippetController.prototype.stopAll = function () {
            var _this = this;
            if (this.isFinished) {
                return;
            }
            this._onStop();
            this.isFinished = true;
            this.listenersToRemove.forEach(function (element) {
                element();
            });
            this.listenersToRemove = [];
            for (var i = 0; i < this.trackedPlaceHolders.length; i++) {
                var ranges = this.trackedPlaceHolders[i].ranges;
                for (var j = 0; j < ranges.length; j++) {
                    this.model.removeTrackedRange(ranges[j]);
                }
            }
            this.trackedPlaceHolders = [];
            this.editor.changeDecorations(function (changeAccessor) {
                var toRemove = [];
                toRemove.push(_this.highlightDecorationId);
                for (var i_2 = 0; i_2 < _this.placeHolderDecorations.length; i_2++) {
                    toRemove.push(_this.placeHolderDecorations[i_2]);
                }
                changeAccessor.deltaDecorations(toRemove, []);
                _this.placeHolderDecorations = [];
                _this.highlightDecorationId = null;
            });
        };
        return InsertSnippetController;
    }());
    function getSnippetController(editor) {
        return editor.getContribution(SnippetController.ID);
    }
    exports.getSnippetController = getSnippetController;
    var SnippetController = (function () {
        function SnippetController(editor, keybindingService) {
            this._editor = editor;
            this._currentController = null;
            this._inSnippetMode = keybindingService.createKey(exports.CONTEXT_SNIPPET_MODE, false);
        }
        SnippetController.prototype.dispose = function () {
            if (this._currentController) {
                this._currentController.dispose();
                this._currentController = null;
            }
        };
        SnippetController.prototype.getId = function () {
            return SnippetController.ID;
        };
        SnippetController.prototype.run = function (snippet, overwriteBefore, overwriteAfter) {
            var prevController = this._currentController;
            this._currentController = null;
            if (snippet.placeHolders.length === 0) {
                // No placeholders => execute for all editor selections
                this._runForAllSelections(snippet, overwriteBefore, overwriteAfter);
            }
            else {
                this._runForPrimarySelection(snippet, overwriteBefore, overwriteAfter);
            }
            if (!this._currentController) {
                // we didn't end up in snippet mode again => restore previous controller
                this._currentController = prevController;
            }
            else {
                // we ended up in snippet mode => dispose previous controller if necessary
                if (prevController) {
                    prevController.dispose();
                }
            }
        };
        SnippetController._getTypeRangeForSelection = function (model, selection, overwriteBefore, overwriteAfter) {
            var typeRange;
            if (overwriteBefore || overwriteAfter) {
                typeRange = model.validateRange(range_1.Range.plusRange(selection, {
                    startLineNumber: selection.positionLineNumber,
                    startColumn: selection.positionColumn - overwriteBefore,
                    endLineNumber: selection.positionLineNumber,
                    endColumn: selection.positionColumn + overwriteAfter
                }));
            }
            else {
                typeRange = selection;
            }
            return typeRange;
        };
        SnippetController._getAdaptedSnippet = function (model, snippet, typeRange) {
            return snippet.bind(model.getLineContent(typeRange.startLineNumber), typeRange.startLineNumber - 1, typeRange.startColumn - 1, model);
        };
        SnippetController._addCommandForSnippet = function (model, adaptedSnippet, typeRange, out) {
            var insertText = adaptedSnippet.lines.join('\n');
            var currentText = model.getValueInRange(typeRange, editorCommon.EndOfLinePreference.LF);
            if (insertText !== currentText) {
                out.push(editOperation_1.EditOperation.replaceMove(typeRange, insertText));
            }
        };
        SnippetController.prototype._runForPrimarySelection = function (snippet, overwriteBefore, overwriteAfter) {
            var _this = this;
            var initialAlternativeVersionId = this._editor.getModel().getAlternativeVersionId();
            var edits = [];
            var prepared = SnippetController._prepareSnippet(this._editor, this._editor.getSelection(), snippet, overwriteBefore, overwriteAfter);
            SnippetController._addCommandForSnippet(this._editor.getModel(), prepared.adaptedSnippet, prepared.typeRange, edits);
            if (edits.length > 0) {
                this._editor.getModel().pushStackElement();
                this._editor.executeEdits('editor.contrib.insertSnippetHelper', edits);
                this._editor.getModel().pushStackElement();
            }
            var cursorOnly = SnippetController._getSnippetCursorOnly(prepared.adaptedSnippet);
            if (cursorOnly) {
                this._editor.setSelection(selection_1.Selection.createSelection(cursorOnly.lineNumber, cursorOnly.column, cursorOnly.lineNumber, cursorOnly.column));
            }
            else if (prepared.adaptedSnippet.placeHolders.length > 0) {
                this._inSnippetMode.set(true);
                this._currentController = new InsertSnippetController(this._editor, prepared.adaptedSnippet, prepared.typeRange.startLineNumber, initialAlternativeVersionId, function () {
                    _this._inSnippetMode.reset();
                });
            }
        };
        SnippetController.prototype._runForAllSelections = function (snippet, overwriteBefore, overwriteAfter) {
            var selections = this._editor.getSelections(), edits = [];
            for (var i = 0; i < selections.length; i++) {
                var prepared = SnippetController._prepareSnippet(this._editor, selections[i], snippet, overwriteBefore, overwriteAfter);
                SnippetController._addCommandForSnippet(this._editor.getModel(), prepared.adaptedSnippet, prepared.typeRange, edits);
            }
            if (edits.length > 0) {
                this._editor.getModel().pushStackElement();
                this._editor.executeEdits('editor.contrib.insertSnippetHelper', edits);
                this._editor.getModel().pushStackElement();
            }
        };
        SnippetController._prepareSnippet = function (editor, selection, snippet, overwriteBefore, overwriteAfter) {
            var model = editor.getModel();
            var typeRange = SnippetController._getTypeRangeForSelection(model, selection, overwriteBefore, overwriteAfter);
            if (snippet.lines.length === 1) {
                var nextTextOnLine = model.getLineContent(typeRange.endLineNumber).substr(typeRange.endColumn - 1);
                var nextInSnippet = snippet.lines[0].substr(overwriteBefore);
                var commonPrefix = strings.commonPrefixLength(nextTextOnLine, nextInSnippet);
                if (commonPrefix > 0) {
                    typeRange = typeRange.setEndPosition(typeRange.endLineNumber, typeRange.endColumn + commonPrefix);
                }
            }
            var adaptedSnippet = SnippetController._getAdaptedSnippet(model, snippet, typeRange);
            return {
                typeRange: typeRange,
                adaptedSnippet: adaptedSnippet
            };
        };
        SnippetController._getSnippetCursorOnly = function (snippet) {
            if (snippet.placeHolders.length !== 1) {
                return null;
            }
            var placeHolder = snippet.placeHolders[0];
            if (placeHolder.value !== '' || placeHolder.occurences.length !== 1) {
                return null;
            }
            var placeHolderRange = placeHolder.occurences[0];
            if (!range_1.Range.isEmpty(placeHolderRange)) {
                return null;
            }
            return {
                lineNumber: placeHolderRange.startLineNumber,
                column: placeHolderRange.startColumn
            };
        };
        SnippetController.prototype.jumpToNextPlaceholder = function () {
            if (this._currentController) {
                this._currentController.onNextPlaceHolder();
            }
        };
        SnippetController.prototype.jumpToPrevPlaceholder = function () {
            if (this._currentController) {
                this._currentController.onPrevPlaceHolder();
            }
        };
        SnippetController.prototype.acceptSnippet = function () {
            if (this._currentController) {
                this._currentController.onAccept();
            }
        };
        SnippetController.prototype.leaveSnippet = function () {
            if (this._currentController) {
                this._currentController.onEscape();
            }
        };
        SnippetController.ID = 'editor.contrib.snippetController';
        SnippetController = __decorate([
            __param(1, keybindingService_1.IKeybindingService)
        ], SnippetController);
        return SnippetController;
    }());
    exports.CONTEXT_SNIPPET_MODE = 'inSnippetMode';
    var weight = editorCommonExtensions_1.CommonEditorRegistry.commandWeight(30);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorContribution(SnippetController);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('jumpToNextSnippetPlaceholder', weight, { primary: keyCodes_1.KeyCode.Tab }, true, exports.CONTEXT_SNIPPET_MODE, function (ctx, editor, args) {
        getSnippetController(editor).jumpToNextPlaceholder();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('jumpToPrevSnippetPlaceholder', weight, { primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Tab }, true, exports.CONTEXT_SNIPPET_MODE, function (ctx, editor, args) {
        getSnippetController(editor).jumpToPrevPlaceholder();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('acceptSnippet', weight, { primary: keyCodes_1.KeyCode.Enter }, true, exports.CONTEXT_SNIPPET_MODE, function (ctx, editor, args) {
        getSnippetController(editor).acceptSnippet();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('leaveSnippet', weight, { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, true, exports.CONTEXT_SNIPPET_MODE, function (ctx, editor, args) {
        getSnippetController(editor).leaveSnippet();
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/defineKeybinding/browser/defineKeybinding", ["require", "exports", 'vs/nls!vs/editor/contrib/defineKeybinding/browser/defineKeybinding', 'vs/base/common/async', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/base/browser/htmlContentRenderer', 'vs/base/browser/keyboardEvent', 'vs/base/browser/styleMutator', 'vs/platform/keybinding/common/keybindingResolver', 'vs/platform/keybinding/common/keybindingService', 'vs/editor/common/core/range', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/editorBrowserExtensions', 'vs/editor/contrib/snippet/common/snippet', 'vs/css!./defineKeybinding'], function (require, exports, nls, async_1, keyCodes_1, lifecycle_1, winjs_base_1, dom, htmlContentRenderer_1, keyboardEvent_1, styleMutator_1, keybindingResolver_1, keybindingService_1, range_1, editorAction_1, editorActionEnablement_1, editorCommon, editorCommonExtensions_1, editorBrowser_1, editorBrowserExtensions_1, snippet_1) {
    'use strict';
    var NLS_LAUNCH_MESSAGE = nls.localize(0, null);
    var NLS_DEFINE_MESSAGE = nls.localize(1, null);
    var NLS_DEFINE_ACTION_LABEL = nls.localize(2, null);
    var NLS_KB_LAYOUT_INFO_MESSAGE = nls.localize(3, null);
    var NLS_KB_LAYOUT_ERROR_MESSAGE = nls.localize(4, null);
    var INTERESTING_FILE = /keybindings\.json$/;
    var DefineKeybindingController = (function () {
        function DefineKeybindingController(editor, keybindingService) {
            var _this = this;
            this._dec = [];
            this._editor = editor;
            this._keybindingService = keybindingService;
            this._toDispose = [];
            this._launchWidget = new DefineKeybindingLauncherWidget(this._editor, keybindingService, function () { return _this.launch(); });
            this._defineWidget = new DefineKeybindingWidget(this._editor, keybindingService, function (keybinding) { return _this._onAccepted(keybinding); });
            this._toDispose.push(this._editor.addListener2(editorCommon.EventType.ConfigurationChanged, function (e) {
                if (isInterestingEditorModel(_this._editor)) {
                    _this._launchWidget.show();
                }
                else {
                    _this._launchWidget.hide();
                }
            }));
            this._toDispose.push(this._editor.addListener2(editorCommon.EventType.ModelChanged, function (e) {
                if (isInterestingEditorModel(_this._editor)) {
                    _this._launchWidget.show();
                }
                else {
                    _this._launchWidget.hide();
                }
                _this._onModel();
            }));
            this._updateDecorations = new async_1.RunOnceScheduler(function () { return _this._updateDecorationsNow(); }, 500);
            this._toDispose.push(this._updateDecorations);
            this._modelToDispose = [];
            this._onModel();
        }
        DefineKeybindingController.get = function (editor) {
            return editor.getContribution(DefineKeybindingController.ID);
        };
        DefineKeybindingController.prototype.getId = function () {
            return DefineKeybindingController.ID;
        };
        DefineKeybindingController.prototype.dispose = function () {
            this._modelToDispose = lifecycle_1.disposeAll(this._modelToDispose);
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
            this._launchWidget.dispose();
            this._launchWidget = null;
            this._defineWidget.dispose();
            this._defineWidget = null;
        };
        DefineKeybindingController.prototype.launch = function () {
            if (isInterestingEditorModel(this._editor)) {
                this._defineWidget.start();
            }
        };
        DefineKeybindingController.prototype._onAccepted = function (keybinding) {
            var snippetText = [
                '{',
                '\t"key": "' + keybinding + '",',
                '\t"command": "{{commandId}}",',
                '\t"when": "{{editorTextFocus}}"',
                '}{{}}'
            ].join('\n');
            snippet_1.getSnippetController(this._editor).run(new snippet_1.CodeSnippet(snippetText), 0, 0);
        };
        DefineKeybindingController.prototype._onModel = function () {
            var _this = this;
            this._modelToDispose = lifecycle_1.disposeAll(this._modelToDispose);
            var model = this._editor.getModel();
            if (!model) {
                return;
            }
            var url = model.getAssociatedResource().toString();
            if (!INTERESTING_FILE.test(url)) {
                return;
            }
            this._modelToDispose.push(model.addListener2(editorCommon.EventType.ModelContentChanged2, function (e) { return _this._updateDecorations.schedule(); }));
            this._modelToDispose.push({
                dispose: function () {
                    _this._dec = _this._editor.deltaDecorations(_this._dec, []);
                    _this._updateDecorations.cancel();
                }
            });
            this._updateDecorations.schedule();
        };
        DefineKeybindingController.prototype._updateDecorationsNow = function () {
            var _this = this;
            var model = this._editor.getModel();
            var regex = keyCodes_1.Keybinding.getUserSettingsKeybindingRegex();
            var m = model.findMatches(regex, false, true, false, false);
            var data = m.map(function (range) {
                var text = model.getValueInRange(range);
                var strKeybinding = text.substring(1, text.length - 1);
                strKeybinding = strKeybinding.replace(/\\\\/g, '\\');
                var numKeybinding = keybindingResolver_1.IOSupport.readKeybinding(strKeybinding);
                var keybinding = new keyCodes_1.Keybinding(numKeybinding);
                return {
                    strKeybinding: strKeybinding,
                    keybinding: keybinding,
                    usLabel: keybinding._toUSLabel(),
                    label: _this._keybindingService.getLabelFor(keybinding),
                    range: range
                };
            });
            data = data.filter(function (entry) {
                return (entry.usLabel !== entry.label);
            });
            var newDecorations = [];
            data.forEach(function (item) {
                var msg;
                var className;
                var inlineClassName;
                var overviewRulerColor;
                if (!item.label) {
                    // this is the error case
                    msg = [{
                            tagName: 'span',
                            text: NLS_KB_LAYOUT_ERROR_MESSAGE
                        }];
                    className = 'keybindingError';
                    inlineClassName = 'inlineKeybindingError';
                    overviewRulerColor = 'rgba(250, 100, 100, 0.6)';
                }
                else {
                    // this is the info case
                    msg = [{
                            tagName: 'span',
                            text: NLS_KB_LAYOUT_INFO_MESSAGE
                        }];
                    msg = msg.concat(_this._keybindingService.getHTMLLabelFor(item.keybinding));
                    className = 'keybindingInfo';
                    inlineClassName = 'inlineKeybindingInfo';
                    overviewRulerColor = 'rgba(100, 100, 250, 0.6)';
                }
                // icon decoration
                newDecorations.push({
                    range: new range_1.Range(item.range.startLineNumber, item.range.startColumn, item.range.startLineNumber, item.range.startColumn + 1),
                    options: {
                        stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
                        inlineClassName: inlineClassName
                    }
                });
                // highlight + message decoration
                newDecorations.push({
                    range: item.range,
                    options: {
                        stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
                        className: className,
                        htmlMessage: msg,
                        overviewRuler: {
                            color: overviewRulerColor,
                            darkColor: overviewRulerColor,
                            position: editorCommon.OverviewRulerLane.Right
                        }
                    }
                });
            });
            this._dec = this._editor.deltaDecorations(this._dec, newDecorations);
        };
        DefineKeybindingController.ID = 'editor.contrib.defineKeybinding';
        DefineKeybindingController = __decorate([
            __param(1, keybindingService_1.IKeybindingService)
        ], DefineKeybindingController);
        return DefineKeybindingController;
    }());
    exports.DefineKeybindingController = DefineKeybindingController;
    var DefineKeybindingLauncherWidget = (function () {
        function DefineKeybindingLauncherWidget(editor, keybindingService, onLaunch) {
            this._editor = editor;
            this._domNode = document.createElement('div');
            this._domNode.className = 'defineKeybindingLauncher';
            this._domNode.style.display = 'none';
            this._isVisible = false;
            var keybinding = keybindingService.lookupKeybindings(DefineKeybindingAction.ID);
            var extra = '';
            if (keybinding.length > 0) {
                extra += ' (' + keybindingService.getLabelFor(keybinding[0]) + ')';
            }
            this._domNode.appendChild(document.createTextNode(NLS_LAUNCH_MESSAGE + extra));
            this._toDispose = [];
            this._toDispose.push(dom.addDisposableListener(this._domNode, 'click', function (e) {
                onLaunch();
            }));
            this._editor.addOverlayWidget(this);
        }
        DefineKeybindingLauncherWidget.prototype.dispose = function () {
            this._editor.removeOverlayWidget(this);
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        DefineKeybindingLauncherWidget.prototype.show = function () {
            if (this._isVisible) {
                return;
            }
            this._domNode.style.display = 'block';
            this._isVisible = true;
            this._editor.layoutOverlayWidget(this);
        };
        DefineKeybindingLauncherWidget.prototype.hide = function () {
            if (!this._isVisible) {
                return;
            }
            this._domNode.style.display = 'none';
            this._isVisible = false;
            this._editor.layoutOverlayWidget(this);
        };
        // ----- IOverlayWidget API
        DefineKeybindingLauncherWidget.prototype.getId = function () {
            return DefineKeybindingLauncherWidget.ID;
        };
        DefineKeybindingLauncherWidget.prototype.getDomNode = function () {
            return this._domNode;
        };
        DefineKeybindingLauncherWidget.prototype.getPosition = function () {
            return {
                preference: this._isVisible ? editorBrowser_1.OverlayWidgetPositionPreference.BOTTOM_RIGHT_CORNER : null
            };
        };
        DefineKeybindingLauncherWidget.ID = 'editor.contrib.defineKeybindingLauncherWidget';
        return DefineKeybindingLauncherWidget;
    }());
    var DefineKeybindingWidget = (function () {
        function DefineKeybindingWidget(editor, keybindingService, onAccepted) {
            var _this = this;
            this._editor = editor;
            this._keybindingService = keybindingService;
            this._onAccepted = onAccepted;
            this._toDispose = [];
            this._lastKeybinding = null;
            this._domNode = document.createElement('div');
            this._domNode.className = 'defineKeybindingWidget';
            styleMutator_1.StyleMutator.setWidth(this._domNode, DefineKeybindingWidget.WIDTH);
            styleMutator_1.StyleMutator.setHeight(this._domNode, DefineKeybindingWidget.HEIGHT);
            this._domNode.style.display = 'none';
            this._isVisible = false;
            this._messageNode = document.createElement('div');
            this._messageNode.className = 'message';
            this._messageNode.innerText = NLS_DEFINE_MESSAGE;
            this._domNode.appendChild(this._messageNode);
            this._inputNode = document.createElement('input');
            this._inputNode.className = 'input';
            this._inputNode.type = 'text';
            this._domNode.appendChild(this._inputNode);
            this._outputNode = document.createElement('div');
            this._outputNode.className = 'output';
            this._domNode.appendChild(this._outputNode);
            this._toDispose.push(dom.addDisposableListener(this._inputNode, 'keydown', function (e) {
                var keyEvent = new keyboardEvent_1.StandardKeyboardEvent(e);
                keyEvent.preventDefault();
                keyEvent.stopPropagation();
                var kb = new keyCodes_1.Keybinding(keyEvent.asKeybinding());
                switch (kb.value) {
                    case keyCodes_1.CommonKeybindings.ENTER:
                        if (_this._lastKeybinding) {
                            _this._onAccepted(_this._lastKeybinding.toUserSettingsLabel());
                        }
                        _this._stop();
                        return;
                    case keyCodes_1.CommonKeybindings.ESCAPE:
                        _this._stop();
                        return;
                }
                _this._lastKeybinding = kb;
                _this._inputNode.value = _this._lastKeybinding.toUserSettingsLabel().toLowerCase();
                _this._inputNode.title = 'keyCode: ' + keyEvent.browserEvent.keyCode;
                dom.clearNode(_this._outputNode);
                var htmlkb = _this._keybindingService.getHTMLLabelFor(_this._lastKeybinding);
                htmlkb.forEach(function (item) { return _this._outputNode.appendChild(htmlContentRenderer_1.renderHtml(item)); });
            }));
            this._toDispose.push(this._editor.addListener2(editorCommon.EventType.ConfigurationChanged, function (e) {
                if (_this._isVisible) {
                    _this._layout();
                }
            }));
            this._toDispose.push(dom.addDisposableListener(this._inputNode, 'blur', function (e) { return _this._stop(); }));
            this._editor.addOverlayWidget(this);
        }
        DefineKeybindingWidget.prototype.dispose = function () {
            this._editor.removeOverlayWidget(this);
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        DefineKeybindingWidget.prototype.getId = function () {
            return DefineKeybindingWidget.ID;
        };
        DefineKeybindingWidget.prototype.getDomNode = function () {
            return this._domNode;
        };
        DefineKeybindingWidget.prototype.getPosition = function () {
            return {
                preference: null
            };
        };
        DefineKeybindingWidget.prototype._show = function () {
            if (this._isVisible) {
                return;
            }
            this._isVisible = true;
            this._layout();
            this._domNode.style.display = 'block';
        };
        DefineKeybindingWidget.prototype._hide = function () {
            if (!this._isVisible) {
                return;
            }
            this._isVisible = false;
            this._domNode.style.display = 'none';
        };
        DefineKeybindingWidget.prototype._layout = function () {
            var editorLayout = this._editor.getLayoutInfo();
            var top = Math.round((editorLayout.height - DefineKeybindingWidget.HEIGHT) / 2);
            styleMutator_1.StyleMutator.setTop(this._domNode, top);
            var left = Math.round((editorLayout.width - DefineKeybindingWidget.WIDTH) / 2);
            styleMutator_1.StyleMutator.setLeft(this._domNode, left);
        };
        DefineKeybindingWidget.prototype.start = function () {
            this._editor.revealPositionInCenterIfOutsideViewport(this._editor.getPosition());
            this._show();
            this._lastKeybinding = null;
            this._inputNode.value = '';
            dom.clearNode(this._outputNode);
            this._inputNode.focus();
        };
        DefineKeybindingWidget.prototype._stop = function () {
            this._editor.focus();
            this._hide();
        };
        DefineKeybindingWidget.ID = 'editor.contrib.defineKeybindingWidget';
        DefineKeybindingWidget.WIDTH = 340;
        DefineKeybindingWidget.HEIGHT = 90;
        return DefineKeybindingWidget;
    }());
    var DefineKeybindingAction = (function (_super) {
        __extends(DefineKeybindingAction, _super);
        function DefineKeybindingAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.UpdateOnModelChange | editorActionEnablement_1.Behaviour.Writeable);
        }
        DefineKeybindingAction.prototype.isSupported = function () {
            if (!_super.prototype.isSupported.call(this)) {
                return false;
            }
            return isInterestingEditorModel(this.editor);
        };
        DefineKeybindingAction.prototype.run = function () {
            var controller = DefineKeybindingController.get(this.editor);
            controller.launch();
            return winjs_base_1.TPromise.as(true);
        };
        DefineKeybindingAction.ID = 'editor.action.defineKeybinding';
        return DefineKeybindingAction;
    }(editorAction_1.EditorAction));
    exports.DefineKeybindingAction = DefineKeybindingAction;
    function isInterestingEditorModel(editor) {
        if (editor.getConfiguration().readOnly) {
            return false;
        }
        var model = editor.getModel();
        if (!model) {
            return false;
        }
        var url = model.getAssociatedResource().toString();
        return INTERESTING_FILE.test(url);
    }
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(DefineKeybindingController);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(DefineKeybindingAction, DefineKeybindingAction.ID, NLS_DEFINE_ACTION_LABEL, {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K)
    }));
});

define("vs/editor/contrib/suggest/common/suggest", ["require", "exports", 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/winjs.base', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry', 'vs/editor/common/modes/supports'], function (require, exports, async_1, errors_1, winjs_base_1, editorCommonExtensions_1, languageFeatureRegistry_1, supports_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.CONTEXT_SUGGEST_WIDGET_VISIBLE = 'suggestWidgetVisible';
    exports.CONTEXT_SUGGESTION_SUPPORTS_ACCEPT_ON_KEY = 'suggestionSupportsAcceptOnKey';
    exports.ACCEPT_SELECTED_SUGGESTION_CMD = 'acceptSelectedSuggestion';
    exports.SuggestRegistry = new languageFeatureRegistry_1.default('suggestSupport');
    function suggest(model, position, triggerCharacter, groups) {
        if (!groups) {
            groups = exports.SuggestRegistry.orderedGroups(model);
        }
        var resource = model.getAssociatedResource();
        var suggestions = [];
        var factory = groups.map(function (supports, index) {
            return function () {
                // stop as soon as a group produced a result
                if (suggestions.length > 0) {
                    return;
                }
                // for each support in the group ask for suggestions
                var promises = supports.map(function (support) {
                    return support.suggest(resource, position, triggerCharacter).then(function (values) {
                        var result = [];
                        for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
                            var suggestResult = values_1[_i];
                            if (!suggestResult
                                || !Array.isArray(suggestResult.suggestions)
                                || suggestResult.suggestions.length === 0) {
                                continue;
                            }
                            result.push({
                                support: support,
                                currentWord: suggestResult.currentWord,
                                incomplete: suggestResult.incomplete,
                                suggestions: suggestResult.suggestions
                            });
                        }
                        return result;
                    }, errors_1.onUnexpectedError);
                });
                return winjs_base_1.TPromise.join(promises).then(function (values) {
                    for (var _i = 0, values_2 = values; _i < values_2.length; _i++) {
                        var value = values_2[_i];
                        if (Array.isArray(value) && value.length > 0) {
                            suggestions.push(value);
                        }
                    }
                });
            };
        });
        return async_1.sequence(factory).then(function () {
            // add snippets to the first group
            var snippets = supports_1.SnippetsRegistry.getSnippets(model, position);
            if (suggestions.length === 0) {
                suggestions.push([snippets]);
            }
            else {
                suggestions[0].push(snippets);
            }
            return suggestions;
        });
    }
    exports.suggest = suggest;
    editorCommonExtensions_1.CommonEditorRegistry.registerDefaultLanguageCommand('_executeCompletionItemProvider', function (model, position, args) {
        var triggerCharacter = args['triggerCharacter'];
        if (typeof triggerCharacter !== 'undefined' && typeof triggerCharacter !== 'string') {
            throw errors_1.illegalArgument('triggerCharacter');
        }
        return suggest(model, position, triggerCharacter);
    });
});

define("vs/editor/contrib/suggest/browser/suggestModel", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/event', 'vs/base/common/lifecycle', 'vs/base/common/strings', 'vs/base/common/winjs.base', 'vs/editor/common/editorCommon', 'vs/editor/contrib/snippet/common/snippet', '../common/suggest', './completionModel'], function (require, exports, errors_1, event_1, lifecycle_1, strings_1, winjs_base_1, editorCommon_1, snippet_1, suggest_1, completionModel_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Context = (function () {
        function Context(editor, auto) {
            this.auto = auto;
            var model = editor.getModel();
            var position = editor.getPosition();
            var lineContent = model.getLineContent(position.lineNumber);
            var wordUnderCursor = model.getWordAtPosition(position);
            if (wordUnderCursor) {
                this.wordBefore = lineContent.substring(wordUnderCursor.startColumn - 1, position.column - 1);
                this.wordAfter = lineContent.substring(position.column - 1, wordUnderCursor.endColumn - 1);
            }
            else {
                this.wordBefore = '';
                this.wordAfter = '';
            }
            this.lineNumber = position.lineNumber;
            this.column = position.column;
            this.lineContentBefore = lineContent.substr(0, position.column - 1);
            this.lineContentAfter = lineContent.substr(position.column - 1);
            this.isInEditableRange = true;
            if (model.hasEditableRange()) {
                var editableRange = model.getEditableRange();
                if (!editableRange.containsPosition(position)) {
                    this.isInEditableRange = false;
                }
            }
            var lineContext = model.getLineContext(position.lineNumber);
            var character = model.getLineContent(position.lineNumber).charAt(position.column - 1);
            var supports = suggest_1.SuggestRegistry.all(model);
            this.isAutoTriggerEnabled = supports.some(function (s) { return s.shouldAutotriggerSuggest(lineContext, position.column - 1, character); });
        }
        Context.prototype.shouldAutoTrigger = function () {
            if (!this.isAutoTriggerEnabled) {
                // Support disallows it
                return false;
            }
            if (this.wordBefore.length === 0) {
                // Word before position is empty
                return false;
            }
            if (!isNaN(Number(this.wordBefore))) {
                // Word before is number only
                return false;
            }
            if (this.wordAfter.length > 0) {
                // Word after position is non empty
                return false;
            }
            return true;
        };
        Context.prototype.isDifferentContext = function (context) {
            if (this.lineNumber !== context.lineNumber) {
                // Line number has changed
                return true;
            }
            if (context.column < this.column - this.wordBefore.length) {
                // column went before word start
                return true;
            }
            if (!strings_1.startsWith(context.lineContentBefore, this.lineContentBefore) || this.lineContentAfter !== context.lineContentAfter) {
                // Line has changed before position
                return true;
            }
            if (context.wordBefore === '' && context.lineContentBefore !== this.lineContentBefore) {
                // Most likely a space has been typed
                return true;
            }
            return false;
        };
        Context.prototype.shouldRetrigger = function (context) {
            if (!strings_1.startsWith(this.lineContentBefore, context.lineContentBefore) || this.lineContentAfter !== context.lineContentAfter) {
                // Doesn't look like the same line
                return false;
            }
            if (this.lineContentBefore.length > context.lineContentBefore.length && this.wordBefore.length === 0) {
                // Text was deleted and previous current word was empty
                return false;
            }
            if (this.auto && context.wordBefore.length === 0) {
                // Currently in auto mode and new current word is empty
                return false;
            }
            return true;
        };
        return Context;
    }());
    var State;
    (function (State) {
        State[State["Idle"] = 0] = "Idle";
        State[State["Manual"] = 1] = "Manual";
        State[State["Auto"] = 2] = "Auto";
    })(State || (State = {}));
    var SuggestModel = (function () {
        function SuggestModel(editor) {
            var _this = this;
            this.editor = editor;
            this._onDidCancel = new event_1.Emitter();
            this._onDidTrigger = new event_1.Emitter();
            this._onDidSuggest = new event_1.Emitter();
            this._onDidAccept = new event_1.Emitter();
            this.state = State.Idle;
            this.triggerAutoSuggestPromise = null;
            this.requestPromise = null;
            this.raw = null;
            this.completionModel = null;
            this.incomplete = false;
            this.context = null;
            this.toDispose = [];
            this.toDispose.push(this.editor.addListener2(editorCommon_1.EventType.ConfigurationChanged, function () { return _this.onEditorConfigurationChange(); }));
            this.toDispose.push(this.editor.addListener2(editorCommon_1.EventType.CursorSelectionChanged, function (e) { return _this.onCursorChange(e); }));
            this.toDispose.push(this.editor.addListener2(editorCommon_1.EventType.ModelChanged, function () { return _this.cancel(); }));
            this.onEditorConfigurationChange();
        }
        Object.defineProperty(SuggestModel.prototype, "onDidCancel", {
            get: function () { return this._onDidCancel.event; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SuggestModel.prototype, "onDidTrigger", {
            get: function () { return this._onDidTrigger.event; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SuggestModel.prototype, "onDidSuggest", {
            get: function () { return this._onDidSuggest.event; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SuggestModel.prototype, "onDidAccept", {
            get: function () { return this._onDidAccept.event; },
            enumerable: true,
            configurable: true
        });
        SuggestModel.prototype.cancel = function (silent, retrigger) {
            if (silent === void 0) { silent = false; }
            if (retrigger === void 0) { retrigger = false; }
            var actuallyCanceled = this.state !== State.Idle;
            if (this.triggerAutoSuggestPromise) {
                this.triggerAutoSuggestPromise.cancel();
                this.triggerAutoSuggestPromise = null;
            }
            if (this.requestPromise) {
                this.requestPromise.cancel();
                this.requestPromise = null;
            }
            this.state = State.Idle;
            this.raw = null;
            this.completionModel = null;
            this.incomplete = false;
            this.context = null;
            if (!silent) {
                this._onDidCancel.fire({ retrigger: retrigger });
            }
            return actuallyCanceled;
        };
        SuggestModel.prototype.getRequestPosition = function () {
            if (!this.context) {
                return null;
            }
            return {
                lineNumber: this.context.lineNumber,
                column: this.context.column
            };
        };
        SuggestModel.prototype.isAutoSuggest = function () {
            return this.state === State.Auto;
        };
        SuggestModel.prototype.onCursorChange = function (e) {
            var _this = this;
            if (!e.selection.isEmpty()) {
                this.cancel();
                return;
            }
            if (e.source !== 'keyboard' || e.reason !== '') {
                this.cancel();
                return;
            }
            if (!suggest_1.SuggestRegistry.has(this.editor.getModel())) {
                return;
            }
            var isInactive = this.state === State.Idle;
            if (isInactive && !this.editor.getConfiguration().quickSuggestions) {
                return;
            }
            var ctx = new Context(this.editor, false);
            if (isInactive) {
                // trigger was not called or it was canceled
                this.cancel();
                if (ctx.shouldAutoTrigger()) {
                    this.triggerAutoSuggestPromise = winjs_base_1.TPromise.timeout(this.autoSuggestDelay);
                    this.triggerAutoSuggestPromise.then(function () {
                        _this.triggerAutoSuggestPromise = null;
                        _this.trigger(true);
                    });
                }
            }
            else if (this.raw && this.incomplete) {
                this.trigger(this.state === State.Auto, undefined, true);
            }
            else {
                this.onNewContext(ctx);
            }
        };
        SuggestModel.prototype.trigger = function (auto, triggerCharacter, retrigger, groups) {
            var _this = this;
            if (retrigger === void 0) { retrigger = false; }
            var model = this.editor.getModel();
            var characterTriggered = !!triggerCharacter;
            groups = groups || suggest_1.SuggestRegistry.orderedGroups(model);
            if (groups.length === 0) {
                return;
            }
            var ctx = new Context(this.editor, auto);
            if (!ctx.isInEditableRange) {
                return;
            }
            // Cancel previous requests, change state & update UI
            this.cancel(false, retrigger);
            this.state = (auto || characterTriggered) ? State.Auto : State.Manual;
            this._onDidTrigger.fire({ auto: this.isAutoSuggest(), characterTriggered: characterTriggered, retrigger: retrigger });
            // Capture context when request was sent
            this.context = ctx;
            var position = this.editor.getPosition();
            this.requestPromise = suggest_1.suggest(model, position, triggerCharacter, groups).then(function (all) {
                _this.requestPromise = null;
                if (_this.state === State.Idle) {
                    return;
                }
                _this.raw = all;
                _this.incomplete = all.reduce(function (r, s) { return r || s.reduce(function (r, s) { return r || s.incomplete; }, false); }, false);
                _this.onNewContext(new Context(_this.editor, auto));
            }).then(null, errors_1.onUnexpectedError);
        };
        SuggestModel.prototype.onNewContext = function (ctx) {
            if (this.context && this.context.isDifferentContext(ctx)) {
                if (this.context.shouldRetrigger(ctx)) {
                    this.trigger(this.state === State.Auto, undefined, true);
                }
                else {
                    this.cancel();
                }
                return;
            }
            if (this.raw) {
                var auto = this.isAutoSuggest();
                var isFrozen = false;
                if (this.completionModel && this.completionModel.raw === this.raw) {
                    var oldCurrentWord = this.completionModel.currentWord;
                    this.completionModel.currentWord = ctx.wordBefore;
                    var visibleCount = this.completionModel.items.length;
                    if (!auto && visibleCount === 0) {
                        this.completionModel.currentWord = oldCurrentWord;
                        isFrozen = true;
                    }
                }
                else {
                    this.completionModel = new completionModel_1.CompletionModel(this.raw, ctx.wordBefore);
                }
                this._onDidSuggest.fire({
                    completionModel: this.completionModel,
                    currentWord: ctx.wordBefore,
                    isFrozen: isFrozen,
                    auto: this.isAutoSuggest()
                });
            }
        };
        SuggestModel.prototype.accept = function (suggestion, overwriteBefore, overwriteAfter) {
            if (this.raw === null) {
                return false;
            }
            this._onDidAccept.fire({
                snippet: new snippet_1.CodeSnippet(suggestion.codeSnippet),
                overwriteBefore: overwriteBefore + (this.editor.getPosition().column - this.context.column),
                overwriteAfter: overwriteAfter
            });
            this.cancel();
            return true;
        };
        SuggestModel.prototype.onEditorConfigurationChange = function () {
            this.autoSuggestDelay = this.editor.getConfiguration().quickSuggestionsDelay;
            if (isNaN(this.autoSuggestDelay) || (!this.autoSuggestDelay && this.autoSuggestDelay !== 0) || this.autoSuggestDelay < 0) {
                this.autoSuggestDelay = 10;
            }
        };
        SuggestModel.prototype.dispose = function () {
            this.cancel(true);
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
        };
        return SuggestModel;
    }());
    exports.SuggestModel = SuggestModel;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/suggest/browser/suggestWidget", ["require", "exports", 'vs/nls!vs/editor/contrib/suggest/browser/suggestWidget', 'vs/base/common/strings', 'vs/base/common/errors', 'vs/base/common/event', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/base/browser/ui/highlightedlabel/highlightedLabel', 'vs/base/browser/ui/list/listWidget', 'vs/base/browser/ui/scrollbar/scrollableElementImpl', 'vs/platform/instantiation/common/instantiation', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/telemetry/common/telemetry', 'vs/editor/common/editorCommon', 'vs/editor/browser/editorBrowser', '../common/suggest', 'vs/base/browser/ui/aria/aria', 'vs/css!./suggest'], function (require, exports, nls, strings, errors_1, event_1, lifecycle_1, winjs_base_1, dom_1, highlightedLabel_1, listWidget_1, scrollableElementImpl_1, instantiation_1, keybindingService_1, telemetry_1, editorCommon_1, editorBrowser_1, suggest_1, aria_1) {
    'use strict';
    var Renderer = (function () {
        function Renderer(widget, keybindingService) {
            this.widget = widget;
            var keybindings = keybindingService.lookupKeybindings('editor.action.triggerSuggest');
            this.triggerKeybindingLabel = keybindings.length === 0 ? '' : " (" + keybindingService.getLabelFor(keybindings[0]) + ")";
        }
        Object.defineProperty(Renderer.prototype, "templateId", {
            get: function () {
                return 'suggestion';
            },
            enumerable: true,
            configurable: true
        });
        Renderer.prototype.renderTemplate = function (container) {
            var data = Object.create(null);
            data.root = container;
            data.icon = dom_1.append(container, dom_1.emmet('.icon'));
            data.colorspan = dom_1.append(data.icon, dom_1.emmet('span.colorspan'));
            var text = dom_1.append(container, dom_1.emmet('.text'));
            var main = dom_1.append(text, dom_1.emmet('.main'));
            data.highlightedLabel = new highlightedLabel_1.HighlightedLabel(main);
            data.typeLabel = dom_1.append(main, dom_1.emmet('span.type-label'));
            var docs = dom_1.append(text, dom_1.emmet('.docs'));
            data.documentation = dom_1.append(docs, dom_1.emmet('span.docs-text'));
            data.documentationDetails = dom_1.append(docs, dom_1.emmet('span.docs-details.octicon.octicon-info'));
            data.documentationDetails.title = nls.localize(0, null, this.triggerKeybindingLabel);
            return data;
        };
        Renderer.prototype.renderElement = function (element, index, templateData) {
            var _this = this;
            var data = templateData;
            var suggestion = element.suggestion;
            if (suggestion.documentationLabel) {
                data.root.setAttribute('aria-label', nls.localize(1, null, suggestion.label));
            }
            else {
                data.root.setAttribute('aria-label', nls.localize(2, null, suggestion.label));
            }
            if (suggestion.type && suggestion.type.charAt(0) === '#') {
                data.icon.className = 'icon customcolor';
                data.colorspan.style.backgroundColor = suggestion.type.substring(1);
            }
            else {
                data.icon.className = 'icon ' + suggestion.type;
                data.colorspan.style.backgroundColor = '';
            }
            data.highlightedLabel.set(suggestion.label, element.highlights);
            data.typeLabel.textContent = suggestion.typeLabel || '';
            data.documentation.textContent = suggestion.documentationLabel || '';
            if (suggestion.documentationLabel) {
                dom_1.show(data.documentationDetails);
                data.documentationDetails.onmousedown = function (e) {
                    e.stopPropagation();
                    e.preventDefault();
                };
                data.documentationDetails.onclick = function (e) {
                    e.stopPropagation();
                    e.preventDefault();
                    _this.widget.toggleDetails();
                };
            }
            else {
                dom_1.hide(data.documentationDetails);
                data.documentationDetails.onmousedown = null;
                data.documentationDetails.onclick = null;
            }
        };
        Renderer.prototype.disposeTemplate = function (templateData) {
            templateData.highlightedLabel.dispose();
        };
        Renderer = __decorate([
            __param(1, keybindingService_1.IKeybindingService)
        ], Renderer);
        return Renderer;
    }());
    var FocusHeight = 35;
    var UnfocusedHeight = 19;
    var Delegate = (function () {
        function Delegate(listProvider) {
            this.listProvider = listProvider;
        }
        Delegate.prototype.getHeight = function (element) {
            var focus = this.listProvider().getFocus()[0];
            if (element.suggestion.documentationLabel && element === focus) {
                return FocusHeight;
            }
            return UnfocusedHeight;
        };
        Delegate.prototype.getTemplateId = function (element) {
            return 'suggestion';
        };
        return Delegate;
    }());
    function computeScore(suggestion, currentWord, currentWordLowerCase) {
        var suggestionLowerCase = suggestion.toLowerCase();
        var score = 0;
        for (var i = 0; i < currentWord.length && i < suggestion.length; i++) {
            if (currentWord[i] === suggestion[i]) {
                score += 2;
            }
            else if (currentWordLowerCase[i] === suggestionLowerCase[i]) {
                score += 1;
            }
            else {
                break;
            }
        }
        return score;
    }
    var State;
    (function (State) {
        State[State["Hidden"] = 0] = "Hidden";
        State[State["Loading"] = 1] = "Loading";
        State[State["Empty"] = 2] = "Empty";
        State[State["Open"] = 3] = "Open";
        State[State["Frozen"] = 4] = "Frozen";
        State[State["Details"] = 5] = "Details";
    })(State || (State = {}));
    var SuggestionDetails = (function () {
        function SuggestionDetails(container, widget) {
            this.widget = widget;
            this.el = dom_1.append(container, dom_1.emmet('.details'));
            var header = dom_1.append(this.el, dom_1.emmet('.header'));
            this.title = dom_1.append(header, dom_1.emmet('span.title'));
            this.back = dom_1.append(header, dom_1.emmet('span.go-back.octicon.octicon-mail-reply'));
            this.back.title = nls.localize(3, null);
            this.body = dom_1.emmet('.body');
            this.scrollable = new scrollableElementImpl_1.ScrollableElement(this.body, {});
            dom_1.append(this.el, this.scrollable.getDomNode());
            this.type = dom_1.append(this.body, dom_1.emmet('p.type'));
            this.docs = dom_1.append(this.body, dom_1.emmet('p.docs'));
            this.ariaLabel = null;
        }
        Object.defineProperty(SuggestionDetails.prototype, "element", {
            get: function () {
                return this.el;
            },
            enumerable: true,
            configurable: true
        });
        SuggestionDetails.prototype.render = function (item) {
            var _this = this;
            if (!item) {
                this.title.textContent = '';
                this.type.textContent = '';
                this.docs.textContent = '';
                this.ariaLabel = null;
                return;
            }
            this.title.innerText = item.suggestion.label;
            this.type.innerText = item.suggestion.typeLabel || '';
            this.docs.innerText = item.suggestion.documentationLabel;
            this.back.onmousedown = function (e) {
                e.preventDefault();
                e.stopPropagation();
            };
            this.back.onclick = function (e) {
                e.preventDefault();
                e.stopPropagation();
                _this.widget.toggleDetails();
            };
            this.scrollable.onElementDimensions();
            this.scrollable.onElementInternalDimensions();
            this.ariaLabel = strings.format('{0}\n{1}\n{2}', item.suggestion.label || '', item.suggestion.typeLabel || '', item.suggestion.documentationLabel || '');
        };
        SuggestionDetails.prototype.getAriaLabel = function () {
            return this.ariaLabel;
        };
        SuggestionDetails.prototype.scrollDown = function (much) {
            if (much === void 0) { much = 8; }
            this.body.scrollTop += much;
        };
        SuggestionDetails.prototype.scrollUp = function (much) {
            if (much === void 0) { much = 8; }
            this.body.scrollTop -= much;
        };
        SuggestionDetails.prototype.pageDown = function () {
            this.scrollDown(80);
        };
        SuggestionDetails.prototype.pageUp = function () {
            this.scrollUp(80);
        };
        SuggestionDetails.prototype.dispose = function () {
            this.el.parentElement.removeChild(this.el);
            this.el = null;
        };
        return SuggestionDetails;
    }());
    var SuggestWidget = (function () {
        function SuggestWidget(editor, model, keybindingService, telemetryService, instantiationService) {
            var _this = this;
            this.editor = editor;
            this.model = model;
            this.allowEditorOverflow = true; // Editor.IContentWidget.allowEditorOverflow
            this._onDidVisibilityChange = new event_1.Emitter();
            this.isAuto = false;
            this.focusedItem = null;
            this.suggestionSupportsAutoAccept = keybindingService.createKey(suggest_1.CONTEXT_SUGGESTION_SUPPORTS_ACCEPT_ON_KEY, true);
            this.telemetryData = null;
            this.telemetryService = telemetryService;
            this.element = dom_1.emmet('.editor-widget.suggest-widget.monaco-editor-background');
            this.element.style.width = SuggestWidget.WIDTH + 'px';
            this.element.style.top = '0';
            this.element.style.left = '0';
            if (!this.editor.getConfiguration().iconsInSuggestions) {
                dom_1.addClass(this.element, 'no-icons');
            }
            this.messageElement = dom_1.append(this.element, dom_1.emmet('.message'));
            this.listElement = dom_1.append(this.element, dom_1.emmet('.tree'));
            this.details = new SuggestionDetails(this.element, this);
            var renderer = instantiationService.createInstance(Renderer, this);
            this.delegate = new Delegate(function () { return _this.list; });
            this.list = new listWidget_1.List(this.listElement, this.delegate, [renderer]);
            this.toDispose = [
                editor.addListener2(editorCommon_1.EventType.ModelChanged, function () { return _this.onModelModeChanged(); }),
                editor.addListener2(editorCommon_1.EventType.ModelModeChanged, function () { return _this.onModelModeChanged(); }),
                editor.addListener2(editorCommon_1.EventType.ModelModeSupportChanged, function (e) { return e.suggestSupport && _this.onModelModeChanged(); }),
                suggest_1.SuggestRegistry.onDidChange(function () { return _this.onModelModeChanged(); }),
                editor.addListener2(editorCommon_1.EventType.EditorTextBlur, function () { return _this.onEditorBlur(); }),
                this.list.onSelectionChange(function (e) { return _this.onListSelection(e); }),
                this.list.onFocusChange(function (e) { return _this.onListFocus(e); }),
                this.editor.addListener2(editorCommon_1.EventType.CursorSelectionChanged, function () { return _this.onCursorSelectionChanged(); }),
                this.model.onDidTrigger(function (e) { return _this.onDidTrigger(e); }),
                this.model.onDidSuggest(function (e) { return _this.onDidSuggest(e); }),
                this.model.onDidCancel(function (e) { return _this.onDidCancel(e); })
            ];
            this.onModelModeChanged();
            this.editor.addContentWidget(this);
            this.setState(State.Hidden);
            // TODO@Alex: this is useful, but spammy
            // var isVisible = false;
            // this.onDidVisibilityChange((newIsVisible) => {
            // 	if (isVisible === newIsVisible) {
            // 		return;
            // 	}
            // 	isVisible = newIsVisible;
            // 	if (isVisible) {
            // 		alert(nls.localize('suggestWidgetAriaVisible', "Suggestions opened"));
            // 	} else {
            // 		alert(nls.localize('suggestWidgetAriaInvisible', "Suggestions closed"));
            // 	}
            // });
        }
        Object.defineProperty(SuggestWidget.prototype, "onDidVisibilityChange", {
            get: function () { return this._onDidVisibilityChange.event; },
            enumerable: true,
            configurable: true
        });
        SuggestWidget.prototype.onCursorSelectionChanged = function () {
            if (this.state === State.Hidden) {
                return;
            }
            this.editor.layoutContentWidget(this);
        };
        SuggestWidget.prototype.onEditorBlur = function () {
            var _this = this;
            this.editorBlurTimeout = winjs_base_1.TPromise.timeout(150).then(function () {
                if (!_this.editor.isFocused()) {
                    _this.setState(State.Hidden);
                }
            });
        };
        SuggestWidget.prototype.onListSelection = function (e) {
            if (!e.elements.length) {
                return;
            }
            this.telemetryData.selectedIndex = 0;
            this.telemetryData.wasCancelled = false;
            this.telemetryData.selectedIndex = e.indexes[0];
            this.submitTelemetryData();
            var item = e.elements[0];
            var container = item.container;
            var overwriteBefore = (typeof item.suggestion.overwriteBefore === 'undefined') ? container.currentWord.length : item.suggestion.overwriteBefore;
            var overwriteAfter = (typeof item.suggestion.overwriteAfter === 'undefined') ? 0 : Math.max(0, item.suggestion.overwriteAfter);
            this.model.accept(item.suggestion, overwriteBefore, overwriteAfter);
            aria_1.alert(nls.localize(6, null, item.suggestion.label));
            this.editor.focus();
        };
        SuggestWidget.prototype._getSuggestionAriaAlertLabel = function (item) {
            if (item.suggestion.documentationLabel) {
                return nls.localize(7, null, item.suggestion.label);
            }
            else {
                return nls.localize(8, null, item.suggestion.label);
            }
        };
        SuggestWidget.prototype._ariaAlert = function (newAriaAlertLabel) {
            if (this._lastAriaAlertLabel === newAriaAlertLabel) {
                return;
            }
            this._lastAriaAlertLabel = newAriaAlertLabel;
            if (this._lastAriaAlertLabel) {
                aria_1.alert(this._lastAriaAlertLabel);
            }
        };
        SuggestWidget.prototype.onListFocus = function (e) {
            var _this = this;
            if (this.currentSuggestionDetails) {
                this.currentSuggestionDetails.cancel();
                this.currentSuggestionDetails = null;
            }
            if (!e.elements.length) {
                this._ariaAlert(null);
                // TODO@Alex: Chromium bug
                // this.editor.setAriaActiveDescendant(null);
                return;
            }
            var item = e.elements[0];
            this._ariaAlert(this._getSuggestionAriaAlertLabel(item));
            // TODO@Alex: Chromium bug
            // // TODO@Alex: the list is not done rendering...
            // setTimeout(() => {
            // 	this.editor.setAriaActiveDescendant(this.list.getElementId(e.indexes[0]));
            // }, 100);
            if (item === this.focusedItem) {
                return;
            }
            var index = e.indexes[0];
            this.suggestionSupportsAutoAccept.set(!item.suggestion.noAutoAccept);
            this.focusedItem = item;
            this.list.setFocus(index);
            this.updateWidgetHeight();
            this.list.reveal(index);
            var resource = this.editor.getModel().getAssociatedResource();
            var position = this.model.getRequestPosition() || this.editor.getPosition();
            this.currentSuggestionDetails = item.resolveDetails(resource, position)
                .then(function (details) {
                item.updateDetails(details);
                _this.list.setFocus(index);
                _this.updateWidgetHeight();
                _this.list.reveal(index);
                _this._ariaAlert(_this._getSuggestionAriaAlertLabel(item));
            })
                .then(null, function (err) { return !errors_1.isPromiseCanceledError(err) && errors_1.onUnexpectedError(err); })
                .then(function () { return _this.currentSuggestionDetails = null; });
        };
        SuggestWidget.prototype.onModelModeChanged = function () {
            var model = this.editor.getModel();
            var supports = suggest_1.SuggestRegistry.all(model);
            this.shouldShowEmptySuggestionList = supports.some(function (s) { return s.shouldShowEmptySuggestionList(); });
        };
        SuggestWidget.prototype.setState = function (state) {
            var stateChanged = this.state !== state;
            this.state = state;
            dom_1.toggleClass(this.element, 'frozen', state === State.Frozen);
            switch (state) {
                case State.Hidden:
                    dom_1.hide(this.messageElement, this.details.element);
                    dom_1.show(this.listElement);
                    this.hide();
                    if (stateChanged) {
                        this.list.splice(0, this.list.length);
                    }
                    break;
                case State.Loading:
                    this.messageElement.innerText = SuggestWidget.LOADING_MESSAGE;
                    dom_1.hide(this.listElement, this.details.element);
                    dom_1.show(this.messageElement);
                    this.show();
                    break;
                case State.Empty:
                    this.messageElement.innerText = SuggestWidget.NO_SUGGESTIONS_MESSAGE;
                    dom_1.hide(this.listElement, this.details.element);
                    dom_1.show(this.messageElement);
                    this.show();
                    break;
                case State.Open:
                    dom_1.hide(this.messageElement, this.details.element);
                    dom_1.show(this.listElement);
                    this.show();
                    break;
                case State.Frozen:
                    dom_1.hide(this.messageElement, this.details.element);
                    dom_1.show(this.listElement);
                    this.show();
                    break;
                case State.Details:
                    dom_1.hide(this.messageElement, this.listElement);
                    dom_1.show(this.details.element);
                    this.show();
                    this._ariaAlert(this.details.getAriaLabel());
                    break;
            }
            if (stateChanged) {
                this.editor.layoutContentWidget(this);
            }
        };
        SuggestWidget.prototype.onDidTrigger = function (e) {
            var _this = this;
            if (this.state !== State.Hidden) {
                return;
            }
            this.telemetryTimer = this.telemetryService.start('suggestWidgetLoadingTime');
            this.isAuto = !!e.auto;
            if (!this.isAuto) {
                this.loadingTimeout = setTimeout(function () {
                    _this.loadingTimeout = null;
                    _this.setState(State.Loading);
                }, 50);
            }
            if (!e.retrigger) {
                this.telemetryData = {
                    wasAutomaticallyTriggered: e.characterTriggered
                };
            }
        };
        SuggestWidget.prototype.onDidSuggest = function (e) {
            clearTimeout(this.loadingTimeout);
            this.completionModel = e.completionModel;
            if (e.isFrozen && this.state !== State.Empty) {
                this.setState(State.Frozen);
                return;
            }
            var visibleCount = this.completionModel.items.length;
            var isEmpty = visibleCount === 0;
            if (isEmpty) {
                if (e.auto) {
                    this.setState(State.Hidden);
                }
                else {
                    if (this.shouldShowEmptySuggestionList) {
                        this.setState(State.Empty);
                    }
                    else {
                        this.setState(State.Hidden);
                    }
                }
                this.completionModel = null;
            }
            else {
                var currentWord_1 = e.currentWord;
                var currentWordLowerCase_1 = currentWord_1.toLowerCase();
                var bestSuggestionIndex_1 = -1;
                var bestScore_1 = -1;
                this.completionModel.items.forEach(function (item, index) {
                    var score = computeScore(item.suggestion.label, currentWord_1, currentWordLowerCase_1);
                    if (score > bestScore_1) {
                        bestScore_1 = score;
                        bestSuggestionIndex_1 = index;
                    }
                });
                this.telemetryData = this.telemetryData || {};
                this.telemetryData.suggestionCount = this.completionModel.items.length;
                this.telemetryData.suggestedIndex = bestSuggestionIndex_1;
                this.telemetryData.hintLength = currentWord_1.length;
                (_a = this.list).splice.apply(_a, [0, this.list.length].concat(this.completionModel.items));
                this.list.setFocus(bestSuggestionIndex_1);
                this.list.reveal(bestSuggestionIndex_1, 0);
                this.setState(State.Open);
            }
            if (this.telemetryTimer) {
                this.telemetryTimer.data = { reason: isEmpty ? 'empty' : 'results' };
                this.telemetryTimer.stop();
                this.telemetryTimer = null;
            }
            var _a;
        };
        SuggestWidget.prototype.onDidCancel = function (e) {
            clearTimeout(this.loadingTimeout);
            if (!e.retrigger) {
                this.setState(State.Hidden);
                if (this.telemetryData) {
                    this.telemetryData.selectedIndex = -1;
                    this.telemetryData.wasCancelled = true;
                    this.submitTelemetryData();
                }
            }
            if (this.telemetryTimer) {
                this.telemetryTimer.data = { reason: 'cancel' };
                this.telemetryTimer.stop();
                this.telemetryTimer = null;
            }
        };
        SuggestWidget.prototype.selectNextPage = function () {
            switch (this.state) {
                case State.Hidden:
                    return false;
                case State.Details:
                    this.details.pageDown();
                    return true;
                case State.Loading:
                    return !this.isAuto;
                default:
                    this.list.focusNextPage();
                    return true;
            }
        };
        SuggestWidget.prototype.selectNext = function () {
            switch (this.state) {
                case State.Hidden:
                    return false;
                case State.Details:
                    this.details.scrollDown();
                    return true;
                case State.Loading:
                    return !this.isAuto;
                default:
                    this.list.focusNext(1, true);
                    return true;
            }
        };
        SuggestWidget.prototype.selectPreviousPage = function () {
            switch (this.state) {
                case State.Hidden:
                    return false;
                case State.Details:
                    this.details.pageUp();
                    return true;
                case State.Loading:
                    return !this.isAuto;
                default:
                    this.list.focusPreviousPage();
                    return true;
            }
        };
        SuggestWidget.prototype.selectPrevious = function () {
            switch (this.state) {
                case State.Hidden:
                    return false;
                case State.Details:
                    this.details.scrollUp();
                    return true;
                case State.Loading:
                    return !this.isAuto;
                default:
                    this.list.focusPrevious(1, true);
                    return true;
            }
        };
        SuggestWidget.prototype.acceptSelectedSuggestion = function () {
            switch (this.state) {
                case State.Hidden:
                    return false;
                case State.Loading:
                    return !this.isAuto;
                default:
                    var focus_1 = this.list.getFocus()[0];
                    if (focus_1) {
                        this.list.setSelection(this.completionModel.items.indexOf(focus_1));
                    }
                    else {
                        this.model.cancel();
                    }
                    return true;
            }
        };
        SuggestWidget.prototype.toggleDetails = function () {
            if (this.state === State.Details) {
                this.setState(State.Open);
                this.editor.focus();
                return;
            }
            if (this.state !== State.Open) {
                return;
            }
            var item = this.list.getFocus()[0];
            if (!item || !item.suggestion.documentationLabel) {
                return;
            }
            this.setState(State.Details);
            this.editor.focus();
        };
        SuggestWidget.prototype.show = function () {
            var _this = this;
            this.updateWidgetHeight();
            this._onDidVisibilityChange.fire(true);
            this.renderDetails();
            this.showTimeout = winjs_base_1.TPromise.timeout(100).then(function () {
                dom_1.addClass(_this.element, 'visible');
            });
        };
        SuggestWidget.prototype.hide = function () {
            this._onDidVisibilityChange.fire(false);
            dom_1.removeClass(this.element, 'visible');
        };
        SuggestWidget.prototype.cancel = function () {
            if (this.state === State.Details) {
                this.toggleDetails();
            }
            else {
                this.model.cancel();
            }
        };
        SuggestWidget.prototype.getPosition = function () {
            if (this.state === State.Hidden) {
                return null;
            }
            return {
                position: this.editor.getPosition(),
                preference: [editorBrowser_1.ContentWidgetPositionPreference.BELOW, editorBrowser_1.ContentWidgetPositionPreference.ABOVE]
            };
        };
        SuggestWidget.prototype.getDomNode = function () {
            return this.element;
        };
        SuggestWidget.prototype.getId = function () {
            return SuggestWidget.ID;
        };
        SuggestWidget.prototype.submitTelemetryData = function () {
            this.telemetryService.publicLog('suggestWidget', this.telemetryData);
            this.telemetryData = null;
        };
        SuggestWidget.prototype.updateWidgetHeight = function () {
            var height = 0;
            if (this.state === State.Empty || this.state === State.Loading) {
                height = UnfocusedHeight;
            }
            else if (this.state === State.Details) {
                height = 12 * UnfocusedHeight;
            }
            else {
                var focus_2 = this.list.getFocus()[0];
                var focusHeight = focus_2 ? this.delegate.getHeight(focus_2) : UnfocusedHeight;
                height = focusHeight;
                var suggestionCount = (this.list.contentHeight - focusHeight) / UnfocusedHeight;
                height += Math.min(suggestionCount, 11) * UnfocusedHeight;
            }
            this.element.style.height = height + 'px';
            this.list.layout(height);
            this.editor.layoutContentWidget(this);
            return height;
        };
        SuggestWidget.prototype.renderDetails = function () {
            if (this.state !== State.Details) {
                this.details.render(null);
            }
            else {
                this.details.render(this.list.getFocus()[0]);
            }
        };
        SuggestWidget.prototype.dispose = function () {
            this.state = null;
            this.suggestionSupportsAutoAccept = null;
            this.currentSuggestionDetails = null;
            this.focusedItem = null;
            this.telemetryData = null;
            this.telemetryService = null;
            this.telemetryTimer = null;
            this.element = null;
            this.messageElement = null;
            this.listElement = null;
            this.details.dispose();
            this.details = null;
            this.list.dispose();
            this.list = null;
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            this._onDidVisibilityChange.dispose();
            this._onDidVisibilityChange = null;
            clearTimeout(this.loadingTimeout);
            this.loadingTimeout = null;
            if (this.editorBlurTimeout) {
                this.editorBlurTimeout.cancel();
                this.editorBlurTimeout = null;
            }
            if (this.showTimeout) {
                this.showTimeout.cancel();
                this.showTimeout = null;
            }
        };
        SuggestWidget.ID = 'editor.widget.suggestWidget';
        SuggestWidget.WIDTH = 438;
        SuggestWidget.LOADING_MESSAGE = nls.localize(4, null);
        SuggestWidget.NO_SUGGESTIONS_MESSAGE = nls.localize(5, null);
        SuggestWidget = __decorate([
            __param(2, keybindingService_1.IKeybindingService),
            __param(3, telemetry_1.ITelemetryService),
            __param(4, instantiation_1.IInstantiationService)
        ], SuggestWidget);
        return SuggestWidget;
    }());
    exports.SuggestWidget = SuggestWidget;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/suggest/browser/suggest", ["require", "exports", 'vs/nls!vs/editor/contrib/suggest/browser/suggest', 'vs/base/common/errors', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/platform/instantiation/common/instantiation', 'vs/platform/keybinding/common/keybindingService', 'vs/editor/common/editorAction', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/browser/editorBrowserExtensions', 'vs/editor/contrib/snippet/common/snippet', 'vs/editor/contrib/suggest/common/suggest', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/editor/common/config/config', './suggestModel', './suggestWidget'], function (require, exports, nls, errors_1, keyCodes_1, lifecycle_1, winjs_base_1, instantiation_1, keybindingService_1, editorAction_1, editorCommon_1, editorCommonExtensions_1, editorBrowserExtensions_1, snippet_1, suggest_1, keybindingsRegistry_1, config_1, suggestModel_1, suggestWidget_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var SuggestController = (function () {
        function SuggestController(editor, keybindingService, instantiationService) {
            var _this = this;
            this.editor = editor;
            this.suggestWidgetVisible = keybindingService.createKey(suggest_1.CONTEXT_SUGGEST_WIDGET_VISIBLE, false);
            this.model = new suggestModel_1.SuggestModel(this.editor);
            this.widget = instantiationService.createInstance(suggestWidget_1.SuggestWidget, this.editor, this.model);
            this.triggerCharacterListeners = [];
            this.toDispose = [];
            this.toDispose.push(this.widget.onDidVisibilityChange(function (visible) { return visible ? _this.suggestWidgetVisible.set(true) : _this.suggestWidgetVisible.reset(); }));
            this.toDispose.push(editor.addListener2(editorCommon_1.EventType.ConfigurationChanged, function () { return _this.update(); }));
            this.toDispose.push(editor.addListener2(editorCommon_1.EventType.ModelChanged, function () { return _this.update(); }));
            this.toDispose.push(editor.addListener2(editorCommon_1.EventType.ModelModeChanged, function () { return _this.update(); }));
            this.toDispose.push(editor.addListener2(editorCommon_1.EventType.ModelModeSupportChanged, function (e) {
                if (e.suggestSupport) {
                    _this.update();
                }
            }));
            this.toDispose.push(suggest_1.SuggestRegistry.onDidChange(this.update, this));
            this.toDispose.push(this.model.onDidAccept(function (e) { return snippet_1.getSnippetController(_this.editor).run(e.snippet, e.overwriteBefore, e.overwriteAfter); }));
            this.update();
        }
        SuggestController.getSuggestController = function (editor) {
            return editor.getContribution(SuggestController.ID);
        };
        SuggestController.prototype.getId = function () {
            return SuggestController.ID;
        };
        SuggestController.prototype.dispose = function () {
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            this.triggerCharacterListeners = lifecycle_1.cAll(this.triggerCharacterListeners);
            if (this.widget) {
                this.widget.dispose();
                this.widget = null;
            }
            if (this.model) {
                this.model.dispose();
                this.model = null;
            }
        };
        SuggestController.prototype.update = function () {
            var _this = this;
            this.triggerCharacterListeners = lifecycle_1.cAll(this.triggerCharacterListeners);
            if (this.editor.getConfiguration().readOnly
                || !this.editor.getModel()
                || !this.editor.getConfiguration().suggestOnTriggerCharacters) {
                return;
            }
            var groups = suggest_1.SuggestRegistry.orderedGroups(this.editor.getModel());
            if (groups.length === 0) {
                return;
            }
            var triggerCharacters = Object.create(null);
            groups.forEach(function (group) {
                var groupTriggerCharacters = Object.create(null);
                group.forEach(function (support) {
                    var localTriggerCharacters = support.getTriggerCharacters();
                    if (localTriggerCharacters) {
                        for (var _i = 0, localTriggerCharacters_1 = localTriggerCharacters; _i < localTriggerCharacters_1.length; _i++) {
                            var ch = localTriggerCharacters_1[_i];
                            var array = groupTriggerCharacters[ch];
                            if (array) {
                                array.push(support);
                            }
                            else {
                                array = [support];
                                groupTriggerCharacters[ch] = array;
                                if (triggerCharacters[ch]) {
                                    triggerCharacters[ch].push(array);
                                }
                                else {
                                    triggerCharacters[ch] = [array];
                                }
                            }
                        }
                    }
                });
            });
            Object.keys(triggerCharacters).forEach(function (ch) {
                _this.triggerCharacterListeners.push(_this.editor.addTypingListener(ch, function () {
                    _this.triggerCharacterHandler(ch, triggerCharacters[ch]);
                }));
            });
        };
        SuggestController.prototype.triggerCharacterHandler = function (character, groups) {
            var position = this.editor.getPosition();
            var lineContext = this.editor.getModel().getLineContext(position.lineNumber);
            groups = groups.map(function (supports) {
                return supports.filter(function (support) { return support.shouldAutotriggerSuggest(lineContext, position.column - 1, character); });
            });
            if (groups.length > 0) {
                this.triggerSuggest(character, groups).done(null, errors_1.onUnexpectedError);
            }
        };
        SuggestController.prototype.triggerSuggest = function (triggerCharacter, groups) {
            this.model.trigger(false, triggerCharacter, false, groups);
            this.editor.focus();
            return winjs_base_1.TPromise.as(false);
        };
        SuggestController.prototype.acceptSelectedSuggestion = function () {
            if (this.widget) {
                this.widget.acceptSelectedSuggestion();
            }
        };
        SuggestController.prototype.cancelSuggestWidget = function () {
            if (this.widget) {
                this.widget.cancel();
            }
        };
        SuggestController.prototype.selectNextSuggestion = function () {
            if (this.widget) {
                this.widget.selectNext();
            }
        };
        SuggestController.prototype.selectNextPageSuggestion = function () {
            if (this.widget) {
                this.widget.selectNextPage();
            }
        };
        SuggestController.prototype.selectPrevSuggestion = function () {
            if (this.widget) {
                this.widget.selectPrevious();
            }
        };
        SuggestController.prototype.selectPrevPageSuggestion = function () {
            if (this.widget) {
                this.widget.selectPreviousPage();
            }
        };
        SuggestController.prototype.toggleSuggestionDetails = function () {
            if (this.widget) {
                this.widget.toggleDetails();
            }
        };
        SuggestController.ID = 'editor.contrib.suggestController';
        SuggestController = __decorate([
            __param(1, keybindingService_1.IKeybindingService),
            __param(2, instantiation_1.IInstantiationService)
        ], SuggestController);
        return SuggestController;
    }());
    exports.SuggestController = SuggestController;
    var TriggerSuggestAction = (function (_super) {
        __extends(TriggerSuggestAction, _super);
        function TriggerSuggestAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        TriggerSuggestAction.prototype.isSupported = function () {
            return suggest_1.SuggestRegistry.has(this.editor.getModel()) && !this.editor.getConfiguration().readOnly;
        };
        TriggerSuggestAction.prototype.run = function () {
            return SuggestController.getSuggestController(this.editor).triggerSuggest();
        };
        TriggerSuggestAction.ID = 'editor.action.triggerSuggest';
        return TriggerSuggestAction;
    }(editorAction_1.EditorAction));
    exports.TriggerSuggestAction = TriggerSuggestAction;
    var weight = editorCommonExtensions_1.CommonEditorRegistry.commandWeight(90);
    // register action
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(TriggerSuggestAction, TriggerSuggestAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.Space,
        mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.Space }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand(suggest_1.ACCEPT_SELECTED_SUGGESTION_CMD, weight, { primary: keyCodes_1.KeyCode.Tab }, true, suggest_1.CONTEXT_SUGGEST_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = SuggestController.getSuggestController(editor);
        controller.acceptSelectedSuggestion();
    });
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
        id: 'acceptSelectedSuggestionOnEnter',
        handler: function (accessor, args) {
            config_1.withCodeEditorFromCommandHandler('acceptSelectedSuggestionOnEnter', accessor, args, function (editor) {
                var controller = SuggestController.getSuggestController(editor);
                controller.acceptSelectedSuggestion();
            });
        },
        weight: weight,
        context: keybindingService_1.KbExpr.and(keybindingService_1.KbExpr.has(suggest_1.CONTEXT_SUGGEST_WIDGET_VISIBLE), keybindingService_1.KbExpr.has('config.editor.acceptSuggestionOnEnter')),
        primary: keyCodes_1.KeyCode.Enter,
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('hideSuggestWidget', weight, { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, true, suggest_1.CONTEXT_SUGGEST_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = SuggestController.getSuggestController(editor);
        controller.cancelSuggestWidget();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('selectNextSuggestion', weight, { primary: keyCodes_1.KeyCode.DownArrow, secondary: [keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.DownArrow] }, true, suggest_1.CONTEXT_SUGGEST_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = SuggestController.getSuggestController(editor);
        controller.selectNextSuggestion();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('selectNextPageSuggestion', weight, { primary: keyCodes_1.KeyCode.PageDown, secondary: [keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.PageDown] }, true, suggest_1.CONTEXT_SUGGEST_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = SuggestController.getSuggestController(editor);
        controller.selectNextPageSuggestion();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('selectPrevSuggestion', weight, { primary: keyCodes_1.KeyCode.UpArrow, secondary: [keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.UpArrow] }, true, suggest_1.CONTEXT_SUGGEST_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = SuggestController.getSuggestController(editor);
        controller.selectPrevSuggestion();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('selectPrevPageSuggestion', weight, { primary: keyCodes_1.KeyCode.PageUp, secondary: [keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.PageUp] }, true, suggest_1.CONTEXT_SUGGEST_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = SuggestController.getSuggestController(editor);
        controller.selectPrevPageSuggestion();
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('toggleSuggestionDetails', weight, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.Space, mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.Space } }, true, suggest_1.CONTEXT_SUGGEST_WIDGET_VISIBLE, function (ctx, editor, args) {
        var controller = SuggestController.getSuggestController(editor);
        controller.toggleSuggestionDetails();
    });
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(SuggestController);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/toggleTabFocusMode/common/toggleTabFocusMode", ["require", "exports", 'vs/nls!vs/editor/contrib/toggleTabFocusMode/common/toggleTabFocusMode', 'vs/base/common/keyCodes', 'vs/base/common/winjs.base', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommonExtensions'], function (require, exports, nls, keyCodes_1, winjs_base_1, editorAction_1, editorActionEnablement_1, editorCommonExtensions_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ToggleTabFocusModeAction = (function (_super) {
        __extends(ToggleTabFocusModeAction, _super);
        function ToggleTabFocusModeAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
        }
        ToggleTabFocusModeAction.prototype.run = function () {
            if (this.editor.getConfiguration().tabFocusMode) {
                this.editor.updateOptions({ tabFocusMode: false });
            }
            else {
                this.editor.updateOptions({ tabFocusMode: true });
            }
            return winjs_base_1.TPromise.as(true);
        };
        ToggleTabFocusModeAction.ID = 'editor.action.toggleTabFocusMode';
        return ToggleTabFocusModeAction;
    }(editorAction_1.EditorAction));
    exports.ToggleTabFocusModeAction = ToggleTabFocusModeAction;
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(ToggleTabFocusModeAction, ToggleTabFocusModeAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_M,
        mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_M }
    }));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/accessibility/browser/accessibility", ["require", "exports", 'vs/nls!vs/editor/contrib/accessibility/browser/accessibility', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/strings', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/base/browser/htmlContentRenderer', 'vs/base/browser/styleMutator', 'vs/base/browser/ui/widget', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/editor/common/config/commonEditorConfig', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/browser/editorBrowserExtensions', 'vs/editor/contrib/toggleTabFocusMode/common/toggleTabFocusMode', 'vs/css!./accessibility'], function (require, exports, nls, keyCodes_1, lifecycle_1, strings, winjs_base_1, dom_1, htmlContentRenderer_1, styleMutator_1, widget_1, keybindingService_1, keybindingsRegistry_1, commonEditorConfig_1, editorAction_1, editorActionEnablement_1, editorCommon_1, editorCommonExtensions_1, editorBrowserExtensions_1, toggleTabFocusMode_1) {
    'use strict';
    var NLS_SHOW_ACCESSIBILITY_HELP_ACTION_LABEL = nls.localize(0, null);
    var CONTEXT_ACCESSIBILITY_WIDGET_VISIBLE = 'accessibilityHelpWidgetVisible';
    var TOGGLE_EXPERIMENTAL_SCREEN_READER_SUPPORT_COMMAND_ID = 'toggleExperimentalScreenReaderSupport';
    var AccessibilityHelpController = (function (_super) {
        __extends(AccessibilityHelpController, _super);
        function AccessibilityHelpController(editor, keybindingService) {
            _super.call(this);
            this._editor = editor;
            this._widget = this._register(new AccessibilityHelpWidget(this._editor, keybindingService));
        }
        AccessibilityHelpController.get = function (editor) {
            return editor.getContribution(AccessibilityHelpController.ID);
        };
        AccessibilityHelpController.prototype.getId = function () {
            return AccessibilityHelpController.ID;
        };
        AccessibilityHelpController.prototype.show = function () {
            this._widget.show();
        };
        AccessibilityHelpController.prototype.hide = function () {
            this._widget.hide();
        };
        AccessibilityHelpController.ID = 'editor.contrib.accessibilityHelpController';
        AccessibilityHelpController = __decorate([
            __param(1, keybindingService_1.IKeybindingService)
        ], AccessibilityHelpController);
        return AccessibilityHelpController;
    }(lifecycle_1.Disposable));
    var AccessibilityHelpWidget = (function (_super) {
        __extends(AccessibilityHelpWidget, _super);
        function AccessibilityHelpWidget(editor, keybindingService) {
            var _this = this;
            _super.call(this);
            this._editor = editor;
            this._keybindingService = keybindingService;
            this._isVisibleKey = keybindingService.createKey(CONTEXT_ACCESSIBILITY_WIDGET_VISIBLE, false);
            this._domNode = document.createElement('div');
            this._domNode.className = 'accessibilityHelpWidget';
            styleMutator_1.StyleMutator.setWidth(this._domNode, AccessibilityHelpWidget.WIDTH);
            styleMutator_1.StyleMutator.setHeight(this._domNode, AccessibilityHelpWidget.HEIGHT);
            this._domNode.style.display = 'none';
            this._domNode.setAttribute('role', 'tooltip');
            this._domNode.setAttribute('aria-hidden', 'true');
            this._isVisible = false;
            this._register(this._editor.addListener2(editorCommon_1.EventType.EditorLayout, function () {
                if (_this._isVisible) {
                    _this._layout();
                }
            }));
            this.onblur(this._domNode, function () {
                _this.hide();
            });
            this._editor.addOverlayWidget(this);
        }
        AccessibilityHelpWidget.prototype.dispose = function () {
            this._editor.removeOverlayWidget(this);
            _super.prototype.dispose.call(this);
        };
        AccessibilityHelpWidget.prototype.getId = function () {
            return AccessibilityHelpWidget.ID;
        };
        AccessibilityHelpWidget.prototype.getDomNode = function () {
            return this._domNode;
        };
        AccessibilityHelpWidget.prototype.getPosition = function () {
            return {
                preference: null
            };
        };
        AccessibilityHelpWidget.prototype.show = function () {
            if (this._isVisible) {
                return;
            }
            this._isVisible = true;
            this._isVisibleKey.set(true);
            this._layout();
            this._domNode.style.display = 'block';
            this._domNode.setAttribute('aria-hidden', 'false');
            this._domNode.tabIndex = 0;
            this._buildContent();
            this._domNode.focus();
        };
        AccessibilityHelpWidget.prototype._descriptionForCommand = function (commandId, msg, noKbMsg) {
            var keybindings = this._keybindingService.lookupKeybindings(commandId);
            if (keybindings.length > 0) {
                return strings.format(msg, this._keybindingService.getAriaLabelFor(keybindings[0]));
            }
            return strings.format(noKbMsg, commandId);
        };
        AccessibilityHelpWidget.prototype._buildContent = function () {
            var opts = this._editor.getConfiguration();
            var text = nls.localize(1, null);
            text += '\n\n' + nls.localize(2, null);
            var NLS_TAB_FOCUS_MODE_ON = nls.localize(3, null);
            var NLS_TAB_FOCUS_MODE_ON_NO_KB = nls.localize(4, null);
            var NLS_TAB_FOCUS_MODE_OFF = nls.localize(5, null);
            var NLS_TAB_FOCUS_MODE_OFF_NO_KB = nls.localize(6, null);
            if (opts.tabFocusMode) {
                text += '\n\n -' + this._descriptionForCommand(toggleTabFocusMode_1.ToggleTabFocusModeAction.ID, NLS_TAB_FOCUS_MODE_ON, NLS_TAB_FOCUS_MODE_ON_NO_KB);
            }
            else {
                text += '\n\n -' + this._descriptionForCommand(toggleTabFocusMode_1.ToggleTabFocusModeAction.ID, NLS_TAB_FOCUS_MODE_OFF, NLS_TAB_FOCUS_MODE_OFF_NO_KB);
            }
            var NLS_EXPERIMENTAL_SCREENREADER_OPTS_ON = nls.localize(7, null);
            var NLS_EXPERIMENTAL_SCREENREADER_SESSION_ON = nls.localize(8, null);
            var NLS_EXPERIMENTAL_SCREENREADER_SESSION_ON_NO_KB = nls.localize(9, null);
            var NLS_EXPERIMENTAL_SCREENREADER_SESSION_OFF = nls.localize(10, null);
            var NLS_EXPERIMENTAL_SCREENREADER_SESSION_OFF_NO_KB = nls.localize(11, null);
            if (opts.experimentalScreenReader) {
                text += '\n\n - ' + NLS_EXPERIMENTAL_SCREENREADER_OPTS_ON;
            }
            else {
                if (commonEditorConfig_1.GlobalScreenReaderNVDA.getValue()) {
                    text += '\n\n -' + this._descriptionForCommand(TOGGLE_EXPERIMENTAL_SCREEN_READER_SUPPORT_COMMAND_ID, NLS_EXPERIMENTAL_SCREENREADER_SESSION_ON, NLS_EXPERIMENTAL_SCREENREADER_SESSION_ON_NO_KB);
                }
                else {
                    text += '\n\n -' + this._descriptionForCommand(TOGGLE_EXPERIMENTAL_SCREEN_READER_SUPPORT_COMMAND_ID, NLS_EXPERIMENTAL_SCREENREADER_SESSION_OFF, NLS_EXPERIMENTAL_SCREENREADER_SESSION_OFF_NO_KB);
                }
            }
            text += '\n\n' + nls.localize(12, null);
            this._domNode.appendChild(htmlContentRenderer_1.renderHtml({
                formattedText: text
            }));
        };
        AccessibilityHelpWidget.prototype.hide = function () {
            if (!this._isVisible) {
                return;
            }
            this._isVisible = false;
            this._isVisibleKey.reset();
            this._domNode.style.display = 'none';
            this._domNode.setAttribute('aria-hidden', 'true');
            this._domNode.tabIndex = -1;
            dom_1.clearNode(this._domNode);
            this._editor.focus();
        };
        AccessibilityHelpWidget.prototype._layout = function () {
            var editorLayout = this._editor.getLayoutInfo();
            var top = Math.round((editorLayout.height - AccessibilityHelpWidget.HEIGHT) / 2);
            styleMutator_1.StyleMutator.setTop(this._domNode, top);
            var left = Math.round((editorLayout.width - AccessibilityHelpWidget.WIDTH) / 2);
            styleMutator_1.StyleMutator.setLeft(this._domNode, left);
        };
        AccessibilityHelpWidget.ID = 'editor.contrib.accessibilityHelpWidget';
        AccessibilityHelpWidget.WIDTH = 500;
        AccessibilityHelpWidget.HEIGHT = 300;
        return AccessibilityHelpWidget;
    }(widget_1.Widget));
    var ShowAccessibilityHelpAction = (function (_super) {
        __extends(ShowAccessibilityHelpAction, _super);
        function ShowAccessibilityHelpAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus);
        }
        ShowAccessibilityHelpAction.prototype.run = function () {
            var controller = AccessibilityHelpController.get(this.editor);
            controller.show();
            return winjs_base_1.TPromise.as(true);
        };
        return ShowAccessibilityHelpAction;
    }(editorAction_1.EditorAction));
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(AccessibilityHelpController);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(ShowAccessibilityHelpAction, editorCommon_1.SHOW_ACCESSIBILITY_HELP_ACTION_ID, NLS_SHOW_ACCESSIBILITY_HELP_ACTION_LABEL, {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.F1
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('closeAccessibilityHelp', editorCommonExtensions_1.CommonEditorRegistry.commandWeight(100), { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, false, CONTEXT_ACCESSIBILITY_WIDGET_VISIBLE, function (ctx, editor, args) {
        AccessibilityHelpController.get(editor).hide();
    });
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
        id: TOGGLE_EXPERIMENTAL_SCREEN_READER_SUPPORT_COMMAND_ID,
        handler: function (accessor, args) {
            var currentValue = commonEditorConfig_1.GlobalScreenReaderNVDA.getValue();
            commonEditorConfig_1.GlobalScreenReaderNVDA.setValue(!currentValue);
        },
        weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.workbenchContrib(),
        context: null,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_R
    });
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/contrib/toggleWordWrap/common/toggleWordWrap", ["require", "exports", 'vs/nls!vs/editor/contrib/toggleWordWrap/common/toggleWordWrap', 'vs/base/common/keyCodes', 'vs/base/common/winjs.base', 'vs/editor/common/config/defaultConfig', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommonExtensions'], function (require, exports, nls, keyCodes_1, winjs_base_1, defaultConfig_1, editorAction_1, editorActionEnablement_1, editorCommonExtensions_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ToggleWordWrapAction = (function (_super) {
        __extends(ToggleWordWrapAction, _super);
        function ToggleWordWrapAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
        }
        ToggleWordWrapAction.prototype.run = function () {
            var wrappingInfo = this.editor.getConfiguration().wrappingInfo;
            if (!wrappingInfo.isViewportWrapping) {
                wrappingInfo.wrappingColumn = 0;
            }
            else {
                wrappingInfo.wrappingColumn = defaultConfig_1.DefaultConfig.editor.wrappingColumn;
            }
            this.editor.updateOptions(wrappingInfo);
            return winjs_base_1.TPromise.as(true);
        };
        ToggleWordWrapAction.ID = 'editor.action.toggleWordWrap';
        return ToggleWordWrapAction;
    }(editorAction_1.EditorAction));
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(ToggleWordWrapAction, ToggleWordWrapAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_Z,
        mac: { primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_Z },
        linux: { primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_Z }
    }));
});

define("vs/editor/contrib/wordHighlighter/common/wordHighlighter", ["require", "exports", 'vs/base/common/async', 'vs/base/common/errors', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/languageFeatureRegistry'], function (require, exports, async_1, errors_1, range_1, editorCommon, editorCommonExtensions_1, languageFeatureRegistry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.OccurrencesRegistry = new languageFeatureRegistry_1.default('occurrencesSupport');
    function getOccurrencesAtPosition(model, position) {
        var resource = model.getAssociatedResource();
        var orderedByScore = exports.OccurrencesRegistry.ordered(model);
        var foundResult = false;
        // in order of score ask the occurrences provider
        // until someone response with a good result
        // (good = none empty array)
        return async_1.sequence(orderedByScore.map(function (provider) {
            return function () {
                if (!foundResult) {
                    return provider.findOccurrences(resource, position).then(function (data) {
                        if (Array.isArray(data) && data.length > 0) {
                            foundResult = true;
                            return data;
                        }
                    }, function (err) {
                        errors_1.onUnexpectedError(err);
                    });
                }
            };
        })).then(function (values) {
            return values[0];
        });
    }
    exports.getOccurrencesAtPosition = getOccurrencesAtPosition;
    editorCommonExtensions_1.CommonEditorRegistry.registerDefaultLanguageCommand('_executeDocumentHighlights', getOccurrencesAtPosition);
    var WordHighlighter = (function () {
        function WordHighlighter(editor) {
            var _this = this;
            this.workerRequestTokenId = 0;
            this.workerRequest = null;
            this.workerRequestCompleted = false;
            this.workerRequestValue = [];
            this.lastCursorPositionChangeTime = 0;
            this.renderDecorationsTimer = -1;
            this.editor = editor;
            this.model = this.editor.getModel();
            this.toUnhook = [];
            this.toUnhook.push(editor.addListener(editorCommon.EventType.CursorPositionChanged, function (e) {
                _this._onPositionChanged(e);
            }));
            this.toUnhook.push(editor.addListener(editorCommon.EventType.ModelChanged, function (e) {
                _this._stopAll();
                _this.model = _this.editor.getModel();
            }));
            this.toUnhook.push(editor.addListener('change', function (e) {
                _this._stopAll();
            }));
            this._lastWordRange = null;
            this._decorationIds = [];
            this.workerRequestTokenId = 0;
            this.workerRequest = null;
            this.workerRequestCompleted = false;
            this.lastCursorPositionChangeTime = 0;
            this.renderDecorationsTimer = -1;
        }
        WordHighlighter.prototype._removeDecorations = function () {
            if (this._decorationIds.length > 0) {
                // remove decorations
                this._decorationIds = this.editor.deltaDecorations(this._decorationIds, []);
            }
        };
        WordHighlighter.prototype._stopAll = function () {
            this._lastWordRange = null;
            // Remove any existing decorations
            this._removeDecorations();
            // Cancel any renderDecorationsTimer
            if (this.renderDecorationsTimer !== -1) {
                window.clearTimeout(this.renderDecorationsTimer);
                this.renderDecorationsTimer = -1;
            }
            // Cancel any worker request
            if (this.workerRequest !== null) {
                this.workerRequest.cancel();
                this.workerRequest = null;
            }
            // Invalidate any worker request callback
            if (!this.workerRequestCompleted) {
                this.workerRequestTokenId++;
                this.workerRequestCompleted = true;
            }
        };
        WordHighlighter.prototype._onPositionChanged = function (e) {
            var _this = this;
            // ignore typing & other
            if (e.reason !== 'explicit') {
                this._stopAll();
                return;
            }
            // no providers for this model
            if (!exports.OccurrencesRegistry.has(this.model)) {
                this._stopAll();
                return;
            }
            var editorSelection = this.editor.getSelection();
            // ignore multiline selection
            if (editorSelection.startLineNumber !== editorSelection.endLineNumber) {
                this._stopAll();
                return;
            }
            var lineNumber = editorSelection.startLineNumber;
            var startColumn = editorSelection.startColumn;
            var endColumn = editorSelection.endColumn;
            var word = this.model.getWordAtPosition({
                lineNumber: lineNumber,
                column: startColumn
            });
            // The selection must be inside a word or surround one word at most
            if (!word || word.startColumn > startColumn || word.endColumn < endColumn) {
                this._stopAll();
                return;
            }
            // All the effort below is trying to achieve this:
            // - when cursor is moved to a word, trigger immediately a findOccurences request
            // - 250ms later after the last cursor move event, render the occurences
            // - no flickering!
            var currentWordRange = new range_1.Range(lineNumber, word.startColumn, lineNumber, word.endColumn);
            var workerRequestIsValid = this._lastWordRange && this._lastWordRange.equalsRange(currentWordRange);
            // Even if we are on a different word, if that word is in the decorations ranges, the request is still valid
            // (Same symbol)
            for (var i = 0, len = this._decorationIds.length; !workerRequestIsValid && i < len; i++) {
                var range = this.model.getDecorationRange(this._decorationIds[i]);
                if (range && range.startLineNumber === lineNumber) {
                    if (range.startColumn <= startColumn && range.endColumn >= endColumn) {
                        workerRequestIsValid = true;
                    }
                }
            }
            // There are 4 cases:
            // a) old workerRequest is valid & completed, renderDecorationsTimer fired
            // b) old workerRequest is valid & completed, renderDecorationsTimer not fired
            // c) old workerRequest is valid, but not completed
            // d) old workerRequest is not valid
            // For a) no action is needed
            // For c), member 'lastCursorPositionChangeTime' will be used when installing the timer so no action is needed
            this.lastCursorPositionChangeTime = (new Date()).getTime();
            if (workerRequestIsValid) {
                if (this.workerRequestCompleted && this.renderDecorationsTimer !== -1) {
                    // case b)
                    // Delay the firing of renderDecorationsTimer by an extra 250 ms
                    window.clearTimeout(this.renderDecorationsTimer);
                    this.renderDecorationsTimer = -1;
                    this._beginRenderDecorations();
                }
            }
            else {
                // case d)
                // Stop all previous actions and start fresh
                this._stopAll();
                var myRequestId = ++this.workerRequestTokenId;
                this.workerRequestCompleted = false;
                this.workerRequest = getOccurrencesAtPosition(this.model, this.editor.getPosition());
                this.workerRequest.then(function (data) {
                    if (myRequestId === _this.workerRequestTokenId) {
                        _this.workerRequestCompleted = true;
                        _this.workerRequestValue = data || [];
                        _this._beginRenderDecorations();
                    }
                }).done();
            }
            this._lastWordRange = currentWordRange;
        };
        WordHighlighter.prototype._beginRenderDecorations = function () {
            var _this = this;
            var currentTime = (new Date()).getTime();
            var minimumRenderTime = this.lastCursorPositionChangeTime + 250;
            if (currentTime >= minimumRenderTime) {
                // Synchronous
                this.renderDecorationsTimer = -1;
                this.renderDecorations();
            }
            else {
                // Asyncrhonous
                this.renderDecorationsTimer = window.setTimeout(function () {
                    _this.renderDecorations();
                }, (minimumRenderTime - currentTime));
            }
        };
        WordHighlighter.prototype.renderDecorations = function () {
            this.renderDecorationsTimer = -1;
            var decorations = [];
            for (var i = 0, len = this.workerRequestValue.length; i < len; i++) {
                var info = this.workerRequestValue[i];
                var className = 'wordHighlight';
                var color = '#A0A0A0';
                if (info.kind === 'write') {
                    className = className + 'Strong';
                }
                else if (info.kind === 'text') {
                    className = 'selectionHighlight';
                }
                decorations.push({
                    range: info.range,
                    options: {
                        stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
                        className: className,
                        overviewRuler: {
                            color: color,
                            darkColor: color,
                            position: editorCommon.OverviewRulerLane.Center
                        }
                    }
                });
            }
            this._decorationIds = this.editor.deltaDecorations(this._decorationIds, decorations);
        };
        WordHighlighter.prototype.destroy = function () {
            this._stopAll();
            while (this.toUnhook.length > 0) {
                this.toUnhook.pop()();
            }
        };
        return WordHighlighter;
    }());
    var WordHighlighterContribution = (function () {
        function WordHighlighterContribution(editor) {
            this.wordHighligher = new WordHighlighter(editor);
        }
        WordHighlighterContribution.prototype.getId = function () {
            return WordHighlighterContribution.ID;
        };
        WordHighlighterContribution.prototype.dispose = function () {
            this.wordHighligher.destroy();
        };
        WordHighlighterContribution.ID = 'editor.contrib.wordHighlighter';
        return WordHighlighterContribution;
    }());
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorContribution(WordHighlighterContribution);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/find/common/findController", ["require", "exports", 'vs/nls!vs/editor/contrib/find/common/findController', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/platform/keybinding/common/keybindingService', 'vs/editor/common/core/range', 'vs/editor/common/core/selection', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/contrib/find/common/findModel', 'vs/editor/contrib/find/common/findState', 'vs/editor/contrib/wordHighlighter/common/wordHighlighter'], function (require, exports, nls, keyCodes_1, lifecycle_1, winjs_base_1, keybindingService_1, range_1, selection_1, editorAction_1, editorActionEnablement_1, editorCommon, editorCommonExtensions_1, findModel_1, findState_1, wordHighlighter_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (FindStartFocusAction) {
        FindStartFocusAction[FindStartFocusAction["NoFocusChange"] = 0] = "NoFocusChange";
        FindStartFocusAction[FindStartFocusAction["FocusFindInput"] = 1] = "FocusFindInput";
        FindStartFocusAction[FindStartFocusAction["FocusReplaceInput"] = 2] = "FocusReplaceInput";
    })(exports.FindStartFocusAction || (exports.FindStartFocusAction = {}));
    var FindStartFocusAction = exports.FindStartFocusAction;
    var CONTEXT_FIND_WIDGET_VISIBLE = 'findWidgetVisible';
    var CommonFindController = (function (_super) {
        __extends(CommonFindController, _super);
        function CommonFindController(editor, keybindingService) {
            var _this = this;
            _super.call(this);
            this._editor = editor;
            this._findWidgetVisible = keybindingService.createKey(CONTEXT_FIND_WIDGET_VISIBLE, false);
            this._state = this._register(new findState_1.FindReplaceState());
            this._register(this._state.addChangeListener(function (e) { return _this._onStateChanged(e); }));
            this._model = null;
            this._register(this._editor.addListener2(editorCommon.EventType.ModelChanged, function () {
                var shouldRestartFind = (_this._editor.getModel() && _this._state.isRevealed);
                _this.disposeModel();
                if (shouldRestartFind) {
                    _this._start({
                        forceRevealReplace: false,
                        seedSearchStringFromSelection: false,
                        seedSearchScopeFromSelection: false,
                        shouldFocus: FindStartFocusAction.NoFocusChange,
                        shouldAnimate: false
                    });
                }
            }));
        }
        CommonFindController.getFindController = function (editor) {
            return editor.getContribution(CommonFindController.ID);
        };
        CommonFindController.prototype.dispose = function () {
            this.disposeModel();
            _super.prototype.dispose.call(this);
        };
        CommonFindController.prototype.disposeModel = function () {
            if (this._model) {
                this._model.dispose();
                this._model = null;
            }
        };
        CommonFindController.prototype.getId = function () {
            return CommonFindController.ID;
        };
        CommonFindController.prototype._onStateChanged = function (e) {
            if (e.isRevealed) {
                if (this._state.isRevealed) {
                    this._findWidgetVisible.set(true);
                }
                else {
                    this._findWidgetVisible.reset();
                    this.disposeModel();
                }
            }
        };
        CommonFindController.prototype.getState = function () {
            return this._state;
        };
        CommonFindController.prototype.closeFindWidget = function () {
            this._state.change({ isRevealed: false }, false);
            this._editor.focus();
        };
        CommonFindController.prototype.toggleCaseSensitive = function () {
            this._state.change({ matchCase: !this._state.matchCase }, false);
        };
        CommonFindController.prototype.toggleWholeWords = function () {
            this._state.change({ wholeWord: !this._state.wholeWord }, false);
        };
        CommonFindController.prototype.toggleRegex = function () {
            this._state.change({ isRegex: !this._state.isRegex }, false);
        };
        CommonFindController.prototype.setSearchString = function (searchString) {
            this._state.change({ searchString: searchString }, false);
        };
        CommonFindController.prototype.getSelectionSearchString = function () {
            var selection = this._editor.getSelection();
            if (selection.startLineNumber === selection.endLineNumber) {
                if (selection.isEmpty()) {
                    var wordAtPosition = this._editor.getModel().getWordAtPosition(selection.getStartPosition());
                    if (wordAtPosition) {
                        return wordAtPosition.word;
                    }
                }
                else {
                    return this._editor.getModel().getValueInRange(selection);
                }
            }
            return null;
        };
        CommonFindController.prototype._start = function (opts) {
            this.disposeModel();
            if (!this._editor.getModel()) {
                // cannot do anything with an editor that doesn't have a model...
                return;
            }
            var stateChanges = {
                isRevealed: true
            };
            // Consider editor selection and overwrite the state with it
            if (opts.seedSearchStringFromSelection) {
                var selectionSearchString = this.getSelectionSearchString();
                if (selectionSearchString) {
                    stateChanges.searchString = selectionSearchString;
                }
            }
            var selection = this._editor.getSelection();
            stateChanges.searchScope = null;
            if (opts.seedSearchScopeFromSelection && selection.startLineNumber < selection.endLineNumber) {
                // Take search scope into account only if it is more than one line.
                stateChanges.searchScope = selection;
            }
            // Overwrite isReplaceRevealed
            if (opts.forceRevealReplace) {
                stateChanges.isReplaceRevealed = true;
            }
            this._state.change(stateChanges, false);
            if (!this._model) {
                this._model = new findModel_1.FindModelBoundToEditorModel(this._editor, this._state);
            }
        };
        CommonFindController.prototype.start = function (opts) {
            this._start(opts);
        };
        CommonFindController.prototype.moveToNextMatch = function () {
            if (this._model) {
                this._model.moveToNextMatch();
                return true;
            }
            return false;
        };
        CommonFindController.prototype.moveToPrevMatch = function () {
            if (this._model) {
                this._model.moveToPrevMatch();
                return true;
            }
            return false;
        };
        CommonFindController.prototype.replace = function () {
            if (this._model) {
                this._model.replace();
                return true;
            }
            return false;
        };
        CommonFindController.prototype.replaceAll = function () {
            if (this._model) {
                this._model.replaceAll();
                return true;
            }
            return false;
        };
        CommonFindController.ID = 'editor.contrib.findController';
        CommonFindController = __decorate([
            __param(1, keybindingService_1.IKeybindingService)
        ], CommonFindController);
        return CommonFindController;
    }(lifecycle_1.Disposable));
    exports.CommonFindController = CommonFindController;
    var StartFindAction = (function (_super) {
        __extends(StartFindAction, _super);
        function StartFindAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus);
        }
        StartFindAction.prototype.run = function () {
            var controller = CommonFindController.getFindController(this.editor);
            controller.start({
                forceRevealReplace: false,
                seedSearchStringFromSelection: true,
                seedSearchScopeFromSelection: true,
                shouldFocus: FindStartFocusAction.FocusFindInput,
                shouldAnimate: true
            });
            return winjs_base_1.TPromise.as(true);
        };
        return StartFindAction;
    }(editorAction_1.EditorAction));
    exports.StartFindAction = StartFindAction;
    var MatchFindAction = (function (_super) {
        __extends(MatchFindAction, _super);
        function MatchFindAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus);
        }
        MatchFindAction.prototype.run = function () {
            var controller = CommonFindController.getFindController(this.editor);
            if (!this._run(controller)) {
                controller.start({
                    forceRevealReplace: false,
                    seedSearchStringFromSelection: (controller.getState().searchString.length === 0),
                    seedSearchScopeFromSelection: false,
                    shouldFocus: FindStartFocusAction.NoFocusChange,
                    shouldAnimate: true
                });
                this._run(controller);
            }
            return winjs_base_1.TPromise.as(true);
        };
        return MatchFindAction;
    }(editorAction_1.EditorAction));
    exports.MatchFindAction = MatchFindAction;
    var NextMatchFindAction = (function (_super) {
        __extends(NextMatchFindAction, _super);
        function NextMatchFindAction() {
            _super.apply(this, arguments);
        }
        NextMatchFindAction.prototype._run = function (controller) {
            return controller.moveToNextMatch();
        };
        return NextMatchFindAction;
    }(MatchFindAction));
    exports.NextMatchFindAction = NextMatchFindAction;
    var PreviousMatchFindAction = (function (_super) {
        __extends(PreviousMatchFindAction, _super);
        function PreviousMatchFindAction() {
            _super.apply(this, arguments);
        }
        PreviousMatchFindAction.prototype._run = function (controller) {
            return controller.moveToPrevMatch();
        };
        return PreviousMatchFindAction;
    }(MatchFindAction));
    exports.PreviousMatchFindAction = PreviousMatchFindAction;
    var SelectionMatchFindAction = (function (_super) {
        __extends(SelectionMatchFindAction, _super);
        function SelectionMatchFindAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus);
        }
        SelectionMatchFindAction.prototype.run = function () {
            var controller = CommonFindController.getFindController(this.editor);
            var selectionSearchString = controller.getSelectionSearchString();
            if (selectionSearchString) {
                controller.setSearchString(selectionSearchString);
            }
            if (!this._run(controller)) {
                controller.start({
                    forceRevealReplace: false,
                    seedSearchStringFromSelection: false,
                    seedSearchScopeFromSelection: false,
                    shouldFocus: FindStartFocusAction.NoFocusChange,
                    shouldAnimate: true
                });
                this._run(controller);
            }
            return winjs_base_1.TPromise.as(true);
        };
        return SelectionMatchFindAction;
    }(editorAction_1.EditorAction));
    exports.SelectionMatchFindAction = SelectionMatchFindAction;
    var NextSelectionMatchFindAction = (function (_super) {
        __extends(NextSelectionMatchFindAction, _super);
        function NextSelectionMatchFindAction() {
            _super.apply(this, arguments);
        }
        NextSelectionMatchFindAction.prototype._run = function (controller) {
            return controller.moveToNextMatch();
        };
        return NextSelectionMatchFindAction;
    }(SelectionMatchFindAction));
    exports.NextSelectionMatchFindAction = NextSelectionMatchFindAction;
    var PreviousSelectionMatchFindAction = (function (_super) {
        __extends(PreviousSelectionMatchFindAction, _super);
        function PreviousSelectionMatchFindAction() {
            _super.apply(this, arguments);
        }
        PreviousSelectionMatchFindAction.prototype._run = function (controller) {
            return controller.moveToPrevMatch();
        };
        return PreviousSelectionMatchFindAction;
    }(SelectionMatchFindAction));
    exports.PreviousSelectionMatchFindAction = PreviousSelectionMatchFindAction;
    var StartFindReplaceAction = (function (_super) {
        __extends(StartFindReplaceAction, _super);
        function StartFindReplaceAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.Writeable);
        }
        StartFindReplaceAction.prototype.run = function () {
            var controller = CommonFindController.getFindController(this.editor);
            controller.start({
                forceRevealReplace: true,
                seedSearchStringFromSelection: true,
                seedSearchScopeFromSelection: true,
                shouldFocus: FindStartFocusAction.FocusReplaceInput,
                shouldAnimate: true
            });
            return winjs_base_1.TPromise.as(true);
        };
        return StartFindReplaceAction;
    }(editorAction_1.EditorAction));
    exports.StartFindReplaceAction = StartFindReplaceAction;
    function multiCursorFind(editor, changeFindSearchString) {
        var controller = CommonFindController.getFindController(editor);
        var state = controller.getState();
        var searchText, nextMatch;
        // In any case, if the find widget was ever opened, the options are taken from it
        var wholeWord = state.wholeWord;
        var matchCase = state.matchCase;
        // Find widget owns what we search for if:
        //  - focus is not in the editor (i.e. it is in the find widget)
        //  - and the search widget is visible
        //  - and the search string is non-empty
        if (!editor.isFocused() && state.isRevealed && state.searchString.length > 0) {
            // Find widget owns what is searched for
            searchText = state.searchString;
        }
        else {
            // Selection owns what is searched for
            var s = editor.getSelection();
            if (s.startLineNumber !== s.endLineNumber) {
                // Cannot search for multiline string... yet...
                return null;
            }
            if (s.isEmpty()) {
                // selection is empty => expand to current word
                var word = editor.getModel().getWordAtPosition(s.getStartPosition());
                if (!word) {
                    return null;
                }
                searchText = word.word;
                nextMatch = selection_1.Selection.createSelection(s.startLineNumber, word.startColumn, s.startLineNumber, word.endColumn);
            }
            else {
                searchText = editor.getModel().getValueInRange(s);
            }
            if (changeFindSearchString) {
                controller.setSearchString(searchText);
            }
        }
        return {
            searchText: searchText,
            matchCase: matchCase,
            wholeWord: wholeWord,
            nextMatch: nextMatch
        };
    }
    var SelectNextFindMatchAction = (function (_super) {
        __extends(SelectNextFindMatchAction, _super);
        function SelectNextFindMatchAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus);
        }
        SelectNextFindMatchAction.prototype._getNextMatch = function () {
            var r = multiCursorFind(this.editor, true);
            if (!r) {
                return null;
            }
            if (r.nextMatch) {
                return r.nextMatch;
            }
            var allSelections = this.editor.getSelections();
            var lastAddedSelection = allSelections[allSelections.length - 1];
            var nextMatch = this.editor.getModel().findNextMatch(r.searchText, lastAddedSelection.getEndPosition(), false, r.matchCase, r.wholeWord);
            if (!nextMatch) {
                return null;
            }
            return selection_1.Selection.createSelection(nextMatch.startLineNumber, nextMatch.startColumn, nextMatch.endLineNumber, nextMatch.endColumn);
        };
        return SelectNextFindMatchAction;
    }(editorAction_1.EditorAction));
    exports.SelectNextFindMatchAction = SelectNextFindMatchAction;
    var AddSelectionToNextFindMatchAction = (function (_super) {
        __extends(AddSelectionToNextFindMatchAction, _super);
        function AddSelectionToNextFindMatchAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        AddSelectionToNextFindMatchAction.prototype.run = function () {
            var nextMatch = this._getNextMatch();
            if (!nextMatch) {
                return winjs_base_1.TPromise.as(false);
            }
            var allSelections = this.editor.getSelections();
            this.editor.setSelections(allSelections.concat(nextMatch));
            this.editor.revealRangeInCenterIfOutsideViewport(nextMatch);
            return winjs_base_1.TPromise.as(true);
        };
        AddSelectionToNextFindMatchAction.ID = findModel_1.FIND_IDS.AddSelectionToNextFindMatchAction;
        return AddSelectionToNextFindMatchAction;
    }(SelectNextFindMatchAction));
    exports.AddSelectionToNextFindMatchAction = AddSelectionToNextFindMatchAction;
    var MoveSelectionToNextFindMatchAction = (function (_super) {
        __extends(MoveSelectionToNextFindMatchAction, _super);
        function MoveSelectionToNextFindMatchAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        MoveSelectionToNextFindMatchAction.prototype.run = function () {
            var nextMatch = this._getNextMatch();
            if (!nextMatch) {
                return winjs_base_1.TPromise.as(false);
            }
            var allSelections = this.editor.getSelections();
            this.editor.setSelections(allSelections.slice(0, allSelections.length - 1).concat(nextMatch));
            this.editor.revealRangeInCenterIfOutsideViewport(nextMatch);
            return winjs_base_1.TPromise.as(true);
        };
        MoveSelectionToNextFindMatchAction.ID = findModel_1.FIND_IDS.MoveSelectionToNextFindMatchAction;
        return MoveSelectionToNextFindMatchAction;
    }(SelectNextFindMatchAction));
    exports.MoveSelectionToNextFindMatchAction = MoveSelectionToNextFindMatchAction;
    var SelectHighlightsAction = (function (_super) {
        __extends(SelectHighlightsAction, _super);
        function SelectHighlightsAction(descriptor, editor) {
            var behaviour = editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.Writeable;
            if (descriptor.id === SelectHighlightsAction.COMPAT_ID) {
                behaviour |= editorActionEnablement_1.Behaviour.ShowInContextMenu;
            }
            _super.call(this, descriptor, editor, behaviour);
        }
        SelectHighlightsAction.prototype.getGroupId = function () {
            return '2_change/1_changeAll';
        };
        SelectHighlightsAction.prototype.run = function () {
            var r = multiCursorFind(this.editor, true);
            if (!r) {
                return winjs_base_1.TPromise.as(false);
            }
            var matches = this.editor.getModel().findMatches(r.searchText, true, false, r.matchCase, r.wholeWord);
            if (matches.length > 0) {
                this.editor.setSelections(matches.map(function (m) { return selection_1.Selection.createSelection(m.startLineNumber, m.startColumn, m.endLineNumber, m.endColumn); }));
            }
            return winjs_base_1.TPromise.as(true);
        };
        SelectHighlightsAction.ID = 'editor.action.selectHighlights';
        SelectHighlightsAction.COMPAT_ID = 'editor.action.changeAll';
        return SelectHighlightsAction;
    }(editorAction_1.EditorAction));
    exports.SelectHighlightsAction = SelectHighlightsAction;
    var SelectionHighlighter = (function (_super) {
        __extends(SelectionHighlighter, _super);
        function SelectionHighlighter(editor) {
            var _this = this;
            _super.call(this);
            this.editor = editor;
            this.decorations = [];
            this._register(editor.addListener2(editorCommon.EventType.CursorSelectionChanged, function (e) {
                if (e.selection.isEmpty()) {
                    if (e.reason === 'explicit') {
                        _this._update();
                    }
                    else {
                        _this.removeDecorations();
                    }
                }
                else {
                    _this._update();
                }
            }));
            this._register(editor.addListener2(editorCommon.EventType.ModelChanged, function (e) {
                _this.removeDecorations();
            }));
            this._register(CommonFindController.getFindController(editor).getState().addChangeListener(function (e) {
                _this._update();
            }));
        }
        SelectionHighlighter.prototype.getId = function () {
            return SelectionHighlighter.ID;
        };
        SelectionHighlighter.prototype.removeDecorations = function () {
            if (this.decorations.length > 0) {
                this.decorations = this.editor.deltaDecorations(this.decorations, []);
            }
        };
        SelectionHighlighter.prototype._update = function () {
            if (!this.editor.getConfiguration().selectionHighlight) {
                return;
            }
            var r = multiCursorFind(this.editor, false);
            if (!r) {
                this.removeDecorations();
                return;
            }
            var model = this.editor.getModel();
            var hasFindOccurences = wordHighlighter_1.OccurrencesRegistry.has(model);
            if (r.nextMatch) {
                // This is an empty selection
                if (hasFindOccurences) {
                    // Do not interfere with semantic word highlighting in the no selection case
                    this.removeDecorations();
                    return;
                }
            }
            if (/^[ \t]+$/.test(r.searchText)) {
                // whitespace only selection
                this.removeDecorations();
                return;
            }
            if (r.searchText.length > 200) {
                // very long selection
                this.removeDecorations();
                return;
            }
            var selections = this.editor.getSelections();
            var firstSelectedText = model.getValueInRange(selections[0]);
            for (var i = 1; i < selections.length; i++) {
                var selectedText = model.getValueInRange(selections[i]);
                if (firstSelectedText !== selectedText) {
                    // not all selections have the same text
                    this.removeDecorations();
                    return;
                }
            }
            var allMatches = model.findMatches(r.searchText, true, false, r.matchCase, r.wholeWord);
            allMatches.sort(range_1.Range.compareRangesUsingStarts);
            selections.sort(range_1.Range.compareRangesUsingStarts);
            // do not overlap with selection (issue #64 and #512)
            var matches = [];
            for (var i = 0, j = 0, len = allMatches.length, lenJ = selections.length; i < len;) {
                var match = allMatches[i];
                if (j >= lenJ) {
                    // finished all editor selections
                    matches.push(match);
                    i++;
                }
                else {
                    var cmp = range_1.Range.compareRangesUsingStarts(match, selections[j]);
                    if (cmp < 0) {
                        // match is before sel
                        matches.push(match);
                        i++;
                    }
                    else if (cmp > 0) {
                        // sel is before match
                        j++;
                    }
                    else {
                        // sel is equal to match
                        i++;
                        j++;
                    }
                }
            }
            var decorations = matches.map(function (r) {
                return {
                    range: r,
                    options: {
                        stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
                        className: 'selectionHighlight',
                        // Show in overviewRuler only if model has no semantic highlighting
                        overviewRuler: (hasFindOccurences ? undefined : {
                            color: '#A0A0A0',
                            darkColor: '#A0A0A0',
                            position: editorCommon.OverviewRulerLane.Center
                        })
                    }
                };
            });
            this.decorations = this.editor.deltaDecorations(this.decorations, decorations);
        };
        SelectionHighlighter.prototype.dispose = function () {
            this.removeDecorations();
            _super.prototype.dispose.call(this);
        };
        SelectionHighlighter.ID = 'editor.contrib.selectionHighlighter';
        return SelectionHighlighter;
    }(lifecycle_1.Disposable));
    exports.SelectionHighlighter = SelectionHighlighter;
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(SelectHighlightsAction, SelectHighlightsAction.ID, nls.localize(0, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_L
    }));
    // register SelectHighlightsAction again to replace the now removed Change All action
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(SelectHighlightsAction, SelectHighlightsAction.COMPAT_ID, nls.localize(1, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.F2
    }));
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(StartFindAction, findModel_1.FIND_IDS.StartFindAction, nls.localize(2, null), {
        context: editorCommonExtensions_1.ContextKey.None,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_F
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(NextMatchFindAction, findModel_1.FIND_IDS.NextMatchFindAction, nls.localize(3, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyCode.F3,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_G, secondary: [keyCodes_1.KeyCode.F3] }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(PreviousMatchFindAction, findModel_1.FIND_IDS.PreviousMatchFindAction, nls.localize(4, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.F3,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_G, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.F3] }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(NextSelectionMatchFindAction, findModel_1.FIND_IDS.NextSelectionMatchFindAction, nls.localize(5, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.F3
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(PreviousSelectionMatchFindAction, findModel_1.FIND_IDS.PreviousSelectionMatchFindAction, nls.localize(6, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.F3
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(StartFindReplaceAction, findModel_1.FIND_IDS.StartFindReplaceAction, nls.localize(7, null), {
        context: editorCommonExtensions_1.ContextKey.None,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_H,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_F }
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(MoveSelectionToNextFindMatchAction, MoveSelectionToNextFindMatchAction.ID, nls.localize(8, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_D)
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(AddSelectionToNextFindMatchAction, AddSelectionToNextFindMatchAction.ID, nls.localize(9, null), {
        context: editorCommonExtensions_1.ContextKey.EditorFocus,
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_D
    }));
    function registerFindCommand(id, callback, keybindings, needsKey) {
        if (needsKey === void 0) { needsKey = null; }
        editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand(id, editorCommonExtensions_1.CommonEditorRegistry.commandWeight(5), keybindings, false, needsKey, function (ctx, editor, args) {
            callback(CommonFindController.getFindController(editor));
        });
    }
    registerFindCommand(findModel_1.FIND_IDS.CloseFindWidgetCommand, function (x) { return x.closeFindWidget(); }, {
        primary: keyCodes_1.KeyCode.Escape,
        secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape]
    }, CONTEXT_FIND_WIDGET_VISIBLE);
    registerFindCommand(findModel_1.FIND_IDS.ToggleCaseSensitiveCommand, function (x) { return x.toggleCaseSensitive(); }, {
        primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_C,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_C }
    });
    registerFindCommand(findModel_1.FIND_IDS.ToggleWholeWordCommand, function (x) { return x.toggleWholeWords(); }, {
        primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_W,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_W }
    });
    registerFindCommand(findModel_1.FIND_IDS.ToggleRegexCommand, function (x) { return x.toggleRegex(); }, {
        primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_R,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.KEY_R }
    });
    registerFindCommand(findModel_1.FIND_IDS.ReplaceOneAction, function (x) { return x.replace(); }, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_1
    });
    registerFindCommand(findModel_1.FIND_IDS.ReplaceAllAction, function (x) { return x.replaceAll(); }, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.Enter
    });
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/find/browser/find", ["require", "exports", 'vs/platform/contextview/browser/contextView', 'vs/platform/keybinding/common/keybindingService', 'vs/editor/browser/editorBrowserExtensions', 'vs/editor/contrib/find/browser/findWidget', 'vs/editor/contrib/find/common/findController'], function (require, exports, contextView_1, keybindingService_1, editorBrowserExtensions_1, findWidget_1, findController_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FindController = (function (_super) {
        __extends(FindController, _super);
        function FindController(editor, contextViewService, keybindingService) {
            _super.call(this, editor, keybindingService);
            this._widget = this._register(new findWidget_1.FindWidget(editor, this, this._state, contextViewService, keybindingService));
        }
        FindController.prototype._start = function (opts) {
            _super.prototype._start.call(this, opts);
            if (opts.shouldFocus === findController_1.FindStartFocusAction.FocusReplaceInput) {
                this._widget.focusReplaceInput();
            }
            else if (opts.shouldFocus === findController_1.FindStartFocusAction.FocusFindInput) {
                this._widget.focusFindInput();
            }
        };
        FindController = __decorate([
            __param(1, contextView_1.IContextViewService),
            __param(2, keybindingService_1.IKeybindingService)
        ], FindController);
        return FindController;
    }(findController_1.CommonFindController));
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(FindController);
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(findController_1.SelectionHighlighter);
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/node/textMate/TMSnippets", ["require", "exports", 'vs/nls!vs/editor/node/textMate/TMSnippets', 'vs/base/common/json', 'vs/base/common/paths', 'vs/base/node/pfs', 'vs/platform/extensions/common/extensionsRegistry', 'vs/editor/common/modes/supports', 'vs/editor/common/services/modeService', 'vs/editor/common/services/modelService', 'vs/editor/contrib/snippet/common/snippet'], function (require, exports, nls, json_1, paths, pfs_1, extensionsRegistry_1, supports_1, modeService_1, modelService_1, snippet_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function snippetUpdated(modeId, filePath) {
        return pfs_1.readFile(filePath).then(function (fileContents) {
            var errors = [];
            var snippetsObj = json_1.parse(fileContents.toString(), errors);
            var adaptedSnippets = TMSnippetsAdaptor.adapt(snippetsObj);
            supports_1.SnippetsRegistry.registerSnippets(modeId, filePath, adaptedSnippets);
        });
    }
    exports.snippetUpdated = snippetUpdated;
    var snippetsExtensionPoint = extensionsRegistry_1.ExtensionsRegistry.registerExtensionPoint('snippets', {
        description: nls.localize(0, null),
        type: 'array',
        defaultSnippets: [{ body: [{ language: '', path: '' }] }],
        items: {
            type: 'object',
            defaultSnippets: [{ body: { language: '{{id}}', path: './snippets/{{id}}.json.' } }],
            properties: {
                language: {
                    description: nls.localize(1, null),
                    type: 'string'
                },
                path: {
                    description: nls.localize(2, null),
                    type: 'string'
                }
            }
        }
    });
    var MainProcessTextMateSnippet = (function () {
        function MainProcessTextMateSnippet(modelService, modeService) {
            var _this = this;
            this._modelService = modelService;
            this._modeService = modeService;
            snippetsExtensionPoint.setHandler(function (extensions) {
                for (var i = 0; i < extensions.length; i++) {
                    var tmSnippets = extensions[i].value;
                    for (var j = 0; j < tmSnippets.length; j++) {
                        _this._withTMSnippetContribution(extensions[i].description.extensionFolderPath, tmSnippets[j], extensions[i].collector);
                    }
                }
            });
        }
        MainProcessTextMateSnippet.prototype._withTMSnippetContribution = function (extensionFolderPath, snippet, collector) {
            var _this = this;
            if (!snippet.language || (typeof snippet.language !== 'string') || !this._modeService.isRegisteredMode(snippet.language)) {
                collector.error(nls.localize(3, null, snippetsExtensionPoint.name, String(snippet.language)));
                return;
            }
            if (!snippet.path || (typeof snippet.path !== 'string')) {
                collector.error(nls.localize(4, null, snippetsExtensionPoint.name, String(snippet.path)));
                return;
            }
            var normalizedAbsolutePath = paths.normalize(paths.join(extensionFolderPath, snippet.path));
            if (normalizedAbsolutePath.indexOf(extensionFolderPath) !== 0) {
                collector.warn(nls.localize(5, null, snippetsExtensionPoint.name, normalizedAbsolutePath, extensionFolderPath));
            }
            var modeId = snippet.language;
            var disposable = this._modeService.onDidCreateMode(function (mode) {
                if (mode.getId() !== modeId) {
                    return;
                }
                _this.registerDefinition(modeId, normalizedAbsolutePath);
                disposable.dispose();
            });
        };
        MainProcessTextMateSnippet.prototype.registerDefinition = function (modeId, filePath) {
            pfs_1.readFile(filePath).then(function (fileContents) {
                var errors = [];
                var snippetsObj = json_1.parse(fileContents.toString(), errors);
                var adaptedSnippets = TMSnippetsAdaptor.adapt(snippetsObj);
                supports_1.SnippetsRegistry.registerDefaultSnippets(modeId, adaptedSnippets);
            });
        };
        MainProcessTextMateSnippet = __decorate([
            __param(0, modelService_1.IModelService),
            __param(1, modeService_1.IModeService)
        ], MainProcessTextMateSnippet);
        return MainProcessTextMateSnippet;
    }());
    exports.MainProcessTextMateSnippet = MainProcessTextMateSnippet;
    var TMSnippetsAdaptor = (function () {
        function TMSnippetsAdaptor() {
        }
        TMSnippetsAdaptor.adapt = function (snippetsObj) {
            var topLevelProperties = Object.keys(snippetsObj), result = [];
            var processSnippet = function (snippet, description) {
                var prefix = snippet['prefix'];
                var bodyStringOrArray = snippet['body'];
                if (Array.isArray(bodyStringOrArray)) {
                    bodyStringOrArray = bodyStringOrArray.join('\n');
                }
                if (typeof prefix === 'string' && typeof bodyStringOrArray === 'string') {
                    var convertedSnippet = TMSnippetsAdaptor.convertSnippet(bodyStringOrArray);
                    if (convertedSnippet !== null) {
                        result.push({
                            type: 'snippet',
                            label: prefix,
                            documentationLabel: snippet['description'] || description,
                            codeSnippet: convertedSnippet,
                            noAutoAccept: true
                        });
                    }
                }
            };
            topLevelProperties.forEach(function (topLevelProperty) {
                var scopeOrTemplate = snippetsObj[topLevelProperty];
                if (scopeOrTemplate['body'] && scopeOrTemplate['prefix']) {
                    processSnippet(scopeOrTemplate, topLevelProperty);
                }
                else {
                    var snippetNames = Object.keys(scopeOrTemplate);
                    snippetNames.forEach(function (name) {
                        processSnippet(scopeOrTemplate[name], name);
                    });
                }
            });
            return result;
        };
        TMSnippetsAdaptor.convertSnippet = function (textMateSnippet) {
            return snippet_1.CodeSnippet.convertExternalSnippet(textMateSnippet, snippet_1.ExternalSnippetType.TextMateSnippet);
        };
        return TMSnippetsAdaptor;
    }());
});

define("vs/platform/telemetry/common/abstractTelemetryService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/errors', 'vs/base/common/types', 'vs/base/common/platform', 'vs/base/common/timer', 'vs/base/common/objects', 'vs/platform/platform', 'vs/platform/telemetry/common/telemetry'], function (require, exports, winjs_base_1, Errors, Types, Platform, timer_1, objects_1, platform_1, telemetry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DefaultTelemetryServiceConfig = {
        enableTelemetry: true,
        enableHardIdle: true,
        enableSoftIdle: true,
        userOptIn: true
    };
    /**
     * Base class for main process telemetry services
     */
    var AbstractTelemetryService = (function () {
        function AbstractTelemetryService(config) {
            var _this = this;
            this.serviceId = telemetry_1.ITelemetryService;
            this.sessionId = 'SESSION_ID_NOT_SET';
            this.timeKeeper = new timer_1.TimeKeeper();
            this.toUnbind = [];
            this.appenders = [];
            this.timeKeeperListener = function (events) { return _this.onTelemetryTimerEventStop(events); };
            this.timeKeeper.addListener(this.timeKeeperListener);
            this.toUnbind.push(Errors.errorHandler.addListener(this.onErrorEvent.bind(this)));
            this.errorBuffer = Object.create(null);
            this.enableGlobalErrorHandler();
            this.errorFlushTimeout = -1;
            this.config = objects_1.withDefaults(config, DefaultTelemetryServiceConfig);
        }
        AbstractTelemetryService.prototype._safeStringify = function (data) {
            return objects_1.safeStringify(data);
        };
        AbstractTelemetryService.prototype.onTelemetryTimerEventStop = function (events) {
            for (var i = 0; i < events.length; i++) {
                var event_1 = events[i];
                var data = event_1.data || {};
                data.duration = event_1.timeTaken();
                this.publicLog(event_1.name, data);
            }
        };
        AbstractTelemetryService.prototype.onErrorEvent = function (e) {
            if (!e) {
                return;
            }
            var error = Object.create(null);
            // unwrap nested errors from loader
            if (e.detail && e.detail.stack) {
                e = e.detail;
            }
            // work around behavior in workerServer.ts that breaks up Error.stack
            var stack = Array.isArray(e.stack) ? e.stack.join('\n') : e.stack;
            var message = e.message ? e.message : this._safeStringify(e);
            // errors without a stack are not useful telemetry
            if (!stack) {
                return;
            }
            error['message'] = this.cleanupInfo(message);
            error['stack'] = this.cleanupInfo(stack);
            this.addErrortoBuffer(error);
        };
        AbstractTelemetryService.prototype.addErrortoBuffer = function (e) {
            if (this.errorBuffer[e.stack]) {
                this.errorBuffer[e.stack].count++;
            }
            else {
                e.count = 1;
                this.errorBuffer[e.stack] = e;
            }
            this.tryScheduleErrorFlush();
        };
        AbstractTelemetryService.prototype.tryScheduleErrorFlush = function () {
            var _this = this;
            if (this.errorFlushTimeout === -1) {
                this.errorFlushTimeout = setTimeout(function () { return _this.flushErrorBuffer(); }, AbstractTelemetryService.ERROR_FLUSH_TIMEOUT);
            }
        };
        AbstractTelemetryService.prototype.flushErrorBuffer = function () {
            if (this.errorBuffer) {
                for (var stack in this.errorBuffer) {
                    this.publicLog('UnhandledError', this.errorBuffer[stack]);
                }
            }
            this.errorBuffer = Object.create(null);
            this.errorFlushTimeout = -1;
        };
        AbstractTelemetryService.prototype.cleanupInfo = function (stack) {
            // `file:///DANGEROUS/PATH/resources/app/Useful/Information`
            var reg = /file:\/\/\/.*?\/resources\/app\//gi;
            stack = stack.replace(reg, '');
            // Any other file path that doesn't match the approved form above should be cleaned.
            reg = /file:\/\/\/.*/gi;
            stack = stack.replace(reg, '');
            // "Error: ENOENT; no such file or directory" is often followed with PII, clean it
            reg = /ENOENT: no such file or directory.*?\'([^\']+)\'/gi;
            stack = stack.replace(reg, 'ENOENT: no such file or directory');
            return stack;
        };
        AbstractTelemetryService.prototype.enableGlobalErrorHandler = function () {
            if (Types.isFunction(Platform.globals.onerror)) {
                this.oldOnError = Platform.globals.onerror;
            }
            var that = this;
            var newHandler = function (message, filename, line, column, e) {
                that.onUncaughtError(message, filename, line, column, e);
                if (that.oldOnError) {
                    that.oldOnError.apply(this, arguments);
                }
            };
            Platform.globals.onerror = newHandler;
        };
        AbstractTelemetryService.prototype.onUncaughtError = function (message, filename, line, column, e) {
            filename = this.cleanupInfo(filename);
            message = this.cleanupInfo(message);
            var data = {
                message: message,
                filename: filename,
                line: line,
                column: column
            };
            if (e) {
                data.error = {
                    name: e.name,
                    message: e.message
                };
                if (e.stack) {
                    if (Array.isArray(e.stack)) {
                        e.stack = e.stack.join('\n');
                    }
                    data.stack = this.cleanupInfo(e.stack);
                }
            }
            if (!data.stack) {
                data.stack = data.message;
            }
            this.addErrortoBuffer(data);
        };
        AbstractTelemetryService.prototype.loadTelemetryAppendersFromRegistery = function () {
            var appendersRegistry = platform_1.Registry.as(exports.Extenstions.TelemetryAppenders).getTelemetryAppenderDescriptors();
            for (var i = 0; i < appendersRegistry.length; i++) {
                var descriptor = appendersRegistry[i];
                var appender = this.instantiationService.createInstance(descriptor);
                this.addTelemetryAppender(appender);
            }
        };
        AbstractTelemetryService.prototype.getSessionId = function () {
            return this.sessionId;
        };
        AbstractTelemetryService.prototype.getMachineId = function () {
            return this.machineId;
        };
        AbstractTelemetryService.prototype.getInstanceId = function () {
            return this.instanceId;
        };
        AbstractTelemetryService.prototype.getTelemetryInfo = function () {
            return winjs_base_1.TPromise.as({
                instanceId: this.instanceId,
                sessionId: this.sessionId,
                machineId: this.machineId
            });
        };
        AbstractTelemetryService.prototype.dispose = function () {
            if (this.errorFlushTimeout !== -1) {
                clearTimeout(this.errorFlushTimeout);
                this.flushErrorBuffer();
            }
            while (this.toUnbind.length) {
                this.toUnbind.pop()();
            }
            this.timeKeeper.removeListener(this.timeKeeperListener);
            this.timeKeeper.dispose();
            for (var i = 0; i < this.appenders.length; i++) {
                this.appenders[i].dispose();
            }
        };
        AbstractTelemetryService.prototype.start = function (name, data) {
            var topic = 'public';
            var event = this.timeKeeper.start(topic, name);
            if (data) {
                event.data = data;
            }
            return event;
        };
        AbstractTelemetryService.prototype.publicLog = function (eventName, data) {
            this.handleEvent(eventName, data);
        };
        AbstractTelemetryService.prototype.getAppendersCount = function () {
            return this.appenders.length;
        };
        AbstractTelemetryService.prototype.getAppenders = function () {
            return this.appenders;
        };
        AbstractTelemetryService.prototype.addTelemetryAppender = function (appender) {
            this.appenders.push(appender);
        };
        AbstractTelemetryService.prototype.removeTelemetryAppender = function (appender) {
            var index = this.appenders.indexOf(appender);
            if (index > -1) {
                this.appenders.splice(index, 1);
            }
        };
        AbstractTelemetryService.prototype.setInstantiationService = function (instantiationService) {
            this.instantiationService = instantiationService;
            if (this.instantiationService) {
                this.loadTelemetryAppendersFromRegistery();
            }
        };
        AbstractTelemetryService.prototype.handleEvent = function (eventName, data) {
            throw new Error('Not implemented!');
        };
        AbstractTelemetryService.ERROR_FLUSH_TIMEOUT = 5 * 1000;
        return AbstractTelemetryService;
    }());
    exports.AbstractTelemetryService = AbstractTelemetryService;
    exports.Extenstions = {
        TelemetryAppenders: 'telemetry.appenders'
    };
    var TelemetryAppendersRegistry = (function () {
        function TelemetryAppendersRegistry() {
            this.telemetryAppenderDescriptors = [];
        }
        TelemetryAppendersRegistry.prototype.registerTelemetryAppenderDescriptor = function (descriptor) {
            this.telemetryAppenderDescriptors.push(descriptor);
        };
        TelemetryAppendersRegistry.prototype.getTelemetryAppenderDescriptors = function () {
            return this.telemetryAppenderDescriptors;
        };
        return TelemetryAppendersRegistry;
    }());
    platform_1.Registry.add(exports.Extenstions.TelemetryAppenders, new TelemetryAppendersRegistry());
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/telemetry/browser/mainTelemetryService", ["require", "exports", 'vs/base/common/platform', 'vs/base/common/uuid', 'vs/platform/telemetry/common/abstractTelemetryService', 'vs/base/browser/idleMonitor'], function (require, exports, Platform, uuid, abstractTelemetryService_1, idleMonitor_1) {
    'use strict';
    var MainTelemetryService = (function (_super) {
        __extends(MainTelemetryService, _super);
        function MainTelemetryService(config) {
            var _this = this;
            _super.call(this, config);
            this.sessionId = this.config.sessionID || (uuid.generateUuid() + Date.now());
            if (this.config.enableHardIdle) {
                this.hardIdleMonitor = new idleMonitor_1.IdleMonitor();
            }
            if (this.config.enableSoftIdle) {
                this.softIdleMonitor = new idleMonitor_1.IdleMonitor(MainTelemetryService.SOFT_IDLE_TIME);
                this.softIdleMonitor.addOneTimeActiveListener(function () { return _this.onUserActive(); });
                this.softIdleMonitor.addOneTimeIdleListener(function () { return _this.onUserIdle(); });
            }
            this.eventCount = 0;
            this.startTime = new Date();
            //holds a cache of predefined events that can be sent regardress of user optin status
            this.optInFriendly = ['optInStatus'];
        }
        MainTelemetryService.prototype.onUserIdle = function () {
            var _this = this;
            this.publicLog(MainTelemetryService.IDLE_START_EVENT_NAME);
            this.softIdleMonitor.addOneTimeIdleListener(function () { return _this.onUserIdle(); });
        };
        MainTelemetryService.prototype.onUserActive = function () {
            var _this = this;
            this.publicLog(MainTelemetryService.IDLE_STOP_EVENT_NAME);
            this.softIdleMonitor.addOneTimeActiveListener(function () { return _this.onUserActive(); });
        };
        MainTelemetryService.prototype.dispose = function () {
            if (this.hardIdleMonitor) {
                this.hardIdleMonitor.dispose();
            }
            if (this.softIdleMonitor) {
                this.softIdleMonitor.dispose();
            }
            _super.prototype.dispose.call(this);
        };
        MainTelemetryService.prototype.handleEvent = function (eventName, data) {
            if (this.hardIdleMonitor && this.hardIdleMonitor.getStatus() === idleMonitor_1.UserStatus.Idle) {
                return;
            }
            // don't send telemetry when channel is not enabled
            if (!this.config.enableTelemetry) {
                return;
            }
            // don't send events when the user is optout unless the event is flaged as optin friendly
            if (!this.config.userOptIn && this.optInFriendly.indexOf(eventName) === -1) {
                return;
            }
            this.eventCount++;
            data = this.addCommonProperties(data);
            var allAppenders = this.getAppenders();
            for (var i = 0; i < allAppenders.length; i++) {
                allAppenders[i].log(eventName, data);
            }
        };
        MainTelemetryService.prototype.addCommonProperties = function (data) {
            data = data || {};
            var eventDate = new Date();
            data['sessionID'] = this.sessionId;
            data['timestamp'] = eventDate;
            data['version'] = this.config.version;
            data['userId'] = this.userIdHash;
            data['commitHash'] = this.config.commitHash;
            data['common.platform'] = Platform.Platform[Platform.platform];
            data['common.timesincesessionstart'] = (eventDate.getTime() - this.startTime.getTime());
            data['common.sequence'] = this.eventCount;
            data['common.instanceId'] = this.instanceId;
            data['common.machineId'] = this.machineId;
            return data;
        };
        // how long of inactivity before a user is considered 'inactive' - 2 minutes
        MainTelemetryService.SOFT_IDLE_TIME = 2 * 60 * 1000;
        MainTelemetryService.IDLE_START_EVENT_NAME = 'UserIdleStart';
        MainTelemetryService.IDLE_STOP_EVENT_NAME = 'UserIdleStop';
        return MainTelemetryService;
    }(abstractTelemetryService_1.AbstractTelemetryService));
    exports.MainTelemetryService = MainTelemetryService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/platform/telemetry/electron-browser/electronTelemetryService", ["require", "exports", 'getmac', 'crypto', 'vs/base/common/winjs.base', 'vs/nls!vs/platform/telemetry/electron-browser/electronTelemetryService', 'vs/base/common/errors', 'vs/base/common/uuid', 'vs/platform/telemetry/browser/mainTelemetryService', 'vs/platform/storage/common/storage', 'vs/platform/configuration/common/configurationRegistry', 'vs/platform/configuration/common/configuration', 'vs/platform/platform'], function (require, exports, getmac, crypto, winjs_base_1, nls, errors, uuid, mainTelemetryService_1, storage_1, configurationRegistry_1, configuration_1, platform_1) {
    "use strict";
    var TELEMETRY_SECTION_ID = 'telemetry';
    var StorageKeys = (function () {
        function StorageKeys() {
        }
        StorageKeys.MachineId = 'telemetry.machineId';
        StorageKeys.InstanceId = 'telemetry.instanceId';
        return StorageKeys;
    }());
    var ElectronTelemetryService = (function (_super) {
        __extends(ElectronTelemetryService, _super);
        function ElectronTelemetryService(configurationService, storageService, config) {
            _super.call(this, config);
            this.configurationService = configurationService;
            this.storageService = storageService;
            this._buffer = [];
            this._optInStatusLoaded = false;
            this.loadOptinSettings();
            this._setupIds = this.setupIds();
        }
        /**
         * override the base getTelemetryInfo to make sure this information is not retrieved before it's ready
         */
        ElectronTelemetryService.prototype.getTelemetryInfo = function () {
            return this._setupIds;
        };
        /**
         * override the base publicLog to prevent reporting any events before the optIn status is read from configuration
         */
        ElectronTelemetryService.prototype.publicLog = function (eventName, data) {
            if (this._optInStatusLoaded) {
                _super.prototype.publicLog.call(this, eventName, data);
            }
            else {
                // in case loading configuration is delayed, make sure the buffer does not grow beyond MAX_BUFFER_SIZE
                if (this._buffer.length > ElectronTelemetryService.MAX_BUFFER_SIZE) {
                    this._buffer = [];
                }
                this._buffer.push({ eventName: eventName, data: data });
            }
        };
        ElectronTelemetryService.prototype.flushBuffer = function () {
            var event = null;
            while (event = this._buffer.pop()) {
                _super.prototype.publicLog.call(this, event.eventName, event.data);
            }
        };
        ElectronTelemetryService.prototype.loadOptinSettings = function () {
            var _this = this;
            this.configurationService.loadConfiguration(TELEMETRY_SECTION_ID).done(function (config) {
                _this.config.userOptIn = config ? config.enableTelemetry : _this.config.userOptIn;
                _this._optInStatusLoaded = true;
                _this.publicLog('optInStatus', { optIn: _this.config.userOptIn });
                _this.flushBuffer();
            });
            this.toUnbind.push(this.configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) {
                _this.config.userOptIn = e.config && e.config[TELEMETRY_SECTION_ID] ? e.config[TELEMETRY_SECTION_ID].enableTelemetry : _this.config.userOptIn;
            }));
        };
        ElectronTelemetryService.prototype.setupIds = function () {
            var _this = this;
            return winjs_base_1.TPromise.join([this.setupInstanceId(), this.setupMachineId()]).then(function () {
                return {
                    machineId: _this.machineId,
                    instanceId: _this.instanceId,
                    sessionId: _this.sessionId
                };
            });
        };
        ElectronTelemetryService.prototype.setupInstanceId = function () {
            var instanceId = this.storageService.get(StorageKeys.InstanceId);
            if (!instanceId) {
                instanceId = uuid.generateUuid();
                this.storageService.store(StorageKeys.InstanceId, instanceId);
            }
            this.instanceId = instanceId;
            return winjs_base_1.TPromise.as(this.instanceId);
        };
        ElectronTelemetryService.prototype.setupMachineId = function () {
            var _this = this;
            var machineId = this.storageService.get(StorageKeys.MachineId);
            if (machineId) {
                this.machineId = machineId;
                return winjs_base_1.TPromise.as(this.machineId);
            }
            else {
                return new winjs_base_1.TPromise(function (resolve, reject) {
                    try {
                        // add a unique machine id as a hash of the macAddress
                        getmac.getMac(function (error, macAddress) {
                            if (!error) {
                                // crypt machine id
                                machineId = crypto.createHash('sha256').update(macAddress, 'utf8').digest('hex');
                            }
                            else {
                                // generate a UUID
                                machineId = uuid.generateUuid();
                            }
                            _this.machineId = machineId;
                            _this.storageService.store(StorageKeys.MachineId, machineId);
                            resolve(_this.machineId);
                        });
                    }
                    catch (err) {
                        errors.onUnexpectedError(err);
                        // generate a UUID
                        machineId = uuid.generateUuid();
                        _this.machineId = machineId;
                        _this.storageService.store(StorageKeys.MachineId, machineId);
                        resolve(_this.machineId);
                    }
                });
            }
        };
        ElectronTelemetryService.MAX_BUFFER_SIZE = 100;
        ElectronTelemetryService = __decorate([
            __param(0, configuration_1.IConfigurationService),
            __param(1, storage_1.IStorageService)
        ], ElectronTelemetryService);
        return ElectronTelemetryService;
    }(mainTelemetryService_1.MainTelemetryService));
    exports.ElectronTelemetryService = ElectronTelemetryService;
    var configurationRegistry = platform_1.Registry.as(configurationRegistry_1.Extensions.Configuration);
    configurationRegistry.registerConfiguration({
        'id': TELEMETRY_SECTION_ID,
        'order': 20,
        'type': 'object',
        'title': nls.localize(0, null),
        'properties': {
            'telemetry.enableTelemetry': {
                'type': 'boolean',
                'description': nls.localize(1, null),
                'default': true
            }
        }
    });
});

define("vs/platform/theme/common/themes", ["require", "exports", 'vs/nls!vs/platform/theme/common/themes'], function (require, exports, nls) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (BaseTheme) {
        BaseTheme[BaseTheme["VS"] = 0] = "VS";
        BaseTheme[BaseTheme["VS_DARK"] = 1] = "VS_DARK";
        BaseTheme[BaseTheme["HIGH_CONTRAST"] = 2] = "HIGH_CONTRAST";
    })(exports.BaseTheme || (exports.BaseTheme = {}));
    var BaseTheme = exports.BaseTheme;
    function getBaseThemes(includeHighContrast) {
        if (includeHighContrast) {
            return [BaseTheme.VS, BaseTheme.VS_DARK, BaseTheme.HIGH_CONTRAST];
        }
        return [BaseTheme.VS, BaseTheme.VS_DARK];
    }
    exports.getBaseThemes = getBaseThemes;
    function toId(theme) {
        switch (theme) {
            case BaseTheme.VS:
                return 'vs';
            case BaseTheme.VS_DARK:
                return 'vs-dark';
        }
        return 'hc-black';
    }
    exports.toId = toId;
    function toLabel(theme) {
        switch (theme) {
            case BaseTheme.VS:
                return nls.localize(0, null);
            case BaseTheme.VS_DARK:
                return nls.localize(1, null);
        }
        return nls.localize(2, null);
    }
    exports.toLabel = toLabel;
    function isLightTheme(themeId) {
        return /vs($| )/.test(themeId);
    }
    exports.isLightTheme = isLightTheme;
    function getSyntaxThemeId(themeId) {
        return themeId.split(' ')[1];
    }
    exports.getSyntaxThemeId = getSyntaxThemeId;
    function getBaseThemeId(themeId) {
        return themeId.split(' ')[0];
    }
    exports.getBaseThemeId = getBaseThemeId;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/viewParts/overviewRuler/decorationsOverviewRuler", ["require", "exports", 'vs/platform/theme/common/themes', 'vs/editor/common/editorCommon', 'vs/editor/browser/view/viewPart', 'vs/editor/browser/viewParts/overviewRuler/overviewRulerImpl'], function (require, exports, themes, editorCommon, viewPart_1, overviewRulerImpl_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DecorationsOverviewRuler = (function (_super) {
        __extends(DecorationsOverviewRuler, _super);
        function DecorationsOverviewRuler(context, scrollHeight, getVerticalOffsetForLine) {
            _super.call(this, context);
            this._overviewRuler = new overviewRulerImpl_1.OverviewRulerImpl(1, 'decorationsOverviewRuler', scrollHeight, this._context.configuration.editor.lineHeight, DecorationsOverviewRuler.DECORATION_HEIGHT, DecorationsOverviewRuler.DECORATION_HEIGHT, getVerticalOffsetForLine);
            this._overviewRuler.setLanesCount(this._context.configuration.editor.overviewRulerLanes, false);
            var theme = this._context.configuration.editor.theme;
            this._overviewRuler.setUseDarkColor(!themes.isLightTheme(theme), false);
            this._shouldUpdateDecorations = true;
            this._zonesFromDecorations = [];
            this._shouldUpdateCursorPosition = true;
            this._hideCursor = this._context.configuration.editor.hideCursorInOverviewRuler;
            this._shouldForceRender = false;
            this._zonesFromCursors = [];
            this._cursorPositions = [];
        }
        DecorationsOverviewRuler.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this._overviewRuler.dispose();
        };
        // ---- begin view event handlers
        DecorationsOverviewRuler.prototype.onCursorPositionChanged = function (e) {
            this._shouldUpdateCursorPosition = true;
            this._cursorPositions = [e.position];
            this._cursorPositions = this._cursorPositions.concat(e.secondaryPositions);
            return true;
        };
        DecorationsOverviewRuler.prototype.onConfigurationChanged = function (e) {
            var prevLanesCount = this._overviewRuler.getLanesCount();
            var newLanesCount = this._context.configuration.editor.overviewRulerLanes;
            var shouldRender = false;
            if (e.lineHeight) {
                this._overviewRuler.setLineHeight(this._context.configuration.editor.lineHeight, false);
                this._shouldForceRender = true;
                shouldRender = true;
            }
            if (prevLanesCount !== newLanesCount) {
                this._overviewRuler.setLanesCount(newLanesCount, false);
                this._shouldForceRender = true;
                shouldRender = true;
            }
            if (e.hideCursorInOverviewRuler) {
                this._hideCursor = this._context.configuration.editor.hideCursorInOverviewRuler;
                this._shouldUpdateCursorPosition = true;
                shouldRender = true;
            }
            if (e.theme) {
                var theme = this._context.configuration.editor.theme;
                this._overviewRuler.setUseDarkColor(!themes.isLightTheme(theme), false);
                this._shouldForceRender = true;
                shouldRender = true;
            }
            return shouldRender;
        };
        DecorationsOverviewRuler.prototype.onLayoutChanged = function (layoutInfo) {
            var _this = this;
            this._shouldForceRender = true;
            this._requestModificationFrame(function () {
                _this._overviewRuler.setLayout(layoutInfo.overviewRuler, false);
            });
            return true;
        };
        DecorationsOverviewRuler.prototype.onZonesChanged = function () {
            return true;
        };
        DecorationsOverviewRuler.prototype.onModelFlushed = function () {
            this._shouldUpdateCursorPosition = true;
            this._shouldUpdateDecorations = true;
            return true;
        };
        DecorationsOverviewRuler.prototype.onModelDecorationsChanged = function (e) {
            this._shouldUpdateDecorations = true;
            return true;
        };
        DecorationsOverviewRuler.prototype.onScrollHeightChanged = function (scrollHeight) {
            this._overviewRuler.setScrollHeight(scrollHeight, false);
            this._shouldForceRender = true;
            return true;
        };
        // ---- end view event handlers
        DecorationsOverviewRuler.prototype.getDomNode = function () {
            return this._overviewRuler.getDomNode();
        };
        DecorationsOverviewRuler.prototype._createZonesFromDecorations = function () {
            var decorations = this._context.model.getAllDecorations(), zones = [], i, len, dec;
            for (i = 0, len = decorations.length; i < len; i++) {
                dec = decorations[i];
                if (dec.options.overviewRuler.color) {
                    zones.push({
                        startLineNumber: dec.range.startLineNumber,
                        endLineNumber: dec.range.endLineNumber,
                        color: dec.options.overviewRuler.color,
                        darkColor: dec.options.overviewRuler.darkColor,
                        position: dec.options.overviewRuler.position
                    });
                }
            }
            return zones;
        };
        DecorationsOverviewRuler.prototype._createZonesFromCursors = function () {
            var zones = [], i, len, cursor;
            for (i = 0, len = this._cursorPositions.length; i < len; i++) {
                cursor = this._cursorPositions[i];
                zones.push({
                    forceHeight: 2,
                    startLineNumber: cursor.lineNumber,
                    endLineNumber: cursor.lineNumber,
                    color: DecorationsOverviewRuler._CURSOR_COLOR,
                    darkColor: DecorationsOverviewRuler._CURSOR_COLOR_DARK,
                    position: editorCommon.OverviewRulerLane.Full
                });
            }
            return zones;
        };
        DecorationsOverviewRuler.prototype._render = function (ctx) {
            var _this = this;
            var shouldForceRender = this._shouldForceRender;
            this._shouldForceRender = false;
            // Update decorations if necessary
            var shouldRender = false;
            if (this._shouldUpdateDecorations || this._shouldUpdateCursorPosition) {
                if (this._shouldUpdateDecorations) {
                    this._shouldUpdateDecorations = false;
                    this._zonesFromDecorations = this._createZonesFromDecorations();
                }
                if (this._shouldUpdateCursorPosition) {
                    this._shouldUpdateCursorPosition = false;
                    if (this._hideCursor) {
                        this._zonesFromCursors = [];
                    }
                    else {
                        this._zonesFromCursors = this._createZonesFromCursors();
                    }
                }
                var allZones = [];
                allZones = allZones.concat(this._zonesFromCursors);
                allZones = allZones.concat(this._zonesFromDecorations);
                this._overviewRuler.setZones(allZones, false);
                shouldRender = true;
            }
            if (shouldRender || shouldForceRender) {
                this._requestModificationFrame(function () {
                    var hasRendered = _this._overviewRuler.render(shouldForceRender);
                    if (hasRendered && overviewRulerImpl_1.OverviewRulerImpl.hasCanvas && _this._overviewRuler.getLanesCount() > 0 && (_this._zonesFromDecorations.length > 0 || _this._zonesFromCursors.length > 0)) {
                        var ctx2 = _this._overviewRuler.getDomNode().getContext('2d');
                        ctx2.beginPath();
                        ctx2.lineWidth = 1;
                        ctx2.strokeStyle = 'rgba(197,197,197,0.8)';
                        ctx2.moveTo(0, 0);
                        ctx2.lineTo(0, _this._overviewRuler.getHeight());
                        ctx2.stroke();
                        ctx2.moveTo(0, 0);
                        ctx2.lineTo(_this._overviewRuler.getWidth(), 0);
                        ctx2.stroke();
                    }
                });
            }
        };
        DecorationsOverviewRuler.DECORATION_HEIGHT = 6;
        DecorationsOverviewRuler._CURSOR_COLOR = 'rgba(0, 0, 102, 0.8)';
        DecorationsOverviewRuler._CURSOR_COLOR_DARK = 'rgba(152, 152, 152, 0.8)';
        return DecorationsOverviewRuler;
    }(viewPart_1.ViewPart));
    exports.DecorationsOverviewRuler = DecorationsOverviewRuler;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/editor/browser/view/viewImpl", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/eventEmitter', 'vs/base/common/lifecycle', 'vs/base/common/timer', 'vs/base/browser/browser', 'vs/base/browser/dom', 'vs/base/browser/styleMutator', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/viewModel/viewEventHandler', 'vs/editor/browser/config/configuration', 'vs/editor/browser/controller/keyboardHandler', 'vs/editor/browser/controller/pointerHandler', 'vs/editor/browser/editorBrowser', 'vs/editor/browser/view/viewController', 'vs/editor/browser/view/viewEventDispatcher', 'vs/editor/browser/view/viewOverlays', 'vs/editor/browser/viewLayout/layoutProvider', 'vs/editor/browser/viewParts/contentWidgets/contentWidgets', 'vs/editor/browser/viewParts/currentLineHighlight/currentLineHighlight', 'vs/editor/browser/viewParts/decorations/decorations', 'vs/editor/browser/viewParts/glyphMargin/glyphMargin', 'vs/editor/browser/viewParts/lineNumbers/lineNumbers', 'vs/editor/browser/viewParts/lines/viewLines', 'vs/editor/browser/viewParts/linesDecorations/linesDecorations', 'vs/editor/browser/viewParts/overlayWidgets/overlayWidgets', 'vs/editor/browser/viewParts/overviewRuler/decorationsOverviewRuler', 'vs/editor/browser/viewParts/overviewRuler/overviewRuler', 'vs/editor/browser/viewParts/rulers/rulers', 'vs/editor/browser/viewParts/scrollDecoration/scrollDecoration', 'vs/editor/browser/viewParts/selections/selections', 'vs/editor/browser/viewParts/viewCursors/viewCursors', 'vs/editor/browser/viewParts/viewZones/viewZones'], function (require, exports, errors_1, eventEmitter_1, lifecycle_1, timer, browser, dom, styleMutator_1, range_1, editorCommon, viewEventHandler_1, configuration_1, keyboardHandler_1, pointerHandler_1, editorBrowser, viewController_1, viewEventDispatcher_1, viewOverlays_1, layoutProvider_1, contentWidgets_1, currentLineHighlight_1, decorations_1, glyphMargin_1, lineNumbers_1, viewLines_1, linesDecorations_1, overlayWidgets_1, decorationsOverviewRuler_1, overviewRuler_1, rulers_1, scrollDecoration_1, selections_1, viewCursors_1, viewZones_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var View = (function (_super) {
        __extends(View, _super);
        function View(editorId, configuration, model, keybindingService) {
            var _this = this;
            _super.call(this);
            this._isDisposed = false;
            this._editorId = editorId;
            this._renderAnimationFrame = null;
            this.outgoingEventBus = new eventEmitter_1.EventEmitter();
            var viewController = new viewController_1.ViewController(model, configuration, this.outgoingEventBus, keybindingService);
            this.listenersToRemove = [];
            this.listenersToDispose = [];
            // The event dispatcher will always go through _renderOnce before dispatching any events
            this.eventDispatcher = new viewEventDispatcher_1.ViewEventDispatcher(function (callback) { return _this._renderOnce(callback); });
            // These two dom nodes must be constructed up front, since references are needed in the layout provider (scrolling & co.)
            this.linesContent = document.createElement('div');
            this.linesContent.className = editorBrowser.ClassNames.LINES_CONTENT + ' monaco-editor-background';
            this.domNode = document.createElement('div');
            configuration_1.Configuration.applyEditorStyling(this.domNode, configuration.editor.stylingInfo);
            this.overflowGuardContainer = document.createElement('div');
            this.overflowGuardContainer.className = editorBrowser.ClassNames.OVERFLOW_GUARD;
            // The layout provider has such responsibilities as:
            // - scrolling (i.e. viewport / full size) & co.
            // - whitespaces (a.k.a. view zones) management & co.
            // - line heights updating & co.
            this.layoutProvider = new layoutProvider_1.LayoutProvider(configuration, model, this.eventDispatcher, this.linesContent, this.domNode, this.overflowGuardContainer);
            this.eventDispatcher.addEventHandler(this.layoutProvider);
            // The view context is passed on to most classes (basically to reduce param. counts in ctors)
            this.context = new ViewContext(editorId, configuration, model, this.eventDispatcher, function (eventHandler) { return _this.eventDispatcher.addEventHandler(eventHandler); }, function (eventHandler) { return _this.eventDispatcher.removeEventHandler(eventHandler); });
            this.createTextArea(keybindingService);
            this.createViewParts();
            // Keyboard handler
            this.keyboardHandler = new keyboardHandler_1.KeyboardHandler(this.context, viewController, this.createKeyboardHandlerHelper());
            // Pointer handler
            this.pointerHandler = new pointerHandler_1.PointerHandler(this.context, viewController, this.createPointerHandlerHelper());
            this.hasFocus = false;
            this.codeEditorHelper = null;
            this.eventDispatcher.addEventHandler(this);
            // The view lines rendering calls model.getLineTokens() that might emit events that its tokens have changed.
            // This delayed processing of incoming model events acts as a guard against undesired/unexpected recursion.
            this.handleAccumulatedModelEventsTimeout = -1;
            this.accumulatedModelEvents = [];
            this.listenersToRemove.push(model.addBulkListener(function (events) {
                _this.accumulatedModelEvents = _this.accumulatedModelEvents.concat(events);
                if (_this.handleAccumulatedModelEventsTimeout === -1) {
                    _this.handleAccumulatedModelEventsTimeout = setTimeout(function () {
                        _this.handleAccumulatedModelEventsTimeout = -1;
                        _this._flushAnyAccumulatedEvents();
                    });
                }
            }));
        }
        View.prototype._flushAnyAccumulatedEvents = function () {
            var toEmit = this.accumulatedModelEvents;
            this.accumulatedModelEvents = [];
            if (toEmit.length > 0) {
                this.eventDispatcher.emitMany(toEmit);
            }
        };
        View.prototype.createTextArea = function (keybindingService) {
            var _this = this;
            // Text Area (The focus will always be in the textarea when the cursor is blinking)
            this.textArea = document.createElement('textarea');
            this._keybindingService = keybindingService.createScoped(this.textArea);
            this._editorTextFocusContextKey = this._keybindingService.createKey(editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS, undefined);
            this.textArea.className = editorBrowser.ClassNames.TEXTAREA;
            this.textArea.setAttribute('wrap', 'off');
            this.textArea.setAttribute('autocorrect', 'off');
            this.textArea.setAttribute('autocapitalize', 'off');
            this.textArea.setAttribute('spellcheck', 'false');
            this.textArea.setAttribute('aria-label', this.context.configuration.editor.ariaLabel);
            this.textArea.setAttribute('role', 'textbox');
            this.textArea.setAttribute('aria-multiline', 'true');
            this.textArea.setAttribute('aria-haspopup', 'false');
            this.textArea.setAttribute('aria-autocomplete', 'both');
            styleMutator_1.StyleMutator.setTop(this.textArea, 0);
            styleMutator_1.StyleMutator.setLeft(this.textArea, 0);
            // Give textarea same font size & line height as editor, for the IME case (when the textarea is visible)
            styleMutator_1.StyleMutator.setFontSize(this.textArea, this.context.configuration.editor.fontSize);
            styleMutator_1.StyleMutator.setLineHeight(this.textArea, this.context.configuration.editor.lineHeight);
            this.listenersToDispose.push(dom.addDisposableListener(this.textArea, 'focus', function () { return _this._setHasFocus(true); }));
            this.listenersToDispose.push(dom.addDisposableListener(this.textArea, 'blur', function () { return _this._setHasFocus(false); }));
            // On top of the text area, we position a dom node to cover it up
            // (there have been reports of tiny blinking cursors)
            // (in WebKit the textarea is 1px by 1px because it cannot handle input to a 0x0 textarea)
            this.textAreaCover = document.createElement('div');
            if (this.context.configuration.editor.glyphMargin) {
                this.textAreaCover.className = 'monaco-editor-background ' + editorBrowser.ClassNames.GLYPH_MARGIN + ' ' + editorBrowser.ClassNames.TEXTAREA_COVER;
            }
            else {
                if (this.context.configuration.editor.lineNumbers) {
                    this.textAreaCover.className = 'monaco-editor-background ' + editorBrowser.ClassNames.LINE_NUMBERS + ' ' + editorBrowser.ClassNames.TEXTAREA_COVER;
                }
                else {
                    this.textAreaCover.className = 'monaco-editor-background ' + editorBrowser.ClassNames.TEXTAREA_COVER;
                }
            }
            this.textAreaCover.style.position = 'absolute';
            styleMutator_1.StyleMutator.setWidth(this.textAreaCover, 1);
            styleMutator_1.StyleMutator.setHeight(this.textAreaCover, 1);
            styleMutator_1.StyleMutator.setTop(this.textAreaCover, 0);
            styleMutator_1.StyleMutator.setLeft(this.textAreaCover, 0);
        };
        View.prototype.createViewParts = function () {
            var _this = this;
            this.viewParts = [];
            // View Lines
            this.viewLines = new viewLines_1.ViewLines(this.context, this.layoutProvider);
            // View Zones
            this.viewZones = new viewZones_1.ViewZones(this.context, this.layoutProvider);
            this.viewParts.push(this.viewZones);
            // Decorations overview ruler
            var decorationsOverviewRuler = new decorationsOverviewRuler_1.DecorationsOverviewRuler(this.context, this.layoutProvider.getScrollHeight(), function (lineNumber) { return _this.layoutProvider.getVerticalOffsetForLineNumber(lineNumber); });
            this.viewParts.push(decorationsOverviewRuler);
            var scrollDecoration = new scrollDecoration_1.ScrollDecorationViewPart(this.context);
            this.viewParts.push(scrollDecoration);
            var contentViewOverlays = new viewOverlays_1.ContentViewOverlays(this.context, this.layoutProvider);
            this.viewParts.push(contentViewOverlays);
            contentViewOverlays.addDynamicOverlay(new currentLineHighlight_1.CurrentLineHighlightOverlay(this.context, this.layoutProvider));
            contentViewOverlays.addDynamicOverlay(new selections_1.SelectionsOverlay(this.context));
            contentViewOverlays.addDynamicOverlay(new decorations_1.DecorationsOverlay(this.context));
            var marginViewOverlays = new viewOverlays_1.MarginViewOverlays(this.context, this.layoutProvider);
            this.viewParts.push(marginViewOverlays);
            marginViewOverlays.addDynamicOverlay(new glyphMargin_1.GlyphMarginOverlay(this.context));
            marginViewOverlays.addDynamicOverlay(new linesDecorations_1.LinesDecorationsOverlay(this.context));
            marginViewOverlays.addDynamicOverlay(new lineNumbers_1.LineNumbersOverlay(this.context));
            // Content widgets
            this.contentWidgets = new contentWidgets_1.ViewContentWidgets(this.context, this.domNode);
            this.viewParts.push(this.contentWidgets);
            var viewCursors = new viewCursors_1.ViewCursors(this.context);
            this.viewParts.push(viewCursors);
            // Overlay widgets
            this.overlayWidgets = new overlayWidgets_1.ViewOverlayWidgets(this.context);
            this.viewParts.push(this.overlayWidgets);
            var rulers = new rulers_1.Rulers(this.context, this.layoutProvider);
            this.viewParts.push(rulers);
            // -------------- Wire dom nodes up
            this.linesContentContainer = this.layoutProvider.getScrollbarContainerDomNode();
            this.linesContentContainer.style.position = 'absolute';
            if (decorationsOverviewRuler) {
                var overviewRulerData = this.layoutProvider.getOverviewRulerInsertData();
                overviewRulerData.parent.insertBefore(decorationsOverviewRuler.getDomNode(), overviewRulerData.insertBefore);
            }
            this.linesContent.appendChild(contentViewOverlays.getDomNode());
            this.linesContent.appendChild(rulers.domNode);
            this.linesContent.appendChild(this.viewZones.domNode);
            this.linesContent.appendChild(this.viewLines.domNode);
            this.linesContent.appendChild(this.contentWidgets.domNode);
            this.linesContent.appendChild(viewCursors.getDomNode());
            this.overflowGuardContainer.appendChild(marginViewOverlays.getDomNode());
            this.overflowGuardContainer.appendChild(this.linesContentContainer);
            this.overflowGuardContainer.appendChild(scrollDecoration.getDomNode());
            this.overflowGuardContainer.appendChild(this.overlayWidgets.domNode);
            this.overflowGuardContainer.appendChild(this.textArea);
            this.overflowGuardContainer.appendChild(this.textAreaCover);
            this.domNode.appendChild(this.overflowGuardContainer);
            this.domNode.appendChild(this.contentWidgets.overflowingContentWidgetsDomNode);
        };
        View.prototype._flushAccumulatedAndRenderNow = function () {
            this._flushAnyAccumulatedEvents();
            this._renderNow();
        };
        View.prototype.createPointerHandlerHelper = function () {
            var _this = this;
            return {
                viewDomNode: this.domNode,
                linesContentDomNode: this.linesContent,
                focusTextArea: function () {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.focusTextArea: View is disposed');
                    }
                    _this.focus();
                },
                isDirty: function () {
                    return (_this.accumulatedModelEvents.length > 0);
                },
                getScrollTop: function () {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.getScrollTop: View is disposed');
                    }
                    return _this.layoutProvider.getScrollTop();
                },
                setScrollTop: function (scrollTop) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.setScrollTop: View is disposed');
                    }
                    _this.layoutProvider.setScrollTop(scrollTop);
                },
                getScrollLeft: function () {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.getScrollLeft: View is disposed');
                    }
                    return _this.layoutProvider.getScrollLeft();
                },
                setScrollLeft: function (scrollLeft) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.setScrollLeft: View is disposed');
                    }
                    _this.layoutProvider.setScrollLeft(scrollLeft);
                },
                isAfterLines: function (verticalOffset) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.isAfterLines: View is disposed');
                    }
                    return _this.layoutProvider.isAfterLines(verticalOffset);
                },
                getLineNumberAtVerticalOffset: function (verticalOffset) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.getLineNumberAtVerticalOffset: View is disposed');
                    }
                    return _this.layoutProvider.getLineNumberAtVerticalOffset(verticalOffset);
                },
                getVerticalOffsetForLineNumber: function (lineNumber) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.getVerticalOffsetForLineNumber: View is disposed');
                    }
                    return _this.layoutProvider.getVerticalOffsetForLineNumber(lineNumber);
                },
                getWhitespaceAtVerticalOffset: function (verticalOffset) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.getWhitespaceAtVerticalOffset: View is disposed');
                    }
                    return _this.layoutProvider.getWhitespaceAtVerticalOffset(verticalOffset);
                },
                shouldSuppressMouseDownOnViewZone: function (viewZoneId) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.shouldSuppressMouseDownOnViewZone: View is disposed');
                    }
                    return _this.viewZones.shouldSuppressMouseDownOnViewZone(viewZoneId);
                },
                getPositionFromDOMInfo: function (spanNode, offset) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.getPositionFromDOMInfo: View is disposed');
                    }
                    _this._flushAccumulatedAndRenderNow();
                    return _this.viewLines.getPositionFromDOMInfo(spanNode, offset);
                },
                visibleRangeForPosition2: function (lineNumber, column) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.visibleRangeForPosition2: View is disposed');
                    }
                    _this._flushAccumulatedAndRenderNow();
                    var visibleRanges = _this.viewLines.visibleRangesForRange2(new range_1.Range(lineNumber, column, lineNumber, column), 0);
                    if (!visibleRanges) {
                        return null;
                    }
                    return visibleRanges[0];
                },
                getLineWidth: function (lineNumber) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.pointerHandler.getLineWidth: View is disposed');
                    }
                    _this._flushAccumulatedAndRenderNow();
                    return _this.viewLines.getLineWidth(lineNumber);
                }
            };
        };
        View.prototype.createKeyboardHandlerHelper = function () {
            var _this = this;
            return {
                viewDomNode: this.domNode,
                textArea: this.textArea,
                visibleRangeForPositionRelativeToEditor: function (lineNumber, column) {
                    if (_this._isDisposed) {
                        throw new Error('ViewImpl.keyboardHandler.visibleRangeForPositionRelativeToEditor: View is disposed');
                    }
                    _this._flushAccumulatedAndRenderNow();
                    var linesViewPortData = _this.layoutProvider.getLinesViewportData();
                    var visibleRanges = _this.viewLines.visibleRangesForRange2(new range_1.Range(lineNumber, column, lineNumber, column), linesViewPortData.visibleRangesDeltaTop);
                    if (!visibleRanges) {
                        return null;
                    }
                    return visibleRanges[0];
                },
                flushAnyAccumulatedEvents: function () {
                    _this._flushAnyAccumulatedEvents();
                }
            };
        };
        View.prototype.setAriaActiveDescendant = function (id) {
            if (id) {
                this.textArea.setAttribute('role', 'combobox');
                if (this.textArea.getAttribute('aria-activedescendant') !== id) {
                    this.textArea.setAttribute('aria-haspopup', 'true');
                    this.textArea.setAttribute('aria-activedescendant', id);
                }
            }
            else {
                this.textArea.setAttribute('role', 'textbox');
                this.textArea.removeAttribute('aria-activedescendant');
                this.textArea.removeAttribute('aria-haspopup');
            }
        };
        // --- begin event handlers
        View.prototype.onLayoutChanged = function (layoutInfo) {
            if (browser.isChrome) {
                /* tslint:disable:no-unused-variable */
                // Access overflowGuardContainer.clientWidth to prevent relayouting bug in Chrome
                // See Bug 19676: Editor misses a layout event
                var clientWidth = this.overflowGuardContainer.clientWidth + 'px';
            }
            styleMutator_1.StyleMutator.setWidth(this.domNode, layoutInfo.width);
            styleMutator_1.StyleMutator.setHeight(this.domNode, layoutInfo.height);
            styleMutator_1.StyleMutator.setWidth(this.overflowGuardContainer, layoutInfo.width);
            styleMutator_1.StyleMutator.setHeight(this.overflowGuardContainer, layoutInfo.height);
            styleMutator_1.StyleMutator.setWidth(this.linesContent, 1000000);
            styleMutator_1.StyleMutator.setHeight(this.linesContent, 1000000);
            styleMutator_1.StyleMutator.setLeft(this.linesContentContainer, layoutInfo.contentLeft);
            styleMutator_1.StyleMutator.setWidth(this.linesContentContainer, layoutInfo.contentWidth);
            styleMutator_1.StyleMutator.setHeight(this.linesContentContainer, layoutInfo.contentHeight);
            this.outgoingEventBus.emit(editorCommon.EventType.ViewLayoutChanged, layoutInfo);
            return false;
        };
        View.prototype.onConfigurationChanged = function (e) {
            if (e.stylingInfo) {
                configuration_1.Configuration.applyEditorStyling(this.domNode, this.context.configuration.editor.stylingInfo);
            }
            if (e.ariaLabel) {
                this.textArea.setAttribute('aria-label', this.context.configuration.editor.ariaLabel);
            }
            return false;
        };
        View.prototype.onScrollChanged = function (e) {
            this.outgoingEventBus.emit('scroll', {
                scrollTop: this.layoutProvider.getScrollTop(),
                scrollLeft: this.layoutProvider.getScrollLeft()
            });
            return false;
        };
        View.prototype.onScrollHeightChanged = function (scrollHeight) {
            this.outgoingEventBus.emit('scrollSize', {
                scrollWidth: this.layoutProvider.getScrollWidth(),
                scrollHeight: this.layoutProvider.getScrollHeight()
            });
            return _super.prototype.onScrollHeightChanged.call(this, scrollHeight);
        };
        View.prototype.onViewFocusChanged = function (isFocused) {
            dom.toggleClass(this.domNode, 'focused', isFocused);
            if (isFocused) {
                this._editorTextFocusContextKey.set(true);
                this.outgoingEventBus.emit(editorCommon.EventType.ViewFocusGained, {});
            }
            else {
                this._editorTextFocusContextKey.reset();
                this.outgoingEventBus.emit(editorCommon.EventType.ViewFocusLost, {});
            }
            return false;
        };
        // --- end event handlers
        View.prototype.dispose = function () {
            this._isDisposed = true;
            if (this.handleAccumulatedModelEventsTimeout !== -1) {
                clearTimeout(this.handleAccumulatedModelEventsTimeout);
                this.handleAccumulatedModelEventsTimeout = -1;
            }
            if (this._renderAnimationFrame !== null) {
                this._renderAnimationFrame.dispose();
                this._renderAnimationFrame = null;
            }
            this.accumulatedModelEvents = [];
            this.eventDispatcher.removeEventHandler(this);
            this.outgoingEventBus.dispose();
            this.listenersToRemove.forEach(function (element) {
                element();
            });
            this.listenersToRemove = [];
            this.listenersToDispose = lifecycle_1.disposeAll(this.listenersToDispose);
            this.keyboardHandler.dispose();
            this.pointerHandler.dispose();
            this.viewLines.dispose();
            // Destroy IViewPart second
            for (var i = 0, len = this.viewParts.length; i < len; i++) {
                this.viewParts[i].dispose();
            }
            this.viewParts = [];
            this.layoutProvider.dispose();
            this._keybindingService.dispose();
        };
        View.prototype.getCodeEditorHelper = function () {
            var _this = this;
            if (!this.codeEditorHelper) {
                this.codeEditorHelper = {
                    getScrollTop: function () {
                        if (_this._isDisposed) {
                            throw new Error('ViewImpl.codeEditorHelper.getScrollTop: View is disposed');
                        }
                        return _this.layoutProvider.getScrollTop();
                    },
                    setScrollTop: function (scrollTop) {
                        if (_this._isDisposed) {
                            throw new Error('ViewImpl.codeEditorHelper.setScrollTop: View is disposed');
                        }
                        _this.layoutProvider.setScrollTop(scrollTop);
                    },
                    getScrollLeft: function () {
                        if (_this._isDisposed) {
                            throw new Error('ViewImpl.codeEditorHelper.getScrollLeft: View is disposed');
                        }
                        return _this.layoutProvider.getScrollLeft();
                    },
                    setScrollLeft: function (scrollLeft) {
                        if (_this._isDisposed) {
                            throw new Error('ViewImpl.codeEditorHelper.setScrollLeft: View is disposed');
                        }
                        _this.layoutProvider.setScrollLeft(scrollLeft);
                    },
                    getScrollHeight: function () {
                        if (_this._isDisposed) {
                            throw new Error('ViewImpl.codeEditorHelper.getScrollHeight: View is disposed');
                        }
                        return _this.layoutProvider.getScrollHeight();
                    },
                    getScrollWidth: function () {
                        if (_this._isDisposed) {
                            throw new Error('ViewImpl.codeEditorHelper.getScrollWidth: View is disposed');
                        }
                        return _this.layoutProvider.getScrollWidth();
                    },
                    getVerticalOffsetForPosition: function (modelLineNumber, modelColumn) {
                        if (_this._isDisposed) {
                            throw new Error('ViewImpl.codeEditorHelper.getVerticalOffsetForPosition: View is disposed');
                        }
                        var modelPosition = _this.context.model.validateModelPosition({
                            lineNumber: modelLineNumber,
                            column: modelColumn
                        });
                        var viewPosition = _this.context.model.convertModelPositionToViewPosition(modelPosition.lineNumber, modelPosition.column);
                        return _this.layoutProvider.getVerticalOffsetForLineNumber(viewPosition.lineNumber);
                    },
                    delegateVerticalScrollbarMouseDown: function (browserEvent) {
                        if (_this._isDisposed) {
                            throw new Error('ViewImpl.codeEditorHelper.delegateVerticalScrollbarMouseDown: View is disposed');
                        }
                        _this.layoutProvider.delegateVerticalScrollbarMouseDown(browserEvent);
                    },
                    getOffsetForColumn: function (modelLineNumber, modelColumn) {
                        if (_this._isDisposed) {
                            throw new Error('ViewImpl.codeEditorHelper.getOffsetForColumn: View is disposed');
                        }
                        var modelPosition = _this.context.model.validateModelPosition({
                            lineNumber: modelLineNumber,
                            column: modelColumn
                        });
                        var viewPosition = _this.context.model.convertModelPositionToViewPosition(modelPosition.lineNumber, modelPosition.column);
                        _this._flushAccumulatedAndRenderNow();
                        var visibleRanges = _this.viewLines.visibleRangesForRange2(new range_1.Range(viewPosition.lineNumber, viewPosition.column, viewPosition.lineNumber, viewPosition.column), 0);
                        if (!visibleRanges) {
                            return -1;
                        }
                        return visibleRanges[0].left;
                    }
                };
            }
            return this.codeEditorHelper;
        };
        View.prototype.getCenteredRangeInViewport = function () {
            if (this._isDisposed) {
                throw new Error('ViewImpl.getCenteredRangeInViewport: View is disposed');
            }
            var viewLineNumber = this.layoutProvider.getCenteredViewLineNumberInViewport();
            var viewModel = this.context.model;
            var currentCenteredViewRange = new range_1.Range(viewLineNumber, 1, viewLineNumber, viewModel.getLineMaxColumn(viewLineNumber));
            return viewModel.convertViewRangeToModelRange(currentCenteredViewRange);
        };
        //	public getLineInfoProvider():view.ILineInfoProvider {
        //		return this.viewLines;
        //	}
        View.prototype.getInternalEventBus = function () {
            if (this._isDisposed) {
                throw new Error('ViewImpl.getInternalEventBus: View is disposed');
            }
            return this.outgoingEventBus;
        };
        View.prototype.saveState = function () {
            if (this._isDisposed) {
                throw new Error('ViewImpl.saveState: View is disposed');
            }
            return this.layoutProvider.saveState();
        };
        View.prototype.restoreState = function (state) {
            if (this._isDisposed) {
                throw new Error('ViewImpl.restoreState: View is disposed');
            }
            this._flushAnyAccumulatedEvents();
            return this.layoutProvider.restoreState(state);
        };
        View.prototype.focus = function () {
            if (this._isDisposed) {
                throw new Error('ViewImpl.focus: View is disposed');
            }
            this.keyboardHandler.focusTextArea();
            // IE does not trigger the focus event immediately, so we must help it a little bit
            this._setHasFocus(true);
        };
        View.prototype.isFocused = function () {
            if (this._isDisposed) {
                throw new Error('ViewImpl.isFocused: View is disposed');
            }
            return this.hasFocus;
        };
        View.prototype.createOverviewRuler = function (cssClassName, minimumHeight, maximumHeight) {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('ViewImpl.createOverviewRuler: View is disposed');
            }
            return new overviewRuler_1.OverviewRuler(this.context, cssClassName, this.layoutProvider.getScrollHeight(), minimumHeight, maximumHeight, function (lineNumber) { return _this.layoutProvider.getVerticalOffsetForLineNumber(lineNumber); });
        };
        View.prototype.change = function (callback) {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('ViewImpl.change: View is disposed');
            }
            var zonesHaveChanged = false;
            this._renderOnce(function () {
                // Handle events to avoid "adjusting" newly inserted view zones
                _this._flushAnyAccumulatedEvents();
                var changeAccessor = {
                    addZone: function (zone) {
                        zonesHaveChanged = true;
                        return _this.viewZones.addZone(zone);
                    },
                    removeZone: function (id) {
                        zonesHaveChanged = _this.viewZones.removeZone(id) || zonesHaveChanged;
                    },
                    layoutZone: function (id) {
                        zonesHaveChanged = _this.viewZones.layoutZone(id) || zonesHaveChanged;
                    }
                };
                var r = null;
                try {
                    r = callback(changeAccessor);
                }
                catch (e) {
                    errors_1.onUnexpectedError(e);
                }
                // Invalidate changeAccessor
                changeAccessor.addZone = null;
                changeAccessor.removeZone = null;
                if (zonesHaveChanged) {
                    _this.context.privateViewEventBus.emit(editorCommon.EventType.ViewZonesChanged, null);
                }
                return r;
            });
            return zonesHaveChanged;
        };
        View.prototype.getWhitespaces = function () {
            if (this._isDisposed) {
                throw new Error('ViewImpl.getWhitespaces: View is disposed');
            }
            return this.layoutProvider.getWhitespaces();
        };
        View.prototype.addContentWidget = function (widgetData) {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('ViewImpl.addContentWidget: View is disposed');
            }
            this._renderOnce(function () {
                _this.contentWidgets.addWidget(widgetData.widget);
                _this.layoutContentWidget(widgetData);
            });
        };
        View.prototype.layoutContentWidget = function (widgetData) {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('ViewImpl.layoutContentWidget: View is disposed');
            }
            this._renderOnce(function () {
                var newPosition = widgetData.position ? widgetData.position.position : null;
                var newPreference = widgetData.position ? widgetData.position.preference : null;
                _this.contentWidgets.setWidgetPosition(widgetData.widget, newPosition, newPreference);
            });
        };
        View.prototype.removeContentWidget = function (widgetData) {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('ViewImpl.removeContentWidget: View is disposed');
            }
            this._renderOnce(function () {
                _this.contentWidgets.removeWidget(widgetData.widget);
            });
        };
        View.prototype.addOverlayWidget = function (widgetData) {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('ViewImpl.addOverlayWidget: View is disposed');
            }
            this._renderOnce(function () {
                _this.overlayWidgets.addWidget(widgetData.widget);
                _this.layoutOverlayWidget(widgetData);
            });
        };
        View.prototype.layoutOverlayWidget = function (widgetData) {
            if (this._isDisposed) {
                throw new Error('ViewImpl.layoutOverlayWidget: View is disposed');
            }
            var newPreference = widgetData.position ? widgetData.position.preference : null;
            var shouldRender = this.overlayWidgets.setWidgetPosition(widgetData.widget, newPreference);
            if (shouldRender) {
                this._scheduleRender();
            }
        };
        View.prototype.removeOverlayWidget = function (widgetData) {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('ViewImpl.removeOverlayWidget: View is disposed');
            }
            this._renderOnce(function () {
                _this.overlayWidgets.removeWidget(widgetData.widget);
            });
        };
        View.prototype.render = function (now, everything) {
            if (this._isDisposed) {
                throw new Error('ViewImpl.render: View is disposed');
            }
            if (everything) {
                // Force a render with a layout event
                this.layoutProvider.emitLayoutChangedEvent();
            }
            if (now) {
                this._flushAccumulatedAndRenderNow();
            }
        };
        View.prototype.renderOnce = function (callback) {
            if (this._isDisposed) {
                throw new Error('ViewImpl.renderOnce: View is disposed');
            }
            return this._renderOnce(callback);
        };
        // --- end Code Editor APIs
        View.prototype._renderOnce = function (callback) {
            var _this = this;
            if (this._isDisposed) {
                throw new Error('ViewImpl._renderOnce: View is disposed');
            }
            return this.outgoingEventBus.deferredEmit(function () {
                try {
                    var r = callback ? callback() : null;
                }
                finally {
                    _this._scheduleRender();
                }
                return r;
            });
        };
        View.prototype._scheduleRender = function () {
            if (this._isDisposed) {
                throw new Error('ViewImpl._scheduleRender: View is disposed');
            }
            if (this._renderAnimationFrame === null) {
                this._renderAnimationFrame = dom.runAtThisOrScheduleAtNextAnimationFrame(this._onRenderScheduled.bind(this), 100);
            }
        };
        View.prototype._onRenderScheduled = function () {
            this._renderAnimationFrame = null;
            this._flushAccumulatedAndRenderNow();
        };
        View.prototype._renderNow = function () {
            if (this._isDisposed) {
                throw new Error('ViewImpl._renderNow: View is disposed');
            }
            this.actualRender();
        };
        View.prototype.createRenderingContext = function (linesViewportData) {
            var _this = this;
            var vInfo = this.layoutProvider.getCurrentViewport();
            var deltaTop = linesViewportData.visibleRangesDeltaTop;
            var r = {
                linesViewportData: linesViewportData,
                scrollWidth: this.layoutProvider.getScrollWidth(),
                scrollHeight: this.layoutProvider.getScrollHeight(),
                visibleRange: linesViewportData.visibleRange,
                bigNumbersDelta: linesViewportData.bigNumbersDelta,
                viewportWidth: vInfo.width,
                viewportHeight: vInfo.height,
                viewportLeft: vInfo.left,
                viewportTop: vInfo.top,
                getScrolledTopFromAbsoluteTop: function (absoluteTop) {
                    return _this.layoutProvider.getScrolledTopFromAbsoluteTop(absoluteTop);
                },
                getViewportVerticalOffsetForLineNumber: function (lineNumber) {
                    var verticalOffset = _this.layoutProvider.getVerticalOffsetForLineNumber(lineNumber);
                    var scrolledTop = _this.layoutProvider.getScrolledTopFromAbsoluteTop(verticalOffset);
                    return scrolledTop;
                },
                getDecorationsInViewport: function () { return linesViewportData.getDecorationsInViewport(); },
                linesVisibleRangesForRange: function (range, includeNewLines) {
                    return _this.viewLines.linesVisibleRangesForRange(range, includeNewLines);
                },
                visibleRangeForPosition: function (position) {
                    var visibleRanges = _this.viewLines.visibleRangesForRange2(new range_1.Range(position.lineNumber, position.column, position.lineNumber, position.column), deltaTop);
                    if (!visibleRanges) {
                        return null;
                    }
                    return visibleRanges[0];
                },
                lineIsVisible: function (lineNumber) {
                    return linesViewportData.visibleRange.startLineNumber <= lineNumber && lineNumber <= linesViewportData.visibleRange.endLineNumber;
                }
            };
            return r;
        };
        View.prototype.actualRender = function () {
            if (this._isDisposed) {
                throw new Error('ViewImpl.actualRender: View is disposed');
            }
            if (!dom.isInDOM(this.domNode)) {
                return;
            }
            var t = timer.start(timer.Topic.EDITOR, 'View.render');
            var i, len;
            try {
                for (i = 0, len = this.viewParts.length; i < len; i++) {
                    this.viewParts[i].onBeforeForcedLayout();
                }
                var linesViewportData = this.viewLines.render();
                var renderingContext = this.createRenderingContext(linesViewportData);
                // Render the rest of the parts
                for (i = 0, len = this.viewParts.length; i < len; i++) {
                    this.viewParts[i].onReadAfterForcedLayout(renderingContext);
                }
                for (i = 0, len = this.viewParts.length; i < len; i++) {
                    this.viewParts[i].onWriteAfterForcedLayout();
                }
            }
            catch (err) {
                errors_1.onUnexpectedError(err);
            }
            t.stop();
        };
        View.prototype._setHasFocus = function (newHasFocus) {
            if (this.hasFocus !== newHasFocus) {
                this.hasFocus = newHasFocus;
                this.context.privateViewEventBus.emit(editorCommon.EventType.ViewFocusChanged, this.hasFocus);
            }
        };
        return View;
    }(viewEventHandler_1.ViewEventHandler));
    exports.View = View;
    var ViewContext = (function () {
        function ViewContext(editorId, configuration, model, privateViewEventBus, addEventHandler, removeEventHandler) {
            this.editorId = editorId;
            this.configuration = configuration;
            this.model = model;
            this.privateViewEventBus = privateViewEventBus;
            this.addEventHandler = addEventHandler;
            this.removeEventHandler = removeEventHandler;
        }
        return ViewContext;
    }());
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/browser/widget/codeEditorWidget", ["require", "exports", 'vs/base/common/errors', 'vs/base/browser/browser', 'vs/base/browser/dom', 'vs/platform/instantiation/common/instantiation', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/telemetry/common/telemetry', 'vs/editor/common/commonCodeEditor', 'vs/editor/common/core/range', 'vs/editor/common/core/selection', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/services/codeEditorService', 'vs/editor/browser/config/configuration', 'vs/editor/browser/editorBrowserExtensions', 'vs/editor/browser/standalone/colorizer', 'vs/editor/browser/view/viewImpl', 'vs/css!./media/default-theme', 'vs/css!./media/editor', 'vs/css!./media/tokens'], function (require, exports, errors_1, browser, dom, instantiation_1, keybindingService_1, telemetry_1, commonCodeEditor_1, range_1, selection_1, editorCommon, editorCommonExtensions_1, codeEditorService_1, configuration_1, editorBrowserExtensions_1, colorizer_1, viewImpl_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var CodeEditorWidget = (function (_super) {
        __extends(CodeEditorWidget, _super);
        function CodeEditorWidget(domElement, options, instantiationService, codeEditorService, keybindingService, telemetryService) {
            var _this = this;
            _super.call(this, domElement, options, instantiationService, codeEditorService, keybindingService, telemetryService);
            // track focus of the domElement and all its anchestors
            this.focusTracker = dom.trackFocus(this.domElement);
            this.focusTracker.addFocusListener(function () {
                if (_this.forcedWidgetFocusCount === 0) {
                    _this._editorFocusContextKey.set(true);
                    _this.emit(editorCommon.EventType.EditorFocus, {});
                }
            });
            this.focusTracker.addBlurListener(function () {
                if (_this.forcedWidgetFocusCount === 0) {
                    _this._editorFocusContextKey.reset();
                    _this.emit(editorCommon.EventType.EditorBlur, {});
                }
            });
            this.contentWidgets = {};
            this.overlayWidgets = {};
            var contributionDescriptors = [].concat(editorBrowserExtensions_1.EditorBrowserRegistry.getEditorContributions()).concat(editorCommonExtensions_1.CommonEditorRegistry.getEditorContributions());
            for (var i = 0, len = contributionDescriptors.length; i < len; i++) {
                try {
                    var contribution = contributionDescriptors[i].createInstance(this._instantiationService, this);
                    this.contributions[contribution.getId()] = contribution;
                }
                catch (err) {
                    console.error('Could not instantiate contribution ' + contribution.getId());
                    errors_1.onUnexpectedError(err);
                }
            }
        }
        CodeEditorWidget.prototype._createConfiguration = function (options) {
            return new configuration_1.Configuration(options, this.domElement);
        };
        CodeEditorWidget.prototype.dispose = function () {
            this.contentWidgets = {};
            this.overlayWidgets = {};
            this.focusTracker.dispose();
            _super.prototype.dispose.call(this);
        };
        CodeEditorWidget.prototype.colorizeModelLine = function (lineNumber, model) {
            if (model === void 0) { model = this.model; }
            if (!model) {
                return '';
            }
            var content = model.getLineContent(lineNumber);
            var tokens = model.getLineTokens(lineNumber, false);
            var inflatedTokens = editorCommon.LineTokensBinaryEncoding.inflateArr(tokens.getBinaryEncodedTokensMap(), tokens.getBinaryEncodedTokens());
            var tabSize = model.getOptions().tabSize;
            return colorizer_1.Colorizer.colorizeLine(content, inflatedTokens, tabSize);
        };
        CodeEditorWidget.prototype.getView = function () {
            return this._view;
        };
        CodeEditorWidget.prototype.getDomNode = function () {
            if (!this.hasView) {
                return null;
            }
            return this._view.domNode;
        };
        CodeEditorWidget.prototype.getCenteredRangeInViewport = function () {
            if (!this.hasView) {
                return null;
            }
            return this._view.getCenteredRangeInViewport();
        };
        CodeEditorWidget.prototype.setScrollTop = function (newScrollTop) {
            if (!this.hasView) {
                return;
            }
            if (typeof newScrollTop !== 'number') {
                throw new Error('Invalid arguments');
            }
            this._view.getCodeEditorHelper().setScrollTop(newScrollTop);
        };
        CodeEditorWidget.prototype.getScrollTop = function () {
            if (!this.hasView) {
                return -1;
            }
            return this._view.getCodeEditorHelper().getScrollTop();
        };
        CodeEditorWidget.prototype.delegateVerticalScrollbarMouseDown = function (browserEvent) {
            if (!this.hasView) {
                return;
            }
            this._view.getCodeEditorHelper().delegateVerticalScrollbarMouseDown(browserEvent);
        };
        CodeEditorWidget.prototype.setScrollLeft = function (newScrollLeft) {
            if (!this.hasView) {
                return;
            }
            if (typeof newScrollLeft !== 'number') {
                throw new Error('Invalid arguments');
            }
            this._view.getCodeEditorHelper().setScrollLeft(newScrollLeft);
        };
        CodeEditorWidget.prototype.getScrollLeft = function () {
            if (!this.hasView) {
                return -1;
            }
            return this._view.getCodeEditorHelper().getScrollLeft();
        };
        CodeEditorWidget.prototype.getScrollWidth = function () {
            if (!this.hasView) {
                return -1;
            }
            return this._view.getCodeEditorHelper().getScrollWidth();
        };
        CodeEditorWidget.prototype.getScrollHeight = function () {
            if (!this.hasView) {
                return -1;
            }
            return this._view.getCodeEditorHelper().getScrollHeight();
        };
        CodeEditorWidget.prototype.saveViewState = function () {
            if (!this.cursor || !this.hasView) {
                return null;
            }
            var contributionsState = {};
            for (var id in this.contributions) {
                var contribution = this.contributions[id];
                if (typeof contribution.saveViewState === 'function') {
                    contributionsState[id] = contribution.saveViewState();
                }
            }
            var cursorState = this.cursor.saveState();
            var viewState = this._view.saveState();
            return {
                cursorState: cursorState,
                viewState: viewState,
                contributionsState: contributionsState
            };
        };
        CodeEditorWidget.prototype.restoreViewState = function (state) {
            if (!this.cursor || !this.hasView) {
                return;
            }
            var s = state;
            if (s && s.cursorState && s.viewState) {
                var codeEditorState = s;
                var cursorState = codeEditorState.cursorState;
                if (Array.isArray(cursorState)) {
                    this.cursor.restoreState(cursorState);
                }
                else {
                    // Backwards compatibility
                    this.cursor.restoreState([cursorState]);
                }
                this._view.restoreState(codeEditorState.viewState);
                var contributionsState = s.contributionsState || {};
                for (var id in this.contributions) {
                    var contribution = this.contributions[id];
                    if (typeof contribution.restoreViewState === 'function') {
                        contribution.restoreViewState(contributionsState[id]);
                    }
                }
            }
        };
        CodeEditorWidget.prototype.layout = function (dimension) {
            this._configuration.observeReferenceElement(dimension);
            this.render();
        };
        CodeEditorWidget.prototype.focus = function () {
            if (!this.hasView) {
                return;
            }
            this._view.focus();
        };
        CodeEditorWidget.prototype.isFocused = function () {
            return this.hasView && this._view.isFocused();
        };
        CodeEditorWidget.prototype.addContentWidget = function (widget) {
            var widgetData = {
                widget: widget,
                position: widget.getPosition()
            };
            if (this.contentWidgets.hasOwnProperty(widget.getId())) {
                console.warn('Overwriting a content widget with the same id.');
            }
            this.contentWidgets[widget.getId()] = widgetData;
            if (this.hasView) {
                this._view.addContentWidget(widgetData);
            }
        };
        CodeEditorWidget.prototype.layoutContentWidget = function (widget) {
            var widgetId = widget.getId();
            if (this.contentWidgets.hasOwnProperty(widgetId)) {
                var widgetData = this.contentWidgets[widgetId];
                widgetData.position = widget.getPosition();
                if (this.hasView) {
                    this._view.layoutContentWidget(widgetData);
                }
            }
        };
        CodeEditorWidget.prototype.removeContentWidget = function (widget) {
            var widgetId = widget.getId();
            if (this.contentWidgets.hasOwnProperty(widgetId)) {
                var widgetData = this.contentWidgets[widgetId];
                delete this.contentWidgets[widgetId];
                if (this.hasView) {
                    this._view.removeContentWidget(widgetData);
                }
            }
        };
        CodeEditorWidget.prototype.addOverlayWidget = function (widget) {
            var widgetData = {
                widget: widget,
                position: widget.getPosition()
            };
            if (this.overlayWidgets.hasOwnProperty(widget.getId())) {
                console.warn('Overwriting an overlay widget with the same id.');
            }
            this.overlayWidgets[widget.getId()] = widgetData;
            if (this.hasView) {
                this._view.addOverlayWidget(widgetData);
            }
        };
        CodeEditorWidget.prototype.layoutOverlayWidget = function (widget) {
            var widgetId = widget.getId();
            if (this.overlayWidgets.hasOwnProperty(widgetId)) {
                var widgetData = this.overlayWidgets[widgetId];
                widgetData.position = widget.getPosition();
                if (this.hasView) {
                    this._view.layoutOverlayWidget(widgetData);
                }
            }
        };
        CodeEditorWidget.prototype.removeOverlayWidget = function (widget) {
            var widgetId = widget.getId();
            if (this.overlayWidgets.hasOwnProperty(widgetId)) {
                var widgetData = this.overlayWidgets[widgetId];
                delete this.overlayWidgets[widgetId];
                if (this.hasView) {
                    this._view.removeOverlayWidget(widgetData);
                }
            }
        };
        CodeEditorWidget.prototype.changeViewZones = function (callback) {
            if (!this.hasView) {
                //			console.warn('Cannot change view zones on editor that is not attached to a model, since there is no view.');
                return;
            }
            var hasChanges = this._view.change(callback);
            if (hasChanges) {
                this.emit(editorCommon.EventType.ViewZonesChanged);
            }
        };
        CodeEditorWidget.prototype.getWhitespaces = function () {
            if (!this.hasView) {
                return [];
            }
            return this._view.getWhitespaces();
        };
        CodeEditorWidget.prototype.getTopForLineNumber = function (lineNumber) {
            if (!this.hasView) {
                return -1;
            }
            return this._view.getCodeEditorHelper().getVerticalOffsetForPosition(lineNumber, 1);
        };
        CodeEditorWidget.prototype.getTopForPosition = function (lineNumber, column) {
            if (!this.hasView) {
                return -1;
            }
            return this._view.getCodeEditorHelper().getVerticalOffsetForPosition(lineNumber, column);
        };
        CodeEditorWidget.prototype.getScrolledVisiblePosition = function (rawPosition) {
            if (!this.hasView) {
                return null;
            }
            var position = this.model.validatePosition(rawPosition);
            var helper = this._view.getCodeEditorHelper();
            var layoutInfo = this._configuration.editor.layoutInfo;
            var top = helper.getVerticalOffsetForPosition(position.lineNumber, position.column) - helper.getScrollTop();
            var left = helper.getOffsetForColumn(position.lineNumber, position.column) + layoutInfo.glyphMarginWidth + layoutInfo.lineNumbersWidth + layoutInfo.decorationsWidth - helper.getScrollLeft();
            return {
                top: top,
                left: left,
                height: this._configuration.editor.lineHeight
            };
        };
        CodeEditorWidget.prototype.getOffsetForColumn = function (lineNumber, column) {
            if (!this.hasView) {
                return -1;
            }
            return this._view.getCodeEditorHelper().getOffsetForColumn(lineNumber, column);
        };
        CodeEditorWidget.prototype.render = function () {
            if (!this.hasView) {
                return;
            }
            this._view.render(true, false);
        };
        CodeEditorWidget.prototype.setHiddenAreas = function (ranges) {
            if (this.viewModel) {
                this.viewModel.setHiddenAreas(ranges);
            }
        };
        CodeEditorWidget.prototype.setAriaActiveDescendant = function (id) {
            if (!this.hasView) {
                return;
            }
            this._view.setAriaActiveDescendant(id);
        };
        CodeEditorWidget.prototype._attachModel = function (model) {
            var _this = this;
            this._view = null;
            _super.prototype._attachModel.call(this, model);
            if (this._view) {
                this.domElement.appendChild(this._view.domNode);
                this._view.renderOnce(function () {
                    var widgetId;
                    for (widgetId in _this.contentWidgets) {
                        if (_this.contentWidgets.hasOwnProperty(widgetId)) {
                            _this._view.addContentWidget(_this.contentWidgets[widgetId]);
                        }
                    }
                    for (widgetId in _this.overlayWidgets) {
                        if (_this.overlayWidgets.hasOwnProperty(widgetId)) {
                            _this._view.addOverlayWidget(_this.overlayWidgets[widgetId]);
                        }
                    }
                    _this._view.render(false, true);
                    _this.hasView = true;
                });
            }
        };
        CodeEditorWidget.prototype._enableEmptySelectionClipboard = function () {
            return browser.enableEmptySelectionClipboard;
        };
        CodeEditorWidget.prototype._createView = function () {
            this._view = new viewImpl_1.View(this.id, this._configuration, this.viewModel, this._keybindingService);
        };
        CodeEditorWidget.prototype._getViewInternalEventBus = function () {
            return this._view.getInternalEventBus();
        };
        CodeEditorWidget.prototype._detachModel = function () {
            var removeDomNode = null;
            if (this._view) {
                this._view.dispose();
                removeDomNode = this._view.domNode;
                this._view = null;
            }
            var result = _super.prototype._detachModel.call(this);
            if (removeDomNode) {
                this.domElement.removeChild(removeDomNode);
            }
            return result;
        };
        CodeEditorWidget = __decorate([
            __param(2, instantiation_1.IInstantiationService),
            __param(3, codeEditorService_1.ICodeEditorService),
            __param(4, keybindingService_1.IKeybindingService),
            __param(5, telemetry_1.ITelemetryService)
        ], CodeEditorWidget);
        return CodeEditorWidget;
    }(commonCodeEditor_1.CommonCodeEditor));
    exports.CodeEditorWidget = CodeEditorWidget;
    var OverlayWidget2 = (function () {
        function OverlayWidget2(id, position) {
            this._id = id;
            this._position = position;
            this._domNode = document.createElement('div');
            this._domNode.className = this._id.replace(/\./g, '-').replace(/[^a-z0-9\-]/, '');
        }
        OverlayWidget2.prototype.getId = function () {
            return this._id;
        };
        OverlayWidget2.prototype.getDomNode = function () {
            return this._domNode;
        };
        OverlayWidget2.prototype.getPosition = function () {
            return this._position;
        };
        return OverlayWidget2;
    }());
    (function (EditCursorState) {
        EditCursorState[EditCursorState["EndOfLastEditOperation"] = 0] = "EndOfLastEditOperation";
    })(exports.EditCursorState || (exports.EditCursorState = {}));
    var EditCursorState = exports.EditCursorState;
    var CommandRunner = (function () {
        function CommandRunner(ops, editCursorState) {
            this._ops = ops;
            this._editCursorState = editCursorState;
        }
        CommandRunner.prototype.getEditOperations = function (model, builder) {
            if (this._ops.length === 0) {
                return;
            }
            // Sort them in ascending order by range starts
            this._ops.sort(function (o1, o2) {
                return range_1.Range.compareRangesUsingStarts(o1.range, o2.range);
            });
            // Merge operations that touch each other
            var resultOps = [];
            var previousOp = this._ops[0];
            for (var i = 1; i < this._ops.length; i++) {
                if (previousOp.range.endLineNumber === this._ops[i].range.startLineNumber && previousOp.range.endColumn === this._ops[i].range.startColumn) {
                    // These operations are one after another and can be merged
                    previousOp.range = range_1.Range.plusRange(previousOp.range, this._ops[i].range);
                    previousOp.text = previousOp.text + this._ops[i].text;
                }
                else {
                    resultOps.push(previousOp);
                    previousOp = this._ops[i];
                }
            }
            resultOps.push(previousOp);
            for (var i = 0; i < resultOps.length; i++) {
                builder.addEditOperation(range_1.Range.lift(resultOps[i].range), resultOps[i].text);
            }
        };
        CommandRunner.prototype.computeCursorState = function (model, helper) {
            var inverseEditOperations = helper.getInverseEditOperations();
            var srcRange = inverseEditOperations[inverseEditOperations.length - 1].range;
            return selection_1.Selection.createSelection(srcRange.endLineNumber, srcRange.endColumn, srcRange.endLineNumber, srcRange.endColumn);
        };
        return CommandRunner;
    }());
    exports.CommandRunner = CommandRunner;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/browser/widget/diffEditorWidget", ["require", "exports", 'vs/base/common/async', 'vs/base/common/eventEmitter', 'vs/base/common/lifecycle', 'vs/base/common/objects', 'vs/base/browser/dom', 'vs/base/browser/styleMutator', 'vs/base/browser/ui/sash/sash', 'vs/platform/instantiation/common/instantiation', 'vs/editor/common/config/defaultConfig', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/services/editorWorkerService', 'vs/editor/common/viewLayout/viewLineParts', 'vs/editor/common/viewLayout/viewLineRenderer', 'vs/editor/browser/widget/codeEditorWidget', 'vs/css!./media/diffEditor'], function (require, exports, async_1, eventEmitter_1, lifecycle_1, objects, dom, styleMutator_1, sash_1, instantiation_1, defaultConfig_1, range_1, editorCommon, editorWorkerService_1, viewLineParts_1, viewLineRenderer_1, codeEditorWidget_1) {
    'use strict';
    var VisualEditorState = (function () {
        function VisualEditorState() {
            this._zones = [];
            this._zonesMap = {};
            this._decorations = [];
        }
        VisualEditorState.prototype.getForeignViewZones = function (allViewZones) {
            var _this = this;
            return allViewZones.filter(function (z) { return !_this._zonesMap[String(z.id)]; });
        };
        VisualEditorState.prototype.clean = function (editor) {
            var _this = this;
            // (1) View zones
            if (this._zones.length > 0) {
                editor.changeViewZones(function (viewChangeAccessor) {
                    for (var i = 0, length = _this._zones.length; i < length; i++) {
                        viewChangeAccessor.removeZone(_this._zones[i]);
                    }
                });
            }
            this._zones = [];
            this._zonesMap = {};
            // (2) Model decorations
            if (this._decorations.length > 0) {
                editor.changeDecorations(function (changeAccessor) {
                    changeAccessor.deltaDecorations(_this._decorations, []);
                });
            }
            this._decorations = [];
        };
        VisualEditorState.prototype.apply = function (editor, overviewRuler, newDecorations) {
            var _this = this;
            var i, length;
            // view zones
            editor.changeViewZones(function (viewChangeAccessor) {
                for (i = 0, length = _this._zones.length; i < length; i++) {
                    viewChangeAccessor.removeZone(_this._zones[i]);
                }
                _this._zones = [];
                _this._zonesMap = {};
                for (i = 0, length = newDecorations.zones.length; i < length; i++) {
                    newDecorations.zones[i].suppressMouseDown = true;
                    var zoneId = viewChangeAccessor.addZone(newDecorations.zones[i]);
                    _this._zones.push(zoneId);
                    _this._zonesMap[String(zoneId)] = true;
                }
            });
            // decorations
            this._decorations = editor.deltaDecorations(this._decorations, newDecorations.decorations);
            // overview ruler
            overviewRuler.setZones(newDecorations.overviewZones);
        };
        return VisualEditorState;
    }());
    var DIFF_EDITOR_ID = 0;
    var DiffEditorWidget = (function (_super) {
        __extends(DiffEditorWidget, _super);
        function DiffEditorWidget(domElement, options, editorWorkerService, instantiationService) {
            var _this = this;
            _super.call(this);
            this._editorWorkerService = editorWorkerService;
            this.id = (++DIFF_EDITOR_ID);
            this._domElement = domElement;
            options = options || {};
            this._theme = options.theme || defaultConfig_1.DefaultConfig.editor.theme;
            // renderSideBySide
            this._renderSideBySide = true;
            if (typeof options.renderSideBySide !== 'undefined') {
                this._renderSideBySide = options.renderSideBySide;
            }
            // ignoreTrimWhitespace
            this._ignoreTrimWhitespace = true;
            if (typeof options.ignoreTrimWhitespace !== 'undefined') {
                this._ignoreTrimWhitespace = options.ignoreTrimWhitespace;
            }
            this._originalIsEditable = false;
            if (typeof options.originalEditable !== 'undefined') {
                this._originalIsEditable = Boolean(options.originalEditable);
            }
            this._updateDecorationsRunner = new async_1.RunOnceScheduler(function () { return _this._updateDecorations(); }, 0);
            this._toDispose = [];
            this._toDispose.push(this._updateDecorationsRunner);
            this._containerDomElement = document.createElement('div');
            this._containerDomElement.className = DiffEditorWidget._getClassName(this._theme, this._renderSideBySide);
            this._containerDomElement.style.position = 'relative';
            this._containerDomElement.style.height = '100%';
            this._domElement.appendChild(this._containerDomElement);
            this._overviewViewportDomElement = document.createElement('div');
            this._overviewViewportDomElement.className = 'diffViewport';
            this._overviewViewportDomElement.style.position = 'absolute';
            this._overviewDomElement = document.createElement('div');
            this._overviewDomElement.className = 'diffOverview';
            this._overviewDomElement.style.position = 'absolute';
            this._overviewDomElement.style.height = '100%';
            this._overviewDomElement.appendChild(this._overviewViewportDomElement);
            this._toDispose.push(dom.addDisposableListener(this._overviewDomElement, 'mousedown', function (e) {
                _this.modifiedEditor.delegateVerticalScrollbarMouseDown(e);
            }));
            this._containerDomElement.appendChild(this._overviewDomElement);
            this._createLeftHandSide();
            this._createRightHandSide();
            this._beginUpdateDecorationsTimeout = -1;
            this._currentlyChangingViewZones = false;
            this._diffComputationToken = 0;
            this._originalEditorState = new VisualEditorState();
            this._modifiedEditorState = new VisualEditorState();
            this._isVisible = true;
            this._isHandlingScrollEvent = false;
            this._width = 0;
            this._height = 0;
            this._lineChanges = null;
            this._createLeftHandSideEditor(options, instantiationService);
            this._createRightHandSideEditor(options, instantiationService);
            if (options.automaticLayout) {
                this._measureDomElementToken = window.setInterval(function () { return _this._measureDomElement(false); }, 100);
            }
            // enableSplitViewResizing
            this._enableSplitViewResizing = true;
            if (typeof options.enableSplitViewResizing !== 'undefined') {
                this._enableSplitViewResizing = options.enableSplitViewResizing;
            }
            if (this._renderSideBySide) {
                this._setStrategy(new DiffEdtorWidgetSideBySide(this._createDataSource(), this._enableSplitViewResizing));
            }
            else {
                this._setStrategy(new DiffEdtorWidgetInline(this._createDataSource(), this._enableSplitViewResizing));
            }
        }
        Object.defineProperty(DiffEditorWidget.prototype, "ignoreTrimWhitespace", {
            get: function () {
                return this._ignoreTrimWhitespace;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DiffEditorWidget.prototype, "renderSideBySide", {
            get: function () {
                return this._renderSideBySide;
            },
            enumerable: true,
            configurable: true
        });
        DiffEditorWidget._getClassName = function (theme, renderSideBySide) {
            var result = 'monaco-diff-editor monaco-editor-background ';
            if (renderSideBySide) {
                result += 'side-by-side ';
            }
            result += theme;
            return result;
        };
        DiffEditorWidget.prototype._recreateOverviewRulers = function () {
            if (this._originalOverviewRuler) {
                this._overviewDomElement.removeChild(this._originalOverviewRuler.getDomNode());
                this._originalOverviewRuler.dispose();
            }
            this._originalOverviewRuler = this.originalEditor.getView().createOverviewRuler('original diffOverviewRuler', 4, Number.MAX_VALUE);
            this._overviewDomElement.appendChild(this._originalOverviewRuler.getDomNode());
            if (this._modifiedOverviewRuler) {
                this._overviewDomElement.removeChild(this._modifiedOverviewRuler.getDomNode());
                this._modifiedOverviewRuler.dispose();
            }
            this._modifiedOverviewRuler = this.modifiedEditor.getView().createOverviewRuler('modified diffOverviewRuler', 4, Number.MAX_VALUE);
            this._overviewDomElement.appendChild(this._modifiedOverviewRuler.getDomNode());
            this._layoutOverviewRulers();
        };
        DiffEditorWidget.prototype._createLeftHandSide = function () {
            this._originalDomNode = document.createElement('div');
            this._originalDomNode.className = 'editor original';
            this._originalDomNode.style.position = 'absolute';
            this._originalDomNode.style.height = '100%';
            this._containerDomElement.appendChild(this._originalDomNode);
        };
        DiffEditorWidget.prototype._createRightHandSide = function () {
            this._modifiedDomNode = document.createElement('div');
            this._modifiedDomNode.className = 'editor modified';
            this._modifiedDomNode.style.position = 'absolute';
            this._modifiedDomNode.style.height = '100%';
            this._containerDomElement.appendChild(this._modifiedDomNode);
        };
        DiffEditorWidget.prototype._createLeftHandSideEditor = function (options, instantiationService) {
            var _this = this;
            this.originalEditor = instantiationService.createInstance(codeEditorWidget_1.CodeEditorWidget, this._originalDomNode, this._adjustOptionsForLeftHandSide(options, this._originalIsEditable));
            this._toDispose.push(this.originalEditor.addBulkListener2(function (events) { return _this._onOriginalEditorEvents(events); }));
            this._toDispose.push(this.addEmitter2(this.originalEditor, 'leftHandSide'));
        };
        DiffEditorWidget.prototype._createRightHandSideEditor = function (options, instantiationService) {
            var _this = this;
            this.modifiedEditor = instantiationService.createInstance(codeEditorWidget_1.CodeEditorWidget, this._modifiedDomNode, this._adjustOptionsForRightHandSide(options));
            this._toDispose.push(this.modifiedEditor.addBulkListener2(function (events) { return _this._onModifiedEditorEvents(events); }));
            this._toDispose.push(this.addEmitter2(this.modifiedEditor, 'rightHandSide'));
        };
        DiffEditorWidget.prototype.destroy = function () {
            this.dispose();
        };
        DiffEditorWidget.prototype.dispose = function () {
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
            window.clearInterval(this._measureDomElementToken);
            this._cleanViewZonesAndDecorations();
            this._originalOverviewRuler.dispose();
            this._modifiedOverviewRuler.dispose();
            this.originalEditor.destroy();
            this.modifiedEditor.destroy();
            this._strategy.dispose();
            _super.prototype.dispose.call(this);
        };
        //------------ begin IDiffEditor methods
        DiffEditorWidget.prototype.getId = function () {
            return this.getEditorType() + ':' + this.id;
        };
        DiffEditorWidget.prototype.getEditorType = function () {
            return editorCommon.EditorType.IDiffEditor;
        };
        DiffEditorWidget.prototype.getLineChanges = function () {
            return this._lineChanges;
        };
        DiffEditorWidget.prototype.getOriginalEditor = function () {
            return this.originalEditor;
        };
        DiffEditorWidget.prototype.getModifiedEditor = function () {
            return this.modifiedEditor;
        };
        DiffEditorWidget.prototype.updateOptions = function (newOptions) {
            // Handle new theme
            this._theme = newOptions && newOptions.theme ? newOptions.theme : this._theme;
            // Handle side by side
            var renderSideBySideChanged = false;
            if (typeof newOptions.renderSideBySide !== 'undefined') {
                if (this._renderSideBySide !== newOptions.renderSideBySide) {
                    this._renderSideBySide = newOptions.renderSideBySide;
                    renderSideBySideChanged = true;
                }
            }
            if (typeof newOptions.ignoreTrimWhitespace !== 'undefined') {
                if (this._ignoreTrimWhitespace !== newOptions.ignoreTrimWhitespace) {
                    this._ignoreTrimWhitespace = newOptions.ignoreTrimWhitespace;
                    // Begin comparing
                    this._beginUpdateDecorations();
                }
            }
            if (typeof newOptions.originalEditable !== 'undefined') {
                this._originalIsEditable = Boolean(newOptions.originalEditable);
            }
            // Update class name
            this._containerDomElement.className = DiffEditorWidget._getClassName(this._theme, this._renderSideBySide);
            this.modifiedEditor.updateOptions(this._adjustOptionsForRightHandSide(newOptions));
            this.originalEditor.updateOptions(this._adjustOptionsForLeftHandSide(newOptions, this._originalIsEditable));
            // enableSplitViewResizing
            if (typeof newOptions.enableSplitViewResizing !== 'undefined') {
                this._enableSplitViewResizing = newOptions.enableSplitViewResizing;
            }
            this._strategy.setEnableSplitViewResizing(this._enableSplitViewResizing);
            // renderSideBySide
            if (renderSideBySideChanged) {
                if (this._renderSideBySide) {
                    this._setStrategy(new DiffEdtorWidgetSideBySide(this._createDataSource(), this._enableSplitViewResizing));
                }
                else {
                    this._setStrategy(new DiffEdtorWidgetInline(this._createDataSource(), this._enableSplitViewResizing));
                }
            }
        };
        DiffEditorWidget.prototype.getValue = function (options) {
            if (options === void 0) { options = null; }
            return this.modifiedEditor.getValue(options);
        };
        DiffEditorWidget.prototype.getModel = function () {
            return {
                original: this.originalEditor.getModel(),
                modified: this.modifiedEditor.getModel()
            };
        };
        DiffEditorWidget.prototype.setModel = function (model) {
            // Guard us against partial null model
            if (model && (!model.original || !model.modified)) {
                throw new Error(!model.original ? 'DiffEditorWidget.setModel: Original model is null' : 'DiffEditorWidget.setModel: Modified model is null');
            }
            // Remove all view zones & decorations
            this._cleanViewZonesAndDecorations();
            // Update code editor models
            this.originalEditor.setModel(model ? model.original : null);
            this.modifiedEditor.setModel(model ? model.modified : null);
            this._updateDecorationsRunner.cancel();
            if (model) {
                this.originalEditor.setScrollTop(0);
                this.modifiedEditor.setScrollTop(0);
            }
            // Disable any diff computations that will come in
            this._lineChanges = null;
            this._diffComputationToken++;
            if (model) {
                this._recreateOverviewRulers();
                // Begin comparing
                this._beginUpdateDecorations();
            }
            else {
                this._lineChanges = null;
            }
            this._layoutOverviewViewport();
        };
        DiffEditorWidget.prototype.getDomNode = function () {
            return this._domElement;
        };
        DiffEditorWidget.prototype.getVisibleColumnFromPosition = function (position) {
            return this.modifiedEditor.getVisibleColumnFromPosition(position);
        };
        DiffEditorWidget.prototype.getPosition = function () {
            return this.modifiedEditor.getPosition();
        };
        DiffEditorWidget.prototype.setPosition = function (position, reveal, revealVerticalInCenter, revealHorizontal) {
            this.modifiedEditor.setPosition(position, reveal, revealVerticalInCenter, revealHorizontal);
        };
        DiffEditorWidget.prototype.revealLine = function (lineNumber) {
            this.modifiedEditor.revealLine(lineNumber);
        };
        DiffEditorWidget.prototype.revealLineInCenter = function (lineNumber) {
            this.modifiedEditor.revealLineInCenter(lineNumber);
        };
        DiffEditorWidget.prototype.revealLineInCenterIfOutsideViewport = function (lineNumber) {
            this.modifiedEditor.revealLineInCenterIfOutsideViewport(lineNumber);
        };
        DiffEditorWidget.prototype.revealPosition = function (position, revealVerticalInCenter, revealHorizontal) {
            if (revealVerticalInCenter === void 0) { revealVerticalInCenter = false; }
            if (revealHorizontal === void 0) { revealHorizontal = false; }
            this.modifiedEditor.revealPosition(position, revealVerticalInCenter, revealHorizontal);
        };
        DiffEditorWidget.prototype.revealPositionInCenter = function (position) {
            this.modifiedEditor.revealPositionInCenter(position);
        };
        DiffEditorWidget.prototype.revealPositionInCenterIfOutsideViewport = function (position) {
            this.modifiedEditor.revealPositionInCenterIfOutsideViewport(position);
        };
        DiffEditorWidget.prototype.getSelection = function () {
            return this.modifiedEditor.getSelection();
        };
        DiffEditorWidget.prototype.getSelections = function () {
            return this.modifiedEditor.getSelections();
        };
        DiffEditorWidget.prototype.setSelection = function (something, reveal, revealVerticalInCenter, revealHorizontal) {
            this.modifiedEditor.setSelection(something, reveal, revealVerticalInCenter, revealHorizontal);
        };
        DiffEditorWidget.prototype.setSelections = function (ranges) {
            this.modifiedEditor.setSelections(ranges);
        };
        DiffEditorWidget.prototype.revealLines = function (startLineNumber, endLineNumber) {
            this.modifiedEditor.revealLines(startLineNumber, endLineNumber);
        };
        DiffEditorWidget.prototype.revealLinesInCenter = function (startLineNumber, endLineNumber) {
            this.modifiedEditor.revealLinesInCenter(startLineNumber, endLineNumber);
        };
        DiffEditorWidget.prototype.revealLinesInCenterIfOutsideViewport = function (startLineNumber, endLineNumber) {
            this.modifiedEditor.revealLinesInCenterIfOutsideViewport(startLineNumber, endLineNumber);
        };
        DiffEditorWidget.prototype.revealRange = function (range, revealVerticalInCenter, revealHorizontal) {
            if (revealVerticalInCenter === void 0) { revealVerticalInCenter = false; }
            if (revealHorizontal === void 0) { revealHorizontal = true; }
            this.modifiedEditor.revealRange(range, revealVerticalInCenter, revealHorizontal);
        };
        DiffEditorWidget.prototype.revealRangeInCenter = function (range) {
            this.modifiedEditor.revealRangeInCenter(range);
        };
        DiffEditorWidget.prototype.revealRangeInCenterIfOutsideViewport = function (range) {
            this.modifiedEditor.revealRangeInCenterIfOutsideViewport(range);
        };
        DiffEditorWidget.prototype.addAction = function (descriptor) {
            this.modifiedEditor.addAction(descriptor);
        };
        DiffEditorWidget.prototype.getActions = function () {
            return this.modifiedEditor.getActions();
        };
        DiffEditorWidget.prototype.getAction = function (id) {
            return this.modifiedEditor.getAction(id);
        };
        DiffEditorWidget.prototype.saveViewState = function () {
            var originalViewState = this.originalEditor.saveViewState();
            var modifiedViewState = this.modifiedEditor.saveViewState();
            return {
                original: originalViewState,
                modified: modifiedViewState
            };
        };
        DiffEditorWidget.prototype.restoreViewState = function (state) {
            var s = state;
            if (s.original && s.original) {
                var diffEditorState = s;
                this.originalEditor.restoreViewState(diffEditorState.original);
                this.modifiedEditor.restoreViewState(diffEditorState.modified);
            }
        };
        DiffEditorWidget.prototype.layout = function (dimension) {
            this._measureDomElement(false, dimension);
        };
        DiffEditorWidget.prototype.focus = function () {
            this.modifiedEditor.focus();
        };
        DiffEditorWidget.prototype.isFocused = function () {
            return this.originalEditor.isFocused() || this.modifiedEditor.isFocused();
        };
        DiffEditorWidget.prototype.onVisible = function () {
            this._isVisible = true;
            this.originalEditor.onVisible();
            this.modifiedEditor.onVisible();
            // Begin comparing
            this._beginUpdateDecorations();
        };
        DiffEditorWidget.prototype.onHide = function () {
            this._isVisible = false;
            this.originalEditor.onHide();
            this.modifiedEditor.onHide();
            // Remove all view zones & decorations
            this._cleanViewZonesAndDecorations();
        };
        DiffEditorWidget.prototype.trigger = function (source, handlerId, payload) {
            this.modifiedEditor.trigger(source, handlerId, payload);
        };
        DiffEditorWidget.prototype.changeDecorations = function (callback) {
            return this.modifiedEditor.changeDecorations(callback);
        };
        //------------ end IDiffEditor methods
        //------------ begin layouting methods
        DiffEditorWidget.prototype._measureDomElement = function (forceDoLayoutCall, dimensions) {
            dimensions = dimensions || dom.getDomNodePosition(this._containerDomElement);
            if (dimensions.width <= 0) {
                this._width = 0;
                this._height = 0;
                return;
            }
            if (!forceDoLayoutCall && dimensions.width === this._width && dimensions.height === this._height) {
                // Nothing has changed
                return;
            }
            this._width = dimensions.width;
            this._height = dimensions.height;
            this._doLayout();
        };
        DiffEditorWidget.prototype._layoutOverviewRulers = function () {
            var freeSpace = DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH - 2 * DiffEditorWidget.ONE_OVERVIEW_WIDTH;
            var layoutInfo = this.modifiedEditor.getLayoutInfo();
            if (layoutInfo) {
                this._originalOverviewRuler.setLayout({
                    top: 0,
                    width: DiffEditorWidget.ONE_OVERVIEW_WIDTH,
                    right: freeSpace + DiffEditorWidget.ONE_OVERVIEW_WIDTH,
                    height: this._height
                });
                this._modifiedOverviewRuler.setLayout({
                    top: 0,
                    right: 0,
                    width: DiffEditorWidget.ONE_OVERVIEW_WIDTH,
                    height: this._height
                });
            }
        };
        //------------ end layouting methods
        DiffEditorWidget.prototype._recomputeIfNecessary = function (events) {
            var _this = this;
            var changed = false;
            for (var i = 0; !changed && i < events.length; i++) {
                var type = events[i].getType();
                changed = changed || type === 'change' || type === editorCommon.EventType.ModelModeChanged;
            }
            if (changed && this._isVisible) {
                // Clear previous timeout if necessary
                if (this._beginUpdateDecorationsTimeout !== -1) {
                    window.clearTimeout(this._beginUpdateDecorationsTimeout);
                    this._beginUpdateDecorationsTimeout = -1;
                }
                this._beginUpdateDecorationsTimeout = window.setTimeout(function () { return _this._beginUpdateDecorations(); }, DiffEditorWidget.UPDATE_DIFF_DECORATIONS_DELAY);
            }
        };
        DiffEditorWidget.prototype._onOriginalEditorEvents = function (events) {
            for (var i = 0; i < events.length; i++) {
                if (events[i].getType() === 'scroll') {
                    this._onOriginalEditorScroll(events[i].getData());
                }
                if (events[i].getType() === editorCommon.EventType.ViewZonesChanged) {
                    this._onViewZonesChanged();
                }
            }
            this._recomputeIfNecessary(events);
        };
        DiffEditorWidget.prototype._onModifiedEditorEvents = function (events) {
            for (var i = 0; i < events.length; i++) {
                if (events[i].getType() === 'scroll') {
                    this._onModifiedEditorScroll(events[i].getData());
                    this._layoutOverviewViewport();
                }
                if (events[i].getType() === 'scrollSize') {
                    this._layoutOverviewViewport();
                }
                if (events[i].getType() === 'viewLayoutChanged') {
                    this._layoutOverviewViewport();
                }
                if (events[i].getType() === editorCommon.EventType.ViewZonesChanged) {
                    this._onViewZonesChanged();
                }
            }
            this._recomputeIfNecessary(events);
        };
        DiffEditorWidget.prototype._onViewZonesChanged = function () {
            if (this._currentlyChangingViewZones) {
                return;
            }
            this._updateDecorationsRunner.schedule();
        };
        DiffEditorWidget.prototype._beginUpdateDecorations = function () {
            var _this = this;
            this._beginUpdateDecorationsTimeout = -1;
            if (!this.modifiedEditor.getModel()) {
                return;
            }
            // Prevent old diff requests to come if a new request has been initiated
            // The best method would be to call cancel on the Promise, but this is not
            // yet supported, so using tokens for now.
            this._diffComputationToken++;
            var currentToken = this._diffComputationToken;
            var currentOriginalModel = this.originalEditor.getModel();
            var currentModifiedModel = this.modifiedEditor.getModel();
            this._editorWorkerService.computeDiff(currentOriginalModel.getAssociatedResource(), currentModifiedModel.getAssociatedResource(), this._ignoreTrimWhitespace).then(function (result) {
                if (currentToken === _this._diffComputationToken
                    && currentOriginalModel === _this.originalEditor.getModel()
                    && currentModifiedModel === _this.modifiedEditor.getModel()) {
                    _this._lineChanges = result;
                    _this._updateDecorationsRunner.schedule();
                    _this.emit(editorCommon.EventType.DiffUpdated, { editor: _this, lineChanges: result });
                }
            }, function (error) {
                if (currentToken === _this._diffComputationToken
                    && currentOriginalModel === _this.originalEditor.getModel()
                    && currentModifiedModel === _this.modifiedEditor.getModel()) {
                    _this._lineChanges = null;
                    _this._updateDecorationsRunner.schedule();
                }
            });
        };
        DiffEditorWidget.prototype._cleanViewZonesAndDecorations = function () {
            this._originalEditorState.clean(this.originalEditor);
            this._modifiedEditorState.clean(this.modifiedEditor);
        };
        DiffEditorWidget.prototype._updateDecorations = function () {
            var lineChanges = this._lineChanges || [];
            var foreignOriginal = this._originalEditorState.getForeignViewZones(this.originalEditor.getWhitespaces());
            var foreignModified = this._modifiedEditorState.getForeignViewZones(this.modifiedEditor.getWhitespaces());
            var diffDecorations = this._strategy.getEditorsDiffDecorations(lineChanges, this._ignoreTrimWhitespace, foreignOriginal, foreignModified, this.originalEditor, this.modifiedEditor);
            try {
                this._currentlyChangingViewZones = true;
                this._originalEditorState.apply(this.originalEditor, this._originalOverviewRuler, diffDecorations.original);
                this._modifiedEditorState.apply(this.modifiedEditor, this._modifiedOverviewRuler, diffDecorations.modified);
            }
            finally {
                this._currentlyChangingViewZones = false;
            }
        };
        DiffEditorWidget.prototype._adjustOptionsForSubEditor = function (options) {
            var clonedOptions = objects.clone(options || {});
            clonedOptions.wrappingColumn = -1;
            clonedOptions.automaticLayout = false;
            clonedOptions.scrollbar = clonedOptions.scrollbar || {};
            clonedOptions.scrollbar.vertical = 'visible';
            clonedOptions.folding = false;
            clonedOptions.referenceInfos = false;
            return clonedOptions;
        };
        DiffEditorWidget.prototype._adjustOptionsForLeftHandSide = function (options, isEditable) {
            var result = this._adjustOptionsForSubEditor(options);
            result.readOnly = !isEditable;
            result.overviewRulerLanes = 1;
            result.theme = this._theme + ' original-in-monaco-diff-editor';
            return result;
        };
        DiffEditorWidget.prototype._adjustOptionsForRightHandSide = function (options) {
            var result = this._adjustOptionsForSubEditor(options);
            result.revealHorizontalRightPadding = defaultConfig_1.DefaultConfig.editor.revealHorizontalRightPadding + DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH;
            result.scrollbar.verticalHasArrows = false;
            result.theme = this._theme + ' modified-in-monaco-diff-editor';
            return result;
        };
        DiffEditorWidget.prototype._onOriginalEditorScroll = function (e) {
            if (this._isHandlingScrollEvent) {
                return;
            }
            this._isHandlingScrollEvent = true;
            this.modifiedEditor.setScrollLeft(e.scrollLeft);
            this.modifiedEditor.setScrollTop(e.scrollTop);
            this._isHandlingScrollEvent = false;
        };
        DiffEditorWidget.prototype._onModifiedEditorScroll = function (e) {
            if (this._isHandlingScrollEvent) {
                return;
            }
            this._isHandlingScrollEvent = true;
            this.originalEditor.setScrollLeft(e.scrollLeft);
            this.originalEditor.setScrollTop(e.scrollTop);
            this._isHandlingScrollEvent = false;
        };
        DiffEditorWidget.prototype._doLayout = function () {
            var splitPoint = this._strategy.layout();
            this._originalDomNode.style.width = splitPoint + 'px';
            this._originalDomNode.style.left = '0px';
            this._modifiedDomNode.style.width = (this._width - splitPoint) + 'px';
            this._modifiedDomNode.style.left = splitPoint + 'px';
            this._overviewDomElement.style.top = '0px';
            this._overviewDomElement.style.width = DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH + 'px';
            this._overviewDomElement.style.left = (this._width - DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH) + 'px';
            this._overviewViewportDomElement.style.width = DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH + 'px';
            this._overviewViewportDomElement.style.height = '30px';
            this.originalEditor.layout({ width: splitPoint, height: this._height });
            this.modifiedEditor.layout({ width: this._width - splitPoint - DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH, height: this._height });
            if (this._originalOverviewRuler || this._modifiedOverviewRuler) {
                this._layoutOverviewRulers();
            }
            this._layoutOverviewViewport();
        };
        DiffEditorWidget.prototype._layoutOverviewViewport = function () {
            var layout = this._computeOverviewViewport();
            if (!layout) {
                styleMutator_1.StyleMutator.setTop(this._overviewViewportDomElement, 0);
                styleMutator_1.StyleMutator.setHeight(this._overviewViewportDomElement, 0);
            }
            else {
                styleMutator_1.StyleMutator.setTop(this._overviewViewportDomElement, layout.top);
                styleMutator_1.StyleMutator.setHeight(this._overviewViewportDomElement, layout.height);
            }
        };
        DiffEditorWidget.prototype._computeOverviewViewport = function () {
            var layoutInfo = this.modifiedEditor.getLayoutInfo();
            if (!layoutInfo) {
                return null;
            }
            var scrollTop = this.modifiedEditor.getScrollTop();
            var scrollHeight = this.modifiedEditor.getScrollHeight();
            var computedAvailableSize = Math.max(0, layoutInfo.contentHeight);
            var computedRepresentableSize = Math.max(0, computedAvailableSize - 2 * 0);
            var computedRatio = scrollHeight > 0 ? (computedRepresentableSize / scrollHeight) : 0;
            var computedSliderSize = Math.max(1, Math.floor(layoutInfo.contentHeight * computedRatio));
            var computedSliderPosition = Math.floor(scrollTop * computedRatio);
            return {
                height: computedSliderSize,
                top: computedSliderPosition
            };
        };
        DiffEditorWidget.prototype._createDataSource = function () {
            var _this = this;
            return {
                getWidth: function () {
                    return _this._width;
                },
                getHeight: function () {
                    return _this._height;
                },
                getContainerDomNode: function () {
                    return _this._containerDomElement;
                },
                relayoutEditors: function () {
                    _this._doLayout();
                },
                getOriginalEditor: function () {
                    return _this.originalEditor;
                },
                getModifiedEditor: function () {
                    return _this.modifiedEditor;
                }
            };
        };
        DiffEditorWidget.prototype._setStrategy = function (newStrategy) {
            if (this._strategy) {
                this._strategy.dispose();
            }
            this._strategy = newStrategy;
            if (this._lineChanges) {
                this._updateDecorations();
            }
            // Just do a layout, the strategy might need it
            this._measureDomElement(true);
        };
        DiffEditorWidget.prototype._getLineChangeAtOrBeforeLineNumber = function (lineNumber, startLineNumberExtractor) {
            if (this._lineChanges.length === 0 || lineNumber < startLineNumberExtractor(this._lineChanges[0])) {
                // There are no changes or `lineNumber` is before the first change
                return null;
            }
            var min = 0, max = this._lineChanges.length - 1;
            while (min < max) {
                var mid = Math.floor((min + max) / 2);
                var midStart = startLineNumberExtractor(this._lineChanges[mid]);
                var midEnd = (mid + 1 <= max ? startLineNumberExtractor(this._lineChanges[mid + 1]) : Number.MAX_VALUE);
                if (lineNumber < midStart) {
                    max = mid - 1;
                }
                else if (lineNumber >= midEnd) {
                    min = mid + 1;
                }
                else {
                    // HIT!
                    min = mid;
                    max = mid;
                }
            }
            return this._lineChanges[min];
        };
        DiffEditorWidget.prototype._getEquivalentLineForOriginalLineNumber = function (lineNumber) {
            var lineChange = this._getLineChangeAtOrBeforeLineNumber(lineNumber, function (lineChange) { return lineChange.originalStartLineNumber; });
            if (!lineChange) {
                return lineNumber;
            }
            var originalEquivalentLineNumber = lineChange.originalStartLineNumber + (lineChange.originalEndLineNumber > 0 ? -1 : 0);
            var modifiedEquivalentLineNumber = lineChange.modifiedStartLineNumber + (lineChange.modifiedEndLineNumber > 0 ? -1 : 0);
            var lineChangeOriginalLength = (lineChange.originalEndLineNumber > 0 ? (lineChange.originalEndLineNumber - lineChange.originalStartLineNumber + 1) : 0);
            var lineChangeModifiedLength = (lineChange.modifiedEndLineNumber > 0 ? (lineChange.modifiedEndLineNumber - lineChange.modifiedStartLineNumber + 1) : 0);
            var delta = lineNumber - originalEquivalentLineNumber;
            if (delta <= lineChangeOriginalLength) {
                return modifiedEquivalentLineNumber + Math.min(delta, lineChangeModifiedLength);
            }
            return modifiedEquivalentLineNumber + lineChangeModifiedLength - lineChangeOriginalLength + delta;
        };
        DiffEditorWidget.prototype._getEquivalentLineForModifiedLineNumber = function (lineNumber) {
            var lineChange = this._getLineChangeAtOrBeforeLineNumber(lineNumber, function (lineChange) { return lineChange.modifiedStartLineNumber; });
            if (!lineChange) {
                return lineNumber;
            }
            var originalEquivalentLineNumber = lineChange.originalStartLineNumber + (lineChange.originalEndLineNumber > 0 ? -1 : 0);
            var modifiedEquivalentLineNumber = lineChange.modifiedStartLineNumber + (lineChange.modifiedEndLineNumber > 0 ? -1 : 0);
            var lineChangeOriginalLength = (lineChange.originalEndLineNumber > 0 ? (lineChange.originalEndLineNumber - lineChange.originalStartLineNumber + 1) : 0);
            var lineChangeModifiedLength = (lineChange.modifiedEndLineNumber > 0 ? (lineChange.modifiedEndLineNumber - lineChange.modifiedStartLineNumber + 1) : 0);
            var delta = lineNumber - modifiedEquivalentLineNumber;
            if (delta <= lineChangeModifiedLength) {
                return originalEquivalentLineNumber + Math.min(delta, lineChangeOriginalLength);
            }
            return originalEquivalentLineNumber + lineChangeOriginalLength - lineChangeModifiedLength + delta;
        };
        DiffEditorWidget.prototype.getDiffLineInformationForOriginal = function (lineNumber) {
            if (!this._lineChanges) {
                // Cannot answer that which I don't know
                return null;
            }
            return {
                equivalentLineNumber: this._getEquivalentLineForOriginalLineNumber(lineNumber)
            };
        };
        DiffEditorWidget.prototype.getDiffLineInformationForModified = function (lineNumber) {
            if (!this._lineChanges) {
                // Cannot answer that which I don't know
                return null;
            }
            return {
                equivalentLineNumber: this._getEquivalentLineForModifiedLineNumber(lineNumber)
            };
        };
        DiffEditorWidget.ONE_OVERVIEW_WIDTH = 15;
        DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH = 30;
        DiffEditorWidget.UPDATE_DIFF_DECORATIONS_DELAY = 200; // ms
        DiffEditorWidget = __decorate([
            __param(2, editorWorkerService_1.IEditorWorkerService),
            __param(3, instantiation_1.IInstantiationService)
        ], DiffEditorWidget);
        return DiffEditorWidget;
    }(eventEmitter_1.EventEmitter));
    exports.DiffEditorWidget = DiffEditorWidget;
    var DiffEditorWidgetStyle = (function () {
        function DiffEditorWidgetStyle(dataSource) {
            this._dataSource = dataSource;
        }
        DiffEditorWidgetStyle.prototype.getEditorsDiffDecorations = function (lineChanges, ignoreTrimWhitespace, originalWhitespaces, modifiedWhitespaces, originalEditor, modifiedEditor) {
            // Get view zones
            modifiedWhitespaces = modifiedWhitespaces.sort(function (a, b) {
                return a.afterLineNumber - b.afterLineNumber;
            });
            originalWhitespaces = originalWhitespaces.sort(function (a, b) {
                return a.afterLineNumber - b.afterLineNumber;
            });
            var zones = this._getViewZones(lineChanges, originalWhitespaces, modifiedWhitespaces, originalEditor, modifiedEditor);
            // Get decorations & overview ruler zones
            var originalDecorations = this._getOriginalEditorDecorations(lineChanges, ignoreTrimWhitespace, originalEditor, modifiedEditor);
            var modifiedDecorations = this._getModifiedEditorDecorations(lineChanges, ignoreTrimWhitespace, originalEditor, modifiedEditor);
            return {
                original: {
                    decorations: originalDecorations.decorations,
                    overviewZones: originalDecorations.overviewZones,
                    zones: zones.original
                },
                modified: {
                    decorations: modifiedDecorations.decorations,
                    overviewZones: modifiedDecorations.overviewZones,
                    zones: zones.modified
                }
            };
        };
        DiffEditorWidgetStyle.prototype._getViewZones = function (lineChanges, originalForeignVZ, modifiedForeignVZ, originalEditor, modifiedEditor) {
            return null;
        };
        DiffEditorWidgetStyle.prototype._getOriginalEditorDecorations = function (lineChanges, ignoreTrimWhitespace, originalEditor, modifiedEditor) {
            return null;
        };
        DiffEditorWidgetStyle.prototype._getModifiedEditorDecorations = function (lineChanges, ignoreTrimWhitespace, originalEditor, modifiedEditor) {
            return null;
        };
        return DiffEditorWidgetStyle;
    }());
    var ForeignViewZonesIterator = (function () {
        function ForeignViewZonesIterator(source) {
            this._source = source;
            this._index = -1;
            this.advance();
        }
        ForeignViewZonesIterator.prototype.advance = function () {
            this._index++;
            if (this._index < this._source.length) {
                this.current = this._source[this._index];
            }
            else {
                this.current = null;
            }
        };
        return ForeignViewZonesIterator;
    }());
    var ViewZonesComputer = (function () {
        function ViewZonesComputer(lineChanges, originalForeignVZ, modifiedForeignVZ) {
            this.lineChanges = lineChanges;
            this.originalForeignVZ = originalForeignVZ;
            this.modifiedForeignVZ = modifiedForeignVZ;
        }
        ViewZonesComputer.prototype.getViewZones = function () {
            var result = {
                original: [],
                modified: []
            };
            var i, length, lineChange;
            var stepOriginal, stepModified, stepOriginalIndex, stepModifiedIndex, lineChangeModifiedLength = 0, lineChangeOriginalLength = 0, originalEquivalentLineNumber = 0, modifiedEquivalentLineNumber = 0, originalEndEquivalentLineNumber = 0, modifiedEndEquivalentLineNumber = 0, viewZoneLineNumber = 0;
            var sortMyViewZones = function (a, b) {
                return a.afterLineNumber - b.afterLineNumber;
            };
            var addAndCombineIfPossible = function (destination, item) {
                if (item.domNode === null && destination.length > 0) {
                    var lastItem = destination[destination.length - 1];
                    if (lastItem.afterLineNumber === item.afterLineNumber && lastItem.domNode === null) {
                        lastItem.heightInLines += item.heightInLines;
                        return;
                    }
                }
                destination.push(item);
            };
            var modifiedForeignVZ = new ForeignViewZonesIterator(this.modifiedForeignVZ);
            var originalForeignVZ = new ForeignViewZonesIterator(this.originalForeignVZ);
            // In order to include foreign view zones after the last line change, the for loop will iterate once more after the end of the `lineChanges` array
            for (i = 0, length = this.lineChanges.length; i <= length; i++) {
                lineChange = (i < length ? this.lineChanges[i] : null);
                if (lineChange !== null) {
                    originalEquivalentLineNumber = lineChange.originalStartLineNumber + (lineChange.originalEndLineNumber > 0 ? -1 : 0);
                    modifiedEquivalentLineNumber = lineChange.modifiedStartLineNumber + (lineChange.modifiedEndLineNumber > 0 ? -1 : 0);
                    lineChangeOriginalLength = (lineChange.originalEndLineNumber > 0 ? (lineChange.originalEndLineNumber - lineChange.originalStartLineNumber + 1) : 0);
                    lineChangeModifiedLength = (lineChange.modifiedEndLineNumber > 0 ? (lineChange.modifiedEndLineNumber - lineChange.modifiedStartLineNumber + 1) : 0);
                    originalEndEquivalentLineNumber = Math.max(lineChange.originalStartLineNumber, lineChange.originalEndLineNumber);
                    modifiedEndEquivalentLineNumber = Math.max(lineChange.modifiedStartLineNumber, lineChange.modifiedEndLineNumber);
                }
                else {
                    // Increase to very large value to get the producing tests of foreign view zones running
                    originalEquivalentLineNumber += 10000000 + lineChangeOriginalLength;
                    modifiedEquivalentLineNumber += 10000000 + lineChangeModifiedLength;
                    originalEndEquivalentLineNumber = originalEquivalentLineNumber;
                    modifiedEndEquivalentLineNumber = modifiedEquivalentLineNumber;
                }
                // Each step produces view zones, and after producing them, we try to cancel them out, to avoid empty-empty view zone cases
                stepOriginal = [];
                stepModified = [];
                // ---------------------------- PRODUCE VIEW ZONES
                // [PRODUCE] View zone(s) in original-side due to foreign view zone(s) in modified-side
                while (modifiedForeignVZ.current && modifiedForeignVZ.current.afterLineNumber <= modifiedEndEquivalentLineNumber) {
                    if (modifiedForeignVZ.current.afterLineNumber <= modifiedEquivalentLineNumber) {
                        viewZoneLineNumber = originalEquivalentLineNumber - modifiedEquivalentLineNumber + modifiedForeignVZ.current.afterLineNumber;
                    }
                    else {
                        viewZoneLineNumber = originalEndEquivalentLineNumber;
                    }
                    stepOriginal.push({
                        afterLineNumber: viewZoneLineNumber,
                        heightInLines: modifiedForeignVZ.current.heightInLines,
                        domNode: null
                    });
                    modifiedForeignVZ.advance();
                }
                // [PRODUCE] View zone(s) in modified-side due to foreign view zone(s) in original-side
                while (originalForeignVZ.current && originalForeignVZ.current.afterLineNumber <= originalEndEquivalentLineNumber) {
                    if (originalForeignVZ.current.afterLineNumber <= originalEquivalentLineNumber) {
                        viewZoneLineNumber = modifiedEquivalentLineNumber - originalEquivalentLineNumber + originalForeignVZ.current.afterLineNumber;
                    }
                    else {
                        viewZoneLineNumber = modifiedEndEquivalentLineNumber;
                    }
                    stepModified.push({
                        afterLineNumber: viewZoneLineNumber,
                        heightInLines: originalForeignVZ.current.heightInLines,
                        domNode: null
                    });
                    originalForeignVZ.advance();
                }
                if (lineChange !== null && isChangeOrInsert(lineChange)) {
                    var r = this._produceOriginalFromDiff(lineChange, lineChangeOriginalLength, lineChangeModifiedLength);
                    if (r) {
                        stepOriginal.push(r);
                    }
                }
                if (lineChange !== null && isChangeOrDelete(lineChange)) {
                    var r = this._produceModifiedFromDiff(lineChange, lineChangeOriginalLength, lineChangeModifiedLength);
                    if (r) {
                        stepModified.push(r);
                    }
                }
                // ---------------------------- END PRODUCE VIEW ZONES
                // ---------------------------- EMIT MINIMAL VIEW ZONES
                // [CANCEL & EMIT] Try to cancel view zones out
                stepOriginalIndex = 0;
                stepModifiedIndex = 0;
                stepOriginal = stepOriginal.sort(sortMyViewZones);
                stepModified = stepModified.sort(sortMyViewZones);
                while (stepOriginalIndex < stepOriginal.length && stepModifiedIndex < stepModified.length) {
                    var original = stepOriginal[stepOriginalIndex];
                    var modified = stepModified[stepModifiedIndex];
                    var originalDelta = original.afterLineNumber - originalEquivalentLineNumber;
                    var modifiedDelta = modified.afterLineNumber - modifiedEquivalentLineNumber;
                    if (originalDelta < modifiedDelta) {
                        addAndCombineIfPossible(result.original, original);
                        stepOriginalIndex++;
                    }
                    else if (modifiedDelta < originalDelta) {
                        addAndCombineIfPossible(result.modified, modified);
                        stepModifiedIndex++;
                    }
                    else if (original.shouldNotShrink) {
                        addAndCombineIfPossible(result.original, original);
                        stepOriginalIndex++;
                    }
                    else if (modified.shouldNotShrink) {
                        addAndCombineIfPossible(result.modified, modified);
                        stepModifiedIndex++;
                    }
                    else {
                        if (original.heightInLines >= modified.heightInLines) {
                            // modified view zone gets removed
                            original.heightInLines -= modified.heightInLines;
                            stepModifiedIndex++;
                        }
                        else {
                            // original view zone gets removed
                            modified.heightInLines -= original.heightInLines;
                            stepOriginalIndex++;
                        }
                    }
                }
                // [EMIT] Remaining original view zones
                while (stepOriginalIndex < stepOriginal.length) {
                    addAndCombineIfPossible(result.original, stepOriginal[stepOriginalIndex]);
                    stepOriginalIndex++;
                }
                // [EMIT] Remaining modified view zones
                while (stepModifiedIndex < stepModified.length) {
                    addAndCombineIfPossible(result.modified, stepModified[stepModifiedIndex]);
                    stepModifiedIndex++;
                }
            }
            var ensureDomNode = function (z) {
                if (!z.domNode) {
                    z.domNode = createFakeLinesDiv();
                }
            };
            result.original.forEach(ensureDomNode);
            result.modified.forEach(ensureDomNode);
            return result;
        };
        ViewZonesComputer.prototype._produceOriginalFromDiff = function (lineChange, lineChangeOriginalLength, lineChangeModifiedLength) {
            throw new Error('NotImplemented');
        };
        ViewZonesComputer.prototype._produceModifiedFromDiff = function (lineChange, lineChangeOriginalLength, lineChangeModifiedLength) {
            throw new Error('NotImplemented');
        };
        return ViewZonesComputer;
    }());
    var DiffEdtorWidgetSideBySide = (function (_super) {
        __extends(DiffEdtorWidgetSideBySide, _super);
        function DiffEdtorWidgetSideBySide(dataSource, enableSplitViewResizing) {
            var _this = this;
            _super.call(this, dataSource);
            this._disableSash = (enableSplitViewResizing === false);
            this._sashRatio = null;
            this._sashPosition = null;
            this._sash = new sash_1.Sash(this._dataSource.getContainerDomNode(), this);
            if (this._disableSash) {
                this._sash.disable();
            }
            this._sash.on('start', function () { return _this._onSashDragStart(); });
            this._sash.on('change', function (e) { return _this._onSashDrag(e); });
            this._sash.on('end', function () { return _this._onSashDragEnd(); });
        }
        DiffEdtorWidgetSideBySide.prototype.dispose = function () {
            this._sash.dispose();
        };
        DiffEdtorWidgetSideBySide.prototype.setEnableSplitViewResizing = function (enableSplitViewResizing) {
            var newDisableSash = (enableSplitViewResizing === false);
            if (this._disableSash !== newDisableSash) {
                this._disableSash = newDisableSash;
                if (this._disableSash) {
                    this._sash.disable();
                }
                else {
                    this._sash.enable();
                }
            }
        };
        DiffEdtorWidgetSideBySide.prototype.layout = function (sashRatio) {
            if (sashRatio === void 0) { sashRatio = this._sashRatio; }
            var w = this._dataSource.getWidth();
            var contentWidth = w - DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH;
            var sashPosition = Math.floor((sashRatio || 0.5) * contentWidth);
            var midPoint = Math.floor(0.5 * contentWidth);
            sashPosition = this._disableSash ? midPoint : sashPosition || midPoint;
            if (contentWidth > DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH * 2) {
                if (sashPosition < DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH) {
                    sashPosition = DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH;
                }
                if (sashPosition > contentWidth - DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH) {
                    sashPosition = contentWidth - DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH;
                }
            }
            else {
                sashPosition = midPoint;
            }
            if (this._sashPosition !== sashPosition) {
                this._sashPosition = sashPosition;
                this._sash.layout();
            }
            return this._sashPosition;
        };
        DiffEdtorWidgetSideBySide.prototype._onSashDragStart = function () {
            this._startSashPosition = this._sashPosition;
        };
        DiffEdtorWidgetSideBySide.prototype._onSashDrag = function (e) {
            var w = this._dataSource.getWidth();
            var contentWidth = w - DiffEditorWidget.ENTIRE_DIFF_OVERVIEW_WIDTH;
            var sashPosition = this.layout((this._startSashPosition + (e.currentX - e.startX)) / contentWidth);
            this._sashRatio = sashPosition / contentWidth;
            this._dataSource.relayoutEditors();
        };
        DiffEdtorWidgetSideBySide.prototype._onSashDragEnd = function () {
            this._sash.layout();
        };
        DiffEdtorWidgetSideBySide.prototype.getVerticalSashTop = function (sash) {
            return 0;
        };
        DiffEdtorWidgetSideBySide.prototype.getVerticalSashLeft = function (sash) {
            return this._sashPosition;
        };
        DiffEdtorWidgetSideBySide.prototype.getVerticalSashHeight = function (sash) {
            return this._dataSource.getHeight();
        };
        DiffEdtorWidgetSideBySide.prototype._getViewZones = function (lineChanges, originalForeignVZ, modifiedForeignVZ, originalEditor, modifiedEditor) {
            var c = new SideBySideViewZonesComputer(lineChanges, originalForeignVZ, modifiedForeignVZ);
            return c.getViewZones();
        };
        DiffEdtorWidgetSideBySide.prototype._getOriginalEditorDecorations = function (lineChanges, ignoreTrimWhitespace, originalEditor, modifiedEditor) {
            var result = {
                decorations: [],
                overviewZones: []
            }, i, length, j, lengthJ, lineChange, charChange, lineNumber, startColumn, endColumn, originalModel = originalEditor.getModel();
            for (i = 0, length = lineChanges.length; i < length; i++) {
                lineChange = lineChanges[i];
                if (isChangeOrDelete(lineChange)) {
                    result.decorations.push(createDecoration(lineChange.originalStartLineNumber, 1, lineChange.originalEndLineNumber, Number.MAX_VALUE, 'line-delete', true));
                    if (!isChangeOrInsert(lineChange) || !lineChange.charChanges) {
                        result.decorations.push(createDecoration(lineChange.originalStartLineNumber, 1, lineChange.originalEndLineNumber, Number.MAX_VALUE, 'char-delete', true));
                    }
                    result.overviewZones.push({
                        startLineNumber: lineChange.originalStartLineNumber,
                        endLineNumber: lineChange.originalEndLineNumber,
                        color: 'rgba(255, 0, 0, 0.4)',
                        darkColor: 'rgba(255, 0, 0, 0.4)',
                        position: editorCommon.OverviewRulerLane.Full
                    });
                    if (lineChange.charChanges) {
                        for (j = 0, lengthJ = lineChange.charChanges.length; j < lengthJ; j++) {
                            charChange = lineChange.charChanges[j];
                            if (isChangeOrDelete(charChange)) {
                                if (ignoreTrimWhitespace) {
                                    for (lineNumber = charChange.originalStartLineNumber; lineNumber <= charChange.originalEndLineNumber; lineNumber++) {
                                        if (lineNumber === charChange.originalStartLineNumber) {
                                            startColumn = charChange.originalStartColumn;
                                        }
                                        else {
                                            startColumn = originalModel.getLineFirstNonWhitespaceColumn(lineNumber);
                                        }
                                        if (lineNumber === charChange.originalEndLineNumber) {
                                            endColumn = charChange.originalEndColumn;
                                        }
                                        else {
                                            endColumn = originalModel.getLineLastNonWhitespaceColumn(lineNumber);
                                        }
                                        result.decorations.push(createDecoration(lineNumber, startColumn, lineNumber, endColumn, 'char-delete', false));
                                    }
                                }
                                else {
                                    result.decorations.push(createDecoration(charChange.originalStartLineNumber, charChange.originalStartColumn, charChange.originalEndLineNumber, charChange.originalEndColumn, 'char-delete', false));
                                }
                            }
                        }
                    }
                }
            }
            return result;
        };
        DiffEdtorWidgetSideBySide.prototype._getModifiedEditorDecorations = function (lineChanges, ignoreTrimWhitespace, originalEditor, modifiedEditor) {
            var result = {
                decorations: [],
                overviewZones: []
            }, i, length, j, lengthJ, lineChange, charChange, lineNumber, startColumn, endColumn, modifiedModel = modifiedEditor.getModel();
            for (i = 0, length = lineChanges.length; i < length; i++) {
                lineChange = lineChanges[i];
                if (isChangeOrInsert(lineChange)) {
                    result.decorations.push(createDecoration(lineChange.modifiedStartLineNumber, 1, lineChange.modifiedEndLineNumber, Number.MAX_VALUE, 'line-insert', true));
                    if (!isChangeOrDelete(lineChange) || !lineChange.charChanges) {
                        result.decorations.push(createDecoration(lineChange.modifiedStartLineNumber, 1, lineChange.modifiedEndLineNumber, Number.MAX_VALUE, 'char-insert', true));
                    }
                    result.overviewZones.push({
                        startLineNumber: lineChange.modifiedStartLineNumber,
                        endLineNumber: lineChange.modifiedEndLineNumber,
                        color: 'rgba(155, 185, 85, 0.4)',
                        darkColor: 'rgba(155, 185, 85, 0.4)',
                        position: editorCommon.OverviewRulerLane.Full
                    });
                    if (lineChange.charChanges) {
                        for (j = 0, lengthJ = lineChange.charChanges.length; j < lengthJ; j++) {
                            charChange = lineChange.charChanges[j];
                            if (isChangeOrInsert(charChange)) {
                                if (ignoreTrimWhitespace) {
                                    for (lineNumber = charChange.modifiedStartLineNumber; lineNumber <= charChange.modifiedEndLineNumber; lineNumber++) {
                                        if (lineNumber === charChange.modifiedStartLineNumber) {
                                            startColumn = charChange.modifiedStartColumn;
                                        }
                                        else {
                                            startColumn = modifiedModel.getLineFirstNonWhitespaceColumn(lineNumber);
                                        }
                                        if (lineNumber === charChange.modifiedEndLineNumber) {
                                            endColumn = charChange.modifiedEndColumn;
                                        }
                                        else {
                                            endColumn = modifiedModel.getLineLastNonWhitespaceColumn(lineNumber);
                                        }
                                        result.decorations.push(createDecoration(lineNumber, startColumn, lineNumber, endColumn, 'char-insert', false));
                                    }
                                }
                                else {
                                    result.decorations.push(createDecoration(charChange.modifiedStartLineNumber, charChange.modifiedStartColumn, charChange.modifiedEndLineNumber, charChange.modifiedEndColumn, 'char-insert', false));
                                }
                            }
                        }
                    }
                }
            }
            return result;
        };
        DiffEdtorWidgetSideBySide.MINIMUM_EDITOR_WIDTH = 100;
        return DiffEdtorWidgetSideBySide;
    }(DiffEditorWidgetStyle));
    var SideBySideViewZonesComputer = (function (_super) {
        __extends(SideBySideViewZonesComputer, _super);
        function SideBySideViewZonesComputer(lineChanges, originalForeignVZ, modifiedForeignVZ) {
            _super.call(this, lineChanges, originalForeignVZ, modifiedForeignVZ);
        }
        SideBySideViewZonesComputer.prototype._produceOriginalFromDiff = function (lineChange, lineChangeOriginalLength, lineChangeModifiedLength) {
            if (lineChangeModifiedLength > lineChangeOriginalLength) {
                return {
                    afterLineNumber: Math.max(lineChange.originalStartLineNumber, lineChange.originalEndLineNumber),
                    heightInLines: (lineChangeModifiedLength - lineChangeOriginalLength),
                    domNode: null
                };
            }
            return null;
        };
        SideBySideViewZonesComputer.prototype._produceModifiedFromDiff = function (lineChange, lineChangeOriginalLength, lineChangeModifiedLength) {
            if (lineChangeOriginalLength > lineChangeModifiedLength) {
                return {
                    afterLineNumber: Math.max(lineChange.modifiedStartLineNumber, lineChange.modifiedEndLineNumber),
                    heightInLines: (lineChangeOriginalLength - lineChangeModifiedLength),
                    domNode: null
                };
            }
            return null;
        };
        return SideBySideViewZonesComputer;
    }(ViewZonesComputer));
    var DiffEdtorWidgetInline = (function (_super) {
        __extends(DiffEdtorWidgetInline, _super);
        function DiffEdtorWidgetInline(dataSource, enableSplitViewResizing) {
            var _this = this;
            _super.call(this, dataSource);
            this.decorationsLeft = dataSource.getOriginalEditor().getLayoutInfo().decorationsLeft;
            this.toDispose = [];
            this.toDispose.push(dataSource.getOriginalEditor().addListener2(editorCommon.EventType.EditorLayout, function (layoutInfo) {
                if (_this.decorationsLeft !== layoutInfo.decorationsLeft) {
                    _this.decorationsLeft = layoutInfo.decorationsLeft;
                    dataSource.relayoutEditors();
                }
            }));
        }
        DiffEdtorWidgetInline.prototype.dispose = function () {
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
        };
        DiffEdtorWidgetInline.prototype.setEnableSplitViewResizing = function (enableSplitViewResizing) {
            // Nothing to do..
        };
        DiffEdtorWidgetInline.prototype._getViewZones = function (lineChanges, originalForeignVZ, modifiedForeignVZ, originalEditor, modifiedEditor) {
            var computer = new InlineViewZonesComputer(lineChanges, originalForeignVZ, modifiedForeignVZ, originalEditor, modifiedEditor);
            return computer.getViewZones();
        };
        DiffEdtorWidgetInline.prototype._getOriginalEditorDecorations = function (lineChanges, ignoreTrimWhitespace, originalEditor, modifiedEditor) {
            var result = {
                decorations: [],
                overviewZones: []
            }, i, length, lineChange;
            for (i = 0, length = lineChanges.length; i < length; i++) {
                lineChange = lineChanges[i];
                // Add overview zones in the overview ruler
                if (isChangeOrDelete(lineChange)) {
                    result.overviewZones.push({
                        startLineNumber: lineChange.originalStartLineNumber,
                        endLineNumber: lineChange.originalEndLineNumber,
                        color: 'rgba(255, 0, 0, 0.4)',
                        darkColor: 'rgba(255, 0, 0, 0.4)',
                        position: editorCommon.OverviewRulerLane.Full
                    });
                }
            }
            return result;
        };
        DiffEdtorWidgetInline.prototype._getModifiedEditorDecorations = function (lineChanges, ignoreTrimWhitespace, originalEditor, modifiedEditor) {
            var result = {
                decorations: [],
                overviewZones: []
            }, i, length, lineChange, j, lengthJ, charChange, lineNumber, startColumn, endColumn, modifiedModel = modifiedEditor.getModel();
            for (i = 0, length = lineChanges.length; i < length; i++) {
                lineChange = lineChanges[i];
                // Add decorations & overview zones
                if (isChangeOrInsert(lineChange)) {
                    result.decorations.push(createDecoration(lineChange.modifiedStartLineNumber, 1, lineChange.modifiedEndLineNumber, Number.MAX_VALUE, 'line-insert', true));
                    result.overviewZones.push({
                        startLineNumber: lineChange.modifiedStartLineNumber,
                        endLineNumber: lineChange.modifiedEndLineNumber,
                        color: 'rgba(155, 185, 85, 0.4)',
                        darkColor: 'rgba(155, 185, 85, 0.4)',
                        position: editorCommon.OverviewRulerLane.Full
                    });
                    if (lineChange.charChanges) {
                        for (j = 0, lengthJ = lineChange.charChanges.length; j < lengthJ; j++) {
                            charChange = lineChange.charChanges[j];
                            if (isChangeOrInsert(charChange)) {
                                if (ignoreTrimWhitespace) {
                                    for (lineNumber = charChange.modifiedStartLineNumber; lineNumber <= charChange.modifiedEndLineNumber; lineNumber++) {
                                        if (lineNumber === charChange.modifiedStartLineNumber) {
                                            startColumn = charChange.modifiedStartColumn;
                                        }
                                        else {
                                            startColumn = modifiedModel.getLineFirstNonWhitespaceColumn(lineNumber);
                                        }
                                        if (lineNumber === charChange.modifiedEndLineNumber) {
                                            endColumn = charChange.modifiedEndColumn;
                                        }
                                        else {
                                            endColumn = modifiedModel.getLineLastNonWhitespaceColumn(lineNumber);
                                        }
                                        result.decorations.push(createDecoration(lineNumber, startColumn, lineNumber, endColumn, 'char-insert', false));
                                    }
                                }
                                else {
                                    result.decorations.push(createDecoration(charChange.modifiedStartLineNumber, charChange.modifiedStartColumn, charChange.modifiedEndLineNumber, charChange.modifiedEndColumn, 'char-insert', false));
                                }
                            }
                        }
                    }
                    else {
                        result.decorations.push(createDecoration(lineChange.modifiedStartLineNumber, 1, lineChange.modifiedEndLineNumber, Number.MAX_VALUE, 'char-insert', true));
                    }
                }
            }
            return result;
        };
        DiffEdtorWidgetInline.prototype.layout = function () {
            // An editor should not be smaller than 5px
            return Math.max(5, this.decorationsLeft);
        };
        return DiffEdtorWidgetInline;
    }(DiffEditorWidgetStyle));
    var InlineViewZonesComputer = (function (_super) {
        __extends(InlineViewZonesComputer, _super);
        function InlineViewZonesComputer(lineChanges, originalForeignVZ, modifiedForeignVZ, originalEditor, modifiedEditor) {
            _super.call(this, lineChanges, originalForeignVZ, modifiedForeignVZ);
            this.originalModel = originalEditor.getModel();
            this.modifiedEditorConfiguration = modifiedEditor.getConfiguration();
            this.modifiedEditorTabSize = modifiedEditor.getModel().getOptions().tabSize;
        }
        InlineViewZonesComputer.prototype._produceOriginalFromDiff = function (lineChange, lineChangeOriginalLength, lineChangeModifiedLength) {
            return {
                afterLineNumber: Math.max(lineChange.originalStartLineNumber, lineChange.originalEndLineNumber),
                heightInLines: lineChangeModifiedLength,
                domNode: document.createElement('div')
            };
        };
        InlineViewZonesComputer.prototype._produceModifiedFromDiff = function (lineChange, lineChangeOriginalLength, lineChangeModifiedLength) {
            var decorations = [], j, lengthJ, charChange, tempDecoration;
            if (lineChange.charChanges) {
                for (j = 0, lengthJ = lineChange.charChanges.length; j < lengthJ; j++) {
                    charChange = lineChange.charChanges[j];
                    if (isChangeOrDelete(charChange)) {
                        tempDecoration = createDecoration(charChange.originalStartLineNumber, charChange.originalStartColumn, charChange.originalEndLineNumber, charChange.originalEndColumn, 'char-delete', false);
                        tempDecoration.options.inlineClassName = tempDecoration.options.className;
                        decorations.push(tempDecoration);
                    }
                }
            }
            var html = [], lineNumber;
            for (lineNumber = lineChange.originalStartLineNumber; lineNumber <= lineChange.originalEndLineNumber; lineNumber++) {
                html = html.concat(this.renderOriginalLine(lineNumber - lineChange.originalStartLineNumber, this.originalModel, this.modifiedEditorConfiguration, this.modifiedEditorTabSize, lineNumber, decorations));
            }
            var domNode = document.createElement('div');
            domNode.className = 'view-lines line-delete';
            domNode.innerHTML = html.join('');
            return {
                shouldNotShrink: true,
                afterLineNumber: (lineChange.modifiedEndLineNumber === 0 ? lineChange.modifiedStartLineNumber : lineChange.modifiedStartLineNumber - 1),
                heightInLines: lineChangeOriginalLength,
                domNode: domNode
            };
        };
        InlineViewZonesComputer.prototype.renderOriginalLine = function (count, originalModel, config, tabSize, lineNumber, decorations) {
            var lineContent = originalModel.getLineContent(lineNumber), lineTokens, parts;
            lineTokens = {
                getTokens: function () {
                    return [{ startIndex: 0, type: '' }];
                },
                getFauxIndentLength: function () {
                    return 0;
                },
                getTextLength: function () {
                    return lineContent.length;
                },
                equals: function (other) {
                    return false;
                },
                findIndexOfOffset: function (offset) {
                    return 0;
                }
            };
            parts = viewLineParts_1.createLineParts(lineNumber, 1, lineContent, lineTokens, decorations, config.renderWhitespace);
            var r = viewLineRenderer_1.renderLine({
                lineContent: lineContent,
                tabSize: tabSize,
                stopRenderingLineAfter: config.stopRenderingLineAfter,
                renderWhitespace: config.renderWhitespace,
                parts: parts.getParts()
            });
            var myResult = [];
            myResult.push('<div class="view-line');
            if (decorations.length === 0) {
                // No char changes
                myResult.push(' char-delete');
            }
            myResult.push('" style="top:');
            myResult.push(String(count * config.lineHeight));
            myResult.push('px;width:1000000px;">');
            myResult = myResult.concat(r.output);
            myResult.push('</div>');
            return myResult;
        };
        return InlineViewZonesComputer;
    }(ViewZonesComputer));
    function isChangeOrInsert(lineChange) {
        return lineChange.modifiedEndLineNumber > 0;
    }
    function isChangeOrDelete(lineChange) {
        return lineChange.originalEndLineNumber > 0;
    }
    function createDecoration(startLineNumber, startColumn, endLineNumber, endColumn, className, isWholeLine) {
        return {
            range: new range_1.Range(startLineNumber, startColumn, endLineNumber, endColumn),
            options: {
                className: className,
                isWholeLine: isWholeLine
            }
        };
    }
    function createFakeLinesDiv() {
        var r = document.createElement('div');
        r.className = 'diagonal-fill';
        return r;
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/browser/widget/embeddedCodeEditorWidget", ["require", "exports", 'vs/base/common/objects', 'vs/platform/instantiation/common/instantiation', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/telemetry/common/telemetry', 'vs/editor/common/editorCommon', 'vs/editor/common/services/codeEditorService', 'vs/editor/browser/widget/codeEditorWidget'], function (require, exports, objects, instantiation_1, keybindingService_1, telemetry_1, editorCommon_1, codeEditorService_1, codeEditorWidget_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EmbeddedCodeEditorWidget = (function (_super) {
        __extends(EmbeddedCodeEditorWidget, _super);
        function EmbeddedCodeEditorWidget(domElement, options, parentEditor, instantiationService, codeEditorService, keybindingService, telemetryService) {
            var _this = this;
            _super.call(this, domElement, parentEditor.getRawConfiguration(), instantiationService, codeEditorService, keybindingService, telemetryService);
            this._parentEditor = parentEditor;
            this._overwriteOptions = options;
            // Overwrite parent's options
            _super.prototype.updateOptions.call(this, this._overwriteOptions);
            this._lifetimeDispose.push(parentEditor.addListener2(editorCommon_1.EventType.ConfigurationChanged, function (e) { return _this._onParentConfigurationChanged(e); }));
        }
        EmbeddedCodeEditorWidget.prototype._onParentConfigurationChanged = function (e) {
            _super.prototype.updateOptions.call(this, this._parentEditor.getRawConfiguration());
            _super.prototype.updateOptions.call(this, this._overwriteOptions);
        };
        EmbeddedCodeEditorWidget.prototype.updateOptions = function (newOptions) {
            objects.mixin(this._overwriteOptions, newOptions, true);
            _super.prototype.updateOptions.call(this, this._overwriteOptions);
        };
        EmbeddedCodeEditorWidget = __decorate([
            __param(3, instantiation_1.IInstantiationService),
            __param(4, codeEditorService_1.ICodeEditorService),
            __param(5, keybindingService_1.IKeybindingService),
            __param(6, telemetry_1.ITelemetryService)
        ], EmbeddedCodeEditorWidget);
        return EmbeddedCodeEditorWidget;
    }(codeEditorWidget_1.CodeEditorWidget));
    exports.EmbeddedCodeEditorWidget = EmbeddedCodeEditorWidget;
});

define("vs/platform/thread/common/thread", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // --- thread service (web workers)
    exports.IThreadService = instantiation.createDecorator('threadService');
    var IRemotableCtorMap = (function () {
        function IRemotableCtorMap() {
        }
        return IRemotableCtorMap;
    }());
    exports.IRemotableCtorMap = IRemotableCtorMap;
    var IRemotableCtorAffinityMap = (function () {
        function IRemotableCtorAffinityMap() {
        }
        return IRemotableCtorAffinityMap;
    }());
    exports.IRemotableCtorAffinityMap = IRemotableCtorAffinityMap;
    var Remotable = (function () {
        function Remotable() {
        }
        Remotable.getId = function (ctor) {
            return (ctor[Remotable.PROP_NAME] || null);
        };
        Remotable.MainContext = function (identifier) {
            return function (target) {
                Remotable._ensureUnique(identifier);
                Remotable.Registry.MainContext[identifier] = target;
                target[Remotable.PROP_NAME] = identifier;
            };
        };
        Remotable.ExtHostContext = function (identifier) {
            return function (target) {
                Remotable._ensureUnique(identifier);
                Remotable.Registry.ExtHostContext[identifier] = target;
                target[Remotable.PROP_NAME] = identifier;
            };
        };
        Remotable.WorkerContext = function (identifier, whichWorker) {
            return function (target) {
                Remotable._ensureUnique(identifier);
                Remotable.Registry.WorkerContext[identifier] = {
                    ctor: target,
                    affinity: whichWorker
                };
                target[Remotable.PROP_NAME] = identifier;
            };
        };
        Remotable._ensureUnique = function (identifier) {
            if (Remotable.Registry.MainContext[identifier] || Remotable.Registry.ExtHostContext[identifier] || Remotable.Registry.WorkerContext[identifier]) {
                throw new Error('Duplicate Remotable identifier found');
            }
        };
        Remotable.PROP_NAME = '$__REMOTABLE_ID';
        Remotable.Registry = {
            MainContext: Object.create(null),
            ExtHostContext: Object.create(null),
            WorkerContext: Object.create(null),
        };
        return Remotable;
    }());
    exports.Remotable = Remotable;
    (function (ThreadAffinity) {
        ThreadAffinity[ThreadAffinity["None"] = 0] = "None";
        ThreadAffinity[ThreadAffinity["Group1"] = 1] = "Group1";
        ThreadAffinity[ThreadAffinity["Group2"] = 2] = "Group2";
        ThreadAffinity[ThreadAffinity["Group3"] = 3] = "Group3";
        ThreadAffinity[ThreadAffinity["Group4"] = 4] = "Group4";
        ThreadAffinity[ThreadAffinity["Group5"] = 5] = "Group5";
        ThreadAffinity[ThreadAffinity["Group6"] = 6] = "Group6";
        ThreadAffinity[ThreadAffinity["Group7"] = 7] = "Group7";
        ThreadAffinity[ThreadAffinity["Group8"] = 8] = "Group8";
        ThreadAffinity[ThreadAffinity["Group9"] = 9] = "Group9";
        ThreadAffinity[ThreadAffinity["All"] = 10] = "All";
    })(exports.ThreadAffinity || (exports.ThreadAffinity = {}));
    var ThreadAffinity = exports.ThreadAffinity;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/common/services/modeServiceImpl", ["require", "exports", 'vs/nls!vs/editor/common/services/modeServiceImpl', 'vs/base/common/errors', 'vs/base/common/event', 'vs/base/common/lifecycle', 'vs/base/common/objects', 'vs/base/common/paths', 'vs/base/common/winjs.base', 'vs/base/common/mime', 'vs/platform/instantiation/common/descriptors', 'vs/platform/extensions/common/extensionsRegistry', 'vs/platform/thread/common/thread', 'vs/editor/common/modes/abstractMode', 'vs/editor/common/modes/modesRegistry', 'vs/editor/common/modes/monarch/monarchCompile', 'vs/editor/common/modes/monarch/monarchDefinition', 'vs/editor/common/modes/monarch/monarchLexer', 'vs/editor/common/modes/supports/declarationSupport', 'vs/editor/common/modes/supports/parameterHintsSupport', 'vs/editor/common/modes/supports/referenceSupport', 'vs/editor/common/modes/supports/richEditSupport', 'vs/editor/common/modes/supports/suggestSupport', 'vs/editor/common/services/languagesRegistry', 'vs/editor/common/services/modeService', 'vs/platform/configuration/common/configuration'], function (require, exports, nls, errors_1, event_1, lifecycle_1, objects, paths, winjs_base_1, mime, descriptors_1, extensionsRegistry_1, thread_1, abstractMode_1, modesRegistry_1, monarchCompile_1, monarchDefinition_1, monarchLexer_1, declarationSupport_1, parameterHintsSupport_1, referenceSupport_1, richEditSupport_1, suggestSupport_1, languagesRegistry_1, modeService_1, configuration_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var languagesExtPoint = extensionsRegistry_1.ExtensionsRegistry.registerExtensionPoint('languages', {
        description: nls.localize(0, null),
        type: 'array',
        defaultSnippets: [{ body: [{ id: '', aliases: [], extensions: [] }] }],
        items: {
            type: 'object',
            defaultSnippets: [{ body: { id: '', extensions: [] } }],
            properties: {
                id: {
                    description: nls.localize(1, null),
                    type: 'string'
                },
                aliases: {
                    description: nls.localize(2, null),
                    type: 'array',
                    items: {
                        type: 'string'
                    }
                },
                extensions: {
                    description: nls.localize(3, null),
                    default: ['.foo'],
                    type: 'array',
                    items: {
                        type: 'string'
                    }
                },
                filenames: {
                    description: nls.localize(4, null),
                    type: 'array',
                    items: {
                        type: 'string'
                    }
                },
                filenamePatterns: {
                    description: nls.localize(5, null),
                    default: ['bar*foo.txt'],
                    type: 'array',
                    items: {
                        type: 'string'
                    }
                },
                mimetypes: {
                    description: nls.localize(6, null),
                    type: 'array',
                    items: {
                        type: 'string'
                    }
                },
                firstLine: {
                    description: nls.localize(7, null),
                    type: 'string'
                },
                configuration: {
                    description: nls.localize(8, null),
                    type: 'string'
                }
            }
        }
    });
    function isUndefinedOrStringArray(value) {
        if (typeof value === 'undefined') {
            return true;
        }
        if (!Array.isArray(value)) {
            return false;
        }
        return value.every(function (item) { return typeof item === 'string'; });
    }
    function isValidLanguageExtensionPoint(value, collector) {
        if (!value) {
            collector.error(nls.localize(9, null, languagesExtPoint.name));
            return false;
        }
        if (typeof value.id !== 'string') {
            collector.error(nls.localize(10, null, 'id'));
            return false;
        }
        if (!isUndefinedOrStringArray(value.extensions)) {
            collector.error(nls.localize(11, null, 'extensions'));
            return false;
        }
        if (!isUndefinedOrStringArray(value.filenames)) {
            collector.error(nls.localize(12, null, 'filenames'));
            return false;
        }
        if (typeof value.firstLine !== 'undefined' && typeof value.firstLine !== 'string') {
            collector.error(nls.localize(13, null, 'firstLine'));
            return false;
        }
        if (typeof value.configuration !== 'undefined' && typeof value.configuration !== 'string') {
            collector.error(nls.localize(14, null, 'configuration'));
            return false;
        }
        if (!isUndefinedOrStringArray(value.aliases)) {
            collector.error(nls.localize(15, null, 'aliases'));
            return false;
        }
        if (!isUndefinedOrStringArray(value.mimetypes)) {
            collector.error(nls.localize(16, null, 'mimetypes'));
            return false;
        }
        return true;
    }
    var ModeServiceImpl = (function () {
        function ModeServiceImpl(threadService, extensionService) {
            var _this = this;
            this.serviceId = modeService_1.IModeService;
            this._onDidAddModes = new event_1.Emitter();
            this.onDidAddModes = this._onDidAddModes.event;
            this._onDidCreateMode = new event_1.Emitter();
            this.onDidCreateMode = this._onDidCreateMode.event;
            this._threadService = threadService;
            this._extensionService = extensionService;
            this._activationPromises = {};
            this._instantiatedModes = {};
            this._config = {};
            this._registry = new languagesRegistry_1.LanguagesRegistry();
            this._registry.onDidAddModes(function (modes) { return _this._onDidAddModes.fire(modes); });
        }
        ModeServiceImpl.prototype.getConfigurationForMode = function (modeId) {
            return this._config[modeId] || {};
        };
        ModeServiceImpl.prototype.configureMode = function (mimetype, options) {
            var modeId = this.getModeId(mimetype);
            if (modeId) {
                this.configureModeById(modeId, options);
            }
        };
        ModeServiceImpl.prototype.configureModeById = function (modeId, options) {
            var previousOptions = this._config[modeId] || {};
            var newOptions = objects.mixin(objects.clone(previousOptions), options);
            if (objects.equals(previousOptions, newOptions)) {
                // This configure call is a no-op
                return;
            }
            this._config[modeId] = newOptions;
            var mode = this.getMode(modeId);
            if (mode && mode.configSupport) {
                mode.configSupport.configure(this.getConfigurationForMode(modeId));
            }
        };
        ModeServiceImpl.prototype.configureAllModes = function (config) {
            var _this = this;
            if (!config) {
                return;
            }
            var modes = this._registry.getRegisteredModes();
            modes.forEach(function (modeIdentifier) {
                var configuration = config[modeIdentifier];
                _this.configureModeById(modeIdentifier, configuration);
            });
        };
        ModeServiceImpl.prototype.isRegisteredMode = function (mimetypeOrModeId) {
            return this._registry.isRegisteredMode(mimetypeOrModeId);
        };
        ModeServiceImpl.prototype.isCompatMode = function (modeId) {
            var compatModeData = this._registry.getCompatMode(modeId);
            return (compatModeData ? true : false);
        };
        ModeServiceImpl.prototype.getRegisteredModes = function () {
            return this._registry.getRegisteredModes();
        };
        ModeServiceImpl.prototype.getRegisteredLanguageNames = function () {
            return this._registry.getRegisteredLanguageNames();
        };
        ModeServiceImpl.prototype.getExtensions = function (alias) {
            return this._registry.getExtensions(alias);
        };
        ModeServiceImpl.prototype.getMimeForMode = function (modeId) {
            return this._registry.getMimeForMode(modeId);
        };
        ModeServiceImpl.prototype.getLanguageName = function (modeId) {
            return this._registry.getLanguageName(modeId);
        };
        ModeServiceImpl.prototype.getModeIdForLanguageName = function (alias) {
            return this._registry.getModeIdForLanguageNameLowercase(alias);
        };
        ModeServiceImpl.prototype.getModeId = function (commaSeparatedMimetypesOrCommaSeparatedIds) {
            var modeIds = this._registry.extractModeIds(commaSeparatedMimetypesOrCommaSeparatedIds);
            if (modeIds.length > 0) {
                return modeIds[0];
            }
            return null;
        };
        ModeServiceImpl.prototype.getConfigurationFiles = function (modeId) {
            return this._registry.getConfigurationFiles(modeId);
        };
        // --- instantiation
        ModeServiceImpl.prototype.lookup = function (commaSeparatedMimetypesOrCommaSeparatedIds) {
            var r = [];
            var modeIds = this._registry.extractModeIds(commaSeparatedMimetypesOrCommaSeparatedIds);
            for (var i = 0; i < modeIds.length; i++) {
                var modeId = modeIds[i];
                r.push({
                    modeId: modeId,
                    isInstantiated: this._instantiatedModes.hasOwnProperty(modeId)
                });
            }
            return r;
        };
        ModeServiceImpl.prototype.getMode = function (commaSeparatedMimetypesOrCommaSeparatedIds) {
            var modeIds = this._registry.extractModeIds(commaSeparatedMimetypesOrCommaSeparatedIds);
            var isPlainText = false;
            for (var i = 0; i < modeIds.length; i++) {
                if (this._instantiatedModes.hasOwnProperty(modeIds[i])) {
                    return this._instantiatedModes[modeIds[i]];
                }
                isPlainText = isPlainText || (modeIds[i] === 'plaintext');
            }
            if (isPlainText) {
                // Try to do it synchronously
                var r = null;
                this.getOrCreateMode(commaSeparatedMimetypesOrCommaSeparatedIds).then(function (mode) {
                    r = mode;
                }).done(null, errors_1.onUnexpectedError);
                return r;
            }
        };
        ModeServiceImpl.prototype.getModeIdByLanguageName = function (languageName) {
            var modeIds = this._registry.getModeIdsFromLanguageName(languageName);
            if (modeIds.length > 0) {
                return modeIds[0];
            }
            return null;
        };
        ModeServiceImpl.prototype.getModeIdByFilenameOrFirstLine = function (filename, firstLine) {
            var modeIds = this._registry.getModeIdsFromFilenameOrFirstLine(filename, firstLine);
            if (modeIds.length > 0) {
                return modeIds[0];
            }
            return null;
        };
        ModeServiceImpl.prototype.onReady = function () {
            return this._extensionService.onReady();
        };
        ModeServiceImpl.prototype.getOrCreateMode = function (commaSeparatedMimetypesOrCommaSeparatedIds) {
            var _this = this;
            return this.onReady().then(function () {
                var modeId = _this.getModeId(commaSeparatedMimetypesOrCommaSeparatedIds);
                // Fall back to plain text if no mode was found
                return _this._getOrCreateMode(modeId || 'plaintext');
            });
        };
        ModeServiceImpl.prototype.getOrCreateModeByLanguageName = function (languageName) {
            var _this = this;
            return this.onReady().then(function () {
                var modeId = _this.getModeIdByLanguageName(languageName);
                // Fall back to plain text if no mode was found
                return _this._getOrCreateMode(modeId || 'plaintext');
            });
        };
        ModeServiceImpl.prototype.getOrCreateModeByFilenameOrFirstLine = function (filename, firstLine) {
            var _this = this;
            return this.onReady().then(function () {
                var modeId = _this.getModeIdByFilenameOrFirstLine(filename, firstLine);
                // Fall back to plain text if no mode was found
                return _this._getOrCreateMode(modeId || 'plaintext');
            });
        };
        ModeServiceImpl.prototype._getOrCreateMode = function (modeId) {
            var _this = this;
            if (this._instantiatedModes.hasOwnProperty(modeId)) {
                return winjs_base_1.TPromise.as(this._instantiatedModes[modeId]);
            }
            if (this._activationPromises.hasOwnProperty(modeId)) {
                return this._activationPromises[modeId];
            }
            var c, e;
            var promise = new winjs_base_1.TPromise(function (cc, ee, pp) { c = cc; e = ee; });
            this._activationPromises[modeId] = promise;
            this._createMode(modeId).then(function (mode) {
                _this._instantiatedModes[modeId] = mode;
                delete _this._activationPromises[modeId];
                _this._onDidCreateMode.fire(mode);
                _this._extensionService.activateByEvent("onLanguage:" + modeId).done(null, errors_1.onUnexpectedError);
                return _this._instantiatedModes[modeId];
            }).then(c, e);
            return promise;
        };
        ModeServiceImpl.prototype._createMode = function (modeId) {
            var _this = this;
            var modeDescriptor = this._createModeDescriptor(modeId);
            var compatModeData = this._registry.getCompatMode(modeId);
            if (compatModeData) {
                // This is a compatibility mode
                var compatModeAsyncDescriptor = descriptors_1.createAsyncDescriptor1(compatModeData.moduleId, compatModeData.ctorName);
                return this._threadService.createInstance(compatModeAsyncDescriptor, modeDescriptor).then(function (compatMode) {
                    if (compatMode.configSupport) {
                        compatMode.configSupport.configure(_this.getConfigurationForMode(modeId));
                    }
                    return compatMode;
                });
            }
            return winjs_base_1.TPromise.as(this._threadService.createInstance(abstractMode_1.FrankensteinMode, modeDescriptor));
        };
        ModeServiceImpl.prototype._createModeDescriptor = function (modeId) {
            var workerParticipants = modesRegistry_1.ModesRegistry.getWorkerParticipantsForMode(modeId);
            return {
                id: modeId,
                workerParticipants: workerParticipants.map(function (p) { return descriptors_1.createAsyncDescriptor0(p.moduleId, p.ctorName); })
            };
        };
        ModeServiceImpl.prototype._registerModeSupport = function (mode, support, callback) {
            if (mode.registerSupport) {
                return mode.registerSupport(support, callback);
            }
            else {
                console.warn('Cannot register support ' + support + ' on mode ' + mode.getId() + ' because it does not support it.');
                return lifecycle_1.empty;
            }
        };
        ModeServiceImpl.prototype.registerModeSupport = function (modeId, support, callback) {
            var _this = this;
            if (this._instantiatedModes.hasOwnProperty(modeId)) {
                return this._registerModeSupport(this._instantiatedModes[modeId], support, callback);
            }
            var cc;
            var promise = new winjs_base_1.TPromise(function (c, e) { cc = c; });
            var disposable = this.onDidCreateMode(function (mode) {
                if (mode.getId() !== modeId) {
                    return;
                }
                cc(_this._registerModeSupport(mode, support, callback));
                disposable.dispose();
            });
            return {
                dispose: function () {
                    promise.done(function (disposable) { return disposable.dispose(); }, null);
                }
            };
        };
        ModeServiceImpl.prototype.doRegisterMonarchDefinition = function (modeId, lexer) {
            var _this = this;
            return lifecycle_1.combinedDispose(this.registerTokenizationSupport(modeId, function (mode) {
                return monarchLexer_1.createTokenizationSupport(_this, mode, lexer);
            }), this.registerRichEditSupport(modeId, monarchDefinition_1.createRichEditSupport(lexer)));
        };
        ModeServiceImpl.prototype.registerMonarchDefinition = function (modelService, editorWorkerService, modeId, language) {
            var lexer = monarchCompile_1.compile(objects.clone(language));
            return this.doRegisterMonarchDefinition(modeId, lexer);
        };
        ModeServiceImpl.prototype.registerCodeLensSupport = function (modeId, support) {
            return this.registerModeSupport(modeId, 'codeLensSupport', function (mode) { return support; });
        };
        ModeServiceImpl.prototype.registerRichEditSupport = function (modeId, support) {
            return this.registerModeSupport(modeId, 'richEditSupport', function (mode) { return new richEditSupport_1.RichEditSupport(modeId, mode.richEditSupport, support); });
        };
        ModeServiceImpl.prototype.registerDeclarativeDeclarationSupport = function (modeId, contribution) {
            return this.registerModeSupport(modeId, 'declarationSupport', function (mode) { return new declarationSupport_1.DeclarationSupport(modeId, contribution); });
        };
        ModeServiceImpl.prototype.registerExtraInfoSupport = function (modeId, support) {
            return this.registerModeSupport(modeId, 'extraInfoSupport', function (mode) { return support; });
        };
        ModeServiceImpl.prototype.registerFormattingSupport = function (modeId, support) {
            return this.registerModeSupport(modeId, 'formattingSupport', function (mode) { return support; });
        };
        ModeServiceImpl.prototype.registerInplaceReplaceSupport = function (modeId, support) {
            return this.registerModeSupport(modeId, 'inplaceReplaceSupport', function (mode) { return support; });
        };
        ModeServiceImpl.prototype.registerOccurrencesSupport = function (modeId, support) {
            return this.registerModeSupport(modeId, 'occurrencesSupport', function (mode) { return support; });
        };
        ModeServiceImpl.prototype.registerOutlineSupport = function (modeId, support) {
            return this.registerModeSupport(modeId, 'outlineSupport', function (mode) { return support; });
        };
        ModeServiceImpl.prototype.registerDeclarativeParameterHintsSupport = function (modeId, support) {
            return this.registerModeSupport(modeId, 'parameterHintsSupport', function (mode) { return new parameterHintsSupport_1.ParameterHintsSupport(modeId, support); });
        };
        ModeServiceImpl.prototype.registerQuickFixSupport = function (modeId, support) {
            return this.registerModeSupport(modeId, 'quickFixSupport', function (mode) { return support; });
        };
        ModeServiceImpl.prototype.registerDeclarativeReferenceSupport = function (modeId, contribution) {
            return this.registerModeSupport(modeId, 'referenceSupport', function (mode) { return new referenceSupport_1.ReferenceSupport(modeId, contribution); });
        };
        ModeServiceImpl.prototype.registerRenameSupport = function (modeId, support) {
            return this.registerModeSupport(modeId, 'renameSupport', function (mode) { return support; });
        };
        ModeServiceImpl.prototype.registerDeclarativeSuggestSupport = function (modeId, declaration) {
            return this.registerModeSupport(modeId, 'suggestSupport', function (mode) { return new suggestSupport_1.SuggestSupport(modeId, declaration); });
        };
        ModeServiceImpl.prototype.registerTokenizationSupport = function (modeId, callback) {
            return this.registerModeSupport(modeId, 'tokenizationSupport', callback);
        };
        return ModeServiceImpl;
    }());
    exports.ModeServiceImpl = ModeServiceImpl;
    var MainThreadModeServiceImpl = (function (_super) {
        __extends(MainThreadModeServiceImpl, _super);
        function MainThreadModeServiceImpl(threadService, extensionService, configurationService) {
            var _this = this;
            _super.call(this, threadService, extensionService);
            this._configurationService = configurationService;
            this._hasInitialized = false;
            languagesExtPoint.setHandler(function (extensions) {
                var allValidLanguages = [];
                for (var i = 0, len = extensions.length; i < len; i++) {
                    var extension = extensions[i];
                    if (!Array.isArray(extension.value)) {
                        extension.collector.error(nls.localize(17, null, languagesExtPoint.name));
                        continue;
                    }
                    for (var j = 0, lenJ = extension.value.length; j < lenJ; j++) {
                        var ext = extension.value[j];
                        if (isValidLanguageExtensionPoint(ext, extension.collector)) {
                            var configuration = (ext.configuration ? paths.join(extension.description.extensionFolderPath, ext.configuration) : ext.configuration);
                            allValidLanguages.push({
                                id: ext.id,
                                extensions: ext.extensions,
                                filenames: ext.filenames,
                                filenamePatterns: ext.filenamePatterns,
                                firstLine: ext.firstLine,
                                aliases: ext.aliases,
                                mimetypes: ext.mimetypes,
                                configuration: configuration
                            });
                        }
                    }
                }
                modesRegistry_1.ModesRegistry.registerLanguages(allValidLanguages);
            });
            this._configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) { return _this.onConfigurationChange(e.config); });
        }
        MainThreadModeServiceImpl.prototype.onReady = function () {
            var _this = this;
            if (!this._onReadyPromise) {
                this._onReadyPromise = this._configurationService.loadConfiguration().then(function (configuration) {
                    return _this._extensionService.onReady().then(function () {
                        _this.onConfigurationChange(configuration);
                        return true;
                    });
                });
            }
            return this._onReadyPromise;
        };
        MainThreadModeServiceImpl.prototype.onConfigurationChange = function (configuration) {
            var _this = this;
            // Clear user configured mime associations
            mime.clearTextMimes(true /* user configured */);
            // Register based on settings
            if (configuration.files && configuration.files.associations) {
                Object.keys(configuration.files.associations).forEach(function (pattern) {
                    mime.registerTextMime({ mime: _this.getMimeForMode(configuration.files.associations[pattern]), filepattern: pattern, userConfigured: true });
                });
            }
        };
        MainThreadModeServiceImpl.prototype._getModeServiceWorkerHelper = function () {
            var r = this._threadService.getRemotable(ModeServiceWorkerHelper);
            if (!this._hasInitialized) {
                this._hasInitialized = true;
                var initData = {
                    compatModes: modesRegistry_1.ModesRegistry.getCompatModes(),
                    languages: modesRegistry_1.ModesRegistry.getLanguages(),
                    workerParticipants: modesRegistry_1.ModesRegistry.getWorkerParticipants()
                };
                r._initialize(initData);
                modesRegistry_1.ModesRegistry.onDidAddCompatModes(function (m) { return r._acceptCompatModes(m); });
                modesRegistry_1.ModesRegistry.onDidAddLanguages(function (m) { return r._acceptLanguages(m); });
            }
            return r;
        };
        MainThreadModeServiceImpl.prototype.configureModeById = function (modeId, options) {
            this._getModeServiceWorkerHelper().configureModeById(modeId, options);
            _super.prototype.configureModeById.call(this, modeId, options);
        };
        MainThreadModeServiceImpl.prototype._createMode = function (modeId) {
            // Instantiate mode also in worker
            this._getModeServiceWorkerHelper().instantiateMode(modeId);
            return _super.prototype._createMode.call(this, modeId);
        };
        MainThreadModeServiceImpl.prototype.registerMonarchDefinition = function (modelService, editorWorkerService, modeId, language) {
            this._getModeServiceWorkerHelper().registerMonarchDefinition(modeId, language);
            var lexer = monarchCompile_1.compile(objects.clone(language));
            return lifecycle_1.combinedDispose(_super.prototype.doRegisterMonarchDefinition.call(this, modeId, lexer), this.registerModeSupport(modeId, 'suggestSupport', function (mode) {
                return monarchDefinition_1.createSuggestSupport(modelService, editorWorkerService, modeId, lexer);
            }));
        };
        return MainThreadModeServiceImpl;
    }(ModeServiceImpl));
    exports.MainThreadModeServiceImpl = MainThreadModeServiceImpl;
    var ModeServiceWorkerHelper = (function () {
        function ModeServiceWorkerHelper(modeService) {
            this._modeService = modeService;
        }
        ModeServiceWorkerHelper.prototype._initialize = function (initData) {
            modesRegistry_1.ModesRegistry.registerCompatModes(initData.compatModes);
            modesRegistry_1.ModesRegistry.registerLanguages(initData.languages);
            modesRegistry_1.ModesRegistry.registerWorkerParticipants(initData.workerParticipants);
        };
        ModeServiceWorkerHelper.prototype._acceptCompatModes = function (modes) {
            modesRegistry_1.ModesRegistry.registerCompatModes(modes);
        };
        ModeServiceWorkerHelper.prototype._acceptLanguages = function (languages) {
            modesRegistry_1.ModesRegistry.registerLanguages(languages);
        };
        ModeServiceWorkerHelper.prototype.instantiateMode = function (modeId) {
            this._modeService.getOrCreateMode(modeId).done(null, errors_1.onUnexpectedError);
        };
        ModeServiceWorkerHelper.prototype.configureModeById = function (modeId, options) {
            this._modeService.configureMode(modeId, options);
        };
        ModeServiceWorkerHelper.prototype.registerMonarchDefinition = function (modeId, language) {
            this._modeService.registerMonarchDefinition(null, null, modeId, language);
        };
        ModeServiceWorkerHelper = __decorate([
            thread_1.Remotable.WorkerContext('ModeServiceWorkerHelper', thread_1.ThreadAffinity.All),
            __param(0, modeService_1.IModeService)
        ], ModeServiceWorkerHelper);
        return ModeServiceWorkerHelper;
    }());
    exports.ModeServiceWorkerHelper = ModeServiceWorkerHelper;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/common/services/modelServiceImpl", ["require", "exports", 'vs/nls!vs/editor/common/services/modelServiceImpl', 'vs/base/common/errors', 'vs/base/common/event', 'vs/base/common/severity', 'vs/base/common/uri', 'vs/platform/telemetry/common/telemetry', 'vs/platform/thread/common/thread', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/model/mirrorModel', 'vs/editor/common/model/model', 'vs/editor/common/services/modeService', 'vs/editor/common/services/modelService', 'vs/editor/common/services/resourceService', 'vs/base/common/platform', 'vs/platform/configuration/common/configuration', 'vs/editor/common/config/defaultConfig'], function (require, exports, nls, errors_1, event_1, severity_1, uri_1, telemetry_1, thread_1, range_1, editorCommon, mirrorModel_1, model_1, modeService_1, modelService_1, resourceService_1, platform, configuration_1, defaultConfig_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function MODEL_ID(resource) {
        return resource.toString();
    }
    var ModelData = (function () {
        function ModelData(model, eventsHandler) {
            var _this = this;
            this.model = model;
            this.isSyncedToWorkers = false;
            this._markerDecorations = [];
            this._modelEventsListener = model.addBulkListener2(function (events) { return eventsHandler(_this, events); });
        }
        ModelData.prototype.dispose = function () {
            this._markerDecorations = this.model.deltaDecorations(this._markerDecorations, []);
            this._modelEventsListener.dispose();
            this._modelEventsListener = null;
            this.model = null;
        };
        ModelData.prototype.getModelId = function () {
            return MODEL_ID(this.model.getAssociatedResource());
        };
        ModelData.prototype.acceptMarkerDecorations = function (newDecorations) {
            this._markerDecorations = this.model.deltaDecorations(this._markerDecorations, newDecorations);
        };
        return ModelData;
    }());
    var ModelMarkerHandler = (function () {
        function ModelMarkerHandler() {
        }
        ModelMarkerHandler.setMarkers = function (modelData, markers) {
            var _this = this;
            // Limit to the first 500 errors/warnings
            markers = markers.slice(0, 500);
            var newModelDecorations = markers.map(function (marker) {
                return {
                    range: _this._createDecorationRange(modelData.model, marker),
                    options: _this._createDecorationOption(marker)
                };
            });
            modelData.acceptMarkerDecorations(newModelDecorations);
        };
        ModelMarkerHandler._createDecorationRange = function (model, rawMarker) {
            var marker = model.validateRange(new range_1.Range(rawMarker.startLineNumber, rawMarker.startColumn, rawMarker.endLineNumber, rawMarker.endColumn));
            var ret = new range_1.Range(marker.startLineNumber, marker.startColumn, marker.endLineNumber, marker.endColumn);
            if (ret.isEmpty()) {
                var word = model.getWordAtPosition(ret.getStartPosition());
                if (word) {
                    ret.startColumn = word.startColumn;
                    ret.endColumn = word.endColumn;
                }
                else {
                    var maxColumn = model.getLineLastNonWhitespaceColumn(marker.startLineNumber) ||
                        model.getLineMaxColumn(marker.startLineNumber);
                    if (maxColumn === 1) {
                    }
                    else if (ret.endColumn >= maxColumn) {
                        // behind eol
                        ret.endColumn = maxColumn;
                        ret.startColumn = maxColumn - 1;
                    }
                    else {
                        // extend marker to width = 1
                        ret.endColumn += 1;
                    }
                }
            }
            else if (rawMarker.endColumn === Number.MAX_VALUE && rawMarker.startColumn === 1 && ret.startLineNumber === ret.endLineNumber) {
                var minColumn = model.getLineFirstNonWhitespaceColumn(rawMarker.startLineNumber);
                if (minColumn < ret.endColumn) {
                    ret.startColumn = minColumn;
                    rawMarker.startColumn = minColumn;
                }
            }
            return ret;
        };
        ModelMarkerHandler._createDecorationOption = function (marker) {
            var className;
            var color;
            var darkColor;
            var htmlMessage = null;
            switch (marker.severity) {
                case severity_1.default.Ignore:
                    // do something
                    break;
                case severity_1.default.Warning:
                case severity_1.default.Info:
                    className = editorCommon.ClassName.EditorWarningDecoration;
                    color = 'rgba(18,136,18,0.7)';
                    darkColor = 'rgba(18,136,18,0.7)';
                    break;
                case severity_1.default.Error:
                default:
                    className = editorCommon.ClassName.EditorErrorDecoration;
                    color = 'rgba(255,18,18,0.7)';
                    darkColor = 'rgba(255,18,18,0.7)';
                    break;
            }
            if (typeof marker.message === 'string') {
                htmlMessage = [{ isText: true, text: marker.message }];
            }
            else if (Array.isArray(marker.message)) {
                htmlMessage = marker.message;
            }
            else if (marker.message) {
                htmlMessage = [marker.message];
            }
            if (marker.source) {
                htmlMessage.unshift({ isText: true, text: "[" + marker.source + "] " });
            }
            return {
                stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
                className: className,
                htmlMessage: htmlMessage,
                overviewRuler: {
                    color: color,
                    darkColor: darkColor,
                    position: editorCommon.OverviewRulerLane.Right
                }
            };
        };
        return ModelMarkerHandler;
    }());
    var ModelServiceImpl = (function () {
        function ModelServiceImpl(threadService, markerService, modeService, configurationService, messageService) {
            var _this = this;
            this.serviceId = modelService_1.IModelService;
            this._modelCreationOptions = {
                tabSize: defaultConfig_1.DEFAULT_INDENTATION.tabSize,
                insertSpaces: defaultConfig_1.DEFAULT_INDENTATION.insertSpaces,
                detectIndentation: defaultConfig_1.DEFAULT_INDENTATION.detectIndentation,
                defaultEOL: (platform.isLinux || platform.isMacintosh) ? editorCommon.DefaultEndOfLine.LF : editorCommon.DefaultEndOfLine.CRLF
            };
            this._threadService = threadService;
            this._markerService = markerService;
            this._modeService = modeService;
            this._workerHelper = this._threadService.getRemotable(ModelServiceWorkerHelper);
            this._configurationService = configurationService;
            this._messageService = messageService;
            this._hasShownMigrationMessage = false;
            var readConfig = function (config) {
                var eol = config.files && config.files.eol;
                var shouldShowMigrationMessage = false;
                var tabSize = defaultConfig_1.DEFAULT_INDENTATION.tabSize;
                if (config.editor && typeof config.editor.tabSize !== 'undefined') {
                    var parsedTabSize = parseInt(config.editor.tabSize, 10);
                    if (!isNaN(parsedTabSize)) {
                        tabSize = parsedTabSize;
                    }
                    shouldShowMigrationMessage = shouldShowMigrationMessage || (config.editor.tabSize === 'auto');
                }
                var insertSpaces = defaultConfig_1.DEFAULT_INDENTATION.insertSpaces;
                if (config.editor && typeof config.editor.insertSpaces !== 'undefined') {
                    insertSpaces = (config.editor.insertSpaces === 'false' ? false : Boolean(config.editor.insertSpaces));
                    shouldShowMigrationMessage = shouldShowMigrationMessage || (config.editor.insertSpaces === 'auto');
                }
                var newDefaultEOL = _this._modelCreationOptions.defaultEOL;
                if (eol === '\r\n') {
                    newDefaultEOL = editorCommon.DefaultEndOfLine.CRLF;
                }
                else if (eol === '\n') {
                    newDefaultEOL = editorCommon.DefaultEndOfLine.LF;
                }
                var detectIndentation = defaultConfig_1.DEFAULT_INDENTATION.detectIndentation;
                if (config.editor && typeof config.editor.detectIndentation !== 'undefined') {
                    detectIndentation = (config.editor.detectIndentation === 'false' ? false : Boolean(config.editor.detectIndentation));
                }
                _this._setModelOptions({
                    tabSize: tabSize,
                    insertSpaces: insertSpaces,
                    detectIndentation: detectIndentation,
                    defaultEOL: newDefaultEOL
                });
                if (shouldShowMigrationMessage && !_this._hasShownMigrationMessage) {
                    _this._hasShownMigrationMessage = true;
                    _this._messageService.show(severity_1.default.Info, nls.localize(0, null));
                }
            };
            this._configurationServiceSubscription = this._configurationService.addListener2(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) {
                readConfig(e.config);
            });
            this._configurationService.loadConfiguration().then(function (config) {
                readConfig(config);
            });
            this._models = {};
            this._onModelAdded = new event_1.Emitter();
            this._onModelRemoved = new event_1.Emitter();
            this._onModelModeChanged = new event_1.Emitter();
            if (this._markerService) {
                this._markerServiceSubscription = this._markerService.onMarkerChanged(this._handleMarkerChange, this);
            }
        }
        ModelServiceImpl.prototype.getCreationOptions = function () {
            return this._modelCreationOptions;
        };
        ModelServiceImpl.prototype._setModelOptions = function (newOpts) {
            if ((this._modelCreationOptions.detectIndentation === newOpts.detectIndentation)
                && (this._modelCreationOptions.insertSpaces === newOpts.insertSpaces)
                && (this._modelCreationOptions.tabSize === newOpts.tabSize)) {
                // Same indent opts, no need to touch created models
                this._modelCreationOptions = newOpts;
                return;
            }
            this._modelCreationOptions = newOpts;
            // Update options on all models
            for (var modelId in this._models) {
                if (this._models.hasOwnProperty(modelId)) {
                    var modelData = this._models[modelId];
                    if (this._modelCreationOptions.detectIndentation) {
                        modelData.model.detectIndentation(this._modelCreationOptions.insertSpaces, this._modelCreationOptions.tabSize);
                    }
                    else {
                        modelData.model.updateOptions({
                            insertSpaces: this._modelCreationOptions.insertSpaces,
                            tabSize: this._modelCreationOptions.tabSize
                        });
                    }
                }
            }
        };
        ModelServiceImpl.prototype.dispose = function () {
            if (this._markerServiceSubscription) {
                this._markerServiceSubscription.dispose();
            }
            this._configurationServiceSubscription.dispose();
        };
        ModelServiceImpl.prototype._handleMarkerChange = function (changedResources) {
            var _this = this;
            changedResources.forEach(function (resource) {
                var modelId = MODEL_ID(resource);
                var modelData = _this._models[modelId];
                if (!modelData) {
                    return;
                }
                ModelMarkerHandler.setMarkers(modelData, _this._markerService.read({ resource: resource, take: 500 }));
            });
        };
        // --- begin IModelService
        ModelServiceImpl.prototype._shouldSyncModelToWorkers = function (model) {
            if (model.isTooLargeForHavingARichMode()) {
                return false;
            }
            // Only sync models with compat modes to the workers
            return this._modeService.isCompatMode(model.getMode().getId());
        };
        ModelServiceImpl.prototype._createModelData = function (value, modeOrPromise, resource) {
            var _this = this;
            // create & save the model
            var model = new model_1.Model(value, this._modelCreationOptions, modeOrPromise, resource);
            var modelId = MODEL_ID(model.getAssociatedResource());
            if (this._models[modelId]) {
                // There already exists a model with this id => this is a programmer error
                throw new Error('ModelService: Cannot add model ' + telemetry_1.anonymize(modelId) + ' because it already exists!');
            }
            var modelData = new ModelData(model, function (modelData, events) { return _this._onModelEvents(modelData, events); });
            this._models[modelId] = modelData;
            return modelData;
        };
        ModelServiceImpl.prototype.createModel = function (value, modeOrPromise, resource) {
            var modelData = this._createModelData(value, modeOrPromise, resource);
            // handle markers (marker service => model)
            if (this._markerService) {
                ModelMarkerHandler.setMarkers(modelData, this._markerService.read({ resource: modelData.model.getAssociatedResource() }));
            }
            if (this._shouldSyncModelToWorkers(modelData.model)) {
                // send this model to the workers
                this._beginWorkerSync(modelData);
            }
            this._onModelAdded.fire(modelData.model);
            return modelData.model;
        };
        ModelServiceImpl.prototype.destroyModel = function (resource) {
            // We need to support that not all models get disposed through this service (i.e. model.dispose() should work!)
            var modelData = this._models[MODEL_ID(resource)];
            if (!modelData) {
                return;
            }
            modelData.model.dispose();
        };
        ModelServiceImpl.prototype.getModels = function () {
            var ret = [];
            for (var modelId in this._models) {
                if (this._models.hasOwnProperty(modelId)) {
                    ret.push(this._models[modelId].model);
                }
            }
            return ret;
        };
        ModelServiceImpl.prototype.getModel = function (resource) {
            var modelId = MODEL_ID(resource);
            var modelData = this._models[modelId];
            if (!modelData) {
                return null;
            }
            return modelData.model;
        };
        Object.defineProperty(ModelServiceImpl.prototype, "onModelAdded", {
            get: function () {
                return this._onModelAdded ? this._onModelAdded.event : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ModelServiceImpl.prototype, "onModelRemoved", {
            get: function () {
                return this._onModelRemoved ? this._onModelRemoved.event : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ModelServiceImpl.prototype, "onModelModeChanged", {
            get: function () {
                return this._onModelModeChanged ? this._onModelModeChanged.event : null;
            },
            enumerable: true,
            configurable: true
        });
        // --- end IModelService
        ModelServiceImpl.prototype._beginWorkerSync = function (modelData) {
            if (modelData.isSyncedToWorkers) {
                throw new Error('Model is already being synced to workers!');
            }
            modelData.isSyncedToWorkers = true;
            this._workerHelper.$_acceptNewModel(ModelServiceImpl._getBoundModelData(modelData.model));
        };
        ModelServiceImpl.prototype._stopWorkerSync = function (modelData) {
            if (!modelData.isSyncedToWorkers) {
                throw new Error('Model is already not being synced to workers!');
            }
            modelData.isSyncedToWorkers = false;
            this._workerHelper.$_acceptDidDisposeModel(modelData.model.getAssociatedResource());
        };
        ModelServiceImpl.prototype._onModelDisposing = function (model) {
            var _this = this;
            var modelId = MODEL_ID(model.getAssociatedResource());
            var modelData = this._models[modelId];
            // TODO@Joh why are we removing markers here?
            if (this._markerService) {
                var markers = this._markerService.read({ resource: model.getAssociatedResource() }), owners = Object.create(null);
                markers.forEach(function (marker) { return owners[marker.owner] = _this; });
                Object.keys(owners).forEach(function (owner) { return _this._markerService.changeOne(owner, model.getAssociatedResource(), []); });
            }
            if (modelData.isSyncedToWorkers) {
                // Dispose model in workers
                this._stopWorkerSync(modelData);
            }
            delete this._models[modelId];
            modelData.dispose();
            this._onModelRemoved.fire(model);
        };
        ModelServiceImpl._getBoundModelData = function (model) {
            return {
                url: model.getAssociatedResource(),
                versionId: model.getVersionId(),
                value: model.toRawText(),
                modeId: model.getMode().getId()
            };
        };
        ModelServiceImpl.prototype._onModelEvents = function (modelData, events) {
            // First look for dispose
            for (var i = 0, len = events.length; i < len; i++) {
                var e = events[i];
                if (e.getType() === editorCommon.EventType.ModelDispose) {
                    this._onModelDisposing(modelData.model);
                    // no more processing since model got disposed
                    return;
                }
            }
            // Second, look for mode change
            for (var i = 0, len = events.length; i < len; i++) {
                var e = events[i];
                if (e.getType() === editorCommon.EventType.ModelModeChanged) {
                    var wasSyncedToWorkers = modelData.isSyncedToWorkers;
                    var shouldSyncToWorkers = this._shouldSyncModelToWorkers(modelData.model);
                    this._onModelModeChanged.fire({
                        model: modelData.model,
                        oldModeId: e.getData().oldMode.getId()
                    });
                    if (wasSyncedToWorkers) {
                        if (shouldSyncToWorkers) {
                            // true -> true
                            // Forward mode change to all the workers
                            this._workerHelper.$_acceptDidChangeModelMode(modelData.getModelId(), modelData.model.getMode().getId());
                        }
                        else {
                            // true -> false
                            // Stop worker sync for this model
                            this._stopWorkerSync(modelData);
                            // no more processing since we have removed the model from the workers
                            return;
                        }
                    }
                    else {
                        if (shouldSyncToWorkers) {
                            // false -> true
                            // Begin syncing this model to the workers
                            this._beginWorkerSync(modelData);
                            // no more processing since we are sending the latest state
                            return;
                        }
                        else {
                            // false -> false
                            // no more processing since this model was not synced and will not be synced
                            return;
                        }
                    }
                }
            }
            if (!modelData.isSyncedToWorkers) {
                return;
            }
            // Finally, look for model content changes
            var eventsForWorkers = { contentChanged: [] };
            for (var i = 0, len = events.length; i < len; i++) {
                var e = events[i];
                if (e.getType() === editorCommon.EventType.ModelContentChanged) {
                    eventsForWorkers.contentChanged.push(e.getData());
                }
            }
            if (eventsForWorkers.contentChanged.length > 0) {
                // Forward events to all the workers
                this._workerHelper.$_acceptModelEvents(modelData.getModelId(), eventsForWorkers);
            }
        };
        return ModelServiceImpl;
    }());
    exports.ModelServiceImpl = ModelServiceImpl;
    var ModelServiceWorkerHelper = (function () {
        function ModelServiceWorkerHelper(resourceService, modeService) {
            this._resourceService = resourceService;
            this._modeService = modeService;
        }
        ModelServiceWorkerHelper.prototype.$_acceptNewModel = function (data) {
            var _this = this;
            // Create & insert the mirror model eagerly in the resource service
            var mirrorModel = new mirrorModel_1.MirrorModel(this._resourceService, data.versionId, data.value, null, data.url);
            this._resourceService.insert(mirrorModel.getAssociatedResource(), mirrorModel);
            // Block worker execution until the mode is instantiated
            return this._modeService.getOrCreateMode(data.modeId).then(function (mode) {
                // Changing mode should trigger a remove & an add, therefore:
                // (1) Remove from resource service
                _this._resourceService.remove(mirrorModel.getAssociatedResource());
                // (2) Change mode
                mirrorModel.setMode(mode);
                // (3) Insert again to resource service (it will have the new mode)
                _this._resourceService.insert(mirrorModel.getAssociatedResource(), mirrorModel);
            });
        };
        ModelServiceWorkerHelper.prototype.$_acceptDidChangeModelMode = function (modelId, newModeId) {
            var _this = this;
            var mirrorModel = this._resourceService.get(uri_1.default.parse(modelId));
            // Block worker execution until the mode is instantiated
            return this._modeService.getOrCreateMode(newModeId).then(function (mode) {
                // Changing mode should trigger a remove & an add, therefore:
                // (1) Remove from resource service
                _this._resourceService.remove(mirrorModel.getAssociatedResource());
                // (2) Change mode
                mirrorModel.setMode(mode);
                // (3) Insert again to resource service (it will have the new mode)
                _this._resourceService.insert(mirrorModel.getAssociatedResource(), mirrorModel);
            });
        };
        ModelServiceWorkerHelper.prototype.$_acceptDidDisposeModel = function (url) {
            var model = this._resourceService.get(url);
            this._resourceService.remove(url);
            if (model) {
                model.dispose();
            }
        };
        ModelServiceWorkerHelper.prototype.$_acceptModelEvents = function (modelId, events) {
            var model = this._resourceService.get(uri_1.default.parse(modelId));
            if (!model) {
                throw new Error('Received model events for missing model ' + telemetry_1.anonymize(modelId));
            }
            try {
                model.onEvents(events);
            }
            catch (err) {
                errors_1.onUnexpectedError(err);
            }
        };
        ModelServiceWorkerHelper = __decorate([
            thread_1.Remotable.WorkerContext('ModelServiceWorkerHelper', thread_1.ThreadAffinity.All),
            __param(0, resourceService_1.IResourceService),
            __param(1, modeService_1.IModeService)
        ], ModelServiceWorkerHelper);
        return ModelServiceWorkerHelper;
    }());
    exports.ModelServiceWorkerHelper = ModelServiceWorkerHelper;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/workerStatusReporter/browser/workerStatusReporter", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/browser/browser', 'vs/platform/thread/common/thread', 'vs/editor/browser/editorBrowser', 'vs/css!./workerStatusReporter'], function (require, exports, lifecycle_1, browser, thread_1, editorBrowser_1) {
    'use strict';
    var WorkerStatusReporter = (function () {
        function WorkerStatusReporter(editor, threadService) {
            this._threadService = threadService;
            this._threadService.addStatusListener(this);
            this._editor = editor;
            this._toDispose = [];
            this._domNodes = [];
            this._domNode = document.createElement('div');
            this._domNode.className = 'monaco-worker-status';
            if (browser.canUseTranslate3d) {
                // Put the worker reporter in its own layer
                this._domNode.style.transform = 'translate3d(0px, 0px, 0px)';
            }
            this._editor.addOverlayWidget(this);
        }
        WorkerStatusReporter.prototype.getId = function () {
            return WorkerStatusReporter.ID;
        };
        WorkerStatusReporter.prototype.dispose = function () {
            this._threadService.removeStatusListener(this);
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        WorkerStatusReporter.prototype.getDomNode = function () {
            return this._domNode;
        };
        WorkerStatusReporter.prototype.getPosition = function () {
            return { preference: editorBrowser_1.OverlayWidgetPositionPreference.TOP_RIGHT_CORNER };
        };
        WorkerStatusReporter.prototype._ensureDomNodes = function (desiredCount) {
            // Remove extra dom nodes
            for (var i = this._domNodes.length - 1; i >= desiredCount; i++) {
                this._domNode.removeChild(this._domNodes[i]);
                this._domNodes.splice(i, 1);
            }
            // Create new dom nodes
            for (var i = this._domNodes.length; i < desiredCount; i++) {
                this._domNodes[i] = document.createElement('div');
                this._domNodes[i].className = 'worker';
                this._domNode.appendChild(this._domNodes[i]);
            }
        };
        WorkerStatusReporter.prototype.onThreadServiceStatus = function (status) {
            this._ensureDomNodes(status.workers.length);
            for (var i = 0; i < status.workers.length; i++) {
                var cnt = status.workers[i].queueSize;
                var workerStatus = 'idle';
                if (cnt > 5) {
                    workerStatus = 'flooded';
                }
                else if (cnt > 0) {
                    workerStatus = 'busy';
                }
                attr(this._domNodes[i], 'status', workerStatus);
            }
        };
        WorkerStatusReporter.ID = 'editor.contrib.workerStatusReporter';
        WorkerStatusReporter = __decorate([
            __param(1, thread_1.IThreadService)
        ], WorkerStatusReporter);
        return WorkerStatusReporter;
    }());
    function attr(target, attrName, attrValue) {
        target.setAttribute(attrName, attrValue);
    }
});
// if (false) { //TODO@Debt
// 	EditorBrowserRegistry.registerEditorContribution(WorkerStatusReporter);
// } 

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
define("vs/platform/markers/common/markerService", ["require", "exports", 'vs/base/common/arrays', 'vs/base/common/network', 'vs/base/common/strings', 'vs/base/common/collections', 'vs/base/common/uri', 'vs/base/common/event', 'vs/base/common/severity', 'vs/platform/thread/common/thread', './markers'], function (require, exports, arrays, network, strings, collections, uri_1, event_1, severity_1, thread_1, markers_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Key;
    (function (Key) {
        function fromValue(value) {
            var regexp = /^(.*)→(.*)$/.exec(value);
            return {
                owner: regexp[1],
                resource: uri_1.default.parse(regexp[2])
            };
        }
        Key.fromValue = fromValue;
        function valueOf(k) {
            return k.owner + '→' + k.resource;
        }
        Key.valueOf = valueOf;
        var _selectorPattern = '^({0})→({1})$';
        function selector(owner, resource) {
            return new RegExp(strings.format(_selectorPattern, owner ? strings.escapeRegExpCharacters(owner) : '.*', resource ? strings.escapeRegExpCharacters(resource.toString()) : '.*'));
        }
        Key.selector = selector;
        function raw(owner, resource) {
            return owner + '→' + resource;
        }
        Key.raw = raw;
    })(Key || (Key = {}));
    var MarkerService = (function () {
        function MarkerService() {
            this.serviceId = markers_1.IMarkerService;
            this._data = Object.create(null);
            this._stats = this._emptyStats();
            this._onMarkerChanged = new event_1.Emitter();
        }
        MarkerService.prototype.getStatistics = function () {
            return this._stats;
        };
        Object.defineProperty(MarkerService.prototype, "onMarkerChanged", {
            // ---- IMarkerService ------------------------------------------
            get: function () {
                return this._onMarkerChanged ? this._onMarkerChanged.event : null;
            },
            enumerable: true,
            configurable: true
        });
        MarkerService.prototype.changeOne = function (owner, resource, markers) {
            if (this._doChangeOne(owner, resource, markers)) {
                this._onMarkerChanged.fire([resource]);
            }
        };
        MarkerService.prototype.remove = function (owner, resources) {
            if (arrays.isFalsyOrEmpty(resources)) {
                return;
            }
            var changedResources;
            for (var _i = 0, resources_1 = resources; _i < resources_1.length; _i++) {
                var resource = resources_1[_i];
                if (this._doChangeOne(owner, resource, undefined)) {
                    if (!changedResources) {
                        changedResources = [];
                    }
                    changedResources.push(resource);
                }
            }
            if (changedResources) {
                this._onMarkerChanged.fire(changedResources);
            }
        };
        MarkerService.prototype._doChangeOne = function (owner, resource, markers) {
            var key = Key.raw(owner, resource), oldMarkers = this._data[key], hasOldMarkers = !arrays.isFalsyOrEmpty(oldMarkers), getsNewMarkers = !arrays.isFalsyOrEmpty(markers), oldStats = this._computeStats(oldMarkers), newStats = this._computeStats(markers);
            if (!hasOldMarkers && !getsNewMarkers) {
                return;
            }
            if (getsNewMarkers) {
                this._data[key] = markers;
            }
            else if (hasOldMarkers) {
                delete this._data[key];
            }
            if (this._isStatRelevant(resource)) {
                this._updateStatsMinus(oldStats);
                this._updateStatsPlus(newStats);
            }
            return true;
        };
        MarkerService.prototype.changeAll = function (owner, data) {
            var _this = this;
            var changedResources = Object.create(null);
            // remove and record old markers
            var oldStats = this._emptyStats();
            this._forEach(owner, undefined, undefined, -1, function (e, r) {
                var resource = Key.fromValue(e.key).resource;
                if (_this._isStatRelevant(resource)) {
                    _this._updateStatsPlus(oldStats, _this._computeStats(e.value));
                }
                changedResources[resource.toString()] = resource;
                r();
            });
            this._updateStatsMinus(oldStats);
            // add and record new markers
            if (!arrays.isFalsyOrEmpty(data)) {
                var newStats_1 = this._emptyStats();
                data.forEach(function (d) {
                    changedResources[d.resource.toString()] = d.resource;
                    collections.lookupOrInsert(_this._data, Key.raw(owner, d.resource), []).push(d.marker);
                    if (_this._isStatRelevant(d.resource)) {
                        _this._updateStatsMarker(newStats_1, d.marker);
                    }
                });
                this._updateStatsPlus(newStats_1);
            }
            this._onMarkerChanged.fire(collections.values(changedResources));
        };
        MarkerService.prototype.read = function (filter) {
            var _this = this;
            if (filter === void 0) { filter = Object.create(null); }
            var ret = [];
            this._forEach(filter.owner, filter.resource, filter.selector, filter.take, function (entry) { return _this._fromEntry(entry, ret); });
            return ret;
        };
        MarkerService.prototype._isStatRelevant = function (resource) {
            //TODO@Dirk this is a hack
            return resource.scheme !== network.Schemas.inMemory;
        };
        MarkerService.prototype._forEach = function (owner, resource, regexp, take, callback) {
            //TODO@Joh: be smart and use an index
            var selector = regexp || Key.selector(owner, resource), took = 0;
            collections.forEach(this._data, function (entry, remove) {
                if (selector.test(entry.key)) {
                    callback(entry, remove);
                    if (take > 0 && took++ >= take) {
                        return false;
                    }
                }
            });
        };
        MarkerService.prototype._fromEntry = function (entry, bucket) {
            var key = Key.fromValue(entry.key);
            entry.value.forEach(function (data) {
                // before reading, we sanitize the data
                MarkerService._sanitize(data);
                bucket.push({
                    owner: key.owner,
                    resource: key.resource,
                    code: data.code,
                    message: data.message,
                    source: data.source,
                    severity: data.severity,
                    startLineNumber: data.startLineNumber,
                    startColumn: data.startColumn,
                    endLineNumber: data.endLineNumber,
                    endColumn: data.endColumn
                });
            });
        };
        MarkerService.prototype._computeStats = function (markers) {
            var errors = 0, warnings = 0, infos = 0, unknwons = 0;
            if (markers) {
                for (var i = 0; i < markers.length; i++) {
                    var marker = markers[i];
                    if (marker.severity) {
                        switch (marker.severity) {
                            case severity_1.default.Error:
                                errors++;
                                break;
                            case severity_1.default.Warning:
                                warnings++;
                                break;
                            case severity_1.default.Info:
                                infos++;
                                break;
                            default:
                                unknwons++;
                                break;
                        }
                    }
                    else {
                        unknwons++;
                    }
                }
            }
            return {
                errors: errors,
                warnings: warnings,
                infos: infos,
                unknwons: unknwons
            };
        };
        MarkerService.prototype._emptyStats = function () {
            return { errors: 0, warnings: 0, infos: 0, unknwons: 0 };
        };
        MarkerService.prototype._updateStatsPlus = function (toUpdate, toAdd) {
            if (!toAdd) {
                toAdd = toUpdate;
                toUpdate = this._stats;
            }
            toUpdate.errors += toAdd.errors;
            toUpdate.warnings += toAdd.warnings;
            toUpdate.infos += toAdd.infos;
            toUpdate.unknwons += toAdd.unknwons;
        };
        MarkerService.prototype._updateStatsMinus = function (toUpdate, toSubtract) {
            if (!toSubtract) {
                toSubtract = toUpdate;
                toUpdate = this._stats;
            }
            toUpdate.errors -= toSubtract.errors;
            toUpdate.warnings -= toSubtract.warnings;
            toUpdate.infos -= toSubtract.infos;
            toUpdate.unknwons -= toSubtract.unknwons;
        };
        MarkerService.prototype._updateStatsMarker = function (toUpdate, marker) {
            switch (marker.severity) {
                case severity_1.default.Error:
                    toUpdate.errors++;
                    break;
                case severity_1.default.Warning:
                    toUpdate.warnings++;
                    break;
                case severity_1.default.Info:
                    toUpdate.infos++;
                    break;
                default:
                    toUpdate.unknwons++;
                    break;
            }
        };
        MarkerService._sanitize = function (data) {
            data.code = data.code || null;
            data.startLineNumber = data.startLineNumber > 0 ? data.startLineNumber : 1;
            data.startColumn = data.startColumn > 0 ? data.startColumn : 1;
            data.endLineNumber = data.endLineNumber >= data.startLineNumber ? data.endLineNumber : data.startLineNumber;
            data.endColumn = data.endColumn > 0 ? data.endColumn : data.startColumn;
        };
        return MarkerService;
    }());
    exports.MarkerService = MarkerService;
    var SecondaryMarkerService = (function (_super) {
        __extends(SecondaryMarkerService, _super);
        function SecondaryMarkerService(threadService) {
            _super.call(this);
            this._proxy = threadService.getRemotable(MainProcessMarkerService);
        }
        SecondaryMarkerService.prototype.changeOne = function (owner, resource, markers) {
            this._proxy.changeOne(owner, resource, markers);
        };
        SecondaryMarkerService.prototype.changeAll = function (owner, data) {
            this._proxy.changeAll(owner, data);
        };
        return SecondaryMarkerService;
    }(MarkerService));
    exports.SecondaryMarkerService = SecondaryMarkerService;
    var MainProcessMarkerService = (function (_super) {
        __extends(MainProcessMarkerService, _super);
        function MainProcessMarkerService(threadService) {
            _super.call(this);
            threadService.registerRemotableInstance(MainProcessMarkerService, this);
        }
        MainProcessMarkerService.prototype.changeOne = function (owner, resource, markers) {
            _super.prototype.changeOne.call(this, owner, resource, markers);
        };
        MainProcessMarkerService.prototype.changeAll = function (owner, data) {
            _super.prototype.changeAll.call(this, owner, data);
        };
        MainProcessMarkerService = __decorate([
            thread_1.Remotable.MainContext('MainProcessMarkerService')
        ], MainProcessMarkerService);
        return MainProcessMarkerService;
    }(MarkerService));
    exports.MainProcessMarkerService = MainProcessMarkerService;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/platform/storage/common/remotable.storage", ["require", "exports", 'vs/base/common/winjs.base', 'vs/platform/thread/common/thread', 'vs/platform/storage/common/storage'], function (require, exports, winjs_base_1, thread_1, storage_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var MainThreadStorage = (function () {
        function MainThreadStorage(storageService) {
            this._storageService = storageService;
        }
        MainThreadStorage.prototype.getValue = function (shared, key) {
            var jsonValue = this._storageService.get(key, shared ? storage_1.StorageScope.GLOBAL : storage_1.StorageScope.WORKSPACE);
            if (!jsonValue) {
                return winjs_base_1.TPromise.as(undefined);
            }
            var value;
            try {
                value = JSON.parse(jsonValue);
                return winjs_base_1.TPromise.as(value);
            }
            catch (err) {
                return winjs_base_1.TPromise.wrapError(err);
            }
        };
        MainThreadStorage.prototype.setValue = function (shared, key, value) {
            var jsonValue;
            try {
                jsonValue = JSON.stringify(value);
                this._storageService.store(key, jsonValue, shared ? storage_1.StorageScope.GLOBAL : storage_1.StorageScope.WORKSPACE);
            }
            catch (err) {
                return winjs_base_1.TPromise.wrapError(err);
            }
        };
        MainThreadStorage = __decorate([
            thread_1.Remotable.MainContext('MainThreadStorage'),
            __param(0, storage_1.IStorageService)
        ], MainThreadStorage);
        return MainThreadStorage;
    }());
    exports.MainThreadStorage = MainThreadStorage;
    var ExtHostStorage = (function () {
        function ExtHostStorage(threadService) {
            this._proxy = threadService.getRemotable(MainThreadStorage);
        }
        ExtHostStorage.prototype.getValue = function (shared, key, defaultValue) {
            return this._proxy.getValue(shared, key).then(function (value) { return value || defaultValue; });
        };
        ExtHostStorage.prototype.setValue = function (shared, key, value) {
            return this._proxy.setValue(shared, key, value);
        };
        return ExtHostStorage;
    }());
    exports.ExtHostStorage = ExtHostStorage;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
define("vs/platform/extensions/common/nativeExtensionService", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/common/paths', 'vs/base/common/severity', 'vs/base/common/winjs.base', 'vs/platform/extensions/common/abstractExtensionService', 'vs/platform/extensions/common/extensionsRegistry', 'vs/platform/storage/common/remotable.storage', 'vs/platform/thread/common/thread'], function (require, exports, lifecycle_1, paths, severity_1, winjs_base_1, abstractExtensionService_1, extensionsRegistry_1, remotable_storage_1, thread_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var hasOwnProperty = Object.hasOwnProperty;
    /**
     * Represents a failed extension in the ext host.
     */
    var MainProcessFailedExtension = (function (_super) {
        __extends(MainProcessFailedExtension, _super);
        function MainProcessFailedExtension() {
            _super.call(this, true);
        }
        return MainProcessFailedExtension;
    }(abstractExtensionService_1.ActivatedExtension));
    /**
     * Represents an extension that was successfully loaded or an
     * empty extension in the ext host.
     */
    var MainProcessSuccessExtension = (function (_super) {
        __extends(MainProcessSuccessExtension, _super);
        function MainProcessSuccessExtension() {
            _super.call(this, false);
        }
        return MainProcessSuccessExtension;
    }(abstractExtensionService_1.ActivatedExtension));
    function messageWithSource(msg) {
        return (msg.source ? '[' + msg.source + ']: ' : '') + msg.message;
    }
    var MainProcessExtensionService = (function (_super) {
        __extends(MainProcessExtensionService, _super);
        /**
         * This class is constructed manually because it is a service, so it doesn't use any ctor injection
         */
        function MainProcessExtensionService(contextService, threadService, messageService, telemetryService) {
            var _this = this;
            _super.call(this, false);
            var config = contextService.getConfiguration();
            this._isDev = !config.env.isBuilt || !!config.env.extensionDevelopmentPath;
            this._messageService = messageService;
            threadService.registerRemotableInstance(MainProcessExtensionService, this);
            this._threadService = threadService;
            this._telemetryService = telemetryService;
            this._proxy = this._threadService.getRemotable(ExtHostExtensionService);
            this._extensionsStatus = {};
            extensionsRegistry_1.ExtensionsRegistry.handleExtensionPoints(function (msg) { return _this._handleMessage(msg); });
        }
        MainProcessExtensionService.prototype._handleMessage = function (msg) {
            this._showMessage(msg.type, messageWithSource(msg));
            if (!this._extensionsStatus[msg.source]) {
                this._extensionsStatus[msg.source] = { messages: [] };
            }
            this._extensionsStatus[msg.source].messages.push(msg);
        };
        MainProcessExtensionService.prototype.$localShowMessage = function (severity, msg) {
            var messageShown = false;
            if (severity === severity_1.default.Error || severity === severity_1.default.Warning) {
                if (this._isDev) {
                    // Only show nasty intrusive messages if doing extension development.
                    this._messageService.show(severity, msg);
                    messageShown = true;
                }
            }
            if (!messageShown) {
                switch (severity) {
                    case severity_1.default.Error:
                        console.error(msg);
                        break;
                    case severity_1.default.Warning:
                        console.warn(msg);
                        break;
                    default:
                        console.log(msg);
                }
            }
        };
        // -- overwriting AbstractExtensionService
        MainProcessExtensionService.prototype.getExtensionsStatus = function () {
            return this._extensionsStatus;
        };
        MainProcessExtensionService.prototype._showMessage = function (severity, msg) {
            this._proxy.$localShowMessage(severity, msg);
            this.$localShowMessage(severity, msg);
        };
        MainProcessExtensionService.prototype._createFailedExtension = function () {
            return new MainProcessFailedExtension();
        };
        MainProcessExtensionService.prototype._actualActivateExtension = function (extensionDescription) {
            var _this = this;
            // redirect extension activation to the extension host
            return this._proxy.$activateExtension(extensionDescription).then(function (_) {
                // the extension host calls $onExtensionActivated, where we write to `_activatedExtensions`
                return _this._activatedExtensions[extensionDescription.id];
            });
        };
        // -- called by extension host
        MainProcessExtensionService.prototype.$onExtensionHostReady = function (extensionDescriptions, messages) {
            var _this = this;
            extensionsRegistry_1.ExtensionsRegistry.registerExtensions(extensionDescriptions);
            messages.forEach(function (entry) { return _this._handleMessage(entry); });
            this._triggerOnReady();
        };
        MainProcessExtensionService.prototype.$onExtensionActivated = function (extensionId) {
            this._activatedExtensions[extensionId] = new MainProcessSuccessExtension();
        };
        MainProcessExtensionService.prototype.$onExtensionActivationFailed = function (extensionId) {
            this._activatedExtensions[extensionId] = new MainProcessFailedExtension();
        };
        MainProcessExtensionService = __decorate([
            thread_1.Remotable.MainContext('MainProcessExtensionService')
        ], MainProcessExtensionService);
        return MainProcessExtensionService;
    }(abstractExtensionService_1.AbstractExtensionService));
    exports.MainProcessExtensionService = MainProcessExtensionService;
    var ExtHostExtension = (function (_super) {
        __extends(ExtHostExtension, _super);
        function ExtHostExtension(activationFailed, module, exports, subscriptions) {
            _super.call(this, activationFailed);
            this.module = module;
            this.exports = exports;
            this.subscriptions = subscriptions;
        }
        return ExtHostExtension;
    }(abstractExtensionService_1.ActivatedExtension));
    exports.ExtHostExtension = ExtHostExtension;
    var ExtHostEmptyExtension = (function (_super) {
        __extends(ExtHostEmptyExtension, _super);
        function ExtHostEmptyExtension() {
            _super.call(this, false, { activate: undefined, deactivate: undefined }, undefined, []);
        }
        return ExtHostEmptyExtension;
    }(ExtHostExtension));
    exports.ExtHostEmptyExtension = ExtHostEmptyExtension;
    var ExtensionMemento = (function () {
        function ExtensionMemento(id, global, storage) {
            var _this = this;
            this._id = id;
            this._shared = global;
            this._storage = storage;
            this._init = this._storage.getValue(this._shared, this._id, Object.create(null)).then(function (value) {
                _this._value = value;
                return _this;
            });
        }
        Object.defineProperty(ExtensionMemento.prototype, "whenReady", {
            get: function () {
                return this._init;
            },
            enumerable: true,
            configurable: true
        });
        ExtensionMemento.prototype.get = function (key, defaultValue) {
            var value = this._value[key];
            if (typeof value === 'undefined') {
                value = defaultValue;
            }
            return value;
        };
        ExtensionMemento.prototype.update = function (key, value) {
            this._value[key] = value;
            return this._storage
                .setValue(this._shared, this._id, this._value)
                .then(function () { return true; });
        };
        return ExtensionMemento;
    }());
    var ExtHostExtensionService = (function (_super) {
        __extends(ExtHostExtensionService, _super);
        /**
         * This class is constructed manually because it is a service, so it doesn't use any ctor injection
         */
        function ExtHostExtensionService(threadService, telemetryService) {
            _super.call(this, true);
            threadService.registerRemotableInstance(ExtHostExtensionService, this);
            this._threadService = threadService;
            this._storage = new remotable_storage_1.ExtHostStorage(threadService);
            this._proxy = this._threadService.getRemotable(MainProcessExtensionService);
            this._telemetryService = telemetryService;
        }
        ExtHostExtensionService.prototype.$localShowMessage = function (severity, msg) {
            switch (severity) {
                case severity_1.default.Error:
                    console.error(msg);
                    break;
                case severity_1.default.Warning:
                    console.warn(msg);
                    break;
                default:
                    console.log(msg);
            }
        };
        ExtHostExtensionService.prototype.get = function (extensionId) {
            if (!hasOwnProperty.call(this._activatedExtensions, extensionId)) {
                throw new Error('Extension `' + extensionId + '` is not known or not activated');
            }
            return this._activatedExtensions[extensionId].exports;
        };
        ExtHostExtensionService.prototype.deactivate = function (extensionId) {
            var extension = this._activatedExtensions[extensionId];
            if (!extension) {
                return;
            }
            // call deactivate if available
            try {
                if (typeof extension.module.deactivate === 'function') {
                    extension.module.deactivate();
                }
            }
            catch (err) {
            }
            // clean up subscriptions
            try {
                lifecycle_1.disposeAll(extension.subscriptions);
            }
            catch (err) {
            }
        };
        ExtHostExtensionService.prototype.registrationDone = function (messages) {
            this._triggerOnReady();
            this._proxy.$onExtensionHostReady(extensionsRegistry_1.ExtensionsRegistry.getAllExtensionDescriptions(), messages);
        };
        // -- overwriting AbstractExtensionService
        ExtHostExtensionService.prototype._showMessage = function (severity, msg) {
            this._proxy.$localShowMessage(severity, msg);
            this.$localShowMessage(severity, msg);
        };
        ExtHostExtensionService.prototype._createFailedExtension = function () {
            return new ExtHostExtension(true, { activate: undefined, deactivate: undefined }, undefined, []);
        };
        ExtHostExtensionService.prototype._loadExtensionContext = function (extensionDescription) {
            var globalState = new ExtensionMemento(extensionDescription.id, true, this._storage);
            var workspaceState = new ExtensionMemento(extensionDescription.id, false, this._storage);
            return winjs_base_1.TPromise.join([globalState.whenReady, workspaceState.whenReady]).then(function () {
                return Object.freeze({
                    globalState: globalState,
                    workspaceState: workspaceState,
                    subscriptions: [],
                    get extensionPath() { return extensionDescription.extensionFolderPath; },
                    asAbsolutePath: function (relativePath) { return paths.normalize(paths.join(extensionDescription.extensionFolderPath, relativePath), true); }
                });
            });
        };
        ExtHostExtensionService.prototype._actualActivateExtension = function (extensionDescription) {
            var _this = this;
            return this._doActualActivateExtension(extensionDescription).then(function (activatedExtension) {
                _this._proxy.$onExtensionActivated(extensionDescription.id);
                return activatedExtension;
            }, function (err) {
                _this._proxy.$onExtensionActivationFailed(extensionDescription.id);
                throw err;
            });
        };
        ExtHostExtensionService.prototype._doActualActivateExtension = function (extensionDescription) {
            var _this = this;
            var event = getTelemetryActivationEvent(extensionDescription);
            this._telemetryService.publicLog('activatePlugin', event);
            if (!extensionDescription.main) {
                // Treat the extension as being empty => NOT AN ERROR CASE
                return winjs_base_1.TPromise.as(new ExtHostEmptyExtension());
            }
            return loadCommonJSModule(extensionDescription.main).then(function (extensionModule) {
                return _this._loadExtensionContext(extensionDescription).then(function (context) {
                    return ExtHostExtensionService._callActivate(extensionModule, context);
                });
            });
        };
        ExtHostExtensionService._callActivate = function (extensionModule, context) {
            // Make sure the extension's surface is not undefined
            extensionModule = extensionModule || {
                activate: undefined,
                deactivate: undefined
            };
            return this._callActivateOptional(extensionModule, context).then(function (extensionExports) {
                return new ExtHostExtension(false, extensionModule, extensionExports, context.subscriptions);
            });
        };
        ExtHostExtensionService._callActivateOptional = function (extensionModule, context) {
            if (typeof extensionModule.activate === 'function') {
                try {
                    return winjs_base_1.TPromise.as(extensionModule.activate.apply(global, [context]));
                }
                catch (err) {
                    return winjs_base_1.TPromise.wrapError(err);
                }
            }
            else {
                // No activate found => the module is the extension's exports
                return winjs_base_1.TPromise.as(extensionModule);
            }
        };
        // -- called by main thread
        ExtHostExtensionService.prototype.$activateExtension = function (extensionDescription) {
            return this._activateExtension(extensionDescription);
        };
        ExtHostExtensionService = __decorate([
            thread_1.Remotable.ExtHostContext('ExtHostExtensionService')
        ], ExtHostExtensionService);
        return ExtHostExtensionService;
    }(abstractExtensionService_1.AbstractExtensionService));
    exports.ExtHostExtensionService = ExtHostExtensionService;
    function loadCommonJSModule(modulePath) {
        var r = null;
        try {
            r = require.__$__nodeRequire(modulePath);
        }
        catch (e) {
            return winjs_base_1.TPromise.wrapError(e);
        }
        return winjs_base_1.TPromise.as(r);
    }
    function getTelemetryActivationEvent(extensionDescription) {
        var event = {
            id: extensionDescription.id,
            name: extensionDescription.name,
            publisherDisplayName: extensionDescription.publisher,
            activationEvents: extensionDescription.activationEvents ? extensionDescription.activationEvents.join(',') : null
        };
        for (var contribution in extensionDescription.contributes) {
            var contributionDetails = extensionDescription.contributes[contribution];
            if (!contributionDetails) {
                continue;
            }
            switch (contribution) {
                case 'debuggers':
                    var types = contributionDetails.reduce(function (p, c) { return p ? p + ',' + c['type'] : c['type']; }, '');
                    event['contribution.debuggers'] = types;
                    break;
                case 'grammars':
                    var grammers = contributionDetails.reduce(function (p, c) { return p ? p + ',' + c['language'] : c['language']; }, '');
                    event['contribution.grammars'] = grammers;
                    break;
                case 'languages':
                    var languages = contributionDetails.reduce(function (p, c) { return p ? p + ',' + c['id'] : c['id']; }, '');
                    event['contribution.languages'] = languages;
                    break;
                case 'tmSnippets':
                    var tmSnippets = contributionDetails.reduce(function (p, c) { return p ? p + ',' + c['languageId'] : c['languageId']; }, '');
                    event['contribution.tmSnippets'] = tmSnippets;
                    break;
                default:
                    event[("contribution." + contribution)] = true;
            }
        }
        return event;
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/platform/telemetry/common/abstractRemoteTelemetryService", ["require", "exports", 'vs/platform/telemetry/common/abstractTelemetryService', 'vs/platform/telemetry/common/telemetry', 'vs/platform/thread/common/thread'], function (require, exports, AbstractTelemetryService, telemetry_1, thread_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Helper always instantiated in the main process to receive telemetry events from remote telemetry services
     */
    var RemoteTelemetryServiceHelper = (function () {
        function RemoteTelemetryServiceHelper(telemetryService) {
            this._telemetryService = telemetryService;
        }
        RemoteTelemetryServiceHelper.prototype._handleRemoteTelemetryEvent = function (eventName, data) {
            this._telemetryService.publicLog(eventName, data);
        };
        RemoteTelemetryServiceHelper.prototype.getTelemetryInfo = function () {
            return this._telemetryService.getTelemetryInfo();
        };
        RemoteTelemetryServiceHelper = __decorate([
            thread_1.Remotable.MainContext('RemoteTelemetryServiceHelper'),
            __param(0, telemetry_1.ITelemetryService)
        ], RemoteTelemetryServiceHelper);
        return RemoteTelemetryServiceHelper;
    }());
    exports.RemoteTelemetryServiceHelper = RemoteTelemetryServiceHelper;
    /**
     * Base class for remote telemetry services (instantiated in extension host or in web workers)
     */
    var AbstractRemoteTelemetryService = (function (_super) {
        __extends(AbstractRemoteTelemetryService, _super);
        function AbstractRemoteTelemetryService(threadService) {
            // Log all events including public, since they will be forwarded to the main which will do the real filtering
            _super.call(this);
            this._proxy = threadService.getRemotable(RemoteTelemetryServiceHelper);
        }
        AbstractRemoteTelemetryService.prototype.getTelemetryInfo = function () {
            return this._proxy.getTelemetryInfo();
        };
        AbstractRemoteTelemetryService.prototype.addTelemetryAppender = function (appender) {
            throw new Error('Telemetry appenders are not supported in this execution envirnoment');
        };
        AbstractRemoteTelemetryService.prototype.handleEvent = function (eventName, data) {
            this._proxy._handleRemoteTelemetryEvent(eventName, data);
        };
        return AbstractRemoteTelemetryService;
    }(AbstractTelemetryService.AbstractTelemetryService));
    exports.AbstractRemoteTelemetryService = AbstractRemoteTelemetryService;
});

define("vs/platform/thread/common/threadService", ["require", "exports", 'vs/base/common/winjs.base', './thread'], function (require, exports, winjs_base_1, thread) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.THREAD_SERVICE_PROPERTY_NAME = '__$$__threadService';
    function findMember(proto, target) {
        for (var i in proto) {
            if (proto[i] === target) {
                return i;
            }
        }
        throw new Error('Member not found in prototype');
    }
    function findThreadService(obj) {
        var threadService = obj[exports.THREAD_SERVICE_PROPERTY_NAME];
        if (!threadService) {
            throw new Error('Objects that use thread attributes must be instantiated with the thread service');
        }
        return threadService;
    }
    function OneWorkerFn(type, target, conditionOrAffinity, affinity) {
        if (affinity === void 0) { affinity = thread.ThreadAffinity.None; }
        var methodName = findMember(type.prototype, target), condition;
        if (typeof conditionOrAffinity === 'function') {
            condition = conditionOrAffinity;
        }
        else if (typeof conditionOrAffinity !== 'undefined') {
            affinity = conditionOrAffinity;
        }
        type.prototype[methodName] = function () {
            var param = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                param[_i - 0] = arguments[_i];
            }
            if (!condition) {
                return findThreadService(this).OneWorker(this, methodName, target, param, affinity);
            }
            else {
                var that_1 = this, promise = condition.call(that_1);
                if (!winjs_base_1.TPromise.is(promise)) {
                    promise = winjs_base_1.TPromise.as(promise);
                }
                return promise.then(function () {
                    return findThreadService(that_1).OneWorker(that_1, methodName, target, param, affinity);
                });
            }
        };
    }
    exports.OneWorkerAttr = OneWorkerFn;
    function AllWorkersAttr(type, target) {
        var methodName = findMember(type.prototype, target);
        type.prototype[methodName] = function () {
            var param = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                param[_i - 0] = arguments[_i];
            }
            return findThreadService(this).AllWorkers(this, methodName, target, param);
        };
    }
    exports.AllWorkersAttr = AllWorkersAttr;
});

define("vs/platform/thread/common/abstractThreadService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/platform/thread/common/thread', 'vs/platform/thread/common/threadService', 'vs/platform/instantiation/common/descriptors'], function (require, exports, winjs_base_1, thread_1, threadService_1, descriptors_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var AbstractThreadService = (function () {
        function AbstractThreadService(isInMainThread) {
            this.isInMainThread = isInMainThread;
            this._boundObjects = {};
            this._pendingObjects = [];
            this._localObjMap = Object.create(null);
            this._proxyObjMap = Object.create(null);
        }
        AbstractThreadService.prototype.setInstantiationService = function (service) {
            this._instantiationService = service;
        };
        AbstractThreadService.prototype.createInstance = function () {
            var params = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                params[_i - 0] = arguments[_i];
            }
            return this._doCreateInstance(params);
        };
        AbstractThreadService.prototype._doCreateInstance = function (params) {
            var _this = this;
            var instanceOrPromise = this._instantiationService.createInstance.apply(this._instantiationService, params);
            if (winjs_base_1.TPromise.is(instanceOrPromise)) {
                var objInstantiated_1;
                objInstantiated_1 = instanceOrPromise.then(function (instance) {
                    if (instance.asyncCtor) {
                        var initPromise = instance.asyncCtor();
                        if (winjs_base_1.TPromise.is(initPromise)) {
                            return initPromise.then(function () {
                                return instance;
                            });
                        }
                    }
                    return instance;
                });
                this._pendingObjects.push(objInstantiated_1);
                return objInstantiated_1.then(function (instance) {
                    var r = _this._finishInstance(instance);
                    for (var i = 0; i < _this._pendingObjects.length; i++) {
                        if (_this._pendingObjects[i] === objInstantiated_1) {
                            _this._pendingObjects.splice(i, 1);
                            break;
                        }
                    }
                    return r;
                });
            }
            return this._finishInstance(instanceOrPromise);
        };
        AbstractThreadService.prototype._finishInstance = function (instance) {
            instance[threadService_1.THREAD_SERVICE_PROPERTY_NAME] = this;
            this._boundObjects[instance.getId()] = instance;
            if (instance.creationDone) {
                instance.creationDone();
            }
            return instance;
        };
        AbstractThreadService.prototype.handle = function (rpcId, methodName, args) {
            if (!this._localObjMap[rpcId]) {
                throw new Error('Unknown actor ' + rpcId);
            }
            var actor = this._localObjMap[rpcId];
            var method = actor[methodName];
            if (typeof method !== 'function') {
                throw new Error('Unknown method ' + methodName + ' on actor ' + rpcId);
            }
            return method.apply(actor, args);
        };
        AbstractThreadService.prototype._getOrCreateProxyInstance = function (remoteCom, id, descriptor) {
            if (this._proxyObjMap[id]) {
                return this._proxyObjMap[id];
            }
            var result = createProxyFromCtor(remoteCom, id, descriptor.ctor);
            this._proxyObjMap[id] = result;
            return result;
        };
        AbstractThreadService.prototype._registerLocalInstance = function (id, obj) {
            this._localObjMap[id] = obj;
        };
        AbstractThreadService.prototype._getOrCreateLocalInstance = function (id, descriptor) {
            if (this._localObjMap[id]) {
                return this._localObjMap[id];
            }
            var result = this._instantiationService.createInstance(descriptor);
            this._registerLocalInstance(id, result);
            return result;
        };
        AbstractThreadService.prototype.getRemotable = function (ctor) {
            var id = thread_1.Remotable.getId(ctor);
            if (!id) {
                throw new Error('Unknown Remotable: <<' + id + '>>');
            }
            var desc = descriptors_1.createSyncDescriptor(ctor);
            if (thread_1.Remotable.Registry.MainContext[id]) {
                return this._registerAndInstantiateMainProcessActor(id, desc);
            }
            if (thread_1.Remotable.Registry.ExtHostContext[id]) {
                return this._registerAndInstantiateExtHostActor(id, desc);
            }
            if (thread_1.Remotable.Registry.WorkerContext[id]) {
                return this._registerAndInstantiateWorkerActor(id, desc, thread_1.Remotable.Registry.WorkerContext[id].affinity);
            }
            throw new Error('Unknown Remotable: <<' + id + '>>');
        };
        AbstractThreadService.prototype.registerRemotableInstance = function (ctor, instance) {
            var id = thread_1.Remotable.getId(ctor);
            if (!id) {
                throw new Error('Unknown Remotable: <<' + id + '>>');
            }
            if (thread_1.Remotable.Registry.MainContext[id]) {
                return this._registerMainProcessActor(id, instance);
            }
            if (thread_1.Remotable.Registry.ExtHostContext[id]) {
                return this._registerExtHostActor(id, instance);
            }
            if (thread_1.Remotable.Registry.WorkerContext[id]) {
                return this._registerWorkerActor(id, instance);
            }
            throw new Error('Unknown Remotable: <<' + id + '>>');
        };
        return AbstractThreadService;
    }());
    exports.AbstractThreadService = AbstractThreadService;
    function createProxyFromCtor(remote, id, ctor) {
        var result = {
            $__IS_REMOTE_OBJ: true
        };
        for (var prop in ctor.prototype) {
            if (typeof ctor.prototype[prop] === 'function') {
                result[prop] = createMethodProxy(remote, id, prop);
            }
        }
        return result;
    }
    function createMethodProxy(remote, proxyId, path) {
        return function () {
            var myArgs = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                myArgs[_i - 0] = arguments[_i];
            }
            return remote.callOnRemote(proxyId, path, myArgs);
        };
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/platform/thread/common/mainThreadService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/worker/workerClient', 'vs/platform/thread/common/abstractThreadService', 'vs/base/common/flags', 'vs/base/common/platform', 'vs/base/common/errors', 'vs/base/common/timer', 'vs/platform/thread/common/thread', 'vs/base/worker/defaultWorkerFactory'], function (require, exports, winjs_base_1, Worker, abstractThreadService, Env, Platform, errors, Timer, thread_1, defaultWorkerFactory_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var MainThreadService = (function (_super) {
        __extends(MainThreadService, _super);
        function MainThreadService(contextService, workerModuleId, defaultWorkerCount) {
            var _this = this;
            _super.call(this, true);
            this.serviceId = thread_1.IThreadService;
            this._contextService = contextService;
            this._workerModuleId = workerModuleId;
            this._defaultWorkerCount = defaultWorkerCount;
            this._workerFactory = new defaultWorkerFactory_1.DefaultWorkerFactory();
            if (!this.isInMainThread) {
                throw new Error('Incorrect Service usage: this service must be used only in the main thread');
            }
            this._workerPool = [];
            this._affinityScrambler = {};
            this._listeners = [];
            this._workersCreatedPromise = new winjs_base_1.TPromise(function (c, e, p) {
                _this._triggerWorkersCreatedPromise = c;
            }, function () {
                // Not cancelable
            });
            // If nobody asks for workers to be created in 5s, the workers are created automatically
            winjs_base_1.TPromise.timeout(MainThreadService.MAXIMUM_WORKER_CREATION_DELAY).then(function () { return _this.ensureWorkers(); });
        }
        MainThreadService.prototype.ensureWorkers = function () {
            if (this._triggerWorkersCreatedPromise) {
                // Workers not created yet
                var createCount = Env.workersCount(this._defaultWorkerCount);
                if (!Platform.hasWebWorkerSupport()) {
                    // Create at most 1 compatibility worker
                    createCount = Math.min(createCount, 1);
                }
                for (var i = 0; i < createCount; i++) {
                    this._createWorker();
                }
                var complete = this._triggerWorkersCreatedPromise;
                this._triggerWorkersCreatedPromise = null;
                complete(null);
            }
        };
        MainThreadService.prototype.addStatusListener = function (listener) {
            for (var i = 0; i < this._listeners.length; i++) {
                if (this._listeners[i] === listener) {
                    // listener is already in
                    return;
                }
            }
            this._listeners.push(listener);
        };
        MainThreadService.prototype.removeStatusListener = function (listener) {
            for (var i = 0; i < this._listeners.length; i++) {
                if (this._listeners[i] === listener) {
                    this._listeners.splice(i, 1);
                    return;
                }
            }
        };
        MainThreadService.prototype._afterWorkers = function () {
            var _this = this;
            var shouldCancelPromise = false;
            return new winjs_base_1.TPromise(function (c, e, p) {
                // hide the initialize promise inside this
                // promise so that it won't be canceled by accident
                _this._workersCreatedPromise.then(function () {
                    if (!shouldCancelPromise) {
                        c(null);
                    }
                }, e, p);
            }, function () {
                // mark that this promise is canceled
                shouldCancelPromise = true;
            });
        };
        MainThreadService.prototype._createWorker = function () {
            this._workerPool.push(this._doCreateWorker());
        };
        MainThreadService.prototype._shortName = function (major, minor) {
            return major.substring(major.length - 14) + '.' + minor.substr(0, 14);
        };
        MainThreadService.prototype._doCreateWorker = function () {
            var _this = this;
            var worker = new Worker.WorkerClient(this._workerFactory, this._workerModuleId, function (msg) {
                if (msg.type === 'threadService') {
                    return _this._shortName(msg.payload[0], msg.payload[1]);
                }
                return msg.type;
            });
            worker.getRemoteCom().setManyHandler(this);
            worker.onModuleLoaded = worker.request('initialize', {
                contextService: {
                    workspace: this._contextService.getWorkspace(),
                    configuration: this._contextService.getConfiguration(),
                    options: this._contextService.getOptions()
                }
            });
            return worker;
        };
        MainThreadService.prototype._getWorkerIndex = function (obj, affinity) {
            if (affinity === thread_1.ThreadAffinity.None) {
                var winners = [0], winnersQueueSize = this._workerPool[0].getQueueSize();
                for (var i = 1; i < this._workerPool.length; i++) {
                    var queueSize = this._workerPool[i].getQueueSize();
                    if (queueSize < winnersQueueSize) {
                        winnersQueueSize = queueSize;
                        winners = [i];
                    }
                    else if (queueSize === winnersQueueSize) {
                        winners.push(i);
                    }
                }
                return winners[Math.floor(Math.random() * winners.length)];
            }
            var scramble = 0;
            if (this._affinityScrambler.hasOwnProperty(obj.getId())) {
                scramble = this._affinityScrambler[obj.getId()];
            }
            else {
                scramble = Math.floor(Math.random() * this._workerPool.length);
                this._affinityScrambler[obj.getId()] = scramble;
            }
            return (scramble + affinity) % this._workerPool.length;
        };
        MainThreadService.prototype.OneWorker = function (obj, methodName, target, params, affinity) {
            var _this = this;
            return this._afterWorkers().then(function () {
                if (_this._workerPool.length === 0) {
                    throw new Error('Cannot fulfill request...');
                }
                var workerIdx = _this._getWorkerIndex(obj, affinity);
                return _this._remoteCall(_this._workerPool[workerIdx], obj, methodName, params);
            });
        };
        MainThreadService.prototype.AllWorkers = function (obj, methodName, target, params) {
            var _this = this;
            return this._afterWorkers().then(function () {
                return winjs_base_1.TPromise.join(_this._workerPool.map(function (w) {
                    return _this._remoteCall(w, obj, methodName, params);
                }));
            });
        };
        MainThreadService.prototype._remoteCall = function (worker, obj, methodName, params) {
            var _this = this;
            var id = obj.getId();
            if (!id) {
                throw new Error('Synchronizable Objects must have an identifier');
            }
            var timerEvent = Timer.start(Timer.Topic.LANGUAGES, this._shortName(id, methodName));
            var stopTimer = function () {
                timerEvent.stop();
                //			console.log(timerEvent.timeTaken(), this._workerPool.indexOf(worker), obj.getId() + ' >>> ' + methodName + ': ', params);
                _this._pingListenersIfNecessary();
            };
            var r = winjs_base_1.decoratePromise(worker.request('threadService', [id, methodName, params]), stopTimer, stopTimer);
            this._pingListenersIfNecessary();
            return r;
        };
        MainThreadService.prototype._pingListenersIfNecessary = function () {
            if (this._listeners.length > 0) {
                var status_1 = this._buildStatus();
                var listeners = this._listeners.slice(0);
                try {
                    for (var i = 0; i < listeners.length; i++) {
                        listeners[i].onThreadServiceStatus(status_1);
                    }
                }
                catch (e) {
                    errors.onUnexpectedError(e);
                }
            }
        };
        MainThreadService.prototype._buildStatus = function () {
            var queueSizes = this._workerPool.map(function (worker) {
                return {
                    queueSize: worker.getQueueSize()
                };
            });
            return {
                workers: queueSizes
            };
        };
        MainThreadService.prototype._registerAndInstantiateMainProcessActor = function (id, descriptor) {
            return this._getOrCreateLocalInstance(id, descriptor);
        };
        MainThreadService.prototype._registerMainProcessActor = function (id, actor) {
            this._registerLocalInstance(id, actor);
        };
        MainThreadService.prototype._registerAndInstantiateExtHostActor = function (id, descriptor) {
            throw new Error('Not supported in this runtime context: Cannot communicate to non-existant Extension Host!');
        };
        MainThreadService.prototype._registerExtHostActor = function (id, actor) {
            throw new Error('Not supported in this runtime context!');
        };
        MainThreadService.prototype._registerAndInstantiateWorkerActor = function (id, descriptor, whichWorker) {
            var helper = this._createWorkerProxyHelper(whichWorker);
            return this._getOrCreateProxyInstance(helper, id, descriptor);
        };
        MainThreadService.prototype._registerWorkerActor = function (id, actor) {
            throw new Error('Not supported in this runtime context!');
        };
        MainThreadService.prototype._createWorkerProxyHelper = function (whichWorker) {
            var _this = this;
            return {
                callOnRemote: function (proxyId, path, args) {
                    return _this._callOnWorker(whichWorker, proxyId, path, args);
                }
            };
        };
        MainThreadService.prototype._callOnWorker = function (whichWorker, proxyId, path, args) {
            var _this = this;
            if (whichWorker === thread_1.ThreadAffinity.None) {
                return winjs_base_1.TPromise.as(null);
            }
            return this._afterWorkers().then(function () {
                if (whichWorker === thread_1.ThreadAffinity.All) {
                    var promises = _this._workerPool.map(function (w) { return w.getRemoteCom(); }).map(function (rCom) { return rCom.callOnRemote(proxyId, path, args); });
                    return winjs_base_1.TPromise.join(promises);
                }
                var workerIdx = whichWorker % _this._workerPool.length;
                var worker = _this._workerPool[workerIdx];
                return worker.getRemoteCom().callOnRemote(proxyId, path, args);
            });
        };
        MainThreadService.MAXIMUM_WORKER_CREATION_DELAY = 500; // 500ms
        return MainThreadService;
    }(abstractThreadService.AbstractThreadService));
    exports.MainThreadService = MainThreadService;
});

define("vs/platform/workspace/common/workspace", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IWorkspaceContextService = instantiation_1.createDecorator('contextService');
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/common/languages.common", ["require", "exports", 'vs/base/common/assert', 'vs/base/common/async', 'vs/base/common/callbackList', 'vs/base/common/cancellation', 'vs/base/common/collections', 'vs/base/common/event', 'vs/base/common/events', 'vs/base/common/lifecycle', 'vs/base/common/paths', 'vs/base/common/uri', 'vs/platform/platform', 'vs/platform/jsonschemas/common/jsonContributionRegistry', 'vs/platform/files/common/files', 'vs/platform/request/common/request', 'vs/platform/workspace/common/workspace', 'vs/platform/telemetry/common/telemetry', 'vs/platform/thread/common/thread', 'vs/platform/thread/common/threadService', 'vs/editor/common/editorCommon', 'vs/editor/common/modes', 'vs/editor/common/modes/abstractMode', 'vs/editor/common/modes/abstractState', 'vs/editor/common/modes/monarch/monarch', 'vs/editor/common/modes/monarch/monarchCompile', 'vs/editor/common/modes/supports/declarationSupport', 'vs/editor/common/modes/supports/parameterHintsSupport', 'vs/editor/common/modes/supports/referenceSupport', 'vs/editor/common/modes/supports/richEditSupport', 'vs/editor/common/modes/supports/suggestSupport', 'vs/editor/common/modes/supports/tokenizationSupport', 'vs/editor/common/services/modelService', 'vs/editor/common/services/modeService'], function (require, exports) {
    'use strict';
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/referenceSearch/browser/referenceSearchWidget", ["require", "exports", 'vs/nls!vs/editor/contrib/referenceSearch/browser/referenceSearchWidget', 'vs/base/common/collections', 'vs/base/common/errors', 'vs/base/common/labels', 'vs/base/common/lifecycle', 'vs/base/common/network', 'vs/base/common/strings', 'vs/base/common/winjs.base', 'vs/base/browser/builder', 'vs/base/browser/dom', 'vs/base/browser/ui/countBadge/countBadge', 'vs/base/browser/ui/filelabel/fileLabel', 'vs/base/browser/ui/leftRightWidget/leftRightWidget', 'vs/base/parts/tree/browser/treeDefaults', 'vs/base/parts/tree/browser/treeImpl', 'vs/platform/workspace/common/workspace', 'vs/editor/common/config/defaultConfig', 'vs/editor/common/core/range', 'vs/editor/common/editorCommon', 'vs/editor/common/model/model', 'vs/editor/browser/widget/embeddedCodeEditorWidget', 'vs/editor/contrib/zoneWidget/browser/peekViewWidget', './referenceSearchModel', 'vs/css!./referenceSearchWidget'], function (require, exports, nls, collections, errors_1, labels_1, lifecycle_1, network_1, strings, winjs_base_1, builder_1, dom, countBadge_1, fileLabel_1, leftRightWidget_1, treeDefaults_1, treeImpl_1, workspace_1, defaultConfig_1, range_1, editorCommon, model_1, embeddedCodeEditorWidget_1, peekViewWidget_1, referenceSearchModel_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DecorationsManager = (function () {
        function DecorationsManager(editor, model) {
            var _this = this;
            this.editor = editor;
            this.model = model;
            this._decorationSet = collections.createStringDictionary();
            this._decorationIgnoreSet = collections.createStringDictionary();
            this.callOnDispose = [];
            this.callOnModelChange = [];
            this.callOnDispose.push(this.editor.addListener(editorCommon.EventType.ModelChanged, function () { return _this.onModelChanged(); }));
            this.onModelChanged();
        }
        DecorationsManager.prototype.dispose = function () {
            this.callOnModelChange = lifecycle_1.cAll(this.callOnModelChange);
            this.callOnDispose = lifecycle_1.cAll(this.callOnDispose);
            this.removeDecorations();
        };
        DecorationsManager.prototype.onModelChanged = function () {
            this.removeDecorations();
            this.callOnModelChange = lifecycle_1.cAll(this.callOnModelChange);
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            for (var i = 0, len = this.model.children.length; i < len; i++) {
                if (this.model.children[i].resource.toString() === model.getAssociatedResource().toString()) {
                    this.addDecorations(this.model.children[i]);
                    return;
                }
            }
        };
        DecorationsManager.prototype.addDecorations = function (reference) {
            var _this = this;
            this.callOnModelChange.push(this.editor.getModel().addListener(editorCommon.EventType.ModelDecorationsChanged, function (event) { return _this.onDecorationChanged(event); }));
            this.editor.getModel().changeDecorations(function (accessor) {
                var newDecorations = [];
                var newDecorationsActualIndex = [];
                for (var i_1 = 0, len = reference.children.length; i_1 < len; i_1++) {
                    var oneReference = reference.children[i_1];
                    if (_this._decorationIgnoreSet[oneReference.id]) {
                        continue;
                    }
                    newDecorations.push({
                        range: oneReference.range,
                        options: DecorationsManager.DecorationOptions
                    });
                    newDecorationsActualIndex.push(i_1);
                }
                var decorations = accessor.deltaDecorations([], newDecorations);
                for (var i = 0; i < decorations.length; i++) {
                    _this._decorationSet[decorations[i]] = reference.children[newDecorationsActualIndex[i]];
                }
            });
        };
        DecorationsManager.prototype.onDecorationChanged = function (event) {
            var _this = this;
            var addedOrChangedDecorations = event.addedOrChangedDecorations, toRemove = [];
            for (var i = 0, len = addedOrChangedDecorations.length; i < len; i++) {
                var reference = collections.lookup(this._decorationSet, addedOrChangedDecorations[i].id);
                if (!reference) {
                    continue;
                }
                var newRange = addedOrChangedDecorations[i].range, ignore = false;
                if (range_1.Range.equalsRange(newRange, reference.range)) {
                    continue;
                }
                else if (range_1.Range.spansMultipleLines(newRange)) {
                    ignore = true;
                }
                else {
                    var lineLength = reference.range.endColumn - reference.range.startColumn, newLineLength = newRange.endColumn - newRange.startColumn;
                    if (lineLength !== newLineLength) {
                        ignore = true;
                    }
                }
                if (ignore) {
                    this._decorationIgnoreSet[reference.id] = reference;
                    toRemove.push(addedOrChangedDecorations[i].id);
                }
                else {
                    reference.range = newRange;
                }
            }
            this.editor.changeDecorations(function (accessor) {
                for (var i_2 = 0, len_1 = toRemove.length; i_2 < len_1; i_2++) {
                    delete _this._decorationSet[toRemove[i_2]];
                }
                accessor.deltaDecorations(toRemove, []);
            });
        };
        DecorationsManager.prototype.removeDecorations = function () {
            var keys = Object.keys(this._decorationSet);
            if (keys.length > 0) {
                this.editor.changeDecorations(function (accessor) {
                    accessor.deltaDecorations(keys, []);
                });
            }
            this._decorationSet = {};
        };
        DecorationsManager.DecorationOptions = {
            stickiness: editorCommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges,
            className: 'reference-decoration'
        };
        return DecorationsManager;
    }());
    var DataSource = (function () {
        function DataSource() {
        }
        DataSource.prototype.getId = function (tree, element) {
            if (element instanceof referenceSearchModel_1.ReferencesModel) {
                return 'root';
            }
            else if (element instanceof referenceSearchModel_1.FileReferences) {
                return element.id;
            }
            else if (element instanceof referenceSearchModel_1.OneReference) {
                return element.id;
            }
        };
        DataSource.prototype.hasChildren = function (tree, element) {
            return element instanceof referenceSearchModel_1.FileReferences || element instanceof referenceSearchModel_1.ReferencesModel;
        };
        DataSource.prototype.getChildren = function (tree, element) {
            if (element instanceof referenceSearchModel_1.ReferencesModel) {
                return winjs_base_1.TPromise.as(element.children);
            }
            else if (element instanceof referenceSearchModel_1.FileReferences) {
                return element.resolve().then(function (val) { return val.children; });
            }
            else {
                return winjs_base_1.TPromise.as([]);
            }
        };
        DataSource.prototype.getParent = function (tree, element) {
            var result = null;
            if (element instanceof referenceSearchModel_1.FileReferences) {
                result = element.parent;
            }
            else if (element instanceof referenceSearchModel_1.OneReference) {
                result = element.parent;
            }
            return winjs_base_1.TPromise.as(result);
        };
        return DataSource;
    }());
    var Controller = (function (_super) {
        __extends(Controller, _super);
        function Controller() {
            _super.apply(this, arguments);
        }
        Controller.prototype.onMouseDown = function (tree, element, event) {
            if (event.leftButton) {
                if (element instanceof referenceSearchModel_1.FileReferences) {
                    event.preventDefault();
                    event.stopPropagation();
                    return this.expandCollapse(tree, element);
                }
                var result = _super.prototype.onClick.call(this, tree, element, event);
                if (event.ctrlKey || event.metaKey) {
                    tree.emit(Controller.Events.OPEN_TO_SIDE, element);
                }
                else if (event.detail === 2) {
                    tree.emit(Controller.Events.SELECTED, element);
                }
                else {
                    tree.emit(Controller.Events.FOCUSED, element);
                }
                return result;
            }
            return false;
        };
        Controller.prototype.onClick = function (tree, element, event) {
            if (event.leftButton) {
                return false; // Already handled by onMouseDown
            }
            return _super.prototype.onClick.call(this, tree, element, event);
        };
        Controller.prototype.expandCollapse = function (tree, element) {
            if (tree.isExpanded(element)) {
                tree.collapse(element).done(null, errors_1.onUnexpectedError);
            }
            else {
                tree.expand(element).done(null, errors_1.onUnexpectedError);
            }
            return true;
        };
        Controller.prototype.onEscape = function (tree, event) {
            return false;
        };
        Controller.prototype.onEnter = function (tree, event) {
            var element = tree.getFocus();
            if (element instanceof referenceSearchModel_1.FileReferences) {
                return this.expandCollapse(tree, element);
            }
            var result = _super.prototype.onEnter.call(this, tree, event);
            if (event.ctrlKey || event.metaKey) {
                tree.emit(Controller.Events.OPEN_TO_SIDE, element);
            }
            else {
                tree.emit(Controller.Events.SELECTED, element);
            }
            return result;
        };
        Controller.prototype.onUp = function (tree, event) {
            _super.prototype.onUp.call(this, tree, event);
            this.fakeFocus(tree, event);
            return true;
        };
        Controller.prototype.onPageUp = function (tree, event) {
            _super.prototype.onPageUp.call(this, tree, event);
            this.fakeFocus(tree, event);
            return true;
        };
        Controller.prototype.onLeft = function (tree, event) {
            _super.prototype.onLeft.call(this, tree, event);
            this.fakeFocus(tree, event);
            return true;
        };
        Controller.prototype.onDown = function (tree, event) {
            _super.prototype.onDown.call(this, tree, event);
            this.fakeFocus(tree, event);
            return true;
        };
        Controller.prototype.onPageDown = function (tree, event) {
            _super.prototype.onPageDown.call(this, tree, event);
            this.fakeFocus(tree, event);
            return true;
        };
        Controller.prototype.onRight = function (tree, event) {
            _super.prototype.onRight.call(this, tree, event);
            this.fakeFocus(tree, event);
            return true;
        };
        Controller.prototype.fakeFocus = function (tree, event) {
            // focus next item
            var focus = tree.getFocus();
            tree.setSelection([focus]);
            // send out event
            tree.emit(Controller.Events.FOCUSED, focus);
        };
        Controller.Events = {
            FOCUSED: 'events/custom/focused',
            SELECTED: 'events/custom/selected',
            OPEN_TO_SIDE: 'events/custom/opentoside'
        };
        return Controller;
    }(treeDefaults_1.DefaultController));
    var Renderer = (function (_super) {
        __extends(Renderer, _super);
        function Renderer(editor, contextService) {
            _super.call(this);
            this.editor = editor;
            this._contextService = contextService;
        }
        Renderer.prototype.getHeight = function (tree, element) {
            return 1.2 * this.editor.getConfiguration().lineHeight;
        };
        Renderer.prototype.render = function (tree, element, container) {
            var _this = this;
            dom.clearNode(container);
            if (element instanceof referenceSearchModel_1.FileReferences) {
                var fileReferences = element, fileReferencesContainer = builder_1.$('.reference-file');
                /* tslint:disable:no-unused-expression */
                new leftRightWidget_1.LeftRightWidget(fileReferencesContainer, function (left) {
                    var resource = fileReferences.resource;
                    new fileLabel_1.FileLabel(left, resource, _this._contextService);
                    return null;
                }, function (right) {
                    var len = fileReferences.children.length;
                    return new countBadge_1.CountBadge(right, len, len > 1 ? nls.localize(0, null, len) : nls.localize(1, null, len));
                });
                /* tslint:enable:no-unused-expression */
                fileReferencesContainer.appendTo(container);
            }
            else if (element instanceof referenceSearchModel_1.OneReference) {
                var oneReference = element, oneReferenceContainer = builder_1.$('.reference'), preview = oneReference.parent.preview.preview(oneReference.range);
                oneReferenceContainer.innerHtml(strings.format('<span>{0}</span><span class="referenceMatch">{1}</span><span>{2}</span>', strings.escape(preview.before), strings.escape(preview.inside), strings.escape(preview.after))).appendTo(container);
            }
            return null;
        };
        Renderer = __decorate([
            __param(1, workspace_1.IWorkspaceContextService)
        ], Renderer);
        return Renderer;
    }(treeDefaults_1.LegacyRenderer));
    /**
     * ZoneWidget that is shown inside the editor
     */
    var ReferenceWidget = (function (_super) {
        __extends(ReferenceWidget, _super);
        function ReferenceWidget(editorService, keybindingService, contextService, instantiationService, editor) {
            _super.call(this, editor, keybindingService, ReferenceWidget.INNER_EDITOR_CONTEXT_KEY, { frameColor: '#007ACC', showFrame: false, showArrow: true });
            this.editorService = editorService;
            this.contextService = contextService;
            this.instantiationService = instantiationService.createChild({ peekViewService: this });
            this.callOnModel = [];
            this.tree = null;
            this.treeContainer = null;
            this.preview = null;
            this.previewContainer = null;
            this.previewDecorations = [];
            this.lastHeight = null;
            this.create();
        }
        ReferenceWidget.prototype._onTitleClick = function (e) {
            if (!this.preview || !this.preview.getModel()) {
                return;
            }
            var model = this.preview.getModel(), lineNumber = this.preview.getPosition().lineNumber, titleRange = new range_1.Range(lineNumber, 1, lineNumber, model.getLineMaxColumn(lineNumber));
            this.emit(ReferenceWidget.Events.EditorDoubleClick, { reference: this.getFocusedReference(), range: titleRange, originalEvent: e });
        };
        ReferenceWidget.prototype._fillBody = function (containerElement) {
            var _this = this;
            var container = builder_1.$(containerElement);
            container.addClass('reference-zone-widget');
            // message pane
            container.div({ 'class': 'messages' }, function (div) {
                _this.messageContainer = div.hide();
            });
            // editor
            container.div({ 'class': 'preview inline' }, function (div) {
                var options = {
                    scrollBeyondLastLine: false,
                    scrollbar: defaultConfig_1.DefaultConfig.editor.scrollbar,
                    overviewRulerLanes: 2
                };
                _this.preview = _this.instantiationService.createInstance(embeddedCodeEditorWidget_1.EmbeddedCodeEditorWidget, div.getHTMLElement(), options, _this.editor);
                _this.previewContainer = div.hide();
                _this.previewNotAvailableMessage = new model_1.Model(nls.localize(2, null), model_1.Model.DEFAULT_CREATION_OPTIONS, null);
            });
            // tree
            container.div({ 'class': 'ref-tree inline' }, function (div) {
                var config = {
                    dataSource: _this.instantiationService.createInstance(DataSource),
                    renderer: _this.instantiationService.createInstance(Renderer, _this.editor),
                    //sorter: new Sorter(),
                    controller: new Controller()
                };
                var options = {
                    allowHorizontalScroll: false,
                    twistiePixels: 20,
                    ariaLabel: nls.localize(3, null)
                };
                _this.tree = new treeImpl_1.Tree(div.getHTMLElement(), config, options);
                _this.treeContainer = div.hide();
            });
        };
        ReferenceWidget.prototype._doLayoutBody = function (heightInPixel) {
            _super.prototype._doLayoutBody.call(this, heightInPixel);
            var h = heightInPixel + 'px';
            if (h === this.lastHeight) {
                return;
            }
            // set height
            this.treeContainer.style({ height: h });
            this.previewContainer.style({ height: h });
            // forward
            this.tree.layout(heightInPixel);
            this.preview.layout();
            this.lastHeight = h;
        };
        ReferenceWidget.prototype.onWidth = function (widthInPixel) {
            this.preview.layout();
        };
        ReferenceWidget.prototype.setModel = function (newModel) {
            // clean up
            this.callOnModel = lifecycle_1.disposeAll(this.callOnModel);
            this.model = newModel;
            if (this.model) {
                this._onNewModel();
            }
        };
        ReferenceWidget.prototype.showMessage = function (message) {
            this.setTitle('');
            this.messageContainer.innerHtml(message).show();
        };
        ReferenceWidget.prototype._onNewModel = function () {
            var _this = this;
            this.messageContainer.hide();
            this.decorationsManager = new DecorationsManager(this.preview, this.model);
            this.callOnModel.push(this.decorationsManager);
            // listen on model changes
            this.callOnModel.push(this.model.addListener2(referenceSearchModel_1.EventType.OnReferenceRangeChanged, function (reference) {
                _this.tree.refresh(reference);
            }));
            // listen on selection and focus
            this.callOnModel.push(this.tree.addListener2(Controller.Events.FOCUSED, function (element) {
                if (element instanceof referenceSearchModel_1.OneReference) {
                    _this.showReferencePreview(element);
                }
            }));
            this.callOnModel.push(this.tree.addListener2(Controller.Events.SELECTED, function (element) {
                if (element instanceof referenceSearchModel_1.OneReference) {
                    _this.showReferencePreview(element);
                    _this.model.currentReference = element;
                }
            }));
            this.callOnModel.push(this.tree.addListener2(Controller.Events.OPEN_TO_SIDE, function (element) {
                if (element instanceof referenceSearchModel_1.OneReference) {
                    _this.editorService.openEditor({
                        resource: element.resource,
                        options: {
                            selection: element.range
                        }
                    }, true);
                }
            }));
            var input = this.model.children.length === 1 ? this.model.children[0] : this.model;
            this.tree.setInput(input).then(function () {
                _this.tree.setSelection([_this.model.currentReference]);
            }).done(null, errors_1.onUnexpectedError);
            // listen on editor
            this.callOnModel.push(this.preview.addListener2(editorCommon.EventType.MouseDown, function (e) {
                if (e.event.detail === 2) {
                    _this.emit(ReferenceWidget.Events.EditorDoubleClick, { reference: _this.getFocusedReference(), range: e.target.range, originalEvent: e.event });
                }
            }));
            // make sure things are rendered
            dom.addClass(this.container, 'results-loaded');
            this.treeContainer.show();
            this.previewContainer.show();
            this.preview.layout();
            this.tree.layout();
            this.focus();
            // preview the current reference
            this.showReferencePreview(this.model.nextReference(this.model.currentReference));
        };
        ReferenceWidget.prototype.getFocusedReference = function () {
            var element = this.tree.getFocus();
            if (element instanceof referenceSearchModel_1.OneReference) {
                return element.resource;
            }
            else if (element instanceof referenceSearchModel_1.FileReferences) {
                var referenceFile = element;
                if (referenceFile.children.length > 0) {
                    return referenceFile.children[0].resource;
                }
            }
            return null;
        };
        ReferenceWidget.prototype.focus = function () {
            this.tree.DOMFocus();
        };
        ReferenceWidget.prototype.showReferencePreview = function (reference) {
            var _this = this;
            // show in editor
            this.editorService.resolveEditorModel({ resource: reference.resource }).done(function (model) {
                if (model) {
                    _this.preview.setModel(model.textEditorModel);
                    var sel = range_1.Range.lift(reference.range).collapseToStart();
                    _this.preview.setSelection(sel);
                    _this.preview.revealRangeInCenter(sel);
                }
                else {
                    _this.preview.setModel(_this.previewNotAvailableMessage);
                }
                // Update widget header
                if (reference.resource.scheme !== network_1.Schemas.inMemory) {
                    _this.setTitle(reference.name, labels_1.getPathLabel(reference.directory, _this.contextService));
                }
                else {
                    _this.setTitle(nls.localize(4, null));
                }
            }, errors_1.onUnexpectedError);
            // show in tree
            this.tree.reveal(reference)
                .then(function () {
                _this.tree.setSelection([reference]);
                _this.tree.setFocus(reference);
            })
                .done(null, errors_1.onUnexpectedError);
        };
        ReferenceWidget.prototype.dispose = function () {
            this.setModel(null);
            lifecycle_1.disposeAll([this.preview, this.previewNotAvailableMessage, this.tree]);
            _super.prototype.dispose.call(this);
        };
        ReferenceWidget.INNER_EDITOR_CONTEXT_KEY = 'inReferenceSearchEditor';
        ReferenceWidget.Events = {
            EditorDoubleClick: 'editorDoubleClick'
        };
        return ReferenceWidget;
    }(peekViewWidget_1.PeekViewWidget));
    exports.ReferenceWidget = ReferenceWidget;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/referenceSearch/browser/referenceSearch", ["require", "exports", 'vs/nls!vs/editor/contrib/referenceSearch/browser/referenceSearch', 'vs/base/common/arrays', 'vs/base/common/errors', 'vs/base/common/keyCodes', 'vs/base/common/lifecycle', 'vs/base/common/severity', 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/platform/editor/common/editor', 'vs/platform/instantiation/common/instantiation', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/platform/message/common/message', 'vs/platform/telemetry/common/telemetry', 'vs/platform/workspace/common/workspace', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/browser/editorBrowserExtensions', 'vs/editor/contrib/zoneWidget/browser/peekViewWidget', '../common/referenceSearch', './referenceSearchModel', './referenceSearchWidget'], function (require, exports, nls, arrays_1, errors_1, keyCodes_1, lifecycle_1, severity_1, uri_1, winjs_base_1, editor_1, instantiation_1, keybindingService_1, keybindingsRegistry_1, message_1, telemetry_1, workspace_1, position_1, range_1, editorAction_1, editorActionEnablement_1, editorCommon, editorCommonExtensions_1, editorBrowserExtensions_1, peekViewWidget_1, referenceSearch_1, referenceSearchModel_1, referenceSearchWidget_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FindReferencesController = (function () {
        function FindReferencesController(editor, editorService, telemetryService, messageService, instantiationService, peekViewService, contextService, keybindingService) {
            this._startTime = -1;
            this.requestIdPool = 0;
            this.callOnClear = [];
            this.editorService = editorService;
            this.telemetryService = telemetryService;
            this.messageService = messageService;
            this.instantiationService = instantiationService;
            this.peekViewService = peekViewService;
            this.contextService = contextService;
            this.keybindingService = keybindingService;
            this.modelRevealing = false;
            this.editor = editor;
            this._referenceSearchVisible = keybindingService.createKey(CONTEXT_REFERENCE_SEARCH_VISIBLE, false);
        }
        FindReferencesController.getController = function (editor) {
            return editor.getContribution(FindReferencesController.ID);
        };
        FindReferencesController.prototype.getId = function () {
            return FindReferencesController.ID;
        };
        FindReferencesController.prototype.dispose = function () {
            if (this.widget) {
                this.widget.dispose();
                this.widget = null;
            }
            this.editor = null;
        };
        FindReferencesController.prototype.isInPeekView = function () {
            return this.peekViewService && this.peekViewService.isActive;
        };
        FindReferencesController.prototype.closeReferenceSearch = function () {
            this.clear();
        };
        FindReferencesController.prototype.processRequest = function (range, referencesPromise, metaTitleFn) {
            var _this = this;
            var widgetPosition = !this.widget ? null : this.widget.position;
            // clean up from previous invocation
            var widgetClosed = this.clear();
            // Close if the position is still the same
            if (widgetClosed && !!widgetPosition && range.containsPosition(widgetPosition)) {
                return null;
            }
            this._referenceSearchVisible.set(true);
            // close the widget on model/mode changes
            this.callOnClear.push(this.editor.addListener(editorCommon.EventType.ModelModeChanged, function () { _this.clear(); }));
            this.callOnClear.push(this.editor.addListener(editorCommon.EventType.ModelChanged, function () {
                if (!_this.modelRevealing) {
                    _this.clear();
                }
            }));
            this.widget = new referenceSearchWidget_1.ReferenceWidget(this.editorService, this.keybindingService, this.contextService, this.instantiationService, this.editor);
            this.widget.setTitle(nls.localize(0, null));
            this.widget.show(range, 18);
            this.callOnClear.push(this.widget.addListener(peekViewWidget_1.Events.Closed, function () {
                _this.widget = null;
                _this.clear();
            }));
            this.callOnClear.push(this.widget.addListener(referenceSearchWidget_1.ReferenceWidget.Events.EditorDoubleClick, function (event) {
                if (!event.reference) {
                    return;
                }
                // open editor
                _this.editorService.openEditor({
                    resource: event.reference,
                    options: { selection: event.range }
                }, event.originalEvent.ctrlKey || event.originalEvent.metaKey).done(null, errors_1.onUnexpectedError);
                // close zone
                if (!(event.originalEvent.ctrlKey || event.originalEvent.metaKey)) {
                    _this.clear();
                }
            }));
            var requestId = ++this.requestIdPool, editorModel = this.editor.getModel();
            var timer = this.telemetryService.start('findReferences', {
                mode: editorModel.getMode().getId()
            });
            referencesPromise.then(function (references) {
                // still current request?
                if (requestId !== _this.requestIdPool) {
                    timer.stop();
                    return;
                }
                // has a result
                if (arrays_1.isFalsyOrEmpty(references)) {
                    _this.widget.showMessage(nls.localize(1, null));
                    timer.stop();
                    return;
                }
                // create result model
                _this.model = new referenceSearchModel_1.ReferencesModel(references, _this.editorService);
                _this.model.currentReference = _this.model.findReference(editorModel.getAssociatedResource(), range.getStartPosition());
                var unbind = _this.model.addListener(referenceSearchModel_1.EventType.CurrentReferenceChanged, function () {
                    _this.modelRevealing = true;
                    _this.editorService.openEditor({
                        resource: _this.model.currentReference.resource,
                        options: { selection: _this.model.currentReference.range }
                    }).done(function (openedEditor) {
                        if (!openedEditor || openedEditor.getControl() !== _this.editor) {
                            // TODO@Alex TODO@Joh
                            // when opening the current reference we might end up
                            // in a different editor instance. that means we also have
                            // a different instance of this reference search controller
                            // and cannot hold onto the widget (which likely doesn't
                            // exist). Instead of bailing out we should find the
                            // 'sister' action and pass our current model on to it.
                            _this.clear();
                            return;
                        }
                        _this.modelRevealing = false;
                        _this.widget.show(_this.model.currentReference.range, 18);
                        _this.widget.focus();
                    }, function (err) {
                        _this.modelRevealing = false;
                        errors_1.onUnexpectedError(err);
                    });
                });
                _this.callOnClear.push(unbind);
                // show widget
                _this._startTime = Date.now();
                if (_this.widget) {
                    _this.widget.setMetaTitle(metaTitleFn(references));
                    _this.widget.setModel(_this.model);
                }
                timer.stop();
            }, function (error) {
                _this.messageService.show(severity_1.default.Error, error);
                timer.stop();
            });
            return this.widget;
        };
        FindReferencesController.prototype.clear = function () {
            if (this._startTime !== -1) {
                this.telemetryService.publicLog('zoneWidgetShown', {
                    mode: 'reference search',
                    elapsedTime: Date.now() - this._startTime
                });
                this._startTime = -1;
            }
            this._referenceSearchVisible.reset();
            lifecycle_1.cAll(this.callOnClear);
            this.model = null;
            var result = false;
            if (this.widget) {
                this.widget.dispose();
                this.widget = null;
                result = true;
            }
            this.editor.focus();
            this.requestIdPool += 1; // Cancel pending requests
            return result;
        };
        FindReferencesController.ID = 'editor.contrib.findReferencesController';
        FindReferencesController = __decorate([
            __param(1, editor_1.IEditorService),
            __param(2, telemetry_1.ITelemetryService),
            __param(3, message_1.IMessageService),
            __param(4, instantiation_1.IInstantiationService),
            __param(5, peekViewWidget_1.IPeekViewService),
            __param(6, workspace_1.IWorkspaceContextService),
            __param(7, keybindingService_1.IKeybindingService)
        ], FindReferencesController);
        return FindReferencesController;
    }());
    exports.FindReferencesController = FindReferencesController;
    var ReferenceAction = (function (_super) {
        __extends(ReferenceAction, _super);
        // state - changes with every invocation
        function ReferenceAction(descriptor, editor, peekViewService, keybindingService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.ShowInContextMenu | editorActionEnablement_1.Behaviour.UpdateOnCursorPositionChange);
            this.label = nls.localize(2, null);
            this.peekViewService = peekViewService;
            if (this.peekViewService) {
                keybindingService.createKey(this.peekViewService.contextKey, true);
            }
        }
        ReferenceAction.prototype.getGroupId = function () {
            return '1_goto/4_references';
        };
        ReferenceAction.prototype.isSupported = function () {
            return referenceSearch_1.ReferenceRegistry.has(this.editor.getModel()) && _super.prototype.isSupported.call(this);
        };
        ReferenceAction.prototype.getEnablementState = function () {
            if (this.peekViewService && this.peekViewService.isActive) {
                return false;
            }
            var model = this.editor.getModel();
            var position = this.editor.getSelection().getStartPosition();
            var context = model.getLineContext(position.lineNumber);
            var offset = position.column - 1;
            return referenceSearch_1.ReferenceRegistry.all(model).some(function (support) {
                return support.canFindReferences(context, offset);
            });
        };
        ReferenceAction.prototype.run = function () {
            var range = this.editor.getSelection();
            var model = this.editor.getModel();
            var request = referenceSearch_1.findReferences(model, range.getStartPosition());
            var controller = FindReferencesController.getController(this.editor);
            return winjs_base_1.TPromise.as(controller.processRequest(range, request, metaTitle)).then(function () { return true; });
        };
        ReferenceAction.ID = 'editor.action.referenceSearch.trigger';
        ReferenceAction = __decorate([
            __param(2, peekViewWidget_1.IPeekViewService),
            __param(3, keybindingService_1.IKeybindingService)
        ], ReferenceAction);
        return ReferenceAction;
    }(editorAction_1.EditorAction));
    exports.ReferenceAction = ReferenceAction;
    function metaTitle(references) {
        if (references.length > 1) {
            return nls.localize(3, null, references.length);
        }
    }
    var findReferencesCommand = function (accessor, args) {
        var resource = args[0];
        var position = args[1];
        if (!(resource instanceof uri_1.default)) {
            throw new Error('illegal argument, uri');
        }
        if (!position) {
            throw new Error('illega argument, position');
        }
        return accessor.get(editor_1.IEditorService).openEditor({ resource: resource }).then(function (editor) {
            var control = editor.getControl();
            if (!control || typeof control.getEditorType !== 'function') {
                return;
            }
            var request = referenceSearch_1.findReferences(control.getModel(), position);
            var controller = FindReferencesController.getController(control);
            var range = new range_1.Range(position.lineNumber, position.column, position.lineNumber, position.column);
            return winjs_base_1.TPromise.as(controller.processRequest(range, request, metaTitle));
        });
    };
    var showReferencesCommand = function (accessor, args) {
        if (!(args[0] instanceof uri_1.default)) {
            throw new Error('illegal argument, uri expected');
        }
        return accessor.get(editor_1.IEditorService).openEditor({ resource: args[0] }).then(function (editor) {
            var control = editor.getControl();
            if (!control || typeof control.getEditorType !== 'function') {
                return;
            }
            var controller = FindReferencesController.getController(control);
            var range = position_1.Position.asEmptyRange(args[1]);
            return winjs_base_1.TPromise.as(controller.processRequest(range_1.Range.lift(range), winjs_base_1.TPromise.as(args[2]), metaTitle)).then(function () { return true; });
        });
    };
    var CONTEXT_REFERENCE_SEARCH_VISIBLE = 'referenceSearchVisible';
    // register action
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(FindReferencesController);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(ReferenceAction, ReferenceAction.ID, nls.localize(4, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.F12
    }));
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
        id: 'editor.action.findReferences',
        handler: findReferencesCommand,
        weight: editorCommonExtensions_1.CommonEditorRegistry.commandWeight(50),
        context: null,
        primary: undefined
    });
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
        id: 'editor.action.showReferences',
        handler: showReferencesCommand,
        weight: editorCommonExtensions_1.CommonEditorRegistry.commandWeight(50),
        context: null,
        primary: undefined,
        description: {
            description: 'Show references at a position in a file',
            args: [
                { name: 'uri', description: 'The text document in which to show references', constraint: uri_1.default },
                { name: 'position', description: 'The position at which to show', constraint: position_1.Position.isIPosition },
                { name: 'locations', description: 'An array of locations.', constraint: Array },
            ]
        }
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('closeReferenceSearch', editorCommonExtensions_1.CommonEditorRegistry.commandWeight(50), { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, false, CONTEXT_REFERENCE_SEARCH_VISIBLE, function (accessor, editor, args) {
        var outerEditor = peekViewWidget_1.getOuterEditor(accessor, args);
        if (outerEditor) {
            var controller = FindReferencesController.getController(outerEditor);
            controller.closeReferenceSearch();
        }
    });
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand('closeReferenceSearchEditor', editorCommonExtensions_1.CommonEditorRegistry.commandWeight(-101), { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, false, referenceSearchWidget_1.ReferenceWidget.INNER_EDITOR_CONTEXT_KEY, function (accessor, editor, args) {
        var outerEditor = peekViewWidget_1.getOuterEditor(accessor, args);
        if (outerEditor) {
            var controller = FindReferencesController.getController(outerEditor);
            controller.closeReferenceSearch();
        }
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/goToDeclaration/browser/goToDeclaration", ["require", "exports", 'vs/nls!vs/editor/contrib/goToDeclaration/browser/goToDeclaration', 'vs/base/common/arrays', 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/keyCodes', 'vs/base/common/platform', 'vs/base/common/severity', 'vs/base/common/strings', 'vs/base/common/winjs.base', 'vs/base/browser/browser', 'vs/platform/editor/common/editor', 'vs/platform/message/common/message', 'vs/platform/request/common/request', 'vs/editor/common/core/range', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/textToHtmlTokenizer', 'vs/editor/browser/editorBrowserExtensions', 'vs/editor/contrib/goToDeclaration/common/goToDeclaration', 'vs/editor/contrib/referenceSearch/browser/referenceSearch', 'vs/css!./goToDeclaration'], function (require, exports, nls, arrays_1, async_1, errors_1, keyCodes_1, platform, severity_1, strings, winjs_base_1, browser, editor_1, message_1, request_1, range_1, editorAction_1, editorActionEnablement_1, editorCommon, editorCommonExtensions_1, textToHtmlTokenizer_1, editorBrowserExtensions_1, goToDeclaration_1, referenceSearch_1) {
    'use strict';
    var DEFAULT_BEHAVIOR = editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.ShowInContextMenu | editorActionEnablement_1.Behaviour.UpdateOnCursorPositionChange;
    function metaTitle(references) {
        if (references.length > 1) {
            return nls.localize(0, null, references.length);
        }
    }
    var GoToTypeAction = (function (_super) {
        __extends(GoToTypeAction, _super);
        function GoToTypeAction(descriptor, editor, _messageService, _editorService, condition) {
            if (condition === void 0) { condition = DEFAULT_BEHAVIOR; }
            _super.call(this, descriptor, editor, condition);
            this._messageService = _messageService;
            this._editorService = _editorService;
        }
        GoToTypeAction.prototype.run = function () {
            var _this = this;
            var model = this.editor.getModel();
            var position = this.editor.getPosition();
            var promise = this._resolve(model.getAssociatedResource(), { lineNumber: position.lineNumber, column: position.column });
            return promise.then(function (references) {
                // remove falsy entries
                references = arrays_1.coalesce(references);
                if (!references || references.length === 0) {
                    return;
                }
                // only use the start position
                references = references.map(function (reference) {
                    return {
                        resource: reference.resource,
                        range: range_1.Range.collapseToStart(reference.range)
                    };
                });
                // open and reveal
                if (references.length === 1 && !_this._showSingleReferenceInPeek()) {
                    return _this._editorService.openEditor({
                        resource: references[0].resource,
                        options: { selection: references[0].range }
                    }, _this.openToTheSide);
                }
                else {
                    var controller = referenceSearch_1.FindReferencesController.getController(_this.editor);
                    return controller.processRequest(_this.editor.getSelection(), winjs_base_1.TPromise.as(references), metaTitle);
                }
            }, function (err) {
                // report an error
                _this._messageService.show(severity_1.default.Error, err);
                return false;
            });
        };
        Object.defineProperty(GoToTypeAction.prototype, "openToTheSide", {
            get: function () {
                return false;
            },
            enumerable: true,
            configurable: true
        });
        GoToTypeAction.prototype._showSingleReferenceInPeek = function () {
            return false;
        };
        return GoToTypeAction;
    }(editorAction_1.EditorAction));
    exports.GoToTypeAction = GoToTypeAction;
    var GoToTypeDeclarationActions = (function (_super) {
        __extends(GoToTypeDeclarationActions, _super);
        function GoToTypeDeclarationActions(descriptor, editor, messageService, editorService) {
            _super.call(this, descriptor, editor, messageService, editorService);
        }
        GoToTypeDeclarationActions.prototype.getGroupId = function () {
            return '1_goto/3_visitTypeDefinition';
        };
        GoToTypeDeclarationActions.prototype.isSupported = function () {
            return !!this.editor.getModel().getMode().typeDeclarationSupport && _super.prototype.isSupported.call(this);
        };
        GoToTypeDeclarationActions.prototype.getEnablementState = function () {
            if (!_super.prototype.getEnablementState.call(this)) {
                return false;
            }
            var model = this.editor.getModel(), position = this.editor.getSelection().getStartPosition();
            return model.getMode().typeDeclarationSupport.canFindTypeDeclaration(model.getLineContext(position.lineNumber), position.column - 1);
        };
        GoToTypeDeclarationActions.prototype._resolve = function (resource, position) {
            var typeDeclarationSupport = this.editor.getModel().getMode().typeDeclarationSupport;
            if (typeDeclarationSupport) {
                return typeDeclarationSupport.findTypeDeclaration(resource, position).then(function (value) { return [value]; });
            }
        };
        GoToTypeDeclarationActions.ID = 'editor.action.goToTypeDeclaration';
        GoToTypeDeclarationActions = __decorate([
            __param(2, message_1.IMessageService),
            __param(3, editor_1.IEditorService)
        ], GoToTypeDeclarationActions);
        return GoToTypeDeclarationActions;
    }(GoToTypeAction));
    exports.GoToTypeDeclarationActions = GoToTypeDeclarationActions;
    var BaseGoToDeclarationAction = (function (_super) {
        __extends(BaseGoToDeclarationAction, _super);
        function BaseGoToDeclarationAction(descriptor, editor, messageService, editorService, condition) {
            _super.call(this, descriptor, editor, messageService, editorService, condition);
        }
        BaseGoToDeclarationAction.prototype.getGroupId = function () {
            return '1_goto/2_visitDefinition';
        };
        BaseGoToDeclarationAction.prototype.isSupported = function () {
            return goToDeclaration_1.DeclarationRegistry.has(this.editor.getModel()) && _super.prototype.isSupported.call(this);
        };
        BaseGoToDeclarationAction.prototype.getEnablementState = function () {
            if (!_super.prototype.getEnablementState.call(this)) {
                return false;
            }
            var model = this.editor.getModel(), position = this.editor.getSelection().getStartPosition();
            return goToDeclaration_1.DeclarationRegistry.all(model).some(function (provider) {
                return provider.canFindDeclaration(model.getLineContext(position.lineNumber), position.column - 1);
            });
        };
        BaseGoToDeclarationAction.prototype._resolve = function (resource, position) {
            return goToDeclaration_1.getDeclarationsAtPosition(this.editor.getModel(), this.editor.getPosition());
        };
        return BaseGoToDeclarationAction;
    }(GoToTypeAction));
    exports.BaseGoToDeclarationAction = BaseGoToDeclarationAction;
    var GoToDeclarationAction = (function (_super) {
        __extends(GoToDeclarationAction, _super);
        function GoToDeclarationAction(descriptor, editor, messageService, editorService) {
            _super.call(this, descriptor, editor, messageService, editorService, DEFAULT_BEHAVIOR);
        }
        GoToDeclarationAction.ID = 'editor.action.goToDeclaration';
        GoToDeclarationAction = __decorate([
            __param(2, message_1.IMessageService),
            __param(3, editor_1.IEditorService)
        ], GoToDeclarationAction);
        return GoToDeclarationAction;
    }(BaseGoToDeclarationAction));
    exports.GoToDeclarationAction = GoToDeclarationAction;
    var OpenDeclarationToTheSideAction = (function (_super) {
        __extends(OpenDeclarationToTheSideAction, _super);
        function OpenDeclarationToTheSideAction(descriptor, editor, messageService, editorService) {
            _super.call(this, descriptor, editor, messageService, editorService, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.UpdateOnCursorPositionChange);
        }
        Object.defineProperty(OpenDeclarationToTheSideAction.prototype, "openToTheSide", {
            get: function () {
                return true;
            },
            enumerable: true,
            configurable: true
        });
        OpenDeclarationToTheSideAction.ID = 'editor.action.openDeclarationToTheSide';
        OpenDeclarationToTheSideAction = __decorate([
            __param(2, message_1.IMessageService),
            __param(3, editor_1.IEditorService)
        ], OpenDeclarationToTheSideAction);
        return OpenDeclarationToTheSideAction;
    }(BaseGoToDeclarationAction));
    exports.OpenDeclarationToTheSideAction = OpenDeclarationToTheSideAction;
    var PreviewDeclarationAction = (function (_super) {
        __extends(PreviewDeclarationAction, _super);
        function PreviewDeclarationAction(descriptor, editor, messageService, editorService) {
            _super.call(this, descriptor, editor, messageService, editorService, DEFAULT_BEHAVIOR);
        }
        PreviewDeclarationAction.prototype._showSingleReferenceInPeek = function () {
            return true;
        };
        PreviewDeclarationAction.ID = 'editor.action.previewDeclaration';
        PreviewDeclarationAction = __decorate([
            __param(2, message_1.IMessageService),
            __param(3, editor_1.IEditorService)
        ], PreviewDeclarationAction);
        return PreviewDeclarationAction;
    }(BaseGoToDeclarationAction));
    exports.PreviewDeclarationAction = PreviewDeclarationAction;
    // --- Editor Contribution to goto definition using the mouse and a modifier key
    var GotoDefinitionWithMouseEditorContribution = (function () {
        function GotoDefinitionWithMouseEditorContribution(editor, requestService, messageService, editorService) {
            var _this = this;
            this.requestService = requestService;
            this.messageService = messageService;
            this.editorService = editorService;
            this.hasRequiredServices = !!this.messageService && !!this.requestService && !!this.editorService;
            this.toUnhook = [];
            this.decorations = [];
            this.editor = editor;
            this.throttler = new async_1.Throttler();
            this.toUnhook.push(this.editor.addListener(editorCommon.EventType.MouseDown, function (e) { return _this.onEditorMouseDown(e); }));
            this.toUnhook.push(this.editor.addListener(editorCommon.EventType.MouseUp, function (e) { return _this.onEditorMouseUp(e); }));
            this.toUnhook.push(this.editor.addListener(editorCommon.EventType.MouseMove, function (e) { return _this.onEditorMouseMove(e); }));
            this.toUnhook.push(this.editor.addListener(editorCommon.EventType.KeyDown, function (e) { return _this.onEditorKeyDown(e); }));
            this.toUnhook.push(this.editor.addListener(editorCommon.EventType.KeyUp, function (e) { return _this.onEditorKeyUp(e); }));
            this.toUnhook.push(this.editor.addListener(editorCommon.EventType.ModelChanged, function (e) { return _this.resetHandler(); }));
            this.toUnhook.push(this.editor.addListener('change', function (e) { return _this.resetHandler(); }));
            this.toUnhook.push(this.editor.addListener('scroll', function () { return _this.resetHandler(); }));
        }
        GotoDefinitionWithMouseEditorContribution.prototype.onEditorMouseMove = function (mouseEvent, withKey) {
            this.lastMouseMoveEvent = mouseEvent;
            this.startFindDefinition(mouseEvent, withKey);
        };
        GotoDefinitionWithMouseEditorContribution.prototype.startFindDefinition = function (mouseEvent, withKey) {
            var _this = this;
            if (!this.isEnabled(mouseEvent, withKey)) {
                this.currentWordUnderMouse = null;
                this.removeDecorations();
                return;
            }
            // Find word at mouse position
            var position = mouseEvent.target.position;
            var word = position ? this.editor.getModel().getWordAtPosition(position) : null;
            if (!word) {
                this.currentWordUnderMouse = null;
                this.removeDecorations();
                return;
            }
            // Return early if word at position is still the same
            if (this.currentWordUnderMouse && this.currentWordUnderMouse.startColumn === word.startColumn && this.currentWordUnderMouse.endColumn === word.endColumn && this.currentWordUnderMouse.word === word.word) {
                return;
            }
            this.currentWordUnderMouse = word;
            // Find definition and decorate word if found
            var state = this.editor.captureState(editorCommon.CodeEditorStateFlag.Position, editorCommon.CodeEditorStateFlag.Value, editorCommon.CodeEditorStateFlag.Selection, editorCommon.CodeEditorStateFlag.Scroll);
            this.throttler.queue(function () {
                return state.validate(_this.editor)
                    ? _this.findDefinition(mouseEvent.target)
                    : winjs_base_1.TPromise.as(null);
            }).then(function (results) {
                if (!results || !results.length || !state.validate(_this.editor)) {
                    _this.removeDecorations();
                    return;
                }
                // Multiple results
                if (results.length > 1) {
                    _this.addDecoration({
                        startLineNumber: position.lineNumber,
                        startColumn: word.startColumn,
                        endLineNumber: position.lineNumber,
                        endColumn: word.endColumn
                    }, nls.localize(1, null, results.length), false);
                }
                else {
                    var result_1 = results[0];
                    _this.editorService.resolveEditorModel({ resource: result_1.resource }).then(function (model) {
                        var source;
                        if (model && model.textEditorModel) {
                            var from = Math.max(1, result_1.range.startLineNumber), to = void 0, editorModel = void 0;
                            editorModel = model.textEditorModel;
                            // if we have a range, take that into consideration for the "to" position, otherwise fallback to MAX_SOURCE_PREVIEW_LINES
                            if (result_1.range.startLineNumber !== result_1.range.endLineNumber || result_1.range.startColumn !== result_1.range.endColumn) {
                                to = Math.min(result_1.range.endLineNumber, result_1.range.startLineNumber + GotoDefinitionWithMouseEditorContribution.MAX_SOURCE_PREVIEW_LINES, editorModel.getLineCount());
                            }
                            else {
                                to = Math.min(from + GotoDefinitionWithMouseEditorContribution.MAX_SOURCE_PREVIEW_LINES, editorModel.getLineCount());
                            }
                            source = editorModel.getValueInRange({
                                startLineNumber: from,
                                startColumn: 1,
                                endLineNumber: to,
                                endColumn: editorModel.getLineMaxColumn(to)
                            }).trim();
                            // remove common leading whitespace
                            var min = Number.MAX_VALUE, regexp = /^[ \t]*/, match = void 0, contents = void 0;
                            while (from <= to && min > 0) {
                                contents = editorModel.getLineContent(from++);
                                if (contents.trim().length === 0) {
                                    // empty or whitespace only
                                    continue;
                                }
                                match = regexp.exec(contents);
                                min = Math.min(min, match[0].length);
                            }
                            source = source.replace(new RegExp("^([ \\t]{" + min + "})", 'gm'), strings.empty);
                            if (result_1.range.endLineNumber - result_1.range.startLineNumber > GotoDefinitionWithMouseEditorContribution.MAX_SOURCE_PREVIEW_LINES) {
                                source += '\n\u2026';
                            }
                        }
                        _this.addDecoration({
                            startLineNumber: position.lineNumber,
                            startColumn: word.startColumn,
                            endLineNumber: position.lineNumber,
                            endColumn: word.endColumn
                        }, source, true);
                    });
                }
            }).done(undefined, errors_1.onUnexpectedError);
        };
        GotoDefinitionWithMouseEditorContribution.prototype.addDecoration = function (range, text, isCode) {
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var htmlMessage = {
                tagName: 'div',
                className: 'goto-definition-link-hover',
                style: "tab-size: " + model.getOptions().tabSize
            };
            if (text && text.trim().length > 0) {
                // not whitespace only
                htmlMessage.children = [isCode ? textToHtmlTokenizer_1.tokenizeToHtmlContent(text, model.getMode()) : { tagName: 'span', text: text }];
            }
            var newDecorations = {
                range: range,
                options: {
                    inlineClassName: 'goto-definition-link',
                    htmlMessage: [htmlMessage]
                }
            };
            this.decorations = this.editor.deltaDecorations(this.decorations, [newDecorations]);
        };
        GotoDefinitionWithMouseEditorContribution.prototype.removeDecorations = function () {
            if (this.decorations.length > 0) {
                this.decorations = this.editor.deltaDecorations(this.decorations, []);
            }
        };
        GotoDefinitionWithMouseEditorContribution.prototype.onEditorKeyDown = function (e) {
            if (this.lastMouseMoveEvent && (e.keyCode === GotoDefinitionWithMouseEditorContribution.TRIGGER_KEY_VALUE ||
                e.keyCode === GotoDefinitionWithMouseEditorContribution.TRIGGER_SIDEBYSIDE_KEY_VALUE && e[GotoDefinitionWithMouseEditorContribution.TRIGGER_MODIFIER] // User pressed Ctrl/Cmd+Alt (goto definition to the side)
            )) {
                this.startFindDefinition(this.lastMouseMoveEvent, e);
            }
            else if (e[GotoDefinitionWithMouseEditorContribution.TRIGGER_MODIFIER]) {
                this.removeDecorations(); // remove decorations if user holds another key with ctrl/cmd to prevent accident goto declaration
            }
        };
        GotoDefinitionWithMouseEditorContribution.prototype.resetHandler = function () {
            this.lastMouseMoveEvent = null;
            this.removeDecorations();
        };
        GotoDefinitionWithMouseEditorContribution.prototype.onEditorMouseDown = function (mouseEvent) {
            // We need to record if we had the trigger key on mouse down because someone might select something in the editor
            // holding the mouse down and then while mouse is down start to press Ctrl/Cmd to start a copy operation and then
            // release the mouse button without wanting to do the navigation.
            // With this flag we prevent goto definition if the mouse was down before the trigger key was pressed.
            this.hasTriggerKeyOnMouseDown = !!mouseEvent.event[GotoDefinitionWithMouseEditorContribution.TRIGGER_MODIFIER];
        };
        GotoDefinitionWithMouseEditorContribution.prototype.onEditorMouseUp = function (mouseEvent) {
            var _this = this;
            if (this.isEnabled(mouseEvent) && this.hasTriggerKeyOnMouseDown) {
                this.gotoDefinition(mouseEvent.target, mouseEvent.event.altKey).done(function () {
                    _this.removeDecorations();
                }, function (error) {
                    _this.removeDecorations();
                    errors_1.onUnexpectedError(error);
                });
            }
        };
        GotoDefinitionWithMouseEditorContribution.prototype.onEditorKeyUp = function (e) {
            if (e.keyCode === GotoDefinitionWithMouseEditorContribution.TRIGGER_KEY_VALUE) {
                this.removeDecorations();
                this.currentWordUnderMouse = null;
            }
        };
        GotoDefinitionWithMouseEditorContribution.prototype.isEnabled = function (mouseEvent, withKey) {
            return this.hasRequiredServices &&
                this.editor.getModel() &&
                (browser.isIE11orEarlier || mouseEvent.event.detail <= 1) &&
                mouseEvent.target.type === editorCommon.MouseTargetType.CONTENT_TEXT &&
                (mouseEvent.event[GotoDefinitionWithMouseEditorContribution.TRIGGER_MODIFIER] || (withKey && withKey.keyCode === GotoDefinitionWithMouseEditorContribution.TRIGGER_KEY_VALUE)) &&
                goToDeclaration_1.DeclarationRegistry.has(this.editor.getModel());
        };
        GotoDefinitionWithMouseEditorContribution.prototype.findDefinition = function (target) {
            var model = this.editor.getModel();
            if (!model) {
                return winjs_base_1.TPromise.as(null);
            }
            return goToDeclaration_1.getDeclarationsAtPosition(this.editor.getModel(), target.position);
        };
        GotoDefinitionWithMouseEditorContribution.prototype.gotoDefinition = function (target, sideBySide) {
            var _this = this;
            var state = this.editor.captureState(editorCommon.CodeEditorStateFlag.Position, editorCommon.CodeEditorStateFlag.Value, editorCommon.CodeEditorStateFlag.Selection, editorCommon.CodeEditorStateFlag.Scroll);
            return this.findDefinition(target).then(function (results) {
                if (!results || !results.length || !state.validate(_this.editor)) {
                    return;
                }
                var position = target.position;
                var word = _this.editor.getModel().getWordAtPosition(position);
                // Find valid target (and not the same position as the current hovered word)
                var validResults = results
                    .filter(function (result) { return result.range && !(word && result.range.startColumn === word.startColumn && result.range.startLineNumber === target.position.lineNumber); })
                    .map(function (result) {
                    return {
                        resource: result.resource,
                        range: range_1.Range.collapseToStart(result.range)
                    };
                });
                if (!validResults.length) {
                    return;
                }
                // Muli result: Show in references UI
                if (validResults.length > 1) {
                    var controller = referenceSearch_1.FindReferencesController.getController(_this.editor);
                    return controller.processRequest(_this.editor.getSelection(), winjs_base_1.TPromise.as(validResults), metaTitle);
                }
                // Single result: Open
                return _this.editorService.openEditor({
                    resource: validResults[0].resource,
                    options: {
                        selection: validResults[0].range
                    }
                }, sideBySide);
            });
        };
        GotoDefinitionWithMouseEditorContribution.prototype.getId = function () {
            return GotoDefinitionWithMouseEditorContribution.ID;
        };
        GotoDefinitionWithMouseEditorContribution.prototype.dispose = function () {
            while (this.toUnhook.length > 0) {
                this.toUnhook.pop()();
            }
        };
        GotoDefinitionWithMouseEditorContribution.ID = 'editor.contrib.gotodefinitionwithmouse';
        GotoDefinitionWithMouseEditorContribution.TRIGGER_MODIFIER = platform.isMacintosh ? 'metaKey' : 'ctrlKey';
        GotoDefinitionWithMouseEditorContribution.TRIGGER_SIDEBYSIDE_KEY_VALUE = keyCodes_1.KeyCode.Alt;
        GotoDefinitionWithMouseEditorContribution.TRIGGER_KEY_VALUE = platform.isMacintosh ? keyCodes_1.KeyCode.Meta : keyCodes_1.KeyCode.Ctrl;
        GotoDefinitionWithMouseEditorContribution.MAX_SOURCE_PREVIEW_LINES = 7;
        GotoDefinitionWithMouseEditorContribution = __decorate([
            __param(1, request_1.IRequestService),
            __param(2, message_1.IMessageService),
            __param(3, editor_1.IEditorService)
        ], GotoDefinitionWithMouseEditorContribution);
        return GotoDefinitionWithMouseEditorContribution;
    }());
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(PreviewDeclarationAction, PreviewDeclarationAction.ID, nls.localize(2, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.F12,
        linux: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.F10 },
    }));
    var goToDeclarationKb;
    if (platform.isWeb) {
        goToDeclarationKb = keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.F12;
    }
    else {
        goToDeclarationKb = keyCodes_1.KeyCode.F12;
    }
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(GoToDeclarationAction, GoToDeclarationAction.ID, nls.localize(3, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: goToDeclarationKb
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(OpenDeclarationToTheSideAction, OpenDeclarationToTheSideAction.ID, nls.localize(4, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, goToDeclarationKb)
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(GoToTypeDeclarationActions, GoToTypeDeclarationActions.ID, nls.localize(5, null)));
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(GotoDefinitionWithMouseEditorContribution);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/platform/telemetry/common/workspaceStats", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/errors', 'vs/base/common/flags', 'vs/platform/files/common/files', 'vs/platform/telemetry/common/telemetry', 'vs/platform/workspace/common/workspace'], function (require, exports, winjs, errors, env, files_1, telemetry_1, workspace_1) {
    'use strict';
    var WorkspaceStats = (function () {
        function WorkspaceStats(fileService, contextService, telemetryService) {
            this.fileService = fileService;
            this.contextService = contextService;
            this.telemetryService = telemetryService;
        }
        WorkspaceStats.prototype.searchArray = function (arr, regEx) {
            return arr.some(function (v) { return v.search(regEx) > -1; });
        };
        WorkspaceStats.prototype.getWorkspaceTags = function () {
            var _this = this;
            var tags = Object.create(null);
            var workspace = this.contextService.getWorkspace();
            if (workspace && this.fileService) {
                return this.fileService.resolveFile(workspace.resource).then(function (stats) {
                    var names = stats.children.map(function (c) { return c.name; });
                    tags['workspace.empty'] = false;
                    tags['workspace.language.cs'] = _this.searchArray(names, /^.+\.cs$/i);
                    tags['workspace.language.js'] = _this.searchArray(names, /^.+\.js$/i);
                    tags['workspace.language.ts'] = _this.searchArray(names, /^.+\.ts$/i);
                    tags['workspace.language.php'] = _this.searchArray(names, /^.+\.php$/i);
                    tags['workspace.language.python'] = _this.searchArray(names, /^.+\.py$/i);
                    tags['workspace.language.vb'] = _this.searchArray(names, /^.+\.vb$/i);
                    tags['workspace.language.aspx'] = _this.searchArray(names, /^.+\.aspx$/i);
                    tags['workspace.grunt'] = _this.searchArray(names, /^gruntfile\.js$/i);
                    tags['workspace.gulp'] = _this.searchArray(names, /^gulpfile\.js$/i);
                    tags['workspace.jake'] = _this.searchArray(names, /^jakefile\.js$/i);
                    tags['workspace.tsconfig'] = _this.searchArray(names, /^tsconfig\.json$/i);
                    tags['workspace.jsconfig'] = _this.searchArray(names, /^jsconfig\.json$/i);
                    tags['workspace.config.xml'] = _this.searchArray(names, /^config\.xml/i);
                    tags['workspace.vsc.extension'] = _this.searchArray(names, /^vsc-extension-quickstart\.md/i);
                    tags['workspace.ASP5'] = _this.searchArray(names, /^project\.json$/i) && tags['workspace.language.cs'];
                    tags['workspace.sln'] = _this.searchArray(names, /^.+\.sln$|^.+\.csproj$/i);
                    tags['workspace.unity'] = _this.searchArray(names, /^Assets$/i) && _this.searchArray(names, /^Library$/i) && _this.searchArray(names, /^ProjectSettings/i);
                    tags['workspace.npm'] = _this.searchArray(names, /^package\.json$|^node_modules$/i);
                    tags['workspace.bower'] = _this.searchArray(names, /^bower\.json$|^bower_components$/i);
                    tags['workspace.yeoman.code'] = _this.searchArray(names, /^vscodequickstart\.md$/i);
                    tags['workspace.yeoman.code.ext'] = _this.searchArray(names, /^vsc-extension-quickstart\.md$/i);
                    var mainActivity = _this.searchArray(names, /^MainActivity\.cs$/i) || _this.searchArray(names, /^MainActivity\.fs$/i);
                    var appDelegate = _this.searchArray(names, /^AppDelegate\.cs$/i) || _this.searchArray(names, /^AppDelegate\.fs$/i);
                    var androidManifest = _this.searchArray(names, /^AndroidManifest\.xml$/i);
                    var platforms = _this.searchArray(names, /^platforms$/i);
                    var plugins = _this.searchArray(names, /^plugins$/i);
                    var www = _this.searchArray(names, /^www$/i);
                    var properties = _this.searchArray(names, /^Properties/i);
                    var resources = _this.searchArray(names, /^Resources/i);
                    var jni = _this.searchArray(names, /^JNI/i);
                    if (tags['workspace.config.xml'] &&
                        !tags['workspace.language.cs'] && !tags['workspace.language.vb'] && !tags['workspace.language.aspx']) {
                        if (platforms && plugins && www) {
                            tags['workspace.cordova.high'] = true;
                        }
                        else {
                            tags['workspace.cordova.low'] = true;
                        }
                    }
                    if (mainActivity && properties && resources) {
                        tags['workspace.xamarin.android'] = true;
                    }
                    if (appDelegate && resources) {
                        tags['workspace.xamarin.ios'] = true;
                    }
                    if (androidManifest && jni) {
                        tags['workspace.android.cpp'] = true;
                    }
                    tags['workspace.reactNative'] = _this.searchArray(names, /^android$/i) && _this.searchArray(names, /^ios$/i) &&
                        _this.searchArray(names, /^index\.android\.js$/i) && _this.searchArray(names, /^index\.ios\.js$/i);
                    tags['workspace.enableTypeScriptServiceModeForJS'] = !!env.enableTypeScriptServiceModeForJS;
                    return tags;
                }, function (error) { errors.onUnexpectedError(error); return null; });
            }
            else {
                tags['workspace.empty'] = true;
                return winjs.TPromise.as(tags);
            }
        };
        WorkspaceStats.prototype.reportWorkspaceTags = function () {
            var _this = this;
            this.getWorkspaceTags().then(function (tags) {
                _this.telemetryService.publicLog('workspce.tags', tags);
            }, function (error) { return errors.onUnexpectedError(error); });
        };
        WorkspaceStats = __decorate([
            __param(0, files_1.IFileService),
            __param(1, workspace_1.IWorkspaceContextService),
            __param(2, telemetry_1.ITelemetryService)
        ], WorkspaceStats);
        return WorkspaceStats;
    }());
    exports.WorkspaceStats = WorkspaceStats;
});

define("vs/platform/workspace/common/baseWorkspaceContextService", ["require", "exports", 'vs/base/common/uri', 'vs/base/common/paths', './workspace'], function (require, exports, uri_1, paths, workspace_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Simple IWorkspaceContextService implementation to allow sharing of this service implementation
     * between different layers of the platform.
     */
    var BaseWorkspaceContextService = (function () {
        function BaseWorkspaceContextService(workspace, configuration, options) {
            if (options === void 0) { options = {}; }
            this.serviceId = workspace_1.IWorkspaceContextService;
            this.workspace = workspace;
            this.configuration = configuration;
            this.options = options;
        }
        BaseWorkspaceContextService.prototype.getWorkspace = function () {
            return this.workspace;
        };
        BaseWorkspaceContextService.prototype.getConfiguration = function () {
            return this.configuration;
        };
        BaseWorkspaceContextService.prototype.getOptions = function () {
            return this.options;
        };
        BaseWorkspaceContextService.prototype.isInsideWorkspace = function (resource) {
            if (resource && this.workspace) {
                return paths.isEqualOrParent(resource.fsPath, this.workspace.resource.fsPath);
            }
            return false;
        };
        BaseWorkspaceContextService.prototype.toWorkspaceRelativePath = function (resource) {
            if (this.isInsideWorkspace(resource)) {
                return paths.normalize(paths.relative(this.workspace.resource.fsPath, resource.fsPath));
            }
            return null;
        };
        BaseWorkspaceContextService.prototype.toResource = function (workspaceRelativePath) {
            if (typeof workspaceRelativePath === 'string' && this.workspace) {
                return uri_1.default.file(paths.join(this.workspace.resource.fsPath, workspaceRelativePath));
            }
            return null;
        };
        return BaseWorkspaceContextService;
    }());
    exports.BaseWorkspaceContextService = BaseWorkspaceContextService;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostConfiguration", ["require", "exports", 'vs/base/common/objects', 'vs/base/common/lifecycle', 'vs/platform/thread/common/thread', 'vs/platform/configuration/common/configuration', 'vs/base/common/event'], function (require, exports, objects_1, lifecycle_1, thread_1, configuration_1, event_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExtHostConfiguration = (function () {
        function ExtHostConfiguration() {
            this._onDidChangeConfiguration = new event_1.Emitter();
        }
        Object.defineProperty(ExtHostConfiguration.prototype, "onDidChangeConfiguration", {
            get: function () {
                return this._onDidChangeConfiguration && this._onDidChangeConfiguration.event;
            },
            enumerable: true,
            configurable: true
        });
        ExtHostConfiguration.prototype._acceptConfigurationChanged = function (config) {
            this._config = config;
            this._hasConfig = true;
            this._onDidChangeConfiguration.fire(undefined);
        };
        ExtHostConfiguration.prototype.getConfiguration = function (section) {
            if (!this._hasConfig) {
                return;
            }
            var config = section
                ? ExtHostConfiguration._lookUp(section, this._config)
                : this._config;
            var result = config ? objects_1.clone(config) : {};
            // result = Object.freeze(result);
            result.has = function (key) {
                return typeof ExtHostConfiguration._lookUp(key, config) !== 'undefined';
            };
            result.get = function (key, defaultValue) {
                var result = ExtHostConfiguration._lookUp(key, config);
                if (typeof result === 'undefined') {
                    result = defaultValue;
                }
                return result;
            };
            return result;
        };
        ExtHostConfiguration._lookUp = function (section, config) {
            if (!section) {
                return;
            }
            var parts = section.split('.');
            var node = config;
            while (node && parts.length) {
                node = node[parts.shift()];
            }
            return node;
        };
        ExtHostConfiguration = __decorate([
            thread_1.Remotable.ExtHostContext('ExtHostConfiguration')
        ], ExtHostConfiguration);
        return ExtHostConfiguration;
    }());
    exports.ExtHostConfiguration = ExtHostConfiguration;
    var MainThreadConfiguration = (function () {
        function MainThreadConfiguration(configurationService, threadService) {
            var _this = this;
            this._configurationService = configurationService;
            this._proxy = threadService.getRemotable(ExtHostConfiguration);
            this._toDispose = [];
            this._toDispose.push(this._configurationService.addListener2(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) {
                _this._proxy._acceptConfigurationChanged(e.config);
            }));
            this._configurationService.loadConfiguration().then(function (config) {
                _this._proxy._acceptConfigurationChanged(config);
            });
        }
        MainThreadConfiguration.prototype.dispose = function () {
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        MainThreadConfiguration = __decorate([
            thread_1.Remotable.MainContext('MainProcessConfigurationServiceHelper'),
            __param(0, configuration_1.IConfigurationService),
            __param(1, thread_1.IThreadService)
        ], MainThreadConfiguration);
        return MainThreadConfiguration;
    }());
    exports.MainThreadConfiguration = MainThreadConfiguration;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostDiagnostics", ["require", "exports", 'vs/platform/thread/common/thread', 'vs/platform/markers/common/markers', 'vs/base/common/uri', 'vs/base/common/severity'], function (require, exports, thread_1, markers_1, uri_1, severity_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DiagnosticCollection = (function () {
        function DiagnosticCollection(name, proxy) {
            this._name = name;
            this._proxy = proxy;
        }
        DiagnosticCollection.prototype.dispose = function () {
            var _this = this;
            if (!this._isDisposed) {
                this._proxy._changeAll(this.name, undefined).then(function () {
                    _this._proxy = undefined;
                    _this._isDisposed = true;
                });
            }
        };
        Object.defineProperty(DiagnosticCollection.prototype, "name", {
            get: function () {
                this._checkDisposed();
                return this._name;
            },
            enumerable: true,
            configurable: true
        });
        DiagnosticCollection.prototype.set = function (first, diagnostics) {
            this._checkDisposed();
            if (first instanceof uri_1.default) {
                // change markers of resource only (max 500)
                var data = void 0;
                if (diagnostics) {
                    data = [];
                    var len = diagnostics.length;
                    if (len > DiagnosticCollection._maxDiagnosticsPerFile) {
                        console.warn('diagnostics for %s will be capped to %d (actually is %d)', first.toString(), DiagnosticCollection._maxDiagnosticsPerFile, len);
                        len = DiagnosticCollection._maxDiagnosticsPerFile;
                    }
                    for (var i = 0; i < len; i++) {
                        data.push(DiagnosticCollection._toMarkerData(diagnostics[i]));
                    }
                }
                // set or reset for this resource
                return this._proxy._changeOne(this.name, first, data);
            }
            else {
                // change all marker of owner
                var entries = first;
                var data = void 0;
                if (entries) {
                    var total = 0;
                    data = [];
                    for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
                        var entry = entries_1[_i];
                        var uri = entry[0], diagnostics_1 = entry[1];
                        if (diagnostics_1) {
                            var len = diagnostics_1.length;
                            if (len > DiagnosticCollection._maxDiagnosticsPerFile) {
                                console.warn('diagnostics for %s will be capped to %d (actually is %d)', uri.toString(), DiagnosticCollection._maxDiagnosticsPerFile, len);
                                len = DiagnosticCollection._maxDiagnosticsPerFile;
                            }
                            for (var i = 0; i < len; i++) {
                                data.push({
                                    resource: uri,
                                    marker: DiagnosticCollection._toMarkerData(diagnostics_1[i])
                                });
                            }
                            total += len;
                            if (total > 10 * DiagnosticCollection._maxDiagnosticsPerFile) {
                                console.warn('too many diagnostics will cap to %d', 10 * DiagnosticCollection._maxDiagnosticsPerFile);
                                break;
                            }
                        }
                    }
                }
                // set or reset all
                this._proxy._changeAll(this.name, data);
            }
        };
        DiagnosticCollection.prototype.delete = function (uri) {
            return this.set(uri, undefined);
        };
        DiagnosticCollection.prototype.clear = function () {
            return this.set(undefined);
        };
        DiagnosticCollection.prototype._checkDisposed = function () {
            if (this._isDisposed) {
                throw new Error('illegal state - object is disposed');
            }
        };
        DiagnosticCollection._toMarkerData = function (diagnostic) {
            var range = diagnostic.range;
            return {
                startLineNumber: range.start.line + 1,
                startColumn: range.start.character + 1,
                endLineNumber: range.end.line + 1,
                endColumn: range.end.character + 1,
                message: diagnostic.message,
                source: diagnostic.source,
                severity: DiagnosticCollection._convertDiagnosticsSeverity(diagnostic.severity),
                code: String(diagnostic.code)
            };
        };
        DiagnosticCollection._convertDiagnosticsSeverity = function (severity) {
            switch (severity) {
                case 0: return severity_1.default.Error;
                case 1: return severity_1.default.Warning;
                case 2: return severity_1.default.Info;
                case 3: return severity_1.default.Ignore;
                default: return severity_1.default.Error;
            }
        };
        DiagnosticCollection._maxDiagnosticsPerFile = 250;
        return DiagnosticCollection;
    }());
    var ExtHostDiagnostics = (function () {
        function ExtHostDiagnostics(threadService) {
            this._proxy = threadService.getRemotable(MainThreadDiagnostics);
        }
        ExtHostDiagnostics.prototype.createDiagnosticCollection = function (name) {
            if (!name) {
                name = '_generated_diagnostic_collection_name_#' + ExtHostDiagnostics._idPool++;
            }
            return new DiagnosticCollection(name, this._proxy);
        };
        ExtHostDiagnostics._idPool = 0;
        return ExtHostDiagnostics;
    }());
    exports.ExtHostDiagnostics = ExtHostDiagnostics;
    var MainThreadDiagnostics = (function () {
        function MainThreadDiagnostics(markerService) {
            this._markerService = markerService;
        }
        MainThreadDiagnostics.prototype._changeOne = function (owner, resource, markers) {
            this._markerService.changeOne(owner, resource, markers);
            return undefined;
        };
        MainThreadDiagnostics.prototype._changeAll = function (owner, data) {
            this._markerService.changeAll(owner, data);
            return undefined;
        };
        MainThreadDiagnostics = __decorate([
            thread_1.Remotable.MainContext('MainThreadDiagnostics'),
            __param(0, markers_1.IMarkerService)
        ], MainThreadDiagnostics);
        return MainThreadDiagnostics;
    }());
    exports.MainThreadDiagnostics = MainThreadDiagnostics;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostLanguages", ["require", "exports", 'vs/base/common/winjs.base', 'vs/platform/thread/common/thread', 'vs/editor/common/services/modeService'], function (require, exports, winjs_base_1, thread_1, modeService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExtHostLanguages = (function () {
        function ExtHostLanguages(threadService) {
            this._proxy = threadService.getRemotable(MainThreadLanguages);
        }
        ExtHostLanguages.prototype.getLanguages = function () {
            return this._proxy._getLanguages();
        };
        ExtHostLanguages = __decorate([
            __param(0, thread_1.IThreadService)
        ], ExtHostLanguages);
        return ExtHostLanguages;
    }());
    exports.ExtHostLanguages = ExtHostLanguages;
    var MainThreadLanguages = (function () {
        function MainThreadLanguages(modeService) {
            this._modeService = modeService;
        }
        MainThreadLanguages.prototype._getLanguages = function () {
            return winjs_base_1.TPromise.as(this._modeService.getRegisteredModes());
        };
        MainThreadLanguages = __decorate([
            thread_1.Remotable.MainContext('MainThreadLanguages'),
            __param(0, modeService_1.IModeService)
        ], MainThreadLanguages);
        return MainThreadLanguages;
    }());
    exports.MainThreadLanguages = MainThreadLanguages;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostMessageService", ["require", "exports", 'vs/nls!vs/workbench/api/node/extHostMessageService', 'vs/platform/thread/common/thread', 'vs/platform/message/common/message', 'vs/base/common/actions', 'vs/base/common/winjs.base'], function (require, exports, nls, thread_1, message_1, actions_1, winjs_base_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExtHostMessageService = (function () {
        function ExtHostMessageService(threadService, commands) {
            this._proxy = threadService.getRemotable(MainThreadMessageService);
            this._commands = commands;
        }
        ExtHostMessageService.prototype.showMessage = function (severity, message, commands) {
            var items = [];
            for (var handle = 0; handle < commands.length; handle++) {
                var command = commands[handle];
                if (typeof command === 'string') {
                    items.push({ title: command, handle: handle });
                }
                else {
                    items.push({ title: command.title, handle: handle });
                }
            }
            return this._proxy.showMessage(severity, message, items).then(function (handle) {
                if (typeof handle === 'number') {
                    return commands[handle];
                }
            });
        };
        ExtHostMessageService = __decorate([
            __param(0, thread_1.IThreadService)
        ], ExtHostMessageService);
        return ExtHostMessageService;
    }());
    exports.ExtHostMessageService = ExtHostMessageService;
    var MainThreadMessageService = (function () {
        function MainThreadMessageService(messageService) {
            this._messageService = messageService;
        }
        MainThreadMessageService.prototype.showMessage = function (severity, message, commands) {
            var _this = this;
            var hide;
            var actions = [];
            actions.push(new actions_1.Action('__close', nls.localize(0, null), undefined, true, function () {
                hide();
                return winjs_base_1.TPromise.as(undefined);
            }));
            commands.forEach(function (command) {
                actions.push(new actions_1.Action('_extension_message_handle_' + command.handle, command.title, undefined, true, function () {
                    hide(command.handle);
                    return winjs_base_1.TPromise.as(undefined);
                }));
            });
            return new winjs_base_1.TPromise(function (c) {
                var messageHide;
                hide = function (handle) {
                    messageHide();
                    c(handle);
                };
                messageHide = _this._messageService.show(severity, {
                    message: message,
                    actions: actions
                });
            });
        };
        MainThreadMessageService = __decorate([
            thread_1.Remotable.MainContext('MainThreadMessageService'),
            __param(0, message_1.IMessageService)
        ], MainThreadMessageService);
        return MainThreadMessageService;
    }());
    exports.MainThreadMessageService = MainThreadMessageService;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/api/node/extHostTypes", ["require", "exports", 'vs/base/common/uri', 'vs/base/common/errors'], function (require, exports, uri_1, errors_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Disposable = (function () {
        function Disposable(callOnDispose) {
            this._callOnDispose = callOnDispose;
        }
        Disposable.from = function () {
            var disposables = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                disposables[_i - 0] = arguments[_i];
            }
            return new Disposable(function () {
                if (disposables) {
                    for (var _i = 0, disposables_1 = disposables; _i < disposables_1.length; _i++) {
                        var disposable = disposables_1[_i];
                        if (disposable && typeof disposable.dispose === 'function') {
                            disposable.dispose();
                        }
                    }
                    disposables = undefined;
                }
            });
        };
        Disposable.prototype.dispose = function () {
            if (typeof this._callOnDispose === 'function') {
                this._callOnDispose();
                this._callOnDispose = undefined;
            }
        };
        return Disposable;
    }());
    exports.Disposable = Disposable;
    var Position = (function () {
        function Position(line, character) {
            if (line < 0) {
                throw errors_1.illegalArgument('line must be positive');
            }
            if (character < 0) {
                throw errors_1.illegalArgument('character must be positive');
            }
            this._line = line;
            this._character = character;
        }
        Position.Min = function () {
            var positions = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                positions[_i - 0] = arguments[_i];
            }
            var result = positions.pop();
            for (var _a = 0, positions_1 = positions; _a < positions_1.length; _a++) {
                var p = positions_1[_a];
                if (p.isBefore(result)) {
                    result = p;
                }
            }
            return result;
        };
        Position.Max = function () {
            var positions = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                positions[_i - 0] = arguments[_i];
            }
            var result = positions.pop();
            for (var _a = 0, positions_2 = positions; _a < positions_2.length; _a++) {
                var p = positions_2[_a];
                if (p.isAfter(result)) {
                    result = p;
                }
            }
            return result;
        };
        Object.defineProperty(Position.prototype, "line", {
            get: function () {
                return this._line;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Position.prototype, "character", {
            get: function () {
                return this._character;
            },
            enumerable: true,
            configurable: true
        });
        Position.prototype.isBefore = function (other) {
            if (this._line < other._line) {
                return true;
            }
            if (other._line < this._line) {
                return false;
            }
            return this._character < other._character;
        };
        Position.prototype.isBeforeOrEqual = function (other) {
            if (this._line < other._line) {
                return true;
            }
            if (other._line < this._line) {
                return false;
            }
            return this._character <= other._character;
        };
        Position.prototype.isAfter = function (other) {
            return !this.isBeforeOrEqual(other);
        };
        Position.prototype.isAfterOrEqual = function (other) {
            return !this.isBefore(other);
        };
        Position.prototype.isEqual = function (other) {
            return this._line === other._line && this._character === other._character;
        };
        Position.prototype.compareTo = function (other) {
            if (this._line < other._line) {
                return -1;
            }
            else if (this._line > other.line) {
                return 1;
            }
            else {
                // equal line
                if (this._character < other._character) {
                    return -1;
                }
                else if (this._character > other._character) {
                    return 1;
                }
                else {
                    // equal line and character
                    return 0;
                }
            }
        };
        Position.prototype.translate = function (lineDelta, characterDelta) {
            if (lineDelta === void 0) { lineDelta = 0; }
            if (characterDelta === void 0) { characterDelta = 0; }
            if (lineDelta === 0 && characterDelta === 0) {
                return this;
            }
            return new Position(this.line + lineDelta, this.character + characterDelta);
        };
        Position.prototype.with = function (line, character) {
            if (line === void 0) { line = this.line; }
            if (character === void 0) { character = this.character; }
            if (line === this.line && character === this.character) {
                return this;
            }
            return new Position(line, character);
        };
        Position.prototype.toJSON = function () {
            return { line: this.line, character: this.character };
        };
        return Position;
    }());
    exports.Position = Position;
    var Range = (function () {
        function Range(startLineOrStart, startColumnOrEnd, endLine, endColumn) {
            var start;
            var end;
            if (typeof startLineOrStart === 'number' && typeof startColumnOrEnd === 'number' && typeof endLine === 'number' && typeof endColumn === 'number') {
                start = new Position(startLineOrStart, startColumnOrEnd);
                end = new Position(endLine, endColumn);
            }
            else if (startLineOrStart instanceof Position && startColumnOrEnd instanceof Position) {
                start = startLineOrStart;
                end = startColumnOrEnd;
            }
            if (!start || !end) {
                throw new Error('Invalid arguments');
            }
            if (start.isBefore(end)) {
                this._start = start;
                this._end = end;
            }
            else {
                this._start = end;
                this._end = start;
            }
        }
        Object.defineProperty(Range.prototype, "start", {
            get: function () {
                return this._start;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Range.prototype, "end", {
            get: function () {
                return this._end;
            },
            enumerable: true,
            configurable: true
        });
        Range.prototype.contains = function (positionOrRange) {
            if (positionOrRange instanceof Range) {
                return this.contains(positionOrRange._start)
                    && this.contains(positionOrRange._end);
            }
            else if (positionOrRange instanceof Position) {
                if (positionOrRange.isBefore(this._start)) {
                    return false;
                }
                if (this._end.isBefore(positionOrRange)) {
                    return false;
                }
                return true;
            }
            return false;
        };
        Range.prototype.isEqual = function (other) {
            return this._start.isEqual(other._start) && this._end.isEqual(other._end);
        };
        Range.prototype.intersection = function (other) {
            var start = Position.Max(other.start, this._start);
            var end = Position.Min(other.end, this._end);
            if (start.isAfter(end)) {
                // this happens when there is no overlap:
                // |-----|
                //          |----|
                return;
            }
            return new Range(start, end);
        };
        Range.prototype.union = function (other) {
            if (this.contains(other)) {
                return this;
            }
            else if (other.contains(this)) {
                return other;
            }
            var start = Position.Min(other.start, this._start);
            var end = Position.Max(other.end, this.end);
            return new Range(start, end);
        };
        Object.defineProperty(Range.prototype, "isEmpty", {
            get: function () {
                return this._start.isEqual(this._end);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Range.prototype, "isSingleLine", {
            get: function () {
                return this._start.line === this._end.line;
            },
            enumerable: true,
            configurable: true
        });
        Range.prototype.with = function (start, end) {
            if (start === void 0) { start = this.start; }
            if (end === void 0) { end = this.end; }
            if (start.isEqual(this._start) && end.isEqual(this.end)) {
                return this;
            }
            return new Range(start, end);
        };
        Range.prototype.toJSON = function () {
            return [this.start, this.end];
        };
        return Range;
    }());
    exports.Range = Range;
    var Selection = (function (_super) {
        __extends(Selection, _super);
        function Selection(anchorLineOrAnchor, anchorColumnOrActive, activeLine, activeColumn) {
            var anchor;
            var active;
            if (typeof anchorLineOrAnchor === 'number' && typeof anchorColumnOrActive === 'number' && typeof activeLine === 'number' && typeof activeColumn === 'number') {
                anchor = new Position(anchorLineOrAnchor, anchorColumnOrActive);
                active = new Position(activeLine, activeColumn);
            }
            else if (anchorLineOrAnchor instanceof Position && anchorColumnOrActive instanceof Position) {
                anchor = anchorLineOrAnchor;
                active = anchorColumnOrActive;
            }
            if (!anchor || !active) {
                throw new Error('Invalid arguments');
            }
            _super.call(this, anchor, active);
            this._anchor = anchor;
            this._active = active;
        }
        Object.defineProperty(Selection.prototype, "anchor", {
            get: function () {
                return this._anchor;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Selection.prototype, "active", {
            get: function () {
                return this._active;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Selection.prototype, "isReversed", {
            get: function () {
                return this._anchor === this._end;
            },
            enumerable: true,
            configurable: true
        });
        Selection.prototype.toJSON = function () {
            return {
                start: this.start,
                end: this.end,
                active: this.active,
                anchor: this.anchor
            };
        };
        return Selection;
    }(Range));
    exports.Selection = Selection;
    var TextEdit = (function () {
        function TextEdit(range, newText) {
            this.range = range;
            this.newText = newText;
        }
        TextEdit.replace = function (range, newText) {
            return new TextEdit(range, newText);
        };
        TextEdit.insert = function (position, newText) {
            return TextEdit.replace(new Range(position, position), newText);
        };
        TextEdit.delete = function (range) {
            return TextEdit.replace(range, '');
        };
        Object.defineProperty(TextEdit.prototype, "range", {
            get: function () {
                return this._range;
            },
            set: function (value) {
                if (!value) {
                    throw errors_1.illegalArgument('range');
                }
                this._range = value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TextEdit.prototype, "newText", {
            get: function () {
                return this._newText || '';
            },
            set: function (value) {
                this._newText = value;
            },
            enumerable: true,
            configurable: true
        });
        TextEdit.prototype.toJSON = function () {
            return {
                range: this.range,
                newText: this.newText
            };
        };
        return TextEdit;
    }());
    exports.TextEdit = TextEdit;
    var Uri = (function (_super) {
        __extends(Uri, _super);
        function Uri() {
            _super.apply(this, arguments);
        }
        return Uri;
    }(uri_1.default));
    exports.Uri = Uri;
    var WorkspaceEdit = (function () {
        function WorkspaceEdit() {
            this._values = [];
            this._index = Object.create(null);
        }
        WorkspaceEdit.prototype.replace = function (uri, range, newText) {
            var edit = new TextEdit(range, newText);
            var array = this.get(uri);
            if (array) {
                array.push(edit);
            }
            else {
                this.set(uri, [edit]);
            }
        };
        WorkspaceEdit.prototype.insert = function (resource, position, newText) {
            this.replace(resource, new Range(position, position), newText);
        };
        WorkspaceEdit.prototype.delete = function (resource, range) {
            this.replace(resource, range, '');
        };
        WorkspaceEdit.prototype.has = function (uri) {
            return typeof this._index[uri.toString()] !== 'undefined';
        };
        WorkspaceEdit.prototype.set = function (uri, edits) {
            var idx = this._index[uri.toString()];
            if (typeof idx === 'undefined') {
                var newLen = this._values.push([uri, edits]);
                this._index[uri.toString()] = newLen - 1;
            }
            else {
                this._values[idx][1] = edits;
            }
        };
        WorkspaceEdit.prototype.get = function (uri) {
            var idx = this._index[uri.toString()];
            return typeof idx !== 'undefined' && this._values[idx][1];
        };
        WorkspaceEdit.prototype.entries = function () {
            return this._values;
        };
        Object.defineProperty(WorkspaceEdit.prototype, "size", {
            get: function () {
                return this._values.length;
            },
            enumerable: true,
            configurable: true
        });
        WorkspaceEdit.prototype.toJSON = function () {
            return this._values;
        };
        return WorkspaceEdit;
    }());
    exports.WorkspaceEdit = WorkspaceEdit;
    (function (DiagnosticSeverity) {
        DiagnosticSeverity[DiagnosticSeverity["Hint"] = 3] = "Hint";
        DiagnosticSeverity[DiagnosticSeverity["Information"] = 2] = "Information";
        DiagnosticSeverity[DiagnosticSeverity["Warning"] = 1] = "Warning";
        DiagnosticSeverity[DiagnosticSeverity["Error"] = 0] = "Error";
    })(exports.DiagnosticSeverity || (exports.DiagnosticSeverity = {}));
    var DiagnosticSeverity = exports.DiagnosticSeverity;
    var Location = (function () {
        function Location(uri, range) {
            this.uri = uri;
            if (range instanceof Range) {
                this.range = range;
            }
            else if (range instanceof Position) {
                this.range = new Range(range, range);
            }
            else {
                throw new Error('Illegal argument');
            }
        }
        Location.prototype.toJSON = function () {
            return {
                uri: this.uri,
                range: this.range
            };
        };
        return Location;
    }());
    exports.Location = Location;
    var Diagnostic = (function () {
        function Diagnostic(range, message, severity) {
            if (severity === void 0) { severity = DiagnosticSeverity.Error; }
            this.range = range;
            this.message = message;
            this.severity = severity;
        }
        Diagnostic.prototype.toJSON = function () {
            return {
                severity: DiagnosticSeverity[this.severity],
                message: this.message,
                range: this.range,
                source: this.source,
                code: this.code,
            };
        };
        return Diagnostic;
    }());
    exports.Diagnostic = Diagnostic;
    var Hover = (function () {
        function Hover(contents, range) {
            if (!contents) {
                throw new Error('Illegal argument');
            }
            if (Array.isArray(contents)) {
                this.contents = contents;
            }
            else {
                this.contents = [contents];
            }
            this.range = range;
        }
        return Hover;
    }());
    exports.Hover = Hover;
    (function (DocumentHighlightKind) {
        DocumentHighlightKind[DocumentHighlightKind["Text"] = 0] = "Text";
        DocumentHighlightKind[DocumentHighlightKind["Read"] = 1] = "Read";
        DocumentHighlightKind[DocumentHighlightKind["Write"] = 2] = "Write";
    })(exports.DocumentHighlightKind || (exports.DocumentHighlightKind = {}));
    var DocumentHighlightKind = exports.DocumentHighlightKind;
    var DocumentHighlight = (function () {
        function DocumentHighlight(range, kind) {
            if (kind === void 0) { kind = DocumentHighlightKind.Text; }
            this.range = range;
            this.kind = kind;
        }
        DocumentHighlight.prototype.toJSON = function () {
            return {
                range: this.range,
                kind: DocumentHighlightKind[this.kind]
            };
        };
        return DocumentHighlight;
    }());
    exports.DocumentHighlight = DocumentHighlight;
    (function (SymbolKind) {
        SymbolKind[SymbolKind["File"] = 0] = "File";
        SymbolKind[SymbolKind["Module"] = 1] = "Module";
        SymbolKind[SymbolKind["Namespace"] = 2] = "Namespace";
        SymbolKind[SymbolKind["Package"] = 3] = "Package";
        SymbolKind[SymbolKind["Class"] = 4] = "Class";
        SymbolKind[SymbolKind["Method"] = 5] = "Method";
        SymbolKind[SymbolKind["Property"] = 6] = "Property";
        SymbolKind[SymbolKind["Field"] = 7] = "Field";
        SymbolKind[SymbolKind["Constructor"] = 8] = "Constructor";
        SymbolKind[SymbolKind["Enum"] = 9] = "Enum";
        SymbolKind[SymbolKind["Interface"] = 10] = "Interface";
        SymbolKind[SymbolKind["Function"] = 11] = "Function";
        SymbolKind[SymbolKind["Variable"] = 12] = "Variable";
        SymbolKind[SymbolKind["Constant"] = 13] = "Constant";
        SymbolKind[SymbolKind["String"] = 14] = "String";
        SymbolKind[SymbolKind["Number"] = 15] = "Number";
        SymbolKind[SymbolKind["Boolean"] = 16] = "Boolean";
        SymbolKind[SymbolKind["Array"] = 17] = "Array";
        SymbolKind[SymbolKind["Object"] = 18] = "Object";
        SymbolKind[SymbolKind["Key"] = 19] = "Key";
        SymbolKind[SymbolKind["Null"] = 20] = "Null";
    })(exports.SymbolKind || (exports.SymbolKind = {}));
    var SymbolKind = exports.SymbolKind;
    var SymbolInformation = (function () {
        function SymbolInformation(name, kind, range, uri, containerName) {
            this.name = name;
            this.kind = kind;
            this.location = new Location(uri, range);
            this.containerName = containerName;
        }
        SymbolInformation.prototype.toJSON = function () {
            return {
                name: this.name,
                kind: SymbolKind[this.kind],
                location: this.location,
                containerName: this.containerName
            };
        };
        return SymbolInformation;
    }());
    exports.SymbolInformation = SymbolInformation;
    var CodeLens = (function () {
        function CodeLens(range, command) {
            this.range = range;
            this.command = command;
        }
        Object.defineProperty(CodeLens.prototype, "isResolved", {
            get: function () {
                return !!this.command;
            },
            enumerable: true,
            configurable: true
        });
        return CodeLens;
    }());
    exports.CodeLens = CodeLens;
    var ParameterInformation = (function () {
        function ParameterInformation(label, documentation) {
            this.label = label;
            this.documentation = documentation;
        }
        return ParameterInformation;
    }());
    exports.ParameterInformation = ParameterInformation;
    var SignatureInformation = (function () {
        function SignatureInformation(label, documentation) {
            this.label = label;
            this.documentation = documentation;
            this.parameters = [];
        }
        return SignatureInformation;
    }());
    exports.SignatureInformation = SignatureInformation;
    var SignatureHelp = (function () {
        function SignatureHelp() {
            this.signatures = [];
        }
        return SignatureHelp;
    }());
    exports.SignatureHelp = SignatureHelp;
    (function (CompletionItemKind) {
        CompletionItemKind[CompletionItemKind["Text"] = 0] = "Text";
        CompletionItemKind[CompletionItemKind["Method"] = 1] = "Method";
        CompletionItemKind[CompletionItemKind["Function"] = 2] = "Function";
        CompletionItemKind[CompletionItemKind["Constructor"] = 3] = "Constructor";
        CompletionItemKind[CompletionItemKind["Field"] = 4] = "Field";
        CompletionItemKind[CompletionItemKind["Variable"] = 5] = "Variable";
        CompletionItemKind[CompletionItemKind["Class"] = 6] = "Class";
        CompletionItemKind[CompletionItemKind["Interface"] = 7] = "Interface";
        CompletionItemKind[CompletionItemKind["Module"] = 8] = "Module";
        CompletionItemKind[CompletionItemKind["Property"] = 9] = "Property";
        CompletionItemKind[CompletionItemKind["Unit"] = 10] = "Unit";
        CompletionItemKind[CompletionItemKind["Value"] = 11] = "Value";
        CompletionItemKind[CompletionItemKind["Enum"] = 12] = "Enum";
        CompletionItemKind[CompletionItemKind["Keyword"] = 13] = "Keyword";
        CompletionItemKind[CompletionItemKind["Snippet"] = 14] = "Snippet";
        CompletionItemKind[CompletionItemKind["Color"] = 15] = "Color";
        CompletionItemKind[CompletionItemKind["File"] = 16] = "File";
        CompletionItemKind[CompletionItemKind["Reference"] = 17] = "Reference";
    })(exports.CompletionItemKind || (exports.CompletionItemKind = {}));
    var CompletionItemKind = exports.CompletionItemKind;
    var CompletionItem = (function () {
        function CompletionItem(label) {
            this.label = label;
        }
        CompletionItem.prototype.toJSON = function () {
            return {
                label: this.label,
                kind: CompletionItemKind[this.kind],
                detail: this.detail,
                documentation: this.documentation,
                sortText: this.sortText,
                filterText: this.filterText,
                insertText: this.insertText,
                textEdit: this.textEdit
            };
        };
        return CompletionItem;
    }());
    exports.CompletionItem = CompletionItem;
    var CompletionList = (function () {
        function CompletionList(items, isIncomplete) {
            if (items === void 0) { items = []; }
            if (isIncomplete === void 0) { isIncomplete = false; }
            this.items = items;
            this.isIncomplete = isIncomplete;
        }
        return CompletionList;
    }());
    exports.CompletionList = CompletionList;
    (function (ViewColumn) {
        ViewColumn[ViewColumn["One"] = 1] = "One";
        ViewColumn[ViewColumn["Two"] = 2] = "Two";
        ViewColumn[ViewColumn["Three"] = 3] = "Three";
    })(exports.ViewColumn || (exports.ViewColumn = {}));
    var ViewColumn = exports.ViewColumn;
    (function (StatusBarAlignment) {
        StatusBarAlignment[StatusBarAlignment["Left"] = 1] = "Left";
        StatusBarAlignment[StatusBarAlignment["Right"] = 2] = "Right";
    })(exports.StatusBarAlignment || (exports.StatusBarAlignment = {}));
    var StatusBarAlignment = exports.StatusBarAlignment;
    (function (EndOfLine) {
        EndOfLine[EndOfLine["LF"] = 1] = "LF";
        EndOfLine[EndOfLine["CRLF"] = 2] = "CRLF";
    })(exports.EndOfLine || (exports.EndOfLine = {}));
    var EndOfLine = exports.EndOfLine;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostFileSystemEventService", ["require", "exports", 'vs/platform/files/common/files', 'vs/platform/thread/common/thread', 'vs/base/common/event', './extHostTypes', 'vs/platform/event/common/event', 'vs/base/common/async', 'vs/base/common/glob'], function (require, exports, files_1, thread_1, event_1, extHostTypes_1, event_2, async_1, glob_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FileSystemWatcher = (function () {
        function FileSystemWatcher(dispatcher, globPattern, ignoreCreateEvents, ignoreChangeEvents, ignoreDeleteEvents) {
            var _this = this;
            this._onDidCreate = new event_1.Emitter();
            this._onDidChange = new event_1.Emitter();
            this._onDidDelete = new event_1.Emitter();
            this._config = 0;
            if (!ignoreCreateEvents) {
                this._config += 1;
            }
            if (!ignoreChangeEvents) {
                this._config += 2;
            }
            if (!ignoreDeleteEvents) {
                this._config += 4;
            }
            var subscription = dispatcher(function (events) {
                if (!ignoreCreateEvents) {
                    for (var _i = 0, _a = events.created; _i < _a.length; _i++) {
                        var created = _a[_i];
                        if (glob_1.match(globPattern, created.fsPath)) {
                            _this._onDidCreate.fire(created);
                        }
                    }
                }
                if (!ignoreChangeEvents) {
                    for (var _b = 0, _c = events.changed; _b < _c.length; _b++) {
                        var changed = _c[_b];
                        if (glob_1.match(globPattern, changed.fsPath)) {
                            _this._onDidChange.fire(changed);
                        }
                    }
                }
                if (!ignoreDeleteEvents) {
                    for (var _d = 0, _e = events.deleted; _d < _e.length; _d++) {
                        var deleted = _e[_d];
                        if (glob_1.match(globPattern, deleted.fsPath)) {
                            _this._onDidDelete.fire(deleted);
                        }
                    }
                }
            });
            this._disposable = extHostTypes_1.Disposable.from(this._onDidCreate, this._onDidChange, this._onDidDelete, subscription);
        }
        Object.defineProperty(FileSystemWatcher.prototype, "ignoreCreateEvents", {
            get: function () {
                return Boolean(this._config & 1);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileSystemWatcher.prototype, "ignoreChangeEvents", {
            get: function () {
                return Boolean(this._config & 2);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileSystemWatcher.prototype, "ignoreDeleteEvents", {
            get: function () {
                return Boolean(this._config & 4);
            },
            enumerable: true,
            configurable: true
        });
        FileSystemWatcher.prototype.dispose = function () {
            this._disposable.dispose();
        };
        Object.defineProperty(FileSystemWatcher.prototype, "onDidCreate", {
            get: function () {
                return this._onDidCreate.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileSystemWatcher.prototype, "onDidChange", {
            get: function () {
                return this._onDidChange.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FileSystemWatcher.prototype, "onDidDelete", {
            get: function () {
                return this._onDidDelete.event;
            },
            enumerable: true,
            configurable: true
        });
        return FileSystemWatcher;
    }());
    exports.FileSystemWatcher = FileSystemWatcher;
    var ExtHostFileSystemEventService = (function () {
        function ExtHostFileSystemEventService() {
            this._emitter = new event_1.Emitter();
        }
        ExtHostFileSystemEventService.prototype.createFileSystemWatcher = function (globPattern, ignoreCreateEvents, ignoreChangeEvents, ignoreDeleteEvents) {
            return new FileSystemWatcher(this._emitter.event, globPattern, ignoreCreateEvents, ignoreChangeEvents, ignoreDeleteEvents);
        };
        ExtHostFileSystemEventService.prototype._onFileEvent = function (events) {
            this._emitter.fire(events);
        };
        ExtHostFileSystemEventService = __decorate([
            thread_1.Remotable.ExtHostContext('ExtHostFileSystemEventService')
        ], ExtHostFileSystemEventService);
        return ExtHostFileSystemEventService;
    }());
    exports.ExtHostFileSystemEventService = ExtHostFileSystemEventService;
    var MainThreadFileSystemEventService = (function () {
        function MainThreadFileSystemEventService(eventService, threadService) {
            var proxy = threadService.getRemotable(ExtHostFileSystemEventService);
            var events = {
                created: [],
                changed: [],
                deleted: []
            };
            var scheduler = new async_1.RunOnceScheduler(function () {
                proxy._onFileEvent(events);
                events.created.length = 0;
                events.changed.length = 0;
                events.deleted.length = 0;
            }, 100);
            eventService.addListener('files:fileChanges', function (event) {
                for (var _i = 0, _a = event.changes; _i < _a.length; _i++) {
                    var change = _a[_i];
                    switch (change.type) {
                        case files_1.FileChangeType.ADDED:
                            events.created.push(change.resource);
                            break;
                        case files_1.FileChangeType.UPDATED:
                            events.changed.push(change.resource);
                            break;
                        case files_1.FileChangeType.DELETED:
                            events.deleted.push(change.resource);
                            break;
                    }
                }
                scheduler.schedule();
            });
        }
        MainThreadFileSystemEventService = __decorate([
            __param(0, event_2.IEventService),
            __param(1, thread_1.IThreadService)
        ], MainThreadFileSystemEventService);
        return MainThreadFileSystemEventService;
    }());
    exports.MainThreadFileSystemEventService = MainThreadFileSystemEventService;
});

define("vs/workbench/api/node/extHostTypeConverters", ["require", "exports", 'vs/base/common/severity', 'vs/base/common/arrays', './extHostTypes', 'vs/platform/editor/common/editor'], function (require, exports, severity_1, arrays_1, types, editor_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function toSelection(selection) {
        var selectionStartLineNumber = selection.selectionStartLineNumber, selectionStartColumn = selection.selectionStartColumn, positionLineNumber = selection.positionLineNumber, positionColumn = selection.positionColumn;
        var start = new types.Position(selectionStartLineNumber - 1, selectionStartColumn - 1);
        var end = new types.Position(positionLineNumber - 1, positionColumn - 1);
        return new types.Selection(start, end);
    }
    exports.toSelection = toSelection;
    function fromSelection(selection) {
        var anchor = selection.anchor, active = selection.active;
        return {
            selectionStartLineNumber: anchor.line + 1,
            selectionStartColumn: anchor.character + 1,
            positionLineNumber: active.line + 1,
            positionColumn: active.character + 1
        };
    }
    exports.fromSelection = fromSelection;
    function fromRange(range) {
        var start = range.start, end = range.end;
        return {
            startLineNumber: start.line + 1,
            startColumn: start.character + 1,
            endLineNumber: end.line + 1,
            endColumn: end.character + 1
        };
    }
    exports.fromRange = fromRange;
    function toRange(range) {
        var startLineNumber = range.startLineNumber, startColumn = range.startColumn, endLineNumber = range.endLineNumber, endColumn = range.endColumn;
        return new types.Range(startLineNumber - 1, startColumn - 1, endLineNumber - 1, endColumn - 1);
    }
    exports.toRange = toRange;
    function toPosition(position) {
        return new types.Position(position.lineNumber - 1, position.column - 1);
    }
    exports.toPosition = toPosition;
    function fromPosition(position) {
        return { lineNumber: position.line + 1, column: position.character + 1 };
    }
    exports.fromPosition = fromPosition;
    function fromDiagnosticSeverity(value) {
        switch (value) {
            case types.DiagnosticSeverity.Error:
                return severity_1.default.Error;
            case types.DiagnosticSeverity.Warning:
                return severity_1.default.Warning;
            case types.DiagnosticSeverity.Information:
                return severity_1.default.Info;
            case types.DiagnosticSeverity.Hint:
                return severity_1.default.Ignore;
        }
        return severity_1.default.Error;
    }
    exports.fromDiagnosticSeverity = fromDiagnosticSeverity;
    function toDiagnosticSeverty(value) {
        switch (value) {
            case severity_1.default.Info:
                return types.DiagnosticSeverity.Information;
            case severity_1.default.Warning:
                return types.DiagnosticSeverity.Warning;
            case severity_1.default.Error:
                return types.DiagnosticSeverity.Error;
            case severity_1.default.Ignore:
                return types.DiagnosticSeverity.Hint;
        }
        return types.DiagnosticSeverity.Error;
    }
    exports.toDiagnosticSeverty = toDiagnosticSeverty;
    function fromViewColumn(column) {
        var editorColumn = editor_1.Position.LEFT;
        if (typeof column !== 'number') {
        }
        else if (column === types.ViewColumn.Two) {
            editorColumn = editor_1.Position.CENTER;
        }
        else if (column === types.ViewColumn.Three) {
            editorColumn = editor_1.Position.RIGHT;
        }
        return editorColumn;
    }
    exports.fromViewColumn = fromViewColumn;
    function toViewColumn(position) {
        if (typeof position !== 'number') {
            return;
        }
        if (position === editor_1.Position.LEFT) {
            return types.ViewColumn.One;
        }
        else if (position === editor_1.Position.CENTER) {
            return types.ViewColumn.Two;
        }
        else if (position === editor_1.Position.RIGHT) {
            return types.ViewColumn.Three;
        }
    }
    exports.toViewColumn = toViewColumn;
    function fromFormattedString(value) {
        if (typeof value === 'string') {
            return { markdown: value };
        }
        else if (typeof value === 'object') {
            return { code: value };
        }
    }
    exports.fromFormattedString = fromFormattedString;
    function toFormattedString(value) {
        if (typeof value.code === 'string') {
            return value.code;
        }
        var markdown = value.markdown, text = value.text;
        return markdown || text || '<???>';
    }
    exports.toFormattedString = toFormattedString;
    function isMarkedStringArr(something) {
        return Array.isArray(something);
    }
    function fromMarkedStringOrMarkedStringArr(something) {
        if (isMarkedStringArr(something)) {
            return something.map(function (msg) { return fromFormattedString(msg); });
        }
        else if (something) {
            return [fromFormattedString(something)];
        }
        else {
            return [];
        }
    }
    function isRangeWithMessage(something) {
        return (typeof something.range !== 'undefined');
    }
    function isRangeWithMessageArr(something) {
        if (something.length === 0) {
            return true;
        }
        return isRangeWithMessage(something[0]) ? true : false;
    }
    function fromRangeOrRangeWithMessage(ranges) {
        if (isRangeWithMessageArr(ranges)) {
            return ranges.map(function (r) {
                return {
                    range: fromRange(r.range),
                    hoverMessage: fromMarkedStringOrMarkedStringArr(r.hoverMessage)
                };
            });
        }
        else {
            return ranges.map(function (r) {
                return {
                    range: fromRange(r)
                };
            });
        }
    }
    exports.fromRangeOrRangeWithMessage = fromRangeOrRangeWithMessage;
    exports.TextEdit = {
        from: function (edit) {
            return {
                text: edit.newText,
                range: fromRange(edit.range)
            };
        },
        to: function (edit) {
            return new types.TextEdit(toRange(edit.range), edit.text);
        }
    };
    var SymbolKind;
    (function (SymbolKind) {
        function from(kind) {
            switch (kind) {
                case types.SymbolKind.Method:
                    return 'method';
                case types.SymbolKind.Function:
                    return 'function';
                case types.SymbolKind.Constructor:
                    return 'constructor';
                case types.SymbolKind.Variable:
                    return 'variable';
                case types.SymbolKind.Class:
                    return 'class';
                case types.SymbolKind.Interface:
                    return 'interface';
                case types.SymbolKind.Namespace:
                    return 'namespace';
                case types.SymbolKind.Package:
                    return 'package';
                case types.SymbolKind.Module:
                    return 'module';
                case types.SymbolKind.Property:
                    return 'property';
                case types.SymbolKind.Enum:
                    return 'enum';
                case types.SymbolKind.String:
                    return 'string';
                case types.SymbolKind.File:
                    return 'file';
                case types.SymbolKind.Array:
                    return 'array';
                case types.SymbolKind.Number:
                    return 'number';
                case types.SymbolKind.Boolean:
                    return 'boolean';
                case types.SymbolKind.Object:
                    return 'object';
                case types.SymbolKind.Key:
                    return 'key';
                case types.SymbolKind.Null:
                    return 'null';
            }
            return 'property';
        }
        SymbolKind.from = from;
        function to(type) {
            switch (type) {
                case 'method':
                    return types.SymbolKind.Method;
                case 'function':
                    return types.SymbolKind.Function;
                case 'constructor':
                    return types.SymbolKind.Constructor;
                case 'variable':
                    return types.SymbolKind.Variable;
                case 'class':
                    return types.SymbolKind.Class;
                case 'interface':
                    return types.SymbolKind.Interface;
                case 'namespace':
                    return types.SymbolKind.Namespace;
                case 'package':
                    return types.SymbolKind.Package;
                case 'module':
                    return types.SymbolKind.Module;
                case 'property':
                    return types.SymbolKind.Property;
                case 'enum':
                    return types.SymbolKind.Enum;
                case 'string':
                    return types.SymbolKind.String;
                case 'file':
                    return types.SymbolKind.File;
                case 'array':
                    return types.SymbolKind.Array;
                case 'number':
                    return types.SymbolKind.Number;
                case 'boolean':
                    return types.SymbolKind.Boolean;
                case 'object':
                    return types.SymbolKind.Object;
                case 'key':
                    return types.SymbolKind.Key;
                case 'null':
                    return types.SymbolKind.Null;
            }
            return types.SymbolKind.Property;
        }
        SymbolKind.to = to;
    })(SymbolKind = exports.SymbolKind || (exports.SymbolKind = {}));
    var SymbolInformation;
    (function (SymbolInformation) {
        function fromOutlineEntry(entry) {
            return new types.SymbolInformation(entry.label, SymbolKind.to(entry.type), toRange(entry.range), undefined, entry.containerLabel);
        }
        SymbolInformation.fromOutlineEntry = fromOutlineEntry;
        function toOutlineEntry(symbol) {
            return {
                type: SymbolKind.from(symbol.kind),
                range: fromRange(symbol.location.range),
                containerLabel: symbol.containerName,
                label: symbol.name,
                icon: undefined,
            };
        }
        SymbolInformation.toOutlineEntry = toOutlineEntry;
    })(SymbolInformation = exports.SymbolInformation || (exports.SymbolInformation = {}));
    function fromSymbolInformation(info) {
        return {
            name: info.name,
            type: types.SymbolKind[info.kind || types.SymbolKind.Property].toLowerCase(),
            range: fromRange(info.location.range),
            resourceUri: info.location.uri,
            containerName: info.containerName,
            parameters: '',
        };
    }
    exports.fromSymbolInformation = fromSymbolInformation;
    function toSymbolInformation(bearing) {
        return new types.SymbolInformation(bearing.name, types.SymbolKind[bearing.type.charAt(0).toUpperCase() + bearing.type.substr(1)], toRange(bearing.range), bearing.resourceUri, bearing.containerName);
    }
    exports.toSymbolInformation = toSymbolInformation;
    exports.location = {
        from: function (value) {
            return {
                range: fromRange(value.range),
                resource: value.uri
            };
        },
        to: function (value) {
            return new types.Location(value.resource, toRange(value.range));
        }
    };
    function fromHover(hover) {
        return {
            range: fromRange(hover.range),
            htmlContent: hover.contents.map(fromFormattedString)
        };
    }
    exports.fromHover = fromHover;
    function toHover(info) {
        return new types.Hover(info.htmlContent.map(toFormattedString), toRange(info.range));
    }
    exports.toHover = toHover;
    function toDocumentHighlight(occurrence) {
        return new types.DocumentHighlight(toRange(occurrence.range), types.DocumentHighlightKind[occurrence.kind.charAt(0).toUpperCase() + occurrence.kind.substr(1)]);
    }
    exports.toDocumentHighlight = toDocumentHighlight;
    exports.Suggest = {
        from: function (item) {
            var suggestion = {
                label: item.label,
                codeSnippet: item.insertText || item.label,
                type: types.CompletionItemKind[item.kind || types.CompletionItemKind.Text].toString().toLowerCase(),
                typeLabel: item.detail,
                documentationLabel: item.documentation,
                sortText: item.sortText,
                filterText: item.filterText
            };
            return suggestion;
        },
        to: function (container, position, suggestion) {
            var result = new types.CompletionItem(suggestion.label);
            result.insertText = suggestion.codeSnippet;
            result.kind = types.CompletionItemKind[suggestion.type.charAt(0).toUpperCase() + suggestion.type.substr(1)];
            result.detail = suggestion.typeLabel;
            result.documentation = suggestion.documentationLabel;
            result.sortText = suggestion.sortText;
            result.filterText = suggestion.filterText;
            var overwriteBefore = (typeof suggestion.overwriteBefore === 'number') ? suggestion.overwriteBefore : container.currentWord.length;
            var startPosition = new types.Position(position.line, Math.max(0, position.character - overwriteBefore));
            var endPosition = position;
            if (typeof suggestion.overwriteAfter === 'number') {
                endPosition = new types.Position(position.line, position.character + suggestion.overwriteAfter);
            }
            result.textEdit = types.TextEdit.replace(new types.Range(startPosition, endPosition), suggestion.codeSnippet);
            return result;
        }
    };
    var SignatureHelp;
    (function (SignatureHelp) {
        function from(signatureHelp) {
            var result = {
                currentSignature: signatureHelp.activeSignature,
                currentParameter: signatureHelp.activeParameter,
                signatures: []
            };
            for (var _i = 0, _a = signatureHelp.signatures; _i < _a.length; _i++) {
                var signature = _a[_i];
                var signatureItem = {
                    label: signature.label,
                    documentation: signature.documentation,
                    parameters: []
                };
                var idx = 0;
                for (var _b = 0, _c = signature.parameters; _b < _c.length; _b++) {
                    var parameter = _c[_b];
                    var parameterItem = {
                        label: parameter.label,
                        documentation: parameter.documentation,
                    };
                    signatureItem.parameters.push(parameterItem);
                    idx = signature.label.indexOf(parameter.label, idx);
                    if (idx >= 0) {
                        parameterItem.signatureLabelOffset = idx;
                        idx += parameter.label.length;
                        parameterItem.signatureLabelEnd = idx;
                    }
                    else {
                        parameterItem.signatureLabelOffset = 0;
                        parameterItem.signatureLabelEnd = 0;
                    }
                }
                result.signatures.push(signatureItem);
            }
            return result;
        }
        SignatureHelp.from = from;
        function to(hints) {
            var result = new types.SignatureHelp();
            result.activeSignature = hints.currentSignature;
            result.activeParameter = hints.currentParameter;
            for (var _i = 0, _a = hints.signatures; _i < _a.length; _i++) {
                var signature = _a[_i];
                var signatureItem = new types.SignatureInformation(signature.label, signature.documentation);
                result.signatures.push(signatureItem);
                for (var _b = 0, _c = signature.parameters; _b < _c.length; _b++) {
                    var parameter = _c[_b];
                    var parameterItem = new types.ParameterInformation(parameter.label, parameter.documentation);
                    signatureItem.parameters.push(parameterItem);
                }
            }
            return result;
        }
        SignatureHelp.to = to;
    })(SignatureHelp = exports.SignatureHelp || (exports.SignatureHelp = {}));
    var Command;
    (function (Command) {
        var _cache = Object.create(null);
        var _idPool = 1;
        function from(command, context) {
            if (!command) {
                return;
            }
            var result = {
                id: command.command,
                title: command.title
            };
            if (!arrays_1.isFalsyOrEmpty(command.arguments)) {
                // keep command around
                var id_1 = command.command + "-no-args-wrapper-" + _idPool++;
                result.id = id_1;
                _cache[id_1] = command;
                var disposable1 = context.commands.registerCommand(id_1, function () { return (_a = context.commands).executeCommand.apply(_a, [command.command].concat(_cache[id_1].arguments)); var _a; });
                var disposable2 = { dispose: function () { delete _cache[id_1]; } };
                context.disposables.push(disposable1, disposable2);
            }
            return result;
        }
        Command.from = from;
        function to(command) {
            var result = _cache[command.id];
            if (!result) {
                result = {
                    command: command.id,
                    title: command.title
                };
            }
            return result;
        }
        Command.to = to;
    })(Command = exports.Command || (exports.Command = {}));
});

define("vs/workbench/api/node/mainThreadEditors", ["require", "exports", 'vs/editor/common/editorCommon', 'vs/base/common/event', 'vs/base/common/lifecycle', 'vs/base/common/async', 'vs/editor/common/core/range', 'vs/editor/common/core/selection', 'vs/workbench/api/node/extHostTypes'], function (require, exports, EditorCommon, event_1, lifecycle_1, async_1, range_1, selection_1, extHostTypes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function configurationsEqual(a, b) {
        if (a && !b || !a && b) {
            return false;
        }
        if (!a && !b) {
            return true;
        }
        return (a.tabSize === b.tabSize
            && a.insertSpaces === b.insertSpaces);
    }
    (function (TextEditorRevealType) {
        TextEditorRevealType[TextEditorRevealType["Default"] = 0] = "Default";
        TextEditorRevealType[TextEditorRevealType["InCenter"] = 1] = "InCenter";
        TextEditorRevealType[TextEditorRevealType["InCenterIfOutsideViewport"] = 2] = "InCenterIfOutsideViewport";
    })(exports.TextEditorRevealType || (exports.TextEditorRevealType = {}));
    var TextEditorRevealType = exports.TextEditorRevealType;
    /**
     * Text Editor that is permanently bound to the same model.
     * It can be bound or not to a CodeEditor.
     */
    var MainThreadTextEditor = (function () {
        function MainThreadTextEditor(id, model, codeEditor, focusTracker, modelService) {
            var _this = this;
            this._id = id;
            this._model = model;
            this._codeEditor = null;
            this._focusTracker = focusTracker;
            this._modelService = modelService;
            this._codeEditorListeners = [];
            this._onSelectionChanged = new event_1.Emitter();
            this._onConfigurationChanged = new event_1.Emitter();
            this._lastSelection = [new selection_1.Selection(1, 1, 1, 1)];
            this._setConfiguration(this._readConfiguration(this._model, this._codeEditor));
            this._modelListeners = [];
            this._modelListeners.push(this._model.addListener2(EditorCommon.EventType.ModelOptionsChanged, function (e) {
                _this._setConfiguration(_this._readConfiguration(_this._model, _this._codeEditor));
            }));
            this.setCodeEditor(codeEditor);
        }
        MainThreadTextEditor.prototype.dispose = function () {
            this._model = null;
            this._modelListeners = lifecycle_1.disposeAll(this._modelListeners);
            this._codeEditor = null;
            this._codeEditorListeners = lifecycle_1.disposeAll(this._codeEditorListeners);
        };
        MainThreadTextEditor.prototype.getId = function () {
            return this._id;
        };
        MainThreadTextEditor.prototype.getModel = function () {
            return this._model;
        };
        MainThreadTextEditor.prototype.hasCodeEditor = function (codeEditor) {
            return (this._codeEditor === codeEditor);
        };
        MainThreadTextEditor.prototype.setCodeEditor = function (codeEditor) {
            var _this = this;
            if (this.hasCodeEditor(codeEditor)) {
                // Nothing to do...
                return;
            }
            this._codeEditorListeners = lifecycle_1.disposeAll(this._codeEditorListeners);
            this._codeEditor = codeEditor;
            if (this._codeEditor) {
                var forwardSelection = function () {
                    _this._lastSelection = _this._codeEditor.getSelections();
                    _this._onSelectionChanged.fire(_this._lastSelection);
                };
                this._codeEditorListeners.push(this._codeEditor.addListener2(EditorCommon.EventType.CursorSelectionChanged, forwardSelection));
                if (!selection_1.Selection.selectionsArrEqual(this._lastSelection, this._codeEditor.getSelections())) {
                    forwardSelection();
                }
                this._codeEditorListeners.push(this._codeEditor.addListener2(EditorCommon.EventType.EditorFocus, function () {
                    _this._focusTracker.onGainedFocus();
                }));
                this._codeEditorListeners.push(this._codeEditor.addListener2(EditorCommon.EventType.EditorBlur, function () {
                    _this._focusTracker.onLostFocus();
                }));
                this._codeEditorListeners.push(this._codeEditor.addListener2(EditorCommon.EventType.ConfigurationChanged, function () {
                    _this._setConfiguration(_this._readConfiguration(_this._model, _this._codeEditor));
                }));
                this._setConfiguration(this._readConfiguration(this._model, this._codeEditor));
            }
        };
        MainThreadTextEditor.prototype.isVisible = function () {
            return !!this._codeEditor;
        };
        Object.defineProperty(MainThreadTextEditor.prototype, "onSelectionChanged", {
            get: function () {
                return this._onSelectionChanged.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainThreadTextEditor.prototype, "onConfigurationChanged", {
            get: function () {
                return this._onConfigurationChanged.event;
            },
            enumerable: true,
            configurable: true
        });
        MainThreadTextEditor.prototype.getSelections = function () {
            if (this._codeEditor) {
                return this._codeEditor.getSelections();
            }
            return this._lastSelection;
        };
        MainThreadTextEditor.prototype.setSelections = function (selections) {
            if (this._codeEditor) {
                this._codeEditor.setSelections(selections);
                return;
            }
            this._lastSelection = selections.map(selection_1.Selection.liftSelection);
            console.warn('setSelections on invisble editor');
        };
        MainThreadTextEditor.prototype.getConfiguration = function () {
            return this._configuration;
        };
        MainThreadTextEditor.prototype._setIndentConfiguration = function (newConfiguration) {
            if (newConfiguration.tabSize === 'auto' || newConfiguration.insertSpaces === 'auto') {
                // one of the options was set to 'auto' => detect indentation
                var creationOpts = this._modelService.getCreationOptions();
                var insertSpaces = creationOpts.insertSpaces;
                var tabSize = creationOpts.tabSize;
                if (newConfiguration.insertSpaces !== 'auto') {
                    if (typeof newConfiguration.insertSpaces !== 'undefined') {
                        insertSpaces = (newConfiguration.insertSpaces === 'false' ? false : Boolean(newConfiguration.insertSpaces));
                    }
                }
                if (newConfiguration.tabSize !== 'auto') {
                    if (typeof newConfiguration.tabSize !== 'undefined') {
                        var parsedTabSize = parseInt(newConfiguration.tabSize, 10);
                        if (!isNaN(parsedTabSize)) {
                            tabSize = parsedTabSize;
                        }
                    }
                }
                this._model.detectIndentation(insertSpaces, tabSize);
                return;
            }
            var newOpts = {};
            if (typeof newConfiguration.insertSpaces !== 'undefined') {
                newOpts.insertSpaces = (newConfiguration.insertSpaces === 'false' ? false : Boolean(newConfiguration.insertSpaces));
            }
            if (typeof newConfiguration.tabSize !== 'undefined') {
                var parsedTabSize = parseInt(newConfiguration.tabSize, 10);
                if (!isNaN(parsedTabSize)) {
                    newOpts.tabSize = parsedTabSize;
                }
            }
            this._model.updateOptions(newOpts);
        };
        MainThreadTextEditor.prototype.setConfiguration = function (newConfiguration) {
            this._setIndentConfiguration(newConfiguration);
            if (newConfiguration.cursorStyle) {
                var newCursorStyle = EditorCommon.cursorStyleToString(newConfiguration.cursorStyle);
                if (!this._codeEditor) {
                    console.warn('setConfiguration on invisible editor');
                    return;
                }
                this._codeEditor.updateOptions({
                    cursorStyle: newCursorStyle
                });
            }
        };
        MainThreadTextEditor.prototype.setDecorations = function (key, ranges) {
            if (!this._codeEditor) {
                console.warn('setDecorations on invisible editor');
                return;
            }
            this._codeEditor.setDecorations(key, ranges);
        };
        MainThreadTextEditor.prototype.revealRange = function (range, revealType) {
            if (!this._codeEditor) {
                console.warn('revealRange on invisible editor');
                return;
            }
            if (revealType === TextEditorRevealType.Default) {
                this._codeEditor.revealRange(range);
            }
            else if (revealType === TextEditorRevealType.InCenter) {
                this._codeEditor.revealRangeInCenter(range);
            }
            else if (revealType === TextEditorRevealType.InCenterIfOutsideViewport) {
                this._codeEditor.revealRangeInCenterIfOutsideViewport(range);
            }
            else {
                console.warn('Unknown revealType');
            }
        };
        MainThreadTextEditor.prototype._readConfiguration = function (model, codeEditor) {
            if (model.isDisposed()) {
                // shutdown time
                return this._configuration;
            }
            var cursorStyle = this._configuration ? this._configuration.cursorStyle : EditorCommon.TextEditorCursorStyle.Line;
            if (codeEditor) {
                var codeEditorOpts = codeEditor.getConfiguration();
                cursorStyle = codeEditorOpts.cursorStyle;
            }
            var indent = model.getOptions();
            return {
                insertSpaces: indent.insertSpaces,
                tabSize: indent.tabSize,
                cursorStyle: cursorStyle
            };
        };
        MainThreadTextEditor.prototype._setConfiguration = function (newConfiguration) {
            if (configurationsEqual(this._configuration, newConfiguration)) {
                return;
            }
            this._configuration = newConfiguration;
            this._onConfigurationChanged.fire(this._configuration);
        };
        MainThreadTextEditor.prototype.isFocused = function () {
            if (this._codeEditor) {
                return this._codeEditor.isFocused();
            }
            return false;
        };
        MainThreadTextEditor.prototype.matches = function (editor) {
            return editor.getControl() === this._codeEditor;
        };
        MainThreadTextEditor.prototype.applyEdits = function (versionIdCheck, edits, setEndOfLine) {
            if (this._model.getVersionId() !== versionIdCheck) {
                console.warn('Model has changed in the meantime!');
                // throw new Error('Model has changed in the meantime!');
                // model changed in the meantime
                return false;
            }
            if (this._codeEditor) {
                if (setEndOfLine === extHostTypes_1.EndOfLine.CRLF) {
                    this._model.setEOL(EditorCommon.EndOfLineSequence.CRLF);
                }
                else if (setEndOfLine === extHostTypes_1.EndOfLine.LF) {
                    this._model.setEOL(EditorCommon.EndOfLineSequence.LF);
                }
                var transformedEdits = edits.map(function (edit) {
                    return {
                        identifier: null,
                        range: range_1.Range.lift(edit.range),
                        text: edit.text,
                        forceMoveMarkers: edit.forceMoveMarkers
                    };
                });
                return this._codeEditor.executeEdits('MainThreadTextEditor', transformedEdits) || true;
            }
            console.warn('applyEdits on invisible editor');
            return false;
        };
        return MainThreadTextEditor;
    }());
    exports.MainThreadTextEditor = MainThreadTextEditor;
    /**
     * Keeps track of what goes on in the main thread and maps models => text editors
     */
    var MainThreadEditorsTracker = (function () {
        function MainThreadEditorsTracker(editorService, modelService) {
            var _this = this;
            this._codeEditorService = editorService;
            this._modelService = modelService;
            this._toDispose = [];
            this._focusedTextEditorId = null;
            this._visibleTextEditorIds = [];
            this._editorModelChangeListeners = Object.create(null);
            this._model2TextEditors = Object.create(null);
            this._onTextEditorAdd = new event_1.Emitter();
            this._onTextEditorRemove = new event_1.Emitter();
            this._onDidUpdateTextEditors = new event_1.Emitter();
            this._onDidChangeFocusedTextEditor = new event_1.Emitter();
            this._focusTracker = {
                onGainedFocus: function () { return _this._updateFocusedTextEditor(); },
                onLostFocus: function () { return _this._updateFocusedTextEditor(); }
            };
            this._modelService.onModelAdded(this._onModelAdded, this, this._toDispose);
            this._modelService.onModelRemoved(this._onModelRemoved, this, this._toDispose);
            this._codeEditorService.onCodeEditorAdd(this._onCodeEditorAdd, this, this._toDispose);
            this._codeEditorService.onCodeEditorRemove(this._onCodeEditorRemove, this, this._toDispose);
            this._updateMapping = new async_1.RunOnceScheduler(function () { return _this._doUpdateMapping(); }, 0);
            this._toDispose.push(this._updateMapping);
        }
        MainThreadEditorsTracker.prototype._nextId = function () {
            return String(++MainThreadEditorsTracker._LAST_TEXT_EDITOR_ID);
        };
        MainThreadEditorsTracker.prototype.dispose = function () {
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        MainThreadEditorsTracker.prototype._onModelAdded = function (model) {
            this._updateMapping.schedule();
        };
        MainThreadEditorsTracker.prototype._onModelRemoved = function (model) {
            this._updateMapping.schedule();
        };
        MainThreadEditorsTracker.prototype._onCodeEditorAdd = function (codeEditor) {
            var _this = this;
            this._editorModelChangeListeners[codeEditor.getId()] = codeEditor.addListener2(EditorCommon.EventType.ModelChanged, function (_) { return _this._updateMapping.schedule(); });
            this._updateMapping.schedule();
        };
        MainThreadEditorsTracker.prototype._onCodeEditorRemove = function (codeEditor) {
            this._editorModelChangeListeners[codeEditor.getId()].dispose();
            delete this._editorModelChangeListeners[codeEditor.getId()];
            this._updateMapping.schedule();
        };
        MainThreadEditorsTracker.prototype._doUpdateMapping = function () {
            var _this = this;
            var allModels = this._modelService.getModels();
            // Same filter as in extHostDocuments
            allModels = allModels.filter(function (model) { return !model.isTooLargeForHavingARichMode(); });
            var allModelsMap = Object.create(null);
            allModels.forEach(function (model) {
                allModelsMap[model.getAssociatedResource().toString()] = model;
            });
            // Remove text editors for models that no longer exist
            Object.keys(this._model2TextEditors).forEach(function (modelUri) {
                if (allModelsMap[modelUri]) {
                    // model still exists, will be updated below
                    return;
                }
                var textEditorsToRemove = _this._model2TextEditors[modelUri];
                delete _this._model2TextEditors[modelUri];
                for (var i = 0; i < textEditorsToRemove.length; i++) {
                    _this._onTextEditorRemove.fire(textEditorsToRemove[i]);
                    textEditorsToRemove[i].dispose();
                }
            });
            // Handle all visible models
            var visibleModels = this._getVisibleModels();
            Object.keys(visibleModels).forEach(function (modelUri) {
                var model = visibleModels[modelUri].model;
                var codeEditors = visibleModels[modelUri].codeEditors;
                if (!_this._model2TextEditors[modelUri]) {
                    _this._model2TextEditors[modelUri] = [];
                }
                var existingTextEditors = _this._model2TextEditors[modelUri];
                // Remove text editors if more exist
                while (existingTextEditors.length > codeEditors.length) {
                    var removedTextEditor = existingTextEditors.pop();
                    _this._onTextEditorRemove.fire(removedTextEditor);
                    removedTextEditor.dispose();
                }
                // Adjust remaining text editors
                for (var i = 0; i < existingTextEditors.length; i++) {
                    existingTextEditors[i].setCodeEditor(codeEditors[i]);
                }
                // Create new editors as needed
                for (var i = existingTextEditors.length; i < codeEditors.length; i++) {
                    var newTextEditor = new MainThreadTextEditor(_this._nextId(), model, codeEditors[i], _this._focusTracker, _this._modelService);
                    existingTextEditors.push(newTextEditor);
                    _this._onTextEditorAdd.fire(newTextEditor);
                }
            });
            // Handle all not visible models
            allModels.forEach(function (model) {
                var modelUri = model.getAssociatedResource().toString();
                if (visibleModels[modelUri]) {
                    // model is visible, already handled above
                    return;
                }
                if (!_this._model2TextEditors[modelUri]) {
                    _this._model2TextEditors[modelUri] = [];
                }
                var existingTextEditors = _this._model2TextEditors[modelUri];
                // Remove extra text editors
                while (existingTextEditors.length > 1) {
                    var removedTextEditor = existingTextEditors.pop();
                    _this._onTextEditorRemove.fire(removedTextEditor);
                    removedTextEditor.dispose();
                }
                // Create new editor if needed or adjust it
                if (existingTextEditors.length === 0) {
                    var newTextEditor = new MainThreadTextEditor(_this._nextId(), model, null, _this._focusTracker, _this._modelService);
                    existingTextEditors.push(newTextEditor);
                    _this._onTextEditorAdd.fire(newTextEditor);
                }
                else {
                    existingTextEditors[0].setCodeEditor(null);
                }
            });
            this._printState();
            this._visibleTextEditorIds = this._findVisibleTextEditorIds();
            this._updateFocusedTextEditor();
            // this is a sync event
            this._onDidUpdateTextEditors.fire(undefined);
        };
        MainThreadEditorsTracker.prototype._updateFocusedTextEditor = function () {
            this._setFocusedTextEditorId(this._findFocusedTextEditorId());
        };
        MainThreadEditorsTracker.prototype._findFocusedTextEditorId = function () {
            var modelUris = Object.keys(this._model2TextEditors);
            for (var i = 0, len = modelUris.length; i < len; i++) {
                var editors = this._model2TextEditors[modelUris[i]];
                for (var j = 0, lenJ = editors.length; j < lenJ; j++) {
                    if (editors[j].isFocused()) {
                        return editors[j].getId();
                    }
                }
            }
            return null;
        };
        MainThreadEditorsTracker.prototype._findVisibleTextEditorIds = function () {
            var result = [];
            var modelUris = Object.keys(this._model2TextEditors);
            for (var i = 0, len = modelUris.length; i < len; i++) {
                var editors = this._model2TextEditors[modelUris[i]];
                for (var j = 0, lenJ = editors.length; j < lenJ; j++) {
                    if (editors[j].isVisible()) {
                        result.push(editors[j].getId());
                    }
                }
            }
            result.sort();
            return result;
        };
        MainThreadEditorsTracker.prototype._setFocusedTextEditorId = function (focusedTextEditorId) {
            if (this._focusedTextEditorId === focusedTextEditorId) {
                // no change
                return;
            }
            this._focusedTextEditorId = focusedTextEditorId;
            this._printState();
            this._onDidChangeFocusedTextEditor.fire(this._focusedTextEditorId);
        };
        MainThreadEditorsTracker.prototype._printState = function () {
            // console.log('----------------------');
            // Object.keys(this._model2TextEditors).forEach((modelUri) => {
            // 	let editors = this._model2TextEditors[modelUri];
            // 	console.log(editors.map((e) => {
            // 		return e.getId() + " (" + (e.getId() === this._focusedTextEditorId ? 'FOCUSED, ': '') + modelUri + ")";
            // 	}).join('\n'));
            // });
        };
        MainThreadEditorsTracker.prototype._getVisibleModels = function () {
            var r = {};
            var allCodeEditors = this._codeEditorService.listCodeEditors();
            // Maintain a certain sorting such that the mapping doesn't change too much all the time
            allCodeEditors.sort(function (a, b) { return strcmp(a.getId(), b.getId()); });
            allCodeEditors.forEach(function (codeEditor) {
                var model = codeEditor.getModel();
                if (!model || model.isTooLargeForHavingARichMode()) {
                    return;
                }
                var modelUri = model.getAssociatedResource().toString();
                r[modelUri] = r[modelUri] || {
                    model: model,
                    codeEditors: []
                };
                r[modelUri].codeEditors.push(codeEditor);
            });
            return r;
        };
        MainThreadEditorsTracker.prototype.getFocusedTextEditorId = function () {
            return this._focusedTextEditorId;
        };
        MainThreadEditorsTracker.prototype.getVisibleTextEditorIds = function () {
            return this._visibleTextEditorIds;
        };
        Object.defineProperty(MainThreadEditorsTracker.prototype, "onTextEditorAdd", {
            get: function () {
                return this._onTextEditorAdd.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainThreadEditorsTracker.prototype, "onTextEditorRemove", {
            get: function () {
                return this._onTextEditorRemove.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainThreadEditorsTracker.prototype, "onDidUpdateTextEditors", {
            get: function () {
                return this._onDidUpdateTextEditors.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MainThreadEditorsTracker.prototype, "onChangedFocusedTextEditor", {
            get: function () {
                return this._onDidChangeFocusedTextEditor.event;
            },
            enumerable: true,
            configurable: true
        });
        MainThreadEditorsTracker.prototype.findTextEditorIdFor = function (codeEditor) {
            var modelUris = Object.keys(this._model2TextEditors);
            for (var i = 0, len = modelUris.length; i < len; i++) {
                var editors = this._model2TextEditors[modelUris[i]];
                for (var j = 0, lenJ = editors.length; j < lenJ; j++) {
                    if (editors[j].hasCodeEditor(codeEditor)) {
                        return editors[j].getId();
                    }
                }
            }
            return null;
        };
        MainThreadEditorsTracker.prototype.registerTextEditorDecorationType = function (key, options) {
            this._codeEditorService.registerDecorationType(key, options);
        };
        MainThreadEditorsTracker.prototype.removeTextEditorDecorationType = function (key) {
            this._codeEditorService.removeDecorationType(key);
        };
        MainThreadEditorsTracker._LAST_TEXT_EDITOR_ID = 0;
        return MainThreadEditorsTracker;
    }());
    exports.MainThreadEditorsTracker = MainThreadEditorsTracker;
    function strcmp(a, b) {
        if (a < b) {
            return -1;
        }
        if (a > b) {
            return 1;
        }
        return 0;
    }
});

define("vs/workbench/browser/actionBarRegistry", ["require", "exports", 'vs/base/common/winjs.base', 'vs/platform/platform', 'vs/base/common/types', 'vs/base/common/collections', 'vs/base/browser/ui/actionbar/actionbar'], function (require, exports, winjs_base_1, platform_1, types, collections, actionbar_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The action bar contributor allows to add actions to an actionbar in a given context.
     */
    var ActionBarContributor = (function () {
        function ActionBarContributor() {
        }
        /**
         * Returns true if this contributor has actions for the given context.
         */
        ActionBarContributor.prototype.hasActions = function (context) {
            return false;
        };
        /**
         * Returns an array of primary actions in the given context.
         */
        ActionBarContributor.prototype.getActions = function (context) {
            return [];
        };
        /**
         * Returns true if this contributor has secondary actions for the given context.
         */
        ActionBarContributor.prototype.hasSecondaryActions = function (context) {
            return false;
        };
        /**
         * Returns an array of secondary actions in the given context.
         */
        ActionBarContributor.prototype.getSecondaryActions = function (context) {
            return [];
        };
        /**
         * Can return a specific IActionItem to render the given action.
         */
        ActionBarContributor.prototype.getActionItem = function (context, action) {
            return null;
        };
        return ActionBarContributor;
    }());
    exports.ActionBarContributor = ActionBarContributor;
    /**
     * Some predefined scopes to contribute actions to
     */
    exports.Scope = {
        /**
         * Actions inside the global activity bar (DEPRECATED)
         */
        GLOBAL: 'global',
        /**
         * Actions inside viewlets.
         */
        VIEWLET: 'viewlet',
        /**
         * Actions inside panels.
         */
        PANEL: 'panel',
        /**
         * Actions inside editors.
         */
        EDITOR: 'editor',
        /**
         * Actions inside tree widgets.
         */
        VIEWER: 'viewer'
    };
    /**
     * The ContributableActionProvider leverages the actionbar contribution model to find actions.
     */
    var ContributableActionProvider = (function () {
        function ContributableActionProvider() {
            this.registry = platform_1.Registry.as(exports.Extensions.Actionbar);
        }
        ContributableActionProvider.prototype.toContext = function (tree, element) {
            return {
                viewer: tree,
                element: element
            };
        };
        ContributableActionProvider.prototype.hasActions = function (tree, element) {
            var context = this.toContext(tree, element);
            var contributors = this.registry.getActionBarContributors(exports.Scope.VIEWER);
            for (var i = 0; i < contributors.length; i++) {
                var contributor = contributors[i];
                if (contributor.hasActions(context)) {
                    return true;
                }
            }
            return false;
        };
        ContributableActionProvider.prototype.getActions = function (tree, element) {
            var actions = [];
            var context = this.toContext(tree, element);
            // Collect Actions
            var contributors = this.registry.getActionBarContributors(exports.Scope.VIEWER);
            for (var i = 0; i < contributors.length; i++) {
                var contributor = contributors[i];
                if (contributor.hasActions(context)) {
                    actions.push.apply(actions, contributor.getActions(context));
                }
            }
            return winjs_base_1.TPromise.as(prepareActions(actions));
        };
        ContributableActionProvider.prototype.hasSecondaryActions = function (tree, element) {
            var context = this.toContext(tree, element);
            var contributors = this.registry.getActionBarContributors(exports.Scope.VIEWER);
            for (var i = 0; i < contributors.length; i++) {
                var contributor = contributors[i];
                if (contributor.hasSecondaryActions(context)) {
                    return true;
                }
            }
            return false;
        };
        ContributableActionProvider.prototype.getSecondaryActions = function (tree, element) {
            var actions = [];
            var context = this.toContext(tree, element);
            // Collect Actions
            var contributors = this.registry.getActionBarContributors(exports.Scope.VIEWER);
            for (var i = 0; i < contributors.length; i++) {
                var contributor = contributors[i];
                if (contributor.hasSecondaryActions(context)) {
                    actions.push.apply(actions, contributor.getSecondaryActions(context));
                }
            }
            return winjs_base_1.TPromise.as(prepareActions(actions));
        };
        ContributableActionProvider.prototype.getActionItem = function (tree, element, action) {
            var contributors = this.registry.getActionBarContributors(exports.Scope.VIEWER);
            var context = this.toContext(tree, element);
            for (var i = contributors.length - 1; i >= 0; i--) {
                var contributor = contributors[i];
                var itemProvider = contributor.getActionItem(context, action);
                if (itemProvider) {
                    return itemProvider;
                }
            }
            return null;
        };
        return ContributableActionProvider;
    }());
    exports.ContributableActionProvider = ContributableActionProvider;
    // Helper function used in parts to massage actions before showing in action areas
    function prepareActions(actions) {
        // Patch order if not provided
        for (var l = 0; l < actions.length; l++) {
            var a = actions[l];
            if (types.isUndefinedOrNull(a.order)) {
                a.order = l;
            }
        }
        // Sort by order
        actions = actions.sort(function (first, second) {
            var firstOrder = first.order;
            var secondOrder = second.order;
            return firstOrder < secondOrder ? -1 : 1;
        });
        // Clean up leading separators
        var firstIndexOfAction = -1;
        for (var i = 0; i < actions.length; i++) {
            if (actions[i].id === actionbar_1.Separator.ID) {
                continue;
            }
            firstIndexOfAction = i;
            break;
        }
        if (firstIndexOfAction === -1) {
            return [];
        }
        actions = actions.slice(firstIndexOfAction);
        // Clean up trailing separators
        for (var h = actions.length - 1; h >= 0; h--) {
            var isSeparator = actions[h].id === actionbar_1.Separator.ID;
            if (isSeparator) {
                actions.splice(h, 1);
            }
            else {
                break;
            }
        }
        // Clean up separator duplicates
        var foundAction = false;
        for (var k = actions.length - 1; k >= 0; k--) {
            var isSeparator = actions[k].id === actionbar_1.Separator.ID;
            if (isSeparator && !foundAction) {
                actions.splice(k, 1);
            }
            else if (!isSeparator) {
                foundAction = true;
            }
            else if (isSeparator) {
                foundAction = false;
            }
        }
        return actions;
    }
    exports.prepareActions = prepareActions;
    exports.Extensions = {
        Actionbar: 'workbench.contributions.actionbar'
    };
    var ActionBarRegistry = (function () {
        function ActionBarRegistry() {
            this.actionBarContributorConstructors = [];
            this.actionBarContributorInstances = {};
        }
        ActionBarRegistry.prototype.setInstantiationService = function (service) {
            this.instantiationService = service;
            while (this.actionBarContributorConstructors.length > 0) {
                var entry = this.actionBarContributorConstructors.shift();
                this.createActionBarContributor(entry.scope, entry.ctor);
            }
        };
        ActionBarRegistry.prototype.createActionBarContributor = function (scope, ctor) {
            var instance = this.instantiationService.createInstance(ctor);
            var target = collections.lookupOrInsert(this.actionBarContributorInstances, scope, []);
            target.push(instance);
        };
        ActionBarRegistry.prototype.getContributors = function (scope) {
            return collections.lookup(this.actionBarContributorInstances, scope, []);
        };
        ActionBarRegistry.prototype.getActionBarActionsForContext = function (scope, context) {
            var actions = [];
            // Go through contributors for scope
            this.getContributors(scope).forEach(function (contributor) {
                // Primary Actions
                if (contributor.hasActions(context)) {
                    actions.push.apply(actions, contributor.getActions(context));
                }
            });
            return actions;
        };
        ActionBarRegistry.prototype.getSecondaryActionBarActionsForContext = function (scope, context) {
            var actions = [];
            // Go through contributors
            this.getContributors(scope).forEach(function (contributor) {
                // Secondary Actions
                if (contributor.hasSecondaryActions(context)) {
                    actions.push.apply(actions, contributor.getSecondaryActions(context));
                }
            });
            return actions;
        };
        ActionBarRegistry.prototype.getActionItemForContext = function (scope, context, action) {
            var contributors = this.getContributors(scope);
            for (var i = 0; i < contributors.length; i++) {
                var contributor = contributors[i];
                var item = contributor.getActionItem(context, action);
                if (item) {
                    return item;
                }
            }
            return null;
        };
        ActionBarRegistry.prototype.registerActionBarContributor = function (scope, ctor) {
            if (!this.instantiationService) {
                this.actionBarContributorConstructors.push({
                    scope: scope,
                    ctor: ctor
                });
            }
            else {
                this.createActionBarContributor(scope, ctor);
            }
        };
        ActionBarRegistry.prototype.getActionBarContributors = function (scope) {
            return this.getContributors(scope).slice(0);
        };
        return ActionBarRegistry;
    }());
    platform_1.Registry.add(exports.Extensions.Actionbar, new ActionBarRegistry());
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/browser/actions/showPerformanceBox", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/browser/ui/timer/timer', 'vs/base/common/actions'], function (require, exports, winjs_base_1, timer_1, actions_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var timeKeeperRenderer = null;
    var ShowPerformanceBox = (function (_super) {
        __extends(ShowPerformanceBox, _super);
        function ShowPerformanceBox(id, label) {
            _super.call(this, id, label, null, true);
        }
        ShowPerformanceBox.prototype.run = function () {
            if (timeKeeperRenderer === null) {
                timeKeeperRenderer = new timer_1.TimeKeeperRenderer(function () {
                    timeKeeperRenderer.destroy();
                    timeKeeperRenderer = null;
                });
            }
            return winjs_base_1.TPromise.as(true);
        };
        return ShowPerformanceBox;
    }(actions_1.Action));
    exports.ShowPerformanceBox = ShowPerformanceBox;
});
// if (false /* Env.enablePerformanceTools */) {
// 	let registry = <IWorkbenchActionRegistry>Registry.as(Extensions.WorkbenchActions);
// 	registry.registerWorkbenchAction(new SyncActionDescriptor(ShowPerformanceBox, ID, LABEL));
// } 

define("vs/workbench/common/constants", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // Preferences
    exports.Preferences = {
        // Global
        THEME: 'workbench.theme'
    };
    // Container Identifiers
    exports.Identifiers = {
        WORKBENCH_CONTAINER: 'workbench.main.container',
        ACTIVITYBAR_PART: 'workbench.parts.activitybar',
        SIDEBAR_PART: 'workbench.parts.sidebar',
        PANEL_PART: 'workbench.parts.panel',
        EDITOR_PART: 'workbench.parts.editor',
        STATUSBAR_PART: 'workbench.parts.statusbar'
    };
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/contributions", ["require", "exports", 'vs/platform/platform'], function (require, exports, platform_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Extensions;
    (function (Extensions) {
        Extensions.Workbench = 'workbench.contributions.kind';
    })(Extensions = exports.Extensions || (exports.Extensions = {}));
    var WorkbenchContributionsRegistry = (function (_super) {
        __extends(WorkbenchContributionsRegistry, _super);
        function WorkbenchContributionsRegistry() {
            _super.apply(this, arguments);
        }
        WorkbenchContributionsRegistry.prototype.registerWorkbenchContribution = function (ctor) {
            _super.prototype._register.call(this, ctor);
        };
        WorkbenchContributionsRegistry.prototype.getWorkbenchContributions = function () {
            return _super.prototype._getInstances.call(this);
        };
        WorkbenchContributionsRegistry.prototype.setWorkbenchContributions = function (contributions) {
            _super.prototype._setInstances.call(this, contributions);
        };
        return WorkbenchContributionsRegistry;
    }(platform_1.BaseRegistry));
    platform_1.Registry.add(Extensions.Workbench, new WorkbenchContributionsRegistry());
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/editor", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/eventEmitter', 'vs/base/common/types', 'vs/base/common/objects'], function (require, exports, winjs_base_1, eventEmitter_1, types, objects) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Editor inputs are lightweight objects that can be passed to the workbench API to open inside the editor part.
     * Each editor input is mapped to an editor that is capable of opening it through the Platform facade.
     */
    var EditorInput = (function (_super) {
        __extends(EditorInput, _super);
        function EditorInput() {
            _super.apply(this, arguments);
        }
        /**
         * Returns the name of this input that can be shown to the user. Examples include showing the name of the input
         * above the editor area when the input is shown.
         */
        EditorInput.prototype.getName = function () {
            return null;
        };
        /**
         * Returns the description of this input that can be shown to the user. Examples include showing the description of
         * the input above the editor area to the side of the name of the input.
         *
         * @param verbose controls if the description should be short or can contain additional details.
         */
        EditorInput.prototype.getDescription = function (verbose) {
            return null;
        };
        /**
         * Returns status information about this input that can be shown to the user. Examples include showing the status
         * of the input when hovering over the name of the input.
         */
        EditorInput.prototype.getStatus = function () {
            return null;
        };
        /**
         * Returns the preferred editor for this input. A list of candidate editors is passed in that whee registered
         * for the input. This allows subclasses to decide late which editor to use for the input on a case by case basis.
         */
        EditorInput.prototype.getPreferredEditorId = function (candidates) {
            if (candidates && candidates.length > 0) {
                return candidates[0];
            }
            return null;
        };
        /**
         * Returns true if this input is identical to the otherInput.
         */
        EditorInput.prototype.matches = function (otherInput) {
            return this === otherInput;
        };
        /**
         * Called when an editor input is no longer needed. Allows to free up any resources taken by
         * resolving the editor input.
         */
        EditorInput.prototype.dispose = function () {
            this.disposed = true;
            this.emit('dispose');
            _super.prototype.dispose.call(this);
        };
        /**
         * Returns whether this input was disposed or not.
         */
        EditorInput.prototype.isDisposed = function () {
            return this.disposed;
        };
        return EditorInput;
    }(eventEmitter_1.EventEmitter));
    exports.EditorInput = EditorInput;
    (function (EncodingMode) {
        /**
         * Instructs the encoding support to encode the current input with the provided encoding
         */
        EncodingMode[EncodingMode["Encode"] = 0] = "Encode";
        /**
         * Instructs the encoding support to decode the current input with the provided encoding
         */
        EncodingMode[EncodingMode["Decode"] = 1] = "Decode";
    })(exports.EncodingMode || (exports.EncodingMode = {}));
    var EncodingMode = exports.EncodingMode;
    /**
     * The base class of untitled editor inputs in the workbench.
     */
    var UntitledEditorInput = (function (_super) {
        __extends(UntitledEditorInput, _super);
        function UntitledEditorInput() {
            _super.apply(this, arguments);
        }
        return UntitledEditorInput;
    }(EditorInput));
    exports.UntitledEditorInput = UntitledEditorInput;
    /**
     * The base class of editor inputs that have an original and modified side.
     */
    var BaseDiffEditorInput = (function (_super) {
        __extends(BaseDiffEditorInput, _super);
        function BaseDiffEditorInput(originalInput, modifiedInput) {
            _super.call(this);
            this._originalInput = originalInput;
            this._modifiedInput = modifiedInput;
        }
        Object.defineProperty(BaseDiffEditorInput.prototype, "originalInput", {
            get: function () {
                return this._originalInput;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseDiffEditorInput.prototype, "modifiedInput", {
            get: function () {
                return this._modifiedInput;
            },
            enumerable: true,
            configurable: true
        });
        BaseDiffEditorInput.prototype.getOriginalInput = function () {
            return this.originalInput;
        };
        BaseDiffEditorInput.prototype.getModifiedInput = function () {
            return this.modifiedInput;
        };
        return BaseDiffEditorInput;
    }(EditorInput));
    exports.BaseDiffEditorInput = BaseDiffEditorInput;
    /**
     * The editor model is the heavyweight counterpart of editor input. Depending on the editor input, it
     * connects to the disk to retrieve content and may allow for saving it back or reverting it. Editor models
     * are typically cached for some while because they are expensive to construct.
     */
    var EditorModel = (function (_super) {
        __extends(EditorModel, _super);
        function EditorModel() {
            _super.apply(this, arguments);
        }
        /**
         * Causes this model to load returning a promise when loading is completed.
         */
        EditorModel.prototype.load = function () {
            return winjs_base_1.TPromise.as(this);
        };
        /**
         * Returns whether this model was loaded or not.
         */
        EditorModel.prototype.isResolved = function () {
            return true;
        };
        /**
         * Subclasses should implement to free resources that have been claimed through loading.
         */
        EditorModel.prototype.dispose = function () {
            this.emit('dispose');
            _super.prototype.dispose.call(this);
        };
        return EditorModel;
    }(eventEmitter_1.EventEmitter));
    exports.EditorModel = EditorModel;
    /**
     * The editor options is the base class of options that can be passed in when opening an editor.
     */
    var EditorOptions = (function () {
        function EditorOptions() {
        }
        /**
         * Helper to create EditorOptions inline.
         */
        EditorOptions.create = function (settings) {
            var options = new EditorOptions();
            options.preserveFocus = settings.preserveFocus;
            options.forceOpen = settings.forceOpen;
            return options;
        };
        /**
         * Returns true if this options is identical to the otherOptions.
         */
        EditorOptions.prototype.matches = function (otherOptions) {
            return this === otherOptions;
        };
        return EditorOptions;
    }());
    exports.EditorOptions = EditorOptions;
    /**
     * Base Text Editor Options.
     */
    var TextEditorOptions = (function (_super) {
        __extends(TextEditorOptions, _super);
        function TextEditorOptions() {
            _super.apply(this, arguments);
        }
        TextEditorOptions.from = function (input) {
            var options = null;
            if (input && input.options) {
                if (input.options.selection || input.options.forceOpen || input.options.preserveFocus) {
                    options = new TextEditorOptions();
                }
                if (input.options.selection) {
                    var selection = input.options.selection;
                    options.selection(selection.startLineNumber, selection.startColumn, selection.endLineNumber, selection.endColumn);
                }
                if (input.options.forceOpen) {
                    options.forceOpen = true;
                }
                if (input.options.preserveFocus) {
                    options.preserveFocus = true;
                }
            }
            return options;
        };
        /**
         * Helper to create TextEditorOptions inline.
         */
        TextEditorOptions.create = function (settings) {
            var options = new TextEditorOptions();
            options.preserveFocus = settings.preserveFocus;
            options.forceActive = settings.forceActive;
            options.forceOpen = settings.forceOpen;
            if (settings.selection) {
                options.startLineNumber = settings.selection.startLineNumber;
                options.startColumn = settings.selection.startColumn;
                options.endLineNumber = settings.selection.endLineNumber || settings.selection.startLineNumber;
                options.endColumn = settings.selection.endColumn || settings.selection.startColumn;
            }
            return options;
        };
        /**
         * Returns if this options object has objects defined for the editor.
         */
        TextEditorOptions.prototype.hasOptionsDefined = function () {
            return !!this.editorViewState || (!types.isUndefinedOrNull(this.startLineNumber) && !types.isUndefinedOrNull(this.startColumn));
        };
        /**
         * Tells the editor to set show the given selection when the editor is being opened.
         */
        TextEditorOptions.prototype.selection = function (startLineNumber, startColumn, endLineNumber, endColumn) {
            if (endLineNumber === void 0) { endLineNumber = startLineNumber; }
            if (endColumn === void 0) { endColumn = startColumn; }
            this.startLineNumber = startLineNumber;
            this.startColumn = startColumn;
            this.endLineNumber = endLineNumber;
            this.endColumn = endColumn;
            return this;
        };
        /**
         * Sets the view state to be used when the editor is opening.
         */
        TextEditorOptions.prototype.viewState = function (viewState) {
            this.editorViewState = viewState;
        };
        /**
         * Apply the view state or selection to the given editor.
         *
         * @return if something was applied
         */
        TextEditorOptions.prototype.apply = function (textEditor) {
            var gotApplied = false;
            // First try viewstate
            if (this.editorViewState) {
                textEditor.restoreViewState(this.editorViewState);
                gotApplied = true;
            }
            else if (!types.isUndefinedOrNull(this.startLineNumber) && !types.isUndefinedOrNull(this.startColumn)) {
                // Select
                if (!types.isUndefinedOrNull(this.endLineNumber) && !types.isUndefinedOrNull(this.endColumn)) {
                    var range = {
                        startLineNumber: this.startLineNumber,
                        startColumn: this.startColumn,
                        endLineNumber: this.endLineNumber,
                        endColumn: this.endColumn
                    };
                    textEditor.setSelection(range);
                    textEditor.revealRangeInCenter(range);
                }
                else {
                    var pos = {
                        lineNumber: this.startLineNumber,
                        column: this.startColumn
                    };
                    textEditor.setPosition(pos);
                    textEditor.revealPositionInCenter(pos);
                }
                gotApplied = true;
            }
            return gotApplied;
        };
        TextEditorOptions.prototype.matches = function (otherOptions) {
            if (_super.prototype.matches.call(this, otherOptions) === true) {
                return true;
            }
            if (otherOptions) {
                return otherOptions instanceof TextEditorOptions &&
                    otherOptions.startLineNumber === this.startLineNumber &&
                    otherOptions.startColumn === this.startColumn &&
                    otherOptions.endLineNumber === this.endLineNumber &&
                    otherOptions.endColumn === this.endColumn &&
                    otherOptions.preserveFocus === this.preserveFocus &&
                    otherOptions.forceOpen === this.forceOpen &&
                    objects.equals(otherOptions.editorViewState, this.editorViewState);
            }
            return false;
        };
        return TextEditorOptions;
    }(EditorOptions));
    exports.TextEditorOptions = TextEditorOptions;
    /**
     * Base Text Diff Editor Options.
     */
    var TextDiffEditorOptions = (function (_super) {
        __extends(TextDiffEditorOptions, _super);
        function TextDiffEditorOptions() {
            _super.apply(this, arguments);
        }
        /**
         * Helper to create TextDiffEditorOptions inline.
         */
        TextDiffEditorOptions.create = function (settings) {
            var options = new TextDiffEditorOptions();
            options.autoRevealFirstChange = settings.autoRevealFirstChange;
            options.preserveFocus = settings.preserveFocus;
            options.forceOpen = settings.forceOpen;
            return options;
        };
        return TextDiffEditorOptions;
    }(TextEditorOptions));
    exports.TextDiffEditorOptions = TextDiffEditorOptions;
    /**
     * Given an input, tries to get the associated URI for it (either file or untitled scheme).
     */
    function getUntitledOrFileResource(input, supportDiff) {
        if (!input) {
            return null;
        }
        // Untitled
        if (input instanceof UntitledEditorInput) {
            return input.getResource();
        }
        // File
        var fileInput = asFileEditorInput(input, supportDiff);
        return fileInput && fileInput.getResource();
    }
    exports.getUntitledOrFileResource = getUntitledOrFileResource;
    /**
     * Returns the object as IFileEditorInput only if it matches the signature.
     */
    function asFileEditorInput(obj, supportDiff) {
        if (!obj) {
            return null;
        }
        // Check for diff if we are asked to
        if (supportDiff && types.isFunction(obj.getModifiedInput)) {
            obj = obj.getModifiedInput();
        }
        var i = obj;
        return i instanceof EditorInput && types.areFunctions(i.setResource, i.setMime, i.setEncoding, i.getEncoding, i.getResource, i.getMime) ? i : null;
    }
    exports.asFileEditorInput = asFileEditorInput;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/editor/binaryEditorModel", ["require", "exports", 'vs/workbench/common/editor'], function (require, exports, editor_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * An editor model that just represents a resource and mime for a resource that can be loaded.
     */
    var BinaryEditorModel = (function (_super) {
        __extends(BinaryEditorModel, _super);
        function BinaryEditorModel(resource, name) {
            _super.call(this);
            this.name = name;
            this.resource = resource;
        }
        /**
         * The name of the binary resource.
         */
        BinaryEditorModel.prototype.getName = function () {
            return this.name;
        };
        /**
         * The resource of the binary resource.
         */
        BinaryEditorModel.prototype.getResource = function () {
            return this.resource;
        };
        return BinaryEditorModel;
    }(editor_1.EditorModel));
    exports.BinaryEditorModel = BinaryEditorModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/editor/diffEditorModel", ["require", "exports", 'vs/base/common/winjs.base', 'vs/workbench/common/editor'], function (require, exports, winjs_base_1, editor_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The base editor model for the diff editor. It is made up of two editor models, the original version
     * and the modified version.
     */
    var DiffEditorModel = (function (_super) {
        __extends(DiffEditorModel, _super);
        function DiffEditorModel(originalModel, modifiedModel) {
            _super.call(this);
            this._originalModel = originalModel;
            this._modifiedModel = modifiedModel;
        }
        Object.defineProperty(DiffEditorModel.prototype, "originalModel", {
            get: function () {
                return this._originalModel;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DiffEditorModel.prototype, "modifiedModel", {
            get: function () {
                return this._modifiedModel;
            },
            enumerable: true,
            configurable: true
        });
        DiffEditorModel.prototype.load = function () {
            var _this = this;
            return winjs_base_1.TPromise.join([
                this._originalModel.load(),
                this._modifiedModel.load()
            ]).then(function () {
                return _this;
            });
        };
        DiffEditorModel.prototype.isResolved = function () {
            return this._originalModel.isResolved() && this._modifiedModel.isResolved();
        };
        DiffEditorModel.prototype.dispose = function () {
            // Do not propagate the dispose() call to the two models inside. We never created the two models
            // (original and modified) so we can not dispose them without sideeffects. Rather rely on the
            // models getting disposed when their related inputs get disposed from the diffEditorInput.
            _super.prototype.dispose.call(this);
        };
        return DiffEditorModel;
    }(editor_1.EditorModel));
    exports.DiffEditorModel = DiffEditorModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/editor/iframeEditorInput", ["require", "exports", 'vs/base/common/winjs.base', 'vs/workbench/common/editor'], function (require, exports, winjs_base_1, editor_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * An editor input to use with the IFrameEditor.
     */
    var IFrameEditorInput = (function (_super) {
        __extends(IFrameEditorInput, _super);
        function IFrameEditorInput(resource, name, description) {
            _super.call(this);
            this.resource = resource;
            this.name = name;
            this.description = description;
        }
        IFrameEditorInput.prototype.getId = function () {
            return IFrameEditorInput.ID;
        };
        IFrameEditorInput.prototype.getResource = function () {
            return this.resource;
        };
        IFrameEditorInput.prototype.getName = function () {
            return this.name;
        };
        IFrameEditorInput.prototype.getDescription = function () {
            return this.description;
        };
        IFrameEditorInput.prototype.resolve = function (refresh) {
            var _this = this;
            var modelPromise;
            // Use Cached Model
            if (this.cachedModel && !refresh) {
                modelPromise = winjs_base_1.TPromise.as(this.cachedModel);
            }
            else if (this.cachedModel && refresh) {
                modelPromise = this.cachedModel.load();
            }
            else {
                var model = this.createModel();
                modelPromise = model.load();
            }
            return modelPromise.then(function (resolvedModel) {
                _this.cachedModel = resolvedModel;
                return _this.cachedModel;
            });
        };
        IFrameEditorInput.prototype.matches = function (otherInput) {
            if (_super.prototype.matches.call(this, otherInput) === true) {
                return true;
            }
            if (otherInput instanceof IFrameEditorInput) {
                var otherIFrameEditorInput = otherInput;
                // Otherwise compare by properties
                return otherIFrameEditorInput.resource.toString() === this.resource.toString();
            }
            return false;
        };
        IFrameEditorInput.prototype.dispose = function () {
            if (this.cachedModel) {
                this.cachedModel.dispose();
                this.cachedModel = null;
            }
            _super.prototype.dispose.call(this);
        };
        IFrameEditorInput.ID = 'workbench.editors.iFrameEditorInput';
        return IFrameEditorInput;
    }(editor_1.EditorInput));
    exports.IFrameEditorInput = IFrameEditorInput;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/editor/iframeEditorModel", ["require", "exports", 'vs/workbench/common/editor'], function (require, exports, editor_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * An editor model that represents the resolved state for an iframe editor input. After the model has been
     * resolved it knows which content to pass to the iframe editor. The contents will be set directly into the
     * iframe. The contents have to ensure that e.g. a base URL is set so that relative links or images can be
     * resolved.
     */
    var IFrameEditorModel = (function (_super) {
        __extends(IFrameEditorModel, _super);
        function IFrameEditorModel(resource) {
            _super.call(this);
            this._resource = resource;
        }
        Object.defineProperty(IFrameEditorModel.prototype, "resource", {
            get: function () {
                return this._resource;
            },
            enumerable: true,
            configurable: true
        });
        IFrameEditorModel.prototype.setContents = function (head, body, tail) {
            this.head = head;
            this.body = body;
            this.tail = tail;
        };
        IFrameEditorModel.prototype.getContents = function () {
            return {
                head: this.head,
                body: this.body,
                tail: this.tail
            };
        };
        return IFrameEditorModel;
    }(editor_1.EditorModel));
    exports.IFrameEditorModel = IFrameEditorModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/editor/textDiffEditorModel", ["require", "exports", 'vs/workbench/common/editor/diffEditorModel'], function (require, exports, diffEditorModel_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The base text editor model for the diff editor. It is made up of two text editor models, the original version
     * and the modified version.
     */
    var TextDiffEditorModel = (function (_super) {
        __extends(TextDiffEditorModel, _super);
        function TextDiffEditorModel(originalModel, modifiedModel) {
            _super.call(this, originalModel, modifiedModel);
            this.updateTextDiffEditorModel();
        }
        TextDiffEditorModel.prototype.load = function () {
            var _this = this;
            return _super.prototype.load.call(this).then(function () {
                _this.updateTextDiffEditorModel();
                return _this;
            });
        };
        TextDiffEditorModel.prototype.updateTextDiffEditorModel = function () {
            if (this.originalModel.isResolved() && this.modifiedModel.isResolved()) {
                // Create new
                if (!this._textDiffEditorModel) {
                    this._textDiffEditorModel = {
                        original: this.originalModel.textEditorModel,
                        modified: this.modifiedModel.textEditorModel
                    };
                }
                else {
                    this._textDiffEditorModel.original = this.originalModel.textEditorModel;
                    this._textDiffEditorModel.modified = this.modifiedModel.textEditorModel;
                }
            }
        };
        Object.defineProperty(TextDiffEditorModel.prototype, "textDiffEditorModel", {
            get: function () {
                return this._textDiffEditorModel;
            },
            enumerable: true,
            configurable: true
        });
        TextDiffEditorModel.prototype.isResolved = function () {
            return !!this._textDiffEditorModel;
        };
        TextDiffEditorModel.prototype.dispose = function () {
            // Free the diff editor model but do not propagate the dispose() call to the two models
            // inside. We never created the two models (original and modified) so we can not dispose
            // them without sideeffects. Rather rely on the models getting disposed when their related
            // inputs get disposed from the diffEditorInput.
            this._textDiffEditorModel = null;
            _super.prototype.dispose.call(this);
        };
        return TextDiffEditorModel;
    }(diffEditorModel_1.DiffEditorModel));
    exports.TextDiffEditorModel = TextDiffEditorModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/common/editor/textEditorModel", ["require", "exports", 'vs/base/common/winjs.base', 'vs/editor/common/editorCommon', 'vs/workbench/common/editor', 'vs/editor/common/services/modeService', 'vs/editor/common/services/modelService', 'vs/editor/common/model/textModel'], function (require, exports, winjs_base_1, editorCommon_1, editor_1, modeService_1, modelService_1, textModel_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The base text editor model leverages the monaco code editor model. This class is only intended to be subclassed and not instantiated.
     */
    var BaseTextEditorModel = (function (_super) {
        __extends(BaseTextEditorModel, _super);
        function BaseTextEditorModel(modelService, modeService, textEditorModelHandle) {
            var _this = this;
            _super.call(this);
            this.modelService = modelService;
            this.modeService = modeService;
            this.textEditorModelHandle = textEditorModelHandle;
            if (textEditorModelHandle) {
                // We need the resource to point to an existing model
                var model = modelService.getModel(textEditorModelHandle);
                if (!model) {
                    throw new Error("Document with resource " + textEditorModelHandle.toString() + " does not exist");
                }
                // Since we did not create the model, we need to listen to it disposing
                // and properly trigger our dispose function so that events get emitted
                var unbind_1 = model.addListener(editorCommon_1.EventType.ModelDispose, function () {
                    _this.textEditorModelHandle = null; // make sure we do not dispose code editor model again
                    unbind_1();
                    _this.dispose();
                });
            }
        }
        Object.defineProperty(BaseTextEditorModel.prototype, "textEditorModel", {
            get: function () {
                return this.textEditorModelHandle ? this.modelService.getModel(this.textEditorModelHandle) : null;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Creates the text editor model with the provided value, mime (can be comma separated for multiple values) and optional resource URL.
         */
        BaseTextEditorModel.prototype.createTextEditorModel = function (value, resource, mime) {
            var _this = this;
            var firstLineText = this.getFirstLineText(value);
            // To avoid flickering, give the mode at most 50ms to load. If the mode doesn't load in 50ms, proceed creating the model with a mode promise
            return winjs_base_1.TPromise.any([winjs_base_1.TPromise.timeout(50), this.getOrCreateMode(this.modeService, mime, firstLineText)]).then(function () {
                var model = _this.modelService.createModel(value, _this.getOrCreateMode(_this.modeService, mime, firstLineText), resource);
                _this.createdEditorModel = true;
                _this.textEditorModelHandle = model.getAssociatedResource();
                return _this;
            });
        };
        BaseTextEditorModel.prototype.getFirstLineText = function (value) {
            var firstLineText = value.substr(0, 100);
            var crIndex = firstLineText.indexOf('\r');
            if (crIndex < 0) {
                crIndex = firstLineText.length;
            }
            var lfIndex = firstLineText.indexOf('\n');
            if (lfIndex < 0) {
                lfIndex = firstLineText.length;
            }
            firstLineText = firstLineText.substr(0, Math.min(crIndex, lfIndex));
            return firstLineText;
        };
        /**
         * Gets the mode for the given identifier. Subclasses can override to provide their own implementation of this lookup.
         *
         * @param firstLineText optional first line of the text buffer to set the mode on. This can be used to guess a mode from content.
         */
        BaseTextEditorModel.prototype.getOrCreateMode = function (modeService, mime, firstLineText) {
            return modeService.getOrCreateMode(mime);
        };
        /**
         * Updates the text editor model with the provided value. If the value is the same as the model has, this is a no-op.
         */
        BaseTextEditorModel.prototype.updateTextEditorModel = function (newValue) {
            if (!this.textEditorModel) {
                return;
            }
            var rawText = textModel_1.RawText.fromStringWithModelOptions(newValue, this.textEditorModel);
            // Return early if the text is already set in that form
            if (this.textEditorModel.equals(rawText)) {
                return;
            }
            // Otherwise update model
            this.textEditorModel.setValueFromRawText(rawText);
        };
        /**
         * Returns the textual value of this editor model or null if it has not yet been created.
         */
        BaseTextEditorModel.prototype.getValue = function () {
            var model = this.textEditorModel;
            if (model) {
                return model.getValue(editorCommon_1.EndOfLinePreference.TextDefined, true /* Preserve BOM */);
            }
            return null;
        };
        BaseTextEditorModel.prototype.dispose = function () {
            if (this.textEditorModelHandle && this.createdEditorModel) {
                this.modelService.destroyModel(this.textEditorModelHandle);
            }
            this.textEditorModelHandle = null;
            this.createdEditorModel = false;
            _super.prototype.dispose.call(this);
        };
        BaseTextEditorModel.prototype.isResolved = function () {
            return !!this.textEditorModelHandle;
        };
        BaseTextEditorModel = __decorate([
            __param(0, modelService_1.IModelService),
            __param(1, modeService_1.IModeService)
        ], BaseTextEditorModel);
        return BaseTextEditorModel;
    }(editor_1.EditorModel));
    exports.BaseTextEditorModel = BaseTextEditorModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/editor/diffEditorInput", ["require", "exports", 'vs/nls!vs/workbench/common/editor/diffEditorInput', 'vs/base/common/winjs.base', 'vs/base/common/types', 'vs/base/common/labels', 'vs/base/common/mime', 'vs/base/common/events', 'vs/workbench/common/editor', 'vs/workbench/common/editor/textEditorModel', 'vs/workbench/common/editor/diffEditorModel', 'vs/workbench/common/editor/textDiffEditorModel'], function (require, exports, nls, winjs_base_1, types, labels_1, mime_1, events_1, editor_1, textEditorModel_1, diffEditorModel_1, textDiffEditorModel_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The base editor input for the diff editor. It is made up of two editor inputs, the original version
     * and the modified version.
     */
    var DiffEditorInput = (function (_super) {
        __extends(DiffEditorInput, _super);
        function DiffEditorInput(name, description, originalInput, modifiedInput, forceOpenAsBinary) {
            _super.call(this, originalInput, modifiedInput);
            this.name = name;
            this.description = description;
            this.forceOpenAsBinary = forceOpenAsBinary;
            this._toUnbind = [];
            this.registerListeners();
        }
        DiffEditorInput.prototype.registerListeners = function () {
            var _this = this;
            // When the original or modified input gets disposed, dispose this diff editor input
            this._toUnbind.push(this.originalInput.addListener(events_1.EventType.DISPOSE, function () {
                if (!_this.isDisposed()) {
                    _this.dispose();
                }
            }));
            this._toUnbind.push(this.modifiedInput.addListener(events_1.EventType.DISPOSE, function () {
                if (!_this.isDisposed()) {
                    _this.dispose();
                }
            }));
        };
        Object.defineProperty(DiffEditorInput.prototype, "toUnbind", {
            get: function () {
                return this._toUnbind;
            },
            enumerable: true,
            configurable: true
        });
        DiffEditorInput.prototype.getId = function () {
            return DiffEditorInput.ID;
        };
        DiffEditorInput.prototype.getName = function () {
            return this.name;
        };
        DiffEditorInput.prototype.getDescription = function () {
            return this.description;
        };
        DiffEditorInput.prototype.getStatus = function () {
            if (this.modifiedInput) {
                var modifiedStatus = this.modifiedInput.getStatus();
                if (modifiedStatus) {
                    return modifiedStatus;
                }
            }
            if (this.originalInput) {
                var originalStatus = this.originalInput.getStatus();
                if (originalStatus) {
                    return originalStatus;
                }
            }
            return _super.prototype.getStatus.call(this);
        };
        DiffEditorInput.prototype.setOriginalInput = function (input) {
            this.originalInput = input;
        };
        DiffEditorInput.prototype.setModifiedInput = function (input) {
            this.modifiedInput = input;
        };
        DiffEditorInput.prototype.resolve = function (refresh) {
            var _this = this;
            var modelPromise;
            // Use Cached Model
            if (this.cachedModel && !refresh) {
                modelPromise = winjs_base_1.TPromise.as(this.cachedModel);
            }
            else {
                modelPromise = this.createModel(refresh);
            }
            return modelPromise.then(function (resolvedModel) {
                if (_this.cachedModel) {
                    _this.cachedModel.dispose();
                }
                _this.cachedModel = resolvedModel;
                return _this.cachedModel;
            });
        };
        DiffEditorInput.prototype.getPreferredEditorId = function (candidates) {
            // Find the right diff editor for the given isBinary/isText state
            var useBinaryEditor = this.forceOpenAsBinary || this.isBinary(this.originalInput) || this.isBinary(this.modifiedInput);
            return !useBinaryEditor ? 'workbench.editors.textDiffEditor' : 'workbench.editors.binaryResourceDiffEditor';
        };
        DiffEditorInput.prototype.isBinary = function (input) {
            var mime;
            // Find mime by checking for IFileEditorInput implementors
            var fileInput = input;
            if (types.isFunction(fileInput.getMime)) {
                mime = fileInput.getMime();
            }
            return mime && mime_1.isBinaryMime(mime);
        };
        DiffEditorInput.prototype.createModel = function (refresh) {
            // Join resolve call over two inputs and build diff editor model
            return winjs_base_1.TPromise.join([
                this.originalInput.resolve(refresh),
                this.modifiedInput.resolve(refresh)
            ]).then(function (models) {
                var originalEditorModel = models[0];
                var modifiedEditorModel = models[1];
                // If both are text models, return textdiffeditor model
                if (modifiedEditorModel instanceof textEditorModel_1.BaseTextEditorModel && originalEditorModel instanceof textEditorModel_1.BaseTextEditorModel) {
                    return new textDiffEditorModel_1.TextDiffEditorModel(originalEditorModel, modifiedEditorModel);
                }
                // Otherwise return normal diff model
                return new diffEditorModel_1.DiffEditorModel(originalEditorModel, modifiedEditorModel);
            });
        };
        DiffEditorInput.prototype.matches = function (otherInput) {
            if (_super.prototype.matches.call(this, otherInput) === true) {
                return true;
            }
            if (otherInput) {
                if (!(otherInput instanceof DiffEditorInput)) {
                    return false;
                }
                var otherDiffInput = otherInput;
                return this.originalInput.matches(otherDiffInput.originalInput) && this.modifiedInput.matches(otherDiffInput.modifiedInput);
            }
            return false;
        };
        DiffEditorInput.prototype.dispose = function () {
            while (this._toUnbind.length) {
                this._toUnbind.pop()();
            }
            // Dispose Model
            if (this.cachedModel) {
                this.cachedModel.dispose();
                this.cachedModel = null;
            }
            // Delegate to Inputs
            this.originalInput.dispose();
            this.modifiedInput.dispose();
            _super.prototype.dispose.call(this);
        };
        DiffEditorInput.ID = 'workbench.editors.diffEditorInput';
        return DiffEditorInput;
    }(editor_1.BaseDiffEditorInput));
    exports.DiffEditorInput = DiffEditorInput;
    function toDiffLabel(res1, res2, context) {
        var leftName = labels_1.getPathLabel(res1.fsPath, context);
        var rightName = labels_1.getPathLabel(res2.fsPath, context);
        return nls.localize(0, null, leftName, rightName);
    }
    exports.toDiffLabel = toDiffLabel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/common/editor/resourceEditorModel", ["require", "exports", 'vs/workbench/common/editor/textEditorModel', 'vs/editor/common/services/modeService', 'vs/editor/common/services/modelService'], function (require, exports, textEditorModel_1, modeService_1, modelService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * An editor model whith an in-memory, readonly content that is backed by an existing editor model.
     */
    var ResourceEditorModel = (function (_super) {
        __extends(ResourceEditorModel, _super);
        function ResourceEditorModel(resource, modeService, modelService) {
            _super.call(this, modelService, modeService, resource);
        }
        ResourceEditorModel = __decorate([
            __param(1, modeService_1.IModeService),
            __param(2, modelService_1.IModelService)
        ], ResourceEditorModel);
        return ResourceEditorModel;
    }(textEditorModel_1.BaseTextEditorModel));
    exports.ResourceEditorModel = ResourceEditorModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/common/editor/resourceEditorInput", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/async', 'vs/workbench/common/editor', 'vs/workbench/common/editor/resourceEditorModel', 'vs/base/common/events', 'vs/platform/instantiation/common/instantiation', 'vs/editor/common/services/modelService'], function (require, exports, winjs_base_1, async_1, editor_1, resourceEditorModel_1, events_1, instantiation_1, modelService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * A read-only text editor input whos contents are made of the provided resource that points to an existing
     * code editor model.
     */
    var ResourceEditorInput = (function (_super) {
        __extends(ResourceEditorInput, _super);
        function ResourceEditorInput(name, description, resource, modelService, instantiationService) {
            _super.call(this);
            this.modelService = modelService;
            this.instantiationService = instantiationService;
            this.name = name;
            this.description = description;
            this.resource = resource;
        }
        ResourceEditorInput.registerResourceContentProvider = function (scheme, provider) {
            var array = ResourceEditorInput.registry[scheme];
            if (!array) {
                array = [provider];
                ResourceEditorInput.registry[scheme] = array;
            }
            else {
                array.unshift(provider);
            }
            return {
                dispose: function () {
                    var array = ResourceEditorInput.registry[scheme];
                    var idx = array.indexOf(provider);
                    if (idx >= 0) {
                        array.splice(idx, 1);
                        if (array.length === 0) {
                            delete ResourceEditorInput.registry[scheme];
                        }
                    }
                }
            };
        };
        ResourceEditorInput.getOrCreateModel = function (modelService, resource) {
            var model = modelService.getModel(resource);
            if (model) {
                return winjs_base_1.TPromise.as(model);
            }
            var loadingModel = ResourceEditorInput.loadingModels[resource.toString()];
            if (!loadingModel) {
                // make sure we have a provider this scheme
                // the resource uses
                var array_1 = ResourceEditorInput.registry[resource.scheme];
                if (!array_1) {
                    return winjs_base_1.TPromise.wrapError("No model with uri '" + resource + "' nor a resolver for the scheme '" + resource.scheme + "'.");
                }
                // load the model-content from the provider and cache
                // the loading such that we don't create the same model
                // twice
                ResourceEditorInput.loadingModels[resource.toString()] = loadingModel = new winjs_base_1.TPromise(function (resolve, reject) {
                    var result;
                    var lastError;
                    async_1.sequence(array_1.map(function (provider) {
                        return function () {
                            if (!result) {
                                return provider.provideTextContent(resource).then(function (value) {
                                    result = value;
                                }, function (err) {
                                    lastError = err;
                                });
                            }
                        };
                    })).then(function () {
                        if (!result && lastError) {
                            reject(lastError);
                        }
                        else {
                            resolve(result);
                        }
                    }, reject);
                }, function () {
                    // no cancellation when caching promises
                });
                // remove the cached promise 'cos the model is now
                // known to the model service (see above)
                loadingModel.then(function () { return delete ResourceEditorInput.loadingModels[resource.toString()]; }, function () { return delete ResourceEditorInput.loadingModels[resource.toString()]; });
            }
            return loadingModel;
        };
        ResourceEditorInput.prototype.getId = function () {
            return ResourceEditorInput.ID;
        };
        ResourceEditorInput.prototype.getName = function () {
            return this.name;
        };
        ResourceEditorInput.prototype.getDescription = function () {
            return this.description;
        };
        ResourceEditorInput.prototype.resolve = function (refresh) {
            var _this = this;
            // Use Cached Model
            if (this.cachedModel) {
                return winjs_base_1.TPromise.as(this.cachedModel);
            }
            // Otherwise Create Model and handle dispose event
            return ResourceEditorInput.getOrCreateModel(this.modelService, this.resource).then(function () {
                var model = _this.instantiationService.createInstance(resourceEditorModel_1.ResourceEditorModel, _this.resource);
                var unbind = model.addListener(events_1.EventType.DISPOSE, function () {
                    _this.cachedModel = null; // make sure we do not dispose model again
                    unbind();
                    _this.dispose();
                });
                // Load it
                return model.load().then(function (resolvedModel) {
                    _this.cachedModel = resolvedModel;
                    return _this.cachedModel;
                });
            });
        };
        ResourceEditorInput.prototype.matches = function (otherInput) {
            if (_super.prototype.matches.call(this, otherInput) === true) {
                return true;
            }
            if (otherInput instanceof ResourceEditorInput) {
                var otherResourceEditorInput = otherInput;
                // Compare by properties
                return otherResourceEditorInput.resource.toString() === this.resource.toString();
            }
            return false;
        };
        ResourceEditorInput.prototype.dispose = function () {
            if (this.cachedModel) {
                this.cachedModel.dispose();
                this.cachedModel = null;
            }
            _super.prototype.dispose.call(this);
        };
        // --- registry logic
        // todo@joh,ben this should maybe be a service that is in charge of loading/resolving a uri from a scheme
        ResourceEditorInput.loadingModels = Object.create(null);
        ResourceEditorInput.registry = Object.create(null);
        ResourceEditorInput.ID = 'workbench.editors.resourceEditorInput';
        ResourceEditorInput = __decorate([
            __param(3, modelService_1.IModelService),
            __param(4, instantiation_1.IInstantiationService)
        ], ResourceEditorInput);
        return ResourceEditorInput;
    }(editor_1.EditorInput));
    exports.ResourceEditorInput = ResourceEditorInput;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/common/editor/stringEditorModel", ["require", "exports", 'vs/base/common/winjs.base', 'vs/workbench/common/editor/textEditorModel', 'vs/editor/common/core/position', 'vs/editor/common/core/range', 'vs/editor/common/services/modeService', 'vs/editor/common/services/modelService', 'vs/editor/common/core/editOperation'], function (require, exports, winjs_base_1, textEditorModel_1, position_1, range_1, modeService_1, modelService_1, editOperation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * An editor model whith an in-memory, readonly content that is not backed by any particular resource.
     */
    var StringEditorModel = (function (_super) {
        __extends(StringEditorModel, _super);
        function StringEditorModel(value, mime, resource, modeService, modelService) {
            _super.call(this, modelService, modeService);
            this.value = value;
            this.mime = mime;
            this.resource = resource;
        }
        /**
         * The value of this string editor model.
         */
        StringEditorModel.prototype.getValue = function () {
            return this.value;
        };
        /**
         * Sets the value of this string editor model.
         */
        StringEditorModel.prototype.setValue = function (value) {
            this.value = value;
            if (this.textEditorModel) {
                this.textEditorModel.setValue(value);
            }
        };
        /**
         * Appends value to this string editor model.
         */
        StringEditorModel.prototype.append = function (value) {
            this.value += value;
            if (this.textEditorModel) {
                var model = this.textEditorModel;
                var lastLine = model.getLineCount();
                var lastLineMaxColumn = model.getLineMaxColumn(lastLine);
                model.applyEdits([editOperation_1.EditOperation.insert(new position_1.Position(lastLine, lastLineMaxColumn), value)]);
            }
        };
        /**
         * Clears the value of this string editor model
         */
        StringEditorModel.prototype.clearValue = function () {
            this.value = '';
            if (this.textEditorModel) {
                var model = this.textEditorModel;
                var lastLine = model.getLineCount();
                model.applyEdits([editOperation_1.EditOperation.delete(new range_1.Range(1, 1, lastLine, model.getLineMaxColumn(lastLine)))]);
            }
        };
        /**
         * Removes all lines from the top if the line number exceeds the given line count. Returns the new value if lines got trimmed.
         */
        StringEditorModel.prototype.trim = function (linecount) {
            if (this.textEditorModel) {
                var model = this.textEditorModel;
                var lastLine = model.getLineCount();
                if (lastLine > linecount) {
                    model.applyEdits([editOperation_1.EditOperation.delete(new range_1.Range(1, 1, lastLine - linecount + 1, 1))]);
                    var newValue = model.getValue();
                    this.value = newValue;
                    return this.value;
                }
            }
            return null;
        };
        StringEditorModel.prototype.getMime = function () {
            return this.mime;
        };
        StringEditorModel.prototype.load = function () {
            // Create text editor model if not yet done
            if (!this.textEditorModel) {
                return this.createTextEditorModel(this.value, this.resource, this.mime);
            }
            else {
                this.updateTextEditorModel(this.value);
            }
            return winjs_base_1.TPromise.as(this);
        };
        StringEditorModel = __decorate([
            __param(3, modeService_1.IModeService),
            __param(4, modelService_1.IModelService)
        ], StringEditorModel);
        return StringEditorModel;
    }(textEditorModel_1.BaseTextEditorModel));
    exports.StringEditorModel = StringEditorModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/common/editor/stringEditorInput", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/mime', 'vs/workbench/common/editor', 'vs/workbench/common/editor/stringEditorModel', 'vs/platform/instantiation/common/instantiation'], function (require, exports, winjs_base_1, mime_1, editor_1, stringEditorModel_1, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * A read-only text editor input whos contents are made of the provided value and mime type.
     */
    var StringEditorInput = (function (_super) {
        __extends(StringEditorInput, _super);
        function StringEditorInput(name, description, value, mime, singleton, instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
            this.name = name;
            this.description = description;
            this.value = value;
            this.mime = mime || mime_1.MIME_TEXT;
            this.singleton = singleton;
        }
        StringEditorInput.prototype.getResource = function () {
            // Subclasses can implement to associate a resource with the input
            return null;
        };
        StringEditorInput.prototype.getId = function () {
            return StringEditorInput.ID;
        };
        StringEditorInput.prototype.getName = function () {
            return this.name;
        };
        StringEditorInput.prototype.getDescription = function () {
            return this.description;
        };
        StringEditorInput.prototype.getValue = function () {
            return this.value;
        };
        StringEditorInput.prototype.getMime = function () {
            return this.mime;
        };
        /**
         * Sets the textual value of this input and will also update the underlying model if this input is resolved.
         */
        StringEditorInput.prototype.setValue = function (value) {
            this.value = value;
            if (this.cachedModel) {
                this.cachedModel.setValue(value);
            }
        };
        /**
         * Clears the textual value of this input and will also update the underlying model if this input is resolved.
         */
        StringEditorInput.prototype.clearValue = function () {
            this.value = '';
            if (this.cachedModel) {
                this.cachedModel.clearValue();
            }
        };
        /**
         * Appends to the textual value of this input and will also update the underlying model if this input is resolved.
         */
        StringEditorInput.prototype.append = function (value) {
            this.value += value;
            if (this.cachedModel) {
                this.cachedModel.append(value);
            }
        };
        /**
         * Removes all lines from the top if the line number exceeds the given line count. Returns the new value if lines got trimmed.
         *
         * Note: This method is a no-op if the input has not yet been resolved.
         */
        StringEditorInput.prototype.trim = function (linecount) {
            if (this.cachedModel) {
                var newValue = this.cachedModel.trim(linecount);
                if (newValue !== null) {
                    this.value = newValue;
                    return this.value;
                }
            }
            return null;
        };
        StringEditorInput.prototype.resolve = function (refresh) {
            var _this = this;
            // Use Cached Model
            if (this.cachedModel) {
                return winjs_base_1.TPromise.as(this.cachedModel);
            }
            //Otherwise Create Model and Load
            var model = this.instantiationService.createInstance(stringEditorModel_1.StringEditorModel, this.value, this.mime, this.getResource());
            return model.load().then(function (resolvedModel) {
                _this.cachedModel = resolvedModel;
                return _this.cachedModel;
            });
        };
        StringEditorInput.prototype.matches = function (otherInput) {
            if (_super.prototype.matches.call(this, otherInput) === true) {
                return true;
            }
            if (otherInput instanceof StringEditorInput) {
                var otherStringEditorInput = otherInput;
                // If both inputs are singletons, check on the mime for equalness
                if (otherStringEditorInput.singleton && this.singleton && otherStringEditorInput.mime === this.mime) {
                    return true;
                }
                // Otherwise compare by properties
                return otherStringEditorInput.value === this.value &&
                    otherStringEditorInput.mime === this.mime &&
                    otherStringEditorInput.description === this.description &&
                    otherStringEditorInput.name === this.name;
            }
            return false;
        };
        StringEditorInput.prototype.dispose = function () {
            if (this.cachedModel) {
                this.cachedModel.dispose();
                this.cachedModel = null;
            }
            _super.prototype.dispose.call(this);
        };
        StringEditorInput.ID = 'workbench.editors.stringEditorInput';
        StringEditorInput = __decorate([
            __param(5, instantiation_1.IInstantiationService)
        ], StringEditorInput);
        return StringEditorInput;
    }(editor_1.EditorInput));
    exports.StringEditorInput = StringEditorInput;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/events", ["require", "exports", 'vs/base/common/events'], function (require, exports, events_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * All workbench events are listed here. For DOM events, see Monaco.Base.DomUtils.EventType.
     */
    var EventType = (function () {
        function EventType() {
        }
        /**
         * Event type for when an editor is opened. This event is only sent once for a specific editor type until another
         * editor type gets opened. For example, when the user opens a file, the editorOpened event will be sent. If another
         * file is opened, this event will not be fired again. If the user opens, e.g. the diff editor, editorOpened will be
         * fired, since another editor type opened.
         */
        EventType.EDITOR_OPENED = 'editorOpened';
        /**
         * Event type for when an editor is closed because another editor type is opened.
         */
        EventType.EDITOR_CLOSED = 'editorClosed';
        /**
         * Event to indciate that an editor input is about to open. This event can be prevented to do something else instead.
         */
        EventType.EDITOR_INPUT_OPENING = 'editorInputOpening';
        /**
         * Event type for when the editor input is about to change. This event is being sent before (!) the input is being set
         * to the active editor. Use EDITOR_INPUT_CHANGED to react after the input has been set and displayed by the editor.
         *
         * Note: This event will also be emitted when multiple editors are open and the user sets focus from the active editor
         * to another one. This allows to detect a focus change of the active editor.
         */
        EventType.EDITOR_INPUT_CHANGING = 'editorInputChanging';
        /**
         * Event type to indicate that the editor options of the current active editor are changing.
         */
        EventType.EDITOR_OPTIONS_CHANGING = 'editorOptionsChanging';
        /**
         * Event type for when the editor input has been changed in the currently active editor. This event is being sent after
         * the input has been set and displayed by the editor.
         *
         * Note: This event will also be emitted when multiple editors are open and the user sets focus from the active editor
         * to another one. This allows to detect a focus change of the active editor.
         */
        EventType.EDITOR_INPUT_CHANGED = 'editorInputChanged';
        /**
         * Event type for when the editor input state changed.
         */
        EventType.EDITOR_INPUT_STATE_CHANGED = 'editorInputStateChanged';
        /**
         * Event type for when the editor input failed to be set to the editor.
         */
        EventType.EDITOR_SET_INPUT_ERROR = 'editorSetInputError';
        /**
         * Event type for when the editor position has been changed
         */
        EventType.EDITOR_POSITION_CHANGED = 'editorPositionChanged';
        /**
         * An event type that fires when a text editor changes its selection.
         */
        EventType.TEXT_EDITOR_SELECTION_CHANGED = 'textEditorSelectionChanged';
        /**
         * An event type that fires when a text editor mode changes.
         */
        EventType.TEXT_EDITOR_MODE_CHANGED = 'textEditorModeChanged';
        /**
         * An event type that fires when a text editor content changes.
         */
        EventType.TEXT_EDITOR_CONTENT_CHANGED = 'textEditorContentChanged';
        /**
         * An event type that fires when a text editor content options changed.
         */
        EventType.TEXT_EDITOR_CONTENT_OPTIONS_CHANGED = 'textEditorContentOptionsChanged';
        /**
         * An event type that fires when a text editor's configuration changes.
         */
        EventType.TEXT_EDITOR_CONFIGURATION_CHANGED = 'textEditorOptionsChanged';
        /**
         * Event type for when a composite is about to open.
         */
        EventType.COMPOSITE_OPENING = 'compositeOpening';
        /**
         * Event type for when a composite is opened.
         */
        EventType.COMPOSITE_OPENED = 'compositeOpened';
        /**
         * Event type for when a composite is closed.
         */
        EventType.COMPOSITE_CLOSED = 'compositeClosed';
        /**
         * Event type for when the workbench has been fully created.
         */
        EventType.WORKBENCH_CREATED = 'workbenchCreated';
        /**
         * Event type for when the workbench is about to being disposed.
         */
        EventType.WORKBENCH_DISPOSING = 'workbenchDisposing';
        /**
         * Event type for when the workbench is fully disposed.
         */
        EventType.WORKBENCH_DISPOSED = 'workbenchDisposed';
        /**
         * Event type for when an untitled file is becoming dirty.
         */
        EventType.UNTITLED_FILE_DIRTY = 'untitledFileDirty';
        /**
         * Event type for when an untitled file is deleted.
         */
        EventType.UNTITLED_FILE_DELETED = 'untitledFileDeleted';
        /**
         * Event type for when a resources encoding changes.
         */
        EventType.RESOURCE_ENCODING_CHANGED = 'resourceEncodingChanged';
        /**
         * Event type for when the workbench options change. Listeners should refresh their
         * assumption on workbench options after this event is emitted.
         */
        EventType.WORKBENCH_OPTIONS_CHANGED = 'workbenchOptionsChanged';
        return EventType;
    }());
    exports.EventType = EventType;
    /**
     * Editor events are being emitted when the editor input changes, shows, is being saved or when the editor content changes.
     */
    var EditorEvent = (function (_super) {
        __extends(EditorEvent, _super);
        function EditorEvent(editor, editorId, editorInput, editorOptions, position, originalEvent) {
            _super.call(this, originalEvent);
            this.editor = editor;
            this.editorId = editorId;
            this.editorInput = editorInput;
            this.editorOptions = editorOptions;
            this.position = position;
        }
        EditorEvent.prototype.prevent = function () {
            this.prevented = true;
        };
        EditorEvent.prototype.isPrevented = function () {
            return this.prevented;
        };
        return EditorEvent;
    }(events_1.Event));
    exports.EditorEvent = EditorEvent;
    /**
     * Editor input events are being emitted when the editor input state changes.
     */
    var EditorInputEvent = (function (_super) {
        __extends(EditorInputEvent, _super);
        function EditorInputEvent(editorInput, originalEvent) {
            _super.call(this, originalEvent);
            this.editorInput = editorInput;
        }
        return EditorInputEvent;
    }(events_1.Event));
    exports.EditorInputEvent = EditorInputEvent;
    /**
     * A subclass of EditorEvent for text editor selection changes.
     */
    var TextEditorSelectionEvent = (function (_super) {
        __extends(TextEditorSelectionEvent, _super);
        function TextEditorSelectionEvent(selection, editor, editorId, editorInput, editorOptions, position, originalEvent) {
            _super.call(this, editor, editorId, editorInput, editorOptions, position, originalEvent);
            this.selection = selection;
        }
        return TextEditorSelectionEvent;
    }(EditorEvent));
    exports.TextEditorSelectionEvent = TextEditorSelectionEvent;
    /**
     * Option change events are send when the options in the running instance change.
     */
    var OptionsChangeEvent = (function (_super) {
        __extends(OptionsChangeEvent, _super);
        function OptionsChangeEvent(key, before, after, originalEvent) {
            _super.call(this, originalEvent);
            this.key = key;
            this.before = before;
            this.after = after;
        }
        return OptionsChangeEvent;
    }(events_1.Event));
    exports.OptionsChangeEvent = OptionsChangeEvent;
    /**
     * Command events are emitted when an action is being executed through a command handler (Keybinding).
     */
    var CommandEvent = (function (_super) {
        __extends(CommandEvent, _super);
        function CommandEvent(actionId, originalEvent) {
            _super.call(this, originalEvent);
            this.actionId = actionId;
        }
        return CommandEvent;
    }(events_1.Event));
    exports.CommandEvent = CommandEvent;
    /**
     * Composite events are emitted when a composite opens or closes in the sidebar or panel.
     */
    var CompositeEvent = (function (_super) {
        __extends(CompositeEvent, _super);
        function CompositeEvent(compositeId, originalEvent) {
            _super.call(this, originalEvent);
            this.compositeId = compositeId;
        }
        return CompositeEvent;
    }(events_1.Event));
    exports.CompositeEvent = CompositeEvent;
    var ResourceEvent = (function (_super) {
        __extends(ResourceEvent, _super);
        function ResourceEvent(resource, originalEvent) {
            _super.call(this, originalEvent);
            this.resource = resource;
        }
        return ResourceEvent;
    }(events_1.Event));
    exports.ResourceEvent = ResourceEvent;
    var UntitledEditorEvent = (function (_super) {
        __extends(UntitledEditorEvent, _super);
        function UntitledEditorEvent() {
            _super.apply(this, arguments);
        }
        return UntitledEditorEvent;
    }(ResourceEvent));
    exports.UntitledEditorEvent = UntitledEditorEvent;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/common/editor/untitledEditorModel", ["require", "exports", 'vs/workbench/common/editor/stringEditorModel', 'vs/editor/common/editorCommon', 'vs/workbench/common/events', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event', 'vs/editor/common/services/modeService', 'vs/editor/common/services/modelService'], function (require, exports, stringEditorModel_1, editorCommon_1, events_1, configuration_1, event_1, modeService_1, modelService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var UntitledEditorModel = (function (_super) {
        __extends(UntitledEditorModel, _super);
        function UntitledEditorModel(value, modeId, resource, hasAssociatedFilePath, modeService, modelService, eventService, configurationService) {
            _super.call(this, value, modeId, resource, modeService, modelService);
            this.eventService = eventService;
            this.configurationService = configurationService;
            this.dirty = hasAssociatedFilePath; // untitled associated to file path are dirty right away
            this.registerListeners();
        }
        UntitledEditorModel.prototype.registerListeners = function () {
            var _this = this;
            // Config Changes
            this.configurationChangeListenerUnbind = this.configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) { return _this.onConfigurationChange(e.config); });
        };
        UntitledEditorModel.prototype.onConfigurationChange = function (configuration) {
            this.configuredEncoding = configuration && configuration.files && configuration.files.encoding;
        };
        UntitledEditorModel.prototype.getValue = function () {
            if (this.textEditorModel) {
                return this.textEditorModel.getValue(editorCommon_1.EndOfLinePreference.TextDefined, true /* Preserve BOM */);
            }
            return null;
        };
        UntitledEditorModel.prototype.getModeId = function () {
            if (this.textEditorModel) {
                return this.textEditorModel.getModeId();
            }
            return null;
        };
        UntitledEditorModel.prototype.getEncoding = function () {
            return this.preferredEncoding || this.configuredEncoding;
        };
        UntitledEditorModel.prototype.setEncoding = function (encoding) {
            var oldEncoding = this.getEncoding();
            this.preferredEncoding = encoding;
            // Emit if it changed
            if (oldEncoding !== this.preferredEncoding) {
                this.eventService.emit(events_1.EventType.RESOURCE_ENCODING_CHANGED, new events_1.ResourceEvent(this.resource));
            }
        };
        UntitledEditorModel.prototype.isDirty = function () {
            return this.dirty;
        };
        UntitledEditorModel.prototype.load = function () {
            var _this = this;
            return _super.prototype.load.call(this).then(function (model) {
                return _this.configurationService.loadConfiguration().then(function (configuration) {
                    // Encoding
                    _this.configuredEncoding = configuration && configuration.files && configuration.files.encoding;
                    // Listen to content changes
                    _this.textModelChangeListener = _this.textEditorModel.addListener(editorCommon_1.EventType.ModelContentChanged, function (e) { return _this.onModelContentChanged(e); });
                    // Emit initial dirty event if we are
                    if (_this.dirty) {
                        setTimeout(function () {
                            _this.eventService.emit(events_1.EventType.UNTITLED_FILE_DIRTY, new events_1.UntitledEditorEvent(_this.resource));
                        }, 0 /* prevent race condition between creating model and emitting dirty event */);
                    }
                    return model;
                });
            });
        };
        UntitledEditorModel.prototype.onModelContentChanged = function (e) {
            if (!this.dirty) {
                this.dirty = true;
                this.eventService.emit(events_1.EventType.UNTITLED_FILE_DIRTY, new events_1.UntitledEditorEvent(this.resource));
            }
        };
        UntitledEditorModel.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            if (this.textModelChangeListener) {
                this.textModelChangeListener();
                this.textModelChangeListener = null;
            }
            if (this.configurationChangeListenerUnbind) {
                this.configurationChangeListenerUnbind();
                this.configurationChangeListenerUnbind = null;
            }
            this.eventService.emit(events_1.EventType.UNTITLED_FILE_DELETED, new events_1.UntitledEditorEvent(this.resource));
        };
        UntitledEditorModel = __decorate([
            __param(4, modeService_1.IModeService),
            __param(5, modelService_1.IModelService),
            __param(6, event_1.IEventService),
            __param(7, configuration_1.IConfigurationService)
        ], UntitledEditorModel);
        return UntitledEditorModel;
    }(stringEditorModel_1.StringEditorModel));
    exports.UntitledEditorModel = UntitledEditorModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/common/editor/untitledEditorInput", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/mime', 'vs/base/common/labels', 'vs/base/common/paths', 'vs/workbench/common/editor', 'vs/workbench/common/editor/untitledEditorModel', 'vs/platform/instantiation/common/instantiation', 'vs/platform/lifecycle/common/lifecycle', 'vs/platform/workspace/common/workspace', 'vs/editor/common/services/modeService'], function (require, exports, winjs_base_1, mime_1, labels, paths, editor_1, untitledEditorModel_1, instantiation_1, lifecycle_1, workspace_1, modeService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * An editor input to be used for untitled text buffers.
     */
    var UntitledEditorInput = (function (_super) {
        __extends(UntitledEditorInput, _super);
        function UntitledEditorInput(resource, hasAssociatedFilePath, modeId, instantiationService, lifecycleService, contextService, modeService) {
            _super.call(this);
            this.instantiationService = instantiationService;
            this.lifecycleService = lifecycleService;
            this.contextService = contextService;
            this.modeService = modeService;
            this.resource = resource;
            this.hasAssociatedFilePath = hasAssociatedFilePath;
            this.modeId = modeId;
        }
        UntitledEditorInput.prototype.getId = function () {
            return UntitledEditorInput.ID;
        };
        UntitledEditorInput.prototype.getResource = function () {
            return this.resource;
        };
        UntitledEditorInput.prototype.getName = function () {
            return this.hasAssociatedFilePath ? paths.basename(this.resource.fsPath) : this.resource.fsPath;
        };
        UntitledEditorInput.prototype.getDescription = function () {
            return this.hasAssociatedFilePath ? labels.getPathLabel(paths.dirname(this.resource.fsPath), this.contextService) : null;
        };
        UntitledEditorInput.prototype.isDirty = function () {
            return this.cachedModel && this.cachedModel.isDirty();
        };
        UntitledEditorInput.prototype.getStatus = function () {
            var isDirty = this.isDirty();
            if (isDirty) {
                return { state: 'dirty', decoration: '\u25cf' };
            }
            return null;
        };
        UntitledEditorInput.prototype.suggestFileName = function () {
            if (!this.hasAssociatedFilePath) {
                var mime = this.getMime();
                if (mime && mime !== mime_1.MIME_TEXT /* do not suggest when the mime type is simple plain text */) {
                    return mime_1.suggestFilename(mime, this.getName());
                }
            }
            return this.getName();
        };
        UntitledEditorInput.prototype.getMime = function () {
            if (this.cachedModel) {
                return this.modeService.getMimeForMode(this.cachedModel.getModeId());
            }
            return null;
        };
        UntitledEditorInput.prototype.getEncoding = function () {
            if (this.cachedModel) {
                return this.cachedModel.getEncoding();
            }
            return null;
        };
        UntitledEditorInput.prototype.setEncoding = function (encoding, mode /* ignored, we only have Encode */) {
            if (this.cachedModel) {
                this.cachedModel.setEncoding(encoding);
            }
        };
        UntitledEditorInput.prototype.resolve = function (refresh) {
            var _this = this;
            // Use Cached Model
            if (this.cachedModel) {
                return winjs_base_1.TPromise.as(this.cachedModel);
            }
            // Otherwise Create Model and load
            var model = this.createModel();
            return model.load().then(function (resolvedModel) {
                _this.cachedModel = resolvedModel;
                return _this.cachedModel;
            });
        };
        UntitledEditorInput.prototype.createModel = function () {
            var content = '';
            var mime = this.modeId;
            if (!mime && this.hasAssociatedFilePath) {
                var mimeFromPath = mime_1.guessMimeTypes(this.resource.fsPath)[0];
                if (!mime_1.isUnspecific(mimeFromPath)) {
                    mime = mimeFromPath; // take most specific mime type if file path is associated and mime is specific
                }
            }
            return this.instantiationService.createInstance(untitledEditorModel_1.UntitledEditorModel, content, mime || mime_1.MIME_TEXT, this.resource, this.hasAssociatedFilePath);
        };
        UntitledEditorInput.prototype.matches = function (otherInput) {
            if (_super.prototype.matches.call(this, otherInput) === true) {
                return true;
            }
            if (otherInput instanceof UntitledEditorInput) {
                var otherUntitledEditorInput = otherInput;
                // Otherwise compare by properties
                return otherUntitledEditorInput.resource.toString() === this.resource.toString();
            }
            return false;
        };
        UntitledEditorInput.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            if (this.cachedModel) {
                this.cachedModel.dispose();
                this.cachedModel = null;
            }
        };
        UntitledEditorInput.ID = 'workbench.editors.untitledEditorInput';
        UntitledEditorInput.SCHEMA = 'untitled';
        UntitledEditorInput = __decorate([
            __param(3, instantiation_1.IInstantiationService),
            __param(4, lifecycle_1.ILifecycleService),
            __param(5, workspace_1.IWorkspaceContextService),
            __param(6, modeService_1.IModeService)
        ], UntitledEditorInput);
        return UntitledEditorInput;
    }(editor_1.UntitledEditorInput));
    exports.UntitledEditorInput = UntitledEditorInput;
});

define("vs/workbench/common/memento", ["require", "exports", 'vs/base/common/types', 'vs/platform/storage/common/storage'], function (require, exports, types, storage_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Supported memento scopes.
     */
    (function (Scope) {
        /**
         * The memento will be scoped to all workspaces of this domain.
         */
        Scope[Scope["GLOBAL"] = 0] = "GLOBAL";
        /**
         * The memento will be scoped to the current workspace.
         */
        Scope[Scope["WORKSPACE"] = 1] = "WORKSPACE";
    })(exports.Scope || (exports.Scope = {}));
    var Scope = exports.Scope;
    /**
     * A memento provides access to a datastructure that is persisted and restored as part of the workbench lifecycle.
     */
    var Memento = (function () {
        function Memento(id) {
            this.id = Memento.COMMON_PREFIX + id.toLowerCase();
        }
        /**
         * Returns a JSON Object that represents the data of this memento. The optional
         * parameter scope allows to specify the scope of the memento to load. If not
         * provided, the scope will be global, Memento.Scope.WORKSPACE can be used to
         * scope the memento to the workspace.
         */
        Memento.prototype.getMemento = function (storageService, scope) {
            if (scope === void 0) { scope = Scope.GLOBAL; }
            // Scope by Workspace
            if (scope === Scope.WORKSPACE) {
                var workspaceMemento = Memento.workspaceMementos[this.id];
                if (!workspaceMemento) {
                    workspaceMemento = new ScopedMemento(this.id, scope, storageService);
                    Memento.workspaceMementos[this.id] = workspaceMemento;
                }
                return workspaceMemento.getMemento();
            }
            // Use global scope
            var globalMemento = Memento.globalMementos[this.id];
            if (!globalMemento) {
                globalMemento = new ScopedMemento(this.id, scope, storageService);
                Memento.globalMementos[this.id] = globalMemento;
            }
            return globalMemento.getMemento();
        };
        /**
         * Saves all data of the mementos that have been loaded to the local storage. This includes
         * global and workspace scope.
         */
        Memento.prototype.saveMemento = function () {
            // Global
            if (Memento.globalMementos[this.id]) {
                Memento.globalMementos[this.id].save();
            }
            // Workspace
            if (Memento.workspaceMementos[this.id]) {
                Memento.workspaceMementos[this.id].save();
            }
        };
        // Mementos are static to ensure that for a given component with an id only ever one memento gets loaded
        Memento.globalMementos = {};
        Memento.workspaceMementos = {};
        Memento.COMMON_PREFIX = 'memento/';
        return Memento;
    }());
    exports.Memento = Memento;
    var ScopedMemento = (function () {
        function ScopedMemento(id, scope, storageService) {
            this.storageService = storageService;
            this.id = id;
            this.scope = scope;
            this.mementoObj = this.loadMemento();
        }
        ScopedMemento.prototype.getMemento = function () {
            return this.mementoObj;
        };
        ScopedMemento.prototype.loadMemento = function () {
            var storageScope = this.scope === Scope.GLOBAL ? storage_1.StorageScope.GLOBAL : storage_1.StorageScope.WORKSPACE;
            var memento = this.storageService.get(this.id, storageScope);
            if (memento) {
                return JSON.parse(memento);
            }
            return {};
        };
        ScopedMemento.prototype.save = function () {
            var storageScope = this.scope === Scope.GLOBAL ? storage_1.StorageScope.GLOBAL : storage_1.StorageScope.WORKSPACE;
            if (!types.isEmptyObject(this.mementoObj)) {
                this.storageService.store(this.id, JSON.stringify(this.mementoObj), storageScope);
            }
            else {
                this.storageService.remove(this.id, storageScope);
            }
        };
        return ScopedMemento;
    }());
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/component", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/workbench/common/memento'], function (require, exports, eventEmitter_1, memento_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var WorkbenchComponent = (function (_super) {
        __extends(WorkbenchComponent, _super);
        function WorkbenchComponent(id) {
            _super.call(this);
            this._toUnbind = [];
            this.id = id;
            this.componentMemento = new memento_1.Memento(this.id);
        }
        Object.defineProperty(WorkbenchComponent.prototype, "toUnbind", {
            get: function () {
                return this._toUnbind;
            },
            enumerable: true,
            configurable: true
        });
        WorkbenchComponent.prototype.getId = function () {
            return this.id;
        };
        WorkbenchComponent.prototype.getMemento = function (storageService, scope) {
            if (scope === void 0) { scope = memento_1.Scope.GLOBAL; }
            return this.componentMemento.getMemento(storageService, scope);
        };
        WorkbenchComponent.prototype.saveMemento = function () {
            this.componentMemento.saveMemento();
        };
        WorkbenchComponent.prototype.shutdown = function () {
            // Save Memento
            this.saveMemento();
        };
        WorkbenchComponent.prototype.dispose = function () {
            while (this._toUnbind.length) {
                this._toUnbind.pop()();
            }
            _super.prototype.dispose.call(this);
        };
        return WorkbenchComponent;
    }(eventEmitter_1.EventEmitter));
    exports.WorkbenchComponent = WorkbenchComponent;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/composite", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/actions', 'vs/workbench/common/component', 'vs/workbench/common/events', 'vs/platform/telemetry/common/telemetry', 'vs/platform/instantiation/common/descriptors', 'vs/platform/selection/common/selection'], function (require, exports, winjs_base_1, actions_1, component_1, events_1, telemetry_1, descriptors_1, selection_1) {
    "use strict";
    /**
     * Internal composite events to communicate with composite container.
     */
    exports.EventType = {
        INTERNAL_COMPOSITE_TITLE_AREA_UPDATE: 'internalCompositeTitleAreaUpdate'
    };
    /**
     * Composites are layed out in the sidebar and panel part of the workbench. At a time only one composite
     * can be open in the sidebar, and only one composite can be open in the panel.
     * Each composite has a minimized representation that is good enough to provide some
     * information about the state of the composite data.
     * The workbench will keep a composite alive after it has been created and show/hide it based on
     * user interaction. The lifecycle of a composite goes in the order create(), setVisible(true|false),
     * layout(), focus(), dispose(). During use of the workbench, a composite will often receive a setVisible,
     * layout and focus call, but only one create and dispose call.
     */
    var Composite = (function (_super) {
        __extends(Composite, _super);
        /**
         * Create a new composite with the given ID and context.
         */
        function Composite(id, _telemetryService) {
            _super.call(this, id);
            this._telemetryService = _telemetryService;
            this._telemetryData = {};
            this.visible = false;
        }
        Composite.prototype.getTitle = function () {
            return null;
        };
        Object.defineProperty(Composite.prototype, "telemetryService", {
            get: function () {
                return this._telemetryService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Composite.prototype, "telemetryData", {
            get: function () {
                return this._telemetryData;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Note: Clients should not call this method, the monaco workbench calls this
         * method. Calling it otherwise may result in unexpected behavior.
         *
         * Called to create this composite on the provided builder. This method is only
         * called once during the lifetime of the workbench.
         * Note that DOM-dependent calculations should be performed from the setVisible()
         * call. Only then the composite will be part of the DOM.
         */
        Composite.prototype.create = function (parent) {
            this.parent = parent;
            return winjs_base_1.TPromise.as(null);
        };
        /**
         * Returns the container this composite is being build in.
         */
        Composite.prototype.getContainer = function () {
            return this.parent;
        };
        /**
         * Note: Clients should not call this method, the monaco workbench calls this
         * method. Calling it otherwise may result in unexpected behavior.
         *
         * Called to indicate that the composite has become visible or hidden. This method
         * is called more than once during workbench lifecycle depending on the user interaction.
         * The composite will be on-DOM if visible is set to true and off-DOM otherwise.
         *
         * The returned promise is complete when the composite is visible. As such it is valid
         * to do a long running operation from this call. Typically this operation should be
         * fast though because setVisible might be called many times during a session.
         */
        Composite.prototype.setVisible = function (visible) {
            this.visible = visible;
            // Reset telemetry data when composite becomes visible
            if (visible) {
                this._telemetryData = {};
                this._telemetryData.startTime = new Date();
            }
            else {
                this._telemetryData.timeSpent = (Date.now() - this._telemetryData.startTime) / 1000;
                delete this._telemetryData.startTime;
                // Only submit telemetry data when not running from an integration test
                if (this._telemetryService && this._telemetryService.publicLog) {
                    var eventName = 'compositeShown';
                    this._telemetryData.composite = this.getId();
                    this._telemetryService.publicLog(eventName, this._telemetryData);
                }
            }
            return winjs_base_1.TPromise.as(null);
        };
        /**
         * Called when this composite should receive keyboard focus.
         */
        Composite.prototype.focus = function () {
            // Subclasses can implement
        };
        /**
         * Returns an array of actions to show in the action bar of the composite.
         */
        Composite.prototype.getActions = function () {
            return [];
        };
        /**
         * Returns an array of actions to show in the action bar of the composite
         * in a less prominent way then action from getActions.
         */
        Composite.prototype.getSecondaryActions = function () {
            return [];
        };
        /**
         * For any of the actions returned by this composite, provide an IActionItem in
         * cases where the implementor of the composite wants to override the presentation
         * of an action. Returns null to indicate that the action is not rendered through
         * an action item.
         */
        Composite.prototype.getActionItem = function (action) {
            return null;
        };
        /**
         * Returns the instance of IActionRunner to use with this composite for the
         * composite tool bar.
         */
        Composite.prototype.getActionRunner = function () {
            if (!this.actionRunner) {
                this.actionRunner = new actions_1.ActionRunner();
            }
            return this.actionRunner;
        };
        /**
         * Method for composite implementors to indicate to the composite container that the title or the actions
         * of the composite have changed. Calling this method will cause the container to ask for title (getTitle())
         * and actions (getActions(), getSecondaryActions()) if the composite is visible or the next time the composite
         * gets visible.
         */
        Composite.prototype.updateTitleArea = function () {
            this.emit(exports.EventType.INTERNAL_COMPOSITE_TITLE_AREA_UPDATE, new events_1.CompositeEvent(this.getId()));
        };
        /**
         * Returns an array of elements that are selected in the composite.
         */
        Composite.prototype.getSelection = function () {
            return selection_1.Selection.EMPTY;
        };
        /**
         * Returns true if this composite is currently visible and false otherwise.
         */
        Composite.prototype.isVisible = function () {
            return this.visible;
        };
        /**
         * Returns the underlying composite control or null if it is not accessible.
         */
        Composite.prototype.getControl = function () {
            return null;
        };
        Composite = __decorate([
            __param(1, telemetry_1.ITelemetryService)
        ], Composite);
        return Composite;
    }(component_1.WorkbenchComponent));
    exports.Composite = Composite;
    /**
     * A composite descriptor is a leightweight descriptor of a composite in the monaco workbench.
     */
    var CompositeDescriptor = (function (_super) {
        __extends(CompositeDescriptor, _super);
        function CompositeDescriptor(moduleId, ctorName, id, name, cssClass, order) {
            _super.call(this, moduleId, ctorName);
            this.id = id;
            this.name = name;
            this.cssClass = cssClass;
            this.order = order;
        }
        return CompositeDescriptor;
    }(descriptors_1.AsyncDescriptor));
    exports.CompositeDescriptor = CompositeDescriptor;
    var CompositeRegistry = (function () {
        function CompositeRegistry() {
            this.composits = [];
        }
        CompositeRegistry.prototype.registerComposite = function (descriptor) {
            if (this.compositById(descriptor.id) !== null) {
                return;
            }
            this.composits.push(descriptor);
        };
        CompositeRegistry.prototype.getComposite = function (id) {
            return this.compositById(id);
        };
        CompositeRegistry.prototype.getComposits = function () {
            return this.composits.slice(0);
        };
        CompositeRegistry.prototype.setComposits = function (compositsToSet) {
            this.composits = compositsToSet;
        };
        CompositeRegistry.prototype.compositById = function (id) {
            for (var i = 0; i < this.composits.length; i++) {
                if (this.composits[i].id === id) {
                    return this.composits[i];
                }
            }
            return null;
        };
        return CompositeRegistry;
    }());
    exports.CompositeRegistry = CompositeRegistry;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/browser/panel", ["require", "exports", 'vs/platform/platform', 'vs/workbench/browser/composite'], function (require, exports, platform_1, composite_1) {
    "use strict";
    var Panel = (function (_super) {
        __extends(Panel, _super);
        function Panel() {
            _super.apply(this, arguments);
        }
        return Panel;
    }(composite_1.Composite));
    exports.Panel = Panel;
    /**
     * A panel descriptor is a leightweight descriptor of a panel in the monaco workbench.
     */
    var PanelDescriptor = (function (_super) {
        __extends(PanelDescriptor, _super);
        function PanelDescriptor(moduleId, ctorName, id, name, cssClass) {
            _super.call(this, moduleId, ctorName, id, name, cssClass);
        }
        return PanelDescriptor;
    }(composite_1.CompositeDescriptor));
    exports.PanelDescriptor = PanelDescriptor;
    var PanelRegistry = (function (_super) {
        __extends(PanelRegistry, _super);
        function PanelRegistry() {
            _super.apply(this, arguments);
        }
        /**
         * Registers a panel to the platform.
         */
        PanelRegistry.prototype.registerPanel = function (descriptor) {
            _super.prototype.registerComposite.call(this, descriptor);
        };
        /**
         * Returns the panel descriptor for the given id or null if none.
         */
        PanelRegistry.prototype.getPanel = function (id) {
            return this.getComposite(id);
        };
        /**
         * Returns an array of registered panels known to the platform.
         */
        PanelRegistry.prototype.getPanels = function () {
            return this.getComposits();
        };
        /**
         * Sets the id of the panel that should open on startup by default.
         */
        PanelRegistry.prototype.setDefaultPanelId = function (id) {
            this.defaultPanelId = id;
        };
        /**
         * Gets the id of the panel that should open on startup by default.
         */
        PanelRegistry.prototype.getDefaultPanelId = function () {
            return this.defaultPanelId;
        };
        return PanelRegistry;
    }(composite_1.CompositeRegistry));
    exports.PanelRegistry = PanelRegistry;
    exports.Extensions = {
        Panels: 'workbench.contributions.panels'
    };
    platform_1.Registry.add(exports.Extensions.Panels, new PanelRegistry());
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/browser/part", ["require", "exports", 'vs/base/browser/builder', 'vs/workbench/common/component', 'vs/css!./media/part'], function (require, exports, builder_1, component_1) {
    'use strict';
    /**
     * Parts are layed out in the workbench and have their own layout that arranges a title,
     * content and status area to show content.
     */
    var Part = (function (_super) {
        __extends(Part, _super);
        function Part(id) {
            _super.call(this, id);
        }
        /**
         * Note: Clients should not call this method, the monaco workbench calls this
         * method. Calling it otherwise may result in unexpected behavior.
         *
         * Called to create title, content and status area of the part.
         */
        Part.prototype.create = function (parent) {
            this.parent = parent;
            this.titleArea = this.createTitleArea(parent);
            this.contentArea = this.createContentArea(parent);
            this.statusArea = this.createStatusArea(parent);
            this.partLayout = new PartLayout(this.parent, this.titleArea, this.contentArea, this.statusArea);
        };
        /**
         * Returns the overall part container.
         */
        Part.prototype.getContainer = function () {
            return this.parent;
        };
        /**
         * Subclasses override to provide a title area implementation.
         */
        Part.prototype.createTitleArea = function (parent) {
            return null;
        };
        /**
         * Returns the title area container.
         */
        Part.prototype.getTitleArea = function () {
            return this.titleArea;
        };
        /**
         * Subclasses override to provide a content area implementation.
         */
        Part.prototype.createContentArea = function (parent) {
            return null;
        };
        /**
         * Returns the content area container.
         */
        Part.prototype.getContentArea = function () {
            return this.contentArea;
        };
        /**
         * Subclasses override to provide a status area implementation.
         */
        Part.prototype.createStatusArea = function (parent) {
            return null;
        };
        /**
         * Returns the status area container.
         */
        Part.prototype.getStatusArea = function () {
            return this.statusArea;
        };
        /**
         * Layout title, content and status area in the given dimension.
         */
        Part.prototype.layout = function (dimension) {
            return this.partLayout.layout(dimension);
        };
        /**
         * Returns the part layout implementation.
         */
        Part.prototype.getLayout = function () {
            return this.partLayout;
        };
        return Part;
    }(component_1.WorkbenchComponent));
    exports.Part = Part;
    var EmptyPart = (function (_super) {
        __extends(EmptyPart, _super);
        function EmptyPart(id) {
            _super.call(this, id);
        }
        return EmptyPart;
    }(Part));
    exports.EmptyPart = EmptyPart;
    var PartLayout = (function () {
        function PartLayout(container, titleArea, contentArea, statusArea) {
            this.container = container;
            this.titleArea = titleArea;
            this.contentArea = contentArea;
            this.statusArea = statusArea;
        }
        PartLayout.prototype.computeStyle = function () {
            var containerStyle = this.container.getComputedStyle();
            this.containerStyle = {
                borderLeftWidth: parseInt(containerStyle.getPropertyValue('border-left-width'), 10),
                borderRightWidth: parseInt(containerStyle.getPropertyValue('border-right-width'), 10),
                borderTopWidth: parseInt(containerStyle.getPropertyValue('border-top-width'), 10),
                borderBottomWidth: parseInt(containerStyle.getPropertyValue('border-bottom-width'), 10)
            };
            if (this.titleArea) {
                var titleStyle = this.titleArea.getComputedStyle();
                this.titleStyle = {
                    display: titleStyle.getPropertyValue('display'),
                    height: this.titleArea.getTotalSize().height
                };
            }
            if (this.statusArea) {
                var statusStyle = this.statusArea.getComputedStyle();
                this.statusStyle = {
                    display: statusStyle.getPropertyValue('display'),
                    height: this.statusArea.getTotalSize().height
                };
            }
        };
        PartLayout.prototype.layout = function (dimension) {
            if (!this.containerStyle) {
                this.computeStyle();
            }
            var width = dimension.width - (this.containerStyle.borderLeftWidth + this.containerStyle.borderRightWidth);
            var height = dimension.height - (this.containerStyle.borderTopWidth + this.containerStyle.borderBottomWidth);
            // Return the applied sizes to title, content and status
            var sizes = [];
            // Title Size: Width (Fill), Height (Variable)
            var titleSize;
            if (this.titleArea && this.titleStyle.display !== 'none') {
                titleSize = new builder_1.Dimension(width, Math.min(height, this.titleStyle.height));
            }
            else {
                titleSize = new builder_1.Dimension(0, 0);
            }
            // Status Size: Width (Fill), Height (Variable)
            var statusSize;
            if (this.statusArea && this.statusStyle.display !== 'none') {
                this.statusArea.getHTMLElement().style.height = this.statusArea.getHTMLElement().style.width = '';
                statusSize = new builder_1.Dimension(width, Math.min(height - titleSize.height, this.statusStyle.height));
            }
            else {
                statusSize = new builder_1.Dimension(0, 0);
            }
            // Content Size: Width (Fill), Height (Variable)
            var contentSize = new builder_1.Dimension(width, height - titleSize.height - statusSize.height);
            sizes.push(titleSize);
            sizes.push(contentSize);
            sizes.push(statusSize);
            // Content
            if (this.contentArea) {
                this.contentArea.size(contentSize.width, contentSize.height);
            }
            return sizes;
        };
        return PartLayout;
    }());
    exports.PartLayout = PartLayout;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/browser/parts/editor/baseEditor", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/actions', 'vs/workbench/browser/actionBarRegistry', 'vs/base/common/types', 'vs/workbench/common/events', 'vs/platform/platform', 'vs/workbench/browser/panel', 'vs/workbench/common/editor', 'vs/platform/editor/common/editor', 'vs/platform/instantiation/common/descriptors'], function (require, exports, winjs_base_1, actions_1, actionBarRegistry_1, types, events_1, platform_1, panel_1, editor_1, editor_2, descriptors_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The base class of editors in the workbench. Editors register themselves for specific editor inputs.
     * Editors are layed out in the editor part of the workbench. Only one editor can be open at a time.
     * Each editor has a minimized representation that is good enough to provide some information about the
     * state of the editor data.
     * The workbench will keep an editor alive after it has been created and show/hide it based on
     * user interaction. The lifecycle of a editor goes in the order create(), setVisible(true|false),
     * layout(), setInput(), focus(), dispose(). During use of the workbench, a editor will often receive a
     * clearInput, setVisible, layout and focus call, but only one create and dispose call.
     *
     * This class is only intended to be subclassed and not instantiated.
     */
    var BaseEditor = (function (_super) {
        __extends(BaseEditor, _super);
        function BaseEditor(id, telemetryService) {
            _super.call(this, id, telemetryService);
        }
        Object.defineProperty(BaseEditor.prototype, "input", {
            get: function () {
                return this._input;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Returns the current input of this editor or null if none.
         */
        BaseEditor.prototype.getInput = function () {
            return this._input || null;
        };
        Object.defineProperty(BaseEditor.prototype, "options", {
            get: function () {
                return this._options;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Returns the current options of this editor or null if none.
         */
        BaseEditor.prototype.getOptions = function () {
            return this._options || null;
        };
        /**
         * Note: Clients should not call this method, the monaco workbench calls this
         * method. Calling it otherwise may result in unexpected behavior.
         *
         * Sets the given input with the options to the part. An editor has to deal with the
         * situation that the same input is being set with different options.
         */
        BaseEditor.prototype.setInput = function (input, options) {
            this._input = input;
            this._options = options;
            return winjs_base_1.TPromise.as(null);
        };
        /**
         * Called to indicate to the editor that the input should be cleared and resources associated with the
         * input should be freed.
         */
        BaseEditor.prototype.clearInput = function () {
            this._input = null;
            this._options = null;
        };
        BaseEditor.prototype.create = function (parent) {
            var res = _super.prototype.create.call(this, parent);
            // Create Editor
            this.createEditor(parent);
            return res;
        };
        /**
         * Overload this function to allow for passing in a position argument.
         */
        BaseEditor.prototype.setVisible = function (visible, position) {
            if (position === void 0) { position = null; }
            var promise = _super.prototype.setVisible.call(this, visible);
            this._position = position;
            return promise;
        };
        /**
         * Called when the position of the editor changes while it is visible.
         */
        BaseEditor.prototype.changePosition = function (position) {
            this._position = position;
            this.emit(events_1.EventType.EDITOR_POSITION_CHANGED, new events_1.EditorEvent(this, this.getId(), this.input, this.options, this.position));
        };
        Object.defineProperty(BaseEditor.prototype, "position", {
            /**
             * The position this editor is showing in or null if none.
             */
            get: function () {
                return this._position;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Controls if the editor shows an action to split the input of the editor to the side. Subclasses should override
         * if they are capable of showing the same editor input side by side.
         */
        BaseEditor.prototype.supportsSplitEditor = function () {
            return false;
        };
        BaseEditor.prototype.dispose = function () {
            this._input = null;
            this._options = null;
            // Super Dispose
            _super.prototype.dispose.call(this);
        };
        return BaseEditor;
    }(panel_1.Panel));
    exports.BaseEditor = BaseEditor;
    /**
     * A lightweight descriptor of an editor. The descriptor is deferred so that heavy editors
     * can load lazily in the workbench.
     */
    var EditorDescriptor = (function (_super) {
        __extends(EditorDescriptor, _super);
        function EditorDescriptor(id, name, moduleId, ctorName) {
            _super.call(this, moduleId, ctorName);
            this.id = id;
            this.name = name;
        }
        EditorDescriptor.prototype.getId = function () {
            return this.id;
        };
        EditorDescriptor.prototype.getName = function () {
            return this.name;
        };
        EditorDescriptor.prototype.describes = function (obj) {
            return obj instanceof BaseEditor && obj.getId() === this.id;
        };
        return EditorDescriptor;
    }(descriptors_1.AsyncDescriptor));
    exports.EditorDescriptor = EditorDescriptor;
    exports.Extensions = {
        Editors: 'workbench.contributions.editors'
    };
    var INPUT_DESCRIPTORS_PROPERTY = '__$inputDescriptors';
    var EditorRegistry = (function () {
        function EditorRegistry() {
            this.editorInputFactoryConstructors = Object.create(null);
            this.editorInputFactoryInstances = Object.create(null);
            this.editors = [];
        }
        EditorRegistry.prototype.setInstantiationService = function (service) {
            this.instantiationService = service;
            for (var key in this.editorInputFactoryConstructors) {
                var element = this.editorInputFactoryConstructors[key];
                this.createEditorInputFactory(key, element);
            }
            this.editorInputFactoryConstructors = {};
        };
        EditorRegistry.prototype.createEditorInputFactory = function (editorInputId, ctor) {
            var instance = this.instantiationService.createInstance(ctor);
            this.editorInputFactoryInstances[editorInputId] = instance;
        };
        EditorRegistry.prototype.registerEditor = function (descriptor, editorInputDescriptor) {
            // Support both non-array and array parameter
            var inputDescriptors = [];
            if (!types.isArray(editorInputDescriptor)) {
                inputDescriptors.push(editorInputDescriptor);
            }
            else {
                inputDescriptors = editorInputDescriptor;
            }
            // Register (Support multiple Editors per Input)
            descriptor[INPUT_DESCRIPTORS_PROPERTY] = inputDescriptors;
            this.editors.push(descriptor);
        };
        EditorRegistry.prototype.getEditor = function (input) {
            var _this = this;
            var findEditorDescriptors = function (input, byInstanceOf) {
                var matchingDescriptors = [];
                for (var i = 0; i < _this.editors.length; i++) {
                    var editor = _this.editors[i];
                    var inputDescriptors = editor[INPUT_DESCRIPTORS_PROPERTY];
                    for (var j = 0; j < inputDescriptors.length; j++) {
                        var inputClass = inputDescriptors[j].ctor;
                        // Direct check on constructor type (ignores prototype chain)
                        if (!byInstanceOf && input.constructor === inputClass) {
                            matchingDescriptors.push(editor);
                            break;
                        }
                        else if (byInstanceOf && input instanceof inputClass) {
                            matchingDescriptors.push(editor);
                            break;
                        }
                    }
                }
                // If no descriptors found, continue search using instanceof and prototype chain
                if (!byInstanceOf && matchingDescriptors.length === 0) {
                    return findEditorDescriptors(input, true);
                }
                if (byInstanceOf) {
                    return matchingDescriptors;
                }
                return matchingDescriptors;
            };
            var descriptors = findEditorDescriptors(input);
            if (descriptors && descriptors.length > 0) {
                // Ask the input for its preferred Editor
                var preferredEditorId = input.getPreferredEditorId(descriptors.map(function (d) { return d.getId(); }));
                if (preferredEditorId) {
                    return this.getEditorById(preferredEditorId);
                }
                // Otherwise, first come first serve
                return descriptors[0];
            }
            return null;
        };
        EditorRegistry.prototype.getEditorById = function (editorId) {
            for (var i = 0; i < this.editors.length; i++) {
                var editor = this.editors[i];
                if (editor.getId() === editorId) {
                    return editor;
                }
            }
            return null;
        };
        EditorRegistry.prototype.getEditors = function () {
            return this.editors.slice(0);
        };
        EditorRegistry.prototype.setEditors = function (editorsToSet) {
            this.editors = editorsToSet;
        };
        EditorRegistry.prototype.getEditorInputs = function () {
            var inputClasses = [];
            for (var i = 0; i < this.editors.length; i++) {
                var editor = this.editors[i];
                var editorInputDescriptors = editor[INPUT_DESCRIPTORS_PROPERTY];
                inputClasses.push.apply(inputClasses, editorInputDescriptors.map(function (descriptor) { return descriptor.ctor; }));
            }
            return inputClasses;
        };
        EditorRegistry.prototype.registerDefaultFileInput = function (editorInputDescriptor) {
            this.defaultFileInputDescriptor = editorInputDescriptor;
        };
        EditorRegistry.prototype.getDefaultFileInput = function () {
            return this.defaultFileInputDescriptor;
        };
        EditorRegistry.prototype.registerEditorInputFactory = function (editorInputId, ctor) {
            if (!this.instantiationService) {
                this.editorInputFactoryConstructors[editorInputId] = ctor;
            }
            else {
                this.createEditorInputFactory(editorInputId, ctor);
            }
        };
        EditorRegistry.prototype.getEditorInputFactory = function (editorInputId) {
            return this.editorInputFactoryInstances[editorInputId];
        };
        return EditorRegistry;
    }());
    platform_1.Registry.add(exports.Extensions.Editors, new EditorRegistry());
    /**
     * A variant of the action bar contributor to register actions to specific editor inputs of the editor. This allows to have more
     * fine grained control over actions compared to contributing an action to a specific editor.
     */
    var EditorInputActionContributor = (function (_super) {
        __extends(EditorInputActionContributor, _super);
        function EditorInputActionContributor() {
            _super.call(this);
            this.mapEditorInputActionContextToPrimaryActions = this.createPositionArray();
            this.mapEditorInputActionContextToSecondaryActions = this.createPositionArray();
        }
        EditorInputActionContributor.prototype.createPositionArray = function () {
            var array = [];
            for (var i = 0; i < editor_2.POSITIONS.length; i++) {
                array[i] = {};
            }
            return array;
        };
        /* Subclasses can override to provide a custom cache implementation */
        EditorInputActionContributor.prototype.toId = function (context) {
            return context.editor.getId() + context.input.getId();
        };
        EditorInputActionContributor.prototype.clearInputsFromCache = function (position, isPrimary) {
            if (isPrimary) {
                this.doClearInputsFromCache(this.mapEditorInputActionContextToPrimaryActions[position]);
            }
            else {
                this.doClearInputsFromCache(this.mapEditorInputActionContextToSecondaryActions[position]);
            }
        };
        EditorInputActionContributor.prototype.doClearInputsFromCache = function (cache) {
            for (var key in cache) {
                if (cache.hasOwnProperty(key)) {
                    var cachedActions = cache[key];
                    cachedActions.forEach(function (action) {
                        action.input = null;
                        action.position = null;
                    });
                }
            }
        };
        /**
         * Returns true if this contributor has actions for the given editor input. Subclasses must not
         * override this method but instead hasActionsForEditorInput();
         */
        EditorInputActionContributor.prototype.hasActions = function (context) {
            if (!this.checkEditorContext(context)) {
                return false;
            }
            // Ask Cache
            if (this.mapEditorInputActionContextToPrimaryActions[context.position][this.toId(context)]) {
                return true;
            }
            // Ask Client
            return this.hasActionsForEditorInput(context);
        };
        /**
         * Returns an array of actions for the given editor input. Subclasses must not override this
         * method but instead getActionsForEditorInput();
         */
        EditorInputActionContributor.prototype.getActions = function (context) {
            if (!this.checkEditorContext(context)) {
                return [];
            }
            // This will cause any cached action to be set with null for the current editor input to prevent
            // leaking actions that still think the current editor input is what was set before.
            this.clearInputsFromCache(context.position, true /* primary actions */);
            // First consult cache
            var editorInput = context.input;
            var editorPosition = context.position;
            var cachedActions = this.mapEditorInputActionContextToPrimaryActions[context.position][this.toId(context)];
            if (cachedActions) {
                // Update the input field and position in all actions to indicate this change and return
                cachedActions.forEach(function (action) {
                    action.input = editorInput;
                    action.position = editorPosition;
                });
                return cachedActions;
            }
            // Otherwise collect and keep in cache
            var actions = this.getActionsForEditorInput(context);
            actions.forEach(function (action) {
                action.input = editorInput;
                action.position = editorPosition;
            });
            this.mapEditorInputActionContextToPrimaryActions[context.position][this.toId(context)] = actions;
            return actions;
        };
        /**
         * Returns true if this contributor has actions for the given editor input. Subclasses must not
         * override this method but instead hasSecondaryActionsForEditorInput();
         */
        EditorInputActionContributor.prototype.hasSecondaryActions = function (context) {
            if (!this.checkEditorContext(context)) {
                return false;
            }
            // Ask Cache
            if (this.mapEditorInputActionContextToSecondaryActions[context.position][this.toId(context)]) {
                return true;
            }
            // Ask Client
            return this.hasSecondaryActionsForEditorInput(context);
        };
        /**
         * Returns an array of actions for the given editor input. Subclasses must not override this
         * method but instead getSecondaryActionsForEditorInput();
         */
        EditorInputActionContributor.prototype.getSecondaryActions = function (context) {
            if (!this.checkEditorContext(context)) {
                return [];
            }
            // This will cause any cached action to be set with null for the current editor input to prevent
            // leaking actions that still think the current editor input is what was set before.
            this.clearInputsFromCache(context.position, false /* secondary actions */);
            // First consult cache
            var editorInput = context.input;
            var editorPosition = context.position;
            var cachedActions = this.mapEditorInputActionContextToSecondaryActions[context.position][this.toId(context)];
            if (cachedActions) {
                // Update the input field and position in all actions to indicate this change and return
                cachedActions.forEach(function (action) {
                    action.input = editorInput;
                    action.position = editorPosition;
                });
                return cachedActions;
            }
            // Otherwise collect and keep in cache
            var actions = this.getSecondaryActionsForEditorInput(context);
            actions.forEach(function (action) {
                action.input = editorInput;
                action.position = editorPosition;
            });
            this.mapEditorInputActionContextToSecondaryActions[context.position][this.toId(context)] = actions;
            return actions;
        };
        EditorInputActionContributor.prototype.checkEditorContext = function (context) {
            return context && context.input instanceof editor_1.EditorInput && context.editor instanceof BaseEditor && !types.isUndefinedOrNull(context.position);
        };
        /**
         * Returns true if this contributor has primary actions for the given editor input.
         */
        EditorInputActionContributor.prototype.hasActionsForEditorInput = function (context) {
            return false;
        };
        /**
         * Returns an array of primary actions for the given editor input.
         */
        EditorInputActionContributor.prototype.getActionsForEditorInput = function (context) {
            return [];
        };
        /**
         * Returns true if this contributor has secondary actions for the given editor input.
         */
        EditorInputActionContributor.prototype.hasSecondaryActionsForEditorInput = function (context) {
            return false;
        };
        /**
         * Returns an array of secondary actions for the given editor input.
         */
        EditorInputActionContributor.prototype.getSecondaryActionsForEditorInput = function (context) {
            return [];
        };
        return EditorInputActionContributor;
    }(actionBarRegistry_1.ActionBarContributor));
    exports.EditorInputActionContributor = EditorInputActionContributor;
    var EditorInputAction = (function (_super) {
        __extends(EditorInputAction, _super);
        function EditorInputAction() {
            _super.apply(this, arguments);
        }
        Object.defineProperty(EditorInputAction.prototype, "input", {
            get: function () {
                return this._input;
            },
            set: function (input) {
                this._input = input;
                this.enabled = this.isEnabled();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditorInputAction.prototype, "position", {
            get: function () {
                return this._position;
            },
            set: function (position) {
                this._position = position;
            },
            enumerable: true,
            configurable: true
        });
        EditorInputAction.prototype.isEnabled = function () {
            return !!this._input;
        };
        return EditorInputAction;
    }(actions_1.Action));
    exports.EditorInputAction = EditorInputAction;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/common/storage", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/base/common/types', 'vs/base/common/errors', 'vs/base/common/strings', 'vs/platform/storage/common/storage'], function (require, exports, eventEmitter_1, types, errors, strings, storage_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var Storage = (function (_super) {
        __extends(Storage, _super);
        function Storage(contextService, globalStorage, workspaceStorage) {
            if (workspaceStorage === void 0) { workspaceStorage = globalStorage; }
            _super.call(this);
            this.serviceId = storage_1.IStorageService;
            var workspace = contextService.getWorkspace();
            this.globalStorage = globalStorage;
            this.workspaceStorage = workspaceStorage;
            this.toUnbind = [];
            // Calculate workspace storage key
            this.workspaceKey = this.getWorkspaceKey(workspace);
            // Make sure to delete all workspace storage if the workspace has been recreated meanwhile
            var workspaceUniqueId = workspace ? workspace.uid : null;
            if (types.isNumber(workspaceUniqueId)) {
                this.cleanupWorkspaceScope(workspaceUniqueId, workspace.name);
            }
        }
        Storage.prototype.getWorkspaceKey = function (workspace) {
            var workspaceUri = null;
            if (workspace && workspace.resource) {
                workspaceUri = workspace.resource.toString();
            }
            return workspaceUri ? this.calculateWorkspaceKey(workspaceUri) : Storage.NO_WORKSPACE_IDENTIFIER;
        };
        Storage.prototype.calculateWorkspaceKey = function (workspaceUrl) {
            var root = 'file:///';
            var index = workspaceUrl.indexOf(root);
            if (index === 0) {
                return strings.rtrim(workspaceUrl.substr(root.length), '/') + '/';
            }
            return workspaceUrl;
        };
        Storage.prototype.cleanupWorkspaceScope = function (workspaceId, workspaceName) {
            var _this = this;
            // Get stored identifier from storage
            var id = this.getInteger(Storage.WORKSPACE_IDENTIFIER, storage_1.StorageScope.WORKSPACE);
            // If identifier differs, assume the workspace got recreated and thus clean all storage for this workspace
            if (types.isNumber(id) && workspaceId !== id) {
                var keyPrefix = this.toStorageKey('', storage_1.StorageScope.WORKSPACE);
                var toDelete = [];
                var length_1 = this.workspaceStorage.length;
                for (var i = 0; i < length_1; i++) {
                    var key = this.workspaceStorage.key(i);
                    if (key.indexOf(Storage.WORKSPACE_PREFIX) < 0) {
                        continue; // ignore stored things that don't belong to storage service or are defined globally
                    }
                    // Check for match on prefix
                    if (key.indexOf(keyPrefix) === 0) {
                        toDelete.push(key);
                    }
                }
                if (toDelete.length > 0) {
                    console.warn('Clearing previous version of local storage for workspace ', workspaceName);
                }
                // Run the delete
                toDelete.forEach(function (keyToDelete) {
                    _this.workspaceStorage.removeItem(keyToDelete);
                });
            }
            // Store workspace identifier now
            if (workspaceId !== id) {
                this.store(Storage.WORKSPACE_IDENTIFIER, workspaceId, storage_1.StorageScope.WORKSPACE);
            }
        };
        Storage.prototype.clear = function () {
            this.globalStorage.clear();
            this.workspaceStorage.clear();
        };
        Storage.prototype.store = function (key, value, scope) {
            if (scope === void 0) { scope = storage_1.StorageScope.GLOBAL; }
            var storage = (scope === storage_1.StorageScope.GLOBAL) ? this.globalStorage : this.workspaceStorage;
            if (types.isUndefinedOrNull(value)) {
                this.remove(key, scope); // we cannot store null or undefined, in that case we remove the key
                return;
            }
            var storageKey = this.toStorageKey(key, scope);
            var before = storage.getItem(storageKey);
            var after = value.toString();
            // Store
            try {
                storage.setItem(storageKey, value);
            }
            catch (error) {
                errors.onUnexpectedError(error);
            }
            // Emit Event
            if (before !== after) {
                this.emit(storage_1.StorageEventType.STORAGE, new storage_1.StorageEvent(key, before, after));
            }
        };
        Storage.prototype.get = function (key, scope, defaultValue) {
            if (scope === void 0) { scope = storage_1.StorageScope.GLOBAL; }
            var storage = (scope === storage_1.StorageScope.GLOBAL) ? this.globalStorage : this.workspaceStorage;
            var value = storage.getItem(this.toStorageKey(key, scope));
            if (types.isUndefinedOrNull(value)) {
                return defaultValue;
            }
            return value;
        };
        Storage.prototype.remove = function (key, scope) {
            if (scope === void 0) { scope = storage_1.StorageScope.GLOBAL; }
            var storage = (scope === storage_1.StorageScope.GLOBAL) ? this.globalStorage : this.workspaceStorage;
            var storageKey = this.toStorageKey(key, scope);
            var before = storage.getItem(storageKey);
            var after = null;
            // Remove
            storage.removeItem(storageKey);
            // Emit Event
            if (before !== after) {
                this.emit(storage_1.StorageEventType.STORAGE, new storage_1.StorageEvent(key, before, after));
            }
        };
        Storage.prototype.swap = function (key, valueA, valueB, scope, defaultValue) {
            if (scope === void 0) { scope = storage_1.StorageScope.GLOBAL; }
            var value = this.get(key, scope);
            if (types.isUndefinedOrNull(value) && defaultValue) {
                this.store(key, defaultValue, scope);
            }
            else if (value === valueA.toString()) {
                this.store(key, valueB, scope);
            }
            else {
                this.store(key, valueA, scope);
            }
        };
        Storage.prototype.getInteger = function (key, scope, defaultValue) {
            if (scope === void 0) { scope = storage_1.StorageScope.GLOBAL; }
            var value = this.get(key, scope, defaultValue);
            if (types.isUndefinedOrNull(value)) {
                return defaultValue;
            }
            return parseInt(value, 10);
        };
        Storage.prototype.getBoolean = function (key, scope, defaultValue) {
            if (scope === void 0) { scope = storage_1.StorageScope.GLOBAL; }
            var value = this.get(key, scope, defaultValue);
            if (types.isUndefinedOrNull(value)) {
                return defaultValue;
            }
            if (types.isString(value)) {
                return value.toLowerCase() === 'true' ? true : false;
            }
            return value ? true : false;
        };
        Storage.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            while (this.toUnbind.length) {
                this.toUnbind.pop()();
            }
        };
        Storage.prototype.toStorageKey = function (key, scope) {
            if (scope === storage_1.StorageScope.GLOBAL) {
                return Storage.GLOBAL_PREFIX + key.toLowerCase();
            }
            return Storage.WORKSPACE_PREFIX + this.workspaceKey + key.toLowerCase();
        };
        Storage.COMMON_PREFIX = 'storage://';
        Storage.GLOBAL_PREFIX = Storage.COMMON_PREFIX + 'global/';
        Storage.WORKSPACE_PREFIX = Storage.COMMON_PREFIX + 'workspace/';
        Storage.WORKSPACE_IDENTIFIER = 'workspaceIdentifier';
        Storage.NO_WORKSPACE_IDENTIFIER = '__$noWorkspace__';
        return Storage;
    }(eventEmitter_1.EventEmitter));
    exports.Storage = Storage;
    // In-Memory Local Storage Implementation
    var InMemoryLocalStorage = (function () {
        function InMemoryLocalStorage() {
            this.store = {};
        }
        Object.defineProperty(InMemoryLocalStorage.prototype, "length", {
            get: function () {
                return Object.keys(this.store).length;
            },
            enumerable: true,
            configurable: true
        });
        InMemoryLocalStorage.prototype.key = function (index) {
            var keys = Object.keys(this.store);
            if (keys.length > index) {
                return keys[index];
            }
            return null;
        };
        InMemoryLocalStorage.prototype.clear = function () {
            this.store = {};
        };
        InMemoryLocalStorage.prototype.setItem = function (key, value) {
            this.store[key] = value.toString();
        };
        InMemoryLocalStorage.prototype.getItem = function (key) {
            var item = this.store[key];
            if (!types.isUndefinedOrNull(item)) {
                return item;
            }
            return null;
        };
        InMemoryLocalStorage.prototype.removeItem = function (key) {
            delete this.store[key];
        };
        return InMemoryLocalStorage;
    }());
    exports.InMemoryLocalStorage = InMemoryLocalStorage;
    exports.inMemoryLocalStorageInstance = new InMemoryLocalStorage();
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/electron-browser/crashReporter", ["require", "exports", 'vs/nls!vs/workbench/electron-browser/crashReporter', 'vs/platform/configuration/common/configurationRegistry', 'vs/platform/configuration/common/configuration', 'vs/platform/telemetry/common/telemetry', 'vs/platform/platform', 'electron'], function (require, exports, nls, configurationRegistry_1, configuration_1, telemetry_1, platform_1, electron_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var TELEMETRY_SECTION_ID = 'telemetry';
    var configurationRegistry = platform_1.Registry.as(configurationRegistry_1.Extensions.Configuration);
    configurationRegistry.registerConfiguration({
        'id': TELEMETRY_SECTION_ID,
        'order': 20,
        'type': 'object',
        'title': nls.localize(0, null),
        'properties': {
            'telemetry.enableCrashReporter': {
                'type': 'boolean',
                'description': nls.localize(1, null),
                'default': true
            }
        }
    });
    var CrashReporter = (function () {
        function CrashReporter(version, commit, telemetryService, configurationService) {
            this.telemetryService = telemetryService;
            this.configurationService = configurationService;
            this.configurationService = configurationService;
            this.telemetryService = telemetryService;
            this.sessionId = this.telemetryService ? this.telemetryService.getSessionId() : null;
            this.version = version;
            this.commit = commit;
            this.isStarted = false;
            this.config = null;
        }
        CrashReporter.prototype.start = function (rawConfiguration) {
            var _this = this;
            if (!this.isStarted) {
                if (!this.config) {
                    this.configurationService.loadConfiguration(TELEMETRY_SECTION_ID).done(function (c) {
                        _this.config = c;
                        if (_this.config && _this.config.enableCrashReporter) {
                            _this.doStart(rawConfiguration);
                        }
                    });
                }
                else {
                    if (this.config.enableCrashReporter) {
                        this.doStart(rawConfiguration);
                    }
                }
            }
        };
        CrashReporter.prototype.doStart = function (rawConfiguration) {
            var config = this.toConfiguration(rawConfiguration);
            electron_1.crashReporter.start(config);
            //notify the main process to start the crash reporter
            electron_1.ipcRenderer.send('vscode:startCrashReporter', config);
        };
        CrashReporter.prototype.toConfiguration = function (rawConfiguration) {
            var _this = this;
            return JSON.parse(JSON.stringify(rawConfiguration, function (key, value) {
                if (value === '$(sessionId)') {
                    return _this.sessionId;
                }
                if (value === '$(version)') {
                    return _this.version;
                }
                if (value === '$(commit)') {
                    return _this.commit;
                }
                return value;
            }));
        };
        CrashReporter = __decorate([
            __param(2, telemetry_1.ITelemetryService),
            __param(3, configuration_1.IConfigurationService)
        ], CrashReporter);
        return CrashReporter;
    }());
    exports.CrashReporter = CrashReporter;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/node/userSettings", ["require", "exports", 'fs', 'path', 'vs/base/common/json', 'vs/base/common/objects', 'vs/base/common/winjs.base', 'vs/base/common/event'], function (require, exports, fs, path, json, objects, winjs_base_1, event_1) {
    'use strict';
    var UserSettings = (function () {
        function UserSettings(appSettingsPath, appKeybindingsPath) {
            this.appSettingsPath = appSettingsPath;
            this.appKeybindingsPath = appKeybindingsPath;
            this._onChange = new event_1.Emitter();
            this.registerWatchers();
        }
        UserSettings.getValue = function (contextService, key, fallback) {
            return new winjs_base_1.TPromise(function (c, e) {
                var appSettingsPath = contextService.getConfiguration().env.appSettingsPath;
                fs.readFile(appSettingsPath, function (error /* ignore */, fileContents) {
                    var root = Object.create(null);
                    var content = fileContents ? fileContents.toString() : '{}';
                    var contents = Object.create(null);
                    try {
                        contents = json.parse(content);
                    }
                    catch (error) {
                    }
                    for (var key_1 in contents) {
                        UserSettings.setNode(root, key_1, contents[key_1]);
                    }
                    return c(UserSettings.doGetValue(root, key, fallback));
                });
            });
        };
        Object.defineProperty(UserSettings.prototype, "onChange", {
            get: function () {
                return this._onChange.event;
            },
            enumerable: true,
            configurable: true
        });
        UserSettings.prototype.getValue = function (key, fallback) {
            return UserSettings.doGetValue(this.globalSettings.settings, key, fallback);
        };
        UserSettings.doGetValue = function (globalSettings, key, fallback) {
            if (!key) {
                return fallback;
            }
            var value = globalSettings;
            var parts = key.split('\.');
            while (parts.length && value) {
                var part = parts.shift();
                value = value[part];
            }
            return typeof value !== 'undefined' ? value : fallback;
        };
        UserSettings.prototype.registerWatchers = function () {
            var _this = this;
            this.watcher = fs.watch(path.dirname(this.appSettingsPath));
            this.watcher.on('change', function (eventType, fileName) { return _this.onSettingsFileChange(eventType, fileName); });
        };
        UserSettings.prototype.onSettingsFileChange = function (eventType, fileName) {
            var _this = this;
            // we can get multiple change events for one change, so we buffer through a timeout
            if (this.timeoutHandle) {
                global.clearTimeout(this.timeoutHandle);
                this.timeoutHandle = null;
            }
            this.timeoutHandle = global.setTimeout(function () {
                // Reload
                var didChange = _this.loadSync();
                // Emit event
                if (didChange) {
                    _this._onChange.fire(_this.globalSettings);
                }
            }, UserSettings.CHANGE_BUFFER_DELAY);
        };
        UserSettings.prototype.loadSync = function () {
            var loadedSettings = this.doLoadSync();
            if (!objects.equals(loadedSettings, this.globalSettings)) {
                // Keep in class
                this.globalSettings = loadedSettings;
                return true; // changed value
            }
            return false; // no changed value
        };
        UserSettings.prototype.doLoadSync = function () {
            var settings = this.doLoadSettingsSync();
            return {
                settings: settings.contents,
                settingsParseErrors: settings.parseErrors,
                keybindings: this.doLoadKeybindingsSync()
            };
        };
        UserSettings.prototype.doLoadSettingsSync = function () {
            var root = Object.create(null);
            var content = '{}';
            try {
                content = fs.readFileSync(this.appSettingsPath).toString();
            }
            catch (error) {
            }
            var contents = Object.create(null);
            try {
                contents = json.parse(content);
            }
            catch (error) {
                // parse problem
                return {
                    contents: Object.create(null),
                    parseErrors: [this.appSettingsPath]
                };
            }
            for (var key in contents) {
                UserSettings.setNode(root, key, contents[key]);
            }
            return {
                contents: root
            };
        };
        UserSettings.setNode = function (root, key, value) {
            var segments = key.split('.');
            var last = segments.pop();
            var curr = root;
            segments.forEach(function (s) {
                var obj = curr[s];
                switch (typeof obj) {
                    case 'undefined':
                        obj = curr[s] = {};
                        break;
                    case 'object':
                        break;
                    default:
                        console.log('Conflicting user settings: ' + key + ' at ' + s + ' with ' + JSON.stringify(obj));
                }
                curr = obj;
            });
            curr[last] = value;
        };
        UserSettings.prototype.doLoadKeybindingsSync = function () {
            try {
                return json.parse(fs.readFileSync(this.appKeybindingsPath).toString());
            }
            catch (error) {
            }
            return [];
        };
        UserSettings.prototype.dispose = function () {
            if (this.watcher) {
                this.watcher.close();
                this.watcher = null;
            }
        };
        UserSettings.CHANGE_BUFFER_DELAY = 300;
        return UserSettings;
    }());
    exports.UserSettings = UserSettings;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/contentprovider/common/contentprovider.contribution", ["require", "exports", 'vs/workbench/common/editor/resourceEditorInput', 'vs/editor/common/services/modelService', 'vs/editor/common/services/modeService', 'vs/base/common/winjs.base', 'vs/platform/jsonschemas/common/jsonContributionRegistry', 'vs/platform/platform', 'vs/workbench/common/contributions'], function (require, exports, resourceEditorInput_1, modelService_1, modeService_1, winjs_base_1, JSONContributionRegistry, platform_1, contributions_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var schemaRegistry = platform_1.Registry.as(JSONContributionRegistry.Extensions.JSONContribution);
    var WorkbenchContentProvider = (function () {
        function WorkbenchContentProvider(modelService, modeService) {
            this.modelService = modelService;
            this.modeService = modeService;
            this.start();
        }
        WorkbenchContentProvider.prototype.getId = function () {
            return 'vs.contentprovider';
        };
        WorkbenchContentProvider.prototype.start = function () {
            var _this = this;
            resourceEditorInput_1.ResourceEditorInput.registerResourceContentProvider('vscode', {
                provideTextContent: function (uri) {
                    if (uri.scheme !== 'vscode') {
                        return null;
                    }
                    if (uri.authority === 'schemas') {
                        var schemas = schemaRegistry.getSchemaContributions().schemas;
                        var schema = schemas[uri.toString()];
                        if (schema) {
                            var modelContent = JSON.stringify(schema);
                            var mode = _this.modeService.getOrCreateMode('json');
                            return winjs_base_1.TPromise.as(_this.modelService.createModel(modelContent, mode, uri));
                        }
                    }
                    return null;
                }
            });
        };
        WorkbenchContentProvider = __decorate([
            __param(0, modelService_1.IModelService),
            __param(1, modeService_1.IModeService)
        ], WorkbenchContentProvider);
        return WorkbenchContentProvider;
    }());
    exports.WorkbenchContentProvider = WorkbenchContentProvider;
    platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(WorkbenchContentProvider);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/debug/browser/debugEditorInputs", ["require", "exports", 'vs/base/common/mime', 'vs/workbench/common/editor/stringEditorInput', 'vs/platform/instantiation/common/instantiation'], function (require, exports, mime, strinput, instantiation_1) {
    "use strict";
    var DebugStringEditorInput = (function (_super) {
        __extends(DebugStringEditorInput, _super);
        function DebugStringEditorInput(name, resourceUrl, description, value, mimeType, singleton, instantiationService) {
            _super.call(this, name, description, value, mimeType || mime.MIME_TEXT, singleton, instantiationService);
            this.resourceUrl = resourceUrl;
        }
        DebugStringEditorInput.prototype.getResource = function () {
            return this.resourceUrl;
        };
        DebugStringEditorInput = __decorate([
            __param(6, instantiation_1.IInstantiationService)
        ], DebugStringEditorInput);
        return DebugStringEditorInput;
    }(strinput.StringEditorInput));
    exports.DebugStringEditorInput = DebugStringEditorInput;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/debug/common/debug", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    "use strict";
    exports.VIEWLET_ID = 'workbench.view.debug';
    exports.REPL_ID = 'workbench.panel.repl';
    exports.DEBUG_SERVICE_ID = 'debugService';
    exports.CONTEXT_IN_DEBUG_MODE = 'inDebugMode';
    exports.EDITOR_CONTRIBUTION_ID = 'editor.contrib.debug';
    // events
    exports.ModelEvents = {
        BREAKPOINTS_UPDATED: 'BreakpointsUpdated',
        CALLSTACK_UPDATED: 'CallStackUpdated',
        WATCH_EXPRESSIONS_UPDATED: 'WatchExpressionsUpdated',
        REPL_ELEMENTS_UPDATED: 'ReplElementsUpdated'
    };
    exports.ViewModelEvents = {
        FOCUSED_STACK_FRAME_UPDATED: 'FocusedStackFrameUpdated',
        SELECTED_EXPRESSION_UPDATED: 'SelectedExpressionUpdated',
        SELECTED_FUNCTION_BREAKPOINT_UPDATED: 'SelectedFunctionBreakpointUpdated'
    };
    exports.ServiceEvents = {
        STATE_CHANGED: 'StateChanged',
        TYPE_NOT_SUPPORTED: 'TypeNotSupported'
    };
    exports.SessionEvents = {
        INITIALIZED: 'initialized',
        STOPPED: 'stopped',
        DEBUGEE_TERMINATED: 'terminated',
        SERVER_EXIT: 'exit',
        CONTINUED: 'continued',
        THREAD: 'thread',
        OUTPUT: 'output',
        BREAKPOINT: 'breakpoint'
    };
    // service enums
    (function (State) {
        State[State["Disabled"] = 0] = "Disabled";
        State[State["Inactive"] = 1] = "Inactive";
        State[State["Initializing"] = 2] = "Initializing";
        State[State["Stopped"] = 3] = "Stopped";
        State[State["Running"] = 4] = "Running";
        State[State["RunningNoDebug"] = 5] = "RunningNoDebug";
    })(exports.State || (exports.State = {}));
    var State = exports.State;
    exports.IDebugService = instantiation_1.createDecorator(exports.DEBUG_SERVICE_ID);
    // utils
    var _formatPIIRegexp = /{([^}]+)}/g;
    function formatPII(value, excludePII, args) {
        return value.replace(_formatPIIRegexp, function (match, group) {
            if (excludePII && group.length > 0 && group[0] !== '_') {
                return match;
            }
            return args && args.hasOwnProperty(group) ?
                args[group] :
                match;
        });
    }
    exports.formatPII = formatPII;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/debug/browser/breakpointWidget", ["require", "exports", 'vs/nls!vs/workbench/parts/debug/browser/breakpointWidget', 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/keyCodes', 'vs/base/common/platform', 'vs/base/common/lifecycle', 'vs/base/browser/dom', 'vs/base/browser/ui/inputbox/inputBox', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/editorCommon', 'vs/editor/contrib/zoneWidget/browser/zoneWidget', 'vs/platform/contextview/browser/contextView', 'vs/platform/keybinding/common/keybindingService', 'vs/workbench/parts/debug/common/debug', 'vs/css!../browser/media/breakpointWidget'], function (require, exports, nls, async, errors, keyCodes_1, platform, lifecycle, dom, inputBox_1, editorCommonExtensions_1, editorcommon, zoneWidget_1, contextView_1, keybindingService_1, debug) {
    "use strict";
    var $ = dom.emmet;
    var CONTEXT_BREAKPOINT_WIDGET_VISIBLE = 'breakpointWidgetVisible';
    var CLOSE_BREAKPOINT_WIDGET_COMMAND_ID = 'closeBreakpointWidget';
    var BreakpointWidget = (function (_super) {
        __extends(BreakpointWidget, _super);
        function BreakpointWidget(editor, lineNumber, contextViewService, debugService, keybindingService) {
            var _this = this;
            _super.call(this, editor, { showFrame: true, showArrow: false });
            this.lineNumber = lineNumber;
            this.contextViewService = contextViewService;
            this.debugService = debugService;
            this.toDispose = [];
            this.create();
            this.breakpointWidgetVisible = keybindingService.createKey(CONTEXT_BREAKPOINT_WIDGET_VISIBLE, false);
            this.breakpointWidgetVisible.set(true);
            BreakpointWidget.INSTANCE = this;
            this.toDispose.push(editor.addListener2(editorcommon.EventType.ModelChanged, function () { return _this.dispose(); }));
        }
        BreakpointWidget.prototype.fillContainer = function (container) {
            var _this = this;
            dom.addClass(container, 'breakpoint-widget');
            var uri = this.editor.getModel().getAssociatedResource();
            var breakpoint = this.debugService.getModel().getBreakpoints().filter(function (bp) { return bp.lineNumber === _this.lineNumber && bp.source.uri.toString() === uri.toString(); }).pop();
            var inputBoxContainer = dom.append(container, $('.inputBoxContainer'));
            this.inputBox = new inputBox_1.InputBox(inputBoxContainer, this.contextViewService, {
                placeholder: nls.localize(0, null, this.lineNumber),
                ariaLabel: nls.localize(1, null)
            });
            this.toDispose.push(this.inputBox);
            dom.addClass(this.inputBox.inputElement, platform.isWindows ? 'windows' : platform.isMacintosh ? 'mac' : 'linux');
            this.inputBox.value = (breakpoint && breakpoint.condition) ? breakpoint.condition : '';
            // Due to an electron bug we have to do the timeout, otherwise we do not get focus
            setTimeout(function () { return _this.inputBox.focus(); }, 0);
            var disposed = false;
            var wrapUp = async.once(function (success) {
                if (!disposed) {
                    disposed = true;
                    if (success) {
                        var raw = {
                            uri: uri,
                            lineNumber: _this.lineNumber,
                            enabled: true,
                            condition: _this.inputBox.value
                        };
                        // if there is already a breakpoint on this location - remove it.
                        if (_this.debugService.getModel().getBreakpoints().some(function (bp) { return bp.lineNumber === _this.lineNumber && bp.source.uri.toString() === uri.toString(); })) {
                            _this.debugService.toggleBreakpoint(raw).done(null, errors.onUnexpectedError);
                        }
                        _this.debugService.toggleBreakpoint(raw).done(null, errors.onUnexpectedError);
                    }
                    _this.dispose();
                }
            });
            this.toDispose.push(dom.addStandardDisposableListener(this.inputBox.inputElement, 'keydown', function (e) {
                var isEscape = e.equals(keyCodes_1.CommonKeybindings.ESCAPE);
                var isEnter = e.equals(keyCodes_1.CommonKeybindings.ENTER);
                if (isEscape || isEnter) {
                    e.stopPropagation();
                    wrapUp(isEnter);
                }
            }));
        };
        BreakpointWidget.prototype.dispose = function () {
            var _this = this;
            _super.prototype.dispose.call(this);
            this.breakpointWidgetVisible.reset();
            BreakpointWidget.INSTANCE = undefined;
            lifecycle.disposeAll(this.toDispose);
            setTimeout(function () { return _this.editor.focus(); }, 0);
        };
        BreakpointWidget = __decorate([
            __param(2, contextView_1.IContextViewService),
            __param(3, debug.IDebugService),
            __param(4, keybindingService_1.IKeybindingService)
        ], BreakpointWidget);
        return BreakpointWidget;
    }(zoneWidget_1.ZoneWidget));
    exports.BreakpointWidget = BreakpointWidget;
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorCommand(CLOSE_BREAKPOINT_WIDGET_COMMAND_ID, editorCommonExtensions_1.CommonEditorRegistry.commandWeight(8), { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, false, CONTEXT_BREAKPOINT_WIDGET_VISIBLE, function (ctx, editor, args) {
        if (BreakpointWidget.INSTANCE) {
            BreakpointWidget.INSTANCE.dispose();
        }
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/debug/browser/debugEditorModelManager", ["require", "exports", 'vs/nls!vs/workbench/parts/debug/browser/debugEditorModelManager', 'vs/base/common/objects', 'vs/base/common/lifecycle', 'vs/editor/common/editorCommon', 'vs/workbench/parts/debug/common/debug', 'vs/editor/common/services/modelService'], function (require, exports, nls, objects, lifecycle, editorcommon, debug_1, modelService_1) {
    "use strict";
    function toMap(arr) {
        var result = {};
        for (var i = 0, len = arr.length; i < len; i++) {
            result[arr[i]] = true;
        }
        return result;
    }
    function createRange(startLineNUmber, startColumn, endLineNumber, endColumn) {
        return {
            startLineNumber: startLineNUmber,
            startColumn: startColumn,
            endLineNumber: endLineNumber,
            endColumn: endColumn
        };
    }
    var DebugEditorModelManager = (function () {
        function DebugEditorModelManager(modelService, debugService) {
            this.modelService = modelService;
            this.debugService = debugService;
            this.modelData = {};
            this.toDispose = [];
            this.registerListeners();
        }
        DebugEditorModelManager.prototype.getId = function () {
            return DebugEditorModelManager.ID;
        };
        DebugEditorModelManager.prototype.dispose = function () {
            for (var modelUrlStr in this.modelData) {
                if (this.modelData.hasOwnProperty(modelUrlStr)) {
                    var modelData = this.modelData[modelUrlStr];
                    lifecycle.disposeAll(modelData.toDispose);
                    modelData.model.deltaDecorations(modelData.breakpointDecorationIds, []);
                    modelData.model.deltaDecorations(modelData.currentStackDecorations, []);
                }
            }
            this.toDispose = lifecycle.disposeAll(this.toDispose);
            this.modelData = null;
        };
        DebugEditorModelManager.prototype.registerListeners = function () {
            var _this = this;
            this.toDispose.push(this.modelService.onModelAdded(this.onModelAdded, this));
            this.modelService.getModels().forEach(function (model) { return _this.onModelAdded(model); });
            this.toDispose.push(this.modelService.onModelRemoved(this.onModelRemoved, this));
            this.toDispose.push(this.debugService.getModel().addListener2(debug_1.ModelEvents.BREAKPOINTS_UPDATED, function () { return _this.onBreakpointsChanged(); }));
            this.toDispose.push(this.debugService.getViewModel().addListener2(debug_1.ViewModelEvents.FOCUSED_STACK_FRAME_UPDATED, function () { return _this.onFocusedStackFrameUpdated(); }));
            this.toDispose.push(this.debugService.addListener2(debug_1.ServiceEvents.STATE_CHANGED, function () {
                if (_this.debugService.getState() === debug_1.State.Inactive) {
                    Object.keys(_this.modelData).forEach(function (key) { return _this.modelData[key].dirty = false; });
                }
            }));
        };
        DebugEditorModelManager.prototype.onModelAdded = function (model) {
            var _this = this;
            var modelUrlStr = model.getAssociatedResource().toString();
            var breakpoints = this.debugService.getModel().getBreakpoints().filter(function (bp) { return bp.source.uri.toString() === modelUrlStr; });
            var currentStackDecorations = model.deltaDecorations([], this.createCallStackDecorations(modelUrlStr));
            var breakPointDecorations = model.deltaDecorations([], this.createBreakpointDecorations(breakpoints));
            var toDispose = [model.addListener2(editorcommon.EventType.ModelDecorationsChanged, function (e) {
                    return _this.onModelDecorationsChanged(modelUrlStr, e);
                })];
            this.modelData[modelUrlStr] = {
                model: model,
                toDispose: toDispose,
                breakpointDecorationIds: breakPointDecorations,
                breakpointLines: breakpoints.map(function (bp) { return bp.lineNumber; }),
                breakpointDecorationsAsMap: toMap(breakPointDecorations),
                currentStackDecorations: currentStackDecorations,
                topStackFrameRange: null,
                dirty: false
            };
        };
        DebugEditorModelManager.prototype.onModelRemoved = function (model) {
            var modelUrlStr = model.getAssociatedResource().toString();
            if (this.modelData.hasOwnProperty(modelUrlStr)) {
                var modelData = this.modelData[modelUrlStr];
                delete this.modelData[modelUrlStr];
                lifecycle.disposeAll(modelData.toDispose);
            }
        };
        // call stack management. Represent data coming from the debug service.
        DebugEditorModelManager.prototype.onFocusedStackFrameUpdated = function () {
            var _this = this;
            Object.keys(this.modelData).forEach(function (modelUrlStr) {
                var modelData = _this.modelData[modelUrlStr];
                modelData.currentStackDecorations = modelData.model.deltaDecorations(modelData.currentStackDecorations, _this.createCallStackDecorations(modelUrlStr));
            });
        };
        DebugEditorModelManager.prototype.createCallStackDecorations = function (modelUrlStr) {
            var _this = this;
            var result = [];
            var focusedStackFrame = this.debugService.getViewModel().getFocusedStackFrame();
            var allThreads = this.debugService.getModel().getThreads();
            if (!focusedStackFrame || !allThreads[focusedStackFrame.threadId] || !allThreads[focusedStackFrame.threadId].getCachedCallStack()) {
                return result;
            }
            // only show decorations for the currently focussed thread.
            var thread = allThreads[focusedStackFrame.threadId];
            thread.getCachedCallStack().filter(function (sf) { return sf.source.uri.toString() === modelUrlStr; }).forEach(function (sf) {
                var wholeLineRange = createRange(sf.lineNumber, sf.column, sf.lineNumber, Number.MAX_VALUE);
                // compute how to decorate the editor. Different decorations are used if this is a top stack frame, focussed stack frame,
                // an exception or a stack frame that did not change the line number (we only decorate the columns, not the whole line).
                if (sf === thread.getCachedCallStack()[0]) {
                    result.push({
                        options: DebugEditorModelManager.TOP_STACK_FRAME_MARGIN,
                        range: createRange(sf.lineNumber, sf.column, sf.lineNumber, sf.column + 1)
                    });
                    if (thread.stoppedDetails.reason === 'exception') {
                        result.push({
                            options: DebugEditorModelManager.TOP_STACK_FRAME_EXCEPTION_DECORATION,
                            range: wholeLineRange
                        });
                    }
                    else {
                        result.push({
                            options: DebugEditorModelManager.TOP_STACK_FRAME_DECORATION,
                            range: wholeLineRange
                        });
                        if (_this.modelData[modelUrlStr]) {
                            if (_this.modelData[modelUrlStr].topStackFrameRange && _this.modelData[modelUrlStr].topStackFrameRange.startLineNumber === wholeLineRange.startLineNumber &&
                                _this.modelData[modelUrlStr].topStackFrameRange.startColumn !== wholeLineRange.startColumn) {
                                result.push({
                                    options: DebugEditorModelManager.TOP_STACK_FRAME_COLUMN_DECORATION,
                                    range: wholeLineRange
                                });
                            }
                            _this.modelData[modelUrlStr].topStackFrameRange = wholeLineRange;
                        }
                    }
                }
                else if (sf === focusedStackFrame) {
                    result.push({
                        options: DebugEditorModelManager.FOCUSED_STACK_FRAME_MARGIN,
                        range: createRange(sf.lineNumber, sf.column, sf.lineNumber, sf.column + 1)
                    });
                    result.push({
                        options: DebugEditorModelManager.FOCUSED_STACK_FRAME_DECORATION,
                        range: wholeLineRange
                    });
                }
            });
            return result;
        };
        // breakpoints management. Represent data coming from the debug service and also send data back.
        DebugEditorModelManager.prototype.onModelDecorationsChanged = function (modelUrlStr, e) {
            var modelData = this.modelData[modelUrlStr];
            if (!e.addedOrChangedDecorations.some(function (d) { return modelData.breakpointDecorationsAsMap.hasOwnProperty(d.id); })) {
                // nothing to do, my decorations did not change.
                return;
            }
            var data = [];
            var enabledAndConditions = {};
            this.debugService.getModel().getBreakpoints().filter(function (bp) { return bp.source.uri.toString() === modelUrlStr; }).forEach(function (bp) {
                enabledAndConditions[bp.lineNumber] = {
                    enabled: bp.enabled,
                    condition: bp.condition
                };
            });
            var modelUrl = modelData.model.getAssociatedResource();
            for (var i = 0, len = modelData.breakpointDecorationIds.length; i < len; i++) {
                var decorationRange = modelData.model.getDecorationRange(modelData.breakpointDecorationIds[i]);
                // check if the line got deleted.
                if (decorationRange.endColumn - decorationRange.startColumn > 0) {
                    // since we know it is collapsed, it cannot grow to multiple lines
                    data.push({
                        uri: modelUrl,
                        lineNumber: decorationRange.startLineNumber,
                        enabled: enabledAndConditions[modelData.breakpointLines[i]].enabled,
                        condition: enabledAndConditions[modelData.breakpointLines[i]].condition
                    });
                }
            }
            modelData.dirty = !!this.debugService.getActiveSession();
            this.debugService.setBreakpointsForModel(modelUrl, data);
        };
        DebugEditorModelManager.prototype.onBreakpointsChanged = function () {
            var _this = this;
            var breakpointsMap = {};
            this.debugService.getModel().getBreakpoints().forEach(function (bp) {
                var uriStr = bp.source.uri.toString();
                if (breakpointsMap[uriStr]) {
                    breakpointsMap[uriStr].push(bp);
                }
                else {
                    breakpointsMap[uriStr] = [bp];
                }
            });
            Object.keys(breakpointsMap).forEach(function (modelUriStr) {
                if (_this.modelData.hasOwnProperty(modelUriStr)) {
                    _this.updateBreakpoints(_this.modelData[modelUriStr], breakpointsMap[modelUriStr]);
                }
            });
            Object.keys(this.modelData).forEach(function (modelUriStr) {
                if (!breakpointsMap.hasOwnProperty(modelUriStr)) {
                    _this.updateBreakpoints(_this.modelData[modelUriStr], []);
                }
            });
        };
        DebugEditorModelManager.prototype.updateBreakpoints = function (modelData, newBreakpoints) {
            modelData.breakpointDecorationIds = modelData.model.deltaDecorations(modelData.breakpointDecorationIds, this.createBreakpointDecorations(newBreakpoints));
            modelData.breakpointDecorationsAsMap = toMap(modelData.breakpointDecorationIds);
            modelData.breakpointLines = newBreakpoints.map(function (bp) { return bp.lineNumber; });
        };
        DebugEditorModelManager.prototype.createBreakpointDecorations = function (breakpoints) {
            var _this = this;
            return breakpoints.map(function (breakpoint) {
                return {
                    options: _this.getBreakpointDecorationOptions(breakpoint),
                    range: createRange(breakpoint.lineNumber, 1, breakpoint.lineNumber, 2)
                };
            });
        };
        DebugEditorModelManager.prototype.getBreakpointDecorationOptions = function (breakpoint) {
            var activated = this.debugService.getModel().areBreakpointsActivated();
            var state = this.debugService.getState();
            var debugActive = state === debug_1.State.Running || state === debug_1.State.Stopped || state === debug_1.State.Initializing;
            var modelData = this.modelData[breakpoint.source.uri.toString()];
            var session = this.debugService.getActiveSession();
            var result = (!breakpoint.enabled || !activated) ? DebugEditorModelManager.BREAKPOINT_DISABLED_DECORATION :
                debugActive && modelData && modelData.dirty ? DebugEditorModelManager.BREAKPOINT_DIRTY_DECORATION :
                    debugActive && !breakpoint.verified ? DebugEditorModelManager.BREAKPOINT_UNVERIFIED_DECORATION :
                        !breakpoint.condition ? DebugEditorModelManager.BREAKPOINT_DECORATION : null;
            if (result && breakpoint.message) {
                result = objects.clone(result);
                result.hoverMessage = breakpoint.message;
            }
            return result ? result :
                !session || session.capabilities.supportsConditionalBreakpoints ? {
                    glyphMarginClassName: 'debug-breakpoint-conditional-glyph',
                    hoverMessage: breakpoint.condition,
                    stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
                } : DebugEditorModelManager.BREAKPOINT_UNSUPPORTED_DECORATION;
        };
        DebugEditorModelManager.ID = 'breakpointManager';
        // editor decorations
        DebugEditorModelManager.BREAKPOINT_DECORATION = {
            glyphMarginClassName: 'debug-breakpoint-glyph',
            hoverMessage: nls.localize(0, null),
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorModelManager.BREAKPOINT_DISABLED_DECORATION = {
            glyphMarginClassName: 'debug-breakpoint-disabled-glyph',
            hoverMessage: nls.localize(1, null),
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorModelManager.BREAKPOINT_UNVERIFIED_DECORATION = {
            glyphMarginClassName: 'debug-breakpoint-unverified-glyph',
            hoverMessage: nls.localize(2, null),
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorModelManager.BREAKPOINT_DIRTY_DECORATION = {
            glyphMarginClassName: 'debug-breakpoint-unverified-glyph',
            hoverMessage: nls.localize(3, null),
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorModelManager.BREAKPOINT_UNSUPPORTED_DECORATION = {
            glyphMarginClassName: 'debug-breakpoint-unsupported-glyph',
            hoverMessage: nls.localize(4, null),
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        // we need a separate decoration for glyph margin, since we do not want it on each line of a multi line statement.
        DebugEditorModelManager.TOP_STACK_FRAME_MARGIN = {
            glyphMarginClassName: 'debug-top-stack-frame-glyph',
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorModelManager.FOCUSED_STACK_FRAME_MARGIN = {
            glyphMarginClassName: 'debug-focused-stack-frame-glyph',
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorModelManager.TOP_STACK_FRAME_DECORATION = {
            isWholeLine: true,
            className: 'debug-top-stack-frame-line',
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorModelManager.TOP_STACK_FRAME_EXCEPTION_DECORATION = {
            isWholeLine: true,
            className: 'debug-top-stack-frame-exception-line',
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorModelManager.TOP_STACK_FRAME_COLUMN_DECORATION = {
            isWholeLine: false,
            className: 'debug-top-stack-frame-column',
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorModelManager.FOCUSED_STACK_FRAME_DECORATION = {
            isWholeLine: true,
            className: 'debug-focused-stack-frame-line',
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorModelManager = __decorate([
            __param(0, modelService_1.IModelService),
            __param(1, debug_1.IDebugService)
        ], DebugEditorModelManager);
        return DebugEditorModelManager;
    }());
    exports.DebugEditorModelManager = DebugEditorModelManager;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/debug/common/debugSource", ["require", "exports", 'vs/base/common/uri', 'vs/base/common/paths'], function (require, exports, uri_1, paths) {
    "use strict";
    var Source = (function () {
        function Source(raw, available) {
            if (available === void 0) { available = true; }
            this.raw = raw;
            this.uri = raw.path ? uri_1.default.file(raw.path) : uri_1.default.parse(Source.INTERNAL_URI_PREFIX + raw.name);
            this.available = available;
        }
        Object.defineProperty(Source.prototype, "name", {
            get: function () {
                return this.raw.name;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Source.prototype, "origin", {
            get: function () {
                return this.raw.origin;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Source.prototype, "reference", {
            get: function () {
                return this.raw.sourceReference;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Source.prototype, "inMemory", {
            get: function () {
                return Source.isInMemory(this.uri);
            },
            enumerable: true,
            configurable: true
        });
        Source.toRawSource = function (uri, model) {
            if (model) {
                // first try to find the raw source amongst the stack frames - since that represenation has more data (source reference),
                var threads = model.getThreads();
                for (var threadId in threads) {
                    if (threads.hasOwnProperty(threadId) && threads[threadId].getCachedCallStack()) {
                        var found = threads[threadId].getCachedCallStack().filter(function (sf) { return sf.source.uri.toString() === uri.toString(); }).pop();
                        if (found) {
                            return found.source.raw;
                        }
                    }
                }
            }
            // did not find the raw source amongst the stack frames, construct the raw stack frame from the limited data you have.
            return Source.isInMemory(uri) ? { name: Source.getName(uri) } :
                { path: paths.normalize(uri.fsPath, true) };
        };
        Source.getName = function (uri) {
            var uriStr = uri.toString();
            return uriStr.substr(uriStr.lastIndexOf('/') + 1);
        };
        Source.isInMemory = function (uri) {
            return uri.toString().indexOf(Source.INTERNAL_URI_PREFIX) === 0;
        };
        Source.INTERNAL_URI_PREFIX = 'debug://internal/';
        return Source;
    }());
    exports.Source = Source;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/debug/common/debugModel", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/strings', 'vs/nls!vs/workbench/parts/debug/common/debugModel', 'vs/base/common/lifecycle', 'vs/base/common/eventEmitter', 'vs/base/common/uuid', 'vs/base/common/types', 'vs/base/common/arrays', 'vs/workbench/parts/debug/common/debug', 'vs/base/common/errors', 'vs/workbench/parts/debug/common/debugSource'], function (require, exports, winjs_base_1, strings, nls, lifecycle, ee, uuid, types, arrays, debug, errors, debugSource_1) {
    "use strict";
    var MAX_REPL_LENGTH = 10000;
    function resolveChildren(debugService, parent) {
        var session = debugService.getActiveSession();
        // only variables with reference > 0 have children.
        if (!session || parent.reference <= 0) {
            return winjs_base_1.TPromise.as([]);
        }
        return session.variables({ variablesReference: parent.reference }).then(function (response) {
            return arrays.distinct(response.body.variables.filter(function (v) { return !!v; }), function (v) { return v.name; }).map(function (v) { return new Variable(parent, v.variablesReference, v.name, v.value); });
        }, function (e) { return [new Variable(parent, 0, null, e.message, false)]; });
    }
    function massageValue(value) {
        return value ? value.replace(/\n/g, '\\n').replace(/\r/g, '\\r').replace(/\t/g, '\\t') : value;
    }
    function evaluateExpression(session, stackFrame, expression, context) {
        if (!session) {
            expression.value = context === 'repl' ? nls.localize(0, null) : Expression.DEFAULT_VALUE;
            expression.available = false;
            expression.reference = 0;
            return winjs_base_1.TPromise.as(expression);
        }
        return session.evaluate({
            expression: expression.name,
            frameId: stackFrame ? stackFrame.frameId : undefined,
            context: context
        }).then(function (response) {
            expression.value = response.body.result;
            expression.available = true;
            expression.reference = response.body.variablesReference;
            return expression;
        }, function (err) {
            expression.value = err.message;
            expression.available = false;
            expression.reference = 0;
            return expression;
        });
    }
    exports.evaluateExpression = evaluateExpression;
    var notPropertySyntax = /^[a-zA-Z_][a-zA-Z0-9_]*$/;
    var arrayElementSyntax = /\[.*\]$/;
    function getFullExpressionName(expression, sessionType) {
        var names = [expression.name];
        if (expression instanceof Variable) {
            var v = expression.parent;
            while (v instanceof Variable || v instanceof Expression) {
                names.push(v.name);
                v = v.parent;
            }
        }
        names = names.reverse();
        var result = null;
        names.forEach(function (name) {
            if (!result) {
                result = name;
            }
            else if (arrayElementSyntax.test(name) || (sessionType === 'node' && !notPropertySyntax.test(name))) {
                // use safe way to access node properties a['property_name']. Also handles array elements.
                result = name && name.indexOf('[') === 0 ? "" + result + name : result + "['" + name + "']";
            }
            else {
                result = result + "." + name;
            }
        });
        return result;
    }
    exports.getFullExpressionName = getFullExpressionName;
    var Thread = (function () {
        function Thread(name, threadId) {
            this.name = name;
            this.threadId = threadId;
            this.promisedCallStack = undefined;
            this.stoppedDetails = undefined;
            this.cachedCallStack = undefined;
            this.stopped = false;
        }
        Thread.prototype.getId = function () {
            return "thread:" + this.name + ":" + this.threadId;
        };
        Thread.prototype.clearCallStack = function () {
            this.promisedCallStack = undefined;
            this.cachedCallStack = undefined;
        };
        Thread.prototype.getCachedCallStack = function () {
            return this.cachedCallStack;
        };
        Thread.prototype.getCallStack = function (debugService) {
            var _this = this;
            if (!this.stopped) {
                return winjs_base_1.TPromise.as([]);
            }
            if (!this.promisedCallStack) {
                this.promisedCallStack = this.getCallStackImpl(debugService);
                this.promisedCallStack.then(function (result) {
                    _this.cachedCallStack = result;
                }, errors.onUnexpectedError);
            }
            return this.promisedCallStack;
        };
        Thread.prototype.getCallStackImpl = function (debugService) {
            var _this = this;
            var session = debugService.getActiveSession();
            return session.stackTrace({ threadId: this.threadId, levels: 20 }).then(function (response) {
                return response.body.stackFrames.map(function (rsf, level) {
                    if (!rsf) {
                        return new StackFrame(_this.threadId, 0, new debugSource_1.Source({ name: 'unknown' }, false), nls.localize(1, null), undefined, undefined);
                    }
                    return new StackFrame(_this.threadId, rsf.id, rsf.source ? new debugSource_1.Source(rsf.source) : new debugSource_1.Source({ name: 'unknown' }, false), rsf.name, rsf.line, rsf.column);
                });
            });
        };
        return Thread;
    }());
    exports.Thread = Thread;
    var OutputElement = (function () {
        function OutputElement(id) {
            if (id === void 0) { id = uuid.generateUuid(); }
            this.id = id;
            // noop
        }
        OutputElement.prototype.getId = function () {
            return this.id;
        };
        return OutputElement;
    }());
    exports.OutputElement = OutputElement;
    var ValueOutputElement = (function (_super) {
        __extends(ValueOutputElement, _super);
        function ValueOutputElement(value, severity, category, counter) {
            if (counter === void 0) { counter = 1; }
            _super.call(this);
            this.value = value;
            this.severity = severity;
            this.category = category;
            this.counter = counter;
        }
        return ValueOutputElement;
    }(OutputElement));
    exports.ValueOutputElement = ValueOutputElement;
    var KeyValueOutputElement = (function (_super) {
        __extends(KeyValueOutputElement, _super);
        function KeyValueOutputElement(key, valueObj, annotation) {
            _super.call(this);
            this.key = key;
            this.valueObj = valueObj;
            this.annotation = annotation;
            this._valueName = null;
        }
        Object.defineProperty(KeyValueOutputElement.prototype, "value", {
            get: function () {
                if (this._valueName === null) {
                    if (this.valueObj === null) {
                        this._valueName = 'null';
                    }
                    else if (Array.isArray(this.valueObj)) {
                        this._valueName = "Array[" + this.valueObj.length + "]";
                    }
                    else if (types.isObject(this.valueObj)) {
                        this._valueName = 'Object';
                    }
                    else if (types.isString(this.valueObj)) {
                        this._valueName = "\"" + massageValue(this.valueObj) + "\"";
                    }
                    else {
                        this._valueName = String(this.valueObj);
                    }
                    if (!this._valueName) {
                        this._valueName = '';
                    }
                }
                return this._valueName;
            },
            enumerable: true,
            configurable: true
        });
        KeyValueOutputElement.prototype.getChildren = function () {
            var _this = this;
            if (!this.children) {
                if (Array.isArray(this.valueObj)) {
                    this.children = this.valueObj.slice(0, KeyValueOutputElement.MAX_CHILDREN).map(function (v, index) { return new KeyValueOutputElement(String(index), v, null); });
                }
                else if (types.isObject(this.valueObj)) {
                    this.children = Object.getOwnPropertyNames(this.valueObj).slice(0, KeyValueOutputElement.MAX_CHILDREN).map(function (key) { return new KeyValueOutputElement(key, _this.valueObj[key], null); });
                }
                else {
                    this.children = [];
                }
            }
            return this.children;
        };
        KeyValueOutputElement.MAX_CHILDREN = 1000; // upper bound of children per value
        return KeyValueOutputElement;
    }(OutputElement));
    exports.KeyValueOutputElement = KeyValueOutputElement;
    var ExpressionContainer = (function () {
        function ExpressionContainer(reference, id, cacheChildren) {
            this.reference = reference;
            this.id = id;
            this.cacheChildren = cacheChildren;
            this.children = null;
        }
        ExpressionContainer.prototype.getChildren = function (debugService) {
            if (!this.cacheChildren) {
                return resolveChildren(debugService, this);
            }
            if (!this.children) {
                this.children = resolveChildren(debugService, this);
            }
            return this.children;
        };
        ExpressionContainer.prototype.getId = function () {
            return this.id;
        };
        ExpressionContainer.allValues = {};
        return ExpressionContainer;
    }());
    exports.ExpressionContainer = ExpressionContainer;
    var Expression = (function (_super) {
        __extends(Expression, _super);
        function Expression(name, cacheChildren, id) {
            if (id === void 0) { id = uuid.generateUuid(); }
            _super.call(this, 0, id, cacheChildren);
            this.name = name;
            this.value = Expression.DEFAULT_VALUE;
            this.available = false;
        }
        Object.defineProperty(Expression.prototype, "value", {
            get: function () {
                return this._value;
            },
            set: function (value) {
                this._value = massageValue(value);
                this.valueChanged = ExpressionContainer.allValues[this.getId()] &&
                    ExpressionContainer.allValues[this.getId()] !== Expression.DEFAULT_VALUE && ExpressionContainer.allValues[this.getId()] !== value;
                ExpressionContainer.allValues[this.getId()] = value;
            },
            enumerable: true,
            configurable: true
        });
        Expression.DEFAULT_VALUE = 'not available';
        return Expression;
    }(ExpressionContainer));
    exports.Expression = Expression;
    var Variable = (function (_super) {
        __extends(Variable, _super);
        function Variable(parent, reference, name, value, available) {
            if (available === void 0) { available = true; }
            _super.call(this, reference, "variable:" + parent.getId() + ":" + name, true);
            this.parent = parent;
            this.name = name;
            this.available = available;
            this.value = massageValue(value);
            this.valueChanged = ExpressionContainer.allValues[this.getId()] && ExpressionContainer.allValues[this.getId()] !== value;
            ExpressionContainer.allValues[this.getId()] = value;
        }
        return Variable;
    }(ExpressionContainer));
    exports.Variable = Variable;
    var Scope = (function () {
        function Scope(threadId, name, reference, expensive) {
            this.threadId = threadId;
            this.name = name;
            this.reference = reference;
            this.expensive = expensive;
            this.children = null;
        }
        Scope.prototype.getId = function () {
            return "scope:" + this.threadId + ":" + this.name + ":" + this.reference;
        };
        Scope.prototype.getChildren = function (debugService) {
            if (!this.children) {
                this.children = resolveChildren(debugService, this);
            }
            return this.children;
        };
        return Scope;
    }());
    exports.Scope = Scope;
    var StackFrame = (function () {
        function StackFrame(threadId, frameId, source, name, lineNumber, column) {
            this.threadId = threadId;
            this.frameId = frameId;
            this.source = source;
            this.name = name;
            this.lineNumber = lineNumber;
            this.column = column;
            this.internalId = uuid.generateUuid();
            this.scopes = null;
        }
        StackFrame.prototype.getId = function () {
            return this.internalId;
        };
        StackFrame.prototype.getScopes = function (debugService) {
            var _this = this;
            if (!this.scopes) {
                this.scopes = debugService.getActiveSession().scopes({ frameId: this.frameId }).then(function (response) {
                    return response.body.scopes.map(function (rs) { return new Scope(_this.threadId, rs.name, rs.variablesReference, rs.expensive); });
                }, function (err) { return []; });
            }
            return this.scopes;
        };
        return StackFrame;
    }());
    exports.StackFrame = StackFrame;
    var Breakpoint = (function () {
        function Breakpoint(source, desiredLineNumber, enabled, condition) {
            this.source = source;
            this.desiredLineNumber = desiredLineNumber;
            this.enabled = enabled;
            this.condition = condition;
            if (enabled === undefined) {
                this.enabled = true;
            }
            this.lineNumber = this.desiredLineNumber;
            this.verified = false;
            this.id = uuid.generateUuid();
        }
        Breakpoint.prototype.getId = function () {
            return this.id;
        };
        return Breakpoint;
    }());
    exports.Breakpoint = Breakpoint;
    var FunctionBreakpoint = (function () {
        function FunctionBreakpoint(name, enabled) {
            this.name = name;
            this.enabled = enabled;
            this.verified = false;
            this.id = uuid.generateUuid();
        }
        FunctionBreakpoint.prototype.getId = function () {
            return this.id;
        };
        return FunctionBreakpoint;
    }());
    exports.FunctionBreakpoint = FunctionBreakpoint;
    var ExceptionBreakpoint = (function () {
        function ExceptionBreakpoint(filter, label, enabled) {
            this.filter = filter;
            this.label = label;
            this.enabled = enabled;
            this.id = uuid.generateUuid();
        }
        ExceptionBreakpoint.prototype.getId = function () {
            return this.id;
        };
        return ExceptionBreakpoint;
    }());
    exports.ExceptionBreakpoint = ExceptionBreakpoint;
    var Model = (function (_super) {
        __extends(Model, _super);
        function Model(breakpoints, breakpointsActivated, functionBreakpoints, exceptionBreakpoints, watchExpressions) {
            _super.call(this);
            this.breakpoints = breakpoints;
            this.breakpointsActivated = breakpointsActivated;
            this.functionBreakpoints = functionBreakpoints;
            this.exceptionBreakpoints = exceptionBreakpoints;
            this.watchExpressions = watchExpressions;
            this.threads = {};
            this.replElements = [];
            this.toDispose = [];
        }
        Model.prototype.getId = function () {
            return 'root';
        };
        Model.prototype.getThreads = function () {
            return this.threads;
        };
        Model.prototype.clearThreads = function (removeThreads, reference) {
            if (reference === void 0) { reference = undefined; }
            if (reference) {
                if (removeThreads) {
                    delete this.threads[reference];
                }
                else {
                    this.threads[reference].clearCallStack();
                    this.threads[reference].stoppedDetails = undefined;
                }
            }
            else {
                if (removeThreads) {
                    this.threads = {};
                    ExpressionContainer.allValues = {};
                }
                else {
                    for (var ref in this.threads) {
                        if (this.threads.hasOwnProperty(ref)) {
                            this.threads[ref].clearCallStack();
                            this.threads[ref].stoppedDetails = undefined;
                        }
                    }
                }
            }
            this.emit(debug.ModelEvents.CALLSTACK_UPDATED);
        };
        Model.prototype.continueThreads = function () {
            for (var ref in this.threads) {
                if (this.threads.hasOwnProperty(ref)) {
                    this.threads[ref].stopped = false;
                }
            }
            this.clearThreads(false);
        };
        Model.prototype.getBreakpoints = function () {
            return this.breakpoints;
        };
        Model.prototype.getFunctionBreakpoints = function () {
            return this.functionBreakpoints;
        };
        Model.prototype.getExceptionBreakpoints = function () {
            return this.exceptionBreakpoints;
        };
        Model.prototype.setExceptionBreakpoints = function (data) {
            var _this = this;
            if (data) {
                this.exceptionBreakpoints = data.map(function (d) {
                    var ebp = _this.exceptionBreakpoints.filter(function (ebp) { return ebp.filter === d.filter; }).pop();
                    return new ExceptionBreakpoint(d.filter, d.label, ebp ? ebp.enabled : d.default);
                });
            }
        };
        Model.prototype.areBreakpointsActivated = function () {
            return this.breakpointsActivated;
        };
        Model.prototype.toggleBreakpointsActivated = function () {
            this.breakpointsActivated = !this.breakpointsActivated;
            this.emit(debug.ModelEvents.BREAKPOINTS_UPDATED);
        };
        Model.prototype.addBreakpoints = function (rawData) {
            var _this = this;
            this.breakpoints = this.breakpoints.concat(rawData.map(function (rawBp) {
                return new Breakpoint(new debugSource_1.Source(debugSource_1.Source.toRawSource(rawBp.uri, _this)), rawBp.lineNumber, rawBp.enabled, rawBp.condition);
            }));
            this.breakpointsActivated = true;
            this.emit(debug.ModelEvents.BREAKPOINTS_UPDATED);
        };
        Model.prototype.removeBreakpoints = function (toRemove) {
            this.breakpoints = this.breakpoints.filter(function (bp) { return !toRemove.some(function (toRemove) { return toRemove.getId() === bp.getId(); }); });
            this.emit(debug.ModelEvents.BREAKPOINTS_UPDATED);
        };
        Model.prototype.updateBreakpoints = function (data) {
            this.breakpoints.forEach(function (bp) {
                var bpData = data[bp.getId()];
                if (bpData) {
                    bp.lineNumber = bpData.line ? bpData.line : bp.lineNumber;
                    bp.verified = bpData.verified;
                    bp.idFromAdapter = bpData.id;
                    bp.message = bpData.message;
                }
            });
            this.emit(debug.ModelEvents.BREAKPOINTS_UPDATED);
        };
        Model.prototype.toggleEnablement = function (element) {
            element.enabled = !element.enabled;
            if (element instanceof Breakpoint && !element.enabled) {
                var breakpoint = element;
                breakpoint.lineNumber = breakpoint.desiredLineNumber;
                breakpoint.verified = false;
            }
            this.emit(debug.ModelEvents.BREAKPOINTS_UPDATED);
        };
        Model.prototype.enableOrDisableAllBreakpoints = function (enabled) {
            this.breakpoints.forEach(function (bp) {
                bp.enabled = enabled;
                if (!enabled) {
                    bp.lineNumber = bp.desiredLineNumber;
                    bp.verified = false;
                }
            });
            this.exceptionBreakpoints.forEach(function (ebp) { return ebp.enabled = enabled; });
            this.functionBreakpoints.forEach(function (fbp) { return fbp.enabled = enabled; });
            this.emit(debug.ModelEvents.BREAKPOINTS_UPDATED);
        };
        Model.prototype.addFunctionBreakpoint = function (functionName) {
            this.functionBreakpoints.push(new FunctionBreakpoint(functionName, true));
            this.emit(debug.ModelEvents.BREAKPOINTS_UPDATED);
        };
        Model.prototype.updateFunctionBreakpoints = function (data) {
            this.functionBreakpoints.forEach(function (fbp) {
                var fbpData = data[fbp.getId()];
                if (fbpData) {
                    fbp.name = fbpData.name || fbp.name;
                    fbp.verified = fbpData.verified;
                    fbp.idFromAdapter = fbpData.id;
                }
            });
            this.emit(debug.ModelEvents.BREAKPOINTS_UPDATED);
        };
        Model.prototype.removeFunctionBreakpoints = function (id) {
            this.functionBreakpoints = id ? this.functionBreakpoints.filter(function (fbp) { return fbp.getId() !== id; }) : [];
            this.emit(debug.ModelEvents.BREAKPOINTS_UPDATED);
        };
        Model.prototype.getReplElements = function () {
            return this.replElements;
        };
        Model.prototype.addReplExpression = function (session, stackFrame, name) {
            var _this = this;
            var expression = new Expression(name, true);
            this.addReplElements([expression]);
            return evaluateExpression(session, stackFrame, expression, 'repl').then(function () {
                return _this.emit(debug.ModelEvents.REPL_ELEMENTS_UPDATED, expression);
            });
        };
        Model.prototype.logToRepl = function (value, severity) {
            var elements = [];
            var previousOutput = this.replElements.length && this.replElements[this.replElements.length - 1];
            // string message
            if (typeof value === 'string') {
                value = strings.removeAnsiEscapeCodes(value);
                if (value && value.trim() && previousOutput && previousOutput.value === value && previousOutput.severity === severity) {
                    previousOutput.counter++; // we got the same output (but not an empty string when trimmed) so we just increment the counter
                }
                else {
                    var lines = value.trim().split('\n');
                    lines.forEach(function (line, index) {
                        elements.push(new ValueOutputElement(line, severity));
                    });
                }
            }
            else {
                elements.push(new KeyValueOutputElement(value.prototype, value, nls.localize(2, null)));
            }
            if (elements.length) {
                this.addReplElements(elements);
                this.emit(debug.ModelEvents.REPL_ELEMENTS_UPDATED, elements);
            }
        };
        Model.prototype.appendReplOutput = function (value, severity) {
            value = strings.removeAnsiEscapeCodes(value);
            var elements = [];
            var previousOutput = this.replElements.length && this.replElements[this.replElements.length - 1];
            var lines = value.split('\n');
            var groupTogether = !!previousOutput && (previousOutput.category === 'output' && severity === previousOutput.severity);
            if (groupTogether) {
                // append to previous line if same group
                previousOutput.value += lines.shift();
            }
            else if (previousOutput && previousOutput.value === '') {
                // remove potential empty lines between different output types
                this.replElements.pop();
            }
            // fill in lines as output value elements
            lines.forEach(function (line, index) {
                elements.push(new ValueOutputElement(line, severity, 'output'));
            });
            this.addReplElements(elements);
            this.emit(debug.ModelEvents.REPL_ELEMENTS_UPDATED, elements);
        };
        Model.prototype.addReplElements = function (newElements) {
            (_a = this.replElements).push.apply(_a, newElements);
            if (this.replElements.length > MAX_REPL_LENGTH) {
                this.replElements.splice(0, this.replElements.length - MAX_REPL_LENGTH);
            }
            var _a;
        };
        Model.prototype.clearReplExpressions = function () {
            if (this.replElements.length > 0) {
                this.replElements = [];
                this.emit(debug.ModelEvents.REPL_ELEMENTS_UPDATED);
            }
        };
        Model.prototype.getWatchExpressions = function () {
            return this.watchExpressions;
        };
        Model.prototype.addWatchExpression = function (session, stackFrame, name) {
            var we = new Expression(name, false);
            this.watchExpressions.push(we);
            if (!name) {
                this.emit(debug.ModelEvents.WATCH_EXPRESSIONS_UPDATED, we);
                return winjs_base_1.TPromise.as(null);
            }
            return this.evaluateWatchExpressions(session, stackFrame, we.getId());
        };
        Model.prototype.renameWatchExpression = function (session, stackFrame, id, newName) {
            var _this = this;
            var filtered = this.watchExpressions.filter(function (we) { return we.getId() === id; });
            if (filtered.length === 1) {
                filtered[0].name = newName;
                return evaluateExpression(session, stackFrame, filtered[0], 'watch').then(function () {
                    _this.emit(debug.ModelEvents.WATCH_EXPRESSIONS_UPDATED, filtered[0]);
                });
            }
            return winjs_base_1.TPromise.as(null);
        };
        Model.prototype.evaluateWatchExpressions = function (session, stackFrame, id) {
            var _this = this;
            if (id === void 0) { id = null; }
            if (id) {
                var filtered_1 = this.watchExpressions.filter(function (we) { return we.getId() === id; });
                if (filtered_1.length !== 1) {
                    return winjs_base_1.TPromise.as(null);
                }
                return evaluateExpression(session, stackFrame, filtered_1[0], 'watch').then(function () {
                    _this.emit(debug.ModelEvents.WATCH_EXPRESSIONS_UPDATED, filtered_1[0]);
                });
            }
            return winjs_base_1.TPromise.join(this.watchExpressions.map(function (we) { return evaluateExpression(session, stackFrame, we, 'watch'); })).then(function () {
                _this.emit(debug.ModelEvents.WATCH_EXPRESSIONS_UPDATED);
            });
        };
        Model.prototype.clearWatchExpressionValues = function () {
            this.watchExpressions.forEach(function (we) {
                we.value = Expression.DEFAULT_VALUE;
                we.available = false;
                we.reference = 0;
            });
            this.emit(debug.ModelEvents.WATCH_EXPRESSIONS_UPDATED);
        };
        Model.prototype.clearWatchExpressions = function (id) {
            if (id === void 0) { id = null; }
            this.watchExpressions = id ? this.watchExpressions.filter(function (we) { return we.getId() !== id; }) : [];
            this.emit(debug.ModelEvents.WATCH_EXPRESSIONS_UPDATED);
        };
        Model.prototype.sourceIsUnavailable = function (source) {
            var _this = this;
            Object.keys(this.threads).forEach(function (key) {
                if (_this.threads[key].getCachedCallStack()) {
                    _this.threads[key].getCachedCallStack().forEach(function (stackFrame) {
                        if (stackFrame.source.uri.toString() === source.uri.toString()) {
                            stackFrame.source.available = false;
                        }
                    });
                }
            });
            this.emit(debug.ModelEvents.CALLSTACK_UPDATED);
        };
        Model.prototype.rawUpdate = function (data) {
            if (data.thread) {
                this.threads[data.threadId] = new Thread(data.thread.name, data.thread.id);
            }
            if (data.stoppedDetails) {
                // Set the availability of the threads' callstacks depending on
                // whether the thread is stopped or not
                for (var ref in this.threads) {
                    if (this.threads.hasOwnProperty(ref)) {
                        if (data.allThreadsStopped) {
                            // Only update the details if all the threads are stopped
                            // because we don't want to overwrite the details of other
                            // threads that have stopped for a different reason
                            this.threads[ref].stoppedDetails = data.stoppedDetails;
                        }
                        this.threads[ref].stopped = data.allThreadsStopped;
                        this.threads[ref].clearCallStack();
                    }
                }
                this.threads[data.threadId].stoppedDetails = data.stoppedDetails;
                this.threads[data.threadId].stopped = true;
            }
            this.emit(debug.ModelEvents.CALLSTACK_UPDATED);
        };
        Model.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this.threads = null;
            this.breakpoints = null;
            this.exceptionBreakpoints = null;
            this.functionBreakpoints = null;
            this.watchExpressions = null;
            this.replElements = null;
            this.toDispose = lifecycle.disposeAll(this.toDispose);
        };
        return Model;
    }(ee.EventEmitter));
    exports.Model = Model;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/debug/common/debugViewModel", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/workbench/parts/debug/common/debug'], function (require, exports, ee, debug) {
    "use strict";
    var ViewModel = (function (_super) {
        __extends(ViewModel, _super);
        function ViewModel() {
            _super.apply(this, arguments);
        }
        ViewModel.prototype.getId = function () {
            return 'root';
        };
        ViewModel.prototype.getFocusedStackFrame = function () {
            return this.focusedStackFrame;
        };
        ViewModel.prototype.setFocusedStackFrame = function (focusedStackFrame) {
            this.focusedStackFrame = focusedStackFrame;
            this.emit(debug.ViewModelEvents.FOCUSED_STACK_FRAME_UPDATED);
        };
        ViewModel.prototype.getFocusedThreadId = function () {
            return this.focusedStackFrame ? this.focusedStackFrame.threadId : 0;
        };
        ViewModel.prototype.getSelectedExpression = function () {
            return this.selectedExpression;
        };
        ViewModel.prototype.setSelectedExpression = function (expression) {
            this.selectedExpression = expression;
            this.emit(debug.ViewModelEvents.SELECTED_EXPRESSION_UPDATED, expression);
        };
        ViewModel.prototype.getSelectedFunctionBreakpoint = function () {
            return this.selectedFunctionBreakpoint;
        };
        ViewModel.prototype.setSelectedFunctionBreakpoint = function (functionBreakpoint) {
            this.selectedFunctionBreakpoint = functionBreakpoint;
            this.emit(debug.ViewModelEvents.SELECTED_FUNCTION_BREAKPOINT_UPDATED, functionBreakpoint);
        };
        return ViewModel;
    }(ee.EventEmitter));
    exports.ViewModel = ViewModel;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/debug/node/debugAdapter", ["require", "exports", 'vs/nls!vs/workbench/parts/debug/node/debugAdapter', 'vs/base/common/paths', 'vs/base/common/platform'], function (require, exports, nls, paths, platform) {
    "use strict";
    var Adapter = (function () {
        function Adapter(rawAdapter, systemVariables, extensionFolderPath) {
            if (rawAdapter.windows) {
                rawAdapter.win = rawAdapter.windows;
            }
            if (platform.isWindows && !process.env.hasOwnProperty('PROCESSOR_ARCHITEW6432') && rawAdapter.winx86) {
                this.runtime = rawAdapter.winx86.runtime;
                this.runtimeArgs = rawAdapter.winx86.runtimeArgs;
                this.program = rawAdapter.winx86.program;
                this.args = rawAdapter.winx86.args;
            }
            else if (platform.isWindows && rawAdapter.win) {
                this.runtime = rawAdapter.win.runtime;
                this.runtimeArgs = rawAdapter.win.runtimeArgs;
                this.program = rawAdapter.win.program;
                this.args = rawAdapter.win.args;
            }
            else if (platform.isMacintosh && rawAdapter.osx) {
                this.runtime = rawAdapter.osx.runtime;
                this.runtimeArgs = rawAdapter.osx.runtimeArgs;
                this.program = rawAdapter.osx.program;
                this.args = rawAdapter.osx.args;
            }
            else if (platform.isLinux && rawAdapter.linux) {
                this.runtime = rawAdapter.linux.runtime;
                this.runtimeArgs = rawAdapter.linux.runtimeArgs;
                this.program = rawAdapter.linux.program;
                this.args = rawAdapter.linux.args;
            }
            this.runtime = this.runtime || rawAdapter.runtime;
            this.runtimeArgs = this.runtimeArgs || rawAdapter.runtimeArgs;
            this.program = this.program || rawAdapter.program;
            this.args = this.args || rawAdapter.args;
            if (this.program) {
                this.program = systemVariables ? systemVariables.resolve(this.program) : this.program;
                this.program = paths.join(extensionFolderPath, this.program);
            }
            if (this.runtime && this.runtime.indexOf('./') === 0) {
                this.runtime = systemVariables ? systemVariables.resolve(this.runtime) : this.runtime;
                this.runtime = paths.join(extensionFolderPath, this.runtime);
            }
            this.type = rawAdapter.type;
            this.configurationAttributes = rawAdapter.configurationAttributes;
            this.initialConfigurations = rawAdapter.initialConfigurations;
            this._label = rawAdapter.label;
            this.enableBreakpointsFor = rawAdapter.enableBreakpointsFor;
            this.aiKey = rawAdapter.aiKey;
        }
        Object.defineProperty(Adapter.prototype, "label", {
            get: function () {
                return this._label || this.type;
            },
            enumerable: true,
            configurable: true
        });
        Adapter.prototype.getSchemaAttributes = function () {
            var _this = this;
            // fill in the default configuration attributes shared by all adapters.
            if (this.configurationAttributes) {
                return Object.keys(this.configurationAttributes).map(function (request) {
                    var attributes = _this.configurationAttributes[request];
                    var defaultRequired = ['name', 'type', 'request'];
                    attributes.required = attributes.required && attributes.required.length ? defaultRequired.concat(attributes.required) : defaultRequired;
                    attributes.additionalProperties = false;
                    attributes.type = 'object';
                    if (!attributes.properties) {
                        attributes.properties = {};
                    }
                    var properties = attributes.properties;
                    properties.type = {
                        enum: [_this.type],
                        description: nls.localize(0, null)
                    };
                    properties.name = {
                        type: 'string',
                        description: nls.localize(1, null),
                        default: 'Launch'
                    };
                    properties.request = {
                        enum: [request],
                        description: nls.localize(2, null),
                    };
                    properties.preLaunchTask = {
                        type: ['string', 'null'],
                        default: null,
                        description: nls.localize(3, null)
                    };
                    _this.warnRelativePaths(properties.outDir);
                    _this.warnRelativePaths(properties.program);
                    _this.warnRelativePaths(properties.cwd);
                    _this.warnRelativePaths(properties.runtimeExecutable);
                    return attributes;
                });
            }
            return null;
        };
        Adapter.prototype.warnRelativePaths = function (attribute) {
            if (attribute) {
                attribute.pattern = '^\\${.*}.*|' + paths.isAbsoluteRegex.source;
                attribute.errorMessage = nls.localize(4, null);
            }
        };
        return Adapter;
    }());
    exports.Adapter = Adapter;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/debug/node/v8Protocol", ["require", "exports", 'vs/base/common/uuid', 'vs/base/common/eventEmitter', 'vs/base/common/winjs.base', 'vs/workbench/parts/debug/common/debug'], function (require, exports, uuid, ee, winjs_base_1, debug) {
    "use strict";
    var V8Protocol = (function (_super) {
        __extends(V8Protocol, _super);
        function V8Protocol() {
            _super.call(this);
            this.flowEventsCount = 0;
            this.emittedStopped = false;
            this.readyForBreakpoints = false;
            this.sequence = 1;
            this.contentLength = -1;
            this.pendingRequests = {};
            this.rawData = new Buffer(0);
            this.id = uuid.generateUuid();
        }
        V8Protocol.prototype.emit = function (eventType, data) {
            if (eventType === debug.SessionEvents.STOPPED) {
                this.emittedStopped = true;
            }
            if (eventType === debug.SessionEvents.INITIALIZED) {
                this.readyForBreakpoints = true;
            }
            if (eventType === debug.SessionEvents.CONTINUED || eventType === debug.SessionEvents.STOPPED ||
                eventType === debug.SessionEvents.DEBUGEE_TERMINATED || eventType === debug.SessionEvents.SERVER_EXIT) {
                this.flowEventsCount++;
            }
            if (data) {
                data.sessionId = this.getId();
            }
            else {
                data = { sessionId: this.getId() };
            }
            _super.prototype.emit.call(this, eventType, data);
        };
        V8Protocol.prototype.getId = function () {
            return this.id;
        };
        V8Protocol.prototype.connect = function (readable, writable) {
            var _this = this;
            this.outputStream = writable;
            readable.on('data', function (data) {
                _this.rawData = Buffer.concat([_this.rawData, data]);
                _this.handleData();
            });
        };
        V8Protocol.prototype.send = function (command, args) {
            var _this = this;
            return new winjs_base_1.TPromise(function (completeDispatch, errorDispatch) {
                _this.doSend(command, args, function (result) {
                    if (result.success) {
                        completeDispatch(result);
                    }
                    else {
                        errorDispatch(result);
                    }
                });
            });
        };
        V8Protocol.prototype.doSend = function (command, args, clb) {
            var request = {
                type: 'request',
                seq: this.sequence++,
                command: command
            };
            if (args && Object.keys(args).length > 0) {
                request.arguments = args;
            }
            // store callback for this request
            this.pendingRequests[request.seq] = clb;
            var json = JSON.stringify(request);
            var length = Buffer.byteLength(json, 'utf8');
            this.outputStream.write('Content-Length: ' + length.toString() + V8Protocol.TWO_CRLF, 'utf8');
            this.outputStream.write(json, 'utf8');
        };
        V8Protocol.prototype.handleData = function () {
            while (true) {
                if (this.contentLength >= 0) {
                    if (this.rawData.length >= this.contentLength) {
                        var message = this.rawData.toString('utf8', 0, this.contentLength);
                        this.rawData = this.rawData.slice(this.contentLength);
                        this.contentLength = -1;
                        if (message.length > 0) {
                            this.dispatch(message);
                        }
                        continue; // there may be more complete messages to process
                    }
                }
                else {
                    var s = this.rawData.toString('utf8', 0, this.rawData.length);
                    var idx = s.indexOf(V8Protocol.TWO_CRLF);
                    if (idx !== -1) {
                        var match = /Content-Length: (\d+)/.exec(s);
                        if (match && match[1]) {
                            this.contentLength = Number(match[1]);
                            this.rawData = this.rawData.slice(idx + V8Protocol.TWO_CRLF.length);
                            continue; // try to handle a complete message
                        }
                    }
                }
                break;
            }
        };
        V8Protocol.prototype.dispatch = function (body) {
            try {
                var rawData = JSON.parse(body);
                if (typeof rawData.event !== 'undefined') {
                    var event_1 = rawData;
                    this.emit(event_1.event, event_1);
                }
                else {
                    var response = rawData;
                    var clb = this.pendingRequests[response.request_seq];
                    if (clb) {
                        delete this.pendingRequests[response.request_seq];
                        clb(response);
                    }
                }
            }
            catch (e) {
                this.onServerError(new Error(e.message || e));
            }
        };
        V8Protocol.TWO_CRLF = '\r\n\r\n';
        return V8Protocol;
    }(ee.EventEmitter));
    exports.V8Protocol = V8Protocol;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/debug/node/rawDebugSession", ["require", "exports", 'vs/nls!vs/workbench/parts/debug/node/rawDebugSession', 'child_process', 'fs', 'net', 'vs/base/common/platform', 'vs/base/common/actions', 'vs/base/common/errors', 'vs/base/common/winjs.base', 'vs/base/common/severity', 'vs/workbench/parts/debug/common/debug', 'vs/workbench/parts/debug/node/v8Protocol', 'vs/base/node/stdFork', 'vs/platform/message/common/message', 'electron'], function (require, exports, nls, cp, fs, net, platform, actions_1, errors, winjs_base_1, severity_1, debug, v8, stdfork, message_1, electron_1) {
    "use strict";
    var RawDebugSession = (function (_super) {
        __extends(RawDebugSession, _super);
        function RawDebugSession(messageService, telemetryService, debugServerPort, adapter, telemtryAdapter) {
            _super.call(this);
            this.messageService = messageService;
            this.telemetryService = telemetryService;
            this.debugServerPort = debugServerPort;
            this.adapter = adapter;
            this.telemtryAdapter = telemtryAdapter;
            this.socket = null;
            this.capabilities = {};
        }
        RawDebugSession.prototype.initServer = function () {
            var _this = this;
            if (this.cachedInitServer) {
                return this.cachedInitServer;
            }
            var serverPromise = this.debugServerPort ? this.connectServer(this.debugServerPort) : this.startServer();
            this.cachedInitServer = serverPromise.then(function () {
                _this.startTime = new Date().getTime();
            }, function (err) {
                _this.cachedInitServer = null;
                return winjs_base_1.TPromise.wrapError(err);
            });
            return this.cachedInitServer;
        };
        RawDebugSession.prototype.send = function (command, args) {
            var _this = this;
            return this.initServer().then(function () { return _super.prototype.send.call(_this, command, args).then(function (response) { return response; }, function (errorResponse) {
                var error = errorResponse.body ? errorResponse.body.error : null;
                var message = error ? debug.formatPII(error.format, false, error.variables) : errorResponse.message;
                if (error && error.sendTelemetry) {
                    _this.telemetryService.publicLog('debugProtocolErrorResponse', { error: message });
                    _this.telemtryAdapter.log('debugProtocolErrorResponse', { error: message });
                }
                if (error && error.url) {
                    var label = error.urlLabel ? error.urlLabel : nls.localize(0, null);
                    return winjs_base_1.TPromise.wrapError(errors.create(message, { actions: [message_1.CloseAction, new actions_1.Action('debug.moreInfo', label, null, true, function () {
                                electron_1.shell.openExternal(error.url);
                                return winjs_base_1.TPromise.as(null);
                            })] }));
                }
                return winjs_base_1.TPromise.wrapError(new Error(message));
            }); });
        };
        RawDebugSession.prototype.initialize = function (args) {
            var _this = this;
            return this.send('initialize', args).then(function (response) {
                _this.capabilities = response.body || _this.capabilities;
                return response;
            });
        };
        RawDebugSession.prototype.launch = function (args) {
            this.isAttach = false;
            return this.sendAndLazyEmit('launch', args);
        };
        RawDebugSession.prototype.attach = function (args) {
            this.isAttach = true;
            return this.sendAndLazyEmit('attach', args);
        };
        RawDebugSession.prototype.next = function (args) {
            return this.sendAndLazyEmit('next', args);
        };
        RawDebugSession.prototype.stepIn = function (args) {
            return this.sendAndLazyEmit('stepIn', args);
        };
        RawDebugSession.prototype.stepOut = function (args) {
            return this.sendAndLazyEmit('stepOut', args);
        };
        RawDebugSession.prototype.continue = function (args) {
            return this.sendAndLazyEmit('continue', args);
        };
        // node sometimes sends "stopped" events earlier than the response for the "step" request.
        // due to this we only emit "continued" if we did not miss a stopped event.
        // we do not emit straight away to reduce viewlet flickering.
        RawDebugSession.prototype.sendAndLazyEmit = function (command, args, eventType) {
            var _this = this;
            if (eventType === void 0) { eventType = debug.SessionEvents.CONTINUED; }
            var count = this.flowEventsCount;
            return this.send(command, args).then(function (response) {
                setTimeout(function () {
                    if (_this.flowEventsCount === count) {
                        _this.emit(eventType);
                    }
                }, 500);
                return response;
            });
        };
        RawDebugSession.prototype.pause = function (args) {
            return this.send('pause', args);
        };
        RawDebugSession.prototype.disconnect = function (restart, force) {
            var _this = this;
            if (restart === void 0) { restart = false; }
            if (force === void 0) { force = false; }
            if (this.stopServerPending && force) {
                return this.stopServer();
            }
            if ((this.serverProcess || this.socket) && !this.stopServerPending) {
                // point of no return: from now on don't report any errors
                this.stopServerPending = true;
                this.restarted = restart;
                return this.send('disconnect', { restart: restart }).then(function () { return _this.stopServer(); }, function () { return _this.stopServer(); });
            }
            return winjs_base_1.TPromise.as(null);
        };
        RawDebugSession.prototype.setBreakpoints = function (args) {
            return this.send('setBreakpoints', args);
        };
        RawDebugSession.prototype.setFunctionBreakpoints = function (args) {
            return this.send('setFunctionBreakpoints', args);
        };
        RawDebugSession.prototype.setExceptionBreakpoints = function (args) {
            return this.send('setExceptionBreakpoints', args);
        };
        RawDebugSession.prototype.configurationDone = function () {
            return this.send('configurationDone', null);
        };
        RawDebugSession.prototype.stackTrace = function (args) {
            return this.send('stackTrace', args);
        };
        RawDebugSession.prototype.scopes = function (args) {
            return this.send('scopes', args);
        };
        RawDebugSession.prototype.variables = function (args) {
            return this.send('variables', args);
        };
        RawDebugSession.prototype.source = function (args) {
            return this.send('source', args);
        };
        RawDebugSession.prototype.threads = function () {
            return this.send('threads', null);
        };
        RawDebugSession.prototype.evaluate = function (args) {
            return this.send('evaluate', args);
        };
        RawDebugSession.prototype.getLengthInSeconds = function () {
            return (new Date().getTime() - this.startTime) / 1000;
        };
        RawDebugSession.prototype.getType = function () {
            return this.adapter.type;
        };
        RawDebugSession.prototype.connectServer = function (port) {
            var _this = this;
            return new winjs_base_1.TPromise(function (c, e) {
                _this.socket = net.createConnection(port, '127.0.0.1', function () {
                    _this.connect(_this.socket, _this.socket);
                    c(null);
                });
                _this.socket.on('error', function (err) {
                    e(err);
                });
            });
        };
        RawDebugSession.prototype.startServer = function () {
            var _this = this;
            if (!this.adapter.program) {
                return winjs_base_1.TPromise.wrapError(new Error(nls.localize(1, null, this.adapter.type)));
            }
            return this.getLaunchDetails().then(function (d) { return _this.launchServer(d).then(function () {
                _this.serverProcess.on('error', function (err) { return _this.onServerError(err); });
                _this.serverProcess.on('exit', function (code, signal) { return _this.onServerExit(); });
                var sanitize = function (s) { return s.toString().replace(/\r?\n$/mg, ''); };
                // this.serverProcess.stdout.on('data', (data: string) => {
                // 	console.log('%c' + sanitize(data), 'background: #ddd; font-style: italic;');
                // });
                _this.serverProcess.stderr.on('data', function (data) {
                    console.log(sanitize(data));
                });
                _this.connect(_this.serverProcess.stdout, _this.serverProcess.stdin);
            }); });
        };
        RawDebugSession.prototype.launchServer = function (launch) {
            var _this = this;
            return new winjs_base_1.TPromise(function (c, e) {
                if (launch.command === 'node') {
                    stdfork.fork(launch.argv[0], launch.argv.slice(1), {}, function (err, child) {
                        if (err) {
                            e(new Error(nls.localize(2, null, launch.argv[0])));
                        }
                        _this.serverProcess = child;
                        c(null);
                    });
                }
                else {
                    _this.serverProcess = cp.spawn(launch.command, launch.argv, {
                        stdio: [
                            'pipe',
                            'pipe',
                            'pipe' // stderr
                        ],
                    });
                    c(null);
                }
            });
        };
        RawDebugSession.prototype.stopServer = function () {
            var _this = this;
            if (this.socket !== null) {
                this.socket.end();
                this.cachedInitServer = null;
                this.emit(debug.SessionEvents.SERVER_EXIT);
            }
            if (!this.serverProcess) {
                return winjs_base_1.TPromise.as(null);
            }
            this.stopServerPending = true;
            var ret;
            // when killing a process in windows its child
            // processes are *not* killed but become root
            // processes. Therefore we use TASKKILL.EXE
            if (platform.isWindows) {
                ret = new winjs_base_1.TPromise(function (c, e) {
                    var killer = cp.exec("taskkill /F /T /PID " + _this.serverProcess.pid, function (err, stdout, stderr) {
                        if (err) {
                            return e(err);
                        }
                    });
                    killer.on('exit', c);
                    killer.on('error', e);
                });
            }
            else {
                this.serverProcess.kill('SIGTERM');
                ret = winjs_base_1.TPromise.as(null);
            }
            return ret;
        };
        RawDebugSession.prototype.getLaunchDetails = function () {
            var _this = this;
            return new winjs_base_1.TPromise(function (c, e) {
                fs.exists(_this.adapter.program, function (exists) {
                    if (exists) {
                        c(null);
                    }
                    else {
                        e(new Error(nls.localize(3, null, _this.adapter.program)));
                    }
                });
            }).then(function () {
                if (_this.adapter.runtime) {
                    return {
                        command: _this.adapter.runtime,
                        argv: (_this.adapter.runtimeArgs || []).concat([_this.adapter.program]).concat(_this.adapter.args || [])
                    };
                }
                return {
                    command: _this.adapter.program,
                    argv: _this.adapter.args || []
                };
            });
        };
        RawDebugSession.prototype.onServerError = function (err) {
            this.messageService.show(severity_1.default.Error, nls.localize(4, null, err.message));
            this.stopServer().done(null, errors.onUnexpectedError);
        };
        RawDebugSession.prototype.onServerExit = function () {
            this.serverProcess = null;
            this.cachedInitServer = null;
            if (!this.stopServerPending) {
                this.messageService.show(severity_1.default.Error, nls.localize(5, null));
            }
            this.emit(debug.SessionEvents.SERVER_EXIT);
        };
        return RawDebugSession;
    }(v8.V8Protocol));
    exports.RawDebugSession = RawDebugSession;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/emmet/node/editorAccessor", ["require", "exports", 'vs/editor/common/editorCommon', 'vs/base/common/strings', 'vs/editor/contrib/snippet/common/snippet', 'vs/editor/common/core/range', 'vs/editor/common/commands/replaceCommand'], function (require, exports, editorCommon_1, strings, snippets, range_1, replaceCommand_1) {
    'use strict';
    var EditorAccessor = (function () {
        function EditorAccessor(editor) {
            this.lineStarts = null;
            this.emmetSupportedModes = ['html', 'razor', 'css', 'less', 'scss', 'xml', 'xsl', 'jade', 'handlebars', 'hbs', 'jsx', 'tsx', 'erb', 'php', 'twig'];
            this.editor = editor;
        }
        EditorAccessor.prototype.noExpansionOccurred = function () {
            // return the tab key handling back to the editor
            this.editor.trigger('emmet', editorCommon_1.Handler.Tab, {});
        };
        EditorAccessor.prototype.isEmmetEnabledMode = function () {
            var syntax = this.getSyntax();
            return (this.emmetSupportedModes.indexOf(syntax) !== -1);
        };
        EditorAccessor.prototype.getSelectionRange = function () {
            var selection = this.editor.getSelection();
            return {
                start: this.getOffsetFromPosition(selection.getStartPosition()),
                end: this.getOffsetFromPosition(selection.getEndPosition())
            };
        };
        EditorAccessor.prototype.getCurrentLineRange = function () {
            var currentLine = this.editor.getSelection().startLineNumber;
            var lineStarts = this.getLineStarts();
            var start = lineStarts[currentLine - 1];
            var end = lineStarts[currentLine];
            return {
                start: start,
                end: end
            };
        };
        EditorAccessor.prototype.getCaretPos = function () {
            var selectionStart = this.editor.getSelection().getStartPosition();
            return this.getOffsetFromPosition(selectionStart);
        };
        EditorAccessor.prototype.setCaretPos = function (pos) {
            //
        };
        EditorAccessor.prototype.getCurrentLine = function () {
            var selectionStart = this.editor.getSelection().getStartPosition();
            return this.editor.getModel().getLineContent(selectionStart.lineNumber);
        };
        EditorAccessor.prototype.replaceContent = function (value, start, end, no_indent) {
            //console.log('value', value);
            var startPosition = this.getPositionFromOffset(start);
            var endPosition = this.getPositionFromOffset(end);
            // test if < or </ are located before the replace range. Either replace these too, or block the expansion
            var currentLine = this.editor.getModel().getLineContent(startPosition.lineNumber).substr(0, startPosition.column); // cpontent before the replaced range
            var match = currentLine.match(/<[/]?$/);
            if (match) {
                if (strings.startsWith(value, match[0])) {
                    startPosition = { lineNumber: startPosition.lineNumber, column: startPosition.column - match[0].length };
                }
                else {
                    return; // ignore
                }
            }
            // shift column by +1 since they are 1 based
            var range = new range_1.Range(startPosition.lineNumber, startPosition.column + 1, endPosition.lineNumber, endPosition.column + 1);
            var deletePreviousChars = 0;
            if (range.startLineNumber === range.endLineNumber) {
                // The snippet will delete
                deletePreviousChars = range.endColumn - range.startColumn;
            }
            else {
                // We must manually delete
                var command = new replaceCommand_1.ReplaceCommand(range, '');
                this.editor.executeCommand('emmet', command);
                deletePreviousChars = 0;
            }
            var snippet = snippets.CodeSnippet.convertExternalSnippet(value, snippets.ExternalSnippetType.EmmetSnippet);
            var codeSnippet = new snippets.CodeSnippet(snippet);
            snippets.getSnippetController(this.editor).run(codeSnippet, deletePreviousChars, 0);
        };
        EditorAccessor.prototype.getContent = function () {
            return this.editor.getModel().getValue();
        };
        EditorAccessor.prototype.createSelection = function (start, end) {
            //
        };
        EditorAccessor.prototype.getSyntax = function () {
            var position = this.editor.getSelection().getStartPosition();
            var mode = this.editor.getModel().getModeAtPosition(position.lineNumber, position.column);
            var syntax = mode.getId().split('.').pop();
            if (/\b(razor|handlebars|erb|php|hbs|twig)\b/.test(syntax)) {
                return 'html';
            }
            if (/\b(typescriptreact|javascriptreact)\b/.test(syntax)) {
                return 'jsx';
            }
            if (syntax === 'sass') {
                return 'scss';
            }
            return syntax;
        };
        EditorAccessor.prototype.getProfileName = function () {
            return null;
        };
        EditorAccessor.prototype.prompt = function (title) {
            //
        };
        EditorAccessor.prototype.getSelection = function () {
            return '';
        };
        EditorAccessor.prototype.getFilePath = function () {
            return '';
        };
        EditorAccessor.prototype.flushCache = function () {
            this.lineStarts = null;
        };
        EditorAccessor.prototype.getPositionFromOffset = function (offset) {
            var lineStarts = this.getLineStarts();
            var low = 0;
            var high = lineStarts.length - 1;
            var mid;
            while (low <= high) {
                mid = low + ((high - low) / 2) | 0;
                if (lineStarts[mid] > offset) {
                    high = mid - 1;
                }
                else {
                    low = mid + 1;
                }
            }
            return {
                lineNumber: low,
                column: offset - lineStarts[low - 1]
            };
        };
        EditorAccessor.prototype.getOffsetFromPosition = function (position) {
            var lineStarts = this.getLineStarts();
            return lineStarts[position.lineNumber - 1] + position.column - 1;
        };
        EditorAccessor.prototype.getLineStarts = function () {
            if (this.lineStarts === null) {
                this.lineStarts = this.computeLineStarts();
            }
            return this.lineStarts;
        };
        EditorAccessor.prototype.computeLineStarts = function () {
            var value = this.editor.getModel().getValue();
            return strings.computeLineStarts(value);
        };
        return EditorAccessor;
    }());
    exports.EditorAccessor = EditorAccessor;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/emmet/node/emmetActions", ["require", "exports", 'vs/base/common/winjs.base', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', './editorAccessor'], function (require, exports, winjs_base_1, editorAction_1, editorActionEnablement_1, editorAccessor_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    /// <reference path="emmet.d.ts" />
    'use strict';
    var ExpandAbbreviationAction = (function (_super) {
        __extends(ExpandAbbreviationAction, _super);
        function ExpandAbbreviationAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
            this.editorAccessor = new editorAccessor_1.EditorAccessor(editor);
        }
        ExpandAbbreviationAction.prototype.run = function () {
            var _this = this;
            return new winjs_base_1.TPromise(function (c, e) {
                require(['emmet'], function (_module) {
                    try {
                        if (!_this.editorAccessor.isEmmetEnabledMode()) {
                            _this.editorAccessor.noExpansionOccurred();
                            return;
                        }
                        if (!_module.run('expand_abbreviation', _this.editorAccessor)) {
                            _this.editorAccessor.noExpansionOccurred();
                        }
                    }
                    catch (err) {
                    }
                    finally {
                        _this.editorAccessor.flushCache();
                    }
                }, e);
            });
        };
        ExpandAbbreviationAction.ID = 'editor.emmet.action.expandAbbreviation';
        return ExpandAbbreviationAction;
    }(editorAction_1.EditorAction));
    exports.ExpandAbbreviationAction = ExpandAbbreviationAction;
});

define("vs/workbench/parts/emmet/node/emmet.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/emmet/node/emmet.contribution', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/editorCommon', './emmetActions', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/base/common/keyCodes', 'vs/platform/keybinding/common/keybindingService'], function (require, exports, nls, editorCommonExtensions_1, editorCommon, emmetActions_1, keybindingsRegistry_1, keyCodes_1, keybindingService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(emmetActions_1.ExpandAbbreviationAction, emmetActions_1.ExpandAbbreviationAction.ID, nls.localize(0, null)));
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandRule({
        id: emmetActions_1.ExpandAbbreviationAction.ID,
        weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.editorContrib(),
        context: keybindingService_1.KbExpr.and(keybindingService_1.KbExpr.has(editorCommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS), keybindingService_1.KbExpr.not(editorCommon.KEYBINDING_CONTEXT_EDITOR_HAS_NON_EMPTY_SELECTION), keybindingService_1.KbExpr.not(editorCommon.KEYBINDING_CONTEXT_EDITOR_HAS_MULTIPLE_SELECTIONS), keybindingService_1.KbExpr.not(editorCommon.KEYBINDING_CONTEXT_EDITOR_TAB_MOVES_FOCUS)),
        primary: keyCodes_1.KeyCode.Tab
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/errorList/browser/errorListConstants", ["require", "exports"], function (require, exports) {
    "use strict";
    exports.ERROR_LIST_PANEL_ID = 'workbench.panel.errorList';
});

define("vs/workbench/parts/execution/common/execution", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IExecutionService = instantiation_1.createDecorator('executionService');
    exports.ITerminalService = instantiation_1.createDecorator('nativeTerminalService');
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/execution/electron-browser/executionService", ["require", "exports", 'vs/nls!vs/workbench/parts/execution/electron-browser/executionService', 'vs/base/common/errors', 'vs/base/common/hash', 'vs/base/common/winjs.base', 'vs/base/common/objects', 'vs/base/common/strings', 'vs/workbench/parts/execution/common/execution', 'vs/base/common/uri', 'child_process', 'vs/base/node/processes'], function (require, exports, nls, errors, hash, winjs_base_1, objects, strings, exec, uri_1, cp, processes) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var AbstractExecutionService = (function () {
        function AbstractExecutionService() {
            this.serviceId = exec.IExecutionService;
            this._executions = Object.create(null);
        }
        AbstractExecutionService.prototype.exec = function (file, args, cwdOrOptions) {
            var options, allEnv = objects.clone(process.env);
            if (typeof cwdOrOptions === 'string') {
                options = {
                    cwd: cwdOrOptions,
                    env: allEnv
                };
            }
            else {
                objects.mixin(allEnv, cwdOrOptions.env);
                cwdOrOptions.env = allEnv;
                options = cwdOrOptions;
            }
            var key = args.concat([file, options.cwd]).reduce(function (p, c) { return hash.combine(hash.computeMurmur2StringHashCode(c), p); }, 17);
            if (this._executions[key]) {
                this._executions[key].cancel();
                delete this._executions[key];
            }
            var ret = this.doExec(file, args, options);
            this._executions[key] = ret;
            return ret.then(function (val) { return val; }, function (err) {
                if (!errors.isPromiseCanceledError(err)) {
                    throw err;
                }
            });
        };
        AbstractExecutionService.prototype.doExec = function (file, args, options) {
            throw errors.notImplemented();
        };
        return AbstractExecutionService;
    }());
    exports.AbstractExecutionService = AbstractExecutionService;
    var WinExecutionService = (function (_super) {
        __extends(WinExecutionService, _super);
        function WinExecutionService() {
            _super.apply(this, arguments);
        }
        WinExecutionService.prototype.doExec = function (file, args, options) {
            var childProcess;
            return new winjs_base_1.TPromise(function (c, e, p) {
                var shell = processes.getWindowsShell();
                // we use `start` to get another shell where `& pause` can be handled
                args = [
                    '/c',
                    'start',
                    '/wait',
                    shell,
                    '/c',
                    strings.format('"{0} {1} & pause"', file, args.join(' '))
                ];
                options = options || {};
                options.windowsVerbatimArguments = true;
                childProcess = cp.spawn(shell, args, options);
                childProcess.on('exit', c);
                childProcess.on('error', e);
                // send out the process once
                p(childProcess);
            }, function () {
                if (!childProcess) {
                    return;
                }
                cp.exec("taskkill /F /T /PID " + childProcess.pid, function (err, stdout, stderr) {
                    if (err) {
                        console.error(err);
                    }
                });
            });
        };
        return WinExecutionService;
    }(AbstractExecutionService));
    exports.WinExecutionService = WinExecutionService;
    var MacExecutionService = (function (_super) {
        __extends(MacExecutionService, _super);
        function MacExecutionService() {
            _super.apply(this, arguments);
        }
        MacExecutionService.prototype.doExec = function (file, args, options) {
            var childProcess;
            return new winjs_base_1.TPromise(function (c, e, p) {
                args = [
                    uri_1.default.parse(require.toUrl('vs/workbench/parts/execution/electron-browser/macHelper.scpt')).fsPath,
                    'cd', ("'" + options.cwd + "'"), ';',
                    file
                ].concat(args);
                childProcess = cp.spawn('/usr/bin/osascript', args, options);
                childProcess.on('exit', c);
                childProcess.on('error', e);
                // send out the process once
                p(childProcess);
            }, function () {
                if (childProcess) {
                    childProcess.kill('SIGTERM');
                }
            });
        };
        return MacExecutionService;
    }(AbstractExecutionService));
    exports.MacExecutionService = MacExecutionService;
    var LinuxExecutionService = (function (_super) {
        __extends(LinuxExecutionService, _super);
        function LinuxExecutionService() {
            _super.apply(this, arguments);
        }
        LinuxExecutionService.prototype.doExec = function (file, args, options) {
            var childProcess;
            return new winjs_base_1.TPromise(function (c, e, p) {
                var flattenedArgs = '';
                if (args.length > 0) {
                    flattenedArgs = '"' + args.join('" "') + '"';
                }
                var cdCommand = '';
                if (options.cwd) {
                    cdCommand = strings.format('cd "{0}"', options.cwd);
                }
                var bashCommand = strings.format('{0}; "{1}" {2} ; echo; read -p "{3}" -n1;', cdCommand, file, flattenedArgs, LinuxExecutionService.WAIT_MESSAGE);
                args = [
                    '-x',
                    'bash',
                    '-c',
                    // wrapping argument in two sets of ' because node is so "friendly" that it removes one set...
                    strings.format('\'\'{0}\'\'', bashCommand)
                ];
                childProcess = cp.spawn(LinuxExecutionService.LINUX_TERM, args, options);
                childProcess.on('exit', c);
                childProcess.on('error', e);
                // send out the process once
                p(childProcess);
            }, function () {
                console.log('SENDING KILL');
                if (childProcess) {
                    childProcess.kill('SIGTERM');
                }
            });
        };
        LinuxExecutionService.LINUX_TERM = '/usr/bin/gnome-terminal'; // '/usr/bin/x-terminal-emulator'
        LinuxExecutionService.WAIT_MESSAGE = nls.localize(0, null);
        return LinuxExecutionService;
    }(AbstractExecutionService));
    exports.LinuxExecutionService = LinuxExecutionService;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/execution/electron-browser/terminalService", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/workbench/parts/execution/common/execution', 'vs/platform/configuration/common/configuration', 'vs/platform/message/common/message', 'child_process', 'vs/base/node/processes'], function (require, exports, errors, uri_1, winjs_base_1, execution_1, configuration_1, message_1, cp, processes) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var WinTerminalService = (function () {
        function WinTerminalService(_configurationService, _messageService) {
            this._configurationService = _configurationService;
            this._messageService = _messageService;
            this.serviceId = execution_1.ITerminalService;
        }
        WinTerminalService.prototype.openTerminal = function (path) {
            cp.spawn(processes.getWindowsShell(), ['/c', 'start', '/wait'], { cwd: path });
        };
        WinTerminalService = __decorate([
            __param(0, configuration_1.IConfigurationService),
            __param(1, message_1.IMessageService)
        ], WinTerminalService);
        return WinTerminalService;
    }());
    exports.WinTerminalService = WinTerminalService;
    var MacTerminalService = (function () {
        function MacTerminalService() {
            this.serviceId = execution_1.ITerminalService;
        }
        MacTerminalService.prototype.openTerminal = function (path) {
            this.getTerminalHelperScriptPath().done(function (helperPath) {
                cp.spawn('/usr/bin/osascript', [helperPath, path]);
            }, errors.onUnexpectedError);
        };
        MacTerminalService.prototype.getTerminalHelperScriptPath = function () {
            if (this._terminalApplicationScriptPath) {
                return this._terminalApplicationScriptPath;
            }
            return this._terminalApplicationScriptPath = new winjs_base_1.TPromise(function (c, e) {
                var version = '';
                var child = cp.spawn('/usr/bin/osascript', ['-e', 'version of application "iTerm"']);
                child.on('error', e);
                child.stdout.on('data', function (data) {
                    version += data.toString();
                });
                child.on('exit', function (code) {
                    var script = 'terminal.scpt';
                    if (code === 0) {
                        var match = /(\d+).(\d+).(\d+)/.exec(version);
                        if (match.length >= 4) {
                            var major = +match[1];
                            var minor = +match[2];
                            var veryMinor = +match[3];
                            if ((major < 2) || (major === 2 && minor < 9) || (major === 2 && minor === 9 && veryMinor < 20150414)) {
                                script = 'iterm.scpt';
                            }
                            else {
                                script = 'itermNew.scpt'; // versions >= 2.9.20150414 use new script syntax
                            }
                        }
                    }
                    c(script);
                });
            }).then(function (name) { return uri_1.default.parse(require.toUrl("vs/workbench/parts/execution/electron-browser/" + name)).fsPath; });
        };
        return MacTerminalService;
    }());
    exports.MacTerminalService = MacTerminalService;
    var LinuxTerminalService = (function () {
        function LinuxTerminalService() {
            this.serviceId = execution_1.ITerminalService;
        }
        LinuxTerminalService.prototype.openTerminal = function (path) {
            cp.spawn('x-terminal-emulator', [], { cwd: path });
        };
        return LinuxTerminalService;
    }());
    exports.LinuxTerminalService = LinuxTerminalService;
});

define("vs/workbench/parts/execution/electron-browser/execution.contribution", ["require", "exports", 'vs/base/common/platform', 'vs/workbench/parts/execution/electron-browser/executionService', 'vs/workbench/parts/execution/electron-browser/terminalService', 'vs/platform/instantiation/common/extensions', 'vs/workbench/parts/execution/common/execution'], function (require, exports, env, executionService_1, terminalService_1, extensions_1, execution_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    if (env.isWindows) {
        extensions_1.registerSingleton(execution_1.IExecutionService, executionService_1.WinExecutionService);
        extensions_1.registerSingleton(execution_1.ITerminalService, terminalService_1.WinTerminalService);
    }
    else if (env.isMacintosh) {
        extensions_1.registerSingleton(execution_1.IExecutionService, executionService_1.MacExecutionService);
        extensions_1.registerSingleton(execution_1.ITerminalService, terminalService_1.MacTerminalService);
    }
    else if (env.isLinux) {
        extensions_1.registerSingleton(execution_1.IExecutionService, executionService_1.LinuxExecutionService);
        extensions_1.registerSingleton(execution_1.ITerminalService, terminalService_1.LinuxTerminalService);
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/extensions/common/extensions", ["require", "exports", 'vs/nls!vs/workbench/parts/extensions/common/extensions', 'vs/platform/instantiation/common/instantiation'], function (require, exports, nls, instantiation_1) {
    'use strict';
    exports.IExtensionsService = instantiation_1.createDecorator('extensionsService');
    exports.IGalleryService = instantiation_1.createDecorator('galleryService');
    exports.IExtensionTipsService = instantiation_1.createDecorator('extensionTipsService');
    exports.ExtensionsLabel = nls.localize(0, null);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/extensions/common/extensionsUtil", ["require", "exports", 'vs/workbench/parts/extensions/common/extensions', 'vs/base/common/winjs.base', 'semver'], function (require, exports, extensions_1, winjs_base_1, semver) {
    "use strict";
    'use strict';
    function extensionEquals(one, other) {
        return one.publisher === other.publisher && one.name === other.name;
    }
    exports.extensionEquals = extensionEquals;
    function getOutdatedExtensions(accessor) {
        var extensionsService = accessor.get(extensions_1.IExtensionsService);
        var galleryService = accessor.get(extensions_1.IGalleryService);
        if (!galleryService.isEnabled()) {
            return winjs_base_1.TPromise.as([]);
        }
        return winjs_base_1.TPromise.join([galleryService.query(), extensionsService.getInstalled()])
            .then(function (result) {
            var available = result[0];
            var installed = result[1];
            return available.filter(function (extension) {
                var local = installed.filter(function (local) { return extensionEquals(local, extension); })[0];
                return local && semver.lt(local.version, extension.version);
            });
        });
    }
    exports.getOutdatedExtensions = getOutdatedExtensions;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/extensions/electron-browser/extensionTipsService", ["require", "exports", 'vs/base/common/objects', 'vs/base/common/lifecycle', 'vs/base/common/glob', 'vs/workbench/parts/extensions/common/extensions', 'vs/editor/common/services/modelService', 'vs/platform/storage/common/storage', 'vs/platform/workspace/common/workspace'], function (require, exports, objects_1, lifecycle_1, glob_1, extensions_1, modelService_1, storage_1, workspace_1) {
    "use strict";
    var ExtensionTipsService = (function () {
        function ExtensionTipsService(_galleryService, _modelService, _storageService, contextService) {
            var _this = this;
            this._galleryService = _galleryService;
            this._modelService = _modelService;
            this._storageService = _storageService;
            this._disposables = [];
            if (!this._galleryService.isEnabled()) {
                return;
            }
            this._recommendations = objects_1.toObject(JSON.parse(_storageService.get('extensionsAssistant/recommendations', storage_1.StorageScope.GLOBAL, '[]')), function (id) { return id; }, function () { return true; });
            var extensionTips = contextService.getConfiguration().env.extensionTips;
            if (extensionTips) {
                this._availableRecommendations = extensionTips;
                this._disposables.push(this._modelService.onModelAdded(function (model) {
                    _this._suggest(model.getAssociatedResource());
                }));
                for (var _i = 0, _a = this._modelService.getModels(); _i < _a.length; _i++) {
                    var model = _a[_i];
                    this._suggest(model.getAssociatedResource());
                }
            }
        }
        ExtensionTipsService.prototype.getRecommendations = function () {
            var _this = this;
            return this._galleryService.query()
                .then(null, function () { return []; })
                .then(function (available) { return objects_1.toObject(available, function (ext) { return (ext.publisher + "." + ext.name); }); })
                .then(function (available) {
                return Object.keys(_this._recommendations)
                    .map(function (id) { return available[id]; })
                    .filter(function (i) { return !!i; });
            });
        };
        ExtensionTipsService.prototype._suggest = function (uri) {
            var _this = this;
            if (!uri) {
                return;
            }
            var ids = Object.keys(this._availableRecommendations);
            var recommendations = ids
                .filter(function (id) { return glob_1.match(_this._availableRecommendations[id], uri.fsPath); });
            recommendations.forEach(function (r) { return _this._recommendations[r] = true; });
            this._storageService.store('extensionsAssistant/recommendations', JSON.stringify(Object.keys(this._recommendations)), storage_1.StorageScope.GLOBAL);
        };
        ExtensionTipsService.prototype.dispose = function () {
            this._disposables = lifecycle_1.disposeAll(this._disposables);
        };
        ExtensionTipsService = __decorate([
            __param(0, extensions_1.IGalleryService),
            __param(1, modelService_1.IModelService),
            __param(2, storage_1.IStorageService),
            __param(3, workspace_1.IWorkspaceContextService)
        ], ExtensionTipsService);
        return ExtensionTipsService;
    }());
    exports.ExtensionTipsService = ExtensionTipsService;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/files/browser/files", ["require", "exports", 'vs/workbench/browser/parts/editor/baseEditor'], function (require, exports, baseEditor_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * A lightweight descriptor of an editor for files. Optionally allows to specify a list of mime types the editor
     * should be used for. This allows for fine grained contribution of editors to the Platform based on mime types. Wildcards
     * can be used (e.g. text/*) to register an editor on a wider range of mime types.
     */
    var FileEditorDescriptor = (function (_super) {
        __extends(FileEditorDescriptor, _super);
        function FileEditorDescriptor(id, name, moduleId, ctorName, mimetypes) {
            _super.call(this, id, name, moduleId, ctorName);
            this.mimetypes = mimetypes;
        }
        FileEditorDescriptor.prototype.getMimeTypes = function () {
            return this.mimetypes;
        };
        return FileEditorDescriptor;
    }(baseEditor_1.EditorDescriptor));
    exports.FileEditorDescriptor = FileEditorDescriptor;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/files/common/explorerViewModel", ["require", "exports", 'vs/base/common/assert', 'vs/base/common/types', 'vs/base/common/uri', 'vs/base/common/platform', 'vs/base/common/paths', 'vs/base/common/mime'], function (require, exports, assert, types, uri_1, platform_1, paths, mime_1) {
    "use strict";
    'use strict';
    (function (StatType) {
        StatType[StatType["FILE"] = 0] = "FILE";
        StatType[StatType["FOLDER"] = 1] = "FOLDER";
        StatType[StatType["ANY"] = 2] = "ANY";
    })(exports.StatType || (exports.StatType = {}));
    var StatType = exports.StatType;
    var FileStat = (function () {
        function FileStat(resource, isDirectory, hasChildren, name, mtime, etag) {
            if (name === void 0) { name = paths.basename(resource.fsPath); }
            this.resource = resource;
            this.name = name;
            this.isDirectory = !!isDirectory;
            this.hasChildren = isDirectory && hasChildren;
            this.mime = !isDirectory ? mime_1.guessMimeTypes(this.resource.fsPath).join(', ') : void (0);
            this.etag = etag;
            this.mtime = mtime;
            // Prepare child stat array
            if (this.isDirectory) {
                this.children = [];
            }
            this.isDirectoryResolved = false;
        }
        FileStat.prototype.getId = function () {
            return this.resource.toString();
        };
        FileStat.create = function (raw, resolveTo) {
            var stat = new FileStat(raw.resource, raw.isDirectory, raw.hasChildren, raw.name, raw.mtime, raw.etag);
            // Recursively add children if present
            if (stat.isDirectory) {
                // isDirectoryResolved is a very important indicator in the stat model that tells if the folder was fully resolved
                // the folder is fully resolved if either it has a list of children or the client requested this by using the resolveTo
                // array of resource path to resolve.
                stat.isDirectoryResolved = !!raw.children || (!!resolveTo && resolveTo.some(function (r) {
                    return paths.isEqualOrParent(r.fsPath, stat.resource.fsPath);
                }));
                // Recurse into children
                if (raw.children) {
                    for (var i = 0, len = raw.children.length; i < len; i++) {
                        var child = FileStat.create(raw.children[i], resolveTo);
                        child.parent = stat;
                        stat.children.push(child);
                        stat.hasChildren = stat.children.length > 0;
                    }
                }
            }
            return stat;
        };
        /**
         * Merges the stat which was resolved from the disk with the local stat by copying over properties
         * and children. The merge will only consider resolved stat elements to avoid overwriting data which
         * exists locally.
         */
        FileStat.mergeLocalWithDisk = function (disk, local) {
            assert.ok(disk.resource.toString() === local.resource.toString(), 'Merging only supported for stats with the same resource');
            // Stop merging when a folder is not resolved to avoid loosing local data
            var mergingDirectories = disk.isDirectory || local.isDirectory;
            if (mergingDirectories && local.isDirectoryResolved && !disk.isDirectoryResolved) {
                return;
            }
            // Properties
            local.resource = disk.resource;
            local.name = disk.name;
            local.isDirectory = disk.isDirectory;
            local.hasChildren = disk.isDirectory && disk.hasChildren;
            local.mtime = disk.mtime;
            local.mime = disk.mime;
            local.isDirectoryResolved = disk.isDirectoryResolved;
            // Merge Children if resolved
            if (mergingDirectories && disk.isDirectoryResolved) {
                // Map resource => stat
                var oldLocalChildren_1 = Object.create(null);
                local.children.forEach(function (localChild) {
                    oldLocalChildren_1[localChild.resource.toString()] = localChild;
                });
                // Clear current children
                local.children = [];
                // Merge received children
                disk.children.forEach(function (diskChild) {
                    var formerLocalChild = oldLocalChildren_1[diskChild.resource.toString()];
                    // Existing child: merge
                    if (formerLocalChild) {
                        FileStat.mergeLocalWithDisk(diskChild, formerLocalChild);
                        formerLocalChild.parent = local;
                        local.children.push(formerLocalChild);
                    }
                    else {
                        diskChild.parent = local;
                        local.children.push(diskChild);
                    }
                });
            }
        };
        /**
         * Returns a deep copy of this model object.
         */
        FileStat.prototype.clone = function () {
            var stat = new FileStat(uri_1.default.parse(this.resource.toString()), this.isDirectory, this.hasChildren, this.name, this.mtime, this.etag);
            stat.isDirectoryResolved = this.isDirectoryResolved;
            if (this.parent) {
                stat.parent = this.parent;
            }
            if (this.isDirectory) {
                this.children.forEach(function (child) {
                    stat.addChild(child.clone());
                });
            }
            return stat;
        };
        /**
         * Adds a child element to this folder.
         */
        FileStat.prototype.addChild = function (child) {
            assert.ok(this.isDirectory, 'Can only add a child to a folder');
            // Overwrite a previous child with the same name
            this.removeChild(child);
            // Inherit some parent properties to child
            child.parent = this;
            child.updateResource(false);
            this.children.push(child);
            this.hasChildren = this.children.length > 0;
        };
        /**
         * Returns true if this stat is a directory that contains a child with the given name.
         *
         * @param ignoreCase if true, will check for the name ignoring case.
         * @param type the type of stat to check for.
         */
        FileStat.prototype.hasChild = function (name, ignoreCase, type) {
            if (type === void 0) { type = StatType.ANY; }
            assert.ok(this.isDirectory, 'Can only call hasChild on a directory');
            assert.ok(types.isString(name), 'Expected parameter of type String');
            for (var i = 0; i < this.children.length; i++) {
                var child = this.children[i];
                if ((type === StatType.FILE && child.isDirectory) || (type === StatType.FOLDER && !child.isDirectory)) {
                    continue;
                }
                // Check for Identity
                if (child.name === name) {
                    return true;
                }
                // Also consider comparing without case
                if (ignoreCase && child.name.toLowerCase() === name.toLowerCase()) {
                    return true;
                }
            }
            return false;
        };
        /**
         * Removes a child element from this folder.
         */
        FileStat.prototype.removeChild = function (child) {
            assert.ok(this.isDirectory, 'Can only remove a child from a directory');
            assert.ok(!!this.children, 'Expected children for directory but found none: ' + this.resource.fsPath);
            for (var i = 0; i < this.children.length; i++) {
                if (this.children[i].resource.toString() === child.resource.toString()) {
                    this.children.splice(i, 1);
                    break;
                }
            }
            this.hasChildren = this.children.length > 0;
        };
        /**
         * Moves this element under a new parent element.
         */
        FileStat.prototype.move = function (newParent, fnBetweenStates, fnDone) {
            var _this = this;
            assert.ok(newParent.isDirectory, 'Can only move an element into a directory');
            if (!fnBetweenStates) {
                fnBetweenStates = function (cb) { cb(); };
            }
            this.parent.removeChild(this);
            fnBetweenStates(function () {
                newParent.addChild(_this);
                _this.updateResource(true);
                if (fnDone) {
                    fnDone();
                }
            });
        };
        FileStat.prototype.updateResource = function (recursive) {
            this.resource = uri_1.default.file(paths.join(this.parent.resource.fsPath, this.name));
            if (recursive) {
                if (this.isDirectory && this.hasChildren && this.children) {
                    this.children.forEach(function (child) {
                        child.updateResource(true);
                    });
                }
            }
        };
        /**
         * Tells this stat that it was renamed. This requires changes to all children of this stat (if any)
         * so that the path property can be updated properly.
         */
        FileStat.prototype.rename = function (renamedStat) {
            // Merge a subset of Properties that can change on rename
            this.name = renamedStat.name;
            this.mime = renamedStat.mime;
            this.mtime = renamedStat.mtime;
            // Update Paths including children
            this.updateResource(true);
        };
        /**
         * Returns a child stat from this stat that matches with the provided path.
         * Will return "null" in case the child does not exist.
         */
        FileStat.prototype.find = function (resource) {
            // Return if path found
            if (this.fileResourceEquals(resource, this.resource)) {
                return this;
            }
            // Return if not having any children
            if (!this.hasChildren) {
                return null;
            }
            for (var i = 0; i < this.children.length; i++) {
                var child = this.children[i];
                if (this.fileResourceEquals(resource, child.resource)) {
                    return child;
                }
                if (child.isDirectory && paths.isEqualOrParent(resource.fsPath, child.resource.fsPath)) {
                    return child.find(resource);
                }
            }
            return null; //Unable to find
        };
        FileStat.prototype.fileResourceEquals = function (r1, r2) {
            var identityEquals = (r1.toString() === r2.toString());
            if (platform_1.isLinux || identityEquals) {
                return identityEquals;
            }
            return r1.toString().toLowerCase() === r2.toString().toLowerCase();
        };
        return FileStat;
    }());
    exports.FileStat = FileStat;
    /* A helper that can be used to show a placeholder when creating a new stat */
    var NewStatPlaceholder = (function (_super) {
        __extends(NewStatPlaceholder, _super);
        function NewStatPlaceholder(isDirectory) {
            _super.call(this, uri_1.default.file(''));
            this.id = NewStatPlaceholder.ID++;
            this.isDirectoryResolved = isDirectory;
        }
        NewStatPlaceholder.prototype.destroy = function () {
            this.parent.removeChild(this);
            this.isDirectoryResolved = void 0;
            this.name = void 0;
            this.isDirectory = void 0;
            this.hasChildren = void 0;
            this.mtime = void 0;
            this.mime = void 0;
        };
        NewStatPlaceholder.prototype.getId = function () {
            return 'new-stat-placeholder:' + this.id + ':' + this.parent.resource.toString();
        };
        /**
         * Returns a deep copy of this model object.
         */
        NewStatPlaceholder.prototype.clone = function () {
            var stat = new NewStatPlaceholder(this.isDirectory);
            stat.parent = this.parent;
            return stat;
        };
        NewStatPlaceholder.prototype.addChild = function (child) {
            throw new Error('Can\'t perform operations in NewStatPlaceholder.');
        };
        NewStatPlaceholder.prototype.hasChild = function (name, ignoreCase) {
            return false;
        };
        NewStatPlaceholder.prototype.removeChild = function (child) {
            throw new Error('Can\'t perform operations in NewStatPlaceholder.');
        };
        NewStatPlaceholder.prototype.move = function (newParent) {
            throw new Error('Can\'t perform operations in NewStatPlaceholder.');
        };
        NewStatPlaceholder.prototype.rename = function (renamedStat) {
            throw new Error('Can\'t perform operations in NewStatPlaceholder.');
        };
        NewStatPlaceholder.prototype.find = function (resource) {
            return null;
        };
        NewStatPlaceholder.addNewStatPlaceholder = function (parent, isDirectory) {
            assert.ok(parent.isDirectory, 'Can only add a child to a folder');
            var child = new NewStatPlaceholder(isDirectory);
            // Inherit some parent properties to child
            child.parent = parent;
            parent.children.push(child);
            parent.hasChildren = parent.children.length > 0;
            return child;
        };
        NewStatPlaceholder.ID = 0;
        return NewStatPlaceholder;
    }(FileStat));
    exports.NewStatPlaceholder = NewStatPlaceholder;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/files/common/files", ["require", "exports", 'vs/base/common/events', 'vs/base/common/mime', 'vs/workbench/common/editor', 'vs/platform/instantiation/common/instantiation', 'vs/workbench/parts/files/common/explorerViewModel'], function (require, exports, events_1, mime_1, editor_1, instantiation_1, explorerViewModel_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Explorer viewlet id.
     */
    exports.VIEWLET_ID = 'workbench.view.explorer';
    /**
     * File editor input id.
     */
    exports.FILE_EDITOR_INPUT_ID = 'workbench.editors.files.fileEditorInput';
    /**
     * Text file editor id.
     */
    exports.TEXT_FILE_EDITOR_ID = 'workbench.editors.files.textFileEditor';
    /**
     * Binary file editor id.
     */
    exports.BINARY_FILE_EDITOR_ID = 'workbench.editors.files.binaryFileEditor';
    /**
     * Marker ID for model entries.
     */
    exports.WORKING_FILES_MODEL_ENTRY_CLASS_ID = 'workbench.workingFiles.model.entry.class';
    /**
     * API class to denote file editor inputs. Internal implementation is provided.
     *
     * Note: This class is not intended to be instantiated.
     */
    var FileEditorInput = (function (_super) {
        __extends(FileEditorInput, _super);
        function FileEditorInput() {
            _super.apply(this, arguments);
        }
        return FileEditorInput;
    }(editor_1.EditorInput));
    exports.FileEditorInput = FileEditorInput;
    /**
     * Helper to get a file resource from an object.
     */
    function asFileResource(obj) {
        if (obj instanceof explorerViewModel_1.FileStat) {
            var stat = obj;
            return {
                resource: stat.resource,
                mimes: stat.mime ? stat.mime.split(', ') : [],
                isDirectory: stat.isDirectory
            };
        }
        if (obj && obj.CLASS_ID === exports.WORKING_FILES_MODEL_ENTRY_CLASS_ID) {
            var entry = obj;
            if (entry.isFile) {
                return {
                    resource: entry.resource,
                    mimes: mime_1.guessMimeTypes(entry.resource.fsPath),
                    isDirectory: false
                };
            }
        }
        return null;
    }
    exports.asFileResource = asFileResource;
    /**
     * List of event types from files.
     */
    exports.EventType = {
        /**
         * Indicates that a file content has changed but not yet saved.
         */
        FILE_DIRTY: 'files:fileDirty',
        /**
         * Indicates that a file is being saved.
         */
        FILE_SAVING: 'files:fileSaving',
        /**
         * Indicates that a file save resulted in an error.
         */
        FILE_SAVE_ERROR: 'files:fileSaveError',
        /**
         * Indicates that a file content has been saved to the disk.
         */
        FILE_SAVED: 'files:fileSaved',
        /**
         * Indicates that a file content has been reverted to the state
         * on disk.
         */
        FILE_REVERTED: 'files:fileReverted'
    };
    /**
     * Local file change events are being emitted when a file is added, removed, moved or its contents got updated. These events
     * are being emitted from within the workbench and are not reflecting the truth on the disk file system. For that, please
     * use FileChangesEvent instead.
     */
    var LocalFileChangeEvent = (function (_super) {
        __extends(LocalFileChangeEvent, _super);
        function LocalFileChangeEvent(before, after, originalEvent) {
            _super.call(this, null, before, after, originalEvent);
        }
        /**
         * Returns the meta information of the file before the event occurred or null if the file is new.
         */
        LocalFileChangeEvent.prototype.getBefore = function () {
            return this.oldValue;
        };
        /**
         * Returns the meta information of the file after the event occurred or null if the file got deleted.
         */
        LocalFileChangeEvent.prototype.getAfter = function () {
            return this.newValue;
        };
        /**
         * Indicates if the file was added as a new file.
         */
        LocalFileChangeEvent.prototype.gotAdded = function () {
            return !this.oldValue && !!this.newValue;
        };
        /**
         * Indicates if the file was moved to a different path.
         */
        LocalFileChangeEvent.prototype.gotMoved = function () {
            return !!this.oldValue && !!this.newValue && this.oldValue.resource.toString() !== this.newValue.resource.toString();
        };
        /**
         * Indicates if the files metadata was updated.
         */
        LocalFileChangeEvent.prototype.gotUpdated = function () {
            return !!this.oldValue && !!this.newValue && !this.gotMoved() && this.oldValue !== this.newValue;
        };
        /**
         * Indicates if the file was deleted.
         */
        LocalFileChangeEvent.prototype.gotDeleted = function () {
            return !!this.oldValue && !this.newValue;
        };
        return LocalFileChangeEvent;
    }(events_1.PropertyChangeEvent));
    exports.LocalFileChangeEvent = LocalFileChangeEvent;
    /**
     * Text file change events are emitted when files are saved or reverted.
     */
    var TextFileChangeEvent = (function (_super) {
        __extends(TextFileChangeEvent, _super);
        function TextFileChangeEvent(model, before, after, originalEvent) {
            if (after === void 0) { after = before; }
            _super.call(this, before, after, originalEvent);
            this._model = model;
        }
        Object.defineProperty(TextFileChangeEvent.prototype, "model", {
            get: function () {
                return this._model;
            },
            enumerable: true,
            configurable: true
        });
        TextFileChangeEvent.prototype.setAutoSaved = function (autoSaved) {
            this._isAutoSaved = autoSaved;
        };
        Object.defineProperty(TextFileChangeEvent.prototype, "isAutoSaved", {
            get: function () {
                return this._isAutoSaved;
            },
            enumerable: true,
            configurable: true
        });
        return TextFileChangeEvent;
    }(LocalFileChangeEvent));
    exports.TextFileChangeEvent = TextFileChangeEvent;
    exports.TEXT_FILE_SERVICE_ID = 'textFileService';
    (function (ConfirmResult) {
        ConfirmResult[ConfirmResult["SAVE"] = 0] = "SAVE";
        ConfirmResult[ConfirmResult["DONT_SAVE"] = 1] = "DONT_SAVE";
        ConfirmResult[ConfirmResult["CANCEL"] = 2] = "CANCEL";
    })(exports.ConfirmResult || (exports.ConfirmResult = {}));
    var ConfirmResult = exports.ConfirmResult;
    (function (AutoSaveMode) {
        AutoSaveMode[AutoSaveMode["OFF"] = 0] = "OFF";
        AutoSaveMode[AutoSaveMode["AFTER_SHORT_DELAY"] = 1] = "AFTER_SHORT_DELAY";
        AutoSaveMode[AutoSaveMode["AFTER_LONG_DELAY"] = 2] = "AFTER_LONG_DELAY";
        AutoSaveMode[AutoSaveMode["ON_FOCUS_CHANGE"] = 3] = "ON_FOCUS_CHANGE";
    })(exports.AutoSaveMode || (exports.AutoSaveMode = {}));
    var AutoSaveMode = exports.AutoSaveMode;
    exports.ITextFileService = instantiation_1.createDecorator(exports.TEXT_FILE_SERVICE_ID);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/common/editors/saveParticipant", ["require", "exports", 'vs/editor/common/services/codeEditorService', 'vs/base/common/errors', 'vs/workbench/parts/files/common/files', 'vs/editor/common/core/selection', 'vs/editor/common/commands/trimTrailingWhitespaceCommand', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event'], function (require, exports, codeEditorService_1, errors, files_1, selection_1, trimTrailingWhitespaceCommand_1, configuration_1, event_1) {
    'use strict';
    // The save participant can change a model before its saved to support various scenarios like trimming trailing whitespace
    var SaveParticipant = (function () {
        function SaveParticipant(configurationService, eventService, codeEditorService) {
            this.configurationService = configurationService;
            this.eventService = eventService;
            this.codeEditorService = codeEditorService;
            this.toUnbind = [];
            this.trimTrailingWhitespace = false;
            this.registerListeners();
            this.loadConfiguration();
        }
        SaveParticipant.prototype.registerListeners = function () {
            var _this = this;
            this.toUnbind.push(this.eventService.addListener(files_1.EventType.FILE_SAVING, function (e) { return _this.onTextFileSaving(e); }));
            this.toUnbind.push(this.configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) { return _this.onConfigurationChange(e.config); }));
        };
        SaveParticipant.prototype.loadConfiguration = function () {
            var _this = this;
            this.configurationService.loadConfiguration().done(function (configuration) {
                _this.onConfigurationChange(configuration);
            }, errors.onUnexpectedError);
        };
        SaveParticipant.prototype.onConfigurationChange = function (configuration) {
            this.trimTrailingWhitespace = configuration && configuration.files && configuration.files.trimTrailingWhitespace;
        };
        SaveParticipant.prototype.getId = function () {
            return 'vs.files.saveparticipant';
        };
        SaveParticipant.prototype.onTextFileSaving = function (e) {
            // Trim Trailing Whitespace if enabled
            if (this.trimTrailingWhitespace) {
                this.doTrimTrailingWhitespace(e.model, e.isAutoSaved);
            }
        };
        /**
         * Trim trailing whitespace on a model and ignore lines on which cursors are sitting if triggered via auto save.
         */
        SaveParticipant.prototype.doTrimTrailingWhitespace = function (model, isAutoSaved) {
            var prevSelection = [selection_1.Selection.createSelection(1, 1, 1, 1)];
            var cursors = [];
            // If this is auto save, try to find active cursors to prevent removing
            // whitespace automatically while the user is typing at the end of a line
            if (isAutoSaved && model.isAttachedToEditor()) {
                var allEditors = this.codeEditorService.listCodeEditors();
                for (var i = 0, len = allEditors.length; i < len; i++) {
                    var editor = allEditors[i];
                    var editorModel = editor.getModel();
                    if (!editorModel) {
                        continue; // empty editor
                    }
                    if (model === editorModel) {
                        prevSelection = editor.getSelections();
                        cursors.push.apply(cursors, prevSelection.map(function (s) {
                            return {
                                lineNumber: s.positionLineNumber,
                                column: s.positionColumn
                            };
                        }));
                    }
                }
            }
            var ops = trimTrailingWhitespaceCommand_1.trimTrailingWhitespace(model, cursors);
            if (!ops.length) {
                return; // Nothing to do
            }
            model.pushEditOperations(prevSelection, ops, function (edits) { return prevSelection; });
        };
        SaveParticipant.prototype.dispose = function () {
            while (this.toUnbind.length) {
                this.toUnbind.pop()();
            }
        };
        SaveParticipant = __decorate([
            __param(0, configuration_1.IConfigurationService),
            __param(1, event_1.IEventService),
            __param(2, codeEditorService_1.ICodeEditorService)
        ], SaveParticipant);
        return SaveParticipant;
    }());
    exports.SaveParticipant = SaveParticipant;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/common/editors/textFileEditorModel", ["require", "exports", 'vs/nls!vs/workbench/parts/files/common/editors/textFileEditorModel', 'vs/base/common/winjs.base', 'vs/base/common/errors', 'vs/base/common/uri', 'vs/base/common/paths', 'vs/base/common/diagnostics', 'vs/base/common/types', 'vs/editor/common/editorCommon', 'vs/workbench/common/events', 'vs/workbench/parts/files/common/files', 'vs/workbench/common/editor', 'vs/workbench/common/editor/textEditorModel', 'vs/platform/files/common/files', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/editor/common/services/modeService', 'vs/editor/common/services/modelService', 'vs/platform/telemetry/common/telemetry'], function (require, exports, nls, winjs_base_1, errors_1, uri_1, paths, diagnostics, types, editorCommon_1, events_1, files_1, editor_1, textEditorModel_1, files_2, event_1, instantiation_1, message_1, modeService_1, modelService_1, telemetry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DefaultSaveErrorHandler = (function () {
        function DefaultSaveErrorHandler(messageService) {
            this.messageService = messageService;
        }
        DefaultSaveErrorHandler.prototype.onSaveError = function (error, model) {
            this.messageService.show(message_1.Severity.Error, nls.localize(0, null, paths.basename(model.getResource().fsPath), errors_1.toErrorMessage(error, false)));
        };
        DefaultSaveErrorHandler = __decorate([
            __param(0, message_1.IMessageService)
        ], DefaultSaveErrorHandler);
        return DefaultSaveErrorHandler;
    }());
    // Diagnostics support
    var diag;
    if (!diag) {
        diag = diagnostics.register('TextFileEditorModelDiagnostics', function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            console.log(args[1] + ' - ' + args[0] + ' (time: ' + args[2].getTime() + ' [' + args[2].toUTCString() + '])');
        });
    }
    /**
     * States the text text file editor model can be in.
     */
    (function (State) {
        State[State["SAVED"] = 0] = "SAVED";
        State[State["DIRTY"] = 1] = "DIRTY";
        State[State["PENDING_SAVE"] = 2] = "PENDING_SAVE";
        State[State["CONFLICT"] = 3] = "CONFLICT";
        State[State["ERROR"] = 4] = "ERROR";
    })(exports.State || (exports.State = {}));
    var State = exports.State;
    /**
     * The text file editor model listens to changes to its underlying code editor model and saves these changes through the file service back to the disk.
     */
    var TextFileEditorModel = (function (_super) {
        __extends(TextFileEditorModel, _super);
        function TextFileEditorModel(resource, preferredEncoding, messageService, modeService, modelService, eventService, fileService, instantiationService, telemetryService, textFileService) {
            _super.call(this, modelService, modeService);
            this.messageService = messageService;
            this.eventService = eventService;
            this.fileService = fileService;
            this.instantiationService = instantiationService;
            this.telemetryService = telemetryService;
            this.textFileService = textFileService;
            this.resource = resource;
            if (this.resource.scheme !== 'file') {
                throw new Error('TextFileEditorModel can only handle file:// resources.');
            }
            this.preferredEncoding = preferredEncoding;
            this.textModelChangeListener = null;
            this.dirty = false;
            this.autoSavePromises = [];
            this.versionId = 0;
            this.lastDirtyTime = 0;
            this.mapPendingSaveToVersionId = {};
            this.updateAutoSaveConfiguration(textFileService.getAutoSaveConfiguration());
            this.registerListeners();
        }
        TextFileEditorModel.prototype.registerListeners = function () {
            var _this = this;
            this.textFileServiceListener = this.textFileService.onAutoSaveConfigurationChange(function (config) { return _this.updateAutoSaveConfiguration(config); });
        };
        TextFileEditorModel.prototype.updateAutoSaveConfiguration = function (config) {
            if (typeof config.autoSaveDelay === 'number' && config.autoSaveDelay > 0) {
                this.autoSaveAfterMillies = config.autoSaveDelay;
                this.autoSaveAfterMilliesEnabled = true;
            }
            else {
                this.autoSaveAfterMillies = void 0;
                this.autoSaveAfterMilliesEnabled = false;
            }
        };
        /**
         * Set a save error handler to install code that executes when save errors occur.
         */
        TextFileEditorModel.setSaveErrorHandler = function (handler) {
            TextFileEditorModel.saveErrorHandler = handler;
        };
        /**
         * When set, will disable any saving (including auto save) until the model is loaded again. This allows to resolve save conflicts
         * without running into subsequent save errors when editing the model.
         */
        TextFileEditorModel.prototype.setConflictResolutionMode = function () {
            diag('setConflictResolutionMode() - enabled conflict resolution mode', this.resource, new Date());
            this.inConflictResolutionMode = true;
        };
        /**
         * Answers if this model is currently in conflic resolution mode or not.
         */
        TextFileEditorModel.prototype.isInConflictResolutionMode = function () {
            return this.inConflictResolutionMode;
        };
        /**
         * Discards any local changes and replaces the model with the contents of the version on disk.
         */
        TextFileEditorModel.prototype.revert = function () {
            var _this = this;
            if (!this.isResolved()) {
                return winjs_base_1.TPromise.as(null);
            }
            // Cancel any running auto-saves
            this.cancelAutoSavePromises();
            // Be prepared to send out a file change event in case reverting changes anything
            var oldStat = this.cloneStat(this.versionOnDiskStat);
            // Unset flags
            var undo = this.setDirty(false);
            // Reload
            return this.load(true /* force */).then(function () {
                // Emit file change event
                _this.emitEvent(files_1.EventType.FILE_REVERTED, new files_1.TextFileChangeEvent(_this.textEditorModel, oldStat, _this.versionOnDiskStat));
            }, function (error) {
                // FileNotFound means the file got deleted meanwhile, so emit revert event because thats ok
                if (error.fileOperationResult === files_2.FileOperationResult.FILE_NOT_FOUND) {
                    _this.emitEvent(files_1.EventType.FILE_REVERTED, new files_1.TextFileChangeEvent(_this.textEditorModel, oldStat, _this.versionOnDiskStat));
                }
                else {
                    undo();
                }
                return winjs_base_1.TPromise.wrapError(error);
            });
        };
        TextFileEditorModel.prototype.load = function (force /* bypass any caches and really go to disk */) {
            var _this = this;
            diag('load() - enter', this.resource, new Date());
            // It is very important to not reload the model when the model is dirty. We only want to reload the model from the disk
            // if no save is pending to avoid data loss. This might cause a save conflict in case the file has been modified on the disk
            // meanwhile, but this is a very low risk.
            if (this.dirty) {
                diag('load() - exit - without loading because model is dirty', this.resource, new Date());
                return winjs_base_1.TPromise.as(this);
            }
            // Decide on etag
            var etag;
            if (force) {
                etag = undefined; // bypass cache if force loading is true
            }
            else if (this.versionOnDiskStat) {
                etag = this.versionOnDiskStat.etag; // otherwise respect etag to support caching
            }
            // Resolve Content
            return this.fileService.resolveContent(this.resource, { acceptTextOnly: true, etag: etag, encoding: this.preferredEncoding }).then(function (content) {
                diag('load() - resolved content', _this.resource, new Date());
                // Telemetry
                _this.telemetryService.publicLog('fileGet', { mimeType: content.mime, ext: paths.extname(_this.resource.fsPath), path: telemetry_1.anonymize(_this.resource.fsPath) });
                // Update our resolved disk stat model
                var resolvedStat = {
                    resource: _this.resource,
                    name: content.name,
                    mtime: content.mtime,
                    etag: content.etag,
                    mime: content.mime,
                    isDirectory: false,
                    hasChildren: false,
                    children: void 0,
                };
                _this.updateVersionOnDiskStat(resolvedStat);
                // Keep the original encoding to not loose it when saving
                var oldEncoding = _this.contentEncoding;
                _this.contentEncoding = content.encoding;
                // Handle events if encoding changed
                if (_this.preferredEncoding) {
                    _this.updatePreferredEncoding(_this.contentEncoding); // make sure to reflect the real encoding of the file (never out of sync)
                }
                else if (oldEncoding !== _this.contentEncoding) {
                    _this.eventService.emit(events_1.EventType.RESOURCE_ENCODING_CHANGED, new events_1.ResourceEvent(_this.resource));
                }
                // Update Existing Model
                if (_this.textEditorModel) {
                    diag('load() - updated text editor model', _this.resource, new Date());
                    _this.setDirty(false); // Ensure we are not tracking a stale state
                    _this.blockModelContentChange = true;
                    try {
                        _this.updateTextEditorModel(content.value);
                    }
                    finally {
                        _this.blockModelContentChange = false;
                    }
                    return winjs_base_1.TPromise.as(_this);
                }
                else if (_this.createTextEditorModelPromise) {
                    diag('load() - join existing text editor model promise', _this.resource, new Date());
                    return _this.createTextEditorModelPromise;
                }
                else {
                    diag('load() - created text editor model', _this.resource, new Date());
                    _this.createTextEditorModelPromise = _this.createTextEditorModel(content.value, content.resource).then(function () {
                        _this.createTextEditorModelPromise = null;
                        _this.setDirty(false); // Ensure we are not tracking a stale state
                        _this.textModelChangeListener = _this.textEditorModel.addListener(editorCommon_1.EventType.ModelContentChanged, function (e) { return _this.onModelContentChanged(e); });
                        return _this;
                    }, function (error) {
                        _this.createTextEditorModelPromise = null;
                        return winjs_base_1.TPromise.wrapError(error);
                    });
                    return _this.createTextEditorModelPromise;
                }
            }, function (error) {
                // NotModified status code is expected and can be handled gracefully
                if (error.fileOperationResult === files_2.FileOperationResult.FILE_NOT_MODIFIED_SINCE) {
                    _this.setDirty(false); // Ensure we are not tracking a stale state
                    return winjs_base_1.TPromise.as(_this);
                }
                // Otherwise bubble up the error
                return winjs_base_1.TPromise.wrapError(error);
            });
        };
        TextFileEditorModel.prototype.getOrCreateMode = function (modeService, preferredModeIds, firstLineText) {
            return modeService.getOrCreateModeByFilenameOrFirstLine(this.resource.fsPath, firstLineText);
        };
        TextFileEditorModel.prototype.onModelContentChanged = function (e) {
            diag('onModelContentChanged(' + e.changeType + ') - enter', this.resource, new Date());
            // In any case increment the version id because it tracks the textual content state of the model at all times
            this.versionId++;
            diag('onModelContentChanged() - new versionId ' + this.versionId, this.resource, new Date());
            // Ignore if blocking model changes
            if (this.blockModelContentChange) {
                return;
            }
            // The contents changed as a matter of Undo and the version reached matches the saved one
            // In this case we clear the dirty flag and emit a SAVED event to indicate this state.
            // Note: we currently only do this check when auto-save is turned off because there you see
            // a dirty indicator that you want to get rid of when undoing to the saved version.
            if (!this.autoSaveAfterMilliesEnabled && this.textEditorModel.getAlternativeVersionId() === this.bufferSavedVersionId) {
                diag('onModelContentChanged() - model content changed back to last saved version', this.resource, new Date());
                // Clear flags
                this.setDirty(false);
                // Emit event
                this.emitEvent(files_1.EventType.FILE_REVERTED, new files_1.TextFileChangeEvent(this.textEditorModel, this.versionOnDiskStat));
                return;
            }
            diag('onModelContentChanged() - model content changed and marked as dirty', this.resource, new Date());
            // Mark as dirty
            this.makeDirty(e);
            // Start auto save process unless we are in conflict resolution mode and unless it is disabled
            if (this.autoSaveAfterMilliesEnabled) {
                if (!this.inConflictResolutionMode) {
                    this.doAutoSave(this.versionId);
                }
                else {
                    diag('makeDirty() - prevented save because we are in conflict resolution mode', this.resource, new Date());
                }
            }
        };
        TextFileEditorModel.prototype.makeDirty = function (e) {
            // Track dirty state and version id
            var wasDirty = this.dirty;
            this.setDirty(true);
            this.lastDirtyTime = new Date().getTime();
            // Emit as Event if we turned dirty
            if (!wasDirty) {
                var stat = this.cloneStat(this.versionOnDiskStat);
                this.emitEvent(files_1.EventType.FILE_DIRTY, new files_1.TextFileChangeEvent(this.textEditorModel, stat, stat, e));
            }
        };
        TextFileEditorModel.prototype.doAutoSave = function (versionId) {
            var _this = this;
            diag('doAutoSave() - enter for versionId ' + versionId, this.resource, new Date());
            // Cancel any currently running auto saves to make this the one that succeeds
            this.cancelAutoSavePromises();
            // Create new save promise and keep it
            var promise = winjs_base_1.TPromise.timeout(this.autoSaveAfterMillies).then(function () {
                // Only trigger save if the version id has not changed meanwhile
                if (versionId === _this.versionId) {
                    _this.doSave(versionId, true); // Very important here to not return the promise because if the timeout promise is canceled it will bubble up the error otherwise - do not change
                }
            });
            this.autoSavePromises.push(promise);
            return promise;
        };
        TextFileEditorModel.prototype.cancelAutoSavePromises = function () {
            while (this.autoSavePromises.length) {
                this.autoSavePromises.pop().cancel();
            }
        };
        /**
         * Saves the current versionId of this editor model if it is dirty.
         */
        TextFileEditorModel.prototype.save = function (overwriteReadonly, overwriteEncoding) {
            if (!this.isResolved()) {
                return winjs_base_1.TPromise.as(null);
            }
            diag('save() - enter', this.resource, new Date());
            // Cancel any currently running auto saves to make this the one that succeeds
            this.cancelAutoSavePromises();
            return this.doSave(this.versionId, false, overwriteReadonly, overwriteEncoding);
        };
        TextFileEditorModel.prototype.doSave = function (versionId, isAutoSave, overwriteReadonly, overwriteEncoding) {
            var _this = this;
            diag('doSave(' + versionId + ') - enter with versionId ' + versionId, this.resource, new Date());
            // Lookup any running pending save for this versionId and return it if found
            var pendingSave = this.mapPendingSaveToVersionId[versionId];
            if (pendingSave) {
                diag('doSave(' + versionId + ') - exit - found a pending save for versionId ' + versionId, this.resource, new Date());
                return pendingSave;
            }
            // Return early if not dirty or version changed meanwhile
            if (!this.dirty || versionId !== this.versionId) {
                diag('doSave(' + versionId + ') - exit - because not dirty and/or versionId is different (this.isDirty: ' + this.dirty + ', this.versionId: ' + this.versionId + ')', this.resource, new Date());
                return winjs_base_1.TPromise.as(null);
            }
            // Return if currently saving by scheduling another auto save. Never ever must 2 saves execute at the same time because
            // this can lead to dirty writes and race conditions
            if (this.isBusySaving()) {
                diag('doSave(' + versionId + ') - exit - because busy saving', this.resource, new Date());
                // Avoid endless loop here and guard if auto save is disabled
                if (this.autoSaveAfterMilliesEnabled) {
                    return this.doAutoSave(versionId);
                }
            }
            // Push all edit operations to the undo stack so that the user has a chance to
            // Ctrl+Z back to the saved version. We only do this when auto-save is turned off
            if (!this.autoSaveAfterMilliesEnabled) {
                this.textEditorModel.pushStackElement();
            }
            // Emit file saving event: Listeners can still change the model now and since we are so close to saving
            // we do not want to trigger another auto save or similar, so we block this
            // In addition we update our version right after in case it changed because of a model change
            var versionOnDiskStatClone = this.cloneStat(this.versionOnDiskStat);
            this.blockModelContentChange = true;
            try {
                var saveEvent = new files_1.TextFileChangeEvent(this.textEditorModel, versionOnDiskStatClone);
                saveEvent.setAutoSaved(isAutoSave);
                this.emitEvent(files_1.EventType.FILE_SAVING, saveEvent);
            }
            finally {
                this.blockModelContentChange = false;
            }
            versionId = this.versionId;
            // Clear error flag since we are trying to save again
            this.inErrorMode = false;
            // Save to Disk
            diag('doSave(' + versionId + ') - before updateContent()', this.resource, new Date());
            this.mapPendingSaveToVersionId[versionId] = this.fileService.updateContent(this.versionOnDiskStat.resource, this.getValue(), {
                overwriteReadonly: overwriteReadonly,
                overwriteEncoding: overwriteEncoding,
                mtime: this.versionOnDiskStat.mtime,
                encoding: this.getEncoding(),
                etag: this.versionOnDiskStat.etag
            }).then(function (stat) {
                diag('doSave(' + versionId + ') - after updateContent()', _this.resource, new Date());
                // Telemetry
                _this.telemetryService.publicLog('filePUT', { mimeType: stat.mime, ext: paths.extname(_this.versionOnDiskStat.resource.fsPath) });
                // Remove from pending saves
                delete _this.mapPendingSaveToVersionId[versionId];
                // Update dirty state unless model has changed meanwhile
                if (versionId === _this.versionId) {
                    diag('doSave(' + versionId + ') - setting dirty to false because versionId did not change', _this.resource, new Date());
                    _this.setDirty(false);
                }
                else {
                    diag('doSave(' + versionId + ') - not setting dirty to false because versionId did change meanwhile', _this.resource, new Date());
                }
                // Updated resolved stat with updated stat, and keep old for event
                var oldStat = _this.versionOnDiskStat;
                _this.updateVersionOnDiskStat(stat);
                // Emit File Change Event
                var oldValue = _this.cloneStat(oldStat);
                var newValue = _this.cloneStat(_this.versionOnDiskStat);
                _this.emitEvent('files.internal:fileChanged', new files_1.TextFileChangeEvent(_this.textEditorModel, oldValue, newValue));
                // Emit File Saved Event
                _this.emitEvent(files_1.EventType.FILE_SAVED, new files_1.TextFileChangeEvent(_this.textEditorModel, oldValue, newValue));
            }, function (error) {
                diag('doSave(' + versionId + ') - exit - resulted in a save error: ' + error.toString(), _this.resource, new Date());
                // Remove from pending saves
                delete _this.mapPendingSaveToVersionId[versionId];
                // Flag as error state
                _this.inErrorMode = true;
                // Show to user
                _this.onSaveError(error);
                // Emit as event
                _this.emitEvent(files_1.EventType.FILE_SAVE_ERROR, new files_1.TextFileChangeEvent(_this.textEditorModel, versionOnDiskStatClone));
            });
            return this.mapPendingSaveToVersionId[versionId];
        };
        TextFileEditorModel.prototype.setDirty = function (dirty) {
            var _this = this;
            var wasDirty = this.dirty;
            var wasInConflictResolutionMode = this.inConflictResolutionMode;
            var wasInErrorMode = this.inErrorMode;
            var oldBufferSavedVersionId = this.bufferSavedVersionId;
            if (!dirty) {
                this.dirty = false;
                this.inConflictResolutionMode = false;
                this.inErrorMode = false;
                // we remember the models alternate version id to remember when the version
                // of the model matches with the saved version on disk. we need to keep this
                // in order to find out if the model changed back to a saved version (e.g.
                // when undoing long enough to reach to a version that is saved and then to
                // clear the dirty flag)
                if (this.textEditorModel) {
                    this.bufferSavedVersionId = this.textEditorModel.getAlternativeVersionId();
                }
            }
            else {
                this.dirty = true;
            }
            // Return function to revert this call
            return function () {
                _this.dirty = wasDirty;
                _this.inConflictResolutionMode = wasInConflictResolutionMode;
                _this.inErrorMode = wasInErrorMode;
                _this.bufferSavedVersionId = oldBufferSavedVersionId;
            };
        };
        TextFileEditorModel.prototype.updateVersionOnDiskStat = function (newVersionOnDiskStat) {
            // First resolve - just take
            if (!this.versionOnDiskStat) {
                this.versionOnDiskStat = newVersionOnDiskStat;
            }
            else if (this.versionOnDiskStat.mtime <= newVersionOnDiskStat.mtime) {
                this.versionOnDiskStat = newVersionOnDiskStat;
            }
        };
        TextFileEditorModel.prototype.onSaveError = function (error) {
            // Prepare handler
            if (!TextFileEditorModel.saveErrorHandler) {
                TextFileEditorModel.setSaveErrorHandler(this.instantiationService.createInstance(DefaultSaveErrorHandler));
            }
            // Handle
            TextFileEditorModel.saveErrorHandler.onSaveError(error, this);
        };
        TextFileEditorModel.prototype.emitEvent = function (type, event) {
            try {
                this.eventService.emit(type, event);
            }
            catch (e) {
                e.friendlyMessage = nls.localize(1, null, type);
                errors_1.onUnexpectedError(e);
            }
        };
        TextFileEditorModel.prototype.isBusySaving = function () {
            return !types.isEmptyObject(this.mapPendingSaveToVersionId);
        };
        /**
         * Returns true if the content of this model has changes that are not yet saved back to the disk.
         */
        TextFileEditorModel.prototype.isDirty = function () {
            return this.dirty;
        };
        /**
         * Returns the time in millies when this working copy was edited by the user.
         */
        TextFileEditorModel.prototype.getLastDirtyTime = function () {
            return this.lastDirtyTime;
        };
        /**
         * Returns the time in millies when this working copy was last modified by the user or some other program.
         */
        TextFileEditorModel.prototype.getLastModifiedTime = function () {
            return this.versionOnDiskStat ? this.versionOnDiskStat.mtime : -1;
        };
        /**
         * Returns the state this text text file editor model is in with regards to changes and saving.
         */
        TextFileEditorModel.prototype.getState = function () {
            if (this.inConflictResolutionMode) {
                return State.CONFLICT;
            }
            if (this.inErrorMode) {
                return State.ERROR;
            }
            if (!this.dirty) {
                return State.SAVED;
            }
            if (this.isBusySaving()) {
                return State.PENDING_SAVE;
            }
            if (this.dirty) {
                return State.DIRTY;
            }
        };
        TextFileEditorModel.prototype.getEncoding = function () {
            return this.preferredEncoding || this.contentEncoding;
        };
        TextFileEditorModel.prototype.setEncoding = function (encoding, mode) {
            if (!this.isNewEncoding(encoding)) {
                return; // return early if the encoding is already the same
            }
            // Encode: Save with encoding
            if (mode === editor_1.EncodingMode.Encode) {
                this.updatePreferredEncoding(encoding);
                // Save
                if (!this.isDirty()) {
                    this.versionId++; // needs to increment because we change the model potentially
                    this.makeDirty();
                }
                if (!this.inConflictResolutionMode) {
                    this.save(false, true /* overwriteEncoding due to forced encoding change */).done(null, errors_1.onUnexpectedError);
                }
            }
            else {
                if (this.isDirty()) {
                    this.messageService.show(message_1.Severity.Info, nls.localize(2, null));
                    return;
                }
                this.updatePreferredEncoding(encoding);
                // Load
                this.load(true /* force because encoding has changed */).done(null, errors_1.onUnexpectedError);
            }
        };
        TextFileEditorModel.prototype.updatePreferredEncoding = function (encoding) {
            if (!this.isNewEncoding(encoding)) {
                return;
            }
            this.preferredEncoding = encoding;
            // Emit
            this.eventService.emit(events_1.EventType.RESOURCE_ENCODING_CHANGED, new events_1.ResourceEvent(this.resource));
        };
        TextFileEditorModel.prototype.isNewEncoding = function (encoding) {
            if (this.preferredEncoding === encoding) {
                return false; // return early if the encoding is already the same
            }
            if (!this.preferredEncoding && this.contentEncoding === encoding) {
                return false; // also return if we don't have a preferred encoding but the content encoding is already the same
            }
            return true;
        };
        TextFileEditorModel.prototype.isResolved = function () {
            return !types.isUndefinedOrNull(this.versionOnDiskStat);
        };
        /**
         * Returns true if the dispose() method of this model has been called.
         */
        TextFileEditorModel.prototype.isDisposed = function () {
            return this.disposed;
        };
        /**
         * Returns the full resource URI of the file this text file editor model is about.
         */
        TextFileEditorModel.prototype.getResource = function () {
            return this.resource;
        };
        TextFileEditorModel.prototype.dispose = function () {
            this.disposed = true;
            this.inConflictResolutionMode = false;
            this.inErrorMode = false;
            this.createTextEditorModelPromise = null;
            if (this.textModelChangeListener) {
                this.textModelChangeListener();
                this.textModelChangeListener = null;
            }
            if (this.textFileServiceListener) {
                this.textFileServiceListener.dispose();
                this.textFileServiceListener = null;
            }
            this.cancelAutoSavePromises();
            exports.CACHE.remove(this.resource);
            _super.prototype.dispose.call(this);
        };
        TextFileEditorModel.prototype.cloneStat = function (stat) {
            return {
                resource: uri_1.default.parse(stat.resource.toString()),
                name: stat.name,
                mtime: stat.mtime,
                etag: stat.etag,
                mime: stat.mime,
                isDirectory: stat.isDirectory,
                hasChildren: stat.hasChildren,
                children: stat.children
            };
        };
        TextFileEditorModel.ID = 'workbench.editors.files.textFileEditorModel';
        TextFileEditorModel = __decorate([
            __param(2, message_1.IMessageService),
            __param(3, modeService_1.IModeService),
            __param(4, modelService_1.IModelService),
            __param(5, event_1.IEventService),
            __param(6, files_2.IFileService),
            __param(7, instantiation_1.IInstantiationService),
            __param(8, telemetry_1.ITelemetryService),
            __param(9, files_1.ITextFileService)
        ], TextFileEditorModel);
        return TextFileEditorModel;
    }(textEditorModel_1.BaseTextEditorModel));
    exports.TextFileEditorModel = TextFileEditorModel;
    var TextFileEditorModelCache = (function () {
        function TextFileEditorModelCache() {
            this.mapResourcePathToModel = Object.create(null);
        }
        TextFileEditorModelCache.prototype.dispose = function (resource) {
            var model = this.get(resource);
            if (model) {
                if (model.isDirty()) {
                    return; // we never dispose dirty models to avoid data loss
                }
                model.dispose();
            }
        };
        TextFileEditorModelCache.prototype.get = function (resource) {
            return this.mapResourcePathToModel[resource.toString()];
        };
        TextFileEditorModelCache.prototype.getAll = function (resource) {
            var _this = this;
            return Object.keys(this.mapResourcePathToModel)
                .filter(function (r) { return !resource || resource.toString() === r; })
                .map(function (r) { return _this.mapResourcePathToModel[r]; });
        };
        TextFileEditorModelCache.prototype.add = function (resource, model) {
            this.mapResourcePathToModel[resource.toString()] = model;
        };
        // Clients should not call this method
        TextFileEditorModelCache.prototype.clear = function () {
            this.mapResourcePathToModel = Object.create(null);
        };
        // Clients should not call this method
        TextFileEditorModelCache.prototype.remove = function (resource) {
            delete this.mapResourcePathToModel[resource.toString()];
        };
        return TextFileEditorModelCache;
    }());
    exports.TextFileEditorModelCache = TextFileEditorModelCache;
    exports.CACHE = new TextFileEditorModelCache();
});

define("vs/workbench/parts/git/common/git", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // Model enums
    (function (StatusType) {
        StatusType[StatusType["INDEX"] = 0] = "INDEX";
        StatusType[StatusType["WORKING_TREE"] = 1] = "WORKING_TREE";
        StatusType[StatusType["MERGE"] = 2] = "MERGE";
    })(exports.StatusType || (exports.StatusType = {}));
    var StatusType = exports.StatusType;
    (function (Status) {
        Status[Status["INDEX_MODIFIED"] = 0] = "INDEX_MODIFIED";
        Status[Status["INDEX_ADDED"] = 1] = "INDEX_ADDED";
        Status[Status["INDEX_DELETED"] = 2] = "INDEX_DELETED";
        Status[Status["INDEX_RENAMED"] = 3] = "INDEX_RENAMED";
        Status[Status["INDEX_COPIED"] = 4] = "INDEX_COPIED";
        Status[Status["MODIFIED"] = 5] = "MODIFIED";
        Status[Status["DELETED"] = 6] = "DELETED";
        Status[Status["UNTRACKED"] = 7] = "UNTRACKED";
        Status[Status["IGNORED"] = 8] = "IGNORED";
        Status[Status["ADDED_BY_US"] = 9] = "ADDED_BY_US";
        Status[Status["ADDED_BY_THEM"] = 10] = "ADDED_BY_THEM";
        Status[Status["DELETED_BY_US"] = 11] = "DELETED_BY_US";
        Status[Status["DELETED_BY_THEM"] = 12] = "DELETED_BY_THEM";
        Status[Status["BOTH_ADDED"] = 13] = "BOTH_ADDED";
        Status[Status["BOTH_DELETED"] = 14] = "BOTH_DELETED";
        Status[Status["BOTH_MODIFIED"] = 15] = "BOTH_MODIFIED";
    })(exports.Status || (exports.Status = {}));
    var Status = exports.Status;
    // Model events
    exports.ModelEvents = {
        MODEL_UPDATED: 'ModelUpdated',
        STATUS_MODEL_UPDATED: 'StatusModelUpdated',
        HEAD_UPDATED: 'HEADUpdated',
        HEADS_UPDATED: 'HEADSUpdated',
        TAGS_UPDATED: 'TagsUpdated',
        REMOTES_UPDATED: 'RemotesUpdated'
    };
    // Service enums
    (function (ServiceState) {
        ServiceState[ServiceState["NotInitialized"] = 0] = "NotInitialized";
        ServiceState[ServiceState["NotARepo"] = 1] = "NotARepo";
        ServiceState[ServiceState["NotAtRepoRoot"] = 2] = "NotAtRepoRoot";
        ServiceState[ServiceState["OK"] = 3] = "OK";
        ServiceState[ServiceState["NoGit"] = 4] = "NoGit";
        ServiceState[ServiceState["Disabled"] = 5] = "Disabled";
        ServiceState[ServiceState["NotAWorkspace"] = 6] = "NotAWorkspace";
    })(exports.ServiceState || (exports.ServiceState = {}));
    var ServiceState = exports.ServiceState;
    (function (RawServiceState) {
        RawServiceState[RawServiceState["OK"] = 0] = "OK";
        RawServiceState[RawServiceState["GitNotFound"] = 1] = "GitNotFound";
        RawServiceState[RawServiceState["Disabled"] = 2] = "Disabled";
    })(exports.RawServiceState || (exports.RawServiceState = {}));
    var RawServiceState = exports.RawServiceState;
    exports.GitErrorCodes = {
        BadConfigFile: 'BadConfigFile',
        AuthenticationFailed: 'AuthenticationFailed',
        NoUserNameConfigured: 'NoUserNameConfigured',
        NoUserEmailConfigured: 'NoUserEmailConfigured',
        NoRemoteRepositorySpecified: 'NoRemoteRepositorySpecified',
        NotAGitRepository: 'NotAGitRepository',
        NotAtRepositoryRoot: 'NotAtRepositoryRoot',
        Conflict: 'Conflict',
        UnmergedChanges: 'UnmergedChanges',
        PushRejected: 'PushRejected',
        RemoteConnectionError: 'RemoteConnectionError',
        DirtyWorkTree: 'DirtyWorkTree',
        CantOpenResource: 'CantOpenResource',
        GitNotFound: 'GitNotFound',
        CantCreatePipe: 'CantCreatePipe',
        CantAccessRemote: 'CantAccessRemote',
        RepositoryNotFound: 'RepositoryNotFound'
    };
    (function (AutoFetcherState) {
        AutoFetcherState[AutoFetcherState["Disabled"] = 0] = "Disabled";
        AutoFetcherState[AutoFetcherState["Inactive"] = 1] = "Inactive";
        AutoFetcherState[AutoFetcherState["Active"] = 2] = "Active";
        AutoFetcherState[AutoFetcherState["Fetching"] = 3] = "Fetching";
    })(exports.AutoFetcherState || (exports.AutoFetcherState = {}));
    var AutoFetcherState = exports.AutoFetcherState;
    // Service events
    exports.ServiceEvents = {
        STATE_CHANGED: 'stateChanged',
        REPO_CHANGED: 'repoChanged',
        OPERATION_START: 'operationStart',
        OPERATION_END: 'operationEnd',
        OPERATION: 'operation',
        ERROR: 'error',
        DISPOSE: 'dispose'
    };
    // Service operations
    exports.ServiceOperations = {
        STATUS: 'status',
        INIT: 'init',
        ADD: 'add',
        STAGE: 'stage',
        BRANCH: 'branch',
        CHECKOUT: 'checkout',
        CLEAN: 'clean',
        UNDO: 'undo',
        RESET: 'reset',
        COMMIT: 'commit',
        COMMAND: 'command',
        BACKGROUND_FETCH: 'backgroundfetch',
        PULL: 'pull',
        PUSH: 'push',
        SYNC: 'sync'
    };
    exports.GIT_SERVICE_ID = 'gitService';
    exports.IGitService = instantiation_1.createDecorator(exports.GIT_SERVICE_ID);
    // Utils
    function isValidBranchName(value) {
        return !/^\.|\/\.|\.\.|~|\^|:|\/$|\.lock$|\.lock\/|\\|\*|\s|^\s*$/.test(value);
    }
    exports.isValidBranchName = isValidBranchName;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/git/browser/gitEditorContributions", ["require", "exports", 'vs/editor/common/editorCommon', 'vs/workbench/parts/git/common/git', 'vs/platform/workspace/common/workspace', 'vs/base/common/lifecycle', 'vs/base/common/async'], function (require, exports, common, git, workspace_1, lifecycle_1, async_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var IGitService = git.IGitService;
    var MergeDecoratorBoundToModel = (function (_super) {
        __extends(MergeDecoratorBoundToModel, _super);
        function MergeDecoratorBoundToModel(editor, model, filePath, gitService) {
            var _this = this;
            _super.call(this);
            this._editor = editor;
            this._model = model;
            this._gitService = gitService;
            this._filePath = filePath;
            this._decorations = [];
            this._redecorateSoon = this._register(new async_1.RunOnceScheduler(function () { return _this.redecorate(); }, 300));
            this._register(this._model.addListener2(common.EventType.ModelContentChanged, function () { return _this._redecorateSoon.schedule(); }));
            this._register(this._gitService.addListener2(git.ServiceEvents.STATE_CHANGED, function () { return _this._redecorateSoon.schedule(); }));
            this._redecorateSoon.schedule();
        }
        MergeDecoratorBoundToModel.prototype.dispose = function () {
            this._setDecorations([]);
            _super.prototype.dispose.call(this);
        };
        MergeDecoratorBoundToModel.prototype._setDecorations = function (newDecorations) {
            this._decorations = this._editor.deltaDecorations(this._decorations, newDecorations);
        };
        MergeDecoratorBoundToModel.prototype.redecorate = function () {
            var gitModel = this._gitService.getModel();
            var mergeStatus = gitModel.getStatus().find(this._filePath, git.StatusType.MERGE);
            if (!mergeStatus) {
                return;
            }
            var decorations = [];
            var lineCount = this._model.getLineCount();
            for (var i = 1; i <= lineCount; i++) {
                var start = this._model.getLineContent(i).substr(0, 7);
                switch (start) {
                    case '<<<<<<<':
                    case '=======':
                    case '>>>>>>>':
                        decorations.push({
                            range: { startLineNumber: i, startColumn: 1, endLineNumber: i, endColumn: 1 },
                            options: MergeDecorator.DECORATION_OPTIONS
                        });
                        break;
                }
            }
            this._setDecorations(decorations);
        };
        return MergeDecoratorBoundToModel;
    }(lifecycle_1.Disposable));
    var MergeDecorator = (function () {
        function MergeDecorator(editor, gitService, contextService) {
            this.gitService = gitService;
            this.contextService = contextService;
            this.editor = editor;
            this.toUnbind = [this.editor.addListener(common.EventType.ModelChanged, this.onModelChanged.bind(this))];
            this.mergeDecorator = null;
        }
        MergeDecorator.prototype.getId = function () {
            return MergeDecorator.ID;
        };
        MergeDecorator.prototype.onModelChanged = function () {
            if (this.mergeDecorator) {
                this.mergeDecorator.dispose();
                this.mergeDecorator = null;
            }
            if (!this.contextService || !this.gitService) {
                return;
            }
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var resource = model.getAssociatedResource();
            if (!resource) {
                return;
            }
            var path = this.contextService.toWorkspaceRelativePath(resource);
            if (!path) {
                return;
            }
            this.mergeDecorator = new MergeDecoratorBoundToModel(this.editor, model, path, this.gitService);
        };
        MergeDecorator.prototype.dispose = function () {
            if (this.mergeDecorator) {
                this.mergeDecorator.dispose();
                this.mergeDecorator = null;
            }
            while (this.toUnbind.length) {
                this.toUnbind.pop()();
            }
        };
        MergeDecorator.ID = 'Monaco.IDE.UI.Viewlets.GitViewlet.Editor.MergeDecorator';
        MergeDecorator.DECORATION_OPTIONS = {
            className: 'git-merge-control-decoration',
            isWholeLine: true,
            overviewRuler: {
                color: 'rgb(197, 118, 0)',
                darkColor: 'rgb(197, 118, 0)',
                position: common.OverviewRulerLane.Left
            },
            stickiness: common.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        MergeDecorator = __decorate([
            __param(1, IGitService),
            __param(2, workspace_1.IWorkspaceContextService)
        ], MergeDecorator);
        return MergeDecorator;
    }());
    exports.MergeDecorator = MergeDecorator;
});

define("vs/workbench/parts/git/browser/gitOperations", ["require", "exports", 'vs/workbench/parts/git/common/git', 'vs/base/common/winjs.base'], function (require, exports, git, winjs) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var GitOperation = (function () {
        function GitOperation(id, fn) {
            this.id = id;
            this.fn = fn;
            // noop
        }
        GitOperation.prototype.run = function () {
            return this.fn();
        };
        GitOperation.prototype.dispose = function () {
            // noop
        };
        return GitOperation;
    }());
    exports.GitOperation = GitOperation;
    var CommandOperation = (function () {
        function CommandOperation(input) {
            this.input = input;
            this.id = git.ServiceOperations.COMMAND;
            // noop
        }
        CommandOperation.prototype.run = function () {
            return winjs.TPromise.as(null);
        };
        CommandOperation.prototype.dispose = function () {
            this.id = null;
            this.input = null;
        };
        return CommandOperation;
    }());
    exports.CommandOperation = CommandOperation;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/git/common/gitModel", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/common/strings', 'vs/base/common/eventEmitter', 'vs/workbench/parts/git/common/git'], function (require, exports, Lifecycle, Strings, EventEmitter, Git) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FileStatus = (function () {
        function FileStatus(path, mimetype, status, rename, isModifiedInIndex) {
            this.path = path;
            this.mimetype = mimetype;
            this.status = status;
            this.rename = rename;
            this.id = FileStatus.typeOf(status) + ':' + path + (rename ? ':' + rename : '') + (isModifiedInIndex ? '$' : '');
            this.pathComponents = path.split('/');
        }
        FileStatus.prototype.getPath = function () {
            return this.path;
        };
        FileStatus.prototype.getPathComponents = function () {
            return this.pathComponents.slice(0);
        };
        FileStatus.prototype.getMimetype = function () {
            return this.mimetype;
        };
        FileStatus.prototype.getStatus = function () {
            return this.status;
        };
        FileStatus.prototype.getRename = function () {
            return this.rename;
        };
        FileStatus.prototype.getId = function () {
            return this.id;
        };
        FileStatus.prototype.getType = function () {
            switch (FileStatus.typeOf(this.status)) {
                case 'index': return Git.StatusType.INDEX;
                case 'workingTree': return Git.StatusType.WORKING_TREE;
                default: return Git.StatusType.MERGE;
            }
        };
        FileStatus.prototype.clone = function () {
            return new FileStatus(this.path, this.mimetype, this.status, this.rename);
        };
        FileStatus.prototype.update = function (other) {
            this.status = other.getStatus();
            this.rename = other.getRename();
        };
        FileStatus.typeOf = function (s) {
            switch (s) {
                case Git.Status.INDEX_MODIFIED:
                case Git.Status.INDEX_ADDED:
                case Git.Status.INDEX_DELETED:
                case Git.Status.INDEX_RENAMED:
                case Git.Status.INDEX_COPIED:
                    return 'index';
                case Git.Status.MODIFIED:
                case Git.Status.DELETED:
                case Git.Status.UNTRACKED:
                case Git.Status.IGNORED:
                    return 'workingTree';
                default:
                    return 'merge';
            }
        };
        return FileStatus;
    }());
    exports.FileStatus = FileStatus;
    var StatusGroup = (function (_super) {
        __extends(StatusGroup, _super);
        function StatusGroup(type) {
            _super.call(this);
            this.type = type;
            this.statusSet = Object.create(null);
            this.statusList = [];
            this.statusByName = Object.create(null);
            this.statusByRename = Object.create(null);
        }
        StatusGroup.prototype.getType = function () {
            return this.type;
        };
        StatusGroup.prototype.update = function (statusList) {
            var toDelete = Object.create(null);
            var id, path, rename;
            var status;
            for (id in this.statusSet) {
                toDelete[id] = this.statusSet[id];
            }
            for (var i = 0; i < statusList.length; i++) {
                status = statusList[i];
                id = status.getId();
                path = status.getPath();
                rename = status.getRename();
                if (toDelete[id]) {
                    this.statusSet[id].update(status);
                    toDelete[id] = null;
                }
                else {
                    this.statusSet[id] = status;
                }
            }
            for (id in toDelete) {
                if (status = toDelete[id]) {
                    this.emit('fileStatus:dispose', status);
                    delete this.statusSet[id];
                }
            }
            this.statusList = [];
            this.statusByName = Object.create(null);
            this.statusByRename = Object.create(null);
            for (id in this.statusSet) {
                status = this.statusSet[id];
                this.statusList.push(status);
                if (status.getRename()) {
                    this.statusByRename[status.getPath()] = status;
                }
                else {
                    this.statusByName[status.getPath()] = status;
                }
            }
        };
        StatusGroup.prototype.all = function () {
            return this.statusList;
        };
        StatusGroup.prototype.find = function (path) {
            return this.statusByName[path] || this.statusByRename[path] || null;
        };
        StatusGroup.prototype.dispose = function () {
            this.type = null;
            this.statusSet = null;
            this.statusList = null;
            this.statusByName = null;
            this.statusByRename = null;
            _super.prototype.dispose.call(this);
        };
        return StatusGroup;
    }(EventEmitter.EventEmitter));
    exports.StatusGroup = StatusGroup;
    var StatusModel = (function (_super) {
        __extends(StatusModel, _super);
        function StatusModel() {
            _super.call(this);
            this.indexStatus = new StatusGroup(Git.StatusType.INDEX);
            this.workingTreeStatus = new StatusGroup(Git.StatusType.WORKING_TREE);
            this.mergeStatus = new StatusGroup(Git.StatusType.MERGE);
            this.toDispose = [
                this.addEmitter2(this.indexStatus),
                this.addEmitter2(this.workingTreeStatus),
                this.addEmitter2(this.mergeStatus)
            ];
        }
        StatusModel.prototype.getSummary = function () {
            return {
                hasWorkingTreeChanges: this.getWorkingTreeStatus().all().length > 0,
                hasIndexChanges: this.getIndexStatus().all().length > 0,
                hasMergeChanges: this.getMergeStatus().all().length > 0
            };
        };
        StatusModel.prototype.update = function (status) {
            var index = [];
            var workingTree = [];
            var merge = [];
            status.forEach(function (raw) {
                switch (raw.x + raw.y) {
                    case '??': return workingTree.push(new FileStatus(raw.path, raw.mimetype, Git.Status.UNTRACKED));
                    case '!!': return workingTree.push(new FileStatus(raw.path, raw.mimetype, Git.Status.IGNORED));
                    case 'DD': return merge.push(new FileStatus(raw.path, raw.mimetype, Git.Status.BOTH_DELETED));
                    case 'AU': return merge.push(new FileStatus(raw.path, raw.mimetype, Git.Status.ADDED_BY_US));
                    case 'UD': return merge.push(new FileStatus(raw.path, raw.mimetype, Git.Status.DELETED_BY_THEM));
                    case 'UA': return merge.push(new FileStatus(raw.path, raw.mimetype, Git.Status.ADDED_BY_THEM));
                    case 'DU': return merge.push(new FileStatus(raw.path, raw.mimetype, Git.Status.DELETED_BY_US));
                    case 'AA': return merge.push(new FileStatus(raw.path, raw.mimetype, Git.Status.BOTH_ADDED));
                    case 'UU': return merge.push(new FileStatus(raw.path, raw.mimetype, Git.Status.BOTH_MODIFIED));
                }
                var isModifiedInIndex = false;
                switch (raw.x) {
                    case 'M':
                        index.push(new FileStatus(raw.path, raw.mimetype, Git.Status.INDEX_MODIFIED));
                        isModifiedInIndex = true;
                        break;
                    case 'A':
                        index.push(new FileStatus(raw.path, raw.mimetype, Git.Status.INDEX_ADDED));
                        break;
                    case 'D':
                        index.push(new FileStatus(raw.path, raw.mimetype, Git.Status.INDEX_DELETED));
                        break;
                    case 'R':
                        index.push(new FileStatus(raw.path, raw.mimetype, Git.Status.INDEX_RENAMED, raw.rename));
                        break;
                    case 'C':
                        index.push(new FileStatus(raw.path, raw.mimetype, Git.Status.INDEX_COPIED));
                        break;
                }
                switch (raw.y) {
                    case 'M':
                        workingTree.push(new FileStatus(raw.path, raw.mimetype, Git.Status.MODIFIED, raw.rename, isModifiedInIndex));
                        break;
                    case 'D':
                        workingTree.push(new FileStatus(raw.path, raw.mimetype, Git.Status.DELETED, raw.rename));
                        break;
                }
            });
            this.indexStatus.update(index);
            this.workingTreeStatus.update(workingTree);
            this.mergeStatus.update(merge);
            this.emit(Git.ModelEvents.STATUS_MODEL_UPDATED);
        };
        StatusModel.prototype.getIndexStatus = function () {
            return this.indexStatus;
        };
        StatusModel.prototype.getWorkingTreeStatus = function () {
            return this.workingTreeStatus;
        };
        StatusModel.prototype.getMergeStatus = function () {
            return this.mergeStatus;
        };
        StatusModel.prototype.getGroups = function () {
            return [this.mergeStatus, this.indexStatus, this.workingTreeStatus];
        };
        StatusModel.prototype.find = function (path, type) {
            var group;
            switch (type) {
                case Git.StatusType.INDEX:
                    group = this.indexStatus;
                    break;
                case Git.StatusType.WORKING_TREE:
                    group = this.workingTreeStatus;
                    break;
                case Git.StatusType.MERGE:
                    group = this.mergeStatus;
                    break;
                default:
                    return null;
            }
            return group.find(path);
        };
        StatusModel.prototype.dispose = function () {
            this.toDispose = Lifecycle.disposeAll(this.toDispose);
            if (this.indexStatus) {
                this.indexStatus.dispose();
                this.indexStatus = null;
            }
            if (this.workingTreeStatus) {
                this.workingTreeStatus.dispose();
                this.workingTreeStatus = null;
            }
            if (this.mergeStatus) {
                this.mergeStatus.dispose();
                this.mergeStatus = null;
            }
            _super.prototype.dispose.call(this);
        };
        return StatusModel;
    }(EventEmitter.EventEmitter));
    exports.StatusModel = StatusModel;
    var Model = (function (_super) {
        __extends(Model, _super);
        function Model() {
            _super.call(this);
            this.toDispose = [];
            this.repositoryRoot = null;
            this.status = new StatusModel();
            this.toDispose.push(this.addEmitter2(this.status));
            this.HEAD = null;
            this.heads = [];
            this.tags = [];
            this.remotes = [];
        }
        Model.prototype.getRepositoryRoot = function () {
            return this.repositoryRoot;
        };
        Model.prototype.getStatus = function () {
            return this.status;
        };
        Model.prototype.getHEAD = function () {
            return this.HEAD;
        };
        Model.prototype.getHeads = function () {
            return this.heads;
        };
        Model.prototype.getTags = function () {
            return this.tags;
        };
        Model.prototype.getRemotes = function () {
            return this.remotes;
        };
        Model.prototype.update = function (status) {
            if (!status) {
                status = {
                    repositoryRoot: null,
                    status: [],
                    HEAD: null,
                    heads: [],
                    tags: [],
                    remotes: []
                };
            }
            this.repositoryRoot = status.repositoryRoot;
            this.status.update(status.status);
            this.HEAD = status.HEAD;
            this.emit(Git.ModelEvents.HEAD_UPDATED);
            this.heads = status.heads;
            this.emit(Git.ModelEvents.HEADS_UPDATED);
            this.tags = status.tags;
            this.emit(Git.ModelEvents.TAGS_UPDATED);
            this.remotes = status.remotes;
            this.emit(Git.ModelEvents.REMOTES_UPDATED);
            this.emit(Git.ModelEvents.MODEL_UPDATED);
        };
        Model.prototype.getStatusSummary = function () {
            var status = this.getStatus();
            return {
                hasWorkingTreeChanges: status.getWorkingTreeStatus().all().length > 0,
                hasIndexChanges: status.getIndexStatus().all().length > 0,
                hasMergeChanges: status.getMergeStatus().all().length > 0
            };
        };
        Model.prototype.getPS1 = function () {
            if (!this.HEAD) {
                return '';
            }
            var label = this.HEAD.name || this.HEAD.commit.substr(0, 8);
            var statusSummary = this.getStatus().getSummary();
            return Strings.format('{0}{1}{2}{3}', label, statusSummary.hasWorkingTreeChanges ? '*' : '', statusSummary.hasIndexChanges ? '+' : '', statusSummary.hasMergeChanges ? '!' : '');
        };
        Model.prototype.dispose = function () {
            this.toDispose = Lifecycle.disposeAll(this.toDispose);
            _super.prototype.dispose.call(this);
        };
        return Model;
    }(EventEmitter.EventEmitter));
    exports.Model = Model;
});

define("vs/workbench/parts/git/common/noopGitService", ["require", "exports", 'vs/workbench/parts/git/common/git', 'vs/base/common/winjs.base'], function (require, exports, git, winjs_base_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var NoOpGitService = (function () {
        function NoOpGitService() {
        }
        NoOpGitService.prototype.getVersion = function () {
            return winjs_base_1.TPromise.as(null);
        };
        NoOpGitService.prototype.serviceState = function () {
            return winjs_base_1.TPromise.as(git.RawServiceState.OK);
        };
        NoOpGitService.prototype.status = function () {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.init = function () {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.add = function (filesPaths) {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.stage = function (filePath, content) {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.branch = function (name, checkout) {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.checkout = function (treeish, filePaths) {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.clean = function (filePaths) {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.undo = function () {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.reset = function (treeish, hard) {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.revertFiles = function (treeish, filePaths) {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.fetch = function () {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.pull = function (rebase) {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.push = function () {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.sync = function () {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.commit = function (message, amend, stage) {
            return winjs_base_1.TPromise.as(NoOpGitService.STATUS);
        };
        NoOpGitService.prototype.detectMimetypes = function (path, treeish) {
            return winjs_base_1.TPromise.as([]);
        };
        NoOpGitService.prototype.show = function (path, treeish) {
            return winjs_base_1.TPromise.as(null);
        };
        NoOpGitService.prototype.onOutput = function () {
            return winjs_base_1.TPromise.as(function () { return null; });
        };
        NoOpGitService.STATUS = {
            repositoryRoot: null,
            state: git.ServiceState.NotAWorkspace,
            status: [],
            HEAD: null,
            heads: [],
            tags: [],
            remotes: []
        };
        return NoOpGitService;
    }());
    exports.NoOpGitService = NoOpGitService;
});

define("vs/workbench/parts/git/common/stageRanges", ["require", "exports", 'vs/editor/common/core/range'], function (require, exports, range_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function sortChanges(changes) {
        changes.sort(function (left, right) {
            if (left.originalStartLineNumber < right.originalStartLineNumber) {
                return -1;
            }
            else if (left.originalStartLineNumber > right.originalStartLineNumber) {
                return 1;
            }
            else if (left.modifiedStartLineNumber < right.modifiedStartLineNumber) {
                return -1;
            }
            return 1;
        });
    }
    function sortSelections(selections) {
        selections.sort(function (left, right) {
            if (left.getStartPosition().lineNumber < right.getStartPosition().lineNumber) {
                return -1;
            }
            return 1;
        });
    }
    function isInsertion(change) {
        return change.originalEndLineNumber <= 0;
    }
    function isDeletion(change) {
        return change.modifiedEndLineNumber <= 0;
    }
    /**
     * Returns a new IModel that has all the selected changes from modified IModel applied to the original IModel.
     */
    function stageRanges(diff) {
        var selections = diff.getSelections();
        var changes = getSelectedChanges(diff.getLineChanges(), selections);
        return applyChangesToModel(diff.getModel().original, diff.getModel().modified, changes);
    }
    exports.stageRanges = stageRanges;
    /**
     * Returns an intersection between a change and a selection.
     * Returns null if intersection does not exist.
     */
    function intersectChangeAndSelection(change, selection) {
        var result = {
            modifiedStartLineNumber: Math.max(change.modifiedStartLineNumber, selection.startLineNumber),
            modifiedEndLineNumber: Math.min(change.modifiedEndLineNumber, selection.endLineNumber),
            originalStartLineNumber: change.originalStartLineNumber,
            originalEndLineNumber: change.originalEndLineNumber
        };
        // Deletions have modifiedEndLineNumber = 0. In that case we can not use the simple check if there is an intersection.
        var isDeletionSelected = isDeletion(result) &&
            (change.modifiedStartLineNumber >= selection.startLineNumber) && (change.modifiedStartLineNumber <= selection.endLineNumber);
        if ((result.modifiedStartLineNumber <= result.modifiedEndLineNumber) || isDeletionSelected) {
            return result;
        }
        return null;
    }
    exports.intersectChangeAndSelection = intersectChangeAndSelection;
    /**
     * Returns all selected changes (there can be multiple selections due to multiple cursors).
     * If a change is partially selected, the selected part of the change will be returned.
     */
    function getSelectedChanges(changes, selections) {
        sortChanges(changes);
        sortSelections(selections);
        var result = [];
        var currentSelection = 0;
        var lastLineAdded = -1;
        for (var i = 0; i < changes.length; ++i) {
            // We have to check the previous selection. Since it can contain two changes.
            currentSelection = Math.max(0, currentSelection - 1);
            // Find all selections that are not after the current change.
            while (currentSelection < selections.length &&
                (selections[currentSelection].startLineNumber <= changes[i].modifiedEndLineNumber || isDeletion(changes[i]))) {
                var intersectedChange = intersectChangeAndSelection(changes[i], selections[currentSelection]);
                if (intersectedChange !== null) {
                    // Each change needs to be disjoint so we check if we already added this line.
                    if (lastLineAdded !== intersectedChange.modifiedStartLineNumber) {
                        result.push(intersectedChange);
                        lastLineAdded = intersectedChange.modifiedEndLineNumber;
                    }
                    else {
                        // Update change such that we do not add same line twice.
                        intersectedChange.modifiedStartLineNumber++;
                        if (intersectedChange.modifiedStartLineNumber <= intersectedChange.modifiedEndLineNumber) {
                            result.push(intersectedChange);
                            lastLineAdded = intersectedChange.modifiedEndLineNumber;
                        }
                    }
                }
                currentSelection++;
            }
        }
        return result;
    }
    exports.getSelectedChanges = getSelectedChanges;
    function appendValueFromRange(base, model, range) {
        var result = base;
        if (result !== '') {
            result += model.getEOL();
        }
        return result + model.getValueInRange(range);
    }
    exports.appendValueFromRange = appendValueFromRange;
    /**
     * Applies a list of changes to the original model and returns the new IModel.
     * First sorts changes by line number.
     */
    function applyChangesToModel(original, modified, changes) {
        sortChanges(changes);
        var result = '';
        var positionInOriginal = 1;
        for (var i = 0; i < changes.length; ++i) {
            // We have to update orginalStartLineNumber for insertions, their start line is always one line behind.
            var originalStartLineUpdated = isInsertion(changes[i]) ? changes[i].originalStartLineNumber + 1 : changes[i].originalStartLineNumber;
            if (positionInOriginal < originalStartLineUpdated) {
                result = appendValueFromRange(result, original, new range_1.Range(positionInOriginal, 1, originalStartLineUpdated - 1, original.getLineMaxColumn(originalStartLineUpdated - 1)));
                positionInOriginal = originalStartLineUpdated;
            }
            if (!isDeletion(changes[i])) {
                result = appendValueFromRange(result, modified, new range_1.Range(changes[i].modifiedStartLineNumber, 1, changes[i].modifiedEndLineNumber, modified.getLineMaxColumn(changes[i].modifiedEndLineNumber)));
            }
            // Update position in the original file where we continue to concatanate.
            // Only update position if it was not an insertion.
            if (!isInsertion(changes[i])) {
                positionInOriginal = changes[i].originalEndLineNumber + 1;
            }
        }
        // Append the last chunk after all the changes.
        if (positionInOriginal <= original.getLineCount()) {
            result = appendValueFromRange(result, original, new range_1.Range(positionInOriginal, 1, original.getLineCount(), original.getLineMaxColumn(original.getLineCount())));
        }
        return result;
    }
    exports.applyChangesToModel = applyChangesToModel;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/git/node/git.lib", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/node/extfs', 'vs/base/common/mime', 'vs/base/common/lifecycle', 'vs/base/common/objects', 'vs/base/common/uuid', 'vs/nls!vs/workbench/parts/git/node/git.lib', 'vs/base/common/strings', 'vs/workbench/parts/git/common/git', 'vs/base/node/mime', 'vs/platform/files/common/files', 'child_process', 'vs/base/node/encoding'], function (require, exports, winjs_base_1, extfs, mime_1, lifecycle_1, objects, uuid, nls, strings, git_1, mime_2, files, child_process_1, encoding_1) {
    "use strict";
    function exec(child, encoding) {
        if (encoding === void 0) { encoding = 'utf8'; }
        var disposables = [];
        var once = function (ee, name, fn) {
            ee.once(name, fn);
            disposables.push(lifecycle_1.toDisposable(function () { return ee.removeListener(name, fn); }));
        };
        var on = function (ee, name, fn) {
            ee.on(name, fn);
            disposables.push(lifecycle_1.toDisposable(function () { return ee.removeListener(name, fn); }));
        };
        return winjs_base_1.TPromise.join([
            new winjs_base_1.TPromise(function (c, e) {
                once(child, 'error', e);
                once(child, 'exit', c);
            }),
            new winjs_base_1.TPromise(function (c) {
                var buffers = [];
                on(child.stdout, 'data', function (b) { return buffers.push(b); });
                once(child.stdout, 'close', function () { return c(encoding_1.decode(Buffer.concat(buffers), encoding)); });
            }),
            new winjs_base_1.TPromise(function (c) {
                var buffers = [];
                on(child.stderr, 'data', function (b) { return buffers.push(b); });
                once(child.stderr, 'close', function () { return c(encoding_1.decode(Buffer.concat(buffers), encoding)); });
            })
        ]).then(function (values) {
            lifecycle_1.disposeAll(disposables);
            return {
                exitCode: values[0],
                stdout: values[1],
                stderr: values[2]
            };
        });
    }
    var GitError = (function () {
        function GitError(data) {
            if (data.error) {
                this.error = data.error;
                this.message = data.error.message;
            }
            else {
                this.error = null;
            }
            this.message = this.message || data.message || 'Git error';
            this.stdout = data.stdout || null;
            this.stderr = data.stderr || null;
            this.exitCode = data.exitCode || null;
            this.gitErrorCode = data.gitErrorCode || null;
            this.gitCommand = data.gitCommand || null;
        }
        GitError.prototype.toString = function () {
            var result = this.message + ' ' + JSON.stringify({
                exitCode: this.exitCode,
                gitErrorCode: this.gitErrorCode,
                gitCommand: this.gitCommand,
                stdout: this.stdout,
                stderr: this.stderr
            }, null, 2);
            if (this.error) {
                result += this.error.stack;
            }
            return result;
        };
        return GitError;
    }());
    exports.GitError = GitError;
    var Git = (function () {
        function Git(options) {
            this.gitPath = options.gitPath;
            this.version = options.version;
            this.tmpPath = options.tmpPath;
            var encoding = options.defaultEncoding || 'utf8';
            this.defaultEncoding = encoding_1.encodingExists(encoding) ? encoding : 'utf8';
            this.env = options.env || {};
            this.outputListeners = [];
        }
        Git.prototype.run = function (cwd, args, options) {
            if (options === void 0) { options = {}; }
            options = objects.assign({ cwd: cwd }, options || {});
            return this.exec(args, options);
        };
        Git.prototype.stream = function (cwd, args, options) {
            if (options === void 0) { options = {}; }
            options = objects.assign({ cwd: cwd }, options || {});
            return this.spawn(args, options);
        };
        Git.prototype.open = function (repository, env) {
            if (env === void 0) { env = {}; }
            return new Repository(this, repository, this.defaultEncoding, env);
        };
        Git.prototype.clone = function (repository, repoURL) {
            var _this = this;
            return this.exec(['clone', repoURL, repository]).then(function () { return true; }, function (err) {
                return new winjs_base_1.TPromise(function (c, e) {
                    // If there's any error, git will still leave the folder in the FS,
                    // so we need to remove it.
                    extfs.del(repository, _this.tmpPath, function (err) {
                        if (err) {
                            return e(err);
                        }
                        c(true);
                    });
                });
            });
        };
        Git.prototype.config = function (name, value) {
            return this.exec(['config', '--global', name, value]);
        };
        Git.prototype.exec = function (args, options) {
            var _this = this;
            if (options === void 0) { options = {}; }
            var child = this.spawn(args, options);
            if (options.input) {
                child.stdin.end(options.input, 'utf8');
            }
            return exec(child).then(function (result) {
                if (result.exitCode) {
                    var gitErrorCode = null;
                    if (/Authentication failed/.test(result.stderr)) {
                        gitErrorCode = git_1.GitErrorCodes.AuthenticationFailed;
                    }
                    else if (/Not a git repository/.test(result.stderr)) {
                        gitErrorCode = git_1.GitErrorCodes.NotAGitRepository;
                    }
                    else if (/bad config file/.test(result.stderr)) {
                        gitErrorCode = git_1.GitErrorCodes.BadConfigFile;
                    }
                    else if (/cannot make pipe for command substitution|cannot create standard input pipe/.test(result.stderr)) {
                        gitErrorCode = git_1.GitErrorCodes.CantCreatePipe;
                    }
                    else if (/Repository not found/.test(result.stderr)) {
                        gitErrorCode = git_1.GitErrorCodes.RepositoryNotFound;
                    }
                    else if (/unable to access/.test(result.stderr)) {
                        gitErrorCode = git_1.GitErrorCodes.CantAccessRemote;
                    }
                    if (options.log !== false) {
                        _this.log(result.stderr);
                    }
                    return winjs_base_1.TPromise.wrapError(new GitError({
                        message: 'Failed to execute git',
                        stdout: result.stdout,
                        stderr: result.stderr,
                        exitCode: result.exitCode,
                        gitErrorCode: gitErrorCode,
                        gitCommand: args[0]
                    }));
                }
                return result;
            });
        };
        Git.prototype.spawn = function (args, options) {
            if (options === void 0) { options = {}; }
            if (!this.gitPath) {
                throw new Error('git could not be found in the system.');
            }
            if (!options) {
                options = {};
            }
            if (!options.stdio && !options.input) {
                options.stdio = ['ignore', null, null]; // Unless provided, ignore stdin and leave default streams for stdout and stderr
            }
            options.env = objects.assign({}, options.env || {});
            options.env = objects.assign(options.env, this.env);
            options.env = objects.assign(options.env, {
                MONACO_REQUEST_GUID: uuid.v4().asHex(),
                VSCODE_GIT_REQUEST_ID: uuid.v4().asHex(),
                MONACO_GIT_COMMAND: args[0]
            });
            if (options.log !== false) {
                this.log(strings.format('git {0}\n', args.join(' ')));
            }
            return child_process_1.spawn(this.gitPath, args, options);
        };
        Git.prototype.onOutput = function (listener) {
            var _this = this;
            this.outputListeners.push(listener);
            return function () { return _this.outputListeners.splice(_this.outputListeners.indexOf(listener), 1); };
        };
        Git.prototype.log = function (output) {
            this.outputListeners.forEach(function (l) { return l(output); });
        };
        return Git;
    }());
    exports.Git = Git;
    var Repository = (function () {
        function Repository(git, repository, defaultEncoding, env) {
            if (env === void 0) { env = {}; }
            this.git = git;
            this.repository = repository;
            this.defaultEncoding = defaultEncoding;
            this.env = env;
        }
        Object.defineProperty(Repository.prototype, "version", {
            get: function () {
                return this.git.version;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Repository.prototype, "path", {
            get: function () {
                return this.repository;
            },
            enumerable: true,
            configurable: true
        });
        Repository.prototype.run = function (args, options) {
            if (options === void 0) { options = {}; }
            options.env = objects.assign({}, options.env || {});
            options.env = objects.assign(options.env, this.env);
            return this.git.run(this.repository, args, options);
        };
        Repository.prototype.stream = function (args, options) {
            if (options === void 0) { options = {}; }
            options.env = objects.assign({}, options.env || {});
            options.env = objects.assign(options.env, this.env);
            return this.git.stream(this.repository, args, options);
        };
        Repository.prototype.spawn = function (args, options) {
            if (options === void 0) { options = {}; }
            options.env = objects.assign({}, options.env || {});
            options.env = objects.assign(options.env, this.env);
            return this.git.spawn(args, options);
        };
        Repository.prototype.init = function () {
            return this.run(['init']);
        };
        Repository.prototype.config = function (scope, key, value, options) {
            var args = ['config'];
            if (scope) {
                args.push('--' + scope);
            }
            args.push(key);
            if (value) {
                args.push(value);
            }
            return this.run(args, options).then(function (result) { return result.stdout; });
        };
        Repository.prototype.show = function (object) {
            return this.stream(['show', object]);
        };
        Repository.prototype.buffer = function (object) {
            var _this = this;
            var child = this.show(object);
            return new winjs_base_1.Promise(function (c, e) {
                mime_2.detectMimesFromStream(child.stdout, null, function (err, result) {
                    if (err) {
                        e(err);
                    }
                    else if (mime_1.isBinaryMime(result.mimes)) {
                        e({
                            message: nls.localize(0, null),
                            fileOperationResult: files.FileOperationResult.FILE_IS_BINARY
                        });
                    }
                    else {
                        c(_this.doBuffer(object));
                    }
                });
            });
        };
        Repository.prototype.doBuffer = function (object) {
            var child = this.show(object);
            return exec(child, this.defaultEncoding).then(function (_a) {
                var exitCode = _a.exitCode, stdout = _a.stdout;
                if (exitCode) {
                    return winjs_base_1.TPromise.wrapError(new GitError({
                        message: 'Could not buffer object.',
                        exitCode: exitCode
                    }));
                }
                return winjs_base_1.TPromise.as(stdout);
            });
        };
        Repository.prototype.add = function (paths) {
            var args = ['add', '-A', '--'];
            if (paths && paths.length) {
                args.push.apply(args, paths);
            }
            else {
                args.push('.');
            }
            return this.run(args);
        };
        Repository.prototype.stage = function (path, data) {
            var _this = this;
            var child = this.stream(['hash-object', '--stdin', '-w'], { stdio: [null, null, null] });
            child.stdin.end(data, 'utf8');
            return exec(child).then(function (_a) {
                var exitCode = _a.exitCode, stdout = _a.stdout;
                if (exitCode) {
                    return winjs_base_1.TPromise.wrapError(new GitError({
                        message: 'Could not hash object.',
                        exitCode: exitCode
                    }));
                }
                return _this.run(['update-index', '--cacheinfo', '100644', stdout, path]);
            });
        };
        Repository.prototype.checkout = function (treeish, paths) {
            var args = ['checkout', '-q'];
            if (treeish) {
                args.push(treeish);
            }
            if (paths && paths.length) {
                args.push('--');
                args.push.apply(args, paths);
            }
            return this.run(args).then(null, function (err) {
                if (/Please, commit your changes or stash them/.test(err.stderr)) {
                    err.gitErrorCode = git_1.GitErrorCodes.DirtyWorkTree;
                }
                return winjs_base_1.Promise.wrapError(err);
            });
        };
        Repository.prototype.commit = function (message, all, amend) {
            var _this = this;
            var args = ['commit', '--quiet', '--allow-empty-message', '--file', '-'];
            if (all) {
                args.push('--all');
            }
            if (amend) {
                args.push('--amend');
            }
            return this.run(args, { input: message || '' }).then(null, function (commitErr) {
                if (/not possible because you have unmerged files/.test(commitErr.stderr)) {
                    commitErr.gitErrorCode = git_1.GitErrorCodes.UnmergedChanges;
                    return winjs_base_1.Promise.wrapError(commitErr);
                }
                return _this.run(['config', '--get-all', 'user.name']).then(null, function (err) {
                    err.gitErrorCode = git_1.GitErrorCodes.NoUserNameConfigured;
                    return winjs_base_1.Promise.wrapError(err);
                }).then(function () {
                    return _this.run(['config', '--get-all', 'user.email']).then(null, function (err) {
                        err.gitErrorCode = git_1.GitErrorCodes.NoUserEmailConfigured;
                        return winjs_base_1.Promise.wrapError(err);
                    }).then(function () {
                        return winjs_base_1.Promise.wrapError(commitErr);
                    });
                });
            });
        };
        Repository.prototype.branch = function (name, checkout) {
            var args = checkout ? ['checkout', '-q', '-b', name] : ['branch', '-q', name];
            return this.run(args);
        };
        Repository.prototype.clean = function (paths) {
            var args = ['clean', '-f', '-q', '--'].concat(paths);
            return this.run(args);
        };
        Repository.prototype.undo = function () {
            var _this = this;
            return this.run(['clean', '-fd']).then(function () {
                return _this.run(['checkout', '--', '.']).then(null, function (err) {
                    if (/did not match any file\(s\) known to git\./.test(err.stderr)) {
                        return winjs_base_1.TPromise.as(null);
                    }
                    return winjs_base_1.Promise.wrapError(err);
                });
            });
        };
        Repository.prototype.reset = function (treeish, hard) {
            if (hard === void 0) { hard = false; }
            var args = ['reset'];
            if (hard) {
                args.push('--hard');
            }
            args.push(treeish);
            return this.run(args);
        };
        Repository.prototype.revertFiles = function (treeish, paths) {
            var _this = this;
            return this.run(['branch']).then(function (result) {
                var args;
                // In case there are no branches, we must use rm --cached
                if (!result.stdout) {
                    args = ['rm', '--cached', '-r', '--'];
                }
                else {
                    args = ['reset', '-q', treeish, '--'];
                }
                if (paths && paths.length) {
                    args.push.apply(args, paths);
                }
                else {
                    args.push('.');
                }
                return _this.run(args).then(null, function (err) {
                    // In case there are merge conflicts to be resolved, git reset will output
                    // some "needs merge" data. We try to get around that.
                    if (/([^:]+: needs merge\n)+/m.test(err.stdout)) {
                        return winjs_base_1.TPromise.as(null);
                    }
                    return winjs_base_1.Promise.wrapError(err);
                });
            });
        };
        Repository.prototype.fetch = function () {
            return this.run(['fetch']).then(null, function (err) {
                if (/No remote repository specified\./.test(err.stderr)) {
                    err.gitErrorCode = git_1.GitErrorCodes.NoRemoteRepositorySpecified;
                }
                else if (/Could not read from remote repository/.test(err.stderr)) {
                    err.gitErrorCode = git_1.GitErrorCodes.RemoteConnectionError;
                }
                return winjs_base_1.Promise.wrapError(err);
            });
        };
        Repository.prototype.pull = function (rebase) {
            var args = ['pull'];
            if (rebase) {
                args.push('-r');
            }
            return this.run(args).then(null, function (err) {
                if (/^CONFLICT \([^)]+\): \b/m.test(err.stdout)) {
                    err.gitErrorCode = git_1.GitErrorCodes.Conflict;
                }
                else if (/Please tell me who you are\./.test(err.stderr)) {
                    err.gitErrorCode = git_1.GitErrorCodes.NoUserNameConfigured;
                }
                else if (/Could not read from remote repository/.test(err.stderr)) {
                    err.gitErrorCode = git_1.GitErrorCodes.RemoteConnectionError;
                }
                else if (/Pull is not possible because you have unmerged files|Cannot pull with rebase: You have unstaged changes|Your local changes to the following files would be overwritten|Please, commit your changes before you can merge/.test(err.stderr)) {
                    err.gitErrorCode = git_1.GitErrorCodes.DirtyWorkTree;
                }
                return winjs_base_1.Promise.wrapError(err);
            });
        };
        Repository.prototype.push = function (remote, name, options) {
            var args = ['push'];
            if (options && options.setUpstream) {
                args.push('-u');
            }
            if (remote) {
                args.push(remote);
            }
            if (name) {
                args.push(name);
            }
            return this.run(args).then(null, function (err) {
                if (/^error: failed to push some refs to\b/m.test(err.stderr)) {
                    err.gitErrorCode = git_1.GitErrorCodes.PushRejected;
                }
                else if (/Could not read from remote repository/.test(err.stderr)) {
                    err.gitErrorCode = git_1.GitErrorCodes.RemoteConnectionError;
                }
                return winjs_base_1.Promise.wrapError(err);
            });
        };
        Repository.prototype.sync = function () {
            var _this = this;
            return this.pull().then(function () { return _this.push(); });
        };
        Repository.prototype.getRoot = function () {
            return this.run(['rev-parse', '--show-toplevel'], { log: false }).then(function (result) { return result.stdout.trim(); });
        };
        Repository.prototype.getStatus = function () {
            return this.run(['status', '-z', '-u'], { log: false }).then(function (executionResult) {
                var status = executionResult.stdout;
                var result = [];
                var current;
                var i = 0;
                function readName() {
                    var start = i;
                    var c;
                    while ((c = status.charAt(i)) !== '\u0000') {
                        i++;
                    }
                    return status.substring(start, i++);
                }
                while (i < status.length) {
                    current = {
                        x: status.charAt(i++),
                        y: status.charAt(i++),
                        path: null,
                        mimetype: null
                    };
                    i++;
                    if (current.x === 'R') {
                        current.rename = readName();
                    }
                    current.path = readName();
                    current.mimetype = mime_1.guessMimeTypes(current.path)[0];
                    // If path ends with slash, it must be a nested git repo
                    if (current.path[current.path.length - 1] === '/') {
                        continue;
                    }
                    result.push(current);
                }
                return winjs_base_1.TPromise.as(result);
            });
        };
        Repository.prototype.getHEAD = function () {
            var _this = this;
            return this.run(['symbolic-ref', '--short', 'HEAD'], { log: false }).then(function (result) {
                if (!result.stdout) {
                    return winjs_base_1.TPromise.wrapError(new Error('Not in a branch'));
                }
                return winjs_base_1.TPromise.as({ name: result.stdout.trim() });
            }, function (err) {
                return _this.run(['rev-parse', 'HEAD'], { log: false }).then(function (result) {
                    if (!result.stdout) {
                        return winjs_base_1.TPromise.wrapError(new Error('Error parsing HEAD'));
                    }
                    return winjs_base_1.TPromise.as({ commit: result.stdout.trim() });
                });
            });
        };
        Repository.prototype.getHeads = function () {
            return this.run(['for-each-ref', '--format', '%(refname:short) %(objectname)', 'refs/heads/'], { log: false }).then(function (result) {
                return result.stdout.trim().split('\n')
                    .filter(function (b) { return !!b; })
                    .map(function (b) { return b.trim().split(' '); })
                    .map(function (a) { return ({ name: a[0], commit: a[1] }); });
            });
        };
        Repository.prototype.getTags = function () {
            return this.run(['for-each-ref', '--format', '%(refname:short) %(objectname)', 'refs/tags/'], { log: false }).then(function (result) {
                return result.stdout.trim().split('\n')
                    .filter(function (b) { return !!b; })
                    .map(function (b) { return b.trim().split(' '); })
                    .map(function (a) { return ({ name: a[0], commit: a[1] }); });
            });
        };
        Repository.prototype.getRemotes = function () {
            return this.run(['remote'], { log: false })
                .then(function (result) { return result.stdout
                .trim()
                .split('\n')
                .filter(function (b) { return !!b; })
                .map(function (name) { return ({ name: name }); }); });
        };
        Repository.prototype.getBranch = function (branch) {
            var _this = this;
            if (branch === 'HEAD') {
                return this.getHEAD();
            }
            return this.run(['rev-parse', branch], { log: false }).then(function (result) {
                if (!result.stdout) {
                    return winjs_base_1.TPromise.wrapError(new Error('No such branch'));
                }
                var commit = result.stdout.trim();
                return _this.run(['rev-parse', '--symbolic-full-name', '--abbrev-ref', branch + '@{u}'], { log: false }).then(function (result) {
                    var upstream = result.stdout.trim();
                    return _this.run(['rev-list', '--left-right', branch + '...' + upstream], { log: false }).then(function (result) {
                        var ahead = 0, behind = 0;
                        var i = 0;
                        while (i < result.stdout.length) {
                            switch (result.stdout.charAt(i)) {
                                case '<':
                                    ahead++;
                                    break;
                                case '>':
                                    behind++;
                                    break;
                                default:
                                    i++;
                                    break;
                            }
                            while (result.stdout.charAt(i++) !== '\n') { }
                        }
                        return {
                            name: branch,
                            commit: commit,
                            upstream: upstream,
                            ahead: ahead,
                            behind: behind
                        };
                    });
                }, function () {
                    return { name: branch, commit: commit };
                });
            });
        };
        Repository.prototype.onOutput = function (listener) {
            return this.git.onOutput(listener);
        };
        return Repository;
    }());
    exports.Repository = Repository;
});

define("vs/workbench/parts/git/node/rawGitService", ["require", "exports", 'path', 'vs/base/common/winjs.base', 'vs/base/node/mime', 'vs/base/node/pfs', 'vs/workbench/parts/git/node/git.lib', 'vs/workbench/parts/git/common/git'], function (require, exports, path, winjs_base_1, mime, pfs, git_lib_1, git_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var RawGitService = (function () {
        function RawGitService(repo) {
            this.repo = repo;
        }
        RawGitService.prototype.getVersion = function () {
            return winjs_base_1.TPromise.as(this.repo.version);
        };
        RawGitService.prototype.getRepositoryRoot = function () {
            return this._repositoryRoot || (this._repositoryRoot = pfs.realpath(this.repo.path));
        };
        RawGitService.prototype.serviceState = function () {
            return winjs_base_1.TPromise.as(this.repo
                ? git_1.RawServiceState.OK
                : git_1.RawServiceState.GitNotFound);
        };
        RawGitService.prototype.status = function () {
            var _this = this;
            return this.repo.getStatus()
                .then(function (status) { return _this.repo.getHEAD()
                .then(function (HEAD) {
                if (HEAD.name) {
                    return _this.repo.getBranch(HEAD.name).then(null, function () { return HEAD; });
                }
                else {
                    return HEAD;
                }
            }, function () { return null; })
                .then(function (HEAD) { return winjs_base_1.Promise.join([_this.getRepositoryRoot(), _this.repo.getHeads(), _this.repo.getTags(), _this.repo.getRemotes()]).then(function (r) {
                return {
                    repositoryRoot: r[0],
                    status: status,
                    HEAD: HEAD,
                    heads: r[1],
                    tags: r[2],
                    remotes: r[3]
                };
            }); }); })
                .then(null, function (err) {
                if (err.gitErrorCode === git_1.GitErrorCodes.BadConfigFile) {
                    return winjs_base_1.Promise.wrapError(err);
                }
                else if (err.gitErrorCode === git_1.GitErrorCodes.NotAtRepositoryRoot) {
                    return winjs_base_1.Promise.wrapError(err);
                }
                return null;
            });
        };
        RawGitService.prototype.init = function () {
            var _this = this;
            return this.repo.init().then(function () { return _this.status(); });
        };
        RawGitService.prototype.add = function (filePaths) {
            var _this = this;
            return this.repo.add(filePaths).then(function () { return _this.status(); });
        };
        RawGitService.prototype.stage = function (filePath, content) {
            var _this = this;
            return this.repo.stage(filePath, content).then(function () { return _this.status(); });
        };
        RawGitService.prototype.branch = function (name, checkout) {
            var _this = this;
            return this.repo.branch(name, checkout).then(function () { return _this.status(); });
        };
        RawGitService.prototype.checkout = function (treeish, filePaths) {
            var _this = this;
            return this.repo.checkout(treeish, filePaths).then(function () { return _this.status(); });
        };
        RawGitService.prototype.clean = function (filePaths) {
            var _this = this;
            return this.repo.clean(filePaths).then(function () { return _this.status(); });
        };
        RawGitService.prototype.undo = function () {
            var _this = this;
            return this.repo.undo().then(function () { return _this.status(); });
        };
        RawGitService.prototype.reset = function (treeish, hard) {
            var _this = this;
            return this.repo.reset(treeish, hard).then(function () { return _this.status(); });
        };
        RawGitService.prototype.revertFiles = function (treeish, filePaths) {
            var _this = this;
            return this.repo.revertFiles(treeish, filePaths).then(function () { return _this.status(); });
        };
        RawGitService.prototype.fetch = function () {
            var _this = this;
            return this.repo.fetch().then(null, function (err) {
                if (err.gitErrorCode === git_1.GitErrorCodes.NoRemoteRepositorySpecified) {
                    return winjs_base_1.TPromise.as(null);
                }
                return winjs_base_1.Promise.wrapError(err);
            }).then(function () { return _this.status(); });
        };
        RawGitService.prototype.pull = function (rebase) {
            var _this = this;
            return this.repo.pull(rebase).then(function () { return _this.status(); });
        };
        RawGitService.prototype.push = function (remote, name, options) {
            var _this = this;
            return this.repo.push(remote, name, options).then(function () { return _this.status(); });
        };
        RawGitService.prototype.sync = function () {
            var _this = this;
            return this.repo.sync().then(function () { return _this.status(); });
        };
        RawGitService.prototype.commit = function (message, amend, stage) {
            var _this = this;
            var promise = winjs_base_1.TPromise.as(null);
            if (stage) {
                promise = this.repo.add(null);
            }
            return promise
                .then(function () { return _this.repo.commit(message, stage, amend); })
                .then(function () { return _this.status(); });
        };
        RawGitService.prototype.detectMimetypes = function (filePath, treeish) {
            var _this = this;
            return pfs.exists(path.join(this.repo.path, filePath)).then(function (exists) {
                if (exists) {
                    return new winjs_base_1.TPromise(function (c, e) {
                        mime.detectMimesFromFile(path.join(_this.repo.path, filePath), function (err, result) {
                            if (err) {
                                e(err);
                            }
                            else {
                                c(result.mimes);
                            }
                        });
                    });
                }
                var child = _this.repo.show(treeish + ':' + filePath);
                return new winjs_base_1.TPromise(function (c, e) {
                    mime.detectMimesFromStream(child.stdout, filePath, function (err, result) {
                        if (err) {
                            e(err);
                        }
                        else {
                            c(result.mimes);
                        }
                    });
                });
            });
        };
        // careful, this buffers the whole object into memory
        RawGitService.prototype.show = function (filePath, treeish) {
            treeish = treeish === '~' ? '' : treeish;
            return this.repo.buffer(treeish + ':' + filePath).then(null, function (e) {
                if (e instanceof git_lib_1.GitError) {
                    return ''; // mostly untracked files end up in a git error
                }
                return winjs_base_1.TPromise.wrapError(e);
            });
        };
        RawGitService.prototype.onOutput = function () {
            var _this = this;
            var cancel;
            return new winjs_base_1.Promise(function (c, e, p) {
                cancel = _this.repo.onOutput(p);
            }, function () { return cancel(); });
        };
        return RawGitService;
    }());
    exports.RawGitService = RawGitService;
    var DelayedRawGitService = (function () {
        function DelayedRawGitService(raw) {
            this.raw = raw;
        }
        DelayedRawGitService.prototype.getVersion = function () {
            return this.raw.then(function (raw) { return raw.getVersion(); });
        };
        DelayedRawGitService.prototype.serviceState = function () {
            return this.raw.then(function (raw) { return raw.serviceState(); });
        };
        DelayedRawGitService.prototype.status = function () {
            return this.raw.then(function (raw) { return raw.status(); });
        };
        DelayedRawGitService.prototype.init = function () {
            return this.raw.then(function (raw) { return raw.init(); });
        };
        DelayedRawGitService.prototype.add = function (filesPaths) {
            return this.raw.then(function (raw) { return raw.add(filesPaths); });
        };
        DelayedRawGitService.prototype.stage = function (filePath, content) {
            return this.raw.then(function (raw) { return raw.stage(filePath, content); });
        };
        DelayedRawGitService.prototype.branch = function (name, checkout) {
            return this.raw.then(function (raw) { return raw.branch(name, checkout); });
        };
        DelayedRawGitService.prototype.checkout = function (treeish, filePaths) {
            return this.raw.then(function (raw) { return raw.checkout(treeish, filePaths); });
        };
        DelayedRawGitService.prototype.clean = function (filePaths) {
            return this.raw.then(function (raw) { return raw.clean(filePaths); });
        };
        DelayedRawGitService.prototype.undo = function () {
            return this.raw.then(function (raw) { return raw.undo(); });
        };
        DelayedRawGitService.prototype.reset = function (treeish, hard) {
            return this.raw.then(function (raw) { return raw.reset(treeish, hard); });
        };
        DelayedRawGitService.prototype.revertFiles = function (treeish, filePaths) {
            return this.raw.then(function (raw) { return raw.revertFiles(treeish, filePaths); });
        };
        DelayedRawGitService.prototype.fetch = function () {
            return this.raw.then(function (raw) { return raw.fetch(); });
        };
        DelayedRawGitService.prototype.pull = function (rebase) {
            return this.raw.then(function (raw) { return raw.pull(rebase); });
        };
        DelayedRawGitService.prototype.push = function (origin, name, options) {
            return this.raw.then(function (raw) { return raw.push(origin, name, options); });
        };
        DelayedRawGitService.prototype.sync = function () {
            return this.raw.then(function (raw) { return raw.sync(); });
        };
        DelayedRawGitService.prototype.commit = function (message, amend, stage) {
            return this.raw.then(function (raw) { return raw.commit(message, amend, stage); });
        };
        DelayedRawGitService.prototype.detectMimetypes = function (path, treeish) {
            return this.raw.then(function (raw) { return raw.detectMimetypes(path, treeish); });
        };
        DelayedRawGitService.prototype.show = function (path, treeish) {
            return this.raw.then(function (raw) { return raw.show(path, treeish); });
        };
        DelayedRawGitService.prototype.onOutput = function () {
            return this.raw.then(function (raw) { return raw.onOutput(); });
        };
        return DelayedRawGitService;
    }());
    exports.DelayedRawGitService = DelayedRawGitService;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/html/common/htmlInput", ["require", "exports", 'vs/workbench/common/editor/resourceEditorInput'], function (require, exports, resourceEditorInput_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var HtmlInput = (function (_super) {
        __extends(HtmlInput, _super);
        function HtmlInput() {
            _super.apply(this, arguments);
        }
        // just a marker class
        HtmlInput.prototype.getResource = function () {
            return this.resource;
        };
        return HtmlInput;
    }(resourceEditorInput_1.ResourceEditorInput));
    exports.HtmlInput = HtmlInput;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/lib/node/systemVariables", ["require", "exports", 'vs/base/common/paths', 'vs/base/common/parsers', 'vs/workbench/common/editor'], function (require, exports, Paths, parsers_1, WorkbenchEditorCommon) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var SystemVariables = (function (_super) {
        __extends(SystemVariables, _super);
        // Optional workspaceRoot there to be used in tests.
        function SystemVariables(editorService, contextService, workspaceRoot) {
            var _this = this;
            if (workspaceRoot === void 0) { workspaceRoot = null; }
            _super.call(this);
            this.editorService = editorService;
            var fsPath = workspaceRoot ? workspaceRoot.fsPath : contextService.getWorkspace().resource.fsPath;
            this._workspaceRoot = Paths.normalize(fsPath, true);
            this._execPath = contextService ? contextService.getConfiguration().env.execPath : null;
            Object.keys(process.env).forEach(function (key) {
                _this[("env." + key)] = process.env[key];
            });
        }
        Object.defineProperty(SystemVariables.prototype, "execPath", {
            get: function () {
                return this._execPath;
            },
            set: function (value) {
                this._execPath = value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemVariables.prototype, "cwd", {
            get: function () {
                return this.workspaceRoot;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemVariables.prototype, "workspaceRoot", {
            get: function () {
                return this._workspaceRoot;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemVariables.prototype, "file", {
            get: function () {
                return this.getFilePath();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemVariables.prototype, "fileBasename", {
            get: function () {
                return Paths.basename(this.getFilePath());
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemVariables.prototype, "fileDirname", {
            get: function () {
                return Paths.dirname(this.getFilePath());
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SystemVariables.prototype, "fileExtname", {
            get: function () {
                return Paths.extname(this.getFilePath());
            },
            enumerable: true,
            configurable: true
        });
        SystemVariables.prototype.getFilePath = function () {
            var input = this.editorService.getActiveEditorInput();
            if (!input) {
                return '';
            }
            var fei = WorkbenchEditorCommon.asFileEditorInput(input);
            if (!fei) {
                return '';
            }
            var resource = fei.getResource();
            return Paths.normalize(resource.fsPath, true);
        };
        return SystemVariables;
    }(parsers_1.AbstractSystemVariables));
    exports.SystemVariables = SystemVariables;
});

define("vs/workbench/parts/markdown/common/markdown", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.MARKDOWN_MIME = 'text/x-web-markdown';
    exports.MARKDOWN_MODE_ID = 'markdown';
});

define("vs/workbench/parts/output/common/output", ["require", "exports", 'vs/platform/platform', 'vs/platform/instantiation/common/instantiation'], function (require, exports, platform_1, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Mime type used by the output editor.
     */
    exports.OUTPUT_MIME = 'text/x-monaco-output';
    /**
     * Id used by the output editor.
     */
    exports.OUTPUT_MODE_ID = 'Log';
    /**
     * Output editor input id.
     */
    exports.OUTPUT_EDITOR_INPUT_ID = 'vs.output';
    /**
     * Output panel id
     */
    exports.OUTPUT_PANEL_ID = 'workbench.panel.output';
    exports.Extensions = {
        OutputChannels: 'workbench.contributions.outputChannels'
    };
    exports.OUTPUT_SERVICE_ID = 'outputService';
    exports.MAX_OUTPUT_LENGTH = 10000 /* Max. number of output lines to show in output */ * 100;
    exports.IOutputService = instantiation_1.createDecorator(exports.OUTPUT_SERVICE_ID);
    var OutputChannelRegistry = (function () {
        function OutputChannelRegistry() {
            this.channels = [];
        }
        OutputChannelRegistry.prototype.registerChannel = function (name) {
            if (this.channels.indexOf(name) === -1) {
                this.channels.push(name);
            }
        };
        OutputChannelRegistry.prototype.getChannels = function () {
            return this.channels.slice(0);
        };
        return OutputChannelRegistry;
    }());
    platform_1.Registry.add(exports.Extensions.OutputChannels, new OutputChannelRegistry());
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/git/browser/gitOutput", ["require", "exports", 'vs/workbench/parts/git/common/git', 'vs/workbench/parts/output/common/output'], function (require, exports, git_1, output_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var GitOutput = (function () {
        function GitOutput(gitService, outputService) {
            var _this = this;
            this.gitService = gitService;
            this.outputService = outputService;
            // we must make sure onOutput is the first thing the git service is asked,
            // so before any service operation, we call onOutput first
            gitService.addListener2(git_1.ServiceEvents.OPERATION_START, function () { return _this.setup(); });
        }
        GitOutput.prototype.getId = function () {
            return GitOutput.ID;
        };
        GitOutput.prototype.setup = function () {
            var _this = this;
            if (this.promise) {
                return;
            }
            this.promise = this.gitService.onOutput().then(function () {
                _this.promise = null;
            }, function (e) {
                if (e && e.name === 'Canceled') {
                    _this.promise = null;
                }
                else {
                    console.error(e);
                }
            }, function (o) { return _this.onOutput(o); });
        };
        GitOutput.prototype.onOutput = function (output) {
            this.outputService.append('Git', output);
        };
        GitOutput.prototype.dispose = function () {
            if (this.promise) {
                this.promise.cancel();
                this.promise = null;
            }
        };
        GitOutput.ID = 'Monaco.IDE.UI.Viewlets.GitViewlet.Workbench.GitOutput';
        GitOutput = __decorate([
            __param(0, git_1.IGitService),
            __param(1, output_1.IOutputService)
        ], GitOutput);
        return GitOutput;
    }());
    exports.GitOutput = GitOutput;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/search/common/search", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/errors', 'vs/editor/common/editorCommonExtensions'], function (require, exports, winjs_base_1, errors_1, editorCommonExtensions_1) {
    'use strict';
    var NavigateTypesSupportRegistry;
    (function (NavigateTypesSupportRegistry) {
        var _supports = [];
        function register(support) {
            if (support) {
                _supports.push(support);
            }
            return {
                dispose: function () {
                    if (support) {
                        var idx = _supports.indexOf(support);
                        if (idx >= 0) {
                            _supports.splice(idx, 1);
                            support = undefined;
                        }
                    }
                }
            };
        }
        NavigateTypesSupportRegistry.register = register;
        function all() {
            return _supports.slice(0);
        }
        NavigateTypesSupportRegistry.all = all;
    })(NavigateTypesSupportRegistry = exports.NavigateTypesSupportRegistry || (exports.NavigateTypesSupportRegistry = {}));
    function getNavigateToItems(query) {
        var promises = NavigateTypesSupportRegistry.all().map(function (support) {
            return support.getNavigateToItems(query).then(function (value) { return value; }, errors_1.onUnexpectedError);
        });
        return winjs_base_1.TPromise.join(promises).then(function (all) {
            var result = [];
            for (var _i = 0, all_1 = all; _i < all_1.length; _i++) {
                var bearings = all_1[_i];
                if (Array.isArray(bearings)) {
                    result.push.apply(result, bearings);
                }
            }
            return result;
        });
    }
    exports.getNavigateToItems = getNavigateToItems;
    editorCommonExtensions_1.CommonEditorRegistry.registerLanguageCommand('_executeWorkspaceSymbolProvider', function (accessor, args) {
        var query = args.query;
        if (typeof query !== 'string') {
            throw errors_1.illegalArgument();
        }
        return getNavigateToItems(query);
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/snippets/electron-browser/snippetsTracker", ["require", "exports", 'vs/base/common/paths', 'vs/base/common/async', 'vs/base/common/winjs.base', 'vs/base/node/extfs', 'vs/base/common/lifecycle', 'vs/editor/node/textMate/TMSnippets', 'vs/platform/files/common/files', 'vs/platform/lifecycle/common/lifecycle', 'vs/platform/workspace/common/workspace', 'fs'], function (require, exports, paths, async, winjs, extfs, lifecycle, tmsnippets, files_1, lifecycle_1, workspace_1, fs) {
    'use strict';
    var SnippetsTracker = (function () {
        function SnippetsTracker(fileService, lifecycleService, contextService) {
            var _this = this;
            this.fileService = fileService;
            this.lifecycleService = lifecycleService;
            this.snippetFolder = paths.join(contextService.getConfiguration().env.appSettingsHome, 'snippets');
            this.toDispose = [];
            this.fileWatchDelayer = new async.ThrottledDelayer(SnippetsTracker.FILE_WATCH_DELAY);
            if (!fs.existsSync(this.snippetFolder)) {
                fs.mkdirSync(this.snippetFolder);
            }
            this.scanUserSnippets().then(function (_) {
                _this.registerListeners();
            });
        }
        SnippetsTracker.prototype.registerListeners = function () {
            var _this = this;
            var scheduler = new async.RunOnceScheduler(function () {
                _this.scanUserSnippets();
            }, 500);
            this.toDispose.push(scheduler);
            try {
                this.watcher = fs.watch(this.snippetFolder); // will be persistent but not recursive
                this.watcher.on('change', function (eventType) {
                    if (eventType === 'delete') {
                        _this.unregisterListener();
                        return;
                    }
                    scheduler.schedule();
                });
            }
            catch (error) {
            }
            this.lifecycleService.onShutdown(this.dispose, this);
        };
        SnippetsTracker.prototype.scanUserSnippets = function () {
            var _this = this;
            return readFilesInDir(this.snippetFolder, /\.json$/).then(function (snippetFiles) {
                return winjs.TPromise.join(snippetFiles.map(function (snippetFile) {
                    var modeId = snippetFile.replace(/\.json$/, '').toLowerCase();
                    var snippetPath = paths.join(_this.snippetFolder, snippetFile);
                    return tmsnippets.snippetUpdated(modeId, snippetPath);
                }));
            });
        };
        SnippetsTracker.prototype.unregisterListener = function () {
            if (this.watcher) {
                this.watcher.close();
                this.watcher = null;
            }
        };
        SnippetsTracker.prototype.getId = function () {
            return 'vs.snippets.snippetsTracker';
        };
        SnippetsTracker.prototype.dispose = function () {
            this.unregisterListener();
            this.toDispose = lifecycle.disposeAll(this.toDispose);
        };
        SnippetsTracker.FILE_WATCH_DELAY = 200;
        SnippetsTracker = __decorate([
            __param(0, files_1.IFileService),
            __param(1, lifecycle_1.ILifecycleService),
            __param(2, workspace_1.IWorkspaceContextService)
        ], SnippetsTracker);
        return SnippetsTracker;
    }());
    exports.SnippetsTracker = SnippetsTracker;
    function readDir(path) {
        return new winjs.TPromise(function (c, e, p) {
            extfs.readdir(path, function (err, files) {
                if (err) {
                    return e(err);
                }
                c(files);
            });
        });
    }
    function fileExists(path) {
        return new winjs.TPromise(function (c, e, p) {
            fs.stat(path, function (err, stats) {
                if (err) {
                    return c(false);
                }
                if (stats.isFile()) {
                    return c(true);
                }
                c(false);
            });
        });
    }
    function readFilesInDir(dirPath, namePattern) {
        if (namePattern === void 0) { namePattern = null; }
        return readDir(dirPath).then(function (children) {
            return winjs.TPromise.join(children.map(function (child) {
                if (namePattern && !namePattern.test(child)) {
                    return winjs.TPromise.as(null);
                }
                return fileExists(paths.join(dirPath, child)).then(function (isFile) {
                    return isFile ? child : null;
                });
            })).then(function (subdirs) {
                return subdirs.filter(function (subdir) { return (subdir !== null); });
            });
        });
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/tasks/common/problemCollectors", ["require", "exports", 'vs/base/common/eventEmitter', 'vs/platform/markers/common/problemMatcher'], function (require, exports, eventEmitter_1, problemMatcher_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ProblemCollectorEvents;
    (function (ProblemCollectorEvents) {
        ProblemCollectorEvents.WatchingBeginDetected = 'watchingBeginDetected';
        ProblemCollectorEvents.WatchingEndDetected = 'watchingEndDetected';
    })(ProblemCollectorEvents = exports.ProblemCollectorEvents || (exports.ProblemCollectorEvents = {}));
    var AbstractProblemCollector = (function (_super) {
        __extends(AbstractProblemCollector, _super);
        function AbstractProblemCollector(problemMatchers, modelService) {
            var _this = this;
            _super.call(this);
            this.modelService = modelService;
            this.matchers = Object.create(null);
            this.bufferLength = 1;
            problemMatchers.map(function (elem) { return problemMatcher_1.createLineMatcher(elem); }).forEach(function (matcher) {
                var length = matcher.matchLength;
                if (length > _this.bufferLength) {
                    _this.bufferLength = length;
                }
                var value = _this.matchers[length];
                if (!value) {
                    value = [];
                    _this.matchers[length] = value;
                }
                value.push(matcher);
            });
            this.buffer = [];
            this.activeMatcher = null;
            this.openModels = Object.create(null);
            this.modelListeners = [];
            this.modelService.onModelAdded(function (model) {
                _this.openModels[model.getAssociatedResource().toString()] = true;
            }, this, this.modelListeners);
            this.modelService.onModelRemoved(function (model) {
                delete _this.openModels[model.getAssociatedResource().toString()];
            }, this, this.modelListeners);
            this.modelService.getModels().forEach(function (model) { return _this.openModels[model.getAssociatedResource().toString()] = true; });
        }
        AbstractProblemCollector.prototype.dispose = function () {
            this.modelListeners.forEach(function (disposable) { return disposable.dispose(); });
        };
        Object.defineProperty(AbstractProblemCollector.prototype, "numberOfMatches", {
            get: function () {
                return this._numberOfMatches;
            },
            enumerable: true,
            configurable: true
        });
        AbstractProblemCollector.prototype.tryFindMarker = function (line) {
            var result = null;
            if (this.activeMatcher) {
                result = this.activeMatcher.next(line);
                if (result) {
                    this._numberOfMatches++;
                    return result;
                }
                this.clearBuffer();
                this.activeMatcher = null;
            }
            if (this.buffer.length < this.bufferLength) {
                this.buffer.push(line);
            }
            else {
                var end = this.buffer.length - 1;
                for (var i = 0; i < end; i++) {
                    this.buffer[i] = this.buffer[i + 1];
                }
                this.buffer[end] = line;
            }
            result = this.tryMatchers();
            if (result) {
                this.clearBuffer();
            }
            return result;
        };
        AbstractProblemCollector.prototype.isOpen = function (resource) {
            return !!this.openModels[resource.toString()];
        };
        AbstractProblemCollector.prototype.shouldApplyMatch = function (result) {
            switch (result.description.applyTo) {
                case problemMatcher_1.ApplyToKind.allDocuments:
                    return true;
                case problemMatcher_1.ApplyToKind.openDocuments:
                    return this.openModels[result.resource.toString()];
                case problemMatcher_1.ApplyToKind.closedDocuments:
                    return !this.openModels[result.resource.toString()];
                default:
                    return true;
            }
        };
        AbstractProblemCollector.prototype.tryMatchers = function () {
            this.activeMatcher = null;
            var length = this.buffer.length;
            for (var startIndex = 0; startIndex < length; startIndex++) {
                var candidates = this.matchers[length - startIndex];
                if (!candidates) {
                    continue;
                }
                for (var i = 0; i < candidates.length; i++) {
                    var matcher = candidates[i];
                    var result = matcher.handle(this.buffer, startIndex);
                    if (result.match) {
                        this._numberOfMatches++;
                        if (result.continue) {
                            this.activeMatcher = matcher;
                        }
                        return result.match;
                    }
                }
            }
            return null;
        };
        AbstractProblemCollector.prototype.clearBuffer = function () {
            if (this.buffer.length > 0) {
                this.buffer = [];
            }
        };
        return AbstractProblemCollector;
    }(eventEmitter_1.EventEmitter));
    exports.AbstractProblemCollector = AbstractProblemCollector;
    (function (ProblemHandlingStrategy) {
        ProblemHandlingStrategy[ProblemHandlingStrategy["Clean"] = 0] = "Clean";
    })(exports.ProblemHandlingStrategy || (exports.ProblemHandlingStrategy = {}));
    var ProblemHandlingStrategy = exports.ProblemHandlingStrategy;
    var StartStopProblemCollector = (function (_super) {
        __extends(StartStopProblemCollector, _super);
        function StartStopProblemCollector(problemMatchers, markerService, modelService, strategy) {
            var _this = this;
            if (strategy === void 0) { strategy = ProblemHandlingStrategy.Clean; }
            _super.call(this, problemMatchers, modelService);
            // Current State
            this.currentResource = null;
            this.currentResourceAsString = null;
            this.markers = Object.create(null);
            var ownerSet = Object.create(null);
            problemMatchers.forEach(function (description) { return ownerSet[description.owner] = true; });
            this.owners = Object.keys(ownerSet);
            this.markerService = markerService;
            this.strategy = strategy;
            this.currentResourcesWithMarkers = Object.create(null);
            this.reportedResourcesWithMarkers = Object.create(null);
            this.owners.forEach(function (owner) {
                _this.currentResourcesWithMarkers[owner] = _this.markerService.read({ owner: owner }).map(function (m) { return m.resource; });
                _this.reportedResourcesWithMarkers[owner] = Object.create(null);
            });
            this.currentResource = null;
            this.currentResourceAsString = null;
            this.markers = Object.create(null);
        }
        StartStopProblemCollector.prototype.processLine = function (line) {
            var _this = this;
            var markerMatch = this.tryFindMarker(line);
            if (!markerMatch) {
                return;
            }
            var owner = markerMatch.description.owner;
            var resource = markerMatch.resource;
            var resourceAsString = resource.toString();
            var shouldApplyMatch = this.shouldApplyMatch(markerMatch);
            if (shouldApplyMatch) {
                if (this.currentResourceAsString !== resourceAsString) {
                    if (this.currentResource) {
                        Object.keys(this.markers).forEach(function (owner) {
                            _this.markerService.changeOne(owner, _this.currentResource, _this.markers[owner]);
                        });
                        this.markers = Object.create(null);
                    }
                    this.reportedResourcesWithMarkers[owner][resourceAsString] = resource;
                    this.currentResource = resource;
                    this.currentResourceAsString = resourceAsString;
                }
                var markerData = this.markers[owner];
                if (!markerData) {
                    markerData = [];
                    this.markers[owner] = markerData;
                }
                markerData.push(markerMatch.marker);
            }
            else {
                this.reportedResourcesWithMarkers[owner][resourceAsString] = resource;
            }
        };
        StartStopProblemCollector.prototype.done = function () {
            var _this = this;
            if (this.currentResource) {
                Object.keys(this.markers).forEach(function (owner) {
                    _this.markerService.changeOne(owner, _this.currentResource, _this.markers[owner]);
                });
            }
            if (this.strategy === ProblemHandlingStrategy.Clean) {
                Object.keys(this.currentResourcesWithMarkers).forEach(function (owner) {
                    var toRemove = [];
                    var withMarkers = _this.reportedResourcesWithMarkers[owner];
                    _this.currentResourcesWithMarkers[owner].forEach(function (resource) {
                        if (!withMarkers[resource.toString()]) {
                            toRemove.push(resource);
                        }
                    });
                    _this.markerService.remove(owner, toRemove);
                });
            }
            this.currentResource = null;
            this.currentResourceAsString = null;
            this.markers = Object.create(null);
        };
        return StartStopProblemCollector;
    }(AbstractProblemCollector));
    exports.StartStopProblemCollector = StartStopProblemCollector;
    var WatchingProblemCollector = (function (_super) {
        __extends(WatchingProblemCollector, _super);
        function WatchingProblemCollector(problemMatchers, markerService, modelService) {
            var _this = this;
            _super.call(this, problemMatchers, modelService);
            this.problemMatchers = problemMatchers;
            this.markerService = markerService;
            this.resetCurrentResource();
            this.resourcesToClean = Object.create(null);
            this.ignoreOpenResourcesByOwner = Object.create(null);
            this.watchingBeginsPatterns = [];
            this.watchingEndsPatterns = [];
            this.problemMatchers.forEach(function (matcher) {
                if (matcher.watching) {
                    _this.watchingBeginsPatterns.push({ problemMatcher: matcher, pattern: matcher.watching.beginsPattern });
                    _this.watchingEndsPatterns.push({ problemMatcher: matcher, pattern: matcher.watching.endsPattern });
                }
            });
        }
        WatchingProblemCollector.prototype.aboutToStart = function () {
            var _this = this;
            this.problemMatchers.forEach(function (matcher) {
                if (matcher.watching && matcher.watching.activeOnStart) {
                    _this.emit(ProblemCollectorEvents.WatchingBeginDetected, {});
                    _this.recordResourcesToClean(matcher.owner);
                }
                var value = _this.ignoreOpenResourcesByOwner[matcher.owner];
                if (!value) {
                    _this.ignoreOpenResourcesByOwner[matcher.owner] = (matcher.applyTo === problemMatcher_1.ApplyToKind.closedDocuments);
                }
                else {
                    var newValue = value && (matcher.applyTo === problemMatcher_1.ApplyToKind.closedDocuments);
                    if (newValue !== value) {
                        _this.ignoreOpenResourcesByOwner[matcher.owner] = newValue;
                    }
                }
            });
        };
        WatchingProblemCollector.prototype.processLine = function (line) {
            if (this.tryBegin(line) || this.tryFinish(line)) {
                return;
            }
            var markerMatch = this.tryFindMarker(line);
            if (!markerMatch) {
                return;
            }
            var resource = markerMatch.resource;
            var owner = markerMatch.description.owner;
            var resourceAsString = resource.toString();
            var shouldApplyMatch = this.shouldApplyMatch(markerMatch);
            if (shouldApplyMatch) {
                if (this.currentResourceAsString !== resourceAsString) {
                    this.removeResourceToClean(owner, resourceAsString);
                    if (this.currentResource) {
                        this.deliverMarkersForCurrentResource();
                    }
                    this.currentResource = resource;
                    this.currentResourceAsString = resourceAsString;
                }
                var markerData = this.markers[owner];
                if (!markerData) {
                    markerData = [];
                    this.markers[owner] = markerData;
                }
                markerData.push(markerMatch.marker);
            }
            else {
                this.removeResourceToClean(owner, resourceAsString);
            }
        };
        WatchingProblemCollector.prototype.forceDelivery = function () {
            var _this = this;
            this.deliverMarkersForCurrentResource(false);
            Object.keys(this.resourcesToClean).forEach(function (owner) {
                _this.cleanMarkers(owner, false);
            });
            this.resourcesToClean = Object.create(null);
        };
        WatchingProblemCollector.prototype.tryBegin = function (line) {
            var result = false;
            for (var i = 0; i < this.watchingBeginsPatterns.length; i++) {
                var beginMatcher = this.watchingBeginsPatterns[i];
                var matches = beginMatcher.pattern.regexp.exec(line);
                if (matches) {
                    this.emit(ProblemCollectorEvents.WatchingBeginDetected, {});
                    result = true;
                    var owner = beginMatcher.problemMatcher.owner;
                    if (matches[1]) {
                        var resource = problemMatcher_1.getResource(matches[1], beginMatcher.problemMatcher);
                        if (this.currentResourceAsString && this.currentResourceAsString === resource.toString()) {
                            this.resetCurrentResource();
                        }
                        this.recordResourceToClean(owner, resource);
                    }
                    else {
                        this.recordResourcesToClean(owner);
                        this.resetCurrentResource();
                    }
                }
            }
            return result;
        };
        WatchingProblemCollector.prototype.tryFinish = function (line) {
            var result = false;
            for (var i = 0; i < this.watchingEndsPatterns.length; i++) {
                var endMatcher = this.watchingEndsPatterns[i];
                var matches = endMatcher.pattern.regexp.exec(line);
                if (matches) {
                    this.emit(ProblemCollectorEvents.WatchingEndDetected, {});
                    result = true;
                    var owner = endMatcher.problemMatcher.owner;
                    this.cleanMarkers(owner);
                    this.deliverMarkersForCurrentResource();
                }
            }
            return result;
        };
        WatchingProblemCollector.prototype.recordResourcesToClean = function (owner) {
            var resourceSetToClean = this.getResourceSetToClean(owner);
            this.markerService.read({ owner: owner }).forEach(function (marker) { return resourceSetToClean[marker.resource.toString()] = marker.resource; });
        };
        WatchingProblemCollector.prototype.recordResourceToClean = function (owner, resource) {
            this.getResourceSetToClean(owner)[resource.toString()] = resource;
        };
        WatchingProblemCollector.prototype.removeResourceToClean = function (owner, resource) {
            var resourceSet = this.resourcesToClean[owner];
            if (resourceSet) {
                delete resourceSet[resource];
            }
        };
        WatchingProblemCollector.prototype.cleanMarkers = function (owner, remove) {
            var _this = this;
            if (remove === void 0) { remove = true; }
            var resourceSet = this.resourcesToClean[owner];
            if (resourceSet) {
                var toClean = Object.keys(resourceSet).map(function (key) { return resourceSet[key]; }).filter(function (resource) {
                    // Check whether we need to ignore open documents for this owner.
                    return _this.ignoreOpenResourcesByOwner[owner] ? !_this.isOpen(resource) : true;
                });
                this.markerService.remove(owner, toClean);
                if (remove) {
                    delete this.resourcesToClean[owner];
                }
            }
        };
        WatchingProblemCollector.prototype.deliverMarkersForCurrentResource = function (resetCurrentResource) {
            var _this = this;
            if (resetCurrentResource === void 0) { resetCurrentResource = true; }
            if (this.currentResource) {
                Object.keys(this.markers).forEach(function (owner) {
                    _this.markerService.changeOne(owner, _this.currentResource, _this.markers[owner]);
                });
            }
            if (resetCurrentResource) {
                this.resetCurrentResource();
            }
        };
        WatchingProblemCollector.prototype.getResourceSetToClean = function (owner) {
            var result = this.resourcesToClean[owner];
            if (!result) {
                result = Object.create(null);
                this.resourcesToClean[owner] = result;
            }
            return result;
        };
        WatchingProblemCollector.prototype.resetCurrentResource = function () {
            this.currentResource = null;
            this.currentResourceAsString = null;
            this.markers = Object.create(null);
        };
        return WatchingProblemCollector;
    }(AbstractProblemCollector));
    exports.WatchingProblemCollector = WatchingProblemCollector;
});

define("vs/workbench/parts/tasks/common/taskSystem", ["require", "exports"], function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (TaskErrors) {
        TaskErrors[TaskErrors["NotConfigured"] = 0] = "NotConfigured";
        TaskErrors[TaskErrors["RunningTask"] = 1] = "RunningTask";
        TaskErrors[TaskErrors["NoBuildTask"] = 2] = "NoBuildTask";
        TaskErrors[TaskErrors["NoTestTask"] = 3] = "NoTestTask";
        TaskErrors[TaskErrors["ConfigValidationError"] = 4] = "ConfigValidationError";
        TaskErrors[TaskErrors["TaskNotFound"] = 5] = "TaskNotFound";
        TaskErrors[TaskErrors["NoValidTaskRunner"] = 6] = "NoValidTaskRunner";
        TaskErrors[TaskErrors["UnknownError"] = 7] = "UnknownError";
    })(exports.TaskErrors || (exports.TaskErrors = {}));
    var TaskErrors = exports.TaskErrors;
    var TaskError = (function () {
        function TaskError(severity, message, code) {
            this.severity = severity;
            this.message = message;
            this.code = code;
        }
        return TaskError;
    }());
    exports.TaskError = TaskError;
    var Triggers;
    (function (Triggers) {
        Triggers.shortcut = 'shortcut';
        Triggers.command = 'command';
    })(Triggers = exports.Triggers || (exports.Triggers = {}));
    (function (ShowOutput) {
        ShowOutput[ShowOutput["Always"] = 0] = "Always";
        ShowOutput[ShowOutput["Silent"] = 1] = "Silent";
        ShowOutput[ShowOutput["Never"] = 2] = "Never";
    })(exports.ShowOutput || (exports.ShowOutput = {}));
    var ShowOutput = exports.ShowOutput;
    var ShowOutput;
    (function (ShowOutput) {
        function fromString(value) {
            value = value.toLowerCase();
            if (value === 'always') {
                return ShowOutput.Always;
            }
            else if (value === 'silent') {
                return ShowOutput.Silent;
            }
            else if (value === 'never') {
                return ShowOutput.Never;
            }
            else {
                return undefined;
            }
        }
        ShowOutput.fromString = fromString;
    })(ShowOutput = exports.ShowOutput || (exports.ShowOutput = {}));
    var TaskSystemEvents;
    (function (TaskSystemEvents) {
        TaskSystemEvents.Active = 'active';
        TaskSystemEvents.Inactive = 'inactive';
    })(TaskSystemEvents = exports.TaskSystemEvents || (exports.TaskSystemEvents = {}));
    (function (TaskType) {
        TaskType[TaskType["SingleRun"] = 0] = "SingleRun";
        TaskType[TaskType["Watching"] = 1] = "Watching";
    })(exports.TaskType || (exports.TaskType = {}));
    var TaskType = exports.TaskType;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/tasks/common/languageServiceTaskSystem", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/eventEmitter', 'vs/workbench/parts/tasks/common/taskSystem'], function (require, exports, winjs_base_1, eventEmitter_1, taskSystem_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LanguageServiceTaskSystem = (function (_super) {
        __extends(LanguageServiceTaskSystem, _super);
        function LanguageServiceTaskSystem(configuration, telemetryService, modeService) {
            _super.call(this);
            this.configuration = configuration;
            this.telemetryService = telemetryService;
            this.modeService = modeService;
        }
        LanguageServiceTaskSystem.prototype.build = function () {
            return this.processMode(function (mode) {
                return mode.taskSupport && mode.taskSupport.build
                    ? mode.taskSupport.build()
                    : null;
            }, 'build', taskSystem_1.Triggers.shortcut);
        };
        LanguageServiceTaskSystem.prototype.rebuild = function () {
            return this.processMode(function (mode) {
                return mode.taskSupport && mode.taskSupport.rebuild
                    ? mode.taskSupport.rebuild()
                    : null;
            }, 'rebuild', taskSystem_1.Triggers.shortcut);
        };
        LanguageServiceTaskSystem.prototype.clean = function () {
            return this.processMode(function (mode) {
                return mode.taskSupport && mode.taskSupport.clean
                    ? mode.taskSupport.clean()
                    : null;
            }, 'clean', taskSystem_1.Triggers.shortcut);
        };
        LanguageServiceTaskSystem.prototype.runTest = function () {
            return { promise: winjs_base_1.TPromise.wrapError('Not implemented yet.') };
        };
        LanguageServiceTaskSystem.prototype.run = function (taskIdentifier) {
            return { promise: winjs_base_1.TPromise.wrapError('Not implemented yet.') };
        };
        LanguageServiceTaskSystem.prototype.isActive = function () {
            return winjs_base_1.TPromise.as(false);
        };
        LanguageServiceTaskSystem.prototype.isActiveSync = function () {
            return false;
        };
        LanguageServiceTaskSystem.prototype.canAutoTerminate = function () {
            return false;
        };
        LanguageServiceTaskSystem.prototype.terminate = function () {
            return winjs_base_1.TPromise.as({ success: true });
        };
        LanguageServiceTaskSystem.prototype.terminateSync = function () {
            return { success: true };
        };
        LanguageServiceTaskSystem.prototype.tasks = function () {
            var result = [];
            return winjs_base_1.TPromise.as(result);
        };
        LanguageServiceTaskSystem.prototype.processMode = function (fn, taskName, trigger) {
            var _this = this;
            var telemetryEvent = {
                trigger: trigger,
                command: 'languageService',
                success: true
            };
            return { promise: winjs_base_1.Promise.join(this.configuration.modes.map(function (mode) {
                    return _this.modeService.getOrCreateMode(mode);
                })).then(function (modes) {
                    var promises = [];
                    modes.forEach(function (mode) {
                        var promise = fn(mode);
                        if (promise) {
                            promises.push(promise);
                        }
                    });
                    return winjs_base_1.Promise.join(promises);
                }).then(function (value) {
                    _this.telemetryService.publicLog(LanguageServiceTaskSystem.TelemetryEventName, telemetryEvent);
                    return value;
                }, function (err) {
                    telemetryEvent.success = false;
                    _this.telemetryService.publicLog(LanguageServiceTaskSystem.TelemetryEventName, telemetryEvent);
                    return winjs_base_1.Promise.wrapError(err);
                }) };
        };
        LanguageServiceTaskSystem.TelemetryEventName = 'taskService';
        return LanguageServiceTaskSystem;
    }(eventEmitter_1.EventEmitter));
    exports.LanguageServiceTaskSystem = LanguageServiceTaskSystem;
});

define("vs/workbench/parts/tasks/common/taskService", ["require", "exports", 'vs/platform/instantiation/common/instantiation', 'vs/workbench/parts/tasks/common/taskSystem'], function (require, exports, instantiation_1, taskSystem_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.TaskType = taskSystem_1.TaskType;
    exports.ITaskService = instantiation_1.createDecorator('taskService');
    var TaskServiceEvents;
    (function (TaskServiceEvents) {
        TaskServiceEvents.Active = 'active';
        TaskServiceEvents.Inactive = 'inactive';
        TaskServiceEvents.ConfigChanged = 'configChanged';
        TaskServiceEvents.Terminated = 'terminated';
    })(TaskServiceEvents = exports.TaskServiceEvents || (exports.TaskServiceEvents = {}));
});

define("vs/workbench/parts/tasks/common/taskTemplates", ["require", "exports", 'vs/nls!vs/workbench/parts/tasks/common/taskTemplates'], function (require, exports, nls) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var gulp = {
        id: 'gulp',
        label: 'Gulp',
        autoDetect: true,
        content: [
            '{',
            '\t// See http://go.microsoft.com/fwlink/?LinkId=733558',
            '\t// for the documentation about the tasks.json format',
            '\t"version": "0.1.0",',
            '\t"command": "gulp",',
            '\t"isShellCommand": true,',
            '\t"args": ["--no-color"],',
            '\t"showOutput": "always"',
            '}'
        ].join('\n')
    };
    var grunt = {
        id: 'grunt',
        label: 'Grunt',
        autoDetect: true,
        content: [
            '{',
            '\t// See http://go.microsoft.com/fwlink/?LinkId=733558',
            '\t// for the documentation about the tasks.json format',
            '\t"version": "0.1.0",',
            '\t"command": "grunt",',
            '\t"isShellCommand": true,',
            '\t"args": ["--no-color"],',
            '\t"showOutput": "always"',
            '}'
        ].join('\n')
    };
    var npm = {
        id: 'npm',
        label: 'npm',
        sort: 'NPM',
        autoDetect: false,
        content: [
            '{',
            '\t// See http://go.microsoft.com/fwlink/?LinkId=733558',
            '\t// for the documentation about the tasks.json format',
            '\t"version": "0.1.0",',
            '\t"command": "npm",',
            '\t"isShellCommand": true,',
            '\t"showOutput": "always",',
            '\t"suppressTaskName": true,',
            '\t"tasks": [',
            '\t\t{',
            '\t\t\t"taskName": "install",',
            '\t\t\t"args": ["install"]',
            '\t\t},',
            '\t\t{',
            '\t\t\t"taskName": "update",',
            '\t\t\t"args": ["update"]',
            '\t\t},',
            '\t\t{',
            '\t\t\t"taskName": "test",',
            '\t\t\t"args": ["run", "test"]',
            '\t\t}',
            '\t]',
            '}'
        ].join('\n')
    };
    var tscConfig = {
        id: 'tsc.config',
        label: 'TypeScript - tsconfig.json',
        autoDetect: false,
        description: nls.localize(0, null),
        content: [
            '{',
            '\t// See http://go.microsoft.com/fwlink/?LinkId=733558',
            '\t// for the documentation about the tasks.json format',
            '\t"version": "0.1.0",',
            '\t"command": "tsc",',
            '\t"isShellCommand": true,',
            '\t"args": ["-p", "."],',
            '\t"showOutput": "silent",',
            '\t"problemMatcher": "$tsc"',
            '}'
        ].join('\n')
    };
    var tscWatch = {
        id: 'tsc.watch',
        label: 'TypeScript - Watch Mode',
        autoDetect: false,
        description: nls.localize(1, null),
        content: [
            '{',
            '\t// See http://go.microsoft.com/fwlink/?LinkId=733558',
            '\t// for the documentation about the tasks.json format',
            '\t"version": "0.1.0",',
            '\t"command": "tsc",',
            '\t"isShellCommand": true,',
            '\t"args": ["-w", "-p", "."],',
            '\t"showOutput": "silent",',
            '\t"isWatching": true,',
            '\t"problemMatcher": "$tsc-watch"',
            '}'
        ].join('\n')
    };
    var dotnetBuild = {
        id: 'dotnetCore',
        label: '.NET Core',
        sort: 'NET Core',
        autoDetect: false,
        description: nls.localize(2, null),
        content: [
            '{',
            '\t// See http://go.microsoft.com/fwlink/?LinkId=733558',
            '\t// for the documentation about the tasks.json format',
            '\t"version": "0.1.0",',
            '\t"command": "dotnet",',
            '\t"isShellCommand": true,',
            '\t"args": [],',
            '\t"tasks": [',
            '\t\t{',
            '\t\t\t"taskName": "build",',
            '\t\t\t"args": [ ],',
            '\t\t\t"isBuildCommand": true,',
            '\t\t\t"showOutput": "silent",',
            '\t\t\t"problemMatcher": "$msCompile"',
            '\t\t}',
            '\t]',
            '}'
        ].join('\n')
    };
    var msbuild = {
        id: 'msbuild',
        label: 'MSBuild',
        autoDetect: false,
        description: nls.localize(3, null),
        content: [
            '{',
            '\t// See http://go.microsoft.com/fwlink/?LinkId=733558',
            '\t// for the documentation about the tasks.json format',
            '\t"version": "0.1.0",',
            '\t"command": "msbuild",',
            '\t"args": [',
            '\t\t// Ask msbuild to generate full paths for file names.',
            '\t\t"/property:GenerateFullPaths=true"',
            '\t],',
            '\t"taskSelector": "/t:",',
            '\t"showOutput": "silent",',
            '\t"tasks": [',
            '\t\t{',
            '\t\t\t"taskName": "build",',
            '\t\t\t// Show the output window only if unrecognized errors occur.',
            '\t\t\t"showOutput": "silent",',
            '\t\t\t// Use the standard MS compiler pattern to detect errors, warnings and infos',
            '\t\t\t"problemMatcher": "$msCompile"',
            '\t\t}',
            '\t]',
            '}'
        ].join('\n')
    };
    var command = {
        id: 'externalCommand',
        label: 'Others',
        autoDetect: false,
        description: nls.localize(4, null),
        content: [
            '{',
            '\t// See http://go.microsoft.com/fwlink/?LinkId=733558',
            '\t// for the documentation about the tasks.json format',
            '\t"version": "0.1.0",',
            '\t"command": "echo",',
            '\t"isShellCommand": true,',
            '\t"args": ["Hello World"],',
            '\t"showOutput": "always"',
            '}'
        ].join('\n')
    };
    exports.templates = [gulp, grunt, tscConfig, tscWatch, dotnetBuild, msbuild, npm].sort(function (a, b) {
        return (a.sort || a.label).localeCompare(b.sort || b.label);
    });
    exports.templates.push(command);
});

define("vs/workbench/parts/tasks/node/processRunnerConfiguration", ["require", "exports", 'vs/nls!vs/workbench/parts/tasks/node/processRunnerConfiguration', 'vs/base/common/objects', 'vs/base/common/platform', 'vs/base/common/types', 'vs/base/common/uuid', 'vs/base/common/parsers', 'vs/platform/markers/common/problemMatcher', 'vs/workbench/parts/tasks/common/taskSystem'], function (require, exports, nls, Objects, Platform, Types, UUID, parsers_1, problemMatcher_1, TaskSystem) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Defines the problem handling strategy
     */
    var ProblemHandling = (function () {
        function ProblemHandling() {
        }
        /**
         * Cleans all problems for the owner defined in the
         * error pattern.
         */
        ProblemHandling.clean = 'cleanMatcherMatchers';
        return ProblemHandling;
    }());
    exports.ProblemHandling = ProblemHandling;
    var ProblemMatcherKind;
    (function (ProblemMatcherKind) {
        ProblemMatcherKind[ProblemMatcherKind["Unknown"] = 0] = "Unknown";
        ProblemMatcherKind[ProblemMatcherKind["String"] = 1] = "String";
        ProblemMatcherKind[ProblemMatcherKind["ProblemMatcher"] = 2] = "ProblemMatcher";
        ProblemMatcherKind[ProblemMatcherKind["Array"] = 3] = "Array";
    })(ProblemMatcherKind || (ProblemMatcherKind = {}));
    var ConfigurationParser = (function () {
        function ConfigurationParser(logger) {
            this.logger = logger;
            this.validationStatus = new parsers_1.ValidationStatus();
            this.namedProblemMatchers = Object.create(null);
        }
        ConfigurationParser.prototype.log = function (value) {
            this.logger.log(value);
        };
        ConfigurationParser.prototype.run = function (fileConfig) {
            return {
                validationStatus: this.validationStatus,
                configuration: this.createTaskRunnerConfiguration(fileConfig),
                defaultBuildTaskIdentifier: this.defaultBuildTaskIdentifier,
                defaultTestTaskIdentifier: this.defaultTestTaskIdentifier
            };
        };
        ConfigurationParser.prototype.createTaskRunnerConfiguration = function (fileConfig) {
            var globals = this.createGlobals(fileConfig);
            var result = this.createBaseTaskRunnerConfiguration(fileConfig, { isMain: true, globals: globals });
            if (!this.validationStatus.isFatal()) {
                var osConfig = null;
                var osContext = { isMain: false, globals: globals };
                if (fileConfig.windows && Platform.platform === Platform.Platform.Windows) {
                    osConfig = this.createBaseTaskRunnerConfiguration(fileConfig.windows, osContext);
                }
                else if (fileConfig.osx && Platform.platform === Platform.Platform.Mac) {
                    osConfig = this.createBaseTaskRunnerConfiguration(fileConfig.osx, osContext);
                }
                else if (fileConfig.linux && Platform.platform === Platform.Platform.Linux) {
                    osConfig = this.createBaseTaskRunnerConfiguration(fileConfig.linux, osContext);
                }
                if (!this.validationStatus.isFatal()) {
                    if (osConfig) {
                        this.mergeTaskRunnerConigurations(result, osConfig);
                    }
                    if (Types.isUndefined(result.options.cwd)) {
                        result.options.cwd = '${workspaceRoot}';
                    }
                }
            }
            if (!result.command) {
                this.validationStatus.state = parsers_1.ValidationState.Fatal;
                this.log(nls.localize(0, null));
                return null;
            }
            return result;
        };
        ConfigurationParser.prototype.createGlobals = function (fileConfig) {
            var result = this.parseGlobals(fileConfig);
            var osGlobals = null;
            if (fileConfig.windows && Platform.platform === Platform.Platform.Windows) {
                osGlobals = this.parseGlobals(fileConfig.windows);
            }
            else if (fileConfig.osx && Platform.platform === Platform.Platform.Mac) {
                osGlobals = this.parseGlobals(fileConfig.osx);
            }
            else if (fileConfig.linux && Platform.platform === Platform.Platform.Linux) {
                osGlobals = this.parseGlobals(fileConfig.linux);
            }
            if (osGlobals) {
                Objects.mixin(result, osGlobals, true);
            }
            if (Types.isUndefined(result.isShellCommand)) {
                result.isShellCommand = false;
            }
            if (Types.isUndefined(result.showOutput)) {
                result.showOutput = TaskSystem.ShowOutput.Always;
            }
            if (Types.isUndefined(result.echoCommand)) {
                result.echoCommand = false;
            }
            if (Types.isUndefined(result.suppressTaskName)) {
                result.suppressTaskName = false;
            }
            return result;
        };
        ConfigurationParser.prototype.parseGlobals = function (fileConfig) {
            var result = {};
            if (Types.isString(fileConfig.command)) {
                result.command = fileConfig.command;
            }
            if (Types.isBoolean(fileConfig.isShellCommand)) {
                result.isShellCommand = fileConfig.isShellCommand;
            }
            if (Types.isString(fileConfig.showOutput)) {
                result.showOutput = TaskSystem.ShowOutput.fromString(fileConfig.showOutput);
            }
            if (!Types.isUndefined(fileConfig.echoCommand)) {
                result.echoCommand = !!fileConfig.echoCommand;
            }
            if (!Types.isUndefined(fileConfig.suppressTaskName)) {
                result.suppressTaskName = !!fileConfig.suppressTaskName;
            }
            if (Types.isString(fileConfig.taskSelector)) {
                result.taskSelector = fileConfig.taskSelector;
            }
            return result;
        };
        ConfigurationParser.prototype.mergeTaskRunnerConigurations = function (result, osConfig) {
            if (osConfig.command) {
                result.command = osConfig.command;
            }
            if (osConfig.args) {
                result.args = result.args ? result.args.concat(osConfig.args) : osConfig.args;
            }
            if (!Types.isUndefined(osConfig.isShellCommand)) {
                result.isShellCommand = osConfig.isShellCommand;
            }
            if (osConfig.options) {
                if (Types.isString(osConfig.options.cwd)) {
                    result.options.cwd = osConfig.options.cwd;
                }
                if (osConfig.options.env) {
                    var osEnv_1 = osConfig.options.env;
                    var env_1 = result.options.env;
                    if (env_1) {
                        Object.keys(osEnv_1).forEach(function (key) {
                            env_1[key] = osEnv_1[key];
                        });
                    }
                    else {
                        result.options.env = osEnv_1;
                    }
                }
            }
            if (osConfig.tasks) {
                var taskNames_1 = Object.create(null);
                Object.keys(result.tasks).forEach(function (key) {
                    var task = result.tasks[key];
                    taskNames_1[task.name] = task.id;
                });
                var osTaskNames_1 = Object.create(null);
                Object.keys(osConfig.tasks).forEach(function (key) {
                    var task = osConfig.tasks[key];
                    osTaskNames_1[task.name] = task.id;
                });
                Object.keys(osTaskNames_1).forEach(function (taskName) {
                    var id = taskNames_1[taskName];
                    var osId = osTaskNames_1[taskName];
                    // Same name exists globally
                    if (id) {
                        delete result.tasks[id];
                    }
                    result.tasks[osId] = osConfig.tasks[osId];
                });
            }
        };
        ConfigurationParser.prototype.createBaseTaskRunnerConfiguration = function (fileConfig, context) {
            var globals = context.globals;
            var result = {
                isShellCommand: globals.isShellCommand,
                args: [],
            };
            if (Types.isString(fileConfig.command)) {
                result.command = fileConfig.command;
            }
            if (!Types.isUndefined(fileConfig.isShellCommand)) {
                result.isShellCommand = fileConfig.isShellCommand;
            }
            var argsIsValid = Types.isUndefined(fileConfig.args);
            if (Types.isStringArray(fileConfig.args)) {
                argsIsValid = true;
                result.args = fileConfig.args.slice();
            }
            if (!argsIsValid) {
                this.validationStatus.state = parsers_1.ValidationState.Fatal;
                this.log(nls.localize(1, null, fileConfig.args ? JSON.stringify(fileConfig.args, null, 4) : 'undefined'));
            }
            result.options = this.createCommandOptions(fileConfig.options);
            if (context.isMain) {
                this.namedProblemMatchers = this.createNamedProblemMatchers(fileConfig);
            }
            var hasGlobalMatcher = !Types.isUndefined(fileConfig.problemMatcher);
            var hasTasks = Types.isArray(fileConfig.tasks);
            if (hasTasks) {
                result.tasks = this.createTasks(fileConfig.tasks, context);
                if (hasGlobalMatcher) {
                    this.validationStatus.state = parsers_1.ValidationState.Warning;
                    this.log(nls.localize(2, null));
                }
            }
            else if (context.isMain) {
                var isWatching = false;
                if (!Types.isUndefined(fileConfig.isWatching)) {
                    isWatching = !!fileConfig.isWatching;
                }
                var promptOnClose = true;
                if (!Types.isUndefined(fileConfig.promptOnClose)) {
                    promptOnClose = !!fileConfig.promptOnClose;
                }
                else {
                    promptOnClose = !isWatching;
                }
                var task = {
                    id: UUID.generateUuid(),
                    name: globals.command,
                    showOutput: globals.showOutput,
                    suppressTaskName: true,
                    isWatching: isWatching,
                    promptOnClose: promptOnClose,
                    echoCommand: globals.echoCommand,
                };
                if (hasGlobalMatcher) {
                    var problemMatchers = this.createProblemMatchers(fileConfig.problemMatcher);
                    task.problemMatchers = problemMatchers;
                }
                else {
                    task.problemMatchers = [];
                }
                // ToDo@dirkb: this is very special for the tsc watch mode. We should find
                // a exensible solution for this.
                for (var _i = 0, _a = task.problemMatchers; _i < _a.length; _i++) {
                    var matcher = _a[_i];
                    if (matcher.tscWatch) {
                        task.tscWatch = true;
                        break;
                    }
                }
                this.defaultBuildTaskIdentifier = task.id;
                result.tasks = Object.create(null);
                result.tasks[task.id] = task;
            }
            return result;
        };
        ConfigurationParser.prototype.createCommandOptions = function (fileOptions) {
            var result = {};
            if (fileOptions) {
                if (!Types.isUndefined(fileOptions.cwd)) {
                    if (Types.isString(fileOptions.cwd)) {
                        result.cwd = fileOptions.cwd;
                    }
                    else {
                        this.validationStatus.state = parsers_1.ValidationState.Warning;
                        this.log(nls.localize(3, null, fileOptions.cwd));
                    }
                }
                if (!Types.isUndefined(fileOptions.env)) {
                    result.env = Objects.clone(fileOptions.env);
                }
            }
            return result;
        };
        ConfigurationParser.prototype.createNamedProblemMatchers = function (fileConfig) {
            var _this = this;
            var result = Object.create(null);
            if (!Types.isArray(fileConfig.declares)) {
                return result;
            }
            var values = fileConfig.declares;
            values.forEach(function (value) {
                var namedProblemMatcher = _this.createNamedProblemMatcher(value);
                if (namedProblemMatcher) {
                    result[namedProblemMatcher.name] = namedProblemMatcher;
                }
            });
            return result;
        };
        ConfigurationParser.prototype.createNamedProblemMatcher = function (value) {
            var result = (new problemMatcher_1.ProblemMatcherParser(problemMatcher_1.registry, this.logger, this.validationStatus)).parse(value);
            if (problemMatcher_1.isNamedProblemMatcher(result)) {
                return result;
            }
            else {
                this.validationStatus.state = parsers_1.ValidationState.Error;
                this.log(nls.localize(4, null, JSON.stringify(value, null, 4)));
                return null;
            }
        };
        ConfigurationParser.prototype.createTasks = function (tasks, context) {
            var _this = this;
            var result = Object.create(null);
            if (!tasks) {
                return result;
            }
            var defaultBuildTask = { id: null, exact: -1 };
            var defaultTestTask = { id: null, exact: -1 };
            tasks.forEach(function (externalTask) {
                var taskName = externalTask.taskName;
                if (!taskName) {
                    _this.validationStatus.state = parsers_1.ValidationState.Fatal;
                    _this.log(nls.localize(5, null, JSON.stringify(externalTask, null, 4)));
                    return;
                }
                var problemMatchers = _this.createProblemMatchers(externalTask.problemMatcher);
                var task = { id: UUID.generateUuid(), name: taskName, showOutput: context.globals.showOutput };
                if (Types.isStringArray(externalTask.args)) {
                    task.args = externalTask.args.slice();
                }
                task.isWatching = false;
                if (!Types.isUndefined(externalTask.isWatching)) {
                    task.isWatching = !!externalTask.isWatching;
                }
                task.promptOnClose = true;
                if (!Types.isUndefined(externalTask.promptOnClose)) {
                    task.promptOnClose = !!externalTask.promptOnClose;
                }
                else {
                    task.promptOnClose = !task.isWatching;
                }
                if (Types.isString(externalTask.showOutput)) {
                    task.showOutput = TaskSystem.ShowOutput.fromString(externalTask.showOutput);
                }
                if (Types.isUndefined(task.showOutput)) {
                    task.showOutput = context.globals.showOutput;
                }
                task.echoCommand = context.globals.echoCommand;
                if (!Types.isUndefined(externalTask.echoCommand)) {
                    task.echoCommand = !!externalTask.echoCommand;
                }
                task.suppressTaskName = context.globals.suppressTaskName;
                if (!Types.isUndefined(externalTask.suppressTaskName)) {
                    task.suppressTaskName = !!externalTask.suppressTaskName;
                }
                if (problemMatchers) {
                    task.problemMatchers = problemMatchers;
                }
                // ToDo@dirkb: this is very special for the tsc watch mode. We should find
                // a exensible solution for this.
                for (var _i = 0, _a = task.problemMatchers; _i < _a.length; _i++) {
                    var matcher = _a[_i];
                    if (matcher.tscWatch) {
                        task.tscWatch = true;
                        break;
                    }
                }
                result[task.id] = task;
                if (!Types.isUndefined(externalTask.isBuildCommand) && externalTask.isBuildCommand && defaultBuildTask.exact < 2) {
                    defaultBuildTask.id = task.id;
                    defaultBuildTask.exact = 2;
                }
                else if (taskName === 'build' && defaultBuildTask.exact < 2) {
                    defaultBuildTask.id = task.id;
                    defaultBuildTask.exact = 1;
                }
                if (!Types.isUndefined(externalTask.isTestCommand) && externalTask.isTestCommand && defaultTestTask.exact < 2) {
                    defaultTestTask.id = task.id;
                    defaultTestTask.exact = 2;
                }
                else if (taskName === 'test' && defaultTestTask.exact < 2) {
                    defaultTestTask.id = task.id;
                    defaultTestTask.exact = 1;
                }
            });
            if (defaultBuildTask.exact > 0) {
                this.defaultBuildTaskIdentifier = defaultBuildTask.id;
            }
            if (defaultTestTask.exact > 0) {
                this.defaultTestTaskIdentifier = defaultTestTask.id;
            }
            return result;
        };
        ConfigurationParser.prototype.createProblemMatchers = function (problemMatcher) {
            var _this = this;
            var result = [];
            if (Types.isUndefined(problemMatcher)) {
                return result;
            }
            var kind = this.getProblemMatcherKind(problemMatcher);
            if (kind === ProblemMatcherKind.Unknown) {
                this.validationStatus.state = parsers_1.ValidationState.Warning;
                this.log(nls.localize(6, null, JSON.stringify(problemMatcher, null, 4)));
                return result;
            }
            else if (kind === ProblemMatcherKind.String || kind === ProblemMatcherKind.ProblemMatcher) {
                var matcher = this.resolveProblemMatcher(problemMatcher);
                if (matcher) {
                    result.push(matcher);
                }
            }
            else if (kind === ProblemMatcherKind.Array) {
                var problemMatchers = problemMatcher;
                problemMatchers.forEach(function (problemMatcher) {
                    var matcher = _this.resolveProblemMatcher(problemMatcher);
                    if (matcher) {
                        result.push(matcher);
                    }
                });
            }
            return result;
        };
        ConfigurationParser.prototype.getProblemMatcherKind = function (value) {
            if (Types.isString(value)) {
                return ProblemMatcherKind.String;
            }
            else if (Types.isArray(value)) {
                return ProblemMatcherKind.Array;
            }
            else if (!Types.isUndefined(value)) {
                return ProblemMatcherKind.ProblemMatcher;
            }
            else {
                return ProblemMatcherKind.Unknown;
            }
        };
        ConfigurationParser.prototype.resolveProblemMatcher = function (value) {
            if (Types.isString(value)) {
                var variableName = value;
                if (variableName.length > 1 && variableName[0] === '$') {
                    variableName = variableName.substring(1);
                    var global_1 = problemMatcher_1.registry.get(variableName);
                    if (global_1) {
                        return Objects.clone(global_1);
                    }
                    var localProblemMatcher = this.namedProblemMatchers[variableName];
                    if (localProblemMatcher) {
                        localProblemMatcher = Objects.clone(localProblemMatcher);
                        // remove the name
                        delete localProblemMatcher.name;
                        return localProblemMatcher;
                    }
                }
                this.validationStatus.state = parsers_1.ValidationState.Error;
                this.log(nls.localize(7, null, value));
                return null;
            }
            else {
                var json = value;
                return new problemMatcher_1.ProblemMatcherParser(problemMatcher_1.registry, this.logger, this.validationStatus).parse(json);
            }
        };
        return ConfigurationParser;
    }());
    function parse(configuration, logger) {
        return (new ConfigurationParser(logger)).run(configuration);
    }
    exports.parse = parse;
});

define("vs/workbench/parts/tasks/node/processRunnerDetector", ["require", "exports", 'vs/nls!vs/workbench/parts/tasks/node/processRunnerDetector', 'vs/base/common/strings', 'vs/base/common/processes', 'vs/base/node/processes'], function (require, exports, nls, Strings, processes_1, processes_2) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var build = 'build';
    var test = 'test';
    var RegexpTaskMatcher = (function () {
        function RegexpTaskMatcher(regExp) {
            this.regexp = regExp;
        }
        RegexpTaskMatcher.prototype.init = function () {
        };
        RegexpTaskMatcher.prototype.match = function (tasks, line) {
            var matches = this.regexp.exec(line);
            if (matches && matches.length > 0) {
                tasks.push(matches[1]);
            }
        };
        return RegexpTaskMatcher;
    }());
    var GruntTaskMatcher = (function () {
        function GruntTaskMatcher() {
        }
        GruntTaskMatcher.prototype.init = function () {
            this.tasksStart = false;
            this.tasksEnd = false;
            this.descriptionOffset = null;
        };
        GruntTaskMatcher.prototype.match = function (tasks, line) {
            // grunt lists tasks as follows (description is wrapped into a new line if too long):
            // ...
            // Available tasks
            //         uglify  Minify files with UglifyJS. *
            //         jshint  Validate files with JSHint. *
            //           test  Alias for "jshint", "qunit" tasks.
            //        default  Alias for "jshint", "qunit", "concat", "uglify" tasks.
            //           long  Alias for "eslint", "qunit", "browserify", "sass",
            //                 "autoprefixer", "uglify", tasks.
            //
            // Tasks run in the order specified
            if (!this.tasksStart && !this.tasksEnd) {
                if (line.indexOf('Available tasks') === 0) {
                    this.tasksStart = true;
                }
            }
            else if (this.tasksStart && !this.tasksEnd) {
                if (line.indexOf('Tasks run in the order specified') === 0) {
                    this.tasksEnd = true;
                }
                else {
                    if (this.descriptionOffset === null) {
                        this.descriptionOffset = line.match(/\S  \S/).index + 1;
                    }
                    var taskName = line.substr(0, this.descriptionOffset).trim();
                    if (taskName.length > 0) {
                        tasks.push(taskName);
                    }
                }
            }
        };
        return GruntTaskMatcher;
    }());
    var ProcessRunnerDetector = (function () {
        function ProcessRunnerDetector(fileService, contextService, variables, config) {
            if (config === void 0) { config = null; }
            this.fileService = fileService;
            this.contextService = contextService;
            this.variables = variables;
            this.taskConfiguration = config;
            this._stderr = [];
            this._stdout = [];
        }
        ProcessRunnerDetector.supports = function (runner) {
            return ProcessRunnerDetector.SupportedRunners[runner];
        };
        ProcessRunnerDetector.detectorConfig = function (runner) {
            return ProcessRunnerDetector.TaskMatchers[runner];
        };
        Object.defineProperty(ProcessRunnerDetector.prototype, "stderr", {
            get: function () {
                return this._stderr;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ProcessRunnerDetector.prototype, "stdout", {
            get: function () {
                return this._stdout;
            },
            enumerable: true,
            configurable: true
        });
        ProcessRunnerDetector.prototype.detect = function (list, detectSpecific) {
            var _this = this;
            if (list === void 0) { list = false; }
            if (this.taskConfiguration && this.taskConfiguration.command && ProcessRunnerDetector.supports(this.taskConfiguration.command)) {
                var config = ProcessRunnerDetector.detectorConfig(this.taskConfiguration.command);
                var args = (this.taskConfiguration.args || []).concat(config.arg);
                var options = this.taskConfiguration.options ? processes_1.resolveCommandOptions(this.taskConfiguration.options, this.variables) : { cwd: this.variables.workspaceRoot };
                var isShellCommand = !!this.taskConfiguration.isShellCommand;
                return this.runDetection(new processes_2.LineProcess(this.taskConfiguration.command, this.variables.resolve(args), isShellCommand, options), this.taskConfiguration.command, isShellCommand, config.matcher, ProcessRunnerDetector.DefaultProblemMatchers, list);
            }
            else {
                if (detectSpecific) {
                    var detectorPromise = void 0;
                    if ('gulp' === detectSpecific) {
                        detectorPromise = this.tryDetectGulp(list);
                    }
                    else if ('jake' === detectSpecific) {
                        detectorPromise = this.tryDetectJake(list);
                    }
                    else if ('grunt' === detectSpecific) {
                        detectorPromise = this.tryDetectGrunt(list);
                    }
                    return detectorPromise.then(function (value) {
                        if (value) {
                            return value;
                        }
                        else {
                            return { config: null, stdout: _this.stdout, stderr: _this.stderr };
                        }
                    });
                }
                else {
                    return this.tryDetectGulp(list).then(function (value) {
                        if (value) {
                            return value;
                        }
                        return _this.tryDetectJake(list).then(function (value) {
                            if (value) {
                                return value;
                            }
                            return _this.tryDetectGrunt(list).then(function (value) {
                                if (value) {
                                    return value;
                                }
                                return { config: null, stdout: _this.stdout, stderr: _this.stderr };
                            });
                        });
                    });
                }
            }
        };
        ProcessRunnerDetector.prototype.tryDetectGulp = function (list) {
            var _this = this;
            return this.fileService.resolveFile(this.contextService.toResource('gulpfile.js')).then(function (stat) {
                var config = ProcessRunnerDetector.detectorConfig('gulp');
                var process = new processes_2.LineProcess('gulp', [config.arg, '--no-color'], true, { cwd: _this.variables.workspaceRoot });
                return _this.runDetection(process, 'gulp', true, config.matcher, ProcessRunnerDetector.DefaultProblemMatchers, list);
            }, function (err) {
                return null;
            });
        };
        ProcessRunnerDetector.prototype.tryDetectGrunt = function (list) {
            var _this = this;
            return this.fileService.resolveFile(this.contextService.toResource('Gruntfile.js')).then(function (stat) {
                var config = ProcessRunnerDetector.detectorConfig('grunt');
                var process = new processes_2.LineProcess('grunt', [config.arg, '--no-color'], true, { cwd: _this.variables.workspaceRoot });
                return _this.runDetection(process, 'grunt', true, config.matcher, ProcessRunnerDetector.DefaultProblemMatchers, list);
            }, function (err) {
                return null;
            });
        };
        ProcessRunnerDetector.prototype.tryDetectJake = function (list) {
            var _this = this;
            var run = function () {
                var config = ProcessRunnerDetector.detectorConfig('jake');
                var process = new processes_2.LineProcess('jake', [config.arg], true, { cwd: _this.variables.workspaceRoot });
                return _this.runDetection(process, 'jake', true, config.matcher, ProcessRunnerDetector.DefaultProblemMatchers, list);
            };
            return this.fileService.resolveFile(this.contextService.toResource('Jakefile')).then(function (stat) {
                return run();
            }, function (err) {
                return _this.fileService.resolveFile(_this.contextService.toResource('Jakefile.js')).then(function (stat) {
                    return run();
                }, function (err) {
                    return null;
                });
            });
        };
        ProcessRunnerDetector.prototype.runDetection = function (process, command, isShellCommand, matcher, problemMatchers, list) {
            var _this = this;
            var tasks = [];
            matcher.init();
            return process.start().then(function (success) {
                if (tasks.length === 0) {
                    if (success.cmdCode !== 0) {
                        if (command === 'gulp') {
                            _this._stderr.push(nls.localize(0, null));
                        }
                        else if (command === 'jake') {
                            _this._stderr.push(nls.localize(1, null));
                        }
                    }
                    return { config: null, stdout: _this._stdout, stderr: _this._stderr };
                }
                var result = {
                    version: ProcessRunnerDetector.Version,
                    command: command,
                    isShellCommand: isShellCommand
                };
                // Hack. We need to remove this.
                if (command === 'gulp') {
                    result.args = ['--no-color'];
                }
                result.tasks = _this.createTaskDescriptions(tasks, problemMatchers, list);
                return { config: result, stdout: _this._stdout, stderr: _this._stderr };
            }, function (err) {
                var error = err.error;
                if (error.code === 'ENOENT') {
                    if (command === 'gulp') {
                        _this._stderr.push(nls.localize(2, null));
                    }
                    else if (command === 'jake') {
                        _this._stderr.push(nls.localize(3, null));
                    }
                    else if (command === 'grunt') {
                        _this._stderr.push(nls.localize(4, null));
                    }
                }
                else {
                    _this._stderr.push(nls.localize(5, null, command, error.message));
                }
                return { config: null, stdout: _this._stdout, stderr: _this._stderr };
            }, function (progress) {
                if (progress.source === processes_1.Source.stderr) {
                    _this._stderr.push(progress.line);
                    return;
                }
                var line = Strings.removeAnsiEscapeCodes(progress.line);
                var matches = matcher.match(tasks, line);
                if (matches && matches.length > 0) {
                    tasks.push(matches[1]);
                }
            });
        };
        ProcessRunnerDetector.prototype.createTaskDescriptions = function (tasks, problemMatchers, list) {
            var _this = this;
            var taskConfigs = [];
            if (list) {
                tasks.forEach(function (task) {
                    taskConfigs.push({
                        taskName: task,
                        args: [],
                        isWatching: false
                    });
                });
            }
            else {
                var taskInfos_1 = {
                    build: { index: -1, exact: -1 },
                    test: { index: -1, exact: -1 }
                };
                tasks.forEach(function (task, index) {
                    _this.testBuild(taskInfos_1.build, task, index);
                    _this.testTest(taskInfos_1.test, task, index);
                });
                if (taskInfos_1.build.index !== -1) {
                    var name_1 = tasks[taskInfos_1.build.index];
                    this._stdout.push(nls.localize(6, null, name_1));
                    taskConfigs.push({
                        taskName: name_1,
                        args: [],
                        isBuildCommand: true,
                        isWatching: false,
                        problemMatcher: problemMatchers
                    });
                }
                if (taskInfos_1.test.index !== -1) {
                    var name_2 = tasks[taskInfos_1.test.index];
                    this._stdout.push(nls.localize(7, null, name_2));
                    taskConfigs.push({
                        taskName: name_2,
                        args: [],
                        isTestCommand: true
                    });
                }
            }
            return taskConfigs;
        };
        ProcessRunnerDetector.prototype.testBuild = function (taskInfo, taskName, index) {
            if (taskName === build) {
                taskInfo.index = index;
                taskInfo.exact = 3;
            }
            else if ((Strings.startsWith(taskName, build) || Strings.endsWith(taskName, build)) && taskInfo.exact < 3) {
                taskInfo.index = index;
                taskInfo.exact = 2;
            }
            else if (taskName.indexOf(build) !== -1 && taskInfo.exact < 2) {
                taskInfo.index = index;
                taskInfo.exact = 1;
            }
        };
        ProcessRunnerDetector.prototype.testTest = function (taskInfo, taskName, index) {
            if (taskName === test) {
                taskInfo.index = index;
                taskInfo.exact = 3;
            }
            else if ((Strings.startsWith(taskName, test) || Strings.endsWith(taskName, test)) && taskInfo.exact < 3) {
                taskInfo.index = index;
                taskInfo.exact = 2;
            }
            else if (taskName.indexOf(test) !== -1 && taskInfo.exact < 2) {
                taskInfo.index = index;
                taskInfo.exact = 1;
            }
        };
        ProcessRunnerDetector.Version = '0.1.0';
        ProcessRunnerDetector.SupportedRunners = {
            'gulp': true,
            'jake': true,
            'grunt': true
        };
        ProcessRunnerDetector.TaskMatchers = {
            'gulp': { matcher: new RegexpTaskMatcher(/^(.*)$/), arg: '--tasks-simple' },
            'jake': { matcher: new RegexpTaskMatcher(/^jake\s+([^\s]+)\s/), arg: '--tasks' },
            'grunt': { matcher: new GruntTaskMatcher(), arg: '--help' },
        };
        ProcessRunnerDetector.DefaultProblemMatchers = ['$lessCompile', '$tsc', '$jshint'];
        return ProcessRunnerDetector;
    }());
    exports.ProcessRunnerDetector = ProcessRunnerDetector;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/tasks/node/processRunnerSystem", ["require", "exports", 'vs/nls!vs/workbench/parts/tasks/node/processRunnerSystem', 'vs/base/common/objects', 'vs/base/common/types', 'vs/base/common/platform', 'vs/base/common/winjs.base', 'vs/base/common/async', 'vs/base/common/severity', 'vs/base/common/strings', 'vs/base/common/eventEmitter', 'vs/base/node/processes', 'vs/workbench/parts/tasks/common/problemCollectors', 'vs/workbench/parts/tasks/common/taskSystem', './processRunnerConfiguration'], function (require, exports, nls, Objects, Types, Platform, winjs_base_1, Async, severity_1, Strings, eventEmitter_1, processes_1, problemCollectors_1, taskSystem_1, FileConfig) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ProcessRunnerSystem = (function (_super) {
        __extends(ProcessRunnerSystem, _super);
        function ProcessRunnerSystem(fileConfig, variables, markerService, modelService, telemetryService, outputService, outputChannel, clearOutput) {
            if (clearOutput === void 0) { clearOutput = true; }
            _super.call(this);
            this.fileConfig = fileConfig;
            this.variables = variables;
            this.markerService = markerService;
            this.modelService = modelService;
            this.outputChannel = outputChannel;
            this.outputService = outputService;
            this.telemetryService = telemetryService;
            this.defaultBuildTaskIdentifier = null;
            this.defaultTestTaskIdentifier = null;
            this.childProcess = null;
            this.activeTaskIdentifier = null;
            if (clearOutput) {
                this.clearOutput();
            }
            this.errorsShown = false;
            var parseResult = FileConfig.parse(fileConfig, this);
            this.validationStatus = parseResult.validationStatus;
            this.configuration = parseResult.configuration;
            this.defaultBuildTaskIdentifier = parseResult.defaultBuildTaskIdentifier;
            this.defaultTestTaskIdentifier = parseResult.defaultTestTaskIdentifier;
            if (!this.validationStatus.isOK()) {
                this.outputService.showOutput(this.outputChannel, true);
            }
        }
        ProcessRunnerSystem.prototype.build = function () {
            if (!this.defaultBuildTaskIdentifier) {
                throw new taskSystem_1.TaskError(severity_1.default.Info, nls.localize(0, null), taskSystem_1.TaskErrors.NoBuildTask);
            }
            return this.executeTask(this.defaultBuildTaskIdentifier, taskSystem_1.Triggers.shortcut);
        };
        ProcessRunnerSystem.prototype.rebuild = function () {
            throw new Error('Task - Rebuild: not implemented yet');
        };
        ProcessRunnerSystem.prototype.clean = function () {
            throw new Error('Task - Clean: not implemented yet');
        };
        ProcessRunnerSystem.prototype.runTest = function () {
            if (!this.defaultTestTaskIdentifier) {
                throw new taskSystem_1.TaskError(severity_1.default.Info, nls.localize(1, null), taskSystem_1.TaskErrors.NoTestTask);
            }
            return this.executeTask(this.defaultTestTaskIdentifier, taskSystem_1.Triggers.shortcut);
        };
        ProcessRunnerSystem.prototype.run = function (taskIdentifier) {
            return this.executeTask(taskIdentifier);
        };
        ProcessRunnerSystem.prototype.isActive = function () {
            return winjs_base_1.TPromise.as(!!this.childProcess);
        };
        ProcessRunnerSystem.prototype.isActiveSync = function () {
            return !!this.childProcess;
        };
        ProcessRunnerSystem.prototype.canAutoTerminate = function () {
            if (this.childProcess) {
                if (this.activeTaskIdentifier) {
                    var task = this.configuration.tasks[this.activeTaskIdentifier];
                    if (task) {
                        return !task.promptOnClose;
                    }
                }
                return false;
            }
            return true;
        };
        ProcessRunnerSystem.prototype.terminate = function () {
            if (this.childProcess) {
                return this.childProcess.terminate();
            }
            return winjs_base_1.TPromise.as({ success: true });
        };
        ProcessRunnerSystem.prototype.tasks = function () {
            var _this = this;
            var result;
            if (!this.configuration || !this.configuration.tasks) {
                result = [];
            }
            else {
                result = Object.keys(this.configuration.tasks).map(function (key) { return _this.configuration.tasks[key]; });
            }
            return winjs_base_1.TPromise.as(result);
        };
        ProcessRunnerSystem.prototype.executeTask = function (taskIdentifier, trigger) {
            var _this = this;
            if (trigger === void 0) { trigger = taskSystem_1.Triggers.command; }
            if (this.validationStatus.isFatal()) {
                throw new taskSystem_1.TaskError(severity_1.default.Error, nls.localize(2, null), taskSystem_1.TaskErrors.ConfigValidationError);
            }
            var task = this.configuration.tasks[taskIdentifier];
            if (!task) {
                throw new taskSystem_1.TaskError(severity_1.default.Info, nls.localize(3, null), taskSystem_1.TaskErrors.TaskNotFound);
            }
            var telemetryEvent = {
                trigger: trigger,
                command: 'other',
                success: true
            };
            try {
                var result = this.doExecuteTask(task, telemetryEvent);
                result.promise = result.promise.then(function (success) {
                    _this.telemetryService.publicLog(ProcessRunnerSystem.TelemetryEventName, telemetryEvent);
                    return success;
                }, function (err) {
                    telemetryEvent.success = false;
                    _this.telemetryService.publicLog(ProcessRunnerSystem.TelemetryEventName, telemetryEvent);
                    return winjs_base_1.TPromise.wrapError(err);
                });
                return result;
            }
            catch (err) {
                telemetryEvent.success = false;
                this.telemetryService.publicLog(ProcessRunnerSystem.TelemetryEventName, telemetryEvent);
                if (err instanceof taskSystem_1.TaskError) {
                    throw err;
                }
                else if (err instanceof Error) {
                    var error = err;
                    this.outputService.append(this.outputChannel, error.message);
                    throw new taskSystem_1.TaskError(severity_1.default.Error, error.message, taskSystem_1.TaskErrors.UnknownError);
                }
                else {
                    this.outputService.append(this.outputChannel, err.toString());
                    throw new taskSystem_1.TaskError(severity_1.default.Error, nls.localize(4, null), taskSystem_1.TaskErrors.UnknownError);
                }
            }
        };
        ProcessRunnerSystem.prototype.doExecuteTask = function (task, telemetryEvent) {
            var _this = this;
            var taskSummary = {};
            var configuration = this.configuration;
            if (!this.validationStatus.isOK() && !this.errorsShown) {
                this.showOutput();
                this.errorsShown = true;
            }
            else {
                this.clearOutput();
            }
            var args = this.configuration.args ? this.configuration.args.slice() : [];
            // We need to first pass the task name
            if (!task.suppressTaskName) {
                if (this.fileConfig.taskSelector) {
                    args.push(this.fileConfig.taskSelector + task.name);
                }
                else {
                    args.push(task.name);
                }
            }
            // And then additional arguments
            if (task.args) {
                args = args.concat(task.args);
            }
            args = this.resolveVariables(args);
            var command = this.resolveVariable(configuration.command);
            this.childProcess = new processes_1.LineProcess(command, args, configuration.isShellCommand, this.resolveOptions(configuration.options));
            telemetryEvent.command = this.childProcess.getSanitizedCommand();
            // we have no problem matchers defined. So show the output log
            if (task.showOutput === taskSystem_1.ShowOutput.Always || (task.showOutput === taskSystem_1.ShowOutput.Silent && task.problemMatchers.length === 0)) {
                this.showOutput();
            }
            if (task.echoCommand) {
                var prompt_1 = Platform.isWindows ? '>' : '$';
                this.log("running command" + prompt_1 + " " + command + " " + args.join(' '));
            }
            if (task.isWatching) {
                var watchingProblemMatcher_1 = new problemCollectors_1.WatchingProblemCollector(this.resolveMatchers(task.problemMatchers), this.markerService, this.modelService);
                var toUnbind_1 = [];
                var event_1 = { taskId: task.id, taskName: task.name, type: taskSystem_1.TaskType.Watching };
                var eventCounter_1 = 0;
                toUnbind_1.push(watchingProblemMatcher_1.on(problemCollectors_1.ProblemCollectorEvents.WatchingBeginDetected, function () {
                    eventCounter_1++;
                    _this.emit(taskSystem_1.TaskSystemEvents.Active, event_1);
                }));
                toUnbind_1.push(watchingProblemMatcher_1.on(problemCollectors_1.ProblemCollectorEvents.WatchingEndDetected, function () {
                    eventCounter_1--;
                    _this.emit(taskSystem_1.TaskSystemEvents.Inactive, event_1);
                }));
                watchingProblemMatcher_1.aboutToStart();
                var delayer_1 = null;
                this.activeTaskIdentifier = task.id;
                var promise = this.childProcess.start().then(function (success) {
                    _this.childProcessEnded();
                    watchingProblemMatcher_1.dispose();
                    toUnbind_1.forEach(function (unbind) { return unbind(); });
                    toUnbind_1 = null;
                    for (var i = 0; i < eventCounter_1; i++) {
                        _this.emit(taskSystem_1.TaskSystemEvents.Inactive, event_1);
                    }
                    eventCounter_1 = 0;
                    if (!_this.checkTerminated(task, success)) {
                        _this.log(nls.localize(5, null));
                    }
                    if (success.cmdCode && success.cmdCode === 1 && watchingProblemMatcher_1.numberOfMatches === 0 && task.showOutput !== taskSystem_1.ShowOutput.Never) {
                        _this.showOutput();
                    }
                    taskSummary.exitCode = success.cmdCode;
                    return taskSummary;
                }, function (error) {
                    _this.childProcessEnded();
                    watchingProblemMatcher_1.dispose();
                    toUnbind_1.forEach(function (unbind) { return unbind(); });
                    toUnbind_1 = null;
                    for (var i = 0; i < eventCounter_1; i++) {
                        _this.emit(taskSystem_1.TaskSystemEvents.Inactive, event_1);
                    }
                    eventCounter_1 = 0;
                    return _this.handleError(task, error);
                }, function (progress) {
                    var line = Strings.removeAnsiEscapeCodes(progress.line);
                    _this.outputService.append(_this.outputChannel, line + '\n');
                    watchingProblemMatcher_1.processLine(line);
                    if (delayer_1 === null) {
                        delayer_1 = new Async.Delayer(3000);
                    }
                    delayer_1.trigger(function () {
                        watchingProblemMatcher_1.forceDelivery();
                        return null;
                    }).then(function () {
                        delayer_1 = null;
                    });
                });
                var result = task.tscWatch ? { restartOnFileChanges: '**/*.ts', promise: promise } : { promise: promise };
                return result;
            }
            else {
                var event_2 = { taskId: task.id, taskName: task.name, type: taskSystem_1.TaskType.SingleRun };
                this.emit(taskSystem_1.TaskSystemEvents.Active, event_2);
                var startStopProblemMatcher_1 = new problemCollectors_1.StartStopProblemCollector(this.resolveMatchers(task.problemMatchers), this.markerService, this.modelService);
                this.activeTaskIdentifier = task.id;
                var promise = this.childProcess.start().then(function (success) {
                    _this.childProcessEnded();
                    startStopProblemMatcher_1.done();
                    startStopProblemMatcher_1.dispose();
                    _this.checkTerminated(task, success);
                    _this.emit(taskSystem_1.TaskSystemEvents.Inactive, event_2);
                    if (success.cmdCode && success.cmdCode === 1 && startStopProblemMatcher_1.numberOfMatches === 0 && task.showOutput !== taskSystem_1.ShowOutput.Never) {
                        _this.showOutput();
                    }
                    taskSummary.exitCode = success.cmdCode;
                    return taskSummary;
                }, function (error) {
                    _this.childProcessEnded();
                    startStopProblemMatcher_1.dispose();
                    _this.emit(taskSystem_1.TaskSystemEvents.Inactive, event_2);
                    return _this.handleError(task, error);
                }, function (progress) {
                    var line = Strings.removeAnsiEscapeCodes(progress.line);
                    _this.outputService.append(_this.outputChannel, line + '\n');
                    startStopProblemMatcher_1.processLine(line);
                });
                return { promise: promise };
            }
        };
        ProcessRunnerSystem.prototype.childProcessEnded = function () {
            this.childProcess = null;
            this.activeTaskIdentifier = null;
        };
        ProcessRunnerSystem.prototype.handleError = function (task, error) {
            var makeVisible = false;
            if (error.error && !error.terminated) {
                var args = this.configuration.args ? this.configuration.args.join(' ') : '';
                this.log(nls.localize(6, null, this.configuration.command, args));
                this.outputService.append(this.outputChannel, error.error.message);
                makeVisible = true;
            }
            if (error.stdout) {
                this.outputService.append(this.outputChannel, error.stdout);
                makeVisible = true;
            }
            if (error.stderr) {
                this.outputService.append(this.outputChannel, error.stderr);
                makeVisible = true;
            }
            makeVisible = this.checkTerminated(task, error) || makeVisible;
            if (makeVisible) {
                this.showOutput();
            }
            return winjs_base_1.Promise.wrapError(error);
        };
        ProcessRunnerSystem.prototype.checkTerminated = function (task, data) {
            if (data.terminated) {
                this.log(nls.localize(7, null, task.name));
                return true;
            }
            return false;
        };
        ProcessRunnerSystem.prototype.resolveOptions = function (options) {
            var _this = this;
            var result = { cwd: this.resolveVariable(options.cwd) };
            if (options.env) {
                result.env = Object.create(null);
                Object.keys(options.env).forEach(function (key) {
                    var value = options.env[key];
                    if (Types.isString(value)) {
                        result.env[key] = _this.resolveVariable(value);
                    }
                    else {
                        result.env[key] = value.toString();
                    }
                });
            }
            return result;
        };
        ProcessRunnerSystem.prototype.resolveVariables = function (value) {
            var _this = this;
            return value.map(function (s) { return _this.resolveVariable(s); });
        };
        ProcessRunnerSystem.prototype.resolveMatchers = function (values) {
            var _this = this;
            if (values.length === 0) {
                return values;
            }
            var result = [];
            values.forEach(function (matcher) {
                if (!matcher.filePrefix) {
                    result.push(matcher);
                }
                else {
                    var copy = Objects.clone(matcher);
                    copy.filePrefix = _this.resolveVariable(copy.filePrefix);
                    result.push(copy);
                }
            });
            return result;
        };
        ProcessRunnerSystem.prototype.resolveVariable = function (value) {
            var _this = this;
            var regexp = /\$\{(.*?)\}/g;
            return value.replace(regexp, function (match, name) {
                var value = _this.variables[name];
                if (value) {
                    return value;
                }
                else {
                    return match;
                }
            });
        };
        ProcessRunnerSystem.prototype.log = function (value) {
            this.outputService.append(this.outputChannel, value + '\n');
        };
        ProcessRunnerSystem.prototype.showOutput = function () {
            this.outputService.showOutput(this.outputChannel, true);
        };
        ProcessRunnerSystem.prototype.clearOutput = function () {
            this.outputService.clearOutput(this.outputChannel);
        };
        ProcessRunnerSystem.TelemetryEventName = 'taskService';
        return ProcessRunnerSystem;
    }(eventEmitter_1.EventEmitter));
    exports.ProcessRunnerSystem = ProcessRunnerSystem;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/activity/common/activityService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var BaseBadge = (function () {
        function BaseBadge(descriptorFn) {
            this.descriptorFn = descriptorFn;
        }
        /* protected */ BaseBadge.prototype.getDescription = function () {
            return this.descriptorFn(null);
        };
        return BaseBadge;
    }());
    exports.BaseBadge = BaseBadge;
    var NumberBadge = (function (_super) {
        __extends(NumberBadge, _super);
        function NumberBadge(number, descriptorFn) {
            _super.call(this, descriptorFn);
            this.number = number;
        }
        /* protected */ NumberBadge.prototype.getDescription = function () {
            return this.descriptorFn(this.number);
        };
        return NumberBadge;
    }(BaseBadge));
    exports.NumberBadge = NumberBadge;
    var TextBadge = (function (_super) {
        __extends(TextBadge, _super);
        function TextBadge(text, descriptorFn) {
            _super.call(this, descriptorFn);
            this.text = text;
        }
        return TextBadge;
    }(BaseBadge));
    exports.TextBadge = TextBadge;
    var IconBadge = (function (_super) {
        __extends(IconBadge, _super);
        function IconBadge(descriptorFn) {
            _super.call(this, descriptorFn);
        }
        return IconBadge;
    }(BaseBadge));
    exports.IconBadge = IconBadge;
    var ProgressBadge = (function (_super) {
        __extends(ProgressBadge, _super);
        function ProgressBadge() {
            _super.apply(this, arguments);
        }
        return ProgressBadge;
    }(BaseBadge));
    exports.ProgressBadge = ProgressBadge;
    exports.IActivityService = instantiation_1.createDecorator('activityService');
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/browser/parts/activitybar/activityAction", ["require", "exports", 'vs/nls!vs/workbench/browser/parts/activitybar/activityAction', 'vs/base/browser/builder', 'vs/base/browser/dnd', 'vs/base/common/actions', 'vs/base/browser/ui/actionbar/actionbar', 'vs/workbench/services/activity/common/activityService', 'vs/css!./media/activityaction'], function (require, exports, nls, builder_1, dnd_1, actions_1, actionbar_1, activityService_1) {
    'use strict';
    var ActivityAction = (function (_super) {
        __extends(ActivityAction, _super);
        function ActivityAction(id, name, clazz) {
            _super.call(this, id, name, clazz);
            this.badge = null;
        }
        ActivityAction.prototype.activate = function () {
            if (!this.checked) {
                this.checked = true;
                this.emit('checked', { source: this });
            }
        };
        ActivityAction.prototype.deactivate = function () {
            if (this.checked) {
                this.checked = false;
                this.emit('checked', { source: this });
            }
        };
        ActivityAction.prototype.getBadge = function () {
            return this.badge;
        };
        ActivityAction.prototype.setBadge = function (badge) {
            this.badge = badge;
            this.emit(ActivityAction.BADGE, { source: this });
        };
        ActivityAction.BADGE = 'badge';
        return ActivityAction;
    }(actions_1.Action));
    exports.ActivityAction = ActivityAction;
    var ActivityActionItem = (function (_super) {
        __extends(ActivityActionItem, _super);
        function ActivityActionItem(action, activityName, keybinding) {
            if (activityName === void 0) { activityName = action.label; }
            if (keybinding === void 0) { keybinding = null; }
            _super.call(this, null, action);
            this.cssClass = action.class;
            this.name = activityName;
            this._keybinding = keybinding;
        }
        ActivityActionItem.prototype.render = function (container) {
            var _this = this;
            _super.prototype.render.call(this, container);
            this.$e = builder_1.$('a.action-label').attr({
                tabIndex: '0',
                role: 'button'
            }).appendTo(this.builder);
            if (this.cssClass) {
                this.$e.addClass(this.cssClass);
            }
            this.$badge = this.builder.div({ 'class': 'badge' }, function (badge) {
                _this.$badgeContent = badge.div({ 'class': 'badge-content' });
            });
            this.$badge.hide();
            this.keybinding = this._keybinding; // force update
            // Activate on drag over to reveal targets
            [this.$badge, this.$e].forEach(function (b) { return new dnd_1.DelayedDragHandler(b.getHTMLElement(), function () {
                if (!_this.getAction().checked) {
                    _this.getAction().run();
                }
            }); });
        };
        ActivityActionItem.prototype.focus = function () {
            this.$e.domFocus();
        };
        ActivityActionItem.prototype.setBadge = function (badge) {
            this.updateBadge(badge);
        };
        Object.defineProperty(ActivityActionItem.prototype, "keybinding", {
            set: function (keybinding) {
                this._keybinding = keybinding;
                if (!this.$e) {
                    return;
                }
                var title;
                if (keybinding) {
                    title = nls.localize(0, null, this.name, keybinding);
                }
                else {
                    title = this.name;
                }
                this.$e.title(title);
                this.$badge.title(title);
            },
            enumerable: true,
            configurable: true
        });
        ActivityActionItem.prototype.updateBadge = function (badge) {
            this.$badgeContent.empty();
            this.$badge.hide();
            if (badge) {
                // Number
                if (badge instanceof activityService_1.NumberBadge) {
                    var n = badge.number;
                    if (n) {
                        this.$badgeContent.text(n > 99 ? '99+' : n.toString());
                        this.$badge.show();
                    }
                }
                else if (badge instanceof activityService_1.TextBadge) {
                    this.$badgeContent.text(badge.text);
                    this.$badge.show();
                }
                else if (badge instanceof activityService_1.IconBadge) {
                    this.$badge.show();
                }
                else if (badge instanceof activityService_1.ProgressBadge) {
                    this.$badge.show();
                }
                this.$e.attr('aria-label', this.name + ' - ' + badge.getDescription());
            }
        };
        ActivityActionItem.prototype._updateClass = function () {
            if (this.cssClass) {
                this.$badge.removeClass(this.cssClass);
            }
            this.cssClass = this.getAction().class;
            this.$badge.addClass(this.cssClass);
        };
        ActivityActionItem.prototype._updateChecked = function () {
            if (this.getAction().checked) {
                this.$e.addClass('active');
            }
            else {
                this.$e.removeClass('active');
            }
        };
        ActivityActionItem.prototype._updateUnknown = function (event) {
            if (event.getType() === ActivityAction.BADGE) {
                var action = this.getAction();
                if (action instanceof ActivityAction) {
                    this.updateBadge(action.getBadge());
                }
            }
        };
        ActivityActionItem.prototype._updateEnabled = function () {
            if (this.getAction().enabled) {
                this.builder.removeClass('disabled');
            }
            else {
                this.builder.addClass('disabled');
            }
        };
        ActivityActionItem.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            this.$badge.destroy();
            this.$e.destroy();
        };
        return ActivityActionItem;
    }(actionbar_1.BaseActionItem));
    exports.ActivityActionItem = ActivityActionItem;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/configuration/node/configurationService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/uri', 'vs/base/common/strings', 'vs/base/common/platform', 'vs/base/common/paths', 'vs/base/node/extfs', 'vs/base/common/objects', 'vs/platform/configuration/common/configurationService', 'vs/workbench/common/events', 'vs/platform/configuration/common/configuration', 'fs'], function (require, exports, winjs_base_1, uri_1, strings, platform, paths, extfs, objects, configurationService_1, events_1, configuration_1, fs) {
    'use strict';
    var ConfigurationService = (function (_super) {
        __extends(ConfigurationService, _super);
        function ConfigurationService(contextService, eventService) {
            _super.call(this, contextService, eventService);
            this.serviceId = configuration_1.IConfigurationService;
            this.registerListeners();
        }
        ConfigurationService.prototype.resolveContents = function (resources) {
            var _this = this;
            var contents = [];
            return winjs_base_1.TPromise.join(resources.map(function (resource) {
                return _this.resolveContent(resource).then(function (content) {
                    contents.push(content);
                });
            })).then(function () { return contents; });
        };
        ConfigurationService.prototype.resolveContent = function (resource) {
            return new winjs_base_1.TPromise(function (c, e) {
                fs.readFile(resource.fsPath, function (error, contents) {
                    if (error) {
                        e(error);
                    }
                    else {
                        c({
                            resource: resource,
                            value: contents.toString()
                        });
                    }
                });
            });
        };
        ConfigurationService.prototype.resolveStat = function (resource) {
            return new winjs_base_1.TPromise(function (c, e) {
                extfs.readdir(resource.fsPath, function (error, children) {
                    if (error) {
                        if (error.code === 'ENOTDIR') {
                            c({
                                resource: resource,
                                isDirectory: false
                            });
                        }
                        else {
                            e(error);
                        }
                    }
                    else {
                        c({
                            resource: resource,
                            isDirectory: true,
                            children: children.map(function (child) {
                                if (platform.isMacintosh) {
                                    child = strings.normalizeNFC(child); // Mac: uses NFD unicode form on disk, but we want NFC
                                }
                                return {
                                    resource: uri_1.default.file(paths.join(resource.fsPath, child))
                                };
                            })
                        });
                    }
                });
            });
        };
        ConfigurationService.prototype.registerListeners = function () {
            var _this = this;
            this.toDispose = this.eventService.addListener(events_1.EventType.WORKBENCH_OPTIONS_CHANGED, function (e) { return _this.onOptionsChanged(e); });
        };
        ConfigurationService.prototype.onOptionsChanged = function (e) {
            if (e.key === 'globalSettings') {
                this.reloadAndEmit();
            }
        };
        ConfigurationService.prototype.loadWorkspaceConfiguration = function (section) {
            // Return early if we don't have a workspace
            if (!this.contextService.getWorkspace()) {
                return winjs_base_1.TPromise.as({});
            }
            // Migrate as needed (.settings => .vscode)
            return _super.prototype.loadWorkspaceConfiguration.call(this, section);
        };
        ConfigurationService.prototype.loadGlobalConfiguration = function () {
            var _this = this;
            return _super.prototype.loadGlobalConfiguration.call(this).then(function (defaults) {
                var globalSettings = _this.contextService.getOptions().globalSettings;
                return {
                    contents: objects.mixin(objects.clone(defaults.contents), // target: default values (but don't modify!)
                    globalSettings.settings, // source: global configured values
                    true // overwrite
                    ),
                    parseErrors: globalSettings.settingsParseErrors
                };
            });
        };
        ConfigurationService.prototype.dispose = function () {
            this.toDispose();
        };
        return ConfigurationService;
    }(configurationService_1.ConfigurationService));
    exports.ConfigurationService = ConfigurationService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/contextview/electron-browser/contextmenuService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/severity', 'vs/base/browser/ui/actionbar/actionbar', 'vs/base/browser/dom', 'vs/base/browser/builder', 'vs/platform/contextview/browser/contextView', 'electron'], function (require, exports, winjs_base_1, severity_1, actionbar_1, dom, builder_1, contextView_1, electron_1) {
    'use strict';
    var ContextMenuService = (function () {
        function ContextMenuService(messageService, telemetryService, keybindingService) {
            this.serviceId = contextView_1.IContextMenuService;
            this.messageService = messageService;
            this.telemetryService = telemetryService;
            this.keybindingService = keybindingService;
        }
        ContextMenuService.prototype.showContextMenu = function (delegate) {
            var _this = this;
            delegate.getActions().then(function (actions) {
                if (!actions.length) {
                    return winjs_base_1.TPromise.as(null);
                }
                return winjs_base_1.TPromise.timeout(0).then(function () {
                    var menu = new electron_1.remote.Menu();
                    var actionToRun = null;
                    actions.forEach(function (a) {
                        if (a instanceof actionbar_1.Separator) {
                            menu.append(new electron_1.remote.MenuItem({ type: 'separator' }));
                        }
                        else {
                            var keybinding = !!delegate.getKeyBinding ? delegate.getKeyBinding(a) : undefined;
                            var accelerator = keybinding && _this.keybindingService.getElectronAcceleratorFor(keybinding);
                            var item = new electron_1.remote.MenuItem({
                                label: a.label,
                                checked: a.checked,
                                accelerator: accelerator,
                                enabled: a.enabled,
                                click: function () {
                                    actionToRun = a;
                                }
                            });
                            menu.append(item);
                        }
                    });
                    var anchor = delegate.getAnchor();
                    var x, y;
                    if (dom.isHTMLElement(anchor)) {
                        var $anchor = builder_1.$(anchor);
                        var elementPosition = $anchor.getPosition();
                        var elementSize = $anchor.getTotalSize();
                        x = elementPosition.left;
                        y = elementPosition.top + elementSize.height;
                    }
                    else {
                        var pos = anchor;
                        x = pos.x;
                        y = pos.y;
                    }
                    menu.popup(electron_1.remote.getCurrentWindow(), Math.floor(x), Math.floor(y));
                    if (delegate.onHide) {
                        delegate.onHide(false);
                    }
                    if (!actionToRun) {
                        return;
                    }
                    _this.telemetryService.publicLog('workbenchActionExecuted', { id: actionToRun.id, from: 'contextMenu' });
                    var context = delegate.getActionsContext ? delegate.getActionsContext() : null;
                    return actionToRun.run(context) || winjs_base_1.TPromise.as(null);
                });
            })
                .done(null, function (e) { return _this.messageService.show(severity_1.default.Error, e); });
        };
        return ContextMenuService;
    }());
    exports.ContextMenuService = ContextMenuService;
});

define("vs/workbench/services/editor/common/editorService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (EditorArrangement) {
        EditorArrangement[EditorArrangement["MINIMIZE_OTHERS"] = 0] = "MINIMIZE_OTHERS";
        EditorArrangement[EditorArrangement["EVEN_WIDTH"] = 1] = "EVEN_WIDTH";
    })(exports.EditorArrangement || (exports.EditorArrangement = {}));
    var EditorArrangement = exports.EditorArrangement;
    exports.IWorkbenchEditorService = instantiation_1.createDecorator('editorService');
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostOutputService", ["require", "exports", 'vs/base/common/errors', 'vs/platform/thread/common/thread', 'vs/workbench/parts/output/common/output', 'vs/workbench/services/editor/common/editorService'], function (require, exports, errors_1, thread_1, output_1, editorService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExtHostOutputChannel = (function () {
        function ExtHostOutputChannel(name, proxy) {
            this._name = name;
            this._proxy = proxy;
        }
        Object.defineProperty(ExtHostOutputChannel.prototype, "name", {
            get: function () {
                return this._name;
            },
            enumerable: true,
            configurable: true
        });
        ExtHostOutputChannel.prototype.dispose = function () {
            var _this = this;
            if (!this._disposed) {
                this._proxy.clear(this._name).then(function () {
                    _this._disposed = true;
                });
            }
        };
        ExtHostOutputChannel.prototype.append = function (value) {
            this._proxy.append(this._name, value);
        };
        ExtHostOutputChannel.prototype.appendLine = function (value) {
            this.append(value + '\n');
        };
        ExtHostOutputChannel.prototype.clear = function () {
            this._proxy.clear(this._name);
        };
        ExtHostOutputChannel.prototype.show = function (columnOrPreserveFocus, preserveFocus) {
            if (typeof columnOrPreserveFocus === 'boolean') {
                preserveFocus = columnOrPreserveFocus;
            }
            this._proxy.reveal(this._name, preserveFocus);
        };
        ExtHostOutputChannel.prototype.hide = function () {
            this._proxy.close(this._name);
        };
        return ExtHostOutputChannel;
    }());
    exports.ExtHostOutputChannel = ExtHostOutputChannel;
    var ExtHostOutputService = (function () {
        function ExtHostOutputService(threadService) {
            this._proxy = threadService.getRemotable(MainThreadOutputService);
        }
        ExtHostOutputService.prototype.createOutputChannel = function (name) {
            name = name.trim();
            if (!name) {
                throw new Error('illegal argument `name`. must not be falsy');
            }
            else {
                return new ExtHostOutputChannel(name, this._proxy);
            }
        };
        return ExtHostOutputService;
    }());
    exports.ExtHostOutputService = ExtHostOutputService;
    var MainThreadOutputService = (function () {
        function MainThreadOutputService(outputService, editorService) {
            this._outputService = outputService;
            this._editorService = editorService;
        }
        MainThreadOutputService.prototype.append = function (channel, value) {
            this._outputService.append(channel, value);
            return undefined;
        };
        MainThreadOutputService.prototype.clear = function (channel) {
            this._outputService.clearOutput(channel);
            return undefined;
        };
        MainThreadOutputService.prototype.reveal = function (channel, preserveFocus) {
            this._outputService.showOutput(channel, preserveFocus);
            return undefined;
        };
        MainThreadOutputService.prototype.close = function (channel) {
            var editors = this._editorService.getVisibleEditors();
            for (var _i = 0, editors_1 = editors; _i < editors_1.length; _i++) {
                var editor = editors_1[_i];
                if (editor.input.getId() === output_1.OUTPUT_EDITOR_INPUT_ID) {
                    this._editorService.closeEditor(editor).done(null, errors_1.onUnexpectedError);
                    return undefined;
                }
            }
        };
        MainThreadOutputService = __decorate([
            thread_1.Remotable.MainContext('MainThreadOutputService'),
            __param(0, output_1.IOutputService),
            __param(1, editorService_1.IWorkbenchEditorService)
        ], MainThreadOutputService);
        return MainThreadOutputService;
    }());
    exports.MainThreadOutputService = MainThreadOutputService;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostWorkspace", ["require", "exports", 'vs/base/common/errors', 'vs/platform/search/common/search', 'vs/platform/workspace/common/workspace', 'vs/platform/thread/common/thread', 'vs/platform/event/common/event', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/parts/files/common/files', 'vs/editor/common/services/bulkEdit', 'vs/base/common/winjs.base', 'vs/workbench/api/node/extHostTypeConverters'], function (require, exports, errors_1, search_1, workspace_1, thread_1, event_1, editorService_1, files_1, bulkEdit_1, winjs_base_1, extHostTypeConverters_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExtHostWorkspace = (function () {
        function ExtHostWorkspace(threadService, workspacePath) {
            this._proxy = threadService.getRemotable(MainThreadWorkspace);
            this._workspacePath = workspacePath;
        }
        ExtHostWorkspace.prototype.getPath = function () {
            return this._workspacePath;
        };
        ExtHostWorkspace.prototype.getRelativePath = function (pathOrUri) {
            var path;
            if (typeof pathOrUri === 'string') {
                path = pathOrUri;
            }
            else {
                path = pathOrUri.fsPath;
            }
            if (this._workspacePath && this._workspacePath.length < path.length) {
                // return relative(workspacePath, path);
                return path.substring(this._workspacePath.length);
            }
            return path;
        };
        ExtHostWorkspace.prototype.findFiles = function (include, exclude, maxResults, token) {
            var _this = this;
            var requestId = ExtHostWorkspace._requestIdPool++;
            var result = this._proxy.$startSearch(include, exclude, maxResults, requestId);
            if (token) {
                token.onCancellationRequested(function () { return _this._proxy.$cancelSearch(requestId); });
            }
            return result;
        };
        ExtHostWorkspace.prototype.saveAll = function (includeUntitled) {
            return this._proxy.$saveAll(includeUntitled);
        };
        ExtHostWorkspace.prototype.appyEdit = function (edit) {
            var resourceEdits = [];
            var entries = edit.entries();
            for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
                var entry = entries_1[_i];
                var uri = entry[0], edits = entry[1];
                for (var _a = 0, edits_1 = edits; _a < edits_1.length; _a++) {
                    var edit_1 = edits_1[_a];
                    resourceEdits.push({
                        resource: uri,
                        newText: edit_1.newText,
                        range: extHostTypeConverters_1.fromRange(edit_1.range)
                    });
                }
            }
            return this._proxy.$applyWorkspaceEdit(resourceEdits);
        };
        ExtHostWorkspace._requestIdPool = 0;
        ExtHostWorkspace = __decorate([
            __param(0, thread_1.IThreadService)
        ], ExtHostWorkspace);
        return ExtHostWorkspace;
    }());
    exports.ExtHostWorkspace = ExtHostWorkspace;
    var MainThreadWorkspace = (function () {
        function MainThreadWorkspace(searchService, contextService, textFileService, editorService, eventService) {
            this._activeSearches = Object.create(null);
            this._searchService = searchService;
            this._workspace = contextService.getWorkspace();
            this._textFileService = textFileService;
            this._editorService = editorService;
            this._eventService = eventService;
        }
        MainThreadWorkspace.prototype.$startSearch = function (include, exclude, maxResults, requestId) {
            var _this = this;
            if (!this._workspace) {
                return;
            }
            var search = this._searchService.search({
                folderResources: [this._workspace.resource],
                type: search_1.QueryType.File,
                maxResults: maxResults,
                includePattern: (_a = {}, _a[include] = true, _a),
                excludePattern: (_b = {}, _b[exclude] = true, _b),
            }).then(function (result) {
                return result.results.map(function (m) { return m.resource; });
            }, function (err) {
                if (!errors_1.isPromiseCanceledError(err)) {
                    return winjs_base_1.TPromise.wrapError(err);
                }
            });
            this._activeSearches[requestId] = search;
            var onDone = function () { return delete _this._activeSearches[requestId]; };
            search.done(onDone, onDone);
            return search;
            var _a, _b;
        };
        MainThreadWorkspace.prototype.$cancelSearch = function (requestId) {
            var search = this._activeSearches[requestId];
            if (search) {
                delete this._activeSearches[requestId];
                search.cancel();
                return winjs_base_1.TPromise.as(true);
            }
        };
        MainThreadWorkspace.prototype.$saveAll = function (includeUntitled) {
            return this._textFileService.saveAll(includeUntitled).then(function (result) {
                return result.results.every(function (each) { return each.success === true; });
            });
        };
        MainThreadWorkspace.prototype.$applyWorkspaceEdit = function (edits) {
            var codeEditor;
            var editor = this._editorService.getActiveEditor();
            if (editor) {
                var candidate = editor.getControl();
                if (typeof candidate.getEditorType === 'function') {
                    // enough proof
                    codeEditor = candidate;
                }
            }
            return bulkEdit_1.bulkEdit(this._eventService, this._editorService, codeEditor, edits)
                .then(function () { return true; });
        };
        MainThreadWorkspace = __decorate([
            thread_1.Remotable.MainContext('MainThreadWorkspace'),
            __param(0, search_1.ISearchService),
            __param(1, workspace_1.IWorkspaceContextService),
            __param(2, files_1.ITextFileService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, event_1.IEventService)
        ], MainThreadWorkspace);
        return MainThreadWorkspace;
    }());
    exports.MainThreadWorkspace = MainThreadWorkspace;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/editor/binaryDiffEditor", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/browser/parts/editor/binaryDiffEditor', 'vs/base/browser/ui/sash/sash', 'vs/base/browser/builder', 'vs/base/browser/ui/resourceviewer/resourceViewer', 'vs/base/browser/ui/scrollbar/scrollableElementImpl', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor/binaryEditorModel', 'vs/workbench/services/editor/common/editorService', 'vs/platform/telemetry/common/telemetry', 'vs/css!./media/binarydiffeditor'], function (require, exports, winjs_base_1, nls, sash_1, builder_1, resourceViewer_1, scrollableElementImpl_1, baseEditor_1, binaryEditorModel_1, editorService_1, telemetry_1) {
    'use strict';
    /**
     * An implementation of editor for diffing binary files like images or videos.
     */
    var BinaryResourceDiffEditor = (function (_super) {
        __extends(BinaryResourceDiffEditor, _super);
        function BinaryResourceDiffEditor(telemetryService, editorService) {
            _super.call(this, BinaryResourceDiffEditor.ID, telemetryService);
            this.editorService = editorService;
        }
        BinaryResourceDiffEditor.prototype.getTitle = function () {
            return this.getInput() ? this.getInput().getName() : nls.localize(0, null);
        };
        BinaryResourceDiffEditor.prototype.createEditor = function (parent) {
            var _this = this;
            // Left Container for Binary
            var leftBinaryContainerElement = document.createElement('div');
            leftBinaryContainerElement.className = 'binary-container';
            this.leftBinaryContainer = builder_1.$(leftBinaryContainerElement);
            this.leftBinaryContainer.tabindex(0); // enable focus support from the editor part (do not remove)
            // Left Custom Scrollbars
            this.leftScrollbar = new scrollableElementImpl_1.ScrollableElement(leftBinaryContainerElement, { horizontal: 'hidden', vertical: 'hidden' });
            parent.getHTMLElement().appendChild(this.leftScrollbar.getDomNode());
            builder_1.$(this.leftScrollbar.getDomNode()).addClass('binarydiff-left');
            // Sash
            this.sash = new sash_1.Sash(parent.getHTMLElement(), this);
            this.sash.addListener('start', function () { return _this.onSashDragStart(); });
            this.sash.addListener('change', function (e) { return _this.onSashDrag(e); });
            this.sash.addListener('end', function () { return _this.onSashDragEnd(); });
            // Right Container for Binary
            var rightBinaryContainerElement = document.createElement('div');
            rightBinaryContainerElement.className = 'binary-container';
            this.rightBinaryContainer = builder_1.$(rightBinaryContainerElement);
            this.rightBinaryContainer.tabindex(0); // enable focus support from the editor part (do not remove)
            // Right Custom Scrollbars
            this.rightScrollbar = new scrollableElementImpl_1.ScrollableElement(rightBinaryContainerElement, { horizontal: 'hidden', vertical: 'hidden' });
            parent.getHTMLElement().appendChild(this.rightScrollbar.getDomNode());
            builder_1.$(this.rightScrollbar.getDomNode()).addClass('binarydiff-right');
        };
        BinaryResourceDiffEditor.prototype.setInput = function (input, options) {
            var _this = this;
            var oldInput = this.getInput();
            _super.prototype.setInput.call(this, input, options);
            // Detect options
            var forceOpen = options && options.forceOpen;
            // Same Input
            if (!forceOpen && input.matches(oldInput)) {
                return winjs_base_1.TPromise.as(null);
            }
            // Different Input (Reload)
            return this.editorService.resolveEditorModel(input, true /* Reload */).then(function (resolvedModel) {
                // Assert model instance
                if (!(resolvedModel.originalModel instanceof binaryEditorModel_1.BinaryEditorModel) || !(resolvedModel.modifiedModel instanceof binaryEditorModel_1.BinaryEditorModel)) {
                    return winjs_base_1.TPromise.wrapError(nls.localize(1, null));
                }
                // Assert that the current input is still the one we expect. This prevents a race condition when loading a diff takes long and another input was set meanwhile
                if (!_this.getInput() || _this.getInput() !== input) {
                    return null;
                }
                // Render original
                var original = resolvedModel.originalModel;
                _this.renderInput(original.getName(), original.getResource(), true);
                // Render modified
                var modified = resolvedModel.modifiedModel;
                _this.renderInput(modified.getName(), modified.getResource(), false);
            });
        };
        BinaryResourceDiffEditor.prototype.renderInput = function (name, resource, isOriginal) {
            // Reset Sash to default 50/50 ratio if needed
            if (this.leftContainerWidth && this.dimension && this.leftContainerWidth !== this.dimension.width / 2) {
                this.leftContainerWidth = this.dimension.width / 2;
                this.layoutContainers();
                this.sash.layout();
            }
            // Pass to ResourceViewer
            var container = isOriginal ? this.leftBinaryContainer : this.rightBinaryContainer;
            var scrollbar = isOriginal ? this.leftScrollbar : this.rightScrollbar;
            resourceViewer_1.ResourceViewer.show(name, resource, container, scrollbar);
        };
        BinaryResourceDiffEditor.prototype.clearInput = function () {
            // Empty HTML Container
            builder_1.$(this.leftBinaryContainer).empty();
            builder_1.$(this.rightBinaryContainer).empty();
            _super.prototype.clearInput.call(this);
        };
        BinaryResourceDiffEditor.prototype.layout = function (dimension) {
            var oldDimension = this.dimension;
            this.dimension = dimension;
            // Calculate left hand container width based on sash move or fallback to 50% by default
            if (!this.leftContainerWidth || !oldDimension) {
                this.leftContainerWidth = this.dimension.width / 2;
            }
            else {
                var sashRatio = this.leftContainerWidth / oldDimension.width;
                this.leftContainerWidth = this.dimension.width * sashRatio;
            }
            // Sash positioning
            this.sash.layout();
            // Pass on to Binary Containers and Scrollbars
            this.layoutContainers();
        };
        BinaryResourceDiffEditor.prototype.layoutContainers = function () {
            // Size left container
            this.leftBinaryContainer.size(this.leftContainerWidth, this.dimension.height);
            this.leftScrollbar.onElementDimensions();
            this.leftScrollbar.onElementInternalDimensions();
            // Size right container
            this.rightBinaryContainer.size(this.dimension.width - this.leftContainerWidth, this.dimension.height);
            this.rightScrollbar.onElementDimensions();
            this.rightScrollbar.onElementInternalDimensions();
        };
        BinaryResourceDiffEditor.prototype.onSashDragStart = function () {
            this.startLeftContainerWidth = this.leftContainerWidth;
        };
        BinaryResourceDiffEditor.prototype.onSashDrag = function (e) {
            // Update Widths and keep in bounds of MIN_CONTAINER_WIDTH for both sides
            var newLeftContainerWidth = this.startLeftContainerWidth + e.currentX - e.startX;
            this.leftContainerWidth = Math.max(BinaryResourceDiffEditor.MIN_CONTAINER_WIDTH, newLeftContainerWidth);
            if (this.dimension.width - this.leftContainerWidth < BinaryResourceDiffEditor.MIN_CONTAINER_WIDTH) {
                this.leftContainerWidth = this.dimension.width - BinaryResourceDiffEditor.MIN_CONTAINER_WIDTH;
            }
            // Pass on to Binary Containers and Scrollbars
            this.layoutContainers();
        };
        BinaryResourceDiffEditor.prototype.onSashDragEnd = function () {
            this.sash.layout();
        };
        BinaryResourceDiffEditor.prototype.getVerticalSashTop = function (sash) {
            return 0;
        };
        BinaryResourceDiffEditor.prototype.getVerticalSashLeft = function (sash) {
            return this.leftContainerWidth;
        };
        BinaryResourceDiffEditor.prototype.getVerticalSashHeight = function (sash) {
            return this.dimension.height;
        };
        BinaryResourceDiffEditor.prototype.focus = function () {
            this.rightBinaryContainer.domFocus();
        };
        BinaryResourceDiffEditor.prototype.dispose = function () {
            // Sash
            this.sash.dispose();
            // Dispose Scrollbar
            this.leftScrollbar.dispose();
            this.rightScrollbar.dispose();
            // Destroy Container
            this.leftBinaryContainer.destroy();
            this.rightBinaryContainer.destroy();
            _super.prototype.dispose.call(this);
        };
        BinaryResourceDiffEditor.ID = 'workbench.editors.binaryResourceDiffEditor';
        BinaryResourceDiffEditor.MIN_CONTAINER_WIDTH = 100;
        BinaryResourceDiffEditor = __decorate([
            __param(0, telemetry_1.ITelemetryService),
            __param(1, editorService_1.IWorkbenchEditorService)
        ], BinaryResourceDiffEditor);
        return BinaryResourceDiffEditor;
    }(baseEditor_1.BaseEditor));
    exports.BinaryResourceDiffEditor = BinaryResourceDiffEditor;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/editor/binaryEditor", ["require", "exports", 'vs/nls!vs/workbench/browser/parts/editor/binaryEditor', 'vs/base/common/winjs.base', 'vs/base/browser/builder', 'vs/base/browser/ui/resourceviewer/resourceViewer', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor/binaryEditorModel', 'vs/workbench/services/editor/common/editorService', 'vs/platform/telemetry/common/telemetry', 'vs/css!./media/binaryeditor'], function (require, exports, nls, winjs_base_1, builder_1, resourceViewer_1, baseEditor_1, binaryEditorModel_1, editorService_1, telemetry_1) {
    'use strict';
    /*
     * This class is only intended to be subclassed and not instantiated.
     */
    var BaseBinaryResourceEditor = (function (_super) {
        __extends(BaseBinaryResourceEditor, _super);
        function BaseBinaryResourceEditor(id, telemetryService, _editorService) {
            _super.call(this, id, telemetryService);
            this._editorService = _editorService;
        }
        BaseBinaryResourceEditor.prototype.getTitle = function () {
            return this.getInput() ? this.getInput().getName() : nls.localize(0, null);
        };
        Object.defineProperty(BaseBinaryResourceEditor.prototype, "editorService", {
            get: function () {
                return this._editorService;
            },
            enumerable: true,
            configurable: true
        });
        BaseBinaryResourceEditor.prototype.createEditor = function (parent) {
            // Container for Binary
            var binaryContainerElement = document.createElement('div');
            binaryContainerElement.className = 'binary-container monaco-editor-background'; // Inherit the background color from selected theme'
            this.binaryContainer = builder_1.$(binaryContainerElement);
            this.binaryContainer.tabindex(0); // enable focus support from the editor part (do not remove)
            parent.getHTMLElement().appendChild(this.binaryContainer.getHTMLElement());
        };
        BaseBinaryResourceEditor.prototype.setInput = function (input, options) {
            var _this = this;
            var oldInput = this.getInput();
            _super.prototype.setInput.call(this, input, options);
            // Detect options
            var forceOpen = options && options.forceOpen;
            // Same Input
            if (!forceOpen && input.matches(oldInput)) {
                return winjs_base_1.TPromise.as(null);
            }
            // Different Input (Reload)
            return this._editorService.resolveEditorModel(input, true /* Reload */).then(function (resolvedModel) {
                // Assert Model instance
                if (!(resolvedModel instanceof binaryEditorModel_1.BinaryEditorModel)) {
                    return winjs_base_1.TPromise.wrapError('Invalid editor input. Binary resource editor requires a model instance of BinaryEditorModel.');
                }
                // Assert that the current input is still the one we expect. This prevents a race condition when loading takes long and another input was set meanwhile
                if (!_this.getInput() || _this.getInput() !== input) {
                    return null;
                }
                // Render Input
                var binaryResourceModel = resolvedModel;
                resourceViewer_1.ResourceViewer.show(binaryResourceModel.getName(), binaryResourceModel.getResource(), _this.binaryContainer);
                return winjs_base_1.TPromise.as(null);
            });
        };
        BaseBinaryResourceEditor.prototype.clearInput = function () {
            // Empty HTML Container
            builder_1.$(this.binaryContainer).empty();
            _super.prototype.clearInput.call(this);
        };
        BaseBinaryResourceEditor.prototype.layout = function (dimension) {
            // Pass on to Binary Container
            this.binaryContainer.size(dimension.width, dimension.height);
        };
        BaseBinaryResourceEditor.prototype.focus = function () {
            this.binaryContainer.domFocus();
        };
        BaseBinaryResourceEditor.prototype.dispose = function () {
            // Destroy Container
            this.binaryContainer.destroy();
            _super.prototype.dispose.call(this);
        };
        return BaseBinaryResourceEditor;
    }(baseEditor_1.BaseEditor));
    exports.BaseBinaryResourceEditor = BaseBinaryResourceEditor;
    /**
     * An implementation of editor for binary files like images or videos.
     */
    var BinaryResourceEditor = (function (_super) {
        __extends(BinaryResourceEditor, _super);
        function BinaryResourceEditor(telemetryService, editorService) {
            _super.call(this, BinaryResourceEditor.ID, telemetryService, editorService);
        }
        BinaryResourceEditor.ID = 'workbench.editors.binaryResourceEditor';
        BinaryResourceEditor = __decorate([
            __param(0, telemetry_1.ITelemetryService),
            __param(1, editorService_1.IWorkbenchEditorService)
        ], BinaryResourceEditor);
        return BinaryResourceEditor;
    }(BaseBinaryResourceEditor));
    exports.BinaryResourceEditor = BinaryResourceEditor;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/editor/iframeEditor", ["require", "exports", 'vs/nls!vs/workbench/browser/parts/editor/iframeEditor', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/base/browser/builder', 'vs/base/common/errors', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor/iframeEditorInput', 'vs/workbench/common/editor/iframeEditorModel', 'vs/platform/storage/common/storage', 'vs/platform/telemetry/common/telemetry', 'vs/workbench/services/editor/common/editorService', 'vs/css!./media/iframeeditor'], function (require, exports, nls, winjs_base_1, DOM, builder_1, errors, baseEditor_1, iframeEditorInput_1, iframeEditorModel_1, storage_1, telemetry_1, editorService_1) {
    'use strict';
    /**
     * An implementation of editor for showing HTML content in an IFrame by leveraging the IFrameEditorInput.
     */
    var IFrameEditor = (function (_super) {
        __extends(IFrameEditor, _super);
        function IFrameEditor(telemetryService, editorService, storageService) {
            _super.call(this, IFrameEditor.ID, telemetryService);
            this.editorService = editorService;
            this.storageService = storageService;
        }
        IFrameEditor.prototype.getTitle = function () {
            return this.getInput() ? this.getInput().getName() : nls.localize(0, null);
        };
        IFrameEditor.prototype.createEditor = function (parent) {
            // Container for IFrame
            var iframeContainerElement = document.createElement('div');
            iframeContainerElement.className = 'iframe-container monaco-editor-background'; // Inherit the background color from selected theme
            this.iframeContainer = builder_1.$(iframeContainerElement);
            this.iframeContainer.tabindex(0); // enable focus support from the editor part (do not remove)
            // IFrame
            this.iframeBuilder = builder_1.$(this.iframeContainer).element('iframe').addClass('iframe');
            this.iframeBuilder.attr({ 'frameborder': '0' });
            this.iframeBuilder.removeProperty(IFrameEditor.RESOURCE_PROPERTY);
            parent.getHTMLElement().appendChild(iframeContainerElement);
        };
        IFrameEditor.prototype.setInput = function (input, options) {
            var oldInput = this.getInput();
            _super.prototype.setInput.call(this, input, options);
            // Detect options
            var forceOpen = options && options.forceOpen;
            // Same Input
            if (!forceOpen && input.matches(oldInput)) {
                return winjs_base_1.TPromise.as(null);
            }
            // Assert Input
            if (!(input instanceof iframeEditorInput_1.IFrameEditorInput)) {
                return winjs_base_1.TPromise.wrapError('Invalid editor input. IFrame editor requires an input instance of IFrameEditorInput.');
            }
            // Different Input (Reload)
            return this.doSetInput(input, true /* isNewInput */);
        };
        IFrameEditor.prototype.doSetInput = function (input, isNewInput) {
            var _this = this;
            return this.editorService.resolveEditorModel(input, true /* Reload */).then(function (resolvedModel) {
                // Assert Model interface
                if (!(resolvedModel instanceof iframeEditorModel_1.IFrameEditorModel)) {
                    return winjs_base_1.TPromise.wrapError('Invalid editor input. IFrame editor requires a model instance of IFrameEditorModel.');
                }
                // Assert that the current input is still the one we expect. This prevents a race condition when loading takes long and another input was set meanwhile
                if (!_this.getInput() || _this.getInput() !== input) {
                    return null;
                }
                // Set IFrame contents
                var iframeModel = resolvedModel;
                var isUpdate = !isNewInput && !!_this.iframeBuilder.getProperty(IFrameEditor.RESOURCE_PROPERTY);
                var contents = iframeModel.getContents();
                // Crazy hack to get keybindings to bubble out of the iframe to us
                contents.body = contents.body + _this.enableKeybindings();
                // Set Contents
                try {
                    _this.setFrameContents(iframeModel.resource, isUpdate ? contents.body : [contents.head, contents.body, contents.tail].join('\n'), isUpdate /* body only */);
                }
                catch (error) {
                    setTimeout(function () { return _this.reload(true /* clear */); }, 1000); // retry in case of an error which indicates the iframe (only) might be on a different URL
                }
                // When content is fully replaced, we also need to recreate the focus tracker
                if (!isUpdate) {
                    _this.clearFocusTracker();
                }
                // Track focus on contents and make the editor active when focus is received
                if (!_this.focusTracker) {
                    _this.focusTracker = DOM.trackFocus(_this.iframeBuilder.getHTMLElement().contentWindow);
                    _this.focusTracker.addFocusListener(function () {
                        _this.editorService.activateEditor(_this.position);
                    });
                }
            });
        };
        IFrameEditor.prototype.setFrameContents = function (resource, contents, isUpdate) {
            var iframeWindow = this.iframeBuilder.getHTMLElement().contentWindow;
            // Update body only if this is an update of the same resource (preserves scroll position and does not flicker)
            if (isUpdate) {
                iframeWindow.document.body.innerHTML = contents;
            }
            else {
                iframeWindow.document.open('text/html', 'replace');
                iframeWindow.document.write(contents);
                iframeWindow.document.close();
                // Reset scroll
                iframeWindow.scrollTo(0, 0);
                // Associate resource with iframe
                this.iframeBuilder.setProperty(IFrameEditor.RESOURCE_PROPERTY, resource.toString());
            }
        };
        IFrameEditor.prototype.enableKeybindings = function () {
            return [
                '<script>',
                'var ignoredKeys = [9 /* tab */, 32 /* space */, 33 /* page up */, 34 /* page down */, 38 /* up */, 40 /* down */];',
                'var ignoredCtrlCmdKeys = [65 /* a */, 67 /* c */];',
                'var ignoredShiftKeys = [9 /* tab */];',
                'window.document.body.addEventListener("keydown", function(event) {',
                '	try {',
                '		if (ignoredKeys.some(function(i) { return i === event.keyCode; })) {',
                '			if (!event.ctrlKey && !event.altKey && !event.shiftKey && !event.metaKey) {',
                '				return;',
                '			}',
                '		}',
                '',
                '		if (ignoredCtrlCmdKeys.some(function(i) { return i === event.keyCode; })) {',
                '			if (event.ctrlKey || event.metaKey) {',
                '				return;',
                '			}',
                '		}',
                '',
                '		if (ignoredShiftKeys.some(function(i) { return i === event.keyCode; })) {',
                '			if (event.shiftKey) {',
                '				return;',
                '			}',
                '		}',
                '',
                '		event.preventDefault();',
                '',
                '		var fakeEvent = document.createEvent("KeyboardEvent");',
                '		Object.defineProperty(fakeEvent, "keyCode", {',
                '			get : function() {',
                '				return event.keyCode;',
                '			}',
                '		});',
                '		Object.defineProperty(fakeEvent, "which", {',
                '			get : function() {',
                '				return event.keyCode;',
                '			}',
                '		});',
                '		Object.defineProperty(fakeEvent, "target", {',
                '			get : function() {',
                '				return window && window.parent.document.body;',
                '			}',
                '		});',
                '',
                '		fakeEvent.initKeyboardEvent("keydown", true, true, document.defaultView, null, null, event.ctrlKey, event.altKey, event.shiftKey, event.metaKey);',
                '',
                '		window.parent.document.dispatchEvent(fakeEvent);',
                '	} catch (error) {}',
                '});',
                // disable dropping into iframe!
                'window.document.addEventListener("dragover", function (e) {',
                '	e.preventDefault();',
                '});',
                'window.document.addEventListener("drop", function (e) {',
                '	e.preventDefault();',
                '});',
                'window.document.body.addEventListener("dragover", function (e) {',
                '	e.preventDefault();',
                '});',
                'window.document.body.addEventListener("drop", function (e) {',
                '	e.preventDefault();',
                '});',
                '</script>'
            ].join('\n');
        };
        IFrameEditor.prototype.clearInput = function () {
            // Reset IFrame
            this.clearIFrame();
            _super.prototype.clearInput.call(this);
        };
        IFrameEditor.prototype.clearIFrame = function () {
            this.iframeBuilder.src('about:blank');
            this.iframeBuilder.removeProperty(IFrameEditor.RESOURCE_PROPERTY);
            // Focus Listener
            this.clearFocusTracker();
        };
        IFrameEditor.prototype.clearFocusTracker = function () {
            if (this.focusTracker) {
                this.focusTracker.dispose();
                this.focusTracker = null;
            }
        };
        IFrameEditor.prototype.layout = function (dimension) {
            // Pass on to IFrame Container and IFrame
            this.iframeContainer.size(dimension.width, dimension.height);
            this.iframeBuilder.size(dimension.width, dimension.height);
        };
        IFrameEditor.prototype.focus = function () {
            this.iframeContainer.domFocus();
        };
        IFrameEditor.prototype.changePosition = function (position) {
            var _this = this;
            _super.prototype.changePosition.call(this, position);
            // reparenting an IFRAME into another DOM element yields weird results when the contents are made
            // of a string and not a URL. to be on the safe side we reload the iframe when the position changes
            // and we do it using a timeout of 0 to reload only after the position has been changed in the DOM
            setTimeout(function () { return _this.reload(true); });
        };
        IFrameEditor.prototype.supportsSplitEditor = function () {
            return true;
        };
        /**
         * Reloads the contents of the iframe in this editor by reapplying the input.
         */
        IFrameEditor.prototype.reload = function (clearIFrame) {
            if (this.input) {
                if (clearIFrame) {
                    this.clearIFrame();
                }
                this.doSetInput(this.input).done(null, errors.onUnexpectedError);
            }
        };
        IFrameEditor.prototype.dispose = function () {
            // Destroy Container
            this.iframeContainer.destroy();
            // Focus Listener
            this.clearFocusTracker();
            _super.prototype.dispose.call(this);
        };
        IFrameEditor.ID = 'workbench.editors.iFrameEditor';
        IFrameEditor.RESOURCE_PROPERTY = 'resource';
        IFrameEditor = __decorate([
            __param(0, telemetry_1.ITelemetryService),
            __param(1, editorService_1.IWorkbenchEditorService),
            __param(2, storage_1.IStorageService)
        ], IFrameEditor);
        return IFrameEditor;
    }(baseEditor_1.BaseEditor));
    exports.IFrameEditor = IFrameEditor;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/editors/binaryFileEditor", ["require", "exports", 'vs/nls!vs/workbench/parts/files/browser/editors/binaryFileEditor', 'vs/workbench/browser/parts/editor/binaryEditor', 'vs/workbench/parts/files/common/files', 'vs/platform/telemetry/common/telemetry', 'vs/workbench/services/editor/common/editorService'], function (require, exports, nls, binaryEditor_1, files_1, telemetry_1, editorService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * An implementation of editor for binary files like images or videos leveraging the FileEditorInput.
     */
    var BinaryFileEditor = (function (_super) {
        __extends(BinaryFileEditor, _super);
        function BinaryFileEditor(telemetryService, editorService) {
            _super.call(this, BinaryFileEditor.ID, telemetryService, editorService);
        }
        BinaryFileEditor.prototype.getTitle = function () {
            return this.getInput() ? this.getInput().getName() : nls.localize(0, null);
        };
        BinaryFileEditor.prototype.supportsSplitEditor = function () {
            return true; // yes, we can!
        };
        BinaryFileEditor.ID = files_1.BINARY_FILE_EDITOR_ID;
        BinaryFileEditor = __decorate([
            __param(0, telemetry_1.ITelemetryService),
            __param(1, editorService_1.IWorkbenchEditorService)
        ], BinaryFileEditor);
        return BinaryFileEditor;
    }(binaryEditor_1.BaseBinaryResourceEditor));
    exports.BinaryFileEditor = BinaryFileEditor;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/electron-browser/electronFileActions", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/actions', 'vs/nls!vs/workbench/parts/files/electron-browser/electronFileActions', 'vs/base/common/paths', 'vs/base/common/labels', 'vs/base/common/platform', 'vs/base/common/severity', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/common/editor', 'vs/platform/message/common/message', 'electron'], function (require, exports, winjs_base_1, actions_1, nls, paths, labels, platform, severity_1, editorService_1, editor_1, message_1, electron_1) {
    'use strict';
    var RevealInOSAction = (function (_super) {
        __extends(RevealInOSAction, _super);
        function RevealInOSAction(resource) {
            _super.call(this, 'workbench.action.files.revealInWindows', platform.isWindows ? nls.localize(0, null) : (platform.isMacintosh ? nls.localize(1, null) : nls.localize(2, null)));
            this.resource = resource;
            this.order = 45;
        }
        RevealInOSAction.prototype.run = function () {
            electron_1.shell.showItemInFolder(paths.normalize(this.resource.fsPath, true));
            return winjs_base_1.TPromise.as(true);
        };
        return RevealInOSAction;
    }(actions_1.Action));
    exports.RevealInOSAction = RevealInOSAction;
    var GlobalRevealInOSAction = (function (_super) {
        __extends(GlobalRevealInOSAction, _super);
        function GlobalRevealInOSAction(id, label, editorService, messageService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.messageService = messageService;
        }
        GlobalRevealInOSAction.prototype.run = function () {
            var fileInput = editor_1.asFileEditorInput(this.editorService.getActiveEditorInput(), true);
            if (fileInput) {
                electron_1.shell.showItemInFolder(paths.normalize(fileInput.getResource().fsPath, true));
            }
            else {
                this.messageService.show(severity_1.default.Info, nls.localize(6, null));
            }
            return winjs_base_1.TPromise.as(true);
        };
        GlobalRevealInOSAction.ID = 'workbench.action.files.revealActiveFileInWindows';
        GlobalRevealInOSAction.LABEL = platform.isWindows ? nls.localize(3, null) : (platform.isMacintosh ? nls.localize(4, null) : nls.localize(5, null));
        GlobalRevealInOSAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, message_1.IMessageService)
        ], GlobalRevealInOSAction);
        return GlobalRevealInOSAction;
    }(actions_1.Action));
    exports.GlobalRevealInOSAction = GlobalRevealInOSAction;
    var CopyPathAction = (function (_super) {
        __extends(CopyPathAction, _super);
        function CopyPathAction(resource) {
            _super.call(this, 'workbench.action.files.copyPath', nls.localize(7, null));
            this.resource = resource;
            this.order = 140;
        }
        CopyPathAction.prototype.run = function () {
            electron_1.clipboard.writeText(labels.getPathLabel(this.resource));
            return winjs_base_1.TPromise.as(true);
        };
        return CopyPathAction;
    }(actions_1.Action));
    exports.CopyPathAction = CopyPathAction;
    var GlobalCopyPathAction = (function (_super) {
        __extends(GlobalCopyPathAction, _super);
        function GlobalCopyPathAction(id, label, editorService, messageService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.messageService = messageService;
        }
        GlobalCopyPathAction.prototype.run = function () {
            var fileInput = editor_1.asFileEditorInput(this.editorService.getActiveEditorInput(), true);
            if (fileInput) {
                electron_1.clipboard.writeText(labels.getPathLabel(fileInput.getResource()));
                this.editorService.focusEditor(); // focus back to editor
            }
            else {
                this.messageService.show(severity_1.default.Info, nls.localize(9, null));
            }
            return winjs_base_1.TPromise.as(true);
        };
        GlobalCopyPathAction.ID = 'workbench.action.files.copyPathOfActiveFile';
        GlobalCopyPathAction.LABEL = nls.localize(8, null);
        GlobalCopyPathAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, message_1.IMessageService)
        ], GlobalCopyPathAction);
        return GlobalCopyPathAction;
    }(actions_1.Action));
    exports.GlobalCopyPathAction = GlobalCopyPathAction;
    var BaseOpenAction = (function (_super) {
        __extends(BaseOpenAction, _super);
        function BaseOpenAction(id, label, ipcMsg) {
            _super.call(this, id, label);
            this.ipcMsg = ipcMsg;
        }
        BaseOpenAction.prototype.run = function () {
            electron_1.ipcRenderer.send(this.ipcMsg); // Handle in browser process
            return winjs_base_1.TPromise.as(true);
        };
        return BaseOpenAction;
    }(actions_1.Action));
    exports.BaseOpenAction = BaseOpenAction;
    exports.OPEN_FILE_ID = 'workbench.action.files.openFile';
    exports.OPEN_FILE_LABEL = nls.localize(10, null);
    var OpenFileAction = (function (_super) {
        __extends(OpenFileAction, _super);
        function OpenFileAction(id, label) {
            _super.call(this, id, label, 'vscode:openFilePicker');
        }
        return OpenFileAction;
    }(BaseOpenAction));
    exports.OpenFileAction = OpenFileAction;
    exports.OPEN_FOLDER_ID = 'workbench.action.files.openFolder';
    exports.OPEN_FOLDER_LABEL = nls.localize(11, null);
    var OpenFolderAction = (function (_super) {
        __extends(OpenFolderAction, _super);
        function OpenFolderAction(id, label) {
            _super.call(this, id, label, 'vscode:openFolderPicker');
        }
        return OpenFolderAction;
    }(BaseOpenAction));
    exports.OpenFolderAction = OpenFolderAction;
    exports.OPEN_FILE_FOLDER_ID = 'workbench.action.files.openFileFolder';
    exports.OPEN_FILE_FOLDER_LABEL = nls.localize(12, null);
    var OpenFileFolderAction = (function (_super) {
        __extends(OpenFileFolderAction, _super);
        function OpenFileFolderAction(id, label) {
            _super.call(this, id, label, 'vscode:openFileFolderPicker');
        }
        return OpenFileFolderAction;
    }(BaseOpenAction));
    exports.OpenFileFolderAction = OpenFileFolderAction;
    var ShowOpenedFileInNewWindow = (function (_super) {
        __extends(ShowOpenedFileInNewWindow, _super);
        function ShowOpenedFileInNewWindow(id, label, editorService, messageService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.messageService = messageService;
        }
        ShowOpenedFileInNewWindow.prototype.run = function () {
            var fileInput = editor_1.asFileEditorInput(this.editorService.getActiveEditorInput(), true);
            if (fileInput) {
                electron_1.ipcRenderer.send('vscode:windowOpen', [fileInput.getResource().fsPath], true /* force new window */); // handled from browser process
            }
            else {
                this.messageService.show(severity_1.default.Info, nls.localize(14, null));
            }
            return winjs_base_1.TPromise.as(true);
        };
        ShowOpenedFileInNewWindow.ID = 'workbench.action.files.showOpenedFileInNewWindow';
        ShowOpenedFileInNewWindow.LABEL = nls.localize(13, null);
        ShowOpenedFileInNewWindow = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, message_1.IMessageService)
        ], ShowOpenedFileInNewWindow);
        return ShowOpenedFileInNewWindow;
    }(actions_1.Action));
    exports.ShowOpenedFileInNewWindow = ShowOpenedFileInNewWindow;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/git/browser/gitEditorInputs", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/common/async', 'vs/workbench/common/editor/stringEditorInput', 'vs/workbench/common/editor/diffEditorInput', 'vs/workbench/parts/git/common/git', 'vs/workbench/services/editor/common/editorService', 'vs/platform/instantiation/common/instantiation'], function (require, exports, lifecycle, async, stringei, diffei, git, editorService_1, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var IGitService = git.IGitService;
    function isGitEditorInput(input) {
        return input instanceof GitDiffEditorInput || input instanceof NativeGitIndexStringEditorInput;
    }
    exports.isGitEditorInput = isGitEditorInput;
    var GitDiffEditorInput = (function (_super) {
        __extends(GitDiffEditorInput, _super);
        function GitDiffEditorInput(name, description, originalInput, modifiedInput, status) {
            _super.call(this, name, description, originalInput, modifiedInput);
            this.status = status;
        }
        GitDiffEditorInput.prototype.getId = function () {
            throw new Error('To implement.');
        };
        GitDiffEditorInput.prototype.getFileStatus = function () {
            return this.status;
        };
        GitDiffEditorInput.prototype.contains = function (otherInput) {
            if (this.matches(otherInput)) {
                return true;
            }
            var originalInput = this.getOriginalInput();
            if (originalInput && originalInput.matches(otherInput)) {
                return true;
            }
            var modifiedInput = this.getModifiedInput();
            if (modifiedInput && modifiedInput.matches(otherInput)) {
                return true;
            }
            return false;
        };
        return GitDiffEditorInput;
    }(diffei.DiffEditorInput));
    exports.GitDiffEditorInput = GitDiffEditorInput;
    var GitWorkingTreeDiffEditorInput = (function (_super) {
        __extends(GitWorkingTreeDiffEditorInput, _super);
        function GitWorkingTreeDiffEditorInput(name, description, originalInput, modifiedInput, status) {
            _super.call(this, name, description, originalInput, modifiedInput, status);
        }
        GitWorkingTreeDiffEditorInput.prototype.getId = function () {
            return GitWorkingTreeDiffEditorInput.ID;
        };
        GitWorkingTreeDiffEditorInput.ID = 'Monaco.IDE.UI.Viewlets.GitViewlet.GitWorkingTreeDiffEditorInput';
        return GitWorkingTreeDiffEditorInput;
    }(GitDiffEditorInput));
    exports.GitWorkingTreeDiffEditorInput = GitWorkingTreeDiffEditorInput;
    var GitIndexDiffEditorInput = (function (_super) {
        __extends(GitIndexDiffEditorInput, _super);
        function GitIndexDiffEditorInput(name, description, originalInput, modifiedInput, status) {
            _super.call(this, name, description, originalInput, modifiedInput, status);
        }
        GitIndexDiffEditorInput.prototype.getId = function () {
            return GitIndexDiffEditorInput.ID;
        };
        GitIndexDiffEditorInput.ID = 'Monaco.IDE.UI.Viewlets.GitViewlet.GitIndexDiffEditorInput';
        return GitIndexDiffEditorInput;
    }(GitDiffEditorInput));
    exports.GitIndexDiffEditorInput = GitIndexDiffEditorInput;
    var NativeGitIndexStringEditorInput = (function (_super) {
        __extends(NativeGitIndexStringEditorInput, _super);
        function NativeGitIndexStringEditorInput(name, description, mime, status, path, treeish, gitService, editorService, instantiationService) {
            var _this = this;
            _super.call(this, name, description, null, mime, false, instantiationService);
            this.gitService = gitService;
            this.editorService = editorService;
            this.status = status;
            this.path = path;
            this.treeish = treeish;
            this.delayer = new async.ThrottledDelayer(1000);
            this.toDispose = [];
            this.toDispose.push(this.gitService.addListener2(git.ServiceEvents.STATE_CHANGED, function () { return _this.onGitServiceStateChange(); }));
            this.toDispose.push(this.gitService.addListener2(git.ServiceEvents.OPERATION_END, function () { return _this.onGitServiceStateChange(); }));
        }
        NativeGitIndexStringEditorInput.prototype.getId = function () {
            return NativeGitIndexStringEditorInput.ID;
        };
        NativeGitIndexStringEditorInput.prototype.getFileStatus = function () {
            return this.status;
        };
        NativeGitIndexStringEditorInput.prototype.resolve = function (refresh) {
            var _this = this;
            if (refresh || !this.getValue()) {
                return this.gitService.buffer(this.path, this.treeish).then(function (contents) {
                    if (_this.getValue() !== contents) {
                        _this.setValue(contents);
                    }
                    return _super.prototype.resolve.call(_this, refresh);
                });
            }
            else {
                return _super.prototype.resolve.call(this, refresh);
            }
        };
        NativeGitIndexStringEditorInput.prototype.onGitServiceStateChange = function () {
            var _this = this;
            var isVisible = this.editorService.isVisible(this, true);
            if (!isVisible) {
                return;
            }
            this.delayer.trigger(function () { return _this.resolve(true); });
        };
        NativeGitIndexStringEditorInput.prototype.dispose = function () {
            if (this.delayer) {
                this.delayer.cancel();
                this.delayer = null;
            }
            this.toDispose = lifecycle.disposeAll(this.toDispose);
            _super.prototype.dispose.call(this);
        };
        NativeGitIndexStringEditorInput.ID = 'Monaco.IDE.UI.Viewlets.GitViewlet.NativeGitIndexStringEditorInput';
        NativeGitIndexStringEditorInput = __decorate([
            __param(6, IGitService),
            __param(7, editorService_1.IWorkbenchEditorService),
            __param(8, instantiation_1.IInstantiationService)
        ], NativeGitIndexStringEditorInput);
        return NativeGitIndexStringEditorInput;
    }(stringei.StringEditorInput));
    exports.NativeGitIndexStringEditorInput = NativeGitIndexStringEditorInput;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/git/browser/gitServices", ["require", "exports", 'vs/nls!vs/workbench/parts/git/browser/gitServices', 'vs/base/common/platform', 'vs/base/common/winjs.base', 'vs/base/common/lifecycle', 'vs/base/common/types', 'vs/base/common/actions', 'vs/base/common/errors', 'vs/base/common/mime', 'vs/base/common/paths', 'vs/base/common/eventEmitter', 'vs/workbench/common/events', 'vs/workbench/parts/git/common/git', 'vs/workbench/parts/git/common/gitModel', 'vs/workbench/parts/git/browser/gitEditorInputs', 'vs/workbench/parts/git/browser/gitOperations', 'vs/workbench/parts/files/common/files', 'vs/platform/files/common/files', 'vs/base/common/async', 'vs/base/common/severity', 'vs/workbench/parts/output/common/output', 'vs/workbench/services/editor/common/editorService', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/platform/workspace/common/workspace', 'vs/platform/lifecycle/common/lifecycle', 'vs/base/common/uri', 'semver', 'electron'], function (require, exports, nls, platform, winjs, lifecycle, types, actions, errors, mime, paths, ee, wbevents, git, model, giteditorinputs, operations, filesCommon, files_1, async, severity_1, output_1, editorService_1, configuration_1, event_1, instantiation_1, message_1, workspace_1, lifecycle_1, uri_1, semver, electron_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function toReadablePath(path) {
        if (!platform.isWindows) {
            return path;
        }
        return path.replace(/\//g, '\\');
    }
    var EditorInputCache = (function () {
        function EditorInputCache(gitService, // gitService passed as argument, not by injection
            instantiationService, fileService, eventService, editorService, contextService) {
            var _this = this;
            this.instantiationService = instantiationService;
            this.fileService = fileService;
            this.eventService = eventService;
            this.editorService = editorService;
            this.contextService = contextService;
            this.gitService = gitService;
            this.cache = {};
            this.toDispose = [];
            this.toDispose.push(this.gitService.getModel().addListener2('fileStatus:dispose', function (fileStatus) { return _this.onFileStatusDispose(fileStatus); }));
        }
        EditorInputCache.prototype.getInput = function (status) {
            var result = this.cache[status.getId()];
            if (result) {
                return result;
            }
            result = this.createInput(status);
            this.cache[status.getId()] = result;
            return result;
        };
        EditorInputCache.prototype.createInput = function (status) {
            return winjs.TPromise.join([this.createLeftInput(status), this.createRightInput(status)]).then(function (result) {
                var leftInput = result[0];
                var rightInput = result[1];
                var fileSegment;
                var folderSegment;
                if (status.getStatus() === git.Status.INDEX_RENAMED) {
                    var pathComponents = status.getRename().split('/');
                    fileSegment = pathComponents[pathComponents.length - 1];
                    folderSegment = toReadablePath(pathComponents.slice(0, pathComponents.length - 1).join('/'));
                }
                else {
                    var pathComponents = status.getPathComponents();
                    fileSegment = pathComponents[pathComponents.length - 1];
                    folderSegment = toReadablePath(pathComponents.slice(0, pathComponents.length - 1).join('/'));
                }
                if (!leftInput) {
                    if (!rightInput) {
                        var error = new Error(nls.localize(0, null));
                        error.gitErrorCode = git.GitErrorCodes.CantOpenResource;
                        return winjs.Promise.wrapError(error);
                    }
                    return winjs.TPromise.as(rightInput);
                }
                switch (status.getStatus()) {
                    case git.Status.INDEX_MODIFIED:
                        return winjs.TPromise.as(new giteditorinputs.GitIndexDiffEditorInput(fileSegment, nls.localize(1, null, folderSegment), leftInput, rightInput, status));
                    case git.Status.INDEX_RENAMED:
                        return winjs.TPromise.as(new giteditorinputs.GitIndexDiffEditorInput(fileSegment, nls.localize(2, null, folderSegment), leftInput, rightInput, status));
                    case git.Status.MODIFIED:
                        return winjs.TPromise.as(new giteditorinputs.GitWorkingTreeDiffEditorInput(fileSegment, nls.localize(3, null, folderSegment), leftInput, rightInput, status));
                    default:
                        return winjs.TPromise.as(new giteditorinputs.GitDiffEditorInput(fileSegment, nls.localize(4, null, folderSegment), leftInput, rightInput, status));
                }
            }).then(function (editorInput) {
                return editorInput;
            }, function (errs) {
                return winjs.Promise.wrapError(types.isArray(errs) ? errs[0] || errs[1] : errs);
            });
        };
        EditorInputCache.prototype.createLeftInput = function (status) {
            var path = status.getPath();
            var model = this.gitService.getModel();
            switch (status.getStatus()) {
                case git.Status.INDEX_MODIFIED:
                case git.Status.INDEX_RENAMED:
                    return this.gitService.show(path, status, 'HEAD', status.getMimetype());
                case git.Status.MODIFIED:
                    var indexStatus = model.getStatus().find(path, git.StatusType.INDEX);
                    if (indexStatus && indexStatus.getStatus() === git.Status.INDEX_RENAMED) {
                        return this.gitService.show(indexStatus.getRename(), status, '~', status.getMimetype());
                    }
                    if (indexStatus) {
                        return this.gitService.show(path, status, '~', status.getMimetype());
                    }
                    return this.gitService.show(path, status, 'HEAD', status.getMimetype());
                default:
                    return winjs.TPromise.as(null);
            }
        };
        EditorInputCache.prototype.createRightInput = function (status) {
            var model = this.gitService.getModel();
            var path = status.getPath();
            var resource = uri_1.default.file(paths.join(model.getRepositoryRoot(), path));
            switch (status.getStatus()) {
                case git.Status.INDEX_MODIFIED:
                case git.Status.INDEX_ADDED:
                case git.Status.INDEX_COPIED:
                    return this.gitService.show(path, status, '~', status.getMimetype());
                case git.Status.INDEX_RENAMED:
                    return this.gitService.show(status.getRename(), status, '~', status.getMimetype());
                case git.Status.INDEX_DELETED:
                case git.Status.DELETED:
                    return this.gitService.show(path, status, 'HEAD', status.getMimetype());
                case git.Status.MODIFIED:
                case git.Status.UNTRACKED:
                case git.Status.IGNORED:
                    var indexStatus = model.getStatus().find(path, git.StatusType.INDEX);
                    if (indexStatus && indexStatus.getStatus() === git.Status.INDEX_RENAMED) {
                        resource = uri_1.default.file(paths.join(model.getRepositoryRoot(), indexStatus.getRename()));
                    }
                    return this.editorService.inputToType({ resource: resource });
                case git.Status.BOTH_MODIFIED:
                    return this.editorService.inputToType({ resource: resource });
                default:
                    return winjs.TPromise.as(null);
            }
        };
        EditorInputCache.prototype.onFileStatusDispose = function (fileStatus) {
            var _this = this;
            var id = fileStatus.getId();
            var editorInputPromise = this.cache[id];
            if (editorInputPromise) {
                editorInputPromise.done(function (editorInput) { _this.eventuallyDispose(editorInput); });
                delete this.cache[id];
            }
        };
        /**
         * If the disposed status is the same as this input's status, we must try to dispose the input.
         * But we should not do it while the input is still open. This method will eventually call dispose
         * when the editor input goes out of the visible editors.
         */
        EditorInputCache.prototype.eventuallyDispose = function (editorInput) {
            var _this = this;
            if (!this.maybeDispose(editorInput)) {
                var listener = this.eventService.addListener2(wbevents.EventType.EDITOR_INPUT_CHANGED, function () {
                    if (_this.maybeDispose(editorInput)) {
                        listener.dispose();
                    }
                });
            }
        };
        EditorInputCache.prototype.maybeDispose = function (editorInput) {
            if (!this.editorService.getVisibleEditors().some(function (editor) { return editor.input && editor.input.matches(editorInput); })) {
                editorInput.dispose();
                return true;
            }
            return false;
        };
        EditorInputCache.prototype.dispose = function () {
            var _this = this;
            Object.keys(this.cache).forEach(function (key) {
                _this.cache[key].done(function (editorInput) { editorInput.dispose(); });
                delete _this.cache[key];
            });
            this.toDispose = lifecycle.disposeAll(this.toDispose);
        };
        EditorInputCache = __decorate([
            // gitService passed as argument, not by injection
            __param(1, instantiation_1.IInstantiationService),
            __param(2, files_1.IFileService),
            __param(3, event_1.IEventService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, workspace_1.IWorkspaceContextService)
        ], EditorInputCache);
        return EditorInputCache;
    }());
    var AutoFetcher = (function () {
        function AutoFetcher(gitService, // gitService passed as argument, not by injection
            eventService, messageService, editorService, configurationService, instantiationService) {
            var _this = this;
            this._state = git.AutoFetcherState.Disabled;
            this.gitService = gitService;
            this.eventService = eventService;
            this.messageService = messageService;
            this.configurationService = configurationService;
            this.instantiationService = instantiationService;
            this.currentRequest = null;
            this.timeout = AutoFetcher.MIN_TIMEOUT;
            this.toDispose = [];
            this.toDispose.push(this.configurationService.addListener2(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) { return _this.onConfiguration(e.config.git); }));
            configurationService.loadConfiguration('git').done(function (c) { return _this.onConfiguration(c); });
        }
        Object.defineProperty(AutoFetcher.prototype, "state", {
            get: function () {
                return this._state;
            },
            enumerable: true,
            configurable: true
        });
        AutoFetcher.prototype.onConfiguration = function (config) {
            if (config.autofetch === false) {
                this.disable();
            }
            else {
                this.enable();
            }
        };
        AutoFetcher.prototype.enable = function () {
            var _this = this;
            if (this._state !== git.AutoFetcherState.Disabled) {
                return;
            }
            this.gitServiceStateDisposable = this.gitService.addListener2(git.ServiceEvents.STATE_CHANGED, function (e) { return _this.onGitServiceStateChange(e); });
            this._state = git.AutoFetcherState.Active;
            this.onGitServiceStateChange(this.gitService.getState());
        };
        AutoFetcher.prototype.disable = function () {
            if (this.gitServiceStateDisposable) {
                this.gitServiceStateDisposable.dispose();
                this.gitServiceStateDisposable = null;
            }
            this.deactivate();
            this._state = git.AutoFetcherState.Disabled;
        };
        AutoFetcher.prototype.onGitServiceStateChange = function (state) {
            if (state === git.ServiceState.OK) {
                this.activate();
            }
            else {
                this.deactivate();
            }
        };
        AutoFetcher.prototype.activate = function () {
            if (this.currentRequest) {
                this.currentRequest.cancel();
            }
            this._state = git.AutoFetcherState.Active;
            this.loop();
        };
        AutoFetcher.prototype.deactivate = function () {
            if (!this.currentRequest) {
                return;
            }
            this._state = git.AutoFetcherState.Inactive;
            this.currentRequest.cancel();
            this.currentRequest = null;
        };
        AutoFetcher.prototype.loop = function () {
            var _this = this;
            this._state = git.AutoFetcherState.Fetching;
            this.currentRequest = this.gitService.fetch().then(function () {
                _this.timeout = AutoFetcher.MIN_TIMEOUT;
            }, function (err) {
                if (errors.isPromiseCanceledError(err)) {
                    return winjs.Promise.wrapError(err);
                }
                else if (err.gitErrorCode === git.GitErrorCodes.AuthenticationFailed) {
                    return winjs.Promise.wrapError(err);
                }
                else {
                    _this.timeout = Math.min(Math.round(_this.timeout * 1.2), AutoFetcher.MAX_TIMEOUT); // backoff
                }
            });
            this.currentRequest.then(function () {
                _this._state = git.AutoFetcherState.Active;
                _this.currentRequest = winjs.TPromise.timeout(_this.timeout);
                return _this.currentRequest;
            }).then(function () { return _this.loop(); }, function (err) { return _this.deactivate(); });
        };
        AutoFetcher.prototype.dispose = function () {
            this.disable();
        };
        AutoFetcher.MIN_TIMEOUT = 2 * 60 * 1000; // every two minutes
        AutoFetcher.MAX_TIMEOUT = 5 * 60 * 1000; // every five minutes
        AutoFetcher = __decorate([
            // gitService passed as argument, not by injection
            __param(1, event_1.IEventService),
            __param(2, message_1.IMessageService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, configuration_1.IConfigurationService),
            __param(5, instantiation_1.IInstantiationService)
        ], AutoFetcher);
        return AutoFetcher;
    }());
    exports.AutoFetcher = AutoFetcher;
    var GitService = (function (_super) {
        __extends(GitService, _super);
        function GitService(raw, instantiationService, eventService, messageService, editorService, outputService, contextService, lifecycleService) {
            _super.call(this);
            this.serviceId = git.IGitService;
            this.instantiationService = instantiationService;
            this.eventService = eventService;
            this.messageService = messageService;
            this.editorService = editorService;
            this.outputService = outputService;
            this.contextService = contextService;
            this.lifecycleService = lifecycleService;
            this.raw = raw;
            this.state = git.ServiceState.NotInitialized;
            this.operations = [];
            this.model = new model.Model();
            this.toDispose = [];
            this.needsRefresh = false;
            this.refreshDelayer = new async.PeriodThrottledDelayer(500, 10000);
            this.autoFetcher = this.instantiationService.createInstance(AutoFetcher, this);
            this.registerListeners();
            this.inputCache = this.instantiationService.createInstance(EditorInputCache, this);
            this.triggerStatus(true); // trigger initial status
            this.raw.getVersion().done(function (version) {
                version = version || '';
                version = version.replace(/^(\d+\.\d+\.\d+).*$/, '$1');
                version = semver.valid(version);
                if (version && semver.satisfies(version, '<2.0.0')) {
                    messageService.show(severity_1.default.Warning, {
                        message: nls.localize(5, null, version),
                        actions: [
                            message_1.CancelAction,
                            new actions.Action('downloadLatest', nls.localize(6, null), '', true, function () {
                                electron_1.shell.openExternal('https://git-scm.com/');
                                return null;
                            })
                        ]
                    });
                }
            });
        }
        GitService.prototype.registerListeners = function () {
            var _this = this;
            this.toDispose.push(this.eventService.addListener2(files_1.EventType.FILE_CHANGES, function (e) { return _this.onFileChanges(e); }));
            this.toDispose.push(this.eventService.addListener2(filesCommon.EventType.FILE_SAVED, function (e) { return _this.onLocalFileChange(e); }));
            this.toDispose.push(this.eventService.addListener2(filesCommon.EventType.FILE_REVERTED, function (e) { return _this.onLocalFileChange(e); }));
            this.lifecycleService.onShutdown(this.dispose, this);
        };
        GitService.prototype.triggerStatus = function (force) {
            var _this = this;
            if (force === void 0) { force = false; }
            if (this.isInitialized() && !this.isIdle() && !force) {
                this.refreshDelayer.cancel();
                return;
            }
            var onError = async.once(function (e) {
                if (!errors.isPromiseCanceledError(e)) {
                    _this.messageService.show(severity_1.default.Error, e);
                }
            });
            this.refreshDelayer.trigger(function () { return _this.status(); }).done(null, onError);
        };
        GitService.prototype.onLocalFileChange = function (e) {
            var shouldTriggerStatus = e.gotUpdated() && paths.basename(e.getAfter().resource.fsPath) === '.gitignore';
            if (!shouldTriggerStatus) {
                return;
            }
            this.triggerStatus();
        };
        GitService.prototype.onFileChanges = function (e) {
            var _this = this;
            var isIdle = this.isIdle();
            var shouldTriggerStatus = e.changes.some(function (c) {
                var workspacePath = _this.contextService.toWorkspaceRelativePath(c.resource);
                if (!workspacePath) {
                    return false; // ignore out of workspace files
                }
                // for .gitindex, the service must be idle
                if ('.git/index' === workspacePath) {
                    return isIdle;
                }
                // for anything other that .git*
                if (!/^\.git/.test(workspacePath)) {
                    return true;
                }
                // added or deleted .git folder
                if (workspacePath === '.git') {
                    return c.type === files_1.FileChangeType.ADDED || c.type === files_1.FileChangeType.DELETED;
                }
                return ['.git/index.lock', '.git/FETCH_HEAD', '.gitignore', '.gitmodules'].indexOf(workspacePath) === -1;
            });
            if (!shouldTriggerStatus) {
                return;
            }
            this.triggerStatus();
        };
        GitService.prototype.onGitServiceOperationEnd = function (e) {
            if (e.operation.id === git.ServiceOperations.COMMAND) {
                this.triggerStatus();
            }
        };
        GitService.prototype.getState = function () {
            return this.state;
        };
        GitService.prototype.getModel = function () {
            return this.model;
        };
        GitService.prototype.status = function () {
            var _this = this;
            return this.run(git.ServiceOperations.STATUS, function () { return _this.raw.status(); });
        };
        GitService.prototype.init = function () {
            var _this = this;
            return this.run(git.ServiceOperations.INIT, function () { return _this.raw.init(); });
        };
        GitService.prototype.add = function (files) {
            var _this = this;
            return this.run(git.ServiceOperations.ADD, function () { return _this.raw.add(GitService.toPaths(files)); });
        };
        GitService.prototype.stage = function (filePath, content) {
            var _this = this;
            return this.run(git.ServiceOperations.STAGE, function () { return _this.raw.stage(filePath, content); });
        };
        GitService.prototype.branch = function (name, checkout) {
            var _this = this;
            if (checkout === void 0) { checkout = false; }
            return this.run(git.ServiceOperations.BRANCH, function () { return _this.raw.branch(name, checkout); });
        };
        GitService.prototype.checkout = function (treeish, files) {
            var _this = this;
            if (treeish === void 0) { treeish = ''; }
            if (files === void 0) { files = null; }
            return this.run(git.ServiceOperations.CHECKOUT, function () { return _this.raw.checkout(treeish, GitService.toPaths(files)); });
        };
        GitService.prototype.clean = function (files) {
            var _this = this;
            return this.run(git.ServiceOperations.CLEAN, function () { return _this.raw.clean(files.map(function (s) { return s.getPath(); })); });
        };
        GitService.prototype.undo = function () {
            var _this = this;
            return this.run(git.ServiceOperations.UNDO, function () { return _this.raw.undo(); });
        };
        GitService.prototype.reset = function (treeish, hard) {
            var _this = this;
            return this.run(git.ServiceOperations.RESET, function () { return _this.raw.reset(treeish, hard); });
        };
        GitService.prototype.revertFiles = function (treeish, files) {
            var _this = this;
            return this.run(git.ServiceOperations.RESET, function () { return _this.raw.revertFiles(treeish, (files || []).map(function (s) { return s.getPath(); })); });
        };
        GitService.prototype.fetch = function () {
            var _this = this;
            return this.run(git.ServiceOperations.BACKGROUND_FETCH, function () { return _this.raw.fetch(); });
        };
        GitService.prototype.pull = function (rebase) {
            var _this = this;
            return this.run(git.ServiceOperations.PULL, function () { return _this.raw.pull(rebase); });
        };
        GitService.prototype.push = function (remote, name, options) {
            var _this = this;
            return this.run(git.ServiceOperations.PUSH, function () { return _this.raw.push(remote, name, options); });
        };
        GitService.prototype.sync = function (rebase) {
            var _this = this;
            var head = this.model.getHEAD();
            var isAhead = head && head.upstream && !!head.ahead;
            if (!isAhead) {
                return this.run(git.ServiceOperations.SYNC, function () { return _this.raw.pull(rebase); });
            }
            else {
                return this.run(git.ServiceOperations.SYNC, function () { return _this.raw.sync(); });
            }
        };
        GitService.prototype.commit = function (message, amend, stage) {
            var _this = this;
            if (amend === void 0) { amend = false; }
            if (stage === void 0) { stage = false; }
            return this.run(git.ServiceOperations.COMMIT, function () { return _this.raw.commit(message, amend, stage); });
        };
        GitService.prototype.detectMimetypes = function (path, treeish) {
            if (treeish === void 0) { treeish = '~'; }
            return this.raw.detectMimetypes(path, treeish);
        };
        GitService.prototype.run = function (operationId, fn) {
            var _this = this;
            return this.raw.serviceState().then(function (state) {
                if (state === git.RawServiceState.GitNotFound) {
                    _this.transition(git.ServiceState.NoGit);
                    return winjs.TPromise.as(null);
                }
                else if (state === git.RawServiceState.Disabled) {
                    _this.transition(git.ServiceState.Disabled);
                    return winjs.TPromise.as(null);
                }
                else {
                    return _this._run(operationId, fn);
                }
            });
        };
        GitService.prototype._run = function (operationId, fn) {
            var _this = this;
            var operation = new operations.GitOperation(operationId, fn);
            this.operations.push(operation);
            this.emit(git.ServiceEvents.OPERATION_START, operation);
            this.emit(git.ServiceEvents.OPERATION, operation);
            var onDone = function (error) {
                if (error === void 0) { error = null; }
                var index = _this.operations.indexOf(operation);
                if (index > -1) {
                    _this.operations.splice(index, 1);
                }
                var e = { operation: operation, error: error };
                _this.emit(git.ServiceEvents.OPERATION_END, e);
                _this.onGitServiceOperationEnd(e);
                _this.emit(git.ServiceEvents.OPERATION, operation);
            };
            return operation.run().then(function (status) {
                _this.model.update(status);
                onDone();
                if (status) {
                    _this.transition(types.isUndefinedOrNull(status.state) ? git.ServiceState.OK : status.state);
                }
                else {
                    _this.transition(git.ServiceState.NotARepo);
                }
                return _this.model;
            }, function (e) {
                onDone(e);
                if (errors.isPromiseCanceledError(e)) {
                    return winjs.Promise.wrapError(e);
                }
                var gitErrorCode = e.gitErrorCode || null;
                if (gitErrorCode === git.GitErrorCodes.NotAtRepositoryRoot) {
                    _this.transition(git.ServiceState.NotAtRepoRoot);
                    return winjs.TPromise.as(_this.model);
                }
                _this.emit(git.ServiceEvents.ERROR, e);
                _this.transition(git.ServiceState.OK);
                if (gitErrorCode === git.GitErrorCodes.NoUserNameConfigured || gitErrorCode === git.GitErrorCodes.NoUserEmailConfigured) {
                    _this.messageService.show(severity_1.default.Warning, nls.localize(7, null));
                    return winjs.TPromise.as(null);
                }
                else if (gitErrorCode === git.GitErrorCodes.BadConfigFile) {
                    _this.messageService.show(severity_1.default.Error, nls.localize(8, null, e.message));
                    return winjs.TPromise.as(null);
                }
                else if (gitErrorCode === git.GitErrorCodes.UnmergedChanges) {
                    _this.messageService.show(severity_1.default.Warning, nls.localize(9, null));
                    return winjs.TPromise.as(null);
                }
                var error;
                var showOutputAction = new actions.Action('show.gitOutput', nls.localize(10, null), null, true, function () { return _this.outputService.showOutput('Git'); });
                var cancelAction = new actions.Action('close.message', nls.localize(11, null), null, true, function () { return winjs.TPromise.as(true); });
                error = errors.create(nls.localize(12, null), { actions: [showOutputAction, cancelAction] });
                error.gitErrorCode = gitErrorCode;
                return winjs.Promise.wrapError(error);
            });
        };
        GitService.prototype.transition = function (state) {
            var oldState = this.state;
            this.state = state;
            if (state !== oldState) {
                this.emit(git.ServiceEvents.STATE_CHANGED, state);
            }
        };
        GitService.prototype.buffer = function (path, treeish) {
            if (treeish === void 0) { treeish = '~'; }
            return this.raw.show(path, treeish);
        };
        GitService.prototype.show = function (path, status, treeish, mimetype) {
            var _this = this;
            if (treeish === void 0) { treeish = '~'; }
            if (mimetype === void 0) { mimetype = 'text/plain'; }
            return this.detectMimetypes(path, treeish).then(function (mimetypes) {
                var pathComponents = status.getPathComponents();
                var fileSegment = pathComponents[pathComponents.length - 1];
                var folderSegment = toReadablePath(pathComponents.slice(0, pathComponents.length - 1).join('/'));
                var description;
                if (treeish === '~') {
                    description = nls.localize(13, null, folderSegment);
                }
                else {
                    description = nls.localize(14, null, folderSegment, treeish);
                }
                if (mime.isUnspecific(mimetypes)) {
                    mimetypes = mime.guessMimeTypes(path); // guess from path if our detection did not yield results
                }
                // Binary: our story is weak here for binary files on the index. Since we run natively, we do not have a way currently
                // to e.g. show images as binary inside the renderer because images need to be served through a URL to show. We could revisit this by
                // allowing to use data URLs for resource inputs to render them. However, this would mean potentially loading a large file into memory
                //
                // Our solution now is to detect binary files and immediately return an input that is flagged as binary unknown mime type.
                if (mime.isBinaryMime(mime.guessMimeTypes(path)) || mimetypes.indexOf(mime.MIME_BINARY) >= 0) {
                    return winjs.Promise.wrapError(new Error('The resource seems to be binary and cannot be displayed'));
                }
                // Text
                return winjs.TPromise.as(_this.instantiationService.createInstance(giteditorinputs.NativeGitIndexStringEditorInput, fileSegment, description, mimetypes.join(', '), status, path, treeish));
            });
        };
        GitService.prototype.getInput = function (status) {
            var _this = this;
            return this.inputCache.getInput(status).then(null, function (err) {
                if (err.gitErrorCode = git.GitErrorCodes.CantOpenResource) {
                    _this.messageService.show(severity_1.default.Warning, nls.localize(15, null));
                    return winjs.TPromise.as(null);
                }
                return winjs.Promise.wrapError(err);
            });
        };
        GitService.prototype.isInitialized = function () {
            return this.state === git.ServiceState.OK;
        };
        GitService.prototype.isIdle = function () {
            return this.isInitialized() && !this.operations.some(function (op) { return op.id !== git.ServiceOperations.BACKGROUND_FETCH; });
        };
        GitService.prototype.getRunningOperations = function () {
            return this.operations;
        };
        GitService.prototype.onOutput = function () {
            return this.raw.onOutput();
        };
        GitService.prototype.getAutoFetcher = function () {
            return this.autoFetcher;
        };
        GitService.toPaths = function (files) {
            if (!files) {
                return null;
            }
            return files.map(function (status) {
                /*	In the case that a file was renamed in the index and (changed || deleted) in the
                    working tree, we must use its new name, running the checkout command.
                */
                switch (status.getStatus()) {
                    case git.Status.MODIFIED:
                    case git.Status.DELETED:
                        if (status.getRename()) {
                            return status.getRename();
                        }
                    default:
                        return status.getPath();
                }
            });
        };
        GitService.prototype.dispose = function () {
            this.emit(git.ServiceEvents.DISPOSE);
            if (this.model) {
                this.model.dispose();
                this.model = null;
            }
            if (this.remoteListenerUnbind) {
                this.remoteListenerUnbind();
                this.remoteListenerUnbind = null;
            }
            _super.prototype.dispose.call(this);
        };
        GitService.ID = 'Monaco.IDE.UI.Services.GitService';
        GitService = __decorate([
            __param(1, instantiation_1.IInstantiationService),
            __param(2, event_1.IEventService),
            __param(3, message_1.IMessageService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, output_1.IOutputService),
            __param(6, workspace_1.IWorkspaceContextService),
            __param(7, lifecycle_1.ILifecycleService)
        ], GitService);
        return GitService;
    }(ee.EventEmitter));
    exports.GitService = GitService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/git/electron-browser/electronGitService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/workbench/parts/git/common/git', 'vs/workbench/parts/git/common/noopGitService', 'vs/workbench/parts/git/browser/gitServices', 'vs/platform/lifecycle/common/lifecycle', 'vs/workbench/parts/output/common/output', 'vs/workbench/services/editor/common/editorService', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/platform/workspace/common/workspace', 'vs/base/node/service.cp', 'vs/workbench/parts/git/node/rawGitService', 'vs/base/common/uri', 'child_process', 'path', 'electron'], function (require, exports, winjs_base_1, git_1, noopGitService_1, gitServices_1, lifecycle_1, output_1, editorService_1, configuration_1, event_1, instantiation_1, message_1, workspace_1, service_cp_1, rawGitService_1, uri_1, child_process_1, path_1, electron_1) {
    "use strict";
    function parseVersion(raw) {
        return raw.replace(/^git version /, '');
    }
    function findSpecificGit(path) {
        return new winjs_base_1.TPromise(function (c, e) {
            var buffers = [];
            var child = child_process_1.spawn(path, ['--version']);
            child.stdout.on('data', function (b) { return buffers.push(b); });
            child.on('error', e);
            child.on('exit', function (code) { return code ? e(new Error('Not found')) : c({ path: path, version: parseVersion(Buffer.concat(buffers).toString('utf8').trim()) }); });
        });
    }
    function findGitDarwin() {
        return new winjs_base_1.TPromise(function (c, e) {
            child_process_1.exec('which git', function (err, gitPathBuffer) {
                if (err) {
                    return e('git not found');
                }
                var path = gitPathBuffer.toString().replace(/^\s+|\s+$/g, '');
                function getVersion(path) {
                    // make sure git executes
                    child_process_1.exec('git --version', function (err, stdout) {
                        if (err) {
                            return e('git not found');
                        }
                        return c({ path: path, version: parseVersion(stdout.toString('utf8').trim()) });
                    });
                }
                if (path !== '/usr/bin/git') {
                    return getVersion(path);
                }
                // must check if XCode is installed
                child_process_1.exec('xcode-select -p', function (err) {
                    if (err && err.code === 2) {
                        // git is not installed, and launching /usr/bin/git
                        // will prompt the user to install it
                        return e('git not found');
                    }
                    getVersion(path);
                });
            });
        });
    }
    function findSystemGitWin32(base) {
        if (!base) {
            return winjs_base_1.TPromise.wrapError('Not found');
        }
        return findSpecificGit(path_1.join(base, 'Git', 'cmd', 'git.exe'));
    }
    function findGitWin32() {
        return findSystemGitWin32(process.env['ProgramW6432'])
            .then(null, function () { return findSystemGitWin32(process.env['ProgramFiles(x86)']); })
            .then(null, function () { return findSystemGitWin32(process.env['ProgramFiles']); })
            .then(null, function () { return findSpecificGit('git'); });
    }
    function findGit(hint) {
        var first = hint ? findSpecificGit(hint) : winjs_base_1.TPromise.wrapError(null);
        return first.then(null, function () {
            switch (process.platform) {
                case 'darwin': return findGitDarwin();
                case 'win32': return findGitWin32();
                default: return findSpecificGit('git');
            }
        });
    }
    var UnavailableRawGitService = (function (_super) {
        __extends(UnavailableRawGitService, _super);
        function UnavailableRawGitService() {
            _super.call(this, null);
        }
        return UnavailableRawGitService;
    }(rawGitService_1.RawGitService));
    var DisabledRawGitService = (function (_super) {
        __extends(DisabledRawGitService, _super);
        function DisabledRawGitService() {
            _super.call(this, null);
        }
        DisabledRawGitService.prototype.serviceState = function () {
            return winjs_base_1.TPromise.as(git_1.RawServiceState.Disabled);
        };
        return DisabledRawGitService;
    }(rawGitService_1.RawGitService));
    function createNativeRawGitService(workspaceRoot, path, defaultEncoding) {
        return findGit(path).then(function (_a) {
            var path = _a.path, version = _a.version;
            var client = new service_cp_1.Client(uri_1.default.parse(require.toUrl('bootstrap')).fsPath, {
                serverName: 'Git',
                timeout: 1000 * 60,
                args: [path, workspaceRoot, defaultEncoding, electron_1.remote.process.execPath, version],
                env: {
                    ATOM_SHELL_INTERNAL_RUN_AS_NODE: 1,
                    AMD_ENTRYPOINT: 'vs/workbench/parts/git/electron-browser/gitApp'
                }
            });
            return client.getService('GitService', rawGitService_1.RawGitService);
        }, function () { return new UnavailableRawGitService(); });
    }
    var ElectronRawGitService = (function (_super) {
        __extends(ElectronRawGitService, _super);
        function ElectronRawGitService(workspaceRoot, configurationService) {
            _super.call(this, configurationService.loadConfiguration().then(function (conf) {
                var enabled = conf.git ? conf.git.enabled : true;
                if (!enabled) {
                    return winjs_base_1.TPromise.as(new DisabledRawGitService());
                }
                var gitPath = (conf.git && conf.git.path) || null;
                var encoding = (conf.files && conf.files.encoding) || 'utf8';
                return createNativeRawGitService(workspaceRoot, gitPath, encoding);
            }));
        }
        ElectronRawGitService = __decorate([
            __param(1, configuration_1.IConfigurationService)
        ], ElectronRawGitService);
        return ElectronRawGitService;
    }(rawGitService_1.DelayedRawGitService));
    var ElectronGitService = (function (_super) {
        __extends(ElectronGitService, _super);
        function ElectronGitService(instantiationService, eventService, messageService, editorService, outputService, contextService, lifecycleService) {
            var workspace = contextService.getWorkspace();
            var raw = !workspace
                ? new noopGitService_1.NoOpGitService()
                : instantiationService.createInstance(ElectronRawGitService, workspace.resource.fsPath);
            _super.call(this, raw, instantiationService, eventService, messageService, editorService, outputService, contextService, lifecycleService);
        }
        ElectronGitService = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, event_1.IEventService),
            __param(2, message_1.IMessageService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, output_1.IOutputService),
            __param(5, workspace_1.IWorkspaceContextService),
            __param(6, lifecycle_1.ILifecycleService)
        ], ElectronGitService);
        return ElectronGitService;
    }(gitServices_1.GitService));
    exports.ElectronGitService = ElectronGitService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/files/node/watcher/common", ["require", "exports", 'vs/base/common/uri', 'vs/base/common/paths', 'vs/platform/files/common/files'], function (require, exports, uri_1, paths, files_1) {
    'use strict';
    function toFileChangesEvent(changes) {
        // map to file changes event that talks about URIs
        return new files_1.FileChangesEvent(changes.map(function (c) {
            return {
                type: c.type,
                resource: uri_1.default.file(c.path)
            };
        }));
    }
    exports.toFileChangesEvent = toFileChangesEvent;
    /**
     * Given events that occurred, applies some rules to normalize the events
     */
    function normalize(changes) {
        // Build deltas
        var normalizer = new EventNormalizer();
        for (var i = 0; i < changes.length; i++) {
            var event_1 = changes[i];
            normalizer.processEvent(event_1);
        }
        return normalizer.normalize();
    }
    exports.normalize = normalize;
    var EventNormalizer = (function () {
        function EventNormalizer() {
            this.normalized = [];
            this.mapPathToChange = Object.create(null);
        }
        EventNormalizer.prototype.processEvent = function (event) {
            // Event path already exists
            var existingEvent = this.mapPathToChange[event.path];
            if (existingEvent) {
                var currentChangeType = existingEvent.type;
                var newChangeType = event.type;
                // ignore CREATE followed by DELETE in one go
                if (currentChangeType === files_1.FileChangeType.ADDED && newChangeType === files_1.FileChangeType.DELETED) {
                    delete this.mapPathToChange[event.path];
                    this.normalized.splice(this.normalized.indexOf(existingEvent), 1);
                }
                else if (currentChangeType === files_1.FileChangeType.DELETED && newChangeType === files_1.FileChangeType.ADDED) {
                    existingEvent.type = files_1.FileChangeType.UPDATED;
                }
                else if (currentChangeType === files_1.FileChangeType.ADDED && newChangeType === files_1.FileChangeType.UPDATED) {
                }
                else {
                    existingEvent.type = newChangeType;
                }
            }
            else {
                this.normalized.push(event);
                this.mapPathToChange[event.path] = event;
            }
        };
        EventNormalizer.prototype.normalize = function () {
            var _this = this;
            var addedChangeEvents = [];
            var deletedPaths = [];
            // This algorithm will remove all DELETE events up to the root folder
            // that got deleted if any. This ensures that we are not producing
            // DELETE events for each file inside a folder that gets deleted.
            //
            // 1.) split ADD/CHANGE and DELETED events
            // 2.) sort short deleted paths to the top
            // 3.) for each DELETE, check if there is a deleted parent and ignore the event in that case
            return this.normalized.filter(function (e) {
                if (e.type !== 2) {
                    addedChangeEvents.push(e);
                    return false; // remove ADD / CHANGE
                }
                return true; // keep DELETE
            }).sort(function (e1, e2) {
                return e1.path.length - e2.path.length; // shortest path first
            }).filter(function (e) {
                if (deletedPaths.some(function (d) { return _this.isParent(e.path, d); })) {
                    return false; // DELETE is ignored if parent is deleted already
                }
                // otherwise mark as deleted
                deletedPaths.push(e.path);
                return true;
            }).concat(addedChangeEvents);
        };
        EventNormalizer.prototype.isParent = function (p, candidate) {
            return p.indexOf(candidate + paths.nativeSep) === 0;
        };
        return EventNormalizer;
    }());
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/files/node/watcher/unix/watcherService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/node/service.cp', 'vs/base/common/uri', 'vs/platform/files/common/files', 'vs/workbench/services/files/node/watcher/common'], function (require, exports, winjs_base_1, service_cp_1, uri_1, files_1, common_1) {
    'use strict';
    var WatcherService = (function () {
        function WatcherService() {
        }
        WatcherService.prototype.watch = function (request) {
            throw new Error('not implemented');
        };
        return WatcherService;
    }());
    exports.WatcherService = WatcherService;
    var FileWatcher = (function () {
        function FileWatcher(basePath, ignored, eventEmitter, errorLogger, verboseLogging) {
            this.basePath = basePath;
            this.ignored = ignored;
            this.eventEmitter = eventEmitter;
            this.errorLogger = errorLogger;
            this.verboseLogging = verboseLogging;
            this.isDisposed = false;
            this.restartCounter = 0;
        }
        FileWatcher.prototype.startWatching = function () {
            var _this = this;
            var client = new service_cp_1.Client(uri_1.default.parse(require.toUrl('bootstrap')).fsPath, {
                serverName: 'Watcher',
                args: ['--type=watcherService'],
                env: {
                    AMD_ENTRYPOINT: 'vs/workbench/services/files/node/watcher/unix/watcherApp',
                    PIPE_LOGGING: 'true',
                    VERBOSE_LOGGING: this.verboseLogging
                }
            });
            var service = client.getService('WatcherService', WatcherService);
            // Start watching
            service.watch({ basePath: this.basePath, ignored: this.ignored, verboseLogging: this.verboseLogging }).then(null, function (err) {
                if (!(err instanceof Error && err.name === 'Canceled' && err.message === 'Canceled')) {
                    return winjs_base_1.TPromise.wrapError(err); // the service lib uses the promise cancel error to indicate the process died, we do not want to bubble this up
                }
            }, function (events) { return _this.onRawFileEvents(events); }).done(function () {
                // our watcher app should never be completed because it keeps on watching. being in here indicates
                // that the watcher process died and we want to restart it here. we only do it a max number of times
                if (!_this.isDisposed) {
                    if (_this.restartCounter <= FileWatcher.MAX_RESTARTS) {
                        _this.errorLogger('Watcher terminated unexpectedly and is restarted again...');
                        _this.restartCounter++;
                        _this.startWatching();
                    }
                    else {
                        _this.errorLogger('Watcher failed to start after retrying for some time, giving up. Please report this as a bug report!');
                    }
                }
            }, this.errorLogger);
            return function () {
                client.dispose();
                _this.isDisposed = true;
            };
        };
        FileWatcher.prototype.onRawFileEvents = function (events) {
            // Emit through broadcast service
            if (events.length > 0) {
                this.eventEmitter.emit(files_1.EventType.FILE_CHANGES, common_1.toFileChangesEvent(events));
            }
        };
        FileWatcher.MAX_RESTARTS = 5;
        return FileWatcher;
    }());
    exports.FileWatcher = FileWatcher;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/files/node/watcher/win32/csharpWatcherService", ["require", "exports", 'child_process', 'vs/platform/files/common/files', 'vs/base/node/decoder', 'vs/base/common/glob', 'vs/base/common/uri'], function (require, exports, cp, files_1, decoder, glob, uri_1) {
    'use strict';
    var OutOfProcessWin32FolderWatcher = (function () {
        function OutOfProcessWin32FolderWatcher(watchedFolder, ignored, errorLogger, eventCallback, verboseLogging) {
            this.watchedFolder = watchedFolder;
            this.ignored = ignored;
            this.eventCallback = eventCallback;
            this.errorLogger = errorLogger;
            this.verboseLogging = verboseLogging;
            this.startWatcher();
        }
        OutOfProcessWin32FolderWatcher.prototype.startWatcher = function () {
            var _this = this;
            var args = [this.watchedFolder];
            if (this.verboseLogging) {
                args.push('-verbose');
            }
            this.handle = cp.spawn(uri_1.default.parse(require.toUrl('vs/workbench/services/files/node/watcher/win32/CodeHelper.exe')).fsPath, args);
            var stdoutLineDecoder = new decoder.LineDecoder();
            // Events over stdout
            this.handle.stdout.on('data', function (data) {
                // Collect raw events from output
                var rawEvents = [];
                stdoutLineDecoder.write(data).forEach(function (line) {
                    var eventParts = line.split('|');
                    if (eventParts.length === 2) {
                        var changeType = Number(eventParts[0]);
                        var absolutePath_1 = eventParts[1];
                        // File Change Event (0 Changed, 1 Created, 2 Deleted)
                        if (changeType >= 0 && changeType < 3) {
                            // Support ignores
                            if (_this.ignored && _this.ignored.some(function (ignore) { return glob.match(ignore, absolutePath_1); })) {
                                return;
                            }
                            // Otherwise record as event
                            rawEvents.push({
                                type: OutOfProcessWin32FolderWatcher.changeTypeMap[changeType],
                                path: absolutePath_1
                            });
                        }
                        else {
                            console.log('%c[File Watcher]', 'color: darkgreen', eventParts[1]);
                        }
                    }
                });
                // Trigger processing of events through the delayer to batch them up properly
                if (rawEvents.length > 0) {
                    _this.eventCallback(rawEvents);
                }
            });
            // Errors
            this.handle.on('error', function (error) { return _this.onError(error); });
            this.handle.stderr.on('data', function (data) { return _this.onError(data); });
            // Exit
            this.handle.on('exit', function (code, signal) { return _this.onExit(code, signal); });
        };
        OutOfProcessWin32FolderWatcher.prototype.onError = function (error) {
            this.errorLogger('[FileWatcher] process error: ' + error.toString());
        };
        OutOfProcessWin32FolderWatcher.prototype.onExit = function (code, signal) {
            if (this.handle) {
                this.errorLogger('[FileWatcher] terminated unexpectedly (code: ' + code + ', signal: ' + signal + ')');
                this.startWatcher(); // restart
            }
        };
        OutOfProcessWin32FolderWatcher.prototype.dispose = function () {
            if (this.handle) {
                this.handle.kill();
                this.handle = null;
            }
        };
        OutOfProcessWin32FolderWatcher.changeTypeMap = [files_1.FileChangeType.UPDATED, files_1.FileChangeType.ADDED, files_1.FileChangeType.DELETED];
        return OutOfProcessWin32FolderWatcher;
    }());
    exports.OutOfProcessWin32FolderWatcher = OutOfProcessWin32FolderWatcher;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/files/node/watcher/win32/watcherService", ["require", "exports", 'vs/platform/files/common/files', 'vs/workbench/services/files/node/watcher/common', 'vs/workbench/services/files/node/watcher/win32/csharpWatcherService'], function (require, exports, files_1, watcher, csharpWatcherService_1) {
    'use strict';
    var FileWatcher = (function () {
        function FileWatcher(basePath, ignored, eventEmitter, errorLogger, verboseLogging) {
            this.basePath = basePath;
            this.ignored = ignored;
            this.errorLogger = errorLogger;
            this.verboseLogging = verboseLogging;
            this.eventEmitter = eventEmitter;
        }
        FileWatcher.prototype.startWatching = function () {
            var _this = this;
            var watcher = new csharpWatcherService_1.OutOfProcessWin32FolderWatcher(this.basePath, this.ignored, this.errorLogger, function (events) { return _this.onRawFileEvents(events); }, this.verboseLogging);
            return function () { return watcher.dispose(); };
        };
        FileWatcher.prototype.onRawFileEvents = function (events) {
            // Emit through broadcast service
            if (events.length > 0) {
                this.eventEmitter.emit(files_1.EventType.FILE_CHANGES, watcher.toFileChangesEvent(events));
            }
        };
        return FileWatcher;
    }());
    exports.FileWatcher = FileWatcher;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/files/node/fileService", ["require", "exports", 'path', 'fs', 'os', 'crypto', 'assert', 'vs/platform/files/common/files', 'vs/base/common/strings', 'vs/base/common/arrays', 'vs/base/common/mime', 'vs/base/common/paths', 'vs/base/common/winjs.base', 'vs/base/common/types', 'vs/base/common/objects', 'vs/base/node/extfs', 'vs/base/common/async', 'vs/base/common/uri', 'vs/nls!vs/workbench/services/files/node/fileService', 'vs/base/node/pfs', 'vs/base/node/encoding', 'vs/base/node/mime', 'vs/base/node/flow', 'vs/workbench/services/files/node/watcher/unix/watcherService', 'vs/workbench/services/files/node/watcher/win32/watcherService', 'vs/workbench/services/files/node/watcher/common'], function (require, exports, paths, fs, os, crypto, assert, files, strings, arrays, baseMime, basePaths, winjs_base_1, types, objects, extfs, async_1, uri_1, nls, pfs, encoding, mime, flow, watcherService_1, watcherService_2, common_1) {
    'use strict';
    function etag(arg1, arg2) {
        var size;
        var mtime;
        if (typeof arg2 === 'number') {
            size = arg1;
            mtime = arg2;
        }
        else {
            size = arg1.size;
            mtime = arg1.mtime.getTime();
        }
        return '"' + crypto.createHash('sha1').update(String(size) + String(mtime)).digest('hex') + '"';
    }
    var FileService = (function () {
        function FileService(basePath, eventEmitter, options) {
            this.serviceId = files.IFileService;
            this.basePath = basePath ? paths.normalize(basePath) : void 0;
            if (this.basePath && this.basePath.indexOf('\\\\') === 0 && strings.endsWith(this.basePath, paths.sep)) {
                // for some weird reason, node adds a trailing slash to UNC paths
                // we never ever want trailing slashes as our base path unless
                // someone opens root ("/").
                // See also https://github.com/nodejs/io.js/issues/1765
                this.basePath = strings.rtrim(this.basePath, paths.sep);
            }
            if (this.basePath && !paths.isAbsolute(basePath)) {
                throw new Error('basePath has to be an absolute path');
            }
            this.options = options || Object.create(null);
            this.eventEmitter = eventEmitter;
            this.tmpPath = this.options.tmpDir || os.tmpdir();
            if (this.options && !this.options.errorLogger) {
                this.options.errorLogger = console.error;
            }
            if (this.basePath && !this.options.disableWatcher) {
                if (process.platform === 'win32') {
                    this.setupWin32WorkspaceWatching();
                }
                else {
                    this.setupUnixWorkspaceWatching();
                }
            }
            this.activeFileChangesWatchers = Object.create(null);
            this.fileChangesWatchDelayer = new async_1.ThrottledDelayer(FileService.FS_EVENT_DELAY);
            this.undeliveredRawFileChangesEvents = [];
        }
        FileService.prototype.updateOptions = function (options) {
            if (options) {
                objects.mixin(this.options, options); // overwrite current options
            }
        };
        FileService.prototype.setupWin32WorkspaceWatching = function () {
            this.workspaceWatcherToDispose = new watcherService_2.FileWatcher(this.basePath, this.options.watcherIgnoredPatterns, this.eventEmitter, this.options.errorLogger, this.options.verboseLogging).startWatching();
        };
        FileService.prototype.setupUnixWorkspaceWatching = function () {
            this.workspaceWatcherToDispose = new watcherService_1.FileWatcher(this.basePath, this.options.watcherIgnoredPatterns, this.eventEmitter, this.options.errorLogger, this.options.verboseLogging).startWatching();
        };
        FileService.prototype.resolveFile = function (resource, options) {
            return this.resolve(resource, options);
        };
        FileService.prototype.resolveContent = function (resource, options) {
            var _this = this;
            var absolutePath = this.toAbsolutePath(resource);
            // 1.) detect mimes
            return async_1.nfcall(mime.detectMimesFromFile, absolutePath).then(function (detected) {
                var isText = detected.mimes.indexOf(baseMime.MIME_BINARY) === -1;
                // Return error early if client only accepts text and this is not text
                if (options && options.acceptTextOnly && !isText) {
                    return winjs_base_1.TPromise.wrapError({
                        message: nls.localize(0, null),
                        fileOperationResult: files.FileOperationResult.FILE_IS_BINARY
                    });
                }
                var preferredEncoding;
                if (options && options.encoding) {
                    if (detected.encoding === encoding.UTF8 && options.encoding === encoding.UTF8) {
                        preferredEncoding = encoding.UTF8_with_bom; // indicate the file has BOM if we are to resolve with UTF 8
                    }
                    else {
                        preferredEncoding = options.encoding; // give passed in encoding highest priority
                    }
                }
                else if (detected.encoding) {
                    if (detected.encoding === encoding.UTF8) {
                        preferredEncoding = encoding.UTF8_with_bom; // if we detected UTF-8, it can only be because of a BOM
                    }
                    else {
                        preferredEncoding = detected.encoding;
                    }
                }
                else if (_this.options.encoding === encoding.UTF8_with_bom) {
                    preferredEncoding = encoding.UTF8; // if we did not detect UTF 8 BOM before, this can only be UTF 8 then
                }
                // 2.) get content
                return _this.resolveFileContent(resource, options && options.etag, preferredEncoding).then(function (content) {
                    // set our knowledge about the mime on the content obj
                    content.mime = detected.mimes.join(', ');
                    return content;
                });
            }, function (error) {
                // bubble up existing file operation results
                if (!types.isUndefinedOrNull(error.fileOperationResult)) {
                    return winjs_base_1.TPromise.wrapError(error);
                }
                // on error check if the file does not exist or is a folder and return with proper error result
                return pfs.exists(absolutePath).then(function (exists) {
                    // Return if file not found
                    if (!exists) {
                        return winjs_base_1.TPromise.wrapError({
                            message: nls.localize(1, null, absolutePath),
                            fileOperationResult: files.FileOperationResult.FILE_NOT_FOUND
                        });
                    }
                    // Otherwise check for file being a folder?
                    return pfs.stat(absolutePath).then(function (stat) {
                        if (stat.isDirectory()) {
                            return winjs_base_1.TPromise.wrapError({
                                message: nls.localize(2, null, absolutePath),
                                fileOperationResult: files.FileOperationResult.FILE_IS_DIRECTORY
                            });
                        }
                        // otherwise just give up
                        return winjs_base_1.TPromise.wrapError(error);
                    });
                });
            });
        };
        FileService.prototype.resolveContents = function (resources) {
            var _this = this;
            var limiter = new async_1.Limiter(FileService.MAX_DEGREE_OF_PARALLEL_FS_OPS);
            var contentPromises = [];
            resources.forEach(function (resource) {
                contentPromises.push(limiter.queue(function () { return _this.resolveFileContent(resource).then(function (content) { return content; }, function (error) { return winjs_base_1.TPromise.as(null /* ignore errors gracefully */); }); }));
            });
            return winjs_base_1.TPromise.join(contentPromises).then(function (contents) {
                return arrays.coalesce(contents);
            });
        };
        FileService.prototype.updateContent = function (resource, value, options) {
            var _this = this;
            if (options === void 0) { options = Object.create(null); }
            var absolutePath = this.toAbsolutePath(resource);
            // 1.) check file
            return this.checkFile(absolutePath, options).then(function (exists) {
                var createParentsPromise;
                if (exists) {
                    createParentsPromise = winjs_base_1.TPromise.as(null);
                }
                else {
                    createParentsPromise = pfs.mkdirp(paths.dirname(absolutePath));
                }
                // 2.) create parents as needed
                return createParentsPromise.then(function () {
                    var encodingToWrite = _this.getEncoding(resource, options.encoding);
                    var addBomPromise = winjs_base_1.TPromise.as(false);
                    // UTF_16 BE and LE as well as UTF_8 with BOM always have a BOM
                    if (encodingToWrite === encoding.UTF16be || encodingToWrite === encoding.UTF16le || encodingToWrite === encoding.UTF8_with_bom) {
                        addBomPromise = winjs_base_1.TPromise.as(true);
                    }
                    else if (exists && encodingToWrite === encoding.UTF8) {
                        if (options.overwriteEncoding) {
                            addBomPromise = winjs_base_1.TPromise.as(false); // if we are to overwrite the encoding, we do not preserve it if found
                        }
                        else {
                            addBomPromise = async_1.nfcall(encoding.detectEncodingByBOM, absolutePath).then(function (enc) { return enc === encoding.UTF8; }); // otherwise preserve it if found
                        }
                    }
                    // 3.) check to add UTF BOM
                    return addBomPromise.then(function (addBom) {
                        var writeFilePromise;
                        // Write fast if we do UTF 8 without BOM
                        if (!addBom && encodingToWrite === encoding.UTF8) {
                            writeFilePromise = pfs.writeFile(absolutePath, value, encoding.UTF8);
                        }
                        else {
                            var encoded = encoding.encode(value, encodingToWrite, { addBOM: addBom });
                            writeFilePromise = pfs.writeFile(absolutePath, encoded);
                        }
                        // 4.) set contents
                        return writeFilePromise.then(function () {
                            // 5.) resolve
                            return _this.resolve(resource);
                        });
                    });
                });
            });
        };
        FileService.prototype.createFile = function (resource, content) {
            if (content === void 0) { content = ''; }
            return this.updateContent(resource, content);
        };
        FileService.prototype.createFolder = function (resource) {
            var _this = this;
            // 1.) create folder
            var absolutePath = this.toAbsolutePath(resource);
            return pfs.mkdirp(absolutePath).then(function () {
                // 2.) resolve
                return _this.resolve(resource);
            });
        };
        FileService.prototype.rename = function (resource, newName) {
            var newPath = paths.join(paths.dirname(resource.fsPath), newName);
            return this.moveFile(resource, uri_1.default.file(newPath));
        };
        FileService.prototype.moveFile = function (source, target, overwrite) {
            return this.moveOrCopyFile(source, target, false, overwrite);
        };
        FileService.prototype.copyFile = function (source, target, overwrite) {
            return this.moveOrCopyFile(source, target, true, overwrite);
        };
        FileService.prototype.moveOrCopyFile = function (source, target, keepCopy, overwrite) {
            var _this = this;
            var sourcePath = this.toAbsolutePath(source);
            var targetPath = this.toAbsolutePath(target);
            // 1.) move / copy
            return this.doMoveOrCopyFile(sourcePath, targetPath, keepCopy, overwrite).then(function () {
                // 2.) resolve
                return _this.resolve(target);
            });
        };
        FileService.prototype.doMoveOrCopyFile = function (sourcePath, targetPath, keepCopy, overwrite) {
            var _this = this;
            // 1.) check if target exists
            return pfs.exists(targetPath).then(function (exists) {
                var isCaseRename = sourcePath.toLowerCase() === targetPath.toLowerCase();
                // Return early with conflict if target exists and we are not told to overwrite
                if (exists && !isCaseRename && !overwrite) {
                    return winjs_base_1.TPromise.wrapError({
                        fileOperationResult: files.FileOperationResult.FILE_MOVE_CONFLICT
                    });
                }
                // 2.) make sure target is deleted before we move/copy unless this is a case rename of the same file
                var deleteTargetPromise = winjs_base_1.TPromise.as(null);
                if (exists && !isCaseRename) {
                    if (basePaths.isEqualOrParent(sourcePath, targetPath)) {
                        return winjs_base_1.TPromise.wrapError(nls.localize(3, null)); // catch this corner case!
                    }
                    deleteTargetPromise = _this.del(uri_1.default.file(targetPath));
                }
                return deleteTargetPromise.then(function () {
                    // 3.) make sure parents exists
                    return pfs.mkdirp(paths.dirname(targetPath)).then(function () {
                        // 4.) copy/move
                        if (keepCopy) {
                            return async_1.nfcall(extfs.copy, sourcePath, targetPath);
                        }
                        else {
                            return async_1.nfcall(extfs.mv, sourcePath, targetPath);
                        }
                    }).then(function () { return exists; });
                });
            });
        };
        FileService.prototype.importFile = function (source, targetFolder) {
            var _this = this;
            var sourcePath = this.toAbsolutePath(source);
            var targetResource = uri_1.default.file(paths.join(targetFolder.fsPath, paths.basename(source.fsPath)));
            var targetPath = this.toAbsolutePath(targetResource);
            // 1.) resolve
            return pfs.stat(sourcePath).then(function (stat) {
                if (stat.isDirectory()) {
                    return winjs_base_1.TPromise.wrapError(nls.localize(4, null)); // for now we do not allow to import a folder into a workspace
                }
                // 2.) copy
                return _this.doMoveOrCopyFile(sourcePath, targetPath, true, true).then(function (exists) {
                    // 3.) resolve
                    return _this.resolve(targetResource).then(function (stat) { return { isNew: !exists, stat: stat }; });
                });
            });
        };
        FileService.prototype.del = function (resource) {
            var absolutePath = this.toAbsolutePath(resource);
            return async_1.nfcall(extfs.del, absolutePath, this.tmpPath);
        };
        // Helpers
        FileService.prototype.toAbsolutePath = function (arg1) {
            var resource;
            if (arg1 instanceof uri_1.default) {
                resource = arg1;
            }
            else {
                resource = arg1.resource;
            }
            assert.ok(resource && resource.scheme === 'file', 'Invalid resource: ' + resource);
            return paths.normalize(resource.fsPath);
        };
        FileService.prototype.resolve = function (resource, options) {
            if (options === void 0) { options = Object.create(null); }
            return this.toStatResolver(resource)
                .then(function (model) { return model.resolve(options); });
        };
        FileService.prototype.toStatResolver = function (resource) {
            var _this = this;
            var absolutePath = this.toAbsolutePath(resource);
            return pfs.stat(absolutePath).then(function (stat) {
                return new StatResolver(resource, stat.isDirectory(), stat.mtime.getTime(), stat.size, _this.options.verboseLogging);
            });
        };
        FileService.prototype.resolveFileContent = function (resource, etag, enc) {
            var _this = this;
            var absolutePath = this.toAbsolutePath(resource);
            // 1.) stat
            return this.resolve(resource).then(function (model) {
                // Return early if file not modified since
                if (etag && etag === model.etag) {
                    return winjs_base_1.TPromise.wrapError({
                        fileOperationResult: files.FileOperationResult.FILE_NOT_MODIFIED_SINCE
                    });
                }
                // Return early if file is too large to load
                if (types.isNumber(model.size) && model.size > files.MAX_FILE_SIZE) {
                    return winjs_base_1.TPromise.wrapError({
                        fileOperationResult: files.FileOperationResult.FILE_TOO_LARGE
                    });
                }
                // 2.) read contents
                return new winjs_base_1.TPromise(function (c, e) {
                    var done = false;
                    var chunks = [];
                    var fileEncoding = _this.getEncoding(model.resource, enc);
                    var reader = fs.createReadStream(absolutePath).pipe(encoding.decodeStream(fileEncoding)); // decode takes care of stripping any BOMs from the file content
                    reader.on('data', function (buf) {
                        chunks.push(buf);
                    });
                    reader.on('error', function (error) {
                        if (!done) {
                            done = true;
                            e(error);
                        }
                    });
                    reader.on('end', function () {
                        var content = model;
                        content.value = chunks.join('');
                        content.encoding = fileEncoding; // make sure to store the encoding in the model to restore it later when writing
                        if (!done) {
                            done = true;
                            c(content);
                        }
                    });
                });
            });
        };
        FileService.prototype.getEncoding = function (resource, preferredEncoding) {
            var fileEncoding;
            var override = this.getEncodingOverride(resource);
            if (override) {
                fileEncoding = override;
            }
            else if (preferredEncoding) {
                fileEncoding = preferredEncoding;
            }
            else {
                fileEncoding = this.options.encoding;
            }
            if (!fileEncoding || !encoding.encodingExists(fileEncoding)) {
                fileEncoding = encoding.UTF8; // the default is UTF 8
            }
            return fileEncoding;
        };
        FileService.prototype.getEncodingOverride = function (resource) {
            if (resource && this.options.encodingOverride && this.options.encodingOverride.length) {
                for (var i = 0; i < this.options.encodingOverride.length; i++) {
                    var override = this.options.encodingOverride[i];
                    // check if the resource is a child of the resource with override and use
                    // the provided encoding in that case
                    if (resource.toString().indexOf(override.resource.toString() + '/') === 0) {
                        return override.encoding;
                    }
                }
            }
            return null;
        };
        FileService.prototype.checkFile = function (absolutePath, options) {
            return pfs.exists(absolutePath).then(function (exists) {
                if (exists) {
                    return pfs.stat(absolutePath).then(function (stat) {
                        if (stat.isDirectory()) {
                            return winjs_base_1.TPromise.wrapError(new Error('Expected file is actually a directory'));
                        }
                        // Dirty write prevention
                        if (typeof options.mtime === 'number' && typeof options.etag === 'string' && options.mtime < stat.mtime.getTime()) {
                            // Find out if content length has changed
                            if (options.etag !== etag(stat.size, options.mtime)) {
                                return winjs_base_1.TPromise.wrapError({
                                    message: 'File Modified Since',
                                    fileOperationResult: files.FileOperationResult.FILE_MODIFIED_SINCE
                                });
                            }
                        }
                        var mode = stat.mode;
                        var readonly = !(mode & 128);
                        // Throw if file is readonly and we are not instructed to overwrite
                        if (readonly && !options.overwriteReadonly) {
                            return winjs_base_1.TPromise.wrapError({
                                message: nls.localize(5, null),
                                fileOperationResult: files.FileOperationResult.FILE_READ_ONLY
                            });
                        }
                        if (readonly) {
                            mode = mode | 128;
                            return pfs.chmod(absolutePath, mode).then(function () { return exists; });
                        }
                        return winjs_base_1.TPromise.as(exists);
                    });
                }
                return winjs_base_1.TPromise.as(exists);
            });
        };
        FileService.prototype.watchFileChanges = function (resource) {
            var _this = this;
            assert.ok(resource && resource.scheme === 'file', 'Invalid resource for watching: ' + resource);
            var fsPath = resource.fsPath;
            // Create or get watcher for provided path
            var watcher = this.activeFileChangesWatchers[resource.toString()];
            if (!watcher) {
                try {
                    watcher = fs.watch(fsPath); // will be persistent but not recursive
                }
                catch (error) {
                    // the path might not exist anymore, ignore this error and return
                    return;
                }
                this.activeFileChangesWatchers[resource.toString()] = watcher;
                // eventType is either 'rename' or 'change'
                watcher.on('change', function (eventType) {
                    if (eventType !== 'change') {
                        return; // only care about changes for now ('rename' is not reliable and can be send even if the file is still there with some tools)
                    }
                    // add to bucket of undelivered events
                    _this.undeliveredRawFileChangesEvents.push({
                        type: files.FileChangeType.UPDATED,
                        path: fsPath
                    });
                    // handle emit through delayer to accommodate for bulk changes
                    _this.fileChangesWatchDelayer.trigger(function () {
                        var buffer = _this.undeliveredRawFileChangesEvents;
                        _this.undeliveredRawFileChangesEvents = [];
                        // Normalize
                        var normalizedEvents = common_1.normalize(buffer);
                        // Emit
                        _this.eventEmitter.emit(files.EventType.FILE_CHANGES, common_1.toFileChangesEvent(normalizedEvents));
                        return winjs_base_1.TPromise.as(null);
                    });
                });
            }
        };
        FileService.prototype.unwatchFileChanges = function (arg1) {
            var resource = (typeof arg1 === 'string') ? uri_1.default.parse(arg1) : arg1;
            var watcher = this.activeFileChangesWatchers[resource.toString()];
            if (watcher) {
                watcher.close();
                delete this.activeFileChangesWatchers[resource.toString()];
            }
        };
        FileService.prototype.dispose = function () {
            if (this.workspaceWatcherToDispose) {
                this.workspaceWatcherToDispose();
                this.workspaceWatcherToDispose = null;
            }
            for (var key in this.activeFileChangesWatchers) {
                var watcher = this.activeFileChangesWatchers[key];
                watcher.close();
            }
            this.activeFileChangesWatchers = Object.create(null);
        };
        FileService.FS_EVENT_DELAY = 50; // aggregate and only emit events when changes have stopped for this duration (in ms)
        FileService.MAX_DEGREE_OF_PARALLEL_FS_OPS = 10; // degree of parallel fs calls that we accept at the same time
        return FileService;
    }());
    exports.FileService = FileService;
    var StatResolver = (function () {
        function StatResolver(resource, isDirectory, mtime, size, verboseLogging) {
            assert.ok(resource && resource.scheme === 'file', 'Invalid resource: ' + resource);
            this.resource = resource;
            this.isDirectory = isDirectory;
            this.mtime = mtime;
            this.name = paths.basename(resource.fsPath);
            this.mime = !this.isDirectory ? baseMime.guessMimeTypes(resource.fsPath).join(', ') : null;
            this.etag = etag(size, mtime);
            this.size = size;
            this.verboseLogging = verboseLogging;
        }
        StatResolver.prototype.resolve = function (options) {
            var _this = this;
            // General Data
            var fileStat = {
                resource: this.resource,
                isDirectory: this.isDirectory,
                hasChildren: undefined,
                name: this.name,
                etag: this.etag,
                size: this.size,
                mtime: this.mtime,
                mime: this.mime
            };
            // File Specific Data
            if (!this.isDirectory) {
                return winjs_base_1.TPromise.as(fileStat);
            }
            else {
                // Convert the paths from options.resolveTo to absolute paths
                var absoluteTargetPaths_1 = null;
                if (options && options.resolveTo) {
                    absoluteTargetPaths_1 = [];
                    options.resolveTo.forEach(function (resource) {
                        absoluteTargetPaths_1.push(resource.fsPath);
                    });
                }
                return new winjs_base_1.TPromise(function (c, e) {
                    // Load children
                    _this.resolveChildren(_this.resource.fsPath, absoluteTargetPaths_1, options && options.resolveSingleChildDescendants, function (children) {
                        children = arrays.coalesce(children); // we don't want those null children (could be permission denied when reading a child)
                        fileStat.hasChildren = children && children.length > 0;
                        fileStat.children = children || [];
                        c(fileStat);
                    });
                });
            }
        };
        StatResolver.prototype.resolveChildren = function (absolutePath, absoluteTargetPaths, resolveSingleChildDescendants, callback) {
            var _this = this;
            extfs.readdir(absolutePath, function (error, files) {
                if (error) {
                    if (_this.verboseLogging) {
                        console.error(error);
                    }
                    return callback(null); // return - we might not have permissions to read the folder
                }
                // for each file in the folder
                flow.parallel(files, function (file, clb) {
                    var fileResource = uri_1.default.file(paths.resolve(absolutePath, file));
                    var fileStat;
                    var $this = _this;
                    flow.sequence(function onError(error) {
                        if ($this.verboseLogging) {
                            console.error(error);
                        }
                        clb(null, null); // return - we might not have permissions to read the folder or stat the file
                    }, function stat() {
                        fs.stat(fileResource.fsPath, this);
                    }, function countChildren(fsstat) {
                        var _this = this;
                        fileStat = fsstat;
                        if (fileStat.isDirectory()) {
                            extfs.readdir(fileResource.fsPath, function (error, result) {
                                _this(null, result ? result.length : 0);
                            });
                        }
                        else {
                            this(null, 0);
                        }
                    }, function resolve(childCount) {
                        var childStat = {
                            resource: fileResource,
                            isDirectory: fileStat.isDirectory(),
                            hasChildren: childCount > 0,
                            name: file,
                            mtime: fileStat.mtime.getTime(),
                            etag: etag(fileStat),
                            size: fileStat.size,
                            mime: !fileStat.isDirectory() ? baseMime.guessMimeTypes(fileResource.fsPath).join(', ') : undefined
                        };
                        // Return early for files
                        if (!fileStat.isDirectory()) {
                            return clb(null, childStat);
                        }
                        // Handle Folder
                        var resolveFolderChildren = false;
                        if (files.length === 1 && resolveSingleChildDescendants) {
                            resolveFolderChildren = true;
                        }
                        else if (childCount > 0 && absoluteTargetPaths && absoluteTargetPaths.some(function (targetPath) { return basePaths.isEqualOrParent(targetPath, fileResource.fsPath); })) {
                            resolveFolderChildren = true;
                        }
                        // Continue resolving children based on condition
                        if (resolveFolderChildren) {
                            $this.resolveChildren(fileResource.fsPath, absoluteTargetPaths, resolveSingleChildDescendants, function (children) {
                                children = arrays.coalesce(children); // we don't want those null children
                                childStat.hasChildren = children && children.length > 0;
                                childStat.children = children || [];
                                clb(null, childStat);
                            });
                        }
                        else {
                            clb(null, childStat);
                        }
                    });
                }, function (errors, result) {
                    callback(result);
                });
            });
        };
        return StatResolver;
    }());
    exports.StatResolver = StatResolver;
});

define("vs/workbench/services/files/electron-browser/fileService", ["require", "exports", 'vs/nls!vs/workbench/services/files/electron-browser/fileService', 'vs/base/common/winjs.base', 'vs/base/common/paths', 'vs/base/node/encoding', 'vs/base/common/errors', 'vs/base/common/strings', 'vs/base/common/uri', 'vs/base/common/timer', 'vs/platform/files/common/files', 'vs/workbench/services/files/node/fileService', 'vs/platform/configuration/common/configuration', 'electron'], function (require, exports, nls, winjs_base_1, paths, encoding, errors, strings, uri_1, timer, files, fileService_1, configuration_1, electron_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FileService = (function () {
        function FileService(configurationService, eventService, contextService) {
            var _this = this;
            this.configurationService = configurationService;
            this.eventService = eventService;
            this.contextService = contextService;
            this.serviceId = files.IFileService;
            // Init raw implementation
            this.raw = this.configurationService.loadConfiguration().then(function (configuration) {
                // adjust encodings (TODO@Ben knowledge on settings location ('.vscode') is hardcoded)
                var encodingOverride = [];
                encodingOverride.push({ resource: uri_1.default.file(_this.contextService.getConfiguration().env.appSettingsHome), encoding: encoding.UTF8 });
                if (_this.contextService.getWorkspace()) {
                    encodingOverride.push({ resource: uri_1.default.file(paths.join(_this.contextService.getWorkspace().resource.fsPath, '.vscode')), encoding: encoding.UTF8 });
                }
                var watcherIgnoredPatterns = [];
                if (configuration.files && configuration.files.watcherExclude) {
                    watcherIgnoredPatterns = Object.keys(configuration.files.watcherExclude).filter(function (k) { return !!configuration.files.watcherExclude[k]; });
                }
                // build config
                var fileServiceConfig = {
                    errorLogger: function (msg) { return errors.onUnexpectedError(msg); },
                    encoding: configuration.files && configuration.files.encoding,
                    encodingOverride: encodingOverride,
                    watcherIgnoredPatterns: watcherIgnoredPatterns,
                    verboseLogging: _this.contextService.getConfiguration().env.verboseLogging
                };
                // create service
                var workspace = _this.contextService.getWorkspace();
                return new fileService_1.FileService(workspace ? workspace.resource.fsPath : void 0, _this.eventService, fileServiceConfig);
            });
            // Listeners
            this.raw.done(function (raw) {
                _this.registerListeners();
            }, errors.onUnexpectedError);
        }
        FileService.prototype.registerListeners = function () {
            var _this = this;
            // Config Changes
            this.configurationChangeListenerUnbind = this.configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) { return _this.onConfigurationChange(e.config); });
        };
        FileService.prototype.onConfigurationChange = function (configuration) {
            this.updateOptions(configuration.files);
        };
        FileService.prototype.updateOptions = function (options) {
            this.raw.done(function (raw) {
                raw.updateOptions(options);
            }, errors.onUnexpectedError);
        };
        FileService.prototype.resolveFile = function (resource, options) {
            return this.raw.then(function (raw) {
                return raw.resolveFile(resource, options);
            });
        };
        FileService.prototype.resolveContent = function (resource, options) {
            var contentId = resource.toString();
            var timerEvent = timer.start(timer.Topic.WORKBENCH, strings.format('Load {0}', contentId));
            return this.raw.then(function (raw) {
                return raw.resolveContent(resource, options).then(function (result) {
                    timerEvent.stop();
                    return result;
                });
            });
        };
        FileService.prototype.resolveContents = function (resources) {
            return this.raw.then(function (raw) {
                return raw.resolveContents(resources);
            });
        };
        FileService.prototype.updateContent = function (resource, value, options) {
            var timerEvent = timer.start(timer.Topic.WORKBENCH, strings.format('Save {0}', resource.toString()));
            return this.raw.then(function (raw) {
                return raw.updateContent(resource, value, options).then(function (result) {
                    timerEvent.stop();
                    return result;
                }, function (error) {
                    timerEvent.stop();
                    return winjs_base_1.TPromise.wrapError(error);
                });
            });
        };
        FileService.prototype.moveFile = function (source, target, overwrite) {
            return this.raw.then(function (raw) {
                return raw.moveFile(source, target, overwrite);
            });
        };
        FileService.prototype.copyFile = function (source, target, overwrite) {
            return this.raw.then(function (raw) {
                return raw.copyFile(source, target, overwrite);
            });
        };
        FileService.prototype.createFile = function (resource, content) {
            return this.raw.then(function (raw) {
                return raw.createFile(resource, content);
            });
        };
        FileService.prototype.createFolder = function (resource) {
            return this.raw.then(function (raw) {
                return raw.createFolder(resource);
            });
        };
        FileService.prototype.rename = function (resource, newName) {
            return this.raw.then(function (raw) {
                return raw.rename(resource, newName);
            });
        };
        FileService.prototype.del = function (resource, useTrash) {
            if (useTrash) {
                return this.doMoveItemToTrash(resource);
            }
            return this.raw.then(function (raw) {
                return raw.del(resource);
            });
        };
        FileService.prototype.doMoveItemToTrash = function (resource) {
            var workspace = this.contextService.getWorkspace();
            if (!workspace) {
                return winjs_base_1.TPromise.wrapError('Need a workspace to use this');
            }
            var absolutePath = resource.fsPath;
            var result = electron_1.shell.moveItemToTrash(absolutePath);
            if (!result) {
                return winjs_base_1.TPromise.wrapError(new Error(nls.localize(0, null, paths.basename(absolutePath))));
            }
            return winjs_base_1.TPromise.as(null);
        };
        FileService.prototype.importFile = function (source, targetFolder) {
            return this.raw.then(function (raw) {
                return raw.importFile(source, targetFolder).then(function (result) {
                    return {
                        isNew: result && result.isNew,
                        stat: result && result.stat
                    };
                });
            });
        };
        FileService.prototype.watchFileChanges = function (resource) {
            if (!resource) {
                return;
            }
            if (resource.scheme !== 'file') {
                return; // only support files
            }
            // return early if the resource is inside the workspace for which we have another watcher in place
            if (this.contextService.isInsideWorkspace(resource)) {
                return;
            }
            this.raw.then(function (raw) {
                raw.watchFileChanges(resource);
            });
        };
        FileService.prototype.unwatchFileChanges = function (arg1) {
            this.raw.then(function (raw) {
                raw.unwatchFileChanges(arg1);
            });
        };
        FileService.prototype.dispose = function () {
            // Listeners
            if (this.configurationChangeListenerUnbind) {
                this.configurationChangeListenerUnbind();
                this.configurationChangeListenerUnbind = null;
            }
            // Dispose service
            this.raw.done(function (raw) { return raw.dispose(); });
        };
        return FileService;
    }());
    exports.FileService = FileService;
});

define("vs/workbench/services/history/common/history", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IHistoryService = instantiation_1.createDecorator('historyService');
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/keybinding/electron-browser/nativeKeymap", ["require", "exports", 'native-keymap', 'vs/base/common/keyCodes', 'vs/base/browser/keyboardEvent', 'vs/base/common/platform'], function (require, exports, nativeKeymap, keyCodes_1, keyboardEvent_1, Platform) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var getNativeKeymap = (function () {
        var called = false;
        var result;
        return function getNativeKeymap() {
            if (!called) {
                called = true;
                result = nativeKeymap.getKeyMap();
            }
            return result;
        };
    })();
    // See https://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx
    // See https://github.com/Microsoft/node-native-keymap/blob/master/deps/chromium/keyboard_codes_win.h
    var NATIVE_KEY_CODE_TO_KEY_CODE = {
        VKEY_BACK: keyCodes_1.KeyCode.Backspace,
        VKEY_TAB: keyCodes_1.KeyCode.Tab,
        VKEY_CLEAR: keyCodes_1.KeyCode.Unknown,
        VKEY_RETURN: keyCodes_1.KeyCode.Enter,
        VKEY_SHIFT: keyCodes_1.KeyCode.Shift,
        VKEY_CONTROL: keyCodes_1.KeyCode.Ctrl,
        VKEY_MENU: keyCodes_1.KeyCode.Alt,
        VKEY_PAUSE: keyCodes_1.KeyCode.PauseBreak,
        VKEY_CAPITAL: keyCodes_1.KeyCode.CapsLock,
        VKEY_KANA: keyCodes_1.KeyCode.Unknown,
        VKEY_HANGUL: keyCodes_1.KeyCode.Unknown,
        VKEY_JUNJA: keyCodes_1.KeyCode.Unknown,
        VKEY_FINAL: keyCodes_1.KeyCode.Unknown,
        VKEY_HANJA: keyCodes_1.KeyCode.Unknown,
        VKEY_KANJI: keyCodes_1.KeyCode.Unknown,
        VKEY_ESCAPE: keyCodes_1.KeyCode.Escape,
        VKEY_CONVERT: keyCodes_1.KeyCode.Unknown,
        VKEY_NONCONVERT: keyCodes_1.KeyCode.Unknown,
        VKEY_ACCEPT: keyCodes_1.KeyCode.Unknown,
        VKEY_MODECHANGE: keyCodes_1.KeyCode.Unknown,
        VKEY_SPACE: keyCodes_1.KeyCode.Space,
        VKEY_PRIOR: keyCodes_1.KeyCode.PageUp,
        VKEY_NEXT: keyCodes_1.KeyCode.PageDown,
        VKEY_END: keyCodes_1.KeyCode.End,
        VKEY_HOME: keyCodes_1.KeyCode.Home,
        VKEY_LEFT: keyCodes_1.KeyCode.LeftArrow,
        VKEY_UP: keyCodes_1.KeyCode.UpArrow,
        VKEY_RIGHT: keyCodes_1.KeyCode.RightArrow,
        VKEY_DOWN: keyCodes_1.KeyCode.DownArrow,
        VKEY_SELECT: keyCodes_1.KeyCode.Unknown,
        VKEY_PRINT: keyCodes_1.KeyCode.Unknown,
        VKEY_EXECUTE: keyCodes_1.KeyCode.Unknown,
        VKEY_SNAPSHOT: keyCodes_1.KeyCode.Unknown,
        VKEY_INSERT: keyCodes_1.KeyCode.Insert,
        VKEY_DELETE: keyCodes_1.KeyCode.Delete,
        VKEY_HELP: keyCodes_1.KeyCode.Unknown,
        VKEY_0: keyCodes_1.KeyCode.KEY_0,
        VKEY_1: keyCodes_1.KeyCode.KEY_1,
        VKEY_2: keyCodes_1.KeyCode.KEY_2,
        VKEY_3: keyCodes_1.KeyCode.KEY_3,
        VKEY_4: keyCodes_1.KeyCode.KEY_4,
        VKEY_5: keyCodes_1.KeyCode.KEY_5,
        VKEY_6: keyCodes_1.KeyCode.KEY_6,
        VKEY_7: keyCodes_1.KeyCode.KEY_7,
        VKEY_8: keyCodes_1.KeyCode.KEY_8,
        VKEY_9: keyCodes_1.KeyCode.KEY_9,
        VKEY_A: keyCodes_1.KeyCode.KEY_A,
        VKEY_B: keyCodes_1.KeyCode.KEY_B,
        VKEY_C: keyCodes_1.KeyCode.KEY_C,
        VKEY_D: keyCodes_1.KeyCode.KEY_D,
        VKEY_E: keyCodes_1.KeyCode.KEY_E,
        VKEY_F: keyCodes_1.KeyCode.KEY_F,
        VKEY_G: keyCodes_1.KeyCode.KEY_G,
        VKEY_H: keyCodes_1.KeyCode.KEY_H,
        VKEY_I: keyCodes_1.KeyCode.KEY_I,
        VKEY_J: keyCodes_1.KeyCode.KEY_J,
        VKEY_K: keyCodes_1.KeyCode.KEY_K,
        VKEY_L: keyCodes_1.KeyCode.KEY_L,
        VKEY_M: keyCodes_1.KeyCode.KEY_M,
        VKEY_N: keyCodes_1.KeyCode.KEY_N,
        VKEY_O: keyCodes_1.KeyCode.KEY_O,
        VKEY_P: keyCodes_1.KeyCode.KEY_P,
        VKEY_Q: keyCodes_1.KeyCode.KEY_Q,
        VKEY_R: keyCodes_1.KeyCode.KEY_R,
        VKEY_S: keyCodes_1.KeyCode.KEY_S,
        VKEY_T: keyCodes_1.KeyCode.KEY_T,
        VKEY_U: keyCodes_1.KeyCode.KEY_U,
        VKEY_V: keyCodes_1.KeyCode.KEY_V,
        VKEY_W: keyCodes_1.KeyCode.KEY_W,
        VKEY_X: keyCodes_1.KeyCode.KEY_X,
        VKEY_Y: keyCodes_1.KeyCode.KEY_Y,
        VKEY_Z: keyCodes_1.KeyCode.KEY_Z,
        VKEY_LWIN: keyCodes_1.KeyCode.Meta,
        VKEY_COMMAND: keyCodes_1.KeyCode.Meta,
        VKEY_RWIN: keyCodes_1.KeyCode.Meta,
        VKEY_APPS: keyCodes_1.KeyCode.Unknown,
        VKEY_SLEEP: keyCodes_1.KeyCode.Unknown,
        VKEY_NUMPAD0: keyCodes_1.KeyCode.NUMPAD_0,
        VKEY_NUMPAD1: keyCodes_1.KeyCode.NUMPAD_1,
        VKEY_NUMPAD2: keyCodes_1.KeyCode.NUMPAD_2,
        VKEY_NUMPAD3: keyCodes_1.KeyCode.NUMPAD_3,
        VKEY_NUMPAD4: keyCodes_1.KeyCode.NUMPAD_4,
        VKEY_NUMPAD5: keyCodes_1.KeyCode.NUMPAD_5,
        VKEY_NUMPAD6: keyCodes_1.KeyCode.NUMPAD_6,
        VKEY_NUMPAD7: keyCodes_1.KeyCode.NUMPAD_7,
        VKEY_NUMPAD8: keyCodes_1.KeyCode.NUMPAD_8,
        VKEY_NUMPAD9: keyCodes_1.KeyCode.NUMPAD_9,
        VKEY_MULTIPLY: keyCodes_1.KeyCode.NUMPAD_MULTIPLY,
        VKEY_ADD: keyCodes_1.KeyCode.NUMPAD_ADD,
        VKEY_SEPARATOR: keyCodes_1.KeyCode.NUMPAD_SEPARATOR,
        VKEY_SUBTRACT: keyCodes_1.KeyCode.NUMPAD_SUBTRACT,
        VKEY_DECIMAL: keyCodes_1.KeyCode.NUMPAD_DECIMAL,
        VKEY_DIVIDE: keyCodes_1.KeyCode.NUMPAD_DIVIDE,
        VKEY_F1: keyCodes_1.KeyCode.F1,
        VKEY_F2: keyCodes_1.KeyCode.F2,
        VKEY_F3: keyCodes_1.KeyCode.F3,
        VKEY_F4: keyCodes_1.KeyCode.F4,
        VKEY_F5: keyCodes_1.KeyCode.F5,
        VKEY_F6: keyCodes_1.KeyCode.F6,
        VKEY_F7: keyCodes_1.KeyCode.F7,
        VKEY_F8: keyCodes_1.KeyCode.F8,
        VKEY_F9: keyCodes_1.KeyCode.F9,
        VKEY_F10: keyCodes_1.KeyCode.F10,
        VKEY_F11: keyCodes_1.KeyCode.F11,
        VKEY_F12: keyCodes_1.KeyCode.F12,
        VKEY_F13: keyCodes_1.KeyCode.F13,
        VKEY_F14: keyCodes_1.KeyCode.F14,
        VKEY_F15: keyCodes_1.KeyCode.F15,
        VKEY_F16: keyCodes_1.KeyCode.F16,
        VKEY_F17: keyCodes_1.KeyCode.F17,
        VKEY_F18: keyCodes_1.KeyCode.F18,
        VKEY_F19: keyCodes_1.KeyCode.F19,
        VKEY_F20: keyCodes_1.KeyCode.Unknown,
        VKEY_F21: keyCodes_1.KeyCode.Unknown,
        VKEY_F22: keyCodes_1.KeyCode.Unknown,
        VKEY_F23: keyCodes_1.KeyCode.Unknown,
        VKEY_F24: keyCodes_1.KeyCode.Unknown,
        VKEY_NUMLOCK: keyCodes_1.KeyCode.NumLock,
        VKEY_SCROLL: keyCodes_1.KeyCode.ScrollLock,
        VKEY_LSHIFT: keyCodes_1.KeyCode.Shift,
        VKEY_RSHIFT: keyCodes_1.KeyCode.Shift,
        VKEY_LCONTROL: keyCodes_1.KeyCode.Ctrl,
        VKEY_RCONTROL: keyCodes_1.KeyCode.Ctrl,
        VKEY_LMENU: keyCodes_1.KeyCode.Unknown,
        VKEY_RMENU: keyCodes_1.KeyCode.Unknown,
        VKEY_BROWSER_BACK: keyCodes_1.KeyCode.Unknown,
        VKEY_BROWSER_FORWARD: keyCodes_1.KeyCode.Unknown,
        VKEY_BROWSER_REFRESH: keyCodes_1.KeyCode.Unknown,
        VKEY_BROWSER_STOP: keyCodes_1.KeyCode.Unknown,
        VKEY_BROWSER_SEARCH: keyCodes_1.KeyCode.Unknown,
        VKEY_BROWSER_FAVORITES: keyCodes_1.KeyCode.Unknown,
        VKEY_BROWSER_HOME: keyCodes_1.KeyCode.Unknown,
        VKEY_VOLUME_MUTE: keyCodes_1.KeyCode.Unknown,
        VKEY_VOLUME_DOWN: keyCodes_1.KeyCode.Unknown,
        VKEY_VOLUME_UP: keyCodes_1.KeyCode.Unknown,
        VKEY_MEDIA_NEXT_TRACK: keyCodes_1.KeyCode.Unknown,
        VKEY_MEDIA_PREV_TRACK: keyCodes_1.KeyCode.Unknown,
        VKEY_MEDIA_STOP: keyCodes_1.KeyCode.Unknown,
        VKEY_MEDIA_PLAY_PAUSE: keyCodes_1.KeyCode.Unknown,
        VKEY_MEDIA_LAUNCH_MAIL: keyCodes_1.KeyCode.Unknown,
        VKEY_MEDIA_LAUNCH_MEDIA_SELECT: keyCodes_1.KeyCode.Unknown,
        VKEY_MEDIA_LAUNCH_APP1: keyCodes_1.KeyCode.Unknown,
        VKEY_MEDIA_LAUNCH_APP2: keyCodes_1.KeyCode.Unknown,
        VKEY_OEM_1: keyCodes_1.KeyCode.US_SEMICOLON,
        VKEY_OEM_PLUS: keyCodes_1.KeyCode.US_EQUAL,
        VKEY_OEM_COMMA: keyCodes_1.KeyCode.US_COMMA,
        VKEY_OEM_MINUS: keyCodes_1.KeyCode.US_MINUS,
        VKEY_OEM_PERIOD: keyCodes_1.KeyCode.US_DOT,
        VKEY_OEM_2: keyCodes_1.KeyCode.US_SLASH,
        VKEY_OEM_3: keyCodes_1.KeyCode.US_BACKTICK,
        VKEY_OEM_4: keyCodes_1.KeyCode.US_OPEN_SQUARE_BRACKET,
        VKEY_OEM_5: keyCodes_1.KeyCode.US_BACKSLASH,
        VKEY_OEM_6: keyCodes_1.KeyCode.US_CLOSE_SQUARE_BRACKET,
        VKEY_OEM_7: keyCodes_1.KeyCode.US_QUOTE,
        VKEY_OEM_8: keyCodes_1.KeyCode.OEM_8,
        VKEY_OEM_102: keyCodes_1.KeyCode.OEM_102,
        VKEY_PROCESSKEY: keyCodes_1.KeyCode.Unknown,
        VKEY_PACKET: keyCodes_1.KeyCode.Unknown,
        VKEY_DBE_SBCSCHAR: keyCodes_1.KeyCode.Unknown,
        VKEY_DBE_DBCSCHAR: keyCodes_1.KeyCode.Unknown,
        VKEY_ATTN: keyCodes_1.KeyCode.Unknown,
        VKEY_CRSEL: keyCodes_1.KeyCode.Unknown,
        VKEY_EXSEL: keyCodes_1.KeyCode.Unknown,
        VKEY_EREOF: keyCodes_1.KeyCode.Unknown,
        VKEY_PLAY: keyCodes_1.KeyCode.Unknown,
        VKEY_ZOOM: keyCodes_1.KeyCode.Unknown,
        VKEY_NONAME: keyCodes_1.KeyCode.Unknown,
        VKEY_PA1: keyCodes_1.KeyCode.Unknown,
        VKEY_OEM_CLEAR: keyCodes_1.KeyCode.Unknown,
        VKEY_UNKNOWN: keyCodes_1.KeyCode.Unknown,
        // Windows does not have a specific key code for AltGr. We use the unused
        // VK_OEM_AX to represent AltGr, matching the behaviour of Firefox on Linux.
        VKEY_ALTGR: keyCodes_1.KeyCode.Unknown,
    };
    var _b24_fixedVirtualKeyCodes = [
        { char: ';', virtualKeyCode: 186 },
        { char: ':', virtualKeyCode: 186 },
        { char: '=', virtualKeyCode: 187 },
        { char: '+', virtualKeyCode: 187 },
        { char: ',', virtualKeyCode: 188 },
        { char: '<', virtualKeyCode: 188 },
        { char: '-', virtualKeyCode: 189 },
        { char: '_', virtualKeyCode: 189 },
        { char: '.', virtualKeyCode: 190 },
        { char: '>', virtualKeyCode: 190 },
        { char: '/', virtualKeyCode: 191 },
        { char: '?', virtualKeyCode: 191 },
        { char: '`', virtualKeyCode: 192 },
        { char: '~', virtualKeyCode: 192 },
        { char: '[', virtualKeyCode: 219 },
        { char: '{', virtualKeyCode: 219 },
        { char: '\\', virtualKeyCode: 220 },
        { char: '|', virtualKeyCode: 220 },
        { char: ']', virtualKeyCode: 221 },
        { char: '}', virtualKeyCode: 221 },
        { char: '\'', virtualKeyCode: 222 },
        { char: '"', virtualKeyCode: 222 },
    ];
    var _b24_interestingChars = Object.create(null);
    _b24_fixedVirtualKeyCodes.forEach(function (el) { return _b24_interestingChars[el.char] = true; });
    var _b24_interestingVirtualKeyCodes = Object.create(null);
    _b24_fixedVirtualKeyCodes.forEach(function (el) { return _b24_interestingVirtualKeyCodes[el.virtualKeyCode] = true; });
    var _b24_getActualKeyCodeMap = (function () {
        var result = null;
        return function () {
            if (!result) {
                result = Object.create(null);
                var nativeMappings = getNativeKeymap();
                for (var i = 0, len = nativeMappings.length; i < len; i++) {
                    var nativeMapping = nativeMappings[i];
                    if (nativeMapping.value && _b24_interestingChars[nativeMapping.value]) {
                        // console.log(nativeMapping.value + " is made by " + nativeMapping.key_code);
                        var keyCode = NATIVE_KEY_CODE_TO_KEY_CODE[nativeMapping.key_code];
                        if (keyCode && keyCode !== keyCodes_1.KeyCode.Unknown) {
                            if (!result[nativeMapping.value] || result[nativeMapping.value] > keyCode) {
                                result[nativeMapping.value] = keyCode;
                            }
                        }
                    }
                    if (nativeMapping.withShift && _b24_interestingChars[nativeMapping.withShift]) {
                        // console.log(nativeMapping.withShift + " is made by " + nativeMapping.key_code);
                        var keyCode = NATIVE_KEY_CODE_TO_KEY_CODE[nativeMapping.key_code];
                        if (keyCode && keyCode !== keyCodes_1.KeyCode.Unknown) {
                            if (!result[nativeMapping.withShift] || result[nativeMapping.withShift] > keyCode) {
                                result[nativeMapping.withShift] = keyCode;
                            }
                        }
                    }
                }
            }
            return result;
        };
    })();
    keyboardEvent_1.setExtractKeyCode(function (e) {
        if (e.charCode) {
            // "keypress" events mostly
            var char = String.fromCharCode(e.charCode).toUpperCase();
            return keyCodes_1.KeyCode.fromString(char);
        }
        if (Platform.isMacintosh && _b24_interestingVirtualKeyCodes[e.keyCode] && typeof e.keyIdentifier === 'string') {
            var keyIdentifier = e.keyIdentifier;
            var strCharCode = keyIdentifier.substr(2);
            try {
                var charCode = parseInt(strCharCode, 16);
                var char = String.fromCharCode(charCode);
                var unfixMap = _b24_getActualKeyCodeMap();
                if (unfixMap[char]) {
                    return unfixMap[char];
                }
            }
            catch (err) {
            }
        }
        // _b24_getActualKeyCodeMap();
        // console.log('injected!!!');
        return keyboardEvent_1.lookupKeyCode(e);
    });
    var nativeAriaLabelProvider = null;
    function getNativeAriaLabelProvider() {
        if (!nativeAriaLabelProvider) {
            var remaps = getNativeLabelProviderRemaps();
            nativeAriaLabelProvider = new NativeAriaKeyLabelProvider(remaps);
        }
        return nativeAriaLabelProvider;
    }
    exports.getNativeAriaLabelProvider = getNativeAriaLabelProvider;
    var nativeLabelProvider = null;
    function getNativeLabelProvider() {
        if (!nativeLabelProvider) {
            var remaps = getNativeLabelProviderRemaps();
            if (Platform.isMacintosh) {
                nativeLabelProvider = new NativeMacUIKeyLabelProvider(remaps);
            }
            else {
                nativeLabelProvider = new NativeClassicUIKeyLabelProvider(remaps);
            }
        }
        return nativeLabelProvider;
    }
    exports.getNativeLabelProvider = getNativeLabelProvider;
    var nativeLabelRemaps = null;
    function getNativeLabelProviderRemaps() {
        if (!nativeLabelRemaps) {
            // See https://msdn.microsoft.com/en-us/library/windows/desktop/dd375731(v=vs.85).aspx
            // See https://github.com/Microsoft/node-native-keymap/blob/master/deps/chromium/keyboard_codes_win.h
            var interestingKeyCodes = {
                VKEY_OEM_1: true,
                VKEY_OEM_PLUS: true,
                VKEY_OEM_COMMA: true,
                VKEY_OEM_MINUS: true,
                VKEY_OEM_PERIOD: true,
                VKEY_OEM_2: true,
                VKEY_OEM_3: true,
                VKEY_OEM_4: true,
                VKEY_OEM_5: true,
                VKEY_OEM_6: true,
                VKEY_OEM_7: true,
                VKEY_OEM_8: true,
                VKEY_OEM_102: true,
            };
            nativeLabelRemaps = [];
            for (var i = 0, len = keyCodes_1.KeyCode.MAX_VALUE; i < len; i++) {
                nativeLabelRemaps[i] = null;
            }
            var nativeMappings = getNativeKeymap();
            var hadRemap = false;
            for (var i = 0, len = nativeMappings.length; i < len; i++) {
                var nativeMapping = nativeMappings[i];
                if (interestingKeyCodes[nativeMapping.key_code]) {
                    var newValue = nativeMapping.value || nativeMapping.withShift;
                    if (newValue.length > 0) {
                        hadRemap = true;
                        nativeLabelRemaps[NATIVE_KEY_CODE_TO_KEY_CODE[nativeMapping.key_code]] = newValue;
                    }
                    else {
                    }
                }
            }
            if (hadRemap) {
                for (var interestingKeyCode in interestingKeyCodes) {
                    if (interestingKeyCodes.hasOwnProperty(interestingKeyCode)) {
                        var keyCode = NATIVE_KEY_CODE_TO_KEY_CODE[interestingKeyCode];
                        nativeLabelRemaps[keyCode] = nativeLabelRemaps[keyCode] || '';
                    }
                }
            }
        }
        return nativeLabelRemaps;
    }
    var NativeMacUIKeyLabelProvider = (function (_super) {
        __extends(NativeMacUIKeyLabelProvider, _super);
        function NativeMacUIKeyLabelProvider(remaps) {
            _super.call(this);
            this.remaps = remaps;
        }
        NativeMacUIKeyLabelProvider.prototype.getLabelForKey = function (keyCode) {
            if (this.remaps[keyCode] !== null) {
                return this.remaps[keyCode];
            }
            return _super.prototype.getLabelForKey.call(this, keyCode);
        };
        return NativeMacUIKeyLabelProvider;
    }(keyCodes_1.MacUIKeyLabelProvider));
    var NativeClassicUIKeyLabelProvider = (function (_super) {
        __extends(NativeClassicUIKeyLabelProvider, _super);
        function NativeClassicUIKeyLabelProvider(remaps) {
            _super.call(this);
            this.remaps = remaps;
        }
        NativeClassicUIKeyLabelProvider.prototype.getLabelForKey = function (keyCode) {
            if (this.remaps[keyCode] !== null) {
                return this.remaps[keyCode];
            }
            return _super.prototype.getLabelForKey.call(this, keyCode);
        };
        return NativeClassicUIKeyLabelProvider;
    }(keyCodes_1.ClassicUIKeyLabelProvider));
    var NativeAriaKeyLabelProvider = (function (_super) {
        __extends(NativeAriaKeyLabelProvider, _super);
        function NativeAriaKeyLabelProvider(remaps) {
            _super.call(this);
            this.remaps = remaps;
        }
        NativeAriaKeyLabelProvider.prototype.getLabelForKey = function (keyCode) {
            if (this.remaps[keyCode] !== null) {
                return this.remaps[keyCode];
            }
            return _super.prototype.getLabelForKey.call(this, keyCode);
        };
        return NativeAriaKeyLabelProvider;
    }(keyCodes_1.AriaKeyLabelProvider));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/keybinding/electron-browser/keybindingService", ["require", "exports", 'vs/nls!vs/workbench/services/keybinding/electron-browser/keybindingService', 'vs/base/common/platform', 'vs/base/common/winjs.base', 'vs/platform/extensions/common/extensionsRegistry', 'vs/platform/jsonschemas/common/jsonContributionRegistry', 'vs/platform/keybinding/browser/keybindingServiceImpl', 'vs/platform/keybinding/common/keybindingResolver', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/platform/platform', 'vs/workbench/common/events', 'vs/workbench/services/keybinding/electron-browser/nativeKeymap'], function (require, exports, nls, platform, winjs_base_1, extensionsRegistry_1, jsonContributionRegistry_1, keybindingServiceImpl_1, keybindingResolver_1, keybindingsRegistry_1, platform_1, events_1, nativeKeymap_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function isContributedKeyBindingsArray(thing) {
        return Array.isArray(thing);
    }
    function isValidContributedKeyBinding(keyBinding, rejects) {
        if (!keyBinding) {
            rejects.push(nls.localize(0, null));
            return false;
        }
        if (typeof keyBinding.command !== 'string') {
            rejects.push(nls.localize(1, null, 'command'));
            return false;
        }
        if (typeof keyBinding.key !== 'string') {
            rejects.push(nls.localize(2, null, 'key'));
            return false;
        }
        if (keyBinding.when && typeof keyBinding.when !== 'string') {
            rejects.push(nls.localize(3, null, 'when'));
            return false;
        }
        if (keyBinding.mac && typeof keyBinding.mac !== 'string') {
            rejects.push(nls.localize(4, null, 'mac'));
            return false;
        }
        if (keyBinding.linux && typeof keyBinding.linux !== 'string') {
            rejects.push(nls.localize(5, null, 'linux'));
            return false;
        }
        if (keyBinding.win && typeof keyBinding.win !== 'string') {
            rejects.push(nls.localize(6, null, 'win'));
            return false;
        }
        return true;
    }
    var keybindingType = {
        type: 'object',
        default: { command: '', key: '' },
        properties: {
            command: {
                description: nls.localize(7, null),
                type: 'string'
            },
            key: {
                description: nls.localize(8, null),
                type: 'string'
            },
            mac: {
                description: nls.localize(9, null),
                type: 'string'
            },
            linux: {
                description: nls.localize(10, null),
                type: 'string'
            },
            win: {
                description: nls.localize(11, null),
                type: 'string'
            },
            when: {
                description: nls.localize(12, null),
                type: 'string'
            }
        }
    };
    var keybindingsExtPoint = extensionsRegistry_1.ExtensionsRegistry.registerExtensionPoint('keybindings', {
        description: nls.localize(13, null),
        oneOf: [
            keybindingType,
            {
                type: 'array',
                items: keybindingType
            }
        ]
    });
    var WorkbenchKeybindingService = (function (_super) {
        __extends(WorkbenchKeybindingService, _super);
        function WorkbenchKeybindingService(configurationService, contextService, eventService, telemetryService, domNode) {
            var _this = this;
            _super.call(this, configurationService);
            this.contextService = contextService;
            this.eventService = eventService;
            this.telemetryService = telemetryService;
            this.toDispose = this.eventService.addListener(events_1.EventType.WORKBENCH_OPTIONS_CHANGED, function (e) { return _this.onOptionsChanged(e); });
            this._eventService = eventService;
            keybindingsExtPoint.setHandler(function (extensions) {
                var commandAdded = false;
                for (var _i = 0, extensions_1 = extensions; _i < extensions_1.length; _i++) {
                    var extension = extensions_1[_i];
                    commandAdded = _this._handleKeybindingsExtensionPointUser(extension.description.isBuiltin, extension.value, extension.collector) || commandAdded;
                }
                if (commandAdded) {
                    _this.updateResolver();
                }
            });
            this._beginListening(domNode);
        }
        WorkbenchKeybindingService.prototype.setExtensionService = function (extensionService) {
            this._extensionService = extensionService;
        };
        WorkbenchKeybindingService.prototype.customKeybindingsCount = function () {
            var opts = this.contextService.getOptions();
            if (opts.globalSettings && opts.globalSettings.keybindings && Array.isArray(opts.globalSettings.keybindings)) {
                return opts.globalSettings.keybindings.length;
            }
            return 0;
        };
        WorkbenchKeybindingService.prototype._getExtraKeybindings = function (isFirstTime) {
            var extras = [];
            var opts = this.contextService.getOptions();
            if (opts.globalSettings && opts.globalSettings.keybindings) {
                if (!isFirstTime) {
                    var cnt = 0;
                    if (Array.isArray(opts.globalSettings.keybindings)) {
                        cnt = opts.globalSettings.keybindings.length;
                    }
                    this.telemetryService.publicLog('customKeybindingsChanged', {
                        keyCount: cnt
                    });
                }
                if (Array.isArray(opts.globalSettings.keybindings)) {
                    extras = opts.globalSettings.keybindings;
                }
            }
            return extras.map(function (k, i) { return keybindingResolver_1.IOSupport.readKeybindingItem(k, i); });
        };
        WorkbenchKeybindingService.prototype.onOptionsChanged = function (e) {
            if (e.key === 'globalSettings') {
                this.updateResolver();
            }
        };
        WorkbenchKeybindingService.prototype.dispose = function () {
            this.toDispose();
        };
        WorkbenchKeybindingService.prototype.getLabelFor = function (keybinding) {
            return keybinding.toCustomLabel(nativeKeymap_1.getNativeLabelProvider());
        };
        WorkbenchKeybindingService.prototype.getHTMLLabelFor = function (keybinding) {
            return keybinding.toCustomHTMLLabel(nativeKeymap_1.getNativeLabelProvider());
        };
        WorkbenchKeybindingService.prototype.getAriaLabelFor = function (keybinding) {
            return keybinding.toCustomLabel(nativeKeymap_1.getNativeAriaLabelProvider());
        };
        WorkbenchKeybindingService.prototype.getElectronAcceleratorFor = function (keybinding) {
            if (platform.isWindows) {
                // electron menus always do the correct rendering on Windows
                return _super.prototype.getElectronAcceleratorFor.call(this, keybinding);
            }
            var usLabel = keybinding._toUSLabel();
            var label = this.getLabelFor(keybinding);
            if (usLabel !== label) {
                // electron menus are incorrect in rendering (linux) and in rendering and interpreting (mac)
                // for non US standard keyboard layouts
                return null;
            }
            return _super.prototype.getElectronAcceleratorFor.call(this, keybinding);
        };
        WorkbenchKeybindingService.prototype._handleKeybindingsExtensionPointUser = function (isBuiltin, keybindings, collector) {
            if (isContributedKeyBindingsArray(keybindings)) {
                var commandAdded = false;
                for (var i = 0, len = keybindings.length; i < len; i++) {
                    commandAdded = this._handleKeybinding(isBuiltin, i + 1, keybindings[i], collector) || commandAdded;
                }
                return commandAdded;
            }
            else {
                return this._handleKeybinding(isBuiltin, 1, keybindings, collector);
            }
        };
        WorkbenchKeybindingService.prototype._handleKeybinding = function (isBuiltin, idx, keybindings, collector) {
            var rejects = [];
            var commandAdded = false;
            if (isValidContributedKeyBinding(keybindings, rejects)) {
                var rule = this._asCommandRule(isBuiltin, idx++, keybindings);
                if (rule) {
                    keybindingsRegistry_1.KeybindingsRegistry.registerCommandRule(rule);
                    commandAdded = true;
                }
            }
            if (rejects.length > 0) {
                collector.error(nls.localize(14, null, keybindingsExtPoint.name, rejects.join('\n')));
            }
            return commandAdded;
        };
        WorkbenchKeybindingService.prototype._invokeHandler = function (commandId, args) {
            var _this = this;
            if (this._extensionService) {
                return this._extensionService.activateByEvent('onCommand:' + commandId).then(function (_) {
                    return _super.prototype._invokeHandler.call(_this, commandId, args);
                });
            }
            return winjs_base_1.TPromise.as(null);
        };
        WorkbenchKeybindingService.prototype._asCommandRule = function (isBuiltin, idx, binding) {
            var command = binding.command, when = binding.when, key = binding.key, mac = binding.mac, linux = binding.linux, win = binding.win;
            var weight;
            if (isBuiltin) {
                weight = keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.builtinExtension(idx);
            }
            else {
                weight = keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.externalExtension(idx);
            }
            var desc = {
                id: command,
                context: keybindingResolver_1.IOSupport.readKeybindingContexts(when),
                weight: weight,
                primary: keybindingResolver_1.IOSupport.readKeybinding(key),
                mac: mac && { primary: keybindingResolver_1.IOSupport.readKeybinding(mac) },
                linux: linux && { primary: keybindingResolver_1.IOSupport.readKeybinding(linux) },
                win: win && { primary: keybindingResolver_1.IOSupport.readKeybinding(win) }
            };
            if (!desc.primary && !desc.mac && !desc.linux && !desc.win) {
                return;
            }
            return desc;
        };
        return WorkbenchKeybindingService;
    }(keybindingServiceImpl_1.KeybindingService));
    exports.WorkbenchKeybindingService = WorkbenchKeybindingService;
    var schemaId = 'vscode://schemas/keybindings';
    var schema = {
        'id': schemaId,
        'type': 'array',
        'title': nls.localize(15, null),
        'items': {
            'required': ['key'],
            'type': 'object',
            'defaultSnippets': [{ 'body': { 'key': '{{_}}', 'command': '{{_}}', 'when': '{{_}}' } }],
            'properties': {
                'key': {
                    'type': 'string',
                    'description': nls.localize(16, null),
                },
                'command': {
                    'description': nls.localize(17, null),
                },
                'when': {
                    'type': 'string',
                    'description': nls.localize(18, null)
                }
            }
        }
    };
    var schemaRegistry = platform_1.Registry.as(jsonContributionRegistry_1.Extensions.JSONContribution);
    schemaRegistry.registerSchema(schemaId, schema);
    schemaRegistry.addSchemaFileAssociation('vscode://defaultsettings/keybindings.json', schemaId);
    schemaRegistry.addSchemaFileAssociation('%APP_SETTINGS_HOME%/keybindings.json', schemaId);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/lifecycle/electron-browser/lifecycleService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/errors', 'vs/platform/lifecycle/common/baseLifecycleService', 'vs/base/common/severity', 'electron'], function (require, exports, winjs_base_1, errors, baseLifecycleService_1, severity_1, electron_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var LifecycleService = (function (_super) {
        __extends(LifecycleService, _super);
        function LifecycleService(messageService, windowService) {
            _super.call(this);
            this.messageService = messageService;
            this.windowService = windowService;
            this.registerListeners();
        }
        LifecycleService.prototype.registerListeners = function () {
            var _this = this;
            var windowId = this.windowService.getWindowId();
            // Main side indicates that window is about to unload, check for vetos
            electron_1.ipcRenderer.on('vscode:beforeUnload', function (event, reply) {
                var veto = _this.beforeUnload();
                if (typeof veto === 'boolean') {
                    electron_1.ipcRenderer.send(veto ? reply.cancelChannel : reply.okChannel, windowId);
                }
                else {
                    veto.done(function (v) { return electron_1.ipcRenderer.send(v ? reply.cancelChannel : reply.okChannel, windowId); });
                }
            });
        };
        LifecycleService.prototype.beforeUnload = function () {
            var _this = this;
            var veto = this.vetoShutdown();
            if (typeof veto === 'boolean') {
                return this.handleVeto(veto);
            }
            else {
                return veto.then(function (v) { return _this.handleVeto(v); });
            }
        };
        LifecycleService.prototype.handleVeto = function (veto) {
            if (!veto) {
                try {
                    this.fireShutdown();
                }
                catch (error) {
                    errors.onUnexpectedError(error); // unexpected program error and we cause shutdown to cancel in this case
                    return false;
                }
            }
            return veto;
        };
        LifecycleService.prototype.vetoShutdown = function () {
            var _this = this;
            var participants = this.beforeShutdownParticipants;
            var vetoPromises = [];
            var hasPromiseWithVeto = false;
            for (var i = 0; i < participants.length; i++) {
                var participantVeto = participants[i].beforeShutdown();
                if (participantVeto === true) {
                    return true; // return directly when any veto was provided
                }
                else if (participantVeto === false) {
                    continue; // skip
                }
                // We have a promise
                var vetoPromise = participantVeto.then(function (veto) {
                    if (veto) {
                        hasPromiseWithVeto = true;
                    }
                }, function (error) {
                    hasPromiseWithVeto = true;
                    _this.messageService.show(severity_1.default.Error, errors.toErrorMessage(error));
                });
                vetoPromises.push(vetoPromise);
            }
            if (vetoPromises.length === 0) {
                return false; // return directly when no veto was provided
            }
            return winjs_base_1.TPromise.join(vetoPromises).then(function () { return hasPromiseWithVeto; });
        };
        return LifecycleService;
    }(baseLifecycleService_1.BaseLifecycleService));
    exports.LifecycleService = LifecycleService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/panel/common/panelService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    "use strict";
    exports.IPanelService = instantiation_1.createDecorator('panelService');
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/output/common/outputEditorInput", ["require", "exports", 'vs/nls!vs/workbench/parts/output/common/outputEditorInput', 'vs/base/common/lifecycle', 'vs/base/common/async', 'vs/workbench/common/editor/stringEditorInput', 'vs/workbench/parts/output/common/output', 'vs/platform/instantiation/common/instantiation', 'vs/platform/event/common/event', 'vs/workbench/common/events', 'vs/workbench/services/panel/common/panelService'], function (require, exports, nls, lifecycle, async_1, stringEditorInput_1, output_1, instantiation_1, event_1, events_1, panelService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Output Editor Input
     */
    var OutputEditorInput = (function (_super) {
        __extends(OutputEditorInput, _super);
        function OutputEditorInput(channel, instantiationService, outputService, panelService, eventService) {
            var _this = this;
            _super.call(this, nls.localize(0, null), channel ? nls.localize(1, null, channel) : '', '', output_1.OUTPUT_MIME, true, instantiationService);
            this.outputService = outputService;
            this.panelService = panelService;
            this.eventService = eventService;
            this.channel = channel;
            this.bufferedOutput = '';
            this.toDispose = [];
            this.toDispose.push(this.outputService.onOutput(this.onOutputReceived, this));
            this.toDispose.push(this.outputService.onActiveOutputChannel(function () { return _this.scheduleOutputAppend(); }));
            this.toDispose.push(this.eventService.addListener2(events_1.EventType.COMPOSITE_OPENED, function (e) {
                if (e.compositeId === output_1.OUTPUT_PANEL_ID) {
                    _this.appendOutput();
                }
            }));
            this.appendOutputScheduler = new async_1.RunOnceScheduler(function () {
                if (_this.isVisible()) {
                    _this.appendOutput();
                }
            }, OutputEditorInput.OUTPUT_DELAY);
        }
        OutputEditorInput.getInstances = function () {
            return Object.keys(OutputEditorInput.instances).map(function (key) { return OutputEditorInput.instances[key]; });
        };
        OutputEditorInput.getInstance = function (instantiationService, channel) {
            if (OutputEditorInput.instances[channel]) {
                return OutputEditorInput.instances[channel];
            }
            OutputEditorInput.instances[channel] = instantiationService.createInstance(OutputEditorInput, channel);
            return OutputEditorInput.instances[channel];
        };
        OutputEditorInput.prototype.appendOutput = function () {
            if (this.value.length + this.bufferedOutput.length > output_1.MAX_OUTPUT_LENGTH) {
                this.setValue(this.outputService.getOutput(this.channel));
            }
            else {
                this.append(this.bufferedOutput);
            }
            this.bufferedOutput = '';
            var panel = this.panelService.getActivePanel();
            panel.revealLastLine();
        };
        OutputEditorInput.prototype.onOutputReceived = function (e) {
            if (this.outputSet && e.channel === this.channel) {
                if (e.output) {
                    // TODO@Isidor extract this output trimming to common string with tests
                    var newLength = this.bufferedOutput.length + e.output.length;
                    if (newLength > output_1.MAX_OUTPUT_LENGTH) {
                        this.bufferedOutput = '...' + this.bufferedOutput.substr(newLength - output_1.MAX_OUTPUT_LENGTH);
                    }
                    if (e.output.length > output_1.MAX_OUTPUT_LENGTH) {
                        this.bufferedOutput += e.output.substr(e.output.length - output_1.MAX_OUTPUT_LENGTH);
                    }
                    else {
                        this.bufferedOutput += e.output;
                    }
                    this.scheduleOutputAppend();
                }
                else if (e.output === null) {
                    this.clearValue(); // special output indicates we should clear
                }
            }
        };
        OutputEditorInput.prototype.isVisible = function () {
            var panel = this.panelService.getActivePanel();
            return panel && panel.getId() === output_1.OUTPUT_PANEL_ID && this.outputService.getActiveChannel() === this.channel;
        };
        OutputEditorInput.prototype.scheduleOutputAppend = function () {
            if (this.isVisible() && this.bufferedOutput && !this.appendOutputScheduler.isScheduled()) {
                this.appendOutputScheduler.schedule();
            }
        };
        OutputEditorInput.prototype.getId = function () {
            return output_1.OUTPUT_EDITOR_INPUT_ID;
        };
        OutputEditorInput.prototype.resolve = function (refresh) {
            var _this = this;
            return _super.prototype.resolve.call(this, refresh).then(function (model) {
                // Just return model if output already set
                if (_this.outputSet) {
                    return model;
                }
                _this.setValue(_this.outputService.getOutput(_this.channel));
                _this.outputSet = true;
                return model;
            });
        };
        OutputEditorInput.prototype.getChannel = function () {
            return this.channel;
        };
        OutputEditorInput.prototype.matches = function (otherInput) {
            if (otherInput instanceof OutputEditorInput) {
                var otherOutputEditorInput = otherInput;
                if (otherOutputEditorInput.getChannel() === this.channel) {
                    return _super.prototype.matches.call(this, otherInput);
                }
            }
            return false;
        };
        OutputEditorInput.prototype.dispose = function () {
            this.appendOutputScheduler.dispose();
            this.toDispose = lifecycle.disposeAll(this.toDispose);
            _super.prototype.dispose.call(this);
        };
        OutputEditorInput.OUTPUT_DELAY = 300; // delay in ms to accumulate output before emitting an event about it
        OutputEditorInput.instances = Object.create(null);
        OutputEditorInput = __decorate([
            __param(1, instantiation_1.IInstantiationService),
            __param(2, output_1.IOutputService),
            __param(3, panelService_1.IPanelService),
            __param(4, event_1.IEventService)
        ], OutputEditorInput);
        return OutputEditorInput;
    }(stringEditorInput_1.StringEditorInput));
    exports.OutputEditorInput = OutputEditorInput;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/output/common/outputServices", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/strings', 'vs/base/common/event', 'vs/platform/event/common/event', 'vs/platform/lifecycle/common/lifecycle', 'vs/platform/instantiation/common/instantiation', 'vs/platform/storage/common/storage', 'vs/platform/platform', 'vs/workbench/common/editor', 'vs/workbench/parts/output/common/output', 'vs/workbench/parts/output/common/outputEditorInput', 'vs/workbench/services/panel/common/panelService'], function (require, exports, winjs_base_1, strings, event_1, event_2, lifecycle_1, instantiation_1, storage_1, platform_1, editor_1, output_1, outputEditorInput_1, panelService_1) {
    "use strict";
    var OUTPUT_ACTIVE_CHANNEL_KEY = 'output.activechannel';
    var OutputService = (function () {
        function OutputService(storageService, instantiationService, eventService, lifecycleService, panelService) {
            this.storageService = storageService;
            this.instantiationService = instantiationService;
            this.eventService = eventService;
            this.lifecycleService = lifecycleService;
            this.panelService = panelService;
            this.serviceId = output_1.IOutputService;
            this._onOutput = new event_1.Emitter();
            this._onOutputChannel = new event_1.Emitter();
            this._onActiveOutputChannel = new event_1.Emitter();
            this.receivedOutput = Object.create(null);
            var channels = platform_1.Registry.as(output_1.Extensions.OutputChannels).getChannels();
            this.activeChannel = this.storageService.get(OUTPUT_ACTIVE_CHANNEL_KEY, storage_1.StorageScope.WORKSPACE, channels && channels.length > 0 ? channels[0] : null);
        }
        Object.defineProperty(OutputService.prototype, "onOutput", {
            get: function () {
                return this._onOutput.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(OutputService.prototype, "onOutputChannel", {
            get: function () {
                return this._onOutputChannel.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(OutputService.prototype, "onActiveOutputChannel", {
            get: function () {
                return this._onActiveOutputChannel.event;
            },
            enumerable: true,
            configurable: true
        });
        OutputService.prototype.append = function (channel, output) {
            // Initialize
            if (!this.receivedOutput[channel]) {
                this.receivedOutput[channel] = '';
                this._onOutputChannel.fire(channel); // emit event that we have a new channel
            }
            // Sanitize
            output = strings.removeAnsiEscapeCodes(output);
            // Store
            if (output) {
                var curLength = this.receivedOutput[channel].length;
                var addLength = output.length;
                // Still below MAX_OUTPUT, so just add
                if (addLength + curLength <= output_1.MAX_OUTPUT_LENGTH) {
                    this.receivedOutput[channel] += output;
                }
                else {
                    // New output exceeds MAX_OUTPUT, so trim beginning and use as received output
                    if (addLength > output_1.MAX_OUTPUT_LENGTH) {
                        this.receivedOutput[channel] = '...' + output.substr(addLength - output_1.MAX_OUTPUT_LENGTH);
                    }
                    else {
                        var diff = output_1.MAX_OUTPUT_LENGTH - addLength;
                        this.receivedOutput[channel] = '...' + this.receivedOutput[channel].substr(curLength - diff) + output;
                    }
                }
            }
            this._onOutput.fire({ output: output, channel: channel });
        };
        OutputService.prototype.getOutput = function (channel) {
            return this.receivedOutput[channel] || '';
        };
        OutputService.prototype.getChannels = function () {
            return Object.keys(this.receivedOutput);
        };
        OutputService.prototype.getActiveChannel = function () {
            return this.activeChannel;
        };
        OutputService.prototype.clearOutput = function (channel) {
            this.receivedOutput[channel] = '';
            this._onOutput.fire({ channel: channel, output: null /* indicator to clear output */ });
        };
        OutputService.prototype.showOutput = function (channel, preserveFocus) {
            var _this = this;
            var panel = this.panelService.getActivePanel();
            if (this.activeChannel === channel && panel && panel.getId() === output_1.OUTPUT_PANEL_ID) {
                return winjs_base_1.TPromise.as(panel);
            }
            this.activeChannel = channel;
            this.storageService.store(OUTPUT_ACTIVE_CHANNEL_KEY, this.activeChannel, storage_1.StorageScope.WORKSPACE);
            this._onActiveOutputChannel.fire(channel); // emit event that a new channel is active
            return this.panelService.openPanel(output_1.OUTPUT_PANEL_ID, !preserveFocus).then(function (outputPanel) {
                return outputPanel && outputPanel.setInput(outputEditorInput_1.OutputEditorInput.getInstance(_this.instantiationService, channel), editor_1.EditorOptions.create({ preserveFocus: preserveFocus })).
                    then(function () { return outputPanel; });
            });
        };
        OutputService = __decorate([
            __param(0, storage_1.IStorageService),
            __param(1, instantiation_1.IInstantiationService),
            __param(2, event_2.IEventService),
            __param(3, lifecycle_1.ILifecycleService),
            __param(4, panelService_1.IPanelService)
        ], OutputService);
        return OutputService;
    }());
    exports.OutputService = OutputService;
});

define("vs/workbench/services/part/common/partService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    (function (Parts) {
        Parts[Parts["ACTIVITYBAR_PART"] = 0] = "ACTIVITYBAR_PART";
        Parts[Parts["SIDEBAR_PART"] = 1] = "SIDEBAR_PART";
        Parts[Parts["PANEL_PART"] = 2] = "PANEL_PART";
        Parts[Parts["EDITOR_PART"] = 3] = "EDITOR_PART";
        Parts[Parts["STATUSBAR_PART"] = 4] = "STATUSBAR_PART";
    })(exports.Parts || (exports.Parts = {}));
    var Parts = exports.Parts;
    (function (Position) {
        Position[Position["LEFT"] = 0] = "LEFT";
        Position[Position["RIGHT"] = 1] = "RIGHT";
    })(exports.Position || (exports.Position = {}));
    var Position = exports.Position;
    exports.IPartService = instantiation_1.createDecorator('partService');
});

define("vs/workbench/common/actionRegistry", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/collections', 'vs/platform/platform', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/workbench/services/part/common/partService', 'vs/platform/message/common/message', 'vs/platform/telemetry/common/telemetry', 'vs/platform/instantiation/common/instantiation', 'vs/base/common/severity'], function (require, exports, winjs_base_1, collections, platform_1, keybindingsRegistry_1, partService_1, message_1, telemetry_1, instantiation_1, severity_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.Extensions = {
        WorkbenchActions: 'workbench.contributions.actions'
    };
    var WorkbenchActionRegistry = (function () {
        function WorkbenchActionRegistry() {
            this.workbenchActions = Object.create(null);
            this.mapActionIdToCategory = Object.create(null);
        }
        WorkbenchActionRegistry.prototype.registerWorkbenchAction = function (descriptor, category) {
            if (!this.workbenchActions[descriptor.id]) {
                this.workbenchActions[descriptor.id] = descriptor;
                registerWorkbenchCommandFromAction(descriptor);
                if (category) {
                    this.mapActionIdToCategory[descriptor.id] = category;
                }
            }
        };
        WorkbenchActionRegistry.prototype.unregisterWorkbenchAction = function (id) {
            if (!this.workbenchActions[id]) {
                return false;
            }
            delete this.workbenchActions[id];
            delete this.mapActionIdToCategory[id];
            return true;
        };
        WorkbenchActionRegistry.prototype.getWorkbenchAction = function (id) {
            return this.workbenchActions[id] || null;
        };
        WorkbenchActionRegistry.prototype.getCategory = function (id) {
            return this.mapActionIdToCategory[id] || null;
        };
        WorkbenchActionRegistry.prototype.getWorkbenchActions = function () {
            return collections.values(this.workbenchActions);
        };
        WorkbenchActionRegistry.prototype.setWorkbenchActions = function (actions) {
            var _this = this;
            this.workbenchActions = Object.create(null);
            this.mapActionIdToCategory = Object.create(null);
            actions.forEach(function (action) { return _this.registerWorkbenchAction(action); }, this);
        };
        return WorkbenchActionRegistry;
    }());
    platform_1.Registry.add(exports.Extensions.WorkbenchActions, new WorkbenchActionRegistry());
    function registerWorkbenchCommandFromAction(descriptor) {
        var context = descriptor.keybindingContext;
        var weight = (typeof descriptor.keybindingWeight === 'undefined' ? keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.workbenchContrib() : descriptor.keybindingWeight);
        var keybindings = descriptor.keybindings;
        var desc = {
            id: descriptor.id,
            handler: createCommandHandler(descriptor),
            weight: weight,
            context: context,
            primary: keybindings && keybindings.primary,
            secondary: keybindings && keybindings.secondary,
            win: keybindings && keybindings.win,
            mac: keybindings && keybindings.mac,
            linux: keybindings && keybindings.linux
        };
        keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc(desc);
    }
    function createCommandHandler(descriptor) {
        return function (accessor, args) {
            var messageService = accessor.get(message_1.IMessageService);
            var instantiationService = accessor.get(instantiation_1.IInstantiationService);
            var telemetryServce = accessor.get(telemetry_1.ITelemetryService);
            var partService = accessor.get(partService_1.IPartService);
            winjs_base_1.TPromise.as(triggerAndDisposeAction(instantiationService, telemetryServce, partService, descriptor, args)).done(null, function (err) {
                messageService.show(severity_1.default.Error, err);
            });
        };
    }
    exports.createCommandHandler = createCommandHandler;
    function triggerAndDisposeAction(instantitationService, telemetryService, partService, descriptor, args) {
        var actionInstance = instantitationService.createInstance(descriptor.syncDescriptor);
        actionInstance.label = descriptor.label || actionInstance.label;
        // don't run the action when not enabled
        if (!actionInstance.enabled) {
            actionInstance.dispose();
            return;
        }
        if (telemetryService) {
            telemetryService.publicLog('workbenchActionExecuted', { id: actionInstance.id, from: args.from || 'keybinding' });
        }
        // run action when workbench is created
        return partService.joinCreation().then(function () {
            try {
                return winjs_base_1.TPromise.as(actionInstance.run()).then(function () {
                    actionInstance.dispose();
                }, function (err) {
                    actionInstance.dispose();
                    return winjs_base_1.TPromise.wrapError(err);
                });
            }
            catch (err) {
                actionInstance.dispose();
                return winjs_base_1.TPromise.wrapError(err);
            }
        });
    }
    exports.triggerAndDisposeAction = triggerAndDisposeAction;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/actions/toggleSidebarPosition", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/browser/actions/toggleSidebarPosition', 'vs/platform/platform', 'vs/base/common/actions', 'vs/platform/actions/common/actions', 'vs/workbench/common/actionRegistry', 'vs/workbench/services/part/common/partService'], function (require, exports, winjs_base_1, nls, platform_1, actions_1, actions_2, actionRegistry_1, partService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ID = 'workbench.action.toggleSidebarPosition';
    var LABEL = nls.localize(0, null);
    var ToggleSidebarPositionAction = (function (_super) {
        __extends(ToggleSidebarPositionAction, _super);
        function ToggleSidebarPositionAction(id, label, partService) {
            _super.call(this, id, label);
            this.partService = partService;
            this.enabled = !!this.partService;
        }
        ToggleSidebarPositionAction.prototype.run = function () {
            var position = this.partService.getSideBarPosition();
            this.partService.setSideBarPosition(position === partService_1.Position.LEFT ? partService_1.Position.RIGHT : partService_1.Position.LEFT);
            return winjs_base_1.TPromise.as(null);
        };
        ToggleSidebarPositionAction = __decorate([
            __param(2, partService_1.IPartService)
        ], ToggleSidebarPositionAction);
        return ToggleSidebarPositionAction;
    }(actions_1.Action));
    exports.ToggleSidebarPositionAction = ToggleSidebarPositionAction;
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(ToggleSidebarPositionAction, ID, LABEL), nls.localize(1, null));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/actions/toggleSidebarVisibility", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/browser/actions/toggleSidebarVisibility', 'vs/platform/platform', 'vs/base/common/actions', 'vs/platform/actions/common/actions', 'vs/workbench/common/actionRegistry', 'vs/workbench/services/part/common/partService', 'vs/base/common/keyCodes'], function (require, exports, winjs_base_1, nls, platform_1, actions_1, actions_2, actionRegistry_1, partService_1, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ID = 'workbench.action.toggleSidebarVisibility';
    var LABEL = nls.localize(0, null);
    var ToggleSidebarVisibilityAction = (function (_super) {
        __extends(ToggleSidebarVisibilityAction, _super);
        function ToggleSidebarVisibilityAction(id, label, partService) {
            _super.call(this, id, label);
            this.partService = partService;
            this.enabled = !!this.partService;
        }
        ToggleSidebarVisibilityAction.prototype.run = function () {
            var hideSidebar = !this.partService.isSideBarHidden();
            this.partService.setSideBarHidden(hideSidebar);
            return winjs_base_1.TPromise.as(null);
        };
        ToggleSidebarVisibilityAction = __decorate([
            __param(2, partService_1.IPartService)
        ], ToggleSidebarVisibilityAction);
        return ToggleSidebarVisibilityAction;
    }(actions_1.Action));
    exports.ToggleSidebarVisibilityAction = ToggleSidebarVisibilityAction;
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(ToggleSidebarVisibilityAction, ID, LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_B }), nls.localize(1, null));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/actions/triggerNavigation", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/browser/actions/triggerNavigation', 'vs/platform/platform', 'vs/base/common/actions', 'vs/platform/actions/common/actions', 'vs/workbench/common/actionRegistry', 'vs/base/common/keyCodes', 'vs/workbench/services/history/common/history'], function (require, exports, winjs_base_1, nls, platform_1, actions_1, actions_2, actionRegistry_1, keyCodes_1, history_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var NAVIGATE_FORWARD_ID = 'workbench.action.navigateForward';
    var NAVIGATE_FORWARD_LABEL = nls.localize(0, null);
    var NavigateForwardAction = (function (_super) {
        __extends(NavigateForwardAction, _super);
        function NavigateForwardAction(id, label, historyService) {
            _super.call(this, id, label);
            this.historyService = historyService;
        }
        NavigateForwardAction.prototype.run = function () {
            this.historyService.forward();
            return winjs_base_1.TPromise.as(null);
        };
        NavigateForwardAction = __decorate([
            __param(2, history_1.IHistoryService)
        ], NavigateForwardAction);
        return NavigateForwardAction;
    }(actions_1.Action));
    exports.NavigateForwardAction = NavigateForwardAction;
    var NAVIGATE_BACKWARDS_ID = 'workbench.action.navigateBack';
    var NAVIGATE_BACKWARDS_LABEL = nls.localize(1, null);
    var NavigateBackwardsAction = (function (_super) {
        __extends(NavigateBackwardsAction, _super);
        function NavigateBackwardsAction(id, label, historyService) {
            _super.call(this, id, label);
            this.historyService = historyService;
        }
        NavigateBackwardsAction.prototype.run = function () {
            this.historyService.back();
            return winjs_base_1.TPromise.as(null);
        };
        NavigateBackwardsAction = __decorate([
            __param(2, history_1.IHistoryService)
        ], NavigateBackwardsAction);
        return NavigateBackwardsAction;
    }(actions_1.Action));
    exports.NavigateBackwardsAction = NavigateBackwardsAction;
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(NavigateForwardAction, NAVIGATE_FORWARD_ID, NAVIGATE_FORWARD_LABEL, {
        primary: null,
        win: { primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.RightArrow },
        mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.US_MINUS },
        linux: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.US_MINUS }
    }));
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(NavigateBackwardsAction, NAVIGATE_BACKWARDS_ID, NAVIGATE_BACKWARDS_LABEL, {
        primary: null,
        win: { primary: keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.LeftArrow },
        mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.US_MINUS },
        linux: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.US_MINUS }
    }));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/errorList/browser/errorList.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/errorList/browser/errorList.contribution', 'vs/base/common/actions', 'vs/base/common/winjs.base', 'vs/platform/platform', 'vs/workbench/browser/panel', 'vs/workbench/parts/errorList/browser/errorListConstants', 'vs/workbench/services/part/common/partService', 'vs/workbench/services/panel/common/panelService'], function (require, exports, nls, actions_1, winjs_base_1, platform, panel, errorListConstants_1, partService_1, panelService_1) {
    "use strict";
    var ToggleErrorListAction = (function (_super) {
        __extends(ToggleErrorListAction, _super);
        function ToggleErrorListAction(id, label, partService, panelService) {
            _super.call(this, id, label);
            this.partService = partService;
            this.panelService = panelService;
        }
        ToggleErrorListAction.prototype.run = function (event) {
            var panel = this.panelService.getActivePanel();
            if (panel && panel.getId() === errorListConstants_1.ERROR_LIST_PANEL_ID) {
                this.partService.setPanelHidden(true);
                return winjs_base_1.TPromise.as(null);
            }
            return this.panelService.openPanel(errorListConstants_1.ERROR_LIST_PANEL_ID, true);
        };
        ToggleErrorListAction.ID = 'workbench.action.errorList.toggle';
        ToggleErrorListAction.LABEL = nls.localize(0, null);
        ToggleErrorListAction = __decorate([
            __param(2, partService_1.IPartService),
            __param(3, panelService_1.IPanelService)
        ], ToggleErrorListAction);
        return ToggleErrorListAction;
    }(actions_1.Action));
    // register panel
    platform.Registry.as(panel.Extensions.Panels).registerPanel(new panel.PanelDescriptor('vs/workbench/parts/errorList/browser/errorList', 'ErrorList', errorListConstants_1.ERROR_LIST_PANEL_ID, nls.localize(1, null), 'errorList'));
});
// register toggle output action globally
// let actionRegistry = <IWorkbenchActionRegistry>platform.Registry.as(ActionExtensions.WorkbenchActions);
// actionRegistry.registerWorkbenchAction(new SyncActionDescriptor(ToggleErrorListAction, ToggleErrorListAction.ID, ToggleErrorListAction.LABEL, {
// 	primary: KeyMod.CtrlCmd | KeyMod.Shift | KeyCode.KEY_A,
// 	linux: {
// 		primary: KeyMod.CtrlCmd | KeyMod.Shift | KeyCode.KEY_A
// 	}
// }), nls.localize('viewCategory', "View"));

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/views/emptyView", ["require", "exports", 'vs/nls!vs/workbench/parts/files/browser/views/emptyView', 'vs/base/common/platform', 'vs/base/browser/dom', 'vs/base/common/winjs.base', 'vs/base/browser/ui/button/button', 'vs/base/browser/builder', 'vs/base/browser/ui/splitview/splitview', 'vs/platform/platform', 'vs/workbench/common/actionRegistry', 'vs/platform/instantiation/common/instantiation', 'vs/platform/selection/common/selection'], function (require, exports, nls, env, DOM, winjs_base_1, button_1, builder_1, splitview_1, platform_1, actionRegistry_1, instantiation_1, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var EmptyView = (function (_super) {
        __extends(EmptyView, _super);
        function EmptyView(instantiationService) {
            _super.call(this, {
                minimumSize: 2 * 22,
                ariaHeaderLabel: nls.localize(0, null)
            });
            this.instantiationService = instantiationService;
        }
        EmptyView.prototype.renderHeader = function (container) {
            var titleDiv = builder_1.$('div.title').appendTo(container);
            builder_1.$('span').text(nls.localize(1, null)).appendTo(titleDiv);
        };
        EmptyView.prototype.renderBody = function (container) {
            var _this = this;
            DOM.addClass(container, 'explorer-empty-view');
            var titleDiv = builder_1.$('div.section').appendTo(container);
            builder_1.$('p').text(nls.localize(2, null)).appendTo(titleDiv);
            var section = builder_1.$('div.section').appendTo(container);
            var button = new button_1.Button(section);
            button.label = nls.localize(3, null);
            button.on('click', function () {
                _this.runWorkbenchAction(env.isMacintosh ? 'workbench.action.files.openFileFolder' : 'workbench.action.files.openFolder');
            });
        };
        EmptyView.prototype.runWorkbenchAction = function (actionId) {
            var actionRegistry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
            var actionDescriptor = actionRegistry.getWorkbenchAction(actionId);
            var action = this.instantiationService.createInstance(actionDescriptor.syncDescriptor);
            return action.run().done(function () { return action.dispose(); });
        };
        EmptyView.prototype.create = function () {
            return winjs_base_1.TPromise.as(null);
        };
        EmptyView.prototype.refresh = function (focus, reveal, instantProgress) {
            return winjs_base_1.TPromise.as(null);
        };
        EmptyView.prototype.setVisible = function (visible) {
            return winjs_base_1.TPromise.as(null);
        };
        EmptyView.prototype.focusBody = function () {
            // Ignore
        };
        EmptyView.prototype.getSelection = function () {
            return new selection_1.StructuredSelection([]);
        };
        EmptyView.prototype.reveal = function (element, relativeTop) {
            return winjs_base_1.TPromise.as(null);
        };
        EmptyView.prototype.getActions = function () {
            return [];
        };
        EmptyView.prototype.getSecondaryActions = function () {
            return [];
        };
        EmptyView.prototype.getActionItem = function (action) {
            return null;
        };
        EmptyView.prototype.shutdown = function () {
            // Subclass to implement
        };
        EmptyView = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], EmptyView);
        return EmptyView;
    }(splitview_1.CollapsibleView));
    exports.EmptyView = EmptyView;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/output/browser/outputActions", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/output/browser/outputActions', 'vs/platform/platform', 'vs/base/common/arrays', 'vs/base/common/actions', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/workbench/parts/output/common/output', 'vs/base/browser/ui/actionbar/actionbar', 'vs/workbench/services/part/common/partService', 'vs/workbench/services/panel/common/panelService'], function (require, exports, winjs_base_1, nls, platform_1, arrays, actions_1, editorAction_1, editorActionEnablement_1, output_1, actionbar_1, partService_1, panelService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ToggleOutputAction = (function (_super) {
        __extends(ToggleOutputAction, _super);
        function ToggleOutputAction(id, label, partService, panelService, outputService) {
            _super.call(this, id, label);
            this.partService = partService;
            this.panelService = panelService;
            this.outputService = outputService;
        }
        ToggleOutputAction.prototype.run = function (event) {
            var panel = this.panelService.getActivePanel();
            if (panel && panel.getId() === output_1.OUTPUT_PANEL_ID) {
                this.partService.setPanelHidden(true);
                return winjs_base_1.TPromise.as(null);
            }
            return this.outputService.showOutput(this.outputService.getActiveChannel());
        };
        ToggleOutputAction.ID = 'workbench.action.output.toggleOutput';
        ToggleOutputAction.LABEL = nls.localize(0, null);
        ToggleOutputAction = __decorate([
            __param(2, partService_1.IPartService),
            __param(3, panelService_1.IPanelService),
            __param(4, output_1.IOutputService)
        ], ToggleOutputAction);
        return ToggleOutputAction;
    }(actions_1.Action));
    exports.ToggleOutputAction = ToggleOutputAction;
    var ClearOutputAction = (function (_super) {
        __extends(ClearOutputAction, _super);
        function ClearOutputAction(outputService, panelService) {
            _super.call(this, 'workbench.output.action.clearOutput', nls.localize(1, null), 'output-action clear-output');
            this.outputService = outputService;
            this.panelService = panelService;
        }
        ClearOutputAction.prototype.run = function () {
            this.outputService.clearOutput(this.outputService.getActiveChannel());
            this.panelService.getActivePanel().focus();
            return winjs_base_1.TPromise.as(true);
        };
        ClearOutputAction = __decorate([
            __param(0, output_1.IOutputService),
            __param(1, panelService_1.IPanelService)
        ], ClearOutputAction);
        return ClearOutputAction;
    }(actions_1.Action));
    exports.ClearOutputAction = ClearOutputAction;
    var ClearOutputEditorAction = (function (_super) {
        __extends(ClearOutputEditorAction, _super);
        function ClearOutputEditorAction(descriptor, editor, outputService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.ShowInContextMenu);
            this.outputService = outputService;
        }
        ClearOutputEditorAction.prototype.getGroupId = function () {
            return 'clear';
        };
        ClearOutputEditorAction.prototype.isSupported = function () {
            var model = this.editor.getModel();
            var mode = model && model.getMode();
            return mode && mode.getId() === output_1.OUTPUT_MODE_ID && _super.prototype.isSupported.call(this);
        };
        ClearOutputEditorAction.prototype.run = function () {
            this.outputService.clearOutput(this.outputService.getActiveChannel());
            return winjs_base_1.TPromise.as(false);
        };
        ClearOutputEditorAction.ID = 'editor.action.clearoutput';
        ClearOutputEditorAction = __decorate([
            __param(2, output_1.IOutputService)
        ], ClearOutputEditorAction);
        return ClearOutputEditorAction;
    }(editorAction_1.EditorAction));
    exports.ClearOutputEditorAction = ClearOutputEditorAction;
    var SwitchOutputAction = (function (_super) {
        __extends(SwitchOutputAction, _super);
        function SwitchOutputAction(outputService) {
            _super.call(this, SwitchOutputAction.ID, nls.localize(2, null));
            this.outputService = outputService;
            this.class = 'output-action switch-to-output';
        }
        SwitchOutputAction.prototype.run = function (channel) {
            return this.outputService.showOutput(channel);
        };
        SwitchOutputAction.ID = 'workbench.output.action.switchBetweenOutputs';
        SwitchOutputAction = __decorate([
            __param(0, output_1.IOutputService)
        ], SwitchOutputAction);
        return SwitchOutputAction;
    }(actions_1.Action));
    exports.SwitchOutputAction = SwitchOutputAction;
    var SwitchOutputActionItem = (function (_super) {
        __extends(SwitchOutputActionItem, _super);
        function SwitchOutputActionItem(action, outputService) {
            _super.call(this, null, action, SwitchOutputActionItem.getChannels(outputService), Math.max(0, SwitchOutputActionItem.getChannels(outputService).indexOf(outputService.getActiveChannel())));
            this.outputService = outputService;
            this.toDispose.push(this.outputService.onOutputChannel(this.onOutputChannel, this));
            this.toDispose.push(this.outputService.onActiveOutputChannel(this.onOutputChannel, this));
        }
        SwitchOutputActionItem.prototype.onOutputChannel = function () {
            var channels = SwitchOutputActionItem.getChannels(this.outputService);
            var selected = Math.max(0, channels.indexOf(this.outputService.getActiveChannel()));
            this.setOptions(channels, selected);
        };
        SwitchOutputActionItem.getChannels = function (outputService) {
            var contributedChannels = platform_1.Registry.as(output_1.Extensions.OutputChannels).getChannels();
            var usedChannels = outputService.getChannels();
            return arrays.distinct(contributedChannels.concat(usedChannels)).sort(); // sort by name
        };
        SwitchOutputActionItem = __decorate([
            __param(1, output_1.IOutputService)
        ], SwitchOutputActionItem);
        return SwitchOutputActionItem;
    }(actionbar_1.SelectActionItem));
    exports.SwitchOutputActionItem = SwitchOutputActionItem;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/output/browser/output.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/output/browser/output.contribution', 'vs/base/common/keyCodes', 'vs/editor/common/editorCommonExtensions', 'vs/editor/common/modes/modesRegistry', 'vs/platform/platform', 'vs/platform/actions/common/actions', 'vs/platform/instantiation/common/extensions', 'vs/workbench/common/actionRegistry', 'vs/workbench/parts/output/common/outputServices', 'vs/workbench/parts/output/browser/outputActions', 'vs/workbench/parts/output/common/output', 'vs/workbench/browser/panel', 'vs/css!../browser/media/output.contribution'], function (require, exports, nls, keyCodes_1, editorCommonExtensions_1, modesRegistry_1, platform, actions_1, extensions_1, actionRegistry_1, outputServices_1, outputActions_1, output_1, panel) {
    "use strict";
    // Register Service
    extensions_1.registerSingleton(output_1.IOutputService, outputServices_1.OutputService);
    // Register Output Mode
    modesRegistry_1.ModesRegistry.registerCompatMode({
        id: output_1.OUTPUT_MODE_ID,
        extensions: [],
        aliases: [null],
        mimetypes: [output_1.OUTPUT_MIME],
        moduleId: 'vs/workbench/parts/output/common/outputMode',
        ctorName: 'OutputMode'
    });
    // Register Output Panel
    platform.Registry.as(panel.Extensions.Panels).registerPanel(new panel.PanelDescriptor('vs/workbench/parts/output/browser/outputPanel', 'OutputPanel', output_1.OUTPUT_PANEL_ID, nls.localize(0, null), 'output'));
    // register toggle output action globally
    var actionRegistry = platform.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    actionRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(outputActions_1.ToggleOutputAction, outputActions_1.ToggleOutputAction.ID, outputActions_1.ToggleOutputAction.LABEL, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_U,
        linux: {
            primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_H // On Ubuntu Ctrl+Shift+U is taken by some global OS command
        }
    }), nls.localize(1, null));
    // Contribute to Context Menu of Output Window
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(outputActions_1.ClearOutputEditorAction, outputActions_1.ClearOutputEditorAction.ID, nls.localize(2, null)));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/progress/browser/progressService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/types', 'vs/workbench/common/events', 'vs/platform/progress/common/progress'], function (require, exports, winjs_base_1, types, events_1, progress_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ScopedService = (function () {
        function ScopedService(eventService, scopeId) {
            this._eventService = eventService;
            this.scopeId = scopeId;
            this.registerListeners();
        }
        Object.defineProperty(ScopedService.prototype, "eventService", {
            get: function () {
                return this._eventService;
            },
            enumerable: true,
            configurable: true
        });
        ScopedService.prototype.registerListeners = function () {
            var _this = this;
            this.eventService.addListener(events_1.EventType.EDITOR_CLOSED, function (e) {
                if (e.editorId === _this.scopeId) {
                    _this.onScopeDeactivated();
                }
            });
            this.eventService.addListener(events_1.EventType.EDITOR_OPENED, function (e) {
                if (e.editorId === _this.scopeId) {
                    _this.onScopeActivated();
                }
            });
            this.eventService.addListener(events_1.EventType.COMPOSITE_CLOSED, function (e) {
                if (e.compositeId === _this.scopeId) {
                    _this.onScopeDeactivated();
                }
            });
            this.eventService.addListener(events_1.EventType.COMPOSITE_OPENED, function (e) {
                if (e.compositeId === _this.scopeId) {
                    _this.onScopeActivated();
                }
            });
        };
        return ScopedService;
    }());
    exports.ScopedService = ScopedService;
    var WorkbenchProgressService = (function (_super) {
        __extends(WorkbenchProgressService, _super);
        function WorkbenchProgressService(eventService, progressbar, scopeId, isActive) {
            _super.call(this, eventService, scopeId);
            this.serviceId = progress_1.IProgressService;
            this.progressbar = progressbar;
            this.isActive = isActive || types.isUndefinedOrNull(scopeId); // If service is unscoped, enable by default
            this.progressState = {};
        }
        WorkbenchProgressService.prototype.onScopeDeactivated = function () {
            this.isActive = false;
        };
        WorkbenchProgressService.prototype.onScopeActivated = function () {
            this.isActive = true;
            // Return early if progress state indicates that progress is done
            if (this.progressState.done) {
                return;
            }
            // Replay Infinite Progress from Promise
            if (this.progressState.whilePromise) {
                this.doShowWhile();
            }
            else if (this.progressState.infinite) {
                this.progressbar.infinite().getContainer().show();
            }
            else {
                if (this.progressState.total) {
                    this.progressbar.total(this.progressState.total).getContainer().show();
                }
                if (this.progressState.worked) {
                    this.progressbar.worked(this.progressState.worked).getContainer().show();
                }
            }
        };
        WorkbenchProgressService.prototype.clearProgressState = function () {
            delete this.progressState.infinite;
            delete this.progressState.done;
            delete this.progressState.worked;
            delete this.progressState.total;
            delete this.progressState.whilePromise;
        };
        WorkbenchProgressService.prototype.show = function (infiniteOrTotal, delay) {
            var _this = this;
            var infinite;
            var total;
            // Sort out Arguments
            if (infiniteOrTotal === false || infiniteOrTotal === true) {
                infinite = infiniteOrTotal;
            }
            else {
                total = infiniteOrTotal;
            }
            // Reset State
            this.clearProgressState();
            // Keep in State
            this.progressState.infinite = infinite;
            this.progressState.total = total;
            // Active: Show Progress
            if (this.isActive) {
                // Infinite: Start Progressbar and Show after Delay
                if (!types.isUndefinedOrNull(infinite)) {
                    if (types.isUndefinedOrNull(delay)) {
                        this.progressbar.infinite().getContainer().show();
                    }
                    else {
                        this.progressbar.infinite().getContainer().showDelayed(delay);
                    }
                }
                else if (!types.isUndefinedOrNull(total)) {
                    if (types.isUndefinedOrNull(delay)) {
                        this.progressbar.total(total).getContainer().show();
                    }
                    else {
                        this.progressbar.total(total).getContainer().showDelayed(delay);
                    }
                }
            }
            return {
                total: function (total) {
                    _this.progressState.infinite = false;
                    _this.progressState.total = total;
                    if (_this.isActive) {
                        _this.progressbar.total(total);
                    }
                },
                worked: function (worked) {
                    // Verify first that we are either not active or the progressbar has a total set
                    if (!_this.isActive || _this.progressbar.hasTotal()) {
                        _this.progressState.infinite = false;
                        if (_this.progressState.worked) {
                            _this.progressState.worked += worked;
                        }
                        else {
                            _this.progressState.worked = worked;
                        }
                        if (_this.isActive) {
                            _this.progressbar.worked(worked);
                        }
                    }
                    else {
                        _this.progressState.infinite = true;
                        delete _this.progressState.worked;
                        delete _this.progressState.total;
                        _this.progressbar.infinite().getContainer().show();
                    }
                },
                done: function () {
                    _this.progressState.infinite = false;
                    _this.progressState.done = true;
                    if (_this.isActive) {
                        _this.progressbar.stop().getContainer().hide();
                    }
                }
            };
        };
        WorkbenchProgressService.prototype.showWhile = function (promise, delay) {
            var _this = this;
            var stack = !!this.progressState.whilePromise;
            // Reset State
            if (!stack) {
                this.clearProgressState();
            }
            else {
                promise = winjs_base_1.TPromise.join([promise, this.progressState.whilePromise]);
            }
            // Keep Promise in State
            this.progressState.whilePromise = promise;
            var stop = function () {
                // If this is not the last promise in the list of joined promises, return early
                if (!!_this.progressState.whilePromise && _this.progressState.whilePromise !== promise) {
                    return;
                }
                // The while promise is either null or equal the promise we last hooked on
                _this.clearProgressState();
                if (_this.isActive) {
                    _this.progressbar.stop().getContainer().hide();
                }
            };
            this.doShowWhile(delay);
            return promise.then(stop, stop);
        };
        WorkbenchProgressService.prototype.doShowWhile = function (delay) {
            // Show Progress when active
            if (this.isActive) {
                if (types.isUndefinedOrNull(delay)) {
                    this.progressbar.infinite().getContainer().show();
                }
                else {
                    this.progressbar.infinite().getContainer().showDelayed(delay);
                }
            }
        };
        return WorkbenchProgressService;
    }(ScopedService));
    exports.WorkbenchProgressService = WorkbenchProgressService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/browser/parts/compositePart", ["require", "exports", 'vs/nls!vs/workbench/browser/parts/compositePart', 'vs/base/common/timer', 'vs/base/common/uuid', 'vs/base/common/winjs.base', 'vs/platform/platform', 'vs/base/browser/builder', 'vs/base/common/events', 'vs/base/common/strings', 'vs/base/common/types', 'vs/base/common/errors', 'vs/base/browser/ui/toolbar/toolbar', 'vs/base/browser/ui/actionbar/actionbar', 'vs/base/browser/ui/progressbar/progressbar', 'vs/workbench/browser/actionBarRegistry', 'vs/workbench/browser/part', 'vs/workbench/common/events', 'vs/workbench/browser/composite', 'vs/workbench/services/progress/browser/progressService', 'vs/platform/storage/common/storage', 'vs/platform/message/common/message', 'vs/css!./media/compositePart'], function (require, exports, nls, timer, uuid, winjs_base_1, platform_1, builder_1, events, strings, types, errors, toolbar_1, actionbar_1, progressbar_1, actionBarRegistry_1, part_1, events_1, composite_1, progressService_1, storage_1, message_1) {
    "use strict";
    var CompositePart = (function (_super) {
        __extends(CompositePart, _super);
        function CompositePart(messageService, storageService, eventService, telemetryService, contextMenuService, partService, keybindingService, registry, activeCompositeSettingsKey, nameForTelemetry, compositeCssClass, actionContributionScope, id) {
            _super.call(this, id);
            this.messageService = messageService;
            this.storageService = storageService;
            this.eventService = eventService;
            this.telemetryService = telemetryService;
            this.contextMenuService = contextMenuService;
            this.partService = partService;
            this.keybindingService = keybindingService;
            this.registry = registry;
            this.activeCompositeSettingsKey = activeCompositeSettingsKey;
            this.nameForTelemetry = nameForTelemetry;
            this.compositeCssClass = compositeCssClass;
            this.actionContributionScope = actionContributionScope;
            this.activeCompositeListeners = [];
            this.instantiatedCompositeListeners = [];
            this.mapCompositeToCompositeContainer = {};
            this.mapActionsBindingToComposite = {};
            this.mapProgressServiceToComposite = {};
            this.activeComposite = null;
            this.instantiatedComposits = [];
            this.compositeLoaderPromises = {};
        }
        CompositePart.prototype.setInstantiationService = function (service) {
            this.instantiationService = service;
        };
        CompositePart.prototype.openComposite = function (id, focus) {
            // Check if composite already visible and just focus in that case
            if (this.activeComposite && this.activeComposite.getId() === id) {
                if (focus) {
                    this.activeComposite.focus();
                }
                // Fullfill promise with composite that is being opened
                return winjs_base_1.TPromise.as(this.activeComposite);
            }
            // Open
            return this.doOpenComposite(id, focus);
        };
        CompositePart.prototype.doOpenComposite = function (id, focus) {
            var _this = this;
            var timerEvent = timer.start(timer.Topic.WORKBENCH, strings.format('Open Composite {0}', id.substr(id.lastIndexOf('.') + 1)));
            // Use a generated token to avoid race conditions from long running promises
            var currentCompositeOpenToken = uuid.generateUuid();
            this.currentCompositeOpenToken = currentCompositeOpenToken;
            // Emit Composite Opening Event
            this.emit(events_1.EventType.COMPOSITE_OPENING, new events_1.CompositeEvent(id));
            // Hide current
            var hidePromise;
            if (this.activeComposite) {
                hidePromise = this.hideActiveComposite();
            }
            else {
                hidePromise = winjs_base_1.TPromise.as(null);
            }
            return hidePromise.then(function () {
                // Update Title
                _this.updateTitle(id);
                // Create composite
                return _this.createComposite(id, true).then(function (composite) {
                    // Check if another composite opened meanwhile and return in that case
                    if ((_this.currentCompositeOpenToken !== currentCompositeOpenToken) || (_this.activeComposite && _this.activeComposite.getId() !== composite.getId())) {
                        timerEvent.stop();
                        return winjs_base_1.TPromise.as(null);
                    }
                    // Check if composite already visible and just focus in that case
                    if (_this.activeComposite && _this.activeComposite.getId() === composite.getId()) {
                        if (focus) {
                            composite.focus();
                        }
                        timerEvent.stop();
                        // Fullfill promise with composite that is being opened
                        return winjs_base_1.TPromise.as(composite);
                    }
                    // Show Composite and Focus
                    return _this.showComposite(composite).then(function () {
                        if (focus) {
                            composite.focus();
                        }
                        timerEvent.stop();
                        // Fullfill promise with composite that is being opened
                        return composite;
                    });
                });
            });
        };
        CompositePart.prototype.createComposite = function (id, isActive) {
            var _this = this;
            // Check if composite is already created
            for (var i = 0; i < this.instantiatedComposits.length; i++) {
                if (this.instantiatedComposits[i].getId() === id) {
                    return winjs_base_1.TPromise.as(this.instantiatedComposits[i]);
                }
            }
            // Instantiate composite from registry otherwise
            var compositeDescriptor = this.registry.getComposite(id);
            if (compositeDescriptor) {
                var loaderPromise = this.compositeLoaderPromises[id];
                if (!loaderPromise) {
                    var progressService_2 = new progressService_1.WorkbenchProgressService(this.eventService, this.progressBar, compositeDescriptor.id, isActive);
                    var services = {
                        progressService: progressService_2
                    };
                    var compositeInstantiationService = this.instantiationService.createChild(services);
                    loaderPromise = compositeInstantiationService.createInstance(compositeDescriptor).then(function (composite) {
                        _this.mapProgressServiceToComposite[composite.getId()] = progressService_2;
                        // Remember as Instantiated
                        _this.instantiatedComposits.push(composite);
                        // Register to title area update events from the composite
                        _this.instantiatedCompositeListeners.push(composite.addListener(composite_1.EventType.INTERNAL_COMPOSITE_TITLE_AREA_UPDATE, function (e) { _this.onTitleAreaUpdate(e); }));
                        // Remove from Promises Cache since Loaded
                        delete _this.compositeLoaderPromises[id];
                        return composite;
                    });
                    // Report progress for slow loading composits
                    progressService_2.showWhile(loaderPromise, this.partService.isCreated() ? 800 : 3200 /* less ugly initial startup */);
                    // Add to Promise Cache until Loaded
                    this.compositeLoaderPromises[id] = loaderPromise;
                }
                return loaderPromise;
            }
            throw new Error(strings.format('Unable to find composite with id {0}', id));
        };
        CompositePart.prototype.showComposite = function (composite) {
            var _this = this;
            // Remember Composite
            this.activeComposite = composite;
            // Store in preferences
            this.storageService.store(this.activeCompositeSettingsKey, this.activeComposite.getId(), storage_1.StorageScope.WORKSPACE);
            // Remember
            this.lastActiveCompositeId = this.activeComposite.getId();
            // Register as Emitter to Workbench Bus
            this.activeCompositeListeners.push(this.eventService.addEmitter(this.activeComposite, this.activeComposite.getId()));
            var createCompositePromise;
            // Composits created for the first time
            var compositeContainer = this.mapCompositeToCompositeContainer[composite.getId()];
            if (!compositeContainer) {
                // Build Container off-DOM
                compositeContainer = builder_1.$().div({
                    'class': ['composite', this.compositeCssClass],
                    id: composite.getId()
                }, function (div) {
                    createCompositePromise = composite.create(div);
                });
                // Remember composite container
                this.mapCompositeToCompositeContainer[composite.getId()] = compositeContainer;
            }
            else {
                createCompositePromise = winjs_base_1.TPromise.as(null);
            }
            // Report progress for slow loading composits (but only if we did not create the composits before already)
            var progressService = this.mapProgressServiceToComposite[composite.getId()];
            if (progressService && !compositeContainer) {
                this.mapProgressServiceToComposite[composite.getId()].showWhile(createCompositePromise, this.partService.isCreated() ? 800 : 3200 /* less ugly initial startup */);
            }
            // Fill Content and Actions
            return createCompositePromise.then(function () {
                // Make sure that the user meanwhile did not open another composite or closed the part containing the composite
                if (!_this.activeComposite || composite.getId() !== _this.activeComposite.getId()) {
                    return;
                }
                // Take Composite on-DOM and show
                compositeContainer.build(_this.getContentArea());
                compositeContainer.show();
                // Setup action runner
                _this.toolBar.actionRunner = composite.getActionRunner();
                // Update title with composite title if it differs from descriptor
                var descriptor = _this.registry.getComposite(composite.getId());
                if (descriptor && descriptor.name !== composite.getTitle()) {
                    _this.updateTitle(composite.getId(), composite.getTitle());
                }
                // Handle Composite Actions
                var actionsBinding = _this.mapActionsBindingToComposite[composite.getId()];
                if (!actionsBinding) {
                    actionsBinding = _this.collectCompositeActions(composite);
                    _this.mapActionsBindingToComposite[composite.getId()] = actionsBinding;
                }
                actionsBinding();
                if (_this.telemetryActionsListener) {
                    _this.telemetryActionsListener.dispose();
                    _this.telemetryActionsListener = null;
                }
                // Action Run Handling
                _this.telemetryActionsListener = _this.toolBar.actionRunner.addListener2(events.EventType.RUN, function (e) {
                    // Check for Error
                    if (e.error && !errors.isPromiseCanceledError(e.error)) {
                        _this.messageService.show(message_1.Severity.Error, e.error);
                    }
                    // Log in telemetry
                    if (_this.telemetryService) {
                        _this.telemetryService.publicLog('workbenchActionExecuted', { id: e.action.id, from: _this.nameForTelemetry });
                    }
                });
                // Indicate to composite that it is now visible
                return composite.setVisible(true).then(function () {
                    // Make sure that the user meanwhile did not open another composite or closed the part containing the composite
                    if (!_this.activeComposite || composite.getId() !== _this.activeComposite.getId()) {
                        return;
                    }
                    // Make sure the composite is layed out
                    if (_this.contentAreaSize) {
                        composite.layout(_this.contentAreaSize);
                    }
                    // Emit Composite Opened Event
                    _this.emit(events_1.EventType.COMPOSITE_OPENED, new events_1.CompositeEvent(_this.activeComposite.getId()));
                });
            }, function (error) { return _this.onError(error); });
        };
        CompositePart.prototype.onTitleAreaUpdate = function (e) {
            // Active Composite
            if (this.activeComposite && this.activeComposite.getId() === e.compositeId) {
                // Title
                this.updateTitle(this.activeComposite.getId(), this.activeComposite.getTitle());
                // Actions
                var actionsBinding = this.collectCompositeActions(this.activeComposite);
                this.mapActionsBindingToComposite[this.activeComposite.getId()] = actionsBinding;
                actionsBinding();
            }
            else {
                delete this.mapActionsBindingToComposite[e.compositeId];
            }
        };
        CompositePart.prototype.updateTitle = function (compositeId, compositeTitle) {
            var _this = this;
            var compositeDescriptor = this.registry.getComposite(compositeId);
            if (!compositeDescriptor) {
                return;
            }
            if (!compositeTitle) {
                compositeTitle = compositeDescriptor.name;
            }
            var keybinding = null;
            var keys = this.keybindingService.lookupKeybindings(compositeId).map(function (k) { return _this.keybindingService.getLabelFor(k); });
            if (keys && keys.length) {
                keybinding = keys[0];
            }
            this.titleLabel.safeInnerHtml(compositeTitle);
            this.titleLabel.title(keybinding ? nls.localize(0, null, compositeTitle, keybinding) : compositeTitle);
            this.toolBar.setAriaLabel(nls.localize(1, null, compositeTitle));
        };
        CompositePart.prototype.collectCompositeActions = function (composite) {
            // From Composite
            var primaryActions = composite.getActions();
            var secondaryActions = composite.getSecondaryActions();
            // From Part
            primaryActions.push.apply(primaryActions, this.getActions());
            secondaryActions.push.apply(secondaryActions, this.getSecondaryActions());
            // From Contributions
            var actionBarRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
            primaryActions.push.apply(primaryActions, actionBarRegistry.getActionBarActionsForContext(this.actionContributionScope, composite));
            secondaryActions.push.apply(secondaryActions, actionBarRegistry.getSecondaryActionBarActionsForContext(this.actionContributionScope, composite));
            // Return fn to set into toolbar
            return this.toolBar.setActions(actionBarRegistry_1.prepareActions(primaryActions), actionBarRegistry_1.prepareActions(secondaryActions));
        };
        CompositePart.prototype.getActiveComposite = function () {
            return this.activeComposite;
        };
        CompositePart.prototype.getLastActiveCompositetId = function () {
            return this.lastActiveCompositeId;
        };
        CompositePart.prototype.hideActiveComposite = function () {
            var _this = this;
            if (!this.activeComposite) {
                return winjs_base_1.TPromise.as(null); // Nothing to do
            }
            var composite = this.activeComposite;
            this.activeComposite = null;
            var compositeContainer = this.mapCompositeToCompositeContainer[composite.getId()];
            // Indicate to Composite
            return composite.setVisible(false).then(function () {
                // Take Container Off-DOM and hide
                compositeContainer.offDOM();
                compositeContainer.hide();
                // Clear any running Progress
                _this.progressBar.stop().getContainer().hide();
                // Empty Actions
                _this.toolBar.setActions([])();
                // Clear Listeners
                while (_this.activeCompositeListeners.length) {
                    _this.activeCompositeListeners.pop()();
                }
                // Emit Composite Closed Event
                _this.emit(events_1.EventType.COMPOSITE_CLOSED, new events_1.CompositeEvent(composite.getId()));
            });
        };
        CompositePart.prototype.createTitleArea = function (parent) {
            var _this = this;
            // Title Area Container
            var titleArea = builder_1.$(parent).div({
                'class': ['composite', 'title']
            });
            // Right Actions Container
            builder_1.$(titleArea).div({
                'class': 'title-actions'
            }, function (div) {
                // Toolbar
                _this.toolBar = new toolbar_1.ToolBar(div.getHTMLElement(), _this.contextMenuService, {
                    actionItemProvider: function (action) { return _this.actionItemProvider(action); },
                    orientation: actionbar_1.ActionsOrientation.HORIZONTAL
                });
            });
            // Left Title Label
            builder_1.$(titleArea).div({
                'class': 'title-label'
            }, function (div) {
                _this.titleLabel = div.span();
            });
            return titleArea;
        };
        CompositePart.prototype.actionItemProvider = function (action) {
            var actionItem;
            // Check Active Composite
            if (this.activeComposite) {
                actionItem = this.activeComposite.getActionItem(action);
            }
            // Check Registry
            if (!actionItem) {
                var actionBarRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
                actionItem = actionBarRegistry.getActionItemForContext(this.actionContributionScope, toolbar_1.CONTEXT, action);
            }
            return actionItem;
        };
        CompositePart.prototype.createContentArea = function (parent) {
            var _this = this;
            return builder_1.$(parent).div({
                'class': 'content'
            }, function (div) {
                _this.progressBar = new progressbar_1.ProgressBar(div);
                _this.progressBar.getContainer().hide();
            });
        };
        CompositePart.prototype.onError = function (error) {
            this.messageService.show(message_1.Severity.Error, types.isString(error) ? new Error(error) : error);
        };
        CompositePart.prototype.getActions = function () {
            return [];
        };
        CompositePart.prototype.getSecondaryActions = function () {
            return [];
        };
        CompositePart.prototype.layout = function (dimension) {
            // Pass to super
            var sizes = _super.prototype.layout.call(this, dimension);
            // Pass Contentsize to composite
            this.contentAreaSize = sizes[1];
            if (this.activeComposite) {
                this.activeComposite.layout(this.contentAreaSize);
            }
            return sizes;
        };
        CompositePart.prototype.shutdown = function () {
            this.instantiatedComposits.forEach(function (i) { return i.shutdown(); });
            _super.prototype.shutdown.call(this);
        };
        CompositePart.prototype.dispose = function () {
            this.mapCompositeToCompositeContainer = null;
            this.mapProgressServiceToComposite = null;
            this.mapActionsBindingToComposite = null;
            for (var i = 0; i < this.instantiatedComposits.length; i++) {
                this.instantiatedComposits[i].dispose();
            }
            this.instantiatedComposits = [];
            while (this.activeCompositeListeners.length) {
                this.activeCompositeListeners.pop()();
            }
            while (this.instantiatedCompositeListeners.length) {
                this.instantiatedCompositeListeners.pop()();
            }
            this.progressBar.dispose();
            this.toolBar.dispose();
            // Super Dispose
            _super.prototype.dispose.call(this);
        };
        return CompositePart;
    }(part_1.Part));
    exports.CompositePart = CompositePart;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/panel/panelPart", ["require", "exports", 'vs/nls!vs/workbench/browser/parts/panel/panelPart', 'vs/base/common/winjs.base', 'vs/base/common/keyCodes', 'vs/base/common/actions', 'vs/base/browser/dom', 'vs/platform/platform', 'vs/workbench/browser/actionBarRegistry', 'vs/platform/actions/common/actions', 'vs/workbench/common/actionRegistry', 'vs/workbench/browser/parts/compositePart', 'vs/workbench/browser/panel', 'vs/workbench/services/panel/common/panelService', 'vs/workbench/services/part/common/partService', 'vs/css!./media/panelPart'], function (require, exports, nls, winjs_base_1, keyCodes_1, actions_1, dom, platform_1, actionBarRegistry_1, actions_2, actionRegistry_1, compositePart_1, panel_1, panelService_1, partService_1) {
    "use strict";
    var PanelPart = (function (_super) {
        __extends(PanelPart, _super);
        function PanelPart(messageService, storageService, eventService, telemetryService, contextMenuService, partService, keybindingService, id) {
            _super.call(this, messageService, storageService, eventService, telemetryService, contextMenuService, partService, keybindingService, platform_1.Registry.as(panel_1.Extensions.Panels), PanelPart.activePanelSettingsKey, 'panel', 'panel', actionBarRegistry_1.Scope.PANEL, id);
            this.serviceId = panelService_1.IPanelService;
        }
        PanelPart.prototype.create = function (parent) {
            var _this = this;
            _super.prototype.create.call(this, parent);
            dom.addStandardDisposableListener(this.getContainer().getHTMLElement(), 'keyup', function (e) {
                if (e.equals(keyCodes_1.CommonKeybindings.ESCAPE)) {
                    _this.partService.setPanelHidden(true);
                    e.preventDefault();
                }
            });
        };
        PanelPart.prototype.openPanel = function (id, focus) {
            if (this.blockOpeningPanel) {
                return winjs_base_1.TPromise.as(null); // Workaround against a potential race condition
            }
            // First check if panel is hidden and show if so
            if (this.partService.isPanelHidden()) {
                try {
                    this.blockOpeningPanel = true;
                    this.partService.setPanelHidden(false);
                }
                finally {
                    this.blockOpeningPanel = false;
                }
            }
            return this.openComposite(id, focus);
        };
        PanelPart.prototype.getActions = function () {
            return [this.instantiationService.createInstance(ClosePanelAction, ClosePanelAction.ID, ClosePanelAction.LABEL)];
        };
        PanelPart.prototype.getActivePanel = function () {
            return this.getActiveComposite();
        };
        PanelPart.prototype.getLastActivePanelId = function () {
            return this.getLastActiveCompositetId();
        };
        PanelPart.prototype.hideActivePanel = function () {
            return this.hideActiveComposite();
        };
        PanelPart.activePanelSettingsKey = 'workbench.panelpart.activepanelid';
        return PanelPart;
    }(compositePart_1.CompositePart));
    exports.PanelPart = PanelPart;
    var ClosePanelAction = (function (_super) {
        __extends(ClosePanelAction, _super);
        function ClosePanelAction(id, name, partService) {
            _super.call(this, id, name, 'close-editor-action');
            this.partService = partService;
        }
        ClosePanelAction.prototype.run = function () {
            this.partService.setPanelHidden(true);
            return winjs_base_1.TPromise.as(true);
        };
        ClosePanelAction.ID = 'workbench.action.closePanel';
        ClosePanelAction.LABEL = nls.localize(0, null);
        ClosePanelAction = __decorate([
            __param(2, partService_1.IPartService)
        ], ClosePanelAction);
        return ClosePanelAction;
    }(actions_1.Action));
    var TogglePanelAction = (function (_super) {
        __extends(TogglePanelAction, _super);
        function TogglePanelAction(id, name, partService) {
            _super.call(this, id, name, null);
            this.partService = partService;
        }
        TogglePanelAction.prototype.run = function () {
            this.partService.setPanelHidden(!this.partService.isPanelHidden());
            return winjs_base_1.TPromise.as(true);
        };
        TogglePanelAction.ID = 'workbench.action.togglePanel';
        TogglePanelAction.LABEL = nls.localize(1, null);
        TogglePanelAction = __decorate([
            __param(2, partService_1.IPartService)
        ], TogglePanelAction);
        return TogglePanelAction;
    }(actions_1.Action));
    var actionRegistry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    actionRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(TogglePanelAction, TogglePanelAction.ID, TogglePanelAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_J }), nls.localize(2, null));
});

define("vs/workbench/services/quickopen/common/quickOpenService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IQuickOpenService = instantiation_1.createDecorator('quickOpenService');
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/contrib/indentation/common/indentation", ["require", "exports", 'vs/nls!vs/editor/contrib/indentation/common/indentation', 'vs/base/common/winjs.base', 'vs/editor/common/editorAction', 'vs/editor/common/editorCommonExtensions', 'vs/editor/contrib/indentation/common/indentationCommands', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/editor/common/services/modelService', 'vs/editor/common/editorActionEnablement'], function (require, exports, nls, winjs_base_1, editorAction_1, editorCommonExtensions_1, indentationCommands_1, quickOpenService_1, modelService_1, editorActionEnablement_1) {
    "use strict";
    var IndentationToSpacesAction = (function (_super) {
        __extends(IndentationToSpacesAction, _super);
        function IndentationToSpacesAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        IndentationToSpacesAction.prototype.run = function () {
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var modelOpts = model.getOptions();
            var command = new indentationCommands_1.IndentationToSpacesCommand(this.editor.getSelection(), modelOpts.tabSize);
            this.editor.executeCommands(this.id, [command]);
            model.updateOptions({
                insertSpaces: true
            });
            return winjs_base_1.TPromise.as(true);
        };
        IndentationToSpacesAction.ID = 'editor.action.indentationToSpaces';
        return IndentationToSpacesAction;
    }(editorAction_1.EditorAction));
    exports.IndentationToSpacesAction = IndentationToSpacesAction;
    var IndentationToTabsAction = (function (_super) {
        __extends(IndentationToTabsAction, _super);
        function IndentationToTabsAction(descriptor, editor) {
            _super.call(this, descriptor, editor);
        }
        IndentationToTabsAction.prototype.run = function () {
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var modelOpts = model.getOptions();
            var command = new indentationCommands_1.IndentationToTabsCommand(this.editor.getSelection(), modelOpts.tabSize);
            this.editor.executeCommands(this.id, [command]);
            model.updateOptions({
                insertSpaces: false
            });
            return winjs_base_1.TPromise.as(true);
        };
        IndentationToTabsAction.ID = 'editor.action.indentationToTabs';
        return IndentationToTabsAction;
    }(editorAction_1.EditorAction));
    exports.IndentationToTabsAction = IndentationToTabsAction;
    var ChangeIndentationSizeAction = (function (_super) {
        __extends(ChangeIndentationSizeAction, _super);
        function ChangeIndentationSizeAction(descriptor, editor, insertSpaces, quickOpenService, modelService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.Writeable);
            this.insertSpaces = insertSpaces;
            this.quickOpenService = quickOpenService;
            this.modelService = modelService;
        }
        ChangeIndentationSizeAction.prototype.run = function () {
            var _this = this;
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var creationOpts = this.modelService.getCreationOptions();
            var picks = [1, 2, 3, 4, 5, 6, 7, 8].map(function (n) { return ({
                id: n.toString(),
                label: n.toString(),
                // add description for tabSize value set in the configuration
                description: n === creationOpts.tabSize ? nls.localize(0, null) : null
            }); });
            // auto focus the tabSize set for the current editor
            var autoFocusIndex = Math.min(model.getOptions().tabSize - 1, 7);
            return winjs_base_1.TPromise.timeout(50 /* quick open is sensitive to being opened so soon after another */).then(function () {
                return _this.quickOpenService.pick(picks, { placeHolder: nls.localize(1, null), autoFocus: { autoFocusIndex: autoFocusIndex } }).then(function (pick) {
                    if (pick) {
                        model.updateOptions({
                            tabSize: parseInt(pick.label, 10),
                            insertSpaces: _this.insertSpaces
                        });
                    }
                    return true;
                });
            });
        };
        return ChangeIndentationSizeAction;
    }(editorAction_1.EditorAction));
    exports.ChangeIndentationSizeAction = ChangeIndentationSizeAction;
    var IndentUsingTabs = (function (_super) {
        __extends(IndentUsingTabs, _super);
        function IndentUsingTabs(descriptor, editor, quickOpenService, modelService) {
            _super.call(this, descriptor, editor, false, quickOpenService, modelService);
        }
        IndentUsingTabs.ID = 'editor.action.indentUsingTabs';
        IndentUsingTabs = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService),
            __param(3, modelService_1.IModelService)
        ], IndentUsingTabs);
        return IndentUsingTabs;
    }(ChangeIndentationSizeAction));
    exports.IndentUsingTabs = IndentUsingTabs;
    var IndentUsingSpaces = (function (_super) {
        __extends(IndentUsingSpaces, _super);
        function IndentUsingSpaces(descriptor, editor, quickOpenService, modelService) {
            _super.call(this, descriptor, editor, true, quickOpenService, modelService);
        }
        IndentUsingSpaces.ID = 'editor.action.indentUsingSpaces';
        IndentUsingSpaces = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService),
            __param(3, modelService_1.IModelService)
        ], IndentUsingSpaces);
        return IndentUsingSpaces;
    }(ChangeIndentationSizeAction));
    exports.IndentUsingSpaces = IndentUsingSpaces;
    var DetectIndentation = (function (_super) {
        __extends(DetectIndentation, _super);
        function DetectIndentation(descriptor, editor, modelService) {
            _super.call(this, descriptor, editor);
            this.modelService = modelService;
        }
        DetectIndentation.prototype.run = function () {
            var model = this.editor.getModel();
            if (!model) {
                return;
            }
            var creationOpts = this.modelService.getCreationOptions();
            model.detectIndentation(creationOpts.insertSpaces, creationOpts.tabSize);
        };
        DetectIndentation.ID = 'editor.action.detectIndentation';
        DetectIndentation = __decorate([
            __param(2, modelService_1.IModelService)
        ], DetectIndentation);
        return DetectIndentation;
    }(editorAction_1.EditorAction));
    exports.DetectIndentation = DetectIndentation;
    var ToggleRenderWhitespaceAction = (function (_super) {
        __extends(ToggleRenderWhitespaceAction, _super);
        function ToggleRenderWhitespaceAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
        }
        ToggleRenderWhitespaceAction.prototype.run = function () {
            this.editor.updateOptions({
                renderWhitespace: !this.editor.getConfiguration().renderWhitespace
            });
            return winjs_base_1.TPromise.as(true);
        };
        ToggleRenderWhitespaceAction.ID = 'editor.action.toggleRenderWhitespace';
        return ToggleRenderWhitespaceAction;
    }(editorAction_1.EditorAction));
    exports.ToggleRenderWhitespaceAction = ToggleRenderWhitespaceAction;
    // register actions
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(IndentationToSpacesAction, IndentationToSpacesAction.ID, nls.localize(2, null)));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(IndentationToTabsAction, IndentationToTabsAction.ID, nls.localize(3, null)));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(IndentUsingSpaces, IndentUsingSpaces.ID, nls.localize(4, null)));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(IndentUsingTabs, IndentUsingTabs.ID, nls.localize(5, null)));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(DetectIndentation, DetectIndentation.ID, nls.localize(6, null)));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(ToggleRenderWhitespaceAction, ToggleRenderWhitespaceAction.ID, nls.localize(7, null)));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/editor/browser/editor.all", ["require", "exports", 'vs/editor/browser/widget/codeEditorWidget', 'vs/editor/browser/widget/diffEditorWidget', 'vs/editor/contrib/accessibility/browser/accessibility', 'vs/editor/contrib/clipboard/browser/clipboard', 'vs/editor/contrib/codelens/browser/codelens', 'vs/editor/contrib/color/browser/color', 'vs/editor/contrib/comment/common/comment', 'vs/editor/contrib/contextmenu/browser/contextmenu', 'vs/editor/contrib/diffNavigator/common/diffNavigator', 'vs/editor/contrib/find/browser/find', 'vs/editor/contrib/format/common/formatActions', 'vs/editor/contrib/goToDeclaration/browser/goToDeclaration', 'vs/editor/contrib/gotoError/browser/gotoError', 'vs/editor/contrib/hover/browser/hover', 'vs/editor/contrib/inPlaceReplace/common/inPlaceReplace', 'vs/editor/contrib/iPadShowKeyboard/browser/iPadShowKeyboard', 'vs/editor/contrib/linesOperations/common/linesOperations', 'vs/editor/contrib/links/browser/links', 'vs/editor/contrib/multicursor/common/multicursor', 'vs/editor/contrib/outlineMarker/browser/outlineMarker', 'vs/editor/contrib/parameterHints/browser/parameterHints', 'vs/editor/contrib/quickFix/browser/quickFix', 'vs/editor/contrib/referenceSearch/browser/referenceSearch', 'vs/editor/contrib/rename/browser/rename2', 'vs/editor/contrib/smartSelect/common/smartSelect', 'vs/editor/contrib/smartSelect/common/jumpToBracket', 'vs/editor/contrib/snippet/common/snippet', 'vs/editor/contrib/snippet/browser/snippet', 'vs/editor/contrib/suggest/browser/suggest', 'vs/editor/contrib/toggleTabFocusMode/common/toggleTabFocusMode', 'vs/editor/contrib/toggleWordWrap/common/toggleWordWrap', 'vs/editor/contrib/wordHighlighter/common/wordHighlighter', 'vs/editor/contrib/workerStatusReporter/browser/workerStatusReporter', 'vs/editor/contrib/defineKeybinding/browser/defineKeybinding', 'vs/editor/contrib/folding/browser/folding', 'vs/editor/contrib/indentation/common/indentation', 'vs/editor/common/languages.common'], function (require, exports) {
    'use strict';
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostQuickOpen", ["require", "exports", 'vs/base/common/winjs.base', 'vs/platform/thread/common/thread', 'vs/workbench/services/quickopen/common/quickOpenService'], function (require, exports, winjs_base_1, thread_1, quickOpenService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExtHostQuickOpen = (function () {
        function ExtHostQuickOpen(threadService) {
            this._proxy = threadService.getRemotable(MainThreadQuickOpen);
        }
        ExtHostQuickOpen.prototype.show = function (itemsOrItemsPromise, options) {
            var _this = this;
            // clear state from last invocation
            this._onDidSelectItem = undefined;
            var itemsPromise;
            if (!Array.isArray(itemsOrItemsPromise)) {
                itemsPromise = itemsOrItemsPromise;
            }
            else {
                itemsPromise = winjs_base_1.TPromise.as(itemsOrItemsPromise);
            }
            var quickPickWidget = this._proxy.$show({
                autoFocus: { autoFocusFirstEntry: true },
                placeHolder: options && options.placeHolder,
                matchOnDescription: options && options.matchOnDescription,
                matchOnDetail: options && options.matchOnDetail
            });
            return itemsPromise.then(function (items) {
                var pickItems = [];
                for (var handle = 0; handle < items.length; handle++) {
                    var item = items[handle];
                    var label = void 0;
                    var description = void 0;
                    var detail = void 0;
                    if (typeof item === 'string') {
                        label = item;
                    }
                    else {
                        label = item.label;
                        description = item.description;
                        detail = item.detail;
                    }
                    pickItems.push({
                        label: label,
                        description: description,
                        handle: handle,
                        detail: detail
                    });
                }
                // handle selection changes
                if (options && typeof options.onDidSelectItem === 'function') {
                    _this._onDidSelectItem = function (handle) {
                        options.onDidSelectItem(items[handle]);
                    };
                }
                // show items
                _this._proxy.$setItems(pickItems);
                return quickPickWidget.then(function (handle) {
                    if (typeof handle === 'number') {
                        return items[handle];
                    }
                });
            }, function (err) {
                _this._proxy.$setError(err);
                return winjs_base_1.TPromise.wrapError(err);
            });
        };
        ExtHostQuickOpen.prototype.$onItemSelected = function (handle) {
            if (this._onDidSelectItem) {
                this._onDidSelectItem(handle);
            }
        };
        // ---- input
        ExtHostQuickOpen.prototype.input = function (options) {
            this._validateInput = options.validateInput;
            return this._proxy.$input(options, typeof options.validateInput === 'function');
        };
        ExtHostQuickOpen.prototype.$validateInput = function (input) {
            if (this._validateInput) {
                return winjs_base_1.TPromise.as(this._validateInput(input));
            }
        };
        ExtHostQuickOpen = __decorate([
            thread_1.Remotable.ExtHostContext('ExtHostQuickOpen'),
            __param(0, thread_1.IThreadService)
        ], ExtHostQuickOpen);
        return ExtHostQuickOpen;
    }());
    exports.ExtHostQuickOpen = ExtHostQuickOpen;
    var MainThreadQuickOpen = (function () {
        function MainThreadQuickOpen(threadService, quickOpenService) {
            this._token = 0;
            this._proxy = threadService.getRemotable(ExtHostQuickOpen);
            this._quickOpenService = quickOpenService;
        }
        MainThreadQuickOpen.prototype.$show = function (options) {
            var _this = this;
            var myToken = ++this._token;
            this._contents = new winjs_base_1.TPromise(function (c, e) {
                _this._doSetItems = function (items) {
                    if (myToken === _this._token) {
                        c(items);
                    }
                };
                _this._doSetError = function (error) {
                    if (myToken === _this._token) {
                        e(error);
                    }
                };
            });
            return this._quickOpenService.pick(this._contents, options).then(function (item) {
                if (item) {
                    return item.handle;
                }
            }, undefined, function (progress) {
                if (progress) {
                    _this._proxy.$onItemSelected(progress.handle);
                }
            });
        };
        MainThreadQuickOpen.prototype.$setItems = function (items) {
            if (this._doSetItems) {
                this._doSetItems(items);
                return;
            }
        };
        MainThreadQuickOpen.prototype.$setError = function (error) {
            if (this._doSetError) {
                this._doSetError(error);
                return;
            }
        };
        // ---- input
        MainThreadQuickOpen.prototype.$input = function (options, validateInput) {
            var _this = this;
            var inputOptions = Object.create(null);
            if (options) {
                inputOptions.password = options.password;
                inputOptions.placeHolder = options.placeHolder;
                inputOptions.prompt = options.prompt;
                inputOptions.value = options.value;
            }
            if (validateInput) {
                inputOptions.validateInput = function (value) {
                    return _this._proxy.$validateInput(value);
                };
            }
            return this._quickOpenService.input(inputOptions);
        };
        MainThreadQuickOpen = __decorate([
            thread_1.Remotable.MainContext('MainThreadQuickOpen'),
            __param(0, thread_1.IThreadService),
            __param(1, quickOpenService_1.IQuickOpenService)
        ], MainThreadQuickOpen);
        return MainThreadQuickOpen;
    }());
    exports.MainThreadQuickOpen = MainThreadQuickOpen;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/actions/quickOpenAction", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/actions', 'vs/workbench/services/quickopen/common/quickOpenService'], function (require, exports, winjs_base_1, actions_1, quickOpenService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var QuickOpenAction = (function (_super) {
        __extends(QuickOpenAction, _super);
        function QuickOpenAction(actionId, actionLabel, prefix, quickOpenService) {
            _super.call(this, actionId, actionLabel);
            this.quickOpenService = quickOpenService;
            this.prefix = prefix;
            this.enabled = !!this.quickOpenService;
        }
        QuickOpenAction.prototype.run = function () {
            // Show with prefix
            this.quickOpenService.show(this.prefix);
            return winjs_base_1.TPromise.as(null);
        };
        QuickOpenAction = __decorate([
            __param(3, quickOpenService_1.IQuickOpenService)
        ], QuickOpenAction);
        return QuickOpenAction;
    }(actions_1.Action));
    exports.QuickOpenAction = QuickOpenAction;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/actions/triggerQuickOpen", ["require", "exports", 'vs/nls!vs/workbench/browser/actions/triggerQuickOpen', 'vs/workbench/common/actionRegistry', 'vs/platform/platform', 'vs/base/common/actions', 'vs/base/common/winjs.base', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/actions/common/actions', 'vs/base/common/keyCodes'], function (require, exports, nls, actionRegistry_1, platform_1, actions_1, winjs_base_1, keybindingsRegistry_1, quickOpenService_1, keybindingService_1, actions_2, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // Trigger Quick Open
    var GlobalQuickOpenAction = (function (_super) {
        __extends(GlobalQuickOpenAction, _super);
        function GlobalQuickOpenAction(id, label, quickOpenService) {
            _super.call(this, id, label);
            this.quickOpenService = quickOpenService;
            this.order = 100; // Allow other actions to position before or after
            this.class = 'quickopen';
        }
        GlobalQuickOpenAction.prototype.run = function () {
            this.quickOpenService.show(null);
            return winjs_base_1.TPromise.as(true);
        };
        GlobalQuickOpenAction.ID = 'workbench.action.quickOpen';
        GlobalQuickOpenAction.LABEL = nls.localize(0, null);
        GlobalQuickOpenAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], GlobalQuickOpenAction);
        return GlobalQuickOpenAction;
    }(actions_1.Action));
    // Open Previous Editor
    var OpenPreviousEditorAction = (function (_super) {
        __extends(OpenPreviousEditorAction, _super);
        function OpenPreviousEditorAction(id, label, quickOpenService, keybindingService) {
            _super.call(this, id, label);
            this.quickOpenService = quickOpenService;
            this.keybindingService = keybindingService;
        }
        OpenPreviousEditorAction.prototype.run = function () {
            var keys = this.keybindingService.lookupKeybindings(this.id);
            this.quickOpenService.show(null, {
                keybindings: keys
            });
            return winjs_base_1.TPromise.as(true);
        };
        OpenPreviousEditorAction.ID = 'workbench.action.openPreviousEditor';
        OpenPreviousEditorAction.LABEL = nls.localize(1, null);
        OpenPreviousEditorAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService),
            __param(3, keybindingService_1.IKeybindingService)
        ], OpenPreviousEditorAction);
        return OpenPreviousEditorAction;
    }(actions_1.Action));
    var BaseQuickOpenNavigateAction = (function (_super) {
        __extends(BaseQuickOpenNavigateAction, _super);
        function BaseQuickOpenNavigateAction(id, label, navigateNext, quickOpenService, keybindingService) {
            _super.call(this, id, label);
            this.quickOpenService = quickOpenService;
            this.keybindingService = keybindingService;
            this.navigateNext = navigateNext;
        }
        BaseQuickOpenNavigateAction.prototype.run = function (event) {
            var keys = this.keybindingService.lookupKeybindings(this.id);
            this.quickOpenService.quickNavigate({
                keybindings: keys
            }, this.navigateNext);
            return winjs_base_1.TPromise.as(true);
        };
        BaseQuickOpenNavigateAction = __decorate([
            __param(3, quickOpenService_1.IQuickOpenService),
            __param(4, keybindingService_1.IKeybindingService)
        ], BaseQuickOpenNavigateAction);
        return BaseQuickOpenNavigateAction;
    }(actions_1.Action));
    var QuickOpenNavigateNextAction = (function (_super) {
        __extends(QuickOpenNavigateNextAction, _super);
        function QuickOpenNavigateNextAction(id, label, quickOpenService, keybindingService) {
            _super.call(this, id, label, true, quickOpenService, keybindingService);
        }
        QuickOpenNavigateNextAction.ID = 'workbench.action.quickOpenNavigateNext';
        QuickOpenNavigateNextAction.LABEL = nls.localize(2, null);
        QuickOpenNavigateNextAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService),
            __param(3, keybindingService_1.IKeybindingService)
        ], QuickOpenNavigateNextAction);
        return QuickOpenNavigateNextAction;
    }(BaseQuickOpenNavigateAction));
    var QuickOpenNavigatePreviousAction = (function (_super) {
        __extends(QuickOpenNavigatePreviousAction, _super);
        function QuickOpenNavigatePreviousAction(id, label, quickOpenService, keybindingService) {
            _super.call(this, id, label, false, quickOpenService, keybindingService);
        }
        QuickOpenNavigatePreviousAction.ID = 'workbench.action.quickOpenNavigatePrevious';
        QuickOpenNavigatePreviousAction.LABEL = nls.localize(3, null);
        QuickOpenNavigatePreviousAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService),
            __param(3, keybindingService_1.IKeybindingService)
        ], QuickOpenNavigatePreviousAction);
        return QuickOpenNavigatePreviousAction;
    }(BaseQuickOpenNavigateAction));
    var quickOpenKb = {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_P,
        secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_E]
    };
    var QUICK_NAVIGATE_KEY = keyCodes_1.KeyCode.Tab;
    var prevEditorKb = {
        primary: keyCodes_1.KeyMod.CtrlCmd | QUICK_NAVIGATE_KEY,
        secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | QUICK_NAVIGATE_KEY],
        mac: {
            primary: keyCodes_1.KeyMod.WinCtrl | QUICK_NAVIGATE_KEY,
            secondary: [keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyMod.Shift | QUICK_NAVIGATE_KEY]
        }
    };
    function navigateKeybinding(shift) {
        if (shift) {
            return {
                primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_P,
                secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_E, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | QUICK_NAVIGATE_KEY],
                mac: {
                    primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_P,
                    secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_E, keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyMod.Shift | QUICK_NAVIGATE_KEY]
                }
            };
        }
        else {
            return {
                primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_P,
                secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_E, keyCodes_1.KeyMod.CtrlCmd | QUICK_NAVIGATE_KEY],
                mac: {
                    primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_P,
                    secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_E, keyCodes_1.KeyMod.WinCtrl | QUICK_NAVIGATE_KEY]
                }
            };
        }
    }
    // Contribute Quick Open
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(GlobalQuickOpenAction, GlobalQuickOpenAction.ID, GlobalQuickOpenAction.LABEL, quickOpenKb));
    // Contribute Quick Navigate
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(OpenPreviousEditorAction, OpenPreviousEditorAction.ID, OpenPreviousEditorAction.LABEL, prevEditorKb));
    // Contribute Quick Navigate in Quick Open
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(QuickOpenNavigateNextAction, QuickOpenNavigateNextAction.ID, QuickOpenNavigateNextAction.LABEL, navigateKeybinding(false), keybindingService_1.KbExpr.has('inQuickOpen')));
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(QuickOpenNavigatePreviousAction, QuickOpenNavigatePreviousAction.ID, QuickOpenNavigatePreviousAction.LABEL, navigateKeybinding(true), keybindingService_1.KbExpr.has('inQuickOpen'), keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.workbenchContrib(50)));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/quickopen", ["require", "exports", 'vs/nls!vs/workbench/browser/quickopen', 'vs/base/common/winjs.base', 'vs/base/common/filters', 'vs/base/common/arrays', 'vs/base/common/strings', 'vs/base/common/types', 'vs/base/common/errors', 'vs/platform/platform', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/workbench/common/editor', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/platform/instantiation/common/descriptors'], function (require, exports, nls, winjs_base_1, filters, arrays, strings, types, errors, platform_1, quickOpen_1, quickOpenModel_1, editor_1, quickOpenService_1, descriptors_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var QuickOpenHandler = (function () {
        function QuickOpenHandler() {
        }
        /**
         * A quick open handler returns results for a given input string. The resolved promise
         * returns an instance of quick open model. It is up to the handler to keep and reuse an
         * instance of the same model across multiple calls. This helps in situations where the user is
         * narrowing down a search and the model is just filtering some items out.
         *
         * As such, returning the same model instance across multiple searches will yield best
         * results in terms of performance when many items are shown.
         */
        QuickOpenHandler.prototype.getResults = function (searchValue) {
            return winjs_base_1.TPromise.as(null);
        };
        /**
         * The ARIA label to apply when this quick open handler is active in quick open.
         */
        QuickOpenHandler.prototype.getAriaLabel = function () {
            return null;
        };
        /**
         * Extra CSS class name to add to the quick open widget to do custom styling of entries.
         */
        QuickOpenHandler.prototype.getClass = function () {
            return null;
        };
        /**
         * Indicates if the handler can run in the current environment. Return a string if the handler cannot run but has
         * a good message to show in this case.
         */
        QuickOpenHandler.prototype.canRun = function () {
            return true;
        };
        /**
         * Indicates if the handler wishes the quick open widget to automatically select the first result entry or an entry
         * based on a specific prefix match.
         */
        QuickOpenHandler.prototype.getAutoFocus = function (searchValue) {
            return {};
        };
        /**
         * Indicates to the handler that the quick open widget has been closed. Allows to free up any resources as needed.
         * The parameter canceled indicates if the quick open widget was closed with an entry being run or not.
         */
        QuickOpenHandler.prototype.onClose = function (canceled) {
            return;
        };
        /**
         * Allows to return a label that will be placed to the side of the results from this handler or null if none.
         */
        QuickOpenHandler.prototype.getGroupLabel = function () {
            return null;
        };
        /**
         * Allows to return a label that will be used when there are no results found
         */
        QuickOpenHandler.prototype.getEmptyLabel = function (searchString) {
            if (searchString.length > 0) {
                return nls.localize(0, null);
            }
            return nls.localize(1, null);
        };
        return QuickOpenHandler;
    }());
    exports.QuickOpenHandler = QuickOpenHandler;
    /**
     * A lightweight descriptor of a quick open handler.
     */
    var QuickOpenHandlerDescriptor = (function (_super) {
        __extends(QuickOpenHandlerDescriptor, _super);
        function QuickOpenHandlerDescriptor(moduleId, ctorName, prefix, param, instantProgress) {
            if (instantProgress === void 0) { instantProgress = false; }
            _super.call(this, moduleId, ctorName);
            this.prefix = prefix;
            this.id = moduleId + ctorName;
            this.instantProgress = instantProgress;
            if (types.isString(param)) {
                this.description = param;
            }
            else {
                this.helpEntries = param;
            }
        }
        QuickOpenHandlerDescriptor.prototype.getId = function () {
            return this.id;
        };
        return QuickOpenHandlerDescriptor;
    }(descriptors_1.AsyncDescriptor));
    exports.QuickOpenHandlerDescriptor = QuickOpenHandlerDescriptor;
    exports.Extensions = {
        Quickopen: 'workbench.contributions.quickopen'
    };
    var QuickOpenRegistry = (function () {
        function QuickOpenRegistry() {
            this.handlers = [];
            this.defaultHandlers = [];
        }
        QuickOpenRegistry.prototype.registerQuickOpenHandler = function (descriptor) {
            this.handlers.push(descriptor);
            // sort the handlers by decreasing prefix length, such that longer
            // prefixes take priority: 'ext' vs 'ext install' - the latter should win
            this.handlers.sort(function (h1, h2) { return h2.prefix.length - h1.prefix.length; });
        };
        QuickOpenRegistry.prototype.registerDefaultQuickOpenHandler = function (descriptor) {
            this.defaultHandlers.push(descriptor);
        };
        QuickOpenRegistry.prototype.getQuickOpenHandlers = function () {
            return this.handlers.slice(0);
        };
        QuickOpenRegistry.prototype.getQuickOpenHandler = function (text) {
            return text ? arrays.first(this.handlers, function (h) { return strings.startsWith(text, h.prefix); }, null) : null;
        };
        QuickOpenRegistry.prototype.getDefaultQuickOpenHandlers = function () {
            return this.defaultHandlers;
        };
        return QuickOpenRegistry;
    }());
    platform_1.Registry.add(exports.Extensions.Quickopen, new QuickOpenRegistry());
    /**
     * A subclass of quick open entry that will open an editor with input and options when running.
     */
    var EditorQuickOpenEntry = (function (_super) {
        __extends(EditorQuickOpenEntry, _super);
        function EditorQuickOpenEntry(_editorService) {
            _super.call(this);
            this._editorService = _editorService;
        }
        Object.defineProperty(EditorQuickOpenEntry.prototype, "editorService", {
            get: function () {
                return this._editorService;
            },
            enumerable: true,
            configurable: true
        });
        EditorQuickOpenEntry.prototype.getInput = function () {
            return null;
        };
        EditorQuickOpenEntry.prototype.getOptions = function () {
            return null;
        };
        EditorQuickOpenEntry.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.OPEN) {
                var event_1 = context.event;
                var sideBySide = (event_1 && (event_1.ctrlKey || event_1.metaKey || (event_1.payload && event_1.payload.originalEvent && (event_1.payload.originalEvent.ctrlKey || event_1.payload.originalEvent.metaKey))));
                var input = this.getInput();
                if (input instanceof editor_1.EditorInput) {
                    this.editorService.openEditor(input, this.getOptions(), sideBySide).done(null, errors.onUnexpectedError);
                }
                else {
                    this.editorService.openEditor(input, sideBySide).done(null, errors.onUnexpectedError);
                }
                return true;
            }
            return false;
        };
        return EditorQuickOpenEntry;
    }(quickOpenModel_1.QuickOpenEntry));
    exports.EditorQuickOpenEntry = EditorQuickOpenEntry;
    /**
     * A subclass of quick open entry group that provides access to editor input and options.
     */
    var EditorQuickOpenEntryGroup = (function (_super) {
        __extends(EditorQuickOpenEntryGroup, _super);
        function EditorQuickOpenEntryGroup() {
            _super.apply(this, arguments);
        }
        EditorQuickOpenEntryGroup.prototype.getInput = function () {
            return null;
        };
        EditorQuickOpenEntryGroup.prototype.getOptions = function () {
            return null;
        };
        return EditorQuickOpenEntryGroup;
    }(quickOpenModel_1.QuickOpenEntryGroup));
    exports.EditorQuickOpenEntryGroup = EditorQuickOpenEntryGroup;
    var CommandEntry = (function (_super) {
        __extends(CommandEntry, _super);
        function CommandEntry(quickOpenService, prefix, command, highlights) {
            _super.call(this, highlights);
            this.quickOpenService = quickOpenService;
            this.prefix = prefix;
            this.command = command;
            this.command = command;
        }
        CommandEntry.prototype.getIcon = function () {
            return this.command.icon || null;
        };
        CommandEntry.prototype.getLabel = function () {
            return this.command.aliases[0];
        };
        CommandEntry.prototype.getAriaLabel = function () {
            return nls.localize(2, null, this.getLabel());
        };
        CommandEntry.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.PREVIEW) {
                return false;
            }
            this.quickOpenService.show(this.prefix + " " + this.command.aliases[0] + " ");
            return false;
        };
        return CommandEntry;
    }(quickOpenModel_1.QuickOpenEntry));
    var CommandQuickOpenHandler = (function (_super) {
        __extends(CommandQuickOpenHandler, _super);
        function CommandQuickOpenHandler(quickOpenService, options) {
            _super.call(this);
            this.quickOpenService = quickOpenService;
            this.prefix = options.prefix;
            this.commands = options.commands.map(function (c) { return ({
                regexp: new RegExp('^(' + c.aliases.join('|') + ')\\b\\W+'),
                command: c
            }); });
            this.defaultCommand = options.defaultCommand || null;
        }
        CommandQuickOpenHandler.prototype.getResults = function (input) {
            var match;
            var command = arrays.first(this.commands, function (c) { return !!(match = input.match(c.regexp)); });
            var promise;
            if (command) {
                promise = command.command.getResults(input.substr(match[0].length));
            }
            else if (this.defaultCommand) {
                promise = this.defaultCommand.getResults(input);
            }
            else {
                promise = this.getCommands(input);
            }
            return promise.then(function (e) { return new quickOpenModel_1.QuickOpenModel(e); });
        };
        CommandQuickOpenHandler.prototype.getCommands = function (input) {
            var _this = this;
            var entries = this.commands
                .map(function (c) { return ({ command: c.command, highlights: filters.matchesFuzzy(input, c.command.aliases[0]) }); })
                .filter(function (_a) {
                var command = _a.command, highlights = _a.highlights;
                return !!highlights || command.aliases.some(function (a) { return input === a; });
            })
                .map(function (_a) {
                var command = _a.command, highlights = _a.highlights;
                return new CommandEntry(_this.quickOpenService, _this.prefix, command, highlights);
            });
            return winjs_base_1.TPromise.as(entries);
        };
        CommandQuickOpenHandler.prototype.getClass = function () {
            return null;
        };
        CommandQuickOpenHandler.prototype.canRun = function () {
            return true;
        };
        CommandQuickOpenHandler.prototype.getAutoFocus = function (input) {
            return { autoFocusFirstEntry: true };
        };
        CommandQuickOpenHandler.prototype.onClose = function (canceled) {
            return;
        };
        CommandQuickOpenHandler.prototype.getGroupLabel = function () {
            return null;
        };
        CommandQuickOpenHandler.prototype.getEmptyLabel = function (input) {
            var match;
            var command = arrays.first(this.commands, function (c) { return !!(match = input.match(c.regexp)); });
            if (!command) {
                return nls.localize(3, null);
            }
            return command.command.getEmptyLabel(input);
        };
        CommandQuickOpenHandler = __decorate([
            __param(0, quickOpenService_1.IQuickOpenService)
        ], CommandQuickOpenHandler);
        return CommandQuickOpenHandler;
    }(QuickOpenHandler));
    exports.CommandQuickOpenHandler = CommandQuickOpenHandler;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/actions/triggerEditorActions", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/browser/actions/triggerEditorActions', 'vs/base/common/types', 'vs/base/common/actions', 'vs/platform/actions/common/actions', 'vs/workbench/common/actionRegistry', 'vs/platform/platform', 'vs/workbench/common/editor', 'vs/workbench/browser/actionBarRegistry', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/workbench/browser/quickopen', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/services/part/common/partService', 'vs/platform/editor/common/editor', 'vs/platform/instantiation/common/instantiation', 'vs/base/common/keyCodes'], function (require, exports, winjs_base_1, nls, types, actions_1, actions_2, actionRegistry_1, platform_1, editor_1, actionBarRegistry_1, quickOpenModel_1, quickopen_1, editorService_1, quickOpenService_1, partService_1, editor_2, instantiation_1, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var SPLIT_EDITOR_ACTION_ID = 'workbench.action.splitEditor';
    var SPLIT_EDITOR_ACTION_LABEL = nls.localize(0, null);
    var SplitEditorAction = (function (_super) {
        __extends(SplitEditorAction, _super);
        function SplitEditorAction(id, label, editorService) {
            _super.call(this, id, label);
            this.editorService = editorService;
        }
        SplitEditorAction.prototype.run = function () {
            var _this = this;
            // Can only split with active editor
            var activeEditor = this.editorService.getActiveEditor();
            if (!activeEditor) {
                return winjs_base_1.TPromise.as(true);
            }
            // Return if the editor to split does not support split editing
            if (!activeEditor.supportsSplitEditor()) {
                return winjs_base_1.TPromise.as(true);
            }
            // Count editors
            var visibleEditors = this.editorService.getVisibleEditors();
            var editorCount = visibleEditors.length;
            var targetPosition;
            switch (editorCount) {
                // Open split editor to the right of left one
                case 1:
                    targetPosition = editor_2.Position.CENTER;
                    break;
                // Special case two editors opened
                case 2:
                    // Continue splitting to the right
                    if (activeEditor.position === editor_2.Position.CENTER) {
                        targetPosition = editor_2.Position.RIGHT;
                    }
                    else if (activeEditor.position === editor_2.Position.LEFT && !!editor_1.getUntitledOrFileResource(activeEditor.input)) {
                        var centerInput_1 = visibleEditors[editor_2.Position.CENTER].input;
                        var options_1 = new editor_1.TextEditorOptions();
                        options_1.preserveFocus = true;
                        return this.editorService.openEditor(activeEditor.input, options_1, editor_2.Position.CENTER).then(function () {
                            return _this.editorService.openEditor(centerInput_1, options_1, editor_2.Position.RIGHT).then(function () {
                                return _this.editorService.focusEditor(editor_2.Position.CENTER);
                            });
                        });
                    }
            }
            // Only split if the input is resource editor input
            if (!types.isUndefinedOrNull(targetPosition) && !!editor_1.getUntitledOrFileResource(activeEditor.input)) {
                return this.editorService.openEditor(activeEditor.input, null, targetPosition);
            }
            return winjs_base_1.TPromise.as(true);
        };
        SplitEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService)
        ], SplitEditorAction);
        return SplitEditorAction;
    }(actions_1.Action));
    exports.SplitEditorAction = SplitEditorAction;
    var CYCLE_EDITOR_ACTION_ID = 'workbench.action.cycleEditor';
    var CYCLE_EDITOR_ACTION_LABEL = nls.localize(1, null);
    var CycleEditorAction = (function (_super) {
        __extends(CycleEditorAction, _super);
        function CycleEditorAction(id, label, editorService) {
            _super.call(this, id, label);
            this.editorService = editorService;
        }
        CycleEditorAction.prototype.run = function () {
            // Can cycle split with active editor
            var activeEditor = this.editorService.getActiveEditor();
            if (!activeEditor) {
                return winjs_base_1.TPromise.as(false);
            }
            // Cycle to the left and use module to start at 0 again
            var visibleEditors = this.editorService.getVisibleEditors();
            var editorCount = visibleEditors.length;
            var newIndex = (activeEditor.position + 1) % editorCount;
            return this.editorService.focusEditor(newIndex);
        };
        CycleEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService)
        ], CycleEditorAction);
        return CycleEditorAction;
    }(actions_1.Action));
    exports.CycleEditorAction = CycleEditorAction;
    var FOCUS_FIRST_EDITOR_ACTION_ID = 'workbench.action.focusFirstEditor';
    var FOCUS_FIRST_EDITOR_ACTION_LABEL = nls.localize(2, null);
    var FocusFirstEditorAction = (function (_super) {
        __extends(FocusFirstEditorAction, _super);
        function FocusFirstEditorAction(id, label, editorService, quickOpenService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.quickOpenService = quickOpenService;
        }
        FocusFirstEditorAction.prototype.run = function () {
            var _this = this;
            // Find left editor and focus it
            var editors = this.editorService.getVisibleEditors();
            for (var _i = 0, editors_1 = editors; _i < editors_1.length; _i++) {
                var editor = editors_1[_i];
                if (editor.position === editor_2.Position.LEFT) {
                    return this.editorService.focusEditor(editor);
                }
            }
            // Since no editor is currently opened, try to open last history entry to the target side
            var history = this.quickOpenService.getEditorHistory();
            for (var _a = 0, history_1 = history; _a < history_1.length; _a++) {
                var input = history_1[_a];
                // For now only support to open resources from history to the side
                if (!!editor_1.getUntitledOrFileResource(input)) {
                    return this.editorService.openEditor(input, null, editor_2.Position.LEFT).then(function () {
                        // Automatically clean up stale history entries when the input can not be opened
                        if (!input.matches(_this.editorService.getActiveEditorInput())) {
                            _this.quickOpenService.removeEditorHistoryEntry(input);
                        }
                    });
                }
            }
            return winjs_base_1.TPromise.as(true);
        };
        FocusFirstEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], FocusFirstEditorAction);
        return FocusFirstEditorAction;
    }(actions_1.Action));
    exports.FocusFirstEditorAction = FocusFirstEditorAction;
    var BaseFocusSideEditorAction = (function (_super) {
        __extends(BaseFocusSideEditorAction, _super);
        function BaseFocusSideEditorAction(id, label, editorService, quickOpenService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.quickOpenService = quickOpenService;
        }
        BaseFocusSideEditorAction.prototype.run = function () {
            var _this = this;
            // Require at least the reference editor to be visible
            var editors = this.editorService.getVisibleEditors();
            var referenceEditor;
            for (var i = 0; i < editors.length; i++) {
                var editor = editors[i];
                // Target editor exists so focus it
                if (editor.position === this.getTargetEditorSide()) {
                    return this.editorService.focusEditor(editor);
                }
                // Remember reference editor
                if (editor.position === this.getReferenceEditorSide()) {
                    referenceEditor = editor;
                }
            }
            // Require the reference editor to be visible and supporting split editor
            if (referenceEditor && referenceEditor.supportsSplitEditor()) {
                return this.editorService.openEditor(referenceEditor.input, null, this.getTargetEditorSide());
            }
            else if (referenceEditor) {
                var history_2 = this.quickOpenService.getEditorHistory();
                for (var _i = 0, history_3 = history_2; _i < history_3.length; _i++) {
                    var input = history_3[_i];
                    // For now only support to open files from history to the side
                    if (!!editor_1.getUntitledOrFileResource(input)) {
                        return this.editorService.openEditor(input, null, this.getTargetEditorSide()).then(function () {
                            // Automatically clean up stale history entries when the input can not be opened
                            if (!input.matches(_this.editorService.getActiveEditorInput())) {
                                _this.quickOpenService.removeEditorHistoryEntry(input);
                            }
                        });
                    }
                }
            }
            return winjs_base_1.TPromise.as(true);
        };
        BaseFocusSideEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], BaseFocusSideEditorAction);
        return BaseFocusSideEditorAction;
    }(actions_1.Action));
    exports.BaseFocusSideEditorAction = BaseFocusSideEditorAction;
    var FOCUS_SECOND_EDITOR_ACTION_ID = 'workbench.action.focusSecondEditor';
    var FOCUS_SECOND_EDITOR_ACTION_LABEL = nls.localize(3, null);
    var FocusSecondEditorAction = (function (_super) {
        __extends(FocusSecondEditorAction, _super);
        function FocusSecondEditorAction(id, label, editorService, quickOpenService) {
            _super.call(this, id, label, editorService, quickOpenService);
        }
        FocusSecondEditorAction.prototype.getReferenceEditorSide = function () {
            return editor_2.Position.LEFT;
        };
        FocusSecondEditorAction.prototype.getTargetEditorSide = function () {
            return editor_2.Position.CENTER;
        };
        FocusSecondEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], FocusSecondEditorAction);
        return FocusSecondEditorAction;
    }(BaseFocusSideEditorAction));
    exports.FocusSecondEditorAction = FocusSecondEditorAction;
    var FOCUS_THIRD_EDITOR_ACTION_ID = 'workbench.action.focusThirdEditor';
    var FOCUS_THIRD_EDITOR_ACTION_LABEL = nls.localize(4, null);
    var FocusThirdEditorAction = (function (_super) {
        __extends(FocusThirdEditorAction, _super);
        function FocusThirdEditorAction(id, label, editorService, quickOpenService) {
            _super.call(this, id, label, editorService, quickOpenService);
        }
        FocusThirdEditorAction.prototype.getReferenceEditorSide = function () {
            return editor_2.Position.CENTER;
        };
        FocusThirdEditorAction.prototype.getTargetEditorSide = function () {
            return editor_2.Position.RIGHT;
        };
        FocusThirdEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], FocusThirdEditorAction);
        return FocusThirdEditorAction;
    }(BaseFocusSideEditorAction));
    exports.FocusThirdEditorAction = FocusThirdEditorAction;
    var NAVIGATE_LEFT_EDITOR_ACTION_ID = 'workbench.action.focusLeftEditor';
    var NAVIGATE_LEFT_EDITOR_ACTION_LABEL = nls.localize(5, null);
    var NavigateToLeftEditorAction = (function (_super) {
        __extends(NavigateToLeftEditorAction, _super);
        function NavigateToLeftEditorAction(id, label, editorService) {
            _super.call(this, id, label);
            this.editorService = editorService;
        }
        NavigateToLeftEditorAction.prototype.run = function () {
            // Require an active editor
            var activeEditor = this.editorService.getActiveEditor();
            if (!activeEditor) {
                return winjs_base_1.TPromise.as(true);
            }
            // Find the next position to the left
            var nextPosition = editor_2.Position.LEFT;
            if (activeEditor.position === editor_2.Position.RIGHT) {
                nextPosition = editor_2.Position.CENTER;
            }
            // Focus next position if provided
            var visibleEditors = this.editorService.getVisibleEditors();
            return this.editorService.focusEditor(visibleEditors[nextPosition]);
        };
        NavigateToLeftEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService)
        ], NavigateToLeftEditorAction);
        return NavigateToLeftEditorAction;
    }(actions_1.Action));
    exports.NavigateToLeftEditorAction = NavigateToLeftEditorAction;
    var NAVIGATE_RIGHT_EDITOR_ACTION_ID = 'workbench.action.focusRightEditor';
    var NAVIGATE_RIGHT_EDITOR_ACTION_LABEL = nls.localize(6, null);
    var NavigateToRightEditorAction = (function (_super) {
        __extends(NavigateToRightEditorAction, _super);
        function NavigateToRightEditorAction(id, label, editorService, instantiationService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.navigateActions = [];
            this.navigateActions[editor_2.Position.LEFT] = instantiationService.createInstance(FocusFirstEditorAction, FOCUS_FIRST_EDITOR_ACTION_ID, FOCUS_FIRST_EDITOR_ACTION_LABEL);
            this.navigateActions[editor_2.Position.CENTER] = instantiationService.createInstance(FocusSecondEditorAction, FOCUS_SECOND_EDITOR_ACTION_ID, FOCUS_SECOND_EDITOR_ACTION_LABEL);
            this.navigateActions[editor_2.Position.RIGHT] = instantiationService.createInstance(FocusThirdEditorAction, FOCUS_THIRD_EDITOR_ACTION_ID, FOCUS_THIRD_EDITOR_ACTION_LABEL);
        }
        NavigateToRightEditorAction.prototype.run = function (event) {
            // Find the next position to the right to use
            var nextPosition;
            var activeEditor = this.editorService.getActiveEditor();
            if (!activeEditor) {
                nextPosition = editor_2.Position.LEFT;
            }
            else if (activeEditor.position === editor_2.Position.LEFT) {
                nextPosition = editor_2.Position.CENTER;
            }
            else if (activeEditor.position === editor_2.Position.CENTER) {
                nextPosition = editor_2.Position.RIGHT;
            }
            // Run the action for the target next position
            if (!types.isUndefinedOrNull(nextPosition) && this.navigateActions[nextPosition]) {
                return this.navigateActions[nextPosition].run(event);
            }
            return winjs_base_1.TPromise.as(true);
        };
        NavigateToRightEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, instantiation_1.IInstantiationService)
        ], NavigateToRightEditorAction);
        return NavigateToRightEditorAction;
    }(actions_1.Action));
    exports.NavigateToRightEditorAction = NavigateToRightEditorAction;
    var OpenToSideAction = (function (_super) {
        __extends(OpenToSideAction, _super);
        function OpenToSideAction(editorService) {
            _super.call(this, OpenToSideAction.OPEN_TO_SIDE_ID, OpenToSideAction.OPEN_TO_SIDE_LABEL);
            this.editorService = editorService;
            this.class = 'quick-open-sidebyside';
            this.updateEnablement();
        }
        OpenToSideAction.prototype.updateEnablement = function () {
            var activeEditor = this.editorService.getActiveEditor();
            this.enabled = (!activeEditor || activeEditor.position !== editor_2.Position.RIGHT);
        };
        OpenToSideAction.prototype.run = function (context) {
            var _this = this;
            var entry = toEditorQuickOpenEntry(context);
            if (entry) {
                var typedInputPromise = void 0;
                var input = entry.getInput();
                if (input instanceof editor_1.EditorInput) {
                    typedInputPromise = winjs_base_1.TPromise.as(input);
                }
                else {
                    typedInputPromise = this.editorService.inputToType(input);
                }
                return typedInputPromise.then(function (typedInput) { return _this.editorService.openEditor(typedInput, entry.getOptions(), true); });
            }
            return winjs_base_1.TPromise.as(false);
        };
        OpenToSideAction.OPEN_TO_SIDE_ID = 'workbench.action.openToSide';
        OpenToSideAction.OPEN_TO_SIDE_LABEL = nls.localize(7, null);
        OpenToSideAction = __decorate([
            __param(0, editorService_1.IWorkbenchEditorService)
        ], OpenToSideAction);
        return OpenToSideAction;
    }(actions_1.Action));
    exports.OpenToSideAction = OpenToSideAction;
    var QuickOpenActionContributor = (function (_super) {
        __extends(QuickOpenActionContributor, _super);
        function QuickOpenActionContributor(instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
        }
        QuickOpenActionContributor.prototype.hasActions = function (context) {
            var entry = this.getEntry(context);
            return !!entry;
        };
        QuickOpenActionContributor.prototype.getActions = function (context) {
            var actions = [];
            var entry = this.getEntry(context);
            if (entry) {
                if (!this.openToSideActionInstance) {
                    this.openToSideActionInstance = this.instantiationService.createInstance(OpenToSideAction);
                }
                actions.push(this.openToSideActionInstance);
            }
            return actions;
        };
        QuickOpenActionContributor.prototype.getEntry = function (context) {
            if (!context || !context.element) {
                return null;
            }
            return toEditorQuickOpenEntry(context.element);
        };
        QuickOpenActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], QuickOpenActionContributor);
        return QuickOpenActionContributor;
    }(actionBarRegistry_1.ActionBarContributor));
    function toEditorQuickOpenEntry(element) {
        // QuickOpenEntryGroup
        if (element instanceof quickOpenModel_1.QuickOpenEntryGroup) {
            var group = element;
            if (group.getEntry()) {
                element = group.getEntry();
            }
        }
        // EditorQuickOpenEntry or EditorQuickOpenEntryGroup both implement IEditorQuickOpenEntry
        if (element instanceof quickopen_1.EditorQuickOpenEntry || element instanceof quickopen_1.EditorQuickOpenEntryGroup) {
            return element;
        }
        return null;
    }
    var CloseEditorAction = (function (_super) {
        __extends(CloseEditorAction, _super);
        function CloseEditorAction(id, label, editorService) {
            _super.call(this, id, label);
            this.editorService = editorService;
        }
        CloseEditorAction.prototype.run = function () {
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor) {
                return this.editorService.closeEditor(activeEditor);
            }
            return winjs_base_1.TPromise.as(false);
        };
        CloseEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService)
        ], CloseEditorAction);
        return CloseEditorAction;
    }(actions_1.Action));
    exports.CloseEditorAction = CloseEditorAction;
    var CLOSE_ALL_EDITORS_ACTION_ID = 'workbench.action.closeAllEditors';
    var CLOSE_ALL_EDITORS_ACTION_LABEL = nls.localize(8, null);
    var CloseAllEditorsAction = (function (_super) {
        __extends(CloseAllEditorsAction, _super);
        function CloseAllEditorsAction(id, label, editorService) {
            _super.call(this, id, label);
            this.editorService = editorService;
        }
        CloseAllEditorsAction.prototype.run = function () {
            return this.editorService.closeEditors();
        };
        CloseAllEditorsAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService)
        ], CloseAllEditorsAction);
        return CloseAllEditorsAction;
    }(actions_1.Action));
    exports.CloseAllEditorsAction = CloseAllEditorsAction;
    var CLOSE_OTHER_EDITORS_ACTION_ID = 'workbench.action.closeOtherEditors';
    var CLOSE_OTHER_EDITORS_ACTION_LABEL = nls.localize(9, null);
    var CloseOtherEditorsAction = (function (_super) {
        __extends(CloseOtherEditorsAction, _super);
        function CloseOtherEditorsAction(id, label, editorService) {
            _super.call(this, id, label);
            this.editorService = editorService;
        }
        CloseOtherEditorsAction.prototype.run = function () {
            return this.editorService.closeEditors(true);
        };
        CloseOtherEditorsAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService)
        ], CloseOtherEditorsAction);
        return CloseOtherEditorsAction;
    }(actions_1.Action));
    exports.CloseOtherEditorsAction = CloseOtherEditorsAction;
    var MOVE_EDITOR_LEFT_ACTION_ID = 'workbench.action.moveActiveEditorLeft';
    var MOVE_EDITOR_LEFT_ACTION_LABEL = nls.localize(10, null);
    var MoveEditorLeftAction = (function (_super) {
        __extends(MoveEditorLeftAction, _super);
        function MoveEditorLeftAction(id, label, editorService) {
            _super.call(this, id, label);
            this.editorService = editorService;
        }
        MoveEditorLeftAction.prototype.run = function () {
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor && (activeEditor.position === editor_2.Position.CENTER || activeEditor.position === editor_2.Position.RIGHT)) {
                var newPosition = (activeEditor.position === editor_2.Position.CENTER) ? editor_2.Position.LEFT : editor_2.Position.CENTER;
                // Move editor
                this.editorService.moveEditor(activeEditor.position, newPosition);
            }
            return winjs_base_1.TPromise.as(false);
        };
        MoveEditorLeftAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService)
        ], MoveEditorLeftAction);
        return MoveEditorLeftAction;
    }(actions_1.Action));
    exports.MoveEditorLeftAction = MoveEditorLeftAction;
    var MOVE_EDITOR_RIGHT_ACTION_ID = 'workbench.action.moveActiveEditorRight';
    var MOVE_EDITOR_RIGHT_ACTION_LABEL = nls.localize(11, null);
    var MoveEditorRightAction = (function (_super) {
        __extends(MoveEditorRightAction, _super);
        function MoveEditorRightAction(id, label, editorService) {
            _super.call(this, id, label);
            this.editorService = editorService;
        }
        MoveEditorRightAction.prototype.run = function () {
            var editors = this.editorService.getVisibleEditors();
            var activeEditor = this.editorService.getActiveEditor();
            if ((editors.length === 2 && activeEditor.position === editor_2.Position.LEFT) || (editors.length === 3 && activeEditor.position !== editor_2.Position.RIGHT)) {
                var newPosition = (activeEditor.position === editor_2.Position.LEFT) ? editor_2.Position.CENTER : editor_2.Position.RIGHT;
                // Move editor
                this.editorService.moveEditor(activeEditor.position, newPosition);
            }
            return winjs_base_1.TPromise.as(false);
        };
        MoveEditorRightAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService)
        ], MoveEditorRightAction);
        return MoveEditorRightAction;
    }(actions_1.Action));
    exports.MoveEditorRightAction = MoveEditorRightAction;
    var MINIMIZE_EDITORS_ACTION_ID = 'workbench.action.minimizeOtherEditors';
    var MINIMIZE_EDITORS_ACTION_LABEL = nls.localize(12, null);
    var MinimizeOtherEditorsAction = (function (_super) {
        __extends(MinimizeOtherEditorsAction, _super);
        function MinimizeOtherEditorsAction(id, label, editorService) {
            _super.call(this, id, label);
            this.editorService = editorService;
        }
        MinimizeOtherEditorsAction.prototype.run = function () {
            this.editorService.arrangeEditors(editorService_1.EditorArrangement.MINIMIZE_OTHERS);
            return winjs_base_1.TPromise.as(false);
        };
        MinimizeOtherEditorsAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService)
        ], MinimizeOtherEditorsAction);
        return MinimizeOtherEditorsAction;
    }(actions_1.Action));
    exports.MinimizeOtherEditorsAction = MinimizeOtherEditorsAction;
    var EVEN_EDITOR_WIDTHS_ACTION_ID = 'workbench.action.evenEditorWidths';
    var EVEN_EDITOR_WIDTHS_ACTION_LABEL = nls.localize(13, null);
    var EvenEditorWidthsAction = (function (_super) {
        __extends(EvenEditorWidthsAction, _super);
        function EvenEditorWidthsAction(id, label, editorService) {
            _super.call(this, id, label);
            this.editorService = editorService;
        }
        EvenEditorWidthsAction.prototype.run = function () {
            this.editorService.arrangeEditors(editorService_1.EditorArrangement.EVEN_WIDTH);
            return winjs_base_1.TPromise.as(false);
        };
        EvenEditorWidthsAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService)
        ], EvenEditorWidthsAction);
        return EvenEditorWidthsAction;
    }(actions_1.Action));
    exports.EvenEditorWidthsAction = EvenEditorWidthsAction;
    var MAXIMIZE_EDITOR_ACTION_ID = 'workbench.action.maximizeEditor';
    var MAXIMIZE_EDITOR_ACTION_LABEL = nls.localize(14, null);
    var MaximizeEditorAction = (function (_super) {
        __extends(MaximizeEditorAction, _super);
        function MaximizeEditorAction(id, label, editorService, partService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.partService = partService;
        }
        MaximizeEditorAction.prototype.run = function () {
            if (this.editorService.getActiveEditor()) {
                this.editorService.arrangeEditors(editorService_1.EditorArrangement.MINIMIZE_OTHERS);
                this.partService.setSideBarHidden(true);
            }
            return winjs_base_1.TPromise.as(false);
        };
        MaximizeEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, partService_1.IPartService)
        ], MaximizeEditorAction);
        return MaximizeEditorAction;
    }(actions_1.Action));
    exports.MaximizeEditorAction = MaximizeEditorAction;
    // Contribute to Quick Open
    var actionBarRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
    actionBarRegistry.registerActionBarContributor(actionBarRegistry_1.Scope.VIEWER, QuickOpenActionContributor);
    // Contribute to Workbench Actions
    var category = nls.localize(15, null);
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(CloseAllEditorsAction, CLOSE_ALL_EDITORS_ACTION_ID, CLOSE_ALL_EDITORS_ACTION_LABEL), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(CloseOtherEditorsAction, CLOSE_OTHER_EDITORS_ACTION_ID, CLOSE_OTHER_EDITORS_ACTION_LABEL), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(SplitEditorAction, SPLIT_EDITOR_ACTION_ID, SPLIT_EDITOR_ACTION_LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.US_BACKSLASH }), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(CycleEditorAction, CYCLE_EDITOR_ACTION_ID, CYCLE_EDITOR_ACTION_LABEL, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.US_BACKTICK,
        // on mac this keybinding is reserved to cycle between windows
        mac: { primary: null }
    }), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(FocusFirstEditorAction, FOCUS_FIRST_EDITOR_ACTION_ID, FOCUS_FIRST_EDITOR_ACTION_LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_1 }), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(FocusSecondEditorAction, FOCUS_SECOND_EDITOR_ACTION_ID, FOCUS_SECOND_EDITOR_ACTION_LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_2 }), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(FocusThirdEditorAction, FOCUS_THIRD_EDITOR_ACTION_ID, FOCUS_THIRD_EDITOR_ACTION_LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_3 }), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(EvenEditorWidthsAction, EVEN_EDITOR_WIDTHS_ACTION_ID, EVEN_EDITOR_WIDTHS_ACTION_LABEL), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(MaximizeEditorAction, MAXIMIZE_EDITOR_ACTION_ID, MAXIMIZE_EDITOR_ACTION_LABEL), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(MinimizeOtherEditorsAction, MINIMIZE_EDITORS_ACTION_ID, MINIMIZE_EDITORS_ACTION_LABEL), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(MoveEditorLeftAction, MOVE_EDITOR_LEFT_ACTION_ID, MOVE_EDITOR_LEFT_ACTION_LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.LeftArrow) }), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(MoveEditorRightAction, MOVE_EDITOR_RIGHT_ACTION_ID, MOVE_EDITOR_RIGHT_ACTION_LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.RightArrow) }), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(NavigateToLeftEditorAction, NAVIGATE_LEFT_EDITOR_ACTION_ID, NAVIGATE_LEFT_EDITOR_ACTION_LABEL, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.LeftArrow,
        linux: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.LeftArrow }
    }), category);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(NavigateToRightEditorAction, NAVIGATE_RIGHT_EDITOR_ACTION_ID, NAVIGATE_RIGHT_EDITOR_ACTION_LABEL, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.RightArrow,
        linux: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.Alt | keyCodes_1.KeyCode.RightArrow },
    }), category);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/browser/parts/quickopen/editorHistoryModel", ["require", "exports", 'vs/nls!vs/workbench/browser/parts/quickopen/editorHistoryModel', 'vs/platform/platform', 'vs/base/common/filters', 'vs/base/common/types', 'vs/base/common/paths', 'vs/base/common/labels', 'vs/base/common/events', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/workbench/common/editor', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/browser/quickopen'], function (require, exports, nls, platform_1, filters, types, paths, labels, events_1, quickOpen_1, quickOpenModel_1, editor_1, baseEditor_1, quickopen_1) {
    'use strict';
    var MAX_ENTRIES = 200;
    var EditorHistoryEntry = (function (_super) {
        __extends(EditorHistoryEntry, _super);
        function EditorHistoryEntry(editorService, contextService, input, labelHighlights, descriptionHighlights, model) {
            _super.call(this, editorService);
            this.contextService = contextService;
            this.input = input;
            this.model = model;
            this.resource = editor_1.getUntitledOrFileResource(input);
            this.setHighlights(labelHighlights, descriptionHighlights);
        }
        EditorHistoryEntry.prototype.clone = function (labelHighlights, descriptionHighlights) {
            return new EditorHistoryEntry(this.editorService, this.contextService, this.input, labelHighlights, descriptionHighlights, this.model);
        };
        EditorHistoryEntry.prototype.getPrefix = function () {
            var status = this.input.getStatus();
            if (status && status.decoration) {
                return status.decoration + " ";
            }
            return void 0;
        };
        EditorHistoryEntry.prototype.getLabel = function () {
            return this.input.getName();
        };
        EditorHistoryEntry.prototype.getAriaLabel = function () {
            return nls.localize(0, null, this.getLabel());
        };
        EditorHistoryEntry.prototype.getDescription = function () {
            return this.input.getDescription();
        };
        EditorHistoryEntry.prototype.getResource = function () {
            return this.resource;
        };
        EditorHistoryEntry.prototype.getInput = function () {
            return this.input;
        };
        EditorHistoryEntry.prototype.matches = function (input) {
            return this.input.matches(input);
        };
        EditorHistoryEntry.prototype.run = function (mode, context) {
            var _this = this;
            if (mode === quickOpen_1.Mode.OPEN) {
                var event_1 = context.event;
                var sideBySide = !context.quickNavigateConfiguration && (event_1 && (event_1.ctrlKey || event_1.metaKey || (event_1.payload && event_1.payload.originalEvent && (event_1.payload.originalEvent.ctrlKey || event_1.payload.originalEvent.metaKey))));
                this.editorService.openEditor(this.input, null, sideBySide).done(function () {
                    // Automatically clean up stale history entries when the input can not be opened
                    if (!_this.input.matches(_this.editorService.getActiveEditorInput())) {
                        _this.model.remove(_this.input);
                    }
                });
                return true;
            }
            return false;
        };
        return EditorHistoryEntry;
    }(quickopen_1.EditorQuickOpenEntry));
    exports.EditorHistoryEntry = EditorHistoryEntry;
    var EditorHistoryModel = (function (_super) {
        __extends(EditorHistoryModel, _super);
        function EditorHistoryModel(editorService, instantiationService, contextService) {
            _super.call(this);
            this.editorService = editorService;
            this.instantiationService = instantiationService;
            this.contextService = contextService;
        }
        EditorHistoryModel.prototype.add = function (entry) {
            var _this = this;
            // Ensure we have at least a name to show
            if (!entry.getName()) {
                return;
            }
            // Remove on Dispose
            var unbind = entry.addListener(events_1.EventType.DISPOSE, function () {
                _this.remove(entry);
                unbind();
            });
            // Remove any existing entry and add to the beginning
            this.remove(entry);
            this.entries.unshift(new EditorHistoryEntry(this.editorService, this.contextService, entry, null, null, this));
            // Respect max entries setting
            if (this.entries.length > MAX_ENTRIES) {
                this.entries = this.entries.slice(0, MAX_ENTRIES);
            }
        };
        EditorHistoryModel.prototype.remove = function (entry) {
            var index = this.indexOf(entry);
            if (index >= 0) {
                this.entries.splice(index, 1);
            }
        };
        EditorHistoryModel.prototype.indexOf = function (entryToFind) {
            for (var i = 0; i < this.entries.length; i++) {
                var entry = this.entries[i];
                if (entry.matches(entryToFind)) {
                    return i;
                }
            }
            return -1;
        };
        EditorHistoryModel.prototype.saveTo = function (memento) {
            var registry = platform_1.Registry.as(baseEditor_1.Extensions.Editors);
            var entries = [];
            for (var i = this.entries.length - 1; i >= 0; i--) {
                var entry = this.entries[i];
                var input = entry.getInput();
                var factory = registry.getEditorInputFactory(input.getId());
                if (factory) {
                    var value = factory.serialize(input);
                    if (types.isString(value)) {
                        entries.push({
                            id: input.getId(),
                            value: value
                        });
                    }
                }
            }
            if (entries.length > 0) {
                memento.entries = entries;
            }
        };
        EditorHistoryModel.prototype.loadFrom = function (memento) {
            var registry = platform_1.Registry.as(baseEditor_1.Extensions.Editors);
            var entries = memento.entries;
            if (entries && entries.length > 0) {
                for (var i = 0; i < entries.length; i++) {
                    var entry = entries[i];
                    var factory = registry.getEditorInputFactory(entry.id);
                    if (factory && types.isString(entry.value)) {
                        var input = factory.deserialize(this.instantiationService, entry.value);
                        if (input) {
                            this.add(input);
                        }
                    }
                }
            }
        };
        EditorHistoryModel.prototype.getEntries = function () {
            return this.entries.slice(0);
        };
        EditorHistoryModel.prototype.getResults = function (searchValue) {
            searchValue = searchValue.trim();
            var searchInPath = searchValue.indexOf(paths.nativeSep) >= 0;
            var results = [];
            for (var i = 0; i < this.entries.length; i++) {
                var entry = this.entries[i];
                if (!entry.getResource()) {
                    continue; //For now, only support to match on inputs that provide resource information
                }
                // Check if this entry is a match for the search value
                var targetToMatch = searchInPath ? labels.getPathLabel(entry.getResource(), this.contextService) : entry.getLabel();
                if (!filters.matchesFuzzy(searchValue, targetToMatch)) {
                    continue;
                }
                // Apply highlights
                var _a = quickOpenModel_1.QuickOpenEntry.highlight(entry, searchValue), labelHighlights = _a.labelHighlights, descriptionHighlights = _a.descriptionHighlights;
                results.push(entry.clone(labelHighlights, descriptionHighlights));
            }
            // Sort
            return results.sort(function (elementA, elementB) { return quickOpenModel_1.QuickOpenEntry.compare(elementA, elementB, searchValue); });
        };
        return EditorHistoryModel;
    }(quickOpenModel_1.QuickOpenModel));
    exports.EditorHistoryModel = EditorHistoryModel;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/browser/parts/quickopen/quickOpenController", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/browser/parts/quickopen/quickOpenController', 'vs/base/browser/builder', 'vs/base/common/strings', 'vs/base/common/filters', 'vs/base/common/uuid', 'vs/base/common/types', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/base/parts/quickopen/browser/quickOpenWidget', 'vs/workbench/browser/actionBarRegistry', 'vs/platform/platform', 'vs/workbench/common/component', 'vs/workbench/common/events', 'vs/base/common/event', 'vs/workbench/common/constants', 'vs/workbench/common/memento', 'vs/workbench/browser/quickopen', 'vs/workbench/browser/parts/quickopen/editorHistoryModel', 'vs/base/common/errors', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/platform/message/common/message', 'vs/css!./media/quickopen'], function (require, exports, winjs_base_1, nls, builder_1, strings, filters, uuid, types, quickOpen_1, quickOpenModel_1, quickOpenWidget_1, actionBarRegistry_1, platform_1, component_1, events_1, event_1, constants_1, memento_1, quickopen_1, editorHistoryModel_1, errors, quickOpenService_1, message_1) {
    'use strict';
    var ID = 'workbench.component.quickopen';
    var EDITOR_HISTORY_STORAGE_KEY = 'quickopen.editorhistory';
    var HELP_PREFIX = '?';
    var AUTO_SAVE_HISTORY_THRESHOLD = 5;
    var QUICK_OPEN_MODE = 'inQuickOpen';
    var QuickOpenController = (function (_super) {
        __extends(QuickOpenController, _super);
        function QuickOpenController(eventService, storageService, editorService, viewletService, messageService, telemetryService, contextService, keybindingService) {
            _super.call(this, ID);
            this.eventService = eventService;
            this.storageService = storageService;
            this.editorService = editorService;
            this.viewletService = viewletService;
            this.messageService = messageService;
            this.telemetryService = telemetryService;
            this.contextService = contextService;
            this.serviceId = quickOpenService_1.IQuickOpenService;
            this.actionProvider = new actionBarRegistry_1.ContributableActionProvider();
            this.previousValue = '';
            this.mapResolvedHandlersToPrefix = {};
            this.autoSaveHistoryCounter = 0;
            this.promisesToCompleteOnHide = [];
            this.inQuickOpenMode = keybindingService.createKey(QUICK_OPEN_MODE, false);
            this._onShow = new event_1.Emitter();
            this._onHide = new event_1.Emitter();
        }
        Object.defineProperty(QuickOpenController.prototype, "onShow", {
            get: function () {
                return this._onShow.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QuickOpenController.prototype, "onHide", {
            get: function () {
                return this._onHide.event;
            },
            enumerable: true,
            configurable: true
        });
        QuickOpenController.prototype.setInstantiationService = function (service) {
            this.instantiationService = service;
        };
        QuickOpenController.prototype.getEditorHistoryModel = function () {
            return this.editorHistoryModel;
        };
        QuickOpenController.prototype.create = function () {
            var _this = this;
            // Listen on Editor Input Changes to show in MRU List
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.EDITOR_INPUT_CHANGING, function (e) { return _this.onEditorInputChanging(e); }));
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.EDITOR_SET_INPUT_ERROR, function (e) { return _this.onEditorInputSetError(e); }));
            // Editor History Model
            this.editorHistoryModel = new editorHistoryModel_1.EditorHistoryModel(this.editorService, this.instantiationService, this.contextService);
            this.memento = this.getMemento(this.storageService, memento_1.Scope.WORKSPACE);
            if (this.memento[EDITOR_HISTORY_STORAGE_KEY]) {
                this.editorHistoryModel.loadFrom(this.memento[EDITOR_HISTORY_STORAGE_KEY]);
            }
        };
        QuickOpenController.prototype.onEditorInputChanging = function (e) {
            if (e.editorInput) {
                // If an active editor is set, but is different from the one from the event, return early
                var activeEditor = this.editorService.getActiveEditor();
                if (activeEditor && e.editor && activeEditor !== e.editor) {
                    return;
                }
                // Add to History
                this.editorHistoryModel.add(e.editorInput);
                // Save to Local Storage periodically
                if (this.autoSaveHistoryCounter++ >= AUTO_SAVE_HISTORY_THRESHOLD) {
                    this.saveEditorHistory(true);
                    this.autoSaveHistoryCounter = 0;
                }
            }
        };
        QuickOpenController.prototype.onEditorInputSetError = function (e) {
            if (e.editorInput) {
                this.removeEditorHistoryEntry(e.editorInput); // make sure this input does not show up in history if it failed to open
            }
        };
        QuickOpenController.prototype.getEditorHistory = function () {
            return this.editorHistoryModel ? this.editorHistoryModel.getEntries().map(function (entry) { return entry.getInput(); }) : [];
        };
        QuickOpenController.prototype.removeEditorHistoryEntry = function (input) {
            this.editorHistoryModel.remove(input);
        };
        QuickOpenController.prototype.quickNavigate = function (configuration, next) {
            if (this.quickOpenWidget) {
                this.quickOpenWidget.quickNavigate(configuration, next);
            }
        };
        QuickOpenController.prototype.input = function (options) {
            var _this = this;
            var defaultMessage = options && options.prompt
                ? nls.localize(0, null, options.prompt)
                : nls.localize(1, null);
            var currentPick = defaultMessage;
            var currentValidation = winjs_base_1.TPromise.as(true);
            var currentDecoration;
            var lastValue = options && options.value || '';
            var init = function (resolve, reject) {
                // open quick pick with just one choise. we will recurse whenever
                // the validation/success message changes
                _this.doPick(winjs_base_1.TPromise.as([{ label: currentPick }]), {
                    ignoreFocusLost: true,
                    autoFocus: { autoFocusFirstEntry: true },
                    password: options.password,
                    placeHolder: options.placeHolder,
                    value: options.value,
                    inputDecoration: currentDecoration,
                    onDidType: function (value) {
                        lastValue = value;
                        if (options.validateInput) {
                            if (currentValidation) {
                                currentValidation.cancel();
                            }
                            currentValidation = winjs_base_1.TPromise.timeout(100).then(function () {
                                return options.validateInput(value).then(function (message) {
                                    currentDecoration = !!message ? message_1.Severity.Error : void 0;
                                    var newPick = message || defaultMessage;
                                    if (newPick !== currentPick) {
                                        currentPick = newPick;
                                        resolve(new winjs_base_1.TPromise(init));
                                    }
                                    return !message;
                                });
                            }, function (err) {
                                // ignore
                            });
                        }
                    }
                }).then(resolve, reject);
            };
            return new winjs_base_1.TPromise(init).then(function (item) {
                return currentValidation.then(function (valid) {
                    if (valid && item) {
                        return lastValue;
                    }
                });
            });
        };
        QuickOpenController.prototype.pick = function (arg1, options) {
            var _this = this;
            if (!options) {
                options = Object.create(null);
            }
            var arrayPromise;
            if (Array.isArray(arg1)) {
                arrayPromise = winjs_base_1.TPromise.as(arg1);
            }
            else if (winjs_base_1.TPromise.is(arg1)) {
                arrayPromise = arg1;
            }
            else {
                throw new Error('illegal input');
            }
            var isAboutStrings = false;
            var entryPromise = arrayPromise.then(function (elements) {
                return elements.map(function (element) {
                    if (typeof element === 'string') {
                        isAboutStrings = true;
                        return { label: element };
                    }
                    else {
                        return element;
                    }
                });
            });
            return new winjs_base_1.TPromise(function (resolve, reject, progress) {
                function onItem(item) {
                    return item && isAboutStrings ? item.label : item;
                }
                _this.doPick(entryPromise, options).then(function (item) { return resolve(onItem(item)); }, function (err) { return reject(err); }, function (item) { return progress(onItem(item)); });
            });
        };
        QuickOpenController.prototype.doPick = function (picksPromise, options) {
            var _this = this;
            var autoFocus = options.autoFocus;
            // Use a generated token to avoid race conditions from long running promises
            var currentPickerToken = uuid.generateUuid();
            this.currentPickerToken = currentPickerToken;
            // Create upon first open
            if (!this.pickOpenWidget) {
                this.pickOpenWidget = new quickOpenWidget_1.QuickOpenWidget(builder_1.withElementById(constants_1.Identifiers.WORKBENCH_CONTAINER).getHTMLElement(), {
                    onOk: function () { },
                    onCancel: function () { },
                    onType: function (value) { },
                    onShow: function () { return _this.emitQuickOpenVisibilityChange(true); },
                    onHide: function () {
                        _this.restoreFocus(); // focus back to editor or viewlet
                        _this.emitQuickOpenVisibilityChange(false); // event
                    }
                }, {
                    inputPlaceHolder: options.placeHolder || ''
                }, this.telemetryService);
                this.pickOpenWidget.create();
            }
            else {
                this.pickOpenWidget.setPlaceHolder(options.placeHolder || '');
            }
            // Respect input value
            if (options.value) {
                this.pickOpenWidget.setValue(options.value);
            }
            // Respect password
            this.pickOpenWidget.setPassword(options.password);
            // Input decoration
            if (!types.isUndefinedOrNull(options.inputDecoration)) {
                this.pickOpenWidget.showInputDecoration(options.inputDecoration);
            }
            else {
                this.pickOpenWidget.clearInputDecoration();
            }
            // Layout
            if (this.layoutDimensions) {
                this.pickOpenWidget.layout(this.layoutDimensions);
            }
            return new winjs_base_1.TPromise(function (complete, error, progress) {
                var picksPromiseDone = false;
                // Resolve picks
                picksPromise.then(function (picks) {
                    if (_this.currentPickerToken !== currentPickerToken) {
                        return; // Return if another request came after
                    }
                    picksPromiseDone = true;
                    // Reset Progress
                    _this.pickOpenWidget.getProgressBar().stop().getContainer().hide();
                    // Model
                    var model = new quickOpenModel_1.QuickOpenModel();
                    var entries = picks.map(function (e) {
                        var entry = e;
                        if (entry.height && entry.render) {
                            return new PickOpenItem(entry.label, entry.description, entry.height, entry.render.bind(entry), function () { return progress(e); });
                        }
                        return new PickOpenEntry(entry.label, entry.description, entry.detail, function () { return progress(e); }, entry.separator && entry.separator.border, entry.separator && entry.separator.label);
                    });
                    if (picks.length === 0) {
                        entries.push(new PickOpenEntry(nls.localize(2, null)));
                    }
                    model.setEntries(entries);
                    // Handlers
                    _this.pickOpenWidget.setCallbacks({
                        onOk: function () {
                            if (picks.length === 0) {
                                return complete(null);
                            }
                            var index = -1;
                            entries.forEach(function (entry, i) {
                                if (entry.selected) {
                                    index = i;
                                }
                            });
                            complete(picks[index] || null);
                        },
                        onCancel: function () { return complete(void 0); },
                        onFocusLost: function () { return options.ignoreFocusLost; },
                        onType: function (value) {
                            // the caller takes care of all input
                            if (options.onDidType) {
                                options.onDidType(value);
                                return;
                            }
                            if (picks.length === 0) {
                                return;
                            }
                            value = value ? strings.trim(value) : value;
                            // Reset filtering
                            if (!value) {
                                entries.forEach(function (e) {
                                    e.setHighlights(null);
                                    e.setHidden(false);
                                });
                            }
                            else {
                                entries.forEach(function (entry) {
                                    var labelHighlights = filters.matchesFuzzy(value, entry.getLabel());
                                    var descriptionHighlights = options.matchOnDescription
                                        && filters.matchesFuzzy(value, entry.getDescription());
                                    var detailHighlights = options.matchOnDetail && entry.getDetail()
                                        && filters.matchesFuzzy(value, entry.getDetail());
                                    if (labelHighlights || descriptionHighlights || detailHighlights) {
                                        entry.setHighlights(labelHighlights, descriptionHighlights, detailHighlights);
                                        entry.setHidden(false);
                                    }
                                    else {
                                        entry.setHighlights(null, null, null);
                                        entry.setHidden(true);
                                    }
                                });
                            }
                            _this.pickOpenWidget.refresh(model, value ? { autoFocusFirstEntry: true } : autoFocus);
                        },
                        onShow: function () {
                            _this.emitQuickOpenVisibilityChange(true); // event
                        },
                        onHide: function () {
                            _this.restoreFocus(); // focus back to editor or viewlet
                            _this.emitQuickOpenVisibilityChange(false); // event
                        }
                    });
                    // Set input
                    if (!_this.pickOpenWidget.isVisible()) {
                        _this.pickOpenWidget.show(model, autoFocus);
                    }
                    else {
                        _this.pickOpenWidget.setInput(model, autoFocus);
                    }
                }, function (err) {
                    _this.pickOpenWidget.hide();
                    error(err);
                });
                // Progress if task takes a long time
                winjs_base_1.TPromise.timeout(800).then(function () {
                    if (!picksPromiseDone && _this.currentPickerToken === currentPickerToken) {
                        _this.pickOpenWidget.getProgressBar().infinite().getContainer().show();
                    }
                });
                // Show picker empty if resolving takes a while
                if (!picksPromiseDone) {
                    _this.pickOpenWidget.show(new quickOpenModel_1.QuickOpenModel());
                }
            });
        };
        QuickOpenController.prototype.emitQuickOpenVisibilityChange = function (isVisible) {
            var _this = this;
            if (this.visibilityChangeTimeoutHandle) {
                window.clearTimeout(this.visibilityChangeTimeoutHandle);
            }
            this.visibilityChangeTimeoutHandle = setTimeout(function () {
                if (isVisible) {
                    _this._onShow.fire();
                }
                else {
                    _this._onHide.fire();
                }
                _this.visibilityChangeTimeoutHandle = void 0;
            }, 100 /* to prevent flashing, we accumulate visibility changes over a timeout of 100ms */);
        };
        QuickOpenController.prototype.refresh = function (input) {
            if (!this.quickOpenWidget.isVisible()) {
                return winjs_base_1.TPromise.as(null);
            }
            if (input && this.previousValue !== input) {
                return winjs_base_1.TPromise.as(null);
            }
            return this.show(this.previousValue);
        };
        QuickOpenController.prototype.show = function (prefix, quickNavigateConfiguration) {
            var _this = this;
            this.previousValue = prefix;
            var promiseCompletedOnHide = new winjs_base_1.TPromise(function (c) {
                _this.promisesToCompleteOnHide.push(c);
            });
            // Telemetry: log that quick open is shown and log the mode
            var registry = platform_1.Registry.as(quickopen_1.Extensions.Quickopen);
            var handlerDescriptor = registry.getQuickOpenHandler(prefix);
            if (!handlerDescriptor) {
                var defaultHandlerDescriptors = registry.getDefaultQuickOpenHandlers();
                if (defaultHandlerDescriptors.length > 0) {
                    handlerDescriptor = defaultHandlerDescriptors[0];
                }
            }
            if (handlerDescriptor) {
                this.telemetryService.publicLog('quickOpenWidgetShown', { mode: handlerDescriptor.getId(), quickNavigate: !!quickNavigateConfiguration });
            }
            // Create upon first open
            if (!this.quickOpenWidget) {
                this.quickOpenWidget = new quickOpenWidget_1.QuickOpenWidget(builder_1.withElementById(constants_1.Identifiers.WORKBENCH_CONTAINER).getHTMLElement(), {
                    onOk: function () { return _this.onClose(false); },
                    onCancel: function () { return _this.onCancel(); },
                    onType: function (value) { return _this.onType(value || ''); },
                    onShow: function () {
                        _this.inQuickOpenMode.set(true);
                        _this.emitQuickOpenVisibilityChange(true);
                    },
                    onHide: function () {
                        _this.inQuickOpenMode.reset();
                        // Complete promises that are waiting
                        while (_this.promisesToCompleteOnHide.length) {
                            _this.promisesToCompleteOnHide.pop()(true);
                        }
                        _this.restoreFocus(); // focus back to editor or viewlet
                        _this.emitQuickOpenVisibilityChange(false);
                    }
                }, {
                    inputPlaceHolder: this.hasHandler(HELP_PREFIX) ? nls.localize(3, null) : ''
                }, this.telemetryService);
                this.quickOpenWidget.create();
            }
            // Layout
            if (this.layoutDimensions) {
                this.quickOpenWidget.layout(this.layoutDimensions);
            }
            // Show quick open with prefix or editor history
            if (!this.quickOpenWidget.isVisible() || quickNavigateConfiguration) {
                if (prefix) {
                    this.quickOpenWidget.show(prefix);
                }
                else {
                    var editorHistory = this.getEditorHistoryModelWithGroupLabel();
                    if (editorHistory.getEntries().length < 2) {
                        quickNavigateConfiguration = null; // If no entries can be shown, default to normal quick open mode
                    }
                    var autoFocus = void 0;
                    if (!quickNavigateConfiguration) {
                        autoFocus = { autoFocusFirstEntry: true };
                    }
                    else {
                        var visibleEditorCount = this.editorService.getVisibleEditors().length;
                        autoFocus = { autoFocusFirstEntry: visibleEditorCount === 0, autoFocusSecondEntry: visibleEditorCount !== 0 };
                    }
                    this.quickOpenWidget.show(editorHistory, autoFocus, quickNavigateConfiguration);
                }
            }
            else {
                this.quickOpenWidget.show(prefix || '');
            }
            return promiseCompletedOnHide;
        };
        QuickOpenController.prototype.hasHandler = function (prefix) {
            return !!platform_1.Registry.as(quickopen_1.Extensions.Quickopen).getQuickOpenHandler(prefix);
        };
        QuickOpenController.prototype.getEditorHistoryModelWithGroupLabel = function () {
            var entries = this.editorHistoryModel.getEntries();
            // Apply label to first entry
            if (entries.length > 0) {
                entries[0] = new quickOpenModel_1.QuickOpenEntryGroup(entries[0], nls.localize(4, null), false);
            }
            return new quickOpenModel_1.QuickOpenModel(entries, this.actionProvider);
        };
        QuickOpenController.prototype.onCancel = function (notifyHandlers) {
            if (notifyHandlers === void 0) { notifyHandlers = true; }
            // Indicate to handlers
            if (notifyHandlers) {
                this.onClose(true);
            }
        };
        QuickOpenController.prototype.onClose = function (canceled) {
            // Clear state
            this.previousActiveHandlerDescriptor = null;
            // Pass to handlers
            for (var prefix in this.mapResolvedHandlersToPrefix) {
                if (this.mapResolvedHandlersToPrefix.hasOwnProperty(prefix)) {
                    var handler = this.mapResolvedHandlersToPrefix[prefix];
                    handler.onClose(canceled);
                }
            }
        };
        QuickOpenController.prototype.restoreFocus = function () {
            // Try to focus active editor
            var editor = this.editorService.getActiveEditor();
            if (editor) {
                editor.focus();
            }
        };
        QuickOpenController.prototype.onType = function (value) {
            var _this = this;
            this.previousValue = value;
            // Use a generated token to avoid race conditions from long running promises
            var currentResultToken = uuid.generateUuid();
            this.currentResultToken = currentResultToken;
            // Reset Progress
            this.quickOpenWidget.getProgressBar().stop().getContainer().hide();
            // Reset Extra Class
            this.quickOpenWidget.setExtraClass(null);
            // Remove leading and trailing whitespace
            var trimmedValue = strings.trim(value);
            // If no value provided, default to editor history
            if (!trimmedValue) {
                this.quickOpenWidget.setInput(this.getEditorHistoryModelWithGroupLabel(), { autoFocusFirstEntry: true });
                return;
            }
            var resultPromise;
            var resultPromiseDone = false;
            // look for a handler
            var registry = platform_1.Registry.as(quickopen_1.Extensions.Quickopen);
            var handlerDescriptor = registry.getQuickOpenHandler(value);
            if (handlerDescriptor) {
                resultPromise = this.handleSpecificHandler(handlerDescriptor, value, currentResultToken);
            }
            else {
                var defaultHandlers = registry.getDefaultQuickOpenHandlers();
                resultPromise = this.handleDefaultHandlers(defaultHandlers, value, currentResultToken);
            }
            // Remember as the active one
            this.previousActiveHandlerDescriptor = handlerDescriptor;
            // Progress if task takes a long time
            winjs_base_1.TPromise.timeout(handlerDescriptor && handlerDescriptor.instantProgress ? 0 : 800).then(function () {
                if (!resultPromiseDone && currentResultToken === _this.currentResultToken) {
                    _this.quickOpenWidget.getProgressBar().infinite().getContainer().show();
                }
            });
            // Promise done handling
            resultPromise.done(function () {
                resultPromiseDone = true;
                if (currentResultToken === _this.currentResultToken) {
                    _this.quickOpenWidget.getProgressBar().getContainer().hide();
                }
            }, function (error) {
                resultPromiseDone = true;
                errors.onUnexpectedError(error);
                _this.messageService.show(message_1.Severity.Error, types.isString(error) ? new Error(error) : error);
            });
        };
        QuickOpenController.prototype.handleDefaultHandlers = function (defaultHandlers, value, currentResultToken) {
            var _this = this;
            // Fill in history results if matching
            var matchingHistoryEntries = this.editorHistoryModel.getResults(value);
            if (matchingHistoryEntries.length > 0) {
                matchingHistoryEntries[0] = new quickOpenModel_1.QuickOpenEntryGroup(matchingHistoryEntries[0], nls.localize(5, null), false);
            }
            var quickOpenModel = new quickOpenModel_1.QuickOpenModel(matchingHistoryEntries, this.actionProvider);
            // Set input and await additional results from handlers coming in later
            this.quickOpenWidget.setInput(quickOpenModel, { autoFocusFirstEntry: true });
            // If no handler present, return early
            if (defaultHandlers.length === 0) {
                return winjs_base_1.TPromise.as(null);
            }
            // Resolve all default handlers
            var resolvePromises = [];
            defaultHandlers.forEach(function (defaultHandler) {
                resolvePromises.push(_this.resolveHandler(defaultHandler));
            });
            return winjs_base_1.TPromise.join(resolvePromises).then(function (resolvedHandlers) {
                var resultPromises = [];
                resolvedHandlers.forEach(function (resolvedHandler) {
                    // Return early if the handler can not run in the current environment
                    var canRun = resolvedHandler.canRun();
                    if (types.isUndefinedOrNull(canRun) || (typeof canRun === 'boolean' && !canRun) || typeof canRun === 'string') {
                        return;
                    }
                    // Receive Results from Handler and apply
                    resultPromises.push(resolvedHandler.getResults(value).then(function (result) {
                        if (_this.currentResultToken === currentResultToken) {
                            var handlerResults = result && result.entries;
                            if (!handlerResults) {
                                handlerResults = []; // guard against handler returning nothing
                            }
                            _this.mergeResults(quickOpenModel, handlerResults, resolvedHandler.getGroupLabel());
                        }
                    }));
                });
                return winjs_base_1.TPromise.join(resultPromises).then(function () { return void 0; });
            });
        };
        QuickOpenController.prototype.mergeResults = function (quickOpenModel, handlerResults, groupLabel) {
            // Remove results already showing by checking for a "resource" property
            var mapEntryToResource = this.mapEntriesToResource(quickOpenModel);
            var additionalHandlerResults = [];
            for (var i = 0; i < handlerResults.length; i++) {
                var result = handlerResults[i];
                var resource = result.getResource();
                if (!resource || !mapEntryToResource[resource.toString()]) {
                    additionalHandlerResults.push(result);
                }
            }
            // Show additional handler results below any existing results
            if (additionalHandlerResults.length > 0) {
                var useTopBorder = quickOpenModel.getEntries().length > 0;
                additionalHandlerResults[0] = new quickOpenModel_1.QuickOpenEntryGroup(additionalHandlerResults[0], groupLabel, useTopBorder);
                quickOpenModel.addEntries(additionalHandlerResults);
                this.quickOpenWidget.refresh(quickOpenModel, { autoFocusFirstEntry: true });
            }
            else if (quickOpenModel.getEntries().length === 0) {
                quickOpenModel.addEntries([new PlaceholderQuickOpenEntry(nls.localize(6, null))]);
                this.quickOpenWidget.refresh(quickOpenModel, { autoFocusFirstEntry: true });
            }
        };
        QuickOpenController.prototype.handleSpecificHandler = function (handlerDescriptor, value, currentResultToken) {
            var _this = this;
            return this.resolveHandler(handlerDescriptor).then(function (resolvedHandler) {
                // Remove handler prefix from search value
                value = value.substr(handlerDescriptor.prefix.length);
                // Return early if the handler can not run in the current environment and inform the user
                var canRun = resolvedHandler.canRun();
                if (types.isUndefinedOrNull(canRun) || (typeof canRun === 'boolean' && !canRun) || typeof canRun === 'string') {
                    var placeHolderLabel = (typeof canRun === 'string') ? canRun : nls.localize(7, null);
                    var model = new quickOpenModel_1.QuickOpenModel([new PlaceholderQuickOpenEntry(placeHolderLabel)], _this.actionProvider);
                    _this.showModel(model, resolvedHandler.getAutoFocus(value), resolvedHandler.getAriaLabel());
                    return winjs_base_1.TPromise.as(null);
                }
                // Support extra class from handler
                var extraClass = resolvedHandler.getClass();
                if (extraClass) {
                    _this.quickOpenWidget.setExtraClass(extraClass);
                }
                // When handlers change, clear the result list first before loading the new results
                if (_this.previousActiveHandlerDescriptor !== handlerDescriptor) {
                    _this.clearModel();
                }
                // Receive Results from Handler and apply
                return resolvedHandler.getResults(value).then(function (result) {
                    if (_this.currentResultToken === currentResultToken) {
                        if (!result || !result.entries.length) {
                            var model = new quickOpenModel_1.QuickOpenModel([new PlaceholderQuickOpenEntry(resolvedHandler.getEmptyLabel(value))]);
                            _this.showModel(model, resolvedHandler.getAutoFocus(value), resolvedHandler.getAriaLabel());
                        }
                        else {
                            _this.showModel(result, resolvedHandler.getAutoFocus(value), resolvedHandler.getAriaLabel());
                        }
                    }
                });
            });
        };
        QuickOpenController.prototype.showModel = function (model, autoFocus, ariaLabel) {
            // If the given model is already set in the widget, refresh and return early
            if (this.quickOpenWidget.getInput() === model) {
                this.quickOpenWidget.refresh(model, autoFocus);
                return;
            }
            // Otherwise just set it
            this.quickOpenWidget.setInput(model, autoFocus, ariaLabel);
        };
        QuickOpenController.prototype.clearModel = function () {
            this.showModel(new quickOpenModel_1.QuickOpenModel(), null);
        };
        QuickOpenController.prototype.mapEntriesToResource = function (model) {
            var entries = model.getEntries();
            var mapEntryToPath = {};
            entries.forEach(function (entry) {
                if (entry.getResource()) {
                    mapEntryToPath[entry.getResource().toString()] = entry;
                }
            });
            return mapEntryToPath;
        };
        QuickOpenController.prototype.resolveHandler = function (handler) {
            var _this = this;
            var id = handler.getId();
            // Return Cached
            if (this.mapResolvedHandlersToPrefix[id]) {
                return winjs_base_1.TPromise.as(this.mapResolvedHandlersToPrefix[id]);
            }
            // Otherwise load and create
            return this.instantiationService.createInstance(handler).then(function (resolvedHandler) {
                _this.mapResolvedHandlersToPrefix[id] = resolvedHandler;
                return resolvedHandler;
            }, function (error) {
                return winjs_base_1.TPromise.wrapError('Unable to instanciate quick open handler ' + handler.moduleName + ' - ' + handler.ctorName + ': ' + JSON.stringify(error));
            });
        };
        QuickOpenController.prototype.shutdown = function () {
            // Save Editor Input History
            this.saveEditorHistory(false);
            // Call Super
            _super.prototype.shutdown.call(this);
        };
        QuickOpenController.prototype.saveEditorHistory = function (toLocalStorage) {
            if (!this.memento[EDITOR_HISTORY_STORAGE_KEY]) {
                this.memento[EDITOR_HISTORY_STORAGE_KEY] = {};
            }
            this.editorHistoryModel.saveTo(this.memento[EDITOR_HISTORY_STORAGE_KEY]);
            if (toLocalStorage) {
                this.saveMemento();
            }
        };
        QuickOpenController.prototype.layout = function (dimension) {
            this.layoutDimensions = dimension;
            if (this.quickOpenWidget) {
                this.quickOpenWidget.layout(this.layoutDimensions);
            }
            if (this.pickOpenWidget) {
                this.pickOpenWidget.layout(this.layoutDimensions);
            }
        };
        QuickOpenController.prototype.dispose = function () {
            if (this.quickOpenWidget) {
                this.quickOpenWidget.dispose();
            }
            if (this.pickOpenWidget) {
                this.pickOpenWidget.dispose();
            }
            _super.prototype.dispose.call(this);
        };
        return QuickOpenController;
    }(component_1.WorkbenchComponent));
    exports.QuickOpenController = QuickOpenController;
    var PlaceholderQuickOpenEntry = (function (_super) {
        __extends(PlaceholderQuickOpenEntry, _super);
        function PlaceholderQuickOpenEntry(placeHolderLabel) {
            _super.call(this);
            this.placeHolderLabel = placeHolderLabel;
        }
        PlaceholderQuickOpenEntry.prototype.getLabel = function () {
            return this.placeHolderLabel;
        };
        return PlaceholderQuickOpenEntry;
    }(quickOpenModel_1.QuickOpenEntryGroup));
    var PickOpenEntry = (function (_super) {
        __extends(PickOpenEntry, _super);
        function PickOpenEntry(label, description, detail, onPreview, hasSeparator, separatorLabel) {
            _super.call(this, label);
            this.onPreview = onPreview;
            this.hasSeparator = hasSeparator;
            this.separatorLabel = separatorLabel;
            this.description = description;
            this.detail = detail;
        }
        Object.defineProperty(PickOpenEntry.prototype, "selected", {
            get: function () {
                return this._selected;
            },
            enumerable: true,
            configurable: true
        });
        PickOpenEntry.prototype.getDescription = function () {
            return this.description;
        };
        PickOpenEntry.prototype.getDetail = function () {
            return this.detail;
        };
        PickOpenEntry.prototype.showBorder = function () {
            return this.hasSeparator;
        };
        PickOpenEntry.prototype.getGroupLabel = function () {
            return this.separatorLabel;
        };
        PickOpenEntry.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.OPEN) {
                this._selected = true;
                return true;
            }
            if (mode === quickOpen_1.Mode.PREVIEW && this.onPreview) {
                this.onPreview();
            }
            return false;
        };
        return PickOpenEntry;
    }(PlaceholderQuickOpenEntry));
    var PickOpenItem = (function (_super) {
        __extends(PickOpenItem, _super);
        function PickOpenItem(label, description, height, renderFn, onPreview) {
            _super.call(this);
            this.label = label;
            this.description = description;
            this.height = height;
            this.renderFn = renderFn;
            this.onPreview = onPreview;
        }
        PickOpenItem.prototype.getHeight = function () {
            return this.height;
        };
        PickOpenItem.prototype.render = function (tree, container, previousCleanupFn) {
            return this.renderFn(tree, container, previousCleanupFn);
        };
        Object.defineProperty(PickOpenItem.prototype, "selected", {
            get: function () {
                return this._selected;
            },
            enumerable: true,
            configurable: true
        });
        PickOpenItem.prototype.getLabel = function () {
            return this.label;
        };
        PickOpenItem.prototype.getDescription = function () {
            return this.description;
        };
        PickOpenItem.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.OPEN) {
                this._selected = true;
                return true;
            }
            if (mode === quickOpen_1.Mode.PREVIEW && this.onPreview) {
                this.onPreview();
            }
            return false;
        };
        return PickOpenItem;
    }(quickOpenModel_1.QuickOpenEntryItem));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/extensions/electron-browser/extensionsWidgets", ["require", "exports", 'vs/nls!vs/workbench/parts/extensions/electron-browser/extensionsWidgets', 'vs/base/common/severity', 'vs/base/common/async', 'vs/base/browser/dom', 'vs/base/common/lifecycle', 'vs/base/common/errors', 'vs/base/common/objects', 'vs/workbench/parts/output/common/output', 'vs/platform/extensions/common/extensions', 'vs/platform/instantiation/common/instantiation', 'vs/workbench/parts/extensions/common/extensions', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/parts/extensions/common/extensionsUtil'], function (require, exports, nls, severity_1, async_1, dom_1, lifecycle_1, errors_1, objects_1, output_1, extensions_1, instantiation_1, extensions_2, quickOpenService_1, extensionsUtil_1) {
    "use strict";
    var InitialState = {
        errors: [],
        installing: [],
        outdated: []
    };
    function extensionEquals(one, other) {
        return one.publisher === other.publisher && one.name === other.name;
    }
    var OutdatedPeriod = 5 * 60 * 1000; // every 5 minutes
    var ExtensionsStatusbarItem = (function () {
        function ExtensionsStatusbarItem(extensionService, outputService, extensionsService, instantiationService, quickOpenService) {
            this.extensionService = extensionService;
            this.outputService = outputService;
            this.extensionsService = extensionsService;
            this.instantiationService = instantiationService;
            this.quickOpenService = quickOpenService;
            this.state = InitialState;
            this.outdatedDelayer = new async_1.ThrottledDelayer(OutdatedPeriod);
        }
        ExtensionsStatusbarItem.prototype.render = function (container) {
            var _this = this;
            this.domNode = dom_1.append(container, dom_1.emmet('a.extensions-statusbar'));
            dom_1.append(this.domNode, dom_1.emmet('.icon'));
            this.domNode.onclick = function () { return _this.onClick(); };
            this.checkErrors();
            this.checkOutdated();
            var disposables = [];
            this.extensionsService.onInstallExtension(this.onInstallExtension, this, disposables);
            this.extensionsService.onDidInstallExtension(this.onDidInstallExtension, this, disposables);
            this.extensionsService.onDidUninstallExtension(this.onDidUninstallExtension, this, disposables);
            return lifecycle_1.combinedDispose.apply(void 0, disposables);
        };
        ExtensionsStatusbarItem.prototype.updateState = function (obj) {
            this.state = objects_1.assign(this.state, obj);
            this.onStateChange();
        };
        Object.defineProperty(ExtensionsStatusbarItem.prototype, "hasErrors", {
            get: function () { return this.state.errors.length > 0; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtensionsStatusbarItem.prototype, "isInstalling", {
            get: function () { return this.state.installing.length > 0; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtensionsStatusbarItem.prototype, "hasUpdates", {
            get: function () { return this.state.outdated.length > 0; },
            enumerable: true,
            configurable: true
        });
        ExtensionsStatusbarItem.prototype.onStateChange = function () {
            dom_1.toggleClass(this.domNode, 'has-errors', this.hasErrors);
            dom_1.toggleClass(this.domNode, 'is-installing', !this.hasErrors && this.isInstalling);
            dom_1.toggleClass(this.domNode, 'has-updates', !this.hasErrors && !this.isInstalling && this.hasUpdates);
            if (this.hasErrors) {
                var singular = nls.localize(0, null);
                var plural = nls.localize(1, null, this.state.errors.length);
                this.domNode.title = this.state.errors.length > 1 ? plural : singular;
            }
            else if (this.isInstalling) {
                this.domNode.title = nls.localize(2, null, this.state.installing.length);
            }
            else if (this.hasUpdates) {
                var singular = nls.localize(3, null);
                var plural = nls.localize(4, null, this.state.outdated.length);
                this.domNode.title = this.state.outdated.length > 1 ? plural : singular;
            }
            else {
                this.domNode.title = nls.localize(5, null);
            }
        };
        ExtensionsStatusbarItem.prototype.onClick = function () {
            if (this.hasErrors) {
                this.showErrors(this.state.errors);
                this.updateState({ errors: [] });
            }
            else if (this.hasUpdates) {
                this.quickOpenService.show("ext update ");
            }
            else {
                this.quickOpenService.show(">" + extensions_2.ExtensionsLabel + ": ");
            }
        };
        ExtensionsStatusbarItem.prototype.showErrors = function (errors) {
            var _this = this;
            var promise = errors_1.onUnexpectedPromiseError(this.extensionsService.getInstalled());
            promise.done(function (installed) {
                errors.forEach(function (m) {
                    var extension = installed.filter(function (ext) { return ext.path === m.source; }).pop();
                    var name = extension && extension.name;
                    var message = name ? name + ": " + m.message : m.message;
                    _this.outputService.append(extensions_2.ExtensionsLabel, message);
                    _this.outputService.showOutput(extensions_2.ExtensionsLabel, true);
                });
            });
        };
        ExtensionsStatusbarItem.prototype.onInstallExtension = function (manifest) {
            var installing = this.state.installing.concat([manifest]);
            this.updateState({ installing: installing });
        };
        ExtensionsStatusbarItem.prototype.onDidInstallExtension = function (_a) {
            var _this = this;
            var extension = _a.extension;
            var installing = this.state.installing
                .filter(function (e) { return !extensionEquals(extension, e); });
            this.updateState({ installing: installing });
            this.outdatedDelayer.trigger(function () { return _this.checkOutdated(); }, 0);
        };
        ExtensionsStatusbarItem.prototype.onDidUninstallExtension = function () {
            var _this = this;
            this.outdatedDelayer.trigger(function () { return _this.checkOutdated(); }, 0);
        };
        ExtensionsStatusbarItem.prototype.checkErrors = function () {
            var _this = this;
            var promise = errors_1.onUnexpectedPromiseError(this.extensionService.onReady());
            promise.done(function () {
                var status = _this.extensionService.getExtensionsStatus();
                var errors = Object.keys(status)
                    .map(function (k) { return status[k].messages; })
                    .reduce(function (r, m) { return r.concat(m); }, [])
                    .filter(function (m) { return m.type > severity_1.default.Info; });
                _this.updateState({ errors: errors });
            });
        };
        ExtensionsStatusbarItem.prototype.checkOutdated = function () {
            var _this = this;
            return this.instantiationService.invokeFunction(extensionsUtil_1.getOutdatedExtensions)
                .then(null, function (_) { return []; }) // ignore errors
                .then(function (outdated) {
                _this.updateState({ outdated: outdated });
                // repeat this later
                _this.outdatedDelayer.trigger(function () { return _this.checkOutdated(); });
            });
        };
        ExtensionsStatusbarItem = __decorate([
            __param(0, extensions_1.IExtensionService),
            __param(1, output_1.IOutputService),
            __param(2, extensions_2.IExtensionsService),
            __param(3, instantiation_1.IInstantiationService),
            __param(4, quickOpenService_1.IQuickOpenService)
        ], ExtensionsStatusbarItem);
        return ExtensionsStatusbarItem;
    }());
    exports.ExtensionsStatusbarItem = ExtensionsStatusbarItem;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/git/browser/gitQuickOpen", ["require", "exports", 'vs/nls!vs/workbench/parts/git/browser/gitQuickOpen', 'vs/base/common/filters', 'vs/base/common/winjs.base', 'vs/base/common/severity', 'vs/workbench/parts/git/common/git', 'vs/workbench/browser/quickopen', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/platform/message/common/message'], function (require, exports, nls, filters, winjs, severity_1, git, quickopenwb, quickopen, model, quickOpenService_1, message_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var IGitService = git.IGitService;
    // Entries
    var AbstractRefEntry = (function (_super) {
        __extends(AbstractRefEntry, _super);
        function AbstractRefEntry(gitService, messageService, head, highlights) {
            _super.call(this, highlights);
            this.gitService = gitService;
            this.messageService = messageService;
            this.head = head;
        }
        AbstractRefEntry.prototype.getIcon = function () { return 'git'; };
        AbstractRefEntry.prototype.getLabel = function () { return this.head.name; };
        AbstractRefEntry.prototype.getDescription = function () { return ''; };
        AbstractRefEntry.prototype.getAriaLabel = function () { return nls.localize(0, null, this.getLabel()); };
        AbstractRefEntry.prototype.run = function (mode, context) {
            if (mode === quickopen.Mode.PREVIEW) {
                return false;
            }
            return true;
        };
        return AbstractRefEntry;
    }(model.QuickOpenEntry));
    var CheckoutHeadEntry = (function (_super) {
        __extends(CheckoutHeadEntry, _super);
        function CheckoutHeadEntry() {
            _super.apply(this, arguments);
        }
        CheckoutHeadEntry.prototype.getDescription = function () { return nls.localize(1, null, this.head.commit.substr(0, 8)); };
        CheckoutHeadEntry.prototype.run = function (mode, context) {
            var _this = this;
            if (mode === quickopen.Mode.PREVIEW) {
                return false;
            }
            this.gitService.checkout(this.head.name).done(null, function (e) { return _this.messageService.show(severity_1.default.Error, e); });
            return true;
        };
        return CheckoutHeadEntry;
    }(AbstractRefEntry));
    var CheckoutTagEntry = (function (_super) {
        __extends(CheckoutTagEntry, _super);
        function CheckoutTagEntry() {
            _super.apply(this, arguments);
        }
        CheckoutTagEntry.prototype.getDescription = function () { return nls.localize(2, null, this.head.commit.substr(0, 8)); };
        CheckoutTagEntry.prototype.run = function (mode, context) {
            var _this = this;
            if (mode === quickopen.Mode.PREVIEW) {
                return false;
            }
            this.gitService.checkout(this.head.name).done(null, function (e) { return _this.messageService.show(severity_1.default.Error, e); });
            return true;
        };
        return CheckoutTagEntry;
    }(AbstractRefEntry));
    var CurrentHeadEntry = (function (_super) {
        __extends(CurrentHeadEntry, _super);
        function CurrentHeadEntry() {
            _super.apply(this, arguments);
        }
        CurrentHeadEntry.prototype.getDescription = function () { return nls.localize(3, null, this.head.name); };
        return CurrentHeadEntry;
    }(AbstractRefEntry));
    var BranchEntry = (function (_super) {
        __extends(BranchEntry, _super);
        function BranchEntry(gitService, messageService, name) {
            _super.call(this, [{ start: 0, end: name.length }]);
            this.gitService = gitService;
            this.messageService = messageService;
            this.name = name;
        }
        BranchEntry.prototype.getIcon = function () { return 'git'; };
        BranchEntry.prototype.getLabel = function () { return this.name; };
        BranchEntry.prototype.getAriaLabel = function () { return nls.localize(4, null, this.getLabel()); };
        BranchEntry.prototype.getDescription = function () { return nls.localize(5, null, this.name); };
        BranchEntry.prototype.run = function (mode, context) {
            var _this = this;
            if (mode === quickopen.Mode.PREVIEW) {
                return false;
            }
            this.gitService.branch(this.name, true).done(null, function (e) { return _this.messageService.show(severity_1.default.Error, e); });
            return true;
        };
        return BranchEntry;
    }(model.QuickOpenEntry));
    // Commands
    var CheckoutCommand = (function () {
        function CheckoutCommand(gitService, messageService) {
            this.gitService = gitService;
            this.messageService = messageService;
            this.aliases = ['checkout', 'co'];
            this.icon = 'git';
            // noop
        }
        CheckoutCommand.prototype.getResults = function (input) {
            var _this = this;
            input = input.trim();
            var gitModel = this.gitService.getModel();
            var currentHead = gitModel.getHEAD();
            var headMatches = gitModel.getHeads()
                .map(function (head) { return ({ head: head, highlights: filters.matchesContiguousSubString(input, head.name) }); })
                .filter(function (_a) {
                var highlights = _a.highlights;
                return !!highlights;
            });
            var headEntries = headMatches
                .filter(function (_a) {
                var head = _a.head;
                return head.name !== currentHead.name;
            })
                .map(function (_a) {
                var head = _a.head, highlights = _a.highlights;
                return new CheckoutHeadEntry(_this.gitService, _this.messageService, head, highlights);
            });
            var tagMatches = gitModel.getTags()
                .map(function (tag) { return ({ tag: tag, highlights: filters.matchesContiguousSubString(input, tag.name) }); })
                .filter(function (_a) {
                var highlights = _a.highlights;
                return !!highlights;
            });
            var tagEntries = tagMatches
                .filter(function (_a) {
                var tag = _a.tag;
                return tag.name !== currentHead.name;
            })
                .map(function (_a) {
                var tag = _a.tag, highlights = _a.highlights;
                return new CheckoutTagEntry(_this.gitService, _this.messageService, tag, highlights);
            });
            var entries = headEntries
                .concat(tagEntries)
                .sort(function (a, b) { return a.getLabel().localeCompare(b.getLabel()); });
            if (entries.length > 0) {
                entries[0] = new model.QuickOpenEntryGroup(entries[0], 'checkout', false);
            }
            var exactMatches = headMatches.filter(function (_a) {
                var head = _a.head;
                return head.name === input;
            });
            var currentHeadMatches = exactMatches.filter(function (_a) {
                var head = _a.head;
                return head.name === currentHead.name;
            });
            if (currentHeadMatches.length > 0) {
                entries.unshift(new CurrentHeadEntry(this.gitService, this.messageService, currentHeadMatches[0].head, currentHeadMatches[0].highlights));
            }
            else if (exactMatches.length === 0 && git.isValidBranchName(input)) {
                var branchEntry = new BranchEntry(this.gitService, this.messageService, input);
                entries.push(new model.QuickOpenEntryGroup(branchEntry, 'branch', false));
            }
            return winjs.TPromise.as(entries);
        };
        CheckoutCommand.prototype.getEmptyLabel = function (input) {
            return nls.localize(6, null);
        };
        return CheckoutCommand;
    }());
    var BranchCommand = (function () {
        function BranchCommand(gitService, messageService) {
            this.gitService = gitService;
            this.messageService = messageService;
            this.aliases = ['branch'];
            this.icon = 'git';
            // noop
        }
        BranchCommand.prototype.getResults = function (input) {
            input = input.trim();
            if (!git.isValidBranchName(input)) {
                return winjs.TPromise.as([]);
            }
            var gitModel = this.gitService.getModel();
            var currentHead = gitModel.getHEAD();
            var matches = gitModel.getHeads()
                .map(function (head) { return ({ head: head, highlights: filters.matchesContiguousSubString(input, head.name) }); })
                .filter(function (_a) {
                var highlights = _a.highlights;
                return !!highlights;
            });
            var exactMatches = matches.filter(function (_a) {
                var head = _a.head;
                return head.name === input;
            });
            var headMatches = exactMatches.filter(function (_a) {
                var head = _a.head;
                return head.name === currentHead.name;
            });
            if (headMatches.length > 0) {
                return winjs.TPromise.as([new CurrentHeadEntry(this.gitService, this.messageService, headMatches[0].head, headMatches[0].highlights)]);
            }
            else if (exactMatches.length > 0) {
                return winjs.TPromise.as([new CheckoutHeadEntry(this.gitService, this.messageService, exactMatches[0].head, exactMatches[0].highlights)]);
            }
            var branchEntry = new BranchEntry(this.gitService, this.messageService, input);
            return winjs.TPromise.as([new model.QuickOpenEntryGroup(branchEntry, 'branch', false)]);
        };
        BranchCommand.prototype.getEmptyLabel = function (input) {
            return nls.localize(7, null);
        };
        return BranchCommand;
    }());
    var CommandQuickOpenHandler = (function (_super) {
        __extends(CommandQuickOpenHandler, _super);
        function CommandQuickOpenHandler(quickOpenService, gitService, messageService) {
            _super.call(this, quickOpenService, {
                prefix: 'git',
                commands: [
                    new CheckoutCommand(gitService, messageService),
                    new BranchCommand(gitService, messageService)
                ]
            });
        }
        CommandQuickOpenHandler = __decorate([
            __param(0, quickOpenService_1.IQuickOpenService),
            __param(1, IGitService),
            __param(2, message_1.IMessageService)
        ], CommandQuickOpenHandler);
        return CommandQuickOpenHandler;
    }(quickopenwb.CommandQuickOpenHandler));
    exports.CommandQuickOpenHandler = CommandQuickOpenHandler;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/quickopen/browser/commandsHandler", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/quickopen/browser/commandsHandler', 'vs/base/common/arrays', 'vs/base/common/types', 'vs/base/common/strings', 'vs/base/common/errors', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/platform/actions/common/actions', 'vs/workbench/common/actionRegistry', 'vs/platform/platform', 'vs/workbench/browser/quickopen', 'vs/workbench/browser/actions/quickOpenAction', 'vs/base/common/filters', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/workbench/services/editor/common/editorService', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/platform/telemetry/common/telemetry', 'vs/platform/keybinding/common/keybindingService', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/css!./media/commandsHandler'], function (require, exports, winjs_base_1, nls, arrays, types, strings, errors_1, quickOpen_1, quickOpenModel_1, actions_1, actionRegistry_1, platform_1, quickopen_1, quickOpenAction_1, filters, editorAction_1, editorActionEnablement_1, editorService_1, instantiation_1, message_1, telemetry_1, keybindingService_1, quickOpenService_1) {
    'use strict';
    exports.ALL_COMMANDS_PREFIX = '>';
    exports.EDITOR_COMMANDS_PREFIX = '$';
    var ShowAllCommandsAction = (function (_super) {
        __extends(ShowAllCommandsAction, _super);
        function ShowAllCommandsAction(actionId, actionLabel, quickOpenService) {
            _super.call(this, actionId, actionLabel, exports.ALL_COMMANDS_PREFIX, quickOpenService);
        }
        ShowAllCommandsAction.ID = 'workbench.action.showCommands';
        ShowAllCommandsAction.LABEL = nls.localize(0, null);
        ShowAllCommandsAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], ShowAllCommandsAction);
        return ShowAllCommandsAction;
    }(quickOpenAction_1.QuickOpenAction));
    exports.ShowAllCommandsAction = ShowAllCommandsAction;
    var BaseCommandEntry = (function (_super) {
        __extends(BaseCommandEntry, _super);
        function BaseCommandEntry(keyLabel, keyAriaLabel, description, highlights, messageService, telemetryService) {
            _super.call(this);
            this.messageService = messageService;
            this.telemetryService = telemetryService;
            this.keyLabel = keyLabel;
            this.keyAriaLabel = keyAriaLabel;
            this.description = description;
            this.setHighlights(highlights);
        }
        BaseCommandEntry.prototype.getLabel = function () {
            return this.description;
        };
        BaseCommandEntry.prototype.getAriaLabel = function () {
            if (this.keyAriaLabel) {
                return nls.localize(1, null, this.getLabel(), this.keyAriaLabel);
            }
            return nls.localize(2, null, this.getLabel());
        };
        BaseCommandEntry.prototype.getGroupLabel = function () {
            return this.keyLabel;
        };
        BaseCommandEntry.prototype.onError = function (error) {
            var message = !error ? nls.localize(3, null, this.description) : errors_1.toErrorMessage(error);
            this.messageService.show(message_1.Severity.Error, message);
        };
        BaseCommandEntry.prototype.runAction = function (action) {
            var _this = this;
            // Use a timeout to give the quick open widget a chance to close itself first
            winjs_base_1.TPromise.timeout(50).done(function () {
                if (action && action.enabled) {
                    try {
                        _this.telemetryService.publicLog('workbenchActionExecuted', { id: action.id, from: 'quick open' });
                        (action.run() || winjs_base_1.TPromise.as(null)).done(function () {
                            action.dispose();
                        }, function (err) { return _this.onError(err); });
                    }
                    catch (error) {
                        _this.onError(error);
                    }
                }
                else {
                    _this.messageService.show(message_1.Severity.Info, nls.localize(4, null, _this.getLabel()));
                }
            }, function (err) { return _this.onError(err); });
        };
        BaseCommandEntry = __decorate([
            __param(4, message_1.IMessageService),
            __param(5, telemetry_1.ITelemetryService)
        ], BaseCommandEntry);
        return BaseCommandEntry;
    }(quickOpenModel_1.QuickOpenEntryGroup));
    var CommandEntry = (function (_super) {
        __extends(CommandEntry, _super);
        function CommandEntry(keyLabel, keyAriaLabel, description, highlights, actionDescriptor, editorService, instantiationService, messageService, telemetryService) {
            _super.call(this, keyLabel, keyAriaLabel, description, highlights, messageService, telemetryService);
            this.editorService = editorService;
            this.instantiationService = instantiationService;
            this.actionDescriptor = actionDescriptor;
        }
        CommandEntry.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.OPEN) {
                var action = this.instantiationService.createInstance(this.actionDescriptor.syncDescriptor);
                this.runAction(action);
                return true;
            }
            return false;
        };
        CommandEntry = __decorate([
            __param(5, editorService_1.IWorkbenchEditorService),
            __param(6, instantiation_1.IInstantiationService),
            __param(7, message_1.IMessageService),
            __param(8, telemetry_1.ITelemetryService)
        ], CommandEntry);
        return CommandEntry;
    }(BaseCommandEntry));
    var EditorActionCommandEntry = (function (_super) {
        __extends(EditorActionCommandEntry, _super);
        function EditorActionCommandEntry(keyLabel, keyAriaLabel, description, highlights, action, editorService, messageService, telemetryService) {
            _super.call(this, keyLabel, keyAriaLabel, description, highlights, messageService, telemetryService);
            this.editorService = editorService;
            this.action = action;
        }
        EditorActionCommandEntry.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.OPEN) {
                this.runAction(this.action);
                return true;
            }
            return false;
        };
        EditorActionCommandEntry = __decorate([
            __param(5, editorService_1.IWorkbenchEditorService),
            __param(6, message_1.IMessageService),
            __param(7, telemetry_1.ITelemetryService)
        ], EditorActionCommandEntry);
        return EditorActionCommandEntry;
    }(BaseCommandEntry));
    var ActionCommandEntry = (function (_super) {
        __extends(ActionCommandEntry, _super);
        function ActionCommandEntry(keyLabel, keyAriaLabel, description, highlights, action, messageService, telemetryService) {
            _super.call(this, keyLabel, keyAriaLabel, description, highlights, messageService, telemetryService);
            this.action = action;
        }
        ActionCommandEntry.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.OPEN) {
                this.runAction(this.action);
                return true;
            }
            return false;
        };
        ActionCommandEntry = __decorate([
            __param(5, message_1.IMessageService),
            __param(6, telemetry_1.ITelemetryService)
        ], ActionCommandEntry);
        return ActionCommandEntry;
    }(BaseCommandEntry));
    var CommandsHandler = (function (_super) {
        __extends(CommandsHandler, _super);
        function CommandsHandler(editorService, instantiationService, messageService, keybindingService, actionsService) {
            _super.call(this);
            this.editorService = editorService;
            this.instantiationService = instantiationService;
            this.messageService = messageService;
            this.keybindingService = keybindingService;
            this.actionsService = actionsService;
        }
        CommandsHandler.prototype.includeWorkbenchCommands = function () {
            return true;
        };
        CommandsHandler.prototype.getResults = function (searchValue) {
            searchValue = searchValue.trim();
            // Workbench Actions (if prefix asks for all commands)
            var workbenchEntries = [];
            if (this.includeWorkbenchCommands()) {
                var workbenchActions = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions).getWorkbenchActions();
                workbenchEntries = this.actionDescriptorsToEntries(workbenchActions, searchValue);
            }
            // Editor Actions
            var activeEditor = this.editorService.getActiveEditor();
            var activeEditorControl = (activeEditor ? activeEditor.getControl() : null);
            var editorActions = [];
            if (activeEditorControl && types.isFunction(activeEditorControl.getActions)) {
                editorActions = activeEditorControl.getActions();
            }
            var editorEntries = this.editorActionsToEntries(editorActions, searchValue);
            // Other Actions
            var otherActions = this.actionsService.getActions();
            var otherEntries = this.otherActionsToEntries(otherActions, searchValue);
            // Concat
            var entries = workbenchEntries.concat(editorEntries, otherEntries);
            // Remove duplicates
            entries = arrays.distinct(entries, function (entry) { return entry.getLabel() + entry.getGroupLabel(); });
            // Sort by name
            entries = entries.sort(function (elementA, elementB) { return strings.localeCompare(elementA.getLabel().toLowerCase(), elementB.getLabel().toLowerCase()); });
            return winjs_base_1.TPromise.as(new quickOpenModel_1.QuickOpenModel(entries));
        };
        CommandsHandler.prototype.actionDescriptorsToEntries = function (actionDescriptors, searchValue) {
            var _this = this;
            var entries = [];
            var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
            for (var i = 0; i < actionDescriptors.length; i++) {
                var actionDescriptor = actionDescriptors[i];
                var keys = this.keybindingService.lookupKeybindings(actionDescriptor.id);
                var keyLabel = keys.map(function (k) { return _this.keybindingService.getLabelFor(k); });
                var keyAriaLabel = keys.map(function (k) { return _this.keybindingService.getAriaLabelFor(k); });
                if (actionDescriptor.label) {
                    var label = actionDescriptor.label;
                    var category = registry.getCategory(actionDescriptor.id);
                    if (category) {
                        label = nls.localize(5, null, category, label);
                    }
                    var highlights = filters.matchesFuzzy(searchValue, label);
                    if (highlights) {
                        entries.push(this.instantiationService.createInstance(CommandEntry, keyLabel.length > 0 ? keyLabel.join(', ') : '', keyAriaLabel.length > 0 ? keyAriaLabel.join(', ') : '', label, highlights, actionDescriptor));
                    }
                }
            }
            return entries;
        };
        CommandsHandler.prototype.editorActionsToEntries = function (actions, searchValue) {
            var _this = this;
            var entries = [];
            for (var i = 0; i < actions.length; i++) {
                var action = actions[i];
                var editorAction = action;
                if (!editorAction.isSupported()) {
                    continue; // do not show actions that are not supported in this context
                }
                var keys = this.keybindingService.lookupKeybindings(editorAction.id);
                var keyLabel = keys.map(function (k) { return _this.keybindingService.getLabelFor(k); });
                var keyAriaLabel = keys.map(function (k) { return _this.keybindingService.getAriaLabelFor(k); });
                if (action.label) {
                    var highlights = filters.matchesFuzzy(searchValue, action.label);
                    if (highlights) {
                        entries.push(this.instantiationService.createInstance(EditorActionCommandEntry, keyLabel.length > 0 ? keyLabel.join(', ') : '', keyAriaLabel.length > 0 ? keyAriaLabel.join(', ') : '', action.label, highlights, action));
                    }
                }
            }
            return entries;
        };
        CommandsHandler.prototype.otherActionsToEntries = function (actions, searchValue) {
            var _this = this;
            var entries = [];
            for (var _i = 0, actions_2 = actions; _i < actions_2.length; _i++) {
                var action = actions_2[_i];
                var keys = this.keybindingService.lookupKeybindings(action.id);
                var keyLabel = keys.map(function (k) { return _this.keybindingService.getLabelFor(k); });
                var keyAriaLabel = keys.map(function (k) { return _this.keybindingService.getAriaLabelFor(k); });
                var highlights = filters.matchesFuzzy(searchValue, action.label);
                if (highlights) {
                    entries.push(this.instantiationService.createInstance(ActionCommandEntry, keyLabel.join(', '), keyAriaLabel.join(', '), action.label, highlights, action));
                }
            }
            return entries;
        };
        CommandsHandler.prototype.getAutoFocus = function (searchValue) {
            return {
                autoFocusFirstEntry: true,
                autoFocusPrefixMatch: searchValue.trim()
            };
        };
        CommandsHandler.prototype.getClass = function () {
            return 'commands-handler';
        };
        CommandsHandler.prototype.getEmptyLabel = function (searchString) {
            return nls.localize(6, null);
        };
        CommandsHandler = __decorate([
            __param(0, editorService_1.IWorkbenchEditorService),
            __param(1, instantiation_1.IInstantiationService),
            __param(2, message_1.IMessageService),
            __param(3, keybindingService_1.IKeybindingService),
            __param(4, actions_1.IActionsService)
        ], CommandsHandler);
        return CommandsHandler;
    }(quickopen_1.QuickOpenHandler));
    exports.CommandsHandler = CommandsHandler;
    var EditorCommandsHandler = (function (_super) {
        __extends(EditorCommandsHandler, _super);
        function EditorCommandsHandler() {
            _super.apply(this, arguments);
        }
        EditorCommandsHandler.prototype.includeWorkbenchCommands = function () {
            return false;
        };
        return EditorCommandsHandler;
    }(CommandsHandler));
    exports.EditorCommandsHandler = EditorCommandsHandler;
    var QuickCommandsEditorAction = (function (_super) {
        __extends(QuickCommandsEditorAction, _super);
        function QuickCommandsEditorAction(descriptor, editor, quickOpenService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.WidgetFocus | editorActionEnablement_1.Behaviour.ShowInContextMenu);
            this.quickOpenService = quickOpenService;
            this.label = nls.localize(7, null);
        }
        QuickCommandsEditorAction.prototype.getGroupId = function () {
            return '4_tools/1_commands';
        };
        QuickCommandsEditorAction.prototype.run = function () {
            // Pass focus to editor first before running quick open action
            this.editor.focus();
            // Show with prefix
            this.quickOpenService.show('$');
            return _super.prototype.run.call(this);
        };
        QuickCommandsEditorAction.ID = 'editor.action.quickCommand';
        QuickCommandsEditorAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], QuickCommandsEditorAction);
        return QuickCommandsEditorAction;
    }(editorAction_1.EditorAction));
    exports.QuickCommandsEditorAction = QuickCommandsEditorAction;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/quickopen/browser/helpHandler", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/quickopen/browser/helpHandler', 'vs/base/browser/builder', 'vs/base/common/types', 'vs/platform/platform', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/workbench/browser/quickopen', 'vs/workbench/services/quickopen/common/quickOpenService'], function (require, exports, winjs_base_1, nls, builder_1, types, platform_1, quickOpen_1, quickOpenModel_1, quickopen_1, quickOpenService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.HELP_PREFIX = '?';
    var HelpEntry = (function (_super) {
        __extends(HelpEntry, _super);
        function HelpEntry(prefix, description, quickOpenService, openOnPreview) {
            _super.call(this);
            this.prefix = prefix;
            this.description = description;
            this.quickOpenService = quickOpenService;
            this.openOnPreview = openOnPreview;
        }
        HelpEntry.prototype.getLabel = function () {
            return this.prefix;
        };
        HelpEntry.prototype.getAriaLabel = function () {
            return nls.localize(0, null, this.getLabel());
        };
        HelpEntry.prototype.getDescription = function () {
            return this.description;
        };
        HelpEntry.prototype.getHeight = function () {
            return 22;
        };
        HelpEntry.prototype.getGroupLabel = function () {
            return this.groupLabel;
        };
        HelpEntry.prototype.setGroupLabel = function (groupLabel) {
            this.groupLabel = groupLabel;
        };
        HelpEntry.prototype.showBorder = function () {
            return this.useBorder;
        };
        HelpEntry.prototype.setShowBorder = function (showBorder) {
            this.useBorder = showBorder;
        };
        HelpEntry.prototype.render = function (tree, container, previousCleanupFn) {
            var _this = this;
            var builder = builder_1.$(container);
            builder.addClass('quick-open-entry');
            // Support border
            if (this.showBorder()) {
                builder_1.$(container).addClass('results-group-separator');
            }
            else {
                builder_1.$(container).removeClass('results-group-separator');
            }
            // Add a container for the group
            if (this.getGroupLabel()) {
                builder_1.$(container).div(function (div) {
                    div.addClass('results-group');
                    div.attr({
                        text: _this.getGroupLabel()
                    });
                });
            }
            // Prefix
            var label = builder.clone().div({
                text: this.prefix,
                'class': 'quick-open-help-entry-label'
            });
            if (!this.prefix) {
                label.text('\u2026');
            }
            // Description
            builder.span({
                text: this.description,
                'class': 'quick-open-entry-description'
            });
            return null;
        };
        HelpEntry.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.OPEN || this.openOnPreview) {
                this.quickOpenService.show(this.prefix);
            }
            return false;
        };
        return HelpEntry;
    }(quickOpenModel_1.QuickOpenEntryItem));
    var HelpHandler = (function (_super) {
        __extends(HelpHandler, _super);
        function HelpHandler(quickOpenService) {
            _super.call(this);
            this.quickOpenService = quickOpenService;
        }
        HelpHandler.prototype.getResults = function (searchValue) {
            var _this = this;
            searchValue = searchValue.trim();
            var registry = platform_1.Registry.as(quickopen_1.Extensions.Quickopen);
            var handlerDescriptors = registry.getQuickOpenHandlers();
            var defaultHandlers = registry.getDefaultQuickOpenHandlers();
            if (defaultHandlers.length > 0) {
                handlerDescriptors.push.apply(handlerDescriptors, defaultHandlers);
            }
            var workbenchScoped = [];
            var editorScoped = [];
            var entry;
            handlerDescriptors.sort(function (h1, h2) { return h1.prefix.localeCompare(h2.prefix); }).forEach(function (handlerDescriptor) {
                if (handlerDescriptor.prefix !== exports.HELP_PREFIX) {
                    // Descriptor has multiple help entries
                    if (types.isArray(handlerDescriptor.helpEntries)) {
                        for (var j = 0; j < handlerDescriptor.helpEntries.length; j++) {
                            var helpEntry = handlerDescriptor.helpEntries[j];
                            if (helpEntry.prefix.indexOf(searchValue) === 0) {
                                entry = new HelpEntry(helpEntry.prefix, helpEntry.description, _this.quickOpenService, searchValue.length > 0);
                                if (helpEntry.needsEditor) {
                                    editorScoped.push(entry);
                                }
                                else {
                                    workbenchScoped.push(entry);
                                }
                            }
                        }
                    }
                    else if (handlerDescriptor.prefix.indexOf(searchValue) === 0) {
                        entry = new HelpEntry(handlerDescriptor.prefix, handlerDescriptor.description, _this.quickOpenService, searchValue.length > 0);
                        workbenchScoped.push(entry);
                    }
                }
            });
            // Add separator for workbench scoped handlers
            if (workbenchScoped.length > 0) {
                workbenchScoped[0].setGroupLabel(nls.localize(1, null));
            }
            // Add separator for editor scoped handlers
            if (editorScoped.length > 0) {
                editorScoped[0].setGroupLabel(nls.localize(2, null));
                if (workbenchScoped.length > 0) {
                    editorScoped[0].setShowBorder(true);
                }
            }
            return winjs_base_1.TPromise.as(new quickOpenModel_1.QuickOpenModel(workbenchScoped.concat(editorScoped)));
        };
        HelpHandler.prototype.getAutoFocus = function (searchValue) {
            searchValue = searchValue.trim();
            return {
                autoFocusFirstEntry: searchValue.length > 0,
                autoFocusPrefixMatch: searchValue
            };
        };
        HelpHandler = __decorate([
            __param(0, quickOpenService_1.IQuickOpenService)
        ], HelpHandler);
        return HelpHandler;
    }(quickopen_1.QuickOpenHandler));
    exports.HelpHandler = HelpHandler;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/snippets/electron-browser/snippets.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/snippets/electron-browser/snippets.contribution', 'vs/base/common/winjs.base', 'vs/base/common/paths', 'vs/base/common/actions', 'vs/platform/actions/common/actions', 'vs/platform/platform', 'vs/workbench/common/actionRegistry', 'vs/workbench/common/contributions', './snippetsTracker', 'vs/base/common/errors', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/platform/workspace/common/workspace', 'vs/platform/jsonschemas/common/jsonContributionRegistry', 'vs/editor/common/services/modeService', 'electron', 'fs'], function (require, exports, nls, winjs, paths, actions, actions_1, platform, workbenchActionRegistry, workbenchContributions, snippetsTracker, errors, quickOpenService_1, workspace_1, JSONContributionRegistry, modeService_1, electron_1, fs) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var OpenSnippetsAction = (function (_super) {
        __extends(OpenSnippetsAction, _super);
        function OpenSnippetsAction(id, label, contextService, quickOpenService, modeService) {
            _super.call(this, id, label);
            this.contextService = contextService;
            this.quickOpenService = quickOpenService;
            this.modeService = modeService;
        }
        OpenSnippetsAction.prototype.openFile = function (filePath) {
            electron_1.ipcRenderer.send('vscode:windowOpen', [filePath]); // handled from browser process
        };
        OpenSnippetsAction.prototype.run = function () {
            var _this = this;
            var modeIds = this.modeService.getRegisteredModes();
            var picks = [];
            modeIds.forEach(function (modeId) {
                var name = _this.modeService.getLanguageName(modeId);
                if (name) {
                    picks.push({ label: name, id: modeId });
                }
            });
            picks = picks.sort(function (e1, e2) {
                return e1.label.localeCompare(e2.label);
            });
            return this.quickOpenService.pick(picks, { placeHolder: nls.localize(1, null) }).then(function (language) {
                if (language) {
                    var snippetPath = paths.join(_this.contextService.getConfiguration().env.appSettingsHome, 'snippets', language.id + '.json');
                    return fileExists(snippetPath).then(function (success) {
                        if (success) {
                            _this.openFile(snippetPath);
                            return winjs.TPromise.as(null);
                        }
                        var defaultContent = [
                            '{',
                            '/*',
                            '\t // Place your snippets for ' + language.label + ' here. Each snippet is defined under a snippet name and has a prefix, body and ',
                            '\t // description. The prefix is what is used to trigger the snippet and the body will be expanded and inserted. Possible variables are:',
                            '\t // $1, $2 for tab stops, ${id} and ${id:label} and ${1:label} for variables. Variables with the same id are connected.',
                            '\t // Example:',
                            '\t "Print to console": {',
                            '\t\t"prefix": "log",',
                            '\t\t"body": [',
                            '\t\t\t"console.log(\'$1\');",',
                            '\t\t\t"$2"',
                            '\t\t],',
                            '\t\t"description": "Log output to console"',
                            '\t}',
                            '*/',
                            '}'
                        ].join('\n');
                        return createFile(snippetPath, defaultContent).then(function () {
                            _this.openFile(snippetPath);
                        }, function (err) {
                            errors.onUnexpectedError(nls.localize(2, null, snippetPath));
                        });
                    });
                }
                return winjs.TPromise.as(null);
            });
        };
        OpenSnippetsAction.ID = 'workbench.action.openSnippets';
        OpenSnippetsAction.LABEL = nls.localize(0, null);
        OpenSnippetsAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, quickOpenService_1.IQuickOpenService),
            __param(4, modeService_1.IModeService)
        ], OpenSnippetsAction);
        return OpenSnippetsAction;
    }(actions.Action));
    function fileExists(path) {
        return new winjs.TPromise(function (c, e, p) {
            fs.stat(path, function (err, stats) {
                if (err) {
                    return c(false);
                }
                if (stats.isFile()) {
                    return c(true);
                }
                c(false);
            });
        });
    }
    function createFile(path, content) {
        return new winjs.Promise(function (c, e, p) {
            fs.writeFile(path, content, function (err) {
                if (err) {
                    e(err);
                }
                c(true);
            });
        });
    }
    var preferencesCategory = nls.localize(3, null);
    var workbenchActionsRegistry = platform.Registry.as(workbenchActionRegistry.Extensions.WorkbenchActions);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(OpenSnippetsAction, OpenSnippetsAction.ID, OpenSnippetsAction.LABEL), preferencesCategory);
    platform.Registry.as(workbenchContributions.Extensions.Workbench).registerWorkbenchContribution(snippetsTracker.SnippetsTracker);
    var schemaId = 'vscode://schemas/snippets';
    var schema = {
        'id': schemaId,
        'defaultSnippets': [{
                'label': nls.localize(4, null),
                'body': { '{{snippetName}}': { 'prefix': '{{prefix}}', 'body': '{{snippet}}', 'description': '{{description}}' } }
            }],
        'type': 'object',
        'description': nls.localize(5, null),
        'additionalProperties': {
            'type': 'object',
            'required': ['prefix', 'body'],
            'properties': {
                'prefix': {
                    'description': nls.localize(6, null),
                    'type': 'string'
                },
                'body': {
                    'description': nls.localize(7, null),
                    'type': ['string', 'array'],
                    'items': {
                        'type': 'string'
                    }
                },
                'description': {
                    'description': nls.localize(8, null),
                    'type': 'string'
                }
            },
            'additionalProperties': false
        }
    };
    var schemaRegistry = platform.Registry.as(JSONContributionRegistry.Extensions.JSONContribution);
    schemaRegistry.registerSchema(schemaId, schema);
    schemaRegistry.addSchemaFileAssociation('%APP_SETTINGS_HOME%/snippets/*.json', schemaId);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/tasks/browser/taskQuickOpen", ["require", "exports", 'vs/nls!vs/workbench/parts/tasks/browser/taskQuickOpen', 'vs/base/common/filters', 'vs/workbench/browser/quickopen', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/parts/tasks/common/taskService'], function (require, exports, nls, Filters, Quickopen, QuickOpen, Model, quickOpenService_1, taskService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var TaskEntry = (function (_super) {
        __extends(TaskEntry, _super);
        function TaskEntry(taskService, task, highlights) {
            if (highlights === void 0) { highlights = []; }
            _super.call(this, highlights);
            this.taskService = taskService;
            this.task = task;
        }
        TaskEntry.prototype.getLabel = function () {
            return this.task.name;
        };
        TaskEntry.prototype.getAriaLabel = function () {
            return nls.localize(0, null, this.getLabel());
        };
        TaskEntry.prototype.run = function (mode, context) {
            if (mode === QuickOpen.Mode.PREVIEW) {
                return false;
            }
            this.taskService.run(this.task.id);
            return true;
        };
        return TaskEntry;
    }(Model.QuickOpenEntry));
    var QuickOpenHandler = (function (_super) {
        __extends(QuickOpenHandler, _super);
        function QuickOpenHandler(quickOpenService, taskService) {
            _super.call(this);
            this.quickOpenService = quickOpenService;
            this.taskService = taskService;
        }
        QuickOpenHandler.prototype.getAriaLabel = function () {
            return nls.localize(1, null);
        };
        QuickOpenHandler.prototype.getResults = function (input) {
            var _this = this;
            return this.taskService.tasks().then(function (tasks) { return tasks
                .sort(function (a, b) { return a.name.localeCompare(b.name); })
                .map(function (task) { return ({ task: task, highlights: Filters.matchesContiguousSubString(input, task.name) }); })
                .filter(function (_a) {
                var highlights = _a.highlights;
                return !!highlights;
            })
                .map(function (_a) {
                var task = _a.task, highlights = _a.highlights;
                return new TaskEntry(_this.taskService, task, highlights);
            }); }, function (_) { return []; }).then(function (e) { return new Model.QuickOpenModel(e); });
        };
        QuickOpenHandler.prototype.getClass = function () {
            return null;
        };
        QuickOpenHandler.prototype.canRun = function () {
            return true;
        };
        QuickOpenHandler.prototype.getAutoFocus = function (input) {
            return {
                autoFocusFirstEntry: !!input
            };
        };
        QuickOpenHandler.prototype.onClose = function (canceled) {
            return;
        };
        QuickOpenHandler.prototype.getGroupLabel = function () {
            return null;
        };
        QuickOpenHandler.prototype.getEmptyLabel = function (searchString) {
            if (searchString.length > 0) {
                return nls.localize(2, null);
            }
            return nls.localize(3, null);
        };
        QuickOpenHandler = __decorate([
            __param(0, quickOpenService_1.IQuickOpenService),
            __param(1, taskService_1.ITaskService)
        ], QuickOpenHandler);
        return QuickOpenHandler;
    }(Quickopen.QuickOpenHandler));
    exports.QuickOpenHandler = QuickOpenHandler;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/search/node/fileSearch", ["require", "exports", 'fs', 'path', 'vs/base/common/scorer', 'vs/base/common/arrays', 'vs/base/common/strings', 'vs/base/common/types', 'vs/base/common/glob', 'vs/base/node/extfs', 'vs/base/node/flow'], function (require, exports, fs, paths, scorer, arrays, strings, types, glob, extfs, flow) {
    'use strict';
    var FileWalker = (function () {
        function FileWalker(config) {
            this.config = config;
            this.filePattern = config.filePattern;
            this.excludePattern = config.excludePattern;
            this.includePattern = config.includePattern;
            this.maxResults = config.maxResults || null;
            this.maxFilesize = config.maxFilesize || null;
            this.walkedPaths = Object.create(null);
            this.resultCount = 0;
            this.isLimitHit = false;
            if (this.filePattern) {
                this.filePattern = this.filePattern.replace(/\\/g, '/'); // Normalize file patterns to forward slashes
                this.normalizedFilePatternLowercase = strings.stripWildcards(this.filePattern).toLowerCase();
            }
        }
        FileWalker.prototype.cancel = function () {
            this.isCanceled = true;
        };
        FileWalker.prototype.walk = function (rootFolders, extraFiles, onResult, done) {
            var _this = this;
            // Support that the file pattern is a full path to a file that exists
            this.checkFilePatternAbsoluteMatch(function (exists) {
                if (_this.isCanceled) {
                    return done(null, _this.isLimitHit);
                }
                // Report result from file pattern if matching
                if (exists) {
                    onResult({ path: _this.filePattern });
                    // Optimization: a match on an absolute path is a good result and we do not
                    // continue walking the entire root paths array for other matches because
                    // it is very unlikely that another file would match on the full absolute path
                    return done(null, _this.isLimitHit);
                }
                // For each extra file
                if (extraFiles) {
                    extraFiles.forEach(function (extraFilePath) {
                        if (glob.match(_this.excludePattern, extraFilePath)) {
                            return; // excluded
                        }
                        // File: Check for match on file pattern and include pattern
                        _this.matchFile(onResult, extraFilePath, extraFilePath /* no workspace relative path */);
                    });
                }
                // For each root folder
                flow.parallel(rootFolders, function (absolutePath, perEntryCallback) {
                    extfs.readdir(absolutePath, function (error, files) {
                        if (error || _this.isCanceled || _this.isLimitHit) {
                            return perEntryCallback(null, null);
                        }
                        // Support relative paths to files from a root resource
                        return _this.checkFilePatternRelativeMatch(absolutePath, function (match) {
                            if (_this.isCanceled || _this.isLimitHit) {
                                return perEntryCallback(null, null);
                            }
                            // Report result from file pattern if matching
                            if (match) {
                                onResult({ path: match });
                            }
                            return _this.doWalk(paths.normalize(absolutePath), '', files, onResult, perEntryCallback);
                        });
                    });
                }, function (err, result) {
                    done(err ? err[0] : null, _this.isLimitHit);
                });
            });
        };
        FileWalker.prototype.checkFilePatternAbsoluteMatch = function (clb) {
            if (!this.filePattern || !paths.isAbsolute(this.filePattern)) {
                return clb(false);
            }
            return fs.stat(this.filePattern, function (error, stat) {
                return clb(!error && !stat.isDirectory()); // only existing files
            });
        };
        FileWalker.prototype.checkFilePatternRelativeMatch = function (basePath, clb) {
            if (!this.filePattern || paths.isAbsolute(this.filePattern)) {
                return clb(null);
            }
            var absolutePath = paths.join(basePath, this.filePattern);
            return fs.stat(absolutePath, function (error, stat) {
                return clb(!error && !stat.isDirectory() ? absolutePath : null); // only existing files
            });
        };
        FileWalker.prototype.doWalk = function (absolutePath, relativeParentPathWithSlashes, files, onResult, done) {
            var _this = this;
            // Execute tasks on each file in parallel to optimize throughput
            flow.parallel(files, function (file, clb) {
                // Check canceled
                if (_this.isCanceled || _this.isLimitHit) {
                    return clb(null);
                }
                // If the user searches for the exact file name, we adjust the glob matching
                // to ignore filtering by siblings because the user seems to know what she
                // is searching for and we want to include the result in that case anyway
                var siblings = files;
                if (_this.config.filePattern === file) {
                    siblings = [];
                }
                // Check exclude pattern
                var currentRelativePathWithSlashes = relativeParentPathWithSlashes ? [relativeParentPathWithSlashes, file].join('/') : file;
                if (glob.match(_this.excludePattern, currentRelativePathWithSlashes, siblings)) {
                    return clb(null);
                }
                // Use lstat to detect links
                var currentAbsolutePath = [absolutePath, file].join(paths.sep);
                fs.lstat(currentAbsolutePath, function (error, lstat) {
                    if (error || _this.isCanceled || _this.isLimitHit) {
                        return clb(null);
                    }
                    // If the path is a link, we must instead use fs.stat() to find out if the
                    // link is a directory or not because lstat will always return the stat of
                    // the link which is always a file.
                    _this.statLinkIfNeeded(currentAbsolutePath, lstat, function (error, stat) {
                        if (error || _this.isCanceled || _this.isLimitHit) {
                            return clb(null);
                        }
                        // Directory: Follow directories
                        if (stat.isDirectory()) {
                            // to really prevent loops with links we need to resolve the real path of them
                            return _this.realPathIfNeeded(currentAbsolutePath, lstat, function (error, realpath) {
                                if (error || _this.isCanceled || _this.isLimitHit) {
                                    return clb(null);
                                }
                                if (_this.walkedPaths[realpath]) {
                                    return clb(null); // escape when there are cycles (can happen with symlinks)
                                }
                                _this.walkedPaths[realpath] = true; // remember as walked
                                // Continue walking
                                return extfs.readdir(currentAbsolutePath, function (error, children) {
                                    if (error || _this.isCanceled || _this.isLimitHit) {
                                        return clb(null);
                                    }
                                    _this.doWalk(currentAbsolutePath, currentRelativePathWithSlashes, children, onResult, clb);
                                });
                            });
                        }
                        else {
                            if (currentRelativePathWithSlashes === _this.filePattern) {
                                return clb(null); // ignore file if its path matches with the file pattern because checkFilePatternRelativeMatch() takes care of those
                            }
                            if (_this.maxFilesize && types.isNumber(stat.size) && stat.size > _this.maxFilesize) {
                                return clb(null); // ignore file if max file size is hit
                            }
                            _this.matchFile(onResult, currentAbsolutePath, currentRelativePathWithSlashes);
                        }
                        // Unwind
                        return clb(null);
                    });
                });
            }, function (error) {
                if (error) {
                    error = arrays.coalesce(error); // find any error by removing null values first
                }
                return done(error && error.length > 0 ? error[0] : null, null);
            });
        };
        FileWalker.prototype.matchFile = function (onResult, absolutePath, relativePathWithSlashes) {
            if (this.isFilePatternMatch(relativePathWithSlashes) && (!this.includePattern || glob.match(this.includePattern, relativePathWithSlashes))) {
                this.resultCount++;
                if (this.maxResults && this.resultCount > this.maxResults) {
                    this.isLimitHit = true;
                }
                if (!this.isLimitHit) {
                    onResult({
                        path: absolutePath
                    });
                }
            }
        };
        FileWalker.prototype.isFilePatternMatch = function (path) {
            // Check for search pattern
            if (this.filePattern) {
                if (this.filePattern === '*') {
                    return true; // support the all-matching wildcard
                }
                return scorer.matches(path, this.normalizedFilePatternLowercase);
            }
            // No patterns means we match all
            return true;
        };
        FileWalker.prototype.statLinkIfNeeded = function (path, lstat, clb) {
            if (lstat.isSymbolicLink()) {
                return fs.stat(path, clb); // stat the target the link points to
            }
            return clb(null, lstat); // not a link, so the stat is already ok for us
        };
        FileWalker.prototype.realPathIfNeeded = function (path, lstat, clb) {
            if (lstat.isSymbolicLink()) {
                return fs.realpath(path, function (error, realpath) {
                    if (error) {
                        return clb(error);
                    }
                    return clb(null, realpath);
                });
            }
            return clb(null, path);
        };
        return FileWalker;
    }());
    exports.FileWalker = FileWalker;
    var Engine = (function () {
        function Engine(config) {
            this.rootFolders = config.rootFolders;
            this.extraFiles = config.extraFiles;
            this.walker = new FileWalker(config);
        }
        Engine.prototype.search = function (onResult, onProgress, done) {
            this.walker.walk(this.rootFolders, this.extraFiles, onResult, done);
        };
        Engine.prototype.cancel = function () {
            this.walker.cancel();
        };
        return Engine;
    }());
    exports.Engine = Engine;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/search/node/textSearch", ["require", "exports", 'vs/base/common/strings', 'fs', 'vs/base/common/mime', 'vs/base/node/mime', 'vs/base/node/encoding'], function (require, exports, strings, fs, baseMime, mime_1, encoding_1) {
    'use strict';
    var Engine = (function () {
        function Engine(config, walker) {
            this.rootFolders = config.rootFolders;
            this.extraFiles = config.extraFiles;
            this.walker = walker;
            this.contentPattern = strings.createRegExp(config.contentPattern.pattern, config.contentPattern.isRegExp, config.contentPattern.isCaseSensitive, config.contentPattern.isWordMatch, true);
            this.isCanceled = false;
            this.limitReached = false;
            this.maxResults = config.maxResults;
            this.worked = 0;
            this.total = 0;
            this.fileEncoding = encoding_1.encodingExists(config.fileEncoding) ? config.fileEncoding : encoding_1.UTF8;
        }
        Engine.prototype.cancel = function () {
            this.isCanceled = true;
            this.walker.cancel();
        };
        Engine.prototype.search = function (onResult, onProgress, done) {
            var _this = this;
            var resultCounter = 0;
            var unwind = function (processed) {
                _this.worked += processed;
                // Emit progress() unless we got canceled or hit the limit
                if (processed && !_this.isDone && !_this.isCanceled && !_this.limitReached) {
                    onProgress({ total: _this.total, worked: _this.worked });
                }
                // Emit done()
                if (_this.worked === _this.total && _this.walkerIsDone && !_this.isDone) {
                    _this.isDone = true;
                    done(_this.walkerError, _this.limitReached);
                }
            };
            // Walk over the file system
            this.walker.walk(this.rootFolders, this.extraFiles, function (result) {
                _this.total++;
                // If the result is empty or we have reached the limit or we are canceled, ignore it
                if (_this.limitReached || _this.isCanceled) {
                    return unwind(1);
                }
                // Indicate progress to the outside
                onProgress({ total: _this.total, worked: _this.worked });
                var fileMatch = null;
                var doneCallback = function (error) {
                    if (!error && !_this.isCanceled && fileMatch && !fileMatch.isEmpty()) {
                        onResult(fileMatch.serialize());
                    }
                    return unwind(1);
                };
                var perLineCallback = function (line, lineNumber) {
                    if (_this.limitReached || _this.isCanceled) {
                        return; // return early if canceled or limit reached
                    }
                    var lineMatch = null;
                    var match = _this.contentPattern.exec(line);
                    // Record all matches into file result
                    while (match !== null && match[0].length > 0 && !_this.limitReached && !_this.isCanceled) {
                        resultCounter++;
                        if (_this.maxResults && resultCounter >= _this.maxResults) {
                            _this.limitReached = true;
                        }
                        if (fileMatch === null) {
                            fileMatch = new FileMatch(result.path);
                        }
                        if (lineMatch === null) {
                            lineMatch = new LineMatch(line, lineNumber);
                            fileMatch.addMatch(lineMatch);
                        }
                        lineMatch.addMatch(match.index, match[0].length);
                        match = _this.contentPattern.exec(line);
                    }
                };
                // Read lines buffered to support large files
                _this.readlinesAsync(result.path, perLineCallback, { bufferLength: 8096, encoding: _this.fileEncoding }, doneCallback);
            }, function (error, isLimitHit) {
                _this.walkerIsDone = true;
                _this.walkerError = error;
                unwind(0 /* walker is done, indicate this back to our handler to be able to unwind */);
            });
        };
        Engine.prototype.readlinesAsync = function (filename, perLineCallback, options, callback) {
            var _this = this;
            fs.open(filename, 'r', null, function (error, fd) {
                if (error) {
                    return callback(error);
                }
                var buffer = new Buffer(options.bufferLength);
                var pos;
                var i;
                var line = '';
                var lineNumber = 0;
                var lastBufferHadTraillingCR = false;
                var outer = _this;
                function decodeBuffer(buffer) {
                    if (options.encoding === encoding_1.UTF8 || options.encoding === encoding_1.UTF8_with_bom) {
                        return buffer.toString(); // much faster to use built in toString() when encoding is default
                    }
                    return encoding_1.decode(buffer, options.encoding);
                }
                function lineFinished(offset) {
                    line += decodeBuffer(buffer.slice(pos, i + offset));
                    perLineCallback(line, lineNumber);
                    line = '';
                    lineNumber++;
                    pos = i + offset;
                }
                function readFile(isFirstRead, clb) {
                    if (outer.limitReached || outer.isCanceled) {
                        return clb(null); // return early if canceled or limit reached
                    }
                    fs.read(fd, buffer, 0, buffer.length, null, function (error, bytesRead, buffer) {
                        if (error || bytesRead === 0 || outer.limitReached || outer.isCanceled) {
                            return clb(error); // return early if canceled or limit reached or no more bytes to read
                        }
                        pos = 0;
                        i = 0;
                        // Detect encoding and mime when this is the beginning of the file
                        if (isFirstRead) {
                            var mimeAndEncoding = mime_1.detectMimeAndEncodingFromBuffer(buffer, bytesRead);
                            if (mimeAndEncoding.mimes[mimeAndEncoding.mimes.length - 1] !== baseMime.MIME_TEXT) {
                                return clb(null); // skip files that seem binary
                            }
                            // Check for BOM offset
                            switch (mimeAndEncoding.encoding) {
                                case encoding_1.UTF8:
                                    pos = i = 3;
                                    options.encoding = encoding_1.UTF8;
                                    break;
                                case encoding_1.UTF16be:
                                    pos = i = 2;
                                    options.encoding = encoding_1.UTF16be;
                                    break;
                                case encoding_1.UTF16le:
                                    pos = i = 2;
                                    options.encoding = encoding_1.UTF16le;
                                    break;
                            }
                        }
                        if (lastBufferHadTraillingCR) {
                            if (buffer[i] === 0x0a) {
                                lineFinished(1);
                                i++;
                            }
                            else {
                                lineFinished(0);
                            }
                            lastBufferHadTraillingCR = false;
                        }
                        for (; i < bytesRead; ++i) {
                            if (buffer[i] === 0x0a) {
                                lineFinished(1);
                            }
                            else if (buffer[i] === 0x0d) {
                                if (i + 1 === bytesRead) {
                                    lastBufferHadTraillingCR = true;
                                }
                                else if (buffer[i + 1] === 0x0a) {
                                    lineFinished(2);
                                    i++;
                                }
                                else {
                                    lineFinished(1);
                                }
                            }
                        }
                        line += decodeBuffer(buffer.slice(pos, bytesRead));
                        readFile(false /* isFirstRead */, clb); // Continue reading
                    });
                }
                readFile(true /* isFirstRead */, function (error) {
                    if (error) {
                        return callback(error);
                    }
                    if (line.length) {
                        perLineCallback(line, lineNumber); // handle last line
                    }
                    fs.close(fd, function (error) {
                        callback(error);
                    });
                });
            });
        };
        return Engine;
    }());
    exports.Engine = Engine;
    var FileMatch = (function () {
        function FileMatch(path) {
            this.path = path;
            this.lineMatches = [];
        }
        FileMatch.prototype.addMatch = function (lineMatch) {
            this.lineMatches.push(lineMatch);
        };
        FileMatch.prototype.isEmpty = function () {
            return this.lineMatches.length === 0;
        };
        FileMatch.prototype.serialize = function () {
            var lineMatches = [];
            for (var i = 0; i < this.lineMatches.length; i++) {
                lineMatches.push(this.lineMatches[i].serialize());
            }
            return {
                path: this.path,
                lineMatches: lineMatches
            };
        };
        return FileMatch;
    }());
    var LineMatch = (function () {
        function LineMatch(preview, lineNumber) {
            this.preview = preview.replace(/(\r|\n)*$/, '');
            this.lineNumber = lineNumber;
            this.offsetAndLengths = [];
        }
        LineMatch.prototype.getText = function () {
            return this.preview;
        };
        LineMatch.prototype.getLineNumber = function () {
            return this.lineNumber;
        };
        LineMatch.prototype.addMatch = function (offset, length) {
            this.offsetAndLengths.push([offset, length]);
        };
        LineMatch.prototype.serialize = function () {
            var result = {
                preview: this.preview,
                lineNumber: this.lineNumber,
                offsetAndLengths: this.offsetAndLengths
            };
            return result;
        };
        return LineMatch;
    }());
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/search/node/rawSearchService", ["require", "exports", 'fs', 'graceful-fs', 'vs/base/common/winjs.base', 'vs/platform/files/common/files', 'vs/workbench/services/search/node/fileSearch', 'vs/workbench/services/search/node/textSearch'], function (require, exports, fs, gracefulFs, winjs_base_1, files_1, fileSearch_1, textSearch_1) {
    'use strict';
    gracefulFs.gracefulify(fs);
    var SearchService = (function () {
        function SearchService() {
        }
        SearchService.prototype.fileSearch = function (config) {
            var engine = new fileSearch_1.Engine(config);
            return this.doSearch(engine);
        };
        SearchService.prototype.textSearch = function (config) {
            var engine = new textSearch_1.Engine(config, new fileSearch_1.FileWalker({
                rootFolders: config.rootFolders,
                extraFiles: config.extraFiles,
                includePattern: config.includePattern,
                excludePattern: config.excludePattern,
                filePattern: config.filePattern,
                maxFilesize: files_1.MAX_FILE_SIZE
            }));
            return this.doSearch(engine);
        };
        SearchService.prototype.doSearch = function (engine) {
            return new winjs_base_1.PPromise(function (c, e, p) {
                engine.search(function (match) {
                    if (match) {
                        p(match);
                    }
                }, function (progress) {
                    p(progress);
                }, function (error, isLimitHit) {
                    if (error) {
                        e(error);
                    }
                    else {
                        c({
                            limitHit: isLimitHit
                        });
                    }
                });
            }, function () { return engine.cancel(); });
        };
        return SearchService;
    }());
    exports.SearchService = SearchService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/statusbar/common/statusbarService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    'use strict';
    exports.IStatusbarService = instantiation_1.createDecorator('statusbarService');
    (function (StatusbarAlignment) {
        StatusbarAlignment[StatusbarAlignment["LEFT"] = 0] = "LEFT";
        StatusbarAlignment[StatusbarAlignment["RIGHT"] = 1] = "RIGHT";
    })(exports.StatusbarAlignment || (exports.StatusbarAlignment = {}));
    var StatusbarAlignment = exports.StatusbarAlignment;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostStatusBar", ["require", "exports", 'vs/platform/thread/common/thread', 'vs/workbench/services/statusbar/common/statusbarService', './extHostTypes'], function (require, exports, thread_1, statusbarService_1, extHostTypes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExtHostStatusBarEntry = (function () {
        function ExtHostStatusBarEntry(proxy, alignment, priority) {
            if (alignment === void 0) { alignment = extHostTypes_1.StatusBarAlignment.Left; }
            this._id = ExtHostStatusBarEntry.ID_GEN++;
            this._proxy = proxy;
            this._alignment = alignment;
            this._priority = priority;
        }
        Object.defineProperty(ExtHostStatusBarEntry.prototype, "id", {
            get: function () {
                return this._id;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtHostStatusBarEntry.prototype, "alignment", {
            get: function () {
                return this._alignment;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtHostStatusBarEntry.prototype, "priority", {
            get: function () {
                return this._priority;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtHostStatusBarEntry.prototype, "text", {
            get: function () {
                return this._text;
            },
            set: function (text) {
                this._text = text;
                this.update();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtHostStatusBarEntry.prototype, "tooltip", {
            get: function () {
                return this._tooltip;
            },
            set: function (tooltip) {
                this._tooltip = tooltip;
                this.update();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtHostStatusBarEntry.prototype, "color", {
            get: function () {
                return this._color;
            },
            set: function (color) {
                this._color = color;
                this.update();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtHostStatusBarEntry.prototype, "command", {
            get: function () {
                return this._command;
            },
            set: function (command) {
                this._command = command;
                this.update();
            },
            enumerable: true,
            configurable: true
        });
        ExtHostStatusBarEntry.prototype.show = function () {
            this._visible = true;
            this.update();
        };
        ExtHostStatusBarEntry.prototype.hide = function () {
            this._visible = false;
            this._proxy.dispose(this.id);
        };
        ExtHostStatusBarEntry.prototype.update = function () {
            var _this = this;
            if (this._disposed || !this._visible) {
                return;
            }
            if (this._timeoutHandle) {
                clearTimeout(this._timeoutHandle);
            }
            // Defer the update so that multiple changes to setters dont cause a redraw each
            this._timeoutHandle = setTimeout(function () {
                _this._timeoutHandle = null;
                // Set to status bar
                _this._proxy.setEntry(_this.id, _this.text, _this.tooltip, _this.command, _this.color, _this._alignment === extHostTypes_1.StatusBarAlignment.Left ? statusbarService_1.StatusbarAlignment.LEFT : statusbarService_1.StatusbarAlignment.RIGHT, _this._priority);
            }, 0);
        };
        ExtHostStatusBarEntry.prototype.dispose = function () {
            this.hide();
            this._disposed = true;
        };
        ExtHostStatusBarEntry.ID_GEN = 0;
        return ExtHostStatusBarEntry;
    }());
    exports.ExtHostStatusBarEntry = ExtHostStatusBarEntry;
    var StatusBarMessage = (function () {
        function StatusBarMessage(statusBar) {
            this._messages = [];
            this._item = statusBar.createStatusBarEntry(extHostTypes_1.StatusBarAlignment.Left, Number.MIN_VALUE);
        }
        StatusBarMessage.prototype.dispose = function () {
            this._messages.length = 0;
            this._item.dispose();
        };
        StatusBarMessage.prototype.setMessage = function (message) {
            var _this = this;
            var data = { message: message }; // use object to not confuse equal strings
            this._messages.unshift(data);
            this._update();
            return new extHostTypes_1.Disposable(function () {
                var idx = _this._messages.indexOf(data);
                if (idx >= 0) {
                    _this._messages.splice(idx, 1);
                    _this._update();
                }
            });
        };
        StatusBarMessage.prototype._update = function () {
            if (this._messages.length > 0) {
                this._item.text = this._messages[0].message;
                this._item.show();
            }
            else {
                this._item.hide();
            }
        };
        return StatusBarMessage;
    }());
    var ExtHostStatusBar = (function () {
        function ExtHostStatusBar(threadService) {
            this._proxy = threadService.getRemotable(MainThreadStatusBar);
            this._statusMessage = new StatusBarMessage(this);
        }
        ExtHostStatusBar.prototype.createStatusBarEntry = function (alignment, priority) {
            return new ExtHostStatusBarEntry(this._proxy, alignment, priority);
        };
        ExtHostStatusBar.prototype.setStatusBarMessage = function (text, timeoutOrThenable) {
            var d = this._statusMessage.setMessage(text);
            var handle;
            if (typeof timeoutOrThenable === 'number') {
                handle = setTimeout(function () { return d.dispose(); }, timeoutOrThenable);
            }
            else if (typeof timeoutOrThenable !== 'undefined') {
                timeoutOrThenable.then(function () { return d.dispose(); }, function () { return d.dispose(); });
            }
            return new extHostTypes_1.Disposable(function () {
                d.dispose();
                clearTimeout(handle);
            });
        };
        ExtHostStatusBar = __decorate([
            __param(0, thread_1.IThreadService)
        ], ExtHostStatusBar);
        return ExtHostStatusBar;
    }());
    exports.ExtHostStatusBar = ExtHostStatusBar;
    var MainThreadStatusBar = (function () {
        function MainThreadStatusBar(statusbarService) {
            this.statusbarService = statusbarService;
            this.mapIdToDisposable = Object.create(null);
        }
        MainThreadStatusBar.prototype.setEntry = function (id, text, tooltip, command, color, alignment, priority) {
            // Dispose any old
            this.dispose(id);
            // Add new
            var disposeable = this.statusbarService.addEntry({ text: text, tooltip: tooltip, command: command, color: color }, alignment, priority);
            this.mapIdToDisposable[id] = disposeable;
        };
        MainThreadStatusBar.prototype.dispose = function (id) {
            var disposeable = this.mapIdToDisposable[id];
            if (disposeable) {
                disposeable.dispose();
            }
            delete this.mapIdToDisposable[id];
        };
        MainThreadStatusBar = __decorate([
            thread_1.Remotable.MainContext('MainThreadStatusBar'),
            __param(0, statusbarService_1.IStatusbarService)
        ], MainThreadStatusBar);
        return MainThreadStatusBar;
    }());
    exports.MainThreadStatusBar = MainThreadStatusBar;
});

define("vs/workbench/browser/parts/statusbar/statusbar", ["require", "exports", 'vs/platform/platform', 'vs/workbench/services/statusbar/common/statusbarService', 'vs/platform/instantiation/common/descriptors'], function (require, exports, platform_1, statusbarService, descriptors_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.StatusbarAlignment = statusbarService.StatusbarAlignment;
    var StatusbarItemDescriptor = (function () {
        function StatusbarItemDescriptor(ctor, alignment, priority) {
            this.syncDescriptor = descriptors_1.createSyncDescriptor(ctor);
            this.alignment = alignment || exports.StatusbarAlignment.LEFT;
            this.priority = priority || 0;
        }
        return StatusbarItemDescriptor;
    }());
    exports.StatusbarItemDescriptor = StatusbarItemDescriptor;
    var StatusbarRegistry = (function () {
        function StatusbarRegistry() {
            this._items = [];
        }
        Object.defineProperty(StatusbarRegistry.prototype, "items", {
            get: function () {
                return this._items;
            },
            enumerable: true,
            configurable: true
        });
        StatusbarRegistry.prototype.registerStatusbarItem = function (descriptor) {
            this._items.push(descriptor);
        };
        return StatusbarRegistry;
    }());
    exports.Extensions = {
        Statusbar: 'workbench.contributions.statusbar'
    };
    platform_1.Registry.add(exports.Extensions.Statusbar, new StatusbarRegistry());
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/statusbar/statusbarPart", ["require", "exports", 'vs/base/browser/dom', 'vs/base/common/types', 'vs/nls!vs/workbench/browser/parts/statusbar/statusbarPart', 'vs/base/common/errors', 'vs/base/common/winjs.base', 'vs/base/common/lifecycle', 'vs/base/browser/builder', 'vs/base/browser/ui/octiconLabel/octiconLabel', 'vs/platform/platform', 'vs/platform/keybinding/common/keybindingService', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/browser/part', 'vs/workbench/common/actionRegistry', 'vs/workbench/browser/parts/statusbar/statusbar', 'vs/platform/instantiation/common/instantiation', 'vs/platform/telemetry/common/telemetry', 'vs/platform/message/common/message', 'vs/workbench/services/statusbar/common/statusbarService', 'vs/css!./media/statusbarPart'], function (require, exports, dom, types, nls, errors_1, winjs_base_1, lifecycle_1, builder_1, octiconLabel_1, platform_1, keybindingService_1, editorService_1, part_1, actionRegistry_1, statusbar_1, instantiation_1, telemetry_1, message_1, statusbarService_1) {
    'use strict';
    var StatusbarPart = (function (_super) {
        __extends(StatusbarPart, _super);
        function StatusbarPart(id) {
            _super.call(this, id);
            this.serviceId = statusbarService_1.IStatusbarService;
            this.toDispose = [];
        }
        StatusbarPart.prototype.setInstantiationService = function (service) {
            this.instantiationService = service;
        };
        StatusbarPart.prototype.addEntry = function (entry, alignment, priority) {
            if (priority === void 0) { priority = 0; }
            // Render entry in status bar
            var el = this.doCreateStatusItem(alignment, priority);
            var item = this.instantiationService.createInstance(StatusBarEntryItem, entry);
            var toDispose = item.render(el);
            // Insert according to priority
            var container = this.statusItemsContainer.getHTMLElement();
            var neighbours = this.getEntries(alignment);
            var inserted = false;
            for (var i = 0; i < neighbours.length; i++) {
                var neighbour = neighbours[i];
                var nPriority = builder_1.$(neighbour).getProperty(StatusbarPart.PRIORITY_PROP);
                if (alignment === statusbar_1.StatusbarAlignment.LEFT && nPriority < priority ||
                    alignment === statusbar_1.StatusbarAlignment.RIGHT && nPriority > priority) {
                    container.insertBefore(el, neighbour);
                    inserted = true;
                    break;
                }
            }
            if (!inserted) {
                container.appendChild(el);
            }
            return {
                dispose: function () {
                    builder_1.$(el).destroy();
                    if (toDispose) {
                        toDispose.dispose();
                    }
                }
            };
        };
        StatusbarPart.prototype.getEntries = function (alignment) {
            var entries = [];
            var container = this.statusItemsContainer.getHTMLElement();
            var children = container.children;
            for (var i = 0; i < children.length; i++) {
                var childElement = children.item(i);
                if (builder_1.$(childElement).getProperty(StatusbarPart.ALIGNMENT_PROP) === alignment) {
                    entries.push(childElement);
                }
            }
            return entries;
        };
        StatusbarPart.prototype.createContentArea = function (parent) {
            var _this = this;
            this.statusItemsContainer = builder_1.$(parent);
            // Fill in initial items that were contributed from the registry
            var registry = platform_1.Registry.as(statusbar_1.Extensions.Statusbar);
            var leftDescriptors = registry.items.filter(function (d) { return d.alignment === statusbar_1.StatusbarAlignment.LEFT; }).sort(function (a, b) { return b.priority - a.priority; });
            var rightDescriptors = registry.items.filter(function (d) { return d.alignment === statusbar_1.StatusbarAlignment.RIGHT; }).sort(function (a, b) { return a.priority - b.priority; });
            var descriptors = rightDescriptors.concat(leftDescriptors); // right first because they float
            (_a = this.toDispose).push.apply(_a, descriptors.map(function (descriptor) {
                var item = _this.instantiationService.createInstance(descriptor.syncDescriptor);
                var el = _this.doCreateStatusItem(descriptor.alignment, descriptor.priority);
                var dispose = item.render(el);
                _this.statusItemsContainer.append(el);
                return dispose;
            }));
            return this.statusItemsContainer;
            var _a;
        };
        StatusbarPart.prototype.doCreateStatusItem = function (alignment, priority) {
            if (priority === void 0) { priority = 0; }
            var el = document.createElement('div');
            dom.addClass(el, 'statusbar-item');
            if (alignment === statusbar_1.StatusbarAlignment.RIGHT) {
                dom.addClass(el, 'right');
            }
            else {
                dom.addClass(el, 'left');
            }
            builder_1.$(el).setProperty(StatusbarPart.PRIORITY_PROP, priority);
            builder_1.$(el).setProperty(StatusbarPart.ALIGNMENT_PROP, alignment);
            return el;
        };
        StatusbarPart.prototype.dispose = function () {
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            _super.prototype.dispose.call(this);
        };
        StatusbarPart.PRIORITY_PROP = 'priority';
        StatusbarPart.ALIGNMENT_PROP = 'alignment';
        return StatusbarPart;
    }(part_1.Part));
    exports.StatusbarPart = StatusbarPart;
    var StatusBarEntryItem = (function () {
        function StatusBarEntryItem(entry, keybindingService, instantiationService, messageService, telemetryService, editorService) {
            this.keybindingService = keybindingService;
            this.instantiationService = instantiationService;
            this.messageService = messageService;
            this.telemetryService = telemetryService;
            this.editorService = editorService;
            this.entry = entry;
        }
        StatusBarEntryItem.prototype.render = function (el) {
            var _this = this;
            var toDispose = [];
            dom.addClass(el, 'statusbar-entry');
            // Text Container
            var textContainer;
            if (this.entry.command) {
                textContainer = document.createElement('a');
                builder_1.$(textContainer).on('click', function () { return _this.executeCommand(_this.entry.command); }, toDispose);
            }
            else {
                textContainer = document.createElement('span');
            }
            // Label
            new octiconLabel_1.OcticonLabel(textContainer).text = this.entry.text;
            // Tooltip
            if (this.entry.tooltip) {
                builder_1.$(textContainer).title(this.entry.tooltip);
            }
            // Color
            if (this.entry.color) {
                builder_1.$(textContainer).color(this.entry.color);
            }
            el.appendChild(textContainer);
            return {
                dispose: function () {
                    toDispose = lifecycle_1.disposeAll(toDispose);
                }
            };
        };
        StatusBarEntryItem.prototype.executeCommand = function (id) {
            var _this = this;
            var action;
            var activeEditor = this.editorService.getActiveEditor();
            // Lookup built in commands
            var builtInActionDescriptor = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions).getWorkbenchAction(id);
            if (builtInActionDescriptor) {
                action = this.instantiationService.createInstance(builtInActionDescriptor.syncDescriptor);
            }
            // Lookup editor commands
            if (!action) {
                var activeEditorControl = (activeEditor ? activeEditor.getControl() : null);
                if (activeEditorControl && types.isFunction(activeEditorControl.getAction)) {
                    action = activeEditorControl.getAction(id);
                }
            }
            // Some actions or commands might only be enabled for an active editor, so focus it first
            if (activeEditor) {
                activeEditor.focus();
            }
            // Run it if enabled
            if (action) {
                if (action.enabled) {
                    this.telemetryService.publicLog('workbenchActionExecuted', { id: action.id, from: 'status bar' });
                    (action.run() || winjs_base_1.TPromise.as(null)).done(function () {
                        action.dispose();
                    }, function (err) { return _this.messageService.show(message_1.Severity.Error, errors_1.toErrorMessage(err)); });
                }
                else {
                    this.messageService.show(message_1.Severity.Warning, nls.localize(0, null, action.label || id));
                }
            }
            else {
                this.keybindingService.executeCommand(id).done(undefined, function (err) { return _this.messageService.show(message_1.Severity.Error, errors_1.toErrorMessage(err)); });
            }
        };
        StatusBarEntryItem = __decorate([
            __param(1, keybindingService_1.IKeybindingService),
            __param(2, instantiation_1.IInstantiationService),
            __param(3, message_1.IMessageService),
            __param(4, telemetry_1.ITelemetryService),
            __param(5, editorService_1.IWorkbenchEditorService)
        ], StatusBarEntryItem);
        return StatusBarEntryItem;
    }());
});

define("vs/workbench/services/message/browser/messageService", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/types', 'vs/base/browser/ui/messagelist/messageList', 'vs/workbench/common/constants', 'vs/workbench/browser/parts/statusbar/statusbar', 'vs/platform/message/common/message'], function (require, exports, errors, types, messageList_1, constants_1, statusbar_1, message_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var WorkbenchMessageService = (function () {
        function WorkbenchMessageService(telemetryService, keybindingService) {
            this.telemetryService = telemetryService;
            this.serviceId = message_1.IMessageService;
            this.messagesShowingContextKey = keybindingService.createKey(WorkbenchMessageService.GLOBAL_MESSAGES_SHOWING_CONTEXT, false);
            this.handler = new messageList_1.MessageList(constants_1.Identifiers.WORKBENCH_CONTAINER, telemetryService);
            this.messageBuffer = [];
            this.canShowMessages = true;
            this.disposeables = [];
            this.registerListeners();
        }
        WorkbenchMessageService.prototype.setWorkbenchServices = function (quickOpenService, statusbarService) {
            this.statusbarService = statusbarService;
            this.quickOpenService = quickOpenService;
            this.disposeables.push(this.quickOpenService.onShow(this.onQuickOpenShowing, this));
            this.disposeables.push(this.quickOpenService.onHide(this.onQuickOpenHiding, this));
        };
        WorkbenchMessageService.prototype.registerListeners = function () {
            this.disposeables.push(this.handler.onMessagesShowing(this.onMessagesShowing, this));
            this.disposeables.push(this.handler.onMessagesCleared(this.onMessagesCleared, this));
        };
        WorkbenchMessageService.prototype.onMessagesShowing = function () {
            this.messagesShowingContextKey.set(true);
        };
        WorkbenchMessageService.prototype.onMessagesCleared = function () {
            this.messagesShowingContextKey.reset();
        };
        WorkbenchMessageService.prototype.onQuickOpenShowing = function () {
            this.canShowMessages = false; // when quick open is open, don't show messages behind
            this.handler.hide(); // hide messages when quick open is visible
        };
        WorkbenchMessageService.prototype.onQuickOpenHiding = function () {
            this.canShowMessages = true;
            this.handler.show(); // make sure the handler is visible
            // Release messages from buffer
            while (this.messageBuffer.length) {
                var bufferedMessage = this.messageBuffer.pop();
                bufferedMessage.disposeFn = this.show(bufferedMessage.severity, bufferedMessage.message);
            }
        };
        WorkbenchMessageService.prototype.toBaseSeverity = function (severity) {
            switch (severity) {
                case message_1.Severity.Info:
                    return messageList_1.Severity.Info;
                case message_1.Severity.Warning:
                    return messageList_1.Severity.Warning;
            }
            return messageList_1.Severity.Error;
        };
        WorkbenchMessageService.prototype.show = function (sev, message) {
            var _this = this;
            if (!message) {
                return function () { return void 0; }; // guard against undefined messages
            }
            if (Array.isArray(message)) {
                var closeFns_1 = [];
                message.forEach(function (msg) { return closeFns_1.push(_this.show(sev, msg)); });
                return function () { return closeFns_1.forEach(function (fn) { return fn(); }); };
            }
            if (errors.isPromiseCanceledError(message)) {
                return function () { return void 0; }; // this kind of error should not be shown
            }
            if (types.isNumber(message.severity)) {
                sev = message.severity;
            }
            return this.doShow(sev, message);
        };
        WorkbenchMessageService.prototype.doShow = function (sev, message) {
            var _this = this;
            // Check flag if we can show a message now
            if (!this.canShowMessages) {
                var messageObj_1 = { severity: sev, message: message, disposeFn: function () { return _this.messageBuffer.splice(_this.messageBuffer.indexOf(messageObj_1), 1); } };
                this.messageBuffer.push(messageObj_1);
                // Return function that allows to remove message from buffer
                return function () { return messageObj_1.disposeFn(); };
            }
            // Show in Console
            if (sev === message_1.Severity.Error) {
                console.error(errors.toErrorMessage(message, true));
            }
            // Show in Global Handler
            return this.handler.showMessage(this.toBaseSeverity(sev), message);
        };
        WorkbenchMessageService.prototype.setStatusMessage = function (message, autoDisposeAfter, delayBy) {
            var _this = this;
            if (autoDisposeAfter === void 0) { autoDisposeAfter = -1; }
            if (delayBy === void 0) { delayBy = 0; }
            if (this.statusbarService) {
                if (this.statusMsgDispose) {
                    this.statusMsgDispose.dispose(); // dismiss any previous
                }
                // Create new
                var statusDispose_1;
                var showHandle_1 = setTimeout(function () {
                    statusDispose_1 = _this.statusbarService.addEntry({ text: message }, statusbar_1.StatusbarAlignment.LEFT, Number.MIN_VALUE);
                    showHandle_1 = null;
                }, delayBy);
                var hideHandle_1;
                // Dispose function takes care of timeouts and actual entry
                var dispose_1 = { dispose: function () {
                        if (showHandle_1) {
                            clearTimeout(showHandle_1);
                        }
                        if (hideHandle_1) {
                            clearTimeout(hideHandle_1);
                        }
                        if (statusDispose_1) {
                            statusDispose_1.dispose();
                        }
                    } };
                this.statusMsgDispose = dispose_1;
                if (typeof autoDisposeAfter === 'number' && autoDisposeAfter > 0) {
                    hideHandle_1 = setTimeout(function () { return dispose_1.dispose(); }, autoDisposeAfter);
                }
                return dispose_1;
            }
            return { dispose: function () { } };
        };
        WorkbenchMessageService.prototype.hideAll = function () {
            if (this.handler) {
                this.handler.hideMessages();
            }
        };
        WorkbenchMessageService.prototype.confirm = function (confirmation) {
            var messageText = confirmation.message;
            if (confirmation.detail) {
                messageText = messageText + '\n\n' + confirmation.detail;
            }
            return window.confirm(messageText);
        };
        WorkbenchMessageService.prototype.dispose = function () {
            while (this.disposeables.length) {
                this.disposeables.pop().dispose();
            }
        };
        WorkbenchMessageService.GLOBAL_MESSAGES_SHOWING_CONTEXT = 'globalMessageVisible';
        return WorkbenchMessageService;
    }());
    exports.WorkbenchMessageService = WorkbenchMessageService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/message/electron-browser/messageService", ["require", "exports", 'vs/nls!vs/workbench/services/message/electron-browser/messageService', 'vs/workbench/services/message/browser/messageService', 'vs/base/common/platform'], function (require, exports, nls, messageService_1, platform_1) {
    'use strict';
    var MessageService = (function (_super) {
        __extends(MessageService, _super);
        function MessageService(contextService, windowService, telemetryService, keybindingService) {
            _super.call(this, telemetryService, keybindingService);
            this.contextService = contextService;
            this.windowService = windowService;
        }
        MessageService.prototype.confirm = function (confirmation) {
            if (!confirmation.primaryButton) {
                confirmation.primaryButton = nls.localize(0, null);
            }
            if (!confirmation.secondaryButton) {
                confirmation.secondaryButton = nls.localize(1, null);
            }
            var opts = {
                title: confirmation.title || this.contextService.getConfiguration().env.appName,
                message: confirmation.message,
                buttons: [
                    this.mnemonicLabel(confirmation.primaryButton),
                    this.mnemonicLabel(confirmation.secondaryButton)
                ],
                noLink: true,
                cancelId: 1
            };
            if (confirmation.detail) {
                opts.detail = confirmation.detail;
            }
            var result = this.windowService.getWindow().showMessageBox(opts);
            return result === 0 ? true : false;
        };
        MessageService.prototype.mnemonicLabel = function (label) {
            if (!platform_1.isWindows) {
                return label.replace(/&&/g, ''); // no mnemonic support on mac/linux in buttons yet
            }
            return label.replace(/&&/g, '&');
        };
        return MessageService;
    }(messageService_1.WorkbenchMessageService));
    exports.MessageService = MessageService;
});

define("vs/workbench/services/themes/common/themeService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IThemeService = instantiation_1.createDecorator('themeService');
    exports.DEFAULT_THEME_ID = 'vs-dark vscode-theme-defaults-themes-dark_plus-json';
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/html/browser/htmlPreviewPart", ["require", "exports", 'vs/nls!vs/workbench/parts/html/browser/htmlPreviewPart', 'vs/base/common/winjs.base', 'vs/editor/common/editorCommon', 'vs/base/common/lifecycle', 'vs/base/browser/dom', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/platform/telemetry/common/telemetry', 'vs/platform/workspace/common/workspace', 'vs/platform/storage/common/storage', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/common/editor/textEditorModel', 'vs/workbench/common/constants', 'vs/workbench/parts/html/common/htmlInput', 'vs/platform/theme/common/themes', 'vs/workbench/services/themes/common/themeService'], function (require, exports, nls_1, winjs_base_1, editorCommon_1, lifecycle_1, dom_1, baseEditor_1, telemetry_1, workspace_1, storage_1, editorService_1, textEditorModel_1, constants_1, htmlInput_1, themes_1, themeService_1) {
    'use strict';
    /**
     * An implementation of editor for showing HTML content in an IFrame by leveraging the IFrameEditorInput.
     */
    var HtmlPreviewPart = (function (_super) {
        __extends(HtmlPreviewPart, _super);
        function HtmlPreviewPart(telemetryService, editorService, storageService, contextService) {
            _super.call(this, HtmlPreviewPart.ID, telemetryService);
            this._iFrameMessageSubscription = lifecycle_1.empty;
            this._modelChangeSubscription = lifecycle_1.empty;
            this._themeChangeSubscription = lifecycle_1.empty;
            this._editorService = editorService;
            this._storageService = storageService;
            this._iFrameBase = contextService.toResource('/');
        }
        HtmlPreviewPart.prototype.dispose = function () {
            // remove from dom
            var element = this._iFrameElement.parentElement;
            element.parentElement.removeChild(element);
            // unhook from model
            this._modelChangeSubscription.dispose();
            this._model = undefined;
            this._themeChangeSubscription.dispose();
        };
        HtmlPreviewPart.prototype.createEditor = function (parent) {
            var _this = this;
            // IFrame
            // this.iframeBuilder.removeProperty(IFrameEditor.RESOURCE_PROPERTY);
            this._iFrameElement = document.createElement('iframe');
            this._iFrameElement.setAttribute('frameborder', '0');
            this._iFrameElement.className = 'iframe';
            // Container for IFrame
            var iFrameContainerElement = document.createElement('div');
            iFrameContainerElement.className = 'iframe-container monaco-editor-background'; // Inherit the background color from selected theme
            iFrameContainerElement.tabIndex = 0; // enable focus support from the editor part (do not remove)
            iFrameContainerElement.appendChild(this._iFrameElement);
            parent.getHTMLElement().appendChild(iFrameContainerElement);
            this._themeChangeSubscription = this._storageService.addListener2(storage_1.StorageEventType.STORAGE, function (event) {
                if (event.key === constants_1.Preferences.THEME && _this.isVisible()) {
                    _this._updateIFrameContent(true);
                }
            });
            this._iFrameMessageSubscription = dom_1.addDisposableListener(window, 'message', function (e) {
                if (e.source !== _this._iFrameElement.contentWindow) {
                    return;
                }
                var fakeEvent = document.createEvent('KeyboardEvent'); // create a keyboard event
                Object.defineProperty(fakeEvent, 'keyCode', {
                    get: function () {
                        return e.data.keyCode;
                    }
                });
                Object.defineProperty(fakeEvent, 'which', {
                    get: function () {
                        return e.data.keyCode;
                    }
                });
                Object.defineProperty(fakeEvent, 'target', {
                    get: function () {
                        return window && window.parent.document.body;
                    }
                });
                fakeEvent.initKeyboardEvent('keydown', true, true, document.defaultView, null, null, e.data.ctrlKey, e.data.altKey, e.data.shiftKey, e.data.metaKey); // the API shape of this method is not clear to me, but it works
                document.dispatchEvent(fakeEvent);
            });
        };
        HtmlPreviewPart.prototype.layout = function (dimension) {
            var width = dimension.width, height = dimension.height;
            this._iFrameElement.parentElement.style.width = width + "px";
            this._iFrameElement.parentElement.style.height = height + "px";
            this._iFrameElement.style.width = width + "px";
            this._iFrameElement.style.height = height + "px";
        };
        HtmlPreviewPart.prototype.focus = function () {
            // this.iframeContainer.domFocus();
            this._iFrameElement.focus();
        };
        // --- input
        HtmlPreviewPart.prototype.getTitle = function () {
            if (!this.input) {
                return nls_1.localize(0, null);
            }
            return this.input.getName();
        };
        HtmlPreviewPart.prototype.setVisible = function (visible, position) {
            var _this = this;
            return _super.prototype.setVisible.call(this, visible, position).then(function () {
                if (visible && _this._model) {
                    _this._modelChangeSubscription = _this._model.addListener2(editorCommon_1.EventType.ModelContentChanged2, function () { return _this._updateIFrameContent(); });
                    _this._updateIFrameContent();
                }
                else {
                    _this._modelChangeSubscription.dispose();
                }
            });
        };
        HtmlPreviewPart.prototype.changePosition = function (position) {
            var _this = this;
            _super.prototype.changePosition.call(this, position);
            // reparenting an IFRAME into another DOM element yields weird results when the contents are made
            // of a string and not a URL. to be on the safe side we reload the iframe when the position changes
            // and we do it using a timeout of 0 to reload only after the position has been changed in the DOM
            setTimeout(function () {
                _this._updateIFrameContent(true);
            }, 0);
        };
        HtmlPreviewPart.prototype.setInput = function (input, options) {
            var _this = this;
            this._model = undefined;
            this._modelChangeSubscription.dispose();
            this._lastModelVersion = -1;
            if (!(input instanceof htmlInput_1.HtmlInput)) {
                return winjs_base_1.TPromise.wrapError('Invalid input');
            }
            return this._editorService.resolveEditorModel({ resource: input.getResource() }).then(function (model) {
                if (model instanceof textEditorModel_1.BaseTextEditorModel) {
                    _this._model = model.textEditorModel;
                }
                if (!_this._model) {
                    return winjs_base_1.TPromise.wrapError(nls_1.localize(1, null));
                }
                _this._modelChangeSubscription = _this._model.addListener2(editorCommon_1.EventType.ModelContentChanged2, function () { return _this._updateIFrameContent(); });
                _this._updateIFrameContent();
                return _super.prototype.setInput.call(_this, input, options);
            });
        };
        HtmlPreviewPart.prototype._updateIFrameContent = function (refresh) {
            if (refresh === void 0) { refresh = false; }
            if (!this._model || (!refresh && this._lastModelVersion === this._model.getVersionId())) {
                // nothing to do
                return;
            }
            var html = this._model.getValue();
            var iFrameDocument = this._iFrameElement.contentDocument;
            if (!iFrameDocument) {
                // not visible anymore
                return;
            }
            var parser = new DOMParser();
            var newDocument = parser.parseFromString(html, 'text/html');
            // ensure styles
            var styleElement = Integration.defaultStyle(this._iFrameElement.parentElement, this._storageService.get(constants_1.Preferences.THEME, storage_1.StorageScope.GLOBAL, themeService_1.DEFAULT_THEME_ID));
            if (newDocument.head.hasChildNodes()) {
                newDocument.head.insertBefore(styleElement, newDocument.head.firstChild);
            }
            else {
                newDocument.head.appendChild(styleElement);
            }
            // set baseurl if possible
            if (this._iFrameBase) {
                var baseElement = document.createElement('base');
                baseElement.href = this._iFrameBase.toString();
                newDocument.head.appendChild(baseElement);
            }
            // propagate key events
            newDocument.body.appendChild(Integration.bubbleKeybindings);
            // write new content to iframe
            iFrameDocument.open('text/html', 'replace');
            iFrameDocument.write(newDocument.documentElement.innerHTML);
            iFrameDocument.close();
            this._lastModelVersion = this._model.getVersionId();
        };
        HtmlPreviewPart.ID = 'workbench.editor.htmlPreviewPart';
        HtmlPreviewPart = __decorate([
            __param(0, telemetry_1.ITelemetryService),
            __param(1, editorService_1.IWorkbenchEditorService),
            __param(2, storage_1.IStorageService),
            __param(3, workspace_1.IWorkspaceContextService)
        ], HtmlPreviewPart);
        return HtmlPreviewPart;
    }(baseEditor_1.BaseEditor));
    exports.HtmlPreviewPart = HtmlPreviewPart;
    var Integration;
    (function (Integration) {
        'use strict';
        // scripts
        Integration.bubbleKeybindings = document.createElement('script');
        Integration.bubbleKeybindings.innerHTML = "\n\t\tvar ignoredKeys = [9 /* tab */, 32 /* space */, 33 /* page up */, 34 /* page down */, 38 /* up */, 40 /* down */];\n\t\tvar ignoredCtrlCmdKeys = [65 /* a */, 67 /* c */];\n\t\tvar ignoredShiftKeys = [9 /* tab */];\n\t\twindow.document.body.addEventListener(\"keydown\", function(event) {\n\t\t\ttry {\n\t\t\t\tif (!event.ctrlKey && !event.altKey && !event.shiftKey && !event.metaKey && ignoredKeys.some(function(i) {return i === event.keyCode;})) {\n\t\t\t\t\t\treturn;\n\t\t\t\t}\n\t\t\t\tif ((event.ctrlKey || event.metaKey) && ignoredCtrlCmdKeys.some(function(i) { return i === event.keyCode; })) {\n\t\t\t\t\t\treturn;\n\t\t\t\t}\n\t\t\t\tif (event.shiftKey && ignoredShiftKeys.some(function(i) { return i === event.keyCode; })) {\n\t\t\t\t\treturn;\n\t\t\t\t}\n\t\t\t\tevent.preventDefault();\n\t\t\t\twindow.parent.postMessage({ which: event.which, keyCode: event.keyCode, charCode: event.charCode, metaKey: event.metaKey, altKey: event.altKey, shiftKey: event.shiftKey, ctrlKey: event.ctrlKey }, \"*\");\n\t\t\t} catch (error) { }\n\t\t});\n\t\tfunction defaultPreventHandler(e) { e.preventDefault(); };\n\t\twindow.document.addEventListener(\"dragover\", defaultPreventHandler);\n\t\twindow.document.addEventListener(\"drop\", defaultPreventHandler);\n\t\twindow.document.body.addEventListener(\"dragover\", defaultPreventHandler);\n\t\twindow.document.body.addEventListener(\"drop\", defaultPreventHandler);\n\t";
        // styles
        var defaultLightScrollbarStyle = [
            '::-webkit-scrollbar-thumb {',
            '	background-color: rgba(100, 100, 100, 0.4);',
            '}',
            '::-webkit-scrollbar-thumb:hover {',
            '	background-color: rgba(100, 100, 100, 0.7);',
            '}',
            '::-webkit-scrollbar-thumb:active {',
            '	background-color: rgba(0, 0, 0, 0.6);',
            '}'
        ].join('\n');
        var defaultDarkScrollbarStyle = [
            '::-webkit-scrollbar-thumb {',
            '	background-color: rgba(121, 121, 121, 0.4);',
            '}',
            '::-webkit-scrollbar-thumb:hover {',
            '	background-color: rgba(100, 100, 100, 0.7);',
            '}',
            '::-webkit-scrollbar-thumb:active {',
            '	background-color: rgba(85, 85, 85, 0.8);',
            '}'
        ].join('\n');
        function defaultStyle(element, themeId) {
            var styles = window.getComputedStyle(element);
            var styleElement = document.createElement('style');
            styleElement.innerHTML = "* {\n\t\t\tcolor: " + styles.color + ";\n\t\t\tbackground: " + styles.background + ";\n\t\t\tfont-family: " + styles.fontFamily + ";\n\t\t\tfont-size: " + styles.fontSize + ";\n\t\t}\n\t\timg {\n\t\t\tmax-width: 100%;\n\t\t\tmax-height: 100%;\n\t\t}\n\t\ta:focus,\n\t\tinput:focus,\n\t\tselect:focus,\n\t\ttextarea:focus {\n\t\t\toutline: 1px solid -webkit-focus-ring-color;\n\t\t\toutline-offset: -1px;\n\t\t}\n\t\t::-webkit-scrollbar {\n\t\t\twidth: 14px;\n\t\t\theight: 14px;\n\t\t}\n\t\t" + (themes_1.isLightTheme(themeId)
                ? defaultLightScrollbarStyle
                : defaultDarkScrollbarStyle);
            return styleElement;
        }
        Integration.defaultStyle = defaultStyle;
    })(Integration || (Integration = {}));
});

define("vs/workbench/parts/html/browser/html.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/html/browser/html.contribution', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/platform/instantiation/common/instantiation', 'vs/workbench/services/editor/common/editorService', '../common/htmlInput', 'vs/workbench/parts/html/browser/htmlPreviewPart', 'vs/platform/platform', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/platform/instantiation/common/descriptors'], function (require, exports, nls_1, keybindingsRegistry_1, instantiation_1, editorService_1, htmlInput_1, htmlPreviewPart_1, platform_1, baseEditor_1, descriptors_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // --- Register Editor
    platform_1.Registry.as(baseEditor_1.Extensions.Editors).registerEditor(new baseEditor_1.EditorDescriptor(htmlPreviewPart_1.HtmlPreviewPart.ID, nls_1.localize(0, null), 'vs/workbench/parts/html/browser/htmlPreviewPart', 'HtmlPreviewPart'), [new descriptors_1.SyncDescriptor(htmlInput_1.HtmlInput)]);
    // --- Register Commands
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
        id: '_workbench.previewHtml',
        weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.workbenchContrib(0),
        handler: function (accessor, args) {
            var resource = args[0], position = args[1];
            var name = resource.fsPath;
            var input = accessor.get(instantiation_1.IInstantiationService).createInstance(htmlInput_1.HtmlInput, name, undefined, resource);
            return accessor.get(editorService_1.IWorkbenchEditorService).openEditor(input, null, position)
                .then(function (editor) { return true; });
        },
        context: undefined,
        primary: undefined
    });
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/markdown/common/markdownEditorModel", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/markdown/common/markdownEditorModel', 'vs/base/common/paths', 'vs/workbench/common/editor/iframeEditorModel', 'vs/workbench/common/constants', 'vs/platform/theme/common/themes', 'vs/workbench/services/themes/common/themeService', 'vs/workbench/parts/markdown/common/markdown', 'vs/workbench/services/editor/common/editorService', 'vs/platform/storage/common/storage', 'vs/platform/workspace/common/workspace'], function (require, exports, winjs_base_1, nls, paths, iframeEditorModel_1, constants_1, themes, themeService_1, markdown_1, editorService_1, storage_1, workspace_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The editor model for markdown inputs. Using a library to convert markdown text into HTML from a resource with the provided path.
     */
    var MarkdownEditorModel = (function (_super) {
        __extends(MarkdownEditorModel, _super);
        function MarkdownEditorModel(resource, editorService, contextService, storageService) {
            _super.call(this, resource);
            this.editorService = editorService;
            this.contextService = contextService;
            this.storageService = storageService;
        }
        MarkdownEditorModel.prototype.load = function () {
            var _this = this;
            var isCanceled = false;
            var codeEditorModel;
            // Create a new promise here to be able to return this model even in case of an error
            return new winjs_base_1.TPromise(function (c, e) {
                // On Error: Show error to user as rendered HTML
                var onError = function (error) {
                    try {
                        var theme = _this.storageService.get(constants_1.Preferences.THEME, storage_1.StorageScope.GLOBAL, themeService_1.DEFAULT_THEME_ID);
                        var usesLightTheme = themes.isLightTheme(theme);
                        var markdownError = nls.localize(0, null, paths.basename(_this.resource.fsPath));
                        _this.setContents('<html><head><style type="text/css">body {color: ' + (usesLightTheme ? 'black' : 'white') + '; font-family: "Segoe WPC", "Segoe UI", "HelveticaNeue-Light", sans-serif, "Droid Sans Fallback"; font-size: 13px; margin: 0; line-height: 1.4em; padding-left: 20px;}</style></head><body>', markdownError, '</body></html>');
                        c(_this);
                    }
                    catch (error) {
                        e(error); // be very careful that this promise always completes
                    }
                };
                // On Success: Show output as rendered HTML
                var onSuccess = function (model) {
                    try {
                        var mode = model.getMode();
                        var absoluteWorkerResourcesPath = require.toUrl('vs/languages/markdown/common'); // TODO@Ben technical debt: worker cannot resolve path absolute
                        if (mode && !!mode.emitOutputSupport && mode.getId() === markdown_1.MARKDOWN_MODE_ID) {
                            mode.emitOutputSupport.getEmitOutput(_this.resource, absoluteWorkerResourcesPath).then(function (output) {
                                _this.setContents(output.head, output.body, output.tail);
                                c(_this);
                            }, onError);
                        }
                        else {
                            onError(null); // mode does not support output
                        }
                    }
                    catch (error) {
                        onError(error); // be very careful that this promise always completes
                    }
                };
                // Resolve the text editor model using editor service to benefit from the local editor model cache
                _this.editorService.resolveEditorModel({
                    resource: _this.resource,
                    mime: markdown_1.MARKDOWN_MIME
                }).then(function (model) {
                    if (isCanceled) {
                        return;
                    }
                    codeEditorModel = model.textEditorModel;
                    return codeEditorModel.whenModeIsReady();
                }).then(function () {
                    if (isCanceled) {
                        return;
                    }
                    onSuccess(codeEditorModel);
                }, onError);
            }, function () {
                isCanceled = true;
            });
        };
        MarkdownEditorModel = __decorate([
            __param(1, editorService_1.IWorkbenchEditorService),
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, storage_1.IStorageService)
        ], MarkdownEditorModel);
        return MarkdownEditorModel;
    }(iframeEditorModel_1.IFrameEditorModel));
    exports.MarkdownEditorModel = MarkdownEditorModel;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/markdown/common/markdownEditorInput", ["require", "exports", 'vs/nls!vs/workbench/parts/markdown/common/markdownEditorInput', 'vs/base/common/paths', 'vs/base/common/labels', 'vs/workbench/common/editor/iframeEditorInput', 'vs/workbench/parts/markdown/common/markdownEditorModel', 'vs/platform/instantiation/common/instantiation', 'vs/platform/workspace/common/workspace'], function (require, exports, nls, paths, labels, iframeEditorInput_1, markdownEditorModel_1, instantiation_1, workspace_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * An editor input to show a rendered version of a markdown file.
     */
    var MarkdownEditorInput = (function (_super) {
        __extends(MarkdownEditorInput, _super);
        function MarkdownEditorInput(resource, label, description, instantiationService, contextService) {
            _super.call(this, resource, label || nls.localize(0, null, paths.basename(resource.fsPath)), description || labels.getPathLabel(paths.dirname(resource.fsPath), contextService));
            this.instantiationService = instantiationService;
            this.contextService = contextService;
        }
        MarkdownEditorInput.prototype.createNew = function (resource) {
            return this.instantiationService.createInstance(MarkdownEditorInput, resource, void 0, void 0);
        };
        MarkdownEditorInput.prototype.getId = function () {
            return MarkdownEditorInput.ID;
        };
        MarkdownEditorInput.prototype.createModel = function () {
            return this.instantiationService.createInstance(markdownEditorModel_1.MarkdownEditorModel, this.getResource());
        };
        MarkdownEditorInput.prototype.matches = function (otherInput) {
            if (!(otherInput instanceof MarkdownEditorInput)) {
                return false;
            }
            return _super.prototype.matches.call(this, otherInput);
        };
        MarkdownEditorInput.ID = 'vs.markdown';
        MarkdownEditorInput = __decorate([
            __param(3, instantiation_1.IInstantiationService),
            __param(4, workspace_1.IWorkspaceContextService)
        ], MarkdownEditorInput);
        return MarkdownEditorInput;
    }(iframeEditorInput_1.IFrameEditorInput));
    exports.MarkdownEditorInput = MarkdownEditorInput;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/markdown/browser/markdownActions", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/actions', 'vs/base/common/errors', 'vs/nls!vs/workbench/parts/markdown/browser/markdownActions', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor', 'vs/workbench/parts/markdown/common/markdownEditorInput', 'vs/workbench/services/editor/common/editorService', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/css!./media/markdownactions'], function (require, exports, winjs_base_1, actions_1, errors, nls, baseEditor_1, editor_1, markdownEditorInput_1, editorService_1, instantiation_1, message_1) {
    'use strict';
    var GlobalTogglePreviewMarkdownAction = (function (_super) {
        __extends(GlobalTogglePreviewMarkdownAction, _super);
        function GlobalTogglePreviewMarkdownAction(id, label, instantiationService, editorService, messageService) {
            _super.call(this, id, label);
            this.instantiationService = instantiationService;
            this.editorService = editorService;
            this.messageService = messageService;
        }
        GlobalTogglePreviewMarkdownAction.prototype.run = function (event) {
            var activeInput = this.editorService.getActiveEditorInput();
            // View source if we are in a markdown file already
            if (activeInput instanceof markdownEditorInput_1.MarkdownEditorInput) {
                this.editorService.openEditor({
                    resource: activeInput.getResource()
                }).done(null, errors.onUnexpectedError);
            }
            else {
                var msg = void 0;
                var resource = editor_1.getUntitledOrFileResource(activeInput);
                if (resource) {
                    var action_1 = this.instantiationService.createInstance(PreviewMarkdownAction, resource);
                    action_1.run().done(function () { return action_1.dispose(); }, errors.onUnexpectedError);
                }
                else {
                    msg = nls.localize(1, null);
                }
                if (msg) {
                    this.messageService.show(message_1.Severity.Info, msg);
                }
            }
            return winjs_base_1.TPromise.as(true);
        };
        GlobalTogglePreviewMarkdownAction.ID = 'workbench.action.markdown.togglePreview';
        GlobalTogglePreviewMarkdownAction.LABEL = nls.localize(0, null);
        GlobalTogglePreviewMarkdownAction = __decorate([
            __param(2, instantiation_1.IInstantiationService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, message_1.IMessageService)
        ], GlobalTogglePreviewMarkdownAction);
        return GlobalTogglePreviewMarkdownAction;
    }(actions_1.Action));
    exports.GlobalTogglePreviewMarkdownAction = GlobalTogglePreviewMarkdownAction;
    var OpenPreviewToSideAction = (function (_super) {
        __extends(OpenPreviewToSideAction, _super);
        function OpenPreviewToSideAction(id, label, instantiationService, editorService, messageService) {
            _super.call(this, id, label);
            this.instantiationService = instantiationService;
            this.editorService = editorService;
            this.messageService = messageService;
        }
        OpenPreviewToSideAction.prototype.run = function (event) {
            var activeInput = this.editorService.getActiveEditorInput();
            // Do nothing if already in markdown preview
            if (activeInput instanceof markdownEditorInput_1.MarkdownEditorInput) {
                return winjs_base_1.TPromise.as(true);
            }
            else {
                var msg = void 0;
                var resource = editor_1.getUntitledOrFileResource(activeInput);
                if (resource) {
                    var input = this.instantiationService.createInstance(markdownEditorInput_1.MarkdownEditorInput, resource, void 0, void 0);
                    return this.editorService.openEditor(input, null, true /* to the side */);
                }
                else {
                    msg = nls.localize(3, null);
                }
                if (msg) {
                    this.messageService.show(message_1.Severity.Info, msg);
                }
            }
            return winjs_base_1.TPromise.as(true);
        };
        OpenPreviewToSideAction.ID = 'workbench.action.markdown.openPreviewSideBySide';
        OpenPreviewToSideAction.LABEL = nls.localize(2, null);
        OpenPreviewToSideAction = __decorate([
            __param(2, instantiation_1.IInstantiationService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, message_1.IMessageService)
        ], OpenPreviewToSideAction);
        return OpenPreviewToSideAction;
    }(actions_1.Action));
    exports.OpenPreviewToSideAction = OpenPreviewToSideAction;
    var PreviewMarkdownAction = (function (_super) {
        __extends(PreviewMarkdownAction, _super);
        function PreviewMarkdownAction(markdownResource, instantiationService, editorService) {
            _super.call(this, 'workbench.markdown.action.previewFromExplorer', nls.localize(4, null));
            this.instantiationService = instantiationService;
            this.editorService = editorService;
            this.markdownResource = markdownResource;
        }
        PreviewMarkdownAction.prototype.run = function (event) {
            var input = this.instantiationService.createInstance(markdownEditorInput_1.MarkdownEditorInput, this.markdownResource, void 0, void 0);
            return this.editorService.openEditor(input);
        };
        PreviewMarkdownAction = __decorate([
            __param(1, instantiation_1.IInstantiationService),
            __param(2, editorService_1.IWorkbenchEditorService)
        ], PreviewMarkdownAction);
        return PreviewMarkdownAction;
    }(actions_1.Action));
    exports.PreviewMarkdownAction = PreviewMarkdownAction;
    var PreviewMarkdownEditorInputAction = (function (_super) {
        __extends(PreviewMarkdownEditorInputAction, _super);
        function PreviewMarkdownEditorInputAction(instantiationService, editorService) {
            _super.call(this, 'workbench.markdown.action.previewFromEditor', nls.localize(5, null));
            this.instantiationService = instantiationService;
            this.editorService = editorService;
            this.class = 'markdown-action action-preview';
            this.order = 100; // far end
        }
        PreviewMarkdownEditorInputAction.prototype.run = function (event) {
            var input = this.input;
            var sideBySide = !!(event && (event.ctrlKey || event.metaKey));
            var markdownInput = this.instantiationService.createInstance(markdownEditorInput_1.MarkdownEditorInput, input.getResource(), void 0, void 0);
            return this.editorService.openEditor(markdownInput, null, sideBySide);
        };
        PreviewMarkdownEditorInputAction = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, editorService_1.IWorkbenchEditorService)
        ], PreviewMarkdownEditorInputAction);
        return PreviewMarkdownEditorInputAction;
    }(baseEditor_1.EditorInputAction));
    exports.PreviewMarkdownEditorInputAction = PreviewMarkdownEditorInputAction;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/markdown/browser/markdownActions.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/markdown/browser/markdownActions.contribution', 'vs/platform/platform', 'vs/platform/actions/common/actions', 'vs/workbench/browser/actionBarRegistry', 'vs/workbench/parts/files/common/files', 'vs/base/common/mime', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/parts/markdown/browser/markdownActions', 'vs/workbench/parts/markdown/common/markdown', 'vs/workbench/common/actionRegistry', 'vs/platform/instantiation/common/instantiation', 'vs/base/common/keyCodes'], function (require, exports, nls, platform_1, actions_1, actionBarRegistry_1, files_1, mime, baseEditor_1, markdownActions_1, markdown_1, actionRegistry_1, instantiation_1, keyCodes_1) {
    'use strict';
    var ExplorerViewerActionContributor = (function (_super) {
        __extends(ExplorerViewerActionContributor, _super);
        function ExplorerViewerActionContributor(instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
        }
        ExplorerViewerActionContributor.prototype.hasSecondaryActions = function (context) {
            var element = context.element;
            // Contribute only on file resources
            var fileResource = files_1.asFileResource(element);
            if (!fileResource) {
                return false;
            }
            return !fileResource.isDirectory && mime.guessMimeTypes(fileResource.resource.fsPath).indexOf(markdown_1.MARKDOWN_MIME) >= 0;
        };
        ExplorerViewerActionContributor.prototype.getSecondaryActions = function (context) {
            var actions = [];
            if (this.hasSecondaryActions(context)) {
                var fileResource = files_1.asFileResource(context.element);
                // Open Markdown
                var action = this.instantiationService.createInstance(markdownActions_1.PreviewMarkdownAction, fileResource.resource);
                action.order = 0; // on top of other actions
                actions.push(action);
            }
            return actions;
        };
        ExplorerViewerActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], ExplorerViewerActionContributor);
        return ExplorerViewerActionContributor;
    }(actionBarRegistry_1.ActionBarContributor));
    var MarkdownFilesActionContributor = (function (_super) {
        __extends(MarkdownFilesActionContributor, _super);
        function MarkdownFilesActionContributor(instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
        }
        /* We override toId() to make the caching of actions based on the mime of the input if given */
        MarkdownFilesActionContributor.prototype.toId = function (context) {
            var id = _super.prototype.toId.call(this, context);
            var mime = this.getMimeFromContext(context);
            if (mime) {
                id += mime;
            }
            return id;
        };
        MarkdownFilesActionContributor.prototype.getMimeFromContext = function (context) {
            if (context && context.input && context.input instanceof files_1.FileEditorInput) {
                var fileInput = context.input;
                return fileInput.getMime();
            }
            return null;
        };
        MarkdownFilesActionContributor.prototype.hasActionsForEditorInput = function (context) {
            var input = context.input;
            if (input instanceof files_1.FileEditorInput) {
                var fileResource = input.getResource();
                return mime.guessMimeTypes(fileResource.fsPath).indexOf(markdown_1.MARKDOWN_MIME) >= 0;
            }
            return false;
        };
        MarkdownFilesActionContributor.prototype.getActionsForEditorInput = function (context) {
            if (this.hasActionsForEditorInput(context)) {
                return [
                    this.instantiationService.createInstance(markdownActions_1.PreviewMarkdownEditorInputAction)
                ];
            }
            return [];
        };
        MarkdownFilesActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], MarkdownFilesActionContributor);
        return MarkdownFilesActionContributor;
    }(baseEditor_1.EditorInputActionContributor));
    // Contribute to viewers and editors of markdown files
    var actionBarRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
    actionBarRegistry.registerActionBarContributor(actionBarRegistry_1.Scope.VIEWER, ExplorerViewerActionContributor);
    actionBarRegistry.registerActionBarContributor(actionBarRegistry_1.Scope.EDITOR, MarkdownFilesActionContributor);
    var category = nls.localize(0, null);
    var workbenchActionsRegistry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(markdownActions_1.GlobalTogglePreviewMarkdownAction, markdownActions_1.GlobalTogglePreviewMarkdownAction.ID, markdownActions_1.GlobalTogglePreviewMarkdownAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_V }), category);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(markdownActions_1.OpenPreviewToSideAction, markdownActions_1.OpenPreviewToSideAction.ID, markdownActions_1.OpenPreviewToSideAction.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.KEY_V) }), category);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/themes/electron-browser/themes.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/themes/electron-browser/themes.contribution', 'vs/base/common/actions', 'vs/workbench/common/constants', 'vs/platform/actions/common/actions', 'vs/platform/message/common/message', 'vs/platform/storage/common/storage', 'vs/platform/platform', 'vs/workbench/common/actionRegistry', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/platform/workspace/common/workspace', 'vs/workbench/services/themes/common/themeService', 'vs/base/common/async', 'electron'], function (require, exports, nls, actions, Constants, actions_1, message_1, storage_1, platform, workbenchActionRegistry, quickOpenService_1, workspace_1, themeService_1, async_1, electron_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var SelectThemeAction = (function (_super) {
        __extends(SelectThemeAction, _super);
        function SelectThemeAction(id, label, contextService, quickOpenService, storageService, messageService, themeService) {
            _super.call(this, id, label);
            this.contextService = contextService;
            this.quickOpenService = quickOpenService;
            this.storageService = storageService;
            this.messageService = messageService;
            this.themeService = themeService;
        }
        SelectThemeAction.prototype.run = function () {
            var _this = this;
            return this.themeService.getThemes().then(function (contributedThemes) {
                var currentTheme = _this.storageService.get(Constants.Preferences.THEME, storage_1.StorageScope.GLOBAL, themeService_1.DEFAULT_THEME_ID);
                var picks = [];
                var contributedThemesById = {};
                contributedThemes.forEach(function (theme) {
                    picks.push({ id: theme.id, label: theme.label, description: theme.description });
                    contributedThemes[theme.id] = theme;
                });
                picks = picks.sort(function (t1, t2) { return t1.label.localeCompare(t2.label); });
                var selectedPickIndex;
                picks.forEach(function (p, index) {
                    if (p.id === currentTheme) {
                        selectedPickIndex = index;
                    }
                });
                var selectTheme = function (pick) {
                    if (pick) {
                        var themeId_1 = pick.id;
                        if (!contributedThemesById[themeId_1]) {
                            // built-in theme
                            electron_1.ipcRenderer.send('vscode:changeTheme', themeId_1);
                        }
                        else {
                            // before applying, check that it can be loaded
                            return _this.themeService.applyThemeCSS(themeId_1).then(function (_) {
                                electron_1.ipcRenderer.send('vscode:changeTheme', themeId_1);
                            }, function (error) {
                                _this.messageService.show(message_1.Severity.Info, nls.localize(1, null, error.message));
                            });
                        }
                    }
                    else {
                        // undo changes
                        if (_this.storageService.get(Constants.Preferences.THEME, storage_1.StorageScope.GLOBAL) !== currentTheme) {
                            electron_1.ipcRenderer.send('vscode:changeTheme', currentTheme);
                        }
                    }
                };
                var themeToPreview = null;
                var previewThemeScheduler = new async_1.RunOnceScheduler(function () {
                    selectTheme(themeToPreview);
                }, 200);
                var previewTheme = function (pick) {
                    themeToPreview = pick;
                    previewThemeScheduler.schedule();
                };
                var pickTheme = function (pick) {
                    previewThemeScheduler.dispose();
                    selectTheme(pick);
                };
                return _this.quickOpenService.pick(picks, { placeHolder: nls.localize(2, null), autoFocus: { autoFocusIndex: selectedPickIndex } }).then(pickTheme, null, previewTheme);
            });
        };
        SelectThemeAction.ID = 'workbench.action.selectTheme';
        SelectThemeAction.LABEL = nls.localize(0, null);
        SelectThemeAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, quickOpenService_1.IQuickOpenService),
            __param(4, storage_1.IStorageService),
            __param(5, message_1.IMessageService),
            __param(6, themeService_1.IThemeService)
        ], SelectThemeAction);
        return SelectThemeAction;
    }(actions.Action));
    var category = nls.localize(3, null);
    var workbenchActionsRegistry = platform.Registry.as(workbenchActionRegistry.Extensions.WorkbenchActions);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(SelectThemeAction, SelectThemeAction.ID, SelectThemeAction.LABEL), category);
});

define("vs/workbench/services/themes/node/themeService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/services/themes/node/themeService', 'vs/base/common/paths', 'vs/base/common/json', 'vs/platform/theme/common/themes', 'vs/platform/extensions/common/extensionsRegistry', 'vs/workbench/services/themes/common/themeService', 'vs/base/node/plist', 'vs/base/node/pfs'], function (require, exports, winjs_base_1, nls, Paths, Json, Themes, extensionsRegistry_1, themeService_1, plist, pfs) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // implementation
    var defaultBaseTheme = Themes.getBaseThemeId(themeService_1.DEFAULT_THEME_ID);
    var defaultThemeExtensionId = 'vscode-theme-defaults';
    var oldDefaultThemeExtensionId = 'vscode-theme-colorful-defaults';
    function validateThemeId(theme) {
        // migrations
        switch (theme) {
            case 'vs': return "vs " + defaultThemeExtensionId + "-themes-light_vs-json";
            case 'vs-dark': return "vs-dark " + defaultThemeExtensionId + "-themes-dark_vs-json";
            case 'hc-black': return "hc-black " + defaultThemeExtensionId + "-themes-hc_black-json";
            case "vs " + oldDefaultThemeExtensionId + "-themes-light_plus-tmTheme": return "vs " + defaultThemeExtensionId + "-themes-light_plus-json";
            case "vs-dark " + oldDefaultThemeExtensionId + "-themes-dark_plus-tmTheme": return "vs-dark " + defaultThemeExtensionId + "-themes-dark_plus-json";
        }
        return theme;
    }
    var themesExtPoint = extensionsRegistry_1.ExtensionsRegistry.registerExtensionPoint('themes', {
        description: nls.localize(0, null),
        type: 'array',
        defaultSnippets: [{ body: [{ label: '{{label}}', uiTheme: 'vs-dark', path: './themes/{{id}}.tmTheme.' }] }],
        items: {
            type: 'object',
            defaultSnippets: [{ body: { label: '{{label}}', uiTheme: 'vs-dark', path: './themes/{{id}}.tmTheme.' } }],
            properties: {
                label: {
                    description: nls.localize(1, null),
                    type: 'string'
                },
                uiTheme: {
                    description: nls.localize(2, null),
                    enum: ['vs', 'vs-dark', 'hc-black']
                },
                path: {
                    description: nls.localize(3, null),
                    type: 'string'
                }
            }
        }
    });
    var ThemeService = (function () {
        function ThemeService(extensionService) {
            var _this = this;
            this.extensionService = extensionService;
            this.serviceId = themeService_1.IThemeService;
            this.knownThemes = [];
            themesExtPoint.setHandler(function (extensions) {
                for (var _i = 0, extensions_1 = extensions; _i < extensions_1.length; _i++) {
                    var ext = extensions_1[_i];
                    _this.onThemes(ext.description.extensionFolderPath, ext.description.id, ext.value, ext.collector);
                }
            });
        }
        ThemeService.prototype.loadTheme = function (themeId) {
            themeId = validateThemeId(themeId);
            return this.getThemes().then(function (allThemes) {
                var themes = allThemes.filter(function (t) { return t.id === themeId; });
                if (themes.length > 0) {
                    return themes[0];
                }
                return null;
            });
        };
        ThemeService.prototype.applyThemeCSS = function (themeId) {
            return this.loadTheme(themeId).then(function (theme) {
                if (theme) {
                    return applyTheme(theme);
                }
                return null;
            });
        };
        ThemeService.prototype.getThemes = function () {
            var _this = this;
            return this.extensionService.onReady().then(function (isReady) {
                return _this.knownThemes;
            });
        };
        ThemeService.prototype.onThemes = function (extensionFolderPath, extensionId, themes, collector) {
            var _this = this;
            if (!Array.isArray(themes)) {
                collector.error(nls.localize(4, null, themesExtPoint.name));
                return;
            }
            themes.forEach(function (theme) {
                if (!theme.path || (typeof theme.path !== 'string')) {
                    collector.error(nls.localize(5, null, themesExtPoint.name, String(theme.path)));
                    return;
                }
                var normalizedAbsolutePath = Paths.normalize(Paths.join(extensionFolderPath, theme.path));
                if (normalizedAbsolutePath.indexOf(extensionFolderPath) !== 0) {
                    collector.warn(nls.localize(6, null, themesExtPoint.name, normalizedAbsolutePath, extensionFolderPath));
                }
                var themeSelector = toCssSelector(extensionId + '-' + Paths.normalize(theme.path));
                _this.knownThemes.push({
                    id: (theme.uiTheme || defaultBaseTheme) + " " + themeSelector,
                    label: theme.label || Paths.basename(theme.path),
                    description: theme.description,
                    path: normalizedAbsolutePath
                });
            });
        };
        return ThemeService;
    }());
    exports.ThemeService = ThemeService;
    function toCssSelector(str) {
        return str.replace(/[^_\-a-zA-Z0-9]/g, '-');
    }
    function applyTheme(theme) {
        if (theme.styleSheetContent) {
            _applyRules(theme.styleSheetContent);
        }
        return _loadThemeDocument(theme.path).then(function (themeDocument) {
            var styleSheetContent = _processThemeObject(theme.id, themeDocument);
            theme.styleSheetContent = styleSheetContent;
            _applyRules(styleSheetContent);
            return true;
        }, function (error) {
            return winjs_base_1.TPromise.wrapError(nls.localize(7, null, theme.path));
        });
    }
    function _loadThemeDocument(themePath) {
        return pfs.readFile(themePath).then(function (content) {
            if (Paths.extname(themePath) === '.json') {
                var errors = [];
                var contentValue_1 = Json.parse(content.toString(), errors);
                if (errors.length > 0) {
                    return winjs_base_1.TPromise.wrapError(new Error(nls.localize(8, null, errors.join(', '))));
                }
                if (contentValue_1.include) {
                    return _loadThemeDocument(Paths.join(Paths.dirname(themePath), contentValue_1.include)).then(function (includedValue) {
                        contentValue_1.settings = includedValue.settings.concat(contentValue_1.settings);
                        return winjs_base_1.TPromise.as(contentValue_1);
                    });
                }
                return winjs_base_1.TPromise.as(contentValue_1);
            }
            else {
                var parseResult = plist.parse(content.toString());
                if (parseResult.errors && parseResult.errors.length) {
                    return winjs_base_1.TPromise.wrapError(new Error(nls.localize(9, null, parseResult.errors.join(', '))));
                }
                return winjs_base_1.TPromise.as(parseResult.value);
            }
        });
    }
    function _processThemeObject(themeId, themeDocument) {
        var cssRules = [];
        var themeSettings = themeDocument.settings;
        var editorSettings = {
            background: void 0,
            foreground: void 0,
            caret: void 0,
            invisibles: void 0,
            lineHighlight: void 0,
            selection: void 0
        };
        var themeSelector = Themes.getBaseThemeId(themeId) + "." + Themes.getSyntaxThemeId(themeId);
        if (Array.isArray(themeSettings)) {
            themeSettings.forEach(function (s, index, arr) {
                if (index === 0 && !s.scope) {
                    editorSettings = s.settings;
                }
                else {
                    var scope = s.scope;
                    var settings = s.settings;
                    if (scope && settings) {
                        var rules = Array.isArray(scope) ? scope : scope.split(',');
                        var statements_1 = _settingsToStatements(settings);
                        rules.forEach(function (rule) {
                            rule = rule.trim().replace(/ /g, '.'); // until we have scope hierarchy in the editor dom: replace spaces with .
                            cssRules.push(".monaco-editor." + themeSelector + " .token." + rule + " { " + statements_1 + " }");
                        });
                    }
                }
            });
        }
        if (editorSettings.background) {
            var background = new Color(editorSettings.background);
            //cssRules.push(`.monaco-editor.${themeSelector} { background-color: ${background}; }`);
            cssRules.push(".monaco-editor." + themeSelector + " .monaco-editor-background { background-color: " + background + "; }");
            cssRules.push(".monaco-editor." + themeSelector + " .glyph-margin { background-color: " + background + "; }");
            cssRules.push(".monaco-workbench." + themeSelector + " .monaco-editor-background { background-color: " + background + "; }");
        }
        if (editorSettings.foreground) {
            var foreground = new Color(editorSettings.foreground);
            cssRules.push(".monaco-editor." + themeSelector + " { color: " + foreground + "; }");
            cssRules.push(".monaco-editor." + themeSelector + " .token { color: " + foreground + "; }");
        }
        if (editorSettings.selection) {
            var selection = new Color(editorSettings.selection);
            cssRules.push(".monaco-editor." + themeSelector + " .focused .selected-text { background-color: " + selection + "; }");
            cssRules.push(".monaco-editor." + themeSelector + " .selected-text { background-color: " + selection.transparent(0.5) + "; }");
        }
        if (editorSettings.lineHighlight) {
            var lineHighlight = new Color(editorSettings.lineHighlight);
            cssRules.push(".monaco-editor." + themeSelector + " .current-line { background-color: " + lineHighlight + "; border:0; }");
        }
        if (editorSettings.caret) {
            var caret = new Color(editorSettings.caret);
            cssRules.push(".monaco-editor." + themeSelector + " .cursor { background-color: " + caret + "; border-color: " + caret + "; }");
        }
        if (editorSettings.invisibles) {
            var invisibles = new Color(editorSettings.invisibles);
            cssRules.push(".monaco-editor." + themeSelector + " .token.whitespace { color: " + invisibles + " !important; }");
        }
        return cssRules.join('\n');
    }
    function _settingsToStatements(settings) {
        var statements = [];
        for (var settingName in settings) {
            var value = settings[settingName];
            switch (settingName) {
                case 'foreground':
                    var foreground = new Color(value);
                    statements.push("color: " + foreground + ";");
                    break;
                case 'background':
                    // do not support background color for now, see bug 18924
                    //let background = new Color(value);
                    //statements.push(`background-color: ${background};`);
                    break;
                case 'fontStyle':
                    var segments = value.split(' ');
                    segments.forEach(function (s) {
                        switch (value) {
                            case 'italic':
                                statements.push("font-style: italic;");
                                break;
                            case 'bold':
                                statements.push("font-weight: bold;");
                                break;
                            case 'underline':
                                statements.push("text-decoration: underline;");
                                break;
                        }
                    });
            }
        }
        return statements.join(' ');
    }
    var className = 'contributedColorTheme';
    function _applyRules(styleSheetContent) {
        var themeStyles = document.head.getElementsByClassName(className);
        if (themeStyles.length === 0) {
            var elStyle = document.createElement('style');
            elStyle.type = 'text/css';
            elStyle.className = className;
            elStyle.innerHTML = styleSheetContent;
            document.head.appendChild(elStyle);
        }
        else {
            themeStyles[0].innerHTML = styleSheetContent;
        }
    }
    var Color = (function () {
        function Color(arg) {
            if (typeof arg === 'string') {
                this.parsed = Color.parse(arg);
            }
            else {
                this.parsed = arg;
            }
            this.str = null;
        }
        Color.parse = function (color) {
            function parseHex(str) {
                return parseInt('0x' + str);
            }
            if (color.charAt(0) === '#' && color.length >= 7) {
                var r = parseHex(color.substr(1, 2));
                var g = parseHex(color.substr(3, 2));
                var b = parseHex(color.substr(5, 2));
                var a = color.length === 9 ? parseHex(color.substr(7, 2)) / 0xff : 1;
                return { r: r, g: g, b: b, a: a };
            }
            return { r: 255, g: 0, b: 0, a: 1 };
        };
        Color.prototype.toString = function () {
            if (!this.str) {
                var p = this.parsed;
                this.str = "rgba(" + p.r + ", " + p.g + ", " + p.b + ", " + +p.a.toFixed(2) + ")";
            }
            return this.str;
        };
        Color.prototype.transparent = function (factor) {
            var p = this.parsed;
            return new Color({ r: p.r, g: p.g, b: p.b, a: p.a * factor });
        };
        return Color;
    }());
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/thread/electron-browser/threadService", ["require", "exports", 'vs/nls!vs/workbench/services/thread/electron-browser/threadService', 'vs/base/common/actions', 'vs/base/common/errors', 'vs/base/common/marshalling', 'vs/base/common/objects', 'vs/base/common/strings', 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/base/node/ports', 'vs/platform/extensions/common/ipcRemoteCom', 'vs/platform/message/common/message', 'vs/platform/thread/common/mainThreadService', 'child_process', 'electron'], function (require, exports, nls, actions_1, errors_1, marshalling_1, objects, strings, uri_1, winjs_base_1, ports_1, ipcRemoteCom_1, message_1, mainThreadService_1, child_process_1, electron_1) {
    'use strict';
    exports.PLUGIN_LOG_BROADCAST_CHANNEL = 'vscode:pluginLog';
    exports.PLUGIN_ATTACH_BROADCAST_CHANNEL = 'vscode:pluginAttach';
    // Enable to see detailed message communication between window and extension host
    var logExtensionHostCommunication = false;
    var MainThreadService = (function (_super) {
        __extends(MainThreadService, _super);
        function MainThreadService(contextService, messageService, windowService) {
            var _this = this;
            _super.call(this, contextService, 'vs/editor/common/worker/editorWorkerServer', 1);
            this.extensionHostProcessManager = new ExtensionHostProcessManager(contextService, messageService, windowService);
            var logCommunication = logExtensionHostCommunication || contextService.getConfiguration().env.logPluginHostCommunication;
            // Message: Window --> Extension Host
            this.remoteCom = ipcRemoteCom_1.create(function (msg) {
                if (logCommunication) {
                    console.log('%c[Window \u2192 Extension]%c[len: ' + strings.pad(msg.length, 5, ' ') + ']', 'color: darkgreen', 'color: grey', msg);
                }
                _this.extensionHostProcessManager.postMessage(msg);
            });
            // Message: Extension Host --> Window
            this.extensionHostProcessManager.startExtensionHostProcess(function (msg) {
                if (logCommunication) {
                    console.log('%c[Extension \u2192 Window]%c[len: ' + strings.pad(msg.length, 5, ' ') + ']', 'color: darkgreen', 'color: grey', msg);
                }
                _this.remoteCom.handle(msg);
            });
            this.remoteCom.setManyHandler(this);
        }
        MainThreadService.prototype.dispose = function () {
            this.extensionHostProcessManager.terminate();
        };
        MainThreadService.prototype._registerAndInstantiateExtHostActor = function (id, descriptor) {
            return this._getOrCreateProxyInstance(this.remoteCom, id, descriptor);
        };
        return MainThreadService;
    }(mainThreadService_1.MainThreadService));
    exports.MainThreadService = MainThreadService;
    var ExtensionHostProcessManager = (function () {
        function ExtensionHostProcessManager(contextService, messageService, windowService) {
            this.messageService = messageService;
            this.contextService = contextService;
            this.windowService = windowService;
            // handle extension host lifecycle a bit special when we know we are developing an extension that runs inside
            this.isExtensionDevelopmentHost = !!this.contextService.getConfiguration().env.extensionDevelopmentPath;
            this.unsentMessages = [];
        }
        ExtensionHostProcessManager.prototype.startExtensionHostProcess = function (onExtensionHostMessage) {
            var _this = this;
            var config = this.contextService.getConfiguration();
            var isDev = !config.env.isBuilt || !!config.env.extensionDevelopmentPath;
            var isTestingFromCli = !!config.env.extensionTestsPath && !config.env.debugBrkPluginHost;
            var opts = {
                env: objects.mixin(objects.clone(process.env), { AMD_ENTRYPOINT: 'vs/workbench/node/pluginHostProcess', PIPE_LOGGING: 'true', VERBOSE_LOGGING: true })
            };
            // Help in case we fail to start it
            if (isDev) {
                this.initializeTimer = setTimeout(function () {
                    var msg = config.env.debugBrkPluginHost ? nls.localize(0, null) : nls.localize(1, null);
                    _this.messageService.show(message_1.Severity.Warning, msg);
                }, 10000);
            }
            // Initialize extension host process with hand shakes
            this.initializeExtensionHostProcess = new winjs_base_1.TPromise(function (c, e) {
                // Resolve additional execution args (e.g. debug)
                return _this.resolveDebugPort(config, function (port) {
                    if (port) {
                        opts.execArgv = ['--nolazy', (config.env.debugBrkPluginHost ? '--debug-brk=' : '--debug=') + port];
                    }
                    // Run Extension Host as fork of current process
                    _this.extensionHostProcessHandle = child_process_1.fork(uri_1.default.parse(require.toUrl('bootstrap')).fsPath, ['--type=pluginHost'], opts);
                    // Notify debugger that we are ready to attach to the process if we run a development extension
                    if (config.env.extensionDevelopmentPath && port) {
                        _this.windowService.broadcast({
                            channel: exports.PLUGIN_ATTACH_BROADCAST_CHANNEL,
                            payload: {
                                port: port
                            }
                        }, config.env.extensionDevelopmentPath /* target */);
                    }
                    // Messages from Extension host
                    _this.extensionHostProcessHandle.on('message', function (msg) {
                        // 1) Host is ready to receive messages, initialize it
                        if (msg === 'ready') {
                            if (_this.initializeTimer) {
                                window.clearTimeout(_this.initializeTimer);
                            }
                            var initPayload = marshalling_1.stringify({
                                parentPid: process.pid,
                                contextService: {
                                    workspace: _this.contextService.getWorkspace(),
                                    configuration: _this.contextService.getConfiguration(),
                                    options: _this.contextService.getOptions()
                                },
                            });
                            _this.extensionHostProcessHandle.send(initPayload);
                        }
                        else if (msg === 'initialized') {
                            _this.unsentMessages.forEach(function (m) { return _this.postMessage(m); });
                            _this.unsentMessages = [];
                            c(_this.extensionHostProcessHandle);
                        }
                        else if (msg && msg.type === '__$console') {
                            var logEntry = msg;
                            var args = [];
                            try {
                                var parsed_1 = JSON.parse(logEntry.arguments);
                                args.push.apply(args, Object.getOwnPropertyNames(parsed_1).map(function (o) { return parsed_1[o]; }));
                            }
                            catch (error) {
                                args.push(logEntry.arguments);
                            }
                            // If the first argument is a string, check for % which indicates that the message
                            // uses substitution for variables. In this case, we cannot just inject our colored
                            // [Extension Host] to the front because it breaks substitution.
                            var consoleArgs = [];
                            if (typeof args[0] === 'string' && args[0].indexOf('%') >= 0) {
                                consoleArgs = [("%c[Extension Host]%c " + args[0]), 'color: blue', 'color: black'].concat(args.slice(1));
                            }
                            else {
                                consoleArgs = ['%c[Extension Host]', 'color: blue'].concat(args);
                            }
                            // Send to local console unless we run tests from cli
                            if (!isTestingFromCli) {
                                console[logEntry.severity].apply(console, consoleArgs);
                            }
                            // Log on main side if running tests from cli
                            if (isTestingFromCli) {
                                electron_1.ipcRenderer.send('vscode:log', logEntry);
                            }
                            else if (isDev) {
                                _this.windowService.broadcast({
                                    channel: exports.PLUGIN_LOG_BROADCAST_CHANNEL,
                                    payload: logEntry
                                }, config.env.extensionDevelopmentPath /* target */);
                            }
                        }
                        else {
                            onExtensionHostMessage(msg);
                        }
                    });
                    // Lifecycle
                    var onExit = function () { return _this.terminate(); };
                    process.once('exit', onExit);
                    _this.extensionHostProcessHandle.on('error', function (err) {
                        var errorMessage = errors_1.toErrorMessage(err);
                        if (errorMessage === _this.lastExtensionHostError) {
                            return; // prevent error spam
                        }
                        _this.lastExtensionHostError = errorMessage;
                        _this.messageService.show(message_1.Severity.Error, nls.localize(2, null, errorMessage));
                    });
                    _this.extensionHostProcessHandle.on('exit', function (code, signal) {
                        process.removeListener('exit', onExit);
                        if (!_this.terminating) {
                            // Unexpected termination
                            if (!_this.isExtensionDevelopmentHost) {
                                _this.messageService.show(message_1.Severity.Error, {
                                    message: nls.localize(3, null),
                                    actions: [new actions_1.Action('reloadWindow', nls.localize(4, null), null, true, function () { _this.windowService.getWindow().reload(); return winjs_base_1.TPromise.as(null); })]
                                });
                                console.error('Extension host terminated unexpectedly. Code: ', code, ' Signal: ', signal);
                            }
                            else if (!isTestingFromCli) {
                                _this.windowService.getWindow().close();
                            }
                            else {
                                electron_1.ipcRenderer.send('vscode:exit', code);
                            }
                        }
                    });
                });
            }, function () { return _this.terminate(); });
        };
        ExtensionHostProcessManager.prototype.resolveDebugPort = function (config, clb) {
            // Check for a free debugging port
            if (typeof config.env.debugPluginHostPort === 'number') {
                return ports_1.findFreePort(config.env.debugPluginHostPort, 10 /* try 10 ports */, function (port) {
                    if (!port) {
                        console.warn('%c[Extension Host] %cCould not find a free port for debugging', 'color: blue', 'color: black');
                        return clb(void 0);
                    }
                    if (port !== config.env.debugPluginHostPort) {
                        console.warn('%c[Extension Host] %cProvided debugging port ' + config.env.debugPluginHostPort + ' is not free, using ' + port + ' instead.', 'color: blue', 'color: black');
                    }
                    if (config.env.debugBrkPluginHost) {
                        console.warn('%c[Extension Host] %cSTOPPED on first line for debugging on port ' + port, 'color: blue', 'color: black');
                    }
                    else {
                        console.info('%c[Extension Host] %cdebugger listening on port ' + port, 'color: blue', 'color: black');
                    }
                    return clb(port);
                });
            }
            else {
                return clb(void 0);
            }
        };
        ExtensionHostProcessManager.prototype.postMessage = function (msg) {
            if (this.initializeExtensionHostProcess) {
                this.initializeExtensionHostProcess.done(function (p) { return p.send(msg); });
            }
            else {
                this.unsentMessages.push(msg);
            }
        };
        ExtensionHostProcessManager.prototype.terminate = function () {
            this.terminating = true;
            if (this.extensionHostProcessHandle) {
                this.extensionHostProcessHandle.send({
                    type: '__$terminate'
                });
            }
        };
        return ExtensionHostProcessManager;
    }());
});

define("vs/workbench/services/untitled/common/untitledEditorService", ["require", "exports", 'vs/base/common/uri', 'vs/platform/instantiation/common/instantiation', 'vs/base/common/events', 'vs/base/common/arrays', 'vs/workbench/common/editor/untitledEditorInput'], function (require, exports, uri_1, instantiation_1, events_1, arrays, untitledEditorInput_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IUntitledEditorService = instantiation_1.createDecorator('untitledEditorService');
    var UntitledEditorService = (function () {
        function UntitledEditorService() {
            this.serviceId = exports.IUntitledEditorService;
        }
        UntitledEditorService.prototype.setInstantiationService = function (service) {
            this.instantiationService = service;
        };
        UntitledEditorService.prototype.get = function (resource) {
            return UntitledEditorService.CACHE[resource.toString()];
        };
        UntitledEditorService.prototype.getAll = function (resources) {
            var _this = this;
            if (resources) {
                return arrays.coalesce(resources.map(function (r) { return _this.get(r); }));
            }
            return Object.keys(UntitledEditorService.CACHE).map(function (key) { return UntitledEditorService.CACHE[key]; });
        };
        UntitledEditorService.prototype.isDirty = function (resource) {
            var input = this.get(resource);
            return input && input.isDirty();
        };
        UntitledEditorService.prototype.getDirty = function () {
            return Object.keys(UntitledEditorService.CACHE)
                .map(function (key) { return UntitledEditorService.CACHE[key]; })
                .filter(function (i) { return i.isDirty(); })
                .map(function (i) { return i.getResource(); });
        };
        UntitledEditorService.prototype.createOrGet = function (resource, modeId) {
            var hasAssociatedFilePath = false;
            if (resource) {
                hasAssociatedFilePath = (resource.scheme === 'file');
                resource = this.resourceToUntitled(resource); // ensure we have the right scheme
                if (hasAssociatedFilePath) {
                    UntitledEditorService.KNOWN_ASSOCIATED_FILE_PATHS[resource.toString()] = true; // remember for future lookups
                }
            }
            // Return existing instance if asked for it
            if (resource && UntitledEditorService.CACHE[resource.toString()]) {
                return UntitledEditorService.CACHE[resource.toString()];
            }
            // Create new otherwise
            return this.doCreate(resource, hasAssociatedFilePath, modeId);
        };
        UntitledEditorService.prototype.doCreate = function (resource, hasAssociatedFilePath, modeId) {
            if (!resource) {
                // Create new taking a resource URI that is not already taken
                var counter = Object.keys(UntitledEditorService.CACHE).length + 1;
                do {
                    resource = uri_1.default.create(untitledEditorInput_1.UntitledEditorInput.SCHEMA, null, 'Untitled-' + counter);
                    counter++;
                } while (Object.keys(UntitledEditorService.CACHE).indexOf(resource.toString()) >= 0);
            }
            var input = this.instantiationService.createInstance(untitledEditorInput_1.UntitledEditorInput, resource, hasAssociatedFilePath, modeId);
            // Remove from cache on dispose
            input.addOneTimeListener(events_1.EventType.DISPOSE, function () {
                delete UntitledEditorService.CACHE[input.getResource().toString()];
                delete UntitledEditorService.KNOWN_ASSOCIATED_FILE_PATHS[input.getResource().toString()];
            });
            // Add to cache
            UntitledEditorService.CACHE[resource.toString()] = input;
            return input;
        };
        UntitledEditorService.prototype.resourceToUntitled = function (resource) {
            if (resource.scheme === untitledEditorInput_1.UntitledEditorInput.SCHEMA) {
                return resource;
            }
            return uri_1.default.create(untitledEditorInput_1.UntitledEditorInput.SCHEMA, null, resource.fsPath);
        };
        UntitledEditorService.prototype.hasAssociatedFilePath = function (resource) {
            return !!UntitledEditorService.KNOWN_ASSOCIATED_FILE_PATHS[resource.toString()];
        };
        UntitledEditorService.CACHE = Object.create(null);
        UntitledEditorService.KNOWN_ASSOCIATED_FILE_PATHS = Object.create(null);
        return UntitledEditorService;
    }());
    exports.UntitledEditorService = UntitledEditorService;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/services/editor/browser/editorService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/uri', 'vs/base/common/network', 'vs/base/common/mime', 'vs/platform/platform', 'vs/base/common/paths', 'vs/base/common/types', 'vs/editor/common/editorCommon', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor', 'vs/workbench/common/editor/resourceEditorInput', 'vs/workbench/common/editor/untitledEditorInput', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/workbench/services/editor/common/editorService', 'vs/platform/instantiation/common/instantiation'], function (require, exports, winjs_base_1, uri_1, network, mime_1, platform_1, paths_1, types, editorCommon_1, baseEditor_1, editor_1, resourceEditorInput_1, untitledEditorInput_1, untitledEditorService_1, editorService_1, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var WorkbenchEditorService = (function () {
        function WorkbenchEditorService(editorPart, untitledEditorService, instantiationService) {
            this.untitledEditorService = untitledEditorService;
            this.instantiationService = instantiationService;
            this.serviceId = editorService_1.IWorkbenchEditorService;
            this.editorPart = editorPart;
            this.fileInputDescriptor = platform_1.Registry.as(baseEditor_1.Extensions.Editors).getDefaultFileInput();
        }
        WorkbenchEditorService.prototype.setInstantiationService = function (service) {
            this.instantiationService = service;
        };
        WorkbenchEditorService.prototype.getActiveEditor = function () {
            return this.editorPart.getActiveEditor();
        };
        WorkbenchEditorService.prototype.getActiveEditorInput = function () {
            return this.editorPart.getActiveEditorInput();
        };
        WorkbenchEditorService.prototype.getVisibleEditors = function () {
            return this.editorPart.getVisibleEditors();
        };
        WorkbenchEditorService.prototype.isVisible = function (input, includeDiff) {
            if (!input) {
                return false;
            }
            return this.getVisibleEditors().some(function (editor) {
                if (!editor.input) {
                    return false;
                }
                if (input.matches(editor.input)) {
                    return true;
                }
                if (includeDiff) {
                    var diffInput = editor.input;
                    if (types.isFunction(diffInput.getOriginalInput) && types.isFunction(diffInput.getModifiedInput)) {
                        return input.matches(diffInput.getModifiedInput()) || input.matches(diffInput.getOriginalInput());
                    }
                }
                return false;
            });
        };
        WorkbenchEditorService.prototype.moveEditor = function (from, to) {
            this.editorPart.moveEditor(from, to);
        };
        WorkbenchEditorService.prototype.arrangeEditors = function (arrangement) {
            this.editorPart.arrangeEditors(arrangement);
        };
        WorkbenchEditorService.prototype.setEditors = function (inputs, options) {
            var _this = this;
            return winjs_base_1.Promise.join(inputs.map(function (input) { return _this.inputToType(input); })).then(function (typedInputs) {
                return _this.editorPart.setEditors(typedInputs, options || inputs.map(function (input) {
                    if (input instanceof editor_1.EditorInput) {
                        return null; // no options for editor inputs
                    }
                    return editor_1.TextEditorOptions.from(input); // ITextInputs can carry settings, so support that!
                }));
            });
        };
        WorkbenchEditorService.prototype.openEditor = function (input, arg2, arg3) {
            var _this = this;
            // Support for closing an opened editor at a position by passing null as input
            if (input === null) {
                return this.doOpenEditor(input, null, (types.isNumber(arg2) || types.isBoolean(arg2)) ? arg2 : arg3);
            }
            // Workbench Input Support
            if (input instanceof editor_1.EditorInput) {
                return this.doOpenEditor(input, arg2, arg3);
            }
            // Support opening foreign resources (such as a http link that points outside of the workbench)
            var resourceInput = input;
            if (resourceInput.resource instanceof uri_1.default) {
                var schema = resourceInput.resource.scheme;
                if (schema === network.Schemas.http || schema === network.Schemas.https) {
                    window.open(resourceInput.resource.toString());
                    return winjs_base_1.TPromise.as(null);
                }
            }
            // Untyped Text Editor Support (required for code that uses this service below workbench level)
            var textInput = input;
            return this.inputToType(textInput).then(function (typedFileInput) {
                if (typedFileInput) {
                    return _this.doOpenEditor(typedFileInput, editor_1.TextEditorOptions.from(textInput), arg2);
                }
                return winjs_base_1.TPromise.as(null);
            });
        };
        WorkbenchEditorService.prototype.doOpenEditor = function (input, options, arg3) {
            return this.editorPart.openEditor(input, options, arg3);
        };
        WorkbenchEditorService.prototype.closeEditor = function (arg) {
            var targetEditor = this.findEditor(arg);
            if (targetEditor) {
                return this.editorPart.openEditor(null, null, targetEditor.position);
            }
            return winjs_base_1.TPromise.as(null);
        };
        WorkbenchEditorService.prototype.closeEditors = function (othersOnly) {
            return this.editorPart.closeEditors(othersOnly);
        };
        WorkbenchEditorService.prototype.focusEditor = function (arg) {
            var targetEditor = this.findEditor(arg);
            if (targetEditor) {
                return this.editorPart.openEditor(targetEditor.input, null, targetEditor.position);
            }
            return winjs_base_1.TPromise.as(null);
        };
        WorkbenchEditorService.prototype.activateEditor = function (arg) {
            var targetEditor = this.findEditor(arg);
            if (targetEditor) {
                this.editorPart.activateEditor(targetEditor);
            }
        };
        WorkbenchEditorService.prototype.findEditor = function (arg) {
            // Editor provided
            if (arg instanceof baseEditor_1.BaseEditor) {
                return arg;
            }
            // Find active editor
            if (types.isUndefinedOrNull(arg)) {
                return this.editorPart.getActiveEditor();
            }
            // Target position provided
            if (types.isNumber(arg)) {
                var position = arg;
                var visibleEditors = this.editorPart.getVisibleEditors();
                for (var i = 0; i < visibleEditors.length; i++) {
                    var editor = visibleEditors[i];
                    if (editor.position === position) {
                        return editor;
                    }
                }
            }
            return null;
        };
        WorkbenchEditorService.prototype.resolveEditorModel = function (input, refresh) {
            return this.inputToType(input).then(function (workbenchInput) {
                if (workbenchInput) {
                    // Resolve if applicable
                    if (workbenchInput instanceof editor_1.EditorInput) {
                        return workbenchInput.resolve(!!refresh);
                    }
                }
                return winjs_base_1.TPromise.as(null);
            });
        };
        WorkbenchEditorService.prototype.inputToType = function (input) {
            // Workbench Input Support
            if (input instanceof editor_1.EditorInput) {
                return winjs_base_1.TPromise.as(input);
            }
            // Base Text Editor Support for inmemory resources
            var resourceInput = input;
            if (resourceInput.resource instanceof uri_1.default && resourceInput.resource.scheme === network.Schemas.inMemory) {
                // For in-memory resources we only support to resolve the input from the current active editor
                // because the workbench does not track editor models by in memory URL. This concept is only
                // being used in the code editor.
                var activeEditor = this.getActiveEditor();
                if (activeEditor) {
                    var control = activeEditor.getControl();
                    if (types.isFunction(control.getEditorType)) {
                        // Single Editor: If code editor model matches, return input from editor
                        if (control.getEditorType() === editorCommon_1.EditorType.ICodeEditor) {
                            var codeEditor = control;
                            var model = this.findModel(codeEditor, input);
                            if (model) {
                                return winjs_base_1.TPromise.as(activeEditor.input);
                            }
                        }
                        else if (control.getEditorType() === editorCommon_1.EditorType.IDiffEditor) {
                            var diffInput = activeEditor.input;
                            var diffCodeEditor = control;
                            var originalModel = this.findModel(diffCodeEditor.getOriginalEditor(), input);
                            if (originalModel) {
                                return winjs_base_1.TPromise.as(diffInput.getOriginalInput());
                            }
                            var modifiedModel = this.findModel(diffCodeEditor.getModifiedEditor(), input);
                            if (modifiedModel) {
                                return winjs_base_1.TPromise.as(diffInput.getModifiedInput());
                            }
                        }
                    }
                }
            }
            else if (resourceInput.resource instanceof uri_1.default && (resourceInput.resource.scheme === untitledEditorInput_1.UntitledEditorInput.SCHEMA)) {
                return winjs_base_1.TPromise.as(this.untitledEditorService.createOrGet(resourceInput.resource));
            }
            else if (this.fileInputDescriptor && resourceInput.resource instanceof uri_1.default && resourceInput.resource.scheme === network.Schemas.file) {
                return this.createFileInput(resourceInput.resource, resourceInput.mime);
            }
            else if (resourceInput.resource instanceof uri_1.default) {
                return winjs_base_1.TPromise.as(this.instantiationService.createInstance(resourceEditorInput_1.ResourceEditorInput, paths_1.basename(resourceInput.resource.fsPath), paths_1.dirname(resourceInput.resource.fsPath), resourceInput.resource));
            }
            return winjs_base_1.TPromise.as(null);
        };
        WorkbenchEditorService.prototype.createFileInput = function (resource, mime) {
            return this.instantiationService.createInstance(this.fileInputDescriptor).then(function (typedFileInput) {
                typedFileInput.setResource(resource);
                typedFileInput.setMime(mime || mime_1.guessMimeTypes(resource.fsPath).join(', '));
                return typedFileInput;
            });
        };
        WorkbenchEditorService.prototype.findModel = function (editor, input) {
            var model = editor.getModel();
            if (!model) {
                return null;
            }
            return model.getAssociatedResource().toString() === input.resource.toString() ? model : null;
        };
        WorkbenchEditorService = __decorate([
            __param(1, untitledEditorService_1.IUntitledEditorService),
            __param(2, instantiation_1.IInstantiationService)
        ], WorkbenchEditorService);
        return WorkbenchEditorService;
    }());
    exports.WorkbenchEditorService = WorkbenchEditorService;
    // Helper that implements IEditorPart through an instance of IEditorService
    var EditorPartDelegate = (function () {
        function EditorPartDelegate(service) {
            this.editorService = service;
        }
        EditorPartDelegate.prototype.setEditors = function (inputs) {
            return this.editorService.setEditors(inputs);
        };
        EditorPartDelegate.prototype.openEditor = function (input, options, arg3) {
            return this.editorService.openEditor(input, options, arg3);
        };
        EditorPartDelegate.prototype.getActiveEditor = function () {
            return this.editorService.getActiveEditor();
        };
        EditorPartDelegate.prototype.activateEditor = function (editor) {
            this.editorService.activateEditor(editor);
        };
        EditorPartDelegate.prototype.getActiveEditorInput = function () {
            return this.editorService.getActiveEditorInput();
        };
        EditorPartDelegate.prototype.getVisibleEditors = function () {
            return this.editorService.getVisibleEditors();
        };
        EditorPartDelegate.prototype.moveEditor = function (from, to) {
            this.editorService.moveEditor(from, to);
        };
        EditorPartDelegate.prototype.arrangeEditors = function (arrangement) {
            this.editorService.arrangeEditors(arrangement);
        };
        EditorPartDelegate.prototype.closeEditors = function (othersOnly) {
            return this.editorService.closeEditors(othersOnly);
        };
        return EditorPartDelegate;
    }());
    /**
     * Subclass of workbench editor service that delegates all calls to the provided editor service. Subclasses can choose to override the behavior
     * of openEditor() by providing a handler. The handler returns a promise that resolves to true or false to indicate if an action has been taken.
     * If false is returned, the service will delegate to editor service for handling the call to openEditor().
     *
     * This gives clients a chance to override the behavior of openEditor() to match their context.
     */
    var DelegatingWorkbenchEditorService = (function (_super) {
        __extends(DelegatingWorkbenchEditorService, _super);
        function DelegatingWorkbenchEditorService(editor, handler, untitledEditorService, instantiationService, editorService) {
            _super.call(this, new EditorPartDelegate(editorService), untitledEditorService, instantiationService);
            this.editor = editor;
            this.handler = handler;
        }
        DelegatingWorkbenchEditorService.prototype.doOpenEditor = function (input, options, arg3) {
            var _this = this;
            return this.handler(this.editor, input, options, arg3).then(function (result) {
                if (result) {
                    return winjs_base_1.TPromise.as(_this.editor);
                }
                return _super.prototype.doOpenEditor.call(_this, input, options, arg3);
            });
        };
        DelegatingWorkbenchEditorService = __decorate([
            __param(2, untitledEditorService_1.IUntitledEditorService),
            __param(3, instantiation_1.IInstantiationService),
            __param(4, editorService_1.IWorkbenchEditorService)
        ], DelegatingWorkbenchEditorService);
        return DelegatingWorkbenchEditorService;
    }(WorkbenchEditorService));
    exports.DelegatingWorkbenchEditorService = DelegatingWorkbenchEditorService;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/services/search/node/searchService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/uri', 'vs/base/common/glob', 'vs/base/common/objects', 'vs/base/common/scorer', 'vs/base/common/strings', 'vs/base/node/service.cp', 'vs/platform/search/common/search', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/editor/common/services/modelService', 'vs/platform/workspace/common/workspace', 'vs/platform/configuration/common/configuration', 'vs/workbench/services/search/node/rawSearchService'], function (require, exports, winjs_base_1, uri_1, glob, objects, scorer, strings, service_cp_1, search_1, untitledEditorService_1, modelService_1, workspace_1, configuration_1, rawSearchService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var SearchService = (function () {
        function SearchService(modelService, untitledEditorService, contextService, configurationService) {
            this.modelService = modelService;
            this.untitledEditorService = untitledEditorService;
            this.contextService = contextService;
            this.configurationService = configurationService;
            this.serviceId = search_1.ISearchService;
            var config = contextService.getConfiguration();
            this.diskSearch = new DiskSearch(!config.env.isBuilt || config.env.verboseLogging);
        }
        SearchService.prototype.search = function (query) {
            var _this = this;
            return this.configurationService.loadConfiguration().then(function (configuration) {
                // Configuration: Encoding
                if (!query.fileEncoding) {
                    var fileEncoding = configuration && configuration.files && configuration.files.encoding;
                    query.fileEncoding = fileEncoding;
                }
                // Configuration: File Excludes
                var fileExcludes = configuration && configuration.files && configuration.files.exclude;
                if (fileExcludes) {
                    if (!query.excludePattern) {
                        query.excludePattern = fileExcludes;
                    }
                    else {
                        objects.mixin(query.excludePattern, fileExcludes, false /* no overwrite */);
                    }
                }
                var rawSearchQuery;
                return new winjs_base_1.PPromise(function (onComplete, onError, onProgress) {
                    // Get local results from dirty/untitled
                    var localResultsFlushed = false;
                    var localResults = _this.getLocalResults(query);
                    var flushLocalResultsOnce = function () {
                        if (!localResultsFlushed) {
                            localResultsFlushed = true;
                            Object.keys(localResults).map(function (key) { return localResults[key]; }).filter(function (res) { return !!res; }).forEach(onProgress);
                        }
                    };
                    // Delegate to parent for real file results
                    rawSearchQuery = _this.diskSearch.search(query).then(
                    // on Complete
                    function (complete) {
                        flushLocalResultsOnce();
                        onComplete({ results: complete.results.filter(function (match) { return typeof localResults[match.resource.toString()] === 'undefined'; }), limitHit: complete.limitHit }); // dont override local results
                    }, 
                    // on Error
                    function (error) {
                        flushLocalResultsOnce();
                        onError(error);
                    }, 
                    // on Progress
                    function (progress) {
                        flushLocalResultsOnce();
                        // Match
                        if (progress.resource) {
                            if (typeof localResults[progress.resource.toString()] === 'undefined') {
                                onProgress(progress);
                            }
                        }
                        else {
                            onProgress(progress);
                        }
                    });
                }, function () { return rawSearchQuery && rawSearchQuery.cancel(); });
            });
        };
        SearchService.prototype.getLocalResults = function (query) {
            var _this = this;
            var localResults = Object.create(null);
            if (query.type === search_1.QueryType.Text) {
                var models = this.modelService.getModels();
                models.forEach(function (model) {
                    var resource = model.getAssociatedResource();
                    if (!resource) {
                        return;
                    }
                    // Support untitled files
                    if (resource.scheme === 'untitled') {
                        if (!_this.untitledEditorService.get(resource)) {
                            return;
                        }
                    }
                    else if (resource.scheme !== 'file') {
                        return;
                    }
                    if (!_this.matches(resource, query.filePattern, query.includePattern, query.excludePattern)) {
                        return; // respect user filters
                    }
                    // Use editor API to find matches
                    var ranges = model.findMatches(query.contentPattern.pattern, false, query.contentPattern.isRegExp, query.contentPattern.isCaseSensitive, query.contentPattern.isWordMatch);
                    if (ranges.length) {
                        var fileMatch_1 = new search_1.FileMatch(resource);
                        localResults[resource.toString()] = fileMatch_1;
                        ranges.forEach(function (range) {
                            fileMatch_1.lineMatches.push(new search_1.LineMatch(model.getLineContent(range.startLineNumber), range.startLineNumber - 1, [[range.startColumn - 1, range.endColumn - range.startColumn]]));
                        });
                    }
                    else {
                        localResults[resource.toString()] = false; // flag as empty result
                    }
                });
            }
            return localResults;
        };
        SearchService.prototype.matches = function (resource, filePattern, includePattern, excludePattern) {
            var workspaceRelativePath = this.contextService.toWorkspaceRelativePath(resource);
            // file pattern
            if (filePattern) {
                if (resource.scheme !== 'file') {
                    return false; // if we match on file pattern, we have to ignore non file resources
                }
                if (!scorer.matches(resource.fsPath, strings.stripWildcards(filePattern).toLowerCase())) {
                    return false;
                }
            }
            // includes
            if (includePattern) {
                if (resource.scheme !== 'file') {
                    return false; // if we match on file patterns, we have to ignore non file resources
                }
                if (!glob.match(includePattern, workspaceRelativePath || resource.fsPath)) {
                    return false;
                }
            }
            // excludes
            if (excludePattern) {
                if (resource.scheme !== 'file') {
                    return true; // e.g. untitled files can never be excluded with file patterns
                }
                if (glob.match(excludePattern, workspaceRelativePath || resource.fsPath)) {
                    return false;
                }
            }
            return true;
        };
        SearchService = __decorate([
            __param(0, modelService_1.IModelService),
            __param(1, untitledEditorService_1.IUntitledEditorService),
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, configuration_1.IConfigurationService)
        ], SearchService);
        return SearchService;
    }());
    exports.SearchService = SearchService;
    var DiskSearch = (function () {
        function DiskSearch(verboseLogging) {
            var client = new service_cp_1.Client(uri_1.default.parse(require.toUrl('bootstrap')).fsPath, {
                serverName: 'Search',
                timeout: 60 * 1000,
                args: ['--type=searchService'],
                env: {
                    AMD_ENTRYPOINT: 'vs/workbench/services/search/node/searchApp',
                    PIPE_LOGGING: 'true',
                    VERBOSE_LOGGING: verboseLogging
                }
            });
            this.raw = client.getService('SearchService', rawSearchService_1.SearchService);
        }
        DiskSearch.prototype.search = function (query) {
            var result = [];
            var request;
            var rawSearch = {
                rootFolders: query.folderResources ? query.folderResources.map(function (r) { return r.fsPath; }) : [],
                extraFiles: query.extraFileResources ? query.extraFileResources.map(function (r) { return r.fsPath; }) : [],
                filePattern: query.filePattern,
                excludePattern: query.excludePattern,
                includePattern: query.includePattern,
                maxResults: query.maxResults
            };
            if (query.type === search_1.QueryType.Text) {
                rawSearch.contentPattern = query.contentPattern;
                rawSearch.fileEncoding = query.fileEncoding;
            }
            if (query.type === search_1.QueryType.File) {
                request = this.raw.fileSearch(rawSearch);
            }
            else {
                request = this.raw.textSearch(rawSearch);
            }
            return new winjs_base_1.PPromise(function (c, e, p) {
                request.done(function (complete) {
                    c({
                        limitHit: complete.limitHit,
                        results: result
                    });
                }, e, function (data) {
                    // Match
                    if (data.path) {
                        var fileMatch = new search_1.FileMatch(uri_1.default.file(data.path));
                        result.push(fileMatch);
                        if (data.lineMatches) {
                            for (var j = 0; j < data.lineMatches.length; j++) {
                                fileMatch.lineMatches.push(new search_1.LineMatch(data.lineMatches[j].preview, data.lineMatches[j].lineNumber, data.lineMatches[j].offsetAndLengths));
                            }
                        }
                        p(fileMatch);
                    }
                    else {
                        p(data);
                    }
                });
            }, function () { return request.cancel(); });
        };
        return DiskSearch;
    }());
});

define("vs/workbench/services/viewlet/common/viewletService", ["require", "exports", 'vs/platform/instantiation/common/instantiation'], function (require, exports, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IViewletService = instantiation_1.createDecorator('viewletService');
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/viewlet", ["require", "exports", 'vs/nls!vs/workbench/browser/viewlet', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/base/common/errors', 'vs/platform/platform', 'vs/base/browser/builder', 'vs/base/common/actions', 'vs/base/browser/ui/actionbar/actionbar', 'vs/workbench/browser/actionBarRegistry', 'vs/base/browser/ui/toolbar/toolbar', 'vs/base/browser/dnd', 'vs/base/common/lifecycle', 'vs/base/browser/ui/splitview/splitview', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/browser/composite', 'vs/platform/contextview/browser/contextView', 'vs/platform/message/common/message', 'vs/platform/selection/common/selection'], function (require, exports, nls, winjs_base_1, DOM, errors, platform_1, builder_1, actions_1, actionbar_1, actionBarRegistry_1, toolbar_1, dnd_1, lifecycle_1, splitview_1, viewletService_1, editorService_1, composite_1, contextView_1, message_1, selection_1) {
    "use strict";
    var Viewlet = (function (_super) {
        __extends(Viewlet, _super);
        function Viewlet() {
            _super.apply(this, arguments);
        }
        return Viewlet;
    }(composite_1.Composite));
    exports.Viewlet = Viewlet;
    /**
     * Helper subtype of viewlet for those that use a tree inside.
     */
    var ViewerViewlet = (function (_super) {
        __extends(ViewerViewlet, _super);
        function ViewerViewlet() {
            _super.apply(this, arguments);
        }
        ViewerViewlet.prototype.create = function (parent) {
            var _this = this;
            _super.prototype.create.call(this, parent);
            // Container for Viewer
            this.viewerContainer = parent.div();
            // Viewer
            this.viewer = this.createViewer(this.viewerContainer);
            // Eventing
            this.toUnbind.push(this.viewer.addListener('selection', function (e) { return _this.onSelection(e); }));
            this.toUnbind.push(this.viewer.addListener('focus', function (e) { return _this.onFocus(e); }));
            return winjs_base_1.TPromise.as(null);
        };
        /**
         * Returns the viewer that is contained in this viewlet.
         */
        ViewerViewlet.prototype.getViewer = function () {
            return this.viewer;
        };
        ViewerViewlet.prototype.setVisible = function (visible) {
            var promise;
            if (visible) {
                promise = _super.prototype.setVisible.call(this, visible);
                this.getViewer().onVisible();
            }
            else {
                this.getViewer().onHidden();
                promise = _super.prototype.setVisible.call(this, visible);
            }
            return promise;
        };
        ViewerViewlet.prototype.focus = function () {
            if (!this.viewer) {
                return; // return early if viewlet has not yet been created
            }
            // Make sure the current selected element is revealed
            var selection = this.viewer.getSelection();
            if (selection.length > 0) {
                this.reveal(selection[0], 0.5).done(null, errors.onUnexpectedError);
            }
            // Pass Focus to Viewer
            this.viewer.DOMFocus();
        };
        ViewerViewlet.prototype.reveal = function (element, relativeTop) {
            if (!this.viewer) {
                return winjs_base_1.TPromise.as(null); // return early if viewlet has not yet been created
            }
            // The viewer cannot properly reveal without being layed out, so force it if not yet done
            if (!this.wasLayouted) {
                this.viewer.layout();
            }
            // Now reveal
            return this.viewer.reveal(element, relativeTop);
        };
        ViewerViewlet.prototype.layout = function (dimension) {
            if (!this.viewer) {
                return; // return early if viewlet has not yet been created
            }
            // Pass on to Viewer
            this.wasLayouted = true;
            this.viewer.layout(dimension.height);
        };
        ViewerViewlet.prototype.getControl = function () {
            return this.viewer;
        };
        ViewerViewlet.prototype.getSelection = function () {
            if (!this.viewer) {
                return new selection_1.StructuredSelection([]); // return early if viewlet has not yet been created
            }
            return new selection_1.StructuredSelection(this.viewer.getSelection());
        };
        ViewerViewlet.prototype.dispose = function () {
            // Dispose Viewer
            if (this.viewer) {
                this.viewer.dispose();
            }
            _super.prototype.dispose.call(this);
        };
        return ViewerViewlet;
    }(Viewlet));
    exports.ViewerViewlet = ViewerViewlet;
    /**
     * A viewlet descriptor is a leightweight descriptor of a viewlet in the monaco workbench.
     */
    var ViewletDescriptor = (function (_super) {
        __extends(ViewletDescriptor, _super);
        function ViewletDescriptor() {
            _super.apply(this, arguments);
        }
        return ViewletDescriptor;
    }(composite_1.CompositeDescriptor));
    exports.ViewletDescriptor = ViewletDescriptor;
    exports.Extensions = {
        Viewlets: 'workbench.contributions.viewlets'
    };
    var ViewletRegistry = (function (_super) {
        __extends(ViewletRegistry, _super);
        function ViewletRegistry() {
            _super.apply(this, arguments);
        }
        /**
         * Registers a viewlet to the platform.
         */
        ViewletRegistry.prototype.registerViewlet = function (descriptor) {
            _super.prototype.registerComposite.call(this, descriptor);
        };
        /**
         * Returns the viewlet descriptor for the given id or null if none.
         */
        ViewletRegistry.prototype.getViewlet = function (id) {
            return this.getComposite(id);
        };
        /**
         * Returns an array of registered viewlets known to the platform.
         */
        ViewletRegistry.prototype.getViewlets = function () {
            return this.getComposits();
        };
        /**
         * Sets the id of the viewlet that should open on startup by default.
         */
        ViewletRegistry.prototype.setDefaultViewletId = function (id) {
            this.defaultViewletId = id;
        };
        /**
         * Gets the id of the viewlet that should open on startup by default.
         */
        ViewletRegistry.prototype.getDefaultViewletId = function () {
            return this.defaultViewletId;
        };
        return ViewletRegistry;
    }(composite_1.CompositeRegistry));
    exports.ViewletRegistry = ViewletRegistry;
    platform_1.Registry.add(exports.Extensions.Viewlets, new ViewletRegistry());
    /**
     * A reusable action to toggle a viewlet with a specific id.
     */
    var ToggleViewletAction = (function (_super) {
        __extends(ToggleViewletAction, _super);
        function ToggleViewletAction(id, name, viewletId, viewletService, editorService) {
            _super.call(this, id, name);
            this.viewletService = viewletService;
            this.editorService = editorService;
            this.viewletId = viewletId;
            this.enabled = !!this.viewletService && !!this.editorService;
        }
        ToggleViewletAction.prototype.run = function () {
            // Pass focus to viewlet if not open or focussed
            if (this.otherViewletShowing() || !this.sidebarHasFocus()) {
                return this.viewletService.openViewlet(this.viewletId, true);
            }
            // Otherwise pass focus to editor if possible
            var editor = this.editorService.getActiveEditor();
            if (editor) {
                editor.focus();
            }
            return winjs_base_1.TPromise.as(true);
        };
        ToggleViewletAction.prototype.otherViewletShowing = function () {
            var activeViewlet = this.viewletService.getActiveViewlet();
            return !activeViewlet || activeViewlet.getId() !== this.viewletId;
        };
        ToggleViewletAction.prototype.sidebarHasFocus = function () {
            var activeViewlet = this.viewletService.getActiveViewlet();
            var activeElement = document.activeElement;
            return activeViewlet && activeElement && DOM.isAncestor(activeElement, activeViewlet.getContainer().getHTMLElement());
        };
        ToggleViewletAction = __decorate([
            __param(3, viewletService_1.IViewletService),
            __param(4, editorService_1.IWorkbenchEditorService)
        ], ToggleViewletAction);
        return ToggleViewletAction;
    }(actions_1.Action));
    exports.ToggleViewletAction = ToggleViewletAction;
    // Collapse All action
    var CollapseAction = (function (_super) {
        __extends(CollapseAction, _super);
        function CollapseAction(viewer, enabled, clazz) {
            _super.call(this, 'workbench.action.collapse', nls.localize(0, null), clazz, enabled, function (context) {
                if (viewer.getHighlight()) {
                    return winjs_base_1.TPromise.as(null); // Global action disabled if user is in edit mode from another action
                }
                viewer.collapseAll();
                viewer.clearSelection();
                viewer.clearFocus();
                viewer.DOMFocus();
                viewer.focusFirst();
                return winjs_base_1.TPromise.as(null);
            });
        }
        return CollapseAction;
    }(actions_1.Action));
    exports.CollapseAction = CollapseAction;
    /**
     * The AdaptiveCollapsibleViewletView can grow with the content inside dynamically.
     */
    var AdaptiveCollapsibleViewletView = (function (_super) {
        __extends(AdaptiveCollapsibleViewletView, _super);
        function AdaptiveCollapsibleViewletView(actionRunner, initialBodySize, collapsed, viewName, messageService, contextMenuService) {
            _super.call(this, {
                expandedBodySize: initialBodySize,
                headerSize: 22,
                initialState: collapsed ? splitview_1.CollapsibleState.COLLAPSED : splitview_1.CollapsibleState.EXPANDED,
                ariaHeaderLabel: viewName
            });
            this.viewName = viewName;
            this.messageService = messageService;
            this.contextMenuService = contextMenuService;
            this.actionRunner = actionRunner;
            this.toDispose = [];
        }
        AdaptiveCollapsibleViewletView.prototype.create = function () {
            return winjs_base_1.TPromise.as(null);
        };
        AdaptiveCollapsibleViewletView.prototype.renderHeader = function (container) {
            var _this = this;
            // Tool bar
            this.toolBar = new toolbar_1.ToolBar(builder_1.$('div.actions').appendTo(container).getHTMLElement(), this.contextMenuService, {
                orientation: actionbar_1.ActionsOrientation.HORIZONTAL,
                actionItemProvider: function (action) { return _this.getActionItem(action); },
                ariaLabel: nls.localize(1, null, this.viewName)
            });
            this.toolBar.actionRunner = this.actionRunner;
            this.toolBar.setActions(actionBarRegistry_1.prepareActions(this.getActions()), actionBarRegistry_1.prepareActions(this.getSecondaryActions()))();
            // Expand on drag over
            this.dragHandler = new dnd_1.DelayedDragHandler(container, function () {
                if (!_this.isExpanded()) {
                    _this.expand();
                }
            });
        };
        AdaptiveCollapsibleViewletView.prototype.changeState = function (state) {
            updateTreeVisibility(this.tree, state === splitview_1.CollapsibleState.EXPANDED);
            _super.prototype.changeState.call(this, state);
        };
        AdaptiveCollapsibleViewletView.prototype.renderViewTree = function (container) {
            return renderViewTree(container);
        };
        AdaptiveCollapsibleViewletView.prototype.getViewer = function () {
            return this.tree;
        };
        AdaptiveCollapsibleViewletView.prototype.refresh = function (focus, reveal, instantProgress) {
            return winjs_base_1.TPromise.as(null);
        };
        AdaptiveCollapsibleViewletView.prototype.setVisible = function (visible) {
            this.isVisible = visible;
            updateTreeVisibility(this.tree, this.state === splitview_1.CollapsibleState.EXPANDED);
            return winjs_base_1.TPromise.as(null);
        };
        AdaptiveCollapsibleViewletView.prototype.focusBody = function () {
            focus(this.tree);
        };
        AdaptiveCollapsibleViewletView.prototype.getSelection = function () {
            return new selection_1.StructuredSelection(this.tree.getSelection());
        };
        AdaptiveCollapsibleViewletView.prototype.reveal = function (element, relativeTop) {
            return reveal(this.tree, element, relativeTop);
        };
        AdaptiveCollapsibleViewletView.prototype.layoutBody = function (size) {
            this.treeContainer.style.height = size + 'px';
            this.tree.layout(size);
        };
        AdaptiveCollapsibleViewletView.prototype.getActions = function () {
            return [];
        };
        AdaptiveCollapsibleViewletView.prototype.getSecondaryActions = function () {
            return [];
        };
        AdaptiveCollapsibleViewletView.prototype.getActionItem = function (action) {
            return null;
        };
        AdaptiveCollapsibleViewletView.prototype.shutdown = function () {
            // Subclass to implement
        };
        AdaptiveCollapsibleViewletView.prototype.dispose = function () {
            this.isDisposed = true;
            this.treeContainer = null;
            this.tree.dispose();
            this.dragHandler.dispose();
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            if (this.toolBar) {
                this.toolBar.dispose();
            }
            _super.prototype.dispose.call(this);
        };
        AdaptiveCollapsibleViewletView = __decorate([
            __param(4, message_1.IMessageService),
            __param(5, contextView_1.IContextMenuService)
        ], AdaptiveCollapsibleViewletView);
        return AdaptiveCollapsibleViewletView;
    }(splitview_1.FixedCollapsibleView));
    exports.AdaptiveCollapsibleViewletView = AdaptiveCollapsibleViewletView;
    var CollapsibleViewletView = (function (_super) {
        __extends(CollapsibleViewletView, _super);
        function CollapsibleViewletView(actionRunner, collapsed, viewName, messageService, contextMenuService) {
            _super.call(this, {
                minimumSize: 2 * 22,
                initialState: collapsed ? splitview_1.CollapsibleState.COLLAPSED : splitview_1.CollapsibleState.EXPANDED,
                ariaHeaderLabel: viewName
            });
            this.viewName = viewName;
            this.messageService = messageService;
            this.contextMenuService = contextMenuService;
            this.actionRunner = actionRunner;
            this.toDispose = [];
        }
        CollapsibleViewletView.prototype.changeState = function (state) {
            updateTreeVisibility(this.tree, state === splitview_1.CollapsibleState.EXPANDED);
            _super.prototype.changeState.call(this, state);
        };
        CollapsibleViewletView.prototype.create = function () {
            return winjs_base_1.TPromise.as(null);
        };
        CollapsibleViewletView.prototype.renderHeader = function (container) {
            var _this = this;
            // Tool bar
            this.toolBar = new toolbar_1.ToolBar(builder_1.$('div.actions').appendTo(container).getHTMLElement(), this.contextMenuService, {
                orientation: actionbar_1.ActionsOrientation.HORIZONTAL,
                actionItemProvider: function (action) { return _this.getActionItem(action); },
                ariaLabel: nls.localize(2, null, this.viewName)
            });
            this.toolBar.actionRunner = this.actionRunner;
            this.toolBar.setActions(actionBarRegistry_1.prepareActions(this.getActions()), actionBarRegistry_1.prepareActions(this.getSecondaryActions()))();
            // Expand on drag over
            this.dragHandler = new dnd_1.DelayedDragHandler(container, function () {
                if (!_this.isExpanded()) {
                    _this.expand();
                }
            });
        };
        CollapsibleViewletView.prototype.renderViewTree = function (container) {
            return renderViewTree(container);
        };
        CollapsibleViewletView.prototype.getViewer = function () {
            return this.tree;
        };
        CollapsibleViewletView.prototype.refresh = function (focus, reveal, instantProgress) {
            return winjs_base_1.TPromise.as(null);
        };
        CollapsibleViewletView.prototype.setVisible = function (visible) {
            this.isVisible = visible;
            updateTreeVisibility(this.tree, this.state === splitview_1.CollapsibleState.EXPANDED);
            return winjs_base_1.TPromise.as(null);
        };
        CollapsibleViewletView.prototype.focusBody = function () {
            focus(this.tree);
        };
        CollapsibleViewletView.prototype.getSelection = function () {
            return new selection_1.StructuredSelection(this.tree.getSelection());
        };
        CollapsibleViewletView.prototype.reveal = function (element, relativeTop) {
            return reveal(this.tree, element, relativeTop);
        };
        CollapsibleViewletView.prototype.layoutBody = function (size) {
            this.treeContainer.style.height = size + 'px';
            this.tree.layout(size);
        };
        CollapsibleViewletView.prototype.getActions = function () {
            return [];
        };
        CollapsibleViewletView.prototype.getSecondaryActions = function () {
            return [];
        };
        CollapsibleViewletView.prototype.getActionItem = function (action) {
            return null;
        };
        CollapsibleViewletView.prototype.shutdown = function () {
            // Subclass to implement
        };
        CollapsibleViewletView.prototype.dispose = function () {
            this.isDisposed = true;
            this.treeContainer = null;
            this.tree.dispose();
            this.dragHandler.dispose();
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            if (this.toolBar) {
                this.toolBar.dispose();
            }
            _super.prototype.dispose.call(this);
        };
        CollapsibleViewletView = __decorate([
            __param(3, message_1.IMessageService),
            __param(4, contextView_1.IContextMenuService)
        ], CollapsibleViewletView);
        return CollapsibleViewletView;
    }(splitview_1.CollapsibleView));
    exports.CollapsibleViewletView = CollapsibleViewletView;
    function renderViewTree(container) {
        var treeContainer = document.createElement('div');
        container.appendChild(treeContainer);
        return treeContainer;
    }
    function updateTreeVisibility(tree, isVisible) {
        if (!tree) {
            return;
        }
        if (isVisible) {
            builder_1.$(tree.getHTMLElement()).show();
        }
        else {
            builder_1.$(tree.getHTMLElement()).hide(); // make sure the tree goes out of the tabindex world by hiding it
        }
        if (isVisible) {
            tree.onVisible();
        }
        else {
            tree.onHidden();
        }
    }
    function focus(tree) {
        if (!tree) {
            return; // return early if viewlet has not yet been created
        }
        // Make sure the current selected element is revealed
        var selection = tree.getSelection();
        if (selection.length > 0) {
            reveal(tree, selection[0], 0.5);
        }
        // Pass Focus to Viewer
        tree.DOMFocus();
    }
    function reveal(tree, element, relativeTop) {
        if (!tree) {
            return winjs_base_1.TPromise.as(null); // return early if viewlet has not yet been created
        }
        return tree.reveal(element, relativeTop);
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/activitybar/activitybarPart", ["require", "exports", 'vs/nls!vs/workbench/browser/parts/activitybar/activitybarPart', 'vs/base/common/winjs.base', 'vs/base/browser/builder', 'vs/base/common/errors', 'vs/base/browser/ui/actionbar/actionbar', 'vs/platform/platform', 'vs/workbench/common/events', 'vs/workbench/browser/viewlet', 'vs/workbench/browser/part', 'vs/workbench/browser/parts/activitybar/activityAction', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/activity/common/activityService', 'vs/workbench/services/part/common/partService', 'vs/css!./media/activityBarPart'], function (require, exports, nls, winjs_base_1, builder_1, errors, actionbar_1, platform_1, events_1, viewlet_1, part_1, activityAction_1, viewletService_1, activityService_1, partService_1) {
    'use strict';
    var ActivitybarPart = (function (_super) {
        __extends(ActivitybarPart, _super);
        function ActivitybarPart(viewletService, messageService, telemetryService, eventService, contextMenuService, keybindingService, id) {
            _super.call(this, id);
            this.viewletService = viewletService;
            this.messageService = messageService;
            this.telemetryService = telemetryService;
            this.eventService = eventService;
            this.contextMenuService = contextMenuService;
            this.keybindingService = keybindingService;
            this.serviceId = activityService_1.IActivityService;
            this.activityActionItems = {};
            this.viewletIdToActions = {};
            this.registerListeners();
        }
        ActivitybarPart.prototype.setInstantiationService = function (service) {
            this.instantiationService = service;
        };
        ActivitybarPart.prototype.registerListeners = function () {
            var _this = this;
            // Activate viewlet action on opening of a viewlet
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.COMPOSITE_OPENING, function (e) { return _this.onCompositeOpening(e); }));
            // Deactivate viewlet action on close
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.COMPOSITE_CLOSED, function (e) { return _this.onCompositeClosed(e); }));
        };
        ActivitybarPart.prototype.onCompositeOpening = function (e) {
            if (this.viewletIdToActions[e.compositeId]) {
                this.viewletIdToActions[e.compositeId].activate();
                // There can only be one active viewlet action
                for (var key in this.viewletIdToActions) {
                    if (this.viewletIdToActions.hasOwnProperty(key) && key !== e.compositeId) {
                        this.viewletIdToActions[key].deactivate();
                    }
                }
            }
        };
        ActivitybarPart.prototype.onCompositeClosed = function (e) {
            if (this.viewletIdToActions[e.compositeId]) {
                this.viewletIdToActions[e.compositeId].deactivate();
            }
        };
        ActivitybarPart.prototype.showActivity = function (viewletId, badge, clazz) {
            var action = this.viewletIdToActions[viewletId];
            if (action) {
                action.setBadge(badge);
                if (clazz) {
                    action.class = clazz;
                }
            }
        };
        ActivitybarPart.prototype.clearActivity = function (viewletId) {
            this.showActivity(viewletId, null);
        };
        ActivitybarPart.prototype.createContentArea = function (parent) {
            var $el = builder_1.$(parent);
            var $result = builder_1.$('.content').appendTo($el);
            // Top Actionbar with action items for each viewlet action
            this.createViewletSwitcher($result.clone());
            // Bottom Toolbar with action items for global actions
            // this.createGlobalToolBarArea($result.clone()); // not used currently
            return $result;
        };
        ActivitybarPart.prototype.createViewletSwitcher = function (div) {
            var _this = this;
            // Viewlet switcher is on top
            this.viewletSwitcherBar = new actionbar_1.ActionBar(div, {
                actionItemProvider: function (action) { return _this.activityActionItems[action.id]; },
                orientation: actionbar_1.ActionsOrientation.VERTICAL,
                ariaLabel: nls.localize(0, null)
            });
            this.viewletSwitcherBar.getContainer().addClass('position-top');
            // Build Viewlet Actions in correct order
            var activeViewlet = this.viewletService.getActiveViewlet();
            var registry = platform_1.Registry.as(viewlet_1.Extensions.Viewlets);
            var viewletActions = registry.getViewlets()
                .sort(function (v1, v2) { return v1.order - v2.order; })
                .map(function (viewlet) {
                var action = _this.instantiationService.createInstance(ViewletActivityAction, viewlet.id + '.activity-bar-action', viewlet);
                var keybinding = null;
                var keys = _this.keybindingService.lookupKeybindings(viewlet.id).map(function (k) { return _this.keybindingService.getLabelFor(k); });
                if (keys && keys.length) {
                    keybinding = keys[0];
                }
                _this.activityActionItems[action.id] = new activityAction_1.ActivityActionItem(action, viewlet.name, keybinding);
                _this.viewletIdToActions[viewlet.id] = action;
                // Mark active viewlet action as active
                if (activeViewlet && activeViewlet.getId() === viewlet.id) {
                    action.activate();
                }
                return action;
            });
            // Add to viewlet switcher
            this.viewletSwitcherBar.push(viewletActions, { label: true, icon: true });
        };
        // private createGlobalToolBarArea(div: Builder): void {
        // 	// Global action bar is on the bottom
        // 	this.globalToolBar = new ToolBar(div.getHTMLElement(), this.contextMenuService, {
        // 		actionItemProvider: (action: Action) => this.activityActionItems[action.id],
        // 		orientation: ActionsOrientation.VERTICAL
        // 	});
        // 	this.globalToolBar.getContainer().addClass('global');
        // 	this.globalToolBar.actionRunner.addListener(events.EventType.RUN, (e: any) => {
        // 		// Check for Error
        // 		if (e.error && !errors.isPromiseCanceledError(e.error)) {
        // 			this.messageService.show(Severity.Error, e.error);
        // 		}
        // 		// Log in telemetry
        // 		if (this.telemetryService) {
        // 			this.telemetryService.publicLog('workbenchActionExecuted', { id: e.action.id, from: 'activityBar' });
        // 		}
        // 	});
        // 	// Build Global Actions in correct order
        // 	let primaryActions = this.getGlobalActions(true);
        // 	let secondaryActions = this.getGlobalActions(false);
        // 	if (primaryActions.length + secondaryActions.length > 0) {
        // 		this.globalToolBar.getContainer().addClass('position-bottom');
        // 	}
        // 	// Add to global action bar
        // 	this.globalToolBar.setActions(prepareActions(primaryActions), prepareActions(secondaryActions))();
        // }
        // private getGlobalActions(primary: boolean): IAction[] {
        // 	let actionBarRegistry = <IActionBarRegistry>Registry.as(ActionBarExtensions.Actionbar);
        // 	// Collect actions from actionbar contributor
        // 	let actions: IAction[];
        // 	if (primary) {
        // 		actions = actionBarRegistry.getActionBarActionsForContext(Scope.GLOBAL, CONTEXT);
        // 	} else {
        // 		actions = actionBarRegistry.getSecondaryActionBarActionsForContext(Scope.GLOBAL, CONTEXT);
        // 	}
        // 	return actions.map((action: Action) => {
        // 		if (primary) {
        // 			let keybinding: string = null;
        // 			let keys = this.keybindingService.lookupKeybindings(action.id).map(k => this.keybindingService.getLabelFor(k));
        // 			if (keys && keys.length) {
        // 				keybinding = keys[0];
        // 			}
        // 			let actionItem = actionBarRegistry.getActionItemForContext(Scope.GLOBAL, CONTEXT, action);
        // 			if (!actionItem) {
        // 				actionItem = new ActivityActionItem(action, action.label, keybinding);
        // 			}
        // 			if (actionItem instanceof ActivityActionItem) {
        // 				(<ActivityActionItem> actionItem).keybinding = keybinding;
        // 			}
        // 			this.activityActionItems[action.id] = actionItem;
        // 		}
        // 		return action;
        // 	});
        // }
        ActivitybarPart.prototype.dispose = function () {
            if (this.viewletSwitcherBar) {
                this.viewletSwitcherBar.dispose();
                this.viewletSwitcherBar = null;
            }
            if (this.globalToolBar) {
                this.globalToolBar.dispose();
                this.globalToolBar = null;
            }
            _super.prototype.dispose.call(this);
        };
        return ActivitybarPart;
    }(part_1.Part));
    exports.ActivitybarPart = ActivitybarPart;
    var ViewletActivityAction = (function (_super) {
        __extends(ViewletActivityAction, _super);
        function ViewletActivityAction(id, viewlet, viewletService, partService) {
            _super.call(this, id, viewlet.name, viewlet.cssClass);
            this.viewletService = viewletService;
            this.partService = partService;
            this.viewlet = viewlet;
        }
        ViewletActivityAction.prototype.run = function () {
            // cheap trick to prevent accident trigger on a doubleclick (to help nervous people)
            var now = new Date().getTime();
            if (now - ViewletActivityAction.lastRun < ViewletActivityAction.preventDoubleClickDelay) {
                return winjs_base_1.TPromise.as(true);
            }
            ViewletActivityAction.lastRun = now;
            var sideBarHidden = this.partService.isSideBarHidden();
            var activeViewlet = this.viewletService.getActiveViewlet();
            // Hide sidebar if selected viewlet already visible
            if (!sideBarHidden && activeViewlet && activeViewlet.getId() === this.viewlet.id) {
                this.partService.setSideBarHidden(true);
            }
            else {
                this.viewletService.openViewlet(this.viewlet.id, true).done(null, errors.onUnexpectedError);
                this.activate();
            }
            return winjs_base_1.TPromise.as(true);
        };
        ViewletActivityAction.preventDoubleClickDelay = 300;
        ViewletActivityAction.lastRun = 0;
        ViewletActivityAction = __decorate([
            __param(2, viewletService_1.IViewletService),
            __param(3, partService_1.IPartService)
        ], ViewletActivityAction);
        return ViewletActivityAction;
    }(activityAction_1.ActivityAction));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/sidebar/sidebarPart", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/browser/parts/sidebar/sidebarPart', 'vs/platform/platform', 'vs/base/common/actions', 'vs/workbench/browser/parts/compositePart', 'vs/workbench/browser/viewlet', 'vs/workbench/common/actionRegistry', 'vs/platform/actions/common/actions', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/part/common/partService', 'vs/workbench/browser/actionBarRegistry', 'vs/base/common/keyCodes', 'vs/css!./media/sidebarpart'], function (require, exports, winjs_base_1, nls, platform_1, actions_1, compositePart_1, viewlet_1, actionRegistry_1, actions_2, viewletService_1, partService_1, actionBarRegistry_1, keyCodes_1) {
    "use strict";
    var SidebarPart = (function (_super) {
        __extends(SidebarPart, _super);
        function SidebarPart(messageService, storageService, eventService, telemetryService, contextMenuService, partService, keybindingService, id) {
            _super.call(this, messageService, storageService, eventService, telemetryService, contextMenuService, partService, keybindingService, platform_1.Registry.as(viewlet_1.Extensions.Viewlets), SidebarPart.activeViewletSettingsKey, 'sideBar', 'viewlet', actionBarRegistry_1.Scope.VIEWLET, id);
            this.serviceId = viewletService_1.IViewletService;
        }
        SidebarPart.prototype.openViewlet = function (id, focus) {
            if (this.blockOpeningViewlet) {
                return winjs_base_1.TPromise.as(null); // Workaround against a potential race condition
            }
            // First check if sidebar is hidden and show if so
            if (this.partService.isSideBarHidden()) {
                try {
                    this.blockOpeningViewlet = true;
                    this.partService.setSideBarHidden(false);
                }
                finally {
                    this.blockOpeningViewlet = false;
                }
            }
            return this.openComposite(id, focus);
        };
        SidebarPart.prototype.getActiveViewlet = function () {
            return this.getActiveComposite();
        };
        SidebarPart.prototype.getLastActiveViewletId = function () {
            return this.getLastActiveCompositetId();
        };
        SidebarPart.prototype.hideActiveViewlet = function () {
            return this.hideActiveComposite();
        };
        SidebarPart.activeViewletSettingsKey = 'workbench.sidebar.activeviewletid';
        return SidebarPart;
    }(compositePart_1.CompositePart));
    exports.SidebarPart = SidebarPart;
    var FocusSideBarAction = (function (_super) {
        __extends(FocusSideBarAction, _super);
        function FocusSideBarAction(id, label, viewletService, partService) {
            _super.call(this, id, label);
            this.viewletService = viewletService;
            this.partService = partService;
        }
        FocusSideBarAction.prototype.run = function () {
            // Show side bar
            if (this.partService.isSideBarHidden()) {
                this.partService.setSideBarHidden(false);
            }
            else {
                var viewlet = this.viewletService.getActiveViewlet();
                if (viewlet) {
                    viewlet.focus();
                }
            }
            return winjs_base_1.TPromise.as(true);
        };
        FocusSideBarAction.ID = 'workbench.action.focusSideBar';
        FocusSideBarAction.LABEL = nls.localize(0, null);
        FocusSideBarAction = __decorate([
            __param(2, viewletService_1.IViewletService),
            __param(3, partService_1.IPartService)
        ], FocusSideBarAction);
        return FocusSideBarAction;
    }(actions_1.Action));
    exports.FocusSideBarAction = FocusSideBarAction;
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(FocusSideBarAction, FocusSideBarAction.ID, FocusSideBarAction.LABEL, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_0
    }), nls.localize(1, null));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/electron-browser/window", ["require", "exports", 'vs/base/common/platform', 'vs/base/common/paths', 'vs/base/common/uri', 'vs/workbench/common/constants', 'vs/workbench/common/events', 'vs/workbench/common/editor', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/editor/common/editorService', 'vs/base/browser/dom', 'vs/platform/storage/common/storage', 'vs/platform/event/common/event', 'vs/platform/workspace/common/workspace', 'electron'], function (require, exports, platform, paths, uri_1, constants_1, events_1, workbenchEditorCommon, viewletService_1, editorService_1, dom, storage_1, event_1, workspace_1, electron_1) {
    'use strict';
    var dialog = electron_1.remote.dialog;
    var ElectronWindow = (function () {
        function ElectronWindow(win, shellContainer, contextService, eventService, storageService, editorService, viewletService) {
            this.contextService = contextService;
            this.eventService = eventService;
            this.storageService = storageService;
            this.editorService = editorService;
            this.viewletService = viewletService;
            this.win = win;
            this.windowId = win.id;
            this.registerListeners();
        }
        ElectronWindow.prototype.registerListeners = function () {
            var _this = this;
            // React to editor input changes (Mac only)
            if (platform.platform === platform.Platform.Mac) {
                this.eventService.addListener(events_1.EventType.EDITOR_INPUT_CHANGED, function (e) {
                    var fileInput = workbenchEditorCommon.asFileEditorInput(e.editorInput, true);
                    var representedFilename = '';
                    if (fileInput) {
                        representedFilename = fileInput.getResource().fsPath;
                    }
                    electron_1.ipcRenderer.send('vscode:setRepresentedFilename', _this.windowId, representedFilename);
                });
            }
            // Prevent a dropped file from opening as nw application
            window.document.body.addEventListener('dragover', function (e) {
                e.preventDefault();
            });
            // Let a dropped file open inside Monaco (only if dropped over editor area)
            window.document.body.addEventListener('drop', function (e) {
                e.preventDefault();
                var editorArea = window.document.getElementById(constants_1.Identifiers.EDITOR_PART);
                if (dom.isAncestor(e.toElement, editorArea)) {
                    var pathsOpened = false;
                    // Check for native file transfer
                    if (e.dataTransfer && e.dataTransfer.files) {
                        var thepaths = [];
                        for (var i = 0; i < e.dataTransfer.files.length; i++) {
                            if (e.dataTransfer.files[i] && e.dataTransfer.files[i].path) {
                                thepaths.push(e.dataTransfer.files[i].path);
                            }
                        }
                        if (thepaths.length) {
                            pathsOpened = true;
                            _this.focus(); // make sure this window has focus so that the open call reaches the right window!
                            _this.open(thepaths);
                        }
                    }
                    // Otherwise check for special webkit transfer
                    if (!pathsOpened && e.dataTransfer && e.dataTransfer.items) {
                        var items = e.dataTransfer.items;
                        if (items.length && typeof items[0].getAsString === 'function') {
                            items[0].getAsString(function (str) {
                                try {
                                    var resource = uri_1.default.parse(str);
                                    if (resource.scheme === 'file') {
                                        // Do not allow to drop a child of the currently active workspace. This prevents an issue
                                        // where one would drop a folder from the explorer by accident into the editor area and
                                        // loose all the context.
                                        var workspace = _this.contextService.getWorkspace();
                                        if (workspace && paths.isEqualOrParent(resource.fsPath, workspace.resource.fsPath)) {
                                            return;
                                        }
                                        _this.focus(); // make sure this window has focus so that the open call reaches the right window!
                                        _this.open([decodeURIComponent(resource.fsPath)]);
                                    }
                                }
                                catch (error) {
                                }
                            });
                        }
                    }
                }
            });
            // Handle window.open() calls
            window.open = function (url, target, features, replace) {
                electron_1.shell.openExternal(url);
                return null;
            };
        };
        ElectronWindow.prototype.open = function (arg1) {
            var pathsToOpen;
            if (Array.isArray(arg1)) {
                pathsToOpen = arg1;
            }
            else if (typeof arg1 === 'string') {
                pathsToOpen = [arg1];
            }
            else {
                pathsToOpen = [arg1.fsPath];
            }
            electron_1.ipcRenderer.send('vscode:windowOpen', pathsToOpen); // handled from browser process
        };
        ElectronWindow.prototype.openNew = function () {
            electron_1.ipcRenderer.send('vscode:openNewWindow'); // handled from browser process
        };
        ElectronWindow.prototype.close = function () {
            this.win.close();
        };
        ElectronWindow.prototype.reload = function () {
            electron_1.ipcRenderer.send('vscode:reloadWindow', this.windowId);
        };
        ElectronWindow.prototype.showMessageBox = function (options) {
            return dialog.showMessageBox(this.win, options);
        };
        ElectronWindow.prototype.showSaveDialog = function (options, callback) {
            if (callback) {
                return dialog.showSaveDialog(this.win, options, callback);
            }
            return dialog.showSaveDialog(this.win, options);
        };
        ElectronWindow.prototype.setFullScreen = function (fullscreen) {
            electron_1.ipcRenderer.send('vscode:setFullScreen', this.windowId, fullscreen); // handled from browser process
        };
        ElectronWindow.prototype.openDevTools = function () {
            electron_1.ipcRenderer.send('vscode:openDevTools', this.windowId); // handled from browser process
        };
        ElectronWindow.prototype.setMenuBarVisibility = function (visible) {
            electron_1.ipcRenderer.send('vscode:setMenuBarVisibility', this.windowId, visible); // handled from browser process
        };
        ElectronWindow.prototype.focus = function () {
            electron_1.ipcRenderer.send('vscode:focusWindow', this.windowId); // handled from browser process
        };
        ElectronWindow.prototype.flashFrame = function () {
            electron_1.ipcRenderer.send('vscode:flashFrame', this.windowId); // handled from browser process
        };
        ElectronWindow = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, event_1.IEventService),
            __param(4, storage_1.IStorageService),
            __param(5, editorService_1.IWorkbenchEditorService),
            __param(6, viewletService_1.IViewletService)
        ], ElectronWindow);
        return ElectronWindow;
    }());
    exports.ElectronWindow = ElectronWindow;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/debug/electron-browser/debugActions", ["require", "exports", 'vs/nls!vs/workbench/parts/debug/electron-browser/debugActions', 'vs/base/common/actions', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/editor/common/core/range', 'vs/editor/common/editorAction', 'vs/editor/common/editorActionEnablement', 'vs/platform/event/common/event', 'vs/platform/keybinding/common/keybindingService', 'vs/workbench/common/events', 'vs/workbench/parts/debug/common/debug', 'vs/workbench/parts/debug/common/debugModel', 'vs/workbench/services/part/common/partService', 'vs/workbench/services/panel/common/panelService', 'vs/workbench/services/viewlet/common/viewletService', 'electron'], function (require, exports, nls, actions, lifecycle, winjs_base_1, range_1, editorAction_1, editorActionEnablement_1, event_1, keybindingService_1, events_1, debug, model, partService_1, panelService_1, viewletService_1, electron_1) {
    "use strict";
    var IDebugService = debug.IDebugService;
    var AbstractDebugAction = (function (_super) {
        __extends(AbstractDebugAction, _super);
        function AbstractDebugAction(id, label, cssClass, debugService, keybindingService) {
            var _this = this;
            _super.call(this, id, label, cssClass, false);
            this.debugService = debugService;
            this.keybindingService = keybindingService;
            this.toDispose = [];
            this.toDispose.push(this.debugService.addListener2(debug.ServiceEvents.STATE_CHANGED, function () { return _this.updateEnablement(); }));
            var keys = this.keybindingService.lookupKeybindings(id).map(function (k) { return _this.keybindingService.getLabelFor(k); });
            if (keys && keys.length) {
                this.keybinding = keys[0];
            }
            this.updateLabel(label);
            this.updateEnablement();
        }
        AbstractDebugAction.prototype.run = function (e) {
            throw new Error('implement me');
        };
        AbstractDebugAction.prototype.updateLabel = function (newLabel) {
            if (this.keybinding) {
                this.label = nls.localize(0, null, newLabel, this.keybinding);
            }
            else {
                this.label = newLabel;
            }
        };
        AbstractDebugAction.prototype.updateEnablement = function () {
            this.enabled = this.isEnabled();
        };
        AbstractDebugAction.prototype.isEnabled = function () {
            return this.debugService.getState() !== debug.State.Disabled;
        };
        AbstractDebugAction.prototype.dispose = function () {
            this.debugService = null;
            this.toDispose = lifecycle.disposeAll(this.toDispose);
            _super.prototype.dispose.call(this);
        };
        AbstractDebugAction = __decorate([
            __param(3, IDebugService),
            __param(4, keybindingService_1.IKeybindingService)
        ], AbstractDebugAction);
        return AbstractDebugAction;
    }(actions.Action));
    exports.AbstractDebugAction = AbstractDebugAction;
    var ConfigureAction = (function (_super) {
        __extends(ConfigureAction, _super);
        function ConfigureAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action configure', debugService, keybindingService);
        }
        ConfigureAction.prototype.run = function (event) {
            var sideBySide = !!(event && (event.ctrlKey || event.metaKey));
            return this.debugService.openConfigFile(sideBySide);
        };
        ConfigureAction.ID = 'workbench.action.debug.configure';
        ConfigureAction.LABEL = nls.localize(1, null, 'launch.json');
        ConfigureAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], ConfigureAction);
        return ConfigureAction;
    }(AbstractDebugAction));
    exports.ConfigureAction = ConfigureAction;
    var SelectConfigAction = (function (_super) {
        __extends(SelectConfigAction, _super);
        function SelectConfigAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action select-active-config', debugService, keybindingService);
        }
        SelectConfigAction.prototype.run = function (configName) {
            return this.debugService.setConfiguration(configName);
        };
        SelectConfigAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() === debug.State.Inactive;
        };
        SelectConfigAction.ID = 'workbench.debug.action.setActiveConfig';
        SelectConfigAction.LABEL = nls.localize(2, null);
        SelectConfigAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], SelectConfigAction);
        return SelectConfigAction;
    }(AbstractDebugAction));
    exports.SelectConfigAction = SelectConfigAction;
    var StartDebugAction = (function (_super) {
        __extends(StartDebugAction, _super);
        function StartDebugAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action start', debugService, keybindingService);
            this.updateEnablement();
        }
        StartDebugAction.prototype.run = function () {
            return this.debugService.createSession(false);
        };
        StartDebugAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() === debug.State.Inactive;
        };
        StartDebugAction.ID = 'workbench.action.debug.start';
        StartDebugAction.LABEL = nls.localize(3, null);
        StartDebugAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], StartDebugAction);
        return StartDebugAction;
    }(AbstractDebugAction));
    exports.StartDebugAction = StartDebugAction;
    var RestartDebugAction = (function (_super) {
        __extends(RestartDebugAction, _super);
        function RestartDebugAction(id, label, debugService, keybindingService) {
            var _this = this;
            _super.call(this, id, label, 'debug-action restart', debugService, keybindingService);
            this.updateEnablement();
            this.toDispose.push(this.debugService.addListener2(debug.ServiceEvents.STATE_CHANGED, function () {
                var session = _this.debugService.getActiveSession();
                if (session) {
                    _this.updateLabel(session.isAttach ? RestartDebugAction.RECONNECT_LABEL : RestartDebugAction.LABEL);
                }
            }));
        }
        RestartDebugAction.prototype.run = function () {
            return this.debugService.restartSession();
        };
        RestartDebugAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() !== debug.State.Inactive;
        };
        RestartDebugAction.ID = 'workbench.action.debug.restart';
        RestartDebugAction.LABEL = nls.localize(4, null);
        RestartDebugAction.RECONNECT_LABEL = nls.localize(5, null);
        RestartDebugAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], RestartDebugAction);
        return RestartDebugAction;
    }(AbstractDebugAction));
    exports.RestartDebugAction = RestartDebugAction;
    var StepOverDebugAction = (function (_super) {
        __extends(StepOverDebugAction, _super);
        function StepOverDebugAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action step-over', debugService, keybindingService);
        }
        StepOverDebugAction.prototype.run = function () {
            return this.debugService.getActiveSession().next({ threadId: this.debugService.getViewModel().getFocusedThreadId() });
        };
        StepOverDebugAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() === debug.State.Stopped;
        };
        StepOverDebugAction.ID = 'workbench.action.debug.stepOver';
        StepOverDebugAction.LABEL = nls.localize(6, null);
        StepOverDebugAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], StepOverDebugAction);
        return StepOverDebugAction;
    }(AbstractDebugAction));
    exports.StepOverDebugAction = StepOverDebugAction;
    var StepIntoDebugAction = (function (_super) {
        __extends(StepIntoDebugAction, _super);
        function StepIntoDebugAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action step-into', debugService, keybindingService);
        }
        StepIntoDebugAction.prototype.run = function () {
            return this.debugService.getActiveSession().stepIn({ threadId: this.debugService.getViewModel().getFocusedThreadId() });
        };
        StepIntoDebugAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() === debug.State.Stopped;
        };
        StepIntoDebugAction.ID = 'workbench.action.debug.stepInto';
        StepIntoDebugAction.LABEL = nls.localize(7, null);
        StepIntoDebugAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], StepIntoDebugAction);
        return StepIntoDebugAction;
    }(AbstractDebugAction));
    exports.StepIntoDebugAction = StepIntoDebugAction;
    var StepOutDebugAction = (function (_super) {
        __extends(StepOutDebugAction, _super);
        function StepOutDebugAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action step-out', debugService, keybindingService);
        }
        StepOutDebugAction.prototype.run = function () {
            return this.debugService.getActiveSession().stepOut({ threadId: this.debugService.getViewModel().getFocusedThreadId() });
        };
        StepOutDebugAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() === debug.State.Stopped;
        };
        StepOutDebugAction.ID = 'workbench.action.debug.stepOut';
        StepOutDebugAction.LABEL = nls.localize(8, null);
        StepOutDebugAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], StepOutDebugAction);
        return StepOutDebugAction;
    }(AbstractDebugAction));
    exports.StepOutDebugAction = StepOutDebugAction;
    var StopDebugAction = (function (_super) {
        __extends(StopDebugAction, _super);
        function StopDebugAction(id, label, debugService, keybindingService) {
            var _this = this;
            _super.call(this, id, label, 'debug-action stop', debugService, keybindingService);
            this.toDispose.push(this.debugService.addListener2(debug.ServiceEvents.STATE_CHANGED, function () {
                var session = _this.debugService.getActiveSession();
                if (session) {
                    _this.updateLabel(session.isAttach ? StopDebugAction.DISCONNECT_LABEL : StopDebugAction.LABEL);
                }
            }));
        }
        StopDebugAction.prototype.run = function () {
            var session = this.debugService.getActiveSession();
            return session ? session.disconnect(false, true) : winjs_base_1.TPromise.as(null);
        };
        StopDebugAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() !== debug.State.Inactive;
        };
        StopDebugAction.ID = 'workbench.action.debug.stop';
        StopDebugAction.LABEL = nls.localize(9, null);
        StopDebugAction.DISCONNECT_LABEL = nls.localize(10, null);
        StopDebugAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], StopDebugAction);
        return StopDebugAction;
    }(AbstractDebugAction));
    exports.StopDebugAction = StopDebugAction;
    var ContinueAction = (function (_super) {
        __extends(ContinueAction, _super);
        function ContinueAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action continue', debugService, keybindingService);
        }
        ContinueAction.prototype.run = function () {
            return this.debugService.getActiveSession().continue({ threadId: this.debugService.getViewModel().getFocusedThreadId() });
        };
        ContinueAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() === debug.State.Stopped;
        };
        ContinueAction.ID = 'workbench.action.debug.continue';
        ContinueAction.LABEL = nls.localize(11, null);
        ContinueAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], ContinueAction);
        return ContinueAction;
    }(AbstractDebugAction));
    exports.ContinueAction = ContinueAction;
    var PauseAction = (function (_super) {
        __extends(PauseAction, _super);
        function PauseAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action pause', debugService, keybindingService);
        }
        PauseAction.prototype.run = function () {
            return this.debugService.getActiveSession().pause({ threadId: this.debugService.getViewModel().getFocusedThreadId() });
        };
        PauseAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() === debug.State.Running;
        };
        PauseAction.ID = 'workbench.action.debug.pause';
        PauseAction.LABEL = nls.localize(12, null);
        PauseAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], PauseAction);
        return PauseAction;
    }(AbstractDebugAction));
    exports.PauseAction = PauseAction;
    var RemoveBreakpointAction = (function (_super) {
        __extends(RemoveBreakpointAction, _super);
        function RemoveBreakpointAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action remove', debugService, keybindingService);
            this.updateEnablement();
        }
        RemoveBreakpointAction.prototype.run = function (breakpoint) {
            return breakpoint instanceof model.Breakpoint ? this.debugService.toggleBreakpoint({ uri: breakpoint.source.uri, lineNumber: breakpoint.lineNumber })
                : this.debugService.removeFunctionBreakpoints(breakpoint.getId());
        };
        RemoveBreakpointAction.ID = 'workbench.debug.viewlet.action.removeBreakpoint';
        RemoveBreakpointAction.LABEL = nls.localize(13, null);
        RemoveBreakpointAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], RemoveBreakpointAction);
        return RemoveBreakpointAction;
    }(AbstractDebugAction));
    exports.RemoveBreakpointAction = RemoveBreakpointAction;
    var RemoveAllBreakpointsAction = (function (_super) {
        __extends(RemoveAllBreakpointsAction, _super);
        function RemoveAllBreakpointsAction(id, label, debugService, keybindingService) {
            var _this = this;
            _super.call(this, id, label, 'debug-action remove-all', debugService, keybindingService);
            this.toDispose.push(this.debugService.getModel().addListener2(debug.ModelEvents.BREAKPOINTS_UPDATED, function () { return _this.updateEnablement(); }));
        }
        RemoveAllBreakpointsAction.prototype.run = function () {
            return winjs_base_1.TPromise.join([this.debugService.removeAllBreakpoints(), this.debugService.removeFunctionBreakpoints()]);
        };
        RemoveAllBreakpointsAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && (this.debugService.getModel().getBreakpoints().length > 0 || this.debugService.getModel().getFunctionBreakpoints().length > 0);
        };
        RemoveAllBreakpointsAction.ID = 'workbench.debug.viewlet.action.removeAllBreakpoints';
        RemoveAllBreakpointsAction.LABEL = nls.localize(14, null);
        RemoveAllBreakpointsAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], RemoveAllBreakpointsAction);
        return RemoveAllBreakpointsAction;
    }(AbstractDebugAction));
    exports.RemoveAllBreakpointsAction = RemoveAllBreakpointsAction;
    var ToggleEnablementAction = (function (_super) {
        __extends(ToggleEnablementAction, _super);
        function ToggleEnablementAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action toggle-enablement', debugService, keybindingService);
        }
        ToggleEnablementAction.prototype.run = function (element) {
            return this.debugService.toggleEnablement(element);
        };
        ToggleEnablementAction.ID = 'workbench.debug.viewlet.action.toggleBreakpointEnablement';
        ToggleEnablementAction.LABEL = nls.localize(15, null);
        ToggleEnablementAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], ToggleEnablementAction);
        return ToggleEnablementAction;
    }(AbstractDebugAction));
    exports.ToggleEnablementAction = ToggleEnablementAction;
    var EnableAllBreakpointsAction = (function (_super) {
        __extends(EnableAllBreakpointsAction, _super);
        function EnableAllBreakpointsAction(id, label, debugService, keybindingService) {
            var _this = this;
            _super.call(this, id, label, 'debug-action enable-all-breakpoints', debugService, keybindingService);
            this.toDispose.push(this.debugService.getModel().addListener2(debug.ModelEvents.BREAKPOINTS_UPDATED, function () { return _this.updateEnablement(); }));
        }
        EnableAllBreakpointsAction.prototype.run = function () {
            return this.debugService.enableOrDisableAllBreakpoints(true);
        };
        EnableAllBreakpointsAction.prototype.isEnabled = function () {
            var model = this.debugService.getModel();
            return _super.prototype.isEnabled.call(this) && model.getBreakpoints().concat(model.getFunctionBreakpoints()).concat(model.getExceptionBreakpoints()).some(function (bp) { return !bp.enabled; });
        };
        EnableAllBreakpointsAction.ID = 'workbench.debug.viewlet.action.enableAllBreakpoints';
        EnableAllBreakpointsAction.LABEL = nls.localize(16, null);
        EnableAllBreakpointsAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], EnableAllBreakpointsAction);
        return EnableAllBreakpointsAction;
    }(AbstractDebugAction));
    exports.EnableAllBreakpointsAction = EnableAllBreakpointsAction;
    var DisableAllBreakpointsAction = (function (_super) {
        __extends(DisableAllBreakpointsAction, _super);
        function DisableAllBreakpointsAction(id, label, debugService, keybindingService) {
            var _this = this;
            _super.call(this, id, label, 'debug-action disable-all-breakpoints', debugService, keybindingService);
            this.toDispose.push(this.debugService.getModel().addListener2(debug.ModelEvents.BREAKPOINTS_UPDATED, function () { return _this.updateEnablement(); }));
        }
        DisableAllBreakpointsAction.prototype.run = function () {
            return this.debugService.enableOrDisableAllBreakpoints(false);
        };
        DisableAllBreakpointsAction.prototype.isEnabled = function () {
            var model = this.debugService.getModel();
            return _super.prototype.isEnabled.call(this) && model.getBreakpoints().concat(model.getFunctionBreakpoints()).concat(model.getExceptionBreakpoints()).some(function (bp) { return bp.enabled; });
        };
        DisableAllBreakpointsAction.ID = 'workbench.debug.viewlet.action.disableAllBreakpoints';
        DisableAllBreakpointsAction.LABEL = nls.localize(17, null);
        DisableAllBreakpointsAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], DisableAllBreakpointsAction);
        return DisableAllBreakpointsAction;
    }(AbstractDebugAction));
    exports.DisableAllBreakpointsAction = DisableAllBreakpointsAction;
    var ToggleBreakpointsActivatedAction = (function (_super) {
        __extends(ToggleBreakpointsActivatedAction, _super);
        function ToggleBreakpointsActivatedAction(id, label, debugService, keybindingService) {
            var _this = this;
            _super.call(this, id, label, 'debug-action breakpoints-activate', debugService, keybindingService);
            this.updateLabel(this.debugService.getModel().areBreakpointsActivated() ? ToggleBreakpointsActivatedAction.DEACTIVATE_LABEL : ToggleBreakpointsActivatedAction.ACTIVATE_LABEL);
            this.toDispose.push(this.debugService.getModel().addListener2(debug.ModelEvents.BREAKPOINTS_UPDATED, function () {
                _this.updateLabel(_this.debugService.getModel().areBreakpointsActivated() ? ToggleBreakpointsActivatedAction.DEACTIVATE_LABEL : ToggleBreakpointsActivatedAction.ACTIVATE_LABEL);
                _this.updateEnablement();
            }));
        }
        ToggleBreakpointsActivatedAction.prototype.run = function () {
            return this.debugService.toggleBreakpointsActivated();
        };
        ToggleBreakpointsActivatedAction.prototype.isEnabled = function () {
            return (this.debugService.getModel().getFunctionBreakpoints().length + this.debugService.getModel().getBreakpoints().length) > 0;
        };
        ToggleBreakpointsActivatedAction.ID = 'workbench.debug.viewlet.action.toggleBreakpointsActivatedAction';
        ToggleBreakpointsActivatedAction.ACTIVATE_LABEL = nls.localize(18, null);
        ToggleBreakpointsActivatedAction.DEACTIVATE_LABEL = nls.localize(19, null);
        ToggleBreakpointsActivatedAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], ToggleBreakpointsActivatedAction);
        return ToggleBreakpointsActivatedAction;
    }(AbstractDebugAction));
    exports.ToggleBreakpointsActivatedAction = ToggleBreakpointsActivatedAction;
    var ReapplyBreakpointsAction = (function (_super) {
        __extends(ReapplyBreakpointsAction, _super);
        function ReapplyBreakpointsAction(id, label, debugService, keybindingService) {
            var _this = this;
            _super.call(this, id, label, null, debugService, keybindingService);
            this.toDispose.push(this.debugService.getModel().addListener2(debug.ModelEvents.BREAKPOINTS_UPDATED, function () { return _this.updateEnablement(); }));
        }
        ReapplyBreakpointsAction.prototype.run = function () {
            return this.debugService.sendAllBreakpoints();
        };
        ReapplyBreakpointsAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() !== debug.State.Disabled && this.debugService.getState() !== debug.State.Inactive &&
                ((this.debugService.getModel().getFunctionBreakpoints().length + this.debugService.getModel().getBreakpoints().length) > 0);
        };
        ReapplyBreakpointsAction.ID = 'workbench.debug.viewlet.action.reapplyBreakpointsAction';
        ReapplyBreakpointsAction.LABEL = nls.localize(20, null);
        ReapplyBreakpointsAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], ReapplyBreakpointsAction);
        return ReapplyBreakpointsAction;
    }(AbstractDebugAction));
    exports.ReapplyBreakpointsAction = ReapplyBreakpointsAction;
    var AddFunctionBreakpointAction = (function (_super) {
        __extends(AddFunctionBreakpointAction, _super);
        function AddFunctionBreakpointAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action add-function-breakpoint', debugService, keybindingService);
        }
        AddFunctionBreakpointAction.prototype.run = function () {
            this.debugService.addFunctionBreakpoint();
            return winjs_base_1.TPromise.as(null);
        };
        AddFunctionBreakpointAction.ID = 'workbench.debug.viewlet.action.addFunctionBreakpointAction';
        AddFunctionBreakpointAction.LABEL = nls.localize(21, null);
        AddFunctionBreakpointAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], AddFunctionBreakpointAction);
        return AddFunctionBreakpointAction;
    }(AbstractDebugAction));
    exports.AddFunctionBreakpointAction = AddFunctionBreakpointAction;
    var RenameFunctionBreakpointAction = (function (_super) {
        __extends(RenameFunctionBreakpointAction, _super);
        function RenameFunctionBreakpointAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, null, debugService, keybindingService);
        }
        RenameFunctionBreakpointAction.prototype.run = function (fbp) {
            this.debugService.getViewModel().setSelectedFunctionBreakpoint(fbp);
            return winjs_base_1.TPromise.as(null);
        };
        RenameFunctionBreakpointAction.ID = 'workbench.debug.viewlet.action.renameFunctionBreakpointAction';
        RenameFunctionBreakpointAction.LABEL = nls.localize(22, null);
        RenameFunctionBreakpointAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], RenameFunctionBreakpointAction);
        return RenameFunctionBreakpointAction;
    }(AbstractDebugAction));
    exports.RenameFunctionBreakpointAction = RenameFunctionBreakpointAction;
    var AddConditionalBreakpointAction = (function (_super) {
        __extends(AddConditionalBreakpointAction, _super);
        function AddConditionalBreakpointAction(id, label, editor, lineNumber, debugService, keybindingService) {
            _super.call(this, id, label, null, debugService, keybindingService);
            this.editor = editor;
            this.lineNumber = lineNumber;
        }
        AddConditionalBreakpointAction.prototype.run = function () {
            return this.debugService.editBreakpoint(this.editor, this.lineNumber);
        };
        AddConditionalBreakpointAction.ID = 'workbench.debug.viewlet.action.addConditionalBreakpointAction';
        AddConditionalBreakpointAction.LABEL = nls.localize(23, null);
        AddConditionalBreakpointAction = __decorate([
            __param(4, IDebugService),
            __param(5, keybindingService_1.IKeybindingService)
        ], AddConditionalBreakpointAction);
        return AddConditionalBreakpointAction;
    }(AbstractDebugAction));
    exports.AddConditionalBreakpointAction = AddConditionalBreakpointAction;
    var EditConditionalBreakpointAction = (function (_super) {
        __extends(EditConditionalBreakpointAction, _super);
        function EditConditionalBreakpointAction(id, label, editor, lineNumber, debugService, keybindingService) {
            _super.call(this, id, label, null, debugService, keybindingService);
            this.editor = editor;
            this.lineNumber = lineNumber;
        }
        EditConditionalBreakpointAction.prototype.run = function (breakpoint) {
            return this.debugService.editBreakpoint(this.editor, this.lineNumber);
        };
        EditConditionalBreakpointAction.ID = 'workbench.debug.viewlet.action.editConditionalBreakpointAction';
        EditConditionalBreakpointAction.LABEL = nls.localize(24, null);
        EditConditionalBreakpointAction = __decorate([
            __param(4, IDebugService),
            __param(5, keybindingService_1.IKeybindingService)
        ], EditConditionalBreakpointAction);
        return EditConditionalBreakpointAction;
    }(AbstractDebugAction));
    exports.EditConditionalBreakpointAction = EditConditionalBreakpointAction;
    var ToggleBreakpointAction = (function (_super) {
        __extends(ToggleBreakpointAction, _super);
        function ToggleBreakpointAction(descriptor, editor, debugService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
            this.debugService = debugService;
        }
        ToggleBreakpointAction.prototype.run = function () {
            if (this.debugService.getState() !== debug.State.Disabled) {
                var lineNumber = this.editor.getPosition().lineNumber;
                var modelUrl = this.editor.getModel().getAssociatedResource();
                if (this.debugService.canSetBreakpointsIn(this.editor.getModel())) {
                    return this.debugService.toggleBreakpoint({ uri: modelUrl, lineNumber: lineNumber });
                }
            }
            return winjs_base_1.TPromise.as(null);
        };
        ToggleBreakpointAction.ID = 'editor.debug.action.toggleBreakpoint';
        ToggleBreakpointAction = __decorate([
            __param(2, IDebugService)
        ], ToggleBreakpointAction);
        return ToggleBreakpointAction;
    }(editorAction_1.EditorAction));
    exports.ToggleBreakpointAction = ToggleBreakpointAction;
    var EditorConditionalBreakpointAction = (function (_super) {
        __extends(EditorConditionalBreakpointAction, _super);
        function EditorConditionalBreakpointAction(descriptor, editor, debugService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
            this.debugService = debugService;
        }
        EditorConditionalBreakpointAction.prototype.run = function () {
            if (this.debugService.getState() !== debug.State.Disabled) {
                var lineNumber = this.editor.getPosition().lineNumber;
                if (this.debugService.canSetBreakpointsIn(this.editor.getModel())) {
                    return this.debugService.editBreakpoint(this.editor, lineNumber);
                }
            }
            return winjs_base_1.TPromise.as(null);
        };
        EditorConditionalBreakpointAction.ID = 'editor.debug.action.conditionalBreakpoint';
        EditorConditionalBreakpointAction = __decorate([
            __param(2, IDebugService)
        ], EditorConditionalBreakpointAction);
        return EditorConditionalBreakpointAction;
    }(editorAction_1.EditorAction));
    exports.EditorConditionalBreakpointAction = EditorConditionalBreakpointAction;
    var CopyValueAction = (function (_super) {
        __extends(CopyValueAction, _super);
        function CopyValueAction(id, label, value, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action copy-value', debugService, keybindingService);
            this.value = value;
        }
        CopyValueAction.prototype.run = function () {
            var _this = this;
            if (this.value instanceof model.Variable) {
                var frameId = this.debugService.getViewModel().getFocusedStackFrame().frameId;
                var session = this.debugService.getActiveSession();
                return session.evaluate({ expression: model.getFullExpressionName(this.value, session.getType()), frameId: frameId }).then(function (result) {
                    electron_1.clipboard.writeText(result.body.result);
                }, function (err) { return electron_1.clipboard.writeText(_this.value.value); });
            }
            electron_1.clipboard.writeText(this.value);
            return winjs_base_1.TPromise.as(null);
        };
        CopyValueAction.ID = 'workbench.debug.viewlet.action.copyValue';
        CopyValueAction.LABEL = nls.localize(25, null);
        CopyValueAction = __decorate([
            __param(3, IDebugService),
            __param(4, keybindingService_1.IKeybindingService)
        ], CopyValueAction);
        return CopyValueAction;
    }(AbstractDebugAction));
    exports.CopyValueAction = CopyValueAction;
    var RunToCursorAction = (function (_super) {
        __extends(RunToCursorAction, _super);
        function RunToCursorAction(descriptor, editor, debugService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
            this.debugService = debugService;
        }
        RunToCursorAction.prototype.run = function () {
            var _this = this;
            var lineNumber = this.editor.getPosition().lineNumber;
            var uri = this.editor.getModel().getAssociatedResource();
            this.debugService.getActiveSession().addOneTimeListener(debug.SessionEvents.STOPPED, function () {
                _this.debugService.toggleBreakpoint({ uri: uri, lineNumber: lineNumber });
            });
            return this.debugService.toggleBreakpoint({ uri: uri, lineNumber: lineNumber }).then(function () {
                return _this.debugService.getActiveSession().continue({ threadId: _this.debugService.getViewModel().getFocusedThreadId() }).then(function (response) {
                    return response.success;
                });
            });
        };
        RunToCursorAction.prototype.getGroupId = function () {
            return '5_debug/1_run_to_cursor';
        };
        RunToCursorAction.prototype.shouldShowInContextMenu = function () {
            if (this.debugService.getState() !== debug.State.Stopped) {
                return false;
            }
            var lineNumber = this.editor.getPosition().lineNumber;
            var uri = this.editor.getModel().getAssociatedResource();
            var bps = this.debugService.getModel().getBreakpoints().filter(function (bp) { return bp.lineNumber === lineNumber && bp.source.uri.toString() === uri.toString(); });
            // breakpoint must not be on position (no need for this action).
            return bps.length === 0;
        };
        RunToCursorAction.ID = 'editor.debug.action.runToCursor';
        RunToCursorAction = __decorate([
            __param(2, IDebugService)
        ], RunToCursorAction);
        return RunToCursorAction;
    }(editorAction_1.EditorAction));
    exports.RunToCursorAction = RunToCursorAction;
    var AddWatchExpressionAction = (function (_super) {
        __extends(AddWatchExpressionAction, _super);
        function AddWatchExpressionAction(id, label, debugService, keybindingService) {
            var _this = this;
            _super.call(this, id, label, 'debug-action add-watch-expression', debugService, keybindingService);
            this.toDispose.push(this.debugService.getModel().addListener2(debug.ModelEvents.WATCH_EXPRESSIONS_UPDATED, function () { return _this.updateEnablement(); }));
        }
        AddWatchExpressionAction.prototype.run = function () {
            return this.debugService.addWatchExpression();
        };
        AddWatchExpressionAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getModel().getWatchExpressions().every(function (we) { return !!we.name; });
        };
        AddWatchExpressionAction.ID = 'workbench.debug.viewlet.action.addWatchExpression';
        AddWatchExpressionAction.LABEL = nls.localize(26, null);
        AddWatchExpressionAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], AddWatchExpressionAction);
        return AddWatchExpressionAction;
    }(AbstractDebugAction));
    exports.AddWatchExpressionAction = AddWatchExpressionAction;
    var SelectionToWatchExpressionsAction = (function (_super) {
        __extends(SelectionToWatchExpressionsAction, _super);
        function SelectionToWatchExpressionsAction(descriptor, editor, debugService, viewletService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
            this.debugService = debugService;
            this.viewletService = viewletService;
        }
        SelectionToWatchExpressionsAction.prototype.run = function () {
            var _this = this;
            var text = this.editor.getModel().getValueInRange(this.editor.getSelection());
            return this.viewletService.openViewlet(debug.VIEWLET_ID).then(function () { return _this.debugService.addWatchExpression(text); });
        };
        SelectionToWatchExpressionsAction.prototype.getGroupId = function () {
            return '5_debug/3_selection_to_watch';
        };
        SelectionToWatchExpressionsAction.prototype.shouldShowInContextMenu = function () {
            var selection = this.editor.getSelection();
            var text = this.editor.getModel().getValueInRange(selection);
            return !!selection && !selection.isEmpty() && this.debugService.getConfigurationName() && text && /\S/.test(text);
        };
        SelectionToWatchExpressionsAction.ID = 'editor.debug.action.selectionToWatch';
        SelectionToWatchExpressionsAction = __decorate([
            __param(2, IDebugService),
            __param(3, viewletService_1.IViewletService)
        ], SelectionToWatchExpressionsAction);
        return SelectionToWatchExpressionsAction;
    }(editorAction_1.EditorAction));
    exports.SelectionToWatchExpressionsAction = SelectionToWatchExpressionsAction;
    var SelectionToReplAction = (function (_super) {
        __extends(SelectionToReplAction, _super);
        function SelectionToReplAction(descriptor, editor, debugService) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
            this.debugService = debugService;
        }
        SelectionToReplAction.prototype.run = function () {
            var _this = this;
            var text = this.editor.getModel().getValueInRange(this.editor.getSelection());
            return this.debugService.addReplExpression(text).then(function () { return _this.debugService.revealRepl(); });
        };
        SelectionToReplAction.prototype.getGroupId = function () {
            return '5_debug/2_selection_to_repl';
        };
        SelectionToReplAction.prototype.shouldShowInContextMenu = function () {
            var selection = this.editor.getSelection();
            return !!selection && !selection.isEmpty() && this.debugService.getState() === debug.State.Stopped;
        };
        SelectionToReplAction.ID = 'editor.debug.action.selectionToRepl';
        SelectionToReplAction = __decorate([
            __param(2, IDebugService)
        ], SelectionToReplAction);
        return SelectionToReplAction;
    }(editorAction_1.EditorAction));
    exports.SelectionToReplAction = SelectionToReplAction;
    var ShowDebugHoverAction = (function (_super) {
        __extends(ShowDebugHoverAction, _super);
        function ShowDebugHoverAction(descriptor, editor) {
            _super.call(this, descriptor, editor, editorActionEnablement_1.Behaviour.TextFocus);
        }
        ShowDebugHoverAction.prototype.run = function () {
            var position = this.editor.getPosition();
            var word = this.editor.getModel().getWordAtPosition(position);
            if (!word) {
                return winjs_base_1.TPromise.as(null);
            }
            var range = new range_1.Range(position.lineNumber, position.column, position.lineNumber, word.endColumn);
            return this.editor.getContribution(debug.EDITOR_CONTRIBUTION_ID).showHover(range, word.word, true);
        };
        ShowDebugHoverAction.ID = 'editor.debug.action.showDebugHover';
        return ShowDebugHoverAction;
    }(editorAction_1.EditorAction));
    exports.ShowDebugHoverAction = ShowDebugHoverAction;
    var AddToWatchExpressionsAction = (function (_super) {
        __extends(AddToWatchExpressionsAction, _super);
        function AddToWatchExpressionsAction(id, label, expression, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action add-to-watch', debugService, keybindingService);
            this.expression = expression;
        }
        AddToWatchExpressionsAction.prototype.run = function () {
            return this.debugService.addWatchExpression(model.getFullExpressionName(this.expression, this.debugService.getActiveSession().getType()));
        };
        AddToWatchExpressionsAction.ID = 'workbench.debug.viewlet.action.addToWatchExpressions';
        AddToWatchExpressionsAction.LABEL = nls.localize(27, null);
        AddToWatchExpressionsAction = __decorate([
            __param(3, IDebugService),
            __param(4, keybindingService_1.IKeybindingService)
        ], AddToWatchExpressionsAction);
        return AddToWatchExpressionsAction;
    }(AbstractDebugAction));
    exports.AddToWatchExpressionsAction = AddToWatchExpressionsAction;
    var RenameWatchExpressionAction = (function (_super) {
        __extends(RenameWatchExpressionAction, _super);
        function RenameWatchExpressionAction(id, label, expression, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action rename', debugService, keybindingService);
            this.expression = expression;
        }
        RenameWatchExpressionAction.prototype.run = function () {
            this.debugService.getViewModel().setSelectedExpression(this.expression);
            return winjs_base_1.TPromise.as(null);
        };
        RenameWatchExpressionAction.ID = 'workbench.debug.viewlet.action.renameWatchExpression';
        RenameWatchExpressionAction.LABEL = nls.localize(28, null);
        RenameWatchExpressionAction = __decorate([
            __param(3, IDebugService),
            __param(4, keybindingService_1.IKeybindingService)
        ], RenameWatchExpressionAction);
        return RenameWatchExpressionAction;
    }(AbstractDebugAction));
    exports.RenameWatchExpressionAction = RenameWatchExpressionAction;
    var RemoveWatchExpressionAction = (function (_super) {
        __extends(RemoveWatchExpressionAction, _super);
        function RemoveWatchExpressionAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action remove', debugService, keybindingService);
        }
        RemoveWatchExpressionAction.prototype.run = function (expression) {
            this.debugService.clearWatchExpressions(expression.getId());
            return winjs_base_1.TPromise.as(null);
        };
        RemoveWatchExpressionAction.ID = 'workbench.debug.viewlet.action.removeWatchExpression';
        RemoveWatchExpressionAction.LABEL = nls.localize(29, null);
        RemoveWatchExpressionAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], RemoveWatchExpressionAction);
        return RemoveWatchExpressionAction;
    }(AbstractDebugAction));
    exports.RemoveWatchExpressionAction = RemoveWatchExpressionAction;
    var RemoveAllWatchExpressionsAction = (function (_super) {
        __extends(RemoveAllWatchExpressionsAction, _super);
        function RemoveAllWatchExpressionsAction(id, label, debugService, keybindingService) {
            var _this = this;
            _super.call(this, id, label, 'debug-action remove-all', debugService, keybindingService);
            this.toDispose.push(this.debugService.getModel().addListener2(debug.ModelEvents.WATCH_EXPRESSIONS_UPDATED, function () { return _this.updateEnablement(); }));
        }
        RemoveAllWatchExpressionsAction.prototype.run = function () {
            this.debugService.clearWatchExpressions();
            return winjs_base_1.TPromise.as(null);
        };
        RemoveAllWatchExpressionsAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getModel().getWatchExpressions().length > 0;
        };
        RemoveAllWatchExpressionsAction.ID = 'workbench.debug.viewlet.action.removeAllWatchExpressions';
        RemoveAllWatchExpressionsAction.LABEL = nls.localize(30, null);
        RemoveAllWatchExpressionsAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], RemoveAllWatchExpressionsAction);
        return RemoveAllWatchExpressionsAction;
    }(AbstractDebugAction));
    exports.RemoveAllWatchExpressionsAction = RemoveAllWatchExpressionsAction;
    var ClearReplAction = (function (_super) {
        __extends(ClearReplAction, _super);
        function ClearReplAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, 'debug-action clear-repl', debugService, keybindingService);
        }
        ClearReplAction.prototype.run = function () {
            this.debugService.clearReplExpressions();
            // focus back to repl
            return this.debugService.revealRepl();
        };
        ClearReplAction.ID = 'workbench.debug.panel.action.clearReplAction';
        ClearReplAction.LABEL = nls.localize(31, null);
        ClearReplAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], ClearReplAction);
        return ClearReplAction;
    }(AbstractDebugAction));
    exports.ClearReplAction = ClearReplAction;
    var ToggleReplAction = (function (_super) {
        __extends(ToggleReplAction, _super);
        function ToggleReplAction(id, label, debugService, partService, panelService, keybindingService, eventService) {
            _super.call(this, id, label, 'debug-action toggle-repl', debugService, keybindingService);
            this.partService = partService;
            this.panelService = panelService;
            this.eventService = eventService;
            this.enabled = this.debugService.getState() !== debug.State.Disabled;
            this.registerListeners();
        }
        ToggleReplAction.prototype.run = function () {
            if (this.isReplVisible()) {
                this.partService.setPanelHidden(true);
                return winjs_base_1.TPromise.as(null);
            }
            return this.debugService.revealRepl();
        };
        ToggleReplAction.prototype.registerListeners = function () {
            var _this = this;
            this.toDispose.push(this.debugService.getModel().addListener2(debug.ModelEvents.REPL_ELEMENTS_UPDATED, function () {
                if (!_this.isReplVisible()) {
                    _this.class = 'debug-action toggle-repl notification';
                }
            }));
            this.toDispose.push(this.eventService.addListener2(events_1.EventType.COMPOSITE_OPENED, function (e) {
                if (e.compositeId === debug.REPL_ID) {
                    _this.class = 'debug-action toggle-repl';
                }
            }));
        };
        ToggleReplAction.prototype.isReplVisible = function () {
            var panel = this.panelService.getActivePanel();
            return panel && panel.getId() === debug.REPL_ID;
        };
        ToggleReplAction.ID = 'workbench.debug.action.toggleRepl';
        ToggleReplAction.LABEL = nls.localize(32, null);
        ToggleReplAction = __decorate([
            __param(2, IDebugService),
            __param(3, partService_1.IPartService),
            __param(4, panelService_1.IPanelService),
            __param(5, keybindingService_1.IKeybindingService),
            __param(6, event_1.IEventService)
        ], ToggleReplAction);
        return ToggleReplAction;
    }(AbstractDebugAction));
    exports.ToggleReplAction = ToggleReplAction;
    var RunAction = (function (_super) {
        __extends(RunAction, _super);
        function RunAction(id, label, debugService, keybindingService) {
            _super.call(this, id, label, null, debugService, keybindingService);
        }
        RunAction.prototype.run = function () {
            return this.debugService.createSession(true);
        };
        RunAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.debugService.getState() === debug.State.Inactive;
        };
        RunAction.ID = 'workbench.action.debug.run';
        RunAction.LABEL = nls.localize(33, null);
        RunAction = __decorate([
            __param(2, IDebugService),
            __param(3, keybindingService_1.IKeybindingService)
        ], RunAction);
        return RunAction;
    }(AbstractDebugAction));
    exports.RunAction = RunAction;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/debug/browser/debugActionsWidget", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/common/errors', 'vs/base/common/severity', 'vs/base/browser/builder', 'vs/base/common/events', 'vs/base/browser/ui/actionbar/actionbar', 'vs/workbench/common/constants', 'vs/workbench/parts/debug/common/debug', 'vs/workbench/parts/debug/electron-browser/debugActions', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/platform/telemetry/common/telemetry'], function (require, exports, lifecycle, errors, severity_1, builder, events, actionbar, constants, debug, dbgactions, instantiation_1, message_1, telemetry_1) {
    "use strict";
    var IDebugService = debug.IDebugService;
    var $ = builder.$;
    var DebugActionsWidget = (function () {
        function DebugActionsWidget(messageService, telemetryService, debugService, instantiationService) {
            this.messageService = messageService;
            this.telemetryService = telemetryService;
            this.debugService = debugService;
            this.instantiationService = instantiationService;
            this.$el = $().div().addClass('debug-actions-widget');
            this.toDispose = [];
            this.actionBar = new actionbar.ActionBar(this.$el, {
                orientation: actionbar.ActionsOrientation.HORIZONTAL
            });
            this.toDispose.push(this.actionBar);
            this.registerListeners();
            this.hide();
            this.isBuilt = false;
        }
        DebugActionsWidget.prototype.registerListeners = function () {
            var _this = this;
            this.toDispose.push(this.debugService.addListener2(debug.ServiceEvents.STATE_CHANGED, function () {
                _this.onDebugStateChange();
            }));
            this.toDispose.push(this.actionBar.actionRunner.addListener2(events.EventType.RUN, function (e) {
                // check for error
                if (e.error && !errors.isPromiseCanceledError(e.error)) {
                    _this.messageService.show(severity_1.default.Error, e.error);
                }
                // log in telemetry
                if (_this.telemetryService) {
                    _this.telemetryService.publicLog('workbenchActionExecuted', { id: e.action.id, from: 'debugActionsWidget' });
                }
            }));
        };
        DebugActionsWidget.prototype.getId = function () {
            return DebugActionsWidget.ID;
        };
        DebugActionsWidget.prototype.onDebugStateChange = function () {
            if (this.debugService.getState() === debug.State.Inactive) {
                return this.hide();
            }
            this.actionBar.clear();
            this.actionBar.push(this.getActions(this.instantiationService, this.debugService.getState()), { icon: true, label: false });
            this.show();
        };
        DebugActionsWidget.prototype.show = function () {
            if (this.isVisible) {
                return;
            }
            if (!this.isBuilt) {
                this.isBuilt = true;
                this.$el.build(builder.withElementById(constants.Identifiers.WORKBENCH_CONTAINER).getHTMLElement());
            }
            this.isVisible = true;
            this.$el.show();
        };
        DebugActionsWidget.prototype.hide = function () {
            this.isVisible = false;
            this.$el.hide();
        };
        DebugActionsWidget.prototype.getActions = function (instantiationService, state) {
            var _this = this;
            if (!this.actions) {
                this.continueAction = instantiationService.createInstance(dbgactions.ContinueAction, dbgactions.ContinueAction.ID, dbgactions.ContinueAction.LABEL);
                this.pauseAction = instantiationService.createInstance(dbgactions.PauseAction, dbgactions.PauseAction.ID, dbgactions.PauseAction.LABEL);
                this.actions = [
                    this.continueAction,
                    instantiationService.createInstance(dbgactions.StepOverDebugAction, dbgactions.StepOverDebugAction.ID, dbgactions.StepOverDebugAction.LABEL),
                    instantiationService.createInstance(dbgactions.StepIntoDebugAction, dbgactions.StepIntoDebugAction.ID, dbgactions.StepIntoDebugAction.LABEL),
                    instantiationService.createInstance(dbgactions.StepOutDebugAction, dbgactions.StepOutDebugAction.ID, dbgactions.StepOutDebugAction.LABEL),
                    instantiationService.createInstance(dbgactions.RestartDebugAction, dbgactions.RestartDebugAction.ID, dbgactions.RestartDebugAction.LABEL),
                    instantiationService.createInstance(dbgactions.StopDebugAction, dbgactions.StopDebugAction.ID, dbgactions.StopDebugAction.LABEL)
                ];
                this.actions.forEach(function (a) {
                    _this.toDispose.push(a);
                });
                this.toDispose.push(this.pauseAction);
            }
            this.actions[0] = state === debug.State.Running ? this.pauseAction : this.continueAction;
            return this.actions;
        };
        DebugActionsWidget.prototype.dispose = function () {
            this.toDispose = lifecycle.disposeAll(this.toDispose);
            if (this.$el) {
                this.$el.destroy();
                delete this.$el;
            }
        };
        DebugActionsWidget.ID = 'debug.actionsWidget';
        DebugActionsWidget = __decorate([
            __param(0, message_1.IMessageService),
            __param(1, telemetry_1.ITelemetryService),
            __param(2, IDebugService),
            __param(3, instantiation_1.IInstantiationService)
        ], DebugActionsWidget);
        return DebugActionsWidget;
    }());
    exports.DebugActionsWidget = DebugActionsWidget;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/debug/browser/debugViewer", ["require", "exports", 'vs/nls!vs/workbench/parts/debug/browser/debugViewer', 'vs/base/common/winjs.base', 'vs/base/common/lifecycle', 'vs/base/common/keyCodes', 'vs/base/common/paths', 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/strings', 'vs/base/common/platform', 'vs/base/browser/dom', 'vs/base/common/labels', 'vs/base/browser/ui/actionbar/actionbar', 'vs/base/browser/ui/inputbox/inputBox', 'vs/base/parts/tree/browser/treeDefaults', 'vs/workbench/parts/debug/common/debug', 'vs/workbench/parts/debug/common/debugModel', 'vs/workbench/parts/debug/common/debugViewModel', 'vs/workbench/parts/debug/electron-browser/debugActions', 'vs/platform/contextview/browser/contextView', 'vs/platform/workspace/common/workspace', 'vs/platform/message/common/message'], function (require, exports, nls, winjs_base_1, lifecycle, keyCodes_1, paths, async, errors, strings, platform_1, dom, labels, actionbar, inputbox, treedefaults, debug, model, viewmodel, debugactions, contextView_1, workspace_1, message_1) {
    "use strict";
    var $ = dom.emmet;
    var booleanRegex = /^true|false$/i;
    var stringRegex = /^(['"]).*\1$/;
    function renderExpressionValue(expressionOrValue, container, showChanged) {
        var value = typeof expressionOrValue === 'string' ? expressionOrValue : expressionOrValue.value;
        // remove stale classes
        container.className = 'value';
        // when resolving expressions we represent errors from the server as a variable with name === null.
        if (value === null || ((expressionOrValue instanceof model.Expression || expressionOrValue instanceof model.Variable) && !expressionOrValue.available)) {
            dom.addClass(container, 'unavailable');
            if (value !== model.Expression.DEFAULT_VALUE) {
                dom.addClass(container, 'error');
            }
        }
        else if (!isNaN(+value)) {
            dom.addClass(container, 'number');
        }
        else if (booleanRegex.test(value)) {
            dom.addClass(container, 'boolean');
        }
        else if (stringRegex.test(value)) {
            dom.addClass(container, 'string');
        }
        if (showChanged && expressionOrValue.valueChanged) {
            // value changed color has priority over other colors.
            container.className = 'value changed';
        }
        container.textContent = value;
        container.title = value;
    }
    exports.renderExpressionValue = renderExpressionValue;
    function renderVariable(tree, variable, data, showChanged) {
        if (variable.available) {
            data.name.textContent = variable.name + ':';
        }
        if (variable.value) {
            renderExpressionValue(variable, data.value, showChanged);
        }
        else {
            data.value.textContent = '';
            data.value.title = '';
        }
    }
    exports.renderVariable = renderVariable;
    function renderRenameBox(debugService, contextViewService, tree, element, container, placeholder, ariaLabel) {
        var inputBoxContainer = dom.append(container, $('.inputBoxContainer'));
        var inputBox = new inputbox.InputBox(inputBoxContainer, contextViewService, {
            validationOptions: {
                validation: null,
                showMessage: false
            },
            placeholder: placeholder,
            ariaLabel: ariaLabel
        });
        inputBox.value = element.name ? element.name : '';
        inputBox.focus();
        var disposed = false;
        var toDispose = [inputBox];
        var wrapUp = async.once(function (renamed) {
            if (!disposed) {
                disposed = true;
                if (element instanceof model.Expression && renamed && inputBox.value) {
                    debugService.renameWatchExpression(element.getId(), inputBox.value).done(null, errors.onUnexpectedError);
                }
                else if (element instanceof model.Expression && !element.name) {
                    debugService.clearWatchExpressions(element.getId());
                }
                else if (element instanceof model.FunctionBreakpoint && renamed && inputBox.value) {
                    debugService.renameFunctionBreakpoint(element.getId(), inputBox.value).done(null, errors.onUnexpectedError);
                }
                else if (element instanceof model.FunctionBreakpoint && !element.name) {
                    debugService.removeFunctionBreakpoints(element.getId()).done(null, errors.onUnexpectedError);
                }
                tree.clearHighlight();
                tree.DOMFocus();
                tree.setFocus(element);
                // need to remove the input box since this template will be reused.
                container.removeChild(inputBoxContainer);
                lifecycle.disposeAll(toDispose);
            }
        });
        toDispose.push(dom.addStandardDisposableListener(inputBox.inputElement, 'keydown', function (e) {
            var isEscape = e.equals(keyCodes_1.CommonKeybindings.ESCAPE);
            var isEnter = e.equals(keyCodes_1.CommonKeybindings.ENTER);
            if (isEscape || isEnter) {
                wrapUp(isEnter);
            }
        }));
        toDispose.push(dom.addDisposableListener(inputBox.inputElement, 'blur', function () {
            wrapUp(true);
        }));
    }
    function getSourceName(source, contextService) {
        if (source.inMemory) {
            return source.name;
        }
        return labels.getPathLabel(paths.basename(source.uri.fsPath), contextService);
    }
    var BaseDebugController = (function (_super) {
        __extends(BaseDebugController, _super);
        function BaseDebugController(debugService, contextMenuService, actionProvider, focusOnContextMenu) {
            if (focusOnContextMenu === void 0) { focusOnContextMenu = true; }
            _super.call(this);
            this.debugService = debugService;
            this.contextMenuService = contextMenuService;
            this.actionProvider = actionProvider;
            this.focusOnContextMenu = focusOnContextMenu;
            if (platform_1.isMacintosh) {
                this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.CTRLCMD_BACKSPACE, this.onDelete.bind(this));
            }
            else {
                this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.DELETE, this.onDelete.bind(this));
                this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.SHIFT_DELETE, this.onDelete.bind(this));
            }
        }
        BaseDebugController.prototype.onContextMenu = function (tree, element, event) {
            var _this = this;
            if (event.target && event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
                return false;
            }
            event.preventDefault();
            event.stopPropagation();
            if (this.focusOnContextMenu) {
                tree.setFocus(element);
            }
            if (this.actionProvider.hasSecondaryActions(tree, element)) {
                var anchor_1 = { x: event.posx + 1, y: event.posy };
                this.contextMenuService.showContextMenu({
                    getAnchor: function () { return anchor_1; },
                    getActions: function () { return _this.actionProvider.getSecondaryActions(tree, element); },
                    onHide: function (wasCancelled) {
                        if (wasCancelled) {
                            tree.DOMFocus();
                        }
                    },
                    getActionsContext: function () { return element; }
                });
                return true;
            }
            return false;
        };
        BaseDebugController.prototype.onDelete = function (tree, event) {
            return false;
        };
        return BaseDebugController;
    }(treedefaults.DefaultController));
    exports.BaseDebugController = BaseDebugController;
    // call stack
    var CallStackDataSource = (function () {
        function CallStackDataSource(debugService) {
            this.debugService = debugService;
            // noop
        }
        CallStackDataSource.prototype.getId = function (tree, element) {
            return element.getId();
        };
        CallStackDataSource.prototype.hasChildren = function (tree, element) {
            return element instanceof model.Model || element instanceof model.Thread;
        };
        CallStackDataSource.prototype.getChildren = function (tree, element) {
            if (element instanceof model.Thread) {
                return element.getCallStack(this.debugService);
            }
            var threads = element.getThreads();
            var threadsArray = [];
            Object.keys(threads).forEach(function (threadId) {
                threadsArray.push(threads[threadId]);
            });
            if (threadsArray.length === 1) {
                return threadsArray[0].getCallStack(this.debugService);
            }
            else {
                return winjs_base_1.TPromise.as(threadsArray);
            }
        };
        CallStackDataSource.prototype.getParent = function (tree, element) {
            return winjs_base_1.TPromise.as(null);
        };
        CallStackDataSource = __decorate([
            __param(0, debug.IDebugService)
        ], CallStackDataSource);
        return CallStackDataSource;
    }());
    exports.CallStackDataSource = CallStackDataSource;
    var CallStackRenderer = (function () {
        function CallStackRenderer(contextService) {
            this.contextService = contextService;
            // noop
        }
        CallStackRenderer.prototype.getHeight = function (tree, element) {
            return 22;
        };
        CallStackRenderer.prototype.getTemplateId = function (tree, element) {
            if (element instanceof model.Thread) {
                return CallStackRenderer.THREAD_TEMPLATE_ID;
            }
            if (element instanceof model.StackFrame) {
                return CallStackRenderer.STACK_FRAME_TEMPLATE_ID;
            }
            return null;
        };
        CallStackRenderer.prototype.renderTemplate = function (tree, templateId, container) {
            if (templateId === CallStackRenderer.THREAD_TEMPLATE_ID) {
                var data_1 = Object.create(null);
                data_1.name = dom.append(container, $('.thread'));
                return data_1;
            }
            var data = Object.create(null);
            data.stackFrame = dom.append(container, $('.stack-frame'));
            data.label = dom.append(data.stackFrame, $('span.label'));
            data.file = dom.append(data.stackFrame, $('.file'));
            data.fileName = dom.append(data.file, $('span.file-name'));
            data.lineNumber = dom.append(data.file, $('span.line-number'));
            return data;
        };
        CallStackRenderer.prototype.renderElement = function (tree, element, templateId, templateData) {
            if (templateId === CallStackRenderer.THREAD_TEMPLATE_ID) {
                this.renderThread(element, templateData);
            }
            else {
                this.renderStackFrame(element, templateData);
            }
        };
        CallStackRenderer.prototype.renderThread = function (thread, data) {
            data.name.textContent = thread.name;
        };
        CallStackRenderer.prototype.renderStackFrame = function (stackFrame, data) {
            stackFrame.source.available ? dom.removeClass(data.stackFrame, 'disabled') : dom.addClass(data.stackFrame, 'disabled');
            data.file.title = stackFrame.source.uri.fsPath;
            data.label.textContent = stackFrame.name;
            data.label.title = stackFrame.name;
            data.fileName.textContent = getSourceName(stackFrame.source, this.contextService);
            data.lineNumber.textContent = stackFrame.lineNumber !== undefined ? "" + stackFrame.lineNumber : '';
        };
        CallStackRenderer.prototype.disposeTemplate = function (tree, templateId, templateData) {
            // noop
        };
        CallStackRenderer.THREAD_TEMPLATE_ID = 'thread';
        CallStackRenderer.STACK_FRAME_TEMPLATE_ID = 'stackFrame';
        CallStackRenderer = __decorate([
            __param(0, workspace_1.IWorkspaceContextService)
        ], CallStackRenderer);
        return CallStackRenderer;
    }());
    exports.CallStackRenderer = CallStackRenderer;
    var CallstackAccessibilityProvider = (function () {
        function CallstackAccessibilityProvider(contextService) {
            this.contextService = contextService;
            // noop
        }
        CallstackAccessibilityProvider.prototype.getAriaLabel = function (tree, element) {
            if (element instanceof model.Thread) {
                return nls.localize(0, null, element.name);
            }
            if (element instanceof model.StackFrame) {
                return nls.localize(1, null, element.name, element.lineNumber, getSourceName(element.source, this.contextService));
            }
            return null;
        };
        CallstackAccessibilityProvider = __decorate([
            __param(0, workspace_1.IWorkspaceContextService)
        ], CallstackAccessibilityProvider);
        return CallstackAccessibilityProvider;
    }());
    exports.CallstackAccessibilityProvider = CallstackAccessibilityProvider;
    // variables
    var VariablesActionProvider = (function () {
        function VariablesActionProvider(instantiationService) {
            this.instantiationService = instantiationService;
        }
        VariablesActionProvider.prototype.hasActions = function (tree, element) {
            return false;
        };
        VariablesActionProvider.prototype.getActions = function (tree, element) {
            return winjs_base_1.TPromise.as([]);
        };
        VariablesActionProvider.prototype.hasSecondaryActions = function (tree, element) {
            return element instanceof model.Variable;
        };
        VariablesActionProvider.prototype.getSecondaryActions = function (tree, element) {
            var actions = [];
            var variable = element;
            actions.push(this.instantiationService.createInstance(debugactions.AddToWatchExpressionsAction, debugactions.AddToWatchExpressionsAction.ID, debugactions.AddToWatchExpressionsAction.LABEL, variable));
            if (variable.reference === 0) {
                actions.push(this.instantiationService.createInstance(debugactions.CopyValueAction, debugactions.CopyValueAction.ID, debugactions.CopyValueAction.LABEL, variable));
            }
            return winjs_base_1.TPromise.as(actions);
        };
        VariablesActionProvider.prototype.getActionItem = function (tree, element, action) {
            return null;
        };
        return VariablesActionProvider;
    }());
    exports.VariablesActionProvider = VariablesActionProvider;
    var VariablesDataSource = (function () {
        function VariablesDataSource(debugService) {
            this.debugService = debugService;
            // noop
        }
        VariablesDataSource.prototype.getId = function (tree, element) {
            return element.getId();
        };
        VariablesDataSource.prototype.hasChildren = function (tree, element) {
            if (element instanceof viewmodel.ViewModel || element instanceof model.Scope) {
                return true;
            }
            var variable = element;
            return variable.reference !== 0 && !strings.equalsIgnoreCase(variable.value, 'null');
        };
        VariablesDataSource.prototype.getChildren = function (tree, element) {
            if (element instanceof viewmodel.ViewModel) {
                var focusedStackFrame = element.getFocusedStackFrame();
                return focusedStackFrame ? focusedStackFrame.getScopes(this.debugService) : winjs_base_1.TPromise.as([]);
            }
            var scope = element;
            return scope.getChildren(this.debugService);
        };
        VariablesDataSource.prototype.getParent = function (tree, element) {
            return winjs_base_1.TPromise.as(null);
        };
        return VariablesDataSource;
    }());
    exports.VariablesDataSource = VariablesDataSource;
    var VariablesRenderer = (function () {
        function VariablesRenderer() {
        }
        VariablesRenderer.prototype.getHeight = function (tree, element) {
            return 22;
        };
        VariablesRenderer.prototype.getTemplateId = function (tree, element) {
            if (element instanceof model.Scope) {
                return VariablesRenderer.SCOPE_TEMPLATE_ID;
            }
            if (element instanceof model.Variable) {
                return VariablesRenderer.VARIABLE_TEMPLATE_ID;
            }
            return null;
        };
        VariablesRenderer.prototype.renderTemplate = function (tree, templateId, container) {
            if (templateId === VariablesRenderer.SCOPE_TEMPLATE_ID) {
                var data_2 = Object.create(null);
                data_2.name = dom.append(container, $('.scope'));
                return data_2;
            }
            var data = Object.create(null);
            data.expression = dom.append(container, $(platform_1.isMacintosh ? '.expression.mac' : '.expression.win-linux'));
            data.name = dom.append(data.expression, $('span.name'));
            data.value = dom.append(data.expression, $('span.value'));
            return data;
        };
        VariablesRenderer.prototype.renderElement = function (tree, element, templateId, templateData) {
            if (templateId === VariablesRenderer.SCOPE_TEMPLATE_ID) {
                this.renderScope(element, templateData);
            }
            else {
                renderVariable(tree, element, templateData, true);
            }
        };
        VariablesRenderer.prototype.renderScope = function (scope, data) {
            data.name.textContent = scope.name;
        };
        VariablesRenderer.prototype.disposeTemplate = function (tree, templateId, templateData) {
            // noop
        };
        VariablesRenderer.SCOPE_TEMPLATE_ID = 'scope';
        VariablesRenderer.VARIABLE_TEMPLATE_ID = 'variable';
        return VariablesRenderer;
    }());
    exports.VariablesRenderer = VariablesRenderer;
    var VariablesAccessibilityProvider = (function () {
        function VariablesAccessibilityProvider() {
        }
        VariablesAccessibilityProvider.prototype.getAriaLabel = function (tree, element) {
            if (element instanceof model.Scope) {
                return nls.localize(2, null, element.name);
            }
            if (element instanceof model.Variable) {
                return nls.localize(3, null, element.name, element.value);
            }
            return null;
        };
        return VariablesAccessibilityProvider;
    }());
    exports.VariablesAccessibilityProvider = VariablesAccessibilityProvider;
    // watch expressions
    var WatchExpressionsActionProvider = (function () {
        function WatchExpressionsActionProvider(instantiationService) {
            this.instantiationService = instantiationService;
        }
        WatchExpressionsActionProvider.prototype.hasActions = function (tree, element) {
            return element instanceof model.Expression && element.name;
        };
        WatchExpressionsActionProvider.prototype.hasSecondaryActions = function (tree, element) {
            return true;
        };
        WatchExpressionsActionProvider.prototype.getActions = function (tree, element) {
            return winjs_base_1.TPromise.as(this.getExpressionActions());
        };
        WatchExpressionsActionProvider.prototype.getExpressionActions = function () {
            return [this.instantiationService.createInstance(debugactions.RemoveWatchExpressionAction, debugactions.RemoveWatchExpressionAction.ID, debugactions.RemoveWatchExpressionAction.LABEL)];
        };
        WatchExpressionsActionProvider.prototype.getSecondaryActions = function (tree, element) {
            var actions = [];
            if (element instanceof model.Expression) {
                var expression = element;
                actions.push(this.instantiationService.createInstance(debugactions.AddWatchExpressionAction, debugactions.AddWatchExpressionAction.ID, debugactions.AddWatchExpressionAction.LABEL));
                actions.push(this.instantiationService.createInstance(debugactions.RenameWatchExpressionAction, debugactions.RenameWatchExpressionAction.ID, debugactions.RenameWatchExpressionAction.LABEL, expression));
                if (expression.reference === 0) {
                    actions.push(this.instantiationService.createInstance(debugactions.CopyValueAction, debugactions.CopyValueAction.ID, debugactions.CopyValueAction.LABEL, expression.value));
                }
                actions.push(new actionbar.Separator());
                actions.push(this.instantiationService.createInstance(debugactions.RemoveWatchExpressionAction, debugactions.RemoveWatchExpressionAction.ID, debugactions.RemoveWatchExpressionAction.LABEL));
                actions.push(this.instantiationService.createInstance(debugactions.RemoveAllWatchExpressionsAction, debugactions.RemoveAllWatchExpressionsAction.ID, debugactions.RemoveAllWatchExpressionsAction.LABEL));
            }
            else {
                actions.push(this.instantiationService.createInstance(debugactions.AddWatchExpressionAction, debugactions.AddWatchExpressionAction.ID, debugactions.AddWatchExpressionAction.LABEL));
                if (element instanceof model.Variable) {
                    var variable = element;
                    if (variable.reference === 0) {
                        actions.push(this.instantiationService.createInstance(debugactions.CopyValueAction, debugactions.CopyValueAction.ID, debugactions.CopyValueAction.LABEL, variable.value));
                    }
                    actions.push(new actionbar.Separator());
                }
                actions.push(this.instantiationService.createInstance(debugactions.RemoveAllWatchExpressionsAction, debugactions.RemoveAllWatchExpressionsAction.ID, debugactions.RemoveAllWatchExpressionsAction.LABEL));
            }
            return winjs_base_1.TPromise.as(actions);
        };
        WatchExpressionsActionProvider.prototype.getActionItem = function (tree, element, action) {
            return null;
        };
        return WatchExpressionsActionProvider;
    }());
    exports.WatchExpressionsActionProvider = WatchExpressionsActionProvider;
    var WatchExpressionsDataSource = (function () {
        function WatchExpressionsDataSource(debugService) {
            this.debugService = debugService;
            // noop
        }
        WatchExpressionsDataSource.prototype.getId = function (tree, element) {
            return element.getId();
        };
        WatchExpressionsDataSource.prototype.hasChildren = function (tree, element) {
            if (element instanceof model.Model) {
                return true;
            }
            var watchExpression = element;
            return watchExpression.reference !== 0 && !strings.equalsIgnoreCase(watchExpression.value, 'null');
        };
        WatchExpressionsDataSource.prototype.getChildren = function (tree, element) {
            if (element instanceof model.Model) {
                return winjs_base_1.TPromise.as(element.getWatchExpressions());
            }
            var expression = element;
            return expression.getChildren(this.debugService);
        };
        WatchExpressionsDataSource.prototype.getParent = function (tree, element) {
            return winjs_base_1.TPromise.as(null);
        };
        return WatchExpressionsDataSource;
    }());
    exports.WatchExpressionsDataSource = WatchExpressionsDataSource;
    var WatchExpressionsRenderer = (function () {
        function WatchExpressionsRenderer(actionProvider, actionRunner, messageService, debugService, contextViewService) {
            this.actionRunner = actionRunner;
            this.messageService = messageService;
            this.debugService = debugService;
            this.contextViewService = contextViewService;
            this.toDispose = [];
            this.actionProvider = actionProvider;
        }
        WatchExpressionsRenderer.prototype.getHeight = function (tree, element) {
            return 22;
        };
        WatchExpressionsRenderer.prototype.getTemplateId = function (tree, element) {
            if (element instanceof model.Expression) {
                return WatchExpressionsRenderer.WATCH_EXPRESSION_TEMPLATE_ID;
            }
            return WatchExpressionsRenderer.VARIABLE_TEMPLATE_ID;
        };
        WatchExpressionsRenderer.prototype.renderTemplate = function (tree, templateId, container) {
            var data = Object.create(null);
            if (templateId === WatchExpressionsRenderer.WATCH_EXPRESSION_TEMPLATE_ID) {
                data.actionBar = new actionbar.ActionBar(container, { actionRunner: this.actionRunner });
                data.actionBar.push(this.actionProvider.getExpressionActions(), { icon: true, label: false });
            }
            data.expression = dom.append(container, $(platform_1.isMacintosh ? '.expression.mac' : '.expression.win-linux'));
            data.name = dom.append(data.expression, $('span.name'));
            data.value = dom.append(data.expression, $('span.value'));
            return data;
        };
        WatchExpressionsRenderer.prototype.renderElement = function (tree, element, templateId, templateData) {
            if (templateId === WatchExpressionsRenderer.WATCH_EXPRESSION_TEMPLATE_ID) {
                this.renderWatchExpression(tree, element, templateData);
            }
            else {
                renderVariable(tree, element, templateData, true);
            }
        };
        WatchExpressionsRenderer.prototype.renderWatchExpression = function (tree, watchExpression, data) {
            var selectedExpression = this.debugService.getViewModel().getSelectedExpression();
            if ((selectedExpression instanceof model.Expression && selectedExpression.getId() === watchExpression.getId()) || (watchExpression instanceof model.Expression && !watchExpression.name)) {
                renderRenameBox(this.debugService, this.contextViewService, tree, watchExpression, data.expression, nls.localize(4, null), nls.localize(5, null));
            }
            data.actionBar.context = watchExpression;
            data.name.textContent = watchExpression.name + ":";
            if (watchExpression.value) {
                renderExpressionValue(watchExpression, data.value, true);
            }
        };
        WatchExpressionsRenderer.prototype.disposeTemplate = function (tree, templateId, templateData) {
            // noop
        };
        WatchExpressionsRenderer.prototype.dispose = function () {
            this.toDispose = lifecycle.disposeAll(this.toDispose);
        };
        WatchExpressionsRenderer.WATCH_EXPRESSION_TEMPLATE_ID = 'watchExpression';
        WatchExpressionsRenderer.VARIABLE_TEMPLATE_ID = 'variables';
        WatchExpressionsRenderer = __decorate([
            __param(2, message_1.IMessageService),
            __param(3, debug.IDebugService),
            __param(4, contextView_1.IContextViewService)
        ], WatchExpressionsRenderer);
        return WatchExpressionsRenderer;
    }());
    exports.WatchExpressionsRenderer = WatchExpressionsRenderer;
    var WatchExpressionsAccessibilityProvider = (function () {
        function WatchExpressionsAccessibilityProvider() {
        }
        WatchExpressionsAccessibilityProvider.prototype.getAriaLabel = function (tree, element) {
            if (element instanceof model.Expression) {
                return nls.localize(6, null, element.name, element.value);
            }
            if (element instanceof model.Variable) {
                return nls.localize(7, null, element.name, element.value);
            }
            return null;
        };
        return WatchExpressionsAccessibilityProvider;
    }());
    exports.WatchExpressionsAccessibilityProvider = WatchExpressionsAccessibilityProvider;
    var WatchExpressionsController = (function (_super) {
        __extends(WatchExpressionsController, _super);
        function WatchExpressionsController(debugService, contextMenuService, actionProvider) {
            _super.call(this, debugService, contextMenuService, actionProvider);
            if (platform_1.isMacintosh) {
                this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.ENTER, this.onRename.bind(this));
            }
            else {
                this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.F2, this.onRename.bind(this));
            }
        }
        WatchExpressionsController.prototype.onLeftClick = function (tree, element, event) {
            // double click on primitive value: open input box to be able to select and copy value.
            if (element instanceof model.Expression && event.detail === 2) {
                var expression = element;
                if (expression.reference === 0) {
                    this.debugService.getViewModel().setSelectedExpression(expression);
                }
                return true;
            }
            return _super.prototype.onLeftClick.call(this, tree, element, event);
        };
        WatchExpressionsController.prototype.onRename = function (tree, event) {
            var element = tree.getFocus();
            if (element instanceof model.Expression) {
                var watchExpression = element;
                if (watchExpression.reference === 0) {
                    this.debugService.getViewModel().setSelectedExpression(watchExpression);
                }
                return true;
            }
            return false;
        };
        WatchExpressionsController.prototype.onDelete = function (tree, event) {
            var element = tree.getFocus();
            if (element instanceof model.Expression) {
                var we = element;
                this.debugService.clearWatchExpressions(we.getId());
                return true;
            }
            return false;
        };
        return WatchExpressionsController;
    }(BaseDebugController));
    exports.WatchExpressionsController = WatchExpressionsController;
    // breakpoints
    var BreakpointsActionProvider = (function () {
        function BreakpointsActionProvider(instantiationService) {
            this.instantiationService = instantiationService;
            // noop
        }
        BreakpointsActionProvider.prototype.hasActions = function (tree, element) {
            return element instanceof model.Breakpoint;
        };
        BreakpointsActionProvider.prototype.hasSecondaryActions = function (tree, element) {
            return element instanceof model.Breakpoint || element instanceof model.ExceptionBreakpoint || element instanceof model.FunctionBreakpoint;
        };
        BreakpointsActionProvider.prototype.getActions = function (tree, element) {
            if (element instanceof model.Breakpoint) {
                return winjs_base_1.TPromise.as(this.getBreakpointActions());
            }
            return winjs_base_1.TPromise.as([]);
        };
        BreakpointsActionProvider.prototype.getBreakpointActions = function () {
            return [this.instantiationService.createInstance(debugactions.RemoveBreakpointAction, debugactions.RemoveBreakpointAction.ID, debugactions.RemoveBreakpointAction.LABEL)];
        };
        BreakpointsActionProvider.prototype.getSecondaryActions = function (tree, element) {
            var actions = [this.instantiationService.createInstance(debugactions.ToggleEnablementAction, debugactions.ToggleEnablementAction.ID, debugactions.ToggleEnablementAction.LABEL)];
            actions.push(new actionbar.Separator());
            if (element instanceof model.Breakpoint || element instanceof model.FunctionBreakpoint) {
                actions.push(this.instantiationService.createInstance(debugactions.RemoveBreakpointAction, debugactions.RemoveBreakpointAction.ID, debugactions.RemoveBreakpointAction.LABEL));
            }
            actions.push(this.instantiationService.createInstance(debugactions.RemoveAllBreakpointsAction, debugactions.RemoveAllBreakpointsAction.ID, debugactions.RemoveAllBreakpointsAction.LABEL));
            actions.push(new actionbar.Separator());
            actions.push(this.instantiationService.createInstance(debugactions.ToggleBreakpointsActivatedAction, debugactions.ToggleBreakpointsActivatedAction.ID, debugactions.ToggleBreakpointsActivatedAction.LABEL));
            actions.push(new actionbar.Separator());
            actions.push(this.instantiationService.createInstance(debugactions.EnableAllBreakpointsAction, debugactions.EnableAllBreakpointsAction.ID, debugactions.EnableAllBreakpointsAction.LABEL));
            actions.push(this.instantiationService.createInstance(debugactions.DisableAllBreakpointsAction, debugactions.DisableAllBreakpointsAction.ID, debugactions.DisableAllBreakpointsAction.LABEL));
            actions.push(new actionbar.Separator());
            actions.push(this.instantiationService.createInstance(debugactions.AddFunctionBreakpointAction, debugactions.AddFunctionBreakpointAction.ID, debugactions.AddFunctionBreakpointAction.LABEL));
            if (element instanceof model.FunctionBreakpoint) {
                actions.push(this.instantiationService.createInstance(debugactions.RenameFunctionBreakpointAction, debugactions.RenameFunctionBreakpointAction.ID, debugactions.RenameFunctionBreakpointAction.LABEL));
            }
            actions.push(new actionbar.Separator());
            actions.push(this.instantiationService.createInstance(debugactions.ReapplyBreakpointsAction, debugactions.ReapplyBreakpointsAction.ID, debugactions.ReapplyBreakpointsAction.LABEL));
            return winjs_base_1.TPromise.as(actions);
        };
        BreakpointsActionProvider.prototype.getActionItem = function (tree, element, action) {
            return null;
        };
        return BreakpointsActionProvider;
    }());
    exports.BreakpointsActionProvider = BreakpointsActionProvider;
    var BreakpointsDataSource = (function () {
        function BreakpointsDataSource() {
        }
        BreakpointsDataSource.prototype.getId = function (tree, element) {
            return element.getId();
        };
        BreakpointsDataSource.prototype.hasChildren = function (tree, element) {
            return element instanceof model.Model;
        };
        BreakpointsDataSource.prototype.getChildren = function (tree, element) {
            var model = element;
            var exBreakpoints = model.getExceptionBreakpoints();
            return winjs_base_1.TPromise.as(exBreakpoints.concat(model.getFunctionBreakpoints()).concat(model.getBreakpoints()));
        };
        BreakpointsDataSource.prototype.getParent = function (tree, element) {
            return winjs_base_1.TPromise.as(null);
        };
        return BreakpointsDataSource;
    }());
    exports.BreakpointsDataSource = BreakpointsDataSource;
    var BreakpointsRenderer = (function () {
        function BreakpointsRenderer(actionProvider, actionRunner, messageService, contextService, debugService, contextViewService) {
            this.actionProvider = actionProvider;
            this.actionRunner = actionRunner;
            this.messageService = messageService;
            this.contextService = contextService;
            this.debugService = debugService;
            this.contextViewService = contextViewService;
            // noop
        }
        BreakpointsRenderer.prototype.getHeight = function (tree, element) {
            return 22;
        };
        BreakpointsRenderer.prototype.getTemplateId = function (tree, element) {
            if (element instanceof model.Breakpoint) {
                return BreakpointsRenderer.BREAKPOINT_TEMPLATE_ID;
            }
            if (element instanceof model.FunctionBreakpoint) {
                return BreakpointsRenderer.FUNCTION_BREAKPOINT_TEMPLATE_ID;
            }
            if (element instanceof model.ExceptionBreakpoint) {
                return BreakpointsRenderer.EXCEPTION_BREAKPOINT_TEMPLATE_ID;
            }
            return null;
        };
        BreakpointsRenderer.prototype.renderTemplate = function (tree, templateId, container) {
            var data = Object.create(null);
            if (templateId === BreakpointsRenderer.BREAKPOINT_TEMPLATE_ID || templateId === BreakpointsRenderer.FUNCTION_BREAKPOINT_TEMPLATE_ID) {
                data.actionBar = new actionbar.ActionBar(container, { actionRunner: this.actionRunner });
                data.actionBar.push(this.actionProvider.getBreakpointActions(), { icon: true, label: false });
            }
            data.breakpoint = dom.append(container, $('.breakpoint'));
            data.toDisposeBeforeRender = [];
            data.checkbox = $('input');
            data.checkbox.type = 'checkbox';
            if (!platform_1.isMacintosh) {
                data.checkbox.className = 'checkbox-win-linux';
            }
            dom.append(data.breakpoint, data.checkbox);
            data.name = dom.append(data.breakpoint, $('span.name'));
            if (templateId === BreakpointsRenderer.BREAKPOINT_TEMPLATE_ID) {
                data.lineNumber = dom.append(data.breakpoint, $('span.line-number'));
                data.filePath = dom.append(data.breakpoint, $('span.file-path'));
            }
            return data;
        };
        BreakpointsRenderer.prototype.renderElement = function (tree, element, templateId, templateData) {
            var _this = this;
            templateData.toDisposeBeforeRender = lifecycle.disposeAll(templateData.toDisposeBeforeRender);
            templateData.toDisposeBeforeRender.push(dom.addStandardDisposableListener(templateData.checkbox, 'change', function (e) {
                _this.debugService.toggleEnablement(element);
            }));
            if (templateId === BreakpointsRenderer.EXCEPTION_BREAKPOINT_TEMPLATE_ID) {
                this.renderExceptionBreakpoint(element, templateData);
            }
            else if (templateId === BreakpointsRenderer.FUNCTION_BREAKPOINT_TEMPLATE_ID) {
                this.renderFunctionBreakpoint(tree, element, templateData);
            }
            else {
                this.renderBreakpoint(tree, element, templateData);
            }
        };
        BreakpointsRenderer.prototype.renderExceptionBreakpoint = function (exceptionBreakpoint, data) {
            data.name.textContent = exceptionBreakpoint.label || exceptionBreakpoint.filter + " exceptions";
            ;
            data.checkbox.checked = exceptionBreakpoint.enabled;
        };
        BreakpointsRenderer.prototype.renderFunctionBreakpoint = function (tree, functionBreakpoint, data) {
            var selected = this.debugService.getViewModel().getSelectedFunctionBreakpoint();
            if (!functionBreakpoint.name || (selected && selected.getId() === functionBreakpoint.getId())) {
                renderRenameBox(this.debugService, this.contextViewService, tree, functionBreakpoint, data.breakpoint, nls.localize(8, null), nls.localize(9, null));
            }
            else {
                this.debugService.getModel().areBreakpointsActivated() ? tree.removeTraits('disabled', [functionBreakpoint]) : tree.addTraits('disabled', [functionBreakpoint]);
                data.name.textContent = functionBreakpoint.name;
                data.checkbox.checked = functionBreakpoint.enabled;
            }
            data.actionBar.context = functionBreakpoint;
        };
        BreakpointsRenderer.prototype.renderBreakpoint = function (tree, breakpoint, data) {
            this.debugService.getModel().areBreakpointsActivated() ? tree.removeTraits('disabled', [breakpoint]) : tree.addTraits('disabled', [breakpoint]);
            data.name.textContent = labels.getPathLabel(paths.basename(breakpoint.source.uri.fsPath), this.contextService);
            data.lineNumber.textContent = breakpoint.desiredLineNumber !== breakpoint.lineNumber ? breakpoint.desiredLineNumber + ' \u2192 ' + breakpoint.lineNumber : '' + breakpoint.lineNumber;
            data.filePath.textContent = labels.getPathLabel(paths.dirname(breakpoint.source.uri.fsPath), this.contextService);
            data.checkbox.checked = breakpoint.enabled;
            data.actionBar.context = breakpoint;
            if (breakpoint.condition) {
                data.breakpoint.title = breakpoint.condition;
            }
        };
        BreakpointsRenderer.prototype.disposeTemplate = function (tree, templateId, templateData) {
            if (templateId === BreakpointsRenderer.BREAKPOINT_TEMPLATE_ID || templateId === BreakpointsRenderer.FUNCTION_BREAKPOINT_TEMPLATE_ID) {
                templateData.actionBar.dispose();
            }
        };
        BreakpointsRenderer.EXCEPTION_BREAKPOINT_TEMPLATE_ID = 'exceptionBreakpoint';
        BreakpointsRenderer.FUNCTION_BREAKPOINT_TEMPLATE_ID = 'functionBreakpoint';
        BreakpointsRenderer.BREAKPOINT_TEMPLATE_ID = 'breakpoint';
        BreakpointsRenderer = __decorate([
            __param(2, message_1.IMessageService),
            __param(3, workspace_1.IWorkspaceContextService),
            __param(4, debug.IDebugService),
            __param(5, contextView_1.IContextViewService)
        ], BreakpointsRenderer);
        return BreakpointsRenderer;
    }());
    exports.BreakpointsRenderer = BreakpointsRenderer;
    var BreakpointsAccessibilityProvider = (function () {
        function BreakpointsAccessibilityProvider(contextService) {
            this.contextService = contextService;
            // noop
        }
        BreakpointsAccessibilityProvider.prototype.getAriaLabel = function (tree, element) {
            if (element instanceof model.Breakpoint) {
                return nls.localize(10, null, element.lineNumber, getSourceName(element.source, this.contextService));
            }
            if (element instanceof model.FunctionBreakpoint) {
                return nls.localize(11, null, element.name);
            }
            if (element instanceof model.ExceptionBreakpoint) {
                return nls.localize(12, null, element.filter);
            }
            return null;
        };
        BreakpointsAccessibilityProvider = __decorate([
            __param(0, workspace_1.IWorkspaceContextService)
        ], BreakpointsAccessibilityProvider);
        return BreakpointsAccessibilityProvider;
    }());
    exports.BreakpointsAccessibilityProvider = BreakpointsAccessibilityProvider;
    var BreakpointsController = (function (_super) {
        __extends(BreakpointsController, _super);
        function BreakpointsController() {
            _super.apply(this, arguments);
        }
        BreakpointsController.prototype.onLeftClick = function (tree, element, event) {
            if (element instanceof model.FunctionBreakpoint && event.detail === 2) {
                this.debugService.getViewModel().setSelectedFunctionBreakpoint(element);
                return true;
            }
            return _super.prototype.onLeftClick.call(this, tree, element, event);
        };
        BreakpointsController.prototype.onSpace = function (tree, event) {
            _super.prototype.onSpace.call(this, tree, event);
            var element = tree.getFocus();
            this.debugService.toggleEnablement(element).done(null, errors.onUnexpectedError);
            return true;
        };
        BreakpointsController.prototype.onDelete = function (tree, event) {
            var element = tree.getFocus();
            if (element instanceof model.Breakpoint) {
                var bp = element;
                this.debugService.toggleBreakpoint({ uri: bp.source.uri, lineNumber: bp.lineNumber }).done(null, errors.onUnexpectedError);
                return true;
            }
            else if (element instanceof model.FunctionBreakpoint) {
                var fbp = element;
                this.debugService.removeFunctionBreakpoints(fbp.getId()).done(null, errors.onUnexpectedError);
                return true;
            }
            return false;
        };
        return BreakpointsController;
    }(BaseDebugController));
    exports.BreakpointsController = BreakpointsController;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/debug/browser/debugHover", ["require", "exports", 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/base/common/errors', 'vs/base/common/keyCodes', 'vs/base/browser/dom', 'vs/nls!vs/workbench/parts/debug/browser/debugHover', 'vs/base/parts/tree/browser/treeImpl', 'vs/base/parts/tree/browser/treeDefaults', 'vs/editor/browser/editorBrowser', 'vs/workbench/parts/debug/common/debugModel', 'vs/workbench/parts/debug/browser/debugViewer'], function (require, exports, lifecycle, winjs_base_1, errors, keyCodes_1, dom, nls, treeImpl_1, treeDefaults_1, editorbrowser, debugModel_1, viewer) {
    "use strict";
    var $ = dom.emmet;
    var debugTreeOptions = {
        indentPixels: 6,
        twistiePixels: 15,
        ariaLabel: nls.localize(0, null)
    };
    var MAX_ELEMENTS_SHOWN = 18;
    var DebugHoverWidget = (function () {
        function DebugHoverWidget(editor, debugService, instantiationService) {
            var _this = this;
            this.editor = editor;
            this.debugService = debugService;
            this.instantiationService = instantiationService;
            // editor.IContentWidget.allowEditorOverflow
            this.allowEditorOverflow = true;
            this.domNode = $('.debug-hover-widget monaco-editor-background');
            this.treeContainer = dom.append(this.domNode, $('.debug-hover-tree'));
            this.treeContainer.setAttribute('role', 'tree');
            this.tree = new treeImpl_1.Tree(this.treeContainer, {
                dataSource: new viewer.VariablesDataSource(this.debugService),
                renderer: this.instantiationService.createInstance(VariablesHoverRenderer),
                controller: new DebugHoverController(editor)
            }, debugTreeOptions);
            this.toDispose = [];
            this.toDispose.push(this.tree.addListener2('item:expanded', function () {
                _this.layoutTree();
            }));
            this.toDispose.push(this.tree.addListener2('item:collapsed', function () {
                _this.layoutTree();
            }));
            this.toDispose.push(dom.addStandardDisposableListener(this.domNode, 'keydown', function (e) {
                if (e.equals(keyCodes_1.CommonKeybindings.ESCAPE)) {
                    _this.hide();
                }
            }));
            this.valueContainer = dom.append(this.domNode, $('.value'));
            this.valueContainer.tabIndex = 0;
            this.valueContainer.setAttribute('role', 'tooltip');
            this.isVisible = false;
            this.showAtPosition = null;
            this.highlightDecorations = [];
            this.editor.addContentWidget(this);
        }
        DebugHoverWidget.prototype.getId = function () {
            return DebugHoverWidget.ID;
        };
        DebugHoverWidget.prototype.getDomNode = function () {
            return this.domNode;
        };
        DebugHoverWidget.prototype.showAt = function (range, hoveringOver, focus) {
            var _this = this;
            var pos = range.getStartPosition();
            var model = this.editor.getModel();
            var focusedStackFrame = this.debugService.getViewModel().getFocusedStackFrame();
            if (!hoveringOver || !focusedStackFrame || (focusedStackFrame.source.uri.toString() !== model.getAssociatedResource().toString())) {
                return;
            }
            // string magic to get the parents of the variable (a and b for a.b.foo)
            var lineContent = model.getLineContent(pos.lineNumber);
            var namesToFind = lineContent.substring(0, lineContent.indexOf('.' + hoveringOver))
                .split('.').map(function (word) { return word.trim(); }).filter(function (word) { return !!word; });
            namesToFind.push(hoveringOver);
            namesToFind[0] = namesToFind[0].substring(namesToFind[0].lastIndexOf(' ') + 1);
            return this.getExpression(namesToFind).then(function (expression) {
                if (!expression || !expression.available) {
                    _this.hide();
                    return;
                }
                // show it
                _this.highlightDecorations = _this.editor.deltaDecorations(_this.highlightDecorations, [{
                        range: {
                            startLineNumber: pos.lineNumber,
                            endLineNumber: pos.lineNumber,
                            startColumn: lineContent.indexOf(hoveringOver) + 1,
                            endColumn: lineContent.indexOf(hoveringOver) + 1 + hoveringOver.length
                        },
                        options: {
                            className: 'hoverHighlight'
                        }
                    }]);
                return _this.doShow(pos, expression, focus);
            });
        };
        DebugHoverWidget.prototype.getExpression = function (namesToFind) {
            var _this = this;
            var session = this.debugService.getActiveSession();
            var focusedStackFrame = this.debugService.getViewModel().getFocusedStackFrame();
            if (session.capabilities.supportsEvaluateForHovers) {
                return debugModel_1.evaluateExpression(session, focusedStackFrame, new debugModel_1.Expression(namesToFind.join('.'), true), 'hover');
            }
            var variables = [];
            return focusedStackFrame.getScopes(this.debugService).then(function (scopes) {
                // flatten out scopes lists
                return scopes.reduce(function (accum, scopes) { return accum.concat(scopes); }, [])
                    .filter(function (scope) { return !scope.expensive; })
                    .map(function (scope) { return scope.getChildren(_this.debugService).done(function (children) {
                    // look for our variable in the list. First find the parents of the hovered variable if there are any.
                    for (var i = 0; i < namesToFind.length && children; i++) {
                        // some languages pass the type as part of the name, so need to check if the last word of the name matches.
                        var filtered = children.filter(function (v) { return typeof v.name === 'string' && (namesToFind[i] === v.name || namesToFind[i] === v.name.substr(v.name.lastIndexOf(' ') + 1)); });
                        if (filtered.length !== 1) {
                            break;
                        }
                        if (i === namesToFind.length - 1) {
                            variables.push(filtered[0]);
                        }
                        else {
                            filtered[0].getChildren(_this.debugService).done(function (c) { return children = c; }, children = null);
                        }
                    }
                }, errors.onUnexpectedError); });
                // only show if there are no duplicates across scopes
            }).then(function () { return variables.length === 1 ? winjs_base_1.TPromise.as(variables[0]) : winjs_base_1.TPromise.as(null); });
        };
        DebugHoverWidget.prototype.doShow = function (position, expression, focus, forceValueHover) {
            var _this = this;
            if (forceValueHover === void 0) { forceValueHover = false; }
            this.showAtPosition = position;
            this.isVisible = true;
            this.stoleFocus = focus;
            if (expression.reference === 0 || forceValueHover) {
                this.treeContainer.hidden = true;
                this.valueContainer.hidden = false;
                viewer.renderExpressionValue(expression, this.valueContainer, false);
                this.valueContainer.title = '';
                this.editor.layoutContentWidget(this);
                if (focus) {
                    this.editor.render();
                    this.valueContainer.focus();
                }
                return winjs_base_1.TPromise.as(null);
            }
            this.valueContainer.hidden = true;
            this.treeContainer.hidden = false;
            return this.tree.setInput(expression).then(function () {
                _this.layoutTree();
                _this.editor.layoutContentWidget(_this);
                if (focus) {
                    _this.editor.render();
                    _this.tree.DOMFocus();
                }
            });
        };
        DebugHoverWidget.prototype.layoutTree = function () {
            var navigator = this.tree.getNavigator();
            var visibleElementsCount = 0;
            while (navigator.next()) {
                visibleElementsCount++;
            }
            if (visibleElementsCount === 0) {
                this.doShow(this.showAtPosition, this.tree.getInput(), false, true);
            }
            else {
                var height = Math.min(visibleElementsCount, MAX_ELEMENTS_SHOWN) * 18;
                if (this.treeContainer.clientHeight !== height) {
                    this.treeContainer.style.height = height + "px";
                    this.tree.layout();
                }
            }
        };
        DebugHoverWidget.prototype.hide = function () {
            if (!this.isVisible) {
                // already not visible
                return;
            }
            this.isVisible = false;
            this.editor.deltaDecorations(this.highlightDecorations, []);
            this.highlightDecorations = [];
            this.editor.layoutContentWidget(this);
            if (this.stoleFocus) {
                this.editor.focus();
            }
        };
        DebugHoverWidget.prototype.getPosition = function () {
            return this.isVisible ? {
                position: this.showAtPosition,
                preference: [
                    editorbrowser.ContentWidgetPositionPreference.ABOVE,
                    editorbrowser.ContentWidgetPositionPreference.BELOW
                ]
            } : null;
        };
        DebugHoverWidget.prototype.dispose = function () {
            this.toDispose = lifecycle.disposeAll(this.toDispose);
        };
        DebugHoverWidget.ID = 'debug.hoverWidget';
        return DebugHoverWidget;
    }());
    exports.DebugHoverWidget = DebugHoverWidget;
    var DebugHoverController = (function (_super) {
        __extends(DebugHoverController, _super);
        function DebugHoverController(editor) {
            _super.call(this);
            this.editor = editor;
        }
        /* protected */ DebugHoverController.prototype.onLeftClick = function (tree, element, eventish, origin) {
            if (origin === void 0) { origin = 'mouse'; }
            if (element.reference > 0) {
                _super.prototype.onLeftClick.call(this, tree, element, eventish, origin);
                tree.clearFocus();
                tree.deselect(element);
                this.editor.focus();
            }
            return true;
        };
        return DebugHoverController;
    }(treeDefaults_1.DefaultController));
    var VariablesHoverRenderer = (function (_super) {
        __extends(VariablesHoverRenderer, _super);
        function VariablesHoverRenderer() {
            _super.apply(this, arguments);
        }
        VariablesHoverRenderer.prototype.getHeight = function (tree, element) {
            return 18;
        };
        return VariablesHoverRenderer;
    }(viewer.VariablesRenderer));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/search/browser/search.contribution", ["require", "exports", 'vs/platform/platform', 'vs/workbench/browser/viewlet', 'vs/platform/configuration/common/configurationRegistry', 'vs/nls!vs/workbench/parts/search/browser/search.contribution', 'vs/workbench/parts/files/common/files', 'vs/platform/actions/common/actions', 'vs/base/browser/ui/actionbar/actionbar', 'vs/workbench/browser/actionBarRegistry', 'vs/workbench/common/actionRegistry', 'vs/workbench/browser/quickopen', 'vs/workbench/browser/actions/quickOpenAction', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/platform/instantiation/common/instantiation', 'vs/platform/instantiation/common/descriptors', 'vs/platform/workspace/common/workspace', 'vs/platform/keybinding/common/keybindingService', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/editor/common/editorService', 'vs/base/common/keyCodes', 'vs/css!./media/search.contribution'], function (require, exports, platform_1, viewlet_1, configurationRegistry_1, nls, files_1, actions_1, actionbar_1, actionBarRegistry_1, actionRegistry_1, quickopen_1, quickOpenAction_1, keybindingsRegistry_1, instantiation_1, descriptors_1, workspace_1, keybindingService_1, quickOpenService_1, viewletService_1, editorService_1, keyCodes_1) {
    'use strict';
    exports.VIEWLET_ID = 'workbench.view.search';
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
        id: 'workbench.action.search.toggleQueryDetails',
        weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.workbenchContrib(),
        context: keybindingService_1.KbExpr.has('searchViewletVisible'),
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_J,
        handler: function (accessor) {
            var viewletService = accessor.get(viewletService_1.IViewletService);
            viewletService.openViewlet(exports.VIEWLET_ID, true)
                .then(function (viewlet) { return viewlet.toggleFileTypes(); });
        }
    });
    var OpenSearchViewletAction = (function (_super) {
        __extends(OpenSearchViewletAction, _super);
        function OpenSearchViewletAction(id, label, viewletService, editorService) {
            _super.call(this, id, label, exports.VIEWLET_ID, viewletService, editorService);
        }
        OpenSearchViewletAction.ID = exports.VIEWLET_ID;
        OpenSearchViewletAction.LABEL = nls.localize(0, null);
        OpenSearchViewletAction = __decorate([
            __param(2, viewletService_1.IViewletService),
            __param(3, editorService_1.IWorkbenchEditorService)
        ], OpenSearchViewletAction);
        return OpenSearchViewletAction;
    }(viewlet_1.ToggleViewletAction));
    var ExplorerViewerActionContributor = (function (_super) {
        __extends(ExplorerViewerActionContributor, _super);
        function ExplorerViewerActionContributor(instantiationService, contextService) {
            _super.call(this);
            this._instantiationService = instantiationService;
            this._contextService = contextService;
        }
        ExplorerViewerActionContributor.prototype.hasSecondaryActions = function (context) {
            var element = context.element;
            // Contribute only on file resources
            var fileResource = files_1.asFileResource(element);
            if (!fileResource) {
                return false;
            }
            return fileResource.isDirectory;
        };
        ExplorerViewerActionContributor.prototype.getSecondaryActions = function (context) {
            var actions = [];
            if (this.hasSecondaryActions(context)) {
                var fileResource = files_1.asFileResource(context.element);
                var action = new actions_1.DeferredAction(this._instantiationService, new descriptors_1.AsyncDescriptor('vs/workbench/parts/search/browser/searchViewlet', 'FindInFolderAction', fileResource.resource), 'workbench.search.action.findInFolder', nls.localize(1, null));
                action.order = 55;
                actions.push(action);
                actions.push(new actionbar_1.Separator('', 56));
            }
            return actions;
        };
        ExplorerViewerActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, workspace_1.IWorkspaceContextService)
        ], ExplorerViewerActionContributor);
        return ExplorerViewerActionContributor;
    }(actionBarRegistry_1.ActionBarContributor));
    var ACTION_ID = 'workbench.action.showAllSymbols';
    var ACTION_LABEL = nls.localize(2, null);
    var ALL_SYMBOLS_PREFIX = '#';
    var ShowAllSymbolsAction = (function (_super) {
        __extends(ShowAllSymbolsAction, _super);
        function ShowAllSymbolsAction(actionId, actionLabel, quickOpenService) {
            _super.call(this, actionId, actionLabel, ALL_SYMBOLS_PREFIX, quickOpenService);
        }
        ShowAllSymbolsAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], ShowAllSymbolsAction);
        return ShowAllSymbolsAction;
    }(quickOpenAction_1.QuickOpenAction));
    // Register Viewlet
    platform_1.Registry.as(viewlet_1.Extensions.Viewlets).registerViewlet(new viewlet_1.ViewletDescriptor('vs/workbench/parts/search/browser/searchViewlet', 'SearchViewlet', exports.VIEWLET_ID, nls.localize(3, null), 'search', 10));
    // Register Action to Open Viewlet
    var openSearchViewletKb = {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_F
    };
    platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions).registerWorkbenchAction(new actions_1.SyncActionDescriptor(OpenSearchViewletAction, OpenSearchViewletAction.ID, OpenSearchViewletAction.LABEL, openSearchViewletKb), nls.localize(4, null));
    // Contribute to Explorer Viewer
    var actionBarRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
    actionBarRegistry.registerActionBarContributor(actionBarRegistry_1.Scope.VIEWER, ExplorerViewerActionContributor);
    // Register Quick Open Handler
    platform_1.Registry.as(quickopen_1.Extensions.Quickopen).registerDefaultQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/search/browser/openAnythingHandler', 'OpenAnythingHandler', '', nls.localize(5, null)));
    platform_1.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/search/browser/openAnythingHandler', 'OpenSymbolHandler', ALL_SYMBOLS_PREFIX, [
        {
            prefix: ALL_SYMBOLS_PREFIX,
            needsEditor: false,
            description: nls.localize(6, null)
        }
    ]));
    // Actions
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(ShowAllSymbolsAction, ACTION_ID, ACTION_LABEL, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_T
    }));
    // Configuration
    var configurationRegistry = platform_1.Registry.as(configurationRegistry_1.Extensions.Configuration);
    configurationRegistry.registerConfiguration({
        'id': 'search',
        'order': 10,
        'title': nls.localize(7, null),
        'type': 'object',
        'properties': {
            'search.exclude': {
                'type': 'object',
                'description': nls.localize(8, null),
                'default': { '**/node_modules': true, '**/bower_components': true },
                'additionalProperties': {
                    'anyOf': [
                        {
                            'type': 'boolean',
                            'description': nls.localize(9, null),
                        },
                        {
                            'type': 'object',
                            'properties': {
                                'when': {
                                    'type': 'string',
                                    'pattern': '\\w*\\$\\(basename\\)\\w*',
                                    'default': '$(basename).ext',
                                    'description': nls.localize(10, null)
                                }
                            }
                        }
                    ]
                }
            }
        }
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/window/electron-browser/windowService", ["require", "exports", 'vs/platform/instantiation/common/instantiation', 'vs/base/common/event', 'electron'], function (require, exports, instantiation_1, event_1, electron_1) {
    'use strict';
    var windowId = electron_1.remote.getCurrentWindow().id;
    exports.IWindowService = instantiation_1.createDecorator('windowService');
    var WindowService = (function () {
        function WindowService() {
            this.serviceId = exports.IWindowService;
            this._onBroadcast = new event_1.Emitter();
            this.windowId = windowId;
            this.registerListeners();
        }
        WindowService.prototype.registerListeners = function () {
            var _this = this;
            electron_1.ipcRenderer.on('vscode:broadcast', function (event, b) {
                _this._onBroadcast.fire(b);
            });
        };
        Object.defineProperty(WindowService.prototype, "onBroadcast", {
            get: function () {
                return this._onBroadcast.event;
            },
            enumerable: true,
            configurable: true
        });
        WindowService.prototype.getWindowId = function () {
            return this.windowId;
        };
        WindowService.prototype.getWindow = function () {
            return this.win;
        };
        WindowService.prototype.registerWindow = function (win) {
            this.win = win;
        };
        WindowService.prototype.broadcast = function (b, target) {
            electron_1.ipcRenderer.send('vscode:broadcast', this.getWindowId(), target, {
                channel: b.channel,
                payload: b.payload
            });
        };
        return WindowService;
    }());
    exports.WindowService = WindowService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/electron-browser/actions", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/timer', 'vs/base/common/paths', 'vs/base/common/actions', 'vs/workbench/services/window/electron-browser/windowService', 'vs/workbench/services/editor/common/editorService', 'vs/nls!vs/workbench/electron-browser/actions', 'vs/platform/message/common/message', 'vs/platform/workspace/common/workspace', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/platform/configuration/common/configuration', 'electron'], function (require, exports, winjs_base_1, timer, paths, actions_1, windowService_1, editorService_1, nls, message_1, workspace_1, quickOpenService_1, configuration_1, electron_1) {
    'use strict';
    var CloseEditorAction = (function (_super) {
        __extends(CloseEditorAction, _super);
        function CloseEditorAction(id, label, editorService, windowService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.windowService = windowService;
        }
        CloseEditorAction.prototype.run = function () {
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor) {
                return this.editorService.closeEditor(activeEditor);
            }
            this.windowService.getWindow().close();
            return winjs_base_1.TPromise.as(false);
        };
        CloseEditorAction.ID = 'workbench.action.closeActiveEditor';
        CloseEditorAction.LABEL = nls.localize(0, null);
        CloseEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, windowService_1.IWindowService)
        ], CloseEditorAction);
        return CloseEditorAction;
    }(actions_1.Action));
    exports.CloseEditorAction = CloseEditorAction;
    var CloseWindowAction = (function (_super) {
        __extends(CloseWindowAction, _super);
        function CloseWindowAction(id, label, windowService) {
            _super.call(this, id, label);
            this.windowService = windowService;
        }
        CloseWindowAction.prototype.run = function () {
            this.windowService.getWindow().close();
            return winjs_base_1.TPromise.as(true);
        };
        CloseWindowAction.ID = 'workbench.action.closeWindow';
        CloseWindowAction.LABEL = nls.localize(1, null);
        CloseWindowAction = __decorate([
            __param(2, windowService_1.IWindowService)
        ], CloseWindowAction);
        return CloseWindowAction;
    }(actions_1.Action));
    exports.CloseWindowAction = CloseWindowAction;
    var CloseFolderAction = (function (_super) {
        __extends(CloseFolderAction, _super);
        function CloseFolderAction(id, label, contextService, messageService, windowService) {
            _super.call(this, id, label);
            this.contextService = contextService;
            this.messageService = messageService;
            this.windowService = windowService;
        }
        CloseFolderAction.prototype.run = function () {
            if (this.contextService.getWorkspace()) {
                electron_1.ipcRenderer.send('vscode:closeFolder', this.windowService.getWindowId()); // handled from browser process
            }
            else {
                this.messageService.show(message_1.Severity.Info, nls.localize(3, null));
            }
            return winjs_base_1.TPromise.as(true);
        };
        CloseFolderAction.ID = 'workbench.action.closeFolder';
        CloseFolderAction.LABEL = nls.localize(2, null);
        CloseFolderAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, message_1.IMessageService),
            __param(4, windowService_1.IWindowService)
        ], CloseFolderAction);
        return CloseFolderAction;
    }(actions_1.Action));
    exports.CloseFolderAction = CloseFolderAction;
    var NewWindowAction = (function (_super) {
        __extends(NewWindowAction, _super);
        function NewWindowAction(id, label, windowService) {
            _super.call(this, id, label);
            this.windowService = windowService;
        }
        NewWindowAction.prototype.run = function () {
            this.windowService.getWindow().openNew();
            return winjs_base_1.TPromise.as(true);
        };
        NewWindowAction.ID = 'workbench.action.newWindow';
        NewWindowAction.LABEL = nls.localize(4, null);
        NewWindowAction = __decorate([
            __param(2, windowService_1.IWindowService)
        ], NewWindowAction);
        return NewWindowAction;
    }(actions_1.Action));
    exports.NewWindowAction = NewWindowAction;
    var ToggleFullScreenAction = (function (_super) {
        __extends(ToggleFullScreenAction, _super);
        function ToggleFullScreenAction(id, label, windowService) {
            _super.call(this, id, label);
            this.windowService = windowService;
        }
        ToggleFullScreenAction.prototype.run = function () {
            electron_1.ipcRenderer.send('vscode:toggleFullScreen', this.windowService.getWindowId());
            return winjs_base_1.TPromise.as(true);
        };
        ToggleFullScreenAction.ID = 'workbench.action.toggleFullScreen';
        ToggleFullScreenAction.LABEL = nls.localize(5, null);
        ToggleFullScreenAction = __decorate([
            __param(2, windowService_1.IWindowService)
        ], ToggleFullScreenAction);
        return ToggleFullScreenAction;
    }(actions_1.Action));
    exports.ToggleFullScreenAction = ToggleFullScreenAction;
    var ToggleMenuBarAction = (function (_super) {
        __extends(ToggleMenuBarAction, _super);
        function ToggleMenuBarAction(id, label, windowService) {
            _super.call(this, id, label);
            this.windowService = windowService;
        }
        ToggleMenuBarAction.prototype.run = function () {
            electron_1.ipcRenderer.send('vscode:toggleMenuBar', this.windowService.getWindowId());
            return winjs_base_1.TPromise.as(true);
        };
        ToggleMenuBarAction.ID = 'workbench.action.toggleMenuBar';
        ToggleMenuBarAction.LABEL = nls.localize(6, null);
        ToggleMenuBarAction = __decorate([
            __param(2, windowService_1.IWindowService)
        ], ToggleMenuBarAction);
        return ToggleMenuBarAction;
    }(actions_1.Action));
    exports.ToggleMenuBarAction = ToggleMenuBarAction;
    var ToggleDevToolsAction = (function (_super) {
        __extends(ToggleDevToolsAction, _super);
        function ToggleDevToolsAction(id, label, windowService) {
            _super.call(this, id, label);
            this.windowService = windowService;
        }
        ToggleDevToolsAction.prototype.run = function () {
            electron_1.ipcRenderer.send('vscode:toggleDevTools', this.windowService.getWindowId());
            return winjs_base_1.TPromise.as(true);
        };
        ToggleDevToolsAction.ID = 'workbench.action.toggleDevTools';
        ToggleDevToolsAction.LABEL = nls.localize(7, null);
        ToggleDevToolsAction = __decorate([
            __param(2, windowService_1.IWindowService)
        ], ToggleDevToolsAction);
        return ToggleDevToolsAction;
    }(actions_1.Action));
    exports.ToggleDevToolsAction = ToggleDevToolsAction;
    var ZoomInAction = (function (_super) {
        __extends(ZoomInAction, _super);
        function ZoomInAction(id, label) {
            _super.call(this, id, label);
        }
        ZoomInAction.prototype.run = function () {
            electron_1.webFrame.setZoomLevel(electron_1.webFrame.getZoomLevel() + 1);
            return winjs_base_1.TPromise.as(true);
        };
        ZoomInAction.ID = 'workbench.action.zoomIn';
        ZoomInAction.LABEL = nls.localize(8, null);
        return ZoomInAction;
    }(actions_1.Action));
    exports.ZoomInAction = ZoomInAction;
    var BaseZoomAction = (function (_super) {
        __extends(BaseZoomAction, _super);
        function BaseZoomAction(id, label, configurationService) {
            _super.call(this, id, label);
            this.configurationService = configurationService;
        }
        BaseZoomAction.prototype.run = function () {
            return winjs_base_1.TPromise.as(false); // Subclass to implement
        };
        BaseZoomAction.prototype.loadConfiguredZoomLevel = function () {
            return this.configurationService.loadConfiguration().then(function (windowConfig) {
                if (windowConfig.window && typeof windowConfig.window.zoomLevel === 'number') {
                    return windowConfig.window.zoomLevel;
                }
                return 0; // default
            });
        };
        BaseZoomAction = __decorate([
            __param(2, configuration_1.IConfigurationService)
        ], BaseZoomAction);
        return BaseZoomAction;
    }(actions_1.Action));
    exports.BaseZoomAction = BaseZoomAction;
    var ZoomOutAction = (function (_super) {
        __extends(ZoomOutAction, _super);
        function ZoomOutAction(id, label, configurationService) {
            _super.call(this, id, label, configurationService);
        }
        ZoomOutAction.prototype.run = function () {
            return this.loadConfiguredZoomLevel().then(function (level) {
                var newZoomLevelCandiate = electron_1.webFrame.getZoomLevel() - 1;
                if (newZoomLevelCandiate < 0 && newZoomLevelCandiate < level) {
                    newZoomLevelCandiate = Math.min(level, 0); // do not zoom below configured level or below 0
                }
                electron_1.webFrame.setZoomLevel(newZoomLevelCandiate);
                return true;
            });
        };
        ZoomOutAction.ID = 'workbench.action.zoomOut';
        ZoomOutAction.LABEL = nls.localize(9, null);
        ZoomOutAction = __decorate([
            __param(2, configuration_1.IConfigurationService)
        ], ZoomOutAction);
        return ZoomOutAction;
    }(BaseZoomAction));
    exports.ZoomOutAction = ZoomOutAction;
    var ZoomResetAction = (function (_super) {
        __extends(ZoomResetAction, _super);
        function ZoomResetAction(id, label, configurationService) {
            _super.call(this, id, label, configurationService);
        }
        ZoomResetAction.prototype.run = function () {
            return this.loadConfiguredZoomLevel().then(function (level) {
                electron_1.webFrame.setZoomLevel(level);
                return true;
            });
        };
        ZoomResetAction.ID = 'workbench.action.zoomReset';
        ZoomResetAction.LABEL = nls.localize(10, null);
        ZoomResetAction = __decorate([
            __param(2, configuration_1.IConfigurationService)
        ], ZoomResetAction);
        return ZoomResetAction;
    }(BaseZoomAction));
    exports.ZoomResetAction = ZoomResetAction;
    /* Copied from loader.ts */
    var LoaderEventType;
    (function (LoaderEventType) {
        LoaderEventType[LoaderEventType["LoaderAvailable"] = 1] = "LoaderAvailable";
        LoaderEventType[LoaderEventType["BeginLoadingScript"] = 10] = "BeginLoadingScript";
        LoaderEventType[LoaderEventType["EndLoadingScriptOK"] = 11] = "EndLoadingScriptOK";
        LoaderEventType[LoaderEventType["EndLoadingScriptError"] = 12] = "EndLoadingScriptError";
        LoaderEventType[LoaderEventType["BeginInvokeFactory"] = 21] = "BeginInvokeFactory";
        LoaderEventType[LoaderEventType["EndInvokeFactory"] = 22] = "EndInvokeFactory";
        LoaderEventType[LoaderEventType["NodeBeginEvaluatingScript"] = 31] = "NodeBeginEvaluatingScript";
        LoaderEventType[LoaderEventType["NodeEndEvaluatingScript"] = 32] = "NodeEndEvaluatingScript";
        LoaderEventType[LoaderEventType["NodeBeginNativeRequire"] = 33] = "NodeBeginNativeRequire";
        LoaderEventType[LoaderEventType["NodeEndNativeRequire"] = 34] = "NodeEndNativeRequire";
    })(LoaderEventType || (LoaderEventType = {}));
    var ShowStartupPerformance = (function (_super) {
        __extends(ShowStartupPerformance, _super);
        function ShowStartupPerformance(id, label, windowService, contextService) {
            _super.call(this, id, label);
            this.windowService = windowService;
            this.contextService = contextService;
            this.enabled = contextService.getConfiguration().env.enablePerformance;
        }
        ShowStartupPerformance.prototype._analyzeLoaderTimes = function () {
            var stats = require.getStats();
            var result = [];
            var total = 0;
            for (var i = 0, len = stats.length; i < len; i++) {
                if (stats[i].type === LoaderEventType.NodeEndNativeRequire) {
                    if (stats[i - 1].type === LoaderEventType.NodeBeginNativeRequire && stats[i - 1].detail === stats[i].detail) {
                        var entry = {};
                        entry['Event'] = 'nodeRequire ' + stats[i].detail;
                        entry['Took (ms)'] = (stats[i].timestamp - stats[i - 1].timestamp);
                        total += (stats[i].timestamp - stats[i - 1].timestamp);
                        entry['Start (ms)'] = '**' + stats[i - 1].timestamp;
                        entry['End (ms)'] = '**' + stats[i - 1].timestamp;
                        result.push(entry);
                    }
                }
            }
            if (total > 0) {
                var entry = {};
                entry['Event'] = '===nodeRequire TOTAL';
                entry['Took (ms)'] = total;
                entry['Start (ms)'] = '**';
                entry['End (ms)'] = '**';
                result.push(entry);
            }
            return result;
        };
        ShowStartupPerformance.prototype.run = function () {
            var table = [];
            table.push.apply(table, this._analyzeLoaderTimes());
            var start = Math.round(electron_1.remote.getGlobal('programStart') || electron_1.remote.getGlobal('vscodeStart'));
            var windowShowTime = Math.round(electron_1.remote.getGlobal('windowShow'));
            var lastEvent;
            var events = timer.getTimeKeeper().getCollectedEvents();
            events.forEach(function (e) {
                if (e.topic === 'Startup') {
                    lastEvent = e;
                    var entry = {};
                    entry['Event'] = e.name;
                    entry['Took (ms)'] = e.stopTime.getTime() - e.startTime.getTime();
                    entry['Start (ms)'] = Math.max(e.startTime.getTime() - start, 0);
                    entry['End (ms)'] = e.stopTime.getTime() - start;
                    table.push(entry);
                }
            });
            table.push({ Event: '---------------------------' });
            var windowShowEvent = {};
            windowShowEvent['Event'] = 'Show Window at';
            windowShowEvent['Start (ms)'] = windowShowTime - start;
            table.push(windowShowEvent);
            var sum = {};
            sum['Event'] = 'Total';
            sum['Took (ms)'] = lastEvent.stopTime.getTime() - start;
            table.push(sum);
            // Show dev tools
            this.windowService.getWindow().openDevTools();
            // Print to console
            setTimeout(function () {
                console.warn('Run the action again if you do not see the numbers!');
                console.table(table);
            }, 1000);
            return winjs_base_1.TPromise.as(true);
        };
        ShowStartupPerformance.ID = 'workbench.action.appPerf';
        ShowStartupPerformance.LABEL = nls.localize(11, null);
        ShowStartupPerformance = __decorate([
            __param(2, windowService_1.IWindowService),
            __param(3, workspace_1.IWorkspaceContextService)
        ], ShowStartupPerformance);
        return ShowStartupPerformance;
    }(actions_1.Action));
    exports.ShowStartupPerformance = ShowStartupPerformance;
    var ReloadWindowAction = (function (_super) {
        __extends(ReloadWindowAction, _super);
        function ReloadWindowAction(id, label, windowService) {
            _super.call(this, id, label);
            this.windowService = windowService;
        }
        ReloadWindowAction.prototype.run = function () {
            this.windowService.getWindow().reload();
            return winjs_base_1.TPromise.as(true);
        };
        ReloadWindowAction.ID = 'workbench.action.reloadWindow';
        ReloadWindowAction.LABEL = nls.localize(12, null);
        ReloadWindowAction = __decorate([
            __param(2, windowService_1.IWindowService)
        ], ReloadWindowAction);
        return ReloadWindowAction;
    }(actions_1.Action));
    exports.ReloadWindowAction = ReloadWindowAction;
    var OpenRecentAction = (function (_super) {
        __extends(OpenRecentAction, _super);
        function OpenRecentAction(id, label, contextService, quickOpenService) {
            _super.call(this, id, label);
            this.contextService = contextService;
            this.quickOpenService = quickOpenService;
        }
        OpenRecentAction.prototype.run = function () {
            var picks = this.contextService.getConfiguration().env.recentPaths.map(function (p) {
                return {
                    label: paths.basename(p),
                    description: paths.dirname(p),
                    path: p
                };
            });
            var hasWorkspace = !!this.contextService.getWorkspace();
            return this.quickOpenService.pick(picks, {
                autoFocus: { autoFocusFirstEntry: !hasWorkspace, autoFocusSecondEntry: hasWorkspace },
                placeHolder: nls.localize(14, null),
                matchOnDescription: true
            }).then(function (p) {
                if (p) {
                    electron_1.ipcRenderer.send('vscode:windowOpen', [p.path]);
                }
                return true;
            });
        };
        OpenRecentAction.ID = 'workbench.action.openRecent';
        OpenRecentAction.LABEL = nls.localize(13, null);
        OpenRecentAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], OpenRecentAction);
        return OpenRecentAction;
    }(actions_1.Action));
    exports.OpenRecentAction = OpenRecentAction;
    var CloseMessagesAction = (function (_super) {
        __extends(CloseMessagesAction, _super);
        function CloseMessagesAction(id, label, messageService, editorService) {
            _super.call(this, id, label);
            this.messageService = messageService;
            this.editorService = editorService;
        }
        CloseMessagesAction.prototype.run = function () {
            // Close any Message if visible
            this.messageService.hideAll();
            // Restore focus if we got an editor
            var editor = this.editorService.getActiveEditor();
            if (editor) {
                editor.focus();
            }
            return winjs_base_1.TPromise.as(true);
        };
        CloseMessagesAction.ID = 'workbench.action.closeMessages';
        CloseMessagesAction.LABEL = nls.localize(15, null);
        CloseMessagesAction = __decorate([
            __param(2, message_1.IMessageService),
            __param(3, editorService_1.IWorkbenchEditorService)
        ], CloseMessagesAction);
        return CloseMessagesAction;
    }(actions_1.Action));
    exports.CloseMessagesAction = CloseMessagesAction;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/electron-browser/main.contribution", ["require", "exports", 'vs/platform/platform', 'vs/nls!vs/workbench/electron-browser/main.contribution', 'vs/platform/actions/common/actions', 'vs/platform/configuration/common/configurationRegistry', 'vs/workbench/common/actionRegistry', 'vs/base/common/keyCodes', 'vs/base/common/platform', 'vs/platform/keybinding/common/keybindingService', 'vs/workbench/services/message/browser/messageService', 'vs/workbench/electron-browser/actions'], function (require, exports, platform_1, nls, actions_1, configurationRegistry_1, actionRegistry_1, keyCodes_1, platform, keybindingService_1, messageService_1, actions_2) {
    'use strict';
    // Contribute Global Actions
    var viewCategory = nls.localize(0, null);
    var developerCategory = nls.localize(1, null);
    var fileCategory = nls.localize(2, null);
    var workbenchActionsRegistry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.NewWindowAction, actions_2.NewWindowAction.ID, actions_2.NewWindowAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_N }));
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.CloseWindowAction, actions_2.CloseWindowAction.ID, actions_2.CloseWindowAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_W }));
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.CloseFolderAction, actions_2.CloseFolderAction.ID, actions_2.CloseFolderAction.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.KEY_F) }), fileCategory);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.OpenRecentAction, actions_2.OpenRecentAction.ID, actions_2.OpenRecentAction.LABEL), fileCategory);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.ToggleDevToolsAction, actions_2.ToggleDevToolsAction.ID, actions_2.ToggleDevToolsAction.LABEL), developerCategory);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.ZoomInAction, actions_2.ZoomInAction.ID, actions_2.ZoomInAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.US_EQUAL }), viewCategory);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.ZoomOutAction, actions_2.ZoomOutAction.ID, actions_2.ZoomOutAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.US_MINUS }), viewCategory);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.ZoomResetAction, actions_2.ZoomResetAction.ID, actions_2.ZoomResetAction.LABEL), viewCategory);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.ShowStartupPerformance, actions_2.ShowStartupPerformance.ID, actions_2.ShowStartupPerformance.LABEL), developerCategory);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.ReloadWindowAction, actions_2.ReloadWindowAction.ID, actions_2.ReloadWindowAction.LABEL));
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.CloseMessagesAction, actions_2.CloseMessagesAction.ID, actions_2.CloseMessagesAction.LABEL, { primary: keyCodes_1.KeyCode.Escape, secondary: [keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.Escape] }, keybindingService_1.KbExpr.has(messageService_1.WorkbenchMessageService.GLOBAL_MESSAGES_SHOWING_CONTEXT)));
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.CloseEditorAction, actions_2.CloseEditorAction.ID, actions_2.CloseEditorAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_W, win: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.F4, secondary: [keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_W] } }), viewCategory);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.ToggleFullScreenAction, actions_2.ToggleFullScreenAction.ID, actions_2.ToggleFullScreenAction.LABEL, { primary: keyCodes_1.KeyCode.F11, mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_F } }), viewCategory);
    if (platform.isWindows || platform.isLinux) {
        workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(actions_2.ToggleMenuBarAction, actions_2.ToggleMenuBarAction.ID, actions_2.ToggleMenuBarAction.LABEL), viewCategory);
    }
    // Configuration: Window
    var configurationRegistry = platform_1.Registry.as(configurationRegistry_1.Extensions.Configuration);
    configurationRegistry.registerConfiguration({
        'id': 'window',
        'order': 6,
        'title': nls.localize(3, null),
        'type': 'object',
        'properties': {
            'window.openFilesInNewWindow': {
                'type': 'boolean',
                'default': true,
                'description': nls.localize(4, null)
            },
            'window.reopenFolders': {
                'type': 'string',
                'enum': ['none', 'one', 'all'],
                'default': 'one',
                'description': nls.localize(5, null)
            },
            'window.zoomLevel': {
                'type': 'number',
                'default': 0,
                'description': nls.localize(6, null)
            }
        }
    });
    // Configuration: Update
    configurationRegistry.registerConfiguration({
        'id': 'update',
        'order': 10,
        'title': nls.localize(7, null),
        'type': 'object',
        'properties': {
            'update.channel': {
                'type': 'string',
                'enum': ['none', 'default'],
                'default': 'default',
                'description': nls.localize(8, null)
            }
        }
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/extensions/electron-browser/extensionsActions", ["require", "exports", 'vs/nls!vs/workbench/parts/extensions/electron-browser/extensionsActions', 'vs/base/common/winjs.base', 'vs/base/common/actions', 'vs/base/common/severity', 'vs/platform/instantiation/common/instantiation', 'vs/platform/telemetry/common/telemetry', 'vs/platform/message/common/message', 'vs/workbench/electron-browser/actions', 'vs/workbench/parts/extensions/common/extensions', 'vs/workbench/parts/extensions/common/extensionsUtil', 'vs/workbench/services/quickopen/common/quickOpenService'], function (require, exports, nls, winjs_base_1, actions_1, severity_1, instantiation_1, telemetry_1, message_1, actions_2, extensions_1, extensionsUtil_1, quickOpenService_1) {
    "use strict";
    var CloseAction = new actions_1.Action('action.close', nls.localize(0, null));
    var ListExtensionsAction = (function (_super) {
        __extends(ListExtensionsAction, _super);
        function ListExtensionsAction(id, label, extensionsService, quickOpenService) {
            _super.call(this, id, label, null, true);
            this.extensionsService = extensionsService;
            this.quickOpenService = quickOpenService;
        }
        ListExtensionsAction.prototype.run = function () {
            return this.quickOpenService.show('ext ');
        };
        ListExtensionsAction.prototype.isEnabled = function () {
            return true;
        };
        ListExtensionsAction.ID = 'workbench.extensions.action.listExtensions';
        ListExtensionsAction.LABEL = nls.localize(1, null);
        ListExtensionsAction = __decorate([
            __param(2, extensions_1.IExtensionsService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], ListExtensionsAction);
        return ListExtensionsAction;
    }(actions_1.Action));
    exports.ListExtensionsAction = ListExtensionsAction;
    var InstallExtensionAction = (function (_super) {
        __extends(InstallExtensionAction, _super);
        function InstallExtensionAction(id, label, extensionsService, quickOpenService) {
            _super.call(this, id, label, null, true);
            this.extensionsService = extensionsService;
            this.quickOpenService = quickOpenService;
        }
        InstallExtensionAction.prototype.run = function () {
            return this.quickOpenService.show('ext install ');
        };
        InstallExtensionAction.prototype.isEnabled = function () {
            return true;
        };
        InstallExtensionAction.ID = 'workbench.extensions.action.installExtension';
        InstallExtensionAction.LABEL = nls.localize(2, null);
        InstallExtensionAction = __decorate([
            __param(2, extensions_1.IExtensionsService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], InstallExtensionAction);
        return InstallExtensionAction;
    }(actions_1.Action));
    exports.InstallExtensionAction = InstallExtensionAction;
    var ListOutdatedExtensionsAction = (function (_super) {
        __extends(ListOutdatedExtensionsAction, _super);
        function ListOutdatedExtensionsAction(id, label, extensionsService, quickOpenService) {
            _super.call(this, id, label, null, true);
            this.extensionsService = extensionsService;
            this.quickOpenService = quickOpenService;
        }
        ListOutdatedExtensionsAction.prototype.run = function () {
            return this.quickOpenService.show('ext update ');
        };
        ListOutdatedExtensionsAction.prototype.isEnabled = function () {
            return true;
        };
        ListOutdatedExtensionsAction.ID = 'workbench.extensions.action.listOutdatedExtensions';
        ListOutdatedExtensionsAction.LABEL = nls.localize(3, null);
        ListOutdatedExtensionsAction = __decorate([
            __param(2, extensions_1.IExtensionsService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], ListOutdatedExtensionsAction);
        return ListOutdatedExtensionsAction;
    }(actions_1.Action));
    exports.ListOutdatedExtensionsAction = ListOutdatedExtensionsAction;
    var ListSuggestedExtensionsAction = (function (_super) {
        __extends(ListSuggestedExtensionsAction, _super);
        function ListSuggestedExtensionsAction(id, label, extensionsService, quickOpenService) {
            _super.call(this, id, label, null, true);
            this.extensionsService = extensionsService;
            this.quickOpenService = quickOpenService;
        }
        ListSuggestedExtensionsAction.prototype.run = function () {
            return this.quickOpenService.show('ext recommend ');
        };
        ListSuggestedExtensionsAction.prototype.isEnabled = function () {
            return true;
        };
        ListSuggestedExtensionsAction.ID = 'workbench.extensions.action.listSuggestedExtensions';
        ListSuggestedExtensionsAction.LABEL = nls.localize(4, null);
        ListSuggestedExtensionsAction = __decorate([
            __param(2, extensions_1.IExtensionsService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], ListSuggestedExtensionsAction);
        return ListSuggestedExtensionsAction;
    }(actions_1.Action));
    exports.ListSuggestedExtensionsAction = ListSuggestedExtensionsAction;
    var InstallAction = (function (_super) {
        __extends(InstallAction, _super);
        function InstallAction(label, quickOpenService, extensionsService, messageService, telemetryService, instantiationService) {
            _super.call(this, 'extensions.install', label, 'octicon octicon-cloud-download', true);
            this.quickOpenService = quickOpenService;
            this.extensionsService = extensionsService;
            this.messageService = messageService;
            this.telemetryService = telemetryService;
            this.instantiationService = instantiationService;
        }
        InstallAction.prototype.run = function (extension) {
            var _this = this;
            this.enabled = false;
            return this.extensionsService.getInstalled()
                .then(function (installed) { return installed.some(function (e) { return extensionsUtil_1.extensionEquals(e, extension); }); })
                .then(function (isUpdate) {
                return _this.extensionsService
                    .install(extension)
                    .then(function () { return _this.onSuccess(extension, isUpdate); }, function (err) { return _this.onError(err, extension, isUpdate); })
                    .then(function () { return _this.enabled = true; })
                    .then(function () { return null; });
            });
        };
        InstallAction.prototype.onSuccess = function (extension, isUpdate) {
            this.reportTelemetry(extension, isUpdate, true);
            this.messageService.show(severity_1.default.Info, {
                message: nls.localize(5, null, extension.displayName),
                actions: [
                    CloseAction,
                    this.instantiationService.createInstance(actions_2.ReloadWindowAction, actions_2.ReloadWindowAction.ID, nls.localize(6, null))
                ]
            });
        };
        InstallAction.prototype.onError = function (err, extension, isUpdate) {
            this.reportTelemetry(extension, isUpdate, false);
            this.messageService.show(severity_1.default.Error, err);
        };
        InstallAction.prototype.reportTelemetry = function (extension, isUpdate, success) {
            var event = isUpdate ? 'extensionGallery:update' : 'extensionGallery:install';
            var data = {
                success: success,
                id: extension.galleryInformation ? extension.galleryInformation.id : null,
                name: extension.name,
                publisherId: extension.galleryInformation ? extension.galleryInformation.publisherId : null,
                publisherName: extension.publisher,
                publisherDisplayName: extension.galleryInformation ? extension.galleryInformation.publisherDisplayName : null
            };
            this.telemetryService.publicLog(event, data);
        };
        InstallAction = __decorate([
            __param(1, quickOpenService_1.IQuickOpenService),
            __param(2, extensions_1.IExtensionsService),
            __param(3, message_1.IMessageService),
            __param(4, telemetry_1.ITelemetryService),
            __param(5, instantiation_1.IInstantiationService)
        ], InstallAction);
        return InstallAction;
    }(actions_1.Action));
    exports.InstallAction = InstallAction;
    var UninstallAction = (function (_super) {
        __extends(UninstallAction, _super);
        function UninstallAction(quickOpenService, extensionsService, messageService, telemetryService, instantiationService) {
            _super.call(this, 'extensions.uninstall', nls.localize(7, null), 'octicon octicon-x', true);
            this.quickOpenService = quickOpenService;
            this.extensionsService = extensionsService;
            this.messageService = messageService;
            this.telemetryService = telemetryService;
            this.instantiationService = instantiationService;
        }
        UninstallAction.prototype.run = function (extension) {
            var _this = this;
            if (!window.confirm(nls.localize(8, null, extension.displayName))) {
                return winjs_base_1.TPromise.as(null);
            }
            this.enabled = false;
            return this.extensionsService.uninstall(extension)
                .then(function () { return _this.onSuccess(extension); }, function (err) { return _this.onError(err, extension); })
                .then(function () { return _this.enabled = true; })
                .then(function () { return null; });
        };
        UninstallAction.prototype.onSuccess = function (extension) {
            this.reportTelemetry(extension, true);
            this.messageService.show(severity_1.default.Info, {
                message: nls.localize(9, null, extension.displayName),
                actions: [
                    CloseAction,
                    this.instantiationService.createInstance(actions_2.ReloadWindowAction, actions_2.ReloadWindowAction.ID, nls.localize(10, null))
                ]
            });
        };
        UninstallAction.prototype.onError = function (err, extension) {
            this.reportTelemetry(extension, false);
            this.messageService.show(severity_1.default.Error, err);
        };
        UninstallAction.prototype.reportTelemetry = function (extension, success) {
            this.telemetryService.publicLog('extensionGallery:uninstall', {
                success: success,
                id: extension.galleryInformation ? extension.galleryInformation.id : null,
                name: extension.name,
                publisherId: extension.galleryInformation ? extension.galleryInformation.publisherId : null,
                publisherName: extension.publisher,
                publisherDisplayName: extension.galleryInformation ? extension.galleryInformation.publisherDisplayName : null
            });
        };
        UninstallAction = __decorate([
            __param(0, quickOpenService_1.IQuickOpenService),
            __param(1, extensions_1.IExtensionsService),
            __param(2, message_1.IMessageService),
            __param(3, telemetry_1.ITelemetryService),
            __param(4, instantiation_1.IInstantiationService)
        ], UninstallAction);
        return UninstallAction;
    }(actions_1.Action));
    exports.UninstallAction = UninstallAction;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/workspace/common/contextService", ["require", "exports", 'vs/workbench/common/events', 'vs/platform/instantiation/common/instantiation', 'vs/platform/workspace/common/baseWorkspaceContextService'], function (require, exports, events_1, instantiation_1, baseWorkspaceContextService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.IWorkspaceContextService = instantiation_1.createDecorator('contextService');
    var WorkspaceContextService = (function (_super) {
        __extends(WorkspaceContextService, _super);
        function WorkspaceContextService(eventService, workspace, configuration, options) {
            if (options === void 0) { options = {}; }
            _super.call(this, workspace, configuration, options);
            this.eventService = eventService;
            this.serviceId = exports.IWorkspaceContextService;
        }
        WorkspaceContextService.prototype.updateOptions = function (key, value) {
            var oldValue = this.options[key];
            this.options[key] = value;
            this.eventService.emit(events_1.EventType.WORKBENCH_OPTIONS_CHANGED, new events_1.OptionsChangeEvent(key, oldValue, value));
        };
        return WorkspaceContextService;
    }(baseWorkspaceContextService_1.BaseWorkspaceContextService));
    exports.WorkspaceContextService = WorkspaceContextService;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/actions/configureLocale", ["require", "exports", 'vs/nls!vs/workbench/browser/actions/configureLocale', 'vs/base/common/paths', 'vs/base/common/uri', 'vs/base/common/labels', 'vs/base/common/actions', 'vs/platform/platform', 'vs/workbench/common/actionRegistry', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/files/common/files', 'vs/platform/actions/common/actions'], function (require, exports, nls, Path, uri_1, Labels, actions_1, platform_1, actionRegistry_1, editorService_1, contextService_1, files_1, actions_2) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ConfigureLocaleAction = (function (_super) {
        __extends(ConfigureLocaleAction, _super);
        function ConfigureLocaleAction(id, label, fileService, contextService, editorService) {
            _super.call(this, id, label);
            this.fileService = fileService;
            this.contextService = contextService;
            this.editorService = editorService;
        }
        ConfigureLocaleAction.prototype.run = function (event) {
            var _this = this;
            var file = uri_1.default.file(Path.join(this.contextService.getConfiguration().env.appSettingsHome, 'locale.json'));
            return this.fileService.resolveFile(file).then(null, function (error) {
                return _this.fileService.createFile(file, ConfigureLocaleAction.DEFAULT_CONTENT);
            }).then(function (stat) {
                if (!stat) {
                    return;
                }
                return _this.editorService.openEditor({
                    resource: stat.resource,
                    options: {
                        forceOpen: true
                    }
                });
            }, function (error) {
                throw new Error(nls.localize(4, null, Labels.getPathLabel(file, _this.contextService), error));
            });
        };
        ConfigureLocaleAction.ID = 'workbench.action.configureLocale';
        ConfigureLocaleAction.LABEL = nls.localize(0, null);
        ConfigureLocaleAction.DEFAULT_CONTENT = [
            '{',
            ("\t// " + nls.localize(1, null)),
            ("\t// " + nls.localize(2, null, 'http://go.microsoft.com/fwlink/?LinkId=761051')),
            ("\t// " + nls.localize(3, null)),
            "\t\"locale\":\"en-US\"",
            '}'
        ].join('\n');
        ConfigureLocaleAction = __decorate([
            __param(2, files_1.IFileService),
            __param(3, contextService_1.IWorkspaceContextService),
            __param(4, editorService_1.IWorkbenchEditorService)
        ], ConfigureLocaleAction);
        return ConfigureLocaleAction;
    }(actions_1.Action));
    var workbenchActionsRegistry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(ConfigureLocaleAction, ConfigureLocaleAction.ID, ConfigureLocaleAction.LABEL));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/actions/openSettings", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/browser/actions/openSettings', 'vs/base/common/uri', 'vs/base/common/labels', 'vs/platform/platform', 'vs/base/common/actions', 'vs/workbench/common/actionRegistry', 'vs/workbench/common/editor/stringEditorInput', 'vs/platform/configuration/common/model', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/configuration/common/configuration', 'vs/platform/editor/common/editor', 'vs/platform/storage/common/storage', 'vs/platform/files/common/files', 'vs/platform/message/common/message', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/actions/common/actions', 'vs/platform/instantiation/common/instantiation', 'vs/base/common/keyCodes'], function (require, exports, winjs_base_1, nls, uri_1, labels, platform_1, actions_1, actionRegistry_1, stringEditorInput_1, model_1, editorService_1, contextService_1, configuration_1, editor_1, storage_1, files_1, message_1, keybindingService_1, actions_2, instantiation_1, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var BaseTwoEditorsAction = (function (_super) {
        __extends(BaseTwoEditorsAction, _super);
        function BaseTwoEditorsAction(id, label, editorService, fileService, configurationService, messageService, contextService, keybindingService, instantiationService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.fileService = fileService;
            this.configurationService = configurationService;
            this.messageService = messageService;
            this.contextService = contextService;
            this.keybindingService = keybindingService;
            this.instantiationService = instantiationService;
            this.enabled = true;
        }
        BaseTwoEditorsAction.prototype.createIfNotExists = function (resource, contents) {
            var _this = this;
            return this.fileService.resolveContent(resource, { acceptTextOnly: true }).then(null, function (error) {
                if (error.fileOperationResult === files_1.FileOperationResult.FILE_NOT_FOUND) {
                    return _this.fileService.updateContent(resource, contents).then(null, function (error) {
                        return winjs_base_1.TPromise.wrapError(new Error(nls.localize(0, null, labels.getPathLabel(resource, _this.contextService), error)));
                    });
                }
                return winjs_base_1.TPromise.wrapError(error);
            });
        };
        BaseTwoEditorsAction.prototype.openTwoEditors = function (leftHandDefaultInput, editableResource, defaultEditableContents) {
            var _this = this;
            // Create as needed and open in editor
            return this.createIfNotExists(editableResource, defaultEditableContents).then(function () {
                return _this.editorService.inputToType({ resource: editableResource }).then(function (typedRightHandEditableInput) {
                    return _this.editorService.setEditors([leftHandDefaultInput, typedRightHandEditableInput]).then(function () {
                        return _this.editorService.focusEditor(editor_1.Position.CENTER);
                    });
                });
            });
        };
        BaseTwoEditorsAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, files_1.IFileService),
            __param(4, configuration_1.IConfigurationService),
            __param(5, message_1.IMessageService),
            __param(6, contextService_1.IWorkspaceContextService),
            __param(7, keybindingService_1.IKeybindingService),
            __param(8, instantiation_1.IInstantiationService)
        ], BaseTwoEditorsAction);
        return BaseTwoEditorsAction;
    }(actions_1.Action));
    exports.BaseTwoEditorsAction = BaseTwoEditorsAction;
    var BaseOpenSettingsAction = (function (_super) {
        __extends(BaseOpenSettingsAction, _super);
        function BaseOpenSettingsAction(id, label, editorService, fileService, configurationService, messageService, contextService, keybindingService, instantiationService) {
            _super.call(this, id, label, editorService, fileService, configurationService, messageService, contextService, keybindingService, instantiationService);
        }
        BaseOpenSettingsAction.prototype.open = function (emptySettingsContents, settingsResource) {
            return this.openTwoEditors(DefaultSettingsInput.getInstance(this.instantiationService), settingsResource, emptySettingsContents);
        };
        BaseOpenSettingsAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, files_1.IFileService),
            __param(4, configuration_1.IConfigurationService),
            __param(5, message_1.IMessageService),
            __param(6, contextService_1.IWorkspaceContextService),
            __param(7, keybindingService_1.IKeybindingService),
            __param(8, instantiation_1.IInstantiationService)
        ], BaseOpenSettingsAction);
        return BaseOpenSettingsAction;
    }(BaseTwoEditorsAction));
    exports.BaseOpenSettingsAction = BaseOpenSettingsAction;
    var OpenGlobalSettingsAction = (function (_super) {
        __extends(OpenGlobalSettingsAction, _super);
        function OpenGlobalSettingsAction(id, label, editorService, fileService, configurationService, messageService, contextService, keybindingService, instantiationService, storageService) {
            _super.call(this, id, label, editorService, fileService, configurationService, messageService, contextService, keybindingService, instantiationService);
            this.storageService = storageService;
        }
        OpenGlobalSettingsAction.prototype.run = function (event) {
            var _this = this;
            // Inform user about workspace settings
            if (this.configurationService.hasWorkspaceConfiguration() && !this.storageService.getBoolean(OpenGlobalSettingsAction.SETTINGS_INFO_IGNORE_KEY, storage_1.StorageScope.WORKSPACE)) {
                this.messageService.show(message_1.Severity.Info, {
                    message: nls.localize(2, null),
                    actions: [
                        message_1.CloseAction,
                        new actions_1.Action('neverShowAgain', nls.localize(3, null), null, true, function () {
                            _this.storageService.store(OpenGlobalSettingsAction.SETTINGS_INFO_IGNORE_KEY, true, storage_1.StorageScope.WORKSPACE);
                            return winjs_base_1.TPromise.as(true);
                        }),
                        new actions_1.Action('open.workspaceSettings', nls.localize(4, null), null, true, function () {
                            var editorCount = _this.editorService.getVisibleEditors().length;
                            return _this.editorService.inputToType({ resource: _this.contextService.toResource('.vscode/settings.json') }).then(function (typedInput) {
                                return _this.editorService.openEditor(typedInput, null, editorCount === 2 ? editor_1.Position.RIGHT : editorCount === 1 ? editor_1.Position.CENTER : void 0);
                            });
                        })
                    ]
                });
            }
            // Open settings
            var emptySettingsHeader = nls.localize(5, null);
            return this.open('// ' + emptySettingsHeader + '\n{\n}', uri_1.default.file(this.contextService.getConfiguration().env.appSettingsPath));
        };
        OpenGlobalSettingsAction.ID = 'workbench.action.openGlobalSettings';
        OpenGlobalSettingsAction.LABEL = nls.localize(1, null);
        OpenGlobalSettingsAction.SETTINGS_INFO_IGNORE_KEY = 'settings.workspace.info.ignore';
        OpenGlobalSettingsAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, files_1.IFileService),
            __param(4, configuration_1.IConfigurationService),
            __param(5, message_1.IMessageService),
            __param(6, contextService_1.IWorkspaceContextService),
            __param(7, keybindingService_1.IKeybindingService),
            __param(8, instantiation_1.IInstantiationService),
            __param(9, storage_1.IStorageService)
        ], OpenGlobalSettingsAction);
        return OpenGlobalSettingsAction;
    }(BaseOpenSettingsAction));
    exports.OpenGlobalSettingsAction = OpenGlobalSettingsAction;
    var OpenGlobalKeybindingsAction = (function (_super) {
        __extends(OpenGlobalKeybindingsAction, _super);
        function OpenGlobalKeybindingsAction(id, label, editorService, fileService, configurationService, messageService, contextService, keybindingService, instantiationService) {
            _super.call(this, id, label, editorService, fileService, configurationService, messageService, contextService, keybindingService, instantiationService);
        }
        OpenGlobalKeybindingsAction.prototype.run = function (event) {
            var emptyContents = '// ' + nls.localize(7, null) + '\n[\n]';
            return this.openTwoEditors(DefaultKeybindingsInput.getInstance(this.instantiationService, this.keybindingService), uri_1.default.file(this.contextService.getConfiguration().env.appKeybindingsPath), emptyContents);
        };
        OpenGlobalKeybindingsAction.ID = 'workbench.action.openGlobalKeybindings';
        OpenGlobalKeybindingsAction.LABEL = nls.localize(6, null);
        OpenGlobalKeybindingsAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, files_1.IFileService),
            __param(4, configuration_1.IConfigurationService),
            __param(5, message_1.IMessageService),
            __param(6, contextService_1.IWorkspaceContextService),
            __param(7, keybindingService_1.IKeybindingService),
            __param(8, instantiation_1.IInstantiationService)
        ], OpenGlobalKeybindingsAction);
        return OpenGlobalKeybindingsAction;
    }(BaseTwoEditorsAction));
    exports.OpenGlobalKeybindingsAction = OpenGlobalKeybindingsAction;
    var OpenWorkspaceSettingsAction = (function (_super) {
        __extends(OpenWorkspaceSettingsAction, _super);
        function OpenWorkspaceSettingsAction() {
            _super.apply(this, arguments);
        }
        OpenWorkspaceSettingsAction.prototype.run = function (event) {
            if (!this.contextService.getWorkspace()) {
                this.messageService.show(message_1.Severity.Info, nls.localize(9, null));
                return;
            }
            var emptySettingsHeader = [
                '// ' + nls.localize(10, null),
                '{',
                '}'
            ].join('\n');
            return this.open(emptySettingsHeader, this.contextService.toResource('.vscode/settings.json'));
        };
        OpenWorkspaceSettingsAction.ID = 'workbench.action.openWorkspaceSettings';
        OpenWorkspaceSettingsAction.LABEL = nls.localize(8, null);
        return OpenWorkspaceSettingsAction;
    }(BaseOpenSettingsAction));
    exports.OpenWorkspaceSettingsAction = OpenWorkspaceSettingsAction;
    var DefaultSettingsInput = (function (_super) {
        __extends(DefaultSettingsInput, _super);
        function DefaultSettingsInput() {
            _super.apply(this, arguments);
        }
        DefaultSettingsInput.getInstance = function (instantiationService) {
            if (!DefaultSettingsInput.INSTANCE) {
                var defaults = model_1.getDefaultValuesContent();
                var defaultsHeader = '// ' + nls.localize(11, null);
                DefaultSettingsInput.INSTANCE = instantiationService.createInstance(DefaultSettingsInput, nls.localize(12, null), null, defaultsHeader + '\n' + defaults, 'application/json', false);
            }
            return DefaultSettingsInput.INSTANCE;
        };
        DefaultSettingsInput.prototype.getResource = function () {
            return uri_1.default.create('vscode', 'defaultsettings', '/settings.json'); // URI is used to register JSON schema support
        };
        return DefaultSettingsInput;
    }(stringEditorInput_1.StringEditorInput));
    var DefaultKeybindingsInput = (function (_super) {
        __extends(DefaultKeybindingsInput, _super);
        function DefaultKeybindingsInput() {
            _super.apply(this, arguments);
        }
        DefaultKeybindingsInput.getInstance = function (instantiationService, keybindingService) {
            if (!DefaultKeybindingsInput.INSTANCE) {
                var defaultsHeader = '// ' + nls.localize(13, null);
                var defaultContents = keybindingService.getDefaultKeybindings();
                DefaultKeybindingsInput.INSTANCE = instantiationService.createInstance(DefaultKeybindingsInput, nls.localize(14, null), null, defaultsHeader + '\n' + defaultContents, 'application/json', false);
            }
            return DefaultKeybindingsInput.INSTANCE;
        };
        DefaultKeybindingsInput.prototype.getResource = function () {
            return uri_1.default.create('vscode', 'defaultsettings', '/keybindings.json'); // URI is used to register JSON schema support
        };
        return DefaultKeybindingsInput;
    }(stringEditorInput_1.StringEditorInput));
    // Contribute Global Actions
    var category = nls.localize(15, null);
    var actionRegistry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    actionRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(OpenGlobalSettingsAction, OpenGlobalSettingsAction.ID, OpenGlobalSettingsAction.LABEL, {
        primary: null,
        mac: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.US_COMMA }
    }), category);
    actionRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(OpenGlobalKeybindingsAction, OpenGlobalKeybindingsAction.ID, OpenGlobalKeybindingsAction.LABEL), category);
    actionRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(OpenWorkspaceSettingsAction, OpenWorkspaceSettingsAction.ID, OpenWorkspaceSettingsAction.LABEL), category);
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/layout", ["require", "exports", 'vs/base/browser/builder', 'vs/workbench/common/constants', 'vs/workbench/common/events', 'vs/base/browser/ui/sash/sash', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/part/common/partService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/storage/common/storage', 'vs/platform/contextview/browser/contextView', 'vs/platform/event/common/event'], function (require, exports, builder_1, constants_1, events_1, sash_1, editorService_1, partService_1, contextService_1, storage_1, contextView_1, event_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var DEFAULT_MIN_PART_WIDTH = 170;
    var DEFAULT_MIN_PANEL_PART_HEIGHT = 170;
    var DEFAULT_MIN_EDITOR_PART_HEIGHT = 170;
    var HIDE_SIDEBAR_WIDTH_THRESHOLD = 50;
    var HIDE_PANEL_HEIGHT_THRESHOLD = 50;
    var LAYOUT_RELATED_PREFERENCES = [
        constants_1.Preferences.THEME
    ];
    var LayoutOptions = (function () {
        function LayoutOptions() {
            this.margin = new builder_1.Box(0, 0, 0, 0);
        }
        LayoutOptions.prototype.setMargin = function (margin) {
            this.margin = margin;
            return this;
        };
        return LayoutOptions;
    }());
    exports.LayoutOptions = LayoutOptions;
    /**
     * The workbench layout is responsible to lay out all parts that make the Monaco Workbench.
     */
    var WorkbenchLayout = (function () {
        // Take parts as an object bag since instatation service does not have typings for constructors with 9+ arguments
        function WorkbenchLayout(parent, workbenchContainer, parts, quickopen, options, storageService, eventService, contextViewService, editorService, contextService, partService) {
            this.storageService = storageService;
            this.eventService = eventService;
            this.contextViewService = contextViewService;
            this.editorService = editorService;
            this.contextService = contextService;
            this.partService = partService;
            this.parent = parent;
            this.workbenchContainer = workbenchContainer;
            this.activitybar = parts.activitybar;
            this.editor = parts.editor;
            this.sidebar = parts.sidebar;
            this.panel = parts.panel;
            this.statusbar = parts.statusbar;
            this.quickopen = quickopen;
            this.options = options || new LayoutOptions();
            this.toUnbind = [];
            this.computedStyles = null;
            this.sashX = new sash_1.Sash(this.workbenchContainer.getHTMLElement(), this, {
                baseSize: 5
            });
            this.sashY = new sash_1.Sash(this.workbenchContainer.getHTMLElement(), this, {
                baseSize: 5,
                orientation: sash_1.Orientation.HORIZONTAL
            });
            this.sidebarWidth = this.storageService.getInteger(WorkbenchLayout.sashXWidthSettingsKey, storage_1.StorageScope.GLOBAL, -1);
            this.panelHeight = this.storageService.getInteger(WorkbenchLayout.sashYHeightSettingsKey, storage_1.StorageScope.GLOBAL, 0);
            this.registerListeners();
            this.registerSashListeners();
        }
        WorkbenchLayout.prototype.registerListeners = function () {
            var _this = this;
            this.toUnbind.push(this.eventService.addListener(storage_1.StorageEventType.STORAGE, function (e) { return _this.onPreferenceChange(e); }));
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.EDITOR_INPUT_CHANGING, function (e) { return _this.onEditorInputChanging(e); }));
        };
        WorkbenchLayout.prototype.registerSashListeners = function () {
            var _this = this;
            var startX = 0;
            var startY = 0;
            this.sashX.addListener('start', function (e) {
                _this.startSidebarWidth = _this.sidebarWidth;
                startX = e.startX;
            });
            this.sashY.addListener('start', function (e) {
                _this.startPanelHeight = _this.panelHeight;
                startY = e.startY;
            });
            this.sashX.addListener('change', function (e) {
                var doLayout = false;
                var sidebarPosition = _this.partService.getSideBarPosition();
                var isSidebarHidden = _this.partService.isSideBarHidden();
                var newSashWidth = (sidebarPosition === partService_1.Position.LEFT) ? _this.startSidebarWidth + e.currentX - startX : _this.startSidebarWidth - e.currentX + startX;
                // Sidebar visible
                if (!isSidebarHidden) {
                    // Automatically hide side bar when a certain threshold is met
                    if (newSashWidth + HIDE_SIDEBAR_WIDTH_THRESHOLD < _this.computedStyles.sidebar.minWidth) {
                        var dragCompensation = DEFAULT_MIN_PART_WIDTH - HIDE_SIDEBAR_WIDTH_THRESHOLD;
                        _this.partService.setSideBarHidden(true);
                        startX = (sidebarPosition === partService_1.Position.LEFT) ? Math.max(_this.computedStyles.activitybar.minWidth, e.currentX - dragCompensation) : Math.min(e.currentX + dragCompensation, _this.workbenchSize.width - _this.computedStyles.activitybar.minWidth);
                        _this.sidebarWidth = _this.startSidebarWidth; // when restoring sidebar, restore to the sidebar width we started from
                    }
                    else {
                        _this.sidebarWidth = Math.max(_this.computedStyles.sidebar.minWidth, newSashWidth); // Sidebar can not become smaller than MIN_PART_WIDTH
                        doLayout = newSashWidth >= _this.computedStyles.sidebar.minWidth;
                    }
                }
                else {
                    if ((sidebarPosition === partService_1.Position.LEFT && e.currentX - startX >= _this.computedStyles.sidebar.minWidth) ||
                        (sidebarPosition === partService_1.Position.RIGHT && startX - e.currentX >= _this.computedStyles.sidebar.minWidth)) {
                        _this.startSidebarWidth = _this.computedStyles.sidebar.minWidth - (sidebarPosition === partService_1.Position.LEFT ? e.currentX - startX : startX - e.currentX);
                        _this.sidebarWidth = _this.computedStyles.sidebar.minWidth;
                        _this.partService.setSideBarHidden(false);
                    }
                }
                if (doLayout) {
                    _this.layout();
                }
            });
            this.sashY.addListener('change', function (e) {
                var doLayout = false;
                var isPanelHidden = _this.partService.isPanelHidden();
                var newSashHeight = _this.startPanelHeight - (e.currentY - startY);
                // Panel visible
                if (!isPanelHidden) {
                    // Automatically hide panel when a certain threshold is met
                    if (newSashHeight + HIDE_PANEL_HEIGHT_THRESHOLD < _this.computedStyles.panel.minHeight) {
                        var dragCompensation = DEFAULT_MIN_PANEL_PART_HEIGHT - HIDE_PANEL_HEIGHT_THRESHOLD;
                        _this.partService.setPanelHidden(true);
                        startY = Math.min(_this.sidebarHeight - _this.computedStyles.statusbar.height, e.currentY + dragCompensation);
                        _this.panelHeight = _this.startPanelHeight; // when restoring panel, restore to the panel width we started from
                    }
                    else {
                        _this.panelHeight = Math.max(_this.computedStyles.panel.minHeight, newSashHeight); // Panel can not become smaller than MIN_PART_HEIGHT
                        doLayout = newSashHeight >= _this.computedStyles.panel.minHeight;
                    }
                }
                else {
                    if (startY - e.currentY >= _this.computedStyles.panel.minHeight) {
                        _this.startPanelHeight = 0;
                        _this.panelHeight = _this.computedStyles.panel.minHeight;
                        _this.partService.setPanelHidden(false);
                    }
                }
                if (doLayout) {
                    _this.layout();
                }
            });
            this.sashX.addListener('end', function () {
                _this.storageService.store(WorkbenchLayout.sashXWidthSettingsKey, _this.sidebarWidth, storage_1.StorageScope.GLOBAL);
            });
            this.sashY.addListener('end', function () {
                _this.storageService.store(WorkbenchLayout.sashYHeightSettingsKey, _this.panelHeight, storage_1.StorageScope.GLOBAL);
            });
        };
        WorkbenchLayout.prototype.onEditorInputChanging = function (e) {
            // Make sure that we layout properly in case we detect that the sidebar is large enought to cause
            // multiple opened editors to go below minimal size. The fix is to trigger a layout for any editor
            // input change that falls into this category.
            if (this.workbenchSize && this.sidebarWidth) {
                var visibleEditors = this.editorService.getVisibleEditors().length;
                if (visibleEditors > 1 && this.workbenchSize.width - this.sidebarWidth < visibleEditors * DEFAULT_MIN_PART_WIDTH) {
                    this.layout();
                }
            }
        };
        WorkbenchLayout.prototype.onPreferenceChange = function (e) {
            if (e.key && LAYOUT_RELATED_PREFERENCES.indexOf(e.key) >= 0) {
                // Recompute Styles
                this.computeStyle();
                this.editor.getLayout().computeStyle();
                this.sidebar.getLayout().computeStyle();
                this.panel.getLayout().computeStyle();
                // Trigger Layout
                this.layout();
            }
        };
        WorkbenchLayout.prototype.computeStyle = function () {
            var sidebarStyle = this.sidebar.getContainer().getComputedStyle();
            var panelStyle = this.panel.getContainer().getComputedStyle();
            var editorStyle = this.editor.getContainer().getComputedStyle();
            var activitybarStyle = this.activitybar.getContainer().getComputedStyle();
            this.computedStyles = {
                activitybar: {
                    minWidth: parseInt(activitybarStyle.getPropertyValue('min-width'), 10) || 0
                },
                sidebar: {
                    minWidth: parseInt(sidebarStyle.getPropertyValue('min-width'), 10) || DEFAULT_MIN_PART_WIDTH
                },
                panel: {
                    minHeight: parseInt(panelStyle.getPropertyValue('min-height'), 10) || DEFAULT_MIN_PANEL_PART_HEIGHT
                },
                editor: {
                    minWidth: parseInt(editorStyle.getPropertyValue('min-width'), 10) || DEFAULT_MIN_PART_WIDTH
                },
                statusbar: {
                    height: 0
                }
            };
            if (this.statusbar) {
                var statusbarStyle = this.statusbar.getContainer().getComputedStyle();
                this.computedStyles.statusbar.height = parseInt(statusbarStyle.getPropertyValue('height'), 10) || 18;
            }
        };
        WorkbenchLayout.prototype.layout = function (forceStyleReCompute) {
            if (forceStyleReCompute) {
                this.computeStyle();
                this.editor.getLayout().computeStyle();
                this.sidebar.getLayout().computeStyle();
                this.panel.getLayout().computeStyle();
            }
            if (!this.computedStyles) {
                this.computeStyle();
            }
            this.workbenchSize = this.getWorkbenchArea();
            var isSidebarHidden = this.partService.isSideBarHidden();
            var isPanelHidden = this.partService.isPanelHidden();
            var sidebarPosition = this.partService.getSideBarPosition();
            // Sidebar
            var sidebarWidth;
            if (isSidebarHidden) {
                sidebarWidth = 0;
            }
            else if (this.sidebarWidth !== -1) {
                sidebarWidth = Math.max(this.computedStyles.sidebar.minWidth, this.sidebarWidth);
            }
            else {
                sidebarWidth = this.workbenchSize.width / 5;
                this.sidebarWidth = sidebarWidth;
            }
            this.sidebarHeight = this.workbenchSize.height - this.computedStyles.statusbar.height;
            var sidebarSize = new builder_1.Dimension(sidebarWidth, this.sidebarHeight);
            // Activity Bar
            var activityBarMinWidth = this.computedStyles.activitybar.minWidth;
            var activityBarSize = new builder_1.Dimension(activityBarMinWidth, sidebarSize.height);
            // Panel part
            var panelHeight;
            if (isPanelHidden) {
                panelHeight = 0;
            }
            else if (this.panelHeight > 0) {
                panelHeight = Math.min(sidebarSize.height - DEFAULT_MIN_EDITOR_PART_HEIGHT, Math.max(this.computedStyles.panel.minHeight, this.panelHeight));
            }
            else {
                panelHeight = sidebarSize.height * 0.4;
            }
            var panelDimension = new builder_1.Dimension(this.workbenchSize.width - sidebarSize.width - activityBarSize.width, panelHeight);
            this.panelWidth = panelDimension.width;
            // Editor
            var editorSize = {
                width: 0,
                height: 0,
                remainderLeft: 0,
                remainderRight: 0
            };
            var editorDimension = new builder_1.Dimension(panelDimension.width, sidebarSize.height - panelDimension.height);
            editorSize.width = editorDimension.width;
            editorSize.height = editorDimension.height;
            // Sidebar hidden
            if (isSidebarHidden) {
                editorSize.width = Math.min(this.workbenchSize.width - activityBarSize.width, this.workbenchSize.width - activityBarMinWidth);
                if (sidebarPosition === partService_1.Position.LEFT) {
                    editorSize.remainderLeft = Math.round((this.workbenchSize.width - editorSize.width + activityBarSize.width) / 2);
                    editorSize.remainderRight = this.workbenchSize.width - editorSize.width - editorSize.remainderLeft;
                }
                else {
                    editorSize.remainderRight = Math.round((this.workbenchSize.width - editorSize.width + activityBarSize.width) / 2);
                    editorSize.remainderLeft = this.workbenchSize.width - editorSize.width - editorSize.remainderRight;
                }
            }
            // Assert Sidebar and Editor Size to not overflow
            var editorMinWidth = this.computedStyles.editor.minWidth;
            var visibleEditorCount = this.editorService.getVisibleEditors().length;
            if (visibleEditorCount > 1) {
                editorMinWidth *= visibleEditorCount;
            }
            if (editorSize.width < editorMinWidth) {
                var diff = editorMinWidth - editorSize.width;
                editorSize.width = editorMinWidth;
                panelDimension.width = editorMinWidth;
                sidebarSize.width -= diff;
                sidebarSize.width = Math.max(DEFAULT_MIN_PART_WIDTH, sidebarSize.width);
            }
            if (!isSidebarHidden) {
                this.sidebarWidth = sidebarSize.width;
                this.storageService.store(WorkbenchLayout.sashXWidthSettingsKey, this.sidebarWidth, storage_1.StorageScope.GLOBAL);
            }
            if (!isPanelHidden) {
                this.panelHeight = panelDimension.height;
                this.storageService.store(WorkbenchLayout.sashYHeightSettingsKey, this.panelHeight, storage_1.StorageScope.GLOBAL);
            }
            // Workbench
            this.workbenchContainer
                .position(this.options.margin.top, this.options.margin.right, this.options.margin.bottom, this.options.margin.left, 'relative')
                .size(this.workbenchSize.width, this.workbenchSize.height);
            // Bug on Chrome: Sometimes Chrome wants to scroll the workbench container on layout changes. The fix is to reset scrollTop in this case.
            if (this.workbenchContainer.getHTMLElement().scrollTop > 0) {
                this.workbenchContainer.getHTMLElement().scrollTop = 0;
            }
            // Editor Part and Panel part
            this.editor.getContainer().size(editorSize.width, editorSize.height);
            this.panel.getContainer().size(panelDimension.width, panelDimension.height);
            var editorBottom = this.computedStyles.statusbar.height + panelDimension.height;
            if (isSidebarHidden) {
                this.editor.getContainer().position(0, editorSize.remainderRight, editorBottom, editorSize.remainderLeft);
                this.panel.getContainer().position(editorDimension.height, editorSize.remainderRight, this.computedStyles.statusbar.height, editorSize.remainderLeft);
            }
            else if (sidebarPosition === partService_1.Position.LEFT) {
                this.editor.getContainer().position(0, 0, editorBottom, sidebarSize.width + activityBarSize.width);
                this.panel.getContainer().position(editorDimension.height, 0, this.computedStyles.statusbar.height, sidebarSize.width + activityBarSize.width);
            }
            else {
                this.editor.getContainer().position(0, sidebarSize.width, editorBottom, 0);
                this.panel.getContainer().position(editorDimension.height, sidebarSize.width, this.computedStyles.statusbar.height, 0);
            }
            // Activity Bar Part
            this.activitybar.getContainer().size(null, activityBarSize.height);
            if (sidebarPosition === partService_1.Position.LEFT) {
                this.activitybar.getContainer().getHTMLElement().style.right = '';
                this.activitybar.getContainer().position(0, null, 0, 0);
            }
            else {
                this.activitybar.getContainer().getHTMLElement().style.left = '';
                this.activitybar.getContainer().position(0, 0, 0, null);
            }
            // Sidebar Part
            this.sidebar.getContainer().size(sidebarSize.width, sidebarSize.height);
            if (sidebarPosition === partService_1.Position.LEFT) {
                this.sidebar.getContainer().position(0, editorSize.width, 0, activityBarSize.width);
            }
            else {
                this.sidebar.getContainer().position(0, null, 0, editorSize.width);
            }
            // Statusbar Part
            if (this.statusbar) {
                this.statusbar.getContainer().position(this.workbenchSize.height - this.computedStyles.statusbar.height);
            }
            // Quick open
            this.quickopen.layout(this.workbenchSize);
            // Sashes
            this.sashX.layout();
            this.sashY.layout();
            // Propagate to Part Layouts
            this.editor.layout(new builder_1.Dimension(editorSize.width, editorSize.height));
            this.sidebar.layout(sidebarSize);
            this.panel.layout(panelDimension);
            // Propagate to Context View
            if (this.contextViewService) {
                this.contextViewService.layout();
            }
        };
        WorkbenchLayout.prototype.getWorkbenchArea = function () {
            // Client Area: Parent
            var clientArea = this.parent.getClientArea();
            // Workbench: Client Area - Margins
            return clientArea.substract(this.options.margin);
        };
        WorkbenchLayout.prototype.getVerticalSashTop = function (sash) {
            return 0;
        };
        WorkbenchLayout.prototype.getVerticalSashLeft = function (sash) {
            var isSidebarHidden = this.partService.isSideBarHidden();
            var sidebarPosition = this.partService.getSideBarPosition();
            var activitybarWidth = this.computedStyles.activitybar.minWidth;
            if (sidebarPosition === partService_1.Position.LEFT) {
                return !isSidebarHidden ? this.sidebarWidth + activitybarWidth : activitybarWidth;
            }
            return !isSidebarHidden ? this.workbenchSize.width - this.sidebarWidth - activitybarWidth : this.workbenchSize.width - activitybarWidth;
        };
        WorkbenchLayout.prototype.getVerticalSashHeight = function (sash) {
            return this.sidebarHeight;
        };
        WorkbenchLayout.prototype.getHorizontalSashTop = function (sash) {
            return this.partService.isPanelHidden() ? this.sidebarHeight : this.sidebarHeight - this.panelHeight;
        };
        WorkbenchLayout.prototype.getHorizontalSashLeft = function (sash) {
            return this.partService.getSideBarPosition() === partService_1.Position.LEFT ? this.getVerticalSashLeft(sash) : 0;
        };
        WorkbenchLayout.prototype.getHorizontalSashWidth = function (sash) {
            return this.panelWidth;
        };
        WorkbenchLayout.prototype.dispose = function () {
            while (this.toUnbind.length) {
                this.toUnbind.pop()();
            }
        };
        WorkbenchLayout.sashXWidthSettingsKey = 'workbench.sidebar.width';
        WorkbenchLayout.sashYHeightSettingsKey = 'workbench.panel.height';
        WorkbenchLayout = __decorate([
            __param(5, storage_1.IStorageService),
            __param(6, event_1.IEventService),
            __param(7, contextView_1.IContextViewService),
            __param(8, editorService_1.IWorkbenchEditorService),
            __param(9, contextService_1.IWorkspaceContextService),
            __param(10, partService_1.IPartService)
        ], WorkbenchLayout);
        return WorkbenchLayout;
    }());
    exports.WorkbenchLayout = WorkbenchLayout;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/editor/sideBySideEditorControl", ["require", "exports", 'vs/nls!vs/workbench/browser/parts/editor/sideBySideEditorControl', 'vs/platform/platform', 'vs/workbench/browser/actionBarRegistry', 'vs/base/common/actions', 'vs/base/common/arrays', 'vs/base/common/eventEmitter', 'vs/base/browser/mouseEvent', 'vs/base/common/errors', 'vs/base/common/platform', 'vs/base/common/types', 'vs/base/browser/builder', 'vs/base/browser/ui/sash/sash', 'vs/base/browser/ui/progressbar/progressbar', 'vs/workbench/common/editor', 'vs/workbench/common/editor/diffEditorInput', 'vs/base/common/events', 'vs/workbench/common/events', 'vs/base/browser/dom', 'vs/base/browser/ui/actionbar/actionbar', 'vs/base/browser/ui/toolbar/toolbar', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/contextview/browser/contextView', 'vs/platform/editor/common/editor', 'vs/platform/event/common/event', 'vs/platform/message/common/message', 'vs/platform/telemetry/common/telemetry', 'vs/css!./media/sidebyside'], function (require, exports, nls, platform_1, actionBarRegistry_1, actions_1, arrays, eventEmitter_1, mouseEvent_1, errors, platform_2, types, builder_1, sash_1, progressbar_1, editor_1, diffEditorInput_1, events_1, events_2, DOM, actionbar_1, toolbar_1, editorService_1, quickOpenService_1, contextService_1, contextView_1, editor_2, event_1, message_1, telemetry_1) {
    'use strict';
    (function (Rochade) {
        Rochade[Rochade["NONE"] = 0] = "NONE";
        Rochade[Rochade["CENTER_TO_LEFT"] = 1] = "CENTER_TO_LEFT";
        Rochade[Rochade["RIGHT_TO_CENTER"] = 2] = "RIGHT_TO_CENTER";
        Rochade[Rochade["CENTER_AND_RIGHT_TO_LEFT"] = 3] = "CENTER_AND_RIGHT_TO_LEFT";
    })(exports.Rochade || (exports.Rochade = {}));
    var Rochade = exports.Rochade;
    exports.EventType = {
        EDITOR_FOCUS_CHANGED: 'editorFocusChanged'
    };
    /**
     * Helper class to manage multiple side by side editors for the editor part.
     */
    var SideBySideEditorControl = (function (_super) {
        __extends(SideBySideEditorControl, _super);
        function SideBySideEditorControl(parent, editorService, messageService, quickOpenService, telemetryService, contextViewService, contextMenuService, eventService, contextService) {
            var _this = this;
            _super.call(this);
            this.editorService = editorService;
            this.messageService = messageService;
            this.quickOpenService = quickOpenService;
            this.telemetryService = telemetryService;
            this.contextViewService = contextViewService;
            this.contextMenuService = contextMenuService;
            this.eventService = eventService;
            this.contextService = contextService;
            this.parent = parent;
            this.dimension = new builder_1.Dimension(0, 0);
            this.containers = [];
            this.containerWidth = [];
            this.titleContainer = [];
            this.titleLabel = [];
            this.titleDescription = [];
            this.editorInputStateDescription = [];
            this.editorActionsToolbar = [];
            this.progressBar = [];
            this.visibleEditors = [];
            this.visibleEditorContainers = [];
            this.visibleEditorFocusTrackers = [];
            this.closeEditorAction = editor_2.POSITIONS.map(function (position) {
                return _this.makeCloseEditorAction(position);
            });
            this.splitEditorAction = this.makeSplitEditorAction();
            this.initStyles();
            this.registerListeners();
            // Create
            this.create(this.parent);
        }
        SideBySideEditorControl.prototype.registerListeners = function () {
            var _this = this;
            // Update editor input state indicators on state changes
            this.editorInputStateChangeListener = this.eventService.addListener(events_2.EventType.EDITOR_INPUT_STATE_CHANGED, function (event) {
                _this.updateEditorInputStateIndicator(event);
            });
        };
        SideBySideEditorControl.prototype.initStyles = function () {
            var grabCursor = platform_2.isWindows ? 'cursor: url("' + require.toUrl('vs/workbench/browser/parts/editor/media/grab.cur') + '"), move;' : 'cursor: -webkit-grab;';
            var grabbingCursor = platform_2.isWindows ? 'cursor: url("' + require.toUrl('vs/workbench/browser/parts/editor/media/grabbing.cur') + '"), move;' : 'cursor: -webkit-grabbing;';
            DOM.createCSSRule('.monaco-workbench > .part.editor > .content.multiple-editors .one-editor-container .title, ' +
                '.monaco-workbench > .part.editor > .content.multiple-editors .one-editor-container .title .title-label a, ' +
                '.monaco-workbench > .part.editor > .content.multiple-editors .one-editor-container .title .title-label span', grabCursor);
            DOM.createCSSRule('#monaco-workbench-editor-move-overlay, ' +
                '.monaco-workbench > .part.editor > .content.multiple-editors .one-editor-container.dragged, ' +
                '.monaco-workbench > .part.editor > .content.multiple-editors .one-editor-container.dragged .title, ' +
                '.monaco-workbench > .part.editor > .content.multiple-editors .one-editor-container.dragged .title .title-label a, ' +
                '.monaco-workbench > .part.editor > .content.multiple-editors .one-editor-container.dragged .title .title-label span, ' +
                '.monaco-workbench > .part.editor > .content.multiple-editors .one-editor-container.dragged .monaco-editor .view-lines', grabbingCursor);
        };
        SideBySideEditorControl.prototype.makeCloseEditorAction = function (position) {
            var _this = this;
            return new actions_1.Action('close.editor.action', nls.localize(0, null), 'close-editor-action', true, function () {
                return _this.editorService.closeEditor(position);
            });
        };
        SideBySideEditorControl.prototype.makeSplitEditorAction = function () {
            var _this = this;
            return new actions_1.Action('split.editor.action', nls.localize(1, null), 'split-editor-action', true, function () {
                var activeEditor = _this.editorService.getActiveEditor();
                var editorCount = _this.getVisibleEditorCount();
                // Special case: If the user wants to split the left hand editor with 2 editors open, push the center one to the right
                if (editorCount > 1 && activeEditor.position === editor_2.Position.LEFT) {
                    var centerInput_1 = _this.visibleEditors[editor_2.Position.CENTER].input;
                    var options_1 = new editor_1.TextEditorOptions();
                    options_1.preserveFocus = true;
                    return _this.editorService.openEditor(activeEditor.input, options_1, editor_2.Position.CENTER).then(function () {
                        return _this.editorService.openEditor(centerInput_1, options_1, editor_2.Position.RIGHT).then(function () {
                            return _this.editorService.focusEditor(editor_2.Position.CENTER);
                        });
                    });
                }
                // Otherwise just continue to open to the side
                return _this.editorService.openEditor(activeEditor.input, null, true);
            });
        };
        SideBySideEditorControl.prototype.show = function (editor, container, position, preserveActive, widthRatios) {
            var visibleEditorCount = this.getVisibleEditorCount();
            // Store into editor bucket
            this.visibleEditors[position] = editor;
            this.visibleEditorContainers[position] = container;
            // Store as active unless preserveActive is set
            if (!preserveActive || !this.lastActiveEditor) {
                this.doSetActive(editor, position);
            }
            // Track focus
            this.trackFocus(editor, position);
            // Find target container and build into
            var target = this.containers[position];
            container.build(target);
            // Adjust layout according to provided ratios (used when restoring multiple editors at once)
            if (widthRatios && (widthRatios.length === 2 || widthRatios.length === 3)) {
                var hasLayoutInfo = this.dimension && this.dimension.width;
                // We received width ratios but were not layouted yet. So we keep these ratios for when we layout()
                if (!hasLayoutInfo) {
                    this.containerInitialRatios = widthRatios;
                }
                // Adjust layout: -> [!][!]
                if (widthRatios.length === 2) {
                    if (hasLayoutInfo) {
                        this.containerWidth[position] = this.dimension.width * widthRatios[position];
                    }
                }
                else if (widthRatios.length === 3) {
                    if (hasLayoutInfo) {
                        this.containerWidth[position] = this.dimension.width * widthRatios[position];
                    }
                    if (this.rightSash.isHidden()) {
                        this.rightSash.show();
                        this.rightSash.layout();
                    }
                }
                if (this.leftSash.isHidden()) {
                    this.leftSash.show();
                    this.leftSash.layout();
                }
                if (hasLayoutInfo) {
                    this.layoutContainers();
                }
            }
            else if (visibleEditorCount === 0 && this.dimension) {
                this.containerWidth[position] = this.dimension.width;
                this.layoutContainers();
            }
            else if (position === editor_2.Position.CENTER && this.leftSash.isHidden() && this.rightSash.isHidden() && this.dimension) {
                this.containerWidth[editor_2.Position.LEFT] = this.dimension.width / 2;
                this.containerWidth[editor_2.Position.CENTER] = this.dimension.width - this.containerWidth[editor_2.Position.LEFT];
                this.leftSash.show();
                this.leftSash.layout();
                this.layoutContainers();
            }
            else if (position === editor_2.Position.RIGHT && this.rightSash.isHidden() && this.dimension) {
                this.containerWidth[editor_2.Position.LEFT] = this.dimension.width / 3;
                this.containerWidth[editor_2.Position.CENTER] = this.dimension.width / 3;
                this.containerWidth[editor_2.Position.RIGHT] = this.dimension.width - this.containerWidth[editor_2.Position.LEFT] - this.containerWidth[editor_2.Position.CENTER];
                this.leftSash.layout();
                this.rightSash.show();
                this.rightSash.layout();
                this.layoutContainers();
            }
            // Show editor container
            container.show();
            // Styles
            this.updateParentStyle();
        };
        SideBySideEditorControl.prototype.getVisibleEditorCount = function () {
            var c = 0;
            this.visibleEditors.forEach(function (editor) {
                if (editor) {
                    c++;
                }
            });
            return c;
        };
        SideBySideEditorControl.prototype.indexOf = function (editor) {
            return this.visibleEditors.indexOf(editor);
        };
        SideBySideEditorControl.prototype.trackFocus = function (editor, position) {
            var _this = this;
            // In case there is a previous tracker on the position, dispose it first
            if (this.visibleEditorFocusTrackers[position]) {
                this.visibleEditorFocusTrackers[position].dispose();
            }
            // Track focus on editor container
            this.visibleEditorFocusTrackers[position] = DOM.trackFocus(editor.getContainer().getHTMLElement());
            this.visibleEditorFocusTrackers[position].addFocusListener(function () {
                _this.onFocusGained(editor);
            });
        };
        SideBySideEditorControl.prototype.onFocusGained = function (editor) {
            this.setActive(editor);
        };
        SideBySideEditorControl.prototype.setActive = function (editor) {
            var _this = this;
            // Update active editor and position
            if (this.lastActiveEditor !== editor) {
                this.doSetActive(editor, this.indexOf(editor));
                // Automatically maximize this position if it has min editor width
                if (this.containerWidth[this.lastActivePosition] === SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    // Log this fact in telemetry
                    if (this.telemetryService) {
                        this.telemetryService.publicLog('workbenchEditorMaximized');
                    }
                    var remainingWidth_1 = this.dimension.width;
                    // Minimize all other positions to min width
                    editor_2.POSITIONS.forEach(function (p) {
                        if (_this.lastActivePosition !== p && !!_this.visibleEditors[p]) {
                            _this.containerWidth[p] = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                            remainingWidth_1 -= _this.containerWidth[p];
                        }
                    });
                    // Grow focussed position if there is more width to spend
                    if (remainingWidth_1 > SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                        this.containerWidth[this.lastActivePosition] = remainingWidth_1;
                        if (!this.leftSash.isHidden()) {
                            this.leftSash.layout();
                        }
                        if (!this.rightSash.isHidden()) {
                            this.rightSash.layout();
                        }
                        this.layoutContainers();
                    }
                }
                // Re-emit to outside
                this.emit(exports.EventType.EDITOR_FOCUS_CHANGED);
            }
        };
        SideBySideEditorControl.prototype.focusNextNonMinimized = function () {
            var _this = this;
            // If the current focussed editor is minimized, try to focus the next largest editor
            if (!types.isUndefinedOrNull(this.lastActivePosition) && this.containerWidth[this.lastActivePosition] === SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                var candidate_1 = null;
                var currentWidth_1 = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                editor_2.POSITIONS.forEach(function (position) {
                    // Skip current active position and check if the editor is larger than min width
                    if (position !== _this.lastActivePosition) {
                        if (_this.visibleEditors[position] && _this.containerWidth[position] > currentWidth_1) {
                            candidate_1 = position;
                            currentWidth_1 = _this.containerWidth[position];
                        }
                    }
                });
                // Focus editor if a candidate has been found
                if (!types.isUndefinedOrNull(candidate_1)) {
                    this.editorService.focusEditor(this.visibleEditors[candidate_1]).done(null, errors.onUnexpectedError);
                }
            }
        };
        SideBySideEditorControl.prototype.hide = function (editor, container, position, layoutAndRochade) {
            var result = Rochade.NONE;
            var visibleEditorCount = this.getVisibleEditorCount();
            var hasCenter = !!this.visibleEditors[editor_2.Position.CENTER];
            var hasRight = !!this.visibleEditors[editor_2.Position.RIGHT];
            // If editor is not showing for position, return
            if (editor !== this.visibleEditors[position]) {
                return result;
            }
            // Clear Position
            this.clearPosition(position);
            // Take editor container offdom and hide
            container.offDOM();
            container.hide();
            // Adjust layout and rochade if instructed to do so
            if (layoutAndRochade) {
                // Adjust layout: [x] ->
                if (visibleEditorCount === 1) {
                    this.containerWidth[position] = 0;
                    this.leftSash.hide();
                    this.rightSash.hide();
                    this.layoutContainers();
                }
                else if (hasCenter && !hasRight) {
                    this.containerWidth[editor_2.Position.LEFT] = this.dimension.width;
                    this.containerWidth[editor_2.Position.CENTER] = 0;
                    this.leftSash.hide();
                    this.rightSash.hide();
                    // Move CENTER to LEFT ([x]|[] -> [])
                    if (position === editor_2.Position.LEFT) {
                        this.rochade(editor_2.Position.CENTER, editor_2.Position.LEFT);
                        result = Rochade.CENTER_TO_LEFT;
                        this.clearTitle(editor_2.Position.CENTER); // center closes so clear title
                    }
                    this.layoutContainers();
                }
                else if (hasCenter && hasRight) {
                    this.containerWidth[editor_2.Position.LEFT] = this.dimension.width / 2;
                    this.containerWidth[editor_2.Position.CENTER] = this.dimension.width - this.containerWidth[editor_2.Position.LEFT];
                    this.containerWidth[editor_2.Position.RIGHT] = 0;
                    this.leftSash.layout();
                    this.rightSash.hide();
                    // Move RIGHT to CENTER ([]|[x]|[] -> [ ]|[ ])
                    if (position === editor_2.Position.CENTER) {
                        this.rochade(editor_2.Position.RIGHT, editor_2.Position.CENTER);
                        result = Rochade.RIGHT_TO_CENTER;
                        this.clearTitle(editor_2.Position.RIGHT); // right closes so clear title
                    }
                    else if (position === editor_2.Position.LEFT) {
                        this.rochade(editor_2.Position.CENTER, editor_2.Position.LEFT);
                        this.rochade(editor_2.Position.RIGHT, editor_2.Position.CENTER);
                        result = Rochade.CENTER_AND_RIGHT_TO_LEFT;
                        this.clearTitle(editor_2.Position.RIGHT); // right closes so clear title
                    }
                    this.layoutContainers();
                }
            }
            // Automatically pick the next editor as active if any
            if (this.lastActiveEditor === editor) {
                // Clear old
                this.doSetActive(null, null);
                // Find new active position by taking the next one close to the closed one to the left
                if (layoutAndRochade) {
                    var newActivePosition = void 0;
                    switch (position) {
                        case editor_2.Position.LEFT:
                            newActivePosition = hasCenter ? editor_2.Position.LEFT : null;
                            break;
                        case editor_2.Position.CENTER:
                            newActivePosition = editor_2.Position.LEFT;
                            break;
                        case editor_2.Position.RIGHT:
                            newActivePosition = editor_2.Position.CENTER;
                            break;
                    }
                    if (!types.isUndefinedOrNull(newActivePosition)) {
                        this.doSetActive(this.visibleEditors[newActivePosition], newActivePosition);
                    }
                }
            }
            // Styles
            this.updateParentStyle();
            return result;
        };
        SideBySideEditorControl.prototype.updateParentStyle = function () {
            var editorCount = this.getVisibleEditorCount();
            if (editorCount > 1) {
                this.parent.addClass('multiple-editors');
            }
            else {
                this.parent.removeClass('multiple-editors');
            }
        };
        SideBySideEditorControl.prototype.doSetActive = function (editor, newActive) {
            var oldActive = this.lastActivePosition;
            this.lastActivePosition = newActive;
            this.lastActiveEditor = editor;
            if (!types.isUndefinedOrNull(oldActive)) {
                this.containers[oldActive].addClass('inactive');
                this.containers[oldActive].removeClass('active');
            }
            if (!types.isUndefinedOrNull(newActive)) {
                this.containers[newActive].removeClass('inactive');
                this.containers[newActive].addClass('active');
            }
        };
        SideBySideEditorControl.prototype.clearPosition = function (position) {
            // Unregister Listeners
            if (this.visibleEditorFocusTrackers[position]) {
                this.visibleEditorFocusTrackers[position].dispose();
                this.visibleEditorFocusTrackers[position] = null;
            }
            // Clear from active editors
            this.visibleEditors[position] = null;
            this.visibleEditorContainers[position] = null;
        };
        SideBySideEditorControl.prototype.rochade = function (from, to) {
            // Move editor to new position
            var editorContainer = this.visibleEditorContainers[from];
            var editor = this.visibleEditors[from];
            editorContainer.offDOM();
            editorContainer.build(this.containers[to]);
            editor.changePosition(to);
            // Change data structures
            var listeners = this.visibleEditorFocusTrackers[from];
            this.visibleEditorContainers[to] = editorContainer;
            this.visibleEditors[to] = editor;
            this.visibleEditorFocusTrackers[to] = listeners;
            this.visibleEditorContainers[from] = null;
            this.visibleEditors[from] = null;
            this.visibleEditorFocusTrackers[from] = null;
            // Update last active position
            if (this.lastActivePosition === from) {
                this.doSetActive(this.lastActiveEditor, to);
            }
        };
        SideBySideEditorControl.prototype.move = function (from, to) {
            var editorContainerPos1;
            var editorPos1;
            var editorContainerPos2;
            var editorPos2;
            // Distance 1: Swap Editors
            if (Math.abs(from - to) === 1) {
                // Move editors to new position
                editorContainerPos1 = this.visibleEditorContainers[from];
                editorPos1 = this.visibleEditors[from];
                editorContainerPos1.offDOM();
                editorContainerPos1.build(this.containers[to]);
                editorPos1.changePosition(to);
                editorContainerPos2 = this.visibleEditorContainers[to];
                editorPos2 = this.visibleEditors[to];
                editorContainerPos2.offDOM();
                editorContainerPos2.build(this.containers[from]);
                editorPos2.changePosition(from);
                // Update last active position accordingly
                if (this.lastActivePosition === from) {
                    this.doSetActive(this.lastActiveEditor, to);
                }
                else if (this.lastActivePosition === to) {
                    this.doSetActive(this.lastActiveEditor, from);
                }
            }
            else {
                // Find new positions
                var newLeftPosition = void 0;
                var newCenterPosition = void 0;
                var newRightPosition = void 0;
                if (from === editor_2.Position.LEFT) {
                    newLeftPosition = editor_2.Position.RIGHT;
                    newCenterPosition = editor_2.Position.LEFT;
                    newRightPosition = editor_2.Position.CENTER;
                }
                else {
                    newLeftPosition = editor_2.Position.CENTER;
                    newCenterPosition = editor_2.Position.RIGHT;
                    newRightPosition = editor_2.Position.LEFT;
                }
                // Move editors to new position
                editorContainerPos1 = this.visibleEditorContainers[editor_2.Position.LEFT];
                editorPos1 = this.visibleEditors[editor_2.Position.LEFT];
                editorContainerPos1.offDOM();
                editorContainerPos1.build(this.containers[newLeftPosition]);
                editorPos1.changePosition(newLeftPosition);
                editorContainerPos2 = this.visibleEditorContainers[editor_2.Position.CENTER];
                editorPos2 = this.visibleEditors[editor_2.Position.CENTER];
                editorContainerPos2.offDOM();
                editorContainerPos2.build(this.containers[newCenterPosition]);
                editorPos2.changePosition(newCenterPosition);
                var editorContainerPos3 = this.visibleEditorContainers[editor_2.Position.RIGHT];
                var editorPos3 = this.visibleEditors[editor_2.Position.RIGHT];
                editorContainerPos3.offDOM();
                editorContainerPos3.build(this.containers[newRightPosition]);
                editorPos3.changePosition(newRightPosition);
                // Update last active position accordingly
                if (this.lastActivePosition === editor_2.Position.LEFT) {
                    this.doSetActive(this.lastActiveEditor, newLeftPosition);
                }
                else if (this.lastActivePosition === editor_2.Position.CENTER) {
                    this.doSetActive(this.lastActiveEditor, newCenterPosition);
                }
                else if (this.lastActivePosition === editor_2.Position.RIGHT) {
                    this.doSetActive(this.lastActiveEditor, newRightPosition);
                }
            }
            // Change data structures
            arrays.move(this.visibleEditorContainers, from, to);
            arrays.move(this.visibleEditors, from, to);
            arrays.move(this.visibleEditorFocusTrackers, from, to);
            arrays.move(this.containerWidth, from, to);
            // Layout
            if (!this.leftSash.isHidden()) {
                this.leftSash.layout();
            }
            if (!this.rightSash.isHidden()) {
                this.rightSash.layout();
            }
            this.layoutContainers();
        };
        SideBySideEditorControl.prototype.arrangeEditors = function (arrangement) {
            var _this = this;
            if (!this.dimension) {
                return; // too early
            }
            var availableWidth = this.dimension.width;
            var visibleEditors = this.getVisibleEditorCount();
            if (visibleEditors <= 1) {
                return; // need more editors
            }
            // Minimize Others
            if (arrangement === editorService_1.EditorArrangement.MINIMIZE_OTHERS) {
                editor_2.POSITIONS.forEach(function (position) {
                    if (_this.visibleEditors[position]) {
                        if (position !== _this.lastActivePosition) {
                            _this.containerWidth[position] = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                            availableWidth -= SideBySideEditorControl.MIN_EDITOR_WIDTH;
                        }
                    }
                });
                this.containerWidth[this.lastActivePosition] = availableWidth;
            }
            else if (arrangement === editorService_1.EditorArrangement.EVEN_WIDTH) {
                editor_2.POSITIONS.forEach(function (position) {
                    if (_this.visibleEditors[position]) {
                        _this.containerWidth[position] = availableWidth / visibleEditors;
                    }
                });
            }
            this.layoutControl(this.dimension);
        };
        SideBySideEditorControl.prototype.getWidthRatios = function () {
            var _this = this;
            var ratio = [];
            if (this.dimension) {
                var fullWidth_1 = this.dimension.width;
                editor_2.POSITIONS.forEach(function (position) {
                    if (_this.visibleEditors[position]) {
                        ratio.push(_this.containerWidth[position] / fullWidth_1);
                    }
                });
            }
            return ratio;
        };
        SideBySideEditorControl.prototype.getActiveEditor = function () {
            return this.lastActiveEditor;
        };
        SideBySideEditorControl.prototype.getActivePosition = function () {
            return this.lastActivePosition;
        };
        SideBySideEditorControl.prototype.create = function (parent) {
            var _this = this;
            // Left Container
            this.containers[editor_2.Position.LEFT] = builder_1.$(parent).div({ class: 'one-editor-container editor-left' });
            // Left Sash
            this.leftSash = new sash_1.Sash(parent.getHTMLElement(), this, { baseSize: 5 });
            this.leftSash.addListener('start', function () { return _this.onLeftSashDragStart(); });
            this.leftSash.addListener('change', function (e) { return _this.onLeftSashDrag(e); });
            this.leftSash.addListener('end', function () { return _this.onLeftSashDragEnd(); });
            this.leftSash.hide();
            // Center Container
            this.containers[editor_2.Position.CENTER] = builder_1.$(parent).div({ class: 'one-editor-container editor-center' });
            // Right Sash
            this.rightSash = new sash_1.Sash(parent.getHTMLElement(), this, { baseSize: 5 });
            this.rightSash.addListener('start', function () { return _this.onRightSashDragStart(); });
            this.rightSash.addListener('change', function (e) { return _this.onRightSashDrag(e); });
            this.rightSash.addListener('end', function () { return _this.onRightSashDragEnd(); });
            this.rightSash.hide();
            // Right Container
            this.containers[editor_2.Position.RIGHT] = builder_1.$(parent).div({ class: 'one-editor-container editor-right' });
            // Title containers
            editor_2.POSITIONS.forEach(function (position) {
                _this.titleContainer[position] = builder_1.$(_this.containers[position]).div({ 'class': 'title monaco-editor-background' });
                _this.fillTitleArea(builder_1.$(_this.titleContainer[position]), position);
            });
            // Progress Bars per position
            editor_2.POSITIONS.forEach(function (position) {
                _this.progressBar[position] = new progressbar_1.ProgressBar(builder_1.$(_this.containers[position]));
                _this.progressBar[position].getContainer().hide();
            });
        };
        SideBySideEditorControl.prototype.updateEditorInputStateIndicator = function (inputEvent) {
            var _this = this;
            editor_2.POSITIONS.forEach(function (position) {
                if (_this.visibleEditors[position]) {
                    if (_this.isInputRelated(_this.visibleEditors[position].input, inputEvent.editorInput)) {
                        _this.setEditorInputStateIndicator(inputEvent.editorInput, inputEvent.editorInput.getStatus(), position);
                    }
                }
            });
        };
        SideBySideEditorControl.prototype.setEditorInputStateIndicator = function (input, status, position) {
            // Decoration
            var titleLabel = (input && input.getName()) || '';
            if (status && status.decoration) {
                titleLabel = nls.localize(2, null, status.decoration, titleLabel);
            }
            this.titleLabel[position].safeInnerHtml(titleLabel);
        };
        SideBySideEditorControl.prototype.isInputRelated = function (sourceInput, targetInput) {
            if (!sourceInput || !targetInput) {
                return false;
            }
            if (sourceInput === targetInput) {
                return true;
            }
            if (sourceInput instanceof diffEditorInput_1.DiffEditorInput) {
                var modifiedInput = sourceInput.getModifiedInput();
                if (modifiedInput === targetInput) {
                    return true;
                }
            }
            return false;
        };
        SideBySideEditorControl.prototype.fillTitleArea = function (parent, position) {
            var _this = this;
            var ignoreClick = false;
            var wasDragged = false;
            // Allow to reorder positions by dragging the title
            parent.on(DOM.EventType.MOUSE_DOWN, function (e) {
                // Reset flag
                ignoreClick = false;
                wasDragged = false;
                // Return early if there is only one editor active or the user clicked into the toolbar
                if (_this.getVisibleEditorCount() <= 1 || DOM.isAncestor(e.target || e.srcElement, _this.editorActionsToolbar[position].getContainer().getHTMLElement())) {
                    return;
                }
                // Only allow for first mouse button click!
                if (e.button !== 0) {
                    return;
                }
                DOM.EventHelper.stop(e);
                // Overlay the editor area with a div to be able to capture all mouse events (helps when iframes are used in any editor)
                var overlayDiv = builder_1.$('div').style({
                    position: 'absolute',
                    top: SideBySideEditorControl.EDITOR_TITLE_HEIGHT + 'px',
                    left: 0,
                    width: '100%',
                    height: '100%',
                    zIndex: 3000000
                }).id('monaco-workbench-editor-move-overlay');
                overlayDiv.appendTo(_this.parent);
                // Update flag
                _this.dragging = true;
                var visibleEditorCount = _this.getVisibleEditorCount();
                var mouseDownEvent = new mouseEvent_1.StandardMouseEvent(e);
                var startX = mouseDownEvent.posx;
                var oldNewLeft = null;
                _this.containers[position].style({
                    zIndex: 2000000
                });
                var $window = builder_1.$(window);
                $window.on(DOM.EventType.MOUSE_MOVE, function (e) {
                    DOM.EventHelper.stop(e, false);
                    var mouseMoveEvent = new mouseEvent_1.StandardMouseEvent(e);
                    var diffX = mouseMoveEvent.posx - startX;
                    var newLeft = null;
                    if (Math.abs(diffX) > 5) {
                        wasDragged = true;
                    }
                    switch (position) {
                        // [ ! ]|[ ]: Moves only to the right but not outside of dimension width to the right
                        case editor_2.Position.LEFT: {
                            newLeft = Math.max(-1 /* 1px border accomodation */, Math.min(diffX, _this.dimension.width - _this.containerWidth[editor_2.Position.LEFT]));
                            break;
                        }
                        case editor_2.Position.CENTER: {
                            // [ ]|[ ! ]: Moves only to the left but not outside of dimension width to the left
                            if (visibleEditorCount === 2) {
                                newLeft = Math.min(_this.containerWidth[editor_2.Position.LEFT], Math.max(-1 /* 1px border accomodation */, _this.containerWidth[editor_2.Position.LEFT] + diffX));
                            }
                            else {
                                newLeft = Math.min(_this.dimension.width - _this.containerWidth[editor_2.Position.CENTER], Math.max(-1 /* 1px border accomodation */, _this.containerWidth[editor_2.Position.LEFT] + diffX));
                            }
                            break;
                        }
                        // [ ]|[ ]|[ ! ]: Moves to the right but not outside of dimension width on the left side
                        case editor_2.Position.RIGHT: {
                            newLeft = Math.min(_this.containerWidth[editor_2.Position.LEFT] + _this.containerWidth[editor_2.Position.CENTER], Math.max(-1 /* 1px border accomodation */, _this.containerWidth[editor_2.Position.LEFT] + _this.containerWidth[editor_2.Position.CENTER] + diffX));
                            break;
                        }
                    }
                    // Return early if position did not change
                    if (oldNewLeft === newLeft) {
                        return;
                    }
                    oldNewLeft = newLeft;
                    // Live drag Feedback
                    var moveTo = _this.findMoveTarget(position, diffX);
                    switch (position) {
                        case editor_2.Position.LEFT: {
                            if (moveTo === editor_2.Position.LEFT || moveTo === null) {
                                _this.containers[editor_2.Position.CENTER].style({ left: _this.containerWidth[editor_2.Position.LEFT] + 'px', right: 'auto', borderLeftWidth: '1px' });
                                _this.containers[editor_2.Position.RIGHT].style({ left: 'auto', right: 0 });
                            }
                            else if (moveTo === editor_2.Position.CENTER) {
                                _this.containers[editor_2.Position.CENTER].style({ left: 0, right: 'auto', borderLeftWidth: 0 });
                                _this.containers[editor_2.Position.CENTER].addClass('draggedunder');
                                _this.containers[editor_2.Position.RIGHT].style({ left: 'auto', right: 0 });
                            }
                            else if (moveTo === editor_2.Position.RIGHT) {
                                _this.containers[editor_2.Position.CENTER].style({ left: 0, right: 'auto' });
                                _this.containers[editor_2.Position.RIGHT].style({ left: 'auto', right: _this.containerWidth[editor_2.Position.LEFT] + 'px' });
                                _this.containers[editor_2.Position.RIGHT].addClass('draggedunder');
                            }
                            break;
                        }
                        case editor_2.Position.CENTER: {
                            if (moveTo === editor_2.Position.LEFT) {
                                _this.containers[editor_2.Position.LEFT].style({ left: _this.containerWidth[editor_2.Position.CENTER] + 'px', right: 'auto' });
                                _this.containers[editor_2.Position.LEFT].addClass('draggedunder');
                            }
                            else if (moveTo === editor_2.Position.CENTER || moveTo === null) {
                                _this.containers[editor_2.Position.LEFT].style({ left: 0, right: 'auto' });
                                _this.containers[editor_2.Position.RIGHT].style({ left: 'auto', right: 0 });
                            }
                            else if (moveTo === editor_2.Position.RIGHT) {
                                _this.containers[editor_2.Position.RIGHT].style({ left: 'auto', right: _this.containerWidth[editor_2.Position.CENTER] + 'px' });
                                _this.containers[editor_2.Position.RIGHT].addClass('draggedunder');
                                _this.containers[editor_2.Position.LEFT].style({ left: 0, right: 'auto' });
                            }
                            break;
                        }
                        case editor_2.Position.RIGHT: {
                            if (moveTo === editor_2.Position.LEFT) {
                                _this.containers[editor_2.Position.LEFT].style({ left: _this.containerWidth[editor_2.Position.RIGHT] + 'px', right: 'auto' });
                                _this.containers[editor_2.Position.LEFT].addClass('draggedunder');
                            }
                            else if (moveTo === editor_2.Position.CENTER) {
                                _this.containers[editor_2.Position.LEFT].style({ left: 0, right: 'auto' });
                                _this.containers[editor_2.Position.CENTER].style({ left: (_this.containerWidth[editor_2.Position.LEFT] + _this.containerWidth[editor_2.Position.RIGHT]) + 'px', right: 'auto' });
                                _this.containers[editor_2.Position.CENTER].addClass('draggedunder');
                            }
                            else if (moveTo === editor_2.Position.RIGHT || moveTo === null) {
                                _this.containers[editor_2.Position.LEFT].style({ left: 0, right: 'auto' });
                                _this.containers[editor_2.Position.CENTER].style({ left: _this.containerWidth[editor_2.Position.LEFT] + 'px', right: 'auto' });
                            }
                            break;
                        }
                    }
                    // Move the editor to provide feedback to the user and add class
                    if (newLeft !== null) {
                        _this.containers[position].style({ left: newLeft + 'px' });
                        _this.containers[position].addClass('dragged');
                        _this.parent.addClass('dragged');
                    }
                }).once(DOM.EventType.MOUSE_UP, function (e) {
                    DOM.EventHelper.stop(e, false);
                    // Destroy overlay
                    overlayDiv.destroy();
                    // Update flag
                    _this.dragging = false;
                    // Restore styles
                    _this.parent.removeClass('dragged');
                    _this.containers[position].removeClass('dragged');
                    _this.containers[position].style({ zIndex: 'auto' });
                    editor_2.POSITIONS.forEach(function (p) { return _this.containers[p].removeClass('draggedunder'); });
                    _this.containers[editor_2.Position.LEFT].style({ left: 0, right: 'auto' });
                    _this.containers[editor_2.Position.CENTER].style({ left: 'auto', right: 'auto', borderLeftWidth: '1px' });
                    _this.containers[editor_2.Position.RIGHT].style({ left: 'auto', right: 0, borderLeftWidth: '1px' });
                    // Find move target
                    var mouseUpEvent = new mouseEvent_1.StandardMouseEvent(e);
                    var diffX = mouseUpEvent.posx - startX;
                    var moveTo = _this.findMoveTarget(position, diffX);
                    // Move to valid position if any
                    if (moveTo !== null) {
                        _this.editorService.moveEditor(position, moveTo);
                    }
                    else {
                        _this.layoutContainers();
                    }
                    // Ignore next click if the user dragged the title some distance
                    if (wasDragged) {
                        ignoreClick = true;
                    }
                    else if (position !== _this.getActivePosition()) {
                        _this.editorService.focusEditor(_this.visibleEditors[position]).done(null, errors.onUnexpectedError);
                    }
                    $window.off('mousemove');
                });
            });
            // Close editor on middle mouse click
            parent.on(DOM.EventType.MOUSE_UP, function (e) {
                DOM.EventHelper.stop(e, false);
                // Close editor on middle mouse click
                if (e.button === 1 /* Middle Button */) {
                    _this.editorService.closeEditor(position).done(null, errors.onUnexpectedError);
                }
                else if (_this.getVisibleEditorCount() === 1 && !DOM.isAncestor(e.target || e.srcElement, _this.editorActionsToolbar[position].getContainer().getHTMLElement())) {
                    _this.editorService.focusEditor(_this.visibleEditors[position]).done(null, errors.onUnexpectedError);
                }
            });
            // Right Actions Container
            parent.div({
                'class': 'title-actions'
            }, function (div) {
                // Toolbar
                _this.editorActionsToolbar[position] = new toolbar_1.ToolBar(div.getHTMLElement(), _this.contextMenuService, {
                    actionItemProvider: function (action) { return _this.actionItemProvider(action, position); },
                    orientation: actionbar_1.ActionsOrientation.HORIZONTAL,
                    ariaLabel: nls.localize(3, null)
                });
                // Action Run Handling
                _this.editorActionsToolbar[position].actionRunner.addListener(events_1.EventType.RUN, function (e) {
                    // Check for Error
                    if (e.error && !errors.isPromiseCanceledError(e.error)) {
                        _this.messageService.show(message_1.Severity.Error, e.error);
                    }
                    // Log in telemetry
                    if (_this.telemetryService) {
                        _this.telemetryService.publicLog('workbenchActionExecuted', { id: e.action.id, from: 'editorPart' });
                    }
                });
            });
            // Left Title Label (click opens quick open unless we are configured to ignore click or we are not the active title)
            parent.div({
                'class': 'title-label'
            }, function (div) {
                var clickHandler = function (e) {
                    if (ignoreClick) {
                        return;
                    }
                    DOM.EventHelper.stop(e, true);
                    _this.quickOpenService.show();
                };
                // Clickable label (focus editor and bring up quick open)
                _this.titleLabel[position] = builder_1.$(div).a().on(DOM.EventType.CLICK, clickHandler);
                // Subtle Description
                _this.titleDescription[position] = builder_1.$(div).span().on(DOM.EventType.CLICK, clickHandler);
            });
        };
        SideBySideEditorControl.prototype.findMoveTarget = function (position, diffX) {
            var visibleEditorCount = this.getVisibleEditorCount();
            switch (position) {
                case editor_2.Position.LEFT: {
                    // [ ! ]|[] -> []|[ ! ]
                    if (visibleEditorCount === 2 && (diffX >= this.containerWidth[editor_2.Position.LEFT] / 2 || diffX >= this.containerWidth[editor_2.Position.CENTER] / 2)) {
                        return editor_2.Position.CENTER;
                    }
                    // [ ! ]|[]|[] -> []|[]|[ ! ]
                    if (visibleEditorCount === 3 && (diffX >= this.containerWidth[editor_2.Position.LEFT] / 2 + this.containerWidth[editor_2.Position.CENTER] || diffX >= this.containerWidth[editor_2.Position.RIGHT] / 2 + this.containerWidth[editor_2.Position.CENTER])) {
                        return editor_2.Position.RIGHT;
                    }
                    // [ ! ]|[]|[] -> []|[ ! ]|[]
                    if (visibleEditorCount === 3 && (diffX >= this.containerWidth[editor_2.Position.LEFT] / 2 || diffX >= this.containerWidth[editor_2.Position.CENTER] / 2)) {
                        return editor_2.Position.CENTER;
                    }
                    break;
                }
                case editor_2.Position.CENTER: {
                    if (visibleEditorCount === 2 && diffX > 0) {
                        return null; // Return early since CENTER cannot be moved to the RIGHT unless there is a RIGHT position
                    }
                    // []|[ ! ] -> [ ! ]|[]
                    if (visibleEditorCount === 2 && (Math.abs(diffX) >= this.containerWidth[editor_2.Position.CENTER] / 2 || Math.abs(diffX) >= this.containerWidth[editor_2.Position.LEFT] / 2)) {
                        return editor_2.Position.LEFT;
                    }
                    // []|[ ! ]|[] -> [ ! ]|[]|[]
                    if (visibleEditorCount === 3 && ((diffX < 0 && Math.abs(diffX) >= this.containerWidth[editor_2.Position.CENTER] / 2) || (diffX < 0 && Math.abs(diffX) >= this.containerWidth[editor_2.Position.LEFT] / 2))) {
                        return editor_2.Position.LEFT;
                    }
                    // []|[ ! ]|[] -> []|[]|[ ! ]
                    if (visibleEditorCount === 3 && ((diffX > 0 && Math.abs(diffX) >= this.containerWidth[editor_2.Position.CENTER] / 2) || (diffX > 0 && Math.abs(diffX) >= this.containerWidth[editor_2.Position.RIGHT] / 2))) {
                        return editor_2.Position.RIGHT;
                    }
                    break;
                }
                case editor_2.Position.RIGHT: {
                    if (diffX > 0) {
                        return null; // Return early since RIGHT cannot be moved more to the RIGHT
                    }
                    // []|[]|[ ! ] -> [ ! ]|[]|[]
                    if (Math.abs(diffX) >= this.containerWidth[editor_2.Position.RIGHT] / 2 + this.containerWidth[editor_2.Position.CENTER] || Math.abs(diffX) >= this.containerWidth[editor_2.Position.LEFT] / 2 + this.containerWidth[editor_2.Position.CENTER]) {
                        return editor_2.Position.LEFT;
                    }
                    // []|[]|[ ! ] -> []|[ ! ]|[]
                    if (Math.abs(diffX) >= this.containerWidth[editor_2.Position.RIGHT] / 2 || Math.abs(diffX) >= this.containerWidth[editor_2.Position.CENTER] / 2) {
                        return editor_2.Position.CENTER;
                    }
                    break;
                }
            }
            return null;
        };
        SideBySideEditorControl.prototype.actionItemProvider = function (action, position) {
            var actionItem;
            // Check Active Editor
            var editor = this.visibleEditors[position];
            if (editor) {
                actionItem = editor.getActionItem(action);
            }
            // Check Registry
            if (!actionItem) {
                var actionBarRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
                actionItem = actionBarRegistry.getActionItemForContext(actionBarRegistry_1.Scope.EDITOR, { input: editor && editor.input, editor: editor, position: position }, action);
            }
            return actionItem;
        };
        SideBySideEditorControl.prototype.setTitle = function (position, input, primaryActions, secondaryActions, isActive) {
            // Activity class
            if (isActive) {
                this.containers[position].removeClass('inactive');
                this.containers[position].addClass('active');
            }
            else {
                this.containers[position].addClass('inactive');
                this.containers[position].removeClass('active');
            }
            // Editor Title (Label + Description)
            var name = input.getName() || '';
            var description = isActive ? (input.getDescription() || '') : '';
            var verboseDescription = isActive ? (input.getDescription(true) || '') : '';
            if (description === verboseDescription) {
                verboseDescription = ''; // dont repeat what is already shown
            }
            this.titleLabel[position].safeInnerHtml(name);
            this.titleLabel[position].title(verboseDescription);
            this.titleDescription[position].safeInnerHtml(description);
            this.titleDescription[position].title(verboseDescription);
            // Editor Input State Description
            if (input) {
                this.setEditorInputStateIndicator(input, input.getStatus(), position);
            }
            else {
                this.setEditorInputStateIndicator(null, null, null);
            }
            // Support split editor action if visible editor count is < 3 and editor supports it
            if (isActive && this.getVisibleEditorCount() < 3 && this.lastActiveEditor.supportsSplitEditor()) {
                primaryActions.unshift(this.splitEditorAction);
            }
            // Set Primary/Secondary Actions
            this.editorActionsToolbar[position].setActions(primaryActions, secondaryActions)();
            // Add a close action
            if (input) {
                this.editorActionsToolbar[position].addPrimaryAction(this.closeEditorAction[position])();
            }
        };
        SideBySideEditorControl.prototype.setLoading = function (position, input) {
            // Editor Title and Description
            this.titleLabel[position].safeInnerHtml(input.getName() || '');
            this.titleDescription[position].safeInnerHtml(nls.localize(4, null));
            // Clear Primary/Secondary Actions
            this.editorActionsToolbar[position].setActions([], [])();
            // Add a close action
            if (input) {
                this.editorActionsToolbar[position].addPrimaryAction(this.closeEditorAction[position])();
            }
        };
        SideBySideEditorControl.prototype.clearTitle = function (position) {
            // Editor Title
            this.titleLabel[position].safeInnerHtml('');
            this.titleDescription[position].safeInnerHtml('');
            // Toolbar
            this.editorActionsToolbar[position].setActions([], [])();
        };
        SideBySideEditorControl.prototype.onLeftSashDragStart = function () {
            this.startLeftContainerWidth = this.containerWidth[editor_2.Position.LEFT];
        };
        SideBySideEditorControl.prototype.onLeftSashDrag = function (e) {
            var oldLeftContainerWidth = this.containerWidth[editor_2.Position.LEFT];
            var newLeftContainerWidth = this.startLeftContainerWidth + e.currentX - e.startX;
            // Side-by-Side
            if (this.rightSash.isHidden()) {
                // []|[      ] : left side can not get smaller than MIN_EDITOR_WIDTH
                if (newLeftContainerWidth < SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    newLeftContainerWidth = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                }
                else if (this.dimension.width - newLeftContainerWidth < SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    newLeftContainerWidth = this.dimension.width - SideBySideEditorControl.MIN_EDITOR_WIDTH;
                }
                else if (newLeftContainerWidth - SideBySideEditorControl.SNAP_TO_MINIMIZED_THRESHOLD <= SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    newLeftContainerWidth = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                }
                else if (this.dimension.width - newLeftContainerWidth - SideBySideEditorControl.SNAP_TO_MINIMIZED_THRESHOLD <= SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    newLeftContainerWidth = this.dimension.width - SideBySideEditorControl.MIN_EDITOR_WIDTH;
                }
                this.containerWidth[editor_2.Position.LEFT] = newLeftContainerWidth;
                this.containerWidth[editor_2.Position.CENTER] = this.dimension.width - newLeftContainerWidth;
            }
            else {
                // [!]|[      ]|[  ] : left side can not get smaller than MIN_EDITOR_WIDTH
                if (newLeftContainerWidth < SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    newLeftContainerWidth = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                }
                else if (this.dimension.width - newLeftContainerWidth - this.containerWidth[editor_2.Position.RIGHT] < SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    // [      ]|[ ]|[!] : right side can not get smaller than MIN_EDITOR_WIDTH
                    if (this.dimension.width - newLeftContainerWidth - this.containerWidth[editor_2.Position.CENTER] < SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                        newLeftContainerWidth = this.dimension.width - (2 * SideBySideEditorControl.MIN_EDITOR_WIDTH);
                        this.containerWidth[editor_2.Position.CENTER] = this.containerWidth[editor_2.Position.RIGHT] = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                    }
                    else if (this.dimension.width - newLeftContainerWidth - this.containerWidth[editor_2.Position.CENTER] - SideBySideEditorControl.SNAP_TO_MINIMIZED_THRESHOLD <= SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                        this.containerWidth[editor_2.Position.RIGHT] = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                    }
                    else {
                        this.containerWidth[editor_2.Position.RIGHT] = this.containerWidth[editor_2.Position.RIGHT] - (newLeftContainerWidth - oldLeftContainerWidth);
                    }
                    this.rightSash.layout();
                }
                else if (newLeftContainerWidth - SideBySideEditorControl.SNAP_TO_MINIMIZED_THRESHOLD <= SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    newLeftContainerWidth = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                }
                else if (this.dimension.width - this.containerWidth[editor_2.Position.RIGHT] - newLeftContainerWidth - SideBySideEditorControl.SNAP_TO_MINIMIZED_THRESHOLD <= SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    newLeftContainerWidth = this.dimension.width - this.containerWidth[editor_2.Position.RIGHT] - SideBySideEditorControl.MIN_EDITOR_WIDTH;
                }
                this.containerWidth[editor_2.Position.LEFT] = newLeftContainerWidth;
                this.containerWidth[editor_2.Position.CENTER] = this.dimension.width - this.containerWidth[editor_2.Position.LEFT] - this.containerWidth[editor_2.Position.RIGHT];
            }
            // Pass on to containers
            this.layoutContainers();
        };
        SideBySideEditorControl.prototype.onLeftSashDragEnd = function () {
            this.leftSash.layout();
            this.rightSash.layout(); // Moving left sash might have also moved right sash, so layout() both
            this.focusNextNonMinimized();
        };
        SideBySideEditorControl.prototype.onRightSashDragStart = function () {
            this.startRightContainerWidth = this.containerWidth[editor_2.Position.RIGHT];
        };
        SideBySideEditorControl.prototype.onRightSashDrag = function (e) {
            var oldRightContainerWidth = this.containerWidth[editor_2.Position.RIGHT];
            var newRightContainerWidth = this.startRightContainerWidth - e.currentX + e.startX;
            // [  ]|[      ]|[!] : right side can not get smaller than MIN_EDITOR_WIDTH
            if (newRightContainerWidth < SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                newRightContainerWidth = SideBySideEditorControl.MIN_EDITOR_WIDTH;
            }
            else if (this.dimension.width - newRightContainerWidth - this.containerWidth[editor_2.Position.LEFT] < SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                // [!]|[ ]|[    ] : left side can not get smaller than MIN_EDITOR_WIDTH
                if (this.dimension.width - newRightContainerWidth - this.containerWidth[editor_2.Position.CENTER] < SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    newRightContainerWidth = this.dimension.width - (2 * SideBySideEditorControl.MIN_EDITOR_WIDTH);
                    this.containerWidth[editor_2.Position.LEFT] = this.containerWidth[editor_2.Position.CENTER] = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                }
                else if (this.dimension.width - newRightContainerWidth - this.containerWidth[editor_2.Position.CENTER] - SideBySideEditorControl.SNAP_TO_MINIMIZED_THRESHOLD <= SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                    this.containerWidth[editor_2.Position.LEFT] = SideBySideEditorControl.MIN_EDITOR_WIDTH;
                }
                else {
                    this.containerWidth[editor_2.Position.LEFT] = this.containerWidth[editor_2.Position.LEFT] - (newRightContainerWidth - oldRightContainerWidth);
                }
                this.leftSash.layout();
            }
            else if (newRightContainerWidth - SideBySideEditorControl.SNAP_TO_MINIMIZED_THRESHOLD <= SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                newRightContainerWidth = SideBySideEditorControl.MIN_EDITOR_WIDTH;
            }
            else if (this.dimension.width - this.containerWidth[editor_2.Position.LEFT] - newRightContainerWidth - SideBySideEditorControl.SNAP_TO_MINIMIZED_THRESHOLD <= SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                newRightContainerWidth = this.dimension.width - this.containerWidth[editor_2.Position.LEFT] - SideBySideEditorControl.MIN_EDITOR_WIDTH;
            }
            this.containerWidth[editor_2.Position.RIGHT] = newRightContainerWidth;
            this.containerWidth[editor_2.Position.CENTER] = this.dimension.width - this.containerWidth[editor_2.Position.LEFT] - this.containerWidth[editor_2.Position.RIGHT];
            this.layoutContainers();
        };
        SideBySideEditorControl.prototype.onRightSashDragEnd = function () {
            this.leftSash.layout(); // Moving right sash might have also moved left sash, so layout() both
            this.rightSash.layout();
            this.focusNextNonMinimized();
        };
        SideBySideEditorControl.prototype.getVerticalSashTop = function (sash) {
            return 0;
        };
        SideBySideEditorControl.prototype.getVerticalSashLeft = function (sash) {
            return sash === this.leftSash ? this.containerWidth[editor_2.Position.LEFT] : this.containerWidth[editor_2.Position.CENTER] + this.containerWidth[editor_2.Position.LEFT];
        };
        SideBySideEditorControl.prototype.getVerticalSashHeight = function (sash) {
            return this.dimension.height;
        };
        SideBySideEditorControl.prototype.isDragging = function () {
            return this.dragging;
        };
        SideBySideEditorControl.prototype.layout = function (arg) {
            if (arg instanceof builder_1.Dimension) {
                this.layoutControl(arg);
            }
            else {
                this.layoutEditor(arg);
            }
        };
        SideBySideEditorControl.prototype.layoutControl = function (dimension) {
            var _this = this;
            var oldDimension = this.dimension;
            this.dimension = dimension;
            // Use the current dimension in case an editor was opened before we had any dimension
            if (!oldDimension || !oldDimension.width || !oldDimension.height) {
                oldDimension = dimension;
            }
            // Apply to visible editors
            var totalWidth = 0;
            // Set preferred dimensions based on ratio to previous dimenions
            editor_2.POSITIONS.forEach(function (position) {
                if (_this.visibleEditors[position]) {
                    // Keep minimized editors in tact by not letting them grow if we have width to give
                    if (_this.containerWidth[position] !== SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                        var sashWidthRatio = void 0;
                        // We have some stored initial ratios when the editor was restored on startup
                        // Use those ratios over anything else but only once.
                        if (_this.containerInitialRatios && types.isNumber(_this.containerInitialRatios[position])) {
                            sashWidthRatio = _this.containerInitialRatios[position];
                            delete _this.containerInitialRatios[position]; // dont use again
                        }
                        else {
                            sashWidthRatio = _this.containerWidth[position] / oldDimension.width;
                        }
                        _this.containerWidth[position] = Math.max(Math.round(_this.dimension.width * sashWidthRatio), SideBySideEditorControl.MIN_EDITOR_WIDTH);
                    }
                    totalWidth += _this.containerWidth[position];
                }
            });
            // Compensate for overflow either through rounding error or min editor width
            if (totalWidth > 0) {
                var overflow_1 = totalWidth - this.dimension.width;
                // We have width to give
                if (overflow_1 < 0) {
                    // Find the first position from left to right that is not minimized
                    // to give width. This ensures that minimized editors are left like
                    // that if the user chose this layout.
                    var positionToGive_1 = null;
                    editor_2.POSITIONS.forEach(function (position) {
                        if (_this.visibleEditors[position] && positionToGive_1 === null && _this.containerWidth[position] !== SideBySideEditorControl.MIN_EDITOR_WIDTH) {
                            positionToGive_1 = position;
                        }
                    });
                    if (positionToGive_1 === null) {
                        positionToGive_1 = editor_2.Position.LEFT; // maybe all are minimized, so give LEFT the extra width
                    }
                    this.containerWidth[positionToGive_1] -= overflow_1;
                }
                else if (overflow_1 > 0) {
                    editor_2.POSITIONS.forEach(function (position) {
                        var maxCompensation = _this.containerWidth[position] - SideBySideEditorControl.MIN_EDITOR_WIDTH;
                        if (maxCompensation >= overflow_1) {
                            _this.containerWidth[position] -= overflow_1;
                            overflow_1 = 0;
                        }
                        else if (maxCompensation > 0) {
                            var compensation = overflow_1 - maxCompensation;
                            _this.containerWidth[position] -= compensation;
                            overflow_1 -= compensation;
                        }
                    });
                }
            }
            // Sash positioning
            this.leftSash.layout();
            this.rightSash.layout();
            // Pass on to Editor Containers
            this.layoutContainers();
        };
        SideBySideEditorControl.prototype.layoutContainers = function () {
            var _this = this;
            // Layout containers
            editor_2.POSITIONS.forEach(function (position) {
                _this.containers[position].size(_this.containerWidth[position], _this.dimension.height);
            });
            // Position center depending on visibility of right hand editor
            if (this.visibleEditors[editor_2.Position.RIGHT]) {
                this.containers[editor_2.Position.CENTER].position(null, this.containerWidth[editor_2.Position.RIGHT]);
            }
            else {
                this.containers[editor_2.Position.CENTER].position(null, this.dimension.width - this.containerWidth[editor_2.Position.LEFT] - this.containerWidth[editor_2.Position.CENTER]);
            }
            // Visibility
            editor_2.POSITIONS.forEach(function (position) {
                if (_this.visibleEditors[position] && _this.containers[position].isHidden()) {
                    _this.containers[position].show();
                }
                else if (!_this.visibleEditors[position] && !_this.containers[position].isHidden()) {
                    _this.containers[position].hide();
                }
            });
            // Layout active editors
            editor_2.POSITIONS.forEach(function (position) {
                _this.layoutEditor(position);
            });
        };
        SideBySideEditorControl.prototype.layoutEditor = function (position) {
            var editorWidth = this.containerWidth[position];
            if (editorWidth && this.visibleEditors[position]) {
                this.visibleEditors[position].layout(new builder_1.Dimension(editorWidth, this.dimension.height - SideBySideEditorControl.EDITOR_TITLE_HEIGHT));
            }
        };
        SideBySideEditorControl.prototype.getProgressBar = function (position) {
            return this.progressBar[position];
        };
        SideBySideEditorControl.prototype.dispose = function () {
            var _this = this;
            // Positions
            editor_2.POSITIONS.forEach(function (position) {
                _this.clearPosition(position);
            });
            // Toolbars
            this.editorActionsToolbar.forEach(function (toolbar) {
                toolbar.dispose();
            });
            // Progress bars
            this.progressBar.forEach(function (bar) {
                bar.dispose();
            });
            // Actions
            this.closeEditorAction.forEach(function (action) {
                action.dispose();
            });
            // Sash
            this.leftSash.dispose();
            this.rightSash.dispose();
            // Destroy Container
            this.containers.forEach(function (container) {
                container.destroy();
            });
            if (this.editorInputStateChangeListener) {
                this.editorInputStateChangeListener();
            }
            this.lastActiveEditor = null;
            this.lastActivePosition = null;
            this.visibleEditors = null;
            this.visibleEditorContainers = null;
            _super.prototype.dispose.call(this);
        };
        SideBySideEditorControl.MIN_EDITOR_WIDTH = 170;
        SideBySideEditorControl.EDITOR_TITLE_HEIGHT = 35;
        SideBySideEditorControl.SNAP_TO_MINIMIZED_THRESHOLD = 50;
        SideBySideEditorControl = __decorate([
            __param(1, editorService_1.IWorkbenchEditorService),
            __param(2, message_1.IMessageService),
            __param(3, quickOpenService_1.IQuickOpenService),
            __param(4, telemetry_1.ITelemetryService),
            __param(5, contextView_1.IContextViewService),
            __param(6, contextView_1.IContextMenuService),
            __param(7, event_1.IEventService),
            __param(8, contextService_1.IWorkspaceContextService)
        ], SideBySideEditorControl);
        return SideBySideEditorControl;
    }(eventEmitter_1.EventEmitter));
    exports.SideBySideEditorControl = SideBySideEditorControl;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/editor/textEditor", ["require", "exports", 'vs/base/common/objects', 'vs/base/common/errors', 'vs/editor/browser/widget/codeEditorWidget', 'vs/workbench/common/constants', 'vs/workbench/common/events', 'vs/workbench/common/memento', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/editor/common/config/commonEditorConfig', 'vs/editor/common/editorCommon', 'vs/workbench/services/workspace/common/contextService', 'vs/workbench/services/themes/common/themeService', 'vs/platform/storage/common/storage', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/platform/telemetry/common/telemetry', 'vs/workbench/services/editor/common/editorService', 'vs/editor/common/services/modeService', 'vs/css!./media/texteditor'], function (require, exports, objects, errors, codeEditorWidget_1, constants_1, events_1, memento_1, baseEditor_1, commonEditorConfig_1, editorCommon_1, contextService_1, themeService_1, storage_1, configuration_1, event_1, instantiation_1, message_1, telemetry_1, editorService_1, modeService_1) {
    'use strict';
    var EDITOR_VIEW_STATE_PREFERENCE_KEY = 'editorViewState';
    /**
     * The base class of editors that leverage the monaco text editor for the editing experience. This class is only intended to
     * be subclassed and not instantiated.
     */
    var BaseTextEditor = (function (_super) {
        __extends(BaseTextEditor, _super);
        function BaseTextEditor(id, telemetryService, _instantiationService, _contextService, _storageService, _messageService, configurationService, _eventService, _editorService, _modeService) {
            var _this = this;
            _super.call(this, id, telemetryService);
            this._instantiationService = _instantiationService;
            this._contextService = _contextService;
            this._storageService = _storageService;
            this._messageService = _messageService;
            this.configurationService = configurationService;
            this._eventService = _eventService;
            this._editorService = _editorService;
            this._modeService = _modeService;
            this.toUnbind.push(this._eventService.addListener(storage_1.StorageEventType.STORAGE, function (e) { return _this.onPreferencesChanged(e); }));
            this.toUnbind.push(this._eventService.addListener(events_1.EventType.WORKBENCH_OPTIONS_CHANGED, function (e) { return _this.onOptionsChanged(e); }));
            this.toUnbind.push(this.configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) { return _this.applyConfiguration(e.config); }));
        }
        Object.defineProperty(BaseTextEditor.prototype, "instantiationService", {
            get: function () {
                return this._instantiationService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseTextEditor.prototype, "contextService", {
            get: function () {
                return this._contextService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseTextEditor.prototype, "storageService", {
            get: function () {
                return this._storageService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseTextEditor.prototype, "messageService", {
            get: function () {
                return this._messageService;
            },
            enumerable: true,
            configurable: true
        });
        BaseTextEditor.prototype.applyConfiguration = function (configuration) {
            // Update Editor with configuration and editor settings
            if (this.editorControl) {
                var specificEditorSettings = this.getCodeEditorOptions();
                configuration = objects.clone(configuration); // dont modify original config
                objects.assign(configuration[commonEditorConfig_1.EditorConfiguration.EDITOR_SECTION], specificEditorSettings);
                commonEditorConfig_1.EditorConfiguration.apply(configuration, this.editorControl);
            }
            // Update Languages
            this._modeService.configureAllModes(configuration);
        };
        BaseTextEditor.prototype.onOptionsChanged = function (event) {
            if (this.editorControl) {
                this.editorControl.updateOptions(this.getCodeEditorOptions());
            }
        };
        BaseTextEditor.prototype.onPreferencesChanged = function (e) {
            // Update Theme in Editor Control
            if (e.key === constants_1.Preferences.THEME) {
                this.editorControl.updateOptions(this.getCodeEditorOptions());
            }
        };
        BaseTextEditor.prototype.getCodeEditorOptions = function () {
            var baseOptions = {
                overviewRulerLanes: 3,
                readOnly: this.contextService.getOptions().readOnly,
                glyphMargin: true,
                lineNumbersMinChars: 3,
                theme: this._storageService.get(constants_1.Preferences.THEME, storage_1.StorageScope.GLOBAL, themeService_1.DEFAULT_THEME_ID)
            };
            // Always mixin editor options from the context into our set to allow for override
            return objects.mixin(baseOptions, this.contextService.getOptions().editor);
        };
        Object.defineProperty(BaseTextEditor.prototype, "eventService", {
            get: function () {
                return this._eventService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseTextEditor.prototype, "editorService", {
            get: function () {
                return this._editorService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseTextEditor.prototype, "editorContainer", {
            get: function () {
                return this._editorContainer;
            },
            enumerable: true,
            configurable: true
        });
        BaseTextEditor.prototype.createEditor = function (parent) {
            var _this = this;
            // Editor for Text
            this._editorContainer = parent;
            this.editorControl = this.createEditorControl(parent);
            // Hook Listener for Selection changes
            this.toUnbind.push(this.editorControl.addListener(editorCommon_1.EventType.CursorPositionChanged, function (event) {
                var selection = _this.editorControl.getSelection();
                _this.eventService.emit(events_1.EventType.TEXT_EDITOR_SELECTION_CHANGED, new events_1.TextEditorSelectionEvent(selection, _this, _this.getId(), _this.input, null, _this.position, event));
            }));
            // Hook Listener for mode changes
            this.toUnbind.push(this.editorControl.addListener(editorCommon_1.EventType.ModelModeChanged, function (event) {
                _this.eventService.emit(events_1.EventType.TEXT_EDITOR_MODE_CHANGED, new events_1.EditorEvent(_this, _this.getId(), _this.input, null, _this.position, event));
            }));
            // Hook Listener for content changes
            this.toUnbind.push(this.editorControl.addListener(editorCommon_1.EventType.ModelContentChanged, function (event) {
                _this.eventService.emit(events_1.EventType.TEXT_EDITOR_CONTENT_CHANGED, new events_1.EditorEvent(_this, _this.getId(), _this.input, null, _this.position, event));
            }));
            // Hook Listener for content options changes
            this.toUnbind.push(this.editorControl.addListener(editorCommon_1.EventType.ModelOptionsChanged, function (event) {
                _this.eventService.emit(events_1.EventType.TEXT_EDITOR_CONTENT_OPTIONS_CHANGED, new events_1.EditorEvent(_this, _this.getId(), _this.input, null, _this.position, event));
            }));
            // Hook Listener for options changes
            this.toUnbind.push(this.editorControl.addListener(editorCommon_1.EventType.ConfigurationChanged, function (event) {
                _this.eventService.emit(events_1.EventType.TEXT_EDITOR_CONFIGURATION_CHANGED, new events_1.EditorEvent(_this, _this.getId(), _this.input, null, _this.position, event));
            }));
            // Configuration
            this.configurationService.loadConfiguration().then(function (config) {
                _this.applyConfiguration(config);
            }, errors.onUnexpectedError);
        };
        /**
         * This method creates and returns the text editor control to be used. Subclasses can override to
         * provide their own editor control that should be used (e.g. a DiffEditor).
         */
        BaseTextEditor.prototype.createEditorControl = function (parent) {
            return this._instantiationService.createInstance(codeEditorWidget_1.CodeEditorWidget, parent.getHTMLElement(), this.getCodeEditorOptions());
        };
        BaseTextEditor.prototype.setInput = function (input, options) {
            var _this = this;
            return _super.prototype.setInput.call(this, input, options).then(function () {
                _this.editorControl.updateOptions(_this.getCodeEditorOptions()); // support input specific editor options
            });
        };
        BaseTextEditor.prototype.setVisible = function (visible, position) {
            if (position === void 0) { position = null; }
            var promise = _super.prototype.setVisible.call(this, visible, position);
            // Pass on to Editor
            if (visible) {
                this.editorControl.onVisible();
            }
            else {
                this.editorControl.onHide();
            }
            return promise;
        };
        BaseTextEditor.prototype.focus = function () {
            this.editorControl.focus();
        };
        BaseTextEditor.prototype.layout = function (dimension) {
            // Pass on to Editor
            this.editorControl.layout(dimension);
        };
        BaseTextEditor.prototype.getControl = function () {
            return this.editorControl;
        };
        BaseTextEditor.prototype.getSelection = function () {
            return this.editorControl.getSelection();
        };
        /**
         * Saves the text editor view state under the given key.
         */
        BaseTextEditor.prototype.saveTextEditorViewState = function (storageService, key) {
            var editorViewState = this.editorControl.saveViewState();
            var memento = this.getMemento(storageService, memento_1.Scope.WORKSPACE);
            var editorViewStateMemento = memento[EDITOR_VIEW_STATE_PREFERENCE_KEY];
            if (!editorViewStateMemento) {
                editorViewStateMemento = {};
                memento[EDITOR_VIEW_STATE_PREFERENCE_KEY] = editorViewStateMemento;
            }
            editorViewStateMemento[key] = editorViewState;
        };
        /**
         * Clears the text editor view state under the given key.
         */
        BaseTextEditor.prototype.clearTextEditorViewState = function (storageService, keys) {
            var memento = this.getMemento(storageService, memento_1.Scope.WORKSPACE);
            var editorViewStateMemento = memento[EDITOR_VIEW_STATE_PREFERENCE_KEY];
            if (editorViewStateMemento) {
                keys.forEach(function (key) { return delete editorViewStateMemento[key]; });
            }
        };
        /**
         * Loads the text editor view state for the given key and returns it.
         */
        BaseTextEditor.prototype.loadTextEditorViewState = function (storageService, key) {
            var memento = this.getMemento(storageService, memento_1.Scope.WORKSPACE);
            var editorViewStateMemento = memento[EDITOR_VIEW_STATE_PREFERENCE_KEY];
            if (editorViewStateMemento) {
                return editorViewStateMemento[key];
            }
            return null;
        };
        BaseTextEditor.prototype.dispose = function () {
            // Destroy Editor Control
            this.editorControl.destroy();
            _super.prototype.dispose.call(this);
        };
        BaseTextEditor = __decorate([
            __param(1, telemetry_1.ITelemetryService),
            __param(2, instantiation_1.IInstantiationService),
            __param(3, contextService_1.IWorkspaceContextService),
            __param(4, storage_1.IStorageService),
            __param(5, message_1.IMessageService),
            __param(6, configuration_1.IConfigurationService),
            __param(7, event_1.IEventService),
            __param(8, editorService_1.IWorkbenchEditorService),
            __param(9, modeService_1.IModeService)
        ], BaseTextEditor);
        return BaseTextEditor;
    }(baseEditor_1.BaseEditor));
    exports.BaseTextEditor = BaseTextEditor;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/editor/editorStatus", ["require", "exports", 'vs/nls!vs/workbench/browser/parts/editor/editorStatus', 'vs/base/common/winjs.base', 'vs/base/browser/dom', 'vs/base/common/strings', 'vs/base/common/paths', 'vs/base/common/types', 'vs/base/common/errors', 'vs/base/common/actions', 'vs/workbench/common/editor/untitledEditorInput', 'vs/workbench/common/editor', 'vs/base/common/lifecycle', 'vs/platform/message/common/message', 'vs/workbench/browser/actions/openSettings', 'vs/editor/common/editorCommon', 'vs/editor/contrib/indentation/common/indentation', 'vs/workbench/common/events', 'vs/workbench/browser/parts/editor/textEditor', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event', 'vs/platform/files/common/files', 'vs/platform/instantiation/common/instantiation', 'vs/editor/common/services/modeService', 'vs/base/browser/styleMutator', 'vs/css!./media/editorstatus'], function (require, exports, nls, winjs_base_1, dom_1, strings, paths, types, errors, actions_1, untitledEditorInput_1, editor_1, lifecycle_1, message_1, openSettings_1, editorCommon_1, indentation_1, events_1, textEditor_1, editorService_1, quickOpenService_1, configuration_1, event_1, files_1, instantiation_1, modeService_1, styleMutator_1) {
    'use strict';
    function getCodeEditor(e) {
        if (e instanceof textEditor_1.BaseTextEditor) {
            var editorWidget = e.getControl();
            if (editorWidget.getEditorType() === editorCommon_1.EditorType.IDiffEditor) {
                return editorWidget.getModifiedEditor();
            }
            if (editorWidget.getEditorType() === editorCommon_1.EditorType.ICodeEditor) {
                return editorWidget;
            }
            return null;
        }
        return null;
    }
    function getTextModel(editorWidget) {
        var textModel;
        // Support for diff
        var model = editorWidget.getModel();
        if (model && !!model.modified) {
            textModel = model.modified;
        }
        else {
            textModel = model;
        }
        return textModel;
    }
    function asFileOrUntitledEditorInput(input) {
        if (input instanceof untitledEditorInput_1.UntitledEditorInput) {
            return input;
        }
        return editor_1.asFileEditorInput(input, true /* support diff editor */);
    }
    var State = (function () {
        function State() {
            this._selectionStatus = null;
            this._mode = null;
            this._encoding = null;
            this._EOL = null;
            this._tabFocusMode = false;
        }
        Object.defineProperty(State.prototype, "selectionStatus", {
            get: function () { return this._selectionStatus; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(State.prototype, "mode", {
            get: function () { return this._mode; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(State.prototype, "encoding", {
            get: function () { return this._encoding; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(State.prototype, "EOL", {
            get: function () { return this._EOL; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(State.prototype, "indentation", {
            get: function () { return this._indentation; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(State.prototype, "tabFocusMode", {
            get: function () { return this._tabFocusMode; },
            enumerable: true,
            configurable: true
        });
        State.prototype.update = function (update) {
            var e = {
                selectionStatus: false,
                mode: false,
                encoding: false,
                EOL: false,
                tabFocusMode: false,
                indentation: false
            };
            var somethingChanged = false;
            if (typeof update.selectionStatus !== 'undefined') {
                if (this._selectionStatus !== update.selectionStatus) {
                    this._selectionStatus = update.selectionStatus;
                    somethingChanged = true;
                    e.selectionStatus = true;
                }
            }
            if (typeof update.indentation !== 'undefined') {
                if (this._indentation !== update.indentation) {
                    this._indentation = update.indentation;
                    somethingChanged = true;
                    e.indentation = true;
                }
            }
            if (typeof update.mode !== 'undefined') {
                if (this._mode !== update.mode) {
                    this._mode = update.mode;
                    somethingChanged = true;
                    e.mode = true;
                }
            }
            if (typeof update.encoding !== 'undefined') {
                if (this._encoding !== update.encoding) {
                    this._encoding = update.encoding;
                    somethingChanged = true;
                    e.encoding = true;
                }
            }
            if (typeof update.EOL !== 'undefined') {
                if (this._EOL !== update.EOL) {
                    this._EOL = update.EOL;
                    somethingChanged = true;
                    e.EOL = true;
                }
            }
            if (typeof update.tabFocusMode !== 'undefined') {
                if (this._tabFocusMode !== update.tabFocusMode) {
                    this._tabFocusMode = update.tabFocusMode;
                    somethingChanged = true;
                    e.tabFocusMode = true;
                }
            }
            if (somethingChanged) {
                return e;
            }
            return null;
        };
        return State;
    }());
    var nlsSingleSelectionRange = nls.localize(0, null);
    var nlsSingleSelection = nls.localize(1, null);
    var nlsMultiSelectionRange = nls.localize(2, null);
    var nlsMultiSelection = nls.localize(3, null);
    var nlsEOLLF = nls.localize(4, null);
    var nlsEOLCRLF = nls.localize(5, null);
    var nlsTabFocusMode = nls.localize(6, null);
    function show(el) {
        styleMutator_1.StyleMutator.setDisplay(el, '');
    }
    function hide(el) {
        styleMutator_1.StyleMutator.setDisplay(el, 'none');
    }
    var EditorStatus = (function () {
        function EditorStatus(editorService, quickOpenService, instantiationService, eventService, modeService, configurationService) {
            this.editorService = editorService;
            this.quickOpenService = quickOpenService;
            this.instantiationService = instantiationService;
            this.eventService = eventService;
            this.modeService = modeService;
            this.configurationService = configurationService;
            this.toDispose = [];
            this.state = new State();
        }
        EditorStatus.prototype.render = function (container) {
            var _this = this;
            this.element = dom_1.append(container, dom_1.emmet('.editor-statusbar-item'));
            this.tabFocusModeElement = dom_1.append(this.element, dom_1.emmet('a.editor-status-tabfocusmode'));
            this.tabFocusModeElement.title = nls.localize(7, null);
            this.tabFocusModeElement.onclick = function () { return _this.onTabFocusModeClick(); };
            this.tabFocusModeElement.textContent = nlsTabFocusMode;
            hide(this.tabFocusModeElement);
            this.selectionElement = dom_1.append(this.element, dom_1.emmet('a.editor-status-selection'));
            this.selectionElement.title = nls.localize(8, null);
            this.selectionElement.onclick = function () { return _this.onSelectionClick(); };
            hide(this.selectionElement);
            this.indentationElement = dom_1.append(this.element, dom_1.emmet('a.editor-status-indentation'));
            this.indentationElement.title = nls.localize(9, null);
            this.indentationElement.onclick = function () { return _this.onIndentationClick(); };
            hide(this.indentationElement);
            this.encodingElement = dom_1.append(this.element, dom_1.emmet('a.editor-status-encoding'));
            this.encodingElement.title = nls.localize(10, null);
            this.encodingElement.onclick = function () { return _this.onEncodingClick(); };
            hide(this.encodingElement);
            this.eolElement = dom_1.append(this.element, dom_1.emmet('a.editor-status-eol'));
            this.eolElement.title = nls.localize(11, null);
            this.eolElement.onclick = function () { return _this.onEOLClick(); };
            hide(this.eolElement);
            this.modeElement = dom_1.append(this.element, dom_1.emmet('a.editor-status-mode'));
            this.modeElement.title = nls.localize(12, null);
            this.modeElement.onclick = function () { return _this.onModeClick(); };
            hide(this.modeElement);
            this.toDispose.push(this.eventService.addListener2(events_1.EventType.EDITOR_INPUT_CHANGED, function (e) { return _this.onEditorInputChange(e.editor); }), this.eventService.addListener2(events_1.EventType.RESOURCE_ENCODING_CHANGED, function (e) { return _this.onResourceEncodingChange(e.resource); }), this.eventService.addListener2(events_1.EventType.TEXT_EDITOR_SELECTION_CHANGED, function (e) { return _this.onSelectionChange(e.editor); }), this.eventService.addListener2(events_1.EventType.TEXT_EDITOR_MODE_CHANGED, function (e) { return _this.onModeChange(e.editor); }), this.eventService.addListener2(events_1.EventType.TEXT_EDITOR_CONTENT_CHANGED, function (e) { return _this.onEOLChange(e.editor); }), this.eventService.addListener2(events_1.EventType.TEXT_EDITOR_CONFIGURATION_CHANGED, function (e) { return _this.onTabFocusModeChange(e.editor); }), this.eventService.addListener2(events_1.EventType.TEXT_EDITOR_CONTENT_OPTIONS_CHANGED, function (e) { return _this.onIndentationChange(e.editor); }));
            return lifecycle_1.combinedDispose.apply(void 0, this.toDispose);
        };
        EditorStatus.prototype.updateState = function (update) {
            var changed = this.state.update(update);
            if (!changed) {
                // Nothing really changed
                return;
            }
            if (changed.tabFocusMode) {
                if (this.state.tabFocusMode && this.state.tabFocusMode === true) {
                    show(this.tabFocusModeElement);
                }
                else {
                    hide(this.tabFocusModeElement);
                }
            }
            if (changed.indentation) {
                if (this.state.indentation) {
                    this.indentationElement.textContent = this.state.indentation;
                    show(this.indentationElement);
                }
                else {
                    hide(this.indentationElement);
                }
            }
            if (changed.selectionStatus) {
                if (this.state.selectionStatus) {
                    this.selectionElement.textContent = this.state.selectionStatus;
                    show(this.selectionElement);
                }
                else {
                    hide(this.selectionElement);
                }
            }
            if (changed.encoding) {
                if (this.state.encoding) {
                    this.encodingElement.textContent = this.state.encoding;
                    show(this.encodingElement);
                }
                else {
                    hide(this.encodingElement);
                }
            }
            if (changed.EOL) {
                if (this.state.EOL) {
                    this.eolElement.textContent = this.state.EOL === '\r\n' ? nlsEOLCRLF : nlsEOLLF;
                    show(this.eolElement);
                }
                else {
                    hide(this.eolElement);
                }
            }
            if (changed.mode) {
                if (this.state.mode) {
                    this.modeElement.textContent = this.state.mode;
                    show(this.modeElement);
                }
                else {
                    hide(this.modeElement);
                }
            }
        };
        EditorStatus.prototype.getSelectionLabel = function (info) {
            if (!info || !info.selections) {
                return null;
            }
            if (info.selections.length === 1) {
                if (info.charactersSelected) {
                    return strings.format(nlsSingleSelectionRange, info.selections[0].positionLineNumber, info.selections[0].positionColumn, info.charactersSelected);
                }
                else {
                    return strings.format(nlsSingleSelection, info.selections[0].positionLineNumber, info.selections[0].positionColumn);
                }
            }
            else {
                if (info.charactersSelected) {
                    return strings.format(nlsMultiSelectionRange, info.selections.length, info.charactersSelected);
                }
                else {
                    return strings.format(nlsMultiSelection, info.selections.length);
                }
            }
        };
        EditorStatus.prototype.onModeClick = function () {
            var action = this.instantiationService.createInstance(ChangeModeAction, ChangeModeAction.ID, ChangeModeAction.LABEL);
            action.run().done(null, errors.onUnexpectedError);
            action.dispose();
        };
        EditorStatus.prototype.onIndentationClick = function () {
            var action = this.instantiationService.createInstance(ChangeIndentationAction, ChangeIndentationAction.ID, ChangeIndentationAction.LABEL);
            action.run().done(null, errors.onUnexpectedError);
            action.dispose();
        };
        EditorStatus.prototype.onSelectionClick = function () {
            this.quickOpenService.show(':'); // "Go to line"
        };
        EditorStatus.prototype.onEOLClick = function () {
            var action = this.instantiationService.createInstance(ChangeEOLAction, ChangeEOLAction.ID, ChangeEOLAction.LABEL);
            action.run().done(null, errors.onUnexpectedError);
            action.dispose();
        };
        EditorStatus.prototype.onEncodingClick = function () {
            var action = this.instantiationService.createInstance(ChangeEncodingAction, ChangeEncodingAction.ID, ChangeEncodingAction.LABEL);
            action.run().done(null, errors.onUnexpectedError);
            action.dispose();
        };
        EditorStatus.prototype.onTabFocusModeClick = function () {
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor instanceof textEditor_1.BaseTextEditor && isCodeEditorWithTabFocusMode(activeEditor)) {
                activeEditor.getControl().updateOptions({ tabFocusMode: false });
            }
        };
        EditorStatus.prototype.onEditorInputChange = function (e) {
            this.onSelectionChange(e);
            this.onModeChange(e);
            this.onEOLChange(e);
            this.onEncodingChange(e);
            this.onTabFocusModeChange(e);
            this.onIndentationChange(e);
        };
        EditorStatus.prototype.onModeChange = function (e) {
            if (e && !this.isActiveEditor(e)) {
                return;
            }
            var info = { mode: null };
            // We only support text based editors
            if (e instanceof textEditor_1.BaseTextEditor) {
                var editorWidget = e.getControl();
                var textModel = getTextModel(editorWidget);
                if (textModel) {
                    // Compute mode
                    if (!!textModel.getMode) {
                        var mode = textModel.getMode();
                        if (mode) {
                            info = { mode: this.modeService.getLanguageName(mode.getId()) };
                        }
                    }
                }
            }
            this.updateState(info);
        };
        EditorStatus.prototype.onIndentationChange = function (e) {
            if (e && !this.isActiveEditor(e)) {
                return;
            }
            var update = { indentation: null };
            if (e instanceof textEditor_1.BaseTextEditor) {
                var editorWidget = e.getControl();
                if (editorWidget) {
                    if (editorWidget.getEditorType() === editorCommon_1.EditorType.IDiffEditor) {
                        editorWidget = editorWidget.getModifiedEditor();
                    }
                    var model = editorWidget.getModel();
                    if (model) {
                        var modelOpts = model.getOptions();
                        update.indentation = (modelOpts.insertSpaces
                            ? nls.localize(13, null, modelOpts.tabSize)
                            : nls.localize(14, null, modelOpts.tabSize));
                    }
                }
            }
            this.updateState(update);
        };
        EditorStatus.prototype.onSelectionChange = function (e) {
            if (e && !this.isActiveEditor(e)) {
                return;
            }
            var info = {};
            // We only support text based editors
            if (e instanceof textEditor_1.BaseTextEditor) {
                var editorWidget = e.getControl();
                // Compute selection(s)
                info.selections = editorWidget.getSelections() || [];
                // Compute selection length
                info.charactersSelected = 0;
                var textModel_1 = getTextModel(editorWidget);
                if (textModel_1) {
                    info.selections.forEach(function (selection) {
                        info.charactersSelected += textModel_1.getValueLengthInRange(selection);
                    });
                }
                // Compute the visible column for one selection. This will properly handle tabs and their configured widths
                if (info.selections.length === 1) {
                    var visibleColumn = editorWidget.getVisibleColumnFromPosition(editorWidget.getPosition());
                    var selectionClone = info.selections[0].clone(); // do not modify the original position we got from the editor
                    selectionClone.positionColumn = visibleColumn;
                    info.selections[0] = selectionClone;
                }
            }
            this.updateState({ selectionStatus: this.getSelectionLabel(info) });
        };
        EditorStatus.prototype.onEOLChange = function (e) {
            if (e && !this.isActiveEditor(e)) {
                return;
            }
            var info = { EOL: null };
            var codeEditor = getCodeEditor(e);
            if (codeEditor && !codeEditor.getConfiguration().readOnly) {
                var codeEditorModel = codeEditor.getModel();
                if (codeEditorModel) {
                    info.EOL = codeEditorModel.getEOL();
                }
            }
            this.updateState(info);
        };
        EditorStatus.prototype.onEncodingChange = function (e) {
            if (e && !this.isActiveEditor(e)) {
                return;
            }
            var info = { encoding: null };
            // We only support text based editors
            if (e instanceof textEditor_1.BaseTextEditor) {
                var encodingSupport = asFileOrUntitledEditorInput(e.input);
                if (encodingSupport && types.isFunction(encodingSupport.getEncoding)) {
                    var rawEncoding = encodingSupport.getEncoding();
                    var encodingInfo = files_1.SUPPORTED_ENCODINGS[rawEncoding];
                    if (encodingInfo) {
                        info.encoding = encodingInfo.labelShort; // if we have a label, take it from there
                    }
                    else {
                        info.encoding = rawEncoding; // otherwise use it raw
                    }
                }
            }
            this.updateState(info);
        };
        EditorStatus.prototype.onResourceEncodingChange = function (resource) {
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor) {
                var activeResource = editor_1.getUntitledOrFileResource(activeEditor.input, true);
                if (activeResource && activeResource.toString() === resource.toString()) {
                    return this.onEncodingChange(activeEditor); // only update if the encoding changed for the active resource
                }
            }
        };
        EditorStatus.prototype.onTabFocusModeChange = function (e) {
            if (e && !this.isActiveEditor(e)) {
                return;
            }
            var info = { tabFocusMode: false };
            // We only support text based editors
            if (e instanceof textEditor_1.BaseTextEditor && isCodeEditorWithTabFocusMode(e)) {
                info = { tabFocusMode: true };
            }
            this.updateState(info);
        };
        EditorStatus.prototype.isActiveEditor = function (e) {
            var activeEditor = this.editorService.getActiveEditor();
            return activeEditor && e && activeEditor === e;
        };
        EditorStatus = __decorate([
            __param(0, editorService_1.IWorkbenchEditorService),
            __param(1, quickOpenService_1.IQuickOpenService),
            __param(2, instantiation_1.IInstantiationService),
            __param(3, event_1.IEventService),
            __param(4, modeService_1.IModeService),
            __param(5, configuration_1.IConfigurationService)
        ], EditorStatus);
        return EditorStatus;
    }());
    exports.EditorStatus = EditorStatus;
    function isCodeEditorWithTabFocusMode(e) {
        var editorWidget = e.getControl();
        if (editorWidget.getEditorType() === editorCommon_1.EditorType.IDiffEditor) {
            editorWidget = editorWidget.getModifiedEditor();
        }
        if (editorWidget.getEditorType() !== editorCommon_1.EditorType.ICodeEditor) {
            return false;
        }
        var editorConfig = editorWidget.getConfiguration();
        return editorConfig.tabFocusMode && !editorConfig.readOnly;
    }
    function isWritableCodeEditor(e) {
        var editorWidget = e.getControl();
        if (editorWidget.getEditorType() === editorCommon_1.EditorType.IDiffEditor) {
            editorWidget = editorWidget.getModifiedEditor();
        }
        return (editorWidget.getEditorType() === editorCommon_1.EditorType.ICodeEditor &&
            !editorWidget.getConfiguration().readOnly);
    }
    var ChangeModeAction = (function (_super) {
        __extends(ChangeModeAction, _super);
        function ChangeModeAction(actionId, actionLabel, modeService, editorService, messageService, instantiationService, quickOpenService) {
            _super.call(this, actionId, actionLabel);
            this.modeService = modeService;
            this.editorService = editorService;
            this.messageService = messageService;
            this.instantiationService = instantiationService;
            this.quickOpenService = quickOpenService;
        }
        ChangeModeAction.prototype.run = function () {
            var _this = this;
            var languages = this.modeService.getRegisteredLanguageNames();
            var activeEditor = this.editorService.getActiveEditor();
            if (!(activeEditor instanceof textEditor_1.BaseTextEditor)) {
                return this.quickOpenService.pick([{ label: nls.localize(16, null) }]);
            }
            var editorWidget = activeEditor.getControl();
            var textModel = getTextModel(editorWidget);
            var fileinput = editor_1.asFileEditorInput(activeEditor.input, true);
            // Compute mode
            var currentModeId;
            if (!!textModel.getMode) {
                var mode = textModel.getMode();
                if (mode) {
                    currentModeId = this.modeService.getLanguageName(mode.getId());
                }
            }
            // All languages are valid picks
            var picks = languages.sort().map(function (lang, index) {
                return {
                    label: lang,
                    description: currentModeId === lang ? nls.localize(17, null) : void 0
                };
            });
            picks[0].separator = { border: true, label: nls.localize(18, null) };
            // Offer action to configure via settings
            var configureLabel = nls.localize(19, null);
            if (fileinput) {
                var resource = fileinput.getResource();
                var ext = paths.extname(resource.fsPath) || paths.basename(resource.fsPath);
                if (ext) {
                    configureLabel = nls.localize(20, null, ext);
                }
            }
            var configureModeAssociations = {
                label: configureLabel
            };
            picks.unshift(configureModeAssociations);
            // Offer to "Auto Detect"
            var autoDetectMode = {
                label: nls.localize(21, null)
            };
            picks.unshift(autoDetectMode);
            return this.quickOpenService.pick(picks, { placeHolder: nls.localize(22, null) }).then(function (language) {
                if (language) {
                    activeEditor = _this.editorService.getActiveEditor();
                    if (activeEditor instanceof textEditor_1.BaseTextEditor) {
                        var editorWidget_1 = activeEditor.getControl();
                        var models = [];
                        var textModel_2 = getTextModel(editorWidget_1);
                        models.push(textModel_2);
                        // Support for original side of diff
                        var model = editorWidget_1.getModel();
                        if (model && !!model.original) {
                            models.push(model.original);
                        }
                        // Find mode
                        var mode_1;
                        if (language === autoDetectMode) {
                            mode_1 = _this.modeService.getOrCreateModeByFilenameOrFirstLine(editor_1.getUntitledOrFileResource(activeEditor.input, true).fsPath, textModel_2.getLineContent(1));
                        }
                        else if (language === configureModeAssociations) {
                            var action_1 = _this.instantiationService.createInstance(openSettings_1.OpenGlobalSettingsAction, openSettings_1.OpenGlobalSettingsAction.ID, openSettings_1.OpenGlobalSettingsAction.LABEL);
                            action_1.run().done(function () { return action_1.dispose(); }, errors.onUnexpectedError);
                            _this.messageService.show(message_1.Severity.Info, nls.localize(23, null));
                        }
                        else {
                            mode_1 = _this.modeService.getOrCreateModeByLanguageName(language.label);
                        }
                        // Change mode
                        models.forEach(function (textModel) {
                            if (!!textModel.getMode) {
                                textModel.setMode(mode_1);
                            }
                        });
                    }
                }
            });
        };
        ChangeModeAction.ID = 'workbench.action.editor.changeLanguageMode';
        ChangeModeAction.LABEL = nls.localize(15, null);
        ChangeModeAction = __decorate([
            __param(2, modeService_1.IModeService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, message_1.IMessageService),
            __param(5, instantiation_1.IInstantiationService),
            __param(6, quickOpenService_1.IQuickOpenService)
        ], ChangeModeAction);
        return ChangeModeAction;
    }(actions_1.Action));
    exports.ChangeModeAction = ChangeModeAction;
    var ChangeIndentationAction = (function (_super) {
        __extends(ChangeIndentationAction, _super);
        function ChangeIndentationAction(actionId, actionLabel, editorService, quickOpenService) {
            _super.call(this, actionId, actionLabel);
            this.editorService = editorService;
            this.quickOpenService = quickOpenService;
        }
        ChangeIndentationAction.prototype.run = function () {
            var activeEditor = this.editorService.getActiveEditor();
            if (!(activeEditor instanceof textEditor_1.BaseTextEditor)) {
                return this.quickOpenService.pick([{ label: nls.localize(25, null) }]);
            }
            if (!isWritableCodeEditor(activeEditor)) {
                return this.quickOpenService.pick([{ label: nls.localize(26, null) }]);
            }
            var control = activeEditor.getControl();
            var picks = [control.getAction(indentation_1.IndentUsingSpaces.ID), control.getAction(indentation_1.IndentUsingTabs.ID), control.getAction(indentation_1.DetectIndentation.ID), control.getAction(indentation_1.IndentationToSpacesAction.ID), control.getAction(indentation_1.IndentationToTabsAction.ID)];
            picks[0].separator = { label: nls.localize(27, null) };
            picks[3].separator = { label: nls.localize(28, null), border: true };
            return this.quickOpenService.pick(picks, { placeHolder: nls.localize(29, null) }).then(function (action) { return action && action.run(); });
        };
        ChangeIndentationAction.ID = 'workbench.action.editor.changeIndentation';
        ChangeIndentationAction.LABEL = nls.localize(24, null);
        ChangeIndentationAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], ChangeIndentationAction);
        return ChangeIndentationAction;
    }(actions_1.Action));
    var ChangeEOLAction = (function (_super) {
        __extends(ChangeEOLAction, _super);
        function ChangeEOLAction(actionId, actionLabel, editorService, quickOpenService) {
            _super.call(this, actionId, actionLabel);
            this.editorService = editorService;
            this.quickOpenService = quickOpenService;
        }
        ChangeEOLAction.prototype.run = function () {
            var _this = this;
            var activeEditor = this.editorService.getActiveEditor();
            if (!(activeEditor instanceof textEditor_1.BaseTextEditor)) {
                return this.quickOpenService.pick([{ label: nls.localize(31, null) }]);
            }
            if (!isWritableCodeEditor(activeEditor)) {
                return this.quickOpenService.pick([{ label: nls.localize(32, null) }]);
            }
            var editorWidget = activeEditor.getControl();
            var textModel = getTextModel(editorWidget);
            var EOLOptions = [
                { label: nlsEOLLF, eol: editorCommon_1.EndOfLineSequence.LF },
                { label: nlsEOLCRLF, eol: editorCommon_1.EndOfLineSequence.CRLF },
            ];
            var selectedIndex = (textModel.getEOL() === '\n') ? 0 : 1;
            return this.quickOpenService.pick(EOLOptions, { placeHolder: nls.localize(33, null), autoFocus: { autoFocusIndex: selectedIndex } }).then(function (eol) {
                if (eol) {
                    activeEditor = _this.editorService.getActiveEditor();
                    if (activeEditor instanceof textEditor_1.BaseTextEditor && isWritableCodeEditor(activeEditor)) {
                        var editorWidget_2 = activeEditor.getControl();
                        var textModel_3 = getTextModel(editorWidget_2);
                        textModel_3.setEOL(eol.eol);
                    }
                }
            });
        };
        ChangeEOLAction.ID = 'workbench.action.editor.changeEOL';
        ChangeEOLAction.LABEL = nls.localize(30, null);
        ChangeEOLAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, quickOpenService_1.IQuickOpenService)
        ], ChangeEOLAction);
        return ChangeEOLAction;
    }(actions_1.Action));
    exports.ChangeEOLAction = ChangeEOLAction;
    var ChangeEncodingAction = (function (_super) {
        __extends(ChangeEncodingAction, _super);
        function ChangeEncodingAction(actionId, actionLabel, editorService, quickOpenService, configurationService) {
            _super.call(this, actionId, actionLabel);
            this.editorService = editorService;
            this.quickOpenService = quickOpenService;
            this.configurationService = configurationService;
        }
        ChangeEncodingAction.prototype.run = function () {
            var _this = this;
            var activeEditor = this.editorService.getActiveEditor();
            if (!(activeEditor instanceof textEditor_1.BaseTextEditor) || !activeEditor.input) {
                return this.quickOpenService.pick([{ label: nls.localize(35, null) }]);
            }
            var encodingSupport = asFileOrUntitledEditorInput(activeEditor.input);
            if (!types.areFunctions(encodingSupport.setEncoding, encodingSupport.getEncoding)) {
                return this.quickOpenService.pick([{ label: nls.localize(36, null) }]);
            }
            var pickActionPromise;
            var saveWithEncodingPick = { label: nls.localize(37, null) };
            var reopenWithEncodingPick = { label: nls.localize(38, null) };
            if (encodingSupport instanceof untitledEditorInput_1.UntitledEditorInput) {
                pickActionPromise = winjs_base_1.TPromise.as(saveWithEncodingPick);
            }
            else if (!isWritableCodeEditor(activeEditor)) {
                pickActionPromise = winjs_base_1.TPromise.as(reopenWithEncodingPick);
            }
            else {
                pickActionPromise = this.quickOpenService.pick([reopenWithEncodingPick, saveWithEncodingPick], { placeHolder: nls.localize(39, null) });
            }
            return pickActionPromise.then(function (action) {
                if (!action) {
                    return;
                }
                return winjs_base_1.TPromise.timeout(50 /* quick open is sensitive to being opened so soon after another */).then(function () {
                    var isReopenWithEncoding = (action === reopenWithEncodingPick);
                    return _this.configurationService.loadConfiguration().then(function (configuration) {
                        var configuredEncoding = configuration && configuration.files && configuration.files.encoding;
                        var directMatchIndex;
                        var aliasMatchIndex;
                        // All encodings are valid picks
                        var picks = Object.keys(files_1.SUPPORTED_ENCODINGS)
                            .sort(function (k1, k2) {
                            if (k1 === configuredEncoding) {
                                return -1;
                            }
                            else if (k2 === configuredEncoding) {
                                return 1;
                            }
                            return files_1.SUPPORTED_ENCODINGS[k1].order - files_1.SUPPORTED_ENCODINGS[k2].order;
                        })
                            .filter(function (k) {
                            return !isReopenWithEncoding || !files_1.SUPPORTED_ENCODINGS[k].encodeOnly; // hide those that can only be used for encoding if we are about to decode
                        })
                            .map(function (key, index) {
                            if (key === encodingSupport.getEncoding()) {
                                directMatchIndex = index;
                            }
                            else if (files_1.SUPPORTED_ENCODINGS[key].alias === encodingSupport.getEncoding()) {
                                aliasMatchIndex = index;
                            }
                            return { id: key, label: files_1.SUPPORTED_ENCODINGS[key].labelLong };
                        });
                        return _this.quickOpenService.pick(picks, {
                            placeHolder: isReopenWithEncoding ? nls.localize(40, null) : nls.localize(41, null),
                            autoFocus: { autoFocusIndex: typeof directMatchIndex === 'number' ? directMatchIndex : typeof aliasMatchIndex === 'number' ? aliasMatchIndex : void 0 }
                        }).then(function (encoding) {
                            if (encoding) {
                                activeEditor = _this.editorService.getActiveEditor();
                                encodingSupport = asFileOrUntitledEditorInput(activeEditor.input);
                                if (encodingSupport && types.areFunctions(encodingSupport.setEncoding, encodingSupport.getEncoding) && encodingSupport.getEncoding() !== encoding.id) {
                                    encodingSupport.setEncoding(encoding.id, isReopenWithEncoding ? editor_1.EncodingMode.Decode : editor_1.EncodingMode.Encode); // Set new encoding
                                }
                            }
                        });
                    });
                });
            });
        };
        ChangeEncodingAction.ID = 'workbench.action.editor.changeEncoding';
        ChangeEncodingAction.LABEL = nls.localize(34, null);
        ChangeEncodingAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, quickOpenService_1.IQuickOpenService),
            __param(4, configuration_1.IConfigurationService)
        ], ChangeEncodingAction);
        return ChangeEncodingAction;
    }(actions_1.Action));
    exports.ChangeEncodingAction = ChangeEncodingAction;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/editor/stringEditor", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/browser/parts/editor/stringEditor', 'vs/base/common/types', 'vs/workbench/common/editor/textEditorModel', 'vs/workbench/common/editor/untitledEditorInput', 'vs/workbench/browser/parts/editor/textEditor', 'vs/workbench/common/events', 'vs/platform/telemetry/common/telemetry', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/storage/common/storage', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/workbench/services/editor/common/editorService', 'vs/editor/common/services/modeService'], function (require, exports, winjs_base_1, nls, types, textEditorModel_1, untitledEditorInput_1, textEditor_1, events_1, telemetry_1, contextService_1, storage_1, configuration_1, event_1, instantiation_1, message_1, editorService_1, modeService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * An editor implementation that is capable of showing string inputs or promise inputs that resolve to a string.
     * Uses the Monaco TextEditor widget to show the contents.
     */
    var StringEditor = (function (_super) {
        __extends(StringEditor, _super);
        function StringEditor(telemetryService, instantiationService, contextService, storageService, messageService, configurationService, eventService, editorService, modeService) {
            var _this = this;
            _super.call(this, StringEditor.ID, telemetryService, instantiationService, contextService, storageService, messageService, configurationService, eventService, editorService, modeService);
            this.mapResourceToEditorViewState = Object.create(null);
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.UNTITLED_FILE_DELETED, function (e) { return _this.onUntitledDeletedEvent(e); }));
        }
        StringEditor.prototype.onUntitledDeletedEvent = function (e) {
            delete this.mapResourceToEditorViewState[e.resource.toString()];
        };
        StringEditor.prototype.getTitle = function () {
            if (this.input) {
                return this.input.getName();
            }
            return nls.localize(0, null);
        };
        StringEditor.prototype.setInput = function (input, options) {
            var _this = this;
            var oldInput = this.getInput();
            _super.prototype.setInput.call(this, input, options);
            // Detect options
            var forceOpen = options && options.forceOpen;
            // Same Input
            if (!forceOpen && input.matches(oldInput)) {
                // TextOptions (avoiding instanceof here for a reason, do not change!)
                var textOptions = options;
                if (textOptions && types.isFunction(textOptions.apply)) {
                    textOptions.apply(this.getControl());
                }
                return winjs_base_1.TPromise.as(null);
            }
            // Remember view settings if input changes
            if (oldInput instanceof untitledEditorInput_1.UntitledEditorInput) {
                this.mapResourceToEditorViewState[oldInput.getResource().toString()] = this.getControl().saveViewState();
            }
            // Different Input (Reload)
            return this.editorService.resolveEditorModel(input, true /* Reload */).then(function (resolvedModel) {
                // Assert Model instance
                if (!(resolvedModel instanceof textEditorModel_1.BaseTextEditorModel)) {
                    return winjs_base_1.TPromise.wrapError('Invalid editor input. String editor requires a model instance of BaseTextEditorModel.');
                }
                // Assert that the current input is still the one we expect. This prevents a race condition when loading takes long and another input was set meanwhile
                if (!_this.getInput() || _this.getInput() !== input) {
                    return null;
                }
                // Set Editor Model
                var textEditor = _this.getControl();
                var textEditorModel = resolvedModel.textEditorModel;
                textEditor.setModel(textEditorModel);
                // Apply Options from TextOptions
                var optionsGotApplied = false;
                var textOptions = options;
                if (textOptions && types.isFunction(textOptions.apply)) {
                    optionsGotApplied = textOptions.apply(textEditor);
                }
                // Otherwise restore View State
                if (!optionsGotApplied && input instanceof untitledEditorInput_1.UntitledEditorInput) {
                    var viewState = _this.mapResourceToEditorViewState[input.getResource().toString()];
                    if (viewState) {
                        textEditor.restoreViewState(viewState);
                    }
                }
                // Apply options again because input has changed
                textEditor.updateOptions(_this.getCodeEditorOptions());
            });
        };
        StringEditor.prototype.getCodeEditorOptions = function () {
            var options = _super.prototype.getCodeEditorOptions.call(this);
            var input = this.getInput();
            var isUntitled = input instanceof untitledEditorInput_1.UntitledEditorInput;
            var isReadonly = !isUntitled; // all string editors are readonly except for the untitled one
            options.readOnly = isReadonly;
            var ariaLabel;
            var inputName = input && input.getName();
            if (isReadonly) {
                ariaLabel = inputName ? nls.localize(1, null, inputName) : nls.localize(2, null);
            }
            else {
                ariaLabel = inputName ? nls.localize(3, null, inputName) : nls.localize(4, null);
            }
            options.ariaLabel = ariaLabel;
            return options;
        };
        /**
         * Reveals the last line of this editor if it has a model set.
         */
        StringEditor.prototype.revealLastLine = function () {
            var codeEditor = this.getControl();
            var model = codeEditor.getModel();
            if (model) {
                var lastLine = model.getLineCount();
                codeEditor.revealLine(lastLine);
            }
        };
        StringEditor.prototype.supportsSplitEditor = function () {
            return true;
        };
        StringEditor.prototype.clearInput = function () {
            // Keep editor view state in settings to restore when coming back
            if (this.input instanceof untitledEditorInput_1.UntitledEditorInput) {
                this.mapResourceToEditorViewState[this.input.getResource().toString()] = this.getControl().saveViewState();
            }
            // Clear Model
            this.getControl().setModel(null);
            _super.prototype.clearInput.call(this);
        };
        StringEditor.ID = 'workbench.editors.stringEditor';
        StringEditor = __decorate([
            __param(0, telemetry_1.ITelemetryService),
            __param(1, instantiation_1.IInstantiationService),
            __param(2, contextService_1.IWorkspaceContextService),
            __param(3, storage_1.IStorageService),
            __param(4, message_1.IMessageService),
            __param(5, configuration_1.IConfigurationService),
            __param(6, event_1.IEventService),
            __param(7, editorService_1.IWorkbenchEditorService),
            __param(8, modeService_1.IModeService)
        ], StringEditor);
        return StringEditor;
    }(textEditor_1.BaseTextEditor));
    exports.StringEditor = StringEditor;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/editor/textDiffEditor", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/browser/parts/editor/textDiffEditor', 'vs/base/common/actions', 'vs/base/common/errors', 'vs/base/common/types', 'vs/workbench/browser/parts/editor/textEditor', 'vs/workbench/common/editor', 'vs/workbench/common/editor/stringEditorInput', 'vs/workbench/common/editor/resourceEditorInput', 'vs/workbench/common/editor/diffEditorInput', 'vs/editor/contrib/diffNavigator/common/diffNavigator', 'vs/editor/browser/widget/diffEditorWidget', 'vs/workbench/common/editor/textDiffEditorModel', 'vs/workbench/services/editor/browser/editorService', 'vs/platform/files/common/files', 'vs/platform/telemetry/common/telemetry', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/storage/common/storage', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/workbench/services/editor/common/editorService', 'vs/editor/common/services/modeService', 'vs/platform/keybinding/common/keybindingService', 'vs/css!./media/textdiffeditor'], function (require, exports, winjs_base_1, nls, actions_1, errors_1, types, textEditor_1, editor_1, stringEditorInput_1, resourceEditorInput_1, diffEditorInput_1, diffNavigator_1, diffEditorWidget_1, textDiffEditorModel_1, editorService_1, files_1, telemetry_1, contextService_1, storage_1, configuration_1, event_1, instantiation_1, message_1, editorService_2, modeService_1, keybindingService_1) {
    'use strict';
    /**
     * The text editor that leverages the monaco diff text editor for the editing experience.
     */
    var TextDiffEditor = (function (_super) {
        __extends(TextDiffEditor, _super);
        function TextDiffEditor(telemetryService, instantiationService, contextService, storageService, messageService, configurationService, eventService, editorService, modeService, keybindingService) {
            _super.call(this, TextDiffEditor.ID, telemetryService, instantiationService, contextService, storageService, messageService, configurationService, eventService, editorService, modeService);
            this.textDiffEditorVisible = keybindingService.createKey('textCompareEditorVisible', false);
        }
        TextDiffEditor.prototype.getTitle = function () {
            if (this.input) {
                return this.input.getName();
            }
            return nls.localize(0, null);
        };
        TextDiffEditor.prototype.createEditorControl = function (parent) {
            var _this = this;
            this.nextDiffAction = new NavigateAction(this, true);
            this.previousDiffAction = new NavigateAction(this, false);
            var delegatingService = this.instantiationService.createInstance(editorService_1.DelegatingWorkbenchEditorService, this, function (editor, input, options, arg4) {
                // Check if arg4 is a position argument that differs from this editors position
                if (types.isUndefinedOrNull(arg4) || arg4 === false || arg4 === _this.position) {
                    var activeDiffInput = _this.getInput();
                    if (input && options && activeDiffInput) {
                        // Input matches modified side of the diff editor: perform the action on modified side
                        if (input.matches(activeDiffInput.getModifiedInput())) {
                            return _this.setInput(_this.getInput(), options).then(function () {
                                return true;
                            });
                        }
                        else if (input.matches(activeDiffInput.getOriginalInput())) {
                            var originalEditor = _this.getControl().getOriginalEditor();
                            if (options instanceof editor_1.TextEditorOptions) {
                                options.apply(originalEditor);
                                return winjs_base_1.TPromise.as(true);
                            }
                        }
                    }
                }
                return winjs_base_1.TPromise.as(false);
            });
            // Create a special child of instantiator that will delegate all calls to openEditor() to the same diff editor if the input matches with the modified one
            var diffEditorInstantiator = this.instantiationService.createChild({
                editorService: delegatingService
            });
            return diffEditorInstantiator.createInstance(diffEditorWidget_1.DiffEditorWidget, parent.getHTMLElement(), this.getCodeEditorOptions());
        };
        TextDiffEditor.prototype.setInput = function (input, options) {
            var _this = this;
            var oldInput = this.getInput();
            _super.prototype.setInput.call(this, input, options);
            // Detect options
            var forceOpen = options && options.forceOpen;
            // Same Input
            if (!forceOpen && input.matches(oldInput)) {
                // TextOptions (avoiding instanceof here for a reason, do not change!)
                var textOptions = options;
                if (textOptions && types.isFunction(textOptions.apply)) {
                    textOptions.apply(this.getControl());
                }
                return winjs_base_1.TPromise.as(null);
            }
            // Dispose previous diff navigator
            if (this.diffNavigator) {
                this.diffNavigator.dispose();
            }
            // Different Input (Reload)
            return this.editorService.resolveEditorModel(input, true /* Reload */).then(function (resolvedModel) {
                // Assert Model Instance
                if (!(resolvedModel instanceof textDiffEditorModel_1.TextDiffEditorModel) && _this.openAsBinary(input, options)) {
                    return null;
                }
                // Assert that the current input is still the one we expect. This prevents a race condition when loading a diff takes long and another input was set meanwhile
                if (!_this.getInput() || _this.getInput() !== input) {
                    return null;
                }
                // Editor
                var diffEditor = _this.getControl();
                diffEditor.setModel(resolvedModel.textDiffEditorModel);
                // Respect text diff editor options
                var autoRevealFirstChange = true;
                if (options instanceof editor_1.TextDiffEditorOptions) {
                    var textDiffOptions = options;
                    autoRevealFirstChange = !types.isUndefinedOrNull(textDiffOptions.autoRevealFirstChange) ? textDiffOptions.autoRevealFirstChange : autoRevealFirstChange;
                }
                // listen on diff updated changes to reveal the first change
                _this.diffNavigator = new diffNavigator_1.DiffNavigator(diffEditor, {
                    alwaysRevealFirst: autoRevealFirstChange
                });
                _this.diffNavigator.addListener(diffNavigator_1.DiffNavigator.Events.UPDATED, function () {
                    _this.nextDiffAction.updateEnablement();
                    _this.previousDiffAction.updateEnablement();
                });
                // Handle TextOptions
                if (options && types.isFunction(options.apply)) {
                    options.apply(diffEditor);
                }
                // Apply options again because input has changed
                diffEditor.updateOptions(_this.getCodeEditorOptions());
            }, function (error) {
                // In case we tried to open a file and the response indicates that this is not a text file, fallback to binary diff.
                if (_this.isFileBinaryError(error) && _this.openAsBinary(input, options)) {
                    return null;
                }
                // Otherwise make sure the error bubbles up
                return winjs_base_1.TPromise.wrapError(error);
            });
        };
        TextDiffEditor.prototype.openAsBinary = function (input, options) {
            if (input instanceof diffEditorInput_1.DiffEditorInput) {
                var originalInput = input.originalInput;
                var modifiedInput = input.modifiedInput;
                var binaryDiffInput = new diffEditorInput_1.DiffEditorInput(input.getName(), input.getDescription(), originalInput, modifiedInput, true);
                this.editorService.openEditor(binaryDiffInput, options, this.position).done(null, errors_1.onUnexpectedError);
                return true;
            }
            return false;
        };
        TextDiffEditor.prototype.getCodeEditorOptions = function () {
            var options = _super.prototype.getCodeEditorOptions.call(this);
            var input = this.input;
            if (input && types.isFunction(input.getModifiedInput)) {
                var modifiedInput = input.getModifiedInput();
                var readOnly = modifiedInput instanceof stringEditorInput_1.StringEditorInput || modifiedInput instanceof resourceEditorInput_1.ResourceEditorInput;
                options.readOnly = readOnly;
                var ariaLabel = void 0;
                var inputName = input && input.getName();
                if (readOnly) {
                    ariaLabel = inputName ? nls.localize(1, null, inputName) : nls.localize(2, null);
                }
                else {
                    ariaLabel = inputName ? nls.localize(3, null, inputName) : nls.localize(4, null);
                }
                options.ariaLabel = ariaLabel;
            }
            return options;
        };
        TextDiffEditor.prototype.isFileBinaryError = function (error) {
            var _this = this;
            if (types.isArray(error)) {
                var errors = error;
                return errors.some(function (e) { return _this.isFileBinaryError(e); });
            }
            return error.fileOperationResult === files_1.FileOperationResult.FILE_IS_BINARY;
        };
        TextDiffEditor.prototype.clearInput = function () {
            // Dispose previous diff navigator
            if (this.diffNavigator) {
                this.diffNavigator.dispose();
            }
            // Clear Model
            this.getControl().setModel(null);
            // Pass to super
            _super.prototype.clearInput.call(this);
        };
        TextDiffEditor.prototype.setVisible = function (visible, position) {
            this.textDiffEditorVisible.set(visible);
            return _super.prototype.setVisible.call(this, visible, position);
        };
        TextDiffEditor.prototype.getDiffNavigator = function () {
            return this.diffNavigator;
        };
        TextDiffEditor.prototype.getActions = function () {
            return [
                this.previousDiffAction,
                this.nextDiffAction
            ];
        };
        TextDiffEditor.prototype.getSecondaryActions = function () {
            var _this = this;
            var actions = _super.prototype.getSecondaryActions.call(this);
            var control = this.getControl();
            var inlineModeActive = control && !control.renderSideBySide;
            var inlineLabel = nls.localize(5, null);
            var sideBySideLabel = nls.localize(6, null);
            // Action to toggle editor mode from inline to side by side
            var toggleEditorModeAction = new actions_1.Action('toggle.diff.editorMode', inlineModeActive ? sideBySideLabel : inlineLabel, null, true, function () {
                _this.getControl().updateOptions({
                    renderSideBySide: inlineModeActive
                });
                inlineModeActive = !inlineModeActive;
                toggleEditorModeAction.label = inlineModeActive ? sideBySideLabel : inlineLabel;
                return winjs_base_1.TPromise.as(true);
            });
            toggleEditorModeAction.order = 50; // Closer to the end
            actions.push.apply(actions, [
                toggleEditorModeAction
            ]);
            return actions;
        };
        TextDiffEditor.prototype.getControl = function () {
            return _super.prototype.getControl.call(this);
        };
        TextDiffEditor.prototype.dispose = function () {
            // Dispose previous diff navigator
            if (this.diffNavigator) {
                this.diffNavigator.dispose();
            }
            _super.prototype.dispose.call(this);
        };
        TextDiffEditor.ID = 'workbench.editors.textDiffEditor';
        TextDiffEditor = __decorate([
            __param(0, telemetry_1.ITelemetryService),
            __param(1, instantiation_1.IInstantiationService),
            __param(2, contextService_1.IWorkspaceContextService),
            __param(3, storage_1.IStorageService),
            __param(4, message_1.IMessageService),
            __param(5, configuration_1.IConfigurationService),
            __param(6, event_1.IEventService),
            __param(7, editorService_2.IWorkbenchEditorService),
            __param(8, modeService_1.IModeService),
            __param(9, keybindingService_1.IKeybindingService)
        ], TextDiffEditor);
        return TextDiffEditor;
    }(textEditor_1.BaseTextEditor));
    exports.TextDiffEditor = TextDiffEditor;
    var NavigateAction = (function (_super) {
        __extends(NavigateAction, _super);
        function NavigateAction(editor, next) {
            _super.call(this, next ? NavigateAction.ID_NEXT : NavigateAction.ID_PREV);
            this.editor = editor;
            this.next = next;
            this.label = this.next ? nls.localize(7, null) : nls.localize(8, null);
            this.class = this.next ? 'textdiff-editor-action next' : 'textdiff-editor-action previous';
            this.enabled = false;
        }
        NavigateAction.prototype.run = function () {
            if (this.next) {
                this.editor.getDiffNavigator().next();
            }
            else {
                this.editor.getDiffNavigator().previous();
            }
            return null;
        };
        NavigateAction.prototype.updateEnablement = function () {
            this.enabled = this.editor.getDiffNavigator().canNavigate();
        };
        NavigateAction.ID_NEXT = 'workbench.action.compareEditor.nextChange';
        NavigateAction.ID_PREV = 'workbench.action.compareEditor.previousChange';
        return NavigateAction;
    }(actions_1.Action));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/browser/parts/editor/editor.contribution", ["require", "exports", 'vs/platform/platform', 'vs/nls!vs/workbench/browser/parts/editor/editor.contribution', 'vs/base/common/winjs.base', 'vs/platform/workspace/common/workspace', 'vs/workbench/browser/parts/statusbar/statusbar', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor/stringEditorInput', 'vs/workbench/browser/parts/editor/stringEditor', 'vs/workbench/common/editor/diffEditorInput', 'vs/workbench/common/editor/untitledEditorInput', 'vs/workbench/common/editor/resourceEditorInput', 'vs/platform/instantiation/common/instantiation', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/platform/keybinding/common/keybindingService', 'vs/workbench/browser/parts/editor/textDiffEditor', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/browser/parts/editor/binaryDiffEditor', 'vs/workbench/browser/parts/editor/iframeEditor', 'vs/workbench/common/editor/iframeEditorInput', 'vs/workbench/browser/parts/editor/editorStatus', 'vs/workbench/common/actionRegistry', 'vs/workbench/browser/actionBarRegistry', 'vs/platform/actions/common/actions', 'vs/platform/instantiation/common/descriptors', 'vs/base/common/keyCodes'], function (require, exports, platform_1, nls, winjs_base_1, workspace_1, statusbar_1, baseEditor_1, stringEditorInput_1, stringEditor_1, diffEditorInput_1, untitledEditorInput_1, resourceEditorInput_1, instantiation_1, keybindingsRegistry_1, keybindingService_1, textDiffEditor_1, editorService_1, binaryDiffEditor_1, iframeEditor_1, iframeEditorInput_1, editorStatus_1, actionRegistry_1, actionBarRegistry_1, actions_1, descriptors_1, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // Register String Editor
    platform_1.Registry.as(baseEditor_1.Extensions.Editors).registerEditor(new baseEditor_1.EditorDescriptor(stringEditor_1.StringEditor.ID, nls.localize(0, null), 'vs/workbench/browser/parts/editor/stringEditor', 'StringEditor'), [
        new descriptors_1.SyncDescriptor(stringEditorInput_1.StringEditorInput),
        new descriptors_1.SyncDescriptor(untitledEditorInput_1.UntitledEditorInput),
        new descriptors_1.SyncDescriptor(resourceEditorInput_1.ResourceEditorInput)
    ]);
    // Register Text Diff Editor
    platform_1.Registry.as(baseEditor_1.Extensions.Editors).registerEditor(new baseEditor_1.EditorDescriptor(textDiffEditor_1.TextDiffEditor.ID, nls.localize(1, null), 'vs/workbench/browser/parts/editor/textDiffEditor', 'TextDiffEditor'), [
        new descriptors_1.SyncDescriptor(diffEditorInput_1.DiffEditorInput)
    ]);
    // Register Binary Resource Diff Editor
    platform_1.Registry.as(baseEditor_1.Extensions.Editors).registerEditor(new baseEditor_1.EditorDescriptor(binaryDiffEditor_1.BinaryResourceDiffEditor.ID, nls.localize(2, null), 'vs/workbench/browser/parts/editor/binaryDiffEditor', 'BinaryResourceDiffEditor'), [
        new descriptors_1.SyncDescriptor(diffEditorInput_1.DiffEditorInput)
    ]);
    // Register IFrame Editor
    platform_1.Registry.as(baseEditor_1.Extensions.Editors).registerEditor(new baseEditor_1.EditorDescriptor(iframeEditor_1.IFrameEditor.ID, nls.localize(3, null), 'vs/workbench/browser/parts/editor/iframeEditor', 'IFrameEditor'), [
        new descriptors_1.SyncDescriptor(iframeEditorInput_1.IFrameEditorInput)
    ]);
    // Register Editor Status
    var statusBar = platform_1.Registry.as(statusbar_1.Extensions.Statusbar);
    statusBar.registerStatusbarItem(new statusbar_1.StatusbarItemDescriptor(editorStatus_1.EditorStatus, statusbar_1.StatusbarAlignment.RIGHT, 100 /* High Priority */));
    // Register Actions
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(editorStatus_1.ChangeModeAction, editorStatus_1.ChangeModeAction.ID, editorStatus_1.ChangeModeAction.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.KEY_M) }));
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(editorStatus_1.ChangeEOLAction, editorStatus_1.ChangeEOLAction.ID, editorStatus_1.ChangeEOLAction.LABEL));
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(editorStatus_1.ChangeEncodingAction, editorStatus_1.ChangeEncodingAction.ID, editorStatus_1.ChangeEncodingAction.LABEL));
    var ViewSourceEditorInputAction = (function (_super) {
        __extends(ViewSourceEditorInputAction, _super);
        function ViewSourceEditorInputAction(editorService, contextService) {
            _super.call(this, 'workbench.files.action.viewSourceFromEditor', nls.localize(4, null), 'iframe-editor-action view-source');
            this.editorService = editorService;
            this.contextService = contextService;
        }
        ViewSourceEditorInputAction.prototype.run = function (event) {
            var iFrameEditorInput = this.input;
            var sideBySide = !!(event && (event.ctrlKey || event.metaKey));
            return this.editorService.openEditor({
                resource: iFrameEditorInput.getResource()
            }, sideBySide);
        };
        ViewSourceEditorInputAction = __decorate([
            __param(0, editorService_1.IWorkbenchEditorService),
            __param(1, workspace_1.IWorkspaceContextService)
        ], ViewSourceEditorInputAction);
        return ViewSourceEditorInputAction;
    }(baseEditor_1.EditorInputAction));
    exports.ViewSourceEditorInputAction = ViewSourceEditorInputAction;
    var RefreshIFrameEditorInputAction = (function (_super) {
        __extends(RefreshIFrameEditorInputAction, _super);
        function RefreshIFrameEditorInputAction(editorService) {
            _super.call(this, 'workbench.files.action.refreshIFrameEditor', nls.localize(5, null), 'iframe-editor-action refresh');
            this.editorService = editorService;
        }
        RefreshIFrameEditorInputAction.prototype.run = function (event) {
            var editor = this.editorService.getActiveEditor();
            if (editor instanceof iframeEditor_1.IFrameEditor) {
                editor.reload(true);
                editor.focus();
            }
            return winjs_base_1.TPromise.as(null);
        };
        RefreshIFrameEditorInputAction = __decorate([
            __param(0, editorService_1.IWorkbenchEditorService)
        ], RefreshIFrameEditorInputAction);
        return RefreshIFrameEditorInputAction;
    }(baseEditor_1.EditorInputAction));
    exports.RefreshIFrameEditorInputAction = RefreshIFrameEditorInputAction;
    var actionBarRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
    var IFrameEditorActionContributor = (function (_super) {
        __extends(IFrameEditorActionContributor, _super);
        function IFrameEditorActionContributor(instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
        }
        IFrameEditorActionContributor.prototype.hasActionsForEditorInput = function (context) {
            return context.input instanceof iframeEditorInput_1.IFrameEditorInput;
        };
        IFrameEditorActionContributor.prototype.getActionsForEditorInput = function (context) {
            return [
                this.instantiationService.createInstance(RefreshIFrameEditorInputAction),
                this.instantiationService.createInstance(ViewSourceEditorInputAction)
            ];
        };
        IFrameEditorActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], IFrameEditorActionContributor);
        return IFrameEditorActionContributor;
    }(baseEditor_1.EditorInputActionContributor));
    // Contribute to IFrame Editor Inputs
    actionBarRegistry.registerActionBarContributor(actionBarRegistry_1.Scope.EDITOR, IFrameEditorActionContributor);
    // Register keybinding for "Next Change" & "Previous Change" in visible diff editor
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
        id: 'workbench.action.compareEditor.nextChange',
        weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.workbenchContrib(),
        context: keybindingService_1.KbExpr.has('textCompareEditorVisible'),
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.RightArrow),
        handler: function (accessor) { return navigateInDiffEditor(accessor, true); }
    });
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
        id: 'workbench.action.compareEditor.previousChange',
        weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.workbenchContrib(),
        context: keybindingService_1.KbExpr.has('textCompareEditorVisible'),
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.LeftArrow),
        handler: function (accessor) { return navigateInDiffEditor(accessor, false); }
    });
    function navigateInDiffEditor(accessor, next) {
        var editorService = accessor.get(editorService_1.IWorkbenchEditorService);
        var candidates = [editorService.getActiveEditor()].concat(editorService.getVisibleEditors()).filter(function (e) { return e instanceof textDiffEditor_1.TextDiffEditor; });
        if (candidates.length > 0) {
            next ? candidates[0].getDiffNavigator().next() : candidates[0].getDiffNavigator().previous();
        }
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/browser/parts/editor/editorPart", ["require", "exports", 'vs/base/common/winjs.base', 'vs/platform/platform', 'vs/base/common/timer', 'vs/base/common/events', 'vs/base/browser/builder', 'vs/nls!vs/workbench/browser/parts/editor/editorPart', 'vs/base/common/strings', 'vs/base/common/assert', 'vs/base/common/arrays', 'vs/base/common/types', 'vs/base/common/errors', 'vs/workbench/common/memento', 'vs/workbench/browser/actionBarRegistry', 'vs/workbench/browser/part', 'vs/workbench/common/events', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor', 'vs/workbench/browser/parts/editor/textEditor', 'vs/workbench/browser/parts/editor/sideBySideEditorControl', 'vs/workbench/services/progress/browser/progressService', 'vs/platform/editor/common/editor', 'vs/platform/message/common/message', 'vs/css!./media/editorpart', 'vs/workbench/browser/parts/editor/editor.contribution'], function (require, exports, winjs_base_1, platform_1, timer, events_1, builder_1, nls, strings, assert, arrays, types, errors, memento_1, actionBarRegistry_1, part_1, events_2, baseEditor_1, editor_1, textEditor_1, sideBySideEditorControl_1, progressService_1, editor_2, message_1) {
    'use strict';
    var EDITOR_STATE_STORAGE_KEY = 'editorpart.editorState';
    /**
     * The editor part is the container for editors in the workbench. Based on the editor input being opened, it asks the registered
     * editor for the given input to show the contents. The editor part supports up to 3 side-by-side editors.
     */
    var EditorPart = (function (_super) {
        __extends(EditorPart, _super);
        function EditorPart(messageService, eventService, telemetryService, storageService, partService, id) {
            _super.call(this, id);
            this.messageService = messageService;
            this.eventService = eventService;
            this.telemetryService = telemetryService;
            this.storageService = storageService;
            this.partService = partService;
            this.visibleInputs = [];
            this.visibleInputListeners = [];
            this.visibleEditors = [];
            this.editorOpenToken = [];
            for (var i = 0; i < editor_2.POSITIONS.length; i++) {
                this.editorOpenToken[i] = 0;
            }
            this.editorSetInputErrorCounter = [];
            for (var i = 0; i < editor_2.POSITIONS.length; i++) {
                this.editorSetInputErrorCounter[i] = 0;
            }
            this.visibleEditorListeners = this.createPositionArray(true);
            this.instantiatedEditors = this.createPositionArray(true);
            this.mapEditorToEditorContainers = this.createPositionArray(false);
            this.mapActionsToEditors = this.createPositionArray(false);
            this.mapEditorLoadingPromiseToEditor = this.createPositionArray(false);
            this.mapEditorCreationPromiseToEditor = this.createPositionArray(false);
            this.pendingEditorInputsToClose = [];
            this.pendingEditorInputCloseTimeout = null;
        }
        EditorPart.prototype.setInstantiationService = function (service) {
            this.instantiationService = service;
        };
        EditorPart.prototype.createPositionArray = function (multiArray) {
            var array = [];
            for (var i = 0; i < editor_2.POSITIONS.length; i++) {
                array[i] = multiArray ? [] : {};
            }
            return array;
        };
        EditorPart.prototype.getActiveEditorInput = function () {
            var lastActiveEditor = this.sideBySideControl.getActiveEditor();
            return lastActiveEditor ? lastActiveEditor.input : null;
        };
        EditorPart.prototype.getActiveEditor = function () {
            return this.sideBySideControl.getActiveEditor();
        };
        EditorPart.prototype.getVisibleEditors = function () {
            return this.visibleEditors ? this.visibleEditors.filter(function (editor) { return !!editor; }) : [];
        };
        EditorPart.prototype.setEditors = function (inputs, options) {
            var _this = this;
            return this.closeEditors().then(function () {
                return _this.restoreEditorState(inputs, options);
            });
        };
        EditorPart.prototype.openEditor = function (input, options, arg3, widthRatios) {
            var _this = this;
            // Normalize some values
            if (!input) {
                input = null;
            }
            if (!options) {
                options = null;
            }
            // Determine position to open editor in (left, center, right)
            var position = this.findPosition(arg3, widthRatios);
            // In case the position is invalid, return early. This can happen when the user tries to open a side editor
            // when the maximum number of allowed editors is reached and no more side editor can be opened.
            if (position === null) {
                return winjs_base_1.TPromise.as(null);
            }
            // Prevent bad UI issues by ignoring any attempt to open an editor if at the same time an editor is
            // either creating or loading at this position. Not very nice, but helpful and typically should not cause issues.
            if (Object.keys(this.mapEditorLoadingPromiseToEditor[position]).length > 0 || Object.keys(this.mapEditorCreationPromiseToEditor[position]).length > 0) {
                return winjs_base_1.TPromise.as(null);
            }
            // Prevent bad UI issues by ignoring openEditor() calls while the user is dragging an editor
            if (this.sideBySideControl.isDragging()) {
                return winjs_base_1.TPromise.as(null);
            }
            // Emit early open event to allow for veto
            var event = new events_2.EditorEvent(this.visibleEditors[position], this.visibleEditors[position] && this.visibleEditors[position].getId(), input, options, position);
            this.emit(events_2.EventType.EDITOR_INPUT_OPENING, event);
            if (event.isPrevented()) {
                return winjs_base_1.TPromise.as(null);
            }
            // Do ref counting of this method
            this.editorOpenToken[position]++;
            var editorOpenToken = this.editorOpenToken[position];
            // Log side by side use
            if (input && position !== editor_2.Position.LEFT) {
                this.telemetryService.publicLog('workbenchSideEditorOpened', { position: position });
            }
            // Determine options if the editor opens to the side by looking at same input already opened
            if (input && position !== editor_2.Position.LEFT) {
                options = this.findSideOptions(input, options, position);
            }
            // Remember as visible input for this position
            this.visibleInputs[position] = input;
            // Dispose previous input listener if any
            if (this.visibleInputListeners[position]) {
                this.visibleInputListeners[position]();
                this.visibleInputListeners[position] = null;
            }
            // Close editor when input provided and input gets disposed
            if (input) {
                this.visibleInputListeners[position] = input.addListener(events_1.EventType.DISPOSE, function () {
                    // Keep the inputs to close. We use this to support multiple inputs closing
                    // right after each other and this helps avoid layout issues with the delayed
                    // timeout based closing below
                    if (input === _this.visibleInputs[position]) {
                        _this.pendingEditorInputsToClose.push(input);
                        _this.startDelayedCloseEditorsFromInputDispose();
                    }
                });
            }
            // Close any opened editor at position if input is null
            if (input === null) {
                if (this.visibleEditors[position]) {
                    // Reset counter
                    this.editorSetInputErrorCounter[position] = 0;
                    // Emit Input-Changing Event
                    this.emit(events_2.EventType.EDITOR_INPUT_CHANGING, new events_2.EditorEvent(null, null, null, null, position));
                    // Hide Editor
                    return this.hideEditor(this.visibleEditors[position], position, true).then(function () {
                        // Emit Input-Changed Event
                        _this.emit(events_2.EventType.EDITOR_INPUT_CHANGED, new events_2.EditorEvent(null, null, null, null, position));
                        // Focus next editor if there is still an active one left
                        var currentActiveEditor = _this.sideBySideControl.getActiveEditor();
                        if (currentActiveEditor) {
                            return _this.openEditor(currentActiveEditor.input, null, currentActiveEditor.position).then(function () {
                                // Explicitly trigger the focus changed handler because the side by side control will not trigger it unless
                                // the user is actively changing focus with the mouse from left to right.
                                _this.onEditorFocusChanged();
                                return currentActiveEditor;
                            });
                        }
                        return winjs_base_1.TPromise.as(null);
                    });
                }
                return winjs_base_1.TPromise.as(null);
            }
            // Lookup Editor and Assert
            var editorDescriptor = platform_1.Registry.as(baseEditor_1.Extensions.Editors).getEditor(input);
            assert.ok(editorDescriptor, strings.format('Can not find a registered editor for the input {0}', input));
            // Progress Indication
            var loadingPromise = winjs_base_1.TPromise.timeout(this.partService.isCreated() ? 800 : 3200 /* less ugly initial startup */).then(function () {
                if (editorOpenToken === _this.editorOpenToken[position]) {
                    _this.sideBySideControl.getProgressBar(position).infinite().getContainer().show();
                    _this.sideBySideControl.setLoading(position, input);
                }
            });
            // Handle Active Editor showing
            var activeEditorHidePromise;
            if (this.visibleEditors[position]) {
                // Editor can handle Input
                if (editorDescriptor.describes(this.visibleEditors[position])) {
                    // If the editor is currently being created, join this process to avoid issues
                    var pendingEditorCreationPromise = this.mapEditorCreationPromiseToEditor[position][editorDescriptor.getId()];
                    if (!pendingEditorCreationPromise) {
                        pendingEditorCreationPromise = winjs_base_1.TPromise.as(null);
                    }
                    return pendingEditorCreationPromise.then(function () {
                        return _this.setInput(_this.visibleEditors[position], input, options, position, loadingPromise);
                    });
                }
                // Editor can not handle Input (Close this Editor)
                activeEditorHidePromise = this.hideEditor(this.visibleEditors[position], position, false);
            }
            else {
                activeEditorHidePromise = winjs_base_1.TPromise.as(null);
            }
            return activeEditorHidePromise.then(function () {
                var timerEvent = timer.start(timer.Topic.WORKBENCH, strings.format('Creating Editor: {0}', editorDescriptor.getName()));
                // We need the container for this editor now
                var editorContainer = _this.mapEditorToEditorContainers[position][editorDescriptor.getId()];
                var newlyCreatedEditorContainerBuilder;
                if (!editorContainer) {
                    // Build Container off-DOM
                    editorContainer = builder_1.$().div({
                        'class': 'editor-container',
                        id: editorDescriptor.getId()
                    }, function (div) {
                        newlyCreatedEditorContainerBuilder = div;
                    });
                    // Remember editor container
                    _this.mapEditorToEditorContainers[position][editorDescriptor.getId()] = editorContainer;
                }
                // Create or get editor from cache
                var editor = _this.getEditorFromCache(editorDescriptor, position);
                var loadOrGetEditorPromise;
                if (editor === null) {
                    // Check if loading is pending from another openEditor
                    var pendingEditorLoad = _this.mapEditorLoadingPromiseToEditor[position][editorDescriptor.getId()];
                    if (pendingEditorLoad) {
                        loadOrGetEditorPromise = pendingEditorLoad;
                    }
                    else {
                        var loaded_1 = false;
                        loadOrGetEditorPromise = _this.createEditor(editorDescriptor, newlyCreatedEditorContainerBuilder.getHTMLElement(), position).then(function (editor) {
                            loaded_1 = true;
                            _this.instantiatedEditors[position].push(editor);
                            delete _this.mapEditorLoadingPromiseToEditor[position][editorDescriptor.getId()];
                            return editor;
                        }, function (error) {
                            loaded_1 = true;
                            delete _this.mapEditorLoadingPromiseToEditor[position][editorDescriptor.getId()];
                            return winjs_base_1.TPromise.wrapError(error);
                        });
                        if (!loaded_1) {
                            _this.mapEditorLoadingPromiseToEditor[position][editorDescriptor.getId()] = loadOrGetEditorPromise;
                        }
                    }
                }
                else {
                    loadOrGetEditorPromise = winjs_base_1.TPromise.as(editor);
                }
                return loadOrGetEditorPromise.then(function (editor) {
                    // Make sure that the user meanwhile did not open another editor
                    if (editorOpenToken !== _this.editorOpenToken[position]) {
                        timerEvent.stop();
                        // Stop loading promise if any
                        loadingPromise.cancel();
                        return null;
                    }
                    // Remember Editor at position
                    _this.visibleEditors[position] = editor;
                    // Register as Emitter to Workbench Bus
                    _this.visibleEditorListeners[position].push(_this.eventService.addEmitter(_this.visibleEditors[position], _this.visibleEditors[position].getId()));
                    var createEditorPromise;
                    if (newlyCreatedEditorContainerBuilder) {
                        // create editor
                        var created_1 = false;
                        createEditorPromise = editor.create(newlyCreatedEditorContainerBuilder).then(function () {
                            created_1 = true;
                            delete _this.mapEditorCreationPromiseToEditor[position][editorDescriptor.getId()];
                        }, function (error) {
                            created_1 = true;
                            delete _this.mapEditorCreationPromiseToEditor[position][editorDescriptor.getId()];
                            return winjs_base_1.TPromise.wrapError(error);
                        });
                        if (!created_1) {
                            _this.mapEditorCreationPromiseToEditor[position][editorDescriptor.getId()] = createEditorPromise;
                        }
                    }
                    else {
                        // Check if create is pending from another openEditor
                        var pendingEditorCreate = _this.mapEditorCreationPromiseToEditor[position][editorDescriptor.getId()];
                        if (pendingEditorCreate) {
                            createEditorPromise = pendingEditorCreate;
                        }
                        else {
                            createEditorPromise = winjs_base_1.TPromise.as(null);
                        }
                    }
                    // Fill Content and Actions
                    return createEditorPromise.then(function () {
                        // Make sure that the user meanwhile did not open another editor
                        if (!_this.visibleEditors[position] || editor.getId() !== _this.visibleEditors[position].getId()) {
                            timerEvent.stop();
                            // Stop loading promise if any
                            loadingPromise.cancel();
                            return null;
                        }
                        // Show in side by side control
                        _this.sideBySideControl.show(editor, editorContainer, position, options && options.preserveFocus, widthRatios);
                        // Indicate to editor that it is now visible
                        return editor.setVisible(true, position).then(function () {
                            // Make sure the editor is layed out
                            _this.sideBySideControl.layout(position);
                            // Emit Editor-Opened Event
                            _this.emit(events_2.EventType.EDITOR_OPENED, new events_2.EditorEvent(editor, editor.getId(), input, options, position));
                            timerEvent.stop();
                            // Set Input
                            return _this.setInput(editor, input, options, position, loadingPromise);
                        });
                    }, function (e) { return _this.showError(e); });
                });
            });
        };
        EditorPart.prototype.startDelayedCloseEditorsFromInputDispose = function () {
            var _this = this;
            // To prevent race conditions, we call the close in a timeout because it can well be
            // that an input is being disposed with the intent to replace it with some other input
            // right after.
            if (this.pendingEditorInputCloseTimeout === null) {
                this.pendingEditorInputCloseTimeout = setTimeout(function () {
                    _this.closeEditors(false, _this.pendingEditorInputsToClose).done(null, errors.onUnexpectedError);
                    // Reset
                    _this.pendingEditorInputCloseTimeout = null;
                    _this.pendingEditorInputsToClose = [];
                }, 0);
            }
        };
        EditorPart.prototype.closeEditors = function (othersOnly, inputs) {
            var promises = [];
            var editors = this.getVisibleEditors().reverse(); // start from the end to prevent layout to happen through rochade
            for (var i = 0; i < editors.length; i++) {
                var editor = editors[i];
                if (othersOnly && this.getActiveEditor() === editor) {
                    continue;
                }
                if (!inputs || inputs.some(function (inp) { return inp === editor.input; })) {
                    promises.push(this.openEditor(null, null, editor.position));
                }
            }
            return winjs_base_1.TPromise.join(promises).then(function () { return void 0; });
        };
        EditorPart.prototype.findPosition = function (arg1, widthRatios) {
            // With defined width ratios, always trust the provided position
            if (widthRatios && types.isNumber(arg1)) {
                return arg1;
            }
            // No editor open
            var visibleEditors = this.getVisibleEditors();
            var activeEditor = this.getActiveEditor();
            if (visibleEditors.length === 0 || !activeEditor) {
                return editor_2.Position.LEFT; // can only be LEFT
            }
            // Position is unknown: pick last active or LEFT
            if (types.isUndefinedOrNull(arg1) || arg1 === false) {
                var lastActivePosition = this.sideBySideControl.getActivePosition();
                return lastActivePosition || editor_2.Position.LEFT;
            }
            // Position is sideBySide: Find position relative to active editor
            if (arg1 === true) {
                switch (activeEditor.position) {
                    case editor_2.Position.LEFT:
                        return editor_2.Position.CENTER;
                    case editor_2.Position.CENTER:
                        return editor_2.Position.RIGHT;
                    case editor_2.Position.RIGHT:
                        return null; // Cannot open to the side of the right most editor
                }
                return null; // Prevent opening to the side
            }
            // Position is provided, validate it
            if (arg1 === editor_2.Position.RIGHT && visibleEditors.length === 1) {
                return editor_2.Position.CENTER;
            }
            return arg1;
        };
        EditorPart.prototype.findSideOptions = function (input, options, position) {
            // Return early if the input is already showing at the position
            if (this.visibleEditors[position] && input.matches(this.visibleEditors[position].input)) {
                return options;
            }
            // Return early if explicit text options are defined
            if (options instanceof editor_1.TextEditorOptions && options.hasOptionsDefined()) {
                return options;
            }
            // Otherwise try to copy viewstate over from an existing opened editor with same input
            var viewState = null;
            var editors = this.getVisibleEditors();
            for (var i = 0; i < editors.length; i++) {
                var editor = editors[i];
                if (!(editor instanceof textEditor_1.BaseTextEditor)) {
                    continue; // Only works with text editors
                }
                // Found a match
                if (input.matches(editor.input)) {
                    var codeEditor = editor.getControl();
                    viewState = codeEditor.saveViewState();
                    break;
                }
            }
            // Found view state
            if (viewState) {
                var textEditorOptions = null;
                // Merge into existing text editor options if given
                if (options instanceof editor_1.TextEditorOptions) {
                    textEditorOptions = options;
                    textEditorOptions.viewState(viewState);
                    return textEditorOptions;
                }
                // Otherwise create new
                textEditorOptions = new editor_1.TextEditorOptions();
                textEditorOptions.viewState(viewState);
                if (options) {
                    textEditorOptions.forceOpen = options.forceOpen;
                    textEditorOptions.preserveFocus = options.preserveFocus;
                }
                return textEditorOptions;
            }
            return options;
        };
        EditorPart.prototype.rochade = function (arg1, arg2) {
            if (types.isUndefinedOrNull(arg2)) {
                var rochade = arg1;
                switch (rochade) {
                    case sideBySideEditorControl_1.Rochade.CENTER_TO_LEFT:
                        this.rochade(editor_2.Position.CENTER, editor_2.Position.LEFT);
                        break;
                    case sideBySideEditorControl_1.Rochade.RIGHT_TO_CENTER:
                        this.rochade(editor_2.Position.RIGHT, editor_2.Position.CENTER);
                        break;
                    case sideBySideEditorControl_1.Rochade.CENTER_AND_RIGHT_TO_LEFT:
                        this.rochade(editor_2.Position.CENTER, editor_2.Position.LEFT);
                        this.rochade(editor_2.Position.RIGHT, editor_2.Position.CENTER);
                        break;
                }
            }
            else {
                var from = arg1;
                var to = arg2;
                this.doRochade(this.visibleInputs, from, to, null);
                this.doRochade(this.visibleInputListeners, from, to, null);
                this.doRochade(this.visibleEditors, from, to, null);
                this.doRochade(this.editorOpenToken, from, to, null);
                this.doRochade(this.mapEditorLoadingPromiseToEditor, from, to, {});
                this.doRochade(this.mapEditorCreationPromiseToEditor, from, to, {});
                this.doRochade(this.visibleEditorListeners, from, to, []);
                this.doRochade(this.instantiatedEditors, from, to, []);
                this.doRochade(this.mapEditorToEditorContainers, from, to, {});
                this.doRochade(this.mapActionsToEditors, from, to, {});
            }
        };
        EditorPart.prototype.doRochade = function (array, from, to, empty) {
            array[to] = array[from];
            array[from] = empty;
        };
        EditorPart.prototype.moveEditor = function (from, to) {
            if (!this.visibleEditors[from] || !this.visibleEditors[to] || from === to) {
                return; // Ignore if we cannot move
            }
            // Move widgets
            this.sideBySideControl.move(from, to);
            // Move data structures
            arrays.move(this.visibleInputs, from, to);
            arrays.move(this.visibleInputListeners, from, to);
            arrays.move(this.visibleEditors, from, to);
            arrays.move(this.visibleEditorListeners, from, to);
            arrays.move(this.editorOpenToken, from, to);
            arrays.move(this.mapEditorLoadingPromiseToEditor, from, to);
            arrays.move(this.mapEditorCreationPromiseToEditor, from, to);
            arrays.move(this.instantiatedEditors, from, to);
            arrays.move(this.mapEditorToEditorContainers, from, to);
            arrays.move(this.mapActionsToEditors, from, to);
            // Update all title areas
            this.updateEditorTitleArea();
            // Restore focus
            var activeEditor = this.sideBySideControl.getActiveEditor();
            this.openEditor(activeEditor.input, null, activeEditor.position).done(null, errors.onUnexpectedError);
        };
        EditorPart.prototype.arrangeEditors = function (arrangement) {
            this.sideBySideControl.arrangeEditors(arrangement);
        };
        EditorPart.prototype.setInput = function (editor, input, options, position, loadingPromise) {
            var _this = this;
            // Emit Input-/Options-Changed Event as appropiate
            var oldInput = editor.getInput();
            var oldOptions = editor.getOptions();
            var inputChanged = (!oldInput || !oldInput.matches(input) || (options && options.forceOpen));
            if (inputChanged) {
                this.emit(events_2.EventType.EDITOR_INPUT_CHANGING, new events_2.EditorEvent(editor, editor.getId(), input, options, position));
            }
            else if (!oldOptions || !oldOptions.matches(options)) {
                this.emit(events_2.EventType.EDITOR_OPTIONS_CHANGING, new events_2.EditorEvent(editor, editor.getId(), input, options, position));
            }
            // Call into Editor
            var timerEvent = timer.start(timer.Topic.WORKBENCH, strings.format('Set Editor Input: {0}', input.getName()));
            return editor.setInput(input, options).then(function () {
                // Reset counter
                _this.editorSetInputErrorCounter[position] = 0;
                // Stop loading promise if any
                loadingPromise.cancel();
                // Make sure that the user meanwhile has not opened another input
                if (_this.visibleInputs[position] !== input) {
                    timerEvent.stop();
                    // It can happen that the same editor input is being opened rapidly one after the other
                    // (e.g. fast double click on a file). In this case the first open will stop here because
                    // we detect that a second open happens. However, since the input is the same, inputChanged
                    // is false and we are not doing some things that we typically do when opening a file because
                    // we think, the input has not changed.
                    // The fix is to detect if the active input matches with this one that gets canceled and only
                    // in that case notify others about the input change event as well as to make sure that the
                    // editor title area is up to date.
                    if (_this.visibleInputs[position] && _this.visibleInputs[position].matches(input)) {
                        _this.updateEditorTitleArea();
                        _this.emit(events_2.EventType.EDITOR_INPUT_CHANGED, new events_2.EditorEvent(editor, editor.getId(), _this.visibleInputs[position], options, position));
                    }
                    return editor;
                }
                // Focus (unless prevented)
                if (!options || !options.preserveFocus) {
                    editor.focus();
                }
                else if (options && options.forceActive) {
                    _this.sideBySideControl.setActive(editor);
                }
                // Progress Done
                _this.sideBySideControl.getProgressBar(position).done().getContainer().hide();
                // Update Title Area if input changed
                if (inputChanged) {
                    _this.updateEditorTitleArea();
                }
                // Emit Input-Changed Event (if input changed)
                if (inputChanged) {
                    _this.emit(events_2.EventType.EDITOR_INPUT_CHANGED, new events_2.EditorEvent(editor, editor.getId(), input, options, position));
                }
                timerEvent.stop();
                // Fullfill promise with Editor that is being used
                return editor;
            }, function (e) {
                // Keep counter
                _this.editorSetInputErrorCounter[position]++;
                // Stop loading promise if any
                loadingPromise.cancel();
                // Report error
                _this.onSetInputError(e, editor, input, options, position);
                // Recover from this error by closing the editor if the attempt of setInput failed and we are not having any previous input
                if (!oldInput && _this.visibleInputs[position] === input && input) {
                    _this.openEditor(null, null, position).done(null, errors.onUnexpectedError);
                }
                else if (_this.editorSetInputErrorCounter[position] > 1) {
                    _this.openEditor(null, null, position).done(null, errors.onUnexpectedError);
                }
                else if (oldInput) {
                    _this.openEditor(oldInput, null, position).done(null, errors.onUnexpectedError);
                }
            });
        };
        EditorPart.prototype.getEditorFromCache = function (editorDescriptor, position) {
            // Check for existing instantiated editor
            for (var i = 0; i < this.instantiatedEditors[position].length; i++) {
                if (editorDescriptor.describes(this.instantiatedEditors[position][i])) {
                    return this.instantiatedEditors[position][i];
                }
            }
            return null;
        };
        EditorPart.prototype.createEditor = function (editorDescriptor, editorDomNode, position) {
            var services = {
                progressService: new progressService_1.WorkbenchProgressService(this.eventService, this.sideBySideControl.getProgressBar(position), editorDescriptor.getId(), true)
            };
            var editorInstantiationService = this.instantiationService.createChild(services);
            return editorInstantiationService.createInstance(editorDescriptor);
        };
        EditorPart.prototype.hideEditor = function (editor, position, layoutAndRochade) {
            var _this = this;
            var editorContainer = this.mapEditorToEditorContainers[position][editor.getId()];
            // Hide in side by side control
            var rochade = this.sideBySideControl.hide(editor, editorContainer, position, layoutAndRochade);
            // Clear any running Progress
            this.sideBySideControl.getProgressBar(position).stop().getContainer().hide();
            // Clear Listeners
            while (this.visibleEditorListeners[position].length) {
                this.visibleEditorListeners[position].pop()();
            }
            // Indicate to Editor
            editor.clearInput();
            return editor.setVisible(false).then(function () {
                // Clear active editor
                _this.visibleEditors[position] = null;
                // Rochade as needed
                _this.rochade(rochade);
                // Clear Title Area for Position
                _this.sideBySideControl.clearTitle(position);
                // Emit Editor Closed Event
                _this.emit(events_2.EventType.EDITOR_CLOSED, new events_2.EditorEvent(editor, editor.getId(), null, null, position));
                return editor;
            });
        };
        EditorPart.prototype.updateEditorTitleArea = function () {
            var activePosition = this.sideBySideControl.getActivePosition();
            // Update each position individually
            for (var i = 0; i < editor_2.POSITIONS.length; i++) {
                var editor = this.visibleEditors[i];
                var input = editor ? editor.input : null;
                if (input && editor) {
                    this.doUpdateEditorTitleArea(editor, input, i, activePosition === i);
                }
            }
        };
        EditorPart.prototype.doUpdateEditorTitleArea = function (editor, input, position, isActive) {
            var primaryActions = [];
            var secondaryActions = [];
            // Handle toolbar only if side is active
            if (isActive) {
                // Handle Editor Actions
                var editorActions = this.mapActionsToEditors[position][editor.getId()];
                if (!editorActions) {
                    editorActions = this.getEditorActionsForContext(editor, editor, position);
                    this.mapActionsToEditors[position][editor.getId()] = editorActions;
                }
                primaryActions.push.apply(primaryActions, editorActions.primary);
                secondaryActions.push.apply(secondaryActions, editorActions.secondary);
                // Handle Editor Input Actions
                var editorInputActions = this.getEditorActionsForContext({ input: input, editor: editor, position: position }, editor, position);
                primaryActions.push.apply(primaryActions, editorInputActions.primary);
                secondaryActions.push.apply(secondaryActions, editorInputActions.secondary);
            }
            // Apply to title in side by side control
            this.sideBySideControl.setTitle(position, input, actionBarRegistry_1.prepareActions(primaryActions), actionBarRegistry_1.prepareActions(secondaryActions), isActive);
        };
        EditorPart.prototype.getEditorActionsForContext = function (context, editor, position) {
            var primaryActions = [];
            var secondaryActions = [];
            // From Editor
            if (context instanceof baseEditor_1.BaseEditor) {
                primaryActions.push.apply(primaryActions, context.getActions());
                secondaryActions.push.apply(secondaryActions, context.getSecondaryActions());
            }
            // From Contributions
            var actionBarRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
            primaryActions.push.apply(primaryActions, actionBarRegistry.getActionBarActionsForContext(actionBarRegistry_1.Scope.EDITOR, context));
            secondaryActions.push.apply(secondaryActions, actionBarRegistry.getSecondaryActionBarActionsForContext(actionBarRegistry_1.Scope.EDITOR, context));
            return {
                primary: primaryActions,
                secondary: secondaryActions
            };
        };
        EditorPart.prototype.createContentArea = function (parent) {
            var _this = this;
            // Content Container
            var contentArea = builder_1.$(parent)
                .div()
                .addClass('content');
            // Side by Side Control
            this.sideBySideControl = this.instantiationService.createInstance(sideBySideEditorControl_1.SideBySideEditorControl, contentArea);
            this.toUnbind.push(this.sideBySideControl.addListener(sideBySideEditorControl_1.EventType.EDITOR_FOCUS_CHANGED, function () { _this.onEditorFocusChanged(); }));
            // get settings
            this.memento = this.getMemento(this.storageService, memento_1.Scope.WORKSPACE);
            return contentArea;
        };
        EditorPart.prototype.restoreEditorState = function (inputsToOpen, options) {
            var _this = this;
            var activeInput;
            var inputsToRestore;
            var widthRatios;
            // Inputs are given, so just use them
            if (inputsToOpen && inputsToOpen.length) {
                if (inputsToOpen.length > 3) {
                    inputsToOpen = inputsToOpen.slice(inputsToOpen.length - 3); // make sure to reduce the array to the last 3 elements if n > 3
                }
                inputsToRestore = inputsToOpen;
                widthRatios = (inputsToRestore.length === 3) ? [0.33, 0.33, 0.34] : (inputsToRestore.length === 2) ? [0.5, 0.5] : [1];
            }
            else if (this.memento[EDITOR_STATE_STORAGE_KEY]) {
                var editorState = this.memento[EDITOR_STATE_STORAGE_KEY];
                if (editorState && editorState.editors) {
                    // Find inputs to restore
                    var registry = platform_1.Registry.as(baseEditor_1.Extensions.Editors);
                    var inputs = [];
                    widthRatios = editorState.widthRatio;
                    for (var i = 0; i < editorState.editors.length; i++) {
                        var state = editorState.editors[i];
                        var factory = registry.getEditorInputFactory(state.inputId);
                        if (factory && types.isString(state.inputValue)) {
                            var input = factory.deserialize(this.instantiationService, state.inputValue);
                            if (input) {
                                if (state.hasFocus) {
                                    activeInput = input;
                                }
                                inputs.push(input);
                            }
                        }
                    }
                    if (inputs.length) {
                        inputsToRestore = inputs;
                    }
                }
            }
            // Do the restore
            if (inputsToRestore && inputsToRestore.length) {
                // Pick first input if we didnt find any active input from memento
                if (!activeInput && inputsToRestore.length) {
                    activeInput = inputsToRestore[0];
                }
                // Reset width ratios if they dont match with the number of editors to restore
                if (widthRatios && widthRatios.length !== inputsToRestore.length) {
                    widthRatios = inputsToRestore.length === 2 ? [0.5, 0.5] : null;
                }
                // Open editor inputs in parallel if any
                var promises_1 = [];
                inputsToRestore.forEach(function (input, index) {
                    var preserveFocus = (input !== activeInput);
                    var option;
                    if (options && options[index]) {
                        option = options[index];
                        option.preserveFocus = preserveFocus;
                    }
                    else {
                        option = editor_1.EditorOptions.create({ preserveFocus: preserveFocus });
                    }
                    promises_1.push(_this.openEditor(input, option, index, widthRatios));
                });
                return winjs_base_1.TPromise.join(promises_1).then(function (editors) {
                    // Workaround for bad layout issue: If any of the editors fails to load, reset side by side by closing
                    // all editors. This fixes an issue where a side editor might show, but no editor to the left hand side.
                    if (_this.getVisibleEditors().length !== inputsToRestore.length) {
                        _this.closeEditors().done(null, errors.onUnexpectedError);
                    }
                    // Full layout side by side
                    _this.sideBySideControl.layout(_this.dimension);
                    return editors;
                });
            }
            return winjs_base_1.TPromise.as([]);
        };
        EditorPart.prototype.activateEditor = function (editor) {
            if (editor) {
                this.sideBySideControl.setActive(editor);
            }
        };
        EditorPart.prototype.onEditorFocusChanged = function () {
            // Emit as editor input change event so that clients get aware of new active editor
            var activeEditor = this.sideBySideControl.getActiveEditor();
            if (activeEditor) {
                this.emit(events_2.EventType.EDITOR_INPUT_CHANGING, new events_2.EditorEvent(activeEditor, activeEditor.getId(), activeEditor.input, null, activeEditor.position));
                this.emit(events_2.EventType.EDITOR_INPUT_CHANGED, new events_2.EditorEvent(activeEditor, activeEditor.getId(), activeEditor.input, null, activeEditor.position));
            }
            // Update Title Area
            this.updateEditorTitleArea();
        };
        EditorPart.prototype.layout = function (dimension) {
            // Pass to super
            var sizes = _super.prototype.layout.call(this, dimension);
            // Pass to Side by Side Control
            this.dimension = sizes[1];
            this.sideBySideControl.layout(this.dimension);
            return sizes;
        };
        EditorPart.prototype.onSetInputError = function (e, editor, input, options, position) {
            // only show an error if this was not us restoring previous error state
            if (this.partService.isCreated() && !errors.isPromiseCanceledError(e)) {
                var errorMessage = nls.localize(0, null, input.getName(), errors.toErrorMessage(e));
                var error = void 0;
                if (e && e.actions && e.actions.length) {
                    error = errors.create(errorMessage, { actions: e.actions }); // Support error actions from thrower
                }
                else {
                    error = errorMessage;
                }
                this.showError(error);
            }
            this.sideBySideControl.getProgressBar(position).done().getContainer().hide();
            this.emit(events_2.EventType.EDITOR_SET_INPUT_ERROR, new events_2.EditorEvent(editor, editor.getId(), input, options, position));
        };
        EditorPart.prototype.showError = function (e) {
            return this.messageService.show(message_1.Severity.Error, types.isString(e) ? new Error(e) : e);
        };
        EditorPart.prototype.shutdown = function () {
            // Persist Editor State
            this.saveEditorState();
            // Unload all Instantiated Editors
            for (var i = 0; i < this.instantiatedEditors.length; i++) {
                for (var j = 0; j < this.instantiatedEditors[i].length; j++) {
                    this.instantiatedEditors[i][j].shutdown();
                }
            }
            // Pass to super
            _super.prototype.shutdown.call(this);
        };
        EditorPart.prototype.saveEditorState = function () {
            var registry = platform_1.Registry.as(baseEditor_1.Extensions.Editors);
            var editors = this.getVisibleEditors();
            var activeEditor = this.getActiveEditor();
            var widthRatios = this.sideBySideControl.getWidthRatios();
            var editorState = { editors: [], widthRatio: widthRatios };
            this.memento[EDITOR_STATE_STORAGE_KEY] = editorState;
            // For each visible editor
            for (var i = 0; i < editors.length; i++) {
                var editor = editors[i];
                var input = editor.input;
                // Serialize through factory
                if (input) {
                    var factory = registry.getEditorInputFactory(input.getId());
                    if (factory) {
                        var serialized = factory.serialize(input);
                        editorState.editors.push({
                            inputId: input.getId(),
                            inputValue: serialized,
                            hasFocus: activeEditor === editor
                        });
                    }
                }
            }
        };
        EditorPart.prototype.dispose = function () {
            var _this = this;
            this.mapEditorToEditorContainers = null;
            this.mapActionsToEditors = null;
            // Reset Tokens
            this.editorOpenToken = [];
            for (var i_1 = 0; i_1 < editor_2.POSITIONS.length; i_1++) {
                this.editorOpenToken[i_1] = 0;
            }
            // Widgets
            this.sideBySideControl.dispose();
            // Editor listeners
            for (var i_2 = 0; i_2 < this.visibleEditorListeners.length; i_2++) {
                while (this.visibleEditorListeners[i_2].length) {
                    this.visibleEditorListeners[i_2].pop()();
                }
            }
            // Input listeners
            for (var i_3 = 0; i_3 < this.visibleInputListeners.length; i_3++) {
                var listener = this.visibleInputListeners[i_3];
                if (listener) {
                    listener();
                }
                this.visibleInputListeners = [];
            }
            // Pass to active editors
            this.visibleEditors.forEach(function (editor) {
                if (editor) {
                    editor.dispose();
                }
            });
            // Pass to instantiated editors
            for (var i = 0; i < this.instantiatedEditors.length; i++) {
                for (var j = 0; j < this.instantiatedEditors[i].length; j++) {
                    if (this.visibleEditors.some(function (editor) { return editor === _this.instantiatedEditors[i][j]; })) {
                        continue;
                    }
                    this.instantiatedEditors[i][j].dispose();
                }
            }
            this.visibleEditors = null;
            this.visibleInputs = null;
            // Pass to super
            _super.prototype.dispose.call(this);
        };
        return EditorPart;
    }(part_1.Part));
    exports.EditorPart = EditorPart;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/electron-browser/darwin/cli.contribution", ["require", "exports", 'vs/nls!vs/workbench/electron-browser/darwin/cli.contribution', 'path', 'fs', 'os', 'child_process', 'vs/base/node/pfs', 'vs/base/common/async', 'vs/base/common/winjs.base', 'vs/base/common/uri', 'vs/base/common/actions', 'vs/workbench/common/actionRegistry', 'vs/workbench/common/contributions', 'vs/platform/platform', 'vs/platform/actions/common/actions', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/message/common/message', 'vs/platform/editor/common/editor', 'vs/platform/instantiation/common/instantiation'], function (require, exports, nls, path, fs, os, cp, pfs, async_1, winjs_base_1, uri_1, actions_1, actionRegistry_1, contributions_1, platform_1, actions_2, contextService_1, message_1, editor_1, instantiation_1) {
    "use strict";
    function ignore(code, value) {
        if (value === void 0) { value = null; }
        return function (err) { return err.code === code ? winjs_base_1.TPromise.as(value) : winjs_base_1.TPromise.wrapError(err); };
    }
    function readOrEmpty(name) {
        return pfs.readFile(name, 'utf8').then(null, ignore('ENOENT', ''));
    }
    var root = uri_1.default.parse(require.toUrl('')).fsPath;
    var source = path.resolve(root, '..', 'bin', 'code');
    var isAvailable = fs.existsSync(source);
    var InstallAction = (function (_super) {
        __extends(InstallAction, _super);
        function InstallAction(id, label, contextService, messageService, editorService) {
            _super.call(this, id, label);
            this.contextService = contextService;
            this.messageService = messageService;
            this.editorService = editorService;
        }
        Object.defineProperty(InstallAction.prototype, "applicationName", {
            get: function () {
                return this.contextService.getConfiguration().env.applicationName;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(InstallAction.prototype, "target", {
            get: function () {
                return "/usr/local/bin/" + this.applicationName;
            },
            enumerable: true,
            configurable: true
        });
        InstallAction.prototype.run = function () {
            var _this = this;
            return this.checkLegacy()
                .then(function (uses) {
                if (uses.length > 0) {
                    var _a = uses[0], file_1 = _a.file, lineNumber = _a.lineNumber;
                    var resource = uri_1.default.create('file', null, file_1);
                    var env = _this.contextService.getConfiguration().env;
                    var message = nls.localize(1, null, env.darwinBundleIdentifier, file_1, lineNumber);
                    var input_1 = { resource: resource, mime: 'text/x-shellscript' };
                    var actions = [
                        new actions_1.Action('inlineEdit', nls.localize(2, null, file_1), '', true, function () {
                            return _this.editorService.openEditor(input_1).then(function () {
                                var message = nls.localize(3, null, _this.applicationName, file_1);
                                var actions = [
                                    new actions_1.Action('cancel', nls.localize(4, null)),
                                    new actions_1.Action('continue', nls.localize(5, null), '', true, function () { return _this.run(); })
                                ];
                                _this.messageService.show(message_1.Severity.Info, { message: message, actions: actions });
                            });
                        })
                    ];
                    _this.messageService.show(message_1.Severity.Warning, { message: message, actions: actions });
                    return winjs_base_1.TPromise.as(null);
                }
                return _this.isInstalled()
                    .then(function (isInstalled) {
                    if (!isAvailable || isInstalled) {
                        return winjs_base_1.TPromise.as(null);
                    }
                    else {
                        var createSymlink_1 = function () {
                            return pfs.unlink(_this.target)
                                .then(null, ignore('ENOENT'))
                                .then(function () { return pfs.symlink(source, _this.target); });
                        };
                        return createSymlink_1().then(null, function (err) {
                            if (err.code === 'EACCES' || err.code === 'ENOENT') {
                                return _this.createBinFolder()
                                    .then(function () { return createSymlink_1(); });
                            }
                            return winjs_base_1.TPromise.wrapError(err);
                        });
                    }
                })
                    .then(function () { return _this.messageService.show(message_1.Severity.Info, nls.localize(6, null, _this.applicationName)); });
            });
        };
        InstallAction.prototype.isInstalled = function () {
            var _this = this;
            return pfs.lstat(this.target)
                .then(function (stat) { return stat.isSymbolicLink(); })
                .then(function () { return pfs.readlink(_this.target); })
                .then(function (link) { return link === source; })
                .then(null, ignore('ENOENT', false));
        };
        InstallAction.prototype.createBinFolder = function () {
            var _this = this;
            return new winjs_base_1.TPromise(function (c, e) {
                var message = nls.localize(7, null);
                var actions = [
                    new actions_1.Action('cancel2', nls.localize(8, null), '', true, function () { e(new Error(nls.localize(9, null))); return null; }),
                    new actions_1.Action('ok', nls.localize(10, null), '', true, function () {
                        var command = 'osascript -e "do shell script \\"mkdir -p /usr/local/bin && chown \\" & (do shell script (\\"whoami\\")) & \\" /usr/local/bin\\" with administrator privileges"';
                        async_1.nfcall(cp.exec, command, {})
                            .then(null, function (_) { return winjs_base_1.TPromise.wrapError(new Error(nls.localize(11, null))); })
                            .done(c, e);
                        return null;
                    })
                ];
                _this.messageService.show(message_1.Severity.Info, { message: message, actions: actions });
            });
        };
        InstallAction.prototype.checkLegacy = function () {
            var _this = this;
            var files = [
                path.join(os.homedir(), '.bash_profile'),
                path.join(os.homedir(), '.bashrc'),
                path.join(os.homedir(), '.zshrc')
            ];
            return winjs_base_1.TPromise.join(files.map(function (f) { return readOrEmpty(f); })).then(function (result) {
                return result.reduce(function (result, contents, index) {
                    var file = files[index];
                    var env = _this.contextService.getConfiguration().env;
                    var lines = contents.split(/\r?\n/);
                    lines.some(function (line, index) {
                        if (line.indexOf(env.darwinBundleIdentifier) > -1 && !/^\s*#/.test(line)) {
                            result.push({ file: file, lineNumber: index + 1 });
                            return true;
                        }
                        return false;
                    });
                    return result;
                }, []);
            });
        };
        InstallAction.ID = 'workbench.action.installCommandLine';
        InstallAction.LABEL = nls.localize(0, null);
        InstallAction = __decorate([
            __param(2, contextService_1.IWorkspaceContextService),
            __param(3, message_1.IMessageService),
            __param(4, editor_1.IEditorService)
        ], InstallAction);
        return InstallAction;
    }(actions_1.Action));
    var UninstallAction = (function (_super) {
        __extends(UninstallAction, _super);
        function UninstallAction(id, label, contextService, messageService) {
            _super.call(this, id, label);
            this.contextService = contextService;
            this.messageService = messageService;
        }
        Object.defineProperty(UninstallAction.prototype, "applicationName", {
            get: function () {
                return this.contextService.getConfiguration().env.applicationName;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UninstallAction.prototype, "target", {
            get: function () {
                return "/usr/local/bin/" + this.applicationName;
            },
            enumerable: true,
            configurable: true
        });
        UninstallAction.prototype.run = function () {
            var _this = this;
            return pfs.unlink(this.target)
                .then(null, ignore('ENOENT'))
                .then(function () { return _this.messageService.show(message_1.Severity.Info, nls.localize(13, null, _this.applicationName)); });
        };
        UninstallAction.ID = 'workbench.action.uninstallCommandLine';
        UninstallAction.LABEL = nls.localize(12, null);
        UninstallAction = __decorate([
            __param(2, contextService_1.IWorkspaceContextService),
            __param(3, message_1.IMessageService)
        ], UninstallAction);
        return UninstallAction;
    }(actions_1.Action));
    var DarwinCLIHelper = (function () {
        function DarwinCLIHelper(instantiationService, messageService) {
            var installAction = instantiationService.createInstance(InstallAction, InstallAction.ID, InstallAction.LABEL);
            installAction.checkLegacy().done(function (files) {
                if (files.length > 0) {
                    var message = nls.localize(14, null, installAction.applicationName);
                    var now = new actions_1.Action('changeNow', nls.localize(15, null), '', true, function () { return installAction.run(); });
                    var later = new actions_1.Action('later', nls.localize(16, null), '', true, function () {
                        messageService.show(message_1.Severity.Info, nls.localize(17, null, installAction.label));
                        return null;
                    });
                    var actions = [later, now];
                    messageService.show(message_1.Severity.Info, { message: message, actions: actions });
                }
            });
        }
        DarwinCLIHelper.prototype.getId = function () {
            return 'darwin.cli';
        };
        DarwinCLIHelper = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, message_1.IMessageService)
        ], DarwinCLIHelper);
        return DarwinCLIHelper;
    }());
    if (isAvailable && process.platform === 'darwin') {
        var category = nls.localize(18, null);
        var workbenchActionsRegistry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
        workbenchActionsRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(InstallAction, InstallAction.ID, InstallAction.LABEL), category);
        workbenchActionsRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(UninstallAction, UninstallAction.ID, UninstallAction.LABEL), category);
        var workbenchRegistry = platform_1.Registry.as(contributions_1.Extensions.Workbench);
        workbenchRegistry.registerWorkbenchContribution(DarwinCLIHelper);
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/electron-browser/integration", ["require", "exports", 'vs/nls!vs/workbench/electron-browser/integration', 'vs/base/common/winjs.base', 'vs/base/common/errors', 'vs/base/common/arrays', 'vs/base/common/severity', 'vs/base/browser/ui/actionbar/actionbar', 'vs/base/common/actions', 'vs/workbench/services/part/common/partService', 'vs/platform/storage/common/storage', 'vs/platform/message/common/message', 'vs/platform/instantiation/common/instantiation', 'vs/platform/telemetry/common/telemetry', 'vs/platform/contextview/browser/contextView', 'vs/platform/keybinding/common/keybindingService', 'vs/workbench/services/workspace/common/contextService', 'vs/workbench/services/window/electron-browser/windowService', 'vs/platform/configuration/common/configuration', 'vs/workbench/electron-browser/window', 'electron'], function (require, exports, nls, winjs_base_1, errors, arrays, severity_1, actionbar_1, actions_1, partService_1, storage_1, message_1, instantiation_1, telemetry_1, contextView_1, keybindingService_1, contextService_1, windowService_1, configuration_1, win, electron_1) {
    'use strict';
    var currentWindow = electron_1.remote.getCurrentWindow();
    var TextInputActions = [
        new actions_1.Action('undo', nls.localize(0, null), null, true, function () { return document.execCommand('undo') && winjs_base_1.TPromise.as(true); }),
        new actions_1.Action('redo', nls.localize(1, null), null, true, function () { return document.execCommand('redo') && winjs_base_1.TPromise.as(true); }),
        new actionbar_1.Separator(),
        new actions_1.Action('editor.action.clipboardCutAction', nls.localize(2, null), null, true, function () { return document.execCommand('cut') && winjs_base_1.TPromise.as(true); }),
        new actions_1.Action('editor.action.clipboardCopyAction', nls.localize(3, null), null, true, function () { return document.execCommand('copy') && winjs_base_1.TPromise.as(true); }),
        new actions_1.Action('editor.action.clipboardPasteAction', nls.localize(4, null), null, true, function () { return document.execCommand('paste') && winjs_base_1.TPromise.as(true); }),
        new actionbar_1.Separator(),
        new actions_1.Action('editor.action.selectAll', nls.localize(5, null), null, true, function () { return document.execCommand('selectAll') && winjs_base_1.TPromise.as(true); })
    ];
    var ElectronIntegration = (function () {
        function ElectronIntegration(instantiationService, windowService, partService, contextService, telemetryService, configurationService, keybindingService, storageService, messageService, contextMenuService) {
            this.instantiationService = instantiationService;
            this.windowService = windowService;
            this.partService = partService;
            this.contextService = contextService;
            this.telemetryService = telemetryService;
            this.configurationService = configurationService;
            this.keybindingService = keybindingService;
            this.storageService = storageService;
            this.messageService = messageService;
            this.contextMenuService = contextMenuService;
        }
        ElectronIntegration.prototype.integrate = function (shellContainer) {
            var _this = this;
            // Register the active window
            var activeWindow = this.instantiationService.createInstance(win.ElectronWindow, currentWindow, shellContainer);
            this.windowService.registerWindow(activeWindow);
            // Support runAction event
            electron_1.ipcRenderer.on('vscode:runAction', function (event, actionId) {
                _this.keybindingService.executeCommand(actionId, { from: 'menu' }).done(undefined, function (err) { return _this.messageService.show(severity_1.default.Error, err); });
            });
            // Support options change
            electron_1.ipcRenderer.on('vscode:optionsChange', function (event, options) {
                var optionsData = JSON.parse(options);
                for (var key in optionsData) {
                    if (optionsData.hasOwnProperty(key)) {
                        var value = optionsData[key];
                        _this.contextService.updateOptions(key, value);
                    }
                }
            });
            // Support resolve keybindings event
            electron_1.ipcRenderer.on('vscode:resolveKeybindings', function (event, rawActionIds) {
                var actionIds = [];
                try {
                    actionIds = JSON.parse(rawActionIds);
                }
                catch (error) {
                }
                // Resolve keys using the keybinding service and send back to browser process
                _this.resolveKeybindings(actionIds).done(function (keybindings) {
                    if (keybindings.length) {
                        electron_1.ipcRenderer.send('vscode:keybindingsResolved', JSON.stringify(keybindings));
                    }
                }, function () { return errors.onUnexpectedError; });
            });
            electron_1.ipcRenderer.on('vscode:telemetry', function (event, _a) {
                var eventName = _a.eventName, data = _a.data;
                _this.telemetryService.publicLog(eventName, data);
            });
            electron_1.ipcRenderer.on('vscode:reportError', function (event, error) {
                if (error) {
                    var errorParsed = JSON.parse(error);
                    errorParsed.mainProcess = true;
                    errors.onUnexpectedError(errorParsed);
                }
            });
            // Emit event when vscode has loaded
            this.partService.joinCreation().then(function () {
                electron_1.ipcRenderer.send('vscode:workbenchLoaded', _this.windowService.getWindowId());
            });
            // Theme changes
            electron_1.ipcRenderer.on('vscode:changeTheme', function (event, theme) {
                _this.storageService.store('workbench.theme', theme, storage_1.StorageScope.GLOBAL);
            });
            // Message support
            electron_1.ipcRenderer.on('vscode:showInfoMessage', function (event, message) {
                _this.messageService.show(severity_1.default.Info, message);
            });
            // Configuration changes
            var previousConfiguredZoomLevel;
            this.configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) {
                var windowConfig = e.config;
                var newZoomLevel = 0;
                if (windowConfig.window && typeof windowConfig.window.zoomLevel === 'number') {
                    newZoomLevel = windowConfig.window.zoomLevel;
                    // Leave early if the configured zoom level did not change (https://github.com/Microsoft/vscode/issues/1536)
                    if (previousConfiguredZoomLevel === newZoomLevel) {
                        return;
                    }
                    previousConfiguredZoomLevel = newZoomLevel;
                }
                if (electron_1.webFrame.getZoomLevel() !== newZoomLevel) {
                    electron_1.webFrame.setZoomLevel(newZoomLevel);
                }
            });
            // Context menu support in input/textarea
            window.document.addEventListener('contextmenu', function (e) {
                if (e.target instanceof HTMLElement) {
                    var target_1 = e.target;
                    if (target_1.nodeName && (target_1.nodeName.toLowerCase() === 'input' || target_1.nodeName.toLowerCase() === 'textarea')) {
                        e.preventDefault();
                        e.stopPropagation();
                        _this.contextMenuService.showContextMenu({
                            getAnchor: function () { return target_1; },
                            getActions: function () { return winjs_base_1.TPromise.as(TextInputActions); },
                            getKeyBinding: function (action) {
                                var opts = _this.keybindingService.lookupKeybindings(action.id);
                                if (opts.length > 0) {
                                    return opts[0]; // only take the first one
                                }
                                return null;
                            }
                        });
                    }
                }
            });
        };
        ElectronIntegration.prototype.resolveKeybindings = function (actionIds) {
            var _this = this;
            return this.partService.joinCreation().then(function () {
                return arrays.coalesce(actionIds.map(function (id) {
                    var bindings = _this.keybindingService.lookupKeybindings(id);
                    // return the first binding that can be represented by electron
                    for (var i = 0; i < bindings.length; i++) {
                        var binding = bindings[i];
                        var electronAccelerator = _this.keybindingService.getElectronAcceleratorFor(binding);
                        if (electronAccelerator) {
                            return {
                                id: id,
                                binding: binding.value
                            };
                        }
                    }
                    return null;
                }));
            });
        };
        ElectronIntegration = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, windowService_1.IWindowService),
            __param(2, partService_1.IPartService),
            __param(3, contextService_1.IWorkspaceContextService),
            __param(4, telemetry_1.ITelemetryService),
            __param(5, configuration_1.IConfigurationService),
            __param(6, keybindingService_1.IKeybindingService),
            __param(7, storage_1.IStorageService),
            __param(8, message_1.IMessageService),
            __param(9, contextView_1.IContextMenuService)
        ], ElectronIntegration);
        return ElectronIntegration;
    }());
    exports.ElectronIntegration = ElectronIntegration;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/electron-browser/update", ["require", "exports", 'vs/nls!vs/workbench/electron-browser/update', 'vs/base/common/severity', 'vs/base/common/winjs.base', 'vs/base/common/actions', 'electron', 'vs/base/common/platform', 'vs/platform/message/common/message', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/request/common/request'], function (require, exports, nls, severity_1, winjs_base_1, actions_1, electron_1, platform_1, message_1, contextService_1, request_1) {
    'use strict';
    var ApplyUpdateAction = new actions_1.Action('update.applyUpdate', nls.localize(0, null), null, true, function () { electron_1.ipcRenderer.send('vscode:update-apply'); return winjs_base_1.TPromise.as(true); });
    var NotNowAction = new actions_1.Action('update.later', nls.localize(1, null), null, true, function () { return winjs_base_1.TPromise.as(true); });
    exports.ShowReleaseNotesAction = function (releaseNotesUrl, returnValue) {
        if (returnValue === void 0) { returnValue = false; }
        return new actions_1.Action('update.showReleaseNotes', nls.localize(2, null), null, true, function () { electron_1.shell.openExternal(releaseNotesUrl); return winjs_base_1.TPromise.as(returnValue); });
    };
    var Update = (function () {
        function Update(contextService, messageService, requestService) {
            var _this = this;
            this.contextService = contextService;
            this.messageService = messageService;
            this.requestService = requestService;
            var env = this.contextService.getConfiguration().env;
            electron_1.ipcRenderer.on('vscode:update-downloaded', function (event, update) {
                _this.messageService.show(severity_1.default.Info, {
                    message: nls.localize(3, null, env.appName),
                    actions: [exports.ShowReleaseNotesAction(env.releaseNotesUrl), NotNowAction, ApplyUpdateAction]
                });
            });
            electron_1.ipcRenderer.on('vscode:update-not-available', function () {
                _this.messageService.show(severity_1.default.Info, nls.localize(4, null));
            });
            var updateFeedUrl = env.updateFeedUrl;
            // manually check for update on linux
            if (platform_1.isLinux && updateFeedUrl) {
                this.requestService.makeRequest({ url: updateFeedUrl }).done(function (res) {
                    if (res.status !== 200) {
                        return; // no update available
                    }
                    _this.messageService.show(severity_1.default.Info, {
                        message: nls.localize(5, null, env.appName),
                        actions: [
                            new actions_1.Action('pleaseUpdate', nls.localize(6, null), '', true, function () {
                                electron_1.shell.openExternal(env.productDownloadUrl);
                                return winjs_base_1.TPromise.as(true);
                            }),
                            new actions_1.Action('releaseNotes', nls.localize(7, null), '', true, function () {
                                electron_1.shell.openExternal(env.releaseNotesUrl);
                                return winjs_base_1.TPromise.as(false);
                            })
                        ]
                    });
                });
            }
        }
        Update = __decorate([
            __param(0, contextService_1.IWorkspaceContextService),
            __param(1, message_1.IMessageService),
            __param(2, request_1.IRequestService)
        ], Update);
        return Update;
    }());
    exports.Update = Update;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/debug/browser/debugEditorContribution", ["require", "exports", 'vs/nls!vs/workbench/parts/debug/browser/debugEditorContribution', 'vs/base/common/winjs.base', 'vs/base/common/async', 'vs/base/common/lifecycle', 'vs/base/common/platform', 'vs/base/common/actions', 'vs/base/common/keyCodes', 'vs/editor/common/editorCommon', 'vs/workbench/parts/debug/browser/debugHover', 'vs/workbench/parts/debug/electron-browser/debugActions', 'vs/workbench/parts/debug/common/debug', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/instantiation/common/instantiation', 'vs/platform/contextview/browser/contextView'], function (require, exports, nls, winjs_base_1, async_1, lifecycle, env, actions_1, keyCodes_1, editorcommon, debugHover_1, debugactions, debug, contextService_1, instantiation_1, contextView_1) {
    "use strict";
    var HOVER_DELAY = 300;
    var DebugEditorContribution = (function () {
        function DebugEditorContribution(editor, debugService, contextService, contextMenuService, instantiationService) {
            var _this = this;
            this.editor = editor;
            this.debugService = debugService;
            this.contextService = contextService;
            this.contextMenuService = contextMenuService;
            this.instantiationService = instantiationService;
            this.breakpointHintDecoration = [];
            this.hoverWidget = new debugHover_1.DebugHoverWidget(this.editor, this.debugService, this.instantiationService);
            this.toDispose = [this.hoverWidget];
            this.showHoverScheduler = new async_1.RunOnceScheduler(function () { return _this.showHover(_this.hoverRange, _this.hoveringOver, false); }, HOVER_DELAY);
            this.hideHoverScheduler = new async_1.RunOnceScheduler(function () { return _this.hoverWidget.hide(); }, HOVER_DELAY);
            this.registerListeners();
        }
        DebugEditorContribution.getDebugEditorContribution = function (editor) {
            return editor.getContribution(debug.EDITOR_CONTRIBUTION_ID);
        };
        DebugEditorContribution.prototype.getContextMenuActions = function (breakpoint, uri, lineNumber) {
            var _this = this;
            var actions = [];
            if (breakpoint) {
                actions.push(this.instantiationService.createInstance(debugactions.RemoveBreakpointAction, debugactions.RemoveBreakpointAction.ID, debugactions.RemoveBreakpointAction.LABEL));
                actions.push(this.instantiationService.createInstance(debugactions.EditConditionalBreakpointAction, debugactions.EditConditionalBreakpointAction.ID, debugactions.EditConditionalBreakpointAction.LABEL, this.editor, lineNumber));
                actions.push(this.instantiationService.createInstance(debugactions.ToggleEnablementAction, debugactions.ToggleEnablementAction.ID, debugactions.ToggleEnablementAction.LABEL));
            }
            else {
                actions.push(new actions_1.Action('addBreakpoint', nls.localize(0, null), null, true, function () { return _this.debugService.toggleBreakpoint({ uri: uri, lineNumber: lineNumber }); }));
                actions.push(this.instantiationService.createInstance(debugactions.AddConditionalBreakpointAction, debugactions.AddConditionalBreakpointAction.ID, debugactions.AddConditionalBreakpointAction.LABEL, this.editor, lineNumber));
            }
            return winjs_base_1.TPromise.as(actions);
        };
        DebugEditorContribution.prototype.registerListeners = function () {
            var _this = this;
            this.toDispose.push(this.editor.addListener2(editorcommon.EventType.MouseDown, function (e) {
                if (e.target.type !== editorcommon.MouseTargetType.GUTTER_GLYPH_MARGIN || e.target.detail) {
                    return;
                }
                if (!_this.debugService.canSetBreakpointsIn(_this.editor.getModel())) {
                    return;
                }
                var lineNumber = e.target.position.lineNumber;
                var uri = _this.editor.getModel().getAssociatedResource();
                if (e.event.rightButton || (env.isMacintosh && e.event.leftButton && e.event.ctrlKey)) {
                    var anchor_1 = { x: e.event.posx + 1, y: e.event.posy };
                    var breakpoint_1 = _this.debugService.getModel().getBreakpoints().filter(function (bp) { return bp.lineNumber === lineNumber && bp.source.uri.toString() === uri.toString(); }).pop();
                    _this.contextMenuService.showContextMenu({
                        getAnchor: function () { return anchor_1; },
                        getActions: function () { return _this.getContextMenuActions(breakpoint_1, uri, lineNumber); },
                        getActionsContext: function () { return breakpoint_1; }
                    });
                }
                else {
                    _this.debugService.toggleBreakpoint({ uri: uri, lineNumber: lineNumber });
                }
            }));
            this.toDispose.push(this.editor.addListener2(editorcommon.EventType.MouseMove, function (e) {
                var showBreakpointHintAtLineNumber = -1;
                if (e.target.type === editorcommon.MouseTargetType.GUTTER_GLYPH_MARGIN && _this.debugService.canSetBreakpointsIn(_this.editor.getModel())) {
                    if (!e.target.detail) {
                        // is not after last line
                        showBreakpointHintAtLineNumber = e.target.position.lineNumber;
                    }
                }
                _this.ensureBreakpointHintDecoration(showBreakpointHintAtLineNumber);
            }));
            this.toDispose.push(this.editor.addListener2(editorcommon.EventType.MouseLeave, function (e) {
                _this.ensureBreakpointHintDecoration(-1);
            }));
            this.toDispose.push(this.debugService.addListener2(debug.ServiceEvents.STATE_CHANGED, function () { return _this.onDebugStateUpdate(); }));
            // hover listeners & hover widget
            this.toDispose.push(this.editor.addListener2(editorcommon.EventType.MouseDown, function (e) { return _this.onEditorMouseDown(e); }));
            this.toDispose.push(this.editor.addListener2(editorcommon.EventType.MouseMove, function (e) { return _this.onEditorMouseMove(e); }));
            this.toDispose.push(this.editor.addListener2(editorcommon.EventType.MouseLeave, function (e) { return _this.hoverWidget.hide(); }));
            this.toDispose.push(this.editor.addListener2(editorcommon.EventType.KeyDown, function (e) { return _this.onKeyDown(e); }));
            this.toDispose.push(this.editor.addListener2(editorcommon.EventType.ModelChanged, function () { return _this.hideHoverWidget(); }));
            this.toDispose.push(this.editor.addListener2('scroll', function () { return _this.hideHoverWidget; }));
        };
        DebugEditorContribution.prototype.getId = function () {
            return debug.EDITOR_CONTRIBUTION_ID;
        };
        DebugEditorContribution.prototype.showHover = function (range, hoveringOver, focus) {
            return this.hoverWidget.showAt(range, hoveringOver, focus);
        };
        DebugEditorContribution.prototype.ensureBreakpointHintDecoration = function (showBreakpointHintAtLineNumber) {
            var newDecoration = [];
            if (showBreakpointHintAtLineNumber !== -1) {
                newDecoration.push({
                    options: DebugEditorContribution.BREAKPOINT_HELPER_DECORATION,
                    range: {
                        startLineNumber: showBreakpointHintAtLineNumber,
                        startColumn: 1,
                        endLineNumber: showBreakpointHintAtLineNumber,
                        endColumn: 1
                    }
                });
            }
            this.breakpointHintDecoration = this.editor.deltaDecorations(this.breakpointHintDecoration, newDecoration);
        };
        DebugEditorContribution.prototype.onDebugStateUpdate = function () {
            if (this.debugService.getState() !== debug.State.Stopped) {
                this.hideHoverWidget();
            }
            this.contextService.updateOptions('editor', {
                hover: this.debugService.getState() !== debug.State.Stopped
            });
        };
        DebugEditorContribution.prototype.hideHoverWidget = function () {
            if (!this.hideHoverScheduler.isScheduled() && this.hoverWidget.isVisible) {
                this.hideHoverScheduler.schedule();
            }
            this.showHoverScheduler.cancel();
            this.hoveringOver = null;
        };
        // hover business
        DebugEditorContribution.prototype.onEditorMouseDown = function (mouseEvent) {
            if (mouseEvent.target.type === editorcommon.MouseTargetType.CONTENT_WIDGET && mouseEvent.target.detail === debugHover_1.DebugHoverWidget.ID) {
                return;
            }
            this.hideHoverWidget();
        };
        DebugEditorContribution.prototype.onEditorMouseMove = function (mouseEvent) {
            if (this.debugService.getState() !== debug.State.Stopped) {
                return;
            }
            var targetType = mouseEvent.target.type;
            var stopKey = env.isMacintosh ? 'metaKey' : 'ctrlKey';
            if (targetType === editorcommon.MouseTargetType.CONTENT_WIDGET && mouseEvent.target.detail === debugHover_1.DebugHoverWidget.ID && !mouseEvent.event[stopKey]) {
                // mouse moved on top of debug hover widget
                return;
            }
            if (targetType === editorcommon.MouseTargetType.CONTENT_TEXT) {
                var wordAtPosition = this.editor.getModel().getWordAtPosition(mouseEvent.target.range.getStartPosition());
                if (wordAtPosition && this.hoveringOver !== wordAtPosition.word) {
                    this.hoverRange = mouseEvent.target.range;
                    this.hoveringOver = wordAtPosition.word;
                    this.showHoverScheduler.schedule();
                }
            }
            else {
                this.hideHoverWidget();
            }
        };
        DebugEditorContribution.prototype.onKeyDown = function (e) {
            var stopKey = env.isMacintosh ? keyCodes_1.KeyCode.Meta : keyCodes_1.KeyCode.Ctrl;
            if (e.keyCode !== stopKey) {
                // do not hide hover when Ctrl/Meta is pressed
                this.hideHoverWidget();
            }
        };
        DebugEditorContribution.prototype.dispose = function () {
            this.toDispose = lifecycle.disposeAll(this.toDispose);
        };
        // end hover business
        DebugEditorContribution.BREAKPOINT_HELPER_DECORATION = {
            glyphMarginClassName: 'debug-breakpoint-hint-glyph',
            stickiness: editorcommon.TrackedRangeStickiness.NeverGrowsWhenTypingAtEdges
        };
        DebugEditorContribution = __decorate([
            __param(1, debug.IDebugService),
            __param(2, contextService_1.IWorkspaceContextService),
            __param(3, contextView_1.IContextMenuService),
            __param(4, instantiation_1.IInstantiationService)
        ], DebugEditorContribution);
        return DebugEditorContribution;
    }());
    exports.DebugEditorContribution = DebugEditorContribution;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/debug/node/debugConfigurationManager", ["require", "exports", 'path', 'vs/nls!vs/workbench/parts/debug/node/debugConfigurationManager', 'vs/base/common/winjs.base', 'vs/base/common/objects', 'vs/base/common/uri', 'vs/base/common/network', 'vs/base/common/paths', 'vs/platform/extensions/common/extensionsRegistry', 'vs/platform/platform', 'vs/platform/jsonschemas/common/jsonContributionRegistry', 'vs/platform/configuration/common/configuration', 'vs/platform/files/common/files', 'vs/platform/telemetry/common/telemetry', 'vs/workbench/parts/lib/node/systemVariables', 'vs/workbench/parts/debug/node/debugAdapter', 'vs/workbench/services/workspace/common/contextService', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/quickopen/common/quickOpenService'], function (require, exports, path, nls, winjs_base_1, objects, uri_1, network_1, paths, extensionsRegistry, platform, jsonContributionRegistry, configuration_1, files_1, telemetry_1, systemVariables_1, debugAdapter_1, contextService_1, editorService_1, quickOpenService_1) {
    "use strict";
    // debuggers extension point
    exports.debuggersExtPoint = extensionsRegistry.ExtensionsRegistry.registerExtensionPoint('debuggers', {
        description: nls.localize(0, null),
        type: 'array',
        defaultSnippets: [{ body: [{ type: '', extensions: [] }] }],
        items: {
            type: 'object',
            defaultSnippets: [{ body: { type: '', program: '', runtime: '', enableBreakpointsFor: { languageIds: [''] } } }],
            properties: {
                type: {
                    description: nls.localize(1, null),
                    type: 'string'
                },
                label: {
                    description: nls.localize(2, null),
                    type: 'string'
                },
                enableBreakpointsFor: {
                    description: nls.localize(3, null),
                    type: 'object',
                    properties: {
                        languageIds: {
                            description: nls.localize(4, null),
                            type: 'array',
                            items: {
                                type: 'string'
                            }
                        }
                    }
                },
                program: {
                    description: nls.localize(5, null),
                    type: 'string'
                },
                args: {
                    description: nls.localize(6, null),
                    type: 'array'
                },
                runtime: {
                    description: nls.localize(7, null),
                    type: 'string'
                },
                runtimeArgs: {
                    description: nls.localize(8, null),
                    type: 'array'
                },
                initialConfigurations: {
                    description: nls.localize(9, null),
                    type: 'array',
                },
                configurationAttributes: {
                    description: nls.localize(10, null),
                    type: 'object'
                },
                windows: {
                    description: nls.localize(11, null),
                    type: 'object',
                    properties: {
                        runtime: {
                            description: nls.localize(12, null),
                            type: 'string'
                        }
                    }
                },
                osx: {
                    description: nls.localize(13, null),
                    type: 'object',
                    properties: {
                        runtime: {
                            description: nls.localize(14, null),
                            type: 'string'
                        }
                    }
                },
                linux: {
                    description: nls.localize(15, null),
                    type: 'object',
                    properties: {
                        runtime: {
                            description: nls.localize(16, null),
                            type: 'string'
                        }
                    }
                }
            }
        }
    });
    // debug general schema
    exports.schemaId = 'vscode://schemas/launch';
    var schema = {
        id: exports.schemaId,
        type: 'object',
        title: nls.localize(17, null),
        required: ['version', 'configurations'],
        properties: {
            version: {
                type: 'string',
                description: nls.localize(18, null),
                default: '0.2.0'
            },
            configurations: {
                type: 'array',
                description: nls.localize(19, null),
                items: {
                    oneOf: []
                }
            }
        }
    };
    var jsonRegistry = platform.Registry.as(jsonContributionRegistry.Extensions.JSONContribution);
    jsonRegistry.registerSchema(exports.schemaId, schema);
    jsonRegistry.addSchemaFileAssociation('/.vscode/launch.json', exports.schemaId);
    var ConfigurationManager = (function () {
        function ConfigurationManager(configName, contextService, fileService, telemetryService, editorService, configurationService, quickOpenService) {
            this.contextService = contextService;
            this.fileService = fileService;
            this.telemetryService = telemetryService;
            this.editorService = editorService;
            this.configurationService = configurationService;
            this.quickOpenService = quickOpenService;
            this.systemVariables = this.contextService.getWorkspace() ? new systemVariables_1.SystemVariables(this.editorService, this.contextService) : null;
            this.setConfiguration(configName);
            this.adapters = [];
            this.registerListeners();
            this.allModeIdsForBreakpoints = {};
        }
        ConfigurationManager.prototype.registerListeners = function () {
            var _this = this;
            exports.debuggersExtPoint.setHandler(function (extensions) {
                extensions.forEach(function (extension) {
                    extension.value.forEach(function (rawAdapter) {
                        var adapter = new debugAdapter_1.Adapter(rawAdapter, _this.systemVariables, extension.description.extensionFolderPath);
                        var duplicate = _this.adapters.filter(function (a) { return a.type === adapter.type; })[0];
                        if (!rawAdapter.type || (typeof rawAdapter.type !== 'string')) {
                            extension.collector.error(nls.localize(20, null));
                        }
                        if (duplicate) {
                            Object.keys(adapter).forEach(function (attribute) {
                                if (adapter[attribute]) {
                                    if (attribute === 'enableBreakpointsFor') {
                                        Object.keys(adapter.enableBreakpointsFor).forEach(function (languageId) { return duplicate.enableBreakpointsFor[languageId] = true; });
                                    }
                                    else if (duplicate[attribute] && attribute !== 'type') {
                                        // give priority to the later registered extension.
                                        duplicate[attribute] = adapter[attribute];
                                        extension.collector.error(nls.localize(21, null, adapter.type, attribute));
                                    }
                                    else {
                                        duplicate[attribute] = adapter[attribute];
                                    }
                                }
                            });
                        }
                        else {
                            _this.adapters.push(adapter);
                        }
                        if (adapter.enableBreakpointsFor) {
                            adapter.enableBreakpointsFor.languageIds.forEach(function (modeId) {
                                _this.allModeIdsForBreakpoints[modeId] = true;
                            });
                        }
                    });
                });
                // update the schema to include all attributes and types from extensions.
                // debug.schema.properties['configurations'].items.properties.type.enum = this.adapters.map(adapter => adapter.type);
                _this.adapters.forEach(function (adapter) {
                    var schemaAttributes = adapter.getSchemaAttributes();
                    if (schemaAttributes) {
                        (_a = schema.properties['configurations'].items.oneOf).push.apply(_a, schemaAttributes);
                    }
                    var _a;
                });
            });
        };
        ConfigurationManager.prototype.getConfiguration = function () {
            return this.configuration;
        };
        ConfigurationManager.prototype.getConfigurationName = function () {
            return this.configuration ? this.configuration.name : null;
        };
        ConfigurationManager.prototype.getAdapter = function () {
            var _this = this;
            return this.adapters.filter(function (adapter) { return adapter.type === _this.configuration.type; }).pop();
        };
        ConfigurationManager.prototype.setConfiguration = function (name) {
            var _this = this;
            return this.loadLaunchConfig().then(function (config) {
                if (!config || !config.configurations) {
                    _this.configuration = null;
                    return;
                }
                // if the configuration name is not set yet, take the first launch config (can happen if debug viewlet has not been opened yet).
                var filtered = name ? config.configurations.filter(function (cfg) { return cfg.name === name; }) : [config.configurations[0]];
                // massage configuration attributes - append workspace path to relatvie paths, substitute variables in paths.
                _this.configuration = filtered.length === 1 ? objects.deepClone(filtered[0]) : null;
                if (_this.configuration) {
                    if (_this.systemVariables) {
                        Object.keys(_this.configuration).forEach(function (key) {
                            _this.configuration[key] = _this.systemVariables.resolveAny(_this.configuration[key]);
                        });
                    }
                    _this.configuration.debugServer = config.debugServer;
                }
            });
        };
        ConfigurationManager.prototype.openConfigFile = function (sideBySide) {
            var _this = this;
            var resource = uri_1.default.file(paths.join(this.contextService.getWorkspace().resource.fsPath, '/.vscode/launch.json'));
            return this.fileService.resolveContent(resource).then(function (content) { return true; }, function (err) {
                return _this.getInitialConfigFileContent().then(function (content) {
                    if (!content) {
                        return false;
                    }
                    return _this.fileService.updateContent(resource, content).then(function () { return true; });
                });
            }).then(function (configFileCreated) {
                if (!configFileCreated) {
                    return false;
                }
                _this.telemetryService.publicLog('debugConfigure');
                return _this.editorService.openEditor({
                    resource: resource,
                    options: {
                        forceOpen: true
                    }
                }, sideBySide).then(function () { return true; });
            }, function (error) {
                throw new Error(nls.localize(22, null, error));
            });
        };
        ConfigurationManager.prototype.getInitialConfigFileContent = function () {
            var _this = this;
            return this.quickOpenService.pick(this.adapters, { placeHolder: nls.localize(23, null) })
                .then(function (adapter) {
                if (!adapter) {
                    return null;
                }
                return _this.massageInitialConfigurations(adapter).then(function () {
                    return JSON.stringify({
                        version: '0.2.0',
                        configurations: adapter.initialConfigurations ? adapter.initialConfigurations : []
                    }, null, '\t');
                });
            });
        };
        ConfigurationManager.prototype.massageInitialConfigurations = function (adapter) {
            if (!adapter || !adapter.initialConfigurations || adapter.type !== 'node') {
                return winjs_base_1.TPromise.as(undefined);
            }
            // check package.json for 'main' or 'scripts' so we generate a more pecise 'program' attribute in launch.json.
            var packageJsonUri = uri_1.default.file(paths.join(this.contextService.getWorkspace().resource.fsPath, '/package.json'));
            return this.fileService.resolveContent(packageJsonUri).then(function (jsonContent) {
                try {
                    var jsonObject = JSON.parse(jsonContent.value);
                    if (jsonObject.main) {
                        return jsonObject.main;
                    }
                    else if (jsonObject.scripts && typeof jsonObject.scripts.start === 'string') {
                        return jsonObject.scripts.start.split(' ').pop();
                    }
                }
                catch (error) { }
                return null;
            }, function (err) { return null; }).then(function (program) {
                adapter.initialConfigurations.forEach(function (config) {
                    if (program && config.program) {
                        if (!path.isAbsolute(program)) {
                            program = paths.join('${workspaceRoot}', program);
                        }
                        config.program = program;
                    }
                });
            });
        };
        ConfigurationManager.prototype.canSetBreakpointsIn = function (model) {
            if (model.getAssociatedResource().scheme === network_1.Schemas.inMemory) {
                return false;
            }
            var mode = model ? model.getMode() : null;
            var modeId = mode ? mode.getId() : null;
            return !!this.allModeIdsForBreakpoints[modeId];
        };
        ConfigurationManager.prototype.loadLaunchConfig = function () {
            return this.configurationService.loadConfiguration('launch');
        };
        ConfigurationManager = __decorate([
            __param(1, contextService_1.IWorkspaceContextService),
            __param(2, files_1.IFileService),
            __param(3, telemetry_1.ITelemetryService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, configuration_1.IConfigurationService),
            __param(6, quickOpenService_1.IQuickOpenService)
        ], ConfigurationManager);
        return ConfigurationManager;
    }());
    exports.ConfigurationManager = ConfigurationManager;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/debug/electron-browser/debugService", ["require", "exports", 'vs/nls!vs/workbench/parts/debug/electron-browser/debugService', 'vs/base/common/lifecycle', 'vs/base/common/mime', 'vs/base/common/eventEmitter', 'vs/base/common/uri', 'vs/base/common/actions', 'vs/base/common/arrays', 'vs/base/common/types', 'vs/base/common/errors', 'vs/base/common/severity', 'vs/base/common/winjs.base', 'vs/base/browser/ui/aria/aria', 'vs/base/node/aiAdapter', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/markers/common/markers', 'vs/platform/lifecycle/common/lifecycle', 'vs/platform/extensions/common/extensions', 'vs/platform/instantiation/common/instantiation', 'vs/platform/files/common/files', 'vs/platform/event/common/event', 'vs/platform/message/common/message', 'vs/platform/telemetry/common/telemetry', 'vs/platform/storage/common/storage', 'vs/workbench/common/editor', 'vs/workbench/parts/debug/common/debug', 'vs/workbench/parts/debug/node/rawDebugSession', 'vs/workbench/parts/debug/common/debugModel', 'vs/workbench/parts/debug/browser/debugEditorInputs', 'vs/workbench/parts/debug/common/debugViewModel', 'vs/workbench/parts/debug/electron-browser/debugActions', 'vs/workbench/parts/debug/browser/breakpointWidget', 'vs/workbench/parts/debug/node/debugConfigurationManager', 'vs/workbench/parts/debug/common/debugSource', 'vs/workbench/parts/tasks/common/taskService', 'vs/workbench/parts/tasks/common/taskSystem', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/panel/common/panelService', 'vs/workbench/services/part/common/partService', 'vs/workbench/parts/files/common/files', 'vs/workbench/services/workspace/common/contextService', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/window/electron-browser/windowService', 'vs/workbench/services/thread/electron-browser/threadService', 'electron'], function (require, exports, nls, lifecycle, mime, ee, uri_1, actions_1, arrays, types, errors, severity_1, winjs_base_1, aria, aiAdapter_1, keybindingService_1, markers_1, lifecycle_1, extensions_1, instantiation_1, files_1, event_1, message_1, telemetry_1, storage_1, wbeditorcommon, debug, session, model, debugEditorInputs_1, viewmodel, debugactions, breakpointWidget_1, debugConfigurationManager_1, debugSource_1, taskService_1, taskSystem_1, viewletService_1, panelService_1, partService_1, files_2, contextService_1, editorService_1, windowService_1, threadService_1, electron_1) {
    "use strict";
    var DEBUG_BREAKPOINTS_KEY = 'debug.breakpoint';
    var DEBUG_BREAKPOINTS_ACTIVATED_KEY = 'debug.breakpointactivated';
    var DEBUG_FUNCTION_BREAKPOINTS_KEY = 'debug.functionbreakpoint';
    var DEBUG_EXCEPTION_BREAKPOINTS_KEY = 'debug.exceptionbreakpoint';
    var DEBUG_WATCH_EXPRESSIONS_KEY = 'debug.watchexpressions';
    var DEBUG_SELECTED_CONFIG_NAME_KEY = 'debug.selectedconfigname';
    var DebugService = (function (_super) {
        __extends(DebugService, _super);
        function DebugService(storageService, editorService, textFileService, viewletService, panelService, fileService, messageService, partService, windowService, telemetryService, contextService, keybindingService, eventService, lifecycleService, instantiationService, extensionService, markerService) {
            _super.call(this);
            this.storageService = storageService;
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.viewletService = viewletService;
            this.panelService = panelService;
            this.fileService = fileService;
            this.messageService = messageService;
            this.partService = partService;
            this.windowService = windowService;
            this.telemetryService = telemetryService;
            this.contextService = contextService;
            this.lifecycleService = lifecycleService;
            this.instantiationService = instantiationService;
            this.extensionService = extensionService;
            this.markerService = markerService;
            this.serviceId = debug.IDebugService;
            this.toDispose = [];
            this.toDisposeOnSessionEnd = [];
            this.debugStringEditorInputs = [];
            this.session = null;
            this.state = debug.State.Inactive;
            // there is a cycle if taskService gets injected, use a workaround.
            this.taskService = this.instantiationService.getInstance(taskService_1.ITaskService);
            if (!this.contextService.getWorkspace()) {
                this.state = debug.State.Disabled;
            }
            this.configurationManager = this.instantiationService.createInstance(debugConfigurationManager_1.ConfigurationManager, this.storageService.get(DEBUG_SELECTED_CONFIG_NAME_KEY, storage_1.StorageScope.WORKSPACE, 'null'));
            this.inDebugMode = keybindingService.createKey(debug.CONTEXT_IN_DEBUG_MODE, false);
            this.model = new model.Model(this.loadBreakpoints(), this.storageService.getBoolean(DEBUG_BREAKPOINTS_ACTIVATED_KEY, storage_1.StorageScope.WORKSPACE, true), this.loadFunctionBreakpoints(), this.loadExceptionBreakpoints(), this.loadWatchExpressions());
            this.viewModel = new viewmodel.ViewModel();
            this.registerListeners(eventService, lifecycleService);
        }
        DebugService.prototype.registerListeners = function (eventService, lifecycleService) {
            var _this = this;
            this.toDispose.push(eventService.addListener2(files_1.EventType.FILE_CHANGES, function (e) { return _this.onFileChanges(e); }));
            if (this.taskService) {
                this.toDispose.push(this.taskService.addListener2(taskService_1.TaskServiceEvents.Active, function (e) {
                    _this.lastTaskEvent = e;
                }));
                this.toDispose.push(this.taskService.addListener2(taskService_1.TaskServiceEvents.Inactive, function (e) {
                    if (e.type === taskService_1.TaskType.SingleRun) {
                        _this.lastTaskEvent = null;
                    }
                }));
                this.toDispose.push(this.taskService.addListener2(taskService_1.TaskServiceEvents.Terminated, function (e) {
                    _this.lastTaskEvent = null;
                }));
            }
            lifecycleService.onShutdown(this.store, this);
            lifecycleService.onShutdown(this.dispose, this);
            this.windowService.onBroadcast(this.onBroadcast, this);
        };
        DebugService.prototype.onBroadcast = function (broadcast) {
            // attach: PH is ready to be attached to
            if (broadcast.channel === threadService_1.PLUGIN_ATTACH_BROADCAST_CHANNEL) {
                this.rawAttach(broadcast.payload.port);
                return;
            }
            // from this point on we require an active session
            var session = this.getActiveSession();
            if (!session || session.getType() !== 'extensionHost') {
                return; // we are only intersted if we have an active debug session for extensionHost
            }
            // a plugin logged output, show it inside the REPL
            if (broadcast.channel === threadService_1.PLUGIN_LOG_BROADCAST_CHANNEL) {
                var extensionOutput = broadcast.payload;
                var sev = extensionOutput.severity === 'warn' ? severity_1.default.Warning : extensionOutput.severity === 'error' ? severity_1.default.Error : severity_1.default.Info;
                var args = [];
                try {
                    var parsed_1 = JSON.parse(extensionOutput.arguments);
                    args.push.apply(args, Object.getOwnPropertyNames(parsed_1).map(function (o) { return parsed_1[o]; }));
                }
                catch (error) {
                    args.push(extensionOutput.arguments);
                }
                // add output for each argument logged
                var simpleVals = [];
                for (var i = 0; i < args.length; i++) {
                    var a = args[i];
                    // undefined gets printed as 'undefined'
                    if (typeof a === 'undefined') {
                        simpleVals.push('undefined');
                    }
                    else if (a === null) {
                        simpleVals.push('null');
                    }
                    else if (types.isObject(a) || Array.isArray(a)) {
                        // flush any existing simple values logged
                        if (simpleVals.length) {
                            this.logToRepl(simpleVals.join(' '), sev);
                            simpleVals = [];
                        }
                        // show object
                        this.logToRepl(a, sev);
                    }
                    else if (typeof a === 'string') {
                        var buf = '';
                        for (var j = 0, len = a.length; j < len; j++) {
                            if (a[j] === '%' && (a[j + 1] === 's' || a[j + 1] === 'i' || a[j + 1] === 'd')) {
                                i++; // read over substitution
                                buf += !types.isUndefinedOrNull(args[i]) ? args[i] : ''; // replace
                                j++; // read over directive
                            }
                            else {
                                buf += a[j];
                            }
                        }
                        simpleVals.push(buf);
                    }
                    else {
                        simpleVals.push(a);
                    }
                }
                // flush simple values
                if (simpleVals.length) {
                    this.logToRepl(simpleVals.join(' '), sev);
                }
            }
        };
        DebugService.prototype.registerSessionListeners = function () {
            var _this = this;
            this.toDisposeOnSessionEnd.push(this.session.addListener2(debug.SessionEvents.INITIALIZED, function (event) {
                aria.status(nls.localize(0, null));
                _this.sendAllBreakpoints().then(function () {
                    if (_this.session.capabilities.supportsConfigurationDoneRequest) {
                        _this.session.configurationDone().done(null, errors.onUnexpectedError);
                    }
                });
            }));
            this.toDisposeOnSessionEnd.push(this.session.addListener2(debug.SessionEvents.STOPPED, function (event) {
                _this.setStateAndEmit(debug.State.Stopped);
                var threadId = event.body.threadId;
                _this.getThreadData(threadId).done(function () {
                    var thread = _this.model.getThreads()[threadId];
                    _this.model.rawUpdate({
                        threadId: threadId,
                        stoppedDetails: event.body,
                        allThreadsStopped: event.body.allThreadsStopped
                    });
                    thread.getCallStack(_this).then(function (callStack) {
                        _this.windowService.getWindow().focus();
                        if (callStack.length > 0) {
                            // focus first stack frame from top that has source location
                            var stackFrameToFocus = arrays.first(callStack, function (sf) { return sf.source && sf.source.available; }, callStack[0]);
                            _this.setFocusedStackFrameAndEvaluate(stackFrameToFocus);
                            aria.alert(nls.localize(1, null, event.body.reason, stackFrameToFocus.source ? stackFrameToFocus.source.name : '', stackFrameToFocus.lineNumber));
                            return _this.openOrRevealEditor(stackFrameToFocus.source, stackFrameToFocus.lineNumber, false, false);
                        }
                        else {
                            _this.setFocusedStackFrameAndEvaluate(null);
                        }
                    });
                }, errors.onUnexpectedError);
            }));
            this.toDisposeOnSessionEnd.push(this.session.addListener2(debug.SessionEvents.CONTINUED, function () {
                aria.status(nls.localize(2, null));
                _this.model.continueThreads();
                _this.setFocusedStackFrameAndEvaluate(null);
                _this.setStateAndEmit(_this.configurationManager.getConfiguration().noDebug ? debug.State.RunningNoDebug : debug.State.Running);
            }));
            this.toDisposeOnSessionEnd.push(this.session.addListener2(debug.SessionEvents.THREAD, function (event) {
                if (event.body.reason === 'started') {
                    _this.session.threads().done(function (result) {
                        var thread = result.body.threads.filter(function (thread) { return thread.id === event.body.threadId; }).pop();
                        if (thread) {
                            _this.model.rawUpdate({
                                threadId: thread.id,
                                thread: thread
                            });
                        }
                    }, errors.onUnexpectedError);
                }
                else if (event.body.reason === 'exited') {
                    _this.model.clearThreads(true, event.body.threadId);
                }
            }));
            this.toDisposeOnSessionEnd.push(this.session.addListener2(debug.SessionEvents.DEBUGEE_TERMINATED, function (event) {
                aria.status(nls.localize(3, null));
                if (_this.session && _this.session.getId() === event.sessionId) {
                    if (event.body && typeof event.body.restart === 'boolean' && event.body.restart) {
                        _this.restartSession().done(null, function (err) { return _this.messageService.show(severity_1.default.Error, err.message); });
                    }
                    else {
                        _this.session.disconnect().done(null, errors.onUnexpectedError);
                    }
                }
            }));
            this.toDisposeOnSessionEnd.push(this.session.addListener2(debug.SessionEvents.OUTPUT, function (event) {
                if (event.body && event.body.category === 'telemetry') {
                    // only log telemetry events from debug adapter if the adapter provided the telemetry key
                    if (_this.telemetryAdapter) {
                        _this.telemetryAdapter.log(event.body.output, event.body.data);
                    }
                }
                else if (event.body && typeof event.body.output === 'string' && event.body.output.length > 0) {
                    _this.onOutput(event);
                }
            }));
            this.toDisposeOnSessionEnd.push(this.session.addListener2(debug.SessionEvents.BREAKPOINT, function (event) {
                var id = event.body && event.body.breakpoint ? event.body.breakpoint.id : undefined;
                var breakpoint = _this.model.getBreakpoints().filter(function (bp) { return bp.idFromAdapter === id; }).pop();
                if (breakpoint) {
                    _this.model.updateBreakpoints((_a = {}, _a[breakpoint.getId()] = event.body.breakpoint, _a));
                }
                else {
                    var functionBreakpoint = _this.model.getFunctionBreakpoints().filter(function (bp) { return bp.idFromAdapter === id; }).pop();
                    if (functionBreakpoint) {
                        _this.model.updateFunctionBreakpoints((_b = {}, _b[functionBreakpoint.getId()] = event.body.breakpoint, _b));
                    }
                }
                var _a, _b;
            }));
            this.toDisposeOnSessionEnd.push(this.session.addListener2(debug.SessionEvents.SERVER_EXIT, function (event) {
                // 'Run without debugging' mode VSCode must terminate the extension host. More details: #3905
                if (_this.session.getType() === 'extensionHost' && _this.state === debug.State.RunningNoDebug) {
                    electron_1.ipcRenderer.send('vscode:closeExtensionHostWindow', _this.contextService.getWorkspace().resource.fsPath);
                }
                if (_this.session && _this.session.getId() === event.sessionId) {
                    _this.onSessionEnd();
                }
            }));
        };
        DebugService.prototype.onOutput = function (event) {
            var outputSeverity = event.body.category === 'stderr' ? severity_1.default.Error : event.body.category === 'console' ? severity_1.default.Warning : severity_1.default.Info;
            this.appendReplOutput(event.body.output, outputSeverity);
        };
        DebugService.prototype.getThreadData = function (threadId) {
            var _this = this;
            return this.model.getThreads()[threadId] ? winjs_base_1.TPromise.as(undefined) :
                this.session.threads().then(function (response) {
                    var thread = response.body.threads.filter(function (t) { return t.id === threadId; }).pop();
                    if (!thread) {
                        throw new Error(nls.localize(4, null, threadId));
                    }
                    _this.model.rawUpdate({
                        threadId: thread.id,
                        thread: thread
                    });
                });
        };
        DebugService.prototype.loadBreakpoints = function () {
            try {
                return JSON.parse(this.storageService.get(DEBUG_BREAKPOINTS_KEY, storage_1.StorageScope.WORKSPACE, '[]')).map(function (breakpoint) {
                    return new model.Breakpoint(new debugSource_1.Source(breakpoint.source.raw ? breakpoint.source.raw : { path: uri_1.default.parse(breakpoint.source.uri).fsPath, name: breakpoint.source.name }), breakpoint.desiredLineNumber || breakpoint.lineNumber, breakpoint.enabled, breakpoint.condition);
                });
            }
            catch (e) {
                return [];
            }
        };
        DebugService.prototype.loadFunctionBreakpoints = function () {
            try {
                return JSON.parse(this.storageService.get(DEBUG_FUNCTION_BREAKPOINTS_KEY, storage_1.StorageScope.WORKSPACE, '[]')).map(function (fb) {
                    return new model.FunctionBreakpoint(fb.name, fb.enabled);
                });
            }
            catch (e) {
                return [];
            }
        };
        DebugService.prototype.loadExceptionBreakpoints = function () {
            var result = null;
            try {
                result = JSON.parse(this.storageService.get(DEBUG_EXCEPTION_BREAKPOINTS_KEY, storage_1.StorageScope.WORKSPACE, '[]')).map(function (exBreakpoint) {
                    return new model.ExceptionBreakpoint(exBreakpoint.filter || exBreakpoint.name, exBreakpoint.label, exBreakpoint.enabled);
                });
            }
            catch (e) {
                result = [];
            }
            return result;
        };
        DebugService.prototype.loadWatchExpressions = function () {
            try {
                return JSON.parse(this.storageService.get(DEBUG_WATCH_EXPRESSIONS_KEY, storage_1.StorageScope.WORKSPACE, '[]')).map(function (watch) {
                    return new model.Expression(watch.name, false, watch.id);
                });
            }
            catch (e) {
                return [];
            }
        };
        DebugService.prototype.getState = function () {
            return this.state;
        };
        DebugService.prototype.setStateAndEmit = function (newState) {
            this.state = newState;
            this.emit(debug.ServiceEvents.STATE_CHANGED);
        };
        Object.defineProperty(DebugService.prototype, "enabled", {
            get: function () {
                return !!this.contextService.getWorkspace();
            },
            enumerable: true,
            configurable: true
        });
        DebugService.prototype.setFocusedStackFrameAndEvaluate = function (focusedStackFrame) {
            this.viewModel.setFocusedStackFrame(focusedStackFrame);
            if (focusedStackFrame) {
                this.model.evaluateWatchExpressions(this.session, focusedStackFrame);
            }
            else {
                this.model.clearWatchExpressionValues();
            }
        };
        DebugService.prototype.setBreakpointsForModel = function (modelUri, rawData) {
            this.model.removeBreakpoints(this.model.getBreakpoints().filter(function (bp) { return bp.source.uri.toString() === modelUri.toString(); }));
            this.model.addBreakpoints(rawData);
        };
        DebugService.prototype.toggleBreakpoint = function (rawBreakpoint) {
            var breakpoint = this.model.getBreakpoints().filter(function (bp) { return bp.lineNumber === rawBreakpoint.lineNumber && bp.source.uri.toString() === rawBreakpoint.uri.toString(); }).pop();
            if (breakpoint) {
                this.model.removeBreakpoints([breakpoint]);
            }
            else {
                this.model.addBreakpoints([rawBreakpoint]);
            }
            return this.sendBreakpoints(rawBreakpoint.uri);
        };
        DebugService.prototype.enableOrDisableAllBreakpoints = function (enabled) {
            this.model.enableOrDisableAllBreakpoints(enabled);
            return this.sendAllBreakpoints();
        };
        DebugService.prototype.toggleEnablement = function (element) {
            this.model.toggleEnablement(element);
            if (element instanceof model.Breakpoint) {
                var breakpoint = element;
                return this.sendBreakpoints(breakpoint.source.uri);
            }
            else if (element instanceof model.FunctionBreakpoint) {
                return this.sendFunctionBreakpoints();
            }
            return this.sendExceptionBreakpoints();
        };
        DebugService.prototype.removeAllBreakpoints = function () {
            var _this = this;
            var urisToClear = arrays.distinct(this.model.getBreakpoints(), function (bp) { return bp.source.uri.toString(); }).map(function (bp) { return bp.source.uri; });
            this.model.removeBreakpoints(this.model.getBreakpoints());
            return winjs_base_1.TPromise.join(urisToClear.map(function (uri) { return _this.sendBreakpoints(uri); }));
        };
        DebugService.prototype.toggleBreakpointsActivated = function () {
            this.model.toggleBreakpointsActivated();
            return this.sendAllBreakpoints();
        };
        DebugService.prototype.editBreakpoint = function (editor, lineNumber) {
            if (breakpointWidget_1.BreakpointWidget.INSTANCE) {
                breakpointWidget_1.BreakpointWidget.INSTANCE.dispose();
            }
            this.instantiationService.createInstance(breakpointWidget_1.BreakpointWidget, editor, lineNumber);
            breakpointWidget_1.BreakpointWidget.INSTANCE.show({ lineNumber: lineNumber, column: 1 }, 2);
            return winjs_base_1.TPromise.as(null);
        };
        DebugService.prototype.addFunctionBreakpoint = function () {
            this.model.addFunctionBreakpoint('');
        };
        DebugService.prototype.renameFunctionBreakpoint = function (id, newFunctionName) {
            this.model.updateFunctionBreakpoints((_a = {}, _a[id] = { name: newFunctionName }, _a));
            return this.sendFunctionBreakpoints();
            var _a;
        };
        DebugService.prototype.removeFunctionBreakpoints = function (id) {
            this.model.removeFunctionBreakpoints(id);
            return this.sendFunctionBreakpoints();
        };
        DebugService.prototype.addReplExpression = function (name) {
            this.telemetryService.publicLog('debugService/addReplExpression');
            return this.model.addReplExpression(this.session, this.viewModel.getFocusedStackFrame(), name);
        };
        DebugService.prototype.logToRepl = function (value, severity) {
            this.model.logToRepl(value, severity);
        };
        DebugService.prototype.appendReplOutput = function (value, severity) {
            this.model.appendReplOutput(value, severity);
        };
        DebugService.prototype.clearReplExpressions = function () {
            this.model.clearReplExpressions();
        };
        DebugService.prototype.addWatchExpression = function (name) {
            return this.model.addWatchExpression(this.session, this.viewModel.getFocusedStackFrame(), name);
        };
        DebugService.prototype.renameWatchExpression = function (id, newName) {
            return this.model.renameWatchExpression(this.session, this.viewModel.getFocusedStackFrame(), id, newName);
        };
        DebugService.prototype.clearWatchExpressions = function (id) {
            this.model.clearWatchExpressions(id);
        };
        DebugService.prototype.createSession = function (noDebug, changeViewState) {
            var _this = this;
            if (changeViewState === void 0) { changeViewState = !this.partService.isSideBarHidden(); }
            this.clearReplExpressions();
            return this.textFileService.saveAll().then(function () { return _this.extensionService.onReady(); }).then(function () { return _this.configurationManager.setConfiguration(_this.configurationManager.getConfigurationName()); }).then(function () {
                var configuration = _this.configurationManager.getConfiguration();
                if (!configuration) {
                    return _this.configurationManager.openConfigFile(false).then(function (openend) {
                        if (openend) {
                            _this.messageService.show(severity_1.default.Info, nls.localize(5, null));
                        }
                    });
                }
                configuration.noDebug = noDebug;
                if (!_this.configurationManager.getAdapter()) {
                    _this.emit(debug.ServiceEvents.TYPE_NOT_SUPPORTED, configuration.type);
                    return configuration.type ? winjs_base_1.TPromise.wrapError(new Error(nls.localize(6, null, configuration.type)))
                        : winjs_base_1.TPromise.wrapError(errors.create(nls.localize(7, null), { actions: [message_1.CloseAction, _this.instantiationService.createInstance(debugactions.ConfigureAction, debugactions.ConfigureAction.ID, debugactions.ConfigureAction.LABEL)] }));
                }
                return _this.runPreLaunchTask(configuration.preLaunchTask).then(function (taskSummary) {
                    var errorCount = configuration.preLaunchTask ? _this.markerService.getStatistics().errors : 0;
                    var failureExitCode = taskSummary && taskSummary.exitCode !== undefined && taskSummary.exitCode !== 0;
                    if (errorCount === 0 && !failureExitCode) {
                        return _this.doCreateSession(configuration, changeViewState);
                    }
                    _this.messageService.show(severity_1.default.Error, {
                        message: errorCount > 1 ? nls.localize(8, null, configuration.preLaunchTask) :
                            errorCount === 1 ? nls.localize(9, null, configuration.preLaunchTask) :
                                nls.localize(10, null, configuration.preLaunchTask, taskSummary.exitCode),
                        actions: [message_1.CloseAction, new actions_1.Action('debug.continue', nls.localize(11, null), null, true, function () {
                                _this.messageService.hideAll();
                                return _this.doCreateSession(configuration, changeViewState);
                            })]
                    });
                }, function (err) {
                    if (err.code !== taskSystem_1.TaskErrors.NotConfigured) {
                        throw err;
                    }
                    _this.messageService.show(err.severity, {
                        message: err.message,
                        actions: [message_1.CloseAction, _this.taskService.configureAction()]
                    });
                });
            });
        };
        DebugService.prototype.doCreateSession = function (configuration, changeViewState) {
            var _this = this;
            var key = this.configurationManager.getAdapter().aiKey;
            var telemetryInfo = Object.create(null);
            this.telemetryService.getTelemetryInfo().then(function (info) {
                telemetryInfo['common.vscodemachineid'] = info.machineId;
                telemetryInfo['common.vscodesessionid'] = info.sessionId;
            }, errors.onUnexpectedError);
            this.telemetryAdapter = new aiAdapter_1.AIAdapter(key, this.configurationManager.getAdapter().type, null, telemetryInfo);
            this.session = new session.RawDebugSession(this.messageService, this.telemetryService, configuration.debugServer, this.configurationManager.getAdapter(), this.telemetryAdapter);
            this.registerSessionListeners();
            return this.session.initialize({
                adapterID: configuration.type,
                pathFormat: 'path',
                linesStartAt1: true,
                columnsStartAt1: true
            }).then(function (result) {
                if (!_this.session) {
                    return winjs_base_1.TPromise.wrapError(new Error(nls.localize(12, null)));
                }
                _this.setStateAndEmit(debug.State.Initializing);
                _this.model.setExceptionBreakpoints(_this.session.capabilities.exceptionBreakpointFilters);
                return configuration.request === 'attach' ? _this.session.attach(configuration) : _this.session.launch(configuration);
            }).then(function (result) {
                if (changeViewState) {
                    _this.viewletService.openViewlet(debug.VIEWLET_ID);
                    _this.revealRepl(false).done(undefined, errors.onUnexpectedError);
                }
                _this.partService.addClass('debugging');
                _this.contextService.updateOptions('editor', {
                    glyphMargin: true
                });
                _this.inDebugMode.set(true);
                _this.telemetryService.publicLog('debugSessionStart', { type: configuration.type, breakpointCount: _this.model.getBreakpoints().length, exceptionBreakpoints: _this.model.getExceptionBreakpoints(), watchExpressionsCount: _this.model.getWatchExpressions().length });
            }).then(undefined, function (error) {
                _this.telemetryService.publicLog('debugMisconfiguration', { type: configuration ? configuration.type : undefined });
                if (_this.session) {
                    _this.session.disconnect();
                }
                var configureAction = _this.instantiationService.createInstance(debugactions.ConfigureAction, debugactions.ConfigureAction.ID, debugactions.ConfigureAction.LABEL);
                var actions = (error.actions && error.actions.length) ? error.actions.concat([configureAction]) : [message_1.CloseAction, configureAction];
                return winjs_base_1.TPromise.wrapError(errors.create(error.message, { actions: actions }));
            });
        };
        DebugService.prototype.runPreLaunchTask = function (taskName) {
            var _this = this;
            if (!taskName) {
                return winjs_base_1.TPromise.as(null);
            }
            // run a task before starting a debug session
            return this.taskService.tasks().then(function (descriptions) {
                var filteredTasks = descriptions.filter(function (task) { return task.name === taskName; });
                if (filteredTasks.length !== 1) {
                    return winjs_base_1.TPromise.wrapError(errors.create(nls.localize(13, null, taskName), {
                        actions: [
                            message_1.CloseAction,
                            _this.taskService.configureAction(),
                            _this.instantiationService.createInstance(debugactions.ConfigureAction, debugactions.ConfigureAction.ID, debugactions.ConfigureAction.LABEL)
                        ]
                    }));
                }
                // task is already running - nothing to do.
                if (_this.lastTaskEvent && _this.lastTaskEvent.taskName === taskName) {
                    return winjs_base_1.TPromise.as(null);
                }
                if (_this.lastTaskEvent) {
                    // there is a different task running currently.
                    return winjs_base_1.TPromise.wrapError(errors.create(nls.localize(14, null, _this.lastTaskEvent.taskName, taskName)));
                }
                // no task running, execute the preLaunchTask.
                var taskPromise = _this.taskService.run(filteredTasks[0].id).then(function (result) {
                    _this.lastTaskEvent = null;
                    return result;
                }, function (err) {
                    _this.lastTaskEvent = null;
                });
                if (filteredTasks[0].isWatching) {
                    return new winjs_base_1.TPromise(function (c, e) { return _this.taskService.addOneTimeListener(taskService_1.TaskServiceEvents.Inactive, function () { return c(null); }); });
                }
                return taskPromise;
            });
        };
        DebugService.prototype.rawAttach = function (port) {
            if (this.session) {
                if (!this.session.isAttach) {
                    return this.session.attach({ port: port });
                }
                this.session.disconnect().done(null, errors.onUnexpectedError);
            }
            var configuration = this.configurationManager.getConfiguration();
            return this.doCreateSession({
                type: configuration.type,
                request: 'attach',
                port: port,
                sourceMaps: configuration.sourceMaps,
                outDir: configuration.outDir,
                debugServer: configuration.debugServer
            }, false);
        };
        DebugService.prototype.restartSession = function () {
            var _this = this;
            return this.session ? this.session.disconnect(true).then(function () {
                return new winjs_base_1.TPromise(function (c, e) {
                    setTimeout(function () {
                        _this.createSession(false, false).then(function () { return c(null); }, function (err) { return e(err); });
                    }, 300);
                });
            }) : this.createSession(false, false);
        };
        DebugService.prototype.getActiveSession = function () {
            return this.session;
        };
        DebugService.prototype.onSessionEnd = function () {
            try {
                this.debugStringEditorInputs = lifecycle.disposeAll(this.debugStringEditorInputs);
            }
            catch (e) {
            }
            if (this.session) {
                var bpsExist = this.model.getBreakpoints().length > 0;
                this.session.dispose();
                this.telemetryService.publicLog('debugSessionStop', {
                    type: this.session.getType(),
                    success: this.session.emittedStopped || !bpsExist,
                    sessionLengthInSeconds: this.session.getLengthInSeconds(),
                    breakpointCount: this.model.getBreakpoints().length,
                    watchExpressionsCount: this.model.getWatchExpressions().length
                });
            }
            this.session = null;
            this.toDisposeOnSessionEnd = lifecycle.disposeAll(this.toDisposeOnSessionEnd);
            this.partService.removeClass('debugging');
            this.editorService.focusEditor();
            this.model.clearThreads(true);
            this.setFocusedStackFrameAndEvaluate(null);
            this.setStateAndEmit(debug.State.Inactive);
            // set breakpoints back to unverified since the session ended.
            // source reference changes across sessions, so we do not use it to persist the source.
            var data = {};
            this.model.getBreakpoints().forEach(function (bp) {
                delete bp.source.raw.sourceReference;
                data[bp.getId()] = { line: bp.lineNumber, verified: false };
            });
            this.model.updateBreakpoints(data);
            if (this.telemetryAdapter) {
                this.telemetryAdapter.dispose();
                this.telemetryAdapter = null;
            }
            this.inDebugMode.reset();
        };
        DebugService.prototype.getModel = function () {
            return this.model;
        };
        DebugService.prototype.getViewModel = function () {
            return this.viewModel;
        };
        DebugService.prototype.openOrRevealEditor = function (source, lineNumber, preserveFocus, sideBySide) {
            var _this = this;
            var visibleEditors = this.editorService.getVisibleEditors();
            for (var i = 0; i < visibleEditors.length; i++) {
                var fileInput = wbeditorcommon.asFileEditorInput(visibleEditors[i].input);
                if ((fileInput && fileInput.getResource().toString() === source.uri.toString()) ||
                    (visibleEditors[i].input instanceof debugEditorInputs_1.DebugStringEditorInput && visibleEditors[i].input.getResource().toString() === source.uri.toString())) {
                    var control = visibleEditors[i].getControl();
                    if (control) {
                        control.revealLineInCenterIfOutsideViewport(lineNumber);
                        control.setSelection({ startLineNumber: lineNumber, startColumn: 1, endLineNumber: lineNumber, endColumn: 1 });
                        return this.editorService.openEditor(visibleEditors[i].input, wbeditorcommon.TextEditorOptions.create({ preserveFocus: preserveFocus, forceActive: true }), visibleEditors[i].position);
                    }
                    return winjs_base_1.TPromise.as(null);
                }
            }
            if (source.inMemory) {
                // internal module
                if (source.reference !== 0 && this.session) {
                    return this.session.source({ sourceReference: source.reference }).then(function (response) {
                        var editorInput = _this.getDebugStringEditorInput(source, response.body.content, mime.guessMimeTypes(source.name)[0]);
                        return _this.editorService.openEditor(editorInput, wbeditorcommon.TextEditorOptions.create({
                            selection: {
                                startLineNumber: lineNumber,
                                startColumn: 1,
                                endLineNumber: lineNumber,
                                endColumn: 1
                            },
                            preserveFocus: preserveFocus
                        }), sideBySide);
                    });
                }
                return this.sourceIsUnavailable(source, sideBySide);
            }
            return this.fileService.resolveFile(source.uri).then(function () {
                return _this.editorService.openEditor({
                    resource: source.uri,
                    options: {
                        selection: {
                            startLineNumber: lineNumber,
                            startColumn: 1,
                            endLineNumber: lineNumber,
                            endColumn: 1
                        },
                        preserveFocus: preserveFocus
                    }
                }, sideBySide);
            }, function (err) { return _this.sourceIsUnavailable(source, sideBySide); });
        };
        DebugService.prototype.sourceIsUnavailable = function (source, sideBySide) {
            this.model.sourceIsUnavailable(source);
            var editorInput = this.getDebugStringEditorInput(source, nls.localize(15, null, source.uri.fsPath), 'text/plain');
            return this.editorService.openEditor(editorInput, wbeditorcommon.TextEditorOptions.create({ preserveFocus: true }), sideBySide);
        };
        DebugService.prototype.revealRepl = function (focus) {
            var _this = this;
            if (focus === void 0) { focus = true; }
            return this.panelService.openPanel(debug.REPL_ID, focus).then(function (repl) {
                var elements = _this.model.getReplElements();
                if (elements.length > 0) {
                    return repl.reveal(elements[elements.length - 1]);
                }
            });
        };
        DebugService.prototype.canSetBreakpointsIn = function (model) {
            return this.configurationManager.canSetBreakpointsIn(model);
        };
        DebugService.prototype.getConfigurationName = function () {
            return this.configurationManager.getConfigurationName();
        };
        DebugService.prototype.setConfiguration = function (name) {
            return this.configurationManager.setConfiguration(name);
        };
        DebugService.prototype.openConfigFile = function (sideBySide) {
            return this.configurationManager.openConfigFile(sideBySide);
        };
        DebugService.prototype.loadLaunchConfig = function () {
            return this.configurationManager.loadLaunchConfig();
        };
        DebugService.prototype.getDebugStringEditorInput = function (source, value, mtype) {
            var filtered = this.debugStringEditorInputs.filter(function (input) { return input.getResource().toString() === source.uri.toString(); });
            if (filtered.length === 0) {
                var result = this.instantiationService.createInstance(debugEditorInputs_1.DebugStringEditorInput, source.name, source.uri, source.origin, value, mtype, void 0);
                this.debugStringEditorInputs.push(result);
                return result;
            }
            else {
                return filtered[0];
            }
        };
        DebugService.prototype.sendAllBreakpoints = function () {
            var _this = this;
            return winjs_base_1.TPromise.join(arrays.distinct(this.model.getBreakpoints(), function (bp) { return bp.source.uri.toString(); }).map(function (bp) { return _this.sendBreakpoints(bp.source.uri); }))
                .then(function () { return _this.sendFunctionBreakpoints(); })
                .then(function () { return _this.sendExceptionBreakpoints(); });
        };
        DebugService.prototype.sendBreakpoints = function (modelUri) {
            var _this = this;
            if (!this.session || !this.session.readyForBreakpoints) {
                return winjs_base_1.TPromise.as(null);
            }
            var breakpointsToSend = arrays.distinct(this.model.getBreakpoints().filter(function (bp) { return _this.model.areBreakpointsActivated() && bp.enabled && bp.source.uri.toString() === modelUri.toString(); }), function (bp) { return ("" + bp.desiredLineNumber); });
            var rawSource = breakpointsToSend.length > 0 ? breakpointsToSend[0].source.raw : debugSource_1.Source.toRawSource(modelUri, null);
            return this.session.setBreakpoints({ source: rawSource, lines: breakpointsToSend.map(function (bp) { return bp.desiredLineNumber; }),
                breakpoints: breakpointsToSend.map(function (bp) { return ({ line: bp.desiredLineNumber, condition: bp.condition }); }) }).then(function (response) {
                var data = {};
                for (var i = 0; i < breakpointsToSend.length; i++) {
                    data[breakpointsToSend[i].getId()] = response.body.breakpoints[i];
                }
                _this.model.updateBreakpoints(data);
            });
        };
        DebugService.prototype.sendFunctionBreakpoints = function () {
            var _this = this;
            if (!this.session || !this.session.readyForBreakpoints || !this.session.capabilities.supportsFunctionBreakpoints) {
                return winjs_base_1.TPromise.as(null);
            }
            var breakpointsToSend = this.model.getFunctionBreakpoints().filter(function (fbp) { return fbp.enabled; });
            return this.session.setFunctionBreakpoints({ breakpoints: breakpointsToSend }).then(function (response) {
                var data = {};
                for (var i = 0; i < breakpointsToSend.length; i++) {
                    data[breakpointsToSend[i].getId()] = response.body.breakpoints[i];
                }
                _this.model.updateFunctionBreakpoints(data);
            });
        };
        DebugService.prototype.sendExceptionBreakpoints = function () {
            if (!this.session || !this.session.readyForBreakpoints || this.model.getExceptionBreakpoints().length === 0) {
                return winjs_base_1.TPromise.as(null);
            }
            var enabledExceptionBps = this.model.getExceptionBreakpoints().filter(function (exb) { return exb.enabled; });
            return this.session.setExceptionBreakpoints({ filters: enabledExceptionBps.map(function (exb) { return exb.filter; }) });
        };
        DebugService.prototype.onFileChanges = function (fileChangesEvent) {
            this.model.removeBreakpoints(this.model.getBreakpoints().filter(function (bp) {
                return fileChangesEvent.contains(bp.source.uri, files_1.FileChangeType.DELETED);
            }));
        };
        DebugService.prototype.store = function () {
            this.storageService.store(DEBUG_BREAKPOINTS_KEY, JSON.stringify(this.model.getBreakpoints()), storage_1.StorageScope.WORKSPACE);
            this.storageService.store(DEBUG_BREAKPOINTS_ACTIVATED_KEY, this.model.areBreakpointsActivated() ? 'true' : 'false', storage_1.StorageScope.WORKSPACE);
            this.storageService.store(DEBUG_FUNCTION_BREAKPOINTS_KEY, JSON.stringify(this.model.getFunctionBreakpoints()), storage_1.StorageScope.WORKSPACE);
            this.storageService.store(DEBUG_EXCEPTION_BREAKPOINTS_KEY, JSON.stringify(this.model.getExceptionBreakpoints()), storage_1.StorageScope.WORKSPACE);
            this.storageService.store(DEBUG_SELECTED_CONFIG_NAME_KEY, this.configurationManager.getConfigurationName(), storage_1.StorageScope.WORKSPACE);
            this.storageService.store(DEBUG_WATCH_EXPRESSIONS_KEY, JSON.stringify(this.model.getWatchExpressions()), storage_1.StorageScope.WORKSPACE);
        };
        DebugService.prototype.dispose = function () {
            if (this.session) {
                this.session.disconnect();
                this.session = null;
            }
            this.model.dispose();
            this.toDispose = lifecycle.disposeAll(this.toDispose);
            this.toDisposeOnSessionEnd = lifecycle.disposeAll(this.toDisposeOnSessionEnd);
        };
        DebugService = __decorate([
            __param(0, storage_1.IStorageService),
            __param(1, editorService_1.IWorkbenchEditorService),
            __param(2, files_2.ITextFileService),
            __param(3, viewletService_1.IViewletService),
            __param(4, panelService_1.IPanelService),
            __param(5, files_1.IFileService),
            __param(6, message_1.IMessageService),
            __param(7, partService_1.IPartService),
            __param(8, windowService_1.IWindowService),
            __param(9, telemetry_1.ITelemetryService),
            __param(10, contextService_1.IWorkspaceContextService),
            __param(11, keybindingService_1.IKeybindingService),
            __param(12, event_1.IEventService),
            __param(13, lifecycle_1.ILifecycleService),
            __param(14, instantiation_1.IInstantiationService),
            __param(15, extensions_1.IExtensionService),
            __param(16, markers_1.IMarkerService)
        ], DebugService);
        return DebugService;
    }(ee.EventEmitter));
    exports.DebugService = DebugService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/debug/electron-browser/debug.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/debug/electron-browser/debug.contribution', 'vs/base/common/keyCodes', 'vs/editor/common/editorCommon', 'vs/editor/common/editorCommonExtensions', 'vs/platform/actions/common/actions', 'vs/platform/platform', 'vs/platform/instantiation/common/extensions', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/platform/keybinding/common/keybindingService', 'vs/editor/browser/editorBrowserExtensions', 'vs/workbench/common/actionRegistry', 'vs/workbench/browser/viewlet', 'vs/workbench/browser/panel', 'vs/workbench/common/contributions', 'vs/workbench/parts/debug/common/debug', 'vs/workbench/parts/debug/browser/debugEditorModelManager', 'vs/workbench/parts/debug/electron-browser/debugActions', 'vs/workbench/parts/debug/browser/debugActionsWidget', 'vs/workbench/parts/debug/electron-browser/debugService', 'vs/workbench/parts/debug/browser/debugEditorContribution', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/editor/common/editorService', 'vs/css!../browser/media/debug.contribution', 'vs/css!../browser/media/debugHover'], function (require, exports, nls, keyCodes_1, editorcommon, editorCommonExtensions_1, actions_1, platform, extensions_1, keybindingsRegistry_1, keybindingService_1, editorBrowserExtensions_1, wbaregistry, viewlet, panel, wbext, debug, debugEditorModelManager_1, dbgactions, debugwidget, service, debugEditorContribution_1, viewletService_1, editorService_1) {
    "use strict";
    var IDebugService = debug.IDebugService;
    var OpenDebugViewletAction = (function (_super) {
        __extends(OpenDebugViewletAction, _super);
        function OpenDebugViewletAction(id, label, viewletService, editorService) {
            _super.call(this, id, label, debug.VIEWLET_ID, viewletService, editorService);
        }
        OpenDebugViewletAction.ID = debug.VIEWLET_ID;
        OpenDebugViewletAction.LABEL = nls.localize(0, null);
        OpenDebugViewletAction = __decorate([
            __param(2, viewletService_1.IViewletService),
            __param(3, editorService_1.IWorkbenchEditorService)
        ], OpenDebugViewletAction);
        return OpenDebugViewletAction;
    }(viewlet.ToggleViewletAction));
    editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(debugEditorContribution_1.DebugEditorContribution);
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(dbgactions.ToggleBreakpointAction, dbgactions.ToggleBreakpointAction.ID, nls.localize(1, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        primary: keyCodes_1.KeyCode.F9
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(dbgactions.ShowDebugHoverAction, dbgactions.ShowDebugHoverAction.ID, nls.localize(2, null), {
        context: editorCommonExtensions_1.ContextKey.EditorTextFocus,
        kbExpr: keybindingService_1.KbExpr.and(keybindingService_1.KbExpr.has(debug.CONTEXT_IN_DEBUG_MODE), keybindingService_1.KbExpr.has(editorcommon.KEYBINDING_CONTEXT_EDITOR_TEXT_FOCUS)),
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_I)
    }));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(dbgactions.EditorConditionalBreakpointAction, dbgactions.EditorConditionalBreakpointAction.ID, nls.localize(3, null)));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(dbgactions.SelectionToReplAction, dbgactions.SelectionToReplAction.ID, nls.localize(4, null)));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(dbgactions.SelectionToWatchExpressionsAction, dbgactions.SelectionToWatchExpressionsAction.ID, nls.localize(5, null)));
    editorCommonExtensions_1.CommonEditorRegistry.registerEditorAction(new editorCommonExtensions_1.EditorActionDescriptor(dbgactions.RunToCursorAction, dbgactions.RunToCursorAction.ID, nls.localize(6, null)));
    // register viewlet
    platform.Registry.as(viewlet.Extensions.Viewlets).registerViewlet(new viewlet.ViewletDescriptor('vs/workbench/parts/debug/browser/debugViewlet', 'DebugViewlet', debug.VIEWLET_ID, nls.localize(7, null), 'debug', 40));
    var openViewletKb = {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_D
    };
    // register repl panel
    platform.Registry.as(panel.Extensions.Panels).registerPanel(new panel.PanelDescriptor('vs/workbench/parts/debug/browser/repl', 'Repl', debug.REPL_ID, nls.localize(8, null), 'repl'));
    platform.Registry.as(panel.Extensions.Panels).setDefaultPanelId(debug.REPL_ID);
    // register action to open viewlet
    var registry = platform.Registry.as(wbaregistry.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(OpenDebugViewletAction, OpenDebugViewletAction.ID, OpenDebugViewletAction.LABEL, openViewletKb), nls.localize(9, null));
    platform.Registry.as(wbext.Extensions.Workbench).registerWorkbenchContribution(debugEditorModelManager_1.DebugEditorModelManager);
    platform.Registry.as(wbext.Extensions.Workbench).registerWorkbenchContribution(debugwidget.DebugActionsWidget);
    var debugCategory = nls.localize(10, null);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.StartDebugAction, dbgactions.StartDebugAction.ID, dbgactions.StartDebugAction.LABEL, { primary: keyCodes_1.KeyCode.F5 }, keybindingService_1.KbExpr.not(debug.CONTEXT_IN_DEBUG_MODE)), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.StepOverDebugAction, dbgactions.StepOverDebugAction.ID, dbgactions.StepOverDebugAction.LABEL, { primary: keyCodes_1.KeyCode.F10 }, keybindingService_1.KbExpr.has(debug.CONTEXT_IN_DEBUG_MODE)), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.StepIntoDebugAction, dbgactions.StepIntoDebugAction.ID, dbgactions.StepIntoDebugAction.LABEL, { primary: keyCodes_1.KeyCode.F11 }, keybindingService_1.KbExpr.has(debug.CONTEXT_IN_DEBUG_MODE), keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.workbenchContrib(1)), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.StepOutDebugAction, dbgactions.StepOutDebugAction.ID, dbgactions.StepOutDebugAction.LABEL, { primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.F11 }, keybindingService_1.KbExpr.has(debug.CONTEXT_IN_DEBUG_MODE)), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.RestartDebugAction, dbgactions.RestartDebugAction.ID, dbgactions.RestartDebugAction.LABEL, { primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.F5 }, keybindingService_1.KbExpr.has(debug.CONTEXT_IN_DEBUG_MODE)), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.StopDebugAction, dbgactions.StopDebugAction.ID, dbgactions.StopDebugAction.LABEL, { primary: keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.F5 }, keybindingService_1.KbExpr.has(debug.CONTEXT_IN_DEBUG_MODE)), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.ContinueAction, dbgactions.ContinueAction.ID, dbgactions.ContinueAction.LABEL, { primary: keyCodes_1.KeyCode.F5 }, keybindingService_1.KbExpr.has(debug.CONTEXT_IN_DEBUG_MODE)), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.PauseAction, dbgactions.PauseAction.ID, dbgactions.PauseAction.LABEL), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.ConfigureAction, dbgactions.ConfigureAction.ID, dbgactions.ConfigureAction.LABEL), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.ToggleReplAction, dbgactions.ToggleReplAction.ID, dbgactions.ToggleReplAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_Y, }), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.AddFunctionBreakpointAction, dbgactions.AddFunctionBreakpointAction.ID, dbgactions.AddFunctionBreakpointAction.LABEL), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.ReapplyBreakpointsAction, dbgactions.ReapplyBreakpointsAction.ID, dbgactions.ReapplyBreakpointsAction.LABEL), debugCategory);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(dbgactions.RunAction, dbgactions.RunAction.ID, dbgactions.RunAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.F5 }, keybindingService_1.KbExpr.not(debug.CONTEXT_IN_DEBUG_MODE)), debugCategory);
    // register service
    extensions_1.registerSingleton(IDebugService, service.DebugService);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/execution/electron-browser/terminal.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/execution/electron-browser/terminal.contribution', 'vs/base/common/winjs.base', 'vs/platform/platform', 'vs/base/common/platform', 'vs/base/common/actions', 'vs/workbench/common/actionRegistry', 'vs/base/common/paths', 'vs/workbench/browser/actionBarRegistry', 'vs/base/common/uri', 'vs/workbench/parts/files/common/files', 'vs/workbench/services/workspace/common/contextService', 'vs/workbench/parts/execution/common/execution', 'vs/platform/actions/common/actions', 'vs/platform/instantiation/common/instantiation', 'vs/base/common/keyCodes'], function (require, exports, nls, winjs_base_1, platform_1, baseplatform, actions_1, actionRegistry_1, paths, actionBarRegistry_1, uri_1, files_1, contextService_1, execution_1, actions_2, instantiation_1, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var OpenConsoleAction = (function (_super) {
        __extends(OpenConsoleAction, _super);
        function OpenConsoleAction(id, label, terminalService, contextService) {
            _super.call(this, id, label);
            this.terminalService = terminalService;
            this.contextService = contextService;
            this.order = 49; // Allow other actions to position before or after
        }
        OpenConsoleAction.prototype.setResource = function (resource) {
            this.resource = resource;
            this.enabled = !paths.isUNC(this.resource.fsPath);
        };
        OpenConsoleAction.prototype.run = function (event) {
            var workspace = this.contextService.getWorkspace();
            var path = this.resource ? this.resource.fsPath : (workspace && workspace.resource.fsPath);
            if (!path) {
                return winjs_base_1.TPromise.as(null);
            }
            this.terminalService.openTerminal(path);
            return winjs_base_1.TPromise.as(null);
        };
        OpenConsoleAction.ID = 'workbench.action.terminal.openNativeConsole';
        OpenConsoleAction.Label = baseplatform.isWindows ? nls.localize(0, null) :
            nls.localize(1, null);
        OpenConsoleAction.ScopedLabel = baseplatform.isWindows ? nls.localize(2, null) :
            nls.localize(3, null);
        OpenConsoleAction = __decorate([
            __param(2, execution_1.ITerminalService),
            __param(3, contextService_1.IWorkspaceContextService)
        ], OpenConsoleAction);
        return OpenConsoleAction;
    }(actions_1.Action));
    exports.OpenConsoleAction = OpenConsoleAction;
    var FileViewerActionContributor = (function (_super) {
        __extends(FileViewerActionContributor, _super);
        function FileViewerActionContributor(instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
        }
        FileViewerActionContributor.prototype.hasSecondaryActions = function (context) {
            return !!files_1.asFileResource(context.element);
        };
        FileViewerActionContributor.prototype.getSecondaryActions = function (context) {
            var fileResource = files_1.asFileResource(context.element);
            var resource = fileResource.resource;
            if (!fileResource.isDirectory) {
                resource = uri_1.default.file(paths.dirname(resource.fsPath));
            }
            var action = this.instantiationService.createInstance(OpenConsoleAction, OpenConsoleAction.ID, OpenConsoleAction.ScopedLabel);
            action.setResource(resource);
            return [action];
        };
        FileViewerActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], FileViewerActionContributor);
        return FileViewerActionContributor;
    }(actionBarRegistry_1.ActionBarContributor));
    var actionBarRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
    actionBarRegistry.registerActionBarContributor(actionBarRegistry_1.Scope.VIEWER, FileViewerActionContributor);
    // Register Global Action to Open Console
    platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions).registerWorkbenchAction(new actions_2.SyncActionDescriptor(OpenConsoleAction, OpenConsoleAction.ID, OpenConsoleAction.Label, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_C }));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/extensions/common/vsoGalleryService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/workbench/parts/extensions/common/extensions', 'vs/base/common/objects', 'vs/platform/request/common/request', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/telemetry/common/telemetry'], function (require, exports, winjs_base_1, extensions_1, objects_1, request_1, contextService_1, telemetry_1) {
    "use strict";
    function getInstallCount(statistics) {
        if (!statistics) {
            return 0;
        }
        var result = statistics.filter(function (s) { return s.statisticName === 'install'; })[0];
        return result ? result.value : 0;
    }
    var FIVE_MINUTES = 1000 * 60 * 5;
    var GalleryService = (function () {
        function GalleryService(requestService, contextService, telemetryService) {
            this.requestService = requestService;
            this.serviceId = extensions_1.IGalleryService;
            var config = contextService.getConfiguration().env.extensionsGallery;
            this.extensionsGalleryUrl = config && config.serviceUrl;
            this.extensionsCacheUrl = config && config.cacheUrl;
            this.machineId = telemetryService.getTelemetryInfo().then(function (_a) {
                var machineId = _a.machineId;
                return machineId;
            });
        }
        GalleryService.prototype.api = function (path) {
            if (path === void 0) { path = ''; }
            return "" + this.extensionsGalleryUrl + path;
        };
        GalleryService.prototype.isEnabled = function () {
            return !!this.extensionsGalleryUrl;
        };
        GalleryService.prototype.query = function () {
            var _this = this;
            if (!this.isEnabled()) {
                return winjs_base_1.TPromise.wrapError(new Error('No extension gallery service configured.'));
            }
            var raw = this.queryCache()
                .then(null, function (err) { return _this.queryGallery(); })
                .then(function (result) {
                var rawLastModified = result.getResponseHeader('last-modified');
                if (!rawLastModified) {
                    return _this.queryGallery();
                }
                var lastModified = new Date(rawLastModified).getTime();
                var now = new Date().getTime();
                var diff = now - lastModified;
                if (diff > FIVE_MINUTES) {
                    return _this.queryGallery();
                }
                return winjs_base_1.TPromise.as(result);
            });
            return raw
                .then(function (r) { return JSON.parse(r.responseText).results[0].extensions || []; })
                .then(function (extensions) {
                return _this.getRequestHeaders().then(function (downloadHeaders) {
                    return extensions.map(function (e) {
                        var versions = e.versions.map(function (v) { return ({
                            version: v.version,
                            date: v.lastUpdated,
                            downloadHeaders: downloadHeaders,
                            downloadUrl: v.assetUri + "/Microsoft.VisualStudio.Services.VSIXPackage?install=true",
                            manifestUrl: v.assetUri + "/Microsoft.VisualStudio.Code.Manifest"
                        }); });
                        return {
                            name: e.extensionName,
                            displayName: e.displayName || e.extensionName,
                            publisher: e.publisher.publisherName,
                            version: versions[0].version,
                            engines: { vscode: void 0 },
                            description: e.shortDescription || '',
                            galleryInformation: {
                                galleryApiUrl: _this.extensionsGalleryUrl,
                                id: e.extensionId,
                                publisherId: e.publisher.publisherId,
                                publisherDisplayName: e.publisher.displayName,
                                installCount: getInstallCount(e.statistics),
                                versions: versions
                            }
                        };
                    });
                });
            });
        };
        GalleryService.prototype.queryCache = function () {
            var url = this.extensionsCacheUrl;
            if (!url) {
                return winjs_base_1.TPromise.wrapError(new Error('No cache configured.'));
            }
            return this.requestService.makeRequest({ url: url });
        };
        GalleryService.prototype.queryGallery = function () {
            var _this = this;
            var data = JSON.stringify({
                filters: [{
                        criteria: [{
                                filterType: 1,
                                value: 'vscode'
                            }]
                    }],
                flags: 0x1 | 0x4 | 0x80 | 0x100
            });
            return this.getRequestHeaders().then(function (headers) {
                headers = objects_1.assign(headers, {
                    'Content-Type': 'application/json',
                    'Accept': 'application/json;api-version=3.0-preview.1',
                    'Content-Length': data.length
                });
                var request = {
                    type: 'POST',
                    url: _this.api('/extensionquery'),
                    data: data,
                    headers: headers
                };
                return _this.requestService.makeRequest(request);
            });
        };
        GalleryService.prototype.getRequestHeaders = function () {
            return this.machineId.then(function (machineId) {
                var result = {
                    'X-Market-Client-Id': 'VSCode'
                };
                if (machineId) {
                    result['X-Market-User-Id'] = machineId;
                }
                return result;
            });
        };
        GalleryService = __decorate([
            __param(0, request_1.IRequestService),
            __param(1, contextService_1.IWorkspaceContextService),
            __param(2, telemetry_1.ITelemetryService)
        ], GalleryService);
        return GalleryService;
    }());
    exports.GalleryService = GalleryService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/extensions/electron-browser/extensionsQuickOpen", ["require", "exports", 'vs/nls!vs/workbench/parts/extensions/electron-browser/extensionsQuickOpen', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/base/common/types', 'vs/base/browser/dom', 'vs/base/common/severity', 'vs/base/common/errors', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/common/filters', 'vs/workbench/browser/quickopen', 'vs/workbench/parts/extensions/common/extensions', 'vs/workbench/parts/extensions/electron-browser/extensionsActions', 'vs/platform/message/common/message', 'vs/platform/telemetry/common/telemetry', 'vs/platform/instantiation/common/instantiation', 'vs/workbench/services/workspace/common/contextService', 'vs/base/browser/ui/highlightedlabel/highlightedLabel', 'vs/base/common/actions', 'vs/base/browser/ui/actionbar/actionbar', 'electron', 'vs/workbench/parts/extensions/common/extensionsUtil'], function (require, exports, nls, lifecycle_1, winjs_base_1, types_1, dom, severity_1, errors_1, quickOpen_1, filters_1, quickopen_1, extensions_1, extensionsActions_1, message_1, telemetry_1, instantiation_1, contextService_1, highlightedLabel_1, actions_1, actionbar_1, electron_1, extensionsUtil_1) {
    "use strict";
    var $ = dom.emmet;
    var InstallLabel = nls.localize(0, null);
    var UpdateLabel = nls.localize(1, null);
    (function (ExtensionState) {
        ExtensionState[ExtensionState["Uninstalled"] = 0] = "Uninstalled";
        ExtensionState[ExtensionState["Installed"] = 1] = "Installed";
        ExtensionState[ExtensionState["Outdated"] = 2] = "Outdated";
    })(exports.ExtensionState || (exports.ExtensionState = {}));
    var ExtensionState = exports.ExtensionState;
    function getHighlights(input, extension) {
        var id = filters_1.matchesContiguousSubString(input, extension.publisher + "." + extension.name) || [];
        var name = filters_1.matchesContiguousSubString(input, extension.name) || [];
        var displayName = filters_1.matchesContiguousSubString(input, extension.displayName) || [];
        var description = filters_1.matchesContiguousSubString(input, extension.description) || [];
        if (!id.length && !name.length && !displayName.length && !description.length) {
            return null;
        }
        return { id: id, name: name, displayName: displayName, description: description };
    }
    function extensionEntryCompare(one, other) {
        var oneInstallCount = one.extension.galleryInformation ? one.extension.galleryInformation.installCount : 0;
        var otherInstallCount = other.extension.galleryInformation ? other.extension.galleryInformation.installCount : 0;
        var diff = otherInstallCount - oneInstallCount;
        if (diff !== 0) {
            return diff;
        }
        return one.extension.displayName.localeCompare(other.extension.displayName);
    }
    var OpenLicenseAction = (function (_super) {
        __extends(OpenLicenseAction, _super);
        function OpenLicenseAction(contextService) {
            _super.call(this, 'extensions.open-license', nls.localize(2, null), '', true);
            this.contextService = contextService;
        }
        OpenLicenseAction.prototype.run = function (extension) {
            var url = this.contextService.getConfiguration().env.extensionsGallery.itemUrl + "/" + extension.publisher + "." + extension.name + "/license";
            electron_1.shell.openExternal(url);
            return winjs_base_1.TPromise.as(null);
        };
        OpenLicenseAction = __decorate([
            __param(0, contextService_1.IWorkspaceContextService)
        ], OpenLicenseAction);
        return OpenLicenseAction;
    }(actions_1.Action));
    var OpenInGalleryAction = (function (_super) {
        __extends(OpenInGalleryAction, _super);
        function OpenInGalleryAction(promptToInstall, messageService, contextService, instantiationService) {
            _super.call(this, 'extensions.open-in-gallery', nls.localize(3, null), '', true);
            this.promptToInstall = promptToInstall;
            this.messageService = messageService;
            this.contextService = contextService;
            this.instantiationService = instantiationService;
        }
        OpenInGalleryAction.prototype.run = function (extension) {
            var _this = this;
            var url = this.contextService.getConfiguration().env.extensionsGallery.itemUrl + "/" + extension.publisher + "." + extension.name;
            electron_1.shell.openExternal(url);
            if (!this.promptToInstall) {
                return winjs_base_1.TPromise.as(null);
            }
            var hideMessage = this.messageService.show(severity_1.default.Info, {
                message: nls.localize(4, null, extension.displayName),
                actions: [
                    new actions_1.Action('cancelaction', nls.localize(5, null)),
                    new actions_1.Action('installNow', nls.localize(6, null), null, true, function () {
                        hideMessage();
                        var hideInstallMessage = _this.messageService.show(severity_1.default.Info, nls.localize(7, null, extension.displayName));
                        var action = _this.instantiationService.createInstance(extensionsActions_1.InstallAction, '');
                        return action.run(extension).then(function (r) {
                            hideInstallMessage();
                            return winjs_base_1.TPromise.as(r);
                        }, function (e) {
                            hideInstallMessage();
                            return winjs_base_1.TPromise.wrapError(e);
                        });
                    })
                ]
            });
            return winjs_base_1.TPromise.as(null);
        };
        OpenInGalleryAction = __decorate([
            __param(1, message_1.IMessageService),
            __param(2, contextService_1.IWorkspaceContextService),
            __param(3, instantiation_1.IInstantiationService)
        ], OpenInGalleryAction);
        return OpenInGalleryAction;
    }(actions_1.Action));
    var InstallRunner = (function () {
        function InstallRunner(instantiationService) {
            this.instantiationService = instantiationService;
        }
        InstallRunner.prototype.run = function (entry, mode, context) {
            if (mode === quickOpen_1.Mode.PREVIEW) {
                return false;
            }
            if (entry.state === ExtensionState.Installed) {
                return false;
            }
            if (!this.action) {
                this.action = this.instantiationService.createInstance(extensionsActions_1.InstallAction, InstallLabel);
            }
            this.action.run(entry.extension).done(null, errors_1.onUnexpectedError);
            return true;
        };
        InstallRunner = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], InstallRunner);
        return InstallRunner;
    }());
    var AccessibilityProvider = (function () {
        function AccessibilityProvider() {
        }
        AccessibilityProvider.prototype.getAriaLabel = function (entry) {
            return nls.localize(8, null, entry.extension.displayName, entry.extension.description);
        };
        return AccessibilityProvider;
    }());
    var Renderer = (function () {
        function Renderer(instantiationService, extensionsService) {
            this.instantiationService = instantiationService;
            this.extensionsService = extensionsService;
        }
        Renderer.prototype.getHeight = function (entry) {
            return 48;
        };
        Renderer.prototype.getTemplateId = function (entry) {
            return 'extension';
        };
        Renderer.prototype.renderTemplate = function (templateId, container) {
            // Important to preserve order here.
            var root = dom.append(container, $('.extension'));
            var firstRow = dom.append(root, $('.row'));
            var secondRow = dom.append(root, $('.row'));
            var published = dom.append(firstRow, $('.published'));
            var displayName = new highlightedLabel_1.HighlightedLabel(dom.append(firstRow, $('span.name')));
            var installCount = dom.append(firstRow, $('span.installCount'));
            var version = dom.append(published, $('span.version'));
            var author = dom.append(published, $('span.author'));
            return {
                root: root,
                author: author,
                displayName: displayName,
                version: version,
                installCount: installCount,
                actionbar: new actionbar_1.ActionBar(dom.append(secondRow, $('.actions'))),
                description: new highlightedLabel_1.HighlightedLabel(dom.append(secondRow, $('span.description'))),
                disposables: []
            };
        };
        Renderer.prototype.renderElement = function (entry, templateId, data) {
            var _this = this;
            var extension = entry.extension;
            var publisher = extension.galleryInformation ? extension.galleryInformation.publisherDisplayName : extension.publisher;
            var installCount = extension.galleryInformation ? extension.galleryInformation.installCount : null;
            var actionOptions = { icon: true, label: false };
            var updateActions = function () {
                data.actionbar.clear();
                if (entry.extension.galleryInformation) {
                    data.actionbar.push(_this.instantiationService.createInstance(OpenInGalleryAction, entry.state !== ExtensionState.Installed), { label: true, icon: false });
                    data.actionbar.push(_this.instantiationService.createInstance(OpenLicenseAction), { label: true, icon: false });
                }
                switch (entry.state) {
                    case ExtensionState.Uninstalled:
                        if (entry.extension.galleryInformation) {
                            data.actionbar.push(_this.instantiationService.createInstance(extensionsActions_1.InstallAction, InstallLabel), actionOptions);
                        }
                        break;
                    case ExtensionState.Installed:
                        data.actionbar.push(_this.instantiationService.createInstance(extensionsActions_1.UninstallAction), actionOptions);
                        break;
                    case ExtensionState.Outdated:
                        data.actionbar.push(_this.instantiationService.createInstance(extensionsActions_1.UninstallAction), actionOptions);
                        data.actionbar.push(_this.instantiationService.createInstance(extensionsActions_1.InstallAction, UpdateLabel), actionOptions);
                        break;
                }
            };
            var onExtensionStateChange = function (e, state) {
                if (extensionsUtil_1.extensionEquals(e, extension)) {
                    entry.state = state;
                    updateActions();
                }
            };
            data.actionbar.context = extension;
            updateActions();
            data.disposables = lifecycle_1.disposeAll(data.disposables);
            data.disposables.push(this.extensionsService.onDidInstallExtension(function (e) { return onExtensionStateChange(e.extension, ExtensionState.Installed); }));
            data.disposables.push(this.extensionsService.onDidUninstallExtension(function (e) { return onExtensionStateChange(e, ExtensionState.Uninstalled); }));
            data.displayName.set(extension.displayName, entry.highlights.displayName);
            data.displayName.element.title = extension.name;
            data.version.textContent = extension.version;
            if (types_1.isNumber(installCount)) {
                data.installCount.textContent = String(installCount);
                dom.addClass(data.installCount, 'octicon');
                dom.addClass(data.installCount, 'octicon-cloud-download');
                if (!installCount) {
                    data.installCount.title = nls.localize(9, null, extension.displayName);
                }
                else if (installCount === 1) {
                    data.installCount.title = nls.localize(10, null, extension.displayName);
                }
                else {
                    data.installCount.title = nls.localize(11, null, extension.displayName, installCount);
                }
            }
            else {
                data.installCount.textContent = '';
                dom.removeClass(data.installCount, 'octicon');
                dom.removeClass(data.installCount, 'octicon-cloud-download');
            }
            data.author.textContent = publisher;
            data.description.set(extension.description, entry.highlights.description);
            data.description.element.title = extension.description;
        };
        Renderer.prototype.disposeTemplate = function (templateId, data) {
            data.displayName.dispose();
            data.description.dispose();
            data.disposables = lifecycle_1.disposeAll(data.disposables);
        };
        Renderer = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, extensions_1.IExtensionsService)
        ], Renderer);
        return Renderer;
    }());
    var DataSource = (function () {
        function DataSource() {
        }
        DataSource.prototype.getId = function (entry) {
            var extension = entry.extension;
            if (!extension) {
                throw new Error("Not an extension entry. Found " + Object.keys(entry).slice(5) + ",... instead.");
            }
            if (extension.galleryInformation) {
                return extension.galleryInformation.id + "-" + extension.version;
            }
            return "local@" + extension.publisher + "." + extension.name + "-" + extension.version + "@" + (extension.path || '');
        };
        DataSource.prototype.getLabel = function (entry) {
            return entry.extension.name;
        };
        return DataSource;
    }());
    var LocalExtensionsModel = (function () {
        function LocalExtensionsModel(extensions, instantiationService) {
            this.extensions = extensions;
            this.dataSource = new DataSource();
            this.accessibilityProvider = new AccessibilityProvider();
            this.runner = { run: function () { return false; } };
            this.renderer = instantiationService.createInstance(Renderer);
            this.entries = [];
        }
        Object.defineProperty(LocalExtensionsModel.prototype, "input", {
            set: function (input) {
                this.entries = this.extensions
                    .map(function (extension) { return ({ extension: extension, highlights: getHighlights(input.trim(), extension) }); })
                    .filter(function (_a) {
                    var highlights = _a.highlights;
                    return !!highlights;
                })
                    .map(function (_a) {
                    var extension = _a.extension, highlights = _a.highlights;
                    return ({
                        extension: extension,
                        highlights: highlights,
                        state: ExtensionState.Installed
                    });
                })
                    .sort(extensionEntryCompare);
            },
            enumerable: true,
            configurable: true
        });
        LocalExtensionsModel = __decorate([
            __param(1, instantiation_1.IInstantiationService)
        ], LocalExtensionsModel);
        return LocalExtensionsModel;
    }());
    var LocalExtensionsHandler = (function (_super) {
        __extends(LocalExtensionsHandler, _super);
        function LocalExtensionsHandler(instantiationService, extensionsService) {
            _super.call(this);
            this.instantiationService = instantiationService;
            this.extensionsService = extensionsService;
            this.modelPromise = null;
        }
        LocalExtensionsHandler.prototype.getAriaLabel = function () {
            return nls.localize(12, null);
        };
        LocalExtensionsHandler.prototype.getResults = function (input) {
            var _this = this;
            if (!this.modelPromise) {
                this.modelPromise = this.extensionsService.getInstalled()
                    .then(function (extensions) { return _this.instantiationService.createInstance(LocalExtensionsModel, extensions); });
            }
            return this.modelPromise.then(function (model) {
                model.input = input;
                return model;
            });
        };
        LocalExtensionsHandler.prototype.getEmptyLabel = function (input) {
            return nls.localize(13, null);
        };
        LocalExtensionsHandler.prototype.getAutoFocus = function (searchValue) {
            return { autoFocusFirstEntry: true };
        };
        LocalExtensionsHandler.prototype.onClose = function (canceled) {
            this.modelPromise = null;
        };
        LocalExtensionsHandler = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, extensions_1.IExtensionsService)
        ], LocalExtensionsHandler);
        return LocalExtensionsHandler;
    }(quickopen_1.QuickOpenHandler));
    exports.LocalExtensionsHandler = LocalExtensionsHandler;
    var GalleryExtensionsModel = (function () {
        function GalleryExtensionsModel(galleryExtensions, localExtensions, instantiationService) {
            this.galleryExtensions = galleryExtensions;
            this.localExtensions = localExtensions;
            this.dataSource = new DataSource();
            this.accessibilityProvider = new AccessibilityProvider();
            this.renderer = instantiationService.createInstance(Renderer);
            this.runner = instantiationService.createInstance(InstallRunner);
            this.entries = [];
        }
        Object.defineProperty(GalleryExtensionsModel.prototype, "input", {
            set: function (input) {
                var _this = this;
                this.entries = this.galleryExtensions
                    .map(function (extension) { return ({ extension: extension, highlights: getHighlights(input.trim(), extension) }); })
                    .filter(function (_a) {
                    var highlights = _a.highlights;
                    return !!highlights;
                })
                    .map(function (_a) {
                    var extension = _a.extension, highlights = _a.highlights;
                    var local = _this.localExtensions.filter(function (local) { return extensionsUtil_1.extensionEquals(local, extension); })[0];
                    return {
                        extension: extension,
                        highlights: highlights,
                        state: local
                            ? (local.version === extension.version ? ExtensionState.Installed : ExtensionState.Outdated)
                            : ExtensionState.Uninstalled
                    };
                })
                    .sort(extensionEntryCompare);
            },
            enumerable: true,
            configurable: true
        });
        GalleryExtensionsModel = __decorate([
            __param(2, instantiation_1.IInstantiationService)
        ], GalleryExtensionsModel);
        return GalleryExtensionsModel;
    }());
    var GalleryExtensionsHandler = (function (_super) {
        __extends(GalleryExtensionsHandler, _super);
        function GalleryExtensionsHandler(instantiationService, extensionsService, galleryService, telemetryService) {
            _super.call(this);
            this.instantiationService = instantiationService;
            this.extensionsService = extensionsService;
            this.galleryService = galleryService;
            this.telemetryService = telemetryService;
        }
        GalleryExtensionsHandler.prototype.getAriaLabel = function () {
            return nls.localize(14, null);
        };
        GalleryExtensionsHandler.prototype.getResults = function (input) {
            var _this = this;
            if (!this.modelPromise) {
                this.telemetryService.publicLog('extensionGallery:open');
                this.modelPromise = winjs_base_1.TPromise.join([this.galleryService.query(), this.extensionsService.getInstalled()])
                    .then(function (result) { return _this.instantiationService.createInstance(GalleryExtensionsModel, result[0], result[1]); });
            }
            return this.modelPromise.then(function (model) {
                model.input = input;
                return model;
            });
        };
        GalleryExtensionsHandler.prototype.onClose = function (canceled) {
            this.modelPromise = null;
        };
        GalleryExtensionsHandler.prototype.getEmptyLabel = function (input) {
            return nls.localize(15, null);
        };
        GalleryExtensionsHandler.prototype.getAutoFocus = function (searchValue) {
            return { autoFocusFirstEntry: true };
        };
        GalleryExtensionsHandler = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, extensions_1.IExtensionsService),
            __param(2, extensions_1.IGalleryService),
            __param(3, telemetry_1.ITelemetryService)
        ], GalleryExtensionsHandler);
        return GalleryExtensionsHandler;
    }(quickopen_1.QuickOpenHandler));
    exports.GalleryExtensionsHandler = GalleryExtensionsHandler;
    var OutdatedExtensionsModel = (function () {
        function OutdatedExtensionsModel(outdatedExtensions, instantiationService) {
            this.outdatedExtensions = outdatedExtensions;
            this.dataSource = new DataSource();
            this.accessibilityProvider = new AccessibilityProvider();
            this.renderer = instantiationService.createInstance(Renderer);
            this.runner = instantiationService.createInstance(InstallRunner);
            this.entries = [];
        }
        Object.defineProperty(OutdatedExtensionsModel.prototype, "input", {
            set: function (input) {
                this.entries = this.outdatedExtensions
                    .map(function (extension) { return ({ extension: extension, highlights: getHighlights(input.trim(), extension) }); })
                    .filter(function (_a) {
                    var highlights = _a.highlights;
                    return !!highlights;
                })
                    .map(function (_a) {
                    var extension = _a.extension, highlights = _a.highlights;
                    return ({
                        extension: extension,
                        highlights: highlights,
                        state: ExtensionState.Outdated
                    });
                })
                    .sort(extensionEntryCompare);
            },
            enumerable: true,
            configurable: true
        });
        OutdatedExtensionsModel = __decorate([
            __param(1, instantiation_1.IInstantiationService)
        ], OutdatedExtensionsModel);
        return OutdatedExtensionsModel;
    }());
    var OutdatedExtensionsHandler = (function (_super) {
        __extends(OutdatedExtensionsHandler, _super);
        function OutdatedExtensionsHandler(instantiationService, extensionsService, galleryService, telemetryService) {
            _super.call(this);
            this.instantiationService = instantiationService;
            this.extensionsService = extensionsService;
            this.galleryService = galleryService;
            this.telemetryService = telemetryService;
        }
        OutdatedExtensionsHandler.prototype.getAriaLabel = function () {
            return nls.localize(16, null);
        };
        OutdatedExtensionsHandler.prototype.getResults = function (input) {
            var _this = this;
            if (!this.modelPromise) {
                this.telemetryService.publicLog('extensionGallery:open');
                this.modelPromise = this.instantiationService.invokeFunction(extensionsUtil_1.getOutdatedExtensions)
                    .then(function (outdated) { return _this.instantiationService.createInstance(OutdatedExtensionsModel, outdated); });
            }
            return this.modelPromise.then(function (model) {
                model.input = input;
                return model;
            });
        };
        OutdatedExtensionsHandler.prototype.onClose = function (canceled) {
            this.modelPromise = null;
        };
        OutdatedExtensionsHandler.prototype.getEmptyLabel = function (input) {
            return nls.localize(17, null);
        };
        OutdatedExtensionsHandler.prototype.getAutoFocus = function (searchValue) {
            return { autoFocusFirstEntry: true };
        };
        OutdatedExtensionsHandler = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, extensions_1.IExtensionsService),
            __param(2, extensions_1.IGalleryService),
            __param(3, telemetry_1.ITelemetryService)
        ], OutdatedExtensionsHandler);
        return OutdatedExtensionsHandler;
    }(quickopen_1.QuickOpenHandler));
    exports.OutdatedExtensionsHandler = OutdatedExtensionsHandler;
    var SuggestedExtensionsModel = (function () {
        function SuggestedExtensionsModel(suggestedExtensions, localExtensions, instantiationService) {
            this.suggestedExtensions = suggestedExtensions;
            this.localExtensions = localExtensions;
            this.dataSource = new DataSource();
            this.renderer = instantiationService.createInstance(Renderer);
            this.runner = instantiationService.createInstance(InstallRunner);
            this.entries = [];
        }
        Object.defineProperty(SuggestedExtensionsModel.prototype, "input", {
            set: function (input) {
                var _this = this;
                this.entries = this.suggestedExtensions
                    .map(function (extension) { return ({ extension: extension, highlights: getHighlights(input.trim(), extension) }); })
                    .filter(function (_a) {
                    var extension = _a.extension, highlights = _a.highlights;
                    var local = _this.localExtensions.filter(function (local) { return extensionsUtil_1.extensionEquals(local, extension); })[0];
                    return !local && !!highlights;
                })
                    .map(function (_a) {
                    var extension = _a.extension, highlights = _a.highlights;
                    return {
                        extension: extension,
                        highlights: highlights,
                        state: ExtensionState.Uninstalled
                    };
                })
                    .sort(extensionEntryCompare);
            },
            enumerable: true,
            configurable: true
        });
        SuggestedExtensionsModel = __decorate([
            __param(2, instantiation_1.IInstantiationService)
        ], SuggestedExtensionsModel);
        return SuggestedExtensionsModel;
    }());
    var SuggestedExtensionHandler = (function (_super) {
        __extends(SuggestedExtensionHandler, _super);
        function SuggestedExtensionHandler(extensionTipsService, instantiationService, telemetryService, extensionsService) {
            _super.call(this);
            this.extensionTipsService = extensionTipsService;
            this.instantiationService = instantiationService;
            this.telemetryService = telemetryService;
            this.extensionsService = extensionsService;
        }
        SuggestedExtensionHandler.prototype.getResults = function (input) {
            var _this = this;
            if (!this.modelPromise) {
                this.telemetryService.publicLog('extensionRecommendations:open');
                this.modelPromise = winjs_base_1.TPromise.join([this.extensionTipsService.getRecommendations(), this.extensionsService.getInstalled()])
                    .then(function (result) { return _this.instantiationService.createInstance(SuggestedExtensionsModel, result[0], result[1]); });
            }
            return this.modelPromise.then(function (model) {
                model.input = input;
                return model;
            });
        };
        SuggestedExtensionHandler.prototype.onClose = function (canceled) {
            this.modelPromise = null;
        };
        SuggestedExtensionHandler.prototype.getEmptyLabel = function (input) {
            return nls.localize(18, null);
        };
        SuggestedExtensionHandler.prototype.getAutoFocus = function (searchValue) {
            return { autoFocusFirstEntry: true };
        };
        SuggestedExtensionHandler = __decorate([
            __param(0, extensions_1.IExtensionTipsService),
            __param(1, instantiation_1.IInstantiationService),
            __param(2, telemetry_1.ITelemetryService),
            __param(3, extensions_1.IExtensionsService)
        ], SuggestedExtensionHandler);
        return SuggestedExtensionHandler;
    }(quickopen_1.QuickOpenHandler));
    exports.SuggestedExtensionHandler = SuggestedExtensionHandler;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/extensions/electron-browser/extensionsWorkbenchExtension", ["require", "exports", 'vs/nls!vs/workbench/parts/extensions/electron-browser/extensionsWorkbenchExtension', 'vs/base/common/errors', 'vs/platform/platform', 'vs/base/common/winjs.base', 'vs/workbench/parts/extensions/common/extensions', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/base/common/severity', 'vs/workbench/services/workspace/common/contextService', 'vs/workbench/electron-browser/actions', 'vs/workbench/common/actionRegistry', 'vs/platform/actions/common/actions', './extensionsActions', './extensionTipsService', 'vs/workbench/browser/quickopen', 'electron'], function (require, exports, nls, errors, platform, winjs_base_1, extensions_1, instantiation_1, message_1, severity_1, contextService_1, actions_1, wbaregistry, actions_2, extensionsActions_1, extensionTipsService_1, quickopen_1, electron_1) {
    "use strict";
    var ExtensionsWorkbenchExtension = (function () {
        function ExtensionsWorkbenchExtension(instantiationService, extensionsService, messageService, contextService, galleryService) {
            this.instantiationService = instantiationService;
            this.extensionsService = extensionsService;
            this.messageService = messageService;
            this.registerListeners();
            var options = contextService.getOptions();
            // Extensions to install
            if (options.extensionsToInstall && options.extensionsToInstall.length) {
                this.install(options.extensionsToInstall).done(null, errors.onUnexpectedError);
            }
            // add service
            instantiationService.addSingleton(extensions_1.IExtensionTipsService, this.instantiationService.createInstance(extensionTipsService_1.ExtensionTipsService));
            var actionRegistry = platform.Registry.as(wbaregistry.Extensions.WorkbenchActions);
            actionRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(extensionsActions_1.ListExtensionsAction, extensionsActions_1.ListExtensionsAction.ID, extensionsActions_1.ListExtensionsAction.LABEL), extensions_1.ExtensionsLabel);
            platform.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/extensions/electron-browser/extensionsQuickOpen', 'LocalExtensionsHandler', 'ext ', nls.localize(0, null)));
            if (galleryService.isEnabled()) {
                actionRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(extensionsActions_1.InstallExtensionAction, extensionsActions_1.InstallExtensionAction.ID, extensionsActions_1.InstallExtensionAction.LABEL), extensions_1.ExtensionsLabel);
                platform.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/extensions/electron-browser/extensionsQuickOpen', 'GalleryExtensionsHandler', 'ext install ', nls.localize(1, null)));
                actionRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(extensionsActions_1.ListOutdatedExtensionsAction, extensionsActions_1.ListOutdatedExtensionsAction.ID, extensionsActions_1.ListOutdatedExtensionsAction.LABEL), extensions_1.ExtensionsLabel);
                platform.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/extensions/electron-browser/extensionsQuickOpen', 'OutdatedExtensionsHandler', 'ext update ', nls.localize(2, null)));
                // add extension tips services
                actionRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(extensionsActions_1.ListSuggestedExtensionsAction, extensionsActions_1.ListSuggestedExtensionsAction.ID, extensionsActions_1.ListSuggestedExtensionsAction.LABEL), extensions_1.ExtensionsLabel);
                platform.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/extensions/electron-browser/extensionsQuickOpen', 'SuggestedExtensionHandler', 'ext recommend ', nls.localize(3, null)));
            }
        }
        ExtensionsWorkbenchExtension.prototype.registerListeners = function () {
            var _this = this;
            electron_1.ipcRenderer.on('vscode:installExtensions', function (event, request) {
                if (request.extensionsToInstall) {
                    _this.install(request.extensionsToInstall).done(null, errors.onUnexpectedError);
                }
            });
        };
        ExtensionsWorkbenchExtension.prototype.install = function (extensions) {
            var _this = this;
            return winjs_base_1.Promise.join(extensions.map(function (extPath) { return _this.extensionsService.install(extPath); }))
                .then(function (extensions) {
                _this.messageService.show(severity_1.default.Info, {
                    message: extensions.length > 1 ? nls.localize(4, null)
                        : nls.localize(5, null, extensions[0].displayName),
                    actions: [_this.instantiationService.createInstance(actions_1.ReloadWindowAction, actions_1.ReloadWindowAction.ID, nls.localize(6, null))]
                });
            });
        };
        ExtensionsWorkbenchExtension.prototype.getId = function () {
            return 'vs.extensions.workbenchextension';
        };
        ExtensionsWorkbenchExtension = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, extensions_1.IExtensionsService),
            __param(2, message_1.IMessageService),
            __param(3, contextService_1.IWorkspaceContextService),
            __param(4, extensions_1.IGalleryService)
        ], ExtensionsWorkbenchExtension);
        return ExtensionsWorkbenchExtension;
    }());
    exports.ExtensionsWorkbenchExtension = ExtensionsWorkbenchExtension;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/extensions/electron-browser/extensions.contribution", ["require", "exports", 'vs/platform/platform', 'vs/platform/instantiation/common/extensions', 'vs/workbench/browser/parts/statusbar/statusbar', 'vs/workbench/parts/extensions/electron-browser/extensionsWidgets', 'vs/workbench/parts/extensions/common/extensions', 'vs/workbench/parts/extensions/common/vsoGalleryService', 'vs/workbench/common/contributions', 'vs/workbench/parts/extensions/electron-browser/extensionsWorkbenchExtension', 'vs/workbench/parts/output/common/output', 'vs/css!./media/extensions'], function (require, exports, platform_1, extensions_1, statusbar_1, extensionsWidgets_1, extensions_2, vsoGalleryService_1, contributions_1, extensionsWorkbenchExtension_1, output_1) {
    "use strict";
    extensions_1.registerSingleton(extensions_2.IGalleryService, vsoGalleryService_1.GalleryService);
    platform_1.Registry.as(contributions_1.Extensions.Workbench)
        .registerWorkbenchContribution(extensionsWorkbenchExtension_1.ExtensionsWorkbenchExtension);
    platform_1.Registry.as(statusbar_1.Extensions.Statusbar)
        .registerStatusbarItem(new statusbar_1.StatusbarItemDescriptor(extensionsWidgets_1.ExtensionsStatusbarItem, statusbar_1.StatusbarAlignment.LEFT, 10000));
    platform_1.Registry.as(output_1.Extensions.OutputChannels)
        .registerChannel(extensions_2.ExtensionsLabel);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/feedback/browser/feedback", ["require", "exports", 'vs/nls!vs/workbench/parts/feedback/browser/feedback', 'vs/base/browser/builder', 'vs/base/browser/ui/dropdown/dropdown', 'vs/platform/telemetry/common/telemetry', 'vs/workbench/services/workspace/common/contextService', 'vs/css!./media/feedback'], function (require, exports, nls, builder_1, dropdown_1, telemetry_1, contextService_1) {
    'use strict';
    var FormEvent;
    (function (FormEvent) {
        FormEvent[FormEvent["SENDING"] = 0] = "SENDING";
        FormEvent[FormEvent["SENT"] = 1] = "SENT";
        FormEvent[FormEvent["SEND_ERROR"] = 2] = "SEND_ERROR";
    })(FormEvent || (FormEvent = {}));
    var FeedbackDropdown = (function (_super) {
        __extends(FeedbackDropdown, _super);
        function FeedbackDropdown(container, options, telemetryService, contextService) {
            _super.call(this, container, {
                contextViewProvider: options.contextViewProvider,
                labelRenderer: function (container) {
                    builder_1.$(container).addClass('send-feedback');
                    return null;
                }
            });
            this.telemetryService = telemetryService;
            this.contextService = contextService;
            this.$el.addClass('send-feedback');
            this.$el.title(nls.localize(0, null));
            this.feedbackService = options.feedbackService;
            this.feedback = '';
            this.sentiment = 1;
            this.feedbackForm = null;
            this.feedbackDescriptionInput = null;
            this.smileyInput = null;
            this.frownyInput = null;
            this.sendButton = null;
            var env = contextService.getConfiguration().env;
            this.reportIssueLink = env.sendASmile.reportIssueUrl;
            this.requestFeatureLink = env.sendASmile.requestFeatureUrl;
        }
        FeedbackDropdown.prototype.renderContents = function (container) {
            var _this = this;
            var $form = builder_1.$('form.feedback-form').attr({
                action: 'javascript:void(0);',
                tabIndex: '-1'
            }).appendTo(container);
            builder_1.$(container).addClass('monaco-menu-container');
            this.feedbackForm = $form.getHTMLElement();
            builder_1.$('h2.title').text(nls.localize(1, null)).appendTo($form);
            this.invoke(builder_1.$('div.cancel').attr('tabindex', '0'), function () {
                _this.hide();
            }).appendTo($form);
            var $content = builder_1.$('div.content').appendTo($form);
            var $sentimentContainer = builder_1.$('div').appendTo($content);
            builder_1.$('span').text(nls.localize(2, null)).appendTo($sentimentContainer);
            var $feedbackSentiment = builder_1.$('div.feedback-sentiment').appendTo($sentimentContainer);
            this.smileyInput = builder_1.$('div').addClass('sentiment smile').attr({
                'aria-checked': 'false',
                'aria-label': nls.localize(3, null),
                'tabindex': 0,
                'role': 'checkbox'
            });
            this.invoke(this.smileyInput, function () { _this.setSentiment(true); }).appendTo($feedbackSentiment);
            this.frownyInput = builder_1.$('div').addClass('sentiment frown').attr({
                'aria-checked': 'false',
                'aria-label': nls.localize(4, null),
                'tabindex': 0,
                'role': 'checkbox'
            });
            this.invoke(this.frownyInput, function () { _this.setSentiment(false); }).appendTo($feedbackSentiment);
            if (this.sentiment === 1) {
                this.smileyInput.addClass('checked').attr('aria-checked', 'true');
            }
            else {
                this.frownyInput.addClass('checked').attr('aria-checked', 'true');
            }
            var $contactUs = builder_1.$('div.contactus').appendTo($content);
            builder_1.$('span').text(nls.localize(5, null)).appendTo($contactUs);
            var $contactUsContainer = builder_1.$('div.channels').appendTo($contactUs);
            builder_1.$('div').append(builder_1.$('a').attr('target', '_blank').attr('href', this.reportIssueLink).text(nls.localize(6, null)).attr('tabindex', '0'))
                .appendTo($contactUsContainer);
            builder_1.$('div').append(builder_1.$('a').attr('target', '_blank').attr('href', this.requestFeatureLink).text(nls.localize(7, null)).attr('tabindex', '0'))
                .appendTo($contactUsContainer);
            var $charCounter = builder_1.$('span.char-counter').text(this.getCharCountText(0));
            builder_1.$('h3').text(nls.localize(8, null))
                .append($charCounter)
                .appendTo($form);
            this.feedbackDescriptionInput = builder_1.$('textarea.feedback-description').attr({
                rows: 3,
                maxlength: FeedbackDropdown.MAX_FEEDBACK_CHARS,
                'aria-label': nls.localize(9, null)
            })
                .text(this.feedback).attr('required', 'required')
                .on('keyup', function () {
                $charCounter.text(_this.getCharCountText(_this.feedbackDescriptionInput.value.length));
                _this.feedbackDescriptionInput.value ? _this.sendButton.removeAttribute('disabled') : _this.sendButton.attr('disabled', '');
            })
                .appendTo($form).domFocus().getHTMLElement();
            var $buttons = builder_1.$('div.form-buttons').appendTo($form);
            this.sendButton = this.invoke(builder_1.$('input.send').type('submit').attr('disabled', '').value(nls.localize(10, null)).appendTo($buttons), function () {
                if (_this.isSendingFeedback) {
                    return;
                }
                _this.onSubmit();
            });
            return {
                dispose: function () {
                    _this.feedbackForm = null;
                    _this.feedbackDescriptionInput = null;
                    _this.smileyInput = null;
                    _this.frownyInput = null;
                }
            };
        };
        FeedbackDropdown.prototype.getCharCountText = function (charCount) {
            var remaining = FeedbackDropdown.MAX_FEEDBACK_CHARS - charCount;
            var text = (remaining === 1)
                ? nls.localize(11, null)
                : nls.localize(12, null);
            return '(' + remaining + ' ' + text + ')';
        };
        FeedbackDropdown.prototype.setSentiment = function (smile) {
            if (smile) {
                this.smileyInput.addClass('checked');
                this.smileyInput.attr('aria-checked', 'true');
                this.frownyInput.removeClass('checked');
                this.frownyInput.attr('aria-checked', 'false');
            }
            else {
                this.frownyInput.addClass('checked');
                this.frownyInput.attr('aria-checked', 'true');
                this.smileyInput.removeClass('checked');
                this.smileyInput.attr('aria-checked', 'false');
            }
            this.sentiment = smile ? 1 : 0;
        };
        FeedbackDropdown.prototype.invoke = function (element, callback) {
            element.on('click', callback);
            element.on('keypress', function (e) {
                if (e instanceof KeyboardEvent) {
                    var keyboardEvent = e;
                    if (keyboardEvent.keyCode === 13 || keyboardEvent.keyCode === 32) {
                        callback();
                    }
                }
            });
            return element;
        };
        FeedbackDropdown.prototype.hide = function () {
            if (this.feedbackDescriptionInput) {
                this.feedback = this.feedbackDescriptionInput.value;
            }
            if (this.autoHideTimeout) {
                clearTimeout(this.autoHideTimeout);
                this.autoHideTimeout = null;
            }
            _super.prototype.hide.call(this);
        };
        FeedbackDropdown.prototype.onEvent = function (e, activeElement) {
            if (e instanceof KeyboardEvent) {
                var keyboardEvent = e;
                if (keyboardEvent.keyCode === 27) {
                    this.hide();
                }
            }
        };
        FeedbackDropdown.prototype.onSubmit = function () {
            if ((this.feedbackForm.checkValidity && !this.feedbackForm.checkValidity())) {
                return;
            }
            this.changeFormStatus(FormEvent.SENDING);
            this.feedbackService.submitFeedback({
                feedback: this.feedbackDescriptionInput.value,
                sentiment: this.sentiment
            });
            this.changeFormStatus(FormEvent.SENT);
        };
        FeedbackDropdown.prototype.changeFormStatus = function (event) {
            var _this = this;
            switch (event) {
                case FormEvent.SENDING:
                    this.isSendingFeedback = true;
                    this.sendButton.setClass('send in-progress');
                    this.sendButton.value(nls.localize(13, null));
                    break;
                case FormEvent.SENT:
                    this.isSendingFeedback = false;
                    this.sendButton.setClass('send success').value(nls.localize(14, null));
                    this.resetForm();
                    this.autoHideTimeout = setTimeout(function () {
                        _this.hide();
                    }, 1000);
                    this.sendButton.off(['click', 'keypress']);
                    this.invoke(this.sendButton, function () {
                        _this.hide();
                        _this.sendButton.off(['click', 'keypress']);
                    });
                    break;
                case FormEvent.SEND_ERROR:
                    this.isSendingFeedback = false;
                    this.sendButton.setClass('send error').value(nls.localize(15, null));
                    break;
            }
        };
        FeedbackDropdown.prototype.resetForm = function () {
            if (this.feedbackDescriptionInput) {
                this.feedbackDescriptionInput.value = '';
            }
            this.sentiment = 1;
            this.aliasEnabled = false;
        };
        FeedbackDropdown.MAX_FEEDBACK_CHARS = 140;
        FeedbackDropdown = __decorate([
            __param(2, telemetry_1.ITelemetryService),
            __param(3, contextService_1.IWorkspaceContextService)
        ], FeedbackDropdown);
        return FeedbackDropdown;
    }(dropdown_1.Dropdown));
    exports.FeedbackDropdown = FeedbackDropdown;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/feedback/electron-browser/feedbackStatusbarItem", ["require", "exports", 'vs/workbench/parts/feedback/browser/feedback', 'vs/platform/contextview/browser/contextView', 'vs/platform/instantiation/common/instantiation', 'electron'], function (require, exports, feedback_1, contextView_1, instantiation_1, electron_1) {
    'use strict';
    var TwitterFeedbackService = (function () {
        function TwitterFeedbackService() {
        }
        TwitterFeedbackService.prototype.submitFeedback = function (feedback) {
            var queryString = "?" + (feedback.sentiment === 1 ? 'hashtags=HappyCoding&' : null) + "ref_src=twsrc%5Etfw&related=twitterapi%2Ctwitter&text=" + feedback.feedback + "&tw_p=tweetbutton&via=code";
            var url = TwitterFeedbackService.TWITTER_URL + queryString;
            electron_1.shell.openExternal(url);
        };
        TwitterFeedbackService.TWITTER_URL = 'https://twitter.com/intent/tweet';
        return TwitterFeedbackService;
    }());
    var FeedbackStatusbarItem = (function () {
        function FeedbackStatusbarItem(instantiationService, contextViewService) {
            this.instantiationService = instantiationService;
            this.contextViewService = contextViewService;
        }
        FeedbackStatusbarItem.prototype.render = function (element) {
            return this.instantiationService.createInstance(feedback_1.FeedbackDropdown, element, {
                contextViewProvider: this.contextViewService,
                feedbackService: this.instantiationService.createInstance(TwitterFeedbackService)
            });
        };
        FeedbackStatusbarItem = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, contextView_1.IContextViewService)
        ], FeedbackStatusbarItem);
        return FeedbackStatusbarItem;
    }());
    exports.FeedbackStatusbarItem = FeedbackStatusbarItem;
});

define("vs/workbench/parts/feedback/electron-browser/feedback.contribution", ["require", "exports", 'vs/platform/platform', 'vs/base/common/flags', 'vs/workbench/browser/parts/statusbar/statusbar', 'vs/workbench/parts/feedback/electron-browser/feedbackStatusbarItem'], function (require, exports, platform_1, Flags, statusbar_1, feedbackStatusbarItem_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // Register Statusbar item
    if (Flags.enableSendASmile) {
        platform_1.Registry.as(statusbar_1.Extensions.Statusbar).registerStatusbarItem(new statusbar_1.StatusbarItemDescriptor(feedbackStatusbarItem_1.FeedbackStatusbarItem, statusbar_1.StatusbarAlignment.RIGHT, -100 /* Low Priority */));
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/editors/fileEditorInput", ["require", "exports", 'vs/nls!vs/workbench/parts/files/browser/editors/fileEditorInput', 'vs/base/common/winjs.base', 'vs/platform/platform', 'vs/base/common/types', 'vs/base/common/paths', 'vs/base/common/mime', 'vs/base/common/labels', 'vs/base/common/strings', 'vs/base/common/assert', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor/binaryEditorModel', 'vs/platform/files/common/files', 'vs/workbench/parts/files/common/files', 'vs/workbench/parts/files/common/editors/textFileEditorModel', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/instantiation/common/instantiation'], function (require, exports, nls, winjs_base_1, platform_1, types, paths, mime_1, labels, strings, assert, baseEditor_1, binaryEditorModel_1, files_1, files_2, textFileEditorModel_1, contextService_1, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * A file editor input is the input type for the file editor of file system resources.
     */
    var FileEditorInput = (function (_super) {
        __extends(FileEditorInput, _super);
        /**
         * An editor input whos contents are retrieved from file services.
         */
        function FileEditorInput(resource, mime, preferredEncoding, instantiationService, contextService, textFileService) {
            _super.call(this);
            this.instantiationService = instantiationService;
            this.contextService = contextService;
            this.textFileService = textFileService;
            if (resource) {
                this.setResource(resource);
                this.setMime(mime || mime_1.guessMimeTypes(this.resource.fsPath).join(', '));
                this.preferredEncoding = preferredEncoding;
            }
        }
        FileEditorInput.prototype.setResource = function (resource) {
            if (resource.scheme !== 'file') {
                throw new Error('FileEditorInput can only handle file:// resources.');
            }
            this.resource = resource;
            // Reset resource dependent properties
            this.name = null;
            this.description = null;
            this.verboseDescription = null;
        };
        FileEditorInput.prototype.getResource = function () {
            return this.resource;
        };
        FileEditorInput.prototype.getMime = function () {
            return this.mime;
        };
        FileEditorInput.prototype.setMime = function (mime) {
            assert.ok(mime, 'Editor input needs mime type');
            this.mime = mime;
        };
        FileEditorInput.prototype.getEncoding = function () {
            var textModel = textFileEditorModel_1.CACHE.get(this.resource);
            if (textModel) {
                return textModel.getEncoding();
            }
            return this.preferredEncoding;
        };
        FileEditorInput.prototype.setEncoding = function (encoding, mode) {
            this.preferredEncoding = encoding;
            var textModel = textFileEditorModel_1.CACHE.get(this.resource);
            if (textModel) {
                textModel.setEncoding(encoding, mode);
            }
        };
        FileEditorInput.prototype.getId = function () {
            return files_2.FILE_EDITOR_INPUT_ID;
        };
        FileEditorInput.prototype.getName = function () {
            if (!this.name) {
                this.name = paths.basename(this.resource.fsPath);
            }
            return this.name;
        };
        FileEditorInput.prototype.getDescription = function (verbose) {
            if (!verbose) {
                if (!this.description) {
                    this.description = labels.getPathLabel(paths.dirname(this.resource.fsPath), this.contextService);
                }
                return this.description;
            }
            if (!this.verboseDescription) {
                this.verboseDescription = labels.getPathLabel(this.resource.fsPath);
            }
            return this.verboseDescription;
        };
        FileEditorInput.prototype.getStatus = function () {
            var textModel = textFileEditorModel_1.CACHE.get(this.resource);
            if (textModel) {
                var state = textModel.getState();
                switch (state) {
                    case textFileEditorModel_1.State.SAVED: {
                        return { state: 'saved', displayText: FileEditorInput.nlsSavedDisplay, description: FileEditorInput.nlsSavedMeta };
                    }
                    case textFileEditorModel_1.State.DIRTY: {
                        return { state: 'dirty', decoration: (this.textFileService.getAutoSaveMode() !== files_2.AutoSaveMode.AFTER_SHORT_DELAY) ? '\u25cf' : '', displayText: FileEditorInput.nlsDirtyDisplay, description: FileEditorInput.nlsDirtyMeta };
                    }
                    case textFileEditorModel_1.State.PENDING_SAVE:
                        return { state: 'saving', decoration: (this.textFileService.getAutoSaveMode() !== files_2.AutoSaveMode.AFTER_SHORT_DELAY) ? '\u25cf' : '', displayText: FileEditorInput.nlsPendingSaveDisplay, description: FileEditorInput.nlsPendingSaveMeta };
                    case textFileEditorModel_1.State.ERROR:
                        return { state: 'error', decoration: '\u25cf', displayText: FileEditorInput.nlsErrorDisplay, description: FileEditorInput.nlsErrorMeta };
                    case textFileEditorModel_1.State.CONFLICT:
                        return { state: 'conflict', decoration: '\u25cf', displayText: FileEditorInput.nlsConflictDisplay, description: FileEditorInput.nlsConflictMeta };
                }
            }
            return null;
        };
        FileEditorInput.prototype.getPreferredEditorId = function (candidates) {
            var editorRegistry = platform_1.Registry.as(baseEditor_1.Extensions.Editors);
            // Lookup Editor by Mime
            var descriptor;
            var mimes = this.mime.split(',');
            for (var m = 0; m < mimes.length; m++) {
                var mime = strings.trim(mimes[m]);
                for (var i = 0; i < candidates.length; i++) {
                    descriptor = editorRegistry.getEditorById(candidates[i]);
                    if (types.isFunction(descriptor.getMimeTypes)) {
                        var mimetypes = descriptor.getMimeTypes();
                        for (var j = 0; j < mimetypes.length; j++) {
                            var mimetype = mimetypes[j];
                            // Check for direct mime match
                            if (mime === mimetype) {
                                return descriptor.getId();
                            }
                            // Otherwise check for wildcard mime matches
                            if (strings.endsWith(mimetype, '/*') && strings.startsWith(mime, mimetype.substring(0, mimetype.length - 1))) {
                                return descriptor.getId();
                            }
                        }
                    }
                }
            }
            // Otherwise use default editor
            return files_2.BINARY_FILE_EDITOR_ID;
        };
        FileEditorInput.prototype.resolve = function (refresh) {
            var _this = this;
            var modelPromise;
            // Keep clients who resolved the input to support proper disposal
            var clients = FileEditorInput.FILE_EDITOR_MODEL_CLIENTS[this.resource.toString()];
            if (types.isUndefinedOrNull(clients)) {
                FileEditorInput.FILE_EDITOR_MODEL_CLIENTS[this.resource.toString()] = [this];
            }
            else if (this.indexOfClient() === -1) {
                FileEditorInput.FILE_EDITOR_MODEL_CLIENTS[this.resource.toString()].push(this);
            }
            // Check for running loader to ensure the model is only ever loaded once
            if (FileEditorInput.FILE_EDITOR_MODEL_LOADERS[this.resource.toString()]) {
                return FileEditorInput.FILE_EDITOR_MODEL_LOADERS[this.resource.toString()];
            }
            // Use Cached Model if present
            var cachedModel = textFileEditorModel_1.CACHE.get(this.resource);
            if (cachedModel && !refresh) {
                modelPromise = winjs_base_1.TPromise.as(cachedModel);
            }
            else if (cachedModel && refresh) {
                modelPromise = cachedModel.load();
                FileEditorInput.FILE_EDITOR_MODEL_LOADERS[this.resource.toString()] = modelPromise;
            }
            else {
                modelPromise = this.createAndLoadModel();
                FileEditorInput.FILE_EDITOR_MODEL_LOADERS[this.resource.toString()] = modelPromise;
            }
            return modelPromise.then(function (resolvedModel) {
                if (resolvedModel instanceof textFileEditorModel_1.TextFileEditorModel) {
                    textFileEditorModel_1.CACHE.add(_this.resource, resolvedModel); // Store into the text model cache unless this file is binary
                }
                FileEditorInput.FILE_EDITOR_MODEL_LOADERS[_this.resource.toString()] = null; // Remove from pending loaders
                return resolvedModel;
            }, function (error) {
                FileEditorInput.FILE_EDITOR_MODEL_LOADERS[_this.resource.toString()] = null; // Remove from pending loaders in case of an error
                return winjs_base_1.TPromise.wrapError(error);
            });
        };
        FileEditorInput.prototype.indexOfClient = function () {
            if (!types.isUndefinedOrNull(FileEditorInput.FILE_EDITOR_MODEL_CLIENTS[this.resource.toString()])) {
                for (var i = 0; i < FileEditorInput.FILE_EDITOR_MODEL_CLIENTS[this.resource.toString()].length; i++) {
                    var client = FileEditorInput.FILE_EDITOR_MODEL_CLIENTS[this.resource.toString()][i];
                    if (client === this) {
                        return i;
                    }
                }
            }
            return -1;
        };
        FileEditorInput.prototype.createAndLoadModel = function () {
            var _this = this;
            var descriptor = platform_1.Registry.as(baseEditor_1.Extensions.Editors).getEditor(this);
            if (!descriptor) {
                throw new Error('Unable to find an editor in the registry for this input.');
            }
            // Optimistically create a text model assuming that the file is not binary
            var textModel = this.instantiationService.createInstance(textFileEditorModel_1.TextFileEditorModel, this.resource, this.preferredEncoding);
            return textModel.load().then(function () { return textModel; }, function (error) {
                // In case of an error that indicates that the file is binary or too large, just return with the binary editor model
                if (error.fileOperationResult === files_1.FileOperationResult.FILE_IS_BINARY || error.fileOperationResult === files_1.FileOperationResult.FILE_TOO_LARGE) {
                    textModel.dispose();
                    var binaryModel = new binaryEditorModel_1.BinaryEditorModel(_this.resource, _this.getName());
                    return binaryModel.load();
                }
                // Bubble any other error up
                return winjs_base_1.TPromise.wrapError(error);
            });
        };
        FileEditorInput.prototype.dispose = function (force) {
            // TextFileEditorModel
            var cachedModel = textFileEditorModel_1.CACHE.get(this.resource);
            if (cachedModel) {
                // Only dispose if the last client called dispose() unless a forced dispose is triggered
                var index = this.indexOfClient();
                if (index >= 0) {
                    // Remove from Clients List
                    FileEditorInput.FILE_EDITOR_MODEL_CLIENTS[this.resource.toString()].splice(index, 1);
                    // Still clients around, thereby do not dispose yet
                    if (!force && FileEditorInput.FILE_EDITOR_MODEL_CLIENTS[this.resource.toString()].length > 0) {
                        return;
                    }
                    // We typically never want to dispose a file editor model because this means loosing undo/redo history.
                    // For that, we will keep the model around unless someone forces a dispose on the input. A forced dispose
                    // can happen when the model has not been used for a while or was changed outside the application which
                    // means loosing the undo redo history anyways.
                    if (!force) {
                        return;
                    }
                    // Dispose for real
                    textFileEditorModel_1.CACHE.dispose(this.resource);
                }
            }
            _super.prototype.dispose.call(this);
        };
        FileEditorInput.prototype.matches = function (otherInput) {
            if (_super.prototype.matches.call(this, otherInput) === true) {
                return true;
            }
            if (otherInput) {
                // Note that we can not test for the mime type here because we cache resolved file editor input models by resource. And
                // these models have a fixed mode association that can not be changed afterwards. As such, we always treat this input
                // equal if the resource is equal so that there is always just one text editor model (with undo hisotry etc.) around.
                //
                // !!! DO NOT CHANGE THIS ASSUMPTION !!!
                //
                return otherInput instanceof FileEditorInput && otherInput.resource.toString() === this.resource.toString();
            }
            return false;
        };
        /**
         * Exposed so that other internal file API can access the list of all file editor inputs
         * that have been loaded during the session.
         */
        FileEditorInput.getAll = function (desiredFileOrFolderResource) {
            var inputsContainingResource = [];
            var clients = FileEditorInput.FILE_EDITOR_MODEL_CLIENTS;
            for (var resource in clients) {
                var inputs = clients[resource];
                // Check if path is identical or path is a folder that the content is inside
                if (paths.isEqualOrParent(resource, desiredFileOrFolderResource.toString())) {
                    inputsContainingResource.push.apply(inputsContainingResource, inputs);
                }
            }
            return inputsContainingResource;
        };
        // Do ref counting for all inputs that resolved to a model to be able to dispose when count = 0
        FileEditorInput.FILE_EDITOR_MODEL_CLIENTS = Object.create(null);
        // Keep promises that load a file editor model to avoid loading the same model twice
        FileEditorInput.FILE_EDITOR_MODEL_LOADERS = Object.create(null);
        // These nls things are looked up way too often to not cache them..
        FileEditorInput.nlsSavedDisplay = nls.localize(0, null);
        FileEditorInput.nlsSavedMeta = nls.localize(1, null);
        FileEditorInput.nlsDirtyDisplay = nls.localize(2, null);
        FileEditorInput.nlsDirtyMeta = nls.localize(3, null);
        FileEditorInput.nlsPendingSaveDisplay = nls.localize(4, null);
        FileEditorInput.nlsPendingSaveMeta = nls.localize(5, null);
        FileEditorInput.nlsErrorDisplay = nls.localize(6, null);
        FileEditorInput.nlsErrorMeta = nls.localize(7, null);
        FileEditorInput.nlsConflictDisplay = nls.localize(8, null);
        FileEditorInput.nlsConflictMeta = nls.localize(9, null);
        FileEditorInput = __decorate([
            __param(3, instantiation_1.IInstantiationService),
            __param(4, contextService_1.IWorkspaceContextService),
            __param(5, files_2.ITextFileService)
        ], FileEditorInput);
        return FileEditorInput;
    }(files_2.FileEditorInput));
    exports.FileEditorInput = FileEditorInput;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/fileActions", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/files/browser/fileActions', 'vs/base/common/platform', 'vs/base/common/async', 'vs/base/common/mime', 'vs/base/common/paths', 'vs/base/common/uri', 'vs/base/common/errors', 'vs/base/common/strings', 'vs/base/common/events', 'vs/base/common/labels', 'vs/base/common/diagnostics', 'vs/base/common/actions', 'vs/base/browser/ui/inputbox/inputBox', 'vs/base/common/lifecycle', 'vs/workbench/common/events', 'vs/workbench/parts/files/common/files', 'vs/platform/files/common/files', 'vs/workbench/common/editor/diffEditorInput', 'vs/workbench/common/editor', 'vs/workbench/parts/files/browser/editors/fileEditorInput', 'vs/workbench/parts/files/common/explorerViewModel', 'vs/workbench/parts/files/common/editors/textFileEditorModel', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/part/common/partService', 'vs/platform/storage/common/storage', 'vs/platform/editor/common/editor', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/platform/progress/common/progress', 'vs/platform/workspace/common/workspace', 'vs/base/common/keyCodes', 'vs/css!./media/fileactions'], function (require, exports, winjs_base_1, nls, platform_1, async_1, mime_1, paths, uri_1, errors, strings, events_1, labels_1, diagnostics, actions_1, inputBox_1, lifecycle_1, events_2, Files, files_1, diffEditorInput_1, editor_1, fileEditorInput_1, explorerViewModel_1, textFileEditorModel_1, untitledEditorService_1, editorService_1, quickOpenService_1, viewletService_1, partService_1, storage_1, editor_2, event_1, instantiation_1, message_1, progress_1, workspace_1, keyCodes_1) {
    'use strict';
    var ITextFileService = Files.ITextFileService;
    var BaseFileAction = (function (_super) {
        __extends(BaseFileAction, _super);
        function BaseFileAction(id, label, _contextService, _editorService, _fileService, _messageService, _textFileService, _eventService) {
            var _this = this;
            _super.call(this, id, label);
            this._contextService = _contextService;
            this._editorService = _editorService;
            this._fileService = _fileService;
            this._messageService = _messageService;
            this._textFileService = _textFileService;
            this._eventService = _eventService;
            this.enabled = false;
            // update enablement when options change
            this.listenerToUnbind = this._eventService.addListener(events_2.EventType.WORKBENCH_OPTIONS_CHANGED, function () { return _this._updateEnablement(); });
        }
        Object.defineProperty(BaseFileAction.prototype, "contextService", {
            get: function () {
                return this._contextService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseFileAction.prototype, "messageService", {
            get: function () {
                return this._messageService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseFileAction.prototype, "editorService", {
            get: function () {
                return this._editorService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseFileAction.prototype, "fileService", {
            get: function () {
                return this._fileService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseFileAction.prototype, "eventService", {
            get: function () {
                return this._eventService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseFileAction.prototype, "textFileService", {
            get: function () {
                return this._textFileService;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseFileAction.prototype, "element", {
            get: function () {
                return this._element;
            },
            set: function (element) {
                this._element = element;
            },
            enumerable: true,
            configurable: true
        });
        BaseFileAction.prototype._isEnabled = function () {
            return true;
        };
        BaseFileAction.prototype._updateEnablement = function () {
            this.enabled = !!(this._contextService && this._fileService && this._editorService && !this._contextService.getOptions().readOnly && this._isEnabled());
        };
        BaseFileAction.prototype.onError = function (error) {
            this._messageService.show(message_1.Severity.Error, error);
        };
        BaseFileAction.prototype.onWarning = function (warning) {
            this._messageService.show(message_1.Severity.Warning, warning);
        };
        BaseFileAction.prototype.onErrorWithRetry = function (error, retry, extraAction) {
            var actions = [
                message_1.CancelAction,
                new actions_1.Action(this.id, nls.localize(0, null), null, true, function () { return retry(); })
            ];
            if (extraAction) {
                actions.push(extraAction);
            }
            var errorWithRetry = {
                actions: actions,
                message: errors.toErrorMessage(error, false)
            };
            this._messageService.show(message_1.Severity.Error, errorWithRetry);
        };
        BaseFileAction.prototype.handleDirty = function () {
            if (this.textFileService.isDirty(this._element.resource)) {
                var res = this.textFileService.confirmSave([this._element.resource]);
                if (res === Files.ConfirmResult.SAVE) {
                    return this.textFileService.save(this._element.resource).then(function () { return false; });
                }
                if (res === Files.ConfirmResult.DONT_SAVE) {
                    return this.textFileService.revert(this._element.resource).then(function () { return false; });
                }
                return winjs_base_1.TPromise.as(true);
            }
            return winjs_base_1.TPromise.as(false);
        };
        BaseFileAction.prototype.dispose = function () {
            this.listenerToUnbind();
            _super.prototype.dispose.call(this);
        };
        BaseFileAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, files_1.IFileService),
            __param(5, message_1.IMessageService),
            __param(6, ITextFileService),
            __param(7, event_1.IEventService)
        ], BaseFileAction);
        return BaseFileAction;
    }(actions_1.Action));
    exports.BaseFileAction = BaseFileAction;
    var TriggerRenameFileAction = (function (_super) {
        __extends(TriggerRenameFileAction, _super);
        function TriggerRenameFileAction(tree, element, contextService, editorService, fileService, messageService, textFileService, eventService, instantiationService) {
            _super.call(this, TriggerRenameFileAction.ID, nls.localize(1, null), contextService, editorService, fileService, messageService, textFileService, eventService);
            this.tree = tree;
            this.element = element;
            this.renameAction = instantiationService.createInstance(RenameFileAction, element);
            this._updateEnablement();
        }
        TriggerRenameFileAction.prototype.validateFileName = function (parent, name) {
            return this.renameAction.validateFileName(this.element.parent, name);
        };
        TriggerRenameFileAction.prototype.run = function (context) {
            var _this = this;
            if (!context) {
                return winjs_base_1.TPromise.wrapError('No context provided to BaseEnableFileRenameAction.');
            }
            var viewletState = context.viewletState;
            if (!viewletState) {
                return winjs_base_1.TPromise.wrapError('Invalid viewlet state provided to BaseEnableFileRenameAction.');
            }
            var stat = context.stat;
            if (!stat) {
                return winjs_base_1.TPromise.wrapError('Invalid stat provided to BaseEnableFileRenameAction.');
            }
            viewletState.setEditable(stat, {
                action: this.renameAction,
                validator: function (value) {
                    var message = _this.validateFileName(_this.element.parent, value);
                    if (!message) {
                        return null;
                    }
                    return {
                        content: message,
                        formatContent: true,
                        type: inputBox_1.MessageType.ERROR
                    };
                }
            });
            this.tree.refresh(stat, false).then(function () {
                _this.tree.setHighlight(stat);
                var unbind = _this.tree.addListener(events_1.EventType.HIGHLIGHT, function (e) {
                    if (!e.highlight) {
                        viewletState.clearEditable(stat);
                        _this.tree.refresh(stat).done(null, errors.onUnexpectedError);
                        unbind();
                    }
                });
            }).done(null, errors.onUnexpectedError);
        };
        TriggerRenameFileAction.ID = 'workbench.files.action.triggerRename';
        TriggerRenameFileAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, files_1.IFileService),
            __param(5, message_1.IMessageService),
            __param(6, ITextFileService),
            __param(7, event_1.IEventService),
            __param(8, instantiation_1.IInstantiationService)
        ], TriggerRenameFileAction);
        return TriggerRenameFileAction;
    }(BaseFileAction));
    exports.TriggerRenameFileAction = TriggerRenameFileAction;
    var BaseRenameAction = (function (_super) {
        __extends(BaseRenameAction, _super);
        function BaseRenameAction(id, label, element, contextService, editorService, fileService, messageService, textFileService, progressService, eventService) {
            _super.call(this, id, label, contextService, editorService, fileService, messageService, textFileService, eventService);
            this.progressService = progressService;
            this.element = element;
        }
        BaseRenameAction.prototype.run = function (context) {
            var _this = this;
            if (!context) {
                return winjs_base_1.TPromise.wrapError('No context provided to BaseRenameFileAction.');
            }
            var name = context.value;
            if (!name) {
                return winjs_base_1.TPromise.wrapError('No new name provided to BaseRenameFileAction.');
            }
            // Automatically trim whitespaces and trailing dots to produce nice file names
            name = getWellFormedFileName(name);
            var existingName = getWellFormedFileName(this.element.name);
            // Return early if name is invalid or didn't change
            if (name === existingName || this.validateFileName(this.element.parent, name)) {
                return winjs_base_1.TPromise.as(null);
            }
            // Call function and Emit Event through viewer
            var promise = this.runAction(name).then(function (stat) {
                if (stat) {
                    _this.onSuccess(stat);
                }
            }, function (error) {
                _this.onError(error);
            });
            if (this.progressService) {
                this.progressService.showWhile(promise, 800);
            }
            return promise;
        };
        BaseRenameAction.prototype.validateFileName = function (parent, name) {
            var source = this.element.name;
            var target = name;
            if (!platform_1.isLinux) {
                source = source.toLowerCase();
                target = target.toLowerCase();
            }
            if (getWellFormedFileName(source) === getWellFormedFileName(target)) {
                return null;
            }
            return validateFileName(parent, name, false);
        };
        BaseRenameAction.prototype.onSuccess = function (stat) {
            var before = null;
            if (!(this.element instanceof explorerViewModel_1.NewStatPlaceholder)) {
                before = this.element.clone(); // Clone element to not expose viewers element to listeners
            }
            this.eventService.emit('files.internal:fileChanged', new Files.LocalFileChangeEvent(before, stat));
        };
        BaseRenameAction = __decorate([
            __param(3, workspace_1.IWorkspaceContextService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, files_1.IFileService),
            __param(6, message_1.IMessageService),
            __param(7, ITextFileService),
            __param(8, progress_1.IProgressService),
            __param(9, event_1.IEventService)
        ], BaseRenameAction);
        return BaseRenameAction;
    }(BaseFileAction));
    exports.BaseRenameAction = BaseRenameAction;
    var RenameFileAction = (function (_super) {
        __extends(RenameFileAction, _super);
        function RenameFileAction(element, contextService, editorService, fileService, messageService, textFileService, progressService, eventService) {
            _super.call(this, RenameFileAction.ID, nls.localize(2, null), element, contextService, editorService, fileService, messageService, textFileService, progressService, eventService);
            this._updateEnablement();
        }
        RenameFileAction.prototype.runAction = function (newName) {
            var _this = this;
            // Check if file is dirty in editor and save it to avoid data loss
            return this.handleDirty().then(function (cancel) {
                if (cancel) {
                    return winjs_base_1.TPromise.as(null);
                }
                // If the file is still dirty, do not touch it because a save is pending to disk and we can not abort it
                if (_this.textFileService.isDirty(_this.element.resource)) {
                    _this.onWarning(nls.localize(3, null, labels_1.getPathLabel(_this.element.resource)));
                    return winjs_base_1.TPromise.as(null);
                }
                return _this.fileService.rename(_this.element.resource, newName).then(null, function (error) {
                    _this.onErrorWithRetry(error, function () { return _this.runAction(newName); });
                });
            });
        };
        RenameFileAction.ID = 'workbench.files.action.renameFile';
        RenameFileAction = __decorate([
            __param(1, workspace_1.IWorkspaceContextService),
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, files_1.IFileService),
            __param(4, message_1.IMessageService),
            __param(5, ITextFileService),
            __param(6, progress_1.IProgressService),
            __param(7, event_1.IEventService)
        ], RenameFileAction);
        return RenameFileAction;
    }(BaseRenameAction));
    exports.RenameFileAction = RenameFileAction;
    /* Base New File/Folder Action */
    var BaseNewAction = (function (_super) {
        __extends(BaseNewAction, _super);
        function BaseNewAction(id, label, tree, isFile, editableAction, element, contextService, editorService, fileService, messageService, textFileService, eventService) {
            _super.call(this, id, label, contextService, editorService, fileService, messageService, textFileService, eventService);
            if (element) {
                this.presetFolder = element.isDirectory ? element : element.parent;
            }
            this.tree = tree;
            this.isFile = isFile;
            this.renameAction = editableAction;
        }
        BaseNewAction.prototype.run = function (context) {
            var _this = this;
            if (!context) {
                return winjs_base_1.TPromise.wrapError('No context provided to BaseNewAction.');
            }
            var viewletState = context.viewletState;
            if (!viewletState) {
                return winjs_base_1.TPromise.wrapError('Invalid viewlet state provided to BaseNewAction.');
            }
            var folder = this.presetFolder;
            if (!folder) {
                var focus_1 = this.tree.getFocus();
                if (focus_1) {
                    folder = focus_1.isDirectory ? focus_1 : focus_1.parent;
                }
                else {
                    folder = this.tree.getInput();
                }
            }
            if (!folder) {
                return winjs_base_1.TPromise.wrapError('Invalid parent folder to create.');
            }
            return this.tree.reveal(folder, 0.5).then(function () {
                return _this.tree.expand(folder).then(function () {
                    var stat = explorerViewModel_1.NewStatPlaceholder.addNewStatPlaceholder(folder, !_this.isFile);
                    _this.renameAction.element = stat;
                    viewletState.setEditable(stat, {
                        action: _this.renameAction,
                        validator: function (value) {
                            var message = _this.renameAction.validateFileName(folder, value);
                            if (!message) {
                                return null;
                            }
                            return {
                                content: message,
                                formatContent: true,
                                type: inputBox_1.MessageType.ERROR
                            };
                        }
                    });
                    return _this.tree.refresh(folder).then(function () {
                        return _this.tree.expand(folder).then(function () {
                            return _this.tree.reveal(stat, 0.5).then(function () {
                                _this.tree.setHighlight(stat);
                                var unbind = _this.tree.addListener(events_1.EventType.HIGHLIGHT, function (e) {
                                    if (!e.highlight) {
                                        stat.destroy();
                                        _this.tree.refresh(folder).done(null, errors.onUnexpectedError);
                                        unbind();
                                    }
                                });
                            });
                        });
                    });
                });
            });
        };
        BaseNewAction = __decorate([
            __param(6, workspace_1.IWorkspaceContextService),
            __param(7, editorService_1.IWorkbenchEditorService),
            __param(8, files_1.IFileService),
            __param(9, message_1.IMessageService),
            __param(10, ITextFileService),
            __param(11, event_1.IEventService)
        ], BaseNewAction);
        return BaseNewAction;
    }(BaseFileAction));
    exports.BaseNewAction = BaseNewAction;
    /* New File */
    var NewFileAction = (function (_super) {
        __extends(NewFileAction, _super);
        function NewFileAction(tree, element, contextService, editorService, fileService, messageService, textFileService, eventService, instantiationService) {
            _super.call(this, 'workbench.action.files.newFile', nls.localize(4, null), tree, true, instantiationService.createInstance(CreateFileAction, element), null, contextService, editorService, fileService, messageService, textFileService, eventService);
            this.class = 'explorer-action new-file';
            this._updateEnablement();
        }
        NewFileAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, files_1.IFileService),
            __param(5, message_1.IMessageService),
            __param(6, ITextFileService),
            __param(7, event_1.IEventService),
            __param(8, instantiation_1.IInstantiationService)
        ], NewFileAction);
        return NewFileAction;
    }(BaseNewAction));
    exports.NewFileAction = NewFileAction;
    /* New Folder */
    var NewFolderAction = (function (_super) {
        __extends(NewFolderAction, _super);
        function NewFolderAction(tree, element, contextService, editorService, fileService, messageService, textFileService, eventService, instantiationService) {
            _super.call(this, 'workbench.action.files.newFolder', nls.localize(5, null), tree, false, instantiationService.createInstance(CreateFolderAction, element), null, contextService, editorService, fileService, messageService, textFileService, eventService);
            this.class = 'explorer-action new-folder';
            this._updateEnablement();
        }
        NewFolderAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, files_1.IFileService),
            __param(5, message_1.IMessageService),
            __param(6, ITextFileService),
            __param(7, event_1.IEventService),
            __param(8, instantiation_1.IInstantiationService)
        ], NewFolderAction);
        return NewFolderAction;
    }(BaseNewAction));
    exports.NewFolderAction = NewFolderAction;
    var BaseGlobalNewAction = (function (_super) {
        __extends(BaseGlobalNewAction, _super);
        function BaseGlobalNewAction(id, label, viewletService, instantiationService) {
            _super.call(this, id, label);
            this.viewletService = viewletService;
            this.instantiationService = instantiationService;
        }
        BaseGlobalNewAction.prototype.run = function () {
            var _this = this;
            return this.viewletService.openViewlet(Files.VIEWLET_ID, true).then(function (viewlet) {
                return winjs_base_1.TPromise.timeout(100).then(function () {
                    viewlet.focus();
                    var explorer = viewlet;
                    var explorerView = explorer.getExplorerView();
                    if (!explorerView.isExpanded()) {
                        explorerView.expand();
                    }
                    var action = _this.toDispose = _this.instantiationService.createInstance(_this.getAction(), explorerView.getViewer(), null);
                    return explorer.getActionRunner().run(action);
                });
            });
        };
        BaseGlobalNewAction.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            if (this.toDispose) {
                this.toDispose.dispose();
                this.toDispose = null;
            }
        };
        BaseGlobalNewAction = __decorate([
            __param(2, viewletService_1.IViewletService),
            __param(3, instantiation_1.IInstantiationService)
        ], BaseGlobalNewAction);
        return BaseGlobalNewAction;
    }(actions_1.Action));
    exports.BaseGlobalNewAction = BaseGlobalNewAction;
    /* Create new file from anywhere: Open untitled */
    var GlobalNewFileAction = (function (_super) {
        __extends(GlobalNewFileAction, _super);
        function GlobalNewFileAction(id, label, storageService, editorService, textFileService, untitledEditorService) {
            _super.call(this, id, label);
            this.storageService = storageService;
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.untitledEditorService = untitledEditorService;
        }
        GlobalNewFileAction.prototype.run = function () {
            var input = this.untitledEditorService.createOrGet();
            // Make sure this untitled buffer shows up in working files set
            this.textFileService.getWorkingFilesModel().addEntry(input.getResource());
            return this.editorService.openEditor(input);
        };
        GlobalNewFileAction.ID = 'workbench.action.files.newUntitledFile';
        GlobalNewFileAction.LABEL = nls.localize(6, null);
        GlobalNewFileAction = __decorate([
            __param(2, storage_1.IStorageService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, ITextFileService),
            __param(5, untitledEditorService_1.IUntitledEditorService)
        ], GlobalNewFileAction);
        return GlobalNewFileAction;
    }(actions_1.Action));
    exports.GlobalNewFileAction = GlobalNewFileAction;
    /* Create new folder from anywhere */
    var GlobalNewFolderAction = (function (_super) {
        __extends(GlobalNewFolderAction, _super);
        function GlobalNewFolderAction() {
            _super.apply(this, arguments);
        }
        GlobalNewFolderAction.prototype.getAction = function () {
            return NewFolderAction;
        };
        GlobalNewFolderAction.ID = 'workbench.action.files.newFolder';
        GlobalNewFolderAction.LABEL = nls.localize(7, null);
        return GlobalNewFolderAction;
    }(BaseGlobalNewAction));
    exports.GlobalNewFolderAction = GlobalNewFolderAction;
    /* Create New File/Folder (only used internally by explorerViewer) */
    var BaseCreateAction = (function (_super) {
        __extends(BaseCreateAction, _super);
        function BaseCreateAction() {
            _super.apply(this, arguments);
        }
        BaseCreateAction.prototype.validateFileName = function (parent, name) {
            if (this.element instanceof explorerViewModel_1.NewStatPlaceholder) {
                return validateFileName(parent, name, false);
            }
            return _super.prototype.validateFileName.call(this, parent, name);
        };
        return BaseCreateAction;
    }(BaseRenameAction));
    exports.BaseCreateAction = BaseCreateAction;
    /* Create New File (only used internally by explorerViewer) */
    var CreateFileAction = (function (_super) {
        __extends(CreateFileAction, _super);
        function CreateFileAction(element, contextService, editorService, fileService, messageService, textFileService, progressService, eventService) {
            _super.call(this, CreateFileAction.ID, CreateFileAction.LABEL, element, contextService, editorService, fileService, messageService, textFileService, progressService, eventService);
            this._updateEnablement();
        }
        CreateFileAction.prototype.runAction = function (fileName) {
            var _this = this;
            return this.fileService.createFile(uri_1.default.file(paths.join(this.element.parent.resource.fsPath, fileName))).then(function (stat) {
                _this.textFileService.getWorkingFilesModel().addEntry(stat.resource); // add to working files
                return stat;
            }, function (error) {
                _this.onErrorWithRetry(error, function () { return _this.runAction(fileName); });
            });
        };
        CreateFileAction.ID = 'workbench.files.action.createFileFromExplorer';
        CreateFileAction.LABEL = nls.localize(8, null);
        CreateFileAction = __decorate([
            __param(1, workspace_1.IWorkspaceContextService),
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, files_1.IFileService),
            __param(4, message_1.IMessageService),
            __param(5, ITextFileService),
            __param(6, progress_1.IProgressService),
            __param(7, event_1.IEventService)
        ], CreateFileAction);
        return CreateFileAction;
    }(BaseCreateAction));
    exports.CreateFileAction = CreateFileAction;
    /* Create New Folder (only used internally by explorerViewer) */
    var CreateFolderAction = (function (_super) {
        __extends(CreateFolderAction, _super);
        function CreateFolderAction(element, contextService, editorService, fileService, messageService, textFileService, progressService, eventService) {
            _super.call(this, CreateFolderAction.ID, CreateFolderAction.LABEL, null, contextService, editorService, fileService, messageService, textFileService, progressService, eventService);
            this._updateEnablement();
        }
        CreateFolderAction.prototype.runAction = function (fileName) {
            var _this = this;
            return this.fileService.createFolder(uri_1.default.file(paths.join(this.element.parent.resource.fsPath, fileName))).then(null, function (error) {
                _this.onErrorWithRetry(error, function () { return _this.runAction(fileName); });
            });
        };
        CreateFolderAction.ID = 'workbench.files.action.createFolderFromExplorer';
        CreateFolderAction.LABEL = nls.localize(9, null);
        CreateFolderAction = __decorate([
            __param(1, workspace_1.IWorkspaceContextService),
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, files_1.IFileService),
            __param(4, message_1.IMessageService),
            __param(5, ITextFileService),
            __param(6, progress_1.IProgressService),
            __param(7, event_1.IEventService)
        ], CreateFolderAction);
        return CreateFolderAction;
    }(BaseCreateAction));
    exports.CreateFolderAction = CreateFolderAction;
    var BaseDeleteFileAction = (function (_super) {
        __extends(BaseDeleteFileAction, _super);
        function BaseDeleteFileAction(id, label, tree, element, useTrash, contextService, editorService, fileService, messageService, textFileService, eventService) {
            _super.call(this, id, label, contextService, editorService, fileService, messageService, textFileService, eventService);
            this.tree = tree;
            this.element = element;
            this.useTrash = useTrash && !paths.isUNC(element.resource.fsPath); // on UNC shares there is no trash
            this._updateEnablement();
        }
        BaseDeleteFileAction.prototype.run = function () {
            var _this = this;
            // Remove highlight
            if (this.tree) {
                this.tree.clearHighlight();
            }
            // Ask for Confirm
            if (!this.skipConfirm) {
                var confirm_1;
                if (this.useTrash) {
                    confirm_1 = {
                        message: this.element.isDirectory ? nls.localize(10, null, this.element.name) : nls.localize(11, null, this.element.name),
                        detail: platform_1.isWindows ? nls.localize(12, null) : nls.localize(13, null),
                        primaryButton: platform_1.isWindows ? nls.localize(14, null) : nls.localize(15, null)
                    };
                }
                else {
                    confirm_1 = {
                        message: this.element.isDirectory ? nls.localize(16, null, this.element.name) : nls.localize(17, null, this.element.name),
                        detail: nls.localize(18, null),
                        primaryButton: nls.localize(19, null)
                    };
                }
                if (!this.messageService.confirm(confirm_1)) {
                    return winjs_base_1.TPromise.as(null);
                }
            }
            // Check if file is dirty in editor and save it to avoid data loss
            return this.handleDirty().then(function (cancel) {
                if (cancel) {
                    return winjs_base_1.TPromise.as(null);
                }
                // If the file is still dirty, do not touch it because a save is pending to disk and we can not abort it
                if (_this.textFileService.isDirty(_this.element.resource)) {
                    _this.onWarning(nls.localize(20, null, labels_1.getPathLabel(_this.element.resource)));
                    return winjs_base_1.TPromise.as(null);
                }
                // Since a delete operation can take a while we want to emit the event proactively to avoid issues
                // with stale entries in the explorer tree.
                _this.eventService.emit('files.internal:fileChanged', new Files.LocalFileChangeEvent(_this.element.clone(), null));
                // Call function
                var servicePromise = _this.fileService.del(_this.element.resource, _this.useTrash).then(function () {
                    if (_this.element.parent) {
                        _this.tree.setFocus(_this.element.parent); // move focus to parent
                    }
                }, function (error) {
                    // Allow to retry
                    var extraAction;
                    if (_this.useTrash) {
                        extraAction = new actions_1.Action('permanentDelete', nls.localize(21, null), null, true, function () { _this.useTrash = false; _this.skipConfirm = true; return _this.run(); });
                    }
                    _this.onErrorWithRetry(error, function () { return _this.run(); }, extraAction);
                    // Since the delete failed, best we can do is to refresh the explorer from the root to show the current state of files.
                    var event = new Files.LocalFileChangeEvent(new explorerViewModel_1.FileStat(_this.contextService.getWorkspace().resource, true, true), new explorerViewModel_1.FileStat(_this.contextService.getWorkspace().resource, true, true));
                    _this.eventService.emit('files.internal:fileChanged', event);
                    // Focus back to tree
                    _this.tree.DOMFocus();
                });
                return servicePromise;
            });
        };
        BaseDeleteFileAction = __decorate([
            __param(5, workspace_1.IWorkspaceContextService),
            __param(6, editorService_1.IWorkbenchEditorService),
            __param(7, files_1.IFileService),
            __param(8, message_1.IMessageService),
            __param(9, ITextFileService),
            __param(10, event_1.IEventService)
        ], BaseDeleteFileAction);
        return BaseDeleteFileAction;
    }(BaseFileAction));
    exports.BaseDeleteFileAction = BaseDeleteFileAction;
    /* Move File/Folder to trash */
    var MoveFileToTrashAction = (function (_super) {
        __extends(MoveFileToTrashAction, _super);
        function MoveFileToTrashAction(tree, element, contextService, editorService, fileService, messageService, textFileService, eventService) {
            _super.call(this, MoveFileToTrashAction.ID, nls.localize(22, null), tree, element, true, contextService, editorService, fileService, messageService, textFileService, eventService);
        }
        MoveFileToTrashAction.ID = 'workbench.files.action.moveFileToTrash';
        MoveFileToTrashAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, files_1.IFileService),
            __param(5, message_1.IMessageService),
            __param(6, ITextFileService),
            __param(7, event_1.IEventService)
        ], MoveFileToTrashAction);
        return MoveFileToTrashAction;
    }(BaseDeleteFileAction));
    exports.MoveFileToTrashAction = MoveFileToTrashAction;
    /* Delete File/Folder */
    var DeleteFileAction = (function (_super) {
        __extends(DeleteFileAction, _super);
        function DeleteFileAction(tree, element, contextService, editorService, fileService, messageService, textFileService, eventService) {
            _super.call(this, DeleteFileAction.ID, nls.localize(23, null), tree, element, false, contextService, editorService, fileService, messageService, textFileService, eventService);
        }
        DeleteFileAction.ID = 'workbench.files.action.deleteFile';
        DeleteFileAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, files_1.IFileService),
            __param(5, message_1.IMessageService),
            __param(6, ITextFileService),
            __param(7, event_1.IEventService)
        ], DeleteFileAction);
        return DeleteFileAction;
    }(BaseDeleteFileAction));
    exports.DeleteFileAction = DeleteFileAction;
    /* Import File */
    var ImportFileAction = (function (_super) {
        __extends(ImportFileAction, _super);
        function ImportFileAction(tree, element, clazz, contextService, editorService, fileService, messageService, textFileService, eventService, progressService) {
            _super.call(this, ImportFileAction.ID, nls.localize(24, null), contextService, editorService, fileService, messageService, textFileService, eventService);
            this.progressService = progressService;
            this.tree = tree;
            this.element = element;
            if (clazz) {
                this.class = clazz;
            }
            this._updateEnablement();
        }
        ImportFileAction.prototype.getViewer = function () {
            return this.tree;
        };
        ImportFileAction.prototype.run = function (context) {
            var _this = this;
            var multiFileProgressTracker;
            var importPromise = winjs_base_1.TPromise.as(null).then(function () {
                var input = context.input;
                if (input.files && input.files.length > 0) {
                    // Find parent for import
                    var targetElement_1;
                    if (_this.element) {
                        targetElement_1 = _this.element;
                    }
                    else {
                        targetElement_1 = _this.tree.getFocus() || _this.tree.getInput();
                    }
                    if (!targetElement_1.isDirectory) {
                        targetElement_1 = targetElement_1.parent;
                    }
                    // Create real files array
                    var filesArray_1 = [];
                    for (var i = 0; i < input.files.length; i++) {
                        var file = input.files[i];
                        filesArray_1.push(file);
                    }
                    // Resolve target to check for name collisions and ask user
                    return _this.fileService.resolveFile(targetElement_1.resource).then(function (targetStat) {
                        // Check for name collisions
                        var targetNames = {};
                        targetStat.children.forEach(function (child) {
                            targetNames[platform_1.isLinux ? child.name : child.name.toLowerCase()] = child;
                        });
                        var overwrite = true;
                        if (filesArray_1.some(function (file) {
                            return !!targetNames[platform_1.isLinux ? file.name : file.name.toLowerCase()];
                        })) {
                            var confirm_2 = {
                                message: nls.localize(25, null),
                                detail: nls.localize(26, null),
                                primaryButton: nls.localize(27, null)
                            };
                            overwrite = _this.messageService.confirm(confirm_2);
                        }
                        if (!overwrite) {
                            return;
                        }
                        // Progress per file imported
                        if (filesArray_1.length > 1 && _this.progressService) {
                            multiFileProgressTracker = _this.progressService.show(filesArray_1.length);
                        }
                        // Run import in sequence to not consume too many connections
                        var importPromisesFactory = [];
                        filesArray_1.forEach(function (file) {
                            importPromisesFactory.push(function () {
                                return _this.fileService.importFile(uri_1.default.file(file.path), targetElement_1.resource).then(function (result) {
                                    // Progress
                                    if (multiFileProgressTracker) {
                                        multiFileProgressTracker.worked(1);
                                    }
                                    if (result.stat) {
                                        // Emit Deleted Event if file gets replaced
                                        var oldFile = targetNames[platform_1.isLinux ? file.name : file.name.toLowerCase()];
                                        if (oldFile) {
                                            _this.eventService.emit('files.internal:fileChanged', new Files.LocalFileChangeEvent(oldFile, null));
                                        }
                                        // Emit Import Event
                                        _this.eventService.emit('files.internal:fileChanged', new FileImportedEvent(result.stat, result.isNew, context.event));
                                    }
                                }, function (error) {
                                    _this.messageService.show(message_1.Severity.Error, error);
                                });
                            });
                        });
                        return async_1.sequence(importPromisesFactory);
                    });
                }
            });
            if (this.progressService && !multiFileProgressTracker) {
                this.progressService.showWhile(importPromise, 800);
            }
            return importPromise.then(function () {
                _this.tree.clearHighlight();
            }, function (error) {
                _this.onError(error);
                _this.tree.clearHighlight();
            });
        };
        ImportFileAction.ID = 'workbench.files.action.importFile';
        ImportFileAction = __decorate([
            __param(3, workspace_1.IWorkspaceContextService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, files_1.IFileService),
            __param(6, message_1.IMessageService),
            __param(7, ITextFileService),
            __param(8, event_1.IEventService),
            __param(9, progress_1.IProgressService)
        ], ImportFileAction);
        return ImportFileAction;
    }(BaseFileAction));
    exports.ImportFileAction = ImportFileAction;
    /** File import event is emitted when a file is import into the workbench. */
    var FileImportedEvent = (function (_super) {
        __extends(FileImportedEvent, _super);
        function FileImportedEvent(stat, isNew, originalEvent) {
            _super.call(this, null, stat, originalEvent);
            this.isNew = isNew;
        }
        FileImportedEvent.prototype.gotAdded = function () {
            return this.isNew;
        };
        FileImportedEvent.prototype.gotMoved = function () {
            return false;
        };
        FileImportedEvent.prototype.gotUpdated = function () {
            return !this.isNew;
        };
        FileImportedEvent.prototype.gotDeleted = function () {
            return false;
        };
        return FileImportedEvent;
    }(Files.LocalFileChangeEvent));
    exports.FileImportedEvent = FileImportedEvent;
    // Copy File/Folder
    var fileToCopy;
    var CopyFileAction = (function (_super) {
        __extends(CopyFileAction, _super);
        function CopyFileAction(tree, element, contextService, editorService, fileService, messageService, textFileService, eventService) {
            _super.call(this, CopyFileAction.ID, nls.localize(28, null), contextService, editorService, fileService, messageService, textFileService, eventService);
            this.tree = tree;
            this.element = element;
            this._updateEnablement();
        }
        CopyFileAction.prototype.run = function () {
            // Remember as file/folder to copy
            fileToCopy = this.element;
            // Remove highlight
            if (this.tree) {
                this.tree.clearHighlight();
            }
            this.tree.DOMFocus();
            return winjs_base_1.TPromise.as(null);
        };
        CopyFileAction.ID = 'workbench.files.action.copyFile';
        CopyFileAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, files_1.IFileService),
            __param(5, message_1.IMessageService),
            __param(6, ITextFileService),
            __param(7, event_1.IEventService)
        ], CopyFileAction);
        return CopyFileAction;
    }(BaseFileAction));
    exports.CopyFileAction = CopyFileAction;
    // Paste File/Folder
    var PasteFileAction = (function (_super) {
        __extends(PasteFileAction, _super);
        function PasteFileAction(tree, element, contextService, editorService, fileService, messageService, textFileService, eventService, instantiationService) {
            _super.call(this, PasteFileAction.ID, nls.localize(29, null), contextService, editorService, fileService, messageService, textFileService, eventService);
            this.instantiationService = instantiationService;
            this.tree = tree;
            this.element = element;
            this._updateEnablement();
        }
        PasteFileAction.prototype._isEnabled = function () {
            // Need at least a file to copy
            if (!fileToCopy) {
                return false;
            }
            // Check if file was deleted or moved meanwhile
            var root = this.tree.getInput();
            var exists = root.find(fileToCopy.resource);
            if (!exists) {
                fileToCopy = null;
                return false;
            }
            // Check if target is ancestor of pasted folder
            if (this.element.resource.toString() !== fileToCopy.resource.toString() && paths.isEqualOrParent(this.element.resource.fsPath, fileToCopy.resource.fsPath)) {
                return false;
            }
            return true;
        };
        PasteFileAction.prototype.run = function () {
            var _this = this;
            // Find target
            var target;
            if (this.element.resource.toString() === fileToCopy.resource.toString()) {
                target = this.element.parent;
            }
            else {
                target = this.element.isDirectory ? this.element : this.element.parent;
            }
            // Reuse duplicate action
            var pasteAction = this.instantiationService.createInstance(DuplicateFileAction, this.tree, fileToCopy, target);
            return pasteAction.run().then(function () {
                _this.tree.DOMFocus();
            });
        };
        PasteFileAction.ID = 'workbench.files.action.pasteFile';
        PasteFileAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, files_1.IFileService),
            __param(5, message_1.IMessageService),
            __param(6, ITextFileService),
            __param(7, event_1.IEventService),
            __param(8, instantiation_1.IInstantiationService)
        ], PasteFileAction);
        return PasteFileAction;
    }(BaseFileAction));
    exports.PasteFileAction = PasteFileAction;
    // Duplicate File/Folder
    var DuplicateFileAction = (function (_super) {
        __extends(DuplicateFileAction, _super);
        function DuplicateFileAction(tree, element, target, contextService, editorService, fileService, messageService, textFileService, eventService, progressService) {
            _super.call(this, 'workbench.files.action.duplicateFile', nls.localize(30, null), contextService, editorService, fileService, messageService, textFileService, eventService);
            this.progressService = progressService;
            this.tree = tree;
            this.element = element;
            this.target = (target && target.isDirectory) ? target : element.parent;
            this._updateEnablement();
        }
        DuplicateFileAction.prototype.run = function () {
            var _this = this;
            // Remove highlight
            if (this.tree) {
                this.tree.clearHighlight();
            }
            // Copy File and emit event
            var result = this.fileService.copyFile(this.element.resource, this.findTarget()).then(function (stat) {
                _this.eventService.emit('files.internal:fileChanged', new Files.LocalFileChangeEvent(null, stat));
            }, function (error) {
                _this.onError(error);
            });
            if (this.progressService) {
                this.progressService.showWhile(result, 800);
            }
            return result;
        };
        DuplicateFileAction.prototype.onError = function (error) {
            this.messageService.show(message_1.Severity.Error, error);
        };
        DuplicateFileAction.prototype.findTarget = function () {
            var root = this.tree.getInput();
            var name = this.element.name;
            var candidate = uri_1.default.file(paths.join(this.target.resource.fsPath, name));
            while (true) {
                if (!root.find(candidate)) {
                    break;
                }
                name = this.toCopyName(name, this.element.isDirectory);
                candidate = uri_1.default.file(paths.join(this.target.resource.fsPath, name));
            }
            return candidate;
        };
        DuplicateFileAction.prototype.toCopyName = function (name, isFolder) {
            // file.1.txt=>file.2.txt
            if (!isFolder && name.match(/(\d+)(\..*)$/)) {
                return name.replace(/(\d+)(\..*)$/, function (match, g1, g2) { return (parseInt(g1) + 1) + g2; });
            }
            // file.txt=>file.1.txt
            var lastIndexOfDot = name.lastIndexOf('.');
            if (!isFolder && lastIndexOfDot >= 0) {
                return strings.format('{0}.1{1}', name.substr(0, lastIndexOfDot), name.substr(lastIndexOfDot));
            }
            // folder.1=>folder.2
            if (isFolder && name.match(/(\d+)$/)) {
                return name.replace(/(\d+)$/, function (match) {
                    var groups = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        groups[_i - 1] = arguments[_i];
                    }
                    return String(parseInt(groups[0]) + 1);
                });
            }
            // file/folder=>file.1/folder.1
            return strings.format('{0}.1', name);
        };
        DuplicateFileAction = __decorate([
            __param(3, workspace_1.IWorkspaceContextService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, files_1.IFileService),
            __param(6, message_1.IMessageService),
            __param(7, ITextFileService),
            __param(8, event_1.IEventService),
            __param(9, progress_1.IProgressService)
        ], DuplicateFileAction);
        return DuplicateFileAction;
    }(BaseFileAction));
    exports.DuplicateFileAction = DuplicateFileAction;
    // Open to the side
    var OpenToSideAction = (function (_super) {
        __extends(OpenToSideAction, _super);
        function OpenToSideAction(tree, resource, preserveFocus, editorService) {
            _super.call(this, OpenToSideAction.ID, OpenToSideAction.LABEL);
            this.editorService = editorService;
            this.tree = tree;
            this.preserveFocus = preserveFocus;
            this.resource = resource;
            this.updateEnablement();
        }
        OpenToSideAction.prototype.updateEnablement = function () {
            var activeEditor = this.editorService.getActiveEditor();
            this.enabled = (!activeEditor || activeEditor.position !== editor_2.Position.RIGHT);
        };
        OpenToSideAction.prototype.run = function () {
            // Remove highlight
            this.tree.clearHighlight();
            // Set side input
            return this.editorService.openEditor({
                resource: this.resource,
                options: {
                    preserveFocus: this.preserveFocus
                }
            }, true);
        };
        OpenToSideAction.ID = 'workbench.files.action.openToSide';
        OpenToSideAction.LABEL = nls.localize(31, null);
        OpenToSideAction = __decorate([
            __param(3, editorService_1.IWorkbenchEditorService)
        ], OpenToSideAction);
        return OpenToSideAction;
    }(actions_1.Action));
    exports.OpenToSideAction = OpenToSideAction;
    var globalResourceToCompare;
    var SelectResourceForCompareAction = (function (_super) {
        __extends(SelectResourceForCompareAction, _super);
        function SelectResourceForCompareAction(resource, tree) {
            _super.call(this, 'workbench.files.action.selectForCompare', nls.localize(32, null));
            this.tree = tree;
            this.resource = resource;
            this.enabled = true;
        }
        SelectResourceForCompareAction.prototype.run = function () {
            // Remember as source file to compare
            globalResourceToCompare = this.resource;
            // Remove highlight
            if (this.tree) {
                this.tree.clearHighlight();
                this.tree.DOMFocus();
            }
            return winjs_base_1.TPromise.as(null);
        };
        return SelectResourceForCompareAction;
    }(actions_1.Action));
    exports.SelectResourceForCompareAction = SelectResourceForCompareAction;
    // Global Compare with
    var GlobalCompareResourcesAction = (function (_super) {
        __extends(GlobalCompareResourcesAction, _super);
        function GlobalCompareResourcesAction(id, label, quickOpenService, instantiationService, editorService, messageService, eventService) {
            _super.call(this, id, label);
            this.quickOpenService = quickOpenService;
            this.instantiationService = instantiationService;
            this.editorService = editorService;
            this.messageService = messageService;
            this.eventService = eventService;
        }
        GlobalCompareResourcesAction.prototype.run = function () {
            var _this = this;
            var fileInput = editor_1.asFileEditorInput(this.editorService.getActiveEditorInput());
            if (fileInput) {
                // Keep as resource to compare
                globalResourceToCompare = fileInput.getResource();
                // Listen for next editor to open
                var unbind_1 = this.eventService.addListener(events_2.EventType.EDITOR_INPUT_OPENING, function (e) {
                    unbind_1(); // listen once
                    var otherFileInput = editor_1.asFileEditorInput(e.editorInput);
                    if (otherFileInput) {
                        var compareAction_1 = _this.instantiationService.createInstance(CompareResourcesAction, otherFileInput.getResource(), null);
                        if (compareAction_1._isEnabled()) {
                            e.prevent();
                            compareAction_1.run().done(function () { return compareAction_1.dispose(); });
                        }
                        else {
                            _this.messageService.show(message_1.Severity.Info, nls.localize(34, null, paths.basename(globalResourceToCompare.fsPath)));
                        }
                    }
                });
                // Bring up quick open
                this.quickOpenService.show().then(function () {
                    unbind_1(); // make sure to unbind if quick open is closing
                });
            }
            else {
                this.messageService.show(message_1.Severity.Info, nls.localize(35, null));
            }
            return winjs_base_1.TPromise.as(true);
        };
        GlobalCompareResourcesAction.ID = 'workbench.files.action.compareFileWith';
        GlobalCompareResourcesAction.LABEL = nls.localize(33, null);
        GlobalCompareResourcesAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService),
            __param(3, instantiation_1.IInstantiationService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, message_1.IMessageService),
            __param(6, event_1.IEventService)
        ], GlobalCompareResourcesAction);
        return GlobalCompareResourcesAction;
    }(actions_1.Action));
    exports.GlobalCompareResourcesAction = GlobalCompareResourcesAction;
    // Compare with Resource
    var CompareResourcesAction = (function (_super) {
        __extends(CompareResourcesAction, _super);
        function CompareResourcesAction(resource, tree, contextService, instantiationService, editorService) {
            _super.call(this, 'workbench.files.action.compareFiles', CompareResourcesAction.computeLabel());
            this.contextService = contextService;
            this.instantiationService = instantiationService;
            this.editorService = editorService;
            this.tree = tree;
            this.resource = resource;
        }
        CompareResourcesAction.computeLabel = function () {
            if (globalResourceToCompare) {
                return nls.localize(36, null, paths.basename(globalResourceToCompare.fsPath));
            }
            return nls.localize(37, null);
        };
        CompareResourcesAction.prototype.getLabel = function () {
            return CompareResourcesAction.computeLabel();
        };
        CompareResourcesAction.prototype._isEnabled = function () {
            // Need at least a resource to compare
            if (!globalResourceToCompare) {
                return false;
            }
            // Check if file was deleted or moved meanwhile (explorer only)
            if (this.tree) {
                var root = this.tree.getInput();
                if (root instanceof explorerViewModel_1.FileStat) {
                    var exists = root.find(globalResourceToCompare);
                    if (!exists) {
                        globalResourceToCompare = null;
                        return false;
                    }
                }
            }
            // Check if target is identical to source
            if (this.resource.toString() === globalResourceToCompare.toString()) {
                return false;
            }
            var mimeA = mime_1.guessMimeTypes(this.resource.fsPath).join(', ');
            var mimeB = mime_1.guessMimeTypes(globalResourceToCompare.fsPath).join(', ');
            // Check if target has same mime
            if (mimeA === mimeB) {
                return true;
            }
            // Ensure the mode is equal if this is text (limitation of current diff infrastructure)
            var isBinaryA = mime_1.isBinaryMime(mimeA);
            var isBinaryB = mime_1.isBinaryMime(mimeB);
            // Ensure we are not comparing binary with text
            if (isBinaryA !== isBinaryB) {
                return false;
            }
            return true;
        };
        CompareResourcesAction.prototype.run = function () {
            // Remove highlight
            if (this.tree) {
                this.tree.clearHighlight();
            }
            var leftInput = this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, globalResourceToCompare, void 0, void 0);
            var rightInput = this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, this.resource, void 0, void 0);
            return this.editorService.openEditor(new diffEditorInput_1.DiffEditorInput(diffEditorInput_1.toDiffLabel(globalResourceToCompare, this.resource, this.contextService), null, leftInput, rightInput));
        };
        CompareResourcesAction = __decorate([
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, instantiation_1.IInstantiationService),
            __param(4, editorService_1.IWorkbenchEditorService)
        ], CompareResourcesAction);
        return CompareResourcesAction;
    }(actions_1.Action));
    exports.CompareResourcesAction = CompareResourcesAction;
    // Refresh Explorer Viewer
    var RefreshViewExplorerAction = (function (_super) {
        __extends(RefreshViewExplorerAction, _super);
        function RefreshViewExplorerAction(explorerView, clazz) {
            _super.call(this, 'workbench.files.action.refreshExplorer', nls.localize(38, null), clazz, true, function (context) {
                if (explorerView.getViewer().getHighlight()) {
                    return winjs_base_1.TPromise.as(null); // Global action disabled if user is in edit mode from another action
                }
                explorerView.focusBody();
                return explorerView.refresh(true, true, true);
            });
        }
        return RefreshViewExplorerAction;
    }(actions_1.Action));
    exports.RefreshViewExplorerAction = RefreshViewExplorerAction;
    var BaseActionWithErrorReporting = (function (_super) {
        __extends(BaseActionWithErrorReporting, _super);
        function BaseActionWithErrorReporting(id, label, messageService) {
            _super.call(this, id, label);
            this.messageService = messageService;
        }
        BaseActionWithErrorReporting.prototype.run = function () {
            var _this = this;
            return this.doRun().then(function () { return true; }, function (error) {
                _this.messageService.show(message_1.Severity.Error, errors.toErrorMessage(error, false));
            });
        };
        return BaseActionWithErrorReporting;
    }(actions_1.Action));
    exports.BaseActionWithErrorReporting = BaseActionWithErrorReporting;
    var BaseSaveFileAction = (function (_super) {
        __extends(BaseSaveFileAction, _super);
        function BaseSaveFileAction(id, label, editorService, textFileService, untitledEditorService, instantiationService, messageService) {
            _super.call(this, id, label, messageService);
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.untitledEditorService = untitledEditorService;
            this.instantiationService = instantiationService;
            this.enabled = true;
        }
        BaseSaveFileAction.prototype.setResource = function (resource) {
            this.resource = resource;
        };
        BaseSaveFileAction.prototype.doRun = function () {
            var _this = this;
            var source;
            if (this.resource) {
                source = this.resource;
            }
            else {
                source = editor_1.getUntitledOrFileResource(this.editorService.getActiveEditorInput(), true);
            }
            if (source) {
                // Save As (or Save untitled with associated path)
                if (this.isSaveAs() || source.scheme === 'untitled') {
                    var positionsOfSource_1 = findSaveAsPositions(this.editorService, source);
                    var mimeOfSource_1;
                    if (source.scheme === 'untitled') {
                        var selectedMime = this.untitledEditorService.get(source).getMime();
                        if (!mime_1.isUnspecific(selectedMime)) {
                            mimeOfSource_1 = [selectedMime, mime_1.MIME_TEXT].join(', ');
                        }
                    }
                    var encodingOfSource_1;
                    if (source.scheme === 'untitled') {
                        encodingOfSource_1 = this.untitledEditorService.get(source).getEncoding();
                    }
                    else if (source.scheme === 'file') {
                        var textModel = textFileEditorModel_1.CACHE.get(source);
                        encodingOfSource_1 = textModel && textModel.getEncoding(); // text model can be null e.g. if this is a binary file!
                    }
                    var selectionOfSource_1;
                    if (positionsOfSource_1.length) {
                        var activeEditor = this.editorService.getActiveEditor();
                        if (activeEditor && positionsOfSource_1.indexOf(activeEditor.position) >= 0) {
                            selectionOfSource_1 = activeEditor.getSelection();
                        }
                    }
                    // Special case: an untitled file with associated path gets saved directly unless "saveAs" is true
                    var savePromise = void 0;
                    if (!this.isSaveAs() && source.scheme === 'untitled' && this.untitledEditorService.hasAssociatedFilePath(source)) {
                        savePromise = this.textFileService.save(source).then(function (result) {
                            if (result) {
                                return uri_1.default.file(source.fsPath);
                            }
                            return null;
                        });
                    }
                    else {
                        savePromise = this.textFileService.saveAs(source);
                    }
                    return savePromise.then(function (target) {
                        if (!target) {
                            return;
                        }
                        // Reopen editors for the resource based on the positions
                        var reopenPromise = winjs_base_1.TPromise.as(null);
                        if (target.toString() !== source.toString() && positionsOfSource_1.length) {
                            var targetInput_1 = _this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, target, mimeOfSource_1, encodingOfSource_1);
                            var options_1;
                            if (selectionOfSource_1) {
                                options_1 = new editor_1.TextEditorOptions();
                                options_1.selection(selectionOfSource_1.startLineNumber, selectionOfSource_1.startColumn, selectionOfSource_1.endLineNumber, selectionOfSource_1.endColumn);
                            }
                            reopenPromise = _this.editorService.openEditor(targetInput_1, options_1, positionsOfSource_1[0]).then(function () {
                                if (positionsOfSource_1.length > 1) {
                                    return _this.editorService.openEditor(targetInput_1, options_1, positionsOfSource_1[1]).then(function () {
                                        if (positionsOfSource_1.length > 2) {
                                            return _this.editorService.openEditor(targetInput_1, options_1, positionsOfSource_1[2]);
                                        }
                                    });
                                }
                            });
                        }
                        return reopenPromise;
                    });
                }
                // Just save
                return this.textFileService.save(source);
            }
            return winjs_base_1.TPromise.as(false);
        };
        BaseSaveFileAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, ITextFileService),
            __param(4, untitledEditorService_1.IUntitledEditorService),
            __param(5, instantiation_1.IInstantiationService),
            __param(6, message_1.IMessageService)
        ], BaseSaveFileAction);
        return BaseSaveFileAction;
    }(BaseActionWithErrorReporting));
    exports.BaseSaveFileAction = BaseSaveFileAction;
    var SaveFileAction = (function (_super) {
        __extends(SaveFileAction, _super);
        function SaveFileAction() {
            _super.apply(this, arguments);
        }
        SaveFileAction.prototype.isSaveAs = function () {
            return false;
        };
        SaveFileAction.ID = 'workbench.action.files.save';
        SaveFileAction.LABEL = nls.localize(39, null);
        return SaveFileAction;
    }(BaseSaveFileAction));
    exports.SaveFileAction = SaveFileAction;
    var SaveFileAsAction = (function (_super) {
        __extends(SaveFileAsAction, _super);
        function SaveFileAsAction() {
            _super.apply(this, arguments);
        }
        SaveFileAsAction.prototype.isSaveAs = function () {
            return true;
        };
        SaveFileAsAction.ID = 'workbench.action.files.saveAs';
        SaveFileAsAction.LABEL = 'Save As...';
        return SaveFileAsAction;
    }(BaseSaveFileAction));
    exports.SaveFileAsAction = SaveFileAsAction;
    var BaseSaveAllAction = (function (_super) {
        __extends(BaseSaveAllAction, _super);
        function BaseSaveAllAction(id, label, editorService, textFileService, untitledEditorService, instantiationService, eventService, messageService) {
            _super.call(this, id, label, messageService);
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.untitledEditorService = untitledEditorService;
            this.instantiationService = instantiationService;
            this.eventService = eventService;
            this.toDispose = [];
            this.lastIsDirty = this.textFileService.isDirty();
            this.enabled = this.lastIsDirty;
            this.registerListeners();
        }
        BaseSaveAllAction.prototype.registerListeners = function () {
            var _this = this;
            // listen to files being changed locally
            this.toDispose.push(this.eventService.addListener2(Files.EventType.FILE_DIRTY, function (e) { return _this.updateEnablement(true); }));
            this.toDispose.push(this.eventService.addListener2(Files.EventType.FILE_SAVED, function (e) { return _this.updateEnablement(false); }));
            this.toDispose.push(this.eventService.addListener2(Files.EventType.FILE_REVERTED, function (e) { return _this.updateEnablement(false); }));
            this.toDispose.push(this.eventService.addListener2(Files.EventType.FILE_SAVE_ERROR, function (e) { return _this.updateEnablement(true); }));
            if (this.includeUntitled()) {
                this.toDispose.push(this.eventService.addListener2(events_2.EventType.UNTITLED_FILE_DIRTY, function () { return _this.updateEnablement(true); }));
                this.toDispose.push(this.eventService.addListener2(events_2.EventType.UNTITLED_FILE_DELETED, function () { return _this.updateEnablement(false); }));
            }
        };
        BaseSaveAllAction.prototype.updateEnablement = function (isDirty) {
            if (this.lastIsDirty !== isDirty) {
                this.enabled = this.textFileService.isDirty();
                this.lastIsDirty = this.enabled;
            }
        };
        BaseSaveAllAction.prototype.doRun = function () {
            var _this = this;
            // Store mimes per untitled file to restore later
            var mapUntitledToProperties = Object.create(null);
            this.textFileService.getDirty()
                .filter(function (r) { return r.scheme === 'untitled'; }) // All untitled resources^
                .map(function (r) { return _this.untitledEditorService.get(r); }) // Mapped to their inputs
                .filter(function (i) { return !!i; }) // If possible :)
                .forEach(function (i) { return mapUntitledToProperties[i.getResource().toString()] = { mime: i.getMime(), encoding: i.getEncoding() }; });
            // Save all
            return this.textFileService.saveAll(this.includeUntitled()).then(function (result) {
                // all saved - now try to reopen saved untitled ones
                if (_this.includeUntitled()) {
                    var untitledResults = result.results.filter(function (res) { return res.source.scheme === 'untitled'; });
                    var reopenPromises_1 = [];
                    // Create a promise function for each editor open call to reopen
                    untitledResults.forEach(function (res) {
                        if (res.success) {
                            var positions = findSaveAsPositions(_this.editorService, res.source);
                            var mimeOfSource = void 0;
                            var selectedMime = mapUntitledToProperties[res.source.toString()] && mapUntitledToProperties[res.source.toString()].mime;
                            if (!mime_1.isUnspecific(selectedMime)) {
                                mimeOfSource = [selectedMime, mime_1.MIME_TEXT].join(', ');
                            }
                            var encodingOfSource = mapUntitledToProperties[res.source.toString()] && mapUntitledToProperties[res.source.toString()].encoding;
                            var targetInput_2 = _this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, res.target, mimeOfSource, encodingOfSource);
                            var options_2 = new editor_1.EditorOptions();
                            options_2.preserveFocus = true;
                            positions.forEach(function (position) {
                                reopenPromises_1.push(function () {
                                    return _this.editorService.openEditor(targetInput_2, options_2, position);
                                });
                            });
                        }
                    });
                    // Build a promise that completes when reopen is done
                    var reopenPromise = winjs_base_1.TPromise.as(null);
                    if (reopenPromises_1.length) {
                        reopenPromise = reopenPromises_1[0]().then(function () {
                            if (reopenPromises_1.length > 1) {
                                return reopenPromises_1[1]().then(function () {
                                    if (reopenPromises_1.length > 2) {
                                        return reopenPromises_1[2]();
                                    }
                                });
                            }
                        });
                    }
                    return reopenPromise;
                }
            });
        };
        BaseSaveAllAction.prototype.dispose = function () {
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            _super.prototype.dispose.call(this);
        };
        BaseSaveAllAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, ITextFileService),
            __param(4, untitledEditorService_1.IUntitledEditorService),
            __param(5, instantiation_1.IInstantiationService),
            __param(6, event_1.IEventService),
            __param(7, message_1.IMessageService)
        ], BaseSaveAllAction);
        return BaseSaveAllAction;
    }(BaseActionWithErrorReporting));
    exports.BaseSaveAllAction = BaseSaveAllAction;
    function findSaveAsPositions(editorService, outerResource) {
        var activeInput = editorService.getActiveEditorInput();
        return editorService.getVisibleEditors().filter(function (editor) {
            if (outerResource.scheme === 'file' && activeInput !== editor.input) {
                return false; // skip non active if this is about a file; for untitled respect them all
            }
            var innerResource = editor_1.getUntitledOrFileResource(editor.input);
            return innerResource && innerResource.toString() === outerResource.toString();
        }).map(function (editor) { return editor.position; });
    }
    var SaveAllAction = (function (_super) {
        __extends(SaveAllAction, _super);
        function SaveAllAction() {
            _super.apply(this, arguments);
        }
        Object.defineProperty(SaveAllAction.prototype, "class", {
            get: function () {
                return 'explorer-action save-all';
            },
            enumerable: true,
            configurable: true
        });
        SaveAllAction.prototype.includeUntitled = function () {
            return true;
        };
        SaveAllAction.ID = 'workbench.action.files.saveAll';
        SaveAllAction.LABEL = nls.localize(40, null);
        return SaveAllAction;
    }(BaseSaveAllAction));
    exports.SaveAllAction = SaveAllAction;
    var SaveFilesAction = (function (_super) {
        __extends(SaveFilesAction, _super);
        function SaveFilesAction() {
            _super.apply(this, arguments);
        }
        SaveFilesAction.prototype.includeUntitled = function () {
            return false;
        };
        SaveFilesAction.ID = 'workbench.action.files.saveFiles';
        SaveFilesAction.LABEL = nls.localize(41, null);
        return SaveFilesAction;
    }(BaseSaveAllAction));
    exports.SaveFilesAction = SaveFilesAction;
    var RevertFileAction = (function (_super) {
        __extends(RevertFileAction, _super);
        function RevertFileAction(id, label, editorService, textFileService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.enabled = true;
        }
        RevertFileAction.prototype.setResource = function (resource) {
            this.resource = resource;
        };
        RevertFileAction.prototype.run = function () {
            var resource;
            if (this.resource) {
                resource = this.resource;
            }
            else {
                var activeFileInput = editor_1.asFileEditorInput(this.editorService.getActiveEditorInput(), true);
                if (activeFileInput) {
                    resource = activeFileInput.getResource();
                }
            }
            if (resource && resource.scheme !== 'untitled') {
                return this.textFileService.revert(resource, true /* force */);
            }
            return winjs_base_1.TPromise.as(true);
        };
        RevertFileAction.ID = 'workbench.action.files.revert';
        RevertFileAction.LABEL = nls.localize(42, null);
        RevertFileAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, ITextFileService)
        ], RevertFileAction);
        return RevertFileAction;
    }(actions_1.Action));
    exports.RevertFileAction = RevertFileAction;
    var OpenResourcesAction = (function (_super) {
        __extends(OpenResourcesAction, _super);
        function OpenResourcesAction(resources, diffMode, partService, editorService, viewletService, textFileService, contextService) {
            _super.call(this, 'workbench.files.action.openResourcesAction');
            this.partService = partService;
            this.editorService = editorService;
            this.viewletService = viewletService;
            this.textFileService = textFileService;
            this.contextService = contextService;
            this.resources = resources;
            this.diffMode = diffMode;
        }
        OpenResourcesAction.prototype.run = function () {
            var _this = this;
            return this.partService.joinCreation().then(function () {
                var viewletPromise = winjs_base_1.TPromise.as(null);
                if (!_this.partService.isSideBarHidden()) {
                    viewletPromise = _this.viewletService.openViewlet(Files.VIEWLET_ID, false);
                }
                return viewletPromise.then(function () {
                    // Out of workspace files get added right away to working files model
                    _this.resources.forEach(function (fileToOpen) {
                        var resource = fileToOpen.resource;
                        var workspace = _this.contextService.getWorkspace();
                        if (!workspace || !paths.isEqualOrParent(resource.fsPath, workspace.resource.fsPath)) {
                            _this.textFileService.getWorkingFilesModel().addEntry(resource);
                        }
                    });
                    // In diffMode we open 2 resources as diff
                    if (_this.diffMode) {
                        return winjs_base_1.TPromise.join(_this.resources.map(function (f) { return _this.editorService.inputToType(f); })).then(function (inputs) {
                            return _this.editorService.openEditor(new diffEditorInput_1.DiffEditorInput(diffEditorInput_1.toDiffLabel(_this.resources[0].resource, _this.resources[1].resource, _this.contextService), null, inputs[0], inputs[1]));
                        });
                    }
                    // For one file, just put it into the current active editor
                    if (_this.resources.length === 1) {
                        return _this.editorService.openEditor(_this.resources[0]);
                    }
                    // Otherwise replace all
                    return _this.editorService.setEditors(_this.resources);
                });
            });
        };
        OpenResourcesAction = __decorate([
            __param(2, partService_1.IPartService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, viewletService_1.IViewletService),
            __param(5, ITextFileService),
            __param(6, workspace_1.IWorkspaceContextService)
        ], OpenResourcesAction);
        return OpenResourcesAction;
    }(actions_1.Action));
    exports.OpenResourcesAction = OpenResourcesAction;
    var BaseCloseWorkingFileAction = (function (_super) {
        __extends(BaseCloseWorkingFileAction, _super);
        function BaseCloseWorkingFileAction(id, label, clazz, model, elements, untitledEditorService, editorService, textFileService, messageService, quickOpenService) {
            _super.call(this, id, label, clazz);
            this.untitledEditorService = untitledEditorService;
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.messageService = messageService;
            this.quickOpenService = quickOpenService;
            this.model = model;
            this.elements = elements ? elements.map(function (e) { return e.resource; }) : void 0 /* all */;
        }
        BaseCloseWorkingFileAction.prototype.run = function () {
            var _this = this;
            var workingFilesCount = this.model.getEntries().length;
            // Handle dirty
            var isDirty;
            if (this.elements) {
                isDirty = this.elements.some(function (e) { return _this.textFileService.isDirty(e); });
            }
            else {
                isDirty = this.textFileService.isDirty();
            }
            var saveOrRevertPromise = winjs_base_1.TPromise.as(null);
            if (isDirty) {
                var confirmResult = this.textFileService.confirmSave(this.elements);
                switch (confirmResult) {
                    case Files.ConfirmResult.SAVE:
                        if (this.elements) {
                            saveOrRevertPromise = this.textFileService.saveAll(this.elements);
                        }
                        else {
                            saveOrRevertPromise = this.textFileService.saveAll(true /* include untitled */);
                        }
                        break;
                    case Files.ConfirmResult.DONT_SAVE:
                        if (this.elements) {
                            saveOrRevertPromise = this.textFileService.revertAll(this.elements);
                        }
                        else {
                            saveOrRevertPromise = this.textFileService.revertAll();
                        }
                        break;
                    case Files.ConfirmResult.CANCEL:
                        return winjs_base_1.TPromise.as(null);
                }
            }
            return saveOrRevertPromise.then(function (result) {
                // Collect resources to dispose
                var resourcesToDispose = [];
                if (_this.elements) {
                    resourcesToDispose = _this.elements;
                }
                else {
                    resourcesToDispose = _this.model.getEntries().map(function (e) { return e.resource; });
                }
                // Remove those that failed from the save/revert if we had it
                if (result) {
                    var failed_1 = result.results.filter(function (r) { return !r.success; }).map(function (r) { return r.source.toString(); });
                    resourcesToDispose = resourcesToDispose.filter(function (r) { return failed_1.indexOf(r.toString()) < 0; });
                }
                // remove from model
                if (resourcesToDispose.length === workingFilesCount) {
                    _this.model.clear();
                }
                else {
                    resourcesToDispose.forEach(function (r) { return _this.model.removeEntry(r); });
                }
                // dispose
                resourcesToDispose.forEach(function (r) { return _this.disposeResource(r); });
            }, function (error) {
                _this.messageService.show(message_1.Severity.Error, error);
            });
        };
        BaseCloseWorkingFileAction.prototype.disposeResource = function (resource) {
            // file inputs
            fileEditorInputsForResource(resource, this.editorService, this.quickOpenService).forEach(function (input) {
                if (!input.isDisposed()) {
                    input.dispose(true);
                }
            });
            // untitled inputs
            var input = this.untitledEditorService.get(resource);
            if (input) {
                input.dispose();
            }
        };
        BaseCloseWorkingFileAction = __decorate([
            __param(5, untitledEditorService_1.IUntitledEditorService),
            __param(6, editorService_1.IWorkbenchEditorService),
            __param(7, ITextFileService),
            __param(8, message_1.IMessageService),
            __param(9, quickOpenService_1.IQuickOpenService)
        ], BaseCloseWorkingFileAction);
        return BaseCloseWorkingFileAction;
    }(actions_1.Action));
    exports.BaseCloseWorkingFileAction = BaseCloseWorkingFileAction;
    var CloseAllWorkingFilesAction = (function (_super) {
        __extends(CloseAllWorkingFilesAction, _super);
        function CloseAllWorkingFilesAction(model, untitledEditorService, editorService, textFileService, messageService, quickOpenService) {
            _super.call(this, CloseAllWorkingFilesAction.ID, nls.localize(43, null), 'action-close-all-files', model, null, untitledEditorService, editorService, textFileService, messageService, quickOpenService);
            this.enabled = (model.count() > 0);
            this.listenerToDispose = model.onModelChange(this.onModelChange, this);
        }
        CloseAllWorkingFilesAction.prototype.run = function () {
            var _this = this;
            return _super.prototype.run.call(this).then(function () { return closeNonFileEditors(_this.editorService); }); // close non file editors too
        };
        CloseAllWorkingFilesAction.prototype.onModelChange = function (event) {
            this.enabled = (this.model.count() > 0);
        };
        CloseAllWorkingFilesAction.prototype.dispose = function () {
            if (this.listenerToDispose) {
                this.listenerToDispose.dispose();
                this.listenerToDispose = null;
            }
            _super.prototype.dispose.call(this);
        };
        CloseAllWorkingFilesAction.ID = 'workbench.files.action.closeAllWorkingFiles';
        CloseAllWorkingFilesAction = __decorate([
            __param(1, untitledEditorService_1.IUntitledEditorService),
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, ITextFileService),
            __param(4, message_1.IMessageService),
            __param(5, quickOpenService_1.IQuickOpenService)
        ], CloseAllWorkingFilesAction);
        return CloseAllWorkingFilesAction;
    }(BaseCloseWorkingFileAction));
    exports.CloseAllWorkingFilesAction = CloseAllWorkingFilesAction;
    var CloseOneWorkingFileAction = (function (_super) {
        __extends(CloseOneWorkingFileAction, _super);
        function CloseOneWorkingFileAction(model, element, untitledEditorService, editorService, textFileService, messageService, quickOpenService) {
            _super.call(this, CloseOneWorkingFileAction.ID, nls.localize(44, null), element.dirty ? 'action-close-dirty-file' : 'action-close-file', model, [element], untitledEditorService, editorService, textFileService, messageService, quickOpenService);
        }
        CloseOneWorkingFileAction.ID = 'workbench.files.action.closeOneWorkingFile';
        CloseOneWorkingFileAction = __decorate([
            __param(2, untitledEditorService_1.IUntitledEditorService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, ITextFileService),
            __param(5, message_1.IMessageService),
            __param(6, quickOpenService_1.IQuickOpenService)
        ], CloseOneWorkingFileAction);
        return CloseOneWorkingFileAction;
    }(BaseCloseWorkingFileAction));
    exports.CloseOneWorkingFileAction = CloseOneWorkingFileAction;
    var CloseOtherWorkingFilesAction = (function (_super) {
        __extends(CloseOtherWorkingFilesAction, _super);
        function CloseOtherWorkingFilesAction(model, element, untitledEditorService, editorService, textFileService, messageService, quickOpenService) {
            _super.call(this, CloseOtherWorkingFilesAction.ID, nls.localize(45, null), 'action-close-file', model, model.getEntries().filter(function (e) { return e !== element; }), untitledEditorService, editorService, textFileService, messageService, quickOpenService);
        }
        CloseOtherWorkingFilesAction.prototype.run = function () {
            var _this = this;
            return _super.prototype.run.call(this).then(function () { return closeNonFileEditors(_this.editorService); }); // close non file editors too
        };
        CloseOtherWorkingFilesAction.ID = 'workbench.files.action.closeOtherWorkingFiles';
        CloseOtherWorkingFilesAction = __decorate([
            __param(2, untitledEditorService_1.IUntitledEditorService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, ITextFileService),
            __param(5, message_1.IMessageService),
            __param(6, quickOpenService_1.IQuickOpenService)
        ], CloseOtherWorkingFilesAction);
        return CloseOtherWorkingFilesAction;
    }(BaseCloseWorkingFileAction));
    exports.CloseOtherWorkingFilesAction = CloseOtherWorkingFilesAction;
    function disposeNonDirtyFileInputs(editorService, quickopenService, textFileService, exclude) {
        var activeFileInputs = editorService.getVisibleEditors().map(function (e) { return editor_1.asFileEditorInput(e.input, true); }).filter(function (i) { return i instanceof fileEditorInput_1.FileEditorInput; });
        activeFileInputs.forEach(function (f) {
            if (exclude && exclude.toString() === f.getResource().toString()) {
                return; // excluded
            }
            if (textFileService.isDirty(f.getResource())) {
                return; // do not touch dirty
            }
            fileEditorInputsForResource(f.getResource(), editorService, quickopenService).forEach(function (i) {
                if (!i.isDisposed()) {
                    i.dispose(true);
                }
            });
        });
    }
    function closeNonFileEditors(editorService) {
        var nonFileEditors = editorService.getVisibleEditors().filter(function (e) { return !editor_1.getUntitledOrFileResource(e.input, true); });
        return winjs_base_1.TPromise.join(nonFileEditors.map(function (e) { return editorService.closeEditor(e); })).then(function () { return true; }, errors.onUnexpectedError);
    }
    function fileEditorInputsForResource(resource, editorService, quickopenService) {
        // Get cached ones
        var inputs = fileEditorInput_1.FileEditorInput.getAll(resource);
        // Add those from history as well
        var history = quickopenService.getEditorHistory();
        for (var i = 0; i < history.length; i++) {
            var element = history[i];
            if (element instanceof fileEditorInput_1.FileEditorInput && element.getResource().toString() === resource.toString()) {
                inputs.push(element);
            }
        }
        // Add those from visible editors too
        var editors = editorService.getVisibleEditors();
        editors.forEach(function (editor) {
            var input = editor.input;
            if (input instanceof fileEditorInput_1.FileEditorInput && input.getResource().toString() === resource.toString()) {
                inputs.push(input);
            }
        });
        return inputs;
    }
    var CloseFileAction = (function (_super) {
        __extends(CloseFileAction, _super);
        function CloseFileAction(id, label, instantiationService, editorService, textFileService, messageService, quickOpenService) {
            _super.call(this, id, label);
            this.instantiationService = instantiationService;
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.messageService = messageService;
            this.quickOpenService = quickOpenService;
        }
        CloseFileAction.prototype.run = function () {
            var editor = this.editorService.getActiveEditor();
            var input = this.editorService.getActiveEditorInput();
            var resource = editor_1.getUntitledOrFileResource(input, true);
            // For a file or untitled
            if (resource) {
                var model = this.textFileService.getWorkingFilesModel();
                var entry = model.findEntry(resource);
                // Use action to close a working file that will take care of everthing
                if (entry) {
                    var closeAction_1 = this.instantiationService.createInstance(CloseOneWorkingFileAction, model, entry);
                    closeAction_1.run().done(function () { return closeAction_1.dispose(); }, errors.onUnexpectedError);
                }
                else {
                    if (input instanceof diffEditorInput_1.DiffEditorInput) {
                        input = input.getModifiedInput();
                    }
                    // File Input
                    if (input instanceof fileEditorInput_1.FileEditorInput) {
                        fileEditorInputsForResource(input.getResource(), this.editorService, this.quickOpenService).forEach(function (input) {
                            if (!input.isDisposed()) {
                                input.dispose(true);
                            }
                        });
                    }
                    else {
                        input.dispose();
                    }
                }
            }
            else if (editor) {
                this.editorService.closeEditor(editor).done(null, errors.onUnexpectedError);
                ;
            }
            else {
                this.messageService.show(message_1.Severity.Info, nls.localize(47, null));
            }
            return winjs_base_1.TPromise.as(true);
        };
        CloseFileAction.ID = 'workbench.files.action.closeFile';
        CloseFileAction.LABEL = nls.localize(46, null);
        CloseFileAction = __decorate([
            __param(2, instantiation_1.IInstantiationService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, ITextFileService),
            __param(5, message_1.IMessageService),
            __param(6, quickOpenService_1.IQuickOpenService)
        ], CloseFileAction);
        return CloseFileAction;
    }(actions_1.Action));
    exports.CloseFileAction = CloseFileAction;
    var CloseOtherFilesAction = (function (_super) {
        __extends(CloseOtherFilesAction, _super);
        function CloseOtherFilesAction(id, label, instantiationService, editorService, textFileService, messageService, quickOpenService) {
            _super.call(this, id, label);
            this.instantiationService = instantiationService;
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.messageService = messageService;
            this.quickOpenService = quickOpenService;
        }
        CloseOtherFilesAction.prototype.run = function () {
            var _this = this;
            var workingFilesModel = this.textFileService.getWorkingFilesModel();
            var activeResource = editor_1.getUntitledOrFileResource(this.editorService.getActiveEditorInput(), true);
            var actionToRun;
            // Close all but active resource
            if (activeResource && workingFilesModel.hasEntry(activeResource)) {
                actionToRun = this.instantiationService.createInstance(CloseOtherWorkingFilesAction, workingFilesModel, workingFilesModel.findEntry(activeResource));
            }
            else {
                actionToRun = this.instantiationService.createInstance(CloseAllWorkingFilesAction, workingFilesModel);
            }
            return actionToRun.run().then(function () {
                actionToRun.dispose();
                // Dispose remaining non dirty ones except for active one
                disposeNonDirtyFileInputs(_this.editorService, _this.quickOpenService, _this.textFileService, activeResource);
            });
        };
        CloseOtherFilesAction.ID = 'workbench.files.action.closeOtherFiles';
        CloseOtherFilesAction.LABEL = nls.localize(48, null);
        CloseOtherFilesAction = __decorate([
            __param(2, instantiation_1.IInstantiationService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, ITextFileService),
            __param(5, message_1.IMessageService),
            __param(6, quickOpenService_1.IQuickOpenService)
        ], CloseOtherFilesAction);
        return CloseOtherFilesAction;
    }(actions_1.Action));
    exports.CloseOtherFilesAction = CloseOtherFilesAction;
    var CloseAllFilesAction = (function (_super) {
        __extends(CloseAllFilesAction, _super);
        function CloseAllFilesAction(id, label, instantiationService, editorService, textFileService, messageService, quickOpenService) {
            _super.call(this, id, label);
            this.instantiationService = instantiationService;
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.messageService = messageService;
            this.quickOpenService = quickOpenService;
        }
        CloseAllFilesAction.prototype.run = function () {
            var _this = this;
            // Close all Working Files
            var closeAction = this.instantiationService.createInstance(CloseAllWorkingFilesAction, this.textFileService.getWorkingFilesModel());
            return closeAction.run().then(function () {
                closeAction.dispose();
                // Dispose remaining non dirty ones
                disposeNonDirtyFileInputs(_this.editorService, _this.quickOpenService, _this.textFileService);
            });
        };
        CloseAllFilesAction.ID = 'workbench.files.action.closeAllFiles';
        CloseAllFilesAction.LABEL = nls.localize(49, null);
        CloseAllFilesAction = __decorate([
            __param(2, instantiation_1.IInstantiationService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, ITextFileService),
            __param(5, message_1.IMessageService),
            __param(6, quickOpenService_1.IQuickOpenService)
        ], CloseAllFilesAction);
        return CloseAllFilesAction;
    }(actions_1.Action));
    exports.CloseAllFilesAction = CloseAllFilesAction;
    var OpenNextWorkingFile = (function (_super) {
        __extends(OpenNextWorkingFile, _super);
        function OpenNextWorkingFile(id, label, editorService, textFileService, messageService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.messageService = messageService;
        }
        OpenNextWorkingFile.prototype.run = function () {
            var model = this.textFileService.getWorkingFilesModel();
            // Return: No working files
            if (model.count() === 0) {
                this.messageService.show(message_1.Severity.Info, nls.localize(51, null));
            }
            else {
                var resource = editor_1.getUntitledOrFileResource(this.editorService.getActiveEditorInput(), true);
                return this.editorService.openEditor({ resource: model.next(resource).resource });
            }
            return winjs_base_1.TPromise.as(true);
        };
        OpenNextWorkingFile.ID = 'workbench.files.action.openNextWorkingFile';
        OpenNextWorkingFile.LABEL = nls.localize(50, null);
        OpenNextWorkingFile = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, ITextFileService),
            __param(4, message_1.IMessageService)
        ], OpenNextWorkingFile);
        return OpenNextWorkingFile;
    }(actions_1.Action));
    exports.OpenNextWorkingFile = OpenNextWorkingFile;
    var OpenPreviousWorkingFile = (function (_super) {
        __extends(OpenPreviousWorkingFile, _super);
        function OpenPreviousWorkingFile(id, label, editorService, textFileService, messageService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.messageService = messageService;
        }
        OpenPreviousWorkingFile.prototype.run = function () {
            var model = this.textFileService.getWorkingFilesModel();
            // Return: No working files
            if (model.count() === 0) {
                this.messageService.show(message_1.Severity.Info, nls.localize(53, null));
            }
            else {
                var resource = editor_1.getUntitledOrFileResource(this.editorService.getActiveEditorInput(), true);
                return this.editorService.openEditor({ resource: model.previous(resource).resource });
            }
            return winjs_base_1.TPromise.as(true);
        };
        OpenPreviousWorkingFile.ID = 'workbench.files.action.openPreviousWorkingFile';
        OpenPreviousWorkingFile.LABEL = nls.localize(52, null);
        OpenPreviousWorkingFile = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, ITextFileService),
            __param(4, message_1.IMessageService)
        ], OpenPreviousWorkingFile);
        return OpenPreviousWorkingFile;
    }(actions_1.Action));
    exports.OpenPreviousWorkingFile = OpenPreviousWorkingFile;
    var AddToWorkingFiles = (function (_super) {
        __extends(AddToWorkingFiles, _super);
        function AddToWorkingFiles(id, label, editorService, textFileService, messageService) {
            _super.call(this, id, label);
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.messageService = messageService;
        }
        AddToWorkingFiles.prototype.run = function () {
            var fileInput = editor_1.asFileEditorInput(this.editorService.getActiveEditorInput(), true);
            if (fileInput) {
                this.textFileService.getWorkingFilesModel().addEntry(fileInput.getResource());
            }
            else {
                this.messageService.show(message_1.Severity.Info, nls.localize(55, null));
            }
            return winjs_base_1.TPromise.as(true);
        };
        AddToWorkingFiles.ID = 'workbench.files.action.addToWorkingFiles';
        AddToWorkingFiles.LABEL = nls.localize(54, null);
        AddToWorkingFiles = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, ITextFileService),
            __param(4, message_1.IMessageService)
        ], AddToWorkingFiles);
        return AddToWorkingFiles;
    }(actions_1.Action));
    exports.AddToWorkingFiles = AddToWorkingFiles;
    var FocusWorkingFiles = (function (_super) {
        __extends(FocusWorkingFiles, _super);
        function FocusWorkingFiles(id, label, viewletService) {
            _super.call(this, id, label);
            this.viewletService = viewletService;
        }
        FocusWorkingFiles.prototype.run = function () {
            return this.viewletService.openViewlet(Files.VIEWLET_ID, true).then(function (viewlet) {
                viewlet.getWorkingFilesView().expand();
                viewlet.getWorkingFilesView().getViewer().DOMFocus();
            });
        };
        FocusWorkingFiles.ID = 'workbench.files.action.focusWorkingFiles';
        FocusWorkingFiles.LABEL = nls.localize(56, null);
        FocusWorkingFiles = __decorate([
            __param(2, viewletService_1.IViewletService)
        ], FocusWorkingFiles);
        return FocusWorkingFiles;
    }(actions_1.Action));
    exports.FocusWorkingFiles = FocusWorkingFiles;
    var FocusFilesExplorer = (function (_super) {
        __extends(FocusFilesExplorer, _super);
        function FocusFilesExplorer(id, label, viewletService) {
            _super.call(this, id, label);
            this.viewletService = viewletService;
        }
        FocusFilesExplorer.prototype.run = function () {
            return this.viewletService.openViewlet(Files.VIEWLET_ID, true).then(function (viewlet) {
                var view = viewlet.getExplorerView();
                if (view) {
                    view.expand();
                    view.getViewer().DOMFocus();
                }
            });
        };
        FocusFilesExplorer.ID = 'workbench.files.action.focusFilesExplorer';
        FocusFilesExplorer.LABEL = nls.localize(57, null);
        FocusFilesExplorer = __decorate([
            __param(2, viewletService_1.IViewletService)
        ], FocusFilesExplorer);
        return FocusFilesExplorer;
    }(actions_1.Action));
    exports.FocusFilesExplorer = FocusFilesExplorer;
    function keybindingForAction(id) {
        switch (id) {
            case GlobalNewFileAction.ID:
                return new keyCodes_1.Keybinding(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_N);
            case TriggerRenameFileAction.ID:
                return new keyCodes_1.Keybinding(platform_1.isMacintosh ? keyCodes_1.KeyCode.Enter : keyCodes_1.KeyCode.F2);
            case SaveFileAction.ID:
                return new keyCodes_1.Keybinding(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_S);
            case DeleteFileAction.ID:
            case MoveFileToTrashAction.ID:
                return new keyCodes_1.Keybinding(keyCodes_1.KeyCode.Delete);
            case CopyFileAction.ID:
                return new keyCodes_1.Keybinding(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_C);
            case PasteFileAction.ID:
                return new keyCodes_1.Keybinding(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_V);
            case OpenToSideAction.ID:
                if (platform_1.isMacintosh) {
                    return new keyCodes_1.Keybinding(keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.Enter);
                }
                else {
                    return new keyCodes_1.Keybinding(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.Enter);
                }
        }
        return null;
    }
    exports.keybindingForAction = keybindingForAction;
    function validateFileName(parent, name, allowOverwriting) {
        if (allowOverwriting === void 0) { allowOverwriting = false; }
        // Produce a well formed file name
        name = getWellFormedFileName(name);
        // Name not provided
        if (!name || name.length === 0 || /^\s+$/.test(name)) {
            return nls.localize(58, null);
        }
        // Do not allow to overwrite existing file
        if (!allowOverwriting) {
            if (parent.children && parent.children.some(function (c) {
                if (platform_1.isLinux) {
                    return c.name === name;
                }
                return c.name.toLowerCase() === name.toLowerCase();
            })) {
                return nls.localize(59, null, name);
            }
        }
        // Invalid File name
        if (!paths.isValidBasename(name)) {
            return nls.localize(60, null, name);
        }
        // Max length restriction (on Windows)
        if (platform_1.isWindows) {
            var fullPathLength = name.length + parent.resource.fsPath.length + 1;
            if (fullPathLength > 255) {
                return nls.localize(61, null, name);
            }
        }
        return null;
    }
    exports.validateFileName = validateFileName;
    function getWellFormedFileName(filename) {
        if (!filename) {
            return filename;
        }
        // Trim whitespaces
        filename = strings.trim(strings.trim(filename, ' '), '\t');
        // Remove trailing dots
        filename = strings.rtrim(filename, '.');
        return filename;
    }
    exports.getWellFormedFileName = getWellFormedFileName;
    // Diagnostics support
    var diag;
    if (!diag) {
        diag = diagnostics.register('FileActionsDiagnostics', function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            console.log(args[1] + ' - ' + args[0] + ' (time: ' + args[2].getTime() + ' [' + args[2].toUTCString() + '])');
        });
    }
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/fileTracker", ["require", "exports", 'vs/base/common/errors', 'vs/nls!vs/workbench/parts/files/browser/fileTracker', 'vs/base/common/mime', 'vs/base/common/uri', 'vs/base/common/paths', 'vs/workbench/common/editor/diffEditorInput', 'vs/workbench/common/editor', 'vs/workbench/parts/files/common/files', 'vs/platform/files/common/files', 'vs/workbench/parts/files/browser/editors/fileEditorInput', 'vs/workbench/common/editor/iframeEditorInput', 'vs/workbench/parts/files/common/editors/textFileEditorModel', 'vs/workbench/common/events', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/services/activity/common/activityService', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation'], function (require, exports, errors, nls, mime_1, uri_1, paths, diffEditorInput_1, editor_1, files_1, files_2, fileEditorInput_1, iframeEditorInput_1, textFileEditorModel_1, events_1, untitledEditorService_1, editorService_1, quickOpenService_1, activityService_1, event_1, instantiation_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // This extension tracks files for changes to update editors and inputs accordingly.
    var FileTracker = (function () {
        function FileTracker(eventService, quickOpenService, editorService, activityService, textFileService, instantiationService, untitledEditorService) {
            this.eventService = eventService;
            this.quickOpenService = quickOpenService;
            this.editorService = editorService;
            this.activityService = activityService;
            this.textFileService = textFileService;
            this.instantiationService = instantiationService;
            this.untitledEditorService = untitledEditorService;
            this.toUnbind = [];
            this.workingFiles = textFileService.getWorkingFilesModel();
            this.registerListeners();
        }
        FileTracker.prototype.getId = function () {
            return 'vs.files.filetracker';
        };
        FileTracker.prototype.registerListeners = function () {
            var _this = this;
            // Update editors and inputs from local changes and saves
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.EDITOR_INPUT_CHANGED, function (e) { return _this.onEditorInputChanged(e); }));
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.UNTITLED_FILE_DELETED, function (e) { return _this.onUntitledEditorDeleted(e); }));
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.UNTITLED_FILE_DIRTY, function (e) { return _this.onUntitledEditorDirty(e); }));
            this.toUnbind.push(this.eventService.addListener(files_1.EventType.FILE_DIRTY, function (e) { return _this.onTextFileDirty(e); }));
            this.toUnbind.push(this.eventService.addListener(files_1.EventType.FILE_SAVING, function (e) { return _this.onTextFileSaving(e); }));
            this.toUnbind.push(this.eventService.addListener(files_1.EventType.FILE_SAVE_ERROR, function (e) { return _this.onTextFileSaveError(e); }));
            this.toUnbind.push(this.eventService.addListener(files_1.EventType.FILE_SAVED, function (e) { return _this.onTextFileSaved(e); }));
            this.toUnbind.push(this.eventService.addListener(files_1.EventType.FILE_REVERTED, function (e) { return _this.onTextFileReverted(e); }));
            this.toUnbind.push(this.eventService.addListener('files.internal:fileChanged', function (e) { return _this.onLocalFileChange(e); }));
            // Update editors and inputs from disk changes
            this.toUnbind.push(this.eventService.addListener(files_2.EventType.FILE_CHANGES, function (e) { return _this.onFileChanges(e); }));
        };
        FileTracker.prototype.onEditorInputChanged = function (e) {
            this.disposeTextFileModels();
        };
        FileTracker.prototype.onTextFileDirty = function (e) {
            this.emitInputStateChangeEvent(e.getAfter().resource);
            if (this.textFileService.getAutoSaveMode() !== files_1.AutoSaveMode.AFTER_SHORT_DELAY) {
                this.updateActivityBadge(); // no indication needed when auto save is enabled for short delay
            }
        };
        FileTracker.prototype.onTextFileSaving = function (e) {
            this.emitInputStateChangeEvent(e.getAfter().resource);
        };
        FileTracker.prototype.onTextFileSaveError = function (e) {
            this.emitInputStateChangeEvent(e.getAfter().resource);
            this.updateActivityBadge();
        };
        FileTracker.prototype.onTextFileSaved = function (e) {
            this.emitInputStateChangeEvent(e.getAfter().resource);
            if (this.lastDirtyCount > 0) {
                this.updateActivityBadge();
            }
        };
        FileTracker.prototype.onTextFileReverted = function (e) {
            this.emitInputStateChangeEvent(e.getAfter().resource);
            if (this.lastDirtyCount > 0) {
                this.updateActivityBadge();
            }
        };
        FileTracker.prototype.onUntitledEditorDirty = function (e) {
            var input = this.untitledEditorService.get(e.resource);
            if (input) {
                this.eventService.emit(events_1.EventType.EDITOR_INPUT_STATE_CHANGED, new events_1.EditorInputEvent(input));
            }
            this.updateActivityBadge();
        };
        FileTracker.prototype.onUntitledEditorDeleted = function (e) {
            var input = this.untitledEditorService.get(e.resource);
            if (input) {
                this.eventService.emit(events_1.EventType.EDITOR_INPUT_STATE_CHANGED, new events_1.EditorInputEvent(input));
            }
            if (this.lastDirtyCount > 0) {
                this.updateActivityBadge();
            }
        };
        FileTracker.prototype.updateActivityBadge = function () {
            var dirtyCount = this.textFileService.getDirty().length;
            this.lastDirtyCount = dirtyCount;
            if (dirtyCount > 0) {
                this.activityService.showActivity(files_1.VIEWLET_ID, new activityService_1.NumberBadge(dirtyCount, function (num) { return nls.localize(0, null, dirtyCount); }), 'explorer-viewlet-label');
            }
            else {
                this.activityService.clearActivity(files_1.VIEWLET_ID);
            }
        };
        FileTracker.prototype.emitInputStateChangeEvent = function (resource) {
            var _this = this;
            // Find all file editor inputs that are open from the given file resource and emit a editor input state change event.
            // We could do all of this within the file editor input but having all the file change listeners in
            // one place is more elegant and keeps the logic together at once place.
            var editors = this.editorService.getVisibleEditors();
            editors.forEach(function (editor) {
                var input = editor.input;
                if (input instanceof diffEditorInput_1.DiffEditorInput) {
                    input = input.getModifiedInput();
                }
                // File Editor Input
                if (input instanceof fileEditorInput_1.FileEditorInput) {
                    var fileInput = input;
                    if (fileInput.getResource().toString() === resource.toString()) {
                        var inputEvent = editor.input instanceof diffEditorInput_1.DiffEditorInput ? editor.input : fileInput; // make sure to still send around the input from the diff editor if given
                        _this.eventService.emit(events_1.EventType.EDITOR_INPUT_STATE_CHANGED, new events_1.EditorInputEvent(inputEvent));
                    }
                }
            });
        };
        // Note: there is some duplication with the other file event handler below. Since we cannot always rely on the disk events
        // carrying all necessary data in all environments, we also use the local file events to make sure operations are handled.
        // In any case there is no guarantee if the local event is fired first or the disk one. Thus, code must handle the case
        // that the event ordering is random as well as might not carry all information needed.
        FileTracker.prototype.onLocalFileChange = function (e) {
            // Handle moves specially when file is opened
            if (e.gotMoved()) {
                var before = e.getBefore();
                var after = e.getAfter();
                this.handleMovedFileInVisibleEditors(before ? before.resource : null, after ? after.resource : null, after ? after.mime : null);
            }
            // Dispose all known inputs passed on resource
            var oldFile = e.getBefore();
            if ((e.gotMoved() || e.gotDeleted())) {
                this.disposeAll(oldFile.resource, this.quickOpenService.getEditorHistory());
            }
        };
        FileTracker.prototype.onFileChanges = function (e) {
            var _this = this;
            // Dispose inputs that got deleted
            var allDeleted = e.getDeleted();
            if (allDeleted && allDeleted.length > 0) {
                allDeleted.forEach(function (deleted) {
                    _this.disposeAll(deleted.resource, _this.quickOpenService.getEditorHistory());
                });
            }
            // Dispose models that got changed and are not visible. We do this because otherwise
            // cached file models will be stale from the contents on disk.
            e.getUpdated()
                .map(function (u) { return textFileEditorModel_1.CACHE.get(u.resource); })
                .filter(function (model) {
                var canDispose = _this.canDispose(model);
                if (!canDispose) {
                    return false;
                }
                if (new Date().getTime() - model.getLastDirtyTime() < FileTracker.FILE_CHANGE_UPDATE_DELAY) {
                    return false; // this is a weak check to see if the change came from outside the editor or not
                }
                return true; // ok boss
            })
                .forEach(function (model) { return textFileEditorModel_1.CACHE.dispose(model.getResource()); });
            // Update inputs that got updated
            var editors = this.editorService.getVisibleEditors();
            editors.forEach(function (editor) {
                var input = editor.input;
                if (input instanceof diffEditorInput_1.DiffEditorInput) {
                    input = _this.getMatchingFileEditorInputFromDiff(input, e);
                }
                // File Editor Input
                if (input instanceof fileEditorInput_1.FileEditorInput) {
                    var fileInput = input;
                    var fileInputResource = fileInput.getResource();
                    // Input got added or updated, so check for model and update
                    // Note: we also consider the added event because it could be that a file was added
                    // and updated right after.
                    if (e.contains(fileInputResource, files_2.FileChangeType.UPDATED) || e.contains(fileInputResource, files_2.FileChangeType.ADDED)) {
                        var textModel_1 = textFileEditorModel_1.CACHE.get(fileInputResource);
                        // Text file: check for last dirty time
                        if (textModel_1) {
                            var state = textModel_1.getState();
                            // We only ever update models that are in good saved state
                            if (state === textFileEditorModel_1.State.SAVED) {
                                var lastDirtyTime = textModel_1.getLastDirtyTime();
                                // Force a reopen of the input if this change came in later than our wait interval before we consider it
                                if (new Date().getTime() - lastDirtyTime > FileTracker.FILE_CHANGE_UPDATE_DELAY) {
                                    var codeEditor_1 = editor.getControl();
                                    var viewState_1 = codeEditor_1.saveViewState();
                                    var currentMtime_1 = textModel_1.getLastModifiedTime(); // optimize for the case where the file did actually not change
                                    textModel_1.load().done(function () {
                                        if (textModel_1.getLastModifiedTime() !== currentMtime_1 && _this.isEditorShowingPath(editor, textModel_1.getResource())) {
                                            codeEditor_1.restoreViewState(viewState_1);
                                        }
                                    }, errors.onUnexpectedError);
                                }
                            }
                        }
                        else if (editor.getId() === files_1.BINARY_FILE_EDITOR_ID) {
                            var editorOptions = new editor_1.EditorOptions();
                            editorOptions.forceOpen = true;
                            editorOptions.preserveFocus = true;
                            _this.editorService.openEditor(editor.input, editorOptions, editor.position).done(null, errors.onUnexpectedError);
                        }
                    }
                }
                else if (input instanceof iframeEditorInput_1.IFrameEditorInput) {
                    var iFrameInput = input;
                    if (e.contains(iFrameInput.getResource(), files_2.FileChangeType.UPDATED)) {
                        editor.reload();
                    }
                }
            });
        };
        FileTracker.prototype.isEditorShowingPath = function (editor, resource) {
            // Only relevant if Editor is visible
            if (!editor.isVisible()) {
                return false;
            }
            // Only relevant if Input is set
            var input = editor.getInput();
            if (!input) {
                return false;
            }
            // Support diff editor input too
            if (input instanceof diffEditorInput_1.DiffEditorInput) {
                input = input.getModifiedInput();
            }
            return input instanceof fileEditorInput_1.FileEditorInput && input.getResource().toString() === resource.toString();
        };
        FileTracker.prototype.handleMovedFileInVisibleEditors = function (oldResource, newResource, mimeHint) {
            var _this = this;
            var editors = this.editorService.getVisibleEditors();
            editors.forEach(function (editor) {
                var input = editor.input;
                if (input instanceof diffEditorInput_1.DiffEditorInput) {
                    input = input.getModifiedInput();
                }
                var inputResource;
                if (input instanceof fileEditorInput_1.FileEditorInput) {
                    inputResource = input.getResource();
                }
                else if (input instanceof iframeEditorInput_1.IFrameEditorInput) {
                    inputResource = input.getResource();
                }
                // Editor Input with associated Resource
                if (inputResource) {
                    // Update Editor if file (or any parent of the input) got renamed or moved
                    var updateInput = false;
                    if (paths.isEqualOrParent(inputResource.fsPath, oldResource.fsPath)) {
                        updateInput = true;
                    }
                    // Do update from move
                    if (updateInput) {
                        var reopenFileResource = void 0;
                        if (oldResource.toString() === inputResource.toString()) {
                            reopenFileResource = newResource;
                        }
                        else {
                            var index = inputResource.fsPath.indexOf(oldResource.fsPath);
                            reopenFileResource = uri_1.default.file(paths.join(newResource.fsPath, inputResource.fsPath.substr(index + oldResource.fsPath.length + 1))); // update the path by changing the old path value to the new one
                        }
                        var editorInput = void 0;
                        var editorOptions = new editor_1.EditorOptions();
                        editorOptions.preserveFocus = true;
                        // Reopen File Input
                        if (input instanceof fileEditorInput_1.FileEditorInput) {
                            editorInput = _this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, reopenFileResource, mimeHint || mime_1.MIME_UNKNOWN, void 0);
                        }
                        else if (input instanceof iframeEditorInput_1.IFrameEditorInput) {
                            var iFrameInput = input;
                            editorInput = iFrameInput.createNew(reopenFileResource);
                        }
                        if (editorInput) {
                            _this.editorService.openEditor(editorInput, editorOptions, editor.position).done(null, errors.onUnexpectedError);
                        }
                    }
                }
            });
        };
        FileTracker.prototype.getMatchingFileEditorInputFromDiff = function (input, arg) {
            // First try modifiedInput
            var modifiedInput = input.getModifiedInput();
            var res = this.getMatchingFileEditorInputFromInput(modifiedInput, arg);
            if (res) {
                return res;
            }
            // Second try originalInput
            var originalInput = input.getOriginalInput();
            return this.getMatchingFileEditorInputFromInput(originalInput, arg);
        };
        FileTracker.prototype.getMatchingFileEditorInputFromInput = function (input, arg) {
            if (input instanceof fileEditorInput_1.FileEditorInput) {
                if (arg instanceof uri_1.default) {
                    var deletedResource = arg;
                    if (this.containsResource(input, deletedResource)) {
                        return input;
                    }
                }
                else {
                    var updatedFiles = arg;
                    if (updatedFiles.contains(input.getResource(), files_2.FileChangeType.UPDATED)) {
                        return input;
                    }
                }
            }
            return null;
        };
        FileTracker.prototype.disposeAll = function (deletedResource, history) {
            var _this = this;
            if (this.textFileService.isDirty(deletedResource)) {
                return; // never dispose dirty resources
            }
            // Add existing clients matching resource
            var inputsContainingPath = fileEditorInput_1.FileEditorInput.getAll(deletedResource);
            // Add those from history as well
            for (var i = 0; i < history.length; i++) {
                var element = history[i];
                // File Input
                if (element instanceof fileEditorInput_1.FileEditorInput && this.containsResource(element, deletedResource)) {
                    inputsContainingPath.push(element);
                }
                else if (element instanceof iframeEditorInput_1.IFrameEditorInput && this.containsResource(element, deletedResource)) {
                    inputsContainingPath.push(element);
                }
            }
            // Add those from visible editors too
            var editors = this.editorService.getVisibleEditors();
            editors.forEach(function (editor) {
                var input = editor.input;
                if (input instanceof diffEditorInput_1.DiffEditorInput) {
                    input = _this.getMatchingFileEditorInputFromDiff(input, deletedResource);
                    if (input instanceof fileEditorInput_1.FileEditorInput) {
                        inputsContainingPath.push(input);
                    }
                }
                else if (input instanceof fileEditorInput_1.FileEditorInput && _this.containsResource(input, deletedResource)) {
                    inputsContainingPath.push(input);
                }
                else if (input instanceof iframeEditorInput_1.IFrameEditorInput && _this.containsResource(input, deletedResource)) {
                    inputsContainingPath.push(input);
                }
            });
            // Dispose all
            inputsContainingPath.forEach(function (input) {
                if (!input.isDisposed()) {
                    if (input instanceof fileEditorInput_1.FileEditorInput) {
                        var fileInputToDispose = input;
                        fileInputToDispose.dispose(true /* force */);
                    }
                    else {
                        input.dispose();
                    }
                }
            });
        };
        FileTracker.prototype.containsResource = function (input, resource) {
            var fileResource;
            if (input instanceof fileEditorInput_1.FileEditorInput) {
                fileResource = input.getResource();
            }
            else {
                fileResource = input.getResource();
            }
            if (paths.isEqualOrParent(fileResource.fsPath, resource.fsPath)) {
                return true;
            }
            return false;
        };
        FileTracker.prototype.disposeTextFileModels = function () {
            // To not grow our text file model cache infinitly, we dispose models that
            // are not showing up in any editor and are not in the working file set or dirty.
            var _this = this;
            // Get all cached file models
            textFileEditorModel_1.CACHE.getAll()
                .filter(function (model) { return !_this.workingFiles.hasEntry(model.getResource()) && _this.canDispose(model); })
                .forEach(function (model) { return textFileEditorModel_1.CACHE.dispose(model.getResource()); });
        };
        FileTracker.prototype.canDispose = function (textModel) {
            if (!textModel) {
                return false; // we need data!
            }
            if (textModel.textEditorModel && textModel.textEditorModel.isAttachedToEditor()) {
                return false; // never dispose when attached to editor
            }
            if (textModel.getState() !== textFileEditorModel_1.State.SAVED) {
                return false; // never dispose unsaved models
            }
            if (this.editorService.getVisibleEditors().some(function (e) {
                if (e.input instanceof iframeEditorInput_1.IFrameEditorInput) {
                    var iFrameInputResource = e.input.getResource();
                    return iFrameInputResource && iFrameInputResource.toString() === textModel.getResource().toString();
                }
                return false;
            })) {
                return false; // never dispose models that are used in iframe inputs
            }
            return true;
        };
        FileTracker.prototype.dispose = function () {
            while (this.toUnbind.length) {
                this.toUnbind.pop()();
            }
        };
        // Delay in ms that we wait at minimum before we update a model from a file change event.
        // This reduces the chance that a save from the client triggers an update of the editor.
        FileTracker.FILE_CHANGE_UPDATE_DELAY = 2000;
        FileTracker = __decorate([
            __param(0, event_1.IEventService),
            __param(1, quickOpenService_1.IQuickOpenService),
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, activityService_1.IActivityService),
            __param(4, files_1.ITextFileService),
            __param(5, instantiation_1.IInstantiationService),
            __param(6, untitledEditorService_1.IUntitledEditorService)
        ], FileTracker);
        return FileTracker;
    }());
    exports.FileTracker = FileTracker;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/saveErrorHandler", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/files/browser/saveErrorHandler', 'vs/base/common/errors', 'vs/base/common/paths', 'vs/base/common/actions', 'vs/base/common/uri', 'vs/base/common/mime', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor/resourceEditorInput', 'vs/workbench/common/editor/diffEditorInput', 'vs/workbench/parts/files/browser/editors/fileEditorInput', 'vs/workbench/parts/files/browser/fileActions', 'vs/platform/files/common/files', 'vs/workbench/services/editor/common/editorService', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/platform/workspace/common/workspace', 'vs/editor/common/services/modeService', 'vs/editor/common/services/modelService'], function (require, exports, winjs_base_1, nls, errors, paths, actions_1, uri_1, mime_1, baseEditor_1, resourceEditorInput_1, diffEditorInput_1, fileEditorInput_1, fileActions_1, files_1, editorService_1, event_1, instantiation_1, message_1, workspace_1, modeService_1, modelService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // A handler for save error happening with conflict resolution actions
    var SaveErrorHandler = (function () {
        function SaveErrorHandler(messageService, instantiationService) {
            this.messageService = messageService;
            this.instantiationService = instantiationService;
        }
        SaveErrorHandler.prototype.onSaveError = function (error, model) {
            var _this = this;
            var message;
            // Dirty write prevention
            if (error.fileOperationResult === files_1.FileOperationResult.FILE_MODIFIED_SINCE) {
                message = this.instantiationService.createInstance(ResolveSaveConflictMessage, model, null);
            }
            else {
                var isReadonly = error.fileOperationResult === files_1.FileOperationResult.FILE_READ_ONLY;
                var actions = [];
                // Cancel
                actions.push(message_1.CancelAction);
                // Retry
                if (isReadonly) {
                    actions.push(new actions_1.Action('workbench.files.action.overwrite', nls.localize(0, null), null, true, function () {
                        if (!model.isDisposed()) {
                            return model.save(true /* overwrite readonly */).then(function () { return true; });
                        }
                        return winjs_base_1.TPromise.as(true);
                    }));
                }
                else {
                    actions.push(new actions_1.Action('workbench.files.action.retry', nls.localize(1, null), null, true, function () {
                        var saveFileAction = _this.instantiationService.createInstance(fileActions_1.SaveFileAction, fileActions_1.SaveFileAction.ID, fileActions_1.SaveFileAction.LABEL);
                        saveFileAction.setResource(model.getResource());
                        return saveFileAction.run().then(function () { saveFileAction.dispose(); return true; });
                    }));
                }
                // Discard
                actions.push(new actions_1.Action('workbench.files.action.discard', nls.localize(2, null), null, true, function () {
                    var revertFileAction = _this.instantiationService.createInstance(fileActions_1.RevertFileAction, fileActions_1.RevertFileAction.ID, fileActions_1.RevertFileAction.LABEL);
                    revertFileAction.setResource(model.getResource());
                    return revertFileAction.run().then(function () { revertFileAction.dispose(); return true; });
                }));
                // Save As
                actions.push(new actions_1.Action('workbench.files.action.saveAs', fileActions_1.SaveFileAsAction.LABEL, null, true, function () {
                    var saveAsAction = _this.instantiationService.createInstance(fileActions_1.SaveFileAsAction, fileActions_1.SaveFileAsAction.ID, fileActions_1.SaveFileAsAction.LABEL);
                    saveAsAction.setResource(model.getResource());
                    return saveAsAction.run().then(function () { saveAsAction.dispose(); return true; });
                }));
                var errorMessage = void 0;
                if (isReadonly) {
                    errorMessage = nls.localize(3, null, paths.basename(model.getResource().fsPath));
                }
                else {
                    errorMessage = nls.localize(4, null, paths.basename(model.getResource().fsPath), errors.toErrorMessage(error, false));
                }
                message = {
                    message: errorMessage,
                    actions: actions
                };
            }
            if (this.messageService) {
                this.messageService.show(message_1.Severity.Error, message);
            }
        };
        SaveErrorHandler = __decorate([
            __param(0, message_1.IMessageService),
            __param(1, instantiation_1.IInstantiationService)
        ], SaveErrorHandler);
        return SaveErrorHandler;
    }());
    exports.SaveErrorHandler = SaveErrorHandler;
    // Save conflict resolution editor input
    var ConflictResolutionDiffEditorInput = (function (_super) {
        __extends(ConflictResolutionDiffEditorInput, _super);
        function ConflictResolutionDiffEditorInput(model, name, description, originalInput, modifiedInput, messageService, instantiationService, eventService, editorService) {
            _super.call(this, name, description, originalInput, modifiedInput);
            this.messageService = messageService;
            this.instantiationService = instantiationService;
            this.eventService = eventService;
            this.editorService = editorService;
            this.model = model;
        }
        ConflictResolutionDiffEditorInput.prototype.getModel = function () {
            return this.model;
        };
        ConflictResolutionDiffEditorInput.prototype.getId = function () {
            return ConflictResolutionDiffEditorInput.ID;
        };
        ConflictResolutionDiffEditorInput.ID = 'workbench.editors.files.conflictResolutionDiffEditorInput';
        ConflictResolutionDiffEditorInput = __decorate([
            __param(5, message_1.IMessageService),
            __param(6, instantiation_1.IInstantiationService),
            __param(7, event_1.IEventService),
            __param(8, editorService_1.IWorkbenchEditorService)
        ], ConflictResolutionDiffEditorInput);
        return ConflictResolutionDiffEditorInput;
    }(diffEditorInput_1.DiffEditorInput));
    exports.ConflictResolutionDiffEditorInput = ConflictResolutionDiffEditorInput;
    var FileOnDiskEditorInput = (function (_super) {
        __extends(FileOnDiskEditorInput, _super);
        function FileOnDiskEditorInput(fileResource, mime, name, description, modelService, modeService, instantiationService, fileService) {
            // We create a new resource URI here that is different from the file resource because we represent the state of
            // the file as it is on disk and not as it is (potentially cached) in Code. That allows us to have a different
            // model for the left-hand comparision compared to the conflicting one in Code to the right.
            _super.call(this, name, description, uri_1.default.create('disk', null, fileResource.fsPath), modelService, instantiationService);
            this.modeService = modeService;
            this.fileService = fileService;
            this.fileResource = fileResource;
            this.mime = mime;
        }
        FileOnDiskEditorInput.prototype.getLastModified = function () {
            return this.lastModified;
        };
        FileOnDiskEditorInput.prototype.resolve = function (refresh) {
            var _this = this;
            // Make sure our file from disk is resolved up to date
            return this.fileService.resolveContent(this.fileResource).then(function (content) {
                _this.lastModified = content.mtime;
                var codeEditorModel = _this.modelService.getModel(_this.resource);
                if (!codeEditorModel) {
                    _this.modelService.createModel(content.value, _this.modeService.getOrCreateMode(_this.mime), _this.resource);
                    _this.createdEditorModel = true;
                }
                else {
                    codeEditorModel.setValue(content.value);
                }
                return _super.prototype.resolve.call(_this, refresh);
            });
        };
        FileOnDiskEditorInput.prototype.dispose = function () {
            if (this.createdEditorModel) {
                this.modelService.destroyModel(this.resource);
                this.createdEditorModel = false;
            }
            _super.prototype.dispose.call(this);
        };
        FileOnDiskEditorInput = __decorate([
            __param(4, modelService_1.IModelService),
            __param(5, modeService_1.IModeService),
            __param(6, instantiation_1.IInstantiationService),
            __param(7, files_1.IFileService)
        ], FileOnDiskEditorInput);
        return FileOnDiskEditorInput;
    }(resourceEditorInput_1.ResourceEditorInput));
    exports.FileOnDiskEditorInput = FileOnDiskEditorInput;
    // A message with action to resolve a 412 save conflict
    var ResolveSaveConflictMessage = (function () {
        function ResolveSaveConflictMessage(model, message, messageService, instantiationService, contextService, editorService) {
            var _this = this;
            this.messageService = messageService;
            this.instantiationService = instantiationService;
            this.contextService = contextService;
            this.editorService = editorService;
            this.model = model;
            var resource = model.getResource();
            if (message) {
                this.message = message;
            }
            else {
                this.message = nls.localize(5, null, paths.basename(resource.fsPath));
            }
            this.actions = [
                new actions_1.Action('workbench.files.action.resolveConflict', nls.localize(6, null), null, true, function () {
                    if (!_this.model.isDisposed()) {
                        var mime = mime_1.guessMimeTypes(resource.fsPath).join(', ');
                        var originalInput = _this.instantiationService.createInstance(FileOnDiskEditorInput, resource, mime, paths.basename(resource.fsPath), resource.fsPath);
                        var modifiedInput = _this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, resource, mime, void 0);
                        var conflictInput = _this.instantiationService.createInstance(ConflictResolutionDiffEditorInput, _this.model, nls.localize(7, null, modifiedInput.getName(), _this.contextService.getConfiguration().env.appName), nls.localize(8, null, modifiedInput.getDescription()), originalInput, modifiedInput);
                        return _this.editorService.openEditor(conflictInput).then(function () {
                            // We have to bring the model into conflict resolution mode to prevent subsequent save erros when the user makes edits
                            _this.model.setConflictResolutionMode();
                            // Inform user
                            _this.messageService.show(message_1.Severity.Info, nls.localize(9, null));
                        });
                    }
                    return winjs_base_1.TPromise.as(true);
                })
            ];
        }
        ResolveSaveConflictMessage = __decorate([
            __param(2, message_1.IMessageService),
            __param(3, instantiation_1.IInstantiationService),
            __param(4, workspace_1.IWorkspaceContextService),
            __param(5, editorService_1.IWorkbenchEditorService)
        ], ResolveSaveConflictMessage);
        return ResolveSaveConflictMessage;
    }());
    // Accept changes to resolve a conflicting edit
    var AcceptLocalChangesAction = (function (_super) {
        __extends(AcceptLocalChangesAction, _super);
        function AcceptLocalChangesAction(messageService, instantiationService, editorService) {
            _super.call(this, 'workbench.files.action.acceptLocalChanges', nls.localize(10, null), 'conflict-editor-action accept-changes');
            this.messageService = messageService;
            this.instantiationService = instantiationService;
            this.editorService = editorService;
            this.messagesToHide = [];
        }
        AcceptLocalChangesAction.prototype.run = function () {
            var _this = this;
            var conflictInput = this.input;
            var model = conflictInput.getModel();
            var localModelValue = model.getValue();
            // 1.) Get the diff editor model from cache (resolve(false)) to have access to the mtime of the file we currently show to the left
            return conflictInput.resolve(false).then(function (diffModel) {
                var knownLastModified = conflictInput.originalInput.getLastModified();
                // 2.) Revert the model to get the latest copy from disk and to have access to the mtime of the file now
                return model.revert().then(function () {
                    var diskLastModified = model.getLastModifiedTime();
                    // 3. a) If we know that the file on the left hand side was not modified meanwhile, restore the user value and trigger a save
                    if (diskLastModified <= knownLastModified) {
                        // Restore user value
                        model.textEditorModel.setValue(localModelValue);
                        // Trigger save
                        return model.save().then(function () {
                            // Hide any previously shown messages
                            while (_this.messagesToHide.length) {
                                _this.messagesToHide.pop()();
                            }
                            // Reopen file input
                            var input = _this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, model.getResource(), mime_1.guessMimeTypes(model.getResource().fsPath).join(', '), void 0);
                            return _this.editorService.openEditor(input, null, _this.position).then(function () {
                                // Dispose conflict input
                                conflictInput.dispose();
                            });
                        });
                    }
                    else {
                        // Again, we have to bring the model into conflict resolution because revert() would have cleared it
                        model.setConflictResolutionMode();
                        // Restore user value
                        model.textEditorModel.setValue(localModelValue);
                        // Reload the left hand side of the diff editor to show the up to date version and inform the user that he has to redo the action
                        return conflictInput.originalInput.resolve(true).then(function () {
                            _this.messagesToHide.push(_this.messageService.show(message_1.Severity.Info, nls.localize(11, null)));
                        });
                    }
                });
            });
        };
        AcceptLocalChangesAction = __decorate([
            __param(0, message_1.IMessageService),
            __param(1, instantiation_1.IInstantiationService),
            __param(2, editorService_1.IWorkbenchEditorService)
        ], AcceptLocalChangesAction);
        return AcceptLocalChangesAction;
    }(baseEditor_1.EditorInputAction));
    exports.AcceptLocalChangesAction = AcceptLocalChangesAction;
    // Revert changes to resolve a conflicting edit
    var RevertLocalChangesAction = (function (_super) {
        __extends(RevertLocalChangesAction, _super);
        function RevertLocalChangesAction(instantiationService, editorService) {
            _super.call(this, 'workbench.action.files.revert', nls.localize(12, null), 'conflict-editor-action revert-changes');
            this.instantiationService = instantiationService;
            this.editorService = editorService;
        }
        RevertLocalChangesAction.prototype.run = function () {
            var _this = this;
            var conflictInput = this.input;
            var model = conflictInput.getModel();
            // Revert on model
            return model.revert().then(function () {
                // Reopen file input
                var input = _this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, model.getResource(), mime_1.guessMimeTypes(model.getResource().fsPath).join(', '), void 0);
                return _this.editorService.openEditor(input, null, _this.position).then(function () {
                    // Dispose conflict input
                    conflictInput.dispose();
                });
            });
        };
        RevertLocalChangesAction = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, editorService_1.IWorkbenchEditorService)
        ], RevertLocalChangesAction);
        return RevertLocalChangesAction;
    }(baseEditor_1.EditorInputAction));
    exports.RevertLocalChangesAction = RevertLocalChangesAction;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/editors/textFileEditor", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/files/browser/editors/textFileEditor', 'vs/base/common/errors', 'vs/base/common/mime', 'vs/base/common/labels', 'vs/base/common/types', 'vs/base/common/paths', 'vs/base/common/actions', 'vs/workbench/parts/files/common/files', 'vs/workbench/parts/files/browser/saveErrorHandler', 'vs/workbench/browser/parts/editor/textEditor', 'vs/workbench/parts/files/common/editors/textFileEditorModel', 'vs/workbench/common/editor/binaryEditorModel', 'vs/workbench/parts/files/browser/editors/fileEditorInput', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/services/viewlet/common/viewletService', 'vs/platform/files/common/files', 'vs/platform/telemetry/common/telemetry', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/storage/common/storage', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/workbench/services/editor/common/editorService', 'vs/editor/common/services/modeService'], function (require, exports, winjs_base_1, nls, errors, mime_1, labels, types, paths, actions_1, files_1, saveErrorHandler_1, textEditor_1, textFileEditorModel_1, binaryEditorModel_1, fileEditorInput_1, quickOpenService_1, viewletService_1, files_2, telemetry_1, contextService_1, storage_1, configuration_1, event_1, instantiation_1, message_1, editorService_1, modeService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * An implementation of editor for file system resources.
     */
    var TextFileEditor = (function (_super) {
        __extends(TextFileEditor, _super);
        function TextFileEditor(telemetryService, fileService, textFileService, viewletService, quickOpenService, instantiationService, contextService, storageService, messageService, configurationService, eventService, editorService, modeService) {
            var _this = this;
            _super.call(this, TextFileEditor.ID, telemetryService, instantiationService, contextService, storageService, messageService, configurationService, eventService, editorService, modeService);
            this.fileService = fileService;
            this.textFileService = textFileService;
            this.viewletService = viewletService;
            this.quickOpenService = quickOpenService;
            // Since we are the one providing save-support for models, we hook up the error handler for saving
            textFileEditorModel_1.TextFileEditorModel.setSaveErrorHandler(instantiationService.createInstance(saveErrorHandler_1.SaveErrorHandler));
            // Clear view state for deleted files
            this.toUnbind.push(this.eventService.addListener(files_2.EventType.FILE_CHANGES, function (e) { return _this.onFilesChanged(e); }));
        }
        TextFileEditor.prototype.onFilesChanged = function (e) {
            var deleted = e.getDeleted();
            if (deleted && deleted.length) {
                this.clearTextEditorViewState(this.storageService, deleted.map(function (d) { return d.resource.toString(); }));
            }
        };
        TextFileEditor.prototype.getTitle = function () {
            return this.getInput() ? this.getInput().getName() : nls.localize(0, null);
        };
        TextFileEditor.prototype.setInput = function (input, options) {
            var _this = this;
            var oldInput = this.getInput();
            _super.prototype.setInput.call(this, input, options);
            // Detect options
            var forceOpen = options && options.forceOpen;
            // Same Input
            if (!forceOpen && input.matches(oldInput)) {
                // TextOptions (avoiding instanceof here for a reason, do not change!)
                if (options && types.isFunction(options.apply)) {
                    options.apply(this.getControl());
                }
                return winjs_base_1.TPromise.as(null);
            }
            // Remember view settings if input changes
            if (oldInput) {
                this.saveTextEditorViewState(this.storageService, oldInput.getResource().toString());
            }
            // Different Input (Reload)
            return this.editorService.resolveEditorModel(input, true /* Reload */).then(function (resolvedModel) {
                // There is a special case where the text editor has to handle binary file editor input: if a file with application/unknown
                // mime has been resolved and cached before, it maybe an actual instance of BinaryEditorModel. In this case our text
                // editor has to open this model using the binary editor. We return early in this case.
                if (resolvedModel instanceof binaryEditorModel_1.BinaryEditorModel && _this.openAsBinary(input, options)) {
                    return null;
                }
                // Assert Model interface
                if (!(resolvedModel instanceof textFileEditorModel_1.TextFileEditorModel)) {
                    return winjs_base_1.TPromise.wrapError('Invalid editor input. Text file editor requires a model instance of TextFileEditorModel.');
                }
                var textFileModel = resolvedModel;
                var textEditor = _this.getControl();
                // Assert Text Model
                if (!textFileModel.textEditorModel) {
                    return winjs_base_1.TPromise.wrapError('Unable to open the file because the associated text model is undefined.');
                }
                // First assert that the current input is still the one we expect
                // This prevents a race condition when reloading a content takes long
                // and the user meanwhile decided to open another file
                if (!_this.getInput() || _this.getInput().getResource().toString() !== textFileModel.getResource().toString()) {
                    return null;
                }
                // log the time it takes the editor to render the resource
                var mode = textFileModel.textEditorModel.getMode();
                var setModelEvent = _this.telemetryService.start('editorSetModel', {
                    mode: mode && mode.getId(),
                    resource: textFileModel.textEditorModel.getAssociatedResource().toString(),
                });
                // Editor
                textEditor.setModel(textFileModel.textEditorModel);
                // stop the event
                setModelEvent.stop();
                // TextOptions (avoiding instanceof here for a reason, do not change!)
                var optionsGotApplied = false;
                if (options && types.isFunction(options.apply)) {
                    optionsGotApplied = options.apply(textEditor);
                }
                // Otherwise restore View State
                if (!optionsGotApplied) {
                    var editorViewState = _this.loadTextEditorViewState(_this.storageService, _this.getInput().getResource().toString());
                    if (editorViewState) {
                        textEditor.restoreViewState(editorViewState);
                    }
                }
                // Add to working files if file is out of workspace
                if (!_this.contextService.isInsideWorkspace(textFileModel.getResource())) {
                    _this.textFileService.getWorkingFilesModel().addEntry(textFileModel.getResource());
                }
            }, function (error) {
                // In case we tried to open a file inside the text editor and the response
                // indicates that this is not a text file, reopen the file through the binary
                // editor by using application/octet-stream as mime.
                if (error.fileOperationResult === files_2.FileOperationResult.FILE_IS_BINARY && _this.openAsBinary(input, options)) {
                    return;
                }
                // Similar, handle case where we were asked to open a folder in the text editor.
                if (error.fileOperationResult === files_2.FileOperationResult.FILE_IS_DIRECTORY && _this.openAsFolder(input)) {
                    return;
                }
                // Offer to create a file from the error if we have a file not found and the name is valid
                if (error.fileOperationResult === files_2.FileOperationResult.FILE_NOT_FOUND && paths.isValidBasename(paths.basename(input.getResource().fsPath))) {
                    return winjs_base_1.TPromise.wrapError(errors.create(errors.toErrorMessage(error), { actions: [
                            message_1.CancelAction,
                            new actions_1.Action('workbench.files.action.createMissingFile', nls.localize(1, null), null, true, function () {
                                return _this.fileService.updateContent(input.getResource(), '').then(function () {
                                    // Add to working files
                                    _this.textFileService.getWorkingFilesModel().addEntry(input.getResource());
                                    // Open
                                    return _this.editorService.openEditor({
                                        resource: input.getResource(),
                                        mime: mime_1.MIME_TEXT
                                    });
                                });
                            })
                        ] }));
                }
                // Inform the user if the file is too large to open
                if (error.fileOperationResult === files_2.FileOperationResult.FILE_TOO_LARGE) {
                    _this.messageService.show(message_1.Severity.Info, nls.localize(2, null));
                    return;
                }
                // Otherwise make sure the error bubbles up
                return winjs_base_1.TPromise.wrapError(error);
            });
        };
        TextFileEditor.prototype.openAsBinary = function (input, options) {
            if (input instanceof fileEditorInput_1.FileEditorInput) {
                var fileEditorInput = input;
                var fileInputBinary = this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, fileEditorInput.getResource(), mime_1.MIME_BINARY, void 0);
                this.editorService.openEditor(fileInputBinary, options, this.position).done(null, errors.onUnexpectedError);
                return true;
            }
            return false;
        };
        TextFileEditor.prototype.openAsFolder = function (input) {
            var _this = this;
            // Since we cannot open a folder, we have to restore the previous input if any or close the editor
            var handleEditorPromise;
            var previousInput = this.quickOpenService.getEditorHistory()[1];
            if (previousInput) {
                handleEditorPromise = this.editorService.openEditor(previousInput, null, this.position);
            }
            else {
                handleEditorPromise = this.editorService.closeEditor(this);
            }
            handleEditorPromise.done(function () {
                // Best we can do is to reveal the folder in the explorer
                if (input instanceof fileEditorInput_1.FileEditorInput) {
                    var fileEditorInput_2 = input;
                    // Reveal if we have a workspace path
                    if (_this.contextService.isInsideWorkspace(fileEditorInput_2.getResource())) {
                        _this.viewletService.openViewlet(files_1.VIEWLET_ID, true).done(function (viewlet) {
                            return viewlet.getExplorerView().select(fileEditorInput_2.getResource(), true);
                        }, errors.onUnexpectedError);
                    }
                    else {
                        _this.messageService.show(message_1.Severity.Info, nls.localize(3, null, labels.getPathLabel(fileEditorInput_2.getResource())));
                    }
                }
            }, errors.onUnexpectedError);
            return true; // in any case we handled it
        };
        TextFileEditor.prototype.getCodeEditorOptions = function () {
            var options = _super.prototype.getCodeEditorOptions.call(this);
            var input = this.getInput();
            var inputName = input && input.getName();
            options.ariaLabel = inputName ? nls.localize(4, null, inputName) : nls.localize(5, null);
            return options;
        };
        TextFileEditor.prototype.supportsSplitEditor = function () {
            return true; // yes, we can!
        };
        TextFileEditor.prototype.clearInput = function () {
            // Keep editor view state in settings to restore when coming back
            if (this.input) {
                this.saveTextEditorViewState(this.storageService, this.input.getResource().toString());
            }
            // Clear Model
            this.getControl().setModel(null);
            // Pass to super
            _super.prototype.clearInput.call(this);
        };
        TextFileEditor.prototype.shutdown = function () {
            // Save View State
            if (this.input) {
                this.saveTextEditorViewState(this.storageService, this.input.getResource().toString());
            }
            // Call Super
            _super.prototype.shutdown.call(this);
        };
        TextFileEditor.ID = files_1.TEXT_FILE_EDITOR_ID;
        TextFileEditor = __decorate([
            __param(0, telemetry_1.ITelemetryService),
            __param(1, files_2.IFileService),
            __param(2, files_1.ITextFileService),
            __param(3, viewletService_1.IViewletService),
            __param(4, quickOpenService_1.IQuickOpenService),
            __param(5, instantiation_1.IInstantiationService),
            __param(6, contextService_1.IWorkspaceContextService),
            __param(7, storage_1.IStorageService),
            __param(8, message_1.IMessageService),
            __param(9, configuration_1.IConfigurationService),
            __param(10, event_1.IEventService),
            __param(11, editorService_1.IWorkbenchEditorService),
            __param(12, modeService_1.IModeService)
        ], TextFileEditor);
        return TextFileEditor;
    }(textEditor_1.BaseTextEditor));
    exports.TextFileEditor = TextFileEditor;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/fileActions.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/files/browser/fileActions.contribution', 'vs/platform/platform', 'vs/base/browser/ui/actionbar/actionbar', 'vs/workbench/browser/actionBarRegistry', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/parts/files/browser/fileActions', 'vs/workbench/parts/files/browser/saveErrorHandler', 'vs/platform/actions/common/actions', 'vs/workbench/common/actionRegistry', 'vs/platform/instantiation/common/instantiation', 'vs/platform/workspace/common/workspace', 'vs/platform/keybinding/common/keybindingService', 'vs/workbench/parts/files/common/explorerViewModel', 'vs/base/common/keyCodes'], function (require, exports, nls, platform_1, actionbar_1, actionBarRegistry_1, baseEditor_1, fileActions_1, saveErrorHandler_1, actions_1, actionRegistry_1, instantiation_1, workspace_1, keybindingService_1, explorerViewModel_1, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FilesViewerActionContributor = (function (_super) {
        __extends(FilesViewerActionContributor, _super);
        function FilesViewerActionContributor(instantiationService, contextService, keybindingService) {
            _super.call(this);
            this.instantiationService = instantiationService;
            this.contextService = contextService;
            this.keybindingService = keybindingService;
        }
        FilesViewerActionContributor.prototype.hasSecondaryActions = function (context) {
            var element = context.element;
            // Contribute only on Stat Objects (File Explorer)
            return element instanceof explorerViewModel_1.FileStat;
        };
        FilesViewerActionContributor.prototype.getSecondaryActions = function (context) {
            var stat = context.element;
            var tree = context.viewer;
            var actions = [];
            var separateOpen = false;
            // Open side by side
            if (!stat.isDirectory) {
                actions.push(this.instantiationService.createInstance(fileActions_1.OpenToSideAction, tree, stat.resource, false));
                separateOpen = true;
            }
            if (separateOpen) {
                actions.push(new actionbar_1.Separator(null, 50));
            }
            // Directory Actions
            if (stat.isDirectory) {
                // New File
                actions.push(this.instantiationService.createInstance(fileActions_1.NewFileAction, tree, stat));
                // New Folder
                actions.push(this.instantiationService.createInstance(fileActions_1.NewFolderAction, tree, stat));
                actions.push(new actionbar_1.Separator(null, 50));
            }
            else if (!stat.isDirectory) {
                // Run Compare
                var runCompareAction = this.instantiationService.createInstance(fileActions_1.CompareResourcesAction, stat.resource, tree);
                if (runCompareAction._isEnabled()) {
                    actions.push(runCompareAction);
                }
                // Select for Compare
                actions.push(this.instantiationService.createInstance(fileActions_1.SelectResourceForCompareAction, stat.resource, tree));
                actions.push(new actionbar_1.Separator(null, 100));
            }
            var workspace = this.contextService.getWorkspace();
            var isRoot = workspace && stat.resource.toString() === workspace.resource.toString();
            // Copy File/Folder
            if (!isRoot) {
                actions.push(this.instantiationService.createInstance(fileActions_1.CopyFileAction, tree, stat));
            }
            // Paste File/Folder
            if (stat.isDirectory) {
                actions.push(this.instantiationService.createInstance(fileActions_1.PasteFileAction, tree, stat));
            }
            // Rename File/Folder
            if (!isRoot) {
                actions.push(new actionbar_1.Separator(null, 150));
                actions.push(this.instantiationService.createInstance(fileActions_1.TriggerRenameFileAction, tree, stat));
            }
            // Delete File/Folder
            if (!isRoot) {
                actions.push(this.instantiationService.createInstance(fileActions_1.MoveFileToTrashAction, tree, stat));
            }
            // Set Order
            var curOrder = 10;
            for (var i = 0; i < actions.length; i++) {
                var action = actions[i];
                if (!action.order) {
                    curOrder += 10;
                    action.order = curOrder;
                }
                else {
                    curOrder = action.order;
                }
            }
            return actions;
        };
        FilesViewerActionContributor.prototype.getActionItem = function (context, action) {
            if (context && context.element instanceof explorerViewModel_1.FileStat) {
                // Any other item with keybinding
                var keybinding = fileActions_1.keybindingForAction(action.id);
                if (keybinding) {
                    return new actionbar_1.ActionItem(context, action, { label: true, keybinding: this.keybindingService.getLabelFor(keybinding) });
                }
            }
            return null;
        };
        FilesViewerActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, workspace_1.IWorkspaceContextService),
            __param(2, keybindingService_1.IKeybindingService)
        ], FilesViewerActionContributor);
        return FilesViewerActionContributor;
    }(actionBarRegistry_1.ActionBarContributor));
    var ConflictResolutionActionContributor = (function (_super) {
        __extends(ConflictResolutionActionContributor, _super);
        function ConflictResolutionActionContributor(instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
        }
        ConflictResolutionActionContributor.prototype.hasActionsForEditorInput = function (context) {
            return (context.input instanceof saveErrorHandler_1.ConflictResolutionDiffEditorInput);
        };
        ConflictResolutionActionContributor.prototype.getActionsForEditorInput = function (context) {
            return [
                this.instantiationService.createInstance(saveErrorHandler_1.AcceptLocalChangesAction),
                this.instantiationService.createInstance(saveErrorHandler_1.RevertLocalChangesAction)
            ];
        };
        ConflictResolutionActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], ConflictResolutionActionContributor);
        return ConflictResolutionActionContributor;
    }(baseEditor_1.EditorInputActionContributor));
    // Contribute to Viewers that show Files
    var actionBarRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
    actionBarRegistry.registerActionBarContributor(actionBarRegistry_1.Scope.VIEWER, FilesViewerActionContributor);
    // Contribute to Conflict Editor Inputs
    actionBarRegistry.registerActionBarContributor(actionBarRegistry_1.Scope.EDITOR, ConflictResolutionActionContributor);
    // Contribute Global Actions
    var category = nls.localize(0, null);
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.SaveFileAction, fileActions_1.SaveFileAction.ID, fileActions_1.SaveFileAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_S }), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.SaveAllAction, fileActions_1.SaveAllAction.ID, fileActions_1.SaveAllAction.LABEL), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.SaveFilesAction, fileActions_1.SaveFilesAction.ID, null /* only for programmatic trigger */));
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.RevertFileAction, fileActions_1.RevertFileAction.ID, fileActions_1.RevertFileAction.LABEL), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.GlobalNewFolderAction, fileActions_1.GlobalNewFolderAction.ID, fileActions_1.GlobalNewFolderAction.LABEL), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.GlobalCompareResourcesAction, fileActions_1.GlobalCompareResourcesAction.ID, fileActions_1.GlobalCompareResourcesAction.LABEL), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.CloseFileAction, fileActions_1.CloseFileAction.ID, fileActions_1.CloseFileAction.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.KEY_W) }), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.CloseOtherFilesAction, fileActions_1.CloseOtherFilesAction.ID, fileActions_1.CloseOtherFilesAction.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_W) }), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.CloseAllFilesAction, fileActions_1.CloseAllFilesAction.ID, fileActions_1.CloseAllFilesAction.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_W) }), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.OpenNextWorkingFile, fileActions_1.OpenNextWorkingFile.ID, fileActions_1.OpenNextWorkingFile.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.DownArrow) }), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.OpenPreviousWorkingFile, fileActions_1.OpenPreviousWorkingFile.ID, fileActions_1.OpenPreviousWorkingFile.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.UpArrow) }), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.AddToWorkingFiles, fileActions_1.AddToWorkingFiles.ID, fileActions_1.AddToWorkingFiles.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.Enter) }), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.FocusWorkingFiles, fileActions_1.FocusWorkingFiles.ID, fileActions_1.FocusWorkingFiles.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.KEY_E) }), category);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.FocusFilesExplorer, fileActions_1.FocusFilesExplorer.ID, fileActions_1.FocusFilesExplorer.LABEL), category);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/files.contribution", ["require", "exports", 'vs/base/common/uri', 'vs/workbench/browser/viewlet', 'vs/nls!vs/workbench/parts/files/browser/files.contribution', 'vs/platform/actions/common/actions', 'vs/platform/platform', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/browser/actions/quickOpenAction', 'vs/platform/configuration/common/configurationRegistry', 'vs/workbench/common/actionRegistry', 'vs/workbench/common/contributions', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/browser/quickopen', 'vs/workbench/parts/files/browser/files', 'vs/platform/files/common/files', 'vs/workbench/parts/files/common/files', 'vs/workbench/parts/files/browser/fileTracker', 'vs/workbench/parts/files/common/editors/saveParticipant', 'vs/workbench/parts/files/browser/editors/fileEditorInput', 'vs/workbench/parts/files/browser/editors/textFileEditor', 'vs/workbench/parts/files/browser/editors/binaryFileEditor', 'vs/platform/instantiation/common/descriptors', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/editor/common/editorService', 'vs/base/common/keyCodes', 'vs/base/common/platform', 'vs/css!./media/files.contribution'], function (require, exports, uri_1, viewlet_1, nls, actions_1, platform_1, quickOpenService_1, quickOpenAction_1, configurationRegistry_1, actionRegistry_1, contributions_1, baseEditor_1, quickopen_1, files_1, files_2, files_3, fileTracker_1, saveParticipant_1, fileEditorInput_1, textFileEditor_1, binaryFileEditor_1, descriptors_1, viewletService_1, editorService_1, keyCodes_1, platform) {
    'use strict';
    // Viewlet Action
    var OpenExplorerViewletAction = (function (_super) {
        __extends(OpenExplorerViewletAction, _super);
        function OpenExplorerViewletAction(id, label, viewletService, editorService) {
            _super.call(this, id, label, files_3.VIEWLET_ID, viewletService, editorService);
        }
        OpenExplorerViewletAction.ID = files_3.VIEWLET_ID;
        OpenExplorerViewletAction.LABEL = nls.localize(0, null);
        OpenExplorerViewletAction = __decorate([
            __param(2, viewletService_1.IViewletService),
            __param(3, editorService_1.IWorkbenchEditorService)
        ], OpenExplorerViewletAction);
        return OpenExplorerViewletAction;
    }(viewlet_1.ToggleViewletAction));
    exports.OpenExplorerViewletAction = OpenExplorerViewletAction;
    // Register Viewlet
    platform_1.Registry.as(viewlet_1.Extensions.Viewlets).registerViewlet(new viewlet_1.ViewletDescriptor('vs/workbench/parts/files/browser/explorerViewlet', 'ExplorerViewlet', files_3.VIEWLET_ID, nls.localize(1, null), 'explore', 0));
    platform_1.Registry.as(viewlet_1.Extensions.Viewlets).setDefaultViewletId(files_3.VIEWLET_ID);
    var openViewletKb = {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_E
    };
    // Register Action to Open Viewlet
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(OpenExplorerViewletAction, OpenExplorerViewletAction.ID, OpenExplorerViewletAction.LABEL, openViewletKb), nls.localize(2, null));
    // Register file editors
    platform_1.Registry.as(baseEditor_1.Extensions.Editors).registerEditor(new files_1.FileEditorDescriptor(textFileEditor_1.TextFileEditor.ID, // explicit dependency because we don't want these editors lazy loaded
    nls.localize(3, null), 'vs/workbench/parts/files/browser/editors/textFileEditor', 'TextFileEditor', [
        'text/*',
        // In case the mime type is unknown, we prefer the text file editor over the binary editor to leave a chance
        // of opening a potential text file properly. The resolution of the file in the text file editor will fail
        // early on in case the file is actually binary, to prevent downloading a potential large binary file.
        'application/unknown'
    ]), [
        new descriptors_1.SyncDescriptor(fileEditorInput_1.FileEditorInput)
    ]);
    platform_1.Registry.as(baseEditor_1.Extensions.Editors).registerEditor(new files_1.FileEditorDescriptor(binaryFileEditor_1.BinaryFileEditor.ID, // explicit dependency because we don't want these editors lazy loaded
    nls.localize(4, null), 'vs/workbench/parts/files/browser/editors/binaryFileEditor', 'BinaryFileEditor', [
        'image/*',
        'application/pdf',
        'audio/*',
        'video/*',
        'application/octet-stream'
    ]), [
        new descriptors_1.SyncDescriptor(fileEditorInput_1.FileEditorInput)
    ]);
    // Register default file input handler
    // Note: because of service injection, the descriptor needs to have the exact count
    // of arguments as the FileEditorInput constructor. Otherwise when creating an
    // instance through the instantiation service he will inject the services wrong!
    var descriptor = new descriptors_1.AsyncDescriptor('vs/workbench/parts/files/browser/editors/fileEditorInput', 'FileEditorInput', /* DO NOT REMOVE */ void 0, /* DO NOT REMOVE */ void 0, /* DO NOT REMOVE */ void 0);
    platform_1.Registry.as(baseEditor_1.Extensions.Editors).registerDefaultFileInput(descriptor);
    // Register Editor Input Factory
    var FileEditorInputFactory = (function () {
        function FileEditorInputFactory() {
        }
        FileEditorInputFactory.prototype.serialize = function (editorInput) {
            var fileEditorInput = editorInput;
            var fileInput = {
                resource: fileEditorInput.getResource().toString(),
                mime: fileEditorInput.getMime()
            };
            return JSON.stringify(fileInput);
        };
        FileEditorInputFactory.prototype.deserialize = function (instantiationService, serializedEditorInput) {
            var fileInput = JSON.parse(serializedEditorInput);
            return instantiationService.createInstance(fileEditorInput_1.FileEditorInput, uri_1.default.parse(fileInput.resource), fileInput.mime, void 0);
        };
        return FileEditorInputFactory;
    }());
    platform_1.Registry.as(baseEditor_1.Extensions.Editors).registerEditorInputFactory(files_3.FILE_EDITOR_INPUT_ID, FileEditorInputFactory);
    // Register File Tracker
    platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(fileTracker_1.FileTracker);
    // Register Save Participant
    platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(saveParticipant_1.SaveParticipant);
    // Configuration
    var configurationRegistry = platform_1.Registry.as(configurationRegistry_1.Extensions.Configuration);
    configurationRegistry.registerConfiguration({
        'id': 'files',
        'order': 7,
        'title': nls.localize(5, null),
        'type': 'object',
        'properties': {
            'files.exclude': {
                'type': 'object',
                'description': nls.localize(6, null),
                'default': { '**/.git': true, '**/.DS_Store': true },
                'additionalProperties': {
                    'anyOf': [
                        {
                            'type': 'boolean',
                            'description': nls.localize(7, null),
                        },
                        {
                            'type': 'object',
                            'properties': {
                                'when': {
                                    'type': 'string',
                                    'pattern': '\\w*\\$\\(basename\\)\\w*',
                                    'default': '$(basename).ext',
                                    'description': nls.localize(8, null)
                                }
                            }
                        }
                    ]
                }
            },
            'files.associations': {
                'type': 'object',
                'description': nls.localize(9, null),
            },
            'files.encoding': {
                'type': 'string',
                'enum': Object.keys(files_2.SUPPORTED_ENCODINGS),
                'default': 'utf8',
                'description': nls.localize(10, null),
            },
            'files.eol': {
                'type': 'string',
                'enum': [
                    '\n',
                    '\r\n'
                ],
                'default': (platform.isLinux || platform.isMacintosh) ? '\n' : '\r\n',
                'description': nls.localize(11, null),
            },
            'files.trimTrailingWhitespace': {
                'type': 'boolean',
                'default': false,
                'description': nls.localize(12, null)
            },
            'files.autoSave': {
                'type': 'string',
                'enum': [files_2.AutoSaveConfiguration.OFF, files_2.AutoSaveConfiguration.AFTER_DELAY, files_2.AutoSaveConfiguration.ON_FOCUS_CHANGE],
                'default': files_2.AutoSaveConfiguration.OFF,
                'description': nls.localize(13, null, files_2.AutoSaveConfiguration.OFF, files_2.AutoSaveConfiguration.AFTER_DELAY, files_2.AutoSaveConfiguration.ON_FOCUS_CHANGE, files_2.AutoSaveConfiguration.AFTER_DELAY)
            },
            'files.autoSaveDelay': {
                'type': 'number',
                'default': 1000,
                'description': nls.localize(14, null, files_2.AutoSaveConfiguration.AFTER_DELAY)
            },
            'files.watcherExclude': {
                'type': 'object',
                'default': (platform.isLinux || platform.isMacintosh) ? { '**/.git/objects/**': true, '**/node_modules/**': true } : { '**/.git/objects/**': true },
                'description': nls.localize(15, null)
            }
        }
    });
    configurationRegistry.registerConfiguration({
        'id': 'explorer',
        'order': 8,
        'title': nls.localize(16, null),
        'type': 'object',
        'properties': {
            'explorer.workingFiles.maxVisible': {
                'type': 'number',
                'description': nls.localize(17, null),
                'default': 9
            },
            'explorer.workingFiles.dynamicHeight': {
                'type': 'boolean',
                'description': nls.localize(18, null),
                'default': true
            }
        }
    });
    // Register quick open handler for working files
    var ALL_WORKING_FILES_PREFIX = '~';
    var OpenWorkingFileByNameAction = (function (_super) {
        __extends(OpenWorkingFileByNameAction, _super);
        function OpenWorkingFileByNameAction(actionId, actionLabel, quickOpenService) {
            _super.call(this, actionId, actionLabel, ALL_WORKING_FILES_PREFIX, quickOpenService);
        }
        OpenWorkingFileByNameAction.ID = 'workbench.files.action.workingFilesPicker';
        OpenWorkingFileByNameAction.LABEL = nls.localize(19, null);
        OpenWorkingFileByNameAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], OpenWorkingFileByNameAction);
        return OpenWorkingFileByNameAction;
    }(quickOpenAction_1.QuickOpenAction));
    platform_1.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/files/browser/workingFilesPicker', 'WorkingFilesPicker', ALL_WORKING_FILES_PREFIX, [
        {
            prefix: ALL_WORKING_FILES_PREFIX,
            needsEditor: false,
            description: nls.localize(20, null)
        }
    ]));
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(OpenWorkingFileByNameAction, OpenWorkingFileByNameAction.ID, OpenWorkingFileByNameAction.LABEL, {
        primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_P)
    }), nls.localize(21, null));
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/views/explorerViewer", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/files/browser/views/explorerViewer', 'vs/base/common/lifecycle', 'vs/base/common/objects', 'vs/base/browser/dom', 'vs/base/common/uri', 'vs/base/common/mime', 'vs/base/common/async', 'vs/base/common/paths', 'vs/base/common/errors', 'vs/base/common/types', 'vs/base/common/actions', 'vs/base/common/comparers', 'vs/base/browser/ui/inputbox/inputBox', 'vs/base/browser/builder', 'vs/base/common/platform', 'vs/base/common/glob', 'vs/workbench/browser/actionBarRegistry', 'vs/workbench/parts/files/common/files', 'vs/platform/files/common/files', 'vs/workbench/parts/files/browser/editors/fileEditorInput', 'vs/workbench/parts/files/browser/fileActions', 'vs/workbench/common/editor', 'vs/base/parts/tree/browser/tree', 'vs/base/common/labels', 'vs/base/parts/tree/browser/treeDnd', 'vs/base/parts/tree/browser/treeDefaults', 'vs/base/parts/tree/browser/actionsRenderer', 'vs/workbench/parts/files/common/explorerViewModel', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/part/common/partService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/contextview/browser/contextView', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/platform/progress/common/progress', 'vs/platform/telemetry/common/telemetry', 'vs/base/common/keyCodes'], function (require, exports, winjs_base_1, nls, lifecycle, objects, DOM, uri_1, mime_1, async, paths, errors, types_1, Actions, comparers, inputBox_1, builder_1, platform, glob, actionBarRegistry_1, files_1, files_2, fileEditorInput_1, fileActions_1, editor_1, tree_1, labels, treeDnd_1, treeDefaults_1, actionsRenderer_1, explorerViewModel_1, editorService_1, partService_1, contextService_1, contextView_1, event_1, instantiation_1, message_1, progress_1, telemetry_1, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var FileDataSource = (function () {
        function FileDataSource(progressService, messageService, fileService, partService, contextService) {
            this.progressService = progressService;
            this.messageService = messageService;
            this.fileService = fileService;
            this.partService = partService;
            this.workspace = contextService.getWorkspace();
        }
        FileDataSource.prototype.getId = function (tree, stat) {
            return stat.getId();
        };
        FileDataSource.prototype.hasChildren = function (tree, stat) {
            return stat.isDirectory;
        };
        FileDataSource.prototype.getChildren = function (tree, stat) {
            var _this = this;
            // Return early if stat is already resolved
            if (stat.isDirectoryResolved) {
                return winjs_base_1.TPromise.as(stat.children);
            }
            else {
                // Resolve
                var promise = this.fileService.resolveFile(stat.resource, { resolveSingleChildDescendants: true }).then(function (dirStat) {
                    // Convert to view model
                    var modelDirStat = explorerViewModel_1.FileStat.create(dirStat);
                    // Add children to folder
                    for (var i = 0; i < modelDirStat.children.length; i++) {
                        stat.addChild(modelDirStat.children[i]);
                    }
                    stat.isDirectoryResolved = true;
                    return stat.children;
                }, function (e) {
                    _this.messageService.show(message_1.Severity.Error, e);
                    return []; // we could not resolve any children because of an error
                });
                this.progressService.showWhile(promise, this.partService.isCreated() ? 800 : 3200 /* less ugly initial startup */);
                return promise;
            }
        };
        FileDataSource.prototype.getParent = function (tree, stat) {
            if (!stat) {
                return winjs_base_1.TPromise.as(null); // can be null if nothing selected in the tree
            }
            // Return if root reached
            if (this.workspace && stat.resource.toString() === this.workspace.resource.toString()) {
                return winjs_base_1.TPromise.as(null);
            }
            // Return if parent already resolved
            if (stat.parent) {
                return winjs_base_1.TPromise.as(stat.parent);
            }
            // We never actually resolve the parent from the disk for performance reasons. It wouldnt make
            // any sense to resolve parent by parent with requests to walk up the chain. Instead, the explorer
            // makes sure to properly resolve a deep path to a specific file and merges the result with the model.
            return winjs_base_1.TPromise.as(null);
        };
        FileDataSource = __decorate([
            __param(0, progress_1.IProgressService),
            __param(1, message_1.IMessageService),
            __param(2, files_2.IFileService),
            __param(3, partService_1.IPartService),
            __param(4, contextService_1.IWorkspaceContextService)
        ], FileDataSource);
        return FileDataSource;
    }());
    exports.FileDataSource = FileDataSource;
    var FileActionProvider = (function (_super) {
        __extends(FileActionProvider, _super);
        function FileActionProvider(state) {
            _super.call(this);
            this.state = state;
        }
        FileActionProvider.prototype.hasActions = function (tree, stat) {
            if (stat instanceof explorerViewModel_1.NewStatPlaceholder) {
                return false;
            }
            return _super.prototype.hasActions.call(this, tree, stat);
        };
        FileActionProvider.prototype.getActions = function (tree, stat) {
            if (stat instanceof explorerViewModel_1.NewStatPlaceholder) {
                return winjs_base_1.TPromise.as([]);
            }
            return _super.prototype.getActions.call(this, tree, stat);
        };
        FileActionProvider.prototype.hasSecondaryActions = function (tree, stat) {
            if (stat instanceof explorerViewModel_1.NewStatPlaceholder) {
                return false;
            }
            return _super.prototype.hasSecondaryActions.call(this, tree, stat);
        };
        FileActionProvider.prototype.getSecondaryActions = function (tree, stat) {
            if (stat instanceof explorerViewModel_1.NewStatPlaceholder) {
                return winjs_base_1.TPromise.as([]);
            }
            return _super.prototype.getSecondaryActions.call(this, tree, stat);
        };
        FileActionProvider.prototype.runAction = function (tree, stat, arg, context) {
            var _this = this;
            if (context === void 0) { context = {}; }
            context = objects.mixin({
                viewletState: this.state,
                stat: stat
            }, context);
            if (!types_1.isString(arg)) {
                var action = arg;
                if (action.enabled) {
                    return action.run(context);
                }
                return null;
            }
            var id = arg;
            var promise = this.hasActions(tree, stat) ? this.getActions(tree, stat) : winjs_base_1.TPromise.as([]);
            return promise.then(function (actions) {
                for (var i = 0, len = actions.length; i < len; i++) {
                    if (actions[i].id === id && actions[i].enabled) {
                        return actions[i].run(context);
                    }
                }
                promise = _this.hasSecondaryActions(tree, stat) ? _this.getSecondaryActions(tree, stat) : winjs_base_1.TPromise.as([]);
                return promise.then(function (actions) {
                    for (var i = 0, len = actions.length; i < len; i++) {
                        if (actions[i].id === id && actions[i].enabled) {
                            return actions[i].run(context);
                        }
                    }
                    return null;
                });
            });
        };
        return FileActionProvider;
    }(actionBarRegistry_1.ContributableActionProvider));
    exports.FileActionProvider = FileActionProvider;
    var FileViewletState = (function () {
        function FileViewletState() {
            this._actionProvider = new FileActionProvider(this);
            this.editableStats = Object.create(null);
        }
        Object.defineProperty(FileViewletState.prototype, "actionProvider", {
            get: function () {
                return this._actionProvider;
            },
            enumerable: true,
            configurable: true
        });
        FileViewletState.prototype.getEditableData = function (stat) {
            return this.editableStats[stat.resource && stat.resource.toString()];
        };
        FileViewletState.prototype.setEditable = function (stat, editableData) {
            if (editableData) {
                this.editableStats[stat.resource && stat.resource.toString()] = editableData;
            }
        };
        FileViewletState.prototype.clearEditable = function (stat) {
            delete this.editableStats[stat.resource && stat.resource.toString()];
        };
        return FileViewletState;
    }());
    exports.FileViewletState = FileViewletState;
    var ActionRunner = (function (_super) {
        __extends(ActionRunner, _super);
        function ActionRunner(state) {
            _super.call(this);
            this.viewletState = state;
        }
        ActionRunner.prototype.run = function (action, context) {
            return _super.prototype.run.call(this, action, { viewletState: this.viewletState });
        };
        return ActionRunner;
    }(Actions.ActionRunner));
    exports.ActionRunner = ActionRunner;
    // Explorer Renderer
    var FileRenderer = (function (_super) {
        __extends(FileRenderer, _super);
        function FileRenderer(state, actionRunner, contextViewService) {
            _super.call(this, {
                actionProvider: state.actionProvider,
                actionRunner: actionRunner
            });
            this.contextViewService = contextViewService;
            this.state = state;
        }
        FileRenderer.prototype.getContentHeight = function (tree, element) {
            return 22;
        };
        FileRenderer.prototype.renderContents = function (tree, stat, domElement, previousCleanupFn) {
            var _this = this;
            var el = builder_1.$(domElement).clearChildren();
            var item = builder_1.$('.explorer-item').addClass(this.iconClass(stat)).appendTo(el);
            // File/Folder label
            var editableData = this.state.getEditableData(stat);
            if (!editableData) {
                var label = builder_1.$('.explorer-item-label').appendTo(item);
                builder_1.$('a.plain').text(stat.name).appendTo(label);
                return null;
            }
            // Input field (when creating a new file or folder or renaming)
            var inputBox = new inputBox_1.InputBox(item.getHTMLElement(), this.contextViewService, {
                validationOptions: {
                    validation: editableData.validator,
                    showMessage: true
                },
                ariaLabel: nls.localize(0, null)
            });
            var value = stat.name || '';
            var lastDot = value.lastIndexOf('.');
            inputBox.value = value;
            inputBox.select({ start: 0, end: lastDot > 0 && !stat.isDirectory ? lastDot : value.length });
            inputBox.focus();
            var done = async.once(function (commit) {
                tree.clearHighlight();
                if (commit && inputBox.value) {
                    _this.state.actionProvider.runAction(tree, stat, editableData.action, { value: inputBox.value });
                }
                setTimeout(function () {
                    tree.DOMFocus();
                    lifecycle.disposeAll(toDispose);
                }, 0);
            });
            var toDispose = [
                inputBox,
                DOM.addStandardDisposableListener(inputBox.inputElement, DOM.EventType.KEY_DOWN, function (e) {
                    if (e.equals(keyCodes_1.CommonKeybindings.ENTER)) {
                        if (inputBox.validate()) {
                            done(true);
                        }
                    }
                    else if (e.equals(keyCodes_1.CommonKeybindings.ESCAPE)) {
                        done(false);
                    }
                }),
                DOM.addDisposableListener(inputBox.inputElement, 'blur', function () {
                    done(inputBox.isInputValid());
                })
            ];
            return function () { return done(true); };
        };
        FileRenderer.prototype.iconClass = function (element) {
            if (element.isDirectory) {
                return 'folder-icon';
            }
            return 'text-file-icon';
        };
        FileRenderer = __decorate([
            __param(2, contextView_1.IContextViewService)
        ], FileRenderer);
        return FileRenderer;
    }(actionsRenderer_1.ActionsRenderer));
    exports.FileRenderer = FileRenderer;
    // Explorer Accessibility Provider
    var FileAccessibilityProvider = (function () {
        function FileAccessibilityProvider() {
        }
        FileAccessibilityProvider.prototype.getAriaLabel = function (tree, stat) {
            return nls.localize(1, null, stat.name);
        };
        return FileAccessibilityProvider;
    }());
    exports.FileAccessibilityProvider = FileAccessibilityProvider;
    // Explorer Controller
    var FileController = (function (_super) {
        __extends(FileController, _super);
        function FileController(state, editorService, textFileService, contextMenuService, eventService, instantiationService, telemetryService, contextService) {
            _super.call(this, { clickBehavior: treeDefaults_1.ClickBehavior.ON_MOUSE_DOWN });
            this.editorService = editorService;
            this.textFileService = textFileService;
            this.contextMenuService = contextMenuService;
            this.eventService = eventService;
            this.instantiationService = instantiationService;
            this.telemetryService = telemetryService;
            this.contextService = contextService;
            this.workspace = contextService.getWorkspace();
            this.didCatchEnterDown = false;
            this.downKeyBindingDispatcher.set(platform.isMacintosh ? keyCodes_1.CommonKeybindings.CTRLCMD_DOWN_ARROW : keyCodes_1.CommonKeybindings.ENTER, this.onEnterDown.bind(this));
            this.upKeyBindingDispatcher.set(platform.isMacintosh ? keyCodes_1.CommonKeybindings.CTRLCMD_DOWN_ARROW : keyCodes_1.CommonKeybindings.ENTER, this.onEnterUp.bind(this));
            if (platform.isMacintosh) {
                this.upKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.WINCTRL_ENTER, this.onModifierEnterUp.bind(this)); // Mac: somehow Cmd+Enter does not work
            }
            else {
                this.upKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.CTRLCMD_ENTER, this.onModifierEnterUp.bind(this)); // Mac: somehow Cmd+Enter does not work
            }
            this.downKeyBindingDispatcher.set(platform.isMacintosh ? keyCodes_1.CommonKeybindings.ENTER : keyCodes_1.CommonKeybindings.F2, this.onF2.bind(this));
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.CTRLCMD_C, this.onCopy.bind(this));
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.CTRLCMD_V, this.onPaste.bind(this));
            if (platform.isMacintosh) {
                this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.CTRLCMD_UP_ARROW, this.onLeft.bind(this));
                this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.CTRLCMD_BACKSPACE, this.onDelete.bind(this));
            }
            else {
                this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.DELETE, this.onDelete.bind(this));
                this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.SHIFT_DELETE, this.onDelete.bind(this));
            }
            this.state = state;
        }
        /* protected */ FileController.prototype.onLeftClick = function (tree, stat, event, origin) {
            if (origin === void 0) { origin = 'mouse'; }
            var payload = { origin: origin };
            var isDoubleClick = (origin === 'mouse' && event.detail === 2);
            // Handle Highlight Mode
            if (tree.getHighlight()) {
                // Cancel Event
                event.preventDefault();
                event.stopPropagation();
                tree.clearHighlight(payload);
                return false;
            }
            // Handle root
            if (this.workspace && stat.resource.toString() === this.workspace.resource.toString()) {
                tree.clearFocus(payload);
                tree.clearSelection(payload);
                return false;
            }
            // Cancel Event
            var isMouseDown = event && event.browserEvent && event.browserEvent.type === 'mousedown';
            if (!isMouseDown) {
                event.preventDefault(); // we cannot preventDefault onMouseDown because this would break DND otherwise
            }
            event.stopPropagation();
            // Set DOM focus
            tree.DOMFocus();
            // Expand / Collapse
            tree.toggleExpansion(stat);
            // Allow to unselect
            if (event.shiftKey && !(stat instanceof explorerViewModel_1.NewStatPlaceholder)) {
                var selection = tree.getSelection();
                if (selection && selection.length > 0 && selection[0] === stat) {
                    tree.clearSelection(payload);
                }
            }
            else if (!(stat instanceof explorerViewModel_1.NewStatPlaceholder)) {
                var preserveFocus = !isDoubleClick;
                tree.setFocus(stat, payload);
                if (isDoubleClick) {
                    event.preventDefault(); // focus moves to editor, we need to prevent default
                }
                if (!stat.isDirectory) {
                    tree.setSelection([stat], payload);
                    this.openEditor(stat, preserveFocus, event && (event.ctrlKey || event.metaKey));
                    // Doubleclick: add to working files set
                    if (isDoubleClick) {
                        this.textFileService.getWorkingFilesModel().addEntry(stat);
                    }
                }
            }
            return true;
        };
        FileController.prototype.onContextMenu = function (tree, stat, event) {
            var _this = this;
            if (event.target && event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
                return false;
            }
            event.preventDefault();
            event.stopPropagation();
            tree.setFocus(stat);
            if (!this.state.actionProvider.hasSecondaryActions(tree, stat)) {
                return true;
            }
            var anchor = { x: event.posx + 1, y: event.posy };
            this.contextMenuService.showContextMenu({
                getAnchor: function () { return anchor; },
                getActions: function () { return _this.state.actionProvider.getSecondaryActions(tree, stat); },
                getActionItem: this.state.actionProvider.getActionItem.bind(this.state.actionProvider, tree, stat),
                getKeyBinding: function (a) { return fileActions_1.keybindingForAction(a.id); },
                getActionsContext: function () {
                    return {
                        viewletState: _this.state,
                        stat: stat
                    };
                },
                onHide: function (wasCancelled) {
                    if (wasCancelled) {
                        tree.DOMFocus();
                    }
                }
            });
            return true;
        };
        FileController.prototype.onEnterDown = function (tree, event) {
            if (tree.getHighlight()) {
                return false;
            }
            var payload = { origin: 'keyboard' };
            var stat = tree.getFocus();
            if (stat) {
                // Directory: Toggle expansion
                if (stat.isDirectory) {
                    tree.toggleExpansion(stat);
                }
                else {
                    tree.setFocus(stat, payload);
                    this.openEditor(stat, false, false);
                }
            }
            this.didCatchEnterDown = true;
            return true;
        };
        FileController.prototype.onEnterUp = function (tree, event) {
            if (!this.didCatchEnterDown || tree.getHighlight()) {
                return false;
            }
            var stat = tree.getFocus();
            if (stat && !stat.isDirectory) {
                this.openEditor(stat, false, false);
            }
            this.didCatchEnterDown = false;
            return true;
        };
        FileController.prototype.onModifierEnterUp = function (tree, event) {
            if (tree.getHighlight()) {
                return false;
            }
            var stat = tree.getFocus();
            if (stat && !stat.isDirectory) {
                this.openEditor(stat, false, true);
            }
            this.didCatchEnterDown = false;
            return true;
        };
        FileController.prototype.onCopy = function (tree, event) {
            var stat = tree.getFocus();
            if (stat) {
                this.runAction(tree, stat, 'workbench.files.action.copyFile').done();
                return true;
            }
            return false;
        };
        FileController.prototype.onPaste = function (tree, event) {
            var stat = tree.getFocus() || tree.getInput();
            if (stat) {
                var pasteAction = this.instantiationService.createInstance(fileActions_1.PasteFileAction, tree, stat);
                if (pasteAction._isEnabled()) {
                    pasteAction.run().done(null, errors.onUnexpectedError);
                    return true;
                }
            }
            return false;
        };
        FileController.prototype.openEditor = function (stat, preserveFocus, sideBySide) {
            if (stat && !stat.isDirectory) {
                var editorInput = this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, stat.resource, stat.mime, void 0);
                var editorOptions = new editor_1.EditorOptions();
                if (preserveFocus) {
                    editorOptions.preserveFocus = true;
                }
                this.telemetryService.publicLog('workbenchActionExecuted', { id: 'workbench.files.openFile', from: 'explorer' });
                this.editorService.openEditor(editorInput, editorOptions, sideBySide).done(null, errors.onUnexpectedError);
            }
        };
        FileController.prototype.onF2 = function (tree, event) {
            var stat = tree.getFocus();
            if (stat) {
                this.runAction(tree, stat, 'workbench.files.action.triggerRename').done();
                return true;
            }
            return false;
        };
        FileController.prototype.onDelete = function (tree, event) {
            var useTrash = !event.shiftKey;
            var stat = tree.getFocus();
            if (stat) {
                this.runAction(tree, stat, useTrash ? 'workbench.files.action.moveFileToTrash' : 'workbench.files.action.deleteFile').done();
                return true;
            }
            return false;
        };
        FileController.prototype.runAction = function (tree, stat, id) {
            return this.state.actionProvider.runAction(tree, stat, id);
        };
        FileController = __decorate([
            __param(1, editorService_1.IWorkbenchEditorService),
            __param(2, files_1.ITextFileService),
            __param(3, contextView_1.IContextMenuService),
            __param(4, event_1.IEventService),
            __param(5, instantiation_1.IInstantiationService),
            __param(6, telemetry_1.ITelemetryService),
            __param(7, contextService_1.IWorkspaceContextService)
        ], FileController);
        return FileController;
    }(treeDefaults_1.DefaultController));
    exports.FileController = FileController;
    // Explorer Sorter
    var FileSorter = (function () {
        function FileSorter() {
        }
        FileSorter.prototype.compare = function (tree, statA, statB) {
            if (statA.isDirectory && !statB.isDirectory) {
                return -1;
            }
            if (statB.isDirectory && !statA.isDirectory) {
                return 1;
            }
            if (statA.isDirectory && statB.isDirectory) {
                return statA.name.toLowerCase().localeCompare(statB.name.toLowerCase());
            }
            if (statA instanceof explorerViewModel_1.NewStatPlaceholder) {
                return -1;
            }
            if (statB instanceof explorerViewModel_1.NewStatPlaceholder) {
                return 1;
            }
            return comparers.compareFileNames(statA.name, statB.name);
        };
        return FileSorter;
    }());
    exports.FileSorter = FileSorter;
    // Explorer Filter
    var FileFilter = (function () {
        function FileFilter(contextService) {
            this.contextService = contextService;
            this.hiddenExpression = Object.create(null);
        }
        FileFilter.prototype.updateConfiguration = function (configuration) {
            var excludesConfig = (configuration && configuration.files && configuration.files.exclude) || Object.create(null);
            var needsRefresh = !objects.equals(this.hiddenExpression, excludesConfig);
            this.hiddenExpression = objects.clone(excludesConfig); // do not keep the config, as it gets mutated under our hoods
            return needsRefresh;
        };
        FileFilter.prototype.isVisible = function (tree, stat) {
            return this.doIsVisible(stat);
        };
        FileFilter.prototype.doIsVisible = function (stat) {
            if (stat instanceof explorerViewModel_1.NewStatPlaceholder) {
                return true; // always visible
            }
            var siblings = stat.parent && stat.parent.children && stat.parent.children.map(function (c) { return c.name; });
            // Hide those that match Hidden Patterns
            if (glob.match(this.hiddenExpression, this.contextService.toWorkspaceRelativePath(stat.resource), siblings)) {
                return false; // hidden through pattern
            }
            return true;
        };
        FileFilter = __decorate([
            __param(0, contextService_1.IWorkspaceContextService)
        ], FileFilter);
        return FileFilter;
    }());
    exports.FileFilter = FileFilter;
    // Explorer Drag And Drop Controller
    var FileDragAndDrop = (function () {
        function FileDragAndDrop(messageService, contextService, eventService, progressService, fileService, instantiationService, textFileService) {
            this.messageService = messageService;
            this.contextService = contextService;
            this.eventService = eventService;
            this.progressService = progressService;
            this.fileService = fileService;
            this.instantiationService = instantiationService;
            this.textFileService = textFileService;
        }
        FileDragAndDrop.prototype.getDragURI = function (tree, stat) {
            return stat.resource && stat.resource.toString();
        };
        FileDragAndDrop.prototype.onDragStart = function (tree, data, originalEvent) {
            var sources = data.getData();
            var source = null;
            if (sources.length > 0) {
                source = sources[0];
            }
            // When dragging folders, make sure to collapse them to free up some space
            if (source && source.isDirectory && tree.isExpanded(source)) {
                tree.collapse(source, false);
            }
            // Native only: when a DownloadURL attribute is defined on the data transfer it is possible to
            // drag a file from the browser to the desktop and have it downloaded there.
            if (!(data instanceof treeDnd_1.DesktopDragAndDropData)) {
                if (source && !source.isDirectory) {
                    originalEvent.dataTransfer.setData('DownloadURL', [mime_1.MIME_BINARY, source.name, source.resource.toString()].join(':'));
                }
            }
        };
        FileDragAndDrop.prototype.onDragOver = function (tree, data, target, originalEvent) {
            var isCopy = originalEvent && ((originalEvent.ctrlKey && !platform.isMacintosh) || (originalEvent.altKey && platform.isMacintosh));
            var fromDesktop = data instanceof treeDnd_1.DesktopDragAndDropData;
            if (this.contextService.getOptions().readOnly) {
                return tree_1.DRAG_OVER_REJECT;
            }
            // Desktop DND
            if (fromDesktop) {
                var dragData = data.getData();
                var types = dragData.types;
                var typesArray = [];
                for (var i = 0; i < types.length; i++) {
                    typesArray.push(types[i]);
                }
                if (typesArray.length === 0 || !typesArray.some(function (type) { return type === 'Files'; })) {
                    return tree_1.DRAG_OVER_REJECT;
                }
            }
            else if (data instanceof treeDnd_1.ExternalElementsDragAndDropData) {
                return tree_1.DRAG_OVER_REJECT;
            }
            else {
                var sources = data.getData();
                if (!Array.isArray(sources)) {
                    return tree_1.DRAG_OVER_REJECT;
                }
                if (sources.some(function (source) {
                    if (source instanceof explorerViewModel_1.NewStatPlaceholder) {
                        return true; // NewStatPlaceholders can not be moved
                    }
                    if (source.resource.toString() === target.resource.toString()) {
                        return true; // Can not move anything onto itself
                    }
                    if (!isCopy && paths.dirname(source.resource.fsPath) === target.resource.fsPath) {
                        return true; // Can not move a file to the same parent unless we copy
                    }
                    if (paths.isEqualOrParent(target.resource.fsPath, source.resource.fsPath)) {
                        return true; // Can not move a parent folder into one of its children
                    }
                    return false;
                })) {
                    return tree_1.DRAG_OVER_REJECT;
                }
            }
            // All
            if (target.isDirectory) {
                return fromDesktop || isCopy ? tree_1.DRAG_OVER_ACCEPT_BUBBLE_DOWN_COPY : tree_1.DRAG_OVER_ACCEPT_BUBBLE_DOWN;
            }
            if (target.resource.toString() !== this.contextService.getWorkspace().resource.toString()) {
                return fromDesktop || isCopy ? tree_1.DRAG_OVER_ACCEPT_BUBBLE_UP_COPY : tree_1.DRAG_OVER_ACCEPT_BUBBLE_UP;
            }
            return tree_1.DRAG_OVER_REJECT;
        };
        FileDragAndDrop.prototype.drop = function (tree, data, target, originalEvent) {
            var _this = this;
            var promise = winjs_base_1.TPromise.as(null);
            // Desktop DND (Import file)
            if (data instanceof treeDnd_1.DesktopDragAndDropData) {
                var importAction = this.instantiationService.createInstance(fileActions_1.ImportFileAction, tree, target, null);
                promise = importAction.run({
                    input: {
                        files: data.getData().files
                    }
                });
            }
            else {
                var source_1 = data.getData()[0];
                var isCopy_1 = (originalEvent.ctrlKey && !platform.isMacintosh) || (originalEvent.altKey && platform.isMacintosh);
                promise = tree.expand(target).then(function () {
                    // Reuse action if user copies
                    if (isCopy_1) {
                        var copyAction = _this.instantiationService.createInstance(fileActions_1.DuplicateFileAction, tree, source_1, target);
                        return copyAction.run();
                    }
                    // Handle dirty
                    var saveOrRevertPromise = winjs_base_1.TPromise.as(null);
                    if (_this.textFileService.isDirty(source_1.resource)) {
                        var res = _this.textFileService.confirmSave([source_1.resource]);
                        if (res === files_1.ConfirmResult.SAVE) {
                            saveOrRevertPromise = _this.textFileService.save(source_1.resource);
                        }
                        else if (res === files_1.ConfirmResult.DONT_SAVE) {
                            saveOrRevertPromise = _this.textFileService.revert(source_1.resource);
                        }
                        else if (res === files_1.ConfirmResult.CANCEL) {
                            return winjs_base_1.TPromise.as(null);
                        }
                    }
                    // For move, first check if file is dirty and save
                    return saveOrRevertPromise.then(function () {
                        // If the file is still dirty, do not touch it because a save is pending to the disk and we can not abort it
                        if (_this.textFileService.isDirty(source_1.resource)) {
                            _this.messageService.show(message_1.Severity.Warning, nls.localize(2, null, labels.getPathLabel(source_1.resource)));
                            return winjs_base_1.TPromise.as(null);
                        }
                        var targetResource = uri_1.default.file(paths.join(target.resource.fsPath, source_1.name));
                        var didHandleConflict = false;
                        var onMove = function (result) {
                            _this.eventService.emit('files.internal:fileChanged', new files_1.LocalFileChangeEvent(source_1.clone(), result));
                        };
                        // Move File/Folder and emit event
                        return _this.fileService.moveFile(source_1.resource, targetResource).then(onMove, function (error) {
                            // Conflict
                            if (error.fileOperationResult === files_2.FileOperationResult.FILE_MOVE_CONFLICT) {
                                didHandleConflict = true;
                                var confirm_1 = {
                                    message: nls.localize(3, null, source_1.name),
                                    detail: nls.localize(4, null),
                                    primaryButton: nls.localize(5, null)
                                };
                                if (_this.messageService.confirm(confirm_1)) {
                                    return _this.fileService.moveFile(source_1.resource, targetResource, true).then(function (result) {
                                        var fakeTargetState = new explorerViewModel_1.FileStat(targetResource);
                                        _this.eventService.emit('files.internal:fileChanged', new files_1.LocalFileChangeEvent(fakeTargetState, null));
                                        onMove(result);
                                    }, function (error) {
                                        _this.messageService.show(message_1.Severity.Error, error);
                                    });
                                }
                                return;
                            }
                            _this.messageService.show(message_1.Severity.Error, error);
                        });
                    });
                }, errors.onUnexpectedError);
            }
            this.progressService.showWhile(promise, 800);
            promise.done(null, errors.onUnexpectedError);
        };
        FileDragAndDrop = __decorate([
            __param(0, message_1.IMessageService),
            __param(1, contextService_1.IWorkspaceContextService),
            __param(2, event_1.IEventService),
            __param(3, progress_1.IProgressService),
            __param(4, files_2.IFileService),
            __param(5, instantiation_1.IInstantiationService),
            __param(6, files_1.ITextFileService)
        ], FileDragAndDrop);
        return FileDragAndDrop;
    }());
    exports.FileDragAndDrop = FileDragAndDrop;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/views/explorerView", ["require", "exports", 'vs/nls!vs/workbench/parts/files/browser/views/explorerView', 'vs/base/common/winjs.base', 'vs/base/browser/builder', 'vs/base/common/uri', 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/paths', 'vs/workbench/browser/actionBarRegistry', 'vs/base/parts/tree/browser/treeImpl', 'vs/workbench/common/events', 'vs/platform/files/common/files', 'vs/workbench/parts/files/browser/fileActions', 'vs/workbench/parts/files/browser/editors/fileEditorInput', 'vs/workbench/parts/files/browser/views/explorerViewer', 'vs/base/common/lifecycle', 'vs/base/browser/dom', 'vs/workbench/browser/viewlet', 'vs/workbench/parts/files/common/explorerViewModel', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/part/common/partService', 'vs/platform/storage/common/storage', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/progress/common/progress', 'vs/platform/workspace/common/workspace', 'vs/platform/contextview/browser/contextView', 'vs/platform/message/common/message'], function (require, exports, nls, winjs_base_1, builder_1, uri_1, async_1, errors, paths, actionBarRegistry_1, treeImpl_1, events_1, files_1, fileActions_1, fileEditorInput_1, explorerViewer_1, lifecycle, DOM, viewlet_1, explorerViewModel_1, editorService_1, partService_1, storage_1, configuration_1, event_1, instantiation_1, progress_1, workspace_1, contextView_1, message_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExplorerView = (function (_super) {
        __extends(ExplorerView, _super);
        function ExplorerView(viewletState, actionRunner, settings, messageService, contextMenuService, instantiationService, eventService, storageService, contextService, progressService, editorService, fileService, partService, configurationService) {
            _super.call(this, actionRunner, false, nls.localize(0, null), messageService, contextMenuService);
            this.instantiationService = instantiationService;
            this.eventService = eventService;
            this.storageService = storageService;
            this.contextService = contextService;
            this.progressService = progressService;
            this.editorService = editorService;
            this.fileService = fileService;
            this.partService = partService;
            this.configurationService = configurationService;
            this.workspace = contextService.getWorkspace();
            this.settings = settings;
            this.viewletState = viewletState;
            this.actionRunner = actionRunner;
            this.explorerRefreshDelayer = new async_1.ThrottledDelayer(ExplorerView.EXPLORER_FILE_CHANGES_REFRESH_DELAY);
            this.explorerImportDelayer = new async_1.ThrottledDelayer(ExplorerView.EXPLORER_IMPORT_REFRESH_DELAY);
        }
        ExplorerView.prototype.renderHeader = function (container) {
            _super.prototype.renderHeader.call(this, container);
            var titleDiv = builder_1.$('div.title').appendTo(container);
            builder_1.$('span').text(this.workspace.name).appendTo(titleDiv);
        };
        ExplorerView.prototype.renderBody = function (container) {
            this.treeContainer = _super.prototype.renderViewTree.call(this, container);
            DOM.addClass(this.treeContainer, 'explorer-folders-view');
            this.tree = this.createViewer(builder_1.$(this.treeContainer));
            this.fillToolBar();
        };
        ExplorerView.prototype.fillToolBar = function () {
            var actions = [];
            // New File
            actions.push(this.instantiationService.createInstance(fileActions_1.NewFileAction, this.getViewer(), null));
            // New Folder
            actions.push(this.instantiationService.createInstance(fileActions_1.NewFolderAction, this.getViewer(), null));
            // Refresh
            actions.push(this.instantiationService.createInstance(fileActions_1.RefreshViewExplorerAction, this, 'explorer-action refresh-explorer'));
            // Collapse
            actions.push(this.instantiationService.createInstance(viewlet_1.CollapseAction, this.getViewer(), true, 'explorer-action collapse-explorer'));
            // Set Order
            for (var i = 0; i < actions.length; i++) {
                var action = actions[i];
                action.order = 10 * (i + 1);
            }
            this.toolBar.setActions(actionBarRegistry_1.prepareActions(actions), [])();
        };
        ExplorerView.prototype.create = function () {
            var _this = this;
            // Load Config
            return this.configurationService.loadConfiguration().then(function (configuration) {
                // Update configuration
                _this.onConfigurationUpdated(configuration);
                // Load and Fill Viewer
                return _this.refresh(false, false).then(function () {
                    // When the explorer viewer is loaded, listen to changes to the editor input
                    _this.toDispose.push(_this.eventService.addListener2(events_1.EventType.EDITOR_INPUT_CHANGING, function (e) { return _this.onEditorInputChanging(e); }));
                    // Also handle configuration updates
                    _this.toDispose.push(_this.configurationService.addListener2(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) { return _this.onConfigurationUpdated(e.config, true); }));
                });
            });
        };
        ExplorerView.prototype.onEditorInputChanging = function (e) {
            // During workbench startup, the editor area might restore more than one editor from a previous
            // session. When this happens there might be editor input changing events for side editors that
            // don't have focus. In these cases we do not adjust explorer selection for non-focused editors
            // because we only want to react for the editor that has focus.
            if (!this.partService.isCreated() && e.editorOptions && e.editorOptions.preserveFocus) {
                return;
            }
            var clearSelection = true;
            // Handle File Input
            if (e.editorInput && e.editorInput instanceof fileEditorInput_1.FileEditorInput) {
                var fileInput = e.editorInput;
                // Always remember last opened file
                this.settings[ExplorerView.MEMENTO_LAST_ACTIVE_FILE_RESOURCE] = fileInput.getResource().toString();
                // Reveal file if input is FileEditorInput
                if (this.isVisible) {
                    if (this.contextService.isInsideWorkspace(fileInput.getResource())) {
                        this.select(fileInput.getResource(), false /* Prevent reveal so that upon opening a file the tree does not jump around */).done(null, errors.onUnexpectedError);
                        clearSelection = false;
                    }
                }
            }
            // Otherwise clear
            if (this.isVisible && clearSelection) {
                this.explorerViewer.clearSelection();
            }
        };
        ExplorerView.prototype.onConfigurationUpdated = function (configuration, refresh) {
            // Push down config updates to components of viewer
            var needsRefresh = false;
            if (this.filter) {
                needsRefresh = this.filter.updateConfiguration(configuration);
            }
            // Refresh viewer as needed
            if (refresh && needsRefresh) {
                this.refresh(false, false).done(null, errors.onUnexpectedError);
            }
        };
        ExplorerView.prototype.focusBody = function () {
            _super.prototype.focusBody.call(this);
            // Open the focused element in the editor if there is currently no file opened
            var input = this.editorService.getActiveEditorInput();
            if (!input || !(input instanceof fileEditorInput_1.FileEditorInput)) {
                this.openFocusedElement();
            }
        };
        ExplorerView.prototype.setVisible = function (visible) {
            var _this = this;
            return _super.prototype.setVisible.call(this, visible).then(function () {
                // Show
                if (visible) {
                    // If a refresh was requested and we are now visible, run it
                    var refreshPromise = winjs_base_1.TPromise.as(null);
                    if (_this.shouldRefresh) {
                        refreshPromise = _this.refresh(false, false);
                        _this.shouldRefresh = false; // Reset flag
                    }
                    // Always reveal the current navigated file in explorer if input is file editor input
                    var activeResource_1 = _this.getActiveEditorInputResource();
                    if (activeResource_1) {
                        return refreshPromise.then(function () {
                            return _this.select(activeResource_1);
                        });
                    }
                    // Return now if the workbench has not yet been created - in this case the workbench takes care of restoring last used editors
                    if (!_this.partService.isCreated()) {
                        return winjs_base_1.TPromise.as(null);
                    }
                    // Otherwise restore last used file: By lastActiveFileResource
                    var root = _this.getInput();
                    var lastActiveFileResource = void 0;
                    if (_this.settings[ExplorerView.MEMENTO_LAST_ACTIVE_FILE_RESOURCE]) {
                        lastActiveFileResource = uri_1.default.parse(_this.settings[ExplorerView.MEMENTO_LAST_ACTIVE_FILE_RESOURCE]);
                    }
                    if (lastActiveFileResource && root && root.find(lastActiveFileResource)) {
                        var editorInput = _this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, lastActiveFileResource, void 0, void 0);
                        _this.openOrFocusEditor(editorInput).done(null, errors.onUnexpectedError);
                        return refreshPromise;
                    }
                    // Otherwise restore last used file: By Explorer selection
                    return refreshPromise.then(function () {
                        _this.openFocusedElement();
                    });
                }
            });
        };
        ExplorerView.prototype.openFocusedElement = function () {
            var stat = this.explorerViewer.getFocus();
            if (stat && !stat.isDirectory) {
                var editorInput = this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, stat.resource, stat.mime, void 0);
                this.openOrFocusEditor(editorInput).done(null, errors.onUnexpectedError);
                return true;
            }
            return false;
        };
        ExplorerView.prototype.openOrFocusEditor = function (input) {
            // First try to find if input already visible
            var editors = this.editorService.getVisibleEditors();
            if (editors) {
                for (var i = 0; i < editors.length; i++) {
                    var editor = editors[i];
                    if (input.matches(editor.input)) {
                        return this.editorService.focusEditor(editor);
                    }
                }
            }
            // Otherwise open in active slot
            return this.editorService.openEditor(input);
        };
        ExplorerView.prototype.getActiveEditorInputResource = function () {
            // Try with Editor Input
            var input = this.editorService.getActiveEditorInput();
            if (input && input instanceof fileEditorInput_1.FileEditorInput) {
                return input.getResource();
            }
            return null;
        };
        ExplorerView.prototype.getInput = function () {
            return this.explorerViewer ? this.explorerViewer.getInput() : null;
        };
        ExplorerView.prototype.createViewer = function (container) {
            var _this = this;
            var dataSource = this.instantiationService.createInstance(explorerViewer_1.FileDataSource);
            var renderer = this.instantiationService.createInstance(explorerViewer_1.FileRenderer, this.viewletState, this.actionRunner);
            var controller = this.instantiationService.createInstance(explorerViewer_1.FileController, this.viewletState);
            var sorter = new explorerViewer_1.FileSorter();
            this.filter = this.instantiationService.createInstance(explorerViewer_1.FileFilter);
            var dnd = this.instantiationService.createInstance(explorerViewer_1.FileDragAndDrop);
            var accessibility = this.instantiationService.createInstance(explorerViewer_1.FileAccessibilityProvider);
            this.explorerViewer = new treeImpl_1.Tree(container.getHTMLElement(), {
                dataSource: dataSource,
                renderer: renderer,
                controller: controller,
                sorter: sorter,
                filter: this.filter,
                dnd: dnd,
                accessibilityProvider: accessibility
            }, {
                autoExpandSingleChildren: true,
                ariaLabel: nls.localize(1, null)
            });
            this.toDispose.push(lifecycle.toDisposable(function () { return renderer.dispose(); }));
            // Update Viewer based on File Change Events
            this.toDispose.push(this.eventService.addListener2('files.internal:fileChanged', function (e) { return _this.onLocalFileChange(e); }));
            this.toDispose.push(this.eventService.addListener2(files_1.EventType.FILE_CHANGES, function (e) { return _this.onFileChanges(e); }));
            return this.explorerViewer;
        };
        ExplorerView.prototype.onLocalFileChange = function (e) {
            var _this = this;
            var modelElement;
            var parent;
            var parentResource;
            var parentElement;
            // Add
            if (e.gotAdded()) {
                var addedElement = e.getAfter();
                parentResource = uri_1.default.file(paths.dirname(addedElement.resource.fsPath));
                parentElement = this.getInput().find(parentResource);
                if (parentElement) {
                    // Add the new file to its parent (Model)
                    var childElement_1 = explorerViewModel_1.FileStat.create(addedElement);
                    parentElement.addChild(childElement_1);
                    var refreshPromise = function () {
                        // Refresh the Parent (View)
                        return _this.explorerViewer.refresh(parentElement).then(function () {
                            return _this.reveal(childElement_1, 0.5).then(function () {
                                // Focus new element
                                _this.explorerViewer.setFocus(childElement_1);
                                // Open new file in editor
                                if (!childElement_1.isDirectory) {
                                    var editorInput = _this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, childElement_1.resource, childElement_1.mime, void 0);
                                    return _this.editorService.openEditor(editorInput);
                                }
                            });
                        });
                    };
                    // For file imports, use a delayer to not refresh too many times when multiple files are imported
                    if (e instanceof fileActions_1.FileImportedEvent) {
                        this.explorerImportDelayer.trigger(refreshPromise).done(null, errors.onUnexpectedError);
                    }
                    else {
                        refreshPromise().done(null, errors.onUnexpectedError);
                    }
                }
            }
            else if (e.gotMoved()) {
                var oldElement = e.getBefore();
                var newElement = e.getAfter();
                var oldParentResource = uri_1.default.file(paths.dirname(oldElement.resource.fsPath));
                var newParentResource = uri_1.default.file(paths.dirname(newElement.resource.fsPath));
                // Only update focus if renamed/moved element is selected
                var updateFocus_1 = false;
                var focus_1 = this.explorerViewer.getFocus();
                if (focus_1 && focus_1.resource && focus_1.resource.toString() === oldElement.resource.toString()) {
                    updateFocus_1 = true;
                }
                // Handle Rename
                if (oldParentResource && newParentResource && oldParentResource.toString() === newParentResource.toString()) {
                    modelElement = this.getInput().find(oldElement.resource);
                    if (modelElement) {
                        if (!modelElement.isDirectory && !modelElement.mime) {
                            return;
                        }
                        // Rename File (Model)
                        modelElement.rename(newElement);
                        // Update Parent (View)
                        parent = modelElement.parent;
                        if (parent) {
                            this.explorerViewer.refresh(parent).done(function () {
                                // Select in Viewer if set
                                if (updateFocus_1) {
                                    _this.explorerViewer.setFocus(modelElement);
                                }
                            }, errors.onUnexpectedError);
                        }
                    }
                }
                else if (oldParentResource && newParentResource) {
                    var oldParent_1 = this.getInput().find(oldParentResource);
                    var newParent_1 = this.getInput().find(newParentResource);
                    modelElement = this.getInput().find(oldElement.resource);
                    if (oldParent_1 && newParent_1 && modelElement) {
                        // Move in Model
                        modelElement.move(newParent_1, function (callback) {
                            // Update old parent
                            _this.explorerViewer.refresh(oldParent_1, true).done(callback, errors.onUnexpectedError);
                        }, function () {
                            // Update new parent
                            _this.explorerViewer.refresh(newParent_1, true).done(function () {
                                return _this.explorerViewer.expand(newParent_1);
                            }, errors.onUnexpectedError);
                        });
                    }
                }
            }
            else if (e.gotDeleted()) {
                var deletedElement = e.getBefore();
                modelElement = this.getInput().find(deletedElement.resource);
                if (modelElement && modelElement.parent) {
                    parent = modelElement.parent;
                    // Remove Element from Parent (Model)
                    parent.removeChild(modelElement);
                    // Refresh Parent (View)
                    this.explorerViewer.refresh(parent).done(function () {
                        // Ensure viewer has keyboard focus if event originates from viewer
                        _this.explorerViewer.DOMFocus();
                    }, errors.onUnexpectedError);
                }
            }
            else if (e instanceof fileActions_1.FileImportedEvent) {
                var importedElement_1 = e.getAfter();
                parentResource = uri_1.default.file(paths.dirname(importedElement_1.resource.fsPath));
                parentElement = this.getInput().find(parentResource);
                if (parentElement) {
                    this.explorerViewer.refresh(parentElement).then(function () {
                        var editorInput = _this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, importedElement_1.resource, importedElement_1.mime, void 0);
                        return _this.editorService.openEditor(editorInput);
                    }).done(null, errors.onUnexpectedError);
                }
            }
            else if (this.workspace && e.gotUpdated() && e.getAfter().resource.toString() === this.workspace.resource.toString() && !this.explorerViewer.getHighlight()) {
                this.refreshFromEvent();
            }
        };
        ExplorerView.prototype.onFileChanges = function (e) {
            var _this = this;
            // Ensure memento state does not capture a deleted file
            var lastActiveResource = this.settings[ExplorerView.MEMENTO_LAST_ACTIVE_FILE_RESOURCE];
            if (lastActiveResource && e.contains(uri_1.default.parse(lastActiveResource), files_1.FileChangeType.DELETED)) {
                this.settings[ExplorerView.MEMENTO_LAST_ACTIVE_FILE_RESOURCE] = null;
            }
            // Check if an explorer refresh is necessary (delayed to give internal events a chance to react first)
            // Note: there is no guarantee when the internal events are fired vs real ones. Code has to deal with the fact that one might
            // be fired first over the other or not at all.
            winjs_base_1.TPromise.timeout(ExplorerView.EXPLORER_FILE_CHANGES_REACT_DELAY).then(function () {
                if (!_this.shouldRefresh && _this.shouldRefreshFromEvent(e)) {
                    _this.refreshFromEvent();
                }
            });
        };
        ExplorerView.prototype.shouldRefreshFromEvent = function (e) {
            // Filter to the ones we care
            e = this.filterToAddRemovedOnWorkspacePath(e, function (event, segments) {
                if (segments[0] !== '.git') {
                    return true; // we like all things outside .git
                }
                return segments.length === 1; // we only care about the .git folder itself
            });
            // We only ever refresh from files/folders that got added or deleted
            if (e.gotAdded() || e.gotDeleted()) {
                var added = e.getAdded();
                var deleted = e.getDeleted();
                var root = this.getInput();
                if (!root) {
                    return false;
                }
                // Check added: Refresh if added file/folder is not part of resolved root and parent is part of it
                var ignoredPaths = {};
                for (var i = 0; i < added.length; i++) {
                    var change = added[i];
                    if (!this.contextService.isInsideWorkspace(change.resource)) {
                        continue; // out of workspace file
                    }
                    // Find parent
                    var parent_1 = paths.dirname(change.resource.fsPath);
                    // Continue if parent was already determined as to be ignored
                    if (ignoredPaths[parent_1]) {
                        continue;
                    }
                    // Compute if parent is visible and added file not yet part of it
                    var parentStat = root.find(uri_1.default.file(parent_1));
                    if (parentStat && parentStat.isDirectoryResolved && !root.find(change.resource)) {
                        return true;
                    }
                    // Keep track of path that can be ignored for faster lookup
                    if (!parentStat || !parentStat.isDirectoryResolved) {
                        ignoredPaths[parent_1] = true;
                    }
                }
                // Check deleted: Refresh if deleted file/folder part of resolved root
                for (var j = 0; j < deleted.length; j++) {
                    var del = deleted[j];
                    if (!this.contextService.isInsideWorkspace(del.resource)) {
                        continue; // out of workspace file
                    }
                    if (root.find(del.resource)) {
                        return true;
                    }
                }
            }
            return false;
        };
        ExplorerView.prototype.filterToAddRemovedOnWorkspacePath = function (e, fn) {
            var _this = this;
            return new files_1.FileChangesEvent(e.changes.filter(function (change) {
                if (change.type === files_1.FileChangeType.UPDATED) {
                    return false; // we only want added / removed
                }
                var workspacePath = _this.contextService.toWorkspaceRelativePath(change.resource);
                if (!workspacePath) {
                    return false; // not inside workspace
                }
                var segments = workspacePath.split(/\//);
                return fn(change, segments);
            }));
        };
        ExplorerView.prototype.refreshFromEvent = function () {
            var _this = this;
            if (this.isVisible) {
                this.explorerRefreshDelayer.trigger(function () {
                    if (!_this.explorerViewer.getHighlight()) {
                        return _this.refresh(false, false);
                    }
                    return winjs_base_1.TPromise.as(null);
                }).done(null, errors.onUnexpectedError);
            }
            else {
                this.shouldRefresh = true;
            }
        };
        /**
         * Refresh the contents of the explorer to get up to date data from the disk about the file structure.
         *
         * @param focus if set to true, the explorer viewer will receive keyboard focus
         * @param reveal if set to true, the current active input will be revealed in the explorer
         */
        ExplorerView.prototype.refresh = function (focus, reveal, instantProgress) {
            var _this = this;
            var root = this.getInput();
            var targetsToResolve = [];
            var targetsToExpand = [];
            if (this.settings[ExplorerView.MEMENTO_EXPANDED_FOLDER_RESOURCES]) {
                targetsToExpand = this.settings[ExplorerView.MEMENTO_EXPANDED_FOLDER_RESOURCES].map(function (e) { return uri_1.default.parse(e); });
            }
            // First time refresh: Receive target through active editor input or selection and also include settings from previous session
            if (!root) {
                var activeResource = this.getActiveEditorInputResource();
                if (activeResource) {
                    targetsToResolve.push(activeResource);
                }
                if (targetsToExpand.length) {
                    targetsToResolve.push.apply(targetsToResolve, targetsToExpand);
                }
            }
            else {
                this.getResolvedDirectories(root, targetsToResolve);
            }
            // Determine the path to reveal if we are set to reveal
            var revealResource;
            if (reveal) {
                revealResource = this.getActiveEditorInputResource();
                if (!revealResource) {
                    var selection = this.explorerViewer.getSelection();
                    if (selection && selection.length === 1) {
                        revealResource = selection[0].resource;
                    }
                }
            }
            // Load Root Stat with given target path configured
            var options = { resolveTo: targetsToResolve };
            var promise = this.fileService.resolveFile(this.workspace.resource, options).then(function (stat) {
                var explorerPromise;
                // Convert to model
                var modelStat = explorerViewModel_1.FileStat.create(stat, options.resolveTo);
                // First time refresh: The stat becomes the input of the viewer
                if (!root) {
                    explorerPromise = _this.explorerViewer.setInput(modelStat).then(function () {
                        // Make sure to expand all folders that where expanded in the previous session
                        if (targetsToExpand) {
                            return _this.explorerViewer.expandAll(targetsToExpand.map(function (expand) { return _this.getInput().find(expand); }));
                        }
                        return winjs_base_1.TPromise.as(null);
                    });
                }
                else {
                    explorerViewModel_1.FileStat.mergeLocalWithDisk(modelStat, root);
                    explorerPromise = _this.explorerViewer.refresh(root);
                }
                return explorerPromise.then(function () {
                    var revealPromise;
                    // Reveal if path is set
                    if (revealResource) {
                        revealPromise = _this.select(revealResource);
                    }
                    else {
                        revealPromise = winjs_base_1.TPromise.as(null);
                    }
                    return revealPromise.then(function () {
                        // Focus if set
                        if (focus) {
                            _this.explorerViewer.DOMFocus();
                        }
                    });
                });
            }, function (e) { return winjs_base_1.TPromise.wrapError(e); });
            this.progressService.showWhile(promise, instantProgress ? 0 : this.partService.isCreated() ? 800 : 3200 /* less ugly initial startup */);
            return promise;
        };
        /**
         * Given a stat, fills an array of path that make all folders below the stat that are resolved directories.
         */
        ExplorerView.prototype.getResolvedDirectories = function (stat, resolvedDirectories) {
            if (stat.isDirectoryResolved) {
                if (stat.resource.toString() !== this.workspace.resource.toString()) {
                    // Drop those path which are parents of the current one
                    for (var i = resolvedDirectories.length - 1; i >= 0; i--) {
                        var resource = resolvedDirectories[i];
                        if (stat.resource.toString().indexOf(resource.toString()) === 0) {
                            resolvedDirectories.splice(i);
                        }
                    }
                    // Add to the list of path to resolve
                    resolvedDirectories.push(stat.resource);
                }
                // Recurse into children
                for (var i = 0; i < stat.children.length; i++) {
                    var child = stat.children[i];
                    this.getResolvedDirectories(child, resolvedDirectories);
                }
            }
        };
        /**
         * Selects and reveal the file element provided by the given resource if its found in the explorer. Will try to
         * resolve the path from the disk in case the explorer is not yet expanded to the file yet.
         */
        ExplorerView.prototype.select = function (resource, reveal) {
            var _this = this;
            if (reveal === void 0) { reveal = true; }
            // Require valid path
            if (!resource || resource.toString() === this.workspace.resource.toString()) {
                return winjs_base_1.TPromise.as(null);
            }
            // If path already selected, just reveal and return
            var currentSelection = this.explorerViewer.getSelection();
            for (var i = 0; i < currentSelection.length; i++) {
                if (currentSelection[i].resource.toString() === resource.toString()) {
                    return reveal ? this.reveal(currentSelection[i], 0.5) : winjs_base_1.TPromise.as(null);
                }
            }
            // First try to get the stat object from the input to avoid a roundtrip
            var root = this.getInput();
            if (!root) {
                return winjs_base_1.TPromise.as(null);
            }
            var fileStat = root.find(resource);
            if (fileStat) {
                return this.doRevealAndSelect(fileStat);
            }
            // Stat needs to be resolved first and then revealed
            var options = { resolveTo: [resource] };
            return this.fileService.resolveFile(this.workspace.resource, options).then(function (stat) {
                // Convert to model
                var modelStat = explorerViewModel_1.FileStat.create(stat, options.resolveTo);
                // Update Input with disk Stat
                explorerViewModel_1.FileStat.mergeLocalWithDisk(modelStat, root);
                // Select and Reveal
                return _this.explorerViewer.refresh(root).then(function () {
                    return _this.doRevealAndSelect(root.find(resource));
                });
            }, function (e) { return _this.messageService.show(message_1.Severity.Error, e); });
        };
        ExplorerView.prototype.doRevealAndSelect = function (fileStat) {
            var _this = this;
            if (!fileStat) {
                return winjs_base_1.TPromise.as(null);
            }
            // Special case: we are asked to reveal and select an element that is not visible
            // In this case we take the parent element so that we are at least close to it.
            if (!this.filter.isVisible(this.tree, fileStat)) {
                fileStat = fileStat.parent;
                if (!fileStat) {
                    return winjs_base_1.TPromise.as(null);
                }
            }
            return this.reveal(fileStat, 0.5).then(function () {
                if (!fileStat.isDirectory) {
                    _this.explorerViewer.setSelection([fileStat]); // Since folders can not be opened, only select files
                }
                _this.explorerViewer.setFocus(fileStat);
            });
        };
        ExplorerView.prototype.shutdown = function () {
            var _this = this;
            // Keep list of expanded folders to restore on next load
            var root = this.getInput();
            if (root) {
                var expanded = this.explorerViewer.getExpandedElements()
                    .filter(function (e) { return e.resource.toString() !== _this.workspace.resource.toString(); })
                    .map(function (e) { return e.resource.toString(); });
                this.settings[ExplorerView.MEMENTO_EXPANDED_FOLDER_RESOURCES] = expanded;
            }
            _super.prototype.shutdown.call(this);
        };
        ExplorerView.prototype.dispose = function () {
            if (this.toolBar) {
                this.toolBar.dispose();
            }
            _super.prototype.dispose.call(this);
        };
        ExplorerView.EXPLORER_FILE_CHANGES_REACT_DELAY = 500; // delay in ms to react to file changes to give our internal events a chance to react first
        ExplorerView.EXPLORER_FILE_CHANGES_REFRESH_DELAY = 100; // delay in ms to refresh the explorer from disk file changes
        ExplorerView.EXPLORER_IMPORT_REFRESH_DELAY = 300; // delay in ms to refresh the explorer from imports
        ExplorerView.MEMENTO_LAST_ACTIVE_FILE_RESOURCE = 'explorer.memento.lastActiveFileResource';
        ExplorerView.MEMENTO_EXPANDED_FOLDER_RESOURCES = 'explorer.memento.expandedFolderResources';
        ExplorerView = __decorate([
            __param(3, message_1.IMessageService),
            __param(4, contextView_1.IContextMenuService),
            __param(5, instantiation_1.IInstantiationService),
            __param(6, event_1.IEventService),
            __param(7, storage_1.IStorageService),
            __param(8, workspace_1.IWorkspaceContextService),
            __param(9, progress_1.IProgressService),
            __param(10, editorService_1.IWorkbenchEditorService),
            __param(11, files_1.IFileService),
            __param(12, partService_1.IPartService),
            __param(13, configuration_1.IConfigurationService)
        ], ExplorerView);
        return ExplorerView;
    }(viewlet_1.CollapsibleViewletView));
    exports.ExplorerView = ExplorerView;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/common/workingFilesModel", ["require", "exports", 'vs/base/common/arrays', 'vs/base/common/uri', 'vs/base/common/event', 'vs/base/common/paths', 'vs/base/common/errors', 'vs/base/common/labels', 'vs/base/common/lifecycle', 'vs/workbench/parts/files/common/files', 'vs/platform/files/common/files', 'vs/workbench/common/events', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/part/common/partService', 'vs/workbench/services/workspace/common/contextService', 'vs/workbench/common/editor', 'vs/platform/storage/common/storage', 'vs/platform/event/common/event'], function (require, exports, arrays, uri_1, event_1, paths, errors, labels, lifecycle_1, files_1, files_2, events_1, untitledEditorService_1, editorService_1, partService_1, contextService_1, editor_1, storage_1, event_2) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var WorkingFilesModel = (function () {
        function WorkingFilesModel(storageService, eventService, partService, contextService, editorService, untitledEditorService, textFileService) {
            this.storageService = storageService;
            this.eventService = eventService;
            this.partService = partService;
            this.contextService = contextService;
            this.editorService = editorService;
            this.untitledEditorService = untitledEditorService;
            this.textFileService = textFileService;
            this.pathLabelProvider = new labels.PathLabelProvider(this.contextService);
            this.entries = [];
            this.toDispose = [];
            this.mapEntryToResource = Object.create(null);
            this._onModelChange = new event_1.Emitter();
            this._onWorkingFileChange = new event_1.Emitter();
            this.load();
            this.registerListeners();
        }
        WorkingFilesModel.prototype.registerListeners = function () {
            var _this = this;
            // listen to global file changes to clean up model
            this.toDispose.push(this.eventService.addListener2('files.internal:fileChanged', function (e) { return _this.onLocalFileChange(e); }));
            this.toDispose.push(this.eventService.addListener2(files_2.EventType.FILE_CHANGES, function (e) { return _this.onFileChanges(e); }));
            // listen to untitled
            this.toDispose.push(this.eventService.addListener2(events_1.EventType.UNTITLED_FILE_DIRTY, function (e) { return _this.onUntitledFileDirty(e); }));
            this.toDispose.push(this.eventService.addListener2(events_1.EventType.UNTITLED_FILE_DELETED, function (e) { return _this.onUntitledFileDeleted(e); }));
            // listen to files being changed locally
            this.toDispose.push(this.eventService.addListener2(files_1.EventType.FILE_DIRTY, function (e) { return _this.onTextFileDirty(e); }));
            this.toDispose.push(this.eventService.addListener2(files_1.EventType.FILE_SAVE_ERROR, function (e) { return _this.onTextFileSaveError(e); }));
            this.toDispose.push(this.eventService.addListener2(files_1.EventType.FILE_SAVED, function (e) { return _this.onTextFileSaved(e); }));
            this.toDispose.push(this.eventService.addListener2(files_1.EventType.FILE_REVERTED, function (e) { return _this.onTextFileReverted(e); }));
            // clean up model on set input error
            this.toDispose.push(this.eventService.addListener2(events_1.EventType.EDITOR_SET_INPUT_ERROR, function (e) { return _this.onEditorInputSetError(e); }));
        };
        WorkingFilesModel.prototype.onUntitledFileDirty = function (e) {
            this.updateDirtyState(e.resource, true);
        };
        WorkingFilesModel.prototype.onUntitledFileDeleted = function (e) {
            var entry = this.mapEntryToResource[e.resource.toString()];
            if (entry) {
                this.removeEntry(entry);
            }
        };
        WorkingFilesModel.prototype.onTextFileDirty = function (e) {
            if (this.textFileService.getAutoSaveMode() !== files_1.AutoSaveMode.AFTER_SHORT_DELAY) {
                this.updateDirtyState(e.getAfter().resource, true); // no indication needed when auto save is enabled for short delay
            }
            else {
                this.addEntry(e.getAfter().resource);
            }
        };
        WorkingFilesModel.prototype.onTextFileSaveError = function (e) {
            this.updateDirtyState(e.getAfter().resource, true);
        };
        WorkingFilesModel.prototype.onTextFileSaved = function (e) {
            this.updateDirtyState(e.getAfter().resource, false);
        };
        WorkingFilesModel.prototype.onTextFileReverted = function (e) {
            if (this.hasEntry(e.getAfter().resource)) {
                this.updateDirtyState(e.getAfter().resource, false);
            }
        };
        WorkingFilesModel.prototype.updateDirtyState = function (resource, isDirty) {
            // Add to model
            var entry = this.addEntry(resource);
            // Toggle dirty
            var oldDirty = entry.dirty;
            entry.setDirty(isDirty);
            if (oldDirty !== isDirty) {
                this.fireWorkingFileChange(entry);
            }
        };
        WorkingFilesModel.prototype.onLocalFileChange = function (e) {
            if (e.gotMoved()) {
                this.moveEntry(e.getBefore().resource, e.getAfter().resource);
            }
        };
        WorkingFilesModel.prototype.onFileChanges = function (e) {
            for (var resource in this.mapEntryToResource) {
                var entry = this.mapEntryToResource[resource];
                if (!entry.dirty && e.contains(uri_1.default.parse(resource), files_2.FileChangeType.DELETED)) {
                    this.removeEntry(entry);
                }
            }
        };
        WorkingFilesModel.prototype.onEditorInputSetError = function (e) {
            var fileInput = editor_1.asFileEditorInput(e.editorInput);
            if (fileInput) {
                var entry = this.mapEntryToResource[fileInput.getResource().toString()];
                if (entry && !entry.dirty) {
                    this.removeEntry(fileInput.getResource());
                }
            }
        };
        Object.defineProperty(WorkingFilesModel.prototype, "onModelChange", {
            get: function () {
                return this._onModelChange.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WorkingFilesModel.prototype, "onWorkingFileChange", {
            get: function () {
                return this._onWorkingFileChange.event;
            },
            enumerable: true,
            configurable: true
        });
        WorkingFilesModel.prototype.getEntries = function (excludeOutOfContext) {
            return this.entries;
        };
        WorkingFilesModel.prototype.first = function () {
            if (!this.entries.length) {
                return null;
            }
            return this.entries.slice(0).sort(WorkingFilesModel.compare)[0];
        };
        WorkingFilesModel.prototype.last = function () {
            if (!this.entries.length) {
                return null;
            }
            return this.entries.slice(0).sort(WorkingFilesModel.compare)[this.entries.length - 1];
        };
        WorkingFilesModel.prototype.next = function (start) {
            var entry = start && this.findEntry(start);
            if (entry) {
                var sorted = this.entries.slice(0).sort(WorkingFilesModel.compare);
                var index = sorted.indexOf(entry);
                if (index + 1 < sorted.length) {
                    return sorted[index + 1];
                }
            }
            return this.first();
        };
        WorkingFilesModel.prototype.previous = function (start) {
            var entry = start && this.findEntry(start);
            if (entry) {
                var sorted = this.entries.slice(0).sort(WorkingFilesModel.compare);
                var index = sorted.indexOf(entry);
                if (index > 0) {
                    return sorted[index - 1];
                }
            }
            return this.last();
        };
        WorkingFilesModel.prototype.getOutOfWorkspaceContextEntries = function () {
            var _this = this;
            return this.entries.filter(function (entry) { return _this.isOutOfWorkspace(entry.resource); });
        };
        WorkingFilesModel.prototype.isOutOfWorkspace = function (resource) {
            if (resource.scheme !== 'file') {
                return false;
            }
            var workspace = this.contextService.getWorkspace();
            return !workspace || !paths.isEqualOrParent(resource.fsPath, workspace.resource.fsPath);
        };
        WorkingFilesModel.prototype.count = function () {
            return this.entries.length;
        };
        WorkingFilesModel.prototype.addEntry = function (arg1) {
            var resource;
            if (arg1 instanceof WorkingFileEntry) {
                resource = arg1.resource;
            }
            else if (arg1 instanceof uri_1.default) {
                resource = arg1;
            }
            else {
                resource = arg1.resource;
            }
            var existing = this.findEntry(resource);
            if (existing) {
                return existing;
            }
            var entry;
            if (arg1 instanceof WorkingFileEntry) {
                entry = arg1;
            }
            else {
                entry = this.createEntry(resource);
            }
            this.entries.push(entry);
            this.mapEntryToResource[entry.resource.toString()] = entry;
            this.fireModelChange({ added: [entry] });
            return entry;
        };
        WorkingFilesModel.prototype.createEntry = function (resource, index, dirty) {
            return new WorkingFileEntry(resource, typeof index === 'number' ? index : this.entries.length, dirty);
        };
        WorkingFilesModel.prototype.moveEntry = function (oldResource, newResource) {
            var oldEntry = this.findEntry(oldResource);
            if (oldEntry && !this.findEntry(newResource)) {
                // Add new with properties from old one
                var newEntry = this.createEntry(newResource, oldEntry.index, oldEntry.dirty);
                this.entries.push(newEntry);
                this.mapEntryToResource[newResource.toString()] = newEntry;
                // Remove old
                this.entries.splice(this.entries.indexOf(oldEntry), 1);
                delete this.mapEntryToResource[oldResource.toString()];
                this.fireModelChange({ added: [newEntry], removed: [oldEntry] });
            }
        };
        WorkingFilesModel.prototype.removeEntry = function (arg1) {
            var resource = arg1 instanceof WorkingFileEntry ? arg1.resource : arg1;
            var index = this.indexOf(resource);
            if (index >= 0) {
                // Remove entry
                var removed = this.entries.splice(index, 1)[0];
                delete this.mapEntryToResource[resource.toString()];
                // Rebalance index
                var sortedEntries = this.entries.slice(0).sort(WorkingFilesModel.compare);
                var newTopIndex = 0;
                for (var i = 0; i < sortedEntries.length; i++) {
                    if (sortedEntries[i] === removed) {
                        continue;
                    }
                    sortedEntries[i].setIndex(newTopIndex++);
                }
                this.fireModelChange({ removed: [removed] });
                return removed;
            }
            return null;
        };
        WorkingFilesModel.prototype.reorder = function (source, target) {
            var sortedEntries = this.entries.slice(0).sort(WorkingFilesModel.compare);
            var indexOfSource = sortedEntries.indexOf(source);
            var indexOfTarget = sortedEntries.indexOf(target);
            arrays.move(sortedEntries, indexOfSource, indexOfTarget);
            // Rebalance index
            var newTopIndex = 0;
            for (var i = 0; i < sortedEntries.length; i++) {
                sortedEntries[i].setIndex(newTopIndex++);
            }
            // Fire event
            this.fireModelChange({});
        };
        WorkingFilesModel.prototype.clear = function () {
            var deleted = this.entries;
            this.entries = [];
            this.mapEntryToResource = Object.create(null);
            this.fireModelChange({ removed: deleted });
        };
        WorkingFilesModel.prototype.hasEntry = function (resource) {
            return !!this.findEntry(resource);
        };
        WorkingFilesModel.prototype.findEntry = function (resource) {
            return this.mapEntryToResource[resource.toString()];
        };
        WorkingFilesModel.prototype.indexOf = function (resource) {
            var entry = this.findEntry(resource);
            if (entry) {
                return this.entries.indexOf(entry);
            }
            return -1;
        };
        WorkingFilesModel.prototype.shutdown = function () {
            var sortedEntries = this.entries.slice(0).sort(WorkingFilesModel.compare);
            var index = 0;
            var entries = [];
            sortedEntries.forEach(function (entry) {
                if (!entry.isUntitled) {
                    entries.push({
                        resource: entry.resource.toString(),
                        index: index++
                    });
                }
            });
            this.storageService.store(WorkingFilesModel.STORAGE_KEY, JSON.stringify(entries), storage_1.StorageScope.WORKSPACE);
        };
        WorkingFilesModel.prototype.load = function () {
            var _this = this;
            // Load from settings
            var modelRaw = this.storageService.get(WorkingFilesModel.STORAGE_KEY, storage_1.StorageScope.WORKSPACE);
            if (modelRaw) {
                var model = JSON.parse(modelRaw);
                model.forEach(function (entry) {
                    _this.addEntry(new WorkingFileEntry(uri_1.default.parse(entry.resource), entry.index, false));
                });
            }
            // Add those that are set to open on startup
            var options = this.contextService.getOptions();
            var files = (options && options.filesToOpen) || [];
            if (options && options.filesToDiff) {
                files.push.apply(files, options.filesToDiff);
            }
            arrays
                .distinct(files, function (r) { return r.resource.toString(); }) // no duplicates
                .map(function (f) { return f.resource; }) // just the resource
                .filter(function (r) { return r.scheme === 'untitled' || _this.isOutOfWorkspace(r); }) // untitled or out of workspace
                .forEach(function (r) {
                _this.addEntry(r);
            });
            // Add untitled ones (after joinCreation() to make sure we catch everything)
            this.partService.joinCreation().done(function () {
                if (_this.untitledEditorService) {
                    _this.untitledEditorService.getAll().map(function (u) { return u.getResource(); })
                        .filter(function (r) { return !_this.untitledEditorService.hasAssociatedFilePath(r); }) // only those without association
                        .forEach(function (r) {
                        _this.addEntry(r);
                    });
                }
            }, errors.onUnexpectedError);
        };
        WorkingFilesModel.prototype.dispose = function () {
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
        };
        WorkingFilesModel.prototype.fireModelChange = function (event) {
            this._onModelChange.fire(event);
        };
        WorkingFilesModel.prototype.fireWorkingFileChange = function (file) {
            this._onWorkingFileChange.fire(file);
        };
        WorkingFilesModel.compare = function (element, otherElement) {
            return element.index - otherElement.index;
        };
        WorkingFilesModel.STORAGE_KEY = 'workingFiles.model.entries';
        WorkingFilesModel = __decorate([
            __param(0, storage_1.IStorageService),
            __param(1, event_2.IEventService),
            __param(2, partService_1.IPartService),
            __param(3, contextService_1.IWorkspaceContextService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, untitledEditorService_1.IUntitledEditorService),
            __param(6, files_1.ITextFileService)
        ], WorkingFilesModel);
        return WorkingFilesModel;
    }());
    exports.WorkingFilesModel = WorkingFilesModel;
    var WorkingFileEntry = (function () {
        function WorkingFileEntry(resource, index, dirty) {
            this._resource = resource;
            this._index = index;
            this._dirty = dirty;
        }
        Object.defineProperty(WorkingFileEntry.prototype, "resource", {
            get: function () {
                return this._resource;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WorkingFileEntry.prototype, "index", {
            get: function () {
                return this._index;
            },
            enumerable: true,
            configurable: true
        });
        WorkingFileEntry.prototype.setIndex = function (index) {
            this._index = index;
        };
        Object.defineProperty(WorkingFileEntry.prototype, "dirty", {
            get: function () {
                return this._dirty;
            },
            enumerable: true,
            configurable: true
        });
        WorkingFileEntry.prototype.setDirty = function (dirty) {
            this._dirty = dirty;
        };
        Object.defineProperty(WorkingFileEntry.prototype, "CLASS_ID", {
            get: function () {
                return files_1.WORKING_FILES_MODEL_ENTRY_CLASS_ID;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WorkingFileEntry.prototype, "isFile", {
            get: function () {
                return this._resource.scheme === 'file';
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WorkingFileEntry.prototype, "isUntitled", {
            get: function () {
                return this._resource.scheme === 'untitled';
            },
            enumerable: true,
            configurable: true
        });
        return WorkingFileEntry;
    }());
    exports.WorkingFileEntry = WorkingFileEntry;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/textFileServices", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/errors', 'vs/base/common/event', 'vs/workbench/parts/files/browser/editors/fileEditorInput', 'vs/workbench/parts/files/common/editors/textFileEditorModel', 'vs/workbench/parts/files/common/files', 'vs/workbench/common/events', 'vs/workbench/parts/files/common/workingFilesModel', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/files/common/files', 'vs/platform/instantiation/common/instantiation', 'vs/platform/lifecycle/common/lifecycle', 'vs/platform/event/common/event', 'vs/platform/configuration/common/configuration', 'vs/platform/telemetry/common/telemetry'], function (require, exports, winjs_base_1, errors, event_1, fileEditorInput_1, textFileEditorModel_1, files_1, events_1, workingFilesModel_1, contextService_1, files_2, instantiation_1, lifecycle_1, event_2, configuration_1, telemetry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * The workbench file service implementation implements the raw file service spec and adds additional methods on top.
     *
     * It also adds diagnostics and logging around file system operations.
     */
    var TextFileService = (function () {
        function TextFileService(contextService, instantiationService, configurationService, telemetryService, lifecycleService, eventService) {
            this.contextService = contextService;
            this.instantiationService = instantiationService;
            this.configurationService = configurationService;
            this.telemetryService = telemetryService;
            this.lifecycleService = lifecycleService;
            this.eventService = eventService;
            this.serviceId = files_1.ITextFileService;
            this.listenerToUnbind = [];
            this._onAutoSaveConfigurationChange = new event_1.Emitter();
        }
        TextFileService.prototype.init = function () {
            this.registerListeners();
            this.loadConfiguration();
        };
        Object.defineProperty(TextFileService.prototype, "onAutoSaveConfigurationChange", {
            get: function () {
                return this._onAutoSaveConfigurationChange.event;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TextFileService.prototype, "workingFilesModel", {
            get: function () {
                if (!this._workingFilesModel) {
                    this._workingFilesModel = this.instantiationService.createInstance(workingFilesModel_1.WorkingFilesModel);
                }
                return this._workingFilesModel;
            },
            enumerable: true,
            configurable: true
        });
        TextFileService.prototype.registerListeners = function () {
            var _this = this;
            // Lifecycle
            this.lifecycleService.addBeforeShutdownParticipant(this);
            this.lifecycleService.onShutdown(this.dispose, this);
            // Configuration changes
            this.listenerToUnbind.push(this.configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) { return _this.onConfigurationChange(e.config); }));
            // Editor focus change
            window.addEventListener('blur', function () { return _this.onEditorFocusChange(); }, true);
            this.listenerToUnbind.push(this.eventService.addListener(events_1.EventType.EDITOR_INPUT_CHANGED, function () { return _this.onEditorFocusChange(); }));
        };
        TextFileService.prototype.onEditorFocusChange = function () {
            if (this.configuredAutoSaveOnFocusChange && this.getDirty().length) {
                this.saveAll().done(null, errors.onUnexpectedError); // save dirty files when we change focus in the editor area
            }
        };
        TextFileService.prototype.loadConfiguration = function () {
            var _this = this;
            this.configurationService.loadConfiguration().done(function (configuration) {
                _this.onConfigurationChange(configuration);
                // we want to find out about this setting from telemetry
                _this.telemetryService.publicLog('autoSave', _this.getAutoSaveConfiguration());
            }, errors.onUnexpectedError);
        };
        TextFileService.prototype.onConfigurationChange = function (configuration) {
            var wasAutoSaveEnabled = (this.getAutoSaveMode() !== files_1.AutoSaveMode.OFF);
            var autoSaveMode = (configuration && configuration.files && configuration.files.autoSave) || files_2.AutoSaveConfiguration.OFF;
            switch (autoSaveMode) {
                case files_2.AutoSaveConfiguration.AFTER_DELAY:
                    this.configuredAutoSaveDelay = configuration && configuration.files && configuration.files.autoSaveDelay;
                    this.configuredAutoSaveOnFocusChange = false;
                    break;
                case files_2.AutoSaveConfiguration.ON_FOCUS_CHANGE:
                    this.configuredAutoSaveDelay = void 0;
                    this.configuredAutoSaveOnFocusChange = true;
                    break;
                default:
                    this.configuredAutoSaveDelay = void 0;
                    this.configuredAutoSaveOnFocusChange = false;
                    break;
            }
            // Emit as event
            this._onAutoSaveConfigurationChange.fire(this.getAutoSaveConfiguration());
            // save all dirty when enabling auto save
            if (!wasAutoSaveEnabled && this.getAutoSaveMode() !== files_1.AutoSaveMode.OFF) {
                this.saveAll().done(null, errors.onUnexpectedError);
            }
        };
        TextFileService.prototype.getDirty = function (resources) {
            return this.getDirtyFileModels(resources).map(function (m) { return m.getResource(); });
        };
        TextFileService.prototype.isDirty = function (resource) {
            return textFileEditorModel_1.CACHE
                .getAll(resource)
                .some(function (model) { return model.isDirty(); });
        };
        TextFileService.prototype.save = function (resource) {
            return this.saveAll([resource]).then(function (result) { return result.results.length === 1 && result.results[0].success; });
        };
        TextFileService.prototype.saveAll = function (arg1 /* URI[] */) {
            var dirtyFileModels = this.getDirtyFileModels(Array.isArray(arg1) ? arg1 : void 0 /* Save All */);
            var mapResourceToResult = Object.create(null);
            dirtyFileModels.forEach(function (m) {
                mapResourceToResult[m.getResource().toString()] = {
                    source: m.getResource()
                };
            });
            return winjs_base_1.TPromise.join(dirtyFileModels.map(function (model) {
                return model.save().then(function () {
                    if (!model.isDirty()) {
                        mapResourceToResult[model.getResource().toString()].success = true;
                    }
                });
            })).then(function (r) {
                return {
                    results: Object.keys(mapResourceToResult).map(function (k) { return mapResourceToResult[k]; })
                };
            });
        };
        TextFileService.prototype.getFileModels = function (arg1) {
            var _this = this;
            if (Array.isArray(arg1)) {
                var models_1 = [];
                arg1.forEach(function (resource) {
                    models_1.push.apply(models_1, _this.getFileModels(resource));
                });
                return models_1;
            }
            return textFileEditorModel_1.CACHE.getAll(arg1);
        };
        TextFileService.prototype.getDirtyFileModels = function (arg1) {
            return this.getFileModels(arg1).filter(function (model) { return model.isDirty(); });
        };
        TextFileService.prototype.confirmSave = function (resources) {
            throw new Error('Unsupported');
        };
        TextFileService.prototype.revert = function (resource, force) {
            return this.revertAll([resource], force).then(function (result) { return result.results.length === 1 && result.results[0].success; });
        };
        TextFileService.prototype.revertAll = function (resources, force) {
            var _this = this;
            var fileModels = force ? this.getFileModels(resources) : this.getDirtyFileModels(resources);
            var mapResourceToResult = Object.create(null);
            fileModels.forEach(function (m) {
                mapResourceToResult[m.getResource().toString()] = {
                    source: m.getResource()
                };
            });
            return winjs_base_1.TPromise.join(fileModels.map(function (model) {
                return model.revert().then(function () {
                    if (!model.isDirty()) {
                        mapResourceToResult[model.getResource().toString()].success = true;
                    }
                }, function (error) {
                    // FileNotFound means the file got deleted meanwhile, so dispose this model
                    if (error.fileOperationResult === files_2.FileOperationResult.FILE_NOT_FOUND) {
                        var clients = fileEditorInput_1.FileEditorInput.getAll(model.getResource());
                        clients.forEach(function (input) { return input.dispose(true); });
                        // also make sure to have it removed from any working files
                        _this.workingFilesModel.removeEntry(model.getResource());
                        // store as successful revert
                        mapResourceToResult[model.getResource().toString()].success = true;
                    }
                    else {
                        return winjs_base_1.TPromise.wrapError(error);
                    }
                });
            })).then(function (r) {
                return {
                    results: Object.keys(mapResourceToResult).map(function (k) { return mapResourceToResult[k]; })
                };
            });
        };
        TextFileService.prototype.beforeShutdown = function () {
            // Propagate to working files model
            this.workingFilesModel.shutdown();
            return false; // no veto
        };
        TextFileService.prototype.getWorkingFilesModel = function () {
            return this.workingFilesModel;
        };
        TextFileService.prototype.getAutoSaveMode = function () {
            if (this.configuredAutoSaveOnFocusChange) {
                return files_1.AutoSaveMode.ON_FOCUS_CHANGE;
            }
            if (this.configuredAutoSaveDelay && this.configuredAutoSaveDelay > 0) {
                return this.configuredAutoSaveDelay <= 1000 ? files_1.AutoSaveMode.AFTER_SHORT_DELAY : files_1.AutoSaveMode.AFTER_LONG_DELAY;
            }
            return files_1.AutoSaveMode.OFF;
        };
        TextFileService.prototype.getAutoSaveConfiguration = function () {
            return {
                autoSaveDelay: this.configuredAutoSaveDelay && this.configuredAutoSaveDelay > 0 ? this.configuredAutoSaveDelay : void 0,
                autoSaveFocusChange: this.configuredAutoSaveOnFocusChange
            };
        };
        TextFileService.prototype.dispose = function () {
            while (this.listenerToUnbind.length) {
                this.listenerToUnbind.pop()();
            }
            this.workingFilesModel.dispose();
            // Clear all caches
            textFileEditorModel_1.CACHE.clear();
        };
        TextFileService = __decorate([
            __param(0, contextService_1.IWorkspaceContextService),
            __param(1, instantiation_1.IInstantiationService),
            __param(2, configuration_1.IConfigurationService),
            __param(3, telemetry_1.ITelemetryService),
            __param(4, lifecycle_1.ILifecycleService),
            __param(5, event_2.IEventService)
        ], TextFileService);
        return TextFileService;
    }());
    exports.TextFileService = TextFileService;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/views/workingFilesViewer", ["require", "exports", 'vs/nls!vs/workbench/parts/files/browser/views/workingFilesViewer', 'vs/base/common/winjs.base', 'vs/base/common/platform', 'vs/base/browser/builder', 'vs/base/parts/tree/browser/tree', 'vs/base/browser/ui/filelabel/fileLabel', 'vs/base/parts/tree/browser/treeDnd', 'vs/base/parts/tree/browser/treeDefaults', 'vs/base/common/errors', 'vs/base/common/mime', 'vs/base/common/paths', 'vs/base/browser/ui/actionbar/actionbar', 'vs/base/parts/tree/browser/actionsRenderer', 'vs/workbench/browser/actionBarRegistry', 'vs/workbench/parts/files/browser/fileActions', 'vs/workbench/parts/files/common/files', 'vs/workbench/parts/files/common/workingFilesModel', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/contextview/browser/contextView', 'vs/platform/instantiation/common/instantiation', 'vs/platform/telemetry/common/telemetry', 'vs/base/common/keyCodes'], function (require, exports, nls, winjs_base_1, platform, builder_1, tree_1, fileLabel_1, treeDnd_1, treeDefaults_1, errors, mime, paths, actionbar_1, actionsRenderer_1, actionBarRegistry_1, fileActions_1, files_1, workingFilesModel_1, untitledEditorService_1, editorService_1, contextService_1, contextView_1, instantiation_1, telemetry_1, keyCodes_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ROOT_ID = '__WORKING_FILES_ROOT';
    var WorkingFilesDataSource = (function () {
        function WorkingFilesDataSource() {
        }
        WorkingFilesDataSource.prototype.getId = function (tree, element) {
            if (element instanceof workingFilesModel_1.WorkingFileEntry) {
                return element.resource.toString();
            }
            return ROOT_ID;
        };
        WorkingFilesDataSource.prototype.hasChildren = function (tree, element) {
            if (element instanceof workingFilesModel_1.WorkingFilesModel) {
                return element.count() > 0;
            }
            return false;
        };
        WorkingFilesDataSource.prototype.getChildren = function (tree, element) {
            if (element instanceof workingFilesModel_1.WorkingFilesModel) {
                return winjs_base_1.TPromise.as(element.getEntries());
            }
            return winjs_base_1.TPromise.as([]);
        };
        WorkingFilesDataSource.prototype.getParent = function (tree, element) {
            return winjs_base_1.TPromise.as(null);
        };
        return WorkingFilesDataSource;
    }());
    exports.WorkingFilesDataSource = WorkingFilesDataSource;
    var WorkingFilesSorter = (function () {
        function WorkingFilesSorter() {
        }
        WorkingFilesSorter.prototype.compare = function (tree, element, otherElement) {
            return workingFilesModel_1.WorkingFilesModel.compare(element, otherElement);
        };
        return WorkingFilesSorter;
    }());
    exports.WorkingFilesSorter = WorkingFilesSorter;
    var WorkingFilesRenderer = (function (_super) {
        __extends(WorkingFilesRenderer, _super);
        function WorkingFilesRenderer(model, actionProvider, actionRunner, contextService) {
            _super.call(this, {
                actionProvider: actionProvider,
                actionRunner: actionRunner
            });
            this.contextService = contextService;
        }
        WorkingFilesRenderer.prototype.getHeight = function (tree, element) {
            return WorkingFilesRenderer.FILE_ITEM_HEIGHT;
        };
        WorkingFilesRenderer.prototype.renderContents = function (tree, element, container) {
            var entry = element;
            var $el = builder_1.$(container).clearChildren();
            var item = builder_1.$('.working-files-item').appendTo($el);
            var label = builder_1.$('.working-files-item-label').appendTo(item);
            new fileLabel_1.FileLabel(label.getHTMLElement(), entry.resource, this.contextService);
            if (entry.dirty) {
                builder_1.$(container.parentElement).addClass('working-file-dirty');
            }
            else {
                builder_1.$(container.parentElement).removeClass('working-file-dirty');
            }
            return null;
        };
        WorkingFilesRenderer.FILE_ITEM_HEIGHT = 22;
        WorkingFilesRenderer = __decorate([
            __param(3, contextService_1.IWorkspaceContextService)
        ], WorkingFilesRenderer);
        return WorkingFilesRenderer;
    }(actionsRenderer_1.ActionsRenderer));
    exports.WorkingFilesRenderer = WorkingFilesRenderer;
    var WorkingFilesAccessibilityProvider = (function () {
        function WorkingFilesAccessibilityProvider() {
        }
        WorkingFilesAccessibilityProvider.prototype.getAriaLabel = function (tree, element) {
            var entry = element;
            return nls.localize(0, null, paths.basename(entry.resource.fsPath));
        };
        return WorkingFilesAccessibilityProvider;
    }());
    exports.WorkingFilesAccessibilityProvider = WorkingFilesAccessibilityProvider;
    var WorkingFilesActionProvider = (function (_super) {
        __extends(WorkingFilesActionProvider, _super);
        function WorkingFilesActionProvider(model, instantiationService, untitledEditorService, textFileService) {
            _super.call(this);
            this.instantiationService = instantiationService;
            this.untitledEditorService = untitledEditorService;
            this.textFileService = textFileService;
            this.model = model;
        }
        WorkingFilesActionProvider.prototype.hasActions = function (tree, element) {
            return element instanceof workingFilesModel_1.WorkingFileEntry || _super.prototype.hasActions.call(this, tree, element);
        };
        // we don't call into super here because we put only one primary action to the left (Remove/Dirty Indicator)
        WorkingFilesActionProvider.prototype.getActions = function (tree, element) {
            var actions = [];
            if (element instanceof workingFilesModel_1.WorkingFileEntry) {
                actions.push(this.instantiationService.createInstance(fileActions_1.CloseOneWorkingFileAction, this.model, element));
            }
            return winjs_base_1.TPromise.as(actions);
        };
        WorkingFilesActionProvider.prototype.hasSecondaryActions = function (tree, element) {
            return element instanceof workingFilesModel_1.WorkingFileEntry || _super.prototype.hasActions.call(this, tree, element);
        };
        WorkingFilesActionProvider.prototype.getSecondaryActions = function (tree, element) {
            var _this = this;
            return _super.prototype.getSecondaryActions.call(this, tree, element).then(function (actions) {
                if (element instanceof workingFilesModel_1.WorkingFileEntry) {
                    // Open to side
                    var openToSideAction = _this.instantiationService.createInstance(fileActions_1.OpenToSideAction, tree, element.resource, false);
                    actions.unshift(openToSideAction); // be on top
                    // Files: Save / Revert
                    var autoSaveEnabled = (_this.textFileService.getAutoSaveMode() !== files_1.AutoSaveMode.OFF);
                    if ((!autoSaveEnabled || element.dirty) && element.isFile) {
                        actions.push(new actionbar_1.Separator());
                        var saveAction = _this.instantiationService.createInstance(fileActions_1.SaveFileAction, fileActions_1.SaveFileAction.ID, fileActions_1.SaveFileAction.LABEL);
                        saveAction.setResource(element.resource);
                        saveAction.enabled = element.dirty;
                        actions.push(saveAction);
                        var revertAction = _this.instantiationService.createInstance(fileActions_1.RevertFileAction, fileActions_1.RevertFileAction.ID, fileActions_1.RevertFileAction.LABEL);
                        revertAction.setResource(element.resource);
                        revertAction.enabled = element.dirty;
                        actions.push(revertAction);
                    }
                    // Untitled: Save / Save As
                    if (element.isUntitled) {
                        actions.push(new actionbar_1.Separator());
                        if (_this.untitledEditorService.hasAssociatedFilePath(element.resource)) {
                            var saveUntitledAction = _this.instantiationService.createInstance(fileActions_1.SaveFileAction, fileActions_1.SaveFileAction.ID, fileActions_1.SaveFileAction.LABEL);
                            saveUntitledAction.setResource(element.resource);
                            actions.push(saveUntitledAction);
                        }
                        var saveAsAction = _this.instantiationService.createInstance(fileActions_1.SaveFileAsAction, fileActions_1.SaveFileAsAction.ID, fileActions_1.SaveFileAsAction.LABEL);
                        saveAsAction.setResource(element.resource);
                        actions.push(saveAsAction);
                    }
                    // Compare
                    if (!element.isUntitled) {
                        actions.push(new actionbar_1.Separator());
                        // Run Compare
                        var runCompareAction = _this.instantiationService.createInstance(fileActions_1.CompareResourcesAction, element.resource, tree);
                        if (runCompareAction._isEnabled()) {
                            actions.push(runCompareAction);
                        }
                        // Select for Compare
                        actions.push(_this.instantiationService.createInstance(fileActions_1.SelectResourceForCompareAction, element.resource, tree));
                    }
                    // Close
                    actions.push(new actionbar_1.Separator());
                    actions.push(_this.instantiationService.createInstance(fileActions_1.CloseOneWorkingFileAction, _this.model, element));
                    actions.push(_this.instantiationService.createInstance(fileActions_1.CloseAllWorkingFilesAction, _this.model));
                    if (_this.model.count() > 1) {
                        actions.push(_this.instantiationService.createInstance(fileActions_1.CloseOtherWorkingFilesAction, _this.model, element));
                    }
                }
                return actions;
            });
        };
        WorkingFilesActionProvider = __decorate([
            __param(1, instantiation_1.IInstantiationService),
            __param(2, untitledEditorService_1.IUntitledEditorService),
            __param(3, files_1.ITextFileService)
        ], WorkingFilesActionProvider);
        return WorkingFilesActionProvider;
    }(actionBarRegistry_1.ContributableActionProvider));
    exports.WorkingFilesActionProvider = WorkingFilesActionProvider;
    var WorkingFilesDragAndDrop = (function (_super) {
        __extends(WorkingFilesDragAndDrop, _super);
        function WorkingFilesDragAndDrop(model, contextService) {
            _super.call(this);
            this.contextService = contextService;
            this.model = model;
        }
        WorkingFilesDragAndDrop.prototype.getDragURI = function (tree, element) {
            return element.resource.toString();
        };
        WorkingFilesDragAndDrop.prototype.onDragStart = function (tree, data, originalEvent) {
            var sources = data.getData();
            var source = null;
            if (Array.isArray(sources)) {
                source = sources[0];
            }
            // Native only: when a DownloadURL attribute is defined on the data transfer it is possible to
            // drag a file from the browser to the desktop and have it downloaded there.
            if (!(data instanceof treeDnd_1.DesktopDragAndDropData) && source && source.isFile) {
                var name_1 = paths.basename(source.resource.fsPath);
                originalEvent.dataTransfer.setData('DownloadURL', [mime.MIME_BINARY, name_1, source.resource.toString()].join(':'));
            }
        };
        WorkingFilesDragAndDrop.prototype.onDragOver = function (baum, data, target, originalEvent) {
            if (!(target instanceof workingFilesModel_1.WorkingFileEntry)) {
                return tree_1.DRAG_OVER_REJECT;
            }
            if (data instanceof treeDnd_1.ExternalElementsDragAndDropData) {
                var resource = files_1.asFileResource(data.getData()[0]);
                if (!resource) {
                    return tree_1.DRAG_OVER_REJECT;
                }
                return resource.isDirectory ? tree_1.DRAG_OVER_REJECT : tree_1.DRAG_OVER_ACCEPT;
            }
            if (data instanceof treeDnd_1.DesktopDragAndDropData) {
                return tree_1.DRAG_OVER_REJECT;
            }
            if (!(data instanceof treeDnd_1.ElementsDragAndDropData)) {
                return tree_1.DRAG_OVER_REJECT;
            }
            var sourceResource;
            var targetResource = target.resource;
            var draggedData = data.getData()[0];
            if (draggedData instanceof workingFilesModel_1.WorkingFileEntry) {
                sourceResource = draggedData.resource;
            }
            else {
                var source = files_1.asFileResource(draggedData);
                if (!source) {
                    return tree_1.DRAG_OVER_REJECT;
                }
                sourceResource = source.resource;
            }
            if (!targetResource || !sourceResource) {
                return tree_1.DRAG_OVER_REJECT;
            }
            return targetResource.toString() === sourceResource.toString() ? tree_1.DRAG_OVER_REJECT : tree_1.DRAG_OVER_ACCEPT;
        };
        WorkingFilesDragAndDrop.prototype.drop = function (tree, data, target, originalEvent) {
            var draggedElement;
            // Support drop from explorer viewer
            if (data instanceof treeDnd_1.ExternalElementsDragAndDropData) {
                var resource = files_1.asFileResource(data.getData()[0]);
                draggedElement = this.model.addEntry(resource.resource);
            }
            else {
                var source = data.getData();
                if (Array.isArray(source)) {
                    draggedElement = source[0];
                }
            }
            if (draggedElement) {
                this.model.reorder(draggedElement, target);
            }
        };
        WorkingFilesDragAndDrop = __decorate([
            __param(1, contextService_1.IWorkspaceContextService)
        ], WorkingFilesDragAndDrop);
        return WorkingFilesDragAndDrop;
    }(treeDefaults_1.DefaultDragAndDrop));
    exports.WorkingFilesDragAndDrop = WorkingFilesDragAndDrop;
    var WorkingFilesController = (function (_super) {
        __extends(WorkingFilesController, _super);
        function WorkingFilesController(model, provider, editorService, instantiationService, contextMenuService, telemetryService) {
            _super.call(this, { clickBehavior: treeDefaults_1.ClickBehavior.ON_MOUSE_DOWN });
            this.editorService = editorService;
            this.instantiationService = instantiationService;
            this.contextMenuService = contextMenuService;
            this.telemetryService = telemetryService;
            this.model = model;
            this.actionProvider = provider;
            this.downKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.ENTER, this.onEnterDown.bind(this));
            if (platform.isMacintosh) {
                this.upKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.WINCTRL_ENTER, this.onModifierEnterUp.bind(this)); // Mac: somehow Cmd+Enter does not work
            }
            else {
                this.upKeyBindingDispatcher.set(keyCodes_1.CommonKeybindings.CTRLCMD_ENTER, this.onModifierEnterUp.bind(this)); // Mac: somehow Cmd+Enter does not work
            }
        }
        /* protected */ WorkingFilesController.prototype.onClick = function (tree, element, event) {
            // Close working file on middle mouse click
            if (element instanceof workingFilesModel_1.WorkingFileEntry && event.browserEvent && event.browserEvent.button === 1 /* Middle Button */) {
                var closeAction_1 = this.instantiationService.createInstance(fileActions_1.CloseOneWorkingFileAction, this.model, element);
                closeAction_1.run().done(function () {
                    closeAction_1.dispose();
                }, errors.onUnexpectedError);
                return true;
            }
            return _super.prototype.onClick.call(this, tree, element, event);
        };
        /* protected */ WorkingFilesController.prototype.onLeftClick = function (tree, element, event, origin) {
            if (origin === void 0) { origin = 'mouse'; }
            var payload = { origin: origin };
            var isDoubleClick = (origin === 'mouse' && event.detail === 2);
            // Handle outside element click
            if (element instanceof workingFilesModel_1.WorkingFilesModel) {
                tree.clearFocus(payload);
                tree.clearSelection(payload);
                return false;
            }
            // Cancel Event
            var isMouseDown = event && event.browserEvent && event.browserEvent.type === 'mousedown';
            if (!isMouseDown) {
                event.preventDefault(); // we cannot preventDefault onMouseDown because this would break DND otherwise
            }
            event.stopPropagation();
            // Set DOM focus
            tree.DOMFocus();
            // Allow to unselect
            if (event.shiftKey) {
                var selection = tree.getSelection();
                if (selection && selection.length > 0 && selection[0] === element) {
                    tree.clearSelection(payload);
                }
            }
            else {
                var preserveFocus = !isDoubleClick;
                tree.setFocus(element, payload);
                if (isDoubleClick) {
                    event.preventDefault(); // focus moves to editor, we need to prevent default
                }
                tree.setSelection([element], payload);
                this.openEditor(element, preserveFocus, event && (event.ctrlKey || event.metaKey));
            }
            return true;
        };
        WorkingFilesController.prototype.onEnterDown = function (tree, event) {
            var payload = { origin: 'keyboard' };
            var element = tree.getFocus();
            if (element) {
                tree.setFocus(element, payload);
                this.openEditor(element, false, false);
            }
            return true;
        };
        WorkingFilesController.prototype.onModifierEnterUp = function (tree, event) {
            var element = tree.getFocus();
            if (element) {
                this.openEditor(element, false, true);
            }
            return true;
        };
        WorkingFilesController.prototype.onContextMenu = function (tree, element, event) {
            var _this = this;
            if (event.target && event.target.tagName && event.target.tagName.toLowerCase() === 'input') {
                return false;
            }
            event.preventDefault();
            event.stopPropagation();
            tree.setFocus(element);
            var anchor = { x: event.posx + 1, y: event.posy };
            this.contextMenuService.showContextMenu({
                getAnchor: function () { return anchor; },
                getActions: function () { return _this.actionProvider.getSecondaryActions(tree, element); },
                getKeyBinding: function (a) { return _this.getKeyBinding(a); },
                onHide: function (wasCancelled) {
                    if (wasCancelled) {
                        tree.DOMFocus();
                    }
                }
            });
            return true;
        };
        WorkingFilesController.prototype.getKeyBinding = function (action) {
            return fileActions_1.keybindingForAction(action.id);
        };
        WorkingFilesController.prototype.openEditor = function (element, preserveFocus, sideBySide) {
            if (element) {
                this.telemetryService.publicLog('workbenchActionExecuted', { id: 'workbench.files.openFile', from: 'workingSet' });
                this.editorService.openEditor({
                    resource: element.resource,
                    options: {
                        preserveFocus: preserveFocus
                    }
                }, sideBySide).done(null, errors.onUnexpectedError);
            }
        };
        WorkingFilesController = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, instantiation_1.IInstantiationService),
            __param(4, contextView_1.IContextMenuService),
            __param(5, telemetry_1.ITelemetryService)
        ], WorkingFilesController);
        return WorkingFilesController;
    }(treeDefaults_1.DefaultController));
    exports.WorkingFilesController = WorkingFilesController;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/views/workingFilesView", ["require", "exports", 'vs/nls!vs/workbench/parts/files/browser/views/workingFilesView', 'vs/base/browser/builder', 'vs/base/parts/tree/browser/treeImpl', 'vs/workbench/common/editor', 'vs/base/browser/ui/splitview/splitview', 'vs/workbench/parts/files/common/files', 'vs/base/browser/dom', 'vs/base/common/errors', 'vs/workbench/common/events', 'vs/workbench/browser/viewlet', 'vs/workbench/parts/files/browser/fileActions', 'vs/workbench/parts/files/browser/views/workingFilesViewer', 'vs/workbench/services/editor/common/editorService', 'vs/platform/configuration/common/configuration', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/contextview/browser/contextView', 'vs/platform/message/common/message'], function (require, exports, nls, builder_1, treeImpl_1, workbenchEditorCommon, splitview_1, files_1, dom, errors, events_1, viewlet_1, fileActions_1, workingFilesViewer_1, editorService_1, configuration_1, event_1, instantiation_1, contextView_1, message_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var WorkingFilesView = (function (_super) {
        __extends(WorkingFilesView, _super);
        function WorkingFilesView(actionRunner, settings, eventService, instantiationService, messageService, contextMenuService, textFileService, editorService, configurationService) {
            _super.call(this, actionRunner, WorkingFilesView.computeExpandedBodySize(textFileService.getWorkingFilesModel()), !!settings[WorkingFilesView.MEMENTO_COLLAPSED], nls.localize(0, null), messageService, contextMenuService);
            this.eventService = eventService;
            this.instantiationService = instantiationService;
            this.textFileService = textFileService;
            this.editorService = editorService;
            this.configurationService = configurationService;
            this.settings = settings;
            this.model = this.textFileService.getWorkingFilesModel();
            this.lastDirtyCount = 0;
            this.disposeables = [];
        }
        WorkingFilesView.prototype.renderHeader = function (container) {
            _super.prototype.renderHeader.call(this, container);
            var titleDiv = builder_1.$('div.title').appendTo(container);
            builder_1.$('span').text(nls.localize(1, null)).appendTo(titleDiv);
            this.dirtyCountElement = builder_1.$('div.monaco-count-badge').appendTo(titleDiv).hide().getHTMLElement();
            this.updateDirtyIndicator();
        };
        WorkingFilesView.prototype.getActions = function () {
            return [
                this.instantiationService.createInstance(fileActions_1.SaveAllAction, fileActions_1.SaveAllAction.ID, fileActions_1.SaveAllAction.LABEL),
                this.instantiationService.createInstance(fileActions_1.CloseAllWorkingFilesAction, this.model)
            ];
        };
        WorkingFilesView.prototype.renderBody = function (container) {
            this.treeContainer = _super.prototype.renderViewTree.call(this, container);
            dom.addClass(this.treeContainer, 'explorer-working-files');
            this.createViewer(builder_1.$(this.treeContainer));
        };
        WorkingFilesView.prototype.create = function () {
            var _this = this;
            // Load Config
            return this.configurationService.loadConfiguration().then(function (configuration) {
                // Update configuration
                _this.onConfigurationUpdated(configuration);
                // listeners
                _this.registerListeners();
                // highlight active input
                _this.highlightInput(_this.editorService.getActiveEditorInput());
                return _super.prototype.create.call(_this);
            });
        };
        WorkingFilesView.prototype.onConfigurationUpdated = function (configuration) {
            var visibleWorkingFiles = configuration && configuration.explorer && configuration.explorer.workingFiles && configuration.explorer.workingFiles.maxVisible;
            if (typeof visibleWorkingFiles === 'number') {
                this.maxVisibleWorkingFiles = visibleWorkingFiles;
            }
            else {
                this.maxVisibleWorkingFiles = WorkingFilesView.DEFAULT_MAX_VISIBLE_FILES;
            }
            var dynamicHeight = configuration && configuration.explorer && configuration.explorer.workingFiles && configuration.explorer.workingFiles.dynamicHeight;
            if (typeof dynamicHeight === 'boolean') {
                this.dynamicHeight = dynamicHeight;
            }
            else {
                this.dynamicHeight = WorkingFilesView.DEFAULT_DYNAMIC_HEIGHT;
            }
            // Adjust expanded body size
            this.expandedBodySize = this.getExpandedBodySize(this.model);
        };
        WorkingFilesView.prototype.registerListeners = function () {
            var _this = this;
            // update on model changes
            this.disposeables.push(this.model.onModelChange(this.onWorkingFilesModelChange, this));
            this.disposeables.push(this.model.onWorkingFileChange(this.onWorkingFileChange, this));
            // listen to untitled
            this.toDispose.push(this.eventService.addListener2(events_1.EventType.UNTITLED_FILE_DIRTY, function (e) { return _this.onUntitledFileDirty(); }));
            this.toDispose.push(this.eventService.addListener2(events_1.EventType.UNTITLED_FILE_DELETED, function (e) { return _this.onUntitledFileDeleted(); }));
            // listen to files being changed locally
            this.toDispose.push(this.eventService.addListener2(files_1.EventType.FILE_DIRTY, function (e) { return _this.onTextFileDirty(e); }));
            this.toDispose.push(this.eventService.addListener2(files_1.EventType.FILE_SAVED, function (e) { return _this.onTextFileSaved(e); }));
            this.toDispose.push(this.eventService.addListener2(files_1.EventType.FILE_SAVE_ERROR, function (e) { return _this.onTextFileSaveError(e); }));
            this.toDispose.push(this.eventService.addListener2(files_1.EventType.FILE_REVERTED, function (e) { return _this.onTextFileReverted(e); }));
            // listen to files being opened
            this.toDispose.push(this.eventService.addListener2(events_1.EventType.EDITOR_INPUT_CHANGED, function (e) { return _this.onEditorInputChanged(e); }));
            // Also handle configuration updates
            this.toDispose.push(this.configurationService.addListener2(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) { return _this.onConfigurationUpdated(e.config); }));
        };
        WorkingFilesView.prototype.onTextFileDirty = function (e) {
            if (this.textFileService.getAutoSaveMode() !== files_1.AutoSaveMode.AFTER_SHORT_DELAY) {
                this.updateDirtyIndicator(); // no indication needed when auto save is enabled for short delay
            }
        };
        WorkingFilesView.prototype.onTextFileSaved = function (e) {
            if (this.lastDirtyCount > 0) {
                this.updateDirtyIndicator();
            }
        };
        WorkingFilesView.prototype.onTextFileSaveError = function (e) {
            this.updateDirtyIndicator();
        };
        WorkingFilesView.prototype.onTextFileReverted = function (e) {
            if (this.lastDirtyCount > 0) {
                this.updateDirtyIndicator();
            }
        };
        WorkingFilesView.prototype.onUntitledFileDirty = function () {
            this.updateDirtyIndicator();
        };
        WorkingFilesView.prototype.onUntitledFileDeleted = function () {
            if (this.lastDirtyCount > 0) {
                this.updateDirtyIndicator();
            }
        };
        WorkingFilesView.prototype.updateDirtyIndicator = function () {
            var dirty = this.textFileService.getDirty().length;
            this.lastDirtyCount = dirty;
            if (dirty === 0) {
                builder_1.$(this.dirtyCountElement).hide();
            }
            else {
                var label = nls.localize(2, null, dirty);
                builder_1.$(this.dirtyCountElement).show().text(label).title(label);
            }
        };
        WorkingFilesView.prototype.onWorkingFilesModelChange = function (event) {
            if (this.isDisposed) {
                return;
            }
            // View size
            this.expandedBodySize = this.getExpandedBodySize(this.model);
            if (this.tree) {
                // Show in tree
                this.tree.refresh();
                // Make sure to keep active editor input highlighted
                var activeInput = this.editorService.getActiveEditorInput();
                this.highlightInput(activeInput);
            }
        };
        WorkingFilesView.prototype.onWorkingFileChange = function (file) {
            if (this.isDisposed) {
                return;
            }
            if (this.tree) {
                this.tree.refresh(file);
            }
        };
        WorkingFilesView.prototype.getExpandedBodySize = function (model) {
            return WorkingFilesView.computeExpandedBodySize(model, this.maxVisibleWorkingFiles, this.dynamicHeight);
        };
        WorkingFilesView.computeExpandedBodySize = function (model, maxVisibleWorkingFiles, hasDynamicHeight) {
            var entryCount = model.count();
            var visibleWorkingFiles = maxVisibleWorkingFiles;
            if (typeof visibleWorkingFiles !== 'number') {
                visibleWorkingFiles = WorkingFilesView.DEFAULT_MAX_VISIBLE_FILES;
            }
            var dynamicHeight = hasDynamicHeight;
            if (typeof dynamicHeight !== 'boolean') {
                dynamicHeight = WorkingFilesView.DEFAULT_DYNAMIC_HEIGHT;
            }
            var itemsToShow;
            if (dynamicHeight) {
                itemsToShow = Math.min(Math.max(visibleWorkingFiles, 1), entryCount);
            }
            else {
                itemsToShow = Math.max(visibleWorkingFiles, 1);
            }
            return itemsToShow * workingFilesViewer_1.WorkingFilesRenderer.FILE_ITEM_HEIGHT;
        };
        WorkingFilesView.prototype.onEditorInputChanged = function (e) {
            var activeInput = this.editorService.getActiveEditorInput();
            if (activeInput === e.editorInput) {
                this.highlightInput(e.editorInput);
            }
        };
        WorkingFilesView.prototype.highlightInput = function (input) {
            var entry;
            var resource = workbenchEditorCommon.getUntitledOrFileResource(input);
            if (resource) {
                entry = this.model.findEntry(resource);
            }
            if (entry) {
                this.highlightEntry(entry);
            }
            else {
                this.highlightEntry(null);
            }
        };
        WorkingFilesView.prototype.highlightEntry = function (entry) {
            this.tree.clearFocus();
            this.tree.clearSelection();
            if (entry) {
                this.tree.setFocus(entry);
                this.tree.setSelection([entry]);
                this.tree.reveal(entry).done(null, errors.onUnexpectedError);
            }
        };
        WorkingFilesView.prototype.createViewer = function (container) {
            var actionProvider = this.instantiationService.createInstance(workingFilesViewer_1.WorkingFilesActionProvider, this.model);
            var renderer = this.instantiationService.createInstance(workingFilesViewer_1.WorkingFilesRenderer, this.model, actionProvider, this.actionRunner);
            var dataSource = this.instantiationService.createInstance(workingFilesViewer_1.WorkingFilesDataSource);
            var controller = this.instantiationService.createInstance(workingFilesViewer_1.WorkingFilesController, this.model, actionProvider);
            var sorter = this.instantiationService.createInstance(workingFilesViewer_1.WorkingFilesSorter);
            var dnd = this.instantiationService.createInstance(workingFilesViewer_1.WorkingFilesDragAndDrop, this.model);
            var accessibility = this.instantiationService.createInstance(workingFilesViewer_1.WorkingFilesAccessibilityProvider);
            this.tree = new treeImpl_1.Tree(container.getHTMLElement(), {
                dataSource: dataSource,
                renderer: renderer,
                sorter: sorter,
                controller: controller,
                dnd: dnd,
                accessibilityProvider: accessibility
            }, {
                indentPixels: 0,
                twistiePixels: 8,
                ariaLabel: nls.localize(3, null)
            });
            this.tree.setInput(this.model);
            return this.tree;
        };
        WorkingFilesView.prototype.shutdown = function () {
            this.settings[WorkingFilesView.MEMENTO_COLLAPSED] = (this.state === splitview_1.CollapsibleState.COLLAPSED);
            _super.prototype.shutdown.call(this);
        };
        WorkingFilesView.prototype.dispose = function () {
            _super.prototype.dispose.call(this);
            while (this.disposeables.length) {
                this.disposeables.pop().dispose();
            }
        };
        WorkingFilesView.MEMENTO_COLLAPSED = 'workingFiles.memento.collapsed';
        WorkingFilesView.DEFAULT_MAX_VISIBLE_FILES = 9;
        WorkingFilesView.DEFAULT_DYNAMIC_HEIGHT = true;
        WorkingFilesView = __decorate([
            __param(2, event_1.IEventService),
            __param(3, instantiation_1.IInstantiationService),
            __param(4, message_1.IMessageService),
            __param(5, contextView_1.IContextMenuService),
            __param(6, files_1.ITextFileService),
            __param(7, editorService_1.IWorkbenchEditorService),
            __param(8, configuration_1.IConfigurationService)
        ], WorkingFilesView);
        return WorkingFilesView;
    }(viewlet_1.AdaptiveCollapsibleViewletView));
    exports.WorkingFilesView = WorkingFilesView;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/explorerViewlet", ["require", "exports", 'vs/base/common/winjs.base', 'vs/workbench/common/memento', 'vs/workbench/parts/files/common/files', 'vs/workbench/browser/viewlet', 'vs/base/browser/ui/splitview/splitview', 'vs/workbench/parts/files/browser/views/explorerViewer', 'vs/workbench/parts/files/browser/views/explorerView', 'vs/workbench/parts/files/browser/views/emptyView', 'vs/workbench/parts/files/browser/views/workingFilesView', 'vs/platform/storage/common/storage', 'vs/platform/instantiation/common/instantiation', 'vs/platform/workspace/common/workspace', 'vs/platform/telemetry/common/telemetry', 'vs/css!./media/explorerviewlet'], function (require, exports, winjs_base_1, memento_1, files_1, viewlet_1, splitview_1, explorerViewer_1, explorerView_1, emptyView_1, workingFilesView_1, storage_1, instantiation_1, workspace_1, telemetry_1) {
    'use strict';
    var ExplorerViewlet = (function (_super) {
        __extends(ExplorerViewlet, _super);
        function ExplorerViewlet(telemetryService, contextService, storageService, instantiationService) {
            _super.call(this, files_1.VIEWLET_ID, telemetryService);
            this.contextService = contextService;
            this.storageService = storageService;
            this.instantiationService = instantiationService;
            this.views = [];
            this.viewletState = new explorerViewer_1.FileViewletState();
            this.viewletSettings = this.getMemento(storageService, memento_1.Scope.WORKSPACE);
        }
        ExplorerViewlet.prototype.create = function (parent) {
            var _this = this;
            _super.prototype.create.call(this, parent);
            this.viewletContainer = parent.div().addClass('explorer-viewlet');
            this.splitView = new splitview_1.SplitView(this.viewletContainer.getHTMLElement());
            // Working files view
            this.addWorkingFilesView();
            // Explorer view
            this.addExplorerView();
            // Track focus
            this.focusListener = this.splitView.onFocus(function (view) {
                _this.lastFocusedView = view;
            });
            return winjs_base_1.TPromise.join(this.views.map(function (view) { return view.create(); })).then(function () { return void 0; });
        };
        ExplorerViewlet.prototype.addWorkingFilesView = function () {
            this.workingFilesView = this.instantiationService.createInstance(workingFilesView_1.WorkingFilesView, this.getActionRunner(), this.viewletSettings);
            this.splitView.addView(this.workingFilesView);
            this.views.push(this.workingFilesView);
        };
        ExplorerViewlet.prototype.addExplorerView = function () {
            var explorerView;
            // With a Workspace
            if (this.contextService.getWorkspace()) {
                this.explorerView = explorerView = this.instantiationService.createInstance(explorerView_1.ExplorerView, this.viewletState, this.getActionRunner(), this.viewletSettings);
            }
            else {
                explorerView = this.instantiationService.createInstance(emptyView_1.EmptyView);
            }
            this.splitView.addView(explorerView);
            this.views.push(explorerView);
        };
        /**
         * Refresh the contents of the explorer to get up to date data from the disk about the file structure.
         *
         * @param focus if set to true, the explorer viewer will receive keyboard focus
         * @param reveal if set to true, the current active input will be revealed in the explorer
         */
        ExplorerViewlet.prototype.refresh = function (focus, reveal, instantProgress) {
            return winjs_base_1.TPromise.join(this.views.map(function (view) { return view.refresh(focus, reveal, instantProgress); })).then(function () { return void 0; });
        };
        ExplorerViewlet.prototype.getExplorerView = function () {
            return this.explorerView;
        };
        ExplorerViewlet.prototype.getWorkingFilesView = function () {
            return this.workingFilesView;
        };
        ExplorerViewlet.prototype.setVisible = function (visible) {
            var _this = this;
            return _super.prototype.setVisible.call(this, visible).then(function () {
                return winjs_base_1.TPromise.join(_this.views.map(function (view) { return view.setVisible(visible); })).then(function () { return void 0; });
            });
        };
        ExplorerViewlet.prototype.focus = function () {
            _super.prototype.focus.call(this);
            if (this.lastFocusedView && this.lastFocusedView.isExpanded() && this.hasSelectionOrFocus(this.lastFocusedView)) {
                this.lastFocusedView.focusBody();
                return;
            }
            if (this.hasSelectionOrFocus(this.workingFilesView)) {
                return this.workingFilesView.focusBody();
            }
            if (this.hasSelectionOrFocus(this.explorerView)) {
                return this.explorerView.focusBody();
            }
            if (this.workingFilesView && this.workingFilesView.isExpanded()) {
                return this.workingFilesView.focusBody();
            }
            if (this.explorerView && this.explorerView.isExpanded()) {
                return this.explorerView.focusBody();
            }
            return this.workingFilesView.focus();
        };
        ExplorerViewlet.prototype.hasSelectionOrFocus = function (view) {
            if (!view) {
                return false;
            }
            if (!view.isExpanded()) {
                return false;
            }
            var viewer = view.getViewer();
            if (!viewer) {
                return false;
            }
            return !!viewer.getFocus() || (viewer.getSelection() && viewer.getSelection().length > 0);
        };
        ExplorerViewlet.prototype.layout = function (dimension) {
            this.splitView.layout(dimension.height);
        };
        ExplorerViewlet.prototype.getSelection = function () {
            return this.explorerView ? this.explorerView.getSelection() : this.workingFilesView.getSelection();
        };
        ExplorerViewlet.prototype.getActionRunner = function () {
            if (!this.actionRunner) {
                this.actionRunner = new explorerViewer_1.ActionRunner(this.viewletState);
            }
            return this.actionRunner;
        };
        ExplorerViewlet.prototype.shutdown = function () {
            this.views.forEach(function (view) { return view.shutdown(); });
            _super.prototype.shutdown.call(this);
        };
        ExplorerViewlet.prototype.dispose = function () {
            if (this.splitView) {
                this.splitView.dispose();
                this.splitView = null;
            }
            if (this.focusListener) {
                this.focusListener.dispose();
                this.focusListener = null;
            }
        };
        ExplorerViewlet = __decorate([
            __param(0, telemetry_1.ITelemetryService),
            __param(1, workspace_1.IWorkspaceContextService),
            __param(2, storage_1.IStorageService),
            __param(3, instantiation_1.IInstantiationService)
        ], ExplorerViewlet);
        return ExplorerViewlet;
    }(viewlet_1.Viewlet));
    exports.ExplorerViewlet = ExplorerViewlet;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/browser/workingFilesPicker", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/files/browser/workingFilesPicker', 'vs/base/common/paths', 'vs/base/common/labels', 'vs/base/common/errors', 'vs/base/common/strings', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/workbench/parts/files/common/workingFilesModel', 'vs/base/common/scorer', 'vs/workbench/browser/quickopen', 'vs/workbench/parts/files/common/files', 'vs/workbench/services/editor/common/editorService', 'vs/platform/instantiation/common/instantiation', 'vs/platform/workspace/common/workspace'], function (require, exports, winjs_base_1, nls, paths, labels, errors, strings, quickOpen_1, quickOpenModel_1, workingFilesModel_1, scorer, quickopen_1, files_1, editorService_1, instantiation_1, workspace_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var WorkingFilePickerEntry = (function (_super) {
        __extends(WorkingFilePickerEntry, _super);
        function WorkingFilePickerEntry(name, description, entry, editorService) {
            _super.call(this);
            this.editorService = editorService;
            this.workingFilesEntry = entry;
            this.name = name;
            this.description = description;
        }
        WorkingFilePickerEntry.prototype.getPrefix = function () {
            if (this.workingFilesEntry.dirty) {
                return '\u25cf '; // dirty decoration
            }
            return void 0;
        };
        WorkingFilePickerEntry.prototype.getLabel = function () {
            return this.name;
        };
        WorkingFilePickerEntry.prototype.getAriaLabel = function () {
            return nls.localize(0, null, this.getLabel());
        };
        WorkingFilePickerEntry.prototype.getDescription = function () {
            return this.description;
        };
        WorkingFilePickerEntry.prototype.getResource = function () {
            return this.workingFilesEntry.resource;
        };
        WorkingFilePickerEntry.prototype.getWorkingFilesEntry = function () {
            return this.workingFilesEntry;
        };
        WorkingFilePickerEntry.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.OPEN) {
                return this.runOpen(context);
            }
            return _super.prototype.run.call(this, mode, context);
        };
        WorkingFilePickerEntry.prototype.runOpen = function (context) {
            var event = context.event;
            var sideBySide = (event && (event.ctrlKey || event.metaKey || (event.payload && event.payload.originalEvent && (event.payload.originalEvent.ctrlKey || event.payload.originalEvent.metaKey))));
            this.editorService.openEditor({ resource: this.workingFilesEntry.resource }, sideBySide).done(null, errors.onUnexpectedError);
            return true;
        };
        WorkingFilePickerEntry = __decorate([
            __param(3, editorService_1.IWorkbenchEditorService)
        ], WorkingFilePickerEntry);
        return WorkingFilePickerEntry;
    }(quickOpenModel_1.QuickOpenEntryGroup));
    exports.WorkingFilePickerEntry = WorkingFilePickerEntry;
    var WorkingFilesPicker = (function (_super) {
        __extends(WorkingFilesPicker, _super);
        function WorkingFilesPicker(instantiationService, contextService, textFileService) {
            _super.call(this);
            this.instantiationService = instantiationService;
            this.contextService = contextService;
            this.textFileService = textFileService;
            this.scorerCache = Object.create(null);
        }
        WorkingFilesPicker.prototype.getResults = function (searchValue) {
            var _this = this;
            searchValue = searchValue.trim();
            var normalizedSearchValueLowercase = strings.stripWildcards(searchValue).toLowerCase();
            return winjs_base_1.TPromise.as(new quickOpenModel_1.QuickOpenModel(this.textFileService.getWorkingFilesModel().getEntries()
                .map(function (e) {
                var label = paths.basename(e.resource.fsPath);
                var description = labels.getPathLabel(paths.dirname(e.resource.fsPath), _this.contextService);
                if (description === '.') {
                    description = null; // for untitled files
                }
                return _this.instantiationService.createInstance(WorkingFilePickerEntry, label, description, e);
            })
                .filter(function (e) {
                if (!searchValue) {
                    return true;
                }
                var targetToMatch = labels.getPathLabel(e.getResource(), _this.contextService);
                if (!scorer.matches(targetToMatch, normalizedSearchValueLowercase)) {
                    return false;
                }
                var _a = quickOpenModel_1.QuickOpenEntry.highlight(e, searchValue, true /* fuzzy highlight */), labelHighlights = _a.labelHighlights, descriptionHighlights = _a.descriptionHighlights;
                e.setHighlights(labelHighlights, descriptionHighlights);
                return true;
            }).
                // Sort by search value score or natural order if not searching
                sort(function (e1, e2) {
                if (!searchValue) {
                    return workingFilesModel_1.WorkingFilesModel.compare(e1.getWorkingFilesEntry(), e2.getWorkingFilesEntry());
                }
                return quickOpenModel_1.QuickOpenEntry.compareByScore(e1, e2, searchValue, normalizedSearchValueLowercase, _this.scorerCache);
            }).
                // Apply group label
                map(function (e, index) {
                if (index === 0) {
                    e.setGroupLabel(nls.localize(1, null));
                }
                return e;
            })));
        };
        WorkingFilesPicker.prototype.getEmptyLabel = function (searchString) {
            if (searchString) {
                return nls.localize(2, null);
            }
            return nls.localize(3, null);
        };
        WorkingFilesPicker.prototype.getAutoFocus = function (searchValue) {
            if (searchValue) {
                return {
                    autoFocusFirstEntry: true
                };
            }
            return _super.prototype.getAutoFocus.call(this, searchValue);
        };
        WorkingFilesPicker.prototype.onClose = function (canceled) {
            this.scorerCache = Object.create(null);
        };
        WorkingFilesPicker = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, workspace_1.IWorkspaceContextService),
            __param(2, files_1.ITextFileService)
        ], WorkingFilesPicker);
        return WorkingFilesPicker;
    }(quickopen_1.QuickOpenHandler));
    exports.WorkingFilesPicker = WorkingFilesPicker;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/electron-browser/electronFileTracker", ["require", "exports", 'vs/workbench/parts/files/common/files', 'vs/platform/files/common/files', 'vs/workbench/parts/files/browser/fileActions', 'vs/base/common/platform', 'vs/workbench/common/editor', 'vs/base/common/errors', 'vs/workbench/services/editor/common/editorService', 'vs/base/common/uri', 'vs/workbench/services/window/electron-browser/windowService', 'vs/workbench/common/events', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/workbench/services/part/common/partService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/lifecycle/common/lifecycle', 'electron'], function (require, exports, files_1, files_2, fileActions_1, plat, editor_1, errors, editorService_1, uri_1, windowService_1, events_1, untitledEditorService_1, partService_1, contextService_1, event_1, instantiation_1, lifecycle_1, electron_1) {
    'use strict';
    // This extension decorates the window as dirty when auto save is disabled and a file is dirty (mac only) and handles opening of files in the instance.
    var FileTracker = (function () {
        function FileTracker(contextService, eventService, partService, fileService, textFileService, editorService, instantiationService, untitledEditorService, lifecycleService, windowService) {
            this.contextService = contextService;
            this.eventService = eventService;
            this.partService = partService;
            this.fileService = fileService;
            this.textFileService = textFileService;
            this.editorService = editorService;
            this.instantiationService = instantiationService;
            this.untitledEditorService = untitledEditorService;
            this.lifecycleService = lifecycleService;
            this.windowService = windowService;
            this.toUnbind = [];
            this.isDocumentedEdited = false;
            this.activeOutOfWorkspaceWatchers = Object.create(null);
            // Make sure to reset any previous state
            if (plat.platform === plat.Platform.Mac) {
                electron_1.ipcRenderer.send('vscode:setDocumentEdited', this.windowService.getWindowId(), false); // handled from browser process
            }
            this.registerListeners();
        }
        FileTracker.prototype.registerListeners = function () {
            var _this = this;
            // Local text file changes
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.UNTITLED_FILE_DELETED, function () { return _this.onUntitledDeletedEvent(); }));
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.UNTITLED_FILE_DIRTY, function () { return _this.onUntitledDirtyEvent(); }));
            this.toUnbind.push(this.eventService.addListener(files_1.EventType.FILE_DIRTY, function (e) { return _this.onTextFileDirty(e); }));
            this.toUnbind.push(this.eventService.addListener(files_1.EventType.FILE_SAVED, function (e) { return _this.onTextFileSaved(e); }));
            this.toUnbind.push(this.eventService.addListener(files_1.EventType.FILE_SAVE_ERROR, function (e) { return _this.onTextFileSaveError(e); }));
            this.toUnbind.push(this.eventService.addListener(files_1.EventType.FILE_REVERTED, function (e) { return _this.onTextFileReverted(e); }));
            // Support openFiles event for existing and new files
            electron_1.ipcRenderer.on('vscode:openFiles', function (event, request) {
                var inputs = [];
                var diffMode = (request.filesToDiff.length === 2);
                if (!diffMode && request.filesToOpen) {
                    inputs.push.apply(inputs, _this.toInputs(request.filesToOpen, false));
                }
                if (!diffMode && request.filesToCreate) {
                    inputs.push.apply(inputs, _this.toInputs(request.filesToCreate, true));
                }
                if (diffMode) {
                    inputs.push.apply(inputs, _this.toInputs(request.filesToDiff, false));
                }
                if (inputs.length) {
                    var action = _this.instantiationService.createInstance(fileActions_1.OpenResourcesAction, inputs, diffMode);
                    action.run().done(null, errors.onUnexpectedError);
                    action.dispose();
                }
            });
            // Editor input changes
            this.toUnbind.push(this.eventService.addListener(events_1.EventType.EDITOR_INPUT_CHANGED, function () { return _this.onEditorInputChanged(); }));
            // Lifecycle
            this.lifecycleService.onShutdown(this.dispose, this);
        };
        FileTracker.prototype.toInputs = function (paths, isNew) {
            var _this = this;
            return paths.map(function (p) {
                var input = {
                    resource: isNew ? _this.untitledEditorService.createOrGet(uri_1.default.file(p.filePath)).getResource() : uri_1.default.file(p.filePath)
                };
                if (!isNew && p.lineNumber) {
                    input.options = {
                        selection: {
                            startLineNumber: p.lineNumber,
                            startColumn: p.columnNumber
                        }
                    };
                }
                return input;
            });
        };
        FileTracker.prototype.onEditorInputChanged = function () {
            var _this = this;
            var visibleOutOfWorkspaceResources = this.editorService.getVisibleEditors().map(function (editor) {
                return editor_1.asFileEditorInput(editor.input, true);
            }).filter(function (input) {
                return !!input && !_this.contextService.isInsideWorkspace(input.getResource());
            }).map(function (input) {
                return input.getResource().toString();
            });
            // Handle no longer visible out of workspace resources
            Object.keys(this.activeOutOfWorkspaceWatchers).forEach(function (watchedResource) {
                if (visibleOutOfWorkspaceResources.indexOf(watchedResource) < 0) {
                    _this.fileService.unwatchFileChanges(watchedResource);
                    delete _this.activeOutOfWorkspaceWatchers[watchedResource];
                }
            });
            // Handle newly visible out of workspace resources
            visibleOutOfWorkspaceResources.forEach(function (resourceToWatch) {
                if (!_this.activeOutOfWorkspaceWatchers[resourceToWatch]) {
                    _this.fileService.watchFileChanges(uri_1.default.parse(resourceToWatch));
                    _this.activeOutOfWorkspaceWatchers[resourceToWatch] = true;
                }
            });
        };
        FileTracker.prototype.onUntitledDirtyEvent = function () {
            if (!this.isDocumentedEdited) {
                this.updateDocumentEdited();
            }
        };
        FileTracker.prototype.onUntitledDeletedEvent = function () {
            if (this.isDocumentedEdited) {
                this.updateDocumentEdited();
            }
        };
        FileTracker.prototype.onTextFileDirty = function (e) {
            if ((this.textFileService.getAutoSaveMode() !== files_1.AutoSaveMode.AFTER_SHORT_DELAY) && !this.isDocumentedEdited) {
                this.updateDocumentEdited(); // no indication needed when auto save is enabled for short delay
            }
        };
        FileTracker.prototype.onTextFileSaved = function (e) {
            if (this.isDocumentedEdited) {
                this.updateDocumentEdited();
            }
        };
        FileTracker.prototype.onTextFileSaveError = function (e) {
            if (!this.isDocumentedEdited) {
                this.updateDocumentEdited();
            }
        };
        FileTracker.prototype.onTextFileReverted = function (e) {
            if (this.isDocumentedEdited) {
                this.updateDocumentEdited();
            }
        };
        FileTracker.prototype.updateDocumentEdited = function () {
            if (plat.platform === plat.Platform.Mac) {
                var hasDirtyFiles = this.textFileService.isDirty();
                this.isDocumentedEdited = hasDirtyFiles;
                electron_1.ipcRenderer.send('vscode:setDocumentEdited', this.windowService.getWindowId(), hasDirtyFiles); // handled from browser process
            }
        };
        FileTracker.prototype.getId = function () {
            return 'vs.files.electronFileTracker';
        };
        FileTracker.prototype.dispose = function () {
            while (this.toUnbind.length) {
                this.toUnbind.pop()();
            }
            // Dispose watchers if any
            for (var key in this.activeOutOfWorkspaceWatchers) {
                this.fileService.unwatchFileChanges(key);
            }
            this.activeOutOfWorkspaceWatchers = Object.create(null);
        };
        FileTracker = __decorate([
            __param(0, contextService_1.IWorkspaceContextService),
            __param(1, event_1.IEventService),
            __param(2, partService_1.IPartService),
            __param(3, files_2.IFileService),
            __param(4, files_1.ITextFileService),
            __param(5, editorService_1.IWorkbenchEditorService),
            __param(6, instantiation_1.IInstantiationService),
            __param(7, untitledEditorService_1.IUntitledEditorService),
            __param(8, lifecycle_1.ILifecycleService),
            __param(9, windowService_1.IWindowService)
        ], FileTracker);
        return FileTracker;
    }());
    exports.FileTracker = FileTracker;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/electron-browser/textFileServices", ["require", "exports", 'vs/nls!vs/workbench/parts/files/electron-browser/textFileServices', 'vs/base/common/winjs.base', 'vs/base/common/paths', 'vs/base/common/strings', 'vs/base/common/platform', 'vs/base/common/uri', 'vs/platform/event/common/event', 'vs/workbench/parts/files/browser/textFileServices', 'vs/workbench/parts/files/common/editors/textFileEditorModel', 'vs/workbench/parts/files/common/files', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/platform/files/common/files', 'vs/platform/instantiation/common/instantiation', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/lifecycle/common/lifecycle', 'vs/platform/telemetry/common/telemetry', 'vs/platform/configuration/common/configuration', 'vs/editor/common/services/modeService', 'vs/workbench/services/window/electron-browser/windowService'], function (require, exports, nls, winjs_base_1, paths, strings, platform_1, uri_1, event_1, textFileServices_1, textFileEditorModel_1, files_1, untitledEditorService_1, files_2, instantiation_1, contextService_1, lifecycle_1, telemetry_1, configuration_1, modeService_1, windowService_1) {
    'use strict';
    var TextFileService = (function (_super) {
        __extends(TextFileService, _super);
        function TextFileService(contextService, instantiationService, fileService, untitledEditorService, lifecycleService, telemetryService, configurationService, eventService, modeService, windowService) {
            _super.call(this, contextService, instantiationService, configurationService, telemetryService, lifecycleService, eventService);
            this.fileService = fileService;
            this.untitledEditorService = untitledEditorService;
            this.modeService = modeService;
            this.windowService = windowService;
            this.modeService = modeService;
            this.init();
        }
        TextFileService.prototype.beforeShutdown = function () {
            var _this = this;
            _super.prototype.beforeShutdown.call(this);
            // Dirty files need treatment on shutdown
            if (this.getDirty().length) {
                // If auto save is enabled, save all files and then check again for dirty files
                if (this.getAutoSaveMode() !== files_1.AutoSaveMode.OFF) {
                    return this.saveAll(false /* files only */).then(function () {
                        if (_this.getDirty().length) {
                            return _this.confirmBeforeShutdown(); // we still have dirty files around, so confirm normally
                        }
                        return false; // all good, no veto
                    });
                }
                // Otherwise just confirm what to do
                return this.confirmBeforeShutdown();
            }
            return false; // no veto
        };
        TextFileService.prototype.confirmBeforeShutdown = function () {
            var confirm = this.confirmSave();
            // Save
            if (confirm === files_1.ConfirmResult.SAVE) {
                return this.saveAll(true /* includeUntitled */).then(function (result) {
                    if (result.results.some(function (r) { return !r.success; })) {
                        return true; // veto if some saves failed
                    }
                    return false; // no veto
                });
            }
            else if (confirm === files_1.ConfirmResult.DONT_SAVE) {
                return false; // no veto
            }
            else if (confirm === files_1.ConfirmResult.CANCEL) {
                return true; // veto
            }
        };
        TextFileService.prototype.revertAll = function (resources, force) {
            var _this = this;
            // Revert files
            return _super.prototype.revertAll.call(this, resources, force).then(function (r) {
                // Revert untitled
                var untitledInputs = _this.untitledEditorService.getAll(resources);
                untitledInputs.forEach(function (input) {
                    if (input) {
                        input.dispose();
                        r.results.push({
                            source: input.getResource(),
                            success: true
                        });
                    }
                });
                return r;
            });
        };
        TextFileService.prototype.getDirty = function (resources) {
            var _this = this;
            // Collect files
            var dirty = _super.prototype.getDirty.call(this, resources);
            // Add untitled ones
            if (!resources) {
                dirty.push.apply(dirty, this.untitledEditorService.getDirty());
            }
            else {
                var dirtyUntitled = resources.map(function (r) { return _this.untitledEditorService.get(r); }).filter(function (u) { return u && u.isDirty(); }).map(function (u) { return u.getResource(); });
                dirty.push.apply(dirty, dirtyUntitled);
            }
            return dirty;
        };
        TextFileService.prototype.isDirty = function (resource) {
            if (_super.prototype.isDirty.call(this, resource)) {
                return true;
            }
            return this.untitledEditorService.getDirty().some(function (dirty) { return !resource || dirty.toString() === resource.toString(); });
        };
        TextFileService.prototype.confirmSave = function (resources) {
            if (!!this.contextService.getConfiguration().env.extensionDevelopmentPath) {
                return files_1.ConfirmResult.DONT_SAVE; // no veto when we are in extension dev mode because we cannot assum we run interactive (e.g. tests)
            }
            var resourcesToConfirm = this.getDirty(resources);
            if (resourcesToConfirm.length === 0) {
                return files_1.ConfirmResult.DONT_SAVE;
            }
            var message = [
                resourcesToConfirm.length === 1 ? nls.localize(0, null, paths.basename(resourcesToConfirm[0].fsPath)) : nls.localize(1, null)
            ];
            if (resourcesToConfirm.length > 1) {
                message.push('');
                message.push.apply(message, resourcesToConfirm.map(function (r) { return paths.basename(r.fsPath); }));
                message.push('');
            }
            // Button order
            // Windows: Save | Don't Save | Cancel
            // Mac/Linux: Save | Cancel | Don't
            var save = { label: resourcesToConfirm.length > 1 ? this.mnemonicLabel(nls.localize(2, null)) : this.mnemonicLabel(nls.localize(3, null)), result: files_1.ConfirmResult.SAVE };
            var dontSave = { label: this.mnemonicLabel(nls.localize(4, null)), result: files_1.ConfirmResult.DONT_SAVE };
            var cancel = { label: nls.localize(5, null), result: files_1.ConfirmResult.CANCEL };
            var buttons = [save];
            if (platform_1.isWindows) {
                buttons.push(dontSave, cancel);
            }
            else {
                buttons.push(cancel, dontSave);
            }
            var opts = {
                title: this.contextService.getConfiguration().env.appName,
                message: message.join('\n'),
                type: 'warning',
                detail: nls.localize(6, null),
                buttons: buttons.map(function (b) { return b.label; }),
                noLink: true,
                cancelId: buttons.indexOf(cancel)
            };
            var choice = this.windowService.getWindow().showMessageBox(opts);
            return buttons[choice].result;
        };
        TextFileService.prototype.mnemonicLabel = function (label) {
            if (!platform_1.isWindows) {
                return label.replace(/&&/g, ''); // no mnemonic support on mac/linux in buttons yet
            }
            return label.replace(/&&/g, '&');
        };
        TextFileService.prototype.saveAll = function (arg1) {
            // get all dirty
            var toSave = [];
            if (Array.isArray(arg1)) {
                toSave = this.getDirty(arg1);
            }
            else {
                toSave = this.getDirty();
            }
            // split up between files and untitled
            var filesToSave = [];
            var untitledToSave = [];
            toSave.forEach(function (s) {
                if (s.scheme === 'file') {
                    filesToSave.push(s);
                }
                else if ((Array.isArray(arg1) || arg1 === true /* includeUntitled */) && s.scheme === 'untitled') {
                    untitledToSave.push(s);
                }
            });
            return this.doSaveAll(filesToSave, untitledToSave);
        };
        TextFileService.prototype.doSaveAll = function (fileResources, untitledResources) {
            var _this = this;
            // Preflight for untitled to handle cancellation from the dialog
            var targetsForUntitled = [];
            for (var i = 0; i < untitledResources.length; i++) {
                var untitled = this.untitledEditorService.get(untitledResources[i]);
                if (untitled) {
                    var targetPath = void 0;
                    // Untitled with associated file path don't need to prompt
                    if (this.untitledEditorService.hasAssociatedFilePath(untitled.getResource())) {
                        targetPath = untitled.getResource().fsPath;
                    }
                    else {
                        targetPath = this.promptForPathSync(this.suggestFileName(untitledResources[i]));
                        if (!targetPath) {
                            return winjs_base_1.TPromise.as({
                                results: fileResources.concat(untitledResources).map(function (r) {
                                    return {
                                        source: r
                                    };
                                })
                            });
                        }
                    }
                    targetsForUntitled.push(uri_1.default.file(targetPath));
                }
            }
            // Handle files
            return _super.prototype.saveAll.call(this, fileResources).then(function (result) {
                // Handle untitled
                var untitledSaveAsPromises = [];
                targetsForUntitled.forEach(function (target, index) {
                    var untitledSaveAsPromise = _this.saveAs(untitledResources[index], target).then(function (uri) {
                        result.results.push({
                            source: untitledResources[index],
                            target: uri,
                            success: !!uri
                        });
                    });
                    untitledSaveAsPromises.push(untitledSaveAsPromise);
                });
                return winjs_base_1.TPromise.join(untitledSaveAsPromises).then(function () {
                    return result;
                });
            });
        };
        TextFileService.prototype.saveAs = function (resource, target) {
            var _this = this;
            // Get to target resource
            var targetPromise;
            if (target) {
                targetPromise = winjs_base_1.TPromise.as(target);
            }
            else {
                var dialogPath = resource.fsPath;
                if (resource.scheme === 'untitled') {
                    dialogPath = this.suggestFileName(resource);
                }
                targetPromise = this.promptForPathAsync(dialogPath).then(function (path) { return path ? uri_1.default.file(path) : null; });
            }
            return targetPromise.then(function (target) {
                if (!target) {
                    return null; // user canceled
                }
                // Just save if target is same as models own resource
                if (resource.toString() === target.toString()) {
                    return _this.save(resource).then(function () { return resource; });
                }
                // Do it
                return _this.doSaveAs(resource, target);
            });
        };
        TextFileService.prototype.doSaveAs = function (resource, target) {
            var _this = this;
            // Retrieve text model from provided resource if any
            var modelPromise = winjs_base_1.TPromise.as(null);
            if (resource.scheme === 'file') {
                modelPromise = winjs_base_1.TPromise.as(textFileEditorModel_1.CACHE.get(resource));
            }
            else if (resource.scheme === 'untitled') {
                var untitled = this.untitledEditorService.get(resource);
                if (untitled) {
                    modelPromise = untitled.resolve();
                }
            }
            return modelPromise.then(function (model) {
                // We have a model: Use it (can be null e.g. if this file is binary and not a text file or was never opened before)
                if (model) {
                    return _this.fileService.updateContent(target, model.getValue(), { encoding: model.getEncoding() });
                }
                // Otherwise we can only copy
                return _this.fileService.copyFile(resource, target);
            }).then(function () {
                // Add target to working files because this is an operation that indicates activity
                _this.getWorkingFilesModel().addEntry(target);
                // Revert the source
                return _this.revert(resource).then(function () {
                    // Done: return target
                    return target;
                });
            });
        };
        TextFileService.prototype.suggestFileName = function (untitledResource) {
            var workspace = this.contextService.getWorkspace();
            if (workspace) {
                return uri_1.default.file(paths.join(workspace.resource.fsPath, this.untitledEditorService.get(untitledResource).suggestFileName())).fsPath;
            }
            return this.untitledEditorService.get(untitledResource).suggestFileName();
        };
        TextFileService.prototype.promptForPathAsync = function (defaultPath) {
            var _this = this;
            return new winjs_base_1.TPromise(function (c, e) {
                _this.windowService.getWindow().showSaveDialog(_this.getSaveDialogOptions(defaultPath ? paths.normalize(defaultPath, true) : void 0), function (path) {
                    c(path);
                });
            });
        };
        TextFileService.prototype.promptForPathSync = function (defaultPath) {
            return this.windowService.getWindow().showSaveDialog(this.getSaveDialogOptions(defaultPath ? paths.normalize(defaultPath, true) : void 0));
        };
        TextFileService.prototype.getSaveDialogOptions = function (defaultPath) {
            var _this = this;
            var options = {
                defaultPath: defaultPath
            };
            // Filters are working flaky in Electron and there are bugs. On Windows they are working
            // somewhat but we see issues:
            // - https://github.com/atom/electron/issues/3556
            // - https://github.com/Microsoft/vscode/issues/451
            // - Bug on Windows: When "All Files" is picked, the path gets an extra ".*"
            // Until these issues are resolved, we disable the dialog file extension filtering.
            var disable = true; // Simply using if (true) flags the code afterwards as not reachable.
            if (disable) {
                return options;
            }
            // Build the file filter by using our known languages
            var ext = paths.extname(defaultPath);
            var matchingFilter;
            var filters = this.modeService.getRegisteredLanguageNames().map(function (languageName) {
                var extensions = _this.modeService.getExtensions(languageName);
                if (!extensions || !extensions.length) {
                    return null;
                }
                var filter = { name: languageName, extensions: extensions.map(function (e) { return strings.trim(e, '.'); }) };
                if (ext && extensions.indexOf(ext) >= 0) {
                    matchingFilter = filter;
                    return null; // matching filter will be added last to the top
                }
                return filter;
            }).filter(function (f) { return !!f; });
            // Filters are a bit weird on Windows, based on having a match or not:
            // Match: we put the matching filter first so that it shows up selected and the all files last
            // No match: we put the all files filter first
            var allFilesFilter = { name: nls.localize(7, null), extensions: ['*'] };
            if (matchingFilter) {
                filters.unshift(matchingFilter);
                filters.push(allFilesFilter);
            }
            else {
                filters.unshift(allFilesFilter);
            }
            options.filters = filters;
            return options;
        };
        TextFileService = __decorate([
            __param(0, contextService_1.IWorkspaceContextService),
            __param(1, instantiation_1.IInstantiationService),
            __param(2, files_2.IFileService),
            __param(3, untitledEditorService_1.IUntitledEditorService),
            __param(4, lifecycle_1.ILifecycleService),
            __param(5, telemetry_1.ITelemetryService),
            __param(6, configuration_1.IConfigurationService),
            __param(7, event_1.IEventService),
            __param(8, modeService_1.IModeService),
            __param(9, windowService_1.IWindowService)
        ], TextFileService);
        return TextFileService;
    }(textFileServices_1.TextFileService));
    exports.TextFileService = TextFileService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/files/electron-browser/files.electron.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/files/electron-browser/files.electron.contribution', 'vs/platform/platform', 'vs/workbench/browser/actionBarRegistry', 'vs/workbench/common/actionRegistry', 'vs/platform/actions/common/actions', 'vs/base/common/platform', 'vs/workbench/parts/files/common/files', 'vs/workbench/common/contributions', 'vs/workbench/parts/files/browser/fileActions', 'vs/workbench/parts/files/electron-browser/electronFileTracker', 'vs/workbench/parts/files/electron-browser/textFileServices', 'vs/workbench/parts/files/electron-browser/electronFileActions', 'vs/platform/instantiation/common/instantiation', 'vs/platform/instantiation/common/extensions', 'vs/base/common/keyCodes'], function (require, exports, nls, platform_1, actionBarRegistry_1, actionRegistry_1, actions_1, env, files_1, contributions_1, fileActions_1, electronFileTracker_1, textFileServices_1, electronFileActions_1, instantiation_1, extensions_1, keyCodes_1) {
    'use strict';
    var FileViewerActionContributor = (function (_super) {
        __extends(FileViewerActionContributor, _super);
        function FileViewerActionContributor(instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
        }
        FileViewerActionContributor.prototype.hasSecondaryActions = function (context) {
            var element = context.element;
            // Contribute only on Files (File Explorer and Open Files Viewer)
            return !!files_1.asFileResource(element);
        };
        FileViewerActionContributor.prototype.getSecondaryActions = function (context) {
            var actions = [];
            if (this.hasSecondaryActions(context)) {
                var fileResource = files_1.asFileResource(context.element);
                // Reveal file in OS native explorer
                actions.push(this.instantiationService.createInstance(electronFileActions_1.RevealInOSAction, fileResource.resource));
                // Copy Path
                actions.push(this.instantiationService.createInstance(electronFileActions_1.CopyPathAction, fileResource.resource));
            }
            return actions;
        };
        FileViewerActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], FileViewerActionContributor);
        return FileViewerActionContributor;
    }(actionBarRegistry_1.ActionBarContributor));
    // Contribute Actions
    var category = nls.localize(0, null);
    var workbenchActionsRegistry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.SaveFileAsAction, fileActions_1.SaveFileAsAction.ID, fileActions_1.SaveFileAsAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_S }), category);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(fileActions_1.GlobalNewFileAction, fileActions_1.GlobalNewFileAction.ID, fileActions_1.GlobalNewFileAction.LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_N }), category);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(electronFileActions_1.GlobalCopyPathAction, electronFileActions_1.GlobalCopyPathAction.ID, electronFileActions_1.GlobalCopyPathAction.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.KEY_P) }), category);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(electronFileActions_1.GlobalRevealInOSAction, electronFileActions_1.GlobalRevealInOSAction.ID, electronFileActions_1.GlobalRevealInOSAction.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.KEY_R) }), category);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(electronFileActions_1.ShowOpenedFileInNewWindow, electronFileActions_1.ShowOpenedFileInNewWindow.ID, electronFileActions_1.ShowOpenedFileInNewWindow.LABEL, { primary: keyCodes_1.KeyMod.chord(keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_K, keyCodes_1.KeyCode.KEY_O) }), category);
    if (env.isMacintosh) {
        workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(electronFileActions_1.OpenFileFolderAction, electronFileActions_1.OPEN_FILE_FOLDER_ID, electronFileActions_1.OPEN_FILE_FOLDER_LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_O }), category);
    }
    else {
        workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(electronFileActions_1.OpenFileAction, electronFileActions_1.OPEN_FILE_ID, electronFileActions_1.OPEN_FILE_LABEL, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_O }), category);
        workbenchActionsRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(electronFileActions_1.OpenFolderAction, electronFileActions_1.OPEN_FOLDER_ID, electronFileActions_1.OPEN_FOLDER_LABEL), category);
    }
    // Contribute to File Viewers
    var actionsRegistry = platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar);
    actionsRegistry.registerActionBarContributor(actionBarRegistry_1.Scope.VIEWER, FileViewerActionContributor);
    // Register File Workbench Extension
    platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(electronFileTracker_1.FileTracker);
    // Register Service
    extensions_1.registerSingleton(files_1.ITextFileService, textFileServices_1.TextFileService);
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/gettingStarted/common/abstractGettingStarted", ["require", "exports", 'vs/workbench/services/workspace/common/contextService', 'vs/platform/storage/common/storage', 'vs/platform/telemetry/common/telemetry'], function (require, exports, contextService_1, storage_1, telemetry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * This extensions handles the first launch expereince for new users
     */
    var AbstractGettingStarted = (function () {
        function AbstractGettingStarted(storageService, contextService, telemetryService) {
            this.storageService = storageService;
            this.contextService = contextService;
            this.telemetryService = telemetryService;
            var env = contextService.getConfiguration().env;
            this.appName = env.appName;
            if (env.welcomePage && !env.extensionTestsPath /* do not open a browser when we run tests */) {
                this.welcomePageURL = env.welcomePage;
                this.handleWelcome();
            }
        }
        AbstractGettingStarted.prototype.handleWelcome = function () {
            var _this = this;
            var firstStartup = !this.storageService.get(AbstractGettingStarted.hideWelcomeSettingskey);
            if (firstStartup && this.welcomePageURL) {
                this.telemetryService.getTelemetryInfo().then(function (info) {
                    var url = _this.getUrl(info);
                    _this.openExternal(url);
                    _this.storageService.store(AbstractGettingStarted.hideWelcomeSettingskey, true);
                });
            }
        };
        AbstractGettingStarted.prototype.getUrl = function (telemetryInfo) {
            return this.welcomePageURL + "&&from=" + this.appName + "&&id=" + telemetryInfo.machineId;
        };
        AbstractGettingStarted.prototype.openExternal = function (url) {
            throw new Error('implement me');
        };
        AbstractGettingStarted.prototype.getId = function () {
            return 'vs.gettingstarted';
        };
        AbstractGettingStarted.hideWelcomeSettingskey = 'workbench.hide.welcome';
        AbstractGettingStarted = __decorate([
            __param(0, storage_1.IStorageService),
            __param(1, contextService_1.IWorkspaceContextService),
            __param(2, telemetry_1.ITelemetryService)
        ], AbstractGettingStarted);
        return AbstractGettingStarted;
    }());
    exports.AbstractGettingStarted = AbstractGettingStarted;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/parts/gettingStarted/electron-browser/electronGettingStarted", ["require", "exports", 'vs/workbench/parts/gettingStarted/common/abstractGettingStarted', 'electron'], function (require, exports, abstractGettingStarted_1, electron_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ElectronGettingStarted = (function (_super) {
        __extends(ElectronGettingStarted, _super);
        function ElectronGettingStarted() {
            _super.apply(this, arguments);
        }
        ElectronGettingStarted.prototype.openExternal = function (url) {
            electron_1.shell.openExternal(url);
        };
        ElectronGettingStarted.prototype.handleWelcome = function () {
            //make sure the user is online, otherwise refer to the next run to show the welcome page
            if (navigator.onLine) {
                _super.prototype.handleWelcome.call(this);
            }
        };
        return ElectronGettingStarted;
    }(abstractGettingStarted_1.AbstractGettingStarted));
    exports.ElectronGettingStarted = ElectronGettingStarted;
});

define("vs/workbench/parts/gettingStarted/electron-browser/electronGettingStarted.contribution", ["require", "exports", 'vs/platform/platform', 'vs/workbench/parts/gettingStarted/electron-browser/electronGettingStarted', 'vs/workbench/common/contributions'], function (require, exports, platform_1, electronGettingStarted_1, contributions_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(electronGettingStarted_1.ElectronGettingStarted);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/git/browser/gitActions", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/git/browser/gitActions', 'vs/base/common/lifecycle', 'vs/base/common/strings', 'vs/base/common/types', 'vs/base/common/actions', 'vs/workbench/parts/git/common/gitModel', 'vs/workbench/parts/git/browser/gitEditorInputs', 'vs/workbench/common/editor', 'vs/base/common/errors', 'vs/base/common/platform', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/event/common/event', 'vs/platform/files/common/files', 'vs/platform/message/common/message', 'vs/base/common/severity', 'vs/workbench/parts/git/common/git', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/base/common/paths', 'vs/base/common/uri'], function (require, exports, winjs_base_1, nls, lifecycle_1, strings, types_1, actions_1, model, inputs, editor_1, errors, platform, editorService_1, contextService_1, event_1, files_1, message_1, severity_1, git_1, quickOpenService_1, paths, uri_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function flatten(context, preferFocus) {
        if (preferFocus === void 0) { preferFocus = false; }
        if (!context) {
            return context;
        }
        else if (Array.isArray(context)) {
            if (context.some(function (c) { return !(c instanceof model.FileStatus); })) {
                throw new Error('Invalid context.');
            }
            return context;
        }
        else if (context instanceof model.FileStatus) {
            return [context];
        }
        else if (context instanceof model.StatusGroup) {
            return context.all();
        }
        else if (context.tree) {
            var elements = context.tree.getSelection();
            return elements.indexOf(context.fileStatus) > -1 ? elements : [context.fileStatus];
        }
        else if (context.selection) {
            return !preferFocus && context.selection.indexOf(context.focus) > -1 ? context.selection : [context.focus];
        }
        else {
            throw new Error('Invalid context.');
        }
    }
    var GitAction = (function (_super) {
        __extends(GitAction, _super);
        function GitAction(id, label, cssClass, gitService) {
            var _this = this;
            _super.call(this, id, label, cssClass, false);
            this.gitService = gitService;
            this.toDispose = [this.gitService.addBulkListener2(function () { return _this.onGitServiceChange(); })];
            this.onGitServiceChange();
        }
        GitAction.prototype.onGitServiceChange = function () {
            this.updateEnablement();
        };
        GitAction.prototype.updateEnablement = function () {
            this.enabled = this.isEnabled();
        };
        GitAction.prototype.isEnabled = function () {
            return !!this.gitService;
        };
        GitAction.prototype.dispose = function () {
            this.gitService = null;
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            _super.prototype.dispose.call(this);
        };
        return GitAction;
    }(actions_1.Action));
    exports.GitAction = GitAction;
    var OpenChangeAction = (function (_super) {
        __extends(OpenChangeAction, _super);
        function OpenChangeAction(editorService, gitService) {
            _super.call(this, OpenChangeAction.ID, nls.localize(0, null), 'git-action open-change', gitService);
            this.editorService = editorService;
            this.onGitServiceChange();
        }
        OpenChangeAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && !!this.editorService;
        };
        OpenChangeAction.prototype.run = function (context) {
            var _this = this;
            var statuses = flatten(context, true);
            return this.gitService.getInput(statuses[0]).then(function (input) {
                var options = new editor_1.TextDiffEditorOptions();
                options.forceOpen = true;
                return _this.editorService.openEditor(input, options);
            });
        };
        OpenChangeAction.ID = 'workbench.action.git.openChange';
        OpenChangeAction = __decorate([
            __param(0, editorService_1.IWorkbenchEditorService),
            __param(1, git_1.IGitService)
        ], OpenChangeAction);
        return OpenChangeAction;
    }(GitAction));
    exports.OpenChangeAction = OpenChangeAction;
    var OpenFileAction = (function (_super) {
        __extends(OpenFileAction, _super);
        function OpenFileAction(editorService, fileService, gitService, contextService) {
            _super.call(this, OpenFileAction.ID, nls.localize(1, null), 'git-action open-file', gitService);
            this.fileService = fileService;
            this.editorService = editorService;
            this.contextService = contextService;
            this.onGitServiceChange();
        }
        OpenFileAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && !!this.editorService || !!this.fileService;
        };
        OpenFileAction.prototype.getPath = function (status) {
            if (status.getStatus() === git_1.Status.INDEX_RENAMED) {
                return status.getRename();
            }
            else {
                var indexStatus = this.gitService.getModel().getStatus().find(status.getPath(), git_1.StatusType.INDEX);
                if (indexStatus && indexStatus.getStatus() === git_1.Status.INDEX_RENAMED) {
                    return status.getRename();
                }
                else {
                    return status.getPath();
                }
            }
        };
        OpenFileAction.prototype.run = function (context) {
            var _this = this;
            var statuses = flatten(context, true);
            var status = statuses[0];
            if (!(status instanceof model.FileStatus)) {
                return winjs_base_1.Promise.wrapError(new Error('Can\'t open file.'));
            }
            if (OpenFileAction.DELETED_STATES.indexOf(status.getStatus()) > -1) {
                return winjs_base_1.Promise.wrapError(new Error('Can\'t open file which is has been deleted.'));
            }
            var resource = uri_1.default.file(paths.join(this.gitService.getModel().getRepositoryRoot(), this.getPath(status)));
            return this.fileService.resolveFile(resource)
                .then(function (stat) { return _this.editorService.openEditor({
                resource: stat.resource,
                mime: stat.mime,
                options: { forceOpen: true }
            }); });
        };
        OpenFileAction.DELETED_STATES = [git_1.Status.BOTH_DELETED, git_1.Status.DELETED, git_1.Status.DELETED_BY_US, git_1.Status.INDEX_DELETED];
        OpenFileAction.ID = 'workbench.action.git.openFile';
        OpenFileAction = __decorate([
            __param(0, editorService_1.IWorkbenchEditorService),
            __param(1, files_1.IFileService),
            __param(2, git_1.IGitService),
            __param(3, contextService_1.IWorkspaceContextService)
        ], OpenFileAction);
        return OpenFileAction;
    }(GitAction));
    exports.OpenFileAction = OpenFileAction;
    var InitAction = (function (_super) {
        __extends(InitAction, _super);
        function InitAction(gitService) {
            _super.call(this, InitAction.ID, nls.localize(2, null), 'git-action init', gitService);
        }
        InitAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.gitService.getState() === git_1.ServiceState.NotARepo;
        };
        InitAction.prototype.run = function () {
            return this.gitService.init();
        };
        InitAction.ID = 'workbench.action.git.init';
        InitAction = __decorate([
            __param(0, git_1.IGitService)
        ], InitAction);
        return InitAction;
    }(GitAction));
    exports.InitAction = InitAction;
    var RefreshAction = (function (_super) {
        __extends(RefreshAction, _super);
        function RefreshAction(gitService) {
            _super.call(this, RefreshAction.ID, nls.localize(3, null), 'git-action refresh', gitService);
        }
        RefreshAction.prototype.run = function () {
            return this.gitService.status();
        };
        RefreshAction.ID = 'workbench.action.git.refresh';
        RefreshAction = __decorate([
            __param(0, git_1.IGitService)
        ], RefreshAction);
        return RefreshAction;
    }(GitAction));
    exports.RefreshAction = RefreshAction;
    var BaseStageAction = (function (_super) {
        __extends(BaseStageAction, _super);
        function BaseStageAction(id, label, className, gitService, editorService) {
            _super.call(this, id, label, className, gitService);
            this.editorService = editorService;
        }
        BaseStageAction.prototype.run = function (context) {
            var _this = this;
            var flatContext = flatten(context);
            return this.gitService.add(flatten(context)).then(function (status) {
                var targetEditor = _this.findGitWorkingTreeEditor();
                if (!targetEditor) {
                    return winjs_base_1.TPromise.as(status);
                }
                var currentGitEditorInput = targetEditor.input;
                var currentFileStatus = currentGitEditorInput.getFileStatus();
                if (flatContext && flatContext.every(function (f) { return f !== currentFileStatus; })) {
                    return winjs_base_1.TPromise.as(status);
                }
                var path = currentGitEditorInput.getFileStatus().getPath();
                var fileStatus = status.getStatus().find(path, git_1.StatusType.INDEX);
                if (!fileStatus) {
                    return winjs_base_1.TPromise.as(status);
                }
                var editorControl = targetEditor.getControl();
                var viewState = editorControl ? editorControl.saveViewState() : null;
                return _this.gitService.getInput(fileStatus).then(function (input) {
                    var options = new editor_1.TextDiffEditorOptions();
                    options.forceOpen = true;
                    return _this.editorService.openEditor(input, options, targetEditor.position).then(function (editor) {
                        if (viewState) {
                            editorControl.restoreViewState(viewState);
                        }
                        return status;
                    });
                });
            });
        };
        BaseStageAction.prototype.findGitWorkingTreeEditor = function () {
            var editors = this.editorService.getVisibleEditors();
            for (var i = 0; i < editors.length; i++) {
                var editor = editors[i];
                if (inputs.isGitEditorInput(editor.input)) {
                    return editor;
                }
            }
            return null;
        };
        BaseStageAction.prototype.dispose = function () {
            this.editorService = null;
            _super.prototype.dispose.call(this);
        };
        return BaseStageAction;
    }(GitAction));
    exports.BaseStageAction = BaseStageAction;
    var StageAction = (function (_super) {
        __extends(StageAction, _super);
        function StageAction(gitService, editorService) {
            _super.call(this, StageAction.ID, StageAction.LABEL, 'git-action stage', gitService, editorService);
        }
        StageAction.ID = 'workbench.action.git.stage';
        StageAction.LABEL = nls.localize(4, null);
        StageAction = __decorate([
            __param(0, git_1.IGitService),
            __param(1, editorService_1.IWorkbenchEditorService)
        ], StageAction);
        return StageAction;
    }(BaseStageAction));
    exports.StageAction = StageAction;
    var GlobalStageAction = (function (_super) {
        __extends(GlobalStageAction, _super);
        function GlobalStageAction(gitService, editorService) {
            _super.call(this, GlobalStageAction.ID, nls.localize(5, null), 'git-action stage', gitService, editorService);
        }
        GlobalStageAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.gitService.getModel().getStatus().getWorkingTreeStatus().all().length > 0;
        };
        GlobalStageAction.prototype.run = function (context) {
            return _super.prototype.run.call(this);
        };
        GlobalStageAction.ID = 'workbench.action.git.stageAll';
        GlobalStageAction = __decorate([
            __param(0, git_1.IGitService),
            __param(1, editorService_1.IWorkbenchEditorService)
        ], GlobalStageAction);
        return GlobalStageAction;
    }(BaseStageAction));
    exports.GlobalStageAction = GlobalStageAction;
    var BaseUndoAction = (function (_super) {
        __extends(BaseUndoAction, _super);
        function BaseUndoAction(id, label, className, gitService, eventService, messageService, fileService, editorService, contextService) {
            _super.call(this, id, label, className, gitService);
            this.eventService = eventService;
            this.editorService = editorService;
            this.messageService = messageService;
            this.fileService = fileService;
            this.contextService = contextService;
            this.onGitServiceChange();
        }
        BaseUndoAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && !!this.eventService && !!this.editorService && !!this.fileService;
        };
        BaseUndoAction.prototype.run = function (context) {
            var _this = this;
            if (!this.messageService.confirm(this.getConfirm(context))) {
                return winjs_base_1.TPromise.as(null);
            }
            var promises = [];
            if (context instanceof model.StatusGroup) {
                promises = [this.gitService.undo()];
            }
            else {
                var all = flatten(context);
                var toClean = [];
                var toCheckout = [];
                for (var i = 0; i < all.length; i++) {
                    var status = all[i].clone();
                    switch (status.getStatus()) {
                        case git_1.Status.UNTRACKED:
                        case git_1.Status.IGNORED:
                            toClean.push(status);
                            break;
                        default:
                            toCheckout.push(status);
                            break;
                    }
                }
                if (toClean.length > 0) {
                    promises.push(this.gitService.clean(toClean));
                }
                if (toCheckout.length > 0) {
                    promises.push(this.gitService.checkout('', toCheckout));
                }
            }
            return winjs_base_1.Promise.join(promises).then(function (statuses) {
                if (statuses.length === 0) {
                    return winjs_base_1.TPromise.as(null);
                }
                var status = statuses[statuses.length - 1];
                var targetEditor = _this.findWorkingTreeDiffEditor();
                if (!targetEditor) {
                    return winjs_base_1.TPromise.as(status);
                }
                var currentGitEditorInput = targetEditor.input;
                var currentFileStatus = currentGitEditorInput.getFileStatus();
                if (all && all.every(function (f) { return f !== currentFileStatus; })) {
                    return winjs_base_1.TPromise.as(status);
                }
                var path = currentGitEditorInput.getFileStatus().getPath();
                var editor = targetEditor.getControl();
                var modifiedEditorControl = editor ? editor.getModifiedEditor() : null;
                var modifiedViewState = modifiedEditorControl ? modifiedEditorControl.saveViewState() : null;
                return _this.fileService.resolveFile(_this.contextService.toResource(path)).then(function (stat) {
                    return _this.editorService.openEditor({
                        resource: stat.resource,
                        mime: stat.mime,
                        options: {
                            forceOpen: true
                        }
                    }, targetEditor.position).then(function (editor) {
                        if (modifiedViewState) {
                            var codeEditor = targetEditor.getControl();
                            if (codeEditor) {
                                codeEditor.restoreViewState(modifiedViewState);
                            }
                        }
                    });
                });
            }).then(null, function (errors) {
                console.error('One or more errors occurred', errors);
                return winjs_base_1.Promise.wrapError(errors[0]);
            });
        };
        BaseUndoAction.prototype.findWorkingTreeDiffEditor = function () {
            var editors = this.editorService.getVisibleEditors();
            for (var i = 0; i < editors.length; i++) {
                var editor = editors[i];
                if (editor.input instanceof inputs.GitWorkingTreeDiffEditorInput) {
                    return editor;
                }
            }
            return null;
        };
        BaseUndoAction.prototype.getConfirm = function (context) {
            var all = flatten(context);
            if (all.length > 1) {
                var count = all.length;
                return {
                    message: nls.localize(6, null),
                    detail: count === 1
                        ? nls.localize(7, null, count)
                        : nls.localize(8, null, count),
                    primaryButton: nls.localize(9, null)
                };
            }
            var label = all[0].getPathComponents().reverse()[0];
            return {
                message: nls.localize(10, null, label),
                detail: nls.localize(11, null),
                primaryButton: nls.localize(12, null)
            };
        };
        BaseUndoAction.prototype.dispose = function () {
            this.eventService = null;
            this.editorService = null;
            this.fileService = null;
            _super.prototype.dispose.call(this);
        };
        return BaseUndoAction;
    }(GitAction));
    exports.BaseUndoAction = BaseUndoAction;
    var UndoAction = (function (_super) {
        __extends(UndoAction, _super);
        function UndoAction(gitService, eventService, messageService, fileService, editorService, contextService) {
            _super.call(this, UndoAction.ID, nls.localize(13, null), 'git-action undo', gitService, eventService, messageService, fileService, editorService, contextService);
        }
        UndoAction.ID = 'workbench.action.git.undo';
        UndoAction = __decorate([
            __param(0, git_1.IGitService),
            __param(1, event_1.IEventService),
            __param(2, message_1.IMessageService),
            __param(3, files_1.IFileService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, contextService_1.IWorkspaceContextService)
        ], UndoAction);
        return UndoAction;
    }(BaseUndoAction));
    exports.UndoAction = UndoAction;
    var GlobalUndoAction = (function (_super) {
        __extends(GlobalUndoAction, _super);
        function GlobalUndoAction(gitService, eventService, messageService, fileService, editorService, contextService) {
            _super.call(this, GlobalUndoAction.ID, nls.localize(14, null), 'git-action undo', gitService, eventService, messageService, fileService, editorService, contextService);
        }
        GlobalUndoAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.gitService.getModel().getStatus().getWorkingTreeStatus().all().length > 0;
        };
        GlobalUndoAction.prototype.run = function (context) {
            return _super.prototype.run.call(this, this.gitService.getModel().getStatus().getWorkingTreeStatus());
        };
        GlobalUndoAction.ID = 'workbench.action.git.undoAll';
        GlobalUndoAction = __decorate([
            __param(0, git_1.IGitService),
            __param(1, event_1.IEventService),
            __param(2, message_1.IMessageService),
            __param(3, files_1.IFileService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, contextService_1.IWorkspaceContextService)
        ], GlobalUndoAction);
        return GlobalUndoAction;
    }(BaseUndoAction));
    exports.GlobalUndoAction = GlobalUndoAction;
    var BaseUnstageAction = (function (_super) {
        __extends(BaseUnstageAction, _super);
        function BaseUnstageAction(id, label, className, gitService, editorService) {
            _super.call(this, id, label, className, gitService);
            this.editorService = editorService;
            this.onGitServiceChange();
        }
        BaseUnstageAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && !!this.editorService;
        };
        BaseUnstageAction.prototype.run = function (context) {
            var _this = this;
            var flatContext = flatten(context);
            return this.gitService.revertFiles('HEAD', flatContext).then(function (status) {
                var targetEditor = _this.findGitIndexEditor();
                if (!targetEditor) {
                    return winjs_base_1.TPromise.as(status);
                }
                var currentGitEditorInput = targetEditor.input;
                var currentFileStatus = currentGitEditorInput.getFileStatus();
                if (flatContext && flatContext.every(function (f) { return f !== currentFileStatus; })) {
                    return winjs_base_1.TPromise.as(status);
                }
                var path = currentGitEditorInput.getFileStatus().getPath();
                var fileStatus = status.getStatus().find(path, git_1.StatusType.WORKING_TREE);
                if (!fileStatus) {
                    return winjs_base_1.TPromise.as(status);
                }
                var editorControl = targetEditor.getControl();
                var viewState = editorControl ? editorControl.saveViewState() : null;
                return _this.gitService.getInput(fileStatus).then(function (input) {
                    var options = new editor_1.TextDiffEditorOptions();
                    options.forceOpen = true;
                    return _this.editorService.openEditor(input, options, targetEditor.position).then(function (editor) {
                        if (viewState) {
                            editorControl.restoreViewState(viewState);
                        }
                        return status;
                    });
                });
            });
        };
        BaseUnstageAction.prototype.findGitIndexEditor = function () {
            var editors = this.editorService.getVisibleEditors();
            for (var i = 0; i < editors.length; i++) {
                var editor = editors[i];
                if (inputs.isGitEditorInput(editor.input)) {
                    return editor;
                }
            }
            return null;
        };
        BaseUnstageAction.prototype.dispose = function () {
            this.editorService = null;
            _super.prototype.dispose.call(this);
        };
        return BaseUnstageAction;
    }(GitAction));
    exports.BaseUnstageAction = BaseUnstageAction;
    var UnstageAction = (function (_super) {
        __extends(UnstageAction, _super);
        function UnstageAction(gitService, editorService) {
            _super.call(this, UnstageAction.ID, nls.localize(15, null), 'git-action unstage', gitService, editorService);
        }
        UnstageAction.ID = 'workbench.action.git.unstage';
        UnstageAction = __decorate([
            __param(0, git_1.IGitService),
            __param(1, editorService_1.IWorkbenchEditorService)
        ], UnstageAction);
        return UnstageAction;
    }(BaseUnstageAction));
    exports.UnstageAction = UnstageAction;
    var GlobalUnstageAction = (function (_super) {
        __extends(GlobalUnstageAction, _super);
        function GlobalUnstageAction(gitService, editorService) {
            _super.call(this, GlobalUnstageAction.ID, nls.localize(16, null), 'git-action unstage', gitService, editorService);
        }
        GlobalUnstageAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.gitService.getModel().getStatus().getIndexStatus().all().length > 0;
        };
        GlobalUnstageAction.prototype.run = function (context) {
            return _super.prototype.run.call(this);
        };
        GlobalUnstageAction.ID = 'workbench.action.git.unstageAll';
        GlobalUnstageAction = __decorate([
            __param(0, git_1.IGitService),
            __param(1, editorService_1.IWorkbenchEditorService)
        ], GlobalUnstageAction);
        return GlobalUnstageAction;
    }(BaseUnstageAction));
    exports.GlobalUnstageAction = GlobalUnstageAction;
    var LifecycleState;
    (function (LifecycleState) {
        LifecycleState[LifecycleState["Alive"] = 0] = "Alive";
        LifecycleState[LifecycleState["Disposing"] = 1] = "Disposing";
        LifecycleState[LifecycleState["Disposed"] = 2] = "Disposed";
    })(LifecycleState || (LifecycleState = {}));
    var CheckoutAction = (function (_super) {
        __extends(CheckoutAction, _super);
        function CheckoutAction(branch, gitService, editorService) {
            _super.call(this, CheckoutAction.ID, branch.name, 'git-action checkout', gitService);
            this.editorService = editorService;
            this.branch = branch;
            this.HEAD = null;
            this.state = LifecycleState.Alive;
            this.runPromises = [];
            this.onGitServiceChange();
        }
        CheckoutAction.prototype.onGitServiceChange = function () {
            if (this.gitService.getState() === git_1.ServiceState.OK) {
                this.HEAD = this.gitService.getModel().getHEAD();
                if (this.HEAD && this.HEAD.name === this.branch.name) {
                    this.class = 'git-action checkout HEAD';
                }
                else {
                    this.class = 'git-action checkout';
                }
            }
            _super.prototype.onGitServiceChange.call(this);
        };
        CheckoutAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && !!this.HEAD;
        };
        CheckoutAction.prototype.run = function (context) {
            var _this = this;
            if (this.state !== LifecycleState.Alive) {
                return winjs_base_1.Promise.wrapError('action disposed');
            }
            else if (this.HEAD && this.HEAD.name === this.branch.name) {
                return winjs_base_1.TPromise.as(null);
            }
            var result = this.gitService.checkout(this.branch.name).then(null, function (err) {
                if (err.gitErrorCode === git_1.GitErrorCodes.DirtyWorkTree) {
                    return winjs_base_1.Promise.wrapError(new Error(nls.localize(17, null)));
                }
                return winjs_base_1.Promise.wrapError(err);
            });
            this.runPromises.push(result);
            result.done(function () { return _this.runPromises.splice(_this.runPromises.indexOf(result), 1); });
            return result;
        };
        CheckoutAction.prototype.dispose = function () {
            var _this = this;
            if (this.state !== LifecycleState.Alive) {
                return;
            }
            this.state = LifecycleState.Disposing;
            winjs_base_1.Promise.join(this.runPromises).done(function () { return _this.actuallyDispose(); });
        };
        CheckoutAction.prototype.actuallyDispose = function () {
            this.editorService = null;
            this.branch = null;
            this.HEAD = null;
            _super.prototype.dispose.call(this);
            this.state = LifecycleState.Disposed;
        };
        CheckoutAction.ID = 'workbench.action.git.checkout';
        CheckoutAction = __decorate([
            __param(1, git_1.IGitService),
            __param(2, editorService_1.IWorkbenchEditorService)
        ], CheckoutAction);
        return CheckoutAction;
    }(GitAction));
    exports.CheckoutAction = CheckoutAction;
    var BranchAction = (function (_super) {
        __extends(BranchAction, _super);
        function BranchAction(checkout, gitService) {
            _super.call(this, BranchAction.ID, nls.localize(18, null), 'git-action checkout', gitService);
            this.checkout = checkout;
        }
        BranchAction.prototype.run = function (context) {
            if (!types_1.isString(context)) {
                return winjs_base_1.TPromise.as(false);
            }
            return this.gitService.branch(context, this.checkout);
        };
        BranchAction.ID = 'workbench.action.git.branch';
        BranchAction = __decorate([
            __param(1, git_1.IGitService)
        ], BranchAction);
        return BranchAction;
    }(GitAction));
    exports.BranchAction = BranchAction;
    var BaseCommitAction = (function (_super) {
        __extends(BaseCommitAction, _super);
        function BaseCommitAction(commitState, id, label, cssClass, gitService) {
            var _this = this;
            _super.call(this, id, label, cssClass, gitService);
            this.commitState = commitState;
            this.toDispose.push(commitState.addListener2('change/commitInputBox', function () {
                _this.updateEnablement();
            }));
            this.onGitServiceChange();
        }
        BaseCommitAction.prototype.isEnabled = function () {
            return _super.prototype.isEnabled.call(this) && this.gitService.getModel().getStatus().getIndexStatus().all().length > 0;
        };
        BaseCommitAction.prototype.run = function (context) {
            if (!this.commitState.getCommitMessage()) {
                this.commitState.onEmptyCommitMessage();
                return winjs_base_1.TPromise.as(null);
            }
            return this.gitService.commit(this.commitState.getCommitMessage());
        };
        return BaseCommitAction;
    }(GitAction));
    exports.BaseCommitAction = BaseCommitAction;
    var CommitAction = (function (_super) {
        __extends(CommitAction, _super);
        function CommitAction(commitState, gitService) {
            _super.call(this, commitState, CommitAction.ID, nls.localize(19, null), 'git-action commit', gitService);
        }
        CommitAction.ID = 'workbench.action.git.commit';
        CommitAction = __decorate([
            __param(1, git_1.IGitService)
        ], CommitAction);
        return CommitAction;
    }(BaseCommitAction));
    exports.CommitAction = CommitAction;
    var StageAndCommitAction = (function (_super) {
        __extends(StageAndCommitAction, _super);
        function StageAndCommitAction(commitState, gitService) {
            _super.call(this, commitState, StageAndCommitAction.ID, nls.localize(20, null), 'git-action stage-and-commit', gitService);
        }
        StageAndCommitAction.prototype.isEnabled = function () {
            if (!this.gitService) {
                return false;
            }
            if (!this.gitService.isIdle()) {
                return false;
            }
            var status = this.gitService.getModel().getStatus();
            return status.getIndexStatus().all().length > 0
                || status.getWorkingTreeStatus().all().length > 0;
        };
        StageAndCommitAction.prototype.run = function (context) {
            if (!this.commitState.getCommitMessage()) {
                this.commitState.onEmptyCommitMessage();
                return winjs_base_1.TPromise.as(null);
            }
            return this.gitService.commit(this.commitState.getCommitMessage(), false, true);
        };
        StageAndCommitAction.ID = 'workbench.action.git.stageAndCommit';
        StageAndCommitAction = __decorate([
            __param(1, git_1.IGitService)
        ], StageAndCommitAction);
        return StageAndCommitAction;
    }(BaseCommitAction));
    exports.StageAndCommitAction = StageAndCommitAction;
    var SmartCommitAction = (function (_super) {
        __extends(SmartCommitAction, _super);
        function SmartCommitAction(commitState, gitService, messageService) {
            _super.call(this, commitState, SmartCommitAction.ID, SmartCommitAction.ALL, 'git-action smart-commit', gitService);
            this.messageService = messageService;
            this.onGitServiceChange();
        }
        SmartCommitAction.prototype.onGitServiceChange = function () {
            _super.prototype.onGitServiceChange.call(this);
            if (!this.enabled) {
                this.label = SmartCommitAction.ALL;
                return;
            }
            var status = this.gitService.getModel().getStatus();
            if (status.getIndexStatus().all().length > 0) {
                this.label = SmartCommitAction.STAGED;
            }
            else {
                this.label = SmartCommitAction.ALL;
            }
            this.label += ' (' + (platform.isMacintosh ? 'Cmd+Enter' : 'Ctrl+Enter') + ')';
        };
        SmartCommitAction.prototype.isEnabled = function () {
            if (!this.gitService) {
                return false;
            }
            if (!this.gitService.isIdle()) {
                return false;
            }
            var status = this.gitService.getModel().getStatus();
            return status.getIndexStatus().all().length > 0
                || status.getWorkingTreeStatus().all().length > 0;
        };
        SmartCommitAction.prototype.run = function (context) {
            if (!this.commitState.getCommitMessage()) {
                this.commitState.onEmptyCommitMessage();
                return winjs_base_1.TPromise.as(null);
            }
            var status = this.gitService.getModel().getStatus();
            return this.gitService.commit(this.commitState.getCommitMessage(), false, status.getIndexStatus().all().length === 0);
        };
        SmartCommitAction.ID = 'workbench.action.git.commitAll';
        SmartCommitAction.ALL = nls.localize(21, null);
        SmartCommitAction.STAGED = nls.localize(22, null);
        SmartCommitAction = __decorate([
            __param(1, git_1.IGitService),
            __param(2, message_1.IMessageService)
        ], SmartCommitAction);
        return SmartCommitAction;
    }(BaseCommitAction));
    exports.SmartCommitAction = SmartCommitAction;
    var PullAction = (function (_super) {
        __extends(PullAction, _super);
        function PullAction(id, label, gitService) {
            if (id === void 0) { id = PullAction.ID; }
            if (label === void 0) { label = PullAction.LABEL; }
            _super.call(this, id, label, 'git-action pull', gitService);
        }
        PullAction.prototype.isEnabled = function () {
            if (!_super.prototype.isEnabled.call(this)) {
                return false;
            }
            if (!this.gitService.isIdle()) {
                return false;
            }
            var model = this.gitService.getModel();
            var HEAD = model.getHEAD();
            if (!HEAD || !HEAD.name || !HEAD.upstream) {
                return false;
            }
            return true;
        };
        PullAction.prototype.run = function (context) {
            return this.pull();
        };
        PullAction.prototype.pull = function (rebase) {
            if (rebase === void 0) { rebase = false; }
            return this.gitService.pull(rebase).then(null, function (err) {
                if (err.gitErrorCode === git_1.GitErrorCodes.DirtyWorkTree) {
                    return winjs_base_1.Promise.wrapError(errors.create(nls.localize(24, null), { severity: severity_1.default.Warning }));
                }
                else if (err.gitErrorCode === git_1.GitErrorCodes.AuthenticationFailed) {
                    return winjs_base_1.Promise.wrapError(errors.create(nls.localize(25, null)));
                }
                return winjs_base_1.Promise.wrapError(err);
            });
        };
        PullAction.ID = 'workbench.action.git.pull';
        PullAction.LABEL = nls.localize(23, null);
        PullAction = __decorate([
            __param(2, git_1.IGitService)
        ], PullAction);
        return PullAction;
    }(GitAction));
    exports.PullAction = PullAction;
    var PullWithRebaseAction = (function (_super) {
        __extends(PullWithRebaseAction, _super);
        function PullWithRebaseAction(gitService) {
            _super.call(this, PullWithRebaseAction.ID, PullWithRebaseAction.LABEL, gitService);
        }
        PullWithRebaseAction.prototype.run = function (context) {
            return this.pull(true);
        };
        PullWithRebaseAction.ID = 'workbench.action.git.pull.rebase';
        PullWithRebaseAction.LABEL = nls.localize(26, null);
        PullWithRebaseAction = __decorate([
            __param(0, git_1.IGitService)
        ], PullWithRebaseAction);
        return PullWithRebaseAction;
    }(PullAction));
    exports.PullWithRebaseAction = PullWithRebaseAction;
    var PushAction = (function (_super) {
        __extends(PushAction, _super);
        function PushAction(id, label, gitService) {
            if (id === void 0) { id = PushAction.ID; }
            if (label === void 0) { label = PushAction.LABEL; }
            _super.call(this, id, label, 'git-action push', gitService);
        }
        PushAction.prototype.isEnabled = function () {
            if (!_super.prototype.isEnabled.call(this)) {
                return false;
            }
            if (!this.gitService.isIdle()) {
                return false;
            }
            var model = this.gitService.getModel();
            var HEAD = model.getHEAD();
            if (!HEAD || !HEAD.name || !HEAD.upstream) {
                return false;
            }
            if (!HEAD.ahead) {
                return false;
            }
            return true;
        };
        PushAction.prototype.run = function (context) {
            return this.gitService.push().then(null, function (err) {
                if (err.gitErrorCode === git_1.GitErrorCodes.AuthenticationFailed) {
                    return winjs_base_1.Promise.wrapError(errors.create(nls.localize(28, null)));
                }
                return winjs_base_1.Promise.wrapError(err);
            });
        };
        PushAction.ID = 'workbench.action.git.push';
        PushAction.LABEL = nls.localize(27, null);
        PushAction = __decorate([
            __param(2, git_1.IGitService)
        ], PushAction);
        return PushAction;
    }(GitAction));
    exports.PushAction = PushAction;
    var PublishAction = (function (_super) {
        __extends(PublishAction, _super);
        function PublishAction(id, label, gitService, quickOpenService, messageService) {
            if (id === void 0) { id = PublishAction.ID; }
            if (label === void 0) { label = PublishAction.LABEL; }
            _super.call(this, id, label, 'git-action publish', gitService);
            this.quickOpenService = quickOpenService;
            this.messageService = messageService;
        }
        PublishAction.prototype.isEnabled = function () {
            if (!_super.prototype.isEnabled.call(this)) {
                return false;
            }
            if (!this.gitService.isIdle()) {
                return false;
            }
            var model = this.gitService.getModel();
            if (model.getRemotes().length === 0) {
                return false;
            }
            var HEAD = model.getHEAD();
            if (!HEAD || !HEAD.name || HEAD.upstream) {
                return false;
            }
            return true;
        };
        PublishAction.prototype.run = function (context) {
            var _this = this;
            var model = this.gitService.getModel();
            var remotes = model.getRemotes();
            var branchName = model.getHEAD().name;
            var promise;
            if (remotes.length === 1) {
                var remoteName = remotes[0].name;
                var result = this.messageService.confirm({
                    message: nls.localize(30, null, branchName, remoteName),
                    primaryButton: nls.localize(31, null)
                });
                promise = winjs_base_1.TPromise.as(result ? remoteName : null);
            }
            else {
                promise = this.quickOpenService.pick(remotes.map(function (r) { return r.name; }), {
                    placeHolder: nls.localize(32, null, branchName)
                });
            }
            return promise
                .then(function (remote) { return remote && _this.gitService.push(remote, branchName, { setUpstream: true }); })
                .then(null, function (err) {
                if (err.gitErrorCode === git_1.GitErrorCodes.AuthenticationFailed) {
                    return winjs_base_1.Promise.wrapError(errors.create(nls.localize(33, null)));
                }
                return winjs_base_1.Promise.wrapError(err);
            });
        };
        PublishAction.ID = 'workbench.action.git.publish';
        PublishAction.LABEL = nls.localize(29, null);
        PublishAction = __decorate([
            __param(2, git_1.IGitService),
            __param(3, quickOpenService_1.IQuickOpenService),
            __param(4, message_1.IMessageService)
        ], PublishAction);
        return PublishAction;
    }(GitAction));
    exports.PublishAction = PublishAction;
    var BaseSyncAction = (function (_super) {
        __extends(BaseSyncAction, _super);
        function BaseSyncAction(id, label, className, gitService) {
            _super.call(this, id, label, className, gitService);
        }
        BaseSyncAction.prototype.isEnabled = function () {
            if (!_super.prototype.isEnabled.call(this)) {
                return false;
            }
            if (!this.gitService.isIdle()) {
                return false;
            }
            var model = this.gitService.getModel();
            var HEAD = model.getHEAD();
            if (!HEAD || !HEAD.name || !HEAD.upstream) {
                return false;
            }
            return true;
        };
        BaseSyncAction.prototype.run = function (context) {
            if (!this.enabled) {
                return winjs_base_1.TPromise.as(null);
            }
            return this.gitService.sync().then(null, function (err) {
                if (err.gitErrorCode === git_1.GitErrorCodes.AuthenticationFailed) {
                    return winjs_base_1.Promise.wrapError(errors.create(nls.localize(34, null)));
                }
                return winjs_base_1.Promise.wrapError(err);
            });
        };
        return BaseSyncAction;
    }(GitAction));
    exports.BaseSyncAction = BaseSyncAction;
    var SyncAction = (function (_super) {
        __extends(SyncAction, _super);
        function SyncAction(id, label, gitService) {
            _super.call(this, id, label, 'git-action sync', gitService);
        }
        SyncAction.ID = 'workbench.action.git.sync';
        SyncAction.LABEL = nls.localize(35, null);
        SyncAction = __decorate([
            __param(2, git_1.IGitService)
        ], SyncAction);
        return SyncAction;
    }(BaseSyncAction));
    exports.SyncAction = SyncAction;
    var LiveSyncAction = (function (_super) {
        __extends(LiveSyncAction, _super);
        function LiveSyncAction(gitService) {
            _super.call(this, LiveSyncAction.ID, nls.localize(36, null), LiveSyncAction.CLASS_NAME, gitService);
        }
        LiveSyncAction.prototype.onGitServiceChange = function () {
            _super.prototype.onGitServiceChange.call(this);
            if (this.gitService.getRunningOperations().some(function (op) {
                return op.id === git_1.ServiceOperations.SYNC ||
                    op.id === git_1.ServiceOperations.PULL ||
                    op.id === git_1.ServiceOperations.PUSH;
            })) {
                this.label = '';
                this.class = LiveSyncAction.CLASS_NAME_LOADING;
                this.tooltip = nls.localize(37, null);
            }
            else {
                this.class = LiveSyncAction.CLASS_NAME;
                var model = this.gitService.getModel();
                var HEAD = model.getHEAD();
                if (!HEAD) {
                    this.label = '';
                    this.tooltip = '';
                }
                else if (!HEAD.name) {
                    this.label = '';
                    this.tooltip = nls.localize(38, null);
                }
                else if (!HEAD.upstream) {
                    this.label = '';
                    this.tooltip = nls.localize(39, null, HEAD.name);
                }
                else if (!HEAD.ahead && !HEAD.behind) {
                    this.label = '';
                    this.tooltip = nls.localize(40, null, HEAD.name);
                }
                else {
                    this.label = strings.format('{0}↓ {1}↑', HEAD.behind, HEAD.ahead);
                    if (model.getStatus().getGroups().some(function (g) { return g.all().length > 0; })) {
                        this.tooltip = nls.localize(41, null);
                    }
                    else if (HEAD.behind === 1 && HEAD.ahead === 1) {
                        this.tooltip = nls.localize(42, null, HEAD.name, HEAD.behind, HEAD.ahead, HEAD.upstream);
                    }
                    else if (HEAD.behind === 1) {
                        this.tooltip = nls.localize(43, null, HEAD.name, HEAD.behind, HEAD.ahead, HEAD.upstream);
                    }
                    else if (HEAD.ahead === 1) {
                        this.tooltip = nls.localize(44, null, HEAD.name, HEAD.behind, HEAD.ahead, HEAD.upstream);
                    }
                    else {
                        this.tooltip = nls.localize(45, null, HEAD.name, HEAD.behind, HEAD.ahead, HEAD.upstream);
                    }
                }
            }
        };
        LiveSyncAction.ID = 'workbench.action.git.liveSync';
        LiveSyncAction.CLASS_NAME = 'git-action live-sync';
        LiveSyncAction.CLASS_NAME_LOADING = 'git-action live-sync loading';
        LiveSyncAction = __decorate([
            __param(0, git_1.IGitService)
        ], LiveSyncAction);
        return LiveSyncAction;
    }(BaseSyncAction));
    exports.LiveSyncAction = LiveSyncAction;
    var UndoLastCommitAction = (function (_super) {
        __extends(UndoLastCommitAction, _super);
        function UndoLastCommitAction(gitService) {
            _super.call(this, UndoLastCommitAction.ID, nls.localize(46, null), 'git-action undo-last-commit', gitService);
        }
        UndoLastCommitAction.prototype.run = function () {
            return this.gitService.reset('HEAD~');
        };
        UndoLastCommitAction.ID = 'workbench.action.git.undoLastCommit';
        UndoLastCommitAction = __decorate([
            __param(0, git_1.IGitService)
        ], UndoLastCommitAction);
        return UndoLastCommitAction;
    }(GitAction));
    exports.UndoLastCommitAction = UndoLastCommitAction;
    var StartGitCheckoutAction = (function (_super) {
        __extends(StartGitCheckoutAction, _super);
        function StartGitCheckoutAction(id, label, quickOpenService) {
            _super.call(this, id, label);
            this.quickOpenService = quickOpenService;
        }
        StartGitCheckoutAction.prototype.run = function (event) {
            this.quickOpenService.show('git checkout ');
            return winjs_base_1.TPromise.as(null);
        };
        StartGitCheckoutAction.ID = 'workbench.action.git.startGitCheckout';
        StartGitCheckoutAction.LABEL = nls.localize(47, null);
        StartGitCheckoutAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], StartGitCheckoutAction);
        return StartGitCheckoutAction;
    }(actions_1.Action));
    exports.StartGitCheckoutAction = StartGitCheckoutAction;
    var StartGitBranchAction = (function (_super) {
        __extends(StartGitBranchAction, _super);
        function StartGitBranchAction(id, label, quickOpenService) {
            _super.call(this, id, label);
            this.quickOpenService = quickOpenService;
        }
        StartGitBranchAction.prototype.run = function (event) {
            this.quickOpenService.show('git branch ');
            return winjs_base_1.TPromise.as(null);
        };
        StartGitBranchAction.ID = 'workbench.action.git.startGitBranch';
        StartGitBranchAction.LABEL = nls.localize(48, null);
        StartGitBranchAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], StartGitBranchAction);
        return StartGitBranchAction;
    }(actions_1.Action));
    exports.StartGitBranchAction = StartGitBranchAction;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/git/browser/gitWidgets", ["require", "exports", 'vs/nls!vs/workbench/parts/git/browser/gitWidgets', 'vs/base/common/strings', 'vs/base/common/async', 'vs/base/browser/dom', 'vs/base/common/lifecycle', 'vs/workbench/parts/git/common/git', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/platform/instantiation/common/instantiation', './gitActions', 'vs/base/common/severity', 'vs/platform/message/common/message'], function (require, exports, nls, strings, async_1, dom_1, lifecycle_1, git_1, quickOpenService_1, instantiation_1, gitActions_1, severity_1, message_1) {
    "use strict";
    var DisablementDelay = 500;
    var GitStatusbarItem = (function () {
        function GitStatusbarItem(instantiationService, gitService, quickOpenService, messageService) {
            this.messageService = messageService;
            this.instantiationService = instantiationService;
            this.gitService = gitService;
            this.quickOpenService = quickOpenService;
            this.disablementDelayer = new async_1.Delayer(DisablementDelay);
            this.syncAction = instantiationService.createInstance(gitActions_1.SyncAction, gitActions_1.SyncAction.ID, gitActions_1.SyncAction.LABEL);
            this.publishAction = instantiationService.createInstance(gitActions_1.PublishAction, gitActions_1.PublishAction.ID, gitActions_1.PublishAction.LABEL);
            this.toDispose = [
                this.syncAction,
                this.publishAction
            ];
            this.state = {
                serviceState: git_1.ServiceState.NotInitialized,
                isBusy: false,
                isSyncing: false,
                HEAD: null,
                remotes: [],
                ps1: ''
            };
        }
        GitStatusbarItem.prototype.render = function (container) {
            var _this = this;
            this.element = dom_1.append(container, dom_1.emmet('.git-statusbar-group'));
            this.branchElement = dom_1.append(this.element, dom_1.emmet('a'));
            this.publishElement = dom_1.append(this.element, dom_1.emmet('a.octicon.octicon-cloud-upload'));
            this.publishElement.title = nls.localize(0, null);
            this.publishElement.onclick = function () { return _this.onPublishClick(); };
            this.syncElement = dom_1.append(this.element, dom_1.emmet('a.git-statusbar-sync-item'));
            this.syncElement.title = nls.localize(1, null);
            this.syncElement.onclick = function () { return _this.onSyncClick(); };
            dom_1.append(this.syncElement, dom_1.emmet('span.octicon.octicon-sync'));
            this.syncLabelElement = dom_1.append(this.syncElement, dom_1.emmet('span.ahead-behind'));
            this.setState(this.state);
            this.toDispose.push(this.gitService.addBulkListener2(function () { return _this.onGitServiceChange(); }));
            return lifecycle_1.combinedDispose.apply(void 0, this.toDispose);
        };
        GitStatusbarItem.prototype.onGitServiceChange = function () {
            var model = this.gitService.getModel();
            this.setState({
                serviceState: this.gitService.getState(),
                isBusy: this.gitService.getRunningOperations().some(function (op) { return op.id === git_1.ServiceOperations.CHECKOUT || op.id === git_1.ServiceOperations.BRANCH; }),
                isSyncing: this.gitService.getRunningOperations().some(function (op) { return op.id === git_1.ServiceOperations.SYNC; }),
                HEAD: model.getHEAD(),
                remotes: model.getRemotes(),
                ps1: model.getPS1()
            });
        };
        GitStatusbarItem.prototype.setState = function (state) {
            var _this = this;
            this.state = state;
            var isGitDisabled = false;
            var className = 'git-statusbar-branch-item';
            var textContent;
            var aheadBehindLabel = '';
            var title = '';
            var onclick = null;
            if (state.serviceState !== git_1.ServiceState.OK) {
                isGitDisabled = true;
                className += ' disabled';
                title = nls.localize(2, null);
                textContent = '\u00a0';
            }
            else {
                var HEAD = state.HEAD;
                if (state.isBusy) {
                    className += ' busy';
                }
                else {
                    onclick = function () { return _this.onBranchClick(); };
                }
                if (!HEAD) {
                    textContent = state.ps1;
                }
                else if (!HEAD.name) {
                    textContent = state.ps1;
                    className += ' headless';
                }
                else if (!HEAD.commit || !HEAD.upstream || (!HEAD.ahead && !HEAD.behind)) {
                    textContent = state.ps1;
                }
                else {
                    textContent = state.ps1;
                    aheadBehindLabel = strings.format('{0}↓ {1}↑', HEAD.behind, HEAD.ahead);
                }
            }
            this.branchElement.className = className;
            this.branchElement.title = title;
            this.branchElement.textContent = textContent;
            this.branchElement.onclick = onclick;
            this.syncLabelElement.textContent = aheadBehindLabel;
            if (isGitDisabled) {
                dom_1.hide(this.branchElement);
                dom_1.hide(this.publishElement);
                dom_1.hide(this.syncElement);
            }
            else {
                dom_1.show(this.branchElement);
                if (state.HEAD && !!state.HEAD.upstream) {
                    dom_1.show(this.syncElement);
                    dom_1.toggleClass(this.syncElement, 'syncing', this.state.isSyncing);
                    dom_1.toggleClass(this.syncElement, 'empty', !aheadBehindLabel);
                    this.disablementDelayer.trigger(function () { return dom_1.toggleClass(_this.syncElement, 'disabled', !_this.syncAction.enabled); }, this.syncAction.enabled ? 0 : DisablementDelay);
                    dom_1.hide(this.publishElement);
                }
                else if (state.remotes.length > 0) {
                    dom_1.hide(this.syncElement);
                    dom_1.show(this.publishElement);
                    this.disablementDelayer.trigger(function () { return dom_1.toggleClass(_this.publishElement, 'disabled', !_this.publishAction.enabled); }, this.publishAction.enabled ? 0 : DisablementDelay);
                }
                else {
                    dom_1.hide(this.syncElement);
                    dom_1.hide(this.publishElement);
                }
            }
        };
        GitStatusbarItem.prototype.onBranchClick = function () {
            this.quickOpenService.show('git checkout ');
        };
        GitStatusbarItem.prototype.onPublishClick = function () {
            var _this = this;
            if (!this.publishAction.enabled) {
                return;
            }
            this.publishAction.run()
                .done(null, function (err) { return _this.messageService.show(severity_1.default.Error, err); });
        };
        GitStatusbarItem.prototype.onSyncClick = function () {
            var _this = this;
            if (!this.syncAction.enabled) {
                return;
            }
            this.syncAction.run()
                .done(null, function (err) { return _this.messageService.show(severity_1.default.Error, err); });
        };
        GitStatusbarItem = __decorate([
            __param(0, instantiation_1.IInstantiationService),
            __param(1, git_1.IGitService),
            __param(2, quickOpenService_1.IQuickOpenService),
            __param(3, message_1.IMessageService)
        ], GitStatusbarItem);
        return GitStatusbarItem;
    }());
    exports.GitStatusbarItem = GitStatusbarItem;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/git/browser/gitWorkbenchContributions", ["require", "exports", 'vs/nls!vs/workbench/parts/git/browser/gitWorkbenchContributions', 'vs/base/common/async', 'vs/base/common/errors', 'vs/base/common/paths', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/workbench/common/contributions', 'vs/workbench/parts/git/common/git', 'vs/workbench/common/events', 'vs/editor/common/editorCommon', 'vs/editor/browser/widget/codeEditorWidget', 'vs/workbench/browser/viewlet', 'vs/workbench/browser/parts/statusbar/statusbar', 'vs/platform/platform', 'vs/workbench/parts/git/browser/gitWidgets', 'vs/workbench/common/actionRegistry', 'vs/workbench/parts/git/browser/gitOutput', 'vs/workbench/parts/output/common/output', 'vs/platform/actions/common/actions', 'vs/editor/browser/editorBrowserExtensions', 'vs/platform/configuration/common/configurationRegistry', 'vs/workbench/browser/quickopen', 'vs/workbench/parts/git/browser/gitEditorContributions', 'vs/workbench/services/activity/common/activityService', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/platform/message/common/message', 'vs/platform/workspace/common/workspace', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/editor/common/editorService', 'vs/base/common/keyCodes', 'vs/editor/common/services/modelService', 'vs/editor/common/model/textModel', 'vs/editor/common/services/editorWorkerService', 'vs/css!./media/git.contribution'], function (require, exports, nls, async, errors, paths, lifecycle, winjs, ext, git, workbenchEvents, common, widget, viewlet, statusbar, platform, widgets, wbar, gitoutput, output, actions_1, editorBrowserExtensions_1, confregistry, quickopen, editorcontrib, activityService_1, event_1, instantiation_1, message_1, workspace_1, viewletService_1, editorService_1, keyCodes_1, modelService_1, textModel_1, editorWorkerService_1) {
    'use strict';
    var IGitService = git.IGitService;
    var StatusUpdater = (function () {
        function StatusUpdater(gitService, eventService, activityService, messageService) {
            var _this = this;
            this.gitService = gitService;
            this.eventService = eventService;
            this.activityService = activityService;
            this.messageService = messageService;
            this.progressBadgeDelayer = new async.Delayer(200);
            this.toDispose = [];
            this.toDispose.push(this.gitService.addBulkListener2(function (e) { return _this.onGitServiceChange(); }));
        }
        StatusUpdater.prototype.onGitServiceChange = function () {
            var _this = this;
            if (this.gitService.getState() !== git.ServiceState.OK) {
                this.progressBadgeDelayer.cancel();
                this.activityService.showActivity('workbench.view.git', null, 'git-viewlet-label');
            }
            else if (this.gitService.isIdle()) {
                this.showChangesBadge();
            }
            else {
                this.progressBadgeDelayer.trigger(function () {
                    _this.activityService.showActivity('workbench.view.git', new activityService_1.ProgressBadge(function () { return nls.localize(0, null); }), 'git-viewlet-label-progress');
                });
            }
        };
        StatusUpdater.prototype.showChangesBadge = function () {
            var count = this.gitService.getModel().getStatus().getGroups().map(function (g1) {
                return g1.all().length;
            }).reduce(function (a, b) { return a + b; }, 0);
            var badge = new activityService_1.NumberBadge(count, function (num) { return nls.localize(1, null, num); });
            this.progressBadgeDelayer.cancel();
            this.activityService.showActivity('workbench.view.git', badge, 'git-viewlet-label');
        };
        StatusUpdater.prototype.getId = function () {
            return StatusUpdater.ID;
        };
        StatusUpdater.prototype.dispose = function () {
            this.toDispose = lifecycle.disposeAll(this.toDispose);
        };
        StatusUpdater.ID = 'Monaco.IDE.UI.Viewlets.GitViewlet.Workbench.StatusUpdater';
        StatusUpdater = __decorate([
            __param(0, IGitService),
            __param(1, event_1.IEventService),
            __param(2, activityService_1.IActivityService),
            __param(3, message_1.IMessageService)
        ], StatusUpdater);
        return StatusUpdater;
    }());
    exports.StatusUpdater = StatusUpdater;
    var DirtyDiffModelDecorator = (function () {
        function DirtyDiffModelDecorator(model, path, modelService, editorWorkerService, editorService, contextService, gitService) {
            var _this = this;
            this.modelService = modelService;
            this.editorWorkerService = editorWorkerService;
            this.editorService = editorService;
            this.contextService = contextService;
            this.gitService = gitService;
            this.model = model;
            this._originalContentsURI = model.getAssociatedResource().withScheme(DirtyDiffModelDecorator.GIT_ORIGINAL_SCHEME);
            this.path = path;
            this.decorations = [];
            this.delayer = new async.ThrottledDelayer(500);
            this.diffDelayer = new async.ThrottledDelayer(200);
            this.toDispose = [];
            this.toDispose.push(model.addListener2(common.EventType.ModelContentChanged, function () { return _this.triggerDiff(); }));
            this.toDispose.push(this.gitService.addListener2(git.ServiceEvents.STATE_CHANGED, function () { return _this.onChanges(); }));
            this.toDispose.push(this.gitService.addListener2(git.ServiceEvents.OPERATION_END, function (e) {
                if (e.operation.id !== git.ServiceOperations.BACKGROUND_FETCH) {
                    _this.onChanges();
                }
            }));
            this.onChanges();
        }
        DirtyDiffModelDecorator.prototype.onChanges = function () {
            if (!this.gitService) {
                return;
            }
            if (this.gitService.getState() !== git.ServiceState.OK) {
                return;
            }
            // go through all interesting models
            this.trigger();
        };
        DirtyDiffModelDecorator.prototype.trigger = function () {
            var _this = this;
            this.delayer
                .trigger(function () { return _this.diffOriginalContents(); })
                .done(null, errors.onUnexpectedError);
        };
        DirtyDiffModelDecorator.prototype.diffOriginalContents = function () {
            var _this = this;
            return this.getOriginalContents()
                .then(function (contents) {
                if (!_this.model || _this.model.isDisposed()) {
                    return; // disposed
                }
                if (!contents) {
                    // untracked file
                    _this.modelService.destroyModel(_this._originalContentsURI);
                    return _this.triggerDiff();
                }
                var originalModel = _this.modelService.getModel(_this._originalContentsURI);
                if (originalModel) {
                    var contentsRawText = textModel_1.RawText.fromStringWithModelOptions(contents, originalModel);
                    // return early if nothing has changed
                    if (originalModel.equals(contentsRawText)) {
                        return winjs.TPromise.as(null);
                    }
                    // we already have the original contents
                    originalModel.setValueFromRawText(contentsRawText);
                }
                else {
                    // this is the first time we load the original contents
                    _this.modelService.createModel(contents, null, _this._originalContentsURI);
                }
                return _this.triggerDiff();
            });
        };
        DirtyDiffModelDecorator.prototype.getOriginalContents = function () {
            var gitModel = this.gitService.getModel();
            var treeish = gitModel.getStatus().find(this.path, git.StatusType.INDEX) ? '~' : 'HEAD';
            return this.gitService.buffer(this.path, treeish);
        };
        DirtyDiffModelDecorator.prototype.triggerDiff = function () {
            var _this = this;
            if (!this.diffDelayer) {
                return winjs.TPromise.as(null);
            }
            return this.diffDelayer.trigger(function () {
                if (!_this.model || _this.model.isDisposed()) {
                    return winjs.TPromise.as([]); // disposed
                }
                return _this.editorWorkerService.computeDirtyDiff(_this._originalContentsURI, _this.model.getAssociatedResource(), true);
            }).then(function (diff) {
                if (!_this.model || _this.model.isDisposed()) {
                    return; // disposed
                }
                return _this.decorations = _this.model.deltaDecorations(_this.decorations, DirtyDiffModelDecorator.changesToDecorations(diff || []));
            });
        };
        DirtyDiffModelDecorator.changesToDecorations = function (diff) {
            return diff.map(function (change) {
                var startLineNumber = change.modifiedStartLineNumber;
                var endLineNumber = change.modifiedEndLineNumber || startLineNumber;
                // Added
                if (change.originalEndLineNumber === 0) {
                    return {
                        range: {
                            startLineNumber: startLineNumber, startColumn: 1,
                            endLineNumber: endLineNumber, endColumn: 1
                        },
                        options: DirtyDiffModelDecorator.ADDED_DECORATION_OPTIONS
                    };
                }
                // Removed
                if (change.modifiedEndLineNumber === 0) {
                    return {
                        range: {
                            startLineNumber: startLineNumber, startColumn: 1,
                            endLineNumber: startLineNumber, endColumn: 1
                        },
                        options: DirtyDiffModelDecorator.DELETED_DECORATION_OPTIONS
                    };
                }
                // Modified
                return {
                    range: {
                        startLineNumber: startLineNumber, startColumn: 1,
                        endLineNumber: endLineNumber, endColumn: 1
                    },
                    options: DirtyDiffModelDecorator.MODIFIED_DECORATION_OPTIONS
                };
            });
        };
        DirtyDiffModelDecorator.prototype.dispose = function () {
            this.modelService.destroyModel(this._originalContentsURI);
            this.toDispose = lifecycle.disposeAll(this.toDispose);
            if (this.model && !this.model.isDisposed()) {
                this.model.deltaDecorations(this.decorations, []);
            }
            this.model = null;
            this.decorations = null;
            if (this.delayer) {
                this.delayer.cancel();
                this.delayer = null;
            }
            if (this.diffDelayer) {
                this.diffDelayer.cancel();
                this.diffDelayer = null;
            }
        };
        DirtyDiffModelDecorator.GIT_ORIGINAL_SCHEME = 'git-index';
        DirtyDiffModelDecorator.ID = 'Monaco.IDE.UI.Viewlets.GitViewlet.Editor.DirtyDiffDecorator';
        DirtyDiffModelDecorator.MODIFIED_DECORATION_OPTIONS = {
            linesDecorationsClassName: 'git-dirty-modified-diff-glyph',
            isWholeLine: true,
            overviewRuler: {
                color: 'rgba(0, 122, 204, 0.6)',
                darkColor: 'rgba(0, 122, 204, 0.6)',
                position: common.OverviewRulerLane.Left
            }
        };
        DirtyDiffModelDecorator.ADDED_DECORATION_OPTIONS = {
            linesDecorationsClassName: 'git-dirty-added-diff-glyph',
            isWholeLine: true,
            overviewRuler: {
                color: 'rgba(0, 122, 204, 0.6)',
                darkColor: 'rgba(0, 122, 204, 0.6)',
                position: common.OverviewRulerLane.Left
            }
        };
        DirtyDiffModelDecorator.DELETED_DECORATION_OPTIONS = {
            linesDecorationsClassName: 'git-dirty-deleted-diff-glyph',
            isWholeLine: true,
            overviewRuler: {
                color: 'rgba(0, 122, 204, 0.6)',
                darkColor: 'rgba(0, 122, 204, 0.6)',
                position: common.OverviewRulerLane.Left
            }
        };
        DirtyDiffModelDecorator = __decorate([
            __param(2, modelService_1.IModelService),
            __param(3, editorWorkerService_1.IEditorWorkerService),
            __param(4, editorService_1.IWorkbenchEditorService),
            __param(5, workspace_1.IWorkspaceContextService),
            __param(6, IGitService)
        ], DirtyDiffModelDecorator);
        return DirtyDiffModelDecorator;
    }());
    var DirtyDiffDecorator = (function () {
        function DirtyDiffDecorator(gitService, messageService, editorService, eventService, contextService, instantiationService) {
            var _this = this;
            this.gitService = gitService;
            this.messageService = messageService;
            this.editorService = editorService;
            this.eventService = eventService;
            this.contextService = contextService;
            this.instantiationService = instantiationService;
            this.models = [];
            this.decorators = Object.create(null);
            this.toDispose = [];
            this.toDispose.push(eventService.addListener2(workbenchEvents.EventType.EDITOR_INPUT_CHANGED, function () { return _this.onEditorInputChange(); }));
            this.toDispose.push(gitService.addListener2(git.ServiceEvents.DISPOSE, function () { return _this.dispose(); }));
        }
        DirtyDiffDecorator.prototype.getId = function () {
            return 'git.DirtyDiffModelDecorator';
        };
        DirtyDiffDecorator.prototype.onEditorInputChange = function () {
            // HACK: This is the best current way of figuring out whether to draw these decorations
            // or not. Needs context from the editor, to know whether it is a diff editor, in place editor
            // etc.
            var _this = this;
            var repositoryRoot = this.gitService.getModel().getRepositoryRoot();
            // If there is no repository root, just wait until that changes
            if (typeof repositoryRoot !== 'string') {
                this.gitService.addOneTimeListener(git.ServiceEvents.STATE_CHANGED, function () { return _this.onEditorInputChange(); });
                this.models.forEach(function (m) { return _this.onModelInvisible(m); });
                this.models = [];
                return;
            }
            var models = this.editorService.getVisibleEditors()
                .map(function (e) { return e.getControl(); })
                .filter(function (c) { return c instanceof widget.CodeEditorWidget; })
                .map(function (e) { return e.getModel(); })
                .filter(function (m, i, a) { return !!m && a.indexOf(m, i + 1) === -1; })
                .map(function (m) { return ({ model: m, resource: m.getAssociatedResource() }); })
                .filter(function (p) { return !!p.resource &&
                // and invalid resources
                (p.resource.scheme === 'file' && paths.isEqualOrParent(p.resource.fsPath, repositoryRoot)); })
                .map(function (p) { return ({ model: p.model, path: paths.normalize(paths.relative(repositoryRoot, p.resource.fsPath)) }); })
                .filter(function (p) { return !!p.path && p.path.indexOf('.git/') === -1; });
            var newModels = models.filter(function (p) { return _this.models.every(function (m) { return p.model !== m; }); });
            var oldModels = this.models.filter(function (m) { return models.every(function (p) { return p.model !== m; }); });
            newModels.forEach(function (p) { return _this.onModelVisible(p.model, p.path); });
            oldModels.forEach(function (m) { return _this.onModelInvisible(m); });
            this.models = models.map(function (p) { return p.model; });
        };
        DirtyDiffDecorator.prototype.onModelVisible = function (model, path) {
            this.decorators[model.id] = this.instantiationService.createInstance(DirtyDiffModelDecorator, model, path);
        };
        DirtyDiffDecorator.prototype.onModelInvisible = function (model) {
            this.decorators[model.id].dispose();
            delete this.decorators[model.id];
        };
        DirtyDiffDecorator.prototype.dispose = function () {
            var _this = this;
            this.toDispose = lifecycle.disposeAll(this.toDispose);
            this.models.forEach(function (m) { return _this.decorators[m.id].dispose(); });
            this.models = null;
            this.decorators = null;
        };
        DirtyDiffDecorator = __decorate([
            __param(0, IGitService),
            __param(1, message_1.IMessageService),
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, event_1.IEventService),
            __param(4, workspace_1.IWorkspaceContextService),
            __param(5, instantiation_1.IInstantiationService)
        ], DirtyDiffDecorator);
        return DirtyDiffDecorator;
    }());
    exports.DirtyDiffDecorator = DirtyDiffDecorator;
    exports.VIEWLET_ID = 'workbench.view.git';
    var OpenGitViewletAction = (function (_super) {
        __extends(OpenGitViewletAction, _super);
        function OpenGitViewletAction(id, label, viewletService, editorService) {
            _super.call(this, id, label, exports.VIEWLET_ID, viewletService, editorService);
        }
        OpenGitViewletAction.ID = exports.VIEWLET_ID;
        OpenGitViewletAction.LABEL = nls.localize(2, null);
        OpenGitViewletAction = __decorate([
            __param(2, viewletService_1.IViewletService),
            __param(3, editorService_1.IWorkbenchEditorService)
        ], OpenGitViewletAction);
        return OpenGitViewletAction;
    }(viewlet.ToggleViewletAction));
    function registerContributions() {
        // Register Statusbar item
        platform.Registry.as(statusbar.Extensions.Statusbar).registerStatusbarItem(new statusbar.StatusbarItemDescriptor(widgets.GitStatusbarItem, statusbar.StatusbarAlignment.LEFT, 100 /* High Priority */));
        // Register Output Channel
        var outputChannelRegistry = platform.Registry.as(output.Extensions.OutputChannels);
        outputChannelRegistry.registerChannel('Git');
        // Register Git Output
        platform.Registry.as(ext.Extensions.Workbench).registerWorkbenchContribution(gitoutput.GitOutput);
        // Register Viewlet
        platform.Registry.as(viewlet.Extensions.Viewlets).registerViewlet(new viewlet.ViewletDescriptor('vs/workbench/parts/git/browser/gitViewlet', 'GitViewlet', exports.VIEWLET_ID, nls.localize(3, null), 'git', 35));
        // Register Action to Open Viewlet
        platform.Registry.as(wbar.Extensions.WorkbenchActions).registerWorkbenchAction(new actions_1.SyncActionDescriptor(OpenGitViewletAction, OpenGitViewletAction.ID, OpenGitViewletAction.LABEL, {
            primary: null,
            win: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_G },
            linux: { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_G },
            mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_G }
        }), nls.localize(4, null));
        // Register MergeDecorator
        editorBrowserExtensions_1.EditorBrowserRegistry.registerEditorContribution(editorcontrib.MergeDecorator);
        // Register StatusUpdater
        platform.Registry.as(ext.Extensions.Workbench).registerWorkbenchContribution(StatusUpdater);
        // Register DirtyDiffDecorator
        platform.Registry.as(ext.Extensions.Workbench).registerWorkbenchContribution(DirtyDiffDecorator);
        // Register Quick Open for git
        platform.Registry.as(quickopen.Extensions.Quickopen).registerQuickOpenHandler(new quickopen.QuickOpenHandlerDescriptor('vs/workbench/parts/git/browser/gitQuickOpen', 'CommandQuickOpenHandler', 'git ', nls.localize(5, null)));
        // Register configuration
        var configurationRegistry = platform.Registry.as(confregistry.Extensions.Configuration);
        configurationRegistry.registerConfiguration({
            id: 'git',
            order: 10,
            title: nls.localize(6, null),
            type: 'object',
            properties: {
                'git.enabled': {
                    type: 'boolean',
                    description: nls.localize(7, null),
                    default: true
                },
                'git.path': {
                    type: ['string', 'null'],
                    description: nls.localize(8, null),
                    default: null
                },
                'git.autofetch': {
                    type: 'boolean',
                    description: nls.localize(9, null),
                    default: true
                }
            }
        });
    }
    exports.registerContributions = registerContributions;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/git/browser/gitActions.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/git/browser/gitActions.contribution', 'vs/base/common/lifecycle', 'vs/platform/platform', 'vs/workbench/browser/actionBarRegistry', 'vs/base/common/winjs.base', 'vs/editor/common/editorCommon', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor', 'vs/workbench/browser/parts/editor/textDiffEditor', 'vs/workbench/browser/parts/editor/textEditor', 'vs/workbench/parts/files/common/files', 'vs/workbench/parts/git/browser/gitWorkbenchContributions', 'vs/workbench/parts/git/common/git', 'vs/workbench/parts/git/browser/gitEditorInputs', 'vs/workbench/parts/git/common/stageRanges', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/part/common/partService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/files/common/files', 'vs/platform/instantiation/common/instantiation', 'vs/workbench/common/actionRegistry', 'vs/platform/actions/common/actions', './gitActions', 'vs/base/common/paths', 'vs/base/common/uri'], function (require, exports, nls, lifecycle, platform, abr, winjs_base_1, editorcommon, baseeditor, WorkbenchEditorCommon, tdeditor, teditor, filesCommon, gitcontrib, git_1, gitei, stageranges, editorService_1, viewletService_1, partService_1, contextService_1, files_1, instantiation_1, wbar, actions_1, gitActions_1, paths, uri_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function getStatus(gitService, contextService, input) {
        var model = gitService.getModel();
        var repositoryRoot = model.getRepositoryRoot();
        var statusModel = model.getStatus();
        var repositoryRelativePath = paths.normalize(paths.relative(repositoryRoot, input.getResource().fsPath));
        return statusModel.getWorkingTreeStatus().find(repositoryRelativePath) ||
            statusModel.getIndexStatus().find(repositoryRelativePath);
    }
    var OpenInDiffAction = (function (_super) {
        __extends(OpenInDiffAction, _super);
        function OpenInDiffAction(editorService, gitService, viewletService, partService, contextService) {
            var _this = this;
            _super.call(this, OpenInDiffAction.ID, OpenInDiffAction.Label);
            this.class = 'git-action open-in-diff';
            this.gitService = gitService;
            this.viewletService = viewletService;
            this.editorService = editorService;
            this.partService = partService;
            this.contextService = contextService;
            this.toDispose = [this.gitService.addBulkListener2(function () { return _this.onGitStateChanged(); })];
            this.enabled = this.isEnabled();
        }
        OpenInDiffAction.prototype.isEnabled = function () {
            if (!_super.prototype.isEnabled.call(this)) {
                return false;
            }
            if (!(typeof this.gitService.getModel().getRepositoryRoot() === 'string')) {
                return false;
            }
            var status = this.getStatus();
            return status && (status.getStatus() === git_1.Status.MODIFIED ||
                status.getStatus() === git_1.Status.INDEX_MODIFIED ||
                status.getStatus() === git_1.Status.INDEX_RENAMED);
        };
        OpenInDiffAction.prototype.onGitStateChanged = function () {
            if (this.gitService.isIdle()) {
                this.enabled = this.isEnabled();
            }
        };
        OpenInDiffAction.prototype.getStatus = function () {
            return getStatus(this.gitService, this.contextService, this.input);
        };
        OpenInDiffAction.prototype.run = function (event) {
            var _this = this;
            var sideBySide = !!(event && (event.ctrlKey || event.metaKey));
            var editor = this.editorService.getActiveEditor().getControl();
            var viewState = editor ? editor.saveViewState() : null;
            return this.gitService.getInput(this.getStatus()).then(function (input) {
                var promise = winjs_base_1.TPromise.as(null);
                if (_this.partService.isVisible(partService_1.Parts.SIDEBAR_PART)) {
                    promise = _this.viewletService.openViewlet(gitcontrib.VIEWLET_ID, false);
                }
                return promise.then(function () {
                    var options = new WorkbenchEditorCommon.TextDiffEditorOptions();
                    options.forceOpen = true;
                    options.autoRevealFirstChange = false;
                    return _this.editorService.openEditor(input, options, sideBySide).then(function (editor) {
                        if (viewState) {
                            var codeEditor = _this.editorService.getActiveEditor().getControl();
                            codeEditor.restoreViewState({
                                original: {},
                                modified: viewState
                            });
                        }
                    });
                });
            });
        };
        OpenInDiffAction.prototype.dispose = function () {
            this.toDispose = lifecycle.disposeAll(this.toDispose);
        };
        OpenInDiffAction.ID = 'workbench.action.git.openInDiff';
        OpenInDiffAction.Label = nls.localize(0, null);
        OpenInDiffAction = __decorate([
            __param(0, editorService_1.IWorkbenchEditorService),
            __param(1, git_1.IGitService),
            __param(2, viewletService_1.IViewletService),
            __param(3, partService_1.IPartService),
            __param(4, contextService_1.IWorkspaceContextService)
        ], OpenInDiffAction);
        return OpenInDiffAction;
    }(baseeditor.EditorInputAction));
    var OpenInEditorAction = (function (_super) {
        __extends(OpenInEditorAction, _super);
        function OpenInEditorAction(fileService, editorService, gitService, viewletService, partService, contextService) {
            _super.call(this, OpenInEditorAction.ID, OpenInEditorAction.LABEL);
            this.class = 'git-action open-in-editor';
            this.gitService = gitService;
            this.fileService = fileService;
            this.viewletService = viewletService;
            this.editorService = editorService;
            this.partService = partService;
            this.contextService = contextService;
            this.enabled = this.isEnabled();
        }
        OpenInEditorAction.prototype.isEnabled = function () {
            if (!_super.prototype.isEnabled.call(this)) {
                return false;
            }
            if (!(typeof this.gitService.getModel().getRepositoryRoot() === 'string')) {
                return false;
            }
            var status = this.input.getFileStatus();
            if (OpenInEditorAction.DELETED_STATES.indexOf(status.getStatus()) > -1) {
                return false;
            }
            return true;
        };
        OpenInEditorAction.prototype.run = function (event) {
            var _this = this;
            var model = this.gitService.getModel();
            var resource = uri_1.default.file(paths.join(model.getRepositoryRoot(), this.getRepositoryRelativePath()));
            var sideBySide = !!(event && (event.ctrlKey || event.metaKey));
            var modifiedViewState = this.saveTextViewState();
            return this.fileService.resolveFile(resource).then(function (stat) {
                return _this.editorService.openEditor({
                    resource: stat.resource,
                    mime: stat.mime,
                    options: {
                        forceOpen: true
                    }
                }, sideBySide).then(function (editor) {
                    _this.restoreTextViewState(modifiedViewState);
                    if (_this.partService.isVisible(partService_1.Parts.SIDEBAR_PART)) {
                        return _this.viewletService.openViewlet(filesCommon.VIEWLET_ID, false);
                    }
                });
            });
        };
        OpenInEditorAction.prototype.saveTextViewState = function () {
            var textEditor = this.getTextEditor();
            if (textEditor) {
                return textEditor.saveViewState();
            }
            return null;
        };
        OpenInEditorAction.prototype.restoreTextViewState = function (state) {
            var textEditor = this.getTextEditor();
            if (textEditor) {
                return textEditor.restoreViewState(state);
            }
        };
        OpenInEditorAction.prototype.getTextEditor = function () {
            var editor = this.editorService.getActiveEditor();
            if (editor instanceof tdeditor.TextDiffEditor) {
                return editor.getControl().getModifiedEditor();
            }
            else if (editor instanceof teditor.BaseTextEditor) {
                return editor.getControl();
            }
            return null;
        };
        OpenInEditorAction.prototype.getRepositoryRelativePath = function () {
            var status = this.input.getFileStatus();
            if (status.getStatus() === git_1.Status.INDEX_RENAMED) {
                return status.getRename();
            }
            else {
                var indexStatus = this.gitService.getModel().getStatus().find(status.getPath(), git_1.StatusType.INDEX);
                if (indexStatus && indexStatus.getStatus() === git_1.Status.INDEX_RENAMED) {
                    return indexStatus.getRename();
                }
                else {
                    return status.getPath();
                }
            }
        };
        OpenInEditorAction.DELETED_STATES = [git_1.Status.BOTH_DELETED, git_1.Status.DELETED, git_1.Status.DELETED_BY_US, git_1.Status.INDEX_DELETED];
        OpenInEditorAction.ID = 'workbench.action.git.openInEditor';
        OpenInEditorAction.LABEL = nls.localize(1, null);
        OpenInEditorAction = __decorate([
            __param(0, files_1.IFileService),
            __param(1, editorService_1.IWorkbenchEditorService),
            __param(2, git_1.IGitService),
            __param(3, viewletService_1.IViewletService),
            __param(4, partService_1.IPartService),
            __param(5, contextService_1.IWorkspaceContextService)
        ], OpenInEditorAction);
        return OpenInEditorAction;
    }(baseeditor.EditorInputAction));
    var StageRangesAction = (function (_super) {
        __extends(StageRangesAction, _super);
        function StageRangesAction(editor, gitService, editorService) {
            _super.call(this, 'workbench.action.git.stageRanges', nls.localize(2, null));
            this.editorService = editorService;
            this.gitService = gitService;
            this.editor = editor.getControl();
            this.editor.addListener(editorcommon.EventType.CursorSelectionChanged, this.updateEnablement.bind(this));
            this.editor.addListener(editorcommon.EventType.DiffUpdated, this.updateEnablement.bind(this));
            this.class = 'git-action stage-ranges';
        }
        StageRangesAction.prototype.isEnabled = function () {
            if (!_super.prototype.isEnabled.call(this)) {
                return false;
            }
            if (!this.gitService || !this.editorService) {
                return false;
            }
            var changes = this.editor.getLineChanges();
            var selections = this.editor.getSelections();
            if (!changes || !selections || selections.length === 0) {
                return false;
            }
            return stageranges.getSelectedChanges(changes, selections).length > 0;
        };
        StageRangesAction.prototype.run = function () {
            var _this = this;
            var result = stageranges.stageRanges(this.editor);
            var status = this.input.getFileStatus();
            var path = status.getPath();
            var viewState = this.editor.saveViewState();
            return this.gitService.stage(status.getPath(), result).then(function () {
                var statusModel = _this.gitService.getModel().getStatus();
                status = statusModel.getWorkingTreeStatus().find(path) || statusModel.getIndexStatus().find(path);
                if (status) {
                    return _this.gitService.getInput(status).then(function (input) {
                        var options = new WorkbenchEditorCommon.TextDiffEditorOptions();
                        options.forceOpen = true;
                        options.autoRevealFirstChange = false;
                        return _this.editorService.openEditor(input, options, _this.position).then(function () {
                            _this.editor.restoreViewState(viewState);
                        });
                    });
                }
            });
        };
        StageRangesAction.prototype.updateEnablement = function () {
            this.enabled = this.isEnabled();
        };
        StageRangesAction = __decorate([
            __param(1, git_1.IGitService),
            __param(2, editorService_1.IWorkbenchEditorService)
        ], StageRangesAction);
        return StageRangesAction;
    }(baseeditor.EditorInputAction));
    exports.StageRangesAction = StageRangesAction;
    var FileEditorActionContributor = (function (_super) {
        __extends(FileEditorActionContributor, _super);
        function FileEditorActionContributor(instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
        }
        FileEditorActionContributor.prototype.hasActionsForEditorInput = function (context) {
            return context.input instanceof filesCommon.FileEditorInput;
        };
        FileEditorActionContributor.prototype.getActionsForEditorInput = function (context) {
            return [this.instantiationService.createInstance(OpenInDiffAction)];
        };
        FileEditorActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], FileEditorActionContributor);
        return FileEditorActionContributor;
    }(baseeditor.EditorInputActionContributor));
    var GitEditorActionContributor = (function (_super) {
        __extends(GitEditorActionContributor, _super);
        function GitEditorActionContributor(instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
        }
        GitEditorActionContributor.prototype.hasActionsForEditorInput = function (context) {
            return gitei.isGitEditorInput(context.input);
        };
        GitEditorActionContributor.prototype.getActionsForEditorInput = function (context) {
            return [this.instantiationService.createInstance(OpenInEditorAction)];
        };
        GitEditorActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], GitEditorActionContributor);
        return GitEditorActionContributor;
    }(baseeditor.EditorInputActionContributor));
    var GitWorkingTreeDiffEditorActionContributor = (function (_super) {
        __extends(GitWorkingTreeDiffEditorActionContributor, _super);
        function GitWorkingTreeDiffEditorActionContributor(instantiationService) {
            _super.call(this);
            this.instantiationService = instantiationService;
        }
        GitWorkingTreeDiffEditorActionContributor.prototype.hasSecondaryActionsForEditorInput = function (context) {
            return (context.input instanceof gitei.GitWorkingTreeDiffEditorInput && context.editor instanceof tdeditor.TextDiffEditor);
        };
        GitWorkingTreeDiffEditorActionContributor.prototype.getSecondaryActionsForEditorInput = function (context) {
            return [this.instantiationService.createInstance(StageRangesAction, context.editor)];
        };
        GitWorkingTreeDiffEditorActionContributor = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], GitWorkingTreeDiffEditorActionContributor);
        return GitWorkingTreeDiffEditorActionContributor;
    }(baseeditor.EditorInputActionContributor));
    var GlobalOpenChangeAction = (function (_super) {
        __extends(GlobalOpenChangeAction, _super);
        function GlobalOpenChangeAction(id, label, editorService, gitService, contextService, viewletService, partService) {
            _super.call(this, editorService, gitService);
            this.contextService = contextService;
            this.viewletService = viewletService;
            this.partService = partService;
        }
        GlobalOpenChangeAction.prototype.getInput = function () {
            return WorkbenchEditorCommon.asFileEditorInput(this.editorService.getActiveEditorInput());
        };
        GlobalOpenChangeAction.prototype.run = function (context) {
            var _this = this;
            var input = this.getInput();
            if (!input) {
                return winjs_base_1.TPromise.as(null);
            }
            var status = getStatus(this.gitService, this.contextService, input);
            if (!status) {
                return winjs_base_1.TPromise.as(null);
            }
            var sideBySide = !!(context && (context.ctrlKey || context.metaKey));
            var editor = this.editorService.getActiveEditor().getControl();
            var viewState = editor ? editor.saveViewState() : null;
            return this.gitService.getInput(status).then(function (input) {
                var promise = winjs_base_1.TPromise.as(null);
                if (_this.partService.isVisible(partService_1.Parts.SIDEBAR_PART)) {
                    promise = _this.viewletService.openViewlet(gitcontrib.VIEWLET_ID, false);
                }
                return promise.then(function () {
                    var options = new WorkbenchEditorCommon.TextDiffEditorOptions();
                    options.forceOpen = true;
                    options.autoRevealFirstChange = false;
                    return _this.editorService.openEditor(input, options, sideBySide).then(function (editor) {
                        if (viewState) {
                            var codeEditor = _this.editorService.getActiveEditor().getControl();
                            codeEditor.restoreViewState({
                                original: {},
                                modified: viewState
                            });
                        }
                    });
                });
            });
        };
        GlobalOpenChangeAction.ID = 'workbench.action.git.globalOpenChange';
        GlobalOpenChangeAction.LABEL = nls.localize(3, null);
        GlobalOpenChangeAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, git_1.IGitService),
            __param(4, contextService_1.IWorkspaceContextService),
            __param(5, viewletService_1.IViewletService),
            __param(6, partService_1.IPartService)
        ], GlobalOpenChangeAction);
        return GlobalOpenChangeAction;
    }(gitActions_1.OpenChangeAction));
    var GlobalOpenInEditorAction = (function (_super) {
        __extends(GlobalOpenInEditorAction, _super);
        function GlobalOpenInEditorAction(id, label, editorService, fileService, gitService, contextService) {
            if (id === void 0) { id = GlobalOpenInEditorAction.ID; }
            if (label === void 0) { label = GlobalOpenInEditorAction.LABEL; }
            _super.call(this, editorService, fileService, gitService, contextService);
        }
        GlobalOpenInEditorAction.prototype.run = function (event) {
            var input = WorkbenchEditorCommon.asFileEditorInput(this.editorService.getActiveEditorInput(), true);
            if (!input) {
                return winjs_base_1.TPromise.as(null);
            }
            var status = getStatus(this.gitService, this.contextService, input);
            if (!status) {
                return winjs_base_1.TPromise.as(null);
            }
            return _super.prototype.run.call(this, status);
        };
        GlobalOpenInEditorAction.ID = 'workbench.action.git.globalOpenFile';
        GlobalOpenInEditorAction.LABEL = nls.localize(4, null);
        GlobalOpenInEditorAction = __decorate([
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, files_1.IFileService),
            __param(4, git_1.IGitService),
            __param(5, contextService_1.IWorkspaceContextService)
        ], GlobalOpenInEditorAction);
        return GlobalOpenInEditorAction;
    }(gitActions_1.OpenFileAction));
    var actionBarRegistry = platform.Registry.as(abr.Extensions.Actionbar);
    actionBarRegistry.registerActionBarContributor(abr.Scope.EDITOR, FileEditorActionContributor);
    actionBarRegistry.registerActionBarContributor(abr.Scope.EDITOR, GitEditorActionContributor);
    actionBarRegistry.registerActionBarContributor(abr.Scope.EDITOR, GitWorkingTreeDiffEditorActionContributor);
    var workbenchActionRegistry = platform.Registry.as(wbar.Extensions.WorkbenchActions);
    // Register Actions
    var category = nls.localize(5, null);
    workbenchActionRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(GlobalOpenChangeAction, GlobalOpenChangeAction.ID, GlobalOpenChangeAction.LABEL), category);
    workbenchActionRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(GlobalOpenInEditorAction, GlobalOpenInEditorAction.ID, GlobalOpenInEditorAction.LABEL), category);
    workbenchActionRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(gitActions_1.PullAction, gitActions_1.PullAction.ID, gitActions_1.PullAction.LABEL), category);
    workbenchActionRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(gitActions_1.PushAction, gitActions_1.PushAction.ID, gitActions_1.PushAction.LABEL), category);
    workbenchActionRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(gitActions_1.SyncAction, gitActions_1.SyncAction.ID, gitActions_1.SyncAction.LABEL), category);
    workbenchActionRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(gitActions_1.PublishAction, gitActions_1.PublishAction.ID, gitActions_1.PublishAction.LABEL), category);
    workbenchActionRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(gitActions_1.StartGitBranchAction, gitActions_1.StartGitBranchAction.ID, gitActions_1.StartGitBranchAction.LABEL), category);
    workbenchActionRegistry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(gitActions_1.StartGitCheckoutAction, gitActions_1.StartGitCheckoutAction.ID, gitActions_1.StartGitCheckoutAction.LABEL), category);
});

define("vs/workbench/parts/git/electron-browser/git.contribution", ["require", "exports", 'vs/workbench/parts/git/browser/gitWorkbenchContributions', 'vs/workbench/parts/git/electron-browser/electronGitService', 'vs/workbench/parts/git/common/git', 'vs/platform/instantiation/common/extensions'], function (require, exports, gitWorkbenchContributions_1, electronGitService_1, git_1, extensions_1) {
    "use strict";
    gitWorkbenchContributions_1.registerContributions();
    // Register Service
    extensions_1.registerSingleton(git_1.IGitService, electronGitService_1.ElectronGitService);
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/markdown/browser/markdownExtension", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/types', 'vs/base/common/events', 'vs/platform/files/common/files', 'vs/base/common/paths', 'vs/editor/common/editorCommon', 'vs/workbench/common/constants', 'vs/platform/theme/common/themes', 'vs/workbench/browser/parts/editor/iframeEditor', 'vs/workbench/parts/markdown/common/markdownEditorInput', 'vs/workbench/common/events', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/storage/common/storage', 'vs/platform/configuration/common/configuration', 'vs/editor/common/services/modelService', 'vs/platform/event/common/event', 'vs/platform/instantiation/common/instantiation', 'vs/editor/common/services/modeService'], function (require, exports, errors, types, events_1, files_1, paths, editorCommon_1, constants_1, themes, iframeEditor_1, markdownEditorInput_1, events_2, editorService_1, contextService_1, storage_1, configuration_1, modelService_1, event_1, instantiation_1, modeService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // This extension tracks markdown files for changes to update markdown editors and inputs accordingly.
    var MarkdownFileTracker = (function () {
        function MarkdownFileTracker(modeService, eventService, editorService, configurationService, storageService, contextService, modelService, instantiationService) {
            this.modeService = modeService;
            this.eventService = eventService;
            this.editorService = editorService;
            this.configurationService = configurationService;
            this.storageService = storageService;
            this.contextService = contextService;
            this.modelService = modelService;
            this.instantiationService = instantiationService;
            this.markdownConfigurationPaths = [];
            this.hasModelListenerOnResourcePath = Object.create(null);
            this.configureMode(this.storageService.get(constants_1.Preferences.THEME, storage_1.StorageScope.GLOBAL));
            this.registerListeners();
        }
        MarkdownFileTracker.prototype.registerListeners = function () {
            var _this = this;
            this.fileChangeListener = this.eventService.addListener(files_1.EventType.FILE_CHANGES, function (e) { return _this.onFileChanges(e); });
            this.configFileChangeListener = this.configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) { return _this.onConfigFileChange(e); });
            // reload markdown editors when their resources change
            this.editorInputChangeListener = this.eventService.addListener(events_2.EventType.EDITOR_INPUT_CHANGED, function (e) { return _this.onEditorInputChanged(e); });
            // initially read the config for CSS styles in preview
            this.configurationService.loadConfiguration().done(function (config) {
                _this.readMarkdownConfiguration(config);
            }, errors.onUnexpectedError);
            // listen to theme changes
            this.themeChangeListener = this.storageService.addListener(storage_1.StorageEventType.STORAGE, function (e) {
                if (e.key === constants_1.Preferences.THEME) {
                    _this.configureMode(e.newValue);
                    _this.reloadMarkdownEditors(true);
                }
            });
        };
        MarkdownFileTracker.prototype.onEditorInputChanged = function (e) {
            var _this = this;
            var input = e.editorInput;
            if (input instanceof markdownEditorInput_1.MarkdownEditorInput) {
                var markdownResource_1 = input.getResource();
                var editorModel = this.modelService.getModel(markdownResource_1);
                if (editorModel && !this.hasModelListenerOnResourcePath[markdownResource_1.toString()]) {
                    var toUnbind_1 = [];
                    var unbind_1 = function () {
                        while (toUnbind_1.length) {
                            toUnbind_1.pop()();
                        }
                        _this.hasModelListenerOnResourcePath[markdownResource_1.toString()] = false;
                    };
                    // Listen on changes to the underlying resource of the markdown preview
                    toUnbind_1.push(editorModel.addListener(editorCommon_1.EventType.ModelContentChanged, function (modelEvent) {
                        if (_this.reloadTimeout) {
                            window.clearTimeout(_this.reloadTimeout);
                        }
                        _this.reloadTimeout = setTimeout(function () {
                            if (!_this.reloadMarkdownEditors(false, markdownResource_1)) {
                                unbind_1();
                            }
                        }, MarkdownFileTracker.RELOAD_MARKDOWN_DELAY);
                    }));
                    // Mark as being listened
                    this.hasModelListenerOnResourcePath[markdownResource_1.toString()] = true;
                    // Unbind when input or model gets disposed
                    toUnbind_1.push(input.addListener(events_1.EventType.DISPOSE, unbind_1));
                    toUnbind_1.push(editorModel.addListener(editorCommon_1.EventType.ModelDispose, unbind_1));
                }
            }
        };
        MarkdownFileTracker.prototype.configureMode = function (theme) {
            if (theme) {
                var baseTheme = themes.getBaseThemeId(theme);
                this.modeService.configureMode('text/x-web-markdown', { theme: baseTheme });
            }
        };
        MarkdownFileTracker.prototype.getId = function () {
            return 'vs.markdown.filetracker';
        };
        MarkdownFileTracker.prototype.onConfigFileChange = function (e) {
            // reload markdown editors if styles change
            if (this.readMarkdownConfiguration(e.config)) {
                this.reloadMarkdownEditors(true);
            }
        };
        MarkdownFileTracker.prototype.readMarkdownConfiguration = function (languageConfiguration) {
            var oldMarkdownConfigurationThumbprint = this.markdownConfigurationThumbprint;
            var newMarkdownConfigurationThumbprint;
            // Reset old
            this.markdownConfigurationThumbprint = null;
            this.markdownConfigurationPaths = [];
            if (languageConfiguration) {
                var markdownConfiguration = languageConfiguration.markdown;
                if (markdownConfiguration && types.isArray(markdownConfiguration.styles)) {
                    newMarkdownConfigurationThumbprint = markdownConfiguration.styles.join('');
                    var styles = markdownConfiguration.styles.map(function (style) { return paths.makeAbsolute(paths.normalize(style)); });
                    this.markdownConfigurationPaths = styles;
                }
            }
            // Remember as current
            this.markdownConfigurationThumbprint = newMarkdownConfigurationThumbprint;
            return (oldMarkdownConfigurationThumbprint !== newMarkdownConfigurationThumbprint);
        };
        MarkdownFileTracker.prototype.onFileChanges = function (e) {
            var _this = this;
            // If any of the markdown CSS styles have updated, reload all markdown editors
            if (this.markdownConfigurationPaths.length && e.containsAny(this.markdownConfigurationPaths.map(function (p) { return _this.contextService.toResource(p); }), files_1.FileChangeType.UPDATED)) {
                this.reloadMarkdownEditors(true);
            }
        };
        MarkdownFileTracker.prototype.reloadMarkdownEditors = function (clearIFrame, resource) {
            var didReload = false;
            var editors = this.editorService.getVisibleEditors();
            editors.forEach(function (editor) {
                // Only applicable to markdown editor inputs in iframe editors
                var input = editor.input;
                if (input instanceof markdownEditorInput_1.MarkdownEditorInput && editor instanceof iframeEditor_1.IFrameEditor) {
                    if (!resource || resource.toString() === input.getResource().toString()) {
                        editor.reload(clearIFrame);
                        didReload = true;
                    }
                }
            });
            return didReload;
        };
        MarkdownFileTracker.prototype.dispose = function () {
            if (this.fileChangeListener) {
                this.fileChangeListener();
                this.fileChangeListener = null;
            }
            if (this.configFileChangeListener) {
                this.configFileChangeListener();
                this.configFileChangeListener = null;
            }
            if (this.editorInputChangeListener) {
                this.editorInputChangeListener();
                this.editorInputChangeListener = null;
            }
        };
        MarkdownFileTracker.RELOAD_MARKDOWN_DELAY = 300; // delay before reloading markdown preview after user typing
        MarkdownFileTracker = __decorate([
            __param(0, modeService_1.IModeService),
            __param(1, event_1.IEventService),
            __param(2, editorService_1.IWorkbenchEditorService),
            __param(3, configuration_1.IConfigurationService),
            __param(4, storage_1.IStorageService),
            __param(5, contextService_1.IWorkspaceContextService),
            __param(6, modelService_1.IModelService),
            __param(7, instantiation_1.IInstantiationService)
        ], MarkdownFileTracker);
        return MarkdownFileTracker;
    }());
    exports.MarkdownFileTracker = MarkdownFileTracker;
});

define("vs/workbench/parts/markdown/browser/markdown.contribution", ["require", "exports", 'vs/platform/platform', 'vs/base/common/uri', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/parts/markdown/common/markdownEditorInput', 'vs/workbench/parts/markdown/browser/markdownExtension', 'vs/workbench/common/contributions'], function (require, exports, platform_1, uri_1, baseEditor_1, markdownEditorInput_1, markdownExtension_1, contributions_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // Register Editor Input Factory
    var MarkdownInputFactory = (function () {
        function MarkdownInputFactory() {
        }
        MarkdownInputFactory.prototype.serialize = function (editorInput) {
            var markdownInput = editorInput;
            return markdownInput.getResource().toString();
        };
        MarkdownInputFactory.prototype.deserialize = function (instantiationService, resourceRaw) {
            return instantiationService.createInstance(markdownEditorInput_1.MarkdownEditorInput, uri_1.default.parse(resourceRaw), void 0, void 0);
        };
        return MarkdownInputFactory;
    }());
    platform_1.Registry.as(baseEditor_1.Extensions.Editors).registerEditorInputFactory(markdownEditorInput_1.MarkdownEditorInput.ID, MarkdownInputFactory);
    // Register Markdown File Tracker
    platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(markdownExtension_1.MarkdownFileTracker);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/quickopen/browser/gotoLineHandler", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/quickopen/browser/gotoLineHandler', 'vs/base/common/types', 'vs/base/common/errors', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/workbench/browser/quickopen', 'vs/workbench/browser/actions/quickOpenAction', 'vs/workbench/common/editor', 'vs/workbench/browser/parts/editor/textEditor', 'vs/editor/common/editorCommon', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/quickopen/common/quickOpenService'], function (require, exports, winjs_base_1, nls, types, errors, quickOpen_1, quickOpenModel_1, quickopen_1, quickOpenAction_1, editor_1, textEditor_1, editorCommon_1, editorService_1, quickOpenService_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    exports.GOTO_LINE_PREFIX = ':';
    var GotoLineAction = (function (_super) {
        __extends(GotoLineAction, _super);
        function GotoLineAction(actionId, actionLabel, quickOpenService) {
            _super.call(this, actionId, actionLabel, exports.GOTO_LINE_PREFIX, quickOpenService);
        }
        GotoLineAction.ID = 'workbench.action.gotoLine';
        GotoLineAction.LABEL = nls.localize(0, null);
        GotoLineAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], GotoLineAction);
        return GotoLineAction;
    }(quickOpenAction_1.QuickOpenAction));
    exports.GotoLineAction = GotoLineAction;
    var GotoLineEntry = (function (_super) {
        __extends(GotoLineEntry, _super);
        function GotoLineEntry(line, editorService, handler) {
            _super.call(this, editorService);
            this.parseInput(line);
            this.handler = handler;
        }
        GotoLineEntry.prototype.parseInput = function (line) {
            var numbers = line.split(/,|:|#/).map(function (part) { return parseInt(part, 10); }).filter(function (part) { return !isNaN(part); });
            this.line = numbers[0];
            this.column = numbers[1];
        };
        GotoLineEntry.prototype.getLabel = function () {
            // Inform user about valid range if input is invalid
            var maxLineNumber = this.getMaxLineNumber();
            if (this.invalidRange(maxLineNumber)) {
                if (maxLineNumber > 0) {
                    return nls.localize(1, null, maxLineNumber);
                }
                return nls.localize(2, null);
            }
            // Input valid, indicate action
            return this.column ? nls.localize(3, null, this.line, this.column) : nls.localize(4, null, this.line);
        };
        GotoLineEntry.prototype.invalidRange = function (maxLineNumber) {
            if (maxLineNumber === void 0) { maxLineNumber = this.getMaxLineNumber(); }
            return !this.line || !types.isNumber(this.line) || (maxLineNumber > 0 && types.isNumber(this.line) && this.line > maxLineNumber);
        };
        GotoLineEntry.prototype.getMaxLineNumber = function () {
            var editor = this.editorService.getActiveEditor();
            var editorControl = editor.getControl();
            var model = editorControl.getModel();
            if (model && model.modified && model.original) {
                model = model.modified; // Support for diff editor models
            }
            return model && types.isFunction(model.getLineCount) ? model.getLineCount() : -1;
        };
        GotoLineEntry.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.OPEN) {
                return this.runOpen(context);
            }
            return this.runPreview();
        };
        GotoLineEntry.prototype.getInput = function () {
            return this.editorService.getActiveEditorInput();
        };
        GotoLineEntry.prototype.getOptions = function () {
            var range = this.toSelection();
            var options = new editor_1.TextEditorOptions();
            options.selection(range.startLineNumber, range.startColumn, range.startLineNumber, range.startColumn);
            return options;
        };
        GotoLineEntry.prototype.runOpen = function (context) {
            // No-op if range is not valid
            if (this.invalidRange()) {
                return false;
            }
            // Check for sideBySide use
            var event = context.event;
            var sideBySide = (event && (event.ctrlKey || event.metaKey || (event.payload && event.payload.originalEvent && (event.payload.originalEvent.ctrlKey || event.payload.originalEvent.metaKey))));
            if (sideBySide) {
                this.editorService.openEditor(this.getInput(), this.getOptions(), true).done(null, errors.onUnexpectedError);
            }
            // Apply selection and focus
            var range = this.toSelection();
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor) {
                var editor = activeEditor.getControl();
                editor.setSelection(range);
                editor.revealRangeInCenter(range);
            }
            return true;
        };
        GotoLineEntry.prototype.runPreview = function () {
            // No-op if range is not valid
            if (this.invalidRange()) {
                this.handler.clearDecorations();
                return false;
            }
            // Select Line Position
            var range = this.toSelection();
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor) {
                var editorControl = activeEditor.getControl();
                editorControl.revealRangeInCenter(range);
                // Decorate if possible
                if (types.isFunction(editorControl.changeDecorations)) {
                    this.handler.decorateOutline(range, editorControl, activeEditor.position);
                }
            }
            return false;
        };
        GotoLineEntry.prototype.toSelection = function () {
            return {
                startLineNumber: this.line,
                startColumn: this.column || 1,
                endLineNumber: this.line,
                endColumn: this.column || 1
            };
        };
        return GotoLineEntry;
    }(quickopen_1.EditorQuickOpenEntry));
    var GotoLineHandler = (function (_super) {
        __extends(GotoLineHandler, _super);
        function GotoLineHandler(editorService) {
            _super.call(this);
            this.editorService = editorService;
        }
        GotoLineHandler.prototype.getAriaLabel = function () {
            return nls.localize(5, null);
        };
        GotoLineHandler.prototype.getResults = function (searchValue) {
            searchValue = searchValue.trim();
            // Remember view state to be able to restore on cancel
            if (!this.lastKnownEditorViewState) {
                var editor = this.editorService.getActiveEditor();
                this.lastKnownEditorViewState = editor.getControl().saveViewState();
            }
            return winjs_base_1.TPromise.as(new quickOpenModel_1.QuickOpenModel([new GotoLineEntry(searchValue, this.editorService, this)]));
        };
        GotoLineHandler.prototype.canRun = function () {
            var canRun = this.editorService.getActiveEditor() instanceof textEditor_1.BaseTextEditor;
            return canRun ? true : nls.localize(6, null);
        };
        GotoLineHandler.prototype.decorateOutline = function (range, editor, position) {
            var _this = this;
            editor.changeDecorations(function (changeAccessor) {
                var deleteDecorations = [];
                if (_this.lineHighlightDecorationId) {
                    deleteDecorations.push(_this.lineHighlightDecorationId.lineDecorationId);
                    deleteDecorations.push(_this.lineHighlightDecorationId.lineHighlightId);
                    _this.lineHighlightDecorationId = null;
                }
                var newDecorations = [
                    // lineHighlight at index 0
                    {
                        range: range,
                        options: {
                            className: 'lineHighlight',
                            isWholeLine: true
                        }
                    },
                    // lineDecoration at index 1
                    {
                        range: range,
                        options: {
                            overviewRuler: {
                                color: 'rgba(0, 122, 204, 0.6)',
                                darkColor: 'rgba(0, 122, 204, 0.6)',
                                position: editorCommon_1.OverviewRulerLane.Full
                            }
                        }
                    }
                ];
                var decorations = changeAccessor.deltaDecorations(deleteDecorations, newDecorations);
                var lineHighlightId = decorations[0];
                var lineDecorationId = decorations[1];
                _this.lineHighlightDecorationId = {
                    lineHighlightId: lineHighlightId,
                    lineDecorationId: lineDecorationId,
                    position: position
                };
            });
        };
        GotoLineHandler.prototype.clearDecorations = function () {
            var _this = this;
            if (this.lineHighlightDecorationId) {
                this.editorService.getVisibleEditors().forEach(function (editor) {
                    if (editor.position === _this.lineHighlightDecorationId.position) {
                        var editorControl = editor.getControl();
                        editorControl.changeDecorations(function (changeAccessor) {
                            changeAccessor.deltaDecorations([
                                _this.lineHighlightDecorationId.lineDecorationId,
                                _this.lineHighlightDecorationId.lineHighlightId
                            ], []);
                        });
                    }
                });
                this.lineHighlightDecorationId = null;
            }
        };
        GotoLineHandler.prototype.onClose = function (canceled) {
            // Clear Highlight Decorations if present
            this.clearDecorations();
            // Restore selection if canceled
            if (canceled && this.lastKnownEditorViewState) {
                var activeEditor = this.editorService.getActiveEditor();
                if (activeEditor) {
                    var editor = activeEditor.getControl();
                    editor.restoreViewState(this.lastKnownEditorViewState);
                }
            }
            this.lastKnownEditorViewState = null;
        };
        GotoLineHandler.prototype.getAutoFocus = function (searchValue) {
            return {
                autoFocusFirstEntry: searchValue.trim().length > 0
            };
        };
        GotoLineHandler = __decorate([
            __param(0, editorService_1.IWorkbenchEditorService)
        ], GotoLineHandler);
        return GotoLineHandler;
    }(quickopen_1.QuickOpenHandler));
    exports.GotoLineHandler = GotoLineHandler;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/quickopen/browser/gotoSymbolHandler", ["require", "exports", 'vs/base/common/winjs.base', 'vs/nls!vs/workbench/parts/quickopen/browser/gotoSymbolHandler', 'vs/base/common/arrays', 'vs/base/common/errors', 'vs/base/common/types', 'vs/base/common/strings', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/workbench/browser/quickopen', 'vs/workbench/browser/actions/quickOpenAction', 'vs/workbench/browser/parts/editor/textEditor', 'vs/workbench/common/editor', 'vs/base/common/filters', 'vs/editor/common/editorCommon', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/platform/workspace/common/workspace', 'vs/editor/contrib/quickOpen/common/quickOpen', 'vs/css!./media/gotoSymbolHandler'], function (require, exports, winjs_base_1, nls, arrays, errors, types, strings, quickOpen_1, quickOpenModel_1, quickopen_1, quickOpenAction_1, textEditor_1, editor_1, filters, editorCommon_1, editorService_1, quickOpenService_1, workspace_1, quickOpen_2) {
    'use strict';
    exports.GOTO_SYMBOL_PREFIX = '@';
    exports.SCOPE_PREFIX = ':';
    var GotoSymbolAction = (function (_super) {
        __extends(GotoSymbolAction, _super);
        function GotoSymbolAction(actionId, actionLabel, quickOpenService) {
            _super.call(this, actionId, actionLabel, exports.GOTO_SYMBOL_PREFIX, quickOpenService);
        }
        GotoSymbolAction.ID = 'workbench.action.gotoSymbol';
        GotoSymbolAction.LABEL = nls.localize(0, null);
        GotoSymbolAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], GotoSymbolAction);
        return GotoSymbolAction;
    }(quickOpenAction_1.QuickOpenAction));
    exports.GotoSymbolAction = GotoSymbolAction;
    var OutlineModel = (function (_super) {
        __extends(OutlineModel, _super);
        function OutlineModel(outline, entries) {
            _super.call(this, entries);
            this.outline = outline;
        }
        OutlineModel.prototype.dofilter = function (searchValue) {
            // Normalize search
            var normalizedSearchValue = searchValue;
            if (searchValue.indexOf(exports.SCOPE_PREFIX) === 0) {
                normalizedSearchValue = normalizedSearchValue.substr(exports.SCOPE_PREFIX.length);
            }
            // Check for match and update visibility and group label
            this.entries.forEach(function (entry) {
                // Clear all state first
                entry.setGroupLabel(null);
                entry.setShowBorder(false);
                entry.setHighlights(null);
                entry.setHidden(false);
                // Filter by search
                if (normalizedSearchValue) {
                    var highlights = filters.matchesFuzzy(normalizedSearchValue, entry.getLabel());
                    if (highlights) {
                        entry.setHighlights(highlights);
                        entry.setHidden(false);
                    }
                    else if (!entry.isHidden()) {
                        entry.setHidden(true);
                    }
                }
            });
            // Sort properly if actually searching
            if (searchValue) {
                if (searchValue.indexOf(exports.SCOPE_PREFIX) === 0) {
                    this.entries.sort(this.sortScoped.bind(this, searchValue.toLowerCase()));
                }
                else {
                    this.entries.sort(this.sortNormal.bind(this, searchValue.toLowerCase()));
                }
            }
            else {
                this.entries.sort(function (a, b) { return a.getIndex() - b.getIndex(); });
            }
            // Mark all type groups
            var visibleResults = this.getEntries(true);
            if (visibleResults.length > 0 && searchValue.indexOf(exports.SCOPE_PREFIX) === 0) {
                var currentType = null;
                var currentResult = null;
                var typeCounter = 0;
                for (var i = 0; i < visibleResults.length; i++) {
                    var result = visibleResults[i];
                    // Found new type
                    if (currentType !== result.getType()) {
                        // Update previous result with count
                        if (currentResult) {
                            currentResult.setGroupLabel(this.renderGroupLabel(currentType, typeCounter, this.outline));
                        }
                        currentType = result.getType();
                        currentResult = result;
                        typeCounter = 1;
                        result.setShowBorder(i > 0);
                    }
                    else {
                        typeCounter++;
                    }
                }
                // Update previous result with count
                if (currentResult) {
                    currentResult.setGroupLabel(this.renderGroupLabel(currentType, typeCounter, this.outline));
                }
            }
            else if (visibleResults.length > 0) {
                visibleResults[0].setGroupLabel(nls.localize(1, null, visibleResults.length));
            }
        };
        OutlineModel.prototype.sortNormal = function (searchValue, elementA, elementB) {
            // Handle hidden elements
            if (elementA.isHidden() && elementB.isHidden()) {
                return 0;
            }
            else if (elementA.isHidden()) {
                return 1;
            }
            else if (elementB.isHidden()) {
                return -1;
            }
            var elementAName = elementA.getLabel().toLowerCase();
            var elementBName = elementB.getLabel().toLowerCase();
            // Compare by name
            var r = strings.localeCompare(elementAName, elementBName);
            if (r !== 0) {
                return r;
            }
            // If name identical sort by range instead
            var elementARange = elementA.getRange();
            var elementBRange = elementB.getRange();
            return elementARange.startLineNumber - elementBRange.startLineNumber;
        };
        OutlineModel.prototype.sortScoped = function (searchValue, elementA, elementB) {
            // Handle hidden elements
            if (elementA.isHidden() && elementB.isHidden()) {
                return 0;
            }
            else if (elementA.isHidden()) {
                return 1;
            }
            else if (elementB.isHidden()) {
                return -1;
            }
            // Remove scope char
            searchValue = searchValue.substr(exports.SCOPE_PREFIX.length);
            // Sort by type first if scoped search
            var elementAType = elementA.getType();
            var elementBType = elementB.getType();
            var r = strings.localeCompare(elementAType, elementBType);
            if (r !== 0) {
                return r;
            }
            // Special sort when searching in scoped mode
            if (searchValue) {
                var elementAName = elementA.getLabel().toLowerCase();
                var elementBName = elementB.getLabel().toLowerCase();
                // Compare by name
                r = strings.localeCompare(elementAName, elementBName);
                if (r !== 0) {
                    return r;
                }
            }
            // Default to sort by range
            var elementARange = elementA.getRange();
            var elementBRange = elementB.getRange();
            return elementARange.startLineNumber - elementBRange.startLineNumber;
        };
        OutlineModel.prototype.renderGroupLabel = function (type, count, outline) {
            var pattern = outline.outlineGroupLabel[type] || OutlineModel.getDefaultGroupLabelPatterns()[type];
            if (pattern) {
                return strings.format(pattern, count);
            }
            return type;
        };
        OutlineModel.getDefaultGroupLabelPatterns = function () {
            var result = Object.create(null);
            result['method'] = nls.localize(2, null);
            result['function'] = nls.localize(3, null);
            result['constructor'] = nls.localize(4, null);
            result['variable'] = nls.localize(5, null);
            result['class'] = nls.localize(6, null);
            result['interface'] = nls.localize(7, null);
            result['namespace'] = nls.localize(8, null);
            result['package'] = nls.localize(9, null);
            result['module'] = nls.localize(10, null);
            result['property'] = nls.localize(11, null);
            result['enum'] = nls.localize(12, null);
            result['string'] = nls.localize(13, null);
            result['rule'] = nls.localize(14, null);
            result['file'] = nls.localize(15, null);
            result['array'] = nls.localize(16, null);
            result['number'] = nls.localize(17, null);
            result['boolean'] = nls.localize(18, null);
            result['object'] = nls.localize(19, null);
            result['key'] = nls.localize(20, null);
            return result;
        };
        return OutlineModel;
    }(quickOpenModel_1.QuickOpenModel));
    var SymbolEntry = (function (_super) {
        __extends(SymbolEntry, _super);
        function SymbolEntry(index, name, type, description, icon, range, highlights, editorService, handler) {
            _super.call(this);
            this.index = index;
            this.name = name;
            this.type = type;
            this.icon = icon;
            this.description = description;
            this.range = range;
            this.setHighlights(highlights);
            this.editorService = editorService;
            this.handler = handler;
        }
        SymbolEntry.prototype.getIndex = function () {
            return this.index;
        };
        SymbolEntry.prototype.getLabel = function () {
            return this.name;
        };
        SymbolEntry.prototype.getAriaLabel = function () {
            return nls.localize(21, null, this.getLabel());
        };
        SymbolEntry.prototype.getIcon = function () {
            return this.icon;
        };
        SymbolEntry.prototype.getDescription = function () {
            return this.description;
        };
        SymbolEntry.prototype.getType = function () {
            return this.type;
        };
        SymbolEntry.prototype.getRange = function () {
            return this.range;
        };
        SymbolEntry.prototype.getInput = function () {
            return this.editorService.getActiveEditorInput();
        };
        SymbolEntry.prototype.getOptions = function () {
            var options = new editor_1.TextEditorOptions();
            options.selection(this.range.startLineNumber, this.range.startColumn, this.range.startLineNumber, this.range.startColumn);
            return options;
        };
        SymbolEntry.prototype.run = function (mode, context) {
            if (mode === quickOpen_1.Mode.OPEN) {
                return this.runOpen(context);
            }
            return this.runPreview();
        };
        SymbolEntry.prototype.runOpen = function (context) {
            // Check for sideBySide use
            var event = context.event;
            var sideBySide = (event && (event.ctrlKey || event.metaKey || (event.payload && event.payload.originalEvent && (event.payload.originalEvent.ctrlKey || event.payload.originalEvent.metaKey))));
            if (sideBySide) {
                this.editorService.openEditor(this.getInput(), this.getOptions(), true).done(null, errors.onUnexpectedError);
            }
            else {
                var range = this.toSelection();
                var activeEditor = this.editorService.getActiveEditor();
                if (activeEditor) {
                    var editor = activeEditor.getControl();
                    editor.setSelection(range);
                    editor.revealRangeInCenter(range);
                }
            }
            return true;
        };
        SymbolEntry.prototype.runPreview = function () {
            // Select Outline Position
            var range = this.toSelection();
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor) {
                var editorControl = activeEditor.getControl();
                editorControl.revealRangeInCenter(range);
                // Decorate if possible
                if (types.isFunction(editorControl.changeDecorations)) {
                    this.handler.decorateOutline(this.range, range, editorControl, activeEditor.position);
                }
            }
            return false;
        };
        SymbolEntry.prototype.toSelection = function () {
            return {
                startLineNumber: this.range.startLineNumber,
                startColumn: this.range.startColumn || 1,
                endLineNumber: this.range.startLineNumber,
                endColumn: this.range.startColumn || 1
            };
        };
        return SymbolEntry;
    }(quickopen_1.EditorQuickOpenEntryGroup));
    var GotoSymbolHandler = (function (_super) {
        __extends(GotoSymbolHandler, _super);
        function GotoSymbolHandler(editorService, contextService) {
            _super.call(this);
            this.editorService = editorService;
            this.contextService = contextService;
            this.outlineToModelCache = {};
        }
        GotoSymbolHandler.prototype.getResults = function (searchValue) {
            searchValue = searchValue.trim();
            // Remember view state to be able to restore on cancel
            if (!this.lastKnownEditorViewState) {
                var editor = this.editorService.getActiveEditor();
                this.lastKnownEditorViewState = editor.getControl().saveViewState();
            }
            // Resolve Outline Model
            return this.getActiveOutline().then(function (outline) {
                // Filter by search
                outline.dofilter(searchValue);
                return outline;
            });
        };
        GotoSymbolHandler.prototype.getEmptyLabel = function (searchString) {
            if (searchString.length > 0) {
                return nls.localize(22, null);
            }
            return nls.localize(23, null);
        };
        GotoSymbolHandler.prototype.getAriaLabel = function () {
            return nls.localize(24, null);
        };
        GotoSymbolHandler.prototype.canRun = function () {
            var canRun = false;
            var editor = this.editorService.getActiveEditor();
            if (editor instanceof textEditor_1.BaseTextEditor) {
                var editorControl = editor.getControl();
                var model = editorControl.getModel();
                if (model && model.modified && model.original) {
                    model = model.modified; // Support for diff editor models
                }
                if (model && types.isFunction(model.getMode)) {
                    canRun = quickOpen_2.OutlineRegistry.has(model);
                }
            }
            return canRun ? true : editor instanceof textEditor_1.BaseTextEditor ? nls.localize(25, null) : nls.localize(26, null);
        };
        GotoSymbolHandler.prototype.getAutoFocus = function (searchValue) {
            searchValue = searchValue.trim();
            // Remove any type pattern (:) from search value as needed
            if (searchValue.indexOf(exports.SCOPE_PREFIX) === 0) {
                searchValue = searchValue.substr(exports.SCOPE_PREFIX.length);
            }
            return {
                autoFocusPrefixMatch: searchValue,
                autoFocusFirstEntry: !!searchValue
            };
        };
        GotoSymbolHandler.prototype.toQuickOpenEntries = function (outline) {
            var results = [];
            // Flatten
            var flattened = [];
            if (outline) {
                this.flatten(outline.entries, flattened);
            }
            for (var i = 0; i < flattened.length; i++) {
                var element = flattened[i];
                var label = strings.trim(element.label);
                // Show parent scope as description
                var description = element.containerLabel;
                if (element.parentScope) {
                    description = arrays.tail(element.parentScope);
                }
                // Add
                var icon = element.icon || element.type;
                results.push(new SymbolEntry(i, label, element.type, description, icon, element.range, null, this.editorService, this));
            }
            return results;
        };
        GotoSymbolHandler.prototype.flatten = function (outline, flattened, parentScope) {
            for (var i = 0; i < outline.length; i++) {
                var element = outline[i];
                flattened.push(element);
                if (parentScope) {
                    element.parentScope = parentScope;
                }
                if (element.children) {
                    var elementScope = [];
                    if (parentScope) {
                        elementScope = parentScope.slice(0);
                    }
                    elementScope.push(element.label);
                    this.flatten(element.children, flattened, elementScope);
                }
            }
        };
        GotoSymbolHandler.prototype.getActiveOutline = function () {
            if (!this.activeOutlineRequest) {
                this.activeOutlineRequest = this.doGetActiveOutline();
            }
            return this.activeOutlineRequest;
        };
        GotoSymbolHandler.prototype.doGetActiveOutline = function () {
            var _this = this;
            var editor = this.editorService.getActiveEditor();
            if (editor instanceof textEditor_1.BaseTextEditor) {
                var editorControl = editor.getControl();
                var model = editorControl.getModel();
                if (model && model.modified && model.original) {
                    model = model.modified; // Support for diff editor models
                }
                if (model && types.isFunction(model.getMode)) {
                    // Ask cache first
                    var modelId_1 = model.id;
                    if (this.outlineToModelCache[modelId_1]) {
                        return winjs_base_1.TPromise.as(this.outlineToModelCache[modelId_1]);
                    }
                    return quickOpen_2.getOutlineEntries(model).then(function (outline) {
                        var model = new OutlineModel(outline, _this.toQuickOpenEntries(outline));
                        _this.outlineToModelCache = {}; // Clear cache, only keep 1 outline
                        _this.outlineToModelCache[modelId_1] = model;
                        return model;
                    });
                }
            }
            return winjs_base_1.TPromise.as(null);
        };
        GotoSymbolHandler.prototype.decorateOutline = function (fullRange, startRange, editor, position) {
            var _this = this;
            editor.changeDecorations(function (changeAccessor) {
                var deleteDecorations = [];
                if (_this.lineHighlightDecorationId) {
                    deleteDecorations.push(_this.lineHighlightDecorationId.lineDecorationId);
                    deleteDecorations.push(_this.lineHighlightDecorationId.lineHighlightId);
                    _this.lineHighlightDecorationId = null;
                }
                var newDecorations = [
                    // lineHighlight at index 0
                    {
                        range: fullRange,
                        options: {
                            className: 'lineHighlight',
                            isWholeLine: true
                        }
                    },
                    // lineDecoration at index 1
                    {
                        range: startRange,
                        options: {
                            overviewRuler: {
                                color: 'rgba(0, 122, 204, 0.6)',
                                darkColor: 'rgba(0, 122, 204, 0.6)',
                                position: editorCommon_1.OverviewRulerLane.Full
                            }
                        }
                    }
                ];
                var decorations = changeAccessor.deltaDecorations(deleteDecorations, newDecorations);
                var lineHighlightId = decorations[0];
                var lineDecorationId = decorations[1];
                _this.lineHighlightDecorationId = {
                    lineHighlightId: lineHighlightId,
                    lineDecorationId: lineDecorationId,
                    position: position
                };
            });
        };
        GotoSymbolHandler.prototype.clearDecorations = function () {
            var _this = this;
            if (this.lineHighlightDecorationId) {
                this.editorService.getVisibleEditors().forEach(function (editor) {
                    if (editor.position === _this.lineHighlightDecorationId.position) {
                        var editorControl = editor.getControl();
                        editorControl.changeDecorations(function (changeAccessor) {
                            changeAccessor.deltaDecorations([
                                _this.lineHighlightDecorationId.lineDecorationId,
                                _this.lineHighlightDecorationId.lineHighlightId
                            ], []);
                        });
                    }
                });
                this.lineHighlightDecorationId = null;
            }
        };
        GotoSymbolHandler.prototype.onClose = function (canceled) {
            // Clear Cache
            this.outlineToModelCache = {};
            // Clear Highlight Decorations if present
            this.clearDecorations();
            // Restore selection if canceled
            if (canceled && this.lastKnownEditorViewState) {
                var activeEditor = this.editorService.getActiveEditor();
                if (activeEditor) {
                    var editor = activeEditor.getControl();
                    editor.restoreViewState(this.lastKnownEditorViewState);
                }
            }
            this.lastKnownEditorViewState = null;
            this.activeOutlineRequest = null;
        };
        GotoSymbolHandler = __decorate([
            __param(0, editorService_1.IWorkbenchEditorService),
            __param(1, workspace_1.IWorkspaceContextService)
        ], GotoSymbolHandler);
        return GotoSymbolHandler;
    }(quickopen_1.QuickOpenHandler));
    exports.GotoSymbolHandler = GotoSymbolHandler;
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/quickopen/browser/markersHandler", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/strings', 'vs/base/common/network', 'vs/base/common/errors', 'vs/base/browser/dom', 'vs/nls!vs/workbench/parts/quickopen/browser/markersHandler', 'vs/base/common/labels', 'vs/base/common/severity', 'vs/workbench/browser/quickopen', 'vs/workbench/browser/parts/editor/textEditor', 'vs/workbench/browser/actions/quickOpenAction', 'vs/base/parts/quickopen/common/quickOpen', 'vs/base/parts/quickopen/browser/quickOpenModel', 'vs/workbench/services/editor/common/editorService', 'vs/platform/markers/common/markers', 'vs/platform/workspace/common/workspace', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/editor/common/services/codeEditorService', 'vs/base/common/filters', 'vs/base/browser/ui/highlightedlabel/highlightedLabel', 'vs/css!./media/markerHandler'], function (require, exports, winjs_base_1, strings, network, errors, dom, nls, labels_1, severity_1, quickopen_1, textEditor_1, quickOpenAction_1, quickOpen_1, quickOpenModel_1, editorService_1, markers_1, workspace_1, quickOpenService_1, codeEditorService_1, filters_1, highlightedLabel_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var MarkerEntry = (function (_super) {
        __extends(MarkerEntry, _super);
        function MarkerEntry(editorService, codeEditorService, contextService, marker) {
            _super.call(this);
            this._editorService = editorService;
            this._codeEditorService = codeEditorService;
            this._labelProvider = new labels_1.PathLabelProvider(contextService);
            this._marker = marker;
            var message = marker.message, source = marker.source, resource = marker.resource, startLineNumber = marker.startLineNumber, startColumn = marker.startColumn;
            this._label = source ? nls.localize(0, null, source, message) : message;
            this._description = nls.localize(1, null, this._labelProvider.getLabel(resource.fsPath), startLineNumber, startColumn);
        }
        MarkerEntry.prototype.update = function (query) {
            if (this._marker.resource.scheme === network.Schemas.inMemory) {
                // ignore inmemory-models
                this.setHidden(true);
                return;
            }
            var labelHighlights = MarkerEntry._filter(query, this._label);
            var descHighlights = MarkerEntry._filter(query, this._description);
            this.setHighlights(labelHighlights, descHighlights);
            this.setHidden(!labelHighlights && !descHighlights);
        };
        MarkerEntry.prototype.getAriaLabel = function () {
            return nls.localize(2, null, this._label);
        };
        MarkerEntry.prototype.getHeight = function () {
            return 48;
        };
        MarkerEntry.prototype.render = function (tree, container, previousCleanupFn) {
            dom.clearNode(container);
            var _a = this.getHighlights(), labelHighlights = _a[0], descHighlights = _a[1];
            var row1 = document.createElement('div');
            dom.addClass(row1, 'row');
            var row2 = document.createElement('div');
            dom.addClass(row2, 'row');
            // fill first row with icon and label
            var icon = document.createElement('div');
            dom.addClass(icon, "severity " + severity_1.default.toString(this._marker.severity).toLowerCase());
            row1.appendChild(icon);
            var labelContainer = document.createElement('div');
            dom.addClass(labelContainer, 'inline');
            new highlightedLabel_1.HighlightedLabel(labelContainer).set(this._label, labelHighlights);
            row1.appendChild(labelContainer);
            // fill second row with descriptions
            var descContainer = document.createElement('div');
            dom.addClass(descContainer, 'inline description');
            new highlightedLabel_1.HighlightedLabel(descContainer).set(this._description, descHighlights);
            row2.appendChild(descContainer);
            container.appendChild(row1);
            container.appendChild(row2);
            return;
        };
        MarkerEntry.prototype.run = function (mode, context) {
            switch (mode) {
                case quickOpen_1.Mode.OPEN:
                    this._open();
                    return true;
                case quickOpen_1.Mode.PREVIEW:
                    this._preview();
                    return true;
                default:
                    return false;
            }
        };
        MarkerEntry.prototype._open = function () {
            this._editorService.openEditor({
                resource: this._marker.resource,
                options: {
                    selection: {
                        startLineNumber: this._marker.startLineNumber,
                        startColumn: this._marker.startColumn,
                        endLineNumber: this._marker.endLineNumber,
                        endColumn: this._marker.endColumn
                    }
                }
            }).done(null, errors.onUnexpectedError);
        };
        MarkerEntry.prototype._preview = function () {
            var editors = this._codeEditorService.listCodeEditors();
            var editor;
            for (var _i = 0, editors_1 = editors; _i < editors_1.length; _i++) {
                var candidate = editors_1[_i];
                if (!candidate.getModel()
                    || candidate.getModel().getAssociatedResource().toString() !== this._marker.resource.toString()) {
                    continue;
                }
                if (!editor || this._editorService.getActiveEditor()
                    && candidate === this._editorService.getActiveEditor().getControl()) {
                    editor = candidate;
                }
            }
            if (editor) {
                editor.revealRangeInCenter(this._marker);
            }
        };
        MarkerEntry._filter = filters_1.or(filters_1.matchesPrefix, filters_1.matchesContiguousSubString);
        return MarkerEntry;
    }(quickOpenModel_1.QuickOpenEntryItem));
    var MarkersHandler = (function (_super) {
        __extends(MarkersHandler, _super);
        function MarkersHandler(markerService, editorService, codeEditorService, contextService) {
            _super.call(this);
            this._markerService = markerService;
            this._editorService = editorService;
            this._codeEditorService = codeEditorService;
            this._contextService = contextService;
        }
        MarkersHandler.prototype.getAriaLabel = function () {
            return nls.localize(3, null);
        };
        MarkersHandler.prototype.getResults = function (searchValue) {
            var _this = this;
            if (!this._activeSession) {
                // 1st model
                var model_1 = new quickOpenModel_1.QuickOpenModel(this._markerService.read({ take: 500 })
                    .sort(MarkersHandler._sort)
                    .map(function (marker) { return new MarkerEntry(_this._editorService, _this._codeEditorService, _this._contextService, marker); }));
                // 2nd viewstate
                var editor = this._editorService.getActiveEditor();
                var viewState = void 0;
                if (editor instanceof textEditor_1.BaseTextEditor) {
                    viewState = editor.getControl().saveViewState();
                }
                this._activeSession = [model_1, viewState];
            }
            // filter
            searchValue = searchValue.trim();
            var model = this._activeSession[0];
            for (var _i = 0, _a = model.entries; _i < _a.length; _i++) {
                var entry = _a[_i];
                entry.update(searchValue);
            }
            return winjs_base_1.TPromise.as(model);
        };
        MarkersHandler.prototype.onClose = function (canceled) {
            if (this._activeSession) {
                if (canceled) {
                    var _a = this._activeSession, viewState = _a[1];
                    if (viewState) {
                        var editor = this._editorService.getActiveEditor();
                        editor.getControl().restoreViewState(viewState);
                    }
                }
                this._activeSession = undefined;
            }
        };
        MarkersHandler._sort = function (a, b) {
            var ret;
            // severity matters first
            ret = severity_1.default.compare(a.severity, b.severity);
            if (ret !== 0) {
                return ret;
            }
            // source matters
            if (a.source && b.source) {
                ret = a.source.localeCompare(b.source);
                if (ret !== 0) {
                    return ret;
                }
            }
            // file name matters for equal severity
            ret = strings.localeCompare(a.resource.fsPath, b.resource.fsPath);
            if (ret !== 0) {
                return ret;
            }
            // start line matters
            ret = a.startLineNumber - b.startLineNumber;
            if (ret !== 0) {
                return ret;
            }
            // start column matters
            ret = a.startColumn - b.startColumn;
            if (ret !== 0) {
                return ret;
            }
            return 0;
        };
        MarkersHandler.prototype.getClass = function () {
            return 'marker-handler';
        };
        MarkersHandler.prototype.getAutoFocus = function (searchValue) {
            return {
                autoFocusFirstEntry: !!searchValue
            };
        };
        MarkersHandler.prototype.getEmptyLabel = function (searchString) {
            if (searchString.length > 0) {
                return nls.localize(4, null);
            }
            return nls.localize(5, null);
        };
        MarkersHandler = __decorate([
            __param(0, markers_1.IMarkerService),
            __param(1, editorService_1.IWorkbenchEditorService),
            __param(2, codeEditorService_1.ICodeEditorService),
            __param(3, workspace_1.IWorkspaceContextService)
        ], MarkersHandler);
        return MarkersHandler;
    }(quickopen_1.QuickOpenHandler));
    exports.MarkersHandler = MarkersHandler;
    var GotoMarkerAction = (function (_super) {
        __extends(GotoMarkerAction, _super);
        function GotoMarkerAction(actionId, actionLabel, quickOpenService) {
            _super.call(this, actionId, actionLabel, GotoMarkerAction.Prefix, quickOpenService);
        }
        GotoMarkerAction.Prefix = '!';
        GotoMarkerAction.Id = 'workbench.action.showErrorsWarnings';
        GotoMarkerAction.Label = nls.localize(6, null);
        GotoMarkerAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], GotoMarkerAction);
        return GotoMarkerAction;
    }(quickOpenAction_1.QuickOpenAction));
    exports.GotoMarkerAction = GotoMarkerAction;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/parts/quickopen/browser/quickopen.contribution", ["require", "exports", 'vs/base/common/platform', 'vs/nls!vs/workbench/parts/quickopen/browser/quickopen.contribution', 'vs/workbench/browser/quickopen', 'vs/platform/platform', 'vs/platform/actions/common/actions', 'vs/workbench/common/actionRegistry', 'vs/base/common/keyCodes', 'vs/workbench/parts/quickopen/browser/gotoSymbolHandler', 'vs/workbench/parts/quickopen/browser/commandsHandler', 'vs/workbench/parts/quickopen/browser/gotoLineHandler', 'vs/workbench/parts/quickopen/browser/helpHandler', 'vs/workbench/parts/quickopen/browser/markersHandler'], function (require, exports, env, nls, quickopen_1, platform_1, actions_1, actionRegistry_1, keyCodes_1, gotoSymbolHandler_1, commandsHandler_1, gotoLineHandler_1, helpHandler_1, markersHandler_1) {
    'use strict';
    // Register Actions
    var registry = platform_1.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(markersHandler_1.GotoMarkerAction, markersHandler_1.GotoMarkerAction.Id, markersHandler_1.GotoMarkerAction.Label, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_M
    }));
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(commandsHandler_1.ShowAllCommandsAction, commandsHandler_1.ShowAllCommandsAction.ID, commandsHandler_1.ShowAllCommandsAction.LABEL, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_P,
        secondary: [keyCodes_1.KeyCode.F1]
    }));
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(gotoLineHandler_1.GotoLineAction, gotoLineHandler_1.GotoLineAction.ID, gotoLineHandler_1.GotoLineAction.LABEL, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyCode.KEY_G,
        mac: { primary: keyCodes_1.KeyMod.WinCtrl | keyCodes_1.KeyCode.KEY_G }
    }));
    registry.registerWorkbenchAction(new actions_1.SyncActionDescriptor(gotoSymbolHandler_1.GotoSymbolAction, gotoSymbolHandler_1.GotoSymbolAction.ID, gotoSymbolHandler_1.GotoSymbolAction.LABEL, {
        primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_O
    }));
    // Register Quick Open Handler
    platform_1.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/quickopen/browser/markersHandler', 'MarkersHandler', markersHandler_1.GotoMarkerAction.Prefix, [
        {
            prefix: markersHandler_1.GotoMarkerAction.Prefix,
            needsEditor: false,
            description: env.isMacintosh ? nls.localize(0, null) : nls.localize(1, null)
        },
    ]));
    platform_1.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/quickopen/browser/commandsHandler', 'CommandsHandler', commandsHandler_1.ALL_COMMANDS_PREFIX, nls.localize(2, null)));
    platform_1.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/quickopen/browser/gotoLineHandler', 'GotoLineHandler', gotoLineHandler_1.GOTO_LINE_PREFIX, [
        {
            prefix: gotoLineHandler_1.GOTO_LINE_PREFIX,
            needsEditor: true,
            description: env.isMacintosh ? nls.localize(3, null) : nls.localize(4, null)
        },
    ]));
    platform_1.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/quickopen/browser/gotoSymbolHandler', 'GotoSymbolHandler', gotoSymbolHandler_1.GOTO_SYMBOL_PREFIX, [
        {
            prefix: gotoSymbolHandler_1.GOTO_SYMBOL_PREFIX,
            needsEditor: true,
            description: env.isMacintosh ? nls.localize(5, null) : nls.localize(6, null)
        },
        {
            prefix: gotoSymbolHandler_1.GOTO_SYMBOL_PREFIX + gotoSymbolHandler_1.SCOPE_PREFIX,
            needsEditor: true,
            description: nls.localize(7, null)
        }
    ]));
    platform_1.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/quickopen/browser/helpHandler', 'HelpHandler', helpHandler_1.HELP_PREFIX, nls.localize(8, null)));
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/tasks/electron-browser/task.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/tasks/electron-browser/task.contribution', 'vs/base/common/flags', 'vs/base/common/winjs.base', 'vs/base/common/severity', 'vs/base/common/objects', 'vs/base/common/actions', 'vs/base/browser/dom', 'vs/base/common/lifecycle', 'vs/base/common/eventEmitter', 'vs/base/browser/builder', 'vs/base/common/types', 'vs/base/common/keyCodes', 'vs/base/common/glob', 'vs/base/common/platform', 'vs/platform/platform', 'vs/platform/lifecycle/common/lifecycle', 'vs/platform/actions/common/actions', 'vs/platform/instantiation/common/extensions', 'vs/platform/event/common/event', 'vs/platform/message/common/message', 'vs/platform/markers/common/markers', 'vs/platform/telemetry/common/telemetry', 'vs/platform/configuration/common/configuration', 'vs/platform/files/common/files', 'vs/platform/instantiation/common/instantiation', 'vs/platform/extensions/common/extensions', 'vs/editor/common/services/modeService', 'vs/editor/common/services/modelService', 'vs/platform/jsonschemas/common/jsonContributionRegistry', 'vs/workbench/common/contributions', 'vs/workbench/common/actionRegistry', 'vs/workbench/browser/parts/statusbar/statusbar', 'vs/workbench/browser/quickopen', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/workspace/common/contextService', 'vs/workbench/parts/lib/node/systemVariables', 'vs/workbench/parts/files/common/files', 'vs/workbench/parts/output/common/output', 'vs/workbench/parts/tasks/common/taskSystem', 'vs/workbench/parts/tasks/common/taskService', 'vs/workbench/parts/tasks/common/taskTemplates', 'vs/workbench/parts/tasks/common/languageServiceTaskSystem', 'vs/workbench/parts/tasks/node/processRunnerSystem', 'vs/workbench/parts/tasks/node/processRunnerDetector', 'vs/css!./media/task.contribution', 'vs/workbench/parts/tasks/browser/taskQuickOpen'], function (require, exports, nls, Env, winjs_base_1, severity_1, Objects, actions_1, Dom, lifecycle_1, eventEmitter_1, Builder, Types, keyCodes_1, glob_1, platform_1, platform_2, lifecycle_2, actions_2, extensions_1, event_1, message_1, markers_1, telemetry_1, configuration_1, files_1, instantiation_1, extensions_2, modeService_1, modelService_1, jsonContributionRegistry, contributions_1, actionRegistry_1, statusbar_1, quickopen_1, quickOpenService_1, editorService_1, contextService_1, systemVariables_1, files_2, output_1, taskSystem_1, taskService_1, taskTemplates_1, languageServiceTaskSystem_1, processRunnerSystem_1, processRunnerDetector_1) {
    'use strict';
    var $ = Builder.$;
    var AbstractTaskAction = (function (_super) {
        __extends(AbstractTaskAction, _super);
        function AbstractTaskAction(id, label, taskService, telemetryService) {
            _super.call(this, id, label);
            this.taskService = taskService;
            this.telemetryService = telemetryService;
        }
        AbstractTaskAction = __decorate([
            __param(2, taskService_1.ITaskService),
            __param(3, telemetry_1.ITelemetryService)
        ], AbstractTaskAction);
        return AbstractTaskAction;
    }(actions_1.Action));
    var BuildAction = (function (_super) {
        __extends(BuildAction, _super);
        function BuildAction(id, label, taskService, telemetryService) {
            _super.call(this, id, label, taskService, telemetryService);
        }
        BuildAction.prototype.run = function () {
            return this.taskService.build();
        };
        BuildAction.ID = 'workbench.action.tasks.build';
        BuildAction.TEXT = nls.localize(0, null);
        BuildAction = __decorate([
            __param(2, taskService_1.ITaskService),
            __param(3, telemetry_1.ITelemetryService)
        ], BuildAction);
        return BuildAction;
    }(AbstractTaskAction));
    var TestAction = (function (_super) {
        __extends(TestAction, _super);
        function TestAction(id, label, taskService, telemetryService) {
            _super.call(this, id, label, taskService, telemetryService);
        }
        TestAction.prototype.run = function () {
            return this.taskService.runTest();
        };
        TestAction.ID = 'workbench.action.tasks.test';
        TestAction.TEXT = nls.localize(1, null);
        TestAction = __decorate([
            __param(2, taskService_1.ITaskService),
            __param(3, telemetry_1.ITelemetryService)
        ], TestAction);
        return TestAction;
    }(AbstractTaskAction));
    var RebuildAction = (function (_super) {
        __extends(RebuildAction, _super);
        function RebuildAction(id, label, taskService, telemetryService) {
            _super.call(this, id, label, taskService, telemetryService);
        }
        RebuildAction.prototype.run = function () {
            return this.taskService.rebuild();
        };
        RebuildAction.ID = 'workbench.action.tasks.rebuild';
        RebuildAction.TEXT = nls.localize(2, null);
        RebuildAction = __decorate([
            __param(2, taskService_1.ITaskService),
            __param(3, telemetry_1.ITelemetryService)
        ], RebuildAction);
        return RebuildAction;
    }(AbstractTaskAction));
    var CleanAction = (function (_super) {
        __extends(CleanAction, _super);
        function CleanAction(id, label, taskService, telemetryService) {
            _super.call(this, id, label, taskService, telemetryService);
        }
        CleanAction.prototype.run = function () {
            return this.taskService.clean();
        };
        CleanAction.ID = 'workbench.action.tasks.clean';
        CleanAction.TEXT = nls.localize(3, null);
        CleanAction = __decorate([
            __param(2, taskService_1.ITaskService),
            __param(3, telemetry_1.ITelemetryService)
        ], CleanAction);
        return CleanAction;
    }(AbstractTaskAction));
    var ConfigureTaskRunnerAction = (function (_super) {
        __extends(ConfigureTaskRunnerAction, _super);
        function ConfigureTaskRunnerAction(id, label, configurationService, editorService, fileService, contextService, outputService, messageService, quickOpenService) {
            _super.call(this, id, label);
            this.configurationService = configurationService;
            this.editorService = editorService;
            this.fileService = fileService;
            this.contextService = contextService;
            this.outputService = outputService;
            this.messageService = messageService;
            this.quickOpenService = quickOpenService;
        }
        ConfigureTaskRunnerAction.prototype.run = function (event) {
            var _this = this;
            if (!this.contextService.getWorkspace()) {
                this.messageService.show(severity_1.default.Info, nls.localize(5, null));
                return winjs_base_1.TPromise.as(undefined);
            }
            var sideBySide = !!(event && (event.ctrlKey || event.metaKey));
            return this.fileService.resolveFile(this.contextService.toResource('.vscode/tasks.json')).then(function (success) {
                return success;
            }, function (err) {
                ;
                return _this.quickOpenService.pick(taskTemplates_1.templates, { placeHolder: nls.localize(6, null) }).then(function (selection) {
                    if (!selection) {
                        return undefined;
                    }
                    var contentPromise;
                    if (selection.autoDetect) {
                        _this.outputService.showOutput(TaskService.OutputChannel);
                        _this.outputService.append(TaskService.OutputChannel, nls.localize(7, null, selection.id) + '\n');
                        var detector = new processRunnerDetector_1.ProcessRunnerDetector(_this.fileService, _this.contextService, new systemVariables_1.SystemVariables(_this.editorService, _this.contextService));
                        contentPromise = detector.detect(false, selection.id).then(function (value) {
                            var config = value.config;
                            if (value.stderr && value.stderr.length > 0) {
                                value.stderr.forEach(function (line) {
                                    _this.outputService.append(TaskService.OutputChannel, line + '\n');
                                });
                                _this.messageService.show(severity_1.default.Warning, nls.localize(8, null));
                                return selection.content;
                            }
                            else if (config) {
                                if (value.stdout && value.stdout.length > 0) {
                                    value.stdout.forEach(function (line) { return _this.outputService.append(TaskService.OutputChannel, line + '\n'); });
                                }
                                var content = JSON.stringify(config, null, '\t');
                                content = [
                                    '{',
                                    '\t// See http://go.microsoft.com/fwlink/?LinkId=733558',
                                    '\t// for the documentation about the tasks.json format',
                                ].join('\n') + content.substr(1);
                                return content;
                            }
                            else {
                                return selection.content;
                            }
                        });
                    }
                    else {
                        contentPromise = winjs_base_1.TPromise.as(selection.content);
                    }
                    return contentPromise.then(function (content) {
                        return _this.fileService.createFile(_this.contextService.toResource('.vscode/tasks.json'), content);
                    });
                });
            }).then(function (stat) {
                if (!stat) {
                    return undefined;
                }
                // // (2) Open editor with configuration file
                return _this.editorService.openEditor({
                    resource: stat.resource,
                    options: {
                        forceOpen: true
                    }
                }, sideBySide);
            }, function (error) {
                throw new Error(nls.localize(9, null));
            });
        };
        ConfigureTaskRunnerAction.ID = 'workbench.action.tasks.configureTaskRunner';
        ConfigureTaskRunnerAction.TEXT = nls.localize(4, null);
        ConfigureTaskRunnerAction = __decorate([
            __param(2, configuration_1.IConfigurationService),
            __param(3, editorService_1.IWorkbenchEditorService),
            __param(4, files_1.IFileService),
            __param(5, contextService_1.IWorkspaceContextService),
            __param(6, output_1.IOutputService),
            __param(7, message_1.IMessageService),
            __param(8, quickOpenService_1.IQuickOpenService)
        ], ConfigureTaskRunnerAction);
        return ConfigureTaskRunnerAction;
    }(actions_1.Action));
    var CloseMessageAction = (function (_super) {
        __extends(CloseMessageAction, _super);
        function CloseMessageAction() {
            _super.call(this, CloseMessageAction.ID, CloseMessageAction.TEXT);
        }
        CloseMessageAction.prototype.run = function () {
            if (this.closeFunction) {
                this.closeFunction();
            }
            return winjs_base_1.TPromise.as(null);
        };
        CloseMessageAction.ID = 'workbench.action.build.closeMessage';
        CloseMessageAction.TEXT = nls.localize(10, null);
        return CloseMessageAction;
    }(actions_1.Action));
    var TerminateAction = (function (_super) {
        __extends(TerminateAction, _super);
        function TerminateAction(id, label, taskService, telemetryService) {
            _super.call(this, id, label, taskService, telemetryService);
        }
        TerminateAction.prototype.run = function () {
            var _this = this;
            return this.taskService.isActive().then(function (active) {
                if (active) {
                    return _this.taskService.terminate().then(function (response) {
                        if (response.success) {
                            return;
                        }
                        else {
                            return winjs_base_1.Promise.wrapError(nls.localize(12, null));
                        }
                    });
                }
            });
        };
        TerminateAction.ID = 'workbench.action.tasks.terminate';
        TerminateAction.TEXT = nls.localize(11, null);
        TerminateAction = __decorate([
            __param(2, taskService_1.ITaskService),
            __param(3, telemetry_1.ITelemetryService)
        ], TerminateAction);
        return TerminateAction;
    }(AbstractTaskAction));
    var ShowLogAction = (function (_super) {
        __extends(ShowLogAction, _super);
        function ShowLogAction(id, label, taskService, telemetryService, outputService) {
            _super.call(this, id, label, taskService, telemetryService);
            this.outputService = outputService;
        }
        ShowLogAction.prototype.run = function () {
            return this.outputService.showOutput(TaskService.OutputChannel);
        };
        ShowLogAction.ID = 'workbench.action.tasks.showLog';
        ShowLogAction.TEXT = nls.localize(13, null);
        ShowLogAction = __decorate([
            __param(2, taskService_1.ITaskService),
            __param(3, telemetry_1.ITelemetryService),
            __param(4, output_1.IOutputService)
        ], ShowLogAction);
        return ShowLogAction;
    }(AbstractTaskAction));
    var RunTaskAction = (function (_super) {
        __extends(RunTaskAction, _super);
        function RunTaskAction(id, label, quickOpenService) {
            _super.call(this, id, label);
            this.quickOpenService = quickOpenService;
        }
        RunTaskAction.prototype.run = function (event) {
            this.quickOpenService.show('task ');
            return winjs_base_1.TPromise.as(null);
        };
        RunTaskAction.ID = 'workbench.action.tasks.runTask';
        RunTaskAction.TEXT = nls.localize(14, null);
        RunTaskAction = __decorate([
            __param(2, quickOpenService_1.IQuickOpenService)
        ], RunTaskAction);
        return RunTaskAction;
    }(actions_1.Action));
    var StatusBarItem = (function () {
        function StatusBarItem(quickOpenService, markerService, outputService, taskService) {
            this.quickOpenService = quickOpenService;
            this.markerService = markerService;
            this.outputService = outputService;
            this.taskService = taskService;
            this.activeCount = 0;
        }
        StatusBarItem.prototype.render = function (container) {
            var _this = this;
            var callOnDispose = [], element = document.createElement('div'), 
            // icon = document.createElement('a'),
            progress = document.createElement('div'), label = document.createElement('a'), error = document.createElement('div'), warning = document.createElement('div'), info = document.createElement('div');
            Dom.addClass(element, 'task-statusbar-item');
            // dom.addClass(icon, 'task-statusbar-item-icon');
            // element.appendChild(icon);
            Dom.addClass(progress, 'task-statusbar-item-progress');
            element.appendChild(progress);
            progress.innerHTML = StatusBarItem.progressChars[0];
            $(progress).hide();
            Dom.addClass(label, 'task-statusbar-item-label');
            element.appendChild(label);
            Dom.addClass(error, 'task-statusbar-item-label-error');
            error.innerHTML = '0';
            label.appendChild(error);
            Dom.addClass(warning, 'task-statusbar-item-label-warning');
            warning.innerHTML = '0';
            label.appendChild(warning);
            Dom.addClass(info, 'task-statusbar-item-label-info');
            label.appendChild(info);
            $(info).hide();
            //		callOnDispose.push(dom.addListener(icon, 'click', (e:MouseEvent) => {
            //			this.outputService.showOutput(TaskService.OutputChannel, e.ctrlKey || e.metaKey, true);
            //		}));
            callOnDispose.push(Dom.addDisposableListener(label, 'click', function (e) {
                _this.quickOpenService.show('!');
            }));
            var updateStatus = function (element, stats) {
                if (stats > 0) {
                    element.innerHTML = stats.toString();
                    $(element).show();
                    return true;
                }
                else {
                    $(element).hide();
                    return false;
                }
            };
            var manyMarkers = nls.localize(15, null);
            var updateLabel = function (stats) {
                error.innerHTML = stats.errors < 100 ? stats.errors.toString() : manyMarkers;
                warning.innerHTML = stats.warnings < 100 ? stats.warnings.toString() : manyMarkers;
                updateStatus(info, stats.infos);
            };
            this.markerService.onMarkerChanged(function (changedResources) {
                updateLabel(_this.markerService.getStatistics());
            });
            callOnDispose.push(this.taskService.addListener2(taskService_1.TaskServiceEvents.Active, function () {
                _this.activeCount++;
                if (_this.activeCount === 1) {
                    var index_1 = 1;
                    var chars_1 = StatusBarItem.progressChars;
                    progress.innerHTML = chars_1[0];
                    _this.intervalToken = setInterval(function () {
                        progress.innerHTML = chars_1[index_1];
                        index_1++;
                        if (index_1 >= chars_1.length) {
                            index_1 = 0;
                        }
                    }, 50);
                    $(progress).show();
                }
            }));
            callOnDispose.push(this.taskService.addListener2(taskService_1.TaskServiceEvents.Inactive, function (data) {
                _this.activeCount--;
                if (_this.activeCount === 0) {
                    $(progress).hide();
                    clearInterval(_this.intervalToken);
                    _this.intervalToken = null;
                }
            }));
            callOnDispose.push(this.taskService.addListener2(taskService_1.TaskServiceEvents.Terminated, function () {
                if (_this.activeCount !== 0) {
                    $(progress).hide();
                    if (_this.intervalToken) {
                        clearInterval(_this.intervalToken);
                        _this.intervalToken = null;
                    }
                    _this.activeCount = 0;
                }
            }));
            container.appendChild(element);
            return {
                dispose: function () {
                    callOnDispose = lifecycle_1.disposeAll(callOnDispose);
                }
            };
        };
        StatusBarItem.progressChars = '|/-\\';
        StatusBarItem = __decorate([
            __param(0, quickOpenService_1.IQuickOpenService),
            __param(1, markers_1.IMarkerService),
            __param(2, output_1.IOutputService),
            __param(3, taskService_1.ITaskService)
        ], StatusBarItem);
        return StatusBarItem;
    }());
    var TaskService = (function (_super) {
        __extends(TaskService, _super);
        function TaskService(modeService, configurationService, markerService, outputService, messageService, editorService, fileService, contextService, telemetryService, textFileService, lifecycleService, eventService, modelService, extensionService, quickOpenService) {
            var _this = this;
            _super.call(this);
            this.serviceId = taskService_1.ITaskService;
            this.modeService = modeService;
            this.configurationService = configurationService;
            this.markerService = markerService;
            this.outputService = outputService;
            this.messageService = messageService;
            this.editorService = editorService;
            this.fileService = fileService;
            this.contextService = contextService;
            this.telemetryService = telemetryService;
            this.textFileService = textFileService;
            this.eventService = eventService;
            this.modelService = modelService;
            this.extensionService = extensionService;
            this.quickOpenService = quickOpenService;
            this.taskSystemListeners = [];
            this.clearTaskSystemPromise = false;
            this.configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function () {
                _this.emit(taskService_1.TaskServiceEvents.ConfigChanged);
                if (_this._taskSystem && _this._taskSystem.isActiveSync()) {
                    _this.clearTaskSystemPromise = true;
                }
                else {
                    _this._taskSystem = null;
                    _this._taskSystemPromise = null;
                }
                _this.disposeTaskSystemListeners();
            });
            lifecycleService.addBeforeShutdownParticipant(this);
        }
        TaskService.prototype.disposeTaskSystemListeners = function () {
            this.taskSystemListeners.forEach(function (unbind) { return unbind(); });
            this.taskSystemListeners = [];
        };
        TaskService.prototype.disposeFileChangesListener = function () {
            if (this.fileChangesListener) {
                this.fileChangesListener();
                this.fileChangesListener = null;
            }
        };
        Object.defineProperty(TaskService.prototype, "taskSystemPromise", {
            get: function () {
                var _this = this;
                if (!this._taskSystemPromise) {
                    var variables_1 = new systemVariables_1.SystemVariables(this.editorService, this.contextService);
                    var clearOutput_1 = true;
                    this._taskSystemPromise = this.configurationService.loadConfiguration('tasks').then(function (config) {
                        var parseErrors = config ? config.$parseErrors : null;
                        if (parseErrors) {
                            var isAffected = false;
                            for (var i = 0; i < parseErrors.length; i++) {
                                if (/tasks\.json$/.test(parseErrors[i])) {
                                    isAffected = true;
                                    break;
                                }
                            }
                            if (isAffected) {
                                _this.outputService.append(TaskService.OutputChannel, nls.localize(16, null));
                                _this.outputService.showOutput(TaskService.OutputChannel, true);
                                return winjs_base_1.TPromise.wrapError({});
                            }
                        }
                        var configPromise;
                        if (config) {
                            if (_this.isRunnerConfig(config) && _this.hasDetectorSupport(config)) {
                                var fileConfig_1 = config;
                                configPromise = new processRunnerDetector_1.ProcessRunnerDetector(_this.fileService, _this.contextService, variables_1, fileConfig_1).detect(true).then(function (value) {
                                    clearOutput_1 = _this.printStderr(value.stderr);
                                    var detectedConfig = value.config;
                                    if (!detectedConfig) {
                                        return config;
                                    }
                                    var result = Objects.clone(fileConfig_1);
                                    var configuredTasks = Object.create(null);
                                    if (!result.tasks) {
                                        if (detectedConfig.tasks) {
                                            result.tasks = detectedConfig.tasks;
                                        }
                                    }
                                    else {
                                        result.tasks.forEach(function (task) { return configuredTasks[task.taskName] = task; });
                                        detectedConfig.tasks.forEach(function (task) {
                                            if (!configuredTasks[task.taskName]) {
                                                result.tasks.push(task);
                                            }
                                        });
                                    }
                                    return result;
                                });
                            }
                            else {
                                configPromise = winjs_base_1.TPromise.as(config);
                            }
                        }
                        else {
                            configPromise = new processRunnerDetector_1.ProcessRunnerDetector(_this.fileService, _this.contextService, variables_1).detect(true).then(function (value) {
                                clearOutput_1 = _this.printStderr(value.stderr);
                                return value.config;
                            });
                        }
                        return configPromise.then(function (config) {
                            if (!config) {
                                _this._taskSystemPromise = null;
                                throw new taskSystem_1.TaskError(severity_1.default.Info, nls.localize(17, null), taskSystem_1.TaskErrors.NotConfigured);
                            }
                            var result = null;
                            if (config.buildSystem === 'service') {
                                result = new languageServiceTaskSystem_1.LanguageServiceTaskSystem(config, _this.telemetryService, _this.modeService);
                            }
                            else if (_this.isRunnerConfig(config)) {
                                result = new processRunnerSystem_1.ProcessRunnerSystem(config, variables_1, _this.markerService, _this.modelService, _this.telemetryService, _this.outputService, TaskService.OutputChannel, clearOutput_1);
                            }
                            if (result === null) {
                                _this._taskSystemPromise = null;
                                throw new taskSystem_1.TaskError(severity_1.default.Info, nls.localize(18, null), taskSystem_1.TaskErrors.NoValidTaskRunner);
                            }
                            _this.taskSystemListeners.push(result.addListener(taskSystem_1.TaskSystemEvents.Active, function (event) { return _this.emit(taskService_1.TaskServiceEvents.Active, event); }));
                            _this.taskSystemListeners.push(result.addListener(taskSystem_1.TaskSystemEvents.Inactive, function (event) { return _this.emit(taskService_1.TaskServiceEvents.Inactive, event); }));
                            _this._taskSystem = result;
                            return result;
                        }, function (err) {
                            _this.handleError(err);
                            return winjs_base_1.Promise.wrapError(err);
                        });
                    });
                }
                return this._taskSystemPromise;
            },
            enumerable: true,
            configurable: true
        });
        TaskService.prototype.printStderr = function (stderr) {
            var _this = this;
            var result = true;
            if (stderr && stderr.length > 0) {
                stderr.forEach(function (line) {
                    result = false;
                    _this.outputService.append(TaskService.OutputChannel, line + '\n');
                });
                this.outputService.showOutput(TaskService.OutputChannel, true);
            }
            return result;
        };
        TaskService.prototype.isRunnerConfig = function (config) {
            return !config.buildSystem || config.buildSystem === 'program';
        };
        TaskService.prototype.hasDetectorSupport = function (config) {
            if (!config.command) {
                return false;
            }
            return processRunnerDetector_1.ProcessRunnerDetector.supports(config.command);
        };
        TaskService.prototype.configureAction = function () {
            return new ConfigureTaskRunnerAction(ConfigureTaskRunnerAction.ID, ConfigureTaskRunnerAction.TEXT, this.configurationService, this.editorService, this.fileService, this.contextService, this.outputService, this.messageService, this.quickOpenService);
        };
        TaskService.prototype.build = function () {
            return this.executeTarget(function (taskSystem) { return taskSystem.build(); });
        };
        TaskService.prototype.rebuild = function () {
            return this.executeTarget(function (taskSystem) { return taskSystem.rebuild(); });
        };
        TaskService.prototype.clean = function () {
            return this.executeTarget(function (taskSystem) { return taskSystem.clean(); });
        };
        TaskService.prototype.runTest = function () {
            return this.executeTarget(function (taskSystem) { return taskSystem.runTest(); });
        };
        TaskService.prototype.run = function (taskIdentifier) {
            return this.executeTarget(function (taskSystem) { return taskSystem.run(taskIdentifier); });
        };
        TaskService.prototype.executeTarget = function (fn) {
            var _this = this;
            return this.textFileService.saveAll().then(function (value) {
                return _this.taskSystemPromise.
                    then(function (taskSystem) {
                    return taskSystem.isActive().then(function (active) {
                        if (!active) {
                            return fn(taskSystem);
                        }
                        else {
                            throw new taskSystem_1.TaskError(severity_1.default.Warning, nls.localize(19, null), taskSystem_1.TaskErrors.RunningTask);
                        }
                    });
                }).
                    then(function (runResult) {
                    if (runResult.restartOnFileChanges) {
                        var pattern_1 = runResult.restartOnFileChanges;
                        _this.fileChangesListener = _this.eventService.addListener(files_1.EventType.FILE_CHANGES, function (event) {
                            var needsRestart = event.changes.some(function (change) {
                                return (change.type === files_1.FileChangeType.ADDED || change.type === files_1.FileChangeType.DELETED) && !!glob_1.match(pattern_1, change.resource.fsPath);
                            });
                            if (needsRestart) {
                                _this.terminate().done(function () {
                                    // We need to give the child process a change to stop.
                                    platform_1.setTimeout(function () {
                                        _this.executeTarget(fn);
                                    }, 2000);
                                });
                            }
                        });
                    }
                    return runResult.promise.then(function (value) {
                        if (_this.clearTaskSystemPromise) {
                            _this._taskSystemPromise = null;
                            _this.clearTaskSystemPromise = false;
                        }
                        return value;
                    });
                }, function (err) {
                    _this.handleError(err);
                });
            });
        };
        TaskService.prototype.isActive = function () {
            if (this._taskSystemPromise) {
                return this.taskSystemPromise.then(function (taskSystem) { return taskSystem.isActive(); });
            }
            return winjs_base_1.TPromise.as(false);
        };
        TaskService.prototype.terminate = function () {
            var _this = this;
            if (this._taskSystemPromise) {
                return this.taskSystemPromise.then(function (taskSystem) {
                    return taskSystem.terminate();
                }).then(function (response) {
                    if (response.success) {
                        if (_this.clearTaskSystemPromise) {
                            _this._taskSystemPromise = null;
                            _this.clearTaskSystemPromise = false;
                        }
                        _this.emit(taskService_1.TaskServiceEvents.Terminated, {});
                        _this.disposeFileChangesListener();
                    }
                    return response;
                });
            }
            return winjs_base_1.TPromise.as({ success: true });
        };
        TaskService.prototype.tasks = function () {
            return this.taskSystemPromise.then(function (taskSystem) { return taskSystem.tasks(); });
        };
        TaskService.prototype.beforeShutdown = function () {
            var _this = this;
            if (this._taskSystem && this._taskSystem.isActiveSync()) {
                if (this._taskSystem.canAutoTerminate() || this.messageService.confirm({
                    message: nls.localize(20, null),
                    primaryButton: nls.localize(21, null)
                })) {
                    return this._taskSystem.terminate().then(function (response) {
                        if (response.success) {
                            _this.emit(taskService_1.TaskServiceEvents.Terminated, {});
                            _this._taskSystem = null;
                            _this.disposeFileChangesListener();
                            _this.disposeTaskSystemListeners();
                            return false; // no veto
                        }
                        return true; // veto
                    }, function (err) {
                        return true; // veto
                    });
                }
                else {
                    return true; // veto
                }
            }
            return false; // Nothing to do here
        };
        TaskService.prototype.handleError = function (err) {
            var showOutput = true;
            if (err instanceof taskSystem_1.TaskError) {
                var buildError = err;
                var needsConfig = buildError.code === taskSystem_1.TaskErrors.NotConfigured || buildError.code === taskSystem_1.TaskErrors.NoBuildTask || buildError.code === taskSystem_1.TaskErrors.NoTestTask;
                var needsTerminate = buildError.code === taskSystem_1.TaskErrors.RunningTask;
                if (needsConfig || needsTerminate) {
                    var closeAction = new CloseMessageAction();
                    var action = needsConfig
                        ? this.configureAction()
                        : new TerminateAction(TerminateAction.ID, TerminateAction.TEXT, this, this.telemetryService);
                    closeAction.closeFunction = this.messageService.show(buildError.severity, { message: buildError.message, actions: [closeAction, action] });
                }
                else {
                    this.messageService.show(buildError.severity, buildError.message);
                }
            }
            else if (err instanceof Error) {
                var error = err;
                this.messageService.show(severity_1.default.Error, error.message);
            }
            else if (Types.isString(err)) {
                this.messageService.show(severity_1.default.Error, err);
            }
            else {
                this.messageService.show(severity_1.default.Error, nls.localize(22, null));
            }
            if (showOutput) {
                this.outputService.showOutput(TaskService.OutputChannel, true);
            }
        };
        TaskService.SERVICE_ID = 'taskService';
        TaskService.OutputChannel = 'Tasks';
        TaskService = __decorate([
            __param(0, modeService_1.IModeService),
            __param(1, configuration_1.IConfigurationService),
            __param(2, markers_1.IMarkerService),
            __param(3, output_1.IOutputService),
            __param(4, message_1.IMessageService),
            __param(5, editorService_1.IWorkbenchEditorService),
            __param(6, files_1.IFileService),
            __param(7, contextService_1.IWorkspaceContextService),
            __param(8, telemetry_1.ITelemetryService),
            __param(9, files_2.ITextFileService),
            __param(10, lifecycle_2.ILifecycleService),
            __param(11, event_1.IEventService),
            __param(12, modelService_1.IModelService),
            __param(13, extensions_2.IExtensionService),
            __param(14, quickOpenService_1.IQuickOpenService)
        ], TaskService);
        return TaskService;
    }(eventEmitter_1.EventEmitter));
    var TaskServiceParticipant = (function () {
        function TaskServiceParticipant(instantiationService) {
            this.instantiationService = instantiationService;
            // Force loading the language worker service
            this.instantiationService.getInstance(taskService_1.ITaskService);
        }
        TaskServiceParticipant.prototype.getId = function () {
            return 'vs.taskService';
        };
        TaskServiceParticipant = __decorate([
            __param(0, instantiation_1.IInstantiationService)
        ], TaskServiceParticipant);
        return TaskServiceParticipant;
    }());
    exports.TaskServiceParticipant = TaskServiceParticipant;
    var tasksCategory = nls.localize(23, null);
    var workbenchActionsRegistry = platform_2.Registry.as(actionRegistry_1.Extensions.WorkbenchActions);
    workbenchActionsRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(ConfigureTaskRunnerAction, ConfigureTaskRunnerAction.ID, ConfigureTaskRunnerAction.TEXT), tasksCategory);
    if (Env.enableTasks) {
        // Task Service
        extensions_1.registerSingleton(taskService_1.ITaskService, TaskService);
        // Actions
        workbenchActionsRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(BuildAction, BuildAction.ID, BuildAction.TEXT, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_B }), tasksCategory);
        workbenchActionsRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(TestAction, TestAction.ID, TestAction.TEXT, { primary: keyCodes_1.KeyMod.CtrlCmd | keyCodes_1.KeyMod.Shift | keyCodes_1.KeyCode.KEY_T }), tasksCategory);
        // workbenchActionsRegistry.registerWorkbenchAction(new SyncActionDescriptor(RebuildAction, RebuildAction.ID, RebuildAction.TEXT), tasksCategory);
        // workbenchActionsRegistry.registerWorkbenchAction(new SyncActionDescriptor(CleanAction, CleanAction.ID, CleanAction.TEXT), tasksCategory);
        workbenchActionsRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(TerminateAction, TerminateAction.ID, TerminateAction.TEXT), tasksCategory);
        workbenchActionsRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(ShowLogAction, ShowLogAction.ID, ShowLogAction.TEXT), tasksCategory);
        workbenchActionsRegistry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(RunTaskAction, RunTaskAction.ID, RunTaskAction.TEXT), tasksCategory);
        // Register Quick Open
        platform_2.Registry.as(quickopen_1.Extensions.Quickopen).registerQuickOpenHandler(new quickopen_1.QuickOpenHandlerDescriptor('vs/workbench/parts/tasks/browser/taskQuickOpen', 'QuickOpenHandler', 'task ', nls.localize(24, null)));
        // Status bar
        var statusbarRegistry = platform_2.Registry.as(statusbar_1.Extensions.Statusbar);
        statusbarRegistry.registerStatusbarItem(new statusbar_1.StatusbarItemDescriptor(StatusBarItem, statusbar_1.StatusbarAlignment.LEFT, 50 /* Medium Priority */));
        // Output channel
        var outputChannelRegistry = platform_2.Registry.as(output_1.Extensions.OutputChannels);
        outputChannelRegistry.registerChannel(TaskService.OutputChannel);
        platform_2.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(TaskServiceParticipant);
        // tasks.json validation
        var schemaId = 'vscode://schemas/tasks';
        var schema = {
            'id': schemaId,
            'description': 'Task definition file',
            'type': 'object',
            'default': {
                'version': '0.1.0',
                'command': 'myCommand',
                'isShellCommand': false,
                'args': [],
                'showOutput': 'always',
                'tasks': [
                    {
                        'taskName': 'build',
                        'showOutput': 'silent',
                        'isBuildCommand': true,
                        'problemMatcher': ['$tsc', '$lessCompile']
                    }
                ]
            },
            'definitions': {
                'showOutputType': {
                    'type': 'string',
                    'enum': ['always', 'silent', 'never'],
                    'default': 'silent'
                },
                'patternType': {
                    'anyOf': [
                        {
                            'type': 'string',
                            'enum': ['$tsc', '$tsc-watch', '$msCompile', '$lessCompile', '$gulp-tsc', '$cpp', '$csc', '$vb', '$jshint', '$jshint-stylish', '$eslint-compact', '$eslint-stylish']
                        },
                        {
                            '$ref': '#/definitions/pattern'
                        },
                        {
                            'type': 'array',
                            'items': {
                                '$ref': '#/definitions/pattern'
                            }
                        }
                    ]
                },
                'pattern': {
                    'default': {
                        'regexp': '^([^\\\\s].*)\\\\((\\\\d+,\\\\d+)\\\\):\\\\s*(.*)$',
                        'file': 1,
                        'location': 2,
                        'message': 3
                    },
                    'additionalProperties': false,
                    'properties': {
                        'regexp': {
                            'type': 'string',
                            'description': nls.localize(25, null)
                        },
                        'file': {
                            'type': 'integer',
                            'description': nls.localize(26, null)
                        },
                        'location': {
                            'type': 'integer',
                            'description': nls.localize(27, null)
                        },
                        'line': {
                            'type': 'integer',
                            'description': nls.localize(28, null)
                        },
                        'column': {
                            'type': 'integer',
                            'description': nls.localize(29, null)
                        },
                        'endLine': {
                            'type': 'integer',
                            'description': nls.localize(30, null)
                        },
                        'endColumn': {
                            'type': 'integer',
                            'description': nls.localize(31, null)
                        },
                        'severity': {
                            'type': 'integer',
                            'description': nls.localize(32, null)
                        },
                        'code': {
                            'type': 'integer',
                            'description': nls.localize(33, null)
                        },
                        'message': {
                            'type': 'integer',
                            'description': nls.localize(34, null)
                        },
                        'loop': {
                            'type': 'boolean',
                            'description': nls.localize(35, null)
                        }
                    }
                },
                'problemMatcherType': {
                    'oneOf': [
                        {
                            'type': 'string',
                            'enum': ['$tsc', '$tsc-watch', '$msCompile', '$lessCompile', '$gulp-tsc', '$jshint', '$jshint-stylish', '$eslint-compact', '$eslint-stylish']
                        },
                        {
                            '$ref': '#/definitions/problemMatcher'
                        },
                        {
                            'type': 'array',
                            'items': {
                                'anyOf': [
                                    {
                                        '$ref': '#/definitions/problemMatcher'
                                    },
                                    {
                                        'type': 'string',
                                        'enum': ['$tsc', '$tsc-watch', '$msCompile', '$lessCompile', '$gulp-tsc', '$jshint', '$jshint-stylish', '$eslint-compact', '$eslint-stylish']
                                    }
                                ]
                            }
                        }
                    ]
                },
                'watchingPattern': {
                    'type': 'object',
                    'additionalProperties': false,
                    'properties': {
                        'regexp': {
                            'type': 'string',
                            'description': nls.localize(36, null)
                        },
                        'file': {
                            'type': 'integer',
                            'description': nls.localize(37, null)
                        },
                    }
                },
                'problemMatcher': {
                    'type': 'object',
                    'additionalProperties': false,
                    'properties': {
                        'base': {
                            'type': 'string',
                            'enum': ['$tsc', '$tsc-watch', '$msCompile', '$lessCompile', '$gulp-tsc', '$jshint', '$jshint-stylish', '$eslint-compact', '$eslint-stylish'],
                            'description': nls.localize(38, null)
                        },
                        'owner': {
                            'type': 'string',
                            'description': nls.localize(39, null)
                        },
                        'severity': {
                            'type': 'string',
                            'enum': ['error', 'warning', 'info'],
                            'description': nls.localize(40, null)
                        },
                        'applyTo': {
                            'type': 'string',
                            'enum': ['allDocuments', 'openDocuments', 'closedDocuments'],
                            'description': nls.localize(41, null)
                        },
                        'pattern': {
                            '$ref': '#/definitions/patternType',
                            'description': nls.localize(42, null)
                        },
                        'fileLocation': {
                            'oneOf': [
                                {
                                    'type': 'string',
                                    'enum': ['absolute', 'relative']
                                },
                                {
                                    'type': 'array',
                                    'items': {
                                        'type': 'string'
                                    }
                                }
                            ],
                            'description': nls.localize(43, null)
                        },
                        'watching': {
                            'type': 'object',
                            'additionalProperties': false,
                            'properties': {
                                'activeOnStart': {
                                    'type': 'boolean',
                                    'description': nls.localize(44, null)
                                },
                                'beginsPattern': {
                                    'oneOf': [
                                        {
                                            'type': 'string'
                                        },
                                        {
                                            'type': '#/definitions/watchingPattern'
                                        }
                                    ],
                                    'description': nls.localize(45, null)
                                },
                                'endsPattern': {
                                    'oneOf': [
                                        {
                                            'type': 'string'
                                        },
                                        {
                                            'type': '#/definitions/watchingPattern'
                                        }
                                    ],
                                    'description': nls.localize(46, null)
                                }
                            }
                        },
                        'watchedTaskBeginsRegExp': {
                            'type': 'string',
                            'description': nls.localize(47, null)
                        },
                        'watchedTaskEndsRegExp': {
                            'type': 'string',
                            'description': nls.localize(48, null)
                        }
                    }
                },
                'baseTaskRunnerConfiguration': {
                    'type': 'object',
                    'properties': {
                        'command': {
                            'type': 'string',
                            'description': nls.localize(49, null)
                        },
                        'isShellCommand': {
                            'type': 'boolean',
                            'default': true,
                            'description': nls.localize(50, null)
                        },
                        'args': {
                            'type': 'array',
                            'description': nls.localize(51, null),
                            'items': {
                                'type': 'string'
                            }
                        },
                        'options': {
                            'type': 'object',
                            'description': nls.localize(52, null),
                            'properties': {
                                'cwd': {
                                    'type': 'string',
                                    'description': nls.localize(53, null)
                                },
                                'env': {
                                    'type': 'object',
                                    'additionalProperties': {
                                        'type': 'string'
                                    },
                                    'description': nls.localize(54, null)
                                }
                            },
                            'additionalProperties': {
                                'type': ['string', 'array', 'object']
                            }
                        },
                        'showOutput': {
                            '$ref': '#/definitions/showOutputType',
                            'description': nls.localize(55, null)
                        },
                        'isWatching': {
                            'type': 'boolean',
                            'description': nls.localize(56, null),
                            'default': true
                        },
                        'promptOnClose': {
                            'type': 'boolean',
                            'description': nls.localize(57, null),
                            'default': false
                        },
                        'echoCommand': {
                            'type': 'boolean',
                            'description': nls.localize(58, null),
                            'default': true
                        },
                        'suppressTaskName': {
                            'type': 'boolean',
                            'description': nls.localize(59, null),
                            'default': true
                        },
                        'taskSelector': {
                            'type': 'string',
                            'description': nls.localize(60, null)
                        },
                        'problemMatcher': {
                            '$ref': '#/definitions/problemMatcherType',
                            'description': nls.localize(61, null)
                        },
                        'tasks': {
                            'type': 'array',
                            'description': nls.localize(62, null),
                            'items': {
                                'type': 'object',
                                '$ref': '#/definitions/taskDescription'
                            }
                        }
                    }
                },
                'taskDescription': {
                    'type': 'object',
                    'required': ['taskName'],
                    'additionalProperties': false,
                    'properties': {
                        'taskName': {
                            'type': 'string',
                            'description': nls.localize(63, null)
                        },
                        'args': {
                            'type': 'array',
                            'description': nls.localize(64, null),
                            'items': {
                                'type': 'string'
                            }
                        },
                        'suppressTaskName': {
                            'type': 'boolean',
                            'description': nls.localize(65, null),
                            'default': true
                        },
                        'showOutput': {
                            '$ref': '#/definitions/showOutputType',
                            'description': nls.localize(66, null)
                        },
                        'echoCommand': {
                            'type': 'boolean',
                            'description': nls.localize(67, null),
                            'default': true
                        },
                        'isWatching': {
                            'type': 'boolean',
                            'description': nls.localize(68, null),
                            'default': true
                        },
                        'isBuildCommand': {
                            'type': 'boolean',
                            'description': nls.localize(69, null),
                            'default': true
                        },
                        'isTestCommand': {
                            'type': 'boolean',
                            'description': nls.localize(70, null),
                            'default': true
                        },
                        'problemMatcher': {
                            '$ref': '#/definitions/problemMatcherType',
                            'description': nls.localize(71, null)
                        }
                    },
                    'defaultSnippets': [
                        {
                            'label': 'Empty task',
                            'body': {
                                'taskName': '{{taskName}}'
                            }
                        }
                    ]
                }
            },
            'allOf': [
                {
                    'type': 'object',
                    'required': ['version'],
                    'properties': {
                        'version': {
                            'type': 'string',
                            'enum': ['0.1.0'],
                            'description': nls.localize(72, null)
                        },
                        'windows': {
                            '$ref': '#/definitions/baseTaskRunnerConfiguration',
                            'description': nls.localize(73, null)
                        },
                        'osx': {
                            '$ref': '#/definitions/baseTaskRunnerConfiguration',
                            'description': nls.localize(74, null)
                        },
                        'linux': {
                            '$ref': '#/definitions/baseTaskRunnerConfiguration',
                            'description': nls.localize(75, null)
                        }
                    }
                },
                {
                    '$ref': '#/definitions/baseTaskRunnerConfiguration'
                }
            ]
        };
        var jsonRegistry = platform_2.Registry.as(jsonContributionRegistry.Extensions.JSONContribution);
        jsonRegistry.registerSchema(schemaId, schema);
        jsonRegistry.addSchemaFileAssociation('/.vscode/tasks.json', schemaId);
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/update/electron-browser/update.contribution", ["require", "exports", 'vs/nls!vs/workbench/parts/update/electron-browser/update.contribution', 'vs/platform/platform', 'vs/base/common/winjs.base', 'vs/workbench/common/contributions', 'vs/platform/storage/common/storage', 'vs/platform/workspace/common/workspace', 'vs/platform/message/common/message', 'vs/base/common/severity', 'vs/workbench/electron-browser/update', 'vs/base/common/actions', 'electron', 'semver'], function (require, exports, nls, platform_1, winjs_base_1, contributions_1, storage_1, workspace_1, message_1, severity_1, update_1, actions_1, electron_1, semver) {
    'use strict';
    var CloseAction = new actions_1.Action('close', nls.localize(0, null), '', true, function () { return null; });
    var ShowLicenseAction = function (licenseUrl) { return new actions_1.Action('update.showLicense', nls.localize(1, null), null, true, function () { electron_1.shell.openExternal(licenseUrl); return winjs_base_1.TPromise.as(null); }); };
    var UpdateContribution = (function () {
        function UpdateContribution(storageService, contextService, messageService) {
            var env = contextService.getConfiguration().env;
            var lastVersion = storageService.get(UpdateContribution.KEY, storage_1.StorageScope.GLOBAL, '');
            // was there an update?
            if (env.releaseNotesUrl && lastVersion && env.version !== lastVersion) {
                setTimeout(function () {
                    messageService.show(severity_1.default.Info, {
                        message: nls.localize(2, null, env.appName, env.version),
                        actions: [
                            CloseAction,
                            update_1.ShowReleaseNotesAction(env.releaseNotesUrl, true)
                        ]
                    });
                }, 0);
            }
            // should we show the new license?
            if (env.licenseUrl && lastVersion && semver.satisfies(lastVersion, '<1.0.0') && semver.satisfies(env.version, '>=1.0.0')) {
                setTimeout(function () {
                    messageService.show(severity_1.default.Info, {
                        message: nls.localize(3, null, env.appName, env.version),
                        actions: [
                            CloseAction,
                            ShowLicenseAction(env.licenseUrl)
                        ]
                    });
                }, 0);
            }
            storageService.store(UpdateContribution.KEY, env.version, storage_1.StorageScope.GLOBAL);
        }
        UpdateContribution.prototype.getId = function () { return 'vs.update'; };
        UpdateContribution.KEY = 'releaseNotes/lastVersion';
        UpdateContribution = __decorate([
            __param(0, storage_1.IStorageService),
            __param(1, workspace_1.IWorkspaceContextService),
            __param(2, message_1.IMessageService)
        ], UpdateContribution);
        return UpdateContribution;
    }());
    exports.UpdateContribution = UpdateContribution;
    platform_1.Registry.as(contributions_1.Extensions.Workbench)
        .registerWorkbenchContribution(UpdateContribution);
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/history/browser/history", ["require", "exports", 'vs/base/common/errors', 'vs/base/common/platform', 'vs/nls!vs/workbench/services/history/browser/history', 'vs/base/common/events', 'vs/workbench/common/editor', 'vs/workbench/browser/parts/editor/textEditor', 'vs/workbench/common/events', 'vs/workbench/services/history/common/history', 'vs/editor/common/core/selection'], function (require, exports, errors, platform, nls, events_1, editor_1, textEditor_1, events_2, history_1, selection_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * Stores the selection & view state of an editor and allows to compare it to other selection states.
     */
    var EditorState = (function () {
        function EditorState(_editorInput, _selection) {
            this._editorInput = _editorInput;
            this._selection = _selection;
            //
        }
        Object.defineProperty(EditorState.prototype, "editorInput", {
            get: function () {
                return this._editorInput;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditorState.prototype, "selection", {
            get: function () {
                return this._selection;
            },
            enumerable: true,
            configurable: true
        });
        EditorState.prototype.justifiesNewPushState = function (other) {
            if (!this._editorInput.matches(other._editorInput)) {
                // push different editor inputs
                return true;
            }
            if (!selection_1.Selection.isISelection(this._selection) || !selection_1.Selection.isISelection(other._selection)) {
                // unknown selections
                return true;
            }
            var liftedSelection = selection_1.Selection.liftSelection(this._selection);
            var liftedOtherSelection = selection_1.Selection.liftSelection(other._selection);
            if (Math.abs(liftedSelection.getStartPosition().lineNumber - liftedOtherSelection.getStartPosition().lineNumber) < EditorState.EDITOR_SELECTION_THRESHOLD) {
                // ignore selection changes in the range of EditorState.EDITOR_SELECTION_THRESHOLD lines
                return false;
            }
            return true;
        };
        EditorState.EDITOR_SELECTION_THRESHOLD = 5; // number of lines to move in editor to justify for new state
        return EditorState;
    }());
    exports.EditorState = EditorState;
    var BaseHistoryService = (function () {
        function BaseHistoryService(eventService, editorService, contextService) {
            var _this = this;
            this.eventService = eventService;
            this.editorService = editorService;
            this.contextService = contextService;
            this.toUnbind = [];
            // Window Title
            window.document.title = this.getWindowTitle(null);
            // Editor Input Changes
            this.toUnbind.push(this.eventService.addListener(events_2.EventType.EDITOR_INPUT_CHANGED, function (e) { return _this.onEditorInputChanged(e); }));
            // Editor Input State Changes
            this.toUnbind.push(this.eventService.addListener(events_2.EventType.EDITOR_INPUT_STATE_CHANGED, function (e) { return _this.onEditorInputStateChanged(e.editorInput); }));
            // Text Editor Selection Changes
            this.toUnbind.push(this.eventService.addListener(events_2.EventType.TEXT_EDITOR_SELECTION_CHANGED, function (event) { return _this.onTextEditorSelectionChanged(event); }));
        }
        BaseHistoryService.prototype.onEditorInputStateChanged = function (input) {
            // If an active editor is set, but is different from the one from the event, prevent update because the editor is not active.
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor && !input.matches(activeEditor.input)) {
                return;
            }
            // Calculate New Window Title
            this.updateWindowTitle(input);
        };
        BaseHistoryService.prototype.onTextEditorSelectionChanged = function (event) {
            // If an active editor is set, but is different from the one from the event, prevent update because the editor is not active.
            var editor = event.editor;
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor && editor && activeEditor !== editor) {
                return;
            }
            // Delegate to implementors
            this.handleEditorSelectionChangeEvent(event.editor);
        };
        BaseHistoryService.prototype.onEditorInputChanged = function (event) {
            this.onEditorEvent(event.editor);
        };
        BaseHistoryService.prototype.onEditorEvent = function (editor) {
            var input = editor ? editor.input : null;
            // If an active editor is set, but is different from the one from the event, prevent update because the editor is not active.
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor && editor && activeEditor !== editor) {
                return;
            }
            // Calculate New Window Title
            this.updateWindowTitle(input);
            // Delegate to implementors
            this.handleEditorInputChangeEvent(editor);
        };
        BaseHistoryService.prototype.updateWindowTitle = function (input) {
            var windowTitle = null;
            if (input && input.getName()) {
                windowTitle = this.getWindowTitle(input);
            }
            else {
                windowTitle = this.getWindowTitle(null);
            }
            window.document.title = windowTitle;
        };
        BaseHistoryService.prototype.getWindowTitle = function (input) {
            var title = this.doGetWindowTitle(input);
            // Extension Development Host gets a special title to identify itself
            if (this.contextService.getConfiguration().env.extensionDevelopmentPath) {
                return nls.localize(0, null, title);
            }
            return title;
        };
        BaseHistoryService.prototype.doGetWindowTitle = function (input) {
            var appName = this.contextService.getConfiguration().env.appName;
            var prefix = input && input.getName();
            if (prefix && input) {
                var status_1 = input.getStatus();
                if (status_1 && status_1.decoration && !platform.isMacintosh /* Mac has its own decoration in window */) {
                    prefix = nls.localize(1, null, status_1.decoration, prefix);
                }
            }
            var workspace = this.contextService.getWorkspace();
            if (workspace) {
                var wsName = workspace.name;
                if (prefix) {
                    if (platform.isMacintosh) {
                        return nls.localize(2, null, prefix, wsName); // Mac: do not append base title
                    }
                    return nls.localize(3, null, prefix, wsName, appName);
                }
                if (platform.isMacintosh) {
                    return wsName; // Mac: do not append base title
                }
                return nls.localize(4, null, wsName, appName);
            }
            if (prefix) {
                if (platform.isMacintosh) {
                    return prefix; // Mac: do not append base title
                }
                return nls.localize(5, null, prefix, appName);
            }
            return appName;
        };
        BaseHistoryService.prototype.findVisibleEditorPosition = function (input) {
            var activeEditor = this.editorService.getActiveEditor();
            if (activeEditor && input.matches(activeEditor.input)) {
                return activeEditor.position;
            }
            var editors = this.editorService.getVisibleEditors();
            for (var i = 0; i < editors.length; i++) {
                var editor = editors[i];
                if (editor !== activeEditor && input.matches(editor.input)) {
                    return editor.position;
                }
            }
            return null;
        };
        BaseHistoryService.prototype.dispose = function () {
            while (this.toUnbind.length) {
                this.toUnbind.pop()();
            }
        };
        return BaseHistoryService;
    }());
    exports.BaseHistoryService = BaseHistoryService;
    var HistoryService = (function (_super) {
        __extends(HistoryService, _super);
        function HistoryService(eventService, editorService, contextService, quickOpenService) {
            _super.call(this, eventService, editorService, contextService);
            this.serviceId = history_1.IHistoryService;
            this.quickOpenService = quickOpenService;
            this.index = -1;
        }
        Object.defineProperty(HistoryService.prototype, "stack", {
            get: function () {
                // Seed our stack from the persisted editor history
                if (!this._stack) {
                    this._stack = [];
                    var history_2 = this.quickOpenService.getEditorHistory();
                    for (var i = history_2.length - 1; i >= 0; i--) {
                        this.addToStack(history_2[i]);
                    }
                }
                return this._stack;
            },
            enumerable: true,
            configurable: true
        });
        HistoryService.prototype.forward = function () {
            if (this.stack.length > this.index + 1) {
                this.index++;
                this.navigate();
            }
        };
        HistoryService.prototype.back = function () {
            if (this.index > 0) {
                this.index--;
                this.navigate();
            }
        };
        HistoryService.prototype.navigate = function () {
            var _this = this;
            var state = this.stack[this.index];
            this.blockEditorEvent = true;
            this.editorService.openEditor(state.input, state.options, this.findVisibleEditorPosition(state.input)).done(function () {
                _this.blockEditorEvent = false;
            }, function (error) {
                _this.blockEditorEvent = false;
                errors.onUnexpectedError(error);
            });
        };
        HistoryService.prototype.handleEditorSelectionChangeEvent = function (editor) {
            this.handleEditorEvent(editor, true);
        };
        HistoryService.prototype.handleEditorInputChangeEvent = function (editor) {
            this.handleEditorEvent(editor, false);
        };
        HistoryService.prototype.handleEditorEvent = function (editor, storeSelection) {
            if (this.blockEditorEvent) {
                return; // while we open an editor due to a navigation, we do not want to update our stack
            }
            if (editor instanceof textEditor_1.BaseTextEditor && editor.input) {
                this.handleTextEditorEvent(editor, storeSelection);
                return;
            }
            this.currentFileEditorState = null; // at this time we have no active file editor view state
            if (editor && editor.input) {
                this.handleNonTextEditorEvent(editor);
            }
        };
        HistoryService.prototype.handleTextEditorEvent = function (editor, storeSelection) {
            var stateCandidate = new EditorState(editor.input, editor.getSelection());
            if (!this.currentFileEditorState || this.currentFileEditorState.justifiesNewPushState(stateCandidate)) {
                this.currentFileEditorState = stateCandidate;
                var options = void 0;
                if (storeSelection) {
                    options = new editor_1.TextEditorOptions();
                    options.selection(editor.getSelection().startLineNumber, editor.getSelection().startColumn);
                }
                this.addToStack(editor.input, options);
            }
        };
        HistoryService.prototype.handleNonTextEditorEvent = function (editor) {
            var currentStack = this.stack[this.index];
            if (currentStack && currentStack.input.matches(editor.input)) {
                return; // do not push same editor input again
            }
            this.addToStack(editor.input);
        };
        HistoryService.prototype.addToStack = function (input, options) {
            var _this = this;
            // Overwrite an entry in the stack if we have a matching input that comes
            // with editor options to indicate that this entry is more specific.
            var replace = false;
            if (this.stack[this.index]) {
                var currentEntry = this.stack[this.index];
                if (currentEntry.input.matches(input) && !currentEntry.options) {
                    replace = true;
                }
            }
            var entry = {
                input: input,
                options: options
            };
            // Replace at current position
            if (replace) {
                this.stack[this.index] = entry;
            }
            else {
                this.index++;
                this.stack.splice(this.index, 0, entry);
                // Check for limit
                if (this.stack.length > HistoryService.MAX_HISTORY_ITEMS) {
                    this.stack.shift(); // remove first
                    if (this.index > 0) {
                        this.index--;
                    }
                }
            }
            // Take out on dispose
            input.addOneTimeListener(events_1.EventType.DISPOSE, function () {
                _this.stack.forEach(function (e, i) {
                    if (e.input.matches(input)) {
                        _this.stack.splice(i, 1);
                        if (_this.index >= i) {
                            _this.index--; // reduce index if the element is before index
                        }
                    }
                });
            });
        };
        HistoryService.MAX_HISTORY_ITEMS = 200;
        return HistoryService;
    }(BaseHistoryService));
    exports.HistoryService = HistoryService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/browser/workbench", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/types', 'vs/base/common/lifecycle', 'vs/base/common/strings', 'vs/base/browser/dom', 'vs/base/browser/builder', 'vs/base/common/async', 'vs/base/common/assert', 'vs/base/common/timer', 'vs/base/common/errors', 'vs/platform/platform', 'vs/workbench/common/constants', 'vs/workbench/common/events', 'vs/workbench/common/contributions', 'vs/workbench/browser/parts/editor/baseEditor', 'vs/workbench/common/editor', 'vs/workbench/services/history/browser/history', 'vs/workbench/browser/parts/activitybar/activitybarPart', 'vs/workbench/browser/parts/editor/editorPart', 'vs/workbench/browser/parts/sidebar/sidebarPart', 'vs/workbench/browser/parts/panel/panelPart', 'vs/workbench/browser/parts/statusbar/statusbarPart', 'vs/workbench/browser/layout', 'vs/workbench/browser/actionBarRegistry', 'vs/workbench/browser/viewlet', 'vs/workbench/browser/panel', 'vs/workbench/browser/parts/quickopen/quickOpenController', 'vs/workbench/common/editor/diffEditorInput', 'vs/platform/instantiation/common/extensions', 'vs/platform/keybinding/browser/keybindingServiceImpl', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/workbench/services/editor/browser/editorService', 'vs/workbench/services/part/common/partService', 'vs/workbench/services/themes/common/themeService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/storage/common/storage', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/contextview/browser/contextView', 'vs/workbench/services/activity/common/activityService', 'vs/workbench/services/viewlet/common/viewletService', 'vs/workbench/services/panel/common/panelService', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/services/quickopen/common/quickOpenService', 'vs/workbench/services/history/common/history', 'vs/platform/event/common/event', 'vs/platform/lifecycle/common/lifecycle', 'vs/platform/message/common/message', 'vs/platform/telemetry/common/telemetry', 'vs/platform/thread/common/thread', 'vs/platform/extensions/common/extensions', 'vs/workbench/services/statusbar/common/statusbarService', 'vs/css!./media/workbench'], function (require, exports, winjs_base_1, types, lifecycle_1, strings, DOM, builder_1, async_1, assert, timer, errors, platform_1, constants_1, events_1, contributions_1, baseEditor_1, editor_1, history_1, activitybarPart_1, editorPart_1, sidebarPart_1, panelPart_1, statusbarPart_1, layout_1, actionBarRegistry_1, viewlet_1, panel_1, quickOpenController_1, diffEditorInput_1, extensions_1, keybindingServiceImpl_1, untitledEditorService_1, editorService_1, partService_1, themeService_1, contextService_1, storage_1, keybindingService_1, contextView_1, activityService_1, viewletService_1, panelService_1, editorService_2, quickOpenService_1, history_2, event_1, lifecycle_2, message_1, telemetry_1, thread_1, extensions_2, statusbarService_1) {
    'use strict';
    /**
     * The workbench creates and lays out all parts that make up the Monaco Workbench.
     */
    var Workbench = (function () {
        function Workbench(container, workspace, configuration, options, instantiationService) {
            var _this = this;
            this.serviceId = partService_1.IPartService;
            // Validate params
            this.validateParams(container, configuration, options);
            // If String passed in as container, try to find it in DOM
            if (types.isString(container)) {
                var element = builder_1.withElementById(container.toString());
                this.container = element.getHTMLElement();
            }
            else {
                this.container = container;
            }
            this.workbenchParams = {
                workspace: workspace,
                configuration: configuration,
                options: options || {},
                instantiationService: instantiationService
            };
            this.currentTheme = null;
            this.toDispose = [];
            this.toShutdown = [];
            this.editorBackgroundDelayer = new async_1.Delayer(50);
            this.creationPromise = new winjs_base_1.TPromise(function (c, e, p) {
                _this.creationPromiseComplete = c;
            });
        }
        Workbench.prototype.validateParams = function (container, configuration, options) {
            // Container
            assert.ok(container, 'Workbench requires a container to be created with');
            if (types.isString(container)) {
                var element = builder_1.withElementById(container.toString());
                assert.ok(element, strings.format('Can not find HTMLElement with id \'{0}\'.', container));
            }
        };
        /**
         * Starts the workbench and creates the HTML elements on the container. A workbench can only be started
         * once. Use the shutdown function to free up resources created by the workbench on startup.
         */
        Workbench.prototype.startup = function (callbacks) {
            var _this = this;
            assert.ok(!this.workbenchStarted, 'Can not start a workbench that was already started');
            assert.ok(!this.workbenchShutdown, 'Can not start a workbench that was shutdown');
            try {
                this.workbenchStarted = true;
                this.callbacks = callbacks;
                // Create Workbench
                this.createWorkbench();
                // Services
                this.initServices();
                if (this.callbacks && this.callbacks.onServicesCreated) {
                    this.callbacks.onServicesCreated();
                }
                // Register Listeners
                this.registerListeners();
                // Settings
                this.initSettings();
                // Create Workbench and Parts
                this.renderWorkbench();
                // Workbench Layout
                this.createWorkbenchLayout();
                // Register Emitters
                this.registerEmitters();
                // Load composits and editors in parallel
                var compositeAndEditorPromises = [];
                // Show default viewlet unless sidebar is hidden or we dont have a default viewlet
                var viewletRegistry = platform_1.Registry.as(viewlet_1.Extensions.Viewlets);
                var viewletId = viewletRegistry.getDefaultViewletId();
                if (!this.workbenchParams.configuration.env.isBuilt) {
                    viewletId = this.storageService.get(sidebarPart_1.SidebarPart.activeViewletSettingsKey, storage_1.StorageScope.WORKSPACE, viewletRegistry.getDefaultViewletId()); // help developers and restore last view
                }
                if (!this.sideBarHidden && !!viewletId) {
                    var viewletTimerEvent_1 = timer.start(timer.Topic.STARTUP, strings.format('Opening Viewlet: {0}', viewletId));
                    compositeAndEditorPromises.push(this.sidebarPart.openViewlet(viewletId, false).then(function () { return viewletTimerEvent_1.stop(); }));
                }
                var panelRegistry = platform_1.Registry.as(panel_1.Extensions.Panels);
                var panelId = this.storageService.get(panelPart_1.PanelPart.activePanelSettingsKey, storage_1.StorageScope.WORKSPACE, panelRegistry.getDefaultPanelId());
                if (!this.panelHidden && !!panelId) {
                    compositeAndEditorPromises.push(this.panelPart.openPanel(panelId, false));
                }
                // Check for configured options to open files on startup and resolve if any or open untitled for empty workbench
                var editorTimerEvent_1 = timer.start(timer.Topic.STARTUP, strings.format('Restoring Editor(s)'));
                var resolveEditorInputsPromise = winjs_base_1.TPromise.as(null);
                var options_1 = [];
                // Files to open, diff or create
                var wbopt = this.workbenchParams.options;
                if ((wbopt.filesToCreate && wbopt.filesToCreate.length) || (wbopt.filesToOpen && wbopt.filesToOpen.length) || (wbopt.filesToDiff && wbopt.filesToDiff.length)) {
                    var filesToCreate = wbopt.filesToCreate || [];
                    var filesToOpen_1 = wbopt.filesToOpen || [];
                    var filesToDiff_1 = wbopt.filesToDiff;
                    // Files to diff is exclusive
                    if (filesToDiff_1 && filesToDiff_1.length) {
                        resolveEditorInputsPromise = winjs_base_1.TPromise.join(filesToDiff_1.map(function (resourceInput) { return _this.editorService.inputToType(resourceInput); })).then(function (inputsToDiff) {
                            return [new diffEditorInput_1.DiffEditorInput(diffEditorInput_1.toDiffLabel(filesToDiff_1[0].resource, filesToDiff_1[1].resource, _this.contextService), null, inputsToDiff[0], inputsToDiff[1])];
                        });
                    }
                    else {
                        var inputs_1 = [];
                        // Files to create
                        inputs_1.push.apply(inputs_1, filesToCreate.map(function (resourceInput) { return _this.untitledEditorService.createOrGet(resourceInput.resource); }));
                        options_1.push.apply(options_1, filesToCreate.map(function (r) { return null; })); // fill empty options for files to create because we dont have options there
                        // Files to open
                        resolveEditorInputsPromise = winjs_base_1.TPromise.join(filesToOpen_1.map(function (resourceInput) { return _this.editorService.inputToType(resourceInput); })).then(function (inputsToOpen) {
                            inputs_1.push.apply(inputs_1, inputsToOpen);
                            options_1.push.apply(options_1, filesToOpen_1.map(function (resourceInput) { return editor_1.TextEditorOptions.from(resourceInput); }));
                            return inputs_1;
                        });
                    }
                }
                else if (!this.workbenchParams.workspace) {
                    resolveEditorInputsPromise = winjs_base_1.TPromise.as([this.untitledEditorService.createOrGet()]);
                }
                // Restore editor state (either from last session or with given inputs)
                compositeAndEditorPromises.push(resolveEditorInputsPromise.then(function (inputs) {
                    return _this.editorPart.restoreEditorState(inputs, options_1).then(function () {
                        _this.onEditorOpenedOrClosed(); // make sure we show the proper background in the editor area
                        editorTimerEvent_1.stop();
                    });
                }));
                // Flag workbench as created once done
                var workbenchDone_1 = function (error) {
                    _this.workbenchCreated = true;
                    _this.eventService.emit(events_1.EventType.WORKBENCH_CREATED);
                    _this.creationPromiseComplete(true);
                    if (_this.callbacks && _this.callbacks.onWorkbenchStarted) {
                        _this.callbacks.onWorkbenchStarted();
                    }
                    if (error) {
                        errors.onUnexpectedError(error);
                    }
                };
                winjs_base_1.TPromise.join(compositeAndEditorPromises).then(function () { return workbenchDone_1(); }, function (error) { return workbenchDone_1(error); });
            }
            catch (error) {
                // Print out error
                console.error(errors.toErrorMessage(error, true));
                // Rethrow
                throw error;
            }
        };
        Workbench.prototype.initServices = function () {
            this.instantiationService = this.workbenchParams.instantiationService;
            // Services we expect
            this.eventService = this.instantiationService.getInstance(event_1.IEventService);
            this.storageService = this.instantiationService.getInstance(storage_1.IStorageService);
            this.keybindingService = this.instantiationService.getInstance(keybindingService_1.IKeybindingService);
            this.contextService = this.instantiationService.getInstance(contextService_1.IWorkspaceContextService);
            this.telemetryService = this.instantiationService.getInstance(telemetry_1.ITelemetryService);
            var messageService = this.instantiationService.getInstance(message_1.IMessageService);
            if (this.keybindingService instanceof keybindingServiceImpl_1.AbstractKeybindingService) {
                this.keybindingService.setMessageService(messageService);
            }
            var threadService = this.instantiationService.getInstance(thread_1.IThreadService);
            this.instantiationService.getInstance(extensions_2.IExtensionService);
            this.lifecycleService = this.instantiationService.getInstance(lifecycle_2.ILifecycleService);
            this.toDispose.push(this.lifecycleService.onShutdown(this.shutdownComponents, this));
            var contextMenuService = this.instantiationService.getInstance(contextView_1.IContextMenuService);
            this.untitledEditorService = this.instantiationService.getInstance(untitledEditorService_1.IUntitledEditorService);
            // Services we contribute
            this.instantiationService.addSingleton(partService_1.IPartService, this);
            // Viewlet service (sidebar part)
            this.sidebarPart = new sidebarPart_1.SidebarPart(messageService, this.storageService, this.eventService, this.telemetryService, contextMenuService, this, this.keybindingService, constants_1.Identifiers.SIDEBAR_PART);
            this.toDispose.push(this.sidebarPart);
            this.toShutdown.push(this.sidebarPart);
            this.instantiationService.addSingleton(viewletService_1.IViewletService, this.sidebarPart);
            // Panel service (panel part)
            this.panelPart = new panelPart_1.PanelPart(messageService, this.storageService, this.eventService, this.telemetryService, contextMenuService, this, this.keybindingService, constants_1.Identifiers.PANEL_PART);
            this.toDispose.push(this.panelPart);
            this.toShutdown.push(this.panelPart);
            this.instantiationService.addSingleton(panelService_1.IPanelService, this.panelPart);
            // Activity service (activitybar part)
            this.activitybarPart = new activitybarPart_1.ActivitybarPart(this.sidebarPart, messageService, this.telemetryService, this.eventService, contextMenuService, this.keybindingService, constants_1.Identifiers.ACTIVITYBAR_PART);
            this.toDispose.push(this.activitybarPart);
            this.toShutdown.push(this.activitybarPart);
            this.instantiationService.addSingleton(activityService_1.IActivityService, this.activitybarPart);
            // Editor service (editor part)
            this.editorPart = new editorPart_1.EditorPart(messageService, this.eventService, this.telemetryService, this.storageService, this, constants_1.Identifiers.EDITOR_PART);
            this.toDispose.push(this.editorPart);
            this.toShutdown.push(this.editorPart);
            this.editorService = new editorService_1.WorkbenchEditorService(this.editorPart, this.untitledEditorService);
            this.instantiationService.addSingleton(editorService_2.IWorkbenchEditorService, this.editorService);
            // Quick open service (quick open controller)
            this.quickOpen = new quickOpenController_1.QuickOpenController(this.eventService, this.storageService, this.editorService, this.sidebarPart, messageService, this.telemetryService, this.contextService, this.keybindingService);
            this.toDispose.push(this.quickOpen);
            this.toShutdown.push(this.quickOpen);
            this.instantiationService.addSingleton(quickOpenService_1.IQuickOpenService, this.quickOpen);
            // Status bar
            this.statusbarPart = new statusbarPart_1.StatusbarPart(constants_1.Identifiers.STATUSBAR_PART);
            this.toDispose.push(this.statusbarPart);
            this.toShutdown.push(this.statusbarPart);
            this.instantiationService.addSingleton(statusbarService_1.IStatusbarService, this.statusbarPart);
            // History
            this.instantiationService.addSingleton(history_2.IHistoryService, new history_1.HistoryService(this.eventService, this.editorService, this.contextService, this.quickOpen));
            // a new way to contribute services...
            var contributedServices = extensions_1.getServices();
            for (var _i = 0, contributedServices_1 = contributedServices; _i < contributedServices_1.length; _i++) {
                var contributedService = contributedServices_1[_i];
                this.instantiationService.addSingleton(contributedService.id, contributedService.descriptor);
            }
            // Some services need to be set explicitly after all services are created
            threadService.setInstantiationService(this.instantiationService);
            this.telemetryService.setInstantiationService(this.instantiationService);
            messageService.setWorkbenchServices(this.quickOpen, this.statusbarPart);
            this.quickOpen.setInstantiationService(this.instantiationService);
            this.statusbarPart.setInstantiationService(this.instantiationService);
            this.activitybarPart.setInstantiationService(this.instantiationService);
            this.sidebarPart.setInstantiationService(this.instantiationService);
            this.panelPart.setInstantiationService(this.instantiationService);
            this.editorPart.setInstantiationService(this.instantiationService);
            this.untitledEditorService.setInstantiationService(this.instantiationService);
            this.editorService.setInstantiationService(this.instantiationService);
            // Set the some services to registries that have been created eagerly
            this.keybindingService.setInstantiationService(this.instantiationService);
            platform_1.Registry.as(actionBarRegistry_1.Extensions.Actionbar).setInstantiationService(this.instantiationService);
            platform_1.Registry.as(contributions_1.Extensions.Workbench).setInstantiationService(this.instantiationService);
            platform_1.Registry.as(baseEditor_1.Extensions.Editors).setInstantiationService(this.instantiationService);
        };
        Workbench.prototype.initSettings = function () {
            // Sidebar visibility
            this.sideBarHidden = this.storageService.getBoolean(Workbench.sidebarHiddenSettingKey, storage_1.StorageScope.WORKSPACE, false);
            if (!!this.workbenchParams.options.singleFileMode) {
                this.sideBarHidden = true; // we hide sidebar in single-file-mode
            }
            var viewletRegistry = platform_1.Registry.as(viewlet_1.Extensions.Viewlets);
            if (!viewletRegistry.getDefaultViewletId()) {
                this.sideBarHidden = true; // can only hide sidebar if we dont have a default viewlet id
            }
            // Panel part visibility
            var panelRegistry = platform_1.Registry.as(panel_1.Extensions.Panels);
            this.panelHidden = this.storageService.getBoolean(Workbench.panelHiddenSettingKey, storage_1.StorageScope.WORKSPACE, true);
            if (!!this.workbenchParams.options.singleFileMode || !panelRegistry.getDefaultPanelId()) {
                // we hide panel part in single-file-mode or if there is no default panel
                this.panelHidden = true;
            }
            // Sidebar position
            var rawPosition = this.storageService.get(Workbench.sidebarPositionSettingKey, storage_1.StorageScope.GLOBAL, 'left');
            this.sideBarPosition = (rawPosition === 'left') ? partService_1.Position.LEFT : partService_1.Position.RIGHT;
        };
        /**
         * Returns whether the workbench has been started.
         */
        Workbench.prototype.isStarted = function () {
            return this.workbenchStarted && !this.workbenchShutdown;
        };
        /**
         * Returns whether the workbench has been fully created.
         */
        Workbench.prototype.isCreated = function () {
            return this.workbenchCreated && this.workbenchStarted;
        };
        Workbench.prototype.joinCreation = function () {
            return this.creationPromise;
        };
        Workbench.prototype.hasFocus = function (part) {
            var activeElement = document.activeElement;
            if (!activeElement) {
                return false;
            }
            var container = null;
            switch (part) {
                case partService_1.Parts.ACTIVITYBAR_PART:
                    container = this.activitybarPart.getContainer();
                    break;
                case partService_1.Parts.SIDEBAR_PART:
                    container = this.sidebarPart.getContainer();
                    break;
                case partService_1.Parts.PANEL_PART:
                    container = this.panelPart.getContainer();
                    break;
                case partService_1.Parts.EDITOR_PART:
                    container = this.editorPart.getContainer();
                    break;
                case partService_1.Parts.STATUSBAR_PART:
                    if (!this.statusbarPart) {
                        return false; // could be disabled by options
                    }
                    container = this.statusbarPart.getContainer();
                    break;
            }
            return DOM.isAncestor(activeElement, container.getHTMLElement());
        };
        Workbench.prototype.isVisible = function (part) {
            if (part === partService_1.Parts.SIDEBAR_PART) {
                return !this.sideBarHidden;
            }
            if (part === partService_1.Parts.PANEL_PART) {
                return !this.panelHidden;
            }
            return true; // any other part cannot be hidden
        };
        Workbench.prototype.isSideBarHidden = function () {
            return this.sideBarHidden;
        };
        Workbench.prototype.setSideBarHidden = function (hidden, skipLayout) {
            this.sideBarHidden = hidden;
            // Adjust CSS
            if (hidden) {
                this.workbench.addClass('nosidebar');
            }
            else {
                this.workbench.removeClass('nosidebar');
            }
            // Layout
            if (!skipLayout) {
                this.workbenchLayout.layout(true);
            }
            // If sidebar becomes hidden, also hide the current active viewlet if any
            if (hidden && this.sidebarPart.getActiveViewlet()) {
                this.sidebarPart.hideActiveViewlet();
                // Pass Focus to Editor if Sidebar is now hidden
                var editor = this.editorPart.getActiveEditor();
                if (editor) {
                    editor.focus();
                }
            }
            else if (!hidden && !this.sidebarPart.getActiveViewlet()) {
                var registry = platform_1.Registry.as(viewlet_1.Extensions.Viewlets);
                var viewletToOpen = this.sidebarPart.getLastActiveViewletId() || registry.getDefaultViewletId();
                if (viewletToOpen) {
                    this.sidebarPart.openViewlet(viewletToOpen, true).done(null, errors.onUnexpectedError);
                }
            }
            // Remember in settings
            this.storageService.store(Workbench.sidebarHiddenSettingKey, hidden ? 'true' : 'false', storage_1.StorageScope.WORKSPACE);
        };
        Workbench.prototype.isPanelHidden = function () {
            return this.panelHidden;
        };
        Workbench.prototype.setPanelHidden = function (hidden, skipLayout) {
            this.panelHidden = hidden;
            // Layout
            if (!skipLayout) {
                this.workbenchLayout.layout(true);
            }
            // If panel part becomes hidden, also hide the current active panel if any
            if (hidden && this.panelPart.getActivePanel()) {
                this.panelPart.hideActivePanel();
                // Pass Focus to Editor if Panel part is now hidden
                var editor = this.editorPart.getActiveEditor();
                if (editor) {
                    editor.focus();
                }
            }
            else if (!hidden && !this.panelPart.getActivePanel()) {
                var registry = platform_1.Registry.as(panel_1.Extensions.Panels);
                var panelToOpen = this.panelPart.getLastActivePanelId() || registry.getDefaultPanelId();
                if (panelToOpen) {
                    this.panelPart.openPanel(panelToOpen, true).done(null, errors.onUnexpectedError);
                }
            }
            // Remember in settings
            this.storageService.store(Workbench.panelHiddenSettingKey, hidden ? 'true' : 'false', storage_1.StorageScope.WORKSPACE);
        };
        Workbench.prototype.getSideBarPosition = function () {
            return this.sideBarPosition;
        };
        Workbench.prototype.setSideBarPosition = function (position) {
            if (this.sideBarHidden) {
                this.setSideBarHidden(false, true /* Skip Layout */);
            }
            var newPositionValue = (position === partService_1.Position.LEFT) ? 'left' : 'right';
            var oldPositionValue = (this.sideBarPosition === partService_1.Position.LEFT) ? 'left' : 'right';
            this.sideBarPosition = position;
            // Adjust CSS
            this.activitybarPart.getContainer().removeClass(oldPositionValue);
            this.sidebarPart.getContainer().removeClass(oldPositionValue);
            this.activitybarPart.getContainer().addClass(newPositionValue);
            this.sidebarPart.getContainer().addClass(newPositionValue);
            // Layout
            this.workbenchLayout.layout(true);
            // Remember in settings
            this.storageService.store(Workbench.sidebarPositionSettingKey, position === partService_1.Position.LEFT ? 'left' : 'right', storage_1.StorageScope.GLOBAL);
        };
        /**
         * Frees up resources of the workbench. Can only be called once and only on a workbench that was started. With the
         * optional parameter "force", the workbench can be shutdown ignoring any workbench components that might prevent
         * shutdown for user interaction (e.g. a dirty editor waiting for save to occur).
         */
        Workbench.prototype.shutdown = function (force) {
            if (this.isStarted()) {
                if (!force) {
                    this.shutdownComponents();
                }
                // Event
                this.eventService.emit(events_1.EventType.WORKBENCH_DISPOSING);
                this.workbenchShutdown = true;
                // Dispose
                this.dispose();
            }
            return null;
        };
        Workbench.prototype.dispose = function () {
            // Dispose all
            this.toDispose = lifecycle_1.disposeAll(this.toDispose);
            // Event
            this.eventService.emit(events_1.EventType.WORKBENCH_DISPOSED);
        };
        /**
         * Asks the workbench and all its UI components inside to lay out according to
         * the containers dimension the workbench is living in.
         */
        Workbench.prototype.layout = function () {
            if (this.isStarted()) {
                this.workbenchLayout.layout();
            }
        };
        Workbench.prototype.shutdownComponents = function () {
            // Pass shutdown on to each participant
            this.toShutdown.forEach(function (s) { return s.shutdown(); });
        };
        Workbench.prototype.registerEmitters = function () {
            // Part Emitters
            this.hookPartListeners(this.activitybarPart);
            this.hookPartListeners(this.editorPart);
            this.hookPartListeners(this.sidebarPart);
            this.hookPartListeners(this.panelPart);
            // Storage Emitter
            this.toDispose.push(this.toDisposable(this.eventService.addEmitter(this.storageService)));
        };
        Workbench.prototype.hookPartListeners = function (part) {
            this.toDispose.push(this.toDisposable(this.eventService.addEmitter(part, part.getId())));
        };
        Workbench.prototype.registerListeners = function () {
            var _this = this;
            // Listen to Preference changes
            this.toDispose.push(this.toDisposable(this.eventService.addListener(storage_1.StorageEventType.STORAGE, function (e) {
                switch (e.key) {
                    case constants_1.Preferences.THEME:
                        _this.applyTheme(e.newValue);
                        break;
                }
            })));
            // Listen to editor changes
            this.toDispose.push(this.toDisposable(this.eventService.addListener(events_1.EventType.EDITOR_CLOSED, function () { return _this.onEditorOpenedOrClosed(); })));
            this.toDispose.push(this.toDisposable(this.eventService.addListener(events_1.EventType.EDITOR_OPENED, function () { return _this.onEditorOpenedOrClosed(); })));
        };
        Workbench.prototype.onEditorOpenedOrClosed = function () {
            var _this = this;
            var visibleEditors = this.editorService.getVisibleEditors().length;
            // We update the editorpart class to indicate if an editor is opened or not
            // through a delay to accomodate for fast editor switching
            if (visibleEditors === 0) {
                this.editorBackgroundDelayer.trigger(function () { return _this.editorPart.getContainer().addClass('empty'); });
            }
            else {
                this.editorBackgroundDelayer.trigger(function () { return _this.editorPart.getContainer().removeClass('empty'); });
            }
        };
        Workbench.prototype.toDisposable = function (fn) {
            return {
                dispose: function () {
                    fn();
                }
            };
        };
        Workbench.prototype.createWorkbenchLayout = function () {
            var options = new layout_1.LayoutOptions();
            options.setMargin(new builder_1.Box(0, 0, 0, 0));
            this.workbenchLayout = this.instantiationService.createInstance(layout_1.WorkbenchLayout, builder_1.$(this.container), // Parent
            this.workbench, // Workbench Container
            {
                activitybar: this.activitybarPart,
                editor: this.editorPart,
                sidebar: this.sidebarPart,
                panel: this.panelPart,
                statusbar: this.statusbarPart,
            }, this.quickOpen, // Quickopen
            options // Layout Options
            );
            this.toDispose.push(this.workbenchLayout);
        };
        Workbench.prototype.createWorkbench = function () {
            // Create Workbench DIV Off-DOM
            this.workbenchContainer = builder_1.$('.monaco-workbench-container');
            this.workbench = builder_1.$().div({ 'class': 'monaco-workbench', id: constants_1.Identifiers.WORKBENCH_CONTAINER }).appendTo(this.workbenchContainer);
        };
        Workbench.prototype.renderWorkbench = function () {
            // Apply sidebar state as CSS class
            if (this.sideBarHidden) {
                this.workbench.addClass('nosidebar');
            }
            // Apply readonly state as CSS class
            if (this.workbenchParams.options.readOnly) {
                this.workbench.addClass('readonly');
            }
            // Apply no-workspace state as CSS class
            if (!this.workbenchParams.workspace) {
                this.workbench.addClass('no-workspace');
            }
            // Create Parts
            this.createActivityBarPart();
            this.createSidebarPart();
            this.createEditorPart();
            this.createPanelPart();
            this.createStatusbarPart();
            // Create QuickOpen
            this.createQuickOpen();
            // Check theme in preferences
            var currentTheme = this.storageService.get(constants_1.Preferences.THEME, storage_1.StorageScope.GLOBAL, themeService_1.DEFAULT_THEME_ID);
            // Apply theme
            this.applyTheme(currentTheme);
            // Add Workbench to DOM
            this.workbenchContainer.build(this.container);
        };
        Workbench.prototype.applyTheme = function (theme) {
            if (this.currentTheme) {
                this.workbench.removeClass(this.currentTheme);
            }
            this.currentTheme = theme || null;
            if (this.currentTheme) {
                this.workbench.addClass(this.currentTheme);
            }
        };
        Workbench.prototype.createActivityBarPart = function () {
            var activitybarPartContainer = builder_1.$(this.workbench)
                .div({
                'class': ['part', 'activitybar', this.sideBarPosition === partService_1.Position.LEFT ? 'left' : 'right'],
                id: constants_1.Identifiers.ACTIVITYBAR_PART,
                role: 'navigation'
            });
            this.activitybarPart.create(activitybarPartContainer);
        };
        Workbench.prototype.createSidebarPart = function () {
            var sidebarPartContainer = builder_1.$(this.workbench)
                .div({
                'class': ['part', 'sidebar', this.sideBarPosition === partService_1.Position.LEFT ? 'left' : 'right'],
                id: constants_1.Identifiers.SIDEBAR_PART,
                role: 'complementary'
            });
            this.sidebarPart.create(sidebarPartContainer);
        };
        Workbench.prototype.createPanelPart = function () {
            var panelPartContainer = builder_1.$(this.workbench)
                .div({
                'class': ['part', 'panel', 'monaco-editor-background'],
                id: constants_1.Identifiers.PANEL_PART,
                role: 'complementary'
            });
            this.panelPart.create(panelPartContainer);
        };
        Workbench.prototype.createEditorPart = function () {
            var editorContainer = builder_1.$(this.workbench)
                .div({
                'class': ['part', 'editor', 'monaco-editor-background'],
                id: constants_1.Identifiers.EDITOR_PART,
                role: 'main'
            });
            this.editorPart.create(editorContainer);
        };
        Workbench.prototype.createStatusbarPart = function () {
            var statusbarContainer = builder_1.$(this.workbench).div({
                'class': ['part', 'statusbar'],
                id: constants_1.Identifiers.STATUSBAR_PART,
                role: 'contentinfo'
            });
            this.statusbarPart.create(statusbarContainer);
        };
        Workbench.prototype.createQuickOpen = function () {
            this.quickOpen.create();
        };
        Workbench.prototype.getEditorPart = function () {
            assert.ok(this.workbenchStarted, 'Workbench is not started. Call startup() first.');
            return this.editorPart;
        };
        Workbench.prototype.getSidebarPart = function () {
            assert.ok(this.workbenchStarted, 'Workbench is not started. Call startup() first.');
            return this.sidebarPart;
        };
        Workbench.prototype.getPanelPart = function () {
            assert.ok(this.workbenchStarted, 'Workbench is not started. Call startup() first.');
            return this.panelPart;
        };
        Workbench.prototype.getInstantiationService = function () {
            assert.ok(this.workbenchStarted, 'Workbench is not started. Call startup() first.');
            return this.instantiationService;
        };
        Workbench.prototype.addClass = function (clazz) {
            if (this.workbench) {
                this.workbench.addClass(clazz);
            }
        };
        Workbench.prototype.removeClass = function (clazz) {
            if (this.workbench) {
                this.workbench.removeClass(clazz);
            }
        };
        Workbench.sidebarPositionSettingKey = 'workbench.sidebar.position';
        Workbench.sidebarHiddenSettingKey = 'workbench.sidebar.hidden';
        Workbench.panelHiddenSettingKey = 'workbench.panel.hidden';
        return Workbench;
    }());
    exports.Workbench = Workbench;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/editor/node/textMate/TMSyntax", ["require", "exports", 'vs/nls!vs/editor/node/textMate/TMSyntax', 'vs/base/common/errors', 'vs/base/common/paths', 'vs/platform/extensions/common/extensionsRegistry', 'vs/editor/common/modes/TMState', 'vs/editor/common/modes/supports', 'vs/editor/common/services/modeService', 'vscode-textmate'], function (require, exports, nls, errors_1, paths, extensionsRegistry_1, TMState_1, supports_1, modeService_1, vscode_textmate_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var grammarsExtPoint = extensionsRegistry_1.ExtensionsRegistry.registerExtensionPoint('grammars', {
        description: nls.localize(0, null),
        type: 'array',
        defaultSnippets: [{ body: [{ id: '', extensions: [] }] }],
        items: {
            type: 'object',
            defaultSnippets: [{ body: { language: '{{id}}', scopeName: 'source.{{id}}', path: './syntaxes/{{id}}.tmLanguage.' } }],
            properties: {
                language: {
                    description: nls.localize(1, null),
                    type: 'string'
                },
                scopeName: {
                    description: nls.localize(2, null),
                    type: 'string'
                },
                path: {
                    description: nls.localize(3, null),
                    type: 'string'
                }
            }
        }
    });
    var MainProcessTextMateSyntax = (function () {
        function MainProcessTextMateSyntax(modeService) {
            var _this = this;
            this._modeService = modeService;
            this._grammarRegistry = new vscode_textmate_1.Registry({
                getFilePath: function (scopeName) {
                    return _this._scopeNameToFilePath[scopeName];
                }
            });
            this._scopeNameToFilePath = {};
            grammarsExtPoint.setHandler(function (extensions) {
                for (var i = 0; i < extensions.length; i++) {
                    var grammars = extensions[i].value;
                    for (var j = 0; j < grammars.length; j++) {
                        _this._handleGrammarExtensionPointUser(extensions[i].description.extensionFolderPath, grammars[j], extensions[i].collector);
                    }
                }
            });
        }
        MainProcessTextMateSyntax.prototype._handleGrammarExtensionPointUser = function (extensionFolderPath, syntax, collector) {
            var _this = this;
            if (syntax.language && ((typeof syntax.language !== 'string') || !this._modeService.isRegisteredMode(syntax.language))) {
                collector.error(nls.localize(4, null, grammarsExtPoint.name, String(syntax.language)));
                return;
            }
            if (!syntax.scopeName || (typeof syntax.scopeName !== 'string')) {
                collector.error(nls.localize(5, null, grammarsExtPoint.name, String(syntax.scopeName)));
                return;
            }
            if (!syntax.path || (typeof syntax.path !== 'string')) {
                collector.error(nls.localize(6, null, grammarsExtPoint.name, String(syntax.path)));
                return;
            }
            var normalizedAbsolutePath = paths.normalize(paths.join(extensionFolderPath, syntax.path));
            if (normalizedAbsolutePath.indexOf(extensionFolderPath) !== 0) {
                collector.warn(nls.localize(7, null, grammarsExtPoint.name, normalizedAbsolutePath, extensionFolderPath));
            }
            this._scopeNameToFilePath[syntax.scopeName] = normalizedAbsolutePath;
            var modeId = syntax.language;
            if (modeId) {
                var disposable_1 = this._modeService.onDidCreateMode(function (mode) {
                    if (mode.getId() !== modeId) {
                        return;
                    }
                    _this.registerDefinition(modeId, syntax.scopeName);
                    disposable_1.dispose();
                });
            }
        };
        MainProcessTextMateSyntax.prototype.registerDefinition = function (modeId, scopeName) {
            var _this = this;
            this._grammarRegistry.loadGrammar(scopeName, function (err, grammar) {
                if (err) {
                    errors_1.onUnexpectedError(err);
                    return;
                }
                _this._modeService.registerTokenizationSupport(modeId, function (mode) {
                    return createTokenizationSupport(mode, grammar);
                });
            });
        };
        MainProcessTextMateSyntax = __decorate([
            __param(0, modeService_1.IModeService)
        ], MainProcessTextMateSyntax);
        return MainProcessTextMateSyntax;
    }());
    exports.MainProcessTextMateSyntax = MainProcessTextMateSyntax;
    function createTokenizationSupport(mode, grammar) {
        var tokenizer = new Tokenizer(mode.getId(), grammar);
        return {
            shouldGenerateEmbeddedModels: false,
            getInitialState: function () { return new TMState_1.TMState(mode, null, null); },
            tokenize: function (line, state, offsetDelta, stopAtOffset) { return tokenizer.tokenize(line, state, offsetDelta, stopAtOffset); }
        };
    }
    var Tokenizer = (function () {
        function Tokenizer(modeId, grammar) {
            this._modeId = modeId;
            this._grammar = grammar;
        }
        Tokenizer.prototype.tokenize = function (line, state, offsetDelta, stopAtOffset) {
            if (offsetDelta === void 0) { offsetDelta = 0; }
            if (line.length >= 20000) {
                return {
                    tokens: [{
                            startIndex: offsetDelta,
                            type: ''
                        }],
                    actualStopOffset: offsetDelta,
                    endState: state,
                    modeTransitions: [{ startIndex: offsetDelta, mode: state.getMode() }],
                };
            }
            var freshState = state.clone();
            var textMateResult = this._grammar.tokenizeLine(line, freshState.getRuleStack());
            freshState.setRuleStack(textMateResult.ruleStack);
            // Create the result early and fill in the tokens later
            var ret = {
                tokens: [],
                actualStopOffset: offsetDelta + line.length,
                endState: freshState,
                modeTransitions: [{ startIndex: offsetDelta, mode: freshState.getMode() }],
            };
            var lastTokenType = null;
            for (var tokenIndex = 0, len = textMateResult.tokens.length; tokenIndex < len; tokenIndex++) {
                var token = textMateResult.tokens[tokenIndex];
                var tokenStartIndex = token.startIndex;
                var t = decodeTextMateToken(this._modeId, token);
                // do not push a new token if the type is exactly the same (also helps with ligatures)
                if (t.tokenType !== lastTokenType) {
                    ret.tokens.push(new supports_1.Token(tokenStartIndex + offsetDelta, t.tokenType));
                    lastTokenType = t.tokenType;
                }
            }
            return ret;
        };
        return Tokenizer;
    }());
    function decodeTextMateToken(modeId, entry) {
        var tokenTypeArray = [];
        for (var level = 1; level < entry.scopes.length; ++level) {
            tokenTypeArray = tokenTypeArray.concat(entry.scopes[level].split('.'));
        }
        var modeToken = '';
        if (entry.scopes.length > 0) {
            var dotIndex = entry.scopes[0].lastIndexOf('.');
            if (dotIndex >= 0) {
                modeToken = entry.scopes[0].substr(dotIndex + 1);
            }
        }
        var tokenTypes = [];
        dedupTokens(tokenTypeArray, modeToken, tokenTypes);
        return {
            tokenType: tokenTypes.join('.'),
            modeToken: modeId
        };
    }
    /**
     * Remove duplicate entries, collect result in `result`, place `modeToken` at the end
     * and detect if this is a comment => return true if it looks like a comment
     */
    function dedupTokens(tokenTypeArray, modeToken, result) {
        tokenTypeArray.sort();
        var prev = null, curr = null;
        for (var i = 0, len = tokenTypeArray.length; i < len; i++) {
            prev = curr;
            curr = tokenTypeArray[i];
            if (curr === prev || curr === modeToken) {
                continue;
            }
            result.push(curr);
        }
        result.push(modeToken);
    }
});

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostDocuments", ["require", "exports", 'vs/base/common/errors', 'vs/editor/common/services/modelService', 'vs/editor/common/editorCommon', 'vs/editor/common/model/mirrorModel2', 'vs/platform/thread/common/thread', 'vs/base/common/event', 'vs/base/common/uri', 'vs/base/common/lifecycle', 'vs/workbench/api/node/extHostTypes', 'vs/platform/event/common/event', 'vs/workbench/services/editor/common/editorService', 'vs/workbench/parts/files/common/files', './extHostTypeConverters', 'vs/base/common/winjs.base', 'vs/editor/common/model/textModelWithTokensHelpers', 'vs/platform/files/common/files', 'vs/editor/common/services/modeService', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/workbench/common/editor/resourceEditorInput', 'vs/base/common/async', 'weak'], function (require, exports, errors_1, modelService_1, EditorCommon, mirrorModel2_1, thread_1, event_1, uri_1, lifecycle_1, extHostTypes_1, event_2, editorService_1, files_1, TypeConverters, winjs_base_1, textModelWithTokensHelpers_1, files_2, modeService_1, untitledEditorService_1, resourceEditorInput_1, async_1, weak) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var _modeId2WordDefinition = Object.create(null);
    function setWordDefinitionFor(modeId, wordDefinition) {
        _modeId2WordDefinition[modeId] = wordDefinition;
    }
    exports.setWordDefinitionFor = setWordDefinitionFor;
    function getWordDefinitionFor(modeId) {
        return _modeId2WordDefinition[modeId];
    }
    exports.getWordDefinitionFor = getWordDefinitionFor;
    var ExtHostModelService = (function () {
        function ExtHostModelService(threadService) {
            this._proxy = threadService.getRemotable(MainThreadDocuments);
            this._onDidAddDocumentEventEmitter = new event_1.Emitter();
            this.onDidAddDocument = this._onDidAddDocumentEventEmitter.event;
            this._onDidRemoveDocumentEventEmitter = new event_1.Emitter();
            this.onDidRemoveDocument = this._onDidRemoveDocumentEventEmitter.event;
            this._onDidChangeDocumentEventEmitter = new event_1.Emitter();
            this.onDidChangeDocument = this._onDidChangeDocumentEventEmitter.event;
            this._onDidSaveDocumentEventEmitter = new event_1.Emitter();
            this.onDidSaveDocument = this._onDidSaveDocumentEventEmitter.event;
            this._documentData = Object.create(null);
            this._documentLoader = Object.create(null);
            this._documentContentProviders = Object.create(null);
        }
        ExtHostModelService.prototype.getAllDocumentData = function () {
            var result = [];
            for (var key in this._documentData) {
                result.push(this._documentData[key]);
            }
            return result;
        };
        ExtHostModelService.prototype.getDocumentData = function (resource) {
            if (!resource) {
                return;
            }
            var data = this._documentData[resource.toString()];
            if (data) {
                return data;
            }
        };
        ExtHostModelService.prototype.ensureDocumentData = function (uri) {
            var _this = this;
            var cached = this._documentData[uri.toString()];
            if (cached) {
                return winjs_base_1.TPromise.as(cached);
            }
            var promise = this._documentLoader[uri.toString()];
            if (!promise) {
                promise = this._proxy._tryOpenDocument(uri).then(function () {
                    delete _this._documentLoader[uri.toString()];
                    return _this._documentData[uri.toString()];
                }, function (err) {
                    delete _this._documentLoader[uri.toString()];
                    return winjs_base_1.TPromise.wrapError(err);
                });
                this._documentLoader[uri.toString()] = promise;
            }
            return promise;
        };
        ExtHostModelService.prototype.registerTextDocumentContentProvider = function (scheme, provider) {
            var _this = this;
            if (scheme === 'file' || scheme === 'untitled') {
                throw new Error("scheme '" + scheme + "' already registered");
            }
            var handle = ExtHostModelService._handlePool++;
            this._documentContentProviders[handle] = provider;
            this._proxy.$registerTextContentProvider(handle, scheme);
            var subscription;
            if (typeof provider.onDidChange === 'function') {
                subscription = provider.onDidChange(function (uri) {
                    if (_this._documentData[uri.toString()]) {
                        _this.$provideTextDocumentContent(handle, uri).then(function (value) {
                            return _this._proxy.$onVirtualDocumentChange(uri, value);
                        }, errors_1.onUnexpectedError);
                    }
                });
            }
            return new extHostTypes_1.Disposable(function () {
                if (delete _this._documentContentProviders[handle]) {
                    _this._proxy.$unregisterTextContentProvider(handle);
                }
                if (subscription) {
                    subscription.dispose();
                    subscription = undefined;
                }
            });
        };
        ExtHostModelService.prototype.$provideTextDocumentContent = function (handle, uri) {
            var provider = this._documentContentProviders[handle];
            if (!provider) {
                return winjs_base_1.TPromise.wrapError("unsupported uri-scheme: " + uri.scheme);
            }
            return async_1.asWinJsPromise(function (token) { return provider.provideTextDocumentContent(uri, token); });
        };
        ExtHostModelService.prototype.$isDocumentReferenced = function (uri) {
            var key = uri.toString();
            var document = this._documentData[key];
            if (document) {
                return winjs_base_1.TPromise.as(document.isDocumentReferenced);
            }
        };
        ExtHostModelService.prototype._acceptModelAdd = function (initData) {
            var data = new ExtHostDocumentData(this._proxy, initData.url, initData.value.lines, initData.value.EOL, initData.modeId, initData.versionId, initData.isDirty);
            var key = data.document.uri.toString();
            if (this._documentData[key]) {
                throw new Error('Document `' + key + '` already exists.');
            }
            this._documentData[key] = data;
            this._onDidAddDocumentEventEmitter.fire(data.document);
        };
        ExtHostModelService.prototype._acceptModelModeChanged = function (strURL, oldModeId, newModeId) {
            var data = this._documentData[strURL];
            // Treat a mode change as a remove + add
            this._onDidRemoveDocumentEventEmitter.fire(data.document);
            data._acceptLanguageId(newModeId);
            this._onDidAddDocumentEventEmitter.fire(data.document);
        };
        ExtHostModelService.prototype._acceptModelSaved = function (strURL) {
            var data = this._documentData[strURL];
            data._acceptIsDirty(false);
            this._onDidSaveDocumentEventEmitter.fire(data.document);
        };
        ExtHostModelService.prototype._acceptModelDirty = function (strURL) {
            var document = this._documentData[strURL];
            document._acceptIsDirty(true);
        };
        ExtHostModelService.prototype._acceptModelReverted = function (strURL) {
            var document = this._documentData[strURL];
            document._acceptIsDirty(false);
        };
        ExtHostModelService.prototype._acceptModelRemoved = function (strURL) {
            if (!this._documentData[strURL]) {
                throw new Error('Document `' + strURL + '` does not exist.');
            }
            var data = this._documentData[strURL];
            delete this._documentData[strURL];
            this._onDidRemoveDocumentEventEmitter.fire(data.document);
            data.dispose();
        };
        ExtHostModelService.prototype._acceptModelChanged = function (strURL, events) {
            var data = this._documentData[strURL];
            data.onEvents(events);
            this._onDidChangeDocumentEventEmitter.fire({
                document: data.document,
                contentChanges: events.map(function (e) {
                    return {
                        range: TypeConverters.toRange(e.range),
                        rangeLength: e.rangeLength,
                        text: e.text
                    };
                })
            });
        };
        ExtHostModelService._handlePool = 0;
        ExtHostModelService = __decorate([
            thread_1.Remotable.ExtHostContext('ExtHostModelService'),
            __param(0, thread_1.IThreadService)
        ], ExtHostModelService);
        return ExtHostModelService;
    }());
    exports.ExtHostModelService = ExtHostModelService;
    var ExtHostDocumentData = (function (_super) {
        __extends(ExtHostDocumentData, _super);
        function ExtHostDocumentData(proxy, uri, lines, eol, languageId, versionId, isDirty) {
            _super.call(this, uri, lines, eol, versionId);
            this._proxy = proxy;
            this._languageId = languageId;
            this._isDirty = isDirty;
            this._textLines = [];
        }
        ExtHostDocumentData.prototype.dispose = function () {
            this._textLines.length = 0;
            this._isDirty = false;
            _super.prototype.dispose.call(this);
        };
        Object.defineProperty(ExtHostDocumentData.prototype, "document", {
            get: function () {
                // dereferences or creates the actual document for this
                // document data. keeps a weak reference only such that
                // we later when a document isn't needed anymore
                if (!this.isDocumentReferenced) {
                    var data_1 = this;
                    var doc = {
                        get uri() { return data_1._uri; },
                        get fileName() { return data_1._uri.fsPath; },
                        get isUntitled() { return data_1._uri.scheme !== 'file'; },
                        get languageId() { return data_1._languageId; },
                        get version() { return data_1._versionId; },
                        get isDirty() { return data_1._isDirty; },
                        save: function () { return data_1._proxy._trySaveDocument(data_1._uri); },
                        getText: function (range) { return range ? data_1._getTextInRange(range) : data_1.getText(); },
                        get lineCount() { return data_1._lines.length; },
                        lineAt: function (lineOrPos) { return data_1.lineAt(lineOrPos); },
                        offsetAt: function (pos) { return data_1.offsetAt(pos); },
                        positionAt: function (offset) { return data_1.positionAt(offset); },
                        validateRange: function (ran) { return data_1.validateRange(ran); },
                        validatePosition: function (pos) { return data_1.validatePosition(pos); },
                        getWordRangeAtPosition: function (pos) { return data_1.getWordRangeAtPosition(pos); }
                    };
                    this._documentRef = weak(doc);
                }
                return weak.get(this._documentRef);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtHostDocumentData.prototype, "isDocumentReferenced", {
            get: function () {
                return this._documentRef && !weak.isDead(this._documentRef);
            },
            enumerable: true,
            configurable: true
        });
        ExtHostDocumentData.prototype._acceptLanguageId = function (newLanguageId) {
            this._languageId = newLanguageId;
        };
        ExtHostDocumentData.prototype._acceptIsDirty = function (isDirty) {
            this._isDirty = isDirty;
        };
        ExtHostDocumentData.prototype._getTextInRange = function (_range) {
            var range = this.validateRange(_range);
            if (range.isEmpty) {
                return '';
            }
            if (range.isSingleLine) {
                return this._lines[range.start.line].substring(range.start.character, range.end.character);
            }
            var lineEnding = this._eol, startLineIndex = range.start.line, endLineIndex = range.end.line, resultLines = [];
            resultLines.push(this._lines[startLineIndex].substring(range.start.character));
            for (var i = startLineIndex + 1; i < endLineIndex; i++) {
                resultLines.push(this._lines[i]);
            }
            resultLines.push(this._lines[endLineIndex].substring(0, range.end.character));
            return resultLines.join(lineEnding);
        };
        ExtHostDocumentData.prototype.lineAt = function (lineOrPosition) {
            var line;
            if (lineOrPosition instanceof extHostTypes_1.Position) {
                line = lineOrPosition.line;
            }
            else if (typeof lineOrPosition === 'number') {
                line = lineOrPosition;
            }
            if (line < 0 || line >= this._lines.length) {
                throw new Error('Illegal value ' + line + ' for `line`');
            }
            var result = this._textLines[line];
            if (!result || result.lineNumber !== line || result.text !== this._lines[line]) {
                var text = this._lines[line];
                var firstNonWhitespaceCharacterIndex = /^(\s*)/.exec(text)[1].length;
                var range = new extHostTypes_1.Range(line, 0, line, text.length);
                var rangeIncludingLineBreak = new extHostTypes_1.Range(line, 0, line + 1, 0);
                result = Object.freeze({
                    lineNumber: line,
                    range: range,
                    rangeIncludingLineBreak: rangeIncludingLineBreak,
                    text: text,
                    firstNonWhitespaceCharacterIndex: firstNonWhitespaceCharacterIndex,
                    isEmptyOrWhitespace: firstNonWhitespaceCharacterIndex === text.length
                });
                this._textLines[line] = result;
            }
            return result;
        };
        ExtHostDocumentData.prototype.offsetAt = function (position) {
            position = this.validatePosition(position);
            this._ensureLineStarts();
            return this._lineStarts.getAccumulatedValue(position.line - 1) + position.character;
        };
        ExtHostDocumentData.prototype.positionAt = function (offset) {
            offset = Math.floor(offset);
            offset = Math.max(0, offset);
            this._ensureLineStarts();
            var out = { index: 0, remainder: 0 };
            this._lineStarts.getIndexOf(offset, out);
            var lineLength = this._lines[out.index].length;
            // Ensure we return a valid position
            return new extHostTypes_1.Position(out.index, Math.min(out.remainder, lineLength));
        };
        // ---- range math
        ExtHostDocumentData.prototype.validateRange = function (range) {
            if (!(range instanceof extHostTypes_1.Range)) {
                throw new Error('Invalid argument');
            }
            var start = this.validatePosition(range.start);
            var end = this.validatePosition(range.end);
            if (start === range.start && end === range.end) {
                return range;
            }
            return new extHostTypes_1.Range(start.line, start.character, end.line, end.character);
        };
        ExtHostDocumentData.prototype.validatePosition = function (position) {
            if (!(position instanceof extHostTypes_1.Position)) {
                throw new Error('Invalid argument');
            }
            var line = position.line, character = position.character;
            var hasChanged = false;
            if (line < 0) {
                line = 0;
                hasChanged = true;
            }
            if (line >= this._lines.length) {
                line = this._lines.length - 1;
                hasChanged = true;
            }
            if (character < 0) {
                character = 0;
                hasChanged = true;
            }
            var maxCharacter = this._lines[line].length;
            if (character > maxCharacter) {
                character = maxCharacter;
                hasChanged = true;
            }
            if (!hasChanged) {
                return position;
            }
            return new extHostTypes_1.Position(line, character);
        };
        ExtHostDocumentData.prototype.getWordRangeAtPosition = function (_position) {
            var position = this.validatePosition(_position);
            var wordAtText = textModelWithTokensHelpers_1.WordHelper._getWordAtText(position.character + 1, textModelWithTokensHelpers_1.WordHelper.ensureValidWordDefinition(getWordDefinitionFor(this._languageId)), this._lines[position.line], 0);
            if (wordAtText) {
                return new extHostTypes_1.Range(position.line, wordAtText.startColumn - 1, position.line, wordAtText.endColumn - 1);
            }
        };
        return ExtHostDocumentData;
    }(mirrorModel2_1.MirrorModel2));
    exports.ExtHostDocumentData = ExtHostDocumentData;
    var MainThreadDocuments = (function () {
        function MainThreadDocuments(threadService, modelService, modeService, eventService, textFileService, editorService, fileService, untitledEditorService) {
            var _this = this;
            this._modelService = modelService;
            this._modeService = modeService;
            this._textFileService = textFileService;
            this._editorService = editorService;
            this._fileService = fileService;
            this._untitledEditorService = untitledEditorService;
            this._proxy = threadService.getRemotable(ExtHostModelService);
            this._modelIsSynced = {};
            this._toDispose = [];
            modelService.onModelAdded(this._onModelAdded, this, this._toDispose);
            modelService.onModelRemoved(this._onModelRemoved, this, this._toDispose);
            modelService.onModelModeChanged(this._onModelModeChanged, this, this._toDispose);
            this._toDispose.push(eventService.addListener2(files_1.EventType.FILE_SAVED, function (e) {
                _this._proxy._acceptModelSaved(e.getAfter().resource.toString());
            }));
            this._toDispose.push(eventService.addListener2(files_1.EventType.FILE_REVERTED, function (e) {
                _this._proxy._acceptModelReverted(e.getAfter().resource.toString());
            }));
            this._toDispose.push(eventService.addListener2(files_1.EventType.FILE_DIRTY, function (e) {
                _this._proxy._acceptModelDirty(e.getAfter().resource.toString());
            }));
            var handle = setInterval(function () { return _this._runDocumentCleanup(); }, 30 * 1000);
            this._toDispose.push({ dispose: function () { clearInterval(handle); } });
            this._modelToDisposeMap = Object.create(null);
            this._resourceContentProvider = Object.create(null);
            this._virtualDocumentSet = Object.create(null);
        }
        MainThreadDocuments.prototype.dispose = function () {
            var _this = this;
            Object.keys(this._modelToDisposeMap).forEach(function (modelUrl) {
                _this._modelToDisposeMap[modelUrl].dispose();
            });
            this._modelToDisposeMap = Object.create(null);
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        MainThreadDocuments.prototype._onModelAdded = function (model) {
            var _this = this;
            // Same filter as in mainThreadEditors
            if (model.isTooLargeForHavingARichMode()) {
                // don't synchronize too large models
                return null;
            }
            var modelUrl = model.getAssociatedResource();
            this._modelIsSynced[modelUrl.toString()] = true;
            this._modelToDisposeMap[modelUrl.toString()] = model.addBulkListener2(function (events) { return _this._onModelEvents(modelUrl, events); });
            this._proxy._acceptModelAdd({
                url: model.getAssociatedResource(),
                versionId: model.getVersionId(),
                value: model.toRawText(),
                modeId: model.getMode().getId(),
                isDirty: this._textFileService.isDirty(modelUrl)
            });
        };
        MainThreadDocuments.prototype._onModelModeChanged = function (event) {
            var model = event.model, oldModeId = event.oldModeId;
            var modelUrl = model.getAssociatedResource();
            if (!this._modelIsSynced[modelUrl.toString()]) {
                return;
            }
            this._proxy._acceptModelModeChanged(model.getAssociatedResource().toString(), oldModeId, model.getMode().getId());
        };
        MainThreadDocuments.prototype._onModelRemoved = function (model) {
            var modelUrl = model.getAssociatedResource();
            if (!this._modelIsSynced[modelUrl.toString()]) {
                return;
            }
            delete this._modelIsSynced[modelUrl.toString()];
            this._modelToDisposeMap[modelUrl.toString()].dispose();
            delete this._modelToDisposeMap[modelUrl.toString()];
            this._proxy._acceptModelRemoved(modelUrl.toString());
        };
        MainThreadDocuments.prototype._onModelEvents = function (modelUrl, events) {
            var changedEvents = [];
            for (var i = 0, len = events.length; i < len; i++) {
                var e = events[i];
                switch (e.getType()) {
                    case EditorCommon.EventType.ModelContentChanged2:
                        changedEvents.push(e.getData());
                        break;
                }
            }
            if (changedEvents.length > 0) {
                this._proxy._acceptModelChanged(modelUrl.toString(), changedEvents);
            }
        };
        // --- from extension host process
        MainThreadDocuments.prototype._trySaveDocument = function (uri) {
            return this._textFileService.save(uri);
        };
        MainThreadDocuments.prototype._tryOpenDocument = function (uri) {
            if (!uri.scheme || !(uri.fsPath || uri.authority)) {
                return winjs_base_1.TPromise.wrapError("Invalid uri. Scheme and authority or path must be set.");
            }
            var promise;
            switch (uri.scheme) {
                case 'untitled':
                    promise = this._handleUnititledScheme(uri);
                    break;
                case 'file':
                default:
                    promise = this._handleAsResourceInput(uri);
                    break;
            }
            return promise.then(function (success) {
                if (!success) {
                    return winjs_base_1.TPromise.wrapError('cannot open ' + uri.toString());
                }
            }, function (err) {
                return winjs_base_1.TPromise.wrapError('cannot open ' + uri.toString() + '. Detail: ' + errors_1.toErrorMessage(err));
            });
        };
        MainThreadDocuments.prototype._handleAsResourceInput = function (uri) {
            return this._editorService.resolveEditorModel({ resource: uri }).then(function (model) {
                return !!model;
            });
        };
        MainThreadDocuments.prototype._handleUnititledScheme = function (uri) {
            var _this = this;
            var asFileUri = uri_1.default.file(uri.fsPath);
            return this._fileService.resolveFile(asFileUri).then(function (stats) {
                // don't create a new file ontop of an existing file
                return winjs_base_1.TPromise.wrapError('file already exists on disk');
            }, function (err) {
                var input = _this._untitledEditorService.createOrGet(asFileUri); // using file-uri makes it show in 'Working Files' section
                return input.resolve(true).then(function (model) {
                    if (input.getResource().toString() !== uri.toString()) {
                        throw new Error("expected URI " + uri.toString() + " BUT GOT " + input.getResource().toString());
                    }
                    return _this._proxy._acceptModelDirty(uri.toString()); // mark as dirty
                }).then(function () {
                    return true;
                });
            });
        };
        // --- virtual document logic
        MainThreadDocuments.prototype.$registerTextContentProvider = function (handle, scheme) {
            var _this = this;
            this._resourceContentProvider[handle] = resourceEditorInput_1.ResourceEditorInput.registerResourceContentProvider(scheme, {
                provideTextContent: function (uri) {
                    return _this._proxy.$provideTextDocumentContent(handle, uri).then(function (value) {
                        if (value) {
                            _this._virtualDocumentSet[uri.toString()] = true;
                            var firstLineText = value.substr(0, 1 + value.search(/\r?\n/));
                            var mode = _this._modeService.getOrCreateModeByFilenameOrFirstLine(uri.fsPath, firstLineText);
                            return _this._modelService.createModel(value, mode, uri);
                        }
                    });
                }
            });
        };
        MainThreadDocuments.prototype.$unregisterTextContentProvider = function (handle) {
            var registration = this._resourceContentProvider[handle];
            if (registration) {
                registration.dispose();
                delete this._resourceContentProvider[handle];
            }
        };
        MainThreadDocuments.prototype.$onVirtualDocumentChange = function (uri, value) {
            var model = this._modelService.getModel(uri);
            if (model) {
                model.setValue(value);
            }
        };
        MainThreadDocuments.prototype._runDocumentCleanup = function () {
            var _this = this;
            var toBeDisposed = [];
            winjs_base_1.TPromise.join(Object.keys(this._virtualDocumentSet).map(function (key) {
                var resource = uri_1.default.parse(key);
                return _this._proxy.$isDocumentReferenced(resource).then(function (referenced) {
                    if (!referenced) {
                        return _this._editorService.inputToType({ resource: resource }).then(function (input) {
                            if (!_this._editorService.isVisible(input, true)) {
                                toBeDisposed.push(resource);
                            }
                        });
                    }
                });
            })).then(function () {
                for (var _i = 0, toBeDisposed_1 = toBeDisposed; _i < toBeDisposed_1.length; _i++) {
                    var resource = toBeDisposed_1[_i];
                    _this._modelService.destroyModel(resource);
                    delete _this._virtualDocumentSet[resource.toString()];
                }
            }, errors_1.onUnexpectedError);
        };
        MainThreadDocuments = __decorate([
            thread_1.Remotable.MainContext('MainThreadDocuments'),
            __param(0, thread_1.IThreadService),
            __param(1, modelService_1.IModelService),
            __param(2, modeService_1.IModeService),
            __param(3, event_2.IEventService),
            __param(4, files_1.ITextFileService),
            __param(5, editorService_1.IWorkbenchEditorService),
            __param(6, files_2.IFileService),
            __param(7, untitledEditorService_1.IUntitledEditorService)
        ], MainThreadDocuments);
        return MainThreadDocuments;
    }());
    exports.MainThreadDocuments = MainThreadDocuments;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostEditors", ["require", "exports", 'vs/base/common/event', 'vs/base/common/lifecycle', 'vs/base/common/winjs.base', 'vs/platform/thread/common/thread', 'vs/workbench/api/node/extHostDocuments', './extHostTypes', 'vs/editor/common/editorCommon', 'vs/editor/common/services/codeEditorService', 'vs/workbench/services/editor/common/editorService', 'vs/editor/common/services/modelService', 'vs/workbench/api/node/mainThreadEditors', './extHostTypeConverters', 'vs/workbench/common/events', 'vs/platform/telemetry/common/telemetry', 'vs/platform/event/common/event', 'vs/base/common/arrays', 'vs/base/common/objects'], function (require, exports, event_1, lifecycle_1, winjs_base_1, thread_1, extHostDocuments_1, extHostTypes_1, editorCommon_1, codeEditorService_1, editorService_1, modelService_1, mainThreadEditors_1, TypeConverters, events_1, telemetry_1, event_2, arrays_1, objects_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExtHostEditors = (function () {
        function ExtHostEditors(threadService) {
            this._onDidChangeTextEditorSelection = new event_1.Emitter();
            this.onDidChangeTextEditorSelection = this._onDidChangeTextEditorSelection.event;
            this._onDidChangeTextEditorOptions = new event_1.Emitter();
            this.onDidChangeTextEditorOptions = this._onDidChangeTextEditorOptions.event;
            this._onDidChangeTextEditorViewColumn = new event_1.Emitter();
            this.onDidChangeTextEditorViewColumn = this._onDidChangeTextEditorViewColumn.event;
            this._modelService = threadService.getRemotable(extHostDocuments_1.ExtHostModelService);
            this._proxy = threadService.getRemotable(MainThreadEditors);
            this._onDidChangeActiveTextEditor = new event_1.Emitter();
            this._editors = Object.create(null);
            this._visibleEditorIds = [];
        }
        ExtHostEditors.prototype.getActiveTextEditor = function () {
            return this._activeEditorId && this._editors[this._activeEditorId];
        };
        ExtHostEditors.prototype.getVisibleTextEditors = function () {
            var _this = this;
            return this._visibleEditorIds.map(function (id) { return _this._editors[id]; });
        };
        Object.defineProperty(ExtHostEditors.prototype, "onDidChangeActiveTextEditor", {
            get: function () {
                return this._onDidChangeActiveTextEditor && this._onDidChangeActiveTextEditor.event;
            },
            enumerable: true,
            configurable: true
        });
        ExtHostEditors.prototype.showTextDocument = function (document, column, preserveFocus) {
            var _this = this;
            return this._proxy._tryShowTextDocument(document.uri, TypeConverters.fromViewColumn(column), preserveFocus).then(function (id) {
                var editor = _this._editors[id];
                if (editor) {
                    return editor;
                }
                else {
                    throw new Error('Failed to create editor with id: ' + id);
                }
            });
        };
        ExtHostEditors.prototype.createTextEditorDecorationType = function (options) {
            return new TextEditorDecorationType(this._proxy, options);
        };
        // --- called from main thread
        ExtHostEditors.prototype._acceptTextEditorAdd = function (data) {
            var document = this._modelService.getDocumentData(data.document);
            var newEditor = new ExtHostTextEditor(this._proxy, data.id, document, data.selections.map(TypeConverters.toSelection), data.options, TypeConverters.toViewColumn(data.editorPosition));
            this._editors[data.id] = newEditor;
        };
        ExtHostEditors.prototype._acceptOptionsChanged = function (id, opts) {
            var editor = this._editors[id];
            editor._acceptOptions(opts);
            this._onDidChangeTextEditorOptions.fire({
                textEditor: editor,
                options: opts
            });
        };
        ExtHostEditors.prototype._acceptSelectionsChanged = function (id, _selections) {
            var selections = _selections.map(TypeConverters.toSelection);
            var editor = this._editors[id];
            editor._acceptSelections(selections);
            this._onDidChangeTextEditorSelection.fire({
                textEditor: editor,
                selections: selections
            });
        };
        ExtHostEditors.prototype._acceptActiveEditorAndVisibleEditors = function (id, visibleIds) {
            this._visibleEditorIds = visibleIds;
            if (this._activeEditorId === id) {
                // nothing to do
                return;
            }
            this._activeEditorId = id;
            this._onDidChangeActiveTextEditor.fire(this.getActiveTextEditor());
        };
        ExtHostEditors.prototype._acceptEditorPositionData = function (data) {
            for (var id in data) {
                var textEditor = this._editors[id];
                var viewColumn = TypeConverters.toViewColumn(data[id]);
                if (textEditor.viewColumn !== viewColumn) {
                    textEditor._acceptViewColumn(viewColumn);
                    this._onDidChangeTextEditorViewColumn.fire({ textEditor: textEditor, viewColumn: viewColumn });
                }
            }
        };
        ExtHostEditors.prototype._acceptTextEditorRemove = function (id) {
            // make sure the removed editor is not visible
            var newVisibleEditors = this._visibleEditorIds.filter(function (visibleEditorId) { return visibleEditorId !== id; });
            if (this._activeEditorId === id) {
                // removing the current active editor
                this._acceptActiveEditorAndVisibleEditors(undefined, newVisibleEditors);
            }
            else {
                this._acceptActiveEditorAndVisibleEditors(this._activeEditorId, newVisibleEditors);
            }
            var editor = this._editors[id];
            editor.dispose();
            delete this._editors[id];
        };
        ExtHostEditors = __decorate([
            thread_1.Remotable.ExtHostContext('ExtHostEditors'),
            __param(0, thread_1.IThreadService)
        ], ExtHostEditors);
        return ExtHostEditors;
    }());
    exports.ExtHostEditors = ExtHostEditors;
    var TextEditorDecorationType = (function () {
        function TextEditorDecorationType(proxy, options) {
            this.key = 'TextEditorDecorationType' + (++TextEditorDecorationType.LAST_ID);
            this._proxy = proxy;
            this._proxy._registerTextEditorDecorationType(this.key, options);
        }
        TextEditorDecorationType.prototype.dispose = function () {
            this._proxy._removeTextEditorDecorationType(this.key);
        };
        TextEditorDecorationType.LAST_ID = 0;
        return TextEditorDecorationType;
    }());
    var TextEditorEdit = (function () {
        function TextEditorEdit(document) {
            this._documentVersionId = document.version;
            this._collectedEdits = [];
            this._setEndOfLine = 0;
        }
        TextEditorEdit.prototype.finalize = function () {
            return {
                documentVersionId: this._documentVersionId,
                edits: this._collectedEdits,
                setEndOfLine: this._setEndOfLine
            };
        };
        TextEditorEdit.prototype.replace = function (location, value) {
            var range = null;
            if (location instanceof extHostTypes_1.Position) {
                range = new extHostTypes_1.Range(location, location);
            }
            else if (location instanceof extHostTypes_1.Range) {
                range = location;
            }
            else if (location instanceof extHostTypes_1.Selection) {
                range = new extHostTypes_1.Range(location.start, location.end);
            }
            else {
                throw new Error('Unrecognized location');
            }
            this._collectedEdits.push({
                range: range,
                text: value,
                forceMoveMarkers: false
            });
        };
        TextEditorEdit.prototype.insert = function (location, value) {
            this._collectedEdits.push({
                range: new extHostTypes_1.Range(location, location),
                text: value,
                forceMoveMarkers: true
            });
        };
        TextEditorEdit.prototype.delete = function (location) {
            var range = null;
            if (location instanceof extHostTypes_1.Range) {
                range = location;
            }
            else if (location instanceof extHostTypes_1.Selection) {
                range = new extHostTypes_1.Range(location.start, location.end);
            }
            else {
                throw new Error('Unrecognized location');
            }
            this._collectedEdits.push({
                range: range,
                text: null,
                forceMoveMarkers: true
            });
        };
        TextEditorEdit.prototype.setEndOfLine = function (endOfLine) {
            if (endOfLine !== extHostTypes_1.EndOfLine.LF && endOfLine !== extHostTypes_1.EndOfLine.CRLF) {
                throw illegalArg('endOfLine');
            }
            this._setEndOfLine = endOfLine;
        };
        return TextEditorEdit;
    }());
    exports.TextEditorEdit = TextEditorEdit;
    function readonly(name, alt) {
        var message = "The property '" + name + "' is readonly.";
        if (alt) {
            message += " Use '" + alt + "' instead.";
        }
        return new Error(message);
    }
    function illegalArg(name) {
        return new Error("illgeal argument '" + name + "'");
    }
    function deprecated(name, message) {
        if (message === void 0) { message = 'Refer to the documentation for further details.'; }
        return function (target, key, descriptor) {
            var originalMethod = descriptor.value;
            descriptor.value = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i - 0] = arguments[_i];
                }
                console.warn("[Deprecation Warning] method '" + name + "' is deprecated and should no longer be used. " + message);
                return originalMethod.apply(this, args);
            };
            return descriptor;
        };
    }
    var ExtHostTextEditor = (function () {
        function ExtHostTextEditor(proxy, id, document, selections, options, viewColumn) {
            this._proxy = proxy;
            this._id = id;
            this._documentData = document;
            this._selections = selections;
            this._options = options;
            this._viewColumn = viewColumn;
        }
        ExtHostTextEditor.prototype.dispose = function () {
            this._documentData = null;
        };
        ExtHostTextEditor.prototype.show = function (column) {
            this._proxy._tryShowEditor(this._id, TypeConverters.fromViewColumn(column));
        };
        ExtHostTextEditor.prototype.hide = function () {
            this._proxy._tryHideEditor(this._id);
        };
        Object.defineProperty(ExtHostTextEditor.prototype, "document", {
            // ---- the document
            get: function () {
                return this._documentData.document;
            },
            set: function (value) {
                throw readonly('document');
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtHostTextEditor.prototype, "options", {
            // ---- options
            get: function () {
                return this._options;
            },
            set: function (value) {
                var _this = this;
                this._options = value;
                this._runOnProxy(function () {
                    return _this._proxy._trySetOptions(_this._id, _this._options);
                }, true);
            },
            enumerable: true,
            configurable: true
        });
        ExtHostTextEditor.prototype._acceptOptions = function (options) {
            this._options = options;
        };
        Object.defineProperty(ExtHostTextEditor.prototype, "viewColumn", {
            // ---- view column
            get: function () {
                return this._viewColumn;
            },
            set: function (value) {
                throw readonly('viewColumn');
            },
            enumerable: true,
            configurable: true
        });
        ExtHostTextEditor.prototype._acceptViewColumn = function (value) {
            this._viewColumn = value;
        };
        Object.defineProperty(ExtHostTextEditor.prototype, "selection", {
            // ---- selections
            get: function () {
                return this._selections && this._selections[0];
            },
            set: function (value) {
                if (!(value instanceof extHostTypes_1.Selection)) {
                    throw illegalArg('selection');
                }
                this._selections = [value];
                this._trySetSelection(true);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ExtHostTextEditor.prototype, "selections", {
            get: function () {
                return this._selections;
            },
            set: function (value) {
                if (!Array.isArray(value) || value.some(function (a) { return !(a instanceof extHostTypes_1.Selection); })) {
                    throw illegalArg('selections');
                }
                this._selections = value;
                this._trySetSelection(true);
            },
            enumerable: true,
            configurable: true
        });
        ExtHostTextEditor.prototype.setDecorations = function (decorationType, ranges) {
            var _this = this;
            this._runOnProxy(function () { return _this._proxy._trySetDecorations(_this._id, decorationType.key, TypeConverters.fromRangeOrRangeWithMessage(ranges)); }, true);
        };
        ExtHostTextEditor.prototype.revealRange = function (range, revealType) {
            var _this = this;
            this._runOnProxy(function () { return _this._proxy._tryRevealRange(_this._id, TypeConverters.fromRange(range), revealType || mainThreadEditors_1.TextEditorRevealType.Default); }, true);
        };
        ExtHostTextEditor.prototype._trySetSelection = function (silent) {
            var _this = this;
            var selection = this._selections.map(TypeConverters.fromSelection);
            return this._runOnProxy(function () { return _this._proxy._trySetSelections(_this._id, selection); }, silent);
        };
        ExtHostTextEditor.prototype._acceptSelections = function (selections) {
            this._selections = selections;
        };
        // ---- editing
        ExtHostTextEditor.prototype.edit = function (callback) {
            var edit = new TextEditorEdit(this._documentData.document);
            callback(edit);
            return this._applyEdit(edit);
        };
        ExtHostTextEditor.prototype._applyEdit = function (editBuilder) {
            var editData = editBuilder.finalize();
            // prepare data for serialization
            var edits = editData.edits.map(function (edit) {
                return {
                    range: TypeConverters.fromRange(edit.range),
                    text: edit.text,
                    forceMoveMarkers: edit.forceMoveMarkers
                };
            });
            return this._proxy._tryApplyEdits(this._id, editData.documentVersionId, edits, editData.setEndOfLine);
        };
        // ---- util
        ExtHostTextEditor.prototype._runOnProxy = function (callback, silent) {
            var _this = this;
            return callback().then(function () { return _this; }, function (err) {
                if (!silent) {
                    return winjs_base_1.TPromise.wrapError(silent);
                }
                console.warn(err);
            });
        };
        __decorate([
            deprecated('TextEditor.show')
        ], ExtHostTextEditor.prototype, "show", null);
        __decorate([
            deprecated('TextEditor.hide')
        ], ExtHostTextEditor.prototype, "hide", null);
        return ExtHostTextEditor;
    }());
    var MainThreadEditors = (function () {
        function MainThreadEditors(threadService, workbenchEditorService, telemetryService, editorService, eventService, modelService) {
            var _this = this;
            this._proxy = threadService.getRemotable(ExtHostEditors);
            this._workbenchEditorService = workbenchEditorService;
            this._telemetryService = telemetryService;
            this._toDispose = [];
            this._textEditorsListenersMap = Object.create(null);
            this._textEditorsMap = Object.create(null);
            this._activeTextEditor = null;
            this._visibleEditors = [];
            this._editorPositionData = null;
            this._editorTracker = new mainThreadEditors_1.MainThreadEditorsTracker(editorService, modelService);
            this._toDispose.push(this._editorTracker);
            this._toDispose.push(this._editorTracker.onTextEditorAdd(function (textEditor) { return _this._onTextEditorAdd(textEditor); }));
            this._toDispose.push(this._editorTracker.onTextEditorRemove(function (textEditor) { return _this._onTextEditorRemove(textEditor); }));
            this._toDispose.push(this._editorTracker.onDidUpdateTextEditors(function () { return _this._updateActiveAndVisibleTextEditors(); }));
            this._toDispose.push(this._editorTracker.onChangedFocusedTextEditor(function (focusedTextEditorId) { return _this._updateActiveAndVisibleTextEditors(); }));
            this._toDispose.push(eventService.addListener2(events_1.EventType.EDITOR_INPUT_CHANGED, function () { return _this._updateActiveAndVisibleTextEditors(); }));
            this._toDispose.push(eventService.addListener2(events_1.EventType.EDITOR_POSITION_CHANGED, function () { return _this._updateActiveAndVisibleTextEditors(); }));
        }
        MainThreadEditors.prototype.dispose = function () {
            var _this = this;
            Object.keys(this._textEditorsListenersMap).forEach(function (editorId) {
                lifecycle_1.disposeAll(_this._textEditorsListenersMap[editorId]);
            });
            this._textEditorsListenersMap = Object.create(null);
            this._toDispose = lifecycle_1.disposeAll(this._toDispose);
        };
        MainThreadEditors.prototype._onTextEditorAdd = function (textEditor) {
            var _this = this;
            var id = textEditor.getId();
            var toDispose = [];
            toDispose.push(textEditor.onConfigurationChanged(function (opts) {
                _this._proxy._acceptOptionsChanged(id, opts);
            }));
            toDispose.push(textEditor.onSelectionChanged(function (selection) {
                _this._proxy._acceptSelectionsChanged(id, selection);
            }));
            this._proxy._acceptTextEditorAdd({
                id: id,
                document: textEditor.getModel().getAssociatedResource(),
                options: textEditor.getConfiguration(),
                selections: textEditor.getSelections(),
                editorPosition: this._findEditorPosition(textEditor)
            });
            this._textEditorsListenersMap[id] = toDispose;
            this._textEditorsMap[id] = textEditor;
        };
        MainThreadEditors.prototype._onTextEditorRemove = function (textEditor) {
            var id = textEditor.getId();
            lifecycle_1.disposeAll(this._textEditorsListenersMap[id]);
            delete this._textEditorsListenersMap[id];
            delete this._textEditorsMap[id];
            this._proxy._acceptTextEditorRemove(id);
        };
        MainThreadEditors.prototype._updateActiveAndVisibleTextEditors = function () {
            // active and visible editors
            var visibleEditors = this._editorTracker.getVisibleTextEditorIds();
            var activeEditor = this._findActiveTextEditorId();
            if (activeEditor !== this._activeTextEditor || !arrays_1.equals(this._visibleEditors, visibleEditors, function (a, b) { return a === b; })) {
                this._activeTextEditor = activeEditor;
                this._visibleEditors = visibleEditors;
                this._proxy._acceptActiveEditorAndVisibleEditors(this._activeTextEditor, this._visibleEditors);
            }
            // editor columns
            var editorPositionData = this._getTextEditorPositionData();
            if (!objects_1.equals(this._editorPositionData, editorPositionData)) {
                this._editorPositionData = editorPositionData;
                this._proxy._acceptEditorPositionData(this._editorPositionData);
            }
        };
        MainThreadEditors.prototype._findActiveTextEditorId = function () {
            var focusedTextEditorId = this._editorTracker.getFocusedTextEditorId();
            if (focusedTextEditorId) {
                return focusedTextEditorId;
            }
            var activeEditor = this._workbenchEditorService.getActiveEditor();
            if (!activeEditor) {
                return null;
            }
            var editor = activeEditor.getControl();
            // Substitute for (editor instanceof ICodeEditor)
            if (!editor || typeof editor.getEditorType !== 'function') {
                // Not a text editor...
                return null;
            }
            if (editor.getEditorType() === editorCommon_1.EditorType.ICodeEditor) {
                return this._editorTracker.findTextEditorIdFor(editor);
            }
            // Must be a diff editor => use the modified side
            return this._editorTracker.findTextEditorIdFor(editor.getModifiedEditor());
        };
        MainThreadEditors.prototype._findEditorPosition = function (editor) {
            for (var _i = 0, _a = this._workbenchEditorService.getVisibleEditors(); _i < _a.length; _i++) {
                var workbenchEditor = _a[_i];
                if (editor.matches(workbenchEditor)) {
                    return workbenchEditor.position;
                }
            }
        };
        MainThreadEditors.prototype._getTextEditorPositionData = function () {
            var result = Object.create(null);
            for (var _i = 0, _a = this._workbenchEditorService.getVisibleEditors(); _i < _a.length; _i++) {
                var workbenchEditor = _a[_i];
                var editor = workbenchEditor.getControl();
                // Substitute for (editor instanceof ICodeEditor)
                if (!editor || typeof editor.getEditorType !== 'function') {
                    // Not a text editor...
                    continue;
                }
                if (editor.getEditorType() === editorCommon_1.EditorType.ICodeEditor) {
                    var id = this._editorTracker.findTextEditorIdFor(editor);
                    if (id) {
                        result[id] = workbenchEditor.position;
                    }
                }
            }
            return result;
        };
        // --- from extension host process
        MainThreadEditors.prototype._tryShowTextDocument = function (resource, position, preserveFocus) {
            var _this = this;
            var input = {
                resource: resource,
                options: { preserveFocus: preserveFocus }
            };
            return this._workbenchEditorService.openEditor(input, position).then(function (editor) {
                return new winjs_base_1.TPromise(function (c) {
                    // not very nice but the way it is: changes to the editor state aren't
                    // send to the ext host as they happen but stuff is delayed a little. in
                    // order to provide the real editor on #openTextEditor we need to sync on
                    // that update
                    var subscription;
                    var handle;
                    function contd() {
                        subscription.dispose();
                        clearTimeout(handle);
                        c(undefined);
                    }
                    subscription = _this._editorTracker.onDidUpdateTextEditors(function () {
                        contd();
                    });
                    handle = setTimeout(function () {
                        contd();
                    }, 1000);
                }).then(function () {
                    // find the editor we have just opened and return the
                    // id we have assigned to it.
                    for (var id in _this._textEditorsMap) {
                        if (_this._textEditorsMap[id].matches(editor)) {
                            return id;
                        }
                    }
                });
            });
        };
        MainThreadEditors.prototype._tryShowEditor = function (id, position) {
            // check how often this is used
            this._telemetryService.publicLog('api.deprecated', { function: 'TextEditor.show' });
            var mainThreadEditor = this._textEditorsMap[id];
            if (mainThreadEditor) {
                var model = mainThreadEditor.getModel();
                return this._workbenchEditorService.openEditor({
                    resource: model.getAssociatedResource(),
                    options: { preserveFocus: false }
                }, position).then(function () { return; });
            }
        };
        MainThreadEditors.prototype._tryHideEditor = function (id) {
            // check how often this is used
            this._telemetryService.publicLog('api.deprecated', { function: 'TextEditor.hide' });
            var mainThreadEditor = this._textEditorsMap[id];
            if (mainThreadEditor) {
                var editors = this._workbenchEditorService.getVisibleEditors();
                for (var _i = 0, editors_1 = editors; _i < editors_1.length; _i++) {
                    var editor = editors_1[_i];
                    if (mainThreadEditor.matches(editor)) {
                        return this._workbenchEditorService.closeEditor(editor).then(function () { return; });
                    }
                }
            }
        };
        MainThreadEditors.prototype._trySetSelections = function (id, selections) {
            if (!this._textEditorsMap[id]) {
                return winjs_base_1.TPromise.wrapError('TextEditor disposed');
            }
            this._textEditorsMap[id].setSelections(selections);
            return winjs_base_1.TPromise.as(null);
        };
        MainThreadEditors.prototype._trySetDecorations = function (id, key, ranges) {
            if (!this._textEditorsMap[id]) {
                return winjs_base_1.TPromise.wrapError('TextEditor disposed');
            }
            this._textEditorsMap[id].setDecorations(key, ranges);
            return winjs_base_1.TPromise.as(null);
        };
        MainThreadEditors.prototype._tryRevealRange = function (id, range, revealType) {
            if (!this._textEditorsMap[id]) {
                return winjs_base_1.TPromise.wrapError('TextEditor disposed');
            }
            this._textEditorsMap[id].revealRange(range, revealType);
        };
        MainThreadEditors.prototype._trySetOptions = function (id, options) {
            if (!this._textEditorsMap[id]) {
                return winjs_base_1.TPromise.wrapError('TextEditor disposed');
            }
            this._textEditorsMap[id].setConfiguration(options);
            return winjs_base_1.TPromise.as(null);
        };
        MainThreadEditors.prototype._tryApplyEdits = function (id, modelVersionId, edits, setEndOfLine) {
            if (!this._textEditorsMap[id]) {
                return winjs_base_1.TPromise.wrapError('TextEditor disposed');
            }
            return winjs_base_1.TPromise.as(this._textEditorsMap[id].applyEdits(modelVersionId, edits, setEndOfLine));
        };
        MainThreadEditors.prototype._registerTextEditorDecorationType = function (key, options) {
            this._editorTracker.registerTextEditorDecorationType(key, options);
        };
        MainThreadEditors.prototype._removeTextEditorDecorationType = function (key) {
            this._editorTracker.removeTextEditorDecorationType(key);
        };
        MainThreadEditors = __decorate([
            thread_1.Remotable.MainContext('MainThreadEditors'),
            __param(0, thread_1.IThreadService),
            __param(1, editorService_1.IWorkbenchEditorService),
            __param(2, telemetry_1.ITelemetryService),
            __param(3, codeEditorService_1.ICodeEditorService),
            __param(4, event_2.IEventService),
            __param(5, modelService_1.IModelService)
        ], MainThreadEditors);
        return MainThreadEditors;
    }());
    exports.MainThreadEditors = MainThreadEditors;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostCommands", ["require", "exports", 'vs/platform/thread/common/thread', 'vs/base/common/types', 'vs/platform/keybinding/common/keybindingsRegistry', 'vs/platform/keybinding/common/keybindingService', 'vs/base/common/winjs.base', 'vs/workbench/api/node/extHostEditors', 'vs/workbench/api/node/extHostTypes', 'vs/workbench/api/node/extHostTypeConverters', 'vs/base/common/objects'], function (require, exports, thread_1, types_1, keybindingsRegistry_1, keybindingService_1, winjs_base_1, extHostEditors_1, extHostTypes, extHostTypeConverter, objects_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var ExtHostCommands = (function () {
        function ExtHostCommands(threadService) {
            this._commands = Object.create(null);
            this._extHostEditors = threadService.getRemotable(extHostEditors_1.ExtHostEditors);
            this._proxy = threadService.getRemotable(MainThreadCommands);
        }
        ExtHostCommands.prototype.registerCommand = function (id, callback, thisArg, description) {
            var _this = this;
            if (!id.trim().length) {
                throw new Error('invalid id');
            }
            if (this._commands[id]) {
                throw new Error('command with id already exists');
            }
            this._commands[id] = { callback: callback, thisArg: thisArg, description: description };
            this._proxy.$registerCommand(id);
            return new extHostTypes.Disposable(function () { return delete _this._commands[id]; });
        };
        ExtHostCommands.prototype.executeCommand = function (id) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            if (this._commands[id]) {
                // we stay inside the extension host and support
                // to pass any kind of parameters around
                return this.$executeContributedCommand.apply(this, [id].concat(args));
            }
            else {
                // automagically convert some argument types
                args = objects_1.cloneAndChange(args, function (value) {
                    if (value instanceof extHostTypes.Position) {
                        return extHostTypeConverter.fromPosition(value);
                    }
                    if (value instanceof extHostTypes.Range) {
                        return extHostTypeConverter.fromRange(value);
                    }
                    if (value instanceof extHostTypes.Location) {
                        return extHostTypeConverter.location.from(value);
                    }
                    if (!Array.isArray(value)) {
                        return value;
                    }
                });
                return this._proxy.$executeCommand(id, args);
            }
        };
        ExtHostCommands.prototype.$executeContributedCommand = function (id) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var command = this._commands[id];
            if (!command) {
                return Promise.reject("Contributed command '" + id + "' does not exist.");
            }
            try {
                var callback = command.callback, thisArg = command.thisArg, description = command.description;
                if (description) {
                    for (var i = 0; i < description.args.length; i++) {
                        types_1.validateConstraint(args[i], description.args[i].constraint);
                    }
                }
                var result = callback.apply(thisArg, args);
                return Promise.resolve(result);
            }
            catch (err) {
                // console.log(err);
                // try {
                // 	console.log(toErrorMessage(err));
                // } catch (err) {
                // 	//
                // }
                return Promise.reject("Running the contributed command:'" + id + "' failed.");
            }
        };
        ExtHostCommands.prototype.getCommands = function (filterUnderscoreCommands) {
            if (filterUnderscoreCommands === void 0) { filterUnderscoreCommands = false; }
            return this._proxy.$getCommands().then(function (result) {
                if (filterUnderscoreCommands) {
                    result = result.filter(function (command) { return command[0] !== '_'; });
                }
                return result;
            });
        };
        ExtHostCommands.prototype.$getContributedCommandHandlerDescriptions = function () {
            var result = Object.create(null);
            for (var id in this._commands) {
                var description = this._commands[id].description;
                if (description) {
                    result[id] = description;
                }
            }
            return winjs_base_1.TPromise.as(result);
        };
        ExtHostCommands = __decorate([
            thread_1.Remotable.ExtHostContext('ExtHostCommands'),
            __param(0, thread_1.IThreadService)
        ], ExtHostCommands);
        return ExtHostCommands;
    }());
    exports.ExtHostCommands = ExtHostCommands;
    var MainThreadCommands = (function () {
        function MainThreadCommands(threadService, keybindingService) {
            this._threadService = threadService;
            this._keybindingService = keybindingService;
            this._proxy = this._threadService.getRemotable(ExtHostCommands);
        }
        MainThreadCommands.prototype.$registerCommand = function (id) {
            var _this = this;
            keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
                id: id,
                handler: function (serviceAccessor) {
                    var args = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        args[_i - 1] = arguments[_i];
                    }
                    return (_a = _this._proxy).$executeContributedCommand.apply(_a, [id].concat(args));
                    var _a;
                },
                weight: undefined,
                context: undefined,
                win: undefined,
                mac: undefined,
                linux: undefined,
                primary: undefined,
                secondary: undefined
            });
            return undefined;
        };
        MainThreadCommands.prototype.$executeCommand = function (id, args) {
            return this._keybindingService.executeCommand(id, args);
        };
        MainThreadCommands.prototype.$getCommands = function () {
            return winjs_base_1.TPromise.as(Object.keys(keybindingsRegistry_1.KeybindingsRegistry.getCommands()));
        };
        MainThreadCommands = __decorate([
            thread_1.Remotable.MainContext('MainThreadCommands'),
            __param(0, thread_1.IThreadService),
            __param(1, keybindingService_1.IKeybindingService)
        ], MainThreadCommands);
        return MainThreadCommands;
    }());
    exports.MainThreadCommands = MainThreadCommands;
    // --- command doc
    keybindingsRegistry_1.KeybindingsRegistry.registerCommandDesc({
        id: '_generateCommandsDocumentation',
        handler: function (accessor) {
            return accessor.get(thread_1.IThreadService).getRemotable(ExtHostCommands).$getContributedCommandHandlerDescriptions().then(function (result) {
                // add local commands
                var commands = keybindingsRegistry_1.KeybindingsRegistry.getCommands();
                for (var id in commands) {
                    var description = commands[id].description;
                    if (description) {
                        result[id] = description;
                    }
                }
                // print all as markdown
                var all = [];
                for (var id in result) {
                    all.push('`' + id + '` - ' + _generateMarkdown(result[id]));
                }
                console.log(all.join('\n'));
            });
        },
        context: undefined,
        weight: keybindingsRegistry_1.KeybindingsRegistry.WEIGHT.builtinExtension(0),
        primary: undefined
    });
    function _generateMarkdown(description) {
        if (typeof description === 'string') {
            return description;
        }
        else {
            var parts = [description.description];
            parts.push('\n\n');
            if (description.args) {
                for (var _i = 0, _a = description.args; _i < _a.length; _i++) {
                    var arg = _a[_i];
                    parts.push("* _" + arg.name + "_ " + (arg.description || '') + "\n");
                }
            }
            if (description.returns) {
                parts.push("* _(returns)_ " + description.returns);
            }
            parts.push('\n\n');
            return parts.join('');
        }
    }
});

define("vs/workbench/api/node/extHostApiCommands", ["require", "exports", 'vs/base/common/uri', 'vs/base/common/winjs.base', 'vs/workbench/api/node/extHostTypeConverters', 'vs/workbench/api/node/extHostTypes', 'vs/workbench/api/node/extHostCommands'], function (require, exports, uri_1, winjs_base_1, typeConverters, types, extHostCommands_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    function registerApiCommands(threadService) {
        var commands = threadService.getRemotable(extHostCommands_1.ExtHostCommands);
        new ExtHostApiCommands(commands).registerCommands();
    }
    exports.registerApiCommands = registerApiCommands;
    var ExtHostApiCommands = (function () {
        function ExtHostApiCommands(commands) {
            this._disposables = [];
            this._commands = commands;
        }
        ExtHostApiCommands.prototype.registerCommands = function () {
            var _this = this;
            this._register('vscode.executeWorkspaceSymbolProvider', this._executeWorkspaceSymbolProvider, {
                description: 'Execute all workspace symbol provider.',
                args: [{ name: 'query', description: 'Search string', constraint: String }],
                returns: 'A promise that resolves to an array of SymbolInformation-instances.'
            });
            this._register('vscode.executeDefinitionProvider', this._executeDefinitionProvider, {
                description: 'Execute all definition provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'position', description: 'Position of a symbol', constraint: types.Position }
                ],
                returns: 'A promise that resolves to an array of Location-instances.'
            });
            this._register('vscode.executeHoverProvider', this._executeHoverProvider, {
                description: 'Execute all hover provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'position', description: 'Position of a symbol', constraint: types.Position }
                ],
                returns: 'A promise that resolves to an array of Hover-instances.'
            });
            this._register('vscode.executeDocumentHighlights', this._executeDocumentHighlights, {
                description: 'Execute document highlight provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'position', description: 'Position in a text document', constraint: types.Position }
                ],
                returns: 'A promise that resolves to an array of DocumentHighlight-instances.'
            });
            this._register('vscode.executeReferenceProvider', this._executeReferenceProvider, {
                description: 'Execute reference provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'position', description: 'Position in a text document', constraint: types.Position }
                ],
                returns: 'A promise that resolves to an array of Location-instances.'
            });
            this._register('vscode.executeDocumentRenameProvider', this._executeDocumentRenameProvider, {
                description: 'Execute rename provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'position', description: 'Position in a text document', constraint: types.Position },
                    { name: 'newName', description: 'The new symbol name', constraint: String }
                ],
                returns: 'A promise that resolves to a WorkspaceEdit.'
            });
            this._register('vscode.executeSignatureHelpProvider', this._executeSignatureHelpProvider, {
                description: 'Execute signature help provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'position', description: 'Position in a text document', constraint: types.Position }
                ],
                returns: 'A promise that resolves to SignatureHelp.'
            });
            this._register('vscode.executeDocumentSymbolProvider', this._executeDocumentSymbolProvider, {
                description: 'Execute document symbol provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default }
                ],
                returns: 'A promise that resolves to an array of SymbolInformation-instances.'
            });
            this._register('vscode.executeCompletionItemProvider', this._executeCompletionItemProvider, {
                description: 'Execute completion item provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'position', description: 'Position in a text document', constraint: types.Position }
                ],
                returns: 'A promise that resolves to a CompletionList-instance.'
            });
            this._register('vscode.executeCodeActionProvider', this._executeCodeActionProvider, {
                description: 'Execute code action provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'range', description: 'Range in a text document', constraint: types.Range }
                ],
                returns: 'A promise that resolves to an array of CompletionItem-instances.'
            });
            this._register('vscode.executeCodeLensProvider', this._executeCodeLensProvider, {
                description: 'Execute completion item provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default }
                ],
                returns: 'A promise that resolves to an array of Commands.'
            });
            this._register('vscode.executeFormatDocumentProvider', this._executeFormatDocumentProvider, {
                description: 'Execute document format provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'options', description: 'Formatting options' }
                ],
                returns: 'A promise that resolves to an array of TextEdits.'
            });
            this._register('vscode.executeFormatRangeProvider', this._executeFormatRangeProvider, {
                description: 'Execute range format provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'range', description: 'Range in a text document', constraint: types.Range },
                    { name: 'options', description: 'Formatting options' }
                ],
                returns: 'A promise that resolves to an array of TextEdits.'
            });
            this._register('vscode.executeFormatOnTypeProvider', this._executeFormatOnTypeProvider, {
                description: 'Execute document format provider.',
                args: [
                    { name: 'uri', description: 'Uri of a text document', constraint: uri_1.default },
                    { name: 'position', description: 'Position in a text document', constraint: types.Position },
                    { name: 'ch', description: 'Character that got typed', constraint: String },
                    { name: 'options', description: 'Formatting options' }
                ],
                returns: 'A promise that resolves to an array of TextEdits.'
            });
            this._register('vscode.previewHtml', function (uri, position) {
                return _this._commands.executeCommand('_workbench.previewHtml', uri, typeof position === 'number' ? typeConverters.fromViewColumn(position) : void 0);
            }, {
                description: 'Preview an html document.',
                args: [
                    { name: 'uri', description: 'Uri of the document to preview.', constraint: uri_1.default },
                    { name: 'column', description: '(optional) Column in which to preview.' },
                ]
            });
        };
        // --- command impl
        ExtHostApiCommands.prototype._register = function (id, handler, description) {
            var disposable = this._commands.registerCommand(id, handler, this, description);
            this._disposables.push(disposable);
        };
        /**
         * Execute workspace symbol provider.
         *
         * @param query Search string to match query symbol names
         * @return A promise that resolves to an array of symbol information.
         */
        ExtHostApiCommands.prototype._executeWorkspaceSymbolProvider = function (query) {
            return this._commands.executeCommand('_executeWorkspaceSymbolProvider', { query: query }).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(typeConverters.toSymbolInformation);
                }
            });
        };
        ExtHostApiCommands.prototype._executeDefinitionProvider = function (resource, position) {
            var args = {
                resource: resource,
                position: position && typeConverters.fromPosition(position)
            };
            return this._commands.executeCommand('_executeDefinitionProvider', args).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(typeConverters.location.to);
                }
            });
        };
        ExtHostApiCommands.prototype._executeHoverProvider = function (resource, position) {
            var args = {
                resource: resource,
                position: position && typeConverters.fromPosition(position)
            };
            return this._commands.executeCommand('_executeHoverProvider', args).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(typeConverters.toHover);
                }
            });
        };
        ExtHostApiCommands.prototype._executeDocumentHighlights = function (resource, position) {
            var args = {
                resource: resource,
                position: position && typeConverters.fromPosition(position)
            };
            return this._commands.executeCommand('_executeDocumentHighlights', args).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(typeConverters.toDocumentHighlight);
                }
            });
        };
        ExtHostApiCommands.prototype._executeReferenceProvider = function (resource, position) {
            var args = {
                resource: resource,
                position: position && typeConverters.fromPosition(position)
            };
            return this._commands.executeCommand('_executeDocumentHighlights', args).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(typeConverters.location.to);
                }
            });
        };
        ExtHostApiCommands.prototype._executeDocumentRenameProvider = function (resource, position, newName) {
            var args = {
                resource: resource,
                position: position && typeConverters.fromPosition(position),
                newName: newName
            };
            return this._commands.executeCommand('_executeDocumentRenameProvider', args).then(function (value) {
                if (!value) {
                    return;
                }
                if (value.rejectReason) {
                    return winjs_base_1.TPromise.wrapError(value.rejectReason);
                }
                var workspaceEdit = new types.WorkspaceEdit();
                for (var _i = 0, _a = value.edits; _i < _a.length; _i++) {
                    var edit = _a[_i];
                    workspaceEdit.replace(edit.resource, typeConverters.toRange(edit.range), edit.newText);
                }
                return workspaceEdit;
            });
        };
        ExtHostApiCommands.prototype._executeSignatureHelpProvider = function (resource, position, triggerCharacter) {
            var args = {
                resource: resource,
                position: position && typeConverters.fromPosition(position),
                triggerCharacter: triggerCharacter
            };
            return this._commands.executeCommand('_executeSignatureHelpProvider', args).then(function (value) {
                if (value) {
                    return typeConverters.SignatureHelp.to(value);
                }
            });
        };
        ExtHostApiCommands.prototype._executeCompletionItemProvider = function (resource, position, triggerCharacter) {
            var args = {
                resource: resource,
                position: position && typeConverters.fromPosition(position),
                triggerCharacter: triggerCharacter
            };
            return this._commands.executeCommand('_executeCompletionItemProvider', args).then(function (value) {
                if (value) {
                    var items = [];
                    var incomplete = void 0;
                    for (var _i = 0, value_1 = value; _i < value_1.length; _i++) {
                        var group = value_1[_i];
                        for (var _a = 0, group_1 = group; _a < group_1.length; _a++) {
                            var suggestions = group_1[_a];
                            incomplete = suggestions.incomplete || incomplete;
                            for (var _b = 0, _c = suggestions.suggestions; _b < _c.length; _b++) {
                                var suggestion = _c[_b];
                                var item = typeConverters.Suggest.to(suggestions, position, suggestion);
                                items.push(item);
                            }
                        }
                    }
                    return new types.CompletionList(items, incomplete);
                }
            });
        };
        ExtHostApiCommands.prototype._executeDocumentSymbolProvider = function (resource) {
            var args = {
                resource: resource
            };
            return this._commands.executeCommand('_executeDocumentSymbolProvider', args).then(function (value) {
                if (value && Array.isArray(value.entries)) {
                    return value.entries.map(typeConverters.SymbolInformation.fromOutlineEntry);
                }
            });
        };
        ExtHostApiCommands.prototype._executeCodeActionProvider = function (resource, range) {
            var args = {
                resource: resource,
                range: typeConverters.fromRange(range)
            };
            return this._commands.executeCommand('_executeCodeActionProvider', args).then(function (value) {
                if (!Array.isArray(value)) {
                    return;
                }
                return value.map(function (quickFix) { return typeConverters.Command.to(quickFix.command); });
            });
        };
        ExtHostApiCommands.prototype._executeCodeLensProvider = function (resource) {
            var args = { resource: resource };
            return this._commands.executeCommand('_executeCodeLensProvider', args).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(function (item) {
                        return new types.CodeLens(typeConverters.toRange(item.symbol.range), typeConverters.Command.to(item.symbol.command));
                    });
                }
            });
        };
        ExtHostApiCommands.prototype._executeFormatDocumentProvider = function (resource, options) {
            var args = {
                resource: resource,
                options: options
            };
            return this._commands.executeCommand('_executeFormatDocumentProvider', args).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(function (edit) { return new types.TextEdit(typeConverters.toRange(edit.range), edit.text); });
                }
            });
        };
        ExtHostApiCommands.prototype._executeFormatRangeProvider = function (resource, range, options) {
            var args = {
                resource: resource,
                range: typeConverters.fromRange(range),
                options: options
            };
            return this._commands.executeCommand('_executeFormatRangeProvider', args).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(function (edit) { return new types.TextEdit(typeConverters.toRange(edit.range), edit.text); });
                }
            });
        };
        ExtHostApiCommands.prototype._executeFormatOnTypeProvider = function (resource, position, ch, options) {
            var args = {
                resource: resource,
                position: typeConverters.fromPosition(position),
                ch: ch,
                options: options
            };
            return this._commands.executeCommand('_executeFormatOnTypeProvider', args).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(function (edit) { return new types.TextEdit(typeConverters.toRange(edit.range), edit.text); });
                }
            });
        };
        return ExtHostApiCommands;
    }());
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHostLanguageFeatures", ["require", "exports", 'vs/editor/common/modes/modesFilters', 'vs/base/common/winjs.base', 'vs/base/common/lifecycle', 'vs/platform/thread/common/thread', 'vs/editor/common/core/range', 'vs/workbench/api/node/extHostTypeConverters', 'vs/workbench/api/node/extHostTypes', 'vs/workbench/api/node/extHostDocuments', 'vs/platform/markers/common/markers', 'vs/workbench/api/node/extHostCommands', 'vs/editor/contrib/goToDeclaration/common/goToDeclaration', 'vs/editor/contrib/hover/common/hover', 'vs/editor/contrib/wordHighlighter/common/wordHighlighter', 'vs/editor/contrib/referenceSearch/common/referenceSearch', 'vs/editor/contrib/quickFix/common/quickFix', 'vs/editor/contrib/quickOpen/common/quickOpen', 'vs/workbench/parts/search/common/search', 'vs/editor/contrib/rename/common/rename', 'vs/editor/contrib/format/common/format', 'vs/editor/contrib/codelens/common/codelens', 'vs/editor/contrib/parameterHints/common/parameterHints', 'vs/editor/contrib/suggest/common/suggest', 'vs/base/common/async'], function (require, exports, modesFilters_1, winjs_base_1, lifecycle_1, thread_1, range_1, TypeConverters, extHostTypes_1, extHostDocuments_1, markers_1, extHostCommands_1, goToDeclaration_1, hover_1, wordHighlighter_1, referenceSearch_1, quickFix_1, quickOpen_1, search_1, rename_1, format_1, codelens_1, parameterHints_1, suggest_1, async_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    // --- adapter
    var OutlineAdapter = (function () {
        function OutlineAdapter(documents, provider) {
            this._documents = documents;
            this._provider = provider;
        }
        OutlineAdapter.prototype.getOutline = function (resource) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideDocumentSymbols(doc, token); }).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(TypeConverters.SymbolInformation.toOutlineEntry);
                }
            });
        };
        return OutlineAdapter;
    }());
    var CodeLensAdapter = (function () {
        function CodeLensAdapter(documents, commands, provider) {
            this._cache = Object.create(null);
            this._documents = documents;
            this._commands = commands;
            this._provider = provider;
        }
        CodeLensAdapter.prototype.findCodeLensSymbols = function (resource) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var version = doc.version;
            var key = resource.toString();
            // from cache
            var entry = this._cache[key];
            if (entry && entry.version === version) {
                return new async_1.ShallowCancelThenPromise(entry.data.then(function (cached) { return cached.symbols; }));
            }
            var newCodeLensData = async_1.asWinJsPromise(function (token) { return _this._provider.provideCodeLenses(doc, token); }).then(function (lenses) {
                if (!Array.isArray(lenses)) {
                    return;
                }
                var data = {
                    lenses: lenses,
                    symbols: [],
                    disposables: [],
                };
                lenses.forEach(function (lens, i) {
                    data.symbols.push({
                        id: String(i),
                        range: TypeConverters.fromRange(lens.range),
                        command: TypeConverters.Command.from(lens.command, { commands: _this._commands, disposables: data.disposables })
                    });
                });
                return data;
            });
            this._cache[key] = {
                version: version,
                data: newCodeLensData
            };
            return new async_1.ShallowCancelThenPromise(newCodeLensData.then(function (newCached) {
                if (entry) {
                    // only now dispose old commands et al
                    entry.data.then(function (oldCached) { return lifecycle_1.disposeAll(oldCached.disposables); });
                }
                return newCached && newCached.symbols;
            }));
        };
        CodeLensAdapter.prototype.resolveCodeLensSymbol = function (resource, symbol) {
            var _this = this;
            var entry = this._cache[resource.toString()];
            if (!entry) {
                return;
            }
            return entry.data.then(function (cachedData) {
                if (!cachedData) {
                    return;
                }
                var lens = cachedData.lenses[Number(symbol.id)];
                if (!lens) {
                    return;
                }
                var resolve;
                if (typeof _this._provider.resolveCodeLens !== 'function' || lens.isResolved) {
                    resolve = winjs_base_1.TPromise.as(lens);
                }
                else {
                    resolve = async_1.asWinJsPromise(function (token) { return _this._provider.resolveCodeLens(lens, token); });
                }
                return resolve.then(function (newLens) {
                    lens = newLens || lens;
                    var command = lens.command;
                    if (!command) {
                        command = {
                            title: '<<MISSING COMMAND>>',
                            command: 'missing',
                        };
                    }
                    symbol.command = TypeConverters.Command.from(command, { commands: _this._commands, disposables: cachedData.disposables });
                    return symbol;
                });
            });
        };
        return CodeLensAdapter;
    }());
    var DeclarationAdapter = (function () {
        function DeclarationAdapter(documents, provider) {
            this._documents = documents;
            this._provider = provider;
        }
        DeclarationAdapter.prototype.canFindDeclaration = function () {
            return true;
        };
        DeclarationAdapter.prototype.findDeclaration = function (resource, position) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var pos = TypeConverters.toPosition(position);
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideDefinition(doc, pos, token); }).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(DeclarationAdapter._convertLocation);
                }
                else if (value) {
                    return DeclarationAdapter._convertLocation(value);
                }
            });
        };
        DeclarationAdapter._convertLocation = function (location) {
            if (!location) {
                return;
            }
            return {
                resource: location.uri,
                range: TypeConverters.fromRange(location.range)
            };
        };
        return DeclarationAdapter;
    }());
    var ExtraInfoAdapter = (function () {
        function ExtraInfoAdapter(documents, provider) {
            this._documents = documents;
            this._provider = provider;
        }
        ExtraInfoAdapter.prototype.computeInfo = function (resource, position) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var pos = TypeConverters.toPosition(position);
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideHover(doc, pos, token); }).then(function (value) {
                if (!value) {
                    return;
                }
                if (!value.range) {
                    value.range = doc.getWordRangeAtPosition(pos);
                }
                if (!value.range) {
                    value.range = new extHostTypes_1.Range(pos, pos);
                }
                return TypeConverters.fromHover(value);
            });
        };
        return ExtraInfoAdapter;
    }());
    var OccurrencesAdapter = (function () {
        function OccurrencesAdapter(documents, provider) {
            this._documents = documents;
            this._provider = provider;
        }
        OccurrencesAdapter.prototype.findOccurrences = function (resource, position) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var pos = TypeConverters.toPosition(position);
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideDocumentHighlights(doc, pos, token); }).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(OccurrencesAdapter._convertDocumentHighlight);
                }
            });
        };
        OccurrencesAdapter._convertDocumentHighlight = function (documentHighlight) {
            return {
                range: TypeConverters.fromRange(documentHighlight.range),
                kind: extHostTypes_1.DocumentHighlightKind[documentHighlight.kind].toString().toLowerCase()
            };
        };
        return OccurrencesAdapter;
    }());
    var ReferenceAdapter = (function () {
        function ReferenceAdapter(documents, provider) {
            this._documents = documents;
            this._provider = provider;
        }
        ReferenceAdapter.prototype.canFindReferences = function () {
            return true;
        };
        ReferenceAdapter.prototype.findReferences = function (resource, position, includeDeclaration) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var pos = TypeConverters.toPosition(position);
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideReferences(doc, pos, { includeDeclaration: includeDeclaration }, token); }).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(ReferenceAdapter._convertLocation);
                }
            });
        };
        ReferenceAdapter._convertLocation = function (location) {
            return {
                resource: location.uri,
                range: TypeConverters.fromRange(location.range)
            };
        };
        return ReferenceAdapter;
    }());
    var QuickFixAdapter = (function () {
        function QuickFixAdapter(documents, commands, provider) {
            this._cachedCommands = [];
            this._documents = documents;
            this._commands = commands;
            this._provider = provider;
        }
        QuickFixAdapter.prototype.getQuickFixes = function (resource, range, markers) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var ran = TypeConverters.toRange(range);
            var diagnostics = markers.map(function (marker) {
                var diag = new extHostTypes_1.Diagnostic(TypeConverters.toRange(marker), marker.message);
                diag.code = marker.code;
                diag.severity = TypeConverters.toDiagnosticSeverty(marker.severity);
                return diag;
            });
            this._cachedCommands = lifecycle_1.disposeAll(this._cachedCommands);
            var ctx = { commands: this._commands, disposables: this._cachedCommands };
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideCodeActions(doc, ran, { diagnostics: diagnostics }, token); }).then(function (commands) {
                if (!Array.isArray(commands)) {
                    return;
                }
                return commands.map(function (command, i) {
                    return {
                        command: TypeConverters.Command.from(command, ctx),
                        score: i
                    };
                });
            });
        };
        QuickFixAdapter.prototype.runQuickFixAction = function (resource, range, quickFix) {
            var command = TypeConverters.Command.to(quickFix.command);
            return (_a = this._commands).executeCommand.apply(_a, [command.command].concat(command.arguments));
            var _a;
        };
        return QuickFixAdapter;
    }());
    var DocumentFormattingAdapter = (function () {
        function DocumentFormattingAdapter(documents, provider) {
            this._documents = documents;
            this._provider = provider;
        }
        DocumentFormattingAdapter.prototype.formatDocument = function (resource, options) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideDocumentFormattingEdits(doc, options, token); }).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(TypeConverters.TextEdit.from);
                }
            });
        };
        return DocumentFormattingAdapter;
    }());
    var RangeFormattingAdapter = (function () {
        function RangeFormattingAdapter(documents, provider) {
            this._documents = documents;
            this._provider = provider;
        }
        RangeFormattingAdapter.prototype.formatRange = function (resource, range, options) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var ran = TypeConverters.toRange(range);
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideDocumentRangeFormattingEdits(doc, ran, options, token); }).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(TypeConverters.TextEdit.from);
                }
            });
        };
        return RangeFormattingAdapter;
    }());
    var OnTypeFormattingAdapter = (function () {
        function OnTypeFormattingAdapter(documents, provider) {
            this.autoFormatTriggerCharacters = []; // not here
            this._documents = documents;
            this._provider = provider;
        }
        OnTypeFormattingAdapter.prototype.formatAfterKeystroke = function (resource, position, ch, options) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var pos = TypeConverters.toPosition(position);
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideOnTypeFormattingEdits(doc, pos, ch, options, token); }).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(TypeConverters.TextEdit.from);
                }
            });
        };
        return OnTypeFormattingAdapter;
    }());
    var NavigateTypeAdapter = (function () {
        function NavigateTypeAdapter(provider) {
            this._provider = provider;
        }
        NavigateTypeAdapter.prototype.getNavigateToItems = function (search) {
            var _this = this;
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideWorkspaceSymbols(search, token); }).then(function (value) {
                if (Array.isArray(value)) {
                    return value.map(TypeConverters.fromSymbolInformation);
                }
            });
        };
        return NavigateTypeAdapter;
    }());
    var RenameAdapter = (function () {
        function RenameAdapter(documents, provider) {
            this._documents = documents;
            this._provider = provider;
        }
        RenameAdapter.prototype.rename = function (resource, position, newName) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var pos = TypeConverters.toPosition(position);
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideRenameEdits(doc, pos, newName, token); }).then(function (value) {
                if (!value) {
                    return;
                }
                var result = {
                    currentName: undefined,
                    edits: []
                };
                for (var _i = 0, _a = value.entries(); _i < _a.length; _i++) {
                    var entry = _a[_i];
                    var uri = entry[0], textEdits = entry[1];
                    for (var _b = 0, textEdits_1 = textEdits; _b < textEdits_1.length; _b++) {
                        var textEdit = textEdits_1[_b];
                        result.edits.push({
                            resource: uri,
                            newText: textEdit.newText,
                            range: TypeConverters.fromRange(textEdit.range)
                        });
                    }
                }
                return result;
            }, function (err) {
                if (typeof err === 'string') {
                    return {
                        currentName: undefined,
                        edits: undefined,
                        rejectReason: err
                    };
                }
                return winjs_base_1.TPromise.wrapError(err);
            });
        };
        return RenameAdapter;
    }());
    var SuggestAdapter = (function () {
        function SuggestAdapter(documents, provider) {
            this._cache = Object.create(null);
            this._documents = documents;
            this._provider = provider;
        }
        SuggestAdapter.prototype.suggest = function (resource, position) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var pos = TypeConverters.toPosition(position);
            var ran = doc.getWordRangeAtPosition(pos);
            var key = resource.toString();
            delete this._cache[key];
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideCompletionItems(doc, pos, token); }).then(function (value) {
                var defaultSuggestions = {
                    suggestions: [],
                    currentWord: ran ? doc.getText(new extHostTypes_1.Range(ran.start.line, ran.start.character, pos.line, pos.character)) : '',
                };
                var allSuggestions = [defaultSuggestions];
                var list;
                if (Array.isArray(value)) {
                    list = new extHostTypes_1.CompletionList(value);
                }
                else if (value instanceof extHostTypes_1.CompletionList) {
                    list = value;
                    defaultSuggestions.incomplete = list.isIncomplete;
                }
                else {
                    return;
                }
                for (var i = 0; i < list.items.length; i++) {
                    var item = list.items[i];
                    var suggestion = TypeConverters.Suggest.from(item);
                    if (item.textEdit) {
                        var editRange = item.textEdit.range;
                        // invalid text edit
                        if (!editRange.isSingleLine || editRange.start.line !== pos.line) {
                            console.warn('INVALID text edit, must be single line and on the same line');
                            continue;
                        }
                        // insert the text of the edit and create a dedicated
                        // suggestion-container with overwrite[Before|After]
                        suggestion.codeSnippet = item.textEdit.newText;
                        suggestion.overwriteBefore = pos.character - editRange.start.character;
                        suggestion.overwriteAfter = editRange.end.character - pos.character;
                        allSuggestions.push({
                            currentWord: doc.getText(editRange),
                            suggestions: [suggestion],
                            incomplete: list.isIncomplete
                        });
                    }
                    else {
                        defaultSuggestions.suggestions.push(suggestion);
                    }
                    // assign identifier to suggestion
                    suggestion.id = String(i);
                }
                // cache for details call
                _this._cache[key] = list;
                return allSuggestions;
            });
        };
        SuggestAdapter.prototype.getSuggestionDetails = function (resource, position, suggestion) {
            var _this = this;
            if (typeof this._provider.resolveCompletionItem !== 'function') {
                return winjs_base_1.TPromise.as(suggestion);
            }
            var list = this._cache[resource.toString()];
            if (!list) {
                return winjs_base_1.TPromise.as(suggestion);
            }
            var item = list.items[Number(suggestion.id)];
            if (!item) {
                return winjs_base_1.TPromise.as(suggestion);
            }
            return async_1.asWinJsPromise(function (token) { return _this._provider.resolveCompletionItem(item, token); }).then(function (resolvedItem) {
                return TypeConverters.Suggest.from(resolvedItem || item);
            });
        };
        SuggestAdapter.prototype.getFilter = function () {
            throw new Error('illegal state');
        };
        SuggestAdapter.prototype.getTriggerCharacters = function () {
            throw new Error('illegal state');
        };
        SuggestAdapter.prototype.shouldShowEmptySuggestionList = function () {
            throw new Error('illegal state');
        };
        SuggestAdapter.prototype.shouldAutotriggerSuggest = function (context, offset, triggeredByCharacter) {
            throw new Error('illegal state');
        };
        return SuggestAdapter;
    }());
    var ParameterHintsAdapter = (function () {
        function ParameterHintsAdapter(documents, provider) {
            this._documents = documents;
            this._provider = provider;
        }
        ParameterHintsAdapter.prototype.getParameterHints = function (resource, position, triggerCharacter) {
            var _this = this;
            var doc = this._documents.getDocumentData(resource).document;
            var pos = TypeConverters.toPosition(position);
            return async_1.asWinJsPromise(function (token) { return _this._provider.provideSignatureHelp(doc, pos, token); }).then(function (value) {
                if (value instanceof extHostTypes_1.SignatureHelp) {
                    return TypeConverters.SignatureHelp.from(value);
                }
            });
        };
        ParameterHintsAdapter.prototype.getParameterHintsTriggerCharacters = function () {
            throw new Error('illegal state');
        };
        ParameterHintsAdapter.prototype.shouldTriggerParameterHints = function (context, offset) {
            throw new Error('illegal state');
        };
        return ParameterHintsAdapter;
    }());
    var ExtHostLanguageFeatures = (function () {
        function ExtHostLanguageFeatures(threadService) {
            this._adapter = Object.create(null);
            this._proxy = threadService.getRemotable(MainThreadLanguageFeatures);
            this._documents = threadService.getRemotable(extHostDocuments_1.ExtHostModelService);
            this._commands = threadService.getRemotable(extHostCommands_1.ExtHostCommands);
        }
        ExtHostLanguageFeatures.prototype._createDisposable = function (handle) {
            var _this = this;
            return new extHostTypes_1.Disposable(function () {
                delete _this._adapter[handle];
                _this._proxy.$unregister(handle);
            });
        };
        ExtHostLanguageFeatures.prototype._nextHandle = function () {
            return ExtHostLanguageFeatures._handlePool++;
        };
        ExtHostLanguageFeatures.prototype._withAdapter = function (handle, ctor, callback) {
            var adapter = this._adapter[handle];
            if (!(adapter instanceof ctor)) {
                return winjs_base_1.TPromise.wrapError(new Error('no adapter found'));
            }
            return callback(adapter);
        };
        // --- outline
        ExtHostLanguageFeatures.prototype.registerDocumentSymbolProvider = function (selector, provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new OutlineAdapter(this._documents, provider);
            this._proxy.$registerOutlineSupport(handle, selector);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$getOutline = function (handle, resource) {
            return this._withAdapter(handle, OutlineAdapter, function (adapter) { return adapter.getOutline(resource); });
        };
        // --- code lens
        ExtHostLanguageFeatures.prototype.registerCodeLensProvider = function (selector, provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new CodeLensAdapter(this._documents, this._commands, provider);
            this._proxy.$registerCodeLensSupport(handle, selector);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$findCodeLensSymbols = function (handle, resource) {
            return this._withAdapter(handle, CodeLensAdapter, function (adapter) { return adapter.findCodeLensSymbols(resource); });
        };
        ExtHostLanguageFeatures.prototype.$resolveCodeLensSymbol = function (handle, resource, symbol) {
            return this._withAdapter(handle, CodeLensAdapter, function (adapter) { return adapter.resolveCodeLensSymbol(resource, symbol); });
        };
        // --- declaration
        ExtHostLanguageFeatures.prototype.registerDefinitionProvider = function (selector, provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new DeclarationAdapter(this._documents, provider);
            this._proxy.$registerDeclaractionSupport(handle, selector);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$findDeclaration = function (handle, resource, position) {
            return this._withAdapter(handle, DeclarationAdapter, function (adapter) { return adapter.findDeclaration(resource, position); });
        };
        // --- extra info
        ExtHostLanguageFeatures.prototype.registerHoverProvider = function (selector, provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new ExtraInfoAdapter(this._documents, provider);
            this._proxy.$registerExtraInfoSupport(handle, selector);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$computeInfo = function (handle, resource, position) {
            return this._withAdapter(handle, ExtraInfoAdapter, function (adpater) { return adpater.computeInfo(resource, position); });
        };
        // --- occurrences
        ExtHostLanguageFeatures.prototype.registerDocumentHighlightProvider = function (selector, provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new OccurrencesAdapter(this._documents, provider);
            this._proxy.$registerOccurrencesSupport(handle, selector);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$findOccurrences = function (handle, resource, position) {
            return this._withAdapter(handle, OccurrencesAdapter, function (adapter) { return adapter.findOccurrences(resource, position); });
        };
        // --- references
        ExtHostLanguageFeatures.prototype.registerReferenceProvider = function (selector, provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new ReferenceAdapter(this._documents, provider);
            this._proxy.$registerReferenceSupport(handle, selector);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$findReferences = function (handle, resource, position, includeDeclaration) {
            return this._withAdapter(handle, ReferenceAdapter, function (adapter) { return adapter.findReferences(resource, position, includeDeclaration); });
        };
        // --- quick fix
        ExtHostLanguageFeatures.prototype.registerCodeActionProvider = function (selector, provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new QuickFixAdapter(this._documents, this._commands, provider);
            this._proxy.$registerQuickFixSupport(handle, selector);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$getQuickFixes = function (handle, resource, range, marker) {
            return this._withAdapter(handle, QuickFixAdapter, function (adapter) { return adapter.getQuickFixes(resource, range, marker); });
        };
        ExtHostLanguageFeatures.prototype.$runQuickFixAction = function (handle, resource, range, quickFix) {
            return this._withAdapter(handle, QuickFixAdapter, function (adapter) { return adapter.runQuickFixAction(resource, range, quickFix); });
        };
        // --- formatting
        ExtHostLanguageFeatures.prototype.registerDocumentFormattingEditProvider = function (selector, provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new DocumentFormattingAdapter(this._documents, provider);
            this._proxy.$registerDocumentFormattingSupport(handle, selector);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$formatDocument = function (handle, resource, options) {
            return this._withAdapter(handle, DocumentFormattingAdapter, function (adapter) { return adapter.formatDocument(resource, options); });
        };
        ExtHostLanguageFeatures.prototype.registerDocumentRangeFormattingEditProvider = function (selector, provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new RangeFormattingAdapter(this._documents, provider);
            this._proxy.$registerRangeFormattingSupport(handle, selector);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$formatRange = function (handle, resource, range, options) {
            return this._withAdapter(handle, RangeFormattingAdapter, function (adapter) { return adapter.formatRange(resource, range, options); });
        };
        ExtHostLanguageFeatures.prototype.registerOnTypeFormattingEditProvider = function (selector, provider, triggerCharacters) {
            var handle = this._nextHandle();
            this._adapter[handle] = new OnTypeFormattingAdapter(this._documents, provider);
            this._proxy.$registerOnTypeFormattingSupport(handle, selector, triggerCharacters);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$formatAfterKeystroke = function (handle, resource, position, ch, options) {
            return this._withAdapter(handle, OnTypeFormattingAdapter, function (adapter) { return adapter.formatAfterKeystroke(resource, position, ch, options); });
        };
        // --- navigate types
        ExtHostLanguageFeatures.prototype.registerWorkspaceSymbolProvider = function (provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new NavigateTypeAdapter(provider);
            this._proxy.$registerNavigateTypeSupport(handle);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$getNavigateToItems = function (handle, search) {
            return this._withAdapter(handle, NavigateTypeAdapter, function (adapter) { return adapter.getNavigateToItems(search); });
        };
        // --- rename
        ExtHostLanguageFeatures.prototype.registerRenameProvider = function (selector, provider) {
            var handle = this._nextHandle();
            this._adapter[handle] = new RenameAdapter(this._documents, provider);
            this._proxy.$registerRenameSupport(handle, selector);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$rename = function (handle, resource, position, newName) {
            return this._withAdapter(handle, RenameAdapter, function (adapter) { return adapter.rename(resource, position, newName); });
        };
        // --- suggestion
        ExtHostLanguageFeatures.prototype.registerCompletionItemProvider = function (selector, provider, triggerCharacters) {
            var handle = this._nextHandle();
            this._adapter[handle] = new SuggestAdapter(this._documents, provider);
            this._proxy.$registerSuggestSupport(handle, selector, triggerCharacters);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$suggest = function (handle, resource, position) {
            return this._withAdapter(handle, SuggestAdapter, function (adapter) { return adapter.suggest(resource, position); });
        };
        ExtHostLanguageFeatures.prototype.$getSuggestionDetails = function (handle, resource, position, suggestion) {
            return this._withAdapter(handle, SuggestAdapter, function (adapter) { return adapter.getSuggestionDetails(resource, position, suggestion); });
        };
        // --- parameter hints
        ExtHostLanguageFeatures.prototype.registerSignatureHelpProvider = function (selector, provider, triggerCharacters) {
            var handle = this._nextHandle();
            this._adapter[handle] = new ParameterHintsAdapter(this._documents, provider);
            this._proxy.$registerParameterHintsSupport(handle, selector, triggerCharacters);
            return this._createDisposable(handle);
        };
        ExtHostLanguageFeatures.prototype.$getParameterHints = function (handle, resource, position, triggerCharacter) {
            return this._withAdapter(handle, ParameterHintsAdapter, function (adapter) { return adapter.getParameterHints(resource, position, triggerCharacter); });
        };
        ExtHostLanguageFeatures._handlePool = 0;
        ExtHostLanguageFeatures = __decorate([
            thread_1.Remotable.ExtHostContext('ExtHostLanguageFeatures'),
            __param(0, thread_1.IThreadService)
        ], ExtHostLanguageFeatures);
        return ExtHostLanguageFeatures;
    }());
    exports.ExtHostLanguageFeatures = ExtHostLanguageFeatures;
    var MainThreadLanguageFeatures = (function () {
        function MainThreadLanguageFeatures(threadService, markerService) {
            this._registrations = Object.create(null);
            this._proxy = threadService.getRemotable(ExtHostLanguageFeatures);
            this._markerService = markerService;
        }
        MainThreadLanguageFeatures.prototype.$unregister = function (handle) {
            var registration = this._registrations[handle];
            if (registration) {
                registration.dispose();
                delete this._registrations[handle];
            }
            return undefined;
        };
        // --- outline
        MainThreadLanguageFeatures.prototype.$registerOutlineSupport = function (handle, selector) {
            var _this = this;
            this._registrations[handle] = quickOpen_1.OutlineRegistry.register(selector, {
                getOutline: function (resource) {
                    return _this._proxy.$getOutline(handle, resource);
                }
            });
            return undefined;
        };
        // --- code lens
        MainThreadLanguageFeatures.prototype.$registerCodeLensSupport = function (handle, selector) {
            var _this = this;
            this._registrations[handle] = codelens_1.CodeLensRegistry.register(selector, {
                findCodeLensSymbols: function (resource) {
                    return _this._proxy.$findCodeLensSymbols(handle, resource);
                },
                resolveCodeLensSymbol: function (resource, symbol) {
                    return _this._proxy.$resolveCodeLensSymbol(handle, resource, symbol);
                }
            });
            return undefined;
        };
        // --- declaration
        MainThreadLanguageFeatures.prototype.$registerDeclaractionSupport = function (handle, selector) {
            var _this = this;
            this._registrations[handle] = goToDeclaration_1.DeclarationRegistry.register(selector, {
                canFindDeclaration: function () {
                    return true;
                },
                findDeclaration: function (resource, position) {
                    return _this._proxy.$findDeclaration(handle, resource, position);
                }
            });
            return undefined;
        };
        // --- extra info
        MainThreadLanguageFeatures.prototype.$registerExtraInfoSupport = function (handle, selector) {
            var _this = this;
            this._registrations[handle] = hover_1.ExtraInfoRegistry.register(selector, {
                computeInfo: function (resource, position) {
                    return _this._proxy.$computeInfo(handle, resource, position);
                }
            });
            return undefined;
        };
        // --- occurrences
        MainThreadLanguageFeatures.prototype.$registerOccurrencesSupport = function (handle, selector) {
            var _this = this;
            this._registrations[handle] = wordHighlighter_1.OccurrencesRegistry.register(selector, {
                findOccurrences: function (resource, position) {
                    return _this._proxy.$findOccurrences(handle, resource, position);
                }
            });
            return undefined;
        };
        // --- references
        MainThreadLanguageFeatures.prototype.$registerReferenceSupport = function (handle, selector) {
            var _this = this;
            this._registrations[handle] = referenceSearch_1.ReferenceRegistry.register(selector, {
                canFindReferences: function () {
                    return true;
                },
                findReferences: function (resource, position, includeDeclaration) {
                    return _this._proxy.$findReferences(handle, resource, position, includeDeclaration);
                }
            });
            return undefined;
        };
        // --- quick fix
        MainThreadLanguageFeatures.prototype.$registerQuickFixSupport = function (handle, selector) {
            var _this = this;
            this._registrations[handle] = quickFix_1.QuickFixRegistry.register(selector, {
                getQuickFixes: function (resource, range) {
                    var markers = [];
                    _this._markerService.read({ resource: resource }).forEach(function (marker) {
                        if (range_1.Range.lift(marker).intersectRanges(range)) {
                            markers.push(marker);
                        }
                    });
                    return _this._proxy.$getQuickFixes(handle, resource, range, markers);
                },
                runQuickFixAction: function (resource, range, quickFix) {
                    return _this._proxy.$runQuickFixAction(handle, resource, range, quickFix);
                }
            });
            return undefined;
        };
        // --- formatting
        MainThreadLanguageFeatures.prototype.$registerDocumentFormattingSupport = function (handle, selector) {
            var _this = this;
            this._registrations[handle] = format_1.FormatRegistry.register(selector, {
                formatDocument: function (resource, options) {
                    return _this._proxy.$formatDocument(handle, resource, options);
                }
            });
            return undefined;
        };
        MainThreadLanguageFeatures.prototype.$registerRangeFormattingSupport = function (handle, selector) {
            var _this = this;
            this._registrations[handle] = format_1.FormatRegistry.register(selector, {
                formatRange: function (resource, range, options) {
                    return _this._proxy.$formatRange(handle, resource, range, options);
                }
            });
            return undefined;
        };
        MainThreadLanguageFeatures.prototype.$registerOnTypeFormattingSupport = function (handle, selector, autoFormatTriggerCharacters) {
            var _this = this;
            this._registrations[handle] = format_1.FormatOnTypeRegistry.register(selector, {
                autoFormatTriggerCharacters: autoFormatTriggerCharacters,
                formatAfterKeystroke: function (resource, position, ch, options) {
                    return _this._proxy.$formatAfterKeystroke(handle, resource, position, ch, options);
                }
            });
            return undefined;
        };
        // --- navigate type
        MainThreadLanguageFeatures.prototype.$registerNavigateTypeSupport = function (handle) {
            var _this = this;
            this._registrations[handle] = search_1.NavigateTypesSupportRegistry.register({
                getNavigateToItems: function (search) {
                    return _this._proxy.$getNavigateToItems(handle, search);
                }
            });
            return undefined;
        };
        // --- rename
        MainThreadLanguageFeatures.prototype.$registerRenameSupport = function (handle, selector) {
            var _this = this;
            this._registrations[handle] = rename_1.RenameRegistry.register(selector, {
                rename: function (resource, position, newName) {
                    return _this._proxy.$rename(handle, resource, position, newName);
                }
            });
            return undefined;
        };
        // --- suggest
        MainThreadLanguageFeatures.prototype.$registerSuggestSupport = function (handle, selector, triggerCharacters) {
            var _this = this;
            this._registrations[handle] = suggest_1.SuggestRegistry.register(selector, {
                suggest: function (resource, position, triggerCharacter) {
                    return _this._proxy.$suggest(handle, resource, position);
                },
                getSuggestionDetails: function (resource, position, suggestion) {
                    return _this._proxy.$getSuggestionDetails(handle, resource, position, suggestion);
                },
                getFilter: function () {
                    return modesFilters_1.DefaultFilter;
                },
                getTriggerCharacters: function () {
                    return triggerCharacters;
                },
                shouldShowEmptySuggestionList: function () {
                    return true;
                },
                shouldAutotriggerSuggest: function () {
                    return true;
                }
            });
            return undefined;
        };
        // --- parameter hints
        MainThreadLanguageFeatures.prototype.$registerParameterHintsSupport = function (handle, selector, triggerCharacter) {
            var _this = this;
            this._registrations[handle] = parameterHints_1.ParameterHintsRegistry.register(selector, {
                getParameterHints: function (resource, position, triggerCharacter) {
                    return _this._proxy.$getParameterHints(handle, resource, position, triggerCharacter);
                },
                getParameterHintsTriggerCharacters: function () {
                    return triggerCharacter;
                },
                shouldTriggerParameterHints: function (context, offset) {
                    return true;
                }
            });
            return undefined;
        };
        MainThreadLanguageFeatures = __decorate([
            thread_1.Remotable.MainContext('MainThreadLanguageFeatures'),
            __param(0, thread_1.IThreadService),
            __param(1, markers_1.IMarkerService)
        ], MainThreadLanguageFeatures);
        return MainThreadLanguageFeatures;
    }());
    exports.MainThreadLanguageFeatures = MainThreadLanguageFeatures;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/api/node/extHost.api.impl", ["require", "exports", 'vs/base/common/event', 'vs/editor/common/modes/languageSelector', 'vs/base/common/platform', 'vs/base/common/strings', 'vs/platform/thread/common/thread', 'vs/base/common/errors', 'vs/workbench/api/node/extHostFileSystemEventService', 'vs/workbench/api/node/extHostDocuments', 'vs/workbench/api/node/extHostConfiguration', 'vs/workbench/api/node/extHostDiagnostics', 'vs/workbench/api/node/extHostWorkspace', 'vs/workbench/api/node/extHostQuickOpen', 'vs/workbench/api/node/extHostStatusBar', 'vs/workbench/api/node/extHostCommands', 'vs/workbench/api/node/extHostOutputService', 'vs/workbench/api/node/extHostMessageService', 'vs/workbench/api/node/extHostEditors', 'vs/workbench/api/node/extHostLanguages', 'vs/workbench/api/node/extHostLanguageFeatures', 'vs/workbench/api/node/extHostApiCommands', 'vs/workbench/api/node/extHostTypes', 'vs/editor/common/modes', 'vs/editor/common/services/modeService', 'vs/base/common/uri', 'vs/base/common/severity', 'vs/editor/common/editorCommon', 'vs/platform/extensions/common/extensions', 'vs/platform/extensions/common/extensionsRegistry', 'vs/platform/workspace/common/workspace', 'vs/base/common/cancellation', 'vs/workbench/api/node/mainThreadEditors', 'vs/base/common/paths', 'vs/platform/telemetry/common/telemetry'], function (require, exports, event_1, languageSelector_1, Platform, strings_1, thread_1, errors, extHostFileSystemEventService_1, extHostDocuments_1, extHostConfiguration_1, extHostDiagnostics_1, extHostWorkspace_1, extHostQuickOpen_1, extHostStatusBar_1, extHostCommands_1, extHostOutputService_1, extHostMessageService_1, extHostEditors_1, extHostLanguages_1, extHostLanguageFeatures_1, extHostApiCommands_1, extHostTypes, Modes, modeService_1, uri_1, severity_1, EditorCommon, extensions_1, extensionsRegistry_1, workspace_1, cancellation_1, mainThreadEditors_1, paths, telemetry_1) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    /**
     * This class implements the API described in vscode.d.ts,
     * for the case of the extensionHost host process
     */
    var ExtHostAPIImplementation = (function () {
        function ExtHostAPIImplementation(threadService, extensionService, contextService, telemetryService) {
            var _this = this;
            this._threadService = threadService;
            this._proxy = threadService.getRemotable(MainProcessVSCodeAPIHelper);
            this.version = contextService.getConfiguration().env.version;
            this.Uri = uri_1.default;
            this.Location = extHostTypes.Location;
            this.Diagnostic = extHostTypes.Diagnostic;
            this.DiagnosticSeverity = extHostTypes.DiagnosticSeverity;
            this.EventEmitter = event_1.Emitter;
            this.Disposable = extHostTypes.Disposable;
            this.TextEdit = extHostTypes.TextEdit;
            this.WorkspaceEdit = extHostTypes.WorkspaceEdit;
            this.Position = extHostTypes.Position;
            this.Range = extHostTypes.Range;
            this.Selection = extHostTypes.Selection;
            this.CancellationTokenSource = cancellation_1.CancellationTokenSource;
            this.Hover = extHostTypes.Hover;
            this.SymbolKind = extHostTypes.SymbolKind;
            this.SymbolInformation = extHostTypes.SymbolInformation;
            this.DocumentHighlightKind = extHostTypes.DocumentHighlightKind;
            this.DocumentHighlight = extHostTypes.DocumentHighlight;
            this.CodeLens = extHostTypes.CodeLens;
            this.ParameterInformation = extHostTypes.ParameterInformation;
            this.SignatureInformation = extHostTypes.SignatureInformation;
            this.SignatureHelp = extHostTypes.SignatureHelp;
            this.CompletionItem = extHostTypes.CompletionItem;
            this.CompletionItemKind = extHostTypes.CompletionItemKind;
            this.CompletionList = extHostTypes.CompletionList;
            this.ViewColumn = extHostTypes.ViewColumn;
            this.StatusBarAlignment = extHostTypes.StatusBarAlignment;
            this.IndentAction = Modes.IndentAction;
            this.OverviewRulerLane = EditorCommon.OverviewRulerLane;
            this.TextEditorRevealType = mainThreadEditors_1.TextEditorRevealType;
            this.EndOfLine = extHostTypes.EndOfLine;
            this.TextEditorCursorStyle = EditorCommon.TextEditorCursorStyle;
            errors.setUnexpectedErrorHandler(function (err) {
                _this._proxy.onUnexpectedExtHostError(errors.transformErrorForSerialization(err));
            });
            var extHostCommands = this._threadService.getRemotable(extHostCommands_1.ExtHostCommands);
            var extHostEditors = this._threadService.getRemotable(extHostEditors_1.ExtHostEditors);
            var extHostMessageService = new extHostMessageService_1.ExtHostMessageService(this._threadService, this.commands);
            var extHostQuickOpen = this._threadService.getRemotable(extHostQuickOpen_1.ExtHostQuickOpen);
            var extHostStatusBar = new extHostStatusBar_1.ExtHostStatusBar(this._threadService);
            var extHostOutputService = new extHostOutputService_1.ExtHostOutputService(this._threadService);
            // env namespace
            var telemetryInfo;
            this.env = Object.freeze({
                get machineId() { return telemetryInfo.machineId; },
                get sessionId() { return telemetryInfo.sessionId; },
                get language() { return Platform.language; }
            });
            telemetryService.getTelemetryInfo().then(function (info) { return telemetryInfo = info; }, errors.onUnexpectedError);
            // commands namespace
            this.commands = {
                registerCommand: function (id, command, thisArgs) {
                    return extHostCommands.registerCommand(id, command, thisArgs);
                },
                registerTextEditorCommand: function (id, callback, thisArg) {
                    var actualCallback = thisArg ? callback.bind(thisArg) : callback;
                    return extHostCommands.registerCommand(id, function () {
                        var activeTextEditor = extHostEditors.getActiveTextEditor();
                        if (!activeTextEditor) {
                            console.warn('Cannot execute ' + id + ' because there is no active text editor.');
                            return;
                        }
                        activeTextEditor.edit(function (edit) {
                            actualCallback(activeTextEditor, edit);
                        }).then(function (result) {
                            if (!result) {
                                console.warn('Edits from command ' + id + ' were not applied.');
                            }
                        }, function (err) {
                            console.warn('An error occured while running command ' + id, err);
                        });
                    });
                },
                executeCommand: function (id) {
                    var args = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        args[_i - 1] = arguments[_i];
                    }
                    return extHostCommands.executeCommand.apply(extHostCommands, [id].concat(args));
                },
                getCommands: function (filterInternal) {
                    if (filterInternal === void 0) { filterInternal = false; }
                    return extHostCommands.getCommands(filterInternal);
                }
            };
            this.window = {
                get activeTextEditor() {
                    return extHostEditors.getActiveTextEditor();
                },
                get visibleTextEditors() {
                    return extHostEditors.getVisibleTextEditors();
                },
                showTextDocument: function (document, column, preserveFocus) {
                    return extHostEditors.showTextDocument(document, column, preserveFocus);
                },
                createTextEditorDecorationType: function (options) {
                    return extHostEditors.createTextEditorDecorationType(options);
                },
                onDidChangeActiveTextEditor: extHostEditors.onDidChangeActiveTextEditor.bind(extHostEditors),
                onDidChangeTextEditorSelection: function (listener, thisArgs, disposables) {
                    return extHostEditors.onDidChangeTextEditorSelection(listener, thisArgs, disposables);
                },
                onDidChangeTextEditorOptions: function (listener, thisArgs, disposables) {
                    return extHostEditors.onDidChangeTextEditorOptions(listener, thisArgs, disposables);
                },
                onDidChangeTextEditorViewColumn: function (listener, thisArg, disposables) {
                    return extHostEditors.onDidChangeTextEditorViewColumn(listener, thisArg, disposables);
                },
                showInformationMessage: function (message) {
                    var items = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        items[_i - 1] = arguments[_i];
                    }
                    return extHostMessageService.showMessage(severity_1.default.Info, message, items);
                },
                showWarningMessage: function (message) {
                    var items = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        items[_i - 1] = arguments[_i];
                    }
                    return extHostMessageService.showMessage(severity_1.default.Warning, message, items);
                },
                showErrorMessage: function (message) {
                    var items = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        items[_i - 1] = arguments[_i];
                    }
                    return extHostMessageService.showMessage(severity_1.default.Error, message, items);
                },
                showQuickPick: function (items, options) {
                    return extHostQuickOpen.show(items, options);
                },
                showInputBox: extHostQuickOpen.input.bind(extHostQuickOpen),
                createStatusBarItem: function (position, priority) {
                    return extHostStatusBar.createStatusBarEntry(position, priority);
                },
                setStatusBarMessage: function (text, timeoutOrThenable) {
                    return extHostStatusBar.setStatusBarMessage(text, timeoutOrThenable);
                },
                createOutputChannel: function (name) {
                    return extHostOutputService.createOutputChannel(name);
                }
            };
            //
            var workspacePath = contextService.getWorkspace() ? contextService.getWorkspace().resource.fsPath : undefined;
            var extHostFileSystemEvent = threadService.getRemotable(extHostFileSystemEventService_1.ExtHostFileSystemEventService);
            var extHostWorkspace = new extHostWorkspace_1.ExtHostWorkspace(this._threadService, workspacePath);
            var extHostDocuments = this._threadService.getRemotable(extHostDocuments_1.ExtHostModelService);
            this.workspace = Object.freeze({
                get rootPath() {
                    return extHostWorkspace.getPath();
                },
                set rootPath(value) {
                    throw errors.readonly();
                },
                asRelativePath: function (pathOrUri) {
                    return extHostWorkspace.getRelativePath(pathOrUri);
                },
                findFiles: function (include, exclude, maxResults, token) {
                    return extHostWorkspace.findFiles(include, exclude, maxResults, token);
                },
                saveAll: function (includeUntitled) {
                    return extHostWorkspace.saveAll(includeUntitled);
                },
                applyEdit: function (edit) {
                    return extHostWorkspace.appyEdit(edit);
                },
                createFileSystemWatcher: function (pattern, ignoreCreate, ignoreChange, ignoreDelete) {
                    return extHostFileSystemEvent.createFileSystemWatcher(pattern, ignoreCreate, ignoreChange, ignoreDelete);
                },
                get textDocuments() {
                    return extHostDocuments.getAllDocumentData().map(function (data) { return data.document; });
                },
                set textDocuments(value) {
                    throw errors.readonly();
                },
                openTextDocument: function (uriOrFileName) {
                    var uri;
                    if (typeof uriOrFileName === 'string') {
                        uri = uri_1.default.file(uriOrFileName);
                    }
                    else if (uriOrFileName instanceof uri_1.default) {
                        uri = uriOrFileName;
                    }
                    else {
                        throw new Error('illegal argument - uriOrFileName');
                    }
                    return extHostDocuments.ensureDocumentData(uri).then(function () {
                        var data = extHostDocuments.getDocumentData(uri);
                        return data && data.document;
                    });
                },
                registerTextDocumentContentProvider: function (scheme, provider) {
                    return extHostDocuments.registerTextDocumentContentProvider(scheme, provider);
                },
                onDidOpenTextDocument: function (listener, thisArgs, disposables) {
                    return extHostDocuments.onDidAddDocument(listener, thisArgs, disposables);
                },
                onDidCloseTextDocument: function (listener, thisArgs, disposables) {
                    return extHostDocuments.onDidRemoveDocument(listener, thisArgs, disposables);
                },
                onDidChangeTextDocument: function (listener, thisArgs, disposables) {
                    return extHostDocuments.onDidChangeDocument(listener, thisArgs, disposables);
                },
                onDidSaveTextDocument: function (listener, thisArgs, disposables) {
                    return extHostDocuments.onDidSaveDocument(listener, thisArgs, disposables);
                },
                onDidChangeConfiguration: function (listener, thisArgs, disposables) {
                    return extHostConfiguration.onDidChangeConfiguration(listener, thisArgs, disposables);
                },
                getConfiguration: function (section) {
                    return extHostConfiguration.getConfiguration(section);
                }
            });
            //
            extHostApiCommands_1.registerApiCommands(threadService);
            //
            var languages = new extHostLanguages_1.ExtHostLanguages(this._threadService);
            var extHostDiagnostics = new extHostDiagnostics_1.ExtHostDiagnostics(this._threadService);
            var languageFeatures = threadService.getRemotable(extHostLanguageFeatures_1.ExtHostLanguageFeatures);
            this.languages = {
                createDiagnosticCollection: function (name) {
                    return extHostDiagnostics.createDiagnosticCollection(name);
                },
                getLanguages: function () {
                    return languages.getLanguages();
                },
                match: function (selector, document) {
                    return languageSelector_1.score(selector, document.uri, document.languageId);
                },
                registerCodeActionsProvider: function (selector, provider) {
                    return languageFeatures.registerCodeActionProvider(selector, provider);
                },
                registerCodeLensProvider: function (selector, provider) {
                    return languageFeatures.registerCodeLensProvider(selector, provider);
                },
                registerDefinitionProvider: function (selector, provider) {
                    return languageFeatures.registerDefinitionProvider(selector, provider);
                },
                registerHoverProvider: function (selector, provider) {
                    return languageFeatures.registerHoverProvider(selector, provider);
                },
                registerDocumentHighlightProvider: function (selector, provider) {
                    return languageFeatures.registerDocumentHighlightProvider(selector, provider);
                },
                registerReferenceProvider: function (selector, provider) {
                    return languageFeatures.registerReferenceProvider(selector, provider);
                },
                registerRenameProvider: function (selector, provider) {
                    return languageFeatures.registerRenameProvider(selector, provider);
                },
                registerDocumentSymbolProvider: function (selector, provider) {
                    return languageFeatures.registerDocumentSymbolProvider(selector, provider);
                },
                registerWorkspaceSymbolProvider: function (provider) {
                    return languageFeatures.registerWorkspaceSymbolProvider(provider);
                },
                registerDocumentFormattingEditProvider: function (selector, provider) {
                    return languageFeatures.registerDocumentFormattingEditProvider(selector, provider);
                },
                registerDocumentRangeFormattingEditProvider: function (selector, provider) {
                    return languageFeatures.registerDocumentRangeFormattingEditProvider(selector, provider);
                },
                registerOnTypeFormattingEditProvider: function (selector, provider, firstTriggerCharacter) {
                    var moreTriggerCharacters = [];
                    for (var _i = 3; _i < arguments.length; _i++) {
                        moreTriggerCharacters[_i - 3] = arguments[_i];
                    }
                    return languageFeatures.registerOnTypeFormattingEditProvider(selector, provider, [firstTriggerCharacter].concat(moreTriggerCharacters));
                },
                registerSignatureHelpProvider: function (selector, provider) {
                    var triggerCharacters = [];
                    for (var _i = 2; _i < arguments.length; _i++) {
                        triggerCharacters[_i - 2] = arguments[_i];
                    }
                    return languageFeatures.registerSignatureHelpProvider(selector, provider, triggerCharacters);
                },
                registerCompletionItemProvider: function (selector, provider) {
                    var triggerCharacters = [];
                    for (var _i = 2; _i < arguments.length; _i++) {
                        triggerCharacters[_i - 2] = arguments[_i];
                    }
                    return languageFeatures.registerCompletionItemProvider(selector, provider, triggerCharacters);
                },
                setLanguageConfiguration: function (language, configuration) {
                    return _this._setLanguageConfiguration(language, configuration);
                }
            };
            var extHostConfiguration = threadService.getRemotable(extHostConfiguration_1.ExtHostConfiguration);
            //
            this.extensions = {
                getExtension: function (extensionId) {
                    var desc = extensionsRegistry_1.ExtensionsRegistry.getExtensionDescription(extensionId);
                    if (desc) {
                        return new Extension(extensionService, desc);
                    }
                },
                get all() {
                    return extensionsRegistry_1.ExtensionsRegistry.getAllExtensionDescriptions().map(function (desc) { return new Extension(extensionService, desc); });
                }
            };
            // Intentionally calling a function for typechecking purposes
            defineAPI(this);
        }
        ExtHostAPIImplementation.generateDisposeToken = function () {
            return String(++ExtHostAPIImplementation._LAST_REGISTER_TOKEN);
        };
        ExtHostAPIImplementation.prototype._disposableFromToken = function (disposeToken) {
            var _this = this;
            return new extHostTypes.Disposable(function () { return _this._proxy.disposeByToken(disposeToken); });
        };
        ExtHostAPIImplementation.prototype._setLanguageConfiguration = function (modeId, configuration) {
            var wordPattern = configuration.wordPattern;
            // check for a valid word pattern
            if (wordPattern && strings_1.regExpLeadsToEndlessLoop(wordPattern)) {
                throw new Error("Invalid language configuration: wordPattern '" + wordPattern + "' is not allowed to match the empty string.");
            }
            // word definition
            if (wordPattern) {
                extHostDocuments_1.setWordDefinitionFor(modeId, wordPattern);
            }
            else {
                extHostDocuments_1.setWordDefinitionFor(modeId, null);
            }
            return this.Modes_RichEditSupport_register(modeId, configuration);
        };
        ExtHostAPIImplementation.prototype.Modes_RichEditSupport_register = function (modeId, configuration) {
            var disposeToken = ExtHostAPIImplementation.generateDisposeToken();
            this._proxy.Modes_RichEditSupport_register(disposeToken, modeId, configuration);
            return this._disposableFromToken(disposeToken);
        };
        ExtHostAPIImplementation._LAST_REGISTER_TOKEN = 0;
        ExtHostAPIImplementation = __decorate([
            __param(0, thread_1.IThreadService),
            __param(1, extensions_1.IExtensionService),
            __param(2, workspace_1.IWorkspaceContextService),
            __param(3, telemetry_1.ITelemetryService)
        ], ExtHostAPIImplementation);
        return ExtHostAPIImplementation;
    }());
    exports.ExtHostAPIImplementation = ExtHostAPIImplementation;
    var Extension = (function () {
        function Extension(extensionService, description) {
            this._extensionService = extensionService;
            this.id = description.id;
            this.extensionPath = paths.normalize(description.extensionFolderPath, true);
            this.packageJSON = description;
        }
        Object.defineProperty(Extension.prototype, "isActive", {
            get: function () {
                return this._extensionService.isActivated(this.id);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Extension.prototype, "exports", {
            get: function () {
                return this._extensionService.get(this.id);
            },
            enumerable: true,
            configurable: true
        });
        Extension.prototype.activate = function () {
            var _this = this;
            return this._extensionService.activateById(this.id).then(function () { return _this.exports; });
        };
        return Extension;
    }());
    function defineAPI(impl) {
        var node_module = require.__$__nodeRequire('module');
        var original = node_module._load;
        node_module._load = function load(request, parent, isMain) {
            if (request === 'vscode') {
                return impl;
            }
            return original.apply(this, arguments);
        };
        define('vscode', [], impl);
    }
    var MainProcessVSCodeAPIHelper = (function () {
        function MainProcessVSCodeAPIHelper(modeService) {
            this._modeService = modeService;
            this._token2Dispose = {};
        }
        MainProcessVSCodeAPIHelper.prototype.onUnexpectedExtHostError = function (err) {
            errors.onUnexpectedError(err);
        };
        MainProcessVSCodeAPIHelper.prototype.disposeByToken = function (disposeToken) {
            if (this._token2Dispose[disposeToken]) {
                this._token2Dispose[disposeToken].dispose();
                delete this._token2Dispose[disposeToken];
            }
        };
        MainProcessVSCodeAPIHelper.prototype.Modes_RichEditSupport_register = function (disposeToken, modeId, configuration) {
            this._token2Dispose[disposeToken] = this._modeService.registerRichEditSupport(modeId, configuration);
        };
        MainProcessVSCodeAPIHelper = __decorate([
            thread_1.Remotable.MainContext('MainProcessVSCodeAPIHelper'),
            __param(0, modeService_1.IModeService)
        ], MainProcessVSCodeAPIHelper);
        return MainProcessVSCodeAPIHelper;
    }());
    exports.MainProcessVSCodeAPIHelper = MainProcessVSCodeAPIHelper;
});

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/telemetry/node/nodeAppInsightsTelemetryAppender", ["require", "exports", 'vs/base/common/errors', 'vs/platform/storage/common/storage', 'vs/platform/workspace/common/workspace', 'vs/base/node/aiAdapter', 'winreg', 'os'], function (require, exports, errors, storage_1, workspace_1, aiAdapter_1, winreg, os) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var StorageKeys = (function () {
        function StorageKeys() {
        }
        StorageKeys.sqmUserId = 'telemetry.sqm.userId';
        StorageKeys.sqmMachineId = 'telemetry.sqm.machineId';
        StorageKeys.lastSessionDate = 'telemetry.lastSessionDate';
        StorageKeys.firstSessionDate = 'telemetry.firstSessionDate';
        return StorageKeys;
    }());
    var NodeAppInsightsTelemetryAppender = (function () {
        function NodeAppInsightsTelemetryAppender(storageService, contextService, 
            /* for test only */
            client) {
            this.commonProperties = {};
            this.commonMetrics = {};
            this.contextService = contextService;
            this.storageService = storageService;
            var config = this.contextService.getConfiguration().env.aiConfig;
            var key = config ? config.key : null;
            var asimovKey = config ? config.asimovKey : null;
            // for test
            if (client) {
                this.appInsights = client;
                if (asimovKey) {
                    this.appInsightsVortex = client;
                }
                return;
            }
            if (key) {
                this.appInsights = new aiAdapter_1.AIAdapter(key, NodeAppInsightsTelemetryAppender.EVENT_NAME_PREFIX);
            }
            if (asimovKey) {
                this.appInsightsVortex = new aiAdapter_1.AIAdapter(asimovKey, NodeAppInsightsTelemetryAppender.EVENT_NAME_PREFIX);
            }
            this.loadAddtionaProperties();
        }
        NodeAppInsightsTelemetryAppender.prototype.loadAddtionaProperties = function () {
            var _this = this;
            // add shell & render version
            if (process.versions) {
                this.commonProperties['version.shell'] = process.versions['electron'];
                this.commonProperties['version.renderer'] = process.versions['chrome'];
            }
            // add SQM data for windows machines
            if (process.platform === 'win32') {
                var sqmUserId = this.storageService.get(StorageKeys.sqmUserId);
                if (sqmUserId) {
                    this.commonProperties['sqm.userid'] = sqmUserId;
                }
                else {
                    this.getWinRegKeyData(NodeAppInsightsTelemetryAppender.SQM_KEY, 'UserId', winreg.HKCU, function (error, result) {
                        if (!error && result) {
                            _this.commonProperties['sqm.userid'] = result;
                            _this.storageService.store(StorageKeys.sqmUserId, result);
                        }
                    });
                }
                var sqmMachineId = this.storageService.get(StorageKeys.sqmMachineId);
                if (sqmMachineId) {
                    this.commonProperties['sqm.machineid'] = sqmMachineId;
                }
                else {
                    this.getWinRegKeyData(NodeAppInsightsTelemetryAppender.SQM_KEY, 'MachineId', winreg.HKLM, function (error, result) {
                        if (!error && result) {
                            _this.commonProperties['sqm.machineid'] = result;
                            _this.storageService.store(StorageKeys.sqmMachineId, result);
                        }
                    });
                }
            }
            var firstSessionDate = this.storageService.get(StorageKeys.firstSessionDate);
            if (!firstSessionDate) {
                firstSessionDate = (new Date()).toUTCString();
                this.storageService.store(StorageKeys.firstSessionDate, firstSessionDate);
            }
            this.commonProperties['firstSessionDate'] = firstSessionDate;
            //report last session date and isNewSession flag
            var lastSessionDate = this.storageService.get(StorageKeys.lastSessionDate);
            if (!lastSessionDate) {
                this.commonMetrics['isNewSession'] = 1;
            }
            else {
                this.commonMetrics['isNewSession'] = 0;
                this.commonProperties['lastSessionDate'] = lastSessionDate;
            }
            this.storageService.store(StorageKeys.lastSessionDate, (new Date()).toUTCString());
            if (os) {
                this.commonProperties['osVersion'] = os.release();
            }
        };
        NodeAppInsightsTelemetryAppender.prototype.getWinRegKeyData = function (key, name, hive, callback) {
            if (process.platform === 'win32') {
                try {
                    var reg = new winreg({
                        hive: hive,
                        key: key
                    });
                    reg.get(name, function (e, result) {
                        if (e || !result) {
                            callback(e, null);
                        }
                        else {
                            callback(null, result.value);
                        }
                    });
                }
                catch (err) {
                    errors.onUnexpectedError(err);
                    callback(err, null);
                }
            }
            else {
                callback(null, null);
            }
        };
        NodeAppInsightsTelemetryAppender.prototype.log = function (eventName, data) {
            data = data || Object.create(null);
            data = this.addCommonMetrics(data);
            data = this.addCommonProperties(data);
            if (this.appInsights) {
                this.appInsights.log(eventName, data);
            }
            if (this.appInsightsVortex) {
                this.appInsightsVortex.log(eventName, data);
            }
        };
        NodeAppInsightsTelemetryAppender.prototype.dispose = function () {
            if (this.appInsights) {
                this.appInsights.dispose();
            }
            if (this.appInsightsVortex) {
                this.appInsightsVortex.dispose();
            }
            this.appInsights = null;
            this.appInsightsVortex = null;
        };
        NodeAppInsightsTelemetryAppender.prototype.addCommonProperties = function (properties) {
            for (var prop in this.commonProperties) {
                properties['common.' + prop] = this.commonProperties[prop];
            }
            return properties;
        };
        NodeAppInsightsTelemetryAppender.prototype.addCommonMetrics = function (metrics) {
            for (var prop in this.commonMetrics) {
                metrics['common.' + prop] = this.commonMetrics[prop];
            }
            return metrics;
        };
        NodeAppInsightsTelemetryAppender.EVENT_NAME_PREFIX = 'monacoworkbench';
        NodeAppInsightsTelemetryAppender.SQM_KEY = '\\Software\\Microsoft\\SQMClient';
        NodeAppInsightsTelemetryAppender = __decorate([
            __param(0, storage_1.IStorageService),
            __param(1, workspace_1.IWorkspaceContextService)
        ], NodeAppInsightsTelemetryAppender);
        return NodeAppInsightsTelemetryAppender;
    }());
    exports.NodeAppInsightsTelemetryAppender = NodeAppInsightsTelemetryAppender;
});

define("vs/workbench/parts/telemetry/node/appInsights.telemetry.contribution", ["require", "exports", 'vs/platform/platform', 'vs/platform/telemetry/common/abstractTelemetryService', 'vs/workbench/parts/telemetry/node/nodeAppInsightsTelemetryAppender', 'vs/platform/instantiation/common/descriptors'], function (require, exports, Platform, AbstractTelemetryService, AppInsightsTelemetryAppender, descriptors_1) {
    "use strict";
    var descriptor = descriptors_1.createSyncDescriptor(AppInsightsTelemetryAppender.NodeAppInsightsTelemetryAppender);
    Platform.Registry.as(AbstractTelemetryService.Extenstions.TelemetryAppenders).registerTelemetryAppenderDescriptor(descriptor);
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/base/node/zip", ["require", "exports", 'vs/nls!vs/base/node/zip', 'path', 'fs', 'vs/base/common/async', 'vs/base/node/pfs', 'vs/base/common/winjs.base', 'yauzl'], function (require, exports, nls, path, fs_1, async_1, pfs_1, winjs_base_1, yauzl_1) {
    "use strict";
    function modeFromEntry(entry) {
        var attr = entry.externalFileAttributes >> 16 || 33188;
        return [448 /* S_IRWXU */, 56 /* S_IRWXG */, 7 /* S_IRWXO */]
            .map(function (mask) { return attr & mask; })
            .reduce(function (a, b) { return a + b; }, attr & 61440 /* S_IFMT */);
    }
    function extractEntry(zipfile, entry, targetPath, options) {
        var fileName = entry.fileName.replace(options.sourcePathRegex, '');
        var dirName = path.dirname(fileName);
        var targetDirName = path.join(targetPath, dirName);
        var targetFileName = path.join(targetPath, fileName);
        var mode = modeFromEntry(entry);
        return async_1.ninvoke(zipfile, zipfile.openReadStream, entry)
            .then(function (ostream) { return pfs_1.mkdirp(targetDirName)
            .then(function () { return new winjs_base_1.Promise(function (c, e) {
            var istream = fs_1.createWriteStream(targetFileName, { mode: mode });
            istream.once('finish', function () { return c(null); });
            istream.once('error', e);
            ostream.once('error', e);
            ostream.pipe(istream);
        }); }); });
    }
    function extractZip(zipfile, targetPath, options) {
        return new winjs_base_1.Promise(function (c, e) {
            var promises = [];
            zipfile.once('error', e);
            zipfile.on('entry', function (entry) {
                if (!options.sourcePathRegex.test(entry.fileName)) {
                    return;
                }
                promises.push(extractEntry(zipfile, entry, targetPath, options));
            });
            zipfile.once('close', function () { return winjs_base_1.Promise.join(promises).done(c, e); });
        });
    }
    function extract(zipPath, targetPath, options) {
        var sourcePathRegex = new RegExp(options.sourcePath ? "^" + options.sourcePath : '');
        var promise = async_1.nfcall(yauzl_1.open, zipPath);
        if (options.overwrite) {
            promise = promise.then(function (zipfile) { pfs_1.rimraf(targetPath); return zipfile; });
        }
        return promise.then(function (zipfile) { return extractZip(zipfile, targetPath, { sourcePathRegex: sourcePathRegex }); });
    }
    exports.extract = extract;
    function read(zipPath, filePath) {
        return async_1.nfcall(yauzl_1.open, zipPath).then(function (zipfile) {
            return new winjs_base_1.TPromise(function (c, e) {
                zipfile.on('entry', function (entry) {
                    if (entry.fileName === filePath) {
                        async_1.ninvoke(zipfile, zipfile.openReadStream, entry).done(function (stream) { return c(stream); }, function (err) { return e(err); });
                    }
                });
                zipfile.once('close', function () { return e(new Error(nls.localize(0, null, filePath))); });
            });
        });
    }
    function buffer(zipPath, filePath) {
        return read(zipPath, filePath).then(function (stream) {
            return new winjs_base_1.TPromise(function (c, e) {
                var buffers = [];
                stream.once('error', e);
                stream.on('data', function (b) { return buffers.push(b); });
                stream.on('end', function () { return c(Buffer.concat(buffers)); });
            });
        });
    }
    exports.buffer = buffer;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
define("vs/workbench/parts/extensions/node/extensionsService", ["require", "exports", 'vs/nls!vs/workbench/parts/extensions/node/extensionsService', 'os', 'path', 'vs/base/common/types', 'vs/base/common/service', 'vs/base/node/pfs', 'vs/base/common/objects', 'vs/base/common/arrays', 'vs/base/node/zip', 'vs/base/common/winjs.base', 'vs/workbench/parts/extensions/common/extensions', 'vs/base/node/request', 'vs/base/node/proxy', 'vs/workbench/services/workspace/common/contextService', 'vs/base/common/async', 'vs/base/common/event', 'vs/workbench/node/userSettings', 'semver', 'vs/base/common/collections', 'vs/platform/extensions/node/extensionValidator'], function (require, exports, nls, os_1, path, types, service_1, pfs, objects_1, arrays_1, zip_1, winjs_base_1, extensions_1, request_1, proxy_1, contextService_1, async_1, event_1, userSettings_1, semver, collections_1, extensionValidator_1) {
    'use strict';
    function parseManifest(raw) {
        return new winjs_base_1.Promise(function (c, e) {
            try {
                c(JSON.parse(raw));
            }
            catch (err) {
                e(new Error(nls.localize(0, null)));
            }
        });
    }
    function validate(zipPath, extension, version) {
        if (version === void 0) { version = extension && extension.version; }
        return zip_1.buffer(zipPath, 'extension/package.json')
            .then(function (buffer) { return parseManifest(buffer.toString('utf8')); })
            .then(function (manifest) {
            if (extension) {
                if (extension.name !== manifest.name) {
                    return winjs_base_1.Promise.wrapError(Error(nls.localize(1, null)));
                }
                if (extension.publisher !== manifest.publisher) {
                    return winjs_base_1.Promise.wrapError(Error(nls.localize(2, null)));
                }
                if (version !== manifest.version) {
                    return winjs_base_1.Promise.wrapError(Error(nls.localize(3, null)));
                }
            }
            return winjs_base_1.TPromise.as(manifest);
        });
    }
    function createExtension(manifest, galleryInformation, path) {
        var extension = {
            name: manifest.name,
            displayName: manifest.displayName || manifest.name,
            publisher: manifest.publisher,
            version: manifest.version,
            engines: { vscode: manifest.engines.vscode },
            description: manifest.description || ''
        };
        if (galleryInformation) {
            extension.galleryInformation = galleryInformation;
        }
        if (path) {
            extension.path = path;
        }
        return extension;
    }
    function getExtensionId(extension, version) {
        if (version === void 0) { version = extension.version; }
        return extension.publisher + "." + extension.name + "-" + version;
    }
    var ExtensionsService = (function () {
        function ExtensionsService(contextService) {
            this.contextService = contextService;
            this.serviceId = extensions_1.IExtensionsService;
            this._onInstallExtension = new event_1.Emitter();
            this.onInstallExtension = this._onInstallExtension.event;
            this._onDidInstallExtension = new event_1.Emitter();
            this.onDidInstallExtension = this._onDidInstallExtension.event;
            this._onUninstallExtension = new event_1.Emitter();
            this.onUninstallExtension = this._onUninstallExtension.event;
            this._onDidUninstallExtension = new event_1.Emitter();
            this.onDidUninstallExtension = this._onDidUninstallExtension.event;
            var env = contextService.getConfiguration().env;
            this.extensionsPath = env.userExtensionsHome;
            this.obsoletePath = path.join(this.extensionsPath, '.obsolete');
            this.obsoleteFileLimiter = new async_1.Limiter(1);
        }
        ExtensionsService.prototype.install = function (arg) {
            var _this = this;
            if (types.isString(arg)) {
                return this.installFromZip(arg);
            }
            var extension = arg;
            return this.isObsolete(extension).then(function (obsolete) {
                if (obsolete) {
                    return winjs_base_1.TPromise.wrapError(new Error(nls.localize(4, null, extension.name)));
                }
                return _this.installFromGallery(arg);
            });
        };
        ExtensionsService.prototype.installFromGallery = function (extension) {
            var _this = this;
            var galleryInformation = extension.galleryInformation;
            if (!galleryInformation) {
                return winjs_base_1.TPromise.wrapError(new Error(nls.localize(5, null)));
            }
            this._onInstallExtension.fire(extension);
            return this.getLastValidExtensionVersion(extension, extension.galleryInformation.versions).then(function (versionInfo) {
                var version = versionInfo.version;
                var url = versionInfo.downloadUrl;
                var headers = versionInfo.downloadHeaders;
                var zipPath = path.join(os_1.tmpdir(), galleryInformation.id);
                var extensionPath = path.join(_this.extensionsPath, getExtensionId(extension, version));
                var manifestPath = path.join(extensionPath, 'package.json');
                return _this.request(url)
                    .then(function (opts) { return objects_1.assign(opts, { headers: headers }); })
                    .then(function (opts) { return request_1.download(zipPath, opts); })
                    .then(function () { return validate(zipPath, extension, version); })
                    .then(function (manifest) { return zip_1.extract(zipPath, extensionPath, { sourcePath: 'extension', overwrite: true }).then(function () { return manifest; }); })
                    .then(function (manifest) { return objects_1.assign({ __metadata: galleryInformation }, manifest); })
                    .then(function (manifest) { return pfs.writeFile(manifestPath, JSON.stringify(manifest, null, '\t')); })
                    .then(function () { _this._onDidInstallExtension.fire({ extension: extension }); return extension; })
                    .then(null, function (error) { _this._onDidInstallExtension.fire({ extension: extension, error: error }); return winjs_base_1.TPromise.wrapError(error); });
            });
        };
        ExtensionsService.prototype.getLastValidExtensionVersion = function (extension, versions) {
            var _this = this;
            if (!versions.length) {
                return winjs_base_1.TPromise.wrapError(new Error(nls.localize(6, null, extension.displayName)));
            }
            var version = versions[0];
            return this.request(version.manifestUrl)
                .then(function (opts) { return request_1.json(opts); })
                .then(function (manifest) {
                var codeVersion = _this.contextService.getConfiguration().env.version;
                var desc = {
                    isBuiltin: false,
                    engines: { vscode: manifest.engines.vscode },
                    main: manifest.main
                };
                if (!extensionValidator_1.isValidExtensionVersion(codeVersion, desc, [])) {
                    return _this.getLastValidExtensionVersion(extension, versions.slice(1));
                }
                return version;
            });
        };
        ExtensionsService.prototype.installFromZip = function (zipPath) {
            var _this = this;
            return validate(zipPath).then(function (manifest) {
                var extensionPath = path.join(_this.extensionsPath, getExtensionId(manifest));
                _this._onInstallExtension.fire(manifest);
                return zip_1.extract(zipPath, extensionPath, { sourcePath: 'extension', overwrite: true })
                    .then(function () { return createExtension(manifest, manifest.__metadata, extensionPath); })
                    .then(function (extension) { _this._onDidInstallExtension.fire({ extension: extension }); return extension; });
            });
        };
        ExtensionsService.prototype.uninstall = function (extension) {
            var _this = this;
            var extensionPath = extension.path || path.join(this.extensionsPath, getExtensionId(extension));
            return pfs.exists(extensionPath)
                .then(function (exists) { return exists ? null : winjs_base_1.Promise.wrapError(new Error(nls.localize(7, null))); })
                .then(function () { return _this._onUninstallExtension.fire(extension); })
                .then(function () { return _this.setObsolete(extension); })
                .then(function () { return pfs.rimraf(extensionPath); })
                .then(function () { return _this.unsetObsolete(extension); })
                .then(function () { return _this._onDidUninstallExtension.fire(extension); });
        };
        ExtensionsService.prototype.getInstalled = function (includeDuplicateVersions) {
            if (includeDuplicateVersions === void 0) { includeDuplicateVersions = false; }
            var all = this.getAllInstalled();
            if (includeDuplicateVersions) {
                return all;
            }
            return all.then(function (extensions) {
                var byId = collections_1.values(collections_1.groupBy(extensions, function (p) { return (p.publisher + "." + p.name); }));
                return byId.map(function (p) { return p.sort(function (a, b) { return semver.rcompare(a.version, b.version); })[0]; });
            });
        };
        ExtensionsService.prototype.getAllInstalled = function () {
            var _this = this;
            var limiter = new async_1.Limiter(10);
            return this.getObsoleteExtensions()
                .then(function (obsolete) {
                return pfs.readdir(_this.extensionsPath)
                    .then(function (extensions) { return extensions.filter(function (e) { return !obsolete[e]; }); })
                    .then(function (extensions) { return winjs_base_1.Promise.join(extensions.map(function (e) {
                    var extensionPath = path.join(_this.extensionsPath, e);
                    return limiter.queue(function () { return pfs.readFile(path.join(extensionPath, 'package.json'), 'utf8')
                        .then(function (raw) { return parseManifest(raw); })
                        .then(function (manifest) { return createExtension(manifest, manifest.__metadata, extensionPath); })
                        .then(null, function () { return null; }); });
                })); })
                    .then(function (result) { return result.filter(function (a) { return !!a; }); });
            });
        };
        ExtensionsService.prototype.removeDeprecatedExtensions = function () {
            var _this = this;
            var outdated = this.getOutdatedExtensions()
                .then(function (extensions) { return extensions.map(function (e) { return getExtensionId(e); }); });
            var obsolete = this.getObsoleteExtensions()
                .then(function (obsolete) { return Object.keys(obsolete); });
            return winjs_base_1.TPromise.join([outdated, obsolete])
                .then(function (result) { return arrays_1.flatten(result); })
                .then(function (extensionsIds) {
                return winjs_base_1.TPromise.join(extensionsIds.map(function (id) {
                    return pfs.rimraf(path.join(_this.extensionsPath, id))
                        .then(function () { return _this.withObsoleteExtensions(function (obsolete) { return delete obsolete[id]; }); });
                }));
            });
        };
        ExtensionsService.prototype.getOutdatedExtensions = function () {
            return this.getAllInstalled().then(function (plugins) {
                var byId = collections_1.values(collections_1.groupBy(plugins, function (p) { return (p.publisher + "." + p.name); }));
                var extensions = arrays_1.flatten(byId.map(function (p) { return p.sort(function (a, b) { return semver.rcompare(a.version, b.version); }).slice(1); }));
                return extensions
                    .filter(function (e) { return !!e.path; });
            });
        };
        ExtensionsService.prototype.isObsolete = function (extension) {
            var id = getExtensionId(extension);
            return this.withObsoleteExtensions(function (obsolete) { return !!obsolete[id]; });
        };
        ExtensionsService.prototype.setObsolete = function (extension) {
            var id = getExtensionId(extension);
            return this.withObsoleteExtensions(function (obsolete) { return objects_1.assign(obsolete, (_a = {}, _a[id] = true, _a)); var _a; });
        };
        ExtensionsService.prototype.unsetObsolete = function (extension) {
            var id = getExtensionId(extension);
            return this.withObsoleteExtensions(function (obsolete) { return delete obsolete[id]; });
        };
        ExtensionsService.prototype.getObsoleteExtensions = function () {
            return this.withObsoleteExtensions(function (obsolete) { return obsolete; });
        };
        ExtensionsService.prototype.withObsoleteExtensions = function (fn) {
            var _this = this;
            return this.obsoleteFileLimiter.queue(function () {
                var result = null;
                return pfs.readFile(_this.obsoletePath, 'utf8')
                    .then(null, function (err) { return err.code === 'ENOENT' ? winjs_base_1.TPromise.as('{}') : winjs_base_1.TPromise.wrapError(err); })
                    .then(function (raw) { return JSON.parse(raw); })
                    .then(function (obsolete) { result = fn(obsolete); return obsolete; })
                    .then(function (obsolete) {
                    if (Object.keys(obsolete).length === 0) {
                        return pfs.rimraf(_this.obsoletePath);
                    }
                    else {
                        var raw = JSON.stringify(obsolete);
                        return pfs.writeFile(_this.obsoletePath, raw);
                    }
                })
                    .then(function () { return result; });
            });
        };
        // Helper for proxy business... shameful.
        // This should be pushed down and not rely on the context service
        ExtensionsService.prototype.request = function (url) {
            var settings = winjs_base_1.TPromise.join([
                userSettings_1.UserSettings.getValue(this.contextService, 'http.proxy'),
                userSettings_1.UserSettings.getValue(this.contextService, 'http.proxyStrictSSL')
            ]);
            return settings.then(function (settings) {
                var proxyUrl = settings[0];
                var strictSSL = settings[1];
                var agent = proxy_1.getProxyAgent(url, { proxyUrl: proxyUrl, strictSSL: strictSSL });
                return { url: url, agent: agent, strictSSL: strictSSL };
            });
        };
        __decorate([
            service_1.ServiceEvent
        ], ExtensionsService.prototype, "onInstallExtension", void 0);
        __decorate([
            service_1.ServiceEvent
        ], ExtensionsService.prototype, "onDidInstallExtension", void 0);
        __decorate([
            service_1.ServiceEvent
        ], ExtensionsService.prototype, "onUninstallExtension", void 0);
        __decorate([
            service_1.ServiceEvent
        ], ExtensionsService.prototype, "onDidUninstallExtension", void 0);
        ExtensionsService = __decorate([
            __param(0, contextService_1.IWorkspaceContextService)
        ], ExtensionsService);
        return ExtensionsService;
    }());
    exports.ExtensionsService = ExtensionsService;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/services/request/node/rawHttpService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/base/common/objects', 'vs/base/node/request', 'vs/base/node/proxy', 'zlib'], function (require, exports, winjs_base_1, objects_1, request_1, proxy_1, zlib_1) {
    'use strict';
    var proxyUrl = null;
    var strictSSL = true;
    function configure(_proxyUrl, _strictSSL) {
        proxyUrl = _proxyUrl;
        strictSSL = _strictSSL;
    }
    exports.configure = configure;
    function xhr(options) {
        var agent = proxy_1.getProxyAgent(options.url, { proxyUrl: proxyUrl, strictSSL: strictSSL });
        options = objects_1.assign({}, options);
        options = objects_1.assign(options, { agent: agent, strictSSL: strictSSL });
        return request_1.request(options).then(function (result) { return new winjs_base_1.TPromise(function (c, e, p) {
            var res = result.res;
            var stream = res;
            if (res.headers['content-encoding'] === 'gzip') {
                stream = stream.pipe(zlib_1.createGunzip());
            }
            var data = [];
            stream.on('data', function (c) { return data.push(c); });
            stream.on('end', function () {
                var status = res.statusCode;
                if (options.followRedirects > 0 && (status >= 300 && status <= 303 || status === 307)) {
                    var location_1 = res.headers['location'];
                    if (location_1) {
                        var newOptions = {
                            type: options.type, url: location_1, user: options.user, password: options.password, responseType: options.responseType, headers: options.headers,
                            timeout: options.timeout, followRedirects: options.followRedirects - 1, data: options.data
                        };
                        xhr(newOptions).done(c, e, p);
                        return;
                    }
                }
                var response = {
                    responseText: data.join(''),
                    status: status,
                    getResponseHeader: function (header) { return res.headers[header]; },
                    readyState: 4
                };
                if ((status >= 200 && status < 300) || status === 1223) {
                    c(response);
                }
                else {
                    e(response);
                }
            });
        }, function (err) {
            var message;
            if (agent) {
                message = 'Unable to to connect to ' + options.url + ' through a proxy . Error: ' + err.message;
            }
            else {
                message = 'Unable to to connect to ' + options.url + '. Error: ' + err.message;
            }
            return winjs_base_1.TPromise.wrapError({
                responseText: message,
                status: 404
            });
        }); });
    }
    exports.xhr = xhr;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
define("vs/workbench/services/request/node/requestService", ["require", "exports", 'vs/base/common/winjs.base', 'vs/platform/configuration/common/configurationRegistry', 'vs/base/common/strings', 'vs/nls!vs/workbench/services/request/node/requestService', 'vs/base/common/lifecycle', 'vs/base/common/timer', 'vs/platform/platform', 'vs/base/common/async', 'vs/platform/configuration/common/configuration', 'vs/platform/request/common/baseRequestService', 'vs/workbench/services/request/node/rawHttpService'], function (require, exports, winjs_base_1, configurationRegistry_1, strings, nls, lifecycle, timer, platform, async, configuration_1, baseRequestService_1, rawHttpService) {
    'use strict';
    var RequestService = (function (_super) {
        __extends(RequestService, _super);
        function RequestService(contextService, configurationService, telemetryService) {
            var _this = this;
            _super.call(this, contextService, telemetryService);
            this.configurationService = configurationService;
            this.callOnDispose = [];
            // proxy setting updating
            this.callOnDispose.push(configurationService.addListener(configuration_1.ConfigurationServiceEventTypes.UPDATED, function (e) {
                _this.rawHttpServicePromise.then(function (rawHttpService) {
                    rawHttpService.configure(e.config.http && e.config.http.proxy, e.config.http.proxyStrictSSL);
                });
            }));
        }
        Object.defineProperty(RequestService.prototype, "rawHttpServicePromise", {
            get: function () {
                if (!this._rawHttpServicePromise) {
                    this._rawHttpServicePromise = this.configurationService.loadConfiguration().then(function (configuration) {
                        rawHttpService.configure(configuration.http && configuration.http.proxy, configuration.http.proxyStrictSSL);
                        return rawHttpService;
                    });
                }
                return this._rawHttpServicePromise;
            },
            enumerable: true,
            configurable: true
        });
        RequestService.prototype.dispose = function () {
            lifecycle.cAll(this.callOnDispose);
        };
        RequestService.prototype.makeRequest = function (options) {
            var url = options.url;
            if (!url) {
                throw new Error('IRequestService.makeRequest: Url is required.');
            }
            // Support file:// in native environment through XHR
            if (strings.startsWith(url, 'file://')) {
                return winjs_base_1.xhr(options).then(null, function (xhr) {
                    if (xhr.status === 0 && xhr.responseText) {
                        return xhr; // loading resources locally returns a status of 0 which in WinJS is an error so we need to handle it here
                    }
                    return winjs_base_1.Promise.wrapError({ status: 404, responseText: nls.localize(0, null) });
                });
            }
            return _super.prototype.makeRequest.call(this, options);
        };
        /**
         * Make a cross origin request using NodeJS.
         * Note: This method is also called from workers.
         */
        RequestService.prototype.makeCrossOriginRequest = function (options) {
            var timerVar = timer.nullEvent;
            return this.rawHttpServicePromise.then(function (rawHttpService) {
                return async.always(rawHttpService.xhr(options), (function (xhr) {
                    if (timerVar.data) {
                        timerVar.data.status = xhr.status;
                    }
                    timerVar.stop();
                }));
            });
        };
        return RequestService;
    }(baseRequestService_1.BaseRequestService));
    exports.RequestService = RequestService;
    // Configuration
    var confRegistry = platform.Registry.as(configurationRegistry_1.Extensions.Configuration);
    confRegistry.registerConfiguration({
        'id': 'http',
        'order': 9,
        'title': nls.localize(1, null),
        'type': 'object',
        'properties': {
            'http.proxy': {
                'type': 'string',
                'description': nls.localize(2, null)
            },
            'http.proxyStrictSSL': {
                'type': 'boolean',
                'default': true,
                'description': nls.localize(3, null)
            }
        }
    });
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/electron-browser/shell", ["require", "exports", 'vs/nls!vs/workbench/electron-browser/shell', 'vs/base/browser/builder', 'vs/base/common/objects', 'vs/base/browser/dom', 'vs/base/browser/ui/aria/aria', 'vs/base/common/lifecycle', 'vs/base/common/errors', 'vs/platform/contextview/browser/contextViewService', 'vs/workbench/services/contextview/electron-browser/contextmenuService', 'vs/workbench/common/constants', 'vs/base/common/timer', 'vs/workbench/browser/workbench', 'vs/workbench/common/storage', 'vs/platform/telemetry/common/telemetry', 'vs/platform/telemetry/electron-browser/electronTelemetryService', 'vs/workbench/electron-browser/integration', 'vs/workbench/electron-browser/update', 'vs/platform/telemetry/common/workspaceStats', 'vs/workbench/services/window/electron-browser/windowService', 'vs/workbench/services/message/electron-browser/messageService', 'vs/workbench/services/request/node/requestService', 'vs/platform/configuration/common/configuration', 'vs/workbench/services/configuration/node/configurationService', 'vs/workbench/services/files/electron-browser/fileService', 'vs/workbench/services/search/node/searchService', 'vs/workbench/services/lifecycle/electron-browser/lifecycleService', 'vs/workbench/services/keybinding/electron-browser/keybindingService', 'vs/workbench/services/thread/electron-browser/threadService', 'vs/platform/markers/common/markerService', 'vs/platform/actions/common/actions', 'vs/platform/actions/common/actionsService', 'vs/editor/common/services/modelService', 'vs/editor/common/services/modelServiceImpl', 'vs/editor/browser/services/codeEditorServiceImpl', 'vs/editor/common/services/codeEditorService', 'vs/editor/common/services/editorWorkerServiceImpl', 'vs/editor/common/services/editorWorkerService', 'vs/workbench/api/node/extHost.api.impl', 'vs/platform/extensions/common/nativeExtensionService', 'vs/workbench/api/node/extHostDocuments', 'vs/editor/node/textMate/TMSyntax', 'vs/editor/node/textMate/TMSnippets', 'vs/platform/jsonschemas/common/jsonValidationExtensionPoint', 'vs/editor/node/languageConfiguration', 'vs/workbench/api/node/extHostFileSystemEventService', 'vs/workbench/api/node/extHostQuickOpen', 'vs/workbench/api/node/extHostStatusBar', 'vs/workbench/api/node/extHostCommands', 'vs/platform/telemetry/common/abstractRemoteTelemetryService', 'vs/workbench/api/node/extHostDiagnostics', 'vs/workbench/api/node/extHostOutputService', 'vs/workbench/api/node/extHostMessageService', 'vs/workbench/api/node/extHostLanguages', 'vs/workbench/api/node/extHostEditors', 'vs/workbench/api/node/extHostWorkspace', 'vs/workbench/api/node/extHostConfiguration', 'vs/workbench/api/node/extHostLanguageFeatures', 'vs/platform/event/common/eventService', 'vs/workbench/services/workspace/common/contextService', 'vs/platform/storage/common/storage', 'vs/platform/storage/common/remotable.storage', 'vs/platform/instantiation/common/instantiationService', 'vs/platform/contextview/browser/contextView', 'vs/platform/event/common/event', 'vs/platform/files/common/files', 'vs/platform/keybinding/common/keybindingService', 'vs/platform/lifecycle/common/lifecycle', 'vs/platform/markers/common/markers', 'vs/platform/message/common/message', 'vs/platform/request/common/request', 'vs/platform/search/common/search', 'vs/platform/thread/common/thread', 'vs/platform/workspace/common/workspace', 'vs/platform/extensions/common/extensions', 'vs/editor/common/services/modeServiceImpl', 'vs/editor/common/services/modeService', 'vs/workbench/services/untitled/common/untitledEditorService', 'vs/workbench/electron-browser/crashReporter', 'vs/workbench/services/themes/common/themeService', 'vs/workbench/services/themes/node/themeService', 'vs/base/common/service', 'vs/base/node/service.net', 'vs/workbench/parts/extensions/common/extensions', 'vs/workbench/parts/extensions/node/extensionsService', 'vs/workbench/electron-browser/actions', 'vs/css!./media/shell', 'vs/css!vs/workbench/browser/media/vs-theme', 'vs/css!vs/workbench/browser/media/vs-dark-theme', 'vs/css!vs/workbench/browser/media/hc-black-theme'], function (require, exports, nls, builder_1, objects, dom, aria, lifecycle_1, errors, contextViewService_1, contextmenuService_1, constants_1, timer, workbench_1, storage_1, telemetry_1, electronTelemetryService_1, integration_1, update_1, workspaceStats_1, windowService_1, messageService_1, requestService_1, configuration_1, configurationService_1, fileService_1, searchService_1, lifecycleService_1, keybindingService_1, threadService_1, markerService_1, actions_1, actionsService_1, modelService_1, modelServiceImpl_1, codeEditorServiceImpl_1, codeEditorService_1, editorWorkerServiceImpl_1, editorWorkerService_1, extHost_api_impl_1, nativeExtensionService_1, extHostDocuments_1, TMSyntax_1, TMSnippets_1, jsonValidationExtensionPoint_1, languageConfiguration_1, extHostFileSystemEventService_1, extHostQuickOpen_1, extHostStatusBar_1, extHostCommands_1, abstractRemoteTelemetryService_1, extHostDiagnostics_1, extHostOutputService_1, extHostMessageService_1, extHostLanguages_1, extHostEditors_1, extHostWorkspace_1, extHostConfiguration_1, extHostLanguageFeatures_1, eventService_1, contextService_1, storage_2, remotable_storage_1, instantiationService_1, contextView_1, event_1, files_1, keybindingService_2, lifecycle_2, markers_1, message_1, request_1, search_1, thread_1, workspace_1, extensions_1, modeServiceImpl_1, modeService_1, untitledEditorService_1, crashReporter_1, themeService_1, themeService_2, service_1, service_net_1, extensions_2, extensionsService_1, actions_2) {
    'use strict';
    /**
     * The Monaco Workbench Shell contains the Monaco workbench with a rich header containing navigation and the activity bar.
     * With the Shell being the top level element in the page, it is also responsible for driving the layouting.
     */
    var WorkbenchShell = (function () {
        function WorkbenchShell(container, workspace, configuration, options) {
            this.container = container;
            this.workspace = workspace;
            this.configuration = configuration;
            this.options = objects.mixin({}, options);
            this.toUnbind = [];
            this.previousErrorTime = 0;
        }
        WorkbenchShell.prototype.createContents = function (parent) {
            var _this = this;
            // ARIA
            aria.setARIAContainer(document.body);
            // Workbench Container
            var workbenchContainer = builder_1.$(parent).div();
            // Instantiation service with services
            var instantiationService = this.initInstantiationService();
            //crash reporting
            if (!!this.configuration.env.crashReporter) {
                var crashReporter = instantiationService.createInstance(crashReporter_1.CrashReporter, this.configuration.env.version, this.configuration.env.commitHash);
                crashReporter.start(this.configuration.env.crashReporter);
            }
            var sharedProcessClientPromise = service_net_1.connect(process.env['VSCODE_SHARED_IPC_HOOK']);
            sharedProcessClientPromise.done(function (service) {
                service.onClose(function () {
                    _this.messageService.show(message_1.Severity.Error, {
                        message: nls.localize(0, null),
                        actions: [instantiationService.createInstance(actions_2.ReloadWindowAction, actions_2.ReloadWindowAction.ID, actions_2.ReloadWindowAction.LABEL)]
                    });
                });
            }, errors.onUnexpectedError);
            instantiationService.addSingleton(extensions_2.IExtensionsService, service_1.getService(sharedProcessClientPromise, 'ExtensionService', extensionsService_1.ExtensionsService));
            // Workbench
            this.workbench = new workbench_1.Workbench(workbenchContainer.getHTMLElement(), this.workspace, this.configuration, this.options, instantiationService);
            this.workbench.startup({
                onServicesCreated: function () {
                    _this.initExtensionSystem();
                },
                onWorkbenchStarted: function () {
                    _this.onWorkbenchStarted();
                }
            });
            // Electron integration
            this.workbench.getInstantiationService().createInstance(integration_1.ElectronIntegration).integrate(this.container);
            // Update
            this.workbench.getInstantiationService().createInstance(update_1.Update);
            // Handle case where workbench is not starting up properly
            var timeoutHandle = setTimeout(function () {
                console.warn('Workbench did not finish loading in 10 seconds, that might be a problem that should be reported.');
            }, 10000);
            this.workbench.joinCreation().then(function () {
                clearTimeout(timeoutHandle);
            });
            return workbenchContainer;
        };
        WorkbenchShell.prototype.onWorkbenchStarted = function () {
            // Log to telemetry service
            var windowSize = {
                innerHeight: window.innerHeight,
                innerWidth: window.innerWidth,
                outerHeight: window.outerHeight,
                outerWidth: window.outerWidth
            };
            this.telemetryService.publicLog('workspaceLoad', {
                userAgent: navigator.userAgent,
                windowSize: windowSize,
                emptyWorkbench: !this.contextService.getWorkspace(),
                customKeybindingsCount: this.keybindingService.customKeybindingsCount(),
                theme: this.currentTheme
            });
            var workspaceStats = this.workbench.getInstantiationService().createInstance(workspaceStats_1.WorkspaceStats);
            workspaceStats.reportWorkspaceTags();
        };
        WorkbenchShell.prototype.initInstantiationService = function () {
            var _this = this;
            var eventService = new eventService_1.EventService();
            this.contextService = new contextService_1.WorkspaceContextService(eventService, this.workspace, this.configuration, this.options);
            this.windowService = new windowService_1.WindowService();
            var disableWorkspaceStorage = this.configuration.env.extensionTestsPath || (!this.workspace && !this.configuration.env.extensionDevelopmentPath); // without workspace or in any extension test, we use inMemory storage unless we develop an extension where we want to preserve state
            this.storageService = new storage_1.Storage(this.contextService, window.localStorage, disableWorkspaceStorage ? storage_1.inMemoryLocalStorageInstance : window.localStorage);
            var configService = new configurationService_1.ConfigurationService(this.contextService, eventService);
            // no telemetry in a window for extension development!
            var enableTelemetry = this.configuration.env.isBuilt && !this.configuration.env.extensionDevelopmentPath ? !!this.configuration.env.enableTelemetry : false;
            this.telemetryService = new electronTelemetryService_1.ElectronTelemetryService(configService, this.storageService, { enableTelemetry: enableTelemetry, version: this.configuration.env.version, commitHash: this.configuration.env.commitHash });
            this.keybindingService = new keybindingService_1.WorkbenchKeybindingService(configService, this.contextService, eventService, this.telemetryService, window);
            this.messageService = new messageService_1.MessageService(this.contextService, this.windowService, this.telemetryService, this.keybindingService);
            this.keybindingService.setMessageService(this.messageService);
            var fileService = new fileService_1.FileService(configService, eventService, this.contextService);
            this.contextViewService = new contextViewService_1.ContextViewService(this.container, this.telemetryService, this.messageService);
            var lifecycleService = new lifecycleService_1.LifecycleService(this.messageService, this.windowService);
            lifecycleService.onShutdown(function () { return fileService.dispose(); });
            this.threadService = new threadService_1.MainThreadService(this.contextService, this.messageService, this.windowService);
            lifecycleService.onShutdown(function () { return _this.threadService.dispose(); });
            var requestService = new requestService_1.RequestService(this.contextService, configService, this.telemetryService);
            lifecycleService.onShutdown(function () { return requestService.dispose(); });
            var markerService = new markerService_1.MainProcessMarkerService(this.threadService);
            var extensionService = new nativeExtensionService_1.MainProcessExtensionService(this.contextService, this.threadService, this.messageService, this.telemetryService);
            this.keybindingService.setExtensionService(extensionService);
            var modeService = new modeServiceImpl_1.MainThreadModeServiceImpl(this.threadService, extensionService, configService);
            var modelService = new modelServiceImpl_1.ModelServiceImpl(this.threadService, markerService, modeService, configService, this.messageService);
            var editorWorkerService = new editorWorkerServiceImpl_1.EditorWorkerServiceImpl(modelService);
            var untitledEditorService = new untitledEditorService_1.UntitledEditorService();
            this.themeService = new themeService_2.ThemeService(extensionService);
            var result = instantiationService_1.createInstantiationService();
            result.addSingleton(telemetry_1.ITelemetryService, this.telemetryService);
            result.addSingleton(event_1.IEventService, eventService);
            result.addSingleton(request_1.IRequestService, requestService);
            result.addSingleton(workspace_1.IWorkspaceContextService, this.contextService);
            result.addSingleton(contextView_1.IContextViewService, this.contextViewService);
            result.addSingleton(contextView_1.IContextMenuService, new contextmenuService_1.ContextMenuService(this.messageService, this.telemetryService, this.keybindingService));
            result.addSingleton(message_1.IMessageService, this.messageService);
            result.addSingleton(storage_2.IStorageService, this.storageService);
            result.addSingleton(lifecycle_2.ILifecycleService, lifecycleService);
            result.addSingleton(thread_1.IThreadService, this.threadService);
            result.addSingleton(extensions_1.IExtensionService, extensionService);
            result.addSingleton(modeService_1.IModeService, modeService);
            result.addSingleton(files_1.IFileService, fileService);
            result.addSingleton(untitledEditorService_1.IUntitledEditorService, untitledEditorService);
            result.addSingleton(search_1.ISearchService, new searchService_1.SearchService(modelService, untitledEditorService, this.contextService, configService));
            result.addSingleton(windowService_1.IWindowService, this.windowService);
            result.addSingleton(configuration_1.IConfigurationService, configService);
            result.addSingleton(keybindingService_2.IKeybindingService, this.keybindingService);
            result.addSingleton(markers_1.IMarkerService, markerService);
            result.addSingleton(modelService_1.IModelService, modelService);
            result.addSingleton(codeEditorService_1.ICodeEditorService, new codeEditorServiceImpl_1.CodeEditorServiceImpl());
            result.addSingleton(editorWorkerService_1.IEditorWorkerService, editorWorkerService);
            result.addSingleton(themeService_1.IThemeService, this.themeService);
            result.addSingleton(actions_1.IActionsService, new actionsService_1.default(extensionService, this.keybindingService));
            return result;
        };
        // TODO@Alex, TODO@Joh move this out of here?
        WorkbenchShell.prototype.initExtensionSystem = function () {
            this.threadService.getRemotable(extHost_api_impl_1.MainProcessVSCodeAPIHelper);
            this.threadService.getRemotable(extHostDocuments_1.MainThreadDocuments);
            this.threadService.getRemotable(abstractRemoteTelemetryService_1.RemoteTelemetryServiceHelper);
            this.workbench.getInstantiationService().createInstance(TMSyntax_1.MainProcessTextMateSyntax);
            this.workbench.getInstantiationService().createInstance(TMSnippets_1.MainProcessTextMateSnippet);
            this.workbench.getInstantiationService().createInstance(jsonValidationExtensionPoint_1.JSONValidationExtensionPoint);
            this.workbench.getInstantiationService().createInstance(languageConfiguration_1.LanguageConfigurationFileHandler);
            this.threadService.getRemotable(extHostConfiguration_1.MainThreadConfiguration);
            this.threadService.getRemotable(extHostQuickOpen_1.MainThreadQuickOpen);
            this.threadService.getRemotable(extHostStatusBar_1.MainThreadStatusBar);
            this.workbench.getInstantiationService().createInstance(extHostFileSystemEventService_1.MainThreadFileSystemEventService);
            this.threadService.getRemotable(extHostCommands_1.MainThreadCommands);
            this.threadService.getRemotable(extHostOutputService_1.MainThreadOutputService);
            this.threadService.getRemotable(extHostDiagnostics_1.MainThreadDiagnostics);
            this.threadService.getRemotable(extHostMessageService_1.MainThreadMessageService);
            this.threadService.getRemotable(extHostLanguages_1.MainThreadLanguages);
            this.threadService.getRemotable(extHostWorkspace_1.MainThreadWorkspace);
            this.threadService.getRemotable(extHostEditors_1.MainThreadEditors);
            this.threadService.getRemotable(remotable_storage_1.MainThreadStorage);
            this.threadService.getRemotable(extHostLanguageFeatures_1.MainThreadLanguageFeatures);
        };
        WorkbenchShell.prototype.open = function () {
            var _this = this;
            // Listen on unexpected errors
            errors.setUnexpectedErrorHandler(function (error) {
                _this.onUnexpectedError(error);
            });
            // Shell Class for CSS Scoping
            builder_1.$(this.container).addClass('monaco-shell');
            // Controls
            this.content = builder_1.$('.monaco-shell-content').appendTo(this.container).getHTMLElement();
            // Handle Load Performance Timers
            this.writeTimers();
            // Create Contents
            this.contentsContainer = this.createContents(builder_1.$(this.content));
            // Layout
            this.layout();
            // Listeners
            this.registerListeners();
            // Enable theme support
            var themeId = this.storageService.get(constants_1.Preferences.THEME, storage_2.StorageScope.GLOBAL, null);
            if (!themeId) {
                themeId = themeService_1.DEFAULT_THEME_ID;
                this.storageService.store(constants_1.Preferences.THEME, themeId, storage_2.StorageScope.GLOBAL);
            }
            this.setTheme(themeId, false);
            this.toUnbind.push(this.storageService.addListener2(storage_2.StorageEventType.STORAGE, function (e) {
                if (e.key === constants_1.Preferences.THEME) {
                    _this.setTheme(e.newValue);
                }
            }));
        };
        WorkbenchShell.prototype.setTheme = function (themeId, layout) {
            var _this = this;
            if (layout === void 0) { layout = true; }
            if (!themeId) {
                return;
            }
            var applyTheme = function (themeId) {
                if (_this.currentTheme) {
                    builder_1.$(_this.container).removeClass(_this.currentTheme);
                }
                _this.currentTheme = themeId;
                builder_1.$(_this.container).addClass(_this.currentTheme);
                if (layout) {
                    _this.layout();
                }
            };
            this.themeService.loadTheme(themeId).then(function (theme) {
                var newThemeId = theme ? theme.id : themeService_1.DEFAULT_THEME_ID;
                _this.themeService.applyThemeCSS(newThemeId);
                applyTheme(newThemeId);
                if (newThemeId !== themeId) {
                    _this.storageService.store(constants_1.Preferences.THEME, newThemeId, storage_2.StorageScope.GLOBAL);
                }
            }, function (error) {
                errors.onUnexpectedError(error);
            });
        };
        WorkbenchShell.prototype.registerListeners = function () {
            var _this = this;
            // Resize
            builder_1.$(window).on(dom.EventType.RESIZE, function () { return _this.layout(); }, this.toUnbind);
        };
        WorkbenchShell.prototype.writeTimers = function () {
            var timers = window.MonacoEnvironment.timers;
            if (timers) {
                var events = [];
                // Program
                if (timers.beforeProgram) {
                    events.push({
                        startTime: timers.beforeProgram,
                        stopTime: timers.afterProgram,
                        topic: 'Startup',
                        name: 'Program Start',
                        description: 'Time it takes to pass control to VSCodes main method'
                    });
                }
                // Window
                if (timers.vscodeStart) {
                    events.push({
                        startTime: timers.vscodeStart,
                        stopTime: timers.beforeLoad,
                        topic: 'Startup',
                        name: 'VSCode Startup',
                        description: 'Time it takes to create a window and startup VSCode'
                    });
                }
                // Load
                events.push({
                    startTime: timers.beforeLoad,
                    stopTime: timers.afterLoad,
                    topic: 'Startup',
                    name: 'Load Modules',
                    description: 'Time it takes to load VSCodes main modules'
                });
                // Ready
                events.push({
                    startTime: timers.beforeReady,
                    stopTime: timers.afterReady,
                    topic: 'Startup',
                    name: 'Event DOMContentLoaded',
                    description: 'Time it takes for the DOM to emit DOMContentLoaded event'
                });
                // Write to Timer
                timer.getTimeKeeper().setInitialCollectedEvents(events, timers.start);
            }
        };
        WorkbenchShell.prototype.onUnexpectedError = function (error) {
            var errorMsg = errors.toErrorMessage(error, true);
            if (!errorMsg) {
                return;
            }
            var now = new Date().getTime();
            if (errorMsg === this.previousErrorValue && now - this.previousErrorTime <= 1000) {
                return; // Return if error message identical to previous and shorter than 1 second
            }
            this.previousErrorTime = now;
            this.previousErrorValue = errorMsg;
            // Log to console
            console.error(errorMsg);
            // Show to user if friendly message provided
            if (error && error.friendlyMessage && this.messageService) {
                this.messageService.show(message_1.Severity.Error, error.friendlyMessage);
            }
        };
        WorkbenchShell.prototype.layout = function () {
            var clArea = builder_1.$(this.container).getClientArea();
            var contentsSize = new builder_1.Dimension(clArea.width, clArea.height);
            this.contentsContainer.size(contentsSize.width, contentsSize.height);
            this.contextViewService.layout();
            this.workbench.layout();
        };
        WorkbenchShell.prototype.joinCreation = function () {
            return this.workbench.joinCreation();
        };
        WorkbenchShell.prototype.dispose = function (force) {
            // Workbench
            if (this.workbench) {
                var veto = this.workbench.shutdown(force);
                // If Workbench vetos dispose, return early
                if (veto) {
                    return;
                }
            }
            this.contextViewService.dispose();
            this.storageService.dispose();
            // Listeners
            this.toUnbind = lifecycle_1.disposeAll(this.toUnbind);
            // Container
            builder_1.$(this.container).empty();
        };
        return WorkbenchShell;
    }());
    exports.WorkbenchShell = WorkbenchShell;
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
define("vs/workbench/electron-browser/main", ["require", "exports", 'vs/base/common/winjs.base', 'vs/workbench/electron-browser/shell', 'vs/base/common/errors', 'vs/base/common/platform', 'vs/base/common/paths', 'vs/base/common/timer', 'vs/base/common/objects', 'vs/base/common/uri', 'vs/base/common/strings', 'path', 'fs', 'graceful-fs'], function (require, exports, winjs, shell_1, errors, platform, paths, timer, objects_1, uri_1, strings, path, fs, gracefulFs) {
    'use strict';
    gracefulFs.gracefulify(fs);
    function startup(environment, globalSettings) {
        // Inherit the user environment
        objects_1.assign(process.env, environment.userEnv);
        // Shell Configuration
        var shellConfiguration = {
            env: environment
        };
        // Shell Options
        var filesToOpen = environment.filesToOpen && environment.filesToOpen.length ? toInputs(environment.filesToOpen) : null;
        var filesToCreate = environment.filesToCreate && environment.filesToCreate.length ? toInputs(environment.filesToCreate) : null;
        var filesToDiff = environment.filesToDiff && environment.filesToDiff.length ? toInputs(environment.filesToDiff) : null;
        var shellOptions = {
            singleFileMode: !environment.workspacePath,
            filesToOpen: filesToOpen,
            filesToCreate: filesToCreate,
            filesToDiff: filesToDiff,
            extensionsToInstall: environment.extensionsToInstall,
            globalSettings: globalSettings
        };
        if (environment.enablePerformance) {
            timer.ENABLE_TIMER = true;
        }
        // Open workbench
        return openWorkbench(getWorkspace(environment), shellConfiguration, shellOptions);
    }
    exports.startup = startup;
    function toInputs(paths) {
        return paths.map(function (p) {
            var input = {
                resource: uri_1.default.file(p.filePath)
            };
            if (p.lineNumber) {
                input.options = {
                    selection: {
                        startLineNumber: p.lineNumber,
                        startColumn: p.columnNumber
                    }
                };
            }
            return input;
        });
    }
    function getWorkspace(environment) {
        if (!environment.workspacePath) {
            return null;
        }
        var realWorkspacePath = path.normalize(fs.realpathSync(environment.workspacePath));
        if (paths.isUNC(realWorkspacePath) && strings.endsWith(realWorkspacePath, paths.nativeSep)) {
            // for some weird reason, node adds a trailing slash to UNC paths
            // we never ever want trailing slashes as our workspace path unless
            // someone opens root ("/").
            // See also https://github.com/nodejs/io.js/issues/1765
            realWorkspacePath = strings.rtrim(realWorkspacePath, paths.nativeSep);
        }
        var workspaceResource = uri_1.default.file(realWorkspacePath);
        var folderName = path.basename(realWorkspacePath) || realWorkspacePath;
        var folderStat = fs.statSync(realWorkspacePath);
        var workspace = {
            'resource': workspaceResource,
            'id': platform.isLinux ? realWorkspacePath : realWorkspacePath.toLowerCase(),
            'name': folderName,
            'uid': platform.isLinux ? folderStat.ino : folderStat.birthtime.getTime(),
            'mtime': folderStat.mtime.getTime()
        };
        return workspace;
    }
    function openWorkbench(workspace, configuration, options) {
        window.MonacoEnvironment.timers.beforeReady = new Date();
        return winjs.Utilities.ready(function () {
            window.MonacoEnvironment.timers.afterReady = new Date();
            // Monaco Workbench Shell
            var beforeOpen = new Date();
            var shell = new shell_1.WorkbenchShell(document.body, workspace, configuration, options);
            shell.open();
            shell.joinCreation().then(function () {
                timer.start(timer.Topic.STARTUP, 'Open Shell, Viewlet & Editor', beforeOpen, 'Workbench has opened after this event with viewlet and editor restored').stop();
            });
            // Inform user about loading issues from the loader
            self.require.config({
                onError: function (err) {
                    if (err.errorCode === 'load') {
                        shell.onUnexpectedError(errors.loaderError(err));
                    }
                }
            });
        }, true);
    }
});

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/

require.config({
	ignoreDuplicateModules: [
		'vs/workbench/parts/search/common/searchModel',
		'vs/workbench/parts/search/common/searchQuery'
	]
});

define("vs/workbench/workbench.main", [

	// Base
	'vs/base/common/strings',
	'vs/base/common/errors',

	// Editor
	'vs/editor/contrib/selectionClipboard/electron-browser/selectionClipboard',
	'vs/editor/browser/editor.all',

	// Languages
	'vs/languages/languages.main',

	// Workbench
	'vs/workbench/browser/actions/toggleSidebarVisibility',
	'vs/workbench/browser/actions/toggleSidebarPosition',
	'vs/workbench/browser/actions/triggerQuickOpen',
	'vs/workbench/browser/actions/triggerEditorActions',
	'vs/workbench/browser/actions/triggerNavigation',
	'vs/workbench/browser/actions/showPerformanceBox',
	'vs/workbench/browser/actions/openSettings',
	'vs/workbench/browser/actions/configureLocale',

	'vs/workbench/parts/quickopen/browser/quickopen.contribution',

	'vs/workbench/parts/files/browser/explorerViewlet',
	'vs/workbench/parts/files/browser/workingFilesPicker',
	'vs/workbench/parts/files/browser/fileActions.contribution',
	'vs/workbench/parts/files/browser/files.contribution',
	'vs/workbench/parts/files/electron-browser/files.electron.contribution',

	'vs/workbench/parts/search/browser/search.contribution',

	'vs/workbench/parts/git/electron-browser/git.contribution',
	'vs/workbench/parts/git/browser/gitQuickOpen',
	'vs/workbench/parts/git/browser/gitActions.contribution',

	'vs/workbench/parts/debug/electron-browser/debug.contribution',

	'vs/workbench/parts/errorList/browser/errorList.contribution',

	'vs/workbench/parts/html/browser/html.contribution',

	'vs/workbench/parts/extensions/electron-browser/extensions.contribution',
	'vs/workbench/parts/extensions/electron-browser/extensionsQuickOpen',

	'vs/workbench/parts/output/browser/output.contribution',

	'vs/workbench/parts/markdown/browser/markdown.contribution',
	'vs/workbench/parts/markdown/browser/markdownActions.contribution',

	'vs/workbench/browser/workbench',

	'vs/workbench/parts/tasks/electron-browser/task.contribution',

	'vs/workbench/parts/emmet/node/emmet.contribution',

	'vs/workbench/parts/execution/electron-browser/execution.contribution',
	'vs/workbench/parts/execution/electron-browser/terminal.contribution',

	'vs/workbench/parts/snippets/electron-browser/snippets.contribution',

	'vs/workbench/parts/contentprovider/common/contentprovider.contribution',

	'vs/workbench/parts/telemetry/node/appInsights.telemetry.contribution',

	'vs/workbench/parts/themes/electron-browser/themes.contribution',

	'vs/workbench/parts/feedback/electron-browser/feedback.contribution',

	'vs/workbench/parts/gettingStarted/electron-browser/electronGettingStarted.contribution',

	'vs/workbench/parts/update/electron-browser/update.contribution',

	'vs/workbench/electron-browser/darwin/cli.contribution',

	'vs/workbench/electron-browser/main.contribution',
	'vs/workbench/electron-browser/main'

], function() {
	'use strict';
});

//# sourceMappingURL=workbench.main.js.map
